import ctypes
import ctypes.wintypes as wintypes
from enum import IntEnum

from nmspy.data import common

class eNoiseVoxelType(IntEnum):
    ENoiseVoxelType_Base = 0x0
    ENoiseVoxelType_Rock = 0x1
    ENoiseVoxelType_Mountain = 0x2
    ENoiseVoxelType_Sand = 0x3
    ENoiseVoxelType_Cave = 0x4
    ENoiseVoxelType_Substance_1 = 0x5
    ENoiseVoxelType_Substance_2 = 0x6
    ENoiseVoxelType_Substance_3 = 0x7
    ENoiseVoxelType_RandomRock = 0x8
    ENoiseVoxelType_RandomRockOrSubstance = 0x9
    ENoiseVoxelType_NumTypes = 0xA

class eCurve(IntEnum):
    ECurve_Linear = 0x0
    ECurve_SmoothInOut = 0x1
    ECurve_FastInSlowOut = 0x2
    ECurve_BellSquared = 0x3
    ECurve_Squared = 0x4
    ECurve_Cubed = 0x5
    ECurve_Logarithmic = 0x6
    ECurve_SlowIn = 0x7
    ECurve_SlowOut = 0x8
    ECurve_ReallySlowOut = 0x9
    ECurve_SmootherStep = 0xA
    ECurve_SmoothFastInSlowOut = 0xB
    ECurve_SmoothSlowInFastOut = 0xC
    ECurve_EaseInSine = 0xD
    ECurve_EaseOutSine = 0xE
    ECurve_EaseInOutSine = 0xF
    ECurve_EaseInQuad = 0x10
    ECurve_EaseOutQuad = 0x11
    ECurve_EaseInOutQuad = 0x12
    ECurve_EaseInQuart = 0x13
    ECurve_EaseOutQuart = 0x14
    ECurve_EaseInOutQuart = 0x15
    ECurve_EaseInQuint = 0x16
    ECurve_EaseOutQuint = 0x17
    ECurve_EaseInOutQuint = 0x18
    ECurve_EaseInExpo = 0x19
    ECurve_EaseOutExpo = 0x1A
    ECurve_EaseInOutExpo = 0x1B
    ECurve_EaseInCirc = 0x1C
    ECurve_EaseOutCirc = 0x1D
    ECurve_EaseInOutCirc = 0x1E
    ECurve_EaseInBack = 0x1F
    ECurve_EaseOutBack = 0x20
    ECurve_EaseInOutBack = 0x21
    ECurve_EaseInElastic = 0x22
    ECurve_EaseOutElastic = 0x23
    ECurve_EaseInOutElastic = 0x24
    ECurve_EaseInBounce = 0x25
    ECurve_EaseOutBounce = 0x26
    ECurve_EaseInOutBounce = 0x27
    ECurve_NumTypes = 0x28

class eNoiseLayerTypes(IntEnum):
    ENoiseLayerTypes_Base = 0x0
    ENoiseLayerTypes_Hill = 0x1
    ENoiseLayerTypes_Mountain = 0x2
    ENoiseLayerTypes_Rock = 0x3
    ENoiseLayerTypes_UnderWater = 0x4
    ENoiseLayerTypes_Texture = 0x5
    ENoiseLayerTypes_Elevation = 0x6
    ENoiseLayerTypes_Continent = 0x7
    ENoiseLayerTypes_NumTypes = 0x8

class eInputValidation(IntEnum):
    EInputValidation_Held = 0x0
    EInputValidation_Pressed = 0x1
    EInputValidation_Released = 0x2
    EInputValidation_HeldOver = 0x3
    EInputValidation_NumTypes = 0x4

class eDebugNoiseType(IntEnum):
    EDebugNoiseType_Plane = 0x0
    EDebugNoiseType_Check = 0x1
    EDebugNoiseType_Sine = 0x2
    EDebugNoiseType_Uber = 0x3
    EDebugNoiseType_NumTypes = 0x4

class eInputHand(IntEnum):
    EInputHand_None = 0x0
    EInputHand_Left = 0x1
    EInputHand_Right = 0x2
    EInputHand_NumTypes = 0x3

class eRestriction(IntEnum):
    ERestriction_NoMoreThan = 0x0
    ERestriction_AtLeast = 0x1
    ERestriction_AtLeastIfICan = 0x2
    ERestriction_NumTypes = 0x3

class eLinesPosition(IntEnum):
    ELinesPosition_Absolute = 0x0
    ELinesPosition_Relative = 0x1
    ELinesPosition_NumTypes = 0x2

class eCaveTypes(IntEnum):
    ECaveTypes_Underground = 0x0
    ECaveTypes_NumTypes = 0x1

class eOffsetType(IntEnum):
    EOffsetType_Zero = 0x0
    EOffsetType_Base = 0x1
    EOffsetType_All = 0x2
    EOffsetType_SeaLevel = 0x3
    EOffsetType_NumTypes = 0x4

class eWaterFade(IntEnum):
    EWaterFade_None = 0x0
    EWaterFade_Above = 0x1
    EWaterFade_Below = 0x2
    EWaterFade_NumTypes = 0x3

class eCoordinateOrientation(IntEnum):
    ECoordinateOrientation_None = 0x0
    ECoordinateOrientation_Random = 0x1
    ECoordinateOrientation_NumTypes = 0x2

class eGridLayerTypes(IntEnum):
    EGridLayerTypes_Small = 0x0
    EGridLayerTypes_Large = 0x1
    EGridLayerTypes_Resources_Heridium = 0x2
    EGridLayerTypes_Resources_Iridium = 0x3
    EGridLayerTypes_Resources_Copper = 0x4
    EGridLayerTypes_Resources_Nickel = 0x5
    EGridLayerTypes_Resources_Aluminium = 0x6
    EGridLayerTypes_Resources_Gold = 0x7
    EGridLayerTypes_Resources_Emeril = 0x8
    EGridLayerTypes_NumTypes = 0x9

class eEmissionRateType(IntEnum):
    EEmissionRateType_PerParticle = 0x0
    EEmissionRateType_Distance = 0x1
    EEmissionRateType_NumTypes = 0x2

class XMLError(IntEnum):
    eXMLErrorNone = 0x0
    eXMLErrorMissingEndTag = 0x1
    eXMLErrorNoXMLTagFound = 0x2
    eXMLErrorEmpty = 0x3
    eXMLErrorMissingTagName = 0x4
    eXMLErrorMissingEndTagName = 0x5
    eXMLErrorUnmatchedEndTag = 0x6
    eXMLErrorUnmatchedEndClearTag = 0x7
    eXMLErrorUnexpectedToken = 0x8
    eXMLErrorNoElements = 0x9
    eXMLErrorFileNotFound = 0xA
    eXMLErrorFirstTagNotFound = 0xB
    eXMLErrorUnknownCharacterEntity = 0xC
    eXMLErrorCharConversionError = 0xD
    eXMLErrorCannotOpenWriteFile = 0xE
    eXMLErrorCannotWriteFile = 0xF
    eXMLErrorBase64DataSizeIsNotMultipleOf4 = 0x10
    eXMLErrorBase64DecodeIllegalCharacter = 0x11
    eXMLErrorBase64DecodeTruncatedData = 0x12
    eXMLErrorBase64DecodeBufferTooSmall = 0x13

class eFeatureTypes(IntEnum):
    EFeatureTypes_River = 0x0
    EFeatureTypes_Crater = 0x1
    EFeatureTypes_Arches = 0x2
    EFeatureTypes_ArchesSmall = 0x3
    EFeatureTypes_Blobs = 0x4
    EFeatureTypes_BlobsSmall = 0x5
    EFeatureTypes_Substance = 0x6
    EFeatureTypes_NumTypes = 0x7

class eInputButton(IntEnum):
    EInputButton_None = 0x0
    EInputButton_Space = 0x20
    EInputButton_Exclamation = 0x21
    EInputButton_Quotes = 0x22
    EInputButton_Hash = 0x23
    EInputButton_Dollar = 0x24
    EInputButton_Percent = 0x25
    EInputButton_Ampersand = 0x26
    EInputButton_Apostrophe = 0x27
    EInputButton_LeftBracket = 0x28
    EInputButton_RightBracket = 0x29
    EInputButton_Asterisk = 0x2A
    EInputButton_Plus = 0x2B
    EInputButton_Comma = 0x2C
    EInputButton_Hyphen = 0x2D
    EInputButton_Period = 0x2E
    EInputButton_Slash = 0x2F
    EInputButton_Key0 = 0x30
    EInputButton_Key1 = 0x31
    EInputButton_Key2 = 0x32
    EInputButton_Key3 = 0x33
    EInputButton_Key4 = 0x34
    EInputButton_Key5 = 0x35
    EInputButton_Key6 = 0x36
    EInputButton_Key7 = 0x37
    EInputButton_Key8 = 0x38
    EInputButton_Key9 = 0x39
    EInputButton_Colon = 0x3A
    EInputButton_Semicolon = 0x3B
    EInputButton_LessThan = 0x3C
    EInputButton_Equals = 0x3D
    EInputButton_GreaterThan = 0x3E
    EInputButton_QuestionMark = 0x3F
    EInputButton_At = 0x40
    EInputButton_KeyA = 0x41
    EInputButton_KeyB = 0x42
    EInputButton_KeyC = 0x43
    EInputButton_KeyD = 0x44
    EInputButton_KeyE = 0x45
    EInputButton_KeyF = 0x46
    EInputButton_KeyG = 0x47
    EInputButton_KeyH = 0x48
    EInputButton_KeyI = 0x49
    EInputButton_KeyJ = 0x4A
    EInputButton_KeyK = 0x4B
    EInputButton_KeyL = 0x4C
    EInputButton_KeyM = 0x4D
    EInputButton_KeyN = 0x4E
    EInputButton_KeyO = 0x4F
    EInputButton_KeyP = 0x50
    EInputButton_KeyQ = 0x51
    EInputButton_KeyR = 0x52
    EInputButton_KeyS = 0x53
    EInputButton_KeyT = 0x54
    EInputButton_KeyU = 0x55
    EInputButton_KeyV = 0x56
    EInputButton_KeyW = 0x57
    EInputButton_KeyX = 0x58
    EInputButton_KeyY = 0x59
    EInputButton_KeyZ = 0x5A
    EInputButton_LeftSquare = 0x5B
    EInputButton_BackSlash = 0x5C
    EInputButton_RightSquare = 0x5D
    EInputButton_Caret = 0x5E
    EInputButton_Underscode = 0x5F
    EInputButton_Grave = 0x60
    EInputButton_LeftCurly = 0x7B
    EInputButton_Bar = 0x7C
    EInputButton_RightCurly = 0x7D
    EInputButton_Tilde = 0x7E
    EInputButton_NumTypes = 0xAE
    EInputButton_Escape = 0x100
    EInputButton_Enter = 0x101
    EInputButton_Backspace = 0x102
    EInputButton_Insert = 0x103
    EInputButton_Delete = 0x104
    EInputButton_Home = 0x105
    EInputButton_End = 0x106
    EInputButton_PageUp = 0x107
    EInputButton_PageDown = 0x108
    EInputButton_F1 = 0x109
    EInputButton_F2 = 0x10A
    EInputButton_F3 = 0x10B
    EInputButton_F4 = 0x10C
    EInputButton_F5 = 0x10D
    EInputButton_F6 = 0x10E
    EInputButton_F7 = 0x10F
    EInputButton_F8 = 0x110
    EInputButton_F9 = 0x111
    EInputButton_F10 = 0x112
    EInputButton_F11 = 0x113
    EInputButton_F12 = 0x114
    EInputButton_Tab = 0x115
    EInputButton_Shift = 0x116
    EInputButton_LShift = 0x117
    EInputButton_RShift = 0x118
    EInputButton_Alt = 0x119
    EInputButton_LAlt = 0x11A
    EInputButton_RAlt = 0x11B
    EInputButton_Ctrl = 0x11C
    EInputButton_LCtrl = 0x11D
    EInputButton_RCtrl = 0x11E
    EInputButton_Up = 0x11F
    EInputButton_Down = 0x120
    EInputButton_Left = 0x121
    EInputButton_Right = 0x122
    EInputButton_KeyboardUnbound = 0x123
    EInputButton_Mouse1 = 0x124
    EInputButton_Mouse2 = 0x125
    EInputButton_Mouse3 = 0x126
    EInputButton_Mouse4 = 0x127
    EInputButton_Mouse5 = 0x128
    EInputButton_Mouse6 = 0x129
    EInputButton_Mouse7 = 0x12A
    EInputButton_Mouse8 = 0x12B
    EInputButton_MouseWheelUp = 0x12C
    EInputButton_MouseWheelDown = 0x12D
    EInputButton_MouseUnbound = 0x12E
    EInputButton_PadA = 0x12F
    EInputButton_PadB = 0x130
    EInputButton_PadC = 0x131
    EInputButton_PadD = 0x132
    EInputButton_PadStart = 0x133
    EInputButton_PadSelect = 0x134
    EInputButton_PadLeftShoulder1 = 0x135
    EInputButton_PadRightShoulder1 = 0x136
    EInputButton_PadLeftShoulder2 = 0x137
    EInputButton_PadRightShoulder2 = 0x138
    EInputButton_PadLeftTrigger = 0x139
    EInputButton_PadRightTrigger = 0x13A
    EInputButton_PadLeftThumb = 0x13B
    EInputButton_PadRightThumb = 0x13C
    EInputButton_PadUp = 0x13D
    EInputButton_PadDown = 0x13E
    EInputButton_PadLeft = 0x13F
    EInputButton_PadRight = 0x140
    EInputButton_LeftHandA = 0x141
    EInputButton_LeftHandB = 0x142
    EInputButton_LeftHandC = 0x143
    EInputButton_LeftHandD = 0x144
    EInputButton_ChordBothShoulders = 0x145
    EInputButton_PadLeftTriggerSpecial = 0x146
    EInputButton_PadRightTriggerSpecial = 0x147
    EInputButton_TouchscreenPress = 0x148
    EInputButton_PadSpecial0 = 0x149
    EInputButton_PadSpecial1 = 0x14A
    EInputButton_PadSpecial2 = 0x14B
    EInputButton_PadSpecial3 = 0x14C
    EInputButton_PadSpecial4 = 0x14D
    EInputButton_PadSpecial5 = 0x14E
    EInputButton_PadSpecial6 = 0x14F
    EInputButton_PadSpecial7 = 0x150
    EInputButton_PadSpecial8 = 0x151
    EInputButton_PadSpecial9 = 0x152
    EInputButton_PadSpecial10 = 0x153
    EInputButton_PadSpecial11 = 0x154
    EInputButton_PadSpecial12 = 0x155
    EInputButton_PadSpecial13 = 0x156
    EInputButton_PadSpecial14 = 0x157
    EInputButton_PadSpecial15 = 0x158
    EInputButton_PadSpecial16 = 0x159
    EInputButton_PadSpecial17 = 0x15A
    EInputButton_PadSpecial18 = 0x15B
    EInputButton_PadSpecial19 = 0x15C
    EInputButton_PadUnbound = 0x15D
    EInputButton_Gesture = 0x15E
    EInputButton_GestureLeftWrist = 0x15F
    EInputButton_GestureRightWrist = 0x160
    EInputButton_GestureBinoculars = 0x161
    EInputButton_GestureBackpack = 0x162
    EInputButton_GestureExitVehicle = 0x163
    EInputButton_GestureThrottle = 0x164
    EInputButton_GestureFlightStick = 0x165
    EInputButton_GestureTeleport = 0x166
    EInputButton_MaxEnumValue = 0x167

class eDragType(IntEnum):
    EDragType_IgnoreGravity = 0x0
    EDragType_PhysicallyBased = 0x1
    EDragType_ApplyWind = 0x2
    EDragType_NumTypes = 0x3

class eSpawnOffsetType(IntEnum):
    ESpawnOffsetType_Sphere = 0x0
    ESpawnOffsetType_Box = 0x1
    ESpawnOffsetType_Disc = 0x2
    ESpawnOffsetType_Cone = 0x3
    ESpawnOffsetType_Donut = 0x4
    ESpawnOffsetType_Point = 0x5
    ESpawnOffsetType_NumTypes = 0x6

class eFeatureType(IntEnum):
    EFeatureType_Tube = 0x0
    EFeatureType_Blob = 0x1
    EFeatureType_NumTypes = 0x2

class eMergeProbabilityOptions(IntEnum):
    EMergeProbabilityOptions_Balance = 0x0
    EMergeProbabilityOptions_Prioritize = 0x1
    EMergeProbabilityOptions_Replace = 0x2
    EMergeProbabilityOptions_NumTypes = 0x3

class eAuthoring(IntEnum):
    EAuthoring_FixedValue = 0x0
    EAuthoring_RandomRangeFloat = 0x1
    EAuthoring_Curves = 0x2
    EAuthoring_NumTypes = 0x3

class eAlignment(IntEnum):
    EAlignment_Rotation = 0x0
    EAlignment_Velocity = 0x1
    EAlignment_VelocityScreenSpace = 0x2
    EAlignment_NumTypes = 0x3

class eBillboardAlignment(IntEnum):
    EBillboardAlignment_Screen = 0x0
    EBillboardAlignment_XLocal = 0x1
    EBillboardAlignment_YLocal = 0x2
    EBillboardAlignment_ZLocal = 0x3
    EBillboardAlignment_NegativeXLocal = 0x4
    EBillboardAlignment_NegativeYLocal = 0x5
    EBillboardAlignment_NegativeZLocal = 0x6
    EBillboardAlignment_ScreenWorld = 0x7
    EBillboardAlignment_NumTypes = 0x8

class eFlipbookPlaybackRate(IntEnum):
    EFlipbookPlaybackRate_Absolute = 0x0
    EFlipbookPlaybackRate_RelativeToMax = 0x1
    EFlipbookPlaybackRate_OnceToCompletion = 0x2
    EFlipbookPlaybackRate_Random = 0x3
    EFlipbookPlaybackRate_NumTypes = 0x4

class eOnRefractionsDisabled(IntEnum):
    EOnRefractionsDisabled_Hide = 0x0
    EOnRefractionsDisabled_AlphaBlend = 0x1
    EOnRefractionsDisabled_NumTypes = 0x2

class eFlattening(IntEnum):
    EFlattening_None = 0x0
    EFlattening_Flatten = 0x1
    EFlattening_TerrainEdits = 0x2
    EFlattening_NumTypes = 0x3

class eWaterPlacement(IntEnum):
    EWaterPlacement_None = 0x0
    EWaterPlacement_OnWater = 0x1
    EWaterPlacement_Underwater = 0x2
    EWaterPlacement_UnderwaterOnly = 0x3
    EWaterPlacement_NumTypes = 0x4

class eRuleType(IntEnum):
    ERuleType_Default = 0x0
    ERuleType_BaseRule = 0x1
    ERuleType_NumTypes = 0x2

class eEmitterShape(IntEnum):
    EEmitterShape_Sphere = 0x0
    EEmitterShape_UpperHalfSphere = 0x1
    EEmitterShape_BottomHalfSphere = 0x2
    EEmitterShape_NumTypes = 0x3

class eFeedbackType(IntEnum):
    EFeedbackType_None = 0x0
    EFeedbackType_Feedback = 0x1
    EFeedbackType_Weapon = 0x2
    EFeedbackType_Vibration = 0x3
    EFeedbackType_NumTypes = 0x4

class eInputAxis(IntEnum):
    EInputAxis_None = 0x0
    EInputAxis_LeftStick = 0x1
    EInputAxis_LeftStickX = 0x2
    EInputAxis_LeftStickY = 0x3
    EInputAxis_RightStick = 0x4
    EInputAxis_RightStickX = 0x5
    EInputAxis_RightStickY = 0x6
    EInputAxis_LeftTrigger = 0x7
    EInputAxis_RightTrigger = 0x8
    EInputAxis_Mouse = 0x9
    EInputAxis_MouseX = 0xA
    EInputAxis_MouseY = 0xB
    EInputAxis_MousePositiveX = 0xC
    EInputAxis_MouseNegativeX = 0xD
    EInputAxis_MousePositiveY = 0xE
    EInputAxis_MouseNegativeY = 0xF
    EInputAxis_MouseWheel = 0x10
    EInputAxis_MouseWheelPositive = 0x11
    EInputAxis_MouseWheelNegative = 0x12
    EInputAxis_Touchpad = 0x13
    EInputAxis_TouchpadX = 0x14
    EInputAxis_TouchpadY = 0x15
    EInputAxis_TouchpadPositiveX = 0x16
    EInputAxis_TouchpadNegativeX = 0x17
    EInputAxis_TouchpadPositiveY = 0x18
    EInputAxis_TouchpadNegativeY = 0x19
    EInputAxis_LeftTouchpad = 0x1A
    EInputAxis_LeftTouchpadX = 0x1B
    EInputAxis_LeftTouchpadY = 0x1C
    EInputAxis_LeftTouchpadPositiveX = 0x1D
    EInputAxis_LeftTouchpadNegativeX = 0x1E
    EInputAxis_LeftTouchpadPositiveY = 0x1F
    EInputAxis_LeftTouchpadNegativeY = 0x20
    EInputAxis_LeftGrip = 0x21
    EInputAxis_RightGrip = 0x22
    EInputAxis_LeftStickPositiveX = 0x23
    EInputAxis_LeftStickNegativeX = 0x24
    EInputAxis_LeftStickPositiveY = 0x25
    EInputAxis_LeftStickNegativeY = 0x26
    EInputAxis_RightStickPositiveX = 0x27
    EInputAxis_RightStickNegativeX = 0x28
    EInputAxis_RightStickPositiveY = 0x29
    EInputAxis_RightStickNegativeY = 0x2A
    EInputAxis_DirectionalPadX = 0x2B
    EInputAxis_DirectionalPadY = 0x2C
    EInputAxis_DirectionalButtonsX = 0x2D
    EInputAxis_DirectionalButtonsY = 0x2E
    EInputAxis_ChordAD = 0x2F
    EInputAxis_FakeLeftStick = 0x30
    EInputAxis_FakeRightStick = 0x31
    EInputAxis_NumTypes = 0x33

class eTkMemType(IntEnum):
    EMemoryPool_Physics = 0x0
    EMemoryPool_SingleBuffer = 0x1
    EMemoryPool_DoubleBuffer = 0x2
    EMemoryPool_DoubleBufferPausable = 0x3
    EMemoryPool_IO = 0x4
    EMemoryPool_Network = 0x5
    EMemoryPool_Resources = 0x6
    EMemoryPool_Audio = 0x7
    EMemoryPool_Graphics = 0x8
    EMemoryPool_Compute = 0x9
    EMemoryPool_Nodes = 0xA
    EMemoryPool_AnimStages = 0xB
    EMemoryPool_BlockAllocator = 0xC
    EMemoryPool_GraphicsContext = 0xD
    EMemoryPool_CommandBuffers = 0xE
    EMemoryPool_Debug = 0xF
    EMemoryPool_Metadata = 0x10
    EMemoryPool_NodeAllocator = 0x11
    ENumTkMemoryPools = 0x12

class eMetadataReadMask(IntEnum):
    EMetadataReadMask_Default = 0x1
    EMetadataReadMask_SaveWhenMultiplayerClient = 0x2
    EMetadataReadMask_SavePlayerPosition = 0x4
    EMetadataReadMask_NumTypes = 0x5
    EMetadataReadMask_SavePlayerInventory = 0x8
    EMetadataReadMask_SaveDifficultySettings = 0x10

class eNoiseGridType(IntEnum):
    ENoiseGridType_Cube = 0x0
    ENoiseGridType_Cone = 0x1
    ENoiseGridType_Torus = 0x2
    ENoiseGridType_Sphere = 0x3
    ENoiseGridType_Cylinder = 0x4
    ENoiseGridType_Capsule = 0x5
    ENoiseGridType_Corridor = 0x6
    ENoiseGridType_Pipe = 0x7
    ENoiseGridType_Puck = 0x8
    ENoiseGridType_SuperPrimitiveRandom = 0x9
    ENoiseGridType_SuperFormula_01 = 0xA
    ENoiseGridType_SuperFormula_02 = 0xB
    ENoiseGridType_SuperFormula_03 = 0xC
    ENoiseGridType_SuperFormula_04 = 0xD
    ENoiseGridType_SuperFormula_05 = 0xE
    ENoiseGridType_SuperFormula_06 = 0xF
    ENoiseGridType_SuperFormula_07 = 0x10
    ENoiseGridType_SuperFormula_08 = 0x11
    ENoiseGridType_SuperFormulaRandom = 0x12
    ENoiseGridType_SuperFormula = 0x13
    ENoiseGridType_SuperPrimitive = 0x14
    ENoiseGridType_File = 0x15
    ENoiseGridType_NumTypes = 0x16

class eTerrainSettings(IntEnum):
    ETerrainSettings_FloatingIslands = 0x0
    ETerrainSettings_GrandCanyon = 0x1
    ETerrainSettings_MountainRavines = 0x2
    ETerrainSettings_HugeArches = 0x3
    ETerrainSettings_Alien = 0x4
    ETerrainSettings_Craters = 0x5
    ETerrainSettings_Caverns = 0x6
    ETerrainSettings_Alpine = 0x7
    ETerrainSettings_LilyPad = 0x8
    ETerrainSettings_Desert = 0x9
    ETerrainSettings_WaterworldPrime = 0xA
    ETerrainSettings_FloatingIslandsPrime = 0xB
    ETerrainSettings_GrandCanyonPrime = 0xC
    ETerrainSettings_MountainRavinesPrime = 0xD
    ETerrainSettings_HugeArchesPrime = 0xE
    ETerrainSettings_AlienPrime = 0xF
    ETerrainSettings_CratersPrime = 0x10
    ETerrainSettings_CavernsPrime = 0x11
    ETerrainSettings_AlpinePrime = 0x12
    ETerrainSettings_LilyPadPrime = 0x13
    ETerrainSettings_DesertPrime = 0x14
    ETerrainSettings_NumTypes = 0x15

class eNoiseType(IntEnum):
    ENoiseType_Plane = 0x0
    ENoiseType_Check = 0x1
    ENoiseType_Sine = 0x2
    ENoiseType_Smooth = 0x3
    ENoiseType_Fractal = 0x4
    ENoiseType_Ridged = 0x5
    ENoiseType_Billow = 0x6
    ENoiseType_Erosion = 0x7
    ENoiseType_Volcanic = 0x8
    ENoiseType_Glacial = 0x9
    ENoiseType_Plateau = 0xA
    ENoiseType_NumTypes = 0xB

class eShader(IntEnum):
    EShader_UberShader = 0x0
    EShader_Sky = 0x1
    EShader_Screen = 0x2
    EShader_UberHack = 0x3
    EShader_UIScreen = 0x4
    EShader_ShaderMillParticle = 0x5
    EShader_ShaderMillUber = 0x6
    EShader_NumTypes = 0x7

class eDecalNormal(IntEnum):
    EDecalNormal_None = 0x0
    EDecalNormal_Decal = 0x1
    EDecalNormal_Vertex = 0x2
    EDecalNormal_NumTypes = 0x3

class eInputAction(IntEnum):
    EInputAction_Invalid = 0x0
    EInputAction_Player_Forward = 0x1
    EInputAction_Player_Back = 0x2
    EInputAction_Player_Left = 0x3
    EInputAction_Player_Right = 0x4
    EInputAction_Player_SwimUp = 0x5
    EInputAction_Player_SwimDown = 0x6
    EInputAction_Player_Interact = 0x7
    EInputAction_Player_Melee = 0x8
    EInputAction_Player_Scan = 0x9
    EInputAction_Player_Torch = 0xA
    EInputAction_Player_Binoculars = 0xB
    EInputAction_Player_Zoom = 0xC
    EInputAction_Player_ShowHUD = 0xD
    EInputAction_Player_Jump = 0xE
    EInputAction_Player_Run = 0xF
    EInputAction_Player_Shoot = 0x10
    EInputAction_Player_Grenade = 0x11
    EInputAction_Player_Reload = 0x12
    EInputAction_Player_ChangeWeapon = 0x13
    EInputAction_Player_Deconstruct = 0x14
    EInputAction_Player_ChangeAltWeapon = 0x15
    EInputAction_Player_PlaceMarker = 0x16
    EInputAction_Quick_Menu = 0x17
    EInputAction_Build_Menu = 0x18
    EInputAction_Ship_AltLeft = 0x19
    EInputAction_Ship_AltRight = 0x1A
    EInputAction_Ship_Thrust = 0x1B
    EInputAction_Ship_Brake = 0x1C
    EInputAction_Ship_Boost = 0x1D
    EInputAction_Ship_RollLeft = 0x1E
    EInputAction_Ship_RollRight = 0x1F
    EInputAction_Ship_Exit = 0x20
    EInputAction_Ship_Land = 0x21
    EInputAction_Ship_Shoot = 0x22
    EInputAction_Ship_ChangeWeapon = 0x23
    EInputAction_Ship_Scan = 0x24
    EInputAction_Ship_PulseJump = 0x25
    EInputAction_Ship_GalacticMap = 0x26
    EInputAction_Ship_TurnLeft = 0x27
    EInputAction_Ship_TurnRight = 0x28
    EInputAction_Ship_FreeLook = 0x29
    EInputAction_Ship_AutoFollow_Toggle = 0x2A
    EInputAction_Ship_AutoFollow_Hold = 0x2B
    EInputAction_Vehicle_Forward = 0x2C
    EInputAction_Vehicle_Reverse = 0x2D
    EInputAction_Vehicle_Left = 0x2E
    EInputAction_Vehicle_Right = 0x2F
    EInputAction_Vehicle_Exit = 0x30
    EInputAction_Vehicle_Shoot = 0x31
    EInputAction_Vehicle_ChangeWeapon = 0x32
    EInputAction_Vehicle_Scan = 0x33
    EInputAction_Vehicle_Boost = 0x34
    EInputAction_Vehicle_Jump = 0x35
    EInputAction_Vehicle_Horn = 0x36
    EInputAction_Vehicle_AddCheckpoint = 0x37
    EInputAction_Vehicle_DeleteCheckpoint = 0x38
    EInputAction_Fe_Select = 0x39
    EInputAction_Fe_AltSelect = 0x3A
    EInputAction_Fe_SelectX = 0x3B
    EInputAction_Fe_Back = 0x3C
    EInputAction_Fe_Alt1 = 0x3D
    EInputAction_Fe_Alt1X = 0x3E
    EInputAction_Fe_Transfer = 0x3F
    EInputAction_Fe_Destroy = 0x40
    EInputAction_Gm_Select = 0x41
    EInputAction_Gm_Restart = 0x42
    EInputAction_UI_Left = 0x43
    EInputAction_UI_Right = 0x44
    EInputAction_UI_Left_Sub = 0x45
    EInputAction_UI_Right_Sub = 0x46
    EInputAction_UI_Down_Sub = 0x47
    EInputAction_UI_Up_Sub = 0x48
    EInputAction_UI_NetworkPageShortcut = 0x49
    EInputAction_UI_StackSplitUp = 0x4A
    EInputAction_UI_StackSplitDown = 0x4B
    EInputAction_Fe_ExitMenu = 0x4C
    EInputAction_Fe_Options = 0x4D
    EInputAction_Fe_Quit = 0x4E
    EInputAction_Fe_MsgSkip = 0x4F
    EInputAction_Fe_TouchscreenPress = 0x50
    EInputAction_Quick_Left = 0x51
    EInputAction_Quick_Right = 0x52
    EInputAction_Quick_Action = 0x53
    EInputAction_Quick_Back = 0x54
    EInputAction_Quick_Up = 0x55
    EInputAction_Quick_Down = 0x56
    EInputAction_Build_Place = 0x57
    EInputAction_Build_Rotate_Left = 0x58
    EInputAction_Build_Rotate_Right = 0x59
    EInputAction_Build_AnalogRotateMode1 = 0x5A
    EInputAction_Build_AnalogRotateMode2 = 0x5B
    EInputAction_Build_AnalogRotateLeftY = 0x5C
    EInputAction_Build_AnalogRotateRightY = 0x5D
    EInputAction_Build_AnalogRotateY = 0x5E
    EInputAction_Build_AnalogRotateLeftZ = 0x5F
    EInputAction_Build_AnalogRotateRightZ = 0x60
    EInputAction_Build_AnalogRotateZ = 0x61
    EInputAction_Build_ScaleUp = 0x62
    EInputAction_Build_ScaleDown = 0x63
    EInputAction_Build_AnalogueScale = 0x64
    EInputAction_Build_SelectionMode = 0x65
    EInputAction_Build_Camera = 0x66
    EInputAction_Photo_Hide = 0x67
    EInputAction_Photo_Sun = 0x68
    EInputAction_Photo_Cam = 0x69
    EInputAction_Photo_Exit = 0x6A
    EInputAction_Photo_CamDown = 0x6B
    EInputAction_Photo_CamUp = 0x6C
    EInputAction_Photo_Capture = 0x6D
    EInputAction_Ambient_Camera = 0x6E
    EInputAction_Ambient_Planet = 0x6F
    EInputAction_Ambient_System = 0x70
    EInputAction_Ambient_Photo = 0x71
    EInputAction_Ambient_NxtSong = 0x72
    EInputAction_Ambient_Spawn = 0x73
    EInputAction_Terrain_Edit = 0x74
    EInputAction_Terrain_ModeBack = 0x75
    EInputAction_Terrain_Menu = 0x76
    EInputAction_Terrain_SizeUp = 0x77
    EInputAction_Terrain_SizeDown = 0x78
    EInputAction_Terrain_RotTerrainLeft = 0x79
    EInputAction_Terrain_RotTerrainRight = 0x7A
    EInputAction_Terrain_ChangeShape = 0x7B
    EInputAction_Ship_NextTarget = 0x7C
    EInputAction_Ship_PreviousTarget = 0x7D
    EInputAction_Ship_ClosestTarget = 0x7E
    EInputAction_CameraLook = 0x7F
    EInputAction_CameraLookX = 0x80
    EInputAction_CameraLookY = 0x81
    EInputAction_PlayerMove = 0x82
    EInputAction_PlayerMoveX = 0x83
    EInputAction_PlayerMoveY = 0x84
    EInputAction_SpaceshipThrust = 0x85
    EInputAction_SpaceshipBrake = 0x86
    EInputAction_VehicleMove = 0x87
    EInputAction_VehicleSteer = 0x88
    EInputAction_VehicleThrust = 0x89
    EInputAction_VehicleBrake = 0x8A
    EInputAction_ShipStrafe = 0x8B
    EInputAction_ShipStrafeHorizontal = 0x8C
    EInputAction_ShipStrafeVertical = 0x8D
    EInputAction_HeldRotate = 0x8E
    EInputAction_HeldRotateLeft = 0x8F
    EInputAction_HeldRotateRight = 0x90
    EInputAction_ShipSteer = 0x91
    EInputAction_ShipTurn = 0x92
    EInputAction_ShipPitch = 0x93
    EInputAction_ShipLook = 0x94
    EInputAction_ShipLookX = 0x95
    EInputAction_ShipLookY = 0x96
    EInputAction_ShipLand = 0x97
    EInputAction_ShipPulse = 0x98
    EInputAction_PlayerSmoothTurnLeft = 0x99
    EInputAction_PlayerSmoothTurnRight = 0x9A
    EInputAction_PlayerSnapTurnLeft = 0x9B
    EInputAction_PlayerSnapTurnRight = 0x9C
    EInputAction_PlayerSnapTurnAround = 0x9D
    EInputAction_PlayerMoveAround = 0x9E
    EInputAction_TeleportDirection = 0x9F
    EInputAction_PlayerAutoWalk = 0xA0
    EInputAction_InteractLeft = 0xA1
    EInputAction_MeleeLeft = 0xA2
    EInputAction_HandCtrlHolster = 0xA3
    EInputAction_ShipUp = 0xA4
    EInputAction_ShipDown = 0xA5
    EInputAction_ShipLeft = 0xA6
    EInputAction_ShipRight = 0xA7
    EInputAction_ShipZoom = 0xA8
    EInputAction_Inventory = 0xA9
    EInputAction_DiscoveryNetworkRetry = 0xAA
    EInputAction_QuitGame = 0xAB
    EInputAction_ReportBase = 0xAC
    EInputAction_GalacticMap_Select = 0xAD
    EInputAction_GalacticMap_Deselect = 0xAE
    EInputAction_GalacticMap_Exit = 0xAF
    EInputAction_GalacticMap_Scan = 0xB0
    EInputAction_GalacticMap_Home = 0xB1
    EInputAction_GalacticMap_PlanetBase = 0xB2
    EInputAction_GalacticMap_Accelerate = 0xB3
    EInputAction_GalacticMap_ExpandMenu = 0xB4
    EInputAction_GalacticMap_ScreenshotToggle = 0xB5
    EInputAction_GalacticMap_ScanChooseNext = 0xB6
    EInputAction_GalacticMap_ToggleWaypoint = 0xB7
    EInputAction_GalacticMap_ClearAllWaypoints = 0xB8
    EInputAction_GalacticMap_NextNavType = 0xB9
    EInputAction_GalacticMap_PreviousNavType = 0xBA
    EInputAction_GalacticMap_PreviousFilter = 0xBB
    EInputAction_GalacticMap_NextFilter = 0xBC
    EInputAction_GalacticMap_CameraLook = 0xBD
    EInputAction_GalacticMap_CameraLookX = 0xBE
    EInputAction_GalacticMap_CameraLookY = 0xBF
    EInputAction_GalacticMap_PlayerMove = 0xC0
    EInputAction_GalacticMap_PlayerMoveX = 0xC1
    EInputAction_GalacticMap_PlayerMoveY = 0xC2
    EInputAction_GalacticMap_PlayerMoveForward = 0xC3
    EInputAction_GalacticMap_PlayerMoveBackward = 0xC4
    EInputAction_GalacticMap_PlayerMoveLeft = 0xC5
    EInputAction_GalacticMap_PlayerMoveRight = 0xC6
    EInputAction_GalacticMap_Up = 0xC7
    EInputAction_GalacticMap_Down = 0xC8
    EInputAction_GalacticMap_Gesture = 0xC9
    EInputAction_UI_Cursor = 0xCA
    EInputAction_UI_CursorX = 0xCB
    EInputAction_UI_CursorY = 0xCC
    EInputAction_UI_Camera = 0xCD
    EInputAction_UI_CameraX = 0xCE
    EInputAction_UI_CameraY = 0xCF
    EInputAction_UI_ViewPlayerInfo = 0xD0
    EInputAction_UI_NextInputControls = 0xD1
    EInputAction_UI_PrevInputControls = 0xD2
    EInputAction_UI_ToggleBuySell = 0xD3
    EInputAction_UI_ToggleTradeInventory = 0xD4
    EInputAction_UI_TouchScrollY = 0xD5
    EInputAction_UI_TouchScrollX = 0xD6
    EInputAction_CharacterCustomisation_ShowCharacter = 0xD7
    EInputAction_UI_CharacterCustomisation_Camera = 0xD8
    EInputAction_UI_CharacterCustomisation_RotateCamera = 0xD9
    EInputAction_UI_CharacterCustomisation_PitchCamera = 0xDA
    EInputAction_GameMode_CancelDelete = 0xDB
    EInputAction_GameMode_TitleStart = 0xDC
    EInputAction_GameMode_ChangeUser = 0xDD
    EInputAction_Binocs_NextMode = 0xDE
    EInputAction_Binocs_PrevMode = 0xDF
    EInputAction_BaseBuilding_PinRecipe = 0xE0
    EInputAction_BaseBuilding_SwitchBase = 0xE1
    EInputAction_PhotoMode_CatLeft = 0xE2
    EInputAction_PhotoMode_CatRight = 0xE3
    EInputAction_PhotoMode_ValueIncrease = 0xE4
    EInputAction_PhotoMode_ValueDecrease = 0xE5
    EInputAction_PhotoMode_OptionUp = 0xE6
    EInputAction_PhotoMode_OptionDown = 0xE7
    EInputAction_PhotoMode_CameraRollLeft = 0xE8
    EInputAction_PhotoMode_CameraRollRight = 0xE9
    EInputAction_PhotoMode_PauseApplication = 0xEA
    EInputAction_PhotoMode_CopyLocation = 0xEB
    EInputAction_PhotoMode_HideLocation = 0xEC
    EInputAction_UI_Up_Sub_Discovery = 0xED
    EInputAction_UI_Down_Sub_Discovery = 0xEE
    EInputAction_Fe_Upload_Discovery = 0xEF
    EInputAction_HMD_Recenter = 0xF0
    EInputAction_HMD_Recenter2 = 0xF1
    EInputAction_HMD_FEOpen = 0xF2
    EInputAction_TextChatOpenClose = 0xF3
    EInputAction_TextChatSend = 0xF4
    EInputAction_TextChatPasteHold = 0xF5
    EInputAction_TextChatPaste = 0xF6
    EInputAction_TextChatAutocomplete = 0xF7
    EInputAction_TextChatAutocompleteModifier = 0xF8
    EInputAction_TextChatCursorLeft = 0xF9
    EInputAction_TextChatCursorRight = 0xFA
    EInputAction_TextChatCursorHome = 0xFB
    EInputAction_TextChatCursorEnd = 0xFC
    EInputAction_TextChatDelete = 0xFD
    EInputAction_Player_InteractSecondary = 0xFE
    EInputAction_BaseBuilding_ToggleVisions = 0xFF
    EInputAction_BaseBuilding_Browse = 0x100
    EInputAction_BaseBuilding_Pickup = 0x101
    EInputAction_BaseBuilding_Duplicate = 0x102
    EInputAction_BaseBuilding_Delete = 0x103
    EInputAction_BaseBuilding_ToggleRotationAxis = 0x104
    EInputAction_Build_AnalogRotateZ2 = 0x105
    EInputAction_BaseBuilding_ToggleSnapping = 0x106
    EInputAction_BaseBuilding_ToggleWiring = 0x107
    EInputAction_BaseBuilding_Paint = 0x108
    EInputAction_BaseBuilding_NextPart = 0x109
    EInputAction_TogglePause = 0x10A
    EInputAction_TogglePlanet = 0x10B
    EInputAction_Suicide = 0x10C
    EInputAction_Reset = 0x10D
    EInputAction_AddLastToolbox = 0x10E
    EInputAction_AddLastToolboxAtPos = 0x10F
    EInputAction_TerrainInvalidate = 0x110
    EInputAction_TogglePipeline = 0x111
    EInputAction_TakeScreenshot = 0x112
    EInputAction_TakeExrScreenshot = 0x113
    EInputAction_ToggleDebugStats = 0x114
    EInputAction_ToggleDebugSubpage = 0x115
    EInputAction_DumpNodeStats = 0x116
    EInputAction_ToggleTaa = 0x117
    EInputAction_DebugDropMeasurementAnchor = 0x118
    EInputAction_QuickWarp = 0x119
    EInputAction_DumpStats = 0x11A
    EInputAction_DiscoverOwnBase = 0x11B
    EInputAction_ClearTerrainEdits = 0x11C
    EInputAction_SelectRegion = 0x11D
    EInputAction_SwitchRegionRow = 0x11E
    EInputAction_SwitchRegionAxis = 0x11F
    EInputAction_OpenLog = 0x120
    EInputAction_DumpVertStats = 0x121
    EInputAction_Player_TagMarker = 0x122
    EInputAction_NumTypes = 0x123

class XMLElementType(IntEnum):
    eNodeChild = 0x0
    eNodeAttribute = 0x1
    eNodeText = 0x2
    eNodeClear = 0x3
    eNodeNULL = 0x4

class eFileOpenMode(IntEnum):
    EFOM_Read = 0x0
    EFOM_Write = 0x1
    EFOM_Append = 0x2

class TkFileAsyncOpStatus(IntEnum):
    TK_FILE_ASYNC_OP_STATUS_PENDING = 0x0
    TK_FILE_ASYNC_OP_STATUS_COMPLETE = 0x1
    TK_FILE_ASYNC_OP_STATUS_ERROR = 0x2
    TK_FILE_ASYNC_OP_STATUS_INVALID = 0x3

class eSeekMode(IntEnum):
    ESM_Set = 0x0
    ESM_Current = 0x1
    ESM_End = 0x2

class eInstancing(IntEnum):
    EInstancing_PerVertex = 0x0
    EInstancing_PerModel = 0x1
    EInstancing_NumTypes = 0x2

class eTriggerType(IntEnum):
    ETriggerType_Disabled = 0x0
    ETriggerType_Interrupt = 0x1
    ETriggerType_RunParallel = 0x2
    ETriggerType_Blocked = 0x3
    ETriggerType_QueueLatest = 0x4
    ETriggerType_QueueAll = 0x5
    ETriggerType_NumTypes = 0x6

class eTextureType(IntEnum):
    ETextureType_Diffuse = 0x0
    ETextureType_Normal = 0x1
    ETextureType_Ambient = 0x2
    ETextureType_Environment = 0x3
    ETextureType_NumTypes = 0x4

class eMaterialFlag(IntEnum):
    EMaterialFlag__F01_DIFFUSEMAP = 0x0
    EMaterialFlag__F02_SKINNED = 0x1
    EMaterialFlag__F03_NORMALMAP = 0x2
    EMaterialFlag__F04_FEATURESMAP = 0x3
    EMaterialFlag__F05_INVERT_ALPHA = 0x4
    EMaterialFlag__F06_BRIGHT_EDGE = 0x5
    EMaterialFlag__F07_UNLIT = 0x6
    EMaterialFlag__F08_REFLECTIVE = 0x7
    EMaterialFlag__F09_TRANSPARENT = 0x8
    EMaterialFlag__F10_NORECEIVESHADOW = 0x9
    EMaterialFlag__F11_ALPHACUTOUT = 0xA
    EMaterialFlag__F12_BATCHED_BILLBOARD = 0xB
    EMaterialFlag__F13_UVANIMATION = 0xC
    EMaterialFlag__F14_UVSCROLL = 0xD
    EMaterialFlag__F15_WIND = 0xE
    EMaterialFlag__F16_DIFFUSE2MAP = 0xF
    EMaterialFlag__F17_MULTIPLYDIFFUSE2MAP = 0x10
    EMaterialFlag__F18_UVTILES = 0x11
    EMaterialFlag__F19_BILLBOARD = 0x12
    EMaterialFlag__F20_PARALLAXMAP = 0x13
    EMaterialFlag__F21_VERTEXCOLOUR = 0x14
    EMaterialFlag__F22_TRANSPARENT_SCALAR = 0x15
    EMaterialFlag__F23_TRANSLUCENT = 0x16
    EMaterialFlag__F24_AOMAP = 0x17
    EMaterialFlag__F25_ROUGHNESS_MASK = 0x18
    EMaterialFlag__F26_ = 0x19
    EMaterialFlag__F27_VBTANGENT = 0x1A
    EMaterialFlag__F28_VBSKINNED = 0x1B
    EMaterialFlag__F29_VBCOLOUR = 0x1C
    EMaterialFlag__F30_REFRACTION = 0x1D
    EMaterialFlag__F31_DISPLACEMENT = 0x1E
    EMaterialFlag__F32_REFRACTION_MASK = 0x1F
    EMaterialFlag__F33_SHELLS = 0x20
    EMaterialFlag__F34_GLOW = 0x21
    EMaterialFlag__F35_GLOW_MASK = 0x22
    EMaterialFlag__F36_DOUBLESIDED = 0x23
    EMaterialFlag__F37_ = 0x24
    EMaterialFlag__F38_NO_DEFORM = 0x25
    EMaterialFlag__F39_METALLIC_MASK = 0x26
    EMaterialFlag__F40_SUBSURFACE_MASK = 0x27
    EMaterialFlag__F41_DETAIL_DIFFUSE = 0x28
    EMaterialFlag__F42_DETAIL_NORMAL = 0x29
    EMaterialFlag__F43_NORMAL_TILING = 0x2A
    EMaterialFlag__F44_IMPOSTER = 0x2B
    EMaterialFlag__F45_VERTEX_BLEND = 0x2C
    EMaterialFlag__F46_BILLBOARD_AT = 0x2D
    EMaterialFlag__F47_REFLECTION_PROBE = 0x2E
    EMaterialFlag__F48_WARPED_DIFFUSE_LIGHTING = 0x2F
    EMaterialFlag__F49_DISABLE_AMBIENT = 0x30
    EMaterialFlag__F50_DISABLE_POSTPROCESS = 0x31
    EMaterialFlag__F51_DECAL_DIFFUSE = 0x32
    EMaterialFlag__F52_DECAL_NORMAL = 0x33
    EMaterialFlag__F53_COLOURISABLE = 0x34
    EMaterialFlag__F54_COLOURMASK = 0x35
    EMaterialFlag__F55_MULTITEXTURE = 0x36
    EMaterialFlag__F56_MATCH_GROUND = 0x37
    EMaterialFlag__F57_ENV_OVERLAY = 0x38
    EMaterialFlag__F58_USE_CENTRAL_NORMAL = 0x39
    EMaterialFlag__F59_SCREENSPACE_FADE = 0x3A
    EMaterialFlag__F60_ACUTE_ANGLE_FADE = 0x3B
    EMaterialFlag__F61_CLAMP_AMBIENT = 0x3C
    EMaterialFlag__F62_DETAIL_ALPHACUTOUT = 0x3D
    EMaterialFlag__F63_DISSOLVE = 0x3E
    EMaterialFlag__F64_ = 0x3F
    EMaterialFlag_NumTypes = 0x40

class eLanguage(IntEnum):
    ELanguage_Default = 0x0
    ELanguage_English = 0x1
    ELanguage_USEnglish = 0x2
    ELanguage_French = 0x3
    ELanguage_Italian = 0x4
    ELanguage_German = 0x5
    ELanguage_Spanish = 0x6
    ELanguage_Russian = 0x7
    ELanguage_Polish = 0x8
    ELanguage_Dutch = 0x9
    ELanguage_Portuguese = 0xA
    ELanguage_LatinAmericanSpanish = 0xB
    ELanguage_BrazilianPortuguese = 0xC
    ELanguage_Japanese = 0xD
    ELanguage_TraditionalChinese = 0xE
    ELanguage_SimplifiedChinese = 0xF
    ELanguage_TencentChinese = 0x10
    ELanguage_Korean = 0x11
    ELanguage_NumTypes = 0x12

class eTextureAddressMode(IntEnum):
    ETextureAddressMode_Wrap = 0x0
    ETextureAddressMode_Clamp = 0x1
    ETextureAddressMode_ClampToBorder = 0x2
    ETextureAddressMode_Mirror = 0x3
    ETextureAddressMode_NumTypes = 0x4

class eTextureFilterMode(IntEnum):
    ETextureFilterMode_None = 0x0
    ETextureFilterMode_Bilinear = 0x1
    ETextureFilterMode_Trilinear = 0x2
    ETextureFilterMode_NumTypes = 0x3

class eVertical(IntEnum):
    EVertical_Top = 0x0
    EVertical_Middle = 0x1
    EVertical_Bottom = 0x2
    EVertical_NumTypes = 0x3

class eHorizontal(IntEnum):
    EHorizontal_Left = 0x0
    EHorizontal_Center = 0x1
    EHorizontal_Right = 0x2
    EHorizontal_NumTypes = 0x3

class ePalette(IntEnum):
    EPalette_Grass = 0x0
    EPalette_Plant = 0x1
    EPalette_Leaf = 0x2
    EPalette_Wood = 0x3
    EPalette_Rock = 0x4
    EPalette_Stone = 0x5
    EPalette_Crystal = 0x6
    EPalette_Sand = 0x7
    EPalette_Dirt = 0x8
    EPalette_Metal = 0x9
    EPalette_Paint = 0xA
    EPalette_Plastic = 0xB
    EPalette_Fur = 0xC
    EPalette_Scale = 0xD
    EPalette_Feather = 0xE
    EPalette_Water = 0xF
    EPalette_Cloud = 0x10
    EPalette_Sky = 0x11
    EPalette_Space = 0x12
    EPalette_Underbelly = 0x13
    EPalette_Undercoat = 0x14
    EPalette_Snow = 0x15
    EPalette_SkyHorizon = 0x16
    EPalette_SkyFog = 0x17
    EPalette_SkyHeightFog = 0x18
    EPalette_SkySunset = 0x19
    EPalette_SkyNight = 0x1A
    EPalette_WaterNear = 0x1B
    EPalette_SpaceCloud = 0x1C
    EPalette_SpaceBottom = 0x1D
    EPalette_SpaceSolar = 0x1E
    EPalette_SpaceLight = 0x1F
    EPalette_Warrior = 0x20
    EPalette_Scientific = 0x21
    EPalette_Trader = 0x22
    EPalette_WarriorAlt = 0x23
    EPalette_ScientificAlt = 0x24
    EPalette_TraderAlt = 0x25
    EPalette_RockSaturated = 0x26
    EPalette_RockLight = 0x27
    EPalette_RockDark = 0x28
    EPalette_PlanetRing = 0x29
    EPalette_Custom_Head = 0x2A
    EPalette_Custom_Torso = 0x2B
    EPalette_Custom_Chest_Armour = 0x2C
    EPalette_Custom_Backpack = 0x2D
    EPalette_Custom_Arms = 0x2E
    EPalette_Custom_Hands = 0x2F
    EPalette_Custom_Legs = 0x30
    EPalette_Custom_Feet = 0x31
    EPalette_Cave = 0x32
    EPalette_GrassAlt = 0x33
    EPalette_BioShip_Body = 0x34
    EPalette_BioShip_Underbelly = 0x35
    EPalette_BioShip_Cockpit = 0x36
    EPalette_SailShip_Sails = 0x37
    EPalette_Freighter = 0x38
    EPalette_FreighterPaint = 0x39
    EPalette_NumTypes = 0x3A

class eColourAlt(IntEnum):
    EColourAlt_Primary = 0x0
    EColourAlt_Alternative1 = 0x1
    EColourAlt_Alternative2 = 0x2
    EColourAlt_Alternative3 = 0x3
    EColourAlt_Alternative4 = 0x4
    EColourAlt_Unique = 0x5
    EColourAlt_MatchGround = 0x6
    EColourAlt_None = 0x7
    EColourAlt_NumTypes = 0x8

class eWindowState(IntEnum):
    EWindowState_Open = 0x0
    EWindowState_Minimised = 0x1
    EWindowState_Closed = 0x2
    EWindowState_NumTypes = 0x3

class eProbability(IntEnum):
    EProbability_Common = 0x0
    EProbability_Uncommon = 0x1
    EProbability_Rare = 0x2
    EProbability_Extraordinary = 0x3
    EProbability_NumTypes = 0x4

class eBlackboardType(IntEnum):
    EBlackboardType_Invalid = 0x0
    EBlackboardType_Float = 0x1
    EBlackboardType_Integer = 0x2
    EBlackboardType_Bool = 0x3
    EBlackboardType_Id = 0x4
    EBlackboardType_Vector = 0x5
    EBlackboardType_Attachment = 0x6
    EBlackboardType_NumTypes = 0x7

class eTrophy(IntEnum):
    ETrophy_Trophy0 = 0x0
    ETrophy_Trophy1 = 0x1
    ETrophy_Trophy2 = 0x2
    ETrophy_Trophy3 = 0x3
    ETrophy_Trophy4 = 0x4
    ETrophy_NumTypes = 0x6
    ETrophy_None = 0xFFFFFFFF

class eTextureGameplayUse(IntEnum):
    ETextureGameplayUse_IgnoreName = 0x0
    ETextureGameplayUse_MatchName = 0x1
    ETextureGameplayUse_DoNotMatchName = 0x2
    ETextureGameplayUse_NumTypes = 0x3

class eNGuiEditorText(IntEnum):
    ENGuiEditorText_Text = 0x0
    ENGuiEditorText_Button = 0x1
    ENGuiEditorText_WindowTab = 0x2
    ENGuiEditorText_WindowTabInactive = 0x3
    ENGuiEditorText_TreeNode = 0x4
    ENGuiEditorText_CheckBox = 0x5
    ENGuiEditorText_TextInput = 0x6
    ENGuiEditorText_TextInputLabel = 0x7
    ENGuiEditorText_TextInputLabelHeader = 0x8
    ENGuiEditorText_Category = 0x9
    ENGuiEditorText_TaskBar = 0xA
    ENGuiEditorText_GroupTitle = 0xB
    ENGuiEditorText_TreeNodeSelected = 0xC
    ENGuiEditorText_DynamicPanelTitle = 0xD
    ENGuiEditorText_NumTypes = 0xE

class eNGuiEditorGraphic(IntEnum):
    ENGuiEditorGraphic_Panel = 0x0
    ENGuiEditorGraphic_Button = 0x1
    ENGuiEditorGraphic_Text = 0x2
    ENGuiEditorGraphic_Graphic = 0x3
    ENGuiEditorGraphic_WindowTitleBar = 0x4
    ENGuiEditorGraphic_WindowTitleBarInactive = 0x5
    ENGuiEditorGraphic_WindowTab = 0x6
    ENGuiEditorGraphic_WindowTabInactive = 0x7
    ENGuiEditorGraphic_Window = 0x8
    ENGuiEditorGraphic_WindowPane = 0x9
    ENGuiEditorGraphic_WindowResize = 0xA
    ENGuiEditorGraphic_WindowClose = 0xB
    ENGuiEditorGraphic_WindowMinimize = 0xC
    ENGuiEditorGraphic_WindowMaximize = 0xD
    ENGuiEditorGraphic_ScrollBarBackground = 0xE
    ENGuiEditorGraphic_ScrollBarForeground = 0xF
    ENGuiEditorGraphic_TreeNodeCollapsed = 0x10
    ENGuiEditorGraphic_TreeNodeExpanded = 0x11
    ENGuiEditorGraphic_CheckBoxTrue = 0x12
    ENGuiEditorGraphic_CheckBoxFalse = 0x13
    ENGuiEditorGraphic_TextInput = 0x14
    ENGuiEditorGraphic_Increment = 0x15
    ENGuiEditorGraphic_Decrement = 0x16
    ENGuiEditorGraphic_Cursor = 0x17
    ENGuiEditorGraphic_TextSelection = 0x18
    ENGuiEditorGraphic_Separator = 0x19
    ENGuiEditorGraphic_EditorResize = 0x1A
    ENGuiEditorGraphic_EditorMove = 0x1B
    ENGuiEditorGraphic_EditorOverlay = 0x1C
    ENGuiEditorGraphic_FileBrowser = 0x1D
    ENGuiEditorGraphic_ColourEdit = 0x1E
    ENGuiEditorGraphic_IconButton = 0x1F
    ENGuiEditorGraphic_SliderKnob = 0x20
    ENGuiEditorGraphic_SliderBar = 0x21
    ENGuiEditorGraphic_IconButtonText = 0x22
    ENGuiEditorGraphic_TextInputLabel = 0x23
    ENGuiEditorGraphic_Category = 0x24
    ENGuiEditorGraphic_Taskbar = 0x25
    ENGuiEditorGraphic_TaskbarItem = 0x26
    ENGuiEditorGraphic_StartBarWindow = 0x27
    ENGuiEditorGraphic_ComboBox = 0x28
    ENGuiEditorGraphic_ComboBoxWindow = 0x29
    ENGuiEditorGraphic_IconListItem = 0x2A
    ENGuiEditorGraphic_IconListItemSelected = 0x2B
    ENGuiEditorGraphic_ImageButton = 0x2C
    ENGuiEditorGraphic_Toolbar = 0x2D
    ENGuiEditorGraphic_ToolbarGraphic = 0x2E
    ENGuiEditorGraphic_ToolbarOptions = 0x2F
    ENGuiEditorGraphic_Rectangle = 0x30
    ENGuiEditorGraphic_TreeNodeBorder = 0x31
    ENGuiEditorGraphic_Background = 0x32
    ENGuiEditorGraphic_GroupTitle = 0x33
    ENGuiEditorGraphic_TextLabelSeparator = 0x34
    ENGuiEditorGraphic_AlignmentAnchor = 0x35
    ENGuiEditorGraphic_MinimiseHighlight = 0x36
    ENGuiEditorGraphic_Table = 0x37
    ENGuiEditorGraphic_TableBorder = 0x38
    ENGuiEditorGraphic_TableFolderButton = 0x39
    ENGuiEditorGraphic_TableAddEntryButton = 0x3A
    ENGuiEditorGraphic_TreeNode = 0x3B
    ENGuiEditorGraphic_CategoryCollapsed = 0x3C
    ENGuiEditorGraphic_CategoryExpanded = 0x3D
    ENGuiEditorGraphic_WindowTitleBarDragTarget = 0x3E
    ENGuiEditorGraphic_IconButtonSelected = 0x3F
    ENGuiEditorGraphic_Line = 0x40
    ENGuiEditorGraphic_TreeNodeBackground = 0x41
    ENGuiEditorGraphic_TreeNodeCategoryBackground = 0x42
    ENGuiEditorGraphic_SceneNodeBackground = 0x43
    ENGuiEditorGraphic_PinChildren = 0x44
    ENGuiEditorGraphic_UnpinChildren = 0x45
    ENGuiEditorGraphic_DynamicPanel = 0x46
    ENGuiEditorGraphic_DynamicPanelTitle = 0x47
    ENGuiEditorGraphic_NumTypes = 0x48

class eAuthProvider(IntEnum):
    EAuthProvider_Null = 0x0
    EAuthProvider_PSN = 0x1
    EAuthProvider_Steam = 0x2
    EAuthProvider_Galaxy = 0x3
    EAuthProvider_Xbox = 0x4
    EAuthProvider_WeGame = 0x5
    EAuthProvider_NSO = 0x6
    EAuthProvider_NumTypes = 0x7

class eNGuiEditorComponentSize(IntEnum):
    ENGuiEditorComponentSize_WindowResize = 0x0
    ENGuiEditorComponentSize_WindowButton = 0x1
    ENGuiEditorComponentSize_Indent = 0x2
    ENGuiEditorComponentSize_SeparatorHeight = 0x3
    ENGuiEditorComponentSize_SeparatorWidth = 0x4
    ENGuiEditorComponentSize_TreeNodeExpander = 0x5
    ENGuiEditorComponentSize_CheckBox = 0x6
    ENGuiEditorComponentSize_Adjuster = 0x7
    ENGuiEditorComponentSize_Cursor = 0x8
    ENGuiEditorComponentSize_TextEditSeparator = 0x9
    ENGuiEditorComponentSize_DefaultLineHeight = 0xA
    ENGuiEditorComponentSize_ColourEditHeight = 0xB
    ENGuiEditorComponentSize_ColourEditWidth = 0xC
    ENGuiEditorComponentSize_FileBrowser = 0xD
    ENGuiEditorComponentSize_EditorResize = 0xE
    ENGuiEditorComponentSize_EditorMove = 0xF
    ENGuiEditorComponentSize_IconButton = 0x10
    ENGuiEditorComponentSize_SliderKnob = 0x11
    ENGuiEditorComponentSize_SliderBarWidth = 0x12
    ENGuiEditorComponentSize_SliderBarHeight = 0x13
    ENGuiEditorComponentSize_CategoryHeight = 0x14
    ENGuiEditorComponentSize_WindowTitle = 0x15
    ENGuiEditorComponentSize_ScrollSpeed = 0x16
    ENGuiEditorComponentSize_ComboBox = 0x17
    ENGuiEditorComponentSize_Taskbar = 0x18
    ENGuiEditorComponentSize_IconListItem = 0x19
    ENGuiEditorComponentSize_Toolbar = 0x1A
    ENGuiEditorComponentSize_ToolbarOptions = 0x1B
    ENGuiEditorComponentSize_StartBarWidth = 0x1C
    ENGuiEditorComponentSize_StartBarHeight = 0x1D
    ENGuiEditorComponentSize_StartBarSearchWidth = 0x1E
    ENGuiEditorComponentSize_TreeNodeSpacing = 0x1F
    ENGuiEditorComponentSize_VectorSpacing = 0x20
    ENGuiEditorComponentSize_SliderMinSpacing = 0x21
    ENGuiEditorComponentSize_VectorMinSpacing = 0x22
    ENGuiEditorComponentSize_ColourAlphaMinsize = 0x23
    ENGuiEditorComponentSize_SpacingGap = 0x24
    ENGuiEditorComponentSize_Scroll = 0x25
    ENGuiEditorComponentSize_TextLabelSeparator = 0x26
    ENGuiEditorComponentSize_AlignmentAnchor = 0x27
    ENGuiEditorComponentSize_MinimiseHighlightHeight = 0x28
    ENGuiEditorComponentSize_TableButtonSpacing = 0x29
    ENGuiEditorComponentSize_TableHeaderHeight = 0x2A
    ENGuiEditorComponentSize_TreeNodeHeight = 0x2B
    ENGuiEditorComponentSize_ScrollMargin = 0x2C
    ENGuiEditorComponentSize_ScrollIncrement = 0x2D
    ENGuiEditorComponentSize_EditorPin = 0x2E
    ENGuiEditorComponentSize_DynamicPanelTitle = 0x2F
    ENGuiEditorComponentSize_NumTypes = 0x30

class eShape(IntEnum):
    EShape_Rectangle = 0x0
    EShape_Ellipse = 0x1
    EShape_Line = 0x2
    EShape_LineInverted = 0x3
    EShape_Bezier = 0x4
    EShape_BezierInverted = 0x5
    EShape_BezierWide = 0x6
    EShape_BezierWideInverted = 0x7
    EShape_NumTypes = 0x8

class eGradient(IntEnum):
    EGradient_None = 0x0
    EGradient_Vertical = 0x1
    EGradient_Horizontal = 0x2
    EGradient_HorizontalBounce = 0x3
    EGradient_Radial = 0x4
    EGradient_Box = 0x5
    EGradient_NumTypes = 0x6

class eSucceedWhen(IntEnum):
    ESucceedWhen_AllChildrenSucceed = 0x0
    ESucceedWhen_AnyChildSucceeds = 0x1
    ESucceedWhen_NumTypes = 0x2

class eFailWhen(IntEnum):
    EFailWhen_AnyChildFails = 0x0
    EFailWhen_AllChildrenFail = 0x1
    EFailWhen_NumTypes = 0x2

class eBlackboardCategory(IntEnum):
    EBlackboardCategory_Local = 0x0
    EBlackboardCategory_Archetype = 0x1
    EBlackboardCategory_PlayerControl = 0x2
    EBlackboardCategory_NumTypes = 0x3

class eComparisonType(IntEnum):
    EComparisonType_Equal = 0x0
    EComparisonType_NotEqual = 0x1
    EComparisonType_GreaterThan = 0x2
    EComparisonType_GreaterThanEqual = 0x3
    EComparisonType_LessThan = 0x4
    EComparisonType_LessThanEqual = 0x5
    EComparisonType_NumTypes = 0x6

class eCondition(IntEnum):
    ECondition_Equal = 0x0
    ECondition_NotEqual = 0x1
    ECondition_Greater = 0x2
    ECondition_Less = 0x3
    ECondition_GreaterEqual = 0x4
    ECondition_LessEqual = 0x5
    ECondition_NumTypes = 0x6

class eAnimate(IntEnum):
    EAnimate_None = 0x0
    EAnimate_WipeRightToLeft = 0x1
    EAnimate_SimpleWipe = 0x2
    EAnimate_SimpleWipeDown = 0x3
    EAnimate_CustomWipe = 0x4
    EAnimate_CustomWipeAlpha = 0x5
    EAnimate_NumTypes = 0x6

class eEqualityEnum(IntEnum):
    EEqualityEnum_Equal = 0x0
    EEqualityEnum_Greater = 0x1
    EEqualityEnum_Less = 0x2
    EEqualityEnum_GreaterEqual = 0x3
    EEqualityEnum_LessEqual = 0x4
    EEqualityEnum_NumTypes = 0x5

class eNGuiForcedStyle(IntEnum):
    ENGuiForcedStyle_None = 0x0
    ENGuiForcedStyle_Default = 0x1
    ENGuiForcedStyle_Highlight = 0x2
    ENGuiForcedStyle_Active = 0x3
    ENGuiForcedStyle_NumTypes = 0x4

class eAssertsLevel(IntEnum):
    EAssertsLevel_Disabled = 0x0
    EAssertsLevel_Ignored = 0x1
    EAssertsLevel_Skipped = 0x2
    EAssertsLevel_Enabled = 0x3
    EAssertsLevel_NumTypes = 0x4

class exEnabledChannels(IntEnum):
    FEnabledChannels_None = 0x0
    FEnabledChannels_Default = 0x1
    FEnabledChannels_Note = 0x2
    FEnabledChannels_Error = 0x4
    FEnabledChannels_Warning = 0x8
    FEnabledChannels_Info = 0x10
    FEnabledChannels_Alt = 0x20
    FEnabledChannels_AltWarn = 0x40
    FEnabledChannels_AltError = 0x80
    FEnabledChannels_Mask = 0xFF

class eThumbnailMode(IntEnum):
    EThumbnailMode_None = 0x0
    EThumbnailMode_HUD = 0x1
    EThumbnailMode_GUI = 0x2
    EThumbnailMode_NumTypes = 0x3

class eFocusType(IntEnum):
    EFocusType_ResourceBounds = 0x0
    EFocusType_ResourceBoundingHeight = 0x1
    EFocusType_NodeBoundingBox = 0x2
    EFocusType_DiscoveryView = 0x3
    EFocusType_NumTypes = 0x4

class eWikiTopicType(IntEnum):
    EWikiTopicType_Substances = 0x0
    EWikiTopicType_CustomSubstanceList = 0x1
    EWikiTopicType_Products = 0x2
    EWikiTopicType_CustomProductList = 0x3
    EWikiTopicType_CustomItemList = 0x4
    EWikiTopicType_Technologies = 0x5
    EWikiTopicType_CustomTechnologyList = 0x6
    EWikiTopicType_BuildableTech = 0x7
    EWikiTopicType_Construction = 0x8
    EWikiTopicType_TradeCommodities = 0x9
    EWikiTopicType_Curiosities = 0xA
    EWikiTopicType_Cooking = 0xB
    EWikiTopicType_StoneRunes = 0xC
    EWikiTopicType_Words = 0xD
    EWikiTopicType_RecipesAll = 0xE
    EWikiTopicType_RecipesCooker = 0xF
    EWikiTopicType_RecipesRefiner1 = 0x10
    EWikiTopicType_RecipesRefiner2 = 0x11
    EWikiTopicType_RecipesRefiner3 = 0x12
    EWikiTopicType_Guide = 0x13
    EWikiTopicType_Stories = 0x14
    EWikiTopicType_TreasureWonders = 0x15
    EWikiTopicType_WeirdBasePartWonders = 0x16
    EWikiTopicType_PlanetWonders = 0x17
    EWikiTopicType_CreatureWonders = 0x18
    EWikiTopicType_FloraWonders = 0x19
    EWikiTopicType_MineralWonders = 0x1A
    EWikiTopicType_CustomWonders = 0x1B
    EWikiTopicType_DebugSweep = 0x1C
    EWikiTopicType_NumTypes = 0x1D
    EWikiTopicType_Unspecified = 0xFFFFFFFF

class eBlendOperation(IntEnum):
    EBlendOperation_Blend = 0x0
    EBlendOperation_Add = 0x1
    EBlendOperation_NumTypes = 0x2

class eWonderCategoryComparisonType(IntEnum):
    EWonderCategoryComparisonType_Max = 0x0
    EWonderCategoryComparisonType_Min = 0x1
    EWonderCategoryComparisonType_NumTypes = 0x2

class eBlockPlayerWeapon(IntEnum):
    EBlockPlayerWeapon_Unblocked = 0x0
    EBlockPlayerWeapon_Sheathed = 0x1
    EBlockPlayerWeapon_OutButCannotFire = 0x2
    EBlockPlayerWeapon_NumTypes = 0x3

class eWonderCreatureCategory(IntEnum):
    EWonderCreatureCategory_HerbivoreSizeMax = 0x0
    EWonderCreatureCategory_HerbivoreSizeMin = 0x1
    EWonderCreatureCategory_CarnivoreSizeMax = 0x2
    EWonderCreatureCategory_CarnivoreSizeMin = 0x3
    EWonderCreatureCategory_IntelligenceMax = 0x4
    EWonderCreatureCategory_ViciousnessMax = 0x5
    EWonderCreatureCategory_Hot = 0x6
    EWonderCreatureCategory_Cold = 0x7
    EWonderCreatureCategory_Tox = 0x8
    EWonderCreatureCategory_Rad = 0x9
    EWonderCreatureCategory_Weird = 0xA
    EWonderCreatureCategory_Water = 0xB
    EWonderCreatureCategory_Robot = 0xC
    EWonderCreatureCategory_Flyer = 0xD
    EWonderCreatureCategory_Cave = 0xE
    EWonderCreatureCategory_NumTypes = 0xF

class eSurfaceProperties(IntEnum):
    ESurfaceProperties_None = 0x0
    ESurfaceProperties_Glass = 0x1
    ESurfaceProperties_NumTypes = 0x2

class eAnimType(IntEnum):
    EAnimType_Loop = 0x0
    EAnimType_OneShot = 0x1
    EAnimType_Control = 0x2
    EAnimType_OneShotBlendable = 0x3
    EAnimType_NumTypes = 0x4

class eWonderCustomCategory(IntEnum):
    EWonderCustomCategory_Custom01 = 0x0
    EWonderCustomCategory_Custom02 = 0x1
    EWonderCustomCategory_Custom03 = 0x2
    EWonderCustomCategory_Custom04 = 0x3
    EWonderCustomCategory_Custom05 = 0x4
    EWonderCustomCategory_Custom06 = 0x5
    EWonderCustomCategory_Custom07 = 0x6
    EWonderCustomCategory_Custom08 = 0x7
    EWonderCustomCategory_Custom09 = 0x8
    EWonderCustomCategory_Custom10 = 0x9
    EWonderCustomCategory_Custom11 = 0xA
    EWonderCustomCategory_Custom12 = 0xB
    EWonderCustomCategory_NumTypes = 0xC

class eCreatureSize(IntEnum):
    ECreatureSize_AllSizes = 0x0
    ECreatureSize_SmallOnly = 0x1
    ECreatureSize_LargeOnly = 0x2
    ECreatureSize_NumTypes = 0x3

class eWonderFloraCategory(IntEnum):
    EWonderFloraCategory_GeneralFact0 = 0x0
    EWonderFloraCategory_GeneralFact1 = 0x1
    EWonderFloraCategory_GeneralFact2 = 0x2
    EWonderFloraCategory_GeneralFact3 = 0x3
    EWonderFloraCategory_ColdFact = 0x4
    EWonderFloraCategory_HotFact = 0x5
    EWonderFloraCategory_RadFact = 0x6
    EWonderFloraCategory_ToxFact = 0x7
    EWonderFloraCategory_NumTypes = 0x8

class eWonderMineralCategory(IntEnum):
    EWonderMineralCategory_GeneralFact0 = 0x0
    EWonderMineralCategory_GeneralFact1 = 0x1
    EWonderMineralCategory_GeneralFact2 = 0x2
    EWonderMineralCategory_MetalFact = 0x3
    EWonderMineralCategory_ColdFact = 0x4
    EWonderMineralCategory_HotFact = 0x5
    EWonderMineralCategory_RadFact = 0x6
    EWonderMineralCategory_ToxFact = 0x7
    EWonderMineralCategory_NumTypes = 0x8

class eWonderPlanetCategory(IntEnum):
    EWonderPlanetCategory_TemperatureMax = 0x0
    EWonderPlanetCategory_TemperatureMin = 0x1
    EWonderPlanetCategory_ToxicityMax = 0x2
    EWonderPlanetCategory_RadiationMax = 0x3
    EWonderPlanetCategory_AnomalyMax = 0x4
    EWonderPlanetCategory_RadiusMax = 0x5
    EWonderPlanetCategory_RadiusMin = 0x6
    EWonderPlanetCategory_AltitudeReachedMax = 0x7
    EWonderPlanetCategory_AltitudeReachedMin = 0x8
    EWonderPlanetCategory_PerfectionMax = 0x9
    EWonderPlanetCategory_PerfectionMin = 0xA
    EWonderPlanetCategory_NumTypes = 0xB

class eWonderType(IntEnum):
    EWonderType_Treasure = 0x0
    EWonderType_WeirdBasePart = 0x1
    EWonderType_Planet = 0x2
    EWonderType_Creature = 0x3
    EWonderType_Flora = 0x4
    EWonderType_Mineral = 0x5
    EWonderType_Custom = 0x6
    EWonderType_NumTypes = 0x7

class eWonderTreasureCategory(IntEnum):
    EWonderTreasureCategory_Loot = 0x0
    EWonderTreasureCategory_Document = 0x1
    EWonderTreasureCategory_BioSample = 0x2
    EWonderTreasureCategory_Fossil = 0x3
    EWonderTreasureCategory_Plant = 0x4
    EWonderTreasureCategory_Tool = 0x5
    EWonderTreasureCategory_Farm = 0x6
    EWonderTreasureCategory_SeaLoot = 0x7
    EWonderTreasureCategory_SeaHorror = 0x8
    EWonderTreasureCategory_Salvage = 0x9
    EWonderTreasureCategory_Bones = 0xA
    EWonderTreasureCategory_SpaceHorror = 0xB
    EWonderTreasureCategory_SpaceBones = 0xC
    EWonderTreasureCategory_NumTypes = 0xD

class eWonderWeirdBasePartCategory(IntEnum):
    EWonderWeirdBasePartCategory_EngineOrb = 0x0
    EWonderWeirdBasePartCategory_BeamStone = 0x1
    EWonderWeirdBasePartCategory_BubbleCluster = 0x2
    EWonderWeirdBasePartCategory_MedGeometric = 0x3
    EWonderWeirdBasePartCategory_Shard = 0x4
    EWonderWeirdBasePartCategory_StarJoint = 0x5
    EWonderWeirdBasePartCategory_BoneGarden = 0x6
    EWonderWeirdBasePartCategory_ContourPod = 0x7
    EWonderWeirdBasePartCategory_HydroPod = 0x8
    EWonderWeirdBasePartCategory_ShellWhite = 0x9
    EWonderWeirdBasePartCategory_WeirdCube = 0xA
    EWonderWeirdBasePartCategory_NumTypes = 0xB

class eReplicationRange(IntEnum):
    EReplicationRange_NotSet = 0x0
    EReplicationRange_System = 0x1
    EReplicationRange_SystemLocal = 0x2
    EReplicationRange_Planet = 0x3
    EReplicationRange_PlanetLocal = 0x4
    EReplicationRange_Space = 0x5
    EReplicationRange_SpaceStation = 0x6
    EReplicationRange_Nexus = 0x7
    EReplicationRange_NumTypes = 0x8

class eSpawnType(IntEnum):
    ESpawnType_Basic = 0x0
    ESpawnType_Creature = 0x1
    ESpawnType_NumTypes = 0x2

class eCoordinates(IntEnum):
    ECoordinates_Polar = 0x0
    ECoordinates_Cartesian = 0x1
    ECoordinates_NumTypes = 0x2

class eBlendOp(IntEnum):
    EBlendOp_Blend = 0x0
    EBlendOp_Add = 0x1
    EBlendOp_NumTypes = 0x2

class eVsyncEx(IntEnum):
    EVsyncEx_Off = 0x0
    EVsyncEx_On = 0x1
    EVsyncEx_Adaptive = 0x2
    EVsyncEx_Triple = 0x3
    EVsyncEx_NumTypes = 0x4

class eTextureStreamingVk(IntEnum):
    ETextureStreamingVk_Off = 0x0
    ETextureStreamingVk_On = 0x1
    ETextureStreamingVk_Auto = 0x2
    ETextureStreamingVk_NonDynamic = 0x3
    ETextureStreamingVk_NumTypes = 0x4

class eHDRMode(IntEnum):
    EHDRMode_Off = 0x0
    EHDRMode_HDR400 = 0x1
    EHDRMode_HDR600 = 0x2
    EHDRMode_HDR1000 = 0x3
    EHDRMode_NumTypes = 0x4

class ePrimaryAxis(IntEnum):
    EPrimaryAxis_Z = 0x0
    EPrimaryAxis_ZNeg = 0x1
    EPrimaryAxis_X = 0x2
    EPrimaryAxis_XNeg = 0x3
    EPrimaryAxis_Y = 0x4
    EPrimaryAxis_YNeg = 0x5
    EPrimaryAxis_NumTypes = 0x6

class eActionSetType(IntEnum):
    EActionSetType_None = 0x0
    EActionSetType_FRONTEND = 0x1
    EActionSetType_OnFootControls = 0x2
    EActionSetType_OnFootQuickMenu = 0x3
    EActionSetType_ShipControls = 0x4
    EActionSetType_ShipQuickMenu = 0x5
    EActionSetType_VehicleMode = 0x6
    EActionSetType_GalacticMap = 0x7
    EActionSetType_PhotoModeMenu = 0x8
    EActionSetType_PhotoModeMvCam = 0x9
    EActionSetType_AmbientMode = 0xA
    EActionSetType_DebugMode = 0xB
    EActionSetType_BuildMenuAnalogueRotate = 0xC
    EActionSetType_TextChat = 0xD
    EActionSetType_BuildMenuSelectionMode = 0xE
    EActionSetType_BuildMenuPlacementMode = 0xF
    EActionSetType_NumTypes = 0x10

class eGraphicDetail(IntEnum):
    EGraphicDetail_Low = 0x0
    EGraphicDetail_Medium = 0x1
    EGraphicDetail_High = 0x2
    EGraphicDetail_Ultra = 0x3
    EGraphicDetail_NumTypes = 0x4

class eVolumeTriggerType(IntEnum):
    EVolumeTriggerType_Open = 0x0
    EVolumeTriggerType_GenericInterior = 0x1
    EVolumeTriggerType_GenericGlassInterior = 0x2
    EVolumeTriggerType_Corridor = 0x3
    EVolumeTriggerType_SmallRoom = 0x4
    EVolumeTriggerType_LargeRoom = 0x5
    EVolumeTriggerType_OpenCovered = 0x6
    EVolumeTriggerType_HazardProtection = 0x7
    EVolumeTriggerType_FieldBoundary = 0x8
    EVolumeTriggerType_Custom_Biodome = 0x9
    EVolumeTriggerType_Portal = 0xA
    EVolumeTriggerType_VehicleBoost = 0xB
    EVolumeTriggerType_NexusPlaza = 0xC
    EVolumeTriggerType_NexusCommunityHub = 0xD
    EVolumeTriggerType_NexusHangar = 0xE
    EVolumeTriggerType_RaceObstacle = 0xF
    EVolumeTriggerType_HazardProtectionCold = 0x10
    EVolumeTriggerType_SpaceStorm = 0x11
    EVolumeTriggerType_NumTypes = 0x12

class eBuildingClass(IntEnum):
    EBuildingClass_None = 0x0
    EBuildingClass_TerrainResource = 0x1
    EBuildingClass_Shelter = 0x2
    EBuildingClass_Abandoned = 0x3
    EBuildingClass_Terminal = 0x4
    EBuildingClass_Shop = 0x5
    EBuildingClass_Outpost = 0x6
    EBuildingClass_Waypoint = 0x7
    EBuildingClass_Beacon = 0x8
    EBuildingClass_RadioTower = 0x9
    EBuildingClass_Observatory = 0xA
    EBuildingClass_Depot = 0xB
    EBuildingClass_Factory = 0xC
    EBuildingClass_Harvester = 0xD
    EBuildingClass_Plaque = 0xE
    EBuildingClass_Monolith = 0xF
    EBuildingClass_Portal = 0x10
    EBuildingClass_Ruin = 0x11
    EBuildingClass_Debris = 0x12
    EBuildingClass_DamagedMachine = 0x13
    EBuildingClass_DistressSignal = 0x14
    EBuildingClass_LandingPad = 0x15
    EBuildingClass_Base = 0x16
    EBuildingClass_MissionTower = 0x17
    EBuildingClass_CrashedFreighter = 0x18
    EBuildingClass_GraveInCave = 0x19
    EBuildingClass_StoryGlitch = 0x1A
    EBuildingClass_TreasureRuins = 0x1B
    EBuildingClass_GameStartSpawn = 0x1C
    EBuildingClass_WaterCrashedFreighter = 0x1D
    EBuildingClass_WaterTreasureRuins = 0x1E
    EBuildingClass_WaterAbandoned = 0x1F
    EBuildingClass_WaterDistressSignal = 0x20
    EBuildingClass_NPCDistressSignal = 0x21
    EBuildingClass_NPCDebris = 0x22
    EBuildingClass_LargeBuilding = 0x23
    EBuildingClass_Settlement_Hub = 0x24
    EBuildingClass_Settlement_LandingZone = 0x25
    EBuildingClass_Settlement_Bar = 0x26
    EBuildingClass_Settlement_Tower = 0x27
    EBuildingClass_Settlement_Market = 0x28
    EBuildingClass_Settlement_Small = 0x29
    EBuildingClass_Settlement_SmallIndustrial = 0x2A
    EBuildingClass_Settlement_Medium = 0x2B
    EBuildingClass_Settlement_Large = 0x2C
    EBuildingClass_Settlement_Monument = 0x2D
    EBuildingClass_Settlement_SheriffsOffice = 0x2E
    EBuildingClass_Settlement_Double = 0x2F
    EBuildingClass_Settlement_Farm = 0x30
    EBuildingClass_Settlement_Factory = 0x31
    EBuildingClass_Settlement_Clump = 0x32
    EBuildingClass_DroneHive = 0x33
    EBuildingClass_NumTypes = 0x34

class eDocOptionalEnum(IntEnum):
    EDocOptionalEnum_SomeValue1 = 0x0
    EDocOptionalEnum_SomeValue2 = 0x1
    EDocOptionalEnum_SomeValue3 = 0x2
    EDocOptionalEnum_SomeValue4 = 0x3
    EDocOptionalEnum_NumTypes = 0x4

class eTestEnum(IntEnum):
    ETestEnum_Default = 0x0
    ETestEnum_Option1 = 0x1
    ETestEnum_Option2 = 0x2
    ETestEnum_Option3 = 0x3
    ETestEnum_NumTypes = 0x4

class eTestEnumArray(IntEnum):
    ETestEnumArray_Default = 0x0
    ETestEnumArray_Option1 = 0x1
    ETestEnumArray_Option2 = 0x2
    ETestEnumArray_Option3 = 0x3
    ETestEnumArray_NumTypes = 0x4

class exTestFlags(IntEnum):
    FTestFlags_None = 0x0
    FTestFlags_Flag1 = 0x1
    FTestFlags_Flag2 = 0x2
    FTestFlags_Flag3 = 0x4
    FTestFlags_Mask = 0x7

class eDynamicResScalingAggressiveness(IntEnum):
    EDynamicResScalingAggressiveness_Moderate = 0x0
    EDynamicResScalingAggressiveness_Balanced = 0x1
    EDynamicResScalingAggressiveness_Aggressive = 0x2
    EDynamicResScalingAggressiveness_NumTypes = 0x3

class eData(IntEnum):
    EData_Left = 0x0
    EData_Right = 0x1
    EData_NumTypes = 0x2

class eEngineSetting(IntEnum):
    EEngineSetting_FullScreen = 0x0
    EEngineSetting_Borderless = 0x1
    EEngineSetting_ResolutionWidth = 0x2
    EEngineSetting_ResolutionHeight = 0x3
    EEngineSetting_ResolutionScale = 0x4
    EEngineSetting_Monitor = 0x5
    EEngineSetting_FoVOnFoot = 0x6
    EEngineSetting_FoVInShip = 0x7
    EEngineSetting_VSync = 0x8
    EEngineSetting_TextureQuality = 0x9
    EEngineSetting_AnimationQuality = 0xA
    EEngineSetting_ShadowQuality = 0xB
    EEngineSetting_ReflectionsMultiplier = 0xC
    EEngineSetting_ReflectionProbesMultiplier = 0xD
    EEngineSetting_ReflectionProbes = 0xE
    EEngineSetting_ScreenSpaceReflections = 0xF
    EEngineSetting_ReflectionsQuality = 0x10
    EEngineSetting_PostProcessingEffects = 0x11
    EEngineSetting_VolumetricsQuality = 0x12
    EEngineSetting_TerrainTessellation = 0x13
    EEngineSetting_PlanetQuality = 0x14
    EEngineSetting_BaseQuality = 0x15
    EEngineSetting_UIQuality = 0x16
    EEngineSetting_DLSSQuality = 0x17
    EEngineSetting_FFXSRQuality = 0x18
    EEngineSetting_FFXSR2Quality = 0x19
    EEngineSetting_XESSQuality = 0x1A
    EEngineSetting_DynamicResScaling = 0x1B
    EEngineSetting_EnableTessellation = 0x1C
    EEngineSetting_AntiAliasing = 0x1D
    EEngineSetting_AnisotropyLevel = 0x1E
    EEngineSetting_Brightness = 0x1F
    EEngineSetting_VignetteAndScanlines = 0x20
    EEngineSetting_AvailableMonitors = 0x21
    EEngineSetting_MaxFrameRate = 0x22
    EEngineSetting_NumLowThreads = 0x23
    EEngineSetting_NumHighThreads = 0x24
    EEngineSetting_NumGraphicsThreads = 0x25
    EEngineSetting_TextureStreaming = 0x26
    EEngineSetting_TexturePageSizeKb = 0x27
    EEngineSetting_MotionBlurStrength = 0x28
    EEngineSetting_ShowRequirementsWarnings = 0x29
    EEngineSetting_AmbientOcclusion = 0x2A
    EEngineSetting_MaxTextureMemoryMb = 0x2B
    EEngineSetting_FixedTextureMemory = 0x2C
    EEngineSetting_UseArbSparseTexture = 0x2D
    EEngineSetting_UseTerrainTextureCache = 0x2E
    EEngineSetting_AdapterIndex = 0x2F
    EEngineSetting_UseHDR = 0x30
    EEngineSetting_MinGPUMode = 0x31
    EEngineSetting_NumTypes = 0x32

class eAlienRace(IntEnum):
    EAlienRace_Traders = 0x0
    EAlienRace_Warriors = 0x1
    EAlienRace_Explorers = 0x2
    EAlienRace_Robots = 0x3
    EAlienRace_Atlas = 0x4
    EAlienRace_Diplomats = 0x5
    EAlienRace_Exotics = 0x6
    EAlienRace_None = 0x7
    EAlienRace_NumTypes = 0x8
    EAlienRace_Unspecified = 0xFFFFFFFF

class eUIQuality(IntEnum):
    EUIQuality_Normal = 0x0
    EUIQuality_4K = 0x1
    EUIQuality_NumTypes = 0x2

class eDLSSQuality(IntEnum):
    EDLSSQuality_MaxPerformance = 0x0
    EDLSSQuality_Balanced = 0x1
    EDLSSQuality_MaxQuality = 0x2
    EDLSSQuality_UltraPerformance = 0x3
    EDLSSQuality_UltraQuality = 0x4
    EDLSSQuality_NumTypes = 0x5

class eFFXSRQuality(IntEnum):
    EFFXSRQuality_Off = 0x0
    EFFXSRQuality_UltraQuality = 0x1
    EFFXSRQuality_Quality = 0x2
    EFFXSRQuality_Balanced = 0x3
    EFFXSRQuality_Performance = 0x4
    EFFXSRQuality_NumTypes = 0x5

class eFFXSR2Quality(IntEnum):
    EFFXSR2Quality_UltraPerformance = 0x0
    EFFXSR2Quality_Performance = 0x1
    EFFXSR2Quality_Balanced = 0x2
    EFFXSR2Quality_Quality = 0x3
    EFFXSR2Quality_NumTypes = 0x4

class eXESSQuality(IntEnum):
    EXESSQuality_UltraPerformance = 0x0
    EXESSQuality_Performance = 0x1
    EXESSQuality_Balanced = 0x2
    EXESSQuality_MaxQuality = 0x3
    EXESSQuality_UltraQuality = 0x4
    EXESSQuality_NumTypes = 0x5

class eAmbientOcclusion(IntEnum):
    EAmbientOcclusion_Off = 0x0
    EAmbientOcclusion_GTAO_Low = 0x1
    EAmbientOcclusion_GTAO_Medium = 0x2
    EAmbientOcclusion_GTAO_High = 0x3
    EAmbientOcclusion_GTAO_Ultra = 0x4
    EAmbientOcclusion_HBAO_Low = 0x5
    EAmbientOcclusion_HBAO_High = 0x6
    EAmbientOcclusion_NumTypes = 0x7

class eAnisotropyLevel(IntEnum):
    EAnisotropyLevel_1 = 0x0
    EAnisotropyLevel_2 = 0x1
    EAnisotropyLevel_4 = 0x2
    EAnisotropyLevel_8 = 0x3
    EAnisotropyLevel_16 = 0x4
    EAnisotropyLevel_NumTypes = 0x5

class eAntiAliasing(IntEnum):
    EAntiAliasing_None = 0x0
    EAntiAliasing_TAA_LOW = 0x1
    EAntiAliasing_TAA = 0x2
    EAntiAliasing_FXAA = 0x3
    EAntiAliasing_FFXSR2 = 0x4
    EAntiAliasing_DLSS = 0x5
    EAntiAliasing_DLAA = 0x6
    EAntiAliasing_XESS = 0x7
    EAntiAliasing_NumTypes = 0x8

class eInteractionType(IntEnum):
    EInteractionType_None = 0x0
    EInteractionType_Shop = 0x1
    EInteractionType_NPC = 0x2
    EInteractionType_NPC_Secondary = 0x3
    EInteractionType_NPC_Anomaly = 0x4
    EInteractionType_NPC_Anomaly_Secondary = 0x5
    EInteractionType_Ship = 0x6
    EInteractionType_Outpost = 0x7
    EInteractionType_SpaceStation = 0x8
    EInteractionType_RadioTower = 0x9
    EInteractionType_Monolith = 0xA
    EInteractionType_Factory = 0xB
    EInteractionType_AbandonedShip = 0xC
    EInteractionType_Harvester = 0xD
    EInteractionType_Observatory = 0xE
    EInteractionType_TradingPost = 0xF
    EInteractionType_DistressBeacon = 0x10
    EInteractionType_Portal = 0x11
    EInteractionType_Plaque = 0x12
    EInteractionType_AtlasStation = 0x13
    EInteractionType_AbandonedBuildings = 0x14
    EInteractionType_WeaponTerminal = 0x15
    EInteractionType_SuitTerminal = 0x16
    EInteractionType_SignalScanner = 0x17
    EInteractionType_Teleporter_Base = 0x18
    EInteractionType_Teleporter_Station = 0x19
    EInteractionType_ClaimBase = 0x1A
    EInteractionType_NPC_Freighter_Captain = 0x1B
    EInteractionType_NPC_HIRE_Weapons = 0x1C
    EInteractionType_NPC_HIRE_Weapons_Wait = 0x1D
    EInteractionType_NPC_HIRE_Farmer = 0x1E
    EInteractionType_NPC_HIRE_Farmer_Wait = 0x1F
    EInteractionType_NPC_HIRE_Builder = 0x20
    EInteractionType_NPC_HIRE_Builder_Wait = 0x21
    EInteractionType_NPC_HIRE_Vehicles = 0x22
    EInteractionType_NPC_HIRE_Vehicles_Wait = 0x23
    EInteractionType_MessageBeacon = 0x24
    EInteractionType_NPC_HIRE_Scientist = 0x25
    EInteractionType_NPC_HIRE_Scientist_Wait = 0x26
    EInteractionType_NPC_Recruit = 0x27
    EInteractionType_NPC_Freighter_Captain_Secondary = 0x28
    EInteractionType_NPC_Recruit_Secondary = 0x29
    EInteractionType_Vehicle = 0x2A
    EInteractionType_MessageModule = 0x2B
    EInteractionType_TechShop = 0x2C
    EInteractionType_VehicleRaceStart = 0x2D
    EInteractionType_BuildingShop = 0x2E
    EInteractionType_MissionGiver = 0x2F
    EInteractionType_HoloHub = 0x30
    EInteractionType_HoloExplorer = 0x31
    EInteractionType_HoloSceptic = 0x32
    EInteractionType_HoloNoone = 0x33
    EInteractionType_PortalRuneEntry = 0x34
    EInteractionType_PortalActivate = 0x35
    EInteractionType_CrashedFreighter = 0x36
    EInteractionType_GraveInCave = 0x37
    EInteractionType_GlitchyStoryBox = 0x38
    EInteractionType_NetworkPlayer = 0x39
    EInteractionType_NetworkMonument = 0x3A
    EInteractionType_AnomalyComputer = 0x3B
    EInteractionType_AtlasPlinth = 0x3C
    EInteractionType_Epilogue = 0x3D
    EInteractionType_GuildEnvoy = 0x3E
    EInteractionType_ManageFleet = 0x3F
    EInteractionType_ManageExpeditions = 0x40
    EInteractionType_Frigate = 0x41
    EInteractionType_CustomiseCharacter = 0x42
    EInteractionType_CustomiseShip = 0x43
    EInteractionType_CustomiseWeapon = 0x44
    EInteractionType_CustomiseVehicle = 0x45
    EInteractionType_ClaimBaseAnywhere = 0x46
    EInteractionType_FleetNavigator = 0x47
    EInteractionType_FleetCommandPost = 0x48
    EInteractionType_StoryUtility = 0x49
    EInteractionType_MPMissionGiver = 0x4A
    EInteractionType_SpecialsShop = 0x4B
    EInteractionType_WaterRuin = 0x4C
    EInteractionType_LocationScanner = 0x4D
    EInteractionType_ByteBeat = 0x4E
    EInteractionType_NPC_CrashSite = 0x4F
    EInteractionType_NPC_Scavenger = 0x50
    EInteractionType_BaseGridPart = 0x51
    EInteractionType_NPC_Freighter_Crew = 0x52
    EInteractionType_NPC_Freighter_Crew_Owned = 0x53
    EInteractionType_AbandonedShip_With_NPC = 0x54
    EInteractionType_ShipPilot = 0x55
    EInteractionType_NexusMilestones = 0x56
    EInteractionType_NexusDailyMission = 0x57
    EInteractionType_CreatureFeeder = 0x58
    EInteractionType_ExoticExtra1 = 0x59
    EInteractionType_ExoticExtra2 = 0x5A
    EInteractionType_ExoticExtra3 = 0x5B
    EInteractionType_ExoticExtra4 = 0x5C
    EInteractionType_ExoticExtra5 = 0x5D
    EInteractionType_ExoticExtra6 = 0x5E
    EInteractionType_MapShop = 0x5F
    EInteractionType_NPC_Closure = 0x60
    EInteractionType_StorageContainer = 0x61
    EInteractionType_Teleporter_Nexus = 0x62
    EInteractionType_ShipSalvage = 0x63
    EInteractionType_ByteBeatSwitch = 0x64
    EInteractionType_AbandonedFreighterIntro = 0x65
    EInteractionType_AbandonedFreighterEnd = 0x66
    EInteractionType_AbandonedFreighterProcText = 0x67
    EInteractionType_AbandonedFreighterCaptLog = 0x68
    EInteractionType_AbandonedFreighterCrewLog = 0x69
    EInteractionType_AbandonedFreighterShop = 0x6A
    EInteractionType_CustomiseFreighter = 0x6B
    EInteractionType_LibraryVault = 0x6C
    EInteractionType_LibraryMainTerminal = 0x6D
    EInteractionType_LibraryMap = 0x6E
    EInteractionType_WeaponUpgrade = 0x6F
    EInteractionType_Pet = 0x70
    EInteractionType_Creature = 0x71
    EInteractionType_FreighterGalacticMap = 0x72
    EInteractionType_RecipeStation = 0x73
    EInteractionType_StationCore = 0x74
    EInteractionType_NPC_Settlement_SpecialWorker = 0x75
    EInteractionType_NPC_Settlement_Secondary = 0x76
    EInteractionType_SettlementHub = 0x77
    EInteractionType_SettlementBuildingSite = 0x78
    EInteractionType_SettlementAdminTerminal = 0x79
    EInteractionType_FriendlyDrone = 0x7A
    EInteractionType_DroneHive = 0x7B
    EInteractionType_RocketLocker = 0x7C
    EInteractionType_FrigateCaptain = 0x7D
    EInteractionType_PirateShop = 0x7E
    EInteractionType_NPC_PirateSecondary = 0x7F
    EInteractionType_NPC_FreighterBase_SquadronPilot = 0x80
    EInteractionType_NPC_FreighterBase_FrigateCaptain = 0x81
    EInteractionType_NPC_FreighterBase_Worker = 0x82
    EInteractionType_NumTypes = 0x83

class eBiome(IntEnum):
    EBiome_Lush = 0x0
    EBiome_Toxic = 0x1
    EBiome_Scorched = 0x2
    EBiome_Radioactive = 0x3
    EBiome_Frozen = 0x4
    EBiome_Barren = 0x5
    EBiome_Dead = 0x6
    EBiome_Weird = 0x7
    EBiome_Red = 0x8
    EBiome_Green = 0x9
    EBiome_Blue = 0xA
    EBiome_Test = 0xB
    EBiome_Swamp = 0xC
    EBiome_Lava = 0xD
    EBiome_Waterworld = 0xE
    EBiome_All = 0xF
    EBiome_NumTypes = 0x10

class eAkEvent(IntEnum):
    EAkEvent_INVALID_EVENT = 0x0
    EAkEvent_NumTypes = 0x9AB
    EAkEvent_MUS_STORYMODE_MUSICCUE_11_LP_STOP = 0x22CA1
    EAkEvent_NPC_FOLEY_CLOTHING_1H_UNSHEATH_IPAD = 0xEEA97
    EAkEvent_PLAY_WT_DRUMS_INDEX_06_EV_DRUMLOOPS_138BPM_9 = 0x105917
    EAkEvent_STOP_WT_DRUMS_INDEX_09_RIPLEY_DRUMLOOPS_120BPM_1 = 0x10F5BF
    EAkEvent_FORFHAPPY01 = 0x40C85F
    EAkEvent_BLD_SCIENTIFIC_FACTORY_ROOF = 0x47BC05
    EAkEvent_PLAY_WT_MELODY_INDEX_04_KOAECAX_SLOWSYNTH_2 = 0x5B6A46
    EAkEvent_MECH_SENTINEL_STEP_RUN = 0x85D18C
    EAkEvent_PL_HAZARD_RECOVERY = 0x9A6776
    EAkEvent_BINOCULARS_ZOOM03 = 0xBCD758
    EAkEvent_BINOCULARS_ZOOM02 = 0xBCD759
    EAkEvent_BINOCULARS_ZOOM01 = 0xBCD75A
    EAkEvent_POWERMODULE_NOTE_G_STOP = 0xD42B64
    EAkEvent_MAINTENANCE_BROKEN = 0xEEE46E
    EAkEvent_PLAY_WT_DRUMS_INDEX_28_SCRAPER_WANTEDBEATS_8 = 0xF5818A
    EAkEvent_NOTIFY_AREANOTCLEAR = 0x129A400
    EAkEvent_MECH_JETPACK_STARTUP = 0x16C249B
    EAkEvent_UI_MAP_PLANET_UNSELECT = 0x16C9D0D
    EAkEvent_FIBREGLASS_DOOR1_SETTLEMENT_CLOSE = 0x1A13F00
    EAkEvent_SHORELINE = 0x1A1A962
    EAkEvent_UI_MAP_TEXT = 0x1ADD72A
    EAkEvent_PLAY_WT_MELODY_INDEX_23_ECHO_SPACEHIGHINTEREST_1 = 0x1C772C4
    EAkEvent_ATLAS_VANISH = 0x1DBBF4C
    EAkEvent_FRIGATE_LIVING_TALK = 0x1E4E7B6
    EAkEvent_MUS_FIRSTTIMEWARP = 0x21015BC
    EAkEvent_PLAY_SP_CHORDS_INDEX_17_APORIA_SPACECHORDS_16 = 0x22601B2
    EAkEvent_NOTIFY_LANDING = 0x244555A
    EAkEvent_PLAY_PL_FX_01_INDEX_14_EOTWSALT_CRACKLE_2 = 0x2519853
    EAkEvent_NOTIFY_MISSILEREADY = 0x2704818
    EAkEvent_MSTRUCTURE = 0x2E46515
    EAkEvent_PLAY_WT_GUITAR_INDEX_02_ASIM_JOEGTRLOUD_BMIN_18 = 0x315EC7A
    EAkEvent_PLAY_SP_SYNTHPADS_INDEX_10_TETSUO_SPACEPADS_17 = 0x32F3C5E
    EAkEvent_EXPL_FIREWORK_LAUNCH_SLIME = 0x37F4611
    EAkEvent_UI_PHOTOMODE_IN = 0x3C4C263
    EAkEvent_CLAMSHELL_OPEN = 0x3F0E2BF
    EAkEvent_PLAY_SP_ARP_HI_INDEX_13_ARPBEYOND_A9 = 0x3F9FF61
    EAkEvent_PLAY_SP_CHORDS_INDEX_03_SM_CLUSTERKEYS_DMIN_2 = 0x416181A
    EAkEvent_HUD_JUMPING = 0x4238C05
    EAkEvent_JETPACK_DEPLETED = 0x4392405
    EAkEvent_CATATTACK1 = 0x49AFA08
    EAkEvent_CATATTACK3 = 0x49AFA0A
    EAkEvent_CATATTACK2 = 0x49AFA0B
    EAkEvent_BASE_AQUARIUM = 0x4A30FB0
    EAkEvent_BLD_POD_DOOR_OPEN = 0x4C4F68F
    EAkEvent_PET_UI_CALLPET = 0x507485B
    EAkEvent_NOTIFY_TOXICPRO_OFFLINE = 0x52C65D7
    EAkEvent_NPC_VOCAL_1H_NEUTRAL_STAND_03 = 0x5519FD7
    EAkEvent_UI_FRIGATE_LIVING_FEED = 0x5679A6D
    EAkEvent_UI_SENTINELFORCES_DEACTIVATED = 0x56D6F15
    EAkEvent_FIEND_DIE = 0x57FEA6E
    EAkEvent_FREIGHTER_STELLAREXTRACTOR = 0x5806FFF
    EAkEvent_CORETERMINAL_COLLECTDATA = 0x5A8D1FF
    EAkEvent_VO_TUT_STATIONCOORDSRECEIVED = 0x5B74DAC
    EAkEvent_PL_DAMAGE_WARNING_SHIELD_LOSS = 0x5BE3084
    EAkEvent_PL_FOLEY_CLOTHING_MULTITOOL_RELOAD = 0x5C1C938
    EAkEvent_PL_FOLEY_CLOTHING_DEATH_SWIM = 0x5FC94D6
    EAkEvent_SPIDERATTACK2 = 0x6017C74
    EAkEvent_SPIDERATTACK3 = 0x6017C75
    EAkEvent_ANTELOPESHAKE = 0x60DF35B
    EAkEvent_UI_SHOP_ITEMS_MENU = 0x6192287
    EAkEvent_UI_INTERACT_TIMER = 0x62988FE
    EAkEvent_DEATHMARKER_TRANSFORM = 0x651ED76
    EAkEvent_NOTIFY_RADPRO_OFFLINE = 0x66C488F
    EAkEvent_HEXAGON = 0x66DC42F
    EAkEvent_PLAY_PL_HIGHSYNTH_INDEX_25_SCRAPER_PLANETGTRMELS_4 = 0x67648F5
    EAkEvent_DRONE_EMOTE_ONESHOT = 0x6890081
    EAkEvent_PLAY_PL_FX_01_INDEX_15_HAUNTED_ARCTIC_EMIN_3 = 0x68DFC6E
    EAkEvent_SENTINEL_MECH_WPN_FIRE_STOP = 0x7112EA1
    EAkEvent_PLAY_SP_SYNTHHIGH_INDEX_28_SCRAPER_SPACEARPS_5 = 0x725D9C8
    EAkEvent_PLAY_SP_ARP_HI_INDEX_05_REDPARA_SPACEARP_AMIN_7 = 0x73AC58C
    EAkEvent_NEXUS_PLANTGROWBOX = 0x7490B24
    EAkEvent_TORCH_OFF = 0x74CB5DB
    EAkEvent_PORTAL_TERMINAL_OPEN = 0x7606BD5
    EAkEvent_STOP_WT_DRUMS_INDEX_21_OCT_WANTEDBEATS_1 = 0x772C6BC
    EAkEvent_PLAY_WT_GUITAR_INDEX_27_OVERSEER_WANTEDGTR_7 = 0x7730B6E
    EAkEvent_BLD_AMB_DEPOTS = 0x793F92E
    EAkEvent_BLD_WARRIOR_ROOF_OBS = 0x79E2A24
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_LOOKRIGHT01 = 0x7A2CAEC
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_LOOKRIGHT03 = 0x7A2CAEE
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_LOOKRIGHT02 = 0x7A2CAEF
    EAkEvent_CRT_GRUNT_WINGFLAPRUN = 0x7B6E923
    EAkEvent_NPC_VOCAL_IDLE_BASIC01 = 0x7EB4B73
    EAkEvent_PLAY_WT_NOISE_INDEX_01_MONOLITH_WANTED_NOISEONESHOTS_2 = 0x7EBF2A9
    EAkEvent_ROVER_HORN_STOP = 0x80EDB9C
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_FALLKNEESLP = 0x846370D
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_04_BLUEPRINT_BOWEDGTR_EMIN_31 = 0x8831B83
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_LOOKARND = 0x8BC36BE
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_23_ECHO_SPACEHIGHINTERESTREV_1 = 0x8C39509
    EAkEvent_SHIP_PIRATE_SAILS_SQUARE_UNFOLD = 0x8EF70A7
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_25_DASHKA_JOEGTR_5 = 0x9165FEF
    EAkEvent_PLAY_SP_MISC_HI_INDEX_20_TUNG_SPACEHIGHINTEREST_13 = 0x93AE2D3
    EAkEvent_STOP_MAP_DRONENOISE_02_INDEX_01_MAP_SPACE_NOISE_DRONE_RR = 0x983118E
    EAkEvent_TERMINAL_ABANDONED_GROWTH_OPEN = 0x9845265
    EAkEvent_SUIT_INIT_END = 0x9A042E1
    EAkEvent_UI_VR_WRISTMENU_TOUCH = 0x9C3C5AF
    EAkEvent_PLAY_SP_GUITAR_INDEX_21_OCT_SPACEGTR_31 = 0x9DA8B57
    EAkEvent_STOP_WT_DRUMS_INDEX_01_MONOLITH_WANTED_DRUMLOOPS_20 = 0x9DAB5CC
    EAkEvent_ATLAS_BALLOFLIGHT = 0x9E3D0CB
    EAkEvent_PLAQUE_END = 0xA28CC67
    EAkEvent_PL_FOLEY_CLOTHING_LADDER_CYCLE_DOWN = 0xA5310BB
    EAkEvent_UI_KEYBINDING = 0xA541E36
    EAkEvent_PLAY_SP_DRONE_01_INDEX_06_EV_SPACEDRONEONE_ENEUTRAL_2 = 0xA8735D7
    EAkEvent_AMB_ABANDONEDBUILDING_STOP = 0xA891AC5
    EAkEvent_SPGEK_CHATTER = 0xA9537B9
    EAkEvent_PILGRIM_BOOST_STOP = 0xAB53D70
    EAkEvent_DRONE_RECHARGE = 0xACC011B
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_CNSL_OUT = 0xAD6D0F2
    EAkEvent_PL_SHIP_BURN = 0xB0660BE
    EAkEvent_VEHICLE_RACE_START = 0xB3AF6D8
    EAkEvent_STOP_WT_DRUMS_INDEX_06_EV_DRUMLOOPS_138BPM_9 = 0xB42E15D
    EAkEvent_PL_FOLEY_EMOTE_JOY = 0xB65D913
    EAkEvent_GRUNTPOUNCE = 0xB66812B
    EAkEvent_PLAY_WT_GUITARS_02_INDEX_17_APORIA_WANTEDGTRS_1 = 0xBD78E4D
    EAkEvent_UI_PHOTOMODE_TAKEPHOTO = 0xBF99F39
    EAkEvent_DRAGONFLY_HORN_START = 0xC06C8F4
    EAkEvent_PL_VEHICLE_REMOTE_BOOST = 0xC188029
    EAkEvent_UI_MAP_PLANET_SELECT = 0xC49E774
    EAkEvent_EXPL_FIREWORK = 0xC78B45E
    EAkEvent_WALKER_LEG_HIT_LEFT_IN_01 = 0xC8452F0
    EAkEvent_WALKER_LEG_HIT_LEFT_IN_02 = 0xC8452F3
    EAkEvent_SHIP_SQUID_NOSE_LANDING = 0xCC0E542
    EAkEvent_PLAY_SP_BASS_INDEX_09_RIPLEY_SPACEBASS_EMIN_1 = 0xCD3A288
    EAkEvent_BLD_TONEINT_HANGER_START_01 = 0xCF108C8
    EAkEvent_MECH_JETPACK_TRIGGER = 0xCF7D714
    EAkEvent_PLANT_GATHER_FROZENTUBERS = 0xCFD83A4
    EAkEvent_RODENTATTACK1 = 0xD181620
    EAkEvent_RODENTATTACK3 = 0xD181622
    EAkEvent_RODENTATTACK2 = 0xD181623
    EAkEvent_SHIP_ALIEN_TAKEOFF = 0xD387BE6
    EAkEvent_BUTTERFLY = 0xD410A32
    EAkEvent_PLAY_WT_DRUMS_INDEX_07_EOTWS_WANTEDBEATS_1 = 0xD4BB78E
    EAkEvent_ROBOT_METAL_IMPACTS = 0xD7582BF
    EAkEvent_DRAGONFLY_BOOST_STOP = 0xD7CF88E
    EAkEvent_PIPE_SMOKE = 0xD9CB4AF
    EAkEvent_PLAY_SP_DRONE_01_INDEX_12_ATWOOD_DRONESUB_1 = 0xDCAB9F9
    EAkEvent_MUS_STORYMODE_MUSICCUE_10_LP = 0xDD4A2FD
    EAkEvent_BLD_TRADER_FACTORY_ROOF = 0xDD99760
    EAkEvent_BASE_BIOFUELDISCHARGED = 0xDF54E5D
    EAkEvent_BLD_TRADER_DOOR_CLOSE = 0xE74FE36
    EAkEvent_POWER_SWITCH_BUTTON_OFF = 0xE959674
    EAkEvent_PLAY_WT_DRONE_INDEX_15_MAC_MAINPADSFULL_CSHARP_MIN_2 = 0xEC4E0A5
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_LOOKDISTANCEINTO = 0xEC9B5CE
    EAkEvent_UI_GAMEMODE_ERASE_COMPLETE = 0xEDE52A4
    EAkEvent_CREATURE_VOCALS_SQUAWK_FLEE = 0xEF169B9
    EAkEvent_SETTLEMENT_GLOWGLOBE = 0xF2985AF
    EAkEvent_STARSHIP_VIEW_SWITCH_EXTERIOR = 0xF31B791
    EAkEvent_PLAY_SP_CHORDS_INDEX_18_NMS2017_SPACECHORDSYNTH_10 = 0xF4FCC08
    EAkEvent_SUIT_UPGRADE_TERMINAL_INTERACT = 0xF69FA17
    EAkEvent_DRONE_MEDIC_ATTACK = 0xF6B0A69
    EAkEvent_PLAY_PL_GUITAR_02_INDEX_04_BLUEPRINT_GTRTEXTURE_EMIN_5 = 0xF7BDE53
    EAkEvent_PL_SHIP_TRANSITION_TO_SPACE_END = 0xF8BCF39
    EAkEvent_WPN_SHIP_INFRAKNIFE_FIRE = 0xFA63876
    EAkEvent_UI_INCOMING_STARSHIP_WARNING = 0xFB031F6
    EAkEvent_MUS_STORYMODE_MUSICCUE_12_LP = 0x102487FB
    EAkEvent_PL_SHIP_TRANSITION_TO_PLANET_END = 0x102A0FD3
    EAkEvent_UI_SAVE = 0x10420AD9
    EAkEvent_MINIWORM_BURSTOUT = 0x105FB33A
    EAkEvent_HAZ_CRIT_SHIELD_FALLING = 0x10651173
    EAkEvent_PLAY_PL_DRONE_02_INDEX_08_TOMORROW_FM_CHOPPED_1 = 0x1087A406
    EAkEvent_GRABBYPLANT_ATTACK_LP = 0x10A22325
    EAkEvent_PL_FOLEY_CLOTHING_1H_OVERHEAT = 0x10B47D33
    EAkEvent_PLAYER_WATER_TRANSITION_INTO_WATER = 0x10DA6251
    EAkEvent_ATLASSTARTSCENEORB_ACTIVATEIDLE = 0x10DCFBD1
    EAkEvent_FREIGHTER_ALARM = 0x10F2436D
    EAkEvent_MUS_STORYMODE_MUSICCUE_22_LP = 0x10FAF564
    EAkEvent_SPORE = 0x11268644
    EAkEvent_UI_VR_HANDMENU_OPEN = 0x112C18F2
    EAkEvent_PLAY_PL_SYNTH_INDEX_26_MELB_PLANETBURNT_4 = 0x11362C7D
    EAkEvent_UI_CLICK_GENERIC = 0x114B52C4
    EAkEvent_PLAY_PL_MELODY_INDEX_28_SCRAPER_PLANETWEIRDARP_2 = 0x114B8E0E
    EAkEvent_PLAY_SP_DRONE_03_INDEX_22_VOSTOK_LOW_2 = 0x11579679
    EAkEvent_SPACESTATION_ATLAS_DOOR_OPEN = 0x115E98AD
    EAkEvent_NOTIFY_MISSILEINCOMING = 0x117B54CF
    EAkEvent_SENTINEL_HIVE_NODE = 0x118A84ED
    EAkEvent_WOOSH_DROID_SMALL_01_FRONT = 0x11BC8ADA
    EAkEvent_PLAY_PL_HIGHSYNTH_INDEX_24_DOMO_PLANETHIGHINTEREST_2 = 0x11BE54A5
    EAkEvent_SPACE_STATION_EXTERIOR = 0x11D75F19
    EAkEvent_MUS_MONOLITH_STOP = 0x120E6A40
    EAkEvent_UI_CREDITS_INCREASE = 0x12183687
    EAkEvent_WALKER_MOVE = 0x121B6AD1
    EAkEvent_UI_FRONTEND_TEXTOVERLAY_SUBSTANCE = 0x1240C80B
    EAkEvent_PLAQUE_START = 0x12497354
    EAkEvent_PLAY_MAP_CENTRECHORD_02_INDEX_01_MAP_CENTRECHORD_02 = 0x124A4FFA
    EAkEvent_INTERACTION_ENGINEREACTOR = 0x1255F34E
    EAkEvent_WPN_PL_SHOTGUN = 0x128E00C2
    EAkEvent_PLAY_BASE_POWER_CONNECT = 0x12E999E4
    EAkEvent_PL_SURVEYING_SWITCHTO = 0x12FAB8BA
    EAkEvent_PLAY_SP_ARP_HI_INDEX_21_OCT_ARPS_10 = 0x13035ED0
    EAkEvent_PLAY_PL_FX_01_INDEX_27_OVERSEER_PLANETTHUMPS_9 = 0x1311F2DC
    EAkEvent_PLAY_PL_DRONE_01_INDEX_16_BOREALIS_GTRDELAY_AMIN_7 = 0x133D7B99
    EAkEvent_SENTINEL_HIVE_TERMINAL = 0x1352B0A9
    EAkEvent_MOTION_PS5_01 = 0x139A94EC
    EAkEvent_WPN_NOAMMO = 0x139F02B6
    EAkEvent_PLAY_WT_DRUMS_INDEX_27_OVERSEER_WANTEDDRUMS_6 = 0x13D2BB3C
    EAkEvent_UI_MAP_WAYPOINT_DELETED = 0x13DF9858
    EAkEvent_WINGFLAP = 0x13E172E1
    EAkEvent_COOKER_COOKING = 0x1409D575
    EAkEvent_NPC_CONSTRUCTION_TERMINAL = 0x1433F1D1
    EAkEvent_AMB_BUILDING_START = 0x143EDD47
    EAkEvent_CRATE_WEAPON = 0x144B7049
    EAkEvent_UI_FRONTEND_TEXTOVERLAY_TECH = 0x14743037
    EAkEvent_EXPL_DEPOT = 0x14D13D39
    EAkEvent_PLANT_GATHER_HEPTAWHEAT = 0x153FCA3A
    EAkEvent_WARN_HOSTILE_SCAN = 0x15472E9E
    EAkEvent_FOURTHRACE_PROC_IDLE00 = 0x1593DBFE
    EAkEvent_FOURTHRACE_ARTEMIS_GREET04 = 0x15AA44F0
    EAkEvent_FOURTHRACE_ARTEMIS_GREET00 = 0x15AA44F4
    EAkEvent_FOURTHRACE_ARTEMIS_GREET01 = 0x15AA44F5
    EAkEvent_FOURTHRACE_ARTEMIS_GREET02 = 0x15AA44F6
    EAkEvent_FOURTHRACE_ARTEMIS_GREET03 = 0x15AA44F7
    EAkEvent_E3_ZOOMOUT = 0x15C6E177
    EAkEvent_UI_PHOTOMODE_VALUEDOWN = 0x15D19C71
    EAkEvent_UI_MAP_MENUUP = 0x15E6E0CD
    EAkEvent_ANTELOPETWOPERFORM = 0x1627CCF6
    EAkEvent_PLAY_WT_MELODY_INDEX_06_EV_PIANOWANTED_AMIN_12 = 0x162A4ED3
    EAkEvent_PLAY_SP_ARP_HI_INDEX_15_MACREADY_4 = 0x165044CF
    EAkEvent_JETPACK_BOOST_PICKUP = 0x1662D4A8
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_RT_01_OUT = 0x168E23FE
    EAkEvent_PLAY_WT_MELODY_INDEX_18_NMS2017_WANTEDLOOPS_3 = 0x16A15695
    EAkEvent_PLAY_WT_BASS_INDEX_08_MONOLITH_WANTED_SYNTHLOW_1 = 0x16BE8EB1
    EAkEvent_BLD_AMB_ATM = 0x16C1B9A5
    EAkEvent_FREIGHTER_DOOR_REINFORCED_OPEN = 0x16C6217B
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_FEED = 0x16F3DD78
    EAkEvent_ATLAS_ORB_ILLUMINATE = 0x1736FCDD
    EAkEvent_NOTIFY_HIT_RAD = 0x176C6800
    EAkEvent_PL_SHIELD_HIT = 0x17D34A45
    EAkEvent_PL_SHIP_ENTER = 0x17D409FD
    EAkEvent_MECH_JETPACK_LANDING_SKID = 0x180A89A5
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_IDLE02 = 0x1826E278
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_IDLE03 = 0x1826E279
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_IDLE01 = 0x1826E27B
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_IDLE06 = 0x1826E27C
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_IDLE07 = 0x1826E27D
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_IDLE04 = 0x1826E27E
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_IDLE05 = 0x1826E27F
    EAkEvent_TERMINAL_BOOTUP = 0x183AB7F3
    EAkEvent_WPN_PL_GRENADE_EXPLODE = 0x183BF870
    EAkEvent_BASEBUILD_ITEM_COMMUNICATIONSSTATION_CANCEL = 0x1855CDED
    EAkEvent_UI_FRONTEND_TEXTOVERLAY_DISCOVERY = 0x185DAFCB
    EAkEvent_ABANDONED_FIEND_IDLE = 0x187DE821
    EAkEvent_BLD_LIGHTS = 0x1892FE37
    EAkEvent_LOWERORDERGREET1IPAD = 0x18A08D70
    EAkEvent_TIMBER_DOOR1_SETTLEMENT_OPEN = 0x18A8586D
    EAkEvent_UI_ACTION_CONFIRMED = 0x18AECCF6
    EAkEvent_HOVERCRAFT_START = 0x18B0E3B0
    EAkEvent_MUS_MONOLITH = 0x18D5A84D
    EAkEvent_OBJECT_ANALYSING = 0x18EF30C7
    EAkEvent_STOP_MAP_ZONE01_INDEX_01_MAP_ZONE01 = 0x18F8DB64
    EAkEvent_NOTIFY_PIRATESSPOTTED = 0x19003798
    EAkEvent_BUGGY_HORN_STOP = 0x19167876
    EAkEvent_FRIGATE_LIVING_WARPIN = 0x195615B7
    EAkEvent_NOTIFY_PRO_PERCENT = 0x19661DC8
    EAkEvent_SUITBOOT_WEAPON = 0x19BC0505
    EAkEvent_PLAY_PL_VOCAL_INDEX_10_TETSUO_VOXSOFT_1 = 0x19D7608A
    EAkEvent_STOP_WT_DRUMS_INDEX_07_EOTWS_WANTEDBEATS_1 = 0x19E54EC8
    EAkEvent_NOTIFY_HOSTILES = 0x19F50A20
    EAkEvent_TURRET_INACTIVE_IDLE = 0x19F5429E
    EAkEvent_POWERMODULE_NOTE_B_STOP = 0x1A3E6F83
    EAkEvent_SHIP_PIRATE_SAILS_TRIANGLE_UNFOLD = 0x1A6698D8
    EAkEvent_NOTIFY_DAMAGEVEHICLESUSTAINED = 0x1A9FD076
    EAkEvent_PILGRIM_START = 0x1AA44330
    EAkEvent_WPN_SHIP_GUN = 0x1AAC86AC
    EAkEvent_PL_DAMAGE_WARNING_LOW_HEALTH_LP_STOP = 0x1AD6B960
    EAkEvent_WEIRDROLL_STOP = 0x1AD9EA60
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_THUMBSUP = 0x1B2E5FE8
    EAkEvent_PLAY_SP_DRONE_01_INDEX_20_TUNG_SPACEMODAL_14 = 0x1B3D3175
    EAkEvent_BASE_LARGEMONITORSTATION = 0x1B3F5918
    EAkEvent_NOTIFY_UNITS_DEPLETED = 0x1B43FD00
    EAkEvent_CHARACTEREDITOR_WOOSH = 0x1B532920
    EAkEvent_BLD_SCIENTIFIC_WINDOW_OPEN = 0x1B615F21
    EAkEvent_SPVYKEEN_CHATTER = 0x1B77C414
    EAkEvent_POI_RELAYBEACON = 0x1B7DE839
    EAkEvent_VO_INVENTORY_FULL = 0x1B7EE4FB
    EAkEvent_DRONE_INVESTIGATING = 0x1B83BC48
    EAkEvent_NPC_FOLEY_CLOTHING_1H_SAD_LP01 = 0x1B899810
    EAkEvent_BASE_GARAGE_DOOR_OPEN = 0x1B8E3148
    EAkEvent_GROUND_SIZZLE_STOP = 0x1BAC5A39
    EAkEvent_PLAY_WT_GUITAR_INDEX_23_ECHO_WANTEDGTRS_1 = 0x1BB1747F
    EAkEvent_NOTIFY_TRANSFERRED_PRODUCT = 0x1BBA22D9
    EAkEvent_NPC_VOCAL_SIT_ANGRY_02 = 0x1BC6EB34
    EAkEvent_NPC_VOCAL_SIT_ANGRY_01 = 0x1BC6EB37
    EAkEvent_PL_FOLEY_CLOTHING_MULTITOOL_OVERHEAT = 0x1BD229F3
    EAkEvent_UI_FRONTEND_PRODUCTSBUILD = 0x1C2B169B
    EAkEvent_FISHFIEND_SPAWN = 0x1C3DC95F
    EAkEvent_NOTIFY_HAZ_CRIT_HEAT = 0x1C46AC7E
    EAkEvent_MUS_FIRSTTIMESPACESTATION = 0x1C477914
    EAkEvent_UI_SETTLEMENT_INVEST = 0x1C799E89
    EAkEvent_UI_NEW_SUIT_SLOT = 0x1C7D7573
    EAkEvent_VR_FOLEY_HAND_CLENCH_CLOSE = 0x1CA287DE
    EAkEvent_BASE_MINERALEXTRACTOR_OFF = 0x1CB23CA5
    EAkEvent_NOTIFY_HAZ_CRIT_RADIATION = 0x1CDFE22D
    EAkEvent_PL_PROTECTION_LP = 0x1D037974
    EAkEvent_FREIGHTER_ELEVATOR = 0x1D26E750
    EAkEvent_SETTLEMENT_ANTENNA3_SPIN = 0x1D5ED15A
    EAkEvent_ATLAS_GALAXYSTART_SINGLE = 0x1D6B1966
    EAkEvent_BASE_ELECTRICALWIRING_PLACE = 0x1D81BC71
    EAkEvent_PLAY_SP_SYNTH_INDEX_13_BEYOND_SLOWSYNTH_6 = 0x1D942713
    EAkEvent_BLD_SCIENTIFIC_WINDOW_CLOSE = 0x1DEC7157
    EAkEvent_BYTEBEAT_PLAYER = 0x1DF2F9B5
    EAkEvent_GLITCHY_MONITOR = 0x1E04A85C
    EAkEvent_FRIGATE_LIVING_DEVOUR = 0x1E1153E9
    EAkEvent_CRT_GRUNT_WINGFLAPWALK = 0x1E4355FF
    EAkEvent_FOURTHRACE_ARTEMIS_CHATTER00 = 0x1E577730
    EAkEvent_PLAY_WT_DRUMS_INDEX_17_APORIA_WANTEDBEATS_2 = 0x1E5E0C9A
    EAkEvent_PLAY_PL_PIANO_INDEX_20_TUNG_PLANET_PIANO_2 = 0x1E867B77
    EAkEvent_MUS_LOADING_STOP = 0x1E870702
    EAkEvent_PL_SHIP_TAKEOFF = 0x1EA46787
    EAkEvent_FOURTHRACE_ARTEMIS_ANGRY00 = 0x1EA5A71C
    EAkEvent_BLD_RUIN_MONOLITH = 0x1EDC03C1
    EAkEvent_UI_INCOMING_NORMANDY_WARNING = 0x1EE13682
    EAkEvent_TERRAIN_UNDO = 0x1EEA9A75
    EAkEvent_UI_MENU_INCORRECT = 0x1F60B11B
    EAkEvent_PORTAL_JUMP_STORY_END = 0x1F67ACB6
    EAkEvent_POI_LINE_GEO = 0x1F696682
    EAkEvent_TELEPORT = 0x1F992208
    EAkEvent_NEXUS_WALLFAN = 0x1FA26F80
    EAkEvent_PLAY_WT_BASS_INDEX_27_OVERSEER_WANTEDBASS_2 = 0x1FEA96F4
    EAkEvent_CATALTATTACK2 = 0x20004E5E
    EAkEvent_CATALTATTACK3 = 0x20004E5F
    EAkEvent_PLANT_GATHER_GRAHFRUIT = 0x20134DA1
    EAkEvent_NOTIFY_HOT_ROCK = 0x2022A12E
    EAkEvent_EXPL_FIREWORK_LAUNCH = 0x203840E0
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_STROKE = 0x2056D3D0
    EAkEvent_NOTIFY_THRUSTERSOFFLOCAL = 0x206BDF55
    EAkEvent_BLD_TONEINT_GENERIC_08 = 0x207AA534
    EAkEvent_BLD_TONEINT_GENERIC_09 = 0x207AA535
    EAkEvent_BLD_TONEINT_GENERIC_04 = 0x207AA538
    EAkEvent_BLD_TONEINT_GENERIC_05 = 0x207AA539
    EAkEvent_BLD_TONEINT_GENERIC_06 = 0x207AA53A
    EAkEvent_BLD_TONEINT_GENERIC_07 = 0x207AA53B
    EAkEvent_BLD_TONEINT_GENERIC_01 = 0x207AA53D
    EAkEvent_BLD_TONEINT_GENERIC_02 = 0x207AA53E
    EAkEvent_BLD_TONEINT_GENERIC_03 = 0x207AA53F
    EAkEvent_STOP_WT_DRUMS_INDEX_25_DASHKA_WANTEDBEATS_3 = 0x2090B95E
    EAkEvent_VO_TECH_RECHARGED = 0x20A524C7
    EAkEvent_PORTAL_JUMP_NORMAL_END = 0x20AA1BB2
    EAkEvent_JETPACK_WATER_TRANSITION_INTO_WATER = 0x20BEC596
    EAkEvent_POI_ATLAS_BEACON_01 = 0x20E5FCAC
    EAkEvent_POI_ATLAS_BEACON_02 = 0x20E5FCAF
    EAkEvent_UI_PULSEDRIVE_OFFLINE = 0x210BD31B
    EAkEvent_INTERACTION_CARGOBAY = 0x2153E9AE
    EAkEvent_NEXUS_SMALLDRONE_FRAME640 = 0x21755640
    EAkEvent_BLD_TONEINT_GENERIC_18 = 0x217AA6A7
    EAkEvent_BLD_TONEINT_GENERIC_17 = 0x217AA6A8
    EAkEvent_BLD_TONEINT_GENERIC_16 = 0x217AA6A9
    EAkEvent_BLD_TONEINT_GENERIC_15 = 0x217AA6AA
    EAkEvent_BLD_TONEINT_GENERIC_14 = 0x217AA6AB
    EAkEvent_BLD_TONEINT_GENERIC_13 = 0x217AA6AC
    EAkEvent_BLD_TONEINT_GENERIC_12 = 0x217AA6AD
    EAkEvent_BLD_TONEINT_GENERIC_11 = 0x217AA6AE
    EAkEvent_BLD_TONEINT_GENERIC_10 = 0x217AA6AF
    EAkEvent_COWHINDROAR01 = 0x2197E780
    EAkEvent_MUS_STORYMODE_MUSICCUE_10_LP_STOP = 0x21A4CB50
    EAkEvent_WEIRDCONTOUR = 0x21ADDAD6
    EAkEvent_NEXUS_ROBOARM_MOVE1_REV = 0x21CFF112
    EAkEvent_POLICE_CHATTER_START = 0x22042BF0
    EAkEvent_MONOLITH_BEAM_START = 0x22177738
    EAkEvent_UI_COMBAT_UNIT_DEPLOYED_03 = 0x221ADFB0
    EAkEvent_UI_COMBAT_UNIT_DEPLOYED_02 = 0x221ADFB1
    EAkEvent_UI_COMBAT_UNIT_DEPLOYED_01 = 0x221ADFB2
    EAkEvent_ANOMALY_SIMULATION_CONSOLE_START = 0x226E710D
    EAkEvent_PLAY_SP_ARP_HI_INDEX_16_MONOALT_7 = 0x228B67A7
    EAkEvent_NEXUS_ROBOARM_MOVE1 = 0x229C720C
    EAkEvent_NOTIFY_VEHICLEMOBILISED = 0x22ACF15B
    EAkEvent_SETTLEMENT_FURNACE = 0x22B71075
    EAkEvent_PULSE_EMPHASISER = 0x22CEF034
    EAkEvent_FREIGHTER_REFINER_RUNNING = 0x22D0D8CD
    EAkEvent_NOTIFY_TARGETLOCKED = 0x22D490DC
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_06_EV_HIGHINTEREST_1 = 0x22EB2562
    EAkEvent_BASE_BATTERY_DISCHARGING = 0x230E6054
    EAkEvent_SENTINEL_MECH_WPN_FIRE = 0x2345B1E2
    EAkEvent_DRONE_ATTACK = 0x234F1F00
    EAkEvent_PL_SHIP_COCKPIT_RAIN = 0x2380A5E5
    EAkEvent_CATPOUNCE = 0x23978AD9
    EAkEvent_PET_UI_COLLECT_EGGS = 0x23B82F2F
    EAkEvent_CREATURE_VOCALS_SQUAWK_ATTACK = 0x23D5A471
    EAkEvent_NEXUS_VENT = 0x2440A0FC
    EAkEvent_STOP_MAP_CENTRECHORD_01_INDEX_01_MAP_CENTRECHORD_01 = 0x244CA730
    EAkEvent_NPC_SHIP_IDLE = 0x24526C1C
    EAkEvent_AF_DOOR_PRESSURISING = 0x24E7DF92
    EAkEvent_NOTIFY_PULSEOFFPLANET = 0x24FCA7F9
    EAkEvent_GROUNDWORM_OPEN = 0x25023634
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_16_MONOALT_PLANETMALLETFX_6 = 0x255A1F5C
    EAkEvent_POI_DERELICT_03 = 0x25A14BD0
    EAkEvent_POI_DERELICT_02 = 0x25A14BD1
    EAkEvent_POI_DERELICT_01 = 0x25A14BD2
    EAkEvent_PLAY_WT_GUITARS_02_INDEX_05_REDPARA_EBOWDRONE_5 = 0x25B9F145
    EAkEvent_NOTIFY_WARP = 0x261F5C3B
    EAkEvent_STONE_DOOR1_SETTLEMENT_OPEN = 0x2623C975
    EAkEvent_PLAY_WT_ONESHOTS_INDEX_01_MONOLITH_WANTED_FASTONESHOTS_13 = 0x2634FD60
    EAkEvent_ABANDONED_DOOR_UNLOCK = 0x264BED2D
    EAkEvent_NEXUS_SMALLDRONE_FRAME192 = 0x26731F8A
    EAkEvent_HOVERCRAFT_SUSPENSION = 0x26A065CF
    EAkEvent_SPVYKEEN_CHATTER_STOP = 0x26C45A13
    EAkEvent_TELEPORT_START = 0x26CB3885
    EAkEvent_ANTELOPE2LEGPOUNCE1 = 0x26DF7660
    EAkEvent_OBJ_REFINERYLARGE_STOP = 0x2745AFEF
    EAkEvent_HAZ_CRIT_SHIELD_STABLE = 0x274BF05F
    EAkEvent_NOTIFY_MISSILESNEUTRALISED = 0x27D01838
    EAkEvent_PL_VEHICLE_REMOTE_IDLE = 0x27EBC5B6
    EAkEvent_SPKORVAX_CHATTER = 0x27F01D43
    EAkEvent_PLAY_SP_DRONE_02_INDEX_22_VOSTOK_SPACEGRAIN_6 = 0x27F67AEF
    EAkEvent_PLAY_PL_PIANO_INDEX_23_ECHO_PIANOHIGH_2 = 0x280B0EAA
    EAkEvent_BASE_PIPE_PLACE = 0x280CACAD
    EAkEvent_STOP_WT_DRUMS_INDEX_10_TETSUO_DRUMLOOPS_1 = 0x28264EB3
    EAkEvent_FIRESTORM = 0x2841CE64
    EAkEvent_SHARK_LP = 0x28B07C37
    EAkEvent_PET_UI_TREAT = 0x28CCAA60
    EAkEvent_COMPUTER_TEXT_TYPE = 0x28D3F683
    EAkEvent_DRONE_MEDIC_ENGAGE = 0x28DE13F8
    EAkEvent_PLAY_WT_ARP_INDEX_06_EV_WANTEDARPS_4 = 0x29011642
    EAkEvent_PORTAL_RUNES_APPEAR = 0x2926B953
    EAkEvent_HOVERCRAFT_IDLE_EXTERIOR = 0x293C4C27
    EAkEvent_TERMINAL_ABANDONED_OPEN = 0x294C2E5B
    EAkEvent_AMB_SETTLEMENT_HUB = 0x297F17E3
    EAkEvent_UI_DECISIONTEXT_MOUSEOVER = 0x2986B81F
    EAkEvent_RODENTPOUNCE2 = 0x29931AF1
    EAkEvent_MINIWORM_DEATH = 0x2996DB4A
    EAkEvent_PLAY_SP_MISC_HI_INDEX_24_DOMO_SPACEHIGHINTEREST_9 = 0x29AB4507
    EAkEvent_EXOCRAFT_SUMMONING_STATION = 0x29EF00E2
    EAkEvent_WALKER_FALL_IMPACT = 0x2A1F88A2
    EAkEvent_CREATURE_STEPS = 0x2A6F6752
    EAkEvent_NOTIFY_PULSEOFFATMOS = 0x2A8B274B
    EAkEvent_PLAY_WT_FX_01_INDEX_25_DASHKA_WANTEDNOISE_5 = 0x2A8CCF98
    EAkEvent_PL_OUTOFBREATH = 0x2AC9FF29
    EAkEvent_SUBMARINE_JUMP = 0x2ACCB55A
    EAkEvent_CRYOCHAMBER_CLOSE = 0x2ADCB15D
    EAkEvent_SLIMEDOOR_BREAKA = 0x2AE26AA0
    EAkEvent_SLIMEDOOR_BREAKC = 0x2AE26AA2
    EAkEvent_SLIMEDOOR_BREAKB = 0x2AE26AA3
    EAkEvent_SLIMEDOOR_BREAKD = 0x2AE26AA5
    EAkEvent_NPC_SCIENCE_WORKSTATION_ARMS = 0x2AE4A2D5
    EAkEvent_WALKER_EMOTE_LP = 0x2AE78935
    EAkEvent_UI_MARKER = 0x2B07DD4C
    EAkEvent_FREIGHTER_ALARM_START = 0x2B11860C
    EAkEvent_UI_MISSION_NEWSECONDARY_OBJECTIVE = 0x2BAC95E3
    EAkEvent_WPN_TERRAIN_BUILD_END = 0x2BDF7F4B
    EAkEvent_MUS_GAMESTART_STOP = 0x2BE88C64
    EAkEvent_PORTAL_PILLARSDETACH = 0x2BF21928
    EAkEvent_PLAY_PL_FX_03_INDEX_03_SM_HHPATTER_6 = 0x2C589B01
    EAkEvent_BLD_ALARM_LP = 0x2CC0151E
    EAkEvent_PL_SHIP_GROUND_IMPACT = 0x2CE59259
    EAkEvent_EXPL_SHIP_SMALL = 0x2D0884BB
    EAkEvent_PLAY_WT_GUITAR_INDEX_17_APORIA_WANTEDSTABS_1 = 0x2D251D20
    EAkEvent_COMMUNICATOR_CLOSE = 0x2D2D0DB1
    EAkEvent_OBJ_SHIELD_REPAIRER_OFF = 0x2D439567
    EAkEvent_PLAY_WT_DRONE_02_INDEX_09_RIPLEY_GRAIN__1 = 0x2D7961A7
    EAkEvent_PLAY_SP_MELODY_INDEX_03_SM_JUNOCHORDS_DMIN_18 = 0x2D887AF5
    EAkEvent_INTERACT_COLLECT = 0x2DB05350
    EAkEvent_ANTPOUNCE1 = 0x2DD423DB
    EAkEvent_OBJ_SPINNINGGLOBE = 0x2E2F03EC
    EAkEvent_PLAY_MAP_DRONETONE_01_INDEX_01_MUS_MAP1_DRONE_E = 0x2E489D33
    EAkEvent_MUS_SPACEVIRGIN = 0x2E518E96
    EAkEvent_WEIRDROLL_LP = 0x2E855EEC
    EAkEvent_UI_GAMEMODE_SELECT = 0x2E8FBD50
    EAkEvent_COWATTACKHIND = 0x2E946A79
    EAkEvent_NOTIFY_NEWBOUNTY = 0x2ED364C4
    EAkEvent_UI_LAUNCH_NOTIFICATION = 0x2EDEE04B
    EAkEvent_PLAY_SP_FX_01_INDEX_22_VOSTOK_ECHO_6 = 0x2EDF484E
    EAkEvent_SHIP_PIRATE_SAILS_ROUNDSAILS_UNFOLD = 0x2F203F2A
    EAkEvent_STOP_WT_DRUMS_INDEX_14_EOTWS_WANTEDDRUMS_2 = 0x2F23E907
    EAkEvent_METALFORMATION_COLLECT = 0x2F24E23C
    EAkEvent_NEXUS_SMALLDRONE_FRAME0 = 0x2F28D10A
    EAkEvent_COMPUTER_TEXT = 0x2F2F62DC
    EAkEvent_PL_SHIP_EXIT = 0x2F3259B1
    EAkEvent_BLD_WARRIOR_WALLEX_CAP = 0x2F73E8B3
    EAkEvent_WEIRDBEAMSTONE = 0x2F81A4D0
    EAkEvent_SUITBOOT_FINISHED = 0x2FA0AF23
    EAkEvent_NEXUS_MIDDLEDRONE_IDLE = 0x2FA246D9
    EAkEvent_UI_GAMEMODE_WARNING = 0x2FB112AC
    EAkEvent_PLAY_SP_MELODY_INDEX_04_BLUEPRINT_ORGANSCATTER_EMIN_10 = 0x2FC2C04A
    EAkEvent_BLD_SCIENTIFIC_PISTON = 0x2FEB96E5
    EAkEvent_CATALTPERFORM01 = 0x2FF4EBE6
    EAkEvent_PLAY_PL_GUITARFAST_INDEX_01_CELESTIAL_GTRLOOP_2 = 0x30097230
    EAkEvent_SIGNALSCANNER = 0x300E2B2D
    EAkEvent_VO_TECH_OFFLINE = 0x301397AB
    EAkEvent_DEATH_DAMAGE_REBUILD = 0x30258C33
    EAkEvent_UI_INSUFFICIENT_FUEL = 0x302A36D2
    EAkEvent_PLAY_VO_WARNINGSENTINELELITE = 0x302ED34F
    EAkEvent_PL_FOLEY_CLOTHING_2H_MULTITOOLRELOAD = 0x3036188A
    EAkEvent_BASE_SPHERECREATE = 0x304F20F2
    EAkEvent_NOTIFY_MISSILESINCOMINGNEUTRALISED = 0x305C90C8
    EAkEvent_LOCKER_OPEN = 0x305CA008
    EAkEvent_DRONE_FRIENDLY_WANTED = 0x30684DA9
    EAkEvent_SUIT_INIT_LIFESUPPORT = 0x307E8019
    EAkEvent_PLAY_WT_ARP_INDEX_19_TUES_WANTEDARP_2 = 0x309679DF
    EAkEvent_UI_NEXT_MESSAGE = 0x30D44C11
    EAkEvent_PLAY_WT_DRUMS_INDEX_25_DASHKA_WANTEDBEATS_3 = 0x310645C0
    EAkEvent_PL_FOLEY_CLOTHING_JOG_TO_IDLE = 0x31229EF8
    EAkEvent_AMB_RAIN_SURFACE_STOP = 0x31243A67
    EAkEvent_OBJ_PORTABLEREFINERY_START = 0x318F8A43
    EAkEvent_MUS_STORYMODE_MUSICCUE_08_LP = 0x31987684
    EAkEvent_COWPOUNCE1 = 0x31C0A479
    EAkEvent_OBJ_SPINNINGCHAIR = 0x31D21E68
    EAkEvent_MONOLITH_INTERACTIONPLATFORM_START = 0x31D40DBC
    EAkEvent_SUBMARINE_HORN_STOP = 0x31E249AC
    EAkEvent_POWER_SWITCH_AUTOSWITCH_OFF = 0x31E73B95
    EAkEvent_COWFLOATSAD = 0x31FB447C
    EAkEvent_SUBMARINE_BOOST_START = 0x3227AA1A
    EAkEvent_SC_TR_HARV = 0x3234D002
    EAkEvent_UI_LANDINGPOST_LOCKON = 0x3241196A
    EAkEvent_VR_PULLTOUSE_STOP = 0x324C4A2E
    EAkEvent_LOWERORDERHAPPY2IPAD = 0x324DAE34
    EAkEvent_ANOMALY_SIMULATION_LIGHTSON = 0x32B01899
    EAkEvent_POI_CRYSTALS = 0x333E286B
    EAkEvent_PLAY_SP_ARP_HI_INDEX_04_BLUEPRINT_ARPEMIN_128BPM_3 = 0x3397A0BD
    EAkEvent_EYESTALK = 0x33AF58C1
    EAkEvent_WPN_TERRAIN_END = 0x33B1CD16
    EAkEvent_BLD_WARRIOR_FACTORY_ROOF = 0x33B9723E
    EAkEvent_MUS_ENDING = 0x3414FF98
    EAkEvent_OBJ_REFINERYLARGE_IDLE = 0x344C4FB9
    EAkEvent_STOP_WT_DRUMS_INDEX_11_AKIRAWANTEDBEATS3 = 0x34EAABBF
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_LOOK_R02 = 0x350207D8
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_LOOK_R01 = 0x350207DB
    EAkEvent_PLAY_SP_ARP_HI_INDEX_10_ARPTETSUO_B9 = 0x350D57DC
    EAkEvent_DRONE_SHIELD_ACTIVATE = 0x3530F375
    EAkEvent_NOTIFY_CRITICAL_COLD = 0x3534E9B7
    EAkEvent_NPC_VOCAL_GREET_STAND_02 = 0x35471998
    EAkEvent_NPC_VOCAL_GREET_STAND_03 = 0x35471999
    EAkEvent_NPC_VOCAL_GREET_STAND_01 = 0x3547199B
    EAkEvent_NPC_VOCAL_GREET_STAND_06 = 0x3547199C
    EAkEvent_NPC_VOCAL_GREET_STAND_07 = 0x3547199D
    EAkEvent_NPC_VOCAL_GREET_STAND_04 = 0x3547199E
    EAkEvent_SUBMARINE_HORN_START = 0x35490700
    EAkEvent_JETPACK_STOP = 0x35973C50
    EAkEvent_BASE_CUBEGADGET = 0x35F1697A
    EAkEvent_PLAY_WT_MELODY_INDEX_07_EOTWS_PLANETHIINTEREST_1 = 0x35FF80B0
    EAkEvent_PLACEMARKER_LP = 0x36097177
    EAkEvent_CRT_BIRD_FLAP = 0x361B8FB6
    EAkEvent_UI_VR_HANDMENU_CLOSE = 0x364CF0A6
    EAkEvent_NOTIFY_PRODUCT_RECEIVED = 0x36B5EED4
    EAkEvent_BASE_BATTERY_FULLYCHARGED = 0x36F5BDDF
    EAkEvent_NPC_VOCAL_ANGRY_STAND02 = 0x36FAD9E9
    EAkEvent_NPC_VOCAL_ANGRY_STAND01 = 0x36FAD9EA
    EAkEvent_FIRESTORM_STOP = 0x37098E83
    EAkEvent_HOLOHUB_LADDER_END = 0x372BBD97
    EAkEvent_PLAY_WT_DRONE_INDEX_16_MONOALT_GRAIN_10 = 0x37317CEA
    EAkEvent_PLAY_SP_DRONE_01_INDEX_02_ASIM_KORGSPACESYNTH_BMIN_9 = 0x37AD4C1C
    EAkEvent_STOP_SP_ARP_HI_INDEX_07_ARPEOTWS_A11 = 0x37B14CD4
    EAkEvent_MUS_STORYMODE_MUSICCUE_06_LP_STOP = 0x37EA3DD1
    EAkEvent_LANDINGBAY_ACTIVATE = 0x3816AA16
    EAkEvent_WPN_PL_SHOTGUN_PATHFINDER = 0x381A3A7C
    EAkEvent_PL_PROTECTION_END = 0x38657B27
    EAkEvent_PLAY_WT_GUITARS_02_INDEX_04_BLUEPRINT_GTREND_EMIN_6 = 0x387EC3EA
    EAkEvent_STOP_MAP_DRONETONE_01_INDEX_01_MUS_MAP1_DRONE_E = 0x38932291
    EAkEvent_POWER_SWITCH_PROXIMITY_ON = 0x38AE2063
    EAkEvent_CUSTOMISE_IDLE = 0x38B3D8BA
    EAkEvent_POI_SPACECLOCK = 0x38EF2B82
    EAkEvent_PL_SHIP_ENGINES_START_REMOTE = 0x38FAB124
    EAkEvent_BASE_GARAGE_DOOR_CLOSE = 0x3935565C
    EAkEvent_CREATURE_VOCALS_SQUAWK_DIE = 0x395F9689
    EAkEvent_LOWERORDERGREET5IPAD = 0x39664C44
    EAkEvent_SETTLEMENT_ALARM = 0x39A0EC12
    EAkEvent_SEAURCHINCLOSE = 0x39AEF411
    EAkEvent_POWERMODULE_NOTE_D_STOP = 0x39B74521
    EAkEvent_PLAY_SP_SYNTHHIGH_INDEX_17_APORIA_SPACE_HIGHINTEREST_3 = 0x39C50559
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_27_OVERSEER_PLANETACOUSTIC_13 = 0x39F704ED
    EAkEvent_NOTIFY_GAS_PLANT = 0x3A05F2D6
    EAkEvent_UI_MESSAGE_GENERIC = 0x3A148ACD
    EAkEvent_PL_SHIP_SPARKS = 0x3A41BA4B
    EAkEvent_UI_MAP_INTERACT_TIMER = 0x3A90F61B
    EAkEvent_NOTIFY_RADLEAK_DAMAGE = 0x3A928F91
    EAkEvent_HUD_JUMP_ENGAGE = 0x3AC8D717
    EAkEvent_UI_QUICKMENU_TAB = 0x3AC9E576
    EAkEvent_PL_VEHICLE_REMOTE_SUSPENSION = 0x3B127ED3
    EAkEvent_POI_NEURON = 0x3B4DCF27
    EAkEvent_STOP_SP_ARP_HI_INDEX_10_ARPTETSUO_B9 = 0x3B73F3F2
    EAkEvent_PL_VEHICLE_REMOTE_SURFACE = 0x3B76DFB9
    EAkEvent_UI_NPC_SHIP_INTERACTION = 0x3BAC3D09
    EAkEvent_BLD_BLACKBOX_OPEN = 0x3BBECA3F
    EAkEvent_POWERMODULE_NOTE_C_STOP = 0x3BEAA2A8
    EAkEvent_UI_ERROR = 0x3C2700B8
    EAkEvent_BLD_TR_HAR_INT = 0x3C3CB2A0
    EAkEvent_NPC_WPN_TURRET_STOP = 0x3C50938A
    EAkEvent_PL_SUIT_STARTUP_INTERACT_TIMER = 0x3C572AD4
    EAkEvent_DRONE_DEBRIS = 0x3CCD53D1
    EAkEvent_NPC_VOCAL_SIT_GREET_02 = 0x3CF60EF0
    EAkEvent_NPC_VOCAL_SIT_GREET_01 = 0x3CF60EF3
    EAkEvent_PLAY_PL_GUITARFAST_INDEX_08_CELESTIAL_GTRLOOP_3 = 0x3D1FC51C
    EAkEvent_PL_FOLEY_CLOTHING_LADDER_CYCLE_UP = 0x3D3DDF5C
    EAkEvent_JETPACK_BOOST_TIMEOUT = 0x3D3E8565
    EAkEvent_PLAY_WT_BASS_INDEX_26_MELB_PLANETBASS_2 = 0x3D53BDE3
    EAkEvent_PLAY_SP_ARP_HI_INDEX_07_ARPEOTWS_A11 = 0x3D5692D2
    EAkEvent_BYTEBEAT = 0x3DB63CCF
    EAkEvent_PL_FOLEY_CLOTHING_0H_SWIM_BACK = 0x3DEA4F58
    EAkEvent_WARRIORHAPPY01 = 0x3DEECF18
    EAkEvent_WARRIORHAPPY03 = 0x3DEECF1A
    EAkEvent_WARRIORHAPPY02 = 0x3DEECF1B
    EAkEvent_PLAY_WT_GUITAR_INDEX_10_TETSUO_GTR1 = 0x3E3A0819
    EAkEvent_WPN_PL_SMG = 0x3E5AE267
    EAkEvent_LOWERORDERCHATTERIPAD = 0x3E606FBB
    EAkEvent_PLAYER_DIE_SURFACE = 0x3E80F715
    EAkEvent_PORTAL_RUNE_UNAVAILABLE = 0x3E81D195
    EAkEvent_GAS_BASTARD_POP = 0x3EA92568
    EAkEvent_PL_FOLEY_EMOTE_HELP = 0x3EAC0F72
    EAkEvent_JETPACK_WATER_TRANSITION_INTO_ATMOS = 0x3EBFB6F9
    EAkEvent_BINOCULARS_EXIT = 0x3EEB7CAE
    EAkEvent_TURRET_LASER_STOP = 0x3EF4C726
    EAkEvent_FREIGHTER_DOOR_INTERNAL_OPEN = 0x3F55DE61
    EAkEvent_ATLASSTARTSCENEORB_DISMISS = 0x3F5AD170
    EAkEvent_ATLASSTARTSCENEORB_ACTIVATE = 0x3F60B597
    EAkEvent_UI_QUICKMENU_TECHCHARGE = 0x3F8067B9
    EAkEvent_VR_FOLEY_ARMS_WOOSH = 0x3FAF1A30
    EAkEvent_DRONE_FRIENDLY_SUMMONED = 0x3FFEDE7A
    EAkEvent_PLAY_PL_DRONE_01_INDEX_01_EQUINOXSOFT_EMIN_2 = 0x400244DF
    EAkEvent_UI_TRADING_TEXT_TRANSLATE = 0x4022E06A
    EAkEvent_WEIRDBUBBLE_STOP = 0x40235AED
    EAkEvent_TERMINAL_ABANDONED_GROWTH_CLOSE = 0x403391D3
    EAkEvent_COMMS_CHATTER_POLICE_HIT = 0x40407039
    EAkEvent_FRIGATE_LIVING_REVEAL = 0x4045C9B3
    EAkEvent_PL_WPN_SHIP_MISSILE_SHOOT = 0x404C8523
    EAkEvent_SPIDERATTACK = 0x4090D662
    EAkEvent_BLD_SCIENTIFIC_DOOR_CLOSE = 0x40961F37
    EAkEvent_OBJ_HARVESTER = 0x4099D393
    EAkEvent_BUGGY_JUMP = 0x40AED174
    EAkEvent_WPN_SHIP_LASER_STOP = 0x40FBA3DA
    EAkEvent_DRONE_SCAN_STOP = 0x410B5462
    EAkEvent_ATLASSTARTSCENEORB_IDLE = 0x4123CCDA
    EAkEvent_UI_BASEBUILD_MENU_OPEN = 0x412ACD7A
    EAkEvent_PLAY_SP_SYNTHPADS_INDEX_27_OVERSEER_SPACEDRIFTS_10 = 0x415027FE
    EAkEvent_NOTIFY_HOSTILESYSTEMENGAGE = 0x4161E437
    EAkEvent_FORFANGRY01 = 0x419C389A
    EAkEvent_SHIP_ROYAL_LANDINGGEAR_TAKEOFF = 0x41A85C03
    EAkEvent_COMMUNICATOR_OPEN = 0x41B5F833
    EAkEvent_WARN_HOSTILE_APPROACH = 0x41DD6191
    EAkEvent_UI_MAP_LINES = 0x41EB3474
    EAkEvent_VO_STARSHIP_SIGNAL = 0x41EB6874
    EAkEvent_PLAY_WT_MELODY_INDEX_28_SCRAPER_WANTEDMELS_10 = 0x41FA07E6
    EAkEvent_WPN_GROUND_SIZZLE = 0x4222F8E2
    EAkEvent_PLAY_WT_MELODY_INDEX_03_SM_JOEGTRBURTS_DMIN_19 = 0x42473AD4
    EAkEvent_TENTPLANT_CLOSE = 0x4248A730
    EAkEvent_EXPLOSION_RESOURCE_GATHER = 0x42613325
    EAkEvent_PLANT_ILLUMINATED = 0x426CF1E7
    EAkEvent_BLD_WARRIOR_DOOR_CLOSE = 0x42737C4C
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_16_MONOALT_VIOLIN_2 = 0x42820E95
    EAkEvent_BLD_ALARM_LP_STOP = 0x42B994FD
    EAkEvent_PLAY_SP_ARP_HI_INDEX_14_EOTWSALT_11 = 0x42BAC2B1
    EAkEvent_WPN_SHIP_CYCLOTRON_OVERHEAT = 0x42DE8842
    EAkEvent_PORTAL_RUNE_SELECT = 0x42F26699
    EAkEvent_TERMINAL_INTERACT = 0x4387FCEE
    EAkEvent_ABANDONED_JELLYFISH_DEATH = 0x439E085F
    EAkEvent_TIMBER_DOOR1_SETTLEMENT_CLOSE = 0x43A6512B
    EAkEvent_NOTIFY_ASSISTANT = 0x44073D5F
    EAkEvent_CRYSTALALTAR_COLLECT = 0x441AEFA0
    EAkEvent_BLD_WARRIOR_DOOR_OPEN = 0x44339BD8
    EAkEvent_ROBOT_GUNMELEE_IMPACTS = 0x4436A6B4
    EAkEvent_UI_FRONTEND_TRANSFER = 0x44713ED8
    EAkEvent_BASE_MINERALEXTRACTOR = 0x44AD4073
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_ANGRY05 = 0x44CB2FE2
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_ANGRY02 = 0x44CB2FE5
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_ANGRY01 = 0x44CB2FE6
    EAkEvent_PLAY_SP_MELODY_INDEX_15_MAC_SPACEPADS_4 = 0x44EA076C
    EAkEvent_PLAYER_ORB = 0x44F92B10
    EAkEvent_MILK_BOTTLING = 0x4511C1AE
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_GREET_WAVE = 0x451D518D
    EAkEvent_STOP_SP_ARP_HI_INDEX_08_HUSK_7 = 0x4545DC2D
    EAkEvent_OBSERVATORY_HOLOGRAM = 0x4587B823
    EAkEvent_DRONE_MEDIC_DISENGAGE = 0x45F4D79A
    EAkEvent_NOTIFY_SHIELD_DOWN = 0x4619ABB5
    EAkEvent_PLAY_PL_FX_01_INDEX_03_SM_ATMOS_DMIN_4 = 0x4635AF44
    EAkEvent_HAZ_CRIT_SHIELD = 0x4643A77B
    EAkEvent_VEHICLE_MARKER_PLACE = 0x46C78620
    EAkEvent_PL_SHIP_ENGINES_STOP_REMOTE = 0x46E00B8A
    EAkEvent_MUS_STORYMODE_MUSICCUE_22_LP_STOP = 0x46E34383
    EAkEvent_NOTIFY_TARGETACQUIRED = 0x4734B5E6
    EAkEvent_ATLAS_CORE_INTERACT = 0x4734CC65
    EAkEvent_PLAY_PL_DRONE_02_INDEX_18_NMS2017_POLYHIGH_1 = 0x4753E199
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_11_AKIRA_DULCECLUSTER_7 = 0x478AC911
    EAkEvent_PLAY_PL_SYNTH_INDEX_15_HAUNTED_DIGITALSHARK_EMIN_2 = 0x480E9420
    EAkEvent_PL_FOLEY_CLOTHING_IDLE_WEIGHT_SHIFT = 0x48554711
    EAkEvent_ATLASSTARTSCENEORB_END = 0x4855A843
    EAkEvent_COMMS_CHATTER_PIRATES_AMBIENT = 0x48838B34
    EAkEvent_STOP_SP_ARP_HI_INDEX_16_MONOALT_7 = 0x4892E98D
    EAkEvent_SETTLEMENT_CONSTRUCTION_TERMINAL = 0x48A1CD9B
    EAkEvent_QUAD_LASER_STOP = 0x48BCC9CB
    EAkEvent_UI_PHOTOMODE_CAMERAPLACE_START = 0x48E446C9
    EAkEvent_HOVERCRAFT_STOP = 0x4909EE1C
    EAkEvent_NEXUS_BUBBLINGTUBE_SMALL = 0x495F0A84
    EAkEvent_FIREPLACE_LP = 0x499F8DC1
    EAkEvent_ROBOT_LASER_IMPACTS = 0x49D29DA7
    EAkEvent_UI_SENTINELINVESTIGATORS_INCOMING = 0x49D6DFF3
    EAkEvent_PLAY_PL_GUITAR_02_INDEX_22_VOSTOK_BOWEDGTRREV_18 = 0x49DDA12D
    EAkEvent_UI_WEIRDBIOME_COUNTDOWN_END = 0x4A1515D5
    EAkEvent_FOURTHRACE_PROC_GREET03 = 0x4A1A67C8
    EAkEvent_FOURTHRACE_PROC_GREET02 = 0x4A1A67C9
    EAkEvent_FOURTHRACE_PROC_GREET01 = 0x4A1A67CA
    EAkEvent_FOURTHRACE_PROC_GREET00 = 0x4A1A67CB
    EAkEvent_FOURTHRACE_PROC_GREET04 = 0x4A1A67CF
    EAkEvent_UI_WEIRDBIOME_COUNTDOWN = 0x4A2D6533
    EAkEvent_OBJ_REFINERYMEDIUM_STOP = 0x4A31510D
    EAkEvent_PL_SHIP_HYPERSPACE_START = 0x4AA4CB30
    EAkEvent_NOTIFY_HAZ_CRIT_TOXIC = 0x4AA5AE85
    EAkEvent_FIBREGLASS_DOOR1_SETTLEMENT_OPEN = 0x4ADAF334
    EAkEvent_PORTAL_PILLARSLOCK = 0x4B8CCC60
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_INTO = 0x4BB40382
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_GREET03 = 0x4BCB4C30
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_GREET02 = 0x4BCB4C31
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_GREET01 = 0x4BCB4C32
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_GREET07 = 0x4BCB4C34
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_GREET06 = 0x4BCB4C35
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_GREET05 = 0x4BCB4C36
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_GREET04 = 0x4BCB4C37
    EAkEvent_TREXATTACK = 0x4BDEEBF0
    EAkEvent_WARN_HOSTILE_DETECTED = 0x4BE3C41F
    EAkEvent_HAZ_CRIT_COLD = 0x4C14E836
    EAkEvent_PLAYER_SHIP_TRANSITION = 0x4C30CFEB
    EAkEvent_POWER_SWITCH_FLOOR_ON = 0x4C5B965C
    EAkEvent_AMB_ABANDONEDBUILDING = 0x4C987896
    EAkEvent_PORTAL_STOP_ALL = 0x4CAFE42A
    EAkEvent_AF_CAPSULE = 0x4CDD3358
    EAkEvent_PORTAL_CAMERA_TOTERMINAL = 0x4CF219CD
    EAkEvent_JELLYFISH_SPAWN = 0x4CF735B9
    EAkEvent_LAVA_VOLCANO_AMBIENT = 0x4D0F2CED
    EAkEvent_QUAD_POUNCE = 0x4D21BF6D
    EAkEvent_ANOMALY_SIMULATION_CONSOLE_FLASH = 0x4D525F5B
    EAkEvent_UI_STANDING_INCREASED = 0x4D5E4C1F
    EAkEvent_PLANT_ATTACK = 0x4D6B4503
    EAkEvent_ROVER_HORN_START = 0x4D72C430
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_19_TUES_PLANETGTR_3 = 0x4D7ACD5F
    EAkEvent_OBJ_FLAG = 0x4D907B09
    EAkEvent_UI_WEAPON_FOUND = 0x4DBA89E7
    EAkEvent_TURRET_DEPLOY = 0x4DD50ACB
    EAkEvent_ABANDONED_JELLYFISH_SWIM = 0x4DD7469F
    EAkEvent_FREIGHTER_DOOR_REINFORCED_CLOSE = 0x4DE5D229
    EAkEvent_SHIP_PIRATE_LANDING_WINGS = 0x4DE7AD60
    EAkEvent_FIBREGLASS_DOOR2_SETTLEMENT_OPEN = 0x4E200377
    EAkEvent_ROCKCREATURE_TRANSFORM = 0x4E24239C
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_15_HAUNTED_GTRSHARK_EMIN_4 = 0x4E2457C5
    EAkEvent_NOTIFY_NEW_SHIP = 0x4E3FD7DE
    EAkEvent_HEATER_ON = 0x4E481692
    EAkEvent_PL_CAMOUFLAGE_IN = 0x4E6C24EA
    EAkEvent_FIEND_ATTACK = 0x4EBD3690
    EAkEvent_PLANT_GATHER_JADEPEAS = 0x4ED86D0A
    EAkEvent_PLAY_SP_ARP_HI_INDEX_03_SUPERMOON_17 = 0x4EDC8A96
    EAkEvent_SHIP_ALIEN_LANDING = 0x4EF74A6B
    EAkEvent_LOWERORDERGREET4IPAD = 0x4F012E65
    EAkEvent_MUS_STORYMODE_MUSICCUE_08_LP_STOP = 0x4F40ADE3
    EAkEvent_BLACKHOLE_WARP = 0x4F9D6EE1
    EAkEvent_UI_MAP_SELECT = 0x4FA1AADF
    EAkEvent_UI_SERVER_SELECT = 0x4FC49B2E
    EAkEvent_UI_ITEM_TRANSFERTOSUIT = 0x4FDFB677
    EAkEvent_PLAY_PL_DRONE_01_INDEX_13_BEYOND_PLANETDRONE_9 = 0x4FF2963A
    EAkEvent_SHIP_ROYAL_WINGSB_LANDING = 0x4FF8C854
    EAkEvent_CREATURE_VOCALS_SQUAWK_ROAR_ANGRY = 0x4FF9BF6F
    EAkEvent_UI_SERVER_SEARCHING_START = 0x502FCE43
    EAkEvent_BLD_DMG_MACH_POD_LP = 0x5035727B
    EAkEvent_COMMS_CHATTER_STOP_ALL = 0x50409A1D
    EAkEvent_NOTIFY_NEW_SLOT = 0x5048D544
    EAkEvent_VO_PRODUCT_CONSTRUCTED = 0x50544311
    EAkEvent_PL_SHIP_MINIWARP_COUNTDOWN = 0x5064D24A
    EAkEvent_PL_FOLEY_CLOTHING_0H_SWIM_IDLE = 0x50A7E0CF
    EAkEvent_EXPL_ELECTRICAL_STUN_STOP = 0x50AC08C1
    EAkEvent_PLAY_WT_ONESHOTS_INDEX_17_APORIA_PLANETSTABS_37 = 0x50D748A5
    EAkEvent_SHIP_ROYAL_WINGSA_LANDING = 0x50E01CA7
    EAkEvent_FREIGHTER_STORAGE_TERMINAL = 0x511D7268
    EAkEvent_PLAY_WT_DRUMS_INDEX_13_BEYONDTOMORROW1 = 0x5188A113
    EAkEvent_MECH_JETPACK_LANDING = 0x520C6999
    EAkEvent_UI_PURCHASE = 0x5222F6E7
    EAkEvent_POI_EYE_BEAM_LP = 0x52AF1964
    EAkEvent_STARSHIP_VIEW_SWITCH_INTERIOR = 0x52B4A22F
    EAkEvent_PLAY_WT_BASS_INDEX_25_DASHKA_WANTEDBASSSYNTH_5 = 0x52D2E4AE
    EAkEvent_UI_VR_WRISTMENU_SELECT = 0x52EBA056
    EAkEvent_MUS_LOADING50 = 0x53A310A0
    EAkEvent_MUS_LOADING51 = 0x53A310A1
    EAkEvent_MUS_LOADING52 = 0x53A310A2
    EAkEvent_PL_SHIP_MINIWARP_END = 0x53B4B6F4
    EAkEvent_SUITBOOT_INIT = 0x53D1EA23
    EAkEvent_PL_SHIP_START_ACTIVE = 0x53F54244
    EAkEvent_BASE_SOL_OPEN = 0x53FBD758
    EAkEvent_STOP_MAP_DRONENOISE_01_INDEX_01_MAP_SPACE_NOISE_DRONE = 0x542550E2
    EAkEvent_MUS_LOADING43 = 0x54A31210
    EAkEvent_MUS_LOADING42 = 0x54A31211
    EAkEvent_MUS_LOADING41 = 0x54A31212
    EAkEvent_MUS_LOADING40 = 0x54A31213
    EAkEvent_MUS_LOADING47 = 0x54A31214
    EAkEvent_MUS_LOADING46 = 0x54A31215
    EAkEvent_MUS_LOADING45 = 0x54A31216
    EAkEvent_MUS_LOADING44 = 0x54A31217
    EAkEvent_MUS_LOADING49 = 0x54A3121A
    EAkEvent_MUS_LOADING48 = 0x54A3121B
    EAkEvent_STOP_MAP_ZONE02_INDEX_01_MAP_ZONE02 = 0x54A95BC0
    EAkEvent_MUS_STORYMODE_STOPALL = 0x5519D8C5
    EAkEvent_MECH_STEP_WALK = 0x5569365D
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_RT_02_IN = 0x558369C6
    EAkEvent_HAZ_CRIT_TOXIC_STABLE = 0x55AB8951
    EAkEvent_PLAY_SP_MELODY_INDEX_01_ASCENSION_ELEKTRIK_EMIN_1 = 0x55ADB9E1
    EAkEvent_TXT_STATICNOISE_START = 0x55C1C063
    EAkEvent_NOTIFY_VEHICLEDEPOLYED = 0x55CB1197
    EAkEvent_PLAY_PL_DRONE_01_INDEX_24_DOMO_PLANETIRIS_2 = 0x5652655A
    EAkEvent_PLAY_PL_BASS_INDEX_26_MELB_PLANETBASS_8 = 0x5659A098
    EAkEvent_BUGGY_START = 0x574EA9C4
    EAkEvent_UI_WAKEUP = 0x57610D41
    EAkEvent_PLAY_VO_WARNINGSENTINELINCOMING = 0x57798EBA
    EAkEvent_GROUND_SIZZLE = 0x577DF00A
    EAkEvent_MUS_LOADING18 = 0x57A316E4
    EAkEvent_MUS_LOADING19 = 0x57A316E5
    EAkEvent_MUS_LOADING14 = 0x57A316E8
    EAkEvent_MUS_LOADING15 = 0x57A316E9
    EAkEvent_MUS_LOADING16 = 0x57A316EA
    EAkEvent_MUS_LOADING17 = 0x57A316EB
    EAkEvent_MUS_LOADING10 = 0x57A316EC
    EAkEvent_MUS_LOADING11 = 0x57A316ED
    EAkEvent_MUS_LOADING12 = 0x57A316EE
    EAkEvent_MUS_LOADING13 = 0x57A316EF
    EAkEvent_NOTIFY_INVENTORY_SUIT_FULL = 0x57A8357F
    EAkEvent_PLAY_PL_PIANO_INDEX_09_RIPLEY_PIANOHI_1 = 0x57B0FC0A
    EAkEvent_UI_MAP_FILTER_CHANGE = 0x57BB81D0
    EAkEvent_MUS_STORYMODE_MUSICCUE_11_LP = 0x57BEDBE2
    EAkEvent_PLAY_MAP_DRONENOISE_02_INDEX_01_MAP_SPACE_NOISE_DRONE_RR = 0x57C811D0
    EAkEvent_PLAY_PL_DRONE_02_INDEX_01_TOMORROW_FM_1 = 0x57C9311D
    EAkEvent_MUS_GAMESTART_LUSHWALK_STOP = 0x57DB907A
    EAkEvent_DEATHMARKER_PULSE = 0x57F9C699
    EAkEvent_CREATURE_POOP = 0x57FC4B17
    EAkEvent_NOTIFY_SENTINELSARRIVED = 0x5807CBC3
    EAkEvent_HAZ_CRIT_RAD = 0x582BD949
    EAkEvent_SENTINELS_INCOMING1 = 0x5862BAD0
    EAkEvent_SENTINELS_INCOMING3 = 0x5862BAD2
    EAkEvent_SENTINELS_INCOMING2 = 0x5862BAD3
    EAkEvent_SENTINELS_INCOMING5 = 0x5862BAD4
    EAkEvent_SENTINELS_INCOMING4 = 0x5862BAD5
    EAkEvent_UI_INTERACT_TIMER_STOP = 0x587C411D
    EAkEvent_NOTIFY_HOSTILES_DEFEATED = 0x5886A77D
    EAkEvent_PLAY_PL_FX_01_INDEX_01_BOWEDCYMBAL_CLEAN_6 = 0x588F99C5
    EAkEvent_MUS_LOADING09 = 0x58A31856
    EAkEvent_MUS_LOADING08 = 0x58A31857
    EAkEvent_MUS_LOADING07 = 0x58A31858
    EAkEvent_MUS_LOADING06 = 0x58A31859
    EAkEvent_MUS_LOADING05 = 0x58A3185A
    EAkEvent_MUS_LOADING04 = 0x58A3185B
    EAkEvent_MUS_LOADING03 = 0x58A3185C
    EAkEvent_MUS_LOADING02 = 0x58A3185D
    EAkEvent_MUS_LOADING01 = 0x58A3185E
    EAkEvent_DRONE_SUMMONER_UNSIGNAL = 0x58AEE782
    EAkEvent_WPN_SHIP_LASER = 0x58B37AEF
    EAkEvent_CREATURE_ROAR = 0x58EB908D
    EAkEvent_PLAY_MAP_ZONE01_INDEX_01_MAP_ZONE01 = 0x5908A1EE
    EAkEvent_SPOREVENT_IDLE = 0x598D4C12
    EAkEvent_BUGGY_BOOST_STOP = 0x5998369C
    EAkEvent_MUS_LOADING32 = 0x59A31A30
    EAkEvent_MUS_LOADING33 = 0x59A31A31
    EAkEvent_MUS_LOADING30 = 0x59A31A32
    EAkEvent_MUS_LOADING31 = 0x59A31A33
    EAkEvent_MUS_LOADING36 = 0x59A31A34
    EAkEvent_MUS_LOADING37 = 0x59A31A35
    EAkEvent_MUS_LOADING34 = 0x59A31A36
    EAkEvent_MUS_LOADING35 = 0x59A31A37
    EAkEvent_MUS_LOADING38 = 0x59A31A3A
    EAkEvent_MUS_LOADING39 = 0x59A31A3B
    EAkEvent_JETPACK_BOOST_START = 0x59EEB4F8
    EAkEvent_ATLAS_SPHERE_WIND = 0x59F53DC7
    EAkEvent_WPN_SHIP_CHANGE = 0x5A129B56
    EAkEvent_DRILL_LP_STOP = 0x5A24BEF6
    EAkEvent_UI_QUICKMENU_CLOSE = 0x5A2D907B
    EAkEvent_DRONE_ARMOURED_BRACE = 0x5A7DE16F
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_RT_02_LP = 0x5A8371F7
    EAkEvent_FISHFIEND_EAT = 0x5A853604
    EAkEvent_SPOREBAG_WOBBLE = 0x5AA09996
    EAkEvent_MUS_LOADING25 = 0x5AA31BA0
    EAkEvent_MUS_LOADING24 = 0x5AA31BA1
    EAkEvent_MUS_LOADING27 = 0x5AA31BA2
    EAkEvent_MUS_LOADING26 = 0x5AA31BA3
    EAkEvent_MUS_LOADING21 = 0x5AA31BA4
    EAkEvent_MUS_LOADING20 = 0x5AA31BA5
    EAkEvent_MUS_LOADING23 = 0x5AA31BA6
    EAkEvent_MUS_LOADING22 = 0x5AA31BA7
    EAkEvent_MUS_LOADING29 = 0x5AA31BAC
    EAkEvent_MUS_LOADING28 = 0x5AA31BAD
    EAkEvent_NPC_CLOTHING_1H_INTERACT_IPAD02 = 0x5AA49489
    EAkEvent_NPC_CLOTHING_1H_INTERACT_IPAD01 = 0x5AA4948A
    EAkEvent_VAULT_OPEN = 0x5AE8C76E
    EAkEvent_VEHICLE_SPAWN = 0x5B82CC5F
    EAkEvent_PLAY_SP_SYNTH_INDEX_27_OVERSEER_SPACESYNTH_6 = 0x5B9A5C75
    EAkEvent_NOTIFY_HAZ_WATER_HEAT = 0x5BC840DB
    EAkEvent_ANTATTACK2 = 0x5BCBDC96
    EAkEvent_POD_BEAT = 0x5BD80B25
    EAkEvent_PLAYER_DIE = 0x5BEE16D1
    EAkEvent_ANOMALY_SIMULATION_LIGHTSOFF = 0x5C36AC25
    EAkEvent_BLD_POD_STARTUP = 0x5C4B561F
    EAkEvent_PLAY_PL_FX_01_INDEX_06_EV_PLANETNOISE_9 = 0x5C56712E
    EAkEvent_TRIATTACK = 0x5C7D1FC4
    EAkEvent_UI_PICKUP_LOOT = 0x5CA94A23
    EAkEvent_PLAY_PL_MUSICBOX_INDEX_01_MUSICBOXLONG_EMIN = 0x5CCD8909
    EAkEvent_AMB_BUILDING_STOP = 0x5CE58FED
    EAkEvent_WPN_PL_TAKEOUT = 0x5CE8E2AB
    EAkEvent_OBJ_STANDINGSTONES = 0x5CEA7805
    EAkEvent_TRIATTACK2 = 0x5CF901BE
    EAkEvent_TRIATTACK3 = 0x5CF901BF
    EAkEvent_PL_SURVEYING_DISCOVER = 0x5CFC7900
    EAkEvent_PL_SURVEYING_HIGHLIGHT = 0x5D0CC4E3
    EAkEvent_OBJ_SHIELD_REPAIRER_ON = 0x5D1B7B93
    EAkEvent_BLD_RUIN_PLAQUE = 0x5D206C15
    EAkEvent_BASE_SPHEREROLL = 0x5D2DB27F
    EAkEvent_PLAY_SP_MISC_HI_INDEX_01_MONOLITH_MISC_GMIN_1 = 0x5D2FCB47
    EAkEvent_SPIDERPOUNCE = 0x5D37D550
    EAkEvent_EGGRESOURCE_COLLECT = 0x5D44C34B
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_LOOKDOWN01 = 0x5D508CC4
    EAkEvent_PILGRIM_BOOST_START = 0x5D72659C
    EAkEvent_DRONE_MEDIC_IDLE = 0x5D87B953
    EAkEvent_FLOATING_GASBAG_EXPLODE = 0x5D91DBB7
    EAkEvent_POWERMODULE_NOTE_E_STOP = 0x5DCA590E
    EAkEvent_VILESPAWN_LP_STOP = 0x5DCB9CCA
    EAkEvent_NOTIFY_PRO_OFFLINE = 0x5DEDA806
    EAkEvent_POI_ASTEROID_BEACON = 0x5E085E18
    EAkEvent_UI_MAP_PLANET_HOVER = 0x5E11CDF2
    EAkEvent_PL_SHIP_SPARKS_STOP = 0x5E131FC6
    EAkEvent_PLAY_PL_SYNTH_INDEX_25_DASHKA_PLANETCOUNTER_314 = 0x5E16C9DD
    EAkEvent_SHIP_ROYAL_LANDINGGEAR_LANDING = 0x5E8606D2
    EAkEvent_BLD_WORDSTATION = 0x5EADD344
    EAkEvent_UI_BASEBUILD_PLACEBUILDING = 0x5EB87AEF
    EAkEvent_OBJ_MINING_AMU = 0x5F04D3FD
    EAkEvent_AMB_FREIGHTER_INTERNAL = 0x5F322152
    EAkEvent_STOP_SP_ARP_HI_INDEX_05_REDPARA_SPACEARP_AMIN_7 = 0x5F7A92B2
    EAkEvent_AF_EXPLODINGCAPSULE = 0x5F8A1444
    EAkEvent_FOURTHRACE_PROC_CHATTER00 = 0x5FB0BF2B
    EAkEvent_NOTIFY_FREIGHTERACQUIRED = 0x5FC34813
    EAkEvent_PLACEMARKER_IMPACT = 0x5FC8A561
    EAkEvent_SHUTTLE_TOPTHRUSTER = 0x5FE94583
    EAkEvent_TREXATTACK2 = 0x5FED6AE2
    EAkEvent_TREXATTACK3 = 0x5FED6AE3
    EAkEvent_BASEBUILD_ITEM_CONSTRUCTIONTERMINAL = 0x5FF59BA4
    EAkEvent_OILSTREAM = 0x607B82D7
    EAkEvent_UI_GLITCHED_MEMORY = 0x608D8720
    EAkEvent_PET_UI_FOOD_PELLET = 0x60B750C1
    EAkEvent_PLAY_SP_DRONE_01_INDEX_01_MONOLITH_WANTEDGTRDRONES_GMIN_3 = 0x60EAFF30
    EAkEvent_SENTINEL_MECH_LAND = 0x60F02DC9
    EAkEvent_HOVERCRAFT_JUMP = 0x611F8178
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_28_SCRAPER_PLANETGTRMELS_7 = 0x6123AC43
    EAkEvent_TERMINAL_INDOORS = 0x6138EE88
    EAkEvent_OBJ_CRATE_LARGE = 0x613B2D40
    EAkEvent_PLAY_MAP_SINGLES_01_INDEX_01_MUS_MAP01_SINGLES1_04 = 0x614FBC55
    EAkEvent_NPC_FOLEY_CLOTHING_0H_TURN_L_120 = 0x61557725
    EAkEvent_DEATH_DAMAGE = 0x617076E7
    EAkEvent_CATALTHAPPY = 0x6179CA9A
    EAkEvent_EXPL_SHIPS_MED = 0x618DEA29
    EAkEvent_PLAY_PL_STRINGS_INDEX_09_RIPLEY_CELLO_1 = 0x61AF6512
    EAkEvent_VEHICLE_DESTRUCTIBLE = 0x61DED67C
    EAkEvent_SHIP_ROYAL_ENGINEC_TAKEOFF = 0x61EDB2E6
    EAkEvent_NPC_FOLEY_CLOTHING_1H_STAND_ANGRY03 = 0x62053463
    EAkEvent_NPC_FOLEY_CLOTHING_1H_STAND_ANGRY04 = 0x62053464
    EAkEvent_SHUTTLE_THRUSTERS_STOP = 0x6211B2EE
    EAkEvent_QUAD_EMOTE_LP = 0x62200770
    EAkEvent_BASE_GASEXTRACTOR_OFF = 0x6270A7CE
    EAkEvent_SHIP_NORMANDY_ENGINES = 0x627E5670
    EAkEvent_RUININTERACTION_END = 0x629E486F
    EAkEvent_BLD_HOLOGRAM_APPEAR = 0x62AE0EA3
    EAkEvent_AMBIENT_EMPHASISER = 0x62B568C5
    EAkEvent_PL_SHIP_BOOST_POWERUP = 0x62CDCDDB
    EAkEvent_NOTIFY_PULSEOFFLARGESHIP = 0x62E50DE2
    EAkEvent_SHIP_ALIEN_NOSTRILFLARE = 0x6300644D
    EAkEvent_SPOREBAGEXPLODE = 0x631A514D
    EAkEvent_ANTELOPETWOLEGPOUNCE3 = 0x633AFD2E
    EAkEvent_ANTELOPETWOLEGPOUNCE2 = 0x633AFD2F
    EAkEvent_PLAY_WT_DRUMS_INDEX_15_MAC_DRUMLOOP_5 = 0x63BD3B06
    EAkEvent_POI_RELICGATE = 0x63E4FEA6
    EAkEvent_PLAY_SP_BASS_INDEX_26_MELB_SPACELOWS_5 = 0x6410B4E6
    EAkEvent_UI_PICKUP_PLANT = 0x6470457C
    EAkEvent_CREATURE_SWIPE = 0x64797DF9
    EAkEvent_NOTIFY_HOMEPLANETESTABLISHED = 0x648ABCEC
    EAkEvent_PL_SURVEYING = 0x64B2EA2A
    EAkEvent_WARN_DISTRESS_DETECTED = 0x64B78982
    EAkEvent_NOTIFY_ITEM_RECEIVED = 0x64B8C086
    EAkEvent_PET_UI_MENU_HOVER = 0x64BD01EE
    EAkEvent_POI_RADIO_PILLAR = 0x64CC1342
    EAkEvent_PLAY_SP_SYNTHPADS_INDEX_14_EOTWS2_SPACEPADS_3 = 0x64FBB0CF
    EAkEvent_UI_PICKUP_NITROGENPLANT = 0x65062DDA
    EAkEvent_STOP_WT_DRUMS_INDEX_19_TUES_WANTEDBEATS4 = 0x650EEF52
    EAkEvent_PLAY_WT_DRONE_INDEX_22_VOSTOK_ECHOREV_7 = 0x65358E7C
    EAkEvent_STOP_SP_ARP_HI_INDEX_02_ASIMOVARP_A1 = 0x657B7953
    EAkEvent_UI_MAP_NAV_MODE_NEXT = 0x65D24B80
    EAkEvent_BASE_POWER_POWERUP = 0x6627C5F9
    EAkEvent_POWERMODULE_NOTE_F = 0x662C5960
    EAkEvent_POWERMODULE_NOTE_G = 0x662C5961
    EAkEvent_POWERMODULE_NOTE_D = 0x662C5962
    EAkEvent_POWERMODULE_NOTE_E = 0x662C5963
    EAkEvent_POWERMODULE_NOTE_B = 0x662C5964
    EAkEvent_POWERMODULE_NOTE_C = 0x662C5965
    EAkEvent_POWERMODULE_NOTE_A = 0x662C5967
    EAkEvent_POWERMODULE_NOTE_A_STOP = 0x664B9122
    EAkEvent_EXPL_FIREWORK_TAIL_SLIME = 0x6665BA8A
    EAkEvent_PL_SHIP_TRANSITION_TO_PLANET_START = 0x6693D868
    EAkEvent_NOTIFY_CRITICAL_RAD = 0x6694BE72
    EAkEvent_TIMBER_DOOR2_SETTLEMENT_OPEN = 0x66990C1E
    EAkEvent_NOTIFY_WARPING = 0x66D34BA5
    EAkEvent_RADIO_CHATTER_STOP = 0x66DF5E6B
    EAkEvent_PORTAL_JUMP_NORMAL_END_AMBIENCES = 0x6713C8E4
    EAkEvent_PLAY_PL_GUITAR_04_INDEX_02_ASIM_PLANET_JOEGTRWASHSOFT_BMIN_4 = 0x6730E015
    EAkEvent_PET_UI_DISMISS = 0x6797B7FC
    EAkEvent_PLAY_SP_BASS_INDEX_07_EOTWS_REAMP_1 = 0x67A6502B
    EAkEvent_PL_SHIP_TAKEOFF_WHENREMAININSHIP = 0x67C6E21D
    EAkEvent_PLAY_PL_BASS_INDEX_01_MONOLITH_BOWEDBASSGTR_GMIN_6 = 0x67E0BFCD
    EAkEvent_PLAY_WT_DRUMS_INDEX_22_VOSTOK_WANTEDDRUMS_2 = 0x67E42272
    EAkEvent_PLANT_GATHER_SWEETROOT = 0x682532C3
    EAkEvent_BLD_DEBRISLARGE_OPEN = 0x6829FAA1
    EAkEvent_PL_SUIT_STARTUP = 0x68319AB9
    EAkEvent_GRABBYPLANT_ATTACK_END = 0x684EBBB8
    EAkEvent_UI_GAMEMODE_ERASE_ERROR = 0x68702A95
    EAkEvent_ATLAS_GALAXY_SPIN_LP = 0x68977FD7
    EAkEvent_SPACESTATION_PLANET_HOLOGRAM = 0x68D3A8E6
    EAkEvent_MUS_EXPEDITION_START = 0x68E79865
    EAkEvent_SUBMARINE_IDLE_EXTERIOR = 0x68EC56E9
    EAkEvent_PLAY_SP_LOWATMOS_INDEX_08_ASCENSION_LONGRISE_2 = 0x69291BF7
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_19_TUES_SPACEHIGHONESHOTS_11 = 0x69573671
    EAkEvent_ROVER_BOOST_STOP = 0x696003E2
    EAkEvent_PILGRIM_IDLE_EXTERIOR = 0x698C1BA7
    EAkEvent_PLAY_PL_DRONE_01_INDEX_14_EOTWSALT_STRINGHOLD_7 = 0x69C8FFA1
    EAkEvent_NEXUS_COOKINGFOOD = 0x6A2D035F
    EAkEvent_PLAY_SP_BASS_INDEX_18_NMS2017_SPACEBASS_7 = 0x6A2EFC7F
    EAkEvent_TXT_SPECIAL_CHIME = 0x6A37C076
    EAkEvent_UI_INCOMING_MESSAGE = 0x6A39A46E
    EAkEvent_VR_PULLTOUSE = 0x6A454EC3
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_NEUTRAL01 = 0x6A7ED79C
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_NEUTRAL02 = 0x6A7ED79F
    EAkEvent_BLD_AMB_DEPOTS_STOP = 0x6A9C012D
    EAkEvent_PLAY_WT_DRONE_INDEX_11_AKIRA_PLANETGRAIN_17 = 0x6AAB05BA
    EAkEvent_UI_PHOTOMODE_OUT = 0x6AAF1EDE
    EAkEvent_NPC_WPN_TURRET_START = 0x6AB593D2
    EAkEvent_PL_SHIP_BOOST_STOP = 0x6B0B4731
    EAkEvent_UNDERWATER_STEAMVENT_VENTING_END = 0x6B30806A
    EAkEvent_EGG_MACHINE_CLOSE = 0x6B395F97
    EAkEvent_PLAY_WT_FX_01_INDEX_04_BLUEPRINT_MOPHOSTUTTER_EMIN_4 = 0x6B45AE08
    EAkEvent_STOP_SP_ARP_HI_INDEX_14_EOTWSALT_11 = 0x6B902D07
    EAkEvent_CRT_COW_WINGFLAP = 0x6B97E021
    EAkEvent_OBJ_CRATE_SMALLA = 0x6B9851E9
    EAkEvent_OBJ_CRATE_SMALLB = 0x6B9851EA
    EAkEvent_BASE_POWER_SWITCH_OFF = 0x6BD0C41D
    EAkEvent_OBJ_SPINNINGSHAPE = 0x6BD9CA70
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_STRETCH = 0x6C12BE63
    EAkEvent_PLAY_PL_GUITAR_03_INDEX_02_ASIM_PLANET_MALLETGTRTWO_BMIN_2 = 0x6C2FABF8
    EAkEvent_UI_MAP_NAV_MODE_BACK = 0x6C37A7AE
    EAkEvent_PLAY_PL_DRONE_01_INDEX_25_SCRAPER_PLANETROOTS_6 = 0x6C589FF4
    EAkEvent_UI_DESTINATION_REACHED = 0x6C649E5D
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_HEROICINTO = 0x6C7928E2
    EAkEvent_CREATURE_VOCALS_SQUAWK_PAIN = 0x6C8842D5
    EAkEvent_STRIDERROAR = 0x6C9FC7B4
    EAkEvent_NEXUS_SMALLDRONE_LP = 0x6CC12C6F
    EAkEvent_WPN_PL_NEUTRON_CANNON_CHARGE_END = 0x6CF3B5E8
    EAkEvent_UI_SERVER_SELECT_POPUP = 0x6D1A2065
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_CNSL_IN = 0x6D1BD65F
    EAkEvent_PLAY_WT_DRONE_INDEX_03_DEPARTURE_GTR_EMIN_9 = 0x6D1E8BF2
    EAkEvent_UI_BLUEPRINT_KNOWN = 0x6D5C3D63
    EAkEvent_GROUNDWORM_CLOSE = 0x6D6FB800
    EAkEvent_PLAY_SP_FX_01_INDEX_28_SCRAPER_VERBAMBIENCE_7 = 0x6D70AB77
    EAkEvent_CREATURE_VOCALS_SQUAWK_IDLE = 0x6DCF4D8B
    EAkEvent_BASE_SERVER = 0x6DD9D114
    EAkEvent_JELLYFISH_SWIM = 0x6DF627F0
    EAkEvent_PLAY_PL_PIANO_INDEX_12_ATWOOD_PIANO_5 = 0x6E236852
    EAkEvent_SANDWORM_LP = 0x6E353C11
    EAkEvent_MUS_GAMESTART = 0x6E416461
    EAkEvent_OBJ_PLAQUE = 0x6E4AAD8B
    EAkEvent_SPGEK_CHATTER_STOP = 0x6E4D844C
    EAkEvent_NOTIFY_HAZ_WATER_TOXIC = 0x6E79BAEA
    EAkEvent_FISHFIEND_DIE = 0x6E831730
    EAkEvent_STOP_MAP_PIANO_INDEX_01_MUS_MAP01_PIANO_01 = 0x6EC06ADD
    EAkEvent_DRONE_SEARCHING = 0x6EC52C40
    EAkEvent_HOVERCRAFT_HORN_STOP = 0x6EEC151A
    EAkEvent_PILGRIM_SUSPENSION = 0x6EFEA44F
    EAkEvent_DRAGONFLY_SUSPENSION = 0x6F051BC9
    EAkEvent_UI_KNOWLEDGE_EXPANDED = 0x6F259D9A
    EAkEvent_HAZ_CRIT_RAD_FALLING = 0x6F3906B1
    EAkEvent_FOOTSTEP = 0x6F394B77
    EAkEvent_ATLAS_SPHERE_PULSE = 0x6F545734
    EAkEvent_BASE_GASEXTRACTOR = 0x6F563648
    EAkEvent_BUGGY_IDLE_EXTERIOR = 0x6F5E01BB
    EAkEvent_PLAY_WT_DRUMS_INDEX_23_ECHO_WANTEDLOOPS_2 = 0x6F73905E
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_02_ASIM_PLANET_PAULGTR_5 = 0x6FBE88C7
    EAkEvent_BLD_POD_SHUTDOWN = 0x6FD14262
    EAkEvent_UI_TRADING_TEXT = 0x7015F9ED
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_CNSL_LP = 0x701BDA9A
    EAkEvent_PLAY_PL_SYNTH_INDEX_08_CELESTIAL_MOD_8 = 0x70304420
    EAkEvent_SPACESTATION_ENGINES_INTERIOR = 0x7039CB96
    EAkEvent_BASE_MONITORSTATION = 0x703BBA57
    EAkEvent_STRIDERATTACK = 0x70596ECC
    EAkEvent_CREATURE_STEP_BLOB_WALK = 0x707CF181
    EAkEvent_PLAY_SP_SYNTHINTERFERENCE_INDEX_10_TETSUO_SPACEHIGHINTEREST_1 = 0x70929769
    EAkEvent_UI_PRODUCT_CREATED = 0x7094C6BE
    EAkEvent_WPN_PL_RAILGUN_PATHFINDER = 0x709D11EC
    EAkEvent_PLAY_WT_ONESHOTS_INDEX_11_BLUEPRINT_GTREND_EMIN_5 = 0x70AADAA4
    EAkEvent_NOTIFY_DAMAGESUSTAINED = 0x70C2A03A
    EAkEvent_SCAN_OFF = 0x70FDCEA8
    EAkEvent_SPIDERROAR = 0x711887FE
    EAkEvent_OBJ_ATMOSPHERE_HARVESTER = 0x71246308
    EAkEvent_NOTIFY_HAZ_NIGHT_TOXIC = 0x714CA8DD
    EAkEvent_PET_UI_SUMMON = 0x716F5B99
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_SITINTO = 0x71A2E776
    EAkEvent_NPC_SHIP_LASERS02 = 0x71E894F0
    EAkEvent_NPC_SHIP_LASERS03 = 0x71E894F1
    EAkEvent_NPC_SHIP_LASERS01 = 0x71E894F3
    EAkEvent_NPC_SHIP_LASERS04 = 0x71E894F6
    EAkEvent_SFX_FADEUP = 0x71E96FAC
    EAkEvent_UI_SLEEP = 0x71F138DB
    EAkEvent_BASETERMINAL = 0x71FD32FE
    EAkEvent_DISCOVER_PLANET = 0x721715FD
    EAkEvent_FEEDERDISPENSE = 0x722708E9
    EAkEvent_VENUSFLYTRAP_CLOSE = 0x722B8CBB
    EAkEvent_BASE_POWER_ON = 0x72477AB0
    EAkEvent_NEXUS_LANDING_RINGS = 0x725742B2
    EAkEvent_RACING_DOME_NOTIFIER = 0x72791D60
    EAkEvent_MINIWORM_LURKING = 0x72AF95EC
    EAkEvent_VIEW_THIRDPERSON = 0x72BDEFFF
    EAkEvent_MUS_MILESTONE_REWARDS = 0x72C807B0
    EAkEvent_UI_MISSION_NOTIFYSCAN = 0x73345F47
    EAkEvent_PLAY_VO_WARNINGWALKER = 0x7334AE90
    EAkEvent_PLAY_WT_ONESHOTS_INDEX_14_EOTWS_WANTEDBURN_4 = 0x733E0945
    EAkEvent_PL_FOLEY_CLOTHING_RUN_TO_IDLE = 0x73450387
    EAkEvent_CUSTOMISE_END = 0x734712A3
    EAkEvent_FOURTHRACE_APOLLO_CHATTER00 = 0x73545F86
    EAkEvent_ROVER_SUSPENSION = 0x73640735
    EAkEvent_FOURTHRACE_ARTEMIS_HAPPY01 = 0x738920B8
    EAkEvent_FOURTHRACE_ARTEMIS_HAPPY00 = 0x738920B9
    EAkEvent_FOURTHRACE_ARTEMIS_HAPPY02 = 0x738920BB
    EAkEvent_PLAY_WT_DRUMS_INDEX_21_OCT_WANTEDBEATS_1 = 0x738B234E
    EAkEvent_PL_BREATHING_STOP = 0x73C9C55D
    EAkEvent_PLOUGH = 0x73DFF4A4
    EAkEvent_FOURTHRACE_PROC_HAPPY02 = 0x7402CDD0
    EAkEvent_FOURTHRACE_PROC_HAPPY00 = 0x7402CDD2
    EAkEvent_FOURTHRACE_PROC_HAPPY01 = 0x7402CDD3
    EAkEvent_SUBMARINE_SUBMERGE = 0x74187E58
    EAkEvent_MESSAGEMODULE_IDLE = 0x74347F1F
    EAkEvent_BLOBROAR = 0x743C03A4
    EAkEvent_BLD_TONEINT_HANGER_STOP_01 = 0x74461F80
    EAkEvent_FOURTHRACE_APOLLO_IDLE00 = 0x747677FD
    EAkEvent_UI_TECH_INSTALLED = 0x748BD54D
    EAkEvent_FIEND_POUNCE = 0x74BB47B2
    EAkEvent_POI_JELLYFISH = 0x74BD0818
    EAkEvent_PL_SHIELD_SHIP_HIT_BIG = 0x74EB2D51
    EAkEvent_CATPERFORM01 = 0x74F5CCB3
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_ANGRY02 = 0x7502B291
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_ANGRY01 = 0x7502B292
    EAkEvent_PLANT_GATHER_ALOEFLESH = 0x755AFF3C
    EAkEvent_PLAY_SP_DRONE_02_INDEX_06_EV_SPACEDRONETWO_ENEUTRAL_4 = 0x75705BDA
    EAkEvent_QUAD_POUNCE_FORWARD = 0x75A11A07
    EAkEvent_UI_BUILD_ERROR = 0x75AC6211
    EAkEvent_UI_BEACON_DISCOVER = 0x75B9A824
    EAkEvent_CREATURE_VOCALS_SQUAWK_AGGRESSION = 0x75F5A46D
    EAkEvent_WPN_SHIP_CYCLOTRON_FIRE = 0x7611E876
    EAkEvent_PL_SHIELD_SHIP_HIT_SMALL = 0x768B8A3E
    EAkEvent_MECH_JETPACK_RETRIGGER = 0x76E0CB75
    EAkEvent_PLAY_WT_ONESHOTS_INDEX_03_TOMORROW_FM_CHOPPED_5 = 0x76EACDAC
    EAkEvent_SUITBOOT_SHIELD = 0x76ED680A
    EAkEvent_DRONE_DIE = 0x772B483E
    EAkEvent_UI_SHOP_SCROLL_UP = 0x77487C8A
    EAkEvent_OBJ_HEALTHSTATION_OFF = 0x777473D5
    EAkEvent_NOTIFY_UNITS_INSUFFICIENT = 0x77EA828C
    EAkEvent_PORTAL_PILLARSATTACH = 0x77EE9DE6
    EAkEvent_NOTIFY_MISSILESINCOMINGDESTROYED = 0x7844BE83
    EAkEvent_BASE_ELECTRICCUBE = 0x78527FE3
    EAkEvent_IMPACTS_GROUND_GENERIC_SMALL = 0x785AA13A
    EAkEvent_LANDING_PLATFORM_OPEN = 0x78991C27
    EAkEvent_PL_CAMOUFLAGE_OUT = 0x793836C5
    EAkEvent_WEIRDOFRACTCUBE = 0x79920C4A
    EAkEvent_NPC_FOLEY_CLOTHING_0H_TURN_R_120 = 0x799CCD57
    EAkEvent_UI_BASEBUILD_SNAP = 0x79A35452
    EAkEvent_PLAY_WT_ARP_INDEX_08_MONOLITH_SPACE_ARP_GMIN_4 = 0x79BDCBD2
    EAkEvent_NOTIFY_VEHICLEACQUIRED = 0x79BFA8D7
    EAkEvent_UI_MENU_BUILD_TAB = 0x79CE128A
    EAkEvent_PLAY_WT_SYNTH_INDEX_17_APORIA_SPACECHORDS_1 = 0x79F49AA7
    EAkEvent_NOTIFY_REFUELVEHICLE = 0x7A00F4CE
    EAkEvent_POWER_SWITCH_AUTOSWITCH_ON = 0x7A01D829
    EAkEvent_SEAURCHINEXTEND = 0x7A044893
    EAkEvent_ANOMALY_MAINAIRLOCK_OPEN = 0x7A31D97C
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_26_MELB_PLANETGTR_3 = 0x7A3F4310
    EAkEvent_BUGGY_IMPACTS = 0x7A3FF7D1
    EAkEvent_AMB_WEATHER_THUNDER_STOP = 0x7A68768C
    EAkEvent_DRONE_FRIENDLY_COMBAT_STOP = 0x7A6EDC0B
    EAkEvent_DRONE_ALERT = 0x7A84D746
    EAkEvent_AMB_BLACKHOLE = 0x7A8710CD
    EAkEvent_CRATE_WEAPON_CLOSE = 0x7ACC6D60
    EAkEvent_NOTIFY_THRUSTERSOFFATMOS = 0x7AD9AE32
    EAkEvent_PL_FOLEY_EMOTE_OVERHERE = 0x7B06DE1F
    EAkEvent_COMMUNICATOR_HAIL = 0x7B52749D
    EAkEvent_PLAY_MAP_ZONE02_SINGLES_INDEX_01_ZONE02_SINGLES02 = 0x7B91D5E9
    EAkEvent_PLAY_PL_DRONE_01_INDEX_08_ASCENSION_SPACE_6 = 0x7BD641CF
    EAkEvent_SUIT_INIT_STARTUP = 0x7BE29729
    EAkEvent_EGG_MACHINE_AMBIENT_LP = 0x7BE7B7BA
    EAkEvent_BASE_BATTERY_CHARGING = 0x7C13B3BA
    EAkEvent_OBJECT_OF_INTEREST = 0x7C61D8C5
    EAkEvent_PLAY_WT_GUITARS_02_INDEX_20_TUNG_WANTEDBASS_3 = 0x7C61FF9A
    EAkEvent_UI_BASEBUILD_MENU_CLOSE = 0x7C7C81DE
    EAkEvent_FREIGHTER_CORRIDOR = 0x7D0215EC
    EAkEvent_SETTLEMENT_GENERATOR = 0x7D11D44A
    EAkEvent_PLAY_WT_DRONE_02_INDEX_08_DEPARTURE_NOISE_EMIN_1 = 0x7D3F9967
    EAkEvent_PLAY_WT_DRONE_INDEX_08_BOREALIS_GTRFEEDBACK_AMIN_1 = 0x7D546579
    EAkEvent_NEXUS_ROTATINGSIGN = 0x7D5A7B6A
    EAkEvent_PILGRIM_IMPACTS = 0x7D8D4FFD
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_21_OCT_HIGHMEL_11 = 0x7D9658D5
    EAkEvent_NADA_GREET_08 = 0x7D9C920C
    EAkEvent_NADA_GREET_09 = 0x7D9C920D
    EAkEvent_PL_SHIP_SHIELD_DOWN = 0x7DC5CDE9
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_CHATTER02 = 0x7DE81D75
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_CHATTER01 = 0x7DE81D76
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_16_MONOALT_REVPIANO_11 = 0x7E23BA9F
    EAkEvent_UI_BASEBUILD_CALLFREIGHTER = 0x7E2FF586
    EAkEvent_BASE_POWERFLOW_ON = 0x7E373308
    EAkEvent_PLANT_CLOSE = 0x7E3854F5
    EAkEvent_PLAY_PL_BASS_INDEX_07_MONOLITH_BOWEDBASSGTR_GMIN_1 = 0x7E462BF8
    EAkEvent_RADIO_CHATTER = 0x7E66279C
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_05_REDPARA_CELESTA_9 = 0x7EC0D2BC
    EAkEvent_PLAY_SP_ARP_HI_INDEX_25_DASHKA_SPACEGTR_8 = 0x7ED77A28
    EAkEvent_SETTLEMENT_ANTENNA3_LOOP = 0x7EE99A9C
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_RT_02_OUT = 0x7EEEB7C9
    EAkEvent_DRONE_ENGINE = 0x7F0625AA
    EAkEvent_UI_TECH_BROKEN = 0x7F1E1E22
    EAkEvent_UI_BASEBUILD_CHANGECATEGORY = 0x7F2F183A
    EAkEvent_SUITBOOT_FREIGHTERINTRO_02 = 0x7F32ABF0
    EAkEvent_SUITBOOT_FREIGHTERINTRO_03 = 0x7F32ABF1
    EAkEvent_SUITBOOT_FREIGHTERINTRO_01 = 0x7F32ABF3
    EAkEvent_SUITBOOT_FREIGHTERINTRO_06 = 0x7F32ABF4
    EAkEvent_SUITBOOT_FREIGHTERINTRO_07 = 0x7F32ABF5
    EAkEvent_SUITBOOT_FREIGHTERINTRO_04 = 0x7F32ABF6
    EAkEvent_SUITBOOT_FREIGHTERINTRO_05 = 0x7F32ABF7
    EAkEvent_WALKER_LEG_HIT_LEFT_OUT_01 = 0x7F48B349
    EAkEvent_WALKER_LEG_HIT_LEFT_OUT_02 = 0x7F48B34A
    EAkEvent_PLAY_MAP_SINGLES_02_INDEX_01_MUS_MAP01_SINGLES2_09 = 0x7F6217FC
    EAkEvent_PET_UI_CUSTOM_SELECT = 0x7F84507C
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_CALLPET = 0x7F9262A7
    EAkEvent_PL_SHIP_ENGINES_START = 0x7FBDB4F7
    EAkEvent_NOTIFY_INVENTORY_SHIP_FULL = 0x7FEAB5CC
    EAkEvent_BLD_DOOR_EXPLODE = 0x801AE19A
    EAkEvent_SCAN_ON = 0x8022EEB2
    EAkEvent_PORTAL_PILLARSUNLOCK = 0x8060BB55
    EAkEvent_LANTERNCLOSE = 0x808EAE35
    EAkEvent_UI_MAP_NAV_MODE_SWITCH_FAIL = 0x809B54D6
    EAkEvent_PLAY_WT_ONESHOTS_INDEX_05_REDPARA_WANTEDSONGONESHOT_2 = 0x80B04A24
    EAkEvent_UI_WANTED_INCREASE = 0x80BA51BA
    EAkEvent_CREATURE_VOCALS_SQUAWK_ROAR = 0x80D02225
    EAkEvent_NEXUS_MARKETDOOR_OPEN = 0x81306600
    EAkEvent_PLAY_SP_MISC_HI_INDEX_22_VOSTOK_SPACEHIGHINT_60 = 0x814B3BC2
    EAkEvent_PLAY_WT_ARP_INDEX_17_APORIA_WANTEDARPS_1 = 0x816AC8F4
    EAkEvent_ABANDONED_FIEND_SPIT = 0x81754059
    EAkEvent_BEACON_TRIGGER_STOP = 0x819AACAF
    EAkEvent_PLAY_WT_BASS_INDEX_22_VOSTOK_WANTEDSTABS_3 = 0x819C2011
    EAkEvent_COMMS_CHATTER_PIRATES_HIT = 0x81CCB593
    EAkEvent_NOTIFY_HAZ_WATER_COLD = 0x81DE761F
    EAkEvent_DRAGONFLY_IDLE_EXTERIOR = 0x81E3589D
    EAkEvent_FREIGHTER_REFINER_ROOM = 0x82217F99
    EAkEvent_WPN_PL_NEUTRON_CANNON_FIRE_CHARGED = 0x8238361B
    EAkEvent_TURRET_RETRACT = 0x823861B5
    EAkEvent_PLAY_PL_GUITARDRONE_INDEX_08_DEPARTURE_GTR_EMIN_1 = 0x823E3728
    EAkEvent_PL_SHIP_SPACESTATION_LAUNCH_END = 0x824C1ACD
    EAkEvent_UI_MAP_NAV_PATH_NEXT = 0x82556BF0
    EAkEvent_NPC_HANDHOLOGRAM = 0x826D3313
    EAkEvent_PLAY_MAP_ZONE02_INDEX_01_MAP_ZONE02 = 0x827E6B9A
    EAkEvent_PLAY_PL_DRONE_01_INDEX_18_NMS2017_MIDS_52 = 0x828A5D91
    EAkEvent_WPN_PL_HANDLASER_STOP = 0x82A0A48B
    EAkEvent_PL_WOUND = 0x82AA4045
    EAkEvent_LAVA_VOLCANO_EXPLODE = 0x82B62E7E
    EAkEvent_SHIP_PIRATE_TAKEOFF_WINGS = 0x82FAE5D5
    EAkEvent_NOTIFY_FREIGHTERWARP = 0x82FD0FD1
    EAkEvent_SUBMARINE_IMPACTS = 0x83505C1F
    EAkEvent_BLOB_TERRARIUM = 0x836231DE
    EAkEvent_INTERACTION_CARGOBAYSTUCK = 0x8374264A
    EAkEvent_PIRATE_SALVAGE_CHAIR_SPIN = 0x839C9669
    EAkEvent_NOTIFY_EXTREMESENTINELPLANET = 0x83C94F75
    EAkEvent_PLAY_PL_PIANO_INDEX_17_APORIA_PLANETSINEKEYS_2 = 0x83E153A2
    EAkEvent_SHIP_NORMANDY_REVEAL = 0x83EEF348
    EAkEvent_PLAQUE_LOOP = 0x841DC68A
    EAkEvent_PLAY_WT_GUITAR_INDEX_21_OCT_WANTEDGTR_32 = 0x8421736D
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_DANCEINTO = 0x843BC963
    EAkEvent_ATLAS_PLINTH_RISE = 0x8464FB10
    EAkEvent_VR_FOLEY_ARM_MOVEMENTS_RIGHT = 0x84729F52
    EAkEvent_WPN_TERRAIN_DESTROY = 0x8487F81F
    EAkEvent_WARRIORANGRY = 0x8489D2FC
    EAkEvent_DRILL_LP = 0x84E6E75B
    EAkEvent_UI_PHOTOMODE_CAMERAPLACE_END = 0x84F4E2DA
    EAkEvent_PLAY_WT_MELODY_INDEX_27_OVERSEER_WANTEDMELS_4 = 0x84FB5085
    EAkEvent_PLAY_PL_HIGHCELLO_INDEX_09_RIPLEY_CELLOHIGHER_1 = 0x851502B6
    EAkEvent_LANDINGBAY_DEACTIVATE = 0x853967B7
    EAkEvent_LANDING_PLATFORM_CLOSE = 0x854717ED
    EAkEvent_LOWERORDERHAPPYIPAD = 0x8547CFD2
    EAkEvent_PLAY_PL_SYNTH_INDEX_17_APORIA_PLANETREAMPTEXTURE_22 = 0x85937240
    EAkEvent_POI_EYE_BEAM_END = 0x8593A2D7
    EAkEvent_FIEND_ROAR = 0x859699D0
    EAkEvent_PLAY_WT_SYNTH_INDEX_26_MELB_WANTEDKEYS_5 = 0x85C4424B
    EAkEvent_NOTIFY_BUILDLIMIT = 0x85C9120C
    EAkEvent_BASE_OSCILLOSCOPE = 0x8603C216
    EAkEvent_SANDWORM_INTOGROUND = 0x86327B5A
    EAkEvent_RESOURCE_GATHER = 0x86333C19
    EAkEvent_SUBMARINE_BOOST_STOP = 0x865165B2
    EAkEvent_POLICE_CHATTER_STOP = 0x86855F5C
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_POINT = 0x8685F006
    EAkEvent_NOTIFY_HIT_ENERGY = 0x869CE84F
    EAkEvent_STOP_WT_DRUMS_INDEX_16_MONOALT_WANTEDLOOPS_8 = 0x86A0D375
    EAkEvent_RUININTERACTION_IDLE = 0x86C1F9AE
    EAkEvent_UI_DISMANTLE = 0x86CF5DA3
    EAkEvent_CRYSTAL_RARE_LP = 0x86DDA9C7
    EAkEvent_AMB_PLANET_ALL_STOP = 0x870A61EF
    EAkEvent_SUITBOOT_LIFESUPPORT = 0x871648A8
    EAkEvent_PLANT_GATHER_FIREBERRY = 0x8716FE03
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_03_SM_PLANET_ATMOSGTR_DMIN_3 = 0x8732317B
    EAkEvent_UI_ITEM_TELEPORTTOSHIP = 0x8738FC46
    EAkEvent_PET_EGGSPLOSION = 0x874B2413
    EAkEvent_UI_SHOP_SCROLL_DOWN = 0x874F66BD
    EAkEvent_VIEW_FIRSTPERSON = 0x87AD2FC2
    EAkEvent_EXPLORERNEUTRALIPAD = 0x87AE24B1
    EAkEvent_NPC_SHIP_LAND = 0x87C20BE9
    EAkEvent_MILK_PUMPING = 0x87C5888F
    EAkEvent_NOTIFY_INSUFFICIENTRESOURCES = 0x87CD5C9B
    EAkEvent_DRILL_IN = 0x87E6EB8A
    EAkEvent_POWER_SWITCH_FLOOR_OFF = 0x882DA65A
    EAkEvent_PL_SHIP_VR_EXIT = 0x88814C78
    EAkEvent_PLAY_WT_DRUMS_INDEX_05_REDPARA_WANTEDDRUMLOOPS_1 = 0x88B928BF
    EAkEvent_TREXHAPPY = 0x88CF37AE
    EAkEvent_WALKER_LEG_HIT_IDLE = 0x891868C1
    EAkEvent_ROVER_JUMP = 0x892690CA
    EAkEvent_NOTIFY_NEWFREIGHTERACQUIRED = 0x8943C7D3
    EAkEvent_CLAMSHELL_IDLE = 0x896C2B0D
    EAkEvent_PARTICLES_SPARKS_COMMON = 0x89739B49
    EAkEvent_UI_UNITS_RECEIVED = 0x898CD56B
    EAkEvent_WPN_TERRAIN_UNDO = 0x89B7D91D
    EAkEvent_STOP_WT_DRUMS_INDEX_13_BEYONDTOMORROW1 = 0x89E4122D
    EAkEvent_MUS_XBOX_SPLASHSCREEN_STOP = 0x89EE888F
    EAkEvent_ANOMALY_WEAPONCRATE_OPEN = 0x8A1736EF
    EAkEvent_CREATURE_STEP_BLOB = 0x8A27FDBB
    EAkEvent_UI_ALERT_POLICE = 0x8A53362D
    EAkEvent_SUMMONSHIPBEACON_CLOSE = 0x8A5AA3F3
    EAkEvent_UI_PHOTOMODE_VALUEUP = 0x8A6C0E96
    EAkEvent_RUININTERACTION_ACTIVE = 0x8A6EEEE6
    EAkEvent_NOTIFY_MISSILEINBOUND = 0x8A853C06
    EAkEvent_BEETLE_WALK = 0x8AD063BC
    EAkEvent_SHIP_PIRATE_SAILS_SQUARE_FOLD = 0x8ADE0CE2
    EAkEvent_LANTERNOPEN = 0x8AFD33DF
    EAkEvent_PLAY_PL_VOX_INDEX_13_LULL_VOX_16 = 0x8B3A1916
    EAkEvent_SUBMARINE_START = 0x8B430A02
    EAkEvent_PLAY_PL_SYNTH_INDEX_05_REDPARA_SPACEFROST_AMIN_1 = 0x8B892521
    EAkEvent_UI_REFINERY_DECREASE = 0x8BA635D9
    EAkEvent_SHUTTLE_MAINTHRUSTER = 0x8BB99737
    EAkEvent_WPN_PL_PUTAWAY = 0x8C2DCD79
    EAkEvent_NPC_VOCAL_CHATTER_STAND_04 = 0x8C317B1A
    EAkEvent_NPC_VOCAL_CHATTER_STAND_02 = 0x8C317B1C
    EAkEvent_NPC_VOCAL_CHATTER_STAND_03 = 0x8C317B1D
    EAkEvent_NPC_VOCAL_CHATTER_STAND_01 = 0x8C317B1F
    EAkEvent_MINIWORM_IDLE = 0x8C3BC9E2
    EAkEvent_SETTLEMENT_LIGHTOUTDOOR = 0x8C6C2D83
    EAkEvent_GRUNTROAR = 0x8C8B6075
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_MINDBLOWN = 0x8CC7D7CE
    EAkEvent_JELLYFISH_DIE = 0x8D0341AE
    EAkEvent_PL_SHIP_WARP = 0x8D21AECF
    EAkEvent_UI_CRITICAL_HIT = 0x8D307B3B
    EAkEvent_NOTIFY_PIRATESLARGE = 0x8D320F36
    EAkEvent_CATROAR = 0x8D40571B
    EAkEvent_PLAY_PL_FX_01_INDEX_19_TUES_PLANETTREMOR_17 = 0x8D44671A
    EAkEvent_NOTIFY_DAMAGEVEHICLE = 0x8D4D1E66
    EAkEvent_VAULT_CLOSE = 0x8D7D844A
    EAkEvent_UI_CHOICETEXT_WOOSH = 0x8D7D9943
    EAkEvent_PL_BREATHING_START = 0x8D7EFCF7
    EAkEvent_NOTIFY_TECHNOLOGYCRITICALLYDAMAGED = 0x8D9A8992
    EAkEvent_QUAD_ATTACK = 0x8DCE66F7
    EAkEvent_FREIGHTER_BRIDGE = 0x8DDFA449
    EAkEvent_STOP_WT_DRUMS_INDEX_02_ASIM_DRUMSDIRTY_128BPM_6 = 0x8DEF0354
    EAkEvent_CREATURE_VOCALS_SQUAWK_SNORE = 0x8DFE9A1A
    EAkEvent_IDENTIFY_SCAN_OFF = 0x8E533775
    EAkEvent_BEAMSTONE = 0x8EC94FFF
    EAkEvent_SCRAP_DESTROY = 0x8ED2EF33
    EAkEvent_EXPL_FREIGHTER_CONTAINER_SMALL = 0x8ED7A637
    EAkEvent_PLAY_PL_FX_02_INDEX_03_BOWEDCYMBAL_CHOPDISTORT_22 = 0x8F5FF60A
    EAkEvent_POI_GYRO_BALL = 0x8FAB3C89
    EAkEvent_WPN_PL_MELEE = 0x8FC09CD8
    EAkEvent_NPC_VOCAL_SIT_HAPPY_01 = 0x8FD013FC
    EAkEvent_NPC_VOCAL_SIT_HAPPY_02 = 0x8FD013FF
    EAkEvent_WPN_SHIP_POSITRON_OVERHEAT = 0x8FDEE823
    EAkEvent_DRONE_SHIELD_GLITCH = 0x8FE1B631
    EAkEvent_PL_FOLEY_CLOTHING_LADDER_DISMOUNT_BOTTOM = 0x8FF84B91
    EAkEvent_FIBREGLASS_DOOR2_SETTLEMENT_CLOSE = 0x9004F65D
    EAkEvent_NOTIFY_MISSILEINCOMINGNEUTRALISED = 0x900B6253
    EAkEvent_PLAY_WT_PADS_INDEX_04_BLUEPRINT_INTROPADS_EMIN_48 = 0x9011770E
    EAkEvent_SMALLPROPC = 0x90279B9C
    EAkEvent_EGGOPENS = 0x9058C8F7
    EAkEvent_PLANT_GATHER_IMPULSEBEANS = 0x9063D3AF
    EAkEvent_PLAY_WT_DRUMS_INDEX_26_MELB_WANTEDBEATS_6 = 0x90D1B150
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_WONDER = 0x90D499C5
    EAkEvent_NOTIFY_HAZ_NIGHT_RADIATION = 0x90E58735
    EAkEvent_TXT_RADIONOISE = 0x911C13D3
    EAkEvent_UI_NANITES_INSUFFICIENT = 0x9134500C
    EAkEvent_PLANT_ILLUMINATED_STOP = 0x914B3AA2
    EAkEvent_PL_FOLEY_BODYHITS = 0x915FF2EE
    EAkEvent_SPARKS = 0x9160603F
    EAkEvent_PLAY_PL_MELODY_INDEX_25_DASHKA_PLANETLOOPS_6 = 0x9161064A
    EAkEvent_VEHICLE_RACE_END = 0x916A2903
    EAkEvent_ATLAS_CORE_ACTIVATE = 0x916A8160
    EAkEvent_NOTIFY_LOW_ENERGY = 0x916DB99C
    EAkEvent_BASEBUILD_ITEM_COMMUNICATIONSSTATION = 0x91C32F14
    EAkEvent_PL_FOLEY_EMOTE_UNDERWATER_THUMBSDOWN = 0x920C1178
    EAkEvent_PLAY_WT_DRONE_INDEX_12_ATWOOD_DRONEGTR_16 = 0x92198342
    EAkEvent_NPC_SHIP_LASER = 0x922FB54B
    EAkEvent_SPOREVENT_INFLATE = 0x92414141
    EAkEvent_CREATURE_VOCALS_SQUAWK_ROAR_HAPPY = 0x925E724A
    EAkEvent_PET_UI_EGG_HATCH = 0x92C0559C
    EAkEvent_ANTELOPETWOLEGATTACK3 = 0x92D5D2AC
    EAkEvent_ANTELOPETWOLEGATTACK2 = 0x92D5D2AD
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_02_ASIM_SPACEDUDE_9 = 0x92E2D7C0
    EAkEvent_NOTIFY_TRANSFERRED_ELEMENT = 0x92F3EE84
    EAkEvent_BYTEBEAT_STOP = 0x932527FA
    EAkEvent_BLD_SERVER = 0x933C9559
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_LOOKLEFT02 = 0x93461FF4
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_LOOKLEFT03 = 0x93461FF5
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_LOOKLEFT01 = 0x93461FF7
    EAkEvent_PL_VEHICLE_REMOTE_JUMP = 0x934AB5F4
    EAkEvent_LOWERORDERHAPPY4IPAD = 0x93565F52
    EAkEvent_MECH_MOVE_WALK = 0x936BCEAA
    EAkEvent_UI_ALIEN_RESPONSE = 0x938DD3E3
    EAkEvent_FIEND_IDLE = 0x9393197A
    EAkEvent_ANIMATEDDOOR_OPEN = 0x93B90D65
    EAkEvent_WPN_PL_NEUTRON_CANNON_FIRE = 0x93C21A7E
    EAkEvent_CATALTROAR = 0x93C693EC
    EAkEvent_PS5_VIBRATION_TONE_STOP = 0x93D00DEE
    EAkEvent_PLAY_WT_FX_01_INDEX_17_APORIA_SPACEBOWS_49 = 0x9461CB1B
    EAkEvent_FOURTHRACE_ARTEMIS_IDLE00 = 0x946BA2B3
    EAkEvent_BASE_ELECTRICALWIRING_DELETE = 0x94811EC9
    EAkEvent_PL_FOLEY_EMOTE_LAUGH = 0x953BF62E
    EAkEvent_EGG_MACHINE_OPEN = 0x955F5B61
    EAkEvent_PLAY_SP_MELODY_INDEX_11_AKIRA_SPACEORGAN_15 = 0x956D3D20
    EAkEvent_PL_SHIP_BURN_STOP = 0x958071DD
    EAkEvent_DRILL_OUT = 0x958077E5
    EAkEvent_PLAY_WT_FX_01_INDEX_19_TUES_SONIC_2 = 0x958C95B8
    EAkEvent_PL_HAZARD_CRITICAL_HIT = 0x95BC4F32
    EAkEvent_BASE_POCKETREALITYGENERATOR = 0x95D9B358
    EAkEvent_ANOMALY_CORE = 0x95FDDC1A
    EAkEvent_FOURTHRACE_NULL_GREET00 = 0x960B9B68
    EAkEvent_FOURTHRACE_NULL_GREET01 = 0x960B9B69
    EAkEvent_FOURTHRACE_NULL_GREET02 = 0x960B9B6A
    EAkEvent_FOURTHRACE_NULL_GREET03 = 0x960B9B6B
    EAkEvent_FOURTHRACE_NULL_GREET04 = 0x960B9B6C
    EAkEvent_NOTIFY_SENTINELSTARSHIPSCOMBAT = 0x965E55FA
    EAkEvent_PL_SHIELD_CHARGE = 0x969598CA
    EAkEvent_MESSAGEMODULE_START = 0x96AB9597
    EAkEvent_SHIP_PIRATE_SAILS_TRIANGLE_FOLD = 0x96B28FFD
    EAkEvent_SUMMONSHIPBEACON_OPEN = 0x96E60D05
    EAkEvent_EGG_MACHINE_START = 0x97160567
    EAkEvent_POD_BURST = 0x9731DC65
    EAkEvent_PLAY_PL_STRINGS_INDEX_17_APORIA_PLANETSOFTBED_2 = 0x975FE591
    EAkEvent_PLAY_WT_DRUMS_INDEX_01_MONOLITH_WANTED_DRUMLOOPS_20 = 0x976BCC42
    EAkEvent_ANOMALY_MAINAIRLOCK_CLOSE = 0x97933C38
    EAkEvent_VO_SECURE_TRADEMODULE_LOCATED = 0x97980A7C
    EAkEvent_PET_EGG_INDUCE = 0x979DE509
    EAkEvent_NOTIFY_ATTACKSHIPSENGAGED = 0x97E57481
    EAkEvent_MINIWORM_RETRACT = 0x97EB3C43
    EAkEvent_POI_SPACE_EGG = 0x983E577A
    EAkEvent_UI_WANTED_INCREASE_01 = 0x98402038
    EAkEvent_UI_WANTED_INCREASE_03 = 0x9840203A
    EAkEvent_UI_WANTED_INCREASE_02 = 0x9840203B
    EAkEvent_UI_WANTED_INCREASE_05 = 0x9840203C
    EAkEvent_UI_WANTED_INCREASE_04 = 0x9840203D
    EAkEvent_BASE_PIPE_DELETE = 0x987092CD
    EAkEvent_EXPL_FREIGHTER_CONTAINER_LARGE = 0x98E3FACB
    EAkEvent_PROTOROLLER = 0x98F8B9B1
    EAkEvent_UI_TEXT_WOOSH = 0x990CE3CE
    EAkEvent_BASEBUILD_ITEM_SCIENCETERMINAL = 0x99231F73
    EAkEvent_NEXUS_MARKETDOOR_CLOSE = 0x99262904
    EAkEvent_FORFGREET01 = 0x9934D7C6
    EAkEvent_OBJ_STANDINGSTONE_ACTIVATE = 0x994C77D2
    EAkEvent_BUGGY_SUSPENSION = 0x9979BE53
    EAkEvent_MUS_XBOX_SPLASHSCREEN = 0x99AFDCA0
    EAkEvent_PLAY_WT_DRUMS_INDEX_18_NMS2017_WANTEDSYNTHBEATS_1 = 0x99B4CD28
    EAkEvent_WEIRDBUBBLE = 0x99F5D6EE
    EAkEvent_UI_INSUFFICIENT_TECH_FRAGMENTS = 0x9A0D4494
    EAkEvent_TORNADO = 0x9A2BAEE8
    EAkEvent_FIEND_GROUNDAPPEAR = 0x9A41A76E
    EAkEvent_PL_SHIP_BOOST_FAIL = 0x9A4E27BD
    EAkEvent_PORTAL_DIAL_SPIN_START = 0x9AA1C4A6
    EAkEvent_BASE_SOL_CLOSE = 0x9AA926CC
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SHUTDWN_OUT = 0x9AAF193D
    EAkEvent_MONUMENT = 0x9AB78CBA
    EAkEvent_SHIP_ROYAL_WINGSC_LANDING = 0x9ACB3D2D
    EAkEvent_PLAY_WT_DRUMS_INDEX_14_EOTWS_WANTEDDRUMS_2 = 0x9ACE2CC5
    EAkEvent_CATALTATTACK = 0x9AD42B64
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_27_OVERSEER_SPACEHIGHINTEREST_10 = 0x9ADFD246
    EAkEvent_PL_DAMAGE_WARNING_LOW_HEALTH_LP = 0x9AE9352D
    EAkEvent_BUGGY_STOP = 0x9AF38178
    EAkEvent_AMB_NEBULA_STORM = 0x9B1672B5
    EAkEvent_PL_FOLEY_CLOTHING_MULTITOOL_DRAW = 0x9B2BB175
    EAkEvent_MUS_STORYMODE_MUSICCUE_13_LP = 0x9B4C59A0
    EAkEvent_NOTIFY_HIT_TOXIC = 0x9B632C5C
    EAkEvent_PLAY_SP_SYNTHHIGH_INDEX_26_MELB_SPACEHIGHINTEREST6 = 0x9B772B4C
    EAkEvent_BEACON_TRIGGER_START = 0x9B8CB96D
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER10 = 0x9B9C88B0
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER14 = 0x9B9C88B4
    EAkEvent_AF_ROTATINGDEVICE = 0x9BBD3FD5
    EAkEvent_UI_ALERT_DAMAGE_WARNING_CRITICAL_LP = 0x9BC95F06
    EAkEvent_PLAY_PL_FX_01_INDEX_18_NMS2017_IMPACTS_3 = 0x9BF2BB69
    EAkEvent_UI_MAP_POINT_SELECT = 0x9BFF4F86
    EAkEvent_UI_TARGET_ONSCREEN = 0x9C212C4F
    EAkEvent_PLAY_WT_GUITARS_02_INDEX_26_MELB_DIST_2 = 0x9C2E6721
    EAkEvent_PLAY_SP_DRONE_01_INDEX_13_BEYOND_SPACEKORG_13 = 0x9C46D223
    EAkEvent_COWATTACK3HIND = 0x9C8C20B2
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER03 = 0x9C9C8A20
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER02 = 0x9C9C8A21
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER01 = 0x9C9C8A22
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER07 = 0x9C9C8A24
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER06 = 0x9C9C8A25
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER05 = 0x9C9C8A26
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER04 = 0x9C9C8A27
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER09 = 0x9C9C8A2A
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_CHATTER08 = 0x9C9C8A2B
    EAkEvent_STATION_CONSOLE_SWITCH = 0x9CD9E964
    EAkEvent_WEAPONTECHSTATION_IN = 0x9D11B593
    EAkEvent_CLAMOPEN = 0x9D20C9FE
    EAkEvent_BLD_WARRIOR_SATELLITE = 0x9D2AD184
    EAkEvent_NOTIFY_BASEOUTRANGE_RETURN = 0x9D895AAE
    EAkEvent_PARTICLES_SPARKS_COMMON_STOP = 0x9D9126FC
    EAkEvent_PL_FOLEY_CLOTHING_1H_MULTITOOL_MELEE = 0x9D96CD91
    EAkEvent_QUAD_DIE = 0x9DA4A083
    EAkEvent_PLAY_PL_DRONE_01_INDEX_28_SCRAPER_PLANETROOTS_11 = 0x9DE016C7
    EAkEvent_SETTLEMENT_ALARM_SHIP = 0x9DE3B01B
    EAkEvent_DRONE_LASER = 0x9DF60645
    EAkEvent_UI_WPN_RETICULE_LOCK = 0x9E114361
    EAkEvent_NPC_FOLEY_CLOTHING_1H_INTERACT_IPAD03 = 0x9E246668
    EAkEvent_NPC_FOLEY_CLOTHING_1H_INTERACT_IPAD05 = 0x9E24666E
    EAkEvent_NPC_FOLEY_CLOTHING_1H_INTERACT_IPAD04 = 0x9E24666F
    EAkEvent_UI_FRONTEND_BACK = 0x9E331224
    EAkEvent_CLAMCLOSE = 0x9E33F27A
    EAkEvent_WPN_PL_SCAN = 0x9E35101F
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_FALLKNEESINTO = 0x9E81C443
    EAkEvent_BASE_POWER_OFF = 0x9E862FCE
    EAkEvent_MUS_STATUS_UPDATE = 0x9E8F516D
    EAkEvent_GRUNTFLOATSLEEP = 0x9ED114F6
    EAkEvent_MINIDRONE_ACTIVE = 0x9F140B13
    EAkEvent_UI_NEW_DISCOVERY = 0x9F19BAC7
    EAkEvent_UI_WPN_LOCKON = 0x9F4B5814
    EAkEvent_NPC_VOCAL_1H_ANGRY_STAND_03 = 0x9F5CFFE9
    EAkEvent_NPC_VOCAL_1H_ANGRY_STAND_04 = 0x9F5CFFEE
    EAkEvent_PLAY_PL_MUSICBOX_INDEX_08_MUSICBOXLONG_EMIN = 0x9F8539B4
    EAkEvent_QUAD_CROUCHTOSTAND = 0x9F9B4E96
    EAkEvent_BINOCULARS_ENTER = 0x9FD4BC6C
    EAkEvent_QUAD_TONE = 0x9FFA9753
    EAkEvent_VO_TUT_COORDSRECEIVED = 0xA023F16A
    EAkEvent_CRYSTAL_RARE_EXPLODE = 0xA0786AA6
    EAkEvent_TERRAIN_CREATE = 0xA095FDCF
    EAkEvent_PORTAL_JUMP_STORY_START = 0xA0AB3F9D
    EAkEvent_WPN_PL_SHOTGUN_RELOAD = 0xA0D247B6
    EAkEvent_MINIWORM_ROAR = 0xA0EA53E8
    EAkEvent_POI_GAS_BRAIN = 0xA158AFF4
    EAkEvent_DRONE_ENGINE_CORRUPTED = 0xA1D3CD87
    EAkEvent_PL_SUIT_STOP = 0xA1E440DA
    EAkEvent_COWPERFORM01 = 0xA1E54730
    EAkEvent_UI_TRADING_TEXT_ONESHOT = 0xA1F6BCE4
    EAkEvent_WPN_MELEE_IMPACT_CREATURE = 0xA2108648
    EAkEvent_UI_LAUNCH_CONFIRMATION = 0xA217EA09
    EAkEvent_UI_LEVEL_UP_TEXT = 0xA22D01BE
    EAkEvent_PL_SUIT_START = 0xA22F68E2
    EAkEvent_BRIDGETERMINAL_OPEN = 0xA26E57D5
    EAkEvent_UI_REFINERY_INCREASE = 0xA2840089
    EAkEvent_PLAY_SP_SYNTH_INDEX_24_DOMO_SPACEARP_22 = 0xA28F7AB7
    EAkEvent_ANTELOPETWOLEGROAR = 0xA2B4EF51
    EAkEvent_BASEBEACON = 0xA2D231B2
    EAkEvent_BLD_POD_DOOR_CLOSE = 0xA2DC6005
    EAkEvent_HAZ_CRIT_TOXIC = 0xA2EE4605
    EAkEvent_NEXUS_EXTERIORLEG = 0xA2F9E64D
    EAkEvent_NPC_SHIPS_WARPIN = 0xA3176202
    EAkEvent_ATLAS_ORB_FLARE = 0xA3217125
    EAkEvent_DRONE_FRIENDLY_WANTED_END = 0xA3274B4B
    EAkEvent_UI_WORD_LEARNT = 0xA3561BAF
    EAkEvent_SHIP_ROYAL_ENGINEC_LANDING = 0xA3AC816B
    EAkEvent_WEIRDFLOAT_STOP = 0xA3CC4851
    EAkEvent_ATLAS_GALAXYSTART_MULTIPLE = 0xA3DFC892
    EAkEvent_ATLAS_ARCH = 0xA4494F25
    EAkEvent_NOTIFY_HOMEPLANET = 0xA44B6BB2
    EAkEvent_PLAY_SP_HARP_INDEX_01_ASCENSION_HARP_3 = 0xA45C3554
    EAkEvent_NEXUS_SMALLDRONE_FRAME879 = 0xA45C8CFA
    EAkEvent_OBJ_STANDINGSTONE_RARE = 0xA4603971
    EAkEvent_FREIGHTER_DRONE = 0xA4898134
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_RT_01_LP = 0xA4982EFE
    EAkEvent_UI_MAP_ENTRY = 0xA4B39221
    EAkEvent_NPC_SHIPSMALL_WARPOUT = 0xA4CB6FF3
    EAkEvent_POWER_SWITCH_PROXIMITY_OFF = 0xA51D0817
    EAkEvent_DOOR_BROKEN_SPARKS = 0xA5397178
    EAkEvent_PLACEMARKER_FLARE = 0xA539AE71
    EAkEvent_PL_SHIP_MINIWARP_JUMP = 0xA54A1259
    EAkEvent_STOP_SP_ARP_HI_INDEX_15_MACREADY_4 = 0xA55C6A95
    EAkEvent_UI_BASEBUILD_SETCOLOUR = 0xA55D8932
    EAkEvent_NEXUS_SMALLDRONE_FRAME289 = 0xA56B98D9
    EAkEvent_HOVERCRAFT_HORN_START = 0xA5828922
    EAkEvent_DRAGONFLY_IMPACTS = 0xA58ADC43
    EAkEvent_PLAY_SP_DRONE_01_INDEX_05_REDPARA_SPACEPULSE_AMIN_7 = 0xA5A09195
    EAkEvent_AF_FAN = 0xA5F03712
    EAkEvent_WPN_BROKEN = 0xA5FE3EFA
    EAkEvent_PLAY_SP_ARP_HI_INDEX_08_HUSK_7 = 0xA6145F83
    EAkEvent_MINIDRONE_IDLE = 0xA651E327
    EAkEvent_CRASHED_FREIGHTER = 0xA672C68C
    EAkEvent_EXPLORERHAPPYIPAD2 = 0xA6982AB8
    EAkEvent_EXPLORERHAPPYIPAD3 = 0xA6982AB9
    EAkEvent_FOURTHRACE_PROC_ANGRY00 = 0xA69C07BF
    EAkEvent_DRONE_ATTACK_STOP = 0xA6B07F2F
    EAkEvent_PLAY_WT_DRUMS_INDEX_03_SM_WANTED_DRUMLOOPS_15 = 0xA6B6A988
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_02_ASIM_PLANETJUNO_BMIN_7 = 0xA6CE8694
    EAkEvent_MECH_JETPACK_LP_END = 0xA6FCEB2A
    EAkEvent_FIEND_BURY = 0xA732976C
    EAkEvent_PLAY_SP_SYNTH_INDEX_14_EOTWS2_SPACESTABS_5 = 0xA781CF68
    EAkEvent_UI_RETICULE_ONTARGET = 0xA784CC00
    EAkEvent_STOP_MAP_CENTRECHORD_02_INDEX_01_MAP_CENTRECHORD_02 = 0xA78A1418
    EAkEvent_EXPLODINGBARREL = 0xA7AE41A9
    EAkEvent_SHIP_SQUID_LANDINGGEAR_LANDING = 0xA7B106E1
    EAkEvent_LOWERORDERANGRYIPAD = 0xA7B5482F
    EAkEvent_OBJ_REFINERYLARGE_START = 0xA7C2D6AD
    EAkEvent_UI_PICKUP_RESOURCE = 0xA7CBF529
    EAkEvent_PLAY_VO_WARNINGDRONE_COMBAT = 0xA7DEA5CF
    EAkEvent_UI_MANAGE_EXOSUIT_INVENTORY = 0xA8202B36
    EAkEvent_SHUTTLE_CLOSE = 0xA8262171
    EAkEvent_CREATURE_BITE = 0xA83D7AAB
    EAkEvent_NPC_SHIP_ENGINE_FREIGHTER_FRONT = 0xA84C5D19
    EAkEvent_MUS_STORYMODE_MUSICCUE_12_LP_STOP = 0xA8509856
    EAkEvent_BLD_TRADER_DOOR_OPEN = 0xA876DCE2
    EAkEvent_NOTIFY_NEWASSISTANT = 0xA87F871F
    EAkEvent_MECH_SENTINEL_STEP_WALK = 0xA892E81A
    EAkEvent_WALKER_LASER = 0xA8AC2E4F
    EAkEvent_BUGGY_BOOST_START = 0xA8AD0530
    EAkEvent_PLAY_SP_SYNTHHIGH_INDEX_13_BEYOND_HIGHMELODY_14 = 0xA8B7067F
    EAkEvent_UI_MAP_NAV_PATH_BACK = 0xA8B966DE
    EAkEvent_VO_BASECOMPUTER_ONLINE = 0xA8C4CF0F
    EAkEvent_STRIDERATTACK2 = 0xA8C96B16
    EAkEvent_STRIDERATTACK3 = 0xA8C96B17
    EAkEvent_BASE_POWER_CONNECT = 0xA8DC0783
    EAkEvent_CREATURE_VOCALS_SQUAWK_ROAR_NEUTRAL = 0xA8E86F75
    EAkEvent_INTERACTION_TURRETSHELLS = 0xA8F90361
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_03_SM_PIANOHIGH_DMIN_5 = 0xA92BA318
    EAkEvent_UI_MAP_HIGHLIGHT_HOVER = 0xA9310FCE
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_RT_01_IN = 0xA9983603
    EAkEvent_FREIGHTER_HANGARDOOR_CLOSE = 0xA9AF6976
    EAkEvent_PLAY_PL_GUITAR_02_INDEX_02_ASIM_PLANET_MALLETGTR_BMIN_4 = 0xA9F582C1
    EAkEvent_UI_FRONTEND_ENTER = 0xAA0B552D
    EAkEvent_PLAY_SP_ARP_HI_INDEX_09_RIPLEY_12 = 0xAA2EFA26
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_RELAXINTO = 0xAA4B1AAC
    EAkEvent_SENTINEL_HIVE_LP = 0xAA570111
    EAkEvent_PL_FOLEY_CLOTHING_MULTITOOL_HOLSTER = 0xAA65F0D4
    EAkEvent_PL_FOLEY_CLOTHING_FALL_CONTROL = 0xAA776AEA
    EAkEvent_HAZ_CRIT_HEAT = 0xAA7866FE
    EAkEvent_NPC_SHIP_TAKEOFF = 0xAA8DD0FA
    EAkEvent_PLAY_SP_DRONE_01_INDEX_16_MONOALT_SPACEDRONE_5 = 0xAAA6DF83
    EAkEvent_DRONE_SHIELD_DEACTIVATE = 0xAABD2664
    EAkEvent_SANDWORM_LP_STOP = 0xAAC8AD74
    EAkEvent_ORBIDLE_LOOP = 0xAAD2AC93
    EAkEvent_PLAY_SP_GUITAR_INDEX_07_EOTWS_GTRCLEANNOTES_16 = 0xAAE231EC
    EAkEvent_AF_DOOR_PRESSURISING2 = 0xAB04F2E4
    EAkEvent_AF_DOOR_PRESSURISING3 = 0xAB04F2E5
    EAkEvent_AF_DOOR_PRESSURISING1 = 0xAB04F2E7
    EAkEvent_UI_BASEBUILD_REPAIRTECH = 0xAB2C9A05
    EAkEvent_BASE_DOOR_OPEN = 0xAB8E00D8
    EAkEvent_TREXROAR = 0xABA96530
    EAkEvent_TORCH_ON = 0xABE43D67
    EAkEvent_PL_VEHICLE_REMOTE_ENGINE = 0xABFB7216
    EAkEvent_QUAD_STEP = 0xAC0E650D
    EAkEvent_PLAY_WT_DRUMS_INDEX_19_TUES_WANTEDBEATS4 = 0xAC12AE00
    EAkEvent_FOURTHRACE_APOLLO_ANGRY00 = 0xAC2665AE
    EAkEvent_TURRET_SHOOT = 0xAC2966DB
    EAkEvent_GEMCRYSTAL_COLLECT = 0xAC4DFF83
    EAkEvent_CATLARGEATTACK = 0xAC68C044
    EAkEvent_UI_MAP_MENUDOWN = 0xAC6E86A6
    EAkEvent_VILESPAWN_LP = 0xAC9C927F
    EAkEvent_UI_NPC_SHIP_HIT_ALERT = 0xACC079B1
    EAkEvent_PORTAL_TERMINAL_CLOSE = 0xACE8E6E3
    EAkEvent_PLAY_PL_DRONE_02_INDEX_28_SCRAPER_PLANETGTRTEXTURE_8 = 0xACFC51A7
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY08 = 0xAD111892
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY09 = 0xAD111893
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY02 = 0xAD111898
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY03 = 0xAD111899
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY01 = 0xAD11189B
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY06 = 0xAD11189C
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY07 = 0xAD11189D
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY04 = 0xAD11189E
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_HAPPY05 = 0xAD11189F
    EAkEvent_IMPACT_CREATURE_SQUISH = 0xAD1494AB
    EAkEvent_FOURTHRACE_NULL_ANGRY00 = 0xAD16E1B0
    EAkEvent_STOP_WT_DRUMS_INDEX_15_MAC_DRUMLOOP_5 = 0xAD5677B4
    EAkEvent_HAZ_CRIT_RAD_STABLE = 0xAD62460D
    EAkEvent_PLAY_WT_DRONE_INDEX_10_TETSUO_GRAINSPACE_1 = 0xADA17A3A
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_12_ATWOOD_GTRVERB_7 = 0xADA95082
    EAkEvent_NEXUS_TELEPORTER = 0xADAB0AF1
    EAkEvent_PLAY_SP_SYNTHHIGH_INDEX_10_TETSUO_MOPHOHIGHINTEREST_1 = 0xADF95C78
    EAkEvent_GRAV_BALLS_STOP = 0xAE129547
    EAkEvent_STOP_SP_ARP_HI_INDEX_04_BLUEPRINT_ARPEMIN_128BPM_3 = 0xAE4D8FAB
    EAkEvent_WALKER_SIT = 0xAE6B8C16
    EAkEvent_WALKER_DIE = 0xAE8BE068
    EAkEvent_FOLEY_STOP_ALL = 0xAEA67841
    EAkEvent_UI_SELECT_GENERIC = 0xAEF4BC9C
    EAkEvent_PLAY_WT_GUITAR_INDEX_26_MELB_PLANETGTR_2 = 0xAEFE5FBA
    EAkEvent_ANOMALY_SIMULATION_REVEAL_OPEN = 0xAF06F8C5
    EAkEvent_FOOT_LOCKER_OPEN = 0xAF12AD4F
    EAkEvent_QUAD_IDLE = 0xAF14F599
    EAkEvent_EXPL_FIREWORK_TAIL = 0xAF543557
    EAkEvent_NOTIFY_CRITIAL_ENERGY = 0xAF6B907C
    EAkEvent_NOTIFY_PULSEOFFFUEL = 0xAFCB0F67
    EAkEvent_OBJ_MONOLITH = 0xAFF212EB
    EAkEvent_VR_TELEPORTMOVE = 0xAFFEF3EA
    EAkEvent_UI_BASEBUILD_DELETE = 0xB00017EF
    EAkEvent_UI_TRADING_MESSAGE = 0xB002EB0D
    EAkEvent_LOWERORDERGREET2IPAD = 0xB00B7183
    EAkEvent_NEXUS_BRIDGEENGINE_IDLE = 0xB01A5461
    EAkEvent_MUS_AMBIENT_MODE_STOP = 0xB0299432
    EAkEvent_PLANT_RADIOACTIVE_STEAMER = 0xB02F895A
    EAkEvent_NOTIFY_EXTERNALVIEW = 0xB03C84EF
    EAkEvent_PL_HEALTH = 0xB050FA76
    EAkEvent_WALKER_LEG_HIT_RIGHT_OUT_01 = 0xB0B8D754
    EAkEvent_WALKER_LEG_HIT_RIGHT_OUT_02 = 0xB0B8D757
    EAkEvent_PLAY_WT_ONESHOTS_INDEX_04_BLUEPRINT_JOESTUTTER_EMIN_11 = 0xB0C277CA
    EAkEvent_BASE_POWERFLOW_OFF = 0xB0E54966
    EAkEvent_UI_PIRATES_WARNING_STOP = 0xB0E8ACC8
    EAkEvent_BASE_POWER_POWERDOWN = 0xB0EC3B5A
    EAkEvent_PLAY_SP_DRONE_01_INDEX_24_DOMO_SPACEDRONE_15 = 0xB0ECE2C0
    EAkEvent_AMB_PLANET_ALL = 0xB0FEDFC0
    EAkEvent_MINIWORM_LUNGE = 0xB1195BC9
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_LOOK_L01 = 0xB11AD035
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_LOOK_L02 = 0xB11AD036
    EAkEvent_STOP_WT_DRUMS_INDEX_20_TUNG_WANTEDBEATS_3 = 0xB1534DD1
    EAkEvent_BASEBUILD_ITEM_WEAPONTERMINAL = 0xB15A99B7
    EAkEvent_AMB_NEXUS = 0xB15F1A93
    EAkEvent_PLAY_VO_WARNINGDRONE_ACTIVATED = 0xB16E06F0
    EAkEvent_MECH_JETPACK_LANDING_INTRO = 0xB183200A
    EAkEvent_NOTIFY_LIFESUPPORTFALLING = 0xB1A96BA5
    EAkEvent_EXPLORERANGRYIPAD = 0xB1BD172B
    EAkEvent_UI_VR_WRISTMENU_HIGHLIGHT = 0xB1E07F62
    EAkEvent_NOTIFY_MISSILENEUTRALISED = 0xB1E6896F
    EAkEvent_NEXUS_CONTROLROOM_HOLOGRAM = 0xB1F10DCD
    EAkEvent_NOTIFY_TRANSFERRED = 0xB1FB7B81
    EAkEvent_DRONE_FRIENDLY_DEVICE_ON = 0xB1FEF1B2
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SHUTDWN_IN = 0xB237B602
    EAkEvent_UI_MAP_EXIT = 0xB2526227
    EAkEvent_NPC_SHIP_ENGINES = 0xB264012B
    EAkEvent_NOTIFY_MISSILELOCKED = 0xB2B6FCF5
    EAkEvent_WALKER_IDLE = 0xB2C7FAE0
    EAkEvent_PLAY_PL_SYNTH_INDEX_21_OCT_SLOWMEL_25 = 0xB30EE13F
    EAkEvent_PLAY_WT_DRUMS_INDEX_11_AKIRAWANTEDBEATS3 = 0xB31FEE49
    EAkEvent_PL_SHIP_LANDING = 0xB35579F6
    EAkEvent_PLAY_WT_FX_01_INDEX_03_SM_ATMOS_DMIN_5 = 0xB37E574E
    EAkEvent_WPN_PL_OUTOFAMMO_GUN = 0xB3D2327A
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_10_TETSUO_PLANETHI_1 = 0xB4503E43
    EAkEvent_PLAY_PL_HIGHSYNTH_INDEX_08_CELESTIAL_HIGH_2 = 0xB4547CC4
    EAkEvent_DRAGONFLY_STOP = 0xB4782DD6
    EAkEvent_DRAGONFLY = 0xB48AC47B
    EAkEvent_PLAY_WT_DRUMS_INDEX_12_ATWOOD_WANTEDARPBEATS_24 = 0xB4B96ABC
    EAkEvent_PLAY_SP_DRONE_01_INDEX_22_VOSTOK_SPACETEXTURE_33 = 0xB4D09396
    EAkEvent_NOTIFY_CRITIAL_HIT = 0xB4DC4AF7
    EAkEvent_PLAY_SP_BASS_INDEX_17_APORIA_SPACEBASS_6 = 0xB4F25703
    EAkEvent_BLD_SC_OBS_IN = 0xB504D119
    EAkEvent_WPN_SHIP_INFRAKNIFE_OVERHEAT = 0xB567D842
    EAkEvent_PLAY_WT_GUITAR_INDEX_19_TUES_WANTEDGTR_3 = 0xB56B6CBD
    EAkEvent_NOTIFY_HAZ_TEMP_STABLE = 0xB56C8D2F
    EAkEvent_PL_SHIP_HASLANDED = 0xB585E053
    EAkEvent_PL_FOLEY_CLOTHING_LADDER_DISMOUNT_TOP = 0xB5A5F559
    EAkEvent_PLAY_WT_DRUMS_INDEX_08_HUSKWANTEDBEATS11 = 0xB5A776F9
    EAkEvent_EXPL_ROCK = 0xB5CC968E
    EAkEvent_MUS_STORYMODE_MUSICCUE_13_LP_STOP = 0xB5CCBF8F
    EAkEvent_ROUNDTABLE_CLOSE = 0xB601B704
    EAkEvent_CRATEM_OPEN = 0xB602DA46
    EAkEvent_PL_SHIP_TRANSITION_TO_SPACE_START = 0xB630765A
    EAkEvent_WALKER_STEP = 0xB63357EC
    EAkEvent_DRONE_MEDIC_FIX = 0xB63AAB4A
    EAkEvent_PLAY_SP_LOWATMOS_INDEX_01_ASCENSION_LONGRISE_2 = 0xB66CB2AC
    EAkEvent_UNDERWATER_STEAMVENT_VENTING = 0xB67C3BF4
    EAkEvent_NEXUS_SMALLDRONE_FRAME512 = 0xB67CFCF6
    EAkEvent_WPN_PL_JAVELIN_CHARGE_END = 0xB6B9D056
    EAkEvent_SUITBOOT_SCANNER = 0xB6C45955
    EAkEvent_DRONE_SUSPICIOUS = 0xB6CE0AD5
    EAkEvent_FRACTALCUBE = 0xB6F17CBB
    EAkEvent_PL_SHIP_THRUST_REVERSE_ONESHOTS = 0xB71468B0
    EAkEvent_UI_SETTLEMENT_BUILDINGCOMPLETE = 0xB72E6ADF
    EAkEvent_SPIDERPERFORM01 = 0xB731D638
    EAkEvent_NPC_SHIP_LAND_ROTATE_STOP = 0xB736C13C
    EAkEvent_BRIDGETERMINALSCREEN = 0xB7549556
    EAkEvent_WPN_PL_CHANGE = 0xB7746754
    EAkEvent_TELEPORT_ACTIVATED_LOOP = 0xB794E78D
    EAkEvent_PL_FOLEY_CLOTHING_LOCO = 0xB79EA60A
    EAkEvent_VEHICLE_MARKER_DELETE = 0xB7B05DC2
    EAkEvent_PLAY_PL_SYNTH_INDEX_13_BEYOND_SLOWSYNTH_6 = 0xB7D8EC9E
    EAkEvent_PLAY_WT_MELODY_INDEX_25_DASHKA_SPACEPIANO_6 = 0xB88F3749
    EAkEvent_NPC_FOLEY_CLOTHING_1H_STAND_NEUTRAL03 = 0xB895DC69
    EAkEvent_NPC_FOLEY_CLOTHING_1H_STAND_NEUTRAL04 = 0xB895DC6E
    EAkEvent_WALKER_CHARGEUP = 0xB8A1D0D9
    EAkEvent_STORMCRYSTAL = 0xB8B92596
    EAkEvent_PORTAL_EXPLOSION = 0xB8CAC8DB
    EAkEvent_EXPL_SHIPS_SMALL = 0xB8D9B746
    EAkEvent_WALKER_SQUAWKER = 0xB8DD2DCB
    EAkEvent_PLAY_WT_MELODY_INDEX_21_OCT_WANTEDMELS_4 = 0xB93D3088
    EAkEvent_PLAY_SP_BASS_INDEX_28_SCRAPER_SPACEROOTS_7 = 0xB966DBA0
    EAkEvent_BEETLE_FLY = 0xB990B31A
    EAkEvent_PLAY_MAP_PIANO_INDEX_01_MUS_MAP01_PIANO_01 = 0xB99E1F47
    EAkEvent_QUAD_GUN = 0xB9A70BFF
    EAkEvent_PLAY_WT_GUITAR_INDEX_20_TUNG_WANTEDGTRS_5 = 0xBA1CBEC9
    EAkEvent_FREIGHTER_CALL = 0xBA2BE15A
    EAkEvent_PET_UI_ADOPT = 0xBA4E41AE
    EAkEvent_NPC_VOCAL_SIT_CHATTER_02 = 0xBA70E1B8
    EAkEvent_NPC_VOCAL_SIT_CHATTER_01 = 0xBA70E1BB
    EAkEvent_STONE_DOOR1_SETTLEMENT_CLOSE = 0xBA754903
    EAkEvent_PL_FOLEY_CLOTHING_0H_SWIM_LEFT_RIGHT = 0xBA7587F5
    EAkEvent_TORNADO_STOP = 0xBAAC64C7
    EAkEvent_EGG_MACHINE_COOK = 0xBAB81BD1
    EAkEvent_MECH_IDLE = 0xBB021D37
    EAkEvent_WALKER_ENRAGE = 0xBB04EB24
    EAkEvent_BLD_SC_FACT_INT = 0xBB08AD3D
    EAkEvent_STOP_WT_DRUMS_INDEX_28_SCRAPER_WANTEDBEATS_8 = 0xBB18CA00
    EAkEvent_WPN_TERRAIN_BUILD = 0xBB4541A9
    EAkEvent_EXPL_SHIPS_LARGE = 0xBB5CA45A
    EAkEvent_PET_UI_CUSTOM_FLANK = 0xBB6990AA
    EAkEvent_NOTIFY_OBJECTIVE_COMPLETE = 0xBB7E1EA4
    EAkEvent_TENTACLE_OUT = 0xBB8CBCBE
    EAkEvent_PL_SHIP_COCKPIT_RAIN_STOP = 0xBBD86828
    EAkEvent_UI_MAP_TEXT_OUT = 0xBC076EE7
    EAkEvent_UI_FRONTEND_TEXTOVERLAY_EMPTY = 0xBC089316
    EAkEvent_ATLAS_CORE = 0xBC1CE2F8
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_23_ECHO_PLANETGTRBED_9 = 0xBC265BCF
    EAkEvent_SHIP_ROYAL_WINGSC_TAKEOFF = 0xBC3F8694
    EAkEvent_QUAD_SCAN_STOP = 0xBC4B8A4F
    EAkEvent_CAPSULE_COLLECT = 0xBC55E8B1
    EAkEvent_MAP_STAR_WOOSH = 0xBC7AB0AD
    EAkEvent_SHIP_ROYAL_WINGSA_TAKEOFF = 0xBC7EFDA2
    EAkEvent_PLAY_WT_GUITAR_INDEX_15_MAC_GTRMEL_2 = 0xBC898CEF
    EAkEvent_SHIP_PIRATE_SAILS_ROUNDSAILS_FOLD = 0xBC9C3093
    EAkEvent_DRONE_EMOTE_LP_STOP = 0xBC9CA56A
    EAkEvent_POI_GEKHEAD = 0xBCAA303B
    EAkEvent_MECH_EXIT = 0xBCC570FD
    EAkEvent_PLAY_SP_FX_01_INDEX_02_BOWEDCYMBAL_CLEAN_22 = 0xBCD640CD
    EAkEvent_POWER_SWITCH_BUTTON_ON = 0xBD66A4EE
    EAkEvent_SETTLEMENT_TERMINAL = 0xBD68511F
    EAkEvent_MOLE = 0xBD7B65F0
    EAkEvent_STOP_WT_DRUMS_INDEX_03_SM_WANTED_DRUMLOOPS_15 = 0xBD85203A
    EAkEvent_FOURTHRACE_NULL_HAPPY01 = 0xBD9D7CAC
    EAkEvent_FOURTHRACE_NULL_HAPPY00 = 0xBD9D7CAD
    EAkEvent_FOURTHRACE_NULL_HAPPY02 = 0xBD9D7CAF
    EAkEvent_WPN_NOTOOL = 0xBDA317B2
    EAkEvent_HAZ_CRIT_O2 = 0xBDA886E7
    EAkEvent_UI_OXYGEN_REPLENISHED = 0xBDCC5A4E
    EAkEvent_NOTIFY_HAZ_CRIT_COLD = 0xBDE32DB6
    EAkEvent_WPN_SHIP_OVERHEAT = 0xBDEC12CA
    EAkEvent_E3_ZOOMIN = 0xBE030510
    EAkEvent_MUS_STORYMODE_MUSICCUE_30 = 0xBE2BDD24
    EAkEvent_MUS_STORYMODE_MUSICCUE_31 = 0xBE2BDD25
    EAkEvent_MUS_STORYMODE_MUSICCUE_32 = 0xBE2BDD26
    EAkEvent_MUS_STORYMODE_MUSICCUE_33 = 0xBE2BDD27
    EAkEvent_NOTIFY_SENTINELSDEACTIVATED = 0xBE2FA622
    EAkEvent_UI_STAT_CHANGE = 0xBE30492B
    EAkEvent_COWFLOATHAPPY = 0xBE89AD0A
    EAkEvent_BASE_BIOFUELRUNNING = 0xBE89F34E
    EAkEvent_NOTIFY_MISSILEDESTROYED = 0xBE9C28A0
    EAkEvent_WPN_PL_NEUTRON_CANNON_LP = 0xBEB6F46E
    EAkEvent_WPN_PL_HANDLASER = 0xBEDA84FC
    EAkEvent_UI_MAP_WAYPOINT_ERROR = 0xBEEBD855
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_14_EOTWS_PIANOLOOP_4 = 0xBF265D1A
    EAkEvent_MUS_STORYMODE_MUSICCUE_27 = 0xBF2BDE90
    EAkEvent_MUS_STORYMODE_MUSICCUE_26 = 0xBF2BDE91
    EAkEvent_MUS_STORYMODE_MUSICCUE_25 = 0xBF2BDE92
    EAkEvent_MUS_STORYMODE_MUSICCUE_24 = 0xBF2BDE93
    EAkEvent_MUS_STORYMODE_MUSICCUE_23 = 0xBF2BDE94
    EAkEvent_MUS_STORYMODE_MUSICCUE_21 = 0xBF2BDE96
    EAkEvent_MUS_STORYMODE_MUSICCUE_20 = 0xBF2BDE97
    EAkEvent_MUS_STORYMODE_MUSICCUE_29 = 0xBF2BDE9E
    EAkEvent_MUS_STORYMODE_MUSICCUE_28 = 0xBF2BDE9F
    EAkEvent_PLAY_SP_ARP_HI_INDEX_12_ATWOOD_10 = 0xBF34FC67
    EAkEvent_JETPACK_START = 0xBF36A0FC
    EAkEvent_QUAD_HOP = 0xBF9A4960
    EAkEvent_ROVER_STOP = 0xBFA7324A
    EAkEvent_NPC_SHIPSMALL_WARPIN = 0xC0106C54
    EAkEvent_MUS_STORYMODE_MUSICCUE_18 = 0xC02BE002
    EAkEvent_MUS_STORYMODE_MUSICCUE_19 = 0xC02BE003
    EAkEvent_MUS_STORYMODE_MUSICCUE_16 = 0xC02BE00C
    EAkEvent_MUS_STORYMODE_MUSICCUE_17 = 0xC02BE00D
    EAkEvent_MUS_STORYMODE_MUSICCUE_14 = 0xC02BE00E
    EAkEvent_MUS_STORYMODE_MUSICCUE_15 = 0xC02BE00F
    EAkEvent_SPKORVAX_CHATTER_STOP = 0xC02D15AE
    EAkEvent_PLAY_WT_DRUMS_INDEX_24_DOMO_WANTEDDRUMS_3 = 0xC03ABC3C
    EAkEvent_EXPLORERIPADGREET = 0xC03F5D47
    EAkEvent_UI_NEW_TECH = 0xC0882ABB
    EAkEvent_PLAY_SP_MELODY_INDEX_08_ASCENSION_ELEKTRIK_EMIN_1 = 0xC089EA38
    EAkEvent_POI_CUBE_STATION = 0xC09FF838
    EAkEvent_NOTIFY_MISSILEENEMY = 0xC0A031A5
    EAkEvent_PLAY_PL_CELLO_INDEX_07_EOTWS_CELLO_1 = 0xC0A831E6
    EAkEvent_BASESHOP_OPEN = 0xC0D6FE45
    EAkEvent_STOP_SP_ARP_HI_INDEX_03_SUPERMOON_17 = 0xC0DE7CE0
    EAkEvent_ATLAS_SPHERE_SPARK = 0xC0F94248
    EAkEvent_PL_FOLEY_EMOTE_UNDERWATER_OK = 0xC10135E9
    EAkEvent_UI_NOWHERETOLAND = 0xC10CD558
    EAkEvent_MUS_STORYMODE_MUSICCUE_09 = 0xC12BE1F4
    EAkEvent_MUS_STORYMODE_MUSICCUE_05 = 0xC12BE1F8
    EAkEvent_MUS_STORYMODE_MUSICCUE_04 = 0xC12BE1F9
    EAkEvent_MUS_STORYMODE_MUSICCUE_07 = 0xC12BE1FA
    EAkEvent_MUS_STORYMODE_MUSICCUE_01 = 0xC12BE1FC
    EAkEvent_MUS_STORYMODE_MUSICCUE_03 = 0xC12BE1FE
    EAkEvent_MUS_STORYMODE_MUSICCUE_02 = 0xC12BE1FF
    EAkEvent_PLAY_PL_DRONE_01_INDEX_10_TETSUO_VOXTEXTURE_1 = 0xC156BA58
    EAkEvent_PLAY_PL_SYNTH_INDEX_01_CELESTIAL_MOD_1 = 0xC1740844
    EAkEvent_UI_BUY = 0xC18CADCA
    EAkEvent_PL_SHIP_HYPERSPACE_END = 0xC1AE6E4B
    EAkEvent_VO_TECH_COMP_REPAIRED = 0xC1C549DE
    EAkEvent_PLAY_SP_ARP_HI_INDEX_01_MONOLITH_SPACE_ARP_GMIN_11 = 0xC22E661B
    EAkEvent_NOTIFY_BASEOUTRANGE = 0xC25586D1
    EAkEvent_HOVERCRAFT_BOOST_STOP = 0xC256FEF0
    EAkEvent_COLLECTABLE_RECEIVED = 0xC266F931
    EAkEvent_OBJ_REFINERYMEDIUM_START = 0xC27FE2E7
    EAkEvent_STOP_WT_DRUMS_INDEX_05_REDPARA_WANTEDDRUMLOOPS_1 = 0xC295CDD1
    EAkEvent_PLAY_PL_DRONE_02_INDEX_25_SCRAPER_PLANETWEIRDARP_1 = 0xC2C438B5
    EAkEvent_SETTLEMENT_PUMP = 0xC2F73565
    EAkEvent_PLAY_WT_DRONE_INDEX_14_DEPARTURE_GTR_EMIN_1 = 0xC2FC8B7E
    EAkEvent_PLAY_WT_DRUMS_INDEX_16_MONOALT_WANTEDLOOPS_8 = 0xC300036F
    EAkEvent_VENUSFLYTRAP_OPEN = 0xC33559DD
    EAkEvent_MUS_STORYMODE_MUSICCUE_06_LP = 0xC345F572
    EAkEvent_FREIGHTER_HANGAR = 0xC3642BEB
    EAkEvent_DRONE_FRIENDLY_UNSUMMONED = 0xC39240FB
    EAkEvent_AMB_FRIGATE = 0xC397F586
    EAkEvent_WPN_PL_PISTOL = 0xC3B24F39
    EAkEvent_BRIDGETERMINAL_CLOSE = 0xC3D36AE3
    EAkEvent_SUIT_INIT_JETPACK = 0xC3E71CD0
    EAkEvent_PIRATE_STATION_GENERATOR = 0xC421C6D5
    EAkEvent_NOTIFY_PULSEOFFLOCAL = 0xC42AB9F0
    EAkEvent_MUS_COMMUNITYRESEARCH = 0xC44ABF93
    EAkEvent_TXT_STATICNOISE_END = 0xC44B58B8
    EAkEvent_PL_SURVEYING_R = 0xC44E4621
    EAkEvent_PL_SURVEYING_C = 0xC44E4630
    EAkEvent_PL_SURVEYING_L = 0xC44E463F
    EAkEvent_UI_MISSION_NOTIFYSCAN_END = 0xC46B3C49
    EAkEvent_TERMINAL_OUTDOORS = 0xC48D4C4B
    EAkEvent_PLAY_WT_PADS_INDEX_13_BEYOND_WANTEDPADS_6 = 0xC4907E3D
    EAkEvent_UI_INSUFFICIENT_STANDING = 0xC49AAA32
    EAkEvent_NOTIFY_HAZ_CAVE_HEAT = 0xC4AD486F
    EAkEvent_CREATURE_JUMPSWIPE = 0xC4B64355
    EAkEvent_PROTOROLLER_STOP = 0xC4BC74D4
    EAkEvent_WALKER_STAND = 0xC4C8E4FE
    EAkEvent_PLAY_SP_DRONE_01_INDEX_25_DASHKA_SPACEAMBIENCE_7 = 0xC4CE4289
    EAkEvent_WPN_TERRAIN_DESTROY_END = 0xC4E8D681
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_GREET02 = 0xC4F0B05D
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_GREET01 = 0xC4F0B05E
    EAkEvent_WPN_PL_SCATTERBLASTER = 0xC4F25AF5
    EAkEvent_BASE_BATTERY_FULLYDISCHARGED = 0xC50227AD
    EAkEvent_POI_EYE = 0xC5038395
    EAkEvent_OBJ_PORTABLEREFINERY_STOP = 0xC52354C1
    EAkEvent_PL_FOLEY_CLOTHING_DEATH_KNEES = 0xC5403F26
    EAkEvent_UNDERWATER_STEAMVENT_RUMBLE = 0xC54EB9A6
    EAkEvent_DRAGONFLY_HORN_STOP = 0xC55D6968
    EAkEvent_WEIRDWIRECELL = 0xC5896FBB
    EAkEvent_PL_TELEPORT_WARP_START = 0xC5955A1B
    EAkEvent_IDENTIFY_SCAN_ON = 0xC5976489
    EAkEvent_FOURTHRACE_NULL_IDLE00 = 0xC5B3E657
    EAkEvent_ATLAS_GALAXYBEAM = 0xC5B56B54
    EAkEvent_OBJ_CRATE_MED = 0xC5E4BAC7
    EAkEvent_STOP_WT_DRUMS_INDEX_18_NMS2017_WANTEDSYNTHBEATS_1 = 0xC65EED56
    EAkEvent_QUAD_LASER = 0xC667CC3C
    EAkEvent_LIGHTNING = 0xC6BFE289
    EAkEvent_NOTIFY_HAZ_CAVE_TOXIC = 0xC6F1B1A6
    EAkEvent_COOKER_OPEN = 0xC703639B
    EAkEvent_PLAY_SP_MELODY_INDEX_26_MELB_SPACEMELS_3 = 0xC71C04F9
    EAkEvent_HAZ_CRIT_TOXIC_FALLING = 0xC71CDD4D
    EAkEvent_GRAV_BALLS = 0xC72A3468
    EAkEvent_PL_FOLEY_EMOTE_UNDERWATER_THUMBSUP = 0xC732E21B
    EAkEvent_VR_THEREMIN = 0xC7432116
    EAkEvent_PL_FOLEY_CLOTHING_EMOTE_NEG_FINGERWAG = 0xC74E5AF9
    EAkEvent_UI_STANDING_DECREASED = 0xC781ADCF
    EAkEvent_EXPLORERHAPPYIPAD = 0xC786698E
    EAkEvent_DRAGONFLY_BOOST_START = 0xC7ADCCAE
    EAkEvent_EXPL_SMALL_SPACE_GENERIC = 0xC7DD7E03
    EAkEvent_UI_NEWTECH_WOOSH = 0xC814BFB5
    EAkEvent_SUBMARINE_SUSPENSION = 0xC8164665
    EAkEvent_QUAD_SCAN = 0xC82A6860
    EAkEvent_PORTAL_ACTIVATE_NORMAL = 0xC84F6011
    EAkEvent_PIPE_SMOKE_STOP = 0xC85D689A
    EAkEvent_ABANDONED_FIEND_WALK = 0xC8910DD0
    EAkEvent_ANOMALY_SIMULATION_CONSOLE_END = 0xC89FD926
    EAkEvent_BASEBUILD_ITEM_TERMINALGENERIC = 0xC8E2F31A
    EAkEvent_PL_VEHICLE_REMOTE_HORN = 0xC8F8145B
    EAkEvent_TREXPERFORM03 = 0xC8FBA2B8
    EAkEvent_PLAY_WT_DRUMS_INDEX_10_TETSUO_DRUMLOOPS_1 = 0xC919BA1D
    EAkEvent_STOP_SP_ARP_HI_INDEX_12_ATWOOD_10 = 0xC93C9B99
    EAkEvent_UI_FRONTEND_EXIT = 0xC9410141
    EAkEvent_NEXUS_FUELPUMP = 0xC949845F
    EAkEvent_UI_BASEBUILD_CHANGESELECTION = 0xC950BE50
    EAkEvent_OBJ_BARREL_MED = 0xC9953608
    EAkEvent_PLAY_PL_DRONE_01_INDEX_23_ECHO_PLANETLOWEND_15 = 0xC995CA0F
    EAkEvent_SHIP_ROYAL_WINGSB_TAKEOFF = 0xC9965D21
    EAkEvent_UI_PIRATES_WARNING = 0xC9AF9145
    EAkEvent_STOP_WT_DRUMS_INDEX_24_DOMO_WANTEDDRUMS_3 = 0xC9F1D682
    EAkEvent_PLACEMARKER_WOOSH = 0xCA202279
    EAkEvent_STOP_WT_DRUMS_INDEX_12_ATWOOD_WANTEDARPBEATS_24 = 0xCA2352DA
    EAkEvent_GROUNDWORM_KILL = 0xCA498284
    EAkEvent_UI_FRONTEND_TECHBUILD = 0xCA798E63
    EAkEvent_UI_INTERACTIVEFLORA_COLLECT = 0xCAC28EDB
    EAkEvent_NEW_TECH = 0xCAC84920
    EAkEvent_NPC_SHIPS_WARPOUT = 0xCACADD3D
    EAkEvent_PL_TELEPORT_WARP_END = 0xCAD30640
    EAkEvent_WPN_MELEE_IMPACT_GENERIC = 0xCB00CD9E
    EAkEvent_ANTELOPEROAR = 0xCB03FA5B
    EAkEvent_QUAD_POUNCE_BACK = 0xCB08B251
    EAkEvent_PL_SUIT_STARTUP_CONFIRM = 0xCB37B9FC
    EAkEvent_PL_SHIELD_LOWSWEEP = 0xCBBD44C4
    EAkEvent_VEHICLE_MARKER_COLLECT = 0xCC0C57CF
    EAkEvent_UI_PAGE_CHANGE = 0xCC61961E
    EAkEvent_NORMANDY_BEACON = 0xCC905DCE
    EAkEvent_STOP_SP_ARP_HI_INDEX_09_RIPLEY_12 = 0xCCAC66C8
    EAkEvent_PLAY_VO_WARNINGDRONE_ENGAGED = 0xCCDF400E
    EAkEvent_WALKER_LASER_STOP = 0xCCF69C7A
    EAkEvent_LOWERORDERHAPPY3IPAD = 0xCCFEEDD5
    EAkEvent_ROVER_IDLE_EXTERIOR = 0xCD1A7599
    EAkEvent_PLAY_WT_MELODY_INDEX_12_ATWOOD_CRACKLE_5 = 0xCD321398
    EAkEvent_TIMBER_DOOR2_SETTLEMENT_CLOSE = 0xCD38E81A
    EAkEvent_MAP_WAYPOINT_TONE_LP = 0xCD4C651B
    EAkEvent_WPN_PL_GRENADE_BOUNCE = 0xCD5D244D
    EAkEvent_PL_SHIP_BOOST_START = 0xCDB230F3
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_11_AKIRA_PLANETPIANO_1 = 0xCDB93632
    EAkEvent_CRYOCHAMBER = 0xCDD3E924
    EAkEvent_FREIGHTER_TELEPORTER = 0xCDD40E2A
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_SOFA_OUTOF = 0xCE067C73
    EAkEvent_PLAY_WT_BASS_INDEX_02_SKANNERLOW_EMIN_1 = 0xCE67C6FA
    EAkEvent_WPN_PL_RAILGUN_RELOAD = 0xCE68FDE6
    EAkEvent_TENTACLE_LP = 0xCE73F7BE
    EAkEvent_NPC_SHIP_POLICE_SIREN = 0xCEA4E032
    EAkEvent_BASE_FORCEFIELD_DOOR = 0xCEB18629
    EAkEvent_PILGRIM_STOP = 0xCEC2C09C
    EAkEvent_WEAPONTECHSTATION_OUT = 0xCED353AE
    EAkEvent_EXPL_ELECTRICAL_STUN = 0xCEDE2742
    EAkEvent_PLANT_GATHER_PULPYROOTS = 0xCEF5472C
    EAkEvent_BASE_SOLARPANEL_CLOSE = 0xCF003B2B
    EAkEvent_WOOSH_DROID_SMALL_01_MID = 0xCF0FF131
    EAkEvent_PS5_VIBRATION_TONE = 0xCF5E1783
    EAkEvent_HOLOHUB_END = 0xCF84178C
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_01_MONOLITH_PLANET_GTRSOUNDSCAPE_5 = 0xCF8CADD5
    EAkEvent_METEORITE_INCOMING = 0xCFBF792E
    EAkEvent_NOTIFY_MISSILEENEMYINBOUND = 0xCFCA9E0C
    EAkEvent_NOTIFY_PULSEOFFSTATION = 0xCFF19999
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_ANGRY_LP02 = 0xD00508BC
    EAkEvent_NPC_FOLEY_CLOTHING_0H_STAND_ANGRY_LP01 = 0xD00508BF
    EAkEvent_WPN_PL_NEUTRON_CANNON_BUILDUP = 0xD02F55E1
    EAkEvent_SUITBOOT_JETPACK = 0xD03EEDFD
    EAkEvent_UI_SPECIALS_INSUFFICIENT = 0xD05421AC
    EAkEvent_DRAGONFLY_JUMP = 0xD07A3EA6
    EAkEvent_PLAY_SP_SYNTH_INDEX_11_AKIRA_SPACESYNTH_9 = 0xD0A18574
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_13_LULL_GTR_16 = 0xD0ACF4C1
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_18_NMS2017_MODMELS_2 = 0xD0BF75D2
    EAkEvent_PL_HAZARD_WARNING = 0xD0D988E1
    EAkEvent_UI_VR_HANDMENU_OPENLP = 0xD0DCFE1E
    EAkEvent_SUBMARINE_SURFACE = 0xD10F00A7
    EAkEvent_PORTAL_WEIRD_BUILDUP = 0xD11EC5BF
    EAkEvent_ANIMATEDDOOR_CLOSE = 0xD135F2D3
    EAkEvent_HARRYTEST = 0xD13E408D
    EAkEvent_FOURTHRACE_APOLLO_HAPPY02 = 0xD14ED305
    EAkEvent_FOURTHRACE_APOLLO_HAPPY01 = 0xD14ED306
    EAkEvent_FOURTHRACE_APOLLO_HAPPY00 = 0xD14ED307
    EAkEvent_MUS_MILESTONE_COMPLETE = 0xD15B7779
    EAkEvent_UI_SHOP_SWITCH_INVENTORY = 0xD1746824
    EAkEvent_TURRET_LASER = 0xD1A672EB
    EAkEvent_PL_SUIT_STARTUP_INTERACT_TIMER_STOP = 0xD1B12CD3
    EAkEvent_NOTIFY_THRUSTERSOFFLARGESHIP = 0xD1B3E833
    EAkEvent_SIXLEGCATPERFORM01 = 0xD1CA7463
    EAkEvent_STOP_WT_DRUMS_INDEX_22_VOSTOK_WANTEDDRUMS_2 = 0xD1F2B5C0
    EAkEvent_POD_SHAKE = 0xD1F704ED
    EAkEvent_PLAY_WT_DRUMS_INDEX_02_ASIM_DRUMSDIRTY_128BPM_6 = 0xD1FD662A
    EAkEvent_TRIROAR = 0xD1FE084C
    EAkEvent_SPACE_STATION_INTERIOR_01 = 0xD20E04AB
    EAkEvent_ROVER_BOOST_START = 0xD22AAF8A
    EAkEvent_UI_BADGE = 0xD22C90A7
    EAkEvent_OBJ_HEALTHSTATION_ON = 0xD2519969
    EAkEvent_STOP_WT_DRUMS_INDEX_27_OVERSEER_WANTEDDRUMS_6 = 0xD262EB32
    EAkEvent_OBJ_MINING_GAS_HARVESTER = 0xD278D30E
    EAkEvent_PLAYER_WATER_TRANSITION_INTO_ATMOS = 0xD2A5E6C2
    EAkEvent_UI_RETICULE_LOCK = 0xD2C5A1B9
    EAkEvent_MUS_ATLASSTATION = 0xD2C903D2
    EAkEvent_UI_HEALTH_INCREASE = 0xD2D6DFE3
    EAkEvent_STOP_SP_ARP_HI_INDEX_13_ARPBEYOND_A9 = 0xD3038103
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_01_MONLITH_PLANET_PIANOMELVERB_1 = 0xD3074360
    EAkEvent_PLAY_WT_DRONE_INDEX_04_DEPARTURE_NOISE_EMIN_5 = 0xD33A4CFA
    EAkEvent_WPN_PL_HANDLASER_OVERHEAT = 0xD348E403
    EAkEvent_TENTACLE_IN = 0xD373FFC3
    EAkEvent_NOTIFY_HIT_COLD = 0xD38A3CED
    EAkEvent_UI_RECORD_UPLOADED = 0xD3A1A28C
    EAkEvent_ROUNDTABLE_OPEN = 0xD3D76000
    EAkEvent_NPC_SHIP_LAND_IMPACT = 0xD3FDB23A
    EAkEvent_POI_BONES = 0xD43CA593
    EAkEvent_SENTINEL_MECH_JUMP = 0xD469C69E
    EAkEvent_COWHAPPYHIND = 0xD48DDDFF
    EAkEvent_WPN_SHIP_MISSILE_OVERHEAT = 0xD49513E9
    EAkEvent_FOURTHRACE_NULL_CHATTER00 = 0xD4C3C08C
    EAkEvent_PLAY_SP_SYNTH_INDEX_22_VOSTOK_ECHOREV_7 = 0xD4D22C6C
    EAkEvent_DRONE_SCAN_START = 0xD4DA690A
    EAkEvent_UI_SETTLEMENT_FLOORCOMPLETE = 0xD4E70FC1
    EAkEvent_PLAY_SP_BASS_INDEX_25_DASHKA_SPACEBASS_3 = 0xD4FA04F5
    EAkEvent_AMB_RAIN_SURFACE = 0xD5115048
    EAkEvent_PL_RUN_FAIL = 0xD51DCF4E
    EAkEvent_COMMS_CHATTER_POLICE = 0xD547E7BB
    EAkEvent_LOWERORDERGREET3IPAD = 0xD559CDA2
    EAkEvent_HEATER_OFF = 0xD57B9408
    EAkEvent_EGG_MACHINE_FINISH = 0xD5B01638
    EAkEvent_PLAY_WT_DRUMS_INDEX_04_BLUEPRINT_DRMSGLITCHYBEATS_128BPM_5 = 0xD5CC957C
    EAkEvent_STOP_WT_DRUMS_INDEX_04_BLUEPRINT_DRMSGLITCHYBEATS_128BPM_5 = 0xD5DE6FAE
    EAkEvent_DRONE_EMOTE_LP = 0xD5FDCCDF
    EAkEvent_HOLOHUB_START = 0xD60EC7EF
    EAkEvent_TERRAIN_DESTROY = 0xD6350E47
    EAkEvent_PL_FOLEY_CLOTHING_2H_MULTITOOL_MELEE = 0xD63657DC
    EAkEvent_ABANDONED_FIEND_ROAR = 0xD6403FEF
    EAkEvent_NOTIFY_SENTINELSTARSHIPS = 0xD6463F36
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_BASIC02 = 0xD6467658
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_BASIC03 = 0xD6467659
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_BASIC01 = 0xD646765B
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_BASIC04 = 0xD646765E
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_BASIC05 = 0xD646765F
    EAkEvent_COWHINDPERFORM01 = 0xD64E7029
    EAkEvent_DRONE_ARMOURED_UNBRACE = 0xD65D31C4
    EAkEvent_PLAY_MAP_CENTRECHORD_01_INDEX_01_MAP_CENTRECHORD_01 = 0xD6A75CD2
    EAkEvent_PLAY_SP_SYNTH_INDEX_08_ASCENSION_SYNTHSTAB_9 = 0xD6A9E64B
    EAkEvent_PLAY_PL_VOX_INDEX_03_SM_VOX_DMIN_3 = 0xD6ACAB19
    EAkEvent_ANOMALY_SIMULATION_TERMINAL_DIE = 0xD6AD5022
    EAkEvent_STOP_MAP_SINGLES_01_INDEX_01_MUS_MAP01_SINGLES1_04 = 0xD6E366A7
    EAkEvent_PLAY_SP_BASS_INDEX_02_SKANNERLOW_EMIN_4 = 0xD6FCD897
    EAkEvent_VO_TECH_REPAIRED = 0xD718EFE6
    EAkEvent_MECH_MOVE_RUN = 0xD7C5239C
    EAkEvent_GRABBYPLANT_DIE = 0xD7FAF53C
    EAkEvent_NPC_SHIP_SCAN_PULSE = 0xD80B1123
    EAkEvent_ANTELOPEATTACK3 = 0xD81B03CA
    EAkEvent_SHUTTLE_OPEN = 0xD82737F3
    EAkEvent_PL_FOLEY_EMOTE_DANCELOOP = 0xD832CAD2
    EAkEvent_METEORITE_EXPLODE = 0xD846F01D
    EAkEvent_EXPL_FIREWORK_SLIME = 0xD86955AF
    EAkEvent_POWERMODULE_NOTE_F_STOP = 0xD89D1D4F
    EAkEvent_UI_FRONTEND_SELECTGENERIC = 0xD8AF6F74
    EAkEvent_UI_MISSION_NEWSECONDARY_COMPLETE = 0xD8AFAE1F
    EAkEvent_PLAY_WT_DRONE_INDEX_07_EOTWS_GRAINSPACE_1 = 0xD8E07776
    EAkEvent_PLAY_WT_ORGAN_INDEX_06_EV_ORGANWANTED_AMIN_4 = 0xD8EA60FF
    EAkEvent_STOP_MAP_SINGLES_02_INDEX_01_MUS_MAP01_SINGLES2_09 = 0xD92015F6
    EAkEvent_PL_FOLEY_CLOTHING_1H_RELOAD = 0xD9219578
    EAkEvent_PL_FOLEY_SLOPE_SLIDE_START = 0xD922A3D6
    EAkEvent_NOTIFY_HAZ_NIGHT_COLD = 0xD9846BCE
    EAkEvent_WPN_NOTECHTOOL = 0xD9985CA2
    EAkEvent_NOTIFY_MISSILEINCOMINGDESTROYED = 0xD9BC2ADC
    EAkEvent_NEXUS_CONTROLROOM_CENTRE = 0xD9E873E9
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_13_BEYOND_HIGHMELODY_12 = 0xD9F7F0CD
    EAkEvent_VO_SECURE_TRANSMISSIONMODULE_LOCATED = 0xD9F965C8
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_HAPPY02 = 0xDA2DA234
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_HAPPY01 = 0xDA2DA237
    EAkEvent_PLAY_WT_ARP_INDEX_23_ECHO_WANTEDCELESTA_3 = 0xDA34E957
    EAkEvent_PL_SHIP_THRUST_ONESHOTS = 0xDA41B521
    EAkEvent_BASE_LIGHTBOX = 0xDA4C4B9A
    EAkEvent_BLD_TRADER_RADIOTOWEREFFECT = 0xDA81C2AC
    EAkEvent_PLAY_SP_MELODY_INDEX_23_ECHO_SPACETHUD_19 = 0xDA845B1A
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_15_HAUNTED_SHARKUPCLOSE_EMIN_7 = 0xDAB6CF40
    EAkEvent_INTERACTION_TURRETTURN = 0xDB15CA55
    EAkEvent_WALKER_LEG_HIT_RIGHT_IN_02 = 0xDB3B90BC
    EAkEvent_WALKER_LEG_HIT_RIGHT_IN_01 = 0xDB3B90BF
    EAkEvent_VILESPAWN_EGG_COLLECT = 0xDB837279
    EAkEvent_VR_FOLEY_ARM_MOVEMENTS_LEFT = 0xDB88808F
    EAkEvent_MUS_LEVEL_UP = 0xDBB97803
    EAkEvent_PLAY_PL_DRONE_01_INDEX_27_OVERSEER_PLANETACOUSTICBED_5 = 0xDBCF152F
    EAkEvent_WEATHER_WARN = 0xDBD65A16
    EAkEvent_SETTLEMENT_ANTENNA0 = 0xDBD71864
    EAkEvent_SETTLEMENT_ANTENNA1 = 0xDBD71865
    EAkEvent_SETTLEMENT_ANTENNA2 = 0xDBD71866
    EAkEvent_RECIPE_RECEIVE = 0xDBD7DDFD
    EAkEvent_NPC_WPN_FREIGHTER_GUN = 0xDBF6ADA4
    EAkEvent_STOP_WT_DRUMS_INDEX_17_APORIA_WANTEDBEATS_2 = 0xDC40C4C8
    EAkEvent_NPC_FOLEY_CLOTHING_0H_IDLE_LOOKUP01 = 0xDC60245B
    EAkEvent_WARRIORHONOUR = 0xDC70021E
    EAkEvent_TELEPORT_LOOP_STOP = 0xDCD2E4FC
    EAkEvent_FREIGHTER_TELEPORT = 0xDCD9243F
    EAkEvent_NPC_WPN_SMALLSHIP_LASER = 0xDD05AA14
    EAkEvent_UI_VR_WRISTMENU_OUT = 0xDD62551A
    EAkEvent_BASEBUILD_ITEM_SIGNALBOOSTER_START = 0xDD70498C
    EAkEvent_TELEPORT_END = 0xDDE9F36E
    EAkEvent_PLAY_PL_DRONE_01_INDEX_03_SM_VOX_GRANULAR_DMIN_1 = 0xDE1DF727
    EAkEvent_MECH_JETPACK_LP = 0xDE5305B4
    EAkEvent_COOKER_CLOSE = 0xDE567DC9
    EAkEvent_GRUNTATTACK3 = 0xDE8A7D60
    EAkEvent_GRUNTATTACK2 = 0xDE8A7D61
    EAkEvent_PILGRIM_HORN_STOP = 0xDE95EC9A
    EAkEvent_PLAY_SP_MELODY_INDEX_19_TUES_MUSICBOX_12 = 0xDED04AC9
    EAkEvent_PLAY_SP_BASS_INDEX_01_SPARKGROWLER_EMIN_4 = 0xDF4601B1
    EAkEvent_UI_SENTINELFORCES_ALERTED = 0xDF61DB92
    EAkEvent_BASE_CALCISHROOM = 0xDFC60F7F
    EAkEvent_COMMS_CHATTER_FREIGHTER = 0xDFEA2C37
    EAkEvent_MECH_STEP_RUN = 0xE008E6CD
    EAkEvent_BASEBUILD_ITEM_BEACON = 0xE011D583
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_22_VOSTOK_PLANETHIGH_7 = 0xE059651B
    EAkEvent_ORBIDLE = 0xE0625692
    EAkEvent_PLAY_PL_FX_01_INDEX_17_APORIA_PLANET_ROBOT_14 = 0xE0D1741E
    EAkEvent_NOTIFY_VEHICLESUMMONED = 0xE0F39F69
    EAkEvent_LAVA_VOLCANO_ERUPT = 0xE111872B
    EAkEvent_UI_PROTECTION_OFFLINE = 0xE12D76D3
    EAkEvent_UI_BASEBUILD_REPAIRTECH_ERROR = 0xE1303E62
    EAkEvent_UI_TIP = 0xE15FC005
    EAkEvent_MINIDRONE_SHOOT = 0xE1A418B6
    EAkEvent_EXPLORERCHATTER = 0xE1C4E07B
    EAkEvent_CUSTOMISE_ACTIVATE = 0xE1CCDB77
    EAkEvent_DRONE_FRIENDLY_COMBAT = 0xE1F7777C
    EAkEvent_PL_PROTECTION_CHARGE = 0xE20B8CE4
    EAkEvent_PL_FOLEY_SLOPE_SLIDE_STOP = 0xE2280BD6
    EAkEvent_CHARACTEREDITOR_SELECT = 0xE22A1DC0
    EAkEvent_COMPUTER_TEXT_TYPE_STOP = 0xE271EAEE
    EAkEvent_PL_FOLEY_CLOTHING_FALL_PANIC = 0xE28BA57C
    EAkEvent_PL_SHIP_SCANNED = 0xE2A36147
    EAkEvent_NOTIFY_UNITS_RECEIVED = 0xE2A736C0
    EAkEvent_PL_VEHICLE_REMOTE_START = 0xE2B75F44
    EAkEvent_SHIP_SQUID_NOSE_TAKEOFF = 0xE2EA8853
    EAkEvent_CRYSTALALTAR_LOOP = 0xE2EAC53C
    EAkEvent_COWROAR = 0xE2F1E4D6
    EAkEvent_POLICE_INCOMING = 0xE2FDAA5A
    EAkEvent_NOTIFY_HOSTILESYSTEMWEAPONS = 0xE3138F29
    EAkEvent_NOTIFY_VEHICLE = 0xE31EB0A7
    EAkEvent_ANOMALY_WEAPONCRATE_CLOSE = 0xE35624E5
    EAkEvent_PORTAL_JUMP_NORMAL_START = 0xE3B61E91
    EAkEvent_UI_TRADING_TEXT_STOP = 0xE3C5B820
    EAkEvent_NEW_WEAPON = 0xE3FA979C
    EAkEvent_BASE_BIOFUELCHARGED = 0xE43B51CF
    EAkEvent_NPC_VOCAL_NEUTRAL_STAND_01 = 0xE4463E89
    EAkEvent_NPC_VOCAL_NEUTRAL_STAND_02 = 0xE4463E8A
    EAkEvent_MUS_GAMESTART_LUSHWALK = 0xE4582A4F
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_22_VOSTOK_BOWEDGTR_1 = 0xE45DFE19
    EAkEvent_NPC_SPEECH_CHATTER = 0xE46F6261
    EAkEvent_PARTICLES_ELECTRICAL_SPARKS_STOP = 0xE475E6EF
    EAkEvent_FORFCHATTER01 = 0xE497504E
    EAkEvent_NPC_TALK_STOP = 0xE4A1A5E6
    EAkEvent_NOTIFY_NEWASSISTANTACQUIRED = 0xE4B6193F
    EAkEvent_UI_MAP_WAYPOINT_ADDED = 0xE4BFD309
    EAkEvent_NOTIFY_ENERGY_PERCENT = 0xE4CE6907
    EAkEvent_SHIP_SQUID_LANDINGGEAR_TAKEOFF = 0xE4DA94B8
    EAkEvent_PLAY_PL_GUITARDRONE_INDEX_01_DEPARTURE_GTR_EMIN_1 = 0xE4ED3E1B
    EAkEvent_SANDWORM_OUTOFGROUND = 0xE4FE7575
    EAkEvent_BLD_DAMAGEDMACH_BEAM = 0xE5821724
    EAkEvent_NOTIFY_INTERNALVIEW = 0xE5A5363D
    EAkEvent_SUIT_UPGRADE_TERMINAL_LP = 0xE5E4EDD9
    EAkEvent_NOTIFY_HOSTILESYSTEM = 0xE60E20FE
    EAkEvent_SUIT_INIT_SCANNER = 0xE637BC54
    EAkEvent_SPACE_STATION_INTERIOR_01_STOP = 0xE63C38E6
    EAkEvent_UI_VR_HANDMENU_HIGHLIGHT = 0xE672686E
    EAkEvent_DRONE_SUMMONER_SIGNAL = 0xE67CDB17
    EAkEvent_JETPACK_BOOST_STOP = 0xE6CC9004
    EAkEvent_PILGRIM_JUMP = 0xE6D853F8
    EAkEvent_WALKER_EMOTE_LP_STOP = 0xE77997D8
    EAkEvent_ANOMALY_DOOR_CLOSE = 0xE7C7565A
    EAkEvent_NOTIFY_CRITICAL_TOXIC = 0xE80C3762
    EAkEvent_FIEND_GROUNDEFFECT = 0xE816A4C6
    EAkEvent_INTERACTIVECASE_OPEN = 0xE81E1242
    EAkEvent_UI_ALERT_DAMAGE_WARNING_CRITICAL_LP_STOP = 0xE832F3D5
    EAkEvent_BLD_SCIENTIFIC_DOOR_OPEN = 0xE8572281
    EAkEvent_PLAY_PL_PIANO_INDEX_24_DOMO_PLANETBURN_3 = 0xE874446A
    EAkEvent_PLAY_WT_BASS_INDEX_20_TUNG_PLANET_PIANOLOWINT2 = 0xE893B9AA
    EAkEvent_PLAY_SP_SYNTH_INDEX_01_ASCENSION_SYNTHSTAB_6 = 0xE8943E79
    EAkEvent_NOTIFY_FUELREQUIRED = 0xE89842EA
    EAkEvent_PL_SHIP_ENGINES_STOP = 0xE8A32D5D
    EAkEvent_PLAY_WT_ARP_INDEX_05_REDPARA_SPACEARP_AMIN_3 = 0xE8ABFEA4
    EAkEvent_PLAY_SP_GUITAR_INDEX_09_RIPLEY_SPACEGTRHI_1 = 0xE8C36DF6
    EAkEvent_WPN_PL_SCAN_STOP = 0xE8E06EAA
    EAkEvent_PLAY_SP_DRONE_01_INDEX_04_BLUEPRINT_MOPHODRONE_EMIN_4 = 0xE91F0CFF
    EAkEvent_NOTIFY_HIT_HEAT = 0xE92FA0E9
    EAkEvent_CAPSULE_PANEL_OPEN = 0xE95C7888
    EAkEvent_WPN_SHIP_POSITRON_FIRE = 0xE9733AD7
    EAkEvent_PLAY_WT_DRONE_INDEX_09_SM_ATMOS_DMIN_1 = 0xE9CDD90E
    EAkEvent_WOOSH_DROID_SMALL_01_REAR = 0xEA2BF77F
    EAkEvent_PLAY_PL_HIGHSYNTH_INDEX_20_TUNG_PLANET_PIANOHIGHINT_21 = 0xEA6A0F56
    EAkEvent_WPN_PL_PISTOL_RELOAD = 0xEA835E0B
    EAkEvent_AMB_WEATHER_THUNDER = 0xEA8B35F9
    EAkEvent_STOP_WT_DRUMS_INDEX_08_HUSKWANTEDBEATS11 = 0xEA903FC3
    EAkEvent_EXPLORERIPADGREET4 = 0xEABFD6F1
    EAkEvent_EXPLORERIPADGREET3 = 0xEABFD6F6
    EAkEvent_EXPLORERIPADGREET2 = 0xEABFD6F7
    EAkEvent_NOTIFY_NEW_SYSTEM = 0xEB09F91B
    EAkEvent_OBJ_WAYPOINT = 0xEB0C2542
    EAkEvent_MUS_AMBIENT_MODE = 0xEB3C1A97
    EAkEvent_UI_RARE_ITEM = 0xEB5C5104
    EAkEvent_DEATHSHOUT = 0xEBA2C6C4
    EAkEvent_NPC_FOOTSTEPS_GENERIC = 0xEBEBBF24
    EAkEvent_NPC_VOCAL_HAPPY_STAND_08 = 0xEBF77875
    EAkEvent_NPC_VOCAL_HAPPY_STAND_05 = 0xEBF77878
    EAkEvent_NPC_VOCAL_HAPPY_STAND_04 = 0xEBF77879
    EAkEvent_NPC_VOCAL_HAPPY_STAND_07 = 0xEBF7787A
    EAkEvent_NPC_VOCAL_HAPPY_STAND_06 = 0xEBF7787B
    EAkEvent_NPC_VOCAL_HAPPY_STAND_01 = 0xEBF7787C
    EAkEvent_NPC_VOCAL_HAPPY_STAND_03 = 0xEBF7787E
    EAkEvent_NPC_VOCAL_HAPPY_STAND_02 = 0xEBF7787F
    EAkEvent_ROVER_START = 0xEC117B92
    EAkEvent_NOTIFY_WARNING = 0xEC145583
    EAkEvent_PLAY_PL_DRONE_01_INDEX_20_TUNG_PLANET_TEXTURE_2 = 0xEC21FBBE
    EAkEvent_AMB_SPACESTATION_HANGER = 0xEC3F2892
    EAkEvent_BASE_CYCLONIC_LATHE = 0xEC4E25A8
    EAkEvent_UI_SERVER_SCANDISCOVERIES = 0xECE09C45
    EAkEvent_MOTION_PS5_STOP = 0xECE193FD
    EAkEvent_SENTINEL_MECH_WPN_GRENADE = 0xECF59048
    EAkEvent_RODENTPOUNCE = 0xED010011
    EAkEvent_PLAY_PL_DRONE_02_INDEX_14_EOTWSALT_STRINGLOOP_3 = 0xED13C371
    EAkEvent_UI_BASEBUILD_ROTATE_LOOP_STOP = 0xED2E9987
    EAkEvent_UI_SETTLEMENT_DECISION = 0xED34CBC8
    EAkEvent_BLOBATTACK2 = 0xED3690C6
    EAkEvent_BLOBATTACK3 = 0xED3690C7
    EAkEvent_UI_BASEBUILD_ROTATE_LOOP_START = 0xED43B465
    EAkEvent_DRAGONFLY_START = 0xED4829D6
    EAkEvent_BASEBUILD_ITEM_PLANTINCUBATOR = 0xED58F63F
    EAkEvent_PLAY_WT_DRUMS_INDEX_20_TUNG_WANTEDBEATS_3 = 0xED622083
    EAkEvent_PIRATE_STATIONCORE = 0xED62DD76
    EAkEvent_SFX_FADEDOWN = 0xED6AB3EB
    EAkEvent_UI_PLACEMARKER = 0xED754EB1
    EAkEvent_BASE_POWER_SWITCH_ON = 0xED8BA271
    EAkEvent_PL_VEHICLE_REMOTE_STOP = 0xED8F65F8
    EAkEvent_PLAY_WT_DRONE_INDEX_24_DOMO_WANTEDBACKGROUND_2 = 0xEDA0C79F
    EAkEvent_WARRIORGREET1 = 0xEDB7C38D
    EAkEvent_WARRIORGREET2 = 0xEDB7C38E
    EAkEvent_WARRIORGREET3 = 0xEDB7C38F
    EAkEvent_WPN_SHIP_OUTOFAMMO = 0xEDB8EC03
    EAkEvent_WPN_PL_RAILGUN = 0xEDBB26D2
    EAkEvent_TENTPLANT_OPEN = 0xEDC2EF44
    EAkEvent_PILGRIM_HORN_START = 0xEDE0C7A2
    EAkEvent_PLACEMARKER_LP_STOP = 0xEDF59752
    EAkEvent_UI_GAMEMODE_HOVER = 0xEE0CC5FE
    EAkEvent_STING_LOGO = 0xEE2BDB9E
    EAkEvent_UI_TEXTENTRY = 0xEE38EFB9
    EAkEvent_DRONE_FRIENDLY_DEVICE_OFF = 0xEE5687A8
    EAkEvent_FOURTHRACE_APOLLO_GREET02 = 0xEE5A2A20
    EAkEvent_FOURTHRACE_APOLLO_GREET03 = 0xEE5A2A21
    EAkEvent_FOURTHRACE_APOLLO_GREET00 = 0xEE5A2A22
    EAkEvent_FOURTHRACE_APOLLO_GREET01 = 0xEE5A2A23
    EAkEvent_FOURTHRACE_APOLLO_GREET04 = 0xEE5A2A26
    EAkEvent_UI_NANITES_RECEIVED = 0xEE872C40
    EAkEvent_NOTIFY_CRITICAL_HEAT = 0xEE9FF4E3
    EAkEvent_PLAY_SP_BASS_INDEX_20_TUNG_SPACEBASS2 = 0xEEDCEB62
    EAkEvent_TUT_ELEMENT_TRANSFER_SUCCESS = 0xEEF3B67D
    EAkEvent_NOTIFY_FUELDEPLETED = 0xEF3000F4
    EAkEvent_INTERACTION_CARGOCONTLOAD = 0xEF521ED6
    EAkEvent_HOLOHUB_LADDER_START = 0xEF70C184
    EAkEvent_UI_SHOP_SWITCH_BUYSELL = 0xEF95C71C
    EAkEvent_UI_BASEBUILD_ROTATE = 0xEF9C456B
    EAkEvent_PLAY_PL_HIGHSYNTH_INDEX_27_OVERSEER_PLANETHIGHINTEREST_5 = 0xEF9D0195
    EAkEvent_UI_SERVER_FOUND = 0xF00BB46A
    EAkEvent_PLAY_SP_FX_01_INDEX_14_EOTWS2_SPACECHIME_1 = 0xF0244729
    EAkEvent_OBJ_REFINERYMEDIUM_IDLE = 0xF037F80F
    EAkEvent_WPN_PL_JAVELIN_CHARGE = 0xF04467B0
    EAkEvent_SHIP_TOUCHDOWN_IMPACT = 0xF0555EF0
    EAkEvent_SPIDERFLOATSAD = 0xF0824E34
    EAkEvent_WPN_PL_GRENADE = 0xF099543E
    EAkEvent_HOVERCRAFT_BOOST_START = 0xF116021C
    EAkEvent_PLAY_SP_HARP_INDEX_08_ASCENSION_HARP_4 = 0xF145DEC0
    EAkEvent_WEIRDSHARDS = 0xF151439F
    EAkEvent_PLAY_WT_SYNTH_INDEX_20_TUNG_SPACEMODAL_2 = 0xF17FFB5A
    EAkEvent_NPC_CLOTHING_1H_SITTING_INTERACT_IPAD02 = 0xF18063B0
    EAkEvent_NPC_CLOTHING_1H_SITTING_INTERACT_IPAD01 = 0xF18063B3
    EAkEvent_NOTIFY_HAZ_CAVE_RADIATION = 0xF18F48E6
    EAkEvent_ROLLER = 0xF1A7A7AD
    EAkEvent_UI_SCAN_PORTAL = 0xF1AA86CE
    EAkEvent_POI_SPINNER = 0xF1CA2105
    EAkEvent_DRONE_SUMMONER_ALARM = 0xF1CE63E2
    EAkEvent_VO_TECH_INSTALLED = 0xF1E0AD04
    EAkEvent_UI_BASEBUILD_INVALIDPOSITION = 0xF1FDD4EA
    EAkEvent_PLAY_SP_ARP_HI_INDEX_02_ASIMOVARP_A1 = 0xF21C52D1
    EAkEvent_BASE_SOLARPANEL_OPEN = 0xF241066D
    EAkEvent_BURNING_BARREL = 0xF24D4A35
    EAkEvent_COWATTACK2 = 0xF26FF8FC
    EAkEvent_COWATTACK3 = 0xF26FF8FD
    EAkEvent_COWATTACK1 = 0xF26FF8FF
    EAkEvent_PLAY_MAP_DRONENOISE_01_INDEX_01_MAP_SPACE_NOISE_DRONE = 0xF270DBF0
    EAkEvent_BUGGY_HORN_START = 0xF277A3F6
    EAkEvent_TRIPERFORM01 = 0xF294FEC6
    EAkEvent_UI_MISSIONPRIMARY_UPDATE = 0xF2CE94C6
    EAkEvent_PLAY_WT_GUITAR_INDEX_05_REDPARA_GTRTREM_9 = 0xF30ED157
    EAkEvent_NOTIFY_HAZ_CAVE_COLD = 0xF31B087B
    EAkEvent_ENGINEREACTOR_OFF_IDLE = 0xF322160A
    EAkEvent_PLAY_PL_PIANO_INDEX_22_VOSTOK_PLANETKEYS_19 = 0xF34CEC83
    EAkEvent_STARSHIP_OUTFITTING_LP = 0xF353E7CE
    EAkEvent_ANOMALY_DOOR_OPEN = 0xF376E95E
    EAkEvent_SPACESTATION_ATLAS_DOOR_CLOSE = 0xF3BCBD6B
    EAkEvent_POI_HORROR = 0xF3D9CFF2
    EAkEvent_KEYCONTAINER = 0xF4218B47
    EAkEvent_PLAY_WT_ONESHOTS_INDEX_20_TUNG_SPACEHIGHINTEREST_16 = 0xF42D762D
    EAkEvent_NOTIFY_EXTREMEHAZARDPLANET = 0xF43765ED
    EAkEvent_PL_VEHICLE_REMOTE_DESTRUCTIBLE = 0xF466485A
    EAkEvent_PLAY_PL_DRONE_01_INDEX_21_OCT_TEXTURE_39 = 0xF466613E
    EAkEvent_ABANDONED_FIEND_RUN = 0xF4780FEE
    EAkEvent_NOTIFY_ADDITIONALRESOURCES = 0xF4860355
    EAkEvent_UI_WPN_TARGET_FOCUS = 0xF4A8A5B0
    EAkEvent_UI_TERRAIN_CHANGESIZE = 0xF4C6C733
    EAkEvent_PL_FOLEY_CLOTHING_LADDER_MOUNT_BOTTOM = 0xF4ED6D9D
    EAkEvent_UI_BIOSHIP_ADDORGAN = 0xF5166DC5
    EAkEvent_NOTIFY_INV_TRANSFERRED = 0xF5199CCD
    EAkEvent_PLAY_WT_BASS_INDEX_03_SM_WANTED_BASSSEQ_118BPM_DMIN_2 = 0xF54EDD74
    EAkEvent_EQUIP_DAMAGED = 0xF56C53BB
    EAkEvent_BASE_DOOR_CLOSE = 0xF5C07B4C
    EAkEvent_UI_SENTINELDETECTOR = 0xF6022E3C
    EAkEvent_HAZ_CRIT_THERM_FALLING = 0xF60D19A6
    EAkEvent_PL_FOLEY_EMOTE_NEEDRESOURCE = 0xF62E41F3
    EAkEvent_FREIGHTER_HANGARDOOR_OPEN = 0xF6307F22
    EAkEvent_HOVERCRAFT_IMPACTS = 0xF638247D
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_RT_04 = 0xF65FDE5A
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INT_RT_03 = 0xF65FDE5D
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_13_LULL_PIANO_8 = 0xF678F5FC
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_06 = 0xF6916F70
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_07 = 0xF6916F71
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_04 = 0xF6916F72
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_05 = 0xF6916F73
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_02 = 0xF6916F74
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_03 = 0xF6916F75
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_01 = 0xF6916F77
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_08 = 0xF6916F7E
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_09 = 0xF6916F7F
    EAkEvent_WEATHER_OVER = 0xF6B6BE20
    EAkEvent_WARRIORCHATTER = 0xF6BE1370
    EAkEvent_CHAT = 0xF6C8069B
    EAkEvent_PLAY_WT_MELODY_INDEX_24_DOMO_WANTEDMELS_1 = 0xF6EADAF3
    EAkEvent_NOTIFY_CRITICAL_NOOXYGEN = 0xF718BE58
    EAkEvent_PLAY_PL_GUITAR_01_INDEX_06_EV_PLANETGTR_5 = 0xF7207851
    EAkEvent_PLAY_PL_PIANO_INDEX_11_AKIRA_PLANETPIANOREV_18 = 0xF72C407C
    EAkEvent_BLOBATTACK = 0xF73A93BC
    EAkEvent_NPC_SHIP_SPACESTATION_FLYIN = 0xF746B237
    EAkEvent_BASEBUILD_ITEM_SIGNALBOOSTER_LOOP = 0xF746FF22
    EAkEvent_ANTELOPETWOLEGATTACK = 0xF74CC245
    EAkEvent_TERMINAL_ABANDONED_GROWTH = 0xF753A4AA
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_11 = 0xF79170E8
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_10 = 0xF79170E9
    EAkEvent_NPC_FOLEY_CLOTHING_0H_INTER_LISTEN_12 = 0xF79170EB
    EAkEvent_UI_QUICKMENU_NOCHARGE = 0xF7957664
    EAkEvent_SENTINEL_MECH_WPN_GUN = 0xF7A08AA0
    EAkEvent_NOTIFY_HAZ_NIGHT_HEAT = 0xF7B71686
    EAkEvent_NADA_CHATTER_13 = 0xF7BB5BF4
    EAkEvent_NADA_CHATTER_12 = 0xF7BB5BF5
    EAkEvent_NADA_CHATTER_11 = 0xF7BB5BF6
    EAkEvent_SPOREBAG_IDLE = 0xF7CA9B97
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_12_ATWOOD_SPACEHIGHINTEREST_2 = 0xF7DCC916
    EAkEvent_CRATELOCKOPEN = 0xF7DED48F
    EAkEvent_PLAY_PL_HIGHSYNTH_INDEX_19_TUES_PLANETHIGH_2 = 0xF7E7F471
    EAkEvent_QUAD_EVADE = 0xF7EBDFCA
    EAkEvent_NOTIFY_LANDINGSEQUENCE = 0xF8167905
    EAkEvent_PORTAL_DIAL_SPIN_STOP = 0xF831E3E6
    EAkEvent_DISCOVERY_UPLOADED = 0xF8D5FE88
    EAkEvent_PLAY_PL_FX_01_INDEX_25_DASKA_PLANETTHUD = 0xF8EB165F
    EAkEvent_WPN_PL_DEPLETED = 0xF90065F9
    EAkEvent_ASTEROID_EXPLODE = 0xF91DD0FC
    EAkEvent_PL_VEHICLE_REMOTE_IMPACTS = 0xF9B16951
    EAkEvent_VR_FOLEY_HAND_CLENCH_OPEN = 0xF9C86F7A
    EAkEvent_QUAD_EMOTE_LP_STOP = 0xF9C8B07F
    EAkEvent_NOTIFY_NEW_WEAPON = 0xF9D4781C
    EAkEvent_UI_INCOMING_FRIGATE_LIVING_WARNING = 0xF9E01984
    EAkEvent_COOKER_END = 0xF9F58DD6
    EAkEvent_SIXLEGGEDCOWPERFORM01 = 0xFA1323E2
    EAkEvent_ROVER_IMPACTS = 0xFA34F30F
    EAkEvent_FIEND_SPIT = 0xFA3ABC76
    EAkEvent_FREIGHTER_DOOR_INTERNAL_CLOSE = 0xFA499897
    EAkEvent_NPC_SHIP_LAND_ROTATE = 0xFA6D3689
    EAkEvent_STOP_WT_DRUMS_INDEX_26_MELB_WANTEDBEATS_6 = 0xFA930376
    EAkEvent_UI_MAP_INFO = 0xFA9400A7
    EAkEvent_PLAY_SP_SYNTH_INDEX_19_TUES_JUNO_6 = 0xFA97E7FE
    EAkEvent_NPC_WPN_SMALLSHIP_LASER_STOP = 0xFAA48C13
    EAkEvent_PLAY_SP_SYNTH_INDEX_23_ECHO_SPACEPOLY_7 = 0xFAB5FB24
    EAkEvent_PL_FOLEY_CLOTHING_0H_SWIM_FORWARD = 0xFABD31D4
    EAkEvent_WEIRDFLOAT_LP = 0xFAD2C455
    EAkEvent_VR_FOLEY_ARM_MOVEMENTS = 0xFAE20B59
    EAkEvent_ATLASINTERACT_CLOSE = 0xFAED1D89
    EAkEvent_WEIRDBEAMSTONE_STOP = 0xFB01B69F
    EAkEvent_VO_ANOMALYDETECTED = 0xFB08E100
    EAkEvent_PL_SHIP_VR_EXIT_STOP = 0xFB5202B7
    EAkEvent_UI_EPIC_ITEM = 0xFB7EBFED
    EAkEvent_PLAY_WT_GUITAR_INDEX_18_NMS2017_EBOWGTR_5 = 0xFB8B7983
    EAkEvent_NPC_SHIP_POLICE_SIREN_STOP = 0xFB949B91
    EAkEvent_UI_VR_WRISTMENU_IN = 0xFC42EEA7
    EAkEvent_SUBMARINE_STOP = 0xFC46C37A
    EAkEvent_PL_SHIP_IDLE = 0xFC4A8773
    EAkEvent_MUS_ENCLAVE_DISCOVER = 0xFC4C03AD
    EAkEvent_BLD_AMB_DEBRISLARGE = 0xFC66BDC7
    EAkEvent_PLAY_PL_MELODY_HI_INDEX_25_DASKA_PLANETHIGHINTEREST_12 = 0xFC6719FE
    EAkEvent_PARTICLES_ELECTRICAL_SPARKS = 0xFC91F6C0
    EAkEvent_UI_VEGETABLE_COLLECT = 0xFC94B9A4
    EAkEvent_PLAY_SP_MELODY_HI_INDEX_06_EV_SPACEPIANO_3_REV = 0xFCE3BF92
    EAkEvent_STOP_WT_DRUMS_INDEX_23_ECHO_WANTEDLOOPS_2 = 0xFD2B3514
    EAkEvent_NOTIFY_PULSE = 0xFDA5D118
    EAkEvent_UI_WHALESONG = 0xFDC0FA36
    EAkEvent_UI_QUICKMENU_OPEN = 0xFDCE559D
    EAkEvent_RODENTROAR = 0xFDD30E03
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_NEUTRAL01 = 0xFDD53340
    EAkEvent_NPC_FOLEY_CLOTHING_0H_SIT_NEUTRAL02 = 0xFDD53343
    EAkEvent_PL_FOLEY_CLOTHING_LADDER_MOUNT_TOP = 0xFE176015
    EAkEvent_COMPUTER_TEXT_STOP = 0xFE197CAB
    EAkEvent_WPN_SHIP_READY = 0xFE1BE6F7
    EAkEvent_PLAY_WT_GUITAR_INDEX_04_BLUEPRINT_GTRBUILD_EMIN_9 = 0xFE2DAC64
    EAkEvent_ASTEROID_HIT = 0xFE51061C
    EAkEvent_INTERACTION_MAINTPOINT = 0xFE58FBEF
    EAkEvent_PLAY_SP_SYNTH_INDEX_21_OCT_SPACEKEYS_14 = 0xFE626DB9
    EAkEvent_NOTIFY_MISSILESDESTROYED = 0xFEA1EB33
    EAkEvent_PET_UI_PAT = 0xFEAFB237
    EAkEvent_COWATTACK2HIND = 0xFED92B2F
    EAkEvent_ANTPOUNCE02 = 0xFEF4701C
    EAkEvent_ANTPOUNCE03 = 0xFEF4701D
    EAkEvent_POI_WEIRD_METAL = 0xFF2F4C51
    EAkEvent_UI_MENU_OPTION_SWITCH = 0xFF2FBCA4
    EAkEvent_BASEBUILD_ITEM_COMMUNICATIONSSTATION_ACCEPT = 0xFF644EEF
    EAkEvent_PLAY_SP_MISC_HI_INDEX_23_ECHO_WANTEDCELESTA_3 = 0xFF89DCAA
    EAkEvent_PLAY_WT_DRUMS_INDEX_09_RIPLEY_DRUMLOOPS_120BPM_1 = 0xFFA06FAD
    EAkEvent_FOOTSTEP_FALLLAND = 0xFFA89F82
    EAkEvent_NOTIFY_THRUSTERSOFFSTATION = 0xFFAE5BB8
    EAkEvent_GRUNTFLOATHAPPY = 0xFFBDD045
    EAkEvent_ATLASINTERACT_OPEN = 0xFFC5D75B

class eStatsType(IntEnum):
    EStatsType_Unspecified = 0x0
    EStatsType_Weapon_Laser = 0x1
    EStatsType_Weapon_Laser_Damage = 0x2
    EStatsType_Weapon_Laser_Mining_Speed = 0x3
    EStatsType_Weapon_Laser_HeatTime = 0x4
    EStatsType_Weapon_Laser_Bounce = 0x5
    EStatsType_Weapon_Laser_ReloadTime = 0x6
    EStatsType_Weapon_Laser_Recoil = 0x7
    EStatsType_Weapon_Laser_Drain = 0x8
    EStatsType_Weapon_Laser_StrongLaser = 0x9
    EStatsType_Weapon_Laser_ChargeTime = 0xA
    EStatsType_Weapon_Laser_MiningBonus = 0xB
    EStatsType_Weapon_Projectile = 0xC
    EStatsType_Weapon_Projectile_Damage = 0xD
    EStatsType_Weapon_Projectile_Range = 0xE
    EStatsType_Weapon_Projectile_Rate = 0xF
    EStatsType_Weapon_Projectile_ClipSize = 0x10
    EStatsType_Weapon_Projectile_ReloadTime = 0x11
    EStatsType_Weapon_Projectile_Recoil = 0x12
    EStatsType_Weapon_Projectile_Bounce = 0x13
    EStatsType_Weapon_Projectile_Homing = 0x14
    EStatsType_Weapon_Projectile_Dispersion = 0x15
    EStatsType_Weapon_Projectile_BulletsPerShot = 0x16
    EStatsType_Weapon_Projectile_MinimumCharge = 0x17
    EStatsType_Weapon_Projectile_MaximumCharge = 0x18
    EStatsType_Weapon_Projectile_BurstCap = 0x19
    EStatsType_Weapon_Projectile_BurstCooldown = 0x1A
    EStatsType_Weapon_ChargedProjectile = 0x1B
    EStatsType_Weapon_ChargedProjectile_ChargeTime = 0x1C
    EStatsType_Weapon_ChargedProjectile_CooldownDuration = 0x1D
    EStatsType_Weapon_ChargedProjectile_Drain = 0x1E
    EStatsType_Weapon_ChargedProjectile_ExtraSpeed = 0x1F
    EStatsType_Weapon_Rail = 0x20
    EStatsType_Weapon_Shotgun = 0x21
    EStatsType_Weapon_Burst = 0x22
    EStatsType_Weapon_Flame = 0x23
    EStatsType_Weapon_Cannon = 0x24
    EStatsType_Weapon_Grenade = 0x25
    EStatsType_Weapon_Grenade_Damage = 0x26
    EStatsType_Weapon_Grenade_Radius = 0x27
    EStatsType_Weapon_Grenade_Speed = 0x28
    EStatsType_Weapon_Grenade_Bounce = 0x29
    EStatsType_Weapon_Grenade_Homing = 0x2A
    EStatsType_Weapon_Grenade_Clusterbomb = 0x2B
    EStatsType_Weapon_TerrainEdit = 0x2C
    EStatsType_Weapon_SunLaser = 0x2D
    EStatsType_Weapon_SoulLaser = 0x2E
    EStatsType_Weapon_MineGrenade = 0x2F
    EStatsType_Weapon_FrontShield = 0x30
    EStatsType_Weapon_Scope = 0x31
    EStatsType_Weapon_Spawner = 0x32
    EStatsType_Weapon_SpawnerAlt = 0x33
    EStatsType_Weapon_Melee = 0x34
    EStatsType_Weapon_StunGrenade = 0x35
    EStatsType_Weapon_Stealth = 0x36
    EStatsType_Weapon_Scan = 0x37
    EStatsType_Weapon_Scan_Radius = 0x38
    EStatsType_Weapon_Scan_Recharge_Time = 0x39
    EStatsType_Weapon_Scan_Types = 0x3A
    EStatsType_Weapon_Scan_Binoculars = 0x3B
    EStatsType_Weapon_Scan_Discovery_Creature = 0x3C
    EStatsType_Weapon_Scan_Discovery_Flora = 0x3D
    EStatsType_Weapon_Scan_Discovery_Mineral = 0x3E
    EStatsType_Weapon_Scan_Secondary = 0x3F
    EStatsType_Weapon_Scan_Terrain_Resource = 0x40
    EStatsType_Weapon_Scan_Surveying = 0x41
    EStatsType_Weapon_Stun = 0x42
    EStatsType_Weapon_Stun_Duration = 0x43
    EStatsType_Weapon_Stun_Damage_Multiplier = 0x44
    EStatsType_Weapon_FireDOT = 0x45
    EStatsType_Weapon_FireDOT_Duration = 0x46
    EStatsType_Weapon_FireDOT_DPS = 0x47
    EStatsType_Weapon_FireDOT_Damage_Multiplier = 0x48
    EStatsType_Suit_Armour_Health = 0x49
    EStatsType_Suit_Armour_Shield = 0x4A
    EStatsType_Suit_Armour_Shield_Strength = 0x4B
    EStatsType_Suit_Energy = 0x4C
    EStatsType_Suit_Energy_Regen = 0x4D
    EStatsType_Suit_Protection = 0x4E
    EStatsType_Suit_Protection_Cold = 0x4F
    EStatsType_Suit_Protection_Heat = 0x50
    EStatsType_Suit_Protection_Toxic = 0x51
    EStatsType_Suit_Protection_Radiation = 0x52
    EStatsType_Suit_Underwater = 0x53
    EStatsType_Suit_UnderwaterLifeSupport = 0x54
    EStatsType_Suit_DamageReduce_Cold = 0x55
    EStatsType_Suit_DamageReduce_Heat = 0x56
    EStatsType_Suit_DamageReduce_Toxic = 0x57
    EStatsType_Suit_DamageReduce_Radiation = 0x58
    EStatsType_Suit_Protection_HeatDrain = 0x59
    EStatsType_Suit_Protection_ColdDrain = 0x5A
    EStatsType_Suit_Protection_ToxDrain = 0x5B
    EStatsType_Suit_Protection_RadDrain = 0x5C
    EStatsType_Suit_Protection_WaterDrain = 0x5D
    EStatsType_Suit_Stamina_Strength = 0x5E
    EStatsType_Suit_Stamina_Speed = 0x5F
    EStatsType_Suit_Stamina_Recovery = 0x60
    EStatsType_Suit_Jetpack = 0x61
    EStatsType_Suit_Jetpack_Tank = 0x62
    EStatsType_Suit_Jetpack_Drain = 0x63
    EStatsType_Suit_Jetpack_Refill = 0x64
    EStatsType_Suit_Jetpack_Ignition = 0x65
    EStatsType_Suit_Jetpack_DoubleJump = 0x66
    EStatsType_Suit_Jetpack_WaterEfficiency = 0x67
    EStatsType_Suit_Jetpack_MidairRefill = 0x68
    EStatsType_Suit_Refiner = 0x69
    EStatsType_Suit_AutoTranslator = 0x6A
    EStatsType_Suit_Utility = 0x6B
    EStatsType_Suit_RocketLocker = 0x6C
    EStatsType_Ship_Weapons_Guns = 0x6D
    EStatsType_Ship_Weapons_Guns_Damage = 0x6E
    EStatsType_Ship_Weapons_Guns_Rate = 0x6F
    EStatsType_Ship_Weapons_Guns_HeatTime = 0x70
    EStatsType_Ship_Weapons_Guns_CoolTime = 0x71
    EStatsType_Ship_Weapons_Guns_Scale = 0x72
    EStatsType_Ship_Weapons_Guns_BulletsPerShot = 0x73
    EStatsType_Ship_Weapons_Guns_Dispersion = 0x74
    EStatsType_Ship_Weapons_Guns_Range = 0x75
    EStatsType_Ship_Weapons_Guns_Damage_Radius = 0x76
    EStatsType_Ship_Weapons_Lasers = 0x77
    EStatsType_Ship_Weapons_Lasers_Damage = 0x78
    EStatsType_Ship_Weapons_Lasers_HeatTime = 0x79
    EStatsType_Ship_Weapons_Missiles = 0x7A
    EStatsType_Ship_Weapons_Missiles_NumPerShot = 0x7B
    EStatsType_Ship_Weapons_Missiles_Speed = 0x7C
    EStatsType_Ship_Weapons_Missiles_Damage = 0x7D
    EStatsType_Ship_Weapons_Missiles_Size = 0x7E
    EStatsType_Ship_Weapons_Shotgun = 0x7F
    EStatsType_Ship_Weapons_MiniGun = 0x80
    EStatsType_Ship_Weapons_Plasma = 0x81
    EStatsType_Ship_Weapons_Rockets = 0x82
    EStatsType_Ship_Weapons_ShieldLeech = 0x83
    EStatsType_Ship_Armour_Shield = 0x84
    EStatsType_Ship_Armour_Shield_Strength = 0x85
    EStatsType_Ship_Armour_Health = 0x86
    EStatsType_Ship_Scan = 0x87
    EStatsType_Ship_Scan_EconomyFilter = 0x88
    EStatsType_Ship_Scan_ConflictFilter = 0x89
    EStatsType_Ship_Hyperdrive = 0x8A
    EStatsType_Ship_Hyperdrive_JumpDistance = 0x8B
    EStatsType_Ship_Hyperdrive_JumpsPerCell = 0x8C
    EStatsType_Ship_Hyperdrive_QuickWarp = 0x8D
    EStatsType_Ship_Launcher = 0x8E
    EStatsType_Ship_Launcher_TakeOffCost = 0x8F
    EStatsType_Ship_Launcher_AutoCharge = 0x90
    EStatsType_Ship_PulseDrive = 0x91
    EStatsType_Ship_PulseDrive_MiniJumpFuelSpending = 0x92
    EStatsType_Ship_PulseDrive_MiniJumpSpeed = 0x93
    EStatsType_Ship_Boost = 0x94
    EStatsType_Ship_Maneuverability = 0x95
    EStatsType_Ship_BoostManeuverability = 0x96
    EStatsType_Ship_LifeSupport = 0x97
    EStatsType_Ship_Drift = 0x98
    EStatsType_Ship_Teleport = 0x99
    EStatsType_Ship_CargoShield = 0x9A
    EStatsType_Freighter_Hyperdrive = 0x9B
    EStatsType_Freighter_Hyperdrive_JumpDistance = 0x9C
    EStatsType_Freighter_Hyperdrive_JumpsPerCell = 0x9D
    EStatsType_Freighter_MegaWarp = 0x9E
    EStatsType_Freighter_Teleport = 0x9F
    EStatsType_Freighter_Fleet_Boost = 0xA0
    EStatsType_Freighter_Fleet_Speed = 0xA1
    EStatsType_Freighter_Fleet_Fuel = 0xA2
    EStatsType_Freighter_Fleet_Combat = 0xA3
    EStatsType_Freighter_Fleet_Trade = 0xA4
    EStatsType_Freighter_Fleet_Explore = 0xA5
    EStatsType_Freighter_Fleet_Mine = 0xA6
    EStatsType_Vehicle_Boost = 0xA7
    EStatsType_Vehicle_Engine = 0xA8
    EStatsType_Vehicle_Scan = 0xA9
    EStatsType_Vehicle_EngineFuelUse = 0xAA
    EStatsType_Vehicle_EngineTopSpeed = 0xAB
    EStatsType_Vehicle_BoostSpeed = 0xAC
    EStatsType_Vehicle_BoostTanks = 0xAD
    EStatsType_Vehicle_Grip = 0xAE
    EStatsType_Vehicle_SkidGrip = 0xAF
    EStatsType_Vehicle_SubBoostSpeed = 0xB0
    EStatsType_Vehicle_Laser = 0xB1
    EStatsType_Vehicle_LaserDamage = 0xB2
    EStatsType_Vehicle_LaserHeatTime = 0xB3
    EStatsType_Vehicle_LaserStrongLaser = 0xB4
    EStatsType_Vehicle_Gun = 0xB5
    EStatsType_Vehicle_GunDamage = 0xB6
    EStatsType_Vehicle_GunHeatTime = 0xB7
    EStatsType_Vehicle_GunRate = 0xB8
    EStatsType_Vehicle_StunGun = 0xB9
    EStatsType_Vehicle_TerrainEdit = 0xBA
    EStatsType_Vehicle_FuelRegen = 0xBB
    EStatsType_Vehicle_AutoPilot = 0xBC
    EStatsType_NumTypes = 0xBD

class eVehicleWeaponMode(IntEnum):
    EVehicleWeaponMode_Laser = 0x0
    EVehicleWeaponMode_Gun = 0x1
    EVehicleWeaponMode_TerrainEdit = 0x2
    EVehicleWeaponMode_StunGun = 0x3
    EVehicleWeaponMode_NumTypes = 0x4

class eWeaponClass(IntEnum):
    EWeaponClass_None = 0x0
    EWeaponClass_Projectile = 0x1
    EWeaponClass_ChargedProjectile = 0x2
    EWeaponClass_Laser = 0x3
    EWeaponClass_Grenade = 0x4
    EWeaponClass_Utility = 0x5
    EWeaponClass_TerrainEditor = 0x6
    EWeaponClass_Spawner = 0x7
    EWeaponClass_SpawnerAlt = 0x8
    EWeaponClass_NumTypes = 0x9

class eCurrency(IntEnum):
    ECurrency_Units = 0x0
    ECurrency_Nanites = 0x1
    ECurrency_Specials = 0x2
    ECurrency_NumTypes = 0x3
    ECurrency_Unspecified = 0xFFFFFFFF

class eTypeOfCost(IntEnum):
    ETypeOfCost_Currency = 0x0
    ETypeOfCost_Substance = 0x1
    ETypeOfCost_Product = 0x2
    ETypeOfCost_NumTypes = 0x3

class eVehicleType(IntEnum):
    EVehicleType_Buggy = 0x0
    EVehicleType_Bike = 0x1
    EVehicleType_Truck = 0x2
    EVehicleType_WheeledBike = 0x3
    EVehicleType_Hovercraft = 0x4
    EVehicleType_Submarine = 0x5
    EVehicleType_Mech = 0x6
    EVehicleType_NumTypes = 0x7

class eDeathConsequencesDifficulty(IntEnum):
    EDeathConsequencesDifficulty_None = 0x0
    EDeathConsequencesDifficulty_ItemGrave = 0x1
    EDeathConsequencesDifficulty_DestroyItems = 0x2
    EDeathConsequencesDifficulty_DestroySave = 0x3
    EDeathConsequencesDifficulty_NumTypes = 0x4

class eRemoteWeaponType(IntEnum):
    ERemoteWeaponType_Laser = 0x0
    ERemoteWeaponType_VehicleLaser = 0x1
    ERemoteWeaponType_AIMechLaser = 0x2
    ERemoteWeaponType_ShipLaser = 0x3
    ERemoteWeaponType_ShipLaser2 = 0x4
    ERemoteWeaponType_RailLaser = 0x5
    ERemoteWeaponType_NumLasers = 0x6
    ERemoteWeaponType_BoltCaster = 0x7
    ERemoteWeaponType_Shotgun = 0x8
    ERemoteWeaponType_Cannon = 0x9
    ERemoteWeaponType_Burst = 0xA
    ERemoteWeaponType_MineGrenade = 0xB
    ERemoteWeaponType_BounceGrenade = 0xC
    ERemoteWeaponType_StunGrenade = 0xD
    ERemoteWeaponType_VehicleCanon = 0xE
    ERemoteWeaponType_AIMechCanon = 0xF
    ERemoteWeaponType_ShipPhoton = 0x10
    ERemoteWeaponType_ShipShotgun = 0x11
    ERemoteWeaponType_ShipMinigun = 0x12
    ERemoteWeaponType_ShipPlasma = 0x13
    ERemoteWeaponType_ShipRocket = 0x14
    ERemoteWeaponType_None = 0x15
    ERemoteWeaponType_NumTypes = 0x16

class eDifficultyPresetType(IntEnum):
    EDifficultyPresetType_Invalid = 0x0
    EDifficultyPresetType_Custom = 0x1
    EDifficultyPresetType_Normal = 0x2
    EDifficultyPresetType_Creative = 0x3
    EDifficultyPresetType_Relaxed = 0x4
    EDifficultyPresetType_Survival = 0x5
    EDifficultyPresetType_Permadeath = 0x6
    EDifficultyPresetType_NumTypes = 0x7

class eCheckpointType(IntEnum):
    ECheckpointType_Checkpoint = 0x0
    ECheckpointType_Start = 0x1
    ECheckpointType_NumTypes = 0x2

class eRaceType(IntEnum):
    ERaceType_Vehicle = 0x0
    ERaceType_Spaceship = 0x1
    ERaceType_NumTypes = 0x2

class eDamageReceivedDifficulty(IntEnum):
    EDamageReceivedDifficulty_None = 0x0
    EDamageReceivedDifficulty_Low = 0x1
    EDamageReceivedDifficulty_Normal = 0x2
    EDamageReceivedDifficulty_High = 0x3
    EDamageReceivedDifficulty_NumTypes = 0x4

class eDamageGivenDifficulty(IntEnum):
    EDamageGivenDifficulty_High = 0x0
    EDamageGivenDifficulty_Normal = 0x1
    EDamageGivenDifficulty_Low = 0x2
    EDamageGivenDifficulty_NumTypes = 0x3

class eActiveSurvivalBarsDifficulty(IntEnum):
    EActiveSurvivalBarsDifficulty_None = 0x0
    EActiveSurvivalBarsDifficulty_HealthOnly = 0x1
    EActiveSurvivalBarsDifficulty_HealthAndHazard = 0x2
    EActiveSurvivalBarsDifficulty_All = 0x3
    EActiveSurvivalBarsDifficulty_NumTypes = 0x4

class eHazardDrainDifficulty(IntEnum):
    EHazardDrainDifficulty_Slow = 0x0
    EHazardDrainDifficulty_Normal = 0x1
    EHazardDrainDifficulty_Fast = 0x2
    EHazardDrainDifficulty_NumTypes = 0x3

class eEnergyDrainDifficulty(IntEnum):
    EEnergyDrainDifficulty_Slow = 0x0
    EEnergyDrainDifficulty_Normal = 0x1
    EEnergyDrainDifficulty_Fast = 0x2
    EEnergyDrainDifficulty_NumTypes = 0x3

class eSubstanceCollectionDifficulty(IntEnum):
    ESubstanceCollectionDifficulty_High = 0x0
    ESubstanceCollectionDifficulty_Normal = 0x1
    ESubstanceCollectionDifficulty_Low = 0x2
    ESubstanceCollectionDifficulty_NumTypes = 0x3

class eWeaponMode(IntEnum):
    EWeaponMode_Bolt = 0x0
    EWeaponMode_Shotgun = 0x1
    EWeaponMode_Burst = 0x2
    EWeaponMode_Rail = 0x3
    EWeaponMode_Cannon = 0x4
    EWeaponMode_Laser = 0x5
    EWeaponMode_Grenade = 0x6
    EWeaponMode_MineGrenade = 0x7
    EWeaponMode_Scope = 0x8
    EWeaponMode_FrontShield = 0x9
    EWeaponMode_Melee = 0xA
    EWeaponMode_TerrainEdit = 0xB
    EWeaponMode_SunLaser = 0xC
    EWeaponMode_Spawner = 0xD
    EWeaponMode_SpawnerAlt = 0xE
    EWeaponMode_SoulLaser = 0xF
    EWeaponMode_Flamethrower = 0x10
    EWeaponMode_StunGrenade = 0x11
    EWeaponMode_Stealth = 0x12
    EWeaponMode_NumTypes = 0x13

class eInventoryStackLimitsDifficulty(IntEnum):
    EInventoryStackLimitsDifficulty_High = 0x0
    EInventoryStackLimitsDifficulty_Normal = 0x1
    EInventoryStackLimitsDifficulty_Low = 0x2
    EInventoryStackLimitsDifficulty_NumTypes = 0x3

class eChargingRequirementsDifficulty(IntEnum):
    EChargingRequirementsDifficulty_None = 0x0
    EChargingRequirementsDifficulty_Low = 0x1
    EChargingRequirementsDifficulty_Normal = 0x2
    EChargingRequirementsDifficulty_High = 0x3
    EChargingRequirementsDifficulty_NumTypes = 0x4

class eFuelUseDifficulty(IntEnum):
    EFuelUseDifficulty_Free = 0x0
    EFuelUseDifficulty_Cheap = 0x1
    EFuelUseDifficulty_Normal = 0x2
    EFuelUseDifficulty_Expensive = 0x3
    EFuelUseDifficulty_NumTypes = 0x4

class eLaunchFuelCostDifficulty(IntEnum):
    ELaunchFuelCostDifficulty_Free = 0x0
    ELaunchFuelCostDifficulty_Low = 0x1
    ELaunchFuelCostDifficulty_Normal = 0x2
    ELaunchFuelCostDifficulty_High = 0x3
    ELaunchFuelCostDifficulty_NumTypes = 0x4

class eCurrencyCostDifficulty(IntEnum):
    ECurrencyCostDifficulty_Free = 0x0
    ECurrencyCostDifficulty_Cheap = 0x1
    ECurrencyCostDifficulty_Normal = 0x2
    ECurrencyCostDifficulty_Expensive = 0x3
    ECurrencyCostDifficulty_NumTypes = 0x4

class eItemShopAvailabilityDifficulty(IntEnum):
    EItemShopAvailabilityDifficulty_High = 0x0
    EItemShopAvailabilityDifficulty_Normal = 0x1
    EItemShopAvailabilityDifficulty_Low = 0x2
    EItemShopAvailabilityDifficulty_NumTypes = 0x3

class eBreakTechOnDamageProbability(IntEnum):
    EBreakTechOnDamageProbability_None = 0x0
    EBreakTechOnDamageProbability_Low = 0x1
    EBreakTechOnDamageProbability_High = 0x2
    EBreakTechOnDamageProbability_NumTypes = 0x3

class eScannerRechargeDifficulty(IntEnum):
    EScannerRechargeDifficulty_VeryFast = 0x0
    EScannerRechargeDifficulty_Fast = 0x1
    EScannerRechargeDifficulty_Normal = 0x2
    EScannerRechargeDifficulty_Slow = 0x3
    EScannerRechargeDifficulty_NumTypes = 0x4

class eReputationGainDifficulty(IntEnum):
    EReputationGainDifficulty_VeryFast = 0x0
    EReputationGainDifficulty_Fast = 0x1
    EReputationGainDifficulty_Normal = 0x2
    EReputationGainDifficulty_Slow = 0x3
    EReputationGainDifficulty_NumTypes = 0x4

class eCreatureHostilityDifficulty(IntEnum):
    ECreatureHostilityDifficulty_NeverAttack = 0x0
    ECreatureHostilityDifficulty_AttackIfProvoked = 0x1
    ECreatureHostilityDifficulty_FullEcosystem = 0x2
    ECreatureHostilityDifficulty_NumTypes = 0x3

class eCombatTimerDifficultyOption(IntEnum):
    ECombatTimerDifficultyOption_Off = 0x0
    ECombatTimerDifficultyOption_Slow = 0x1
    ECombatTimerDifficultyOption_Normal = 0x2
    ECombatTimerDifficultyOption_Fast = 0x3
    ECombatTimerDifficultyOption_NumTypes = 0x4

class eSprintingCostDifficulty(IntEnum):
    ESprintingCostDifficulty_Free = 0x0
    ESprintingCostDifficulty_Low = 0x1
    ESprintingCostDifficulty_Full = 0x2
    ESprintingCostDifficulty_NumTypes = 0x3

class eAIShipWeapon(IntEnum):
    EAIShipWeapon_Projectile = 0x0
    EAIShipWeapon_Laser = 0x1
    EAIShipWeapon_MiningLaser = 0x2
    EAIShipWeapon_NumTypes = 0x3

class eShipWeapon(IntEnum):
    EShipWeapon_Laser = 0x0
    EShipWeapon_Projectile = 0x1
    EShipWeapon_Shotgun = 0x2
    EShipWeapon_Minigun = 0x3
    EShipWeapon_Plasma = 0x4
    EShipWeapon_Missile = 0x5
    EShipWeapon_Rocket = 0x6
    EShipWeapon_NumTypes = 0x7

class eDifficultyOptionGroup(IntEnum):
    EDifficultyOptionGroup_Survival = 0x0
    EDifficultyOptionGroup_Crafting = 0x1
    EDifficultyOptionGroup_Combat = 0x2
    EDifficultyOptionGroup_Ease = 0x3
    EDifficultyOptionGroup_NumTypes = 0x4

class ePivotAnchorsTo(IntEnum):
    EPivotAnchorsTo_Itself = 0x0
    EPivotAnchorsTo_Parent = 0x1
    EPivotAnchorsTo_Node = 0x2
    EPivotAnchorsTo_NodeWithAnchor = 0x3
    EPivotAnchorsTo_NumTypes = 0x4

class eApplyMotionScaleIn(IntEnum):
    EApplyMotionScaleIn_Disabled = 0x0
    EApplyMotionScaleIn_Uniform = 0x1
    EApplyMotionScaleIn_Itself = 0x2
    EApplyMotionScaleIn_Parent = 0x3
    EApplyMotionScaleIn_Component = 0x4
    EApplyMotionScaleIn_NumTypes = 0x5

class eApplyMotionLimitsIn(IntEnum):
    EApplyMotionLimitsIn_Disabled = 0x0
    EApplyMotionLimitsIn_Uniform = 0x1
    EApplyMotionLimitsIn_Itself = 0x2
    EApplyMotionLimitsIn_Parent = 0x3
    EApplyMotionLimitsIn_Component = 0x4
    EApplyMotionLimitsIn_NumTypes = 0x5

class eApplyAngularMotionScaleIn(IntEnum):
    EApplyAngularMotionScaleIn_Disabled = 0x0
    EApplyAngularMotionScaleIn_Uniform = 0x1
    EApplyAngularMotionScaleIn_Itself = 0x2
    EApplyAngularMotionScaleIn_Parent = 0x3
    EApplyAngularMotionScaleIn_Component = 0x4
    EApplyAngularMotionScaleIn_NumTypes = 0x5

class eApplyAngularLimitsIn(IntEnum):
    EApplyAngularLimitsIn_Disabled = 0x0
    EApplyAngularLimitsIn_Itself = 0x1
    EApplyAngularLimitsIn_Parent = 0x2
    EApplyAngularLimitsIn_Component = 0x3
    EApplyAngularLimitsIn_NumTypes = 0x4

class eDifficultySetting(IntEnum):
    EDifficultySetting_SettingsLocked = 0x0
    EDifficultySetting_InventoriesAlwaysInRange = 0x1
    EDifficultySetting_AllSlotsUnlocked = 0x2
    EDifficultySetting_WarpDriveRequirements = 0x3
    EDifficultySetting_CraftingIsFree = 0x4
    EDifficultySetting_TutorialEnabled = 0x5
    EDifficultySetting_StartWithAllItemsKnown = 0x6
    EDifficultySetting_BaseAutoPower = 0x7
    EDifficultySetting_DeathConsequences = 0x8
    EDifficultySetting_DamageReceived = 0x9
    EDifficultySetting_DamageGiven = 0xA
    EDifficultySetting_ActiveSurvivalBars = 0xB
    EDifficultySetting_HazardDrain = 0xC
    EDifficultySetting_EnergyDrain = 0xD
    EDifficultySetting_SubstanceCollection = 0xE
    EDifficultySetting_InventoryStackLimits = 0xF
    EDifficultySetting_ChargingRequirements = 0x10
    EDifficultySetting_FuelUse = 0x11
    EDifficultySetting_LaunchFuelCost = 0x12
    EDifficultySetting_CurrencyCost = 0x13
    EDifficultySetting_ScannerRecharge = 0x14
    EDifficultySetting_ReputationGain = 0x15
    EDifficultySetting_CreatureHostility = 0x16
    EDifficultySetting_SpaceCombat = 0x17
    EDifficultySetting_GroundCombat = 0x18
    EDifficultySetting_ItemShopAvailablity = 0x19
    EDifficultySetting_SprintingCost = 0x1A
    EDifficultySetting_BreakTechOnDamage = 0x1B
    EDifficultySetting_NumTypes = 0x1C

class eMovementDirection(IntEnum):
    EMovementDirection_WorldRelative = 0x0
    EMovementDirection_BodyRelative = 0x1
    EMovementDirection_HeadRelative = 0x2
    EMovementDirection_NotSet = 0x3
    EMovementDirection_NumTypes = 0x4

class eQuickMenuActions(IntEnum):
    EQuickMenuActions_None = 0x0
    EQuickMenuActions_CallFreighter = 0x1
    EQuickMenuActions_DismissFreighter = 0x2
    EQuickMenuActions_SummonNexus = 0x3
    EQuickMenuActions_CallShip = 0x4
    EQuickMenuActions_CallSquadron = 0x5
    EQuickMenuActions_SummonVehicleSubMenu = 0x6
    EQuickMenuActions_SummonBuggy = 0x7
    EQuickMenuActions_SummonBike = 0x8
    EQuickMenuActions_SummonTruck = 0x9
    EQuickMenuActions_SummonWheeledBike = 0xA
    EQuickMenuActions_SummonHovercraft = 0xB
    EQuickMenuActions_SummonSubmarine = 0xC
    EQuickMenuActions_SummonMech = 0xD
    EQuickMenuActions_VehicleAIToggle = 0xE
    EQuickMenuActions_VehicleScan = 0xF
    EQuickMenuActions_VehicleScanSelect = 0x10
    EQuickMenuActions_VehicleRestartRace = 0x11
    EQuickMenuActions_Torch = 0x12
    EQuickMenuActions_GalaxyMap = 0x13
    EQuickMenuActions_PhotoMode = 0x14
    EQuickMenuActions_ChargeMenu = 0x15
    EQuickMenuActions_Charge = 0x16
    EQuickMenuActions_ChargeSubMenu = 0x17
    EQuickMenuActions_Repair = 0x18
    EQuickMenuActions_BuildMenu = 0x19
    EQuickMenuActions_CommunicatorReceive = 0x1A
    EQuickMenuActions_CommunicatorInitiate = 0x1B
    EQuickMenuActions_ThirdPersonCharacter = 0x1C
    EQuickMenuActions_ThirdPersonShip = 0x1D
    EQuickMenuActions_ThirdPersonVehicle = 0x1E
    EQuickMenuActions_EconomyScan = 0x1F
    EQuickMenuActions_EmoteMenu = 0x20
    EQuickMenuActions_Emote = 0x21
    EQuickMenuActions_UtilitySubMenu = 0x22
    EQuickMenuActions_SummonSubMenu = 0x23
    EQuickMenuActions_SummonShipSubMenu = 0x24
    EQuickMenuActions_ChangeSecondaryWeaponMenu = 0x25
    EQuickMenuActions_ChangeSecondaryWeapon = 0x26
    EQuickMenuActions_ChooseCreatureFoodMenu = 0x27
    EQuickMenuActions_ChooseCreatureFood = 0x28
    EQuickMenuActions_EmergencyWarp = 0x29
    EQuickMenuActions_SwapMultitool = 0x2A
    EQuickMenuActions_SwapMultitoolSubMenu = 0x2B
    EQuickMenuActions_CreatureSubMenu = 0x2C
    EQuickMenuActions_SummonPet = 0x2D
    EQuickMenuActions_SummonPetSubMenu = 0x2E
    EQuickMenuActions_WarpToNexus = 0x2F
    EQuickMenuActions_PetUI = 0x30
    EQuickMenuActions_ByteBeatSubMenu = 0x31
    EQuickMenuActions_ByteBeatPlay = 0x32
    EQuickMenuActions_ByteBeatStop = 0x33
    EQuickMenuActions_ByteBeatLibrary = 0x34
    EQuickMenuActions_ReportBase = 0x35
    EQuickMenuActions_CargoShield = 0x36
    EQuickMenuActions_CallRocket = 0x37
    EQuickMenuActions_Invalid = 0x38
    EQuickMenuActions_NumTypes = 0x39

class eActiveModeOnFoot(IntEnum):
    EActiveModeOnFoot_None = 0x0
    EActiveModeOnFoot_ScopeOnly = 0x1
    EActiveModeOnFoot_ScopeOrFiring = 0x2
    EActiveModeOnFoot_Always = 0x3
    EActiveModeOnFoot_NumTypes = 0x4

class eActiveModeWhenBuilding(IntEnum):
    EActiveModeWhenBuilding_None = 0x0
    EActiveModeWhenBuilding_BuildPlacementOnly = 0x1
    EActiveModeWhenBuilding_SelectionModeOnly = 0x2
    EActiveModeWhenBuilding_Always = 0x3
    EActiveModeWhenBuilding_NumTypes = 0x4

class eEnableGyroInBuildingFreeCam(IntEnum):
    EEnableGyroInBuildingFreeCam_Never = 0x0
    EEnableGyroInBuildingFreeCam_MatchActiveModeWhenBuilding = 0x1
    EEnableGyroInBuildingFreeCam_Always = 0x2
    EEnableGyroInBuildingFreeCam_NumTypes = 0x3

class eActiveModeInExocraft(IntEnum):
    EActiveModeInExocraft_None = 0x0
    EActiveModeInExocraft_Firing = 0x1
    EActiveModeInExocraft_Always = 0x2
    EActiveModeInExocraft_NumTypes = 0x3

class eLookStickEnabled(IntEnum):
    ELookStickEnabled_None = 0x0
    ELookStickEnabled_Disabled = 0x1
    ELookStickEnabled_Enabled = 0x2
    ELookStickEnabled_NumTypes = 0x3

class eCursorLookStickEnabled(IntEnum):
    ECursorLookStickEnabled_None = 0x0
    ECursorLookStickEnabled_Disabled = 0x1
    ECursorLookStickEnabled_NumTypes = 0x2

class eHandedness(IntEnum):
    EHandedness_Left = 0x0
    EHandedness_Right = 0x1
    EHandedness_NumTypes = 0x2

class eGyroRotationSpace(IntEnum):
    EGyroRotationSpace_Local = 0x0
    EGyroRotationSpace_Player = 0x1
    EGyroRotationSpace_NumTypes = 0x2

class eGyroRotationSpaceHandheld(IntEnum):
    EGyroRotationSpaceHandheld_Local = 0x0
    EGyroRotationSpaceHandheld_Player = 0x1
    EGyroRotationSpaceHandheld_NumTypes = 0x2

class eDifficultySettingType(IntEnum):
    EDifficultySettingType_Toggle = 0x0
    EDifficultySettingType_OptionList = 0x1
    EDifficultySettingType_NumTypes = 0x2

class eDifficultySettingEditability(IntEnum):
    EDifficultySettingEditability_FullyEditable = 0x0
    EDifficultySettingEditability_IncreaseOnly = 0x1
    EDifficultySettingEditability_DecreaseOnly = 0x2
    EDifficultySettingEditability_LockedVisible = 0x3
    EDifficultySettingEditability_LockedHidden = 0x4
    EDifficultySettingEditability_NumTypes = 0x5

class eYawAxisDirection(IntEnum):
    EYawAxisDirection_Disabled = 0x0
    EYawAxisDirection_Standard = 0x1
    EYawAxisDirection_Inverted = 0x2
    EYawAxisDirection_NumTypes = 0x3

class eRollAxisDirection(IntEnum):
    ERollAxisDirection_Disabled = 0x0
    ERollAxisDirection_Standard = 0x1
    ERollAxisDirection_Inverted = 0x2
    ERollAxisDirection_NumTypes = 0x3

class ePitchAxisDirection(IntEnum):
    EPitchAxisDirection_Disabled = 0x0
    EPitchAxisDirection_Standard = 0x1
    EPitchAxisDirection_Inverted = 0x2
    EPitchAxisDirection_NumTypes = 0x3

class eHotActionMenuTypes(IntEnum):
    EHotActionMenuTypes_OnFoot = 0x0
    EHotActionMenuTypes_InShip = 0x1
    EHotActionMenuTypes_InExocraft = 0x2
    EHotActionMenuTypes_NumTypes = 0x3

class eHand(IntEnum):
    EHand_Right = 0x0
    EHand_Left = 0x1
    EHand_NumTypes = 0x2
    EHand_Unspecified = 0xFFFFFFFF

class ePS4FixedFPS(IntEnum):
    EPS4FixedFPS_Invalid = 0x0
    EPS4FixedFPS_False = 0x1
    EPS4FixedFPS_True = 0x2
    EPS4FixedFPS_NumTypes = 0x3

class eBaseSharingMode(IntEnum):
    EBaseSharingMode_Undecided = 0x0
    EBaseSharingMode_On = 0x1
    EBaseSharingMode_Off = 0x2
    EBaseSharingMode_NumTypes = 0x3

class eTemperatureUnit(IntEnum):
    ETemperatureUnit_Invalid = 0x0
    ETemperatureUnit_C = 0x1
    ETemperatureUnit_F = 0x2
    ETemperatureUnit_K = 0x3
    ETemperatureUnit_NumTypes = 0x4

class eMovementMode(IntEnum):
    EMovementMode_Teleporter = 0x0
    EMovementMode_Smooth = 0x1
    EMovementMode_NumTypes = 0x2

class eTurnMode(IntEnum):
    ETurnMode_Smooth = 0x0
    ETurnMode_Snap = 0x1
    ETurnMode_NumTypes = 0x2

class eUIColourScheme(IntEnum):
    EUIColourScheme_Default = 0x0
    EUIColourScheme_Protanopia = 0x1
    EUIColourScheme_Deuteranopia = 0x2
    EUIColourScheme_Tritanopia = 0x3
    EUIColourScheme_NumTypes = 0x4

class eSpaceCombatFollowMode(IntEnum):
    ESpaceCombatFollowMode_Disabled = 0x0
    ESpaceCombatFollowMode_Hold = 0x1
    ESpaceCombatFollowMode_Toggle = 0x2
    ESpaceCombatFollowMode_NumTypes = 0x3

class exEyeTrackingFlags(IntEnum):
    FEyeTrackingFlags_None = 0x0
    FEyeTrackingFlags_BaseBuilding = 0x1
    FEyeTrackingFlags_WristMenus = 0x2
    FEyeTrackingFlags_LookVector = 0x4
    FEyeTrackingFlags_BinocScanner = 0x8
    FEyeTrackingFlags_Mask = 0xF

class eHighResVRUI(IntEnum):
    EHighResVRUI_High = 0x0
    EHighResVRUI_Low = 0x1
    EHighResVRUI_NumTypes = 0x2

class eInventoryType(IntEnum):
    EInventoryType_Substance = 0x0
    EInventoryType_Technology = 0x1
    EInventoryType_Product = 0x2
    EInventoryType_NumTypes = 0x3
    EInventoryType_Unspecified = 0xFFFFFFFF

class eOptionsUIHeaderIconType(IntEnum):
    EOptionsUIHeaderIconType_General = 0x0
    EOptionsUIHeaderIconType_Ship = 0x1
    EOptionsUIHeaderIconType_Cog = 0x2
    EOptionsUIHeaderIconType_Scanner = 0x3
    EOptionsUIHeaderIconType_Advanced = 0x4
    EOptionsUIHeaderIconType_NumTypes = 0x5

class exPersistentBaseDifficultyFlags(IntEnum):
    FPersistentBaseDifficultyFlags_None = 0x0
    FPersistentBaseDifficultyFlags_Locked = 0x1

class eDepthOfFieldSetting(IntEnum):
    EDepthOfFieldSetting_Off = 0x0
    EDepthOfFieldSetting_Mid = 0x1
    EDepthOfFieldSetting_On = 0x2
    EDepthOfFieldSetting_Macro = 0x3
    EDepthOfFieldSetting_NumTypes = 0x4

class eInventoryClass(IntEnum):
    EInventoryClass_C = 0x0
    EInventoryClass_B = 0x1
    EInventoryClass_A = 0x2
    EInventoryClass_S = 0x3
    EInventoryClass_NumTypes = 0x4

class eSurvivalBar(IntEnum):
    ESurvivalBar_Health = 0x0
    ESurvivalBar_Hazard = 0x1
    ESurvivalBar_Energy = 0x2
    ESurvivalBar_NumTypes = 0x3

class eInventoryStackSizeGroup(IntEnum):
    EInventoryStackSizeGroup_Default = 0x0
    EInventoryStackSizeGroup_Personal = 0x1
    EInventoryStackSizeGroup_PersonalCargo = 0x2
    EInventoryStackSizeGroup_Ship = 0x3
    EInventoryStackSizeGroup_ShipCargo = 0x4
    EInventoryStackSizeGroup_Freighter = 0x5
    EInventoryStackSizeGroup_FreighterCargo = 0x6
    EInventoryStackSizeGroup_Vehicle = 0x7
    EInventoryStackSizeGroup_Chest = 0x8
    EInventoryStackSizeGroup_BaseCapsule = 0x9
    EInventoryStackSizeGroup_MaintenanceObject = 0xA
    EInventoryStackSizeGroup_UIPopup = 0xB
    EInventoryStackSizeGroup_NumTypes = 0xC

class eInventorySpecialSlotType(IntEnum):
    EInventorySpecialSlotType_Broken = 0x0
    EInventorySpecialSlotType_TechOnly = 0x1
    EInventorySpecialSlotType_Cargo = 0x2
    EInventorySpecialSlotType_BlockedByBrokenTech = 0x3
    EInventorySpecialSlotType_TechBonus = 0x4
    EInventorySpecialSlotType_NumTypes = 0x5
    EInventorySpecialSlotType_Unspecified = 0xFFFFFFFF

class eMechMeshPart(IntEnum):
    EMechMeshPart_Scanner = 0x0
    EMechMeshPart_Body = 0x1
    EMechMeshPart_Legs = 0x2
    EMechMeshPart_LeftArm = 0x3
    EMechMeshPart_RightArm = 0x4
    EMechMeshPart_NumTypes = 0x5

class eMechMeshType(IntEnum):
    EMechMeshType_Exocraft = 0x0
    EMechMeshType_Sentinel = 0x1
    EMechMeshType_NumTypes = 0x2

class eWeather(IntEnum):
    EWeather_Clear = 0x0
    EWeather_Dust = 0x1
    EWeather_Humid = 0x2
    EWeather_Snow = 0x3
    EWeather_Toxic = 0x4
    EWeather_Scorched = 0x5
    EWeather_Radioactive = 0x6
    EWeather_RedWeather = 0x7
    EWeather_GreenWeather = 0x8
    EWeather_BlueWeather = 0x9
    EWeather_Swamp = 0xA
    EWeather_Lava = 0xB
    EWeather_Bubble = 0xC
    EWeather_Weird = 0xD
    EWeather_Fire = 0xE
    EWeather_ClearCold = 0xF
    EWeather_NumTypes = 0x10

class eMechWeaponLocation(IntEnum):
    EMechWeaponLocation_TurretExocraft = 0x0
    EMechWeaponLocation_TurretSentinel = 0x1
    EMechWeaponLocation_ArmLeft = 0x2
    EMechWeaponLocation_ArmRight = 0x3
    EMechWeaponLocation_NumTypes = 0x4

class eAIShipRole(IntEnum):
    EAIShipRole_Standard = 0x0
    EAIShipRole_PlayerSquadron = 0x1
    EAIShipRole_Freighter = 0x2
    EAIShipRole_CapitalFreighter = 0x3
    EAIShipRole_SmallFreighter = 0x4
    EAIShipRole_TinyFreighter = 0x5
    EAIShipRole_Frigate = 0x6
    EAIShipRole_NumTypes = 0x7

class eShipsToMark(IntEnum):
    EShipsToMark_None = 0x0
    EShipsToMark_Leader = 0x1
    EShipsToMark_All = 0x2
    EShipsToMark_NumTypes = 0x3

class eShipClass(IntEnum):
    EShipClass_Freighter = 0x0
    EShipClass_Dropship = 0x1
    EShipClass_Fighter = 0x2
    EShipClass_Scientific = 0x3
    EShipClass_Shuttle = 0x4
    EShipClass_PlayerFreighter = 0x5
    EShipClass_Royal = 0x6
    EShipClass_Alien = 0x7
    EShipClass_Sail = 0x8
    EShipClass_NumTypes = 0x9

class eFrigateClass(IntEnum):
    EFrigateClass_Combat = 0x0
    EFrigateClass_Exploration = 0x1
    EFrigateClass_Mining = 0x2
    EFrigateClass_Diplomacy = 0x3
    EFrigateClass_Support = 0x4
    EFrigateClass_Normandy = 0x5
    EFrigateClass_DeepSpace = 0x6
    EFrigateClass_DeepSpaceCommon = 0x7
    EFrigateClass_NumTypes = 0x8
    EFrigateClass_Unspecified = 0xFFFFFFFF

class eFrigateFlybyType(IntEnum):
    EFrigateFlybyType_SingleShip = 0x0
    EFrigateFlybyType_AmbientGroup = 0x1
    EFrigateFlybyType_ScriptedGroup = 0x2
    EFrigateFlybyType_DeepSpace = 0x3
    EFrigateFlybyType_DeepSpaceCommon = 0x4
    EFrigateFlybyType_NumTypes = 0x5
    EFrigateFlybyType_Unspecified = 0xFFFFFFFF

class eSpawnShape(IntEnum):
    ESpawnShape_Sphere = 0x0
    ESpawnShape_Cone = 0x1
    ESpawnShape_NumTypes = 0x2

class eShipType(IntEnum):
    EShipType_None = 0x0
    EShipType_Pirate = 0x1
    EShipType_Police = 0x2
    EShipType_Trader = 0x3
    EShipType_Freighter = 0x4
    EShipType_PlayerSquadron = 0x5
    EShipType_DefenceForce = 0x6
    EShipType_NumTypes = 0x7

class eFlareType(IntEnum):
    EFlareType_Default = 0x0
    EFlareType_NumTypes = 0x1

class eWealthClass(IntEnum):
    EWealthClass_Poor = 0x0
    EWealthClass_Average = 0x1
    EWealthClass_Wealthy = 0x2
    EWealthClass_Pirate = 0x3
    EWealthClass_NumTypes = 0x4

class eTradingClass(IntEnum):
    ETradingClass_Mining = 0x0
    ETradingClass_HighTech = 0x1
    ETradingClass_Trading = 0x2
    ETradingClass_Manufacturing = 0x3
    ETradingClass_Fusion = 0x4
    ETradingClass_Scientific = 0x5
    ETradingClass_PowerGeneration = 0x6
    ETradingClass_NumTypes = 0x7

class eAIFaction(IntEnum):
    EAIFaction_Player = 0x0
    EAIFaction_Civilian = 0x1
    EAIFaction_Pirate = 0x2
    EAIFaction_Police = 0x3
    EAIFaction_Creature = 0x4
    EAIFaction_NumTypes = 0x5

class eConflictLevel(IntEnum):
    EConflictLevel_Low = 0x0
    EConflictLevel_Default = 0x1
    EConflictLevel_High = 0x2
    EConflictLevel_Pirate = 0x3
    EConflictLevel_NumTypes = 0x4

class eSolarSystemClass(IntEnum):
    ESolarSystemClass_Default = 0x0
    ESolarSystemClass_Initial = 0x1
    ESolarSystemClass_Anomaly = 0x2
    ESolarSystemClass_GameStart = 0x3
    ESolarSystemClass_NumTypes = 0x4

class eMessageType(IntEnum):
    EMessageType_Leave = 0x0
    EMessageType_Fight = 0x1
    EMessageType_NumTypes = 0x2

class eDialogueTree(IntEnum):
    EDialogueTree_Bribe = 0x0
    EDialogueTree_Beg = 0x1
    EDialogueTree_Ambush = 0x2
    EDialogueTree_Trade = 0x3
    EDialogueTree_Help = 0x4
    EDialogueTree_Goods = 0x5
    EDialogueTree_Hostile = 0x6
    EDialogueTree_NumTypes = 0x7

class eCommunicatorType(IntEnum):
    ECommunicatorType_HoloExplorer = 0x0
    ECommunicatorType_HoloSceptic = 0x1
    ECommunicatorType_HoloNoone = 0x2
    ECommunicatorType_Generic = 0x3
    ECommunicatorType_PlayerFreighterCaptain = 0x4
    ECommunicatorType_Polo = 0x5
    ECommunicatorType_Nada = 0x6
    ECommunicatorType_QuicksilverBot = 0x7
    ECommunicatorType_PlayerSettlementResident = 0x8
    ECommunicatorType_CargoScanDrone = 0x9
    ECommunicatorType_Tethys = 0xA
    ECommunicatorType_NumTypes = 0xB

class eSentinelLevel(IntEnum):
    ESentinelLevel_Low = 0x0
    ESentinelLevel_Default = 0x1
    ESentinelLevel_Aggressive = 0x2
    ESentinelLevel_NumTypes = 0x3

class ePlanetSize(IntEnum):
    EPlanetSize_Large = 0x0
    EPlanetSize_Medium = 0x1
    EPlanetSize_Small = 0x2
    EPlanetSize_Moon = 0x3
    EPlanetSize_NumTypes = 0x4

class eCoverageType(IntEnum):
    ECoverageType_Total = 0x0
    ECoverageType_SmoothPatch = 0x1
    ECoverageType_GridPatch = 0x2
    ECoverageType_NumTypes = 0x3

class eWeatherColourSet(IntEnum):
    EWeatherColourSet_Common = 0x0
    EWeatherColourSet_Rare = 0x1
    EWeatherColourSet_NumTypes = 0x2

class eScreenFilter(IntEnum):
    EScreenFilter_Default = 0x0
    EScreenFilter_DefaultStorm = 0x1
    EScreenFilter_Frozen = 0x2
    EScreenFilter_FrozenStorm = 0x3
    EScreenFilter_Toxic = 0x4
    EScreenFilter_ToxicStorm = 0x5
    EScreenFilter_Radioactive = 0x6
    EScreenFilter_RadioactiveStorm = 0x7
    EScreenFilter_Scorched = 0x8
    EScreenFilter_ScorchedStorm = 0x9
    EScreenFilter_Barren = 0xA
    EScreenFilter_BarrenStorm = 0xB
    EScreenFilter_Weird1 = 0xC
    EScreenFilter_Weird2 = 0xD
    EScreenFilter_Weird3 = 0xE
    EScreenFilter_Weird4 = 0xF
    EScreenFilter_Weird5 = 0x10
    EScreenFilter_Weird6 = 0x11
    EScreenFilter_Weird7 = 0x12
    EScreenFilter_Weird8 = 0x13
    EScreenFilter_Vintage = 0x14
    EScreenFilter_HyperReal = 0x15
    EScreenFilter_Desaturated = 0x16
    EScreenFilter_Amber = 0x17
    EScreenFilter_GBGreen = 0x18
    EScreenFilter_Apocalypse = 0x19
    EScreenFilter_Diffusion = 0x1A
    EScreenFilter_Green = 0x1B
    EScreenFilter_BlackAndWhite = 0x1C
    EScreenFilter_Isolation = 0x1D
    EScreenFilter_Sepia = 0x1E
    EScreenFilter_Filmic = 0x1F
    EScreenFilter_GBGrey = 0x20
    EScreenFilter_Binoculars = 0x21
    EScreenFilter_Surveying = 0x22
    EScreenFilter_Nexus = 0x23
    EScreenFilter_SpaceStation = 0x24
    EScreenFilter_Freighter = 0x25
    EScreenFilter_FreighterAbandoned = 0x26
    EScreenFilter_Frigate = 0x27
    EScreenFilter_MissionSurvey = 0x28
    EScreenFilter_NewVibrant = 0x29
    EScreenFilter_NewVibrantBright = 0x2A
    EScreenFilter_NewVibrantWarm = 0x2B
    EScreenFilter_NewVintageBright = 0x2C
    EScreenFilter_NewVintageWash = 0x2D
    EScreenFilter_Drama = 0x2E
    EScreenFilter_MemoryBold = 0x2F
    EScreenFilter_Memory = 0x30
    EScreenFilter_MemoryWash = 0x31
    EScreenFilter_Autumn = 0x32
    EScreenFilter_AutumnFade = 0x33
    EScreenFilter_ClassicBright = 0x34
    EScreenFilter_Classic = 0x35
    EScreenFilter_ClassicWash = 0x36
    EScreenFilter_BlackAndWhiteDream = 0x37
    EScreenFilter_ColourTouchB = 0x38
    EScreenFilter_ColourTouchC = 0x39
    EScreenFilter_NegativePrint = 0x3A
    EScreenFilter_SepiaExtreme = 0x3B
    EScreenFilter_Solarise = 0x3C
    EScreenFilter_TwoToneStrong = 0x3D
    EScreenFilter_TwoTone = 0x3E
    EScreenFilter_Dramatic = 0x3F
    EScreenFilter_Fuchsia = 0x40
    EScreenFilter_Violet = 0x41
    EScreenFilter_Cyan = 0x42
    EScreenFilter_GreenNew = 0x43
    EScreenFilter_AmberNew = 0x44
    EScreenFilter_Red = 0x45
    EScreenFilter_HueShiftA = 0x46
    EScreenFilter_HueShiftB = 0x47
    EScreenFilter_HueShiftC = 0x48
    EScreenFilter_HueShiftD = 0x49
    EScreenFilter_WarmStripe = 0x4A
    EScreenFilter_NMSRetroA = 0x4B
    EScreenFilter_NMSRetroB = 0x4C
    EScreenFilter_NMSRetroC = 0x4D
    EScreenFilter_NMSRetroD = 0x4E
    EScreenFilter_NMSRetroE = 0x4F
    EScreenFilter_NMSRetroF = 0x50
    EScreenFilter_NMSRetroG = 0x51
    EScreenFilter_NumTypes = 0x52

class eRainbowType(IntEnum):
    ERainbowType_Always = 0x0
    ERainbowType_Occasional = 0x1
    ERainbowType_Storm = 0x2
    ERainbowType_None = 0x3
    ERainbowType_NumTypes = 0x4

class eWeatherIntensity(IntEnum):
    EWeatherIntensity_Default = 0x0
    EWeatherIntensity_Extreme = 0x1
    EWeatherIntensity_NumTypes = 0x2

class eStormFrequency(IntEnum):
    EStormFrequency_None = 0x0
    EStormFrequency_Low = 0x1
    EStormFrequency_High = 0x2
    EStormFrequency_NumTypes = 0x3

class eAtmosphereType(IntEnum):
    EAtmosphereType_None = 0x0
    EAtmosphereType_Normal = 0x1
    EAtmosphereType_NumTypes = 0x2

class eGalaxyStarType(IntEnum):
    EGalaxyStarType_Yellow = 0x0
    EGalaxyStarType_Green = 0x1
    EGalaxyStarType_Blue = 0x2
    EGalaxyStarType_Red = 0x3
    EGalaxyStarType_NumTypes = 0x4

class eLifeSetting(IntEnum):
    ELifeSetting_Dead = 0x0
    ELifeSetting_Low = 0x1
    ELifeSetting_Mid = 0x2
    ELifeSetting_Full = 0x3
    ELifeSetting_NumTypes = 0x4

class eBiomeSubType(IntEnum):
    EBiomeSubType_None = 0x0
    EBiomeSubType_Standard = 0x1
    EBiomeSubType_HighQuality = 0x2
    EBiomeSubType_Structure = 0x3
    EBiomeSubType_Beam = 0x4
    EBiomeSubType_Hexagon = 0x5
    EBiomeSubType_FractCube = 0x6
    EBiomeSubType_Bubble = 0x7
    EBiomeSubType_Shards = 0x8
    EBiomeSubType_Contour = 0x9
    EBiomeSubType_Shell = 0xA
    EBiomeSubType_BoneSpire = 0xB
    EBiomeSubType_WireCell = 0xC
    EBiomeSubType_HydroGarden = 0xD
    EBiomeSubType_HugePlant = 0xE
    EBiomeSubType_HugeLush = 0xF
    EBiomeSubType_HugeRing = 0x10
    EBiomeSubType_HugeRock = 0x11
    EBiomeSubType_HugeScorch = 0x12
    EBiomeSubType_HugeToxic = 0x13
    EBiomeSubType_Variant_A = 0x14
    EBiomeSubType_Variant_B = 0x15
    EBiomeSubType_Variant_C = 0x16
    EBiomeSubType_Variant_D = 0x17
    EBiomeSubType_Infested = 0x18
    EBiomeSubType_Swamp = 0x19
    EBiomeSubType_Lava = 0x1A
    EBiomeSubType_NumTypes = 0x1B

class ePlanetClass(IntEnum):
    EPlanetClass_Default = 0x0
    EPlanetClass_Initial = 0x1
    EPlanetClass_InInitialSystem = 0x2
    EPlanetClass_NumTypes = 0x3

class eLocatorType(IntEnum):
    ELocatorType_Generic1 = 0x0
    ELocatorType_Generic2 = 0x1
    ELocatorType_Generic3 = 0x2
    ELocatorType_Generic4 = 0x3
    ELocatorType_NumTypes = 0x4

class eChoice(IntEnum):
    EChoice_LookupName = 0x0
    EChoice_AnyOfType = 0x1
    EChoice_SpecificIndex = 0x2
    EChoice_InFrontOfPlayer = 0x3
    EChoice_NumTypes = 0x4

class eSpawnMode(IntEnum):
    ESpawnMode_None = 0x0
    ESpawnMode_UseSeed = 0x1
    ESpawnMode_UseAltID = 0x2
    ESpawnMode_NumTypes = 0x3

class eSentinelType(IntEnum):
    ESentinelType_PatrolDrone = 0x0
    ESentinelType_CombatDrone = 0x1
    ESentinelType_MedicDrone = 0x2
    ESentinelType_SummonerDrone = 0x3
    ESentinelType_CorruptedDrone = 0x4
    ESentinelType_Quad = 0x5
    ESentinelType_Mech = 0x6
    ESentinelType_Walker = 0x7
    ESentinelType_FriendlyDrone = 0x8
    ESentinelType_NumTypes = 0x9

class eEvent(IntEnum):
    EEvent_None = 0x0
    EEvent_Pirates = 0x1
    EEvent_Police = 0x2
    EEvent_Traders = 0x3
    EEvent_Walker = 0x4
    EEvent_NumTypes = 0x5

class eNavAreaType(IntEnum):
    ENavAreaType_Normal = 0x0
    ENavAreaType_BuildingWithExterior = 0x1
    ENavAreaType_Debris = 0x2
    ENavAreaType_Ship = 0x3
    ENavAreaType_Mech = 0x4
    ENavAreaType_PlanetMech = 0x5
    ENavAreaType_Demo = 0x6
    ENavAreaType_WFCBase = 0x7
    ENavAreaType_FreighterBase = 0x8
    ENavAreaType_NumTypes = 0x9

class eDiet(IntEnum):
    EDiet_Carnivore = 0x0
    EDiet_Omnivore = 0x1
    EDiet_Herbivore = 0x2
    EDiet_Robot = 0x3
    EDiet_NumTypes = 0x4

class eGameIcons(IntEnum):
    EGameIcons_Stamina = 0x0
    EGameIcons_NoStamina = 0x1
    EGameIcons_EnergyCharge = 0x2
    EGameIcons_Scanner = 0x3
    EGameIcons_NoScanner = 0x4
    EGameIcons_Grave = 0x5
    EGameIcons_Resources = 0x6
    EGameIcons_Inventory = 0x7
    EGameIcons_InventoryFull = 0x8
    EGameIcons_RareItems = 0x9
    EGameIcons_Pirates = 0xA
    EGameIcons_PirateScan = 0xB
    EGameIcons_Drone = 0xC
    EGameIcons_Quad = 0xD
    EGameIcons_Mech = 0xE
    EGameIcons_Walker = 0xF
    EGameIcons_DroneOff = 0x10
    EGameIcons_Police = 0x11
    EGameIcons_AtlasStation = 0x12
    EGameIcons_BlackHole = 0x13
    EGameIcons_SaveGame = 0x14
    EGameIcons_SaveInventory = 0x15
    EGameIcons_Jetpack = 0x16
    EGameIcons_JetpackEmpty = 0x17
    EGameIcons_VehicleBoost = 0x18
    EGameIcons_VehicleBoostRecharge = 0x19
    EGameIcons_Fuel = 0x1A
    EGameIcons_GekStanding = 0x1B
    EGameIcons_VykeenStanding = 0x1C
    EGameIcons_KorvaxStanding = 0x1D
    EGameIcons_GekDiamondStanding = 0x1E
    EGameIcons_VykeenDiamondStanding = 0x1F
    EGameIcons_KorvaxDiamondStanding = 0x20
    EGameIcons_TradeGuildStanding = 0x21
    EGameIcons_WarGuildStanding = 0x22
    EGameIcons_ExplorationGuildStanding = 0x23
    EGameIcons_TradeGuildDiamondStanding = 0x24
    EGameIcons_WarGuildDiamondStanding = 0x25
    EGameIcons_ExplorationGuildDiamondStanding = 0x26
    EGameIcons_GMPathToCentre = 0x27
    EGameIcons_GMAtlas = 0x28
    EGameIcons_GMBlackHole = 0x29
    EGameIcons_GMUserWaypoint = 0x2A
    EGameIcons_GMUserMission = 0x2B
    EGameIcons_GMSeasonal = 0x2C
    EGameIcons_TransferPersonal = 0x2D
    EGameIcons_TransferPersonalCargo = 0x2E
    EGameIcons_TransferShip = 0x2F
    EGameIcons_TransferBike = 0x30
    EGameIcons_TransferBuggy = 0x31
    EGameIcons_TransferTruck = 0x32
    EGameIcons_TransferWheeledBike = 0x33
    EGameIcons_TransferHovercraft = 0x34
    EGameIcons_TransferSubmarine = 0x35
    EGameIcons_TransferMech = 0x36
    EGameIcons_TransferFreighter = 0x37
    EGameIcons_TransferBase = 0x38
    EGameIcons_HazardIndicatorHot = 0x39
    EGameIcons_HazardIndicatorCold = 0x3A
    EGameIcons_HazardIndicatorRadiation = 0x3B
    EGameIcons_HazardIndicatorToxic = 0x3C
    EGameIcons_TerrainAdd = 0x3D
    EGameIcons_TerrainRemove = 0x3E
    EGameIcons_TerrainFlatten = 0x3F
    EGameIcons_TerrainUndo = 0x40
    EGameIcons_SpacePhone = 0x41
    EGameIcons_GarageMarkerBuggy = 0x42
    EGameIcons_GarageMarkerBike = 0x43
    EGameIcons_GarageMarkerTruck = 0x44
    EGameIcons_GarageMarkerWheeledBike = 0x45
    EGameIcons_GarageMarkerHovercraft = 0x46
    EGameIcons_CorruptedDrone = 0x47
    EGameIcons_HandHold = 0x48
    EGameIcons_ShipThumbnailBG = 0x49
    EGameIcons_CClass = 0x4A
    EGameIcons_BClass = 0x4B
    EGameIcons_AClass = 0x4C
    EGameIcons_SClass = 0x4D
    EGameIcons_NoSaveWarning = 0x4E
    EGameIcons_ExploreMissionPlanetIcon = 0x4F
    EGameIcons_ExploreMissionSystemIcon = 0x50
    EGameIcons_PetThumbnailBG = 0x51
    EGameIcons_SettlementOSD = 0x52
    EGameIcons_Stealth = 0x53
    EGameIcons_StealthEmpty = 0x54
    EGameIcons_DefenceForce = 0x55
    EGameIcons_NumTypes = 0x56

class eSubstanceCategory(IntEnum):
    ESubstanceCategory_Fuel = 0x0
    ESubstanceCategory_Metal = 0x1
    ESubstanceCategory_Catalyst = 0x2
    ESubstanceCategory_Stellar = 0x3
    ESubstanceCategory_Flora = 0x4
    ESubstanceCategory_Earth = 0x5
    ESubstanceCategory_Exotic = 0x6
    ESubstanceCategory_Special = 0x7
    ESubstanceCategory_BuildingPart = 0x8
    ESubstanceCategory_NumTypes = 0x9
    ESubstanceCategory_Unspecified = 0xFFFFFFFF

class eEncounterType(IntEnum):
    EEncounterType_FactoryGuards = 0x0
    EEncounterType_HarvesterGuards = 0x1
    EEncounterType_ScrapHeap = 0x2
    EEncounterType_Reward = 0x3
    EEncounterType_CorruptedDroneInteract = 0x4
    EEncounterType_GroundWorms = 0x5
    EEncounterType_DroneHiveGuards = 0x6
    EEncounterType_NumTypes = 0x7

class eRarity(IntEnum):
    ERarity_Common = 0x0
    ERarity_Uncommon = 0x1
    ERarity_Rare = 0x2
    ERarity_NumTypes = 0x3
    ERarity_Unspecified = 0xFFFFFFFF

class eTileType(IntEnum):
    ETileType_Air = 0x0
    ETileType_Base = 0x1
    ETileType_Rock = 0x2
    ETileType_Mountain = 0x3
    ETileType_Underwater = 0x4
    ETileType_Cave = 0x5
    ETileType_Dirt = 0x6
    ETileType_Liquid = 0x7
    ETileType_Substance = 0x8
    ETileType_NumTypes = 0x9

class eCreatureType(IntEnum):
    ECreatureType_None = 0x0
    ECreatureType_Bird = 0x1
    ECreatureType_FlyingLizard = 0x2
    ECreatureType_FlyingSnake = 0x3
    ECreatureType_Butterfly = 0x4
    ECreatureType_FlyingBeetle = 0x5
    ECreatureType_Beetle = 0x6
    ECreatureType_Fish = 0x7
    ECreatureType_Shark = 0x8
    ECreatureType_Crab = 0x9
    ECreatureType_Snake = 0xA
    ECreatureType_Dino = 0xB
    ECreatureType_Antelope = 0xC
    ECreatureType_Rodent = 0xD
    ECreatureType_Cat = 0xE
    ECreatureType_Fiend = 0xF
    ECreatureType_Drone = 0x10
    ECreatureType_Quad = 0x11
    ECreatureType_Walker = 0x12
    ECreatureType_Predator = 0x13
    ECreatureType_PlayerPredator = 0x14
    ECreatureType_Prey = 0x15
    ECreatureType_Passive = 0x16
    ECreatureType_FishPredator = 0x17
    ECreatureType_FishPrey = 0x18
    ECreatureType_FiendFishSmall = 0x19
    ECreatureType_FiendFishBig = 0x1A
    ECreatureType_Jellyfish = 0x1B
    ECreatureType_LandJellyfish = 0x1C
    ECreatureType_RockCreature = 0x1D
    ECreatureType_MiniFiend = 0x1E
    ECreatureType_Floater = 0x1F
    ECreatureType_Scuttler = 0x20
    ECreatureType_Slug = 0x21
    ECreatureType_MiniDrone = 0x22
    ECreatureType_MiniRobo = 0x23
    ECreatureType_SpaceFloater = 0x24
    ECreatureType_Weird = 0x25
    ECreatureType_SeaSnake = 0x26
    ECreatureType_SandWorm = 0x27
    ECreatureType_ProtoRoller = 0x28
    ECreatureType_ProtoFlyer = 0x29
    ECreatureType_ProtoDigger = 0x2A
    ECreatureType_Plough = 0x2B
    ECreatureType_Digger = 0x2C
    ECreatureType_Drill = 0x2D
    ECreatureType_Brainless = 0x2E
    ECreatureType_Pet = 0x2F
    ECreatureType_NumTypes = 0x30

class eResourceLevel(IntEnum):
    EResourceLevel_Low = 0x0
    EResourceLevel_High = 0x1
    EResourceLevel_NumTypes = 0x2

class eSentinelMechWeaponMode(IntEnum):
    ESentinelMechWeaponMode_Gun = 0x0
    ESentinelMechWeaponMode_Canon = 0x1
    ESentinelMechWeaponMode_Flamethrower = 0x2
    ESentinelMechWeaponMode_NumTypes = 0x3

class eCreatureRole(IntEnum):
    ECreatureRole_None = 0x0
    ECreatureRole_Predator = 0x1
    ECreatureRole_PlayerPredator = 0x2
    ECreatureRole_Prey = 0x3
    ECreatureRole_Passive = 0x4
    ECreatureRole_Bird = 0x5
    ECreatureRole_FishPrey = 0x6
    ECreatureRole_FishPredator = 0x7
    ECreatureRole_Butterfly = 0x8
    ECreatureRole_Robot = 0x9
    ECreatureRole_Pet = 0xA
    ECreatureRole_NumTypes = 0xB

class eAsteroidLevel(IntEnum):
    EAsteroidLevel_NoRares = 0x0
    EAsteroidLevel_SomeRares = 0x1
    EAsteroidLevel_LotsOfRares = 0x2
    EAsteroidLevel_NumTypes = 0x3

class eImpact(IntEnum):
    EImpact_Default = 0x0
    EImpact_Terrain = 0x1
    EImpact_Substance = 0x2
    EImpact_Rock = 0x3
    EImpact_Asteroid = 0x4
    EImpact_Shield = 0x5
    EImpact_Creature = 0x6
    EImpact_Robot = 0x7
    EImpact_Freighter = 0x8
    EImpact_Cargo = 0x9
    EImpact_Ship = 0xA
    EImpact_Plant = 0xB
    EImpact_NeedsTech = 0xC
    EImpact_Player = 0xD
    EImpact_OtherPlayer = 0xE
    EImpact_SentinelShield = 0xF
    EImpact_SpaceshipShield = 0x10
    EImpact_NumTypes = 0x11

class eCreatureHemiSphere(IntEnum):
    ECreatureHemiSphere_Any = 0x0
    ECreatureHemiSphere_Northern = 0x1
    ECreatureHemiSphere_Southern = 0x2
    ECreatureHemiSphere_NumTypes = 0x3

class eDroneType(IntEnum):
    EDroneType_Patrol = 0x0
    EDroneType_Combat = 0x1
    EDroneType_Corrupted = 0x2
    EDroneType_NumTypes = 0x3

class eImpactAlignment(IntEnum):
    EImpactAlignment_ImpactNormal = 0x0
    EImpactAlignment_GravityUp = 0x1
    EImpactAlignment_NumTypes = 0x2

class eImpactAttachment(IntEnum):
    EImpactAttachment_World = 0x0
    EImpactAttachment_HitBody = 0x1
    EImpactAttachment_NumTypes = 0x2

class eCombatEffectType(IntEnum):
    ECombatEffectType_None = 0x0
    ECombatEffectType_Fire = 0x1
    ECombatEffectType_Stun = 0x2
    ECombatEffectType_Slow = 0x3
    ECombatEffectType_NumTypes = 0x4

class eFiendCrime(IntEnum):
    EFiendCrime_None = 0x0
    EFiendCrime_EggDamaged = 0x1
    EFiendCrime_EggDestroyed = 0x2
    EFiendCrime_EggCollected = 0x3
    EFiendCrime_UnderwaterPropDamaged = 0x4
    EFiendCrime_UnderwaterPropCollected = 0x5
    EFiendCrime_RockTransform = 0x6
    EFiendCrime_GroundPropDamage = 0x7
    EFiendCrime_ShotWorm = 0x8
    EFiendCrime_NumTypes = 0x9

class eHazardValue(IntEnum):
    EHazardValue_Ambient = 0x0
    EHazardValue_Water = 0x1
    EHazardValue_Cave = 0x2
    EHazardValue_Storm = 0x3
    EHazardValue_Night = 0x4
    EHazardValue_NumTypes = 0x5

class eDamageType(IntEnum):
    EDamageType_Gun = 0x0
    EDamageType_Laser = 0x1
    EDamageType_Shotgun = 0x2
    EDamageType_Burst = 0x3
    EDamageType_Rail = 0x4
    EDamageType_Cannon = 0x5
    EDamageType_Explosion = 0x6
    EDamageType_Melee = 0x7
    EDamageType_ShipGun = 0x8
    EDamageType_ShipLaser = 0x9
    EDamageType_ShipShotgun = 0xA
    EDamageType_ShipMinigun = 0xB
    EDamageType_ShipRockets = 0xC
    EDamageType_ShipPlasma = 0xD
    EDamageType_VehicleGun = 0xE
    EDamageType_VehicleLaser = 0xF
    EDamageType_SentinelLaser = 0x10
    EDamageType_PlayerDamage = 0x11
    EDamageType_PlayerWeapons = 0x12
    EDamageType_ShipWeapons = 0x13
    EDamageType_VehicleWeapons = 0x14
    EDamageType_CombatEffects = 0x15
    EDamageType_Fiend = 0x16
    EDamageType_NumTypes = 0x17

class eCreatureSizeClass(IntEnum):
    ECreatureSizeClass_Small = 0x0
    ECreatureSizeClass_Medium = 0x1
    ECreatureSizeClass_Large = 0x2
    ECreatureSizeClass_Huge = 0x3
    ECreatureSizeClass_NumTypes = 0x4

class eDensity(IntEnum):
    EDensity_Sparse = 0x0
    EDensity_Normal = 0x1
    EDensity_Dense = 0x2
    EDensity_VeryDense = 0x3
    EDensity_NumTypes = 0x4

class eCreatureActiveTime(IntEnum):
    ECreatureActiveTime_OnlyDay = 0x0
    ECreatureActiveTime_MostlyDay = 0x1
    ECreatureActiveTime_AnyTime = 0x2
    ECreatureActiveTime_MostlyNight = 0x3
    ECreatureActiveTime_OnlyNight = 0x4
    ECreatureActiveTime_NumTypes = 0x5

class eBuildingDensity(IntEnum):
    EBuildingDensity_Dead = 0x0
    EBuildingDensity_Low = 0x1
    EBuildingDensity_Mid = 0x2
    EBuildingDensity_Full = 0x3
    EBuildingDensity_Weird = 0x4
    EBuildingDensity_HalfWeird = 0x5
    EBuildingDensity_NumTypes = 0x6

class eGcStatEnum(IntEnum):
    EGcStatEnum_None = 0x0
    EGcStatEnum_DEPOTS_BROKEN = 0x1
    EGcStatEnum_FPODS_BROKEN = 0x2
    EGcStatEnum_PLANTS_PLANTED = 0x3
    EGcStatEnum_SALVAGE_LOOTED = 0x4
    EGcStatEnum_TREASURE_FOUND = 0x5
    EGcStatEnum_QUADS_KILLED = 0x6
    EGcStatEnum_WALKERS_KILLED = 0x7
    EGcStatEnum_FLORA_KILLED = 0x8
    EGcStatEnum_PLANTS_GATHERED = 0x9
    EGcStatEnum_BONES_FOUND = 0xA
    EGcStatEnum_C_SENT_KILLS = 0xB
    EGcStatEnum_STORM_CRYSTALS = 0xC
    EGcStatEnum_BURIED_PROPS = 0xD
    EGcStatEnum_MINIWORM_KILL = 0xE
    EGcStatEnum_POOP_COLLECTED = 0xF
    EGcStatEnum_GRAVBALLS = 0x10
    EGcStatEnum_EGG_PODS = 0x11
    EGcStatEnum_NumTypes = 0x12

class eCloudiness(IntEnum):
    ECloudiness_CloudyWithClearSpells = 0x0
    ECloudiness_ClearWithCloudySpells = 0x1
    ECloudiness_NumTypes = 0x2

class eType(IntEnum):
    EType_Instanced = 0x0
    EType_Single = 0x1
    EType_NumTypes = 0x2

class ePlacementPriority(IntEnum):
    EPlacementPriority_Low = 0x0
    EPlacementPriority_Normal = 0x1
    EPlacementPriority_High = 0x2
    EPlacementPriority_NumTypes = 0x3

class eLargeObjectCoverage(IntEnum):
    ELargeObjectCoverage_DoNotPlace = 0x0
    ELargeObjectCoverage_DoNotPlaceIgnoreFootprint = 0x1
    ELargeObjectCoverage_DoNotPlaceClose = 0x2
    ELargeObjectCoverage_DoNotPlaceAnywhereNear = 0x3
    ELargeObjectCoverage_OnlyPlaceAround = 0x4
    ELargeObjectCoverage_OnlyPlaceAroundIgnoreFootprint = 0x5
    ELargeObjectCoverage_AlwaysPlace = 0x6
    ELargeObjectCoverage_NumTypes = 0x7

class eOverlapStyle(IntEnum):
    EOverlapStyle_None = 0x0
    EOverlapStyle_SameSeed = 0x1
    EOverlapStyle_All = 0x2
    EOverlapStyle_NumTypes = 0x3

class eGroundColourIndex(IntEnum):
    EGroundColourIndex_Auto = 0x0
    EGroundColourIndex_Main = 0x1
    EGroundColourIndex_Patch = 0x2
    EGroundColourIndex_NumTypes = 0x3

class exBehaviourFlags(IntEnum):
    FBehaviourFlags_None = 0x0
    FBehaviourFlags_DestroyTerrain = 0x1
    FBehaviourFlags_DestroyAsteroids = 0x2
    FBehaviourFlags_GatherResources = 0x4
    FBehaviourFlags_Homing = 0x8
    FBehaviourFlags_HomingLaser = 0x10
    FBehaviourFlags_ScareCreatures = 0x20
    FBehaviourFlags_ExplosionForce = 0x40
    FBehaviourFlags_Mask = 0x7F

class eClass(IntEnum):
    EClass_Player = 0x0
    EClass_PlayerShip = 0x1
    EClass_Ship = 0x2
    EClass_Robot = 0x3
    EClass_NumTypes = 0x4

class ePetMood(IntEnum):
    EPetMood_Hungry = 0x0
    EPetMood_Lonely = 0x1
    EPetMood_NumTypes = 0x2

class eTechnologyCategory(IntEnum):
    ETechnologyCategory_Ship = 0x0
    ETechnologyCategory_Weapon = 0x1
    ETechnologyCategory_Suit = 0x2
    ETechnologyCategory_Personal = 0x3
    ETechnologyCategory_All = 0x4
    ETechnologyCategory_None = 0x5
    ETechnologyCategory_Freighter = 0x6
    ETechnologyCategory_Maintenance = 0x7
    ETechnologyCategory_Exocraft = 0x8
    ETechnologyCategory_Submarine = 0x9
    ETechnologyCategory_Mech = 0xA
    ETechnologyCategory_AllVehicles = 0xB
    ETechnologyCategory_AlienShip = 0xC
    ETechnologyCategory_AllShips = 0xD
    ETechnologyCategory_NumTypes = 0xE
    ETechnologyCategory_Unspecified = 0xFFFFFFFF

class eMissionType(IntEnum):
    EMissionType_SpaceCombat = 0x0
    EMissionType_GroundCombat = 0x1
    EMissionType_Research = 0x2
    EMissionType_MissingPerson = 0x3
    EMissionType_Repair = 0x4
    EMissionType_Cargo = 0x5
    EMissionType_Piracy = 0x6
    EMissionType_Photo = 0x7
    EMissionType_Feeding = 0x8
    EMissionType_Planting = 0x9
    EMissionType_Construction = 0xA
    EMissionType_NumTypes = 0xB

class ePersistentBaseTypes(IntEnum):
    EPersistentBaseTypes_HomePlanetBase = 0x0
    EPersistentBaseTypes_FreighterBase = 0x1
    EPersistentBaseTypes_ExternalPlanetBase = 0x2
    EPersistentBaseTypes_CivilianFreighterBase = 0x3
    EPersistentBaseTypes_FriendsPlanetBase = 0x4
    EPersistentBaseTypes_FriendsFreighterBase = 0x5
    EPersistentBaseTypes_SpaceBase = 0x6
    EPersistentBaseTypes_GeneratedPlanetBase = 0x7
    EPersistentBaseTypes_GeneratedPlanetBaseEdits = 0x8
    EPersistentBaseTypes_NumTypes = 0x9

class eBuildPartSearchType(IntEnum):
    EBuildPartSearchType_Base = 0x0
    EBuildPartSearchType_Freighter = 0x1
    EBuildPartSearchType_AllPlayerOwned = 0x2
    EBuildPartSearchType_OtherPlayerBase = 0x3
    EBuildPartSearchType_NumTypes = 0x4

class eScanEventGPSHint(IntEnum):
    EScanEventGPSHint_None = 0x0
    EScanEventGPSHint_Accurate = 0x1
    EScanEventGPSHint_OffsetNarrow = 0x2
    EScanEventGPSHint_OffsetMid = 0x3
    EScanEventGPSHint_OffsetWide = 0x4
    EScanEventGPSHint_Obfuscated = 0x5
    EScanEventGPSHint_PartObfuscated = 0x6
    EScanEventGPSHint_NumTypes = 0x7

class eAge(IntEnum):
    EAge_Regular = 0x0
    EAge_Weird = 0x1
    EAge_NumTypes = 0x2

class eAnimationIntensity(IntEnum):
    EAnimationIntensity_Low = 0x0
    EAnimationIntensity_Medium = 0x1
    EAnimationIntensity_High = 0x2
    EAnimationIntensity_None = 0x3
    EAnimationIntensity_NumTypes = 0x4

class eMissionCategory(IntEnum):
    EMissionCategory_Info = 0x0
    EMissionCategory_Mission = 0x1
    EMissionCategory_Danger = 0x2
    EMissionCategory_Urgent = 0x3
    EMissionCategory_NumTypes = 0x4

class eNPCProp(IntEnum):
    ENPCProp_None = 0x0
    ENPCProp_IPad = 0x1
    ENPCProp_HoloBlob = 0x2
    ENPCProp_HoloFrigate = 0x3
    ENPCProp_HoloShip = 0x4
    ENPCProp_HoloMultitool = 0x5
    ENPCProp_HoloSolarSystem = 0x6
    ENPCProp_Container = 0x7
    ENPCProp_Box = 0x8
    ENPCProp_Cup = 0x9
    ENPCProp_DontCare = 0xA
    ENPCProp_RandomHologram = 0xB
    ENPCProp_HoloDrone = 0xC
    ENPCProp_NumTypes = 0xD

class eConditionTest(IntEnum):
    EConditionTest_AnyFalse = 0x0
    EConditionTest_AllFalse = 0x1
    EConditionTest_AnyTrue = 0x2
    EConditionTest_AllTrue = 0x3
    EConditionTest_NumTypes = 0x4

class eNPCSeatedPosture(IntEnum):
    ENPCSeatedPosture_Sofa = 0x0
    ENPCSeatedPosture_Sit = 0x1
    ENPCSeatedPosture_NumTypes = 0x2

class eRequireMode(IntEnum):
    ERequireMode_Seated = 0x0
    ERequireMode_Standing = 0x1
    ERequireMode_None = 0x2
    ERequireMode_NumTypes = 0x3

class eMood(IntEnum):
    EMood_Neutral = 0x0
    EMood_Positive = 0x1
    EMood_VeryPositive = 0x2
    EMood_Negative = 0x3
    EMood_VeryNegative = 0x4
    EMood_Pity = 0x5
    EMood_Sad = 0x6
    EMood_Dead = 0x7
    EMood_Confused = 0x8
    EMood_NumTypes = 0x9

class eMissionDifficulty(IntEnum):
    EMissionDifficulty_Easy = 0x0
    EMissionDifficulty_Normal = 0x1
    EMissionDifficulty_Hard = 0x2
    EMissionDifficulty_NumTypes = 0x3

class eMissionFaction(IntEnum):
    EMissionFaction_Gek = 0x0
    EMissionFaction_Korvax = 0x1
    EMissionFaction_Vykeen = 0x2
    EMissionFaction_TradeGuild = 0x3
    EMissionFaction_WarriorGuild = 0x4
    EMissionFaction_ExplorerGuild = 0x5
    EMissionFaction_Nexus = 0x6
    EMissionFaction_Pirates = 0x7
    EMissionFaction_None = 0x8
    EMissionFaction_NumTypes = 0x9

class egcwordcategorytableEnum(IntEnum):
    EgcwordcategorytableEnum_MISC = 0x0
    EgcwordcategorytableEnum_DIRECTIONS = 0x1
    EgcwordcategorytableEnum_HELP = 0x2
    EgcwordcategorytableEnum_TRADE = 0x3
    EgcwordcategorytableEnum_LORE = 0x4
    EgcwordcategorytableEnum_TECH = 0x5
    EgcwordcategorytableEnum_THREAT = 0x6
    EgcwordcategorytableEnum_NumTypes = 0x7

class eGalacticFeature(IntEnum):
    EGalacticFeature_Anomaly = 0x0
    EGalacticFeature_Atlas = 0x1
    EGalacticFeature_BlackHole = 0x2
    EGalacticFeature_NumTypes = 0x3

class eNPCSettlementBehaviourAreaProperty(IntEnum):
    ENPCSettlementBehaviourAreaProperty_ContainsPlayer = 0x0
    ENPCSettlementBehaviourAreaProperty_ContainsNPCs = 0x1
    ENPCSettlementBehaviourAreaProperty_NumTypes = 0x2

class eGalacticPoint(IntEnum):
    EGalacticPoint_Atlas = 0x0
    EGalacticPoint_BlackHole = 0x1
    EGalacticPoint_NumTypes = 0x2

class eMissionPageHint(IntEnum):
    EMissionPageHint_None = 0x0
    EMissionPageHint_Suit = 0x1
    EMissionPageHint_Ship = 0x2
    EMissionPageHint_Weapon = 0x3
    EMissionPageHint_Vehicle = 0x4
    EMissionPageHint_Freighter = 0x5
    EMissionPageHint_Wiki = 0x6
    EMissionPageHint_Catalogue = 0x7
    EMissionPageHint_MissionLog = 0x8
    EMissionPageHint_Discovery = 0x9
    EMissionPageHint_Journey = 0xA
    EMissionPageHint_Expedition = 0xB
    EMissionPageHint_NumTypes = 0xC

class eNPCInteractiveObjectType(IntEnum):
    ENPCInteractiveObjectType_Idle = 0x0
    ENPCInteractiveObjectType_Generic = 0x1
    ENPCInteractiveObjectType_Chair = 0x2
    ENPCInteractiveObjectType_Conversation = 0x3
    ENPCInteractiveObjectType_WatchShip = 0x4
    ENPCInteractiveObjectType_Shop = 0x5
    ENPCInteractiveObjectType_Dance = 0x6
    ENPCInteractiveObjectType_None = 0x7
    ENPCInteractiveObjectType_NumTypes = 0x8

class eMilestone(IntEnum):
    EMilestone_Bronze = 0x0
    EMilestone_Silver = 0x1
    EMilestone_Gold = 0x2
    EMilestone_NumTypes = 0x3

class ePhotoBuildingType(IntEnum):
    EPhotoBuildingType_Shelter = 0x0
    EPhotoBuildingType_Abandoned = 0x1
    EPhotoBuildingType_Shop = 0x2
    EPhotoBuildingType_Outpost = 0x3
    EPhotoBuildingType_RadioTower = 0x4
    EPhotoBuildingType_Observatory = 0x5
    EPhotoBuildingType_Depot = 0x6
    EPhotoBuildingType_Monolith = 0x7
    EPhotoBuildingType_Factory = 0x8
    EPhotoBuildingType_Portal = 0x9
    EPhotoBuildingType_Ruin = 0xA
    EPhotoBuildingType_MissionTower = 0xB
    EPhotoBuildingType_LargeBuilding = 0xC
    EPhotoBuildingType_NumTypes = 0xD

class ePetTrait(IntEnum):
    EPetTrait_Helpfulness = 0x0
    EPetTrait_Aggression = 0x1
    EPetTrait_Independence = 0x2
    EPetTrait_NumTypes = 0x3

class ePhotoCreatureType(IntEnum):
    EPhotoCreatureType_Ground = 0x0
    EPhotoCreatureType_Water = 0x1
    EPhotoCreatureType_Air = 0x2
    EPhotoCreatureType_NumTypes = 0x3

class ePhotoPlantType(IntEnum):
    EPhotoPlantType_Sodium = 0x0
    EPhotoPlantType_Oxygen = 0x1
    EPhotoPlantType_BluePlant = 0x2
    EPhotoPlantType_NumTypes = 0x3

class eGrabType(IntEnum):
    EGrabType_Default = 0x0
    EGrabType_EjectHandle = 0x1
    EGrabType_ControlStickLeft = 0x2
    EGrabType_ControlStickRight = 0x3
    EGrabType_NumTypes = 0x4

class eNPCSettlementBehaviourState(IntEnum):
    ENPCSettlementBehaviourState_Generic = 0x0
    ENPCSettlementBehaviourState_Sociable = 0x1
    ENPCSettlementBehaviourState_Productive = 0x2
    ENPCSettlementBehaviourState_Tired = 0x3
    ENPCSettlementBehaviourState_Afraid = 0x4
    ENPCSettlementBehaviourState_NumTypes = 0x5

class eScanType(IntEnum):
    EScanType_Tool = 0x0
    EScanType_Beacon = 0x1
    EScanType_RadioTower = 0x2
    EScanType_Observatory = 0x3
    EScanType_DistressSignal = 0x4
    EScanType_Waypoint = 0x5
    EScanType_Ship = 0x6
    EScanType_DebugPlanet = 0x7
    EScanType_DebugSpace = 0x8
    EScanType_VisualOnly = 0x9
    EScanType_VisualOnlyAerial = 0xA
    EScanType_NumTypes = 0xB

class eCharacterState(IntEnum):
    ECharacterState_Idle = 0x0
    ECharacterState_Jog = 0x1
    ECharacterState_JogUphill = 0x2
    ECharacterState_JogDownhill = 0x3
    ECharacterState_SteepSlope = 0x4
    ECharacterState_Sliding = 0x5
    ECharacterState_Run = 0x6
    ECharacterState_Airborne = 0x7
    ECharacterState_JetpackBoost = 0x8
    ECharacterState_RocketBoots = 0x9
    ECharacterState_Riding = 0xA
    ECharacterState_Swimming = 0xB
    ECharacterState_SwimmingJetpack = 0xC
    ECharacterState_Death = 0xD
    ECharacterState_FullBodyOverride = 0xE
    ECharacterState_InSpace = 0xF
    ECharacterState_LowGWalk = 0x10
    ECharacterState_LowGRun = 0x11
    ECharacterState_NumTypes = 0x12

class eZoomType(IntEnum):
    EZoomType_None = 0x0
    EZoomType_Far = 0x1
    EZoomType_Mid = 0x2
    EZoomType_Close = 0x3
    EZoomType_NumTypes = 0x4

class eScanEffectType(IntEnum):
    EScanEffectType_Building = 0x0
    EScanEffectType_TargetShip = 0x1
    EScanEffectType_Creature = 0x2
    EScanEffectType_Ground = 0x3
    EScanEffectType_Objects = 0x4
    EScanEffectType_NumTypes = 0x5

class eHazard(IntEnum):
    EHazard_None = 0x0
    EHazard_NoOxygen = 0x1
    EHazard_ExtremeHeat = 0x2
    EHazard_ExtremeCold = 0x3
    EHazard_ToxicGas = 0x4
    EHazard_Radiation = 0x5
    EHazard_NumTypes = 0x6

class eGlobalMission(IntEnum):
    EGlobalMission_Atlas = 0x0
    EGlobalMission_BlackHole = 0x1
    EGlobalMission_Anomaly = 0x2
    EGlobalMission_Explore = 0x3
    EGlobalMission_NumTypes = 0x4

class eKillTarget(IntEnum):
    EKillTarget_Robots = 0x0
    EKillTarget_Drones = 0x1
    EKillTarget_Quads = 0x2
    EKillTarget_Walkers = 0x3
    EKillTarget_Predators = 0x4
    EKillTarget_Creatures = 0x5
    EKillTarget_Pirates = 0x6
    EKillTarget_Traders = 0x7
    EKillTarget_Fiends = 0x8
    EKillTarget_HazardousFlora = 0x9
    EKillTarget_Worms = 0xA
    EKillTarget_NumTypes = 0xB

class eOSDMessageStyle(IntEnum):
    EOSDMessageStyle_Standard = 0x0
    EOSDMessageStyle_Fancy = 0x1
    EOSDMessageStyle_Stats = 0x2
    EOSDMessageStyle_Settlement = 0x3
    EOSDMessageStyle_NumTypes = 0x4

class eModifyType(IntEnum):
    EModifyType_Set = 0x0
    EModifyType_Add = 0x1
    EModifyType_Subtract = 0x2
    EModifyType_NumTypes = 0x3

class eMissionUpdateMessage(IntEnum):
    EMissionUpdateMessage_Start = 0x0
    EMissionUpdateMessage_End = 0x1
    EMissionUpdateMessage_NumTypes = 0x2

class ePlayMusicSting(IntEnum):
    EPlayMusicSting_None = 0x0
    EPlayMusicSting_Start = 0x1
    EPlayMusicSting_End = 0x2
    EPlayMusicSting_NumTypes = 0x3

class eDefaultItemTypeForInitialWarp(IntEnum):
    EDefaultItemTypeForInitialWarp_None = 0x0
    EDefaultItemTypeForInitialWarp_PrimaryProduct = 0x1
    EDefaultItemTypeForInitialWarp_PrimarySubstance = 0x2
    EDefaultItemTypeForInitialWarp_SecondaryProduct = 0x3
    EDefaultItemTypeForInitialWarp_SecondarySubstance = 0x4
    EDefaultItemTypeForInitialWarp_NumTypes = 0x5

class eRewardInventoryOverride(IntEnum):
    ERewardInventoryOverride_None = 0x0
    ERewardInventoryOverride_Suit = 0x1
    ERewardInventoryOverride_Ship = 0x2
    ERewardInventoryOverride_Vehicle = 0x3
    ERewardInventoryOverride_Freighter = 0x4
    ERewardInventoryOverride_NumTypes = 0x5

class eMissionMarkup(IntEnum):
    EMissionMarkup_KillFiend = 0x0
    EMissionMarkup_KillPirate = 0x1
    EMissionMarkup_KillSentinel = 0x2
    EMissionMarkup_KillHazardousPlants = 0x3
    EMissionMarkup_KillTraders = 0x4
    EMissionMarkup_KillCreatures = 0x5
    EMissionMarkup_KillPredators = 0x6
    EMissionMarkup_KillDepot = 0x7
    EMissionMarkup_KillWorms = 0x8
    EMissionMarkup_FeedCreature = 0x9
    EMissionMarkup_CollectBones = 0xA
    EMissionMarkup_CollectScrap = 0xB
    EMissionMarkup_Discover = 0xC
    EMissionMarkup_CollectSubstanceProduct = 0xD
    EMissionMarkup_Build = 0xE
    EMissionMarkup_Always = 0xF
    EMissionMarkup_None = 0x10
    EMissionMarkup_NumTypes = 0x11
    EMissionMarkup_Unspecified = 0xFFFFFFFF

class eScanTable(IntEnum):
    EScanTable_Space = 0x0
    EScanTable_Planet = 0x1
    EScanTable_Missions = 0x2
    EScanTable_Tutorial = 0x3
    EScanTable_MissionsCreative = 0x4
    EScanTable_Vehicle = 0x5
    EScanTable_NPCPlanetSite = 0x6
    EScanTable_Seasonal = 0x7
    EScanTable_NumTypes = 0x8

class eAlienPuzzleCategory(IntEnum):
    EAlienPuzzleCategory_Default = 0x0
    EAlienPuzzleCategory_GuildTraderNone = 0x1
    EAlienPuzzleCategory_GuildTraderLow = 0x2
    EAlienPuzzleCategory_GuildTraderMed = 0x3
    EAlienPuzzleCategory_GuildTraderHigh = 0x4
    EAlienPuzzleCategory_GuildTraderBest = 0x5
    EAlienPuzzleCategory_GuildWarriorNone = 0x6
    EAlienPuzzleCategory_GuildWarriorLow = 0x7
    EAlienPuzzleCategory_GuildWarriorMed = 0x8
    EAlienPuzzleCategory_GuildWarriorHigh = 0x9
    EAlienPuzzleCategory_GuildWarriorBest = 0xA
    EAlienPuzzleCategory_GuildExplorerNone = 0xB
    EAlienPuzzleCategory_GuildExplorerLow = 0xC
    EAlienPuzzleCategory_GuildExplorerMed = 0xD
    EAlienPuzzleCategory_GuildExplorerHigh = 0xE
    EAlienPuzzleCategory_GuildExplorerBest = 0xF
    EAlienPuzzleCategory_BiomeHot = 0x10
    EAlienPuzzleCategory_BiomeCold = 0x11
    EAlienPuzzleCategory_BiomeLush = 0x12
    EAlienPuzzleCategory_BiomeDusty = 0x13
    EAlienPuzzleCategory_BiomeTox = 0x14
    EAlienPuzzleCategory_BiomeRad = 0x15
    EAlienPuzzleCategory_BiomeWeird = 0x16
    EAlienPuzzleCategory_LocationSpaceStation = 0x17
    EAlienPuzzleCategory_LocationShop = 0x18
    EAlienPuzzleCategory_LocationOutpost = 0x19
    EAlienPuzzleCategory_LocationObservatory = 0x1A
    EAlienPuzzleCategory_Walking = 0x1B
    EAlienPuzzleCategory_ExtremeWeather = 0x1C
    EAlienPuzzleCategory_ExtremeSentinels = 0x1D
    EAlienPuzzleCategory_WaterPlanet = 0x1E
    EAlienPuzzleCategory_FreighterCrew = 0x1F
    EAlienPuzzleCategory_FreighterCrewOwned = 0x20
    EAlienPuzzleCategory_ShipShop = 0x21
    EAlienPuzzleCategory_SuitShop = 0x22
    EAlienPuzzleCategory_WeapShop = 0x23
    EAlienPuzzleCategory_VehicleShop = 0x24
    EAlienPuzzleCategory_MoodVeryPositive = 0x25
    EAlienPuzzleCategory_MoodPositive = 0x26
    EAlienPuzzleCategory_MoodNeutral = 0x27
    EAlienPuzzleCategory_MoodNegative = 0x28
    EAlienPuzzleCategory_MoodVeryNegative = 0x29
    EAlienPuzzleCategory_Proc = 0x2A
    EAlienPuzzleCategory_FirstAbandonedFreighter = 0x2B
    EAlienPuzzleCategory_StandardAbandonedFreighter = 0x2C
    EAlienPuzzleCategory_BiomeSwamp = 0x2D
    EAlienPuzzleCategory_BiomeLava = 0x2E
    EAlienPuzzleCategory_AbandonedSystem = 0x2F
    EAlienPuzzleCategory_InhabitedSystem = 0x30
    EAlienPuzzleCategory_SettlementOwned = 0x31
    EAlienPuzzleCategory_SettlementNotOwned = 0x32
    EAlienPuzzleCategory_PirateStation = 0x33
    EAlienPuzzleCategory_StandardPilot = 0x34
    EAlienPuzzleCategory_NumTypes = 0x35

class eParticipantType(IntEnum):
    EParticipantType_None = 0x0
    EParticipantType_MissionGiver = 0x1
    EParticipantType_MissionGiverReference = 0x2
    EParticipantType_Primary = 0x3
    EParticipantType_Secondary1 = 0x4
    EParticipantType_Secondary2 = 0x5
    EParticipantType_Secondary3 = 0x6
    EParticipantType_Secondary4 = 0x7
    EParticipantType_Secondary5 = 0x8
    EParticipantType_Secondary6 = 0x9
    EParticipantType_Secondary7 = 0xA
    EParticipantType_Secondary8 = 0xB
    EParticipantType_Secondary9 = 0xC
    EParticipantType_NumTypes = 0xD

class eFactionOption(IntEnum):
    EFactionOption_DataDefined = 0x0
    EFactionOption_CurrentMission = 0x1
    EFactionOption_CurrentSystem = 0x2
    EFactionOption_NumTypes = 0x3

class eMissionClass(IntEnum):
    EMissionClass_Primary = 0x0
    EMissionClass_Secondary = 0x1
    EMissionClass_ChainedSecondary = 0x2
    EMissionClass_Guide = 0x3
    EMissionClass_Wiki = 0x4
    EMissionClass_Seasonal = 0x5
    EMissionClass_Milestone = 0x6
    EMissionClass_NumTypes = 0x7

class eMessageComplete(IntEnum):
    EMessageComplete_Default = 0x0
    EMessageComplete_Always = 0x1
    EMessageComplete_Never = 0x2
    EMessageComplete_NumTypes = 0x3

class eMessageStart(IntEnum):
    EMessageStart_Default = 0x0
    EMessageStart_Always = 0x1
    EMessageStart_Never = 0x2
    EMessageStart_NumTypes = 0x3

class eAutoStart(IntEnum):
    EAutoStart_None = 0x0
    EAutoStart_AllModes = 0x1
    EAutoStart_Seasonal = 0x2
    EAutoStart_OnSelected = 0x3
    EAutoStart_NumTypes = 0x4

class eGalaxyStarAnomaly(IntEnum):
    EGalaxyStarAnomaly_None = 0x0
    EGalaxyStarAnomaly_AtlasStation = 0x1
    EGalaxyStarAnomaly_AtlasStationFinal = 0x2
    EGalaxyStarAnomaly_BlackHole = 0x3
    EGalaxyStarAnomaly_MiniStation = 0x4
    EGalaxyStarAnomaly_NumTypes = 0x5

class eTeleporterDestinationType(IntEnum):
    ETeleporterDestinationType_Base = 0x0
    ETeleporterDestinationType_Spacestation = 0x1
    ETeleporterDestinationType_Atlas = 0x2
    ETeleporterDestinationType_PlanetAwayFromShip = 0x3
    ETeleporterDestinationType_ExternalBase = 0x4
    ETeleporterDestinationType_EmergencyGalaxyFix = 0x5
    ETeleporterDestinationType_OnNexus = 0x6
    ETeleporterDestinationType_SpacestationFixPosition = 0x7
    ETeleporterDestinationType_Settlement = 0x8
    ETeleporterDestinationType_Freighter = 0x9
    ETeleporterDestinationType_NumTypes = 0xA

class eRepeatLogic(IntEnum):
    ERepeatLogic_None = 0x0
    ERepeatLogic_Loop = 0x1
    ERepeatLogic_RestartOnConditionFail = 0x2
    ERepeatLogic_NumTypes = 0x3

class eIconStyle(IntEnum):
    EIconStyle_Default = 0x0
    EIconStyle_Large = 0x1
    EIconStyle_Square = 0x2
    EIconStyle_NoFrame = 0x3
    EIconStyle_NumTypes = 0x4

class eDefaultProductType(IntEnum):
    EDefaultProductType_None = 0x0
    EDefaultProductType_PrimaryProduct = 0x1
    EDefaultProductType_SecondaryProduct = 0x2
    EDefaultProductType_NumTypes = 0x3

class eMonth(IntEnum):
    EMonth_January = 0x0
    EMonth_February = 0x1
    EMonth_March = 0x2
    EMonth_April = 0x3
    EMonth_May = 0x4
    EMonth_June = 0x5
    EMonth_July = 0x6
    EMonth_August = 0x7
    EMonth_September = 0x8
    EMonth_October = 0x9
    EMonth_November = 0xA
    EMonth_December = 0xB
    EMonth_NumTypes = 0xC

class eDay(IntEnum):
    EDay_Sunday = 0x0
    EDay_Monday = 0x1
    EDay_Tuesday = 0x2
    EDay_Wednesday = 0x3
    EDay_Thursday = 0x4
    EDay_Friday = 0x5
    EDay_Saturday = 0x6
    EDay_NumTypes = 0x7

class eDefaultSubstanceType(IntEnum):
    EDefaultSubstanceType_None = 0x0
    EDefaultSubstanceType_PrimarySubstance = 0x1
    EDefaultSubstanceType_SecondarySubstance = 0x2
    EDefaultSubstanceType_NumTypes = 0x3

class eItemPurpose(IntEnum):
    EItemPurpose_None = 0x0
    EItemPurpose_Crafting = 0x1
    EItemPurpose_Building = 0x2
    EItemPurpose_Repairing = 0x3
    EItemPurpose_Charging = 0x4
    EItemPurpose_Paying = 0x5
    EItemPurpose_NumTypes = 0x6
    EItemPurpose_Unspecified = 0xFFFFFFFF

class eScannerIconHighlightType(IntEnum):
    EScannerIconHighlightType_Diamond = 0x0
    EScannerIconHighlightType_Hexagon = 0x1
    EScannerIconHighlightType_Tag = 0x2
    EScannerIconHighlightType_Octagon = 0x3
    EScannerIconHighlightType_Circle = 0x4
    EScannerIconHighlightType_NumTypes = 0x5

class ePointState(IntEnum):
    EPointState_Statement = 0x0
    EPointState_ObjectiveIncomplete = 0x1
    EPointState_ObjectiveComplete = 0x2
    EPointState_NumTypes = 0x3

class eSettlementStatType(IntEnum):
    ESettlementStatType_Population = 0x0
    ESettlementStatType_Happiness = 0x1
    ESettlementStatType_Production = 0x2
    ESettlementStatType_Upkeep = 0x3
    ESettlementStatType_Sentinels = 0x4
    ESettlementStatType_Debt = 0x5
    ESettlementStatType_Alert = 0x6
    ESettlementStatType_NumTypes = 0x7

class eDiscoverTarget(IntEnum):
    EDiscoverTarget_Animal = 0x0
    EDiscoverTarget_Vegetable = 0x1
    EDiscoverTarget_Mineral = 0x2
    EDiscoverTarget_NumTypes = 0x3

class eSettlementJudgementType(IntEnum):
    ESettlementJudgementType_None = 0x0
    ESettlementJudgementType_StrangerVisit = 0x1
    ESettlementJudgementType_Policy = 0x2
    ESettlementJudgementType_NewBuilding = 0x3
    ESettlementJudgementType_BuildingChoice = 0x4
    ESettlementJudgementType_Conflict = 0x5
    ESettlementJudgementType_Request = 0x6
    ESettlementJudgementType_NumTypes = 0x7

class eDiscoverTargetOnThisPlanet(IntEnum):
    EDiscoverTargetOnThisPlanet_Animal = 0x0
    EDiscoverTargetOnThisPlanet_Vegetable = 0x1
    EDiscoverTargetOnThisPlanet_Mineral = 0x2
    EDiscoverTargetOnThisPlanet_NumTypes = 0x3

class eMenuToClose(IntEnum):
    EMenuToClose_QuickMenu = 0x0
    EMenuToClose_BuildMenu = 0x1
    EMenuToClose_Inventory = 0x2
    EMenuToClose_AllDetailMessages = 0x3
    EMenuToClose_NumTypes = 0x4

class eEngineStatus(IntEnum):
    EEngineStatus_Thrusting = 0x0
    EEngineStatus_Braking = 0x1
    EEngineStatus_Landing = 0x2
    EEngineStatus_Landed = 0x3
    EEngineStatus_Boosting = 0x4
    EEngineStatus_Pulsing = 0x5
    EEngineStatus_LowFlight = 0x6
    EEngineStatus_Inverted = 0x7
    EEngineStatus_NumTypes = 0x8

class eTranslateAlienTextOverride(IntEnum):
    ETranslateAlienTextOverride_None = 0x0
    ETranslateAlienTextOverride_Translate = 0x1
    ETranslateAlienTextOverride_DoNotTranslate = 0x2
    ETranslateAlienTextOverride_NumTypes = 0x3

class eBracketsOverride(IntEnum):
    EBracketsOverride_None = 0x0
    EBracketsOverride_Brackets = 0x1
    EBracketsOverride_NoBrackets = 0x2
    EBracketsOverride_NumTypes = 0x3

class eAdditionalOptions(IntEnum):
    EAdditionalOptions_None = 0x0
    EAdditionalOptions_LearnWord = 0x1
    EAdditionalOptions_SayWord = 0x2
    EAdditionalOptions_NumTypes = 0x3

class ePersistancyBufferOverride(IntEnum):
    EPersistancyBufferOverride_None = 0x0
    EPersistancyBufferOverride_AlwaysPersonal = 0x1
    EPersistancyBufferOverride_AlwaysFireteam = 0x2
    EPersistancyBufferOverride_NumTypes = 0x3

class eTestEncounterType(IntEnum):
    ETestEncounterType_AnyEncounter = 0x0
    ETestEncounterType_AnyObject = 0x1
    ETestEncounterType_SpecificObject = 0x2
    ETestEncounterType_Trader = 0x3
    ETestEncounterType_AbandonedFreighter = 0x4
    ETestEncounterType_AnyEncounterOtherThanTheNamedOne = 0x5
    ETestEncounterType_NumTypes = 0x6

class eEventStartType(IntEnum):
    EEventStartType_None = 0x0
    EEventStartType_Special = 0x1
    EEventStartType_Discovered = 0x2
    EEventStartType_Timer = 0x3
    EEventStartType_ObjectScan = 0x4
    EEventStartType_LeaveBuilding = 0x5
    EEventStartType_NumTypes = 0x6

class eEventEndType(IntEnum):
    EEventEndType_None = 0x0
    EEventEndType_Proximity = 0x1
    EEventEndType_Interact = 0x2
    EEventEndType_EnterBuilding = 0x3
    EEventEndType_TimedInteract = 0x4
    EEventEndType_NumTypes = 0x5

class eEventPriority(IntEnum):
    EEventPriority_Regular = 0x0
    EEventPriority_High = 0x1
    EEventPriority_NumTypes = 0x2

class eBuildingLocation(IntEnum):
    EBuildingLocation_Nearest = 0x0
    EBuildingLocation_AllNearest = 0x1
    EBuildingLocation_Random = 0x2
    EBuildingLocation_RandomOnNearPlanet = 0x3
    EBuildingLocation_RandomOnFarPlanet = 0x4
    EBuildingLocation_PlanetSearch = 0x5
    EBuildingLocation_PlayerSettlement = 0x6
    EBuildingLocation_NumTypes = 0x7

class eBuildingType(IntEnum):
    EBuildingType_Any = 0x0
    EBuildingType_AnyShelter = 0x1
    EBuildingType_AnyNPC = 0x2
    EBuildingType_BuildingClass = 0x3
    EBuildingType_SpaceStation = 0x4
    EBuildingType_SpaceAnomaly = 0x5
    EBuildingType_Atlas = 0x6
    EBuildingType_Freighter = 0x7
    EBuildingType_FreighterBase = 0x8
    EBuildingType_ExternalPlanetBase = 0x9
    EBuildingType_PlanetBaseTerminal = 0xA
    EBuildingType_Expedition = 0xB
    EBuildingType_TutorialShelter = 0xC
    EBuildingType_MPMissionFreighter = 0xD
    EBuildingType_Nexus = 0xE
    EBuildingType_InitialDistressSignal = 0xF
    EBuildingType_SpaceMarker = 0x10
    EBuildingType_NexusEggMachine = 0x11
    EBuildingType_PhotoTarget = 0x12
    EBuildingType_SettlementConstruction = 0x13
    EBuildingType_UnownedSettlement = 0x14
    EBuildingType_NPC_HideOut = 0x15
    EBuildingType_FriendlyDrone = 0x16
    EBuildingType_NumTypes = 0x17

class eSolarSystemLocation(IntEnum):
    ESolarSystemLocation_Local = 0x0
    ESolarSystemLocation_Near = 0x1
    ESolarSystemLocation_LocalOrNear = 0x2
    ESolarSystemLocation_NearWithNoExpeditions = 0x3
    ESolarSystemLocation_FromList = 0x4
    ESolarSystemLocation_SeasonParty = 0x5
    ESolarSystemLocation_NumTypes = 0x6

class eRewardChoice(IntEnum):
    ERewardChoice_GiveAll = 0x0
    ERewardChoice_Select = 0x1
    ERewardChoice_SelectAlways = 0x2
    ERewardChoice_TryEach = 0x3
    ERewardChoice_SelectSilent = 0x4
    ERewardChoice_GiveAllSilent = 0x5
    ERewardChoice_TryFirst_ThenSelectAlways = 0x6
    ERewardChoice_SelectFromSuccess = 0x7
    ERewardChoice_SelectAlwaysSilent = 0x8
    ERewardChoice_SelectFromSuccessSilent = 0x9
    ERewardChoice_NumTypes = 0xA

class eRequiredSentinelLevel(IntEnum):
    ERequiredSentinelLevel_None = 0x0
    ERequiredSentinelLevel_Low = 0x1
    ERequiredSentinelLevel_Default = 0x2
    ERequiredSentinelLevel_Aggressive = 0x3
    ERequiredSentinelLevel_NumTypes = 0x4

class ePortalType(IntEnum):
    EPortalType_Any = 0x0
    EPortalType_Story = 0x1
    EPortalType_NotStory = 0x2
    EPortalType_NumTypes = 0x3

class eUsingCameraMode(IntEnum):
    EUsingCameraMode_OnFoot = 0x0
    EUsingCameraMode_Ship = 0x1
    EUsingCameraMode_Vehicle = 0x2
    EUsingCameraMode_NumTypes = 0x3

class eMissionObject(IntEnum):
    EMissionObject_PlayerShip = 0x0
    EMissionObject_PlayerVehicle = 0x1
    EMissionObject_StoryPortal = 0x2
    EMissionObject_OpenStandardPortal = 0x3
    EMissionObject_NumTypes = 0x4

class ePoleCondition(IntEnum):
    EPoleCondition_North = 0x0
    EPoleCondition_South = 0x1
    EPoleCondition_NumTypes = 0x2

class eForHotspotType(IntEnum):
    EForHotspotType_Any = 0x0
    EForHotspotType_Power = 0x1
    EForHotspotType_Gas = 0x2
    EForHotspotType_Minerals = 0x3
    EForHotspotType_NumTypes = 0x4

class eInventoryTest(IntEnum):
    EInventoryTest_Current = 0x0
    EInventoryTest_Personal = 0x1
    EInventoryTest_Ship = 0x2
    EInventoryTest_Vehicle = 0x3
    EInventoryTest_NumTypes = 0x4

class eNPCHabitationType(IntEnum):
    ENPCHabitationType_WeaponsExpert = 0x0
    ENPCHabitationType_Farmer = 0x1
    ENPCHabitationType_Builder = 0x2
    ENPCHabitationType_Vehicles = 0x3
    ENPCHabitationType_Scientist = 0x4
    ENPCHabitationType_NumTypes = 0x5

class eWorkerInCurrentBase(IntEnum):
    EWorkerInCurrentBase_DontCare = 0x0
    EWorkerInCurrentBase_Yes = 0x1
    EWorkerInCurrentBase_No = 0x2
    EWorkerInCurrentBase_NumTypes = 0x3

class eCombatType(IntEnum):
    ECombatType_GroundCombat = 0x0
    ECombatType_SpaceCombat = 0x1
    ECombatType_FiendCombat = 0x2
    ECombatType_BigFishFiendCombat = 0x3
    ECombatType_CorruptedSentinelCombat = 0x4
    ECombatType_GroundWormCombat = 0x5
    ECombatType_RewardEncounter = 0x6
    ECombatType_NumTypes = 0x7

class eAbandonedFreighterDoorStatus(IntEnum):
    EAbandonedFreighterDoorStatus_DungeonNotReady = 0x0
    EAbandonedFreighterDoorStatus_Locked = 0x1
    EAbandonedFreighterDoorStatus_Opening = 0x2
    EAbandonedFreighterDoorStatus_Open = 0x3
    EAbandonedFreighterDoorStatus_NumTypes = 0x4

class eProceduralProductCategory(IntEnum):
    EProceduralProductCategory_Loot = 0x0
    EProceduralProductCategory_Document = 0x1
    EProceduralProductCategory_BioSample = 0x2
    EProceduralProductCategory_Fossil = 0x3
    EProceduralProductCategory_Plant = 0x4
    EProceduralProductCategory_Tool = 0x5
    EProceduralProductCategory_Farm = 0x6
    EProceduralProductCategory_SeaLoot = 0x7
    EProceduralProductCategory_SeaHorror = 0x8
    EProceduralProductCategory_Salvage = 0x9
    EProceduralProductCategory_Bones = 0xA
    EProceduralProductCategory_SpaceHorror = 0xB
    EProceduralProductCategory_SpaceBones = 0xC
    EProceduralProductCategory_FreighterPassword = 0xD
    EProceduralProductCategory_FreighterCaptLog = 0xE
    EProceduralProductCategory_FreighterCrewList = 0xF
    EProceduralProductCategory_FreighterTechHyp = 0x10
    EProceduralProductCategory_FreighterTechSpeed = 0x11
    EProceduralProductCategory_FreighterTechFuel = 0x12
    EProceduralProductCategory_FreighterTechTrade = 0x13
    EProceduralProductCategory_FreighterTechCombat = 0x14
    EProceduralProductCategory_FreighterTechMine = 0x15
    EProceduralProductCategory_FreighterTechExp = 0x16
    EProceduralProductCategory_DismantleBio = 0x17
    EProceduralProductCategory_DismantleTech = 0x18
    EProceduralProductCategory_DismantleData = 0x19
    EProceduralProductCategory_NumTypes = 0x1A

class eMissionPlayerLocation(IntEnum):
    EMissionPlayerLocation_OnPlanet = 0x0
    EMissionPlayerLocation_OnPlanetInVehicle = 0x1
    EMissionPlayerLocation_InShipLanded = 0x2
    EMissionPlayerLocation_InShipInPlanetOrbit = 0x3
    EMissionPlayerLocation_InShipInSpace = 0x4
    EMissionPlayerLocation_InShipAnywhere = 0x5
    EMissionPlayerLocation_InSpaceStation = 0x6
    EMissionPlayerLocation_InFreighter = 0x7
    EMissionPlayerLocation_InYourFreighter = 0x8
    EMissionPlayerLocation_InOtherPlayerFreighter = 0x9
    EMissionPlayerLocation_Underground = 0xA
    EMissionPlayerLocation_InBuilding = 0xB
    EMissionPlayerLocation_Frigate = 0xC
    EMissionPlayerLocation_Underwater = 0xD
    EMissionPlayerLocation_UnderwaterSwimming = 0xE
    EMissionPlayerLocation_DeepUnderwater = 0xF
    EMissionPlayerLocation_InSubmarine = 0x10
    EMissionPlayerLocation_Frigate_Damaged = 0x11
    EMissionPlayerLocation_FreighterConstructionArea = 0x12
    EMissionPlayerLocation_FriendsPlanetBase = 0x13
    EMissionPlayerLocation_OnPlanetSurface = 0x14
    EMissionPlayerLocation_InNexus = 0x15
    EMissionPlayerLocation_InNexusOnFoot = 0x16
    EMissionPlayerLocation_AbandonedFreighterExterior = 0x17
    EMissionPlayerLocation_AbandonedFreighterInterior = 0x18
    EMissionPlayerLocation_AbandonedFreighterAirlock = 0x19
    EMissionPlayerLocation_AtlasStation = 0x1A
    EMissionPlayerLocation_NumTypes = 0x1B

class eIsOnCurrentSystem(IntEnum):
    EIsOnCurrentSystem_DontCare = 0x0
    EIsOnCurrentSystem_Yes = 0x1
    EIsOnCurrentSystem_No = 0x2
    EIsOnCurrentSystem_NumTypes = 0x3

class ePartInCurrentBase(IntEnum):
    EPartInCurrentBase_DontCare = 0x0
    EPartInCurrentBase_YesAllPlayerOwned = 0x1
    EPartInCurrentBase_NumTypes = 0x2

class eLinkNetworkType(IntEnum):
    ELinkNetworkType_Power = 0x0
    ELinkNetworkType_Resources = 0x1
    ELinkNetworkType_Fuel = 0x2
    ELinkNetworkType_Portals = 0x3
    ELinkNetworkType_PlantGrowth = 0x4
    ELinkNetworkType_ByteBeat = 0x5
    ELinkNetworkType_NumTypes = 0x6

class eWeaponStatClass(IntEnum):
    EWeaponStatClass_Pistol = 0x0
    EWeaponStatClass_Rifle = 0x1
    EWeaponStatClass_Pristine = 0x2
    EWeaponStatClass_Alien = 0x3
    EWeaponStatClass_Royal = 0x4
    EWeaponStatClass_NumTypes = 0x5

class eMatchNodeType(IntEnum):
    EMatchNodeType_Any = 0x0
    EMatchNodeType_Mesh = 0x1
    EMatchNodeType_Model = 0x2
    EMatchNodeType_Joint = 0x3
    EMatchNodeType_NumTypes = 0x4

class eMatchName(IntEnum):
    EMatchName_ContainsString = 0x0
    EMatchName_ExactString = 0x1
    EMatchName_NumTypes = 0x2

class eExplodeAction(IntEnum):
    EExplodeAction_RelativeToParent = 0x0
    EExplodeAction_DontMove = 0x1
    EExplodeAction_SaveCenter = 0x2
    EExplodeAction_RelativeToSaved = 0x3
    EExplodeAction_NumTypes = 0x4

class eProgressType(IntEnum):
    EProgressType_ShipOrWeapon = 0x0
    EProgressType_Ship = 0x1
    EProgressType_Weapon = 0x2
    EProgressType_NumTypes = 0x3

class eMissionSystemType(IntEnum):
    EMissionSystemType_Either = 0x0
    EMissionSystemType_Empty = 0x1
    EMissionSystemType_Abandoned = 0x2
    EMissionSystemType_NumTypes = 0x3

class eLastKnownPlayerState(IntEnum):
    ELastKnownPlayerState_OnFoot = 0x0
    ELastKnownPlayerState_InShip = 0x1
    ELastKnownPlayerState_InStation = 0x2
    ELastKnownPlayerState_AboardFleet = 0x3
    ELastKnownPlayerState_InNexus = 0x4
    ELastKnownPlayerState_AbandonedFreighter = 0x5
    ELastKnownPlayerState_InShipLanded = 0x6
    ELastKnownPlayerState_NumTypes = 0x7

class eCollisionGroup(IntEnum):
    ECollisionGroup_Normal = 0x1

class eHotspotType(IntEnum):
    EHotspotType_Power = 0x1
    EHotspotType_Mineral1 = 0x2
    EHotspotType_Mineral2 = 0x4
    EHotspotType_NumTypes = 0x6
    EHotspotType_Mineral3 = 0x8
    EHotspotType_Gas1 = 0x10
    EHotspotType_Gas2 = 0x20

class eExpeditionCategory(IntEnum):
    EExpeditionCategory_Combat = 0x0
    EExpeditionCategory_Exploration = 0x1
    EExpeditionCategory_Mining = 0x2
    EExpeditionCategory_Diplomacy = 0x3
    EExpeditionCategory_Balanced = 0x4
    EExpeditionCategory_NumTypes = 0x5

class eFrigateStatType(IntEnum):
    EFrigateStatType_Combat = 0x0
    EFrigateStatType_Exploration = 0x1
    EFrigateStatType_Mining = 0x2
    EFrigateStatType_Diplomatic = 0x3
    EFrigateStatType_FuelBurnRate = 0x4
    EFrigateStatType_FuelCapacity = 0x5
    EFrigateStatType_Speed = 0x6
    EFrigateStatType_ExtraLoot = 0x7
    EFrigateStatType_Repair = 0x8
    EFrigateStatType_Invulnerable = 0x9
    EFrigateStatType_Stealth = 0xA
    EFrigateStatType_NumTypes = 0xB

class eFrigateTraitStrength(IntEnum):
    EFrigateTraitStrength_NegativeLarge = 0x0
    EFrigateTraitStrength_NegativeMedium = 0x1
    EFrigateTraitStrength_NegativeSmall = 0x2
    EFrigateTraitStrength_TertiarySmall = 0x3
    EFrigateTraitStrength_TertiaryMedium = 0x4
    EFrigateTraitStrength_TertiaryLarge = 0x5
    EFrigateTraitStrength_SecondarySmall = 0x6
    EFrigateTraitStrength_SecondaryMedium = 0x7
    EFrigateTraitStrength_SecondaryLarge = 0x8
    EFrigateTraitStrength_Primary = 0x9
    EFrigateTraitStrength_NumTypes = 0xA

class eWFCDecorationTheme(IntEnum):
    EWFCDecorationTheme_Default = 0x0
    EWFCDecorationTheme_Construction = 0x1
    EWFCDecorationTheme_NumTypes = 0x2

class eExpeditionDuration(IntEnum):
    EExpeditionDuration_VeryShort = 0x0
    EExpeditionDuration_Short = 0x1
    EExpeditionDuration_Medium = 0x2
    EExpeditionDuration_Long = 0x3
    EExpeditionDuration_VeryLong = 0x4
    EExpeditionDuration_NumTypes = 0x5

class eStyle(IntEnum):
    EStyle_None = 0x0
    EStyle_Wood = 0x1
    EStyle_Metal = 0x2
    EStyle_Concrete = 0x3
    EStyle_Stone = 0x4
    EStyle_Timber = 0x5
    EStyle_Fibreglass = 0x6
    EStyle_NumTypes = 0x7

class eAkRTPC(IntEnum):
    EAkRTPC_INVALID_RTPC = 0x0
    EAkRTPC_NumTypes = 0x60
    EAkRTPC_PULSE_EVENT_PANLR = 0x114F803
    EAkRTPC_SHORELINE = 0x1A1A962
    EAkRTPC_SETTLEMENT_INTENSITY = 0x11CAF8FA
    EAkRTPC_PL_AMB_HEIGHT = 0x12F4388A
    EAkRTPC_HG_VA_EMOTE = 0x181937FF
    EAkRTPC_SQUADRON_SHIPS = 0x199ACEC2
    EAkRTPC_VEHICLE_JUMP = 0x1E1DDD32
    EAkRTPC_INTERACT_TIMER = 0x1EE7B825
    EAkRTPC_DOPPLER_DROID_SMALL = 0x1F092F38
    EAkRTPC_HG_VA_SEED = 0x232F7C0E
    EAkRTPC_THEREMIN_VOLUME = 0x26294964
    EAkRTPC_GLOBAL_HEALTH_LEVEL = 0x2A61033E
    EAkRTPC_UI_VR_MENU = 0x2C7EDD8C
    EAkRTPC_HG_VA_SIZE = 0x2E25003A
    EAkRTPC_VEHICLE_SUSPENSION = 0x3016F2FD
    EAkRTPC_PL_WPN_NUMBEROFBULLETS = 0x3CF9013F
    EAkRTPC_PULSE_BUS01_MASTER_VOLUME = 0x42E24B14
    EAkRTPC_COMMS_CHATTER_FREIGHTERATTACKED = 0x46A238DC
    EAkRTPC_POD_SQUISH = 0x47DC0016
    EAkRTPC_VEHICLE_TORQUE = 0x480D482C
    EAkRTPC_SETTLEMENT_DISTANCE = 0x4B8316B2
    EAkRTPC_PLANET_TIME = 0x4C034D71
    EAkRTPC_SHUTTLE_THRUST = 0x51D5A621
    EAkRTPC_COMMS_CHATTER_PIRATES = 0x54E82B11
    EAkRTPC_VEHICLE_SPEED = 0x5979CECB
    EAkRTPC_MASTER_CHAT_LEVEL = 0x5B8E4667
    EAkRTPC_PL_WALK_SPEED = 0x5BF0E7CB
    EAkRTPC_WPN_PL_NEUTRON_CANNON_CHARGE = 0x60C92E72
    EAkRTPC_STORM = 0x648999E0
    EAkRTPC_BINOCULARS_EFFECT = 0x65306505
    EAkRTPC_PS5_HEADPHONES = 0x6B9EA36F
    EAkRTPC_JETPACK_HEIGHT = 0x70B5E6C1
    EAkRTPC_PULSE_EVENT_PITCH = 0x70C3F202
    EAkRTPC_BYTEBEAT_VOLUME = 0x71703CA8
    EAkRTPC_PL_UNDERWATER_DEPTH = 0x72C84A65
    EAkRTPC_MOTION_DRIVER_A = 0x732F78BC
    EAkRTPC_MOTION_DRIVER_B = 0x732F78BF
    EAkRTPC_PL_FALL_SPEED = 0x76233CFB
    EAkRTPC_PL_WPN_LASER_RESOURCEGATHER = 0x767BB3A5
    EAkRTPC_COMMS_CHATTER_DISTANCE = 0x7A371A94
    EAkRTPC_BASE_BATTERY_CHARGING = 0x7C13B3BA
    EAkRTPC_PL_HAZARD_PROTECTION = 0x7EF62492
    EAkRTPC_NPC_SHIP_DISTANCE = 0x810FD033
    EAkRTPC_SENTINEL_DETECTOR = 0x8157313E
    EAkRTPC_RUMBLE_INTENSITY = 0x81780120
    EAkRTPC_RAIN_INTENSITY_BUILDING = 0x83ACB2F0
    EAkRTPC_QUAD_LASERBUILDUP = 0x84FF38B3
    EAkRTPC_VEHICLE_IMPACTS = 0x855298E7
    EAkRTPC_PL_SHIP_VR_EXIT = 0x88814C78
    EAkRTPC_HG_VA_GAMEOBJECTS = 0x8904C9A1
    EAkRTPC_BURN_INTENSITY = 0x8B1E9F48
    EAkRTPC_PL_FOLEY_CLOTHING_LOCO_SPEED = 0x8B826A1A
    EAkRTPC_GAMEOBJECT_DISTANCE = 0x8EB54518
    EAkRTPC_PL_SHIP_YAW = 0x9142CD7A
    EAkRTPC_NPC_SHIP_SPEED = 0x925EFD57
    EAkRTPC_PL_SHIP_SPEED_REV = 0x9407EEA4
    EAkRTPC_RAIN_INTENSITY = 0x9637D55D
    EAkRTPC_PL_WPN_OVERHEAT = 0x96D534DC
    EAkRTPC_PROTOROLLER = 0x98F8B9B1
    EAkRTPC_PL_WPN_LASERPOWER = 0x9918F44C
    EAkRTPC_PL_CAVE_ENCLOSED = 0x99475573
    EAkRTPC_PL_EXERTION = 0x9C64F46C
    EAkRTPC_VEHICLE_SKID = 0xA1303CF3
    EAkRTPC_PULSE_EVENT_SENDBUS_03 = 0xB0196734
    EAkRTPC_PULSE_EVENT_SENDBUS_02 = 0xB0196735
    EAkRTPC_PULSE_EVENT_SENDBUS_01 = 0xB0196736
    EAkRTPC_PULSE_EVENT_SENDBUS_00 = 0xB0196737
    EAkRTPC_BASE_SPHERE_ROLLSPEED = 0xB7D53D81
    EAkRTPC_PLAYER_VR_FOLEY_ARMS = 0xBA619E61
    EAkRTPC_MECH_IDLE = 0xBB021D37
    EAkRTPC_CREATURE_EXISTENCE = 0xBBAE19A3
    EAkRTPC_MAP_STAR_WOOSH = 0xBC7AB0AD
    EAkRTPC_PL_SHIP_LANDINGDISTANCE = 0xC99A4E93
    EAkRTPC_PL_SHIP_SURFACE_WATER = 0xC99CF8A6
    EAkRTPC_MASTER_SFX_LEVEL = 0xC9C3F2F8
    EAkRTPC_RAIN_SHIP_EXTERIOR = 0xCB736AFD
    EAkRTPC_PULSE_EVENT_VOLUME = 0xCE889A56
    EAkRTPC_METEORITE_INCOMING = 0xCFBF792E
    EAkRTPC_PL_SHIP_SPEED = 0xD26BA1DA
    EAkRTPC_COMMS_CHATTER_POLICE = 0xD547E7BB
    EAkRTPC_THEREMIN_PITCH = 0xD774D3B8
    EAkRTPC_NPC_SHIP_DOPPLER = 0xD8BAE8F6
    EAkRTPC_BYTEBEAT_FX = 0xDC013338
    EAkRTPC_CREATURES_STEP_SIZE = 0xE1067D02
    EAkRTPC_PL_SHIP_THRUST = 0xE61896C3
    EAkRTPC_GLOBAL_SHIELD_LEVEL = 0xEA9FE763
    EAkRTPC_WPN_PL_JAVELIN_CHARGE = 0xF04467B0
    EAkRTPC_VEHICLE_EXIT = 0xF4378552
    EAkRTPC_HG_VA_HEADBODYRATIO = 0xF6293C64
    EAkRTPC_PULSE_EVENT_PANFR = 0xF714E8C1
    EAkRTPC_MASTER_MUSIC_LEVEL = 0xF8F6ACB4
    EAkRTPC_RAIN_ROOF = 0xFAC73584
    EAkRTPC_WALKER_MOOD = 0xFB1B461B
    EAkRTPC_GLOBAL_HAZARD_LEVEL = 0xFDD1B808
    EAkRTPC_PL_SHIP_HEIGHT = 0xFEF18CB4

class eGalaxyWaypointType(IntEnum):
    EGalaxyWaypointType_User = 0x0
    EGalaxyWaypointType_Gameplay_AtlasStation = 0x1
    EGalaxyWaypointType_Gameplay_DistressBeacon = 0x2
    EGalaxyWaypointType_Gameplay_BlackHole = 0x3
    EGalaxyWaypointType_Gameplay_Mission = 0x4
    EGalaxyWaypointType_Gameplay_SeasonParty = 0x5
    EGalaxyWaypointType_NumTypes = 0x6

class eTransform(IntEnum):
    ETransform_None = 0x0
    ETransform_Rotated90 = 0x1
    ETransform_Rotated180 = 0x2
    ETransform_Rotated270 = 0x3
    ETransform_FlippedHorizontally = 0x4
    ETransform_NumTypes = 0x5

class eEggItemType(IntEnum):
    EEggItemType_Egg = 0x0
    EEggItemType_ValidCatalyst = 0x1
    EEggItemType_NumTypes = 0x2

class eDirection(IntEnum):
    EDirection_Left = 0x0
    EDirection_Back = 0x1
    EDirection_Right = 0x2
    EDirection_Forward = 0x3
    EDirection_LeftBack = 0x4
    EDirection_RightBack = 0x5
    EDirection_RightForward = 0x6
    EDirection_LeftForward = 0x7
    EDirection_All = 0x8
    EDirection_NumTypes = 0x9

class eLevels(IntEnum):
    ELevels_Lower = 0x0
    ELevels_Upper = 0x1
    ELevels_Both = 0x2
    ELevels_NumTypes = 0x3

class eTerrain(IntEnum):
    ETerrain_RequireAbove = 0x0
    ETerrain_RequireBelow = 0x1
    ETerrain_NumTypes = 0x2

class eTerrainEdits(IntEnum):
    ETerrainEdits_None = 0x0
    ETerrainEdits_ClearEntireBlock = 0x1
    ETerrainEdits_UseScene = 0x2
    ETerrainEdits_UseBasebuildingEdits = 0x3
    ETerrainEdits_NumTypes = 0x4

class eFreighterModuleType(IntEnum):
    EFreighterModuleType_None = 0x0
    EFreighterModuleType_Room = 0x1
    EFreighterModuleType_Corridor = 0x2
    EFreighterModuleType_NumTypes = 0x3

class eExperienceDebugTriggerAction(IntEnum):
    EExperienceDebugTriggerAction_None = 0x0
    EExperienceDebugTriggerAction_Drones = 0x1
    EExperienceDebugTriggerAction_FlyBy = 0x2
    EExperienceDebugTriggerAction_FrigateFlyByBegin = 0x3
    EExperienceDebugTriggerAction_FrigateFlyByEnd = 0x4
    EExperienceDebugTriggerAction_PirateCargoAttack = 0x5
    EExperienceDebugTriggerAction_PirateRaid = 0x6
    EExperienceDebugTriggerAction_FreighterAttack = 0x7
    EExperienceDebugTriggerAction_SpawnShips = 0x8
    EExperienceDebugTriggerAction_LaunchShips = 0x9
    EExperienceDebugTriggerAction_Mechs = 0xA
    EExperienceDebugTriggerAction_SpaceBattle = 0xB
    EExperienceDebugTriggerAction_RespawnInShip = 0xC
    EExperienceDebugTriggerAction_DebugWalker = 0xD
    EExperienceDebugTriggerAction_DebugWalkerTitanFall = 0xE
    EExperienceDebugTriggerAction_SpawnNexus = 0xF
    EExperienceDebugTriggerAction_Freighters = 0x10
    EExperienceDebugTriggerAction_NPCs = 0x11
    EExperienceDebugTriggerAction_Sandworm = 0x12
    EExperienceDebugTriggerAction_SpacePOI = 0x13
    EExperienceDebugTriggerAction_BackgroundSpaceEncounter = 0x14
    EExperienceDebugTriggerAction_Creatures = 0x15
    EExperienceDebugTriggerAction_CameraPath = 0x16
    EExperienceDebugTriggerAction_SummonFleet = 0x17
    EExperienceDebugTriggerAction_SummonSquadron = 0x18
    EExperienceDebugTriggerAction_ResetScene = 0x19
    EExperienceDebugTriggerAction_ResetPlayerPos = 0x1A
    EExperienceDebugTriggerAction_CameraSpin = 0x1B
    EExperienceDebugTriggerAction_SpawnEnemyShips = 0x1C
    EExperienceDebugTriggerAction_PetHappy = 0x1D
    EExperienceDebugTriggerAction_PetSad = 0x1E
    EExperienceDebugTriggerAction_PetFollow = 0x1F
    EExperienceDebugTriggerAction_PetFollowClose = 0x20
    EExperienceDebugTriggerAction_PetRest = 0x21
    EExperienceDebugTriggerAction_PetNatural = 0x22
    EExperienceDebugTriggerAction_PetMine = 0x23
    EExperienceDebugTriggerAction_PetMineAndDeposit = 0x24
    EExperienceDebugTriggerAction_RidePet = 0x25
    EExperienceDebugTriggerAction_Normandy = 0x26
    EExperienceDebugTriggerAction_LivingFrigate = 0x27
    EExperienceDebugTriggerAction_UpgradeSettlement = 0x28
    EExperienceDebugTriggerAction_NumTypes = 0x29

class eObjectPlacement(IntEnum):
    EObjectPlacement_None = 0x0
    EObjectPlacement_ResourceSmall = 0x1
    EObjectPlacement_ResourceMedium = 0x2
    EObjectPlacement_ResourceLarge = 0x3
    EObjectPlacement_ResourceDebris = 0x4
    EObjectPlacement_NumTypes = 0x5

class eSpawnConditions(IntEnum):
    ESpawnConditions_Anytime = 0x0
    ESpawnConditions_DuringStorm = 0x1
    ESpawnConditions_AtNight = 0x2
    ESpawnConditions_NotInStorm = 0x3
    ESpawnConditions_AtNightNotInStorm = 0x4
    ESpawnConditions_NumTypes = 0x5

class eWeatherEffectBehaviour(IntEnum):
    EWeatherEffectBehaviour_Static = 0x0
    EWeatherEffectBehaviour_Wander = 0x1
    EWeatherEffectBehaviour_ChasePlayer = 0x2
    EWeatherEffectBehaviour_NumTypes = 0x3

class eWeatherEffectSpawnType(IntEnum):
    EWeatherEffectSpawnType_Single = 0x0
    EWeatherEffectSpawnType_Cluster = 0x1
    EWeatherEffectSpawnType_Patch = 0x2
    EWeatherEffectSpawnType_NumTypes = 0x3

class eFreighterInCurrentSystem(IntEnum):
    EFreighterInCurrentSystem_DontCare = 0x0
    EFreighterInCurrentSystem_Yes = 0x1
    EFreighterInCurrentSystem_No = 0x2
    EFreighterInCurrentSystem_NumTypes = 0x3

class eCanWalk(IntEnum):
    ECanWalk_None = 0x0
    ECanWalk_RequireCanWalk = 0x1
    ECanWalk_RequireCanNotWalk = 0x2
    ECanWalk_NumTypes = 0x3

class eInsideOutside(IntEnum):
    EInsideOutside_Both = 0x0
    EInsideOutside_InteriorOnly = 0x1
    EInsideOutside_ExteriorOnly = 0x2
    EInsideOutside_NumTypes = 0x3

class eLevel(IntEnum):
    ELevel_Everywhere = 0x0
    ELevel_GroundLevelOnly = 0x1
    ELevel_AboveGroundOnly = 0x2
    ELevel_NumTypes = 0x3

class ePetBehaviour(IntEnum):
    EPetBehaviour_None = 0x0
    EPetBehaviour_Idle = 0x1
    EPetBehaviour_Eat = 0x2
    EPetBehaviour_Poop = 0x3
    EPetBehaviour_LayEgg = 0x4
    EPetBehaviour_FollowPlayer = 0x5
    EPetBehaviour_AdoptedFollowPlayer = 0x6
    EPetBehaviour_ScanForResource = 0x7
    EPetBehaviour_FindResource = 0x8
    EPetBehaviour_FindHazards = 0x9
    EPetBehaviour_AttackHazard = 0xA
    EPetBehaviour_FindBuilding = 0xB
    EPetBehaviour_Fetch = 0xC
    EPetBehaviour_Explore = 0xD
    EPetBehaviour_Emote = 0xE
    EPetBehaviour_GestureReact = 0xF
    EPetBehaviour_OrderedToPos = 0x10
    EPetBehaviour_ComeHere = 0x11
    EPetBehaviour_Mine = 0x12
    EPetBehaviour_Summoned = 0x13
    EPetBehaviour_Adopted = 0x14
    EPetBehaviour_Hatched = 0x15
    EPetBehaviour_PostInteract = 0x16
    EPetBehaviour_Rest = 0x17
    EPetBehaviour_Attack = 0x18
    EPetBehaviour_Watch = 0x19
    EPetBehaviour_Greet = 0x1A
    EPetBehaviour_TeleportToPlayer = 0x1B
    EPetBehaviour_NumTypes = 0x1C

class eHazardModifier(IntEnum):
    EHazardModifier_Temperature = 0x0
    EHazardModifier_Toxicity = 0x1
    EHazardModifier_Radiation = 0x2
    EHazardModifier_LifeSupportDrain = 0x3
    EHazardModifier_Gravity = 0x4
    EHazardModifier_NumTypes = 0x5

class eKeyPress(IntEnum):
    EKeyPress_1 = 0x0
    EKeyPress_2 = 0x1
    EKeyPress_3 = 0x2
    EKeyPress_4 = 0x3
    EKeyPress_5 = 0x4
    EKeyPress_6 = 0x5
    EKeyPress_7 = 0x6
    EKeyPress_8 = 0x7
    EKeyPress_9 = 0x8
    EKeyPress_PadUp = 0x9
    EKeyPress_PadDown = 0xA
    EKeyPress_PadLeft = 0xB
    EKeyPress_PadRight = 0xC
    EKeyPress_NumTypes = 0xD

class eFreighterBattleStatus(IntEnum):
    EFreighterBattleStatus_None = 0x0
    EFreighterBattleStatus_Active = 0x1
    EFreighterBattleStatus_Joined = 0x2
    EFreighterBattleStatus_Reward = 0x3
    EFreighterBattleStatus_NumTypes = 0x4

class eWaypointType(IntEnum):
    EWaypointType_Move = 0x0
    EWaypointType_MoveAlt = 0x1
    EWaypointType_Idle = 0x2
    EWaypointType_NumTypes = 0x3

class eOnComplete(IntEnum):
    EOnComplete_Hold = 0x0
    EOnComplete_Loop = 0x1
    EOnComplete_Destroy = 0x2
    EOnComplete_NumTypes = 0x3

class ePresetGameMode(IntEnum):
    EPresetGameMode_Unspecified = 0x0
    EPresetGameMode_Normal = 0x1
    EPresetGameMode_Creative = 0x2
    EPresetGameMode_Survival = 0x3
    EPresetGameMode_Ambient = 0x4
    EPresetGameMode_Permadeath = 0x5
    EPresetGameMode_Seasonal = 0x6
    EPresetGameMode_NumTypes = 0x7

class eGalaxyMarkerType(IntEnum):
    EGalaxyMarkerType_StartingLocation = 0x0
    EGalaxyMarkerType_Home = 0x1
    EGalaxyMarkerType_Waypoint = 0x2
    EGalaxyMarkerType_Contact = 0x3
    EGalaxyMarkerType_Blackhole = 0x4
    EGalaxyMarkerType_AtlasStation = 0x5
    EGalaxyMarkerType_Selection = 0x6
    EGalaxyMarkerType_PlanetBase = 0x7
    EGalaxyMarkerType_Visited = 0x8
    EGalaxyMarkerType_ScanEvent = 0x9
    EGalaxyMarkerType_Expedition = 0xA
    EGalaxyMarkerType_NetworkPlayer = 0xB
    EGalaxyMarkerType_Freighter = 0xC
    EGalaxyMarkerType_PathIcon = 0xD
    EGalaxyMarkerType_SeasonParty = 0xE
    EGalaxyMarkerType_Settlement = 0xF
    EGalaxyMarkerType_NumTypes = 0x10

class eIncrementor(IntEnum):
    EIncrementor_None = 0x0
    EIncrementor_Resource = 0x1
    EIncrementor_ResourceAway = 0x2
    EIncrementor_HeavyAir = 0x3
    EIncrementor_Drone = 0x4
    EIncrementor_Deer = 0x5
    EIncrementor_DeerScan = 0x6
    EIncrementor_DeerWords = 0x7
    EIncrementor_DeerWordsAway = 0x8
    EIncrementor_Diplo = 0x9
    EIncrementor_DiploScan = 0xA
    EIncrementor_DiploWords = 0xB
    EIncrementor_DiploWordsAway = 0xC
    EIncrementor_Flyby = 0xD
    EIncrementor_Beast = 0xE
    EIncrementor_Wingmen = 0xF
    EIncrementor_Scouts = 0x10
    EIncrementor_Fleet = 0x11
    EIncrementor_Attackers = 0x12
    EIncrementor_AttackersFromBehind = 0x13
    EIncrementor_Flee = 0x14
    EIncrementor_RemoveFleet = 0x15
    EIncrementor_Fighters = 0x16
    EIncrementor_PostFighters = 0x17
    EIncrementor_Escape = 0x18
    EIncrementor_Warp = 0x19
    EIncrementor_NumTypes = 0x1A

class eNPCTrigger(IntEnum):
    ENPCTrigger_None = 0x0
    ENPCTrigger_Idle = 0x1
    ENPCTrigger_Greet = 0x2
    ENPCTrigger_Mood = 0x3
    ENPCTrigger_StartDead = 0x4
    ENPCTrigger_Talk_Start = 0x5
    ENPCTrigger_Talk_Stop = 0x6
    ENPCTrigger_Interact_Start = 0x7
    ENPCTrigger_Interact_Stop = 0x8
    ENPCTrigger_Interact_BeginHold = 0x9
    ENPCTrigger_Interact_CancelHold = 0xA
    ENPCTrigger_LookAt_Player_Start = 0xB
    ENPCTrigger_LookAt_Player_Stop = 0xC
    ENPCTrigger_SetProp = 0xD
    ENPCTrigger_Interact_StartFromRemote = 0xE
    ENPCTrigger_NumTypes = 0xF

class ePetAccessory(IntEnum):
    EPetAccessory_None = 0x0
    EPetAccessory_CargoCylinder = 0x1
    EPetAccessory_Containers = 0x2
    EPetAccessory_ShieldArmour = 0x3
    EPetAccessory_SolarBattery = 0x4
    EPetAccessory_Tank = 0x5
    EPetAccessory_WingPanel = 0x6
    EPetAccessory_TravelPack = 0x7
    EPetAccessory_SpacePack = 0x8
    EPetAccessory_CargoLong = 0x9
    EPetAccessory_Antennae = 0xA
    EPetAccessory_Computer = 0xB
    EPetAccessory_Toolbelt = 0xC
    EPetAccessory_LeftCanisters = 0xD
    EPetAccessory_LeftEnergyCoil = 0xE
    EPetAccessory_LeftFrigateTurret = 0xF
    EPetAccessory_LeftHeadLights = 0x10
    EPetAccessory_LeftArmourPlate = 0x11
    EPetAccessory_LeftTurret = 0x12
    EPetAccessory_LeftSupportSystem = 0x13
    EPetAccessory_RightCanisters = 0x14
    EPetAccessory_RightEnergyCoil = 0x15
    EPetAccessory_RightFrigateTurret = 0x16
    EPetAccessory_RightHeadLights = 0x17
    EPetAccessory_RightArmourPlate = 0x18
    EPetAccessory_RightTurret = 0x19
    EPetAccessory_RightSupportSystem = 0x1A
    EPetAccessory_NumTypes = 0x1B

class eAttractorType(IntEnum):
    EAttractorType_Food = 0x0
    EAttractorType_Harvester = 0x1
    EAttractorType_NumTypes = 0x2

class eCreatureRoleFrequencyModifier(IntEnum):
    ECreatureRoleFrequencyModifier_Never = 0x0
    ECreatureRoleFrequencyModifier_Low = 0x1
    ECreatureRoleFrequencyModifier_Normal = 0x2
    ECreatureRoleFrequencyModifier_High = 0x3
    ECreatureRoleFrequencyModifier_NumTypes = 0x4

class eCreatureRarity(IntEnum):
    ECreatureRarity_Common = 0x0
    ECreatureRarity_Uncommon = 0x1
    ECreatureRarity_Rare = 0x2
    ECreatureRarity_SuperRare = 0x3
    ECreatureRarity_NumTypes = 0x4
    ECreatureRarity_Unspecified = 0xFFFFFFFF

class eBuildingSystemType(IntEnum):
    EBuildingSystemType_Normal = 0x0
    EBuildingSystemType_AbandonedSystem = 0x1
    EBuildingSystemType_NumTypes = 0x2

class eCreatureGroup(IntEnum):
    ECreatureGroup_Solo = 0x0
    ECreatureGroup_Couple = 0x1
    ECreatureGroup_Group = 0x2
    ECreatureGroup_Herd = 0x3
    ECreatureGroup_NumTypes = 0x4

class eCreatureIkType(IntEnum):
    ECreatureIkType_Foot = 0x0
    ECreatureIkType_Hinge_X = 0x1
    ECreatureIkType_Hinge_Y = 0x2
    ECreatureIkType_Hinge_Z = 0x3
    ECreatureIkType_Locked = 0x4
    ECreatureIkType_Head = 0x5
    ECreatureIkType_Toe = 0x6
    ECreatureIkType_SpaceshipFoot = 0x7
    ECreatureIkType_SpaceshipToe = 0x8
    ECreatureIkType_NumTypes = 0x9

class eSpawnerMode(IntEnum):
    ESpawnerMode_Hidden = 0x0
    ESpawnerMode_Visible = 0x1
    ESpawnerMode_HideOnSpawn = 0x2
    ESpawnerMode_HiddenTimer = 0x3
    ESpawnerMode_NumTypes = 0x4

class eResourceOrigin(IntEnum):
    EResourceOrigin_Terrain = 0x0
    EResourceOrigin_Crystal = 0x1
    EResourceOrigin_Asteroid = 0x2
    EResourceOrigin_Robot = 0x3
    EResourceOrigin_Depot = 0x4
    EResourceOrigin_NumTypes = 0x5

class ePetBehaviourValidity(IntEnum):
    EPetBehaviourValidity_Everywhere = 0x0
    EPetBehaviourValidity_OnPlanet = 0x1
    EPetBehaviourValidity_NumTypes = 0x2

class ePetAction(IntEnum):
    EPetAction_Tickle = 0x0
    EPetAction_Treat = 0x1
    EPetAction_Ride = 0x2
    EPetAction_Customise = 0x3
    EPetAction_Abandon = 0x4
    EPetAction_LayEgg = 0x5
    EPetAction_Adopt = 0x6
    EPetAction_Milk = 0x7
    EPetAction_HarvestSpecial = 0x8
    EPetAction_NumTypes = 0x9

class eObjectRenderType(IntEnum):
    EObjectRenderType_Instanced = 0x0
    EObjectRenderType_Single = 0x1
    EObjectRenderType_NumTypes = 0x2

class eSizeClass(IntEnum):
    ESizeClass_Tiny = 0x0
    ESizeClass_Small = 0x1
    ESizeClass_Medium = 0x2
    ESizeClass_Large = 0x3
    ESizeClass_Massive = 0x4
    ESizeClass_NumTypes = 0x5

class eObjectCoverageType(IntEnum):
    EObjectCoverageType_Blanket = 0x0
    EObjectCoverageType_Cluster = 0x1
    EObjectCoverageType_Solo = 0x2
    EObjectCoverageType_NumTypes = 0x3

class eLifeType(IntEnum):
    ELifeType_Rock = 0x0
    ELifeType_DryPlant = 0x1
    ELifeType_LushPlant = 0x2
    ELifeType_Artificial = 0x3
    ELifeType_NumTypes = 0x4

class eLocationType(IntEnum):
    ELocationType_AboveGround = 0x0
    ELocationType_UnderGround = 0x1
    ELocationType_WaterSurface = 0x2
    ELocationType_UnderWater = 0x3
    ELocationType_NumTypes = 0x4

class eObjectAlignment(IntEnum):
    EObjectAlignment_Upright = 0x0
    EObjectAlignment_SlightOffsetFromUpright = 0x1
    EObjectAlignment_LargeOffsetFromUpright = 0x2
    EObjectAlignment_ToNormal = 0x3
    EObjectAlignment_SlightOffsetFromNormal = 0x4
    EObjectAlignment_LargeOffsetFromNormal = 0x5
    EObjectAlignment_NumTypes = 0x6

class eDeathType(IntEnum):
    EDeathType_Explode = 0x0
    EDeathType_Drop = 0x1
    EDeathType_NumTypes = 0x2

class eSubType(IntEnum):
    ESubType_Crystal = 0x0
    ESubType_Tentacle = 0x1
    ESubType_NumTypes = 0x2

class eModifyIntType(IntEnum):
    EModifyIntType_SetValue = 0x0
    EModifyIntType_IncrementValue = 0x1
    EModifyIntType_NumTypes = 0x2

class eBehaviourMoveSpeed(IntEnum):
    EBehaviourMoveSpeed_Normal = 0x0
    EBehaviourMoveSpeed_Fast = 0x1
    EBehaviourMoveSpeed_Dynamic = 0x2
    EBehaviourMoveSpeed_NumTypes = 0x3

class eCustomisationDataType(IntEnum):
    ECustomisationDataType_Player = 0x0
    ECustomisationDataType_Vehicle = 0x1
    ECustomisationDataType_Weapon = 0x2
    ECustomisationDataType_Ship_01 = 0x3
    ECustomisationDataType_Ship_02 = 0x4
    ECustomisationDataType_Ship_03 = 0x5
    ECustomisationDataType_Ship_04 = 0x6
    ECustomisationDataType_Ship_05 = 0x7
    ECustomisationDataType_Ship_06 = 0x8
    ECustomisationDataType_Vehicle_Bike = 0x9
    ECustomisationDataType_Vehicle_Truck = 0xA
    ECustomisationDataType_Vehicle_WheeledBike = 0xB
    ECustomisationDataType_Vehicle_Hovercraft = 0xC
    ECustomisationDataType_Vehicle_Submarine = 0xD
    ECustomisationDataType_Vehicle_Mech = 0xE
    ECustomisationDataType_Freighter = 0xF
    ECustomisationDataType_Pet = 0x10
    ECustomisationDataType_Ship_07 = 0x11
    ECustomisationDataType_Ship_08 = 0x12
    ECustomisationDataType_Ship_09 = 0x13
    ECustomisationDataType_Ship_10 = 0x14
    ECustomisationDataType_Ship_11 = 0x15
    ECustomisationDataType_Ship_12 = 0x16
    ECustomisationDataType_NumTypes = 0x17

class eSwarmMovementType(IntEnum):
    ESwarmMovementType_None = 0x0
    ESwarmMovementType_Circle = 0x1
    ESwarmMovementType_Random = 0x2
    ESwarmMovementType_Search = 0x3
    ESwarmMovementType_FollowPlayer = 0x4
    ESwarmMovementType_FollowPlayerLimited = 0x5
    ESwarmMovementType_NumTypes = 0x6

class eVocalEmote(IntEnum):
    EVocalEmote_EmoteIdle = 0x0
    EVocalEmote_EmoteFlee = 0x1
    EVocalEmote_EmoteAggression = 0x2
    EVocalEmote_EmoteRoar = 0x3
    EVocalEmote_EmotePain = 0x4
    EVocalEmote_EmoteAttack = 0x5
    EVocalEmote_EmoteDie = 0x6
    EVocalEmote_EmoteMiniRoarNeutral = 0x7
    EVocalEmote_EmoteMiniRoarHappy = 0x8
    EVocalEmote_EmoteMiniRoarAngry = 0x9
    EVocalEmote_NumTypes = 0xA

class eMoveMode(IntEnum):
    EMoveMode_Roll = 0x0
    EMoveMode_Float = 0x1
    EMoveMode_Drill = 0x2
    EMoveMode_NumTypes = 0x3

class eMoveSpeed(IntEnum):
    EMoveSpeed_Always = 0x0
    EMoveSpeed_Walk = 0x1
    EMoveSpeed_Run = 0x2
    EMoveSpeed_NumTypes = 0x3

class eSentinelCoverState(IntEnum):
    ESentinelCoverState_Deploying = 0x0
    ESentinelCoverState_Deployed = 0x1
    ESentinelCoverState_ShuttingDown = 0x2
    ESentinelCoverState_ShutDown = 0x3
    ESentinelCoverState_NumTypes = 0x4

class eFreighterBaseGenerationMode(IntEnum):
    EFreighterBaseGenerationMode_Prefab = 0x0
    EFreighterBaseGenerationMode_WFC = 0x1
    EFreighterBaseGenerationMode_NumTypes = 0x2

class eGenericIconType(IntEnum):
    EGenericIconType_None = 0x0
    EGenericIconType_Interaction = 0x1
    EGenericIconType_SpaceStation = 0x2
    EGenericIconType_SpaceAnomaly = 0x3
    EGenericIconType_SpaceAtlas = 0x4
    EGenericIconType_Nexus = 0x5
    EGenericIconType_NumTypes = 0x6

class eShotBy(IntEnum):
    EShotBy_Player = 0x0
    EShotBy_Anything = 0x1
    EShotBy_PlayerOrRemotePlayer = 0x2
    EShotBy_NumTypes = 0x3

class eDisplayMode(IntEnum):
    EDisplayMode_Always = 0x0
    EDisplayMode_SpaceOnly = 0x1
    EDisplayMode_PlanetOnly = 0x2
    EDisplayMode_NumTypes = 0x3

class eRequirePlayerAction(IntEnum):
    ERequirePlayerAction_None = 0x0
    ERequirePlayerAction_Fire = 0x1
    ERequirePlayerAction_InShip = 0x2
    ERequirePlayerAction_OnFoot = 0x3
    ERequirePlayerAction_OnFootOutside = 0x4
    ERequirePlayerAction_Upload = 0x5
    ERequirePlayerAction_NumTypes = 0x6

class eDistanceCheckType(IntEnum):
    EDistanceCheckType_Radius = 0x0
    EDistanceCheckType_BoundingBox = 0x1
    EDistanceCheckType_NumTypes = 0x2

class eInventory(IntEnum):
    EInventory_Vehicle = 0x0
    EInventory_NumTypes = 0x1

class eLookAtType(IntEnum):
    ELookAtType_Player = 0x0
    ELookAtType_NumTypes = 0x1

class eLegacyBehaviour(IntEnum):
    ELegacyBehaviour_Riding = 0x0
    ELegacyBehaviour_Interaction = 0x1
    ELegacyBehaviour_Attracted = 0x2
    ELegacyBehaviour_Flee = 0x3
    ELegacyBehaviour_Defend = 0x4
    ELegacyBehaviour_FollowPlayer = 0x5
    ELegacyBehaviour_AvoidPlayer = 0x6
    ELegacyBehaviour_NoticePlayer = 0x7
    ELegacyBehaviour_FollowRoutine = 0x8
    ELegacyBehaviour_NumTypes = 0x9

class eMaintenanceGroup(IntEnum):
    EMaintenanceGroup_Custom = 0x0
    EMaintenanceGroup_Farming = 0x1
    EMaintenanceGroup_Fuelling = 0x2
    EMaintenanceGroup_Repairing = 0x3
    EMaintenanceGroup_EasyRepairing = 0x4
    EMaintenanceGroup_Cleaning = 0x5
    EMaintenanceGroup_Frigate = 0x6
    EMaintenanceGroup_NumTypes = 0x7

class eBroadcastLevel(IntEnum):
    EBroadcastLevel_Scene = 0x0
    EBroadcastLevel_LocalModel = 0x1
    EBroadcastLevel_Local = 0x2
    EBroadcastLevel_NumTypes = 0x3

class eInteractable(IntEnum):
    EInteractable_OnlyWhenComplete = 0x0
    EInteractable_OnlyWhenNotComplete = 0x1
    EInteractable_Always = 0x2
    EInteractable_Never = 0x3
    EInteractable_NumTypes = 0x4

class eModelRendererResource(IntEnum):
    EModelRendererResource_ModelNode = 0x0
    EModelRendererResource_MasterModelNode = 0x1
    EModelRendererResource_NumTypes = 0x2

class eHeadUpAxis(IntEnum):
    EHeadUpAxis_X = 0x0
    EHeadUpAxis_XNeg = 0x1
    EHeadUpAxis_Y = 0x2
    EHeadUpAxis_YNeg = 0x3
    EHeadUpAxis_Z = 0x4
    EHeadUpAxis_ZNeg = 0x5
    EHeadUpAxis_NumTypes = 0x6

class eNumColours(IntEnum):
    ENumColours_Inactive = 0x0
    ENumColours_1 = 0x1
    ENumColours_4 = 0x2
    ENumColours_8 = 0x3
    ENumColours_16 = 0x4
    ENumColours_All = 0x5
    ENumColours_NumTypes = 0x6

class eBaseDefenceStatus(IntEnum):
    EBaseDefenceStatus_AttackingTarget = 0x0
    EBaseDefenceStatus_Alert = 0x1
    EBaseDefenceStatus_SearchingForTarget = 0x2
    EBaseDefenceStatus_Disabled = 0x3
    EBaseDefenceStatus_Security = 0x4
    EBaseDefenceStatus_NumTypes = 0x5

class eMoveArea(IntEnum):
    EMoveArea_Ground = 0x0
    EMoveArea_Water = 0x1
    EMoveArea_Air = 0x2
    EMoveArea_Space = 0x3
    EMoveArea_NumTypes = 0x4

class eUpdateType(IntEnum):
    EUpdateType_UpdatesAlways = 0x0
    EUpdateType_UpdateOnlyWhenComplete = 0x1
    EUpdateType_UpdateOnlyWhenNotComplete = 0x2
    EUpdateType_NumTypes = 0x3

class eDamageStatus(IntEnum):
    EDamageStatus_Damaged = 0x0
    EDamageStatus_Repaired = 0x1
    EDamageStatus_Random = 0x2
    EDamageStatus_NumTypes = 0x3

class eCompletionRequirement(IntEnum):
    ECompletionRequirement_FullyChargedAndRepaired = 0x0
    ECompletionRequirement_AnyChargeAndRepaired = 0x1
    ECompletionRequirement_FullyRepaired = 0x2
    ECompletionRequirement_NotFullyCharged = 0x3
    ECompletionRequirement_EmptySlot = 0x4
    ECompletionRequirement_NoRequirement = 0x5
    ECompletionRequirement_UserInstalls = 0x6
    ECompletionRequirement_HasIngredients = 0x7
    ECompletionRequirement_NumTypes = 0x8

class eDesiredTechProbability(IntEnum):
    EDesiredTechProbability_Never = 0x0
    EDesiredTechProbability_Rare = 0x1
    EDesiredTechProbability_Common = 0x2
    EDesiredTechProbability_Always = 0x3
    EDesiredTechProbability_NumTypes = 0x4

class ePlayerHeadUpAxis(IntEnum):
    EPlayerHeadUpAxis_X = 0x0
    EPlayerHeadUpAxis_XNeg = 0x1
    EPlayerHeadUpAxis_Y = 0x2
    EPlayerHeadUpAxis_YNeg = 0x3
    EPlayerHeadUpAxis_Z = 0x4
    EPlayerHeadUpAxis_ZNeg = 0x5
    EPlayerHeadUpAxis_NumTypes = 0x6

class eSynchroniseScale(IntEnum):
    ESynchroniseScale_Never = 0x0
    ESynchroniseScale_Once = 0x1
    ESynchroniseScale_Always = 0x2
    ESynchroniseScale_NumTypes = 0x3

class eDiscoveryType(IntEnum):
    EDiscoveryType_Unknown = 0x0
    EDiscoveryType_SolarSystem = 0x1
    EDiscoveryType_Planet = 0x2
    EDiscoveryType_Animal = 0x3
    EDiscoveryType_Flora = 0x4
    EDiscoveryType_Mineral = 0x5
    EDiscoveryType_Sector = 0x6
    EDiscoveryType_Building = 0x7
    EDiscoveryType_Interactable = 0x8
    EDiscoveryType_Sentinel = 0x9
    EDiscoveryType_Starship = 0xA
    EDiscoveryType_Artifact = 0xB
    EDiscoveryType_Mystery = 0xC
    EDiscoveryType_Treasure = 0xD
    EDiscoveryType_Control = 0xE
    EDiscoveryType_HarvestPlant = 0xF
    EDiscoveryType_FriendlyDrone = 0x10
    EDiscoveryType_NumTypes = 0x11

class eScanIconType(IntEnum):
    EScanIconType_None = 0x0
    EScanIconType_Health = 0x1
    EScanIconType_Shield = 0x2
    EScanIconType_Hazard = 0x3
    EScanIconType_LifeSupport = 0x4
    EScanIconType_Tech = 0x5
    EScanIconType_BluePlant = 0x6
    EScanIconType_CaveSubstance = 0x7
    EScanIconType_LaunchCrystals = 0x8
    EScanIconType_Power = 0x9
    EScanIconType_Carbon = 0xA
    EScanIconType_CarbonPlus = 0xB
    EScanIconType_Oxygen = 0xC
    EScanIconType_Mineral = 0xD
    EScanIconType_Sodium = 0xE
    EScanIconType_SodiumPlus = 0xF
    EScanIconType_Crate = 0x10
    EScanIconType_Artifact = 0x11
    EScanIconType_Plant = 0x12
    EScanIconType_HazardPlant = 0x13
    EScanIconType_ArtifactCrate = 0x14
    EScanIconType_BuriedTech = 0x15
    EScanIconType_BuriedRare = 0x16
    EScanIconType_Drone = 0x17
    EScanIconType_CustomMarker = 0x18
    EScanIconType_SignalBooster = 0x19
    EScanIconType_Refiner = 0x1A
    EScanIconType_Grave = 0x1B
    EScanIconType_Rare1 = 0x1C
    EScanIconType_Rare2 = 0x1D
    EScanIconType_Rare3 = 0x1E
    EScanIconType_Pearl = 0x1F
    EScanIconType_RareEgg = 0x20
    EScanIconType_HazardEgg = 0x21
    EScanIconType_FishFiend = 0x22
    EScanIconType_Clam = 0x23
    EScanIconType_CaveStone = 0x24
    EScanIconType_StormCrystal = 0x25
    EScanIconType_BiomeTrophy = 0x26
    EScanIconType_PowerHotspot = 0x27
    EScanIconType_MineralHotspot = 0x28
    EScanIconType_GasHotspot = 0x29
    EScanIconType_HarvestPlant = 0x2A
    EScanIconType_Cooker = 0x2B
    EScanIconType_CreaturePoop = 0x2C
    EScanIconType_FreighterTeleporter = 0x2D
    EScanIconType_FreighterDoor = 0x2E
    EScanIconType_FreighterTerminal = 0x2F
    EScanIconType_FreighterHeater = 0x30
    EScanIconType_FreighterDataPad = 0x31
    EScanIconType_LandedPilot = 0x32
    EScanIconType_PetEgg = 0x33
    EScanIconType_Sandworm = 0x34
    EScanIconType_FriendlyDrone = 0x35
    EScanIconType_NumTypes = 0x36

class eScannableType(IntEnum):
    EScannableType_Binoculars = 0x0
    EScannableType_BinocularsHotspots = 0x1
    EScannableType_Scanner = 0x2
    EScannableType_Marker = 0x3
    EScannableType_None = 0x4
    EScannableType_NumTypes = 0x5

class eUpgradeBaseType(IntEnum):
    EUpgradeBaseType_AllMatching = 0x0
    EUpgradeBaseType_NearestMatching = 0x1
    EUpgradeBaseType_NumTypes = 0x2

class eSizeIndicator(IntEnum):
    ESizeIndicator_Small = 0x0
    ESizeIndicator_Medium = 0x1
    ESizeIndicator_Large = 0x2
    ESizeIndicator_NumTypes = 0x3
    ESizeIndicator_Unspecified = 0xFFFFFFFF

class eAxis(IntEnum):
    EAxis_X = 0x0
    EAxis_Y = 0x1
    EAxis_Z = 0x2
    EAxis_NegativeX = 0x3
    EAxis_NegativeY = 0x4
    EAxis_NegativeZ = 0x5
    EAxis_CustomAxis = 0x6
    EAxis_NumTypes = 0x7

class eAntagonistGroup(IntEnum):
    EAntagonistGroup_Player = 0x0
    EAntagonistGroup_Fiends = 0x1
    EAntagonistGroup_Creatures = 0x2
    EAntagonistGroup_Sentinels = 0x3
    EAntagonistGroup_Turrets = 0x4
    EAntagonistGroup_Walls = 0x5
    EAntagonistGroup_NumTypes = 0x6

class eGeneratorUnitType(IntEnum):
    EGeneratorUnitType_MiningUnit = 0x0
    EGeneratorUnitType_GasHarvester = 0x1
    EGeneratorUnitType_SystemHoover = 0x2
    EGeneratorUnitType_NumTypes = 0x3

class eSettlementConstructionLevel(IntEnum):
    ESettlementConstructionLevel_Start = 0x0
    ESettlementConstructionLevel_GroundStorey = 0x1
    ESettlementConstructionLevel_RegularStorey = 0x2
    ESettlementConstructionLevel_Roof = 0x3
    ESettlementConstructionLevel_Decoration = 0x4
    ESettlementConstructionLevel_Other = 0x5
    ESettlementConstructionLevel_NumTypes = 0x6

class eTeleportRewardType(IntEnum):
    ETeleportRewardType_None = 0x0
    ETeleportRewardType_ToBase = 0x1
    ETeleportRewardType_Station0 = 0x2
    ETeleportRewardType_Station1 = 0x3
    ETeleportRewardType_Station2 = 0x4
    ETeleportRewardType_Station3 = 0x5
    ETeleportRewardType_Atlas = 0x6
    ETeleportRewardType_NumTypes = 0x7

class eMode(IntEnum):
    EMode_SetVelocity = 0x0
    EMode_ApplyForce = 0x1
    EMode_NumTypes = 0x2

class eCharacterControlInputValidity(IntEnum):
    ECharacterControlInputValidity_Always = 0x0
    ECharacterControlInputValidity_PadOnly = 0x1
    ECharacterControlInputValidity_KeyboardAnMouseOnly = 0x2
    ECharacterControlInputValidity_NumTypes = 0x3

class eCharacterControlOutputSpace(IntEnum):
    ECharacterControlOutputSpace_CameraRelative = 0x0
    ECharacterControlOutputSpace_CameraRelativeTopDown = 0x1
    ECharacterControlOutputSpace_Raw = 0x2
    ECharacterControlOutputSpace_NumTypes = 0x3

class eInputMouseMode(IntEnum):
    EInputMouseMode_ScreenCentrePos = 0x0
    EInputMouseMode_NumTypes = 0x1

class eScaleInTransformApplicationOnCloth(IntEnum):
    EScaleInTransformApplicationOnCloth_Multiply = 0x0
    EScaleInTransformApplicationOnCloth_Divide = 0x1
    EScaleInTransformApplicationOnCloth_Ignore = 0x2
    EScaleInTransformApplicationOnCloth_NumTypes = 0x3

class eScaleInTransformApplicationOnCollision(IntEnum):
    EScaleInTransformApplicationOnCollision_Multiply = 0x0
    EScaleInTransformApplicationOnCollision_Divide = 0x1
    EScaleInTransformApplicationOnCollision_Ignore = 0x2
    EScaleInTransformApplicationOnCollision_NumTypes = 0x3

class eScaleInTransformApplicationOnAttachment(IntEnum):
    EScaleInTransformApplicationOnAttachment_Multiply = 0x0
    EScaleInTransformApplicationOnAttachment_Divide = 0x1
    EScaleInTransformApplicationOnAttachment_Ignore = 0x2
    EScaleInTransformApplicationOnAttachment_NumTypes = 0x3

class eInitialShapeSource(IntEnum):
    EInitialShapeSource_Rectangular = 0x0
    EInitialShapeSource_TakenFromRenderMesh = 0x1
    EInitialShapeSource_Saved = 0x2
    EInitialShapeSource_NumTypes = 0x3

class eSettlementStatStrength(IntEnum):
    ESettlementStatStrength_PositiveWide = 0x0
    ESettlementStatStrength_PositiveLarge = 0x1
    ESettlementStatStrength_PositiveMedium = 0x2
    ESettlementStatStrength_PositiveSmall = 0x3
    ESettlementStatStrength_NegativeSmall = 0x4
    ESettlementStatStrength_NegativeMedium = 0x5
    ESettlementStatStrength_NegativeLarge = 0x6
    ESettlementStatStrength_NumTypes = 0x7

class eInteractionAction(IntEnum):
    EInteractionAction_PressButton = 0x0
    EInteractionAction_HoldButton = 0x1
    EInteractionAction_Shoot = 0x2
    EInteractionAction_NumTypes = 0x3

class eOverrideInteriorExteriorMarker(IntEnum):
    EOverrideInteriorExteriorMarker_No = 0x0
    EOverrideInteriorExteriorMarker_Interior = 0x1
    EOverrideInteriorExteriorMarker_Exterior = 0x2
    EOverrideInteriorExteriorMarker_NumTypes = 0x3

class eInputRemapMode(IntEnum):
    EInputRemapMode_ReplaceOriginalBinding = 0x0
    EInputRemapMode_AdditionalBinding = 0x1
    EInputRemapMode_NumTypes = 0x2

class eOrigin(IntEnum):
    EOrigin_SpaceStationInternals = 0x0
    EOrigin_SpaceStationBack = 0x1
    EOrigin_FreighterBattle = 0x2
    EOrigin_Freighter = 0x3
    EOrigin_FreighterHangar = 0x4
    EOrigin_AtlasStation = 0x5
    EOrigin_BlackHole = 0x6
    EOrigin_Anomaly = 0x7
    EOrigin_NumTypes = 0x8

class eCameraOrigin(IntEnum):
    ECameraOrigin_ExternalBase = 0x0
    ECameraOrigin_NumTypes = 0x1

class eNPCs(IntEnum):
    ENPCs_None = 0x0
    ENPCs_One = 0x1
    ENPCs_Two = 0x2
    ENPCs_NumTypes = 0x3

class eViewShape(IntEnum):
    EViewShape_Pyramid = 0x0
    EViewShape_Cone = 0x1
    EViewShape_NumTypes = 0x2

class eNodeActiveState(IntEnum):
    ENodeActiveState_Activate = 0x0
    ENodeActiveState_Deactivate = 0x1
    ENodeActiveState_Toggle = 0x2
    ENodeActiveState_Add = 0x3
    ENodeActiveState_Remove = 0x4
    ENodeActiveState_RemoveChildren = 0x5
    ENodeActiveState_NumTypes = 0x6

class eUseCorePartsOnly(IntEnum):
    EUseCorePartsOnly_False = 0x0
    EUseCorePartsOnly_True = 0x1
    EUseCorePartsOnly_DontCare = 0x2
    EUseCorePartsOnly_NumTypes = 0x3

class eAtlasPathProgressType(IntEnum):
    EAtlasPathProgressType_IncrementPathProgress = 0x0
    EAtlasPathProgressType_FinalStoryAtlas = 0x1
    EAtlasPathProgressType_NumTypes = 0x2

class eLegality(IntEnum):
    ELegality_Legal = 0x0
    ELegality_Illegal = 0x1
    ELegality_NumTypes = 0x2
    ELegality_Unspecified = 0xFFFFFFFF

class eTradeCategory(IntEnum):
    ETradeCategory_Mineral = 0x0
    ETradeCategory_Tech = 0x1
    ETradeCategory_Commodity = 0x2
    ETradeCategory_Component = 0x3
    ETradeCategory_Alloy = 0x4
    ETradeCategory_Exotic = 0x5
    ETradeCategory_Energy = 0x6
    ETradeCategory_None = 0x7
    ETradeCategory_SpecialShop = 0x8
    ETradeCategory_NumTypes = 0x9
    ETradeCategory_Unspecified = 0xFFFFFFFF

class eHUDTextDisplayType(IntEnum):
    EHUDTextDisplayType_Full = 0x0
    EHUDTextDisplayType_Compact = 0x1
    EHUDTextDisplayType_EyeLevel = 0x2
    EHUDTextDisplayType_Prompt = 0x3
    EHUDTextDisplayType_Tooltip = 0x4
    EHUDTextDisplayType_NumTypes = 0x5

class eWarpType(IntEnum):
    EWarpType_BlackHole = 0x0
    EWarpType_SpacePOI = 0x1
    EWarpType_NumTypes = 0x2

class eAssistanceType(IntEnum):
    EAssistanceType_Police = 0x0
    EAssistanceType_Wingmen = 0x1
    EAssistanceType_NumTypes = 0x2

class eActionType(IntEnum):
    EActionType_Repair = 0x0
    EActionType_UpdateDamagedComponents = 0x1
    EActionType_CargoPhoneCall = 0x2
    EActionType_NumTypes = 0x3

class eSubmitType(IntEnum):
    ESubmitType_Value = 0x0
    ESubmitType_Stat = 0x1
    ESubmitType_StatsDiff = 0x2
    ESubmitType_NumTypes = 0x3

class eSignalScanType(IntEnum):
    ESignalScanType_None = 0x0
    ESignalScanType_DropPod = 0x1
    ESignalScanType_Shelter = 0x2
    ESignalScanType_Search = 0x3
    ESignalScanType_Relic = 0x4
    ESignalScanType_Industrial = 0x5
    ESignalScanType_Alien = 0x6
    ESignalScanType_CrashedFreighter = 0x7
    ESignalScanType_NumTypes = 0x8

class eTechnologyRarity(IntEnum):
    ETechnologyRarity_Normal = 0x0
    ETechnologyRarity_VeryCommon = 0x1
    ETechnologyRarity_Common = 0x2
    ETechnologyRarity_Rare = 0x3
    ETechnologyRarity_VeryRare = 0x4
    ETechnologyRarity_Impossible = 0x5
    ETechnologyRarity_Always = 0x6
    ETechnologyRarity_NumTypes = 0x7
    ETechnologyRarity_Unspecified = 0xFFFFFFFF

class eAbandonedFreighterMissionState(IntEnum):
    EAbandonedFreighterMissionState_EndRoomComplete = 0x0
    EAbandonedFreighterMissionState_CrewManifestRead = 0x1
    EAbandonedFreighterMissionState_CaptainsLogRead = 0x2
    EAbandonedFreighterMissionState_HazardOn = 0x3
    EAbandonedFreighterMissionState_SlowWalkOn = 0x4
    EAbandonedFreighterMissionState_OpenDoors = 0x5
    EAbandonedFreighterMissionState_NumTypes = 0x6

class eWeightingCurve(IntEnum):
    EWeightingCurve_NoWeighting = 0x0
    EWeightingCurve_MaxIsUncommon = 0x1
    EWeightingCurve_MaxIsRare = 0x2
    EWeightingCurve_MaxIsSuperRare = 0x3
    EWeightingCurve_MinIsUncommon = 0x4
    EWeightingCurve_MinIsRare = 0x5
    EWeightingCurve_MinIsSuperRare = 0x6
    EWeightingCurve_NumTypes = 0x7

class eUnlockableItemTree(IntEnum):
    EUnlockableItemTree_Test = 0x0
    EUnlockableItemTree_BasicBaseParts = 0x1
    EUnlockableItemTree_BasicTechParts = 0x2
    EUnlockableItemTree_BaseParts = 0x3
    EUnlockableItemTree_SpecialBaseParts = 0x4
    EUnlockableItemTree_SuitTech = 0x5
    EUnlockableItemTree_ShipTech = 0x6
    EUnlockableItemTree_WeapTech = 0x7
    EUnlockableItemTree_ExocraftTech = 0x8
    EUnlockableItemTree_CraftProducts = 0x9
    EUnlockableItemTree_FreighterTech = 0xA
    EUnlockableItemTree_S9BaseParts = 0xB
    EUnlockableItemTree_S9ExoTech = 0xC
    EUnlockableItemTree_S9ShipTech = 0xD
    EUnlockableItemTree_NumTypes = 0xE

class eSizeType(IntEnum):
    ESizeType_SciSmall = 0x0
    ESizeType_SciMedium = 0x1
    ESizeType_SciLarge = 0x2
    ESizeType_FgtSmall = 0x3
    ESizeType_FgtMedium = 0x4
    ESizeType_FgtLarge = 0x5
    ESizeType_ShuSmall = 0x6
    ESizeType_ShtMedium = 0x7
    ESizeType_ShtLarge = 0x8
    ESizeType_DrpSmall = 0x9
    ESizeType_DrpMedium = 0xA
    ESizeType_DrpLarge = 0xB
    ESizeType_RoySmall = 0xC
    ESizeType_RoyMedium = 0xD
    ESizeType_RoyLarge = 0xE
    ESizeType_AlienSmall = 0xF
    ESizeType_AlienMedium = 0x10
    ESizeType_AlienLarge = 0x11
    ESizeType_SailSmall = 0x12
    ESizeType_SailMedium = 0x13
    ESizeType_SailLarge = 0x14
    ESizeType_WeaponSmall = 0x15
    ESizeType_WeaponMedium = 0x16
    ESizeType_WeaponLarge = 0x17
    ESizeType_FreighterSmall = 0x18
    ESizeType_FreighterMedium = 0x19
    ESizeType_FreighterLarge = 0x1A
    ESizeType_VehicleSmall = 0x1B
    ESizeType_VehicleMedium = 0x1C
    ESizeType_VehicleLarge = 0x1D
    ESizeType_ChestSmall = 0x1E
    ESizeType_ChestMedium = 0x1F
    ESizeType_ChestLarge = 0x20
    ESizeType_ChestCapsule = 0x21
    ESizeType_Suit = 0x22
    ESizeType_MaintObject = 0x23
    ESizeType_RocketLocker = 0x24
    ESizeType_NumTypes = 0x25

class eTechListRewardOrder(IntEnum):
    ETechListRewardOrder_OneRandom = 0x0
    ETechListRewardOrder_InOrder = 0x1
    ETechListRewardOrder_TryAllRandom = 0x2
    ETechListRewardOrder_NumTypes = 0x3

class eProductListRewardOrder(IntEnum):
    EProductListRewardOrder_OneRandom = 0x0
    EProductListRewardOrder_InOrder = 0x1
    EProductListRewardOrder_TryAllRandom = 0x2
    EProductListRewardOrder_TryUnknownRandom = 0x3
    EProductListRewardOrder_NumTypes = 0x4

class eItemOutOfBoundsBehaviour(IntEnum):
    EItemOutOfBoundsBehaviour_NoCost = 0x0
    EItemOutOfBoundsBehaviour_UseFirst = 0x1
    EItemOutOfBoundsBehaviour_UseLast = 0x2
    EItemOutOfBoundsBehaviour_NumTypes = 0x3

class eItemIndexProvider(IntEnum):
    EItemIndexProvider_None = 0x0
    EItemIndexProvider_Biome = 0x1
    EItemIndexProvider_SubBiome = 0x2
    EItemIndexProvider_NumTypes = 0x3

class eIndexType(IntEnum):
    EIndexType_Regular = 0x0
    EIndexType_Seeded = 0x1
    EIndexType_Random = 0x2
    EIndexType_NumTypes = 0x3

class eProductCategory(IntEnum):
    EProductCategory_Component = 0x0
    EProductCategory_Consumable = 0x1
    EProductCategory_Tradeable = 0x2
    EProductCategory_Curiosity = 0x3
    EProductCategory_BuildingPart = 0x4
    EProductCategory_Procedural = 0x5
    EProductCategory_Emote = 0x6
    EProductCategory_CustomisationPart = 0x7
    EProductCategory_CreatureEgg = 0x8
    EProductCategory_NumTypes = 0x9
    EProductCategory_Unspecified = 0xFFFFFFFF

class eInventoryFilter(IntEnum):
    EInventoryFilter_All = 0x0
    EInventoryFilter_Substance = 0x1
    EInventoryFilter_HighValue = 0x2
    EInventoryFilter_Consumable = 0x3
    EInventoryFilter_Deployable = 0x4
    EInventoryFilter_NumTypes = 0x5

class eCatalogueGroup(IntEnum):
    ECatalogueGroup_MaterialsAndItems = 0x0
    ECatalogueGroup_CraftingAndTechnology = 0x1
    ECatalogueGroup_Buildables = 0x2
    ECatalogueGroup_Recipes = 0x3
    ECatalogueGroup_Wonders = 0x4
    ECatalogueGroup_NumTypes = 0x5

class eStats(IntEnum):
    EStats_Suit = 0x0
    EStats_Weapon = 0x1
    EStats_Ship = 0x2
    EStats_Freighter = 0x3
    EStats_Vehicle = 0x4
    EStats_NumTypes = 0x5

class eOutOfBoundsBehaviour(IntEnum):
    EOutOfBoundsBehaviour_NoCost = 0x0
    EOutOfBoundsBehaviour_UseFirst = 0x1
    EOutOfBoundsBehaviour_UseLast = 0x2
    EOutOfBoundsBehaviour_NumTypes = 0x3

class eIndexProvider(IntEnum):
    EIndexProvider_None = 0x0
    EIndexProvider_ShipSlot = 0x1
    EIndexProvider_ShipClass = 0x2
    EIndexProvider_DailyFreighters = 0x3
    EIndexProvider_WeaponClass = 0x4
    EIndexProvider_WeaponSlot = 0x5
    EIndexProvider_PetSlot = 0x6
    EIndexProvider_PilotSlot = 0x7
    EIndexProvider_PilotRank = 0x8
    EIndexProvider_NumTypes = 0x9

class eProceduralTechnologyCategory(IntEnum):
    EProceduralTechnologyCategory_None = 0x0
    EProceduralTechnologyCategory_Combat = 0x1
    EProceduralTechnologyCategory_Mining = 0x2
    EProceduralTechnologyCategory_Scanning = 0x3
    EProceduralTechnologyCategory_Protection = 0x4
    EProceduralTechnologyCategory_NumTypes = 0x5

class eSeasonSaveStateOnDeath(IntEnum):
    ESeasonSaveStateOnDeath_Standard = 0x0
    ESeasonSaveStateOnDeath_ResetAndQuit = 0x1
    ESeasonSaveStateOnDeath_ResetPosSaveAndQuit = 0x2
    ESeasonSaveStateOnDeath_SaveAndQuit = 0x3
    ESeasonSaveStateOnDeath_NumTypes = 0x4
    ESeasonSaveStateOnDeath_Unspecified = 0xFFFFFFFF

class eQuality(IntEnum):
    EQuality_Normal = 0x0
    EQuality_Rare = 0x1
    EQuality_Epic = 0x2
    EQuality_Legendary = 0x3
    EQuality_Illegal = 0x4
    EQuality_Sentinel = 0x5
    EQuality_NumTypes = 0x6

class eScanBuildingIconType(IntEnum):
    EScanBuildingIconType_None = 0x0
    EScanBuildingIconType_Generic = 0x1
    EScanBuildingIconType_Shelter = 0x2
    EScanBuildingIconType_Relic = 0x3
    EScanBuildingIconType_Factory = 0x4
    EScanBuildingIconType_Unknown = 0x5
    EScanBuildingIconType_Distress = 0x6
    EScanBuildingIconType_Beacon = 0x7
    EScanBuildingIconType_Waypoint = 0x8
    EScanBuildingIconType_SpaceStation = 0x9
    EScanBuildingIconType_TechResource = 0xA
    EScanBuildingIconType_FuelResource = 0xB
    EScanBuildingIconType_MineralResource = 0xC
    EScanBuildingIconType_SpaceAnomaly = 0xD
    EScanBuildingIconType_SpaceAtlas = 0xE
    EScanBuildingIconType_ExternalBase = 0xF
    EScanBuildingIconType_PlanetBaseTerminal = 0x10
    EScanBuildingIconType_Nexus = 0x11
    EScanBuildingIconType_AbandonedFreighter = 0x12
    EScanBuildingIconType_Telescope = 0x13
    EScanBuildingIconType_Outpost = 0x14
    EScanBuildingIconType_UpgradePod = 0x15
    EScanBuildingIconType_Cog = 0x16
    EScanBuildingIconType_Ruins = 0x17
    EScanBuildingIconType_Portal = 0x18
    EScanBuildingIconType_Library = 0x19
    EScanBuildingIconType_Abandoned = 0x1A
    EScanBuildingIconType_SmallBuilding = 0x1B
    EScanBuildingIconType_StoryGlitch = 0x1C
    EScanBuildingIconType_GraveInCave = 0x1D
    EScanBuildingIconType_HoloHub = 0x1E
    EScanBuildingIconType_Settlement = 0x1F
    EScanBuildingIconType_DroneHive = 0x20
    EScanBuildingIconType_NumTypes = 0x21

class eFont(IntEnum):
    EFont_Impact = 0x0
    EFont_Bebas = 0x1
    EFont_GeosansLightWide = 0x2
    EFont_GeosansLight = 0x3
    EFont_GeosansLightMedium = 0x4
    EFont_GeosansLightSmall = 0x5
    EFont_Segoeuib = 0x6
    EFont_Segoeui32 = 0x7
    EFont_NumTypes = 0x8

class eWikiCategory(IntEnum):
    EWikiCategory_NotEnabled = 0x0
    EWikiCategory_Crafting = 0x1
    EWikiCategory_Tech = 0x2
    EWikiCategory_Construction = 0x3
    EWikiCategory_Trade = 0x4
    EWikiCategory_Curio = 0x5
    EWikiCategory_Cooking = 0x6
    EWikiCategory_NumTypes = 0x7

class eResponse(IntEnum):
    EResponse_StayOut = 0x0
    EResponse_ReturnHome = 0x1
    EResponse_CheckForMoreDamage = 0x2
    EResponse_ShowDamagedCaptain = 0x3
    EResponse_ShowExpeditionCaptain = 0x4
    EResponse_AbortExpedition = 0x5
    EResponse_NumTypes = 0x6

class eInventoryToInstallIn(IntEnum):
    EInventoryToInstallIn_Personal = 0x0
    EInventoryToInstallIn_PersonalTech = 0x1
    EInventoryToInstallIn_Ship = 0x2
    EInventoryToInstallIn_ShipTech = 0x3
    EInventoryToInstallIn_Freighter = 0x4
    EInventoryToInstallIn_Vehicle = 0x5
    EInventoryToInstallIn_Weapon = 0x6
    EInventoryToInstallIn_NumTypes = 0x7

class eResponseType(IntEnum):
    EResponseType_DontIntervene = 0x0
    EResponseType_InterveneWithMission = 0x1
    EResponseType_MissionSuccess = 0x2
    EResponseType_MissionFailure = 0x3
    EResponseType_NumTypes = 0x4

class eCentreJourneyDestination(IntEnum):
    ECentreJourneyDestination_Next = 0x0
    ECentreJourneyDestination_Abandoned = 0x1
    ECentreJourneyDestination_Vicious = 0x2
    ECentreJourneyDestination_Lush = 0x3
    ECentreJourneyDestination_Balanced = 0x4
    ECentreJourneyDestination_NumTypes = 0x5

class ePageToOpen(IntEnum):
    EPageToOpen_FreighterShipTransfer = 0x0
    EPageToOpen_DisplayPortalUa = 0x1
    EPageToOpen_ExpeditionSelect = 0x2
    EPageToOpen_TraderInventory = 0x3
    EPageToOpen_ExpeditionDetails = 0x4
    EPageToOpen_ExpeditionDebrief = 0x5
    EPageToOpen_BuildingPartsShop = 0x6
    EPageToOpen_ExocraftShop = 0x7
    EPageToOpen_NexusTechShop = 0x8
    EPageToOpen_ScrapDealerShop = 0x9
    EPageToOpen_BuyShip = 0xA
    EPageToOpen_SettlementManagement = 0xB
    EPageToOpen_SquadronManagement = 0xC
    EPageToOpen_SquadronRecruitment = 0xD
    EPageToOpen_FleetManagement = 0xE
    EPageToOpen_NumTypes = 0xF

class eMultiItemRewardType(IntEnum):
    EMultiItemRewardType_Product = 0x0
    EMultiItemRewardType_Substance = 0x1
    EMultiItemRewardType_ProcTech = 0x2
    EMultiItemRewardType_ProcProduct = 0x3
    EMultiItemRewardType_InventorySlot = 0x4
    EMultiItemRewardType_CommunityTierProduct = 0x5
    EMultiItemRewardType_NumTypes = 0x6

class eInventoryToRepair(IntEnum):
    EInventoryToRepair_Personal = 0x0
    EInventoryToRepair_PersonalTech = 0x1
    EInventoryToRepair_Ship = 0x2
    EInventoryToRepair_ShipTech = 0x3
    EInventoryToRepair_Freighter = 0x4
    EInventoryToRepair_Vehicle = 0x5
    EInventoryToRepair_AttachedAbandonedShip = 0x6
    EInventoryToRepair_Weapon = 0x7
    EInventoryToRepair_NumTypes = 0x8

class eNGuiSizeType(IntEnum):
    eSize_Pixels = 0x0
    eSize_WidthPerCent = 0x1
    eSize_HeightPerCent = 0x2
    eSize_Percent = 0x3
    eSize_RemainingHorizontalArea = 0x4
    eSize_RemainingVerticalArea = 0x5
    eSizeTypeMax = 0x6

class eNGuiInputType(IntEnum):
    eInput_None = 0x0
    eInput_Released = 0x1
    eInput_Click = 0x2
    eInput_RightReleased = 0x3
    eInput_RightClick = 0x4
    eInput_RightDragged = 0xF8
    eInput_RightPressed = 0xF9
    eInput_RightHeld = 0xFA
    eInput_Hover = 0xFB
    eInput_Dragged = 0xFC
    eInput_TouchPressReady = 0xFD
    eInput_Pressed = 0xFE
    eInput_Held = 0xFF

class eScanEventTable(IntEnum):
    EScanEventTable_Space = 0x0
    EScanEventTable_Planet = 0x1
    EScanEventTable_Missions = 0x2
    EScanEventTable_Tutorial = 0x3
    EScanEventTable_MissionsCreative = 0x4
    EScanEventTable_NPCPlanetSite = 0x5
    EScanEventTable_NumTypes = 0x6

class eRewardScanEventOutcome(IntEnum):
    ERewardScanEventOutcome_Success = 0x0
    ERewardScanEventOutcome_Interstellar = 0x1
    ERewardScanEventOutcome_BadData = 0x2
    ERewardScanEventOutcome_FailedToFindBase = 0x3
    ERewardScanEventOutcome_Duplicate = 0x4
    ERewardScanEventOutcome_NoBuilding = 0x5
    ERewardScanEventOutcome_NoSystem = 0x6
    ERewardScanEventOutcome_NumTypes = 0x7
    ERewardScanEventOutcome_Unspecified = 0xFFFFFFFF

class eRequirement(IntEnum):
    ERequirement_CanLearn = 0x0
    ERequirement_CanSpeak = 0x1
    ERequirement_NumTypes = 0x2

class eBuildMenuOption(IntEnum):
    EBuildMenuOption_Place = 0x0
    EBuildMenuOption_ChangeColour = 0x1
    EBuildMenuOption_FreeRotate = 0x2
    EBuildMenuOption_Scale = 0x3
    EBuildMenuOption_SnapRotate = 0x4
    EBuildMenuOption_Move = 0x5
    EBuildMenuOption_Duplicate = 0x6
    EBuildMenuOption_Delete = 0x7
    EBuildMenuOption_ToggleBuildCam = 0x8
    EBuildMenuOption_ToggleSnappingAndCollision = 0x9
    EBuildMenuOption_ToggleSelectionMode = 0xA
    EBuildMenuOption_ToggleWiringMode = 0xB
    EBuildMenuOption_ViewRelatives = 0xC
    EBuildMenuOption_CyclePart = 0xD
    EBuildMenuOption_PlaceWire = 0xE
    EBuildMenuOption_CycleRotateMode = 0xF
    EBuildMenuOption_NumTypes = 0x10

class eInventoryToCheck(IntEnum):
    EInventoryToCheck_All = 0x0
    EInventoryToCheck_Suit = 0x1
    EInventoryToCheck_Ship = 0x2
    EInventoryToCheck_Weapon = 0x3
    EInventoryToCheck_Freighter = 0x4
    EInventoryToCheck_Buggy = 0x5
    EInventoryToCheck_NumTypes = 0x6

class eWordInteractEffect(IntEnum):
    EWordInteractEffect_Pain = 0x0
    EWordInteractEffect_Heal = 0x1
    EWordInteractEffect_NumTypes = 0x2

class eFaceDir(IntEnum):
    EFaceDir_Random = 0x0
    EFaceDir_TowardsPlayer = 0x1
    EFaceDir_SpawnerAt = 0x2
    EFaceDir_NumTypes = 0x3

class eExperienceSpawnType(IntEnum):
    EExperienceSpawnType_Freighter = 0x0
    EExperienceSpawnType_Mission = 0x1
    EExperienceSpawnType_NumTypes = 0x2

class eInteractionBufferType(IntEnum):
    EInteractionBufferType_Distress_Signal = 0x0
    EInteractionBufferType_Crate = 0x1
    EInteractionBufferType_Destructable = 0x2
    EInteractionBufferType_Terrain = 0x3
    EInteractionBufferType_Cost = 0x4
    EInteractionBufferType_Building = 0x5
    EInteractionBufferType_Creature = 0x6
    EInteractionBufferType_Maintenance = 0x7
    EInteractionBufferType_Personal = 0x8
    EInteractionBufferType_Personal_Maintenance = 0x9
    EInteractionBufferType_FireteamSync = 0xA
    EInteractionBufferType_NumTypes = 0xB

class ePlayerDamageType(IntEnum):
    EPlayerDamageType_Normal = 0x0
    EPlayerDamageType_Toxic = 0x1
    EPlayerDamageType_Radioactive = 0x2
    EPlayerDamageType_Freeze = 0x3
    EPlayerDamageType_Scorch = 0x4
    EPlayerDamageType_NumTypes = 0x5

class eSyncBufferType(IntEnum):
    ESyncBufferType_Refiner = 0x0
    ESyncBufferType_Example1 = 0x1
    ESyncBufferType_Example2 = 0x2
    ESyncBufferType_Example3 = 0x3
    ESyncBufferType_NumTypes = 0x4

class eSectorName(IntEnum):
    ESectorName_Generic = 0x0
    ESectorName_Elevated = 0x1
    ESectorName_Low = 0x2
    ESectorName_Trees = 0x3
    ESectorName_LushTrees = 0x4
    ESectorName_Lush = 0x5
    ESectorName_Wet = 0x6
    ESectorName_Cave = 0x7
    ESectorName_Dead = 0x8
    ESectorName_Buildings = 0x9
    ESectorName_Water = 0xA
    ESectorName_Ice = 0xB
    ESectorName_NumTypes = 0xC

class eBaseAutoPowerSetting(IntEnum):
    EBaseAutoPowerSetting_UseDefault = 0x0
    EBaseAutoPowerSetting_ForceDisabled = 0x1
    EBaseAutoPowerSetting_ForceEnabled = 0x2
    EBaseAutoPowerSetting_NumTypes = 0x3

class eMarkovSelector(IntEnum):
    EMarkovSelector_Generic = 0x0
    EMarkovSelector_Mineral = 0x1
    EMarkovSelector_Region_NO = 0x2
    EMarkovSelector_Region_RU = 0x3
    EMarkovSelector_Region_CH = 0x4
    EMarkovSelector_Region_JP = 0x5
    EMarkovSelector_Region_LT = 0x6
    EMarkovSelector_Region_FL = 0x7
    EMarkovSelector_NumTypes = 0x8

class eAltMode(IntEnum):
    EAltMode_None = 0x0
    EAltMode_Normal = 0x1
    EAltMode_Alt = 0x2
    EAltMode_NumTypes = 0x3

class eAccessibleOverride_Layout(IntEnum):
    EAccessibleOverride_Layout_PosX = 0x0
    EAccessibleOverride_Layout_PosY = 0x1
    EAccessibleOverride_Layout_LayerWidth = 0x2
    EAccessibleOverride_Layout_LayerHeight = 0x3
    EAccessibleOverride_Layout_MaxWidth = 0x4
    EAccessibleOverride_Layout_NumTypes = 0x5

class eTeleporterType(IntEnum):
    ETeleporterType_Base = 0x0
    ETeleporterType_Spacestation = 0x1
    ETeleporterType_Atlas = 0x2
    ETeleporterType_PlanetAwayFromShip = 0x3
    ETeleporterType_ExternalBase = 0x4
    ETeleporterType_EmergencyGalaxyFix = 0x5
    ETeleporterType_OnNexus = 0x6
    ETeleporterType_SpacestationFixPosition = 0x7
    ETeleporterType_Settlement = 0x8
    ETeleporterType_Freighter = 0x9
    ETeleporterType_Frigate = 0xA
    ETeleporterType_NumTypes = 0xB

class eAccessibleOverride_Text(IntEnum):
    EAccessibleOverride_Text_FontHeight = 0x0
    EAccessibleOverride_Text_NumTypes = 0x1

class eAction(IntEnum):
    EAction_Click = 0x0
    EAction_Hover = 0x1
    EAction_ArrowLeft = 0x2
    EAction_ArrowRight = 0x3
    EAction_NumTypes = 0x4

class eVROverride_Layout(IntEnum):
    EVROverride_Layout_PosX = 0x0
    EVROverride_Layout_PosY = 0x1
    EVROverride_Layout_LayerWidth = 0x2
    EVROverride_Layout_LayerHeight = 0x3
    EVROverride_Layout_MaxWidth = 0x4
    EVROverride_Layout_NumTypes = 0x5

class eByteBeatToken(IntEnum):
    EByteBeatToken_T = 0x0
    EByteBeatToken_AND = 0x1
    EByteBeatToken_OR = 0x2
    EByteBeatToken_XOR = 0x3
    EByteBeatToken_Plus = 0x4
    EByteBeatToken_Minus = 0x5
    EByteBeatToken_Multiply = 0x6
    EByteBeatToken_Divide = 0x7
    EByteBeatToken_Modulo = 0x8
    EByteBeatToken_ShiftLeft = 0x9
    EByteBeatToken_ShiftRight = 0xA
    EByteBeatToken_Greater = 0xB
    EByteBeatToken_GreaterEqual = 0xC
    EByteBeatToken_Less = 0xD
    EByteBeatToken_LessEqual = 0xE
    EByteBeatToken_Number = 0xF
    EByteBeatToken_OpenParenthesis = 0x10
    EByteBeatToken_CloseParenthesis = 0x11
    EByteBeatToken_NumTypes = 0x12
    EByteBeatToken_Unspecified = 0xFFFFFFFF

class eHUDStartup(IntEnum):
    EHUDStartup_WaitingForInit = 0x0
    EHUDStartup_Startup = 0x1
    EHUDStartup_Booting = 0x2
    EHUDStartup_LifeSupport = 0x3
    EHUDStartup_Shield = 0x4
    EHUDStartup_ShieldEffect = 0x5
    EHUDStartup_Scanner = 0x6
    EHUDStartup_Jetpack = 0x7
    EHUDStartup_Weapon = 0x8
    EHUDStartup_Finishing = 0x9
    EHUDStartup_DiscoverPlanet = 0xA
    EHUDStartup_Done = 0xB
    EHUDStartup_Inactive = 0xC
    EHUDStartup_NumTypes = 0xD

class eActionUseType(IntEnum):
    EActionUseType_Active = 0x0
    EActionUseType_ActiveVR = 0x1
    EActionUseType_ActiveXbox = 0x2
    EActionUseType_ActivePS4 = 0x3
    EActionUseType_Hidden = 0x4
    EActionUseType_Debug = 0x5
    EActionUseType_Obsolete = 0x6
    EActionUseType_NumTypes = 0x7

class eAlign(IntEnum):
    EAlign_Center = 0x0
    EAlign_TopLeft = 0x1
    EAlign_TopRight = 0x2
    EAlign_BottomLeft = 0x3
    EAlign_BottomRight = 0x4
    EAlign_NumTypes = 0x5

class eModelViews(IntEnum):
    EModelViews_Suit = 0x0
    EModelViews_Weapon = 0x1
    EModelViews_Ship = 0x2
    EModelViews_Vehicle = 0x3
    EModelViews_DiscoveryMain = 0x4
    EModelViews_DiscoveryThumbnail = 0x5
    EModelViews_WonderThumbnail = 0x6
    EModelViews_WonderThumbnailCreatureSmall = 0x7
    EModelViews_WonderThumbnailCreatureMed = 0x8
    EModelViews_WonderThumbnailCreatureLarge = 0x9
    EModelViews_WonderThumbnailFloraSmall = 0xA
    EModelViews_WonderThumbnailFloraLarge = 0xB
    EModelViews_WonderThumbnailMineralSmall = 0xC
    EModelViews_WonderThumbnailMineralLarge = 0xD
    EModelViews_ToolboxMain = 0xE
    EModelViews_ToolboxThumbnail = 0xF
    EModelViews_TradeSuit = 0x10
    EModelViews_TradeShip = 0x11
    EModelViews_TradeCompareShips = 0x12
    EModelViews_TradeCompareWeapons = 0x13
    EModelViews_HUDThumbnail = 0x14
    EModelViews_Interaction = 0x15
    EModelViews_Freighter = 0x16
    EModelViews_TradeFreighter = 0x17
    EModelViews_TradeChest = 0x18
    EModelViews_TradeCapsule = 0x19
    EModelViews_TradeFrigate = 0x1A
    EModelViews_TerrainBall = 0x1B
    EModelViews_FreighterChest = 0x1C
    EModelViews_Submarine = 0x1D
    EModelViews_TradeCooker = 0x1E
    EModelViews_SuitRefiner = 0x1F
    EModelViews_FreighterRepair = 0x20
    EModelViews_DiscoveryPlanetaryMapping = 0x21
    EModelViews_Mech = 0x22
    EModelViews_PetThumbnail = 0x23
    EModelViews_PetThumbnailUI = 0x24
    EModelViews_PetLarge = 0x25
    EModelViews_SquadronPilotLarge = 0x26
    EModelViews_SquadronPilotThumbnail = 0x27
    EModelViews_SquadronSpaceshipThumbnail = 0x28
    EModelViews_NumTypes = 0x29

class ePunctuationDelays(IntEnum):
    EPunctuationDelays_Default = 0x0
    EPunctuationDelays_Gek = 0x1
    EPunctuationDelays_Korvax = 0x2
    EPunctuationDelays_Vykeen = 0x3
    EPunctuationDelays_FourthRace = 0x4
    EPunctuationDelays_NumTypes = 0x5

class eInputActionInfoFlags(IntEnum):
    EInputActionInfoFlags_None = 0x0
    EInputActionInfoFlags_HideInControlsPage = 0x1
    EInputActionInfoFlags_HideInMenusMenu = 0x2
    EInputActionInfoFlags_OnlyVR = 0x4
    EInputActionInfoFlags_NumTypes = 0x5
    EInputActionInfoFlags_OnlyNonVR = 0x8

class eVROverride_Text(IntEnum):
    EVROverride_Text_FontHeight = 0x0
    EVROverride_Text_FontIndex = 0x1
    EVROverride_Text_NumTypes = 0x2

class eHitType(IntEnum):
    EHitType_Shootable = 0x0
    EHitType_Terrain = 0x1
    EHitType_Generic = 0x2
    EHitType_NumTypes = 0x3

class eCategoryType(IntEnum):
    ECategoryType_Journey = 0x0
    ECategoryType_Race = 0x1
    ECategoryType_Guild = 0x2
    ECategoryType_NumTypes = 0x3
    ECategoryType_Unspecified = 0xFFFFFFFF

class TkHandle(ctypes.Structure):
    def Root(): pass

    def GetLookup(): pass

    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $7424817F9F79BC6C303B6DE053F4460B
    ]
    ___u0: bytes

class eStatType(IntEnum):
    EStatType_Int = 0x0
    EStatType_Float = 0x1
    EStatType_AvgRate = 0x2
    EStatType_NumTypes = 0x3

class eSummonEventType(IntEnum):
    ESummonEventType_Summon = 0x0
    ESummonEventType_Dismiss = 0x1
    ESummonEventType_NumTypes = 0x2

class eMedalType(IntEnum):
    EMedalType_Standings = 0x0
    EMedalType_Missions = 0x1
    EMedalType_Words = 0x2
    EMedalType_Systems = 0x3
    EMedalType_Sentinels = 0x4
    EMedalType_Pirates = 0x5
    EMedalType_Plants = 0x6
    EMedalType_Units = 0x7
    EMedalType_RaceCreatures = 0x8
    EMedalType_DistanceWarped = 0x9
    EMedalType_NumTypes = 0xA
    EMedalType_Unspecified = 0xFFFFFFFF

class eCrime(IntEnum):
    ECrime_AttackCreature = 0x0
    ECrime_AttackSentinel = 0x1
    ECrime_AttackSentinelLaser = 0x2
    ECrime_KillCreature = 0x3
    ECrime_KillSentinel = 0x4
    ECrime_MineResources = 0x5
    ECrime_HitResources = 0x6
    ECrime_AttackSpaceStation = 0x7
    ECrime_AttackShip = 0x8
    ECrime_AttackPolice = 0x9
    ECrime_KillShip = 0xA
    ECrime_KillPolice = 0xB
    ECrime_TimedShootable = 0xC
    ECrime_NumTypes = 0xD

class eAerialViewMode(IntEnum):
    EAerialViewMode_FaceDown = 0x0
    EAerialViewMode_FaceOut = 0x1
    EAerialViewMode_FaceDownThenOut = 0x2
    EAerialViewMode_NumTypes = 0x3

class eFreighterNPCType(IntEnum):
    EFreighterNPCType_SquadronPilot = 0x0
    EFreighterNPCType_FrigateCaptain = 0x1
    EFreighterNPCType_WorkerBio = 0x2
    EFreighterNPCType_WorkerTech = 0x3
    EFreighterNPCType_WorkerIndustry = 0x4
    EFreighterNPCType_NumTypes = 0x5

class eGameWindowMode(IntEnum):
    EGameWindowMode_Bordered = 0x0
    EGameWindowMode_Borderless = 0x1
    EGameWindowMode_Fullscreen = 0x2
    EGameWindowMode_NumTypes = 0x3

class eGameStateMode(IntEnum):
    EGameStateMode_LoadPreset = 0x0
    EGameStateMode_UserStorage = 0x1
    EGameStateMode_FreshStart = 0x2
    EGameStateMode_NumTypes = 0x3

class eBootMode(IntEnum):
    EBootMode_MinimalSolarSystem = 0x0
    EBootMode_SolarSystem = 0x1
    EBootMode_GalaxyMap = 0x2
    EBootMode_SmokeTest = 0x3
    EBootMode_SmokeTestGalaxyMap = 0x4
    EBootMode_Scratchpad = 0x5
    EBootMode_UnitTest = 0x6
    EBootMode_NumTypes = 0x7

class ePlayerSpawnLocationOverride(IntEnum):
    EPlayerSpawnLocationOverride_None = 0x0
    EPlayerSpawnLocationOverride_FromSettings = 0x1
    EPlayerSpawnLocationOverride_Space = 0x2
    EPlayerSpawnLocationOverride_SpaceStation = 0x3
    EPlayerSpawnLocationOverride_RandomPlanet = 0x4
    EPlayerSpawnLocationOverride_GameStartPlanet = 0x5
    EPlayerSpawnLocationOverride_SpecificLocation = 0x6
    EPlayerSpawnLocationOverride_NumTypes = 0x7

class eSolarSystemBoot(IntEnum):
    ESolarSystemBoot_FromSettings = 0x0
    ESolarSystemBoot_Generate = 0x1
    ESolarSystemBoot_NumTypes = 0x2

class eShaderPreload(IntEnum):
    EShaderPreload_Off = 0x0
    EShaderPreload_Full = 0x1
    EShaderPreload_NumTypes = 0x2

class eBootLoadDelay(IntEnum):
    EBootLoadDelay_LoadAll = 0x0
    EBootLoadDelay_WaitForPlanet = 0x1
    EBootLoadDelay_WaitForNothing = 0x2
    EBootLoadDelay_NumTypes = 0x3

class eRealityMode(IntEnum):
    ERealityMode_LoadPreset = 0x0
    ERealityMode_Generate = 0x1
    ERealityMode_NumTypes = 0x2

class eRecordSetting(IntEnum):
    ERecordSetting_None = 0x0
    ERecordSetting_Record = 0x1
    ERecordSetting_Playback = 0x2
    ERecordSetting_NumTypes = 0x3

class eSmokeTestCycleMode(IntEnum):
    ESmokeTestCycleMode_None = 0x0
    ESmokeTestCycleMode_TourPlanet = 0x1
    ESmokeTestCycleMode_TourSolarSystem = 0x2
    ESmokeTestCycleMode_TourGalaxy = 0x3
    ESmokeTestCycleMode_TourUDAs = 0x4
    ESmokeTestCycleMode_TourShortUDAs = 0x5
    ESmokeTestCycleMode_TourRandomWarps = 0x6
    ESmokeTestCycleMode_NumTypes = 0x7

class eMouseSmoothMode(IntEnum):
    EMouseSmoothMode_Off = 0x0
    EMouseSmoothMode_Sprung = 0x1
    EMouseSmoothMode_NumTypes = 0x2

class eControlModes(IntEnum):
    EControlModes_Normal = 0x0
    EControlModes_FlightStick = 0x1
    EControlModes_Inverted = 0x2
    EControlModes_NumTypes = 0x3

class eSmokeTestScenario(IntEnum):
    ESmokeTestScenario_None = 0x0
    ESmokeTestScenario_TerrainSnapShotFromAltitude = 0x1
    ESmokeTestScenario_BelowCloudLayerSnapShot = 0x2
    ESmokeTestScenario_Flying = 0x3
    ESmokeTestScenario_UltraBiomeSnapShot = 0x4
    ESmokeTestScenario_Walking = 0x5
    ESmokeTestScenario_LeakDetector = 0x6
    ESmokeTestScenario_WalkingSnapshot = 0x7
    ESmokeTestScenario_NumTypes = 0x8

class eProxyType(IntEnum):
    EProxyType_None = 0x0
    EProxyType_ManualURI = 0x1
    EProxyType_InetProxy = 0x2
    EProxyType_NumTypes = 0x3

class eServerEnv(IntEnum):
    EServerEnv_default = 0x0
    EServerEnv_dev = 0x1
    EServerEnv_qa = 0x2
    EServerEnv_prodqa = 0x3
    EServerEnv_prod = 0x4
    EServerEnv_custom = 0x5
    EServerEnv_pentest = 0x6
    EServerEnv_merged = 0x7
    EServerEnv_NumTypes = 0x8

class ePetChatType(IntEnum):
    EPetChatType_Adopted = 0x0
    EPetChatType_Hatched = 0x1
    EPetChatType_Summoned = 0x2
    EPetChatType_Greeting = 0x3
    EPetChatType_Hazard = 0x4
    EPetChatType_Scanning = 0x5
    EPetChatType_PositiveEmote = 0x6
    EPetChatType_HungryEmote = 0x7
    EPetChatType_LonelyEmote = 0x8
    EPetChatType_Go_There = 0x9
    EPetChatType_Come_Here = 0xA
    EPetChatType_Planet = 0xB
    EPetChatType_Mine = 0xC
    EPetChatType_Attack = 0xD
    EPetChatType_Chase = 0xE
    EPetChatType_ReceivedTreat = 0xF
    EPetChatType_Tickled = 0x10
    EPetChatType_Ride = 0x11
    EPetChatType_Egg_Laid = 0x12
    EPetChatType_Customise = 0x13
    EPetChatType_Unsummoned = 0x14
    EPetChatType_NumTypes = 0x15

class eForceDebriefEntryType(IntEnum):
    EForceDebriefEntryType_None = 0x0
    EForceDebriefEntryType_PrimarySuccess = 0x1
    EForceDebriefEntryType_WhaleSuccess = 0x2
    EForceDebriefEntryType_PrimaryFailure = 0x3
    EForceDebriefEntryType_PrimaryDamage = 0x4
    EForceDebriefEntryType_SecondarySuccess = 0x5
    EForceDebriefEntryType_SecondaryFailure = 0x6
    EForceDebriefEntryType_SecondaryDamage = 0x7
    EForceDebriefEntryType_GenericSuccess = 0x8
    EForceDebriefEntryType_GenericFailure = 0x9
    EForceDebriefEntryType_WhaleFailure = 0xA
    EForceDebriefEntryType_NumTypes = 0xB

class ePetVocabularyWord(IntEnum):
    EPetVocabularyWord_Attack = 0x0
    EPetVocabularyWord_Dislike = 0x1
    EPetVocabularyWord_Cute = 0x2
    EPetVocabularyWord_Good = 0x3
    EPetVocabularyWord_Happy = 0x4
    EPetVocabularyWord_Hostile = 0x5
    EPetVocabularyWord_Like = 0x6
    EPetVocabularyWord_Lonely = 0x7
    EPetVocabularyWord_Loved = 0x8
    EPetVocabularyWord_Noise = 0x9
    EPetVocabularyWord_OwnerLove = 0xA
    EPetVocabularyWord_SummonedTrait = 0xB
    EPetVocabularyWord_Hungry = 0xC
    EPetVocabularyWord_Tickles = 0xD
    EPetVocabularyWord_Yummy = 0xE
    EPetVocabularyWord_NumTypes = 0xF

class eSpecialPetChatType(IntEnum):
    ESpecialPetChatType_Monster = 0x0
    ESpecialPetChatType_Quad = 0x1
    ESpecialPetChatType_MiniRobo = 0x2
    ESpecialPetChatType_NumTypes = 0x3

class eReplicateTo(IntEnum):
    EReplicateTo_None = 0x0
    EReplicateTo_Fireteam = 0x1
    EReplicateTo_Fireteam_SameUA = 0x2
    EReplicateTo_Global = 0x3
    EReplicateTo_Global_Distance = 0x4
    EReplicateTo_Fireteam_Distance = 0x5
    EReplicateTo_Fireteam_Global_Distance = 0x6
    EReplicateTo_Not_Fireteam = 0x7
    EReplicateTo_NumTypes = 0x8

class eFriendlyDroneChatType(IntEnum):
    EFriendlyDroneChatType_Summoned = 0x0
    EFriendlyDroneChatType_Unsummoned = 0x1
    EFriendlyDroneChatType_BecomeWanted = 0x2
    EFriendlyDroneChatType_LoseWanted = 0x3
    EFriendlyDroneChatType_Idle = 0x4
    EFriendlyDroneChatType_NumTypes = 0x5

class eSwitchType(IntEnum):
    ESwitchType_None = 0x0
    ESwitchType_Debug = 0x1
    ESwitchType_Enabled = 0x2
    ESwitchType_NumTypes = 0x3

class eBaseBuildingDecorationType(IntEnum):
    EBaseBuildingDecorationType_Normal = 0x0
    EBaseBuildingDecorationType_SurfaceNormal = 0x1
    EBaseBuildingDecorationType_Ceiling = 0x2
    EBaseBuildingDecorationType_Terrain = 0x3
    EBaseBuildingDecorationType_Substance = 0x4
    EBaseBuildingDecorationType_Plant = 0x5
    EBaseBuildingDecorationType_BuildingSurfaceNormal = 0x6
    EBaseBuildingDecorationType_NumTypes = 0x7

class eDependentEffect(IntEnum):
    EDependentEffect_None = 0x0
    EDependentEffect_EnablesRate = 0x1
    EDependentEffect_DisablesRate = 0x2
    EDependentEffect_EnablesConnection = 0x3
    EDependentEffect_DisablesConnection = 0x4
    EDependentEffect_NumTypes = 0x5

class eDependsOnEnvironment(IntEnum):
    EDependsOnEnvironment_None = 0x0
    EDependsOnEnvironment_DayNight = 0x1
    EDependsOnEnvironment_Storms = 0x2
    EDependsOnEnvironment_NumTypes = 0x3

class eDependsOnHotspots(IntEnum):
    EDependsOnHotspots_None = 0x0
    EDependsOnHotspots_Power = 0x1
    EDependsOnHotspots_Mineral = 0x2
    EDependsOnHotspots_Gas = 0x3
    EDependsOnHotspots_NumTypes = 0x4

class eUniqueIdType(IntEnum):
    EUniqueIdType_Invalid = 0x0
    EUniqueIdType_Deterministic = 0x1
    EUniqueIdType_UserSpawned = 0x2
    EUniqueIdType_NumTypes = 0x3

class eGunType(IntEnum):
    EGunType_Laser = 0x0
    EGunType_Projectile = 0x1
    EGunType_Missile = 0x2
    EGunType_NumTypes = 0x3

class eSimpleInteractionType(IntEnum):
    ESimpleInteractionType_Interact = 0x0
    ESimpleInteractionType_Treasure = 0x1
    ESimpleInteractionType_Beacon = 0x2
    ESimpleInteractionType_Scan = 0x3
    ESimpleInteractionType_Save = 0x4
    ESimpleInteractionType_CallShip = 0x5
    ESimpleInteractionType_CallVehicle = 0x6
    ESimpleInteractionType_Word = 0x7
    ESimpleInteractionType_Tech = 0x8
    ESimpleInteractionType_GenericReward = 0x9
    ESimpleInteractionType_Feed = 0xA
    ESimpleInteractionType_Ladder = 0xB
    ESimpleInteractionType_ClaimBase = 0xC
    ESimpleInteractionType_TeleportStartPoint = 0xD
    ESimpleInteractionType_TeleportEndPoint = 0xE
    ESimpleInteractionType_Portal = 0xF
    ESimpleInteractionType_Chest = 0x10
    ESimpleInteractionType_ResourceHarvester = 0x11
    ESimpleInteractionType_BaseCapsule = 0x12
    ESimpleInteractionType_Hologram = 0x13
    ESimpleInteractionType_NPCTerminalMessage = 0x14
    ESimpleInteractionType_VehicleBoot = 0x15
    ESimpleInteractionType_BiomeHarvester = 0x16
    ESimpleInteractionType_FreighterGalacticMap = 0x17
    ESimpleInteractionType_FreighterChest = 0x18
    ESimpleInteractionType_Collectable = 0x19
    ESimpleInteractionType_Chair = 0x1A
    ESimpleInteractionType_BaseTreasureChest = 0x1B
    ESimpleInteractionType_SpawnObject = 0x1C
    ESimpleInteractionType_NoiseBox = 0x1D
    ESimpleInteractionType_AbandFreighterTeleporter = 0x1E
    ESimpleInteractionType_PetEgg = 0x1F
    ESimpleInteractionType_SubstancePickup = 0x20
    ESimpleInteractionType_FreighterTeleport = 0x21
    ESimpleInteractionType_MiniPortalTrigger = 0x22
    ESimpleInteractionType_SuperDoopaScanner = 0x23
    ESimpleInteractionType_NumTypes = 0x24

class eNPCNavSubgraphNodeType(IntEnum):
    ENPCNavSubgraphNodeType_Path = 0x0
    ENPCNavSubgraphNodeType_Connection = 0x1
    ENPCNavSubgraphNodeType_PointOfInterest = 0x2
    ENPCNavSubgraphNodeType_NumTypes = 0x3

class eBaseBuildingMode(IntEnum):
    EBaseBuildingMode_Inactive = 0x0
    EBaseBuildingMode_Selection = 0x1
    EBaseBuildingMode_Placement = 0x2
    EBaseBuildingMode_Browse = 0x3
    EBaseBuildingMode_Relatives = 0x4
    EBaseBuildingMode_NumTypes = 0x5

class eSwatchPrimaryColour(IntEnum):
    ESwatchPrimaryColour_Primary = 0x0
    ESwatchPrimaryColour_Secondary = 0x1
    ESwatchPrimaryColour_Ternary = 0x2
    ESwatchPrimaryColour_Quaternary = 0x3
    ESwatchPrimaryColour_NumTypes = 0x4

class eSwatchSecondaryColour(IntEnum):
    ESwatchSecondaryColour_Primary = 0x0
    ESwatchSecondaryColour_Secondary = 0x1
    ESwatchSecondaryColour_Ternary = 0x2
    ESwatchSecondaryColour_Quaternary = 0x3
    ESwatchSecondaryColour_NumTypes = 0x4

class eBasePartAudioLocation(IntEnum):
    EBasePartAudioLocation_None = 0x0
    EBasePartAudioLocation_Freighter_SpaceWalk = 0x1
    EBasePartAudioLocation_Freighter_BioRoom = 0x2
    EBasePartAudioLocation_Freighter_TechRoom = 0x3
    EBasePartAudioLocation_Freighter_IndustrialRoom = 0x4
    EBasePartAudioLocation_NumTypes = 0x5

class eBaseTerrainEditShape(IntEnum):
    EBaseTerrainEditShape_Cube = 0x0
    EBaseTerrainEditShape_Cylinder = 0x1
    EBaseTerrainEditShape_NumTypes = 0x2

class eTwinCriteria(IntEnum):
    ETwinCriteria_None = 0x0
    ETwinCriteria_MoveToTwin = 0x1
    ETwinCriteria_StretchToTwin = 0x2
    ETwinCriteria_StretchToRaycast = 0x3
    ETwinCriteria_MoveToTwinRelative = 0x4
    ETwinCriteria_NumTypes = 0x5

class eFamilyType(IntEnum):
    EFamilyType_Replacements = 0x0
    EFamilyType_Extras = 0x1
    EFamilyType_NumTypes = 0x2

class eStatsAchievements(IntEnum):
    EStatsAchievements_FirstWarp = 0x0
    EStatsAchievements_FirstDiscovery = 0x1
    EStatsAchievements_NumTypes = 0x2

class eStatsOneShot(IntEnum):
    EStatsOneShot_ShipLanded = 0x0
    EStatsOneShot_ShipLaunched = 0x1
    EStatsOneShot_ShipWarped = 0x2
    EStatsOneShot_WeaponFired = 0x3
    EStatsOneShot_NumTypes = 0x4

class eStatsValue(IntEnum):
    EStatsValue_DistanceJetpacked = 0x0
    EStatsValue_DistanceWalked = 0x1
    EStatsValue_DistanceWarped = 0x2
    EStatsValue_DamageSustained = 0x3
    EStatsValue_NumTypes = 0x4

class eStatMessageType(IntEnum):
    EStatMessageType_Full = 0x0
    EStatMessageType_Quick = 0x1
    EStatMessageType_Silent = 0x2
    EStatMessageType_NumTypes = 0x3

class eStatTrackType(IntEnum):
    EStatTrackType_Set = 0x0
    EStatTrackType_Add = 0x1
    EStatTrackType_Max = 0x2
    EStatTrackType_Min = 0x3
    EStatTrackType_NumTypes = 0x4

class eStatDisplayType(IntEnum):
    EStatDisplayType_None = 0x0
    EStatDisplayType_Sols = 0x1
    EStatDisplayType_Distance = 0x2
    EStatDisplayType_NumTypes = 0x3

class eSnapState(IntEnum):
    ESnapState_IsSnapped = 0x0
    ESnapState_NotSnapped = 0x1
    ESnapState_NumTypes = 0x2

class eBaseStat(IntEnum):
    EBaseStat_HasTeleporter = 0x0
    EBaseStat_HasMainframe = 0x1
    EBaseStat_NumTypes = 0x2

class eInvalidPlacementReason(IntEnum):
    EInvalidPlacementReason_Offline = 0x0
    EInvalidPlacementReason_InvalidBiome = 0x1
    EInvalidPlacementReason_InvalidAboveWater = 0x2
    EInvalidPlacementReason_InvalidUnderwater = 0x3
    EInvalidPlacementReason_PlanetLimitReached = 0x4
    EInvalidPlacementReason_BaseLimitReached = 0x5
    EInvalidPlacementReason_RegionLimitReached = 0x6
    EInvalidPlacementReason_InvalidOverlappingAnyBase = 0x7
    EInvalidPlacementReason_InvalidOverlappingSettlement = 0x8
    EInvalidPlacementReason_InvalidOverlappingBase = 0x9
    EInvalidPlacementReason_OutOfBaseRange = 0xA
    EInvalidPlacementReason_OutOfConnectionRange = 0xB
    EInvalidPlacementReason_LinkGridMismatch = 0xC
    EInvalidPlacementReason_InsufficientResources = 0xD
    EInvalidPlacementReason_ComplexityLimitReached = 0xE
    EInvalidPlacementReason_SubstanceOnly = 0xF
    EInvalidPlacementReason_InvalidPosition = 0x10
    EInvalidPlacementReason_InvalidSnap = 0x11
    EInvalidPlacementReason_MustPlaceOnTerrain = 0x12
    EInvalidPlacementReason_Collision = 0x13
    EInvalidPlacementReason_ShipInside = 0x14
    EInvalidPlacementReason_PlayerInside = 0x15
    EInvalidPlacementReason_NumTypes = 0x16

class eDecorationLayer(IntEnum):
    EDecorationLayer_Stairs = 0x0
    EDecorationLayer_Corridor = 0x1
    EDecorationLayer_Room = 0x2
    EDecorationLayer_Door = 0x3
    EDecorationLayer_Decoration1 = 0x4
    EDecorationLayer_Decoration2 = 0x5
    EDecorationLayer_Decoration3 = 0x6
    EDecorationLayer_DecorationCorridor = 0x7
    EDecorationLayer_NumTypes = 0x8

class ePackageType(IntEnum):
    EPackageType_Unknown = 0x0
    EPackageType_Discovery = 0x1
    EPackageType_Message = 0x2
    EPackageType_NumTypes = 0x3

class eMonumentRole(IntEnum):
    EMonumentRole_Creator = 0x0
    EMonumentRole_CoCreator = 0x1
    EMonumentRole_NumTypes = 0x2

class eByteBeatEnvelope(IntEnum):
    EByteBeatEnvelope_Short = 0x0
    EByteBeatEnvelope_Med = 0x1
    EByteBeatEnvelope_Long = 0x2
    EByteBeatEnvelope_NumTypes = 0x3

class eByteBeatWave(IntEnum):
    EByteBeatWave_SawTooth = 0x0
    EByteBeatWave_Sine = 0x1
    EByteBeatWave_Square = 0x2
    EByteBeatWave_Triangle = 0x3
    EByteBeatWave_NumTypes = 0x4

class eAugmentMode(IntEnum):
    EAugmentMode_Add = 0x0
    EAugmentMode_Multiply = 0x1
    EAugmentMode_Max = 0x2
    EAugmentMode_NumTypes = 0x3

class eDestinationSortType(IntEnum):
    EDestinationSortType_NearestPotal = 0x0
    EDestinationSortType_BaseBuildingConnection = 0x1
    EDestinationSortType_AbandonedFreighter = 0x2
    EDestinationSortType_PortalNearestPlayerShip = 0x3
    EDestinationSortType_NumTypes = 0x4

class _USER_ACTIVITY_PRESENCE(IntEnum):
    PowerUserPresent = 0x0
    PowerUserNotPresent = 0x1
    PowerUserInactive = 0x2
    PowerUserMaximum = 0x3

class eCursor(IntEnum):
    ECursor_None = 0x0
    ECursor_Arrow = 0x1
    ECursor_ResizeNWSE = 0x2
    ECursor_ResizeNESW = 0x3
    ECursor_ResizeWE = 0x4
    ECursor_ResizeNS = 0x5
    ECursor_Text = 0x6
    ECursor_No = 0x7
    ECursor_ValueSlider = 0x8
    ECursor_Drop = 0x9
    ECursor_NumTypes = 0xA

class eWindowMode(IntEnum):
    EWindowMode_Bordered = 0x0
    EWindowMode_Borderless = 0x1
    EWindowMode_Fullscreen = 0x2

class eGeneratedInstanceInfo(IntEnum):
    _None = 0x0
    Normal = 0x1
    ContainsRarePart = 0x2
    WeirdAsteroidShape = 0x3

class ReplacesCorHdrNumericDefines(IntEnum):
    COMIMAGE_FLAGS_ILONLY = 0x1
    COMIMAGE_FLAGS_32BITREQUIRED = 0x2
    COMIMAGE_FLAGS_IL_LIBRARY = 0x4
    COR_VERSION_MINOR = 0x5
    COMIMAGE_FLAGS_STRONGNAMESIGNED = 0x8
    COMIMAGE_FLAGS_NATIVE_ENTRYPOINT = 0x10
    IMAGE_COR_EATJ_THUNK_SIZE = 0x20
    COR_ILMETHOD_SECT_SMALL_MAX_DATASIZE = 0xFF
    MAX_CLASS_NAME = 0x400
    COMIMAGE_FLAGS_TRACKDEBUGDATA = 0x10000
    COMIMAGE_FLAGS_32BITPREFERRED = 0x20000

class hkMemoryType(IntEnum):
    Heap = 0x0
    Stack = 0x1
    Temp = 0x2
    DebugHeap = 0x3
    DebugStack = 0x4
    DebugTemp = 0x5
    NumTypes = 0x6
    EachType = 0x7

class _TP_CALLBACK_PRIORITY(IntEnum):
    TP_CALLBACK_PRIORITY_HIGH = 0x0
    TP_CALLBACK_PRIORITY_NORMAL = 0x1
    TP_CALLBACK_PRIORITY_LOW = 0x2
    TP_CALLBACK_PRIORITY_INVALID = 0x3

class eMemoryError(IntEnum):
    EMemoryError_FailedAllocFromFullPool = 0x0

class eMemoryHeapType(IntEnum):
    ELinearHeap = 0x0
    EDynamicHeap = 0x1
    ESmallBlockHeap = 0x2
    ESingleBufferHeap = 0x3
    EDoubleBufferHeap = 0x4
    EDoubleBufferHeapPausable = 0x5
    EAtomicLinearHeap = 0x6
    EHeapType_Num = 0x7

class eNGuiSnapSettings(IntEnum):
    ESnapSetting_ResizeCornerTolerance = 0x0
    ESnapSetting_ResizeSnapWeight = 0x1
    ESnapSetting_DragSnapWeight = 0x2
    ESnapSetting_UnsnapThreshold = 0x3
    ESnapSetting_MaxNum = 0x4

class eNGuiAnimationEvent(IntEnum):
    eAnimationEvent_Hover = 0x0
    eAnimationEvent_Selected = 0x1
    eAnimationEvent_Active = 0x2
    eAnimationEvent_Custom1 = 0x3
    eAnimationEvent_Custom2 = 0x4
    eAnimationEvent_Custom3 = 0x5
    eAnimationEventNum = 0x6
    eAnimationEvent_None = 0xFF

class hkVector4ComparisonMask(ctypes.Structure):
    class Mask(IntEnum):
        INDEX_X = 0x0
        INDEX_Y = 0x1
        INDEX_Z = 0x2
        INDEX_W = 0x3
        MASK_Z = 0x4
        MASK_XZ = 0x5
        MASK_YZ = 0x6
        MASK_XYZ = 0x7
        MASK_W = 0x8
        MASK_XW = 0x9
        MASK_YW = 0xA
        MASK_XYW = 0xB
        MASK_ZW = 0xC
        MASK_XZW = 0xD
        MASK_YZW = 0xE
        MASK_XYZW = 0xF


class hkMathAccuracyMode(IntEnum):
    HK_ACC_FULL = 0x0
    HK_ACC_10_BIT = 0x1
    HK_ACC_5_BIT = 0x2

class hkMathDivByZeroMode(IntEnum):
    HK_DIV_IGNORE = 0x0
    HK_DIV_SET_ZERO = 0x1
    HK_DIV_SET_MAX = 0x2
    HK_DIV_SET_HIGH = 0x3

class hkMathSqrtMode(IntEnum):
    HK_SQRT_IGNORE = 0x0
    HK_SQRT_SET_ZERO = 0x1

class eNGuiAnimationType(IntEnum):
    eAnimationType_Begin = 0x0
    eAnimationType_End = 0x1
    eAnimationTypeNum = 0x2

class eNGuiAlignmentType(IntEnum):
    eAlignment_TopLeft = 0x0
    eAlignment_TopCenter = 0x1
    eAlignment_TopRight = 0x2
    eAlignment_CenterLeft = 0x3
    eAlignment_Center = 0x4
    eAlignment_CenterRight = 0x5
    eAlignment_BottomLeft = 0x6
    eAlignment_BottomCenter = 0x7
    eAlignment_BottomRight = 0x8
    eAlignmentTypeMax = 0x9

class eNGuiKeyType(IntEnum):
    eKey_Tab = 0x0
    eKey_LeftArrow = 0x1
    eKey_RightArrow = 0x2
    eKey_UpArrow = 0x3
    eKey_DownArrow = 0x4
    eKey_Home = 0x5
    eKey_End = 0x6
    eKey_Delete = 0x7
    eKey_Backspace = 0x8
    eKey_Enter = 0x9
    eKey_Escape = 0xA
    eKey_A = 0xB
    eKey_C = 0xC
    eKey_V = 0xD
    eKey_X = 0xE
    eKey_Y = 0xF
    eKey_Z = 0x10
    eKey_F = 0x11
    eKey_N = 0x12
    eKey_COUNT = 0x13

class hkMathIoMode(IntEnum):
    HK_IO_NATIVE_ALIGNED = 0x0
    HK_IO_SIMD_ALIGNED = 0x1

class eNGuiEditorIcons(IntEnum):
    EIcon_camera_retro = 0x21
    EIcon_caret_down = 0x22
    EIcon_caret_down_two = 0x23
    EIcon_caret_left = 0x24
    EIcon_caret_right = 0x25
    EIcon_caret_square_down = 0x26
    EIcon_caret_square_right = 0x27
    EIcon_caret_square_up = 0x28
    EIcon_caret_up = 0x29
    EIcon_caret_up_down = 0x2A
    EIcon_caret_up_two = 0x2B
    EIcon_certificate = 0x2C
    EIcon_chat_bubble = 0x2D
    EIcon_chat_bubble_two = 0x2E
    EIcon_check_mark = 0x2F
    EIcon_briefcase_case_two = 0x30
    EIcon_bug = 0x31
    EIcon_building = 0x32
    EIcon_bullhorn = 0x33
    EIcon_bullseye = 0x34
    EIcon_calendar = 0x35
    EIcon_calendar_1 = 0x36
    EIcon_call_phone = 0x37
    EIcon_call_phone_square = 0x38
    EIcon_camera = 0x39
    EIcon_check_mark_2 = 0x3A
    EIcon_check_mark_circle = 0x3B
    EIcon_check_mark_circle_two = 0x3C
    EIcon_check_square = 0x3D
    EIcon_chevron_down = 0x3E
    EIcon_chevron_down_circle = 0x3F
    EIcon_chevron_left = 0x40
    EIcon_arrow_circle_up_line = 0x41
    EIcon_arrow_down = 0x42
    EIcon_arrow_down_1 = 0x43
    EIcon_arrow_horizontal = 0x44
    EIcon_arrow_left = 0x45
    EIcon_arrow_left_1 = 0x46
    EIcon_arrow_right = 0x47
    EIcon_arrow_right_1 = 0x48
    EIcon_arrow_up = 0x49
    EIcon_arrow_up_1 = 0x4A
    EIcon_arrow_vertical = 0x4B
    EIcon_asterisk = 0x4C
    EIcon_backward = 0x4D
    EIcon_bar_chart = 0x4E
    EIcon_barcode = 0x4F
    EIcon_beer = 0x50
    EIcon_bell = 0x51
    EIcon_bell_two = 0x52
    EIcon_bitbucket = 0x53
    EIcon_bitbucket_square = 0x54
    EIcon_bitcoin = 0x55
    EIcon_bold = 0x56
    EIcon_book = 0x57
    EIcon_bookmark = 0x58
    EIcon_box_inbox = 0x59
    EIcon_briefcase_case = 0x5A
    EIcon_chevron_right = 0x5B
    EIcon_cloud_download = 0x5C
    EIcon_chevron_up = 0x5D
    EIcon_chevron_up_circle = 0x5E
    EIcon_circle = 0x5F
    EIcon_circle_line = 0x60
    EIcon_adn = 0x61
    EIcon_aid_medkit_case = 0x62
    EIcon_align_center = 0x63
    EIcon_align_justify = 0x64
    EIcon_align_justify_1 = 0x65
    EIcon_align_left = 0x66
    EIcon_align_right = 0x67
    EIcon_ambulance = 0x68
    EIcon_anchor = 0x69
    EIcon_android = 0x6A
    EIcon_angle_double_down = 0x6B
    EIcon_angle_double_left = 0x6C
    EIcon_angle_double_right = 0x6D
    EIcon_angle_double_up = 0x6E
    EIcon_angle_down = 0x6F
    EIcon_angle_left = 0x70
    EIcon_angle_right = 0x71
    EIcon_angle_up = 0x72
    EIcon_apple = 0x73
    EIcon_archive = 0x74
    EIcon_arrow_circle_down = 0x75
    EIcon_arrow_circle_down_line = 0x76
    EIcon_arrow_circle_left = 0x77
    EIcon_arrow_circle_right = 0x78
    EIcon_arrow_circle_right_line = 0x79
    EIcon_arrow_circle_up = 0x7A
    EIcon_clipboard = 0x7B
    EIcon_clock = 0x7C
    EIcon_close_delete = 0x7D
    EIcon_cloud = 0x7E
    EIcon_cloud_upload = 0xE000
    EIcon_code = 0xE001
    EIcon_code_fork = 0xE002
    EIcon_cog_gear = 0xE003
    EIcon_columns = 0xE004
    EIcon_compass = 0xE005
    EIcon_contrast = 0xE006
    EIcon_credit_card = 0xE007
    EIcon_crop = 0xE008
    EIcon_crosshair_aim = 0xE009
    EIcon_css3 = 0xE00A
    EIcon_cup = 0xE00B
    EIcon_cursor_move = 0xE00C
    EIcon_cursor_move_two = 0xE00D
    EIcon_cut_scissor = 0xE00E
    EIcon_cutlery = 0xE00F
    EIcon_delete_circle = 0xE010
    EIcon_denied_block = 0xE011
    EIcon_dislike = 0xE012
    EIcon_doctor = 0xE013
    EIcon_document = 0xE014
    EIcon_dollar = 0xE015
    EIcon_download = 0xE016
    EIcon_dribble = 0xE017
    EIcon_dropbox = 0xE018
    EIcon_earser = 0xE019
    EIcon_edit_write = 0xE01A
    EIcon_eject = 0xE01B
    EIcon_ellipsis = 0xE01C
    EIcon_ellipsis_2 = 0xE01D
    EIcon_email_envelope = 0xE01E
    EIcon_euro = 0xE01F
    EIcon_exchange = 0xE020
    EIcon_exclamation = 0xE021
    EIcon_exclamation_circle = 0xE022
    EIcon_exclamation_triangle = 0xE023
    EIcon_external_link = 0xE024
    EIcon_external_link_sqaure = 0xE025
    EIcon_eye = 0xE026
    EIcon_eye_slash_close = 0xE027
    EIcon_facebook = 0xE028
    EIcon_facebook_square = 0xE029
    EIcon_fast_backward = 0xE02A
    EIcon_fast_forward = 0xE02B
    EIcon_female = 0xE02C
    EIcon_fighter_jet = 0xE02D
    EIcon_file = 0xE02E
    EIcon_file_document = 0xE02F
    EIcon_file_text = 0xE030
    EIcon_files = 0xE031
    EIcon_filmstrip = 0xE032
    EIcon_filter = 0xE033
    EIcon_fire = 0xE034
    EIcon_fire_extinguisher = 0xE035
    EIcon_flag = 0xE036
    EIcon_flag_1 = 0xE037
    EIcon_flag_goal = 0xE038
    EIcon_flash = 0xE039
    EIcon_flickr = 0xE03A
    EIcon_folder = 0xE03B
    EIcon_folder2 = 0xE03C
    EIcon_folder3 = 0xE03D
    EIcon_folder_open = 0xE03E
    EIcon_font = 0xE03F
    EIcon_fontawesome_webfont = 0xE040
    EIcon_forward = 0xE041
    EIcon_foursquare = 0xE042
    EIcon_gamepad = 0xE043
    EIcon_gears_setting = 0xE044
    EIcon_github = 0xE045
    EIcon_github_square = 0xE046
    EIcon_gittip = 0xE047
    EIcon_glass = 0xE048
    EIcon_globe_world = 0xE049
    EIcon_google_plus = 0xE04A
    EIcon_google_plus_1 = 0xE04B
    EIcon_group = 0xE04C
    EIcon_hand_down = 0xE04D
    EIcon_hand_right = 0xE04E
    EIcon_hand_up = 0xE04F
    EIcon_harddrive = 0xE050
    EIcon_headphone = 0xE051
    EIcon_heart = 0xE052
    EIcon_heart_empty = 0xE053
    EIcon_home = 0xE054
    EIcon_hospital = 0xE055
    EIcon_hospital_square = 0xE056
    EIcon_hospital_square_two = 0xE057
    EIcon_html5 = 0xE058
    EIcon_indent = 0xE059
    EIcon_information = 0xE05A
    EIcon_information_circle = 0xE05B
    EIcon_instagram = 0xE05C
    EIcon_italic = 0xE05D
    EIcon_key = 0xE05E
    EIcon_keyboard = 0xE05F
    EIcon_laptop = 0xE060
    EIcon_leaf = 0xE061
    EIcon_left = 0xE062
    EIcon_left_circle = 0xE063
    EIcon_legal = 0xE064
    EIcon_lemon = 0xE065
    EIcon_letter_mail = 0xE066
    EIcon_level_down = 0xE067
    EIcon_level_up = 0xE068
    EIcon_lightbulb = 0xE069
    EIcon_like = 0xE06A
    EIcon_like_filled = 0xE06B
    EIcon_link = 0xE06C
    EIcon_link_broken = 0xE06D
    EIcon_linkedin = 0xE06E
    EIcon_linkedin_square = 0xE06F
    EIcon_linux = 0xE070
    EIcon_list = 0xE071
    EIcon_list_2 = 0xE072
    EIcon_list_bullet = 0xE073
    EIcon_list_number = 0xE074
    EIcon_location_arrow = 0xE075
    EIcon_lock = 0xE076
    EIcon_log_in = 0xE077
    EIcon_log_out = 0xE078
    EIcon_magic_wand = 0xE079
    EIcon_magnet = 0xE07A
    EIcon_mail_forward = 0xE07B
    EIcon_male = 0xE07C
    EIcon_maxcdn = 0xE07D
    EIcon_mic = 0xE07E
    EIcon_mic_no = 0xE07F
    EIcon_minus = 0xE080
    EIcon_minus_circle = 0xE081
    EIcon_minus_line = 0xE082
    EIcon_minus_square = 0xE083
    EIcon_minus_square_1 = 0xE084
    EIcon_money_banknote = 0xE085
    EIcon_moon_sleep = 0xE086
    EIcon_music_note = 0xE087
    EIcon_outdent = 0xE088
    EIcon_paper_clip = 0xE089
    EIcon_pause = 0xE08A
    EIcon_pencil = 0xE08B
    EIcon_pencil_square = 0xE08C
    EIcon_phone_mobile = 0xE08D
    EIcon_photo = 0xE08E
    EIcon_pin_map = 0xE08F
    EIcon_pin_map_2 = 0xE090
    EIcon_pinterest = 0xE091
    EIcon_pinterest_square = 0xE092
    EIcon_plane_airport = 0xE093
    EIcon_play = 0xE094
    EIcon_play_circle = 0xE095
    EIcon_plus = 0xE096
    EIcon_plus_circle = 0xE097
    EIcon_plus_square = 0xE098
    EIcon_pound = 0xE099
    EIcon_power_off = 0xE09A
    EIcon_present_gift = 0xE09B
    EIcon_prize_award = 0xE09C
    EIcon_ptint = 0xE09D
    EIcon_puzzle = 0xE09E
    EIcon_qrcode = 0xE09F
    EIcon_question = 0xE0A0
    EIcon_question_1 = 0xE0A1
    EIcon_quote_left = 0xE0A2
    EIcon_quote_right = 0xE0A3
    EIcon_random = 0xE0A4
    EIcon_refresh = 0xE0A5
    EIcon_remove = 0xE0A6
    EIcon_renren = 0xE0A7
    EIcon_repeat_redo = 0xE0A8
    EIcon_reply_all = 0xE0A9
    EIcon_reply_mail = 0xE0AA
    EIcon_resize_down = 0xE0AB
    EIcon_resize_expand = 0xE0AC
    EIcon_retweet = 0xE0AD
    EIcon_right = 0xE0AE
    EIcon_road = 0xE0AF
    EIcon_rocket = 0xE0B0
    EIcon_rss = 0xE0B1
    EIcon_rss_two = 0xE0B2
    EIcon_rupee = 0xE0B3
    EIcon_save_disk = 0xE0B4
    EIcon_science_laboratory = 0xE0B5
    EIcon_screen_desktop = 0xE0B6
    EIcon_search_find = 0xE0B7
    EIcon_share_square = 0xE0B8
    EIcon_share_square_1 = 0xE0B9
    EIcon_shield = 0xE0BA
    EIcon_shopping_cart = 0xE0BB
    EIcon_signal = 0xE0BC
    EIcon_sitemap = 0xE0BD
    EIcon_skype = 0xE0BE
    EIcon_smile = 0xE0BF
    EIcon_smiley_frown = 0xE0C0
    EIcon_smiley_meh = 0xE0C1
    EIcon_sort_alpha = 0xE0C2
    EIcon_sort_alpha_two = 0xE0C3
    EIcon_sort_amount = 0xE0C4
    EIcon_sort_amount_two = 0xE0C5
    EIcon_sort_numeric = 0xE0C6
    EIcon_sort_numeric_two = 0xE0C7
    EIcon_spinner = 0xE0C8
    EIcon_square = 0xE0C9
    EIcon_square_line = 0xE0CA
    EIcon_stackoverflow = 0xE0CB
    EIcon_star = 0xE0CC
    EIcon_star_half = 0xE0CD
    EIcon_star_half_1 = 0xE0CE
    EIcon_star_two = 0xE0CF
    EIcon_step_backward = 0xE0D0
    EIcon_step_forward = 0xE0D1
    EIcon_stethoscope = 0xE0D2
    EIcon_stop = 0xE0D3
    EIcon_strikethrough = 0xE0D4
    EIcon_subscript = 0xE0D5
    EIcon_sun = 0xE0D6
    EIcon_superscript = 0xE0D7
    EIcon_table = 0xE0D8
    EIcon_tablet = 0xE0D9
    EIcon_tachometer = 0xE0DA
    EIcon_tag = 0xE0DB
    EIcon_tag_2 = 0xE0DC
    EIcon_tags = 0xE0DD
    EIcon_talk_chat = 0xE0DE
    EIcon_talk_chat_2 = 0xE0DF
    EIcon_tasks = 0xE0E0
    EIcon_terminal = 0xE0E1
    EIcon_text_height = 0xE0E2
    EIcon_text_width = 0xE0E3
    EIcon_thumbnails = 0xE0E4
    EIcon_thumbnails_large = 0xE0E5
    EIcon_ticket = 0xE0E6
    EIcon_tint = 0xE0E7
    EIcon_trash_bin = 0xE0E8
    EIcon_trello = 0xE0E9
    EIcon_truck = 0xE0EA
    EIcon_tumblr = 0xE0EB
    EIcon_tumblr_square = 0xE0EC
    EIcon_twitter = 0xE0ED
    EIcon_twitter_square = 0xE0EE
    EIcon_umbrella = 0xE0EF
    EIcon_underline = 0xE0F0
    EIcon_undo = 0xE0F1
    EIcon_unlike = 0xE0F2
    EIcon_unlock = 0xE0F3
    EIcon_unlock_2 = 0xE0F4
    EIcon_upload = 0xE0F5
    EIcon_user = 0xE0F6
    EIcon_video_camera = 0xE0F7
    EIcon_vk = 0xE0F8
    EIcon_volume_down = 0xE0F9
    EIcon_volume_down_1 = 0xE0FA
    EIcon_volume_up = 0xE0FB
    EIcon_weibo = 0xE0FC
    EIcon_window_list = 0xE0FD
    EIcon_windows = 0xE0FE
    EIcon_won = 0xE0FF
    EIcon_wrench = 0xE100
    EIcon_xing = 0xE101
    EIcon_xing_square = 0xE102
    EIcon_yen = 0xE103
    EIcon_youtube = 0xE104
    EIcon_youtube_play = 0xE105
    EIcon_youtube_square = 0xE106
    EIcon_yuan = 0xE107
    EIcon_zoom_in = 0xE108
    EIcon_zoom_out = 0xE109

class eTkIDNullFlag(IntEnum):
    NullId = 0x0

class JOB_OBJECT_NET_RATE_CONTROL_FLAGS(IntEnum):
    JOB_OBJECT_NET_RATE_CONTROL_ENABLE = 0x1
    JOB_OBJECT_NET_RATE_CONTROL_MAX_BANDWIDTH = 0x2
    JOB_OBJECT_NET_RATE_CONTROL_DSCP_TAG = 0x4
    JOB_OBJECT_NET_RATE_CONTROL_VALID_FLAGS = 0x7

class JOB_OBJECT_IO_RATE_CONTROL_FLAGS(IntEnum):
    JOB_OBJECT_IO_RATE_CONTROL_ENABLE = 0x1
    JOB_OBJECT_IO_RATE_CONTROL_STANDALONE_VOLUME = 0x2
    JOB_OBJECT_IO_RATE_CONTROL_FORCE_UNIT_ACCESS_ALL = 0x4
    JOB_OBJECT_IO_RATE_CONTROL_FORCE_UNIT_ACCESS_ON_SOFT_CAP = 0x8
    JOB_OBJECT_IO_RATE_CONTROL_VALID_FLAGS = 0xF

class eFrameRate(IntEnum):
    EFPS_Variable = 0x0
    EFPS_15Hz = 0xF
    EFPS_25Hz = 0x19
    EFPS_30Hz = 0x1E
    EFPS_50Hz = 0x32
    EFPS_60Hz = 0x3C

class hkCriticalSectionLock(ctypes.Structure):
    class Mode(IntEnum):
        Enter = 0x0
        Try = 0x1
        Defer = 0x2

    _fields_ = [
        ("m_section", ctypes.c_ulonglong),
        ("m_hasEntered", ctypes.c_ubyte),
    ]
    m_section: int
    m_hasEntered: bool

class DISPLAYCONFIG_SCANLINE_ORDERING(IntEnum):
    DISPLAYCONFIG_SCANLINE_ORDERING_UNSPECIFIED = 0x0
    DISPLAYCONFIG_SCANLINE_ORDERING_PROGRESSIVE = 0x1
    DISPLAYCONFIG_SCANLINE_ORDERING_INTERLACED = 0x2
    DISPLAYCONFIG_SCANLINE_ORDERING_INTERLACED_LOWERFIELDFIRST = 0x3
    DISPLAYCONFIG_SCANLINE_ORDERING_FORCE_UINT32 = 0xFFFFFFFF

class hkVectorConstant(IntEnum):
    HK_QUADREAL_BEGIN = 0x0
    HK_QUADREAL_NAN = 0x1
    HK_QUADREAL_INF = 0x2
    HK_QUADREAL_MINUS_INF = 0x3
    HK_QUADREAL_MINUS1 = 0x4
    HK_QUADREAL_0 = 0x5
    HK_QUADREAL_1 = 0x6
    HK_QUADREAL_2 = 0x7
    HK_QUADREAL_3 = 0x8
    HK_QUADREAL_4 = 0x9
    HK_QUADREAL_5 = 0xA
    HK_QUADREAL_6 = 0xB
    HK_QUADREAL_7 = 0xC
    HK_QUADREAL_8 = 0xD
    HK_QUADREAL_15 = 0xE
    HK_QUADREAL_16 = 0xF
    HK_QUADREAL_255 = 0x10
    HK_QUADREAL_256 = 0x11
    HK_QUADREAL_2_POW_23 = 0x12
    HK_QUADREAL_INV_0 = 0x13
    HK_QUADREAL_INV_1 = 0x14
    HK_QUADREAL_INV_2 = 0x15
    HK_QUADREAL_INV_3 = 0x16
    HK_QUADREAL_INV_4 = 0x17
    HK_QUADREAL_INV_5 = 0x18
    HK_QUADREAL_INV_6 = 0x19
    HK_QUADREAL_INV_7 = 0x1A
    HK_QUADREAL_INV_8 = 0x1B
    HK_QUADREAL_INV_15 = 0x1C
    HK_QUADREAL_INV_127 = 0x1D
    HK_QUADREAL_INV_226 = 0x1E
    HK_QUADREAL_INV_255 = 0x1F
    HK_QUADREAL_1000 = 0x20
    HK_QUADREAL_0100 = 0x21
    HK_QUADREAL_0010 = 0x22
    HK_QUADREAL_0001 = 0x23
    HK_QUADREAL_MAX = 0x24
    HK_QUADREAL_HIGH = 0x25
    HK_QUADREAL_EPS = 0x26
    HK_QUADREAL_EPS_SQRD = 0x27
    HK_QUADREAL_MIN = 0x28
    HK_QUADREAL_MINUS_MAX = 0x29
    HK_QUADREAL_MINUS_MIN = 0x2A
    HK_QUADREAL_m11m11 = 0x2B
    HK_QUADREAL_1m11m1 = 0x2C
    HK_QUADREAL_1010 = 0x2D
    HK_QUADREAL_0101 = 0x2E
    HK_QUADREAL_1100 = 0x2F
    HK_QUADREAL_0011 = 0x30
    HK_QUADREAL_1248 = 0x31
    HK_QUADREAL_8421 = 0x32
    HK_QUADREAL_PACK_HALF = 0x33
    HK_QUADREAL_PACK16_UNIT_VEC = 0x34
    HK_QUADREAL_UNPACK16_UNIT_VEC = 0x35
    HK_QUADREAL_PI = 0x36
    HK_QUADREAL_PI_HALF = 0x37
    HK_QUADREAL_PI_QUARTER = 0x38
    HK_QUADREAL_FOUR_PI_THIRD = 0x39
    HK_QUADREAL_TWO_PI = 0x3A
    HK_QUADREAL_FOUR_PI = 0x3B
    HK_QUADREAL_DEG2RAD = 0x3C
    HK_QUADREAL_RAD2DEG = 0x3D
    HK_QUADREAL_INTEGRATE_COEFF_A = 0x3E
    HK_QUADREAL_INTEGRATE_COEFF_B = 0x3F
    HK_QUADREAL_INTEGRATE_COEFF_C = 0x40
    HK_QUADREAL_END = 0x41

class _HARDWARE_COUNTER_TYPE(IntEnum):
    PMCCounter = 0x0
    MaxHardwareCounterType = 0x1

class _FILE_ID_TYPE(IntEnum):
    FileIdType = 0x0
    ObjectIdType = 0x1
    ExtendedFileIdType = 0x2
    MaximumFileIdType = 0x3

class tagPOINTER_DEVICE_CURSOR_TYPE(IntEnum):
    POINTER_DEVICE_CURSOR_TYPE_UNKNOWN = 0x0
    POINTER_DEVICE_CURSOR_TYPE_TIP = 0x1
    POINTER_DEVICE_CURSOR_TYPE_ERASER = 0x2
    POINTER_DEVICE_CURSOR_TYPE_MAX = 0xFFFFFFFF

class hkBitFieldValue(ctypes.Structure):
    class Initialized(IntEnum):
        ZERO = 0x0
        ONE = 0x1

    class Uninitialized(IntEnum):
        UNINITIALIZED = 0x0


class _AUDIT_EVENT_TYPE(IntEnum):
    AuditEventObjectAccess = 0x0
    AuditEventDirectoryServiceAccess = 0x1

class hkCharSet(ctypes.Structure):
    class Invert(IntEnum):
        pass

    _fields_ = [
        ("m_masks", ctypes.c_uint32 * 0x8),  # unsigned int[8]
    ]
    m_masks: list[ctypes.c_uint32]

class LATENCY_TIME(IntEnum):
    LT_DONT_CARE = 0x0
    LT_LOWEST_LATENCY = 0x1

class ACTCTX_COMPATIBILITY_ELEMENT_TYPE(IntEnum):
    ACTCTX_COMPATIBILITY_ELEMENT_TYPE_UNKNOWN = 0x0
    ACTCTX_COMPATIBILITY_ELEMENT_TYPE_OS = 0x1
    ACTCTX_COMPATIBILITY_ELEMENT_TYPE_MITIGATION = 0x2
    ACTCTX_COMPATIBILITY_ELEMENT_TYPE_MAXVERSIONTESTED = 0x3

class tagINPUT_MESSAGE_ORIGIN_ID(IntEnum):
    IMO_UNAVAILABLE = 0x0
    IMO_HARDWARE = 0x1
    IMO_INJECTED = 0x2
    IMO_SYSTEM = 0x4

class eActivationType(IntEnum):
    EActivationType_GroupNode = 0x0
    EActivationType_Locator = 0x1
    EActivationType_NumTypes = 0x2

class hkJobType(IntEnum):
    HK_JOB_TYPE_DYNAMICS = 0x0
    HK_JOB_TYPE_COLLIDE = 0x1
    HK_JOB_TYPE_COLLISION_QUERY = 0x2
    HK_JOB_TYPE_RAYCAST_QUERY = 0x3
    HK_JOB_TYPE_ANIMATION_SAMPLE_AND_COMBINE = 0x4
    HK_JOB_TYPE_ANIMATION_SAMPLE_AND_BLEND = 0x5
    HK_JOB_TYPE_ANIMATION_MAPPING = 0x6
    HK_JOB_TYPE_CLOTH = 0x7
    HK_JOB_TYPE_UNIT_TEST = 0x8
    HK_JOB_TYPE_CHARACTER_PROXY = 0x9
    HK_JOB_TYPE_VEHICLE = 0xA
    HK_JOB_TYPE_COLLIDE_STATIC_COMPOUND = 0xB
    HK_JOB_TYPE_HAVOK_MAX = 0xC
    HK_JOB_TYPE_MAX = 0xD

class hkAxialRotationTag(IntEnum):
    HK_AXIAL_ROT_IDENTITY = 0x0
    HK_AXIAL_ROT_X90 = 0x1
    HK_AXIAL_ROT_X180 = 0x2
    HK_AXIAL_ROT_X270 = 0x3
    HK_AXIAL_ROT_Y90 = 0x4
    HK_AXIAL_ROT_Y180 = 0x5
    HK_AXIAL_ROT_Y270 = 0x6
    HK_AXIAL_ROT_Z90 = 0x7
    HK_AXIAL_ROT_Z180 = 0x8
    HK_AXIAL_ROT_Z270 = 0x9
    HK_AXIAL_ROT_DIAG1_ROT120 = 0xA
    HK_AXIAL_ROT_DIAG1_ROT240 = 0xB
    HK_AXIAL_ROT_DIAG2_ROT120 = 0xC
    HK_AXIAL_ROT_DIAG2_ROT240 = 0xD
    HK_AXIAL_ROT_DIAG3_ROT120 = 0xE
    HK_AXIAL_ROT_DIAG3_ROT240 = 0xF
    HK_AXIAL_ROT_DIAG4_ROT120 = 0x10
    HK_AXIAL_ROT_DIAG4_ROT240 = 0x11
    HK_AXIAL_ROT_TRANSP1 = 0x12
    HK_AXIAL_ROT_TRANSP2 = 0x13
    HK_AXIAL_ROT_TRANSP3 = 0x14
    HK_AXIAL_ROT_TRANSP4 = 0x15
    HK_AXIAL_ROT_TRANSP5 = 0x16
    HK_AXIAL_ROT_TRANSP6 = 0x17
    HK_NUM_AXIAL_ROTATIONS = 0x18

class _SYSTEM_POWER_STATE(IntEnum):
    PowerSystemUnspecified = 0x0
    PowerSystemWorking = 0x1
    PowerSystemSleeping1 = 0x2
    PowerSystemSleeping2 = 0x3
    PowerSystemSleeping3 = 0x4
    PowerSystemHibernate = 0x5
    PowerSystemShutdown = 0x6
    PowerSystemMaximum = 0x7

class _JOBOBJECTINFOCLASS(IntEnum):
    JobObjectBasicAccountingInformation = 0x1
    JobObjectBasicLimitInformation = 0x2
    JobObjectBasicProcessIdList = 0x3
    JobObjectBasicUIRestrictions = 0x4
    JobObjectSecurityLimitInformation = 0x5
    JobObjectEndOfJobTimeInformation = 0x6
    JobObjectAssociateCompletionPortInformation = 0x7
    JobObjectBasicAndIoAccountingInformation = 0x8
    JobObjectExtendedLimitInformation = 0x9
    JobObjectJobSetInformation = 0xA
    JobObjectGroupInformation = 0xB
    JobObjectNotificationLimitInformation = 0xC
    JobObjectLimitViolationInformation = 0xD
    JobObjectGroupInformationEx = 0xE
    JobObjectCpuRateControlInformation = 0xF
    JobObjectCompletionFilter = 0x10
    JobObjectCompletionCounter = 0x11
    JobObjectReserved1Information = 0x12
    JobObjectReserved2Information = 0x13
    JobObjectReserved3Information = 0x14
    JobObjectReserved4Information = 0x15
    JobObjectReserved5Information = 0x16
    JobObjectReserved6Information = 0x17
    JobObjectReserved7Information = 0x18
    JobObjectReserved8Information = 0x19
    JobObjectReserved9Information = 0x1A
    JobObjectReserved10Information = 0x1B
    JobObjectReserved11Information = 0x1C
    JobObjectReserved12Information = 0x1D
    JobObjectReserved13Information = 0x1E
    JobObjectReserved14Information = 0x1F
    JobObjectNetRateControlInformation = 0x20
    JobObjectNotificationLimitInformation2 = 0x21
    JobObjectLimitViolationInformation2 = 0x22
    JobObjectCreateSilo = 0x23
    JobObjectSiloBasicInformation = 0x24
    JobObjectReserved15Information = 0x25
    JobObjectReserved16Information = 0x26
    JobObjectReserved17Information = 0x27
    JobObjectReserved18Information = 0x28
    JobObjectReserved19Information = 0x29
    JobObjectReserved20Information = 0x2A
    JobObjectReserved21Information = 0x2B
    JobObjectReserved22Information = 0x2C
    JobObjectReserved23Information = 0x2D
    JobObjectReserved24Information = 0x2E
    JobObjectReserved25Information = 0x2F
    MaxJobObjectInfoClass = 0x30

class _RESOURCEMANAGER_INFORMATION_CLASS(IntEnum):
    ResourceManagerBasicInformation = 0x0
    ResourceManagerCompletionInformation = 0x1

class _POWER_PLATFORM_ROLE(IntEnum):
    PlatformRoleUnspecified = 0x0
    PlatformRoleDesktop = 0x1
    PlatformRoleMobile = 0x2
    PlatformRoleWorkstation = 0x3
    PlatformRoleEnterpriseServer = 0x4
    PlatformRoleSOHOServer = 0x5
    PlatformRoleAppliancePC = 0x6
    PlatformRolePerformanceServer = 0x7
    PlatformRoleSlate = 0x8
    PlatformRoleMaximum = 0x9

class _EXCEPTION_DISPOSITION(IntEnum):
    ExceptionContinueExecution = 0x0
    ExceptionContinueSearch = 0x1
    ExceptionNestedException = 0x2
    ExceptionCollidedUnwind = 0x3

class _KTMOBJECT_TYPE(IntEnum):
    KTMOBJECT_TRANSACTION = 0x0
    KTMOBJECT_TRANSACTION_MANAGER = 0x1
    KTMOBJECT_RESOURCE_MANAGER = 0x2
    KTMOBJECT_ENLISTMENT = 0x3
    KTMOBJECT_INVALID = 0x4

class hkFloatParseUtil(ctypes.Structure):
    class Type(IntEnum):
        TYPE_NEG_INF = 0x0
        TYPE_INF = 0x1
        TYPE_NEG_NAN = 0x2
        TYPE_NAN = 0x3
        TYPE_IND = 0x4
        TYPE_FINITE = 0x5
        TYPE_COUNT_OF = 0x6

    def parseFloat(): pass


class hkBool32FalseType(IntEnum):
    hkFalse32 = 0x0

class _TRANSACTION_INFORMATION_CLASS(IntEnum):
    TransactionBasicInformation = 0x0
    TransactionPropertiesInformation = 0x1
    TransactionEnlistmentInformation = 0x2
    TransactionSuperiorEnlistmentInformation = 0x3
    TransactionBindInformation = 0x4
    TransactionDTCPrivateInformation = 0x5

class _CPU_SET_INFORMATION_TYPE(IntEnum):
    CpuSetInformation = 0x0

class _FINDEX_INFO_LEVELS(IntEnum):
    FindExInfoStandard = 0x0
    FindExInfoBasic = 0x1
    FindExInfoMaxInfoLevel = 0x2

class _MM_EXP_ADJ_ENUM(IntEnum):
    _MM_EXPADJ_NONE = 0x0
    _MM_EXPADJ_4 = 0x1
    _MM_EXPADJ_5 = 0x2
    _MM_EXPADJ_8 = 0x3
    _MM_EXPADJ_16 = 0x4
    _MM_EXPADJ_24 = 0x5
    _MM_EXPADJ_31 = 0x6
    _MM_EXPADJ_32 = 0x7

class tagINPUT_MESSAGE_DEVICE_TYPE(IntEnum):
    IMDT_UNAVAILABLE = 0x0
    IMDT_KEYBOARD = 0x1
    IMDT_MOUSE = 0x2
    IMDT_TOUCH = 0x4
    IMDT_PEN = 0x8
    IMDT_TOUCHPAD = 0x10

class _FILE_INFO_BY_HANDLE_CLASS(IntEnum):
    FileBasicInfo = 0x0
    FileStandardInfo = 0x1
    FileNameInfo = 0x2
    FileRenameInfo = 0x3
    FileDispositionInfo = 0x4
    FileAllocationInfo = 0x5
    FileEndOfFileInfo = 0x6
    FileStreamInfo = 0x7
    FileCompressionInfo = 0x8
    FileAttributeTagInfo = 0x9
    FileIdBothDirectoryInfo = 0xA
    FileIdBothDirectoryRestartInfo = 0xB
    FileIoPriorityHintInfo = 0xC
    FileRemoteProtocolInfo = 0xD
    FileFullDirectoryInfo = 0xE
    FileFullDirectoryRestartInfo = 0xF
    FileStorageInfo = 0x10
    FileAlignmentInfo = 0x11
    FileIdInfo = 0x12
    FileIdExtdDirectoryInfo = 0x13
    FileIdExtdDirectoryRestartInfo = 0x14
    FileDispositionInfoEx = 0x15
    FileRenameInfoEx = 0x16
    FileCaseSensitiveInfo = 0x17
    FileNormalizedNameInfo = 0x18
    MaximumFileInfoByHandleClass = 0x19

class _TOKEN_ELEVATION_TYPE(IntEnum):
    TokenElevationTypeDefault = 0x1
    TokenElevationTypeFull = 0x2
    TokenElevationTypeLimited = 0x3

class hkQuaternionAxisOrder(IntEnum):
    XYZ = 0x0
    XZY = 0x1
    YXZ = 0x2
    YZX = 0x3
    ZXY = 0x4
    ZYX = 0x5

class _RTL_UMS_SCHEDULER_REASON(IntEnum):
    UmsSchedulerStartup = 0x0
    UmsSchedulerThreadBlocked = 0x1
    UmsSchedulerThreadYield = 0x2

class _HIBERFILE_BUCKET_SIZE(IntEnum):
    HiberFileBucket1GB = 0x0
    HiberFileBucket2GB = 0x1
    HiberFileBucket4GB = 0x2
    HiberFileBucket8GB = 0x3
    HiberFileBucket16GB = 0x4
    HiberFileBucket32GB = 0x5
    HiberFileBucketUnlimited = 0x6
    HiberFileBucketMax = 0x7

class eDownloadType(IntEnum):
    EDownloadType_Unknown = 0x0
    EDownloadType_Discovery = 0x1
    EDownloadType_Base = 0x2
    EDownloadType_Message = 0x3
    EDownloadType_NumTypes = 0x4

class hkVectorPermutation(ctypes.Structure):
    class Permutation(IntEnum):
        XXXX = 0x0
        XXXY = 0x1
        XXXZ = 0x2
        XXXW = 0x3
        XXYX = 0x10
        XXYY = 0x11
        XXYZ = 0x12
        XXYW = 0x13
        XXZX = 0x20
        XXZY = 0x21
        XXZZ = 0x22
        XXZW = 0x23
        XXWX = 0x30
        XXWY = 0x31
        XXWZ = 0x32
        XXWW = 0x33
        XYXX = 0x100
        XYXY = 0x101
        XYXZ = 0x102
        XYXW = 0x103
        XYYX = 0x110
        XYYY = 0x111
        XYYZ = 0x112
        XYYW = 0x113
        XYZX = 0x120
        XYZY = 0x121
        XYZZ = 0x122
        XYZW = 0x123
        XYWX = 0x130
        XYWY = 0x131
        XYWZ = 0x132
        XYWW = 0x133
        XZXX = 0x200
        XZXY = 0x201
        XZXZ = 0x202
        XZXW = 0x203
        XZYX = 0x210
        XZYY = 0x211
        XZYZ = 0x212
        XZYW = 0x213
        XZZX = 0x220
        XZZY = 0x221
        XZZZ = 0x222
        XZZW = 0x223
        XZWX = 0x230
        XZWY = 0x231
        XZWZ = 0x232
        XZWW = 0x233
        XWXX = 0x300
        XWXY = 0x301
        XWXZ = 0x302
        XWXW = 0x303
        XWYX = 0x310
        XWYY = 0x311
        XWYZ = 0x312
        XWYW = 0x313
        XWZX = 0x320
        XWZY = 0x321
        XWZZ = 0x322
        XWZW = 0x323
        XWWX = 0x330
        XWWY = 0x331
        XWWZ = 0x332
        XWWW = 0x333
        YXXX = 0x1000
        YXXY = 0x1001
        YXXZ = 0x1002
        YXXW = 0x1003
        YXYX = 0x1010
        YXYY = 0x1011
        YXYZ = 0x1012
        YXYW = 0x1013
        YXZX = 0x1020
        YXZY = 0x1021
        YXZZ = 0x1022
        YXZW = 0x1023
        YXWX = 0x1030
        YXWY = 0x1031
        YXWZ = 0x1032
        YXWW = 0x1033
        YYXX = 0x1100
        YYXY = 0x1101
        YYXZ = 0x1102
        YYXW = 0x1103
        YYYX = 0x1110
        YYYY = 0x1111
        YYYZ = 0x1112
        YYYW = 0x1113
        YYZX = 0x1120
        YYZY = 0x1121
        YYZZ = 0x1122
        YYZW = 0x1123
        YYWX = 0x1130
        YYWY = 0x1131
        YYWZ = 0x1132
        YYWW = 0x1133
        YZXX = 0x1200
        YZXY = 0x1201
        YZXZ = 0x1202
        YZXW = 0x1203
        YZYX = 0x1210
        YZYY = 0x1211
        YZYZ = 0x1212
        YZYW = 0x1213
        YZZX = 0x1220
        YZZY = 0x1221
        YZZZ = 0x1222
        YZZW = 0x1223
        YZWX = 0x1230
        YZWY = 0x1231
        YZWZ = 0x1232
        YZWW = 0x1233
        YWXX = 0x1300
        YWXY = 0x1301
        YWXZ = 0x1302
        YWXW = 0x1303
        YWYX = 0x1310
        YWYY = 0x1311
        YWYZ = 0x1312
        YWYW = 0x1313
        YWZX = 0x1320
        YWZY = 0x1321
        YWZZ = 0x1322
        YWZW = 0x1323
        YWWX = 0x1330
        YWWY = 0x1331
        YWWZ = 0x1332
        YWWW = 0x1333
        ZXXX = 0x2000
        ZXXY = 0x2001
        ZXXZ = 0x2002
        ZXXW = 0x2003
        ZXYX = 0x2010
        ZXYY = 0x2011
        ZXYZ = 0x2012
        ZXYW = 0x2013
        ZXZX = 0x2020
        ZXZY = 0x2021
        ZXZZ = 0x2022
        ZXZW = 0x2023
        ZXWX = 0x2030
        ZXWY = 0x2031
        ZXWZ = 0x2032
        ZXWW = 0x2033
        ZYXX = 0x2100
        ZYXY = 0x2101
        ZYXZ = 0x2102
        ZYXW = 0x2103
        ZYYX = 0x2110
        ZYYY = 0x2111
        ZYYZ = 0x2112
        ZYYW = 0x2113
        ZYZX = 0x2120
        ZYZY = 0x2121
        ZYZZ = 0x2122
        ZYZW = 0x2123
        ZYWX = 0x2130
        ZYWY = 0x2131
        ZYWZ = 0x2132
        ZYWW = 0x2133
        ZZXX = 0x2200
        ZZXY = 0x2201
        ZZXZ = 0x2202
        ZZXW = 0x2203
        ZZYX = 0x2210
        ZZYY = 0x2211
        ZZYZ = 0x2212
        ZZYW = 0x2213
        ZZZX = 0x2220
        ZZZY = 0x2221
        ZZZZ = 0x2222
        ZZZW = 0x2223
        ZZWX = 0x2230
        ZZWY = 0x2231
        ZZWZ = 0x2232
        ZZWW = 0x2233
        ZWXX = 0x2300
        ZWXY = 0x2301
        ZWXZ = 0x2302
        ZWXW = 0x2303
        ZWYX = 0x2310
        ZWYY = 0x2311
        ZWYZ = 0x2312
        ZWYW = 0x2313
        ZWZX = 0x2320
        ZWZY = 0x2321
        ZWZZ = 0x2322
        ZWZW = 0x2323
        ZWWX = 0x2330
        ZWWY = 0x2331
        ZWWZ = 0x2332
        ZWWW = 0x2333
        WXXX = 0x3000
        WXXY = 0x3001
        WXXZ = 0x3002
        WXXW = 0x3003
        WXYX = 0x3010
        WXYY = 0x3011
        WXYZ = 0x3012
        WXYW = 0x3013
        WXZX = 0x3020
        WXZY = 0x3021
        WXZZ = 0x3022
        WXZW = 0x3023
        WXWX = 0x3030
        WXWY = 0x3031
        WXWZ = 0x3032
        WXWW = 0x3033
        WYXX = 0x3100
        WYXY = 0x3101
        WYXZ = 0x3102
        WYXW = 0x3103
        WYYX = 0x3110
        WYYY = 0x3111
        WYYZ = 0x3112
        WYYW = 0x3113
        WYZX = 0x3120
        WYZY = 0x3121
        WYZZ = 0x3122
        WYZW = 0x3123
        WYWX = 0x3130
        WYWY = 0x3131
        WYWZ = 0x3132
        WYWW = 0x3133
        WZXX = 0x3200
        WZXY = 0x3201
        WZXZ = 0x3202
        WZXW = 0x3203
        WZYX = 0x3210
        WZYY = 0x3211
        WZYZ = 0x3212
        WZYW = 0x3213
        WZZX = 0x3220
        WZZY = 0x3221
        WZZZ = 0x3222
        WZZW = 0x3223
        WZWX = 0x3230
        WZWY = 0x3231
        WZWZ = 0x3232
        WZWW = 0x3233
        WWXX = 0x3300
        WWXY = 0x3301
        WWXZ = 0x3302
        WWXW = 0x3303
        WWYX = 0x3310
        WWYY = 0x3311
        WWYZ = 0x3312
        WWYW = 0x3313
        WWZX = 0x3320
        WWZY = 0x3321
        WWZZ = 0x3322
        WWZW = 0x3323
        WWWX = 0x3330
        WWWY = 0x3331
        WWWZ = 0x3332
        WWWW = 0x3333


class _SharedVirtualDiskHandleState(IntEnum):
    SharedVirtualDiskHandleStateNone = 0x0
    SharedVirtualDiskHandleStateFileShared = 0x1
    SharedVirtualDiskHandleStateHandleShared = 0x3

class hkThreadLocalBlockStreamAllocator(ctypes.Structure):
    def hkThreadLocalBlockStreamAllocator(): pass

    def tilde_hkThreadLocalBlockStreamAllocator(): pass

    def blockAlloc(): pass

    def blockFree(): pass

    def blockFreeBatch(): pass

    def clear(): pass

    _fields_ = [
        ("m_blockStreamAllocator", ctypes.c_ulonglong),
        ("m_freeBlocks", ctypes.c_char * 0x50),  # hkInplaceArray<hkBlockStream::Block *,8,hkBuiltinContainerAllocator<0,0> >
        ("m_numFreeBlocks", ctypes.c_int32),
        ("m_numBlocksToCache", ctypes.c_int32),
    ]
    m_blockStreamAllocator: int
    m_freeBlocks: bytes
    m_numFreeBlocks: int
    m_numBlocksToCache: int

class hkEndian(ctypes.Structure):
    pass

class _FINDEX_SEARCH_OPS(IntEnum):
    FindExSearchNameMatch = 0x0
    FindExSearchLimitToDirectories = 0x1
    FindExSearchLimitToDevices = 0x2
    FindExSearchMaxSearchOp = 0x3

class hkIntVectorConstant(IntEnum):
    HK_QUADINT_BEGIN = 0x0
    HK_QUADINT_0 = 0x1
    HK_QUADINT_1 = 0x2
    HK_QUADINT_2 = 0x3
    HK_QUADINT_4 = 0x4
    HK_QUADINT_0123 = 0x5
    HK_QUADINT_0123_INT24W = 0x6
    HK_QUADINT_3 = 0x7
    HK_QUADINT_FFFF = 0x8
    HK_QUADINT_PERM_XXXX = 0x9
    HK_QUADINT_PERM_YYYY = 0xA
    HK_QUADINT_PERM_ZZZZ = 0xB
    HK_QUADINT_PERM_WWWW = 0xC
    HK_QUADINT_END = 0xD

class hkMathRoundingMode(IntEnum):
    HK_ROUND_TRUNCATE = 0x0
    HK_ROUND_NEAREST = 0x1

class hkMathValueType(IntEnum):
    HK_VALUE_SIGNED = 0x0
    HK_VALUE_UNSIGNED = 0x1

class hkMathSortDir(IntEnum):
    HK_SORT_ASCENDING = 0x0
    HK_SORT_DESCENDING = 0x1

class hkDeterminantCheck(IntEnum):
    HK_DETERMINANT_CHECK_IGNORE = 0x0
    HK_DETERMINANT_CHECK_ZERO = 0x1
    HK_DETERMINANT_CHECK_EPS = 0x2

class hkInversionFailure(IntEnum):
    HK_INVERSION_FAILURE_IGNORE = 0x0
    HK_INVERSION_FAILURE_SET_ZERO = 0x1
    HK_INVERSION_FAILURE_SET_ZERO_KEEP_NANS = 0x2
    HK_INVERSION_FAILURE_SET_IDENTITY = 0x3

class hkMatrixSpecialization(IntEnum):
    HK_MATRIX_ANY = 0x0
    HK_MATRIX_SYMMETRIC = 0x1
    HK_MATRIX_ORTHONORMAL = 0x2

class _MM_BROADCAST32_ENUM(IntEnum):
    _MM_BROADCAST32_NONE = 0x0
    _MM_BROADCAST_1X16 = 0x1
    _MM_BROADCAST_4X16 = 0x2

class _SERVERSILO_STATE(IntEnum):
    SERVERSILO_INITING = 0x0
    SERVERSILO_STARTED = 0x1
    SERVERSILO_SHUTTING_DOWN = 0x2
    SERVERSILO_TERMINATING = 0x3
    SERVERSILO_TERMINATED = 0x4

class _MM_BROADCAST64_ENUM(IntEnum):
    _MM_BROADCAST64_NONE = 0x0
    _MM_BROADCAST_1X8 = 0x1
    _MM_BROADCAST_4X8 = 0x2

class _MM_ROUND_MODE_ENUM(IntEnum):
    _MM_ROUND_MODE_NEAREST = 0x0
    _MM_ROUND_MODE_DOWN = 0x1
    _MM_ROUND_MODE_UP = 0x2
    _MM_ROUND_MODE_TOWARD_ZERO = 0x3
    _MM_ROUND_MODE_DEFAULT = 0x4
    _MM_ROUND_MODE_NO_EXC = 0x8

class _MM_INDEX_SCALE_ENUM(IntEnum):
    _MM_SCALE_1 = 0x1

class _MM_PERM_ENUM(IntEnum):
    _MM_PERM_AAAA = 0x0
    _MM_PERM_AAAB = 0x1
    _MM_PERM_AAAC = 0x2
    _MM_PERM_AAAD = 0x3
    _MM_PERM_AABA = 0x4
    _MM_PERM_AABB = 0x5
    _MM_PERM_AABC = 0x6
    _MM_PERM_AABD = 0x7
    _MM_PERM_AACA = 0x8
    _MM_PERM_AACB = 0x9
    _MM_PERM_AACC = 0xA
    _MM_PERM_AACD = 0xB
    _MM_PERM_AADA = 0xC
    _MM_PERM_AADB = 0xD
    _MM_PERM_AADC = 0xE
    _MM_PERM_AADD = 0xF
    _MM_PERM_ABAA = 0x10
    _MM_PERM_ABAB = 0x11
    _MM_PERM_ABAC = 0x12
    _MM_PERM_ABAD = 0x13
    _MM_PERM_ABBA = 0x14
    _MM_PERM_ABBB = 0x15
    _MM_PERM_ABBC = 0x16
    _MM_PERM_ABBD = 0x17
    _MM_PERM_ABCA = 0x18
    _MM_PERM_ABCB = 0x19
    _MM_PERM_ABCC = 0x1A
    _MM_PERM_ABCD = 0x1B
    _MM_PERM_ABDA = 0x1C
    _MM_PERM_ABDB = 0x1D
    _MM_PERM_ABDC = 0x1E
    _MM_PERM_ABDD = 0x1F
    _MM_PERM_ACAA = 0x20
    _MM_PERM_ACAB = 0x21
    _MM_PERM_ACAC = 0x22
    _MM_PERM_ACAD = 0x23
    _MM_PERM_ACBA = 0x24
    _MM_PERM_ACBB = 0x25
    _MM_PERM_ACBC = 0x26
    _MM_PERM_ACBD = 0x27
    _MM_PERM_ACCA = 0x28
    _MM_PERM_ACCB = 0x29
    _MM_PERM_ACCC = 0x2A
    _MM_PERM_ACCD = 0x2B
    _MM_PERM_ACDA = 0x2C
    _MM_PERM_ACDB = 0x2D
    _MM_PERM_ACDC = 0x2E
    _MM_PERM_ACDD = 0x2F
    _MM_PERM_ADAA = 0x30
    _MM_PERM_ADAB = 0x31
    _MM_PERM_ADAC = 0x32
    _MM_PERM_ADAD = 0x33
    _MM_PERM_ADBA = 0x34
    _MM_PERM_ADBB = 0x35
    _MM_PERM_ADBC = 0x36
    _MM_PERM_ADBD = 0x37
    _MM_PERM_ADCA = 0x38
    _MM_PERM_ADCB = 0x39
    _MM_PERM_ADCC = 0x3A
    _MM_PERM_ADCD = 0x3B
    _MM_PERM_ADDA = 0x3C
    _MM_PERM_ADDB = 0x3D
    _MM_PERM_ADDC = 0x3E
    _MM_PERM_ADDD = 0x3F
    _MM_PERM_BAAA = 0x40
    _MM_PERM_BAAB = 0x41
    _MM_PERM_BAAC = 0x42
    _MM_PERM_BAAD = 0x43
    _MM_PERM_BABA = 0x44
    _MM_PERM_BABB = 0x45
    _MM_PERM_BABC = 0x46
    _MM_PERM_BABD = 0x47
    _MM_PERM_BACA = 0x48
    _MM_PERM_BACB = 0x49
    _MM_PERM_BACC = 0x4A
    _MM_PERM_BACD = 0x4B
    _MM_PERM_BADA = 0x4C
    _MM_PERM_BADB = 0x4D
    _MM_PERM_BADC = 0x4E
    _MM_PERM_BADD = 0x4F
    _MM_PERM_BBAA = 0x50
    _MM_PERM_BBAB = 0x51
    _MM_PERM_BBAC = 0x52
    _MM_PERM_BBAD = 0x53
    _MM_PERM_BBBA = 0x54
    _MM_PERM_BBBB = 0x55
    _MM_PERM_BBBC = 0x56
    _MM_PERM_BBBD = 0x57
    _MM_PERM_BBCA = 0x58
    _MM_PERM_BBCB = 0x59
    _MM_PERM_BBCC = 0x5A
    _MM_PERM_BBCD = 0x5B
    _MM_PERM_BBDA = 0x5C
    _MM_PERM_BBDB = 0x5D
    _MM_PERM_BBDC = 0x5E
    _MM_PERM_BBDD = 0x5F
    _MM_PERM_BCAA = 0x60
    _MM_PERM_BCAB = 0x61
    _MM_PERM_BCAC = 0x62
    _MM_PERM_BCAD = 0x63
    _MM_PERM_BCBA = 0x64
    _MM_PERM_BCBB = 0x65
    _MM_PERM_BCBC = 0x66
    _MM_PERM_BCBD = 0x67
    _MM_PERM_BCCA = 0x68
    _MM_PERM_BCCB = 0x69
    _MM_PERM_BCCC = 0x6A
    _MM_PERM_BCCD = 0x6B
    _MM_PERM_BCDA = 0x6C
    _MM_PERM_BCDB = 0x6D
    _MM_PERM_BCDC = 0x6E
    _MM_PERM_BCDD = 0x6F
    _MM_PERM_BDAA = 0x70
    _MM_PERM_BDAB = 0x71
    _MM_PERM_BDAC = 0x72
    _MM_PERM_BDAD = 0x73
    _MM_PERM_BDBA = 0x74
    _MM_PERM_BDBB = 0x75
    _MM_PERM_BDBC = 0x76
    _MM_PERM_BDBD = 0x77
    _MM_PERM_BDCA = 0x78
    _MM_PERM_BDCB = 0x79
    _MM_PERM_BDCC = 0x7A
    _MM_PERM_BDCD = 0x7B
    _MM_PERM_BDDA = 0x7C
    _MM_PERM_BDDB = 0x7D
    _MM_PERM_BDDC = 0x7E
    _MM_PERM_BDDD = 0x7F
    _MM_PERM_CAAA = 0x80
    _MM_PERM_CAAB = 0x81
    _MM_PERM_CAAC = 0x82
    _MM_PERM_CAAD = 0x83
    _MM_PERM_CABA = 0x84
    _MM_PERM_CABB = 0x85
    _MM_PERM_CABC = 0x86
    _MM_PERM_CABD = 0x87
    _MM_PERM_CACA = 0x88
    _MM_PERM_CACB = 0x89
    _MM_PERM_CACC = 0x8A
    _MM_PERM_CACD = 0x8B
    _MM_PERM_CADA = 0x8C
    _MM_PERM_CADB = 0x8D
    _MM_PERM_CADC = 0x8E
    _MM_PERM_CADD = 0x8F
    _MM_PERM_CBAA = 0x90
    _MM_PERM_CBAB = 0x91
    _MM_PERM_CBAC = 0x92
    _MM_PERM_CBAD = 0x93
    _MM_PERM_CBBA = 0x94
    _MM_PERM_CBBB = 0x95
    _MM_PERM_CBBC = 0x96
    _MM_PERM_CBBD = 0x97
    _MM_PERM_CBCA = 0x98
    _MM_PERM_CBCB = 0x99
    _MM_PERM_CBCC = 0x9A
    _MM_PERM_CBCD = 0x9B
    _MM_PERM_CBDA = 0x9C
    _MM_PERM_CBDB = 0x9D
    _MM_PERM_CBDC = 0x9E
    _MM_PERM_CBDD = 0x9F
    _MM_PERM_CCAA = 0xA0
    _MM_PERM_CCAB = 0xA1
    _MM_PERM_CCAC = 0xA2
    _MM_PERM_CCAD = 0xA3
    _MM_PERM_CCBA = 0xA4
    _MM_PERM_CCBB = 0xA5
    _MM_PERM_CCBC = 0xA6
    _MM_PERM_CCBD = 0xA7
    _MM_PERM_CCCA = 0xA8
    _MM_PERM_CCCB = 0xA9
    _MM_PERM_CCCC = 0xAA
    _MM_PERM_CCCD = 0xAB
    _MM_PERM_CCDA = 0xAC
    _MM_PERM_CCDB = 0xAD
    _MM_PERM_CCDC = 0xAE
    _MM_PERM_CCDD = 0xAF
    _MM_PERM_CDAA = 0xB0
    _MM_PERM_CDAB = 0xB1
    _MM_PERM_CDAC = 0xB2
    _MM_PERM_CDAD = 0xB3
    _MM_PERM_CDBA = 0xB4
    _MM_PERM_CDBB = 0xB5
    _MM_PERM_CDBC = 0xB6
    _MM_PERM_CDBD = 0xB7
    _MM_PERM_CDCA = 0xB8
    _MM_PERM_CDCB = 0xB9
    _MM_PERM_CDCC = 0xBA
    _MM_PERM_CDCD = 0xBB
    _MM_PERM_CDDA = 0xBC
    _MM_PERM_CDDB = 0xBD
    _MM_PERM_CDDC = 0xBE
    _MM_PERM_CDDD = 0xBF
    _MM_PERM_DAAA = 0xC0
    _MM_PERM_DAAB = 0xC1
    _MM_PERM_DAAC = 0xC2
    _MM_PERM_DAAD = 0xC3
    _MM_PERM_DABA = 0xC4
    _MM_PERM_DABB = 0xC5
    _MM_PERM_DABC = 0xC6
    _MM_PERM_DABD = 0xC7
    _MM_PERM_DACA = 0xC8
    _MM_PERM_DACB = 0xC9
    _MM_PERM_DACC = 0xCA
    _MM_PERM_DACD = 0xCB
    _MM_PERM_DADA = 0xCC
    _MM_PERM_DADB = 0xCD
    _MM_PERM_DADC = 0xCE
    _MM_PERM_DADD = 0xCF
    _MM_PERM_DBAA = 0xD0
    _MM_PERM_DBAB = 0xD1
    _MM_PERM_DBAC = 0xD2
    _MM_PERM_DBAD = 0xD3
    _MM_PERM_DBBA = 0xD4
    _MM_PERM_DBBB = 0xD5
    _MM_PERM_DBBC = 0xD6
    _MM_PERM_DBBD = 0xD7
    _MM_PERM_DBCA = 0xD8
    _MM_PERM_DBCB = 0xD9
    _MM_PERM_DBCC = 0xDA
    _MM_PERM_DBCD = 0xDB
    _MM_PERM_DBDA = 0xDC
    _MM_PERM_DBDB = 0xDD
    _MM_PERM_DBDC = 0xDE
    _MM_PERM_DBDD = 0xDF
    _MM_PERM_DCAA = 0xE0
    _MM_PERM_DCAB = 0xE1
    _MM_PERM_DCAC = 0xE2
    _MM_PERM_DCAD = 0xE3
    _MM_PERM_DCBA = 0xE4
    _MM_PERM_DCBB = 0xE5
    _MM_PERM_DCBC = 0xE6
    _MM_PERM_DCBD = 0xE7
    _MM_PERM_DCCA = 0xE8
    _MM_PERM_DCCB = 0xE9
    _MM_PERM_DCCC = 0xEA
    _MM_PERM_DCCD = 0xEB
    _MM_PERM_DCDA = 0xEC
    _MM_PERM_DCDB = 0xED
    _MM_PERM_DCDC = 0xEE
    _MM_PERM_DCDD = 0xEF
    _MM_PERM_DDAA = 0xF0
    _MM_PERM_DDAB = 0xF1
    _MM_PERM_DDAC = 0xF2
    _MM_PERM_DDAD = 0xF3
    _MM_PERM_DDBA = 0xF4
    _MM_PERM_DDBB = 0xF5
    _MM_PERM_DDBC = 0xF6
    _MM_PERM_DDBD = 0xF7
    _MM_PERM_DDCA = 0xF8
    _MM_PERM_DDCB = 0xF9
    _MM_PERM_DDCC = 0xFA
    _MM_PERM_DDCD = 0xFB
    _MM_PERM_DDDA = 0xFC
    _MM_PERM_DDDB = 0xFD
    _MM_PERM_DDDC = 0xFE
    _MM_PERM_DDDD = 0xFF

class hkMathNonDeterministic(ctypes.Structure):
    class AccuracyMode(IntEnum):
        ACC_FULL = 0x0
        ACC_10_BIT = 0x1
        ACC_5_BIT = 0x2
        ACC_12_BIT = 0x3
        ACC_23_BIT = 0x4

    class NegSqrtMode(IntEnum):
        SQRT_IGNORE = 0x0
        SQRT_SET_ZERO = 0x1


class _MM_FIXUPRESULT_ENUM(IntEnum):
    _MM_FIXUP_NO_CHANGE = 0x0
    _MM_FIXUP_NEG_INF = 0x1
    _MM_FIXUP_NEG_ZERO = 0x2
    _MM_FIXUP_POS_ZERO = 0x3
    _MM_FIXUP_POS_INF = 0x4
    _MM_FIXUP_NAN = 0x5
    _MM_FIXUP_MAX_FLOAT = 0x6
    _MM_FIXUP_MIN_FLOAT = 0x7

class _MM_MANTISSA_NORM_ENUM(IntEnum):
    _MM_MANT_NORM_1_2 = 0x0
    _MM_MANT_NORM_p5_2 = 0x1
    _MM_MANT_NORM_p5_1 = 0x2
    _MM_MANT_NORM_p75_1p5 = 0x3

class _MM_MANTISSA_SIGN_ENUM(IntEnum):
    _MM_MANT_SIGN_src = 0x0
    _MM_MANT_SIGN_zero = 0x1
    _MM_MANT_SIGN_nan = 0x2

class _MM_CMPINT_ENUM(IntEnum):
    _MM_CMPINT_EQ = 0x0
    _MM_CMPINT_LT = 0x1
    _MM_CMPINT_LE = 0x2
    _MM_CMPINT_UNUSED = 0x3
    _MM_CMPINT_NE = 0x4
    _MM_CMPINT_NLT = 0x5
    _MM_CMPINT_NLE = 0x6

class POWER_MONITOR_REQUEST_REASON(IntEnum):
    MonitorRequestReasonUnknown = 0x0
    MonitorRequestReasonPowerButton = 0x1
    MonitorRequestReasonRemoteConnection = 0x2
    MonitorRequestReasonScMonitorpower = 0x3
    MonitorRequestReasonUserInput = 0x4
    MonitorRequestReasonAcDcDisplayBurst = 0x5
    MonitorRequestReasonUserDisplayBurst = 0x6
    MonitorRequestReasonPoSetSystemState = 0x7
    MonitorRequestReasonSetThreadExecutionState = 0x8
    MonitorRequestReasonFullWake = 0x9
    MonitorRequestReasonSessionUnlock = 0xA
    MonitorRequestReasonScreenOffRequest = 0xB
    MonitorRequestReasonIdleTimeout = 0xC
    MonitorRequestReasonPolicyChange = 0xD
    MonitorRequestReasonSleepButton = 0xE
    MonitorRequestReasonLid = 0xF
    MonitorRequestReasonBatteryCountChange = 0x10
    MonitorRequestReasonGracePeriod = 0x11
    MonitorRequestReasonPnP = 0x12
    MonitorRequestReasonDP = 0x13
    MonitorRequestReasonSxTransition = 0x14
    MonitorRequestReasonSystemIdle = 0x15
    MonitorRequestReasonNearProximity = 0x16
    MonitorRequestReasonThermalStandby = 0x17
    MonitorRequestReasonResumePdc = 0x18
    MonitorRequestReasonResumeS4 = 0x19
    MonitorRequestReasonTerminal = 0x1A
    MonitorRequestReasonPdcSignal = 0x1B
    MonitorRequestReasonAcDcDisplayBurstSuppressed = 0x1C
    MonitorRequestReasonSystemStateEntered = 0x1D
    MonitorRequestReasonWinrt = 0x1E
    MonitorRequestReasonUserInputKeyboard = 0x1F
    MonitorRequestReasonUserInputMouse = 0x20
    MonitorRequestReasonUserInputTouch = 0x21
    MonitorRequestReasonUserInputPen = 0x22
    MonitorRequestReasonUserInputAccelerometer = 0x23
    MonitorRequestReasonUserInputHid = 0x24
    MonitorRequestReasonUserInputPoUserPresent = 0x25
    MonitorRequestReasonUserInputSessionSwitch = 0x26
    MonitorRequestReasonUserInputInitialization = 0x27
    MonitorRequestReasonPdcSignalWindowsMobilePwrNotif = 0x28
    MonitorRequestReasonPdcSignalWindowsMobileShell = 0x29
    MonitorRequestReasonPdcSignalHeyCortana = 0x2A
    MonitorRequestReasonPdcSignalHolographicShell = 0x2B
    MonitorRequestReasonPdcSignalFingerprint = 0x2C
    MonitorRequestReasonDirectedDrips = 0x2D
    MonitorRequestReasonDim = 0x2E
    MonitorRequestReasonBuiltinPanel = 0x2F
    MonitorRequestReasonDisplayRequiredUnDim = 0x30
    MonitorRequestReasonBatteryCountChangeSuppressed = 0x31
    MonitorRequestReasonResumeModernStandby = 0x32
    MonitorRequestReasonMax = 0x33

class _POWER_REQUEST_TYPE(IntEnum):
    PowerRequestDisplayRequired = 0x0
    PowerRequestSystemRequired = 0x1
    PowerRequestAwayModeRequired = 0x2
    PowerRequestExecutionRequired = 0x3

class _IMAGE_POLICY_ENTRY_TYPE(IntEnum):
    ImagePolicyEntryTypeNone = 0x0
    ImagePolicyEntryTypeBool = 0x1
    ImagePolicyEntryTypeInt8 = 0x2
    ImagePolicyEntryTypeUInt8 = 0x3
    ImagePolicyEntryTypeInt16 = 0x4
    ImagePolicyEntryTypeUInt16 = 0x5
    ImagePolicyEntryTypeInt32 = 0x6
    ImagePolicyEntryTypeUInt32 = 0x7
    ImagePolicyEntryTypeInt64 = 0x8
    ImagePolicyEntryTypeUInt64 = 0x9
    ImagePolicyEntryTypeAnsiString = 0xA
    ImagePolicyEntryTypeUnicodeString = 0xB
    ImagePolicyEntryTypeOverride = 0xC
    ImagePolicyEntryTypeMaximum = 0xD

class _IMAGE_POLICY_ID(IntEnum):
    ImagePolicyIdNone = 0x0
    ImagePolicyIdEtw = 0x1
    ImagePolicyIdDebug = 0x2
    ImagePolicyIdCrashDump = 0x3
    ImagePolicyIdCrashDumpKey = 0x4
    ImagePolicyIdCrashDumpKeyGuid = 0x5
    ImagePolicyIdParentSd = 0x6
    ImagePolicyIdParentSdRev = 0x7
    ImagePolicyIdSvn = 0x8
    ImagePolicyIdDeviceId = 0x9
    ImagePolicyIdCapability = 0xA
    ImagePolicyIdScenarioId = 0xB
    ImagePolicyIdMaximum = 0xC

class _JOBOBJECT_RATE_CONTROL_TOLERANCE(IntEnum):
    ToleranceLow = 0x1
    ToleranceMedium = 0x2
    ToleranceHigh = 0x3

class DISPLAYCONFIG_PIXELFORMAT(IntEnum):
    DISPLAYCONFIG_PIXELFORMAT_8BPP = 0x1
    DISPLAYCONFIG_PIXELFORMAT_16BPP = 0x2
    DISPLAYCONFIG_PIXELFORMAT_24BPP = 0x3
    DISPLAYCONFIG_PIXELFORMAT_32BPP = 0x4
    DISPLAYCONFIG_PIXELFORMAT_NONGDI = 0x5
    DISPLAYCONFIG_PIXELFORMAT_FORCE_UINT32 = 0xFFFFFFFF

class DIALOG_CONTROL_DPI_CHANGE_BEHAVIORS(IntEnum):
    DCDC_DEFAULT = 0x0
    DCDC_DISABLE_FONT_UPDATE = 0x1
    DCDC_DISABLE_RELAYOUT = 0x2

class TkResourceFindType(IntEnum):
    Fail = 0x0
    Find = 0x1
    Overwrite = 0x2

class _JOBOBJECT_RATE_CONTROL_TOLERANCE_INTERVAL(IntEnum):
    ToleranceIntervalShort = 0x1
    ToleranceIntervalMedium = 0x2
    ToleranceIntervalLong = 0x3

class TkPollResult(IntEnum):
    NotReady = 0x0
    Ready = 0x1

class _TOKEN_TYPE(IntEnum):
    TokenPrimary = 0x1

class DISPLAYCONFIG_ROTATION(IntEnum):
    DISPLAYCONFIG_ROTATION_IDENTITY = 0x1
    DISPLAYCONFIG_ROTATION_ROTATE90 = 0x2
    DISPLAYCONFIG_ROTATION_ROTATE180 = 0x3
    DISPLAYCONFIG_ROTATION_ROTATE270 = 0x4
    DISPLAYCONFIG_ROTATION_FORCE_UINT32 = 0xFFFFFFFF

class _PROCESSOR_CACHE_TYPE(IntEnum):
    CacheUnified = 0x0
    CacheInstruction = 0x1
    CacheData = 0x2
    CacheTrace = 0x3

class POWER_USER_PRESENCE_TYPE(IntEnum):
    UserNotPresent = 0x0
    UserPresent = 0x1
    UserUnknown = 0xFF

class tagAR_STATE(IntEnum):
    AR_ENABLED = 0x0
    AR_DISABLED = 0x1
    AR_SUPPRESSED = 0x2
    AR_REMOTESESSION = 0x4
    AR_MULTIMON = 0x8
    AR_NOSENSOR = 0x10
    AR_NOT_SUPPORTED = 0x20
    AR_DOCKED = 0x40
    AR_LAPTOP = 0x80

class _crt_argv_mode(IntEnum):
    _crt_argv_no_arguments = 0x0
    _crt_argv_unexpanded_arguments = 0x1
    _crt_argv_expanded_arguments = 0x2

class tagPOINTER_BUTTON_CHANGE_TYPE(IntEnum):
    POINTER_CHANGE_NONE = 0x0
    POINTER_CHANGE_FIRSTBUTTON_DOWN = 0x1
    POINTER_CHANGE_FIRSTBUTTON_UP = 0x2
    POINTER_CHANGE_SECONDBUTTON_DOWN = 0x3
    POINTER_CHANGE_SECONDBUTTON_UP = 0x4
    POINTER_CHANGE_THIRDBUTTON_DOWN = 0x5
    POINTER_CHANGE_THIRDBUTTON_UP = 0x6
    POINTER_CHANGE_FOURTHBUTTON_DOWN = 0x7
    POINTER_CHANGE_FOURTHBUTTON_UP = 0x8
    POINTER_CHANGE_FIFTHBUTTON_DOWN = 0x9
    POINTER_CHANGE_FIFTHBUTTON_UP = 0xA

class _LOGICAL_PROCESSOR_RELATIONSHIP(IntEnum):
    RelationProcessorCore = 0x0
    RelationNumaNode = 0x1
    RelationCache = 0x2
    RelationProcessorPackage = 0x3
    RelationGroup = 0x4
    RelationAll = 0xFFFF

class ORIENTATION_PREFERENCE(IntEnum):
    ORIENTATION_PREFERENCE_NONE = 0x0
    ORIENTATION_PREFERENCE_LANDSCAPE = 0x1
    ORIENTATION_PREFERENCE_PORTRAIT = 0x2
    ORIENTATION_PREFERENCE_LANDSCAPE_FLIPPED = 0x4
    ORIENTATION_PREFERENCE_PORTRAIT_FLIPPED = 0x8

class _FIRMWARE_TYPE(IntEnum):
    FirmwareTypeUnknown = 0x0
    FirmwareTypeBios = 0x1
    FirmwareTypeUefi = 0x2
    FirmwareTypeMax = 0x3

class MEM_SECTION_EXTENDED_PARAMETER_TYPE(IntEnum):
    MemSectionExtendedParameterInvalidType = 0x0
    MemSectionExtendedParameterUserPhysicalFlags = 0x1
    MemSectionExtendedParameterNumaNode = 0x2
    MemSectionExtendedParameterMax = 0x3

class COMPARTMENT_ID(IntEnum):
    UNSPECIFIED_COMPARTMENT_ID = 0x0
    DEFAULT_COMPARTMENT_ID = 0x1

class hkGpuProfilingGranularity(ctypes.Structure):
    class Enum(IntEnum):
        PROFILE_SCOPES = 0x0
        PROFILE_RENDERABLES = 0x1
        PROFILE_DRAW_CALL_RECORDING = 0x2
        PROFILE_DRAW_CALL_EXECUTION = 0x3
        PROFILE_PIXEL_COUNT = 0x4
        COUNT = 0x5


class eNGuiInputButtonState(IntEnum):
    eButton_None = 0x0
    eButton_Released = 0x1
    eButton_Pressed = 0x2
    eButton_Held = 0x3

class _TRANSACTIONMANAGER_INFORMATION_CLASS(IntEnum):
    TransactionManagerBasicInformation = 0x0
    TransactionManagerLogInformation = 0x1
    TransactionManagerLogPathInformation = 0x2
    TransactionManagerOnlineProbeInformation = 0x3
    TransactionManagerRecoveryInformation = 0x4
    TransactionManagerOldestTransactionInformation = 0x5

class _RTL_UMS_THREAD_INFO_CLASS(IntEnum):
    UmsThreadInvalidInfoClass = 0x0
    UmsThreadUserContext = 0x1
    UmsThreadPriority = 0x2
    UmsThreadAffinity = 0x3
    UmsThreadTeb = 0x4
    UmsThreadIsSuspended = 0x5
    UmsThreadIsTerminated = 0x6
    UmsThreadMaxInfoClass = 0x7

class _DEVICE_POWER_STATE(IntEnum):
    PowerDeviceUnspecified = 0x0
    PowerDeviceD0 = 0x1
    PowerDeviceD1 = 0x2
    PowerDeviceD2 = 0x3
    PowerDeviceD3 = 0x4
    PowerDeviceMaximum = 0x5

class POWER_INFORMATION_LEVEL(IntEnum):
    SystemPowerPolicyAc = 0x0
    SystemPowerPolicyDc = 0x1
    VerifySystemPolicyAc = 0x2
    VerifySystemPolicyDc = 0x3
    SystemPowerCapabilities = 0x4
    SystemBatteryState = 0x5
    SystemPowerStateHandler = 0x6
    ProcessorStateHandler = 0x7
    SystemPowerPolicyCurrent = 0x8
    AdministratorPowerPolicy = 0x9
    SystemReserveHiberFile = 0xA
    ProcessorInformation = 0xB
    SystemPowerInformation = 0xC
    ProcessorStateHandler2 = 0xD
    LastWakeTime = 0xE
    LastSleepTime = 0xF
    SystemExecutionState = 0x10
    SystemPowerStateNotifyHandler = 0x11
    ProcessorPowerPolicyAc = 0x12
    ProcessorPowerPolicyDc = 0x13
    VerifyProcessorPowerPolicyAc = 0x14
    VerifyProcessorPowerPolicyDc = 0x15
    ProcessorPowerPolicyCurrent = 0x16
    SystemPowerStateLogging = 0x17
    SystemPowerLoggingEntry = 0x18
    SetPowerSettingValue = 0x19
    NotifyUserPowerSetting = 0x1A
    PowerInformationLevelUnused0 = 0x1B
    SystemMonitorHiberBootPowerOff = 0x1C
    SystemVideoState = 0x1D
    TraceApplicationPowerMessage = 0x1E
    TraceApplicationPowerMessageEnd = 0x1F
    ProcessorPerfStates = 0x20
    ProcessorIdleStates = 0x21
    ProcessorCap = 0x22
    SystemWakeSource = 0x23
    SystemHiberFileInformation = 0x24
    TraceServicePowerMessage = 0x25
    ProcessorLoad = 0x26
    PowerShutdownNotification = 0x27
    MonitorCapabilities = 0x28
    SessionPowerInit = 0x29
    SessionDisplayState = 0x2A
    PowerRequestCreate = 0x2B
    PowerRequestAction = 0x2C
    GetPowerRequestList = 0x2D
    ProcessorInformationEx = 0x2E
    NotifyUserModeLegacyPowerEvent = 0x2F
    GroupPark = 0x30
    ProcessorIdleDomains = 0x31
    WakeTimerList = 0x32
    SystemHiberFileSize = 0x33
    ProcessorIdleStatesHv = 0x34
    ProcessorPerfStatesHv = 0x35
    ProcessorPerfCapHv = 0x36
    ProcessorSetIdle = 0x37
    LogicalProcessorIdling = 0x38
    UserPresence = 0x39
    PowerSettingNotificationName = 0x3A
    GetPowerSettingValue = 0x3B
    IdleResiliency = 0x3C
    SessionRITState = 0x3D
    SessionConnectNotification = 0x3E
    SessionPowerCleanup = 0x3F
    SessionLockState = 0x40
    SystemHiberbootState = 0x41
    PlatformInformation = 0x42
    PdcInvocation = 0x43
    MonitorInvocation = 0x44
    FirmwareTableInformationRegistered = 0x45
    SetShutdownSelectedTime = 0x46
    SuspendResumeInvocation = 0x47
    PlmPowerRequestCreate = 0x48
    ScreenOff = 0x49
    CsDeviceNotification = 0x4A
    PlatformRole = 0x4B
    LastResumePerformance = 0x4C
    DisplayBurst = 0x4D
    ExitLatencySamplingPercentage = 0x4E
    RegisterSpmPowerSettings = 0x4F
    PlatformIdleStates = 0x50
    ProcessorIdleVeto = 0x51
    PlatformIdleVeto = 0x52
    SystemBatteryStatePrecise = 0x53
    ThermalEvent = 0x54
    PowerRequestActionInternal = 0x55
    BatteryDeviceState = 0x56
    PowerInformationInternal = 0x57
    ThermalStandby = 0x58
    SystemHiberFileType = 0x59
    PhysicalPowerButtonPress = 0x5A
    QueryPotentialDripsConstraint = 0x5B
    EnergyTrackerCreate = 0x5C
    EnergyTrackerQuery = 0x5D
    UpdateBlackBoxRecorder = 0x5E
    SessionAllowExternalDmaDevices = 0x5F
    PowerInformationLevelMaximum = 0x60

class ADDRESS_MODE(IntEnum):
    AddrMode1616 = 0x0
    AddrMode1632 = 0x1
    AddrModeReal = 0x2
    AddrModeFlat = 0x3

class _PROC_THREAD_ATTRIBUTE_NUM(IntEnum):
    ProcThreadAttributeParentProcess = 0x0
    ProcThreadAttributeHandleList = 0x2
    ProcThreadAttributeGroupAffinity = 0x3
    ProcThreadAttributePreferredNode = 0x4
    ProcThreadAttributeIdealProcessor = 0x5
    ProcThreadAttributeUmsThread = 0x6
    ProcThreadAttributeMitigationPolicy = 0x7
    ProcThreadAttributeSecurityCapabilities = 0x9
    ProcThreadAttributeProtectionLevel = 0xB
    ProcThreadAttributeJobList = 0xD
    ProcThreadAttributeChildProcessPolicy = 0xE
    ProcThreadAttributeAllApplicationPackagesPolicy = 0xF
    ProcThreadAttributeWin32kFilter = 0x10
    ProcThreadAttributeSafeOpenPromptOriginClaim = 0x11
    ProcThreadAttributeDesktopAppPolicy = 0x12
    ProcThreadAttributePseudoConsole = 0x16
    ProcThreadAttributeMitigationAuditPolicy = 0x18

class WIN32_MEMORY_INFORMATION_CLASS(IntEnum):
    MemoryRegionInfo = 0x0

class _GET_FILEEX_INFO_LEVELS(IntEnum):
    GetFileExInfoStandard = 0x0
    GetFileExMaxInfoLevel = 0x1

class _ACTIVATION_CONTEXT_INFO_CLASS(IntEnum):
    ActivationContextBasicInformation = 0x1
    ActivationContextDetailedInformation = 0x2
    AssemblyDetailedInformationInActivationContext = 0x3
    FileInformationInAssemblyOfAssemblyInActivationContext = 0x4
    RunlevelInformationInActivationContext = 0x5
    CompatibilityInformationInActivationContext = 0x6
    ActivationContextManifestResourceName = 0x7
    MaxActivationContextInfoClass = 0x8

class _COPYFILE2_COPY_PHASE(IntEnum):
    COPYFILE2_PHASE_NONE = 0x0
    COPYFILE2_PHASE_PREPARE_SOURCE = 0x1
    COPYFILE2_PHASE_PREPARE_DEST = 0x2
    COPYFILE2_PHASE_READ_SOURCE = 0x3
    COPYFILE2_PHASE_WRITE_DESTINATION = 0x4
    COPYFILE2_PHASE_SERVER_COPY = 0x5
    COPYFILE2_PHASE_NAMEGRAFT_COPY = 0x6
    COPYFILE2_PHASE_MAX = 0x7

class SYSTEM_POWER_CONDITION(IntEnum):
    PoAc = 0x0
    PoDc = 0x1
    PoHot = 0x2
    PoConditionMaximum = 0x3

class _SharedVirtualDiskSupportType(IntEnum):
    SharedVirtualDisksUnsupported = 0x0
    SharedVirtualDisksSupported = 0x1
    SharedVirtualDiskSnapshotsSupported = 0x3
    SharedVirtualDiskCDPSnapshotsSupported = 0x7

class _SE_IMAGE_SIGNATURE_TYPE(IntEnum):
    SeImageSignatureNone = 0x0
    SeImageSignatureEmbedded = 0x1
    SeImageSignatureCache = 0x2
    SeImageSignatureCatalogCached = 0x3
    SeImageSignatureCatalogNotCached = 0x4
    SeImageSignatureCatalogHint = 0x5
    SeImageSignaturePackageCatalog = 0x6

class POWER_ACTION(IntEnum):
    PowerActionNone = 0x0
    PowerActionReserved = 0x1
    PowerActionSleep = 0x2
    PowerActionHibernate = 0x3
    PowerActionShutdown = 0x4
    PowerActionShutdownReset = 0x5
    PowerActionShutdownOff = 0x6
    PowerActionWarmEject = 0x7
    PowerActionDisplayOff = 0x8

class _SECURITY_IMPERSONATION_LEVEL(IntEnum):
    SecurityAnonymous = 0x0
    SecurityIdentification = 0x1
    SecurityImpersonation = 0x2
    SecurityDelegation = 0x3

class ACTCTX_REQUESTED_RUN_LEVEL(IntEnum):
    ACTCTX_RUN_LEVEL_UNSPECIFIED = 0x0
    ACTCTX_RUN_LEVEL_AS_INVOKER = 0x1
    ACTCTX_RUN_LEVEL_HIGHEST_AVAILABLE = 0x2
    ACTCTX_RUN_LEVEL_REQUIRE_ADMIN = 0x3
    ACTCTX_RUN_LEVEL_NUMBERS = 0x4

class _CM_SERVICE_NODE_TYPE(IntEnum):
    DriverType = 0x1

class SERVICE_DIRECTORY_TYPE(IntEnum):
    ServiceDirectoryPersistentState = 0x0
    ServiceDirectoryTypeMax = 0x1

class _SID_NAME_USE(IntEnum):
    SidTypeUser = 0x1
    SidTypeGroup = 0x2
    SidTypeDomain = 0x3
    SidTypeAlias = 0x4
    SidTypeWellKnownGroup = 0x5
    SidTypeDeletedAccount = 0x6
    SidTypeInvalid = 0x7
    SidTypeUnknown = 0x8
    SidTypeComputer = 0x9
    SidTypeLabel = 0xA
    SidTypeLogonSession = 0xB

class _TOKEN_INFORMATION_CLASS(IntEnum):
    TokenUser = 0x1
    TokenGroups = 0x2
    TokenPrivileges = 0x3
    TokenOwner = 0x4
    TokenPrimaryGroup = 0x5
    TokenDefaultDacl = 0x6
    TokenSource = 0x7
    TokenType = 0x8
    TokenImpersonationLevel = 0x9
    TokenStatistics = 0xA
    TokenRestrictedSids = 0xB
    TokenSessionId = 0xC
    TokenGroupsAndPrivileges = 0xD
    TokenSessionReference = 0xE
    TokenSandBoxInert = 0xF
    TokenAuditPolicy = 0x10
    TokenOrigin = 0x11
    TokenElevationType = 0x12
    TokenLinkedToken = 0x13
    TokenElevation = 0x14
    TokenHasRestrictions = 0x15
    TokenAccessInformation = 0x16
    TokenVirtualizationAllowed = 0x17
    TokenVirtualizationEnabled = 0x18
    TokenIntegrityLevel = 0x19
    TokenUIAccess = 0x1A
    TokenMandatoryPolicy = 0x1B
    TokenLogonSid = 0x1C
    TokenIsAppContainer = 0x1D
    TokenCapabilities = 0x1E
    TokenAppContainerSid = 0x1F
    TokenAppContainerNumber = 0x20
    TokenUserClaimAttributes = 0x21
    TokenDeviceClaimAttributes = 0x22
    TokenRestrictedUserClaimAttributes = 0x23
    TokenRestrictedDeviceClaimAttributes = 0x24
    TokenDeviceGroups = 0x25
    TokenRestrictedDeviceGroups = 0x26
    TokenSecurityAttributes = 0x27
    TokenIsRestricted = 0x28
    TokenProcessTrustLevel = 0x29
    TokenPrivateNameSpace = 0x2A
    TokenSingletonAttributes = 0x2B
    TokenBnoIsolation = 0x2C
    TokenChildProcessFlags = 0x2D
    TokenIsLessPrivilegedAppContainer = 0x2E
    TokenIsSandboxed = 0x2F
    TokenOriginatingProcessTrustLevel = 0x30
    MaxTokenInfoClass = 0x31

class MEM_EXTENDED_PARAMETER_TYPE(IntEnum):
    MemExtendedParameterInvalidType = 0x0
    MemExtendedParameterAddressRequirements = 0x1
    MemExtendedParameterNumaNode = 0x2
    MemExtendedParameterPartitionHandle = 0x3
    MemExtendedParameterUserPhysicalHandle = 0x4
    MemExtendedParameterAttributeFlags = 0x5
    MemExtendedParameterMax = 0x6

class _PROCESS_MEMORY_EXHAUSTION_TYPE(IntEnum):
    PMETypeFailFastOnCommitFailure = 0x0
    PMETypeMax = 0x1

class POINTER_FEEDBACK_MODE(IntEnum):
    POINTER_FEEDBACK_DEFAULT = 0x1
    POINTER_FEEDBACK_INDIRECT = 0x2
    POINTER_FEEDBACK_NONE = 0x3

class _ENLISTMENT_INFORMATION_CLASS(IntEnum):
    EnlistmentBasicInformation = 0x0
    EnlistmentRecoveryInformation = 0x1
    EnlistmentCrmInformation = 0x2

class tagHANDEDNESS(IntEnum):
    HANDEDNESS_LEFT = 0x0
    HANDEDNESS_RIGHT = 0x1

class tagPOINTER_DEVICE_TYPE(IntEnum):
    POINTER_DEVICE_TYPE_INTEGRATED_PEN = 0x1
    POINTER_DEVICE_TYPE_EXTERNAL_PEN = 0x2
    POINTER_DEVICE_TYPE_TOUCH = 0x3
    POINTER_DEVICE_TYPE_TOUCH_PAD = 0x4
    POINTER_DEVICE_TYPE_MAX = 0xFFFFFFFF

class _COPYFILE2_MESSAGE_ACTION(IntEnum):
    COPYFILE2_PROGRESS_CONTINUE = 0x0
    COPYFILE2_PROGRESS_CANCEL = 0x1
    COPYFILE2_PROGRESS_STOP = 0x2
    COPYFILE2_PROGRESS_QUIET = 0x3
    COPYFILE2_PROGRESS_PAUSE = 0x4

class hkMxVectorPermutation(ctypes.Structure):
    class Permutation(IntEnum):
        SHIFT_RIGHT_CYCLIC = 0x0
        SHIFT_LEFT_CYCLIC = 0x1
        REVERSE = 0x2


class hkBitFieldLoop(ctypes.Structure):
    class Dir(IntEnum):
        ASCENDING = 0x0
        DESCENDING = 0x1


class hkMxVectorf_Implementation(ctypes.Structure):
    pass

class hkBitFieldBit(ctypes.Structure):
    class Type(IntEnum):
        CLEARED = 0x0
        SET = 0x1


class hkMxVectord_Implementation(ctypes.Structure):
    pass

class hknpCollisionFlags(ctypes.Structure):
    class Enum(IntEnum):
        IS_STATIC = 0x1
        IS_DYNAMIC = 0x2
        IS_KEYFRAMED = 0x4
        IS_ACTIVE = 0x8
        MASK_INTERNAL_BODY_FLAGS = 0xF
        IS_PARTICLES_COLLIDER = 0x10
        DONT_COLLIDE = 0x20
        DONT_BUILD_CONTACT_JACOBIANS = 0x40
        DISABLE_PENETRATION_RECOVERY = 0x80
        RAISE_TRIGGER_EVENTS = 0x100
        RAISE_POTENTIAL_MANIFOLD_EVENTS = 0x200
        RAISE_MANIFOLD_CREATED_EVENTS = 0x400
        RAISE_MANIFOLD_UPDATED_EVENTS = 0x800
        RAISE_MANIFOLD_DESTROYED_EVENTS = 0x1000
        RAISE_ALL_MANIFOLD_EVENTS = 0x1C00
        RAISE_CONTACT_IMPULSE_STARTED_EVENTS = 0x2000
        RAISE_CONTACT_IMPULSE_CONTINUED_EVENTS = 0x4000
        RAISE_CONTACT_IMPULSE_FINISHED_EVENTS = 0x8000
        RAISE_ALL_CONTACT_IMPULSE_EVENTS = 0xE000
        ENABLE_RESTITUTION = 0x10000
        ENABLE_TRIGGER_MODIFIER = 0x20000
        ENABLE_IMPULSE_CLIPPING = 0x40000
        ENABLE_MASS_CHANGER = 0x80000
        ENABLE_SOFT_CONTACTS = 0x100000
        ENABLE_SURFACE_VELOCITY = 0x200000
        MASK_MATERIAL_FLAGS = 0x3F0000
        ENABLE_TIME_FACTOR = 0x400000
        ENABLE_ANIMATED_SHAPE_VELOCITY = 0x800000
        FLIP_BACKFACE_COLLISIONS = 0x1000000
        USER_FLAG_0 = 0x4000000
        USER_FLAG_1 = 0x8000000
        USER_FLAG_2 = 0x10000000
        USER_FLAG_3 = 0x20000000
        USER_FLAG_4 = 0x40000000
        USER_FLAG_5 = 0x80000000
        MASK_ALL_FLAGS = 0xFFFFFFFF

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x4),  # hkFlags<void,int>
    ]
    baseclass_0: bytes

class hknpMotionUtil(ctypes.Structure):
    class AabbSweepMode(IntEnum):
        SWEEP_EXISTING_AABB = 0x0
        CALC_NEW_AABB = 0x1

    class AabbUpdateMode(IntEnum):
        FORCE_UPDATE_ON_AABB_SHRINK = 0x0
        DONT_FORCE_UPDATE = 0x1

    def buildSolverVelocities(): pass

    def calcFutureAabb(): pass

    def calcMotionMaxRotationPerStep(): pass

    def gatherSolverVelocities(): pass

    def integrateMotionTransform(): pass

    def updateBodyAfterMotionIntegration(): pass

    def updateBodyAfterMotionIntegrationInternal(): pass

    def updateBodyTransformFromMotion(): pass

    def updateSweptBodyAabbWithMargin(): pass

    def applyDrag(): pass

    def predictBodyTransform(): pass


class TrileanEnum(IntEnum):
    triFalse = 0x0
    triTrue = 0x1
    triMaybe = 0xFFFFFFFF

class hknpConstraintSolverType(ctypes.Structure):
    class Enum(IntEnum):
        CONTACT_CONSTRAINT_SOLVER = 0x0
        ATOM_CONSTRAINT_SOLVER = 0x1
        USER_1 = 0x2
        USER_2 = 0x3
        USER_3 = 0x4
        USER_4 = 0x5
        USER_5 = 0x6
        USER_6 = 0x7
        NUM_TYPES = 0x8


class hknpImpulseClipMode(ctypes.Structure):
    class Enum(IntEnum):
        CLIP_NEITHER_IMPULSES = 0x0
        CLIP_BOTH_IMPULSES = 0x1
        CLIP_IMPULSE_A = 0x2
        CLIP_IMPULSE_B = 0x3


class hknpActivationState(ctypes.Structure):
    class Enum(IntEnum):
        ACTIVE = 0x0
        INACTIVE = 0x1


class hknpAngularTimType(ctypes.Structure):
    _fields_ = [
        ("m_value", ctypes.c_uint16),
    ]
    m_value: int

class hknpInplaceTriangleShape(ctypes.Structure):
    def hknpInplaceTriangleShape(): pass

    _fields_ = [
        ("m_buffer", ctypes.c_uint8 * 0x110),  # unsigned __int8[272]
    ]
    m_buffer: list[ctypes.c_uint8]

class hknpBroadPhase(ctypes.Structure):
    class AdditionModeHint(IntEnum):
        ADD_NOW = 0x0
        ADD_DURING_UPDATE = 0x1

    class UpdateModeHint(IntEnum):
        UPDATE_IN_NEXT_STEP = 0x0
        UPDATE_NOW = 0x1

    def create(): pass

    def getClosestPointsParticles(): pass

    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hknpCollisionQueryType(ctypes.Structure):
    class Enum(IntEnum):
        UNDEFINED = 0x0
        RAY_CAST = 0x1
        SHAPE_CAST = 0x2
        AABB_CAST = 0x3
        POINT_QUERY = 0x4
        GET_CLOSEST_POINTS = 0x5
        QUERY_AABB = 0x6


class hkcdPointConvexVoronoiCache(ctypes.Structure):
    class Type(IntEnum):
        TYPE_INVALID = 0x0
        TYPE_FACE = 0x1
        TYPE_FACE_POS_DISTANCE = 0x2
        TYPE_EDGE = 0x3
        TYPE_TOO_FAR_AWAY = 0x4

    _fields_ = [
        ("m_type", ctypes.c_char * 0x1),  # hkEnum<enum hkcdPointConvexVoronoiCache::Type,unsigned char>
        ("m_cacheId0", ctypes.c_uint8),
        ("m_cacheId1", ctypes.c_uint8),
    ]
    m_type: bytes
    m_cacheId0: int
    m_cacheId1: int

class hkcdTriangleUtil(ctypes.Structure):
    class CheckFlatConvexQuadMode(IntEnum):
        IGNORE_QUAD_AREA = 0x0
        CHECK_QUAD_AREA = 0x1

    def checkForFlatConvexQuad(): pass

    def isDegenerate(): pass


class hkcdRayCastCapsuleHitType(IntEnum):
    HK_CD_RAY_CAPSULE_NONE = 0x0
    HK_CD_RAY_CAPSULE_CAP0 = 0x1
    HK_CD_RAY_CAPSULE_CAP1 = 0x2
    HK_CD_RAY_CAPSULE_BODY = 0x3

class hknpCollisionControl(ctypes.Structure):
    class Enum(IntEnum):
        DESTROY_COLLISION_CACHES = 0x1
        DROP_NEW_CVX_CVX_COLLISIONS = 0x2
        REBUILD_CONTACT_JACOBIANS = 0x4
        UNCERTAIN_ACTIVATION_STATE = 0x8
        BODY_AABB_CHANGED = 0x10
        MASK_BROADPHASE_OVERLAPS_CHANGED = 0x11
        CELL_INDEX_CHANGED = 0x20
        PERFORM_LOD_CHECK = 0x40
        TRANSFORM_CHANGED = 0x80
        SHAPE_HAS_MUTATED = 0x100
        OTHER_BODY_PROPERTY_CHANGED = 0x200
        NONZERO_VELOCITY = 0x400
        ACTIVATE_COLLISION_CACHES = 0x800
        MASK_BODY_CHANGED_FLAGS = 0xFF0
        APPLIED_RESTITUTION = 0x1000
        CIRCULAR_SYMMETRY_CHANGED = 0x2000
        MASK_UPDATE_COLLISION_CACHE = 0x2341
        CELL_INDEX_CHANGED_MID_STEP = 0x4000
        DEFLECTED_LINEAR_CAST_PERFORMED = 0x8000

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x2),  # hkFlags<void,unsigned short>
    ]
    baseclass_0: bytes

class hknpBodyPropertyKeys(ctypes.Structure):
    pass

class hknpShapePropertyKeys(ctypes.Structure):
    pass

class hknpSimulationType(ctypes.Structure):
    class Enum(IntEnum):
        SINGLE_THREADED = 0x0
        MULTI_THREADED = 0x1


class hkBlockStream_SerializeHeader(ctypes.Structure):
    _fields_ = [
        ("m_magic", ctypes.c_int32),
        ("m_numTotalElements", ctypes.c_int32),
        ("m_numblocks", ctypes.c_int32),
    ]
    m_magic: int
    m_numTotalElements: int
    m_numblocks: int

class hknpEventType(ctypes.Structure):
    class Enum(IntEnum):
        TRIGGER = 0x0
        POTENTIAL_MANIFOLD = 0x1
        MANIFOLD = 0x2
        CONTACT_IMPULSE = 0x3
        CONTACT_IMPULSE_CLIPPED = 0x4
        DEFLECTED_LINEAR_CAST_COLLISION = 0x5
        FULL_CAST_COLLISION = 0x6
        BINARY_BODY_DEBUG_EVENT = 0x7
        PARTICLE_BODY_IMPULSE_APPLIED = 0x8
        USER_BINARY_0 = 0x9
        USER_BINARY_1 = 0xA
        USER_BINARY_2 = 0xB
        USER_BINARY_3 = 0xC
        END_OF_BINARY_BODY_EVENT = 0xD
        BODY_EXITED_SAFE_REGION = 0xE
        LINEAR_INTEGRATION_CLIPPED = 0xF
        BODY_BREACHED_EXPANDED_AABB = 0x10
        UNARY_BODY_DEBUG_EVENT = 0x11
        PARTICLES_COLLIDED_WITH_BODIES = 0x12
        PARTICLES_COLLIDED_WITH_PARTICLES = 0x13
        PARTICLE_PARTICLE_IMPULSE_APPLIED = 0x14
        PARTICLES_EXITED_SAFE_REGION = 0x15
        USER_UNARY_0 = 0x16
        USER_UNARY_1 = 0x17
        USER_UNARY_2 = 0x18
        USER_UNARY_3 = 0x19
        END_OF_UNARY_BODY_EVENTS = 0x1A
        CONSTRAINT_FORCE = 0x1B
        CONSTRAINT_FORCE_EXCEEDED = 0x1C
        RESERVED_0 = 0x1D
        RESERVED_1 = 0x1E
        EVENTS_MAX = 0x1F


class IPPROTO(IntEnum):
    IPPROTO_HOPOPTS = 0x0
    IPPROTO_ICMP = 0x1
    IPPROTO_IGMP = 0x2
    IPPROTO_GGP = 0x3
    IPPROTO_IPV4 = 0x4
    IPPROTO_ST = 0x5
    IPPROTO_TCP = 0x6
    IPPROTO_CBT = 0x7
    IPPROTO_EGP = 0x8
    IPPROTO_IGP = 0x9
    IPPROTO_PUP = 0xC
    IPPROTO_UDP = 0x11
    IPPROTO_IDP = 0x16
    IPPROTO_RDP = 0x1B
    IPPROTO_IPV6 = 0x29
    IPPROTO_ROUTING = 0x2B
    IPPROTO_FRAGMENT = 0x2C
    IPPROTO_ESP = 0x32
    IPPROTO_AH = 0x33
    IPPROTO_ICMPV6 = 0x3A
    IPPROTO_NONE = 0x3B
    IPPROTO_DSTOPTS = 0x3C
    IPPROTO_ND = 0x4D
    IPPROTO_ICLFXBM = 0x4E
    IPPROTO_PIM = 0x67
    IPPROTO_PGM = 0x71
    IPPROTO_L2TP = 0x73
    IPPROTO_SCTP = 0x84
    IPPROTO_RAW = 0xFF
    IPPROTO_MAX = 0x100
    IPPROTO_RESERVED_RAW = 0x101
    IPPROTO_RESERVED_IPSEC = 0x102
    IPPROTO_RESERVED_IPSECOFFLOAD = 0x103
    IPPROTO_RESERVED_WNV = 0x104
    IPPROTO_RESERVED_MAX = 0x105

class SCOPE_LEVEL(IntEnum):
    ScopeLevelInterface = 0x1
    ScopeLevelLink = 0x2
    ScopeLevelSubnet = 0x3
    ScopeLevelAdmin = 0x4
    ScopeLevelSite = 0x5
    ScopeLevelOrganization = 0x8
    ScopeLevelGlobal = 0xE
    ScopeLevelCount = 0x10

class MULTICAST_MODE_TYPE(IntEnum):
    MCAST_INCLUDE = 0x0
    MCAST_EXCLUDE = 0x1

class hknpCompressedMeshShapeCinfo(ctypes.Structure):
    class ConvexShapeType(IntEnum):
        CONVEX_POLYTOPE = 0x0
        COMPRESSED_FACELESS = 0x1

    def hknpCompressedMeshShapeCinfo(): pass

    def tilde_hknpCompressedMeshShapeCinfo(): pass

    def scalar_deleting_destructor(): pass

    def getConvexShapeTag(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_convexRadius", ctypes.c_float),
        ("m_mergeCoplanarTriangles", ctypes.c_ubyte),
        ("m_mergeCoplanarTrianglesTolerance", ctypes.c_float),
        ("m_flagInteriorTriangles", ctypes.c_ubyte),
        ("m_removeDegenerateTriangles", ctypes.c_ubyte),
        ("m_maxVertexError", ctypes.c_float),
        ("m_maxConvexShapeError", ctypes.c_float),
        ("m_preserveVertexOrder", ctypes.c_ubyte),
        ("m_optimizeForSpeed", ctypes.c_ubyte),
        ("m_compressOpenEdges", ctypes.c_ubyte),
        ("m_buildConnectivity", ctypes.c_ubyte),
        ("m_verbose", ctypes.c_ubyte),
        ("_m_convexShapeStorageType", ctypes.c_uint32),
        ("m_triangleIndexToShapeKeyMap", ctypes.c_char * 0x8),  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
        ("m_triangleIndexToVertexOrderMap", ctypes.c_char * 0x8),  # hkArray<unsigned char,hkBuiltinContainerAllocator<0,0> > *
        ("m_convexIndexToShapeKeyMap", ctypes.c_char * 0x8),  # hkArray<hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>,hkBuiltinContainerAllocator<0,0> > *
    ]
    __vftable: int
    m_convexRadius: float
    m_mergeCoplanarTriangles: bool
    m_mergeCoplanarTrianglesTolerance: float
    m_flagInteriorTriangles: bool
    m_removeDegenerateTriangles: bool
    m_maxVertexError: float
    m_maxConvexShapeError: float
    m_preserveVertexOrder: bool
    m_optimizeForSpeed: bool
    m_compressOpenEdges: bool
    m_buildConnectivity: bool
    m_verbose: bool
    _m_convexShapeStorageType: int
    m_triangleIndexToShapeKeyMap: bytes
    m_triangleIndexToVertexOrderMap: bytes
    m_convexIndexToShapeKeyMap: bytes

    @property
    def m_convexShapeStorageType(self):
        return hknpCompressedMeshShapeCinfo.ConvexShapeType(self._m_convexShapeStorageType)


class hknpCollisionCacheWorkItem(ctypes.Structure):
    class CacheSource(IntEnum):
        NEW_THIS_FRAME = 0x0
        INTERRUPTED_ROUND_1 = 0x1
        DEFERED_UNTIL_ROUND_2 = 0x2
        REACTIVATED = 0x3

    _fields_ = [
        ("m_cacheBlock", ctypes.c_ulonglong),
        ("m_blockOffset", ctypes.c_uint16),
        ("m_padding", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
        ("m_cacheSource", ctypes.c_char * 0x1),  # hkEnum<enum hknpCollisionCacheWorkItem::CacheSource,unsigned char>
        ("m_sourceLinkIndex", ctypes.c_int32),
    ]
    m_cacheBlock: int
    m_blockOffset: int
    m_padding: list[ctypes.c_uint8]
    m_cacheSource: bytes
    m_sourceLinkIndex: int

class hkDisplayGeometryType(IntEnum):
    HK_DISPLAY_INVALID = 0x0
    HK_DISPLAY_SPHERE = 0x1
    HK_DISPLAY_BOX = 0x2
    HK_DISPLAY_AABB = 0x3
    HK_DISPLAY_CONE = 0x4
    HK_DISPLAY_SEMICIRCLE = 0x5
    HK_DISPLAY_ANY = 0x6
    HK_DISPLAY_PLANE = 0x7
    HK_DISPLAY_CAPSULE = 0x8
    HK_DISPLAY_CYLINDER = 0x9
    HK_DISPLAY_TAPERED_CAPSULE = 0xA
    HK_NUM_SUPPORTED_DISPLAY_TYPES = 0xB
    HK_DISPLAY_WIREFRAME = 0xC

class hknpContactJacobianUtil(ctypes.Structure):
    class BackstepMode(IntEnum):
        DONT_BACKSTEP = 0x0
        BACKSTEP = 0x1

    def accessPointEffectiveMass(): pass

    def accessPointRhs(): pass

    def calculateManifold(): pass

    def disableContacts(): pass

    def getBodyIdA(): pass

    def getBodyIdB(): pass

    def getBodyIndexA(): pass

    def getBodyIndexB(): pass

    def getManifoldType(): pass

    def getNumContactPoints(): pass

    def getShapeKeyA(): pass

    def getShapeKeyB(): pass

    def scaleEffectiveMass(): pass

    def scaleFriction(): pass

    def scalePenetrations(): pass

    def setMaxImpulse(): pass

    def setSurfaceVelocity(): pass


class hknpGroupCollisionFilterTypes(ctypes.Structure):
    pass

class hknpVelAccumulatorJacobianBuilding(ctypes.Structure):
    class hknpAccumulatorType(IntEnum):
        HK_RIGID_BODY = 0x0
        HK_KEYFRAMED_RIGID_BODY = 0x1
        HK_NO_GRAVITY_RIGID_BODY = 0x2
        HK_END = 0x3

    class hknpAccumulatorContext(IntEnum):
        ACCUMULATOR_CONTEXT_BUILD_JACOBIANS = 0x0
        ACCUMULATOR_CONTEXT_SOLVER = 0x1

    _fields_ = [
        ("m_type", ctypes.c_char * 0x1),  # hkEnum<enum hknpVelAccumulatorJacobianBuilding::hknpAccumulatorType,unsigned char>
        ("m_context", ctypes.c_char * 0x1),  # hkEnum<enum hknpVelAccumulatorJacobianBuilding::hknpAccumulatorContext,unsigned char>
        ("m_deactivationClass", ctypes.c_uint32),
        ("m_gravityFactor", ctypes.c_float),
        ("m_linearVel", common.Vector4f),
        ("m_angularVel", common.Vector4f),
        ("m_invMasses", common.Vector4f),
        ("m_scratch0", common.Vector4f),
        ("m_scratch1", common.Vector4f),
        ("m_scratch2", common.Vector4f),
        ("m_com", common.Vector4f),
    ]
    m_type: bytes
    m_context: bytes
    m_deactivationClass: int
    m_gravityFactor: float
    m_linearVel: common.Vector4f
    m_angularVel: common.Vector4f
    m_invMasses: common.Vector4f
    m_scratch0: common.Vector4f
    m_scratch1: common.Vector4f
    m_scratch2: common.Vector4f
    m_com: common.Vector4f

class hknpLodCollisionInfo(ctypes.Structure):
    class Masks(IntEnum):
        PENETRATION_RECOVERY_CONTROL_MASK = 0x3
        SHAPE_B_LEVEL_MASK = 0x1C
        SHAPE_A_LEVEL_MASK = 0xE0
        SHAPES_LEVEL_MASK = 0xFC

    _fields_ = [
        ("m_data", ctypes.c_uint8),
    ]
    m_data: int

class UnwrapMode(IntEnum):
    UNWRAP_QUERY = 0x0
    UNWRAP_TARGET = 0x1
    UNWRAP_BOTH = 0x2

class ISA_AVAILABILITY(IntEnum):
    __ISA_AVAILABLE_X86 = 0x0
    __ISA_AVAILABLE_SSE2 = 0x1
    __ISA_AVAILABLE_SSE42 = 0x2
    __ISA_AVAILABLE_AVX = 0x3
    __ISA_AVAILABLE_ENFSTRG = 0x4
    __ISA_AVAILABLE_AVX2 = 0x5
    __ISA_AVAILABLE_AVX512 = 0x6

class _NORM_FORM(IntEnum):
    NormalizationOther = 0x0
    NormalizationC = 0x1
    NormalizationD = 0x2
    NormalizationKC = 0x5
    NormalizationKD = 0x6

class _STREAM_INFO_LEVELS(IntEnum):
    FindStreamInfoStandard = 0x0
    FindStreamInfoMaxInfoLevel = 0x1

class _TRANSACTION_STATE(IntEnum):
    TransactionStateNormal = 0x1
    TransactionStateIndoubt = 0x2
    TransactionStateCommittedNotify = 0x3

class DISPLAYCONFIG_VIDEO_OUTPUT_TECHNOLOGY(IntEnum):
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_HD15 = 0x0
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_SVIDEO = 0x1
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_COMPOSITE_VIDEO = 0x2
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_COMPONENT_VIDEO = 0x3
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_DVI = 0x4
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_HDMI = 0x5
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_LVDS = 0x6
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_D_JPN = 0x8
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_SDI = 0x9
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_DISPLAYPORT_EXTERNAL = 0xA
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_DISPLAYPORT_EMBEDDED = 0xB
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_UDI_EXTERNAL = 0xC
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_UDI_EMBEDDED = 0xD
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_SDTVDONGLE = 0xE
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_MIRACAST = 0xF
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_INDIRECT_WIRED = 0x10
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_INDIRECT_VIRTUAL = 0x11
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_INTERNAL = 0x80000000
    DISPLAYCONFIG_OUTPUT_TECHNOLOGY_OTHER = 0xFFFFFFFF

class _READ_DIRECTORY_NOTIFY_INFORMATION_CLASS(IntEnum):
    ReadDirectoryNotifyInformation = 0x1

class _ACL_INFORMATION_CLASS(IntEnum):
    AclRevisionInformation = 0x1

class _CM_ERROR_CONTROL_TYPE(IntEnum):
    IgnoreError = 0x0
    NormalError = 0x1
    SevereError = 0x2
    CriticalError = 0x3

class hknpCharacterProxyProperty(ctypes.Structure):
    _fields_ = [
        ("m_proxyController", ctypes.c_ulonglong),
    ]
    m_proxyController: int

class _CM_SERVICE_LOAD_TYPE(IntEnum):
    BootLoad = 0x0
    SystemLoad = 0x1
    AutoLoad = 0x2
    DemandLoad = 0x3
    DisableLoad = 0x4

class _SC_ACTION_TYPE(IntEnum):
    SC_ACTION_NONE = 0x0
    SC_ACTION_RESTART = 0x1
    SC_ACTION_REBOOT = 0x2
    SC_ACTION_RUN_COMMAND = 0x3
    SC_ACTION_OWN_RESTART = 0x4

class DISPLAYCONFIG_TOPOLOGY_ID(IntEnum):
    DISPLAYCONFIG_TOPOLOGY_INTERNAL = 0x1
    DISPLAYCONFIG_TOPOLOGY_CLONE = 0x2
    DISPLAYCONFIG_TOPOLOGY_EXTEND = 0x4
    DISPLAYCONFIG_TOPOLOGY_EXTERNAL = 0x8
    DISPLAYCONFIG_TOPOLOGY_FORCE_UINT32 = 0xFFFFFFFF

class _TRANSACTION_OUTCOME(IntEnum):
    TransactionOutcomeUndetermined = 0x1
    TransactionOutcomeCommitted = 0x2
    TransactionOutcomeAborted = 0x3

class eMemoryHeapFlags(IntEnum):
    EMemoryHeapFlag_FlexibleMem = 0x1

class hkSurfaceConstraintInteraction(ctypes.Structure):
    class hkStatus(IntEnum):
        HK_STATUS_OK = 0x0
        HK_STATUS_3D_FAILURE = 0x1
        HK_STATUS_2D_FAILURE = 0x2

    _fields_ = [
        ("m_touched", ctypes.c_uint8),
        ("m_stopped", ctypes.c_uint8),
        ("m_surfaceTime", ctypes.c_float),
        ("m_penaltyDistance", ctypes.c_float),
        ("_m_status", ctypes.c_uint32),
    ]
    m_touched: int
    m_stopped: int
    m_surfaceTime: float
    m_penaltyDistance: float
    _m_status: int

    @property
    def m_status(self):
        return hkSurfaceConstraintInteraction.hkStatus(self._m_status)


class DISPLAYCONFIG_SCALING(IntEnum):
    DISPLAYCONFIG_SCALING_IDENTITY = 0x1
    DISPLAYCONFIG_SCALING_CENTERED = 0x2
    DISPLAYCONFIG_SCALING_STRETCHED = 0x3
    DISPLAYCONFIG_SCALING_ASPECTRATIOCENTEREDMAX = 0x4
    DISPLAYCONFIG_SCALING_CUSTOM = 0x5
    DISPLAYCONFIG_SCALING_PREFERRED = 0x80
    DISPLAYCONFIG_SCALING_FORCE_UINT32 = 0xFFFFFFFF

class DISPLAYCONFIG_MODE_INFO_TYPE(IntEnum):
    DISPLAYCONFIG_MODE_INFO_TYPE_SOURCE = 0x1
    DISPLAYCONFIG_MODE_INFO_TYPE_TARGET = 0x2
    DISPLAYCONFIG_MODE_INFO_TYPE_DESKTOP_IMAGE = 0x3
    DISPLAYCONFIG_MODE_INFO_TYPE_FORCE_UINT32 = 0xFFFFFFFF

class eRegion(IntEnum):
    ERegion_NorthAmerica = 0x0
    ERegion_Europe = 0x1
    ERegion_Japan = 0x2
    ENumRegions = 0x3

class hknpSolverSchedulerTaskData(ctypes.Structure):
    class VelocityUpdateFlags(IntEnum):
        UPDATE_NONE = 0x0
        UPDATE_PRESOLVE_A = 0x1
        UPDATE_PRESOLVE_B = 0x2
        UPDATE_POSTSOLVE_A = 0x4
        UPDATE_POSTSOLVE_B = 0x8

    class ProcessType(IntEnum):
        PROCESS_STEP_JACOBIAN_CONSTRAINT = 0x0
        PROCESS_FULL_STEP = 0x1

    _fields_ = [
        ("m_processType", ctypes.c_char * 0x1),  # hkEnum<enum hknpSolverSchedulerTaskData::ProcessType,unsigned char>
        ("m_grid", ctypes.c_char * 0x1),  # hkEnum<enum hknpJacobianGridType::Enum,unsigned char>
        ("m_gridEntryIndex", ctypes.c_uint16),
        ("m_cellA", ctypes.c_uint8),
        ("m_cellB", ctypes.c_uint8),
        ("m_velocityUpdateFlags", ctypes.c_char * 0x1),  # hkFlags<enum hknpSolverSchedulerTaskData::VelocityUpdateFlags,unsigned char>
        ("m_pad", ctypes.c_uint8),
    ]
    m_processType: bytes
    m_grid: bytes
    m_gridEntryIndex: int
    m_cellA: int
    m_cellB: int
    m_velocityUpdateFlags: bytes
    m_pad: int

class hknpConstraintDataUtils(ctypes.Structure):
    class Pivot(IntEnum):
        PIVOT_A = 0x0
        PIVOT_B = 0x1

    def getConstraintPivots(): pass

    def getSolverImpulsesInSolverSpaceFromAtom(): pass

    def isSupportedByDirectSolver(): pass

    def transformPivot(): pass


class _DISPLAYCONFIG_COLOR_ENCODING(IntEnum):
    DISPLAYCONFIG_COLOR_ENCODING_RGB = 0x0
    DISPLAYCONFIG_COLOR_ENCODING_YCBCR444 = 0x1
    DISPLAYCONFIG_COLOR_ENCODING_YCBCR422 = 0x2
    DISPLAYCONFIG_COLOR_ENCODING_YCBCR420 = 0x3
    DISPLAYCONFIG_COLOR_ENCODING_INTENSITY = 0x4
    DISPLAYCONFIG_COLOR_ENCODING_FORCE_UINT32 = 0xFFFFFFFF

class tagFEEDBACK_TYPE(IntEnum):
    FEEDBACK_TOUCH_CONTACTVISUALIZATION = 0x1
    FEEDBACK_PEN_BARRELVISUALIZATION = 0x2
    FEEDBACK_PEN_TAP = 0x3
    FEEDBACK_PEN_DOUBLETAP = 0x4
    FEEDBACK_PEN_PRESSANDHOLD = 0x5
    FEEDBACK_PEN_RIGHTTAP = 0x6
    FEEDBACK_TOUCH_TAP = 0x7
    FEEDBACK_TOUCH_DOUBLETAP = 0x8
    FEEDBACK_TOUCH_PRESSANDHOLD = 0x9
    FEEDBACK_TOUCH_RIGHTTAP = 0xA
    FEEDBACK_GESTURE_PRESSANDTAP = 0xB
    FEEDBACK_MAX = 0xFFFFFFFF

class eIterateReturn(IntEnum):
    Continue = 0x0
    Finish = 0x1

class _TAPE_DRIVE_PROBLEM_TYPE(IntEnum):
    TapeDriveProblemNone = 0x0
    TapeDriveReadWriteWarning = 0x1
    TapeDriveReadWriteError = 0x2
    TapeDriveReadWarning = 0x3
    TapeDriveWriteWarning = 0x4
    TapeDriveReadError = 0x5
    TapeDriveWriteError = 0x6
    TapeDriveHardwareError = 0x7
    TapeDriveUnsupportedMedia = 0x8
    TapeDriveScsiConnectionError = 0x9
    TapeDriveTimetoClean = 0xA
    TapeDriveCleanDriveNow = 0xB
    TapeDriveMediaLifeExpired = 0xC
    TapeDriveSnappedTape = 0xD

class _MONITOR_DISPLAY_STATE(IntEnum):
    PowerMonitorOff = 0x0
    PowerMonitorOn = 0x1
    PowerMonitorDim = 0x2

class EDIT_CONTROL_FEATURE(IntEnum):
    EDIT_CONTROL_FEATURE_ENTERPRISE_DATA_PROTECTION_PASTE_SUPPORT = 0x0
    EDIT_CONTROL_FEATURE_PASTE_NOTIFICATIONS = 0x1

class _POWER_MONITOR_REQUEST_TYPE(IntEnum):
    MonitorRequestTypeOff = 0x0
    MonitorRequestTypeOnAndPresent = 0x1
    MonitorRequestTypeToggleOn = 0x2

class hkBitFieldOps(ctypes.Structure):
    pass

class _DEP_SYSTEM_POLICY_TYPE(IntEnum):
    DEPPolicyAlwaysOff = 0x0
    DEPPolicyAlwaysOn = 0x1
    DEPPolicyOptIn = 0x2
    DEPPolicyOptOut = 0x3
    DEPTotalPolicyCount = 0x4

class hknpCollisionCacheCopyMode(ctypes.Structure):
    class Enum(IntEnum):
        COPY_JACOBIAN_CACHE = 0x0
        CLEAR_JACOBIAN_CACHE = 0x1


class eTkMemoryBackingPool(IntEnum):
    EMemoryBackingPool_TkGeneral = 0x0
    EMemoryBackingPool_SingleBuffer = 0x1
    EMemoryBackingPool_DoubleBuffer = 0x2
    EMemoryBackingPool_DoubleBufferPausable = 0x3
    EMemoryBackingPool_Audio = 0x4
    EMemoryBackingPool_Compute = 0x5
    EMemoryBackingPool_Debug = 0x6
    ENumTkMemoryBackingPools = 0x7

class hknpDeactivatedBodyCopyMode(ctypes.Structure):
    class Enum(IntEnum):
        COPY_COLLISION_CACHE = 0x0
        IGNORE_COLLISION_CACHE = 0x1


class _SC_STATUS_TYPE(IntEnum):
    SC_STATUS_PROCESS_INFO = 0x0

class TkAddNodesHandle(ctypes.Structure):
    class eAddNodesState(IntEnum):
        EAddNodesState_None = 0x0
        EAddNodesState_Searching = 0x1
        EAddNodesState_Adding = 0x2

    class eState(IntEnum):
        EState_Adding = 0x0
        EState_Preparing = 0x1
        EState_Complete = 0x2

    def Reset(): pass

    _fields_ = [
        ("mSceneGraphResource", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mParentNode", ctypes.c_uint32),
        ("mNode", ctypes.c_uint32),
        ("mLastAddedNode", ctypes.c_uint32),
        ("mpAddReferencedResourceHandle", ctypes.c_char * 0x10),  # std::shared_ptr<TkAddNodesHandle>
        ("mu64Timeout", ctypes.c_uint64),
        ("mpLastAddedTemplate", ctypes.c_ulonglong),
        ("mpCurrentAttachment", ctypes.c_ulonglong),
        ("miCount", ctypes.c_int32),
        ("_meAddNodesState", ctypes.c_uint32),
        ("_meState", ctypes.c_uint32),
    ]
    mSceneGraphResource: bytes
    mParentNode: int
    mNode: int
    mLastAddedNode: int
    mpAddReferencedResourceHandle: bytes
    mu64Timeout: int
    mpLastAddedTemplate: int
    mpCurrentAttachment: int
    miCount: int
    _meAddNodesState: int
    _meState: int

    @property
    def meAddNodesState(self):
        return TkAddNodesHandle.eAddNodesState(self._meAddNodesState)

    @property
    def meState(self):
        return TkAddNodesHandle.eState(self._meState)


class eThreadType(IntEnum):
    EThread_Render = 0x0
    EThread_Update = 0x1
    EThread_Sync = 0x2
    EThread_Num = 0x3

class eTempFileOpenMode(IntEnum):
    ETFOM_Read = 0x0
    ETFOM_Write = 0x1

class tagPOINTER_INPUT_TYPE(IntEnum):
    PT_POINTER = 0x1
    PT_TOUCH = 0x2
    PT_PEN = 0x3
    PT_MOUSE = 0x4
    PT_TOUCHPAD = 0x5

class DPI_HOSTING_BEHAVIOR(IntEnum):
    DPI_HOSTING_BEHAVIOR_DEFAULT = 0x0
    DPI_HOSTING_BEHAVIOR_MIXED = 0x1
    DPI_HOSTING_BEHAVIOR_INVALID = 0xFFFFFFFF

class DISPLAYCONFIG_DEVICE_INFO_TYPE(IntEnum):
    DISPLAYCONFIG_DEVICE_INFO_GET_SOURCE_NAME = 0x1
    DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_NAME = 0x2
    DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_PREFERRED_MODE = 0x3
    DISPLAYCONFIG_DEVICE_INFO_GET_ADAPTER_NAME = 0x4
    DISPLAYCONFIG_DEVICE_INFO_SET_TARGET_PERSISTENCE = 0x5
    DISPLAYCONFIG_DEVICE_INFO_GET_TARGET_BASE_TYPE = 0x6
    DISPLAYCONFIG_DEVICE_INFO_GET_SUPPORT_VIRTUAL_RESOLUTION = 0x7
    DISPLAYCONFIG_DEVICE_INFO_SET_SUPPORT_VIRTUAL_RESOLUTION = 0x8
    DISPLAYCONFIG_DEVICE_INFO_GET_ADVANCED_COLOR_INFO = 0x9
    DISPLAYCONFIG_DEVICE_INFO_SET_ADVANCED_COLOR_STATE = 0xA
    DISPLAYCONFIG_DEVICE_INFO_GET_SDR_WHITE_LEVEL = 0xB
    DISPLAYCONFIG_DEVICE_INFO_FORCE_UINT32 = 0xFFFFFFFF

class DPI_AWARENESS(IntEnum):
    DPI_AWARENESS_UNAWARE = 0x0
    DPI_AWARENESS_SYSTEM_AWARE = 0x1
    DPI_AWARENESS_PER_MONITOR_AWARE = 0x2
    DPI_AWARENESS_INVALID = 0xFFFFFFFF

class SYSNLS_FUNCTION(IntEnum):
    COMPARE_STRING = 0x1

class hknpMeshShapeCinfo(ctypes.Structure):
    class ShapeTagInitMode(IntEnum):
        NO_SHAPE_TAG = 0x0
        SHAPE_TAG_FROM_TRIANGLE_MATERIAL = 0x1

    class DebugFlags(IntEnum):
        NO_DEBUG_CHECKS = 0x0
        DEBUG_SKIP_FIRST_DEGENERACY_CHECK = 0x1

    _fields_ = [
        ("m_vertices", ctypes.c_char * 0x10),  # hkStridedArrayView<hkFloat3 const >
        ("m_triangles", ctypes.c_char * 0x10),  # hkStridedArrayView<hkUint3 const >
        ("m_shapeTags", ctypes.c_char * 0x10),  # hkStridedArrayView<unsigned short const >
        ("m_transform", ctypes.c_char * 0x40),  # hkMatrix4Impl<float>
        ("m_removeDegenerateTriangles", ctypes.c_ubyte),
        ("m_mergeCoplanarTriangles", ctypes.c_ubyte),
        ("m_mergeCoplanarTrianglesTolerance", ctypes.c_float),
        ("m_maxVertexError", ctypes.c_float),
        ("m_flagInteriorTriangles", ctypes.c_ubyte),
        ("m_storePrimitiveMapping", ctypes.c_ubyte),
        ("m_verbose", ctypes.c_ubyte),
        ("m_internalDebugFlags", ctypes.c_char * 0x1),  # hkFlags<enum hknpMeshShapeCinfo::DebugFlags,unsigned char>
    ]
    m_vertices: bytes
    m_triangles: bytes
    m_shapeTags: bytes
    m_transform: bytes
    m_removeDegenerateTriangles: bool
    m_mergeCoplanarTriangles: bool
    m_mergeCoplanarTrianglesTolerance: float
    m_maxVertexError: float
    m_flagInteriorTriangles: bool
    m_storePrimitiveMapping: bool
    m_verbose: bool
    m_internalDebugFlags: bytes

class _MANDATORY_LEVEL(IntEnum):
    MandatoryLevelUntrusted = 0x0
    MandatoryLevelLow = 0x1
    MandatoryLevelMedium = 0x2
    MandatoryLevelHigh = 0x3
    MandatoryLevelSystem = 0x4
    MandatoryLevelSecureProcess = 0x5
    MandatoryLevelCount = 0x6

class _MEMORY_RESOURCE_NOTIFICATION_TYPE(IntEnum):
    LowMemoryResourceNotification = 0x0
    HighMemoryResourceNotification = 0x1

class _SE_LEARNING_MODE_DATA_TYPE(IntEnum):
    SeLearningModeInvalidType = 0x0
    SeLearningModeSettings = 0x1
    SeLearningModeMax = 0x2

class hknpTraceCacheTypes(ctypes.Structure):
    class Enum(IntEnum):
        CACHE_EXAMINED = 0x0
        CACHE_CREATED = 0x1
        CACHE_LOD_CHANGED = 0x2
        CACHE_DELETED = 0x3
        CACHE_DEACTIVATED = 0x4
        CACHE_ACTIVATED = 0x5
        CACHE_DEFERRED_SINGLE = 0x6
        CACHE_DEFERRED_ALL = 0x7
        CACHE_TIM1 = 0x8
        CACHE_TIM3 = 0x9
        CACHE_COLLIDE = 0xA


class _PROCESS_MITIGATION_POLICY(IntEnum):
    ProcessDEPPolicy = 0x0
    ProcessASLRPolicy = 0x1
    ProcessDynamicCodePolicy = 0x2
    ProcessStrictHandleCheckPolicy = 0x3
    ProcessSystemCallDisablePolicy = 0x4
    ProcessMitigationOptionsMask = 0x5
    ProcessExtensionPointDisablePolicy = 0x6
    ProcessControlFlowGuardPolicy = 0x7
    ProcessSignaturePolicy = 0x8
    ProcessFontDisablePolicy = 0x9
    ProcessImageLoadPolicy = 0xA
    ProcessSystemCallFilterPolicy = 0xB
    ProcessPayloadRestrictionPolicy = 0xC
    ProcessChildProcessPolicy = 0xD
    ProcessSideChannelIsolationPolicy = 0xE
    ProcessUserShadowStackPolicy = 0xF
    MaxProcessMitigationPolicy = 0x10

class cTkMcQmcLFSRStore(ctypes.Structure):
    class SampleRange(IntEnum):
        P06_s64 = 0x0
        P08_s256 = 0x1
        P10_s1024 = 0x2

    def Construct(): pass

    def Destruct(): pass

    _fields_ = [
        ("mSamplers", ctypes.c_char * 0x18),  # std::array<cTkMcQmcLFSR *,3>
    ]
    mSamplers: bytes

class _COMPUTER_NAME_FORMAT(IntEnum):
    ComputerNameNetBIOS = 0x0
    ComputerNameDnsHostname = 0x1
    ComputerNameDnsDomain = 0x2
    ComputerNameDnsFullyQualified = 0x3
    ComputerNamePhysicalNetBIOS = 0x4
    ComputerNamePhysicalDnsHostname = 0x5
    ComputerNamePhysicalDnsDomain = 0x6
    ComputerNamePhysicalDnsFullyQualified = 0x7
    ComputerNameMax = 0x8

class hknpSimulation(ctypes.Structure):
    class CheckConsistencyFlags(IntEnum):
        CHECK_CACHES = 0x1
        CHECK_CACHE_CELL_INDEX = 0x2
        CHECK_FORCE_BODIES_IN_SIMULATION = 0x4
        CHECK_ALL = 0xFFFFFFFF

    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_solverData", ctypes.c_ulonglong),
        ("_m_type", ctypes.c_uint32),
    ]
    __vftable: int
    m_solverData: int
    _m_type: int

    @property
    def m_type(self):
        return hknpSimulationType.Enum(self._m_type)


class IMPORT_OBJECT_TYPE(IntEnum):
    IMPORT_OBJECT_CODE = 0x0
    IMPORT_OBJECT_DATA = 0x1
    IMPORT_OBJECT_CONST = 0x2

class ARM64_FNPDATA_FLAGS(IntEnum):
    PdataRefToFullXdata = 0x0
    PdataPackedUnwindFunction = 0x1
    PdataPackedUnwindFragment = 0x2

class _SC_EVENT_TYPE(IntEnum):
    SC_EVENT_DATABASE_CHANGE = 0x0
    SC_EVENT_PROPERTY_CHANGE = 0x1
    SC_EVENT_STATUS_CHANGE = 0x2

class cTkMatrix44(ctypes.Structure):
    class eInitFlag(IntEnum):
        EInitFlag_NoInit = 0x0

    def DeterminantPrecise(): pass

    def InvertedPrecise(): pass

    def Determinant(): pass

    def Inverted(): pass

    def Decompose(): pass

    def cTkMatrix44(): pass

    _fields_ = [
        ("___u0", ctypes.c_char * 0x40),  # $6414FEA29EB2BD4331652807C066E5EC
    ]
    ___u0: bytes

class IMAGE_AUX_SYMBOL_TYPE(IntEnum):
    IMAGE_AUX_SYMBOL_TYPE_TOKEN_DEF = 0x1

class hknpParticlesColliderFlags(ctypes.Structure):
    class Enum(IntEnum):
        RAISE_PARTICLE_BODY_IMPULSE_APPLIED_EVENTS = 0x1

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x4),  # hkFlags<void,int>
    ]
    baseclass_0: bytes

class _ACCESS_REASON_TYPE(IntEnum):
    AccessReasonNone = 0x0
    AccessReasonAllowedAce = 0x10000
    AccessReasonDeniedAce = 0x20000
    AccessReasonAllowedParentAce = 0x30000
    AccessReasonDeniedParentAce = 0x40000
    AccessReasonNotGrantedByCape = 0x50000
    AccessReasonNotGrantedByParentCape = 0x60000
    AccessReasonNotGrantedToAppContainer = 0x70000
    AccessReasonMissingPrivilege = 0x100000
    AccessReasonFromPrivilege = 0x200000
    AccessReasonIntegrityLevel = 0x300000
    AccessReasonOwnership = 0x400000
    AccessReasonNullDacl = 0x500000
    AccessReasonEmptyDacl = 0x600000
    AccessReasonNoSD = 0x700000
    AccessReasonNoGrant = 0x800000
    AccessReasonTrustLabel = 0x900000
    AccessReasonFilterAce = 0xA00000

class DIALOG_DPI_CHANGE_BEHAVIORS(IntEnum):
    DDC_DEFAULT = 0x0
    DDC_DISABLE_ALL = 0x1
    DDC_DISABLE_RESIZE = 0x2
    DDC_DISABLE_CONTROL_RELAYOUT = 0x4

class SYSGEOCLASS(IntEnum):
    GEOCLASS_ALL = 0x0
    GEOCLASS_REGION = 0xE
    GEOCLASS_NATION = 0x10

class _OS_DEPLOYEMENT_STATE_VALUES(IntEnum):
    OS_DEPLOYMENT_STANDARD = 0x1

class _HEAP_INFORMATION_CLASS(IntEnum):
    HeapCompatibilityInformation = 0x0
    HeapEnableTerminationOnCorruption = 0x1
    HeapOptimizeResources = 0x3

class _crt_app_type(IntEnum):
    _crt_unknown_app = 0x0
    _crt_console_app = 0x1
    _crt_gui_app = 0x2

class SERVICE_REGISTRY_STATE_TYPE(IntEnum):
    ServiceRegistryStateParameters = 0x0
    ServiceRegistryStatePersistent = 0x1
    MaxServiceRegistryStateType = 0x2

class JOBOBJECT_IO_ATTRIBUTION_CONTROL_FLAGS(IntEnum):
    JOBOBJECT_IO_ATTRIBUTION_CONTROL_ENABLE = 0x1
    JOBOBJECT_IO_ATTRIBUTION_CONTROL_DISABLE = 0x2
    JOBOBJECT_IO_ATTRIBUTION_CONTROL_VALID_FLAGS = 0x3

class hknpCommandDispatchType(ctypes.Structure):
    class Enum(IntEnum):
        POST_SOLVE = 0x0
        AS_SOON_AS_POSSIBLE = 0x1


class IMPORT_OBJECT_NAME_TYPE(IntEnum):
    IMPORT_OBJECT_ORDINAL = 0x0
    IMPORT_OBJECT_NAME = 0x1
    IMPORT_OBJECT_NAME_NO_PREFIX = 0x2
    IMPORT_OBJECT_NAME_UNDECORATE = 0x3
    IMPORT_OBJECT_NAME_EXPORTAS = 0x4

class ARM64_FNPDATA_CR(IntEnum):
    PdataCrUnchained = 0x0
    PdataCrUnchainedSavedLr = 0x1
    PdataCrChainedWithPac = 0x2
    PdataCrChained = 0x3

class _SC_ENUM_TYPE(IntEnum):
    SC_ENUM_PROCESS_INFO = 0x0

class WELL_KNOWN_SID_TYPE(IntEnum):
    WinNullSid = 0x0
    WinWorldSid = 0x1
    WinLocalSid = 0x2
    WinCreatorOwnerSid = 0x3
    WinCreatorGroupSid = 0x4
    WinCreatorOwnerServerSid = 0x5
    WinCreatorGroupServerSid = 0x6
    WinNtAuthoritySid = 0x7
    WinDialupSid = 0x8
    WinNetworkSid = 0x9
    WinBatchSid = 0xA
    WinInteractiveSid = 0xB
    WinServiceSid = 0xC
    WinAnonymousSid = 0xD
    WinProxySid = 0xE
    WinEnterpriseControllersSid = 0xF
    WinSelfSid = 0x10
    WinAuthenticatedUserSid = 0x11
    WinRestrictedCodeSid = 0x12
    WinTerminalServerSid = 0x13
    WinRemoteLogonIdSid = 0x14
    WinLogonIdsSid = 0x15
    WinLocalSystemSid = 0x16
    WinLocalServiceSid = 0x17
    WinNetworkServiceSid = 0x18
    WinBuiltinDomainSid = 0x19
    WinBuiltinAdministratorsSid = 0x1A
    WinBuiltinUsersSid = 0x1B
    WinBuiltinGuestsSid = 0x1C
    WinBuiltinPowerUsersSid = 0x1D
    WinBuiltinAccountOperatorsSid = 0x1E
    WinBuiltinSystemOperatorsSid = 0x1F
    WinBuiltinPrintOperatorsSid = 0x20
    WinBuiltinBackupOperatorsSid = 0x21
    WinBuiltinReplicatorSid = 0x22
    WinBuiltinPreWindows2000CompatibleAccessSid = 0x23
    WinBuiltinRemoteDesktopUsersSid = 0x24
    WinBuiltinNetworkConfigurationOperatorsSid = 0x25
    WinAccountAdministratorSid = 0x26
    WinAccountGuestSid = 0x27
    WinAccountKrbtgtSid = 0x28
    WinAccountDomainAdminsSid = 0x29
    WinAccountDomainUsersSid = 0x2A
    WinAccountDomainGuestsSid = 0x2B
    WinAccountComputersSid = 0x2C
    WinAccountControllersSid = 0x2D
    WinAccountCertAdminsSid = 0x2E
    WinAccountSchemaAdminsSid = 0x2F
    WinAccountEnterpriseAdminsSid = 0x30
    WinAccountPolicyAdminsSid = 0x31
    WinAccountRasAndIasServersSid = 0x32
    WinNTLMAuthenticationSid = 0x33
    WinDigestAuthenticationSid = 0x34
    WinSChannelAuthenticationSid = 0x35
    WinThisOrganizationSid = 0x36
    WinOtherOrganizationSid = 0x37
    WinBuiltinIncomingForestTrustBuildersSid = 0x38
    WinBuiltinPerfMonitoringUsersSid = 0x39
    WinBuiltinPerfLoggingUsersSid = 0x3A
    WinBuiltinAuthorizationAccessSid = 0x3B
    WinBuiltinTerminalServerLicenseServersSid = 0x3C
    WinBuiltinDCOMUsersSid = 0x3D
    WinBuiltinIUsersSid = 0x3E
    WinIUserSid = 0x3F
    WinBuiltinCryptoOperatorsSid = 0x40
    WinUntrustedLabelSid = 0x41
    WinLowLabelSid = 0x42
    WinMediumLabelSid = 0x43
    WinHighLabelSid = 0x44
    WinSystemLabelSid = 0x45
    WinWriteRestrictedCodeSid = 0x46
    WinCreatorOwnerRightsSid = 0x47
    WinCacheablePrincipalsGroupSid = 0x48
    WinNonCacheablePrincipalsGroupSid = 0x49
    WinEnterpriseReadonlyControllersSid = 0x4A
    WinAccountReadonlyControllersSid = 0x4B
    WinBuiltinEventLogReadersGroup = 0x4C
    WinNewEnterpriseReadonlyControllersSid = 0x4D
    WinBuiltinCertSvcDComAccessGroup = 0x4E
    WinMediumPlusLabelSid = 0x4F
    WinLocalLogonSid = 0x50
    WinConsoleLogonSid = 0x51
    WinThisOrganizationCertificateSid = 0x52
    WinApplicationPackageAuthoritySid = 0x53
    WinBuiltinAnyPackageSid = 0x54
    WinCapabilityInternetClientSid = 0x55
    WinCapabilityInternetClientServerSid = 0x56
    WinCapabilityPrivateNetworkClientServerSid = 0x57
    WinCapabilityPicturesLibrarySid = 0x58
    WinCapabilityVideosLibrarySid = 0x59
    WinCapabilityMusicLibrarySid = 0x5A
    WinCapabilityDocumentsLibrarySid = 0x5B
    WinCapabilitySharedUserCertificatesSid = 0x5C
    WinCapabilityEnterpriseAuthenticationSid = 0x5D
    WinCapabilityRemovableStorageSid = 0x5E
    WinBuiltinRDSRemoteAccessServersSid = 0x5F
    WinBuiltinRDSEndpointServersSid = 0x60
    WinBuiltinRDSManagementServersSid = 0x61
    WinUserModeDriversSid = 0x62
    WinBuiltinHyperVAdminsSid = 0x63
    WinAccountCloneableControllersSid = 0x64
    WinBuiltinAccessControlAssistanceOperatorsSid = 0x65
    WinBuiltinRemoteManagementUsersSid = 0x66
    WinAuthenticationAuthorityAssertedSid = 0x67
    WinAuthenticationServiceAssertedSid = 0x68
    WinLocalAccountSid = 0x69
    WinLocalAccountAndAdministratorSid = 0x6A
    WinAccountProtectedUsersSid = 0x6B
    WinCapabilityAppointmentsSid = 0x6C
    WinCapabilityContactsSid = 0x6D
    WinAccountDefaultSystemManagedSid = 0x6E
    WinBuiltinDefaultSystemManagedGroupSid = 0x6F
    WinBuiltinStorageReplicaAdminsSid = 0x70
    WinAccountKeyAdminsSid = 0x71
    WinAccountEnterpriseKeyAdminsSid = 0x72
    WinAuthenticationKeyTrustSid = 0x73
    WinAuthenticationKeyPropertyMFASid = 0x74
    WinAuthenticationKeyPropertyAttestationSid = 0x75
    WinAuthenticationFreshKeyAuthSid = 0x76
    WinBuiltinDeviceOwnersSid = 0x77

class eNGuiInputTextFlags(IntEnum):
    eInputTextFlags_CharsDecimal = 0x1

class _PRIORITY_HINT(IntEnum):
    IoPriorityHintVeryLow = 0x0
    IoPriorityHintLow = 0x1
    IoPriorityHintNormal = 0x2
    MaximumIoPriorityHintType = 0x3

class _crt_exit_return_mode(IntEnum):
    _crt_exit_terminate_process = 0x0
    _crt_exit_return_to_caller = 0x1

class _PROCESS_INFORMATION_CLASS(IntEnum):
    ProcessMemoryPriority = 0x0
    ProcessMemoryExhaustionInfo = 0x1
    ProcessAppMemoryInfo = 0x2
    ProcessInPrivateInfo = 0x3
    ProcessPowerThrottling = 0x4
    ProcessReservedValue1 = 0x5
    ProcessTelemetryCoverageInfo = 0x6
    ProcessProtectionLevelInfo = 0x7
    ProcessLeapSecondInfo = 0x8
    ProcessInformationClassMax = 0x9

class _COPYFILE2_MESSAGE_TYPE(IntEnum):
    COPYFILE2_CALLBACK_NONE = 0x0
    COPYFILE2_CALLBACK_CHUNK_STARTED = 0x1
    COPYFILE2_CALLBACK_CHUNK_FINISHED = 0x2
    COPYFILE2_CALLBACK_STREAM_STARTED = 0x3
    COPYFILE2_CALLBACK_STREAM_FINISHED = 0x4
    COPYFILE2_CALLBACK_POLL_CONTINUE = 0x5
    COPYFILE2_CALLBACK_ERROR = 0x6
    COPYFILE2_CALLBACK_MAX = 0x7

class _THREAD_INFORMATION_CLASS(IntEnum):
    ThreadMemoryPriority = 0x0
    ThreadAbsoluteCpuPriority = 0x1
    ThreadDynamicCodePolicy = 0x2
    ThreadPowerThrottling = 0x3
    ThreadInformationClassMax = 0x4

class cTkAsyncIOManager(ctypes.Structure):
    class EAsyncIOFlags(IntEnum):
        EASYNC_IO_IGNORE_FAILS = 0x1

    def CancelOp(): pass

    def ClearFileCache(): pass

    def Construct(): pass

    def DeleteOpHandle(): pass

    def DeleteOpHandles(): pass

    def Destruct(): pass

    def GetNumActiveOps(): pass

    def GetNumOpsCompleted(): pass

    def GetOpData(): pass

    def GetOpDataSize(): pass

    def GetOpPath(): pass

    def GetOpStatus(): pass

    def IsOpCancelled(): pass

    def LockOpCritical(): pass

    def Read(): pass

    def UnlockOpCritical(): pass


class eNGuiWindowEdges(IntEnum):
    EWindowEdge_Top = 0x0
    EWindowEdge_Bottom = 0x1
    EWindowEdge_Left = 0x2
    EWindowEdge_Right = 0x3

class hknpMotionManager(ctypes.Structure):
    def hknpMotionManager(): pass

    def tilde_hknpMotionManager(): pass

    def activateMotion(): pass

    def allocateMotion(): pass

    def buildCellIndexToSolverIdMap(): pass

    def buildSolverIdToMotionIdMap(): pass

    def deactivateMotion(): pass

    def freeMotion(): pass

    def initialize(): pass

    _fields_ = [
        ("m_bodyManager", ctypes.c_ulonglong),
        ("m_deactivationManager", ctypes.c_ulonglong),
        ("m_motions", ctypes.c_char * 0x30),  # hknpThreadSafeObjectPool<hknpMotion>
        ("m_activeMotionsPerCell", ctypes.c_char * 0x300),  # hkInplaceArray<hknpMotionId,1,hkBuiltinContainerAllocator<0,0> >
        ("m_activeMotionsPerCellCriticalSection", ctypes.c_char * 0x8),  # hkScopedPtr<hkCriticalSection>
    ]
    m_bodyManager: int
    m_deactivationManager: int
    m_motions: bytes
    m_activeMotionsPerCell: bytes
    m_activeMotionsPerCellCriticalSection: bytes

class hknpParticleMeshShapeType(IntEnum):
    SIMD_CMS = 0x0
    SIMD_CMS_WITH_EMBEDED_CONVEX = 0x1
    SIMD_EMS = 0x2
    SIMD_MESH = 0x3
    NO_SIMD_MESH = 0x4

class hknpCollision2DFastProcessUtil(ctypes.Structure):
    class BuildFaceManifoldStatus(IntEnum):
        STATUS_SUCCESS = 0x0
        STATUS_RERUN_GSK = 0x1
        STATUS_FORCE_SINGLE_POINT = 0x2
        STATUS_TOO_FAR = 0x3

    def buildFaceManifold(): pass

    def collideWithInteriorTriangle(): pass


class _crt_exit_cleanup_mode(IntEnum):
    _crt_exit_full_cleanup = 0x0
    _crt_exit_quick_cleanup = 0x1
    _crt_exit_no_cleanup = 0x2

class OFFER_PRIORITY(IntEnum):
    VmOfferPriorityVeryLow = 0x1
    VmOfferPriorityLow = 0x2
    VmOfferPriorityBelowNormal = 0x3
    VmOfferPriorityNormal = 0x4

class hknpConstraintPropertyKeys(ctypes.Structure):
    pass

class cTkStringAssembler(ctypes.Structure):
    def Appendf(): pass

    def cTkStringAssembler(): pass

    def tilde_cTkStringAssembler(): pass

    def AppendCharBufJsonEncoded(): pass

    def EnsureExtraCapacity(): pass

    _fields_ = [
        ("mpacStringBuffer", ctypes.c_char_p),
        ("miBufferSize", ctypes.c_int32),
        ("miWriteIndex", ctypes.c_int32),
        ("miSavedWriteIndex", ctypes.c_int32),
        ("miMemoryPool", ctypes.c_int32),
        ("mbOwnsBufferPtr", ctypes.c_ubyte),
    ]
    mpacStringBuffer: bytes
    miBufferSize: int
    miWriteIndex: int
    miSavedWriteIndex: int
    miMemoryPool: int
    mbOwnsBufferPtr: bool

class ELocalAllocatorSize(IntEnum):
    kLocalAllocator_5MB = 0x0
    kLocalAllocator_8MB = 0x1

class EFileSystemInitFlags(IntEnum):
    EFILESYSTEM_MOUNT_BANKS = 0x1

class SYSGEOTYPE(IntEnum):
    GEO_NATION = 0x1
    GEO_LATITUDE = 0x2
    GEO_LONGITUDE = 0x3
    GEO_ISO2 = 0x4
    GEO_ISO3 = 0x5
    GEO_RFC1766 = 0x6
    GEO_LCID = 0x7
    GEO_FRIENDLYNAME = 0x8
    GEO_OFFICIALNAME = 0x9
    GEO_TIMEZONES = 0xA
    GEO_OFFICIALLANGUAGES = 0xB
    GEO_ISO_UN_NUMBER = 0xC
    GEO_PARENT = 0xD
    GEO_DIALINGCODE = 0xE
    GEO_CURRENCYCODE = 0xF
    GEO_CURRENCYSYMBOL = 0x10
    GEO_NAME = 0x11
    GEO_ID = 0x12

class hknpParticleParticleCollisionsCacheHeader(ctypes.Structure):
    class FlagsEnum(IntEnum):
        CAN_EARLY_OUT = 0x1

    _fields_ = [
        ("m_numBelowContacts", ctypes.c_uint8),
        ("m_numBelowContactsOnEarlyOut", ctypes.c_uint8),
        ("m_numPenetrations", ctypes.c_uint8),
        ("m_numFramesInPenetration", ctypes.c_uint8),
        ("m_numFramesInEarlyOutState", ctypes.c_uint8),
        ("m_flags", ctypes.c_char * 0x1),  # hkFlags<enum hknpParticleParticleCollisionsCacheHeader::FlagsEnum,unsigned char>
    ]
    m_numBelowContacts: int
    m_numBelowContactsOnEarlyOut: int
    m_numPenetrations: int
    m_numFramesInPenetration: int
    m_numFramesInEarlyOutState: int
    m_flags: bytes

class hknpJacobianSchema(ctypes.Structure):
    class SchemaType(IntEnum):
        SCHEMA_TYPE_END = 0x0
        SCHEMA_TYPE_HEADER = 0x1
        SCHEMA_TYPE_GOTO = 0x2
        SCHEMA_TYPE_SHIFT_SOLVER_RESULTS_POINTER = 0x3
        SCHEMA_TYPE_1D_BILATERAL = 0x4
        SCHEMA_TYPE_1D_BILATERAL_WITH_IMPULSE_LIMIT = 0x5
        SCHEMA_TYPE_1D_BILATERAL_USER_TAU = 0x6
        SCHEMA_TYPE_1D_LINEAR_LIMITS = 0x7
        SCHEMA_TYPE_1D_FRICTION = 0x8
        SCHEMA_TYPE_1D_LINEAR_MOTOR = 0x9
        SCHEMA_TYPE_1D_PULLEY = 0xA
        SCHEMA_TYPE_1D_ANGULAR = 0xB
        SCHEMA_TYPE_1D_ANGULAR_USER_TAU = 0xC
        SCHEMA_TYPE_1D_ANGULAR_LIMITS = 0xD
        SCHEMA_TYPE_1D_ANGULAR_FRICTION = 0xE
        SCHEMA_TYPE_1D_ANGULAR_MOTOR = 0xF
        SCHEMA_TYPE_3D_ANGULAR_MOTOR = 0x10
        SCHEMA_TYPE_STABLE_BALLSOCKET = 0x11
        SCHEMA_TYPE_STABLE_STIFFSPRING = 0x12
        SCHEMA_TYPE_2D_FRICTION = 0x13
        SCHEMA_TYPE_3D_FRICTION = 0x14
        SCHEMA_TYPE_SET_MASS = 0x15
        SCHEMA_TYPE_SET_CENTER_OF_MASS = 0x16
        SCHEMA_TYPE_3D_ANGULAR = 0x17
        SCHEMA_TYPE_DEFORMABLE_LIN_3D = 0x18
        SCHEMA_TYPE_DEFORMABLE_ANG_3D = 0x19
        SCHEMA_TYPE_ELLIPTICAL_LIMIT = 0x1A
        SCHEMA_TYPE_TIMER_VOLUME_BEGIN = 0x1B
        SCHEMA_TYPE_TIMER_VOLUME_END = 0x1C
        SCHEMA_TYPE_WHEEL_FRICTION = 0x1D
        SCHEMA_TYPE_CONE_LIMIT = 0x1E
        SCHEMA_TYPE_SINGLE_CONSTRAINT_DIRECT_SOLVER = 0x1F
        SCHEMA_TYPE_STABLE_PULLEY = 0x20
        SCHEMA_TYPE_MAX = 0x21


class HasFlagsRecursive(IntEnum):
    NoCheckFields = 0x0
    CheckFields = 0x1

class CollideDynamicResult(IntEnum):
    MANIFOLD_CREATED = 0x0
    DONT_WELD = 0x1
    MANIFOLD_DEFERRED = 0x2

class hkGeomObbMode(IntEnum):
    HK_GEOM_OBB_MODE_INVALID = 0x0
    HK_GEOM_OBB_MODE_DEFAULT = 0x1
    HK_GEOM_OBB_MODE_BETTER_FIT_BUT_SLOWER = 0x2
    HK_GEOM_OBB_MODE_MAX_ID = 0x3

class hknpPenetrationRecovery(ctypes.Structure):
    class Enum(IntEnum):
        DISABLE = 0x0
        ENABLE = 0x1


class hknpParticleMeshCacheData(ctypes.Structure):
    _fields_ = [
        ("m_tims", ctypes.c_char * 0xA0),  # hknpParticleCompressedTimCache<32>
        ("m_shapeKeys", ctypes.c_char * 0x80),  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>
    ]
    m_tims: bytes
    m_shapeKeys: bytes

class hknpParticleSdfCacheData(ctypes.Structure):
    _fields_ = [
        ("m_lastSolvedNormal", common.Vector4f),
        ("m_normals", common.Vector4f * 0x4),  # hkVector4f[4]
        ("m_distances", ctypes.c_float * 0x4),  # float[4]
    ]
    m_lastSolvedNormal: common.Vector4f
    m_normals: list[common.Vector4f]
    m_distances: list[ctypes.c_float]

class _WINHTTP_REQUEST_TIME_ENTRY(IntEnum):
    WinHttpProxyDetectionStart = 0x0
    WinHttpProxyDetectionEnd = 0x1
    WinHttpConnectionAcquireStart = 0x2
    WinHttpConnectionAcquireWaitEnd = 0x3
    WinHttpConnectionAcquireEnd = 0x4
    WinHttpNameResolutionStart = 0x5
    WinHttpNameResolutionEnd = 0x6
    WinHttpConnectionEstablishmentStart = 0x7
    WinHttpConnectionEstablishmentEnd = 0x8
    WinHttpTlsHandshakeClientLeg1Start = 0x9
    WinHttpTlsHandshakeClientLeg1End = 0xA
    WinHttpTlsHandshakeClientLeg2Start = 0xB
    WinHttpTlsHandshakeClientLeg2End = 0xC
    WinHttpTlsHandshakeClientLeg3Start = 0xD
    WinHttpTlsHandshakeClientLeg3End = 0xE
    WinHttpStreamWaitStart = 0xF
    WinHttpStreamWaitEnd = 0x10
    WinHttpSendRequestStart = 0x11
    WinHttpSendRequestHeadersCompressionStart = 0x12
    WinHttpSendRequestHeadersCompressionEnd = 0x13
    WinHttpSendRequestHeadersEnd = 0x14
    WinHttpSendRequestEnd = 0x15
    WinHttpReceiveResponseStart = 0x16
    WinHttpReceiveResponseHeadersDecompressionStart = 0x17
    WinHttpReceiveResponseHeadersDecompressionEnd = 0x18
    WinHttpReceiveResponseHeadersEnd = 0x19
    WinHttpReceiveResponseBodyDecompressionDelta = 0x1A
    WinHttpReceiveResponseEnd = 0x1B
    WinHttpProxyTunnelStart = 0x1C
    WinHttpProxyTunnelEnd = 0x1D
    WinHttpProxyTlsHandshakeClientLeg1Start = 0x1E
    WinHttpProxyTlsHandshakeClientLeg1End = 0x1F
    WinHttpProxyTlsHandshakeClientLeg2Start = 0x20
    WinHttpProxyTlsHandshakeClientLeg2End = 0x21
    WinHttpProxyTlsHandshakeClientLeg3Start = 0x22
    WinHttpProxyTlsHandshakeClientLeg3End = 0x23
    WinHttpRequestTimeLast = 0x24
    WinHttpRequestTimeMax = 0x40

class _WINHTTP_REQUEST_STAT_ENTRY(IntEnum):
    WinHttpConnectFailureCount = 0x0
    WinHttpProxyFailureCount = 0x1
    WinHttpTlsHandshakeClientLeg1Size = 0x2
    WinHttpTlsHandshakeServerLeg1Size = 0x3
    WinHttpTlsHandshakeClientLeg2Size = 0x4
    WinHttpTlsHandshakeServerLeg2Size = 0x5
    WinHttpRequestHeadersSize = 0x6
    WinHttpRequestHeadersCompressedSize = 0x7
    WinHttpResponseHeadersSize = 0x8
    WinHttpResponseHeadersCompressedSize = 0x9
    WinHttpResponseBodySize = 0xA
    WinHttpResponseBodyCompressedSize = 0xB
    WinHttpProxyTlsHandshakeClientLeg1Size = 0xC
    WinHttpProxyTlsHandshakeServerLeg1Size = 0xD
    WinHttpProxyTlsHandshakeClientLeg2Size = 0xE
    WinHttpProxyTlsHandshakeServerLeg2Size = 0xF
    WinHttpRequestStatLast = 0x10
    WinHttpRequestStatMax = 0x20

class hknpCharacterRigidBodyProperty(ctypes.Structure):
    _fields_ = [
        ("m_characterRB", ctypes.c_ulonglong),
    ]
    m_characterRB: int

class __std_fs_file_attr(IntEnum):
    _Readonly = 0x1
    _Hidden = 0x2
    _System = 0x4
    _Directory = 0x10
    _Archive = 0x20
    _Device = 0x40
    _Normal = 0x80
    _Temporary = 0x100
    _Sparse_file = 0x200
    _Reparse_point = 0x400
    _Invalid = 0xFFFFFFFF

class __std_fs_reparse_tag(IntEnum):
    _None = 0x0
    _Mount_point = 0xA0000003
    _Symlink = 0xA000000C

class __std_access_rights(IntEnum):
    _File_read_attributes = 0x80
    _File_write_attributes = 0x100
    _Delete = 0x10000
    _File_generic_write = 0x120116

class __std_win_error(IntEnum):
    _Success = 0x0
    _Invalid_function = 0x1
    _File_not_found = 0x2
    _Path_not_found = 0x3
    _Access_denied = 0x5
    _Not_enough_memory = 0x8
    _No_more_files = 0x12
    _Sharing_violation = 0x20
    _Not_supported = 0x32
    _Error_bad_netpath = 0x35
    _File_exists = 0x50
    _Invalid_parameter = 0x57
    _Insufficient_buffer = 0x7A
    _Invalid_name = 0x7B
    _Directory_not_empty = 0x91
    _Already_exists = 0xB7
    _Filename_exceeds_range = 0xCE
    _Directory_name_is_invalid = 0x10B
    _Reparse_tag_invalid = 0x1129
    _Max = 0xFFFFFFFF

class __std_code_page(IntEnum):
    _Acp = 0x0
    _Utf8 = 0xFDE9

class hknpMotionWeldingUtil(ctypes.Structure):
    class MotionWeldSetupResult(IntEnum):
        SUCCESS = 0x0
        PENETRATING = 0x1
        VELOCITY_TOO_LOW = 0x2
        FURTHER_THAN_CAST_DISTANCE = 0x3

    def motionWeldCvxVsSdf(): pass

    def setupMotionWeldConfigBodyA(): pass

    def setupMotionWeldConfigBodyB(): pass


class ENodeEval(IntEnum):
    eDiscard = 0x0
    eNode = 0x1
    eLeaf = 0x2

class eFrontendPage(IntEnum):
    EFrontendPage_Suit = 0x0
    EFrontendPage_Ship = 0x1
    EFrontendPage_Vehicle = 0x2
    EFrontendPage_Freighter = 0x3
    EFrontendPage_Weapon = 0x4
    EFrontendPage_Discovery = 0x5
    EFrontendPage_Journey = 0x6
    EFrontendPage_MissionLog = 0x7
    EFrontendPage_Wiki = 0x8
    EFrontendPage_Catalogue = 0x9
    EFrontendPage_InfoPortal = 0xA
    EFrontendPage_Season = 0xB
    EFrontendPage_Options = 0xC
    EFrontendPage_Switcher = 0xD
    EFrontendPage_Controls = 0xE
    EFrontendPage_Network = 0xF
    EFrontendPage_NetworkPlayers = 0x10
    EFrontendPage_NetworkManageFriends = 0x11
    EFrontendPage_NetworkManageBlocked = 0x12
    EFrontendPage_Difficulty = 0x13
    EFrontendPage_Credits = 0x14
    EFrontendPage_Redeem = 0x15
    EFrontendPage_Interact = 0x16
    EFrontendPage_InteractDialog = 0x17
    EFrontendPage_InteractConsole = 0x18
    EFrontendPage_InteractShip = 0x19
    EFrontendPage_Trade = 0x1A
    EFrontendPage_TechTrade = 0x1B
    EFrontendPage_BuildingTrade = 0x1C
    EFrontendPage_SpecialsTrade = 0x1D
    EFrontendPage_MissionList = 0x1E
    EFrontendPage_MissionHandInList = 0x1F
    EFrontendPage_MissionRenounce = 0x20
    EFrontendPage_MissionDescription = 0x21
    EFrontendPage_BuyScreen = 0x22
    EFrontendPage_CompareScreen = 0x23
    EFrontendPage_DisplayTech = 0x24
    EFrontendPage_DisplayProduct = 0x25
    EFrontendPage_DisplayPatchNotes = 0x26
    EFrontendPage_FreighterTransferScreen = 0x27
    EFrontendPage_InventoryTransferScreen = 0x28
    EFrontendPage_Message = 0x29
    EFrontendPage_PhotoMode = 0x2A
    EFrontendPage_ReportBase = 0x2B
    EFrontendPage_PhotoBaseForUpload = 0x2C
    EFrontendPage_BaseUpload = 0x2D
    EFrontendPage_BasePartsMenu = 0x2E
    EFrontendPage_BasePartPalette = 0x2F
    EFrontendPage_Popup = 0x30
    EFrontendPage_Maintenance = 0x31
    EFrontendPage_Portal = 0x32
    EFrontendPage_PortalRunes = 0x33
    EFrontendPage_PortalActivate = 0x34
    EFrontendPage_PortalUaDisplay = 0x35
    EFrontendPage_Refiner = 0x36
    EFrontendPage_SystemHoover = 0x37
    EFrontendPage_EggMachine = 0x38
    EFrontendPage_VehicleRace = 0x39
    EFrontendPage_ManageFleet = 0x3A
    EFrontendPage_ManageExpeditions = 0x3B
    EFrontendPage_ExpeditionDebrief = 0x3C
    EFrontendPage_FrigateDetails = 0x3D
    EFrontendPage_FrigateCaptain = 0x3E
    EFrontendPage_ExpeditionDetails = 0x3F
    EFrontendPage_ExpeditionSelection = 0x40
    EFrontendPage_ExpeditionOutfitting = 0x41
    EFrontendPage_Customisation = 0x42
    EFrontendPage_Teleporter = 0x43
    EFrontendPage_Teleporter_Nexus = 0x44
    EFrontendPage_ByteBeat = 0x45
    EFrontendPage_BaseGridPart = 0x46
    EFrontendPage_UnlockItemTree = 0x47
    EFrontendPage_CreatureFeeder = 0x48
    EFrontendPage_CreatureHarvester = 0x49
    EFrontendPage_Multiplayer_MissionList = 0x4A
    EFrontendPage_Multiplayer_MissionDescription = 0x4B
    EFrontendPage_CraftingTree = 0x4C
    EFrontendPage_ByteBeatSwitch = 0x4D
    EFrontendPage_RadialInteraction = 0x4E
    EFrontendPage_Pet = 0x4F
    EFrontendPage_IntermediateInteraction = 0x50
    EFrontendPage_ByteBeatLibrary = 0x51
    EFrontendPage_SettlementHub = 0x52
    EFrontendPage_SettlementJudgement = 0x53
    EFrontendPage_RocketLockerInventory = 0x54
    EFrontendPage_SquadronRecruitment = 0x55
    EFrontendPage_SquadronManagement = 0x56
    EFrontendPage_SquadronPilotDetails = 0x57
    EFrontendPage_NumPages = 0x58
    EFrontendPage_None = 0xFFFFFFFF

class eGcJobQueue(IntEnum):
    ELastTkJobQueue = 0xD
    EJobQueue_RegionGenerate = 0xE
    EJobQueue_RegionGenerateResume = 0xF
    EJobQueue_RegionPolygonise = 0x10
    EJobQueue_RegionUpdate = 0x11
    EJobQueue_ClothSimulation = 0x12
    EJobQueue_Pathfinding = 0x13
    EJobQueue_PathFloodFill = 0x14
    EJobQueue_Galaxy = 0x15
    ENumTotalJobQueues = 0x16

class eIcon(IntEnum):
    EIcon_Scanner = 0x0
    EIcon_Scanner_Small = 0x1
    EIcon_Binoculars = 0x2
    EIcon_NumTypes = 0x3

class eTkTimeMode(IntEnum):
    ETimeLocal = 0x0
    ETimeGlobal = 0x1

class cTkStringMap(ctypes.Structure):
    _fields_ = [
        ("mpaStringData", ctypes.c_ulonglong),
        ("mnDataVersion", ctypes.c_int32),
        ("miNumStrings", ctypes.c_int32),
    ]
    mpaStringData: int
    mnDataVersion: int
    miNumStrings: int

class eShaderConstType(IntEnum):
    EShaderConst_InverseModelMat4 = 0x0
    EShaderConst_WorldMat4 = 0x1
    EShaderConst_WorldMotionMat4 = 0x2
    EShaderConst_WorldViewProjectionMat4 = 0x3
    EShaderConst_WorldNormalMat4 = 0x4
    EShaderConst_ViewMat4 = 0x5
    EShaderConst_ProjectionMat4 = 0x6
    EShaderConst_ViewProjectionMat4 = 0x7
    EShaderConst_InverseViewProjectionMat4 = 0x8
    EShaderConst_InverseViewProjectionSCMat4 = 0x9
    EShaderConst_InverseViewMat4 = 0xA
    EShaderConst_InverseProjectionMat4 = 0xB
    EShaderConst_ViewPositionVec3 = 0xC
    EShaderConst_LightDirectionVec4 = 0xD
    EShaderConst_LightPositionVec4 = 0xE
    EShaderConst_LightColourVec4 = 0xF
    EShaderConst_LightCustomParamsVec4 = 0x10
    EShaderConst_CameraMat4 = 0x11
    EShaderConst_CameraNoHeadTrackingMat4 = 0x12
    EShaderConst_PrevViewProjectionMat4 = 0x13
    EShaderConst_ThisToPrevViewProjectionMat4 = 0x14
    EShaderConst_MBlurSettingsVec4 = 0x15
    EShaderConst_TaaSettingsVec4 = 0x16
    EShaderConst_TessSettingsVec4 = 0x17
    EShaderConst_ShellsSettingsVec4 = 0x18
    EShaderConst_DeJitterVec3 = 0x19
    EShaderConst_FrameIndex = 0x1A
    EShaderConst_FrameBufferSizeVec4 = 0x1B
    EShaderConst_FoVValuesVec4 = 0x1C
    EShaderConst_Time = 0x1D
    EShaderConst_PrevTime = 0x1E
    EShaderConst_FadeValue = 0x1F
    EShaderConst_ShaderContextVariant = 0x20
    EShaderConst_LodIndex = 0x21
    EShaderConst_ShadowMat4 = 0x22
    EShaderConst_ShadowBias = 0x23
    EShaderConst_SkinMatrixRowsVec4 = 0x24
    EShaderConst_PrevSkinMatrixRowsVec4 = 0x25
    EShaderConst_ParticlePivotOffsetVec2 = 0x26
    EShaderConst_ParticleSoftFadeStrength = 0x27
    EShaderConst_ParticleSettingBits = 0x28
    EShaderConst_ParticleCornersVec4 = 0x29
    EShaderConst_ParticlePositionsVec4 = 0x2A
    EShaderConst_ParticleSizeAndRotationsVec4 = 0x2B
    EShaderConst_ParticleNormalsVec4 = 0x2C
    EShaderConst_ParticleColoursVec4 = 0x2D
    EShaderConst_DoFParamsVec4 = 0x2E
    EShaderConst_ClipPlanesVec4 = 0x2F
    EShaderConst_FrustumDimVec4 = 0x30
    EShaderConst_SpotlightPositionVec4 = 0x31
    EShaderConst_SpotlightDirectionVec4 = 0x32
    EShaderConst_SpotlightUpVec4 = 0x33
    EShaderConst_GenericParam0Vec4 = 0x34
    EShaderConst_GenericParam1Vec4 = 0x35
    EShaderConst_ShadowMapSizeVec4 = 0x36
    EShaderConst_ShadowFadeParamVec4 = 0x37
    EShaderConst_ShadowProjScaleVec3 = 0x38
    EShaderConst_BboxDepthAndClips = 0x39
    EShaderConst_UserData = 0x3A
    EShaderConst_InverseProjectionSCMat4 = 0x3B
    EShaderConst_InverseWorldUpMatVec4 = 0x3C
    EShaderConst_SparseTextureStatusVec4 = 0x3D
    EShaderConst_TextureCoordsAdjustVec4 = 0x3E
    EShaderConst_TextureCoordsBorderVec4 = 0x3F
    EShaderConst_OutTexture0 = 0x40
    EShaderConst_OutTexture1 = 0x41
    EShaderConst_OutTexture2 = 0x42
    EShaderConst_OutTexture3 = 0x43
    EShaderConst_OutTexture4 = 0x44
    EShaderConst_OutTextureDepth = 0x45
    EShaderConst_OutTextureOffsetSize = 0x46
    EShaderConst_VREyeInfoVec3 = 0x47
    EShaderConst_FrustumTanFovVec4 = 0x48
    EShaderConst_SpotlightDirectionMultiVec4 = 0x49
    EShaderConst_SpotlightPositionMultiVec4 = 0x4A
    EShaderConst_SpotlightUpMultiVec4 = 0x4B
    EShaderConst_LightColourMultiVec4 = 0x4C
    EShaderConst_DisableAmbientAllowed = 0x4D
    EShaderConst_TextureCacheBlockColourVec4 = 0x4E
    EShaderConst_ParallaxMapDataVec4 = 0x4F
    EShaderConst_RefractionParamsVec4 = 0x50
    EShaderConst_ProjectionNoJitterMat4 = 0x51
    EShaderConst_ViewProjectionNoJitterMat4 = 0x52
    EShaderConst_InverseViewProjectionNoJitterMat4 = 0x53
    EShaderConst_PrevViewProjectionNoJitterMat4 = 0x54
    EShaderConst_PrevInverseViewProjectionNoJitterMat4 = 0x55
    EShaderConst_PrevInverseViewProjectionSCMat4 = 0x56
    EShaderConst_PrevViewPositionVec3 = 0x57
    EShaderConst_AabbMinVec4 = 0x58
    EShaderConst_AabbMaxVec4 = 0x59
    EShaderConst_RuntimeLaserParamsVec4 = 0x5A
    EShaderConst_PerInstanceData = 0x5B
    EShaderConst_PerInstanceCull = 0x5C
    EShaderConst_SrcInstanceData = 0x5D
    EShaderConst_DstInstanceData = 0x5E
    EShaderConst_FlgInstanceData = 0x5F
    EShaderConst_IndirectCmdData = 0x60
    EShaderConst_IndirectIndices = 0x61
    EShaderConst_InstanceBaseIndex = 0x62
    EShaderConst_InstanceCount = 0x63
    EShaderConst_IndirectIndex = 0x64
    EShaderConst_AntiAliasingIndex = 0x65
    EShaderConst_Num = 0x66

class EComputeShaders(IntEnum):
    eComputeShader_COMPUTENOISE_COMP_TGUNPACKVERTS = 0x0
    eComputeShader_COMPUTENOISE_COMP_TGTRICOUNT = 0x1
    eComputeShader_COMPUTENOISE_COMP_TGINDIRECTARGS = 0x2
    eComputeShader_COMPUTENOISE_COMP_TGTRIPOSITIONS = 0x3
    eComputeShader_COMPUTENOISE_COMP_TGTYPESPERPOS = 0x4
    eComputeShader_COMPUTENOISE_COMP_TGFILTEROBJECTS = 0x5
    eComputeShader_COMPUTENOISE_COMP_TERRAINGEN_SMOO = 0x6
    eComputeShader_COMPUTENOISE_COMP_TGTURBULENCE = 0x7
    eComputeShader_COMPUTENOISE_COMP_TERRAINGEN_UBER = 0x8
    eComputeShader_COMPUTENOISE_COMP_TGVORONOIPOS = 0x9
    eComputeShader_COMPUTENOISE_COMP_TGVORONOISTREAM = 0xA
    eComputeShader_COMPUTENOISE_COMP_TGLARGEREGION = 0xB
    eComputeShader_COMPUTENOISE_COMP_TGEN_REGIONNOIS = 0xC
    eComputeShader_COMPUTENOISE_COMP_TGUBERCOMBINE = 0xD
    eComputeShader_GENMIPMAP_COMP_COMPRESSBC7 = 0xE
    eComputeShader_GENMIPMAP_COMP_COMPRESSBC5 = 0xF
    eComputeShader_GENMIPMAP_COMP_COMPRESSMIP = 0x10
    eComputeShader_GENMIPMAP_COMP_COMPRESSBC1 = 0x11
    eComputeShader_GENMIPMAP_COMP_COMPRESSBC7_TER = 0x12
    eComputeShader_POSTPROCESS_COMP_CONVERTHDR = 0x13
    eComputeShader_POSTPROCESS_COMP_DOWNSAMPLE_FLPY = 0x14
    eComputeShader_POSTPROCESS_COMP_COPY = 0x15
    eComputeShader_NumTypes = 0x16

class EgResourceFlags(ctypes.Structure):
    class Flags(IntEnum):
        NoQuery = 0x1


class eLanguageRegion(IntEnum):
    LR_English = 0x0
    LR_USEnglish = 0x1
    LR_French = 0x2
    LR_Italian = 0x3
    LR_German = 0x4
    LR_Spanish = 0x5
    LR_Russian = 0x6
    LR_Polish = 0x7
    LR_Dutch = 0x8
    LR_Portuguese = 0x9
    LR_LatinAmericanSpanish = 0xA
    LR_BrazilianPortuguese = 0xB
    LR_Japanese = 0xC
    LR_TraditionalChinese = 0xD
    LR_SimplifiedChinese = 0xE
    LR_TencentChinese = 0xF
    LR_Korean = 0x10
    LR_Size = 0x11

class eBehaviourTreeState(IntEnum):
    EBehaviourTree_Ready = 0x0
    EBehaviourTree_Running = 0x1
    EBehaviourTree_Success = 0x2
    EBehaviourTree_Failure = 0x3
    EBehaviourTree_Num = 0x4

class eBehaviourTreeNodeType(IntEnum):
    EBehaviourTreeNodeType_SequenceSelector = 0x0
    EBehaviourTreeNodeType_PrioritySelector = 0x1
    EBehaviourTreeNodeType_ConcurrentSelector = 0x2
    EBehaviourTreeNodeType_StochasticSelector = 0x3
    EBehaviourTreeNodeType_InverseDecorator = 0x4
    EBehaviourTreeNodeType_SwitchDecorator = 0x5
    EBehaviourTreeNodeType_SelectionDecorator = 0x6
    EBehaviourTreeNodeType_SucceedOnSwitchDecorator = 0x7
    EBehaviourTreeNodeType_SucceedOnSelectionDecorator = 0x8
    EBehaviourTreeNodeType_AlwaysSucceedDecorator = 0x9
    EBehaviourTreeNodeType_AlwaysRunDecorator = 0xA
    EBehaviourTreeNodeType_NeverFailDecorator = 0xB
    EBehaviourTreeNodeType_LeafNode = 0xC
    EBehaviourTreeNodeType_NumTypes = 0xD

class cTkPageAllocator(ctypes.Structure):
    _fields_ = [
        ("mPagesAvailable", ctypes.c_char * 0x8000),  # cTkBitArray<unsigned __int64,1,262144>
        ("miPagesAvailableInChunk", ctypes.c_uint32 * 0x200),  # unsigned int[512]
        ("miNumChunks", ctypes.c_uint32),
        ("miTotalPages", ctypes.c_uint32),
        ("miTotalPagesAvailable", ctypes.c_uint32),
        ("mOutOfMemoryCallback", ctypes.c_ulonglong),
    ]
    mPagesAvailable: bytes
    miPagesAvailableInChunk: list[ctypes.c_uint32]
    miNumChunks: int
    miTotalPages: int
    miTotalPagesAvailable: int
    mOutOfMemoryCallback: int

class EgCamera(ctypes.Structure):
    class List(IntEnum):
        PipeResI = 0x258
        OutTexResI = 0x259
        OutBufIndexI = 0x25A
        OutCubeFaceI = 0x25B
        LeftPlaneF = 0x25C
        RightPlaneF = 0x25D
        BottomPlaneF = 0x25E
        TopPlaneF = 0x25F
        NearPlaneF = 0x260
        FarPlaneF = 0x261
        OrthoI = 0x262
        OccCullingI = 0x263
        MipLevelI = 0x264
        ShadowCascadeMaskI = 0x265
        ImposterNumViewsI = 0x266
        ImposterClipDistanceF = 0x267
        ImposterWidthF = 0x268
        ImposterHeightF = 0x269


class ExtensionsUID(ctypes.Structure):
    class List(IntEnum):
        BaseUID_Project_Example = 0x14
        BaseUID_Project_Particle = 0x28
        BaseUID_Project_HeavyAir = 0x3C
        BaseUID_Project_Instanced = 0x50
        BaseUID_Project_Galactic = 0x64
        BaseUID_Project_Terrain = 0x78
        BaseUID_Project_MagicModels = 0x8C
        BaseUID_Project_LineRenderer = 0xA0


class eDebugEditorMode(IntEnum):
    EDebugEditor_Off = 0x0
    EDebugEditor_Edit = 0x1
    EDebugEditor_Play = 0x2

class VertexNodeParams(ctypes.Structure):
    class List(IntEnum):
        MatResI = 0x2710
        FadeF = 0x2711
        XFacingVertStart = 0x2712
        YFacingVertStart = 0x2713
        ZFacingVertStart = 0x2714
        SparseNearDiffTexture = 0x2715
        SparseNearNormTexture = 0x2716
        SparseFarDiffTexture = 0x2717
        SparseFarNormTexture = 0x2718
        StatusMapNearTexture = 0x2719
        StatusMapFarTexture = 0x271A
        MipFeedbackTexture = 0x271B
        MipFeedbackNearIndex = 0x271C
        MipFeedbackFarIndex = 0x271D
        NearMipsPresent = 0x271E
        FarMipsPresent = 0x271F
        TileBlendVertStart = 0x2720


class eTkJobQueue(IntEnum):
    EJobQueue_System = 0x0
    EJobQueue_Physics = 0x1
    EJobQueue_SceneNode = 0x2
    EJobQueue_GalaxyMap = 0x3
    EJobQueue_Culling = 0x4
    EJobQueue_HiCulling = 0x5
    EJobQueue_PostCulling = 0x6
    EJobQueue_BackgroundTasks = 0x7
    EJobQueue_RenderThreadRequest = 0x8
    EJobQueue_UpdateThreadRequest = 0x9
    EJobQueue_RenderThreadRequestStaggered = 0xA
    EJobQueue_UpdateThreadRequestStaggered = 0xB
    EJobQueue_Graphics = 0xC
    EJobQueue_HMD = 0xD
    ENumTkJobQueues = 0xE

class EgRenderCullMode(ctypes.Structure):
    class Mode(IntEnum):
        UpdateEverything_SyncCull = 0x0
        UpdateEverything_AsyncCull = 0x1
        UpdateEverything_SingleThreadCull = 0x2
        NoUpdates_SyncCull = 0x3
        NoUpdates_AsyncCull = 0x4
        NoUpdates_SingleThreadCull = 0x5
        UpdateNodesNotFlags_SyncCull = 0x6
        UpdateNodesNotFlags_AsyncCull = 0x7
        UpdateNodesNotFlags_SingleThreadCull = 0x8
        PassDoesNotUseNodeGraph = 0x9
        UpdateEverything_DrawAllEnabled = 0xA
        NoUpdates_DrawAllEnabled = 0xB
        UpdateNodesNotFlags_DrawAllEnabled = 0xC


class VertexStreamElements(ctypes.Structure):
    class List(IntEnum):
        VertexStreamNormal = 0x0
        VertexStreamOverflow = 0x1
        TextureParamsStream = 0x2


class eTkJobGroup(IntEnum):
    EJobGroup_ThreadRequest = 0x0
    EJobGroup_Background = 0x1
    EJobGroup_TimeCritical = 0x2
    EJobGroup_Graphics = 0x3
    ENumTkJobGroups = 0x4

class eVoxelType(IntEnum):
    EVoxel_Air = 0x0
    EVoxel_Base = 0x1
    EVoxel_Rock = 0x2
    EVoxel_Mountain = 0x3
    EVoxel_Underwater = 0x4
    EVoxel_Cave = 0x5
    EVoxel_Dirt = 0x6
    EVoxel_Liquid = 0x7
    EVoxel_Substance_Res1 = 0x8
    EVoxel_Substance_Res2 = 0x9
    EVoxel_Substance_Res3 = 0xA
    EVoxel_Num = 0xB

class VkStructureType(IntEnum):
    VK_STRUCTURE_TYPE_APPLICATION_INFO = 0x0
    VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO = 0x1
    VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO = 0x2
    VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO = 0x3
    VK_STRUCTURE_TYPE_SUBMIT_INFO = 0x4
    VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO = 0x5
    VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE = 0x6
    VK_STRUCTURE_TYPE_BIND_SPARSE_INFO = 0x7
    VK_STRUCTURE_TYPE_FENCE_CREATE_INFO = 0x8
    VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO = 0x9
    VK_STRUCTURE_TYPE_EVENT_CREATE_INFO = 0xA
    VK_STRUCTURE_TYPE_QUERY_POOL_CREATE_INFO = 0xB
    VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO = 0xC
    VK_STRUCTURE_TYPE_BUFFER_VIEW_CREATE_INFO = 0xD
    VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO = 0xE
    VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO = 0xF
    VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO = 0x10
    VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO = 0x11
    VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO = 0x12
    VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO = 0x13
    VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO = 0x14
    VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_STATE_CREATE_INFO = 0x15
    VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO = 0x16
    VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO = 0x17
    VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO = 0x18
    VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO = 0x19
    VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO = 0x1A
    VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO = 0x1B
    VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO = 0x1C
    VK_STRUCTURE_TYPE_COMPUTE_PIPELINE_CREATE_INFO = 0x1D
    VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO = 0x1E
    VK_STRUCTURE_TYPE_SAMPLER_CREATE_INFO = 0x1F
    VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO = 0x20
    VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_CREATE_INFO = 0x21
    VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO = 0x22
    VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET = 0x23
    VK_STRUCTURE_TYPE_COPY_DESCRIPTOR_SET = 0x24
    VK_STRUCTURE_TYPE_FRAMEBUFFER_CREATE_INFO = 0x25
    VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO = 0x26
    VK_STRUCTURE_TYPE_COMMAND_POOL_CREATE_INFO = 0x27
    VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO = 0x28
    VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_INFO = 0x29
    VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO = 0x2A
    VK_STRUCTURE_TYPE_RENDER_PASS_BEGIN_INFO = 0x2B
    VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER = 0x2C
    VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER = 0x2D
    VK_STRUCTURE_TYPE_MEMORY_BARRIER = 0x2E
    VK_STRUCTURE_TYPE_LOADER_INSTANCE_CREATE_INFO = 0x2F
    VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO = 0x30
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_FEATURES = 0x31
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_1_PROPERTIES = 0x32
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_FEATURES = 0x33
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_2_PROPERTIES = 0x34
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_FEATURES = 0x35
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_1_3_PROPERTIES = 0x36
    VK_STRUCTURE_TYPE_SWAPCHAIN_CREATE_INFO_KHR = 0x3B9ACDE8
    VK_STRUCTURE_TYPE_PRESENT_INFO_KHR = 0x3B9ACDE9
    VK_STRUCTURE_TYPE_DISPLAY_MODE_CREATE_INFO_KHR = 0x3B9AD1D0
    VK_STRUCTURE_TYPE_DISPLAY_SURFACE_CREATE_INFO_KHR = 0x3B9AD1D1
    VK_STRUCTURE_TYPE_DISPLAY_PRESENT_INFO_KHR = 0x3B9AD5B8
    VK_STRUCTURE_TYPE_XLIB_SURFACE_CREATE_INFO_KHR = 0x3B9AD9A0
    VK_STRUCTURE_TYPE_XCB_SURFACE_CREATE_INFO_KHR = 0x3B9ADD88
    VK_STRUCTURE_TYPE_WAYLAND_SURFACE_CREATE_INFO_KHR = 0x3B9AE170
    VK_STRUCTURE_TYPE_ANDROID_SURFACE_CREATE_INFO_KHR = 0x3B9AE940
    VK_STRUCTURE_TYPE_WIN32_SURFACE_CREATE_INFO_KHR = 0x3B9AED28
    VK_STRUCTURE_TYPE_DEBUG_REPORT_CALLBACK_CREATE_INFO_EXT = 0x3B9AF4F8
    VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_RASTERIZATION_ORDER_AMD = 0x3B9B1050
    VK_STRUCTURE_TYPE_DEBUG_MARKER_OBJECT_NAME_INFO_EXT = 0x3B9B1FF0
    VK_STRUCTURE_TYPE_DEBUG_MARKER_OBJECT_TAG_INFO_EXT = 0x3B9B1FF1
    VK_STRUCTURE_TYPE_DEBUG_MARKER_MARKER_INFO_EXT = 0x3B9B1FF2
    VK_STRUCTURE_TYPE_VIDEO_PROFILE_INFO_KHR = 0x3B9B23D8
    VK_STRUCTURE_TYPE_VIDEO_CAPABILITIES_KHR = 0x3B9B23D9
    VK_STRUCTURE_TYPE_VIDEO_PICTURE_RESOURCE_INFO_KHR = 0x3B9B23DA
    VK_STRUCTURE_TYPE_VIDEO_SESSION_MEMORY_REQUIREMENTS_KHR = 0x3B9B23DB
    VK_STRUCTURE_TYPE_BIND_VIDEO_SESSION_MEMORY_INFO_KHR = 0x3B9B23DC
    VK_STRUCTURE_TYPE_VIDEO_SESSION_CREATE_INFO_KHR = 0x3B9B23DD
    VK_STRUCTURE_TYPE_VIDEO_SESSION_PARAMETERS_CREATE_INFO_KHR = 0x3B9B23DE
    VK_STRUCTURE_TYPE_VIDEO_SESSION_PARAMETERS_UPDATE_INFO_KHR = 0x3B9B23DF
    VK_STRUCTURE_TYPE_VIDEO_BEGIN_CODING_INFO_KHR = 0x3B9B23E0
    VK_STRUCTURE_TYPE_VIDEO_END_CODING_INFO_KHR = 0x3B9B23E1
    VK_STRUCTURE_TYPE_VIDEO_CODING_CONTROL_INFO_KHR = 0x3B9B23E2
    VK_STRUCTURE_TYPE_VIDEO_REFERENCE_SLOT_INFO_KHR = 0x3B9B23E3
    VK_STRUCTURE_TYPE_QUEUE_FAMILY_VIDEO_PROPERTIES_KHR = 0x3B9B23E4
    VK_STRUCTURE_TYPE_VIDEO_PROFILE_LIST_INFO_KHR = 0x3B9B23E5
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VIDEO_FORMAT_INFO_KHR = 0x3B9B23E6
    VK_STRUCTURE_TYPE_VIDEO_FORMAT_PROPERTIES_KHR = 0x3B9B23E7
    VK_STRUCTURE_TYPE_QUEUE_FAMILY_QUERY_RESULT_STATUS_PROPERTIES_KHR = 0x3B9B23E8
    VK_STRUCTURE_TYPE_VIDEO_DECODE_INFO_KHR = 0x3B9B27C0
    VK_STRUCTURE_TYPE_VIDEO_DECODE_CAPABILITIES_KHR = 0x3B9B27C1
    VK_STRUCTURE_TYPE_VIDEO_DECODE_USAGE_INFO_KHR = 0x3B9B27C2
    VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_IMAGE_CREATE_INFO_NV = 0x3B9B2F90
    VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_BUFFER_CREATE_INFO_NV = 0x3B9B2F91
    VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_MEMORY_ALLOCATE_INFO_NV = 0x3B9B2F92
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_FEATURES_EXT = 0x3B9B3760
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TRANSFORM_FEEDBACK_PROPERTIES_EXT = 0x3B9B3761
    VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_STREAM_CREATE_INFO_EXT = 0x3B9B3762
    VK_STRUCTURE_TYPE_CU_MODULE_CREATE_INFO_NVX = 0x3B9B3B48
    VK_STRUCTURE_TYPE_CU_FUNCTION_CREATE_INFO_NVX = 0x3B9B3B49
    VK_STRUCTURE_TYPE_CU_LAUNCH_INFO_NVX = 0x3B9B3B4A
    VK_STRUCTURE_TYPE_IMAGE_VIEW_HANDLE_INFO_NVX = 0x3B9B3F30
    VK_STRUCTURE_TYPE_IMAGE_VIEW_ADDRESS_PROPERTIES_NVX = 0x3B9B3F31
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_CAPABILITIES_KHR = 0x3B9B6640
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_PICTURE_INFO_KHR = 0x3B9B6641
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_PROFILE_INFO_KHR = 0x3B9B6643
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_SESSION_PARAMETERS_CREATE_INFO_KHR = 0x3B9B6644
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_SESSION_PARAMETERS_ADD_INFO_KHR = 0x3B9B6645
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H264_DPB_SLOT_INFO_KHR = 0x3B9B6646
    VK_STRUCTURE_TYPE_TEXTURE_LOD_GATHER_FORMAT_PROPERTIES_AMD = 0x3B9B6A28
    VK_STRUCTURE_TYPE_RENDERING_INFO = 0x3B9B75E0
    VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_INFO = 0x3B9B75E1
    VK_STRUCTURE_TYPE_PIPELINE_RENDERING_CREATE_INFO = 0x3B9B75E2
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DYNAMIC_RENDERING_FEATURES = 0x3B9B75E3
    VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_RENDERING_INFO = 0x3B9B75E4
    VK_STRUCTURE_TYPE_RENDERING_FRAGMENT_SHADING_RATE_ATTACHMENT_INFO_KHR = 0x3B9B75E6
    VK_STRUCTURE_TYPE_RENDERING_FRAGMENT_DENSITY_MAP_ATTACHMENT_INFO_EXT = 0x3B9B75E7
    VK_STRUCTURE_TYPE_ATTACHMENT_SAMPLE_COUNT_INFO_AMD = 0x3B9B75E8
    VK_STRUCTURE_TYPE_MULTIVIEW_PER_VIEW_ATTRIBUTES_INFO_NVX = 0x3B9B75E9
    VK_STRUCTURE_TYPE_STREAM_DESCRIPTOR_SURFACE_CREATE_INFO_GGP = 0x3B9B8968
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CORNER_SAMPLED_IMAGE_FEATURES_NV = 0x3B9B8D50
    VK_STRUCTURE_TYPE_RENDER_PASS_MULTIVIEW_CREATE_INFO = 0x3B9B9908
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_FEATURES = 0x3B9B9909
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_PROPERTIES = 0x3B9B990A
    VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_IMAGE_CREATE_INFO_NV = 0x3B9BA4C0
    VK_STRUCTURE_TYPE_EXPORT_MEMORY_ALLOCATE_INFO_NV = 0x3B9BA4C1
    VK_STRUCTURE_TYPE_IMPORT_MEMORY_WIN32_HANDLE_INFO_NV = 0x3B9BA8A8
    VK_STRUCTURE_TYPE_EXPORT_MEMORY_WIN32_HANDLE_INFO_NV = 0x3B9BA8A9
    VK_STRUCTURE_TYPE_WIN32_KEYED_MUTEX_ACQUIRE_RELEASE_INFO_NV = 0x3B9BAC90
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2 = 0x3B9BB078
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROPERTIES_2 = 0x3B9BB079
    VK_STRUCTURE_TYPE_FORMAT_PROPERTIES_2 = 0x3B9BB07A
    VK_STRUCTURE_TYPE_IMAGE_FORMAT_PROPERTIES_2 = 0x3B9BB07B
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_FORMAT_INFO_2 = 0x3B9BB07C
    VK_STRUCTURE_TYPE_QUEUE_FAMILY_PROPERTIES_2 = 0x3B9BB07D
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_PROPERTIES_2 = 0x3B9BB07E
    VK_STRUCTURE_TYPE_SPARSE_IMAGE_FORMAT_PROPERTIES_2 = 0x3B9BB07F
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SPARSE_IMAGE_FORMAT_INFO_2 = 0x3B9BB080
    VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_FLAGS_INFO = 0x3B9BB460
    VK_STRUCTURE_TYPE_DEVICE_GROUP_RENDER_PASS_BEGIN_INFO = 0x3B9BB463
    VK_STRUCTURE_TYPE_DEVICE_GROUP_COMMAND_BUFFER_BEGIN_INFO = 0x3B9BB464
    VK_STRUCTURE_TYPE_DEVICE_GROUP_SUBMIT_INFO = 0x3B9BB465
    VK_STRUCTURE_TYPE_DEVICE_GROUP_BIND_SPARSE_INFO = 0x3B9BB466
    VK_STRUCTURE_TYPE_DEVICE_GROUP_PRESENT_CAPABILITIES_KHR = 0x3B9BB467
    VK_STRUCTURE_TYPE_IMAGE_SWAPCHAIN_CREATE_INFO_KHR = 0x3B9BB468
    VK_STRUCTURE_TYPE_BIND_IMAGE_MEMORY_SWAPCHAIN_INFO_KHR = 0x3B9BB469
    VK_STRUCTURE_TYPE_ACQUIRE_NEXT_IMAGE_INFO_KHR = 0x3B9BB46A
    VK_STRUCTURE_TYPE_DEVICE_GROUP_PRESENT_INFO_KHR = 0x3B9BB46B
    VK_STRUCTURE_TYPE_DEVICE_GROUP_SWAPCHAIN_CREATE_INFO_KHR = 0x3B9BB46C
    VK_STRUCTURE_TYPE_BIND_BUFFER_MEMORY_DEVICE_GROUP_INFO = 0x3B9BB46D
    VK_STRUCTURE_TYPE_BIND_IMAGE_MEMORY_DEVICE_GROUP_INFO = 0x3B9BB46E
    VK_STRUCTURE_TYPE_VALIDATION_FLAGS_EXT = 0x3B9BB848
    VK_STRUCTURE_TYPE_VI_SURFACE_CREATE_INFO_NN = 0x3B9BBC30
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_DRAW_PARAMETERS_FEATURES = 0x3B9BC018
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TEXTURE_COMPRESSION_ASTC_HDR_FEATURES = 0x3B9BCBD0
    VK_STRUCTURE_TYPE_IMAGE_VIEW_ASTC_DECODE_MODE_EXT = 0x3B9BCFB8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ASTC_DECODE_FEATURES_EXT = 0x3B9BCFB9
    VK_STRUCTURE_TYPE_PIPELINE_ROBUSTNESS_CREATE_INFO_EXT = 0x3B9BD3A0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_ROBUSTNESS_FEATURES_EXT = 0x3B9BD3A1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_ROBUSTNESS_PROPERTIES_EXT = 0x3B9BD3A2
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GROUP_PROPERTIES = 0x3B9BDB70
    VK_STRUCTURE_TYPE_DEVICE_GROUP_DEVICE_CREATE_INFO = 0x3B9BDB71
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_IMAGE_FORMAT_INFO = 0x3B9BDF58
    VK_STRUCTURE_TYPE_EXTERNAL_IMAGE_FORMAT_PROPERTIES = 0x3B9BDF59
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_BUFFER_INFO = 0x3B9BDF5A
    VK_STRUCTURE_TYPE_EXTERNAL_BUFFER_PROPERTIES = 0x3B9BDF5B
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ID_PROPERTIES = 0x3B9BDF5C
    VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_BUFFER_CREATE_INFO = 0x3B9BE340
    VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_IMAGE_CREATE_INFO = 0x3B9BE341
    VK_STRUCTURE_TYPE_EXPORT_MEMORY_ALLOCATE_INFO = 0x3B9BE342
    VK_STRUCTURE_TYPE_IMPORT_MEMORY_WIN32_HANDLE_INFO_KHR = 0x3B9BE728
    VK_STRUCTURE_TYPE_EXPORT_MEMORY_WIN32_HANDLE_INFO_KHR = 0x3B9BE729
    VK_STRUCTURE_TYPE_MEMORY_WIN32_HANDLE_PROPERTIES_KHR = 0x3B9BE72A
    VK_STRUCTURE_TYPE_MEMORY_GET_WIN32_HANDLE_INFO_KHR = 0x3B9BE72B
    VK_STRUCTURE_TYPE_IMPORT_MEMORY_FD_INFO_KHR = 0x3B9BEB10
    VK_STRUCTURE_TYPE_MEMORY_FD_PROPERTIES_KHR = 0x3B9BEB11
    VK_STRUCTURE_TYPE_MEMORY_GET_FD_INFO_KHR = 0x3B9BEB12
    VK_STRUCTURE_TYPE_WIN32_KEYED_MUTEX_ACQUIRE_RELEASE_INFO_KHR = 0x3B9BEEF8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_SEMAPHORE_INFO = 0x3B9BF2E0
    VK_STRUCTURE_TYPE_EXTERNAL_SEMAPHORE_PROPERTIES = 0x3B9BF2E1
    VK_STRUCTURE_TYPE_EXPORT_SEMAPHORE_CREATE_INFO = 0x3B9BF6C8
    VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_WIN32_HANDLE_INFO_KHR = 0x3B9BFAB0
    VK_STRUCTURE_TYPE_EXPORT_SEMAPHORE_WIN32_HANDLE_INFO_KHR = 0x3B9BFAB1
    VK_STRUCTURE_TYPE_D3D12_FENCE_SUBMIT_INFO_KHR = 0x3B9BFAB2
    VK_STRUCTURE_TYPE_SEMAPHORE_GET_WIN32_HANDLE_INFO_KHR = 0x3B9BFAB3
    VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_FD_INFO_KHR = 0x3B9BFE98
    VK_STRUCTURE_TYPE_SEMAPHORE_GET_FD_INFO_KHR = 0x3B9BFE99
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PUSH_DESCRIPTOR_PROPERTIES_KHR = 0x3B9C0280
    VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_CONDITIONAL_RENDERING_INFO_EXT = 0x3B9C0668
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CONDITIONAL_RENDERING_FEATURES_EXT = 0x3B9C0669
    VK_STRUCTURE_TYPE_CONDITIONAL_RENDERING_BEGIN_INFO_EXT = 0x3B9C066A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_FLOAT16_INT8_FEATURES = 0x3B9C0A50
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_16BIT_STORAGE_FEATURES = 0x3B9C0E38
    VK_STRUCTURE_TYPE_PRESENT_REGIONS_KHR = 0x3B9C1220
    VK_STRUCTURE_TYPE_DESCRIPTOR_UPDATE_TEMPLATE_CREATE_INFO = 0x3B9C1608
    VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_W_SCALING_STATE_CREATE_INFO_NV = 0x3B9C1DD8
    VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_2_EXT = 0x3B9C2990
    VK_STRUCTURE_TYPE_DISPLAY_POWER_INFO_EXT = 0x3B9C2D78
    VK_STRUCTURE_TYPE_DEVICE_EVENT_INFO_EXT = 0x3B9C2D79
    VK_STRUCTURE_TYPE_DISPLAY_EVENT_INFO_EXT = 0x3B9C2D7A
    VK_STRUCTURE_TYPE_SWAPCHAIN_COUNTER_CREATE_INFO_EXT = 0x3B9C2D7B
    VK_STRUCTURE_TYPE_PRESENT_TIMES_INFO_GOOGLE = 0x3B9C3160
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_PROPERTIES = 0x3B9C3930
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_PER_VIEW_ATTRIBUTES_PROPERTIES_NVX = 0x3B9C44E8
    VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_SWIZZLE_STATE_CREATE_INFO_NV = 0x3B9C48D0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DISCARD_RECTANGLE_PROPERTIES_EXT = 0x3B9C4CB8
    VK_STRUCTURE_TYPE_PIPELINE_DISCARD_RECTANGLE_STATE_CREATE_INFO_EXT = 0x3B9C4CB9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CONSERVATIVE_RASTERIZATION_PROPERTIES_EXT = 0x3B9C5488
    VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_CONSERVATIVE_STATE_CREATE_INFO_EXT = 0x3B9C5489
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_ENABLE_FEATURES_EXT = 0x3B9C5870
    VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_DEPTH_CLIP_STATE_CREATE_INFO_EXT = 0x3B9C5871
    VK_STRUCTURE_TYPE_HDR_METADATA_EXT = 0x3B9C6428
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGELESS_FRAMEBUFFER_FEATURES = 0x3B9C6FE0
    VK_STRUCTURE_TYPE_FRAMEBUFFER_ATTACHMENTS_CREATE_INFO = 0x3B9C6FE1
    VK_STRUCTURE_TYPE_FRAMEBUFFER_ATTACHMENT_IMAGE_INFO = 0x3B9C6FE2
    VK_STRUCTURE_TYPE_RENDER_PASS_ATTACHMENT_BEGIN_INFO = 0x3B9C6FE3
    VK_STRUCTURE_TYPE_ATTACHMENT_DESCRIPTION_2 = 0x3B9C73C8
    VK_STRUCTURE_TYPE_ATTACHMENT_REFERENCE_2 = 0x3B9C73C9
    VK_STRUCTURE_TYPE_SUBPASS_DESCRIPTION_2 = 0x3B9C73CA
    VK_STRUCTURE_TYPE_SUBPASS_DEPENDENCY_2 = 0x3B9C73CB
    VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO_2 = 0x3B9C73CC
    VK_STRUCTURE_TYPE_SUBPASS_BEGIN_INFO = 0x3B9C73CD
    VK_STRUCTURE_TYPE_SUBPASS_END_INFO = 0x3B9C73CE
    VK_STRUCTURE_TYPE_SHARED_PRESENT_SURFACE_CAPABILITIES_KHR = 0x3B9C7B98
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_FENCE_INFO = 0x3B9C7F80
    VK_STRUCTURE_TYPE_EXTERNAL_FENCE_PROPERTIES = 0x3B9C7F81
    VK_STRUCTURE_TYPE_EXPORT_FENCE_CREATE_INFO = 0x3B9C8368
    VK_STRUCTURE_TYPE_IMPORT_FENCE_WIN32_HANDLE_INFO_KHR = 0x3B9C8750
    VK_STRUCTURE_TYPE_EXPORT_FENCE_WIN32_HANDLE_INFO_KHR = 0x3B9C8751
    VK_STRUCTURE_TYPE_FENCE_GET_WIN32_HANDLE_INFO_KHR = 0x3B9C8752
    VK_STRUCTURE_TYPE_IMPORT_FENCE_FD_INFO_KHR = 0x3B9C8B38
    VK_STRUCTURE_TYPE_FENCE_GET_FD_INFO_KHR = 0x3B9C8B39
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PERFORMANCE_QUERY_FEATURES_KHR = 0x3B9C8F20
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PERFORMANCE_QUERY_PROPERTIES_KHR = 0x3B9C8F21
    VK_STRUCTURE_TYPE_QUERY_POOL_PERFORMANCE_CREATE_INFO_KHR = 0x3B9C8F22
    VK_STRUCTURE_TYPE_PERFORMANCE_QUERY_SUBMIT_INFO_KHR = 0x3B9C8F23
    VK_STRUCTURE_TYPE_ACQUIRE_PROFILING_LOCK_INFO_KHR = 0x3B9C8F24
    VK_STRUCTURE_TYPE_PERFORMANCE_COUNTER_KHR = 0x3B9C8F25
    VK_STRUCTURE_TYPE_PERFORMANCE_COUNTER_DESCRIPTION_KHR = 0x3B9C8F26
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_POINT_CLIPPING_PROPERTIES = 0x3B9C9308
    VK_STRUCTURE_TYPE_RENDER_PASS_INPUT_ATTACHMENT_ASPECT_CREATE_INFO = 0x3B9C9309
    VK_STRUCTURE_TYPE_IMAGE_VIEW_USAGE_CREATE_INFO = 0x3B9C930A
    VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_DOMAIN_ORIGIN_STATE_CREATE_INFO = 0x3B9C930B
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SURFACE_INFO_2_KHR = 0x3B9C9AD8
    VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_2_KHR = 0x3B9C9AD9
    VK_STRUCTURE_TYPE_SURFACE_FORMAT_2_KHR = 0x3B9C9ADA
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VARIABLE_POINTERS_FEATURES = 0x3B9C9EC0
    VK_STRUCTURE_TYPE_DISPLAY_PROPERTIES_2_KHR = 0x3B9CA2A8
    VK_STRUCTURE_TYPE_DISPLAY_PLANE_PROPERTIES_2_KHR = 0x3B9CA2A9
    VK_STRUCTURE_TYPE_DISPLAY_MODE_PROPERTIES_2_KHR = 0x3B9CA2AA
    VK_STRUCTURE_TYPE_DISPLAY_PLANE_INFO_2_KHR = 0x3B9CA2AB
    VK_STRUCTURE_TYPE_DISPLAY_PLANE_CAPABILITIES_2_KHR = 0x3B9CA2AC
    VK_STRUCTURE_TYPE_IOS_SURFACE_CREATE_INFO_MVK = 0x3B9CA690
    VK_STRUCTURE_TYPE_MACOS_SURFACE_CREATE_INFO_MVK = 0x3B9CAA78
    VK_STRUCTURE_TYPE_MEMORY_DEDICATED_REQUIREMENTS = 0x3B9CBA18
    VK_STRUCTURE_TYPE_MEMORY_DEDICATED_ALLOCATE_INFO = 0x3B9CBA19
    VK_STRUCTURE_TYPE_DEBUG_UTILS_OBJECT_NAME_INFO_EXT = 0x3B9CBE00
    VK_STRUCTURE_TYPE_DEBUG_UTILS_OBJECT_TAG_INFO_EXT = 0x3B9CBE01
    VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT = 0x3B9CBE02
    VK_STRUCTURE_TYPE_DEBUG_UTILS_MESSENGER_CALLBACK_DATA_EXT = 0x3B9CBE03
    VK_STRUCTURE_TYPE_DEBUG_UTILS_MESSENGER_CREATE_INFO_EXT = 0x3B9CBE04
    VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_USAGE_ANDROID = 0x3B9CC1E8
    VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_PROPERTIES_ANDROID = 0x3B9CC1E9
    VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_FORMAT_PROPERTIES_ANDROID = 0x3B9CC1EA
    VK_STRUCTURE_TYPE_IMPORT_ANDROID_HARDWARE_BUFFER_INFO_ANDROID = 0x3B9CC1EB
    VK_STRUCTURE_TYPE_MEMORY_GET_ANDROID_HARDWARE_BUFFER_INFO_ANDROID = 0x3B9CC1EC
    VK_STRUCTURE_TYPE_EXTERNAL_FORMAT_ANDROID = 0x3B9CC1ED
    VK_STRUCTURE_TYPE_ANDROID_HARDWARE_BUFFER_FORMAT_PROPERTIES_2_ANDROID = 0x3B9CC1EE
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLER_FILTER_MINMAX_PROPERTIES = 0x3B9CC5D0
    VK_STRUCTURE_TYPE_SAMPLER_REDUCTION_MODE_CREATE_INFO = 0x3B9CC5D1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INLINE_UNIFORM_BLOCK_FEATURES = 0x3B9CE510
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INLINE_UNIFORM_BLOCK_PROPERTIES = 0x3B9CE511
    VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_INLINE_UNIFORM_BLOCK = 0x3B9CE512
    VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_INLINE_UNIFORM_BLOCK_CREATE_INFO = 0x3B9CE513
    VK_STRUCTURE_TYPE_SAMPLE_LOCATIONS_INFO_EXT = 0x3B9CF898
    VK_STRUCTURE_TYPE_RENDER_PASS_SAMPLE_LOCATIONS_BEGIN_INFO_EXT = 0x3B9CF899
    VK_STRUCTURE_TYPE_PIPELINE_SAMPLE_LOCATIONS_STATE_CREATE_INFO_EXT = 0x3B9CF89A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLE_LOCATIONS_PROPERTIES_EXT = 0x3B9CF89B
    VK_STRUCTURE_TYPE_MULTISAMPLE_PROPERTIES_EXT = 0x3B9CF89C
    VK_STRUCTURE_TYPE_PROTECTED_SUBMIT_INFO = 0x3B9D0068
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROTECTED_MEMORY_FEATURES = 0x3B9D0069
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROTECTED_MEMORY_PROPERTIES = 0x3B9D006A
    VK_STRUCTURE_TYPE_DEVICE_QUEUE_INFO_2 = 0x3B9D006B
    VK_STRUCTURE_TYPE_BUFFER_MEMORY_REQUIREMENTS_INFO_2 = 0x3B9D0450
    VK_STRUCTURE_TYPE_IMAGE_MEMORY_REQUIREMENTS_INFO_2 = 0x3B9D0451
    VK_STRUCTURE_TYPE_IMAGE_SPARSE_MEMORY_REQUIREMENTS_INFO_2 = 0x3B9D0452
    VK_STRUCTURE_TYPE_MEMORY_REQUIREMENTS_2 = 0x3B9D0453
    VK_STRUCTURE_TYPE_SPARSE_IMAGE_MEMORY_REQUIREMENTS_2 = 0x3B9D0454
    VK_STRUCTURE_TYPE_IMAGE_FORMAT_LIST_CREATE_INFO = 0x3B9D0838
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BLEND_OPERATION_ADVANCED_FEATURES_EXT = 0x3B9D0C20
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BLEND_OPERATION_ADVANCED_PROPERTIES_EXT = 0x3B9D0C21
    VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_ADVANCED_STATE_CREATE_INFO_EXT = 0x3B9D0C22
    VK_STRUCTURE_TYPE_PIPELINE_COVERAGE_TO_COLOR_STATE_CREATE_INFO_NV = 0x3B9D1008
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_BUILD_GEOMETRY_INFO_KHR = 0x3B9D13F0
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_DEVICE_ADDRESS_INFO_KHR = 0x3B9D13F2
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_AABBS_DATA_KHR = 0x3B9D13F3
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_INSTANCES_DATA_KHR = 0x3B9D13F4
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_TRIANGLES_DATA_KHR = 0x3B9D13F5
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_KHR = 0x3B9D13F6
    VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_ACCELERATION_STRUCTURE_KHR = 0x3B9D13F7
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_VERSION_INFO_KHR = 0x3B9D13F9
    VK_STRUCTURE_TYPE_COPY_ACCELERATION_STRUCTURE_INFO_KHR = 0x3B9D13FA
    VK_STRUCTURE_TYPE_COPY_ACCELERATION_STRUCTURE_TO_MEMORY_INFO_KHR = 0x3B9D13FB
    VK_STRUCTURE_TYPE_COPY_MEMORY_TO_ACCELERATION_STRUCTURE_INFO_KHR = 0x3B9D13FC
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ACCELERATION_STRUCTURE_FEATURES_KHR = 0x3B9D13FD
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ACCELERATION_STRUCTURE_PROPERTIES_KHR = 0x3B9D13FE
    VK_STRUCTURE_TYPE_RAY_TRACING_PIPELINE_CREATE_INFO_KHR = 0x3B9D13FF
    VK_STRUCTURE_TYPE_RAY_TRACING_SHADER_GROUP_CREATE_INFO_KHR = 0x3B9D1400
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_CREATE_INFO_KHR = 0x3B9D1401
    VK_STRUCTURE_TYPE_RAY_TRACING_PIPELINE_INTERFACE_CREATE_INFO_KHR = 0x3B9D1402
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_BUILD_SIZES_INFO_KHR = 0x3B9D1404
    VK_STRUCTURE_TYPE_PIPELINE_COVERAGE_MODULATION_STATE_CREATE_INFO_NV = 0x3B9D1BC0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SM_BUILTINS_FEATURES_NV = 0x3B9D2390
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SM_BUILTINS_PROPERTIES_NV = 0x3B9D2391
    VK_STRUCTURE_TYPE_SAMPLER_YCBCR_CONVERSION_CREATE_INFO = 0x3B9D2B60
    VK_STRUCTURE_TYPE_SAMPLER_YCBCR_CONVERSION_INFO = 0x3B9D2B61
    VK_STRUCTURE_TYPE_BIND_IMAGE_PLANE_MEMORY_INFO = 0x3B9D2B62
    VK_STRUCTURE_TYPE_IMAGE_PLANE_MEMORY_REQUIREMENTS_INFO = 0x3B9D2B63
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SAMPLER_YCBCR_CONVERSION_FEATURES = 0x3B9D2B64
    VK_STRUCTURE_TYPE_SAMPLER_YCBCR_CONVERSION_IMAGE_FORMAT_PROPERTIES = 0x3B9D2B65
    VK_STRUCTURE_TYPE_BIND_BUFFER_MEMORY_INFO = 0x3B9D2F48
    VK_STRUCTURE_TYPE_BIND_IMAGE_MEMORY_INFO = 0x3B9D2F49
    VK_STRUCTURE_TYPE_DRM_FORMAT_MODIFIER_PROPERTIES_LIST_EXT = 0x3B9D3330
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_DRM_FORMAT_MODIFIER_INFO_EXT = 0x3B9D3332
    VK_STRUCTURE_TYPE_IMAGE_DRM_FORMAT_MODIFIER_LIST_CREATE_INFO_EXT = 0x3B9D3333
    VK_STRUCTURE_TYPE_IMAGE_DRM_FORMAT_MODIFIER_EXPLICIT_CREATE_INFO_EXT = 0x3B9D3334
    VK_STRUCTURE_TYPE_IMAGE_DRM_FORMAT_MODIFIER_PROPERTIES_EXT = 0x3B9D3335
    VK_STRUCTURE_TYPE_DRM_FORMAT_MODIFIER_PROPERTIES_LIST_2_EXT = 0x3B9D3336
    VK_STRUCTURE_TYPE_VALIDATION_CACHE_CREATE_INFO_EXT = 0x3B9D3B00
    VK_STRUCTURE_TYPE_SHADER_MODULE_VALIDATION_CACHE_CREATE_INFO_EXT = 0x3B9D3B01
    VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_BINDING_FLAGS_CREATE_INFO = 0x3B9D3EE8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_INDEXING_FEATURES = 0x3B9D3EE9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_INDEXING_PROPERTIES = 0x3B9D3EEA
    VK_STRUCTURE_TYPE_DESCRIPTOR_SET_VARIABLE_DESCRIPTOR_COUNT_ALLOCATE_INFO = 0x3B9D3EEB
    VK_STRUCTURE_TYPE_DESCRIPTOR_SET_VARIABLE_DESCRIPTOR_COUNT_LAYOUT_SUPPORT = 0x3B9D3EEC
    VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_SHADING_RATE_IMAGE_STATE_CREATE_INFO_NV = 0x3B9D4AA0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADING_RATE_IMAGE_FEATURES_NV = 0x3B9D4AA1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADING_RATE_IMAGE_PROPERTIES_NV = 0x3B9D4AA2
    VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_COARSE_SAMPLE_ORDER_STATE_CREATE_INFO_NV = 0x3B9D4AA5
    VK_STRUCTURE_TYPE_RAY_TRACING_PIPELINE_CREATE_INFO_NV = 0x3B9D4E88
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_CREATE_INFO_NV = 0x3B9D4E89
    VK_STRUCTURE_TYPE_GEOMETRY_NV = 0x3B9D4E8B
    VK_STRUCTURE_TYPE_GEOMETRY_TRIANGLES_NV = 0x3B9D4E8C
    VK_STRUCTURE_TYPE_GEOMETRY_AABB_NV = 0x3B9D4E8D
    VK_STRUCTURE_TYPE_BIND_ACCELERATION_STRUCTURE_MEMORY_INFO_NV = 0x3B9D4E8E
    VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_ACCELERATION_STRUCTURE_NV = 0x3B9D4E8F
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_INFO_NV = 0x3B9D4E90
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_PROPERTIES_NV = 0x3B9D4E91
    VK_STRUCTURE_TYPE_RAY_TRACING_SHADER_GROUP_CREATE_INFO_NV = 0x3B9D4E93
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_INFO_NV = 0x3B9D4E94
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_REPRESENTATIVE_FRAGMENT_TEST_FEATURES_NV = 0x3B9D5270
    VK_STRUCTURE_TYPE_PIPELINE_REPRESENTATIVE_FRAGMENT_TEST_STATE_CREATE_INFO_NV = 0x3B9D5271
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_3_PROPERTIES = 0x3B9D5A40
    VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_SUPPORT = 0x3B9D5A41
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_VIEW_IMAGE_FORMAT_INFO_EXT = 0x3B9D6210
    VK_STRUCTURE_TYPE_FILTER_CUBIC_IMAGE_VIEW_IMAGE_FORMAT_PROPERTIES_EXT = 0x3B9D6211
    VK_STRUCTURE_TYPE_DEVICE_QUEUE_GLOBAL_PRIORITY_CREATE_INFO_KHR = 0x3B9D71B0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SUBGROUP_EXTENDED_TYPES_FEATURES = 0x3B9D7598
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_8BIT_STORAGE_FEATURES = 0x3B9D7D68
    VK_STRUCTURE_TYPE_IMPORT_MEMORY_HOST_POINTER_INFO_EXT = 0x3B9D8150
    VK_STRUCTURE_TYPE_MEMORY_HOST_POINTER_PROPERTIES_EXT = 0x3B9D8151
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_MEMORY_HOST_PROPERTIES_EXT = 0x3B9D8152
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_INT64_FEATURES = 0x3B9D8920
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CLOCK_FEATURES_KHR = 0x3B9D8D08
    VK_STRUCTURE_TYPE_PIPELINE_COMPILER_CONTROL_CREATE_INFO_AMD = 0x3B9D94D8
    VK_STRUCTURE_TYPE_CALIBRATED_TIMESTAMP_INFO_EXT = 0x3B9D98C0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_PROPERTIES_AMD = 0x3B9D9CA8
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_CAPABILITIES_KHR = 0x3B9DA478
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_SESSION_PARAMETERS_CREATE_INFO_KHR = 0x3B9DA479
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_SESSION_PARAMETERS_ADD_INFO_KHR = 0x3B9DA47A
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_PROFILE_INFO_KHR = 0x3B9DA47B
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_PICTURE_INFO_KHR = 0x3B9DA47C
    VK_STRUCTURE_TYPE_VIDEO_DECODE_H265_DPB_SLOT_INFO_KHR = 0x3B9DA47D
    VK_STRUCTURE_TYPE_DEVICE_MEMORY_OVERALLOCATION_CREATE_INFO_AMD = 0x3B9DAC48
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_PROPERTIES_EXT = 0x3B9DB030
    VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_DIVISOR_STATE_CREATE_INFO_EXT = 0x3B9DB031
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_ATTRIBUTE_DIVISOR_FEATURES_EXT = 0x3B9DB032
    VK_STRUCTURE_TYPE_PRESENT_FRAME_TOKEN_GGP = 0x3B9DB418
    VK_STRUCTURE_TYPE_PIPELINE_CREATION_FEEDBACK_CREATE_INFO = 0x3B9DB800
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DRIVER_PROPERTIES = 0x3B9DC7A0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FLOAT_CONTROLS_PROPERTIES = 0x3B9DCB88
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_STENCIL_RESOLVE_PROPERTIES = 0x3B9DD358
    VK_STRUCTURE_TYPE_SUBPASS_DESCRIPTION_DEPTH_STENCIL_RESOLVE = 0x3B9DD359
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COMPUTE_SHADER_DERIVATIVES_FEATURES_NV = 0x3B9DDB28
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MESH_SHADER_FEATURES_NV = 0x3B9DDF10
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MESH_SHADER_PROPERTIES_NV = 0x3B9DDF11
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_BARYCENTRIC_FEATURES_KHR = 0x3B9DE2F8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_IMAGE_FOOTPRINT_FEATURES_NV = 0x3B9DE6E0
    VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_EXCLUSIVE_SCISSOR_STATE_CREATE_INFO_NV = 0x3B9DEAC8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXCLUSIVE_SCISSOR_FEATURES_NV = 0x3B9DEACA
    VK_STRUCTURE_TYPE_CHECKPOINT_DATA_NV = 0x3B9DEEB0
    VK_STRUCTURE_TYPE_QUEUE_FAMILY_CHECKPOINT_PROPERTIES_NV = 0x3B9DEEB1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TIMELINE_SEMAPHORE_FEATURES = 0x3B9DF298
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TIMELINE_SEMAPHORE_PROPERTIES = 0x3B9DF299
    VK_STRUCTURE_TYPE_SEMAPHORE_TYPE_CREATE_INFO = 0x3B9DF29A
    VK_STRUCTURE_TYPE_TIMELINE_SEMAPHORE_SUBMIT_INFO = 0x3B9DF29B
    VK_STRUCTURE_TYPE_SEMAPHORE_WAIT_INFO = 0x3B9DF29C
    VK_STRUCTURE_TYPE_SEMAPHORE_SIGNAL_INFO = 0x3B9DF29D
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_INTEGER_FUNCTIONS_2_FEATURES_INTEL = 0x3B9DFA68
    VK_STRUCTURE_TYPE_QUERY_POOL_PERFORMANCE_QUERY_CREATE_INFO_INTEL = 0x3B9DFE50
    VK_STRUCTURE_TYPE_INITIALIZE_PERFORMANCE_API_INFO_INTEL = 0x3B9DFE51
    VK_STRUCTURE_TYPE_PERFORMANCE_MARKER_INFO_INTEL = 0x3B9DFE52
    VK_STRUCTURE_TYPE_PERFORMANCE_STREAM_MARKER_INFO_INTEL = 0x3B9DFE53
    VK_STRUCTURE_TYPE_PERFORMANCE_OVERRIDE_INFO_INTEL = 0x3B9DFE54
    VK_STRUCTURE_TYPE_PERFORMANCE_CONFIGURATION_ACQUIRE_INFO_INTEL = 0x3B9DFE55
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VULKAN_MEMORY_MODEL_FEATURES = 0x3B9E0238
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PCI_BUS_INFO_PROPERTIES_EXT = 0x3B9E0620
    VK_STRUCTURE_TYPE_DISPLAY_NATIVE_HDR_SURFACE_CAPABILITIES_AMD = 0x3B9E0A08
    VK_STRUCTURE_TYPE_SWAPCHAIN_DISPLAY_NATIVE_HDR_CREATE_INFO_AMD = 0x3B9E0A09
    VK_STRUCTURE_TYPE_IMAGEPIPE_SURFACE_CREATE_INFO_FUCHSIA = 0x3B9E0DF0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_TERMINATE_INVOCATION_FEATURES = 0x3B9E11D8
    VK_STRUCTURE_TYPE_METAL_SURFACE_CREATE_INFO_EXT = 0x3B9E19A8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_FEATURES_EXT = 0x3B9E1D90
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_PROPERTIES_EXT = 0x3B9E1D91
    VK_STRUCTURE_TYPE_RENDER_PASS_FRAGMENT_DENSITY_MAP_CREATE_INFO_EXT = 0x3B9E1D92
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SCALAR_BLOCK_LAYOUT_FEATURES = 0x3B9E2948
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_SIZE_CONTROL_PROPERTIES = 0x3B9E38E8
    VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_REQUIRED_SUBGROUP_SIZE_CREATE_INFO = 0x3B9E38E9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_SIZE_CONTROL_FEATURES = 0x3B9E38EA
    VK_STRUCTURE_TYPE_FRAGMENT_SHADING_RATE_ATTACHMENT_INFO_KHR = 0x3B9E3CD0
    VK_STRUCTURE_TYPE_PIPELINE_FRAGMENT_SHADING_RATE_STATE_CREATE_INFO_KHR = 0x3B9E3CD1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_PROPERTIES_KHR = 0x3B9E3CD2
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_FEATURES_KHR = 0x3B9E3CD3
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_KHR = 0x3B9E3CD4
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_PROPERTIES_2_AMD = 0x3B9E40B8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COHERENT_MEMORY_FEATURES_AMD = 0x3B9E4888
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_IMAGE_ATOMIC_INT64_FEATURES_EXT = 0x3B9E5C10
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_BUDGET_PROPERTIES_EXT = 0x3B9E67C8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_PRIORITY_FEATURES_EXT = 0x3B9E6BB0
    VK_STRUCTURE_TYPE_MEMORY_PRIORITY_ALLOCATE_INFO_EXT = 0x3B9E6BB1
    VK_STRUCTURE_TYPE_SURFACE_PROTECTED_CAPABILITIES_KHR = 0x3B9E6F98
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEDICATED_ALLOCATION_IMAGE_ALIASING_FEATURES_NV = 0x3B9E7380
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SEPARATE_DEPTH_STENCIL_LAYOUTS_FEATURES = 0x3B9E7768
    VK_STRUCTURE_TYPE_ATTACHMENT_REFERENCE_STENCIL_LAYOUT = 0x3B9E7769
    VK_STRUCTURE_TYPE_ATTACHMENT_DESCRIPTION_STENCIL_LAYOUT = 0x3B9E776A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BUFFER_DEVICE_ADDRESS_FEATURES_EXT = 0x3B9E8320
    VK_STRUCTURE_TYPE_BUFFER_DEVICE_ADDRESS_INFO = 0x3B9E8321
    VK_STRUCTURE_TYPE_BUFFER_DEVICE_ADDRESS_CREATE_INFO_EXT = 0x3B9E8322
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TOOL_PROPERTIES = 0x3B9E8708
    VK_STRUCTURE_TYPE_IMAGE_STENCIL_USAGE_CREATE_INFO = 0x3B9E8AF0
    VK_STRUCTURE_TYPE_VALIDATION_FEATURES_EXT = 0x3B9E8ED8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_WAIT_FEATURES_KHR = 0x3B9E92C0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COOPERATIVE_MATRIX_FEATURES_NV = 0x3B9E96A8
    VK_STRUCTURE_TYPE_COOPERATIVE_MATRIX_PROPERTIES_NV = 0x3B9E96A9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COOPERATIVE_MATRIX_PROPERTIES_NV = 0x3B9E96AA
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COVERAGE_REDUCTION_MODE_FEATURES_NV = 0x3B9E9A90
    VK_STRUCTURE_TYPE_PIPELINE_COVERAGE_REDUCTION_STATE_CREATE_INFO_NV = 0x3B9E9A91
    VK_STRUCTURE_TYPE_FRAMEBUFFER_MIXED_SAMPLES_COMBINATION_NV = 0x3B9E9A92
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_INTERLOCK_FEATURES_EXT = 0x3B9E9E78
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_YCBCR_IMAGE_ARRAYS_FEATURES_EXT = 0x3B9EA260
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_UNIFORM_BUFFER_STANDARD_LAYOUT_FEATURES = 0x3B9EA648
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROVOKING_VERTEX_FEATURES_EXT = 0x3B9EAA30
    VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_PROVOKING_VERTEX_STATE_CREATE_INFO_EXT = 0x3B9EAA31
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PROVOKING_VERTEX_PROPERTIES_EXT = 0x3B9EAA32
    VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_INFO_EXT = 0x3B9EAE18
    VK_STRUCTURE_TYPE_SURFACE_FULL_SCREEN_EXCLUSIVE_WIN32_INFO_EXT = 0x3B9EAE19
    VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_FULL_SCREEN_EXCLUSIVE_EXT = 0x3B9EAE1A
    VK_STRUCTURE_TYPE_HEADLESS_SURFACE_CREATE_INFO_EXT = 0x3B9EB200
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BUFFER_DEVICE_ADDRESS_FEATURES = 0x3B9EB5E8
    VK_STRUCTURE_TYPE_BUFFER_OPAQUE_CAPTURE_ADDRESS_CREATE_INFO = 0x3B9EB5EA
    VK_STRUCTURE_TYPE_MEMORY_OPAQUE_CAPTURE_ADDRESS_ALLOCATE_INFO = 0x3B9EB5EB
    VK_STRUCTURE_TYPE_DEVICE_MEMORY_OPAQUE_CAPTURE_ADDRESS_INFO = 0x3B9EB5EC
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_FEATURES_EXT = 0x3B9EBDB8
    VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_LINE_STATE_CREATE_INFO_EXT = 0x3B9EBDB9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINE_RASTERIZATION_PROPERTIES_EXT = 0x3B9EBDBA
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_FLOAT_FEATURES_EXT = 0x3B9EC1A0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_HOST_QUERY_RESET_FEATURES = 0x3B9EC588
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INDEX_TYPE_UINT8_FEATURES_EXT = 0x3B9ED528
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_FEATURES_EXT = 0x3B9EDCF8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_EXECUTABLE_PROPERTIES_FEATURES_KHR = 0x3B9EE4C8
    VK_STRUCTURE_TYPE_PIPELINE_INFO_KHR = 0x3B9EE4C9
    VK_STRUCTURE_TYPE_PIPELINE_EXECUTABLE_PROPERTIES_KHR = 0x3B9EE4CA
    VK_STRUCTURE_TYPE_PIPELINE_EXECUTABLE_INFO_KHR = 0x3B9EE4CB
    VK_STRUCTURE_TYPE_PIPELINE_EXECUTABLE_STATISTIC_KHR = 0x3B9EE4CC
    VK_STRUCTURE_TYPE_PIPELINE_EXECUTABLE_INTERNAL_REPRESENTATION_KHR = 0x3B9EE4CD
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_ATOMIC_FLOAT_2_FEATURES_EXT = 0x3B9EF468
    VK_STRUCTURE_TYPE_SURFACE_PRESENT_MODE_EXT = 0x3B9EF850
    VK_STRUCTURE_TYPE_SURFACE_PRESENT_SCALING_CAPABILITIES_EXT = 0x3B9EF851
    VK_STRUCTURE_TYPE_SURFACE_PRESENT_MODE_COMPATIBILITY_EXT = 0x3B9EF852
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SWAPCHAIN_MAINTENANCE_1_FEATURES_EXT = 0x3B9EFC38
    VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_FENCE_INFO_EXT = 0x3B9EFC39
    VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_MODES_CREATE_INFO_EXT = 0x3B9EFC3A
    VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_MODE_INFO_EXT = 0x3B9EFC3B
    VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_SCALING_CREATE_INFO_EXT = 0x3B9EFC3C
    VK_STRUCTURE_TYPE_RELEASE_SWAPCHAIN_IMAGES_INFO_EXT = 0x3B9EFC3D
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_DEMOTE_TO_HELPER_INVOCATION_FEATURES = 0x3B9F0020
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEVICE_GENERATED_COMMANDS_PROPERTIES_NV = 0x3B9F0408
    VK_STRUCTURE_TYPE_GRAPHICS_SHADER_GROUP_CREATE_INFO_NV = 0x3B9F0409
    VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_SHADER_GROUPS_CREATE_INFO_NV = 0x3B9F040A
    VK_STRUCTURE_TYPE_INDIRECT_COMMANDS_LAYOUT_TOKEN_NV = 0x3B9F040B
    VK_STRUCTURE_TYPE_INDIRECT_COMMANDS_LAYOUT_CREATE_INFO_NV = 0x3B9F040C
    VK_STRUCTURE_TYPE_GENERATED_COMMANDS_INFO_NV = 0x3B9F040D
    VK_STRUCTURE_TYPE_GENERATED_COMMANDS_MEMORY_REQUIREMENTS_INFO_NV = 0x3B9F040E
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEVICE_GENERATED_COMMANDS_FEATURES_NV = 0x3B9F040F
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INHERITED_VIEWPORT_SCISSOR_FEATURES_NV = 0x3B9F07F0
    VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_VIEWPORT_SCISSOR_INFO_NV = 0x3B9F07F1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_INTEGER_DOT_PRODUCT_FEATURES = 0x3B9F0FC0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_INTEGER_DOT_PRODUCT_PROPERTIES = 0x3B9F0FC1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TEXEL_BUFFER_ALIGNMENT_FEATURES_EXT = 0x3B9F13A8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TEXEL_BUFFER_ALIGNMENT_PROPERTIES = 0x3B9F13A9
    VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_RENDER_PASS_TRANSFORM_INFO_QCOM = 0x3B9F1790
    VK_STRUCTURE_TYPE_RENDER_PASS_TRANSFORM_BEGIN_INFO_QCOM = 0x3B9F1791
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEVICE_MEMORY_REPORT_FEATURES_EXT = 0x3B9F1F60
    VK_STRUCTURE_TYPE_DEVICE_DEVICE_MEMORY_REPORT_CREATE_INFO_EXT = 0x3B9F1F61
    VK_STRUCTURE_TYPE_DEVICE_MEMORY_REPORT_CALLBACK_DATA_EXT = 0x3B9F1F62
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_FEATURES_EXT = 0x3B9F2730
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ROBUSTNESS_2_PROPERTIES_EXT = 0x3B9F2731
    VK_STRUCTURE_TYPE_SAMPLER_CUSTOM_BORDER_COLOR_CREATE_INFO_EXT = 0x3B9F2B18
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_PROPERTIES_EXT = 0x3B9F2B19
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_CUSTOM_BORDER_COLOR_FEATURES_EXT = 0x3B9F2B1A
    VK_STRUCTURE_TYPE_PIPELINE_LIBRARY_CREATE_INFO_KHR = 0x3B9F36D0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_BARRIER_FEATURES_NV = 0x3B9F3EA0
    VK_STRUCTURE_TYPE_SURFACE_CAPABILITIES_PRESENT_BARRIER_NV = 0x3B9F3EA1
    VK_STRUCTURE_TYPE_SWAPCHAIN_PRESENT_BARRIER_CREATE_INFO_NV = 0x3B9F3EA2
    VK_STRUCTURE_TYPE_PRESENT_ID_KHR = 0x3B9F4670
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRESENT_ID_FEATURES_KHR = 0x3B9F4671
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRIVATE_DATA_FEATURES = 0x3B9F4A58
    VK_STRUCTURE_TYPE_DEVICE_PRIVATE_DATA_CREATE_INFO = 0x3B9F4A59
    VK_STRUCTURE_TYPE_PRIVATE_DATA_SLOT_CREATE_INFO = 0x3B9F4A5A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_CREATION_CACHE_CONTROL_FEATURES = 0x3B9F5228
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DIAGNOSTICS_CONFIG_FEATURES_NV = 0x3B9F5DE0
    VK_STRUCTURE_TYPE_DEVICE_DIAGNOSTICS_CONFIG_CREATE_INFO_NV = 0x3B9F5DE1
    VK_STRUCTURE_TYPE_EXPORT_METAL_OBJECT_CREATE_INFO_EXT = 0x3B9F88D8
    VK_STRUCTURE_TYPE_EXPORT_METAL_OBJECTS_INFO_EXT = 0x3B9F88D9
    VK_STRUCTURE_TYPE_EXPORT_METAL_DEVICE_INFO_EXT = 0x3B9F88DA
    VK_STRUCTURE_TYPE_EXPORT_METAL_COMMAND_QUEUE_INFO_EXT = 0x3B9F88DB
    VK_STRUCTURE_TYPE_EXPORT_METAL_BUFFER_INFO_EXT = 0x3B9F88DC
    VK_STRUCTURE_TYPE_IMPORT_METAL_BUFFER_INFO_EXT = 0x3B9F88DD
    VK_STRUCTURE_TYPE_EXPORT_METAL_TEXTURE_INFO_EXT = 0x3B9F88DE
    VK_STRUCTURE_TYPE_IMPORT_METAL_TEXTURE_INFO_EXT = 0x3B9F88DF
    VK_STRUCTURE_TYPE_EXPORT_METAL_IO_SURFACE_INFO_EXT = 0x3B9F88E0
    VK_STRUCTURE_TYPE_IMPORT_METAL_IO_SURFACE_INFO_EXT = 0x3B9F88E1
    VK_STRUCTURE_TYPE_EXPORT_METAL_SHARED_EVENT_INFO_EXT = 0x3B9F88E2
    VK_STRUCTURE_TYPE_IMPORT_METAL_SHARED_EVENT_INFO_EXT = 0x3B9F88E3
    VK_STRUCTURE_TYPE_MEMORY_BARRIER_2 = 0x3B9F9490
    VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER_2 = 0x3B9F9491
    VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER_2 = 0x3B9F9492
    VK_STRUCTURE_TYPE_DEPENDENCY_INFO = 0x3B9F9493
    VK_STRUCTURE_TYPE_SUBMIT_INFO_2 = 0x3B9F9494
    VK_STRUCTURE_TYPE_SEMAPHORE_SUBMIT_INFO = 0x3B9F9495
    VK_STRUCTURE_TYPE_COMMAND_BUFFER_SUBMIT_INFO = 0x3B9F9496
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SYNCHRONIZATION_2_FEATURES = 0x3B9F9497
    VK_STRUCTURE_TYPE_QUEUE_FAMILY_CHECKPOINT_PROPERTIES_2_NV = 0x3B9F9498
    VK_STRUCTURE_TYPE_CHECKPOINT_DATA_2_NV = 0x3B9F9499
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_BUFFER_PROPERTIES_EXT = 0x3B9F9C60
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_BUFFER_DENSITY_MAP_PROPERTIES_EXT = 0x3B9F9C61
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_BUFFER_FEATURES_EXT = 0x3B9F9C62
    VK_STRUCTURE_TYPE_DESCRIPTOR_ADDRESS_INFO_EXT = 0x3B9F9C63
    VK_STRUCTURE_TYPE_DESCRIPTOR_GET_INFO_EXT = 0x3B9F9C64
    VK_STRUCTURE_TYPE_BUFFER_CAPTURE_DESCRIPTOR_DATA_INFO_EXT = 0x3B9F9C65
    VK_STRUCTURE_TYPE_IMAGE_CAPTURE_DESCRIPTOR_DATA_INFO_EXT = 0x3B9F9C66
    VK_STRUCTURE_TYPE_IMAGE_VIEW_CAPTURE_DESCRIPTOR_DATA_INFO_EXT = 0x3B9F9C67
    VK_STRUCTURE_TYPE_SAMPLER_CAPTURE_DESCRIPTOR_DATA_INFO_EXT = 0x3B9F9C68
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_CAPTURE_DESCRIPTOR_DATA_INFO_EXT = 0x3B9F9C69
    VK_STRUCTURE_TYPE_OPAQUE_CAPTURE_DESCRIPTOR_DATA_CREATE_INFO_EXT = 0x3B9F9C6A
    VK_STRUCTURE_TYPE_DESCRIPTOR_BUFFER_BINDING_INFO_EXT = 0x3B9F9C6B
    VK_STRUCTURE_TYPE_DESCRIPTOR_BUFFER_BINDING_PUSH_DESCRIPTOR_BUFFER_HANDLE_EXT = 0x3B9F9C6C
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_FEATURES_EXT = 0x3B9FAC00
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GRAPHICS_PIPELINE_LIBRARY_PROPERTIES_EXT = 0x3B9FAC01
    VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_LIBRARY_CREATE_INFO_EXT = 0x3B9FAC02
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_EARLY_AND_LATE_FRAGMENT_TESTS_FEATURES_AMD = 0x3B9FAFE8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADER_BARYCENTRIC_PROPERTIES_KHR = 0x3B9FB3D0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_SUBGROUP_UNIFORM_CONTROL_FLOW_FEATURES_KHR = 0x3B9FB7B8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ZERO_INITIALIZE_WORKGROUP_MEMORY_FEATURES = 0x3B9FBF88
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_ENUMS_PROPERTIES_NV = 0x3B9FC370
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_SHADING_RATE_ENUMS_FEATURES_NV = 0x3B9FC371
    VK_STRUCTURE_TYPE_PIPELINE_FRAGMENT_SHADING_RATE_ENUM_STATE_CREATE_INFO_NV = 0x3B9FC372
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_MOTION_TRIANGLES_DATA_NV = 0x3B9FC758
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_MOTION_BLUR_FEATURES_NV = 0x3B9FC759
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_MOTION_INFO_NV = 0x3B9FC75A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MESH_SHADER_FEATURES_EXT = 0x3B9FCB40
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MESH_SHADER_PROPERTIES_EXT = 0x3B9FCB41
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_YCBCR_2_PLANE_444_FORMATS_FEATURES_EXT = 0x3B9FD310
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_2_FEATURES_EXT = 0x3B9FDAE0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_2_PROPERTIES_EXT = 0x3B9FDAE1
    VK_STRUCTURE_TYPE_COPY_COMMAND_TRANSFORM_INFO_QCOM = 0x3B9FDEC8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_ROBUSTNESS_FEATURES = 0x3B9FE698
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_WORKGROUP_MEMORY_EXPLICIT_LAYOUT_FEATURES_KHR = 0x3B9FEA80
    VK_STRUCTURE_TYPE_COPY_BUFFER_INFO_2 = 0x3B9FEE68
    VK_STRUCTURE_TYPE_COPY_IMAGE_INFO_2 = 0x3B9FEE69
    VK_STRUCTURE_TYPE_COPY_BUFFER_TO_IMAGE_INFO_2 = 0x3B9FEE6A
    VK_STRUCTURE_TYPE_COPY_IMAGE_TO_BUFFER_INFO_2 = 0x3B9FEE6B
    VK_STRUCTURE_TYPE_BLIT_IMAGE_INFO_2 = 0x3B9FEE6C
    VK_STRUCTURE_TYPE_RESOLVE_IMAGE_INFO_2 = 0x3B9FEE6D
    VK_STRUCTURE_TYPE_BUFFER_COPY_2 = 0x3B9FEE6E
    VK_STRUCTURE_TYPE_IMAGE_COPY_2 = 0x3B9FEE6F
    VK_STRUCTURE_TYPE_IMAGE_BLIT_2 = 0x3B9FEE70
    VK_STRUCTURE_TYPE_BUFFER_IMAGE_COPY_2 = 0x3B9FEE71
    VK_STRUCTURE_TYPE_IMAGE_RESOLVE_2 = 0x3B9FEE72
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_COMPRESSION_CONTROL_FEATURES_EXT = 0x3B9FF250
    VK_STRUCTURE_TYPE_IMAGE_COMPRESSION_CONTROL_EXT = 0x3B9FF251
    VK_STRUCTURE_TYPE_SUBRESOURCE_LAYOUT_2_EXT = 0x3B9FF252
    VK_STRUCTURE_TYPE_IMAGE_SUBRESOURCE_2_EXT = 0x3B9FF253
    VK_STRUCTURE_TYPE_IMAGE_COMPRESSION_PROPERTIES_EXT = 0x3B9FF254
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ATTACHMENT_FEEDBACK_LOOP_LAYOUT_FEATURES_EXT = 0x3B9FF638
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_4444_FORMATS_FEATURES_EXT = 0x3B9FFA20
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FAULT_FEATURES_EXT = 0x3B9FFE08
    VK_STRUCTURE_TYPE_DEVICE_FAULT_COUNTS_EXT = 0x3B9FFE09
    VK_STRUCTURE_TYPE_DEVICE_FAULT_INFO_EXT = 0x3B9FFE0A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RASTERIZATION_ORDER_ATTACHMENT_ACCESS_FEATURES_EXT = 0x3BA001F0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RGBA10X6_FORMATS_FEATURES_EXT = 0x3BA009C0
    VK_STRUCTURE_TYPE_DIRECTFB_SURFACE_CREATE_INFO_EXT = 0x3BA01190
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_PIPELINE_FEATURES_KHR = 0x3BA01578
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_PIPELINE_PROPERTIES_KHR = 0x3BA01579
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_QUERY_FEATURES_KHR = 0x3BA0196D
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MUTABLE_DESCRIPTOR_TYPE_FEATURES_EXT = 0x3BA02518
    VK_STRUCTURE_TYPE_MUTABLE_DESCRIPTOR_TYPE_CREATE_INFO_EXT = 0x3BA0251A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_VERTEX_INPUT_DYNAMIC_STATE_FEATURES_EXT = 0x3BA02900
    VK_STRUCTURE_TYPE_VERTEX_INPUT_BINDING_DESCRIPTION_2_EXT = 0x3BA02901
    VK_STRUCTURE_TYPE_VERTEX_INPUT_ATTRIBUTE_DESCRIPTION_2_EXT = 0x3BA02902
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DRM_PROPERTIES_EXT = 0x3BA02CE8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_ADDRESS_BINDING_REPORT_FEATURES_EXT = 0x3BA030D0
    VK_STRUCTURE_TYPE_DEVICE_ADDRESS_BINDING_CALLBACK_DATA_EXT = 0x3BA030D1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLIP_CONTROL_FEATURES_EXT = 0x3BA034B8
    VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_DEPTH_CLIP_CONTROL_CREATE_INFO_EXT = 0x3BA034B9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRIMITIVE_TOPOLOGY_LIST_RESTART_FEATURES_EXT = 0x3BA038A0
    VK_STRUCTURE_TYPE_FORMAT_PROPERTIES_3 = 0x3BA04840
    VK_STRUCTURE_TYPE_IMPORT_MEMORY_ZIRCON_HANDLE_INFO_FUCHSIA = 0x3BA057E0
    VK_STRUCTURE_TYPE_MEMORY_ZIRCON_HANDLE_PROPERTIES_FUCHSIA = 0x3BA057E1
    VK_STRUCTURE_TYPE_MEMORY_GET_ZIRCON_HANDLE_INFO_FUCHSIA = 0x3BA057E2
    VK_STRUCTURE_TYPE_IMPORT_SEMAPHORE_ZIRCON_HANDLE_INFO_FUCHSIA = 0x3BA05BC8
    VK_STRUCTURE_TYPE_SEMAPHORE_GET_ZIRCON_HANDLE_INFO_FUCHSIA = 0x3BA05BC9
    VK_STRUCTURE_TYPE_BUFFER_COLLECTION_CREATE_INFO_FUCHSIA = 0x3BA05FB0
    VK_STRUCTURE_TYPE_IMPORT_MEMORY_BUFFER_COLLECTION_FUCHSIA = 0x3BA05FB1
    VK_STRUCTURE_TYPE_BUFFER_COLLECTION_IMAGE_CREATE_INFO_FUCHSIA = 0x3BA05FB2
    VK_STRUCTURE_TYPE_BUFFER_COLLECTION_PROPERTIES_FUCHSIA = 0x3BA05FB3
    VK_STRUCTURE_TYPE_BUFFER_CONSTRAINTS_INFO_FUCHSIA = 0x3BA05FB4
    VK_STRUCTURE_TYPE_BUFFER_COLLECTION_BUFFER_CREATE_INFO_FUCHSIA = 0x3BA05FB5
    VK_STRUCTURE_TYPE_IMAGE_CONSTRAINTS_INFO_FUCHSIA = 0x3BA05FB6
    VK_STRUCTURE_TYPE_IMAGE_FORMAT_CONSTRAINTS_INFO_FUCHSIA = 0x3BA05FB7
    VK_STRUCTURE_TYPE_SYSMEM_COLOR_SPACE_FUCHSIA = 0x3BA05FB8
    VK_STRUCTURE_TYPE_BUFFER_COLLECTION_CONSTRAINTS_INFO_FUCHSIA = 0x3BA05FB9
    VK_STRUCTURE_TYPE_SUBPASS_SHADING_PIPELINE_CREATE_INFO_HUAWEI = 0x3BA06B68
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBPASS_SHADING_FEATURES_HUAWEI = 0x3BA06B69
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBPASS_SHADING_PROPERTIES_HUAWEI = 0x3BA06B6A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_INVOCATION_MASK_FEATURES_HUAWEI = 0x3BA06F50
    VK_STRUCTURE_TYPE_MEMORY_GET_REMOTE_ADDRESS_INFO_NV = 0x3BA07338
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTERNAL_MEMORY_RDMA_FEATURES_NV = 0x3BA07339
    VK_STRUCTURE_TYPE_PIPELINE_PROPERTIES_IDENTIFIER_EXT = 0x3BA07720
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_PROPERTIES_FEATURES_EXT = 0x3BA07721
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTISAMPLED_RENDER_TO_SINGLE_SAMPLED_FEATURES_EXT = 0x3BA086C0
    VK_STRUCTURE_TYPE_SUBPASS_RESOLVE_PERFORMANCE_QUERY_EXT = 0x3BA086C1
    VK_STRUCTURE_TYPE_MULTISAMPLED_RENDER_TO_SINGLE_SAMPLED_INFO_EXT = 0x3BA086C2
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_2_FEATURES_EXT = 0x3BA08AA8
    VK_STRUCTURE_TYPE_SCREEN_SURFACE_CREATE_INFO_QNX = 0x3BA08E90
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COLOR_WRITE_ENABLE_FEATURES_EXT = 0x3BA09A48
    VK_STRUCTURE_TYPE_PIPELINE_COLOR_WRITE_CREATE_INFO_EXT = 0x3BA09A49
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PRIMITIVES_GENERATED_QUERY_FEATURES_EXT = 0x3BA09E30
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_MAINTENANCE_1_FEATURES_KHR = 0x3BA0ADD0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_GLOBAL_PRIORITY_QUERY_FEATURES_KHR = 0x3BA0B5A0
    VK_STRUCTURE_TYPE_QUEUE_FAMILY_GLOBAL_PRIORITY_PROPERTIES_KHR = 0x3BA0B5A1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_VIEW_MIN_LOD_FEATURES_EXT = 0x3BA0C158
    VK_STRUCTURE_TYPE_IMAGE_VIEW_MIN_LOD_CREATE_INFO_EXT = 0x3BA0C159
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTI_DRAW_FEATURES_EXT = 0x3BA0C540
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTI_DRAW_PROPERTIES_EXT = 0x3BA0C541
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_2D_VIEW_OF_3D_FEATURES_EXT = 0x3BA0C928
    VK_STRUCTURE_TYPE_MICROMAP_BUILD_INFO_EXT = 0x3BA0D4E0
    VK_STRUCTURE_TYPE_MICROMAP_VERSION_INFO_EXT = 0x3BA0D4E1
    VK_STRUCTURE_TYPE_COPY_MICROMAP_INFO_EXT = 0x3BA0D4E2
    VK_STRUCTURE_TYPE_COPY_MICROMAP_TO_MEMORY_INFO_EXT = 0x3BA0D4E3
    VK_STRUCTURE_TYPE_COPY_MEMORY_TO_MICROMAP_INFO_EXT = 0x3BA0D4E4
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_OPACITY_MICROMAP_FEATURES_EXT = 0x3BA0D4E5
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_OPACITY_MICROMAP_PROPERTIES_EXT = 0x3BA0D4E6
    VK_STRUCTURE_TYPE_MICROMAP_CREATE_INFO_EXT = 0x3BA0D4E7
    VK_STRUCTURE_TYPE_MICROMAP_BUILD_SIZES_INFO_EXT = 0x3BA0D4E8
    VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_TRIANGLES_OPACITY_MICROMAP_EXT = 0x3BA0D4E9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_BORDER_COLOR_SWIZZLE_FEATURES_EXT = 0x3BA10F78
    VK_STRUCTURE_TYPE_SAMPLER_BORDER_COLOR_COMPONENT_MAPPING_CREATE_INFO_EXT = 0x3BA10F79
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PAGEABLE_DEVICE_LOCAL_MEMORY_FEATURES_EXT = 0x3BA11360
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_4_FEATURES = 0x3BA11748
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MAINTENANCE_4_PROPERTIES = 0x3BA11749
    VK_STRUCTURE_TYPE_DEVICE_BUFFER_MEMORY_REQUIREMENTS = 0x3BA1174A
    VK_STRUCTURE_TYPE_DEVICE_IMAGE_MEMORY_REQUIREMENTS = 0x3BA1174B
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_SET_HOST_MAPPING_FEATURES_VALVE = 0x3BA132A0
    VK_STRUCTURE_TYPE_DESCRIPTOR_SET_BINDING_REFERENCE_VALVE = 0x3BA132A1
    VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_HOST_MAPPING_INFO_VALVE = 0x3BA132A2
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DEPTH_CLAMP_ZERO_ONE_FEATURES_EXT = 0x3BA13688
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_NON_SEAMLESS_CUBE_MAP_FEATURES_EXT = 0x3BA13A70
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_OFFSET_FEATURES_QCOM = 0x3BA14628
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FRAGMENT_DENSITY_MAP_OFFSET_PROPERTIES_QCOM = 0x3BA14629
    VK_STRUCTURE_TYPE_SUBPASS_FRAGMENT_DENSITY_MAP_OFFSET_END_INFO_QCOM = 0x3BA1462A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COPY_MEMORY_INDIRECT_FEATURES_NV = 0x3BA14A10
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_COPY_MEMORY_INDIRECT_PROPERTIES_NV = 0x3BA14A11
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_DECOMPRESSION_FEATURES_NV = 0x3BA14DF8
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MEMORY_DECOMPRESSION_PROPERTIES_NV = 0x3BA14DF9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LINEAR_COLOR_ATTACHMENT_FEATURES_NV = 0x3BA159B0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_COMPRESSION_CONTROL_SWAPCHAIN_FEATURES_EXT = 0x3BA17508
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_PROCESSING_FEATURES_QCOM = 0x3BA180C0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_IMAGE_PROCESSING_PROPERTIES_QCOM = 0x3BA180C1
    VK_STRUCTURE_TYPE_IMAGE_VIEW_SAMPLE_WEIGHT_CREATE_INFO_QCOM = 0x3BA180C2
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_FEATURES_EXT = 0x3BA1BB58
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_EXTENDED_DYNAMIC_STATE_3_PROPERTIES_EXT = 0x3BA1BB59
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBPASS_MERGE_FEEDBACK_FEATURES_EXT = 0x3BA1C710
    VK_STRUCTURE_TYPE_RENDER_PASS_CREATION_CONTROL_EXT = 0x3BA1C711
    VK_STRUCTURE_TYPE_RENDER_PASS_CREATION_FEEDBACK_CREATE_INFO_EXT = 0x3BA1C712
    VK_STRUCTURE_TYPE_RENDER_PASS_SUBPASS_FEEDBACK_CREATE_INFO_EXT = 0x3BA1C713
    VK_STRUCTURE_TYPE_DIRECT_DRIVER_LOADING_INFO_LUNARG = 0x3BA1CAF8
    VK_STRUCTURE_TYPE_DIRECT_DRIVER_LOADING_LIST_LUNARG = 0x3BA1CAF9
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_MODULE_IDENTIFIER_FEATURES_EXT = 0x3BA1D6B0
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_MODULE_IDENTIFIER_PROPERTIES_EXT = 0x3BA1D6B1
    VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_MODULE_IDENTIFIER_CREATE_INFO_EXT = 0x3BA1D6B2
    VK_STRUCTURE_TYPE_SHADER_MODULE_IDENTIFIER_EXT = 0x3BA1D6B3
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_OPTICAL_FLOW_FEATURES_NV = 0x3BA1DE80
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_OPTICAL_FLOW_PROPERTIES_NV = 0x3BA1DE81
    VK_STRUCTURE_TYPE_OPTICAL_FLOW_IMAGE_FORMAT_INFO_NV = 0x3BA1DE82
    VK_STRUCTURE_TYPE_OPTICAL_FLOW_IMAGE_FORMAT_PROPERTIES_NV = 0x3BA1DE83
    VK_STRUCTURE_TYPE_OPTICAL_FLOW_SESSION_CREATE_INFO_NV = 0x3BA1DE84
    VK_STRUCTURE_TYPE_OPTICAL_FLOW_EXECUTE_INFO_NV = 0x3BA1DE85
    VK_STRUCTURE_TYPE_OPTICAL_FLOW_SESSION_CREATE_PRIVATE_DATA_INFO_NV = 0x3BA1DE8A
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_LEGACY_DITHERING_FEATURES_EXT = 0x3BA1E268
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_PIPELINE_PROTECTED_ACCESS_FEATURES_EXT = 0x3BA1E650
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_TILE_PROPERTIES_FEATURES_QCOM = 0x3BA22CA0
    VK_STRUCTURE_TYPE_TILE_PROPERTIES_QCOM = 0x3BA22CA1
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_AMIGO_PROFILING_FEATURES_SEC = 0x3BA23088
    VK_STRUCTURE_TYPE_AMIGO_PROFILING_SUBMIT_INFO_SEC = 0x3BA23089
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_MULTIVIEW_PER_VIEW_VIEWPORTS_FEATURES_QCOM = 0x3BA23C40
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_INVOCATION_REORDER_FEATURES_NV = 0x3BA24410
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_RAY_TRACING_INVOCATION_REORDER_PROPERTIES_NV = 0x3BA24411
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_BUILTINS_FEATURES_ARM = 0x3BA25F68
    VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SHADER_CORE_BUILTINS_PROPERTIES_ARM = 0x3BA25F69
    VK_STRUCTURE_TYPE_MAX_ENUM = 0x7FFFFFFF

class eTkFillMode(IntEnum):
    EFillMode_Solid = 0x0
    EFillMode_Wireframe = 0x1

class hknpParticleCollisionTypeShift(ctypes.Structure):
    class Enum(IntEnum):
        MESH_SHIFT = 0x0
        MESH_WITH_CONVEX_SHIFT = 0x1
        SDF_SHIFT = 0x2
        CONVEX_SHIFT = 0x3
        WORLD_SHIFT = 0x4


class eTkFrontFaceWindingOrder(IntEnum):
    EFrontFace_CCW = 0x0
    EFrontFace_CW = 0x1

class eTkSeparateFaceMask(IntEnum):
    ESeparateFaceMask_FrontAndBack = 0x0
    ESeparateFaceMask_Front = 0x1
    ESeparateFaceMask_Back = 0x2
    ESeparateFaceMask_Num = 0x3

class eTkTestType(IntEnum):
    ETestType_Depth = 0x0
    ETestType_Alpha = 0x1
    ETestType_Stencil = 0x2
    ETestType_Scissor = 0x3
    ETestType_Num = 0x4

class eTkTextureType(IntEnum):
    ETextureType_2D = 0x0
    ETextureType_3D = 0x1
    ETextureType_CubeMap = 0x2
    ETextureType_Array = 0x3
    ETextureType_Num = 0x4

class eTkTestFunction(IntEnum):
    ETestFunction_Always = 0x0
    ETestFunction_Greater = 0x1
    ETestFunction_Less = 0x2
    ETestFunction_Equal = 0x3
    ETestFunction_LessEqual = 0x4
    ETestFunction_GreaterEqual = 0x5
    ETestFunction_NotEqual = 0x6
    ETestFunction_Never = 0x7
    ETestFunction_Disabled = 0x8
    ETestFunction_Enabled = 0x9
    ETestFunction_Num = 0xA

class eClearType(IntEnum):
    EClear_Float = 0x0
    EClear_Int = 0x1
    EClear_UInt = 0x2

class ePlanetGenerateRequests(IntEnum):
    EGenerate_Biome = 0x0
    EGenerate_Seed = 0x1
    EGenerate_Hazards = 0x2
    EGenerate_Inhabitants = 0x3
    EGenerate_Buildings = 0x4
    EGenerate_TerrainShape = 0x5
    EGenerate_TerrainShapeTexture = 0x6
    EGenerate_TerrainShapeWater = 0x7
    EGenerate_AllColours = 0x8
    EGenerate_SkyColours = 0x9
    EGenerate_FoliageColours = 0xA
    EGenerate_SingleFoliageColour = 0xB
    EGenerate_TerrainTileSet = 0xC
    EGenerate_TerrainTileColours = 0xD
    EGenerate_SingleTerrainColour = 0xE
    EGenerate_TerrainTexture = 0xF
    EGenerate_ObjectLists = 0x10
    EGenerate_SpawnData = 0x11
    EGenerate_ObjectTextures = 0x12
    EGenerate_ObjectColours = 0x13
    EGenerate_SingleObjectMesh = 0x14
    EGenerate_SingleObjectTexture = 0x15
    EGenerate_SingleObjectColour = 0x16
    EGenerate_CreatureRoles = 0x17
    EGenerate_CreatureMesh = 0x18
    EGenerate_CreatureColour = 0x19
    EGenerate_Weather = 0x1A
    EGenerate_Sky = 0x1B
    EGenerate_Fog = 0x1C
    EGenerate_Clouds = 0x1D
    EGenerate_Lighting = 0x1E
    EGenerate_Water = 0x1F
    EGenerate_DaySetting = 0x20
    EGenerate_Rings = 0x21
    EGenerate_Refresh = 0x22
    EGenerate_RefreshFoliage = 0x23
    ENumPlanetGenerateRequests = 0x24

class ePadType(IntEnum):
    EPad_None = 0x0
    EPad_XInput = 0x1
    EPad_GLFW = 0x2
    EPad_XBoxOne = 0x3
    EPad_XBox360 = 0x4
    EPad_DS4 = 0x5
    EPad_DS5 = 0x6
    EPad_Move = 0x7
    EPad_SteamInput = 0x8
    EPad_Touch = 0x9
    EPad_OpenVR = 0xA
    EPad_SwitchPro = 0xB
    EPad_SwitchHandheld = 0xC
    EPad_GameInput = 0xD
    EPad_SwitchDebugPad = 0xE
    EPad_SwitchJoyConDual = 0xF
    EPad_Num = 0x10

class eInputButtonType(IntEnum):
    EButton_None = 0x0
    EButtonKeyboard = 0x1
    EButtonMouse = 0x2
    EButtonPad = 0x3
    EButtonHands = 0x4
    EButtonGesture = 0x5
    EButtonType_Num = 0x6

class eTkBlendMode(IntEnum):
    EBlendMode_None = 0x0
    EBlendMode_Normal = 0x1
    EBlendMode_Additive = 0x2
    EBlendMode_Subtractive = 0x3
    EBlendMode_SourceRGBIsAlpha = 0x4
    EBlendMode_EngineBlend = 0x5
    EBlendMode_EngineAdd = 0x6
    EBlendMode_EngineMult = 0x7
    EBlendMode_EngineBlend_OutputOneMinusAlpha = 0x8
    EBlendMode_EngineBlend_OutputAlpha = 0x9
    EBlendMode_EngineAdd_OutputOneMinusAlpha = 0xA
    EBlendMode_InvSourceAlpha = 0xB
    EBlendMode_SourceAlpha = 0xC
    EBlendMode_RgbOnly = 0xD
    EBlendMode_AlphaOnly = 0xE
    EBlendMode_Num = 0xF

class eBindingType(IntEnum):
    EBindingType_None = 0x0
    EBindingType_Base = 0x1
    EBindingType_Customised = 0x2
    EBindingType_Num = 0x3

class eInputPort(IntEnum):
    EPort_0 = 0x0
    EPort_1 = 0x1
    EPort_2 = 0x2
    EPort_3 = 0x3
    EPort_Debug = 0x4
    EPort_Console = 0x5
    ENumInputPorts = 0x6
    EPort_None = 0xFFFFFFFF

class eInputDeviceTransform(IntEnum):
    EDeviceTransform_Pad = 0x0
    EDeviceTransform_PrimaryHand = 0x1
    EDeviceTransform_SecondaryHand = 0x2
    EDeviceTransforms = 0x3
    EDeviceTransform_None = 0xFFFFFFFF

class eInputPointerMode(IntEnum):
    EInputPointer_Relative = 0x0
    EInputPointer_Absolute = 0x1

class eInputIconType(IntEnum):
    EInputIcon_Solid = 0x0
    EInputIcon_Overlay = 0x1
    EInputIcon_Special = 0x2
    EInputIconType_Num = 0x3

class VkImageLayout(IntEnum):
    VK_IMAGE_LAYOUT_UNDEFINED = 0x0
    VK_IMAGE_LAYOUT_GENERAL = 0x1
    VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL = 0x2
    VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL = 0x3
    VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL = 0x4
    VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL = 0x5
    VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL = 0x6
    VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL = 0x7
    VK_IMAGE_LAYOUT_PREINITIALIZED = 0x8
    VK_IMAGE_LAYOUT_PRESENT_SRC_KHR = 0x3B9ACDEA
    VK_IMAGE_LAYOUT_VIDEO_DECODE_DST_KHR = 0x3B9B27C0
    VK_IMAGE_LAYOUT_VIDEO_DECODE_SRC_KHR = 0x3B9B27C1
    VK_IMAGE_LAYOUT_VIDEO_DECODE_DPB_KHR = 0x3B9B27C2
    VK_IMAGE_LAYOUT_SHARED_PRESENT_KHR = 0x3B9C7B98
    VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_STENCIL_ATTACHMENT_OPTIMAL = 0x3B9C9308
    VK_IMAGE_LAYOUT_DEPTH_ATTACHMENT_STENCIL_READ_ONLY_OPTIMAL = 0x3B9C9309
    VK_IMAGE_LAYOUT_FRAGMENT_SHADING_RATE_ATTACHMENT_OPTIMAL_KHR = 0x3B9D4AA3
    VK_IMAGE_LAYOUT_FRAGMENT_DENSITY_MAP_OPTIMAL_EXT = 0x3B9E1D90
    VK_IMAGE_LAYOUT_DEPTH_ATTACHMENT_OPTIMAL = 0x3B9E7768
    VK_IMAGE_LAYOUT_DEPTH_READ_ONLY_OPTIMAL = 0x3B9E7769
    VK_IMAGE_LAYOUT_STENCIL_ATTACHMENT_OPTIMAL = 0x3B9E776A
    VK_IMAGE_LAYOUT_STENCIL_READ_ONLY_OPTIMAL = 0x3B9E776B
    VK_IMAGE_LAYOUT_READ_ONLY_OPTIMAL = 0x3B9F9490
    VK_IMAGE_LAYOUT_ATTACHMENT_OPTIMAL = 0x3B9F9491
    VK_IMAGE_LAYOUT_ATTACHMENT_FEEDBACK_LOOP_OPTIMAL_EXT = 0x3B9FF638
    VK_IMAGE_LAYOUT_MAX_ENUM = 0x7FFFFFFF

class VkResult(IntEnum):
    VK_SUCCESS = 0x0
    VK_NOT_READY = 0x1
    VK_TIMEOUT = 0x2
    VK_EVENT_SET = 0x3
    VK_EVENT_RESET = 0x4
    VK_INCOMPLETE = 0x5
    VK_SUBOPTIMAL_KHR = 0x3B9ACDEB
    VK_THREAD_IDLE_KHR = 0x3B9EE0E0
    VK_THREAD_DONE_KHR = 0x3B9EE0E1
    VK_OPERATION_DEFERRED_KHR = 0x3B9EE0E2
    VK_OPERATION_NOT_DEFERRED_KHR = 0x3B9EE0E3
    VK_PIPELINE_COMPILE_REQUIRED = 0x3B9F5228
    VK_RESULT_MAX_ENUM = 0x7FFFFFFF
    VK_ERROR_COMPRESSION_EXHAUSTED_EXT = 0xC4600DB0
    VK_ERROR_INVALID_OPAQUE_CAPTURE_ADDRESS = 0xC4614A18
    VK_ERROR_FULL_SCREEN_EXCLUSIVE_MODE_LOST_EXT = 0xC46151E8
    VK_ERROR_NOT_PERMITTED_KHR = 0xC4628E4F
    VK_ERROR_FRAGMENTATION = 0xC462C118
    VK_ERROR_INVALID_DRM_FORMAT_MODIFIER_PLANE_LAYOUT_EXT = 0xC462CCD0
    VK_ERROR_INVALID_EXTERNAL_HANDLE = 0xC4641CBD
    VK_ERROR_OUT_OF_POOL_MEMORY = 0xC4642878
    VK_ERROR_VIDEO_STD_VERSION_NOT_SUPPORTED_KHR = 0xC464DC23
    VK_ERROR_VIDEO_PROFILE_CODEC_NOT_SUPPORTED_KHR = 0xC464DC24
    VK_ERROR_VIDEO_PROFILE_FORMAT_NOT_SUPPORTED_KHR = 0xC464DC25
    VK_ERROR_VIDEO_PROFILE_OPERATION_NOT_SUPPORTED_KHR = 0xC464DC26
    VK_ERROR_VIDEO_PICTURE_LAYOUT_NOT_SUPPORTED_KHR = 0xC464DC27
    VK_ERROR_IMAGE_USAGE_NOT_SUPPORTED_KHR = 0xC464DC28
    VK_ERROR_INVALID_SHADER_NV = 0xC4650720
    VK_ERROR_VALIDATION_FAILED_EXT = 0xC4650B07
    VK_ERROR_INCOMPATIBLE_DISPLAY_KHR = 0xC4652A47
    VK_ERROR_OUT_OF_DATE_KHR = 0xC4653214
    VK_ERROR_NATIVE_WINDOW_IN_USE_KHR = 0xC46535FF
    VK_ERROR_SURFACE_LOST_KHR = 0xC4653600
    VK_ERROR_UNKNOWN = 0xFFFFFFF3
    VK_ERROR_FRAGMENTED_POOL = 0xFFFFFFF4
    VK_ERROR_FORMAT_NOT_SUPPORTED = 0xFFFFFFF5
    VK_ERROR_TOO_MANY_OBJECTS = 0xFFFFFFF6
    VK_ERROR_INCOMPATIBLE_DRIVER = 0xFFFFFFF7
    VK_ERROR_FEATURE_NOT_PRESENT = 0xFFFFFFF8
    VK_ERROR_EXTENSION_NOT_PRESENT = 0xFFFFFFF9
    VK_ERROR_LAYER_NOT_PRESENT = 0xFFFFFFFA
    VK_ERROR_MEMORY_MAP_FAILED = 0xFFFFFFFB
    VK_ERROR_DEVICE_LOST = 0xFFFFFFFC
    VK_ERROR_INITIALIZATION_FAILED = 0xFFFFFFFD
    VK_ERROR_OUT_OF_DEVICE_MEMORY = 0xFFFFFFFE
    VK_ERROR_OUT_OF_HOST_MEMORY = 0xFFFFFFFF

class VkFormat(IntEnum):
    VK_FORMAT_UNDEFINED = 0x0
    VK_FORMAT_R4G4_UNORM_PACK8 = 0x1
    VK_FORMAT_R4G4B4A4_UNORM_PACK16 = 0x2
    VK_FORMAT_B4G4R4A4_UNORM_PACK16 = 0x3
    VK_FORMAT_R5G6B5_UNORM_PACK16 = 0x4
    VK_FORMAT_B5G6R5_UNORM_PACK16 = 0x5
    VK_FORMAT_R5G5B5A1_UNORM_PACK16 = 0x6
    VK_FORMAT_B5G5R5A1_UNORM_PACK16 = 0x7
    VK_FORMAT_A1R5G5B5_UNORM_PACK16 = 0x8
    VK_FORMAT_R8_UNORM = 0x9
    VK_FORMAT_R8_SNORM = 0xA
    VK_FORMAT_R8_USCALED = 0xB
    VK_FORMAT_R8_SSCALED = 0xC
    VK_FORMAT_R8_UINT = 0xD
    VK_FORMAT_R8_SINT = 0xE
    VK_FORMAT_R8_SRGB = 0xF
    VK_FORMAT_R8G8_UNORM = 0x10
    VK_FORMAT_R8G8_SNORM = 0x11
    VK_FORMAT_R8G8_USCALED = 0x12
    VK_FORMAT_R8G8_SSCALED = 0x13
    VK_FORMAT_R8G8_UINT = 0x14
    VK_FORMAT_R8G8_SINT = 0x15
    VK_FORMAT_R8G8_SRGB = 0x16
    VK_FORMAT_R8G8B8_UNORM = 0x17
    VK_FORMAT_R8G8B8_SNORM = 0x18
    VK_FORMAT_R8G8B8_USCALED = 0x19
    VK_FORMAT_R8G8B8_SSCALED = 0x1A
    VK_FORMAT_R8G8B8_UINT = 0x1B
    VK_FORMAT_R8G8B8_SINT = 0x1C
    VK_FORMAT_R8G8B8_SRGB = 0x1D
    VK_FORMAT_B8G8R8_UNORM = 0x1E
    VK_FORMAT_B8G8R8_SNORM = 0x1F
    VK_FORMAT_B8G8R8_USCALED = 0x20
    VK_FORMAT_B8G8R8_SSCALED = 0x21
    VK_FORMAT_B8G8R8_UINT = 0x22
    VK_FORMAT_B8G8R8_SINT = 0x23
    VK_FORMAT_B8G8R8_SRGB = 0x24
    VK_FORMAT_R8G8B8A8_UNORM = 0x25
    VK_FORMAT_R8G8B8A8_SNORM = 0x26
    VK_FORMAT_R8G8B8A8_USCALED = 0x27
    VK_FORMAT_R8G8B8A8_SSCALED = 0x28
    VK_FORMAT_R8G8B8A8_UINT = 0x29
    VK_FORMAT_R8G8B8A8_SINT = 0x2A
    VK_FORMAT_R8G8B8A8_SRGB = 0x2B
    VK_FORMAT_B8G8R8A8_UNORM = 0x2C
    VK_FORMAT_B8G8R8A8_SNORM = 0x2D
    VK_FORMAT_B8G8R8A8_USCALED = 0x2E
    VK_FORMAT_B8G8R8A8_SSCALED = 0x2F
    VK_FORMAT_B8G8R8A8_UINT = 0x30
    VK_FORMAT_B8G8R8A8_SINT = 0x31
    VK_FORMAT_B8G8R8A8_SRGB = 0x32
    VK_FORMAT_A8B8G8R8_UNORM_PACK32 = 0x33
    VK_FORMAT_A8B8G8R8_SNORM_PACK32 = 0x34
    VK_FORMAT_A8B8G8R8_USCALED_PACK32 = 0x35
    VK_FORMAT_A8B8G8R8_SSCALED_PACK32 = 0x36
    VK_FORMAT_A8B8G8R8_UINT_PACK32 = 0x37
    VK_FORMAT_A8B8G8R8_SINT_PACK32 = 0x38
    VK_FORMAT_A8B8G8R8_SRGB_PACK32 = 0x39
    VK_FORMAT_A2R10G10B10_UNORM_PACK32 = 0x3A
    VK_FORMAT_A2R10G10B10_SNORM_PACK32 = 0x3B
    VK_FORMAT_A2R10G10B10_USCALED_PACK32 = 0x3C
    VK_FORMAT_A2R10G10B10_SSCALED_PACK32 = 0x3D
    VK_FORMAT_A2R10G10B10_UINT_PACK32 = 0x3E
    VK_FORMAT_A2R10G10B10_SINT_PACK32 = 0x3F
    VK_FORMAT_A2B10G10R10_UNORM_PACK32 = 0x40
    VK_FORMAT_A2B10G10R10_SNORM_PACK32 = 0x41
    VK_FORMAT_A2B10G10R10_USCALED_PACK32 = 0x42
    VK_FORMAT_A2B10G10R10_SSCALED_PACK32 = 0x43
    VK_FORMAT_A2B10G10R10_UINT_PACK32 = 0x44
    VK_FORMAT_A2B10G10R10_SINT_PACK32 = 0x45
    VK_FORMAT_R16_UNORM = 0x46
    VK_FORMAT_R16_SNORM = 0x47
    VK_FORMAT_R16_USCALED = 0x48
    VK_FORMAT_R16_SSCALED = 0x49
    VK_FORMAT_R16_UINT = 0x4A
    VK_FORMAT_R16_SINT = 0x4B
    VK_FORMAT_R16_SFLOAT = 0x4C
    VK_FORMAT_R16G16_UNORM = 0x4D
    VK_FORMAT_R16G16_SNORM = 0x4E
    VK_FORMAT_R16G16_USCALED = 0x4F
    VK_FORMAT_R16G16_SSCALED = 0x50
    VK_FORMAT_R16G16_UINT = 0x51
    VK_FORMAT_R16G16_SINT = 0x52
    VK_FORMAT_R16G16_SFLOAT = 0x53
    VK_FORMAT_R16G16B16_UNORM = 0x54
    VK_FORMAT_R16G16B16_SNORM = 0x55
    VK_FORMAT_R16G16B16_USCALED = 0x56
    VK_FORMAT_R16G16B16_SSCALED = 0x57
    VK_FORMAT_R16G16B16_UINT = 0x58
    VK_FORMAT_R16G16B16_SINT = 0x59
    VK_FORMAT_R16G16B16_SFLOAT = 0x5A
    VK_FORMAT_R16G16B16A16_UNORM = 0x5B
    VK_FORMAT_R16G16B16A16_SNORM = 0x5C
    VK_FORMAT_R16G16B16A16_USCALED = 0x5D
    VK_FORMAT_R16G16B16A16_SSCALED = 0x5E
    VK_FORMAT_R16G16B16A16_UINT = 0x5F
    VK_FORMAT_R16G16B16A16_SINT = 0x60
    VK_FORMAT_R16G16B16A16_SFLOAT = 0x61
    VK_FORMAT_R32_UINT = 0x62
    VK_FORMAT_R32_SINT = 0x63
    VK_FORMAT_R32_SFLOAT = 0x64
    VK_FORMAT_R32G32_UINT = 0x65
    VK_FORMAT_R32G32_SINT = 0x66
    VK_FORMAT_R32G32_SFLOAT = 0x67
    VK_FORMAT_R32G32B32_UINT = 0x68
    VK_FORMAT_R32G32B32_SINT = 0x69
    VK_FORMAT_R32G32B32_SFLOAT = 0x6A
    VK_FORMAT_R32G32B32A32_UINT = 0x6B
    VK_FORMAT_R32G32B32A32_SINT = 0x6C
    VK_FORMAT_R32G32B32A32_SFLOAT = 0x6D
    VK_FORMAT_R64_UINT = 0x6E
    VK_FORMAT_R64_SINT = 0x6F
    VK_FORMAT_R64_SFLOAT = 0x70
    VK_FORMAT_R64G64_UINT = 0x71
    VK_FORMAT_R64G64_SINT = 0x72
    VK_FORMAT_R64G64_SFLOAT = 0x73
    VK_FORMAT_R64G64B64_UINT = 0x74
    VK_FORMAT_R64G64B64_SINT = 0x75
    VK_FORMAT_R64G64B64_SFLOAT = 0x76
    VK_FORMAT_R64G64B64A64_UINT = 0x77
    VK_FORMAT_R64G64B64A64_SINT = 0x78
    VK_FORMAT_R64G64B64A64_SFLOAT = 0x79
    VK_FORMAT_B10G11R11_UFLOAT_PACK32 = 0x7A
    VK_FORMAT_E5B9G9R9_UFLOAT_PACK32 = 0x7B
    VK_FORMAT_D16_UNORM = 0x7C
    VK_FORMAT_X8_D24_UNORM_PACK32 = 0x7D
    VK_FORMAT_D32_SFLOAT = 0x7E
    VK_FORMAT_S8_UINT = 0x7F
    VK_FORMAT_D16_UNORM_S8_UINT = 0x80
    VK_FORMAT_D24_UNORM_S8_UINT = 0x81
    VK_FORMAT_D32_SFLOAT_S8_UINT = 0x82
    VK_FORMAT_BC1_RGB_UNORM_BLOCK = 0x83
    VK_FORMAT_BC1_RGB_SRGB_BLOCK = 0x84
    VK_FORMAT_BC1_RGBA_UNORM_BLOCK = 0x85
    VK_FORMAT_BC1_RGBA_SRGB_BLOCK = 0x86
    VK_FORMAT_BC2_UNORM_BLOCK = 0x87
    VK_FORMAT_BC2_SRGB_BLOCK = 0x88
    VK_FORMAT_BC3_UNORM_BLOCK = 0x89
    VK_FORMAT_BC3_SRGB_BLOCK = 0x8A
    VK_FORMAT_BC4_UNORM_BLOCK = 0x8B
    VK_FORMAT_BC4_SNORM_BLOCK = 0x8C
    VK_FORMAT_BC5_UNORM_BLOCK = 0x8D
    VK_FORMAT_BC5_SNORM_BLOCK = 0x8E
    VK_FORMAT_BC6H_UFLOAT_BLOCK = 0x8F
    VK_FORMAT_BC6H_SFLOAT_BLOCK = 0x90
    VK_FORMAT_BC7_UNORM_BLOCK = 0x91
    VK_FORMAT_BC7_SRGB_BLOCK = 0x92
    VK_FORMAT_ETC2_R8G8B8_UNORM_BLOCK = 0x93
    VK_FORMAT_ETC2_R8G8B8_SRGB_BLOCK = 0x94
    VK_FORMAT_ETC2_R8G8B8A1_UNORM_BLOCK = 0x95
    VK_FORMAT_ETC2_R8G8B8A1_SRGB_BLOCK = 0x96
    VK_FORMAT_ETC2_R8G8B8A8_UNORM_BLOCK = 0x97
    VK_FORMAT_ETC2_R8G8B8A8_SRGB_BLOCK = 0x98
    VK_FORMAT_EAC_R11_UNORM_BLOCK = 0x99
    VK_FORMAT_EAC_R11_SNORM_BLOCK = 0x9A
    VK_FORMAT_EAC_R11G11_UNORM_BLOCK = 0x9B
    VK_FORMAT_EAC_R11G11_SNORM_BLOCK = 0x9C
    VK_FORMAT_ASTC_4x4_UNORM_BLOCK = 0x9D
    VK_FORMAT_ASTC_4x4_SRGB_BLOCK = 0x9E
    VK_FORMAT_ASTC_5x4_UNORM_BLOCK = 0x9F
    VK_FORMAT_ASTC_5x4_SRGB_BLOCK = 0xA0
    VK_FORMAT_ASTC_5x5_UNORM_BLOCK = 0xA1
    VK_FORMAT_ASTC_5x5_SRGB_BLOCK = 0xA2
    VK_FORMAT_ASTC_6x5_UNORM_BLOCK = 0xA3
    VK_FORMAT_ASTC_6x5_SRGB_BLOCK = 0xA4
    VK_FORMAT_ASTC_6x6_UNORM_BLOCK = 0xA5
    VK_FORMAT_ASTC_6x6_SRGB_BLOCK = 0xA6
    VK_FORMAT_ASTC_8x5_UNORM_BLOCK = 0xA7
    VK_FORMAT_ASTC_8x5_SRGB_BLOCK = 0xA8
    VK_FORMAT_ASTC_8x6_UNORM_BLOCK = 0xA9
    VK_FORMAT_ASTC_8x6_SRGB_BLOCK = 0xAA
    VK_FORMAT_ASTC_8x8_UNORM_BLOCK = 0xAB
    VK_FORMAT_ASTC_8x8_SRGB_BLOCK = 0xAC
    VK_FORMAT_ASTC_10x5_UNORM_BLOCK = 0xAD
    VK_FORMAT_ASTC_10x5_SRGB_BLOCK = 0xAE
    VK_FORMAT_ASTC_10x6_UNORM_BLOCK = 0xAF
    VK_FORMAT_ASTC_10x6_SRGB_BLOCK = 0xB0
    VK_FORMAT_ASTC_10x8_UNORM_BLOCK = 0xB1
    VK_FORMAT_ASTC_10x8_SRGB_BLOCK = 0xB2
    VK_FORMAT_ASTC_10x10_UNORM_BLOCK = 0xB3
    VK_FORMAT_ASTC_10x10_SRGB_BLOCK = 0xB4
    VK_FORMAT_ASTC_12x10_UNORM_BLOCK = 0xB5
    VK_FORMAT_ASTC_12x10_SRGB_BLOCK = 0xB6
    VK_FORMAT_ASTC_12x12_UNORM_BLOCK = 0xB7
    VK_FORMAT_ASTC_12x12_SRGB_BLOCK = 0xB8
    VK_FORMAT_PVRTC1_2BPP_UNORM_BLOCK_IMG = 0x3B9B9CF0
    VK_FORMAT_PVRTC1_4BPP_UNORM_BLOCK_IMG = 0x3B9B9CF1
    VK_FORMAT_PVRTC2_2BPP_UNORM_BLOCK_IMG = 0x3B9B9CF2
    VK_FORMAT_PVRTC2_4BPP_UNORM_BLOCK_IMG = 0x3B9B9CF3
    VK_FORMAT_PVRTC1_2BPP_SRGB_BLOCK_IMG = 0x3B9B9CF4
    VK_FORMAT_PVRTC1_4BPP_SRGB_BLOCK_IMG = 0x3B9B9CF5
    VK_FORMAT_PVRTC2_2BPP_SRGB_BLOCK_IMG = 0x3B9B9CF6
    VK_FORMAT_PVRTC2_4BPP_SRGB_BLOCK_IMG = 0x3B9B9CF7
    VK_FORMAT_ASTC_4x4_SFLOAT_BLOCK = 0x3B9BCBD0
    VK_FORMAT_ASTC_5x4_SFLOAT_BLOCK = 0x3B9BCBD1
    VK_FORMAT_ASTC_5x5_SFLOAT_BLOCK = 0x3B9BCBD2
    VK_FORMAT_ASTC_6x5_SFLOAT_BLOCK = 0x3B9BCBD3
    VK_FORMAT_ASTC_6x6_SFLOAT_BLOCK = 0x3B9BCBD4
    VK_FORMAT_ASTC_8x5_SFLOAT_BLOCK = 0x3B9BCBD5
    VK_FORMAT_ASTC_8x6_SFLOAT_BLOCK = 0x3B9BCBD6
    VK_FORMAT_ASTC_8x8_SFLOAT_BLOCK = 0x3B9BCBD7
    VK_FORMAT_ASTC_10x5_SFLOAT_BLOCK = 0x3B9BCBD8
    VK_FORMAT_ASTC_10x6_SFLOAT_BLOCK = 0x3B9BCBD9
    VK_FORMAT_ASTC_10x8_SFLOAT_BLOCK = 0x3B9BCBDA
    VK_FORMAT_ASTC_10x10_SFLOAT_BLOCK = 0x3B9BCBDB
    VK_FORMAT_ASTC_12x10_SFLOAT_BLOCK = 0x3B9BCBDC
    VK_FORMAT_ASTC_12x12_SFLOAT_BLOCK = 0x3B9BCBDD
    VK_FORMAT_G8B8G8R8_422_UNORM = 0x3B9D2B60
    VK_FORMAT_B8G8R8G8_422_UNORM = 0x3B9D2B61
    VK_FORMAT_G8_B8_R8_3PLANE_420_UNORM = 0x3B9D2B62
    VK_FORMAT_G8_B8R8_2PLANE_420_UNORM = 0x3B9D2B63
    VK_FORMAT_G8_B8_R8_3PLANE_422_UNORM = 0x3B9D2B64
    VK_FORMAT_G8_B8R8_2PLANE_422_UNORM = 0x3B9D2B65
    VK_FORMAT_G8_B8_R8_3PLANE_444_UNORM = 0x3B9D2B66
    VK_FORMAT_R10X6_UNORM_PACK16 = 0x3B9D2B67
    VK_FORMAT_R10X6G10X6_UNORM_2PACK16 = 0x3B9D2B68
    VK_FORMAT_R10X6G10X6B10X6A10X6_UNORM_4PACK16 = 0x3B9D2B69
    VK_FORMAT_G10X6B10X6G10X6R10X6_422_UNORM_4PACK16 = 0x3B9D2B6A
    VK_FORMAT_B10X6G10X6R10X6G10X6_422_UNORM_4PACK16 = 0x3B9D2B6B
    VK_FORMAT_G10X6_B10X6_R10X6_3PLANE_420_UNORM_3PACK16 = 0x3B9D2B6C
    VK_FORMAT_G10X6_B10X6R10X6_2PLANE_420_UNORM_3PACK16 = 0x3B9D2B6D
    VK_FORMAT_G10X6_B10X6_R10X6_3PLANE_422_UNORM_3PACK16 = 0x3B9D2B6E
    VK_FORMAT_G10X6_B10X6R10X6_2PLANE_422_UNORM_3PACK16 = 0x3B9D2B6F
    VK_FORMAT_G10X6_B10X6_R10X6_3PLANE_444_UNORM_3PACK16 = 0x3B9D2B70
    VK_FORMAT_R12X4_UNORM_PACK16 = 0x3B9D2B71
    VK_FORMAT_R12X4G12X4_UNORM_2PACK16 = 0x3B9D2B72
    VK_FORMAT_R12X4G12X4B12X4A12X4_UNORM_4PACK16 = 0x3B9D2B73
    VK_FORMAT_G12X4B12X4G12X4R12X4_422_UNORM_4PACK16 = 0x3B9D2B74
    VK_FORMAT_B12X4G12X4R12X4G12X4_422_UNORM_4PACK16 = 0x3B9D2B75
    VK_FORMAT_G12X4_B12X4_R12X4_3PLANE_420_UNORM_3PACK16 = 0x3B9D2B76
    VK_FORMAT_G12X4_B12X4R12X4_2PLANE_420_UNORM_3PACK16 = 0x3B9D2B77
    VK_FORMAT_G12X4_B12X4_R12X4_3PLANE_422_UNORM_3PACK16 = 0x3B9D2B78
    VK_FORMAT_G12X4_B12X4R12X4_2PLANE_422_UNORM_3PACK16 = 0x3B9D2B79
    VK_FORMAT_G12X4_B12X4_R12X4_3PLANE_444_UNORM_3PACK16 = 0x3B9D2B7A
    VK_FORMAT_G16B16G16R16_422_UNORM = 0x3B9D2B7B
    VK_FORMAT_B16G16R16G16_422_UNORM = 0x3B9D2B7C
    VK_FORMAT_G16_B16_R16_3PLANE_420_UNORM = 0x3B9D2B7D
    VK_FORMAT_G16_B16R16_2PLANE_420_UNORM = 0x3B9D2B7E
    VK_FORMAT_G16_B16_R16_3PLANE_422_UNORM = 0x3B9D2B7F
    VK_FORMAT_G16_B16R16_2PLANE_422_UNORM = 0x3B9D2B80
    VK_FORMAT_G16_B16_R16_3PLANE_444_UNORM = 0x3B9D2B81
    VK_FORMAT_G8_B8R8_2PLANE_444_UNORM = 0x3B9FD310
    VK_FORMAT_G10X6_B10X6R10X6_2PLANE_444_UNORM_3PACK16 = 0x3B9FD311
    VK_FORMAT_G12X4_B12X4R12X4_2PLANE_444_UNORM_3PACK16 = 0x3B9FD312
    VK_FORMAT_G16_B16R16_2PLANE_444_UNORM = 0x3B9FD313
    VK_FORMAT_A4R4G4B4_UNORM_PACK16 = 0x3B9FFA20
    VK_FORMAT_A4B4G4R4_UNORM_PACK16 = 0x3B9FFA21
    VK_FORMAT_R16G16_S10_5_NV = 0x3BA1DE80
    VK_FORMAT_MAX_ENUM = 0x7FFFFFFF

class VkSharingMode(IntEnum):
    VK_SHARING_MODE_EXCLUSIVE = 0x0
    VK_SHARING_MODE_CONCURRENT = 0x1
    VK_SHARING_MODE_MAX_ENUM = 0x7FFFFFFF

class VkFrontFace(IntEnum):
    VK_FRONT_FACE_COUNTER_CLOCKWISE = 0x0
    VK_FRONT_FACE_CLOCKWISE = 0x1
    VK_FRONT_FACE_MAX_ENUM = 0x7FFFFFFF

class VkPrimitiveTopology(IntEnum):
    VK_PRIMITIVE_TOPOLOGY_POINT_LIST = 0x0
    VK_PRIMITIVE_TOPOLOGY_LINE_LIST = 0x1
    VK_PRIMITIVE_TOPOLOGY_LINE_STRIP = 0x2
    VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST = 0x3
    VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP = 0x4
    VK_PRIMITIVE_TOPOLOGY_TRIANGLE_FAN = 0x5
    VK_PRIMITIVE_TOPOLOGY_LINE_LIST_WITH_ADJACENCY = 0x6
    VK_PRIMITIVE_TOPOLOGY_LINE_STRIP_WITH_ADJACENCY = 0x7
    VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST_WITH_ADJACENCY = 0x8
    VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP_WITH_ADJACENCY = 0x9
    VK_PRIMITIVE_TOPOLOGY_PATCH_LIST = 0xA
    VK_PRIMITIVE_TOPOLOGY_MAX_ENUM = 0x7FFFFFFF

class VkPolygonMode(IntEnum):
    VK_POLYGON_MODE_FILL = 0x0
    VK_POLYGON_MODE_LINE = 0x1
    VK_POLYGON_MODE_POINT = 0x2
    VK_POLYGON_MODE_FILL_RECTANGLE_NV = 0x3B9D1FA8
    VK_POLYGON_MODE_MAX_ENUM = 0x7FFFFFFF

class eAttitudeTowardsPlayer(IntEnum):
    EAttitude_Indifferent = 0x0
    EAttitude_Afraid = 0x1
    EAttitude_TeachALesson = 0x2
    EAttitude_WantsDead = 0x3
    EAttitude_WantsDeadIgnoreBullets = 0x4
    EAttitude_Friendly = 0x5
    EAttitude_VeryFriendly = 0x6

class eFriendlyAttitude(IntEnum):
    EFriendlyAttitude_Hungry = 0x0
    EFriendlyAttitude_Friendly = 0x1
    EFriendlyAttitude_FedBasic = 0x2
    EFriendlyAttitude_FedSuper = 0x3
    EFriendlyAttitude_Harvested = 0x4
    EFriendlyAttitude_Scared = 0x5

class VkIndexType(IntEnum):
    VK_INDEX_TYPE_UINT16 = 0x0
    VK_INDEX_TYPE_UINT32 = 0x1
    VK_INDEX_TYPE_NONE_KHR = 0x3B9D4E88
    VK_INDEX_TYPE_UINT8_EXT = 0x3B9ED528
    VK_INDEX_TYPE_MAX_ENUM = 0x7FFFFFFF

class VkMemoryPropertyFlagBits(IntEnum):
    VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT = 0x1
    VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT = 0x2
    VK_MEMORY_PROPERTY_HOST_COHERENT_BIT = 0x4
    VK_MEMORY_PROPERTY_HOST_CACHED_BIT = 0x8
    VK_MEMORY_PROPERTY_LAZILY_ALLOCATED_BIT = 0x10
    VK_MEMORY_PROPERTY_PROTECTED_BIT = 0x20
    VK_MEMORY_PROPERTY_DEVICE_COHERENT_BIT_AMD = 0x40
    VK_MEMORY_PROPERTY_DEVICE_UNCACHED_BIT_AMD = 0x80
    VK_MEMORY_PROPERTY_RDMA_CAPABLE_BIT_NV = 0x100
    VK_MEMORY_PROPERTY_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkQueueFlagBits(IntEnum):
    VK_QUEUE_GRAPHICS_BIT = 0x1
    VK_QUEUE_COMPUTE_BIT = 0x2
    VK_QUEUE_TRANSFER_BIT = 0x4
    VK_QUEUE_SPARSE_BINDING_BIT = 0x8
    VK_QUEUE_PROTECTED_BIT = 0x10
    VK_QUEUE_VIDEO_DECODE_BIT_KHR = 0x20
    VK_QUEUE_OPTICAL_FLOW_BIT_NV = 0x100
    VK_QUEUE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class EgGeometryResource(ctypes.Structure):
    class List(IntEnum):
        GeometryElem = 0xC8
        GeoIndexCountI = 0xC9
        GeoVertexCountI = 0xCA
        GeoIndices16I = 0xCB
        GeoIndexStream = 0xCC
        GeoVertPosStream = 0xCD
        GeoVertTanStream = 0xCE
        GeoVertStaticStream = 0xCF
        GeoVertInverseBindMtxStream = 0xD0
        GeoBvPosStream = 0xD1


class EgAnimResource(ctypes.Structure):
    class List(IntEnum):
        EntityElem = 0x12C
        EntFrameCountI = 0x12D
        EntHas30HzFramesI = 0x12E


class VkQueryResultFlagBits(IntEnum):
    VK_QUERY_RESULT_64_BIT = 0x1
    VK_QUERY_RESULT_WAIT_BIT = 0x2
    VK_QUERY_RESULT_WITH_AVAILABILITY_BIT = 0x4
    VK_QUERY_RESULT_PARTIAL_BIT = 0x8
    VK_QUERY_RESULT_WITH_STATUS_BIT_KHR = 0x10
    VK_QUERY_RESULT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkBufferUsageFlagBits(IntEnum):
    VK_BUFFER_USAGE_TRANSFER_SRC_BIT = 0x1
    VK_BUFFER_USAGE_TRANSFER_DST_BIT = 0x2
    VK_BUFFER_USAGE_UNIFORM_TEXEL_BUFFER_BIT = 0x4
    VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT = 0x8
    VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT = 0x10
    VK_BUFFER_USAGE_STORAGE_BUFFER_BIT = 0x20
    VK_BUFFER_USAGE_INDEX_BUFFER_BIT = 0x40
    VK_BUFFER_USAGE_VERTEX_BUFFER_BIT = 0x80
    VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT = 0x100
    VK_BUFFER_USAGE_CONDITIONAL_RENDERING_BIT_EXT = 0x200
    VK_BUFFER_USAGE_SHADER_BINDING_TABLE_BIT_KHR = 0x400
    VK_BUFFER_USAGE_TRANSFORM_FEEDBACK_BUFFER_BIT_EXT = 0x800
    VK_BUFFER_USAGE_TRANSFORM_FEEDBACK_COUNTER_BUFFER_BIT_EXT = 0x1000
    VK_BUFFER_USAGE_VIDEO_DECODE_SRC_BIT_KHR = 0x2000
    VK_BUFFER_USAGE_VIDEO_DECODE_DST_BIT_KHR = 0x4000
    VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT = 0x20000
    VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_BUILD_INPUT_READ_ONLY_BIT_KHR = 0x80000
    VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR = 0x100000
    VK_BUFFER_USAGE_SAMPLER_DESCRIPTOR_BUFFER_BIT_EXT = 0x200000
    VK_BUFFER_USAGE_RESOURCE_DESCRIPTOR_BUFFER_BIT_EXT = 0x400000
    VK_BUFFER_USAGE_MICROMAP_BUILD_INPUT_READ_ONLY_BIT_EXT = 0x800000
    VK_BUFFER_USAGE_MICROMAP_STORAGE_BIT_EXT = 0x1000000
    VK_BUFFER_USAGE_PUSH_DESCRIPTORS_DESCRIPTOR_BUFFER_BIT_EXT = 0x4000000
    VK_BUFFER_USAGE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkQueryControlFlagBits(IntEnum):
    VK_QUERY_CONTROL_PRECISE_BIT = 0x1
    VK_QUERY_CONTROL_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class EgPipelineResource(ctypes.Structure):
    class List(IntEnum):
        StageElem = 0x384
        StageNameStr = 0x385
        StageNameHashI = 0x386
        StageActivationI = 0x387


class EgNodeTypes(ctypes.Structure):
    class List(IntEnum):
        Undefined = 0x0
        Group = 0x1
        Model = 0x2
        Mesh = 0x3
        Joint = 0x4
        Light = 0x5
        Camera = 0x6
        Emitter = 0x7
        Collision = 0x8
        Locator = 0x9


class eHmdSystemState(IntEnum):
    EHmdSystemState_Uninitialised = 0x0
    EHmdSystemState_Off = 0x1
    EHmdSystemState_Starting = 0x2
    EHmdSystemState_StartupSuccessful = 0x3
    EHmdSystemState_StartupFailed = 0x4
    EHmdSystemState_On = 0x5
    EHmdSystemState_NumTypes = 0x6

class eNPCNavGraphNodePhysProbeState(IntEnum):
    ENPCNavGraphNodePhysProbeState_Untested = 0x0
    ENPCNavGraphNodePhysProbeState_Clear = 0x1
    ENPCNavGraphNodePhysProbeState_Blocked = 0x2

class AkCallbackType(IntEnum):
    AK_EndOfEvent = 0x1
    AK_EndOfDynamicSequenceItem = 0x2
    AK_Marker = 0x4
    AK_Duration = 0x8
    AK_SpeakerVolumeMatrix = 0x10
    AK_Starvation = 0x20
    AK_MusicPlaylistSelect = 0x40
    AK_MusicPlayStarted = 0x80
    AK_MusicSyncBeat = 0x100
    AK_MusicSyncBar = 0x200
    AK_MusicSyncEntry = 0x400
    AK_MusicSyncExit = 0x800
    AK_MusicSyncGrid = 0x1000
    AK_MusicSyncUserCue = 0x2000
    AK_MusicSyncPoint = 0x4000
    AK_MusicSyncAll = 0x7F00
    AK_MIDIEvent = 0x10000
    AK_CallbackBits = 0xFFFFF
    AK_EnableGetSourcePlayPosition = 0x100000
    AK_EnableGetMusicPlayPosition = 0x200000
    AK_EnableGetSourceStreamBuffering = 0x400000

class eVertexElements(IntEnum):
    EVertexPosition = 0x0
    EVertexTexCoords = 0x1
    EVertexNormal = 0x2
    EVertexTangent = 0x3
    EVertexColor = 0x4
    EVertexJointIndices = 0x5
    EVertexJointWeights = 0x6
    EVertexParticleIndex = 0x7
    EVertexParticleCornerIndex = 0x8
    EVertexTransformR0 = 0x9
    EVertexTransformR1 = 0xA
    EVertexTransformR2 = 0xB
    EVertexTransformR3 = 0xC
    EVertexTransformR4 = 0xD
    EVertexTransformR5 = 0xE
    EVertexCustom1 = 0xF
    EVertexCustom2 = 0x10
    EVertexCustom3 = 0x11
    EVertexPosition3 = 0x12
    ENumVertexElements = 0x13

class eTkCameraTimeCurve(IntEnum):
    ETimeCurveLinear = 0x0
    ETimeCurveSmoothInOut = 0x1

class eTkCameraTimeMode(IntEnum):
    ETimeModeLocal = 0x0
    ETimeModeGlobal = 0x1

class e2dRendererShaderType(IntEnum):
    E2dShader_Untextured = 0x0
    E2dShader_Textured = 0x1
    E2dShader_Num = 0x2

class eSamplerToReplace(IntEnum):
    EReplaceSampler_Primary = 0x0
    EReplaceSampler_Decal = 0x1
    EReplaceSampler_Num = 0x2

class eTerrainEditShape(IntEnum):
    ETerrainEditShape_Sphere = 0x0
    ETerrainEditShape_Cube = 0x1
    ETerrainEditShape_Num = 0x2

class eRegionKnowledgeInterest(IntEnum):
    ERegionKnowledgeInterest_NothingInteresting = 0x0
    ERegionKnowledgeInterest_DeadEnd = 0x1
    ERegionKnowledgeInterest_Shelter = 0x2
    ERegionKnowledgeInterest_WaterEdge = 0x3
    ERegionKnowledgeInterest_WaterInlet = 0x4
    ERegionKnowledgeInterest_WaterShore = 0x5
    ERegionKnowledgeInterest_Forest = 0x6
    ERegionKnowledgeInterest_Meadow = 0x7
    ERegionKnowledgeInterest_MeadowEdge = 0x8
    ERegionKnowledgeInterest_Clearing = 0x9
    ERegionKnowledgeInterest_Building = 0xA
    ERegionKnowledgeInterest_PlayerBase = 0xB
    ERegionKnowledgeInterest_Settlement = 0xC
    ERegionKnowledgeInterest_Spaceship = 0xD
    ERegionKnowledgeInterest_Impassable = 0xE
    ERegionKnowledgeInterest_Count = 0xF

class ePrivilegeType(IntEnum):
    EPrivilegeType_TextChat = 0x0
    EPrivilegeType_PlayerDamage = 0x1
    EPrivilegeType_AddBaseParts = 0x2
    EPrivilegeType_DeleteBaseParts = 0x3
    EPrivilegeType_SeeShipMarker = 0x4
    EPrivilegeType_SeePlayerMarker = 0x5
    EPrivilegeType_FullSimLobby = 0x6
    EPrivilegeType_TerrainEditsInBase = 0x7
    EPrivilegeType_RefinerAccess = 0x8
    EPrivilegeType_NumTypes = 0x9

class ePrivilegeState(IntEnum):
    EPrivilegeState_Nobody = 0x0
    EPrivilegeState_FriendsOnly = 0x1
    EPrivilegeState_FriendsOrFireTeamOnly = 0x2
    EPrivilegeState_Anybody = 0x3
    EPrivilegeState_NumTypes = 0x4

class EModelResourceRendererSyncStage(IntEnum):
    EModelResourceRendererSyncStage_None = 0x0
    EModelResourceRendererSyncStage_RequestLoad = 0x1
    EModelResourceRendererSyncStage_WaitForHighestMips = 0x2
    EModelResourceRendererSyncStage_WaitForGeometryAndMaterials = 0x3
    EModelResourceRendererSyncStage_LoadResources = 0x4
    EModelResourceRendererSyncStage_RequestRenderToTexture = 0x5
    EModelResourceRendererSyncStage_RenderToTexture = 0x6
    EModelResourceRendererSyncStage_GetNodeBoundingBox = 0x7
    EModelResourceRendererSyncStage_UpdateCamera = 0x8
    EModelResourceRendererSyncStage_RenderToTextureDone = 0x9
    EModelResourceRendererSyncStage_RenderToTextureFailed = 0xA
    EModelResourceRendererSyncStage_RemoveNode = 0xB

class eGcMemoryType(IntEnum):
    ELastTkMemType = 0x11
    EMemoryPool_Global = 0x12
    EMemoryPool_Discovery = 0x13
    EMemoryPool_Local = 0x14
    EMemoryPool_SolarSystem = 0x15
    EMemoryPool_Voxels = 0x16
    EMemoryPool_Galactic = 0x17
    EMemoryPool_Asteroids = 0x18

class EModelResourceRendererViewMode(IntEnum):
    EModelResourceRendererViewMode_RotateX = 0x1
    EModelResourceRendererViewMode_RotateY = 0x2
    EModelResourceRendererViewMode_RotateZ = 0x4
    EModelResourceRendererViewMode_RotateXYZ = 0x7

class eClearMode(IntEnum):
    EClearMode_FromTextureRGB = 0x0
    EClearMode_FromTextureRGBWithClear = 0x1
    EClearMode_FromTextureRGBAWithClear = 0x2
    EClearMode_TransparentClear = 0x3
    EClearMode_None = 0x4
    EClearMode_NumModes = 0x5

class ePerformanceProfilePage(IntEnum):
    ePPPage_Overview = 0x0
    ePPPage_Regions = 0x1
    ePPPage_Entities = 0x2
    ePPPage_Render = 0x3
    ePPPage_Physics = 0x4
    ePPPage_AI = 0x5
    ePPPage_Particles = 0x6
    ePPPage_Audio = 0x7
    ePPPage_UI = 0x8
    ePPPage_Misc = 0x9
    ePPPage_Temp = 0xA
    ePPPage_GalaxyMap = 0xB
    ePPPage_Max = 0xC

class eRegionKnowledgeDirection(IntEnum):
    ERegionKnowledgeDirection_NegX = 0x0
    ERegionKnowledgeDirection_PosX = 0x1
    ERegionKnowledgeDirection_NegZ = 0x2
    ERegionKnowledgeDirection_PosZ = 0x3
    ERegionKnowledgeDirection_HorizontalCount = 0x4
    ERegionKnowledgeDirection_PosY = 0x5
    ERegionKnowledgeDirection_Count = 0x6

class eTkJobQueuePriority(IntEnum):
    eJobPriority_Max = 0x0
    eJobPriority_VeryHigh = 0x1
    eJobPriority_High = 0x2
    eJobPriority_Medium = 0x3
    eJobPriority_Low = 0x4
    eJobPriority_Min = 0x5

class eRegionKnowledgeType(IntEnum):
    ERegionKnowledgeType_Impassable = 0x1

class eFfxFsr2QualityLevel(IntEnum):
    EFfxFsr2QualityLevel_UltraPerformance = 0x0
    EFfxFsr2QualityLevel_Performance = 0x1
    EFfxFsr2QualityLevel_Balanced = 0x2
    EFfxFsr2QualityLevel_Quality = 0x3
    EFfxFsr2QualityLevel_MaxQuality = 0x4
    EFfxFsr2QualityLevel_Num = 0x5

class TestModes(ctypes.Structure):
    class List(IntEnum):
        Always = 0x0
        Equal = 0x1
        Less = 0x2
        LessEqual = 0x3
        Greater = 0x4
        GreaterEqual = 0x5
        NotEqual = 0x6


class StencilModes(ctypes.Structure):
    class List(IntEnum):
        Disabled = 0x0
        MaskWrite = 0x1
        MaskRead = 0x2
        MaskReadInvert = 0x3
        MaskWriteInside = 0x4
        MaskWriteOnce = 0x5
        MaskReadWriteNotEqual = 0x6
        MaskReadWriteInvert = 0x7
        MaskReadWrite = 0x8


class EgJoint(ctypes.Structure):
    class List(IntEnum):
        JointIndexI = 0x190


class TexAddressModes(ctypes.Structure):
    class List(IntEnum):
        Wrap = 0x0
        Clamp = 0x1
        ClampToBorder = 0x2
        Mirror = 0x3


class cGcUniverseAddressLayout(ctypes.Structure):
    class Bits(IntEnum):
        Bits_PlanetIndex = 0x4
        Bits_GalaxyVoxelY = 0x8
        Bits_GalaxyVoxelXZ = 0xC

    class Limits(IntEnum):
        Limits_PlanetIndex = 0xF
        Limits_GalaxyY_Max = 0x7F
        Limits_GalaxyVoxelY_Total = 0xFE
        Limits_RealityIteration = 0xFF
        Limits_GalaxyXZ_Max = 0x7FF
        Limits_GalaxyVoxelXZ_Total = 0xFFE
        Limits_SolarSystemsIndex = 0xFFF
        Limits_GalaxyXZ_Min = 0xFFFFF801
        Limits_GalaxyY_Min = 0xFFFFFF81

    class Masks(IntEnum):
        Mask_VoxelXYZ = 0xFFFFFFFF

    class Bitfield: pass


class eTkStencilMode(IntEnum):
    EStencilMode_Disabled = 0x0
    EStencilMode_MaskRead = 0x1
    EStencilMode_MaskReadInvert = 0x2
    EStencilMode_MaskWrite = 0x3
    EStencilMode_MaskWriteInside = 0x4
    EStencilMode_MaskWriteOnce = 0x5
    EStencilMode_MaskReadWriteNotEqual = 0x6
    EStencilMode_MaskReadWriteInvert = 0x7
    EStencilMode_MaskReadWrite = 0x8

class eTkCullMode(IntEnum):
    ECullMode_None = 0x0
    ECullMode_BackFaces = 0x1
    ECullMode_FrontFaces = 0x2
    ECullMode_Disable = 0x3
    ECullMode_Enable = 0x4

class eTkRenderFlags(IntEnum):
    ERenderFlag_ZWrite = 0x0
    ERenderFlag_AlphaToCoverage = 0x1
    ERenderFlag_ColourWrite = 0x2
    ERenderFlag_Num = 0x3

class eTkCheckerboard(IntEnum):
    ECheckerboard_Disabled = 0x0
    ECheckerboard_Even = 0x1
    ECheckerboard_Odd = 0x2

class eTkSamplingRate(IntEnum):
    ESamplingRate_PerPixel = 0x0
    ESamplingRate_PerSample = 0x1

class eTkStencilOp(IntEnum):
    EStencilOp_Keep = 0x0
    EStencilOp_Zero = 0x1
    EStencilOp_Replace = 0x2
    EStencilOp_IncrementClamp = 0x3
    EStencilOp_IncrementWrap = 0x4
    EStencilOp_DecrementClamp = 0x5
    EStencilOp_DecrementWrap = 0x6
    EStencilOp_Invert = 0x7
    EStencilOp_Num = 0x8

class cGcRegionFace(ctypes.Structure):
    class Enum(IntEnum):
        NegativeZ = 0x0
        PositiveZ = 0x1
        NegativeY = 0x2
        PositiveY = 0x3
        NegativeX = 0x4
        PositiveX = 0x5
        NumDirections = 0x6


class SolarSystemLimits(ctypes.Structure):
    class Enum(IntEnum):
        kiMaxNumSuns = 0x1
        kiMaxNumAsteroidFields = 0x3
        kiMaxNumPlanets = 0x6
        kiMaxNumSmallAsteroidFields = 0x8


class ePulseDriveState(IntEnum):
    EPulseDriveState_None = 0x0
    EPulseDriveState_Charge = 0x1
    EPulseDriveState_Jumping = 0x2
    EPulseDriveState_CrashStop = 0x3
    EPulseDriveState_Cooldown = 0x4

class eFadeState(IntEnum):
    EFade_In = 0x0
    EFade_Out = 0x1
    EFade_Visible = 0x2
    EFade_Hidden = 0x3

class cTkAnimCommand(ctypes.Structure):
    class eCommandType(IntEnum):
        ECommandType_Invalid = 0x0
        ECommandType_WriteNilPose = 0x1
        ECommandType_WriteBindPose = 0x2
        ECommandType_EvalFrame = 0x3
        ECommandType_MirrorPose = 0x4
        ECommandType_Blend = 0x5
        ECommandType_Subtract = 0x6
        ECommandType_Add = 0x7
        ECommandType_AddReversed = 0x8
        ECommandType_Multiply = 0x9
        ECommandType_NlerpMultiply = 0xA
        ECommandType_NlerpAdd = 0xB
        ECommandType_NlerpComplete = 0xC
        ECommandType_NumCommands = 0xD

    _fields_ = [
        ("_mCmd", ctypes.c_uint32),
        ("mfParam", ctypes.c_float),
        ("muAnim", ctypes.c_uint16),
    ]
    _mCmd: int
    mfParam: float
    muAnim: int

    @property
    def mCmd(self):
        return cTkAnimCommand.eCommandType(self._mCmd)


class eDebugRenderMode(IntEnum):
    EDebugRenderMode_Off = 0x0
    EDebugRenderMode_All = 0x1
    ENumDebugRenderModes = 0x2

class eAnimLayerState(IntEnum):
    EAnimLayer_Inactive = 0x0
    EAnimLayer_BlendIn = 0x1
    EAnimLayer_BlendOut = 0x2
    EAnimLayer_Active = 0x3
    ENumLayerStates = 0x4

class eComputeType(IntEnum):
    ECompute_None = 0x0
    ECompute_Sync = 0x1
    ECompute_AsyncThisFrame = 0x2
    ECompute_AsyncNextFrame = 0x3

class eTkPixelStoreParam(IntEnum):
    EPixelStoreParam_UnpackAlignment = 0x0
    EPixelStoreParam_UnpackRowLength = 0x1
    EPixelStoreParam_UnpackSkipPixels = 0x2
    EPixelStoreParam_UnpackSkipRows = 0x3
    EPixelStoreParam_PackAlignment = 0x4
    EPixelStoreParam_PackRowLength = 0x5
    EPixelStoreParam_PackSkipPixels = 0x6
    EPixelStoreParam_PackSkipRows = 0x7
    EPixelStoreParam_Num = 0x8

class eTkDepthTextureMode(IntEnum):
    EDepthTextureMode_Luminance = 0x0
    EDepthTextureMode_Intensity = 0x1
    EDepthTextureMode_Alpha = 0x2
    EDepthTextureMode_Num = 0x3

class GcPersistencyHandle(ctypes.Structure):
    def IsValid(): pass

    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $E48CAEC6893996E15941BAF25B28D3CD
    ]
    ___u0: bytes

class cTkPlane(ctypes.Structure):
    class ePlaneTestResult(IntEnum):
        ePR_None = 0x0
        ePR_Positive = 0x1
        ePR_Negative = 0x2

    def cTkPlane(): pass

    _fields_ = [
        ("mNormal", common.Vector3f),
        ("mfDistance", ctypes.c_float),
    ]
    mNormal: common.Vector3f
    mfDistance: float

class eIndexSize(IntEnum):
    EIS_16bit = 0x0
    EIS_32bit = 0x1

class AtlasRequestHandleConstants(IntEnum):
    InvalidAtlasRequestHandle = 0x0

class eTkFlipRate(IntEnum):
    EFlipRate_30hz = 0x0
    EFlipRate_60hz = 0x1
    EFlipRate_120hz = 0x2
    EFlipRate_60hz_120hzVsyncRate = 0x3
    EFlipRate_Undefined = 0x4

class eDestroyedBy(IntEnum):
    EDestroyedBy_Other = 0x0
    EDestroyedBy_Player = 0x1
    EDestroyedBy_Vehicle = 0x2
    EDestroyedBy_RemotePlayer = 0x3

class VkObjectType(IntEnum):
    VK_OBJECT_TYPE_UNKNOWN = 0x0
    VK_OBJECT_TYPE_INSTANCE = 0x1
    VK_OBJECT_TYPE_PHYSICAL_DEVICE = 0x2
    VK_OBJECT_TYPE_DEVICE = 0x3
    VK_OBJECT_TYPE_QUEUE = 0x4
    VK_OBJECT_TYPE_SEMAPHORE = 0x5
    VK_OBJECT_TYPE_COMMAND_BUFFER = 0x6
    VK_OBJECT_TYPE_FENCE = 0x7
    VK_OBJECT_TYPE_DEVICE_MEMORY = 0x8
    VK_OBJECT_TYPE_BUFFER = 0x9
    VK_OBJECT_TYPE_IMAGE = 0xA
    VK_OBJECT_TYPE_EVENT = 0xB
    VK_OBJECT_TYPE_QUERY_POOL = 0xC
    VK_OBJECT_TYPE_BUFFER_VIEW = 0xD
    VK_OBJECT_TYPE_IMAGE_VIEW = 0xE
    VK_OBJECT_TYPE_SHADER_MODULE = 0xF
    VK_OBJECT_TYPE_PIPELINE_CACHE = 0x10
    VK_OBJECT_TYPE_PIPELINE_LAYOUT = 0x11
    VK_OBJECT_TYPE_RENDER_PASS = 0x12
    VK_OBJECT_TYPE_PIPELINE = 0x13
    VK_OBJECT_TYPE_DESCRIPTOR_SET_LAYOUT = 0x14
    VK_OBJECT_TYPE_SAMPLER = 0x15
    VK_OBJECT_TYPE_DESCRIPTOR_POOL = 0x16
    VK_OBJECT_TYPE_DESCRIPTOR_SET = 0x17
    VK_OBJECT_TYPE_FRAMEBUFFER = 0x18
    VK_OBJECT_TYPE_COMMAND_POOL = 0x19
    VK_OBJECT_TYPE_SURFACE_KHR = 0x3B9ACA00
    VK_OBJECT_TYPE_SWAPCHAIN_KHR = 0x3B9ACDE8
    VK_OBJECT_TYPE_DISPLAY_KHR = 0x3B9AD1D0
    VK_OBJECT_TYPE_DISPLAY_MODE_KHR = 0x3B9AD1D1
    VK_OBJECT_TYPE_DEBUG_REPORT_CALLBACK_EXT = 0x3B9AF4F8
    VK_OBJECT_TYPE_VIDEO_SESSION_KHR = 0x3B9B23D8
    VK_OBJECT_TYPE_VIDEO_SESSION_PARAMETERS_KHR = 0x3B9B23D9
    VK_OBJECT_TYPE_CU_MODULE_NVX = 0x3B9B3B48
    VK_OBJECT_TYPE_CU_FUNCTION_NVX = 0x3B9B3B49
    VK_OBJECT_TYPE_DESCRIPTOR_UPDATE_TEMPLATE = 0x3B9C1608
    VK_OBJECT_TYPE_DEBUG_UTILS_MESSENGER_EXT = 0x3B9CBE00
    VK_OBJECT_TYPE_ACCELERATION_STRUCTURE_KHR = 0x3B9D13F0
    VK_OBJECT_TYPE_SAMPLER_YCBCR_CONVERSION = 0x3B9D2B60
    VK_OBJECT_TYPE_VALIDATION_CACHE_EXT = 0x3B9D3B00
    VK_OBJECT_TYPE_ACCELERATION_STRUCTURE_NV = 0x3B9D4E88
    VK_OBJECT_TYPE_PERFORMANCE_CONFIGURATION_INTEL = 0x3B9DFE50
    VK_OBJECT_TYPE_DEFERRED_OPERATION_KHR = 0x3B9EE0E0
    VK_OBJECT_TYPE_INDIRECT_COMMANDS_LAYOUT_NV = 0x3B9F0408
    VK_OBJECT_TYPE_PRIVATE_DATA_SLOT = 0x3B9F4A58
    VK_OBJECT_TYPE_BUFFER_COLLECTION_FUCHSIA = 0x3BA05FB0
    VK_OBJECT_TYPE_MICROMAP_EXT = 0x3BA0D4E0
    VK_OBJECT_TYPE_OPTICAL_FLOW_SESSION_NV = 0x3BA1DE80
    VK_OBJECT_TYPE_MAX_ENUM = 0x7FFFFFFF

class NVSDK_NGX_PerfQuality_Value(IntEnum):
    NVSDK_NGX_PerfQuality_Value_MaxPerf = 0x0
    NVSDK_NGX_PerfQuality_Value_Balanced = 0x1
    NVSDK_NGX_PerfQuality_Value_MaxQuality = 0x2
    NVSDK_NGX_PerfQuality_Value_UltraPerformance = 0x3
    NVSDK_NGX_PerfQuality_Value_UltraQuality = 0x4

class FFXSR_PerfQuality_Value(IntEnum):
    FFXSR_PerfQuality_Value_Off = 0x0
    FFXSR_PerfQuality_Value_UltraQuality = 0x1
    FFXSR_PerfQuality_Value_Quality = 0x2
    FFXSR_PerfQuality_Value_Balanced = 0x3
    FFXSR_PerfQuality_Value_Performance = 0x4

class VkDescriptorType(IntEnum):
    VK_DESCRIPTOR_TYPE_SAMPLER = 0x0
    VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER = 0x1
    VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE = 0x2
    VK_DESCRIPTOR_TYPE_STORAGE_IMAGE = 0x3
    VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER = 0x4
    VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER = 0x5
    VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER = 0x6
    VK_DESCRIPTOR_TYPE_STORAGE_BUFFER = 0x7
    VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC = 0x8
    VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC = 0x9
    VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT = 0xA
    VK_DESCRIPTOR_TYPE_INLINE_UNIFORM_BLOCK = 0x3B9CE510
    VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR = 0x3B9D13F0
    VK_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_NV = 0x3B9D4E88
    VK_DESCRIPTOR_TYPE_MUTABLE_EXT = 0x3BA02518
    VK_DESCRIPTOR_TYPE_SAMPLE_WEIGHT_IMAGE_QCOM = 0x3BA180C0
    VK_DESCRIPTOR_TYPE_BLOCK_MATCH_IMAGE_QCOM = 0x3BA180C1
    VK_DESCRIPTOR_TYPE_MAX_ENUM = 0x7FFFFFFF

class cTkAnimationBlend(ctypes.Structure):
    class eBlendType(IntEnum):
        EBlendType_Blend = 0x1
        EVectorBlendBit = 0x4
        EBlendType_VectorBlend = 0x5
        EBlendType_VectorAdd = 0x6
        E2dBlendBit = 0x8
        EBlendType_2dBlend = 0x9
        EBlendType_2dAdd = 0xA

    def UpdateWeights(): pass

    _fields_ = [
        ("mIndices", ctypes.c_char * 0x38),  # cTkStackVector<short,4,-1>
        ("miNumWeights", ctypes.c_int16),
        ("miBlendIndex", ctypes.c_int16),
        ("_mBlendType", ctypes.c_uint32),
    ]
    mIndices: bytes
    miNumWeights: int
    miBlendIndex: int
    _mBlendType: int

    @property
    def mBlendType(self):
        return cTkAnimationBlend.eBlendType(self._mBlendType)


class eNGuiGameGraphicType(IntEnum):
    eGamENGuiEditorGraphic_Panel = 0x0
    eGameGraphic = 0x1
    eGamENGuiEditorGraphic_TextBackground = 0x2
    eGamENGuiEditorGraphic_NumTypes = 0x3

class eNGuiGameTextType(IntEnum):
    eGameText = 0x0
    eGamENGuiEditorText_NumTypes = 0x1

class eNGuiGameElementType(IntEnum):
    eGuiElement_Layer = 0x0
    eGuiElement_Text = 0x1
    eGuiElement_Text_Special = 0x2
    eGuiElement_Graphic = 0x3
    eGuiElement_Spacing = 0x4

class eQuickMenuSlot(IntEnum):
    EQuickMenuSlot_Left4 = 0x0
    EQuickMenuSlot_Left3 = 0x1
    EQuickMenuSlot_Left2 = 0x2
    EQuickMenuSlot_Left1 = 0x3
    EQuickMenuSlot_Main = 0x4
    EQuickMenuSlot_Right1 = 0x5
    EQuickMenuSlot_Right2 = 0x6
    EQuickMenuSlot_Right3 = 0x7
    EQuickMenuSlot_Right4 = 0x8
    EQuickMenuSlot_TopLeft3 = 0x9
    EQuickMenuSlot_TopLeft2 = 0xA
    EQuickMenuSlot_TopLeft1 = 0xB
    EQuickMenuSlot_TopMain = 0xC
    EQuickMenuSlot_TopRight1 = 0xD
    EQuickMenuSlot_TopRight2 = 0xE
    EQuickMenuSlot_TopRight3 = 0xF
    EQuickMenuSlot_TotalNumSlots = 0x10
    EQuickMenuSlot_None = 0xFFFFFFFF

class eModeSelectPreset(IntEnum):
    EModeSelectPreset_Unspecified = 0x0
    EModeSelectPreset_Normal = 0x1
    EModeSelectPreset_Creative = 0x2
    EModeSelectPreset_Relaxed = 0x3
    EModeSelectPreset_Survival = 0x4
    EModeSelectPreset_Custom = 0x5
    EModeSelectPreset_Seasonal = 0x6
    EModeSelectPreset_NumTypes = 0x7

class eGcTrackArrowTypes(IntEnum):
    ETrackArrow_WantedCombat = 0x0
    ETrackArrow_WantedHunting = 0x1
    ETrackArrow_Investigate = 0x2
    ETrackArrow_Predator = 0x3
    ETrackArrow_HazardHot = 0x4
    ETrackArrow_HazardCold = 0x5
    ETrackArrow_HazardRadiation = 0x6
    ETrackArrow_HazardToxic = 0x7
    ETrackArrow_ShipsArriving = 0x8
    ETrackArrow_PiratesArriving = 0x9
    ETrackArrow_PoliceArriving = 0xA
    ETrackArrow_Ship = 0xB
    ETrackArrow_PrimaryShip = 0xC
    ETrackArrow_SecondaryShip = 0xD
    ETrackArrow_Freighter = 0xE
    ETrackArrow_Missile = 0xF
    ETrackArrow_Building = 0x10
    ETrackArrow_Friend = 0x11
    ETrackArrow_FriendScared = 0x12
    ETrackArrow_FriendSearching = 0x13
    ETrackArrow_NetworkPlayer = 0x14
    ETrackArrow_Fiend = 0x15
    ETrackArrow_Repair = 0x16
    ETrackArrow_Summoning = 0x17
    ETrackArrow_NumTypes = 0x18

class eTkTextureReductionMode(IntEnum):
    ETextureReductionMode_Average = 0x0
    ETextureReductionMode_Min = 0x1
    ETextureReductionMode_Max = 0x2
    ETextureReductionMode_Num = 0x3

class eSMShaderConnType(IntEnum):
    eShaderConnType_Invalid = 0x0
    eShaderConnType_Vector = 0x1
    eShaderConnType_Colour = 0x2
    eShaderConnType_Masks = 0x3
    eShaderConnType_HSV = 0x4
    eShaderConnType_Biome = 0x5
    eShaderConnType_NumTypes = 0x6

class eTkComponentPriority(IntEnum):
    EPriority_Rotation = 0x5
    EPriority_Physics = 0xA
    EPriority_Animation = 0xF
    EPriority_Components = 0x14

class eTkComponentMask(IntEnum):
    EComponentMask_AllowNothing = 0x0
    EComponentMask_AllowAll = 0x1

class cGcDiscoveryPayload(ctypes.Structure):
    class Constants(IntEnum):
        MaxElements = 0x5

    def cGcDiscoveryPayload(): pass

    _fields_ = [
        ("kuSignificantElements", ctypes.c_uint32),
        ("mau64Storage", ctypes.c_char * 0x28),  # std::array<unsigned __int64,5>
        ("muNumElements", ctypes.c_uint32),
    ]
    kuSignificantElements: int
    mau64Storage: bytes
    muNumElements: int

class eWikiPageLayout(IntEnum):
    EWikiPageLayout_List = 0x0
    EWikiPageLayout_Grid = 0x1
    EWikiPageLayout_GridMedium = 0x2
    EWikiPageLayout_GridLarge = 0x3
    EWikiPageLayout_NumTypes = 0x4

class eSteeringInput(IntEnum):
    ESteeringInput_Misc = 0x0
    ESteeringInput_Hysteresis = 0x1
    ESteeringInput_Follow = 0x2
    ESteeringInput_SwarmAlign = 0x3
    ESteeringInput_SwarmCohere = 0x4
    ESteeringInput_SwarmSeparate = 0x5
    ESteeringInput_AvoidCreatures = 0x6
    ESteeringInput_AvoidDanger = 0x7
    ESteeringInput_AdjustHeight = 0x8
    ESteeringInput_AvoidTurning = 0x9
    ESteeringInput_StayUnderwater = 0xA
    ESteeringInput_FaceTarget = 0xB
    ENumSteeringInputs = 0xC

class cGcCreatureSteering(ctypes.Structure):
    def GetMaxForwardInverseDanger(): pass

    def AddDanger(): pass

    def AddEvadeDanger(): pass

    def AddInterest(): pass

    def GetDirection(): pass

    def GetDirection_(): pass

    def PickDirection_(): pass

    def UpdateSpring(): pass

    def Zero(): pass

    def Construct(): pass

    def AddImpassable(): pass

    def RenderDebug(): pass

    _fields_ = [
        ("maDirections", common.Vector3f * 0x24),  # cTkVector3[36]
        ("mafInterest", ctypes.c_float * 0x24),  # float[36]
        ("mafDanger", ctypes.c_float * 0x24),  # float[36]
        ("mafImpassable", ctypes.c_float * 0x24),  # float[36]
        ("mafSteeringWeights", ctypes.c_float * 0xC),  # float[12]
        ("mafRecipTurnRadius", ctypes.c_float * 0x24),  # float[36]
        ("mvCurDirection", common.Vector3f),
        ("mvVelDirection", common.Vector3f),
        ("mfTimeSinceLastUpdate", ctypes.c_float),
        ("mfFrameTick", ctypes.c_float),
        ("mfSpringTime", ctypes.c_float),
        ("miNumDirections", ctypes.c_int32),
        ("mfDangerThreshold", ctypes.c_float),
        ("mbIsUpdateFrame", ctypes.c_ubyte),
    ]
    maDirections: list[common.Vector3f]
    mafInterest: list[ctypes.c_float]
    mafDanger: list[ctypes.c_float]
    mafImpassable: list[ctypes.c_float]
    mafSteeringWeights: list[ctypes.c_float]
    mafRecipTurnRadius: list[ctypes.c_float]
    mvCurDirection: common.Vector3f
    mvVelDirection: common.Vector3f
    mfTimeSinceLastUpdate: float
    mfFrameTick: float
    mfSpringTime: float
    miNumDirections: int
    mfDangerThreshold: float
    mbIsUpdateFrame: bool

class eCameraBehaviours(IntEnum):
    EBehaviourFly = 0x0
    EBehaviourInterpolate = 0x1
    EBehaviourOffset = 0x2
    EBehaviourCharacter = 0x3
    EBehaviourFirstPerson = 0x4
    EBehaviourVehicleThirdPerson = 0x5
    EBehaviourPlayerThirdPerson = 0x6
    EBehaviourGalacticTransition = 0x7
    EBehaviourGalacticNavigation = 0x8
    EBehaviourGalacticLookAt = 0x9
    EBehaviourInteraction = 0xA
    EBehaviourLookAt = 0xB
    EBehaviourAerialView = 0xC
    EBehaviourScreenshot = 0xD
    EBehaviourPhotoMode = 0xE
    EBehaviourAmbient = 0xF
    EBehaviourModelView = 0x10
    EBehaviourAnimated = 0x11
    EBehaviourFollowTarget = 0x12
    EBehaviourShipWarp = 0x13
    EBehaviourBuildingMode = 0x14
    EBehaviourFreighterWarp = 0x15
    EBehaviour_Num = 0x16

class eUserGeneratedContentStatus(IntEnum):
    EUserGeneratedContentStatus_LocalOnly = 0x0
    EUserGeneratedContentStatus_Uploading = 0x1
    EUserGeneratedContentStatus_Synchronised = 0x2
    eUserGeneratedContentStatus_Downloading = 0x3
    EUserGeneratedContentStatus_Deleted = 0xFFFFFFFF

class eGenerationSeed(IntEnum):
    EGenerationSeed_Basics = 0x0
    EGenerationSeed_Inhabitants = 0x1
    EGenerationSeed_ColourPalettes = 0x2
    EGenerationSeed_Buildings = 0x3
    EGenerationSeed_Terrain = 0x4
    EGenerationSeed_TerrainColours = 0x5
    EGenerationSeed_TerrainTextures = 0x6
    EGenerationSeed_ExternalObjectLists = 0x7
    EGenerationSeed_SpawnData = 0x8
    EGenerationSeed_SelectableObjectData = 0x9
    EGenerationSeed_CreatureRoles = 0xA
    EGenerationSeed_CreatureSpawnData = 0xB
    EGenerationSeed_Weather = 0xC
    EGenerationSeed_Sky = 0xD
    EGenerationSeed_Fog = 0xE
    EGenerationSeed_Clouds = 0xF
    EGenerationSeed_Lighting = 0x10
    EGenerationSeed_Water = 0x11
    EGenerationSeed_Hazards = 0x12
    EGenerationSeed_Substances = 0x13
    EGenerationSeed_Chemistry = 0x14
    EGenerationSeed_Info = 0x15
    EGenerationSeed_Sentinels = 0x16
    EGenerationSeed_Trading = 0x17
    EGenerationSeed_Conflict = 0x18
    EGenerationSeed_Rings = 0x19
    EGenerationSeed_Num = 0x1A

class eEditPlaneState(IntEnum):
    EEditPlaneState_Disabled = 0x0
    EEditPlaneState_LockedOnPlane = 0x1
    EEditPlaneState_LockedInFrontOfPlane = 0x2

class DiscoveryUtils(ctypes.Structure):
    class DiscoveryKeys(IntEnum):
        DK_SolarSystem_Magic = 0x20D4608B

    def PopulateDiscoveryDataFromNode(): pass

    def InitialiseResourceDescriptorFromDiscoveryData(): pass

    def DoesPlanetGeneratedCreatureMatchDiscoveryData(): pass

    def AssembleDiscoveryDataForSolarSystem(): pass


class eRegionStatus(IntEnum):
    EStatus_Unassigned = 0x0
    EStatus_Empty = 0x1
    EStatus_RequiredForGeneration = 0x2
    EStatus_Generating = 0x3
    EStatus_Generated = 0x4
    EStatus_MapStreams = 0x5
    EStatus_CheckResourcesLoaded = 0x6
    EStatus_ReadyToPolygonise = 0x7
    EStatus_Polygonising = 0x8
    EStatus_Polygonised = 0x9
    EStatus_ReadyToPostPolygonise = 0xA
    EStatus_PostPolygonised = 0xB
    EStatus_ReadyToPopulate = 0xC
    EStatus_ReadyToBuildKnowledge = 0xD
    EStatus_KnowledgeBuilding = 0xE
    EStatus_KnowledgeBuilt = 0xF
    EStatus_Completed = 0x10
    EStatus_Clearing = 0x11
    EStatus_NumStatus = 0x12

class eInputAxisBasis(IntEnum):
    EInputAxisBasis_None = 0x0
    EInputAxisBasis_Scalar = 0x1
    EInputAxisBasis_Vector1 = 0x2
    EInputAxisBasis_Vector2 = 0x3

class eInputDimension(IntEnum):
    EInputDimension_X = 0x0
    EInputDimension_Y = 0x1
    EInputDimension_Z = 0x2
    EInputDimension_None = 0xFFFFFFFF

class RealityConstants(ctypes.Structure):
    pass

class eCharacterPhysicsDisabledReasons(IntEnum):
    ECharacterPhysicsDisabledReason_None = 0x0
    ECharacterPhysicsDisabledReason_MovementDisabled = 0x1
    ECharacterPhysicsDisabledReason_Sitting = 0x2
    ECharacterPhysicsDisabledReason_Riding = 0x4
    ECharacterPhysicsDisabledReason_ClimbingLadder = 0x8
    ECharacterPhysicsDisabledReason_NavFallback = 0x10
    ECharacterPhysicsDisabledReason_Replicated = 0x20
    ECharacterPhysicsDisabledReason_Any = 0x3F

class eEnergyDischargeRate(IntEnum):
    EEnergyDischarge_Low = 0x0
    EEnergyDischarge_Medium = 0x1
    EEnergyDischarge_High = 0x2
    EEnergyDischarge_NumStates = 0x3
    EEnergyDischarge_Inactive = 0xFFFFFFFF

class eAsyncResult(IntEnum):
    EAS_IssuedRead = 0x0
    EAS_ReadAlreadyIssued = 0x1
    EAS_DataIsReady = 0x2
    EAS_Failed = 0x3

class eEmoteState(IntEnum):
    EEmoteState_None = 0x0
    EEmoteState_Playing = 0x1
    EEmoteState_Playing_Sit = 0x2

class eTransferCategory(IntEnum):
    ETransferCategory_Suit = 0x0
    ETransferCategory_Ship = 0x1
    ETransferCategory_Freighter = 0x2
    ETransferCategory_Vehicle = 0x3
    ETransferCategory_Base = 0x4
    ETransferCategory_CookingIngredients = 0x5
    ETransferCategory_NumTypes = 0x6

class eFilterSlot(IntEnum):
    EFilterSlot_Base = 0x0
    EFilterSlot_Far = 0x1
    EFilterSlot_Storm = 0x2
    EFilterSlot_Effect = 0x3
    EFilterSlot_Num = 0x4
    EFilterSlot_Invalid = 0xFFFFFFFF

class eSpaceSkySettingsList(IntEnum):
    ESpaceSkySettingsList_Common = 0x0
    ESpaceSkySettingsList_Rare = 0x1
    ESpaceSkySettingsList_Num = 0x2

class cGcIdleAnimList(ctypes.Structure):
    def BuildIdleList(): pass

    def GetRandomIdle(): pass

    def OkayToLook(): pass

    _fields_ = [
        ("mCurrentIdle", ctypes.c_ulonglong),
        ("maIdles", ctypes.c_ulonglong * 0xA),  # TkID<128>[10]
        ("miNumIdles", ctypes.c_int32),
        ("maIdlesShuffled", ctypes.c_ulonglong * 0xA),  # TkID<128>[10]
        ("miNumIdlesShuffledRemaining", ctypes.c_int32),
        ("mbHasNonEatIdles", ctypes.c_ubyte),
    ]
    mCurrentIdle: int
    maIdles: list[ctypes.c_ulonglong]
    miNumIdles: int
    maIdlesShuffled: list[ctypes.c_ulonglong]
    miNumIdlesShuffledRemaining: int
    mbHasNonEatIdles: bool

class eGroupNode(IntEnum):
    EGroupNode_Player = 0x0
    EGroupNode_Sky = 0x1
    EGroupNode_Scan = 0x2
    EGroupNode_Effects = 0x3
    EGroupNode_Objects = 0x4
    EGroupNode_Creatures = 0x5
    EGroupNode_NetPending = 0x6
    EGroupNode_RegionDecorator = 0x7
    EGroupNode_Terrain = 0x8
    EGroupNode_Space = 0x9
    EGroupNode_FakeSpace = 0xA
    EGroupNode_Ships = 0xB
    EGroupNode_RemotePlayers = 0xC
    ENumGroupNodes = 0xD

class cGcDiscoveryPageDataSearchTask(ctypes.Structure):
    class eTaskType(IntEnum):
        ESearchTaskType_Invalid = 0x0
        ESearchTaskType_SolarSystem = 0x1
        ESearchTaskType_Planets = 0x2
        ESearchTaskType_PlanetData = 0x3

    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpSearch", ctypes.c_ulonglong),
        ("mDiscoveryData", ctypes.c_ulonglong),
        ("_meTaskType", ctypes.c_uint32),
        ("mbComplete", ctypes.c_ubyte),
    ]
    __vftable: int
    mpSearch: int
    mDiscoveryData: int
    _meTaskType: int
    mbComplete: bool

    @property
    def meTaskType(self):
        return cGcDiscoveryPageDataSearchTask.eTaskType(self._meTaskType)


class ScannedNode(ctypes.Structure):
    class eDressRequest(IntEnum):
        NeedsDressing = 0x0
        NeedsUniqueSmartDressing = 0x1
        NoDressingNeeded = 0x2

    _fields_ = [
        ("mScannedNode", ctypes.c_uint32),
        ("mColour", common.Colour),
        ("mScanColourCallback", ctypes.c_ulonglong),
        ("maScanValueCallbacks", ctypes.c_ulonglong * 0x4),  # float (__fastcall *[4])(const ScannedNode *)
        ("mfActiveTime", ctypes.c_float),
        ("mfLifeTime", ctypes.c_float),
        ("_meDressRequest", ctypes.c_uint32),
        ("mpData", ctypes.c_ulonglong),
    ]
    mScannedNode: int
    mColour: common.Colour
    mScanColourCallback: int
    maScanValueCallbacks: list[ctypes.c_ulonglong]
    mfActiveTime: float
    mfLifeTime: float
    _meDressRequest: int
    mpData: int

    @property
    def meDressRequest(self):
        return ScannedNode.eDressRequest(self._meDressRequest)


class cGcDiscoveryConstants(ctypes.Structure):
    pass

class eTerrainAtlasTextures(IntEnum):
    ETerrainAtlas_Base_Slope = 0x0
    ETerrainAtlas_Base = 0x1
    ETerrainAtlas_BaseAlt_Slope = 0x2
    ETerrainAtlas_BaseAlt = 0x3
    ETerrainAtlas_Mount_Slope = 0x4
    ETerrainAtlas_Mount = 0x5
    ETerrainAtlas_MountAlt_Slope = 0x6
    ETerrainAtlas_MountAlt = 0x7
    ETerrainAtlas_Detail_Base_Slope = 0x8
    ETerrainAtlas_Detail_Base = 0x9
    ETerrainAtlas_Detail_BaseAlt_Slope = 0xA
    ETerrainAtlas_Detail_BaseAlt = 0xB
    ETerrainAtlas_Detail_Mount_Slope = 0xC
    ETerrainAtlas_Detail_Mount = 0xD
    ETerrainAtlas_Detail_MountAlt_Slope = 0xE
    ETerrainAtlas_Detail_MountAlt = 0xF
    ETerrainAtlas_Num = 0x10

class eSubstanceAtlasTextures(IntEnum):
    ESubstanceAtlas_Cave = 0x0
    ESubstanceAtlas_CaveSlope = 0x1
    ESubstanceAtlas_Sand = 0x2
    ESubstanceAtlas_SandSlope = 0x3
    ESubstanceAtlas_Substance1 = 0x4
    ESubstanceAtlas_Substance2 = 0x5
    ESubstanceAtlas_Substance3 = 0x6
    ESubstanceAtlas_Num = 0x7

class ePlayerEffects(IntEnum):
    EPlayerEffects_None = 0x0
    EPlayerEffects_Enter_Vehicle = 0x1
    EPlayerEffects_Exit_Vehicle = 0x2
    EPlayerEffects_Size = 0x3
    EPlayerEffects_EnableCamo = 0x4
    EPlayerEffects_DisableCamo = 0x5
    EPlayerEffects_MiniPortalEnter = 0x6
    EPlayerEffects_MiniPortalExit = 0x7

class eHeightTestAccuracy(IntEnum):
    EHeightTest_Basic = 0x0
    EHeightTest_Accurate = 0x1
    EHeightTest_AccurateFromHeight = 0x2
    EHeightTest_AccurateFromHeightIncludingStructures = 0x3
    EHeightTest_PhysicsOnly = 0x4
    EHeightTest_Knowledge = 0x5

class ePlanetTransitionState(IntEnum):
    EPlanetTransition_None = 0x0
    EPlanetTransition_InSpace = 0x1
    EPlanetTransition_InSpaceNearPlanet = 0x2
    EPlanetTransition_OnPlanet = 0x3
    EPlanetTransition_StartLeavingPlanet = 0x4
    EPlanetTransition_FinishLeavingPlanet = 0x5
    EPlanetTransition_StartEnteringPlanet = 0x6
    EPlanetTransition_FinishEnteringPlanet = 0x7
    EPlanetTransition_InTransition = 0x8
    EPlanetTransition_AerialViewTransition = 0x9
    EPlanetTransition_AmbientLoad = 0xA
    EPlanetTransition_Num = 0xB

class ePlanetLoadRequest(IntEnum):
    EPlanetLoadRequest_Objects = 0x0
    EPlanetLoadRequest_Creatures = 0x1
    EPlanetLoadRequest_Num = 0x2

class eAsteroidType(IntEnum):
    EAsteroidType_None = 0x0
    EAsteroidType_Rare = 0x1
    EAsteroidType_Common = 0x2
    EAsteroidType_Large = 0x3
    EAsteroidType_Ring = 0x4
    EAsteroidType_Fuel = 0x5
    EAsteroidType_NumTypes = 0x6

class eInterest(IntEnum):
    EInterest_Ships = 0x0
    EInterest_Robots = 0x1
    EInterest_SpacePirates = 0x2
    EInterest_PlanetPirates = 0x3
    EInterest_Pulse_Encounter = 0x4
    EInterest_SandWorm = 0x5
    EInterest_Frigates = 0x6
    EInterest_Police = 0x7
    EInterest_BackgroundSpace_Encounter = 0x8
    EInterest_NumTypes = 0x9

class ePoliceEntrance(IntEnum):
    EPoliceEntrance_None = 0x0
    EPoliceEntrance_Start = 0x1
    EPoliceEntrance_Probing = 0x2
    EPoliceEntrance_Incoming = 0x3
    EPoliceEntrance_Arrived = 0x4
    EPoliceEntrance_Escalate = 0x5
    EPoliceEntrance_EscalateProbing = 0x6
    EPoliceEntrance_EscalateIncoming = 0x7
    EPoliceEntrance_CargoScanStart = 0x8
    EPoliceEntrance_CargoScanProbing = 0x9
    EPoliceEntrance_CargoScanAttemptContact = 0xA
    EPoliceEntrance_CargoScanInComms = 0xB
    EPoliceEntrance_CargoScanStartAttack = 0xC

class ePirateSpawn(IntEnum):
    EPirateSpawn_CargoAttackStart = 0x0
    EPirateSpawn_Probing = 0x1
    EPirateSpawn_ProbeHail = 0x2
    EPirateSpawn_ProbeSuccess = 0x3
    EPirateSpawn_ProbeSuccessSilent = 0x4
    EPirateSpawn_AttackWarning = 0x5
    EPirateSpawn_Attacking = 0x6
    EPirateSpawn_FreighterBattleStart = 0x7
    EPirateSpawn_BattleMarker = 0x8
    EPirateSpawn_Battling = 0x9
    EPirateSpawn_BattleJoined = 0xA
    EPirateSpawn_BattleEscaped = 0xB
    EPirateSpawn_BountyStart = 0xC
    EPirateSpawn_BountyPrepare = 0xD
    EPirateSpawn_BountyActive = 0xE
    EPirateSpawn_BountyAttacking = 0xF
    EPirateSpawn_BountyFleeing = 0x10
    EPirateSpawn_PlanetaryRaidStart = 0x11
    EPirateSpawn_PlanetaryRaidApproaching = 0x12
    EPirateSpawn_PlanetaryRaidAttacking = 0x13
    EPirateSpawn_PlanetaryRaidFlee = 0x14
    EPirateSpawn_PlanetaryFlyByStart = 0x15
    EPirateSpawn_PlanetaryFlyByApproach = 0x16
    EPirateSpawn_PlanetaryFlyByAttacking = 0x17
    EPirateSpawn_PlanetaryFlyByFlee = 0x18
    EPirateSpawn_NumTypes = 0x19
    EPirateSpawn_None = 0xFFFFFFFF

class eNavPolySearchDirection(IntEnum):
    UpOrDown = 0x0
    DownOnly = 0x1

class ePulseEncounterState(IntEnum):
    EPulseEncounterState_Prepare = 0x0
    EPulseEncounterState_Waiting_To_Start = 0x1
    EPulseEncounterState_Waiting_To_Exit_Pulse = 0x2
    EPulseEncounterState_Loading = 0x3
    EPulseEncounterState_Spawning = 0x4
    EPulseEncounterState_Active = 0x5
    EPulseEncounterState_CleanUp = 0x6
    EPulseEncounterState_NumTypes = 0x7
    EPulseEncounterState_None = 0xFFFFFFFF

class ePopupType(IntEnum):
    EPopupType_Chemical = 0x0
    EPopupType_Technology = 0x1
    EPopupType_Discovery = 0x2
    EPopupType_DiscoveryItem = 0x3
    EPopupType_DiscoverySystemInfo = 0x4
    EPopupType_Empty = 0x5
    EPopupType_RepairSlot = 0x6
    EPopupType_Mission = 0x7
    EPopupType_Wiki = 0x8
    EPopupType_NetworkPlayer = 0x9
    EPopupType_ByteBeat = 0xA
    EPopupType_ByteBeatToolTip = 0xB
    EPopupType_ByteBeatLibrary = 0xC
    EPopupType_Pet = 0xD
    EPopupType_SeasonMilestone = 0xE
    EPopupType_BuildMenuItem = 0xF
    EPopupType_SquadronPilot = 0x10
    EPopupType_Build = 0x11
    EPopupType_Charge = 0x12
    EPopupType_Transfer = 0x13
    EPopupType_TransferSend = 0x14
    EPopupType_BuySell = 0x15
    EPopupType_BuySellTech = 0x16
    EPopupType_Outfitting = 0x17
    EPopupType_ByteBeatSend = 0x18
    EPopupType_NumTypes = 0x19

class ePopupAction(IntEnum):
    EPopupAction_Transfer = 0x0
    EPopupAction_TransferSend = 0x1
    EPopupAction_BuildTech = 0x2
    EPopupAction_BuildProduct = 0x3
    EPopupAction_TechCharge = 0x4
    EPopupAction_Repair = 0x5
    EPopupAction_Outfitting = 0x6
    EPopupAction_ByteBeatSend = 0x7
    EPopupAction_NumTypes = 0x8
    EPopupAction_None = 0xFFFFFFFF

class ePopupScreenAdjust(IntEnum):
    EPopupScreenAdjust_Default = 0x0
    EPopupScreenAdjust_AlwaysFit = 0x1
    EPopupScreenAdjust_Tooltip = 0x2

class eRecordMode(IntEnum):
    ERecordMode_None = 0x0
    ERecordMode_Record = 0x1
    ERecordMode_Playback = 0x2

class eTkPrimitiveType(IntEnum):
    EPrimitiveType_Lines = 0x1
    EPrimitiveType_Tris = 0x3
    EPrimitiveType_TriStrip = 0x4
    EPrimitiveType_TriFan = 0x5
    EPrimitiveType_Undefined = 0xFFFFFFFF

class eTkPhysicsDebugRenderFlags(IntEnum):
    EPhysicsDebugRender_Wireframe = 0x1
    EPhysicsDebugRender_Constraints = 0x2
    EPhysicsDebugRender_Contacts = 0x3
    EPhysicsDebugRender_Aabbs = 0x4
    EPhysicsDebugRender_Terrain = 0x5
    EPhysicsDebugRender_Instances = 0x6
    EPhysicsDebugRender_Objects = 0x7
    EPhysicsDebugRender_DisableCull = 0x8
    EPhysicsDebugRender_DepthTestDisable = 0x9

class eWFCBuildingStage(IntEnum):
    EWFCBuildingStage_Layout = 0x0
    EWFCBuildingStage_Level0 = 0x1
    EWFCBuildingStage_Level1 = 0x2
    EWFCBuildingStage_HigherLevels = 0x3
    EWFCBuildingStage_Decoration = 0x4
    EWFCBuildingStage_NumTypes = 0x5

class eTkCollisionGroup(IntEnum):
    ETkCollisionGroup_Normal = 0x1

class HavokUserFlags(IntEnum):
    eTkHavokDisableDebugRendering = 0x4000000

class eTkQualityType(IntEnum):
    eQualityType_Debris = 0x0
    eQualityType_Default = 0x1
    eQualityType_Critical = 0x2
    eQualityType_Character = 0x3

class eTkRigidBodyGameplayFlags(IntEnum):
    ERigidBodyGameplayFlags_None = 0x0
    ERigidBodyGameplayFlags_PlayerRiddenCreature = 0x1

class eNavAreaUpdateStage(IntEnum):
    ENavAreaUpdateStage_WaitToStart = 0x0
    ENavAreaUpdateStage_GatherLocators = 0x1
    ENavAreaUpdateStage_CalculateNodeFloorPositions = 0x2
    ENavAreaUpdateStage_GenerateGraph = 0x3
    ENavAreaUpdateStage_ValidateLinks = 0x4
    ENavAreaUpdateStage_SimplifyGraph = 0x5
    ENavAreaUpdateStage_CleanGraph = 0x6
    ENavAreaUpdateStage_ReadyForNavigation = 0x7
    ENavAreaUpdateStage_NumStages = 0x8

class eBlockRotation(IntEnum):
    EBlockRotation_0 = 0x0
    EBlockRotation_90 = 0x1
    EBlockRotation_180 = 0x2
    EBlockRotation_270 = 0x3

class eBlockDirection(IntEnum):
    EBlockDirection_Left = 0x0
    EBlockDirection_Down = 0x1
    EBlockDirection_Back = 0x2
    EBlockDirection_Right = 0x3
    EBlockDirection_Up = 0x4
    EBlockDirection_Forward = 0x5

class eSentinelCrimeResponse(IntEnum):
    ESentinelCrimeResponse_Respond = 0x0
    ESentinelCrimeResponse_Ignore = 0x1

class eNPCNavGraphNodeType(IntEnum):
    ENPCNavGraphNodeType_NavNode = 0x0
    ENPCNavGraphNodeType_NavNode_Door = 0x1
    ENPCNavGraphNodeType_PointOfInterestNode = 0x2
    ENPCNavGraphNodeType_Slope = 0x3
    ENPCNavGraphNodeType_NavAreaConnection = 0x4
    ENPCNavGraphNodeType_NumTypes = 0x5
    ENPCNavGraphNodeType_Invalid = 0x6

class EgRenderParity(ctypes.Structure):
    class List(IntEnum):
        Single = 0x0
        EvenRender = 0x1
        OddRender = 0x2


class EngineUtils(ctypes.Structure):
    class LoadResourcesResult(IntEnum):
        CompletedNoErrors = 0x0
        CompletedWithErrors = 0x1
        PartialNoErrors = 0x2
        PartialWithErrors = 0x3
        NothingLoaded = 0x4

    class LoadResourcesBalancing(IntEnum):
        Unbound = 0x0
        TimeLimitedCallScope = 0x1
        TimeLimitedFrameScope = 0x2

    def FinishAsyncReadResourceFile(): pass

    def ReloadResource(): pass

    def LoadResourcesFromDisk(): pass

    def AreResourcesAwaitingLoad(): pass

    def DumpMessages(): pass


class tagExtentMode(IntEnum):
    DVEXTENT_CONTENT = 0x0
    DVEXTENT_INTEGRAL = 0x1

class tagGLOBALOPT_EH_VALUES(IntEnum):
    COMGLB_EXCEPTION_HANDLE = 0x0
    COMGLB_EXCEPTION_DONOT_HANDLE_FATAL = 0x1
    COMGLB_EXCEPTION_DONOT_HANDLE_ANY = 0x2

class DXGI_FORMAT(IntEnum):
    DXGI_FORMAT_UNKNOWN = 0x0
    DXGI_FORMAT_R32G32B32A32_TYPELESS = 0x1
    DXGI_FORMAT_R32G32B32A32_FLOAT = 0x2
    DXGI_FORMAT_R32G32B32A32_UINT = 0x3
    DXGI_FORMAT_R32G32B32A32_SINT = 0x4
    DXGI_FORMAT_R32G32B32_TYPELESS = 0x5
    DXGI_FORMAT_R32G32B32_FLOAT = 0x6
    DXGI_FORMAT_R32G32B32_UINT = 0x7
    DXGI_FORMAT_R32G32B32_SINT = 0x8
    DXGI_FORMAT_R16G16B16A16_TYPELESS = 0x9
    DXGI_FORMAT_R16G16B16A16_FLOAT = 0xA
    DXGI_FORMAT_R16G16B16A16_UNORM = 0xB
    DXGI_FORMAT_R16G16B16A16_UINT = 0xC
    DXGI_FORMAT_R16G16B16A16_SNORM = 0xD
    DXGI_FORMAT_R16G16B16A16_SINT = 0xE
    DXGI_FORMAT_R32G32_TYPELESS = 0xF
    DXGI_FORMAT_R32G32_FLOAT = 0x10
    DXGI_FORMAT_R32G32_UINT = 0x11
    DXGI_FORMAT_R32G32_SINT = 0x12
    DXGI_FORMAT_R32G8X24_TYPELESS = 0x13
    DXGI_FORMAT_D32_FLOAT_S8X24_UINT = 0x14
    DXGI_FORMAT_R32_FLOAT_X8X24_TYPELESS = 0x15
    DXGI_FORMAT_X32_TYPELESS_G8X24_UINT = 0x16
    DXGI_FORMAT_R10G10B10A2_TYPELESS = 0x17
    DXGI_FORMAT_R10G10B10A2_UNORM = 0x18
    DXGI_FORMAT_R10G10B10A2_UINT = 0x19
    DXGI_FORMAT_R11G11B10_FLOAT = 0x1A
    DXGI_FORMAT_R8G8B8A8_TYPELESS = 0x1B
    DXGI_FORMAT_R8G8B8A8_UNORM = 0x1C
    DXGI_FORMAT_R8G8B8A8_UNORM_SRGB = 0x1D
    DXGI_FORMAT_R8G8B8A8_UINT = 0x1E
    DXGI_FORMAT_R8G8B8A8_SNORM = 0x1F
    DXGI_FORMAT_R8G8B8A8_SINT = 0x20
    DXGI_FORMAT_R16G16_TYPELESS = 0x21
    DXGI_FORMAT_R16G16_FLOAT = 0x22
    DXGI_FORMAT_R16G16_UNORM = 0x23
    DXGI_FORMAT_R16G16_UINT = 0x24
    DXGI_FORMAT_R16G16_SNORM = 0x25
    DXGI_FORMAT_R16G16_SINT = 0x26
    DXGI_FORMAT_R32_TYPELESS = 0x27
    DXGI_FORMAT_D32_FLOAT = 0x28
    DXGI_FORMAT_R32_FLOAT = 0x29
    DXGI_FORMAT_R32_UINT = 0x2A
    DXGI_FORMAT_R32_SINT = 0x2B
    DXGI_FORMAT_R24G8_TYPELESS = 0x2C
    DXGI_FORMAT_D24_UNORM_S8_UINT = 0x2D
    DXGI_FORMAT_R24_UNORM_X8_TYPELESS = 0x2E
    DXGI_FORMAT_X24_TYPELESS_G8_UINT = 0x2F
    DXGI_FORMAT_R8G8_TYPELESS = 0x30
    DXGI_FORMAT_R8G8_UNORM = 0x31
    DXGI_FORMAT_R8G8_UINT = 0x32
    DXGI_FORMAT_R8G8_SNORM = 0x33
    DXGI_FORMAT_R8G8_SINT = 0x34
    DXGI_FORMAT_R16_TYPELESS = 0x35
    DXGI_FORMAT_R16_FLOAT = 0x36
    DXGI_FORMAT_D16_UNORM = 0x37
    DXGI_FORMAT_R16_UNORM = 0x38
    DXGI_FORMAT_R16_UINT = 0x39
    DXGI_FORMAT_R16_SNORM = 0x3A
    DXGI_FORMAT_R16_SINT = 0x3B
    DXGI_FORMAT_R8_TYPELESS = 0x3C
    DXGI_FORMAT_R8_UNORM = 0x3D
    DXGI_FORMAT_R8_UINT = 0x3E
    DXGI_FORMAT_R8_SNORM = 0x3F
    DXGI_FORMAT_R8_SINT = 0x40
    DXGI_FORMAT_A8_UNORM = 0x41
    DXGI_FORMAT_R1_UNORM = 0x42
    DXGI_FORMAT_R9G9B9E5_SHAREDEXP = 0x43
    DXGI_FORMAT_R8G8_B8G8_UNORM = 0x44
    DXGI_FORMAT_G8R8_G8B8_UNORM = 0x45
    DXGI_FORMAT_BC1_TYPELESS = 0x46
    DXGI_FORMAT_BC1_UNORM = 0x47
    DXGI_FORMAT_BC1_UNORM_SRGB = 0x48
    DXGI_FORMAT_BC2_TYPELESS = 0x49
    DXGI_FORMAT_BC2_UNORM = 0x4A
    DXGI_FORMAT_BC2_UNORM_SRGB = 0x4B
    DXGI_FORMAT_BC3_TYPELESS = 0x4C
    DXGI_FORMAT_BC3_UNORM = 0x4D
    DXGI_FORMAT_BC3_UNORM_SRGB = 0x4E
    DXGI_FORMAT_BC4_TYPELESS = 0x4F
    DXGI_FORMAT_BC4_UNORM = 0x50
    DXGI_FORMAT_BC4_SNORM = 0x51
    DXGI_FORMAT_BC5_TYPELESS = 0x52
    DXGI_FORMAT_BC5_UNORM = 0x53
    DXGI_FORMAT_BC5_SNORM = 0x54
    DXGI_FORMAT_B5G6R5_UNORM = 0x55
    DXGI_FORMAT_B5G5R5A1_UNORM = 0x56
    DXGI_FORMAT_B8G8R8A8_UNORM = 0x57
    DXGI_FORMAT_B8G8R8X8_UNORM = 0x58
    DXGI_FORMAT_R10G10B10_XR_BIAS_A2_UNORM = 0x59
    DXGI_FORMAT_B8G8R8A8_TYPELESS = 0x5A
    DXGI_FORMAT_B8G8R8A8_UNORM_SRGB = 0x5B
    DXGI_FORMAT_B8G8R8X8_TYPELESS = 0x5C
    DXGI_FORMAT_B8G8R8X8_UNORM_SRGB = 0x5D
    DXGI_FORMAT_BC6H_TYPELESS = 0x5E
    DXGI_FORMAT_BC6H_UF16 = 0x5F
    DXGI_FORMAT_BC6H_SF16 = 0x60
    DXGI_FORMAT_BC7_TYPELESS = 0x61
    DXGI_FORMAT_BC7_UNORM = 0x62
    DXGI_FORMAT_BC7_UNORM_SRGB = 0x63
    DXGI_FORMAT_AYUV = 0x64
    DXGI_FORMAT_Y410 = 0x65
    DXGI_FORMAT_Y416 = 0x66
    DXGI_FORMAT_NV12 = 0x67
    DXGI_FORMAT_P010 = 0x68
    DXGI_FORMAT_P016 = 0x69
    DXGI_FORMAT_420_OPAQUE = 0x6A
    DXGI_FORMAT_YUY2 = 0x6B
    DXGI_FORMAT_Y210 = 0x6C
    DXGI_FORMAT_Y216 = 0x6D
    DXGI_FORMAT_NV11 = 0x6E
    DXGI_FORMAT_AI44 = 0x6F
    DXGI_FORMAT_IA44 = 0x70
    DXGI_FORMAT_P8 = 0x71
    DXGI_FORMAT_A8P8 = 0x72
    DXGI_FORMAT_B4G4R4A4_UNORM = 0x73
    DXGI_FORMAT_P208 = 0x82
    DXGI_FORMAT_V208 = 0x83
    DXGI_FORMAT_V408 = 0x84
    DXGI_FORMAT_SAMPLER_FEEDBACK_MIN_MIP_OPAQUE = 0xBD
    DXGI_FORMAT_SAMPLER_FEEDBACK_MIP_REGION_USED_OPAQUE = 0xBE
    DXGI_FORMAT_FORCE_UINT = 0xFFFFFFFF

class tagDOMNodeType(IntEnum):
    NODE_INVALID = 0x0
    NODE_ELEMENT = 0x1
    NODE_ATTRIBUTE = 0x2
    NODE_TEXT = 0x3
    NODE_CDATA_SECTION = 0x4
    NODE_ENTITY_REFERENCE = 0x5
    NODE_ENTITY = 0x6
    NODE_PROCESSING_INSTRUCTION = 0x7
    NODE_COMMENT = 0x8
    NODE_DOCUMENT = 0x9
    NODE_DOCUMENT_TYPE = 0xA
    NODE_DOCUMENT_FRAGMENT = 0xB
    NODE_NOTATION = 0xC

class tagXMLEMEM_TYPE(IntEnum):
    XMLELEMTYPE_ELEMENT = 0x0
    XMLELEMTYPE_TEXT = 0x1
    XMLELEMTYPE_COMMENT = 0x2
    XMLELEMTYPE_DOCUMENT = 0x3
    XMLELEMTYPE_DTD = 0x4
    XMLELEMTYPE_PI = 0x5
    XMLELEMTYPE_OTHER = 0x6

class VARENUM(IntEnum):
    VT_EMPTY = 0x0
    VT_NULL = 0x1
    VT_I2 = 0x2
    VT_I4 = 0x3
    VT_R4 = 0x4
    VT_R8 = 0x5
    VT_CY = 0x6
    VT_DATE = 0x7
    VT_BSTR = 0x8
    VT_DISPATCH = 0x9
    VT_ERROR = 0xA
    VT_BOOL = 0xB
    VT_VARIANT = 0xC
    VT_UNKNOWN = 0xD
    VT_DECIMAL = 0xE
    VT_I1 = 0x10
    VT_UI1 = 0x11
    VT_UI2 = 0x12
    VT_UI4 = 0x13
    VT_I8 = 0x14
    VT_UI8 = 0x15
    VT_INT = 0x16
    VT_UINT = 0x17
    VT_VOID = 0x18
    VT_HRESULT = 0x19
    VT_PTR = 0x1A
    VT_SAFEARRAY = 0x1B
    VT_CARRAY = 0x1C
    VT_USERDEFINED = 0x1D
    VT_LPSTR = 0x1E
    VT_LPWSTR = 0x1F
    VT_RECORD = 0x24
    VT_INT_PTR = 0x25
    VT_UINT_PTR = 0x26
    VT_FILETIME = 0x40
    VT_BLOB = 0x41
    VT_STREAM = 0x42
    VT_STORAGE = 0x43
    VT_STREAMED_OBJECT = 0x44
    VT_STORED_OBJECT = 0x45
    VT_BLOB_OBJECT = 0x46
    VT_CF = 0x47
    VT_CLSID = 0x48
    VT_VERSIONED_STREAM = 0x49
    VT_BSTR_BLOB = 0xFFF
    VT_VECTOR = 0x1000
    VT_ARRAY = 0x2000
    VT_BYREF = 0x4000
    VT_RESERVED = 0x8000
    VT_ILLEGAL = 0xFFFF

class tagTYSPEC(IntEnum):
    TYSPEC_CLSID = 0x0
    TYSPEC_FILEEXT = 0x1
    TYSPEC_MIMETYPE = 0x2
    TYSPEC_FILENAME = 0x3
    TYSPEC_PROGID = 0x4
    TYSPEC_PACKAGENAME = 0x5
    TYSPEC_OBJECTID = 0x6

class tagBINDSTATUS(IntEnum):
    BINDSTATUS_FINDINGRESOURCE = 0x1
    BINDSTATUS_CONNECTING = 0x2
    BINDSTATUS_REDIRECTING = 0x3
    BINDSTATUS_BEGINDOWNLOADDATA = 0x4
    BINDSTATUS_DOWNLOADINGDATA = 0x5
    BINDSTATUS_ENDDOWNLOADDATA = 0x6
    BINDSTATUS_BEGINDOWNLOADCOMPONENTS = 0x7
    BINDSTATUS_INSTALLINGCOMPONENTS = 0x8
    BINDSTATUS_ENDDOWNLOADCOMPONENTS = 0x9
    BINDSTATUS_USINGCACHEDCOPY = 0xA
    BINDSTATUS_SENDINGREQUEST = 0xB
    BINDSTATUS_CLASSIDAVAILABLE = 0xC
    BINDSTATUS_MIMETYPEAVAILABLE = 0xD
    BINDSTATUS_CACHEFILENAMEAVAILABLE = 0xE
    BINDSTATUS_BEGINSYNCOPERATION = 0xF
    BINDSTATUS_ENDSYNCOPERATION = 0x10
    BINDSTATUS_BEGINUPLOADDATA = 0x11
    BINDSTATUS_UPLOADINGDATA = 0x12
    BINDSTATUS_ENDUPLOADDATA = 0x13
    BINDSTATUS_PROTOCOLCLASSID = 0x14
    BINDSTATUS_ENCODING = 0x15
    BINDSTATUS_VERIFIEDMIMETYPEAVAILABLE = 0x16
    BINDSTATUS_CLASSINSTALLLOCATION = 0x17
    BINDSTATUS_DECODING = 0x18
    BINDSTATUS_LOADINGMIMEHANDLER = 0x19
    BINDSTATUS_CONTENTDISPOSITIONATTACH = 0x1A
    BINDSTATUS_FILTERREPORTMIMETYPE = 0x1B
    BINDSTATUS_CLSIDCANINSTANTIATE = 0x1C
    BINDSTATUS_IUNKNOWNAVAILABLE = 0x1D
    BINDSTATUS_DIRECTBIND = 0x1E
    BINDSTATUS_RAWMIMETYPE = 0x1F
    BINDSTATUS_PROXYDETECTING = 0x20
    BINDSTATUS_ACCEPTRANGES = 0x21
    BINDSTATUS_COOKIE_SENT = 0x22
    BINDSTATUS_COMPACT_POLICY_RECEIVED = 0x23
    BINDSTATUS_COOKIE_SUPPRESSED = 0x24
    BINDSTATUS_COOKIE_STATE_UNKNOWN = 0x25
    BINDSTATUS_COOKIE_STATE_ACCEPT = 0x26
    BINDSTATUS_COOKIE_STATE_REJECT = 0x27
    BINDSTATUS_COOKIE_STATE_PROMPT = 0x28
    BINDSTATUS_COOKIE_STATE_LEASH = 0x29
    BINDSTATUS_COOKIE_STATE_DOWNGRADE = 0x2A
    BINDSTATUS_POLICY_HREF = 0x2B
    BINDSTATUS_P3P_HEADER = 0x2C
    BINDSTATUS_SESSION_COOKIE_RECEIVED = 0x2D
    BINDSTATUS_PERSISTENT_COOKIE_RECEIVED = 0x2E
    BINDSTATUS_SESSION_COOKIES_ALLOWED = 0x2F
    BINDSTATUS_CACHECONTROL = 0x30
    BINDSTATUS_CONTENTDISPOSITIONFILENAME = 0x31
    BINDSTATUS_MIMETEXTPLAINMISMATCH = 0x32
    BINDSTATUS_PUBLISHERAVAILABLE = 0x33
    BINDSTATUS_DISPLAYNAMEAVAILABLE = 0x34
    BINDSTATUS_SSLUX_NAVBLOCKED = 0x35
    BINDSTATUS_SERVER_MIMETYPEAVAILABLE = 0x36
    BINDSTATUS_SNIFFED_CLASSIDAVAILABLE = 0x37
    BINDSTATUS_64BIT_PROGRESS = 0x38
    BINDSTATUS_RESERVED_0 = 0x39
    BINDSTATUS_RESERVED_1 = 0x3A
    BINDSTATUS_RESERVED_2 = 0x3B
    BINDSTATUS_RESERVED_3 = 0x3C
    BINDSTATUS_RESERVED_4 = 0x3D
    BINDSTATUS_RESERVED_5 = 0x3E
    BINDSTATUS_RESERVED_6 = 0x3F
    BINDSTATUS_RESERVED_7 = 0x40
    BINDSTATUS_RESERVED_8 = 0x41
    BINDSTATUS_RESERVED_9 = 0x42
    BINDSTATUS_RESERVED_A = 0x43
    BINDSTATUS_RESERVED_B = 0x44
    BINDSTATUS_RESERVED_C = 0x45
    BINDSTATUS_RESERVED_D = 0x46
    BINDSTATUS_RESERVED_E = 0x47
    BINDSTATUS_RESERVED_F = 0x48
    BINDSTATUS_RESERVED_10 = 0x49
    BINDSTATUS_RESERVED_11 = 0x4A
    BINDSTATUS_RESERVED_12 = 0x4B
    BINDSTATUS_RESERVED_13 = 0x4C

class __MIDL_ICodeInstall_0001(IntEnum):
    CIP_DISK_FULL = 0x0
    CIP_ACCESS_DENIED = 0x1
    CIP_NEWER_VERSION_EXISTS = 0x2
    CIP_OLDER_VERSION_EXISTS = 0x3
    CIP_NAME_CONFLICT = 0x4
    CIP_TRUST_VERIFICATION_COMPONENT_MISSING = 0x5
    CIP_EXE_SELF_REGISTERATION_TIMEOUT = 0x6
    CIP_UNSAFE_TO_ABORT = 0x7
    CIP_NEED_REBOOT = 0x8
    CIP_NEED_REBOOT_UI_PERMISSION = 0x9

class __MIDL_IUri_0001(IntEnum):
    Uri_PROPERTY_ABSOLUTE_URI = 0x0
    Uri_PROPERTY_AUTHORITY = 0x1
    Uri_PROPERTY_DISPLAY_URI = 0x2
    Uri_PROPERTY_DOMAIN = 0x3
    Uri_PROPERTY_EXTENSION = 0x4
    Uri_PROPERTY_FRAGMENT = 0x5
    Uri_PROPERTY_HOST = 0x6
    Uri_PROPERTY_PASSWORD = 0x7
    Uri_PROPERTY_PATH = 0x8
    Uri_PROPERTY_PATH_AND_QUERY = 0x9
    Uri_PROPERTY_QUERY = 0xA
    Uri_PROPERTY_RAW_URI = 0xB
    Uri_PROPERTY_SCHEME_NAME = 0xC
    Uri_PROPERTY_USER_INFO = 0xD
    Uri_PROPERTY_USER_NAME = 0xE
    Uri_PROPERTY_HOST_TYPE = 0xF
    Uri_PROPERTY_PORT = 0x10
    Uri_PROPERTY_SCHEME = 0x11
    Uri_PROPERTY_ZONE = 0x12

class __MIDL_IUri_0002(IntEnum):
    Uri_HOST_UNKNOWN = 0x0
    Uri_HOST_DNS = 0x1
    Uri_HOST_IPV4 = 0x2
    Uri_HOST_IPV6 = 0x3
    Uri_HOST_IDN = 0x4

class tagBINDSTRING(IntEnum):
    BINDSTRING_HEADERS = 0x1
    BINDSTRING_ACCEPT_MIMES = 0x2
    BINDSTRING_EXTRA_URL = 0x3
    BINDSTRING_LANGUAGE = 0x4
    BINDSTRING_USERNAME = 0x5
    BINDSTRING_PASSWORD = 0x6
    BINDSTRING_UA_PIXELS = 0x7
    BINDSTRING_UA_COLOR = 0x8
    BINDSTRING_OS = 0x9
    BINDSTRING_USER_AGENT = 0xA
    BINDSTRING_ACCEPT_ENCODINGS = 0xB
    BINDSTRING_POST_COOKIE = 0xC
    BINDSTRING_POST_DATA_MIME = 0xD
    BINDSTRING_URL = 0xE
    BINDSTRING_IID = 0xF
    BINDSTRING_FLAG_BIND_TO_OBJECT = 0x10
    BINDSTRING_PTR_BIND_CONTEXT = 0x11
    BINDSTRING_XDR_ORIGIN = 0x12
    BINDSTRING_DOWNLOADPATH = 0x13
    BINDSTRING_ROOTDOC_URL = 0x14
    BINDSTRING_INITIAL_FILENAME = 0x15
    BINDSTRING_PROXY_USERNAME = 0x16
    BINDSTRING_PROXY_PASSWORD = 0x17
    BINDSTRING_ENTERPRISE_ID = 0x18
    BINDSTRING_DOC_URL = 0x19
    BINDSTRING_SAMESITE_COOKIE_LEVEL = 0x1A

class VkImageTiling(IntEnum):
    VK_IMAGE_TILING_OPTIMAL = 0x0
    VK_IMAGE_TILING_LINEAR = 0x1
    VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT = 0x3B9D3330
    VK_IMAGE_TILING_MAX_ENUM = 0x7FFFFFFF

class VkImageType(IntEnum):
    VK_IMAGE_TYPE_1D = 0x0
    VK_IMAGE_TYPE_2D = 0x1
    VK_IMAGE_TYPE_3D = 0x2
    VK_IMAGE_TYPE_MAX_ENUM = 0x7FFFFFFF

class _tagPARSEACTION(IntEnum):
    PARSE_CANONICALIZE = 0x1
    PARSE_FRIENDLY = 0x2
    PARSE_SECURITY_URL = 0x3
    PARSE_ROOTDOCUMENT = 0x4
    PARSE_DOCUMENT = 0x5
    PARSE_ANCHOR = 0x6
    PARSE_ENCODE_IS_UNESCAPE = 0x7
    PARSE_DECODE_IS_ESCAPE = 0x8
    PARSE_PATH_FROM_URL = 0x9
    PARSE_URL_FROM_PATH = 0xA
    PARSE_MIME = 0xB
    PARSE_SERVER = 0xC
    PARSE_SCHEMA = 0xD
    PARSE_SITE = 0xE
    PARSE_DOMAIN = 0xF
    PARSE_LOCATION = 0x10
    PARSE_SECURITY_DOMAIN = 0x11
    PARSE_ESCAPE = 0x12
    PARSE_UNESCAPE = 0x13

class VkQueryType(IntEnum):
    VK_QUERY_TYPE_OCCLUSION = 0x0
    VK_QUERY_TYPE_PIPELINE_STATISTICS = 0x1
    VK_QUERY_TYPE_TIMESTAMP = 0x2
    VK_QUERY_TYPE_RESULT_STATUS_ONLY_KHR = 0x3B9B23D8
    VK_QUERY_TYPE_TRANSFORM_FEEDBACK_STREAM_EXT = 0x3B9B3764
    VK_QUERY_TYPE_PERFORMANCE_QUERY_KHR = 0x3B9C8F20
    VK_QUERY_TYPE_ACCELERATION_STRUCTURE_COMPACTED_SIZE_KHR = 0x3B9D13F0
    VK_QUERY_TYPE_ACCELERATION_STRUCTURE_SERIALIZATION_SIZE_KHR = 0x3B9D13F1
    VK_QUERY_TYPE_ACCELERATION_STRUCTURE_COMPACTED_SIZE_NV = 0x3B9D4E88
    VK_QUERY_TYPE_PERFORMANCE_QUERY_INTEL = 0x3B9DFE50
    VK_QUERY_TYPE_MESH_PRIMITIVES_GENERATED_EXT = 0x3B9FCB40
    VK_QUERY_TYPE_PRIMITIVES_GENERATED_EXT = 0x3BA09E30
    VK_QUERY_TYPE_ACCELERATION_STRUCTURE_SERIALIZATION_BOTTOM_LEVEL_POINTERS_KHR = 0x3BA0ADD0
    VK_QUERY_TYPE_ACCELERATION_STRUCTURE_SIZE_KHR = 0x3BA0ADD1
    VK_QUERY_TYPE_MICROMAP_SERIALIZATION_SIZE_EXT = 0x3BA0D4E0
    VK_QUERY_TYPE_MICROMAP_COMPACTED_SIZE_EXT = 0x3BA0D4E1
    VK_QUERY_TYPE_MAX_ENUM = 0x7FFFFFFF

class _tagPSUACTION(IntEnum):
    PSU_DEFAULT = 0x1

class _tagQUERYOPTION(IntEnum):
    QUERY_EXPIRATION_DATE = 0x1
    QUERY_TIME_OF_LAST_CHANGE = 0x2
    QUERY_CONTENT_ENCODING = 0x3
    QUERY_CONTENT_TYPE = 0x4
    QUERY_REFRESH = 0x5
    QUERY_RECOMBINE = 0x6
    QUERY_CAN_NAVIGATE = 0x7
    QUERY_USES_NETWORK = 0x8
    QUERY_IS_CACHED = 0x9
    QUERY_IS_INSTALLEDENTRY = 0xA
    QUERY_IS_CACHED_OR_MAPPED = 0xB
    QUERY_USES_CACHE = 0xC
    QUERY_IS_SECURE = 0xD
    QUERY_IS_SAFE = 0xE
    QUERY_USES_HISTORYFOLDER = 0xF
    QUERY_IS_CACHED_AND_USABLE_OFFLINE = 0x10

class tagApplicationType(IntEnum):
    ServerApplication = 0x0
    LibraryApplication = 0x1

class tagShutdownType(IntEnum):
    IdleShutdown = 0x0
    ForcedShutdown = 0x1

class _tagINTERNETFEATURELIST(IntEnum):
    FEATURE_OBJECT_CACHING = 0x0
    FEATURE_ZONE_ELEVATION = 0x1
    FEATURE_MIME_HANDLING = 0x2
    FEATURE_MIME_SNIFFING = 0x3
    FEATURE_WINDOW_RESTRICTIONS = 0x4
    FEATURE_WEBOC_POPUPMANAGEMENT = 0x5
    FEATURE_BEHAVIORS = 0x6
    FEATURE_DISABLE_MK_PROTOCOL = 0x7
    FEATURE_LOCALMACHINE_LOCKDOWN = 0x8
    FEATURE_SECURITYBAND = 0x9
    FEATURE_RESTRICT_ACTIVEXINSTALL = 0xA
    FEATURE_VALIDATE_NAVIGATE_URL = 0xB
    FEATURE_RESTRICT_FILEDOWNLOAD = 0xC
    FEATURE_ADDON_MANAGEMENT = 0xD
    FEATURE_PROTOCOL_LOCKDOWN = 0xE
    FEATURE_HTTP_USERNAME_PASSWORD_DISABLE = 0xF
    FEATURE_SAFE_BINDTOOBJECT = 0x10
    FEATURE_UNC_SAVEDFILECHECK = 0x11
    FEATURE_GET_URL_DOM_FILEPATH_UNENCODED = 0x12
    FEATURE_TABBED_BROWSING = 0x13
    FEATURE_SSLUX = 0x14
    FEATURE_DISABLE_NAVIGATION_SOUNDS = 0x15
    FEATURE_DISABLE_LEGACY_COMPRESSION = 0x16
    FEATURE_FORCE_ADDR_AND_STATUS = 0x17
    FEATURE_XMLHTTP = 0x18
    FEATURE_DISABLE_TELNET_PROTOCOL = 0x19
    FEATURE_FEEDS = 0x1A
    FEATURE_BLOCK_INPUT_PROMPTS = 0x1B
    FEATURE_ENTRY_COUNT = 0x1C

class VkFilter(IntEnum):
    VK_FILTER_NEAREST = 0x0
    VK_FILTER_LINEAR = 0x1
    VK_FILTER_CUBIC_EXT = 0x3B9B0498
    VK_FILTER_MAX_ENUM = 0x7FFFFFFF

class VkSamplerAddressMode(IntEnum):
    VK_SAMPLER_ADDRESS_MODE_REPEAT = 0x0
    VK_SAMPLER_ADDRESS_MODE_MIRRORED_REPEAT = 0x1
    VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE = 0x2
    VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER = 0x3
    VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE = 0x4
    VK_SAMPLER_ADDRESS_MODE_MAX_ENUM = 0x7FFFFFFF

class VkSamplerMipmapMode(IntEnum):
    VK_SAMPLER_MIPMAP_MODE_NEAREST = 0x0
    VK_SAMPLER_MIPMAP_MODE_LINEAR = 0x1
    VK_SAMPLER_MIPMAP_MODE_MAX_ENUM = 0x7FFFFFFF

class VkAccessFlagBits(IntEnum):
    VK_ACCESS_NONE = 0x0
    VK_ACCESS_INDIRECT_COMMAND_READ_BIT = 0x1
    VK_ACCESS_INDEX_READ_BIT = 0x2
    VK_ACCESS_VERTEX_ATTRIBUTE_READ_BIT = 0x4
    VK_ACCESS_UNIFORM_READ_BIT = 0x8
    VK_ACCESS_INPUT_ATTACHMENT_READ_BIT = 0x10
    VK_ACCESS_SHADER_READ_BIT = 0x20
    VK_ACCESS_SHADER_WRITE_BIT = 0x40
    VK_ACCESS_COLOR_ATTACHMENT_READ_BIT = 0x80
    VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT = 0x100
    VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_READ_BIT = 0x200
    VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_WRITE_BIT = 0x400
    VK_ACCESS_TRANSFER_READ_BIT = 0x800
    VK_ACCESS_TRANSFER_WRITE_BIT = 0x1000
    VK_ACCESS_HOST_READ_BIT = 0x2000
    VK_ACCESS_HOST_WRITE_BIT = 0x4000
    VK_ACCESS_MEMORY_READ_BIT = 0x8000
    VK_ACCESS_MEMORY_WRITE_BIT = 0x10000
    VK_ACCESS_COMMAND_PREPROCESS_READ_BIT_NV = 0x20000
    VK_ACCESS_COMMAND_PREPROCESS_WRITE_BIT_NV = 0x40000
    VK_ACCESS_COLOR_ATTACHMENT_READ_NONCOHERENT_BIT_EXT = 0x80000
    VK_ACCESS_CONDITIONAL_RENDERING_READ_BIT_EXT = 0x100000
    VK_ACCESS_ACCELERATION_STRUCTURE_READ_BIT_KHR = 0x200000
    VK_ACCESS_ACCELERATION_STRUCTURE_WRITE_BIT_KHR = 0x400000
    VK_ACCESS_FRAGMENT_SHADING_RATE_ATTACHMENT_READ_BIT_KHR = 0x800000
    VK_ACCESS_FRAGMENT_DENSITY_MAP_READ_BIT_EXT = 0x1000000
    VK_ACCESS_TRANSFORM_FEEDBACK_WRITE_BIT_EXT = 0x2000000
    VK_ACCESS_TRANSFORM_FEEDBACK_COUNTER_READ_BIT_EXT = 0x4000000
    VK_ACCESS_TRANSFORM_FEEDBACK_COUNTER_WRITE_BIT_EXT = 0x8000000
    VK_ACCESS_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkImageAspectFlagBits(IntEnum):
    VK_IMAGE_ASPECT_NONE = 0x0
    VK_IMAGE_ASPECT_COLOR_BIT = 0x1
    VK_IMAGE_ASPECT_DEPTH_BIT = 0x2
    VK_IMAGE_ASPECT_STENCIL_BIT = 0x4
    VK_IMAGE_ASPECT_METADATA_BIT = 0x8
    VK_IMAGE_ASPECT_PLANE_0_BIT = 0x10
    VK_IMAGE_ASPECT_PLANE_1_BIT = 0x20
    VK_IMAGE_ASPECT_PLANE_2_BIT = 0x40
    VK_IMAGE_ASPECT_MEMORY_PLANE_0_BIT_EXT = 0x80
    VK_IMAGE_ASPECT_MEMORY_PLANE_1_BIT_EXT = 0x100
    VK_IMAGE_ASPECT_MEMORY_PLANE_2_BIT_EXT = 0x200
    VK_IMAGE_ASPECT_MEMORY_PLANE_3_BIT_EXT = 0x400
    VK_IMAGE_ASPECT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkImageCreateFlagBits(IntEnum):
    VK_IMAGE_CREATE_SPARSE_BINDING_BIT = 0x1
    VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT = 0x2
    VK_IMAGE_CREATE_SPARSE_ALIASED_BIT = 0x4
    VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT = 0x8
    VK_IMAGE_CREATE_CUBE_COMPATIBLE_BIT = 0x10
    VK_IMAGE_CREATE_2D_ARRAY_COMPATIBLE_BIT = 0x20
    VK_IMAGE_CREATE_SPLIT_INSTANCE_BIND_REGIONS_BIT = 0x40
    VK_IMAGE_CREATE_BLOCK_TEXEL_VIEW_COMPATIBLE_BIT = 0x80
    VK_IMAGE_CREATE_EXTENDED_USAGE_BIT = 0x100
    VK_IMAGE_CREATE_DISJOINT_BIT = 0x200
    VK_IMAGE_CREATE_ALIAS_BIT = 0x400
    VK_IMAGE_CREATE_PROTECTED_BIT = 0x800
    VK_IMAGE_CREATE_SAMPLE_LOCATIONS_COMPATIBLE_DEPTH_BIT_EXT = 0x1000
    VK_IMAGE_CREATE_CORNER_SAMPLED_BIT_NV = 0x2000
    VK_IMAGE_CREATE_SUBSAMPLED_BIT_EXT = 0x4000
    VK_IMAGE_CREATE_FRAGMENT_DENSITY_MAP_OFFSET_BIT_QCOM = 0x8000
    VK_IMAGE_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT = 0x10000
    VK_IMAGE_CREATE_2D_VIEW_COMPATIBLE_BIT_EXT = 0x20000
    VK_IMAGE_CREATE_MULTISAMPLED_RENDER_TO_SINGLE_SAMPLED_BIT_EXT = 0x40000
    VK_IMAGE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkSampleCountFlagBits(IntEnum):
    VK_SAMPLE_COUNT_1_BIT = 0x1
    VK_SAMPLE_COUNT_2_BIT = 0x2
    VK_SAMPLE_COUNT_4_BIT = 0x4
    VK_SAMPLE_COUNT_8_BIT = 0x8
    VK_SAMPLE_COUNT_16_BIT = 0x10
    VK_SAMPLE_COUNT_32_BIT = 0x20
    VK_SAMPLE_COUNT_64_BIT = 0x40
    VK_SAMPLE_COUNT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class eCreatureSketchVariable(IntEnum):
    ECreatureSketchVariable_Alert = 0x0
    ECreatureSketchVariable_NumTypes = 0x1

class eCreatureNetworkedEvent(IntEnum):
    ECreatureNetworkedEvent_SandwormIntoGround = 0x0
    ECreatureNetworkedEvent_SandwormOutOfGround = 0x1
    ECreatureNetworkedEvent_SandwormDust = 0x2
    ECreatureNetworkedEvent_KilledByPredator = 0x3
    ECreatureNetworkedEvent_NumTypes = 0x4

class ePetAccessoryState(IntEnum):
    EPetAccessoryState_Neutral = 0x0
    EPetAccessoryState_Fed = 0x1
    EPetAccessoryState_Happy = 0x2
    EPetAccessoryState_Hungry = 0x3
    EPetAccessoryState_Sad = 0x4
    EPetAccessoryState_Busy = 0x5

class VkImageUsageFlagBits(IntEnum):
    VK_IMAGE_USAGE_TRANSFER_SRC_BIT = 0x1
    VK_IMAGE_USAGE_TRANSFER_DST_BIT = 0x2
    VK_IMAGE_USAGE_SAMPLED_BIT = 0x4
    VK_IMAGE_USAGE_STORAGE_BIT = 0x8
    VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT = 0x10
    VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT = 0x20
    VK_IMAGE_USAGE_TRANSIENT_ATTACHMENT_BIT = 0x40
    VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT = 0x80
    VK_IMAGE_USAGE_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR = 0x100
    VK_IMAGE_USAGE_FRAGMENT_DENSITY_MAP_BIT_EXT = 0x200
    VK_IMAGE_USAGE_VIDEO_DECODE_DST_BIT_KHR = 0x400
    VK_IMAGE_USAGE_VIDEO_DECODE_SRC_BIT_KHR = 0x800
    VK_IMAGE_USAGE_VIDEO_DECODE_DPB_BIT_KHR = 0x1000
    VK_IMAGE_USAGE_INVOCATION_MASK_BIT_HUAWEI = 0x40000
    VK_IMAGE_USAGE_ATTACHMENT_FEEDBACK_LOOP_BIT_EXT = 0x80000
    VK_IMAGE_USAGE_SAMPLE_WEIGHT_BIT_QCOM = 0x100000
    VK_IMAGE_USAGE_SAMPLE_BLOCK_MATCH_BIT_QCOM = 0x200000
    VK_IMAGE_USAGE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class tagURLZONE(IntEnum):
    URLZONE_PREDEFINED_MIN = 0x0
    URLZONE_INTRANET = 0x1
    URLZONE_TRUSTED = 0x2
    URLZONE_INTERNET = 0x3
    URLZONE_UNTRUSTED = 0x4
    URLZONE_PREDEFINED_MAX = 0x3E7
    URLZONE_USER_MIN = 0x3E8
    URLZONE_USER_MAX = 0x2710
    URLZONE_INVALID = 0xFFFFFFFF

class _URLZONEREG(IntEnum):
    URLZONEREG_DEFAULT = 0x0
    URLZONEREG_HKLM = 0x1
    URLZONEREG_HKCU = 0x2

class VkPipelineStageFlagBits(IntEnum):
    VK_PIPELINE_STAGE_NONE = 0x0
    VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT = 0x1
    VK_PIPELINE_STAGE_DRAW_INDIRECT_BIT = 0x2
    VK_PIPELINE_STAGE_VERTEX_INPUT_BIT = 0x4
    VK_PIPELINE_STAGE_VERTEX_SHADER_BIT = 0x8
    VK_PIPELINE_STAGE_TESSELLATION_CONTROL_SHADER_BIT = 0x10
    VK_PIPELINE_STAGE_TESSELLATION_EVALUATION_SHADER_BIT = 0x20
    VK_PIPELINE_STAGE_GEOMETRY_SHADER_BIT = 0x40
    VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT = 0x80
    VK_PIPELINE_STAGE_EARLY_FRAGMENT_TESTS_BIT = 0x100
    VK_PIPELINE_STAGE_LATE_FRAGMENT_TESTS_BIT = 0x200
    VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT = 0x400
    VK_PIPELINE_STAGE_COMPUTE_SHADER_BIT = 0x800
    VK_PIPELINE_STAGE_TRANSFER_BIT = 0x1000
    VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT = 0x2000
    VK_PIPELINE_STAGE_HOST_BIT = 0x4000
    VK_PIPELINE_STAGE_ALL_GRAPHICS_BIT = 0x8000
    VK_PIPELINE_STAGE_ALL_COMMANDS_BIT = 0x10000
    VK_PIPELINE_STAGE_COMMAND_PREPROCESS_BIT_NV = 0x20000
    VK_PIPELINE_STAGE_CONDITIONAL_RENDERING_BIT_EXT = 0x40000
    VK_PIPELINE_STAGE_TASK_SHADER_BIT_EXT = 0x80000
    VK_PIPELINE_STAGE_MESH_SHADER_BIT_EXT = 0x100000
    VK_PIPELINE_STAGE_RAY_TRACING_SHADER_BIT_KHR = 0x200000
    VK_PIPELINE_STAGE_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR = 0x400000
    VK_PIPELINE_STAGE_FRAGMENT_DENSITY_PROCESS_BIT_EXT = 0x800000
    VK_PIPELINE_STAGE_TRANSFORM_FEEDBACK_BIT_EXT = 0x1000000
    VK_PIPELINE_STAGE_ACCELERATION_STRUCTURE_BUILD_BIT_KHR = 0x2000000
    VK_PIPELINE_STAGE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class __MIDL_IGetBindHandle_0001(IntEnum):
    BINDHANDLETYPES_APPCACHE = 0x0
    BINDHANDLETYPES_DEPENDENCY = 0x1
    BINDHANDLETYPES_COUNT = 0x2

class tagTYPEKIND(IntEnum):
    TKIND_ENUM = 0x0
    TKIND_RECORD = 0x1
    TKIND_MODULE = 0x2
    TKIND_INTERFACE = 0x3
    TKIND_DISPATCH = 0x4
    TKIND_COCLASS = 0x5
    TKIND_ALIAS = 0x6
    TKIND_UNION = 0x7
    TKIND_MAX = 0x8

class PIDMSI_STATUS_VALUE(IntEnum):
    PIDMSI_STATUS_NORMAL = 0x0
    PIDMSI_STATUS_NEW = 0x1
    PIDMSI_STATUS_PRELIM = 0x2
    PIDMSI_STATUS_DRAFT = 0x3
    PIDMSI_STATUS_INPROGRESS = 0x4
    PIDMSI_STATUS_EDIT = 0x5
    PIDMSI_STATUS_REVIEW = 0x6
    PIDMSI_STATUS_PROOF = 0x7
    PIDMSI_STATUS_FINAL = 0x8
    PIDMSI_STATUS_OTHER = 0x7FFF

class tagCALLCONV(IntEnum):
    CC_FASTCALL = 0x0
    CC_CDECL = 0x1
    CC_MSCPASCAL = 0x2
    CC_MACPASCAL = 0x3
    CC_STDCALL = 0x4
    CC_FPFASTCALL = 0x5
    CC_SYSCALL = 0x6
    CC_MPWCDECL = 0x7
    CC_MPWPASCAL = 0x8
    CC_MAX = 0x9

class tagFUNCKIND(IntEnum):
    FUNC_VIRTUAL = 0x0
    FUNC_PUREVIRTUAL = 0x1
    FUNC_NONVIRTUAL = 0x2
    FUNC_STATIC = 0x3
    FUNC_DISPATCH = 0x4

class tagVARKIND(IntEnum):
    VAR_PERINSTANCE = 0x0
    VAR_STATIC = 0x1
    VAR_CONST = 0x2
    VAR_DISPATCH = 0x3

class tagDESCKIND(IntEnum):
    DESCKIND_NONE = 0x0
    DESCKIND_FUNCDESC = 0x1
    DESCKIND_VARDESC = 0x2
    DESCKIND_TYPECOMP = 0x3
    DESCKIND_IMPLICITAPPOBJ = 0x4
    DESCKIND_MAX = 0x5

class tagSYSKIND(IntEnum):
    SYS_WIN16 = 0x0
    SYS_WIN32 = 0x1
    SYS_MAC = 0x2
    SYS_WIN64 = 0x3

class tagCHANGEKIND(IntEnum):
    CHANGEKIND_ADDMEMBER = 0x0
    CHANGEKIND_DELETEMEMBER = 0x1
    CHANGEKIND_SETNAMES = 0x2
    CHANGEKIND_SETDOCUMENTATION = 0x3
    CHANGEKIND_GENERAL = 0x4
    CHANGEKIND_INVALIDATE = 0x5
    CHANGEKIND_CHANGEFAILED = 0x6
    CHANGEKIND_MAX = 0x7

class NVGwinding(IntEnum):
    NVG_CCW = 0x1

class NVGsolidity(IntEnum):
    NVG_SOLID = 0x1

class NVGlineCap(IntEnum):
    NVG_BUTT = 0x0
    NVG_ROUND = 0x1
    NVG_SQUARE = 0x2
    NVG_BEVEL = 0x3
    NVG_MITER = 0x4

class NVGalign(IntEnum):
    NVG_ALIGN_LEFT = 0x1

class tagCOINITBASE(IntEnum):
    COINITBASE_MULTITHREADED = 0x0

class tagCLSCTX(IntEnum):
    CLSCTX_INPROC_SERVER = 0x1

class VkPhysicalDeviceType(IntEnum):
    VK_PHYSICAL_DEVICE_TYPE_OTHER = 0x0
    VK_PHYSICAL_DEVICE_TYPE_INTEGRATED_GPU = 0x1
    VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU = 0x2
    VK_PHYSICAL_DEVICE_TYPE_VIRTUAL_GPU = 0x3
    VK_PHYSICAL_DEVICE_TYPE_CPU = 0x4
    VK_PHYSICAL_DEVICE_TYPE_MAX_ENUM = 0x7FFFFFFF

class eFadeType(IntEnum):
    FadeInThenFadeOut = 0x0
    CrossFade = 0x1

class eRegionActionIssued(IntEnum):
    ERegionAction_NotRequired = 0x0
    ERegionAction_UnableToPerform = 0x1
    ERegionAction_ActionPerformed = 0x2
    ERegionAction_ThreadActionIssued = 0x3

class cCubeValue(ctypes.Structure):
    class eBlendType(IntEnum):
        EBlendType_None = 0x0
        EBlendType_LowerSlope = 0x1
        EBlendType_TileBlend = 0x2
        EBlendType_UpperSlope = 0x3

    _fields_ = [
        ("mPosition", common.Vector3f),
        ("mNormal", common.Vector3f),
        ("mfTexture", ctypes.c_float),
        ("miFlagIndex", ctypes.c_uint16),
        ("miIsEdited", ctypes.c_uint16),
        ("miOutgoingQuadInd", ctypes.c_int16 * 0x6),  # __int16[6]
        ("maMaterialValues", ctypes.c_char * 0x2C),  # std::array<float,11>
    ]
    mPosition: common.Vector3f
    mNormal: common.Vector3f
    mfTexture: float
    miFlagIndex: int
    miIsEdited: int
    miOutgoingQuadInd: list[ctypes.c_int16]
    maMaterialValues: bytes

class EgTextureResource(ctypes.Structure):
    class List(IntEnum):
        TextureElem = 0x2BC
        ImageElem = 0x2BD
        TexFormatI = 0x2BE
        TexSliceCountI = 0x2BF
        TexReference = 0x2C0
        TexMipCountI = 0x2C1
        ImgWidthI = 0x2C2
        ImgHeightI = 0x2C3
        ImgPixelStream = 0x2C4
        AvgColourR = 0x2C5
        AvgColourG = 0x2C6
        AvgColourB = 0x2C7
        AvgColourA = 0x2C8
        TexStreamingPriority = 0x2C9
        TexUltraMipCount = 0x2CA
        TexVirtualBlockWidth = 0x2CB
        TexVirtualBlockHeight = 0x2CC
        TexVirtualBlockMapped = 0x2CD


class ePolygoniser(IntEnum):
    EPolygoniser_DualContouring = 0x0
    EPolygoniser_MarchingCubes = 0x1
    EPolygoniser_Heightmap = 0x2
    EPolygoniser_NumPolygonisers = 0x3

class eSpawnGroup(IntEnum):
    ESpawnGroup_DetailObjects = 0x0
    ESpawnGroup_Objects = 0x1
    ESpawnGroup_Landmarks = 0x2
    ESpawnGroup_DistantObjects = 0x3
    ESpawnGroup_NumTypes = 0x4

class tagINVOKEKIND(IntEnum):
    INVOKE_FUNC = 0x1

class ePlanetResourceHintColouring(IntEnum):
    On = 0x0
    Off = 0x1

class eResourceLoadState(IntEnum):
    EResourceLoad_Unloaded = 0x0
    EResourceLoad_Requested = 0x1
    EResourceLoad_Loaded = 0x2

class cGcNameGenerator(ctypes.Structure):
    class StructureFlavour(IntEnum):
        Ruin = 0x0
        OutpostOrTerminal = 0x1
        Factory = 0x2
        RadioTower = 0x3
        Monolith = 0x4
        AtlasStation = 0x5
        AbandonedBuilding = 0x6
        Observatory = 0x7
        ClaimBase = 0x8
        CrashedFreighter = 0x9
        GlitchyStoryBox = 0xA
        GraveinCave = 0xB

    def Construct(): pass

    def InitWordData(): pass

    def GenerateGalacticRegionName(): pass

    def GenerateSolarSystemName(): pass

    def GenerateSpaceStationName(): pass

    def GeneratePlanetName(): pass

    def GenerateBeaconName(): pass

    def GenerateAbandonedFreighterName(): pass

    def GenerateStructureName(): pass

    def GenerateLibraryName(): pass

    def GenerateSectorName(): pass

    def GenerateNPCName(): pass

    def GenerateNPCNameWithTitle(): pass

    def GenerateSettlementNPCName(): pass

    def GenerateShipPilotName(): pass

    def GeneratePirateFaction(): pass

    def GeneratePirateName(): pass

    def GeneratePetName(): pass

    def AddPetNameSuffix(): pass

    def GenerateWeirdShipName(): pass

    def GenerateShipName(): pass

    def GenerateFreighterName(): pass

    def GenerateFrigateName(): pass

    def GenerateWeaponName(): pass

    def GenerateProcTechName(): pass

    def GenerateAbandonedFreighterCaptainName(): pass

    def GenerateAbandonedFreighterOfficerName(): pass

    def GetSeededPasswordOfficerIndex(): pass

    def GenerateAbandonedFreighterPassengerName(): pass

    def GenerateArtifactName(): pass

    def GenerateExpeditionName(): pass

    def GenerateExpeditionNameGeneral(): pass

    def GenerateExpeditionNameCombat(): pass

    def GenerateExpeditionNameExplore(): pass

    def GenerateExpeditionNameMining(): pass

    def GenerateExpeditionNameTrade(): pass

    def GenerateCreatureName(): pass

    def GenerateFloraName(): pass

    def GenerateMineralName(): pass

    def GenerateMissionName(): pass

    def GeneratePetProcNoise(): pass

    def GeneratePetFiendProcSentence(): pass

    def GenerateFriendlyDroneBeep(): pass

    def AppendAutoLocalisedAlphanumeric(): pass

    def GetAbandonedFreighterNameFromUA(): pass

    def GenerateAbandonedFreighterLogSnippet(): pass

    def GenerateAbandonedFreighterDoorText(): pass

    def GenerateAbandonedFreighterCrewManifestIntro(): pass

    def GenerateAbandonedFreighterCrewManifestScan(): pass

    def GenerateAbandonedFreighterCrewManifest(): pass

    def GenerateAbandonedFreighterCargoList(): pass

    def GenerateAbandonedFreighterLogFragment(): pass

    def GenerateAbandonedFreighterCrewApprasial(): pass

    def GenerateAbandonedFreighterSecurityLog(): pass

    def GenerateAbandonedFreighterCaptainLogLogin(): pass

    def GenerateAbandonedFreighterCaptainLog(): pass

    def GenerateAbandonedFreighterCaptainLogIndex(): pass

    def GenerateAbandonedFreighterShipAIOutput(): pass

    def GenerateAbandonedFreighterResearchNote(): pass

    def GenerateAbandonedFreighterPersonalItemList(): pass

    def GenerateAbandonedFreighterMedicalReport(): pass

    def GenerateAbandonedFreighterDisciplineRecord(): pass

    def GenerateAbandonedFreighterLocationReport(): pass

    def GenerateAbandonedFreighterPersonalRecording(): pass

    def GenerateFreighterRoomName(): pass

    def GenerateSettlementName(): pass

    def GenerateSettlementStructureArchetypeNameForBuildingClass(): pass

    def GenerateSettlementStructureName(): pass

    def GenerateSettlementPerkName(): pass

    def GetAlienNameFromFormat(): pass

    _fields_ = [
        ("maSectorLocIDs", ctypes.c_char * 0x120),  # std::vector<TkID<256>,TkSTLAllocatorShim<TkID<256>,8,-1> >
        ("gap120", ctypes.c_char * 0x8),
    ]
    maSectorLocIDs: bytes
    gap120: bytes

class VocAlienRegistrationIDs(IntEnum):
    CompanyID = 0x6F
    VocAlienPluginID = 0xDE

class HgVaCCSetIterator(ctypes.Structure):
    class enumCCSet(IntEnum):
        All = 0x0
        Morphed = 0x1
        MorphedNoEnergyPitch = 0x2
        IpadMirrored = 0x3
        Vowel = 0x4
        RandMorphTargetSet1 = 0x5
        RandMorphTargetSet2 = 0x6
        RandMorphTargetSet3 = 0x7
        RandMorphTargetSet4 = 0x8
        RandMorphTargetSet5 = 0x9
        NUM_SETS = 0xA

    _fields_ = [
        ("muCurrentSet", ctypes.c_uint32),
        ("muCurrentIndex", ctypes.c_uint32),
    ]
    muCurrentSet: int
    muCurrentIndex: int

class _SPACTION(IntEnum):
    SPACTION_NONE = 0x0
    SPACTION_MOVING = 0x1
    SPACTION_COPYING = 0x2
    SPACTION_RECYCLING = 0x3
    SPACTION_APPLYINGATTRIBS = 0x4
    SPACTION_DOWNLOADING = 0x5
    SPACTION_SEARCHING_INTERNET = 0x6
    SPACTION_CALCULATING = 0x7
    SPACTION_UPLOADING = 0x8
    SPACTION_SEARCHING_FILES = 0x9
    SPACTION_DELETING = 0xA
    SPACTION_RENAMING = 0xB
    SPACTION_FORMATTING = 0xC
    SPACTION_COPY_MOVING = 0xD

class _SPTEXT(IntEnum):
    SPTEXT_ACTIONDESCRIPTION = 0x1

class AkGlobalCallbackLocation(IntEnum):
    AkGlobalCallbackLocation_Register = 0x1
    AkGlobalCallbackLocation_Begin = 0x2
    AkGlobalCallbackLocation_PreProcessMessageQueueForRender = 0x4
    AkGlobalCallbackLocation_PostMessagesProcessed = 0x8
    AkGlobalCallbackLocation_Num = 0xD
    AkGlobalCallbackLocation_BeginRender = 0x10
    AkGlobalCallbackLocation_EndRender = 0x20
    AkGlobalCallbackLocation_End = 0x40
    AkGlobalCallbackLocation_Term = 0x80
    AkGlobalCallbackLocation_Monitor = 0x100
    AkGlobalCallbackLocation_MonitorRecap = 0x200
    AkGlobalCallbackLocation_Init = 0x400
    AkGlobalCallbackLocation_Suspend = 0x800
    AkGlobalCallbackLocation_WakeupFromSuspend = 0x1000

class AkPluginType(IntEnum):
    AkPluginTypeNone = 0x0
    AkPluginTypeCodec = 0x1
    AkPluginTypeSource = 0x2
    AkPluginTypeEffect = 0x3
    AkPluginTypeMixer = 0x6
    AkPluginTypeSink = 0x7
    AkPluginTypeGlobalExtension = 0x8
    AkPluginTypeMetadata = 0x9
    AkPluginTypeMask = 0xF

class AKRESULT(IntEnum):
    AK_NotImplemented = 0x0
    AK_Success = 0x1
    AK_Fail = 0x2
    AK_PartialSuccess = 0x3
    AK_NotCompatible = 0x4
    AK_AlreadyConnected = 0x5
    AK_InvalidFile = 0x7
    AK_AudioFileHeaderTooLarge = 0x8
    AK_MaxReached = 0x9
    AK_InvalidID = 0xE
    AK_IDNotFound = 0xF
    AK_InvalidInstanceID = 0x10
    AK_NoMoreData = 0x11
    AK_InvalidStateGroup = 0x14
    AK_ChildAlreadyHasAParent = 0x15
    AK_InvalidLanguage = 0x16
    AK_CannotAddItseflAsAChild = 0x17
    AK_InvalidParameter = 0x1F
    AK_ElementAlreadyInList = 0x23
    AK_PathNotFound = 0x24
    AK_PathNoVertices = 0x25
    AK_PathNotRunning = 0x26
    AK_PathNotPaused = 0x27
    AK_PathNodeAlreadyInList = 0x28
    AK_PathNodeNotInList = 0x29
    AK_DataNeeded = 0x2B
    AK_NoDataNeeded = 0x2C
    AK_DataReady = 0x2D
    AK_NoDataReady = 0x2E
    AK_InsufficientMemory = 0x34
    AK_Cancelled = 0x35
    AK_UnknownBankID = 0x36
    AK_BankReadError = 0x38
    AK_InvalidSwitchType = 0x39
    AK_FormatNotReady = 0x3F
    AK_WrongBankVersion = 0x40
    AK_FileNotFound = 0x42
    AK_DeviceNotReady = 0x43
    AK_BankAlreadyLoaded = 0x45
    AK_RenderedFX = 0x47
    AK_ProcessNeeded = 0x48
    AK_ProcessDone = 0x49
    AK_MemManagerNotInitialized = 0x4A
    AK_StreamMgrNotInitialized = 0x4B
    AK_SSEInstructionsNotSupported = 0x4C
    AK_Busy = 0x4D
    AK_UnsupportedChannelConfig = 0x4E
    AK_PluginMediaNotAvailable = 0x4F
    AK_MustBeVirtualized = 0x50
    AK_CommandTooLarge = 0x51
    AK_RejectedByFilter = 0x52
    AK_InvalidCustomPlatformName = 0x53
    AK_DLLCannotLoad = 0x54
    AK_DLLPathNotFound = 0x55
    AK_NoJavaVM = 0x56
    AK_OpenSLError = 0x57
    AK_PluginNotRegistered = 0x58
    AK_DataAlignmentError = 0x59
    AK_DeviceNotCompatible = 0x5A
    AK_DuplicateUniqueID = 0x5B
    AK_InitBankNotLoaded = 0x5C
    AK_DeviceNotFound = 0x5D
    AK_PlayingIDNotFound = 0x5E
    AK_InvalidFloatValue = 0x5F
    AK_FileFormatMismatch = 0x60
    AK_NoDistinctListener = 0x61
    AK_ACP_Error = 0x62
    AK_ResourceInUse = 0x63

class OfflineFolderStatus(IntEnum):
    OFS_ONLINE = 0x0
    OFS_OFFLINE = 0x1
    OFS_SERVERBACK = 0x2
    OFS_DIRTYCACHE = 0x3
    OFS_INACTIVE = 0xFFFFFFFF

class AkOpenMode(IntEnum):
    AK_OpenModeRead = 0x0
    AK_OpenModeWrite = 0x1
    AK_OpenModeWriteOvrwr = 0x2
    AK_OpenModeReadWrite = 0x3

class tagBANDSITECID(IntEnum):
    BSID_BANDADDED = 0x0
    BSID_BANDREMOVED = 0x1

class tagMENUPOPUPSELECT(IntEnum):
    MPOS_EXECUTE = 0x0
    MPOS_FULLCANCEL = 0x1
    MPOS_CANCELLEVEL = 0x2
    MPOS_SELECTLEFT = 0x3
    MPOS_SELECTRIGHT = 0x4
    MPOS_CHILDTRACKING = 0x5

class FILE_USAGE_TYPE(IntEnum):
    FUT_PLAYING = 0x0
    FUT_EDITING = 0x1
    FUT_GENERIC = 0x2

class ASSOCIATIONLEVEL(IntEnum):
    AL_MACHINE = 0x0
    AL_EFFECTIVE = 0x1
    AL_USER = 0x2

class ASSOCIATIONTYPE(IntEnum):
    AT_FILEEXTENSION = 0x0
    AT_URLPROTOCOL = 0x1
    AT_STARTMENUCLIENT = 0x2
    AT_MIMETYPE = 0x3

class FFFP_MODE(IntEnum):
    FFFP_EXACTMATCH = 0x0
    FFFP_NEARESTPARENTMATCH = 0x1

class tagCONDITION_TYPE(IntEnum):
    CT_AND_CONDITION = 0x0
    CT_OR_CONDITION = 0x1
    CT_NOT_CONDITION = 0x2
    CT_LEAF_CONDITION = 0x3

class tagCONDITION_OPERATION(IntEnum):
    COP_IMPLICIT = 0x0
    COP_EQUAL = 0x1
    COP_NOTEQUAL = 0x2
    COP_LESSTHAN = 0x3
    COP_GREATERTHAN = 0x4
    COP_LESSTHANOREQUAL = 0x5
    COP_GREATERTHANOREQUAL = 0x6
    COP_VALUE_STARTSWITH = 0x7
    COP_VALUE_ENDSWITH = 0x8
    COP_VALUE_CONTAINS = 0x9
    COP_VALUE_NOTCONTAINS = 0xA
    COP_DOSWILDCARDS = 0xB
    COP_WORD_EQUAL = 0xC
    COP_WORD_STARTSWITH = 0xD
    COP_APPLICATION_SPECIFIC = 0xE

class LogConstants(IntEnum):
    PrefixSize = 0x7

class PKA_FLAGS(IntEnum):
    PKA_SET = 0x0
    PKA_APPEND = 0x1
    PKA_DELETE = 0x2

class CPVIEW(IntEnum):
    CPVIEW_CLASSIC = 0x0
    CPVIEW_CATEGORY = 0x1

class KNOWNDESTCATEGORY(IntEnum):
    KDC_FREQUENT = 0x1

class APPDOCLISTTYPE(IntEnum):
    ADLT_RECENT = 0x0
    ADLT_FREQUENT = 0x1

class DEFAULTSAVEFOLDERTYPE(IntEnum):
    DSFT_DETECT = 0x1
    DSFT_PRIVATE = 0x2
    DSFT_PUBLIC = 0x3

class AkAudioDeviceState(IntEnum):
    AkDeviceState_Unknown = 0x0
    AkDeviceState_Active = 0x1
    AkDeviceState_Disabled = 0x2
    AkDeviceState_NotPresent = 0x4
    AkDeviceState_Unplugged = 0x8
    AkDeviceState_All = 0xF

class AkCurveInterpolation(IntEnum):
    AkCurveInterpolation_Log3 = 0x0
    AkCurveInterpolation_Sine = 0x1
    AkCurveInterpolation_Log1 = 0x2
    AkCurveInterpolation_InvSCurve = 0x3
    AkCurveInterpolation_Linear = 0x4
    AkCurveInterpolation_SCurve = 0x5
    AkCurveInterpolation_Exp1 = 0x6
    AkCurveInterpolation_SineRecip = 0x7
    AkCurveInterpolation_Exp3 = 0x8
    AkCurveInterpolation_Constant = 0x9

class EC_HOST_UI_MODE(IntEnum):
    ECHUIM_DESKTOP = 0x0
    ECHUIM_IMMERSIVE = 0x1
    ECHUIM_SYSTEM_LAUNCHER = 0x2

class APPLICATION_VIEW_STATE(IntEnum):
    AVS_FULLSCREEN_LANDSCAPE = 0x0
    AVS_FILLED = 0x1
    AVS_SNAPPED = 0x2
    AVS_FULLSCREEN_PORTRAIT = 0x3

class EDGE_GESTURE_KIND(IntEnum):
    EGK_TOUCH = 0x0
    EGK_KEYBOARD = 0x1
    EGK_MOUSE = 0x2

class NATIVE_DISPLAY_ORIENTATION(IntEnum):
    NDO_LANDSCAPE = 0x0
    NDO_PORTRAIT = 0x1

class APPLICATION_VIEW_ORIENTATION(IntEnum):
    AVO_LANDSCAPE = 0x0
    AVO_PORTRAIT = 0x1

class AkPanningRule(IntEnum):
    AkPanningRule_Speakers = 0x0
    AkPanningRule_Headphones = 0x1

class PLACEHOLDER_STATES(IntEnum):
    PS_NONE = 0x0
    PS_MARKED_FOR_OFFLINE_AVAILABILITY = 0x1
    PS_FULL_PRIMARY_STREAM_AVAILABLE = 0x2
    PS_CREATE_FILE_ACCESSIBLE = 0x4
    PS_DEFAULT = 0x7
    PS_CLOUDFILE_PLACEHOLDER = 0x8
    PS_ALL = 0xF

class MERGE_UPDATE_STATUS(IntEnum):
    MUS_COMPLETE = 0x0
    MUS_USERINPUTNEEDED = 0x1
    MUS_FAILED = 0x2

class AkChannelConfigType(IntEnum):
    AK_ChannelConfigType_Anonymous = 0x0
    AK_ChannelConfigType_Standard = 0x1
    AK_ChannelConfigType_Ambisonic = 0x2
    AK_ChannelConfigType_Objects = 0x3
    AK_ChannelConfigType_UseDeviceMain = 0xE
    AK_ChannelConfigType_UseDevicePassthrough = 0xF

class FLYOUT_PLACEMENT(IntEnum):
    FP_DEFAULT = 0x0
    FP_ABOVE = 0x1
    FP_BELOW = 0x2
    FP_LEFT = 0x3
    FP_RIGHT = 0x4

class BANNER_NOTIFICATION_EVENT(IntEnum):
    BNE_Rendered = 0x0
    BNE_Hovered = 0x1
    BNE_Closed = 0x2
    BNE_Dismissed = 0x3
    BNE_Button1Clicked = 0x4
    BNE_Button2Clicked = 0x5

class AkFloorPlane(IntEnum):
    AkFloorPlane_XZ = 0x0
    AkFloorPlane_XY = 0x1
    AkFloorPlane_YZ = 0x2

class SYNC_ENGINE_STATE_FLAGS(IntEnum):
    SESF_NONE = 0x0
    SESF_SERVICE_QUOTA_NEARING_LIMIT = 0x1
    SESF_SERVICE_QUOTA_EXCEEDED_LIMIT = 0x2
    SESF_AUTHENTICATION_ERROR = 0x4
    SESF_PAUSED_DUE_TO_METERED_NETWORK = 0x8
    SESF_PAUSED_DUE_TO_DISK_SPACE_FULL = 0x10
    SESF_PAUSED_DUE_TO_CLIENT_POLICY = 0x20
    SESF_PAUSED_DUE_TO_SERVICE_POLICY = 0x40
    SESF_SERVICE_UNAVAILABLE = 0x80
    SESF_PAUSED_DUE_TO_USER_REQUEST = 0x100
    SESF_ALL_FLAGS = 0x1FF

class _SIGDN(IntEnum):
    SIGDN_NORMALDISPLAY = 0x0
    SIGDN_PARENTRELATIVEPARSING = 0x80018001
    SIGDN_DESKTOPABSOLUTEPARSING = 0x80028000
    SIGDN_PARENTRELATIVEEDITING = 0x80031001
    SIGDN_DESKTOPABSOLUTEEDITING = 0x8004C000
    SIGDN_FILESYSPATH = 0x80058000
    SIGDN_URL = 0x80068000
    SIGDN_PARENTRELATIVEFORADDRESSBAR = 0x8007C001
    SIGDN_PARENTRELATIVE = 0x80080001
    SIGDN_PARENTRELATIVEFORUI = 0x80094001

class AkSpeakerPanningType(IntEnum):
    AK_DirectSpeakerAssignment = 0x0
    AK_BalanceFadeHeight = 0x1
    AK_SteeringPanner = 0x2

class AkMeteringFlags(IntEnum):
    AK_NoMetering = 0x0
    AK_EnableBusMeter_Peak = 0x1
    AK_EnableBusMeter_TruePeak = 0x2
    AK_EnableBusMeter_RMS = 0x4
    AK_EnableBusMeter_KPower = 0x10
    AK_EnableBusMeter_3DMeter = 0x20

class AkConnectionType(IntEnum):
    ConnectionType_Direct = 0x0
    ConnectionType_GameDefSend = 0x1
    ConnectionType_UserDefSend = 0x2
    ConnectionType_ReflectionsSend = 0x3

class VkCompareOp(IntEnum):
    VK_COMPARE_OP_NEVER = 0x0
    VK_COMPARE_OP_LESS = 0x1
    VK_COMPARE_OP_EQUAL = 0x2
    VK_COMPARE_OP_LESS_OR_EQUAL = 0x3
    VK_COMPARE_OP_GREATER = 0x4
    VK_COMPARE_OP_NOT_EQUAL = 0x5
    VK_COMPARE_OP_GREATER_OR_EQUAL = 0x6
    VK_COMPARE_OP_ALWAYS = 0x7
    VK_COMPARE_OP_MAX_ENUM = 0x7FFFFFFF

class VkBorderColor(IntEnum):
    VK_BORDER_COLOR_FLOAT_TRANSPARENT_BLACK = 0x0
    VK_BORDER_COLOR_INT_TRANSPARENT_BLACK = 0x1
    VK_BORDER_COLOR_FLOAT_OPAQUE_BLACK = 0x2
    VK_BORDER_COLOR_INT_OPAQUE_BLACK = 0x3
    VK_BORDER_COLOR_FLOAT_OPAQUE_WHITE = 0x4
    VK_BORDER_COLOR_INT_OPAQUE_WHITE = 0x5
    VK_BORDER_COLOR_FLOAT_CUSTOM_EXT = 0x3B9F2B1B
    VK_BORDER_COLOR_INT_CUSTOM_EXT = 0x3B9F2B1C
    VK_BORDER_COLOR_MAX_ENUM = 0x7FFFFFFF

class Ak3DPositionType(IntEnum):
    AK_3DPositionType_Emitter = 0x0
    AK_3DPositionType_EmitterWithAutomation = 0x1
    AK_3DPositionType_ListenerWithAutomation = 0x2

class Ak3DSpatializationMode(IntEnum):
    AK_SpatializationMode_None = 0x0
    AK_SpatializationMode_PositionOnly = 0x1
    AK_SpatializationMode_PositionAndOrientation = 0x2

class AkMoveMethod(IntEnum):
    AK_MoveBegin = 0x0
    AK_MoveCurrent = 0x1
    AK_MoveEnd = 0x2

class AkStmStatus(IntEnum):
    AK_StmStatusIdle = 0x0
    AK_StmStatusCompleted = 0x1
    AK_StmStatusPending = 0x2
    AK_StmStatusCancelled = 0x3
    AK_StmStatusError = 0x4

class eWFCBaseClusterNavGraphConstructionStage(IntEnum):
    EClusterNavGraphConstructionStage_WaitToStart = 0x0
    EClusterNavGraphConstructionStage_PerformInitialSetup = 0x1
    EClusterNavGraphConstructionStage_WaitForTerrainGeneration = 0x2
    EClusterNavGraphConstructionStage_WaitForBuildingPlacements = 0x3
    EClusterNavGraphConstructionStage_DetermineBuildingShapes = 0x4
    EClusterNavGraphConstructionStage_AddBuildingPerimeterNodes = 0x5
    EClusterNavGraphConstructionStage_ConnectEntranceNodes = 0x6
    EClusterNavGraphConstructionStage_MergeNodes = 0x7
    EClusterNavGraphConstructionStage_CleanUpRedundantNodes = 0x8
    EClusterNavGraphConstructionStage_CreateShortcuts = 0x9
    EClusterNavGraphConstructionStage_UpdateNavArea = 0xA

class eTkClearFlags(IntEnum):
    EClearFlag_DepthBuffer = 0x1

class VkPipelineBindPoint(IntEnum):
    VK_PIPELINE_BIND_POINT_GRAPHICS = 0x0
    VK_PIPELINE_BIND_POINT_COMPUTE = 0x1
    VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR = 0x3B9D4E88
    VK_PIPELINE_BIND_POINT_SUBPASS_SHADING_HUAWEI = 0x3BA06B6B
    VK_PIPELINE_BIND_POINT_MAX_ENUM = 0x7FFFFFFF

class VkStencilOp(IntEnum):
    VK_STENCIL_OP_KEEP = 0x0
    VK_STENCIL_OP_ZERO = 0x1
    VK_STENCIL_OP_REPLACE = 0x2
    VK_STENCIL_OP_INCREMENT_AND_CLAMP = 0x3
    VK_STENCIL_OP_DECREMENT_AND_CLAMP = 0x4
    VK_STENCIL_OP_INVERT = 0x5
    VK_STENCIL_OP_INCREMENT_AND_WRAP = 0x6
    VK_STENCIL_OP_DECREMENT_AND_WRAP = 0x7
    VK_STENCIL_OP_MAX_ENUM = 0x7FFFFFFF

class VkBlendFactor(IntEnum):
    VK_BLEND_FACTOR_ZERO = 0x0
    VK_BLEND_FACTOR_ONE = 0x1
    VK_BLEND_FACTOR_SRC_COLOR = 0x2
    VK_BLEND_FACTOR_ONE_MINUS_SRC_COLOR = 0x3
    VK_BLEND_FACTOR_DST_COLOR = 0x4
    VK_BLEND_FACTOR_ONE_MINUS_DST_COLOR = 0x5
    VK_BLEND_FACTOR_SRC_ALPHA = 0x6
    VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA = 0x7
    VK_BLEND_FACTOR_DST_ALPHA = 0x8
    VK_BLEND_FACTOR_ONE_MINUS_DST_ALPHA = 0x9
    VK_BLEND_FACTOR_CONSTANT_COLOR = 0xA
    VK_BLEND_FACTOR_ONE_MINUS_CONSTANT_COLOR = 0xB
    VK_BLEND_FACTOR_CONSTANT_ALPHA = 0xC
    VK_BLEND_FACTOR_ONE_MINUS_CONSTANT_ALPHA = 0xD
    VK_BLEND_FACTOR_SRC_ALPHA_SATURATE = 0xE
    VK_BLEND_FACTOR_SRC1_COLOR = 0xF
    VK_BLEND_FACTOR_ONE_MINUS_SRC1_COLOR = 0x10
    VK_BLEND_FACTOR_SRC1_ALPHA = 0x11
    VK_BLEND_FACTOR_ONE_MINUS_SRC1_ALPHA = 0x12
    VK_BLEND_FACTOR_MAX_ENUM = 0x7FFFFFFF

class VkBlendOp(IntEnum):
    VK_BLEND_OP_ADD = 0x0
    VK_BLEND_OP_SUBTRACT = 0x1
    VK_BLEND_OP_REVERSE_SUBTRACT = 0x2
    VK_BLEND_OP_MIN = 0x3
    VK_BLEND_OP_MAX = 0x4
    VK_BLEND_OP_ZERO_EXT = 0x3B9D0C20
    VK_BLEND_OP_SRC_EXT = 0x3B9D0C21
    VK_BLEND_OP_DST_EXT = 0x3B9D0C22
    VK_BLEND_OP_SRC_OVER_EXT = 0x3B9D0C23
    VK_BLEND_OP_DST_OVER_EXT = 0x3B9D0C24
    VK_BLEND_OP_SRC_IN_EXT = 0x3B9D0C25
    VK_BLEND_OP_DST_IN_EXT = 0x3B9D0C26
    VK_BLEND_OP_SRC_OUT_EXT = 0x3B9D0C27
    VK_BLEND_OP_DST_OUT_EXT = 0x3B9D0C28
    VK_BLEND_OP_SRC_ATOP_EXT = 0x3B9D0C29
    VK_BLEND_OP_DST_ATOP_EXT = 0x3B9D0C2A
    VK_BLEND_OP_XOR_EXT = 0x3B9D0C2B
    VK_BLEND_OP_MULTIPLY_EXT = 0x3B9D0C2C
    VK_BLEND_OP_SCREEN_EXT = 0x3B9D0C2D
    VK_BLEND_OP_OVERLAY_EXT = 0x3B9D0C2E
    VK_BLEND_OP_DARKEN_EXT = 0x3B9D0C2F
    VK_BLEND_OP_LIGHTEN_EXT = 0x3B9D0C30
    VK_BLEND_OP_COLORDODGE_EXT = 0x3B9D0C31
    VK_BLEND_OP_COLORBURN_EXT = 0x3B9D0C32
    VK_BLEND_OP_HARDLIGHT_EXT = 0x3B9D0C33
    VK_BLEND_OP_SOFTLIGHT_EXT = 0x3B9D0C34
    VK_BLEND_OP_DIFFERENCE_EXT = 0x3B9D0C35
    VK_BLEND_OP_EXCLUSION_EXT = 0x3B9D0C36
    VK_BLEND_OP_INVERT_EXT = 0x3B9D0C37
    VK_BLEND_OP_INVERT_RGB_EXT = 0x3B9D0C38
    VK_BLEND_OP_LINEARDODGE_EXT = 0x3B9D0C39
    VK_BLEND_OP_LINEARBURN_EXT = 0x3B9D0C3A
    VK_BLEND_OP_VIVIDLIGHT_EXT = 0x3B9D0C3B
    VK_BLEND_OP_LINEARLIGHT_EXT = 0x3B9D0C3C
    VK_BLEND_OP_PINLIGHT_EXT = 0x3B9D0C3D
    VK_BLEND_OP_HARDMIX_EXT = 0x3B9D0C3E
    VK_BLEND_OP_HSL_HUE_EXT = 0x3B9D0C3F
    VK_BLEND_OP_HSL_SATURATION_EXT = 0x3B9D0C40
    VK_BLEND_OP_HSL_COLOR_EXT = 0x3B9D0C41
    VK_BLEND_OP_HSL_LUMINOSITY_EXT = 0x3B9D0C42
    VK_BLEND_OP_PLUS_EXT = 0x3B9D0C43
    VK_BLEND_OP_PLUS_CLAMPED_EXT = 0x3B9D0C44
    VK_BLEND_OP_PLUS_CLAMPED_ALPHA_EXT = 0x3B9D0C45
    VK_BLEND_OP_PLUS_DARKER_EXT = 0x3B9D0C46
    VK_BLEND_OP_MINUS_EXT = 0x3B9D0C47
    VK_BLEND_OP_MINUS_CLAMPED_EXT = 0x3B9D0C48
    VK_BLEND_OP_CONTRAST_EXT = 0x3B9D0C49
    VK_BLEND_OP_INVERT_OVG_EXT = 0x3B9D0C4A
    VK_BLEND_OP_RED_EXT = 0x3B9D0C4B
    VK_BLEND_OP_GREEN_EXT = 0x3B9D0C4C
    VK_BLEND_OP_BLUE_EXT = 0x3B9D0C4D
    VK_BLEND_OP_MAX_ENUM = 0x7FFFFFFF

class VkDynamicState(IntEnum):
    VK_DYNAMIC_STATE_VIEWPORT = 0x0
    VK_DYNAMIC_STATE_SCISSOR = 0x1
    VK_DYNAMIC_STATE_LINE_WIDTH = 0x2
    VK_DYNAMIC_STATE_DEPTH_BIAS = 0x3
    VK_DYNAMIC_STATE_BLEND_CONSTANTS = 0x4
    VK_DYNAMIC_STATE_DEPTH_BOUNDS = 0x5
    VK_DYNAMIC_STATE_STENCIL_COMPARE_MASK = 0x6
    VK_DYNAMIC_STATE_STENCIL_WRITE_MASK = 0x7
    VK_DYNAMIC_STATE_STENCIL_REFERENCE = 0x8
    VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_NV = 0x3B9C1DD8
    VK_DYNAMIC_STATE_DISCARD_RECTANGLE_EXT = 0x3B9C4CB8
    VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_EXT = 0x3B9CF898
    VK_DYNAMIC_STATE_VIEWPORT_SHADING_RATE_PALETTE_NV = 0x3B9D4AA4
    VK_DYNAMIC_STATE_VIEWPORT_COARSE_SAMPLE_ORDER_NV = 0x3B9D4AA6
    VK_DYNAMIC_STATE_EXCLUSIVE_SCISSOR_NV = 0x3B9DEAC9
    VK_DYNAMIC_STATE_FRAGMENT_SHADING_RATE_KHR = 0x3B9E3CD0
    VK_DYNAMIC_STATE_LINE_STIPPLE_EXT = 0x3B9EBDB8
    VK_DYNAMIC_STATE_CULL_MODE = 0x3B9EDCF8
    VK_DYNAMIC_STATE_FRONT_FACE = 0x3B9EDCF9
    VK_DYNAMIC_STATE_PRIMITIVE_TOPOLOGY = 0x3B9EDCFA
    VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT = 0x3B9EDCFB
    VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT = 0x3B9EDCFC
    VK_DYNAMIC_STATE_VERTEX_INPUT_BINDING_STRIDE = 0x3B9EDCFD
    VK_DYNAMIC_STATE_DEPTH_TEST_ENABLE = 0x3B9EDCFE
    VK_DYNAMIC_STATE_DEPTH_WRITE_ENABLE = 0x3B9EDCFF
    VK_DYNAMIC_STATE_DEPTH_COMPARE_OP = 0x3B9EDD00
    VK_DYNAMIC_STATE_DEPTH_BOUNDS_TEST_ENABLE = 0x3B9EDD01
    VK_DYNAMIC_STATE_STENCIL_TEST_ENABLE = 0x3B9EDD02
    VK_DYNAMIC_STATE_STENCIL_OP = 0x3B9EDD03
    VK_DYNAMIC_STATE_RAY_TRACING_PIPELINE_STACK_SIZE_KHR = 0x3BA01578
    VK_DYNAMIC_STATE_VERTEX_INPUT_EXT = 0x3BA02900
    VK_DYNAMIC_STATE_PATCH_CONTROL_POINTS_EXT = 0x3BA08AA8
    VK_DYNAMIC_STATE_RASTERIZER_DISCARD_ENABLE = 0x3BA08AA9
    VK_DYNAMIC_STATE_DEPTH_BIAS_ENABLE = 0x3BA08AAA
    VK_DYNAMIC_STATE_LOGIC_OP_EXT = 0x3BA08AAB
    VK_DYNAMIC_STATE_PRIMITIVE_RESTART_ENABLE = 0x3BA08AAC
    VK_DYNAMIC_STATE_COLOR_WRITE_ENABLE_EXT = 0x3BA09A48
    VK_DYNAMIC_STATE_TESSELLATION_DOMAIN_ORIGIN_EXT = 0x3BA1BB5A
    VK_DYNAMIC_STATE_DEPTH_CLAMP_ENABLE_EXT = 0x3BA1BB5B
    VK_DYNAMIC_STATE_POLYGON_MODE_EXT = 0x3BA1BB5C
    VK_DYNAMIC_STATE_RASTERIZATION_SAMPLES_EXT = 0x3BA1BB5D
    VK_DYNAMIC_STATE_SAMPLE_MASK_EXT = 0x3BA1BB5E
    VK_DYNAMIC_STATE_ALPHA_TO_COVERAGE_ENABLE_EXT = 0x3BA1BB5F
    VK_DYNAMIC_STATE_ALPHA_TO_ONE_ENABLE_EXT = 0x3BA1BB60
    VK_DYNAMIC_STATE_LOGIC_OP_ENABLE_EXT = 0x3BA1BB61
    VK_DYNAMIC_STATE_COLOR_BLEND_ENABLE_EXT = 0x3BA1BB62
    VK_DYNAMIC_STATE_COLOR_BLEND_EQUATION_EXT = 0x3BA1BB63
    VK_DYNAMIC_STATE_COLOR_WRITE_MASK_EXT = 0x3BA1BB64
    VK_DYNAMIC_STATE_RASTERIZATION_STREAM_EXT = 0x3BA1BB65
    VK_DYNAMIC_STATE_CONSERVATIVE_RASTERIZATION_MODE_EXT = 0x3BA1BB66
    VK_DYNAMIC_STATE_EXTRA_PRIMITIVE_OVERESTIMATION_SIZE_EXT = 0x3BA1BB67
    VK_DYNAMIC_STATE_DEPTH_CLIP_ENABLE_EXT = 0x3BA1BB68
    VK_DYNAMIC_STATE_SAMPLE_LOCATIONS_ENABLE_EXT = 0x3BA1BB69
    VK_DYNAMIC_STATE_COLOR_BLEND_ADVANCED_EXT = 0x3BA1BB6A
    VK_DYNAMIC_STATE_PROVOKING_VERTEX_MODE_EXT = 0x3BA1BB6B
    VK_DYNAMIC_STATE_LINE_RASTERIZATION_MODE_EXT = 0x3BA1BB6C
    VK_DYNAMIC_STATE_LINE_STIPPLE_ENABLE_EXT = 0x3BA1BB6D
    VK_DYNAMIC_STATE_DEPTH_CLIP_NEGATIVE_ONE_TO_ONE_EXT = 0x3BA1BB6E
    VK_DYNAMIC_STATE_VIEWPORT_W_SCALING_ENABLE_NV = 0x3BA1BB6F
    VK_DYNAMIC_STATE_VIEWPORT_SWIZZLE_NV = 0x3BA1BB70
    VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_ENABLE_NV = 0x3BA1BB71
    VK_DYNAMIC_STATE_COVERAGE_TO_COLOR_LOCATION_NV = 0x3BA1BB72
    VK_DYNAMIC_STATE_COVERAGE_MODULATION_MODE_NV = 0x3BA1BB73
    VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_ENABLE_NV = 0x3BA1BB74
    VK_DYNAMIC_STATE_COVERAGE_MODULATION_TABLE_NV = 0x3BA1BB75
    VK_DYNAMIC_STATE_SHADING_RATE_IMAGE_ENABLE_NV = 0x3BA1BB76
    VK_DYNAMIC_STATE_REPRESENTATIVE_FRAGMENT_TEST_ENABLE_NV = 0x3BA1BB77
    VK_DYNAMIC_STATE_COVERAGE_REDUCTION_MODE_NV = 0x3BA1BB78
    VK_DYNAMIC_STATE_MAX_ENUM = 0x7FFFFFFF

class VkLogicOp(IntEnum):
    VK_LOGIC_OP_CLEAR = 0x0
    VK_LOGIC_OP_AND = 0x1
    VK_LOGIC_OP_AND_REVERSE = 0x2
    VK_LOGIC_OP_COPY = 0x3
    VK_LOGIC_OP_AND_INVERTED = 0x4
    VK_LOGIC_OP_NO_OP = 0x5
    VK_LOGIC_OP_XOR = 0x6
    VK_LOGIC_OP_OR = 0x7
    VK_LOGIC_OP_NOR = 0x8
    VK_LOGIC_OP_EQUIVALENT = 0x9
    VK_LOGIC_OP_INVERT = 0xA
    VK_LOGIC_OP_OR_REVERSE = 0xB
    VK_LOGIC_OP_COPY_INVERTED = 0xC
    VK_LOGIC_OP_OR_INVERTED = 0xD
    VK_LOGIC_OP_NAND = 0xE
    VK_LOGIC_OP_SET = 0xF
    VK_LOGIC_OP_MAX_ENUM = 0x7FFFFFFF

class VkAttachmentLoadOp(IntEnum):
    VK_ATTACHMENT_LOAD_OP_LOAD = 0x0
    VK_ATTACHMENT_LOAD_OP_CLEAR = 0x1
    VK_ATTACHMENT_LOAD_OP_DONT_CARE = 0x2
    VK_ATTACHMENT_LOAD_OP_NONE_EXT = 0x3BA0E480
    VK_ATTACHMENT_LOAD_OP_MAX_ENUM = 0x7FFFFFFF

class VkShaderStageFlagBits(IntEnum):
    VK_SHADER_STAGE_VERTEX_BIT = 0x1
    VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT = 0x2
    VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT = 0x4
    VK_SHADER_STAGE_GEOMETRY_BIT = 0x8
    VK_SHADER_STAGE_FRAGMENT_BIT = 0x10
    VK_SHADER_STAGE_ALL_GRAPHICS = 0x1F
    VK_SHADER_STAGE_COMPUTE_BIT = 0x20
    VK_SHADER_STAGE_TASK_BIT_EXT = 0x40
    VK_SHADER_STAGE_MESH_BIT_EXT = 0x80
    VK_SHADER_STAGE_RAYGEN_BIT_KHR = 0x100
    VK_SHADER_STAGE_ANY_HIT_BIT_KHR = 0x200
    VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR = 0x400
    VK_SHADER_STAGE_MISS_BIT_KHR = 0x800
    VK_SHADER_STAGE_INTERSECTION_BIT_KHR = 0x1000
    VK_SHADER_STAGE_CALLABLE_BIT_KHR = 0x2000
    VK_SHADER_STAGE_SUBPASS_SHADING_BIT_HUAWEI = 0x4000
    VK_SHADER_STAGE_ALL = 0x7FFFFFFF

class VkCullModeFlagBits(IntEnum):
    VK_CULL_MODE_NONE = 0x0
    VK_CULL_MODE_FRONT_BIT = 0x1
    VK_CULL_MODE_BACK_BIT = 0x2
    VK_CULL_MODE_FRONT_AND_BACK = 0x3
    VK_CULL_MODE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkStencilFaceFlagBits(IntEnum):
    VK_STENCIL_FACE_FRONT_BIT = 0x1
    VK_STENCIL_FACE_BACK_BIT = 0x2
    VK_STENCIL_FACE_FRONT_AND_BACK = 0x3
    VK_STENCIL_FACE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class cGcAsyncLoadOps(ctypes.Structure):
    class Operation(IntEnum):
        DrainFileIO = 0x0
        DrainShaders = 0x1
        DrainTasks = 0x2

    class RunResult(IntEnum):
        NothingEnqueued = 0x0
        InProgress = 0x1
        OpsJustCompleted = 0x2

    def Run(): pass

    _fields_ = [
        ("mMidPhaseOps", ctypes.c_char * 0x40),  # cTkStackVector<enum cGcAsyncLoadOps::Operation,4,-1>
        ("mpShaderCompilationState", ctypes.c_ulonglong),
    ]
    mMidPhaseOps: bytes
    mpShaderCompilationState: int

class SceFiosOpFlags(IntEnum):
    SCE_FIOS_OPFLAG_IMMED = 0x1
    SCE_FIOS_OPFLAG_DONTUSECACHE = 0x2
    SCE_FIOS_OPFLAG_DONTFILLRAMCACHE = 0x4
    SCE_FIOS_OPFLAG_DONTFILLDISKCACHE = 0x8
    SCE_FIOS_OPFLAG_DONTFILLCACHE = 0xC
    SCE_FIOS_OPFLAG_NOCACHE = 0xE
    SCE_FIOS_OPFLAG_CACHEPERSIST = 0x10
    SCE_FIOS_OPFLAG_UNCACHEDBUFFER = 0x20
    SCE_FIOS_OPFLAG_NONDMABUFFER = 0x40
    SCE_FIOS_OPFLAG_PERSISTENTPATH = 0x80
    SCE_FIOS_OPFLAG_ALWAYSFILLRAMCACHE = 0x200
    SCE_FIOS_OPFLAG_READONLYCACHE = 0x400
    SCE_FIOS_OPFLAG_REUSECACHE = 0x800

class SceFiosOpEvents(IntEnum):
    SCE_FIOS_OPEVENT_COMPLETE = 0x1
    SCE_FIOS_OPEVENT_DELETE = 0x2
    SCE_FIOS_OPEVENT_START = 0x3

class SceFiosWhence(IntEnum):
    SCE_FIOS_SEEK_SET = 0x0
    SCE_FIOS_SEEK_CUR = 0x1
    SCE_FIOS_SEEK_END = 0x2

class SceFiosOpenFlags(IntEnum):
    SCE_FIOS_O_READ = 0x1
    SCE_FIOS_O_WRITE = 0x2
    SCE_FIOS_O_RDWR = 0x3
    SCE_FIOS_O_APPEND = 0x4
    SCE_FIOS_O_CREAT = 0x8
    SCE_FIOS_O_TRUNC = 0x10
    SCE_FIOS_O_DIRECT = 0x1000

class SceFiosOverlayLimits(IntEnum):
    SCE_FIOS_OVERLAY_MAX_OVERLAYS = 0x40

class SceFiosOverlayType(IntEnum):
    SCE_FIOS_OVERLAY_TYPE_OPAQUE = 0x0
    SCE_FIOS_OVERLAY_TYPE_TRANSLUCENT = 0x1
    SCE_FIOS_OVERLAY_TYPE_NEWER = 0x2
    SCE_FIOS_OVERLAY_TYPE_WRITABLE = 0x3

class SceFiosIOSchedulerAlgorithm(IntEnum):
    SCE_FIOS_IO_SCHED_DEADLINE = 0x0
    SCE_FIOS_IO_SCHED_PRIORITY = 0x1
    SCE_FIOS_IO_SCHED_FIFO = 0x2
    SCE_FIOS_IO_SCHED_MAXIMUM = 0x4

class SceFiosIOThreadCount(IntEnum):
    SCE_FIOS_IO_THREAD_COUNT_MIN = 0x1

class SceFiosSchedulerThreadCount(IntEnum):
    SCE_FIOS_SCHEDULER_THREAD_COUNT_MIN = 0x1
    SCE_FIOS_SCHEDULER_THREAD_COUNT_MAX = 0x3

class SceFiosPsarcDearchiverFlags(IntEnum):
    SCE_FIOS_PSARC_DEARCHIVER_MOUNT_TRANSLUCENT = 0x1

class FiosMemorySizes(IntEnum):
    Fios_DhStorage = 0x470
    Fios_FhStorage = 0x8D040
    Fios_OpStorage = 0xA0040
    Fios_ChunkStorage = 0xA0400

class SceFiosThreadType(IntEnum):
    SCE_FIOS_IO_THREAD = 0x0
    SCE_FIOS_DECOMPRESSOR_THREAD = 0x1
    SCE_FIOS_CALLBACK_THREAD = 0x2
    SCE_FIOS_THREAD_TYPES = 0x3

class FILETYPEATTRIBUTEFLAGS(IntEnum):
    FTA_None = 0x0
    FTA_Exclude = 0x1
    FTA_Show = 0x2
    FTA_HasExtension = 0x4
    FTA_NoEdit = 0x8
    FTA_NoRemove = 0x10
    FTA_NoNewVerb = 0x20
    FTA_NoEditVerb = 0x40
    FTA_NoRemoveVerb = 0x80
    FTA_NoEditDesc = 0x100
    FTA_NoEditIcon = 0x200
    FTA_NoEditDflt = 0x400
    FTA_NoEditVerbCmd = 0x800
    FTA_NoEditVerbExe = 0x1000
    FTA_NoDDE = 0x2000
    FTA_NoEditMIME = 0x8000
    FTA_OpenIsSafe = 0x10000
    FTA_AlwaysUnsafe = 0x20000
    FTA_NoRecentDocs = 0x100000
    FTA_SafeForElevation = 0x200000
    FTA_AlwaysUseDirectInvoke = 0x400000

class KF_CATEGORY(IntEnum):
    KF_CATEGORY_VIRTUAL = 0x1
    KF_CATEGORY_FIXED = 0x2
    KF_CATEGORY_COMMON = 0x3
    KF_CATEGORY_PERUSER = 0x4

class LIBRARYFOLDERFILTER(IntEnum):
    LFF_FORCEFILESYSTEM = 0x1
    LFF_STORAGEITEMS = 0x2
    LFF_ALLITEMS = 0x3

class LIBRARYOPTIONFLAGS(IntEnum):
    LOF_DEFAULT = 0x0
    LOF_PINNEDTONAVPANE = 0x1

class LIBRARYSAVEFLAGS(IntEnum):
    LSF_FAILIFTHERE = 0x0
    LSF_OVERRIDEEXISTING = 0x1
    LSF_MAKEUNIQUENAME = 0x2

class SHELL_LINK_DATA_FLAGS(IntEnum):
    SLDF_DEFAULT = 0x0
    SLDF_HAS_ID_LIST = 0x1
    SLDF_HAS_LINK_INFO = 0x2
    SLDF_HAS_NAME = 0x4
    SLDF_HAS_RELPATH = 0x8
    SLDF_HAS_WORKINGDIR = 0x10
    SLDF_HAS_ARGS = 0x20
    SLDF_HAS_ICONLOCATION = 0x40
    SLDF_UNICODE = 0x80
    SLDF_FORCE_NO_LINKINFO = 0x100
    SLDF_HAS_EXP_SZ = 0x200
    SLDF_RUN_IN_SEPARATE = 0x400
    SLDF_HAS_DARWINID = 0x1000
    SLDF_RUNAS_USER = 0x2000
    SLDF_HAS_EXP_ICON_SZ = 0x4000
    SLDF_NO_PIDL_ALIAS = 0x8000
    SLDF_FORCE_UNCNAME = 0x10000
    SLDF_RUN_WITH_SHIMLAYER = 0x20000
    SLDF_FORCE_NO_LINKTRACK = 0x40000
    SLDF_ENABLE_TARGET_METADATA = 0x80000
    SLDF_DISABLE_LINK_PATH_TRACKING = 0x100000
    SLDF_DISABLE_KNOWNFOLDER_RELATIVE_TRACKING = 0x200000
    SLDF_NO_KF_ALIAS = 0x400000
    SLDF_ALLOW_LINK_TO_LINK = 0x800000
    SLDF_UNALIAS_ON_SAVE = 0x1000000
    SLDF_PREFER_ENVIRONMENT_PATH = 0x2000000
    SLDF_KEEP_LOCAL_IDLIST_FOR_UNC_TARGET = 0x4000000
    SLDF_PERSIST_VOLUME_ID_RELATIVE = 0x8000000
    SLDF_VALID = 0xFFFF7FF
    SLDF_RESERVED = 0x80000000

class KNOWN_FOLDER_FLAG(IntEnum):
    KF_FLAG_DEFAULT = 0x0
    KF_FLAG_SIMPLE_IDLIST = 0x100
    KF_FLAG_NOT_PARENT_RELATIVE = 0x200
    KF_FLAG_DEFAULT_PATH = 0x400
    KF_FLAG_INIT = 0x800
    KF_FLAG_NO_ALIAS = 0x1000
    KF_FLAG_DONT_UNEXPAND = 0x2000
    KF_FLAG_DONT_VERIFY = 0x4000
    KF_FLAG_CREATE = 0x8000
    KF_FLAG_NO_PACKAGE_REDIRECTION = 0x10000
    KF_FLAG_FORCE_PACKAGE_REDIRECTION = 0x20000
    KF_FLAG_RETURN_FILTER_REDIRECTION_TARGET = 0x40000
    KF_FLAG_FORCE_APP_DATA_REDIRECTION = 0x80000
    KF_FLAG_ALIAS_ONLY = 0x80000000

class PendingMessage(ctypes.Structure):
    class ePendingMessageState(IntEnum):
        EPendingMessageState_Free = 0x0
        EPendingMessageState_StartProfanityCheck = 0x1
        EPendingMessageState_WaitingForProfanityCheck = 0x2
        EPendingMessageState_ProfanityCheckComplete = 0x3
        EPendingMessageState_ProfanityCheckSkipped = 0x4

    _fields_ = [
        ("_mePendingMessageState", ctypes.c_uint32),
        ("msUncensoredMessageBody", ctypes.c_char * 0x3FF),  # cTkFixedString<1023,char>
        ("msMessageBody", ctypes.c_char * 0x3FF),  # cTkFixedString<1023,char>
        ("maRecipients", ctypes.c_char * 0x18),  # std::vector<cTkUserIdBase<cTkFixedString<64,char> >,TkSTLAllocatorShim<cTkUserIdBase<cTkFixedString<64,char> >,1,-1> >
        ("mSender", ctypes.c_char * 0x40),  # cTkUserIdBase<cTkFixedString<64,char> >
        ("mfDisplayDuration", ctypes.c_float),
        ("mbShowNamePrefix", ctypes.c_ubyte),
        ("mbIncludeLocalPlayer", ctypes.c_ubyte),
        ("meMessageType", ctypes.c_uint8 * 0x1),  # _BYTE[1]
        ("mBackgroundColour", common.Colour),
        ("mMessageId", ctypes.c_int32),
    ]
    _mePendingMessageState: int
    msUncensoredMessageBody: bytes
    msMessageBody: bytes
    maRecipients: bytes
    mSender: bytes
    mfDisplayDuration: float
    mbShowNamePrefix: bool
    mbIncludeLocalPlayer: bool
    meMessageType: list[ctypes.c_uint8]
    mBackgroundColour: common.Colour
    mMessageId: int

    @property
    def mePendingMessageState(self):
        return PendingMessage.ePendingMessageState(self._mePendingMessageState)


class eScaryThingMask(IntEnum):
    EScaryThingType_Player = 0x1
    EScaryThingType_Obstacle = 0x2
    EScaryThingType_Creature = 0x4
    EScaryThingType_Bullet = 0x8
    EScaryThingType_Cock = 0x10
    EScaryThingType_All = 0xFFFF

class FfxResourceStates(IntEnum):
    FFX_RESOURCE_STATE_UNORDERED_ACCESS = 0x1
    FFX_RESOURCE_STATE_COMPUTE_READ = 0x2
    FFX_RESOURCE_STATE_COPY_SRC = 0x4
    FFX_RESOURCE_STATE_GENERIC_READ = 0x6
    FFX_RESOURCE_STATE_COPY_DEST = 0x8

class NVSDK_NGX_Version(IntEnum):
    NVSDK_NGX_Version_API = 0x14

class NVSDK_NGX_Result(IntEnum):
    NVSDK_NGX_Result_Success = 0x1
    NVSDK_NGX_Result_Fail = 0xBAD00000
    NVSDK_NGX_Result_FAIL_FeatureNotSupported = 0xBAD00001
    NVSDK_NGX_Result_FAIL_PlatformError = 0xBAD00002
    NVSDK_NGX_Result_FAIL_FeatureAlreadyExists = 0xBAD00003
    NVSDK_NGX_Result_FAIL_FeatureNotFound = 0xBAD00004
    NVSDK_NGX_Result_FAIL_InvalidParameter = 0xBAD00005
    NVSDK_NGX_Result_FAIL_ScratchBufferTooSmall = 0xBAD00006
    NVSDK_NGX_Result_FAIL_NotInitialized = 0xBAD00007
    NVSDK_NGX_Result_FAIL_UnsupportedInputFormat = 0xBAD00008
    NVSDK_NGX_Result_FAIL_RWFlagMissing = 0xBAD00009
    NVSDK_NGX_Result_FAIL_MissingInput = 0xBAD0000A
    NVSDK_NGX_Result_FAIL_UnableToInitializeFeature = 0xBAD0000B
    NVSDK_NGX_Result_FAIL_OutOfDate = 0xBAD0000C
    NVSDK_NGX_Result_FAIL_OutOfGPUMemory = 0xBAD0000D
    NVSDK_NGX_Result_FAIL_UnsupportedFormat = 0xBAD0000E
    NVSDK_NGX_Result_FAIL_UnableToWriteToAppDataPath = 0xBAD0000F
    NVSDK_NGX_Result_FAIL_UnsupportedParameter = 0xBAD00010
    NVSDK_NGX_Result_FAIL_Denied = 0xBAD00011

class NVSDK_NGX_Feature(IntEnum):
    NVSDK_NGX_Feature_Reserved0 = 0x0
    NVSDK_NGX_Feature_SuperSampling = 0x1
    NVSDK_NGX_Feature_InPainting = 0x2
    NVSDK_NGX_Feature_ImageSuperResolution = 0x3
    NVSDK_NGX_Feature_SlowMotion = 0x4
    NVSDK_NGX_Feature_VideoSuperResolution = 0x5
    NVSDK_NGX_Feature_Reserved1 = 0x6
    NVSDK_NGX_Feature_Reserved2 = 0x7
    NVSDK_NGX_Feature_Reserved3 = 0x8
    NVSDK_NGX_Feature_ImageSignalProcessing = 0x9
    NVSDK_NGX_Feature_DeepResolve = 0xA
    NVSDK_NGX_Feature_Reserved4 = 0xB
    NVSDK_NGX_Feature_Count = 0xC
    NVSDK_NGX_Feature_Reserved_SDK = 0x7FFC
    NVSDK_NGX_Feature_Reserved_Core = 0x7FFD
    NVSDK_NGX_Feature_Reserved_Unknown = 0x7FFE

class NVSDK_NGX_DLSS_Feature_Flags(IntEnum):
    NVSDK_NGX_DLSS_Feature_Flags_None = 0x0
    NVSDK_NGX_DLSS_Feature_Flags_IsHDR = 0x1
    NVSDK_NGX_DLSS_Feature_Flags_MVLowRes = 0x2
    NVSDK_NGX_DLSS_Feature_Flags_MVJittered = 0x4
    NVSDK_NGX_DLSS_Feature_Flags_DepthInverted = 0x8
    NVSDK_NGX_DLSS_Feature_Flags_Reserved_0 = 0x10
    NVSDK_NGX_DLSS_Feature_Flags_DoSharpening = 0x20
    NVSDK_NGX_DLSS_Feature_Flags_AutoExposure = 0x40
    NVSDK_NGX_DLSS_Feature_Flags_IsInvalid = 0x80000000

class NVSDK_NGX_GBufferType(IntEnum):
    NVSDK_NGX_GBUFFER_ALBEDO = 0x0
    NVSDK_NGX_GBUFFER_ROUGHNESS = 0x1
    NVSDK_NGX_GBUFFER_METALLIC = 0x2
    NVSDK_NGX_GBUFFER_SPECULAR = 0x3
    NVSDK_NGX_GBUFFER_SUBSURFACE = 0x4
    NVSDK_NGX_GBUFFER_NORMALS = 0x5
    NVSDK_NGX_GBUFFER_SHADINGMODELID = 0x6
    NVSDK_NGX_GBUFFER_MATERIALID = 0x7
    NVSDK_NGX_GBUFFERTYPE_NUM = 0x10

class NVSDK_NGX_Resource_VK_Type(IntEnum):
    NVSDK_NGX_RESOURCE_VK_TYPE_VK_IMAGEVIEW = 0x0
    NVSDK_NGX_RESOURCE_VK_TYPE_VK_BUFFER = 0x1

class eFfxFsrShaderConst(IntEnum):
    EFfxFsr2_ShaderConst_fRenderOffsetUV = 0x0
    EFfxFsr2_ShaderConst_iRenderOffset = 0x1
    EFfxFsr2_ShaderConst_iRenderSize = 0x2
    EFfxFsr2_ShaderConst_fDisplayOffsetUV = 0x3
    EFfxFsr2_ShaderConst_iDisplayOffset = 0x4
    EFfxFsr2_ShaderConst_iDisplaySize = 0x5
    EFfxFsr2_ShaderConst_uLumaMipDimensions = 0x6
    EFfxFsr2_ShaderConst_uLumaMipLevelToUse = 0x7
    EFfxFsr2_ShaderConst_uFrameIndex = 0x8
    EFfxFsr2_ShaderConst_fDisplaySizeRcp = 0x9
    EFfxFsr2_ShaderConst_fJitter = 0xA
    EFfxFsr2_ShaderConst_fDeviceToViewDepth = 0xB
    EFfxFsr2_ShaderConst_depthclip_uv_scale = 0xC
    EFfxFsr2_ShaderConst_postprocessed_lockstatus_uv_scale = 0xD
    EFfxFsr2_ShaderConst_reactive_mask_dim_rcp = 0xE
    EFfxFsr2_ShaderConst_MotionVectorScale = 0xF
    EFfxFsr2_ShaderConst_fDownscaleFactor = 0x10
    EFfxFsr2_ShaderConst_fPreExposure = 0x11
    EFfxFsr2_ShaderConst_fTanHalfFOV = 0x12
    EFfxFsr2_ShaderConst_fMotionVectorJitterCancellation = 0x13
    EFfxFsr2_ShaderConst_fJitterSequenceLength = 0x14
    EFfxFsr2_ShaderConst_fLockInitialLifetime = 0x15
    EFfxFsr2_ShaderConst_fLockTickDelta = 0x16
    EFfxFsr2_ShaderConst_fDeltaTime = 0x17
    EFfxFsr2_ShaderConst_fDynamicResChangeFactor = 0x18
    EFfxFsr2_ShaderConst_fLumaMipRcp = 0x19
    EFfxFsr2_ShaderConst_mips = 0x1A
    EFfxFsr2_ShaderConst_numWorkGroups = 0x1B
    EFfxFsr2_ShaderConst_workGroupOffset = 0x1C
    EFfxFsr2_ShaderConst_renderSize = 0x1D
    EFfxFsr2_ShaderConst_rcasConfig = 0x1E
    EFfxFsr2_ShaderConst_Num = 0x1F

class VkSystemAllocationScope(IntEnum):
    VK_SYSTEM_ALLOCATION_SCOPE_COMMAND = 0x0
    VK_SYSTEM_ALLOCATION_SCOPE_OBJECT = 0x1
    VK_SYSTEM_ALLOCATION_SCOPE_CACHE = 0x2
    VK_SYSTEM_ALLOCATION_SCOPE_DEVICE = 0x3
    VK_SYSTEM_ALLOCATION_SCOPE_INSTANCE = 0x4
    VK_SYSTEM_ALLOCATION_SCOPE_MAX_ENUM = 0x7FFFFFFF

class VkInternalAllocationType(IntEnum):
    VK_INTERNAL_ALLOCATION_TYPE_EXECUTABLE = 0x0
    VK_INTERNAL_ALLOCATION_TYPE_MAX_ENUM = 0x7FFFFFFF

class _NvAPI_Status(IntEnum):
    NVAPI_OK = 0x0
    NVAPI_INVALID_DSC_OUTPUT_BPP = 0xFFFFFF1D
    NVAPI_INVALID_DSC_SLICECOUNT = 0xFFFFFF1E
    NVAPI_INVALID_DSC_VERSION = 0xFFFFFF1F
    NVAPI_DISPLAY_MUX_TRANSITION_FAILED = 0xFFFFFF20
    NVAPI_REQUIRE_FURTHER_HDCP_ACTION = 0xFFFFFF21
    NVAPI_WAIT_FOR_HW_RESOURCE = 0xFFFFFF22
    NVAPI_ERROR_DRIVER_RELOAD_IN_PROGRESS = 0xFFFFFF23
    NVAPI_GPU_NOT_POWERED = 0xFFFFFF24
    NVAPI_STEREO_VERSION_MISMATCH = 0xFFFFFF25
    NVAPI_D3D_CONTEXT_NOT_FOUND = 0xFFFFFF26
    NVAPI_GPU_IN_DEBUG_MODE = 0xFFFFFF27
    NVAPI_INVALID_DIRECT_MODE_DISPLAY = 0xFFFFFF28
    NVAPI_MAX_DISPLAY_LIMIT_REACHED = 0xFFFFFF29
    NVAPI_UNSUPPORTED_CONFIG_NON_HDCP_HMD = 0xFFFFFF2A
    NVAPI_DISPLAY_IN_USE = 0xFFFFFF2B
    NVAPI_INCORRECT_VENDOR = 0xFFFFFF2C
    NVAPI_PRIV_SEC_VIOLATION = 0xFFFFFF2D
    NVAPI_NVIDIA_DISPLAY_NOT_FOUND = 0xFFFFFF2E
    NVAPI_API_IN_USE = 0xFFFFFF2F
    NVAPI_HDCP_DISABLED = 0xFFFFFF30
    NVAPI_NO_CONNECTOR_FOUND = 0xFFFFFF31
    NVAPI_PCLK_LIMITATION_FAILED = 0xFFFFFF32
    NVAPI_HDCP_ENCRYPTION_FAILED = 0xFFFFFF33
    NVAPI_TIMING_NOT_SUPPORTED = 0xFFFFFF34
    NVAPI_RESOURCE_NOT_ACQUIRED = 0xFFFFFF35
    NVAPI_D3D_DEVICE_NOT_REGISTERED = 0xFFFFFF36
    NVAPI_LICENSE_CALLER_AUTHENTICATION_FAILED = 0xFFFFFF37
    NVAPI_FIRMWARE_REVISION_NOT_SUPPORTED = 0xFFFFFF38
    NVAPI_FIRMWARE_OUT_OF_DATE = 0xFFFFFF39
    NVAPI_ECID_KEY_VERIFICATION_FAILED = 0xFFFFFF3A
    NVAPI_ECID_SIGN_ALGO_UNSUPPORTED = 0xFFFFFF3B
    NVAPI_INVALID_SYNC_TOPOLOGY = 0xFFFFFF3C
    NVAPI_SYNC_MASTER_NOT_FOUND = 0xFFFFFF3D
    NVAPI_SYNC_NOT_ACTIVE = 0xFFFFFF3E
    NVAPI_STEREO_INIT_ACTIVATION_NOT_DONE = 0xFFFFFF3F
    NVAPI_GPU_WORKSTATION_FEATURE_INCOMPLETE = 0xFFFFFF40
    NVAPI_TIMEOUT = 0xFFFFFF41
    NVAPI_VALUE_ALREADY_SET = 0xFFFFFF42
    NVAPI_INCOMPATIBLE_AUDIO_DRIVER = 0xFFFFFF43
    NVAPI_STREAM_IS_OUT_OF_SYNC = 0xFFFFFF44
    NVAPI_INVALID_DISPLAY_ID = 0xFFFFFF45
    NVAPI_DPMST_DISPLAY_ID_EXPECTED = 0xFFFFFF46
    NVAPI_CLUSTER_ALREADY_EXISTS = 0xFFFFFF47
    NVAPI_DEFAULT_STEREO_PROFILE_DOES_NOT_EXIST = 0xFFFFFF48
    NVAPI_DEFAULT_STEREO_PROFILE_IS_NOT_DEFINED = 0xFFFFFF49
    NVAPI_EXECUTABLE_PATH_IS_AMBIGUOUS = 0xFFFFFF4A
    NVAPI_STEREO_HANDSHAKE_NOT_DONE = 0xFFFFFF4B
    NVAPI_INVALID_CONFIGURATION = 0xFFFFFF4C
    NVAPI_D3D_DEVICE_LOST = 0xFFFFFF4D
    NVAPI_REQUEST_USER_TO_DISABLE_DWM = 0xFFFFFF4E
    NVAPI_SHARE_RESOURCE_RELOCATED = 0xFFFFFF4F
    NVAPI_MOSAIC_NOT_ACTIVE = 0xFFFFFF50
    NVAPI_ACCESS_DENIED = 0xFFFFFF51
    NVAPI_INSUFFICIENT_BUFFER = 0xFFFFFF52
    NVAPI_DPMST_CHANGED = 0xFFFFFF53
    NVAPI_DISPLAYCONFIG_VALIDATION_FAILED = 0xFFFFFF54
    NVAPI_ID_OUT_OF_RANGE = 0xFFFFFF55
    NVAPI_UNREGISTERED_RESOURCE = 0xFFFFFF56
    NVAPI_PROFILE_REMOVED = 0xFFFFFF57
    NVAPI_DATATYPE_MISMATCH = 0xFFFFFF58
    NVAPI_EXECUTABLE_ALREADY_IN_USE = 0xFFFFFF59
    NVAPI_EXECUTABLE_NOT_FOUND = 0xFFFFFF5A
    NVAPI_PROFILE_NAME_EMPTY = 0xFFFFFF5B
    NVAPI_PROFILE_NAME_IN_USE = 0xFFFFFF5C
    NVAPI_PROFILE_NOT_FOUND = 0xFFFFFF5D
    NVAPI_TOO_MANY_SETTINGS_IN_PROFILE = 0xFFFFFF5E
    NVAPI_SETTING_SIZE_TOO_LARGE = 0xFFFFFF5F
    NVAPI_SETTING_NOT_FOUND = 0xFFFFFF60
    NVAPI_ADVANCED_DISPLAY_TOPOLOGY_REQUIRED = 0xFFFFFF61
    NVAPI_SET_NOT_ALLOWED = 0xFFFFFF62
    NVAPI_ERROR_DRIVER_RELOAD_REQUIRED = 0xFFFFFF63
    NVAPI_DRIVER_NOTRUNNING = 0xFFFFFF64
    NVAPI_DRIVER_RUNNING = 0xFFFFFF65
    NVAPI_MATCHING_DEVICE_NOT_FOUND = 0xFFFFFF66
    NVAPI_STRING_TOO_SMALL = 0xFFFFFF67
    NVAPI_INVALID_ADDRESS = 0xFFFFFF68
    NVAPI_D3D11_LIBRARY_NOT_FOUND = 0xFFFFFF69
    NVAPI_MODE_CHANGE_FAILED = 0xFFFFFF6A
    NVAPI_TOPO_NOT_POSSIBLE = 0xFFFFFF6B
    NVAPI_STEREO_FRUSTUM_ADJUST_MODE_NOT_SUPPORTED = 0xFFFFFF6C
    NVAPI_STEREO_PARAMETER_OUT_OF_RANGE = 0xFFFFFF6D
    NVAPI_STEREO_INVALID_DEVICE_INTERFACE = 0xFFFFFF6E
    NVAPI_STEREO_NOT_TURNED_ON = 0xFFFFFF6F
    NVAPI_STEREO_NOT_ENABLED = 0xFFFFFF70
    NVAPI_STEREO_REGISTRY_VALUE_NOT_SUPPORTED = 0xFFFFFF71
    NVAPI_STEREO_REGISTRY_PROFILE_TYPE_NOT_SUPPORTED = 0xFFFFFF72
    NVAPI_STEREO_REGISTRY_ACCESS_FAILED = 0xFFFFFF73
    NVAPI_STEREO_NOT_INITIALIZED = 0xFFFFFF74
    NVAPI_EXPECTED_COMPUTE_GPU_HANDLE = 0xFFFFFF75
    NVAPI_EXPECTED_NON_PRIMARY_DISPLAY_HANDLE = 0xFFFFFF76
    NVAPI_INVALID_USER_PRIVILEGE = 0xFFFFFF77
    NVAPI_FUNCTION_NOT_FOUND = 0xFFFFFF78
    NVAPI_D3D10_1_LIBRARY_NOT_FOUND = 0xFFFFFF79
    NVAPI_INVALID_CALL = 0xFFFFFF7A
    NVAPI_TOO_MANY_UNIQUE_STATE_OBJECTS = 0xFFFFFF7B
    NVAPI_FILE_NOT_FOUND = 0xFFFFFF7C
    NVAPI_WAS_STILL_DRAWING = 0xFFFFFF7D
    NVAPI_OUT_OF_MEMORY = 0xFFFFFF7E
    NVAPI_REQUEST_USER_TO_CLOSE_NON_MIGRATABLE_APPS = 0xFFFFFF7F
    NVAPI_IMPLICIT_SET_GPU_TOPOLOGY_CHANGE_NOT_ALLOWED = 0xFFFFFF80
    NVAPI_SYSWOW64_NOT_SUPPORTED = 0xFFFFFF81
    NVAPI_MIXED_TARGET_TYPES = 0xFFFFFF82
    NVAPI_INVALID_HYBRID_MODE = 0xFFFFFF83
    NVAPI_REQUIRES_REBOOT = 0xFFFFFF84
    NVAPI_NO_VIDLINK = 0xFFFFFF85
    NVAPI_EXPECTED_ANALOG_DISPLAY = 0xFFFFFF86
    NVAPI_DATA_NOT_FOUND = 0xFFFFFF87
    NVAPI_TIMEOUT_RECONFIGURING_GPU_TOPO = 0xFFFFFF88
    NVAPI_UNKNOWN_UNDERSCAN_CONFIG = 0xFFFFFF89
    NVAPI_TESTING_CLOCKS_NOT_SUPPORTED = 0xFFFFFF8A
    NVAPI_DEVICE_SWITCHING_NOT_ALLOWED = 0xFFFFFF8B
    NVAPI_ARGUMENT_EXCEED_MAX_SIZE = 0xFFFFFF8C
    NVAPI_EXPECTED_DIGITAL_FLAT_PANEL = 0xFFFFFF8D
    NVAPI_SLI_RENDERING_MODE_NOTALLOWED = 0xFFFFFF8E
    NVAPI_NO_ACTIVE_SLI_TOPOLOGY = 0xFFFFFF8F
    NVAPI_EXPECTED_TV_DISPLAY_ON_DCONNECTOR = 0xFFFFFF90
    NVAPI_EXPECTED_TV_DISPLAY = 0xFFFFFF91
    NVAPI_PERSIST_DATA_NOT_FOUND = 0xFFFFFF92
    NVAPI_NV_PERSIST_FILE_NOT_FOUND = 0xFFFFFF93
    NVAPI_DEVICE_BUSY = 0xFFFFFF94
    NVAPI_INVALID_PERF_LEVEL = 0xFFFFFF95
    NVAPI_EXPECTED_UNATTACHED_DISPLAY_HANDLE = 0xFFFFFF96
    NVAPI_PORTID_NOT_FOUND = 0xFFFFFF97
    NVAPI_NOT_SUPPORTED = 0xFFFFFF98
    NVAPI_INVALID_COMBINATION = 0xFFFFFF99
    NVAPI_EXPECTED_DISPLAY_HANDLE = 0xFFFFFF9A
    NVAPI_EXPECTED_PHYSICAL_GPU_HANDLE = 0xFFFFFF9B
    NVAPI_EXPECTED_LOGICAL_GPU_HANDLE = 0xFFFFFF9C
    NVAPI_NO_GL_NSIGHT = 0xFFFFFFF1
    NVAPI_INVALID_POINTER = 0xFFFFFFF2
    NVAPI_INSTRUMENTATION_DISABLED = 0xFFFFFFF3
    NVAPI_NO_GL_EXPERT = 0xFFFFFFF4
    NVAPI_OPENGL_CONTEXT_NOT_CURRENT = 0xFFFFFFF5
    NVAPI_HANDLE_INVALIDATED = 0xFFFFFFF6
    NVAPI_INCOMPATIBLE_STRUCT_VERSION = 0xFFFFFFF7
    NVAPI_INVALID_HANDLE = 0xFFFFFFF8
    NVAPI_END_ENUMERATION = 0xFFFFFFF9
    NVAPI_NVIDIA_DEVICE_NOT_FOUND = 0xFFFFFFFA
    NVAPI_INVALID_ARGUMENT = 0xFFFFFFFB
    NVAPI_API_NOT_INITIALIZED = 0xFFFFFFFC
    NVAPI_NO_IMPLEMENTATION = 0xFFFFFFFD
    NVAPI_LIBRARY_NOT_FOUND = 0xFFFFFFFE
    NVAPI_ERROR = 0xFFFFFFFF

class VMA_CACHE_OPERATION(IntEnum):
    VMA_CACHE_FLUSH = 0x0
    VMA_CACHE_INVALIDATE = 0x1

class eTkBufferFlags(IntEnum):
    EBufferFlags_None = 0x0
    EBufferFlags_Mappable = 0x1
    EBufferFlags_Persistent = 0x2
    EBufferFlags_Indirect = 0x4
    EBufferFlags_TransferDst = 0x8

class _NV_DISPLAY_TV_FORMAT(IntEnum):
    NV_DISPLAY_TV_FORMAT_NONE = 0x0
    NV_DISPLAY_TV_FORMAT_SD_NTSCM = 0x1
    NV_DISPLAY_TV_FORMAT_SD_NTSCJ = 0x2
    NV_DISPLAY_TV_FORMAT_SD_PALM = 0x4
    NV_DISPLAY_TV_FORMAT_SD_PALBDGH = 0x8
    NV_DISPLAY_TV_FORMAT_SD_PALN = 0x10
    NV_DISPLAY_TV_FORMAT_SD_PALNC = 0x20
    NV_DISPLAY_TV_FORMAT_SD_576i = 0x100
    NV_DISPLAY_TV_FORMAT_SD_480i = 0x200
    NV_DISPLAY_TV_FORMAT_ED_480p = 0x400
    NV_DISPLAY_TV_FORMAT_ED_576p = 0x800
    NV_DISPLAY_TV_FORMAT_HD_720p = 0x1000
    NV_DISPLAY_TV_FORMAT_HD_1080i = 0x2000
    NV_DISPLAY_TV_FORMAT_HD_1080p = 0x4000
    NV_DISPLAY_TV_FORMAT_HD_720p50 = 0x8000
    NV_DISPLAY_TV_FORMAT_HD_1080p24 = 0x10000
    NV_DISPLAY_TV_FORMAT_HD_1080i50 = 0x20000
    NV_DISPLAY_TV_FORMAT_HD_1080p50 = 0x40000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp30 = 0x80000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp25 = 0x100000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp24 = 0x200000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp24_SMPTE = 0x400000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp50_3840 = 0x800000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp60_3840 = 0x900000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp30_4096 = 0xA00000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp25_4096 = 0xB00000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp24_4096 = 0xC00000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp50_4096 = 0xD00000
    NV_DISPLAY_TV_FORMAT_UHD_4Kp60_4096 = 0xE00000
    NV_DISPLAY_TV_FORMAT_SD_OTHER = 0x1000000
    NV_DISPLAY_TV_FORMAT_ED_OTHER = 0x2000000
    NV_DISPLAY_TV_FORMAT_HD_OTHER = 0x4000000
    NV_DISPLAY_TV_FORMAT_ANY = 0x80000000

class _NV_SCALING(IntEnum):
    NV_SCALING_DEFAULT = 0x0
    NV_SCALING_GPU_SCALING_TO_CLOSEST = 0x1
    NV_SCALING_GPU_SCALING_TO_NATIVE = 0x2
    NV_SCALING_GPU_SCANOUT_TO_NATIVE = 0x3
    NV_SCALING_GPU_SCALING_TO_ASPECT_SCANOUT_TO_NATIVE = 0x5
    NV_SCALING_GPU_SCALING_TO_ASPECT_SCANOUT_TO_CLOSEST = 0x6
    NV_SCALING_GPU_SCANOUT_TO_CLOSEST = 0x7
    NV_SCALING_GPU_INTEGER_ASPECT_SCALING = 0x8
    NV_SCALING_CUSTOMIZED = 0xFF

class VkPresentModeKHR(IntEnum):
    VK_PRESENT_MODE_IMMEDIATE_KHR = 0x0
    VK_PRESENT_MODE_MAILBOX_KHR = 0x1
    VK_PRESENT_MODE_FIFO_KHR = 0x2
    VK_PRESENT_MODE_FIFO_RELAXED_KHR = 0x3
    VK_PRESENT_MODE_SHARED_DEMAND_REFRESH_KHR = 0x3B9C7B98
    VK_PRESENT_MODE_SHARED_CONTINUOUS_REFRESH_KHR = 0x3B9C7B99
    VK_PRESENT_MODE_MAX_ENUM_KHR = 0x7FFFFFFF

class VkColorSpaceKHR(IntEnum):
    VK_COLOR_SPACE_SRGB_NONLINEAR_KHR = 0x0
    VK_COLOR_SPACE_DISPLAY_P3_NONLINEAR_EXT = 0x3B9C6041
    VK_COLOR_SPACE_EXTENDED_SRGB_LINEAR_EXT = 0x3B9C6042
    VK_COLOR_SPACE_DISPLAY_P3_LINEAR_EXT = 0x3B9C6043
    VK_COLOR_SPACE_DCI_P3_NONLINEAR_EXT = 0x3B9C6044
    VK_COLOR_SPACE_BT709_LINEAR_EXT = 0x3B9C6045
    VK_COLOR_SPACE_BT709_NONLINEAR_EXT = 0x3B9C6046
    VK_COLOR_SPACE_BT2020_LINEAR_EXT = 0x3B9C6047
    VK_COLOR_SPACE_HDR10_ST2084_EXT = 0x3B9C6048
    VK_COLOR_SPACE_DOLBYVISION_EXT = 0x3B9C6049
    VK_COLOR_SPACE_HDR10_HLG_EXT = 0x3B9C604A
    VK_COLOR_SPACE_ADOBERGB_LINEAR_EXT = 0x3B9C604B
    VK_COLOR_SPACE_ADOBERGB_NONLINEAR_EXT = 0x3B9C604C
    VK_COLOR_SPACE_PASS_THROUGH_EXT = 0x3B9C604D
    VK_COLOR_SPACE_EXTENDED_SRGB_NONLINEAR_EXT = 0x3B9C604E
    VK_COLOR_SPACE_DISPLAY_NATIVE_AMD = 0x3B9E0A08
    VK_COLOR_SPACE_MAX_ENUM_KHR = 0x7FFFFFFF

class _NV_TIMING_OVERRIDE(IntEnum):
    NV_TIMING_OVERRIDE_CURRENT = 0x0
    NV_TIMING_OVERRIDE_AUTO = 0x1
    NV_TIMING_OVERRIDE_EDID = 0x2
    NV_TIMING_OVERRIDE_DMT = 0x3
    NV_TIMING_OVERRIDE_DMT_RB = 0x4
    NV_TIMING_OVERRIDE_CVT = 0x5
    NV_TIMING_OVERRIDE_CVT_RB = 0x6
    NV_TIMING_OVERRIDE_GTF = 0x7
    NV_TIMING_OVERRIDE_EIA861 = 0x8
    NV_TIMING_OVERRIDE_ANALOG_TV = 0x9
    NV_TIMING_OVERRIDE_CUST = 0xA
    NV_TIMING_OVERRIDE_NV_PREDEFINED = 0xB
    NV_TIMING_OVERRIDE_NV_ASPR = 0xC
    NV_TIMING_OVERRIDE_SDI = 0xD
    NV_TIMING_OVRRIDE_MAX = 0xE

class VkCompositeAlphaFlagBitsKHR(IntEnum):
    VK_COMPOSITE_ALPHA_OPAQUE_BIT_KHR = 0x1
    VK_COMPOSITE_ALPHA_PRE_MULTIPLIED_BIT_KHR = 0x2
    VK_COMPOSITE_ALPHA_POST_MULTIPLIED_BIT_KHR = 0x4
    VK_COMPOSITE_ALPHA_INHERIT_BIT_KHR = 0x8
    VK_COMPOSITE_ALPHA_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkSubpassContents(IntEnum):
    VK_SUBPASS_CONTENTS_INLINE = 0x0
    VK_SUBPASS_CONTENTS_SECONDARY_COMMAND_BUFFERS = 0x1
    VK_SUBPASS_CONTENTS_MAX_ENUM = 0x7FFFFFFF

class VkComponentSwizzle(IntEnum):
    VK_COMPONENT_SWIZZLE_IDENTITY = 0x0
    VK_COMPONENT_SWIZZLE_ZERO = 0x1
    VK_COMPONENT_SWIZZLE_ONE = 0x2
    VK_COMPONENT_SWIZZLE_R = 0x3
    VK_COMPONENT_SWIZZLE_G = 0x4
    VK_COMPONENT_SWIZZLE_B = 0x5
    VK_COMPONENT_SWIZZLE_A = 0x6
    VK_COMPONENT_SWIZZLE_MAX_ENUM = 0x7FFFFFFF

class VkImageViewType(IntEnum):
    VK_IMAGE_VIEW_TYPE_1D = 0x0
    VK_IMAGE_VIEW_TYPE_2D = 0x1
    VK_IMAGE_VIEW_TYPE_3D = 0x2
    VK_IMAGE_VIEW_TYPE_CUBE = 0x3
    VK_IMAGE_VIEW_TYPE_1D_ARRAY = 0x4
    VK_IMAGE_VIEW_TYPE_2D_ARRAY = 0x5
    VK_IMAGE_VIEW_TYPE_CUBE_ARRAY = 0x6
    VK_IMAGE_VIEW_TYPE_MAX_ENUM = 0x7FFFFFFF

class VkAttachmentStoreOp(IntEnum):
    VK_ATTACHMENT_STORE_OP_STORE = 0x0
    VK_ATTACHMENT_STORE_OP_DONT_CARE = 0x1
    VK_ATTACHMENT_STORE_OP_NONE = 0x3B9F61C8
    VK_ATTACHMENT_STORE_OP_MAX_ENUM = 0x7FFFFFFF

class VkCommandBufferLevel(IntEnum):
    VK_COMMAND_BUFFER_LEVEL_PRIMARY = 0x0
    VK_COMMAND_BUFFER_LEVEL_SECONDARY = 0x1
    VK_COMMAND_BUFFER_LEVEL_MAX_ENUM = 0x7FFFFFFF

class VkMemoryHeapFlagBits(IntEnum):
    VK_MEMORY_HEAP_DEVICE_LOCAL_BIT = 0x1
    VK_MEMORY_HEAP_MULTI_INSTANCE_BIT = 0x2
    VK_MEMORY_HEAP_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkCommandPoolCreateFlagBits(IntEnum):
    VK_COMMAND_POOL_CREATE_TRANSIENT_BIT = 0x1
    VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT = 0x2
    VK_COMMAND_POOL_CREATE_PROTECTED_BIT = 0x4
    VK_COMMAND_POOL_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkCommandBufferUsageFlagBits(IntEnum):
    VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT = 0x1
    VK_COMMAND_BUFFER_USAGE_RENDER_PASS_CONTINUE_BIT = 0x2
    VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT = 0x4
    VK_COMMAND_BUFFER_USAGE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class _NV_COLOR_SELECTION_POLICY(IntEnum):
    NV_COLOR_SELECTION_POLICY_USER = 0x0
    NV_COLOR_SELECTION_POLICY_BEST_QUALITY = 0x1
    NV_COLOR_SELECTION_POLICY_UNKNOWN = 0xFF

class _NV_DESKTOP_COLOR_DEPTH(IntEnum):
    NV_DESKTOP_COLOR_DEPTH_DEFAULT = 0x0
    NV_DESKTOP_COLOR_DEPTH_8BPC = 0x1
    NV_DESKTOP_COLOR_DEPTH_10BPC = 0x2
    NV_DESKTOP_COLOR_DEPTH_16BPC_FLOAT = 0x3
    NV_DESKTOP_COLOR_DEPTH_16BPC_FLOAT_WCG = 0x4
    NV_DESKTOP_COLOR_DEPTH_16BPC_FLOAT_HDR = 0x5

class NV_MOSAIC_TOPO(IntEnum):
    NV_MOSAIC_TOPO_NONE = 0x0
    NV_MOSAIC_TOPO_BEGIN_BASIC = 0x1
    NV_MOSAIC_TOPO_2x1_BASIC = 0x2
    NV_MOSAIC_TOPO_1x3_BASIC = 0x3
    NV_MOSAIC_TOPO_3x1_BASIC = 0x4
    NV_MOSAIC_TOPO_1x4_BASIC = 0x5
    NV_MOSAIC_TOPO_4x1_BASIC = 0x6
    NV_MOSAIC_TOPO_2x2_BASIC = 0x7
    NV_MOSAIC_TOPO_2x3_BASIC = 0x8
    NV_MOSAIC_TOPO_2x4_BASIC = 0x9
    NV_MOSAIC_TOPO_3x2_BASIC = 0xA
    NV_MOSAIC_TOPO_4x2_BASIC = 0xB
    NV_MOSAIC_TOPO_1x5_BASIC = 0xC
    NV_MOSAIC_TOPO_1x6_BASIC = 0xD
    NV_MOSAIC_TOPO_7x1_BASIC = 0xE
    NV_MOSAIC_TOPO_END_BASIC = 0x17
    NV_MOSAIC_TOPO_BEGIN_PASSIVE_STEREO = 0x18
    NV_MOSAIC_TOPO_2x1_PASSIVE_STEREO = 0x19
    NV_MOSAIC_TOPO_1x3_PASSIVE_STEREO = 0x1A
    NV_MOSAIC_TOPO_3x1_PASSIVE_STEREO = 0x1B
    NV_MOSAIC_TOPO_1x4_PASSIVE_STEREO = 0x1C
    NV_MOSAIC_TOPO_4x1_PASSIVE_STEREO = 0x1D
    NV_MOSAIC_TOPO_2x2_PASSIVE_STEREO = 0x1E
    NV_MOSAIC_TOPO_END_PASSIVE_STEREO = 0x22
    NV_MOSAIC_TOPO_MAX = 0x23

class VkSurfaceTransformFlagBitsKHR(IntEnum):
    VK_SURFACE_TRANSFORM_IDENTITY_BIT_KHR = 0x1
    VK_SURFACE_TRANSFORM_ROTATE_90_BIT_KHR = 0x2
    VK_SURFACE_TRANSFORM_ROTATE_180_BIT_KHR = 0x4
    VK_SURFACE_TRANSFORM_ROTATE_270_BIT_KHR = 0x8
    VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_BIT_KHR = 0x10
    VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_90_BIT_KHR = 0x20
    VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_180_BIT_KHR = 0x40
    VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_270_BIT_KHR = 0x80
    VK_SURFACE_TRANSFORM_INHERIT_BIT_KHR = 0x100
    VK_SURFACE_TRANSFORM_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class eColourVariantType(IntEnum):
    EColourVariant_ExactMatch = 0x0
    EColourVariant_RandomOffset = 0x1
    EColourVariant_Complement = 0x2
    EColourVariant_Triadic1 = 0x3
    EColourVariant_Triadic2 = 0x4

class VkDebugUtilsMessageSeverityFlagBitsEXT(IntEnum):
    VK_DEBUG_UTILS_MESSAGE_SEVERITY_VERBOSE_BIT_EXT = 0x1
    VK_DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT = 0x10
    VK_DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT = 0x100
    VK_DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT = 0x1000
    VK_DEBUG_UTILS_MESSAGE_SEVERITY_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkDebugUtilsMessageTypeFlagBitsEXT(IntEnum):
    VK_DEBUG_UTILS_MESSAGE_TYPE_GENERAL_BIT_EXT = 0x1
    VK_DEBUG_UTILS_MESSAGE_TYPE_VALIDATION_BIT_EXT = 0x2
    VK_DEBUG_UTILS_MESSAGE_TYPE_PERFORMANCE_BIT_EXT = 0x4
    VK_DEBUG_UTILS_MESSAGE_TYPE_DEVICE_ADDRESS_BINDING_BIT_EXT = 0x8
    VK_DEBUG_UTILS_MESSAGE_TYPE_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class NVSDK_NGX_Logging_Level(IntEnum):
    NVSDK_NGX_LOGGING_LEVEL_OFF = 0x0
    NVSDK_NGX_LOGGING_LEVEL_ON = 0x1
    NVSDK_NGX_LOGGING_LEVEL_VERBOSE = 0x2
    NVSDK_NGX_LOGGING_LEVEL_NUM = 0x3

class NVSDK_NGX_ToneMapperType(IntEnum):
    NVSDK_NGX_TONEMAPPER_STRING = 0x0
    NVSDK_NGX_TONEMAPPER_REINHARD = 0x1
    NVSDK_NGX_TONEMAPPER_ONEOVERLUMA = 0x2
    NVSDK_NGX_TONEMAPPER_ACES = 0x3
    NVSDK_NGX_TONEMAPPERTYPE_NUM = 0x4

class ShadingRate(ctypes.Structure):
    class List(IntEnum):
        NoChange = 0x0
        Reduce_1x2 = 0x1
        Reduce_2x1 = 0x2
        Reduce_2x2 = 0x3
        Reduce_2x4 = 0x4
        Reduce_4x2 = 0x5
        Reduce_4x4 = 0x6


class FillModes(ctypes.Structure):
    class List(IntEnum):
        Solid = 0x0
        Wireframe = 0x1


class eGalacticMapRequest(IntEnum):
    EGalacticMapRequest_BlackHole = 0x0
    EGalacticMapRequest_AtlasStation = 0x1
    EGalacticMapRequest_BlackHoleNoAnim = 0x2
    EGalacticMapRequest_AtlasStationNoAnim = 0x3
    ENumGalacticMapRequests = 0x4

class ePostWarpRequest(IntEnum):
    EPostWarpRequest_TrashInventory = 0x0
    EPostWarpRequest_WarpingFromBlackHole = 0x1
    EPostWarpRequest_WarpingFromRelicGate = 0x2
    EPostWarpRequest_WarpingFromFreighterMegaWarp = 0x3

class eHandGesture(IntEnum):
    EHandGesture_None = 0x0
    EHandGesture_Holster = 0x1
    EHandGesture_Binoculars = 0x2

class eCombineTextureMode(IntEnum):
    ECombine_Recolour = 0x0
    ECombine_RecolourTerrain = 0x1
    ECombine_Maps = 0x2
    ECombine_Masks = 0x3

class VkRayTracingShaderGroupTypeKHR(IntEnum):
    VK_RAY_TRACING_SHADER_GROUP_TYPE_GENERAL_KHR = 0x0
    VK_RAY_TRACING_SHADER_GROUP_TYPE_TRIANGLES_HIT_GROUP_KHR = 0x1
    VK_RAY_TRACING_SHADER_GROUP_TYPE_PROCEDURAL_HIT_GROUP_KHR = 0x2
    VK_RAY_TRACING_SHADER_GROUP_TYPE_MAX_ENUM_KHR = 0x7FFFFFFF

class VkGeometryTypeKHR(IntEnum):
    VK_GEOMETRY_TYPE_TRIANGLES_KHR = 0x0
    VK_GEOMETRY_TYPE_AABBS_KHR = 0x1
    VK_GEOMETRY_TYPE_INSTANCES_KHR = 0x2
    VK_GEOMETRY_TYPE_MAX_ENUM_KHR = 0x7FFFFFFF

class VkAccelerationStructureTypeKHR(IntEnum):
    VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR = 0x0
    VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR = 0x1
    VK_ACCELERATION_STRUCTURE_TYPE_GENERIC_KHR = 0x2
    VK_ACCELERATION_STRUCTURE_TYPE_MAX_ENUM_KHR = 0x7FFFFFFF

class eDebugSections(IntEnum):
    EDebugSection_Ecosystem = 0x0
    EDebugSection_Projectiles = 0x1
    EDebugSection_Particles = 0x2
    EDebugSection_Physics = 0x3
    EDebugSection_Audio = 0x4
    EDebugSection_TriggerBox = 0x5
    EDebugSection_HeightTests = 0x6
    EDebugSection_Spaceships = 0x7
    EDebugSection_ShipTravel = 0x8
    EDebugSection_Simulation = 0x9
    EDebugSection_Environment = 0xA
    EDebugSection_GalaxyMap = 0xB
    EDebugSection_GalaxyMapNetwork = 0xC
    EDebugSection_Robots = 0xD
    EDebugSection_SolarSystem = 0xE
    EDebugSection_PlayerExperience = 0xF
    EDebugSection_Player = 0x10
    EDebugSection_Vehicle = 0x11
    EDebugSection_Cameras = 0x12
    EDebugSection_Networking = 0x13
    EDebugSection_Markers = 0x14
    EDebugSection_ShipLanding = 0x15
    EDebugSection_Maintenance = 0x16
    EDebugSection_Buildings = 0x17
    EDebugSection_Underwater = 0x18
    EDebugSection_Hmd = 0x19
    EDebugSection_HmdShip = 0x1A
    EDebugSection_Grabbables = 0x1B
    EDebugSection_Input = 0x1C
    EDebugSection_Interaction = 0x1D
    EDebugSection_NumSections = 0x1E

class VkCopyAccelerationStructureModeKHR(IntEnum):
    VK_COPY_ACCELERATION_STRUCTURE_MODE_CLONE_KHR = 0x0
    VK_COPY_ACCELERATION_STRUCTURE_MODE_COMPACT_KHR = 0x1
    VK_COPY_ACCELERATION_STRUCTURE_MODE_SERIALIZE_KHR = 0x2
    VK_COPY_ACCELERATION_STRUCTURE_MODE_DESERIALIZE_KHR = 0x3
    VK_COPY_ACCELERATION_STRUCTURE_MODE_MAX_ENUM_KHR = 0x7FFFFFFF

class VkGeometryFlagBitsKHR(IntEnum):
    VK_GEOMETRY_OPAQUE_BIT_KHR = 0x1
    VK_GEOMETRY_NO_DUPLICATE_ANY_HIT_INVOCATION_BIT_KHR = 0x2
    VK_GEOMETRY_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkGeometryInstanceFlagBitsKHR(IntEnum):
    VK_GEOMETRY_INSTANCE_TRIANGLE_FACING_CULL_DISABLE_BIT_KHR = 0x1
    VK_GEOMETRY_INSTANCE_TRIANGLE_FLIP_FACING_BIT_KHR = 0x2
    VK_GEOMETRY_INSTANCE_FORCE_OPAQUE_BIT_KHR = 0x4
    VK_GEOMETRY_INSTANCE_FORCE_NO_OPAQUE_BIT_KHR = 0x8
    VK_GEOMETRY_INSTANCE_FORCE_OPACITY_MICROMAP_2_STATE_EXT = 0x10
    VK_GEOMETRY_INSTANCE_DISABLE_OPACITY_MICROMAPS_EXT = 0x20
    VK_GEOMETRY_INSTANCE_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkBuildAccelerationStructureFlagBitsKHR(IntEnum):
    VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_UPDATE_BIT_KHR = 0x1
    VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_COMPACTION_BIT_KHR = 0x2
    VK_BUILD_ACCELERATION_STRUCTURE_PREFER_FAST_TRACE_BIT_KHR = 0x4
    VK_BUILD_ACCELERATION_STRUCTURE_PREFER_FAST_BUILD_BIT_KHR = 0x8
    VK_BUILD_ACCELERATION_STRUCTURE_LOW_MEMORY_BIT_KHR = 0x10
    VK_BUILD_ACCELERATION_STRUCTURE_MOTION_BIT_NV = 0x20
    VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_OPACITY_MICROMAP_UPDATE_EXT = 0x40
    VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_DISABLE_OPACITY_MICROMAPS_EXT = 0x80
    VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_OPACITY_MICROMAP_DATA_UPDATE_EXT = 0x100
    VK_BUILD_ACCELERATION_STRUCTURE_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkDriverId(IntEnum):
    VK_DRIVER_ID_AMD_PROPRIETARY = 0x1
    VK_DRIVER_ID_AMD_OPEN_SOURCE = 0x2
    VK_DRIVER_ID_MESA_RADV = 0x3
    VK_DRIVER_ID_NVIDIA_PROPRIETARY = 0x4
    VK_DRIVER_ID_INTEL_PROPRIETARY_WINDOWS = 0x5
    VK_DRIVER_ID_INTEL_OPEN_SOURCE_MESA = 0x6
    VK_DRIVER_ID_IMAGINATION_PROPRIETARY = 0x7
    VK_DRIVER_ID_QUALCOMM_PROPRIETARY = 0x8
    VK_DRIVER_ID_ARM_PROPRIETARY = 0x9
    VK_DRIVER_ID_GOOGLE_SWIFTSHADER = 0xA
    VK_DRIVER_ID_GGP_PROPRIETARY = 0xB
    VK_DRIVER_ID_BROADCOM_PROPRIETARY = 0xC
    VK_DRIVER_ID_MESA_LLVMPIPE = 0xD
    VK_DRIVER_ID_MOLTENVK = 0xE
    VK_DRIVER_ID_COREAVI_PROPRIETARY = 0xF
    VK_DRIVER_ID_JUICE_PROPRIETARY = 0x10
    VK_DRIVER_ID_VERISILICON_PROPRIETARY = 0x11
    VK_DRIVER_ID_MESA_TURNIP = 0x12
    VK_DRIVER_ID_MESA_V3DV = 0x13
    VK_DRIVER_ID_MESA_PANVK = 0x14
    VK_DRIVER_ID_SAMSUNG_PROPRIETARY = 0x15
    VK_DRIVER_ID_MESA_VENUS = 0x16
    VK_DRIVER_ID_MESA_DOZEN = 0x17
    VK_DRIVER_ID_MESA_NVK = 0x18
    VK_DRIVER_ID_MAX_ENUM = 0x7FFFFFFF

class VkShaderFloatControlsIndependence(IntEnum):
    VK_SHADER_FLOAT_CONTROLS_INDEPENDENCE_32_BIT_ONLY = 0x0
    VK_SHADER_FLOAT_CONTROLS_INDEPENDENCE_ALL = 0x1
    VK_SHADER_FLOAT_CONTROLS_INDEPENDENCE_NONE = 0x2
    VK_SHADER_FLOAT_CONTROLS_INDEPENDENCE_MAX_ENUM = 0x7FFFFFFF

class VkSamplerReductionMode(IntEnum):
    VK_SAMPLER_REDUCTION_MODE_WEIGHTED_AVERAGE = 0x0
    VK_SAMPLER_REDUCTION_MODE_MIN = 0x1
    VK_SAMPLER_REDUCTION_MODE_MAX = 0x2
    VK_SAMPLER_REDUCTION_MODE_MAX_ENUM = 0x7FFFFFFF

class VkSemaphoreType(IntEnum):
    VK_SEMAPHORE_TYPE_BINARY = 0x0
    VK_SEMAPHORE_TYPE_TIMELINE = 0x1
    VK_SEMAPHORE_TYPE_MAX_ENUM = 0x7FFFFFFF

class VkResolveModeFlagBits(IntEnum):
    VK_RESOLVE_MODE_NONE = 0x0
    VK_RESOLVE_MODE_SAMPLE_ZERO_BIT = 0x1
    VK_RESOLVE_MODE_AVERAGE_BIT = 0x2
    VK_RESOLVE_MODE_MIN_BIT = 0x4
    VK_RESOLVE_MODE_MAX_BIT = 0x8
    VK_RESOLVE_MODE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkDescriptorBindingFlagBits(IntEnum):
    VK_DESCRIPTOR_BINDING_UPDATE_AFTER_BIND_BIT = 0x1
    VK_DESCRIPTOR_BINDING_UPDATE_UNUSED_WHILE_PENDING_BIT = 0x2
    VK_DESCRIPTOR_BINDING_PARTIALLY_BOUND_BIT = 0x4
    VK_DESCRIPTOR_BINDING_VARIABLE_DESCRIPTOR_COUNT_BIT = 0x8
    VK_DESCRIPTOR_BINDING_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkSemaphoreWaitFlagBits(IntEnum):
    VK_SEMAPHORE_WAIT_ANY_BIT = 0x1
    VK_SEMAPHORE_WAIT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkPipelineCreationFeedbackFlagBits(IntEnum):
    VK_PIPELINE_CREATION_FEEDBACK_VALID_BIT = 0x1
    VK_PIPELINE_CREATION_FEEDBACK_APPLICATION_PIPELINE_CACHE_HIT_BIT = 0x2
    VK_PIPELINE_CREATION_FEEDBACK_BASE_PIPELINE_ACCELERATION_BIT = 0x4
    VK_PIPELINE_CREATION_FEEDBACK_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkToolPurposeFlagBits(IntEnum):
    VK_TOOL_PURPOSE_VALIDATION_BIT = 0x1
    VK_TOOL_PURPOSE_PROFILING_BIT = 0x2
    VK_TOOL_PURPOSE_TRACING_BIT = 0x4
    VK_TOOL_PURPOSE_ADDITIONAL_FEATURES_BIT = 0x8
    VK_TOOL_PURPOSE_MODIFYING_FEATURES_BIT = 0x10
    VK_TOOL_PURPOSE_DEBUG_REPORTING_BIT_EXT = 0x20
    VK_TOOL_PURPOSE_DEBUG_MARKERS_BIT_EXT = 0x40
    VK_TOOL_PURPOSE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkSubmitFlagBits(IntEnum):
    VK_SUBMIT_PROTECTED_BIT = 0x1
    VK_SUBMIT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkRenderingFlagBits(IntEnum):
    VK_RENDERING_CONTENTS_SECONDARY_COMMAND_BUFFERS_BIT = 0x1
    VK_RENDERING_SUSPENDING_BIT = 0x2
    VK_RENDERING_RESUMING_BIT = 0x4
    VK_RENDERING_ENABLE_LEGACY_DITHERING_BIT_EXT = 0x8
    VK_RENDERING_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class eUniformBufferMask(IntEnum):
    EUBM_PerFrameUniformBufferMask = 0x0
    EUBM_PerRenderTargetUniformBufferMask = 0x1
    EUBM_PerMaterialUniformBufferMask = 0x2
    EUBM_PerMeshUniformBufferMask = 0x3
    EUBM_Num = 0x4

class VkFormatFeatureFlagBits(IntEnum):
    VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT = 0x1
    VK_FORMAT_FEATURE_STORAGE_IMAGE_BIT = 0x2
    VK_FORMAT_FEATURE_STORAGE_IMAGE_ATOMIC_BIT = 0x4
    VK_FORMAT_FEATURE_UNIFORM_TEXEL_BUFFER_BIT = 0x8
    VK_FORMAT_FEATURE_STORAGE_TEXEL_BUFFER_BIT = 0x10
    VK_FORMAT_FEATURE_STORAGE_TEXEL_BUFFER_ATOMIC_BIT = 0x20
    VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT = 0x40
    VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT = 0x80
    VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BLEND_BIT = 0x100
    VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT = 0x200
    VK_FORMAT_FEATURE_BLIT_SRC_BIT = 0x400
    VK_FORMAT_FEATURE_BLIT_DST_BIT = 0x800
    VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_LINEAR_BIT = 0x1000
    VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_CUBIC_BIT_EXT = 0x2000
    VK_FORMAT_FEATURE_TRANSFER_SRC_BIT = 0x4000
    VK_FORMAT_FEATURE_TRANSFER_DST_BIT = 0x8000
    VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_MINMAX_BIT = 0x10000
    VK_FORMAT_FEATURE_MIDPOINT_CHROMA_SAMPLES_BIT = 0x20000
    VK_FORMAT_FEATURE_SAMPLED_IMAGE_YCBCR_CONVERSION_LINEAR_FILTER_BIT = 0x40000
    VK_FORMAT_FEATURE_SAMPLED_IMAGE_YCBCR_CONVERSION_SEPARATE_RECONSTRUCTION_FILTER_BIT = 0x80000
    VK_FORMAT_FEATURE_SAMPLED_IMAGE_YCBCR_CONVERSION_CHROMA_RECONSTRUCTION_EXPLICIT_BIT = 0x100000
    VK_FORMAT_FEATURE_SAMPLED_IMAGE_YCBCR_CONVERSION_CHROMA_RECONSTRUCTION_EXPLICIT_FORCEABLE_BIT = 0x200000
    VK_FORMAT_FEATURE_DISJOINT_BIT = 0x400000
    VK_FORMAT_FEATURE_COSITED_CHROMA_SAMPLES_BIT = 0x800000
    VK_FORMAT_FEATURE_FRAGMENT_DENSITY_MAP_BIT_EXT = 0x1000000
    VK_FORMAT_FEATURE_VIDEO_DECODE_OUTPUT_BIT_KHR = 0x2000000
    VK_FORMAT_FEATURE_VIDEO_DECODE_DPB_BIT_KHR = 0x4000000
    VK_FORMAT_FEATURE_ACCELERATION_STRUCTURE_VERTEX_BUFFER_BIT_KHR = 0x20000000
    VK_FORMAT_FEATURE_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR = 0x40000000
    VK_FORMAT_FEATURE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkEventCreateFlagBits(IntEnum):
    VK_EVENT_CREATE_DEVICE_ONLY_BIT = 0x1
    VK_EVENT_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkBufferCreateFlagBits(IntEnum):
    VK_BUFFER_CREATE_SPARSE_BINDING_BIT = 0x1
    VK_BUFFER_CREATE_SPARSE_RESIDENCY_BIT = 0x2
    VK_BUFFER_CREATE_SPARSE_ALIASED_BIT = 0x4
    VK_BUFFER_CREATE_PROTECTED_BIT = 0x8
    VK_BUFFER_CREATE_DEVICE_ADDRESS_CAPTURE_REPLAY_BIT = 0x10
    VK_BUFFER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT = 0x20
    VK_BUFFER_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkPipelineCacheCreateFlagBits(IntEnum):
    VK_PIPELINE_CACHE_CREATE_EXTERNALLY_SYNCHRONIZED_BIT = 0x1
    VK_PIPELINE_CACHE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkPipelineCreateFlagBits(IntEnum):
    VK_PIPELINE_CREATE_DISABLE_OPTIMIZATION_BIT = 0x1
    VK_PIPELINE_CREATE_ALLOW_DERIVATIVES_BIT = 0x2
    VK_PIPELINE_CREATE_DERIVATIVE_BIT = 0x4
    VK_PIPELINE_CREATE_VIEW_INDEX_FROM_DEVICE_INDEX_BIT = 0x8
    VK_PIPELINE_CREATE_DISPATCH_BASE_BIT = 0x10
    VK_PIPELINE_CREATE_DEFER_COMPILE_BIT_NV = 0x20
    VK_PIPELINE_CREATE_CAPTURE_STATISTICS_BIT_KHR = 0x40
    VK_PIPELINE_CREATE_CAPTURE_INTERNAL_REPRESENTATIONS_BIT_KHR = 0x80
    VK_PIPELINE_CREATE_FAIL_ON_PIPELINE_COMPILE_REQUIRED_BIT = 0x100
    VK_PIPELINE_CREATE_EARLY_RETURN_ON_FAILURE_BIT = 0x200
    VK_PIPELINE_CREATE_LINK_TIME_OPTIMIZATION_BIT_EXT = 0x400
    VK_PIPELINE_CREATE_LIBRARY_BIT_KHR = 0x800
    VK_PIPELINE_CREATE_RAY_TRACING_SKIP_TRIANGLES_BIT_KHR = 0x1000
    VK_PIPELINE_CREATE_RAY_TRACING_SKIP_AABBS_BIT_KHR = 0x2000
    VK_PIPELINE_CREATE_RAY_TRACING_NO_NULL_ANY_HIT_SHADERS_BIT_KHR = 0x4000
    VK_PIPELINE_CREATE_RAY_TRACING_NO_NULL_CLOSEST_HIT_SHADERS_BIT_KHR = 0x8000
    VK_PIPELINE_CREATE_RAY_TRACING_NO_NULL_MISS_SHADERS_BIT_KHR = 0x10000
    VK_PIPELINE_CREATE_RAY_TRACING_NO_NULL_INTERSECTION_SHADERS_BIT_KHR = 0x20000
    VK_PIPELINE_CREATE_INDIRECT_BINDABLE_BIT_NV = 0x40000
    VK_PIPELINE_CREATE_RAY_TRACING_SHADER_GROUP_HANDLE_CAPTURE_REPLAY_BIT_KHR = 0x80000
    VK_PIPELINE_CREATE_RAY_TRACING_ALLOW_MOTION_BIT_NV = 0x100000
    VK_PIPELINE_CREATE_RENDERING_FRAGMENT_SHADING_RATE_ATTACHMENT_BIT_KHR = 0x200000
    VK_PIPELINE_CREATE_RENDERING_FRAGMENT_DENSITY_MAP_ATTACHMENT_BIT_EXT = 0x400000
    VK_PIPELINE_CREATE_RETAIN_LINK_TIME_OPTIMIZATION_INFO_BIT_EXT = 0x800000
    VK_PIPELINE_CREATE_RAY_TRACING_OPACITY_MICROMAP_BIT_EXT = 0x1000000
    VK_PIPELINE_CREATE_COLOR_ATTACHMENT_FEEDBACK_LOOP_BIT_EXT = 0x2000000
    VK_PIPELINE_CREATE_DEPTH_STENCIL_ATTACHMENT_FEEDBACK_LOOP_BIT_EXT = 0x4000000
    VK_PIPELINE_CREATE_NO_PROTECTED_ACCESS_BIT_EXT = 0x8000000
    VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT = 0x20000000
    VK_PIPELINE_CREATE_PROTECTED_ACCESS_ONLY_BIT_EXT = 0x40000000
    VK_PIPELINE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkPipelineShaderStageCreateFlagBits(IntEnum):
    VK_PIPELINE_SHADER_STAGE_CREATE_ALLOW_VARYING_SUBGROUP_SIZE_BIT = 0x1
    VK_PIPELINE_SHADER_STAGE_CREATE_REQUIRE_FULL_SUBGROUPS_BIT = 0x2
    VK_PIPELINE_SHADER_STAGE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkPipelineDepthStencilStateCreateFlagBits(IntEnum):
    VK_PIPELINE_DEPTH_STENCIL_STATE_CREATE_RASTERIZATION_ORDER_ATTACHMENT_DEPTH_ACCESS_BIT_EXT = 0x1
    VK_PIPELINE_DEPTH_STENCIL_STATE_CREATE_RASTERIZATION_ORDER_ATTACHMENT_STENCIL_ACCESS_BIT_EXT = 0x2
    VK_PIPELINE_DEPTH_STENCIL_STATE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkPipelineColorBlendStateCreateFlagBits(IntEnum):
    VK_PIPELINE_COLOR_BLEND_STATE_CREATE_RASTERIZATION_ORDER_ATTACHMENT_ACCESS_BIT_EXT = 0x1
    VK_PIPELINE_COLOR_BLEND_STATE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkDescriptorPoolCreateFlagBits(IntEnum):
    VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT = 0x1
    VK_DESCRIPTOR_POOL_CREATE_UPDATE_AFTER_BIND_BIT = 0x2
    VK_DESCRIPTOR_POOL_CREATE_HOST_ONLY_BIT_EXT = 0x4
    VK_DESCRIPTOR_POOL_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkDescriptorSetLayoutCreateFlagBits(IntEnum):
    VK_DESCRIPTOR_SET_LAYOUT_CREATE_PUSH_DESCRIPTOR_BIT_KHR = 0x1
    VK_DESCRIPTOR_SET_LAYOUT_CREATE_UPDATE_AFTER_BIND_POOL_BIT = 0x2
    VK_DESCRIPTOR_SET_LAYOUT_CREATE_HOST_ONLY_POOL_BIT_EXT = 0x4
    VK_DESCRIPTOR_SET_LAYOUT_CREATE_DESCRIPTOR_BUFFER_BIT_EXT = 0x10
    VK_DESCRIPTOR_SET_LAYOUT_CREATE_EMBEDDED_IMMUTABLE_SAMPLERS_BIT_EXT = 0x20
    VK_DESCRIPTOR_SET_LAYOUT_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkDependencyFlagBits(IntEnum):
    VK_DEPENDENCY_BY_REGION_BIT = 0x1
    VK_DEPENDENCY_VIEW_LOCAL_BIT = 0x2
    VK_DEPENDENCY_DEVICE_GROUP_BIT = 0x4
    VK_DEPENDENCY_FEEDBACK_LOOP_BIT_EXT = 0x8
    VK_DEPENDENCY_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class LayoutCompareGenerationType(IntEnum):
    MoreThanCurrent = 0x0
    MuchMoreThanCurrent = 0x1
    SameAsCurrent = 0x2
    CompareSizeType = 0x3

class VkFramebufferCreateFlagBits(IntEnum):
    VK_FRAMEBUFFER_CREATE_IMAGELESS_BIT = 0x1
    VK_FRAMEBUFFER_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkSubpassDescriptionFlagBits(IntEnum):
    VK_SUBPASS_DESCRIPTION_PER_VIEW_ATTRIBUTES_BIT_NVX = 0x1
    VK_SUBPASS_DESCRIPTION_PER_VIEW_POSITION_X_ONLY_BIT_NVX = 0x2
    VK_SUBPASS_DESCRIPTION_FRAGMENT_REGION_BIT_QCOM = 0x4
    VK_SUBPASS_DESCRIPTION_SHADER_RESOLVE_BIT_QCOM = 0x8
    VK_SUBPASS_DESCRIPTION_RASTERIZATION_ORDER_ATTACHMENT_COLOR_ACCESS_BIT_EXT = 0x10
    VK_SUBPASS_DESCRIPTION_RASTERIZATION_ORDER_ATTACHMENT_DEPTH_ACCESS_BIT_EXT = 0x20
    VK_SUBPASS_DESCRIPTION_RASTERIZATION_ORDER_ATTACHMENT_STENCIL_ACCESS_BIT_EXT = 0x40
    VK_SUBPASS_DESCRIPTION_ENABLE_LEGACY_DITHERING_BIT_EXT = 0x80
    VK_SUBPASS_DESCRIPTION_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class eSMShaderCategory(IntEnum):
    eShaderCategory_Internal = 0x0
    eShaderCategory_Input = 0x1
    eShaderCategory_Artistic = 0x2
    eShaderCategory_Arithmetic = 0x3
    eShaderCategory_Utility = 0x4
    eShaderCategory_NumCategories = 0x5

class eSMShaderNodeType(IntEnum):
    eShaderNode_Input = 0x0
    eShaderNode_Output = 0x1
    eShaderNode_ReadTexture = 0x2
    eShaderNode_Value = 0x3
    eShaderNode_If = 0x4
    eShaderNode_Max = 0x5
    eShaderNode_Min = 0x6
    eShaderNode_Add = 0x7
    eShaderNode_AddWithScale = 0x8
    eShaderNode_Sub = 0x9
    eShaderNode_Mul = 0xA
    eShaderNode_Div = 0xB
    eShaderNode_Fract = 0xC
    eShaderNode_Floor = 0xD
    eShaderNode_Round = 0xE
    eShaderNode_Copy2 = 0xF
    eShaderNode_Copy3 = 0x10
    eShaderNode_Copy4 = 0x11
    eShaderNode_Sin = 0x12
    eShaderNode_Cos = 0x13
    eShaderNode_Sawtooth = 0x14
    eShaderNode_SquareWave = 0x15
    eShaderNode_Contrast = 0x16
    eShaderNode_Clamp = 0x17
    eShaderNode_Smoothstep = 0x18
    eShaderNode_SmoothRemap = 0x19
    eShaderNode_Saturate = 0x1A
    eShaderNode_Blend = 0x1B
    eShaderNode_RGBToHSV = 0x1C
    eShaderNode_HSVToRGB = 0x1D
    eShaderNode_DecodeNormalMap = 0x1E
    eShaderNode_DegreesToRadians = 0x1F
    eShaderNode_RadiansToDegrees = 0x20
    eShaderNode_UvFlipbook = 0x21
    eShaderNode_UvRotate = 0x22
    eShaderNode_ComputeAngle = 0x23
    eShaderNode_VertexInterpolate = 0x24
    eShaderNode_OneMinus = 0x25
    eShaderNode_Negate = 0x26
    eShaderNode_VectorLength = 0x27
    eShaderNode_Power = 0x28
    eShaderNode_ComputePerspectiveUV = 0x29
    eShaderNode_IndexMatch = 0x2A
    eShaderNode_NumTypes = 0x2B

class VkPerformanceCounterScopeKHR(IntEnum):
    VK_PERFORMANCE_COUNTER_SCOPE_COMMAND_BUFFER_KHR = 0x0
    VK_PERFORMANCE_COUNTER_SCOPE_RENDER_PASS_KHR = 0x1
    VK_PERFORMANCE_COUNTER_SCOPE_COMMAND_KHR = 0x2
    VK_PERFORMANCE_COUNTER_SCOPE_MAX_ENUM_KHR = 0x7FFFFFFF

class VkPerformanceCounterDescriptionFlagBitsKHR(IntEnum):
    VK_PERFORMANCE_COUNTER_DESCRIPTION_PERFORMANCE_IMPACTING_BIT_KHR = 0x1
    VK_PERFORMANCE_COUNTER_DESCRIPTION_CONCURRENTLY_IMPACTED_BIT_KHR = 0x2
    VK_PERFORMANCE_COUNTER_DESCRIPTION_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkQueueGlobalPriorityKHR(IntEnum):
    VK_QUEUE_GLOBAL_PRIORITY_LOW_KHR = 0x80
    VK_QUEUE_GLOBAL_PRIORITY_MEDIUM_KHR = 0x100
    VK_QUEUE_GLOBAL_PRIORITY_HIGH_KHR = 0x200
    VK_QUEUE_GLOBAL_PRIORITY_REALTIME_KHR = 0x400
    VK_QUEUE_GLOBAL_PRIORITY_MAX_ENUM_KHR = 0x7FFFFFFF

class AkNodeType(IntEnum):
    AkNodeType_Default = 0x0
    AkNodeType_Bus = 0x1

class VkDebugReportObjectTypeEXT(IntEnum):
    VK_DEBUG_REPORT_OBJECT_TYPE_UNKNOWN_EXT = 0x0
    VK_DEBUG_REPORT_OBJECT_TYPE_INSTANCE_EXT = 0x1
    VK_DEBUG_REPORT_OBJECT_TYPE_PHYSICAL_DEVICE_EXT = 0x2
    VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_EXT = 0x3
    VK_DEBUG_REPORT_OBJECT_TYPE_QUEUE_EXT = 0x4
    VK_DEBUG_REPORT_OBJECT_TYPE_SEMAPHORE_EXT = 0x5
    VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_BUFFER_EXT = 0x6
    VK_DEBUG_REPORT_OBJECT_TYPE_FENCE_EXT = 0x7
    VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_MEMORY_EXT = 0x8
    VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_EXT = 0x9
    VK_DEBUG_REPORT_OBJECT_TYPE_IMAGE_EXT = 0xA
    VK_DEBUG_REPORT_OBJECT_TYPE_EVENT_EXT = 0xB
    VK_DEBUG_REPORT_OBJECT_TYPE_QUERY_POOL_EXT = 0xC
    VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_VIEW_EXT = 0xD
    VK_DEBUG_REPORT_OBJECT_TYPE_IMAGE_VIEW_EXT = 0xE
    VK_DEBUG_REPORT_OBJECT_TYPE_SHADER_MODULE_EXT = 0xF
    VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_CACHE_EXT = 0x10
    VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_LAYOUT_EXT = 0x11
    VK_DEBUG_REPORT_OBJECT_TYPE_RENDER_PASS_EXT = 0x12
    VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_EXT = 0x13
    VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_SET_LAYOUT_EXT = 0x14
    VK_DEBUG_REPORT_OBJECT_TYPE_SAMPLER_EXT = 0x15
    VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_POOL_EXT = 0x16
    VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_SET_EXT = 0x17
    VK_DEBUG_REPORT_OBJECT_TYPE_FRAMEBUFFER_EXT = 0x18
    VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_POOL_EXT = 0x19
    VK_DEBUG_REPORT_OBJECT_TYPE_SURFACE_KHR_EXT = 0x1A
    VK_DEBUG_REPORT_OBJECT_TYPE_SWAPCHAIN_KHR_EXT = 0x1B
    VK_DEBUG_REPORT_OBJECT_TYPE_DEBUG_REPORT_CALLBACK_EXT_EXT = 0x1C
    VK_DEBUG_REPORT_OBJECT_TYPE_DISPLAY_KHR_EXT = 0x1D
    VK_DEBUG_REPORT_OBJECT_TYPE_DISPLAY_MODE_KHR_EXT = 0x1E
    VK_DEBUG_REPORT_OBJECT_TYPE_VALIDATION_CACHE_EXT_EXT = 0x21
    VK_DEBUG_REPORT_OBJECT_TYPE_CU_MODULE_NVX_EXT = 0x3B9B3B48
    VK_DEBUG_REPORT_OBJECT_TYPE_CU_FUNCTION_NVX_EXT = 0x3B9B3B49
    VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_UPDATE_TEMPLATE_EXT = 0x3B9C1608
    VK_DEBUG_REPORT_OBJECT_TYPE_ACCELERATION_STRUCTURE_KHR_EXT = 0x3B9D13F0
    VK_DEBUG_REPORT_OBJECT_TYPE_SAMPLER_YCBCR_CONVERSION_EXT = 0x3B9D2B60
    VK_DEBUG_REPORT_OBJECT_TYPE_ACCELERATION_STRUCTURE_NV_EXT = 0x3B9D4E88
    VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_COLLECTION_FUCHSIA_EXT = 0x3BA05FB0
    VK_DEBUG_REPORT_OBJECT_TYPE_MAX_ENUM_EXT = 0x7FFFFFFF

class AkMemID(IntEnum):
    AkMemID_Object = 0x0
    AkMemID_Event = 0x1
    AkMemID_Structure = 0x2
    AkMemID_Media = 0x3
    AkMemID_GameObject = 0x4
    AkMemID_Processing = 0x5
    AkMemID_ProcessingPlugin = 0x6
    AkMemID_Streaming = 0x7
    AkMemID_StreamingIO = 0x8
    AkMemID_SpatialAudio = 0x9
    AkMemID_SpatialAudioGeometry = 0xA
    AkMemID_SpatialAudioPaths = 0xB
    AkMemID_GameSim = 0xC
    AkMemID_MonitorQueue = 0xD
    AkMemID_Profiler = 0xE
    AkMemID_FilePackage = 0xF
    AkMemID_SoundEngine = 0x10
    AkMemID_Integration = 0x11
    AkMemID_NUM = 0x12
    AkMemID_MASK = 0x1FFFFFFF
    AkMemType_Media = 0x20000000
    AkMemType_Device = 0x40000000
    AkMemType_NoTrack = 0x80000000

class VkPointClippingBehavior(IntEnum):
    VK_POINT_CLIPPING_BEHAVIOR_ALL_CLIP_PLANES = 0x0
    VK_POINT_CLIPPING_BEHAVIOR_USER_CLIP_PLANES_ONLY = 0x1
    VK_POINT_CLIPPING_BEHAVIOR_MAX_ENUM = 0x7FFFFFFF

class VkSurfaceCounterFlagBitsEXT(IntEnum):
    VK_SURFACE_COUNTER_VBLANK_BIT_EXT = 0x1
    VK_SURFACE_COUNTER_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkTessellationDomainOrigin(IntEnum):
    VK_TESSELLATION_DOMAIN_ORIGIN_UPPER_LEFT = 0x0
    VK_TESSELLATION_DOMAIN_ORIGIN_LOWER_LEFT = 0x1
    VK_TESSELLATION_DOMAIN_ORIGIN_MAX_ENUM = 0x7FFFFFFF

class VkSamplerYcbcrModelConversion(IntEnum):
    VK_SAMPLER_YCBCR_MODEL_CONVERSION_RGB_IDENTITY = 0x0
    VK_SAMPLER_YCBCR_MODEL_CONVERSION_YCBCR_IDENTITY = 0x1
    VK_SAMPLER_YCBCR_MODEL_CONVERSION_YCBCR_709 = 0x2
    VK_SAMPLER_YCBCR_MODEL_CONVERSION_YCBCR_601 = 0x3
    VK_SAMPLER_YCBCR_MODEL_CONVERSION_YCBCR_2020 = 0x4
    VK_SAMPLER_YCBCR_MODEL_CONVERSION_MAX_ENUM = 0x7FFFFFFF

class VkSamplerYcbcrRange(IntEnum):
    VK_SAMPLER_YCBCR_RANGE_ITU_FULL = 0x0
    VK_SAMPLER_YCBCR_RANGE_ITU_NARROW = 0x1
    VK_SAMPLER_YCBCR_RANGE_MAX_ENUM = 0x7FFFFFFF

class VkChromaLocation(IntEnum):
    VK_CHROMA_LOCATION_COSITED_EVEN = 0x0
    VK_CHROMA_LOCATION_MIDPOINT = 0x1
    VK_CHROMA_LOCATION_MAX_ENUM = 0x7FFFFFFF

class VkDescriptorUpdateTemplateType(IntEnum):
    VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_DESCRIPTOR_SET = 0x0
    VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_PUSH_DESCRIPTORS_KHR = 0x1
    VK_DESCRIPTOR_UPDATE_TEMPLATE_TYPE_MAX_ENUM = 0x7FFFFFFF

class VkPeerMemoryFeatureFlagBits(IntEnum):
    VK_PEER_MEMORY_FEATURE_COPY_SRC_BIT = 0x1
    VK_PEER_MEMORY_FEATURE_COPY_DST_BIT = 0x2
    VK_PEER_MEMORY_FEATURE_GENERIC_SRC_BIT = 0x4
    VK_PEER_MEMORY_FEATURE_GENERIC_DST_BIT = 0x8
    VK_PEER_MEMORY_FEATURE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkMemoryAllocateFlagBits(IntEnum):
    VK_MEMORY_ALLOCATE_DEVICE_MASK_BIT = 0x1
    VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT = 0x2
    VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_CAPTURE_REPLAY_BIT = 0x4
    VK_MEMORY_ALLOCATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkExternalMemoryHandleTypeFlagBits(IntEnum):
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT = 0x1
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_BIT = 0x2
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT = 0x4
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_TEXTURE_BIT = 0x8
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_TEXTURE_KMT_BIT = 0x10
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D12_HEAP_BIT = 0x20
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D12_RESOURCE_BIT = 0x40
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_ALLOCATION_BIT_EXT = 0x80
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_HOST_MAPPED_FOREIGN_MEMORY_BIT_EXT = 0x100
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT = 0x200
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_ANDROID_HARDWARE_BUFFER_BIT_ANDROID = 0x400
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_ZIRCON_VMO_BIT_FUCHSIA = 0x800
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_RDMA_ADDRESS_BIT_NV = 0x1000
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkExternalMemoryFeatureFlagBits(IntEnum):
    VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT = 0x1
    VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT = 0x2
    VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT = 0x4
    VK_EXTERNAL_MEMORY_FEATURE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkExternalFenceHandleTypeFlagBits(IntEnum):
    VK_EXTERNAL_FENCE_HANDLE_TYPE_OPAQUE_FD_BIT = 0x1
    VK_EXTERNAL_FENCE_HANDLE_TYPE_OPAQUE_WIN32_BIT = 0x2
    VK_EXTERNAL_FENCE_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT = 0x4
    VK_EXTERNAL_FENCE_HANDLE_TYPE_SYNC_FD_BIT = 0x8
    VK_EXTERNAL_FENCE_HANDLE_TYPE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkExternalFenceFeatureFlagBits(IntEnum):
    VK_EXTERNAL_FENCE_FEATURE_EXPORTABLE_BIT = 0x1
    VK_EXTERNAL_FENCE_FEATURE_IMPORTABLE_BIT = 0x2
    VK_EXTERNAL_FENCE_FEATURE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkFenceImportFlagBits(IntEnum):
    VK_FENCE_IMPORT_TEMPORARY_BIT = 0x1
    VK_FENCE_IMPORT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkSemaphoreImportFlagBits(IntEnum):
    VK_SEMAPHORE_IMPORT_TEMPORARY_BIT = 0x1
    VK_SEMAPHORE_IMPORT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkExternalSemaphoreHandleTypeFlagBits(IntEnum):
    VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_FD_BIT = 0x1
    VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_WIN32_BIT = 0x2
    VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT = 0x4
    VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_D3D12_FENCE_BIT = 0x8
    VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_SYNC_FD_BIT = 0x10
    VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_ZIRCON_EVENT_BIT_FUCHSIA = 0x80
    VK_EXTERNAL_SEMAPHORE_HANDLE_TYPE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkExternalSemaphoreFeatureFlagBits(IntEnum):
    VK_EXTERNAL_SEMAPHORE_FEATURE_EXPORTABLE_BIT = 0x1
    VK_EXTERNAL_SEMAPHORE_FEATURE_IMPORTABLE_BIT = 0x2
    VK_EXTERNAL_SEMAPHORE_FEATURE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkOpticalFlowPerformanceLevelNV(IntEnum):
    VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_UNKNOWN_NV = 0x0
    VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_SLOW_NV = 0x1
    VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_MEDIUM_NV = 0x2
    VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_FAST_NV = 0x3
    VK_OPTICAL_FLOW_PERFORMANCE_LEVEL_MAX_ENUM_NV = 0x7FFFFFFF

class VkPipelineRobustnessImageBehaviorEXT(IntEnum):
    VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_DEVICE_DEFAULT_EXT = 0x0
    VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_DISABLED_EXT = 0x1
    VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_ROBUST_IMAGE_ACCESS_EXT = 0x2
    VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_ROBUST_IMAGE_ACCESS_2_EXT = 0x3
    VK_PIPELINE_ROBUSTNESS_IMAGE_BEHAVIOR_MAX_ENUM_EXT = 0x7FFFFFFF

class VkIndirectCommandsTokenTypeNV(IntEnum):
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_SHADER_GROUP_NV = 0x0
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_STATE_FLAGS_NV = 0x1
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_INDEX_BUFFER_NV = 0x2
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_VERTEX_BUFFER_NV = 0x3
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_PUSH_CONSTANT_NV = 0x4
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_INDEXED_NV = 0x5
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_NV = 0x6
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_TASKS_NV = 0x7
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_MESH_TASKS_NV = 0x3B9FCB40
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_MAX_ENUM_NV = 0x7FFFFFFF

class VkShaderGroupShaderKHR(IntEnum):
    VK_SHADER_GROUP_SHADER_GENERAL_KHR = 0x0
    VK_SHADER_GROUP_SHADER_CLOSEST_HIT_KHR = 0x1
    VK_SHADER_GROUP_SHADER_ANY_HIT_KHR = 0x2
    VK_SHADER_GROUP_SHADER_INTERSECTION_KHR = 0x3
    VK_SHADER_GROUP_SHADER_MAX_ENUM_KHR = 0x7FFFFFFF

class VkPerformanceCounterUnitKHR(IntEnum):
    VK_PERFORMANCE_COUNTER_UNIT_GENERIC_KHR = 0x0
    VK_PERFORMANCE_COUNTER_UNIT_PERCENTAGE_KHR = 0x1
    VK_PERFORMANCE_COUNTER_UNIT_NANOSECONDS_KHR = 0x2
    VK_PERFORMANCE_COUNTER_UNIT_BYTES_KHR = 0x3
    VK_PERFORMANCE_COUNTER_UNIT_BYTES_PER_SECOND_KHR = 0x4
    VK_PERFORMANCE_COUNTER_UNIT_KELVIN_KHR = 0x5
    VK_PERFORMANCE_COUNTER_UNIT_WATTS_KHR = 0x6
    VK_PERFORMANCE_COUNTER_UNIT_VOLTS_KHR = 0x7
    VK_PERFORMANCE_COUNTER_UNIT_AMPS_KHR = 0x8
    VK_PERFORMANCE_COUNTER_UNIT_HERTZ_KHR = 0x9
    VK_PERFORMANCE_COUNTER_UNIT_CYCLES_KHR = 0xA
    VK_PERFORMANCE_COUNTER_UNIT_MAX_ENUM_KHR = 0x7FFFFFFF

class VkPerformanceCounterStorageKHR(IntEnum):
    VK_PERFORMANCE_COUNTER_STORAGE_INT32_KHR = 0x0
    VK_PERFORMANCE_COUNTER_STORAGE_INT64_KHR = 0x1
    VK_PERFORMANCE_COUNTER_STORAGE_UINT32_KHR = 0x2
    VK_PERFORMANCE_COUNTER_STORAGE_UINT64_KHR = 0x3
    VK_PERFORMANCE_COUNTER_STORAGE_FLOAT32_KHR = 0x4
    VK_PERFORMANCE_COUNTER_STORAGE_FLOAT64_KHR = 0x5
    VK_PERFORMANCE_COUNTER_STORAGE_MAX_ENUM_KHR = 0x7FFFFFFF

class StdVideoDecodeH264FieldOrderCount(IntEnum):
    STD_VIDEO_DECODE_H264_FIELD_ORDER_COUNT_TOP = 0x0
    STD_VIDEO_DECODE_H264_FIELD_ORDER_COUNT_BOTTOM = 0x1
    STD_VIDEO_DECODE_H264_FIELD_ORDER_COUNT_INVALID = 0x7FFFFFFF

class eWFCResult(IntEnum):
    EWFCResult_Failure = 0x0
    EWFCResult_Success = 0x1

class eHeavyAirType(IntEnum):
    EHeavyAir_None = 0x0
    EHeavyAir_Planet = 0x1
    EHeavyAir_Cave = 0x2
    EHeavyAir_Underwater = 0x3
    EHeavyAir_Space = 0x4
    EHeavyAir_AbandonedFreighter = 0x5
    EHeavyAir_SpaceStorm = 0x6

class VkFragmentShadingRateTypeNV(IntEnum):
    VK_FRAGMENT_SHADING_RATE_TYPE_FRAGMENT_SIZE_NV = 0x0
    VK_FRAGMENT_SHADING_RATE_TYPE_ENUMS_NV = 0x1
    VK_FRAGMENT_SHADING_RATE_TYPE_MAX_ENUM_NV = 0x7FFFFFFF

class VkFragmentShadingRateNV(IntEnum):
    VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_PIXEL_NV = 0x0
    VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_1X2_PIXELS_NV = 0x1
    VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_2X1_PIXELS_NV = 0x4
    VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_2X2_PIXELS_NV = 0x5
    VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_2X4_PIXELS_NV = 0x6
    VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_4X2_PIXELS_NV = 0x9
    VK_FRAGMENT_SHADING_RATE_1_INVOCATION_PER_4X4_PIXELS_NV = 0xA
    VK_FRAGMENT_SHADING_RATE_2_INVOCATIONS_PER_PIXEL_NV = 0xB
    VK_FRAGMENT_SHADING_RATE_4_INVOCATIONS_PER_PIXEL_NV = 0xC
    VK_FRAGMENT_SHADING_RATE_8_INVOCATIONS_PER_PIXEL_NV = 0xD
    VK_FRAGMENT_SHADING_RATE_16_INVOCATIONS_PER_PIXEL_NV = 0xE
    VK_FRAGMENT_SHADING_RATE_NO_INVOCATIONS_NV = 0xF
    VK_FRAGMENT_SHADING_RATE_MAX_ENUM_NV = 0x7FFFFFFF

class VkFragmentShadingRateCombinerOpKHR(IntEnum):
    VK_FRAGMENT_SHADING_RATE_COMBINER_OP_KEEP_KHR = 0x0
    VK_FRAGMENT_SHADING_RATE_COMBINER_OP_REPLACE_KHR = 0x1
    VK_FRAGMENT_SHADING_RATE_COMBINER_OP_MIN_KHR = 0x2
    VK_FRAGMENT_SHADING_RATE_COMBINER_OP_MAX_KHR = 0x3
    VK_FRAGMENT_SHADING_RATE_COMBINER_OP_MUL_KHR = 0x4
    VK_FRAGMENT_SHADING_RATE_COMBINER_OP_MAX_ENUM_KHR = 0x7FFFFFFF

class VkSparseImageFormatFlagBits(IntEnum):
    VK_SPARSE_IMAGE_FORMAT_SINGLE_MIPTAIL_BIT = 0x1
    VK_SPARSE_IMAGE_FORMAT_ALIGNED_MIP_SIZE_BIT = 0x2
    VK_SPARSE_IMAGE_FORMAT_NONSTANDARD_BLOCK_SIZE_BIT = 0x4
    VK_SPARSE_IMAGE_FORMAT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkTimeDomainEXT(IntEnum):
    VK_TIME_DOMAIN_DEVICE_EXT = 0x0
    VK_TIME_DOMAIN_CLOCK_MONOTONIC_EXT = 0x1
    VK_TIME_DOMAIN_CLOCK_MONOTONIC_RAW_EXT = 0x2
    VK_TIME_DOMAIN_QUERY_PERFORMANCE_COUNTER_EXT = 0x3
    VK_TIME_DOMAIN_MAX_ENUM_EXT = 0x7FFFFFFF

class VkGraphicsPipelineLibraryFlagBitsEXT(IntEnum):
    VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT = 0x1
    VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT = 0x2
    VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT = 0x4
    VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT = 0x8
    VK_GRAPHICS_PIPELINE_LIBRARY_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class cGcBuildingList(ctypes.Structure):
    class eKnownState(IntEnum):
        EKnownBuildings_AllBuildings = 0x0
        EKnownBuildings_OnlyUnknownBuildings = 0x1
        EKnownBuildings_OnlyKnownBuildings = 0x2

    def FindNearest(): pass

    def FindBuilding(): pass

    def GetNearestBuilding(): pass

    def AddBuilding(): pass

    def GetBuildingsInRange(): pass

    def GetBuildingsInRangeLocalAlloc(): pass

    def SortByDistance(): pass

    def RemoveBuilding(): pass

    def CompareBuildingDistances(): pass

    def CompareBuildingIDs(): pass

    _fields_ = [
        ("mpaBuildingArray", ctypes.c_ulonglong),
        ("mBuildingsByDistance", ctypes.c_char * 0x18),  # std::vector<cGcBuilding *,TkSTLAllocatorShim<cGcBuilding *,8,-1> >
        ("mBuildingsByID", ctypes.c_char * 0x18),  # std::vector<cGcBuilding *,TkSTLAllocatorShim<cGcBuilding *,8,-1> >
        ("maOutposts", ctypes.c_char * 0x18),  # std::vector<cGcBuilding *,TkSTLAllocatorShim<cGcBuilding *,8,-1> >
        ("mComparisonPosition", common.Vector3f),
    ]
    mpaBuildingArray: int
    mBuildingsByDistance: bytes
    mBuildingsByID: bytes
    maOutposts: bytes
    mComparisonPosition: common.Vector3f

class AkGroupType(IntEnum):
    AkGroupType_Switch = 0x0
    AkGroupType_State = 0x1

class VkConditionalRenderingFlagBitsEXT(IntEnum):
    VK_CONDITIONAL_RENDERING_INVERTED_BIT_EXT = 0x1
    VK_CONDITIONAL_RENDERING_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkMicromapTypeEXT(IntEnum):
    VK_MICROMAP_TYPE_OPACITY_MICROMAP_EXT = 0x0
    VK_MICROMAP_TYPE_MAX_ENUM_EXT = 0x7FFFFFFF

class VkCoarseSampleOrderTypeNV(IntEnum):
    VK_COARSE_SAMPLE_ORDER_TYPE_DEFAULT_NV = 0x0
    VK_COARSE_SAMPLE_ORDER_TYPE_CUSTOM_NV = 0x1
    VK_COARSE_SAMPLE_ORDER_TYPE_PIXEL_MAJOR_NV = 0x2
    VK_COARSE_SAMPLE_ORDER_TYPE_SAMPLE_MAJOR_NV = 0x3
    VK_COARSE_SAMPLE_ORDER_TYPE_MAX_ENUM_NV = 0x7FFFFFFF

class VkValidationCheckEXT(IntEnum):
    VK_VALIDATION_CHECK_ALL_EXT = 0x0
    VK_VALIDATION_CHECK_SHADERS_EXT = 0x1
    VK_VALIDATION_CHECK_MAX_ENUM_EXT = 0x7FFFFFFF

class VkDeviceFaultAddressTypeEXT(IntEnum):
    VK_DEVICE_FAULT_ADDRESS_TYPE_NONE_EXT = 0x0
    VK_DEVICE_FAULT_ADDRESS_TYPE_READ_INVALID_EXT = 0x1
    VK_DEVICE_FAULT_ADDRESS_TYPE_WRITE_INVALID_EXT = 0x2
    VK_DEVICE_FAULT_ADDRESS_TYPE_EXECUTE_INVALID_EXT = 0x3
    VK_DEVICE_FAULT_ADDRESS_TYPE_INSTRUCTION_POINTER_UNKNOWN_EXT = 0x4
    VK_DEVICE_FAULT_ADDRESS_TYPE_INSTRUCTION_POINTER_INVALID_EXT = 0x5
    VK_DEVICE_FAULT_ADDRESS_TYPE_INSTRUCTION_POINTER_FAULT_EXT = 0x6
    VK_DEVICE_FAULT_ADDRESS_TYPE_MAX_ENUM_EXT = 0x7FFFFFFF

class VkAccelerationStructureMemoryRequirementsTypeNV(IntEnum):
    VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_OBJECT_NV = 0x0
    VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_BUILD_SCRATCH_NV = 0x1
    VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_UPDATE_SCRATCH_NV = 0x2
    VK_ACCELERATION_STRUCTURE_MEMORY_REQUIREMENTS_TYPE_MAX_ENUM_NV = 0x7FFFFFFF

class cTkInteractivityManager(ctypes.Structure):
    class eMixerState(IntEnum):
        E_None = 0x0
        E_Running = 0x1

    def GetInstance(): pass

    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class eTkTextureExtras(IntEnum):
    ETextureExtra_DepthTexture = 0x0
    ETextureExtra_Num = 0x1

class VkDeviceAddressBindingFlagBitsEXT(IntEnum):
    VK_DEVICE_ADDRESS_BINDING_INTERNAL_OBJECT_BIT_EXT = 0x1
    VK_DEVICE_ADDRESS_BINDING_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkSubgroupFeatureFlagBits(IntEnum):
    VK_SUBGROUP_FEATURE_BASIC_BIT = 0x1
    VK_SUBGROUP_FEATURE_VOTE_BIT = 0x2
    VK_SUBGROUP_FEATURE_ARITHMETIC_BIT = 0x4
    VK_SUBGROUP_FEATURE_BALLOT_BIT = 0x8
    VK_SUBGROUP_FEATURE_SHUFFLE_BIT = 0x10
    VK_SUBGROUP_FEATURE_SHUFFLE_RELATIVE_BIT = 0x20
    VK_SUBGROUP_FEATURE_CLUSTERED_BIT = 0x40
    VK_SUBGROUP_FEATURE_QUAD_BIT = 0x80
    VK_SUBGROUP_FEATURE_PARTITIONED_BIT_NV = 0x100
    VK_SUBGROUP_FEATURE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class ePlanetDisplayTargetDisplayState(IntEnum):
    ETargetDisplayState_None = 0x0
    ETargetDisplayState_System = 0x1
    ETargetDisplayState_Planet = 0x2
    ETargetDisplayState_PlanetaryMapping = 0x3
    ETargetDisplayState_Discovery = 0x4

class ePlanetaryMappingDataState(IntEnum):
    ETargetDisplayState_Idle = 0x0
    ETargetDisplayState_SetNodes = 0x1
    ETargetDisplayState_Complete = 0x2
    ETargetDisplayState_CompleteCurrentPlanet = 0x3
    ETargetDisplayState_NumTypes = 0x4

class cTkIndexedQuantizedTriList(ctypes.Structure):
    class eStorageSizeType(IntEnum):
        U8 = 0x0
        U16 = 0x1

    def BuildFromTris(): pass

    def CopyTrisOut(): pass

    def WriteIndBuf(): pass

    _fields_ = [
        ("mpVertices", ctypes.c_void_p),
        ("mpIndices", ctypes.c_void_p),
        ("mpTiles", ctypes.c_ulonglong),
        ("mOutputStartForVertices", ctypes.c_uint64),
        ("mOutputStartForIndeces", ctypes.c_uint64),
        ("mNumVertices", ctypes.c_int32),
        ("mNumIndices", ctypes.c_int32),
        ("mCorner", ctypes.c_char * 0x6),  # cTkQuantVector3<short>
        ("_mVertexStorageType", ctypes.c_uint32),
        ("_mIndexStorageType", ctypes.c_uint32),
    ]
    mpVertices: int
    mpIndices: int
    mpTiles: int
    mOutputStartForVertices: int
    mOutputStartForIndeces: int
    mNumVertices: int
    mNumIndices: int
    mCorner: bytes
    _mVertexStorageType: int
    _mIndexStorageType: int

    @property
    def mVertexStorageType(self):
        return cTkIndexedQuantizedTriList.eStorageSizeType(self._mVertexStorageType)

    @property
    def mIndexStorageType(self):
        return cTkIndexedQuantizedTriList.eStorageSizeType(self._mIndexStorageType)


class VkCopyMicromapModeEXT(IntEnum):
    VK_COPY_MICROMAP_MODE_CLONE_EXT = 0x0
    VK_COPY_MICROMAP_MODE_SERIALIZE_EXT = 0x1
    VK_COPY_MICROMAP_MODE_DESERIALIZE_EXT = 0x2
    VK_COPY_MICROMAP_MODE_COMPACT_EXT = 0x3
    VK_COPY_MICROMAP_MODE_MAX_ENUM_EXT = 0x7FFFFFFF

class VkVideoCodecOperationFlagBitsKHR(IntEnum):
    VK_VIDEO_CODEC_OPERATION_NONE_KHR = 0x0
    VK_VIDEO_CODEC_OPERATION_DECODE_H264_BIT_KHR = 0x1
    VK_VIDEO_CODEC_OPERATION_DECODE_H265_BIT_KHR = 0x2
    VK_VIDEO_CODEC_OPERATION_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkDiscardRectangleModeEXT(IntEnum):
    VK_DISCARD_RECTANGLE_MODE_INCLUSIVE_EXT = 0x0
    VK_DISCARD_RECTANGLE_MODE_EXCLUSIVE_EXT = 0x1
    VK_DISCARD_RECTANGLE_MODE_MAX_ENUM_EXT = 0x7FFFFFFF

class StdVideoH264LevelIdc(IntEnum):
    STD_VIDEO_H264_LEVEL_IDC_1_0 = 0x0
    STD_VIDEO_H264_LEVEL_IDC_1_1 = 0x1
    STD_VIDEO_H264_LEVEL_IDC_1_2 = 0x2
    STD_VIDEO_H264_LEVEL_IDC_1_3 = 0x3
    STD_VIDEO_H264_LEVEL_IDC_2_0 = 0x4
    STD_VIDEO_H264_LEVEL_IDC_2_1 = 0x5
    STD_VIDEO_H264_LEVEL_IDC_2_2 = 0x6
    STD_VIDEO_H264_LEVEL_IDC_3_0 = 0x7
    STD_VIDEO_H264_LEVEL_IDC_3_1 = 0x8
    STD_VIDEO_H264_LEVEL_IDC_3_2 = 0x9
    STD_VIDEO_H264_LEVEL_IDC_4_0 = 0xA
    STD_VIDEO_H264_LEVEL_IDC_4_1 = 0xB
    STD_VIDEO_H264_LEVEL_IDC_4_2 = 0xC
    STD_VIDEO_H264_LEVEL_IDC_5_0 = 0xD
    STD_VIDEO_H264_LEVEL_IDC_5_1 = 0xE
    STD_VIDEO_H264_LEVEL_IDC_5_2 = 0xF
    STD_VIDEO_H264_LEVEL_IDC_6_0 = 0x10
    STD_VIDEO_H264_LEVEL_IDC_6_1 = 0x11
    STD_VIDEO_H264_LEVEL_IDC_6_2 = 0x12
    STD_VIDEO_H264_LEVEL_IDC_INVALID = 0x7FFFFFFF

class VkShaderInfoTypeAMD(IntEnum):
    VK_SHADER_INFO_TYPE_STATISTICS_AMD = 0x0
    VK_SHADER_INFO_TYPE_BINARY_AMD = 0x1
    VK_SHADER_INFO_TYPE_DISASSEMBLY_AMD = 0x2
    VK_SHADER_INFO_TYPE_MAX_ENUM_AMD = 0x7FFFFFFF

class StdVideoH264NonVclNaluType(IntEnum):
    STD_VIDEO_H264_NON_VCL_NALU_TYPE_SPS = 0x0
    STD_VIDEO_H264_NON_VCL_NALU_TYPE_PPS = 0x1
    STD_VIDEO_H264_NON_VCL_NALU_TYPE_AUD = 0x2
    STD_VIDEO_H264_NON_VCL_NALU_TYPE_PREFIX = 0x3
    STD_VIDEO_H264_NON_VCL_NALU_TYPE_END_OF_SEQUENCE = 0x4
    STD_VIDEO_H264_NON_VCL_NALU_TYPE_END_OF_STREAM = 0x5
    STD_VIDEO_H264_NON_VCL_NALU_TYPE_PRECODED = 0x6
    STD_VIDEO_H264_NON_VCL_NALU_TYPE_INVALID = 0x7FFFFFFF

class VkBlendOverlapEXT(IntEnum):
    VK_BLEND_OVERLAP_UNCORRELATED_EXT = 0x0
    VK_BLEND_OVERLAP_DISJOINT_EXT = 0x1
    VK_BLEND_OVERLAP_CONJOINT_EXT = 0x2
    VK_BLEND_OVERLAP_MAX_ENUM_EXT = 0x7FFFFFFF

class VkAccelerationStructureBuildTypeKHR(IntEnum):
    VK_ACCELERATION_STRUCTURE_BUILD_TYPE_HOST_KHR = 0x0
    VK_ACCELERATION_STRUCTURE_BUILD_TYPE_DEVICE_KHR = 0x1
    VK_ACCELERATION_STRUCTURE_BUILD_TYPE_HOST_OR_DEVICE_KHR = 0x2
    VK_ACCELERATION_STRUCTURE_BUILD_TYPE_MAX_ENUM_KHR = 0x7FFFFFFF

class eTkShadingRate(IntEnum):
    EShadingRate_NoChange = 0x0
    EShadingRate_Reduce_1x2 = 0x1
    EShadingRate_Reduce_2x1 = 0x4
    EShadingRate_Reduce_2x2 = 0x5
    EShadingRate_Reduce_2x4 = 0x6
    EShadingRate_Reduce_4x2 = 0x9
    EShadingRate_Reduce_4x4 = 0xA

class VkComponentTypeNV(IntEnum):
    VK_COMPONENT_TYPE_FLOAT16_NV = 0x0
    VK_COMPONENT_TYPE_FLOAT32_NV = 0x1
    VK_COMPONENT_TYPE_FLOAT64_NV = 0x2
    VK_COMPONENT_TYPE_SINT8_NV = 0x3
    VK_COMPONENT_TYPE_SINT16_NV = 0x4
    VK_COMPONENT_TYPE_SINT32_NV = 0x5
    VK_COMPONENT_TYPE_SINT64_NV = 0x6
    VK_COMPONENT_TYPE_UINT8_NV = 0x7
    VK_COMPONENT_TYPE_UINT16_NV = 0x8
    VK_COMPONENT_TYPE_UINT32_NV = 0x9
    VK_COMPONENT_TYPE_UINT64_NV = 0xA
    VK_COMPONENT_TYPE_MAX_ENUM_NV = 0x7FFFFFFF

class StdVideoH265AspectRatioIdc(IntEnum):
    STD_VIDEO_H265_ASPECT_RATIO_IDC_UNSPECIFIED = 0x0
    STD_VIDEO_H265_ASPECT_RATIO_IDC_SQUARE = 0x1
    STD_VIDEO_H265_ASPECT_RATIO_IDC_12_11 = 0x2
    STD_VIDEO_H265_ASPECT_RATIO_IDC_10_11 = 0x3
    STD_VIDEO_H265_ASPECT_RATIO_IDC_16_11 = 0x4
    STD_VIDEO_H265_ASPECT_RATIO_IDC_40_33 = 0x5
    STD_VIDEO_H265_ASPECT_RATIO_IDC_24_11 = 0x6
    STD_VIDEO_H265_ASPECT_RATIO_IDC_20_11 = 0x7
    STD_VIDEO_H265_ASPECT_RATIO_IDC_32_11 = 0x8
    STD_VIDEO_H265_ASPECT_RATIO_IDC_80_33 = 0x9
    STD_VIDEO_H265_ASPECT_RATIO_IDC_18_11 = 0xA
    STD_VIDEO_H265_ASPECT_RATIO_IDC_15_11 = 0xB
    STD_VIDEO_H265_ASPECT_RATIO_IDC_64_33 = 0xC
    STD_VIDEO_H265_ASPECT_RATIO_IDC_160_99 = 0xD
    STD_VIDEO_H265_ASPECT_RATIO_IDC_4_3 = 0xE
    STD_VIDEO_H265_ASPECT_RATIO_IDC_3_2 = 0xF
    STD_VIDEO_H265_ASPECT_RATIO_IDC_2_1 = 0x10
    STD_VIDEO_H265_ASPECT_RATIO_IDC_EXTENDED_SAR = 0xFF
    STD_VIDEO_H265_ASPECT_RATIO_IDC_INVALID = 0x7FFFFFFF

class VkValidationFeatureDisableEXT(IntEnum):
    VK_VALIDATION_FEATURE_DISABLE_ALL_EXT = 0x0
    VK_VALIDATION_FEATURE_DISABLE_SHADERS_EXT = 0x1
    VK_VALIDATION_FEATURE_DISABLE_THREAD_SAFETY_EXT = 0x2
    VK_VALIDATION_FEATURE_DISABLE_API_PARAMETERS_EXT = 0x3
    VK_VALIDATION_FEATURE_DISABLE_OBJECT_LIFETIMES_EXT = 0x4
    VK_VALIDATION_FEATURE_DISABLE_CORE_CHECKS_EXT = 0x5
    VK_VALIDATION_FEATURE_DISABLE_UNIQUE_HANDLES_EXT = 0x6
    VK_VALIDATION_FEATURE_DISABLE_SHADER_VALIDATION_CACHE_EXT = 0x7
    VK_VALIDATION_FEATURE_DISABLE_MAX_ENUM_EXT = 0x7FFFFFFF

class eBaseUploadLoadState(IntEnum):
    EBaseUploadLoadState_Inactive = 0x0
    EBaseUploadLoadState_Active = 0x1
    EBaseUploadLoadState_Setup = 0x2
    EBaseUploadLoadState_TakingScreenshot = 0x3
    EBaseUploadLoadState_ImageTaken = 0x4
    EBaseUploadLoadState_ImageTakenAndSetup = 0x5
    eBaseUploadLoadState_Complete = 0x6

class VkVideoCapabilityFlagBitsKHR(IntEnum):
    VK_VIDEO_CAPABILITY_PROTECTED_CONTENT_BIT_KHR = 0x1
    VK_VIDEO_CAPABILITY_SEPARATE_REFERENCE_IMAGES_BIT_KHR = 0x2
    VK_VIDEO_CAPABILITY_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class FrontendTextInputState(IntEnum):
    EFrontendTextInputState_Free = 0x0
    EFrontendTextInputState_WaitingForText = 0x1
    EFrontendTextInputState_TextReady = 0x2
    EFrontendTextInputState_ProfanityChecking = 0x3
    EFrontendTextInputState_ProfanityChecked = 0x4

class eShipOperation(IntEnum):
    EShipOperation_None = 0x0
    EShipOperation_Teleport = 0x1
    EShipOperation_Full = 0x2

class VkAccelerationStructureCompatibilityKHR(IntEnum):
    VK_ACCELERATION_STRUCTURE_COMPATIBILITY_COMPATIBLE_KHR = 0x0
    VK_ACCELERATION_STRUCTURE_COMPATIBILITY_INCOMPATIBLE_KHR = 0x1
    VK_ACCELERATION_STRUCTURE_COMPATIBILITY_MAX_ENUM_KHR = 0x7FFFFFFF

class VkDirectDriverLoadingModeLUNARG(IntEnum):
    VK_DIRECT_DRIVER_LOADING_MODE_EXCLUSIVE_LUNARG = 0x0
    VK_DIRECT_DRIVER_LOADING_MODE_INCLUSIVE_LUNARG = 0x1
    VK_DIRECT_DRIVER_LOADING_MODE_MAX_ENUM_LUNARG = 0x7FFFFFFF

class eWikiPageType(IntEnum):
    EWikiPageType_Wiki = 0x0
    EWikiPageType_Catalogue = 0x1
    EWikiPageType_MaterialsAndItemsCatalogue = 0x2
    EWikiPageType_CraftingAndTechnologyCatalogue = 0x3
    EWikiPageType_BuildablesCatalogue = 0x4
    EWikiPageType_Stories = 0x5
    EWikiPageType_RecipesCatalogue = 0x6
    EWikiPageType_Wonders = 0x7
    EWikiPageType_NumTypes = 0x8

class VkOpticalFlowSessionBindingPointNV(IntEnum):
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_UNKNOWN_NV = 0x0
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_INPUT_NV = 0x1
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_REFERENCE_NV = 0x2
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_HINT_NV = 0x3
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_FLOW_VECTOR_NV = 0x4
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_BACKWARD_FLOW_VECTOR_NV = 0x5
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_COST_NV = 0x6
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_BACKWARD_COST_NV = 0x7
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_GLOBAL_FLOW_NV = 0x8
    VK_OPTICAL_FLOW_SESSION_BINDING_POINT_MAX_ENUM_NV = 0x7FFFFFFF

class VkLineRasterizationModeEXT(IntEnum):
    VK_LINE_RASTERIZATION_MODE_DEFAULT_EXT = 0x0
    VK_LINE_RASTERIZATION_MODE_RECTANGULAR_EXT = 0x1
    VK_LINE_RASTERIZATION_MODE_BRESENHAM_EXT = 0x2
    VK_LINE_RASTERIZATION_MODE_RECTANGULAR_SMOOTH_EXT = 0x3
    VK_LINE_RASTERIZATION_MODE_MAX_ENUM_EXT = 0x7FFFFFFF

class StdVideoH264ModificationOfPicNumsIdc(IntEnum):
    STD_VIDEO_H264_MODIFICATION_OF_PIC_NUMS_IDC_SHORT_TERM_SUBTRACT = 0x0
    STD_VIDEO_H264_MODIFICATION_OF_PIC_NUMS_IDC_SHORT_TERM_ADD = 0x1
    STD_VIDEO_H264_MODIFICATION_OF_PIC_NUMS_IDC_LONG_TERM = 0x2
    STD_VIDEO_H264_MODIFICATION_OF_PIC_NUMS_IDC_END = 0x3
    STD_VIDEO_H264_MODIFICATION_OF_PIC_NUMS_IDC_INVALID = 0x7FFFFFFF

class VkOpacityMicromapSpecialIndexEXT(IntEnum):
    VK_OPACITY_MICROMAP_SPECIAL_INDEX_MAX_ENUM_EXT = 0x7FFFFFFF
    VK_OPACITY_MICROMAP_SPECIAL_INDEX_FULLY_UNKNOWN_OPAQUE_EXT = 0xFFFFFFFC
    VK_OPACITY_MICROMAP_SPECIAL_INDEX_FULLY_UNKNOWN_TRANSPARENT_EXT = 0xFFFFFFFD
    VK_OPACITY_MICROMAP_SPECIAL_INDEX_FULLY_OPAQUE_EXT = 0xFFFFFFFE
    VK_OPACITY_MICROMAP_SPECIAL_INDEX_FULLY_TRANSPARENT_EXT = 0xFFFFFFFF

class eMapObject(IntEnum):
    EMapMarker_Player = 0x0
    EMapMarker_Outpost = 0x1
    EMapMarker_Num = 0x2

class cGcHUDScreensPool(ctypes.Structure):
    class eType(IntEnum):
        Marker = 0x0
        TrackArrow = 0x1
        Distance = 0x2
        Position = 0x3
        Health = 0x4
        Damage = 0x5
        Num = 0x6

    def GetScreen(): pass

    def ResetPool(): pass

    _fields_ = [
        ("maScreens", ctypes.c_uint64 * 0xC8),  # cTkClassPoolHandle[200]
        ("mapScreenPools", ctypes.c_ulonglong * 0x6),  # cTkClassPoolHandle *[6]
        ("maCurrentScreens", ctypes.c_int32 * 0x6),  # int[6]
    ]
    maScreens: list[ctypes.c_uint64]
    mapScreenPools: list[ctypes.c_ulonglong]
    maCurrentScreens: list[ctypes.c_int32]

class StdVideoH264ChromaFormatIdc(IntEnum):
    STD_VIDEO_H264_CHROMA_FORMAT_IDC_MONOCHROME = 0x0
    STD_VIDEO_H264_CHROMA_FORMAT_IDC_420 = 0x1
    STD_VIDEO_H264_CHROMA_FORMAT_IDC_422 = 0x2
    STD_VIDEO_H264_CHROMA_FORMAT_IDC_444 = 0x3
    STD_VIDEO_H264_CHROMA_FORMAT_IDC_INVALID = 0x7FFFFFFF

class AkBusHierarchyFlags(IntEnum):
    AkBusHierarchy_Primary = 0x1

class VkVertexInputRate(IntEnum):
    VK_VERTEX_INPUT_RATE_VERTEX = 0x0
    VK_VERTEX_INPUT_RATE_INSTANCE = 0x1
    VK_VERTEX_INPUT_RATE_MAX_ENUM = 0x7FFFFFFF

class VkImageViewCreateFlagBits(IntEnum):
    VK_IMAGE_VIEW_CREATE_FRAGMENT_DENSITY_MAP_DYNAMIC_BIT_EXT = 0x1
    VK_IMAGE_VIEW_CREATE_FRAGMENT_DENSITY_MAP_DEFERRED_BIT_EXT = 0x2
    VK_IMAGE_VIEW_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT = 0x4
    VK_IMAGE_VIEW_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkBuildMicromapFlagBitsEXT(IntEnum):
    VK_BUILD_MICROMAP_PREFER_FAST_TRACE_BIT_EXT = 0x1
    VK_BUILD_MICROMAP_PREFER_FAST_BUILD_BIT_EXT = 0x2
    VK_BUILD_MICROMAP_ALLOW_COMPACTION_BIT_EXT = 0x4
    VK_BUILD_MICROMAP_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class StdVideoH264ProfileIdc(IntEnum):
    STD_VIDEO_H264_PROFILE_IDC_BASELINE = 0x42
    STD_VIDEO_H264_PROFILE_IDC_MAIN = 0x4D
    STD_VIDEO_H264_PROFILE_IDC_HIGH = 0x64
    STD_VIDEO_H264_PROFILE_IDC_HIGH_444_PREDICTIVE = 0xF4
    STD_VIDEO_H264_PROFILE_IDC_INVALID = 0x7FFFFFFF

class StdVideoH264PocType(IntEnum):
    STD_VIDEO_H264_POC_TYPE_0 = 0x0
    STD_VIDEO_H264_POC_TYPE_1 = 0x1
    STD_VIDEO_H264_POC_TYPE_2 = 0x2
    STD_VIDEO_H264_POC_TYPE_INVALID = 0x7FFFFFFF

class eMissionTable(IntEnum):
    EMissionTable_Main = 0x0
    EMissionTable_NPC = 0x1
    EMissionTable_Wiki = 0x2
    EMissionTable_Core = 0x3
    EMissionTable_Tutorial = 0x4
    EMissionTable_AtlasPath = 0x5
    EMissionTable_Recurring = 0x6
    EMissionTable_Fleet = 0x7
    EMissionTable_Water = 0x8
    EMissionTable_Multiplayer = 0x9
    EMissionTable_BaseComputer = 0xA
    EMissionTable_Community = 0xB
    EMissionTable_PlanetProc = 0xC
    EMissionTable_SpecialDialog = 0xD
    EMissionTable_SpacePOI = 0xE
    EMissionTable_Seasonal = 0xF
    EMissionTable_SentinelsAndSettlements = 0x10
    EMissionTable_Pirates = 0x11
    EMissionTable_StatStories = 0x12
    EMissionTable_Mods = 0x13
    EMissionTable_NumTables = 0x14

class eFrigateDamageResponse(IntEnum):
    EFrigateDamageResponse_Ignored = 0x0
    EFrigateDamageResponse_Destroyed = 0x1
    EFrigateDamageResponse_Continued = 0x2
    EFrigateDamageResponse_Recalled = 0x3
    EFrigateDamageResponse_RepairedByAnotherFrigate = 0x4
    EFrigateDamageResponse_NumTypes = 0x5

class eInventoryIcon(IntEnum):
    EInventoryIcon_Chemical = 0x0
    EInventoryIcon_Icon = 0x1
    EInventoryIcon_Empty = 0x2
    EInventoryIcon_NumTypes = 0x3

class cGcFrontendInventorySearch(ctypes.Structure):
    class eSearchType(IntEnum):
        ESearchType_InventoryType = 0x0
        ESearchType_SubstanceCatagory = 0x1
        ESearchType_Chargables = 0x2
        ESearchType_Cookable = 0x3
        ESearchType_CookableProduct = 0x4
        ESearchType_Bait = 0x5
        ESearchType_CreatureEgg = 0x6
        ESearchType_CreatureEggModifier = 0x7

    def cGcFrontendInventorySearch(): pass

    _fields_ = [
        ("_meType", ctypes.c_uint32),
        ("mxFlags", ctypes.c_char * 0x4),  # cTkBitArray<unsigned int,1,32>
        ("mxValidInventories", ctypes.c_char * 0x4),  # cTkBitArray<unsigned int,1,32>
        ("mpaSpecifics", ctypes.c_char * 0x8),  # cTkDynamicArray<TkID<128> > *
    ]
    _meType: int
    mxFlags: bytes
    mxValidInventories: bytes
    mpaSpecifics: bytes

    @property
    def meType(self):
        return cGcFrontendInventorySearch.eSearchType(self._meType)


class eInventorySlotType(IntEnum):
    EInventorySlotType_Inactive = 0x0
    EInventorySlotType_Empty = 0x1
    EInventorySlotType_Item = 0x2
    EInventorySlotType_Discovery = 0x3
    EInventorySlotType_Broken = 0x4
    EInventorySlotType_NumTypes = 0x5

class ePopupSlotActions(IntEnum):
    EPopupSlotAction_None = 0x0
    EPopupSlotAction_Discard = 0x1
    EPopupSlotAction_Charge = 0x2
    EPopupSlotAction_Dismantle = 0x4
    EPopupSlotAction_Transfer = 0x8
    EPopupSlotAction_Repair = 0x10
    EPopupSlotAction_Move = 0x20
    EPopupSlotAction_ChargeMenu = 0x40
    EPopupSlotAction_ShowBonusOutlines = 0x80
    EPopupSlotAction_FreighterWarp = 0x100
    EPopupSlotAction_BuildProductStack = 0x200
    EPopupSlotAction_ShowElementCounts = 0x400
    EPopupSlotAction_OutOfRange = 0x800
    EPopupSlotAction_DeployProduct = 0x1000
    EPopupSlotAction_TransferMore = 0x2000
    EPopupSlotAction_SplitStackAllowed = 0x4000
    EPopupSlotAction_Consume = 0x8000
    EPopupSlotAction_Refine = 0x10000
    EPopupSlotAction_PackageTech = 0x20000
    EPopupSlotAction_All = 0x300BF

class cGcImageBuilder(ctypes.Structure):
    class eInternalAngle(IntEnum):
        EInternalOrient_None = 0x0
        EInternalOrient_FlipHorizontal = 0x1
        EInternalOrient_FlipVertical = 0x2
        EInternalOrient_FlipVerticalAndHorizontal = 0x3
        EInternalOrient_Custom = 0x4

    _fields_ = [
        ("mpTexture", ctypes.c_ulonglong),
        ("_meInternalAngle", ctypes.c_uint32),
        ("mColour", common.Colour),
        ("mPos", common.Vector2f),
        ("mAlignment", common.Vector2f),
        ("mSize", common.Vector2f),
        ("mTiling", common.Vector2f),
        ("mfAngle", ctypes.c_float),
    ]
    mpTexture: int
    _meInternalAngle: int
    mColour: common.Colour
    mPos: common.Vector2f
    mAlignment: common.Vector2f
    mSize: common.Vector2f
    mTiling: common.Vector2f
    mfAngle: float

    @property
    def meInternalAngle(self):
        return cGcImageBuilder.eInternalAngle(self._meInternalAngle)


class cGcFrontendPageFunctions(ctypes.Structure):
    class eStatComparison(IntEnum):
        EStatComparison_None = 0x0
        EStatComparison_Worse = 0x1
        EStatComparison_Same = 0x2
        EStatComparison_Better = 0x3

    def DoTransferPopup(): pass

    def AcceptButtonPressed(): pass

    def BackButtonPressed(): pass

    def IsTechOnlySubInventory(): pass

    def Option2ButtonPressed(): pass

    def SetImage(): pass

    def SelectLayer(): pass

    def SetActiveSlotType(): pass

    def SetInventoryIcon(): pass

    def SetIconImage(): pass

    def SetPopupChemical(): pass

    def TouchscreenPressButtonPressed(): pass

    def SetColour(): pass

    def SetPageTitle(): pass

    def SetColourNoAlpha(): pass

    def AnyButtonPressed(): pass

    def SetSlotOutline(): pass

    def CheckMoveAndTransfer(): pass

    def SetInventorySlot(): pass

    def SetItemSlot(): pass

    def SetItemSlotBorder(): pass

    def SetItemSlotBorderRarity(): pass

    def SetPopupBasics(): pass

    def SetPopupDiscoveryTitleBar(): pass

    def GetTradeCategoryString(): pass

    def SetPopupIcon(): pass

    def SetPopupTransfer(): pass

    def SetQuantityBar(): pass

    def SetButton(): pass

    def SetRequirements(): pass

    def DoSlot(): pass

    def DoChemicalSlot(): pass

    def GetFreighterStorageInventories(): pass

    def DoProductSlot(): pass

    def DoTechnologySlot(): pass

    def AllowedToMoveTech(): pass

    def CanTechEvolve(): pass

    def GetEvolveQuality(): pass

    def DoTechStatSection(): pass

    def HideUsedItemPopupElementsForWiki(): pass

    def DoItemPopup(): pass

    def IsItemProcUpgradeProduct(): pass

    def LayerOverlapsPopup(): pass

    def DoBaseStatsWindow(): pass

    def DoLiveStats(): pass

    def CalculateShipStatWeapons(): pass

    def CalculateShipStatShield(): pass

    def CalculateShipStatWarp(): pass

    def CalculateShipStatEngines(): pass

    def CalculateStatFleetBoosts(): pass

    def DoStatBar(): pass

    def DoClassBox(): pass

    def DoIterateLayers(): pass

    def DoIterateAndScaleLayers(): pass

    def CanExitViaBackButton(): pass

    def SetStyleAlpha(): pass

    def PopulateInventoryPageInventory(): pass

    def DoInventoryExpandButton(): pass

    def DoScrollingInventory(): pass

    def DoInventoryPageTabs(): pass

    def PopulateDynamicInventorySlots(): pass

    def CanPickInteract(): pass

    def DoCrossplayPlatform(): pass

    def DoCrossplayDifficulty(): pass

    def CanInstallTech(): pass

    def DoInventorySlots(): pass

    def DoSingleInventorySlot(): pass

    def DoTechSlotPopupButtons(): pass

    def DoPopupCategories(): pass

    def ShowInventoryOutOfRangeMessage(): pass

    def DoTransferSendPopup(): pass

    def TransferItem(): pass

    def DoSlotToInventoryTransfer(): pass

    def GetHotSpotTypeForPage(): pass

    def DoTechChargePopup(): pass

    def DoToolbar(): pass

    def DoNotifyMessage(): pass

    def PopulateCatagorisedItemInventory(): pass

    def CheckSlotHighlight(): pass

    def SetEmptySlotBackground(): pass

    def DoEmptySlotPopup(): pass

    def DoRepairSlotPopup(): pass

    def DoBuildPopup(): pass

    def DoScrollDots(): pass

    def DoScrolling_NoPage(): pass

    def BuildSelectedItem(): pass

    def DoBuildInfoPopup(): pass

    def DoInventoryTitle(): pass

    def DoModelLocators(): pass

    def RenderLocatorArms(): pass

    def DoPlayerBanner(): pass

    def DoBanner(): pass

    def IsCookingIngredient(): pass

    def GetCreatureEggIndexFromProduct(): pass

    def DoCreatureEggPopup(): pass

    def GenerateCreatureEggText(): pass

    def GetExtraEggDescriptions(): pass

    def HandleEggHatchingSuccess(): pass

    def HandleEggHatchingFailure(): pass

    def DoOnlineStatusBar(): pass

    def DoManualTextScrollbox(): pass

    def SelectGraphic(): pass


class VkQueryPipelineStatisticFlagBits(IntEnum):
    VK_QUERY_PIPELINE_STATISTIC_INPUT_ASSEMBLY_VERTICES_BIT = 0x1
    VK_QUERY_PIPELINE_STATISTIC_INPUT_ASSEMBLY_PRIMITIVES_BIT = 0x2
    VK_QUERY_PIPELINE_STATISTIC_VERTEX_SHADER_INVOCATIONS_BIT = 0x4
    VK_QUERY_PIPELINE_STATISTIC_GEOMETRY_SHADER_INVOCATIONS_BIT = 0x8
    VK_QUERY_PIPELINE_STATISTIC_GEOMETRY_SHADER_PRIMITIVES_BIT = 0x10
    VK_QUERY_PIPELINE_STATISTIC_CLIPPING_INVOCATIONS_BIT = 0x20
    VK_QUERY_PIPELINE_STATISTIC_CLIPPING_PRIMITIVES_BIT = 0x40
    VK_QUERY_PIPELINE_STATISTIC_FRAGMENT_SHADER_INVOCATIONS_BIT = 0x80
    VK_QUERY_PIPELINE_STATISTIC_TESSELLATION_CONTROL_SHADER_PATCHES_BIT = 0x100
    VK_QUERY_PIPELINE_STATISTIC_TESSELLATION_EVALUATION_SHADER_INVOCATIONS_BIT = 0x200
    VK_QUERY_PIPELINE_STATISTIC_COMPUTE_SHADER_INVOCATIONS_BIT = 0x400
    VK_QUERY_PIPELINE_STATISTIC_TASK_SHADER_INVOCATIONS_BIT_EXT = 0x800
    VK_QUERY_PIPELINE_STATISTIC_MESH_SHADER_INVOCATIONS_BIT_EXT = 0x1000
    VK_QUERY_PIPELINE_STATISTIC_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkOpacityMicromapFormatEXT(IntEnum):
    VK_OPACITY_MICROMAP_FORMAT_2_STATE_EXT = 0x1
    VK_OPACITY_MICROMAP_FORMAT_4_STATE_EXT = 0x2
    VK_OPACITY_MICROMAP_FORMAT_MAX_ENUM_EXT = 0x7FFFFFFF

class eShipScreenViews(IntEnum):
    EShipScreen_Map = 0x0
    EShipScreen_Target = 0x1
    ENumShipScreenViews = 0x2

class eShipSideScreen(IntEnum):
    EShipSideScreen_Weapons = 0x0
    EShipSideScreen_Speed = 0x1
    EShipSideScreen_Target = 0x2
    EShipSideScreen_SolarSystem = 0x3
    EShipSideScreen_NumTypes = 0x4

class StdVideoH265PictureType(IntEnum):
    STD_VIDEO_H265_PICTURE_TYPE_P = 0x0
    STD_VIDEO_H265_PICTURE_TYPE_B = 0x1
    STD_VIDEO_H265_PICTURE_TYPE_I = 0x2
    STD_VIDEO_H265_PICTURE_TYPE_IDR = 0x3
    STD_VIDEO_H265_PICTURE_TYPE_INVALID = 0x7FFFFFFF

class ePhotoMode(IntEnum):
    EPhotoMode_Inactive = 0x0
    EPhotoMode_Menu = 0x1
    EPhotoMode_MoveCamera = 0x2
    EPhotoMode_NumEntries = 0x3

class eMessageCategory(IntEnum):
    EMessageCategory_Info = 0x0
    EMessageCategory_Mission = 0x1
    EMessageCategory_Danger = 0x2
    EMessageCategory_UrgentInfo = 0x3
    EMessageCategory_NumTypes = 0x4

class StdVideoH264MemMgmtControlOp(IntEnum):
    STD_VIDEO_H264_MEM_MGMT_CONTROL_OP_END = 0x0
    STD_VIDEO_H264_MEM_MGMT_CONTROL_OP_UNMARK_SHORT_TERM = 0x1
    STD_VIDEO_H264_MEM_MGMT_CONTROL_OP_UNMARK_LONG_TERM = 0x2
    STD_VIDEO_H264_MEM_MGMT_CONTROL_OP_MARK_LONG_TERM = 0x3
    STD_VIDEO_H264_MEM_MGMT_CONTROL_OP_SET_MAX_LONG_TERM_INDEX = 0x4
    STD_VIDEO_H264_MEM_MGMT_CONTROL_OP_UNMARK_ALL = 0x5
    STD_VIDEO_H264_MEM_MGMT_CONTROL_OP_MARK_CURRENT_AS_LONG_TERM = 0x6
    STD_VIDEO_H264_MEM_MGMT_CONTROL_OP_INVALID = 0x7FFFFFFF

class eInteractionEvent(IntEnum):
    EInteractionEvent_None = 0x0
    EInteractionEvent_StartSequence = 0x1
    EInteractionEvent_FinishSequence = 0x2
    EInteractionEvent_DoOptionSequence = 0x3
    EInteractionEvent_DoRewardSequence = 0x4
    EInteractionEvent_DoOption = 0x5
    EInteractionEvent_DoOptionNoSave = 0x6
    EInteractionEvent_DoMarkComplete = 0x7
    EInteractionEvent_GiveReward = 0x8
    EInteractionEvent_HandInMission = 0x9
    EInteractionEvent_RenounceMission = 0xA
    EInteractionEvent_DialogDone = 0xB
    EInteractionEvent_DescriptionDone = 0xC
    EInteractionEvent_InteractionEnded = 0xD
    EInteractionEvent_InteractionDismissed = 0xE
    EInteractionEvent_IdleMood = 0xF
    EInteractionEvent_SetProp = 0x10
    EInteractionEvent_ClaimPlayerBase = 0x11
    EInteractionEvent_UploadBase = 0x12
    EInteractionEvent_ReportBase = 0x13
    EInteractionEvent_DeleteBase = 0x14
    EInteractionEvent_MoveLegacyBaseHere = 0x15
    EInteractionEvent_ConvertNPCBaseToPlayerBase = 0x16
    EInteractionEvent_ClaimSettlement = 0x17
    EInteractionEvent_Upload_Message_Beacon = 0x18
    EInteractionEvent_Report_Content = 0x19
    EInteractionEvent_Block_User = 0x1A
    EInteractionEvent_ShowUI = 0x1B
    EInteractionEvent_HideUI = 0x1C
    EInteractionEvent_CameraOn = 0x1D
    EInteractionEvent_CameraOff = 0x1E
    EInteractionEvent_AnimateIn = 0x1F
    EInteractionEvent_AnimateOut = 0x20
    EInteractionEvent_LookAtOn = 0x21
    EInteractionEvent_LookAtOff = 0x22
    EInteractionEvent_Regenerate_Name = 0x23
    EInteractionEvent_Regenerate_Description = 0x24
    EInteractionEvent_FE_Title_On = 0x25
    EInteractionEvent_FE_Title_Off = 0x26
    EInteractionEvent_FE_Dialog_On = 0x27
    EInteractionEvent_FE_Start_Talking = 0x28
    EInteractionEvent_FE_Dialog_Typeout_Finished = 0x29
    EInteractionEvent_FE_Dialog_Off = 0x2A
    EInteractionEvent_FE_Text_On = 0x2B
    EInteractionEvent_FE_Text_Off = 0x2C
    EInteractionEvent_FE_Text_Frame_On = 0x2D
    EInteractionEvent_FE_Text_Frame_Off = 0x2E
    EInteractionEvent_FE_Options_On = 0x2F
    EInteractionEvent_FE_Not_Selected_Options_Off = 0x30
    EInteractionEvent_FE_Options_Off = 0x31
    EInteractionEvent_FE_Response_On = 0x32
    EInteractionEvent_FE_Response_Off = 0x33
    EInteractionEvent_FE_TextEntry_On = 0x34
    EInteractionEvent_FE_TextEntry_Off = 0x35

class eInteractionFrontendElements(IntEnum):
    EInteractionFrontendElement_Bars = 0x0
    EInteractionFrontendElement_Title = 0x1
    EInteractionFrontendElement_Dialog = 0x2
    EInteractionFrontendElement_Text = 0x3
    EInteractionFrontendElement_Text_Frame = 0x4
    EInteractionFrontendElement_Options = 0x5
    EInteractionFrontendElement_SelectedOption = 0x6
    EInteractionFrontendElement_Response = 0x7
    EInteractionFrontendElement_Num = 0x8

class VkProvokingVertexModeEXT(IntEnum):
    VK_PROVOKING_VERTEX_MODE_FIRST_VERTEX_EXT = 0x0
    VK_PROVOKING_VERTEX_MODE_LAST_VERTEX_EXT = 0x1
    VK_PROVOKING_VERTEX_MODE_MAX_ENUM_EXT = 0x7FFFFFFF

class StdVideoH265ChromaFormatIdc(IntEnum):
    STD_VIDEO_H265_CHROMA_FORMAT_IDC_MONOCHROME = 0x0
    STD_VIDEO_H265_CHROMA_FORMAT_IDC_420 = 0x1
    STD_VIDEO_H265_CHROMA_FORMAT_IDC_422 = 0x2
    STD_VIDEO_H265_CHROMA_FORMAT_IDC_444 = 0x3
    STD_VIDEO_H265_CHROMA_FORMAT_IDC_INVALID = 0x7FFFFFFF

class VkMicromapCreateFlagBitsEXT(IntEnum):
    VK_MICROMAP_CREATE_DEVICE_ADDRESS_CAPTURE_REPLAY_BIT_EXT = 0x1
    VK_MICROMAP_CREATE_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkDeviceEventTypeEXT(IntEnum):
    VK_DEVICE_EVENT_TYPE_DISPLAY_HOTPLUG_EXT = 0x0
    VK_DEVICE_EVENT_TYPE_MAX_ENUM_EXT = 0x7FFFFFFF

class VkVideoComponentBitDepthFlagBitsKHR(IntEnum):
    VK_VIDEO_COMPONENT_BIT_DEPTH_INVALID_KHR = 0x0
    VK_VIDEO_COMPONENT_BIT_DEPTH_8_BIT_KHR = 0x1
    VK_VIDEO_COMPONENT_BIT_DEPTH_10_BIT_KHR = 0x4
    VK_VIDEO_COMPONENT_BIT_DEPTH_12_BIT_KHR = 0x10
    VK_VIDEO_COMPONENT_BIT_DEPTH_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class EgShaderResource(ctypes.Structure):
    class List(IntEnum):
        ContextElem = 0x258
        SamplerElem = 0x259
        UniformElem = 0x25A
        ContNameStr = 0x25B
        SampNameStr = 0x25C
        UnifNameStr = 0x25D
        UnifDefValueF4 = 0x25E


class VkPipelineExecutableStatisticFormatKHR(IntEnum):
    VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_BOOL32_KHR = 0x0
    VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_INT64_KHR = 0x1
    VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_UINT64_KHR = 0x2
    VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_FLOAT64_KHR = 0x3
    VK_PIPELINE_EXECUTABLE_STATISTIC_FORMAT_MAX_ENUM_KHR = 0x7FFFFFFF

class EgOptions(ctypes.Structure):
    class List(IntEnum):
        MaxLogLevel = 0x1
        MaxNumMessages = 0x2
        TrilinearFiltering = 0x3
        MaxAnisotropy = 0x4
        TexCompression = 0x5
        LoadTextures = 0x6
        FastAnimation = 0x7
        ShadowMapSize = 0x8
        WireframeMode = 0x9
        DebugViewMode = 0xA
        DumpFailedShaders = 0xB
        GatherTimeStats = 0xC
        TextureQuality = 0xD


class VkDebugReportFlagBitsEXT(IntEnum):
    VK_DEBUG_REPORT_INFORMATION_BIT_EXT = 0x1
    VK_DEBUG_REPORT_WARNING_BIT_EXT = 0x2
    VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT = 0x4
    VK_DEBUG_REPORT_ERROR_BIT_EXT = 0x8
    VK_DEBUG_REPORT_DEBUG_BIT_EXT = 0x10
    VK_DEBUG_REPORT_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkDisplayEventTypeEXT(IntEnum):
    VK_DISPLAY_EVENT_TYPE_FIRST_PIXEL_OUT_EXT = 0x0
    VK_DISPLAY_EVENT_TYPE_MAX_ENUM_EXT = 0x7FFFFFFF

class VkPipelineCacheHeaderVersion(IntEnum):
    VK_PIPELINE_CACHE_HEADER_VERSION_ONE = 0x1
    VK_PIPELINE_CACHE_HEADER_VERSION_MAX_ENUM = 0x7FFFFFFF

class VkQueryPoolSamplingModeINTEL(IntEnum):
    VK_QUERY_POOL_SAMPLING_MODE_MANUAL_INTEL = 0x0
    VK_QUERY_POOL_SAMPLING_MODE_MAX_ENUM_INTEL = 0x7FFFFFFF

class VkOpticalFlowExecuteFlagBitsNV(IntEnum):
    VK_OPTICAL_FLOW_EXECUTE_DISABLE_TEMPORAL_HINTS_BIT_NV = 0x1
    VK_OPTICAL_FLOW_EXECUTE_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class eTkGraphicsFeature(IntEnum):
    EGraphicsFeature_NonPowerOfTwoTextures = 0x0

class eFleetPageMode(IntEnum):
    EFleetPageMode_Default = 0x0
    EFleetPageMode_SelectFrigateForExpedition = 0x1

class eDebriefLogEntryStage(IntEnum):
    EDebriefLogEntryStage_ShowingTitle = 0x0
    EDebriefLogEntryStage_ShowingDescription = 0x1
    EDebriefLogEntryStage_ShowingReward = 0x2
    EDebriefLogEntryStage_Complete = 0x3

class hkKeycodeManager(ctypes.Structure):
    class Products(IntEnum):
        PHYSICS = 0x0
        AI = 0x1
        CLOTH = 0x2
        PHYSICS_2012 = 0x3
        NUM = 0x4

    class ErrorCodes(IntEnum):
        INVALID_PRODUCT = 0xFFFF000A
        EMPTY_KEYCODE = 0xFFFF000B
        INVALID_FORMAT = 0xFFFF000C
        MISMATCHED_PRODUCT = 0xFFFF000D
        EXPIRED_KEYCODE = 0xFFFF000F
        RETRY_INIT_NODE = 0xFFFF00FF
        UNKNOWN_REASON = 0xFFFF0100

    def setKeycodes(): pass

    def getErrorMessage(): pass

    def getProductName(): pass

    def setKeycode(): pass


class EPad_WhiteButton(IntEnum):
    EP_Cross = 0x0
    EP_Cross_Disabled = 0x1
    EP_Circle = 0x2
    EP_Square = 0x3
    EP_SquareDisabled = 0x4
    EP_Triangle = 0x5
    EP_R3 = 0x6
    EP_DiscoveryUpload = 0x7
    EP_Num = 0x8

class eTeleporterNexusPageState(IntEnum):
    ETeleporterNexusPageAction_Idle = 0x0
    ETeleporterNexusPageAction_MoveToNext = 0x1
    ETeleporterNexusPageAction_MoveToPrevious = 0x2
    ETeleporterNexusPageAction_ClosePage = 0x3
    ETeleporterNexusPageAction_NumTypes = 0x4

class eMouseMode(IntEnum):
    EMouseMode_Ignore = 0x0
    EMouseMode_FreeCam = 0x1
    EMouseMode_Stick = 0x2
    EMouseMode_FollowCam = 0x3

class TryStoreMode(IntEnum):
    Commit = 0x0
    Peek = 0x1

class PersonalInventoryIndex(IntEnum):
    Main = 0x0
    Tech = 0x1
    Cargo = 0x2

class eStatsFunction(IntEnum):
    EFunction_Add = 0x0
    EFunction_Multiply = 0x1
    EFunction_Set = 0x2

class ePlayerFinalisePurchaseState(IntEnum):
    EPlayerFinalisePurchaseState_None = 0x0
    EPlayerFinalisePurchaseState_SpawnNewShip = 0x1

class eConstraintType(IntEnum):
    ETKPhysicsConstraint = 0x64
    ETKPhysics2DConstraint = 0x65
    ETKPhysicsSpringConstraint = 0x66
    ETKPhysicsSuspensionConstraint = 0x67
    ETKPhysicsAnimatedRagdollConstraint = 0x68
    ETKEndOfTkConstraintTypes = 0x69

class StdVideoH265LevelIdc(IntEnum):
    STD_VIDEO_H265_LEVEL_IDC_1_0 = 0x0
    STD_VIDEO_H265_LEVEL_IDC_2_0 = 0x1
    STD_VIDEO_H265_LEVEL_IDC_2_1 = 0x2
    STD_VIDEO_H265_LEVEL_IDC_3_0 = 0x3
    STD_VIDEO_H265_LEVEL_IDC_3_1 = 0x4
    STD_VIDEO_H265_LEVEL_IDC_4_0 = 0x5
    STD_VIDEO_H265_LEVEL_IDC_4_1 = 0x6
    STD_VIDEO_H265_LEVEL_IDC_5_0 = 0x7
    STD_VIDEO_H265_LEVEL_IDC_5_1 = 0x8
    STD_VIDEO_H265_LEVEL_IDC_5_2 = 0x9
    STD_VIDEO_H265_LEVEL_IDC_6_0 = 0xA
    STD_VIDEO_H265_LEVEL_IDC_6_1 = 0xB
    STD_VIDEO_H265_LEVEL_IDC_6_2 = 0xC
    STD_VIDEO_H265_LEVEL_IDC_INVALID = 0x7FFFFFFF

class VkDeviceQueueCreateFlagBits(IntEnum):
    VK_DEVICE_QUEUE_CREATE_PROTECTED_BIT = 0x1
    VK_DEVICE_QUEUE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkConservativeRasterizationModeEXT(IntEnum):
    VK_CONSERVATIVE_RASTERIZATION_MODE_DISABLED_EXT = 0x0
    VK_CONSERVATIVE_RASTERIZATION_MODE_OVERESTIMATE_EXT = 0x1
    VK_CONSERVATIVE_RASTERIZATION_MODE_UNDERESTIMATE_EXT = 0x2
    VK_CONSERVATIVE_RASTERIZATION_MODE_MAX_ENUM_EXT = 0x7FFFFFFF

class eSpaceMapObjectType(IntEnum):
    ESpaceMapObjectType_AIShipFriendly = 0x0
    ESpaceMapObjectType_AIShipEnemy = 0x1
    ESpaceMapObjectType_AIShipAttacking = 0x2
    ESpaceMapObjectType_AIShipSquadron = 0x3

class VkBuildAccelerationStructureModeKHR(IntEnum):
    VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR = 0x0
    VK_BUILD_ACCELERATION_STRUCTURE_MODE_UPDATE_KHR = 0x1
    VK_BUILD_ACCELERATION_STRUCTURE_MODE_MAX_ENUM_KHR = 0x7FFFFFFF

class VkImageCompressionFlagBitsEXT(IntEnum):
    VK_IMAGE_COMPRESSION_DEFAULT_EXT = 0x0
    VK_IMAGE_COMPRESSION_FIXED_RATE_DEFAULT_EXT = 0x1
    VK_IMAGE_COMPRESSION_FIXED_RATE_EXPLICIT_EXT = 0x2
    VK_IMAGE_COMPRESSION_DISABLED_EXT = 0x4
    VK_IMAGE_COMPRESSION_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkVideoDecodeCapabilityFlagBitsKHR(IntEnum):
    VK_VIDEO_DECODE_CAPABILITY_DPB_AND_OUTPUT_COINCIDE_BIT_KHR = 0x1
    VK_VIDEO_DECODE_CAPABILITY_DPB_AND_OUTPUT_DISTINCT_BIT_KHR = 0x2
    VK_VIDEO_DECODE_CAPABILITY_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class eWeaponReload(IntEnum):
    EWeaponReload_NotEnoughAmmo = 0x0
    EWeaponReload_FullAmmo = 0x1
    EWeaponReload_Broken = 0x2
    EWeaponReload_NoTech = 0x3
    EWeaponReload_Reload = 0x4

class eTargetPoint(IntEnum):
    ETargetPoint_Look = 0x0
    ETargetPoint_Shoot = 0x1
    ETargetPoint_AutoAim = 0x2

class VkDisplayPowerStateEXT(IntEnum):
    VK_DISPLAY_POWER_STATE_OFF_EXT = 0x0
    VK_DISPLAY_POWER_STATE_SUSPEND_EXT = 0x1
    VK_DISPLAY_POWER_STATE_ON_EXT = 0x2
    VK_DISPLAY_POWER_STATE_MAX_ENUM_EXT = 0x7FFFFFFF

def FindFrameForTightBounds(): pass

class Constants(IntEnum):
    SizeOfShaderSourceLoadBuffer = 0x80000

class EgMaterialResourceFlag(ctypes.Structure):
    class List(IntEnum):
        GlowMask = 0x1


class eTkBufferStorageBits(IntEnum):
    EBufferStorageBits_DynamicStorage = 0x1

class VkScopeNV(IntEnum):
    VK_SCOPE_DEVICE_NV = 0x1
    VK_SCOPE_WORKGROUP_NV = 0x2
    VK_SCOPE_SUBGROUP_NV = 0x3
    VK_SCOPE_QUEUE_FAMILY_NV = 0x5
    VK_SCOPE_MAX_ENUM_NV = 0x7FFFFFFF

class EgNodeParams(ctypes.Structure):
    class List(IntEnum):
        NameStr = 0x1
        IsMasterI = 0x2
        AttachmentStr = 0x3


class eResourceBlobSpawnStyle(IntEnum):
    EResourceBlobSpawnStyle_Generic = 0x0
    EResourceBlobSpawnStyle_Terrain = 0x1
    EResourceBlobSpawnStyle_Asteroid = 0x2
    EResourceBlobSpawnStyle_ChipDamage = 0x3
    EResourceBlobSpawnStyle_Melee = 0x4
    EResourceBlobSpawnStyle_NumTypes = 0x5

class VkVideoDecodeH264PictureLayoutFlagBitsKHR(IntEnum):
    VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_PROGRESSIVE_KHR = 0x0
    VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_INTERLACED_INTERLEAVED_LINES_BIT_KHR = 0x1
    VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_INTERLACED_SEPARATE_PLANES_BIT_KHR = 0x2
    VK_VIDEO_DECODE_H264_PICTURE_LAYOUT_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkDeviceGroupPresentModeFlagBitsKHR(IntEnum):
    VK_DEVICE_GROUP_PRESENT_MODE_LOCAL_BIT_KHR = 0x1
    VK_DEVICE_GROUP_PRESENT_MODE_REMOTE_BIT_KHR = 0x2
    VK_DEVICE_GROUP_PRESENT_MODE_SUM_BIT_KHR = 0x4
    VK_DEVICE_GROUP_PRESENT_MODE_LOCAL_MULTI_DEVICE_BIT_KHR = 0x8
    VK_DEVICE_GROUP_PRESENT_MODE_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkPerformanceParameterTypeINTEL(IntEnum):
    VK_PERFORMANCE_PARAMETER_TYPE_HW_COUNTERS_SUPPORTED_INTEL = 0x0
    VK_PERFORMANCE_PARAMETER_TYPE_STREAM_MARKER_VALID_BITS_INTEL = 0x1
    VK_PERFORMANCE_PARAMETER_TYPE_MAX_ENUM_INTEL = 0x7FFFFFFF

class VkImageCompressionFixedRateFlagBitsEXT(IntEnum):
    VK_IMAGE_COMPRESSION_FIXED_RATE_NONE_EXT = 0x0
    VK_IMAGE_COMPRESSION_FIXED_RATE_1BPC_BIT_EXT = 0x1
    VK_IMAGE_COMPRESSION_FIXED_RATE_2BPC_BIT_EXT = 0x2
    VK_IMAGE_COMPRESSION_FIXED_RATE_3BPC_BIT_EXT = 0x4
    VK_IMAGE_COMPRESSION_FIXED_RATE_4BPC_BIT_EXT = 0x8
    VK_IMAGE_COMPRESSION_FIXED_RATE_5BPC_BIT_EXT = 0x10
    VK_IMAGE_COMPRESSION_FIXED_RATE_6BPC_BIT_EXT = 0x20
    VK_IMAGE_COMPRESSION_FIXED_RATE_7BPC_BIT_EXT = 0x40
    VK_IMAGE_COMPRESSION_FIXED_RATE_8BPC_BIT_EXT = 0x80
    VK_IMAGE_COMPRESSION_FIXED_RATE_9BPC_BIT_EXT = 0x100
    VK_IMAGE_COMPRESSION_FIXED_RATE_10BPC_BIT_EXT = 0x200
    VK_IMAGE_COMPRESSION_FIXED_RATE_11BPC_BIT_EXT = 0x400
    VK_IMAGE_COMPRESSION_FIXED_RATE_12BPC_BIT_EXT = 0x800
    VK_IMAGE_COMPRESSION_FIXED_RATE_13BPC_BIT_EXT = 0x1000
    VK_IMAGE_COMPRESSION_FIXED_RATE_14BPC_BIT_EXT = 0x2000
    VK_IMAGE_COMPRESSION_FIXED_RATE_15BPC_BIT_EXT = 0x4000
    VK_IMAGE_COMPRESSION_FIXED_RATE_16BPC_BIT_EXT = 0x8000
    VK_IMAGE_COMPRESSION_FIXED_RATE_17BPC_BIT_EXT = 0x10000
    VK_IMAGE_COMPRESSION_FIXED_RATE_18BPC_BIT_EXT = 0x20000
    VK_IMAGE_COMPRESSION_FIXED_RATE_19BPC_BIT_EXT = 0x40000
    VK_IMAGE_COMPRESSION_FIXED_RATE_20BPC_BIT_EXT = 0x80000
    VK_IMAGE_COMPRESSION_FIXED_RATE_21BPC_BIT_EXT = 0x100000
    VK_IMAGE_COMPRESSION_FIXED_RATE_22BPC_BIT_EXT = 0x200000
    VK_IMAGE_COMPRESSION_FIXED_RATE_23BPC_BIT_EXT = 0x400000
    VK_IMAGE_COMPRESSION_FIXED_RATE_24BPC_BIT_EXT = 0x800000
    VK_IMAGE_COMPRESSION_FIXED_RATE_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkSubpassMergeStatusEXT(IntEnum):
    VK_SUBPASS_MERGE_STATUS_MERGED_EXT = 0x0
    VK_SUBPASS_MERGE_STATUS_DISALLOWED_EXT = 0x1
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_SIDE_EFFECTS_EXT = 0x2
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_SAMPLES_MISMATCH_EXT = 0x3
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_VIEWS_MISMATCH_EXT = 0x4
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_ALIASING_EXT = 0x5
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_DEPENDENCIES_EXT = 0x6
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_INCOMPATIBLE_INPUT_ATTACHMENT_EXT = 0x7
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_TOO_MANY_ATTACHMENTS_EXT = 0x8
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_INSUFFICIENT_STORAGE_EXT = 0x9
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_DEPTH_STENCIL_COUNT_EXT = 0xA
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_RESOLVE_ATTACHMENT_REUSE_EXT = 0xB
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_SINGLE_SUBPASS_EXT = 0xC
    VK_SUBPASS_MERGE_STATUS_NOT_MERGED_UNSPECIFIED_EXT = 0xD
    VK_SUBPASS_MERGE_STATUS_MAX_ENUM_EXT = 0x7FFFFFFF

class ePlayerInteraction(IntEnum):
    EPlayerInteraction_HoldButton = 0x0
    EPlayerInteraction_Shoot = 0x1
    EPlayerInteraction_Tap = 0x2
    EPlayerInteraction_NumTypes = 0x3
    EPlayerInteraction_None = 0xFFFFFFFF

class ePlayerInteractIcon(IntEnum):
    EPlayerInteractIcon_Hold = 0x0
    EPlayerInteractIcon_Mine = 0x1
    EPlayerInteractIcon_Shoot = 0x2
    EPlayerInteractIcon_Deconstruct = 0x3
    EPlayerInteractIcon_Armour = 0x4
    EPlayerInteractIcon_NumTypes = 0x5

class VkCommandPoolResetFlagBits(IntEnum):
    VK_COMMAND_POOL_RESET_RELEASE_RESOURCES_BIT = 0x1
    VK_COMMAND_POOL_RESET_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class eCantRewardReason(IntEnum):
    ECantRewardReason_InventoryFull = 0x0
    ECantRewardReason_InvalidBase = 0x1
    ECantRewardReason_BrokenShip = 0x2
    ECantRewardReason_NetworkLocked = 0x3
    ECantRewardReason_LandingUnclear = 0x4
    ECantRewardReason_CantOpenYourOwnLoot = 0x5
    ECantRewardReason_PartHasNoPower = 0x6
    ECantRewardReason_NoRoomInEggCollection = 0x7
    ECantRewardReason_NumTypes = 0x8

class VkDeviceFaultVendorBinaryHeaderVersionEXT(IntEnum):
    VK_DEVICE_FAULT_VENDOR_BINARY_HEADER_VERSION_ONE_EXT = 0x1
    VK_DEVICE_FAULT_VENDOR_BINARY_HEADER_VERSION_MAX_ENUM_EXT = 0x7FFFFFFF

class VkPerformanceOverrideTypeINTEL(IntEnum):
    VK_PERFORMANCE_OVERRIDE_TYPE_NULL_HARDWARE_INTEL = 0x0
    VK_PERFORMANCE_OVERRIDE_TYPE_FLUSH_GPU_CACHES_INTEL = 0x1
    VK_PERFORMANCE_OVERRIDE_TYPE_MAX_ENUM_INTEL = 0x7FFFFFFF

class ePlayerSquadronSummonState(IntEnum):
    EPlayerSquadronSummonState_None = 0x0
    EPlayerSquadronSummonState_Incoming = 0x1
    EPlayerSquadronSummonState_Active = 0x2
    EPlayerSquadronSummonState_Leaving = 0x3

class cTkWindEigenvectors(ctypes.Structure):
    def AllocElementVectors(): pass

    def CompactElementVectors(): pass

    _fields_ = [
        ("maWindEigens", ctypes.c_ulonglong),
        ("mpWindData", ctypes.c_ulonglong),
        ("miWindNumUsedEigenvalues", ctypes.c_int32),
        ("miNumWindEigenvalues", ctypes.c_int32),
        ("miNumWindJoints", ctypes.c_int32),
        ("maPackedSkinVectors", ctypes.c_ulonglong),
        ("maiPackedSkinVectorOrder", ctypes.c_ulonglong),
        ("miNumPackedJoints", ctypes.c_int32),
        ("maWindBaseMat", ctypes.c_ulonglong),
        ("maWindJointInds", ctypes.c_ulonglong),
        ("maParents", ctypes.c_ulonglong),
        ("mbJointShouldUseShearWind", ctypes.c_ulonglong),
        ("mfWindGenTime", ctypes.c_float),
        ("mfWindSolveTime", ctypes.c_float),
        ("mbUseImagComponents", ctypes.c_ubyte),
    ]
    maWindEigens: int
    mpWindData: int
    miWindNumUsedEigenvalues: int
    miNumWindEigenvalues: int
    miNumWindJoints: int
    maPackedSkinVectors: int
    maiPackedSkinVectorOrder: int
    miNumPackedJoints: int
    maWindBaseMat: int
    maWindJointInds: int
    maParents: int
    mbJointShouldUseShearWind: int
    mfWindGenTime: float
    mfWindSolveTime: float
    mbUseImagComponents: bool

class VkSparseMemoryBindFlagBits(IntEnum):
    VK_SPARSE_MEMORY_BIND_METADATA_BIT = 0x1
    VK_SPARSE_MEMORY_BIND_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkRenderPassCreateFlagBits(IntEnum):
    VK_RENDER_PASS_CREATE_TRANSFORM_BIT_QCOM = 0x2
    VK_RENDER_PASS_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkPresentScalingFlagBitsEXT(IntEnum):
    VK_PRESENT_SCALING_ONE_TO_ONE_BIT_EXT = 0x1
    VK_PRESENT_SCALING_ASPECT_RATIO_STRETCH_BIT_EXT = 0x2
    VK_PRESENT_SCALING_STRETCH_BIT_EXT = 0x4
    VK_PRESENT_SCALING_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class eTkDefaultShader(IntEnum):
    EDefaultShader_Debug = 0x0
    EDefaultShader_EngineDefault = 0x1

class ePetIncubationStatus(IntEnum):
    EPetIncubationStatus_None = 0x0
    EPetIncubationStatus_ReadyToStart = 0x1
    EPetIncubationStatus_Incubating = 0x2
    EPetIncubationStatus_ReadyToHatch = 0x3

class StdVideoH264WeightedBipredIdc(IntEnum):
    STD_VIDEO_H264_WEIGHTED_BIPRED_IDC_DEFAULT = 0x0
    STD_VIDEO_H264_WEIGHTED_BIPRED_IDC_EXPLICIT = 0x1
    STD_VIDEO_H264_WEIGHTED_BIPRED_IDC_IMPLICIT = 0x2
    STD_VIDEO_H264_WEIGHTED_BIPRED_IDC_INVALID = 0x7FFFFFFF

class VkDisplayPlaneAlphaFlagBitsKHR(IntEnum):
    VK_DISPLAY_PLANE_ALPHA_OPAQUE_BIT_KHR = 0x1
    VK_DISPLAY_PLANE_ALPHA_GLOBAL_BIT_KHR = 0x2
    VK_DISPLAY_PLANE_ALPHA_PER_PIXEL_BIT_KHR = 0x4
    VK_DISPLAY_PLANE_ALPHA_PER_PIXEL_PREMULTIPLIED_BIT_KHR = 0x8
    VK_DISPLAY_PLANE_ALPHA_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkPerformanceConfigurationTypeINTEL(IntEnum):
    VK_PERFORMANCE_CONFIGURATION_TYPE_COMMAND_QUEUE_METRICS_DISCOVERY_ACTIVATED_INTEL = 0x0
    VK_PERFORMANCE_CONFIGURATION_TYPE_MAX_ENUM_INTEL = 0x7FFFFFFF

class VkBuildMicromapModeEXT(IntEnum):
    VK_BUILD_MICROMAP_MODE_BUILD_EXT = 0x0
    VK_BUILD_MICROMAP_MODE_MAX_ENUM_EXT = 0x7FFFFFFF

class VkCoverageReductionModeNV(IntEnum):
    VK_COVERAGE_REDUCTION_MODE_MERGE_NV = 0x0
    VK_COVERAGE_REDUCTION_MODE_TRUNCATE_NV = 0x1
    VK_COVERAGE_REDUCTION_MODE_MAX_ENUM_NV = 0x7FFFFFFF

class StdVideoH264CabacInitIdc(IntEnum):
    STD_VIDEO_H264_CABAC_INIT_IDC_0 = 0x0
    STD_VIDEO_H264_CABAC_INIT_IDC_1 = 0x1
    STD_VIDEO_H264_CABAC_INIT_IDC_2 = 0x2
    STD_VIDEO_H264_CABAC_INIT_IDC_INVALID = 0x7FFFFFFF

class VkExternalMemoryFeatureFlagBitsNV(IntEnum):
    VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT_NV = 0x1
    VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT_NV = 0x2
    VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT_NV = 0x4
    VK_EXTERNAL_MEMORY_FEATURE_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class StdVideoH265ProfileIdc(IntEnum):
    STD_VIDEO_H265_PROFILE_IDC_MAIN = 0x1
    STD_VIDEO_H265_PROFILE_IDC_MAIN_10 = 0x2
    STD_VIDEO_H265_PROFILE_IDC_MAIN_STILL_PICTURE = 0x3
    STD_VIDEO_H265_PROFILE_IDC_FORMAT_RANGE_EXTENSIONS = 0x4
    STD_VIDEO_H265_PROFILE_IDC_SCC_EXTENSIONS = 0x9
    STD_VIDEO_H265_PROFILE_IDC_INVALID = 0x7FFFFFFF

class VkPerformanceValueTypeINTEL(IntEnum):
    VK_PERFORMANCE_VALUE_TYPE_UINT32_INTEL = 0x0
    VK_PERFORMANCE_VALUE_TYPE_UINT64_INTEL = 0x1
    VK_PERFORMANCE_VALUE_TYPE_FLOAT_INTEL = 0x2
    VK_PERFORMANCE_VALUE_TYPE_BOOL_INTEL = 0x3
    VK_PERFORMANCE_VALUE_TYPE_STRING_INTEL = 0x4
    VK_PERFORMANCE_VALUE_TYPE_MAX_ENUM_INTEL = 0x7FFFFFFF

class VkAcquireProfilingLockFlagBitsKHR(IntEnum):
    VK_ACQUIRE_PROFILING_LOCK_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkCoverageModulationModeNV(IntEnum):
    VK_COVERAGE_MODULATION_MODE_NONE_NV = 0x0
    VK_COVERAGE_MODULATION_MODE_RGB_NV = 0x1
    VK_COVERAGE_MODULATION_MODE_ALPHA_NV = 0x2
    VK_COVERAGE_MODULATION_MODE_RGBA_NV = 0x3
    VK_COVERAGE_MODULATION_MODE_MAX_ENUM_NV = 0x7FFFFFFF

class VkSamplerCreateFlagBits(IntEnum):
    VK_SAMPLER_CREATE_SUBSAMPLED_BIT_EXT = 0x1
    VK_SAMPLER_CREATE_SUBSAMPLED_COARSE_RECONSTRUCTION_BIT_EXT = 0x2
    VK_SAMPLER_CREATE_NON_SEAMLESS_CUBE_MAP_BIT_EXT = 0x4
    VK_SAMPLER_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT = 0x8
    VK_SAMPLER_CREATE_IMAGE_PROCESSING_BIT_QCOM = 0x10
    VK_SAMPLER_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkVideoCodingControlFlagBitsKHR(IntEnum):
    VK_VIDEO_CODING_CONTROL_RESET_BIT_KHR = 0x1
    VK_VIDEO_CODING_CONTROL_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkValidationFeatureEnableEXT(IntEnum):
    VK_VALIDATION_FEATURE_ENABLE_GPU_ASSISTED_EXT = 0x0
    VK_VALIDATION_FEATURE_ENABLE_GPU_ASSISTED_RESERVE_BINDING_SLOT_EXT = 0x1
    VK_VALIDATION_FEATURE_ENABLE_BEST_PRACTICES_EXT = 0x2
    VK_VALIDATION_FEATURE_ENABLE_DEBUG_PRINTF_EXT = 0x3
    VK_VALIDATION_FEATURE_ENABLE_SYNCHRONIZATION_VALIDATION_EXT = 0x4
    VK_VALIDATION_FEATURE_ENABLE_MAX_ENUM_EXT = 0x7FFFFFFF

class VkDeviceMemoryReportEventTypeEXT(IntEnum):
    VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_ALLOCATE_EXT = 0x0
    VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_FREE_EXT = 0x1
    VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_IMPORT_EXT = 0x2
    VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_UNIMPORT_EXT = 0x3
    VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_ALLOCATION_FAILED_EXT = 0x4
    VK_DEVICE_MEMORY_REPORT_EVENT_TYPE_MAX_ENUM_EXT = 0x7FFFFFFF

class StdVideoH265SliceType(IntEnum):
    STD_VIDEO_H265_SLICE_TYPE_B = 0x0
    STD_VIDEO_H265_SLICE_TYPE_P = 0x1
    STD_VIDEO_H265_SLICE_TYPE_I = 0x2
    STD_VIDEO_H265_SLICE_TYPE_INVALID = 0x7FFFFFFF

class eTkBufferTarget(IntEnum):
    EBufferTarget_ArrayBuffer = 0x0
    EBufferTarget_ElementArrayBuffer = 0x1
    EBufferTarget_PixelPackBuffer = 0x2
    EBufferTarget_PixelUnpackBuffer = 0x3
    EBufferTarget_Num = 0x4
    EBufferTarget_Invalid = 0xFFFFFFFF

class eDefenceForceSpawn(IntEnum):
    EDefenceForceSpawn_None = 0x0
    EDefenceForceSpawn_Waiting = 0x1
    EDefenceForceSpawn_WaitingForFrigateAssignment = 0x2
    EDefenceForceSpawn_SpawnFrigate = 0x3
    EDefenceForceSpawn_SpawnDefenders = 0x4
    EDefenceForceSpawn_Complete = 0x5

class ePreviousExperienceLocation(IntEnum):
    EPreviousExperienceLocation_Invalid = 0x0
    EPreviousExperienceLocation_Planet = 0x1
    EPreviousExperienceLocation_Space = 0x2
    EPreviousExperienceLocation_Freighter = 0x3

class StdVideoH264SliceType(IntEnum):
    STD_VIDEO_H264_SLICE_TYPE_P = 0x0
    STD_VIDEO_H264_SLICE_TYPE_B = 0x1
    STD_VIDEO_H264_SLICE_TYPE_I = 0x2
    STD_VIDEO_H264_SLICE_TYPE_INVALID = 0x7FFFFFFF

class ePerception(IntEnum):
    EPerception_None = 0x0
    EPerception_Hostile = 0x1
    EPerception_TooBig = 0x2
    EPerception_Ally = 0x3
    EPerception_Squad = 0x4

class eShipNotification(IntEnum):
    EShipNotification_Landing = 0x0
    EShipNotification_Landed = 0x1
    EShipNotification_TakeOff = 0x2
    EShipNotification_Enter = 0x3
    EShipNotification_NumTypes = 0x4

class VkShadingRatePaletteEntryNV(IntEnum):
    VK_SHADING_RATE_PALETTE_ENTRY_NO_INVOCATIONS_NV = 0x0
    VK_SHADING_RATE_PALETTE_ENTRY_16_INVOCATIONS_PER_PIXEL_NV = 0x1
    VK_SHADING_RATE_PALETTE_ENTRY_8_INVOCATIONS_PER_PIXEL_NV = 0x2
    VK_SHADING_RATE_PALETTE_ENTRY_4_INVOCATIONS_PER_PIXEL_NV = 0x3
    VK_SHADING_RATE_PALETTE_ENTRY_2_INVOCATIONS_PER_PIXEL_NV = 0x4
    VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_PIXEL_NV = 0x5
    VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_2X1_PIXELS_NV = 0x6
    VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_1X2_PIXELS_NV = 0x7
    VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_2X2_PIXELS_NV = 0x8
    VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_4X2_PIXELS_NV = 0x9
    VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_2X4_PIXELS_NV = 0xA
    VK_SHADING_RATE_PALETTE_ENTRY_1_INVOCATION_PER_4X4_PIXELS_NV = 0xB
    VK_SHADING_RATE_PALETTE_ENTRY_MAX_ENUM_NV = 0x7FFFFFFF

class VkRayTracingInvocationReorderModeNV(IntEnum):
    VK_RAY_TRACING_INVOCATION_REORDER_MODE_NONE_NV = 0x0
    VK_RAY_TRACING_INVOCATION_REORDER_MODE_REORDER_NV = 0x1
    VK_RAY_TRACING_INVOCATION_REORDER_MODE_MAX_ENUM_NV = 0x7FFFFFFF

class VkAccelerationStructureMotionInstanceTypeNV(IntEnum):
    VK_ACCELERATION_STRUCTURE_MOTION_INSTANCE_TYPE_STATIC_NV = 0x0
    VK_ACCELERATION_STRUCTURE_MOTION_INSTANCE_TYPE_MATRIX_MOTION_NV = 0x1
    VK_ACCELERATION_STRUCTURE_MOTION_INSTANCE_TYPE_SRT_MOTION_NV = 0x2
    VK_ACCELERATION_STRUCTURE_MOTION_INSTANCE_TYPE_MAX_ENUM_NV = 0x7FFFFFFF

class eTkBufferUsageHint(IntEnum):
    EBufferUsageHint_StreamDraw = 0x0
    EBufferUsageHint_StreamRead = 0x1
    EBufferUsageHint_StreamCopy = 0x2
    EBufferUsageHint_StaticDraw = 0x3
    EBufferUsageHint_StaticRead = 0x4
    EBufferUsageHint_StaticCopy = 0x5
    EBufferUsageHint_DynamicDraw = 0x6
    EBufferUsageHint_DynamicRead = 0x7
    EBufferUsageHint_DynamicCopy = 0x8
    EBufferUsageHint_Num = 0x9

class eTkCollisionDrawType(IntEnum):
    ECollisionDrawNone = 0x0
    ECollisionDrawTerrain = 0x1
    ECollisionDrawInstances = 0x2
    ECollisionDrawObjects = 0x4

class VkPresentGravityFlagBitsEXT(IntEnum):
    VK_PRESENT_GRAVITY_MIN_BIT_EXT = 0x1
    VK_PRESENT_GRAVITY_MAX_BIT_EXT = 0x2
    VK_PRESENT_GRAVITY_CENTERED_BIT_EXT = 0x4
    VK_PRESENT_GRAVITY_FLAG_BITS_MAX_ENUM_EXT = 0x7FFFFFFF

class VkDeviceAddressBindingTypeEXT(IntEnum):
    VK_DEVICE_ADDRESS_BINDING_TYPE_BIND_EXT = 0x0
    VK_DEVICE_ADDRESS_BINDING_TYPE_UNBIND_EXT = 0x1
    VK_DEVICE_ADDRESS_BINDING_TYPE_MAX_ENUM_EXT = 0x7FFFFFFF

class AkSourceChannelOrdering(IntEnum):
    SourceChannelOrdering_Standard = 0x0
    SourceChannelOrdering_Film = 0x1
    SourceChannelOrdering_FuMa = 0x2

class VkIndirectStateFlagBitsNV(IntEnum):
    VK_INDIRECT_STATE_FLAG_FRONTFACE_BIT_NV = 0x1
    VK_INDIRECT_STATE_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class VkShaderCorePropertiesFlagBitsAMD(IntEnum):
    VK_SHADER_CORE_PROPERTIES_FLAG_BITS_MAX_ENUM_AMD = 0x7FFFFFFF

class VkColorComponentFlagBits(IntEnum):
    VK_COLOR_COMPONENT_R_BIT = 0x1
    VK_COLOR_COMPONENT_G_BIT = 0x2
    VK_COLOR_COMPONENT_B_BIT = 0x4
    VK_COLOR_COMPONENT_A_BIT = 0x8
    VK_COLOR_COMPONENT_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkDeviceDiagnosticsConfigFlagBitsNV(IntEnum):
    VK_DEVICE_DIAGNOSTICS_CONFIG_ENABLE_SHADER_DEBUG_INFO_BIT_NV = 0x1
    VK_DEVICE_DIAGNOSTICS_CONFIG_ENABLE_RESOURCE_TRACKING_BIT_NV = 0x2
    VK_DEVICE_DIAGNOSTICS_CONFIG_ENABLE_AUTOMATIC_CHECKPOINTS_BIT_NV = 0x4
    VK_DEVICE_DIAGNOSTICS_CONFIG_ENABLE_SHADER_ERROR_REPORTING_BIT_NV = 0x8
    VK_DEVICE_DIAGNOSTICS_CONFIG_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class StdVideoH264AspectRatioIdc(IntEnum):
    STD_VIDEO_H264_ASPECT_RATIO_IDC_UNSPECIFIED = 0x0
    STD_VIDEO_H264_ASPECT_RATIO_IDC_SQUARE = 0x1
    STD_VIDEO_H264_ASPECT_RATIO_IDC_12_11 = 0x2
    STD_VIDEO_H264_ASPECT_RATIO_IDC_10_11 = 0x3
    STD_VIDEO_H264_ASPECT_RATIO_IDC_16_11 = 0x4
    STD_VIDEO_H264_ASPECT_RATIO_IDC_40_33 = 0x5
    STD_VIDEO_H264_ASPECT_RATIO_IDC_24_11 = 0x6
    STD_VIDEO_H264_ASPECT_RATIO_IDC_20_11 = 0x7
    STD_VIDEO_H264_ASPECT_RATIO_IDC_32_11 = 0x8
    STD_VIDEO_H264_ASPECT_RATIO_IDC_80_33 = 0x9
    STD_VIDEO_H264_ASPECT_RATIO_IDC_18_11 = 0xA
    STD_VIDEO_H264_ASPECT_RATIO_IDC_15_11 = 0xB
    STD_VIDEO_H264_ASPECT_RATIO_IDC_64_33 = 0xC
    STD_VIDEO_H264_ASPECT_RATIO_IDC_160_99 = 0xD
    STD_VIDEO_H264_ASPECT_RATIO_IDC_4_3 = 0xE
    STD_VIDEO_H264_ASPECT_RATIO_IDC_3_2 = 0xF
    STD_VIDEO_H264_ASPECT_RATIO_IDC_2_1 = 0x10
    STD_VIDEO_H264_ASPECT_RATIO_IDC_EXTENDED_SAR = 0xFF
    STD_VIDEO_H264_ASPECT_RATIO_IDC_INVALID = 0x7FFFFFFF

class eTkVertexFlag(IntEnum):
    EVertexFlag_Pos = 0x0
    EVertexFlag_Normal = 0x1
    EVertexFlag_Tangent = 0x2
    EVertexFlag_BiTangent = 0x3
    EVertexFlag_Joints = 0x4
    EVertexFlag_Weights = 0x5
    EVertexFlag_TexCoords0 = 0x6
    EVertexFlag_TexCoords1 = 0x7
    EVertexFlag_ParticleId = 0x8
    EVertexFlag_ParticleCornerId = 0x9
    EVertexFlag_Num = 0xA

class VkVideoChromaSubsamplingFlagBitsKHR(IntEnum):
    VK_VIDEO_CHROMA_SUBSAMPLING_INVALID_KHR = 0x0
    VK_VIDEO_CHROMA_SUBSAMPLING_MONOCHROME_BIT_KHR = 0x1
    VK_VIDEO_CHROMA_SUBSAMPLING_420_BIT_KHR = 0x2
    VK_VIDEO_CHROMA_SUBSAMPLING_422_BIT_KHR = 0x4
    VK_VIDEO_CHROMA_SUBSAMPLING_444_BIT_KHR = 0x8
    VK_VIDEO_CHROMA_SUBSAMPLING_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkMemoryOverallocationBehaviorAMD(IntEnum):
    VK_MEMORY_OVERALLOCATION_BEHAVIOR_DEFAULT_AMD = 0x0
    VK_MEMORY_OVERALLOCATION_BEHAVIOR_ALLOWED_AMD = 0x1
    VK_MEMORY_OVERALLOCATION_BEHAVIOR_DISALLOWED_AMD = 0x2
    VK_MEMORY_OVERALLOCATION_BEHAVIOR_MAX_ENUM_AMD = 0x7FFFFFFF

class StdVideoH264PictureType(IntEnum):
    STD_VIDEO_H264_PICTURE_TYPE_P = 0x0
    STD_VIDEO_H264_PICTURE_TYPE_B = 0x1
    STD_VIDEO_H264_PICTURE_TYPE_I = 0x2
    STD_VIDEO_H264_PICTURE_TYPE_IDR = 0x5
    STD_VIDEO_H264_PICTURE_TYPE_INVALID = 0x7FFFFFFF

class _D3DKMT_QUERYRESULT_PREEMPTION_ATTEMPT_RESULT(IntEnum):
    D3DKMT_PreemptionAttempt = 0x0
    D3DKMT_PreemptionAttemptSuccess = 0x1
    D3DKMT_PreemptionAttemptMissNoCommand = 0x2
    D3DKMT_PreemptionAttemptMissNotEnabled = 0x3
    D3DKMT_PreemptionAttemptMissNextFence = 0x4
    D3DKMT_PreemptionAttemptMissPagingCommand = 0x5
    D3DKMT_PreemptionAttemptMissSplittedCommand = 0x6
    D3DKMT_PreemptionAttemptMissFenceCommand = 0x7
    D3DKMT_PreemptionAttemptMissRenderPendingFlip = 0x8
    D3DKMT_PreemptionAttemptMissNotMakingProgress = 0x9
    D3DKMT_PreemptionAttemptMissLessPriority = 0xA
    D3DKMT_PreemptionAttemptMissRemainingQuantum = 0xB
    D3DKMT_PreemptionAttemptMissRemainingPreemptionQuantum = 0xC
    D3DKMT_PreemptionAttemptMissAlreadyPreempting = 0xD
    D3DKMT_PreemptionAttemptMissGlobalBlock = 0xE
    D3DKMT_PreemptionAttemptMissAlreadyRunning = 0xF
    D3DKMT_PreemptionAttemptStatisticsMax = 0x10

class _D3DKMT_QUERYSTATISTICS_DMA_PACKET_TYPE(IntEnum):
    D3DKMT_ClientRenderBuffer = 0x0
    D3DKMT_ClientPagingBuffer = 0x1
    D3DKMT_SystemPagingBuffer = 0x2
    D3DKMT_SystemPreemptionBuffer = 0x3
    D3DKMT_DmaPacketTypeMax = 0x4

class _D3DKMT_QUERYSTATISTICS_QUEUE_PACKET_TYPE(IntEnum):
    D3DKMT_RenderCommandBuffer = 0x0
    D3DKMT_DeferredCommandBuffer = 0x1
    D3DKMT_SystemCommandBuffer = 0x2
    D3DKMT_MmIoFlipCommandBuffer = 0x3
    D3DKMT_WaitCommandBuffer = 0x4
    D3DKMT_SignalCommandBuffer = 0x5
    D3DKMT_DeviceCommandBuffer = 0x6
    D3DKMT_SoftwareCommandBuffer = 0x7
    D3DKMT_QueuePacketTypeMax = 0x8

class _D3DKMT_QUERYSTATISTICS_ALLOCATION_PRIORITY_CLASS(IntEnum):
    D3DKMT_AllocationPriorityClassMinimum = 0x0
    D3DKMT_AllocationPriorityClassLow = 0x1
    D3DKMT_AllocationPriorityClassNormal = 0x2
    D3DKMT_AllocationPriorityClassHigh = 0x3
    D3DKMT_AllocationPriorityClassMaximum = 0x4
    D3DKMT_MaxAllocationPriorityClass = 0x5

class StorageVersions(ctypes.Structure):
    pass

class VkPipelineRobustnessBufferBehaviorEXT(IntEnum):
    VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_DEVICE_DEFAULT_EXT = 0x0
    VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_DISABLED_EXT = 0x1
    VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_EXT = 0x2
    VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_ROBUST_BUFFER_ACCESS_2_EXT = 0x3
    VK_PIPELINE_ROBUSTNESS_BUFFER_BEHAVIOR_MAX_ENUM_EXT = 0x7FFFFFFF

class VkSwapchainCreateFlagBitsKHR(IntEnum):
    VK_SWAPCHAIN_CREATE_SPLIT_INSTANCE_BIND_REGIONS_BIT_KHR = 0x1
    VK_SWAPCHAIN_CREATE_PROTECTED_BIT_KHR = 0x2
    VK_SWAPCHAIN_CREATE_MUTABLE_FORMAT_BIT_KHR = 0x4
    VK_SWAPCHAIN_CREATE_DEFERRED_MEMORY_ALLOCATION_BIT_EXT = 0x8
    VK_SWAPCHAIN_CREATE_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkVendorId(IntEnum):
    VK_VENDOR_ID_VIV = 0x10001
    VK_VENDOR_ID_VSI = 0x10002
    VK_VENDOR_ID_KAZAN = 0x10003
    VK_VENDOR_ID_CODEPLAY = 0x10004
    VK_VENDOR_ID_MESA = 0x10005
    VK_VENDOR_ID_POCL = 0x10006
    VK_VENDOR_ID_MAX_ENUM = 0x7FFFFFFF

class VkPipelineLayoutCreateFlagBits(IntEnum):
    VK_PIPELINE_LAYOUT_CREATE_INDEPENDENT_SETS_BIT_EXT = 0x2
    VK_PIPELINE_LAYOUT_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class ePlayerBoostCheck(IntEnum):
    EPlayerBoostCheck_Allowed = 0x0
    EPlayerBoostCheck_TooCloseToStation = 0x1
    EPlayerBoostCheck_TooCloseToFreighters = 0x2
    EPlayerBoostCheck_EnemiesNearby = 0x3
    EPlayerBoostCheck_InAtmosphere = 0x4
    EPlayerBoostCheck_TooCloseToAnomaly = 0x5
    EPlayerBoostCheck_UsingComms = 0x6
    EPlayerBoostCheck_NumTypes = 0x7

class cGcPlanetMappingManager(ctypes.Structure):
    class eMoveDirection(IntEnum):
        EMoveDirection_Up = 0x0
        EMoveDirection_Right = 0x1
        EMoveDirection_Down = 0x2
        EMoveDirection_Left = 0x3
        EMoveDirection_UpLeft = 0x4
        EMoveDirection_UpRight = 0x5
        EMoveDirection_DownLeft = 0x6
        EMoveDirection_DownRight = 0x7
        EMoveDirection_NUM_DIRECTIONS = 0x8

    def GetMappingDataFromUAWritable(): pass

    def GetSectionIndexesFromWorldPosition(): pass

    def GetCurrentCubeIndexes(): pass

    _fields_ = [
        ("mMappingInternal", ctypes.c_char * 0x18),  # std::vector<MappingDataInternal,TkSTLAllocatorShim<MappingDataInternal,8,-1> >
    ]
    mMappingInternal: bytes

class eMapPoleID(IntEnum):
    EMapPoleID_Null = 0x0
    EMapPoleID_1 = 0x1
    EMapPoleID_2 = 0x2
    EMapPoleID_3 = 0x4
    EMapPoleID_4 = 0x8
    EMapPoleID_5 = 0x10
    EMapPoleID_6 = 0x20
    EMapPoleID_7 = 0x40
    EMapPoleID_8 = 0x80
    EMapPoleID_9 = 0x100
    EMapPoleID_NUM_IDS = 0x101

class eCubeFace(IntEnum):
    ECubeFaceID_Right = 0x0
    ECubeFaceID_Left = 0x1
    ECubeFaceID_Back = 0x2
    ECubeFaceID_Front = 0x3
    ECubeFaceID_Bottom = 0x4
    ECubeFaceID_Top = 0x5
    ECubeFaceID_NUM_FACES = 0x6

class eSettlementStatLevel(IntEnum):
    ESettlementStatLevel_Bad = 0x0
    ESettlementStatLevel_Neutral = 0x1
    ESettlementStatLevel_Good = 0x2

class VkOpticalFlowSessionCreateFlagBitsNV(IntEnum):
    VK_OPTICAL_FLOW_SESSION_CREATE_ENABLE_HINT_BIT_NV = 0x1
    VK_OPTICAL_FLOW_SESSION_CREATE_ENABLE_COST_BIT_NV = 0x2
    VK_OPTICAL_FLOW_SESSION_CREATE_ENABLE_GLOBAL_FLOW_BIT_NV = 0x4
    VK_OPTICAL_FLOW_SESSION_CREATE_ALLOW_REGIONS_BIT_NV = 0x8
    VK_OPTICAL_FLOW_SESSION_CREATE_BOTH_DIRECTIONS_BIT_NV = 0x10
    VK_OPTICAL_FLOW_SESSION_CREATE_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class VkOpticalFlowUsageFlagBitsNV(IntEnum):
    VK_OPTICAL_FLOW_USAGE_UNKNOWN_NV = 0x0
    VK_OPTICAL_FLOW_USAGE_INPUT_BIT_NV = 0x1
    VK_OPTICAL_FLOW_USAGE_OUTPUT_BIT_NV = 0x2
    VK_OPTICAL_FLOW_USAGE_HINT_BIT_NV = 0x4
    VK_OPTICAL_FLOW_USAGE_COST_BIT_NV = 0x8
    VK_OPTICAL_FLOW_USAGE_GLOBAL_FLOW_BIT_NV = 0x10
    VK_OPTICAL_FLOW_USAGE_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class VkCommandBufferResetFlagBits(IntEnum):
    VK_COMMAND_BUFFER_RESET_RELEASE_RESOURCES_BIT = 0x1
    VK_COMMAND_BUFFER_RESET_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkIndirectCommandsLayoutUsageFlagBitsNV(IntEnum):
    VK_INDIRECT_COMMANDS_LAYOUT_USAGE_EXPLICIT_PREPROCESS_BIT_NV = 0x1
    VK_INDIRECT_COMMANDS_LAYOUT_USAGE_INDEXED_SEQUENCES_BIT_NV = 0x2
    VK_INDIRECT_COMMANDS_LAYOUT_USAGE_UNORDERED_SEQUENCES_BIT_NV = 0x4
    VK_INDIRECT_COMMANDS_LAYOUT_USAGE_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class VkVideoSessionCreateFlagBitsKHR(IntEnum):
    VK_VIDEO_SESSION_CREATE_PROTECTED_CONTENT_BIT_KHR = 0x1
    VK_VIDEO_SESSION_CREATE_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkOpticalFlowGridSizeFlagBitsNV(IntEnum):
    VK_OPTICAL_FLOW_GRID_SIZE_UNKNOWN_NV = 0x0
    VK_OPTICAL_FLOW_GRID_SIZE_1X1_BIT_NV = 0x1
    VK_OPTICAL_FLOW_GRID_SIZE_2X2_BIT_NV = 0x2
    VK_OPTICAL_FLOW_GRID_SIZE_4X4_BIT_NV = 0x4
    VK_OPTICAL_FLOW_GRID_SIZE_8X8_BIT_NV = 0x8
    VK_OPTICAL_FLOW_GRID_SIZE_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class ePhotoModeReturnResult(IntEnum):
    EPhotoModeReturnResult_Success = 0x0
    EPhotoModeReturnResult_UserCancelled = 0x1
    EPhotoModeReturnResult_SystemInterrupted = 0x2

class EgSceneGraphResource(ctypes.Structure):
    class List(IntEnum):
        LodDistances = 0x3E8
        LodDistancesDontAdjust = 0x3E9


class VkViewportCoordinateSwizzleNV(IntEnum):
    VK_VIEWPORT_COORDINATE_SWIZZLE_POSITIVE_X_NV = 0x0
    VK_VIEWPORT_COORDINATE_SWIZZLE_NEGATIVE_X_NV = 0x1
    VK_VIEWPORT_COORDINATE_SWIZZLE_POSITIVE_Y_NV = 0x2
    VK_VIEWPORT_COORDINATE_SWIZZLE_NEGATIVE_Y_NV = 0x3
    VK_VIEWPORT_COORDINATE_SWIZZLE_POSITIVE_Z_NV = 0x4
    VK_VIEWPORT_COORDINATE_SWIZZLE_NEGATIVE_Z_NV = 0x5
    VK_VIEWPORT_COORDINATE_SWIZZLE_POSITIVE_W_NV = 0x6
    VK_VIEWPORT_COORDINATE_SWIZZLE_NEGATIVE_W_NV = 0x7
    VK_VIEWPORT_COORDINATE_SWIZZLE_MAX_ENUM_NV = 0x7FFFFFFF

class VkValidationCacheHeaderVersionEXT(IntEnum):
    VK_VALIDATION_CACHE_HEADER_VERSION_ONE_EXT = 0x1
    VK_VALIDATION_CACHE_HEADER_VERSION_MAX_ENUM_EXT = 0x7FFFFFFF

class LineNodeParams(ctypes.Structure):
    class List(IntEnum):
        WidthF = 0x2904
        TimeLeftF = 0x2905
        DamageLevelF = 0x2906
        LengthF = 0x2907
        Colour = 0x2908
        ForceUpdateI = 0x2909


class VkExternalMemoryHandleTypeFlagBitsNV(IntEnum):
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_BIT_NV = 0x1
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT_NV = 0x2
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_IMAGE_BIT_NV = 0x4
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_IMAGE_KMT_BIT_NV = 0x8
    VK_EXTERNAL_MEMORY_HANDLE_TYPE_FLAG_BITS_MAX_ENUM_NV = 0x7FFFFFFF

class VkPipelineCompilerControlFlagBitsAMD(IntEnum):
    VK_PIPELINE_COMPILER_CONTROL_FLAG_BITS_MAX_ENUM_AMD = 0x7FFFFFFF

class VkVideoDecodeUsageFlagBitsKHR(IntEnum):
    VK_VIDEO_DECODE_USAGE_DEFAULT_KHR = 0x0
    VK_VIDEO_DECODE_USAGE_TRANSCODING_BIT_KHR = 0x1
    VK_VIDEO_DECODE_USAGE_OFFLINE_BIT_KHR = 0x2
    VK_VIDEO_DECODE_USAGE_STREAMING_BIT_KHR = 0x4
    VK_VIDEO_DECODE_USAGE_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkAccelerationStructureCreateFlagBitsKHR(IntEnum):
    VK_ACCELERATION_STRUCTURE_CREATE_DEVICE_ADDRESS_CAPTURE_REPLAY_BIT_KHR = 0x1
    VK_ACCELERATION_STRUCTURE_CREATE_MOTION_BIT_NV = 0x4
    VK_ACCELERATION_STRUCTURE_CREATE_DESCRIPTOR_BUFFER_CAPTURE_REPLAY_BIT_EXT = 0x8
    VK_ACCELERATION_STRUCTURE_CREATE_FLAG_BITS_MAX_ENUM_KHR = 0x7FFFFFFF

class VkInstanceCreateFlagBits(IntEnum):
    VK_INSTANCE_CREATE_ENUMERATE_PORTABILITY_BIT_KHR = 0x1
    VK_INSTANCE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class StdVideoH264DisableDeblockingFilterIdc(IntEnum):
    STD_VIDEO_H264_DISABLE_DEBLOCKING_FILTER_IDC_DISABLED = 0x0
    STD_VIDEO_H264_DISABLE_DEBLOCKING_FILTER_IDC_ENABLED = 0x1
    STD_VIDEO_H264_DISABLE_DEBLOCKING_FILTER_IDC_PARTIAL = 0x2
    STD_VIDEO_H264_DISABLE_DEBLOCKING_FILTER_IDC_INVALID = 0x7FFFFFFF

class VkRasterizationOrderAMD(IntEnum):
    VK_RASTERIZATION_ORDER_STRICT_AMD = 0x0
    VK_RASTERIZATION_ORDER_RELAXED_AMD = 0x1
    VK_RASTERIZATION_ORDER_MAX_ENUM_AMD = 0x7FFFFFFF

class VkAttachmentDescriptionFlagBits(IntEnum):
    VK_ATTACHMENT_DESCRIPTION_MAY_ALIAS_BIT = 0x1
    VK_ATTACHMENT_DESCRIPTION_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkFenceCreateFlagBits(IntEnum):
    VK_FENCE_CREATE_SIGNALED_BIT = 0x1
    VK_FENCE_CREATE_FLAG_BITS_MAX_ENUM = 0x7FFFFFFF

class VkQueryResultStatusKHR(IntEnum):
    VK_QUERY_RESULT_STATUS_NOT_READY_KHR = 0x0
    VK_QUERY_RESULT_STATUS_COMPLETE_KHR = 0x1
    VK_QUERY_RESULT_STATUS_MAX_ENUM_KHR = 0x7FFFFFFF
    VK_QUERY_RESULT_STATUS_ERROR_KHR = 0xFFFFFFFF

class eMissileState(IntEnum):
    EMissileState_Ready = 0x0
    EMissileState_LockingOn = 0x1
    EMissileState_Locked = 0x2
    EMissileState_Reloading = 0x3

class AkConvolutionReverbMediaType(IntEnum):
    AkConvoReverbMedia_Original = 0x0
    AkConvoReverbMedia_Acm = 0x1
    AkConvoReverbMedia_HalfPrecision = 0x2
    AkConvoReverbMedia_Xdsp = 0x3

class AkConvolutionAlgoType(IntEnum):
    AKCONVALGOTYPE_DOWNMIX = 0x0
    AKCONVALGOTYPE_DIRECT = 0x1

class AkFilterType(IntEnum):
    AKFILTERTYPE_LOWSHELF = 0x0
    AKFILTERTYPE_PEAKINGEQ = 0x1
    AKFILTERTYPE_HIGHSHELF = 0x2
    AKFILTERTYPE_LOWPASS = 0x3
    AKFILTERTYPE_HIGHPASS = 0x4
    AKFILTERTYPE_BANDPASS = 0x5
    AKFILTERTYPE_NOTCH = 0x6

class AkDistortionType(IntEnum):
    AKDISTORTIONTYPE_NONE = 0x0
    AKDISTORTIONTYPE_OVERDRIVE = 0x1
    AKDISTORTIONTYPE_HEAVY = 0x2
    AKDISTORTIONTYPE_FUZZ = 0x3
    AKDISTORTIONTYPE_CLIP = 0x4

class AkHarmonizerID(IntEnum):
    AKHARMONIZERPARAMID_INPUT = 0x0
    AKHARMONIZERPARAMID_PROCESSLFE = 0x1
    AKHARMONIZERPARAMID_SYNCDRY = 0x2
    AKHARMONIZERPARAMID_DRYLEVEL = 0x3
    AKHARMONIZERPARAMID_WETLEVEL = 0x4
    AKHARMONIZERPARAMID_WINDOWSIZE = 0x5
    AKHARMONIZERPARAMID_VOICE1ENABLE = 0x6
    AKHARMONIZERPARAMID_VOICE1PITCH = 0x7
    AKHARMONIZERPARAMID_VOICE1GAIN = 0x8
    AKHARMONIZERPARAMID_VOICE1FILTERTYPE = 0x9
    AKHARMONIZERPARAMID_VOICE1FILTERGAIN = 0xA
    AKHARMONIZERPARAMID_VOICE1FILTERFREQUENCY = 0xB
    AKHARMONIZERPARAMID_VOICE1FILTERQFACTOR = 0xC
    AKHARMONIZERPARAMID_VOICE2ENABLE = 0xD
    AKHARMONIZERPARAMID_VOICE2PITCH = 0xE
    AKHARMONIZERPARAMID_VOICE2GAIN = 0xF
    AKHARMONIZERPARAMID_VOICE2FILTERTYPE = 0x10
    AKHARMONIZERPARAMID_VOICE2FILTERGAIN = 0x11
    AKHARMONIZERPARAMID_VOICE2FILTERFREQUENCY = 0x12
    AKHARMONIZERPARAMID_VOICE2FILTERQFACTOR = 0x13
    AKHARMONIZERPARAMID_NUM_PARAMS = 0x14

class AkInputType(IntEnum):
    AKINPUTTYPE_ASINPUT = 0x0
    AKINPUTTYPE_CENTER = 0x1
    AKINPUTTYPE_STEREO = 0x2
    AKINPUTCHANNELTYPE_3POINT0 = 0x3
    AKINPUTTYPE_4POINT0 = 0x4
    AKINPUTTYPE_5POINT0 = 0x5
    AKINPUTTYPE_LEFTONLY = 0x6

class AkDelayLengthsMode(IntEnum):
    AKDELAYLENGTHSMODE_DEFAULT = 0x0
    AKDELAYLENGTHSMODE_CUSTOM = 0x1

class ListAllocFlag(IntEnum):
    AkFixedSize = 0x0
    AkAllocAndFree = 0x1
    AkAllocAndKeep = 0x2

class AkRTPC_ParameterID(IntEnum):
    RTPC_ADDITIVE_PARAMS_START = 0x0
    RTPC_LFE = 0x1
    RTPC_Pitch = 0x2
    RTPC_LPF = 0x3
    RTPC_HPF = 0x4
    RTPC_BusVolume = 0x5
    RTPC_InitialDelay = 0x6
    RTPC_MakeUpGain = 0x7
    Deprecated_RTPC_FeedbackVolume = 0x8
    Deprecated_RTPC_FeedbackLowpass = 0x9
    Deprecated_RTPC_FeedbackPitch = 0xA
    RTPC_MidiTransposition = 0xB
    RTPC_MidiVelocityOffset = 0xC
    RTPC_PlaybackSpeed = 0xD
    RTPC_MuteRatio = 0xE
    RTPC_PlayMechanismSpecialTransitionsValue = 0xF
    RTPC_MaxNumInstances = 0x10
    RTPC_OVERRIDABLE_PARAMS_START = 0x11
    RTPC_Position_PAN_X_2D = 0x12
    RTPC_Position_PAN_Y_2D = 0x13
    RTPC_Position_PAN_X_3D = 0x14
    RTPC_Position_PAN_Y_3D = 0x15
    RTPC_Position_PAN_Z_3D = 0x16
    RTPC_PositioningTypeBlend = 0x17
    RTPC_Positioning_Divergence_Center_PCT = 0x18
    RTPC_Positioning_Cone_Attenuation_ON_OFF = 0x19
    RTPC_Positioning_Cone_Attenuation = 0x1A
    RTPC_Positioning_Cone_LPF = 0x1B
    RTPC_Positioning_Cone_HPF = 0x1C
    RTPC_BypassFX0 = 0x1D
    RTPC_BypassFX1 = 0x1E
    RTPC_BypassFX2 = 0x1F
    RTPC_BypassFX3 = 0x20
    RTPC_BypassAllFX = 0x21
    RTPC_HDRBusThreshold = 0x22
    RTPC_HDRBusReleaseTime = 0x23
    RTPC_HDRBusRatio = 0x24
    RTPC_HDRActiveRange = 0x25
    RTPC_GameAuxSendVolume = 0x26
    RTPC_UserAuxSendVolume0 = 0x27
    RTPC_UserAuxSendVolume1 = 0x28
    RTPC_UserAuxSendVolume2 = 0x29
    RTPC_UserAuxSendVolume3 = 0x2A
    RTPC_OutputBusVolume = 0x2B
    RTPC_OutputBusHPF = 0x2C
    RTPC_OutputBusLPF = 0x2D
    RTPC_Positioning_EnableAttenuation = 0x2E
    RTPC_ReflectionsVolume = 0x2F
    RTPC_UserAuxSendLPF0 = 0x30
    RTPC_UserAuxSendLPF1 = 0x31
    RTPC_UserAuxSendLPF2 = 0x32
    RTPC_UserAuxSendLPF3 = 0x33
    RTPC_UserAuxSendHPF0 = 0x34
    RTPC_UserAuxSendHPF1 = 0x35
    RTPC_UserAuxSendHPF2 = 0x36
    RTPC_UserAuxSendHPF3 = 0x37
    RTPC_GameAuxSendLPF = 0x38
    RTPC_GameAuxSendHPF = 0x39
    RTPC_Position_PAN_Z_2D = 0x3A
    RTPC_BypassAllMetadata = 0x3B
    RTPC_MaxNumRTPC = 0x3C

class AkAssignableMidiControl(IntEnum):
    AssignableMidiControl_Invalid = 0x0
    AssignableMidiControl_Start = 0x1
    AssignableMidiControl_CcEnd = 0x81
    AssignableMidiControl_Aftertouch = 0x82
    AssignableMidiControl_PitchBend = 0x83
    AssignableMidiControl_Frequency = 0x84
    AssignableMidiControl_Note = 0x85
    AssignableMidiControl_Max = 0x86

class AkBuiltInParam(IntEnum):
    BuiltInParam_None = 0x0
    BuiltInParam_Start = 0x1
    BuiltInParam_Azimuth = 0x2
    BuiltInParam_Elevation = 0x3
    BuiltInParam_EmitterCone = 0x4
    BuiltInParam_Obsruction = 0x5
    BuiltInParam_Occlusion = 0x6
    BuiltInParam_ListenerCone = 0x7
    BuiltInParam_Diffraction = 0x8
    BuiltInParam_TransmissionLoss = 0x9
    BuiltInParam_Max = 0xA

class AkModulatorPropID(IntEnum):
    AkModulatorPropID_Scope = 0x0
    AkModulatorPropID_Envelope_StopPlayback = 0x1
    AkModulatorPropID_Lfo_Depth = 0x2
    AkModulatorPropID_Lfo_Attack = 0x3
    AkModulatorPropID_Lfo_Frequency = 0x4
    AkModulatorPropID_Lfo_Waveform = 0x5
    AkModulatorPropID_Lfo_Smoothing = 0x6
    AkModulatorPropID_Lfo_PWM = 0x7
    AkModulatorPropID_Lfo_InitialPhase = 0x8
    AkModulatorPropID_Envelope_AttackTime = 0x9
    AkModulatorPropID_Envelope_AttackCurve = 0xA
    AkModulatorPropID_Envelope_DecayTime = 0xB
    AkModulatorPropID_Envelope_SustainLevel = 0xC
    AkModulatorPropID_Envelope_SustainTime = 0xD
    AkModulatorPropID_Envelope_ReleaseTime = 0xE
    AkModulatorPropID_Envelope_TriggerOn = 0xF
    AkModulatorPropID_Time_Duration = 0x10
    AkModulatorPropID_Time_Loops = 0x11
    AkModulatorPropID_Time_PlaybackRate = 0x12
    AkModulatorPropID_Time_InitialDelay = 0x13
    AkModulatorPropID_NUM = 0x14

class AkRTPC_ModulatorParamID(IntEnum):
    RTPC_ModulatorRTPCIDStart = 0x0
    RTPC_ModulatorLfoAttack = 0x1
    RTPC_ModulatorLfoFrequency = 0x2
    RTPC_ModulatorLfoWaveform = 0x3
    RTPC_ModulatorLfoSmoothing = 0x4
    RTPC_ModulatorLfoPWM = 0x5
    RTPC_ModulatorLfoInitialPhase = 0x6
    RTPC_ModulatorLfoRetrigger = 0x7
    RTPC_ModulatorEnvelopeAttackTime = 0x8
    RTPC_ModulatorEnvelopeAttackCurve = 0x9
    RTPC_ModulatorEnvelopeDecayTime = 0xA
    RTPC_ModulatorEnvelopeSustainLevel = 0xB
    RTPC_ModulatorEnvelopeSustainTime = 0xC
    RTPC_ModulatorEnvelopeReleaseTime = 0xD
    RTPC_ModulatorTimePlaybackSpeed = 0xE
    RTPC_ModulatorTimeInitialDelay = 0xF
    RTPC_MaxNumModulatorRTPC = 0x10

class AkSyncType(IntEnum):
    SyncTypeImmediate = 0x0
    SyncTypeNextGrid = 0x1
    SyncTypeNextBar = 0x2
    SyncTypeNextBeat = 0x3
    SyncTypeNextMarker = 0x4
    SyncTypeNextUserMarker = 0x5
    SyncTypeEntryMarker = 0x6
    SyncTypeExitMarker = 0x7
    SyncTypeExitNever = 0x8
    SyncTypeLastExitPosition = 0x9

class AkPropID(IntEnum):
    AkPropID_Volume = 0x0
    AkPropID_LFE = 0x1
    AkPropID_Pitch = 0x2
    AkPropID_LPF = 0x3
    AkPropID_HPF = 0x4
    AkPropID_BusVolume = 0x5
    AkPropID_MakeUpGain = 0x6
    AkPropID_StatePropNum = 0x7
    AkPropID_PriorityDistanceOffset = 0x8
    AkPropID_MuteRatio = 0xB
    AkPropID_PAN_LR = 0xC
    AkPropID_PAN_FR = 0xD
    AkPropID_CenterPCT = 0xE
    AkPropID_DelayTime = 0xF
    AkPropID_TransitionTime = 0x10
    AkPropID_Probability = 0x11
    AkPropID_DialogueMode = 0x12
    AkPropID_UserAuxSendVolume0 = 0x13
    AkPropID_UserAuxSendVolume1 = 0x14
    AkPropID_UserAuxSendVolume2 = 0x15
    AkPropID_UserAuxSendVolume3 = 0x16
    AkPropID_GameAuxSendVolume = 0x17
    AkPropID_OutputBusVolume = 0x18
    AkPropID_OutputBusHPF = 0x19
    AkPropID_OutputBusLPF = 0x1A
    AkPropID_HDRBusThreshold = 0x1B
    AkPropID_HDRBusRatio = 0x1C
    AkPropID_HDRBusReleaseTime = 0x1D
    AkPropID_HDRBusGameParam = 0x1E
    AkPropID_HDRBusGameParamMin = 0x1F
    AkPropID_HDRBusGameParamMax = 0x20
    AkPropID_HDRActiveRange = 0x21
    AkPropID_LoopStart = 0x22
    AkPropID_LoopEnd = 0x23
    AkPropID_TrimInTime = 0x24
    AkPropID_TrimOutTime = 0x25
    AkPropID_FadeInTime = 0x26
    AkPropID_FadeOutTime = 0x27
    AkPropID_FadeInCurve = 0x28
    AkPropID_FadeOutCurve = 0x29
    AkPropID_LoopCrossfadeDuration = 0x2A
    AkPropID_CrossfadeUpCurve = 0x2B
    AkPropID_CrossfadeDownCurve = 0x2C
    AkPropID_MidiTrackingRootNote = 0x2D
    AkPropID_MidiPlayOnNoteType = 0x2E
    AkPropID_MidiTransposition = 0x2F
    AkPropID_MidiVelocityOffset = 0x30
    AkPropID_MidiKeyRangeMin = 0x31
    AkPropID_MidiKeyRangeMax = 0x32
    AkPropID_MidiVelocityRangeMin = 0x33
    AkPropID_MidiVelocityRangeMax = 0x34
    AkPropID_MidiChannelMask = 0x35
    AkPropID_PlaybackSpeed = 0x36
    AkPropID_MidiTempoSource = 0x37
    AkPropID_MidiTargetNode = 0x38
    AkPropID_AttachedPluginFXID = 0x39
    AkPropID_Loop = 0x3A
    AkPropID_InitialDelay = 0x3B
    AkPropID_UserAuxSendLPF0 = 0x3C
    AkPropID_UserAuxSendLPF1 = 0x3D
    AkPropID_UserAuxSendLPF2 = 0x3E
    AkPropID_UserAuxSendLPF3 = 0x3F
    AkPropID_UserAuxSendHPF0 = 0x40
    AkPropID_UserAuxSendHPF1 = 0x41
    AkPropID_UserAuxSendHPF2 = 0x42
    AkPropID_UserAuxSendHPF3 = 0x43
    AkPropID_GameAuxSendLPF = 0x44
    AkPropID_GameAuxSendHPF = 0x45
    AkPropID_AttenuationID = 0x46
    AkPropID_PositioningTypeBlend = 0x47
    AkPropID_ReflectionBusVolume = 0x48
    AkPropID_PAN_UD = 0x49
    AkPropID_NUM = 0x4A

class AkMeterMode(IntEnum):
    AkMeterMode_Peak = 0x0
    AkMeterMode_RMS = 0x1

class AkMeterScope(IntEnum):
    AkMeterScope_Global = 0x0
    AkMeterScope_GameObject = 0x1

class AkBufferPosInformation(ctypes.Structure):
    _fields_ = [
        ("uStartPos", ctypes.c_uint32),
        ("fLastRate", ctypes.c_float),
        ("uFileEnd", ctypes.c_uint32),
        ("uSampleRate", ctypes.c_uint32),
    ]
    uStartPos: int
    fLastRate: float
    uFileEnd: int
    uSampleRate: int

class AkModulatorScope(IntEnum):
    AkModulatorScope_Voice = 0x0
    AkModulatorScope_Note = 0x1
    AkModulatorScope_GameObject = 0x2
    AkModulatorScope_Global = 0x3

class AkPBIStopMode(IntEnum):
    AkPBIStopMode_Normal = 0x0
    AkPBIStopMode_StopAndContinue = 0x1
    AkPBIStopMode_StopAndContinueSequel = 0x2

class AkValueMeaning(IntEnum):
    AkValueMeaning_Default = 0x0
    AkValueMeaning_Independent = 0x1
    AkValueMeaning_Offset = 0x2

class AkLimiterKey(ctypes.Structure):
    class KeyInfoSize(IntEnum):
        KeyInfoSize_LimiterType = 0x3
        KeyInfoSize_Depth = 0x1D
        KeyInfoSize_ShortID = 0x20

    class LimiterType(IntEnum):
        LimiterType_Global = 0x0
        LimiterType_Bus = 0x1
        LimiterType_Default = 0x2

    _fields_ = [
        ("key", ctypes.c_uint64),
    ]
    key: int

class AttenuationCurveID(IntEnum):
    AttenuationCurveID_VolumeDry = 0x0
    AttenuationCurveID_VolumeAuxGameDef = 0x1
    AttenuationCurveID_VolumeAuxUserDef = 0x2
    AttenuationCurveID_LowPassFilter = 0x3
    AttenuationCurveID_HighPassFilter = 0x4
    AttenuationCurveID_Spread = 0x5
    AttenuationCurveID_Focus = 0x6
    AttenuationCurveID_MaxCount = 0x7
    AttenuationCurveID_None = 0xFF

class AkCurveScaling(IntEnum):
    AkCurveScaling_None = 0x0
    AkCurveScaling_dB = 0x2
    AkCurveScaling_Log = 0x3
    AkCurveScaling_dBToLin = 0x4
    AkCurveScaling_MaxNum = 0x8

class AkRtpcAccum(IntEnum):
    AkRtpcAccum_None = 0x0
    AkRtpcAccum_Exclusive = 0x1
    AkRtpcAccum_Additive = 0x2
    AkRtpcAccum_Multiply = 0x3
    AkRtpcAccum_Boolean = 0x4
    AkRtpcAccum_MaxNum = 0x8

class AkNodeCategory(IntEnum):
    AkNodeCategory_Bus = 0x0
    AkNodeCategory_ActorMixer = 0x1
    AkNodeCategory_RanSeqCntr = 0x2
    AkNodeCategory_Sound = 0x3
    AkNodeCategory_SwitchCntr = 0x4
    AkNodeCategory_LayerCntr = 0x5
    AkNodeCategory_MusicTrack = 0x6
    AkNodeCategory_MusicSegment = 0x7
    AkNodeCategory_MusicRanSeqCntr = 0x8
    AkNodeCategory_MusicSwitchCntr = 0x9
    AkNodeCategory_AuxBus = 0xA
    AkNodeCategory_None = 0x3E8

class AkPathStepOrContinuous(IntEnum):
    AkPathStep = 0x0
    AkPathContinuous = 0x2

class AkPathSequenceOrRandom(IntEnum):
    AkPathSequence = 0x0
    AkPathRandom = 0x1

class AkTransitionRampingType(IntEnum):
    AkTransitionRampingType_None = 0x0
    AkTransitionRampingType_SlewRate = 0x1
    AkTransitionRampingType_FilteringOverTime = 0x2

class AkSrcType(IntEnum):
    SrcTypeNone = 0x0
    SrcTypeFile = 0x1
    SrcTypeModelled = 0x2
    SrcTypeMemory = 0x3

class AkVirtualQueueBehavior(IntEnum):
    AkVirtualQueueBehavior_FromBeginning = 0x0
    AkVirtualQueueBehavior_FromElapsedTime = 0x1
    AkVirtualQueueBehavior_Resume = 0x2

class AkBelowThresholdBehavior(IntEnum):
    AkBelowThresholdBehavior_ContinueToPlay = 0x0
    AkBelowThresholdBehavior_KillVoice = 0x1
    AkBelowThresholdBehavior_SetAsVirtualVoice = 0x2
    AkBelowThresholdBehavior_KillIfOneShotElseVirtual = 0x3

class AkDeltaType(IntEnum):
    AkDelta_None = 0x0
    AkDelta_Pipeline = 0x1
    AkDelta_EmitterListener = 0x2
    AkDelta_MultiUpdate = 0x3
    AkDelta_MultiUnset = 0x4
    AkDelta_CachePrecomputedParams = 0x5
    AkDelta_BaseValue = 0x6
    AkDelta_RTPC = 0x7
    AkDelta_Switch = 0x8
    AkDelta_State = 0x9
    AkDelta_Event = 0xA
    AkDelta_Mute = 0xB
    AkDelta_RangeMod = 0xC
    AkDelta_Ducking = 0xD
    AkDelta_AuxUserDef = 0xE
    AkDelta_Distance = 0xF
    AkDelta_Cone = 0x10
    AkDelta_Occlusion = 0x11
    AkDelta_TransmissionLoss = 0x12
    AkDelta_Obstruction = 0x13
    AkDelta_Diffraction = 0x14
    AkDelta_HDRGain = 0x15
    AkDelta_MusicSegmentEnvelope = 0x16
    AkDelta_Modulator = 0x17
    AkDelta_BGMMute = 0x18
    AkDelta_Fade = 0x19
    AkDelta_Pause = 0x1A
    AkDelta_Rays = 0x1B
    AkDelta_RaysFilter = 0x1C
    AkDelta_LiveEdit = 0x1D
    AkDelta_UsePrecomputedParams = 0x1E
    AkDelta_SetGameObjectAuxSendValues = 0x1F
    AkDelta_RoomSend = 0x20
    AkDelta_Normalization = 0x21
    AkDelta_SetGameObjectOutputBusVolume = 0x22
    AkDelta_SetEarlyReflectionsVolume = 0x23
    AkDelta_GameObjectDestroyed = 0x24
    AkDelta_RTPCGlobal = 0x25
    AkDelta_ModulatorGlobal = 0x26
    AkDelta_Count = 0x27

class AkGameObjComponentIdx(IntEnum):
    GameObjComponentIdx_Emitter = 0x0
    GameObjComponentIdx_ConnectedListeners = 0x1
    GameObjComponentIdx_VolumeRay = 0x2
    GameObjComponentIdx_Listener = 0x3
    GameObjComponentIdx_ModifiedNodes = 0x4
    GameObjComponentIdx_SwitchHistory = 0x5
    GameObjComponentIdx_SpatialAudioObj = 0x6
    GameObjComponentIdx_SpatialAudioEmitter = 0x7
    GameObjComponentIdx_SpatialAudioListener = 0x8
    GameObjComponentIdx_SpatialAudioRoom = 0x9
    GameObjComponent_NumIdxs = 0xA

class AkMusicActionType(IntEnum):
    MusicActionTypePlay = 0x0
    MusicActionTypeStop = 0x1
    MusicActionTypeNotif = 0x2
    MusicActionTypePostEvent = 0x3

class SharedSetOverride(IntEnum):
    SharedSetOverride_Bank = 0x0
    SharedSetOverride_Proxy = 0x1
    SharedSetOverride_SDK = 0x2

class PluginPropertyUpdateType(IntEnum):
    PluginPropertyUpdateType_FxBase = 0x0
    PluginPropertyUpdateType_Rtpc = 0x1
    PluginPropertyUpdateType_State = 0x2

class AkCtxDestroyReason(IntEnum):
    CtxDestroyReasonFinished = 0x0
    CtxDestroyReasonPlayFailed = 0x1

class ActionParamType(IntEnum):
    ActionParamType_Stop = 0x0
    ActionParamType_Pause = 0x1
    ActionParamType_Resume = 0x2
    ActionParamType_Break = 0x3
    ActionParamType_Seek = 0x4
    ActionParamType_Release = 0x5

class AkAssociatedActionType(IntEnum):
    AssocActionTypeStinger = 0x0
    AssocActionTypeState = 0x1

class AkMusicTrackType(IntEnum):
    AkMusicTrackType_Normal = 0x0
    AkMusicTrackType_Random = 0x1
    AkMusicTrackType_Sequence = 0x2
    AkMusicTrackType_Switch = 0x3

class AkEntryType(IntEnum):
    EntryTypeEntryMarker = 0x0
    EntryTypeSameTime = 0x1
    EntryTypeRandomMarker = 0x2
    EntryTypeRandomUserMarker = 0x3
    EntryTypeLastExitTime = 0x4

class AkPathMode(IntEnum):
    AkStepSequence = 0x0
    AkStepRandom = 0x1
    AkContinuousSequence = 0x2
    AkContinuousRandom = 0x3

class AkObjectCategory(IntEnum):
    ObjCategory_Undefined = 0x0
    ObjCategory_ActorMixer = 0x1
    ObjCategory_Container = 0x2
    ObjCategory_Sound = 0x4
    ObjCategory_State = 0x5
    ObjCategory_Action = 0x6
    ObjCategory_Event = 0x7
    ObjCategory_Track = 0x8
    ObjCategory_MusicNode = 0x9
    ObjCategory_FeedbackNode = 0xA
    ObjCategory_Bus = 0xB

class AkMidiTempoSource(IntEnum):
    AkMidiTempoSource_Hierarchy = 0x0
    AkMidiTempoSource_File = 0x1

class AkCtxState(IntEnum):
    CtxStateStop = 0x0
    CtxStatePause = 0x1
    CtxStateResume = 0x2
    CtxStatePlay = 0x3
    CtxStateToDestroy = 0x4

class AkMusicOverride(IntEnum):
    AkMusicOverride_ClockSettings = 0x0
    AkMusicOverride_MidiTarget = 0x1
    AkMusicOverride_MidiTempo = 0x2

class AkMusicFlag(IntEnum):
    AkMusicFlag_MidiTargetIsBus = 0x0

class KickFrom(IntEnum):
    KickFrom_Stopped = 0x0
    KickFrom_OverNodeLimit = 0x1
    KickFrom_OverGlobalLimit = 0x2

class AkCtxVirtualHandlingResult(IntEnum):
    VirtualHandling_NotHandled = 0x0
    VirtualHandling_RequiresSeeking = 0x1
    VirtualHandling_ShouldStop = 0x2

class AkPitchShiftType(IntEnum):
    PitchShiftType_LinearInterpolate = 0x0
    PitchShiftType_LinearConstant = 0x1

class AkBankLoadFlag(IntEnum):
    AkBankLoadFlag_None = 0x0
    AkBankLoadFlag_UsingFileID = 0x1
    AkBankLoadFlag_FromMemoryInPlace = 0x2
    AkBankLoadFlag_FromMemoryOutOfPlace = 0x3

class AkRtpcType(IntEnum):
    AkRtpcType_GameParameter = 0x0
    AkRtpcType_MIDIParameter = 0x1
    AkRtpcType_Modulator = 0x2
    AkRtpcType_MaxNum = 0x8

class cGcCreatureMoveSpeedRequest(IntEnum):
    ECreatureMoveSpeed_Invalid = 0x0
    ECreatureMoveSpeed_TurnInPlace = 0x1
    ECreatureMoveSpeed_Normal = 0x2
    ECreatureMoveSpeed_Fast = 0x3

class eNavigationMode(IntEnum):
    ENavigationMode_Invalid = 0x0
    ENavigationMode_Ground = 0x1
    ENavigationMode_Fish = 0x2
    ENavigationMode_Floater = 0x3
    ENavigationMode_WallClimber = 0x4
    ENavigationMode_Bird = 0x5
    ENavigationMode_Robot = 0x6
    ENavigationMode_GroundIndoor = 0x7

class eSteeringMovement(IntEnum):
    ESteeringMovement_None = 0x0
    ESteeringMovement_FollowAttachment = 0x1
    ESteeringMovement_Position = 0x2
    ESteeringMovement_Direction = 0x3
    ESteeringMovement_Facing = 0x4

class cGcCreatureNavTrail(ctypes.Structure):
    def Reset(): pass

    def PushElt(): pass

    def AdjustFootPos(): pass

    def AdjustCoGPos(): pass

    def GetHipAngle(): pass

    def DebugDraw(): pass

    _fields_ = [
        ("mTrailElts", common.Vector3f * 0x40),  # cTkVector3[64]
        ("mvOffset", common.Vector3f),
        ("mvFrontFootPos", common.Vector3f),
        ("miNumElts", ctypes.c_int32),
        ("miIntervalCounter", ctypes.c_int32),
        ("mfScale", ctypes.c_float),
        ("mfFade", ctypes.c_float),
        ("mfFadeTarget", ctypes.c_float),
    ]
    mTrailElts: list[common.Vector3f]
    mvOffset: common.Vector3f
    mvFrontFootPos: common.Vector3f
    miNumElts: int
    miIntervalCounter: int
    mfScale: float
    mfFade: float
    mfFadeTarget: float

class AkSetType(IntEnum):
    SetType_Inclusion = 0x0
    SetType_Exclusion = 0x1

class MidiEventActionType(IntEnum):
    MidiEventActionType_NoAction = 0x0
    MidiEventActionType_Play = 0x1
    MidiEventActionType_Break = 0x2
    MidiEventActionType_Stop = 0x3

class AkActionType(IntEnum):
    AkActionType_None = 0x0
    AkActionType_Stop_E = 0x102
    AkActionType_Stop_E_O = 0x103
    AkActionType_Stop_ALL = 0x104
    AkActionType_Stop_ALL_O = 0x105
    AkActionType_Stop_AE = 0x108
    AkActionType_Stop_AE_O = 0x109
    AkActionType_Pause_E = 0x202
    AkActionType_Pause_E_O = 0x203
    AkActionType_Pause_ALL = 0x204
    AkActionType_Pause_ALL_O = 0x205
    AkActionType_Pause_AE = 0x208
    AkActionType_Pause_AE_O = 0x209
    AkActionType_Resume_E = 0x302
    AkActionType_Resume_E_O = 0x303
    AkActionType_Resume_ALL = 0x304
    AkActionType_Resume_ALL_O = 0x305
    AkActionType_Resume_AE = 0x308
    AkActionType_Resume_AE_O = 0x309
    AkActionType_Play = 0x403
    AkActionType_PlayAndContinue = 0x503
    AkActionType_Mute_M = 0x602
    AkActionType_Mute_O = 0x603
    AkActionType_Unmute_M = 0x702
    AkActionType_Unmute_O = 0x703
    AkActionType_Unmute_ALL = 0x704
    AkActionType_Unmute_ALL_O = 0x705
    AkActionType_Unmute_AE = 0x708
    AkActionType_Unmute_AE_O = 0x709
    AkActionType_SetPitch_M = 0x802
    AkActionType_SetPitch_O = 0x803
    AkActionType_ResetPitch_M = 0x902
    AkActionType_ResetPitch_O = 0x903
    AkActionType_ResetPitch_ALL = 0x904
    AkActionType_ResetPitch_ALL_O = 0x905
    AkActionType_ResetPitch_AE = 0x908
    AkActionType_ResetPitch_AE_O = 0x909
    AkActionType_SetVolume_M = 0xA02
    AkActionType_SetVolume_O = 0xA03
    AkActionType_ResetVolume_M = 0xB02
    AkActionType_ResetVolume_O = 0xB03
    AkActionType_ResetVolume_ALL = 0xB04
    AkActionType_ResetVolume_ALL_O = 0xB05
    AkActionType_ResetVolume_AE = 0xB08
    AkActionType_ResetVolume_AE_O = 0xB09
    AkActionType_SetBusVolume_M = 0xC02
    AkActionType_SetBusVolume_O = 0xC03
    AkActionType_ResetBusVolume_M = 0xD02
    AkActionType_ResetBusVolume_O = 0xD03
    AkActionType_ResetBusVolume_ALL = 0xD04
    AkActionType_ResetBusVolume_AE = 0xD08
    AkActionType_SetLPF_M = 0xE02
    AkActionType_SetLPF_O = 0xE03
    AkActionType_ResetLPF_M = 0xF02
    AkActionType_ResetLPF_O = 0xF03
    AkActionType_ResetLPF_ALL = 0xF04
    AkActionType_ResetLPF_ALL_O = 0xF05
    AkActionType_ResetLPF_AE = 0xF08
    AkActionType_ResetLPF_AE_O = 0xF09
    AkActionType_UseState_E = 0x1002
    AkActionType_UnuseState_E = 0x1102
    AkActionType_SetState = 0x1204
    AkActionType_SetGameParameter = 0x1302
    AkActionType_SetGameParameter_O = 0x1303
    AkActionType_ResetGameParameter = 0x1402
    AkActionType_ResetGameParameter_O = 0x1403
    AkActionType_StopEvent = 0x1511
    AkActionType_PauseEvent = 0x1611
    AkActionType_ResumeEvent = 0x1711
    AkActionType_Duck = 0x1820
    AkActionType_SetSwitch = 0x1901
    AkActionType_BypassFX_M = 0x1A02
    AkActionType_BypassFX_O = 0x1A03
    AkActionType_ResetBypassFX_M = 0x1B02
    AkActionType_ResetBypassFX_O = 0x1B03
    AkActionType_ResetBypassFX_ALL = 0x1B04
    AkActionType_ResetBypassFX_ALL_O = 0x1B05
    AkActionType_ResetBypassFX_AE = 0x1B08
    AkActionType_ResetBypassFX_AE_O = 0x1B09
    AkActionType_Break_E = 0x1C02
    AkActionType_Break_E_O = 0x1C03
    AkActionType_Trigger = 0x1D00
    AkActionType_Trigger_O = 0x1D01
    AkActionType_Seek_E = 0x1E02
    AkActionType_Seek_E_O = 0x1E03
    AkActionType_Seek_ALL = 0x1E04
    AkActionType_Seek_ALL_O = 0x1E05
    AkActionType_Seek_AE = 0x1E08
    AkActionType_Seek_AE_O = 0x1E09
    AkActionType_Release = 0x1F02
    AkActionType_Release_O = 0x1F03
    AkActionType_SetHPF_M = 0x2002
    AkActionType_SetHPF_O = 0x2003
    AkActionType_PlayEvent = 0x2103
    AkActionType_ResetPlaylist_E = 0x2202
    AkActionType_ResetPlaylist_E_O = 0x2203
    AkActionType_ResetHPF_M = 0x3002
    AkActionType_ResetHPF_O = 0x3003
    AkActionType_ResetHPF_ALL = 0x3004
    AkActionType_ResetHPF_ALL_O = 0x3005
    AkActionType_ResetHPF_AE = 0x3008
    AkActionType_ResetHPF_AE_O = 0x3009

class AkPlaybackState(IntEnum):
    PB_Playing = 0x0
    PB_Paused = 0x1

class AkClipAutomationType(IntEnum):
    AutomationType_Volume = 0x0
    AutomationType_LPF = 0x1
    AutomationType_HPF = 0x2
    AutomationType_FadeIn = 0x3
    AutomationType_FadeOut = 0x4

class MidiPlayOnNoteType(IntEnum):
    MidiPlayOnNoteType_NoteOn = 0x1

class AkListenerOp(IntEnum):
    AkListenerOp_Set = 0x0
    AkListenerOp_Add = 0x1
    AkListenerOp_Remove = 0x2

class AkMusicNode_OverrideParentBitPos(IntEnum):
    AkMusicNode_FlagBitPos_OverrideClockSettings = 0x0
    AkMusicNode_FlagBitPos_OverrideMidiTempo = 0x1
    AkMusicNode_FlagBitPos_OverrideMidiTarget = 0x2
    AkMusicNode_FlagBitPos_MidiTargetTypeBus = 0x3

class AkRandomMode(IntEnum):
    RandomMode_Normal = 0x0
    RandomMode_Shuffle = 0x1

class RSType(IntEnum):
    RSType_ContinuousSequence = 0x0
    RSType_StepSequence = 0x1
    RSType_ContinuousRandom = 0x2
    RSType_StepRandom = 0x3

class AkContainerMode(IntEnum):
    ContainerMode_Random = 0x0
    ContainerMode_Sequence = 0x1

class AkBandNumber(IntEnum):
    BAND1 = 0x0
    BAND2 = 0x1
    BAND3 = 0x2

class AkJumpToSelType(IntEnum):
    AkJumpToSelType_StartOfPlaylist = 0x0
    AkJumpToSelType_SpecificItem = 0x1
    AkJumpToSelType_LastPlayedSegment = 0x2
    AkJumpToSelType_NextSegment = 0x3

class AkPitchShifterParamID(IntEnum):
    AKPITCHSHIFTERPARAMID_INPUT = 0x0
    AKPITCHSHIFTERPARAMID_PROCESSLFE = 0x1
    AKPITCHSHIFTERPARAMID_SYNCDRY = 0x2
    AKPITCHSHIFTERPARAMID_DRYLEVEL = 0x3
    AKPITCHSHIFTERPARAMID_WETLEVEL = 0x4
    AKPITCHSHIFTERPARAMID_DELAYTIME = 0x5
    AKPITCHSHIFTERPARAMID_PITCH = 0x6
    AKPITCHSHIFTERPARAMID_FILTERTYPE = 0x7
    AKPITCHSHIFTERPARAMID_FILTERGAIN = 0x8
    AKPITCHSHIFTERPARAMID_FILTERFREQUENCY = 0x9
    AKPITCHSHIFTERPARAMID_FILTERQFACTOR = 0xA
    AKPITCHSHIFTERPARAMID_NUM_PARAMS = 0xB

class FilterInsertType(IntEnum):
    FILTERINSERTTYPE_OFF = 0x0
    FILTERINSERTTYPE_ERONLY = 0x1
    FILTERINSERTTYPE_REVERBONLY = 0x2
    FILTERINSERTTYPE_ERANDREVERB = 0x3

class FilterCurveType(IntEnum):
    FILTERCURVETYPE_LOWSHELF = 0x0
    FILTERCURVETYPE_PEAKING = 0x1
    FILTERCURVETYPE_HIGHSHELF = 0x2

class eEggMachineState(IntEnum):
    EEggMachineStatus_NoState = 0x0
    EEggMachineStatus_Waiting = 0x1
    EEggMachineStatus_InputsChanged = 0x2
    EEggMachineStatus_ValidEgg = 0x3
    EEggMachineStatus_ValidCatalyst = 0x4
    EEggMachineStatus_InvalidInputs = 0x5
    EEggMachineStatus_ReadyToModify = 0x6
    EEggMachineStatus_Modifying = 0x7
    EEggMachineStatus_ModificationDone = 0x8
    EEggMachineStatus_DisplayFinalProduct = 0x9

class eEggMachineInputs(IntEnum):
    EEggMachineInputs_Egg = 0x0
    EEggMachineInputs_Growth = 0x1
    EEggMachineInputs_Accessories = 0x2
    EEggMachineInputs_Colour = 0x3
    EEggMachineInputs_Traits = 0x4
    EEggMachineInputs_NumInputs = 0x5

class AkSrcProcessOrder(IntEnum):
    SrcProcessOrder_SwVoice = 0x0
    SrcProcessOrder_HwVoice = 0x1
    SrcProcessOrder_HwVoiceLowLatency = 0x2

class AkPathStepOnNewSound(IntEnum):
    AkPathStepNormal = 0x0
    AkPathStepNewSound = 0x4

class AkPositioning_ParameterID(IntEnum):
    POSID_Position_PAN_X_2D = 0x12
    POSID_Position_PAN_Y_2D = 0x13
    POSID_Position_PAN_X_3D = 0x14
    POSID_Position_PAN_Y_3D = 0x15
    POSID_Position_PAN_Z_3D = 0x16
    POSID_PositioningTypeBlend = 0x17
    POSID_Positioning_Divergence_Center_PCT = 0x18
    POSID_Positioning_Cone_Attenuation_ON_OFF = 0x19
    POSID_Positioning_Cone_Attenuation = 0x1A
    POSID_Positioning_Cone_LPF = 0x1B
    POSID_Positioning_Cone_HPF = 0x1C
    POSID_PositioningEnableAttenuation = 0x2E
    POSID_Position_PAN_Z_2D = 0x3A
    POSID_PanningType = 0x40
    POSID_IsPositionDynamic = 0x41
    POSID_IsLooping = 0x42
    POSID_Transition = 0x43
    POSID_PathMode = 0x44
    POSID_ListenerRelativeRouting = 0x45
    POSID_Attenuation = 0x46

class NVGcreateFlags(IntEnum):
    NVG_ANTIALIAS = 0x1

class NVGimageFlagsGL(IntEnum):
    NVG_IMAGE_NODELETE = 0x10000

class GLNVGuniformLoc(IntEnum):
    GLNVG_LOC_VIEWSIZE = 0x0
    GLNVG_LOC_TEX = 0x1
    GLNVG_LOC_FRAG = 0x2
    GLNVG_MAX_LOCS = 0x3

class GLNVGshaderType(IntEnum):
    NSVG_SHADER_FILLGRAD = 0x0
    NSVG_SHADER_FILLIMG = 0x1
    NSVG_SHADER_SIMPLE = 0x2
    NSVG_SHADER_IMG = 0x3
    NSVG_SHADER_FILLFLAT = 0x4

class GLNVGcallType(IntEnum):
    GLNVG_NONE = 0x0
    GLNVG_FILL = 0x1
    GLNVG_CONVEXFILL = 0x2
    GLNVG_STROKE = 0x3
    GLNVG_TRIANGLES = 0x4

class NVGimageFlags(IntEnum):
    NVG_IMAGE_GENERATE_MIPMAPS = 0x1

class NVGtexture(IntEnum):
    NVG_TEXTURE_ALPHA = 0x1
    NVG_TEXTURE_RGBA = 0x2
    NVG_TEXTURE_BGRA8 = 0x3
    NVG_TEXTURE_BGRX8 = 0x4

class ePlanetLabelState(IntEnum):
    EPlanetLabelState_None = 0x0
    EPlanetLabelState_Planet = 0x1
    EPlanetLabelState_PlanetTimingOut = 0x2

class ePounceState(IntEnum):
    EPounceState_NotPouncing = 0x0
    EPounceState_Pouncing = 0x1
    EPounceState_PounceHitNothing = 0x2
    EPounceState_PounceHitCreature = 0x3
    EPounceState_PounceHitPlayer = 0x4

class eItemPopupLayer(IntEnum):
    EItemPopupLayer_None = 0x0
    EItemPopupLayer_Charge = 0x1
    EItemPopupLayer_Build = 0x2
    EItemPopupLayer_Repair = 0x3
    EItemPopupLayer_Install = 0x4
    EItemPopupLayer_Substance = 0x5
    EItemPopupLayer_Refine = 0x6
    EItemPopupLayer_Wiki = 0x7
    EItemPopupLayer_NumTypes = 0x8

class eItemSlotTypes(IntEnum):
    EItemSlotType_Substance = 0x0
    EItemSlotType_Technology = 0x1
    EItemSlotType_Product = 0x2
    EItemSlotType_Word = 0x3
    EItemSlotType_NumTypes = 0x4

class eQuantityBarType(IntEnum):
    EQuantityBar_NoTick = 0x0
    EQuantityBar_Tick = 0x1
    EQuantityBar_Repaired = 0x2

class eTeleportOfflineReason(IntEnum):
    ETeleportOfflineReason_Pirates = 0x0
    ETeleportOfflineReason_NumTypes = 0x1

class AkRayCategory(IntEnum):
    AkRayCategory_Direct = 0x1

class AkQueuedMsgType(IntEnum):
    QueuedMsgType_EndOfList = 0x0
    QueuedMsgType_Event = 0x1
    QueuedMsgType_RTPC = 0x2
    QueuedMsgType_RTPCWithTransition = 0x3
    QueuedMsgType_ResetRTPC = 0x4
    QueuedMsgType_ResetRTPCValue = 0x5
    QueuedMsgType_ResetRTPCValueWithTransition = 0x6
    QueuedMsgType_State = 0x7
    QueuedMsgType_Switch = 0x8
    QueuedMsgType_ResetSwitches = 0x9
    QueuedMsgType_Trigger = 0xA
    QueuedMsgType_RegisterGameObj = 0xB
    QueuedMsgType_UnregisterGameObj = 0xC
    QueuedMsgType_GameObjPosition = 0xD
    QueuedMsgType_GameObjActiveListeners = 0xE
    QueuedMsgType_GameObjActiveControllers = 0xF
    QueuedMsgType_DefaultActiveListeners = 0x10
    QueuedMsgType_ResetListeners = 0x11
    QueuedMsgType_ListenerSpatialization = 0x12
    QueuedMsgType_GameObjEnvValues = 0x13
    QueuedMsgType_GameObjDryLevel = 0x14
    QueuedMsgType_GameObjObstruction = 0x15
    QueuedMsgType_GameObjMultipleObstruction = 0x16
    QueuedMsgType_SetOutputVolume = 0x17
    QueuedMsgType_OpenDynamicSequence = 0x18
    QueuedMsgType_DynamicSequenceCmd = 0x19
    QueuedMsgType_KillBank = 0x1A
    QueuedMsgType_StopAll = 0x1B
    QueuedMsgType_AddOutput = 0x1C
    QueuedMsgType_RemoveOutput = 0x1D
    QueuedMsgType_ReplaceOutput = 0x1E
    QueuedMsgType_SetBusDevice = 0x1F
    QueuedMsgType_StopPlayingID = 0x20
    QueuedMsgType_EventAction = 0x21
    QueuedMsgType_EventPostMIDI = 0x22
    QueuedMsgType_EventStopMIDI = 0x23
    QueuedMsgType_LockUnlockStreamCache = 0x24
    QueuedMsgType_GameObjScalingFactor = 0x25
    QueuedMsgType_GameObjMultiPosition = 0x26
    QueuedMsgType_Seek = 0x27
    QueuedMsgType_SourcePluginAction = 0x28
    QueuedMsgType_StartStopOutputCapture = 0x29
    QueuedMsgType_AddOutputCaptureMarker = 0x2A
    QueuedMsgType_SetEffect = 0x2B
    QueuedMsgType_SetDeviceEffect = 0x2C
    QueuedMsgType_SetMixer = 0x2D
    QueuedMsgType_SetBusConfig = 0x2E
    QueuedMsgType_SetPanningRule = 0x2F
    QueuedMsgType_SetSpeakerAngles = 0x30
    QueuedMsgType_SetProcessMode = 0x31
    QueuedMsgType_SendMainOutputToDevice = 0x32
    QueuedMsgType_SetRandomSeed = 0x33
    QueuedMsgType_MuteBackgroundMusic = 0x34
    QueuedMsgType_PluginCustomGameData = 0x35
    QueuedMsgType_SuspendOrWakeup = 0x36
    QueuedMsgType_InitSinkPlugin = 0x37
    QueuedMsgType_ApiExtension = 0x38
    QueuedMsgType_PlayingIDAction = 0x39
    QueuedMsgType_DynamicSequenceSeek = 0x3A
    QueuedMsgType_OfflineRendering = 0x3B
    QueuedMsgType_QueueWrapAround = 0x3C
    QueuedMsgType_Invalid = 0x3D

class __MIDL___MIDL_itf_mmdeviceapi_0000_0000_0001(IntEnum):
    eRender = 0x0
    eCapture = 0x1
    eAll = 0x2
    EDataFlow_enum_count = 0x3

class __MIDL___MIDL_itf_mmdeviceapi_0000_0000_0002(IntEnum):
    eConsole = 0x0
    eMultimedia = 0x1
    eCommunications = 0x2
    ERole_enum_count = 0x3

class __MIDL___MIDL_itf_mmdeviceapi_0000_0000_0003(IntEnum):
    RemoteNetworkDevice = 0x0
    Speakers = 0x1
    LineLevel = 0x2
    Headphones = 0x3
    Microphone = 0x4
    Headset = 0x5
    Handset = 0x6
    UnknownDigitalPassthrough = 0x7
    SPDIF = 0x8
    DigitalAudioDisplayDevice = 0x9
    UnknownFormFactor = 0xA
    EndpointFormFactor_enum_count = 0xB

class AkLoopValue(IntEnum):
    AkLoopVal_Infinite = 0x0
    AkLoopVal_NotLooping = 0x1

class CAkTransitionManager(ctypes.Structure):
    class AkTransitionCategory(IntEnum):
        TC_Fade = 0x0
        TC_State = 0x1

    def AddTransitionToList(): pass

    def AddTransitionUser(): pass

    def ChangeParameter(): pass

    def Pause(): pass

    def PauseAllStateTransitions(): pass

    def ProcessTransitionsList(): pass

    def RemoveTransitionUser(): pass

    def Resume(): pass

    def ResumeAllStateTransitions(): pass

    def Term(): pass

    def TermList(): pass

    _fields_ = [
        ("m_ActiveTransitionsList_Fade", ctypes.c_char * 0x10),  # AkArray<CAkTransition *,CAkTransition *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkTransition *> >
        ("m_ActiveTransitionsList_State", ctypes.c_char * 0x10),  # AkArray<CAkTransition *,CAkTransition *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkTransition *> >
    ]
    m_ActiveTransitionsList_Fade: bytes
    m_ActiveTransitionsList_State: bytes

class AkBuiltInSinks(IntEnum):
    AKPLUGINID_DEFAULT_SINK = 0xAE
    AKPLUGINID_DVR_SINK = 0xAF
    AKPLUGINID_COMMUNICATION_SINK = 0xB0
    AKPLUGINID_PERSONAL_SINK = 0xB1
    AKPLUGINID_VOICE_SINK = 0xB2
    AKPLUGINID_PAD_SINK = 0xB3
    AKPLUGINID_AUX_SINK = 0xB4
    AKPLUGINID_DUMMY_SINK = 0xB5

class tagCOINIT(IntEnum):
    COINIT_MULTITHREADED = 0x0
    COINIT_APARTMENTTHREADED = 0x2
    COINIT_DISABLE_OLE1DDE = 0x4
    COINIT_SPEED_OVER_MEMORY = 0x8

class AkTransitionMode(IntEnum):
    Transition_Disabled = 0x0
    Transition_CrossFadeAmp = 0x1
    Transition_CrossFadePower = 0x2
    Transition_Delay = 0x3
    Transition_SampleAccurate = 0x4
    Transition_TriggerRate = 0x5

class AkSourcePluginActionType(IntEnum):
    AkSourcePluginActionType_Play = 0x0
    AkSourcePluginActionType_Stop = 0x1

class VPLNodeState(IntEnum):
    NodeStateInit = 0x0
    NodeStatePlay = 0x1
    NodeStateStop = 0x2
    NodeStatePause = 0x3
    NodeStateIdle = 0x4

class TransitionTargets(IntEnum):
    TransTarget_Play = 0x1000000

class AkModulatorState(IntEnum):
    AkModulatorState_New = 0x0
    AkModulatorState_Triggered = 0x1
    AkModulatorState_ReleaseTriggered = 0x2
    AkModulatorState_Finished = 0x3
    AkModulatorState_Invalid = 0xFFFFFFFF

class AkModulatorType(IntEnum):
    AkModulatorType_LFO = 0x0
    AkModulatorType_Envelope = 0x1
    AkModulatorType_Time = 0x2
    AkModulatorType_Unknown = 0xFFFFFFFF

class PitchOperatingMode(IntEnum):
    PitchOperatingMode_Bypass = 0x0
    PitchOperatingMode_Fixed = 0x1
    PitchOperatingMode_Interpolating = 0x2
    NumPitchOperatingMode = 0x3

class AkLoudnessFrequencyWeighting(IntEnum):
    FrequencyWeighting_None = 0x0
    FrequencyWeighting_K = 0x1
    FrequencyWeighting_A = 0x2
    AK_NUM_FREQUENCY_WEIGHTINGS = 0x3

class ExtraChannelsMode(IntEnum):
    Zero = 0x0
    Copy = 0x1
    Decorrelate = 0x2

class AkVirtualAcousticsPropID(IntEnum):
    AkAcousticTexturePropID_AbsorptionOffset = 0x0
    AkAcousticTexturePropID_AbsorptionLow = 0x1
    AkAcousticTexturePropID_AbsorptionMidLow = 0x2
    AkAcousticTexturePropID_AbsorptionMidHigh = 0x3
    AkAcousticTexturePropID_AbsorptionHigh = 0x4
    AkAcousticTexturePropID_Scattering = 0x5
    AkVirtualAcousticsPropID_NUM = 0x6

class AkLECmd(ctypes.Structure):
    class Type(IntEnum):
        Type_Play = 0x0
        Type_PlayPause = 0x1
        Type_Pause = 0x2
        Type_Resume = 0x3
        Type_StopLooping = 0x4
        Type_Seek = 0x5

    _fields_ = [
        ("m_pCtx", ctypes.c_ulonglong),
        ("m_ulSequenceNumber", ctypes.c_uint32),
        ("_bf_c", ctypes.c_int16),
    ]
    m_pCtx: int
    m_ulSequenceNumber: int
    _bf_c: int

class shaderc_target_env(IntEnum):
    shaderc_target_env_vulkan = 0x0
    shaderc_target_env_opengl = 0x1
    shaderc_target_env_opengl_compat = 0x2
    shaderc_target_env_webgpu = 0x3

class shaderc_shader_kind(IntEnum):
    shaderc_vertex_shader = 0x0
    shaderc_fragment_shader = 0x1
    shaderc_compute_shader = 0x2
    shaderc_geometry_shader = 0x3
    shaderc_tess_control_shader = 0x4
    shaderc_tess_evaluation_shader = 0x5
    shaderc_glsl_infer_from_source = 0x6
    shaderc_glsl_default_vertex_shader = 0x7
    shaderc_glsl_default_fragment_shader = 0x8
    shaderc_glsl_default_compute_shader = 0x9
    shaderc_glsl_default_geometry_shader = 0xA
    shaderc_glsl_default_tess_control_shader = 0xB
    shaderc_glsl_default_tess_evaluation_shader = 0xC
    shaderc_spirv_assembly = 0xD
    shaderc_raygen_shader = 0xE
    shaderc_anyhit_shader = 0xF
    shaderc_closesthit_shader = 0x10
    shaderc_miss_shader = 0x11
    shaderc_intersection_shader = 0x12
    shaderc_callable_shader = 0x13
    shaderc_glsl_default_raygen_shader = 0x14
    shaderc_glsl_default_anyhit_shader = 0x15
    shaderc_glsl_default_closesthit_shader = 0x16
    shaderc_glsl_default_miss_shader = 0x17
    shaderc_glsl_default_intersection_shader = 0x18
    shaderc_glsl_default_callable_shader = 0x19
    shaderc_task_shader = 0x1A
    shaderc_mesh_shader = 0x1B
    shaderc_glsl_default_task_shader = 0x1C
    shaderc_glsl_default_mesh_shader = 0x1D

class SpvExecutionMode_(IntEnum):
    SpvExecutionModeInvocations = 0x0
    SpvExecutionModeSpacingEqual = 0x1
    SpvExecutionModeSpacingFractionalEven = 0x2
    SpvExecutionModeSpacingFractionalOdd = 0x3
    SpvExecutionModeVertexOrderCw = 0x4
    SpvExecutionModeVertexOrderCcw = 0x5
    SpvExecutionModePixelCenterInteger = 0x6
    SpvExecutionModeOriginUpperLeft = 0x7
    SpvExecutionModeOriginLowerLeft = 0x8
    SpvExecutionModeEarlyFragmentTests = 0x9
    SpvExecutionModePointMode = 0xA
    SpvExecutionModeXfb = 0xB
    SpvExecutionModeDepthReplacing = 0xC
    SpvExecutionModeDepthGreater = 0xE
    SpvExecutionModeDepthLess = 0xF
    SpvExecutionModeDepthUnchanged = 0x10
    SpvExecutionModeLocalSize = 0x11
    SpvExecutionModeLocalSizeHint = 0x12
    SpvExecutionModeInputPoints = 0x13
    SpvExecutionModeInputLines = 0x14
    SpvExecutionModeInputLinesAdjacency = 0x15
    SpvExecutionModeTriangles = 0x16
    SpvExecutionModeInputTrianglesAdjacency = 0x17
    SpvExecutionModeQuads = 0x18
    SpvExecutionModeIsolines = 0x19
    SpvExecutionModeOutputVertices = 0x1A
    SpvExecutionModeOutputPoints = 0x1B
    SpvExecutionModeOutputLineStrip = 0x1C
    SpvExecutionModeOutputTriangleStrip = 0x1D
    SpvExecutionModeVecTypeHint = 0x1E
    SpvExecutionModeContractionOff = 0x1F
    SpvExecutionModeInitializer = 0x21
    SpvExecutionModeFinalizer = 0x22
    SpvExecutionModeSubgroupSize = 0x23
    SpvExecutionModeSubgroupsPerWorkgroup = 0x24
    SpvExecutionModeSubgroupsPerWorkgroupId = 0x25
    SpvExecutionModeLocalSizeId = 0x26
    SpvExecutionModeLocalSizeHintId = 0x27
    SpvExecutionModeSubgroupUniformControlFlowKHR = 0x1145
    SpvExecutionModePostDepthCoverage = 0x115E
    SpvExecutionModeDenormPreserve = 0x116B
    SpvExecutionModeDenormFlushToZero = 0x116C
    SpvExecutionModeSignedZeroInfNanPreserve = 0x116D
    SpvExecutionModeRoundingModeRTE = 0x116E
    SpvExecutionModeRoundingModeRTZ = 0x116F
    SpvExecutionModeEarlyAndLateFragmentTestsAMD = 0x1399
    SpvExecutionModeStencilRefReplacingEXT = 0x13A3
    SpvExecutionModeStencilRefUnchangedFrontAMD = 0x13D7
    SpvExecutionModeStencilRefGreaterFrontAMD = 0x13D8
    SpvExecutionModeStencilRefLessFrontAMD = 0x13D9
    SpvExecutionModeStencilRefUnchangedBackAMD = 0x13DA
    SpvExecutionModeStencilRefGreaterBackAMD = 0x13DB
    SpvExecutionModeStencilRefLessBackAMD = 0x13DC
    SpvExecutionModeOutputLinesEXT = 0x1495
    SpvExecutionModeOutputPrimitivesEXT = 0x1496
    SpvExecutionModeDerivativeGroupQuadsNV = 0x14A9
    SpvExecutionModeDerivativeGroupLinearNV = 0x14AA
    SpvExecutionModeOutputTrianglesEXT = 0x14B2
    SpvExecutionModePixelInterlockOrderedEXT = 0x14F6
    SpvExecutionModePixelInterlockUnorderedEXT = 0x14F7
    SpvExecutionModeSampleInterlockOrderedEXT = 0x14F8
    SpvExecutionModeSampleInterlockUnorderedEXT = 0x14F9
    SpvExecutionModeShadingRateInterlockOrderedEXT = 0x14FA
    SpvExecutionModeShadingRateInterlockUnorderedEXT = 0x14FB
    SpvExecutionModeSharedLocalMemorySizeINTEL = 0x15F2
    SpvExecutionModeRoundingModeRTPINTEL = 0x15F4
    SpvExecutionModeRoundingModeRTNINTEL = 0x15F5
    SpvExecutionModeFloatingPointModeALTINTEL = 0x15F6
    SpvExecutionModeFloatingPointModeIEEEINTEL = 0x15F7
    SpvExecutionModeMaxWorkgroupSizeINTEL = 0x1705
    SpvExecutionModeMaxWorkDimINTEL = 0x1706
    SpvExecutionModeNoGlobalOffsetINTEL = 0x1707
    SpvExecutionModeNumSIMDWorkitemsINTEL = 0x1708
    SpvExecutionModeSchedulerTargetFmaxMhzINTEL = 0x170F
    SpvExecutionModeNamedBarrierCountINTEL = 0x1911
    SpvExecutionModeMax = 0x7FFFFFFF

class AkVirtualAcousticsType(IntEnum):
    AkVirtualAcousticsType_Texture = 0x0

class SpvOp_(IntEnum):
    SpvOpNop = 0x0
    SpvOpUndef = 0x1
    SpvOpSourceContinued = 0x2
    SpvOpSource = 0x3
    SpvOpSourceExtension = 0x4
    SpvOpName = 0x5
    SpvOpMemberName = 0x6
    SpvOpString = 0x7
    SpvOpLine = 0x8
    SpvOpExtension = 0xA
    SpvOpExtInstImport = 0xB
    SpvOpExtInst = 0xC
    SpvOpMemoryModel = 0xE
    SpvOpEntryPoint = 0xF
    SpvOpExecutionMode = 0x10
    SpvOpCapability = 0x11
    SpvOpTypeVoid = 0x13
    SpvOpTypeBool = 0x14
    SpvOpTypeInt = 0x15
    SpvOpTypeFloat = 0x16
    SpvOpTypeVector = 0x17
    SpvOpTypeMatrix = 0x18
    SpvOpTypeImage = 0x19
    SpvOpTypeSampler = 0x1A
    SpvOpTypeSampledImage = 0x1B
    SpvOpTypeArray = 0x1C
    SpvOpTypeRuntimeArray = 0x1D
    SpvOpTypeStruct = 0x1E
    SpvOpTypeOpaque = 0x1F
    SpvOpTypePointer = 0x20
    SpvOpTypeFunction = 0x21
    SpvOpTypeEvent = 0x22
    SpvOpTypeDeviceEvent = 0x23
    SpvOpTypeReserveId = 0x24
    SpvOpTypeQueue = 0x25
    SpvOpTypePipe = 0x26
    SpvOpTypeForwardPointer = 0x27
    SpvOpConstantTrue = 0x29
    SpvOpConstantFalse = 0x2A
    SpvOpConstant = 0x2B
    SpvOpConstantComposite = 0x2C
    SpvOpConstantSampler = 0x2D
    SpvOpConstantNull = 0x2E
    SpvOpSpecConstantTrue = 0x30
    SpvOpSpecConstantFalse = 0x31
    SpvOpSpecConstant = 0x32
    SpvOpSpecConstantComposite = 0x33
    SpvOpSpecConstantOp = 0x34
    SpvOpFunction = 0x36
    SpvOpFunctionParameter = 0x37
    SpvOpFunctionEnd = 0x38
    SpvOpFunctionCall = 0x39
    SpvOpVariable = 0x3B
    SpvOpImageTexelPointer = 0x3C
    SpvOpLoad = 0x3D
    SpvOpStore = 0x3E
    SpvOpCopyMemory = 0x3F
    SpvOpCopyMemorySized = 0x40
    SpvOpAccessChain = 0x41
    SpvOpInBoundsAccessChain = 0x42
    SpvOpPtrAccessChain = 0x43
    SpvOpArrayLength = 0x44
    SpvOpGenericPtrMemSemantics = 0x45
    SpvOpInBoundsPtrAccessChain = 0x46
    SpvOpDecorate = 0x47
    SpvOpMemberDecorate = 0x48
    SpvOpDecorationGroup = 0x49
    SpvOpGroupDecorate = 0x4A
    SpvOpGroupMemberDecorate = 0x4B
    SpvOpVectorExtractDynamic = 0x4D
    SpvOpVectorInsertDynamic = 0x4E
    SpvOpVectorShuffle = 0x4F
    SpvOpCompositeConstruct = 0x50
    SpvOpCompositeExtract = 0x51
    SpvOpCompositeInsert = 0x52
    SpvOpCopyObject = 0x53
    SpvOpTranspose = 0x54
    SpvOpSampledImage = 0x56
    SpvOpImageSampleImplicitLod = 0x57
    SpvOpImageSampleExplicitLod = 0x58
    SpvOpImageSampleDrefImplicitLod = 0x59
    SpvOpImageSampleDrefExplicitLod = 0x5A
    SpvOpImageSampleProjImplicitLod = 0x5B
    SpvOpImageSampleProjExplicitLod = 0x5C
    SpvOpImageSampleProjDrefImplicitLod = 0x5D
    SpvOpImageSampleProjDrefExplicitLod = 0x5E
    SpvOpImageFetch = 0x5F
    SpvOpImageGather = 0x60
    SpvOpImageDrefGather = 0x61
    SpvOpImageRead = 0x62
    SpvOpImageWrite = 0x63
    SpvOpImage = 0x64
    SpvOpImageQueryFormat = 0x65
    SpvOpImageQueryOrder = 0x66
    SpvOpImageQuerySizeLod = 0x67
    SpvOpImageQuerySize = 0x68
    SpvOpImageQueryLod = 0x69
    SpvOpImageQueryLevels = 0x6A
    SpvOpImageQuerySamples = 0x6B
    SpvOpConvertFToU = 0x6D
    SpvOpConvertFToS = 0x6E
    SpvOpConvertSToF = 0x6F
    SpvOpConvertUToF = 0x70
    SpvOpUConvert = 0x71
    SpvOpSConvert = 0x72
    SpvOpFConvert = 0x73
    SpvOpQuantizeToF16 = 0x74
    SpvOpConvertPtrToU = 0x75
    SpvOpSatConvertSToU = 0x76
    SpvOpSatConvertUToS = 0x77
    SpvOpConvertUToPtr = 0x78
    SpvOpPtrCastToGeneric = 0x79
    SpvOpGenericCastToPtr = 0x7A
    SpvOpGenericCastToPtrExplicit = 0x7B
    SpvOpBitcast = 0x7C
    SpvOpSNegate = 0x7E
    SpvOpFNegate = 0x7F
    SpvOpIAdd = 0x80
    SpvOpFAdd = 0x81
    SpvOpISub = 0x82
    SpvOpFSub = 0x83
    SpvOpIMul = 0x84
    SpvOpFMul = 0x85
    SpvOpUDiv = 0x86
    SpvOpSDiv = 0x87
    SpvOpFDiv = 0x88
    SpvOpUMod = 0x89
    SpvOpSRem = 0x8A
    SpvOpSMod = 0x8B
    SpvOpFRem = 0x8C
    SpvOpFMod = 0x8D
    SpvOpVectorTimesScalar = 0x8E
    SpvOpMatrixTimesScalar = 0x8F
    SpvOpVectorTimesMatrix = 0x90
    SpvOpMatrixTimesVector = 0x91
    SpvOpMatrixTimesMatrix = 0x92
    SpvOpOuterProduct = 0x93
    SpvOpDot = 0x94
    SpvOpIAddCarry = 0x95
    SpvOpISubBorrow = 0x96
    SpvOpUMulExtended = 0x97
    SpvOpSMulExtended = 0x98
    SpvOpAny = 0x9A
    SpvOpAll = 0x9B
    SpvOpIsNan = 0x9C
    SpvOpIsInf = 0x9D
    SpvOpIsFinite = 0x9E
    SpvOpIsNormal = 0x9F
    SpvOpSignBitSet = 0xA0
    SpvOpLessOrGreater = 0xA1
    SpvOpOrdered = 0xA2
    SpvOpUnordered = 0xA3
    SpvOpLogicalEqual = 0xA4
    SpvOpLogicalNotEqual = 0xA5
    SpvOpLogicalOr = 0xA6
    SpvOpLogicalAnd = 0xA7
    SpvOpLogicalNot = 0xA8
    SpvOpSelect = 0xA9
    SpvOpIEqual = 0xAA
    SpvOpINotEqual = 0xAB
    SpvOpUGreaterThan = 0xAC
    SpvOpSGreaterThan = 0xAD
    SpvOpUGreaterThanEqual = 0xAE
    SpvOpSGreaterThanEqual = 0xAF
    SpvOpULessThan = 0xB0
    SpvOpSLessThan = 0xB1
    SpvOpULessThanEqual = 0xB2
    SpvOpSLessThanEqual = 0xB3
    SpvOpFOrdEqual = 0xB4
    SpvOpFUnordEqual = 0xB5
    SpvOpFOrdNotEqual = 0xB6
    SpvOpFUnordNotEqual = 0xB7
    SpvOpFOrdLessThan = 0xB8
    SpvOpFUnordLessThan = 0xB9
    SpvOpFOrdGreaterThan = 0xBA
    SpvOpFUnordGreaterThan = 0xBB
    SpvOpFOrdLessThanEqual = 0xBC
    SpvOpFUnordLessThanEqual = 0xBD
    SpvOpFOrdGreaterThanEqual = 0xBE
    SpvOpFUnordGreaterThanEqual = 0xBF
    SpvOpShiftRightLogical = 0xC2
    SpvOpShiftRightArithmetic = 0xC3
    SpvOpShiftLeftLogical = 0xC4
    SpvOpBitwiseOr = 0xC5
    SpvOpBitwiseXor = 0xC6
    SpvOpBitwiseAnd = 0xC7
    SpvOpNot = 0xC8
    SpvOpBitFieldInsert = 0xC9
    SpvOpBitFieldSExtract = 0xCA
    SpvOpBitFieldUExtract = 0xCB
    SpvOpBitReverse = 0xCC
    SpvOpBitCount = 0xCD
    SpvOpDPdx = 0xCF
    SpvOpDPdy = 0xD0
    SpvOpFwidth = 0xD1
    SpvOpDPdxFine = 0xD2
    SpvOpDPdyFine = 0xD3
    SpvOpFwidthFine = 0xD4
    SpvOpDPdxCoarse = 0xD5
    SpvOpDPdyCoarse = 0xD6
    SpvOpFwidthCoarse = 0xD7
    SpvOpEmitVertex = 0xDA
    SpvOpEndPrimitive = 0xDB
    SpvOpEmitStreamVertex = 0xDC
    SpvOpEndStreamPrimitive = 0xDD
    SpvOpControlBarrier = 0xE0
    SpvOpMemoryBarrier = 0xE1
    SpvOpAtomicLoad = 0xE3
    SpvOpAtomicStore = 0xE4
    SpvOpAtomicExchange = 0xE5
    SpvOpAtomicCompareExchange = 0xE6
    SpvOpAtomicCompareExchangeWeak = 0xE7
    SpvOpAtomicIIncrement = 0xE8
    SpvOpAtomicIDecrement = 0xE9
    SpvOpAtomicIAdd = 0xEA
    SpvOpAtomicISub = 0xEB
    SpvOpAtomicSMin = 0xEC
    SpvOpAtomicUMin = 0xED
    SpvOpAtomicSMax = 0xEE
    SpvOpAtomicUMax = 0xEF
    SpvOpAtomicAnd = 0xF0
    SpvOpAtomicOr = 0xF1
    SpvOpAtomicXor = 0xF2
    SpvOpPhi = 0xF5
    SpvOpLoopMerge = 0xF6
    SpvOpSelectionMerge = 0xF7
    SpvOpLabel = 0xF8
    SpvOpBranch = 0xF9
    SpvOpBranchConditional = 0xFA
    SpvOpSwitch = 0xFB
    SpvOpKill = 0xFC
    SpvOpReturn = 0xFD
    SpvOpReturnValue = 0xFE
    SpvOpUnreachable = 0xFF
    SpvOpLifetimeStart = 0x100
    SpvOpLifetimeStop = 0x101
    SpvOpGroupAsyncCopy = 0x103
    SpvOpGroupWaitEvents = 0x104
    SpvOpGroupAll = 0x105
    SpvOpGroupAny = 0x106
    SpvOpGroupBroadcast = 0x107
    SpvOpGroupIAdd = 0x108
    SpvOpGroupFAdd = 0x109
    SpvOpGroupFMin = 0x10A
    SpvOpGroupUMin = 0x10B
    SpvOpGroupSMin = 0x10C
    SpvOpGroupFMax = 0x10D
    SpvOpGroupUMax = 0x10E
    SpvOpGroupSMax = 0x10F
    SpvOpReadPipe = 0x112
    SpvOpWritePipe = 0x113
    SpvOpReservedReadPipe = 0x114
    SpvOpReservedWritePipe = 0x115
    SpvOpReserveReadPipePackets = 0x116
    SpvOpReserveWritePipePackets = 0x117
    SpvOpCommitReadPipe = 0x118
    SpvOpCommitWritePipe = 0x119
    SpvOpIsValidReserveId = 0x11A
    SpvOpGetNumPipePackets = 0x11B
    SpvOpGetMaxPipePackets = 0x11C
    SpvOpGroupReserveReadPipePackets = 0x11D
    SpvOpGroupReserveWritePipePackets = 0x11E
    SpvOpGroupCommitReadPipe = 0x11F
    SpvOpGroupCommitWritePipe = 0x120
    SpvOpEnqueueMarker = 0x123
    SpvOpEnqueueKernel = 0x124
    SpvOpGetKernelNDrangeSubGroupCount = 0x125
    SpvOpGetKernelNDrangeMaxSubGroupSize = 0x126
    SpvOpGetKernelWorkGroupSize = 0x127
    SpvOpGetKernelPreferredWorkGroupSizeMultiple = 0x128
    SpvOpRetainEvent = 0x129
    SpvOpReleaseEvent = 0x12A
    SpvOpCreateUserEvent = 0x12B
    SpvOpIsValidEvent = 0x12C
    SpvOpSetUserEventStatus = 0x12D
    SpvOpCaptureEventProfilingInfo = 0x12E
    SpvOpGetDefaultQueue = 0x12F
    SpvOpBuildNDRange = 0x130
    SpvOpImageSparseSampleImplicitLod = 0x131
    SpvOpImageSparseSampleExplicitLod = 0x132
    SpvOpImageSparseSampleDrefImplicitLod = 0x133
    SpvOpImageSparseSampleDrefExplicitLod = 0x134
    SpvOpImageSparseSampleProjImplicitLod = 0x135
    SpvOpImageSparseSampleProjExplicitLod = 0x136
    SpvOpImageSparseSampleProjDrefImplicitLod = 0x137
    SpvOpImageSparseSampleProjDrefExplicitLod = 0x138
    SpvOpImageSparseFetch = 0x139
    SpvOpImageSparseGather = 0x13A
    SpvOpImageSparseDrefGather = 0x13B
    SpvOpImageSparseTexelsResident = 0x13C
    SpvOpNoLine = 0x13D
    SpvOpAtomicFlagTestAndSet = 0x13E
    SpvOpAtomicFlagClear = 0x13F
    SpvOpImageSparseRead = 0x140
    SpvOpSizeOf = 0x141
    SpvOpTypePipeStorage = 0x142
    SpvOpConstantPipeStorage = 0x143
    SpvOpCreatePipeFromPipeStorage = 0x144
    SpvOpGetKernelLocalSizeForSubgroupCount = 0x145
    SpvOpGetKernelMaxNumSubgroups = 0x146
    SpvOpTypeNamedBarrier = 0x147
    SpvOpNamedBarrierInitialize = 0x148
    SpvOpMemoryNamedBarrier = 0x149
    SpvOpModuleProcessed = 0x14A
    SpvOpExecutionModeId = 0x14B
    SpvOpDecorateId = 0x14C
    SpvOpGroupNonUniformElect = 0x14D
    SpvOpGroupNonUniformAll = 0x14E
    SpvOpGroupNonUniformAny = 0x14F
    SpvOpGroupNonUniformAllEqual = 0x150
    SpvOpGroupNonUniformBroadcast = 0x151
    SpvOpGroupNonUniformBroadcastFirst = 0x152
    SpvOpGroupNonUniformBallot = 0x153
    SpvOpGroupNonUniformInverseBallot = 0x154
    SpvOpGroupNonUniformBallotBitExtract = 0x155
    SpvOpGroupNonUniformBallotBitCount = 0x156
    SpvOpGroupNonUniformBallotFindLSB = 0x157
    SpvOpGroupNonUniformBallotFindMSB = 0x158
    SpvOpGroupNonUniformShuffle = 0x159
    SpvOpGroupNonUniformShuffleXor = 0x15A
    SpvOpGroupNonUniformShuffleUp = 0x15B
    SpvOpGroupNonUniformShuffleDown = 0x15C
    SpvOpGroupNonUniformIAdd = 0x15D
    SpvOpGroupNonUniformFAdd = 0x15E
    SpvOpGroupNonUniformIMul = 0x15F
    SpvOpGroupNonUniformFMul = 0x160
    SpvOpGroupNonUniformSMin = 0x161
    SpvOpGroupNonUniformUMin = 0x162
    SpvOpGroupNonUniformFMin = 0x163
    SpvOpGroupNonUniformSMax = 0x164
    SpvOpGroupNonUniformUMax = 0x165
    SpvOpGroupNonUniformFMax = 0x166
    SpvOpGroupNonUniformBitwiseAnd = 0x167
    SpvOpGroupNonUniformBitwiseOr = 0x168
    SpvOpGroupNonUniformBitwiseXor = 0x169
    SpvOpGroupNonUniformLogicalAnd = 0x16A
    SpvOpGroupNonUniformLogicalOr = 0x16B
    SpvOpGroupNonUniformLogicalXor = 0x16C
    SpvOpGroupNonUniformQuadBroadcast = 0x16D
    SpvOpGroupNonUniformQuadSwap = 0x16E
    SpvOpCopyLogical = 0x190
    SpvOpPtrEqual = 0x191
    SpvOpPtrNotEqual = 0x192
    SpvOpPtrDiff = 0x193
    SpvOpTerminateInvocation = 0x1140
    SpvOpSubgroupBallotKHR = 0x1145
    SpvOpSubgroupFirstInvocationKHR = 0x1146
    SpvOpSubgroupAllKHR = 0x114C
    SpvOpSubgroupAnyKHR = 0x114D
    SpvOpSubgroupAllEqualKHR = 0x114E
    SpvOpGroupNonUniformRotateKHR = 0x114F
    SpvOpSubgroupReadInvocationKHR = 0x1150
    SpvOpTraceRayKHR = 0x115D
    SpvOpExecuteCallableKHR = 0x115E
    SpvOpConvertUToAccelerationStructureKHR = 0x115F
    SpvOpIgnoreIntersectionKHR = 0x1160
    SpvOpTerminateRayKHR = 0x1161
    SpvOpSDot = 0x1162
    SpvOpUDot = 0x1163
    SpvOpSUDot = 0x1164
    SpvOpSDotAccSat = 0x1165
    SpvOpUDotAccSat = 0x1166
    SpvOpSUDotAccSat = 0x1167
    SpvOpTypeRayQueryKHR = 0x1178
    SpvOpRayQueryInitializeKHR = 0x1179
    SpvOpRayQueryTerminateKHR = 0x117A
    SpvOpRayQueryGenerateIntersectionKHR = 0x117B
    SpvOpRayQueryConfirmIntersectionKHR = 0x117C
    SpvOpRayQueryProceedKHR = 0x117D
    SpvOpRayQueryGetIntersectionTypeKHR = 0x117F
    SpvOpGroupIAddNonUniformAMD = 0x1388
    SpvOpGroupFAddNonUniformAMD = 0x1389
    SpvOpGroupFMinNonUniformAMD = 0x138A
    SpvOpGroupUMinNonUniformAMD = 0x138B
    SpvOpGroupSMinNonUniformAMD = 0x138C
    SpvOpGroupFMaxNonUniformAMD = 0x138D
    SpvOpGroupUMaxNonUniformAMD = 0x138E
    SpvOpGroupSMaxNonUniformAMD = 0x138F
    SpvOpFragmentMaskFetchAMD = 0x1393
    SpvOpFragmentFetchAMD = 0x1394
    SpvOpReadClockKHR = 0x13C0
    SpvOpHitObjectRecordHitMotionNV = 0x1481
    SpvOpHitObjectRecordHitWithIndexMotionNV = 0x1482
    SpvOpHitObjectRecordMissMotionNV = 0x1483
    SpvOpHitObjectGetWorldToObjectNV = 0x1484
    SpvOpHitObjectGetObjectToWorldNV = 0x1485
    SpvOpHitObjectGetObjectRayDirectionNV = 0x1486
    SpvOpHitObjectGetObjectRayOriginNV = 0x1487
    SpvOpHitObjectTraceRayMotionNV = 0x1488
    SpvOpHitObjectGetShaderRecordBufferHandleNV = 0x1489
    SpvOpHitObjectGetShaderBindingTableRecordIndexNV = 0x148A
    SpvOpHitObjectRecordEmptyNV = 0x148B
    SpvOpHitObjectTraceRayNV = 0x148C
    SpvOpHitObjectRecordHitNV = 0x148D
    SpvOpHitObjectRecordHitWithIndexNV = 0x148E
    SpvOpHitObjectRecordMissNV = 0x148F
    SpvOpHitObjectExecuteShaderNV = 0x1490
    SpvOpHitObjectGetCurrentTimeNV = 0x1491
    SpvOpHitObjectGetAttributesNV = 0x1492
    SpvOpHitObjectGetHitKindNV = 0x1493
    SpvOpHitObjectGetPrimitiveIndexNV = 0x1494
    SpvOpHitObjectGetGeometryIndexNV = 0x1495
    SpvOpHitObjectGetInstanceIdNV = 0x1496
    SpvOpHitObjectGetInstanceCustomIndexNV = 0x1497
    SpvOpHitObjectGetWorldRayDirectionNV = 0x1498
    SpvOpHitObjectGetWorldRayOriginNV = 0x1499
    SpvOpHitObjectGetRayTMaxNV = 0x149A
    SpvOpHitObjectGetRayTMinNV = 0x149B
    SpvOpHitObjectIsEmptyNV = 0x149C
    SpvOpHitObjectIsHitNV = 0x149D
    SpvOpHitObjectIsMissNV = 0x149E
    SpvOpReorderThreadWithHitObjectNV = 0x149F
    SpvOpReorderThreadWithHintNV = 0x14A0
    SpvOpTypeHitObjectNV = 0x14A1
    SpvOpImageSampleFootprintNV = 0x14A3
    SpvOpEmitMeshTasksEXT = 0x14AE
    SpvOpSetMeshOutputsEXT = 0x14AF
    SpvOpGroupNonUniformPartitionNV = 0x14B0
    SpvOpWritePackedPrimitiveIndices4x8NV = 0x14B3
    SpvOpReportIntersectionKHR = 0x14D6
    SpvOpIgnoreIntersectionNV = 0x14D7
    SpvOpTerminateRayNV = 0x14D8
    SpvOpTraceNV = 0x14D9
    SpvOpTraceMotionNV = 0x14DA
    SpvOpTraceRayMotionNV = 0x14DB
    SpvOpTypeAccelerationStructureKHR = 0x14DD
    SpvOpExecuteCallableNV = 0x14E0
    SpvOpTypeCooperativeMatrixNV = 0x14EE
    SpvOpCooperativeMatrixLoadNV = 0x14EF
    SpvOpCooperativeMatrixStoreNV = 0x14F0
    SpvOpCooperativeMatrixMulAddNV = 0x14F1
    SpvOpCooperativeMatrixLengthNV = 0x14F2
    SpvOpBeginInvocationInterlockEXT = 0x14F4
    SpvOpEndInvocationInterlockEXT = 0x14F5
    SpvOpDemoteToHelperInvocation = 0x1504
    SpvOpIsHelperInvocationEXT = 0x1505
    SpvOpConvertUToImageNV = 0x150F
    SpvOpConvertUToSamplerNV = 0x1510
    SpvOpConvertImageToUNV = 0x1511
    SpvOpConvertSamplerToUNV = 0x1512
    SpvOpConvertUToSampledImageNV = 0x1513
    SpvOpConvertSampledImageToUNV = 0x1514
    SpvOpSamplerImageAddressingModeNV = 0x1515
    SpvOpSubgroupShuffleINTEL = 0x15C3
    SpvOpSubgroupShuffleDownINTEL = 0x15C4
    SpvOpSubgroupShuffleUpINTEL = 0x15C5
    SpvOpSubgroupShuffleXorINTEL = 0x15C6
    SpvOpSubgroupBlockReadINTEL = 0x15C7
    SpvOpSubgroupBlockWriteINTEL = 0x15C8
    SpvOpSubgroupImageBlockReadINTEL = 0x15C9
    SpvOpSubgroupImageBlockWriteINTEL = 0x15CA
    SpvOpSubgroupImageMediaBlockReadINTEL = 0x15CC
    SpvOpSubgroupImageMediaBlockWriteINTEL = 0x15CD
    SpvOpUCountLeadingZerosINTEL = 0x15D1
    SpvOpUCountTrailingZerosINTEL = 0x15D2
    SpvOpAbsISubINTEL = 0x15D3
    SpvOpAbsUSubINTEL = 0x15D4
    SpvOpIAddSatINTEL = 0x15D5
    SpvOpUAddSatINTEL = 0x15D6
    SpvOpIAverageINTEL = 0x15D7
    SpvOpUAverageINTEL = 0x15D8
    SpvOpIAverageRoundedINTEL = 0x15D9
    SpvOpUAverageRoundedINTEL = 0x15DA
    SpvOpISubSatINTEL = 0x15DB
    SpvOpUSubSatINTEL = 0x15DC
    SpvOpIMul32x16INTEL = 0x15DD
    SpvOpUMul32x16INTEL = 0x15DE
    SpvOpConstantFunctionPointerINTEL = 0x15E0
    SpvOpFunctionPointerCallINTEL = 0x15E1
    SpvOpAsmTargetINTEL = 0x15E9
    SpvOpAsmINTEL = 0x15EA
    SpvOpAsmCallINTEL = 0x15EB
    SpvOpAtomicFMinEXT = 0x15EE
    SpvOpAtomicFMaxEXT = 0x15EF
    SpvOpAssumeTrueKHR = 0x15FE
    SpvOpExpectKHR = 0x15FF
    SpvOpDecorateString = 0x1600
    SpvOpMemberDecorateString = 0x1601
    SpvOpVmeImageINTEL = 0x1643
    SpvOpTypeVmeImageINTEL = 0x1644
    SpvOpTypeAvcImePayloadINTEL = 0x1645
    SpvOpTypeAvcRefPayloadINTEL = 0x1646
    SpvOpTypeAvcSicPayloadINTEL = 0x1647
    SpvOpTypeAvcMcePayloadINTEL = 0x1648
    SpvOpTypeAvcMceResultINTEL = 0x1649
    SpvOpTypeAvcImeResultINTEL = 0x164A
    SpvOpTypeAvcImeResultSingleReferenceStreamoutINTEL = 0x164B
    SpvOpTypeAvcImeResultDualReferenceStreamoutINTEL = 0x164C
    SpvOpTypeAvcImeSingleReferenceStreaminINTEL = 0x164D
    SpvOpTypeAvcImeDualReferenceStreaminINTEL = 0x164E
    SpvOpTypeAvcRefResultINTEL = 0x164F
    SpvOpTypeAvcSicResultINTEL = 0x1650
    SpvOpSubgroupAvcMceGetDefaultInterBaseMultiReferencePenaltyINTEL = 0x1651
    SpvOpSubgroupAvcMceSetInterBaseMultiReferencePenaltyINTEL = 0x1652
    SpvOpSubgroupAvcMceGetDefaultInterShapePenaltyINTEL = 0x1653
    SpvOpSubgroupAvcMceSetInterShapePenaltyINTEL = 0x1654
    SpvOpSubgroupAvcMceGetDefaultInterDirectionPenaltyINTEL = 0x1655
    SpvOpSubgroupAvcMceSetInterDirectionPenaltyINTEL = 0x1656
    SpvOpSubgroupAvcMceGetDefaultIntraLumaShapePenaltyINTEL = 0x1657
    SpvOpSubgroupAvcMceGetDefaultInterMotionVectorCostTableINTEL = 0x1658
    SpvOpSubgroupAvcMceGetDefaultHighPenaltyCostTableINTEL = 0x1659
    SpvOpSubgroupAvcMceGetDefaultMediumPenaltyCostTableINTEL = 0x165A
    SpvOpSubgroupAvcMceGetDefaultLowPenaltyCostTableINTEL = 0x165B
    SpvOpSubgroupAvcMceSetMotionVectorCostFunctionINTEL = 0x165C
    SpvOpSubgroupAvcMceGetDefaultIntraLumaModePenaltyINTEL = 0x165D
    SpvOpSubgroupAvcMceGetDefaultNonDcLumaIntraPenaltyINTEL = 0x165E
    SpvOpSubgroupAvcMceGetDefaultIntraChromaModeBasePenaltyINTEL = 0x165F
    SpvOpSubgroupAvcMceSetAcOnlyHaarINTEL = 0x1660
    SpvOpSubgroupAvcMceSetSourceInterlacedFieldPolarityINTEL = 0x1661
    SpvOpSubgroupAvcMceSetSingleReferenceInterlacedFieldPolarityINTEL = 0x1662
    SpvOpSubgroupAvcMceSetDualReferenceInterlacedFieldPolaritiesINTEL = 0x1663
    SpvOpSubgroupAvcMceConvertToImePayloadINTEL = 0x1664
    SpvOpSubgroupAvcMceConvertToImeResultINTEL = 0x1665
    SpvOpSubgroupAvcMceConvertToRefPayloadINTEL = 0x1666
    SpvOpSubgroupAvcMceConvertToRefResultINTEL = 0x1667
    SpvOpSubgroupAvcMceConvertToSicPayloadINTEL = 0x1668
    SpvOpSubgroupAvcMceConvertToSicResultINTEL = 0x1669
    SpvOpSubgroupAvcMceGetMotionVectorsINTEL = 0x166A
    SpvOpSubgroupAvcMceGetInterDistortionsINTEL = 0x166B
    SpvOpSubgroupAvcMceGetBestInterDistortionsINTEL = 0x166C
    SpvOpSubgroupAvcMceGetInterMajorShapeINTEL = 0x166D
    SpvOpSubgroupAvcMceGetInterMinorShapeINTEL = 0x166E
    SpvOpSubgroupAvcMceGetInterDirectionsINTEL = 0x166F
    SpvOpSubgroupAvcMceGetInterMotionVectorCountINTEL = 0x1670
    SpvOpSubgroupAvcMceGetInterReferenceIdsINTEL = 0x1671
    SpvOpSubgroupAvcMceGetInterReferenceInterlacedFieldPolaritiesINTEL = 0x1672
    SpvOpSubgroupAvcImeInitializeINTEL = 0x1673
    SpvOpSubgroupAvcImeSetSingleReferenceINTEL = 0x1674
    SpvOpSubgroupAvcImeSetDualReferenceINTEL = 0x1675
    SpvOpSubgroupAvcImeRefWindowSizeINTEL = 0x1676
    SpvOpSubgroupAvcImeAdjustRefOffsetINTEL = 0x1677
    SpvOpSubgroupAvcImeConvertToMcePayloadINTEL = 0x1678
    SpvOpSubgroupAvcImeSetMaxMotionVectorCountINTEL = 0x1679
    SpvOpSubgroupAvcImeSetUnidirectionalMixDisableINTEL = 0x167A
    SpvOpSubgroupAvcImeSetEarlySearchTerminationThresholdINTEL = 0x167B
    SpvOpSubgroupAvcImeSetWeightedSadINTEL = 0x167C
    SpvOpSubgroupAvcImeEvaluateWithSingleReferenceINTEL = 0x167D
    SpvOpSubgroupAvcImeEvaluateWithDualReferenceINTEL = 0x167E
    SpvOpSubgroupAvcImeEvaluateWithSingleReferenceStreaminINTEL = 0x167F
    SpvOpSubgroupAvcImeEvaluateWithDualReferenceStreaminINTEL = 0x1680
    SpvOpSubgroupAvcImeEvaluateWithSingleReferenceStreamoutINTEL = 0x1681
    SpvOpSubgroupAvcImeEvaluateWithDualReferenceStreamoutINTEL = 0x1682
    SpvOpSubgroupAvcImeEvaluateWithSingleReferenceStreaminoutINTEL = 0x1683
    SpvOpSubgroupAvcImeEvaluateWithDualReferenceStreaminoutINTEL = 0x1684
    SpvOpSubgroupAvcImeConvertToMceResultINTEL = 0x1685
    SpvOpSubgroupAvcImeGetSingleReferenceStreaminINTEL = 0x1686
    SpvOpSubgroupAvcImeGetDualReferenceStreaminINTEL = 0x1687
    SpvOpSubgroupAvcImeStripSingleReferenceStreamoutINTEL = 0x1688
    SpvOpSubgroupAvcImeStripDualReferenceStreamoutINTEL = 0x1689
    SpvOpSubgroupAvcImeGetStreamoutSingleReferenceMajorShapeMotionVectorsINTEL = 0x168A
    SpvOpSubgroupAvcImeGetStreamoutSingleReferenceMajorShapeDistortionsINTEL = 0x168B
    SpvOpSubgroupAvcImeGetStreamoutSingleReferenceMajorShapeReferenceIdsINTEL = 0x168C
    SpvOpSubgroupAvcImeGetStreamoutDualReferenceMajorShapeMotionVectorsINTEL = 0x168D
    SpvOpSubgroupAvcImeGetStreamoutDualReferenceMajorShapeDistortionsINTEL = 0x168E
    SpvOpSubgroupAvcImeGetStreamoutDualReferenceMajorShapeReferenceIdsINTEL = 0x168F
    SpvOpSubgroupAvcImeGetBorderReachedINTEL = 0x1690
    SpvOpSubgroupAvcImeGetTruncatedSearchIndicationINTEL = 0x1691
    SpvOpSubgroupAvcImeGetUnidirectionalEarlySearchTerminationINTEL = 0x1692
    SpvOpSubgroupAvcImeGetWeightingPatternMinimumMotionVectorINTEL = 0x1693
    SpvOpSubgroupAvcImeGetWeightingPatternMinimumDistortionINTEL = 0x1694
    SpvOpSubgroupAvcFmeInitializeINTEL = 0x1695
    SpvOpSubgroupAvcBmeInitializeINTEL = 0x1696
    SpvOpSubgroupAvcRefConvertToMcePayloadINTEL = 0x1697
    SpvOpSubgroupAvcRefSetBidirectionalMixDisableINTEL = 0x1698
    SpvOpSubgroupAvcRefSetBilinearFilterEnableINTEL = 0x1699
    SpvOpSubgroupAvcRefEvaluateWithSingleReferenceINTEL = 0x169A
    SpvOpSubgroupAvcRefEvaluateWithDualReferenceINTEL = 0x169B
    SpvOpSubgroupAvcRefEvaluateWithMultiReferenceINTEL = 0x169C
    SpvOpSubgroupAvcRefEvaluateWithMultiReferenceInterlacedINTEL = 0x169D
    SpvOpSubgroupAvcRefConvertToMceResultINTEL = 0x169E
    SpvOpSubgroupAvcSicInitializeINTEL = 0x169F
    SpvOpSubgroupAvcSicConfigureSkcINTEL = 0x16A0
    SpvOpSubgroupAvcSicConfigureIpeLumaINTEL = 0x16A1
    SpvOpSubgroupAvcSicConfigureIpeLumaChromaINTEL = 0x16A2
    SpvOpSubgroupAvcSicGetMotionVectorMaskINTEL = 0x16A3
    SpvOpSubgroupAvcSicConvertToMcePayloadINTEL = 0x16A4
    SpvOpSubgroupAvcSicSetIntraLumaShapePenaltyINTEL = 0x16A5
    SpvOpSubgroupAvcSicSetIntraLumaModeCostFunctionINTEL = 0x16A6
    SpvOpSubgroupAvcSicSetIntraChromaModeCostFunctionINTEL = 0x16A7
    SpvOpSubgroupAvcSicSetBilinearFilterEnableINTEL = 0x16A8
    SpvOpSubgroupAvcSicSetSkcForwardTransformEnableINTEL = 0x16A9
    SpvOpSubgroupAvcSicSetBlockBasedRawSkipSadINTEL = 0x16AA
    SpvOpSubgroupAvcSicEvaluateIpeINTEL = 0x16AB
    SpvOpSubgroupAvcSicEvaluateWithSingleReferenceINTEL = 0x16AC
    SpvOpSubgroupAvcSicEvaluateWithDualReferenceINTEL = 0x16AD
    SpvOpSubgroupAvcSicEvaluateWithMultiReferenceINTEL = 0x16AE
    SpvOpSubgroupAvcSicEvaluateWithMultiReferenceInterlacedINTEL = 0x16AF
    SpvOpSubgroupAvcSicConvertToMceResultINTEL = 0x16B0
    SpvOpSubgroupAvcSicGetIpeLumaShapeINTEL = 0x16B1
    SpvOpSubgroupAvcSicGetBestIpeLumaDistortionINTEL = 0x16B2
    SpvOpSubgroupAvcSicGetBestIpeChromaDistortionINTEL = 0x16B3
    SpvOpSubgroupAvcSicGetPackedIpeLumaModesINTEL = 0x16B4
    SpvOpSubgroupAvcSicGetIpeChromaModeINTEL = 0x16B5
    SpvOpSubgroupAvcSicGetPackedSkcLumaCountThresholdINTEL = 0x16B6
    SpvOpSubgroupAvcSicGetPackedSkcLumaSumThresholdINTEL = 0x16B7
    SpvOpSubgroupAvcSicGetInterRawSadsINTEL = 0x16B8
    SpvOpVariableLengthArrayINTEL = 0x16BA
    SpvOpSaveMemoryINTEL = 0x16BB
    SpvOpRestoreMemoryINTEL = 0x16BC
    SpvOpArbitraryFloatSinCosPiINTEL = 0x16D0
    SpvOpArbitraryFloatCastINTEL = 0x16D1
    SpvOpArbitraryFloatCastFromIntINTEL = 0x16D2
    SpvOpArbitraryFloatCastToIntINTEL = 0x16D3
    SpvOpArbitraryFloatAddINTEL = 0x16D6
    SpvOpArbitraryFloatSubINTEL = 0x16D7
    SpvOpArbitraryFloatMulINTEL = 0x16D8
    SpvOpArbitraryFloatDivINTEL = 0x16D9
    SpvOpArbitraryFloatGTINTEL = 0x16DA
    SpvOpArbitraryFloatGEINTEL = 0x16DB
    SpvOpArbitraryFloatLTINTEL = 0x16DC
    SpvOpArbitraryFloatLEINTEL = 0x16DD
    SpvOpArbitraryFloatEQINTEL = 0x16DE
    SpvOpArbitraryFloatRecipINTEL = 0x16DF
    SpvOpArbitraryFloatRSqrtINTEL = 0x16E0
    SpvOpArbitraryFloatCbrtINTEL = 0x16E1
    SpvOpArbitraryFloatHypotINTEL = 0x16E2
    SpvOpArbitraryFloatSqrtINTEL = 0x16E3
    SpvOpArbitraryFloatLogINTEL = 0x16E4
    SpvOpArbitraryFloatLog2INTEL = 0x16E5
    SpvOpArbitraryFloatLog10INTEL = 0x16E6
    SpvOpArbitraryFloatLog1pINTEL = 0x16E7
    SpvOpArbitraryFloatExpINTEL = 0x16E8
    SpvOpArbitraryFloatExp2INTEL = 0x16E9
    SpvOpArbitraryFloatExp10INTEL = 0x16EA
    SpvOpArbitraryFloatExpm1INTEL = 0x16EB
    SpvOpArbitraryFloatSinINTEL = 0x16EC
    SpvOpArbitraryFloatCosINTEL = 0x16ED
    SpvOpArbitraryFloatSinCosINTEL = 0x16EE
    SpvOpArbitraryFloatSinPiINTEL = 0x16EF
    SpvOpArbitraryFloatCosPiINTEL = 0x16F0
    SpvOpArbitraryFloatASinINTEL = 0x16F1
    SpvOpArbitraryFloatASinPiINTEL = 0x16F2
    SpvOpArbitraryFloatACosINTEL = 0x16F3
    SpvOpArbitraryFloatACosPiINTEL = 0x16F4
    SpvOpArbitraryFloatATanINTEL = 0x16F5
    SpvOpArbitraryFloatATanPiINTEL = 0x16F6
    SpvOpArbitraryFloatATan2INTEL = 0x16F7
    SpvOpArbitraryFloatPowINTEL = 0x16F8
    SpvOpArbitraryFloatPowRINTEL = 0x16F9
    SpvOpArbitraryFloatPowNINTEL = 0x16FA
    SpvOpLoopControlINTEL = 0x16FF
    SpvOpAliasDomainDeclINTEL = 0x1717
    SpvOpAliasScopeDeclINTEL = 0x1718
    SpvOpAliasScopeListDeclINTEL = 0x1719
    SpvOpFixedSqrtINTEL = 0x1723
    SpvOpFixedRecipINTEL = 0x1724
    SpvOpFixedRsqrtINTEL = 0x1725
    SpvOpFixedSinINTEL = 0x1726
    SpvOpFixedCosINTEL = 0x1727
    SpvOpFixedSinCosINTEL = 0x1728
    SpvOpFixedSinPiINTEL = 0x1729
    SpvOpFixedCosPiINTEL = 0x172A
    SpvOpFixedSinCosPiINTEL = 0x172B
    SpvOpFixedLogINTEL = 0x172C
    SpvOpFixedExpINTEL = 0x172D
    SpvOpPtrCastToCrossWorkgroupINTEL = 0x172E
    SpvOpCrossWorkgroupCastToPtrINTEL = 0x1732
    SpvOpReadPipeBlockingINTEL = 0x173A
    SpvOpWritePipeBlockingINTEL = 0x173B
    SpvOpFPGARegINTEL = 0x173D
    SpvOpRayQueryGetRayTMinKHR = 0x1780
    SpvOpRayQueryGetRayFlagsKHR = 0x1781
    SpvOpRayQueryGetIntersectionTKHR = 0x1782
    SpvOpRayQueryGetIntersectionInstanceCustomIndexKHR = 0x1783
    SpvOpRayQueryGetIntersectionInstanceIdKHR = 0x1784
    SpvOpRayQueryGetIntersectionInstanceShaderBindingTableRecordOffsetKHR = 0x1785
    SpvOpRayQueryGetIntersectionGeometryIndexKHR = 0x1786
    SpvOpRayQueryGetIntersectionPrimitiveIndexKHR = 0x1787
    SpvOpRayQueryGetIntersectionBarycentricsKHR = 0x1788
    SpvOpRayQueryGetIntersectionFrontFaceKHR = 0x1789
    SpvOpRayQueryGetIntersectionCandidateAABBOpaqueKHR = 0x178A
    SpvOpRayQueryGetIntersectionObjectRayDirectionKHR = 0x178B
    SpvOpRayQueryGetIntersectionObjectRayOriginKHR = 0x178C
    SpvOpRayQueryGetWorldRayDirectionKHR = 0x178D
    SpvOpRayQueryGetWorldRayOriginKHR = 0x178E
    SpvOpRayQueryGetIntersectionObjectToWorldKHR = 0x178F
    SpvOpRayQueryGetIntersectionWorldToObjectKHR = 0x1790
    SpvOpAtomicFAddEXT = 0x1793
    SpvOpTypeBufferSurfaceINTEL = 0x17C6
    SpvOpTypeStructContinuedINTEL = 0x17CA
    SpvOpConstantCompositeContinuedINTEL = 0x17CB
    SpvOpSpecConstantCompositeContinuedINTEL = 0x17CC
    SpvOpControlBarrierArriveINTEL = 0x17FE
    SpvOpControlBarrierWaitINTEL = 0x17FF
    SpvOpGroupIMulKHR = 0x1901
    SpvOpGroupFMulKHR = 0x1902
    SpvOpGroupBitwiseAndKHR = 0x1903
    SpvOpGroupBitwiseOrKHR = 0x1904
    SpvOpGroupBitwiseXorKHR = 0x1905
    SpvOpGroupLogicalAndKHR = 0x1906
    SpvOpGroupLogicalOrKHR = 0x1907
    SpvOpGroupLogicalXorKHR = 0x1908
    SpvOpMax = 0x7FFFFFFF

class SpvReflectResult(IntEnum):
    SPV_REFLECT_RESULT_SUCCESS = 0x0
    SPV_REFLECT_RESULT_NOT_READY = 0x1
    SPV_REFLECT_RESULT_ERROR_PARSE_FAILED = 0x2
    SPV_REFLECT_RESULT_ERROR_ALLOC_FAILED = 0x3
    SPV_REFLECT_RESULT_ERROR_RANGE_EXCEEDED = 0x4
    SPV_REFLECT_RESULT_ERROR_NULL_POINTER = 0x5
    SPV_REFLECT_RESULT_ERROR_INTERNAL_ERROR = 0x6
    SPV_REFLECT_RESULT_ERROR_COUNT_MISMATCH = 0x7
    SPV_REFLECT_RESULT_ERROR_ELEMENT_NOT_FOUND = 0x8
    SPV_REFLECT_RESULT_ERROR_SPIRV_INVALID_CODE_SIZE = 0x9
    SPV_REFLECT_RESULT_ERROR_SPIRV_INVALID_MAGIC_NUMBER = 0xA
    SPV_REFLECT_RESULT_ERROR_SPIRV_UNEXPECTED_EOF = 0xB
    SPV_REFLECT_RESULT_ERROR_SPIRV_INVALID_ID_REFERENCE = 0xC
    SPV_REFLECT_RESULT_ERROR_SPIRV_SET_NUMBER_OVERFLOW = 0xD
    SPV_REFLECT_RESULT_ERROR_SPIRV_INVALID_STORAGE_CLASS = 0xE
    SPV_REFLECT_RESULT_ERROR_SPIRV_RECURSION = 0xF
    SPV_REFLECT_RESULT_ERROR_SPIRV_INVALID_INSTRUCTION = 0x10
    SPV_REFLECT_RESULT_ERROR_SPIRV_UNEXPECTED_BLOCK_DATA = 0x11
    SPV_REFLECT_RESULT_ERROR_SPIRV_INVALID_BLOCK_MEMBER_REFERENCE = 0x12
    SPV_REFLECT_RESULT_ERROR_SPIRV_INVALID_ENTRY_POINT = 0x13
    SPV_REFLECT_RESULT_ERROR_SPIRV_INVALID_EXECUTION_MODE = 0x14

class SpvReflectModuleFlagBits(IntEnum):
    SPV_REFLECT_MODULE_FLAG_NONE = 0x0
    SPV_REFLECT_MODULE_FLAG_NO_COPY = 0x1

class SpvReflectTypeFlagBits(IntEnum):
    SPV_REFLECT_TYPE_FLAG_UNDEFINED = 0x0
    SPV_REFLECT_TYPE_FLAG_VOID = 0x1
    SPV_REFLECT_TYPE_FLAG_BOOL = 0x2
    SPV_REFLECT_TYPE_FLAG_INT = 0x4
    SPV_REFLECT_TYPE_FLAG_FLOAT = 0x8
    SPV_REFLECT_TYPE_FLAG_VECTOR = 0x100
    SPV_REFLECT_TYPE_FLAG_MATRIX = 0x200
    SPV_REFLECT_TYPE_FLAG_EXTERNAL_IMAGE = 0x10000
    SPV_REFLECT_TYPE_FLAG_EXTERNAL_SAMPLER = 0x20000
    SPV_REFLECT_TYPE_FLAG_EXTERNAL_SAMPLED_IMAGE = 0x40000
    SPV_REFLECT_TYPE_FLAG_EXTERNAL_BLOCK = 0x80000
    SPV_REFLECT_TYPE_FLAG_EXTERNAL_ACCELERATION_STRUCTURE = 0x100000
    SPV_REFLECT_TYPE_FLAG_EXTERNAL_MASK = 0xFF0000
    SPV_REFLECT_TYPE_FLAG_STRUCT = 0x10000000
    SPV_REFLECT_TYPE_FLAG_ARRAY = 0x20000000

class SpvReflectDescriptorType(IntEnum):
    SPV_REFLECT_DESCRIPTOR_TYPE_SAMPLER = 0x0
    SPV_REFLECT_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER = 0x1
    SPV_REFLECT_DESCRIPTOR_TYPE_SAMPLED_IMAGE = 0x2
    SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_IMAGE = 0x3
    SPV_REFLECT_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER = 0x4
    SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER = 0x5
    SPV_REFLECT_DESCRIPTOR_TYPE_UNIFORM_BUFFER = 0x6
    SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_BUFFER = 0x7
    SPV_REFLECT_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC = 0x8
    SPV_REFLECT_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC = 0x9
    SPV_REFLECT_DESCRIPTOR_TYPE_INPUT_ATTACHMENT = 0xA
    SPV_REFLECT_DESCRIPTOR_TYPE_ACCELERATION_STRUCTURE_KHR = 0x3B9D13F0

class SpvSourceLanguage_(IntEnum):
    SpvSourceLanguageUnknown = 0x0
    SpvSourceLanguageESSL = 0x1
    SpvSourceLanguageGLSL = 0x2
    SpvSourceLanguageOpenCL_C = 0x3
    SpvSourceLanguageOpenCL_CPP = 0x4
    SpvSourceLanguageHLSL = 0x5
    SpvSourceLanguageCPP_for_OpenCL = 0x6
    SpvSourceLanguageSYCL = 0x7
    SpvSourceLanguageMax = 0x7FFFFFFF

class SpvExecutionModel_(IntEnum):
    SpvExecutionModelVertex = 0x0
    SpvExecutionModelTessellationControl = 0x1
    SpvExecutionModelTessellationEvaluation = 0x2
    SpvExecutionModelGeometry = 0x3
    SpvExecutionModelFragment = 0x4
    SpvExecutionModelGLCompute = 0x5
    SpvExecutionModelKernel = 0x6
    SpvExecutionModelTaskNV = 0x1493
    SpvExecutionModelMeshNV = 0x1494
    SpvExecutionModelRayGenerationKHR = 0x14C1
    SpvExecutionModelIntersectionKHR = 0x14C2
    SpvExecutionModelAnyHitKHR = 0x14C3
    SpvExecutionModelClosestHitKHR = 0x14C4
    SpvExecutionModelMissKHR = 0x14C5
    SpvExecutionModelCallableKHR = 0x14C6
    SpvExecutionModelTaskEXT = 0x14F4
    SpvExecutionModelMeshEXT = 0x14F5
    SpvExecutionModelMax = 0x7FFFFFFF

class SpvStorageClass_(IntEnum):
    SpvStorageClassUniformConstant = 0x0
    SpvStorageClassInput = 0x1
    SpvStorageClassUniform = 0x2
    SpvStorageClassOutput = 0x3
    SpvStorageClassWorkgroup = 0x4
    SpvStorageClassCrossWorkgroup = 0x5
    SpvStorageClassPrivate = 0x6
    SpvStorageClassFunction = 0x7
    SpvStorageClassGeneric = 0x8
    SpvStorageClassPushConstant = 0x9
    SpvStorageClassAtomicCounter = 0xA
    SpvStorageClassImage = 0xB
    SpvStorageClassStorageBuffer = 0xC
    SpvStorageClassCallableDataKHR = 0x14D0
    SpvStorageClassIncomingCallableDataKHR = 0x14D1
    SpvStorageClassRayPayloadKHR = 0x14DA
    SpvStorageClassHitAttributeKHR = 0x14DB
    SpvStorageClassIncomingRayPayloadKHR = 0x14DE
    SpvStorageClassShaderRecordBufferKHR = 0x14DF
    SpvStorageClassPhysicalStorageBuffer = 0x14E5
    SpvStorageClassHitObjectAttributeNV = 0x1509
    SpvStorageClassTaskPayloadWorkgroupEXT = 0x151A
    SpvStorageClassCodeSectionINTEL = 0x15E5
    SpvStorageClassDeviceOnlyINTEL = 0x1730
    SpvStorageClassHostOnlyINTEL = 0x1731
    SpvStorageClassMax = 0x7FFFFFFF

class SpvDim_(IntEnum):
    SpvDim1D = 0x0
    SpvDim2D = 0x1
    SpvDim3D = 0x2
    SpvDimCube = 0x3
    SpvDimRect = 0x4
    SpvDimBuffer = 0x5
    SpvDimSubpassData = 0x6
    SpvDimMax = 0x7FFFFFFF

class SpvImageFormat_(IntEnum):
    SpvImageFormatUnknown = 0x0
    SpvImageFormatRgba32f = 0x1
    SpvImageFormatRgba16f = 0x2
    SpvImageFormatR32f = 0x3
    SpvImageFormatRgba8 = 0x4
    SpvImageFormatRgba8Snorm = 0x5
    SpvImageFormatRg32f = 0x6
    SpvImageFormatRg16f = 0x7
    SpvImageFormatR11fG11fB10f = 0x8
    SpvImageFormatR16f = 0x9
    SpvImageFormatRgba16 = 0xA
    SpvImageFormatRgb10A2 = 0xB
    SpvImageFormatRg16 = 0xC
    SpvImageFormatRg8 = 0xD
    SpvImageFormatR16 = 0xE
    SpvImageFormatR8 = 0xF
    SpvImageFormatRgba16Snorm = 0x10
    SpvImageFormatRg16Snorm = 0x11
    SpvImageFormatRg8Snorm = 0x12
    SpvImageFormatR16Snorm = 0x13
    SpvImageFormatR8Snorm = 0x14
    SpvImageFormatRgba32i = 0x15
    SpvImageFormatRgba16i = 0x16
    SpvImageFormatRgba8i = 0x17
    SpvImageFormatR32i = 0x18
    SpvImageFormatRg32i = 0x19
    SpvImageFormatRg16i = 0x1A
    SpvImageFormatRg8i = 0x1B
    SpvImageFormatR16i = 0x1C
    SpvImageFormatR8i = 0x1D
    SpvImageFormatRgba32ui = 0x1E
    SpvImageFormatRgba16ui = 0x1F
    SpvImageFormatRgba8ui = 0x20
    SpvImageFormatR32ui = 0x21
    SpvImageFormatRgb10a2ui = 0x22
    SpvImageFormatRg32ui = 0x23
    SpvImageFormatRg16ui = 0x24
    SpvImageFormatRg8ui = 0x25
    SpvImageFormatR16ui = 0x26
    SpvImageFormatR8ui = 0x27
    SpvImageFormatR64ui = 0x28
    SpvImageFormatR64i = 0x29
    SpvImageFormatMax = 0x7FFFFFFF

class SpvBuiltIn_(IntEnum):
    SpvBuiltInPosition = 0x0
    SpvBuiltInPointSize = 0x1
    SpvBuiltInClipDistance = 0x3
    SpvBuiltInCullDistance = 0x4
    SpvBuiltInVertexId = 0x5
    SpvBuiltInInstanceId = 0x6
    SpvBuiltInPrimitiveId = 0x7
    SpvBuiltInInvocationId = 0x8
    SpvBuiltInLayer = 0x9
    SpvBuiltInViewportIndex = 0xA
    SpvBuiltInTessLevelOuter = 0xB
    SpvBuiltInTessLevelInner = 0xC
    SpvBuiltInTessCoord = 0xD
    SpvBuiltInPatchVertices = 0xE
    SpvBuiltInFragCoord = 0xF
    SpvBuiltInPointCoord = 0x10
    SpvBuiltInFrontFacing = 0x11
    SpvBuiltInSampleId = 0x12
    SpvBuiltInSamplePosition = 0x13
    SpvBuiltInSampleMask = 0x14
    SpvBuiltInFragDepth = 0x16
    SpvBuiltInHelperInvocation = 0x17
    SpvBuiltInNumWorkgroups = 0x18
    SpvBuiltInWorkgroupSize = 0x19
    SpvBuiltInWorkgroupId = 0x1A
    SpvBuiltInLocalInvocationId = 0x1B
    SpvBuiltInGlobalInvocationId = 0x1C
    SpvBuiltInLocalInvocationIndex = 0x1D
    SpvBuiltInWorkDim = 0x1E
    SpvBuiltInGlobalSize = 0x1F
    SpvBuiltInEnqueuedWorkgroupSize = 0x20
    SpvBuiltInGlobalOffset = 0x21
    SpvBuiltInGlobalLinearId = 0x22
    SpvBuiltInSubgroupSize = 0x24
    SpvBuiltInSubgroupMaxSize = 0x25
    SpvBuiltInNumSubgroups = 0x26
    SpvBuiltInNumEnqueuedSubgroups = 0x27
    SpvBuiltInSubgroupId = 0x28
    SpvBuiltInSubgroupLocalInvocationId = 0x29
    SpvBuiltInVertexIndex = 0x2A
    SpvBuiltInInstanceIndex = 0x2B
    SpvBuiltInCoreIDARM = 0x1040
    SpvBuiltInCoreCountARM = 0x1041
    SpvBuiltInCoreMaxIDARM = 0x1042
    SpvBuiltInWarpIDARM = 0x1043
    SpvBuiltInWarpMaxIDARM = 0x1044
    SpvBuiltInSubgroupEqMask = 0x1140
    SpvBuiltInSubgroupGeMask = 0x1141
    SpvBuiltInSubgroupGtMask = 0x1142
    SpvBuiltInSubgroupLeMask = 0x1143
    SpvBuiltInSubgroupLtMask = 0x1144
    SpvBuiltInBaseVertex = 0x1148
    SpvBuiltInBaseInstance = 0x1149
    SpvBuiltInDrawIndex = 0x114A
    SpvBuiltInPrimitiveShadingRateKHR = 0x1150
    SpvBuiltInDeviceIndex = 0x1156
    SpvBuiltInViewIndex = 0x1158
    SpvBuiltInShadingRateKHR = 0x115C
    SpvBuiltInBaryCoordNoPerspAMD = 0x1380
    SpvBuiltInBaryCoordNoPerspCentroidAMD = 0x1381
    SpvBuiltInBaryCoordNoPerspSampleAMD = 0x1382
    SpvBuiltInBaryCoordSmoothAMD = 0x1383
    SpvBuiltInBaryCoordSmoothCentroidAMD = 0x1384
    SpvBuiltInBaryCoordSmoothSampleAMD = 0x1385
    SpvBuiltInBaryCoordPullModelAMD = 0x1386
    SpvBuiltInFragStencilRefEXT = 0x1396
    SpvBuiltInViewportMaskNV = 0x1485
    SpvBuiltInSecondaryPositionNV = 0x1489
    SpvBuiltInSecondaryViewportMaskNV = 0x148A
    SpvBuiltInPositionPerViewNV = 0x148D
    SpvBuiltInViewportMaskPerViewNV = 0x148E
    SpvBuiltInFullyCoveredEXT = 0x1490
    SpvBuiltInTaskCountNV = 0x149A
    SpvBuiltInPrimitiveCountNV = 0x149B
    SpvBuiltInPrimitiveIndicesNV = 0x149C
    SpvBuiltInClipDistancePerViewNV = 0x149D
    SpvBuiltInCullDistancePerViewNV = 0x149E
    SpvBuiltInLayerPerViewNV = 0x149F
    SpvBuiltInMeshViewCountNV = 0x14A0
    SpvBuiltInMeshViewIndicesNV = 0x14A1
    SpvBuiltInBaryCoordKHR = 0x14A6
    SpvBuiltInBaryCoordNoPerspKHR = 0x14A7
    SpvBuiltInFragSizeEXT = 0x14AC
    SpvBuiltInFragInvocationCountEXT = 0x14AD
    SpvBuiltInPrimitivePointIndicesEXT = 0x14AE
    SpvBuiltInPrimitiveLineIndicesEXT = 0x14AF
    SpvBuiltInPrimitiveTriangleIndicesEXT = 0x14B0
    SpvBuiltInCullPrimitiveEXT = 0x14B3
    SpvBuiltInLaunchIdKHR = 0x14C7
    SpvBuiltInLaunchSizeKHR = 0x14C8
    SpvBuiltInWorldRayOriginKHR = 0x14C9
    SpvBuiltInWorldRayDirectionKHR = 0x14CA
    SpvBuiltInObjectRayOriginKHR = 0x14CB
    SpvBuiltInObjectRayDirectionKHR = 0x14CC
    SpvBuiltInRayTminKHR = 0x14CD
    SpvBuiltInRayTmaxKHR = 0x14CE
    SpvBuiltInInstanceCustomIndexKHR = 0x14CF
    SpvBuiltInObjectToWorldKHR = 0x14D2
    SpvBuiltInWorldToObjectKHR = 0x14D3
    SpvBuiltInHitTNV = 0x14D4
    SpvBuiltInHitKindKHR = 0x14D5
    SpvBuiltInCurrentRayTimeNV = 0x14D6
    SpvBuiltInIncomingRayFlagsKHR = 0x14E7
    SpvBuiltInRayGeometryIndexKHR = 0x14E8
    SpvBuiltInWarpsPerSMNV = 0x14FE
    SpvBuiltInSMCountNV = 0x14FF
    SpvBuiltInWarpIDNV = 0x1500
    SpvBuiltInSMIDNV = 0x1501
    SpvBuiltInCullMaskKHR = 0x1785
    SpvBuiltInMax = 0x7FFFFFFF

class SpvCapability_(IntEnum):
    SpvCapabilityMatrix = 0x0
    SpvCapabilityShader = 0x1
    SpvCapabilityGeometry = 0x2
    SpvCapabilityTessellation = 0x3
    SpvCapabilityAddresses = 0x4
    SpvCapabilityLinkage = 0x5
    SpvCapabilityKernel = 0x6
    SpvCapabilityVector16 = 0x7
    SpvCapabilityFloat16Buffer = 0x8
    SpvCapabilityFloat16 = 0x9
    SpvCapabilityFloat64 = 0xA
    SpvCapabilityInt64 = 0xB
    SpvCapabilityInt64Atomics = 0xC
    SpvCapabilityImageBasic = 0xD
    SpvCapabilityImageReadWrite = 0xE
    SpvCapabilityImageMipmap = 0xF
    SpvCapabilityPipes = 0x11
    SpvCapabilityGroups = 0x12
    SpvCapabilityDeviceEnqueue = 0x13
    SpvCapabilityLiteralSampler = 0x14
    SpvCapabilityAtomicStorage = 0x15
    SpvCapabilityInt16 = 0x16
    SpvCapabilityTessellationPointSize = 0x17
    SpvCapabilityGeometryPointSize = 0x18
    SpvCapabilityImageGatherExtended = 0x19
    SpvCapabilityStorageImageMultisample = 0x1B
    SpvCapabilityUniformBufferArrayDynamicIndexing = 0x1C
    SpvCapabilitySampledImageArrayDynamicIndexing = 0x1D
    SpvCapabilityStorageBufferArrayDynamicIndexing = 0x1E
    SpvCapabilityStorageImageArrayDynamicIndexing = 0x1F
    SpvCapabilityClipDistance = 0x20
    SpvCapabilityCullDistance = 0x21
    SpvCapabilityImageCubeArray = 0x22
    SpvCapabilitySampleRateShading = 0x23
    SpvCapabilityImageRect = 0x24
    SpvCapabilitySampledRect = 0x25
    SpvCapabilityGenericPointer = 0x26
    SpvCapabilityInt8 = 0x27
    SpvCapabilityInputAttachment = 0x28
    SpvCapabilitySparseResidency = 0x29
    SpvCapabilityMinLod = 0x2A
    SpvCapabilitySampled1D = 0x2B
    SpvCapabilityImage1D = 0x2C
    SpvCapabilitySampledCubeArray = 0x2D
    SpvCapabilitySampledBuffer = 0x2E
    SpvCapabilityImageBuffer = 0x2F
    SpvCapabilityImageMSArray = 0x30
    SpvCapabilityStorageImageExtendedFormats = 0x31
    SpvCapabilityImageQuery = 0x32
    SpvCapabilityDerivativeControl = 0x33
    SpvCapabilityInterpolationFunction = 0x34
    SpvCapabilityTransformFeedback = 0x35
    SpvCapabilityGeometryStreams = 0x36
    SpvCapabilityStorageImageReadWithoutFormat = 0x37
    SpvCapabilityStorageImageWriteWithoutFormat = 0x38
    SpvCapabilityMultiViewport = 0x39
    SpvCapabilitySubgroupDispatch = 0x3A
    SpvCapabilityNamedBarrier = 0x3B
    SpvCapabilityPipeStorage = 0x3C
    SpvCapabilityGroupNonUniform = 0x3D
    SpvCapabilityGroupNonUniformVote = 0x3E
    SpvCapabilityGroupNonUniformArithmetic = 0x3F
    SpvCapabilityGroupNonUniformBallot = 0x40
    SpvCapabilityGroupNonUniformShuffle = 0x41
    SpvCapabilityGroupNonUniformShuffleRelative = 0x42
    SpvCapabilityGroupNonUniformClustered = 0x43
    SpvCapabilityGroupNonUniformQuad = 0x44
    SpvCapabilityShaderLayer = 0x45
    SpvCapabilityShaderViewportIndex = 0x46
    SpvCapabilityUniformDecoration = 0x47
    SpvCapabilityCoreBuiltinsARM = 0x1045
    SpvCapabilityFragmentShadingRateKHR = 0x1146
    SpvCapabilitySubgroupBallotKHR = 0x1147
    SpvCapabilityDrawParameters = 0x114B
    SpvCapabilityWorkgroupMemoryExplicitLayoutKHR = 0x114C
    SpvCapabilityWorkgroupMemoryExplicitLayout8BitAccessKHR = 0x114D
    SpvCapabilityWorkgroupMemoryExplicitLayout16BitAccessKHR = 0x114E
    SpvCapabilitySubgroupVoteKHR = 0x114F
    SpvCapabilityStorageBuffer16BitAccess = 0x1151
    SpvCapabilityStorageUniform16 = 0x1152
    SpvCapabilityStoragePushConstant16 = 0x1153
    SpvCapabilityStorageInputOutput16 = 0x1154
    SpvCapabilityDeviceGroup = 0x1155
    SpvCapabilityMultiView = 0x1157
    SpvCapabilityVariablePointersStorageBuffer = 0x1159
    SpvCapabilityVariablePointers = 0x115A
    SpvCapabilityAtomicStorageOps = 0x115D
    SpvCapabilitySampleMaskPostDepthCoverage = 0x115F
    SpvCapabilityStorageBuffer8BitAccess = 0x1160
    SpvCapabilityUniformAndStorageBuffer8BitAccess = 0x1161
    SpvCapabilityStoragePushConstant8 = 0x1162
    SpvCapabilityDenormPreserve = 0x1170
    SpvCapabilityDenormFlushToZero = 0x1171
    SpvCapabilitySignedZeroInfNanPreserve = 0x1172
    SpvCapabilityRoundingModeRTE = 0x1173
    SpvCapabilityRoundingModeRTZ = 0x1174
    SpvCapabilityRayQueryProvisionalKHR = 0x1177
    SpvCapabilityRayQueryKHR = 0x1178
    SpvCapabilityRayTraversalPrimitiveCullingKHR = 0x117E
    SpvCapabilityRayTracingKHR = 0x117F
    SpvCapabilityFloat16ImageAMD = 0x1390
    SpvCapabilityImageGatherBiasLodAMD = 0x1391
    SpvCapabilityFragmentMaskAMD = 0x1392
    SpvCapabilityStencilExportEXT = 0x1395
    SpvCapabilityImageReadWriteLodAMD = 0x1397
    SpvCapabilityInt64ImageEXT = 0x1398
    SpvCapabilityShaderClockKHR = 0x13BF
    SpvCapabilitySampleMaskOverrideCoverageNV = 0x1481
    SpvCapabilityGeometryShaderPassthroughNV = 0x1483
    SpvCapabilityShaderViewportIndexLayerEXT = 0x1486
    SpvCapabilityShaderViewportMaskNV = 0x1487
    SpvCapabilityShaderStereoViewNV = 0x148B
    SpvCapabilityPerViewAttributesNV = 0x148C
    SpvCapabilityFragmentFullyCoveredEXT = 0x1491
    SpvCapabilityMeshShadingNV = 0x1492
    SpvCapabilityImageFootprintNV = 0x14A2
    SpvCapabilityMeshShadingEXT = 0x14A3
    SpvCapabilityFragmentBarycentricKHR = 0x14A4
    SpvCapabilityComputeDerivativeGroupQuadsNV = 0x14A8
    SpvCapabilityFragmentDensityEXT = 0x14AB
    SpvCapabilityGroupNonUniformPartitionedNV = 0x14B1
    SpvCapabilityShaderNonUniform = 0x14B5
    SpvCapabilityRuntimeDescriptorArray = 0x14B6
    SpvCapabilityInputAttachmentArrayDynamicIndexing = 0x14B7
    SpvCapabilityUniformTexelBufferArrayDynamicIndexing = 0x14B8
    SpvCapabilityStorageTexelBufferArrayDynamicIndexing = 0x14B9
    SpvCapabilityUniformBufferArrayNonUniformIndexing = 0x14BA
    SpvCapabilitySampledImageArrayNonUniformIndexing = 0x14BB
    SpvCapabilityStorageBufferArrayNonUniformIndexing = 0x14BC
    SpvCapabilityStorageImageArrayNonUniformIndexing = 0x14BD
    SpvCapabilityInputAttachmentArrayNonUniformIndexing = 0x14BE
    SpvCapabilityUniformTexelBufferArrayNonUniformIndexing = 0x14BF
    SpvCapabilityStorageTexelBufferArrayNonUniformIndexing = 0x14C0
    SpvCapabilityRayTracingNV = 0x14DC
    SpvCapabilityRayTracingMotionBlurNV = 0x14DD
    SpvCapabilityVulkanMemoryModel = 0x14E1
    SpvCapabilityVulkanMemoryModelDeviceScope = 0x14E2
    SpvCapabilityPhysicalStorageBufferAddresses = 0x14E3
    SpvCapabilityComputeDerivativeGroupLinearNV = 0x14E6
    SpvCapabilityRayTracingProvisionalKHR = 0x14E9
    SpvCapabilityCooperativeMatrixNV = 0x14ED
    SpvCapabilityFragmentShaderSampleInterlockEXT = 0x14F3
    SpvCapabilityFragmentShaderShadingRateInterlockEXT = 0x14FC
    SpvCapabilityShaderSMBuiltinsNV = 0x14FD
    SpvCapabilityFragmentShaderPixelInterlockEXT = 0x1502
    SpvCapabilityDemoteToHelperInvocation = 0x1503
    SpvCapabilityRayTracingOpacityMicromapEXT = 0x1505
    SpvCapabilityShaderInvocationReorderNV = 0x1507
    SpvCapabilityBindlessTextureNV = 0x150E
    SpvCapabilitySubgroupShuffleINTEL = 0x15C0
    SpvCapabilitySubgroupBufferBlockIOINTEL = 0x15C1
    SpvCapabilitySubgroupImageBlockIOINTEL = 0x15C2
    SpvCapabilitySubgroupImageMediaBlockIOINTEL = 0x15CB
    SpvCapabilityRoundToInfinityINTEL = 0x15CE
    SpvCapabilityFloatingPointModeINTEL = 0x15CF
    SpvCapabilityIntegerFunctions2INTEL = 0x15D0
    SpvCapabilityFunctionPointersINTEL = 0x15E3
    SpvCapabilityIndirectReferencesINTEL = 0x15E4
    SpvCapabilityAsmINTEL = 0x15E6
    SpvCapabilityAtomicFloat32MinMaxEXT = 0x15EC
    SpvCapabilityAtomicFloat64MinMaxEXT = 0x15ED
    SpvCapabilityAtomicFloat16MinMaxEXT = 0x15F0
    SpvCapabilityVectorComputeINTEL = 0x15F1
    SpvCapabilityVectorAnyINTEL = 0x15F3
    SpvCapabilityExpectAssumeKHR = 0x15FD
    SpvCapabilitySubgroupAvcMotionEstimationINTEL = 0x1640
    SpvCapabilitySubgroupAvcMotionEstimationIntraINTEL = 0x1641
    SpvCapabilitySubgroupAvcMotionEstimationChromaINTEL = 0x1642
    SpvCapabilityVariableLengthArrayINTEL = 0x16B9
    SpvCapabilityFunctionFloatControlINTEL = 0x16BD
    SpvCapabilityFPGAMemoryAttributesINTEL = 0x16C0
    SpvCapabilityFPFastMathModeINTEL = 0x16CD
    SpvCapabilityArbitraryPrecisionIntegersINTEL = 0x16D4
    SpvCapabilityArbitraryPrecisionFloatingPointINTEL = 0x16D5
    SpvCapabilityUnstructuredLoopControlsINTEL = 0x16FE
    SpvCapabilityFPGALoopControlsINTEL = 0x1700
    SpvCapabilityKernelAttributesINTEL = 0x1704
    SpvCapabilityFPGAKernelAttributesINTEL = 0x1709
    SpvCapabilityFPGAMemoryAccessesINTEL = 0x170A
    SpvCapabilityFPGAClusterAttributesINTEL = 0x1710
    SpvCapabilityLoopFuseINTEL = 0x1712
    SpvCapabilityFPGADSPControlINTEL = 0x1714
    SpvCapabilityMemoryAccessAliasingINTEL = 0x1716
    SpvCapabilityFPGAInvocationPipeliningAttributesINTEL = 0x171C
    SpvCapabilityFPGABufferLocationINTEL = 0x1720
    SpvCapabilityArbitraryPrecisionFixedPointINTEL = 0x1722
    SpvCapabilityUSMStorageClassesINTEL = 0x172F
    SpvCapabilityRuntimeAlignedAttributeINTEL = 0x1733
    SpvCapabilityIOPipesINTEL = 0x1737
    SpvCapabilityBlockingPipesINTEL = 0x1739
    SpvCapabilityFPGARegINTEL = 0x173C
    SpvCapabilityDotProductInputAll = 0x1780
    SpvCapabilityDotProductInput4x8Bit = 0x1781
    SpvCapabilityDotProductInput4x8BitPacked = 0x1782
    SpvCapabilityDotProduct = 0x1783
    SpvCapabilityRayCullMaskKHR = 0x1784
    SpvCapabilityBitInstructions = 0x1789
    SpvCapabilityGroupNonUniformRotateKHR = 0x178A
    SpvCapabilityAtomicFloat32AddEXT = 0x1791
    SpvCapabilityAtomicFloat64AddEXT = 0x1792
    SpvCapabilityLongConstantCompositeINTEL = 0x17C9
    SpvCapabilityOptNoneINTEL = 0x17CE
    SpvCapabilityAtomicFloat16AddEXT = 0x17CF
    SpvCapabilityDebugInfoModuleINTEL = 0x17E2
    SpvCapabilitySplitBarrierINTEL = 0x17FD
    SpvCapabilityGroupUniformArithmeticKHR = 0x1900
    SpvCapabilityMax = 0x7FFFFFFF

class SpvReflectShaderStageFlagBits(IntEnum):
    SPV_REFLECT_SHADER_STAGE_VERTEX_BIT = 0x1

class SpvReflectGenerator(IntEnum):
    SPV_REFLECT_GENERATOR_KHRONOS_LLVM_SPIRV_TRANSLATOR = 0x6
    SPV_REFLECT_GENERATOR_KHRONOS_SPIRV_TOOLS_ASSEMBLER = 0x7
    SPV_REFLECT_GENERATOR_KHRONOS_GLSLANG_REFERENCE_FRONT_END = 0x8
    SPV_REFLECT_GENERATOR_GOOGLE_SHADERC_OVER_GLSLANG = 0xD
    SPV_REFLECT_GENERATOR_GOOGLE_SPIREGG = 0xE
    SPV_REFLECT_GENERATOR_GOOGLE_RSPIRV = 0xF
    SPV_REFLECT_GENERATOR_X_LEGEND_MESA_MESAIR_SPIRV_TRANSLATOR = 0x10
    SPV_REFLECT_GENERATOR_KHRONOS_SPIRV_TOOLS_LINKER = 0x11
    SPV_REFLECT_GENERATOR_WINE_VKD3D_SHADER_COMPILER = 0x12
    SPV_REFLECT_GENERATOR_CLAY_CLAY_SHADER_COMPILER = 0x13

class SpvReflectResourceType(IntEnum):
    SPV_REFLECT_RESOURCE_FLAG_UNDEFINED = 0x0
    SPV_REFLECT_RESOURCE_FLAG_SAMPLER = 0x1
    SPV_REFLECT_RESOURCE_FLAG_CBV = 0x2
    SPV_REFLECT_RESOURCE_FLAG_SRV = 0x4
    SPV_REFLECT_RESOURCE_FLAG_UAV = 0x8

class SpvReflectFormat(IntEnum):
    SPV_REFLECT_FORMAT_UNDEFINED = 0x0
    SPV_REFLECT_FORMAT_R32_UINT = 0x62
    SPV_REFLECT_FORMAT_R32_SINT = 0x63
    SPV_REFLECT_FORMAT_R32_SFLOAT = 0x64
    SPV_REFLECT_FORMAT_R32G32_UINT = 0x65
    SPV_REFLECT_FORMAT_R32G32_SINT = 0x66
    SPV_REFLECT_FORMAT_R32G32_SFLOAT = 0x67
    SPV_REFLECT_FORMAT_R32G32B32_UINT = 0x68
    SPV_REFLECT_FORMAT_R32G32B32_SINT = 0x69
    SPV_REFLECT_FORMAT_R32G32B32_SFLOAT = 0x6A
    SPV_REFLECT_FORMAT_R32G32B32A32_UINT = 0x6B
    SPV_REFLECT_FORMAT_R32G32B32A32_SINT = 0x6C
    SPV_REFLECT_FORMAT_R32G32B32A32_SFLOAT = 0x6D
    SPV_REFLECT_FORMAT_R64_UINT = 0x6E
    SPV_REFLECT_FORMAT_R64_SINT = 0x6F
    SPV_REFLECT_FORMAT_R64_SFLOAT = 0x70
    SPV_REFLECT_FORMAT_R64G64_UINT = 0x71
    SPV_REFLECT_FORMAT_R64G64_SINT = 0x72
    SPV_REFLECT_FORMAT_R64G64_SFLOAT = 0x73
    SPV_REFLECT_FORMAT_R64G64B64_UINT = 0x74
    SPV_REFLECT_FORMAT_R64G64B64_SINT = 0x75
    SPV_REFLECT_FORMAT_R64G64B64_SFLOAT = 0x76
    SPV_REFLECT_FORMAT_R64G64B64A64_UINT = 0x77
    SPV_REFLECT_FORMAT_R64G64B64A64_SINT = 0x78
    SPV_REFLECT_FORMAT_R64G64B64A64_SFLOAT = 0x79

class ScePadButtonDataOffset(IntEnum):
    SCE_PAD_BUTTON_L3 = 0x2

class AkOnSwitchMode(IntEnum):
    AkOnSwitchMode_PlayToEnd = 0x0
    AkOnSwitchMode_Stop = 0x1

class SpvReflectDecorationFlagBits(IntEnum):
    SPV_REFLECT_DECORATION_NONE = 0x0
    SPV_REFLECT_DECORATION_BLOCK = 0x1
    SPV_REFLECT_DECORATION_BUFFER_BLOCK = 0x2
    SPV_REFLECT_DECORATION_ROW_MAJOR = 0x4
    SPV_REFLECT_DECORATION_COLUMN_MAJOR = 0x8
    SPV_REFLECT_DECORATION_BUILT_IN = 0x10
    SPV_REFLECT_DECORATION_NOPERSPECTIVE = 0x20
    SPV_REFLECT_DECORATION_FLAT = 0x40
    SPV_REFLECT_DECORATION_NON_WRITABLE = 0x80
    SPV_REFLECT_DECORATION_RELAXED_PRECISION = 0x100
    SPV_REFLECT_DECORATION_NON_READABLE = 0x200

class SpvReflectVariableFlagBits(IntEnum):
    SPV_REFLECT_VARIABLE_FLAGS_NONE = 0x0
    SPV_REFLECT_VARIABLE_FLAGS_UNUSED = 0x1

class SpvDecoration_(IntEnum):
    SpvDecorationRelaxedPrecision = 0x0
    SpvDecorationSpecId = 0x1
    SpvDecorationBlock = 0x2
    SpvDecorationBufferBlock = 0x3
    SpvDecorationRowMajor = 0x4
    SpvDecorationColMajor = 0x5
    SpvDecorationArrayStride = 0x6
    SpvDecorationMatrixStride = 0x7
    SpvDecorationGLSLShared = 0x8
    SpvDecorationGLSLPacked = 0x9
    SpvDecorationCPacked = 0xA
    SpvDecorationBuiltIn = 0xB
    SpvDecorationNoPerspective = 0xD
    SpvDecorationFlat = 0xE
    SpvDecorationPatch = 0xF
    SpvDecorationCentroid = 0x10
    SpvDecorationSample = 0x11
    SpvDecorationInvariant = 0x12
    SpvDecorationRestrict = 0x13
    SpvDecorationAliased = 0x14
    SpvDecorationVolatile = 0x15
    SpvDecorationConstant = 0x16
    SpvDecorationCoherent = 0x17
    SpvDecorationNonWritable = 0x18
    SpvDecorationNonReadable = 0x19
    SpvDecorationUniform = 0x1A
    SpvDecorationUniformId = 0x1B
    SpvDecorationSaturatedConversion = 0x1C
    SpvDecorationStream = 0x1D
    SpvDecorationLocation = 0x1E
    SpvDecorationComponent = 0x1F
    SpvDecorationIndex = 0x20
    SpvDecorationBinding = 0x21
    SpvDecorationDescriptorSet = 0x22
    SpvDecorationOffset = 0x23
    SpvDecorationXfbBuffer = 0x24
    SpvDecorationXfbStride = 0x25
    SpvDecorationFuncParamAttr = 0x26
    SpvDecorationFPRoundingMode = 0x27
    SpvDecorationFPFastMathMode = 0x28
    SpvDecorationLinkageAttributes = 0x29
    SpvDecorationNoContraction = 0x2A
    SpvDecorationInputAttachmentIndex = 0x2B
    SpvDecorationAlignment = 0x2C
    SpvDecorationMaxByteOffset = 0x2D
    SpvDecorationAlignmentId = 0x2E
    SpvDecorationMaxByteOffsetId = 0x2F
    SpvDecorationNoSignedWrap = 0x1175
    SpvDecorationNoUnsignedWrap = 0x1176
    SpvDecorationExplicitInterpAMD = 0x1387
    SpvDecorationOverrideCoverageNV = 0x1480
    SpvDecorationPassthroughNV = 0x1482
    SpvDecorationViewportRelativeNV = 0x1484
    SpvDecorationSecondaryViewportRelativeNV = 0x1488
    SpvDecorationPerPrimitiveEXT = 0x1497
    SpvDecorationPerViewNV = 0x1498
    SpvDecorationPerTaskNV = 0x1499
    SpvDecorationPerVertexKHR = 0x14A5
    SpvDecorationNonUniform = 0x14B4
    SpvDecorationRestrictPointer = 0x14EB
    SpvDecorationAliasedPointer = 0x14EC
    SpvDecorationHitObjectShaderRecordBufferNV = 0x150A
    SpvDecorationBindlessSamplerNV = 0x1516
    SpvDecorationBindlessImageNV = 0x1517
    SpvDecorationBoundSamplerNV = 0x1518
    SpvDecorationBoundImageNV = 0x1519
    SpvDecorationSIMTCallINTEL = 0x15DF
    SpvDecorationReferencedIndirectlyINTEL = 0x15E2
    SpvDecorationClobberINTEL = 0x15E7
    SpvDecorationSideEffectsINTEL = 0x15E8
    SpvDecorationVectorComputeVariableINTEL = 0x15F8
    SpvDecorationFuncParamIOKindINTEL = 0x15F9
    SpvDecorationVectorComputeFunctionINTEL = 0x15FA
    SpvDecorationStackCallINTEL = 0x15FB
    SpvDecorationGlobalVariableOffsetINTEL = 0x15FC
    SpvDecorationCounterBuffer = 0x1602
    SpvDecorationHlslSemanticGOOGLE = 0x1603
    SpvDecorationUserTypeGOOGLE = 0x1604
    SpvDecorationFunctionRoundingModeINTEL = 0x16BE
    SpvDecorationFunctionDenormModeINTEL = 0x16BF
    SpvDecorationRegisterINTEL = 0x16C1
    SpvDecorationMemoryINTEL = 0x16C2
    SpvDecorationNumbanksINTEL = 0x16C3
    SpvDecorationBankwidthINTEL = 0x16C4
    SpvDecorationMaxPrivateCopiesINTEL = 0x16C5
    SpvDecorationSinglepumpINTEL = 0x16C6
    SpvDecorationDoublepumpINTEL = 0x16C7
    SpvDecorationMaxReplicatesINTEL = 0x16C8
    SpvDecorationSimpleDualPortINTEL = 0x16C9
    SpvDecorationMergeINTEL = 0x16CA
    SpvDecorationBankBitsINTEL = 0x16CB
    SpvDecorationForcePow2DepthINTEL = 0x16CC
    SpvDecorationBurstCoalesceINTEL = 0x170B
    SpvDecorationCacheSizeINTEL = 0x170C
    SpvDecorationDontStaticallyCoalesceINTEL = 0x170D
    SpvDecorationPrefetchINTEL = 0x170E
    SpvDecorationStallEnableINTEL = 0x1711
    SpvDecorationFuseLoopsInFunctionINTEL = 0x1713
    SpvDecorationMathOpDSPModeINTEL = 0x1715
    SpvDecorationAliasScopeINTEL = 0x171A
    SpvDecorationNoAliasINTEL = 0x171B
    SpvDecorationInitiationIntervalINTEL = 0x171D
    SpvDecorationMaxConcurrencyINTEL = 0x171E
    SpvDecorationPipelineEnableINTEL = 0x171F
    SpvDecorationBufferLocationINTEL = 0x1721
    SpvDecorationIOPipeStorageINTEL = 0x1738
    SpvDecorationFunctionFloatingPointModeINTEL = 0x17C0
    SpvDecorationSingleElementVectorINTEL = 0x17C5
    SpvDecorationVectorComputeCallableFunctionINTEL = 0x17C7
    SpvDecorationMediaBlockIOINTEL = 0x17FC
    SpvDecorationMax = 0x7FFFFFFF

class EnvelopeTriggerMode(IntEnum):
    EnvelopeTriggerMode_Play = 0x1

class KSPROPERTY_CAMERACONTROL_EXTENDED_PROPERTY(IntEnum):
    KSPROPERTY_CAMERACONTROL_EXTENDED_PHOTOMODE = 0x0
    KSPROPERTY_CAMERACONTROL_EXTENDED_PHOTOFRAMERATE = 0x1
    KSPROPERTY_CAMERACONTROL_EXTENDED_PHOTOMAXFRAMERATE = 0x2
    KSPROPERTY_CAMERACONTROL_EXTENDED_PHOTOTRIGGERTIME = 0x3
    KSPROPERTY_CAMERACONTROL_EXTENDED_WARMSTART = 0x4
    KSPROPERTY_CAMERACONTROL_EXTENDED_MAXVIDFPS_PHOTORES = 0x5
    KSPROPERTY_CAMERACONTROL_EXTENDED_PHOTOTHUMBNAIL = 0x6
    KSPROPERTY_CAMERACONTROL_EXTENDED_SCENEMODE = 0x7
    KSPROPERTY_CAMERACONTROL_EXTENDED_TORCHMODE = 0x8
    KSPROPERTY_CAMERACONTROL_EXTENDED_FLASHMODE = 0x9
    KSPROPERTY_CAMERACONTROL_EXTENDED_OPTIMIZATIONHINT = 0xA
    KSPROPERTY_CAMERACONTROL_EXTENDED_WHITEBALANCEMODE = 0xB
    KSPROPERTY_CAMERACONTROL_EXTENDED_EXPOSUREMODE = 0xC
    KSPROPERTY_CAMERACONTROL_EXTENDED_FOCUSMODE = 0xD
    KSPROPERTY_CAMERACONTROL_EXTENDED_ISO = 0xE
    KSPROPERTY_CAMERACONTROL_EXTENDED_FIELDOFVIEW = 0xF
    KSPROPERTY_CAMERACONTROL_EXTENDED_EVCOMPENSATION = 0x10
    KSPROPERTY_CAMERACONTROL_EXTENDED_CAMERAANGLEOFFSET = 0x11
    KSPROPERTY_CAMERACONTROL_EXTENDED_SECURE_MODE = 0x12
    KSPROPERTY_CAMERACONTROL_EXTENDED_END = 0x13

class AkMixer(ctypes.Structure):
    class ApplyGainSpeakerPreset(IntEnum):
        ApplyGainSpeaker_Mono = 0x0
        ApplyGainSpeaker_Stereo = 0x1
        ApplyGainSpeaker_51 = 0x2
        ApplyGainSpeaker_71 = 0x3
        ApplyGainSpeaker_Other = 0x4
        ApplyGainSpeaker_End = 0x5

    def ApplyGain(): pass

    def ApplyGainAndInterleave(): pass

    def InitMixerFunctTable(): pass

    def MixBypassNinNChannels(): pass

    def MixChannel(): pass

    def MixFilterNinNChannels(): pass

    def MixNinNChannels(): pass

    def MixOneInNChannels(): pass


class InputDataType(IntEnum):
    I16_1Chan = 0x0
    I16_2Chan = 0x1
    I16_NChan = 0x2
    Native_1Chan = 0x3
    Native_2Chan = 0x4
    Native_NChan = 0x5
    NumInputDataType = 0x6

class _AUDCLNT_BUFFERFLAGS(IntEnum):
    AUDCLNT_BUFFERFLAGS_DATA_DISCONTINUITY = 0x1

class AMBISONICS_NORMALIZATION(IntEnum):
    AMBISONICS_NORMALIZATION_SN3D = 0x0
    AMBISONICS_NORMALIZATION_N3D = 0x1

class AudioObjectType(IntEnum):
    AudioObjectType_None = 0x0
    AudioObjectType_Dynamic = 0x1
    AudioObjectType_FrontLeft = 0x2
    AudioObjectType_FrontRight = 0x4
    AudioObjectType_FrontCenter = 0x8
    AudioObjectType_LowFrequency = 0x10
    AudioObjectType_SideLeft = 0x20
    AudioObjectType_SideRight = 0x40
    AudioObjectType_BackLeft = 0x80
    AudioObjectType_BackRight = 0x100
    AudioObjectType_TopFrontLeft = 0x200
    AudioObjectType_TopFrontRight = 0x400
    AudioObjectType_TopBackLeft = 0x800
    AudioObjectType_TopBackRight = 0x1000
    AudioObjectType_BottomFrontLeft = 0x2000
    AudioObjectType_BottomFrontRight = 0x4000
    AudioObjectType_BottomBackLeft = 0x8000
    AudioObjectType_BottomBackRight = 0x10000
    AudioObjectType_BackCenter = 0x20000

class TrustLevel(IntEnum):
    BaseTrust = 0x0
    PartialTrust = 0x1
    FullTrust = 0x2

class RO_INIT_TYPE(IntEnum):
    RO_INIT_SINGLETHREADED = 0x0
    RO_INIT_MULTITHREADED = 0x1

class _AUDCLNT_SHAREMODE(IntEnum):
    AUDCLNT_SHAREMODE_SHARED = 0x0
    AUDCLNT_SHAREMODE_EXCLUSIVE = 0x1

class _AUDIO_STREAM_CATEGORY(IntEnum):
    AudioCategory_Other = 0x0
    AudioCategory_ForegroundOnlyMedia = 0x1
    AudioCategory_BackgroundCapableMedia = 0x2
    AudioCategory_Communications = 0x3
    AudioCategory_Alerts = 0x4
    AudioCategory_SoundEffects = 0x5
    AudioCategory_GameEffects = 0x6
    AudioCategory_GameMedia = 0x7
    AudioCategory_GameChat = 0x8
    AudioCategory_Speech = 0x9
    AudioCategory_Movie = 0xA
    AudioCategory_Media = 0xB

class AudioSessionDisconnectReason(IntEnum):
    DisconnectReasonDeviceRemoval = 0x0
    DisconnectReasonServerShutdown = 0x1
    DisconnectReasonFormatChanged = 0x2
    DisconnectReasonSessionLogoff = 0x3
    DisconnectReasonSessionDisconnected = 0x4
    DisconnectReasonExclusiveModeOverride = 0x5

class RO_ERROR_REPORTING_FLAGS(IntEnum):
    RO_ERROR_REPORTING_NONE = 0x0
    RO_ERROR_REPORTING_SUPPRESSEXCEPTIONS = 0x1
    RO_ERROR_REPORTING_FORCEEXCEPTIONS = 0x2
    RO_ERROR_REPORTING_USESETERRORINFO = 0x4
    RO_ERROR_REPORTING_SUPPRESSSETERRORINFO = 0x8

class _AudioSessionState(IntEnum):
    AudioSessionStateInactive = 0x0
    AudioSessionStateActive = 0x1
    AudioSessionStateExpired = 0x2

class AkStmType(IntEnum):
    AK_StmTypeStandard = 0x0
    AK_StmTypeAutomatic = 0x1

class AkStereoDelayParamID(IntEnum):
    AKSTEREODELAYPARAMID_ENABLECROSSFEED = 0x0
    AKSTEREODELAYPARAMID_ENABLEFEEDBACK = 0x1
    AKSTEREODELAYPARAMID_DRYLEVEL = 0x2
    AKSTEREODELAYPARAMID_WETLEVEL = 0x3
    AKSTEREODELAYPARAMID_FRONTREARBALANCE = 0x4
    AKSTEREODELAYPARAMID_FILTERTYPE = 0x5
    AKSTEREODELAYPARAMID_FILTERGAIN = 0x6
    AKSTEREODELAYPARAMID_FILTERFREQUENCY = 0x7
    AKSTEREODELAYPARAMID_FILTERQFACTOR = 0x8
    AKSTEREODELAYPARAMID_LEFTINPUTTYPE = 0x9
    AKSTEREODELAYPARAMID_LEFTDELAYTIME = 0xA
    AKSTEREODELAYPARAMID_LEFTFEEDBACK = 0xB
    AKSTEREODELAYPARAMID_LEFTCROSSFEED = 0xC
    AKSTEREODELAYPARAMID_RIGHTINPUTTYPE = 0xD
    AKSTEREODELAYPARAMID_RIGHTDELAYTIME = 0xE
    AKSTEREODELAYPARAMID_RIGHTFEEDBACK = 0xF
    AKSTEREODELAYPARAMID_RIGHTCROSSFEED = 0x10
    AKSTEREODELAYPARAMID_NUM_PARAMS = 0x11

class AkInputChannelType(IntEnum):
    AKINPUTCHANNELTYPE_LEFT_OR_RIGHT = 0x0
    AKINPUTCHANNELTYPE_CENTER = 0x1
    AKINPUTCHANNELTYPE_DOWNMIX = 0x2
    AKINPUTCHANNELTYPE_NONE = 0x3

class hkcdPlanarGeometryPredicates(ctypes.Structure):
    class Orientation(IntEnum):
        ON_PLANE = 0x0
        IN_FRONT_OF = 0x1
        BEHIND = 0x2
        INTERSECT = 0x3

    class Coplanarity(IntEnum):
        NOT_COPLANAR = 0x0
        COINCIDENT = 0x1
        OPPOSITE = 0x2

    class Winding(IntEnum):
        WINDING_COLLINEAR = 0x0
        WINDING_CCW = 0x1
        WINDING_UNKNOWN = 0x3
        WINDING_CW = 0xFFFFFFFF

    def computeIntersectionDeterminants(): pass


class LicenseType(IntEnum):
    EVALUATION = 0x0
    CLIENT = 0x1

class BinaryPackfile2014(ctypes.Structure):
    _fields_ = [
        ("m_magic", ctypes.c_int32 * 0x2),  # int[2]
        ("m_userTag", ctypes.c_int32),
        ("m_fileVersion", ctypes.c_int32),
    ]
    m_magic: list[ctypes.c_int32]
    m_userTag: int
    m_fileVersion: int

class hkAabbUtil(ctypes.Structure):
    def calcAabb(): pass


class VkIndirectCommandsTokenTypeNVX(IntEnum):
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_PIPELINE_NVX = 0x0
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_DESCRIPTOR_SET_NVX = 0x1
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_INDEX_BUFFER_NVX = 0x2
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_VERTEX_BUFFER_NVX = 0x3
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_PUSH_CONSTANT_NVX = 0x4
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_INDEXED_NVX = 0x5
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_DRAW_NVX = 0x6
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_DISPATCH_NVX = 0x7
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_RANGE_SIZE_NVX = 0x8
    VK_INDIRECT_COMMANDS_TOKEN_TYPE_MAX_ENUM_NVX = 0x7FFFFFFF

class VkObjectEntryTypeNVX(IntEnum):
    VK_OBJECT_ENTRY_TYPE_DESCRIPTOR_SET_NVX = 0x0
    VK_OBJECT_ENTRY_TYPE_PIPELINE_NVX = 0x1
    VK_OBJECT_ENTRY_TYPE_INDEX_BUFFER_NVX = 0x2
    VK_OBJECT_ENTRY_TYPE_VERTEX_BUFFER_NVX = 0x3
    VK_OBJECT_ENTRY_TYPE_PUSH_CONSTANT_NVX = 0x4
    VK_OBJECT_ENTRY_TYPE_RANGE_SIZE_NVX = 0x5
    VK_OBJECT_ENTRY_TYPE_MAX_ENUM_NVX = 0x7FFFFFFF

class NGX_CORE_SUPPORT(ctypes.Structure):
    class SUPPORT_FLAGS(IntEnum):
        SEAMLESS_OTA = 0x1


class _DXGKMDT_OPM_HDCP_PROTECTION_LEVEL(IntEnum):
    DXGKMDT_OPM_HDCP_OFF = 0x0
    DXGKMDT_OPM_HDCP_ON = 0x1
    DXGKMDT_OPM_HDCP_FORCE_ULONG = 0x7FFFFFFF

class _D3DKMDT_VIDEO_OUTPUT_TECHNOLOGY(IntEnum):
    D3DKMDT_VOT_HD15 = 0x0
    D3DKMDT_VOT_SVIDEO = 0x1
    D3DKMDT_VOT_COMPOSITE_VIDEO = 0x2
    D3DKMDT_VOT_COMPONENT_VIDEO = 0x3
    D3DKMDT_VOT_DVI = 0x4
    D3DKMDT_VOT_HDMI = 0x5
    D3DKMDT_VOT_LVDS = 0x6
    D3DKMDT_VOT_D_JPN = 0x8
    D3DKMDT_VOT_SDI = 0x9
    D3DKMDT_VOT_DISPLAYPORT_EXTERNAL = 0xA
    D3DKMDT_VOT_DISPLAYPORT_EMBEDDED = 0xB
    D3DKMDT_VOT_UDI_EXTERNAL = 0xC
    D3DKMDT_VOT_UDI_EMBEDDED = 0xD
    D3DKMDT_VOT_SDTVDONGLE = 0xE
    D3DKMDT_VOT_MIRACAST = 0xF
    D3DKMDT_VOT_INDIRECT_WIRED = 0x10
    D3DKMDT_VOT_INTERNAL = 0x80000000
    D3DKMDT_VOT_UNINITIALIZED = 0xFFFFFFFE
    D3DKMDT_VOT_OTHER = 0xFFFFFFFF

class NvidiaDriverType(IntEnum):
    NvidiaDisplayDriver = 0x0

class _KMTQUERYADAPTERINFOTYPE(IntEnum):
    KMTQAITYPE_UMDRIVERPRIVATE = 0x0
    KMTQAITYPE_UMDRIVERNAME = 0x1
    KMTQAITYPE_UMOPENGLINFO = 0x2
    KMTQAITYPE_GETSEGMENTSIZE = 0x3
    KMTQAITYPE_ADAPTERGUID = 0x4
    KMTQAITYPE_FLIPQUEUEINFO = 0x5
    KMTQAITYPE_ADAPTERADDRESS = 0x6
    KMTQAITYPE_SETWORKINGSETINFO = 0x7
    KMTQAITYPE_ADAPTERREGISTRYINFO = 0x8
    KMTQAITYPE_CURRENTDISPLAYMODE = 0x9
    KMTQAITYPE_MODELIST = 0xA
    KMTQAITYPE_CHECKDRIVERUPDATESTATUS = 0xB
    KMTQAITYPE_VIRTUALADDRESSINFO = 0xC
    KMTQAITYPE_DRIVERVERSION = 0xD
    KMTQAITYPE_ADAPTERTYPE = 0xF
    KMTQAITYPE_OUTPUTDUPLCONTEXTSCOUNT = 0x10
    KMTQAITYPE_WDDM_1_2_CAPS = 0x11
    KMTQAITYPE_UMD_DRIVER_VERSION = 0x12
    KMTQAITYPE_DIRECTFLIP_SUPPORT = 0x13
    KMTQAITYPE_MULTIPLANEOVERLAY_SUPPORT = 0x14
    KMTQAITYPE_DLIST_DRIVER_NAME = 0x15
    KMTQAITYPE_WDDM_1_3_CAPS = 0x16
    KMTQAITYPE_MULTIPLANEOVERLAY_HUD_SUPPORT = 0x17
    KMTQAITYPE_WDDM_2_0_CAPS = 0x18
    KMTQAITYPE_NODEMETADATA = 0x19
    KMTQAITYPE_CPDRIVERNAME = 0x1A
    KMTQAITYPE_XBOX = 0x1B
    KMTQAITYPE_INDEPENDENTFLIP_SUPPORT = 0x1C
    KMTQAITYPE_MIRACASTCOMPANIONDRIVERNAME = 0x1D
    KMTQAITYPE_PHYSICALADAPTERCOUNT = 0x1E
    KMTQAITYPE_PHYSICALADAPTERDEVICEIDS = 0x1F
    KMTQAITYPE_DRIVERCAPS_EXT = 0x20
    KMTQAITYPE_QUERY_MIRACAST_DRIVER_TYPE = 0x21
    KMTQAITYPE_QUERY_GPUMMU_CAPS = 0x22
    KMTQAITYPE_QUERY_MULTIPLANEOVERLAY_DECODE_SUPPORT = 0x23
    KMTQAITYPE_QUERY_HW_PROTECTION_TEARDOWN_COUNT = 0x24
    KMTQAITYPE_QUERY_ISBADDRIVERFORHWPROTECTIONDISABLED = 0x25
    KMTQAITYPE_MULTIPLANEOVERLAY_SECONDARY_SUPPORT = 0x26
    KMTQAITYPE_INDEPENDENTFLIP_SECONDARY_SUPPORT = 0x27
    KMTQAITYPE_PANELFITTER_SUPPORT = 0x28
    KMTQAITYPE_PHYSICALADAPTERPNPKEY = 0x29
    KMTQAITYPE_GETSEGMENTGROUPSIZE = 0x2A
    KMTQAITYPE_MPO3DDI_SUPPORT = 0x2B
    KMTQAITYPE_HWDRM_SUPPORT = 0x2C
    KMTQAITYPE_MPOKERNELCAPS_SUPPORT = 0x2D
    KMTQAITYPE_MULTIPLANEOVERLAY_STRETCH_SUPPORT = 0x2E
    KMTQAITYPE_GET_DEVICE_VIDPN_OWNERSHIP_INFO = 0x2F
    KMTQAITYPE_QUERYREGISTRY = 0x30
    KMTQAITYPE_KMD_DRIVER_VERSION = 0x31
    KMTQAITYPE_BLOCKLIST_KERNEL = 0x32
    KMTQAITYPE_BLOCKLIST_RUNTIME = 0x33
    KMTQAITYPE_ADAPTERGUID_RENDER = 0x34
    KMTQAITYPE_ADAPTERADDRESS_RENDER = 0x35
    KMTQAITYPE_ADAPTERREGISTRYINFO_RENDER = 0x36
    KMTQAITYPE_CHECKDRIVERUPDATESTATUS_RENDER = 0x37
    KMTQAITYPE_DRIVERVERSION_RENDER = 0x38
    KMTQAITYPE_ADAPTERTYPE_RENDER = 0x39
    KMTQAITYPE_WDDM_1_2_CAPS_RENDER = 0x3A
    KMTQAITYPE_WDDM_1_3_CAPS_RENDER = 0x3B
    KMTQAITYPE_QUERY_ADAPTER_UNIQUE_GUID = 0x3C
    KMTQAITYPE_NODEPERFDATA = 0x3D
    KMTQAITYPE_ADAPTERPERFDATA = 0x3E
    KMTQAITYPE_ADAPTERPERFDATA_CAPS = 0x3F
    KMTQUITYPE_GPUVERSION = 0x40

class _D3DDDIFORMAT(IntEnum):
    D3DDDIFMT_UNKNOWN = 0x0
    D3DDDIFMT_R8G8B8 = 0x14
    D3DDDIFMT_A8R8G8B8 = 0x15
    D3DDDIFMT_X8R8G8B8 = 0x16
    D3DDDIFMT_R5G6B5 = 0x17
    D3DDDIFMT_X1R5G5B5 = 0x18
    D3DDDIFMT_A1R5G5B5 = 0x19
    D3DDDIFMT_A4R4G4B4 = 0x1A
    D3DDDIFMT_R3G3B2 = 0x1B
    D3DDDIFMT_A8 = 0x1C
    D3DDDIFMT_A8R3G3B2 = 0x1D
    D3DDDIFMT_X4R4G4B4 = 0x1E
    D3DDDIFMT_A2B10G10R10 = 0x1F
    D3DDDIFMT_A8B8G8R8 = 0x20
    D3DDDIFMT_X8B8G8R8 = 0x21
    D3DDDIFMT_G16R16 = 0x22
    D3DDDIFMT_A2R10G10B10 = 0x23
    D3DDDIFMT_A16B16G16R16 = 0x24
    D3DDDIFMT_A8P8 = 0x28
    D3DDDIFMT_P8 = 0x29
    D3DDDIFMT_L8 = 0x32
    D3DDDIFMT_A8L8 = 0x33
    D3DDDIFMT_A4L4 = 0x34
    D3DDDIFMT_V8U8 = 0x3C
    D3DDDIFMT_L6V5U5 = 0x3D
    D3DDDIFMT_X8L8V8U8 = 0x3E
    D3DDDIFMT_Q8W8V8U8 = 0x3F
    D3DDDIFMT_V16U16 = 0x40
    D3DDDIFMT_W11V11U10 = 0x41
    D3DDDIFMT_A2W10V10U10 = 0x43
    D3DDDIFMT_D16_LOCKABLE = 0x46
    D3DDDIFMT_D32 = 0x47
    D3DDDIFMT_S1D15 = 0x48
    D3DDDIFMT_D15S1 = 0x49
    D3DDDIFMT_S8D24 = 0x4A
    D3DDDIFMT_D24S8 = 0x4B
    D3DDDIFMT_X8D24 = 0x4C
    D3DDDIFMT_D24X8 = 0x4D
    D3DDDIFMT_X4S4D24 = 0x4E
    D3DDDIFMT_D24X4S4 = 0x4F
    D3DDDIFMT_D16 = 0x50
    D3DDDIFMT_L16 = 0x51
    D3DDDIFMT_D32F_LOCKABLE = 0x52
    D3DDDIFMT_D24FS8 = 0x53
    D3DDDIFMT_D32_LOCKABLE = 0x54
    D3DDDIFMT_S8_LOCKABLE = 0x55
    D3DDDIFMT_G8R8 = 0x5B
    D3DDDIFMT_R8 = 0x5C
    D3DDDIFMT_VERTEXDATA = 0x64
    D3DDDIFMT_INDEX16 = 0x65
    D3DDDIFMT_INDEX32 = 0x66
    D3DDDIFMT_Q16W16V16U16 = 0x6E
    D3DDDIFMT_R16F = 0x6F
    D3DDDIFMT_G16R16F = 0x70
    D3DDDIFMT_A16B16G16R16F = 0x71
    D3DDDIFMT_R32F = 0x72
    D3DDDIFMT_G32R32F = 0x73
    D3DDDIFMT_A32B32G32R32F = 0x74
    D3DDDIFMT_CxV8U8 = 0x75
    D3DDDIFMT_A1 = 0x76
    D3DDDIFMT_A2B10G10R10_XR_BIAS = 0x77
    D3DDDIFMT_DXVACOMPBUFFER_BASE = 0x96
    D3DDDIFMT_MACROBLOCKDATA = 0x97
    D3DDDIFMT_RESIDUALDIFFERENCEDATA = 0x98
    D3DDDIFMT_DEBLOCKINGDATA = 0x99
    D3DDDIFMT_INVERSEQUANTIZATIONDATA = 0x9A
    D3DDDIFMT_SLICECONTROLDATA = 0x9B
    D3DDDIFMT_BITSTREAMDATA = 0x9C
    D3DDDIFMT_MOTIONVECTORBUFFER = 0x9D
    D3DDDIFMT_FILMGRAINBUFFER = 0x9E
    D3DDDIFMT_DXVA_RESERVED9 = 0x9F
    D3DDDIFMT_DXVA_RESERVED10 = 0xA0
    D3DDDIFMT_DXVA_RESERVED11 = 0xA1
    D3DDDIFMT_DXVA_RESERVED12 = 0xA2
    D3DDDIFMT_DXVA_RESERVED13 = 0xA3
    D3DDDIFMT_DXVA_RESERVED14 = 0xA4
    D3DDDIFMT_DXVA_RESERVED15 = 0xA5
    D3DDDIFMT_DXVA_RESERVED16 = 0xA6
    D3DDDIFMT_DXVA_RESERVED17 = 0xA7
    D3DDDIFMT_DXVA_RESERVED18 = 0xA8
    D3DDDIFMT_DXVA_RESERVED19 = 0xA9
    D3DDDIFMT_DXVA_RESERVED20 = 0xAA
    D3DDDIFMT_DXVA_RESERVED21 = 0xAB
    D3DDDIFMT_DXVA_RESERVED22 = 0xAC
    D3DDDIFMT_DXVA_RESERVED23 = 0xAD
    D3DDDIFMT_DXVA_RESERVED24 = 0xAE
    D3DDDIFMT_DXVA_RESERVED25 = 0xAF
    D3DDDIFMT_DXVA_RESERVED26 = 0xB0
    D3DDDIFMT_DXVA_RESERVED27 = 0xB1
    D3DDDIFMT_DXVA_RESERVED28 = 0xB2
    D3DDDIFMT_DXVA_RESERVED29 = 0xB3
    D3DDDIFMT_DXVA_RESERVED30 = 0xB4
    D3DDDIFMT_DXVA_RESERVED31 = 0xB5
    D3DDDIFMT_BINARYBUFFER = 0xC7
    D3DDDIFMT_MULTI2_ARGB8 = 0x3154454D
    D3DDDIFMT_DXT1 = 0x31545844
    D3DDDIFMT_DXT2 = 0x32545844
    D3DDDIFMT_YUY2 = 0x32595559
    D3DDDIFMT_DXT3 = 0x33545844
    D3DDDIFMT_DXT4 = 0x34545844
    D3DDDIFMT_DXT5 = 0x35545844
    D3DDDIFMT_G8R8_G8B8 = 0x42475247
    D3DDDIFMT_R8G8_B8G8 = 0x47424752
    D3DDDIFMT_UYVY = 0x59565955
    D3DDDIFMT_FORCE_UINT = 0x7FFFFFFF

class _D3DKMDT_VIDPN_PRESENT_PATH_ROTATION(IntEnum):
    D3DKMDT_VPPR_UNINITIALIZED = 0x0
    D3DKMDT_VPPR_IDENTITY = 0x1
    D3DKMDT_VPPR_ROTATE90 = 0x2
    D3DKMDT_VPPR_ROTATE180 = 0x3
    D3DKMDT_VPPR_ROTATE270 = 0x4
    D3DKMDT_VPPR_IDENTITY_OFFSET90 = 0x5
    D3DKMDT_VPPR_ROTATE90_OFFSET90 = 0x6
    D3DKMDT_VPPR_ROTATE180_OFFSET90 = 0x7
    D3DKMDT_VPPR_ROTATE270_OFFSET90 = 0x8
    D3DKMDT_VPPR_IDENTITY_OFFSET180 = 0x9
    D3DKMDT_VPPR_ROTATE90_OFFSET180 = 0xA
    D3DKMDT_VPPR_ROTATE180_OFFSET180 = 0xB
    D3DKMDT_VPPR_ROTATE270_OFFSET180 = 0xC
    D3DKMDT_VPPR_IDENTITY_OFFSET270 = 0xD
    D3DKMDT_VPPR_ROTATE90_OFFSET270 = 0xE
    D3DKMDT_VPPR_ROTATE180_OFFSET270 = 0xF
    D3DKMDT_VPPR_ROTATE270_OFFSET270 = 0x10
    D3DKMDT_VPPR_UNPINNED = 0xFE
    D3DKMDT_VPPR_NOTSPECIFIED = 0xFF

class _D3DDDI_QUERYREGISTRY_TYPE(IntEnum):
    D3DDDI_QUERYREGISTRY_SERVICEKEY = 0x0
    D3DDDI_QUERYREGISTRY_ADAPTERKEY = 0x1
    D3DDDI_QUERYREGISTRY_DRIVERSTOREPATH = 0x2
    D3DDDI_QUERYREGISTRY_MAX = 0x3

class _D3DDDI_QUERYREGISTRY_STATUS(IntEnum):
    D3DDDI_QUERYREGISTRY_STATUS_SUCCESS = 0x0
    D3DDDI_QUERYREGISTRY_STATUS_BUFFER_OVERFLOW = 0x1
    D3DDDI_QUERYREGISTRY_STATUS_FAIL = 0x2
    D3DDDI_QUERYREGISTRY_STATUS_MAX = 0x3

class _DEVPROPSTORE(IntEnum):
    DEVPROP_STORE_SYSTEM = 0x0
    DEVPROP_STORE_USER = 0x1

class _D3DRESOURCETYPE(IntEnum):
    D3DRTYPE_SURFACE = 0x1
    D3DRTYPE_VOLUME = 0x2
    D3DRTYPE_TEXTURE = 0x3
    D3DRTYPE_VOLUMETEXTURE = 0x4
    D3DRTYPE_CUBETEXTURE = 0x5
    D3DRTYPE_VERTEXBUFFER = 0x6
    D3DRTYPE_INDEXBUFFER = 0x7
    D3DRTYPE_FORCE_DWORD = 0x7FFFFFFF

class WINDOWCOMPOSITIONATTRIB(IntEnum):
    WCA_UNDEFINED = 0x0
    WCA_NCRENDERING_ENABLED = 0x1
    WCA_NCRENDERING_POLICY = 0x2
    WCA_TRANSITIONS_FORCEDISABLED = 0x3
    WCA_ALLOW_NCPAINT = 0x4
    WCA_CAPTION_BUTTON_BOUNDS = 0x5
    WCA_NONCLIENT_RTL_LAYOUT = 0x6
    WCA_FORCE_ICONIC_REPRESENTATION = 0x7
    WCA_EXTENDED_FRAME_BOUNDS = 0x8
    WCA_HAS_ICONIC_BITMAP = 0x9
    WCA_THEME_ATTRIBUTES = 0xA
    WCA_NCRENDERING_EXILED = 0xB
    WCA_NCADORNMENTINFO = 0xC
    WCA_EXCLUDED_FROM_LIVEPREVIEW = 0xD
    WCA_VIDEO_OVERLAY_ACTIVE = 0xE
    WCA_FORCE_ACTIVEWINDOW_APPEARANCE = 0xF
    WCA_DISALLOW_PEEK = 0x10
    WCA_CLOAK = 0x11
    WCA_CLOAKED = 0x12
    WCA_ACCENT_POLICY = 0x13
    WCA_FREEZE_REPRESENTATION = 0x14
    WCA_EVER_UNCLOAKED = 0x15
    WCA_VISUAL_OWNER = 0x16
    WCA_HOLOGRAPHIC = 0x17
    WCA_EXCLUDED_FROM_DDA = 0x18
    WCA_PASSIVEUPDATEMODE = 0x19
    WCA_USEDARKMODECOLORS = 0x1A
    WCA_LAST = 0x1B

class _MACHINE_ATTRIBUTES(IntEnum):
    UserEnabled = 0x1

class WINDOWTHEMEATTRIBUTETYPE(IntEnum):
    WTA_NONCLIENT = 0x1

class DIT_HITTESTATTRIBUTES(IntEnum):
    DIT_HITTESTATTRIBUTES_NONE = 0x0
    DIT_HITTESTATTRIBUTES_MOUSEWHEEL = 0x1
    DIT_HITTESTATTRIBUTES_MOUSEWHEELISINCREASING = 0x2
    DIT_HITTESTATTRIBUTES_MOUSEWHEELISHORIZONTAL = 0x4
    DIT_HITTESTATTRIBUTES_MOUSEBUTTONDOWN = 0x8
    DIT_HITTESTATTRIBUTES_TOUCHTARGETING = 0x10
    DIT_HITTESTATTRIBUTES_NOHOVERPOINTER = 0x20
    DIT_HITTESTATTRIBUTES_BARRELBUTTONDOWN = 0x40
    DIT_HITTESTATTRIBUTES_ERASERDOWN = 0x80
    DIT_HITTESTATTRIBUTES_VALID = 0xFF
    DIT_HITTESTATTRIBUTES_CONTROLDOWN = 0x100

class OCT_BANNED_API(IntEnum):
    OCT_BANNED_FIRST = 0x0
    OCT_BANNED_ClientToScreen = 0x1
    OCT_BANNED_ScreenToClient = 0x2
    OCT_BANNED_MapWindowPoints = 0x3
    OCT_BANNED_GetDC = 0x4
    OCT_BANNED_GetDCEx = 0x5
    OCT_BANNED_EnumDisplayMonitors = 0x6
    OCT_BANNED_GetSystemMetrics = 0x7
    OCT_BANNED_MonitorFromWindow = 0x8
    OCT_BANNED_MonitorFromPoint = 0x9
    OCT_BANNED_MonitorFromRect = 0xA
    OCT_BANNED_GetMonitorInfoW = 0xB
    OCT_BANNED_GetDpiForMonitorInternal = 0xC
    OCT_BANNED_PhysicalToLogicalPoint = 0xD
    OCT_BANNED_LogicalToPhysicalPoint = 0xE
    OCT_BANNED_GetCurrentDpiInfo = 0xF
    OCT_BANNED_ShellScaling_Window = 0x10
    OCT_BANNED_ShellScaling_Monitor = 0x11
    OCT_BANNED_ShellScaling_Device = 0x12
    OCT_BANNED_LAST = 0x13

class DWMWINDOWATTRIBUTE(IntEnum):
    DWMWA_NCRENDERING_ENABLED = 0x1
    DWMWA_NCRENDERING_POLICY = 0x2
    DWMWA_TRANSITIONS_FORCEDISABLED = 0x3
    DWMWA_ALLOW_NCPAINT = 0x4
    DWMWA_CAPTION_BUTTON_BOUNDS = 0x5
    DWMWA_NONCLIENT_RTL_LAYOUT = 0x6
    DWMWA_FORCE_ICONIC_REPRESENTATION = 0x7
    DWMWA_FLIP3D_POLICY = 0x8
    DWMWA_EXTENDED_FRAME_BOUNDS = 0x9
    DWMWA_HAS_ICONIC_BITMAP = 0xA
    DWMWA_DISALLOW_PEEK = 0xB
    DWMWA_EXCLUDED_FROM_PEEK = 0xC
    DWMWA_CLOAK = 0xD
    DWMWA_CLOAKED = 0xE
    DWMWA_FREEZE_REPRESENTATION = 0xF
    DWMWA_PASSIVE_UPDATE_MODE = 0x10
    DWMWA_LAST = 0x11

class D3D_DRIVER_TYPE(IntEnum):
    D3D_DRIVER_TYPE_UNKNOWN = 0x0
    D3D_DRIVER_TYPE_HARDWARE = 0x1
    D3D_DRIVER_TYPE_REFERENCE = 0x2
    D3D_DRIVER_TYPE_NULL = 0x3
    D3D_DRIVER_TYPE_SOFTWARE = 0x4
    D3D_DRIVER_TYPE_WARP = 0x5

class D3D_PRIMITIVE_TOPOLOGY(IntEnum):
    D3D_PRIMITIVE_TOPOLOGY_UNDEFINED = 0x0
    D3D_PRIMITIVE_TOPOLOGY_POINTLIST = 0x1
    D3D_PRIMITIVE_TOPOLOGY_LINELIST = 0x2
    D3D_PRIMITIVE_TOPOLOGY_LINESTRIP = 0x3
    D3D_PRIMITIVE_TOPOLOGY_TRIANGLELIST = 0x4
    D3D_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP = 0x5
    D3D_PRIMITIVE_TOPOLOGY_LINELIST_ADJ = 0xA
    D3D_PRIMITIVE_TOPOLOGY_LINESTRIP_ADJ = 0xB
    D3D_PRIMITIVE_TOPOLOGY_TRIANGLELIST_ADJ = 0xC
    D3D_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP_ADJ = 0xD
    D3D_PRIMITIVE_TOPOLOGY_1_CONTROL_POINT_PATCHLIST = 0x21
    D3D_PRIMITIVE_TOPOLOGY_2_CONTROL_POINT_PATCHLIST = 0x22
    D3D_PRIMITIVE_TOPOLOGY_3_CONTROL_POINT_PATCHLIST = 0x23
    D3D_PRIMITIVE_TOPOLOGY_4_CONTROL_POINT_PATCHLIST = 0x24
    D3D_PRIMITIVE_TOPOLOGY_5_CONTROL_POINT_PATCHLIST = 0x25
    D3D_PRIMITIVE_TOPOLOGY_6_CONTROL_POINT_PATCHLIST = 0x26
    D3D_PRIMITIVE_TOPOLOGY_7_CONTROL_POINT_PATCHLIST = 0x27
    D3D_PRIMITIVE_TOPOLOGY_8_CONTROL_POINT_PATCHLIST = 0x28
    D3D_PRIMITIVE_TOPOLOGY_9_CONTROL_POINT_PATCHLIST = 0x29
    D3D_PRIMITIVE_TOPOLOGY_10_CONTROL_POINT_PATCHLIST = 0x2A
    D3D_PRIMITIVE_TOPOLOGY_11_CONTROL_POINT_PATCHLIST = 0x2B
    D3D_PRIMITIVE_TOPOLOGY_12_CONTROL_POINT_PATCHLIST = 0x2C
    D3D_PRIMITIVE_TOPOLOGY_13_CONTROL_POINT_PATCHLIST = 0x2D
    D3D_PRIMITIVE_TOPOLOGY_14_CONTROL_POINT_PATCHLIST = 0x2E
    D3D_PRIMITIVE_TOPOLOGY_15_CONTROL_POINT_PATCHLIST = 0x2F
    D3D_PRIMITIVE_TOPOLOGY_16_CONTROL_POINT_PATCHLIST = 0x30
    D3D_PRIMITIVE_TOPOLOGY_17_CONTROL_POINT_PATCHLIST = 0x31
    D3D_PRIMITIVE_TOPOLOGY_18_CONTROL_POINT_PATCHLIST = 0x32
    D3D_PRIMITIVE_TOPOLOGY_19_CONTROL_POINT_PATCHLIST = 0x33
    D3D_PRIMITIVE_TOPOLOGY_20_CONTROL_POINT_PATCHLIST = 0x34
    D3D_PRIMITIVE_TOPOLOGY_21_CONTROL_POINT_PATCHLIST = 0x35
    D3D_PRIMITIVE_TOPOLOGY_22_CONTROL_POINT_PATCHLIST = 0x36
    D3D_PRIMITIVE_TOPOLOGY_23_CONTROL_POINT_PATCHLIST = 0x37
    D3D_PRIMITIVE_TOPOLOGY_24_CONTROL_POINT_PATCHLIST = 0x38
    D3D_PRIMITIVE_TOPOLOGY_25_CONTROL_POINT_PATCHLIST = 0x39
    D3D_PRIMITIVE_TOPOLOGY_26_CONTROL_POINT_PATCHLIST = 0x3A
    D3D_PRIMITIVE_TOPOLOGY_27_CONTROL_POINT_PATCHLIST = 0x3B
    D3D_PRIMITIVE_TOPOLOGY_28_CONTROL_POINT_PATCHLIST = 0x3C
    D3D_PRIMITIVE_TOPOLOGY_29_CONTROL_POINT_PATCHLIST = 0x3D
    D3D_PRIMITIVE_TOPOLOGY_30_CONTROL_POINT_PATCHLIST = 0x3E
    D3D_PRIMITIVE_TOPOLOGY_31_CONTROL_POINT_PATCHLIST = 0x3F
    D3D_PRIMITIVE_TOPOLOGY_32_CONTROL_POINT_PATCHLIST = 0x40

class D3D_PRIMITIVE(IntEnum):
    D3D_PRIMITIVE_UNDEFINED = 0x0
    D3D_PRIMITIVE_POINT = 0x1
    D3D_PRIMITIVE_LINE = 0x2
    D3D_PRIMITIVE_TRIANGLE = 0x3
    D3D_PRIMITIVE_LINE_ADJ = 0x6
    D3D_PRIMITIVE_TRIANGLE_ADJ = 0x7
    D3D_PRIMITIVE_1_CONTROL_POINT_PATCH = 0x8
    D3D_PRIMITIVE_2_CONTROL_POINT_PATCH = 0x9
    D3D_PRIMITIVE_3_CONTROL_POINT_PATCH = 0xA
    D3D_PRIMITIVE_4_CONTROL_POINT_PATCH = 0xB
    D3D_PRIMITIVE_5_CONTROL_POINT_PATCH = 0xC
    D3D_PRIMITIVE_6_CONTROL_POINT_PATCH = 0xD
    D3D_PRIMITIVE_7_CONTROL_POINT_PATCH = 0xE
    D3D_PRIMITIVE_8_CONTROL_POINT_PATCH = 0xF
    D3D_PRIMITIVE_9_CONTROL_POINT_PATCH = 0x10
    D3D_PRIMITIVE_10_CONTROL_POINT_PATCH = 0x11
    D3D_PRIMITIVE_11_CONTROL_POINT_PATCH = 0x12
    D3D_PRIMITIVE_12_CONTROL_POINT_PATCH = 0x13
    D3D_PRIMITIVE_13_CONTROL_POINT_PATCH = 0x14
    D3D_PRIMITIVE_14_CONTROL_POINT_PATCH = 0x15
    D3D_PRIMITIVE_15_CONTROL_POINT_PATCH = 0x16
    D3D_PRIMITIVE_16_CONTROL_POINT_PATCH = 0x17
    D3D_PRIMITIVE_17_CONTROL_POINT_PATCH = 0x18
    D3D_PRIMITIVE_18_CONTROL_POINT_PATCH = 0x19
    D3D_PRIMITIVE_19_CONTROL_POINT_PATCH = 0x1A
    D3D_PRIMITIVE_20_CONTROL_POINT_PATCH = 0x1B
    D3D_PRIMITIVE_21_CONTROL_POINT_PATCH = 0x1C
    D3D_PRIMITIVE_22_CONTROL_POINT_PATCH = 0x1D
    D3D_PRIMITIVE_23_CONTROL_POINT_PATCH = 0x1E
    D3D_PRIMITIVE_24_CONTROL_POINT_PATCH = 0x1F
    D3D_PRIMITIVE_25_CONTROL_POINT_PATCH = 0x20
    D3D_PRIMITIVE_26_CONTROL_POINT_PATCH = 0x21
    D3D_PRIMITIVE_27_CONTROL_POINT_PATCH = 0x22
    D3D_PRIMITIVE_28_CONTROL_POINT_PATCH = 0x23
    D3D_PRIMITIVE_29_CONTROL_POINT_PATCH = 0x24
    D3D_PRIMITIVE_30_CONTROL_POINT_PATCH = 0x25
    D3D_PRIMITIVE_31_CONTROL_POINT_PATCH = 0x26
    D3D_PRIMITIVE_32_CONTROL_POINT_PATCH = 0x27

class D3D_SRV_DIMENSION(IntEnum):
    D3D_SRV_DIMENSION_UNKNOWN = 0x0
    D3D_SRV_DIMENSION_BUFFER = 0x1
    D3D_SRV_DIMENSION_TEXTURE1D = 0x2
    D3D_SRV_DIMENSION_TEXTURE1DARRAY = 0x3
    D3D_SRV_DIMENSION_TEXTURE2D = 0x4
    D3D_SRV_DIMENSION_TEXTURE2DARRAY = 0x5
    D3D_SRV_DIMENSION_TEXTURE2DMS = 0x6
    D3D_SRV_DIMENSION_TEXTURE2DMSARRAY = 0x7
    D3D_SRV_DIMENSION_TEXTURE3D = 0x8
    D3D_SRV_DIMENSION_TEXTURECUBE = 0x9
    D3D_SRV_DIMENSION_TEXTURECUBEARRAY = 0xA
    D3D_SRV_DIMENSION_BUFFEREX = 0xB

class _D3D_INCLUDE_TYPE(IntEnum):
    D3D_INCLUDE_LOCAL = 0x0
    D3D_INCLUDE_SYSTEM = 0x1
    D3D_INCLUDE_FORCE_DWORD = 0x7FFFFFFF

class _D3D_SHADER_VARIABLE_CLASS(IntEnum):
    D3D_SVC_SCALAR = 0x0
    D3D_SVC_VECTOR = 0x1
    D3D_SVC_MATRIX_ROWS = 0x2
    D3D_SVC_MATRIX_COLUMNS = 0x3
    D3D_SVC_OBJECT = 0x4
    D3D_SVC_STRUCT = 0x5
    D3D_SVC_INTERFACE_CLASS = 0x6
    D3D_SVC_INTERFACE_POINTER = 0x7
    D3D_SVC_FORCE_DWORD = 0x7FFFFFFF

class MONITOR_DPI_TYPE(IntEnum):
    MDT_EFFECTIVE_DPI = 0x0
    MDT_ANGULAR_DPI = 0x1
    MDT_RAW_DPI = 0x2

class _D3D_SHADER_VARIABLE_FLAGS(IntEnum):
    D3D_SVF_USERPACKED = 0x1
    D3D_SVF_USED = 0x2
    D3D_SVF_INTERFACE_POINTER = 0x4
    D3D_SVF_INTERFACE_PARAMETER = 0x8
    D3D_SVF_FORCE_DWORD = 0x7FFFFFFF

class _D3D_SHADER_VARIABLE_TYPE(IntEnum):
    D3D_SVT_VOID = 0x0
    D3D_SVT_BOOL = 0x1
    D3D_SVT_INT = 0x2
    D3D_SVT_FLOAT = 0x3
    D3D_SVT_STRING = 0x4
    D3D_SVT_TEXTURE = 0x5
    D3D_SVT_TEXTURE1D = 0x6
    D3D_SVT_TEXTURE2D = 0x7
    D3D_SVT_TEXTURE3D = 0x8
    D3D_SVT_TEXTURECUBE = 0x9
    D3D_SVT_SAMPLER = 0xA
    D3D_SVT_SAMPLER1D = 0xB
    D3D_SVT_SAMPLER2D = 0xC
    D3D_SVT_SAMPLER3D = 0xD
    D3D_SVT_SAMPLERCUBE = 0xE
    D3D_SVT_PIXELSHADER = 0xF
    D3D_SVT_VERTEXSHADER = 0x10
    D3D_SVT_PIXELFRAGMENT = 0x11
    D3D_SVT_VERTEXFRAGMENT = 0x12
    D3D_SVT_UINT = 0x13
    D3D_SVT_UINT8 = 0x14
    D3D_SVT_GEOMETRYSHADER = 0x15
    D3D_SVT_RASTERIZER = 0x16
    D3D_SVT_DEPTHSTENCIL = 0x17
    D3D_SVT_BLEND = 0x18
    D3D_SVT_BUFFER = 0x19
    D3D_SVT_CBUFFER = 0x1A
    D3D_SVT_TBUFFER = 0x1B
    D3D_SVT_TEXTURE1DARRAY = 0x1C
    D3D_SVT_TEXTURE2DARRAY = 0x1D
    D3D_SVT_RENDERTARGETVIEW = 0x1E
    D3D_SVT_DEPTHSTENCILVIEW = 0x1F
    D3D_SVT_TEXTURE2DMS = 0x20
    D3D_SVT_TEXTURE2DMSARRAY = 0x21
    D3D_SVT_TEXTURECUBEARRAY = 0x22
    D3D_SVT_HULLSHADER = 0x23
    D3D_SVT_DOMAINSHADER = 0x24
    D3D_SVT_INTERFACE_POINTER = 0x25
    D3D_SVT_COMPUTESHADER = 0x26
    D3D_SVT_DOUBLE = 0x27
    D3D_SVT_RWTEXTURE1D = 0x28
    D3D_SVT_RWTEXTURE1DARRAY = 0x29
    D3D_SVT_RWTEXTURE2D = 0x2A
    D3D_SVT_RWTEXTURE2DARRAY = 0x2B
    D3D_SVT_RWTEXTURE3D = 0x2C
    D3D_SVT_RWBUFFER = 0x2D
    D3D_SVT_BYTEADDRESS_BUFFER = 0x2E
    D3D_SVT_RWBYTEADDRESS_BUFFER = 0x2F
    D3D_SVT_STRUCTURED_BUFFER = 0x30
    D3D_SVT_RWSTRUCTURED_BUFFER = 0x31
    D3D_SVT_APPEND_STRUCTURED_BUFFER = 0x32
    D3D_SVT_CONSUME_STRUCTURED_BUFFER = 0x33
    D3D_SVT_MIN8FLOAT = 0x34
    D3D_SVT_MIN10FLOAT = 0x35
    D3D_SVT_MIN16FLOAT = 0x36
    D3D_SVT_MIN12INT = 0x37
    D3D_SVT_MIN16INT = 0x38
    D3D_SVT_MIN16UINT = 0x39
    D3D_SVT_FORCE_DWORD = 0x7FFFFFFF

class _D3D_SHADER_INPUT_FLAGS(IntEnum):
    D3D_SIF_USERPACKED = 0x1
    D3D_SIF_COMPARISON_SAMPLER = 0x2
    D3D_SIF_TEXTURE_COMPONENT_0 = 0x4
    D3D_SIF_TEXTURE_COMPONENT_1 = 0x8
    D3D_SIF_TEXTURE_COMPONENTS = 0xC
    D3D_SIF_UNUSED = 0x10
    D3D_SIF_FORCE_DWORD = 0x7FFFFFFF

class _D3D_SHADER_INPUT_TYPE(IntEnum):
    D3D_SIT_CBUFFER = 0x0
    D3D_SIT_TBUFFER = 0x1
    D3D_SIT_TEXTURE = 0x2
    D3D_SIT_SAMPLER = 0x3
    D3D_SIT_UAV_RWTYPED = 0x4
    D3D_SIT_STRUCTURED = 0x5
    D3D_SIT_UAV_RWSTRUCTURED = 0x6
    D3D_SIT_BYTEADDRESS = 0x7
    D3D_SIT_UAV_RWBYTEADDRESS = 0x8
    D3D_SIT_UAV_APPEND_STRUCTURED = 0x9
    D3D_SIT_UAV_CONSUME_STRUCTURED = 0xA
    D3D_SIT_UAV_RWSTRUCTURED_WITH_COUNTER = 0xB
    D3D_SIT_RTACCELERATIONSTRUCTURE = 0xC
    D3D_SIT_UAV_FEEDBACKTEXTURE = 0xD

class _D3D_SHADER_CBUFFER_FLAGS(IntEnum):
    D3D_CBF_USERPACKED = 0x1
    D3D_CBF_FORCE_DWORD = 0x7FFFFFFF

class _D3D_CBUFFER_TYPE(IntEnum):
    D3D_CT_CBUFFER = 0x0
    D3D_CT_TBUFFER = 0x1
    D3D_CT_INTERFACE_POINTERS = 0x2
    D3D_CT_RESOURCE_BIND_INFO = 0x3

class D3D_NAME(IntEnum):
    D3D_NAME_UNDEFINED = 0x0
    D3D_NAME_POSITION = 0x1
    D3D_NAME_CLIP_DISTANCE = 0x2
    D3D_NAME_CULL_DISTANCE = 0x3
    D3D_NAME_RENDER_TARGET_ARRAY_INDEX = 0x4
    D3D_NAME_VIEWPORT_ARRAY_INDEX = 0x5
    D3D_NAME_VERTEX_ID = 0x6
    D3D_NAME_PRIMITIVE_ID = 0x7
    D3D_NAME_INSTANCE_ID = 0x8
    D3D_NAME_IS_FRONT_FACE = 0x9
    D3D_NAME_SAMPLE_INDEX = 0xA
    D3D_NAME_FINAL_QUAD_EDGE_TESSFACTOR = 0xB
    D3D_NAME_FINAL_QUAD_INSIDE_TESSFACTOR = 0xC
    D3D_NAME_FINAL_TRI_EDGE_TESSFACTOR = 0xD
    D3D_NAME_FINAL_TRI_INSIDE_TESSFACTOR = 0xE
    D3D_NAME_FINAL_LINE_DETAIL_TESSFACTOR = 0xF
    D3D_NAME_FINAL_LINE_DENSITY_TESSFACTOR = 0x10
    D3D_NAME_BARYCENTRICS = 0x17
    D3D_NAME_SHADINGRATE = 0x18
    D3D_NAME_CULLPRIMITIVE = 0x19
    D3D_NAME_TARGET = 0x40
    D3D_NAME_DEPTH = 0x41
    D3D_NAME_COVERAGE = 0x42
    D3D_NAME_DEPTH_GREATER_EQUAL = 0x43
    D3D_NAME_DEPTH_LESS_EQUAL = 0x44
    D3D_NAME_STENCIL_REF = 0x45
    D3D_NAME_INNER_COVERAGE = 0x46

class D3D_RESOURCE_RETURN_TYPE(IntEnum):
    D3D_RETURN_TYPE_UNORM = 0x1
    D3D_RETURN_TYPE_SNORM = 0x2
    D3D_RETURN_TYPE_SINT = 0x3
    D3D_RETURN_TYPE_UINT = 0x4
    D3D_RETURN_TYPE_FLOAT = 0x5
    D3D_RETURN_TYPE_MIXED = 0x6
    D3D_RETURN_TYPE_DOUBLE = 0x7
    D3D_RETURN_TYPE_CONTINUED = 0x8

class D3D_REGISTER_COMPONENT_TYPE(IntEnum):
    D3D_REGISTER_COMPONENT_UNKNOWN = 0x0
    D3D_REGISTER_COMPONENT_UINT32 = 0x1
    D3D_REGISTER_COMPONENT_SINT32 = 0x2
    D3D_REGISTER_COMPONENT_FLOAT32 = 0x3

class D3D_TESSELLATOR_DOMAIN(IntEnum):
    D3D_TESSELLATOR_DOMAIN_UNDEFINED = 0x0
    D3D_TESSELLATOR_DOMAIN_ISOLINE = 0x1
    D3D_TESSELLATOR_DOMAIN_TRI = 0x2
    D3D_TESSELLATOR_DOMAIN_QUAD = 0x3

class D3D_TESSELLATOR_PARTITIONING(IntEnum):
    D3D_TESSELLATOR_PARTITIONING_UNDEFINED = 0x0
    D3D_TESSELLATOR_PARTITIONING_INTEGER = 0x1
    D3D_TESSELLATOR_PARTITIONING_POW2 = 0x2
    D3D_TESSELLATOR_PARTITIONING_FRACTIONAL_ODD = 0x3
    D3D_TESSELLATOR_PARTITIONING_FRACTIONAL_EVEN = 0x4

class D3D_TESSELLATOR_OUTPUT_PRIMITIVE(IntEnum):
    D3D_TESSELLATOR_OUTPUT_UNDEFINED = 0x0
    D3D_TESSELLATOR_OUTPUT_POINT = 0x1
    D3D_TESSELLATOR_OUTPUT_LINE = 0x2
    D3D_TESSELLATOR_OUTPUT_TRIANGLE_CW = 0x3
    D3D_TESSELLATOR_OUTPUT_TRIANGLE_CCW = 0x4

class __MIDL___MIDL_itf_touchtelemetry_0000_0000_0001(IntEnum):
    TELEMETRY_INTERACTION_LIBRARY_TYPE_UNKNOWN = 0x0
    TELEMETRY_INTERACTION_LIBRARY_TYPE_DIRECT_MANIPULATION = 0x1
    TELEMETRY_INTERACTION_LIBRARY_TYPE_INTERACTION_TRACKER = 0x2
    TELEMETRY_INTERACTION_LIBRARY_TYPE_DIRECT_INK = 0x3

class D3D10_USAGE(IntEnum):
    D3D10_USAGE_DEFAULT = 0x0
    D3D10_USAGE_IMMUTABLE = 0x1
    D3D10_USAGE_DYNAMIC = 0x2
    D3D10_USAGE_STAGING = 0x3

class D3D10_BIND_FLAG(IntEnum):
    D3D10_BIND_VERTEX_BUFFER = 0x1

class D3D10_COLOR_WRITE_ENABLE(IntEnum):
    D3D10_COLOR_WRITE_ENABLE_RED = 0x1
    D3D10_COLOR_WRITE_ENABLE_GREEN = 0x2
    D3D10_COLOR_WRITE_ENABLE_BLUE = 0x4
    D3D10_COLOR_WRITE_ENABLE_ALPHA = 0x8
    D3D10_COLOR_WRITE_ENABLE_ALL = 0xF

class GROUP_WINDOW_ROLE(IntEnum):
    GWR_DEFAULT = 0x0
    GWR_APPLICATION = 0x1
    GWR_FALLBACK = 0x2

class SET_WINDOW_GROUP_OPTIONS(IntEnum):
    SWGO_NONE = 0x0
    SWGO_TEMPLATE = 0x1
    SWGO_NOZORDER = 0x2
    SWGO_ALL = 0x3
    SWGO_RESPECT_MINIMIZE = 0x4

class WICBitmapCreateCacheOption(IntEnum):
    WICBitmapNoCache = 0x0
    WICBitmapCacheOnDemand = 0x1
    WICBitmapCacheOnLoad = 0x2
    WICBITMAPCREATECACHEOPTION_FORCE_DWORD = 0x7FFFFFFF

class WICBitmapInterpolationMode(IntEnum):
    WICBitmapInterpolationModeNearestNeighbor = 0x0
    WICBitmapInterpolationModeLinear = 0x1
    WICBitmapInterpolationModeCubic = 0x2
    WICBitmapInterpolationModeFant = 0x3
    WICBitmapInterpolationModeHighQualityCubic = 0x4
    WICBITMAPINTERPOLATIONMODE_FORCE_DWORD = 0x7FFFFFFF

class WICBitmapPaletteType(IntEnum):
    WICBitmapPaletteTypeCustom = 0x0
    WICBitmapPaletteTypeMedianCut = 0x1
    WICBitmapPaletteTypeFixedBW = 0x2
    WICBitmapPaletteTypeFixedHalftone8 = 0x3
    WICBitmapPaletteTypeFixedHalftone27 = 0x4
    WICBitmapPaletteTypeFixedHalftone64 = 0x5
    WICBitmapPaletteTypeFixedHalftone125 = 0x6
    WICBitmapPaletteTypeFixedHalftone216 = 0x7
    WICBitmapPaletteTypeFixedHalftone252 = 0x8
    WICBitmapPaletteTypeFixedHalftone256 = 0x9
    WICBitmapPaletteTypeFixedGray4 = 0xA
    WICBitmapPaletteTypeFixedGray16 = 0xB
    WICBitmapPaletteTypeFixedGray256 = 0xC
    WICBITMAPPALETTETYPE_FORCE_DWORD = 0x7FFFFFFF

class WICBitmapDitherType(IntEnum):
    WICBitmapDitherTypeNone = 0x0
    WICBitmapDitherTypeOrdered4x4 = 0x1
    WICBitmapDitherTypeOrdered8x8 = 0x2
    WICBitmapDitherTypeOrdered16x16 = 0x3
    WICBitmapDitherTypeSpiral4x4 = 0x4
    WICBitmapDitherTypeSpiral8x8 = 0x5
    WICBitmapDitherTypeDualSpiral4x4 = 0x6
    WICBitmapDitherTypeDualSpiral8x8 = 0x7
    WICBitmapDitherTypeErrorDiffusion = 0x8
    WICBITMAPDITHERTYPE_FORCE_DWORD = 0x7FFFFFFF

class WICBitmapAlphaChannelOption(IntEnum):
    WICBitmapUseAlpha = 0x0
    WICBitmapUsePremultipliedAlpha = 0x1
    WICBitmapIgnoreAlpha = 0x2
    WICBITMAPALPHACHANNELOPTIONS_FORCE_DWORD = 0x7FFFFFFF

class WICBitmapTransformOptions(IntEnum):
    WICBitmapTransformRotate0 = 0x0
    WICBitmapTransformRotate90 = 0x1
    WICBitmapTransformRotate180 = 0x2
    WICBitmapTransformRotate270 = 0x3
    WICBitmapTransformFlipHorizontal = 0x8
    WICBitmapTransformFlipVertical = 0x10
    WICBITMAPTRANSFORMOPTIONS_FORCE_DWORD = 0x7FFFFFFF

class WICBitmapLockFlags(IntEnum):
    WICBitmapLockRead = 0x1
    WICBitmapLockWrite = 0x2
    WICBITMAPLOCKFLAGS_FORCE_DWORD = 0x7FFFFFFF

class D3D10_QUERY(IntEnum):
    D3D10_QUERY_EVENT = 0x0
    D3D10_QUERY_OCCLUSION = 0x1
    D3D10_QUERY_TIMESTAMP = 0x2
    D3D10_QUERY_TIMESTAMP_DISJOINT = 0x3
    D3D10_QUERY_PIPELINE_STATISTICS = 0x4
    D3D10_QUERY_OCCLUSION_PREDICATE = 0x5
    D3D10_QUERY_SO_STATISTICS = 0x6
    D3D10_QUERY_SO_OVERFLOW_PREDICATE = 0x7

class D3D10_COUNTER(IntEnum):
    D3D10_COUNTER_GPU_IDLE = 0x0
    D3D10_COUNTER_VERTEX_PROCESSING = 0x1
    D3D10_COUNTER_GEOMETRY_PROCESSING = 0x2
    D3D10_COUNTER_PIXEL_PROCESSING = 0x3
    D3D10_COUNTER_OTHER_GPU_PROCESSING = 0x4
    D3D10_COUNTER_HOST_ADAPTER_BANDWIDTH_UTILIZATION = 0x5
    D3D10_COUNTER_LOCAL_VIDMEM_BANDWIDTH_UTILIZATION = 0x6
    D3D10_COUNTER_VERTEX_THROUGHPUT_UTILIZATION = 0x7
    D3D10_COUNTER_TRIANGLE_SETUP_THROUGHPUT_UTILIZATION = 0x8
    D3D10_COUNTER_FILLRATE_THROUGHPUT_UTILIZATION = 0x9
    D3D10_COUNTER_VS_MEMORY_LIMITED = 0xA
    D3D10_COUNTER_VS_COMPUTATION_LIMITED = 0xB
    D3D10_COUNTER_GS_MEMORY_LIMITED = 0xC
    D3D10_COUNTER_GS_COMPUTATION_LIMITED = 0xD
    D3D10_COUNTER_PS_MEMORY_LIMITED = 0xE
    D3D10_COUNTER_PS_COMPUTATION_LIMITED = 0xF
    D3D10_COUNTER_POST_TRANSFORM_CACHE_HIT_RATE = 0x10
    D3D10_COUNTER_TEXTURE_CACHE_HIT_RATE = 0x11
    D3D10_COUNTER_DEVICE_DEPENDENT_0 = 0x40000000

class D3D10_COUNTER_TYPE(IntEnum):
    D3D10_COUNTER_TYPE_FLOAT32 = 0x0
    D3D10_COUNTER_TYPE_UINT16 = 0x1
    D3D10_COUNTER_TYPE_UINT32 = 0x2
    D3D10_COUNTER_TYPE_UINT64 = 0x3

class D3D10_MESSAGE_CATEGORY(IntEnum):
    D3D10_MESSAGE_CATEGORY_APPLICATION_DEFINED = 0x0
    D3D10_MESSAGE_CATEGORY_MISCELLANEOUS = 0x1
    D3D10_MESSAGE_CATEGORY_INITIALIZATION = 0x2
    D3D10_MESSAGE_CATEGORY_CLEANUP = 0x3
    D3D10_MESSAGE_CATEGORY_COMPILATION = 0x4
    D3D10_MESSAGE_CATEGORY_STATE_CREATION = 0x5
    D3D10_MESSAGE_CATEGORY_STATE_SETTING = 0x6
    D3D10_MESSAGE_CATEGORY_STATE_GETTING = 0x7
    D3D10_MESSAGE_CATEGORY_RESOURCE_MANIPULATION = 0x8
    D3D10_MESSAGE_CATEGORY_EXECUTION = 0x9
    D3D10_MESSAGE_CATEGORY_SHADER = 0xA

class D3D10_MESSAGE_SEVERITY(IntEnum):
    D3D10_MESSAGE_SEVERITY_CORRUPTION = 0x0
    D3D10_MESSAGE_SEVERITY_ERROR = 0x1
    D3D10_MESSAGE_SEVERITY_WARNING = 0x2
    D3D10_MESSAGE_SEVERITY_INFO = 0x3
    D3D10_MESSAGE_SEVERITY_MESSAGE = 0x4

class D3D10_MESSAGE_ID(IntEnum):
    D3D10_MESSAGE_ID_UNKNOWN = 0x0
    D3D10_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_HAZARD = 0x1
    D3D10_MESSAGE_ID_DEVICE_IASETINDEXBUFFER_HAZARD = 0x2
    D3D10_MESSAGE_ID_DEVICE_VSSETSHADERRESOURCES_HAZARD = 0x3
    D3D10_MESSAGE_ID_DEVICE_VSSETCONSTANTBUFFERS_HAZARD = 0x4
    D3D10_MESSAGE_ID_DEVICE_GSSETSHADERRESOURCES_HAZARD = 0x5
    D3D10_MESSAGE_ID_DEVICE_GSSETCONSTANTBUFFERS_HAZARD = 0x6
    D3D10_MESSAGE_ID_DEVICE_PSSETSHADERRESOURCES_HAZARD = 0x7
    D3D10_MESSAGE_ID_DEVICE_PSSETCONSTANTBUFFERS_HAZARD = 0x8
    D3D10_MESSAGE_ID_DEVICE_OMSETRENDERTARGETS_HAZARD = 0x9
    D3D10_MESSAGE_ID_DEVICE_SOSETTARGETS_HAZARD = 0xA
    D3D10_MESSAGE_ID_STRING_FROM_APPLICATION = 0xB
    D3D10_MESSAGE_ID_CORRUPTED_THIS = 0xC
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER1 = 0xD
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER2 = 0xE
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER3 = 0xF
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER4 = 0x10
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER5 = 0x11
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER6 = 0x12
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER7 = 0x13
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER8 = 0x14
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER9 = 0x15
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER10 = 0x16
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER11 = 0x17
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER12 = 0x18
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER13 = 0x19
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER14 = 0x1A
    D3D10_MESSAGE_ID_CORRUPTED_PARAMETER15 = 0x1B
    D3D10_MESSAGE_ID_CORRUPTED_MULTITHREADING = 0x1C
    D3D10_MESSAGE_ID_MESSAGE_REPORTING_OUTOFMEMORY = 0x1D
    D3D10_MESSAGE_ID_IASETINPUTLAYOUT_UNBINDDELETINGOBJECT = 0x1E
    D3D10_MESSAGE_ID_IASETVERTEXBUFFERS_UNBINDDELETINGOBJECT = 0x1F
    D3D10_MESSAGE_ID_IASETINDEXBUFFER_UNBINDDELETINGOBJECT = 0x20
    D3D10_MESSAGE_ID_VSSETSHADER_UNBINDDELETINGOBJECT = 0x21
    D3D10_MESSAGE_ID_VSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x22
    D3D10_MESSAGE_ID_VSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x23
    D3D10_MESSAGE_ID_VSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x24
    D3D10_MESSAGE_ID_GSSETSHADER_UNBINDDELETINGOBJECT = 0x25
    D3D10_MESSAGE_ID_GSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x26
    D3D10_MESSAGE_ID_GSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x27
    D3D10_MESSAGE_ID_GSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x28
    D3D10_MESSAGE_ID_SOSETTARGETS_UNBINDDELETINGOBJECT = 0x29
    D3D10_MESSAGE_ID_PSSETSHADER_UNBINDDELETINGOBJECT = 0x2A
    D3D10_MESSAGE_ID_PSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x2B
    D3D10_MESSAGE_ID_PSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x2C
    D3D10_MESSAGE_ID_PSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x2D
    D3D10_MESSAGE_ID_RSSETSTATE_UNBINDDELETINGOBJECT = 0x2E
    D3D10_MESSAGE_ID_OMSETBLENDSTATE_UNBINDDELETINGOBJECT = 0x2F
    D3D10_MESSAGE_ID_OMSETDEPTHSTENCILSTATE_UNBINDDELETINGOBJECT = 0x30
    D3D10_MESSAGE_ID_OMSETRENDERTARGETS_UNBINDDELETINGOBJECT = 0x31
    D3D10_MESSAGE_ID_SETPREDICATION_UNBINDDELETINGOBJECT = 0x32
    D3D10_MESSAGE_ID_GETPRIVATEDATA_MOREDATA = 0x33
    D3D10_MESSAGE_ID_SETPRIVATEDATA_INVALIDFREEDATA = 0x34
    D3D10_MESSAGE_ID_SETPRIVATEDATA_INVALIDIUNKNOWN = 0x35
    D3D10_MESSAGE_ID_SETPRIVATEDATA_INVALIDFLAGS = 0x36
    D3D10_MESSAGE_ID_SETPRIVATEDATA_CHANGINGPARAMS = 0x37
    D3D10_MESSAGE_ID_SETPRIVATEDATA_OUTOFMEMORY = 0x38
    D3D10_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDFORMAT = 0x39
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDSAMPLES = 0x3A
    D3D10_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDUSAGE = 0x3B
    D3D10_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDBINDFLAGS = 0x3C
    D3D10_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDCPUACCESSFLAGS = 0x3D
    D3D10_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDMISCFLAGS = 0x3E
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDCPUACCESSFLAGS = 0x3F
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDBINDFLAGS = 0x40
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDINITIALDATA = 0x41
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDDIMENSIONS = 0x42
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDMIPLEVELS = 0x43
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDMISCFLAGS = 0x44
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDARG_RETURN = 0x45
    D3D10_MESSAGE_ID_CREATEBUFFER_OUTOFMEMORY_RETURN = 0x46
    D3D10_MESSAGE_ID_CREATEBUFFER_NULLDESC = 0x47
    D3D10_MESSAGE_ID_CREATEBUFFER_INVALIDCONSTANTBUFFERBINDINGS = 0x48
    D3D10_MESSAGE_ID_CREATEBUFFER_LARGEALLOCATION = 0x49
    D3D10_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDFORMAT = 0x4A
    D3D10_MESSAGE_ID_CREATETEXTURE1D_UNSUPPORTEDFORMAT = 0x4B
    D3D10_MESSAGE_ID_CREATETEXTURE1D_INVALIDSAMPLES = 0x4C
    D3D10_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDUSAGE = 0x4D
    D3D10_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDBINDFLAGS = 0x4E
    D3D10_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDCPUACCESSFLAGS = 0x4F
    D3D10_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDMISCFLAGS = 0x50
    D3D10_MESSAGE_ID_CREATETEXTURE1D_INVALIDCPUACCESSFLAGS = 0x51
    D3D10_MESSAGE_ID_CREATETEXTURE1D_INVALIDBINDFLAGS = 0x52
    D3D10_MESSAGE_ID_CREATETEXTURE1D_INVALIDINITIALDATA = 0x53
    D3D10_MESSAGE_ID_CREATETEXTURE1D_INVALIDDIMENSIONS = 0x54
    D3D10_MESSAGE_ID_CREATETEXTURE1D_INVALIDMIPLEVELS = 0x55
    D3D10_MESSAGE_ID_CREATETEXTURE1D_INVALIDMISCFLAGS = 0x56
    D3D10_MESSAGE_ID_CREATETEXTURE1D_INVALIDARG_RETURN = 0x57
    D3D10_MESSAGE_ID_CREATETEXTURE1D_OUTOFMEMORY_RETURN = 0x58
    D3D10_MESSAGE_ID_CREATETEXTURE1D_NULLDESC = 0x59
    D3D10_MESSAGE_ID_CREATETEXTURE1D_LARGEALLOCATION = 0x5A
    D3D10_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDFORMAT = 0x5B
    D3D10_MESSAGE_ID_CREATETEXTURE2D_UNSUPPORTEDFORMAT = 0x5C
    D3D10_MESSAGE_ID_CREATETEXTURE2D_INVALIDSAMPLES = 0x5D
    D3D10_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDUSAGE = 0x5E
    D3D10_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDBINDFLAGS = 0x5F
    D3D10_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDCPUACCESSFLAGS = 0x60
    D3D10_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDMISCFLAGS = 0x61
    D3D10_MESSAGE_ID_CREATETEXTURE2D_INVALIDCPUACCESSFLAGS = 0x62
    D3D10_MESSAGE_ID_CREATETEXTURE2D_INVALIDBINDFLAGS = 0x63
    D3D10_MESSAGE_ID_CREATETEXTURE2D_INVALIDINITIALDATA = 0x64
    D3D10_MESSAGE_ID_CREATETEXTURE2D_INVALIDDIMENSIONS = 0x65
    D3D10_MESSAGE_ID_CREATETEXTURE2D_INVALIDMIPLEVELS = 0x66
    D3D10_MESSAGE_ID_CREATETEXTURE2D_INVALIDMISCFLAGS = 0x67
    D3D10_MESSAGE_ID_CREATETEXTURE2D_INVALIDARG_RETURN = 0x68
    D3D10_MESSAGE_ID_CREATETEXTURE2D_OUTOFMEMORY_RETURN = 0x69
    D3D10_MESSAGE_ID_CREATETEXTURE2D_NULLDESC = 0x6A
    D3D10_MESSAGE_ID_CREATETEXTURE2D_LARGEALLOCATION = 0x6B
    D3D10_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDFORMAT = 0x6C
    D3D10_MESSAGE_ID_CREATETEXTURE3D_UNSUPPORTEDFORMAT = 0x6D
    D3D10_MESSAGE_ID_CREATETEXTURE3D_INVALIDSAMPLES = 0x6E
    D3D10_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDUSAGE = 0x6F
    D3D10_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDBINDFLAGS = 0x70
    D3D10_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDCPUACCESSFLAGS = 0x71
    D3D10_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDMISCFLAGS = 0x72
    D3D10_MESSAGE_ID_CREATETEXTURE3D_INVALIDCPUACCESSFLAGS = 0x73
    D3D10_MESSAGE_ID_CREATETEXTURE3D_INVALIDBINDFLAGS = 0x74
    D3D10_MESSAGE_ID_CREATETEXTURE3D_INVALIDINITIALDATA = 0x75
    D3D10_MESSAGE_ID_CREATETEXTURE3D_INVALIDDIMENSIONS = 0x76
    D3D10_MESSAGE_ID_CREATETEXTURE3D_INVALIDMIPLEVELS = 0x77
    D3D10_MESSAGE_ID_CREATETEXTURE3D_INVALIDMISCFLAGS = 0x78
    D3D10_MESSAGE_ID_CREATETEXTURE3D_INVALIDARG_RETURN = 0x79
    D3D10_MESSAGE_ID_CREATETEXTURE3D_OUTOFMEMORY_RETURN = 0x7A
    D3D10_MESSAGE_ID_CREATETEXTURE3D_NULLDESC = 0x7B
    D3D10_MESSAGE_ID_CREATETEXTURE3D_LARGEALLOCATION = 0x7C
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_UNRECOGNIZEDFORMAT = 0x7D
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDDESC = 0x7E
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDFORMAT = 0x7F
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDDIMENSIONS = 0x80
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDRESOURCE = 0x81
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_TOOMANYOBJECTS = 0x82
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDARG_RETURN = 0x83
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_OUTOFMEMORY_RETURN = 0x84
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_UNRECOGNIZEDFORMAT = 0x85
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_UNSUPPORTEDFORMAT = 0x86
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDDESC = 0x87
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDFORMAT = 0x88
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDDIMENSIONS = 0x89
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDRESOURCE = 0x8A
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_TOOMANYOBJECTS = 0x8B
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDARG_RETURN = 0x8C
    D3D10_MESSAGE_ID_CREATERENDERTARGETVIEW_OUTOFMEMORY_RETURN = 0x8D
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_UNRECOGNIZEDFORMAT = 0x8E
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDDESC = 0x8F
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDFORMAT = 0x90
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDDIMENSIONS = 0x91
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDRESOURCE = 0x92
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_TOOMANYOBJECTS = 0x93
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDARG_RETURN = 0x94
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_OUTOFMEMORY_RETURN = 0x95
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_OUTOFMEMORY = 0x96
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_TOOMANYELEMENTS = 0x97
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDFORMAT = 0x98
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_INCOMPATIBLEFORMAT = 0x99
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSLOT = 0x9A
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDINPUTSLOTCLASS = 0x9B
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_STEPRATESLOTCLASSMISMATCH = 0x9C
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSLOTCLASSCHANGE = 0x9D
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSTEPRATECHANGE = 0x9E
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDALIGNMENT = 0x9F
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_DUPLICATESEMANTIC = 0xA0
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_UNPARSEABLEINPUTSIGNATURE = 0xA1
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_NULLSEMANTIC = 0xA2
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_MISSINGELEMENT = 0xA3
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_NULLDESC = 0xA4
    D3D10_MESSAGE_ID_CREATEVERTEXSHADER_OUTOFMEMORY = 0xA5
    D3D10_MESSAGE_ID_CREATEVERTEXSHADER_INVALIDSHADERBYTECODE = 0xA6
    D3D10_MESSAGE_ID_CREATEVERTEXSHADER_INVALIDSHADERTYPE = 0xA7
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADER_OUTOFMEMORY = 0xA8
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADER_INVALIDSHADERBYTECODE = 0xA9
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADER_INVALIDSHADERTYPE = 0xAA
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTOFMEMORY = 0xAB
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSHADERBYTECODE = 0xAC
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSHADERTYPE = 0xAD
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDNUMENTRIES = 0xAE
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTPUTSTREAMSTRIDEUNUSED = 0xAF
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UNEXPECTEDDECL = 0xB0
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_EXPECTEDDECL = 0xB1
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTPUTSLOT0EXPECTED = 0xB2
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDOUTPUTSLOT = 0xB3
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_ONLYONEELEMENTPERSLOT = 0xB4
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDCOMPONENTCOUNT = 0xB5
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSTARTCOMPONENTANDCOMPONENTCOUNT = 0xB6
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDGAPDEFINITION = 0xB7
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_REPEATEDOUTPUT = 0xB8
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDOUTPUTSTREAMSTRIDE = 0xB9
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MISSINGSEMANTIC = 0xBA
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MASKMISMATCH = 0xBB
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_CANTHAVEONLYGAPS = 0xBC
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_DECLTOOCOMPLEX = 0xBD
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MISSINGOUTPUTSIGNATURE = 0xBE
    D3D10_MESSAGE_ID_CREATEPIXELSHADER_OUTOFMEMORY = 0xBF
    D3D10_MESSAGE_ID_CREATEPIXELSHADER_INVALIDSHADERBYTECODE = 0xC0
    D3D10_MESSAGE_ID_CREATEPIXELSHADER_INVALIDSHADERTYPE = 0xC1
    D3D10_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDFILLMODE = 0xC2
    D3D10_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDCULLMODE = 0xC3
    D3D10_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDDEPTHBIASCLAMP = 0xC4
    D3D10_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDSLOPESCALEDDEPTHBIAS = 0xC5
    D3D10_MESSAGE_ID_CREATERASTERIZERSTATE_TOOMANYOBJECTS = 0xC6
    D3D10_MESSAGE_ID_CREATERASTERIZERSTATE_NULLDESC = 0xC7
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDDEPTHWRITEMASK = 0xC8
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDDEPTHFUNC = 0xC9
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILFAILOP = 0xCA
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILZFAILOP = 0xCB
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILPASSOP = 0xCC
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILFUNC = 0xCD
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILFAILOP = 0xCE
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILZFAILOP = 0xCF
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILPASSOP = 0xD0
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILFUNC = 0xD1
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_TOOMANYOBJECTS = 0xD2
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_NULLDESC = 0xD3
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_INVALIDSRCBLEND = 0xD4
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_INVALIDDESTBLEND = 0xD5
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_INVALIDBLENDOP = 0xD6
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_INVALIDSRCBLENDALPHA = 0xD7
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_INVALIDDESTBLENDALPHA = 0xD8
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_INVALIDBLENDOPALPHA = 0xD9
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_INVALIDRENDERTARGETWRITEMASK = 0xDA
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_TOOMANYOBJECTS = 0xDB
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_NULLDESC = 0xDC
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDFILTER = 0xDD
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDADDRESSU = 0xDE
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDADDRESSV = 0xDF
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDADDRESSW = 0xE0
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDMIPLODBIAS = 0xE1
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDMAXANISOTROPY = 0xE2
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDCOMPARISONFUNC = 0xE3
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDMINLOD = 0xE4
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDMAXLOD = 0xE5
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_TOOMANYOBJECTS = 0xE6
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_NULLDESC = 0xE7
    D3D10_MESSAGE_ID_CREATEQUERYORPREDICATE_INVALIDQUERY = 0xE8
    D3D10_MESSAGE_ID_CREATEQUERYORPREDICATE_INVALIDMISCFLAGS = 0xE9
    D3D10_MESSAGE_ID_CREATEQUERYORPREDICATE_UNEXPECTEDMISCFLAG = 0xEA
    D3D10_MESSAGE_ID_CREATEQUERYORPREDICATE_NULLDESC = 0xEB
    D3D10_MESSAGE_ID_DEVICE_IASETPRIMITIVETOPOLOGY_TOPOLOGY_UNRECOGNIZED = 0xEC
    D3D10_MESSAGE_ID_DEVICE_IASETPRIMITIVETOPOLOGY_TOPOLOGY_UNDEFINED = 0xED
    D3D10_MESSAGE_ID_IASETVERTEXBUFFERS_INVALIDBUFFER = 0xEE
    D3D10_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_OFFSET_TOO_LARGE = 0xEF
    D3D10_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_BUFFERS_EMPTY = 0xF0
    D3D10_MESSAGE_ID_IASETINDEXBUFFER_INVALIDBUFFER = 0xF1
    D3D10_MESSAGE_ID_DEVICE_IASETINDEXBUFFER_FORMAT_INVALID = 0xF2
    D3D10_MESSAGE_ID_DEVICE_IASETINDEXBUFFER_OFFSET_TOO_LARGE = 0xF3
    D3D10_MESSAGE_ID_DEVICE_IASETINDEXBUFFER_OFFSET_UNALIGNED = 0xF4
    D3D10_MESSAGE_ID_DEVICE_VSSETSHADERRESOURCES_VIEWS_EMPTY = 0xF5
    D3D10_MESSAGE_ID_VSSETCONSTANTBUFFERS_INVALIDBUFFER = 0xF6
    D3D10_MESSAGE_ID_DEVICE_VSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0xF7
    D3D10_MESSAGE_ID_DEVICE_VSSETSAMPLERS_SAMPLERS_EMPTY = 0xF8
    D3D10_MESSAGE_ID_DEVICE_GSSETSHADERRESOURCES_VIEWS_EMPTY = 0xF9
    D3D10_MESSAGE_ID_GSSETCONSTANTBUFFERS_INVALIDBUFFER = 0xFA
    D3D10_MESSAGE_ID_DEVICE_GSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0xFB
    D3D10_MESSAGE_ID_DEVICE_GSSETSAMPLERS_SAMPLERS_EMPTY = 0xFC
    D3D10_MESSAGE_ID_SOSETTARGETS_INVALIDBUFFER = 0xFD
    D3D10_MESSAGE_ID_DEVICE_SOSETTARGETS_OFFSET_UNALIGNED = 0xFE
    D3D10_MESSAGE_ID_DEVICE_PSSETSHADERRESOURCES_VIEWS_EMPTY = 0xFF
    D3D10_MESSAGE_ID_PSSETCONSTANTBUFFERS_INVALIDBUFFER = 0x100
    D3D10_MESSAGE_ID_DEVICE_PSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x101
    D3D10_MESSAGE_ID_DEVICE_PSSETSAMPLERS_SAMPLERS_EMPTY = 0x102
    D3D10_MESSAGE_ID_DEVICE_RSSETVIEWPORTS_INVALIDVIEWPORT = 0x103
    D3D10_MESSAGE_ID_DEVICE_RSSETSCISSORRECTS_INVALIDSCISSOR = 0x104
    D3D10_MESSAGE_ID_CLEARRENDERTARGETVIEW_DENORMFLUSH = 0x105
    D3D10_MESSAGE_ID_CLEARDEPTHSTENCILVIEW_DENORMFLUSH = 0x106
    D3D10_MESSAGE_ID_CLEARDEPTHSTENCILVIEW_INVALID = 0x107
    D3D10_MESSAGE_ID_DEVICE_IAGETVERTEXBUFFERS_BUFFERS_EMPTY = 0x108
    D3D10_MESSAGE_ID_DEVICE_VSGETSHADERRESOURCES_VIEWS_EMPTY = 0x109
    D3D10_MESSAGE_ID_DEVICE_VSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x10A
    D3D10_MESSAGE_ID_DEVICE_VSGETSAMPLERS_SAMPLERS_EMPTY = 0x10B
    D3D10_MESSAGE_ID_DEVICE_GSGETSHADERRESOURCES_VIEWS_EMPTY = 0x10C
    D3D10_MESSAGE_ID_DEVICE_GSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x10D
    D3D10_MESSAGE_ID_DEVICE_GSGETSAMPLERS_SAMPLERS_EMPTY = 0x10E
    D3D10_MESSAGE_ID_DEVICE_SOGETTARGETS_BUFFERS_EMPTY = 0x10F
    D3D10_MESSAGE_ID_DEVICE_PSGETSHADERRESOURCES_VIEWS_EMPTY = 0x110
    D3D10_MESSAGE_ID_DEVICE_PSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x111
    D3D10_MESSAGE_ID_DEVICE_PSGETSAMPLERS_SAMPLERS_EMPTY = 0x112
    D3D10_MESSAGE_ID_DEVICE_RSGETVIEWPORTS_VIEWPORTS_EMPTY = 0x113
    D3D10_MESSAGE_ID_DEVICE_RSGETSCISSORRECTS_RECTS_EMPTY = 0x114
    D3D10_MESSAGE_ID_DEVICE_GENERATEMIPS_RESOURCE_INVALID = 0x115
    D3D10_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDDESTINATIONSUBRESOURCE = 0x116
    D3D10_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDSOURCESUBRESOURCE = 0x117
    D3D10_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDSOURCEBOX = 0x118
    D3D10_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDSOURCE = 0x119
    D3D10_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDDESTINATIONSTATE = 0x11A
    D3D10_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDSOURCESTATE = 0x11B
    D3D10_MESSAGE_ID_COPYRESOURCE_INVALIDSOURCE = 0x11C
    D3D10_MESSAGE_ID_COPYRESOURCE_INVALIDDESTINATIONSTATE = 0x11D
    D3D10_MESSAGE_ID_COPYRESOURCE_INVALIDSOURCESTATE = 0x11E
    D3D10_MESSAGE_ID_UPDATESUBRESOURCE_INVALIDDESTINATIONSUBRESOURCE = 0x11F
    D3D10_MESSAGE_ID_UPDATESUBRESOURCE_INVALIDDESTINATIONBOX = 0x120
    D3D10_MESSAGE_ID_UPDATESUBRESOURCE_INVALIDDESTINATIONSTATE = 0x121
    D3D10_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_DESTINATION_INVALID = 0x122
    D3D10_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_DESTINATION_SUBRESOURCE_INVALID = 0x123
    D3D10_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_SOURCE_INVALID = 0x124
    D3D10_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_SOURCE_SUBRESOURCE_INVALID = 0x125
    D3D10_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_FORMAT_INVALID = 0x126
    D3D10_MESSAGE_ID_BUFFER_MAP_INVALIDMAPTYPE = 0x127
    D3D10_MESSAGE_ID_BUFFER_MAP_INVALIDFLAGS = 0x128
    D3D10_MESSAGE_ID_BUFFER_MAP_ALREADYMAPPED = 0x129
    D3D10_MESSAGE_ID_BUFFER_MAP_DEVICEREMOVED_RETURN = 0x12A
    D3D10_MESSAGE_ID_BUFFER_UNMAP_NOTMAPPED = 0x12B
    D3D10_MESSAGE_ID_TEXTURE1D_MAP_INVALIDMAPTYPE = 0x12C
    D3D10_MESSAGE_ID_TEXTURE1D_MAP_INVALIDSUBRESOURCE = 0x12D
    D3D10_MESSAGE_ID_TEXTURE1D_MAP_INVALIDFLAGS = 0x12E
    D3D10_MESSAGE_ID_TEXTURE1D_MAP_ALREADYMAPPED = 0x12F
    D3D10_MESSAGE_ID_TEXTURE1D_MAP_DEVICEREMOVED_RETURN = 0x130
    D3D10_MESSAGE_ID_TEXTURE1D_UNMAP_INVALIDSUBRESOURCE = 0x131
    D3D10_MESSAGE_ID_TEXTURE1D_UNMAP_NOTMAPPED = 0x132
    D3D10_MESSAGE_ID_TEXTURE2D_MAP_INVALIDMAPTYPE = 0x133
    D3D10_MESSAGE_ID_TEXTURE2D_MAP_INVALIDSUBRESOURCE = 0x134
    D3D10_MESSAGE_ID_TEXTURE2D_MAP_INVALIDFLAGS = 0x135
    D3D10_MESSAGE_ID_TEXTURE2D_MAP_ALREADYMAPPED = 0x136
    D3D10_MESSAGE_ID_TEXTURE2D_MAP_DEVICEREMOVED_RETURN = 0x137
    D3D10_MESSAGE_ID_TEXTURE2D_UNMAP_INVALIDSUBRESOURCE = 0x138
    D3D10_MESSAGE_ID_TEXTURE2D_UNMAP_NOTMAPPED = 0x139
    D3D10_MESSAGE_ID_TEXTURE3D_MAP_INVALIDMAPTYPE = 0x13A
    D3D10_MESSAGE_ID_TEXTURE3D_MAP_INVALIDSUBRESOURCE = 0x13B
    D3D10_MESSAGE_ID_TEXTURE3D_MAP_INVALIDFLAGS = 0x13C
    D3D10_MESSAGE_ID_TEXTURE3D_MAP_ALREADYMAPPED = 0x13D
    D3D10_MESSAGE_ID_TEXTURE3D_MAP_DEVICEREMOVED_RETURN = 0x13E
    D3D10_MESSAGE_ID_TEXTURE3D_UNMAP_INVALIDSUBRESOURCE = 0x13F
    D3D10_MESSAGE_ID_TEXTURE3D_UNMAP_NOTMAPPED = 0x140
    D3D10_MESSAGE_ID_CHECKFORMATSUPPORT_FORMAT_DEPRECATED = 0x141
    D3D10_MESSAGE_ID_CHECKMULTISAMPLEQUALITYLEVELS_FORMAT_DEPRECATED = 0x142
    D3D10_MESSAGE_ID_SETEXCEPTIONMODE_UNRECOGNIZEDFLAGS = 0x143
    D3D10_MESSAGE_ID_SETEXCEPTIONMODE_INVALIDARG_RETURN = 0x144
    D3D10_MESSAGE_ID_SETEXCEPTIONMODE_DEVICEREMOVED_RETURN = 0x145
    D3D10_MESSAGE_ID_REF_SIMULATING_INFINITELY_FAST_HARDWARE = 0x146
    D3D10_MESSAGE_ID_REF_THREADING_MODE = 0x147
    D3D10_MESSAGE_ID_REF_UMDRIVER_EXCEPTION = 0x148
    D3D10_MESSAGE_ID_REF_KMDRIVER_EXCEPTION = 0x149
    D3D10_MESSAGE_ID_REF_HARDWARE_EXCEPTION = 0x14A
    D3D10_MESSAGE_ID_REF_ACCESSING_INDEXABLE_TEMP_OUT_OF_RANGE = 0x14B
    D3D10_MESSAGE_ID_REF_PROBLEM_PARSING_SHADER = 0x14C
    D3D10_MESSAGE_ID_REF_OUT_OF_MEMORY = 0x14D
    D3D10_MESSAGE_ID_REF_INFO = 0x14E
    D3D10_MESSAGE_ID_DEVICE_DRAW_VERTEXPOS_OVERFLOW = 0x14F
    D3D10_MESSAGE_ID_DEVICE_DRAWINDEXED_INDEXPOS_OVERFLOW = 0x150
    D3D10_MESSAGE_ID_DEVICE_DRAWINSTANCED_VERTEXPOS_OVERFLOW = 0x151
    D3D10_MESSAGE_ID_DEVICE_DRAWINSTANCED_INSTANCEPOS_OVERFLOW = 0x152
    D3D10_MESSAGE_ID_DEVICE_DRAWINDEXEDINSTANCED_INSTANCEPOS_OVERFLOW = 0x153
    D3D10_MESSAGE_ID_DEVICE_DRAWINDEXEDINSTANCED_INDEXPOS_OVERFLOW = 0x154
    D3D10_MESSAGE_ID_DEVICE_DRAW_VERTEX_SHADER_NOT_SET = 0x155
    D3D10_MESSAGE_ID_DEVICE_SHADER_LINKAGE_SEMANTICNAME_NOT_FOUND = 0x156
    D3D10_MESSAGE_ID_DEVICE_SHADER_LINKAGE_REGISTERINDEX = 0x157
    D3D10_MESSAGE_ID_DEVICE_SHADER_LINKAGE_COMPONENTTYPE = 0x158
    D3D10_MESSAGE_ID_DEVICE_SHADER_LINKAGE_REGISTERMASK = 0x159
    D3D10_MESSAGE_ID_DEVICE_SHADER_LINKAGE_SYSTEMVALUE = 0x15A
    D3D10_MESSAGE_ID_DEVICE_SHADER_LINKAGE_NEVERWRITTEN_ALWAYSREADS = 0x15B
    D3D10_MESSAGE_ID_DEVICE_DRAW_VERTEX_BUFFER_NOT_SET = 0x15C
    D3D10_MESSAGE_ID_DEVICE_DRAW_INPUTLAYOUT_NOT_SET = 0x15D
    D3D10_MESSAGE_ID_DEVICE_DRAW_CONSTANT_BUFFER_NOT_SET = 0x15E
    D3D10_MESSAGE_ID_DEVICE_DRAW_CONSTANT_BUFFER_TOO_SMALL = 0x15F
    D3D10_MESSAGE_ID_DEVICE_DRAW_SAMPLER_NOT_SET = 0x160
    D3D10_MESSAGE_ID_DEVICE_DRAW_SHADERRESOURCEVIEW_NOT_SET = 0x161
    D3D10_MESSAGE_ID_DEVICE_DRAW_VIEW_DIMENSION_MISMATCH = 0x162
    D3D10_MESSAGE_ID_DEVICE_DRAW_VERTEX_BUFFER_STRIDE_TOO_SMALL = 0x163
    D3D10_MESSAGE_ID_DEVICE_DRAW_VERTEX_BUFFER_TOO_SMALL = 0x164
    D3D10_MESSAGE_ID_DEVICE_DRAW_INDEX_BUFFER_NOT_SET = 0x165
    D3D10_MESSAGE_ID_DEVICE_DRAW_INDEX_BUFFER_FORMAT_INVALID = 0x166
    D3D10_MESSAGE_ID_DEVICE_DRAW_INDEX_BUFFER_TOO_SMALL = 0x167
    D3D10_MESSAGE_ID_DEVICE_DRAW_GS_INPUT_PRIMITIVE_MISMATCH = 0x168
    D3D10_MESSAGE_ID_DEVICE_DRAW_RESOURCE_RETURN_TYPE_MISMATCH = 0x169
    D3D10_MESSAGE_ID_DEVICE_DRAW_POSITION_NOT_PRESENT = 0x16A
    D3D10_MESSAGE_ID_DEVICE_DRAW_OUTPUT_STREAM_NOT_SET = 0x16B
    D3D10_MESSAGE_ID_DEVICE_DRAW_BOUND_RESOURCE_MAPPED = 0x16C
    D3D10_MESSAGE_ID_DEVICE_DRAW_INVALID_PRIMITIVETOPOLOGY = 0x16D
    D3D10_MESSAGE_ID_DEVICE_DRAW_VERTEX_OFFSET_UNALIGNED = 0x16E
    D3D10_MESSAGE_ID_DEVICE_DRAW_VERTEX_STRIDE_UNALIGNED = 0x16F
    D3D10_MESSAGE_ID_DEVICE_DRAW_INDEX_OFFSET_UNALIGNED = 0x170
    D3D10_MESSAGE_ID_DEVICE_DRAW_OUTPUT_STREAM_OFFSET_UNALIGNED = 0x171
    D3D10_MESSAGE_ID_DEVICE_DRAW_RESOURCE_FORMAT_LD_UNSUPPORTED = 0x172
    D3D10_MESSAGE_ID_DEVICE_DRAW_RESOURCE_FORMAT_SAMPLE_UNSUPPORTED = 0x173
    D3D10_MESSAGE_ID_DEVICE_DRAW_RESOURCE_FORMAT_SAMPLE_C_UNSUPPORTED = 0x174
    D3D10_MESSAGE_ID_DEVICE_DRAW_RESOURCE_MULTISAMPLE_UNSUPPORTED = 0x175
    D3D10_MESSAGE_ID_DEVICE_DRAW_SO_TARGETS_BOUND_WITHOUT_SOURCE = 0x176
    D3D10_MESSAGE_ID_DEVICE_DRAW_SO_STRIDE_LARGER_THAN_BUFFER = 0x177
    D3D10_MESSAGE_ID_DEVICE_DRAW_OM_RENDER_TARGET_DOES_NOT_SUPPORT_BLENDING = 0x178
    D3D10_MESSAGE_ID_DEVICE_DRAW_OM_DUAL_SOURCE_BLENDING_CAN_ONLY_HAVE_RENDER_TARGET_0 = 0x179
    D3D10_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_AT_FAULT = 0x17A
    D3D10_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_POSSIBLY_AT_FAULT = 0x17B
    D3D10_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_NOT_AT_FAULT = 0x17C
    D3D10_MESSAGE_ID_DEVICE_OPEN_SHARED_RESOURCE_INVALIDARG_RETURN = 0x17D
    D3D10_MESSAGE_ID_DEVICE_OPEN_SHARED_RESOURCE_OUTOFMEMORY_RETURN = 0x17E
    D3D10_MESSAGE_ID_DEVICE_OPEN_SHARED_RESOURCE_BADINTERFACE_RETURN = 0x17F
    D3D10_MESSAGE_ID_DEVICE_DRAW_VIEWPORT_NOT_SET = 0x180
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_TRAILING_DIGIT_IN_SEMANTIC = 0x181
    D3D10_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_TRAILING_DIGIT_IN_SEMANTIC = 0x182
    D3D10_MESSAGE_ID_DEVICE_RSSETVIEWPORTS_DENORMFLUSH = 0x183
    D3D10_MESSAGE_ID_OMSETRENDERTARGETS_INVALIDVIEW = 0x184
    D3D10_MESSAGE_ID_DEVICE_SETTEXTFILTERSIZE_INVALIDDIMENSIONS = 0x185
    D3D10_MESSAGE_ID_DEVICE_DRAW_SAMPLER_MISMATCH = 0x186
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_TYPE_MISMATCH = 0x187
    D3D10_MESSAGE_ID_BLENDSTATE_GETDESC_LEGACY = 0x188
    D3D10_MESSAGE_ID_SHADERRESOURCEVIEW_GETDESC_LEGACY = 0x189
    D3D10_MESSAGE_ID_CREATEQUERY_OUTOFMEMORY_RETURN = 0x18A
    D3D10_MESSAGE_ID_CREATEPREDICATE_OUTOFMEMORY_RETURN = 0x18B
    D3D10_MESSAGE_ID_CREATECOUNTER_OUTOFRANGE_COUNTER = 0x18C
    D3D10_MESSAGE_ID_CREATECOUNTER_SIMULTANEOUS_ACTIVE_COUNTERS_EXHAUSTED = 0x18D
    D3D10_MESSAGE_ID_CREATECOUNTER_UNSUPPORTED_WELLKNOWN_COUNTER = 0x18E
    D3D10_MESSAGE_ID_CREATECOUNTER_OUTOFMEMORY_RETURN = 0x18F
    D3D10_MESSAGE_ID_CREATECOUNTER_NONEXCLUSIVE_RETURN = 0x190
    D3D10_MESSAGE_ID_CREATECOUNTER_NULLDESC = 0x191
    D3D10_MESSAGE_ID_CHECKCOUNTER_OUTOFRANGE_COUNTER = 0x192
    D3D10_MESSAGE_ID_CHECKCOUNTER_UNSUPPORTED_WELLKNOWN_COUNTER = 0x193
    D3D10_MESSAGE_ID_SETPREDICATION_INVALID_PREDICATE_STATE = 0x194
    D3D10_MESSAGE_ID_QUERY_BEGIN_UNSUPPORTED = 0x195
    D3D10_MESSAGE_ID_PREDICATE_BEGIN_DURING_PREDICATION = 0x196
    D3D10_MESSAGE_ID_QUERY_BEGIN_DUPLICATE = 0x197
    D3D10_MESSAGE_ID_QUERY_BEGIN_ABANDONING_PREVIOUS_RESULTS = 0x198
    D3D10_MESSAGE_ID_PREDICATE_END_DURING_PREDICATION = 0x199
    D3D10_MESSAGE_ID_QUERY_END_ABANDONING_PREVIOUS_RESULTS = 0x19A
    D3D10_MESSAGE_ID_QUERY_END_WITHOUT_BEGIN = 0x19B
    D3D10_MESSAGE_ID_QUERY_GETDATA_INVALID_DATASIZE = 0x19C
    D3D10_MESSAGE_ID_QUERY_GETDATA_INVALID_FLAGS = 0x19D
    D3D10_MESSAGE_ID_QUERY_GETDATA_INVALID_CALL = 0x19E
    D3D10_MESSAGE_ID_DEVICE_DRAW_PS_OUTPUT_TYPE_MISMATCH = 0x19F
    D3D10_MESSAGE_ID_DEVICE_DRAW_RESOURCE_FORMAT_GATHER_UNSUPPORTED = 0x1A0
    D3D10_MESSAGE_ID_DEVICE_DRAW_INVALID_USE_OF_CENTER_MULTISAMPLE_PATTERN = 0x1A1
    D3D10_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_STRIDE_TOO_LARGE = 0x1A2
    D3D10_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_INVALIDRANGE = 0x1A3
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_EMPTY_LAYOUT = 0x1A4
    D3D10_MESSAGE_ID_DEVICE_DRAW_RESOURCE_SAMPLE_COUNT_MISMATCH = 0x1A5
    D3D10_MESSAGE_ID_LIVE_OBJECT_SUMMARY = 0x1A6
    D3D10_MESSAGE_ID_LIVE_BUFFER = 0x1A7
    D3D10_MESSAGE_ID_LIVE_TEXTURE1D = 0x1A8
    D3D10_MESSAGE_ID_LIVE_TEXTURE2D = 0x1A9
    D3D10_MESSAGE_ID_LIVE_TEXTURE3D = 0x1AA
    D3D10_MESSAGE_ID_LIVE_SHADERRESOURCEVIEW = 0x1AB
    D3D10_MESSAGE_ID_LIVE_RENDERTARGETVIEW = 0x1AC
    D3D10_MESSAGE_ID_LIVE_DEPTHSTENCILVIEW = 0x1AD
    D3D10_MESSAGE_ID_LIVE_VERTEXSHADER = 0x1AE
    D3D10_MESSAGE_ID_LIVE_GEOMETRYSHADER = 0x1AF
    D3D10_MESSAGE_ID_LIVE_PIXELSHADER = 0x1B0
    D3D10_MESSAGE_ID_LIVE_INPUTLAYOUT = 0x1B1
    D3D10_MESSAGE_ID_LIVE_SAMPLER = 0x1B2
    D3D10_MESSAGE_ID_LIVE_BLENDSTATE = 0x1B3
    D3D10_MESSAGE_ID_LIVE_DEPTHSTENCILSTATE = 0x1B4
    D3D10_MESSAGE_ID_LIVE_RASTERIZERSTATE = 0x1B5
    D3D10_MESSAGE_ID_LIVE_QUERY = 0x1B6
    D3D10_MESSAGE_ID_LIVE_PREDICATE = 0x1B7
    D3D10_MESSAGE_ID_LIVE_COUNTER = 0x1B8
    D3D10_MESSAGE_ID_LIVE_DEVICE = 0x1B9
    D3D10_MESSAGE_ID_LIVE_SWAPCHAIN = 0x1BA
    D3D10_MESSAGE_ID_D3D10_MESSAGES_END = 0x1BB
    D3D10_MESSAGE_ID_D3D10L9_MESSAGES_START = 0x100000
    D3D10_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_STENCIL_NO_TWO_SIDED = 0x100001
    D3D10_MESSAGE_ID_CREATERASTERIZERSTATE_DepthBiasClamp_NOT_SUPPORTED = 0x100002
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_NO_COMPARISON_SUPPORT = 0x100003
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_EXCESSIVE_ANISOTROPY = 0x100004
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_BORDER_OUT_OF_RANGE = 0x100005
    D3D10_MESSAGE_ID_VSSETSAMPLERS_NOT_SUPPORTED = 0x100006
    D3D10_MESSAGE_ID_VSSETSAMPLERS_TOO_MANY_SAMPLERS = 0x100007
    D3D10_MESSAGE_ID_PSSETSAMPLERS_TOO_MANY_SAMPLERS = 0x100008
    D3D10_MESSAGE_ID_CREATERESOURCE_NO_ARRAYS = 0x100009
    D3D10_MESSAGE_ID_CREATERESOURCE_NO_VB_AND_IB_BIND = 0x10000A
    D3D10_MESSAGE_ID_CREATERESOURCE_NO_TEXTURE_1D = 0x10000B
    D3D10_MESSAGE_ID_CREATERESOURCE_DIMENSION_OUT_OF_RANGE = 0x10000C
    D3D10_MESSAGE_ID_CREATERESOURCE_NOT_BINDABLE_AS_SHADER_RESOURCE = 0x10000D
    D3D10_MESSAGE_ID_OMSETRENDERTARGETS_TOO_MANY_RENDER_TARGETS = 0x10000E
    D3D10_MESSAGE_ID_OMSETRENDERTARGETS_NO_DIFFERING_BIT_DEPTHS = 0x10000F
    D3D10_MESSAGE_ID_IASETVERTEXBUFFERS_BAD_BUFFER_INDEX = 0x100010
    D3D10_MESSAGE_ID_DEVICE_RSSETVIEWPORTS_TOO_MANY_VIEWPORTS = 0x100011
    D3D10_MESSAGE_ID_DEVICE_IASETPRIMITIVETOPOLOGY_ADJACENCY_UNSUPPORTED = 0x100012
    D3D10_MESSAGE_ID_DEVICE_RSSETSCISSORRECTS_TOO_MANY_SCISSORS = 0x100013
    D3D10_MESSAGE_ID_COPYRESOURCE_ONLY_TEXTURE_2D_WITHIN_GPU_MEMORY = 0x100014
    D3D10_MESSAGE_ID_COPYRESOURCE_NO_TEXTURE_3D_READBACK = 0x100015
    D3D10_MESSAGE_ID_COPYRESOURCE_NO_TEXTURE_ONLY_READBACK = 0x100016
    D3D10_MESSAGE_ID_CREATEINPUTLAYOUT_UNSUPPORTED_FORMAT = 0x100017
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_NO_ALPHA_TO_COVERAGE = 0x100018
    D3D10_MESSAGE_ID_CREATERASTERIZERSTATE_DepthClipEnable_MUST_BE_TRUE = 0x100019
    D3D10_MESSAGE_ID_DRAWINDEXED_STARTINDEXLOCATION_MUST_BE_POSITIVE = 0x10001A
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_MUST_USE_LOWEST_LOD = 0x10001B
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_MINLOD_MUST_NOT_BE_FRACTIONAL = 0x10001C
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_MAXLOD_MUST_BE_FLT_MAX = 0x10001D
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_FIRSTARRAYSLICE_MUST_BE_ZERO = 0x10001E
    D3D10_MESSAGE_ID_CREATESHADERRESOURCEVIEW_CUBES_MUST_HAVE_6_SIDES = 0x10001F
    D3D10_MESSAGE_ID_CREATERESOURCE_NOT_BINDABLE_AS_RENDER_TARGET = 0x100020
    D3D10_MESSAGE_ID_CREATERESOURCE_NO_DWORD_INDEX_BUFFER = 0x100021
    D3D10_MESSAGE_ID_CREATERESOURCE_MSAA_PRECLUDES_SHADER_RESOURCE = 0x100022
    D3D10_MESSAGE_ID_CREATERESOURCE_PRESENTATION_PRECLUDES_SHADER_RESOURCE = 0x100023
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_NO_INDEPENDENT_BLEND_ENABLE = 0x100024
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_NO_INDEPENDENT_WRITE_MASKS = 0x100025
    D3D10_MESSAGE_ID_CREATERESOURCE_NO_STREAM_OUT = 0x100026
    D3D10_MESSAGE_ID_CREATERESOURCE_ONLY_VB_IB_FOR_BUFFERS = 0x100027
    D3D10_MESSAGE_ID_CREATERESOURCE_NO_AUTOGEN_FOR_VOLUMES = 0x100028
    D3D10_MESSAGE_ID_CREATERESOURCE_DXGI_FORMAT_R8G8B8A8_CANNOT_BE_SHARED = 0x100029
    D3D10_MESSAGE_ID_VSSHADERRESOURCES_NOT_SUPPORTED = 0x10002A
    D3D10_MESSAGE_ID_GEOMETRY_SHADER_NOT_SUPPORTED = 0x10002B
    D3D10_MESSAGE_ID_STREAM_OUT_NOT_SUPPORTED = 0x10002C
    D3D10_MESSAGE_ID_TEXT_FILTER_NOT_SUPPORTED = 0x10002D
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_NO_SEPARATE_ALPHA_BLEND = 0x10002E
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_NO_MRT_BLEND = 0x10002F
    D3D10_MESSAGE_ID_CREATEBLENDSTATE_OPERATION_NOT_SUPPORTED = 0x100030
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_NO_MIRRORONCE = 0x100031
    D3D10_MESSAGE_ID_DRAWINSTANCED_NOT_SUPPORTED = 0x100032
    D3D10_MESSAGE_ID_DRAWINDEXEDINSTANCED_NOT_SUPPORTED_BELOW_9_3 = 0x100033
    D3D10_MESSAGE_ID_DRAWINDEXED_POINTLIST_UNSUPPORTED = 0x100034
    D3D10_MESSAGE_ID_SETBLENDSTATE_SAMPLE_MASK_CANNOT_BE_ZERO = 0x100035
    D3D10_MESSAGE_ID_CREATERESOURCE_DIMENSION_EXCEEDS_FEATURE_LEVEL_DEFINITION = 0x100036
    D3D10_MESSAGE_ID_CREATERESOURCE_ONLY_SINGLE_MIP_LEVEL_DEPTH_STENCIL_SUPPORTED = 0x100037
    D3D10_MESSAGE_ID_DEVICE_RSSETSCISSORRECTS_NEGATIVESCISSOR = 0x100038
    D3D10_MESSAGE_ID_SLOT_ZERO_MUST_BE_D3D10_INPUT_PER_VERTEX_DATA = 0x100039
    D3D10_MESSAGE_ID_CREATERESOURCE_NON_POW_2_MIPMAP = 0x10003A
    D3D10_MESSAGE_ID_CREATESAMPLERSTATE_BORDER_NOT_SUPPORTED = 0x10003B
    D3D10_MESSAGE_ID_OMSETRENDERTARGETS_NO_SRGB_MRT = 0x10003C
    D3D10_MESSAGE_ID_COPYRESOURCE_NO_3D_MISMATCHED_UPDATES = 0x10003D
    D3D10_MESSAGE_ID_D3D10L9_MESSAGES_END = 0x10003E

class WICNamedWhitePoint(IntEnum):
    WICWhitePointDefault = 0x1
    WICWhitePointDaylight = 0x2
    WICWhitePointCloudy = 0x4
    WICWhitePointShade = 0x8
    WICWhitePointTungsten = 0x10
    WICWhitePointFluorescent = 0x20
    WICWhitePointFlash = 0x40
    WICWhitePointUnderwater = 0x80
    WICWhitePointCustom = 0x100
    WICWhitePointAutoWhiteBalance = 0x200
    WICNAMEDWHITEPOINT_FORCE_DWORD = 0x7FFFFFFF

class WICMetadataCreationOptions(IntEnum):
    WICMetadataCreationDefault = 0x0
    WICMetadataCreationFailUnknown = 0x10000
    WICMetadataCreationMask = 0xFFFF0000

class NVSDK_NGX_EngineType(IntEnum):
    NVSDK_NGX_ENGINE_TYPE_CUSTOM = 0x0
    NVSDK_NGX_ENGINE_TYPE_UNREAL = 0x1
    NVSDK_NGX_ENGINE_TYPE_UNITY = 0x2
    NVSDK_NGX_ENGINE_TYPE_OMNIVERSE = 0x3
    NVSDK_NGX_ENGINE_COUNT = 0x4

class __scrt_native_startup_state(IntEnum):
    uninitialized = 0x0
    initializing = 0x1
    initialized = 0x2

class _FS_BPIO_INFLAGS(IntEnum):
    FSBPIO_INFL_None = 0x0
    FSBPIO_INFL_SKIP_STORAGE_STACK_QUERY = 0x1

class _FS_BPIO_OUTFLAGS(IntEnum):
    FSBPIO_OUTFL_None = 0x0
    FSBPIO_OUTFL_VOLUME_STACK_BYPASS_PAUSED = 0x1
    FSBPIO_OUTFL_STREAM_BYPASS_PAUSED = 0x2
    FSBPIO_OUTFL_FILTER_ATTACH_BLOCKED = 0x4
    FSBPIO_OUTFL_COMPATIBLE_STORAGE_DRIVER = 0x8

class NV_GPU_PERF_RATED_TDP_CLIENT(IntEnum):
    NV_GPU_PERF_RATED_TDP_CLIENT_RM = 0x0
    NV_GPU_PERF_RATED_TDP_CLIENT_GLOBAL = 0x1
    NV_GPU_PERF_RATED_TDP_CLIENT_OS = 0x2
    NV_GPU_PERF_RATED_TDP_CLIENT_PROFILE = 0x3
    NV_GPU_PERF_RATED_TDP_CLIENT_WAR_BUG_1785342 = 0x4
    NV_GPU_PERF_RATED_TDP_CLIENT_NUM_CLIENTS = 0x5

class NV_GPU_CLOCK_CLK_VF_POINT_TYPE(IntEnum):
    NV_GPU_CLOCK_CLK_VF_POINT_TYPE_FREQ = 0x0
    NV_GPU_CLOCK_CLK_VF_POINT_TYPE_VOLT = 0x1
    NV_GPU_CLOCK_CLK_VF_POINT_TYPE_30 = 0x2
    NV_GPU_CLOCK_CLK_VF_POINT_TYPE_35_FREQ = 0x3
    NV_GPU_CLOCK_CLK_VF_POINT_TYPE_35_VOLT = 0x4
    NV_GPU_CLOCK_CLK_VF_POINT_TYPE_35 = 0x5
    NV_GPU_CLOCK_CLK_VF_POINT_TYPE_35_VOLT_SEC = 0x6
    NV_GPU_CLOCK_CLK_VF_POINT_TYPE_UNKNOWN = 0xFFFFFFFF

class _NV_PERF_CLIENT_LIMIT_ID(IntEnum):
    NVAPI_PERF_CLIENT_LIMIT_0_MAX = 0x0
    NVAPI_PERF_CLIENT_LIMIT_0_MIN = 0x1
    NVAPI_PERF_CLIENT_LIMIT_1_MAX = 0x2
    NVAPI_PERF_CLIENT_LIMIT_1_MIN = 0x3
    NVAPI_PERF_CLIENT_LIMIT_2_MAX = 0x4
    NVAPI_PERF_CLIENT_LIMIT_2_MIN = 0x5
    NVAPI_PERF_CLIENT_LIMIT_VOLTAGE = 0x6
    NVAPI_PERF_CLIENT_LIMIT_MAX_LIMITS = 0x20

class NV_GPU_PERF_PERF_LIMITS_ARBITRATION_OUTPUT_TUPLE_IDX(IntEnum):
    NV_GPU_PERF_PERF_LIMITS_ARBITRATION_OUTPUT_TUPLE_IDX_MIN = 0x0
    NV_GPU_PERF_PERF_LIMITS_ARBITRATION_OUTPUT_TUPLE_IDX_MAX = 0x1
    NV_GPU_PERF_PERF_LIMITS_ARBITRATION_OUTPUT_TUPLE_IDX_NUM_IDXS = 0x2

class NvAPIPrivateConstDataSlot(IntEnum):
    PCD_NEXUS_FRAME_AND_DC_INDEX = 0x0
    PCD_NEXUS_DC_EXTRA_INFO = 0x1
    PCD_NEXUS_TRAP_HANDLER_DUMP_GPU_VIRTUAL_ADDRESS = 0x2
    PCD_NEXUS_COMPUTE_START = 0x3
    PCD_NEXUS_COMPUTE_DC_EXTRA_INFO = 0x4
    PCD_COUNT = 0x5

class _NVAPI_VIDEO_STATE_COMPONENT_ID(IntEnum):
    NVAPI_VIDEO_STATE_COMPONENT_BRIGHTNESS = 0x0
    NVAPI_VIDEO_STATE_COMPONENT_CONTRAST = 0x1
    NVAPI_VIDEO_STATE_COMPONENT_HUE = 0x2
    NVAPI_VIDEO_STATE_COMPONENT_SATURATION = 0x3
    NVAPI_VIDEO_STATE_COMPONENT_COLORTEMP = 0x4
    NVAPI_VIDEO_STATE_COMPONENT_Y_GAMMA = 0x5
    NVAPI_VIDEO_STATE_COMPONENT_RGB_GAMMA_R = 0x6
    NVAPI_VIDEO_STATE_COMPONENT_RGB_GAMMA_G = 0x7
    NVAPI_VIDEO_STATE_COMPONENT_RGB_GAMMA_B = 0x8
    NVAPI_VIDEO_STATE_COMPONENT_COLOR_SPACE = 0x9
    NVAPI_VIDEO_STATE_COMPONENT_COLOR_RANGE = 0xA
    NVAPI_VIDEO_STATE_COMPONENT_PLAYBACK_PROFILE = 0xB
    NVAPI_VIDEO_STATE_COMPONENT_DEINTERLACE = 0xC
    NVAPI_VIDEO_STATE_COMPONENT_SCALING = 0xD
    NVAPI_VIDEO_STATE_COMPONENT_CADENCE = 0xE
    NVAPI_VIDEO_STATE_COMPONENT_NOISE_REDUCE = 0xF
    NVAPI_VIDEO_STATE_COMPONENT_EDGE_ENHANCE = 0x10
    NVAPI_VIDEO_STATE_COMPONENT_OVERDRIVE = 0x11
    NVAPI_VIDEO_STATE_COMPONENT_SPLITSCREEN = 0x12
    NVAPI_VIDEO_STATE_COMPONENT_DEBLOCKING = 0x13
    NVAPI_VIDEO_STATE_COMPONENT_DYNAMIC_CONTRAST = 0x14
    NVAPI_VIDEO_STATE_COMPONENT_GREEN_STRETCH = 0x15
    NVAPI_VIDEO_STATE_COMPONENT_BLUE_STRETCH = 0x16
    NVAPI_VIDEO_STATE_COMPONENT_SKIN_TONE_CORRECTION = 0x17
    NVAPI_VIDEO_STATE_COMPONENT_GAMUT_REMAPPING = 0x18
    NVAPI_VIDEO_STATE_COMPONENT_2DTO3D = 0x19
    NVAPI_VIDEO_STATE_COMPONENT_3D_ANALYSIS = 0x1A
    NVAPI_VIDEO_STATE_COMPONENT_FRC = 0x1B
    NVAPI_VIDEO_STATE_COMPONENT_BEN = 0x1C
    NVAPI_VIDEO_STATE_COMPONENT_ID_LAST = 0x1D
    NVAPI_VIDEO_STATE_COMPONENT_ID_NONE = 0xFFFFFFFF

class _NVAPI_FUNCTION_NAME(IntEnum):
    NVAPI_DecodeExecute = 0x0
    NVAPI_DecodeBeginFrame = 0x1
    NVAPI_DecodeEndFrame = 0x2
    NVAPI_VideoProcessBlt = 0x3
    NVAPI_ExtensionExecute = 0x4
    NVAPI_FlipOverlay = 0x5
    NVAPI_FlipOverlayNvApi = 0x6
    NVAPI_OverlayTransfer = 0x7
    NVAPI_nvRenderDeinterlace = 0x8
    NVAPI_PresentVideoShow = 0x9
    NVAPI_LAST_FUNCTION = 0xA

class _NVAPI_DITHER_BITS(IntEnum):
    NVAPI_DITHER_BITS_6BITS = 0x0
    NVAPI_DITHER_BITS_8BITS = 0x1
    NVAPI_DITHER_BITS_10BITS = 0x2

class _NVAPI_DITHER_MODE(IntEnum):
    NVAPI_DITHER_MODE_DYNAMIC_ERR_ACC = 0x0
    NVAPI_DITHER_MODE_STATIC_ERR_ACC = 0x1
    NVAPI_DITHER_MODE_DYNAMIC_2X2 = 0x2
    NVAPI_DITHER_MODE_STATIC_2X2 = 0x3
    NVAPI_DITHER_MODE_TEMPORAL = 0x4

class _NV_GPU_CLIENT_VOLT_DOMAIN_ID(IntEnum):
    NV_GPU_CLIENT_VOLT_DOMAIN_CORE = 0x0
    NV_GPU_CLIENT_VOLT_DOMAIN_MAX_ENTRIES = 0x1

class NV_GPU_POWER_POLICY_GEMINI_GPU(IntEnum):
    NV_GPU_POWER_POLICY_GEMINI_GPU_SLAVE = 0x0
    NV_GPU_POWER_POLICY_GEMINI_GPU_MASTER = 0x1
    NV_GPU_POWER_POLICY_GEMINI_MAX_GPUS = 0x2

class _NV_GPU_PERF_LIMIT_ID(IntEnum):
    NVAPI_PERF_LIMIT_ID_PERFMON = 0x0
    NVAPI_PERF_LIMIT_ID_NO_CLIENT = 0x1
    NVAPI_PERF_LIMIT_ID_SCREEN_SAVER = 0x2
    NVAPI_PERF_LIMIT_ID_NO_HIRES = 0x3
    NVAPI_PERF_LIMIT_ID_OS_LEVEL = 0x4
    NVAPI_PERF_LIMIT_ID_SPDIFF_GLITCH = 0x5
    NVAPI_PERF_LIMIT_ID_DISPLAY_GLITCH = 0x6
    NVAPI_PERF_LIMIT_ID_UNLOAD_DRIVER = 0x7
    NVAPI_PERF_LIMIT_ID_POWERMIZER = 0x8
    NVAPI_PERF_LIMIT_ID_STRESSTEST_FAILURE = 0x9
    NVAPI_PERF_LIMIT_ID_RC_ERROR = 0xA
    NVAPI_PERF_LIMIT_ID_MIN_FAN_LEVEL = 0xB
    NVAPI_PERF_LIMIT_ID_MCLK_CLONE = 0xC
    NVAPI_PERF_LIMIT_ID_OVERLAY = 0xD
    NVAPI_PERF_LIMIT_ID_HIGHRES = 0xE
    NVAPI_PERF_LIMIT_ID_BANDWIDTHFACTOR = 0xF
    NVAPI_PERF_LIMIT_ID_HD_FRAMEDROP_WAR = 0x10
    NVAPI_PERF_LIMIT_ID_ISMODEPOSSIBLE = 0x11
    NVAPI_PERF_LIMIT_ID_STRESSTEST_SETUP = 0x12
    NVAPI_PERF_LIMIT_ID_FORCED = 0x13
    NVAPI_PERF_LIMIT_ID_FORCED_DACPERFTEST = 0x14
    NVAPI_PERF_LIMIT_ID_FORCED_STRESSTEST = 0x15
    NVAPI_PERF_LIMIT_ID_POWERMIZER_HARD = 0x16
    NVAPI_PERF_LIMIT_ID_THERMAL = 0x17
    NVAPI_PERF_LIMIT_ID_SYSPERF = 0x18
    NVAPI_PERF_LIMIT_ID_PWR_SUPPLY_CAPACITY = 0x19
    NVAPI_PERF_LIMIT_ID_SW_BATTPOWER = 0x1A
    NVAPI_PERF_LIMIT_ID_EXT_PERF_CONTROL = 0x1B
    NVAPI_PERF_LIMIT_ID_MXM_ACPOWER = 0x1C
    NVAPI_PERF_LIMIT_ID_AUX_POWER = 0x1D
    NVAPI_PERF_LIMIT_ID_3D_WAR = 0x1E
    NVAPI_PERF_LIMIT_ID_DEEP_IDLE = 0x1F
    NVAPI_PERF_LIMIT_ID_CLIENT_SOFT = 0x20
    NVAPI_PERF_LIMIT_ID_CLIENT_HARD = 0x21
    NVAPI_PERF_LIMIT_ID_OVERCLOCK = 0x22
    NVAPI_PERF_LIMIT_ID_FORCED_LINKTRAIN = 0x23
    NVAPI_PERF_LIMIT_ID_POWER_BALANCE = 0x24
    NVAPI_PERF_LIMIT_ID_BUG_535734 = 0x25
    NVAPI_PERF_LIMIT_ID_BOOST = 0x26
    NVAPI_PERF_LIMIT_ID_PM_DYNAMIC = 0x27
    NVAPI_PERF_LIMIT_ID_MODS_RULES = 0x28
    NVAPI_PERF_LIMIT_ID_EXCEPT_VIDEO = 0x29
    NVAPI_PERF_LIMIT_ID_SDI_INPUT_CAPTURE = 0x2A
    NVAPI_PERF_LIMIT_ID_BANDWIDTH_HCLONE = 0x2B
    NVAPI_PERF_LIMIT_ID_VPS_DISPLAY = 0x2C
    NVAPI_PERF_LIMIT_ID_VPS = 0x2D
    NVAPI_PERF_LIMIT_ID_CANOAS_MODE = 0x2E
    NVAPI_PERF_LIMIT_ID_BUG_660789 = 0x2F
    NVAPI_PERF_LIMIT_ID_P1020_WAR = 0x30
    NVAPI_PERF_LIMIT_ID_LOCKED_DRIVER = 0x31
    NVAPI_PERF_LIMIT_ID_PMU_OVERRIDE = 0x32
    NVAPI_PERF_LIMIT_ID_CLIENT_0_MAX = 0x33
    NVAPI_PERF_LIMIT_ID_CLIENT_0_MIN = 0x34
    NVAPI_PERF_LIMIT_ID_CLIENT_1_MAX = 0x35
    NVAPI_PERF_LIMIT_ID_CLIENT_1_MIN = 0x36
    NVAPI_PERF_LIMIT_ID_CLIENT_2_MAX = 0x37
    NVAPI_PERF_LIMIT_ID_CLIENT_2_MIN = 0x38
    NVAPI_PERF_LIMIT_ID_PERFMON_GROUP_1 = 0x39
    NVAPI_PERF_LIMIT_ID_PERFMON_GROUP_2 = 0x3A
    NVAPI_PERF_LIMIT_ID_PERFMON_GROUP_3 = 0x3B
    NVAPI_PERF_LIMIT_ID_RATED_TDP_MAX = 0x3C
    NVAPI_PERF_LIMIT_ID_OVERVOLTAGE_LOGIC = 0x3D
    NVAPI_PERF_LIMIT_ID_RELIABILITY_LOGIC = 0x3E
    NVAPI_PERF_LIMIT_ID_PMU_DOM_GRP_1 = 0x3F
    NVAPI_PERF_LIMIT_ID_AUX_PWR_STATE = 0x40
    NVAPI_PERF_LIMIT_ID_PERFORMANCE_CAP = 0x41
    NVAPI_PERF_LIMIT_ID_THERM_POLICY_DOM_GRP_0 = 0x42
    NVAPI_PERF_LIMIT_ID_THERM_POLICY_DOM_GRP_1 = 0x43
    NVAPI_PERF_LIMIT_ID_RELIABILITY_ALT_LOGIC = 0x44
    NVAPI_PERF_LIMIT_ID_PERFORMANCE_CAP1 = 0x45
    NVAPI_PERF_LIMIT_ID_RATED_TDP_MIN = 0x46
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_0_MAX = 0x47
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_0_MIN = 0x48
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_1_MAX = 0x49
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_1_MIN = 0x4A
    NVAPI_PERF_LIMIT_ID_THERM_POLICY_NVVDD = 0x4B
    NVAPI_PERF_LIMIT_ID_SLI_GPU_BOOST_DOM_GRP_0 = 0x4C
    NVAPI_PERF_LIMIT_ID_SLI_GPU_BOOST_DOM_GRP_1 = 0x4D
    NVAPI_PERF_LIMIT_ID_ISMODEPOSSIBLE_DISP = 0x4E
    NVAPI_PERF_LIMIT_ID_CUDA_MAX = 0x4F
    NVAPI_PERF_LIMIT_ID_SUSPEND_POWER = 0x50
    NVAPI_PERF_LIMIT_ID_EDP_POLICY_DOM_GRP_1 = 0x51
    NVAPI_PERF_LIMIT_ID_GPU_IS_IDLE = 0x52
    NVAPI_PERF_LIMIT_ID_GPU_IS_IDLE_GROUP1 = 0x53
    NVAPI_PERF_LIMIT_ID_SLI_DOM_GRP_0_MIN = 0x54
    NVAPI_PERF_LIMIT_ID_APPLICATIONCLOCKS = 0x55
    NVAPI_PERF_LIMIT_ID_CUSTOMER_BOOST_MAX = 0x56
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_DRAM_MAX = 0x57
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_GPC_MAX = 0x58
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_DISP_MAX = 0x59
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_DRAM_MIN = 0x5A
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_GPC_MIN = 0x5B
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_DISP_MIN = 0x5C
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_2_MAX = 0x5D
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_2_MIN = 0x5E
    NVAPI_PERF_LIMIT_ID_INTERSECT = 0x5F
    NVAPI_PERF_LIMIT_ID_INTERSECT_LOGIC_1_MAX = 0x60
    NVAPI_PERF_LIMIT_ID_INTERSECT_LOGIC_2_MAX = 0x61
    NVAPI_PERF_LIMIT_ID_INTERSECT_SRAM_0_MAX = 0x62
    NVAPI_PERF_LIMIT_ID_INTERSECT_SRAM_1_MAX = 0x63
    NVAPI_PERF_LIMIT_ID_INTERSECT_SRAM_2_MAX = 0x64
    NVAPI_PERF_LIMIT_ID_INTERSECT_LOGIC_0_MIN = 0x65
    NVAPI_PERF_LIMIT_ID_INTERSECT_LOGIC_1_MIN = 0x66
    NVAPI_PERF_LIMIT_ID_INTERSECT_LOGIC_2_MIN = 0x67
    NVAPI_PERF_LIMIT_ID_INTERSECT_SRAM_0_MIN = 0x68
    NVAPI_PERF_LIMIT_ID_INTERSECT_SRAM_1_MIN = 0x69
    NVAPI_PERF_LIMIT_ID_INTERSECT_SRAM_2_MIN = 0x6A
    NVAPI_PERF_LIMIT_ID_OVERVOLTAGE_SRAM = 0x6B
    NVAPI_PERF_LIMIT_ID_RELIABILITY_ALT_SRAM = 0x6C
    NVAPI_PERF_LIMIT_ID_RELIABILITY_SRAM = 0x6D
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOGIC = 0x6E
    NVAPI_PERF_LIMIT_ID_MODS_RULES_SRAM = 0x6F
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_INTERSECT = 0x70
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_DRAM_MAX = 0x71
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_GPC_MAX = 0x72
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_DISP_MAX = 0x73
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_DRAM_MIN = 0x74
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_GPC_MIN = 0x75
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_DISP_MIN = 0x76
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_0_MAX = 0x77
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_1_MAX = 0x78
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_2_MAX = 0x79
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_0_MIN = 0x7A
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_1_MIN = 0x7B
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_2_MIN = 0x7C
    NVAPI_PERF_LIMIT_ID_MODS_RULES_INTERSECT = 0x7D
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_DRAM_MAX = 0x7E
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_GPC_MAX = 0x7F
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_DISP_MAX = 0x80
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_DRAM_MIN = 0x81
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_GPC_MIN = 0x82
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_DISP_MIN = 0x83
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_0_MAX = 0x84
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_1_MAX = 0x85
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_2_MAX = 0x86
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_0_MIN = 0x87
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_1_MIN = 0x88
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_2_MIN = 0x89
    NVAPI_PERF_LIMIT_ID_VMIN_LOGIC = 0x8A
    NVAPI_PERF_LIMIT_ID_VMIN_SRAM = 0x8B
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_PSTATE_MAX = 0x8C
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_PSTATE_MIN = 0x8D
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_PSTATE_MAX = 0x8E
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_PSTATE_MIN = 0x8F
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_PSTATE_MAX = 0x90
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_PSTATE_MIN = 0x91
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_PCIE_MAX = 0x92
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_STRICT_PCIE_MIN = 0x93
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_PCIE_MAX = 0x94
    NVAPI_PERF_LIMIT_ID_CLIENT_STRICT_PCIE_MIN = 0x95
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_PCIE_MAX = 0x96
    NVAPI_PERF_LIMIT_ID_MODS_RULES_STRICT_PCIE_MIN = 0x97
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_3_MAX = 0x98
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_3_MIN = 0x99
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_4_MAX = 0x9A
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_4_MIN = 0x9B
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_5_MAX = 0x9C
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_5_MIN = 0x9D
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_6_MAX = 0x9E
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_6_MIN = 0x9F
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_7_MAX = 0xA0
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_7_MIN = 0xA1
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_8_MAX = 0xA2
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_8_MIN = 0xA3
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_9_MAX = 0xA4
    NVAPI_PERF_LIMIT_ID_CLIENT_LOOSE_9_MIN = 0xA5
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_3_MAX = 0xA6
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_4_MAX = 0xA7
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_5_MAX = 0xA8
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_6_MAX = 0xA9
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_7_MAX = 0xAA
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_8_MAX = 0xAB
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_9_MAX = 0xAC
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_3_MIN = 0xAD
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_4_MIN = 0xAE
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_5_MIN = 0xAF
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_6_MIN = 0xB0
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_7_MIN = 0xB1
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_8_MIN = 0xB2
    NVAPI_PERF_LIMIT_ID_CLIENT_LOW_LOOSE_9_MIN = 0xB3
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_3_MAX = 0xB4
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_4_MAX = 0xB5
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_5_MAX = 0xB6
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_6_MAX = 0xB7
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_7_MAX = 0xB8
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_8_MAX = 0xB9
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_9_MAX = 0xBA
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_3_MIN = 0xBB
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_4_MIN = 0xBC
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_5_MIN = 0xBD
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_6_MIN = 0xBE
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_7_MIN = 0xBF
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_8_MIN = 0xC0
    NVAPI_PERF_LIMIT_ID_MODS_RULES_LOOSE_9_MIN = 0xC1
    NVAPI_PERF_LIMIT_ID_PERF_CF_CONTROLLER_DRAM_MIN = 0xC2
    NVAPI_PERF_LIMIT_ID_PERF_CF_CONTROLLER_GPC_MIN = 0xC3
    NVAPI_PERF_LIMIT_ID_PERF_CF_CONTROLLER_NVD_MIN = 0xC4
    NVAPI_PERF_LIMIT_ID_BIF_USBC_PMU_PCIE_MIN = 0xC5
    NVAPI_PERF_LIMIT_ID_BIF_USBC_RM_PCIE_MIN = 0xC6
    NVAPI_PERF_LIMIT_ID_PERF_CF_CONTROLLER_DRAM_MAX = 0xC7
    NVAPI_PERF_LIMIT_ID_PERF_CF_CONTROLLER_GPC_MAX = 0xC8
    NVAPI_PERF_LIMIT_ID_PERF_CF_CONTROLLER_NVD_MAX = 0xC9
    NVAPI_PERF_LIMIT_ID_LAST = 0xCA
    NVAPI_PERF_LIMIT_ID_MAX_LIMITS = 0x100
    NVAPI_PERF_LIMIT_ID_UNKNOWN = 0xFFFFFFFF

class NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID(IntEnum):
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_NONE = 0x0
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_WAIT_FOR_MODESET = 0x1
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_KMD_NOTIFY = 0x2
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_PRE_HW = 0x3
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_NOISE_UNAWARE_CLKS = 0x4
    NV_GPU_PERF_CHANGE_SEQ_30_STEP_ID_NOISE_UNAWARE_VMIN = 0x5
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_VOLT_INCREASE = 0x6
    NV_GPU_PERF_CHANGE_SEQ_30_STEP_ID_NOISE_AWARE_CLKS_INCREASE = 0x7
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_SET_LINK_SPEED = 0x8
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_SET_LINK_WIDTH = 0x9
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_SET_ASPM = 0xA
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_SET_DEEP_L1 = 0xB
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_SET_RAM_SVOP = 0xC
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_SET_NVVDD_PSI = 0xD
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_VOLT_DECREASE = 0xE
    NV_GPU_PERF_CHANGE_SEQ_30_STEP_ID_NOISE_AWARE_CLKS_DECREASE = 0xF
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_SET_LTR_VALUE = 0x10
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_LPWR_FEATURES = 0x11
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_POST_HW = 0x12
    NV_GPU_PERF_CHANGE_SEQ_2X_STEP_ID_MAX_STEPS = 0x13

class NV_GPU_PERF_VPSTATES_IDX(IntEnum):
    NV_GPU_PERF_VPSTATES_IDX_D2 = 0x0
    NV_GPU_PERF_VPSTATES_IDX_D3 = 0x1
    NV_GPU_PERF_VPSTATES_IDX_D4 = 0x2
    NV_GPU_PERF_VPSTATES_IDX_D5 = 0x3
    NV_GPU_PERF_VPSTATES_IDX_OVERCUR = 0x4
    NV_GPU_PERF_VPSTATES_IDX_VRHOT = 0x5
    NV_GPU_PERF_VPSTATES_IDX_MAXBATT = 0x6
    NV_GPU_PERF_VPSTATES_IDX_MAXSLI = 0x7
    NV_GPU_PERF_VPSTATES_IDX_MAXTHERMSUS = 0x8
    NV_GPU_PERF_VPSTATES_IDX_SLOWDOWNPWR = 0x9
    NV_GPU_PERF_VPSTATES_IDX_MIDPOINT = 0xA
    NV_GPU_PERF_VPSTATES_IDX_FLAGS = 0xB
    NV_GPU_PERF_VPSTATES_IDX_INFLECTION0 = 0xC
    NV_GPU_PERF_VPSTATES_IDX_FULLDEFLECT = 0xD
    NV_GPU_PERF_VPSTATES_IDX_IMPFIRST = 0xE
    NV_GPU_PERF_VPSTATES_IDX_IMPLAST = 0xF
    NV_GPU_PERF_VPSTATES_IDX_RATEDTDP = 0x10
    NV_GPU_PERF_VPSTATES_IDX_BOOST = 0x11
    NV_GPU_PERF_VPSTATES_IDX_TURBOBOOST = 0x12
    NV_GPU_PERF_VPSTATES_IDX_MAXCUSTOMERBOOST = 0x13
    NV_GPU_PERF_VPSTATES_IDX_INFLECTION1 = 0x14
    NV_GPU_PERF_VPSTATES_IDX_INFLECTION2 = 0x15
    NV_GPU_PERF_VPSTATES_IDX_WHISPER_MODE = 0x16
    NV_GPU_PERF_VPSTATES_IDX_NUM_INDEXES = 0x17
    NV_GPU_PERF_VPSTATES_IDX_INVALID = 0xFF

class NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID(IntEnum):
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_NONE = 0x0
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_PRE_CHANGE_RM = 0x1
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_PRE_CHANGE_PMU = 0x2
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_POST_CHANGE_RM = 0x3
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_POST_CHANGE_PMU = 0x4
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_PRE_PSTATE_RM = 0x5
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_PRE_PSTATE_PMU = 0x6
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_POST_PSTATE_RM = 0x7
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_POST_PSTATE_PMU = 0x8
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_VOLT = 0x9
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_LPWR = 0xA
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_BIF = 0xB
    NV_GPU_PERF_CHANGE_SEQ_31_STEP_ID_NOISE_UNAWARE_CLKS = 0xC
    NV_GPU_PERF_CHANGE_SEQ_31_STEP_ID_NOISE_AWARE_CLKS = 0xD
    NV_GPU_PERF_CHANGE_SEQ_35_STEP_ID_PRE_VOLT_CLKS = 0xE
    NV_GPU_PERF_CHANGE_SEQ_35_STEP_ID_POST_VOLT_CLKS = 0xF
    NV_GPU_PERF_CHANGE_SEQ_PMU_STEP_ID_MAX_STEPS = 0x20

class _NVAPI_VIDEO_CONTROL_COMPONENT_ALGORITHM(IntEnum):
    NV_VCTRL_ColorSpaceBase = 0x1000
    NV_VCTRL_ColorSpace601 = 0x1001
    NV_VCTRL_ColorSpace709 = 0x1002
    NV_VCTRL_ColorSpaceLast = 0x1003
    NV_VCTRL_ColorRangeBase = 0x1100
    NV_VCTRL_ColorRangeStd = 0x1101
    NV_VCTRL_ColorRangeExt = 0x1102
    NV_VCTRL_ColorRangeLast = 0x1103
    NV_VCTRL_DIBase = 0x2000
    NV_VCTRL_DIBob = 0x2001
    NV_VCTRL_DIMedian = 0x2002
    NV_VCTRL_DI3x3 = 0x2003
    NV_VCTRL_DIAdaptiveFourField = 0x2004
    NV_VCTRL_DIDirectionalSpatial = 0x2005
    NV_VCTRL_DIOnePass = 0x2006
    NV_VCTRL_DISony = 0x2007
    NV_VCTRL_DILast = 0x2008
    NV_VCTRL_CadenceBase = 0x3000
    NV_VCTRL_CadenceBasic = 0x3001
    NV_VCTRL_CadenceVideoOnFilm = 0x3002
    NV_VCTRL_CadenceAdv = 0x3003
    NV_VCTRL_CadenceLast = 0x3004
    NV_VCTRL_DCCEBase = 0x4000
    NV_VCTRL_DCCELow = 0x4001
    NV_VCTRL_DCCEMedium = 0x4002
    NV_VCTRL_DCCEHigh = 0x4003
    NV_VCTRL_DCCELast = 0x4004
    NV_VCTRL_ScalingBase = 0x5000
    NV_VCTRL_ScalingSimple = 0x5001
    NV_VCTRL_Scaling4x4Filter = 0x5002
    NV_VCTRL_ScalingSony = 0x5003
    NV_VCTRL_ScalingLast = 0x5004
    NV_VCTRL_GamutRemapBase = 0x6000
    NV_VCTRL_GamutRemapSony = 0x6001
    NV_VCTRL_GamutRemapLast = 0x6002
    NV_VCTRL_SkinToneCorrectionBase = 0x7000
    NV_VCTRL_SkinToneCorrectionSqueeze = 0x7001
    NV_VCTRL_SkinToneCorrectionRed = 0x7002
    NV_VCTRL_SkinToneCorrectionYellow = 0x7003
    NV_VCTRL_SkinToneCorrectionLast = 0x7004

class _NVAPI_PSI_OPCODE(IntEnum):
    NVAPI_PSI_OPCODE_GET_PSISTAT = 0x0
    NVAPI_PSI_SET_CURRENT_MA = 0x1
    NVAPI_PSI_GET_SUPPORT = 0x2
    NVAPI_PSI_GET_SLEEP_MA = 0x3
    NVAPI_PSI_GET_OPTIMAL_MA = 0x4
    NVAPI_PSI_CTRL_RESET_ENGAGE_COUNT = 0x5
    NVAPI_PSI_RESET_ENGAGE_COUNT = 0x6
    NVAPI_PSI_SET_CURRENT_MA_SRAM = 0x7
    NVAPI_PSI_GET_OPTIMAL_MA_SRAM = 0x8
    NVAPI_PSI_GET_SLEEP_MA_SRAM = 0x9
    NVAPI_PSI_GET_PSTATE_SUPPORT = 0xA
    NVAPI_PSI_SET_ENABLE = 0xB

class NV_GPU_PWR_POLICY_EXT_POWER_STATE_ID(IntEnum):
    NV_GPU_PWR_POLICY_EXT_POWER_STATE_ID_D2 = 0x0
    NV_GPU_PWR_POLICY_EXT_POWER_STATE_ID_D3 = 0x1
    NV_GPU_PWR_POLICY_EXT_POWER_STATE_ID_D4 = 0x2
    NV_GPU_PWR_POLICY_EXT_POWER_STATE_ID_D5 = 0x3
    NV_GPU_PWR_POLICY_EXT_POWER_STATE_ID_COUNT = 0x4
    NV_GPU_PWR_POLICY_EXT_POWER_STATE_ID_D1 = 0xFFFFFFFF

class _RTC_ErrorNumber(IntEnum):
    _RTC_CHKSTK = 0x0
    _RTC_CVRT_LOSS_INFO = 0x1
    _RTC_CORRUPT_STACK = 0x2
    _RTC_UNINIT_LOCAL_USE = 0x3
    _RTC_CORRUPTED_ALLOCA = 0x4
    _RTC_ILLEGAL = 0x5

class cTkPolygoniserCuberille(ctypes.Structure):
    class eType(IntEnum):
        EPolygonise_Outside = 0x0
        EPolygonise_OnSurface = 0x1

    def AddQuad(): pass

    def Polygonise(): pass

    def _IsFull(): pass


class BuildTreeError(IntEnum):
    Ok = 0x0
    JumpTooFar = 0x1
    NotEnoughMemory = 0x2

class D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE(IntEnum):
    D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE_8X8 = 0x0
    D3D12_VIDEO_MOTION_ESTIMATOR_SEARCH_BLOCK_SIZE_16X16 = 0x1

class D3D12_DESCRIPTOR_RANGE_TYPE(IntEnum):
    D3D12_DESCRIPTOR_RANGE_TYPE_SRV = 0x0
    D3D12_DESCRIPTOR_RANGE_TYPE_UAV = 0x1
    D3D12_DESCRIPTOR_RANGE_TYPE_CBV = 0x2
    D3D12_DESCRIPTOR_RANGE_TYPE_SAMPLER = 0x3

class D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION(IntEnum):
    D3D12_VIDEO_MOTION_ESTIMATOR_VECTOR_PRECISION_QUARTER_PEL = 0x0

class D3D12_ROOT_PARAMETER_TYPE(IntEnum):
    D3D12_ROOT_PARAMETER_TYPE_DESCRIPTOR_TABLE = 0x0
    D3D12_ROOT_PARAMETER_TYPE_32BIT_CONSTANTS = 0x1
    D3D12_ROOT_PARAMETER_TYPE_CBV = 0x2
    D3D12_ROOT_PARAMETER_TYPE_SRV = 0x3
    D3D12_ROOT_PARAMETER_TYPE_UAV = 0x4

class D3D12_STATIC_BORDER_COLOR(IntEnum):
    D3D12_STATIC_BORDER_COLOR_TRANSPARENT_BLACK = 0x0
    D3D12_STATIC_BORDER_COLOR_OPAQUE_BLACK = 0x1
    D3D12_STATIC_BORDER_COLOR_OPAQUE_WHITE = 0x2

class D2D1_SPRITE_OPTIONS(IntEnum):
    D2D1_SPRITE_OPTIONS_NONE = 0x0
    D2D1_SPRITE_OPTIONS_CLAMP_TO_SOURCE_RECTANGLE = 0x1
    D2D1_SPRITE_OPTIONS_FORCE_DWORD = 0xFFFFFFFF

class D2D1_COLOR_BITMAP_GLYPH_SNAP_OPTION(IntEnum):
    D2D1_COLOR_BITMAP_GLYPH_SNAP_OPTION_DEFAULT = 0x0
    D2D1_COLOR_BITMAP_GLYPH_SNAP_OPTION_DISABLE = 0x1
    D2D1_COLOR_BITMAP_GLYPH_SNAP_OPTION_FORCE_DWORD = 0xFFFFFFFF

class D3D11_CONTEXT_TYPE(IntEnum):
    D3D11_CONTEXT_TYPE_ALL = 0x0
    D3D11_CONTEXT_TYPE_3D = 0x1
    D3D11_CONTEXT_TYPE_COMPUTE = 0x2
    D3D11_CONTEXT_TYPE_COPY = 0x3
    D3D11_CONTEXT_TYPE_VIDEO = 0x4

class D3D11_TEXTURE_LAYOUT(IntEnum):
    D3D11_TEXTURE_LAYOUT_UNDEFINED = 0x0
    D3D11_TEXTURE_LAYOUT_ROW_MAJOR = 0x1
    D3D11_TEXTURE_LAYOUT_64K_STANDARD_SWIZZLE = 0x2

class D3D12_INDIRECT_ARGUMENT_TYPE(IntEnum):
    D3D12_INDIRECT_ARGUMENT_TYPE_DRAW = 0x0
    D3D12_INDIRECT_ARGUMENT_TYPE_DRAW_INDEXED = 0x1
    D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH = 0x2
    D3D12_INDIRECT_ARGUMENT_TYPE_VERTEX_BUFFER_VIEW = 0x3
    D3D12_INDIRECT_ARGUMENT_TYPE_INDEX_BUFFER_VIEW = 0x4
    D3D12_INDIRECT_ARGUMENT_TYPE_CONSTANT = 0x5
    D3D12_INDIRECT_ARGUMENT_TYPE_CONSTANT_BUFFER_VIEW = 0x6
    D3D12_INDIRECT_ARGUMENT_TYPE_SHADER_RESOURCE_VIEW = 0x7
    D3D12_INDIRECT_ARGUMENT_TYPE_UNORDERED_ACCESS_VIEW = 0x8
    D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_RAYS = 0x9
    D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_MESH = 0xA

class D3D11_CONSERVATIVE_RASTERIZATION_MODE(IntEnum):
    D3D11_CONSERVATIVE_RASTERIZATION_MODE_OFF = 0x0
    D3D11_CONSERVATIVE_RASTERIZATION_MODE_ON = 0x1

class D3D11_FILL_MODE(IntEnum):
    D3D11_FILL_WIREFRAME = 0x2
    D3D11_FILL_SOLID = 0x3

class D3D11_CULL_MODE(IntEnum):
    D3D11_CULL_NONE = 0x1
    D3D11_CULL_FRONT = 0x2
    D3D11_CULL_BACK = 0x3

class D3D11_DSV_DIMENSION(IntEnum):
    D3D11_DSV_DIMENSION_UNKNOWN = 0x0
    D3D11_DSV_DIMENSION_TEXTURE1D = 0x1
    D3D11_DSV_DIMENSION_TEXTURE1DARRAY = 0x2
    D3D11_DSV_DIMENSION_TEXTURE2D = 0x3
    D3D11_DSV_DIMENSION_TEXTURE2DARRAY = 0x4
    D3D11_DSV_DIMENSION_TEXTURE2DMS = 0x5
    D3D11_DSV_DIMENSION_TEXTURE2DMSARRAY = 0x6

class D3D11_RTV_DIMENSION(IntEnum):
    D3D11_RTV_DIMENSION_UNKNOWN = 0x0
    D3D11_RTV_DIMENSION_BUFFER = 0x1
    D3D11_RTV_DIMENSION_TEXTURE1D = 0x2
    D3D11_RTV_DIMENSION_TEXTURE1DARRAY = 0x3
    D3D11_RTV_DIMENSION_TEXTURE2D = 0x4
    D3D11_RTV_DIMENSION_TEXTURE2DARRAY = 0x5
    D3D11_RTV_DIMENSION_TEXTURE2DMS = 0x6
    D3D11_RTV_DIMENSION_TEXTURE2DMSARRAY = 0x7
    D3D11_RTV_DIMENSION_TEXTURE3D = 0x8

class D3D11_UAV_DIMENSION(IntEnum):
    D3D11_UAV_DIMENSION_UNKNOWN = 0x0
    D3D11_UAV_DIMENSION_BUFFER = 0x1
    D3D11_UAV_DIMENSION_TEXTURE1D = 0x2
    D3D11_UAV_DIMENSION_TEXTURE1DARRAY = 0x3
    D3D11_UAV_DIMENSION_TEXTURE2D = 0x4
    D3D11_UAV_DIMENSION_TEXTURE2DARRAY = 0x5
    D3D11_UAV_DIMENSION_TEXTURE3D = 0x8

class D3D11_USAGE(IntEnum):
    D3D11_USAGE_DEFAULT = 0x0
    D3D11_USAGE_IMMUTABLE = 0x1
    D3D11_USAGE_DYNAMIC = 0x2
    D3D11_USAGE_STAGING = 0x3

class D3D11_BIND_FLAG(IntEnum):
    D3D11_BIND_VERTEX_BUFFER = 0x1

class DWRITE_FONT_FILE_TYPE(IntEnum):
    DWRITE_FONT_FILE_TYPE_UNKNOWN = 0x0
    DWRITE_FONT_FILE_TYPE_CFF = 0x1
    DWRITE_FONT_FILE_TYPE_TRUETYPE = 0x2
    DWRITE_FONT_FILE_TYPE_OPENTYPE_COLLECTION = 0x3
    DWRITE_FONT_FILE_TYPE_TYPE1_PFM = 0x4
    DWRITE_FONT_FILE_TYPE_TYPE1_PFB = 0x5
    DWRITE_FONT_FILE_TYPE_VECTOR = 0x6
    DWRITE_FONT_FILE_TYPE_BITMAP = 0x7

class DWRITE_FONT_FACE_TYPE(IntEnum):
    DWRITE_FONT_FACE_TYPE_CFF = 0x0
    DWRITE_FONT_FACE_TYPE_TRUETYPE = 0x1
    DWRITE_FONT_FACE_TYPE_OPENTYPE_COLLECTION = 0x2
    DWRITE_FONT_FACE_TYPE_TYPE1 = 0x3
    DWRITE_FONT_FACE_TYPE_VECTOR = 0x4
    DWRITE_FONT_FACE_TYPE_BITMAP = 0x5
    DWRITE_FONT_FACE_TYPE_UNKNOWN = 0x6
    DWRITE_FONT_FACE_TYPE_RAW_CFF = 0x7

class DWRITE_INFORMATIONAL_STRING_ID(IntEnum):
    DWRITE_INFORMATIONAL_STRING_NONE = 0x0
    DWRITE_INFORMATIONAL_STRING_COPYRIGHT_NOTICE = 0x1
    DWRITE_INFORMATIONAL_STRING_VERSION_STRINGS = 0x2
    DWRITE_INFORMATIONAL_STRING_TRADEMARK = 0x3
    DWRITE_INFORMATIONAL_STRING_MANUFACTURER = 0x4
    DWRITE_INFORMATIONAL_STRING_DESIGNER = 0x5
    DWRITE_INFORMATIONAL_STRING_DESIGNER_URL = 0x6
    DWRITE_INFORMATIONAL_STRING_DESCRIPTION = 0x7
    DWRITE_INFORMATIONAL_STRING_FONT_VENDOR_URL = 0x8
    DWRITE_INFORMATIONAL_STRING_LICENSE_DESCRIPTION = 0x9
    DWRITE_INFORMATIONAL_STRING_LICENSE_INFO_URL = 0xA
    DWRITE_INFORMATIONAL_STRING_WIN32_FAMILY_NAMES = 0xB
    DWRITE_INFORMATIONAL_STRING_WIN32_SUBFAMILY_NAMES = 0xC
    DWRITE_INFORMATIONAL_STRING_TYPOGRAPHIC_FAMILY_NAMES = 0xD
    DWRITE_INFORMATIONAL_STRING_TYPOGRAPHIC_SUBFAMILY_NAMES = 0xE
    DWRITE_INFORMATIONAL_STRING_SAMPLE_TEXT = 0xF
    DWRITE_INFORMATIONAL_STRING_FULL_NAME = 0x10
    DWRITE_INFORMATIONAL_STRING_POSTSCRIPT_NAME = 0x11
    DWRITE_INFORMATIONAL_STRING_POSTSCRIPT_CID_NAME = 0x12
    DWRITE_INFORMATIONAL_STRING_WEIGHT_STRETCH_STYLE_FAMILY_NAME = 0x13
    DWRITE_INFORMATIONAL_STRING_DESIGN_SCRIPT_LANGUAGE_TAG = 0x14
    DWRITE_INFORMATIONAL_STRING_SUPPORTED_SCRIPT_LANGUAGE_TAG = 0x15

class D3D11_COMPARISON_FUNC(IntEnum):
    D3D11_COMPARISON_NEVER = 0x1
    D3D11_COMPARISON_LESS = 0x2
    D3D11_COMPARISON_EQUAL = 0x3
    D3D11_COMPARISON_LESS_EQUAL = 0x4
    D3D11_COMPARISON_GREATER = 0x5
    D3D11_COMPARISON_NOT_EQUAL = 0x6
    D3D11_COMPARISON_GREATER_EQUAL = 0x7
    D3D11_COMPARISON_ALWAYS = 0x8

class D3D11_DEPTH_WRITE_MASK(IntEnum):
    D3D11_DEPTH_WRITE_MASK_ZERO = 0x0
    D3D11_DEPTH_WRITE_MASK_ALL = 0x1

class D3D11_STENCIL_OP(IntEnum):
    D3D11_STENCIL_OP_KEEP = 0x1
    D3D11_STENCIL_OP_ZERO = 0x2
    D3D11_STENCIL_OP_REPLACE = 0x3
    D3D11_STENCIL_OP_INCR_SAT = 0x4
    D3D11_STENCIL_OP_DECR_SAT = 0x5
    D3D11_STENCIL_OP_INVERT = 0x6
    D3D11_STENCIL_OP_INCR = 0x7
    D3D11_STENCIL_OP_DECR = 0x8

class DWRITE_MEASURING_MODE(IntEnum):
    DWRITE_MEASURING_MODE_NATURAL = 0x0
    DWRITE_MEASURING_MODE_GDI_CLASSIC = 0x1
    DWRITE_MEASURING_MODE_GDI_NATURAL = 0x2

class D3D11_BLEND(IntEnum):
    D3D11_BLEND_ZERO = 0x1
    D3D11_BLEND_ONE = 0x2
    D3D11_BLEND_SRC_COLOR = 0x3
    D3D11_BLEND_INV_SRC_COLOR = 0x4
    D3D11_BLEND_SRC_ALPHA = 0x5
    D3D11_BLEND_INV_SRC_ALPHA = 0x6
    D3D11_BLEND_DEST_ALPHA = 0x7
    D3D11_BLEND_INV_DEST_ALPHA = 0x8
    D3D11_BLEND_DEST_COLOR = 0x9
    D3D11_BLEND_INV_DEST_COLOR = 0xA
    D3D11_BLEND_SRC_ALPHA_SAT = 0xB
    D3D11_BLEND_BLEND_FACTOR = 0xE
    D3D11_BLEND_INV_BLEND_FACTOR = 0xF
    D3D11_BLEND_SRC1_COLOR = 0x10
    D3D11_BLEND_INV_SRC1_COLOR = 0x11
    D3D11_BLEND_SRC1_ALPHA = 0x12
    D3D11_BLEND_INV_SRC1_ALPHA = 0x13

class D2D1_ALPHA_MODE(IntEnum):
    D2D1_ALPHA_MODE_UNKNOWN = 0x0
    D2D1_ALPHA_MODE_PREMULTIPLIED = 0x1
    D2D1_ALPHA_MODE_STRAIGHT = 0x2
    D2D1_ALPHA_MODE_IGNORE = 0x3
    D2D1_ALPHA_MODE_FORCE_DWORD = 0xFFFFFFFF

class D3D11_BLEND_OP(IntEnum):
    D3D11_BLEND_OP_ADD = 0x1
    D3D11_BLEND_OP_SUBTRACT = 0x2
    D3D11_BLEND_OP_REV_SUBTRACT = 0x3
    D3D11_BLEND_OP_MIN = 0x4
    D3D11_BLEND_OP_MAX = 0x5

class D3D11_COLOR_WRITE_ENABLE(IntEnum):
    D3D11_COLOR_WRITE_ENABLE_RED = 0x1
    D3D11_COLOR_WRITE_ENABLE_GREEN = 0x2
    D3D11_COLOR_WRITE_ENABLE_BLUE = 0x4
    D3D11_COLOR_WRITE_ENABLE_ALPHA = 0x8
    D3D11_COLOR_WRITE_ENABLE_ALL = 0xF

class DWRITE_RENDERING_MODE(IntEnum):
    DWRITE_RENDERING_MODE_DEFAULT = 0x0
    DWRITE_RENDERING_MODE_ALIASED = 0x1
    DWRITE_RENDERING_MODE_GDI_CLASSIC = 0x2
    DWRITE_RENDERING_MODE_GDI_NATURAL = 0x3
    DWRITE_RENDERING_MODE_NATURAL = 0x4
    DWRITE_RENDERING_MODE_NATURAL_SYMMETRIC = 0x5
    DWRITE_RENDERING_MODE_OUTLINE = 0x6

class D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT(IntEnum):
    D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT_Y = 0x0
    D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT_U = 0x1
    D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT_V = 0x2
    D3D11_VIDEO_DECODER_HISTOGRAM_COMPONENT_A = 0x3

class D2D1_GAMMA(IntEnum):
    D2D1_GAMMA_2_2 = 0x0
    D2D1_GAMMA_1_0 = 0x1
    D2D1_GAMMA_FORCE_DWORD = 0xFFFFFFFF

class D2D1_EXTEND_MODE(IntEnum):
    D2D1_EXTEND_MODE_CLAMP = 0x0
    D2D1_EXTEND_MODE_WRAP = 0x1
    D2D1_EXTEND_MODE_MIRROR = 0x2
    D2D1_EXTEND_MODE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_ANTIALIAS_MODE(IntEnum):
    D2D1_ANTIALIAS_MODE_PER_PRIMITIVE = 0x0
    D2D1_ANTIALIAS_MODE_ALIASED = 0x1
    D2D1_ANTIALIAS_MODE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_TEXT_ANTIALIAS_MODE(IntEnum):
    D2D1_TEXT_ANTIALIAS_MODE_DEFAULT = 0x0
    D2D1_TEXT_ANTIALIAS_MODE_CLEARTYPE = 0x1
    D2D1_TEXT_ANTIALIAS_MODE_GRAYSCALE = 0x2
    D2D1_TEXT_ANTIALIAS_MODE_ALIASED = 0x3
    D2D1_TEXT_ANTIALIAS_MODE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_BITMAP_INTERPOLATION_MODE(IntEnum):
    D2D1_BITMAP_INTERPOLATION_MODE_NEAREST_NEIGHBOR = 0x0
    D2D1_BITMAP_INTERPOLATION_MODE_LINEAR = 0x1
    D2D1_BITMAP_INTERPOLATION_MODE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_DRAW_TEXT_OPTIONS(IntEnum):
    D2D1_DRAW_TEXT_OPTIONS_NONE = 0x0
    D2D1_DRAW_TEXT_OPTIONS_NO_SNAP = 0x1
    D2D1_DRAW_TEXT_OPTIONS_CLIP = 0x2
    D2D1_DRAW_TEXT_OPTIONS_ENABLE_COLOR_FONT = 0x4
    D2D1_DRAW_TEXT_OPTIONS_DISABLE_COLOR_BITMAP_SNAPPING = 0x8
    D2D1_DRAW_TEXT_OPTIONS_FORCE_DWORD = 0xFFFFFFFF

class D2D1_CAP_STYLE(IntEnum):
    D2D1_CAP_STYLE_FLAT = 0x0
    D2D1_CAP_STYLE_SQUARE = 0x1
    D2D1_CAP_STYLE_ROUND = 0x2
    D2D1_CAP_STYLE_TRIANGLE = 0x3
    D2D1_CAP_STYLE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_DASH_STYLE(IntEnum):
    D2D1_DASH_STYLE_SOLID = 0x0
    D2D1_DASH_STYLE_DASH = 0x1
    D2D1_DASH_STYLE_DOT = 0x2
    D2D1_DASH_STYLE_DASH_DOT = 0x3
    D2D1_DASH_STYLE_DASH_DOT_DOT = 0x4
    D2D1_DASH_STYLE_CUSTOM = 0x5
    D2D1_DASH_STYLE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_LINE_JOIN(IntEnum):
    D2D1_LINE_JOIN_MITER = 0x0
    D2D1_LINE_JOIN_BEVEL = 0x1
    D2D1_LINE_JOIN_ROUND = 0x2
    D2D1_LINE_JOIN_MITER_OR_BEVEL = 0x3
    D2D1_LINE_JOIN_FORCE_DWORD = 0xFFFFFFFF

class D3D12_LIFETIME_STATE(IntEnum):
    D3D12_LIFETIME_STATE_IN_USE = 0x0
    D3D12_LIFETIME_STATE_NOT_IN_USE = 0x1

class D2D1_LAYER_OPTIONS(IntEnum):
    D2D1_LAYER_OPTIONS_NONE = 0x0
    D2D1_LAYER_OPTIONS_INITIALIZE_FOR_CLEARTYPE = 0x1
    D2D1_LAYER_OPTIONS_FORCE_DWORD = 0xFFFFFFFF

class D2D1_RENDER_TARGET_TYPE(IntEnum):
    D2D1_RENDER_TARGET_TYPE_DEFAULT = 0x0
    D2D1_RENDER_TARGET_TYPE_SOFTWARE = 0x1
    D2D1_RENDER_TARGET_TYPE_HARDWARE = 0x2
    D2D1_RENDER_TARGET_TYPE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_FEATURE_LEVEL(IntEnum):
    D2D1_FEATURE_LEVEL_DEFAULT = 0x0
    D2D1_FEATURE_LEVEL_9 = 0x9100
    D2D1_FEATURE_LEVEL_10 = 0xA000
    D2D1_FEATURE_LEVEL_FORCE_DWORD = 0xFFFFFFFF

class D2D1_RENDER_TARGET_USAGE(IntEnum):
    D2D1_RENDER_TARGET_USAGE_NONE = 0x0
    D2D1_RENDER_TARGET_USAGE_FORCE_BITMAP_REMOTING = 0x1
    D2D1_RENDER_TARGET_USAGE_GDI_COMPATIBLE = 0x2
    D2D1_RENDER_TARGET_USAGE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_PRESENT_OPTIONS(IntEnum):
    D2D1_PRESENT_OPTIONS_NONE = 0x0
    D2D1_PRESENT_OPTIONS_RETAIN_CONTENTS = 0x1
    D2D1_PRESENT_OPTIONS_IMMEDIATELY = 0x2
    D2D1_PRESENT_OPTIONS_FORCE_DWORD = 0xFFFFFFFF

class D3D12_STATE_SUBOBJECT_TYPE(IntEnum):
    D3D12_STATE_SUBOBJECT_TYPE_STATE_OBJECT_CONFIG = 0x0
    D3D12_STATE_SUBOBJECT_TYPE_GLOBAL_ROOT_SIGNATURE = 0x1
    D3D12_STATE_SUBOBJECT_TYPE_LOCAL_ROOT_SIGNATURE = 0x2
    D3D12_STATE_SUBOBJECT_TYPE_NODE_MASK = 0x3
    D3D12_STATE_SUBOBJECT_TYPE_DXIL_LIBRARY = 0x5
    D3D12_STATE_SUBOBJECT_TYPE_EXISTING_COLLECTION = 0x6
    D3D12_STATE_SUBOBJECT_TYPE_SUBOBJECT_TO_EXPORTS_ASSOCIATION = 0x7
    D3D12_STATE_SUBOBJECT_TYPE_DXIL_SUBOBJECT_TO_EXPORTS_ASSOCIATION = 0x8
    D3D12_STATE_SUBOBJECT_TYPE_RAYTRACING_SHADER_CONFIG = 0x9
    D3D12_STATE_SUBOBJECT_TYPE_RAYTRACING_PIPELINE_CONFIG = 0xA
    D3D12_STATE_SUBOBJECT_TYPE_HIT_GROUP = 0xB
    D3D12_STATE_SUBOBJECT_TYPE_RAYTRACING_PIPELINE_CONFIG1 = 0xC
    D3D12_STATE_SUBOBJECT_TYPE_MAX_VALID = 0xD

class D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS(IntEnum):
    D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS_NONE = 0x0
    D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS_GDI_COMPATIBLE = 0x1
    D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS_FORCE_DWORD = 0xFFFFFFFF

class D2D1_FACTORY_TYPE(IntEnum):
    D2D1_FACTORY_TYPE_SINGLE_THREADED = 0x0
    D2D1_FACTORY_TYPE_MULTI_THREADED = 0x1
    D2D1_FACTORY_TYPE_FORCE_DWORD = 0xFFFFFFFF

class D3D12_RAYTRACING_GEOMETRY_TYPE(IntEnum):
    D3D12_RAYTRACING_GEOMETRY_TYPE_TRIANGLES = 0x0
    D3D12_RAYTRACING_GEOMETRY_TYPE_PROCEDURAL_PRIMITIVE_AABBS = 0x1

class D3D11_FILTER(IntEnum):
    D3D11_FILTER_MIN_MAG_MIP_POINT = 0x0
    D3D11_FILTER_MIN_MAG_POINT_MIP_LINEAR = 0x1
    D3D11_FILTER_MIN_POINT_MAG_LINEAR_MIP_POINT = 0x4
    D3D11_FILTER_MIN_POINT_MAG_MIP_LINEAR = 0x5
    D3D11_FILTER_MIN_LINEAR_MAG_MIP_POINT = 0x10
    D3D11_FILTER_MIN_LINEAR_MAG_POINT_MIP_LINEAR = 0x11
    D3D11_FILTER_MIN_MAG_LINEAR_MIP_POINT = 0x14
    D3D11_FILTER_MIN_MAG_MIP_LINEAR = 0x15
    D3D11_FILTER_ANISOTROPIC = 0x55
    D3D11_FILTER_COMPARISON_MIN_MAG_MIP_POINT = 0x80
    D3D11_FILTER_COMPARISON_MIN_MAG_POINT_MIP_LINEAR = 0x81
    D3D11_FILTER_COMPARISON_MIN_POINT_MAG_LINEAR_MIP_POINT = 0x84
    D3D11_FILTER_COMPARISON_MIN_POINT_MAG_MIP_LINEAR = 0x85
    D3D11_FILTER_COMPARISON_MIN_LINEAR_MAG_MIP_POINT = 0x90
    D3D11_FILTER_COMPARISON_MIN_LINEAR_MAG_POINT_MIP_LINEAR = 0x91
    D3D11_FILTER_COMPARISON_MIN_MAG_LINEAR_MIP_POINT = 0x94
    D3D11_FILTER_COMPARISON_MIN_MAG_MIP_LINEAR = 0x95
    D3D11_FILTER_COMPARISON_ANISOTROPIC = 0xD5
    D3D11_FILTER_MINIMUM_MIN_MAG_MIP_POINT = 0x100
    D3D11_FILTER_MINIMUM_MIN_MAG_POINT_MIP_LINEAR = 0x101
    D3D11_FILTER_MINIMUM_MIN_POINT_MAG_LINEAR_MIP_POINT = 0x104
    D3D11_FILTER_MINIMUM_MIN_POINT_MAG_MIP_LINEAR = 0x105
    D3D11_FILTER_MINIMUM_MIN_LINEAR_MAG_MIP_POINT = 0x110
    D3D11_FILTER_MINIMUM_MIN_LINEAR_MAG_POINT_MIP_LINEAR = 0x111
    D3D11_FILTER_MINIMUM_MIN_MAG_LINEAR_MIP_POINT = 0x114
    D3D11_FILTER_MINIMUM_MIN_MAG_MIP_LINEAR = 0x115
    D3D11_FILTER_MINIMUM_ANISOTROPIC = 0x155
    D3D11_FILTER_MAXIMUM_MIN_MAG_MIP_POINT = 0x180
    D3D11_FILTER_MAXIMUM_MIN_MAG_POINT_MIP_LINEAR = 0x181
    D3D11_FILTER_MAXIMUM_MIN_POINT_MAG_LINEAR_MIP_POINT = 0x184
    D3D11_FILTER_MAXIMUM_MIN_POINT_MAG_MIP_LINEAR = 0x185
    D3D11_FILTER_MAXIMUM_MIN_LINEAR_MAG_MIP_POINT = 0x190
    D3D11_FILTER_MAXIMUM_MIN_LINEAR_MAG_POINT_MIP_LINEAR = 0x191
    D3D11_FILTER_MAXIMUM_MIN_MAG_LINEAR_MIP_POINT = 0x194
    D3D11_FILTER_MAXIMUM_MIN_MAG_MIP_LINEAR = 0x195
    D3D11_FILTER_MAXIMUM_ANISOTROPIC = 0x1D5

class D3D11_TEXTURE_ADDRESS_MODE(IntEnum):
    D3D11_TEXTURE_ADDRESS_WRAP = 0x1
    D3D11_TEXTURE_ADDRESS_MIRROR = 0x2
    D3D11_TEXTURE_ADDRESS_CLAMP = 0x3
    D3D11_TEXTURE_ADDRESS_BORDER = 0x4
    D3D11_TEXTURE_ADDRESS_MIRROR_ONCE = 0x5

class D3D11_QUERY(IntEnum):
    D3D11_QUERY_EVENT = 0x0
    D3D11_QUERY_OCCLUSION = 0x1
    D3D11_QUERY_TIMESTAMP = 0x2
    D3D11_QUERY_TIMESTAMP_DISJOINT = 0x3
    D3D11_QUERY_PIPELINE_STATISTICS = 0x4
    D3D11_QUERY_OCCLUSION_PREDICATE = 0x5
    D3D11_QUERY_SO_STATISTICS = 0x6
    D3D11_QUERY_SO_OVERFLOW_PREDICATE = 0x7
    D3D11_QUERY_SO_STATISTICS_STREAM0 = 0x8
    D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM0 = 0x9
    D3D11_QUERY_SO_STATISTICS_STREAM1 = 0xA
    D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM1 = 0xB
    D3D11_QUERY_SO_STATISTICS_STREAM2 = 0xC
    D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM2 = 0xD
    D3D11_QUERY_SO_STATISTICS_STREAM3 = 0xE
    D3D11_QUERY_SO_OVERFLOW_PREDICATE_STREAM3 = 0xF

class D3D11_COUNTER_TYPE(IntEnum):
    D3D11_COUNTER_TYPE_FLOAT32 = 0x0
    D3D11_COUNTER_TYPE_UINT16 = 0x1
    D3D11_COUNTER_TYPE_UINT32 = 0x2
    D3D11_COUNTER_TYPE_UINT64 = 0x3

class D3D11_DEVICE_CONTEXT_TYPE(IntEnum):
    D3D11_DEVICE_CONTEXT_IMMEDIATE = 0x0
    D3D11_DEVICE_CONTEXT_DEFERRED = 0x1

class D3D11_FEATURE(IntEnum):
    D3D11_FEATURE_THREADING = 0x0
    D3D11_FEATURE_DOUBLES = 0x1
    D3D11_FEATURE_FORMAT_SUPPORT = 0x2
    D3D11_FEATURE_FORMAT_SUPPORT2 = 0x3
    D3D11_FEATURE_D3D10_X_HARDWARE_OPTIONS = 0x4
    D3D11_FEATURE_D3D11_OPTIONS = 0x5
    D3D11_FEATURE_ARCHITECTURE_INFO = 0x6
    D3D11_FEATURE_D3D9_OPTIONS = 0x7
    D3D11_FEATURE_SHADER_MIN_PRECISION_SUPPORT = 0x8
    D3D11_FEATURE_D3D9_SHADOW_SUPPORT = 0x9
    D3D11_FEATURE_D3D11_OPTIONS1 = 0xA
    D3D11_FEATURE_D3D9_SIMPLE_INSTANCING_SUPPORT = 0xB
    D3D11_FEATURE_MARKER_SUPPORT = 0xC
    D3D11_FEATURE_D3D9_OPTIONS1 = 0xD
    D3D11_FEATURE_D3D11_OPTIONS2 = 0xE
    D3D11_FEATURE_D3D11_OPTIONS3 = 0xF
    D3D11_FEATURE_GPU_VIRTUAL_ADDRESS_SUPPORT = 0x10
    D3D11_FEATURE_D3D11_OPTIONS4 = 0x11
    D3D11_FEATURE_SHADER_CACHE = 0x12
    D3D11_FEATURE_D3D11_OPTIONS5 = 0x13

class D3D11_SHARED_RESOURCE_TIER(IntEnum):
    D3D11_SHARED_RESOURCE_TIER_0 = 0x0
    D3D11_SHARED_RESOURCE_TIER_1 = 0x1
    D3D11_SHARED_RESOURCE_TIER_2 = 0x2
    D3D11_SHARED_RESOURCE_TIER_3 = 0x3

class D3D12_BACKGROUND_PROCESSING_MODE(IntEnum):
    D3D12_BACKGROUND_PROCESSING_MODE_ALLOWED = 0x0
    D3D12_BACKGROUND_PROCESSING_MODE_ALLOW_INTRUSIVE_MEASUREMENTS = 0x1
    D3D12_BACKGROUND_PROCESSING_MODE_DISABLE_BACKGROUND_WORK = 0x2
    D3D12_BACKGROUND_PROCESSING_MODE_DISABLE_PROFILING_BY_SYSTEM = 0x3

class D3D12_MEASUREMENTS_ACTION(IntEnum):
    D3D12_MEASUREMENTS_ACTION_KEEP_ALL = 0x0
    D3D12_MEASUREMENTS_ACTION_COMMIT_RESULTS = 0x1
    D3D12_MEASUREMENTS_ACTION_COMMIT_RESULTS_HIGH_PRIORITY = 0x2
    D3D12_MEASUREMENTS_ACTION_DISCARD_PREVIOUS = 0x3

class D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE(IntEnum):
    D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE_DISCARD = 0x0
    D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE_PRESERVE = 0x1
    D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE_CLEAR = 0x2
    D3D12_RENDER_PASS_BEGINNING_ACCESS_TYPE_NO_ACCESS = 0x3

class D3D11_TRACE_REGISTER_TYPE(IntEnum):
    D3D11_TRACE_OUTPUT_NULL_REGISTER = 0x0
    D3D11_TRACE_INPUT_REGISTER = 0x1
    D3D11_TRACE_INPUT_PRIMITIVE_ID_REGISTER = 0x2
    D3D11_TRACE_IMMEDIATE_CONSTANT_BUFFER = 0x3
    D3D11_TRACE_TEMP_REGISTER = 0x4
    D3D11_TRACE_INDEXABLE_TEMP_REGISTER = 0x5
    D3D11_TRACE_OUTPUT_REGISTER = 0x6
    D3D11_TRACE_OUTPUT_DEPTH_REGISTER = 0x7
    D3D11_TRACE_CONSTANT_BUFFER = 0x8
    D3D11_TRACE_IMMEDIATE32 = 0x9
    D3D11_TRACE_SAMPLER = 0xA
    D3D11_TRACE_RESOURCE = 0xB
    D3D11_TRACE_RASTERIZER = 0xC
    D3D11_TRACE_OUTPUT_COVERAGE_MASK = 0xD
    D3D11_TRACE_STREAM = 0xE
    D3D11_TRACE_THIS_POINTER = 0xF
    D3D11_TRACE_OUTPUT_CONTROL_POINT_ID_REGISTER = 0x10
    D3D11_TRACE_INPUT_FORK_INSTANCE_ID_REGISTER = 0x11
    D3D11_TRACE_INPUT_JOIN_INSTANCE_ID_REGISTER = 0x12
    D3D11_TRACE_INPUT_CONTROL_POINT_REGISTER = 0x13
    D3D11_TRACE_OUTPUT_CONTROL_POINT_REGISTER = 0x14
    D3D11_TRACE_INPUT_PATCH_CONSTANT_REGISTER = 0x15
    D3D11_TRACE_INPUT_DOMAIN_POINT_REGISTER = 0x16
    D3D11_TRACE_UNORDERED_ACCESS_VIEW = 0x17
    D3D11_TRACE_THREAD_GROUP_SHARED_MEMORY = 0x18
    D3D11_TRACE_INPUT_THREAD_ID_REGISTER = 0x19
    D3D11_TRACE_INPUT_THREAD_GROUP_ID_REGISTER = 0x1A
    D3D11_TRACE_INPUT_THREAD_ID_IN_GROUP_REGISTER = 0x1B
    D3D11_TRACE_INPUT_COVERAGE_MASK_REGISTER = 0x1C
    D3D11_TRACE_INPUT_THREAD_ID_IN_GROUP_FLATTENED_REGISTER = 0x1D
    D3D11_TRACE_INPUT_GS_INSTANCE_ID_REGISTER = 0x1E
    D3D11_TRACE_OUTPUT_DEPTH_GREATER_EQUAL_REGISTER = 0x1F
    D3D11_TRACE_OUTPUT_DEPTH_LESS_EQUAL_REGISTER = 0x20
    D3D11_TRACE_IMMEDIATE64 = 0x21
    D3D11_TRACE_INPUT_CYCLE_COUNTER_REGISTER = 0x22
    D3D11_TRACE_INTERFACE_POINTER = 0x23

class D3D12_RENDER_PASS_ENDING_ACCESS_TYPE(IntEnum):
    D3D12_RENDER_PASS_ENDING_ACCESS_TYPE_DISCARD = 0x0
    D3D12_RENDER_PASS_ENDING_ACCESS_TYPE_PRESERVE = 0x1
    D3D12_RENDER_PASS_ENDING_ACCESS_TYPE_RESOLVE = 0x2
    D3D12_RENDER_PASS_ENDING_ACCESS_TYPE_NO_ACCESS = 0x3

class D3D12_COMMAND_LIST_TYPE(IntEnum):
    D3D12_COMMAND_LIST_TYPE_DIRECT = 0x0
    D3D12_COMMAND_LIST_TYPE_BUNDLE = 0x1
    D3D12_COMMAND_LIST_TYPE_COMPUTE = 0x2
    D3D12_COMMAND_LIST_TYPE_COPY = 0x3
    D3D12_COMMAND_LIST_TYPE_VIDEO_DECODE = 0x4
    D3D12_COMMAND_LIST_TYPE_VIDEO_PROCESS = 0x5
    D3D12_COMMAND_LIST_TYPE_VIDEO_ENCODE = 0x6

class D2D1_BORDER_MODE(IntEnum):
    D2D1_BORDER_MODE_SOFT = 0x0
    D2D1_BORDER_MODE_HARD = 0x1
    D2D1_BORDER_MODE_FORCE_DWORD = 0xFFFFFFFF

class D3D12_COLOR_WRITE_ENABLE(IntEnum):
    D3D12_COLOR_WRITE_ENABLE_RED = 0x1
    D3D12_COLOR_WRITE_ENABLE_GREEN = 0x2
    D3D12_COLOR_WRITE_ENABLE_BLUE = 0x4
    D3D12_COLOR_WRITE_ENABLE_ALPHA = 0x8
    D3D12_COLOR_WRITE_ENABLE_ALL = 0xF

class D3D12_LOGIC_OP(IntEnum):
    D3D12_LOGIC_OP_CLEAR = 0x0
    D3D12_LOGIC_OP_SET = 0x1
    D3D12_LOGIC_OP_COPY = 0x2
    D3D12_LOGIC_OP_COPY_INVERTED = 0x3
    D3D12_LOGIC_OP_NOOP = 0x4
    D3D12_LOGIC_OP_INVERT = 0x5
    D3D12_LOGIC_OP_AND = 0x6
    D3D12_LOGIC_OP_NAND = 0x7
    D3D12_LOGIC_OP_OR = 0x8
    D3D12_LOGIC_OP_NOR = 0x9
    D3D12_LOGIC_OP_XOR = 0xA
    D3D12_LOGIC_OP_EQUIV = 0xB
    D3D12_LOGIC_OP_AND_REVERSE = 0xC
    D3D12_LOGIC_OP_AND_INVERTED = 0xD
    D3D12_LOGIC_OP_OR_REVERSE = 0xE
    D3D12_LOGIC_OP_OR_INVERTED = 0xF

class D3D12_DEBUG_DEVICE_PARAMETER_TYPE(IntEnum):
    D3D12_DEBUG_DEVICE_PARAMETER_FEATURE_FLAGS = 0x0
    D3D12_DEBUG_DEVICE_PARAMETER_GPU_BASED_VALIDATION_SETTINGS = 0x1
    D3D12_DEBUG_DEVICE_PARAMETER_GPU_SLOWDOWN_PERFORMANCE_FACTOR = 0x2

class D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE(IntEnum):
    D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE_NONE = 0x0
    D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE_STATE_TRACKING_ONLY = 0x1
    D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE_UNGUARDED_VALIDATION = 0x2
    D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODE_GUARDED_VALIDATION = 0x3
    NUM_D3D12_GPU_BASED_VALIDATION_SHADER_PATCH_MODES = 0x4

class D2D1_DPICOMPENSATION_PROP(IntEnum):
    D2D1_DPICOMPENSATION_PROP_INTERPOLATION_MODE = 0x0
    D2D1_DPICOMPENSATION_PROP_BORDER_MODE = 0x1
    D2D1_DPICOMPENSATION_PROP_INPUT_DPI = 0x2
    D2D1_DPICOMPENSATION_PROP_FORCE_DWORD = 0xFFFFFFFF

class D3D_FEATURE_LEVEL(IntEnum):
    D3D_FEATURE_LEVEL_1_0_CORE = 0x1000
    D3D_FEATURE_LEVEL_9_1 = 0x9100
    D3D_FEATURE_LEVEL_9_2 = 0x9200
    D3D_FEATURE_LEVEL_9_3 = 0x9300
    D3D_FEATURE_LEVEL_10_0 = 0xA000
    D3D_FEATURE_LEVEL_10_1 = 0xA100
    D3D_FEATURE_LEVEL_11_0 = 0xB000
    D3D_FEATURE_LEVEL_11_1 = 0xB100
    D3D_FEATURE_LEVEL_12_0 = 0xC000
    D3D_FEATURE_LEVEL_12_1 = 0xC100

class D3D12_PIPELINE_STATE_SUBOBJECT_TYPE(IntEnum):
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_ROOT_SIGNATURE = 0x0
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_VS = 0x1
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_PS = 0x2
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_DS = 0x3
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_HS = 0x4
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_GS = 0x5
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_CS = 0x6
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_STREAM_OUTPUT = 0x7
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_BLEND = 0x8
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_SAMPLE_MASK = 0x9
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_RASTERIZER = 0xA
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_DEPTH_STENCIL = 0xB
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_INPUT_LAYOUT = 0xC
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_IB_STRIP_CUT_VALUE = 0xD
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_PRIMITIVE_TOPOLOGY = 0xE
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_RENDER_TARGET_FORMATS = 0xF
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_DEPTH_STENCIL_FORMAT = 0x10
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_SAMPLE_DESC = 0x11
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_NODE_MASK = 0x12
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_CACHED_PSO = 0x13
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_FLAGS = 0x14
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_DEPTH_STENCIL1 = 0x15
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_VIEW_INSTANCING = 0x16
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_AS = 0x18
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_MS = 0x19
    D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_MAX_VALID = 0x1A

class D3D12_MESSAGE_CATEGORY(IntEnum):
    D3D12_MESSAGE_CATEGORY_APPLICATION_DEFINED = 0x0
    D3D12_MESSAGE_CATEGORY_MISCELLANEOUS = 0x1
    D3D12_MESSAGE_CATEGORY_INITIALIZATION = 0x2
    D3D12_MESSAGE_CATEGORY_CLEANUP = 0x3
    D3D12_MESSAGE_CATEGORY_COMPILATION = 0x4
    D3D12_MESSAGE_CATEGORY_STATE_CREATION = 0x5
    D3D12_MESSAGE_CATEGORY_STATE_SETTING = 0x6
    D3D12_MESSAGE_CATEGORY_STATE_GETTING = 0x7
    D3D12_MESSAGE_CATEGORY_RESOURCE_MANIPULATION = 0x8
    D3D12_MESSAGE_CATEGORY_EXECUTION = 0x9
    D3D12_MESSAGE_CATEGORY_SHADER = 0xA

class D3D12_MESSAGE_SEVERITY(IntEnum):
    D3D12_MESSAGE_SEVERITY_CORRUPTION = 0x0
    D3D12_MESSAGE_SEVERITY_ERROR = 0x1
    D3D12_MESSAGE_SEVERITY_WARNING = 0x2
    D3D12_MESSAGE_SEVERITY_INFO = 0x3
    D3D12_MESSAGE_SEVERITY_MESSAGE = 0x4

class D2D1_PROPERTY_TYPE(IntEnum):
    D2D1_PROPERTY_TYPE_UNKNOWN = 0x0
    D2D1_PROPERTY_TYPE_STRING = 0x1
    D2D1_PROPERTY_TYPE_BOOL = 0x2
    D2D1_PROPERTY_TYPE_UINT32 = 0x3
    D2D1_PROPERTY_TYPE_INT32 = 0x4
    D2D1_PROPERTY_TYPE_FLOAT = 0x5
    D2D1_PROPERTY_TYPE_VECTOR2 = 0x6
    D2D1_PROPERTY_TYPE_VECTOR3 = 0x7
    D2D1_PROPERTY_TYPE_VECTOR4 = 0x8
    D2D1_PROPERTY_TYPE_BLOB = 0x9
    D2D1_PROPERTY_TYPE_IUNKNOWN = 0xA
    D2D1_PROPERTY_TYPE_ENUM = 0xB
    D2D1_PROPERTY_TYPE_ARRAY = 0xC
    D2D1_PROPERTY_TYPE_CLSID = 0xD
    D2D1_PROPERTY_TYPE_MATRIX_3X2 = 0xE
    D2D1_PROPERTY_TYPE_MATRIX_4X3 = 0xF
    D2D1_PROPERTY_TYPE_MATRIX_4X4 = 0x10
    D2D1_PROPERTY_TYPE_MATRIX_5X4 = 0x11
    D2D1_PROPERTY_TYPE_COLOR_CONTEXT = 0x12
    D2D1_PROPERTY_TYPE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_BITMAP_OPTIONS(IntEnum):
    D2D1_BITMAP_OPTIONS_NONE = 0x0
    D2D1_BITMAP_OPTIONS_TARGET = 0x1
    D2D1_BITMAP_OPTIONS_CANNOT_DRAW = 0x2
    D2D1_BITMAP_OPTIONS_CPU_READ = 0x4
    D2D1_BITMAP_OPTIONS_GDI_COMPATIBLE = 0x8
    D2D1_BITMAP_OPTIONS_FORCE_DWORD = 0xFFFFFFFF

class D2D1_COMPOSITE_MODE(IntEnum):
    D2D1_COMPOSITE_MODE_SOURCE_OVER = 0x0
    D2D1_COMPOSITE_MODE_DESTINATION_OVER = 0x1
    D2D1_COMPOSITE_MODE_SOURCE_IN = 0x2
    D2D1_COMPOSITE_MODE_DESTINATION_IN = 0x3
    D2D1_COMPOSITE_MODE_SOURCE_OUT = 0x4
    D2D1_COMPOSITE_MODE_DESTINATION_OUT = 0x5
    D2D1_COMPOSITE_MODE_SOURCE_ATOP = 0x6
    D2D1_COMPOSITE_MODE_DESTINATION_ATOP = 0x7
    D2D1_COMPOSITE_MODE_XOR = 0x8
    D2D1_COMPOSITE_MODE_PLUS = 0x9
    D2D1_COMPOSITE_MODE_SOURCE_COPY = 0xA
    D2D1_COMPOSITE_MODE_BOUNDED_SOURCE_COPY = 0xB
    D2D1_COMPOSITE_MODE_MASK_INVERT = 0xC
    D2D1_COMPOSITE_MODE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_INTERPOLATION_MODE(IntEnum):
    D2D1_INTERPOLATION_MODE_NEAREST_NEIGHBOR = 0x0
    D2D1_INTERPOLATION_MODE_LINEAR = 0x1
    D2D1_INTERPOLATION_MODE_CUBIC = 0x2
    D2D1_INTERPOLATION_MODE_MULTI_SAMPLE_LINEAR = 0x3
    D2D1_INTERPOLATION_MODE_ANISOTROPIC = 0x4
    D2D1_INTERPOLATION_MODE_HIGH_QUALITY_CUBIC = 0x5
    D2D1_INTERPOLATION_MODE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_UNIT_MODE(IntEnum):
    D2D1_UNIT_MODE_DIPS = 0x0
    D2D1_UNIT_MODE_PIXELS = 0x1
    D2D1_UNIT_MODE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_COLOR_SPACE(IntEnum):
    D2D1_COLOR_SPACE_CUSTOM = 0x0
    D2D1_COLOR_SPACE_SRGB = 0x1
    D2D1_COLOR_SPACE_SCRGB = 0x2
    D2D1_COLOR_SPACE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_STROKE_TRANSFORM_TYPE(IntEnum):
    D2D1_STROKE_TRANSFORM_TYPE_NORMAL = 0x0
    D2D1_STROKE_TRANSFORM_TYPE_FIXED = 0x1
    D2D1_STROKE_TRANSFORM_TYPE_HAIRLINE = 0x2
    D2D1_STROKE_TRANSFORM_TYPE_FORCE_DWORD = 0xFFFFFFFF

class D2D1_PRIMITIVE_BLEND(IntEnum):
    D2D1_PRIMITIVE_BLEND_SOURCE_OVER = 0x0
    D2D1_PRIMITIVE_BLEND_COPY = 0x1
    D2D1_PRIMITIVE_BLEND_MIN = 0x2
    D2D1_PRIMITIVE_BLEND_ADD = 0x3
    D2D1_PRIMITIVE_BLEND_MAX = 0x4
    D2D1_PRIMITIVE_BLEND_FORCE_DWORD = 0xFFFFFFFF

class D2D1_LAYER_OPTIONS1(IntEnum):
    D2D1_LAYER_OPTIONS1_NONE = 0x0
    D2D1_LAYER_OPTIONS1_INITIALIZE_FROM_BACKGROUND = 0x1
    D2D1_LAYER_OPTIONS1_IGNORE_ALPHA = 0x2
    D2D1_LAYER_OPTIONS1_FORCE_DWORD = 0xFFFFFFFF

class D2D1_PRINT_FONT_SUBSET_MODE(IntEnum):
    D2D1_PRINT_FONT_SUBSET_MODE_DEFAULT = 0x0
    D2D1_PRINT_FONT_SUBSET_MODE_EACHPAGE = 0x1
    D2D1_PRINT_FONT_SUBSET_MODE_NONE = 0x2
    D2D1_PRINT_FONT_SUBSET_MODE_FORCE_DWORD = 0xFFFFFFFF

class D3D12_SHARED_RESOURCE_COMPATIBILITY_TIER(IntEnum):
    D3D12_SHARED_RESOURCE_COMPATIBILITY_TIER_0 = 0x0
    D3D12_SHARED_RESOURCE_COMPATIBILITY_TIER_1 = 0x1
    D3D12_SHARED_RESOURCE_COMPATIBILITY_TIER_2 = 0x2

class D3D12_MESSAGE_ID(IntEnum):
    D3D12_MESSAGE_ID_UNKNOWN = 0x0
    D3D12_MESSAGE_ID_STRING_FROM_APPLICATION = 0x1
    D3D12_MESSAGE_ID_CORRUPTED_THIS = 0x2
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER1 = 0x3
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER2 = 0x4
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER3 = 0x5
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER4 = 0x6
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER5 = 0x7
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER6 = 0x8
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER7 = 0x9
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER8 = 0xA
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER9 = 0xB
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER10 = 0xC
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER11 = 0xD
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER12 = 0xE
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER13 = 0xF
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER14 = 0x10
    D3D12_MESSAGE_ID_CORRUPTED_PARAMETER15 = 0x11
    D3D12_MESSAGE_ID_CORRUPTED_MULTITHREADING = 0x12
    D3D12_MESSAGE_ID_MESSAGE_REPORTING_OUTOFMEMORY = 0x13
    D3D12_MESSAGE_ID_GETPRIVATEDATA_MOREDATA = 0x14
    D3D12_MESSAGE_ID_SETPRIVATEDATA_INVALIDFREEDATA = 0x15
    D3D12_MESSAGE_ID_SETPRIVATEDATA_CHANGINGPARAMS = 0x18
    D3D12_MESSAGE_ID_SETPRIVATEDATA_OUTOFMEMORY = 0x19
    D3D12_MESSAGE_ID_CREATESHADERRESOURCEVIEW_UNRECOGNIZEDFORMAT = 0x1A
    D3D12_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDDESC = 0x1B
    D3D12_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDFORMAT = 0x1C
    D3D12_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDVIDEOPLANESLICE = 0x1D
    D3D12_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDPLANESLICE = 0x1E
    D3D12_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDDIMENSIONS = 0x1F
    D3D12_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDRESOURCE = 0x20
    D3D12_MESSAGE_ID_CREATERENDERTARGETVIEW_UNRECOGNIZEDFORMAT = 0x23
    D3D12_MESSAGE_ID_CREATERENDERTARGETVIEW_UNSUPPORTEDFORMAT = 0x24
    D3D12_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDDESC = 0x25
    D3D12_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDFORMAT = 0x26
    D3D12_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDVIDEOPLANESLICE = 0x27
    D3D12_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDPLANESLICE = 0x28
    D3D12_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDDIMENSIONS = 0x29
    D3D12_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDRESOURCE = 0x2A
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_UNRECOGNIZEDFORMAT = 0x2D
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDDESC = 0x2E
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDFORMAT = 0x2F
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDDIMENSIONS = 0x30
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDRESOURCE = 0x31
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_OUTOFMEMORY = 0x34
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_TOOMANYELEMENTS = 0x35
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDFORMAT = 0x36
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_INCOMPATIBLEFORMAT = 0x37
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSLOT = 0x38
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDINPUTSLOTCLASS = 0x39
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_STEPRATESLOTCLASSMISMATCH = 0x3A
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSLOTCLASSCHANGE = 0x3B
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSTEPRATECHANGE = 0x3C
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDALIGNMENT = 0x3D
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_DUPLICATESEMANTIC = 0x3E
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_UNPARSEABLEINPUTSIGNATURE = 0x3F
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_NULLSEMANTIC = 0x40
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_MISSINGELEMENT = 0x41
    D3D12_MESSAGE_ID_CREATEVERTEXSHADER_OUTOFMEMORY = 0x42
    D3D12_MESSAGE_ID_CREATEVERTEXSHADER_INVALIDSHADERBYTECODE = 0x43
    D3D12_MESSAGE_ID_CREATEVERTEXSHADER_INVALIDSHADERTYPE = 0x44
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADER_OUTOFMEMORY = 0x45
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADER_INVALIDSHADERBYTECODE = 0x46
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADER_INVALIDSHADERTYPE = 0x47
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTOFMEMORY = 0x48
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSHADERBYTECODE = 0x49
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSHADERTYPE = 0x4A
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDNUMENTRIES = 0x4B
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTPUTSTREAMSTRIDEUNUSED = 0x4C
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTPUTSLOT0EXPECTED = 0x4F
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDOUTPUTSLOT = 0x50
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_ONLYONEELEMENTPERSLOT = 0x51
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDCOMPONENTCOUNT = 0x52
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSTARTCOMPONENTANDCOMPONENTCOUNT = 0x53
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDGAPDEFINITION = 0x54
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_REPEATEDOUTPUT = 0x55
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDOUTPUTSTREAMSTRIDE = 0x56
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MISSINGSEMANTIC = 0x57
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MASKMISMATCH = 0x58
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_CANTHAVEONLYGAPS = 0x59
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_DECLTOOCOMPLEX = 0x5A
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MISSINGOUTPUTSIGNATURE = 0x5B
    D3D12_MESSAGE_ID_CREATEPIXELSHADER_OUTOFMEMORY = 0x5C
    D3D12_MESSAGE_ID_CREATEPIXELSHADER_INVALIDSHADERBYTECODE = 0x5D
    D3D12_MESSAGE_ID_CREATEPIXELSHADER_INVALIDSHADERTYPE = 0x5E
    D3D12_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDFILLMODE = 0x5F
    D3D12_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDCULLMODE = 0x60
    D3D12_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDDEPTHBIASCLAMP = 0x61
    D3D12_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDSLOPESCALEDDEPTHBIAS = 0x62
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDDEPTHWRITEMASK = 0x64
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDDEPTHFUNC = 0x65
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILFAILOP = 0x66
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILZFAILOP = 0x67
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILPASSOP = 0x68
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILFUNC = 0x69
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILFAILOP = 0x6A
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILZFAILOP = 0x6B
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILPASSOP = 0x6C
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILFUNC = 0x6D
    D3D12_MESSAGE_ID_CREATEBLENDSTATE_INVALIDSRCBLEND = 0x6F
    D3D12_MESSAGE_ID_CREATEBLENDSTATE_INVALIDDESTBLEND = 0x70
    D3D12_MESSAGE_ID_CREATEBLENDSTATE_INVALIDBLENDOP = 0x71
    D3D12_MESSAGE_ID_CREATEBLENDSTATE_INVALIDSRCBLENDALPHA = 0x72
    D3D12_MESSAGE_ID_CREATEBLENDSTATE_INVALIDDESTBLENDALPHA = 0x73
    D3D12_MESSAGE_ID_CREATEBLENDSTATE_INVALIDBLENDOPALPHA = 0x74
    D3D12_MESSAGE_ID_CREATEBLENDSTATE_INVALIDRENDERTARGETWRITEMASK = 0x75
    D3D12_MESSAGE_ID_CLEARDEPTHSTENCILVIEW_INVALID = 0x87
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_ROOT_SIGNATURE_NOT_SET = 0xC8
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_ROOT_SIGNATURE_MISMATCH = 0xC9
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_VERTEX_BUFFER_NOT_SET = 0xCA
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_VERTEX_BUFFER_STRIDE_TOO_SMALL = 0xD1
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_VERTEX_BUFFER_TOO_SMALL = 0xD2
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_INDEX_BUFFER_NOT_SET = 0xD3
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_INDEX_BUFFER_FORMAT_INVALID = 0xD4
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_INDEX_BUFFER_TOO_SMALL = 0xD5
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_INVALID_PRIMITIVETOPOLOGY = 0xDB
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_VERTEX_STRIDE_UNALIGNED = 0xDD
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_INDEX_OFFSET_UNALIGNED = 0xDE
    D3D12_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_AT_FAULT = 0xE8
    D3D12_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_POSSIBLY_AT_FAULT = 0xE9
    D3D12_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_NOT_AT_FAULT = 0xEA
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_TRAILING_DIGIT_IN_SEMANTIC = 0xEF
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_TRAILING_DIGIT_IN_SEMANTIC = 0xF0
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_TYPE_MISMATCH = 0xF5
    D3D12_MESSAGE_ID_CREATEINPUTLAYOUT_EMPTY_LAYOUT = 0xFD
    D3D12_MESSAGE_ID_LIVE_OBJECT_SUMMARY = 0xFF
    D3D12_MESSAGE_ID_LIVE_DEVICE = 0x112
    D3D12_MESSAGE_ID_LIVE_SWAPCHAIN = 0x113
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDFLAGS = 0x114
    D3D12_MESSAGE_ID_CREATEVERTEXSHADER_INVALIDCLASSLINKAGE = 0x115
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADER_INVALIDCLASSLINKAGE = 0x116
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSTREAMTORASTERIZER = 0x118
    D3D12_MESSAGE_ID_CREATEPIXELSHADER_INVALIDCLASSLINKAGE = 0x11B
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSTREAM = 0x11C
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UNEXPECTEDENTRIES = 0x11D
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UNEXPECTEDSTRIDES = 0x11E
    D3D12_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDNUMSTRIDES = 0x11F
    D3D12_MESSAGE_ID_CREATEHULLSHADER_OUTOFMEMORY = 0x121
    D3D12_MESSAGE_ID_CREATEHULLSHADER_INVALIDSHADERBYTECODE = 0x122
    D3D12_MESSAGE_ID_CREATEHULLSHADER_INVALIDSHADERTYPE = 0x123
    D3D12_MESSAGE_ID_CREATEHULLSHADER_INVALIDCLASSLINKAGE = 0x124
    D3D12_MESSAGE_ID_CREATEDOMAINSHADER_OUTOFMEMORY = 0x126
    D3D12_MESSAGE_ID_CREATEDOMAINSHADER_INVALIDSHADERBYTECODE = 0x127
    D3D12_MESSAGE_ID_CREATEDOMAINSHADER_INVALIDSHADERTYPE = 0x128
    D3D12_MESSAGE_ID_CREATEDOMAINSHADER_INVALIDCLASSLINKAGE = 0x129
    D3D12_MESSAGE_ID_RESOURCE_UNMAP_NOTMAPPED = 0x136
    D3D12_MESSAGE_ID_DEVICE_CHECKFEATURESUPPORT_MISMATCHED_DATA_SIZE = 0x13E
    D3D12_MESSAGE_ID_CREATECOMPUTESHADER_OUTOFMEMORY = 0x141
    D3D12_MESSAGE_ID_CREATECOMPUTESHADER_INVALIDSHADERBYTECODE = 0x142
    D3D12_MESSAGE_ID_CREATECOMPUTESHADER_INVALIDCLASSLINKAGE = 0x143
    D3D12_MESSAGE_ID_DEVICE_CREATEVERTEXSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x14B
    D3D12_MESSAGE_ID_DEVICE_CREATEHULLSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x14C
    D3D12_MESSAGE_ID_DEVICE_CREATEDOMAINSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x14D
    D3D12_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x14E
    D3D12_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_DOUBLEFLOATOPSNOTSUPPORTED = 0x14F
    D3D12_MESSAGE_ID_DEVICE_CREATEPIXELSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x150
    D3D12_MESSAGE_ID_DEVICE_CREATECOMPUTESHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x151
    D3D12_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDRESOURCE = 0x154
    D3D12_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDDESC = 0x155
    D3D12_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDFORMAT = 0x156
    D3D12_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDVIDEOPLANESLICE = 0x157
    D3D12_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDPLANESLICE = 0x158
    D3D12_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDDIMENSIONS = 0x159
    D3D12_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_UNRECOGNIZEDFORMAT = 0x15A
    D3D12_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDFLAGS = 0x162
    D3D12_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDFORCEDSAMPLECOUNT = 0x191
    D3D12_MESSAGE_ID_CREATEBLENDSTATE_INVALIDLOGICOPS = 0x193
    D3D12_MESSAGE_ID_DEVICE_CREATEVERTEXSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x19A
    D3D12_MESSAGE_ID_DEVICE_CREATEHULLSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x19C
    D3D12_MESSAGE_ID_DEVICE_CREATEDOMAINSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x19E
    D3D12_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x1A0
    D3D12_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_DOUBLEEXTENSIONSNOTSUPPORTED = 0x1A2
    D3D12_MESSAGE_ID_DEVICE_CREATEPIXELSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x1A4
    D3D12_MESSAGE_ID_DEVICE_CREATECOMPUTESHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x1A6
    D3D12_MESSAGE_ID_DEVICE_CREATEVERTEXSHADER_UAVSNOTSUPPORTED = 0x1A9
    D3D12_MESSAGE_ID_DEVICE_CREATEHULLSHADER_UAVSNOTSUPPORTED = 0x1AA
    D3D12_MESSAGE_ID_DEVICE_CREATEDOMAINSHADER_UAVSNOTSUPPORTED = 0x1AB
    D3D12_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADER_UAVSNOTSUPPORTED = 0x1AC
    D3D12_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UAVSNOTSUPPORTED = 0x1AD
    D3D12_MESSAGE_ID_DEVICE_CREATEPIXELSHADER_UAVSNOTSUPPORTED = 0x1AE
    D3D12_MESSAGE_ID_DEVICE_CREATECOMPUTESHADER_UAVSNOTSUPPORTED = 0x1AF
    D3D12_MESSAGE_ID_DEVICE_CLEARVIEW_INVALIDSOURCERECT = 0x1BF
    D3D12_MESSAGE_ID_DEVICE_CLEARVIEW_EMPTYRECT = 0x1C0
    D3D12_MESSAGE_ID_UPDATETILEMAPPINGS_INVALID_PARAMETER = 0x1ED
    D3D12_MESSAGE_ID_COPYTILEMAPPINGS_INVALID_PARAMETER = 0x1EE
    D3D12_MESSAGE_ID_CREATEDEVICE_INVALIDARGS = 0x1FA
    D3D12_MESSAGE_ID_CREATEDEVICE_WARNING = 0x1FB
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_TYPE = 0x207
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_NULL_POINTER = 0x208
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_SUBRESOURCE = 0x209
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_RESERVED_BITS = 0x20A
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_MISSING_BIND_FLAGS = 0x20B
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_MISMATCHING_MISC_FLAGS = 0x20C
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_MATCHING_STATES = 0x20D
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_COMBINATION = 0x20E
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_BEFORE_AFTER_MISMATCH = 0x20F
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_RESOURCE = 0x210
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_SAMPLE_COUNT = 0x211
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_FLAGS = 0x212
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_COMBINED_FLAGS = 0x213
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_FLAGS_FOR_FORMAT = 0x214
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_SPLIT_BARRIER = 0x215
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_UNMATCHED_END = 0x216
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_UNMATCHED_BEGIN = 0x217
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_FLAG = 0x218
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_COMMAND_LIST_TYPE = 0x219
    D3D12_MESSAGE_ID_INVALID_SUBRESOURCE_STATE = 0x21A
    D3D12_MESSAGE_ID_COMMAND_ALLOCATOR_CONTENTION = 0x21C
    D3D12_MESSAGE_ID_COMMAND_ALLOCATOR_RESET = 0x21D
    D3D12_MESSAGE_ID_COMMAND_ALLOCATOR_RESET_BUNDLE = 0x21E
    D3D12_MESSAGE_ID_COMMAND_ALLOCATOR_CANNOT_RESET = 0x21F
    D3D12_MESSAGE_ID_COMMAND_LIST_OPEN = 0x220
    D3D12_MESSAGE_ID_INVALID_BUNDLE_API = 0x222
    D3D12_MESSAGE_ID_COMMAND_LIST_CLOSED = 0x223
    D3D12_MESSAGE_ID_WRONG_COMMAND_ALLOCATOR_TYPE = 0x225
    D3D12_MESSAGE_ID_COMMAND_ALLOCATOR_SYNC = 0x228
    D3D12_MESSAGE_ID_COMMAND_LIST_SYNC = 0x229
    D3D12_MESSAGE_ID_SET_DESCRIPTOR_HEAP_INVALID = 0x22A
    D3D12_MESSAGE_ID_CREATE_COMMANDQUEUE = 0x22D
    D3D12_MESSAGE_ID_CREATE_COMMANDALLOCATOR = 0x22E
    D3D12_MESSAGE_ID_CREATE_PIPELINESTATE = 0x22F
    D3D12_MESSAGE_ID_CREATE_COMMANDLIST12 = 0x230
    D3D12_MESSAGE_ID_CREATE_RESOURCE = 0x232
    D3D12_MESSAGE_ID_CREATE_DESCRIPTORHEAP = 0x233
    D3D12_MESSAGE_ID_CREATE_ROOTSIGNATURE = 0x234
    D3D12_MESSAGE_ID_CREATE_LIBRARY = 0x235
    D3D12_MESSAGE_ID_CREATE_HEAP = 0x236
    D3D12_MESSAGE_ID_CREATE_MONITOREDFENCE = 0x237
    D3D12_MESSAGE_ID_CREATE_QUERYHEAP = 0x238
    D3D12_MESSAGE_ID_CREATE_COMMANDSIGNATURE = 0x239
    D3D12_MESSAGE_ID_LIVE_COMMANDQUEUE = 0x23A
    D3D12_MESSAGE_ID_LIVE_COMMANDALLOCATOR = 0x23B
    D3D12_MESSAGE_ID_LIVE_PIPELINESTATE = 0x23C
    D3D12_MESSAGE_ID_LIVE_COMMANDLIST12 = 0x23D
    D3D12_MESSAGE_ID_LIVE_RESOURCE = 0x23F
    D3D12_MESSAGE_ID_LIVE_DESCRIPTORHEAP = 0x240
    D3D12_MESSAGE_ID_LIVE_ROOTSIGNATURE = 0x241
    D3D12_MESSAGE_ID_LIVE_LIBRARY = 0x242
    D3D12_MESSAGE_ID_LIVE_HEAP = 0x243
    D3D12_MESSAGE_ID_LIVE_MONITOREDFENCE = 0x244
    D3D12_MESSAGE_ID_LIVE_QUERYHEAP = 0x245
    D3D12_MESSAGE_ID_LIVE_COMMANDSIGNATURE = 0x246
    D3D12_MESSAGE_ID_DESTROY_COMMANDQUEUE = 0x247
    D3D12_MESSAGE_ID_DESTROY_COMMANDALLOCATOR = 0x248
    D3D12_MESSAGE_ID_DESTROY_PIPELINESTATE = 0x249
    D3D12_MESSAGE_ID_DESTROY_COMMANDLIST12 = 0x24A
    D3D12_MESSAGE_ID_DESTROY_RESOURCE = 0x24C
    D3D12_MESSAGE_ID_DESTROY_DESCRIPTORHEAP = 0x24D
    D3D12_MESSAGE_ID_DESTROY_ROOTSIGNATURE = 0x24E
    D3D12_MESSAGE_ID_DESTROY_LIBRARY = 0x24F
    D3D12_MESSAGE_ID_DESTROY_HEAP = 0x250
    D3D12_MESSAGE_ID_DESTROY_MONITOREDFENCE = 0x251
    D3D12_MESSAGE_ID_DESTROY_QUERYHEAP = 0x252
    D3D12_MESSAGE_ID_DESTROY_COMMANDSIGNATURE = 0x253
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDDIMENSIONS = 0x255
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDMISCFLAGS = 0x257
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDARG_RETURN = 0x25A
    D3D12_MESSAGE_ID_CREATERESOURCE_OUTOFMEMORY_RETURN = 0x25B
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDDESC = 0x25C
    D3D12_MESSAGE_ID_POSSIBLY_INVALID_SUBRESOURCE_STATE = 0x25F
    D3D12_MESSAGE_ID_INVALID_USE_OF_NON_RESIDENT_RESOURCE = 0x260
    D3D12_MESSAGE_ID_POSSIBLE_INVALID_USE_OF_NON_RESIDENT_RESOURCE = 0x261
    D3D12_MESSAGE_ID_BUNDLE_PIPELINE_STATE_MISMATCH = 0x262
    D3D12_MESSAGE_ID_PRIMITIVE_TOPOLOGY_MISMATCH_PIPELINE_STATE = 0x263
    D3D12_MESSAGE_ID_RENDER_TARGET_FORMAT_MISMATCH_PIPELINE_STATE = 0x265
    D3D12_MESSAGE_ID_RENDER_TARGET_SAMPLE_DESC_MISMATCH_PIPELINE_STATE = 0x266
    D3D12_MESSAGE_ID_DEPTH_STENCIL_FORMAT_MISMATCH_PIPELINE_STATE = 0x267
    D3D12_MESSAGE_ID_DEPTH_STENCIL_SAMPLE_DESC_MISMATCH_PIPELINE_STATE = 0x268
    D3D12_MESSAGE_ID_CREATESHADER_INVALIDBYTECODE = 0x26E
    D3D12_MESSAGE_ID_CREATEHEAP_NULLDESC = 0x26F
    D3D12_MESSAGE_ID_CREATEHEAP_INVALIDSIZE = 0x270
    D3D12_MESSAGE_ID_CREATEHEAP_UNRECOGNIZEDHEAPTYPE = 0x271
    D3D12_MESSAGE_ID_CREATEHEAP_UNRECOGNIZEDCPUPAGEPROPERTIES = 0x272
    D3D12_MESSAGE_ID_CREATEHEAP_UNRECOGNIZEDMEMORYPOOL = 0x273
    D3D12_MESSAGE_ID_CREATEHEAP_INVALIDPROPERTIES = 0x274
    D3D12_MESSAGE_ID_CREATEHEAP_INVALIDALIGNMENT = 0x275
    D3D12_MESSAGE_ID_CREATEHEAP_UNRECOGNIZEDMISCFLAGS = 0x276
    D3D12_MESSAGE_ID_CREATEHEAP_INVALIDMISCFLAGS = 0x277
    D3D12_MESSAGE_ID_CREATEHEAP_INVALIDARG_RETURN = 0x278
    D3D12_MESSAGE_ID_CREATEHEAP_OUTOFMEMORY_RETURN = 0x279
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_NULLHEAPPROPERTIES = 0x27A
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_UNRECOGNIZEDHEAPTYPE = 0x27B
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_UNRECOGNIZEDCPUPAGEPROPERTIES = 0x27C
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_UNRECOGNIZEDMEMORYPOOL = 0x27D
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_INVALIDHEAPPROPERTIES = 0x27E
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_UNRECOGNIZEDHEAPMISCFLAGS = 0x27F
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_INVALIDHEAPMISCFLAGS = 0x280
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_INVALIDARG_RETURN = 0x281
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_OUTOFMEMORY_RETURN = 0x282
    D3D12_MESSAGE_ID_GETCUSTOMHEAPPROPERTIES_UNRECOGNIZEDHEAPTYPE = 0x283
    D3D12_MESSAGE_ID_GETCUSTOMHEAPPROPERTIES_INVALIDHEAPTYPE = 0x284
    D3D12_MESSAGE_ID_CREATE_DESCRIPTOR_HEAP_INVALID_DESC = 0x285
    D3D12_MESSAGE_ID_INVALID_DESCRIPTOR_HANDLE = 0x286
    D3D12_MESSAGE_ID_CREATERASTERIZERSTATE_INVALID_CONSERVATIVERASTERMODE = 0x287
    D3D12_MESSAGE_ID_CREATE_CONSTANT_BUFFER_VIEW_INVALID_RESOURCE = 0x289
    D3D12_MESSAGE_ID_CREATE_CONSTANT_BUFFER_VIEW_INVALID_DESC = 0x28A
    D3D12_MESSAGE_ID_CREATE_UNORDEREDACCESS_VIEW_INVALID_COUNTER_USAGE = 0x28C
    D3D12_MESSAGE_ID_COPY_DESCRIPTORS_INVALID_RANGES = 0x28D
    D3D12_MESSAGE_ID_COPY_DESCRIPTORS_WRITE_ONLY_DESCRIPTOR = 0x28E
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_RTV_FORMAT_NOT_UNKNOWN = 0x28F
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INVALID_RENDER_TARGET_COUNT = 0x290
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_VERTEX_SHADER_NOT_SET = 0x291
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INPUTLAYOUT_NOT_SET = 0x292
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_LINKAGE_HS_DS_SIGNATURE_MISMATCH = 0x293
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_LINKAGE_REGISTERINDEX = 0x294
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_LINKAGE_COMPONENTTYPE = 0x295
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_LINKAGE_REGISTERMASK = 0x296
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_LINKAGE_SYSTEMVALUE = 0x297
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_LINKAGE_NEVERWRITTEN_ALWAYSREADS = 0x298
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_LINKAGE_MINPRECISION = 0x299
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_LINKAGE_SEMANTICNAME_NOT_FOUND = 0x29A
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_HS_XOR_DS_MISMATCH = 0x29B
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_HULL_SHADER_INPUT_TOPOLOGY_MISMATCH = 0x29C
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_HS_DS_CONTROL_POINT_COUNT_MISMATCH = 0x29D
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_HS_DS_TESSELLATOR_DOMAIN_MISMATCH = 0x29E
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INVALID_USE_OF_CENTER_MULTISAMPLE_PATTERN = 0x29F
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INVALID_USE_OF_FORCED_SAMPLE_COUNT = 0x2A0
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INVALID_PRIMITIVETOPOLOGY = 0x2A1
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INVALID_SYSTEMVALUE = 0x2A2
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_OM_DUAL_SOURCE_BLENDING_CAN_ONLY_HAVE_RENDER_TARGET_0 = 0x2A3
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_OM_RENDER_TARGET_DOES_NOT_SUPPORT_BLENDING = 0x2A4
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_PS_OUTPUT_TYPE_MISMATCH = 0x2A5
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_OM_RENDER_TARGET_DOES_NOT_SUPPORT_LOGIC_OPS = 0x2A6
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_RENDERTARGETVIEW_NOT_SET = 0x2A7
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_DEPTHSTENCILVIEW_NOT_SET = 0x2A8
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_GS_INPUT_PRIMITIVE_MISMATCH = 0x2A9
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_POSITION_NOT_PRESENT = 0x2AA
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_MISSING_ROOT_SIGNATURE_FLAGS = 0x2AB
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INVALID_INDEX_BUFFER_PROPERTIES = 0x2AC
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INVALID_SAMPLE_DESC = 0x2AD
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_HS_ROOT_SIGNATURE_MISMATCH = 0x2AE
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_DS_ROOT_SIGNATURE_MISMATCH = 0x2AF
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_VS_ROOT_SIGNATURE_MISMATCH = 0x2B0
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_GS_ROOT_SIGNATURE_MISMATCH = 0x2B1
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_PS_ROOT_SIGNATURE_MISMATCH = 0x2B2
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_MISSING_ROOT_SIGNATURE = 0x2B3
    D3D12_MESSAGE_ID_EXECUTE_BUNDLE_OPEN_BUNDLE = 0x2B4
    D3D12_MESSAGE_ID_EXECUTE_BUNDLE_DESCRIPTOR_HEAP_MISMATCH = 0x2B5
    D3D12_MESSAGE_ID_EXECUTE_BUNDLE_TYPE = 0x2B6
    D3D12_MESSAGE_ID_DRAW_EMPTY_SCISSOR_RECTANGLE = 0x2B7
    D3D12_MESSAGE_ID_CREATE_ROOT_SIGNATURE_BLOB_NOT_FOUND = 0x2B8
    D3D12_MESSAGE_ID_CREATE_ROOT_SIGNATURE_DESERIALIZE_FAILED = 0x2B9
    D3D12_MESSAGE_ID_CREATE_ROOT_SIGNATURE_INVALID_CONFIGURATION = 0x2BA
    D3D12_MESSAGE_ID_CREATE_ROOT_SIGNATURE_NOT_SUPPORTED_ON_DEVICE = 0x2BB
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_NULLRESOURCEPROPERTIES = 0x2BC
    D3D12_MESSAGE_ID_CREATERESOURCEANDHEAP_NULLHEAP = 0x2BD
    D3D12_MESSAGE_ID_GETRESOURCEALLOCATIONINFO_INVALIDRDESCS = 0x2BE
    D3D12_MESSAGE_ID_MAKERESIDENT_NULLOBJECTARRAY = 0x2BF
    D3D12_MESSAGE_ID_EVICT_NULLOBJECTARRAY = 0x2C1
    D3D12_MESSAGE_ID_SET_DESCRIPTOR_TABLE_INVALID = 0x2C4
    D3D12_MESSAGE_ID_SET_ROOT_CONSTANT_INVALID = 0x2C5
    D3D12_MESSAGE_ID_SET_ROOT_CONSTANT_BUFFER_VIEW_INVALID = 0x2C6
    D3D12_MESSAGE_ID_SET_ROOT_SHADER_RESOURCE_VIEW_INVALID = 0x2C7
    D3D12_MESSAGE_ID_SET_ROOT_UNORDERED_ACCESS_VIEW_INVALID = 0x2C8
    D3D12_MESSAGE_ID_SET_VERTEX_BUFFERS_INVALID_DESC = 0x2C9
    D3D12_MESSAGE_ID_SET_INDEX_BUFFER_INVALID_DESC = 0x2CB
    D3D12_MESSAGE_ID_SET_STREAM_OUTPUT_BUFFERS_INVALID_DESC = 0x2CD
    D3D12_MESSAGE_ID_CREATERESOURCE_UNRECOGNIZEDDIMENSIONALITY = 0x2CE
    D3D12_MESSAGE_ID_CREATERESOURCE_UNRECOGNIZEDLAYOUT = 0x2CF
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDDIMENSIONALITY = 0x2D0
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDALIGNMENT = 0x2D1
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDMIPLEVELS = 0x2D2
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDSAMPLEDESC = 0x2D3
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDLAYOUT = 0x2D4
    D3D12_MESSAGE_ID_SET_INDEX_BUFFER_INVALID = 0x2D5
    D3D12_MESSAGE_ID_SET_VERTEX_BUFFERS_INVALID = 0x2D6
    D3D12_MESSAGE_ID_SET_STREAM_OUTPUT_BUFFERS_INVALID = 0x2D7
    D3D12_MESSAGE_ID_SET_RENDER_TARGETS_INVALID = 0x2D8
    D3D12_MESSAGE_ID_CREATEQUERY_HEAP_INVALID_PARAMETERS = 0x2D9
    D3D12_MESSAGE_ID_BEGIN_END_QUERY_INVALID_PARAMETERS = 0x2DB
    D3D12_MESSAGE_ID_CLOSE_COMMAND_LIST_OPEN_QUERY = 0x2DC
    D3D12_MESSAGE_ID_RESOLVE_QUERY_DATA_INVALID_PARAMETERS = 0x2DD
    D3D12_MESSAGE_ID_SET_PREDICATION_INVALID_PARAMETERS = 0x2DE
    D3D12_MESSAGE_ID_TIMESTAMPS_NOT_SUPPORTED = 0x2DF
    D3D12_MESSAGE_ID_CREATERESOURCE_UNRECOGNIZEDFORMAT = 0x2E1
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDFORMAT = 0x2E2
    D3D12_MESSAGE_ID_GETCOPYABLEFOOTPRINTS_INVALIDSUBRESOURCERANGE = 0x2E3
    D3D12_MESSAGE_ID_GETCOPYABLEFOOTPRINTS_INVALIDBASEOFFSET = 0x2E4
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_INVALID_HEAP = 0x2E5
    D3D12_MESSAGE_ID_CREATE_SAMPLER_INVALID = 0x2E6
    D3D12_MESSAGE_ID_CREATECOMMANDSIGNATURE_INVALID = 0x2E7
    D3D12_MESSAGE_ID_EXECUTE_INDIRECT_INVALID_PARAMETERS = 0x2E8
    D3D12_MESSAGE_ID_GETGPUVIRTUALADDRESS_INVALID_RESOURCE_DIMENSION = 0x2E9
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDCLEARVALUE = 0x32F
    D3D12_MESSAGE_ID_CREATERESOURCE_UNRECOGNIZEDCLEARVALUEFORMAT = 0x330
    D3D12_MESSAGE_ID_CREATERESOURCE_INVALIDCLEARVALUEFORMAT = 0x331
    D3D12_MESSAGE_ID_CREATERESOURCE_CLEARVALUEDENORMFLUSH = 0x332
    D3D12_MESSAGE_ID_CLEARRENDERTARGETVIEW_MISMATCHINGCLEARVALUE = 0x334
    D3D12_MESSAGE_ID_CLEARDEPTHSTENCILVIEW_MISMATCHINGCLEARVALUE = 0x335
    D3D12_MESSAGE_ID_MAP_INVALIDHEAP = 0x336
    D3D12_MESSAGE_ID_UNMAP_INVALIDHEAP = 0x337
    D3D12_MESSAGE_ID_MAP_INVALIDRESOURCE = 0x338
    D3D12_MESSAGE_ID_UNMAP_INVALIDRESOURCE = 0x339
    D3D12_MESSAGE_ID_MAP_INVALIDSUBRESOURCE = 0x33A
    D3D12_MESSAGE_ID_UNMAP_INVALIDSUBRESOURCE = 0x33B
    D3D12_MESSAGE_ID_MAP_INVALIDRANGE = 0x33C
    D3D12_MESSAGE_ID_UNMAP_INVALIDRANGE = 0x33D
    D3D12_MESSAGE_ID_MAP_INVALIDDATAPOINTER = 0x340
    D3D12_MESSAGE_ID_MAP_INVALIDARG_RETURN = 0x341
    D3D12_MESSAGE_ID_MAP_OUTOFMEMORY_RETURN = 0x342
    D3D12_MESSAGE_ID_EXECUTECOMMANDLISTS_BUNDLENOTSUPPORTED = 0x343
    D3D12_MESSAGE_ID_EXECUTECOMMANDLISTS_COMMANDLISTMISMATCH = 0x344
    D3D12_MESSAGE_ID_EXECUTECOMMANDLISTS_OPENCOMMANDLIST = 0x345
    D3D12_MESSAGE_ID_EXECUTECOMMANDLISTS_FAILEDCOMMANDLIST = 0x346
    D3D12_MESSAGE_ID_COPYBUFFERREGION_NULLDST = 0x347
    D3D12_MESSAGE_ID_COPYBUFFERREGION_INVALIDDSTRESOURCEDIMENSION = 0x348
    D3D12_MESSAGE_ID_COPYBUFFERREGION_DSTRANGEOUTOFBOUNDS = 0x349
    D3D12_MESSAGE_ID_COPYBUFFERREGION_NULLSRC = 0x34A
    D3D12_MESSAGE_ID_COPYBUFFERREGION_INVALIDSRCRESOURCEDIMENSION = 0x34B
    D3D12_MESSAGE_ID_COPYBUFFERREGION_SRCRANGEOUTOFBOUNDS = 0x34C
    D3D12_MESSAGE_ID_COPYBUFFERREGION_INVALIDCOPYFLAGS = 0x34D
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_NULLDST = 0x34E
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_UNRECOGNIZEDDSTTYPE = 0x34F
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTRESOURCEDIMENSION = 0x350
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTRESOURCE = 0x351
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTSUBRESOURCE = 0x352
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTOFFSET = 0x353
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_UNRECOGNIZEDDSTFORMAT = 0x354
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTFORMAT = 0x355
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTDIMENSIONS = 0x356
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTROWPITCH = 0x357
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTPLACEMENT = 0x358
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTDSPLACEDFOOTPRINTFORMAT = 0x359
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_DSTREGIONOUTOFBOUNDS = 0x35A
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_NULLSRC = 0x35B
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_UNRECOGNIZEDSRCTYPE = 0x35C
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCRESOURCEDIMENSION = 0x35D
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCRESOURCE = 0x35E
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCSUBRESOURCE = 0x35F
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCOFFSET = 0x360
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_UNRECOGNIZEDSRCFORMAT = 0x361
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCFORMAT = 0x362
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCDIMENSIONS = 0x363
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCROWPITCH = 0x364
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCPLACEMENT = 0x365
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCDSPLACEDFOOTPRINTFORMAT = 0x366
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_SRCREGIONOUTOFBOUNDS = 0x367
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDDSTCOORDINATES = 0x368
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDSRCBOX = 0x369
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_FORMATMISMATCH = 0x36A
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_EMPTYBOX = 0x36B
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_INVALIDCOPYFLAGS = 0x36C
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_INVALID_SUBRESOURCE_INDEX = 0x36D
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_INVALID_FORMAT = 0x36E
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_RESOURCE_MISMATCH = 0x36F
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_INVALID_SAMPLE_COUNT = 0x370
    D3D12_MESSAGE_ID_CREATECOMPUTEPIPELINESTATE_INVALID_SHADER = 0x371
    D3D12_MESSAGE_ID_CREATECOMPUTEPIPELINESTATE_CS_ROOT_SIGNATURE_MISMATCH = 0x372
    D3D12_MESSAGE_ID_CREATECOMPUTEPIPELINESTATE_MISSING_ROOT_SIGNATURE = 0x373
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_INVALIDCACHEDBLOB = 0x374
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_CACHEDBLOBADAPTERMISMATCH = 0x375
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_CACHEDBLOBDRIVERVERSIONMISMATCH = 0x376
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_CACHEDBLOBDESCMISMATCH = 0x377
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_CACHEDBLOBIGNORED = 0x378
    D3D12_MESSAGE_ID_WRITETOSUBRESOURCE_INVALIDHEAP = 0x379
    D3D12_MESSAGE_ID_WRITETOSUBRESOURCE_INVALIDRESOURCE = 0x37A
    D3D12_MESSAGE_ID_WRITETOSUBRESOURCE_INVALIDBOX = 0x37B
    D3D12_MESSAGE_ID_WRITETOSUBRESOURCE_INVALIDSUBRESOURCE = 0x37C
    D3D12_MESSAGE_ID_WRITETOSUBRESOURCE_EMPTYBOX = 0x37D
    D3D12_MESSAGE_ID_READFROMSUBRESOURCE_INVALIDHEAP = 0x37E
    D3D12_MESSAGE_ID_READFROMSUBRESOURCE_INVALIDRESOURCE = 0x37F
    D3D12_MESSAGE_ID_READFROMSUBRESOURCE_INVALIDBOX = 0x380
    D3D12_MESSAGE_ID_READFROMSUBRESOURCE_INVALIDSUBRESOURCE = 0x381
    D3D12_MESSAGE_ID_READFROMSUBRESOURCE_EMPTYBOX = 0x382
    D3D12_MESSAGE_ID_TOO_MANY_NODES_SPECIFIED = 0x383
    D3D12_MESSAGE_ID_INVALID_NODE_INDEX = 0x384
    D3D12_MESSAGE_ID_GETHEAPPROPERTIES_INVALIDRESOURCE = 0x385
    D3D12_MESSAGE_ID_NODE_MASK_MISMATCH = 0x386
    D3D12_MESSAGE_ID_COMMAND_LIST_OUTOFMEMORY = 0x387
    D3D12_MESSAGE_ID_COMMAND_LIST_MULTIPLE_SWAPCHAIN_BUFFER_REFERENCES = 0x388
    D3D12_MESSAGE_ID_COMMAND_LIST_TOO_MANY_SWAPCHAIN_REFERENCES = 0x389
    D3D12_MESSAGE_ID_COMMAND_QUEUE_TOO_MANY_SWAPCHAIN_REFERENCES = 0x38A
    D3D12_MESSAGE_ID_EXECUTECOMMANDLISTS_WRONGSWAPCHAINBUFFERREFERENCE = 0x38B
    D3D12_MESSAGE_ID_COMMAND_LIST_SETRENDERTARGETS_INVALIDNUMRENDERTARGETS = 0x38C
    D3D12_MESSAGE_ID_CREATE_QUEUE_INVALID_TYPE = 0x38D
    D3D12_MESSAGE_ID_CREATE_QUEUE_INVALID_FLAGS = 0x38E
    D3D12_MESSAGE_ID_CREATESHAREDRESOURCE_INVALIDFLAGS = 0x38F
    D3D12_MESSAGE_ID_CREATESHAREDRESOURCE_INVALIDFORMAT = 0x390
    D3D12_MESSAGE_ID_CREATESHAREDHEAP_INVALIDFLAGS = 0x391
    D3D12_MESSAGE_ID_REFLECTSHAREDPROPERTIES_UNRECOGNIZEDPROPERTIES = 0x392
    D3D12_MESSAGE_ID_REFLECTSHAREDPROPERTIES_INVALIDSIZE = 0x393
    D3D12_MESSAGE_ID_REFLECTSHAREDPROPERTIES_INVALIDOBJECT = 0x394
    D3D12_MESSAGE_ID_KEYEDMUTEX_INVALIDOBJECT = 0x395
    D3D12_MESSAGE_ID_KEYEDMUTEX_INVALIDKEY = 0x396
    D3D12_MESSAGE_ID_KEYEDMUTEX_WRONGSTATE = 0x397
    D3D12_MESSAGE_ID_CREATE_QUEUE_INVALID_PRIORITY = 0x398
    D3D12_MESSAGE_ID_OBJECT_DELETED_WHILE_STILL_IN_USE = 0x399
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_INVALID_FLAGS = 0x39A
    D3D12_MESSAGE_ID_HEAP_ADDRESS_RANGE_HAS_NO_RESOURCE = 0x39B
    D3D12_MESSAGE_ID_COMMAND_LIST_DRAW_RENDER_TARGET_DELETED = 0x39C
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_ALL_RENDER_TARGETS_HAVE_UNKNOWN_FORMAT = 0x39D
    D3D12_MESSAGE_ID_HEAP_ADDRESS_RANGE_INTERSECTS_MULTIPLE_BUFFERS = 0x39E
    D3D12_MESSAGE_ID_EXECUTECOMMANDLISTS_GPU_WRITTEN_READBACK_RESOURCE_MAPPED = 0x39F
    D3D12_MESSAGE_ID_UNMAP_RANGE_NOT_EMPTY = 0x3A1
    D3D12_MESSAGE_ID_MAP_INVALID_NULLRANGE = 0x3A2
    D3D12_MESSAGE_ID_UNMAP_INVALID_NULLRANGE = 0x3A3
    D3D12_MESSAGE_ID_NO_GRAPHICS_API_SUPPORT = 0x3A4
    D3D12_MESSAGE_ID_NO_COMPUTE_API_SUPPORT = 0x3A5
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_RESOURCE_FLAGS_NOT_SUPPORTED = 0x3A6
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_ROOT_ARGUMENT_UNINITIALIZED = 0x3A7
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_DESCRIPTOR_HEAP_INDEX_OUT_OF_BOUNDS = 0x3A8
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_DESCRIPTOR_TABLE_REGISTER_INDEX_OUT_OF_BOUNDS = 0x3A9
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_DESCRIPTOR_UNINITIALIZED = 0x3AA
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_DESCRIPTOR_TYPE_MISMATCH = 0x3AB
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_SRV_RESOURCE_DIMENSION_MISMATCH = 0x3AC
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_UAV_RESOURCE_DIMENSION_MISMATCH = 0x3AD
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_INCOMPATIBLE_RESOURCE_STATE = 0x3AE
    D3D12_MESSAGE_ID_COPYRESOURCE_NULLDST = 0x3AF
    D3D12_MESSAGE_ID_COPYRESOURCE_INVALIDDSTRESOURCE = 0x3B0
    D3D12_MESSAGE_ID_COPYRESOURCE_NULLSRC = 0x3B1
    D3D12_MESSAGE_ID_COPYRESOURCE_INVALIDSRCRESOURCE = 0x3B2
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_NULLDST = 0x3B3
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_INVALIDDSTRESOURCE = 0x3B4
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_NULLSRC = 0x3B5
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_INVALIDSRCRESOURCE = 0x3B6
    D3D12_MESSAGE_ID_PIPELINE_STATE_TYPE_MISMATCH = 0x3B7
    D3D12_MESSAGE_ID_COMMAND_LIST_DISPATCH_ROOT_SIGNATURE_NOT_SET = 0x3B8
    D3D12_MESSAGE_ID_COMMAND_LIST_DISPATCH_ROOT_SIGNATURE_MISMATCH = 0x3B9
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_ZERO_BARRIERS = 0x3BA
    D3D12_MESSAGE_ID_BEGIN_END_EVENT_MISMATCH = 0x3BB
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_POSSIBLE_BEFORE_AFTER_MISMATCH = 0x3BC
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_MISMATCHING_BEGIN_END = 0x3BD
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_INVALID_RESOURCE = 0x3BE
    D3D12_MESSAGE_ID_USE_OF_ZERO_REFCOUNT_OBJECT = 0x3BF
    D3D12_MESSAGE_ID_OBJECT_EVICTED_WHILE_STILL_IN_USE = 0x3C0
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_ROOT_DESCRIPTOR_ACCESS_OUT_OF_BOUNDS = 0x3C1
    D3D12_MESSAGE_ID_CREATEPIPELINELIBRARY_INVALIDLIBRARYBLOB = 0x3C2
    D3D12_MESSAGE_ID_CREATEPIPELINELIBRARY_DRIVERVERSIONMISMATCH = 0x3C3
    D3D12_MESSAGE_ID_CREATEPIPELINELIBRARY_ADAPTERVERSIONMISMATCH = 0x3C4
    D3D12_MESSAGE_ID_CREATEPIPELINELIBRARY_UNSUPPORTED = 0x3C5
    D3D12_MESSAGE_ID_CREATE_PIPELINELIBRARY = 0x3C6
    D3D12_MESSAGE_ID_LIVE_PIPELINELIBRARY = 0x3C7
    D3D12_MESSAGE_ID_DESTROY_PIPELINELIBRARY = 0x3C8
    D3D12_MESSAGE_ID_STOREPIPELINE_NONAME = 0x3C9
    D3D12_MESSAGE_ID_STOREPIPELINE_DUPLICATENAME = 0x3CA
    D3D12_MESSAGE_ID_LOADPIPELINE_NAMENOTFOUND = 0x3CB
    D3D12_MESSAGE_ID_LOADPIPELINE_INVALIDDESC = 0x3CC
    D3D12_MESSAGE_ID_PIPELINELIBRARY_SERIALIZE_NOTENOUGHMEMORY = 0x3CD
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_PS_OUTPUT_RT_OUTPUT_MISMATCH = 0x3CE
    D3D12_MESSAGE_ID_SETEVENTONMULTIPLEFENCECOMPLETION_INVALIDFLAGS = 0x3CF
    D3D12_MESSAGE_ID_CREATE_QUEUE_VIDEO_NOT_SUPPORTED = 0x3D0
    D3D12_MESSAGE_ID_CREATE_COMMAND_ALLOCATOR_VIDEO_NOT_SUPPORTED = 0x3D1
    D3D12_MESSAGE_ID_CREATEQUERY_HEAP_VIDEO_DECODE_STATISTICS_NOT_SUPPORTED = 0x3D2
    D3D12_MESSAGE_ID_CREATE_VIDEODECODECOMMANDLIST = 0x3D3
    D3D12_MESSAGE_ID_CREATE_VIDEODECODER = 0x3D4
    D3D12_MESSAGE_ID_CREATE_VIDEODECODESTREAM = 0x3D5
    D3D12_MESSAGE_ID_LIVE_VIDEODECODECOMMANDLIST = 0x3D6
    D3D12_MESSAGE_ID_LIVE_VIDEODECODER = 0x3D7
    D3D12_MESSAGE_ID_LIVE_VIDEODECODESTREAM = 0x3D8
    D3D12_MESSAGE_ID_DESTROY_VIDEODECODECOMMANDLIST = 0x3D9
    D3D12_MESSAGE_ID_DESTROY_VIDEODECODER = 0x3DA
    D3D12_MESSAGE_ID_DESTROY_VIDEODECODESTREAM = 0x3DB
    D3D12_MESSAGE_ID_DECODE_FRAME_INVALID_PARAMETERS = 0x3DC
    D3D12_MESSAGE_ID_DEPRECATED_API = 0x3DD
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_MISMATCHING_COMMAND_LIST_TYPE = 0x3DE
    D3D12_MESSAGE_ID_COMMAND_LIST_DESCRIPTOR_TABLE_NOT_SET = 0x3DF
    D3D12_MESSAGE_ID_COMMAND_LIST_ROOT_CONSTANT_BUFFER_VIEW_NOT_SET = 0x3E0
    D3D12_MESSAGE_ID_COMMAND_LIST_ROOT_SHADER_RESOURCE_VIEW_NOT_SET = 0x3E1
    D3D12_MESSAGE_ID_COMMAND_LIST_ROOT_UNORDERED_ACCESS_VIEW_NOT_SET = 0x3E2
    D3D12_MESSAGE_ID_DISCARD_INVALID_SUBRESOURCE_RANGE = 0x3E3
    D3D12_MESSAGE_ID_DISCARD_ONE_SUBRESOURCE_FOR_MIPS_WITH_RECTS = 0x3E4
    D3D12_MESSAGE_ID_DISCARD_NO_RECTS_FOR_NON_TEXTURE2D = 0x3E5
    D3D12_MESSAGE_ID_COPY_ON_SAME_SUBRESOURCE = 0x3E6
    D3D12_MESSAGE_ID_SETRESIDENCYPRIORITY_INVALID_PAGEABLE = 0x3E7
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_UNSUPPORTED = 0x3E8
    D3D12_MESSAGE_ID_STATIC_DESCRIPTOR_INVALID_DESCRIPTOR_CHANGE = 0x3E9
    D3D12_MESSAGE_ID_DATA_STATIC_DESCRIPTOR_INVALID_DATA_CHANGE = 0x3EA
    D3D12_MESSAGE_ID_DATA_STATIC_WHILE_SET_AT_EXECUTE_DESCRIPTOR_INVALID_DATA_CHANGE = 0x3EB
    D3D12_MESSAGE_ID_EXECUTE_BUNDLE_STATIC_DESCRIPTOR_DATA_STATIC_NOT_SET = 0x3EC
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_RESOURCE_ACCESS_OUT_OF_BOUNDS = 0x3ED
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_SAMPLER_MODE_MISMATCH = 0x3EE
    D3D12_MESSAGE_ID_CREATE_FENCE_INVALID_FLAGS = 0x3EF
    D3D12_MESSAGE_ID_RESOURCE_BARRIER_DUPLICATE_SUBRESOURCE_TRANSITIONS = 0x3F0
    D3D12_MESSAGE_ID_SETRESIDENCYPRIORITY_INVALID_PRIORITY = 0x3F1
    D3D12_MESSAGE_ID_CREATE_DESCRIPTOR_HEAP_LARGE_NUM_DESCRIPTORS = 0x3F5
    D3D12_MESSAGE_ID_BEGIN_EVENT = 0x3F6
    D3D12_MESSAGE_ID_END_EVENT = 0x3F7
    D3D12_MESSAGE_ID_CREATEDEVICE_DEBUG_LAYER_STARTUP_OPTIONS = 0x3F8
    D3D12_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_DEPTHBOUNDSTEST_UNSUPPORTED = 0x3F9
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_DUPLICATE_SUBOBJECT = 0x3FA
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_UNKNOWN_SUBOBJECT = 0x3FB
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_ZERO_SIZE_STREAM = 0x3FC
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_INVALID_STREAM = 0x3FD
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_CANNOT_DEDUCE_TYPE = 0x3FE
    D3D12_MESSAGE_ID_COMMAND_LIST_STATIC_DESCRIPTOR_RESOURCE_DIMENSION_MISMATCH = 0x3FF
    D3D12_MESSAGE_ID_CREATE_COMMAND_QUEUE_INSUFFICIENT_PRIVILEGE_FOR_GLOBAL_REALTIME = 0x400
    D3D12_MESSAGE_ID_CREATE_COMMAND_QUEUE_INSUFFICIENT_HARDWARE_SUPPORT_FOR_GLOBAL_REALTIME = 0x401
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_INVALID_ARCHITECTURE = 0x402
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_NULL_DST = 0x403
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_INVALID_DST_RESOURCE_DIMENSION = 0x404
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_DST_RANGE_OUT_OF_BOUNDS = 0x405
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_NULL_SRC = 0x406
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_INVALID_SRC_RESOURCE_DIMENSION = 0x407
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_SRC_RANGE_OUT_OF_BOUNDS = 0x408
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_INVALID_OFFSET_ALIGNMENT = 0x409
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_NULL_DEPENDENT_RESOURCES = 0x40A
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_NULL_DEPENDENT_SUBRESOURCE_RANGES = 0x40B
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_INVALID_DEPENDENT_RESOURCE = 0x40C
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_INVALID_DEPENDENT_SUBRESOURCE_RANGE = 0x40D
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_DEPENDENT_SUBRESOURCE_OUT_OF_BOUNDS = 0x40E
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_DEPENDENT_RANGE_OUT_OF_BOUNDS = 0x40F
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_ZERO_DEPENDENCIES = 0x410
    D3D12_MESSAGE_ID_DEVICE_CREATE_SHARED_HANDLE_INVALIDARG = 0x411
    D3D12_MESSAGE_ID_DESCRIPTOR_HANDLE_WITH_INVALID_RESOURCE = 0x412
    D3D12_MESSAGE_ID_SETDEPTHBOUNDS_INVALIDARGS = 0x413
    D3D12_MESSAGE_ID_GPU_BASED_VALIDATION_RESOURCE_STATE_IMPRECISE = 0x414
    D3D12_MESSAGE_ID_COMMAND_LIST_PIPELINE_STATE_NOT_SET = 0x415
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_SHADER_MODEL_MISMATCH = 0x416
    D3D12_MESSAGE_ID_OBJECT_ACCESSED_WHILE_STILL_IN_USE = 0x417
    D3D12_MESSAGE_ID_PROGRAMMABLE_MSAA_UNSUPPORTED = 0x418
    D3D12_MESSAGE_ID_SETSAMPLEPOSITIONS_INVALIDARGS = 0x419
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCEREGION_INVALID_RECT = 0x41A
    D3D12_MESSAGE_ID_CREATE_VIDEODECODECOMMANDQUEUE = 0x41B
    D3D12_MESSAGE_ID_CREATE_VIDEOPROCESSCOMMANDLIST = 0x41C
    D3D12_MESSAGE_ID_CREATE_VIDEOPROCESSCOMMANDQUEUE = 0x41D
    D3D12_MESSAGE_ID_LIVE_VIDEODECODECOMMANDQUEUE = 0x41E
    D3D12_MESSAGE_ID_LIVE_VIDEOPROCESSCOMMANDLIST = 0x41F
    D3D12_MESSAGE_ID_LIVE_VIDEOPROCESSCOMMANDQUEUE = 0x420
    D3D12_MESSAGE_ID_DESTROY_VIDEODECODECOMMANDQUEUE = 0x421
    D3D12_MESSAGE_ID_DESTROY_VIDEOPROCESSCOMMANDLIST = 0x422
    D3D12_MESSAGE_ID_DESTROY_VIDEOPROCESSCOMMANDQUEUE = 0x423
    D3D12_MESSAGE_ID_CREATE_VIDEOPROCESSOR = 0x424
    D3D12_MESSAGE_ID_CREATE_VIDEOPROCESSSTREAM = 0x425
    D3D12_MESSAGE_ID_LIVE_VIDEOPROCESSOR = 0x426
    D3D12_MESSAGE_ID_LIVE_VIDEOPROCESSSTREAM = 0x427
    D3D12_MESSAGE_ID_DESTROY_VIDEOPROCESSOR = 0x428
    D3D12_MESSAGE_ID_DESTROY_VIDEOPROCESSSTREAM = 0x429
    D3D12_MESSAGE_ID_PROCESS_FRAME_INVALID_PARAMETERS = 0x42A
    D3D12_MESSAGE_ID_COPY_INVALIDLAYOUT = 0x42B
    D3D12_MESSAGE_ID_CREATE_CRYPTO_SESSION = 0x42C
    D3D12_MESSAGE_ID_CREATE_CRYPTO_SESSION_POLICY = 0x42D
    D3D12_MESSAGE_ID_CREATE_PROTECTED_RESOURCE_SESSION = 0x42E
    D3D12_MESSAGE_ID_LIVE_CRYPTO_SESSION = 0x42F
    D3D12_MESSAGE_ID_LIVE_CRYPTO_SESSION_POLICY = 0x430
    D3D12_MESSAGE_ID_LIVE_PROTECTED_RESOURCE_SESSION = 0x431
    D3D12_MESSAGE_ID_DESTROY_CRYPTO_SESSION = 0x432
    D3D12_MESSAGE_ID_DESTROY_CRYPTO_SESSION_POLICY = 0x433
    D3D12_MESSAGE_ID_DESTROY_PROTECTED_RESOURCE_SESSION = 0x434
    D3D12_MESSAGE_ID_PROTECTED_RESOURCE_SESSION_UNSUPPORTED = 0x435
    D3D12_MESSAGE_ID_FENCE_INVALIDOPERATION = 0x436
    D3D12_MESSAGE_ID_CREATEQUERY_HEAP_COPY_QUEUE_TIMESTAMPS_NOT_SUPPORTED = 0x437
    D3D12_MESSAGE_ID_SAMPLEPOSITIONS_MISMATCH_DEFERRED = 0x438
    D3D12_MESSAGE_ID_SAMPLEPOSITIONS_MISMATCH_RECORDTIME_ASSUMEDFROMFIRSTUSE = 0x439
    D3D12_MESSAGE_ID_SAMPLEPOSITIONS_MISMATCH_RECORDTIME_ASSUMEDFROMCLEAR = 0x43A
    D3D12_MESSAGE_ID_CREATE_VIDEODECODERHEAP = 0x43B
    D3D12_MESSAGE_ID_LIVE_VIDEODECODERHEAP = 0x43C
    D3D12_MESSAGE_ID_DESTROY_VIDEODECODERHEAP = 0x43D
    D3D12_MESSAGE_ID_OPENEXISTINGHEAP_INVALIDARG_RETURN = 0x43E
    D3D12_MESSAGE_ID_OPENEXISTINGHEAP_OUTOFMEMORY_RETURN = 0x43F
    D3D12_MESSAGE_ID_OPENEXISTINGHEAP_INVALIDADDRESS = 0x440
    D3D12_MESSAGE_ID_OPENEXISTINGHEAP_INVALIDHANDLE = 0x441
    D3D12_MESSAGE_ID_WRITEBUFFERIMMEDIATE_INVALID_DEST = 0x442
    D3D12_MESSAGE_ID_WRITEBUFFERIMMEDIATE_INVALID_MODE = 0x443
    D3D12_MESSAGE_ID_WRITEBUFFERIMMEDIATE_INVALID_ALIGNMENT = 0x444
    D3D12_MESSAGE_ID_WRITEBUFFERIMMEDIATE_NOT_SUPPORTED = 0x445
    D3D12_MESSAGE_ID_SETVIEWINSTANCEMASK_INVALIDARGS = 0x446
    D3D12_MESSAGE_ID_VIEW_INSTANCING_UNSUPPORTED = 0x447
    D3D12_MESSAGE_ID_VIEW_INSTANCING_INVALIDARGS = 0x448
    D3D12_MESSAGE_ID_COPYTEXTUREREGION_MISMATCH_DECODE_REFERENCE_ONLY_FLAG = 0x449
    D3D12_MESSAGE_ID_COPYRESOURCE_MISMATCH_DECODE_REFERENCE_ONLY_FLAG = 0x44A
    D3D12_MESSAGE_ID_CREATE_VIDEO_DECODE_HEAP_CAPS_FAILURE = 0x44B
    D3D12_MESSAGE_ID_CREATE_VIDEO_DECODE_HEAP_CAPS_UNSUPPORTED = 0x44C
    D3D12_MESSAGE_ID_VIDEO_DECODE_SUPPORT_INVALID_INPUT = 0x44D
    D3D12_MESSAGE_ID_CREATE_VIDEO_DECODER_UNSUPPORTED = 0x44E
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_METADATA_ERROR = 0x44F
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_VIEW_INSTANCING_VERTEX_SIZE_EXCEEDED = 0x450
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_RUNTIME_INTERNAL_ERROR = 0x451
    D3D12_MESSAGE_ID_NO_VIDEO_API_SUPPORT = 0x452
    D3D12_MESSAGE_ID_VIDEO_PROCESS_SUPPORT_INVALID_INPUT = 0x453
    D3D12_MESSAGE_ID_CREATE_VIDEO_PROCESSOR_CAPS_FAILURE = 0x454
    D3D12_MESSAGE_ID_VIDEO_PROCESS_SUPPORT_UNSUPPORTED_FORMAT = 0x455
    D3D12_MESSAGE_ID_VIDEO_DECODE_FRAME_INVALID_ARGUMENT = 0x456
    D3D12_MESSAGE_ID_ENQUEUE_MAKE_RESIDENT_INVALID_FLAGS = 0x457
    D3D12_MESSAGE_ID_OPENEXISTINGHEAP_UNSUPPORTED = 0x458
    D3D12_MESSAGE_ID_VIDEO_PROCESS_FRAMES_INVALID_ARGUMENT = 0x459
    D3D12_MESSAGE_ID_VIDEO_DECODE_SUPPORT_UNSUPPORTED = 0x45A
    D3D12_MESSAGE_ID_CREATE_COMMANDRECORDER = 0x45B
    D3D12_MESSAGE_ID_LIVE_COMMANDRECORDER = 0x45C
    D3D12_MESSAGE_ID_DESTROY_COMMANDRECORDER = 0x45D
    D3D12_MESSAGE_ID_CREATE_COMMAND_RECORDER_VIDEO_NOT_SUPPORTED = 0x45E
    D3D12_MESSAGE_ID_CREATE_COMMAND_RECORDER_INVALID_SUPPORT_FLAGS = 0x45F
    D3D12_MESSAGE_ID_CREATE_COMMAND_RECORDER_INVALID_FLAGS = 0x460
    D3D12_MESSAGE_ID_CREATE_COMMAND_RECORDER_MORE_RECORDERS_THAN_LOGICAL_PROCESSORS = 0x461
    D3D12_MESSAGE_ID_CREATE_COMMANDPOOL = 0x462
    D3D12_MESSAGE_ID_LIVE_COMMANDPOOL = 0x463
    D3D12_MESSAGE_ID_DESTROY_COMMANDPOOL = 0x464
    D3D12_MESSAGE_ID_CREATE_COMMAND_POOL_INVALID_FLAGS = 0x465
    D3D12_MESSAGE_ID_CREATE_COMMAND_LIST_VIDEO_NOT_SUPPORTED = 0x466
    D3D12_MESSAGE_ID_COMMAND_RECORDER_SUPPORT_FLAGS_MISMATCH = 0x467
    D3D12_MESSAGE_ID_COMMAND_RECORDER_CONTENTION = 0x468
    D3D12_MESSAGE_ID_COMMAND_RECORDER_USAGE_WITH_CREATECOMMANDLIST_COMMAND_LIST = 0x469
    D3D12_MESSAGE_ID_COMMAND_ALLOCATOR_USAGE_WITH_CREATECOMMANDLIST1_COMMAND_LIST = 0x46A
    D3D12_MESSAGE_ID_CANNOT_EXECUTE_EMPTY_COMMAND_LIST = 0x46B
    D3D12_MESSAGE_ID_CANNOT_RESET_COMMAND_POOL_WITH_OPEN_COMMAND_LISTS = 0x46C
    D3D12_MESSAGE_ID_CANNOT_USE_COMMAND_RECORDER_WITHOUT_CURRENT_TARGET = 0x46D
    D3D12_MESSAGE_ID_CANNOT_CHANGE_COMMAND_RECORDER_TARGET_WHILE_RECORDING = 0x46E
    D3D12_MESSAGE_ID_COMMAND_POOL_SYNC = 0x46F
    D3D12_MESSAGE_ID_EVICT_UNDERFLOW = 0x470
    D3D12_MESSAGE_ID_CREATE_META_COMMAND = 0x471
    D3D12_MESSAGE_ID_LIVE_META_COMMAND = 0x472
    D3D12_MESSAGE_ID_DESTROY_META_COMMAND = 0x473
    D3D12_MESSAGE_ID_COPYBUFFERREGION_INVALID_DST_RESOURCE = 0x474
    D3D12_MESSAGE_ID_COPYBUFFERREGION_INVALID_SRC_RESOURCE = 0x475
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_INVALID_DST_RESOURCE = 0x476
    D3D12_MESSAGE_ID_ATOMICCOPYBUFFER_INVALID_SRC_RESOURCE = 0x477
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_NULL_BUFFER = 0x478
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_NULL_RESOURCE_DESC = 0x479
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_UNSUPPORTED = 0x47A
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_INVALID_BUFFER_DIMENSION = 0x47B
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_INVALID_BUFFER_FLAGS = 0x47C
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_INVALID_BUFFER_OFFSET = 0x47D
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_INVALID_RESOURCE_DIMENSION = 0x47E
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_INVALID_RESOURCE_FLAGS = 0x47F
    D3D12_MESSAGE_ID_CREATEPLACEDRESOURCEONBUFFER_OUTOFMEMORY_RETURN = 0x480
    D3D12_MESSAGE_ID_CANNOT_CREATE_GRAPHICS_AND_VIDEO_COMMAND_RECORDER = 0x481
    D3D12_MESSAGE_ID_UPDATETILEMAPPINGS_POSSIBLY_MISMATCHING_PROPERTIES = 0x482
    D3D12_MESSAGE_ID_CREATE_COMMAND_LIST_INVALID_COMMAND_LIST_TYPE = 0x483
    D3D12_MESSAGE_ID_CLEARUNORDEREDACCESSVIEW_INCOMPATIBLE_WITH_STRUCTURED_BUFFERS = 0x484
    D3D12_MESSAGE_ID_COMPUTE_ONLY_DEVICE_OPERATION_UNSUPPORTED = 0x485
    D3D12_MESSAGE_ID_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_INVALID = 0x486
    D3D12_MESSAGE_ID_EMIT_RAYTRACING_ACCELERATION_STRUCTURE_POSTBUILD_INFO_INVALID = 0x487
    D3D12_MESSAGE_ID_COPY_RAYTRACING_ACCELERATION_STRUCTURE_INVALID = 0x488
    D3D12_MESSAGE_ID_DISPATCH_RAYS_INVALID = 0x489
    D3D12_MESSAGE_ID_GET_RAYTRACING_ACCELERATION_STRUCTURE_PREBUILD_INFO_INVALID = 0x48A
    D3D12_MESSAGE_ID_CREATE_LIFETIMETRACKER = 0x48B
    D3D12_MESSAGE_ID_LIVE_LIFETIMETRACKER = 0x48C
    D3D12_MESSAGE_ID_DESTROY_LIFETIMETRACKER = 0x48D
    D3D12_MESSAGE_ID_DESTROYOWNEDOBJECT_OBJECTNOTOWNED = 0x48E
    D3D12_MESSAGE_ID_CREATE_TRACKEDWORKLOAD = 0x48F
    D3D12_MESSAGE_ID_LIVE_TRACKEDWORKLOAD = 0x490
    D3D12_MESSAGE_ID_DESTROY_TRACKEDWORKLOAD = 0x491
    D3D12_MESSAGE_ID_RENDER_PASS_ERROR = 0x492
    D3D12_MESSAGE_ID_META_COMMAND_ID_INVALID = 0x493
    D3D12_MESSAGE_ID_META_COMMAND_UNSUPPORTED_PARAMS = 0x494
    D3D12_MESSAGE_ID_META_COMMAND_FAILED_ENUMERATION = 0x495
    D3D12_MESSAGE_ID_META_COMMAND_PARAMETER_SIZE_MISMATCH = 0x496
    D3D12_MESSAGE_ID_UNINITIALIZED_META_COMMAND = 0x497
    D3D12_MESSAGE_ID_META_COMMAND_INVALID_GPU_VIRTUAL_ADDRESS = 0x498
    D3D12_MESSAGE_ID_CREATE_VIDEOENCODECOMMANDLIST = 0x499
    D3D12_MESSAGE_ID_LIVE_VIDEOENCODECOMMANDLIST = 0x49A
    D3D12_MESSAGE_ID_DESTROY_VIDEOENCODECOMMANDLIST = 0x49B
    D3D12_MESSAGE_ID_CREATE_VIDEOENCODECOMMANDQUEUE = 0x49C
    D3D12_MESSAGE_ID_LIVE_VIDEOENCODECOMMANDQUEUE = 0x49D
    D3D12_MESSAGE_ID_DESTROY_VIDEOENCODECOMMANDQUEUE = 0x49E
    D3D12_MESSAGE_ID_CREATE_VIDEOMOTIONESTIMATOR = 0x49F
    D3D12_MESSAGE_ID_LIVE_VIDEOMOTIONESTIMATOR = 0x4A0
    D3D12_MESSAGE_ID_DESTROY_VIDEOMOTIONESTIMATOR = 0x4A1
    D3D12_MESSAGE_ID_CREATE_VIDEOMOTIONVECTORHEAP = 0x4A2
    D3D12_MESSAGE_ID_LIVE_VIDEOMOTIONVECTORHEAP = 0x4A3
    D3D12_MESSAGE_ID_DESTROY_VIDEOMOTIONVECTORHEAP = 0x4A4
    D3D12_MESSAGE_ID_MULTIPLE_TRACKED_WORKLOADS = 0x4A5
    D3D12_MESSAGE_ID_MULTIPLE_TRACKED_WORKLOAD_PAIRS = 0x4A6
    D3D12_MESSAGE_ID_OUT_OF_ORDER_TRACKED_WORKLOAD_PAIR = 0x4A7
    D3D12_MESSAGE_ID_CANNOT_ADD_TRACKED_WORKLOAD = 0x4A8
    D3D12_MESSAGE_ID_INCOMPLETE_TRACKED_WORKLOAD_PAIR = 0x4A9
    D3D12_MESSAGE_ID_CREATE_STATE_OBJECT_ERROR = 0x4AA
    D3D12_MESSAGE_ID_GET_SHADER_IDENTIFIER_ERROR = 0x4AB
    D3D12_MESSAGE_ID_GET_SHADER_STACK_SIZE_ERROR = 0x4AC
    D3D12_MESSAGE_ID_GET_PIPELINE_STACK_SIZE_ERROR = 0x4AD
    D3D12_MESSAGE_ID_SET_PIPELINE_STACK_SIZE_ERROR = 0x4AE
    D3D12_MESSAGE_ID_GET_SHADER_IDENTIFIER_SIZE_INVALID = 0x4AF
    D3D12_MESSAGE_ID_CHECK_DRIVER_MATCHING_IDENTIFIER_INVALID = 0x4B0
    D3D12_MESSAGE_ID_CHECK_DRIVER_MATCHING_IDENTIFIER_DRIVER_REPORTED_ISSUE = 0x4B1
    D3D12_MESSAGE_ID_RENDER_PASS_INVALID_RESOURCE_BARRIER = 0x4B2
    D3D12_MESSAGE_ID_RENDER_PASS_DISALLOWED_API_CALLED = 0x4B3
    D3D12_MESSAGE_ID_RENDER_PASS_CANNOT_NEST_RENDER_PASSES = 0x4B4
    D3D12_MESSAGE_ID_RENDER_PASS_CANNOT_END_WITHOUT_BEGIN = 0x4B5
    D3D12_MESSAGE_ID_RENDER_PASS_CANNOT_CLOSE_COMMAND_LIST = 0x4B6
    D3D12_MESSAGE_ID_RENDER_PASS_GPU_WORK_WHILE_SUSPENDED = 0x4B7
    D3D12_MESSAGE_ID_RENDER_PASS_MISMATCHING_SUSPEND_RESUME = 0x4B8
    D3D12_MESSAGE_ID_RENDER_PASS_NO_PRIOR_SUSPEND_WITHIN_EXECUTECOMMANDLISTS = 0x4B9
    D3D12_MESSAGE_ID_RENDER_PASS_NO_SUBSEQUENT_RESUME_WITHIN_EXECUTECOMMANDLISTS = 0x4BA
    D3D12_MESSAGE_ID_TRACKED_WORKLOAD_COMMAND_QUEUE_MISMATCH = 0x4BB
    D3D12_MESSAGE_ID_TRACKED_WORKLOAD_NOT_SUPPORTED = 0x4BC
    D3D12_MESSAGE_ID_RENDER_PASS_MISMATCHING_NO_ACCESS = 0x4BD
    D3D12_MESSAGE_ID_RENDER_PASS_UNSUPPORTED_RESOLVE = 0x4BE
    D3D12_MESSAGE_ID_CLEARUNORDEREDACCESSVIEW_INVALID_RESOURCE_PTR = 0x4BF
    D3D12_MESSAGE_ID_WINDOWS7_FENCE_OUTOFORDER_SIGNAL = 0x4C0
    D3D12_MESSAGE_ID_WINDOWS7_FENCE_OUTOFORDER_WAIT = 0x4C1
    D3D12_MESSAGE_ID_VIDEO_CREATE_MOTION_ESTIMATOR_INVALID_ARGUMENT = 0x4C2
    D3D12_MESSAGE_ID_VIDEO_CREATE_MOTION_VECTOR_HEAP_INVALID_ARGUMENT = 0x4C3
    D3D12_MESSAGE_ID_ESTIMATE_MOTION_INVALID_ARGUMENT = 0x4C4
    D3D12_MESSAGE_ID_RESOLVE_MOTION_VECTOR_HEAP_INVALID_ARGUMENT = 0x4C5
    D3D12_MESSAGE_ID_GETGPUVIRTUALADDRESS_INVALID_HEAP_TYPE = 0x4C6
    D3D12_MESSAGE_ID_SET_BACKGROUND_PROCESSING_MODE_INVALID_ARGUMENT = 0x4C7
    D3D12_MESSAGE_ID_CREATE_COMMAND_LIST_INVALID_COMMAND_LIST_TYPE_FOR_FEATURE_LEVEL = 0x4C8
    D3D12_MESSAGE_ID_CREATE_VIDEOEXTENSIONCOMMAND = 0x4C9
    D3D12_MESSAGE_ID_LIVE_VIDEOEXTENSIONCOMMAND = 0x4CA
    D3D12_MESSAGE_ID_DESTROY_VIDEOEXTENSIONCOMMAND = 0x4CB
    D3D12_MESSAGE_ID_INVALID_VIDEO_EXTENSION_COMMAND_ID = 0x4CC
    D3D12_MESSAGE_ID_VIDEO_EXTENSION_COMMAND_INVALID_ARGUMENT = 0x4CD
    D3D12_MESSAGE_ID_CREATE_ROOT_SIGNATURE_NOT_UNIQUE_IN_DXIL_LIBRARY = 0x4CE
    D3D12_MESSAGE_ID_VARIABLE_SHADING_RATE_NOT_ALLOWED_WITH_TIR = 0x4CF
    D3D12_MESSAGE_ID_GEOMETRY_SHADER_OUTPUTTING_BOTH_VIEWPORT_ARRAY_INDEX_AND_SHADING_RATE_NOT_SUPPORTED_ON_DEVICE = 0x4D0
    D3D12_MESSAGE_ID_RSSETSHADING_RATE_INVALID_SHADING_RATE = 0x4D1
    D3D12_MESSAGE_ID_RSSETSHADING_RATE_SHADING_RATE_NOT_PERMITTED_BY_CAP = 0x4D2
    D3D12_MESSAGE_ID_RSSETSHADING_RATE_INVALID_COMBINER = 0x4D3
    D3D12_MESSAGE_ID_RSSETSHADINGRATEIMAGE_REQUIRES_TIER_2 = 0x4D4
    D3D12_MESSAGE_ID_RSSETSHADINGRATE_REQUIRES_TIER_1 = 0x4D5
    D3D12_MESSAGE_ID_SHADING_RATE_IMAGE_INCORRECT_FORMAT = 0x4D6
    D3D12_MESSAGE_ID_SHADING_RATE_IMAGE_INCORRECT_ARRAY_SIZE = 0x4D7
    D3D12_MESSAGE_ID_SHADING_RATE_IMAGE_INCORRECT_MIP_LEVEL = 0x4D8
    D3D12_MESSAGE_ID_SHADING_RATE_IMAGE_INCORRECT_SAMPLE_COUNT = 0x4D9
    D3D12_MESSAGE_ID_SHADING_RATE_IMAGE_INCORRECT_SAMPLE_QUALITY = 0x4DA
    D3D12_MESSAGE_ID_NON_RETAIL_SHADER_MODEL_WONT_VALIDATE = 0x4DB
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_AS_ROOT_SIGNATURE_MISMATCH = 0x4DC
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_MS_ROOT_SIGNATURE_MISMATCH = 0x4DD
    D3D12_MESSAGE_ID_ADD_TO_STATE_OBJECT_ERROR = 0x4DE
    D3D12_MESSAGE_ID_CREATE_PROTECTED_RESOURCE_SESSION_INVALID_ARGUMENT = 0x4DF
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_MS_PSO_DESC_MISMATCH = 0x4E0
    D3D12_MESSAGE_ID_CREATEPIPELINESTATE_MS_INCOMPLETE_TYPE = 0x4E1
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_AS_NOT_MS_MISMATCH = 0x4E2
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_MS_NOT_PS_MISMATCH = 0x4E3
    D3D12_MESSAGE_ID_NONZERO_SAMPLER_FEEDBACK_MIP_REGION_WITH_INCOMPATIBLE_FORMAT = 0x4E4
    D3D12_MESSAGE_ID_CREATEGRAPHICSPIPELINESTATE_INPUTLAYOUT_SHADER_MISMATCH = 0x4E5
    D3D12_MESSAGE_ID_EMPTY_DISPATCH = 0x4E6
    D3D12_MESSAGE_ID_RESOURCE_FORMAT_REQUIRES_SAMPLER_FEEDBACK_CAPABILITY = 0x4E7
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_MAP_INVALID_MIP_REGION = 0x4E8
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_MAP_INVALID_DIMENSION = 0x4E9
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_MAP_INVALID_SAMPLE_COUNT = 0x4EA
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_MAP_INVALID_SAMPLE_QUALITY = 0x4EB
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_MAP_INVALID_LAYOUT = 0x4EC
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_MAP_REQUIRES_UNORDERED_ACCESS_FLAG = 0x4ED
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_CREATE_UAV_NULL_ARGUMENTS = 0x4EE
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_UAV_REQUIRES_SAMPLER_FEEDBACK_CAPABILITY = 0x4EF
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_CREATE_UAV_REQUIRES_FEEDBACK_MAP_FORMAT = 0x4F0
    D3D12_MESSAGE_ID_CREATEMESHSHADER_INVALIDSHADERBYTECODE = 0x4F1
    D3D12_MESSAGE_ID_CREATEMESHSHADER_OUTOFMEMORY = 0x4F2
    D3D12_MESSAGE_ID_CREATEMESHSHADERWITHSTREAMOUTPUT_INVALIDSHADERTYPE = 0x4F3
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_SAMPLER_FEEDBACK_TRANSCODE_INVALID_FORMAT = 0x4F4
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_SAMPLER_FEEDBACK_INVALID_MIP_LEVEL_COUNT = 0x4F5
    D3D12_MESSAGE_ID_RESOLVESUBRESOURCE_SAMPLER_FEEDBACK_TRANSCODE_ARRAY_SIZE_MISMATCH = 0x4F6
    D3D12_MESSAGE_ID_SAMPLER_FEEDBACK_CREATE_UAV_MISMATCHING_TARGETED_RESOURCE = 0x4F7
    D3D12_MESSAGE_ID_CREATEMESHSHADER_OUTPUTEXCEEDSMAXSIZE = 0x4F8
    D3D12_MESSAGE_ID_CREATEMESHSHADER_GROUPSHAREDEXCEEDSMAXSIZE = 0x4F9
    D3D12_MESSAGE_ID_VERTEX_SHADER_OUTPUTTING_BOTH_VIEWPORT_ARRAY_INDEX_AND_SHADING_RATE_NOT_SUPPORTED_ON_DEVICE = 0x4FA
    D3D12_MESSAGE_ID_MESH_SHADER_OUTPUTTING_BOTH_VIEWPORT_ARRAY_INDEX_AND_SHADING_RATE_NOT_SUPPORTED_ON_DEVICE = 0x4FB
    D3D12_MESSAGE_ID_CREATEMESHSHADER_MISMATCHEDASMSPAYLOADSIZE = 0x4FC
    D3D12_MESSAGE_ID_CREATE_ROOT_SIGNATURE_UNBOUNDED_STATIC_DESCRIPTORS = 0x4FD
    D3D12_MESSAGE_ID_CREATEAMPLIFICATIONSHADER_INVALIDSHADERBYTECODE = 0x4FE
    D3D12_MESSAGE_ID_CREATEAMPLIFICATIONSHADER_OUTOFMEMORY = 0x4FF
    D3D12_MESSAGE_ID_D3D12_MESSAGES_END = 0x500

class D3D11_SHADER_TRACKING_OPTION(IntEnum):
    D3D11_SHADER_TRACKING_OPTION_IGNORE = 0x0
    D3D11_SHADER_TRACKING_OPTION_TRACK_UNINITIALIZED = 0x1
    D3D11_SHADER_TRACKING_OPTION_TRACK_RAW = 0x2
    D3D11_SHADER_TRACKING_OPTION_TRACK_WAR = 0x4
    D3D11_SHADER_TRACKING_OPTION_TRACK_WAW = 0x8
    D3D11_SHADER_TRACKING_OPTION_ALLOW_SAME = 0x10
    D3D11_SHADER_TRACKING_OPTION_TRACK_ATOMIC_CONSISTENCY = 0x20
    D3D11_SHADER_TRACKING_OPTION_TRACK_RAW_ACROSS_THREADGROUPS = 0x40
    D3D11_SHADER_TRACKING_OPTION_TRACK_WAR_ACROSS_THREADGROUPS = 0x80
    D3D11_SHADER_TRACKING_OPTION_TRACK_WAW_ACROSS_THREADGROUPS = 0x100
    D3D11_SHADER_TRACKING_OPTION_TRACK_ATOMIC_CONSISTENCY_ACROSS_THREADGROUPS = 0x200
    D3D11_SHADER_TRACKING_OPTION_UAV_SPECIFIC_FLAGS = 0x3C0
    D3D11_SHADER_TRACKING_OPTION_ALL_HAZARDS = 0x3EE
    D3D11_SHADER_TRACKING_OPTION_ALL_HAZARDS_ALLOWING_SAME = 0x3FE
    D3D11_SHADER_TRACKING_OPTION_ALL_OPTIONS = 0x3FF

class D3D11_MESSAGE_CATEGORY(IntEnum):
    D3D11_MESSAGE_CATEGORY_APPLICATION_DEFINED = 0x0
    D3D11_MESSAGE_CATEGORY_MISCELLANEOUS = 0x1
    D3D11_MESSAGE_CATEGORY_INITIALIZATION = 0x2
    D3D11_MESSAGE_CATEGORY_CLEANUP = 0x3
    D3D11_MESSAGE_CATEGORY_COMPILATION = 0x4
    D3D11_MESSAGE_CATEGORY_STATE_CREATION = 0x5
    D3D11_MESSAGE_CATEGORY_STATE_SETTING = 0x6
    D3D11_MESSAGE_CATEGORY_STATE_GETTING = 0x7
    D3D11_MESSAGE_CATEGORY_RESOURCE_MANIPULATION = 0x8
    D3D11_MESSAGE_CATEGORY_EXECUTION = 0x9
    D3D11_MESSAGE_CATEGORY_SHADER = 0xA

class D3D11_MESSAGE_SEVERITY(IntEnum):
    D3D11_MESSAGE_SEVERITY_CORRUPTION = 0x0
    D3D11_MESSAGE_SEVERITY_ERROR = 0x1
    D3D11_MESSAGE_SEVERITY_WARNING = 0x2
    D3D11_MESSAGE_SEVERITY_INFO = 0x3
    D3D11_MESSAGE_SEVERITY_MESSAGE = 0x4

class D3D11_MESSAGE_ID(IntEnum):
    D3D11_MESSAGE_ID_UNKNOWN = 0x0
    D3D11_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_HAZARD = 0x1
    D3D11_MESSAGE_ID_DEVICE_IASETINDEXBUFFER_HAZARD = 0x2
    D3D11_MESSAGE_ID_DEVICE_VSSETSHADERRESOURCES_HAZARD = 0x3
    D3D11_MESSAGE_ID_DEVICE_VSSETCONSTANTBUFFERS_HAZARD = 0x4
    D3D11_MESSAGE_ID_DEVICE_GSSETSHADERRESOURCES_HAZARD = 0x5
    D3D11_MESSAGE_ID_DEVICE_GSSETCONSTANTBUFFERS_HAZARD = 0x6
    D3D11_MESSAGE_ID_DEVICE_PSSETSHADERRESOURCES_HAZARD = 0x7
    D3D11_MESSAGE_ID_DEVICE_PSSETCONSTANTBUFFERS_HAZARD = 0x8
    D3D11_MESSAGE_ID_DEVICE_OMSETRENDERTARGETS_HAZARD = 0x9
    D3D11_MESSAGE_ID_DEVICE_SOSETTARGETS_HAZARD = 0xA
    D3D11_MESSAGE_ID_STRING_FROM_APPLICATION = 0xB
    D3D11_MESSAGE_ID_CORRUPTED_THIS = 0xC
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER1 = 0xD
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER2 = 0xE
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER3 = 0xF
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER4 = 0x10
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER5 = 0x11
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER6 = 0x12
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER7 = 0x13
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER8 = 0x14
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER9 = 0x15
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER10 = 0x16
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER11 = 0x17
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER12 = 0x18
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER13 = 0x19
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER14 = 0x1A
    D3D11_MESSAGE_ID_CORRUPTED_PARAMETER15 = 0x1B
    D3D11_MESSAGE_ID_CORRUPTED_MULTITHREADING = 0x1C
    D3D11_MESSAGE_ID_MESSAGE_REPORTING_OUTOFMEMORY = 0x1D
    D3D11_MESSAGE_ID_IASETINPUTLAYOUT_UNBINDDELETINGOBJECT = 0x1E
    D3D11_MESSAGE_ID_IASETVERTEXBUFFERS_UNBINDDELETINGOBJECT = 0x1F
    D3D11_MESSAGE_ID_IASETINDEXBUFFER_UNBINDDELETINGOBJECT = 0x20
    D3D11_MESSAGE_ID_VSSETSHADER_UNBINDDELETINGOBJECT = 0x21
    D3D11_MESSAGE_ID_VSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x22
    D3D11_MESSAGE_ID_VSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x23
    D3D11_MESSAGE_ID_VSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x24
    D3D11_MESSAGE_ID_GSSETSHADER_UNBINDDELETINGOBJECT = 0x25
    D3D11_MESSAGE_ID_GSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x26
    D3D11_MESSAGE_ID_GSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x27
    D3D11_MESSAGE_ID_GSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x28
    D3D11_MESSAGE_ID_SOSETTARGETS_UNBINDDELETINGOBJECT = 0x29
    D3D11_MESSAGE_ID_PSSETSHADER_UNBINDDELETINGOBJECT = 0x2A
    D3D11_MESSAGE_ID_PSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x2B
    D3D11_MESSAGE_ID_PSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x2C
    D3D11_MESSAGE_ID_PSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x2D
    D3D11_MESSAGE_ID_RSSETSTATE_UNBINDDELETINGOBJECT = 0x2E
    D3D11_MESSAGE_ID_OMSETBLENDSTATE_UNBINDDELETINGOBJECT = 0x2F
    D3D11_MESSAGE_ID_OMSETDEPTHSTENCILSTATE_UNBINDDELETINGOBJECT = 0x30
    D3D11_MESSAGE_ID_OMSETRENDERTARGETS_UNBINDDELETINGOBJECT = 0x31
    D3D11_MESSAGE_ID_SETPREDICATION_UNBINDDELETINGOBJECT = 0x32
    D3D11_MESSAGE_ID_GETPRIVATEDATA_MOREDATA = 0x33
    D3D11_MESSAGE_ID_SETPRIVATEDATA_INVALIDFREEDATA = 0x34
    D3D11_MESSAGE_ID_SETPRIVATEDATA_INVALIDIUNKNOWN = 0x35
    D3D11_MESSAGE_ID_SETPRIVATEDATA_INVALIDFLAGS = 0x36
    D3D11_MESSAGE_ID_SETPRIVATEDATA_CHANGINGPARAMS = 0x37
    D3D11_MESSAGE_ID_SETPRIVATEDATA_OUTOFMEMORY = 0x38
    D3D11_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDFORMAT = 0x39
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDSAMPLES = 0x3A
    D3D11_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDUSAGE = 0x3B
    D3D11_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDBINDFLAGS = 0x3C
    D3D11_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDCPUACCESSFLAGS = 0x3D
    D3D11_MESSAGE_ID_CREATEBUFFER_UNRECOGNIZEDMISCFLAGS = 0x3E
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDCPUACCESSFLAGS = 0x3F
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDBINDFLAGS = 0x40
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDINITIALDATA = 0x41
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDDIMENSIONS = 0x42
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDMIPLEVELS = 0x43
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDMISCFLAGS = 0x44
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDARG_RETURN = 0x45
    D3D11_MESSAGE_ID_CREATEBUFFER_OUTOFMEMORY_RETURN = 0x46
    D3D11_MESSAGE_ID_CREATEBUFFER_NULLDESC = 0x47
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDCONSTANTBUFFERBINDINGS = 0x48
    D3D11_MESSAGE_ID_CREATEBUFFER_LARGEALLOCATION = 0x49
    D3D11_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDFORMAT = 0x4A
    D3D11_MESSAGE_ID_CREATETEXTURE1D_UNSUPPORTEDFORMAT = 0x4B
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDSAMPLES = 0x4C
    D3D11_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDUSAGE = 0x4D
    D3D11_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDBINDFLAGS = 0x4E
    D3D11_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDCPUACCESSFLAGS = 0x4F
    D3D11_MESSAGE_ID_CREATETEXTURE1D_UNRECOGNIZEDMISCFLAGS = 0x50
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDCPUACCESSFLAGS = 0x51
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDBINDFLAGS = 0x52
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDINITIALDATA = 0x53
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDDIMENSIONS = 0x54
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDMIPLEVELS = 0x55
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDMISCFLAGS = 0x56
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDARG_RETURN = 0x57
    D3D11_MESSAGE_ID_CREATETEXTURE1D_OUTOFMEMORY_RETURN = 0x58
    D3D11_MESSAGE_ID_CREATETEXTURE1D_NULLDESC = 0x59
    D3D11_MESSAGE_ID_CREATETEXTURE1D_LARGEALLOCATION = 0x5A
    D3D11_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDFORMAT = 0x5B
    D3D11_MESSAGE_ID_CREATETEXTURE2D_UNSUPPORTEDFORMAT = 0x5C
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDSAMPLES = 0x5D
    D3D11_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDUSAGE = 0x5E
    D3D11_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDBINDFLAGS = 0x5F
    D3D11_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDCPUACCESSFLAGS = 0x60
    D3D11_MESSAGE_ID_CREATETEXTURE2D_UNRECOGNIZEDMISCFLAGS = 0x61
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDCPUACCESSFLAGS = 0x62
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDBINDFLAGS = 0x63
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDINITIALDATA = 0x64
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDDIMENSIONS = 0x65
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDMIPLEVELS = 0x66
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDMISCFLAGS = 0x67
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDARG_RETURN = 0x68
    D3D11_MESSAGE_ID_CREATETEXTURE2D_OUTOFMEMORY_RETURN = 0x69
    D3D11_MESSAGE_ID_CREATETEXTURE2D_NULLDESC = 0x6A
    D3D11_MESSAGE_ID_CREATETEXTURE2D_LARGEALLOCATION = 0x6B
    D3D11_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDFORMAT = 0x6C
    D3D11_MESSAGE_ID_CREATETEXTURE3D_UNSUPPORTEDFORMAT = 0x6D
    D3D11_MESSAGE_ID_CREATETEXTURE3D_INVALIDSAMPLES = 0x6E
    D3D11_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDUSAGE = 0x6F
    D3D11_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDBINDFLAGS = 0x70
    D3D11_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDCPUACCESSFLAGS = 0x71
    D3D11_MESSAGE_ID_CREATETEXTURE3D_UNRECOGNIZEDMISCFLAGS = 0x72
    D3D11_MESSAGE_ID_CREATETEXTURE3D_INVALIDCPUACCESSFLAGS = 0x73
    D3D11_MESSAGE_ID_CREATETEXTURE3D_INVALIDBINDFLAGS = 0x74
    D3D11_MESSAGE_ID_CREATETEXTURE3D_INVALIDINITIALDATA = 0x75
    D3D11_MESSAGE_ID_CREATETEXTURE3D_INVALIDDIMENSIONS = 0x76
    D3D11_MESSAGE_ID_CREATETEXTURE3D_INVALIDMIPLEVELS = 0x77
    D3D11_MESSAGE_ID_CREATETEXTURE3D_INVALIDMISCFLAGS = 0x78
    D3D11_MESSAGE_ID_CREATETEXTURE3D_INVALIDARG_RETURN = 0x79
    D3D11_MESSAGE_ID_CREATETEXTURE3D_OUTOFMEMORY_RETURN = 0x7A
    D3D11_MESSAGE_ID_CREATETEXTURE3D_NULLDESC = 0x7B
    D3D11_MESSAGE_ID_CREATETEXTURE3D_LARGEALLOCATION = 0x7C
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_UNRECOGNIZEDFORMAT = 0x7D
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDDESC = 0x7E
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDFORMAT = 0x7F
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDDIMENSIONS = 0x80
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDRESOURCE = 0x81
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_TOOMANYOBJECTS = 0x82
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDARG_RETURN = 0x83
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_OUTOFMEMORY_RETURN = 0x84
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_UNRECOGNIZEDFORMAT = 0x85
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_UNSUPPORTEDFORMAT = 0x86
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDDESC = 0x87
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDFORMAT = 0x88
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDDIMENSIONS = 0x89
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDRESOURCE = 0x8A
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_TOOMANYOBJECTS = 0x8B
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDARG_RETURN = 0x8C
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_OUTOFMEMORY_RETURN = 0x8D
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_UNRECOGNIZEDFORMAT = 0x8E
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDDESC = 0x8F
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDFORMAT = 0x90
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDDIMENSIONS = 0x91
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDRESOURCE = 0x92
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_TOOMANYOBJECTS = 0x93
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDARG_RETURN = 0x94
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_OUTOFMEMORY_RETURN = 0x95
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_OUTOFMEMORY = 0x96
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_TOOMANYELEMENTS = 0x97
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDFORMAT = 0x98
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_INCOMPATIBLEFORMAT = 0x99
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSLOT = 0x9A
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDINPUTSLOTCLASS = 0x9B
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_STEPRATESLOTCLASSMISMATCH = 0x9C
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSLOTCLASSCHANGE = 0x9D
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDSTEPRATECHANGE = 0x9E
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_INVALIDALIGNMENT = 0x9F
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_DUPLICATESEMANTIC = 0xA0
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_UNPARSEABLEINPUTSIGNATURE = 0xA1
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_NULLSEMANTIC = 0xA2
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_MISSINGELEMENT = 0xA3
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_NULLDESC = 0xA4
    D3D11_MESSAGE_ID_CREATEVERTEXSHADER_OUTOFMEMORY = 0xA5
    D3D11_MESSAGE_ID_CREATEVERTEXSHADER_INVALIDSHADERBYTECODE = 0xA6
    D3D11_MESSAGE_ID_CREATEVERTEXSHADER_INVALIDSHADERTYPE = 0xA7
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADER_OUTOFMEMORY = 0xA8
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADER_INVALIDSHADERBYTECODE = 0xA9
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADER_INVALIDSHADERTYPE = 0xAA
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTOFMEMORY = 0xAB
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSHADERBYTECODE = 0xAC
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSHADERTYPE = 0xAD
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDNUMENTRIES = 0xAE
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTPUTSTREAMSTRIDEUNUSED = 0xAF
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UNEXPECTEDDECL = 0xB0
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_EXPECTEDDECL = 0xB1
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_OUTPUTSLOT0EXPECTED = 0xB2
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDOUTPUTSLOT = 0xB3
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_ONLYONEELEMENTPERSLOT = 0xB4
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDCOMPONENTCOUNT = 0xB5
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSTARTCOMPONENTANDCOMPONENTCOUNT = 0xB6
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDGAPDEFINITION = 0xB7
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_REPEATEDOUTPUT = 0xB8
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDOUTPUTSTREAMSTRIDE = 0xB9
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MISSINGSEMANTIC = 0xBA
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MASKMISMATCH = 0xBB
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_CANTHAVEONLYGAPS = 0xBC
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_DECLTOOCOMPLEX = 0xBD
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_MISSINGOUTPUTSIGNATURE = 0xBE
    D3D11_MESSAGE_ID_CREATEPIXELSHADER_OUTOFMEMORY = 0xBF
    D3D11_MESSAGE_ID_CREATEPIXELSHADER_INVALIDSHADERBYTECODE = 0xC0
    D3D11_MESSAGE_ID_CREATEPIXELSHADER_INVALIDSHADERTYPE = 0xC1
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDFILLMODE = 0xC2
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDCULLMODE = 0xC3
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDDEPTHBIASCLAMP = 0xC4
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDSLOPESCALEDDEPTHBIAS = 0xC5
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_TOOMANYOBJECTS = 0xC6
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_NULLDESC = 0xC7
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDDEPTHWRITEMASK = 0xC8
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDDEPTHFUNC = 0xC9
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILFAILOP = 0xCA
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILZFAILOP = 0xCB
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILPASSOP = 0xCC
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDFRONTFACESTENCILFUNC = 0xCD
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILFAILOP = 0xCE
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILZFAILOP = 0xCF
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILPASSOP = 0xD0
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_INVALIDBACKFACESTENCILFUNC = 0xD1
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_TOOMANYOBJECTS = 0xD2
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_NULLDESC = 0xD3
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_INVALIDSRCBLEND = 0xD4
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_INVALIDDESTBLEND = 0xD5
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_INVALIDBLENDOP = 0xD6
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_INVALIDSRCBLENDALPHA = 0xD7
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_INVALIDDESTBLENDALPHA = 0xD8
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_INVALIDBLENDOPALPHA = 0xD9
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_INVALIDRENDERTARGETWRITEMASK = 0xDA
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_TOOMANYOBJECTS = 0xDB
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_NULLDESC = 0xDC
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDFILTER = 0xDD
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDADDRESSU = 0xDE
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDADDRESSV = 0xDF
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDADDRESSW = 0xE0
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDMIPLODBIAS = 0xE1
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDMAXANISOTROPY = 0xE2
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDCOMPARISONFUNC = 0xE3
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDMINLOD = 0xE4
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_INVALIDMAXLOD = 0xE5
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_TOOMANYOBJECTS = 0xE6
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_NULLDESC = 0xE7
    D3D11_MESSAGE_ID_CREATEQUERYORPREDICATE_INVALIDQUERY = 0xE8
    D3D11_MESSAGE_ID_CREATEQUERYORPREDICATE_INVALIDMISCFLAGS = 0xE9
    D3D11_MESSAGE_ID_CREATEQUERYORPREDICATE_UNEXPECTEDMISCFLAG = 0xEA
    D3D11_MESSAGE_ID_CREATEQUERYORPREDICATE_NULLDESC = 0xEB
    D3D11_MESSAGE_ID_DEVICE_IASETPRIMITIVETOPOLOGY_TOPOLOGY_UNRECOGNIZED = 0xEC
    D3D11_MESSAGE_ID_DEVICE_IASETPRIMITIVETOPOLOGY_TOPOLOGY_UNDEFINED = 0xED
    D3D11_MESSAGE_ID_IASETVERTEXBUFFERS_INVALIDBUFFER = 0xEE
    D3D11_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_OFFSET_TOO_LARGE = 0xEF
    D3D11_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_BUFFERS_EMPTY = 0xF0
    D3D11_MESSAGE_ID_IASETINDEXBUFFER_INVALIDBUFFER = 0xF1
    D3D11_MESSAGE_ID_DEVICE_IASETINDEXBUFFER_FORMAT_INVALID = 0xF2
    D3D11_MESSAGE_ID_DEVICE_IASETINDEXBUFFER_OFFSET_TOO_LARGE = 0xF3
    D3D11_MESSAGE_ID_DEVICE_IASETINDEXBUFFER_OFFSET_UNALIGNED = 0xF4
    D3D11_MESSAGE_ID_DEVICE_VSSETSHADERRESOURCES_VIEWS_EMPTY = 0xF5
    D3D11_MESSAGE_ID_VSSETCONSTANTBUFFERS_INVALIDBUFFER = 0xF6
    D3D11_MESSAGE_ID_DEVICE_VSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0xF7
    D3D11_MESSAGE_ID_DEVICE_VSSETSAMPLERS_SAMPLERS_EMPTY = 0xF8
    D3D11_MESSAGE_ID_DEVICE_GSSETSHADERRESOURCES_VIEWS_EMPTY = 0xF9
    D3D11_MESSAGE_ID_GSSETCONSTANTBUFFERS_INVALIDBUFFER = 0xFA
    D3D11_MESSAGE_ID_DEVICE_GSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0xFB
    D3D11_MESSAGE_ID_DEVICE_GSSETSAMPLERS_SAMPLERS_EMPTY = 0xFC
    D3D11_MESSAGE_ID_SOSETTARGETS_INVALIDBUFFER = 0xFD
    D3D11_MESSAGE_ID_DEVICE_SOSETTARGETS_OFFSET_UNALIGNED = 0xFE
    D3D11_MESSAGE_ID_DEVICE_PSSETSHADERRESOURCES_VIEWS_EMPTY = 0xFF
    D3D11_MESSAGE_ID_PSSETCONSTANTBUFFERS_INVALIDBUFFER = 0x100
    D3D11_MESSAGE_ID_DEVICE_PSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x101
    D3D11_MESSAGE_ID_DEVICE_PSSETSAMPLERS_SAMPLERS_EMPTY = 0x102
    D3D11_MESSAGE_ID_DEVICE_RSSETVIEWPORTS_INVALIDVIEWPORT = 0x103
    D3D11_MESSAGE_ID_DEVICE_RSSETSCISSORRECTS_INVALIDSCISSOR = 0x104
    D3D11_MESSAGE_ID_CLEARRENDERTARGETVIEW_DENORMFLUSH = 0x105
    D3D11_MESSAGE_ID_CLEARDEPTHSTENCILVIEW_DENORMFLUSH = 0x106
    D3D11_MESSAGE_ID_CLEARDEPTHSTENCILVIEW_INVALID = 0x107
    D3D11_MESSAGE_ID_DEVICE_IAGETVERTEXBUFFERS_BUFFERS_EMPTY = 0x108
    D3D11_MESSAGE_ID_DEVICE_VSGETSHADERRESOURCES_VIEWS_EMPTY = 0x109
    D3D11_MESSAGE_ID_DEVICE_VSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x10A
    D3D11_MESSAGE_ID_DEVICE_VSGETSAMPLERS_SAMPLERS_EMPTY = 0x10B
    D3D11_MESSAGE_ID_DEVICE_GSGETSHADERRESOURCES_VIEWS_EMPTY = 0x10C
    D3D11_MESSAGE_ID_DEVICE_GSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x10D
    D3D11_MESSAGE_ID_DEVICE_GSGETSAMPLERS_SAMPLERS_EMPTY = 0x10E
    D3D11_MESSAGE_ID_DEVICE_SOGETTARGETS_BUFFERS_EMPTY = 0x10F
    D3D11_MESSAGE_ID_DEVICE_PSGETSHADERRESOURCES_VIEWS_EMPTY = 0x110
    D3D11_MESSAGE_ID_DEVICE_PSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x111
    D3D11_MESSAGE_ID_DEVICE_PSGETSAMPLERS_SAMPLERS_EMPTY = 0x112
    D3D11_MESSAGE_ID_DEVICE_RSGETVIEWPORTS_VIEWPORTS_EMPTY = 0x113
    D3D11_MESSAGE_ID_DEVICE_RSGETSCISSORRECTS_RECTS_EMPTY = 0x114
    D3D11_MESSAGE_ID_DEVICE_GENERATEMIPS_RESOURCE_INVALID = 0x115
    D3D11_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDDESTINATIONSUBRESOURCE = 0x116
    D3D11_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDSOURCESUBRESOURCE = 0x117
    D3D11_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDSOURCEBOX = 0x118
    D3D11_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDSOURCE = 0x119
    D3D11_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDDESTINATIONSTATE = 0x11A
    D3D11_MESSAGE_ID_COPYSUBRESOURCEREGION_INVALIDSOURCESTATE = 0x11B
    D3D11_MESSAGE_ID_COPYRESOURCE_INVALIDSOURCE = 0x11C
    D3D11_MESSAGE_ID_COPYRESOURCE_INVALIDDESTINATIONSTATE = 0x11D
    D3D11_MESSAGE_ID_COPYRESOURCE_INVALIDSOURCESTATE = 0x11E
    D3D11_MESSAGE_ID_UPDATESUBRESOURCE_INVALIDDESTINATIONSUBRESOURCE = 0x11F
    D3D11_MESSAGE_ID_UPDATESUBRESOURCE_INVALIDDESTINATIONBOX = 0x120
    D3D11_MESSAGE_ID_UPDATESUBRESOURCE_INVALIDDESTINATIONSTATE = 0x121
    D3D11_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_DESTINATION_INVALID = 0x122
    D3D11_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_DESTINATION_SUBRESOURCE_INVALID = 0x123
    D3D11_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_SOURCE_INVALID = 0x124
    D3D11_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_SOURCE_SUBRESOURCE_INVALID = 0x125
    D3D11_MESSAGE_ID_DEVICE_RESOLVESUBRESOURCE_FORMAT_INVALID = 0x126
    D3D11_MESSAGE_ID_BUFFER_MAP_INVALIDMAPTYPE = 0x127
    D3D11_MESSAGE_ID_BUFFER_MAP_INVALIDFLAGS = 0x128
    D3D11_MESSAGE_ID_BUFFER_MAP_ALREADYMAPPED = 0x129
    D3D11_MESSAGE_ID_BUFFER_MAP_DEVICEREMOVED_RETURN = 0x12A
    D3D11_MESSAGE_ID_BUFFER_UNMAP_NOTMAPPED = 0x12B
    D3D11_MESSAGE_ID_TEXTURE1D_MAP_INVALIDMAPTYPE = 0x12C
    D3D11_MESSAGE_ID_TEXTURE1D_MAP_INVALIDSUBRESOURCE = 0x12D
    D3D11_MESSAGE_ID_TEXTURE1D_MAP_INVALIDFLAGS = 0x12E
    D3D11_MESSAGE_ID_TEXTURE1D_MAP_ALREADYMAPPED = 0x12F
    D3D11_MESSAGE_ID_TEXTURE1D_MAP_DEVICEREMOVED_RETURN = 0x130
    D3D11_MESSAGE_ID_TEXTURE1D_UNMAP_INVALIDSUBRESOURCE = 0x131
    D3D11_MESSAGE_ID_TEXTURE1D_UNMAP_NOTMAPPED = 0x132
    D3D11_MESSAGE_ID_TEXTURE2D_MAP_INVALIDMAPTYPE = 0x133
    D3D11_MESSAGE_ID_TEXTURE2D_MAP_INVALIDSUBRESOURCE = 0x134
    D3D11_MESSAGE_ID_TEXTURE2D_MAP_INVALIDFLAGS = 0x135
    D3D11_MESSAGE_ID_TEXTURE2D_MAP_ALREADYMAPPED = 0x136
    D3D11_MESSAGE_ID_TEXTURE2D_MAP_DEVICEREMOVED_RETURN = 0x137
    D3D11_MESSAGE_ID_TEXTURE2D_UNMAP_INVALIDSUBRESOURCE = 0x138
    D3D11_MESSAGE_ID_TEXTURE2D_UNMAP_NOTMAPPED = 0x139
    D3D11_MESSAGE_ID_TEXTURE3D_MAP_INVALIDMAPTYPE = 0x13A
    D3D11_MESSAGE_ID_TEXTURE3D_MAP_INVALIDSUBRESOURCE = 0x13B
    D3D11_MESSAGE_ID_TEXTURE3D_MAP_INVALIDFLAGS = 0x13C
    D3D11_MESSAGE_ID_TEXTURE3D_MAP_ALREADYMAPPED = 0x13D
    D3D11_MESSAGE_ID_TEXTURE3D_MAP_DEVICEREMOVED_RETURN = 0x13E
    D3D11_MESSAGE_ID_TEXTURE3D_UNMAP_INVALIDSUBRESOURCE = 0x13F
    D3D11_MESSAGE_ID_TEXTURE3D_UNMAP_NOTMAPPED = 0x140
    D3D11_MESSAGE_ID_CHECKFORMATSUPPORT_FORMAT_DEPRECATED = 0x141
    D3D11_MESSAGE_ID_CHECKMULTISAMPLEQUALITYLEVELS_FORMAT_DEPRECATED = 0x142
    D3D11_MESSAGE_ID_SETEXCEPTIONMODE_UNRECOGNIZEDFLAGS = 0x143
    D3D11_MESSAGE_ID_SETEXCEPTIONMODE_INVALIDARG_RETURN = 0x144
    D3D11_MESSAGE_ID_SETEXCEPTIONMODE_DEVICEREMOVED_RETURN = 0x145
    D3D11_MESSAGE_ID_REF_SIMULATING_INFINITELY_FAST_HARDWARE = 0x146
    D3D11_MESSAGE_ID_REF_THREADING_MODE = 0x147
    D3D11_MESSAGE_ID_REF_UMDRIVER_EXCEPTION = 0x148
    D3D11_MESSAGE_ID_REF_KMDRIVER_EXCEPTION = 0x149
    D3D11_MESSAGE_ID_REF_HARDWARE_EXCEPTION = 0x14A
    D3D11_MESSAGE_ID_REF_ACCESSING_INDEXABLE_TEMP_OUT_OF_RANGE = 0x14B
    D3D11_MESSAGE_ID_REF_PROBLEM_PARSING_SHADER = 0x14C
    D3D11_MESSAGE_ID_REF_OUT_OF_MEMORY = 0x14D
    D3D11_MESSAGE_ID_REF_INFO = 0x14E
    D3D11_MESSAGE_ID_DEVICE_DRAW_VERTEXPOS_OVERFLOW = 0x14F
    D3D11_MESSAGE_ID_DEVICE_DRAWINDEXED_INDEXPOS_OVERFLOW = 0x150
    D3D11_MESSAGE_ID_DEVICE_DRAWINSTANCED_VERTEXPOS_OVERFLOW = 0x151
    D3D11_MESSAGE_ID_DEVICE_DRAWINSTANCED_INSTANCEPOS_OVERFLOW = 0x152
    D3D11_MESSAGE_ID_DEVICE_DRAWINDEXEDINSTANCED_INSTANCEPOS_OVERFLOW = 0x153
    D3D11_MESSAGE_ID_DEVICE_DRAWINDEXEDINSTANCED_INDEXPOS_OVERFLOW = 0x154
    D3D11_MESSAGE_ID_DEVICE_DRAW_VERTEX_SHADER_NOT_SET = 0x155
    D3D11_MESSAGE_ID_DEVICE_SHADER_LINKAGE_SEMANTICNAME_NOT_FOUND = 0x156
    D3D11_MESSAGE_ID_DEVICE_SHADER_LINKAGE_REGISTERINDEX = 0x157
    D3D11_MESSAGE_ID_DEVICE_SHADER_LINKAGE_COMPONENTTYPE = 0x158
    D3D11_MESSAGE_ID_DEVICE_SHADER_LINKAGE_REGISTERMASK = 0x159
    D3D11_MESSAGE_ID_DEVICE_SHADER_LINKAGE_SYSTEMVALUE = 0x15A
    D3D11_MESSAGE_ID_DEVICE_SHADER_LINKAGE_NEVERWRITTEN_ALWAYSREADS = 0x15B
    D3D11_MESSAGE_ID_DEVICE_DRAW_VERTEX_BUFFER_NOT_SET = 0x15C
    D3D11_MESSAGE_ID_DEVICE_DRAW_INPUTLAYOUT_NOT_SET = 0x15D
    D3D11_MESSAGE_ID_DEVICE_DRAW_CONSTANT_BUFFER_NOT_SET = 0x15E
    D3D11_MESSAGE_ID_DEVICE_DRAW_CONSTANT_BUFFER_TOO_SMALL = 0x15F
    D3D11_MESSAGE_ID_DEVICE_DRAW_SAMPLER_NOT_SET = 0x160
    D3D11_MESSAGE_ID_DEVICE_DRAW_SHADERRESOURCEVIEW_NOT_SET = 0x161
    D3D11_MESSAGE_ID_DEVICE_DRAW_VIEW_DIMENSION_MISMATCH = 0x162
    D3D11_MESSAGE_ID_DEVICE_DRAW_VERTEX_BUFFER_STRIDE_TOO_SMALL = 0x163
    D3D11_MESSAGE_ID_DEVICE_DRAW_VERTEX_BUFFER_TOO_SMALL = 0x164
    D3D11_MESSAGE_ID_DEVICE_DRAW_INDEX_BUFFER_NOT_SET = 0x165
    D3D11_MESSAGE_ID_DEVICE_DRAW_INDEX_BUFFER_FORMAT_INVALID = 0x166
    D3D11_MESSAGE_ID_DEVICE_DRAW_INDEX_BUFFER_TOO_SMALL = 0x167
    D3D11_MESSAGE_ID_DEVICE_DRAW_GS_INPUT_PRIMITIVE_MISMATCH = 0x168
    D3D11_MESSAGE_ID_DEVICE_DRAW_RESOURCE_RETURN_TYPE_MISMATCH = 0x169
    D3D11_MESSAGE_ID_DEVICE_DRAW_POSITION_NOT_PRESENT = 0x16A
    D3D11_MESSAGE_ID_DEVICE_DRAW_OUTPUT_STREAM_NOT_SET = 0x16B
    D3D11_MESSAGE_ID_DEVICE_DRAW_BOUND_RESOURCE_MAPPED = 0x16C
    D3D11_MESSAGE_ID_DEVICE_DRAW_INVALID_PRIMITIVETOPOLOGY = 0x16D
    D3D11_MESSAGE_ID_DEVICE_DRAW_VERTEX_OFFSET_UNALIGNED = 0x16E
    D3D11_MESSAGE_ID_DEVICE_DRAW_VERTEX_STRIDE_UNALIGNED = 0x16F
    D3D11_MESSAGE_ID_DEVICE_DRAW_INDEX_OFFSET_UNALIGNED = 0x170
    D3D11_MESSAGE_ID_DEVICE_DRAW_OUTPUT_STREAM_OFFSET_UNALIGNED = 0x171
    D3D11_MESSAGE_ID_DEVICE_DRAW_RESOURCE_FORMAT_LD_UNSUPPORTED = 0x172
    D3D11_MESSAGE_ID_DEVICE_DRAW_RESOURCE_FORMAT_SAMPLE_UNSUPPORTED = 0x173
    D3D11_MESSAGE_ID_DEVICE_DRAW_RESOURCE_FORMAT_SAMPLE_C_UNSUPPORTED = 0x174
    D3D11_MESSAGE_ID_DEVICE_DRAW_RESOURCE_MULTISAMPLE_UNSUPPORTED = 0x175
    D3D11_MESSAGE_ID_DEVICE_DRAW_SO_TARGETS_BOUND_WITHOUT_SOURCE = 0x176
    D3D11_MESSAGE_ID_DEVICE_DRAW_SO_STRIDE_LARGER_THAN_BUFFER = 0x177
    D3D11_MESSAGE_ID_DEVICE_DRAW_OM_RENDER_TARGET_DOES_NOT_SUPPORT_BLENDING = 0x178
    D3D11_MESSAGE_ID_DEVICE_DRAW_OM_DUAL_SOURCE_BLENDING_CAN_ONLY_HAVE_RENDER_TARGET_0 = 0x179
    D3D11_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_AT_FAULT = 0x17A
    D3D11_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_POSSIBLY_AT_FAULT = 0x17B
    D3D11_MESSAGE_ID_DEVICE_REMOVAL_PROCESS_NOT_AT_FAULT = 0x17C
    D3D11_MESSAGE_ID_DEVICE_OPEN_SHARED_RESOURCE_INVALIDARG_RETURN = 0x17D
    D3D11_MESSAGE_ID_DEVICE_OPEN_SHARED_RESOURCE_OUTOFMEMORY_RETURN = 0x17E
    D3D11_MESSAGE_ID_DEVICE_OPEN_SHARED_RESOURCE_BADINTERFACE_RETURN = 0x17F
    D3D11_MESSAGE_ID_DEVICE_DRAW_VIEWPORT_NOT_SET = 0x180
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_TRAILING_DIGIT_IN_SEMANTIC = 0x181
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_TRAILING_DIGIT_IN_SEMANTIC = 0x182
    D3D11_MESSAGE_ID_DEVICE_RSSETVIEWPORTS_DENORMFLUSH = 0x183
    D3D11_MESSAGE_ID_OMSETRENDERTARGETS_INVALIDVIEW = 0x184
    D3D11_MESSAGE_ID_DEVICE_SETTEXTFILTERSIZE_INVALIDDIMENSIONS = 0x185
    D3D11_MESSAGE_ID_DEVICE_DRAW_SAMPLER_MISMATCH = 0x186
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_TYPE_MISMATCH = 0x187
    D3D11_MESSAGE_ID_BLENDSTATE_GETDESC_LEGACY = 0x188
    D3D11_MESSAGE_ID_SHADERRESOURCEVIEW_GETDESC_LEGACY = 0x189
    D3D11_MESSAGE_ID_CREATEQUERY_OUTOFMEMORY_RETURN = 0x18A
    D3D11_MESSAGE_ID_CREATEPREDICATE_OUTOFMEMORY_RETURN = 0x18B
    D3D11_MESSAGE_ID_CREATECOUNTER_OUTOFRANGE_COUNTER = 0x18C
    D3D11_MESSAGE_ID_CREATECOUNTER_SIMULTANEOUS_ACTIVE_COUNTERS_EXHAUSTED = 0x18D
    D3D11_MESSAGE_ID_CREATECOUNTER_UNSUPPORTED_WELLKNOWN_COUNTER = 0x18E
    D3D11_MESSAGE_ID_CREATECOUNTER_OUTOFMEMORY_RETURN = 0x18F
    D3D11_MESSAGE_ID_CREATECOUNTER_NONEXCLUSIVE_RETURN = 0x190
    D3D11_MESSAGE_ID_CREATECOUNTER_NULLDESC = 0x191
    D3D11_MESSAGE_ID_CHECKCOUNTER_OUTOFRANGE_COUNTER = 0x192
    D3D11_MESSAGE_ID_CHECKCOUNTER_UNSUPPORTED_WELLKNOWN_COUNTER = 0x193
    D3D11_MESSAGE_ID_SETPREDICATION_INVALID_PREDICATE_STATE = 0x194
    D3D11_MESSAGE_ID_QUERY_BEGIN_UNSUPPORTED = 0x195
    D3D11_MESSAGE_ID_PREDICATE_BEGIN_DURING_PREDICATION = 0x196
    D3D11_MESSAGE_ID_QUERY_BEGIN_DUPLICATE = 0x197
    D3D11_MESSAGE_ID_QUERY_BEGIN_ABANDONING_PREVIOUS_RESULTS = 0x198
    D3D11_MESSAGE_ID_PREDICATE_END_DURING_PREDICATION = 0x199
    D3D11_MESSAGE_ID_QUERY_END_ABANDONING_PREVIOUS_RESULTS = 0x19A
    D3D11_MESSAGE_ID_QUERY_END_WITHOUT_BEGIN = 0x19B
    D3D11_MESSAGE_ID_QUERY_GETDATA_INVALID_DATASIZE = 0x19C
    D3D11_MESSAGE_ID_QUERY_GETDATA_INVALID_FLAGS = 0x19D
    D3D11_MESSAGE_ID_QUERY_GETDATA_INVALID_CALL = 0x19E
    D3D11_MESSAGE_ID_DEVICE_DRAW_PS_OUTPUT_TYPE_MISMATCH = 0x19F
    D3D11_MESSAGE_ID_DEVICE_DRAW_RESOURCE_FORMAT_GATHER_UNSUPPORTED = 0x1A0
    D3D11_MESSAGE_ID_DEVICE_DRAW_INVALID_USE_OF_CENTER_MULTISAMPLE_PATTERN = 0x1A1
    D3D11_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_STRIDE_TOO_LARGE = 0x1A2
    D3D11_MESSAGE_ID_DEVICE_IASETVERTEXBUFFERS_INVALIDRANGE = 0x1A3
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_EMPTY_LAYOUT = 0x1A4
    D3D11_MESSAGE_ID_DEVICE_DRAW_RESOURCE_SAMPLE_COUNT_MISMATCH = 0x1A5
    D3D11_MESSAGE_ID_LIVE_OBJECT_SUMMARY = 0x1A6
    D3D11_MESSAGE_ID_LIVE_BUFFER = 0x1A7
    D3D11_MESSAGE_ID_LIVE_TEXTURE1D = 0x1A8
    D3D11_MESSAGE_ID_LIVE_TEXTURE2D = 0x1A9
    D3D11_MESSAGE_ID_LIVE_TEXTURE3D = 0x1AA
    D3D11_MESSAGE_ID_LIVE_SHADERRESOURCEVIEW = 0x1AB
    D3D11_MESSAGE_ID_LIVE_RENDERTARGETVIEW = 0x1AC
    D3D11_MESSAGE_ID_LIVE_DEPTHSTENCILVIEW = 0x1AD
    D3D11_MESSAGE_ID_LIVE_VERTEXSHADER = 0x1AE
    D3D11_MESSAGE_ID_LIVE_GEOMETRYSHADER = 0x1AF
    D3D11_MESSAGE_ID_LIVE_PIXELSHADER = 0x1B0
    D3D11_MESSAGE_ID_LIVE_INPUTLAYOUT = 0x1B1
    D3D11_MESSAGE_ID_LIVE_SAMPLER = 0x1B2
    D3D11_MESSAGE_ID_LIVE_BLENDSTATE = 0x1B3
    D3D11_MESSAGE_ID_LIVE_DEPTHSTENCILSTATE = 0x1B4
    D3D11_MESSAGE_ID_LIVE_RASTERIZERSTATE = 0x1B5
    D3D11_MESSAGE_ID_LIVE_QUERY = 0x1B6
    D3D11_MESSAGE_ID_LIVE_PREDICATE = 0x1B7
    D3D11_MESSAGE_ID_LIVE_COUNTER = 0x1B8
    D3D11_MESSAGE_ID_LIVE_DEVICE = 0x1B9
    D3D11_MESSAGE_ID_LIVE_SWAPCHAIN = 0x1BA
    D3D11_MESSAGE_ID_D3D10_MESSAGES_END = 0x1BB
    D3D11_MESSAGE_ID_D3D10L9_MESSAGES_START = 0x100000
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILSTATE_STENCIL_NO_TWO_SIDED = 0x100001
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_DepthBiasClamp_NOT_SUPPORTED = 0x100002
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_NO_COMPARISON_SUPPORT = 0x100003
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_EXCESSIVE_ANISOTROPY = 0x100004
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_BORDER_OUT_OF_RANGE = 0x100005
    D3D11_MESSAGE_ID_VSSETSAMPLERS_NOT_SUPPORTED = 0x100006
    D3D11_MESSAGE_ID_VSSETSAMPLERS_TOO_MANY_SAMPLERS = 0x100007
    D3D11_MESSAGE_ID_PSSETSAMPLERS_TOO_MANY_SAMPLERS = 0x100008
    D3D11_MESSAGE_ID_CREATERESOURCE_NO_ARRAYS = 0x100009
    D3D11_MESSAGE_ID_CREATERESOURCE_NO_VB_AND_IB_BIND = 0x10000A
    D3D11_MESSAGE_ID_CREATERESOURCE_NO_TEXTURE_1D = 0x10000B
    D3D11_MESSAGE_ID_CREATERESOURCE_DIMENSION_OUT_OF_RANGE = 0x10000C
    D3D11_MESSAGE_ID_CREATERESOURCE_NOT_BINDABLE_AS_SHADER_RESOURCE = 0x10000D
    D3D11_MESSAGE_ID_OMSETRENDERTARGETS_TOO_MANY_RENDER_TARGETS = 0x10000E
    D3D11_MESSAGE_ID_OMSETRENDERTARGETS_NO_DIFFERING_BIT_DEPTHS = 0x10000F
    D3D11_MESSAGE_ID_IASETVERTEXBUFFERS_BAD_BUFFER_INDEX = 0x100010
    D3D11_MESSAGE_ID_DEVICE_RSSETVIEWPORTS_TOO_MANY_VIEWPORTS = 0x100011
    D3D11_MESSAGE_ID_DEVICE_IASETPRIMITIVETOPOLOGY_ADJACENCY_UNSUPPORTED = 0x100012
    D3D11_MESSAGE_ID_DEVICE_RSSETSCISSORRECTS_TOO_MANY_SCISSORS = 0x100013
    D3D11_MESSAGE_ID_COPYRESOURCE_ONLY_TEXTURE_2D_WITHIN_GPU_MEMORY = 0x100014
    D3D11_MESSAGE_ID_COPYRESOURCE_NO_TEXTURE_3D_READBACK = 0x100015
    D3D11_MESSAGE_ID_COPYRESOURCE_NO_TEXTURE_ONLY_READBACK = 0x100016
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_UNSUPPORTED_FORMAT = 0x100017
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_NO_ALPHA_TO_COVERAGE = 0x100018
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_DepthClipEnable_MUST_BE_TRUE = 0x100019
    D3D11_MESSAGE_ID_DRAWINDEXED_STARTINDEXLOCATION_MUST_BE_POSITIVE = 0x10001A
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_MUST_USE_LOWEST_LOD = 0x10001B
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_MINLOD_MUST_NOT_BE_FRACTIONAL = 0x10001C
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_MAXLOD_MUST_BE_FLT_MAX = 0x10001D
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_FIRSTARRAYSLICE_MUST_BE_ZERO = 0x10001E
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_CUBES_MUST_HAVE_6_SIDES = 0x10001F
    D3D11_MESSAGE_ID_CREATERESOURCE_NOT_BINDABLE_AS_RENDER_TARGET = 0x100020
    D3D11_MESSAGE_ID_CREATERESOURCE_NO_DWORD_INDEX_BUFFER = 0x100021
    D3D11_MESSAGE_ID_CREATERESOURCE_MSAA_PRECLUDES_SHADER_RESOURCE = 0x100022
    D3D11_MESSAGE_ID_CREATERESOURCE_PRESENTATION_PRECLUDES_SHADER_RESOURCE = 0x100023
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_NO_INDEPENDENT_BLEND_ENABLE = 0x100024
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_NO_INDEPENDENT_WRITE_MASKS = 0x100025
    D3D11_MESSAGE_ID_CREATERESOURCE_NO_STREAM_OUT = 0x100026
    D3D11_MESSAGE_ID_CREATERESOURCE_ONLY_VB_IB_FOR_BUFFERS = 0x100027
    D3D11_MESSAGE_ID_CREATERESOURCE_NO_AUTOGEN_FOR_VOLUMES = 0x100028
    D3D11_MESSAGE_ID_CREATERESOURCE_DXGI_FORMAT_R8G8B8A8_CANNOT_BE_SHARED = 0x100029
    D3D11_MESSAGE_ID_VSSHADERRESOURCES_NOT_SUPPORTED = 0x10002A
    D3D11_MESSAGE_ID_GEOMETRY_SHADER_NOT_SUPPORTED = 0x10002B
    D3D11_MESSAGE_ID_STREAM_OUT_NOT_SUPPORTED = 0x10002C
    D3D11_MESSAGE_ID_TEXT_FILTER_NOT_SUPPORTED = 0x10002D
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_NO_SEPARATE_ALPHA_BLEND = 0x10002E
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_NO_MRT_BLEND = 0x10002F
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_OPERATION_NOT_SUPPORTED = 0x100030
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_NO_MIRRORONCE = 0x100031
    D3D11_MESSAGE_ID_DRAWINSTANCED_NOT_SUPPORTED = 0x100032
    D3D11_MESSAGE_ID_DRAWINDEXEDINSTANCED_NOT_SUPPORTED_BELOW_9_3 = 0x100033
    D3D11_MESSAGE_ID_DRAWINDEXED_POINTLIST_UNSUPPORTED = 0x100034
    D3D11_MESSAGE_ID_SETBLENDSTATE_SAMPLE_MASK_CANNOT_BE_ZERO = 0x100035
    D3D11_MESSAGE_ID_CREATERESOURCE_DIMENSION_EXCEEDS_FEATURE_LEVEL_DEFINITION = 0x100036
    D3D11_MESSAGE_ID_CREATERESOURCE_ONLY_SINGLE_MIP_LEVEL_DEPTH_STENCIL_SUPPORTED = 0x100037
    D3D11_MESSAGE_ID_DEVICE_RSSETSCISSORRECTS_NEGATIVESCISSOR = 0x100038
    D3D11_MESSAGE_ID_SLOT_ZERO_MUST_BE_D3D10_INPUT_PER_VERTEX_DATA = 0x100039
    D3D11_MESSAGE_ID_CREATERESOURCE_NON_POW_2_MIPMAP = 0x10003A
    D3D11_MESSAGE_ID_CREATESAMPLERSTATE_BORDER_NOT_SUPPORTED = 0x10003B
    D3D11_MESSAGE_ID_OMSETRENDERTARGETS_NO_SRGB_MRT = 0x10003C
    D3D11_MESSAGE_ID_COPYRESOURCE_NO_3D_MISMATCHED_UPDATES = 0x10003D
    D3D11_MESSAGE_ID_D3D10L9_MESSAGES_END = 0x10003E
    D3D11_MESSAGE_ID_D3D11_MESSAGES_START = 0x200000
    D3D11_MESSAGE_ID_CREATEDEPTHSTENCILVIEW_INVALIDFLAGS = 0x200001
    D3D11_MESSAGE_ID_CREATEVERTEXSHADER_INVALIDCLASSLINKAGE = 0x200002
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADER_INVALIDCLASSLINKAGE = 0x200003
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDNUMSTREAMS = 0x200004
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSTREAMTORASTERIZER = 0x200005
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UNEXPECTEDSTREAMS = 0x200006
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDCLASSLINKAGE = 0x200007
    D3D11_MESSAGE_ID_CREATEPIXELSHADER_INVALIDCLASSLINKAGE = 0x200008
    D3D11_MESSAGE_ID_CREATEDEFERREDCONTEXT_INVALID_COMMANDLISTFLAGS = 0x200009
    D3D11_MESSAGE_ID_CREATEDEFERREDCONTEXT_SINGLETHREADED = 0x20000A
    D3D11_MESSAGE_ID_CREATEDEFERREDCONTEXT_INVALIDARG_RETURN = 0x20000B
    D3D11_MESSAGE_ID_CREATEDEFERREDCONTEXT_INVALID_CALL_RETURN = 0x20000C
    D3D11_MESSAGE_ID_CREATEDEFERREDCONTEXT_OUTOFMEMORY_RETURN = 0x20000D
    D3D11_MESSAGE_ID_FINISHDISPLAYLIST_ONIMMEDIATECONTEXT = 0x20000E
    D3D11_MESSAGE_ID_FINISHDISPLAYLIST_OUTOFMEMORY_RETURN = 0x20000F
    D3D11_MESSAGE_ID_FINISHDISPLAYLIST_INVALID_CALL_RETURN = 0x200010
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDSTREAM = 0x200011
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UNEXPECTEDENTRIES = 0x200012
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UNEXPECTEDSTRIDES = 0x200013
    D3D11_MESSAGE_ID_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_INVALIDNUMSTRIDES = 0x200014
    D3D11_MESSAGE_ID_DEVICE_HSSETSHADERRESOURCES_HAZARD = 0x200015
    D3D11_MESSAGE_ID_DEVICE_HSSETCONSTANTBUFFERS_HAZARD = 0x200016
    D3D11_MESSAGE_ID_HSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x200017
    D3D11_MESSAGE_ID_HSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x200018
    D3D11_MESSAGE_ID_CREATEHULLSHADER_INVALIDCALL = 0x200019
    D3D11_MESSAGE_ID_CREATEHULLSHADER_OUTOFMEMORY = 0x20001A
    D3D11_MESSAGE_ID_CREATEHULLSHADER_INVALIDSHADERBYTECODE = 0x20001B
    D3D11_MESSAGE_ID_CREATEHULLSHADER_INVALIDSHADERTYPE = 0x20001C
    D3D11_MESSAGE_ID_CREATEHULLSHADER_INVALIDCLASSLINKAGE = 0x20001D
    D3D11_MESSAGE_ID_DEVICE_HSSETSHADERRESOURCES_VIEWS_EMPTY = 0x20001E
    D3D11_MESSAGE_ID_HSSETCONSTANTBUFFERS_INVALIDBUFFER = 0x20001F
    D3D11_MESSAGE_ID_DEVICE_HSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x200020
    D3D11_MESSAGE_ID_DEVICE_HSSETSAMPLERS_SAMPLERS_EMPTY = 0x200021
    D3D11_MESSAGE_ID_DEVICE_HSGETSHADERRESOURCES_VIEWS_EMPTY = 0x200022
    D3D11_MESSAGE_ID_DEVICE_HSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x200023
    D3D11_MESSAGE_ID_DEVICE_HSGETSAMPLERS_SAMPLERS_EMPTY = 0x200024
    D3D11_MESSAGE_ID_DEVICE_DSSETSHADERRESOURCES_HAZARD = 0x200025
    D3D11_MESSAGE_ID_DEVICE_DSSETCONSTANTBUFFERS_HAZARD = 0x200026
    D3D11_MESSAGE_ID_DSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x200027
    D3D11_MESSAGE_ID_DSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x200028
    D3D11_MESSAGE_ID_CREATEDOMAINSHADER_INVALIDCALL = 0x200029
    D3D11_MESSAGE_ID_CREATEDOMAINSHADER_OUTOFMEMORY = 0x20002A
    D3D11_MESSAGE_ID_CREATEDOMAINSHADER_INVALIDSHADERBYTECODE = 0x20002B
    D3D11_MESSAGE_ID_CREATEDOMAINSHADER_INVALIDSHADERTYPE = 0x20002C
    D3D11_MESSAGE_ID_CREATEDOMAINSHADER_INVALIDCLASSLINKAGE = 0x20002D
    D3D11_MESSAGE_ID_DEVICE_DSSETSHADERRESOURCES_VIEWS_EMPTY = 0x20002E
    D3D11_MESSAGE_ID_DSSETCONSTANTBUFFERS_INVALIDBUFFER = 0x20002F
    D3D11_MESSAGE_ID_DEVICE_DSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x200030
    D3D11_MESSAGE_ID_DEVICE_DSSETSAMPLERS_SAMPLERS_EMPTY = 0x200031
    D3D11_MESSAGE_ID_DEVICE_DSGETSHADERRESOURCES_VIEWS_EMPTY = 0x200032
    D3D11_MESSAGE_ID_DEVICE_DSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x200033
    D3D11_MESSAGE_ID_DEVICE_DSGETSAMPLERS_SAMPLERS_EMPTY = 0x200034
    D3D11_MESSAGE_ID_DEVICE_DRAW_HS_XOR_DS_MISMATCH = 0x200035
    D3D11_MESSAGE_ID_DEFERRED_CONTEXT_REMOVAL_PROCESS_AT_FAULT = 0x200036
    D3D11_MESSAGE_ID_DEVICE_DRAWINDIRECT_INVALID_ARG_BUFFER = 0x200037
    D3D11_MESSAGE_ID_DEVICE_DRAWINDIRECT_OFFSET_UNALIGNED = 0x200038
    D3D11_MESSAGE_ID_DEVICE_DRAWINDIRECT_OFFSET_OVERFLOW = 0x200039
    D3D11_MESSAGE_ID_RESOURCE_MAP_INVALIDMAPTYPE = 0x20003A
    D3D11_MESSAGE_ID_RESOURCE_MAP_INVALIDSUBRESOURCE = 0x20003B
    D3D11_MESSAGE_ID_RESOURCE_MAP_INVALIDFLAGS = 0x20003C
    D3D11_MESSAGE_ID_RESOURCE_MAP_ALREADYMAPPED = 0x20003D
    D3D11_MESSAGE_ID_RESOURCE_MAP_DEVICEREMOVED_RETURN = 0x20003E
    D3D11_MESSAGE_ID_RESOURCE_MAP_OUTOFMEMORY_RETURN = 0x20003F
    D3D11_MESSAGE_ID_RESOURCE_MAP_WITHOUT_INITIAL_DISCARD = 0x200040
    D3D11_MESSAGE_ID_RESOURCE_UNMAP_INVALIDSUBRESOURCE = 0x200041
    D3D11_MESSAGE_ID_RESOURCE_UNMAP_NOTMAPPED = 0x200042
    D3D11_MESSAGE_ID_DEVICE_DRAW_RASTERIZING_CONTROL_POINTS = 0x200043
    D3D11_MESSAGE_ID_DEVICE_IASETPRIMITIVETOPOLOGY_TOPOLOGY_UNSUPPORTED = 0x200044
    D3D11_MESSAGE_ID_DEVICE_DRAW_HS_DS_SIGNATURE_MISMATCH = 0x200045
    D3D11_MESSAGE_ID_DEVICE_DRAW_HULL_SHADER_INPUT_TOPOLOGY_MISMATCH = 0x200046
    D3D11_MESSAGE_ID_DEVICE_DRAW_HS_DS_CONTROL_POINT_COUNT_MISMATCH = 0x200047
    D3D11_MESSAGE_ID_DEVICE_DRAW_HS_DS_TESSELLATOR_DOMAIN_MISMATCH = 0x200048
    D3D11_MESSAGE_ID_CREATE_CONTEXT = 0x200049
    D3D11_MESSAGE_ID_LIVE_CONTEXT = 0x20004A
    D3D11_MESSAGE_ID_DESTROY_CONTEXT = 0x20004B
    D3D11_MESSAGE_ID_CREATE_BUFFER = 0x20004C
    D3D11_MESSAGE_ID_LIVE_BUFFER_WIN7 = 0x20004D
    D3D11_MESSAGE_ID_DESTROY_BUFFER = 0x20004E
    D3D11_MESSAGE_ID_CREATE_TEXTURE1D = 0x20004F
    D3D11_MESSAGE_ID_LIVE_TEXTURE1D_WIN7 = 0x200050
    D3D11_MESSAGE_ID_DESTROY_TEXTURE1D = 0x200051
    D3D11_MESSAGE_ID_CREATE_TEXTURE2D = 0x200052
    D3D11_MESSAGE_ID_LIVE_TEXTURE2D_WIN7 = 0x200053
    D3D11_MESSAGE_ID_DESTROY_TEXTURE2D = 0x200054
    D3D11_MESSAGE_ID_CREATE_TEXTURE3D = 0x200055
    D3D11_MESSAGE_ID_LIVE_TEXTURE3D_WIN7 = 0x200056
    D3D11_MESSAGE_ID_DESTROY_TEXTURE3D = 0x200057
    D3D11_MESSAGE_ID_CREATE_SHADERRESOURCEVIEW = 0x200058
    D3D11_MESSAGE_ID_LIVE_SHADERRESOURCEVIEW_WIN7 = 0x200059
    D3D11_MESSAGE_ID_DESTROY_SHADERRESOURCEVIEW = 0x20005A
    D3D11_MESSAGE_ID_CREATE_RENDERTARGETVIEW = 0x20005B
    D3D11_MESSAGE_ID_LIVE_RENDERTARGETVIEW_WIN7 = 0x20005C
    D3D11_MESSAGE_ID_DESTROY_RENDERTARGETVIEW = 0x20005D
    D3D11_MESSAGE_ID_CREATE_DEPTHSTENCILVIEW = 0x20005E
    D3D11_MESSAGE_ID_LIVE_DEPTHSTENCILVIEW_WIN7 = 0x20005F
    D3D11_MESSAGE_ID_DESTROY_DEPTHSTENCILVIEW = 0x200060
    D3D11_MESSAGE_ID_CREATE_VERTEXSHADER = 0x200061
    D3D11_MESSAGE_ID_LIVE_VERTEXSHADER_WIN7 = 0x200062
    D3D11_MESSAGE_ID_DESTROY_VERTEXSHADER = 0x200063
    D3D11_MESSAGE_ID_CREATE_HULLSHADER = 0x200064
    D3D11_MESSAGE_ID_LIVE_HULLSHADER = 0x200065
    D3D11_MESSAGE_ID_DESTROY_HULLSHADER = 0x200066
    D3D11_MESSAGE_ID_CREATE_DOMAINSHADER = 0x200067
    D3D11_MESSAGE_ID_LIVE_DOMAINSHADER = 0x200068
    D3D11_MESSAGE_ID_DESTROY_DOMAINSHADER = 0x200069
    D3D11_MESSAGE_ID_CREATE_GEOMETRYSHADER = 0x20006A
    D3D11_MESSAGE_ID_LIVE_GEOMETRYSHADER_WIN7 = 0x20006B
    D3D11_MESSAGE_ID_DESTROY_GEOMETRYSHADER = 0x20006C
    D3D11_MESSAGE_ID_CREATE_PIXELSHADER = 0x20006D
    D3D11_MESSAGE_ID_LIVE_PIXELSHADER_WIN7 = 0x20006E
    D3D11_MESSAGE_ID_DESTROY_PIXELSHADER = 0x20006F
    D3D11_MESSAGE_ID_CREATE_INPUTLAYOUT = 0x200070
    D3D11_MESSAGE_ID_LIVE_INPUTLAYOUT_WIN7 = 0x200071
    D3D11_MESSAGE_ID_DESTROY_INPUTLAYOUT = 0x200072
    D3D11_MESSAGE_ID_CREATE_SAMPLER = 0x200073
    D3D11_MESSAGE_ID_LIVE_SAMPLER_WIN7 = 0x200074
    D3D11_MESSAGE_ID_DESTROY_SAMPLER = 0x200075
    D3D11_MESSAGE_ID_CREATE_BLENDSTATE = 0x200076
    D3D11_MESSAGE_ID_LIVE_BLENDSTATE_WIN7 = 0x200077
    D3D11_MESSAGE_ID_DESTROY_BLENDSTATE = 0x200078
    D3D11_MESSAGE_ID_CREATE_DEPTHSTENCILSTATE = 0x200079
    D3D11_MESSAGE_ID_LIVE_DEPTHSTENCILSTATE_WIN7 = 0x20007A
    D3D11_MESSAGE_ID_DESTROY_DEPTHSTENCILSTATE = 0x20007B
    D3D11_MESSAGE_ID_CREATE_RASTERIZERSTATE = 0x20007C
    D3D11_MESSAGE_ID_LIVE_RASTERIZERSTATE_WIN7 = 0x20007D
    D3D11_MESSAGE_ID_DESTROY_RASTERIZERSTATE = 0x20007E
    D3D11_MESSAGE_ID_CREATE_QUERY = 0x20007F
    D3D11_MESSAGE_ID_LIVE_QUERY_WIN7 = 0x200080
    D3D11_MESSAGE_ID_DESTROY_QUERY = 0x200081
    D3D11_MESSAGE_ID_CREATE_PREDICATE = 0x200082
    D3D11_MESSAGE_ID_LIVE_PREDICATE_WIN7 = 0x200083
    D3D11_MESSAGE_ID_DESTROY_PREDICATE = 0x200084
    D3D11_MESSAGE_ID_CREATE_COUNTER = 0x200085
    D3D11_MESSAGE_ID_DESTROY_COUNTER = 0x200086
    D3D11_MESSAGE_ID_CREATE_COMMANDLIST = 0x200087
    D3D11_MESSAGE_ID_LIVE_COMMANDLIST = 0x200088
    D3D11_MESSAGE_ID_DESTROY_COMMANDLIST = 0x200089
    D3D11_MESSAGE_ID_CREATE_CLASSINSTANCE = 0x20008A
    D3D11_MESSAGE_ID_LIVE_CLASSINSTANCE = 0x20008B
    D3D11_MESSAGE_ID_DESTROY_CLASSINSTANCE = 0x20008C
    D3D11_MESSAGE_ID_CREATE_CLASSLINKAGE = 0x20008D
    D3D11_MESSAGE_ID_LIVE_CLASSLINKAGE = 0x20008E
    D3D11_MESSAGE_ID_DESTROY_CLASSLINKAGE = 0x20008F
    D3D11_MESSAGE_ID_LIVE_DEVICE_WIN7 = 0x200090
    D3D11_MESSAGE_ID_LIVE_OBJECT_SUMMARY_WIN7 = 0x200091
    D3D11_MESSAGE_ID_CREATE_COMPUTESHADER = 0x200092
    D3D11_MESSAGE_ID_LIVE_COMPUTESHADER = 0x200093
    D3D11_MESSAGE_ID_DESTROY_COMPUTESHADER = 0x200094
    D3D11_MESSAGE_ID_CREATE_UNORDEREDACCESSVIEW = 0x200095
    D3D11_MESSAGE_ID_LIVE_UNORDEREDACCESSVIEW = 0x200096
    D3D11_MESSAGE_ID_DESTROY_UNORDEREDACCESSVIEW = 0x200097
    D3D11_MESSAGE_ID_DEVICE_SETSHADER_INTERFACES_FEATURELEVEL = 0x200098
    D3D11_MESSAGE_ID_DEVICE_SETSHADER_INTERFACE_COUNT_MISMATCH = 0x200099
    D3D11_MESSAGE_ID_DEVICE_SETSHADER_INVALID_INSTANCE = 0x20009A
    D3D11_MESSAGE_ID_DEVICE_SETSHADER_INVALID_INSTANCE_INDEX = 0x20009B
    D3D11_MESSAGE_ID_DEVICE_SETSHADER_INVALID_INSTANCE_TYPE = 0x20009C
    D3D11_MESSAGE_ID_DEVICE_SETSHADER_INVALID_INSTANCE_DATA = 0x20009D
    D3D11_MESSAGE_ID_DEVICE_SETSHADER_UNBOUND_INSTANCE_DATA = 0x20009E
    D3D11_MESSAGE_ID_DEVICE_SETSHADER_INSTANCE_DATA_BINDINGS = 0x20009F
    D3D11_MESSAGE_ID_DEVICE_CREATESHADER_CLASSLINKAGE_FULL = 0x2000A0
    D3D11_MESSAGE_ID_DEVICE_CHECKFEATURESUPPORT_UNRECOGNIZED_FEATURE = 0x2000A1
    D3D11_MESSAGE_ID_DEVICE_CHECKFEATURESUPPORT_MISMATCHED_DATA_SIZE = 0x2000A2
    D3D11_MESSAGE_ID_DEVICE_CHECKFEATURESUPPORT_INVALIDARG_RETURN = 0x2000A3
    D3D11_MESSAGE_ID_DEVICE_CSSETSHADERRESOURCES_HAZARD = 0x2000A4
    D3D11_MESSAGE_ID_DEVICE_CSSETCONSTANTBUFFERS_HAZARD = 0x2000A5
    D3D11_MESSAGE_ID_CSSETSHADERRESOURCES_UNBINDDELETINGOBJECT = 0x2000A6
    D3D11_MESSAGE_ID_CSSETCONSTANTBUFFERS_UNBINDDELETINGOBJECT = 0x2000A7
    D3D11_MESSAGE_ID_CREATECOMPUTESHADER_INVALIDCALL = 0x2000A8
    D3D11_MESSAGE_ID_CREATECOMPUTESHADER_OUTOFMEMORY = 0x2000A9
    D3D11_MESSAGE_ID_CREATECOMPUTESHADER_INVALIDSHADERBYTECODE = 0x2000AA
    D3D11_MESSAGE_ID_CREATECOMPUTESHADER_INVALIDSHADERTYPE = 0x2000AB
    D3D11_MESSAGE_ID_CREATECOMPUTESHADER_INVALIDCLASSLINKAGE = 0x2000AC
    D3D11_MESSAGE_ID_DEVICE_CSSETSHADERRESOURCES_VIEWS_EMPTY = 0x2000AD
    D3D11_MESSAGE_ID_CSSETCONSTANTBUFFERS_INVALIDBUFFER = 0x2000AE
    D3D11_MESSAGE_ID_DEVICE_CSSETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x2000AF
    D3D11_MESSAGE_ID_DEVICE_CSSETSAMPLERS_SAMPLERS_EMPTY = 0x2000B0
    D3D11_MESSAGE_ID_DEVICE_CSGETSHADERRESOURCES_VIEWS_EMPTY = 0x2000B1
    D3D11_MESSAGE_ID_DEVICE_CSGETCONSTANTBUFFERS_BUFFERS_EMPTY = 0x2000B2
    D3D11_MESSAGE_ID_DEVICE_CSGETSAMPLERS_SAMPLERS_EMPTY = 0x2000B3
    D3D11_MESSAGE_ID_DEVICE_CREATEVERTEXSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x2000B4
    D3D11_MESSAGE_ID_DEVICE_CREATEHULLSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x2000B5
    D3D11_MESSAGE_ID_DEVICE_CREATEDOMAINSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x2000B6
    D3D11_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x2000B7
    D3D11_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_DOUBLEFLOATOPSNOTSUPPORTED = 0x2000B8
    D3D11_MESSAGE_ID_DEVICE_CREATEPIXELSHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x2000B9
    D3D11_MESSAGE_ID_DEVICE_CREATECOMPUTESHADER_DOUBLEFLOATOPSNOTSUPPORTED = 0x2000BA
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDSTRUCTURESTRIDE = 0x2000BB
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDFLAGS = 0x2000BC
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDRESOURCE = 0x2000BD
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDDESC = 0x2000BE
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDFORMAT = 0x2000BF
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDDIMENSIONS = 0x2000C0
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_UNRECOGNIZEDFORMAT = 0x2000C1
    D3D11_MESSAGE_ID_DEVICE_OMSETRENDERTARGETSANDUNORDEREDACCESSVIEWS_HAZARD = 0x2000C2
    D3D11_MESSAGE_ID_DEVICE_OMSETRENDERTARGETSANDUNORDEREDACCESSVIEWS_OVERLAPPING_OLD_SLOTS = 0x2000C3
    D3D11_MESSAGE_ID_DEVICE_OMSETRENDERTARGETSANDUNORDEREDACCESSVIEWS_NO_OP = 0x2000C4
    D3D11_MESSAGE_ID_CSSETUNORDEREDACCESSVIEWS_UNBINDDELETINGOBJECT = 0x2000C5
    D3D11_MESSAGE_ID_PSSETUNORDEREDACCESSVIEWS_UNBINDDELETINGOBJECT = 0x2000C6
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDARG_RETURN = 0x2000C7
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_OUTOFMEMORY_RETURN = 0x2000C8
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_TOOMANYOBJECTS = 0x2000C9
    D3D11_MESSAGE_ID_DEVICE_CSSETUNORDEREDACCESSVIEWS_HAZARD = 0x2000CA
    D3D11_MESSAGE_ID_CLEARUNORDEREDACCESSVIEW_DENORMFLUSH = 0x2000CB
    D3D11_MESSAGE_ID_DEVICE_CSSETUNORDEREDACCESSS_VIEWS_EMPTY = 0x2000CC
    D3D11_MESSAGE_ID_DEVICE_CSGETUNORDEREDACCESSS_VIEWS_EMPTY = 0x2000CD
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDFLAGS = 0x2000CE
    D3D11_MESSAGE_ID_CREATESHADERRESESOURCEVIEW_TOOMANYOBJECTS = 0x2000CF
    D3D11_MESSAGE_ID_DEVICE_DISPATCHINDIRECT_INVALID_ARG_BUFFER = 0x2000D0
    D3D11_MESSAGE_ID_DEVICE_DISPATCHINDIRECT_OFFSET_UNALIGNED = 0x2000D1
    D3D11_MESSAGE_ID_DEVICE_DISPATCHINDIRECT_OFFSET_OVERFLOW = 0x2000D2
    D3D11_MESSAGE_ID_DEVICE_SETRESOURCEMINLOD_INVALIDCONTEXT = 0x2000D3
    D3D11_MESSAGE_ID_DEVICE_SETRESOURCEMINLOD_INVALIDRESOURCE = 0x2000D4
    D3D11_MESSAGE_ID_DEVICE_SETRESOURCEMINLOD_INVALIDMINLOD = 0x2000D5
    D3D11_MESSAGE_ID_DEVICE_GETRESOURCEMINLOD_INVALIDCONTEXT = 0x2000D6
    D3D11_MESSAGE_ID_DEVICE_GETRESOURCEMINLOD_INVALIDRESOURCE = 0x2000D7
    D3D11_MESSAGE_ID_OMSETDEPTHSTENCIL_UNBINDDELETINGOBJECT = 0x2000D8
    D3D11_MESSAGE_ID_CLEARDEPTHSTENCILVIEW_DEPTH_READONLY = 0x2000D9
    D3D11_MESSAGE_ID_CLEARDEPTHSTENCILVIEW_STENCIL_READONLY = 0x2000DA
    D3D11_MESSAGE_ID_CHECKFEATURESUPPORT_FORMAT_DEPRECATED = 0x2000DB
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_RETURN_TYPE_MISMATCH = 0x2000DC
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_NOT_SET = 0x2000DD
    D3D11_MESSAGE_ID_DEVICE_DRAW_UNORDEREDACCESSVIEW_RENDERTARGETVIEW_OVERLAP = 0x2000DE
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_DIMENSION_MISMATCH = 0x2000DF
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_APPEND_UNSUPPORTED = 0x2000E0
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_ATOMICS_UNSUPPORTED = 0x2000E1
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_STRUCTURE_STRIDE_MISMATCH = 0x2000E2
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_BUFFER_TYPE_MISMATCH = 0x2000E3
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_RAW_UNSUPPORTED = 0x2000E4
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_FORMAT_LD_UNSUPPORTED = 0x2000E5
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_FORMAT_STORE_UNSUPPORTED = 0x2000E6
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_ATOMIC_ADD_UNSUPPORTED = 0x2000E7
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_ATOMIC_BITWISE_OPS_UNSUPPORTED = 0x2000E8
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_ATOMIC_CMPSTORE_CMPEXCHANGE_UNSUPPORTED = 0x2000E9
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_ATOMIC_EXCHANGE_UNSUPPORTED = 0x2000EA
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_ATOMIC_SIGNED_MINMAX_UNSUPPORTED = 0x2000EB
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_ATOMIC_UNSIGNED_MINMAX_UNSUPPORTED = 0x2000EC
    D3D11_MESSAGE_ID_DEVICE_DISPATCH_BOUND_RESOURCE_MAPPED = 0x2000ED
    D3D11_MESSAGE_ID_DEVICE_DISPATCH_THREADGROUPCOUNT_OVERFLOW = 0x2000EE
    D3D11_MESSAGE_ID_DEVICE_DISPATCH_THREADGROUPCOUNT_ZERO = 0x2000EF
    D3D11_MESSAGE_ID_DEVICE_SHADERRESOURCEVIEW_STRUCTURE_STRIDE_MISMATCH = 0x2000F0
    D3D11_MESSAGE_ID_DEVICE_SHADERRESOURCEVIEW_BUFFER_TYPE_MISMATCH = 0x2000F1
    D3D11_MESSAGE_ID_DEVICE_SHADERRESOURCEVIEW_RAW_UNSUPPORTED = 0x2000F2
    D3D11_MESSAGE_ID_DEVICE_DISPATCH_UNSUPPORTED = 0x2000F3
    D3D11_MESSAGE_ID_DEVICE_DISPATCHINDIRECT_UNSUPPORTED = 0x2000F4
    D3D11_MESSAGE_ID_COPYSTRUCTURECOUNT_INVALIDOFFSET = 0x2000F5
    D3D11_MESSAGE_ID_COPYSTRUCTURECOUNT_LARGEOFFSET = 0x2000F6
    D3D11_MESSAGE_ID_COPYSTRUCTURECOUNT_INVALIDDESTINATIONSTATE = 0x2000F7
    D3D11_MESSAGE_ID_COPYSTRUCTURECOUNT_INVALIDSOURCESTATE = 0x2000F8
    D3D11_MESSAGE_ID_CHECKFORMATSUPPORT_FORMAT_NOT_SUPPORTED = 0x2000F9
    D3D11_MESSAGE_ID_DEVICE_CSSETUNORDEREDACCESSVIEWS_INVALIDVIEW = 0x2000FA
    D3D11_MESSAGE_ID_DEVICE_CSSETUNORDEREDACCESSVIEWS_INVALIDOFFSET = 0x2000FB
    D3D11_MESSAGE_ID_DEVICE_CSSETUNORDEREDACCESSVIEWS_TOOMANYVIEWS = 0x2000FC
    D3D11_MESSAGE_ID_CLEARUNORDEREDACCESSVIEWFLOAT_INVALIDFORMAT = 0x2000FD
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_COUNTER_UNSUPPORTED = 0x2000FE
    D3D11_MESSAGE_ID_REF_WARNING = 0x2000FF
    D3D11_MESSAGE_ID_DEVICE_DRAW_PIXEL_SHADER_WITHOUT_RTV_OR_DSV = 0x200100
    D3D11_MESSAGE_ID_SHADER_ABORT = 0x200101
    D3D11_MESSAGE_ID_SHADER_MESSAGE = 0x200102
    D3D11_MESSAGE_ID_SHADER_ERROR = 0x200103
    D3D11_MESSAGE_ID_OFFERRESOURCES_INVALIDRESOURCE = 0x200104
    D3D11_MESSAGE_ID_HSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x200105
    D3D11_MESSAGE_ID_DSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x200106
    D3D11_MESSAGE_ID_CSSETSAMPLERS_UNBINDDELETINGOBJECT = 0x200107
    D3D11_MESSAGE_ID_HSSETSHADER_UNBINDDELETINGOBJECT = 0x200108
    D3D11_MESSAGE_ID_DSSETSHADER_UNBINDDELETINGOBJECT = 0x200109
    D3D11_MESSAGE_ID_CSSETSHADER_UNBINDDELETINGOBJECT = 0x20010A
    D3D11_MESSAGE_ID_ENQUEUESETEVENT_INVALIDARG_RETURN = 0x20010B
    D3D11_MESSAGE_ID_ENQUEUESETEVENT_OUTOFMEMORY_RETURN = 0x20010C
    D3D11_MESSAGE_ID_ENQUEUESETEVENT_ACCESSDENIED_RETURN = 0x20010D
    D3D11_MESSAGE_ID_DEVICE_OMSETRENDERTARGETSANDUNORDEREDACCESSVIEWS_NUMUAVS_INVALIDRANGE = 0x20010E
    D3D11_MESSAGE_ID_USE_OF_ZERO_REFCOUNT_OBJECT = 0x20010F
    D3D11_MESSAGE_ID_D3D11_MESSAGES_END = 0x200110
    D3D11_MESSAGE_ID_D3D11_1_MESSAGES_START = 0x300000
    D3D11_MESSAGE_ID_CREATE_VIDEODECODER = 0x300001
    D3D11_MESSAGE_ID_CREATE_VIDEOPROCESSORENUM = 0x300002
    D3D11_MESSAGE_ID_CREATE_VIDEOPROCESSOR = 0x300003
    D3D11_MESSAGE_ID_CREATE_DECODEROUTPUTVIEW = 0x300004
    D3D11_MESSAGE_ID_CREATE_PROCESSORINPUTVIEW = 0x300005
    D3D11_MESSAGE_ID_CREATE_PROCESSOROUTPUTVIEW = 0x300006
    D3D11_MESSAGE_ID_CREATE_DEVICECONTEXTSTATE = 0x300007
    D3D11_MESSAGE_ID_LIVE_VIDEODECODER = 0x300008
    D3D11_MESSAGE_ID_LIVE_VIDEOPROCESSORENUM = 0x300009
    D3D11_MESSAGE_ID_LIVE_VIDEOPROCESSOR = 0x30000A
    D3D11_MESSAGE_ID_LIVE_DECODEROUTPUTVIEW = 0x30000B
    D3D11_MESSAGE_ID_LIVE_PROCESSORINPUTVIEW = 0x30000C
    D3D11_MESSAGE_ID_LIVE_PROCESSOROUTPUTVIEW = 0x30000D
    D3D11_MESSAGE_ID_LIVE_DEVICECONTEXTSTATE = 0x30000E
    D3D11_MESSAGE_ID_DESTROY_VIDEODECODER = 0x30000F
    D3D11_MESSAGE_ID_DESTROY_VIDEOPROCESSORENUM = 0x300010
    D3D11_MESSAGE_ID_DESTROY_VIDEOPROCESSOR = 0x300011
    D3D11_MESSAGE_ID_DESTROY_DECODEROUTPUTVIEW = 0x300012
    D3D11_MESSAGE_ID_DESTROY_PROCESSORINPUTVIEW = 0x300013
    D3D11_MESSAGE_ID_DESTROY_PROCESSOROUTPUTVIEW = 0x300014
    D3D11_MESSAGE_ID_DESTROY_DEVICECONTEXTSTATE = 0x300015
    D3D11_MESSAGE_ID_CREATEDEVICECONTEXTSTATE_INVALIDFLAGS = 0x300016
    D3D11_MESSAGE_ID_CREATEDEVICECONTEXTSTATE_INVALIDFEATURELEVEL = 0x300017
    D3D11_MESSAGE_ID_CREATEDEVICECONTEXTSTATE_FEATURELEVELS_NOT_SUPPORTED = 0x300018
    D3D11_MESSAGE_ID_CREATEDEVICECONTEXTSTATE_INVALIDREFIID = 0x300019
    D3D11_MESSAGE_ID_DEVICE_DISCARDVIEW_INVALIDVIEW = 0x30001A
    D3D11_MESSAGE_ID_COPYSUBRESOURCEREGION1_INVALIDCOPYFLAGS = 0x30001B
    D3D11_MESSAGE_ID_UPDATESUBRESOURCE1_INVALIDCOPYFLAGS = 0x30001C
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_INVALIDFORCEDSAMPLECOUNT = 0x30001D
    D3D11_MESSAGE_ID_CREATEVIDEODECODER_OUTOFMEMORY_RETURN = 0x30001E
    D3D11_MESSAGE_ID_CREATEVIDEODECODER_NULLPARAM = 0x30001F
    D3D11_MESSAGE_ID_CREATEVIDEODECODER_INVALIDFORMAT = 0x300020
    D3D11_MESSAGE_ID_CREATEVIDEODECODER_ZEROWIDTHHEIGHT = 0x300021
    D3D11_MESSAGE_ID_CREATEVIDEODECODER_DRIVER_INVALIDBUFFERSIZE = 0x300022
    D3D11_MESSAGE_ID_CREATEVIDEODECODER_DRIVER_INVALIDBUFFERUSAGE = 0x300023
    D3D11_MESSAGE_ID_GETVIDEODECODERPROFILECOUNT_OUTOFMEMORY = 0x300024
    D3D11_MESSAGE_ID_GETVIDEODECODERPROFILE_NULLPARAM = 0x300025
    D3D11_MESSAGE_ID_GETVIDEODECODERPROFILE_INVALIDINDEX = 0x300026
    D3D11_MESSAGE_ID_GETVIDEODECODERPROFILE_OUTOFMEMORY_RETURN = 0x300027
    D3D11_MESSAGE_ID_CHECKVIDEODECODERFORMAT_NULLPARAM = 0x300028
    D3D11_MESSAGE_ID_CHECKVIDEODECODERFORMAT_OUTOFMEMORY_RETURN = 0x300029
    D3D11_MESSAGE_ID_GETVIDEODECODERCONFIGCOUNT_NULLPARAM = 0x30002A
    D3D11_MESSAGE_ID_GETVIDEODECODERCONFIGCOUNT_OUTOFMEMORY_RETURN = 0x30002B
    D3D11_MESSAGE_ID_GETVIDEODECODERCONFIG_NULLPARAM = 0x30002C
    D3D11_MESSAGE_ID_GETVIDEODECODERCONFIG_INVALIDINDEX = 0x30002D
    D3D11_MESSAGE_ID_GETVIDEODECODERCONFIG_OUTOFMEMORY_RETURN = 0x30002E
    D3D11_MESSAGE_ID_GETDECODERCREATIONPARAMS_NULLPARAM = 0x30002F
    D3D11_MESSAGE_ID_GETDECODERDRIVERHANDLE_NULLPARAM = 0x300030
    D3D11_MESSAGE_ID_GETDECODERBUFFER_NULLPARAM = 0x300031
    D3D11_MESSAGE_ID_GETDECODERBUFFER_INVALIDBUFFER = 0x300032
    D3D11_MESSAGE_ID_GETDECODERBUFFER_INVALIDTYPE = 0x300033
    D3D11_MESSAGE_ID_GETDECODERBUFFER_LOCKED = 0x300034
    D3D11_MESSAGE_ID_RELEASEDECODERBUFFER_NULLPARAM = 0x300035
    D3D11_MESSAGE_ID_RELEASEDECODERBUFFER_INVALIDTYPE = 0x300036
    D3D11_MESSAGE_ID_RELEASEDECODERBUFFER_NOTLOCKED = 0x300037
    D3D11_MESSAGE_ID_DECODERBEGINFRAME_NULLPARAM = 0x300038
    D3D11_MESSAGE_ID_DECODERBEGINFRAME_HAZARD = 0x300039
    D3D11_MESSAGE_ID_DECODERENDFRAME_NULLPARAM = 0x30003A
    D3D11_MESSAGE_ID_SUBMITDECODERBUFFERS_NULLPARAM = 0x30003B
    D3D11_MESSAGE_ID_SUBMITDECODERBUFFERS_INVALIDTYPE = 0x30003C
    D3D11_MESSAGE_ID_DECODEREXTENSION_NULLPARAM = 0x30003D
    D3D11_MESSAGE_ID_DECODEREXTENSION_INVALIDRESOURCE = 0x30003E
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORENUMERATOR_OUTOFMEMORY_RETURN = 0x30003F
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORENUMERATOR_NULLPARAM = 0x300040
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORENUMERATOR_INVALIDFRAMEFORMAT = 0x300041
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORENUMERATOR_INVALIDUSAGE = 0x300042
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORENUMERATOR_INVALIDINPUTFRAMERATE = 0x300043
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORENUMERATOR_INVALIDOUTPUTFRAMERATE = 0x300044
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORENUMERATOR_INVALIDWIDTHHEIGHT = 0x300045
    D3D11_MESSAGE_ID_GETVIDEOPROCESSORCONTENTDESC_NULLPARAM = 0x300046
    D3D11_MESSAGE_ID_CHECKVIDEOPROCESSORFORMAT_NULLPARAM = 0x300047
    D3D11_MESSAGE_ID_GETVIDEOPROCESSORCAPS_NULLPARAM = 0x300048
    D3D11_MESSAGE_ID_GETVIDEOPROCESSORRATECONVERSIONCAPS_NULLPARAM = 0x300049
    D3D11_MESSAGE_ID_GETVIDEOPROCESSORRATECONVERSIONCAPS_INVALIDINDEX = 0x30004A
    D3D11_MESSAGE_ID_GETVIDEOPROCESSORCUSTOMRATE_NULLPARAM = 0x30004B
    D3D11_MESSAGE_ID_GETVIDEOPROCESSORCUSTOMRATE_INVALIDINDEX = 0x30004C
    D3D11_MESSAGE_ID_GETVIDEOPROCESSORFILTERRANGE_NULLPARAM = 0x30004D
    D3D11_MESSAGE_ID_GETVIDEOPROCESSORFILTERRANGE_UNSUPPORTED = 0x30004E
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOR_OUTOFMEMORY_RETURN = 0x30004F
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOR_NULLPARAM = 0x300050
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTTARGETRECT_NULLPARAM = 0x300051
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTBACKGROUNDCOLOR_NULLPARAM = 0x300052
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTBACKGROUNDCOLOR_INVALIDALPHA = 0x300053
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTCOLORSPACE_NULLPARAM = 0x300054
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTALPHAFILLMODE_NULLPARAM = 0x300055
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTALPHAFILLMODE_UNSUPPORTED = 0x300056
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTALPHAFILLMODE_INVALIDSTREAM = 0x300057
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTALPHAFILLMODE_INVALIDFILLMODE = 0x300058
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTCONSTRICTION_NULLPARAM = 0x300059
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTSTEREOMODE_NULLPARAM = 0x30005A
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTSTEREOMODE_UNSUPPORTED = 0x30005B
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTEXTENSION_NULLPARAM = 0x30005C
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTTARGETRECT_NULLPARAM = 0x30005D
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTBACKGROUNDCOLOR_NULLPARAM = 0x30005E
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTCOLORSPACE_NULLPARAM = 0x30005F
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTALPHAFILLMODE_NULLPARAM = 0x300060
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTCONSTRICTION_NULLPARAM = 0x300061
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTCONSTRICTION_UNSUPPORTED = 0x300062
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTCONSTRICTION_INVALIDSIZE = 0x300063
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTSTEREOMODE_NULLPARAM = 0x300064
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTEXTENSION_NULLPARAM = 0x300065
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMFRAMEFORMAT_NULLPARAM = 0x300066
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMFRAMEFORMAT_INVALIDFORMAT = 0x300067
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMFRAMEFORMAT_INVALIDSTREAM = 0x300068
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMCOLORSPACE_NULLPARAM = 0x300069
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMCOLORSPACE_INVALIDSTREAM = 0x30006A
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMOUTPUTRATE_NULLPARAM = 0x30006B
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMOUTPUTRATE_INVALIDRATE = 0x30006C
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMOUTPUTRATE_INVALIDFLAG = 0x30006D
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMOUTPUTRATE_INVALIDSTREAM = 0x30006E
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSOURCERECT_NULLPARAM = 0x30006F
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSOURCERECT_INVALIDSTREAM = 0x300070
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSOURCERECT_INVALIDRECT = 0x300071
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMDESTRECT_NULLPARAM = 0x300072
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMDESTRECT_INVALIDSTREAM = 0x300073
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMDESTRECT_INVALIDRECT = 0x300074
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMALPHA_NULLPARAM = 0x300075
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMALPHA_INVALIDSTREAM = 0x300076
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMALPHA_INVALIDALPHA = 0x300077
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMPALETTE_NULLPARAM = 0x300078
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMPALETTE_INVALIDSTREAM = 0x300079
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMPALETTE_INVALIDCOUNT = 0x30007A
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMPALETTE_INVALIDALPHA = 0x30007B
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMPIXELASPECTRATIO_NULLPARAM = 0x30007C
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMPIXELASPECTRATIO_INVALIDSTREAM = 0x30007D
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMPIXELASPECTRATIO_INVALIDRATIO = 0x30007E
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMLUMAKEY_NULLPARAM = 0x30007F
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMLUMAKEY_INVALIDSTREAM = 0x300080
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMLUMAKEY_INVALIDRANGE = 0x300081
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMLUMAKEY_UNSUPPORTED = 0x300082
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSTEREOFORMAT_NULLPARAM = 0x300083
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSTEREOFORMAT_INVALIDSTREAM = 0x300084
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSTEREOFORMAT_UNSUPPORTED = 0x300085
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSTEREOFORMAT_FLIPUNSUPPORTED = 0x300086
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSTEREOFORMAT_MONOOFFSETUNSUPPORTED = 0x300087
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSTEREOFORMAT_FORMATUNSUPPORTED = 0x300088
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMSTEREOFORMAT_INVALIDFORMAT = 0x300089
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMAUTOPROCESSINGMODE_NULLPARAM = 0x30008A
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMAUTOPROCESSINGMODE_INVALIDSTREAM = 0x30008B
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMFILTER_NULLPARAM = 0x30008C
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMFILTER_INVALIDSTREAM = 0x30008D
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMFILTER_INVALIDFILTER = 0x30008E
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMFILTER_UNSUPPORTED = 0x30008F
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMFILTER_INVALIDLEVEL = 0x300090
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMEXTENSION_NULLPARAM = 0x300091
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMEXTENSION_INVALIDSTREAM = 0x300092
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMFRAMEFORMAT_NULLPARAM = 0x300093
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMCOLORSPACE_NULLPARAM = 0x300094
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMOUTPUTRATE_NULLPARAM = 0x300095
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMSOURCERECT_NULLPARAM = 0x300096
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMDESTRECT_NULLPARAM = 0x300097
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMALPHA_NULLPARAM = 0x300098
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMPALETTE_NULLPARAM = 0x300099
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMPIXELASPECTRATIO_NULLPARAM = 0x30009A
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMLUMAKEY_NULLPARAM = 0x30009B
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMSTEREOFORMAT_NULLPARAM = 0x30009C
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMAUTOPROCESSINGMODE_NULLPARAM = 0x30009D
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMFILTER_NULLPARAM = 0x30009E
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMEXTENSION_NULLPARAM = 0x30009F
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMEXTENSION_INVALIDSTREAM = 0x3000A0
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_NULLPARAM = 0x3000A1
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDSTREAMCOUNT = 0x3000A2
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_TARGETRECT = 0x3000A3
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDOUTPUT = 0x3000A4
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDPASTFRAMES = 0x3000A5
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDFUTUREFRAMES = 0x3000A6
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDSOURCERECT = 0x3000A7
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDDESTRECT = 0x3000A8
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDINPUTRESOURCE = 0x3000A9
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDARRAYSIZE = 0x3000AA
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDARRAY = 0x3000AB
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_RIGHTEXPECTED = 0x3000AC
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_RIGHTNOTEXPECTED = 0x3000AD
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_STEREONOTENABLED = 0x3000AE
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INVALIDRIGHTRESOURCE = 0x3000AF
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_NOSTEREOSTREAMS = 0x3000B0
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_INPUTHAZARD = 0x3000B1
    D3D11_MESSAGE_ID_VIDEOPROCESSORBLT_OUTPUTHAZARD = 0x3000B2
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_OUTOFMEMORY_RETURN = 0x3000B3
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_NULLPARAM = 0x3000B4
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_INVALIDTYPE = 0x3000B5
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_INVALIDBIND = 0x3000B6
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_UNSUPPORTEDFORMAT = 0x3000B7
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_INVALIDMIP = 0x3000B8
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_UNSUPPORTEMIP = 0x3000B9
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_INVALIDARRAYSIZE = 0x3000BA
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_INVALIDARRAY = 0x3000BB
    D3D11_MESSAGE_ID_CREATEVIDEODECODEROUTPUTVIEW_INVALIDDIMENSION = 0x3000BC
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_OUTOFMEMORY_RETURN = 0x3000BD
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_NULLPARAM = 0x3000BE
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDTYPE = 0x3000BF
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDBIND = 0x3000C0
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDMISC = 0x3000C1
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDUSAGE = 0x3000C2
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDFORMAT = 0x3000C3
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDFOURCC = 0x3000C4
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDMIP = 0x3000C5
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_UNSUPPORTEDMIP = 0x3000C6
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDARRAYSIZE = 0x3000C7
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDARRAY = 0x3000C8
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDDIMENSION = 0x3000C9
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_OUTOFMEMORY_RETURN = 0x3000CA
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_NULLPARAM = 0x3000CB
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_INVALIDTYPE = 0x3000CC
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_INVALIDBIND = 0x3000CD
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_INVALIDFORMAT = 0x3000CE
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_INVALIDMIP = 0x3000CF
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_UNSUPPORTEDMIP = 0x3000D0
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_UNSUPPORTEDARRAY = 0x3000D1
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_INVALIDARRAY = 0x3000D2
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_INVALIDDIMENSION = 0x3000D3
    D3D11_MESSAGE_ID_DEVICE_DRAW_INVALID_USE_OF_FORCED_SAMPLE_COUNT = 0x3000D4
    D3D11_MESSAGE_ID_CREATEBLENDSTATE_INVALIDLOGICOPS = 0x3000D5
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDDARRAYWITHDECODER = 0x3000D6
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDDARRAYWITHDECODER = 0x3000D7
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDDARRAYWITHDECODER = 0x3000D8
    D3D11_MESSAGE_ID_DEVICE_LOCKEDOUT_INTERFACE = 0x3000D9
    D3D11_MESSAGE_ID_REF_WARNING_ATOMIC_INCONSISTENT = 0x3000DA
    D3D11_MESSAGE_ID_REF_WARNING_READING_UNINITIALIZED_RESOURCE = 0x3000DB
    D3D11_MESSAGE_ID_REF_WARNING_RAW_HAZARD = 0x3000DC
    D3D11_MESSAGE_ID_REF_WARNING_WAR_HAZARD = 0x3000DD
    D3D11_MESSAGE_ID_REF_WARNING_WAW_HAZARD = 0x3000DE
    D3D11_MESSAGE_ID_CREATECRYPTOSESSION_NULLPARAM = 0x3000DF
    D3D11_MESSAGE_ID_CREATECRYPTOSESSION_OUTOFMEMORY_RETURN = 0x3000E0
    D3D11_MESSAGE_ID_GETCRYPTOTYPE_NULLPARAM = 0x3000E1
    D3D11_MESSAGE_ID_GETDECODERPROFILE_NULLPARAM = 0x3000E2
    D3D11_MESSAGE_ID_GETCRYPTOSESSIONCERTIFICATESIZE_NULLPARAM = 0x3000E3
    D3D11_MESSAGE_ID_GETCRYPTOSESSIONCERTIFICATE_NULLPARAM = 0x3000E4
    D3D11_MESSAGE_ID_GETCRYPTOSESSIONCERTIFICATE_WRONGSIZE = 0x3000E5
    D3D11_MESSAGE_ID_GETCRYPTOSESSIONHANDLE_WRONGSIZE = 0x3000E6
    D3D11_MESSAGE_ID_NEGOTIATECRPYTOSESSIONKEYEXCHANGE_NULLPARAM = 0x3000E7
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_UNSUPPORTED = 0x3000E8
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_NULLPARAM = 0x3000E9
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_SRC_WRONGDEVICE = 0x3000EA
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_DST_WRONGDEVICE = 0x3000EB
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_FORMAT_MISMATCH = 0x3000EC
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_SIZE_MISMATCH = 0x3000ED
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_SRC_MULTISAMPLED = 0x3000EE
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_DST_NOT_STAGING = 0x3000EF
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_SRC_MAPPED = 0x3000F0
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_DST_MAPPED = 0x3000F1
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_SRC_OFFERED = 0x3000F2
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_DST_OFFERED = 0x3000F3
    D3D11_MESSAGE_ID_ENCRYPTIONBLT_SRC_CONTENT_UNDEFINED = 0x3000F4
    D3D11_MESSAGE_ID_DECRYPTIONBLT_UNSUPPORTED = 0x3000F5
    D3D11_MESSAGE_ID_DECRYPTIONBLT_NULLPARAM = 0x3000F6
    D3D11_MESSAGE_ID_DECRYPTIONBLT_SRC_WRONGDEVICE = 0x3000F7
    D3D11_MESSAGE_ID_DECRYPTIONBLT_DST_WRONGDEVICE = 0x3000F8
    D3D11_MESSAGE_ID_DECRYPTIONBLT_FORMAT_MISMATCH = 0x3000F9
    D3D11_MESSAGE_ID_DECRYPTIONBLT_SIZE_MISMATCH = 0x3000FA
    D3D11_MESSAGE_ID_DECRYPTIONBLT_DST_MULTISAMPLED = 0x3000FB
    D3D11_MESSAGE_ID_DECRYPTIONBLT_SRC_NOT_STAGING = 0x3000FC
    D3D11_MESSAGE_ID_DECRYPTIONBLT_DST_NOT_RENDER_TARGET = 0x3000FD
    D3D11_MESSAGE_ID_DECRYPTIONBLT_SRC_MAPPED = 0x3000FE
    D3D11_MESSAGE_ID_DECRYPTIONBLT_DST_MAPPED = 0x3000FF
    D3D11_MESSAGE_ID_DECRYPTIONBLT_SRC_OFFERED = 0x300100
    D3D11_MESSAGE_ID_DECRYPTIONBLT_DST_OFFERED = 0x300101
    D3D11_MESSAGE_ID_DECRYPTIONBLT_SRC_CONTENT_UNDEFINED = 0x300102
    D3D11_MESSAGE_ID_STARTSESSIONKEYREFRESH_NULLPARAM = 0x300103
    D3D11_MESSAGE_ID_STARTSESSIONKEYREFRESH_INVALIDSIZE = 0x300104
    D3D11_MESSAGE_ID_FINISHSESSIONKEYREFRESH_NULLPARAM = 0x300105
    D3D11_MESSAGE_ID_GETENCRYPTIONBLTKEY_NULLPARAM = 0x300106
    D3D11_MESSAGE_ID_GETENCRYPTIONBLTKEY_INVALIDSIZE = 0x300107
    D3D11_MESSAGE_ID_GETCONTENTPROTECTIONCAPS_NULLPARAM = 0x300108
    D3D11_MESSAGE_ID_CHECKCRYPTOKEYEXCHANGE_NULLPARAM = 0x300109
    D3D11_MESSAGE_ID_CHECKCRYPTOKEYEXCHANGE_INVALIDINDEX = 0x30010A
    D3D11_MESSAGE_ID_CREATEAUTHENTICATEDCHANNEL_NULLPARAM = 0x30010B
    D3D11_MESSAGE_ID_CREATEAUTHENTICATEDCHANNEL_UNSUPPORTED = 0x30010C
    D3D11_MESSAGE_ID_CREATEAUTHENTICATEDCHANNEL_INVALIDTYPE = 0x30010D
    D3D11_MESSAGE_ID_CREATEAUTHENTICATEDCHANNEL_OUTOFMEMORY_RETURN = 0x30010E
    D3D11_MESSAGE_ID_GETAUTHENTICATEDCHANNELCERTIFICATESIZE_INVALIDCHANNEL = 0x30010F
    D3D11_MESSAGE_ID_GETAUTHENTICATEDCHANNELCERTIFICATESIZE_NULLPARAM = 0x300110
    D3D11_MESSAGE_ID_GETAUTHENTICATEDCHANNELCERTIFICATE_INVALIDCHANNEL = 0x300111
    D3D11_MESSAGE_ID_GETAUTHENTICATEDCHANNELCERTIFICATE_NULLPARAM = 0x300112
    D3D11_MESSAGE_ID_GETAUTHENTICATEDCHANNELCERTIFICATE_WRONGSIZE = 0x300113
    D3D11_MESSAGE_ID_NEGOTIATEAUTHENTICATEDCHANNELKEYEXCHANGE_INVALIDCHANNEL = 0x300114
    D3D11_MESSAGE_ID_NEGOTIATEAUTHENTICATEDCHANNELKEYEXCHANGE_NULLPARAM = 0x300115
    D3D11_MESSAGE_ID_QUERYAUTHENTICATEDCHANNEL_NULLPARAM = 0x300116
    D3D11_MESSAGE_ID_QUERYAUTHENTICATEDCHANNEL_WRONGCHANNEL = 0x300117
    D3D11_MESSAGE_ID_QUERYAUTHENTICATEDCHANNEL_UNSUPPORTEDQUERY = 0x300118
    D3D11_MESSAGE_ID_QUERYAUTHENTICATEDCHANNEL_WRONGSIZE = 0x300119
    D3D11_MESSAGE_ID_QUERYAUTHENTICATEDCHANNEL_INVALIDPROCESSINDEX = 0x30011A
    D3D11_MESSAGE_ID_CONFIGUREAUTHENTICATEDCHANNEL_NULLPARAM = 0x30011B
    D3D11_MESSAGE_ID_CONFIGUREAUTHENTICATEDCHANNEL_WRONGCHANNEL = 0x30011C
    D3D11_MESSAGE_ID_CONFIGUREAUTHENTICATEDCHANNEL_UNSUPPORTEDCONFIGURE = 0x30011D
    D3D11_MESSAGE_ID_CONFIGUREAUTHENTICATEDCHANNEL_WRONGSIZE = 0x30011E
    D3D11_MESSAGE_ID_CONFIGUREAUTHENTICATEDCHANNEL_INVALIDPROCESSIDTYPE = 0x30011F
    D3D11_MESSAGE_ID_VSSETCONSTANTBUFFERS_INVALIDBUFFEROFFSETORCOUNT = 0x300120
    D3D11_MESSAGE_ID_DSSETCONSTANTBUFFERS_INVALIDBUFFEROFFSETORCOUNT = 0x300121
    D3D11_MESSAGE_ID_HSSETCONSTANTBUFFERS_INVALIDBUFFEROFFSETORCOUNT = 0x300122
    D3D11_MESSAGE_ID_GSSETCONSTANTBUFFERS_INVALIDBUFFEROFFSETORCOUNT = 0x300123
    D3D11_MESSAGE_ID_PSSETCONSTANTBUFFERS_INVALIDBUFFEROFFSETORCOUNT = 0x300124
    D3D11_MESSAGE_ID_CSSETCONSTANTBUFFERS_INVALIDBUFFEROFFSETORCOUNT = 0x300125
    D3D11_MESSAGE_ID_NEGOTIATECRPYTOSESSIONKEYEXCHANGE_INVALIDSIZE = 0x300126
    D3D11_MESSAGE_ID_NEGOTIATEAUTHENTICATEDCHANNELKEYEXCHANGE_INVALIDSIZE = 0x300127
    D3D11_MESSAGE_ID_OFFERRESOURCES_INVALIDPRIORITY = 0x300128
    D3D11_MESSAGE_ID_GETCRYPTOSESSIONHANDLE_OUTOFMEMORY = 0x300129
    D3D11_MESSAGE_ID_ACQUIREHANDLEFORCAPTURE_NULLPARAM = 0x30012A
    D3D11_MESSAGE_ID_ACQUIREHANDLEFORCAPTURE_INVALIDTYPE = 0x30012B
    D3D11_MESSAGE_ID_ACQUIREHANDLEFORCAPTURE_INVALIDBIND = 0x30012C
    D3D11_MESSAGE_ID_ACQUIREHANDLEFORCAPTURE_INVALIDARRAY = 0x30012D
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMROTATION_NULLPARAM = 0x30012E
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMROTATION_INVALIDSTREAM = 0x30012F
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMROTATION_INVALID = 0x300130
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMROTATION_UNSUPPORTED = 0x300131
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMROTATION_NULLPARAM = 0x300132
    D3D11_MESSAGE_ID_DEVICE_CLEARVIEW_INVALIDVIEW = 0x300133
    D3D11_MESSAGE_ID_DEVICE_CREATEVERTEXSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x300134
    D3D11_MESSAGE_ID_DEVICE_CREATEVERTEXSHADER_SHADEREXTENSIONSNOTSUPPORTED = 0x300135
    D3D11_MESSAGE_ID_DEVICE_CREATEHULLSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x300136
    D3D11_MESSAGE_ID_DEVICE_CREATEHULLSHADER_SHADEREXTENSIONSNOTSUPPORTED = 0x300137
    D3D11_MESSAGE_ID_DEVICE_CREATEDOMAINSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x300138
    D3D11_MESSAGE_ID_DEVICE_CREATEDOMAINSHADER_SHADEREXTENSIONSNOTSUPPORTED = 0x300139
    D3D11_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x30013A
    D3D11_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADER_SHADEREXTENSIONSNOTSUPPORTED = 0x30013B
    D3D11_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_DOUBLEEXTENSIONSNOTSUPPORTED = 0x30013C
    D3D11_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_SHADEREXTENSIONSNOTSUPPORTED = 0x30013D
    D3D11_MESSAGE_ID_DEVICE_CREATEPIXELSHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x30013E
    D3D11_MESSAGE_ID_DEVICE_CREATEPIXELSHADER_SHADEREXTENSIONSNOTSUPPORTED = 0x30013F
    D3D11_MESSAGE_ID_DEVICE_CREATECOMPUTESHADER_DOUBLEEXTENSIONSNOTSUPPORTED = 0x300140
    D3D11_MESSAGE_ID_DEVICE_CREATECOMPUTESHADER_SHADEREXTENSIONSNOTSUPPORTED = 0x300141
    D3D11_MESSAGE_ID_DEVICE_SHADER_LINKAGE_MINPRECISION = 0x300142
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMALPHA_UNSUPPORTED = 0x300143
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMPIXELASPECTRATIO_UNSUPPORTED = 0x300144
    D3D11_MESSAGE_ID_DEVICE_CREATEVERTEXSHADER_UAVSNOTSUPPORTED = 0x300145
    D3D11_MESSAGE_ID_DEVICE_CREATEHULLSHADER_UAVSNOTSUPPORTED = 0x300146
    D3D11_MESSAGE_ID_DEVICE_CREATEDOMAINSHADER_UAVSNOTSUPPORTED = 0x300147
    D3D11_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADER_UAVSNOTSUPPORTED = 0x300148
    D3D11_MESSAGE_ID_DEVICE_CREATEGEOMETRYSHADERWITHSTREAMOUTPUT_UAVSNOTSUPPORTED = 0x300149
    D3D11_MESSAGE_ID_DEVICE_CREATEPIXELSHADER_UAVSNOTSUPPORTED = 0x30014A
    D3D11_MESSAGE_ID_DEVICE_CREATECOMPUTESHADER_UAVSNOTSUPPORTED = 0x30014B
    D3D11_MESSAGE_ID_DEVICE_OMSETRENDERTARGETSANDUNORDEREDACCESSVIEWS_INVALIDOFFSET = 0x30014C
    D3D11_MESSAGE_ID_DEVICE_OMSETRENDERTARGETSANDUNORDEREDACCESSVIEWS_TOOMANYVIEWS = 0x30014D
    D3D11_MESSAGE_ID_DEVICE_CLEARVIEW_NOTSUPPORTED = 0x30014E
    D3D11_MESSAGE_ID_SWAPDEVICECONTEXTSTATE_NOTSUPPORTED = 0x30014F
    D3D11_MESSAGE_ID_UPDATESUBRESOURCE_PREFERUPDATESUBRESOURCE1 = 0x300150
    D3D11_MESSAGE_ID_GETDC_INACCESSIBLE = 0x300151
    D3D11_MESSAGE_ID_DEVICE_CLEARVIEW_INVALIDRECT = 0x300152
    D3D11_MESSAGE_ID_DEVICE_DRAW_SAMPLE_MASK_IGNORED_ON_FL9 = 0x300153
    D3D11_MESSAGE_ID_DEVICE_OPEN_SHARED_RESOURCE1_NOT_SUPPORTED = 0x300154
    D3D11_MESSAGE_ID_DEVICE_OPEN_SHARED_RESOURCE_BY_NAME_NOT_SUPPORTED = 0x300155
    D3D11_MESSAGE_ID_ENQUEUESETEVENT_NOT_SUPPORTED = 0x300156
    D3D11_MESSAGE_ID_OFFERRELEASE_NOT_SUPPORTED = 0x300157
    D3D11_MESSAGE_ID_OFFERRESOURCES_INACCESSIBLE = 0x300158
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSORINPUTVIEW_INVALIDMSAA = 0x300159
    D3D11_MESSAGE_ID_CREATEVIDEOPROCESSOROUTPUTVIEW_INVALIDMSAA = 0x30015A
    D3D11_MESSAGE_ID_DEVICE_CLEARVIEW_INVALIDSOURCERECT = 0x30015B
    D3D11_MESSAGE_ID_DEVICE_CLEARVIEW_EMPTYRECT = 0x30015C
    D3D11_MESSAGE_ID_UPDATESUBRESOURCE_EMPTYDESTBOX = 0x30015D
    D3D11_MESSAGE_ID_COPYSUBRESOURCEREGION_EMPTYSOURCEBOX = 0x30015E
    D3D11_MESSAGE_ID_DEVICE_DRAW_OM_RENDER_TARGET_DOES_NOT_SUPPORT_LOGIC_OPS = 0x30015F
    D3D11_MESSAGE_ID_DEVICE_DRAW_DEPTHSTENCILVIEW_NOT_SET = 0x300160
    D3D11_MESSAGE_ID_DEVICE_DRAW_RENDERTARGETVIEW_NOT_SET = 0x300161
    D3D11_MESSAGE_ID_DEVICE_DRAW_RENDERTARGETVIEW_NOT_SET_DUE_TO_FLIP_PRESENT = 0x300162
    D3D11_MESSAGE_ID_DEVICE_UNORDEREDACCESSVIEW_NOT_SET_DUE_TO_FLIP_PRESENT = 0x300163
    D3D11_MESSAGE_ID_GETDATAFORNEWHARDWAREKEY_NULLPARAM = 0x300164
    D3D11_MESSAGE_ID_CHECKCRYPTOSESSIONSTATUS_NULLPARAM = 0x300165
    D3D11_MESSAGE_ID_GETCRYPTOSESSIONPRIVATEDATASIZE_NULLPARAM = 0x300166
    D3D11_MESSAGE_ID_GETVIDEODECODERCAPS_NULLPARAM = 0x300167
    D3D11_MESSAGE_ID_GETVIDEODECODERCAPS_ZEROWIDTHHEIGHT = 0x300168
    D3D11_MESSAGE_ID_CHECKVIDEODECODERDOWNSAMPLING_NULLPARAM = 0x300169
    D3D11_MESSAGE_ID_CHECKVIDEODECODERDOWNSAMPLING_INVALIDCOLORSPACE = 0x30016A
    D3D11_MESSAGE_ID_CHECKVIDEODECODERDOWNSAMPLING_ZEROWIDTHHEIGHT = 0x30016B
    D3D11_MESSAGE_ID_VIDEODECODERENABLEDOWNSAMPLING_NULLPARAM = 0x30016C
    D3D11_MESSAGE_ID_VIDEODECODERENABLEDOWNSAMPLING_UNSUPPORTED = 0x30016D
    D3D11_MESSAGE_ID_VIDEODECODERUPDATEDOWNSAMPLING_NULLPARAM = 0x30016E
    D3D11_MESSAGE_ID_VIDEODECODERUPDATEDOWNSAMPLING_UNSUPPORTED = 0x30016F
    D3D11_MESSAGE_ID_CHECKVIDEOPROCESSORFORMATCONVERSION_NULLPARAM = 0x300170
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTCOLORSPACE1_NULLPARAM = 0x300171
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTCOLORSPACE1_NULLPARAM = 0x300172
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMCOLORSPACE1_NULLPARAM = 0x300173
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMCOLORSPACE1_INVALIDSTREAM = 0x300174
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMMIRROR_NULLPARAM = 0x300175
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMMIRROR_INVALIDSTREAM = 0x300176
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMMIRROR_UNSUPPORTED = 0x300177
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMCOLORSPACE1_NULLPARAM = 0x300178
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMMIRROR_NULLPARAM = 0x300179
    D3D11_MESSAGE_ID_RECOMMENDVIDEODECODERDOWNSAMPLING_NULLPARAM = 0x30017A
    D3D11_MESSAGE_ID_RECOMMENDVIDEODECODERDOWNSAMPLING_INVALIDCOLORSPACE = 0x30017B
    D3D11_MESSAGE_ID_RECOMMENDVIDEODECODERDOWNSAMPLING_ZEROWIDTHHEIGHT = 0x30017C
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTSHADERUSAGE_NULLPARAM = 0x30017D
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTSHADERUSAGE_NULLPARAM = 0x30017E
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETBEHAVIORHINTS_NULLPARAM = 0x30017F
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETBEHAVIORHINTS_INVALIDSTREAMCOUNT = 0x300180
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETBEHAVIORHINTS_TARGETRECT = 0x300181
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETBEHAVIORHINTS_INVALIDSOURCERECT = 0x300182
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETBEHAVIORHINTS_INVALIDDESTRECT = 0x300183
    D3D11_MESSAGE_ID_GETCRYPTOSESSIONPRIVATEDATASIZE_INVALID_KEY_EXCHANGE_TYPE = 0x300184
    D3D11_MESSAGE_ID_D3D11_1_MESSAGES_END = 0x300185
    D3D11_MESSAGE_ID_D3D11_2_MESSAGES_START = 0x300186
    D3D11_MESSAGE_ID_CREATEBUFFER_INVALIDUSAGE = 0x300187
    D3D11_MESSAGE_ID_CREATETEXTURE1D_INVALIDUSAGE = 0x300188
    D3D11_MESSAGE_ID_CREATETEXTURE2D_INVALIDUSAGE = 0x300189
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_LEVEL9_STEPRATE_NOT_1 = 0x30018A
    D3D11_MESSAGE_ID_CREATEINPUTLAYOUT_LEVEL9_INSTANCING_NOT_SUPPORTED = 0x30018B
    D3D11_MESSAGE_ID_UPDATETILEMAPPINGS_INVALID_PARAMETER = 0x30018C
    D3D11_MESSAGE_ID_COPYTILEMAPPINGS_INVALID_PARAMETER = 0x30018D
    D3D11_MESSAGE_ID_COPYTILES_INVALID_PARAMETER = 0x30018E
    D3D11_MESSAGE_ID_UPDATETILES_INVALID_PARAMETER = 0x30018F
    D3D11_MESSAGE_ID_RESIZETILEPOOL_INVALID_PARAMETER = 0x300190
    D3D11_MESSAGE_ID_TILEDRESOURCEBARRIER_INVALID_PARAMETER = 0x300191
    D3D11_MESSAGE_ID_NULL_TILE_MAPPING_ACCESS_WARNING = 0x300192
    D3D11_MESSAGE_ID_NULL_TILE_MAPPING_ACCESS_ERROR = 0x300193
    D3D11_MESSAGE_ID_DIRTY_TILE_MAPPING_ACCESS = 0x300194
    D3D11_MESSAGE_ID_DUPLICATE_TILE_MAPPINGS_IN_COVERED_AREA = 0x300195
    D3D11_MESSAGE_ID_TILE_MAPPINGS_IN_COVERED_AREA_DUPLICATED_OUTSIDE = 0x300196
    D3D11_MESSAGE_ID_TILE_MAPPINGS_SHARED_BETWEEN_INCOMPATIBLE_RESOURCES = 0x300197
    D3D11_MESSAGE_ID_TILE_MAPPINGS_SHARED_BETWEEN_INPUT_AND_OUTPUT = 0x300198
    D3D11_MESSAGE_ID_CHECKMULTISAMPLEQUALITYLEVELS_INVALIDFLAGS = 0x300199
    D3D11_MESSAGE_ID_GETRESOURCETILING_NONTILED_RESOURCE = 0x30019A
    D3D11_MESSAGE_ID_RESIZETILEPOOL_SHRINK_WITH_MAPPINGS_STILL_DEFINED_PAST_END = 0x30019B
    D3D11_MESSAGE_ID_NEED_TO_CALL_TILEDRESOURCEBARRIER = 0x30019C
    D3D11_MESSAGE_ID_CREATEDEVICE_INVALIDARGS = 0x30019D
    D3D11_MESSAGE_ID_CREATEDEVICE_WARNING = 0x30019E
    D3D11_MESSAGE_ID_CLEARUNORDEREDACCESSVIEWUINT_HAZARD = 0x30019F
    D3D11_MESSAGE_ID_CLEARUNORDEREDACCESSVIEWFLOAT_HAZARD = 0x3001A0
    D3D11_MESSAGE_ID_TILED_RESOURCE_TIER_1_BUFFER_TEXTURE_MISMATCH = 0x3001A1
    D3D11_MESSAGE_ID_CREATE_CRYPTOSESSION = 0x3001A2
    D3D11_MESSAGE_ID_CREATE_AUTHENTICATEDCHANNEL = 0x3001A3
    D3D11_MESSAGE_ID_LIVE_CRYPTOSESSION = 0x3001A4
    D3D11_MESSAGE_ID_LIVE_AUTHENTICATEDCHANNEL = 0x3001A5
    D3D11_MESSAGE_ID_DESTROY_CRYPTOSESSION = 0x3001A6
    D3D11_MESSAGE_ID_DESTROY_AUTHENTICATEDCHANNEL = 0x3001A7
    D3D11_MESSAGE_ID_D3D11_2_MESSAGES_END = 0x3001A8
    D3D11_MESSAGE_ID_D3D11_3_MESSAGES_START = 0x3001A9
    D3D11_MESSAGE_ID_CREATERASTERIZERSTATE_INVALID_CONSERVATIVERASTERMODE = 0x3001AA
    D3D11_MESSAGE_ID_DEVICE_DRAW_INVALID_SYSTEMVALUE = 0x3001AB
    D3D11_MESSAGE_ID_CREATEQUERYORPREDICATE_INVALIDCONTEXTTYPE = 0x3001AC
    D3D11_MESSAGE_ID_CREATEQUERYORPREDICATE_DECODENOTSUPPORTED = 0x3001AD
    D3D11_MESSAGE_ID_CREATEQUERYORPREDICATE_ENCODENOTSUPPORTED = 0x3001AE
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDPLANEINDEX = 0x3001AF
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_INVALIDVIDEOPLANEINDEX = 0x3001B0
    D3D11_MESSAGE_ID_CREATESHADERRESOURCEVIEW_AMBIGUOUSVIDEOPLANEINDEX = 0x3001B1
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDPLANEINDEX = 0x3001B2
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_INVALIDVIDEOPLANEINDEX = 0x3001B3
    D3D11_MESSAGE_ID_CREATERENDERTARGETVIEW_AMBIGUOUSVIDEOPLANEINDEX = 0x3001B4
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDPLANEINDEX = 0x3001B5
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_INVALIDVIDEOPLANEINDEX = 0x3001B6
    D3D11_MESSAGE_ID_CREATEUNORDEREDACCESSVIEW_AMBIGUOUSVIDEOPLANEINDEX = 0x3001B7
    D3D11_MESSAGE_ID_JPEGDECODE_INVALIDSCANDATAOFFSET = 0x3001B8
    D3D11_MESSAGE_ID_JPEGDECODE_NOTSUPPORTED = 0x3001B9
    D3D11_MESSAGE_ID_JPEGDECODE_DIMENSIONSTOOLARGE = 0x3001BA
    D3D11_MESSAGE_ID_JPEGDECODE_INVALIDCOMPONENTS = 0x3001BB
    D3D11_MESSAGE_ID_JPEGDECODE_DESTINATIONNOT2D = 0x3001BC
    D3D11_MESSAGE_ID_JPEGDECODE_TILEDRESOURCESUNSUPPORTED = 0x3001BD
    D3D11_MESSAGE_ID_JPEGDECODE_GUARDRECTSUNSUPPORTED = 0x3001BE
    D3D11_MESSAGE_ID_JPEGDECODE_FORMATUNSUPPORTED = 0x3001BF
    D3D11_MESSAGE_ID_JPEGDECODE_INVALIDSUBRESOURCE = 0x3001C0
    D3D11_MESSAGE_ID_JPEGDECODE_INVALIDMIPLEVEL = 0x3001C1
    D3D11_MESSAGE_ID_JPEGDECODE_EMPTYDESTBOX = 0x3001C2
    D3D11_MESSAGE_ID_JPEGDECODE_DESTBOXNOT2D = 0x3001C3
    D3D11_MESSAGE_ID_JPEGDECODE_DESTBOXNOTSUB = 0x3001C4
    D3D11_MESSAGE_ID_JPEGDECODE_DESTBOXESINTERSECT = 0x3001C5
    D3D11_MESSAGE_ID_JPEGDECODE_XSUBSAMPLEMISMATCH = 0x3001C6
    D3D11_MESSAGE_ID_JPEGDECODE_YSUBSAMPLEMISMATCH = 0x3001C7
    D3D11_MESSAGE_ID_JPEGDECODE_XSUBSAMPLEODD = 0x3001C8
    D3D11_MESSAGE_ID_JPEGDECODE_YSUBSAMPLEODD = 0x3001C9
    D3D11_MESSAGE_ID_JPEGDECODE_OUTPUTDIMENSIONSTOOLARGE = 0x3001CA
    D3D11_MESSAGE_ID_JPEGDECODE_NONPOW2SCALEUNSUPPORTED = 0x3001CB
    D3D11_MESSAGE_ID_JPEGDECODE_FRACTIONALDOWNSCALETOLARGE = 0x3001CC
    D3D11_MESSAGE_ID_JPEGDECODE_CHROMASIZEMISMATCH = 0x3001CD
    D3D11_MESSAGE_ID_JPEGDECODE_LUMACHROMASIZEMISMATCH = 0x3001CE
    D3D11_MESSAGE_ID_JPEGDECODE_INVALIDNUMDESTINATIONS = 0x3001CF
    D3D11_MESSAGE_ID_JPEGDECODE_SUBBOXUNSUPPORTED = 0x3001D0
    D3D11_MESSAGE_ID_JPEGDECODE_1DESTUNSUPPORTEDFORMAT = 0x3001D1
    D3D11_MESSAGE_ID_JPEGDECODE_3DESTUNSUPPORTEDFORMAT = 0x3001D2
    D3D11_MESSAGE_ID_JPEGDECODE_SCALEUNSUPPORTED = 0x3001D3
    D3D11_MESSAGE_ID_JPEGDECODE_INVALIDSOURCESIZE = 0x3001D4
    D3D11_MESSAGE_ID_JPEGDECODE_INVALIDCOPYFLAGS = 0x3001D5
    D3D11_MESSAGE_ID_JPEGDECODE_HAZARD = 0x3001D6
    D3D11_MESSAGE_ID_JPEGDECODE_UNSUPPORTEDSRCBUFFERUSAGE = 0x3001D7
    D3D11_MESSAGE_ID_JPEGDECODE_UNSUPPORTEDSRCBUFFERMISCFLAGS = 0x3001D8
    D3D11_MESSAGE_ID_JPEGDECODE_UNSUPPORTEDDSTTEXTUREUSAGE = 0x3001D9
    D3D11_MESSAGE_ID_JPEGDECODE_BACKBUFFERNOTSUPPORTED = 0x3001DA
    D3D11_MESSAGE_ID_JPEGDECODE_UNSUPPRTEDCOPYFLAGS = 0x3001DB
    D3D11_MESSAGE_ID_JPEGENCODE_NOTSUPPORTED = 0x3001DC
    D3D11_MESSAGE_ID_JPEGENCODE_INVALIDSCANDATAOFFSET = 0x3001DD
    D3D11_MESSAGE_ID_JPEGENCODE_INVALIDCOMPONENTS = 0x3001DE
    D3D11_MESSAGE_ID_JPEGENCODE_SOURCENOT2D = 0x3001DF
    D3D11_MESSAGE_ID_JPEGENCODE_TILEDRESOURCESUNSUPPORTED = 0x3001E0
    D3D11_MESSAGE_ID_JPEGENCODE_GUARDRECTSUNSUPPORTED = 0x3001E1
    D3D11_MESSAGE_ID_JPEGENCODE_XSUBSAMPLEMISMATCH = 0x3001E2
    D3D11_MESSAGE_ID_JPEGENCODE_YSUBSAMPLEMISMATCH = 0x3001E3
    D3D11_MESSAGE_ID_JPEGENCODE_FORMATUNSUPPORTED = 0x3001E4
    D3D11_MESSAGE_ID_JPEGENCODE_INVALIDSUBRESOURCE = 0x3001E5
    D3D11_MESSAGE_ID_JPEGENCODE_INVALIDMIPLEVEL = 0x3001E6
    D3D11_MESSAGE_ID_JPEGENCODE_DIMENSIONSTOOLARGE = 0x3001E7
    D3D11_MESSAGE_ID_JPEGENCODE_HAZARD = 0x3001E8
    D3D11_MESSAGE_ID_JPEGENCODE_UNSUPPORTEDDSTBUFFERUSAGE = 0x3001E9
    D3D11_MESSAGE_ID_JPEGENCODE_UNSUPPORTEDDSTBUFFERMISCFLAGS = 0x3001EA
    D3D11_MESSAGE_ID_JPEGENCODE_UNSUPPORTEDSRCTEXTUREUSAGE = 0x3001EB
    D3D11_MESSAGE_ID_JPEGENCODE_BACKBUFFERNOTSUPPORTED = 0x3001EC
    D3D11_MESSAGE_ID_CREATEQUERYORPREDICATE_UNSUPPORTEDCONTEXTTTYPEFORQUERY = 0x3001ED
    D3D11_MESSAGE_ID_FLUSH1_INVALIDCONTEXTTYPE = 0x3001EE
    D3D11_MESSAGE_ID_DEVICE_SETHARDWAREPROTECTION_INVALIDCONTEXT = 0x3001EF
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTHDRMETADATA_NULLPARAM = 0x3001F0
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETOUTPUTHDRMETADATA_INVALIDSIZE = 0x3001F1
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTHDRMETADATA_NULLPARAM = 0x3001F2
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETOUTPUTHDRMETADATA_INVALIDSIZE = 0x3001F3
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMHDRMETADATA_NULLPARAM = 0x3001F4
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMHDRMETADATA_INVALIDSTREAM = 0x3001F5
    D3D11_MESSAGE_ID_VIDEOPROCESSORSETSTREAMHDRMETADATA_INVALIDSIZE = 0x3001F6
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMHDRMETADATA_NULLPARAM = 0x3001F7
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMHDRMETADATA_INVALIDSTREAM = 0x3001F8
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMHDRMETADATA_INVALIDSIZE = 0x3001F9
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMFRAMEFORMAT_INVALIDSTREAM = 0x3001FA
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMCOLORSPACE_INVALIDSTREAM = 0x3001FB
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMOUTPUTRATE_INVALIDSTREAM = 0x3001FC
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMSOURCERECT_INVALIDSTREAM = 0x3001FD
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMDESTRECT_INVALIDSTREAM = 0x3001FE
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMALPHA_INVALIDSTREAM = 0x3001FF
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMPALETTE_INVALIDSTREAM = 0x300200
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMPIXELASPECTRATIO_INVALIDSTREAM = 0x300201
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMLUMAKEY_INVALIDSTREAM = 0x300202
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMSTEREOFORMAT_INVALIDSTREAM = 0x300203
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMAUTOPROCESSINGMODE_INVALIDSTREAM = 0x300204
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMFILTER_INVALIDSTREAM = 0x300205
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMROTATION_INVALIDSTREAM = 0x300206
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMCOLORSPACE1_INVALIDSTREAM = 0x300207
    D3D11_MESSAGE_ID_VIDEOPROCESSORGETSTREAMMIRROR_INVALIDSTREAM = 0x300208
    D3D11_MESSAGE_ID_CREATE_FENCE = 0x300209
    D3D11_MESSAGE_ID_LIVE_FENCE = 0x30020A
    D3D11_MESSAGE_ID_DESTROY_FENCE = 0x30020B
    D3D11_MESSAGE_ID_CREATE_SYNCHRONIZEDCHANNEL = 0x30020C
    D3D11_MESSAGE_ID_LIVE_SYNCHRONIZEDCHANNEL = 0x30020D
    D3D11_MESSAGE_ID_DESTROY_SYNCHRONIZEDCHANNEL = 0x30020E
    D3D11_MESSAGE_ID_CREATEFENCE_INVALIDFLAGS = 0x30020F
    D3D11_MESSAGE_ID_D3D11_3_MESSAGES_END = 0x300210
    D3D11_MESSAGE_ID_D3D11_5_MESSAGES_START = 0x300211
    D3D11_MESSAGE_ID_NEGOTIATECRYPTOSESSIONKEYEXCHANGEMT_INVALIDKEYEXCHANGETYPE = 0x300212
    D3D11_MESSAGE_ID_NEGOTIATECRYPTOSESSIONKEYEXCHANGEMT_NOT_SUPPORTED = 0x300213
    D3D11_MESSAGE_ID_DECODERBEGINFRAME_INVALID_HISTOGRAM_COMPONENT_COUNT = 0x300214
    D3D11_MESSAGE_ID_DECODERBEGINFRAME_INVALID_HISTOGRAM_COMPONENT = 0x300215
    D3D11_MESSAGE_ID_DECODERBEGINFRAME_INVALID_HISTOGRAM_BUFFER_SIZE = 0x300216
    D3D11_MESSAGE_ID_DECODERBEGINFRAME_INVALID_HISTOGRAM_BUFFER_USAGE = 0x300217
    D3D11_MESSAGE_ID_DECODERBEGINFRAME_INVALID_HISTOGRAM_BUFFER_MISC_FLAGS = 0x300218
    D3D11_MESSAGE_ID_DECODERBEGINFRAME_INVALID_HISTOGRAM_BUFFER_OFFSET = 0x300219
    D3D11_MESSAGE_ID_CREATE_TRACKEDWORKLOAD = 0x30021A
    D3D11_MESSAGE_ID_LIVE_TRACKEDWORKLOAD = 0x30021B
    D3D11_MESSAGE_ID_DESTROY_TRACKEDWORKLOAD = 0x30021C
    D3D11_MESSAGE_ID_CREATE_TRACKED_WORKLOAD_NULLPARAM = 0x30021D
    D3D11_MESSAGE_ID_CREATE_TRACKED_WORKLOAD_INVALID_MAX_INSTANCES = 0x30021E
    D3D11_MESSAGE_ID_CREATE_TRACKED_WORKLOAD_INVALID_DEADLINE_TYPE = 0x30021F
    D3D11_MESSAGE_ID_CREATE_TRACKED_WORKLOAD_INVALID_ENGINE_TYPE = 0x300220
    D3D11_MESSAGE_ID_MULTIPLE_TRACKED_WORKLOADS = 0x300221
    D3D11_MESSAGE_ID_MULTIPLE_TRACKED_WORKLOAD_PAIRS = 0x300222
    D3D11_MESSAGE_ID_INCOMPLETE_TRACKED_WORKLOAD_PAIR = 0x300223
    D3D11_MESSAGE_ID_OUT_OF_ORDER_TRACKED_WORKLOAD_PAIR = 0x300224
    D3D11_MESSAGE_ID_CANNOT_ADD_TRACKED_WORKLOAD = 0x300225
    D3D11_MESSAGE_ID_TRACKED_WORKLOAD_NOT_SUPPORTED = 0x300226
    D3D11_MESSAGE_ID_TRACKED_WORKLOAD_ENGINE_TYPE_NOT_FOUND = 0x300227
    D3D11_MESSAGE_ID_NO_TRACKED_WORKLOAD_SLOT_AVAILABLE = 0x300228
    D3D11_MESSAGE_ID_END_TRACKED_WORKLOAD_INVALID_ARG = 0x300229
    D3D11_MESSAGE_ID_TRACKED_WORKLOAD_DISJOINT_FAILURE = 0x30022A
    D3D11_MESSAGE_ID_D3D11_5_MESSAGES_END = 0x30022B

class D3D12_VIDEO_PROCESS_FILTER(IntEnum):
    D3D12_VIDEO_PROCESS_FILTER_BRIGHTNESS = 0x0
    D3D12_VIDEO_PROCESS_FILTER_CONTRAST = 0x1
    D3D12_VIDEO_PROCESS_FILTER_HUE = 0x2
    D3D12_VIDEO_PROCESS_FILTER_SATURATION = 0x3
    D3D12_VIDEO_PROCESS_FILTER_NOISE_REDUCTION = 0x4
    D3D12_VIDEO_PROCESS_FILTER_EDGE_ENHANCEMENT = 0x5
    D3D12_VIDEO_PROCESS_FILTER_ANAMORPHIC_SCALING = 0x6
    D3D12_VIDEO_PROCESS_FILTER_STEREO_ADJUSTMENT = 0x7

class D3D12_RESOURCE_BARRIER_TYPE(IntEnum):
    D3D12_RESOURCE_BARRIER_TYPE_TRANSITION = 0x0
    D3D12_RESOURCE_BARRIER_TYPE_ALIASING = 0x1
    D3D12_RESOURCE_BARRIER_TYPE_UAV = 0x2

class _DXGI_OFFER_RESOURCE_PRIORITY(IntEnum):
    DXGI_OFFER_RESOURCE_PRIORITY_LOW = 0x1
    DXGI_OFFER_RESOURCE_PRIORITY_NORMAL = 0x2
    DXGI_OFFER_RESOURCE_PRIORITY_HIGH = 0x3

class D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT(IntEnum):
    D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT_Y = 0x0
    D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT_U = 0x1
    D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT_V = 0x2
    D3D12_VIDEO_DECODE_HISTOGRAM_COMPONENT_A = 0x3

class D2D1_SVG_ATTRIBUTE_POD_TYPE(IntEnum):
    D2D1_SVG_ATTRIBUTE_POD_TYPE_FLOAT = 0x0
    D2D1_SVG_ATTRIBUTE_POD_TYPE_COLOR = 0x1
    D2D1_SVG_ATTRIBUTE_POD_TYPE_FILL_MODE = 0x2
    D2D1_SVG_ATTRIBUTE_POD_TYPE_DISPLAY = 0x3
    D2D1_SVG_ATTRIBUTE_POD_TYPE_OVERFLOW = 0x4
    D2D1_SVG_ATTRIBUTE_POD_TYPE_LINE_CAP = 0x5
    D2D1_SVG_ATTRIBUTE_POD_TYPE_LINE_JOIN = 0x6
    D2D1_SVG_ATTRIBUTE_POD_TYPE_VISIBILITY = 0x7
    D2D1_SVG_ATTRIBUTE_POD_TYPE_MATRIX = 0x8
    D2D1_SVG_ATTRIBUTE_POD_TYPE_UNIT_TYPE = 0x9
    D2D1_SVG_ATTRIBUTE_POD_TYPE_EXTEND_MODE = 0xA
    D2D1_SVG_ATTRIBUTE_POD_TYPE_PRESERVE_ASPECT_RATIO = 0xB
    D2D1_SVG_ATTRIBUTE_POD_TYPE_VIEWBOX = 0xC
    D2D1_SVG_ATTRIBUTE_POD_TYPE_LENGTH = 0xD
    D2D1_SVG_ATTRIBUTE_POD_TYPE_FORCE_DWORD = 0xFFFFFFFF

class DXGI_INFO_QUEUE_MESSAGE_CATEGORY(IntEnum):
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_UNKNOWN = 0x0
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_MISCELLANEOUS = 0x1
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_INITIALIZATION = 0x2
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_CLEANUP = 0x3
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_COMPILATION = 0x4
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_STATE_CREATION = 0x5
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_STATE_SETTING = 0x6
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_STATE_GETTING = 0x7
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_RESOURCE_MANIPULATION = 0x8
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_EXECUTION = 0x9
    DXGI_INFO_QUEUE_MESSAGE_CATEGORY_SHADER = 0xA

class DXGI_INFO_QUEUE_MESSAGE_SEVERITY(IntEnum):
    DXGI_INFO_QUEUE_MESSAGE_SEVERITY_CORRUPTION = 0x0
    DXGI_INFO_QUEUE_MESSAGE_SEVERITY_ERROR = 0x1
    DXGI_INFO_QUEUE_MESSAGE_SEVERITY_WARNING = 0x2
    DXGI_INFO_QUEUE_MESSAGE_SEVERITY_INFO = 0x3
    DXGI_INFO_QUEUE_MESSAGE_SEVERITY_MESSAGE = 0x4

class __MIDL___MIDL_itf_textstor_0000_0000_0003(IntEnum):
    TS_RT_PLAIN = 0x0
    TS_RT_HIDDEN = 0x1
    TS_RT_OPAQUE = 0x2

class D3D11_LOGIC_OP(IntEnum):
    D3D11_LOGIC_OP_CLEAR = 0x0
    D3D11_LOGIC_OP_SET = 0x1
    D3D11_LOGIC_OP_COPY = 0x2
    D3D11_LOGIC_OP_COPY_INVERTED = 0x3
    D3D11_LOGIC_OP_NOOP = 0x4
    D3D11_LOGIC_OP_INVERT = 0x5
    D3D11_LOGIC_OP_AND = 0x6
    D3D11_LOGIC_OP_NAND = 0x7
    D3D11_LOGIC_OP_OR = 0x8
    D3D11_LOGIC_OP_NOR = 0x9
    D3D11_LOGIC_OP_XOR = 0xA
    D3D11_LOGIC_OP_EQUIV = 0xB
    D3D11_LOGIC_OP_AND_REVERSE = 0xC
    D3D11_LOGIC_OP_AND_INVERTED = 0xD
    D3D11_LOGIC_OP_OR_REVERSE = 0xE
    D3D11_LOGIC_OP_OR_INVERTED = 0xF

class D2D1_IMAGE_SOURCE_LOADING_OPTIONS(IntEnum):
    D2D1_IMAGE_SOURCE_LOADING_OPTIONS_NONE = 0x0
    D2D1_IMAGE_SOURCE_LOADING_OPTIONS_RELEASE_SOURCE = 0x1
    D2D1_IMAGE_SOURCE_LOADING_OPTIONS_CACHE_ON_DEMAND = 0x2
    D2D1_IMAGE_SOURCE_LOADING_OPTIONS_FORCE_DWORD = 0xFFFFFFFF

class D3D12_DESCRIPTOR_HEAP_TYPE(IntEnum):
    D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV = 0x0
    D3D12_DESCRIPTOR_HEAP_TYPE_SAMPLER = 0x1
    D3D12_DESCRIPTOR_HEAP_TYPE_RTV = 0x2
    D3D12_DESCRIPTOR_HEAP_TYPE_DSV = 0x3
    D3D12_DESCRIPTOR_HEAP_TYPE_NUM_TYPES = 0x4

class TypePosition(IntEnum):
    TYPE_IS_METADATA = 0x0
    TYPE_IS_DATA = 0x1

class _Min_max_mode(IntEnum):
    _Mode_min = 0x1
    _Mode_max = 0x2
    _Mode_both = 0x3

class eInWorldScreens(IntEnum):
    EInWorldScreen_Frontend = 0x0
    EInWorldScreen_HUD = 0x1
    EInWorldScreen_QuickMenu = 0x2
    EInWorldScreen_Compass = 0x3
    EInWorldScreen_Crosshair = 0x4
    EInWorldScreen_WeaponMenu = 0x5
    EInWorldScreen_QuickMenuLauncher = 0x6
    EInWorldScreen_HoldInteract = 0x7
    EInWorldScreen_ShootInteract = 0x8
    EInWorldScreen_GameModeSelect = 0x9
    EInWorldScreen_FaceLockedVR = 0xA
    EInWorldScreen_NumTypes = 0xB

class eOtherStateEvents(IntEnum):
    ELocalLoad_NotifyRestoringToPreviousSave = 0x3E8
    ELocalLoad_SimulateNewColdBoot = 0x3E9
    ELocalload_BootStraightToUGCBase = 0x3EA
    ELocalload_ReturnToApplicationSelect = 0x3EB
    ELocalLoad_ForceRespawnAsFreshStart = 0x3EC

class AzureRoutes(ctypes.Structure):
    class Entry(IntEnum):
        QueryExact = 0x0
        QueryCategory = 0x1
        QueryAll = 0x2
        QueryVoxel = 0x3
        RequestBases = 0x4
        SubmitDiscovery = 0x5
        SubmitMessage = 0x6
        SubmitBase = 0x7
        SubmitBaseWithImage = 0x8
        ReportContent = 0x9
        ReportBase = 0xA
        BlobServe = 0xB
        SubmitMonument = 0xC
        QueryMonument = 0xD
        QueryBases = 0xE
        QueryFeaturedBases = 0xF
        MarkBaseFeatured = 0x10
        MarkBaseFeaturedWithImage = 0x11
        SubmitContribution = 0x12
        QueryTotalContribution = 0x13
        BaseImages = 0x14
        QueryActiveFeaturedBases = 0x15
        MarkFeaturedBasesDeleted = 0x16
        ChangeFeaturedBasesEnv = 0x17
        FeaturedBaseImages = 0x18
        QuerySeasonData = 0x19
        SubmitSettlement = 0x1A
        QuerySettlements = 0x1B
        NumRoutes = 0x1C

    _fields_ = [
        ("mURIs", ctypes.c_char * 0xA8E0),  # std::array<cGcURL,28>
        ("mAdminRoutes", ctypes.c_char * 0x1C),  # std::array<bool,28>
    ]
    mURIs: bytes
    mAdminRoutes: bytes

class eByteBeatSwitchCondition(IntEnum):
    EByteBeatSwitchCondition_OnBeat_1 = 0x0
    EByteBeatSwitchCondition_OnBeat_2 = 0x1
    EByteBeatSwitchCondition_OnBeat_3 = 0x2
    EByteBeatSwitchCondition_OnBeat_4 = 0x3
    EByteBeatSwitchCondition_OnBaseNote_1 = 0x4
    EByteBeatSwitchCondition_OnBaseNote_2 = 0x5
    EByteBeatSwitchCondition_OnBaseNote_3 = 0x6
    EByteBeatSwitchCondition_OnBaseNote_4 = 0x7
    EByteBeatSwitchCondition_OnBaseNote_5 = 0x8
    EByteBeatSwitchCondition_OnBaseNote_6 = 0x9
    EByteBeatSwitchCondition_OnBaseNote_7 = 0xA
    EByteBeatSwitchCondition_OnArpNote_1 = 0xB
    EByteBeatSwitchCondition_OnArpNote_2 = 0xC
    EByteBeatSwitchCondition_OnArpNote_3 = 0xD
    EByteBeatSwitchCondition_OnArpNote_4 = 0xE
    EByteBeatSwitchCondition_OnArpNote_5 = 0xF
    EByteBeatSwitchCondition_OnArpNote_6 = 0x10
    EByteBeatSwitchCondition_OnArpNote_7 = 0x11
    EByteBeatSwitchCondition_OnRhythmHit_1 = 0x12
    EByteBeatSwitchCondition_OnRhythmHit_2 = 0x13
    EByteBeatSwitchCondition_OnRhythmHit_3 = 0x14
    EByteBeatSwitchCondition_NumTypes = 0x15

class eByteBeatRandomiseType(IntEnum):
    EByteBeatRandomiseType_Sequence = 0x1
    EByteBeatRandomiseType_Arpeggio = 0x2
    EByteBeatRandomiseType_Envelope = 0x4
    EByteBeatRandomiseType_RhythmPattern = 0x8
    EByteBeatRandomiseType_RhythmSounds = 0x10
    EByteBeatRandomiseType_All = 0xFFFFFFFF

class eByteBeatArpeggio(IntEnum):
    EByteBeatArpeggio_Up = 0x0
    EByteBeatArpeggio_Down = 0x1
    EByteBeatArpeggio_UpDown = 0x2
    EByteBeatArpeggio_DownUp = 0x3
    EByteBeatArpeggio_NumTypes = 0x4

class eByteBeatCopyType(IntEnum):
    EByteBeatCopyType_None = 0x0
    EByteBeatCopyType_Tree = 0x1
    EByteBeatCopyType_Sequence = 0x2
    EByteBeatCopyType_All = 0x3
    EByteBeatCopyType_NumTypes = 0x4

class eByteBeatSampleRate(IntEnum):
    EByteBeatSampleRate_Low = 0x0
    EByteBeatSampleRate_Med = 0x1
    EByteBeatSampleRate_High = 0x2
    EByteBeatSampleRate_Ultra = 0x3
    EByteBeatSampleRate_NumTypes = 0x4

class eCommunityMissionTierMessage(IntEnum):
    ECommunityMissionTierMessage_Tier_Complete = 0x0
    ECommunityMissionTierMessage_Mission_Complete = 0x1

class cGcFrontendPageByteBeat(ctypes.Structure):
    class eByteBeatPage(IntEnum):
        EByteBeatPage_Sequencer = 0x0
        EByteBeatPage_PartSequencer = 0x1
        EByteBeatPage_Tree = 0x2
        EByteBeatPage_NumTypes = 0x3

    def PrepareToOpen(): pass

    def LoadIcons(): pass

    def DoByteBeat(): pass

    def RebuildGraphicalTree(): pass

    def SetOptionLabel(): pass

    def DoOptionsGroup(): pass

    def DoByteBeatTree(): pass

    def DoByteBeatSequencer(): pass

    def HandleInput(): pass

    def GenerateNonSilentRandomTree(): pass

    def GenerateRandomTree(): pass

    def BuildInternalTree(): pass

    def BuildTokenList(): pass

    def BuildGraphicalTree(): pass

    def SetNodeFromToken(): pass

    def UpdateTreeNode(): pass

    def DoTooltip(): pass

    def ClearRecursive(): pass

    def DoHoverButton(): pass

    def RenderEmbeddedVisualisation(): pass

    def RenderMainVisualisation(): pass

    def RenderVisualisation(): pass

    def RenderTreeLines(): pass

    def DrawLines(): pass

    def RenderSequencer(): pass

    def RenderHighlight(): pass

    def RenderSequencerStep(): pass

    def RenderSequencerArpStep(): pass

    def RenderThisPartSequencer(): pass

    def RenderConnectedPartSequencer(): pass

    def RenderPartSequencer(): pass

    def RenderPlayHead(): pass

    def RenderRhythmSequencer(): pass

    def RenderRhythmStep(): pass

    def RenderArpeggio(): pass

    def RenderDrumMixSlider(): pass

    def RenderArpRangeSlider(): pass

    def RenderArpStepSlider(): pass

    def RenderArpLengthSlider(): pass

    def RenderArpTypeSlider(): pass

    def RenderWaveTypeSlider(): pass

    def RenderEnvFadeIn(): pass

    def RenderEnvFadeOut(): pass

    def RenderEnvLength(): pass

    def RenderNumStepsSlider(): pass

    def RenderArpPowerButton(): pass

    def RenderMelodyPowerButton(): pass

    def RenderRhythmPowerButton(): pass

    def RenderNoteOnButton(): pass

    def RenderCopyButton(): pass

    def RenderPasteButton(): pass

    def RenderClearSequenceButton(): pass

    def RenderClearRhythmButton(): pass

    def RenderRandomiseSequenceButton(): pass

    def RenderRandomiseRhythmButton(): pass

    def RenderRandomiseArpeggioButton(): pass

    def RenderRandomiseEnvelopeButton(): pass

    def RenderRandomiseEverythingButton(): pass

    def RenderSoloButton(): pass

    def RenderRandomiseKickButton(): pass

    def RenderRandomiseSnareButton(): pass

    def RenderRandomiseHiHatButton(): pass

    def RenderPlayButton(): pass

    def RenderEnvelope(): pass

    def RenderSlider(): pass

    def RenderButton(): pass

    def GetRhythmTrackcolour(): pass

    def GetMelodyTrackcolour(): pass


class eInteractionUpdateRequest(IntEnum):
    EUpdateRequest_None = 0x0
    EUpdateRequest_SummonShip = 0x1

class eByteBeatNote(IntEnum):
    EByteBeatNote_C = 0x0
    EByteBeatNote_CSharp = 0x1
    EByteBeatNote_D = 0x2
    EByteBeatNote_EFlat = 0x3
    EByteBeatNote_E = 0x4
    EByteBeatNote_F = 0x5
    EByteBeatNote_FSharp = 0x6
    EByteBeatNote_G = 0x7
    EByteBeatNote_AFlat = 0x8
    EByteBeatNote_A = 0x9
    EByteBeatNote_BFlat = 0xA
    EByteBeatNote_B = 0xB
    EByteBeatNote_NumTypes = 0xC
    EByteBeatNote_None = 0xFFFFFFFF

class eByteBeatKey(IntEnum):
    EByteBeatKey_CMajor = 0x0
    EByteBeatKey_CSharpMajor = 0x1
    EByteBeatKey_DMajor = 0x2
    EByteBeatKey_EFlatMajor = 0x3
    EByteBeatKey_EMajor = 0x4
    EByteBeatKey_FMajor = 0x5
    EByteBeatKey_FSharpMajor = 0x6
    EByteBeatKey_GMajor = 0x7
    EByteBeatKey_AFlatMajor = 0x8
    EByteBeatKey_AMajor = 0x9
    EByteBeatKey_BFlatMajor = 0xA
    EByteBeatKey_BMajor = 0xB
    EByteBeatKey_CMinor = 0xC
    EByteBeatKey_CSharpMinor = 0xD
    EByteBeatKey_DMinor = 0xE
    EByteBeatKey_EFlatMinor = 0xF
    EByteBeatKey_EMinor = 0x10
    EByteBeatKey_FMinor = 0x11
    EByteBeatKey_FSharpMinor = 0x12
    EByteBeatKey_GMinor = 0x13
    EByteBeatKey_AFlatMinor = 0x14
    EByteBeatKey_AMinor = 0x15
    EByteBeatKey_BFlatMinor = 0x16
    EByteBeatKey_BMinor = 0x17
    EByteBeatKey_NumTypes = 0x18

class eByteBeatSequenceLength(IntEnum):
    EByteBeatSequenceLength_Four = 0x0
    EByteBeatSequenceLength_Eight = 0x1
    EByteBeatSequenceLength_Sixteen = 0x2
    EByteBeatSequenceLength_NumTypes = 0x3

class eByteBeatNoteLength(IntEnum):
    EByteBeatNoteLength_Quarter = 0x0
    EByteBeatNoteLength_Half = 0x1
    EByteBeatNoteLength_ThreeQuarter = 0x2
    EByteBeatNoteLength_Full = 0x3
    EByteBeatNoteLength_NumTypes = 0x4

class eByteBeatSubDiv(IntEnum):
    EByteBeatSubDiv_None = 0x0
    EByteBeatSubDiv_Two = 0x1
    EByteBeatSubDiv_Three = 0x2
    EByteBeatSubDiv_Four = 0x3
    EByteBeatSubDiv_NumTypes = 0x4

class eDockState(IntEnum):
    EDockState_None = 0x0
    EDockState_Landing = 0x1
    EDockState_LandingManeuvre = 0x2
    EDockState_Docking = 0x3
    EDockState_DockingRotate = 0x4
    EDockState_Docked = 0x5
    EDockState_TakeOffRequested = 0x6
    EDockState_TakeOffManeuvre = 0x7
    EDockState_TakeOff_Immediate = 0x8
    EDockState_TakeOff = 0x9

class eApplicationEvent(IntEnum):
    EAppEvent_MoveToLocalLoad = 0x0
    EAppEvent_MoveToModeSelector = 0x1
    EAppEvent_MoveToTitleScreen = 0x2
    EAppEvent_ExitGame = 0x3
    ENumAppEvents = 0x4
    EAppEvent_Invalid = 0xFFFFFFFF

class AnimationTransitionType(IntEnum):
    NoTransition = 0x0
    TransitionInto = 0x1
    Intermediate = 0x2

class MissionDetailMessage(ctypes.Structure):
    class eRenderState(IntEnum):
        ERenderState_SetText = 0x0
        ERenderState_WaitAFrame = 0x1
        ERenderSTate_FixSizes = 0x2
        ERenderSTate_Done = 0x3

    _fields_ = [
        ("msTitle", ctypes.c_ulonglong),
        ("msDescription", ctypes.c_ulonglong),
        ("mImageID", ctypes.c_ulonglong),
        ("mAudioEvent", ctypes.c_uint32),
        ("mPinnedItemID", ctypes.c_ulonglong),
        ("maPoints", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcMissionSequenceDetailMessagePoint>
        ("_meState", ctypes.c_uint32),
    ]
    msTitle: int
    msDescription: int
    mImageID: int
    mAudioEvent: int
    mPinnedItemID: int
    maPoints: bytes
    _meState: int

    @property
    def meState(self):
        return MissionDetailMessage.eRenderState(self._meState)


class eQuickMenuState(IntEnum):
    EQuickMenuState_Closed = 0x0
    EQuickMenuState_QuickActions = 0x1
    EQuickMenuState_Build = 0x2
    EQuickMenuState_TerrainEditor = 0x3
    EQuickMenuState_GalaxyPath = 0x4

class eQuickCategory(IntEnum):
    EQuickCategory_Next = 0x0
    EQuickCategory_Current = 0x1
    EQuickCategory_Previous = 0x2
    EQuickCategory_Num = 0x3

class eHotActionType(IntEnum):
    EHotActionType_JustAction = 0x0
    EHotActionType_ActionWithID = 0x1
    EHotActionType_OpenMenu = 0x2
    EHotActionType_OpenParentMenu = 0x3
    EHotActionType_OpenToAction = 0x4
    EHotActionType_NotBound = 0x5
    EHotActionType_None = 0x6

class eConstantMessage(IntEnum):
    EConstantMessage_Sequence = 0x0
    EConstantMessage_NotifyMessage = 0x1
    EConstantMessage_Vehicle = 0x2
    EConstantMessage_ScanEvent = 0x3
    EConstantMessage_TerrainEdit = 0x4
    EConstantMessage_Extremeness = 0x5
    EConstantMessage_Recover_Grave = 0x6
    EConstantMessage_Gun = 0x7
    EConstantMessage_LandShip = 0x8
    EConstantMessage_MeleeHint = 0x9
    EConstantMessage_ZoomHint = 0xA
    EConstantMessage_JetpackHint = 0xB
    EConstantMessage_ScannerHint = 0xC
    EConstantMessage_TorchHint = 0xD
    EConstantMessage_DroneCombat = 0xE
    EConstantMessage_SentinelDeploy = 0xF
    EConstantMessage_ShipBoostHint = 0x10
    EConstantMessage_ShipJumpHint = 0x11
    EConstantMessage_ShipCombat = 0x12
    EConstantMessage_ShipScanEventHint = 0x13
    EConstantMessage_ExperienceMessage = 0x14
    EConstantMessage_EntitlementsHint = 0x15
    EConstantMessage_FreighterBaseTutorial = 0x16
    EConstantMessage_FreighterMultiShips = 0x17
    EConstantMessage_ArtifactHint = 0x18
    EConstantMessage_WorkshopBaseWarning = 0x19
    EConstantMessage_MultiplayerDisconnect = 0x1A
    EConstantMessage_Num = 0x1B

class cGcNotificationSequence(ctypes.Structure):
    class eSequenceFlags(IntEnum):
        eSequenceFlag_None = 0x0
        eSequenceFlag_Community = 0x1
        eSequenceFlag_Recurring = 0x2
        eSequenceFlag_Multiplayer = 0x4
        eSequenceFlag_ExplorationLog = 0x8

    def ShowFutureObjectives(): pass

    def GetCurrentGroup(): pass

    def GetSectionToHideFromLogForIndex(): pass

    def GetCurrentSectionWithTargetID(): pass

    def Begin(): pass

    def GetMissionDataTitle(): pass

    def UseCommunityMissionForLog(): pass

    def GetCurrentCategory(): pass

    def GetCurrentIcon(): pass

    def GetIconAtStage(): pass

    def GetCurrentFrontendPageHint(): pass

    def GetCurrentInventoryHint(): pass

    def GetCurrentFrontendPageLocID(): pass

    def Restart(): pass

    def GoBackToStage(): pass

    def Load(): pass

    def Save(): pass

    def cGcNotificationSequence(): pass

    def vector_deleting_destructor(): pass

    def tilde_cGcNotificationSequence(): pass

    def Update(): pass

    def HasText(): pass

    def GetText(): pass

    def CanBeRestarted(): pass

    def IsBasePartUnlockedByMission(): pass

    def GetSubtitle(): pass

    def GetDescription(): pass

    def ShowCommunityProgress(): pass

    def GetCurrentColour(): pass

    def NGuiRender(): pass

    def ShouldBeInLog(): pass

    def CanBeRenounced(): pass

    def RenounceMission(): pass

    def SelectWeekendEventMissionID(): pass

    def GetMultiplayerMissionType(): pass

    def OnSequenceComplete(): pass

    def GetAppropriateMissionSurveyHint(): pass

    def GetMissionPhotoTargetPhotoBuilding(): pass

    def GetMissionPhotoTargetBuilding(): pass

    def GetCurrentSection(): pass

    def GetCurrentSectionWithIcon(): pass

    def AddStage(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mId", ctypes.c_ulonglong),
        ("mapSequence", ctypes.c_char * 0x18),  # std::vector<cGcNotificationSequenceStage *,TkSTLAllocatorShim<cGcNotificationSequenceStage *,8,-1> >
        ("mapGroups", ctypes.c_char * 0x18),  # std::vector<cGcNotificationSequenceGroup *,TkSTLAllocatorShim<cGcNotificationSequenceGroup *,8,-1> >
        ("mUserData", ctypes.c_uint64),
        ("miActiveStage", ctypes.c_int32),
        ("miBeginCheckFrequency", ctypes.c_int32),
        ("maPendingMessages", ctypes.c_char * 0x18),  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> >
        ("mbNewPendingMessages", ctypes.c_ubyte),
        ("maGenericSections", ctypes.c_char * 0x18),  # std::vector<cGcNotifyGenericSection,TkSTLAllocatorShim<cGcNotifyGenericSection,8,-1> >
        ("mDebugName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbPendingActivate", ctypes.c_ubyte),
        ("mSeed", common.GcSeed),
        ("mSequenceFlags", ctypes.c_int32),
        ("mbUsingFallbackScanEvents", ctypes.c_ubyte),
    ]
    __vftable: int
    mId: int
    mapSequence: bytes
    mapGroups: bytes
    mUserData: int
    miActiveStage: int
    miBeginCheckFrequency: int
    maPendingMessages: bytes
    mbNewPendingMessages: bool
    maGenericSections: bytes
    mDebugName: bytes
    mbPendingActivate: bool
    mSeed: common.GcSeed
    mSequenceFlags: int
    mbUsingFallbackScanEvents: bool

class DiscoverySearchContext(ctypes.Structure):
    class Constants(IntEnum):
        MaxEntriesSearchPerFrame = 0x1F4

    def DiscoverySearchContext(): pass

    _fields_ = [
        ("mpCurrentSearch", ctypes.c_ulonglong),
        ("mRecordsToSort", ctypes.c_char * 0x18),  # std::vector<cGcDiscoveryRecord const *,TkSTLAllocatorShim<cGcDiscoveryRecord const *,8,19> >
        ("mpIterator", ctypes.c_char * 0x8),  # cGcDiscoveryStoreImpl<DefaultDiscoveryDataHashing,19>::ConstIterator *
    ]
    mpCurrentSearch: int
    mRecordsToSort: bytes
    mpIterator: bytes

class eDronePerception(IntEnum):
    EDronePerception_None = 0x0
    EDronePerception_CanHear = 0x1
    EDronePerception_CanSee = 0x2

class eInstanceActivateFlags(IntEnum):
    EInstanceActivateFlag_Destructable = 0x1

class eFreighterNPCPriorityOrdering(IntEnum):
    Descending = 0x0
    Ascending = 0x1

class eNotificationCreation(IntEnum):
    ENotificationCreation_CreateLightWeightTempVersion = 0x0

class eMultiplayerMissionType(IntEnum):
    EMultiplayerMissionType_Standard = 0x0
    EMultiplayerMissionType_Mystery = 0x1
    EMultiplayerMissionType_Quicksilver = 0x2
    EMultiplayerMissionType_Weekend = 0x3
    EMultiplayerMissionType_NotMultiplayer = 0x4

class eSynchronisationLevel(IntEnum):
    SyncDisabled = 0x0
    SyncDataOnly = 0x1
    SyncOnlineBuffers = 0x2
    SyncOfflineBuffers = 0x3

class eNetworkedLandingState(IntEnum):
    ENetLanding_None = 0x0
    ENetLanding_Landing = 0x1
    ENetLanding_LandingFinal = 0x2
    ENetLanding_Landed = 0x3
    ENetLanding_Takeoff = 0x4
    ENetLanding_LandedOnGround = 0x5
    ENetLanding_NumStates = 0x6

class eReplaceableTextureType(IntEnum):
    EReplaceableTextureType_Diffuse = 0x0
    EReplaceableTextureType_Normal = 0x1
    EReplaceableTextureType_Mask = 0x2
    EReplaceableTextureType_Parallax = 0x3
    EReplaceableTextureType_Feature = 0x4
    EReplaceableTextureType_Num = 0x5

class _MINIDUMP_TYPE(IntEnum):
    MiniDumpNormal = 0x0
    MiniDumpWithDataSegs = 0x1
    MiniDumpWithFullMemory = 0x2
    MiniDumpWithHandleData = 0x4
    MiniDumpFilterMemory = 0x8
    MiniDumpScanMemory = 0x10
    MiniDumpWithUnloadedModules = 0x20
    MiniDumpWithIndirectlyReferencedMemory = 0x40
    MiniDumpFilterModulePaths = 0x80
    MiniDumpWithProcessThreadData = 0x100
    MiniDumpWithPrivateReadWriteMemory = 0x200
    MiniDumpWithoutOptionalData = 0x400
    MiniDumpWithFullMemoryInfo = 0x800
    MiniDumpWithThreadInfo = 0x1000
    MiniDumpWithCodeSegs = 0x2000
    MiniDumpWithoutAuxiliaryState = 0x4000
    MiniDumpWithFullAuxiliaryState = 0x8000
    MiniDumpWithPrivateWriteCopyMemory = 0x10000
    MiniDumpIgnoreInaccessibleMemory = 0x20000
    MiniDumpWithTokenInformation = 0x40000
    MiniDumpWithModuleHeaders = 0x80000
    MiniDumpFilterTriage = 0x100000
    MiniDumpWithAvxXStateContext = 0x200000
    MiniDumpWithIptTrace = 0x400000
    MiniDumpScanInaccessiblePartialPages = 0x800000
    MiniDumpValidTypeFlags = 0xFFFFFF

class _MINIDUMP_CALLBACK_TYPE(IntEnum):
    ModuleCallback = 0x0
    ThreadCallback = 0x1
    ThreadExCallback = 0x2
    IncludeThreadCallback = 0x3
    IncludeModuleCallback = 0x4
    MemoryCallback = 0x5
    CancelCallback = 0x6
    WriteKernelMinidumpCallback = 0x7
    KernelMinidumpStatusCallback = 0x8
    RemoveMemoryCallback = 0x9
    IncludeVmRegionCallback = 0xA
    IoStartCallback = 0xB
    IoWriteAllCallback = 0xC
    IoFinishCallback = 0xD
    ReadMemoryFailureCallback = 0xE
    SecondaryFlagsCallback = 0xF
    IsProcessSnapshotCallback = 0x10
    VmStartCallback = 0x11
    VmQueryCallback = 0x12
    VmPreReadCallback = 0x13
    VmPostReadCallback = 0x14

class _MODULE_WRITE_FLAGS(IntEnum):
    ModuleWriteModule = 0x1

class cGcFrontendPageOptions(ctypes.Structure):
    class eCreditsSection(IntEnum):
        ECreditsSection_HG = 0x0
        ECreditsSection_Sony = 0x1
        ECreditsSection_Legal = 0x2

    def RequestOptionsPage(): pass

    def OnSwitchToRemapControls(): pass

    def IsUsingMode1Sensitivity(): pass

    def IsUsingMode2Sensitivity(): pass

    def ShowPopup(): pass

    def HidePopup(): pass

    def UpdateSensitivityMode(): pass

    def DoGeneralOptions(): pass

    def OnSwitchToOptions(): pass

    def DoSwitcher(): pass

    def PrepareGeneralOptions(): pass

    def PrepareAccessibilityOptions(): pass

    def PrepareControlsOptions(): pass

    def PrepareCameraOptions(): pass

    def PrepareVideoOptions(): pass

    def PrepareGyroOptions(): pass

    def PrepareSwitcherMenu(): pass

    def PrepareBootScreenNetworkOptions(): pass

    def Apply(): pass

    def Exit(): pass

    def DoCredits(): pass

    def DoCreditsLine(): pass

    def DoRedeem(): pass

    def DoControlsPC(): pass

    def ShowCredits(): pass

    def UnstuckOnFreighter(): pass

    def ShowControlsPage(): pass

    def ShowGyroOptions(): pass

    def UpdateAvailableResolutions(): pass

    def ApplyPreset(): pass

    def UpdateAntialiasingOptions(): pass

    def UpdateAntialiasingSubOptions(): pass

    def UpdateDLSSSubOptions(): pass

    def HasEnabledAdvancedGyroOptions(): pass

    def GyroIsNotAttachedToScreen(): pass

    def HasEnabledGyroGameplayOptions(): pass

    def HasEnabledGyroCursorOptions(): pass

    def ApplyPreviewAudioVolume(): pass


class eFullScreenMode(IntEnum):
    FullScreen = 0x0
    Bordered = 0x1
    Borderless = 0x2

class FPNS_OptionButton(IntEnum):
    eButton1 = 0x0
    eButton2 = 0x1
    eButton3 = 0x2
    eButton4 = 0x3

class cGcFrontendPageMultiplayerMissionGiver(ctypes.Structure):
    class eJoiningStates(IntEnum):
        EJoiningState_Waiting = 0x0
        EJoiningState_Join = 0x1
        EJoiningState_JoinFireteam = 0x2
        EJoiningState_JoiningFireteam = 0x3
        EJoiningState_LeaveCurrentFireteamPopup = 0x4
        EJoiningState_LeaveCurrentFireteam = 0x5
        EJoiningState_JoinTimeout = 0x6
        EJoiningState_RequestStartSingleplayerNexusMission = 0x7
        EJoiningState_StartSingleplayerNexusMission = 0x8

    def DoMissionPage(): pass

    def DoMultiplayerMissionItem(): pass

    def DoMissionDescriptionPage(): pass

    def MakeParticipantsText(): pass

    def LayoutReward(): pass

    def UpdateMissionDescription(): pass

    def HasSpecialMissionTypes(): pass

    def LeaveCurrentFireteamDialog(): pass

    def StartMissionAloneDialog(): pass

    def GetMultiplayerMissionsList(): pass

    def GetMultiplayerWeekendMissionsList(): pass


class eMissionRewardType(IntEnum):
    EMissionREwardType_None = 0x0
    EMissionRewardType_Product = 0x1
    EMissionRewardType_Substance = 0x2
    EMissionRewardType_Tech = 0x3
    EMissionRewardType_Units = 0x4
    EMissionRewardType_Nanites = 0x5
    EMissionRewardType_Specials = 0x6
    EMissionRewardType_Standing = 0x7

class eMenuPositioning(IntEnum):
    EMenuPositioning_None = 0x0
    EMenuPositioning_Inventory = 0x1
    EMenuPositioning_QuickMenu = 0x2
    EMenuPositioning_QuickMenuLauncher = 0x3
    EMenuPositioning_WeaponMenu = 0x4
    EMenuPositioning_Frontend = 0x5

class eInteractionOrigin(IntEnum):
    EInteractionOrigin_Player = 0x0
    EInteractionOrigin_Camera = 0x1

class eMissionBoardType(IntEnum):
    EMissionBoardType_Standard = 0x0
    EMissionBoardType_Nexus = 0x1
    EMissionBoardType_Pirate = 0x2

class SentinelAttackType(IntEnum):
    Laser = 0x0
    Guns = 0x1
    Grenades = 0x2
    Grenades2 = 0x3
    Flamethrower = 0x4

class eAutoMissionStages(IntEnum):
    EAutoMissionStage_WaitForSelected = 0x0
    EAutoMissionStage_MissionStartMessage = 0x1
    EAutoMissionStage_MissionStartMessageWithNotify = 0x2
    EAutoMissionStage_MultiplayerMissionPlayerRoundUp = 0x3
    EAutoMissionStage_MissionCompleteMessage = 0x4

class eMultiplayerLocationGroup(IntEnum):
    EMultiplayerLocationGroup_InActiveSpaceships = 0x1

class eGcMemoryBackingPool(IntEnum):
    ELastTkBackingPool = 0x7
    EMemoryBackingPool_Global = 0x8
    EMemoryBackingPool_Voxels = 0x9
    EMemoryBackingPool_Asteroids = 0xA

class eGrabPlayerState(IntEnum):
    EGrabPlayer_Defend = 0x0
    EGrabPlayer_Idle = 0x1
    EGrabPlayer_IdleLookAround = 0x2
    EGrabPlayer_IdleLookAtPlayer = 0x3
    EGrabPlayer_Lunge = 0x4
    EGrabPlayer_Grabbed = 0x5
    EGrabPlayer_Cooldown = 0x6

class cGcIntroFeed(ctypes.Structure):
    def Load(): pass

    def Update(): pass

    _fields_ = [
        ("mSelectedNames", ctypes.c_char * 0x17D0),  # std::array<cTkFixedString<127,char>,48>
        ("mSelectedUsers", ctypes.c_char * 0x17D0),  # std::array<cTkFixedString<127,char>,48>
        ("muLoadedNames", ctypes.c_uint32),
        ("miUpdateTimestamp", ctypes.c_int32),
        ("mServerUpdateHandle", ctypes.c_uint64),
        ("mbUpdateTriggered", ctypes.c_ubyte),
        ("mbValid", ctypes.c_ubyte),
    ]
    mSelectedNames: bytes
    mSelectedUsers: bytes
    muLoadedNames: int
    miUpdateTimestamp: int
    mServerUpdateHandle: int
    mbUpdateTriggered: bool
    mbValid: bool

class eActivityEndReason(IntEnum):
    EActivityEndReason_Completed = 0x0
    EActivityEndReason_Failed = 0x1
    EActivityEndReason_Abandoned = 0x2

class eGroundWormState(IntEnum):
    EGroundWorm_Submerged = 0x0
    EGroundWorm_Rumble = 0x1
    EGroundWorm_Emerge = 0x2
    EGroundWorm_Active = 0x3
    EGroundWorm_Roar = 0x4
    EGroundWorm_Attack = 0x5
    EGroundWorm_Submerge = 0x6

class eMechAlertness(IntEnum):
    EMechAlertLevel_Sleep = 0x0
    EMechAlertLevel_Stunned = 0x1
    EMechAlertLevel_Patrol = 0x2
    EMechAlertLevel_Investigate = 0x3
    EMechAlertLevel_Attack = 0x4
    EMechAlertLevel_NumTypes = 0x5

class eTurretTargetResult(IntEnum):
    ETurretTarget_Success = 0x0
    ETurretTarget_OutOfRange = 0x1
    ETurretTarget_NoLineOfSight = 0x2
    ETurretTarget_OutOfExtents = 0x3
    ETurretTarget_GunNotAligned = 0x4
    ETurretTarget_Networked = 0x5
    ETurretTarget_NumTypes = 0x6

class ScePadTriggerEffectState(IntEnum):
    SCE_PAD_TRIGGER_EFFECT_STATE_OFF = 0x0
    SCE_PAD_TRIGGER_EFFECT_STATE_FEEDBACK_STANDBY = 0x1
    SCE_PAD_TRIGGER_EFFECT_STATE_FEEDBACK_ACTIVE = 0x2
    SCE_PAD_TRIGGER_EFFECT_STATE_WEAPON_STANDBY = 0x3
    SCE_PAD_TRIGGER_EFFECT_STATE_WEAPON_PULLING = 0x4
    SCE_PAD_TRIGGER_EFFECT_STATE_WEAPON_FIRED = 0x5
    SCE_PAD_TRIGGER_EFFECT_STATE_VIBRATION_STANDBY = 0x6
    SCE_PAD_TRIGGER_EFFECT_STATE_VIBRATION_ACTIVE = 0x7
    SCE_PAD_TRIGGER_EFFECT_STATE_INTERCEPTED = 0xFFFFFFFF

class eNavSteeringInterest(IntEnum):
    ENavSteeringInterest_Attraction = 0x0
    ENavSteeringInterest_Danger = 0x1
    ENavSteeringInterest_Impassable = 0x2
    ENavSteeringInterest_NumTypes = 0x3

class eNavSteeringInput(IntEnum):
    ENavSteeringInput_FollowPath = 0x0
    ENavSteeringInput_AvoidDifficultTerrain = 0x1
    ENavSteeringInput_AvoidObstruction = 0x2
    ENavSteeringInput_NumTypes = 0x3

class PFMatchmakingStateChangeType(IntEnum):
    TicketStatusChanged = 0x0
    TicketCompleted = 0x1

class eAIMechFaction(IntEnum):
    EAIMechFaction_Player = 0x0
    EAIMechFaction_Sentinel = 0x1

class ESpawnObjectStage(IntEnum):
    eSpawnStage_Complete = 0x0
    eSpawnStage_CheckTileType = 0x1
    eSpawnStage_CheckHeight = 0x2
    eSpawnStage_CheckPosition = 0x3
    eSpawnStage_CheckDensity = 0x4
    eSpawnStage_HasSpawnDensity = 0x5
    eSpawnStage_CheckNoise = 0x6
    eSpawnStage_CheckLargeObjects = 0x7
    eSpawnStage_GetPosition = 0x8
    eSpawnStage_GetMatrix = 0x9
    eSpawnStage_InteractionDist = 0xA
    eSpawnStage_InteractionDot = 0xB
    eSpawnStage_CheckTerrainEdits = 0xC
    eSpawnStage_Num = 0xD

class SendToInventoryOption(ctypes.Structure):
    class eOption(IntEnum):
        Personal = 0x0
        Personal_Cargo = 0x1
        Ship = 0x2
        Ship_Cargo = 0x3
        Vehicle = 0x4
        Freighter = 0x5
        Freighter_Cargo = 0x6
        Base = 0x7
        OtherPlayer = 0x8
        CookingIngredients = 0x9
        NumOptions = 0xA

    def GetDestinationSlots(): pass

    def GetDestinationInventory(): pass

    def DoSend(): pass

    _fields_ = [
        ("_meType", ctypes.c_uint32),
        ("miSubIndex", ctypes.c_int32),
        ("mbEnabled", ctypes.c_ubyte),
        ("mDisabledMessage", ctypes.c_ulonglong),
        ("mOtherPlayer", ctypes.c_char * 0x40),  # cTkUserIdBase<cTkFixedString<64,char> >
    ]
    _meType: int
    miSubIndex: int
    mbEnabled: bool
    mDisabledMessage: int
    mOtherPlayer: bytes

    @property
    def meType(self):
        return SendToInventoryOption.eOption(self._meType)


class eHerbivoreStopType(IntEnum):
    EHerbivoreStopType_Idle = 0x0
    EHerbivoreStopType_Eat = 0x1
    EHerbivoreStopType_Scared = 0x2

class PFLobbyAccessPolicy(IntEnum):
    Public = 0x0
    Friends = 0x1
    Private = 0x2

class PFMultiplayerProtocolType(IntEnum):
    Tcp = 0x0
    Udp = 0x1

class PFLobbyMembershipLock(IntEnum):
    Unlocked = 0x0
    Locked = 0x1

class eSpecialNoiseLayers(IntEnum):
    ESpecialNoiseLayers_Base = 0x0
    ESpecialNoiseLayers_BelowWater = 0x1
    ESpecialNoiseLayers_BuildingResources = 0x2
    ESpecialNoiseLayers_Building = 0x3
    ESpecialNoiseLayers_Max = 0x4

class cGcVoxelDensityDebug(ctypes.Structure):
    class eDebugRenderAxis(IntEnum):
        EDebugRenderAxis_XY = 0x0
        EDebugRenderAxis_ZY = 0x1
        EDebugRenderAxis_Num = 0x2

    _fields_ = [
        ("maStringBuffer", ctypes.c_char * 0x28800),  # cTkFixedString<32,char>
        ("maPositionBuffer", common.Vector3f * 0x1440),  # cTkVector3[5184]
        ("mafDensityBuffer", ctypes.c_float * 0x1440),  # float[5184]
        ("mpDebugRegion", ctypes.c_ulonglong * 0x4),  # cGcRegionTerrain *[4]
        ("mbRenderMaxHeight", ctypes.c_ubyte),
        ("mbRenderDensity", ctypes.c_ubyte),
        ("mbRenderLowDensities", ctypes.c_ubyte),
        ("mbDepthTest", ctypes.c_ubyte),
        ("miCurrentLod", ctypes.c_int32),
        ("miRowIndex", ctypes.c_int32),
        ("_meAxis", ctypes.c_uint32),
    ]
    maStringBuffer: bytes
    maPositionBuffer: list[common.Vector3f]
    mafDensityBuffer: list[ctypes.c_float]
    mpDebugRegion: list[ctypes.c_ulonglong]
    mbRenderMaxHeight: bool
    mbRenderDensity: bool
    mbRenderLowDensities: bool
    mbDepthTest: bool
    miCurrentLod: int
    miRowIndex: int
    _meAxis: int

    @property
    def meAxis(self):
        return cGcVoxelDensityDebug.eDebugRenderAxis(self._meAxis)


class eOctant(IntEnum):
    EOctant_LeftLowerForward = 0x0
    EOctant_LeftLowerBack = 0x1
    EOctant_LeftUpperForward = 0x2
    EOctant_LeftUpperBack = 0x3
    EOctant_RightLowerForward = 0x4
    EOctant_RightLowerBack = 0x5
    EOctant_RightUpperForward = 0x6
    EOctant_RightUpperBack = 0x7
    ENumOctants = 0x8

class GeomProbeBundles(IntEnum):
    EProbe_OverheadIndex = 0x0
    EProbe_SunDirCount = 0x1
    EProbe_DistanceGround = 0x5
    EProbe_GroundCount = 0xA
    EProbe_OverheadCount = 0xC
    EProbe_AroundNearIndex = 0xD
    EProbe_DistanceAroundNear = 0x12
    EProbe_AroundNearCount = 0x18
    EProbe_AroundFarIndex = 0x25
    EProbe_DistanceAroundFar = 0x32
    EProbe_GroundIndex = 0x3D
    EProbe_DirectDownIndex = 0x47
    EProbe_DirectDownNAIndex = 0x48
    EProbe_DistanceOverhead = 0x50
    EProbe_DistanceTowardsSun = 0x100

class eLightningResources(IntEnum):
    ELightningResources_IndicatorDecal = 0x0
    ELightningResources_StaticDecal = 0x1
    ELightningResources_FirstLightning = 0x2

class eMeteorResources(IntEnum):
    EMeteorResources_IndicatorDecal = 0x0
    EMeteorResources_StaticDecal = 0x1
    EMeteorResources_Meteor = 0x2
    EMeteorResources_ImpactEffect = 0x3
    EMeteorResources_ImpactExplode = 0x4
    EMeteorResources_ImpactGift = 0x5
    EMeteorResources_NumResources = 0x6

class eLightningNodes(IntEnum):
    ELightningNode_IndicatorDecal = 0x0
    ELightningNode_StaticDecal = 0x1
    ELightningNode_Lightning = 0x2
    ELightningNode_NumNodes = 0x3

class eMeteorNodes(IntEnum):
    EMeteorNode_IndicatorDecal = 0x0
    EMeteorNode_StaticDecal = 0x1
    EMeteorNode_Meteor = 0x2
    EMeteorNode_NumNodes = 0x3

class cGcDetailAnimList(ctypes.Structure):
    def GetRandomAnim(): pass

    _fields_ = [
        ("maAnims", ctypes.c_ulonglong * 0xA),  # TkID<128>[10]
        ("miNumAnims", ctypes.c_int32),
        ("maAnimsShuffled", ctypes.c_ulonglong * 0xA),  # TkID<128>[10]
        ("miNumAnimsShuffledRemaining", ctypes.c_int32),
    ]
    maAnims: list[ctypes.c_ulonglong]
    miNumAnims: int
    maAnimsShuffled: list[ctypes.c_ulonglong]
    miNumAnimsShuffledRemaining: int

class EWFCTestMode(IntEnum):
    eWFCTestMode_Custom = 0x0
    eWFCTestMode_Building = 0x1
    eWFCTestMode_Preset = 0x2

class EWallTheme(IntEnum):
    eWallTheme_Bottom = 0x0
    eWallTheme_Middle = 0x1
    eWallTheme_Top = 0x2
    eWallTheme_None = 0x3
    eWallTheme_NumTypes = 0x4

class PROCESS_DPI_AWARENESS(IntEnum):
    PROCESS_DPI_UNAWARE = 0x0
    PROCESS_SYSTEM_DPI_AWARE = 0x1
    PROCESS_PER_MONITOR_DPI_AWARE = 0x2

class eEgRenderParity(IntEnum):
    eRenderParity_Single = 0x0
    eRenderParity_EvenRender = 0x1
    eRenderParity_OddRender = 0x2

class eEgEnqueueFlags(IntEnum):
    eEnqueueFlag_NoCull = 0x1

class eShipControlForces(IntEnum):
    EShipControlForce_Thrust = 0x0
    EShipControlForce_Atmosphere = 0x1
    EShipControlForce_Collision = 0x2
    EShipControlForce_Height = 0x3
    EShipControlForce_SpeedMatch = 0x4
    EShipControlForce_MinSpeed = 0x5
    EShipControlForce_DirectionalBrake = 0x6
    EShipControlForce_PlanetDrop = 0x7
    EShipControlForce_NumTypes = 0x8

class PipelineConditions(ctypes.Structure):
    class List(IntEnum):
        _True = 0x0
        _False = 0x1
        Uniform = 0x2
        FrameIndex = 0x3


class Status(IntEnum):
    eInsideTag = 0x0
    eOutsideTag = 0x1

class XMLTokenTypeTag(IntEnum):
    eTokenText = 0x0
    eTokenQuotedText = 0x1
    eTokenTagStart = 0x2
    eTokenTagEnd = 0x3
    eTokenCloseTag = 0x4
    eTokenEquals = 0x5
    eTokenDeclaration = 0x6
    eTokenShortHandClose = 0x7
    eTokenClear = 0x8
    eTokenError = 0x9

class Attrib(IntEnum):
    eAttribName = 0x0
    eAttribEquals = 0x1
    eAttribValue = 0x2

class PFLobbyMemberRemovedReason(IntEnum):
    LocalUserLeftLobby = 0x0
    LocalUserForciblyRemoved = 0x1
    RemoteUserLeftLobby = 0x2

class PFLobbyDisconnectingReason(IntEnum):
    NoLocalMembers = 0x0
    LobbyDeleted = 0x1
    ConnectionInterruption = 0x2

class eListOptionStyle(IntEnum):
    Button = 0x0
    Arrows = 0x1

class eExecutePipelineReturn(IntEnum):
    EPR_Normal = 0x0
    EPR_LastCommandWasRender = 0x1
    EPR_ForceNewContext = 0x2

class cGcFrontendPageNetworkSettings(ctypes.Structure):
    def DoPlayerList(): pass

    def LoadPage(): pass

    def DoOptions(): pass

    def DoPlayers(): pass

    def DoManageFriends(): pass

    def DoFriendCodes(): pass

    def DoBlockedPlayers(): pass

    def DoPopupOption(): pass


class cEgPrivateManagedRenderData(ctypes.Structure):
    class Constants(IntEnum):
        DefaultSceneNodeReservation = 0x154
        DefaultSizeKB = 0x258

    def cEgPrivateManagedRenderData(): pass

    def tilde_cEgPrivateManagedRenderData(): pass

    _fields_ = [
        ("mMemoryBuffer", ctypes.c_char * 0x18),  # std::vector<unsigned char,TkSTLAllocatorShim<unsigned char,16,-1> >
        ("mMemoryBufferRender", ctypes.c_char * 0x18),  # std::vector<unsigned char,TkSTLAllocatorShim<unsigned char,16,-1> >
        ("mMemoryBufferReuse", ctypes.c_char * 0x230),  # cTkStackVector<std::pair<unsigned __int64,unsigned __int64>,32,-1>
        ("mSceneNodeOwners", ctypes.c_char * 0x18),  # std::vector<cEgSceneNode *,TkSTLAllocatorShim<cEgSceneNode *,8,-1> >
        ("mRequests", ctypes.c_int32),
    ]
    mMemoryBuffer: bytes
    mMemoryBufferRender: bytes
    mMemoryBufferReuse: bytes
    mSceneNodeOwners: bytes
    mRequests: int

class NVGcodepointType(IntEnum):
    NVG_SPACE = 0x0
    NVG_NEWLINE = 0x1
    NVG_CHAR = 0x2

class NVGcommands(IntEnum):
    NVG_MOVETO = 0x0
    NVG_LINETO = 0x1
    NVG_BEZIERTO = 0x2
    NVG_CLOSE = 0x3
    NVG_WINDING = 0x4

class NVGpointFlags(IntEnum):
    NVG_PT_CORNER = 0x1

def DbTrimFromPlanet(): pass

class ScePadControllerType(IntEnum):
    SCE_PAD_CONTROLLER_TYPE_NOT_CONNECTED = 0x0
    SCE_PAD_CONTROLLER_TYPE_DUALSHOCK4 = 0x1
    SCE_PAD_CONTROLLER_TYPE_DUALSENSE = 0x2

class cGcRegionGeneratorPerfEvent(ctypes.Structure):
    class EType(IntEnum):
        GenerateBegin = 0x0
        GenerateEnd = 0x1
        Pause = 0x2
        Unpause = 0x3

    _fields_ = [
        ("miX", ctypes.c_int32),
        ("miY", ctypes.c_int32),
        ("miZ", ctypes.c_int32),
        ("mfScale", ctypes.c_float),
        ("muTime", ctypes.c_uint64),
        ("_meType", ctypes.c_uint32),
    ]
    miX: int
    miY: int
    miZ: int
    mfScale: float
    muTime: int
    _meType: int

    @property
    def meType(self):
        return cGcRegionGeneratorPerfEvent.EType(self._meType)


class ScePadTriggerEffectMode(IntEnum):
    SCE_PAD_TRIGGER_EFFECT_MODE_OFF = 0x0
    SCE_PAD_TRIGGER_EFFECT_MODE_FEEDBACK = 0x1
    SCE_PAD_TRIGGER_EFFECT_MODE_WEAPON = 0x2
    SCE_PAD_TRIGGER_EFFECT_MODE_VIBRATION = 0x3
    SCE_PAD_TRIGGER_EFFECT_MODE_MULTIPLE_POSITION_FEEDBACK = 0x4
    SCE_PAD_TRIGGER_EFFECT_MODE_SLOPE_FEEDBACK = 0x5
    SCE_PAD_TRIGGER_EFFECT_MODE_MULTIPLE_POSITION_VIBRATION = 0x6

class VoxelTraversal(ctypes.Structure):
    class WalkResult(IntEnum):
        PathComplete = 0x0
        PathIncomplete = 0x1

    def WalkPathBresenham(): pass


class eVoxelExtra(IntEnum):
    EVoxelExtra_Water = 0x1

class eTkShaderType(IntEnum):
    EShaderType_Vertex = 0x0
    EShaderType_Pixel = 0x1

class ShipFuelTypes(IntEnum):
    HyperDrive = 0x0
    JumpDrive = 0x1

class eMessageBroadcast(IntEnum):
    EMessageBroadcast_Nearby = 0x0
    EMessageBroadcast_SameSystem = 0x1
    EMessageBroadcast_Always = 0x2

class eCaptainHologramType(IntEnum):
    ECaptainHologram_Expedition = 0x0
    ECaptainHologram_Damaged = 0x1

class ScePadVibrationMode(IntEnum):
    SCE_PAD_VIBRATION_MODE_ADVANCED = 0x1
    SCE_PAD_VIBRATION_MODE_COMPATIBLE = 0x2
    SCE_PAD_VIBRATION_MODE_COMPATIBLE2 = 0x3

class eRobotShotType(IntEnum):
    ERobotShotType_None = 0x0
    ERobotShotType_LaserCharge = 0x1
    ERobotShotType_LaserFire = 0x2
    ERobotShotType_Guns = 0x3

class eHeavyAirNodeParams(ctypes.Structure):
    class List(IntEnum):
        RadiusF = 0x7D0
        ParticleActiveScaleF = 0x7D1
        ParticleSpeedScaleF = 0x7D2
        Colour1F = 0x7D3
        Colour2F = 0x7D4
        OpacityF = 0x7D5


class eSpeedLinesNodeParams(ctypes.Structure):
    class List(IntEnum):
        TargetOpacityF = 0x7D0
        CurrentOpacityF = 0x7D1


class TkAtmospherics(ctypes.Structure):
    def InitExtension(): pass


class eTileColours(IntEnum):
    ETileColour_Base_Slope = 0x0
    ETileColour_Base = 0x1
    ETileColour_BaseAlt_Slope = 0x2
    ETileColour_BaseAlt = 0x3
    ETileColour_Mount_Slope = 0x4
    ETileColour_Mount = 0x5
    ETileColour_MountAlt_Slope = 0x6
    ETileColour_MountAlt = 0x7
    ETileColour_Cave = 0x8
    ETileColour_CaveSlope = 0x9
    ETileColour_Sand = 0xA
    ETileColour_SandSlope = 0xB
    ETileColour_Num = 0xC

class eTkUpdateStage(IntEnum):
    EUpdateStage_Main = 0x0
    EUpdateStage_PostPhysics = 0x1

class RenderCaps(ctypes.Structure):
    class List(IntEnum):
        Tex_Float = 0x0
        Tex_NPOT = 0x1
        RT_Multisampling = 0x2


class eThreadMemoryBackingPool(IntEnum):
    EThreadMemoryBackingPool_Local = 0x0

class EModelResourceRendererThumbnailMode(IntEnum):
    EModelResourceRendererThumbnailMode_NoThumbnail = 0x0
    EModelResourceRendererThumbnailMode_GUI = 0x1
    EModelResourceRendererThumbnailMode_HUD = 0x2

class eRefinerStatus(IntEnum):
    ERefinerStatus_NoRecipe = 0x0
    ERefinerStatus_HasRecipeNoIngredients = 0x1
    ERefinerStatus_HasIngredientsOutputFull = 0x2
    ERefinerStatus_HasIngredientsAndCanRefine = 0x3

class endCondition_directive(IntEnum):
    endOnOutputSize = 0x0
    endOnInputSize = 0x1

class tableType_t(IntEnum):
    byPtr = 0x0
    byU32 = 0x1
    byU16 = 0x2

class dict_directive(IntEnum):
    noDict = 0x0
    withPrefix64k = 0x1
    usingExtDict = 0x2

class limitedOutput_directive(IntEnum):
    notLimited = 0x0
    limitedOutput = 0x1

class dictIssue_directive(IntEnum):
    noDictIssue = 0x0
    dictSmall = 0x1

class PlayerCameraRaycastNames(IntEnum):
    PlayerToCamera_AdjustCast = 0x0
    CameraToPlayer_IsInsideCast = 0x1
    PlayerToCamera_IsBlockerInvisible = 0x2
    FustumCornerToPlayerPoints = 0x3
    XOffsetShiftCasts = 0x1B

class eValueDecorator(IntEnum):
    EValueDecorator_None = 0x0
    EValueDecorator_Percent = 0x1
    EValueDecorator_Distance = 0x2
    EValueDecorator_Time = 0x3
    EValueDecorator_Degrees = 0x4
    EValueDecorator_DOFSetting = 0x5
    EValueDecorator_DOFAngle = 0x6
    EValueDecorator_DOFAperture = 0x7

class WICBitmapEncoderCacheOption(IntEnum):
    WICBitmapEncoderCacheInMemory = 0x0
    WICBitmapEncoderCacheTempFile = 0x1
    WICBitmapEncoderNoCache = 0x2
    WICBITMAPENCODERCACHEOPTION_FORCE_DWORD = 0x7FFFFFFF

class WICDecodeOptions(IntEnum):
    WICDecodeMetadataCacheOnDemand = 0x0
    WICDecodeMetadataCacheOnLoad = 0x1
    WICMETADATACACHEOPTION_FORCE_DWORD = 0x7FFFFFFF

class WICComponentType(IntEnum):
    WICDecoder = 0x1
    WICEncoder = 0x2
    WICPixelFormatConverter = 0x4
    WICMetadataReader = 0x8
    WICMetadataWriter = 0x10
    WICPixelFormat = 0x20
    WICAllComponents = 0x3F
    WICCOMPONENTTYPE_FORCE_DWORD = 0x7FFFFFFF

class WICColorContextType(IntEnum):
    WICColorContextUninitialized = 0x0
    WICColorContextProfile = 0x1
    WICColorContextExifColorSpace = 0x2

class eMaterialClass(IntEnum):
    EMaterialClassAdditive = 0x0
    EMaterialClassAdditiveHighQuality = 0x1
    EMaterialClassTranslucent = 0x2
    EMaterialClassGlowTranslucent = 0x3
    EMaterialClassOpaque = 0x4
    EMaterialClass_NumTypes = 0x5

class EKatakanaUnicode(IntEnum):
    EKatakana_IdeographicSpace = 0x3000
    EKatakana_SmallA = 0x30A1
    EKatakana_A = 0x30A2
    EKatakana_SmallI = 0x30A3
    EKatakana_I = 0x30A4
    EKatakana_SmallU = 0x30A5
    EKatakana_U = 0x30A6
    EKatakana_SmallE = 0x30A7
    EKatakana_E = 0x30A8
    EKatakana_SmallO = 0x30A9
    EKatakana_O = 0x30AA
    EKatakana_KA = 0x30AB
    EKatakana_GA = 0x30AC
    EKatakana_KI = 0x30AD
    EKatakana_GI = 0x30AE
    EKatakana_KU = 0x30AF
    EKatakana_GU = 0x30B0
    EKatakana_KE = 0x30B1
    EKatakana_GE = 0x30B2
    EKatakana_KO = 0x30B3
    EKatakana_GO = 0x30B4
    EKatakana_SA = 0x30B5
    EKatakana_ZA = 0x30B6
    EKatakana_SHI = 0x30B7
    EKatakana_JI = 0x30B8
    EKatakana_SU = 0x30B9
    EKatakana_ZU = 0x30BA
    EKatakana_SE = 0x30BB
    EKatakana_ZE = 0x30BC
    EKatakana_SO = 0x30BD
    EKatakana_ZO = 0x30BE
    EKatakana_TA = 0x30BF
    EKatakana_DA = 0x30C0
    EKatakana_CHI = 0x30C1
    EKatakana_DI = 0x30C2
    EKatakana_SmallTSU = 0x30C3
    EKatakana_TSU = 0x30C4
    EKatakana_TE = 0x30C6
    EKatakana_DE = 0x30C7
    EKatakana_TO = 0x30C8
    EKatakana_DO = 0x30C9
    EKatakana_NA = 0x30CA
    EKatakana_NI = 0x30CB
    EKatakana_NU = 0x30CC
    EKatakana_NE = 0x30CD
    EKatakana_NO = 0x30CE
    EKatakana_HA = 0x30CF
    EKatakana_BA = 0x30D0
    EKatakana_PA = 0x30D1
    EKatakana_HI = 0x30D2
    EKatakana_BI = 0x30D3
    EKatakana_PI = 0x30D4
    EKatakana_FU = 0x30D5
    EKatakana_BU = 0x30D6
    EKatakana_PU = 0x30D7
    EKatakana_HE = 0x30D8
    EKatakana_BE = 0x30D9
    EKatakana_PE = 0x30DA
    EKatakana_HO = 0x30DB
    EKatakana_BO = 0x30DC
    EKatakana_PO = 0x30DD
    EKatakana_MA = 0x30DE
    EKatakana_MI = 0x30DF
    EKatakana_MU = 0x30E0
    EKatakana_ME = 0x30E1
    EKatakana_MO = 0x30E2
    EKatakana_SmallYA = 0x30E3
    EKatakana_YA = 0x30E4
    EKatakana_SmallYU = 0x30E5
    EKatakana_YU = 0x30E6
    EKatakana_SmallYO = 0x30E7
    EKatakana_YO = 0x30E8
    EKatakana_RA = 0x30E9
    EKatakana_RI = 0x30EA
    EKatakana_RU = 0x30EB
    EKatakana_RE = 0x30EC
    EKatakana_RO = 0x30ED
    EKatakana_WA = 0x30EF
    EKatakana_N = 0x30F3
    EKatakana_MidDot = 0x30FB
    EKatakana_Prolong = 0x30FC
    EKatakanaHalf_MidDot = 0xFF65
    EKatakanaHalf_WO = 0xFF66
    EKatakanaHalf_SmallA = 0xFF67
    EKatakanaHalf_SmallI = 0xFF68
    EKatakanaHalf_SmallU = 0xFF69
    EKatakanaHalf_SmallE = 0xFF6A
    EKatakanaHalf_SmallO = 0xFF6B
    EKatakanaHalf_SmallYA = 0xFF6C
    EKatakanaHalf_SmallYU = 0xFF6D
    EKatakanaHalf_SmallYO = 0xFF6E
    EKatakanaHalf_SmallTSU = 0xFF6F
    EKatakanaHalf_Prolong = 0xFF70
    EKatakanaHalf_A = 0xFF71
    EKatakanaHalf_I = 0xFF72
    EKatakanaHalf_U = 0xFF73
    EKatakanaHalf_E = 0xFF74
    EKatakanaHalf_O = 0xFF75
    EKatakanaHalf_KA = 0xFF76
    EKatakanaHalf_KI = 0xFF77
    EKatakanaHalf_KU = 0xFF78
    EKatakanaHalf_KE = 0xFF79
    EKatakanaHalf_KO = 0xFF7A
    EKatakanaHalf_SA = 0xFF7B
    EKatakanaHalf_SHI = 0xFF7C
    EKatakanaHalf_SU = 0xFF7D
    EKatakanaHalf_SE = 0xFF7E
    EKatakanaHalf_SO = 0xFF7F
    EKatakanaHalf_TA = 0xFF80
    EKatakanaHalf_CHI = 0xFF81
    EKatakanaHalf_TSU = 0xFF82
    EKatakanaHalf_TE = 0xFF83
    EKatakanaHalf_TO = 0xFF84
    EKatakanaHalf_NA = 0xFF85
    EKatakanaHalf_NI = 0xFF86
    EKatakanaHalf_NU = 0xFF87
    EKatakanaHalf_NE = 0xFF88
    EKatakanaHalf_NO = 0xFF89
    EKatakanaHalf_HA = 0xFF8A
    EKatakanaHalf_HI = 0xFF8B
    EKatakanaHalf_FU = 0xFF8C
    EKatakanaHalf_HE = 0xFF8D
    EKatakanaHalf_HO = 0xFF8E
    EKatakanaHalf_MA = 0xFF8F
    EKatakanaHalf_MI = 0xFF90
    EKatakanaHalf_MU = 0xFF91
    EKatakanaHalf_ME = 0xFF92
    EKatakanaHalf_MO = 0xFF93
    EKatakanaHalf_YA = 0xFF94
    EKatakanaHalf_YU = 0xFF95
    EKatakanaHalf_YO = 0xFF96
    EKatakanaHalf_RA = 0xFF97
    EKatakanaHalf_RI = 0xFF98
    EKatakanaHalf_RU = 0xFF99
    EKatakanaHalf_RE = 0xFF9A
    EKatakanaHalf_RO = 0xFF9B
    EKatakanaHalf_WA = 0xFF9C
    EKatakanaHalf_N = 0xFF9D
    EKatakanaHalf_TENTEN = 0xFF9E
    EKatakanaHalf_MARU = 0xFF9F

class cTkMetaDataClass(ctypes.Structure):
    _fields_ = [
        ("mpacName", ctypes.c_char_p),
        ("muNameHash", ctypes.c_uint64),
        ("muTemplateHash", ctypes.c_uint64),
        ("maMembers", ctypes.c_ulonglong),
        ("miNumMembers", ctypes.c_int32),
    ]
    mpacName: bytes
    muNameHash: int
    muTemplateHash: int
    maMembers: int
    miNumMembers: int

class cTkMetaDataEnumLookup(ctypes.Structure):
    def Lookup(): pass

    _fields_ = [
        ("miValue", ctypes.c_int64),
        ("mpacName", ctypes.c_char_p),
    ]
    miValue: int
    mpacName: bytes

class FloatEditOptions(ctypes.Structure):
    def FloatEditOptions(): pass

    _fields_ = [
        ("mfStepSize", ctypes.c_float),
        ("miDecimalPrecision", ctypes.c_int32),
        ("mbUnspecified", ctypes.c_ubyte),
    ]
    mfStepSize: float
    miDecimalPrecision: int
    mbUnspecified: bool

class FloatLimits(ctypes.Structure):
    def FloatLimits(): pass

    _fields_ = [
        ("mfMin", ctypes.c_float),
        ("mfMax", ctypes.c_float),
        ("mbUnspecified", ctypes.c_ubyte),
    ]
    mfMin: float
    mfMax: float
    mbUnspecified: bool

class cTkNGui_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetFontID", ctypes.c_ulonglong),
        ("GetFontSpacing", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("IsHighlightActive", ctypes.c_ulonglong),
        ("IsNGUIEditor", ctypes.c_ulonglong),
    ]
    GetFontID: int
    GetFontSpacing: int
    IsActive: int
    IsHighlightActive: int
    IsNGUIEditor: int

class cTkVector2(ctypes.Structure):
    _fields_ = [
        ("mfX", ctypes.c_float),
        ("mfY", ctypes.c_float),
    ]
    mfX: float
    mfY: float

class cTkNGuiAnimationData(ctypes.Structure):
    def GetUnitTime(): pass

    def StartActivate(): pass

    def UpdateEvents(): pass

    _fields_ = [
        ("mafEventTime", ctypes.c_float * 0x2),  # float[2][6]
        ("mfUpdated", ctypes.c_float),
    ]
    mafEventTime: list[ctypes.c_float]
    mfUpdated: float

class cTkNGuiElementID(ctypes.Structure):
    _fields_ = [
        ("miCounter", ctypes.c_int64),
        ("miBase", ctypes.c_uint64),
        ("miFrameRenderTracker", ctypes.c_int32),
        ("miPerFrameUseCount", ctypes.c_int32),
    ]
    miCounter: int
    miBase: int
    miFrameRenderTracker: int
    miPerFrameUseCount: int

class cTkBBox2d(ctypes.Structure):
    def GetCenter(): pass

    _fields_ = [
        ("mMin", common.Vector2f),
        ("mMax", common.Vector2f),
    ]
    mMin: common.Vector2f
    mMax: common.Vector2f

class cTkNGuiGraphicStyleEntry(ctypes.Structure):
    _fields_ = [
        ("mpRenderFunction", ctypes.c_ulonglong),
        ("mpMarginFunction", ctypes.c_ulonglong),
        ("mpData", ctypes.c_void_p),
    ]
    mpRenderFunction: int
    mpMarginFunction: int
    mpData: int

class cTkNGuiTextStyleEntry(ctypes.Structure):
    _fields_ = [
        ("mpRenderFunction", ctypes.c_ulonglong),
        ("mpSizeFunction", ctypes.c_ulonglong),
        ("mpData", ctypes.c_void_p),
    ]
    mpRenderFunction: int
    mpSizeFunction: int
    mpData: int

class cTkNGuiLayoutListData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDefault", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbAutosave", ctypes.c_ubyte),
    ]
    macName: bytes
    macFilename: bytes
    macDefault: bytes
    mbAutosave: bool

class cTkNGuiLayoutList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maLayouts", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkNGuiLayoutListData>
    ]
    maLayouts: bytes

class cTkNGuiUserSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macLastLoadedModel", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("macLastActiveLayout", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("maacFileBrowserRecents", ctypes.c_char * 0xA00),  # cTkFixedArray<cTkFixedString<256,char>,10>
    ]
    macLastLoadedModel: bytes
    macLastActiveLayout: bytes
    maacFileBrowserRecents: bytes

class cTkNGuiEditorLayout(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maWindows", ctypes.c_char * 0x10A400),  # cTkFixedArray<cTkNGuiWindowLayoutData,256>
    ]
    maWindows: bytes

class cTkNGuiPopup(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpGui", ctypes.c_ulonglong),
        ("mPosition", common.Vector2f),
        ("mSize", common.Vector2f),
    ]
    __vftable: int
    mpGui: int
    mPosition: common.Vector2f
    mSize: common.Vector2f

class cTkNGuiPopup_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
    ]
    Render: int

class cTkComboBoxPopup_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
    ]
    Render: int

class cTkIconSelectPopup_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
    ]
    Render: int

class cTkNGuiTextEditState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkNGuiTextEditState_dtor_0", ctypes.c_ulonglong),
    ]
    cTkNGuiTextEditState_dtor_0: int

class StbUndoRecord(ctypes.Structure):
    _fields_ = [
        ("where", ctypes.c_int32),
        ("insert_length", ctypes.c_int16),
        ("delete_length", ctypes.c_int16),
        ("char_storage", ctypes.c_int16),
    ]
    where: int
    insert_length: int
    delete_length: int
    char_storage: int

class cTkNGuiScrollData(ctypes.Structure):
    _fields_ = [
        ("mfScrollX", ctypes.c_float),
        ("mfScrollY", ctypes.c_float),
    ]
    mfScrollX: float
    mfScrollY: float

class cTkNGuiTreeData(ctypes.Structure):
    _fields_ = [
        ("mbIsMaximised", ctypes.c_ubyte),
        ("mbTabled", ctypes.c_ubyte),
        ("mStartPos", common.Vector2f),
        ("mTitleStartPos", common.Vector2f),
        ("mfHeight", ctypes.c_float),
    ]
    mbIsMaximised: bool
    mbTabled: bool
    mStartPos: common.Vector2f
    mTitleStartPos: common.Vector2f
    mfHeight: float

class cTkNGuiToggleData(ctypes.Structure):
    _fields_ = [
        ("mbIsActive", ctypes.c_ubyte),
    ]
    mbIsActive: bool

class cTkNGuiPanelData(ctypes.Structure):
    _fields_ = [
        ("mfHeight", ctypes.c_float),
        ("mfLabelWidth", ctypes.c_float),
    ]
    mfHeight: float
    mfLabelWidth: float

class cTkNGuiUndoable(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def Icon(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mDescription", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    __vftable: int
    mDescription: bytes

class cTkNGuiUndoable_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkNGuiUndoable_dtor_0", ctypes.c_ulonglong),
        ("Undo", ctypes.c_ulonglong),
        ("Do", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("GetData", ctypes.c_ulonglong),
        ("ToString", ctypes.c_ulonglong),
        ("Icon", ctypes.c_ulonglong),
    ]
    cTkNGuiUndoable_dtor_0: int
    Undo: int
    Do: int
    End: int
    GetData: int
    ToString: int
    Icon: int

class cTkNGuiUndoStack(ctypes.Structure):
    _fields_ = [
        ("maUndoStack", ctypes.c_char * 0x18),  # std::vector<cTkNGuiUndoable *,TkSTLAllocatorShim<cTkNGuiUndoable *,8,-1> >
        ("miUndoStackIndex", ctypes.c_int32),
    ]
    maUndoStack: bytes
    miUndoStackIndex: int

class __m128: pass

class cTkVector4(ctypes.Structure):
    _fields_ = [
        ("mVal", common.Vector4f),
    ]
    mVal: common.Vector4f

class cTkColour(ctypes.Structure):
    def cTkColour(): pass

    def SetFromHSV(): pass

    def GetAsHSV(): pass

    def NmzInterpolate(): pass

    _fields_ = [
        ("baseclass_0", common.Vector4f),
    ]
    baseclass_0: common.Vector4f

class cTkNGuiEditorStyleColour(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mColour", common.Colour),
    ]
    macName: bytes
    mColour: common.Colour

class cTkNGuiEditorStyleData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mafSizes", ctypes.c_char * 0xC0),  # cTkFixedArray<float,48>
        ("maSkinColours", ctypes.c_char * 0x480),  # cTkFixedArray<cTkNGuiEditorStyleColour,8>
        ("mfSkinFontHeight", ctypes.c_float),
        ("macFont", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maGraphicStyles", ctypes.c_char * 0x8700),  # cTkFixedArray<cTkNGuiGraphicStyle,72>
        ("maTextStyles", ctypes.c_char * 0xFC0),  # cTkFixedArray<cTkNGuiTextStyle,14>
        ("mafSnapSettings", ctypes.c_char * 0x10),  # cTkDynamicArray<float>
    ]
    mafSizes: bytes
    maSkinColours: bytes
    mfSkinFontHeight: float
    macFont: bytes
    maGraphicStyles: bytes
    maTextStyles: bytes
    mafSnapSettings: bytes

class cTkNGuiEditor_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetFontID", ctypes.c_ulonglong),
        ("GetFontSpacing", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("IsHighlightActive", ctypes.c_ulonglong),
        ("IsNGUIEditor", ctypes.c_ulonglong),
    ]
    GetFontID: int
    GetFontSpacing: int
    IsActive: int
    IsHighlightActive: int
    IsNGUIEditor: int

class cTkAllocInfo(ctypes.Structure):
    _fields_ = [
        ("mHashKey", ctypes.c_char * 0x4),  # std::atomic<int>
        ("miLine", ctypes.c_int32),
        ("mpacFile", ctypes.c_char_p),
        ("mpacFunction", ctypes.c_char_p),
        ("miUsage", ctypes.c_char * 0x8),  # std::atomic<__int64>
        ("miMaxUsage", ctypes.c_char * 0x8),  # std::atomic<__int64>
        ("miNumAllocsLive", ctypes.c_char * 0x4),  # std::atomic<int>
        ("miNumAllocsTotal", ctypes.c_char * 0x4),  # std::atomic<int>
    ]
    mHashKey: bytes
    miLine: int
    mpacFile: bytes
    mpacFunction: bytes
    miUsage: bytes
    miMaxUsage: bytes
    miNumAllocsLive: bytes
    miNumAllocsTotal: bytes

class _RTL_CRITICAL_SECTION(ctypes.Structure):
    _fields_ = [
        ("DebugInfo", ctypes.c_ulonglong),
        ("LockCount", ctypes.c_int32),
        ("RecursionCount", ctypes.c_int32),
        ("OwningThread", ctypes.c_void_p),
        ("LockSemaphore", ctypes.c_void_p),
        ("SpinCount", ctypes.c_uint64),
    ]
    DebugInfo: int
    LockCount: int
    RecursionCount: int
    OwningThread: int
    LockSemaphore: int
    SpinCount: int

class _LIST_ENTRY(ctypes.Structure):
    _fields_ = [
        ("Flink", ctypes.c_ulonglong),
        ("Blink", ctypes.c_ulonglong),
    ]
    Flink: int
    Blink: int

class cTkMemoryPoolDefinition(ctypes.Structure):
    _fields_ = [
        ("macName", ctypes.c_char_p),
        ("mePool", ctypes.c_int32),
        ("meBackingPool", ctypes.c_int32),
    ]
    macName: bytes
    mePool: int
    meBackingPool: int

class cTkMemoryPool(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def Realloc(): pass

    def GetHighWaterUsage(): pass

    def GetAllocInfo(): pass

    def FlagError(): pass

    def HasError(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("miType", ctypes.c_int32),
        ("miAlignment", ctypes.c_int32),
        ("muTotalSize", ctypes.c_uint64),
        ("mxErrors", ctypes.c_char * 0x4),  # cTkBitArray<unsigned int,1,32>
    ]
    __vftable: int
    miType: int
    miAlignment: int
    muTotalSize: int
    mxErrors: bytes

class cTkMemoryPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkMemoryPool_dtor_0", ctypes.c_ulonglong),
        ("Create", ctypes.c_ulonglong),
        ("Malloc", ctypes.c_ulonglong),
        ("Malloc_18", ctypes.c_ulonglong),
        ("Free", ctypes.c_ulonglong),
        ("Realloc", ctypes.c_ulonglong),
        ("ContainsAddress", ctypes.c_ulonglong),
        ("GetAllocationSize", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("GetUsage", ctypes.c_ulonglong),
        ("GetHighWaterUsage", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("Cleanup", ctypes.c_ulonglong),
        ("GetAllocInfo", ctypes.c_ulonglong),
        ("SetAllocInfo", ctypes.c_ulonglong),
        ("ClearStats", ctypes.c_ulonglong),
        ("OutputStats", ctypes.c_ulonglong),
        ("FrameSyncEnd", ctypes.c_ulonglong),
        ("NeedsFrameSyncEnd", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("FlagError", ctypes.c_ulonglong),
        ("HasError", ctypes.c_ulonglong),
        ("NeedsMutex", ctypes.c_ulonglong),
        ("SetMutex", ctypes.c_ulonglong),
        ("GetOverflowPool", ctypes.c_ulonglong),
        ("GetBaseAddress", ctypes.c_ulonglong),
    ]
    cTkMemoryPool_dtor_0: int
    Create: int
    Malloc: int
    Malloc_18: int
    Free: int
    Realloc: int
    ContainsAddress: int
    GetAllocationSize: int
    SetAlignment: int
    GetAlignment: int
    GetUsage: int
    GetHighWaterUsage: int
    GetSize: int
    Cleanup: int
    GetAllocInfo: int
    SetAllocInfo: int
    ClearStats: int
    OutputStats: int
    FrameSyncEnd: int
    NeedsFrameSyncEnd: int
    Reset: int
    FlagError: int
    HasError: int
    NeedsMutex: int
    SetMutex: int
    GetOverflowPool: int
    GetBaseAddress: int

class cTkLinearMemoryPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkMemoryPool_dtor_0", ctypes.c_ulonglong),
        ("Create", ctypes.c_ulonglong),
        ("Malloc", ctypes.c_ulonglong),
        ("Malloc_18", ctypes.c_ulonglong),
        ("Free", ctypes.c_ulonglong),
        ("Realloc", ctypes.c_ulonglong),
        ("ContainsAddress", ctypes.c_ulonglong),
        ("GetAllocationSize", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("GetUsage", ctypes.c_ulonglong),
        ("GetHighWaterUsage", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("Cleanup", ctypes.c_ulonglong),
        ("GetAllocInfo", ctypes.c_ulonglong),
        ("SetAllocInfo", ctypes.c_ulonglong),
        ("ClearStats", ctypes.c_ulonglong),
        ("OutputStats", ctypes.c_ulonglong),
        ("FrameSyncEnd", ctypes.c_ulonglong),
        ("NeedsFrameSyncEnd", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("FlagError", ctypes.c_ulonglong),
        ("HasError", ctypes.c_ulonglong),
        ("NeedsMutex", ctypes.c_ulonglong),
        ("SetMutex", ctypes.c_ulonglong),
        ("GetOverflowPool", ctypes.c_ulonglong),
        ("GetBaseAddress", ctypes.c_ulonglong),
    ]
    cTkMemoryPool_dtor_0: int
    Create: int
    Malloc: int
    Malloc_18: int
    Free: int
    Realloc: int
    ContainsAddress: int
    GetAllocationSize: int
    SetAlignment: int
    GetAlignment: int
    GetUsage: int
    GetHighWaterUsage: int
    GetSize: int
    Cleanup: int
    GetAllocInfo: int
    SetAllocInfo: int
    ClearStats: int
    OutputStats: int
    FrameSyncEnd: int
    NeedsFrameSyncEnd: int
    Reset: int
    FlagError: int
    HasError: int
    NeedsMutex: int
    SetMutex: int
    GetOverflowPool: int
    GetBaseAddress: int

class XMLClear(ctypes.Structure):
    _fields_ = [
        ("lpszValue", ctypes.c_char_p),
        ("lpszOpenTag", ctypes.c_char_p),
        ("lpszCloseTag", ctypes.c_char_p),
    ]
    lpszValue: bytes
    lpszOpenTag: bytes
    lpszCloseTag: bytes

class XMLAttribute(ctypes.Structure):
    _fields_ = [
        ("lpszName", ctypes.c_char_p),
        ("lpszValue", ctypes.c_char_p),
    ]
    lpszName: bytes
    lpszValue: bytes

class cTkPostProcessData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfDOFNearPlane", ctypes.c_float),
        ("mfDOFFarPlane", ctypes.c_float),
        ("mfDOFNearAmount", ctypes.c_float),
        ("mfDOFFarAmount", ctypes.c_float),
        ("mfSaturationDepth", ctypes.c_float),
        ("mfBrightnessDepth", ctypes.c_float),
        ("mfContrastDepth", ctypes.c_float),
        ("mfSaturationFinal", ctypes.c_float),
        ("mfBrightnessFinal", ctypes.c_float),
        ("mfContrastFinal", ctypes.c_float),
    ]
    mfDOFNearPlane: float
    mfDOFFarPlane: float
    mfDOFNearAmount: float
    mfDOFFarAmount: float
    mfSaturationDepth: float
    mfBrightnessDepth: float
    mfContrastDepth: float
    mfSaturationFinal: float
    mfBrightnessFinal: float
    mfContrastFinal: float

class ITkDocumentWriter(ctypes.Structure):
    def tilde_ITkDocumentWriter(): pass

    def scalar_deleting_destructor(): pass

    def AddValue(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkSeed(ctypes.Structure):
    def cTkSeed(): pass

    def NGuiRender(): pass

    _fields_ = [
        ("mu64SeedValue", ctypes.c_uint64),
        ("mbValid", ctypes.c_ubyte),
    ]
    mu64SeedValue: int
    mbValid: bool

class cTkHalfVector4(ctypes.Structure):
    _fields_ = [
        ("mfX", ctypes.c_uint16),
        ("mfY", ctypes.c_uint16),
        ("mfZ", ctypes.c_uint16),
        ("mfW", ctypes.c_uint16),
    ]
    mfX: int
    mfY: int
    mfZ: int
    mfW: int

class cTkVector3(ctypes.Structure):
    def cTkVector3(): pass

    _fields_ = [
        ("mVal", common.Vector4f),
    ]
    mVal: common.Vector4f

class ITkDocumentWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkDocumentWriter_dtor_0", ctypes.c_ulonglong),
        ("GetWriterFormat", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(ITkDocumentWriter *this, TkID<128> *result)
        ("Clear", ctypes.c_ulonglong),
        ("PushContext", ctypes.c_ulonglong),
        ("PopContext", ctypes.c_ulonglong),
        ("OpenArray", ctypes.c_ulonglong),
        ("OpenObject", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("AddValue", ctypes.c_ulonglong),
        ("AddValue_48", ctypes.c_ulonglong),
        ("AddValue_50", ctypes.c_ulonglong),
        ("AddValue_58", ctypes.c_ulonglong),
        ("AddValue_60", ctypes.c_char * 0x8),  # void (__fastcall *)(ITkDocumentWriter *this, const TkID<256> *)
        ("AddValue_68", ctypes.c_char * 0x8),  # void (__fastcall *)(ITkDocumentWriter *this, const TkID<128> *)
        ("AddValue_70", ctypes.c_ulonglong),
        ("AddValue_78", ctypes.c_ulonglong),
        ("AddValue_80", ctypes.c_ulonglong),
        ("AddValue_88", ctypes.c_ulonglong),
        ("AddValue_90", ctypes.c_ulonglong),
        ("AddValue_98", ctypes.c_ulonglong),
        ("AddValue_a0", ctypes.c_ulonglong),
        ("AddValue_a8", ctypes.c_ulonglong),
        ("AddValue_b0", ctypes.c_ulonglong),
        ("AddValue_b8", ctypes.c_ulonglong),
        ("AddValue_c0", ctypes.c_ulonglong),
        ("AddKey", ctypes.c_ulonglong),
    ]
    ITkDocumentWriter_dtor_0: int
    GetWriterFormat: bytes
    Clear: int
    PushContext: int
    PopContext: int
    OpenArray: int
    OpenObject: int
    Close: int
    AddValue: int
    AddValue_48: int
    AddValue_50: int
    AddValue_58: int
    AddValue_60: bytes
    AddValue_68: bytes
    AddValue_70: int
    AddValue_78: int
    AddValue_80: int
    AddValue_88: int
    AddValue_90: int
    AddValue_98: int
    AddValue_a0: int
    AddValue_a8: int
    AddValue_b0: int
    AddValue_b8: int
    AddValue_c0: int
    AddKey: int

class cTkDynamicWideString(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # cTkDynamicArray<wchar_t>
    ]
    baseclass_0: bytes

class cTkDynamicString(ctypes.Structure):
    def AllocateFromString(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # cTkDynamicArray<char>
    ]
    baseclass_0: bytes

class ITkDocumentReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkDocumentReader_dtor_0", ctypes.c_ulonglong),
        ("GetReaderFormat", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(ITkDocumentReader *this, TkID<128> *result)
        ("PushContext", ctypes.c_ulonglong),
        ("PopContext", ctypes.c_ulonglong),
        ("MoveCursor", ctypes.c_ulonglong),
        ("MoveCursor_28", ctypes.c_ulonglong),
        ("PopCursor", ctypes.c_ulonglong),
        ("IsCursorNull", ctypes.c_ulonglong),
        ("ReadArraySize", ctypes.c_ulonglong),
        ("ReadStringSize", ctypes.c_ulonglong),
        ("ReadValue", ctypes.c_ulonglong),
        ("ReadValue_58", ctypes.c_ulonglong),
        ("ReadValue_60", ctypes.c_ulonglong),
        ("ReadValue_68", ctypes.c_ulonglong),
        ("ReadValue_70", ctypes.c_ulonglong),
        ("ReadValue_78", ctypes.c_ulonglong),
        ("ReadValue_80", ctypes.c_ulonglong),
        ("ReadValue_88", ctypes.c_ulonglong),
        ("ReadValue_90", ctypes.c_ulonglong),
        ("ReadValue_98", ctypes.c_ulonglong),
        ("ReadValue_a0", ctypes.c_ulonglong),
        ("ReadValue_a8", ctypes.c_ulonglong),
        ("ReadValue_b0", ctypes.c_char * 0x8),  # ITkDocumentReader::ReadResult (__fastcall *)(ITkDocumentReader *this, TkID<256> *)
        ("ReadValue_b8", ctypes.c_char * 0x8),  # ITkDocumentReader::ReadResult (__fastcall *)(ITkDocumentReader *this, TkID<128> *)
        ("ReadValue_c0", ctypes.c_ulonglong),
        ("ReadValue_c8", ctypes.c_ulonglong),
        ("ReadValue_d0", ctypes.c_ulonglong),
        ("ReadValue_d8", ctypes.c_ulonglong),
        ("ReadValue_e0", ctypes.c_ulonglong),
        ("ReadValue_e8", ctypes.c_ulonglong),
        ("ReadValue_f0", ctypes.c_ulonglong),
        ("ReadValue_f8", ctypes.c_ulonglong),
        ("ReadValue_100", ctypes.c_ulonglong),
        ("ReadValue_108", ctypes.c_ulonglong),
        ("ReadValue_110", ctypes.c_ulonglong),
    ]
    ITkDocumentReader_dtor_0: int
    GetReaderFormat: bytes
    PushContext: int
    PopContext: int
    MoveCursor: int
    MoveCursor_28: int
    PopCursor: int
    IsCursorNull: int
    ReadArraySize: int
    ReadStringSize: int
    ReadValue: int
    ReadValue_58: int
    ReadValue_60: int
    ReadValue_68: int
    ReadValue_70: int
    ReadValue_78: int
    ReadValue_80: int
    ReadValue_88: int
    ReadValue_90: int
    ReadValue_98: int
    ReadValue_a0: int
    ReadValue_a8: int
    ReadValue_b0: bytes
    ReadValue_b8: bytes
    ReadValue_c0: int
    ReadValue_c8: int
    ReadValue_d0: int
    ReadValue_d8: int
    ReadValue_e0: int
    ReadValue_e8: int
    ReadValue_f0: int
    ReadValue_f8: int
    ReadValue_100: int
    ReadValue_108: int
    ReadValue_110: int

class cTkPlatformButtonPair(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPlatformId", ctypes.c_ulonglong),
        ("mButtonId", ctypes.c_ulonglong),
        ("mSize", common.Vector2f),
    ]
    mPlatformId: int
    mButtonId: int
    mSize: common.Vector2f

class cTkLSystemGlobalRestriction(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macModel", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maRestrictions", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLSystemRestrictionData>
    ]
    macName: bytes
    macModel: bytes
    maRestrictions: bytes

class cTkRandomComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miSeed", ctypes.c_int32),
    ]
    miSeed: int

class cTkNoiseSuperFormulaData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfForm_m", ctypes.c_float),
        ("mfForm_n1", ctypes.c_float),
        ("mfForm_n2", ctypes.c_float),
        ("mfForm_n3", ctypes.c_float),
    ]
    mfForm_m: float
    mfForm_n1: float
    mfForm_n2: float
    mfForm_n3: float

class cTkNoiseSuperPrimitiveData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfWidth", ctypes.c_float),
        ("mfHeight", ctypes.c_float),
        ("mfDepth", ctypes.c_float),
        ("mfThickness", ctypes.c_float),
        ("mfCornerRadiusXY", ctypes.c_float),
        ("mfCornerRadiusZ", ctypes.c_float),
        ("mfBottomRadiusOffset", ctypes.c_float),
    ]
    mfWidth: float
    mfHeight: float
    mfDepth: float
    mfThickness: float
    mfCornerRadiusXY: float
    mfCornerRadiusZ: float
    mfBottomRadiusOffset: float

class cTkTrailData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfWidth", ctypes.c_float),
        ("miPoints", ctypes.c_int32),
        ("miMaxPointsPerFrame", ctypes.c_int32),
        ("mfDistanceThreshold", ctypes.c_float),
        ("mfPointLife", ctypes.c_float),
        ("miFrontPoints", ctypes.c_int32),
        ("mfFrontUvEnd", ctypes.c_float),
    ]
    mfWidth: float
    miPoints: int
    miMaxPointsPerFrame: int
    mfDistanceThreshold: float
    mfPointLife: float
    miFrontPoints: int
    mfFrontUvEnd: float

class cTkActionButtonLookup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maLookup", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkActionButtonMap>
    ]
    maLookup: bytes

class cTkLSystemGlobalVariation(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macModel", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miVariations", ctypes.c_int32),
    ]
    macName: bytes
    macModel: bytes
    miVariations: int

class cTkEmitterData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macParticle", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macParticle: bytes

class cTkImGuiWindowData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macType", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miWindowX", ctypes.c_int32),
        ("miWindowY", ctypes.c_int32),
        ("miWindowW", ctypes.c_int32),
        ("miWindowH", ctypes.c_int32),
        ("miWindowMinW", ctypes.c_int32),
        ("miWindowMinH", ctypes.c_int32),
        ("mbWindowResize", ctypes.c_ubyte),
        ("miWindowScroll", ctypes.c_int32),
        ("miWindowTab", ctypes.c_int32),
        ("mbWindowOpen", ctypes.c_ubyte),
        ("mbWindowMinimised", ctypes.c_ubyte),
        ("mbWindowUsed", ctypes.c_ubyte),
    ]
    macType: bytes
    miWindowX: int
    miWindowY: int
    miWindowW: int
    miWindowH: int
    miWindowMinW: int
    miWindowMinH: int
    mbWindowResize: bool
    miWindowScroll: int
    miWindowTab: int
    mbWindowOpen: bool
    mbWindowMinimised: bool
    mbWindowUsed: bool

class cTkLSystemLocatorEntry(ctypes.Structure):
    def tilde_cTkLSystemLocatorEntry(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macModel", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfProbability", ctypes.c_float),
        ("maRestrictions", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLSystemRestrictionData>
    ]
    macModel: bytes
    mfProbability: float
    maRestrictions: bytes

class cTkAxisImageLookup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maLookup", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAxisPathMapping>
    ]
    maLookup: bytes

class cTkLSystemRuleTemplate(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macLSystem", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macName: bytes
    macLSystem: bytes

class cTkFoliageData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMaterial", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfScale", ctypes.c_float),
        ("mfDensity", ctypes.c_float),
        ("mfDensityVariance", ctypes.c_float),
        ("mfAngleMultiplier", ctypes.c_float),
        ("mbAngleExponentially", ctypes.c_ubyte),
        ("mColour", common.Colour),
    ]
    macMaterial: bytes
    mfScale: float
    mfDensity: float
    mfDensityVariance: float
    mfAngleMultiplier: float
    mbAngleExponentially: bool
    mColour: common.Colour

class cTkButtonImageLookup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maLookup", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkButtonPathMapping>
    ]
    maLookup: bytes

class cTkLSystemRulesData(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGlobalRestriction", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLSystemGlobalRestriction>
        ("maGlobalVariation", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLSystemGlobalVariation>
        ("maRules", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLSystemRule>
        ("maTemplates", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLSystemRuleTemplate>
    ]
    maGlobalRestriction: bytes
    maGlobalVariation: bytes
    maRules: bytes
    maTemplates: bytes

class cTkHeavyAirCollection(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maHeavyAirSystems", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkHeavyAirData>
    ]
    maHeavyAirSystems: bytes

class cTkChordPathMapping(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maButtonIds", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkInputEnum>
        ("mName", ctypes.c_ulonglong),
        ("mTextTag", ctypes.c_ulonglong),
        ("macSolidIcon", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macOverlayIcon", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macSpecialIcon", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    maButtonIds: bytes
    mName: int
    mTextTag: int
    macSolidIcon: bytes
    macOverlayIcon: bytes
    macSpecialIcon: bytes

class cTkHeavyAirSystem(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMaterial", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mMajorDirection", common.Vector3f),
        ("mScaleRange", common.Vector3f),
        ("mRotationSpeedRange", common.Vector3f),
        ("mFadeSpeedRange", common.Vector3f),
        ("mTwinkleRange", common.Vector3f),
        ("mAmplitudeMin", common.Vector3f),
        ("mAmplitudeMax", common.Vector3f),
        ("mColour1", common.Colour),
        ("mfColour1Alpha", ctypes.c_float),
        ("mColour2", common.Colour),
        ("mfColour2Alpha", ctypes.c_float),
    ]
    macMaterial: bytes
    mMajorDirection: common.Vector3f
    mScaleRange: common.Vector3f
    mRotationSpeedRange: common.Vector3f
    mFadeSpeedRange: common.Vector3f
    mTwinkleRange: common.Vector3f
    mAmplitudeMin: common.Vector3f
    mAmplitudeMax: common.Vector3f
    mColour1: common.Colour
    mfColour1Alpha: float
    mColour2: common.Colour
    mfColour2Alpha: float

class cTkVoxelGeneratorRegionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfPlanetRadius", ctypes.c_float),
        ("mfVoronoiPointDivisions", ctypes.c_float),
        ("miVoronoiSectorSeed", ctypes.c_int32),
        ("miVoronoiPointSeed", ctypes.c_int32),
        ("maFlattenTerrainPoints", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkNoiseFlattenPoint>
        ("mafFlattenTypeChances", ctypes.c_char * 0x10),  # cTkDynamicArray<float>
        ("miWaypointIndex", ctypes.c_int32),
        ("miLandingPadIndex", ctypes.c_int32),
        ("mfAddShelterChance", ctypes.c_float),
        ("maiShelterIndices", ctypes.c_char * 0xC),  # cTkFixedArray<int,3>
        ("miNumShelters", ctypes.c_int32),
    ]
    mfPlanetRadius: float
    mfVoronoiPointDivisions: float
    miVoronoiSectorSeed: int
    miVoronoiPointSeed: int
    maFlattenTerrainPoints: bytes
    mafFlattenTypeChances: bytes
    miWaypointIndex: int
    miLandingPadIndex: int
    mfAddShelterChance: float
    maiShelterIndices: bytes
    miNumShelters: int

class cTkIdSceneFilename(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mId: int
    macFilename: bytes

class cTkChordsImageLookup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maLookup", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkChordPathMapping>
    ]
    maLookup: bytes

class cTkImGuiSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mBackgroundColour", common.Colour),
        ("mWindowTitleColour", common.Colour),
        ("mActiveWindowTitleColour", common.Colour),
        ("mWindowBackgroundColour", common.Colour),
        ("mActiveWindowBackgroundColour", common.Colour),
        ("mWindowHighlight", common.Colour),
        ("mTextColour", common.Colour),
        ("mActiveTextColour", common.Colour),
        ("mTextShadowColour", common.Colour),
        ("mTextDisabledColour", common.Colour),
        ("mButtonColour", common.Colour),
        ("mButtonHighlightColour", common.Colour),
        ("mButtonPressedColour", common.Colour),
        ("mEditBoxColour", common.Colour),
        ("mEditBoxActiveColour", common.Colour),
        ("mEditBoxSelectedColour", common.Colour),
        ("mTaskBarColour", common.Colour),
        ("mTaskBarShadow", common.Colour),
        ("mCloseButtonColour", common.Colour),
        ("mCloseButtonHighlightColour", common.Colour),
        ("mCloseButtonClickColour", common.Colour),
        ("mMinimiseButtonColour", common.Colour),
        ("mMinimiseButtonHighlightColour", common.Colour),
        ("mMinimiseButtonClickColour", common.Colour),
        ("mfScrollSpeed", ctypes.c_float),
        ("mfAltPlacementDistanceScrollSpeed", ctypes.c_float),
    ]
    mBackgroundColour: common.Colour
    mWindowTitleColour: common.Colour
    mActiveWindowTitleColour: common.Colour
    mWindowBackgroundColour: common.Colour
    mActiveWindowBackgroundColour: common.Colour
    mWindowHighlight: common.Colour
    mTextColour: common.Colour
    mActiveTextColour: common.Colour
    mTextShadowColour: common.Colour
    mTextDisabledColour: common.Colour
    mButtonColour: common.Colour
    mButtonHighlightColour: common.Colour
    mButtonPressedColour: common.Colour
    mEditBoxColour: common.Colour
    mEditBoxActiveColour: common.Colour
    mEditBoxSelectedColour: common.Colour
    mTaskBarColour: common.Colour
    mTaskBarShadow: common.Colour
    mCloseButtonColour: common.Colour
    mCloseButtonHighlightColour: common.Colour
    mCloseButtonClickColour: common.Colour
    mMinimiseButtonColour: common.Colour
    mMinimiseButtonHighlightColour: common.Colour
    mMinimiseButtonClickColour: common.Colour
    mfScrollSpeed: float
    mfAltPlacementDistanceScrollSpeed: float

class cTkControllerButtonLookup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("macButtonImageLookupFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mId: int
    macButtonImageLookupFilename: bytes

class cTkInputFrame(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mLeftStick", common.Vector2f),
        ("mRightStick", common.Vector2f),
        ("mfLeftTrigger", ctypes.c_float),
        ("mfRightTrigger", ctypes.c_float),
        ("miButtons", ctypes.c_int16),
    ]
    mLeftStick: common.Vector2f
    mRightStick: common.Vector2f
    mfLeftTrigger: float
    mfRightTrigger: float
    miButtons: int

class cTkControllerList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maControllers", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkControllerButtonLookup>
    ]
    maControllers: bytes

class cTkInputFrameArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maArray", ctypes.c_char * 0x88B80),  # cTkFixedArray<cTkInputFrame,20000>
    ]
    maArray: bytes

class cGcInputActionMapping(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miRemappedKey", ctypes.c_int32),
        ("miRemappedPad", ctypes.c_int32),
    ]
    miRemappedKey: int
    miRemappedPad: int

class cGcInputActionMapping2(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macActionSet", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macAction", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macButton", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macAxis", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
    ]
    macActionSet: bytes
    macAction: bytes
    macButton: bytes
    macAxis: bytes

class cTkOpenVRControllerLookup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macDeviceKeywords", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macDeviceSpec", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mResetVRViewLayerName", ctypes.c_ulonglong),
    ]
    macDeviceKeywords: bytes
    macDeviceSpec: bytes
    mResetVRViewLayerName: int

class cTkOpenVRControllerList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maDevices", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkOpenVRControllerLookup>
    ]
    maDevices: bytes

class cTkMagicModelData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maVertices", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkVector3>
        ("mCentre", common.Vector3f),
        ("mfRadius", ctypes.c_float),
    ]
    maVertices: bytes
    mCentre: common.Vector3f
    mfRadius: float

class cTkStringAPI(ctypes.Structure):
    def Format(): pass


class __crt_locale_pointers(ctypes.Structure):
    _fields_ = [
        ("locinfo", ctypes.c_ulonglong),
        ("mbcinfo", ctypes.c_ulonglong),
    ]
    locinfo: int
    mbcinfo: int

class cTkMetaDataXML(ctypes.Structure):
    def ReadElementXML(): pass

    def WriteXML(): pass

    def IsDefault(): pass

    def GetLookup(): pass

    def ParseXML(): pass

    def ReadClassPointer(): pass

    def ReadClassPointerArrayXML(): pass

    def Register(): pass

    def WriteClassPointer(): pass

    def WriteClassPointerArrayXML(): pass

    def WriteToXML(): pass


class cTkMaterialShaderMillConnect(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("miCount", ctypes.c_int32),
        ("mbExpanded", ctypes.c_ubyte),
    ]
    macName: bytes
    miCount: int
    mbExpanded: bool

class cTkMaterialShaderMillNode(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miId", ctypes.c_int32),
        ("macType", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macValue", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("miIValue", ctypes.c_int32),
        ("miIValue2", ctypes.c_int32),
        ("mfFValue", ctypes.c_float),
        ("mfFValue2", ctypes.c_float),
        ("maInputs", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialShaderMillConnect>
        ("maOutputs", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialShaderMillConnect>
        ("miWindowX", ctypes.c_int32),
        ("miWindowY", ctypes.c_int32),
    ]
    miId: int
    macType: bytes
    macValue: bytes
    miIValue: int
    miIValue2: int
    mfFValue: float
    mfFValue2: float
    maInputs: bytes
    maOutputs: bytes
    miWindowX: int
    miWindowY: int

class cTkVoxelGeneratorSettingsArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTerrainSettings", ctypes.c_char * 0x39420),  # cTkFixedArray<cTkVoxelGeneratorSettingsElement,21>
    ]
    maTerrainSettings: bytes

class cTkMaterialShaderMillLink(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miInputNode", ctypes.c_int32),
        ("macInputConnect", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mInputShuffle", ctypes.c_ulonglong),
        ("miOutputNode", ctypes.c_int32),
        ("macOutputConnect", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mOutputShuffle", ctypes.c_ulonglong),
        ("miCount", ctypes.c_int32),
    ]
    miInputNode: int
    macInputConnect: bytes
    mInputShuffle: int
    miOutputNode: int
    macOutputConnect: bytes
    mOutputShuffle: int
    miCount: int

class cTkMaterialShaderMillFlag(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macFlag", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
    ]
    macFlag: bytes

class cTkMaterialShaderMillComment(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macText", ctypes.c_char * 0x400),  # cTkFixedString<1024,char>
        ("miPosMinX", ctypes.c_int32),
        ("miPosMinY", ctypes.c_int32),
        ("miPosMaxX", ctypes.c_int32),
        ("miPosMaxY", ctypes.c_int32),
    ]
    macText: bytes
    miPosMinX: int
    miPosMinY: int
    miPosMaxX: int
    miPosMaxY: int

class cTkMaterialShaderMillData(ctypes.Structure):
    def tilde_cTkMaterialShaderMillData(): pass

    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maShaderMillNodes", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialShaderMillNode>
        ("maShaderMillLinks", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialShaderMillLink>
        ("maShaderMillFlags", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialShaderMillFlag>
        ("maShaderMillCmts", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialShaderMillComment>
        ("mfScrollX", ctypes.c_float),
        ("mfScrollY", ctypes.c_float),
        ("mfZoom", ctypes.c_float),
        ("miOutputX", ctypes.c_int32),
        ("miOutputY", ctypes.c_int32),
    ]
    maShaderMillNodes: bytes
    maShaderMillLinks: bytes
    maShaderMillFlags: bytes
    maShaderMillCmts: bytes
    mfScrollX: float
    mfScrollY: float
    mfZoom: float
    miOutputX: int
    miOutputY: int

class type_info(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("vfptr", ctypes.c_char * 0x8),
        ("_M_data", ctypes.c_char * 0x8),
        ("_M_d_name", ctypes.c_char * 0x0),
    ]
    vfptr: bytes
    _M_data: bytes
    _M_d_name: bytes

class type_info_vtbl(ctypes.Structure):
    _fields_ = [
        ("type_info_dtor_0", ctypes.c_ulonglong),
    ]
    type_info_dtor_0: int

class __std_type_info_data(ctypes.Structure):
    _fields_ = [
        ("_UndecoratedName", ctypes.c_char_p),
        ("_DecoratedName", ctypes.c_char * 0x1),  # char[1]
    ]
    _UndecoratedName: bytes
    _DecoratedName: list[ctypes.c_char]

class cTkFloatRange(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMinimum", ctypes.c_float),
        ("mfMaximum", ctypes.c_float),
    ]
    mfMinimum: float
    mfMaximum: float

class cTkBaseMath(ctypes.Structure):
    class USplitBits: pass

    def FloatToHalfI(): pass

    def FloatToHalf(): pass


class FIOS2HANDLE(ctypes.Structure):
    _fields_ = [
        ("mFH", ctypes.c_int32),
    ]
    mFH: int

class SceFiosPsarcDearchiverContext(ctypes.Structure):
    _fields_ = [
        ("sizeOfContext", ctypes.c_uint64),
        ("workBufferSize", ctypes.c_uint64),
        ("pWorkBuffer", ctypes.c_void_p),
        ("flags", ctypes.c_int64),
        ("reserved", ctypes.c_int64 * 0x3),  # __int64[3]
    ]
    sizeOfContext: int
    workBufferSize: int
    pWorkBuffer: int
    flags: int
    reserved: list[ctypes.c_int64]

class SceFiosBuffer(ctypes.Structure):
    _fields_ = [
        ("pPtr", ctypes.c_void_p),
        ("length", ctypes.c_uint64),
    ]
    pPtr: int
    length: int

class SceFiosRamCacheContext(ctypes.Structure):
    _fields_ = [
        ("sizeOfContext", ctypes.c_uint64),
        ("workBufferSize", ctypes.c_uint64),
        ("blockSize", ctypes.c_uint64),
        ("pWorkBuffer", ctypes.c_void_p),
        ("pPath", ctypes.c_char_p),
        ("flags", ctypes.c_int64),
        ("pCallback", ctypes.c_ulonglong),
        ("pCallbackData", ctypes.c_void_p),
        ("reserved", ctypes.c_int64),
    ]
    sizeOfContext: int
    workBufferSize: int
    blockSize: int
    pWorkBuffer: int
    pPath: bytes
    flags: int
    pCallback: int
    pCallbackData: int
    reserved: int

class cTkMetaDataFunctionLookup(ctypes.Structure):
    _fields_ = [
        ("mpClassMetadata", ctypes.c_ulonglong),
        ("mCreateDefaultFunction", ctypes.c_ulonglong),
        ("mRenderFunction", ctypes.c_ulonglong),
        ("mFixingFunction", ctypes.c_ulonglong),
        ("mValidateDataFunction", ctypes.c_ulonglong),
        ("mEqualsFunction", ctypes.c_ulonglong),
        ("mCopyFunction", ctypes.c_ulonglong),
        ("mCreatePtrFunction", ctypes.c_ulonglong),
        ("mHashFunction", ctypes.c_ulonglong),
        ("mDestroyFunction", ctypes.c_ulonglong),
    ]
    mpClassMetadata: int
    mCreateDefaultFunction: int
    mRenderFunction: int
    mFixingFunction: int
    mValidateDataFunction: int
    mEqualsFunction: int
    mCopyFunction: int
    mCreatePtrFunction: int
    mHashFunction: int
    mDestroyFunction: int

class cTkByteSwap(ctypes.Structure):
    pass

class SpookyHash(ctypes.Structure):
    def Final(): pass

    def Hash128(): pass

    def Init(): pass

    def Short(): pass

    def Update(): pass

    _fields_ = [
        ("m_data", ctypes.c_uint64 * 0x18),  # unsigned __int64[24]
        ("m_state", ctypes.c_uint64 * 0xC),  # unsigned __int64[12]
        ("m_length", ctypes.c_uint64),
        ("m_remainder", ctypes.c_uint8),
    ]
    m_data: list[ctypes.c_uint64]
    m_state: list[ctypes.c_uint64]
    m_length: int
    m_remainder: int

class cTkVertexLayout(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miElementCount", ctypes.c_int32),
        ("miStride", ctypes.c_int32),
        ("mi64PlatformData", ctypes.c_int64),
        ("maVertexElements", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkVertexElement>
    ]
    miElementCount: int
    miStride: int
    mi64PlatformData: int
    maVertexElements: bytes

class cTkVertexStream(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maVertexStream", ctypes.c_char * 0x10),  # cTkDynamicArray<unsigned char>
    ]
    maVertexStream: bytes

class cTkRagdollData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDensity", ctypes.c_float),
        ("mfGravity", ctypes.c_float),
        ("maacChainEnds", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<32,char> >
        ("maacExcludeJoints", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<32,char> >
    ]
    mfDensity: float
    mfGravity: float
    maacChainEnds: bytes
    maacExcludeJoints: bytes

class cTkMaterialUniform(ctypes.Structure):
    def cTkMaterialUniform(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mValues", common.Vector4f),
        ("maExtendedValues", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkVector4>
    ]
    macName: bytes
    mValues: common.Vector4f
    maExtendedValues: bytes

class cTkMaterialData(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMetamaterial", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("macClass", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("miTransparencyLayerID", ctypes.c_int32),
        ("mbCastShadow", ctypes.c_ubyte),
        ("mbDisableZTest", ctypes.c_ubyte),
        ("mbCreateFur", ctypes.c_ubyte),
        ("macLink", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macShader", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maFlags", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialFlags>
        ("maUniforms", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialUniform>
        ("maSamplers", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMaterialSampler>
        ("mi64ShaderMillDataHash", ctypes.c_int64),
    ]
    macName: bytes
    macMetamaterial: bytes
    macClass: bytes
    miTransparencyLayerID: int
    mbCastShadow: bool
    mbDisableZTest: bool
    mbCreateFur: bool
    macLink: bytes
    macShader: bytes
    maFlags: bytes
    maUniforms: bytes
    maSamplers: bytes
    mi64ShaderMillDataHash: int

class cTkPhysicsData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMass", ctypes.c_float),
        ("mfFriction", ctypes.c_float),
        ("mfRollingFriction", ctypes.c_float),
        ("mfAngularDamping", ctypes.c_float),
        ("mfLinearDamping", ctypes.c_float),
        ("mfGravity", ctypes.c_float),
    ]
    mfMass: float
    mfFriction: float
    mfRollingFriction: float
    mfAngularDamping: float
    mfLinearDamping: float
    mfGravity: float

class cTkAnimNodeData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macNode", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mbCanCompress", ctypes.c_ubyte),
        ("miRotIndex", ctypes.c_int32),
        ("miTransIndex", ctypes.c_int32),
        ("miScaleIndex", ctypes.c_int32),
    ]
    macNode: bytes
    mbCanCompress: bool
    miRotIndex: int
    miTransIndex: int
    miScaleIndex: int

class cTkAnimNodeFrameHalfData(ctypes.Structure):
    def tilde_cTkAnimNodeFrameHalfData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mau16Rotations", ctypes.c_char * 0x10),  # cTkDynamicArray<unsigned short>
        ("maTranslations", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkHalfVector4>
        ("maScales", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkHalfVector4>
    ]
    mau16Rotations: bytes
    maTranslations: bytes
    maScales: bytes

class cTkAnimNodeFrameData(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mau16Rotations", ctypes.c_char * 0x10),  # cTkDynamicArray<unsigned short>
        ("maTranslations", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkVector3>
        ("maScales", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkVector3>
    ]
    mau16Rotations: bytes
    maTranslations: bytes
    maScales: bytes

class cTkSmartResHandle(ctypes.Structure):
    def cTkSmartResHandle(): pass

    def Release(): pass

    def GetHandle(): pass

    def DecRef(): pass

    def GetPResource(): pass

    def IncRef(): pass

    def MakePersistent(): pass

    _fields_ = [
        ("miInternalHandle", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
    ]
    miInternalHandle: bytes

class cTkSceneBoneRemapping(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFromBone", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macToBone", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macFromBone: bytes
    macToBone: bytes

class cTkSceneBoneRemappingTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBoneMappings", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkSceneBoneRemapping>
    ]
    maBoneMappings: bytes

class cTkJointBindingData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mafInvBindMatrix", ctypes.c_char * 0x40),  # cTkFixedArray<float,16>
        ("mafBindTranslate", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
        ("mafBindRotate", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("mafBindScale", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
    ]
    mafInvBindMatrix: bytes
    mafBindTranslate: bytes
    mafBindRotate: bytes
    mafBindScale: bytes

class cTkSceneNodeAttributeData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mAltID", ctypes.c_ulonglong),
        ("macValue", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
    ]
    mName: int
    mAltID: int
    macValue: bytes

class cTkTransformData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfTransX", ctypes.c_float),
        ("mfTransY", ctypes.c_float),
        ("mfTransZ", ctypes.c_float),
        ("mfRotX", ctypes.c_float),
        ("mfRotY", ctypes.c_float),
        ("mfRotZ", ctypes.c_float),
        ("mfScaleX", ctypes.c_float),
        ("mfScaleY", ctypes.c_float),
        ("mfScaleZ", ctypes.c_float),
    ]
    mfTransX: float
    mfTransY: float
    mfTransZ: float
    mfRotX: float
    mfRotY: float
    mfRotZ: float
    mfScaleX: float
    mfScaleY: float
    mfScaleZ: float

class cTkJointExtentData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mafJointExtentMin", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
        ("mafJointExtentMax", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
        ("mafJointExtentCenter", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
        ("mafJointExtentStdDev", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
    ]
    mafJointExtentMin: bytes
    mafJointExtentMax: bytes
    mafJointExtentCenter: bytes
    mafJointExtentStdDev: bytes

class cTkJointMirrorAxis(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfTransMirrorAxisX", ctypes.c_float),
        ("mfTransMirrorAxisY", ctypes.c_float),
        ("mfTransMirrorAxisZ", ctypes.c_float),
        ("mfRotAdjustX", ctypes.c_float),
        ("mfRotAdjustY", ctypes.c_float),
        ("mfRotAdjustZ", ctypes.c_float),
        ("mfRotAdjustW", ctypes.c_float),
        ("mfRotMirrorAxisX", ctypes.c_float),
        ("mfRotMirrorAxisY", ctypes.c_float),
        ("mfRotMirrorAxisZ", ctypes.c_float),
        ("miMirrorAxisMode", ctypes.c_int32),
    ]
    mfTransMirrorAxisX: float
    mfTransMirrorAxisY: float
    mfTransMirrorAxisZ: float
    mfRotAdjustX: float
    mfRotAdjustY: float
    mfRotAdjustZ: float
    mfRotAdjustW: float
    mfRotMirrorAxisX: float
    mfRotMirrorAxisY: float
    mfRotMirrorAxisZ: float
    miMirrorAxisMode: int

class cTkMeshMetaData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macIdString", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mu64Hash", ctypes.c_uint64),
        ("miVertexDataSize", ctypes.c_int32),
        ("miVertexDataOffset", ctypes.c_int32),
        ("miIndexDataSize", ctypes.c_int32),
        ("miIndexDataOffset", ctypes.c_int32),
        ("mbDoubleBufferGeometry", ctypes.c_ubyte),
    ]
    macIdString: bytes
    mu64Hash: int
    miVertexDataSize: int
    miVertexDataOffset: int
    miIndexDataSize: int
    miIndexDataOffset: int
    mbDoubleBufferGeometry: bool

class cTkProceduralTextureChosenOptionSampler(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkProceduralTextureChosenOption>
    ]
    maOptions: bytes

class cTkEntitlementListData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mEntitlementId", ctypes.c_ulonglong),
        ("macServiceID", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    mEntitlementId: int
    macServiceID: bytes

class cTkMeshData(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macIdString", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mu64Hash", ctypes.c_uint64),
        ("miVertexDataSize", ctypes.c_int32),
        ("miIndexDataSize", ctypes.c_int32),
        ("maMeshDataStream", ctypes.c_char * 0x10),  # cTkDynamicArray<unsigned char>
    ]
    macIdString: bytes
    mu64Hash: int
    miVertexDataSize: int
    miIndexDataSize: int
    maMeshDataStream: bytes

class cTkGeometryStreamData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maStreamDataArray", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkMeshData>
    ]
    maStreamDataArray: bytes

class cTkInteractiveControlData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macId", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mRewardId", ctypes.c_ulonglong),
        ("mfCooldown", ctypes.c_float),
    ]
    macId: bytes
    mRewardId: int
    mfCooldown: float

class cTkInteractiveSceneData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macId", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("maControls", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkInteractiveControlData>
    ]
    macId: bytes
    maControls: bytes

class cTkIndexStream(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maiIndexStream", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
    ]
    maiIndexStream: bytes

class cTkEntitlementList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maEntitlements", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkEntitlementListData>
    ]
    maEntitlements: bytes

class cTkProceduralTextureChosenOptionList(ctypes.Structure):
    def cTkProceduralTextureChosenOptionList(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maSamplers", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkProceduralTextureChosenOptionSampler>
    ]
    maSamplers: bytes

class cTkBehaviourTreeSequentialSelectorData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mbLooping", ctypes.c_ubyte),
        ("mbFailWhenAnyChildFails", ctypes.c_ubyte),
        ("maChildren", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
    ]
    mName: int
    mbLooping: bool
    mbFailWhenAnyChildFails: bool
    maChildren: bytes

class cTkInteractivityData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maScenes", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkInteractiveSceneData>
    ]
    maScenes: bytes

class cTkBoundingBoxData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMin", common.Vector3f),
        ("mMax", common.Vector3f),
    ]
    mMin: common.Vector3f
    mMax: common.Vector3f

class cTkProceduralTextureLayer(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mfProbability", ctypes.c_float),
        ("mGroup", ctypes.c_ulonglong),
        ("mbSelectToMatchBase", ctypes.c_ubyte),
        ("maTextures", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkProceduralTexture>
    ]
    mName: int
    mfProbability: float
    mGroup: int
    mbSelectToMatchBase: bool
    maTextures: bytes

class cTkCameraWanderData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbCamWander", ctypes.c_ubyte),
        ("mfCamWanderPhase", ctypes.c_float),
        ("mfCamWanderAmplitude", ctypes.c_float),
    ]
    mbCamWander: bool
    mfCamWanderPhase: float
    mfCamWanderAmplitude: float

class UnorderedMapHashes(ctypes.Structure):
    pass

class cTkProceduralTextureList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maLayers", ctypes.c_char * 0x200),  # cTkFixedArray<cTkProceduralTextureLayer,8>
    ]
    maLayers: bytes

class cTkRawID(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64Value0", ctypes.c_uint64),
        ("mu64Value1", ctypes.c_uint64),
    ]
    mu64Value0: int
    mu64Value1: int

class cTkSketchNodeConnections(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mauConnections", ctypes.c_char * 0x10),  # cTkDynamicArray<unsigned int>
    ]
    mauConnections: bytes

class cTkBlackboardValueBool(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mKey", ctypes.c_ulonglong),
        ("mbValue", ctypes.c_ubyte),
    ]
    mKey: int
    mbValue: bool

class cTkBlackboardValueFloat(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mKey", ctypes.c_ulonglong),
        ("mfValue", ctypes.c_float),
    ]
    mKey: int
    mfValue: float

class cTkBlackboardValueId(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mKey", ctypes.c_ulonglong),
        ("mValue", ctypes.c_ulonglong),
    ]
    mKey: int
    mValue: int

class cTkBlackboardValueInteger(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mKey", ctypes.c_ulonglong),
        ("miValue", ctypes.c_int32),
    ]
    mKey: int
    miValue: int

class cTkBlackboardValueVector(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mKey", ctypes.c_ulonglong),
        ("mValue", common.Vector3f),
    ]
    mKey: int
    mValue: common.Vector3f

class cTkPhysRelVectorData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mLocal", common.Vector3f),
        ("mOffset", common.Vector3f),
    ]
    mLocal: common.Vector3f
    mOffset: common.Vector3f

class cTkTrophyEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTrophyId", ctypes.c_ulonglong),
        ("miPs4Id", ctypes.c_int32),
        ("macPCId", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macXboxOneId", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
    ]
    mTrophyId: int
    miPs4Id: int
    macPCId: bytes
    macXboxOneId: bytes

class cTkTrophyData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTrophies", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkTrophyEntry>
    ]
    maTrophies: bytes

class cTkIdArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maArray", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maArray: bytes

class cTkLanguageFontTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLanguageFontTableEntry>
    ]
    maTable: bytes

class cTkLocalisationTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLocalisationEntry>
    ]
    maTable: bytes

class cTkMetadataFilenameList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacFilenames", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    maacFilenames: bytes

class cGcWonderRecord(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mau64GenerationID", ctypes.c_char * 0x10),  # cTkFixedArray<unsigned __int64,2>
        ("mfWonderStatValue", ctypes.c_float),
        ("mbSeenInFrontend", ctypes.c_ubyte),
    ]
    mau64GenerationID: bytes
    mfWonderStatValue: float
    mbSeenInFrontend: bool

class cTkAudioAnimTrigger(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macSound", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mAnim", ctypes.c_ulonglong),
        ("miFrameStart", ctypes.c_int32),
    ]
    macSound: bytes
    mAnim: int
    miFrameStart: int

class cTkAudioComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macAmbient", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mAmbientState", ctypes.c_ulonglong),
        ("macShutdown", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miMaxDistance", ctypes.c_int32),
        ("maAnimTriggers", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAudioAnimTrigger>
    ]
    macAmbient: bytes
    mAmbientState: int
    macShutdown: bytes
    miMaxDistance: int
    maAnimTriggers: bytes

class cTkAnimationDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAnimationData>
    ]
    maTable: bytes

class cTkCreatureTailJoints(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macStartJoint", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macEndJoint", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mfInterpSpeedHead", ctypes.c_float),
        ("mfInterpSpeedTail", ctypes.c_float),
        ("mfPullSpeedMin", ctypes.c_float),
        ("mfPullSpeedMax", ctypes.c_float),
        ("mfStrengthX", ctypes.c_float),
        ("mfStrengthY", ctypes.c_float),
        ("mfStrengthZ", ctypes.c_float),
        ("mfSwimPhaseOffset", ctypes.c_float),
    ]
    macStartJoint: bytes
    macEndJoint: bytes
    mfInterpSpeedHead: float
    mfInterpSpeedTail: float
    mfPullSpeedMin: float
    mfPullSpeedMax: float
    mfStrengthX: float
    mfStrengthY: float
    mfStrengthZ: float
    mfSwimPhaseOffset: float

class cTkCreatureTailParams(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPartName", ctypes.c_ulonglong),
        ("maJoints", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkCreatureTailJoints>
        ("mfAnimationMix", ctypes.c_float),
        ("mfSwimSpeed", ctypes.c_float),
        ("mfSwimReps", ctypes.c_float),
        ("mfMaxTurnForSwim", ctypes.c_float),
        ("mfSwimBlendInTime", ctypes.c_float),
        ("mfSwimBlendOutTime", ctypes.c_float),
        ("mfSwimMagnitude", ctypes.c_float),
        ("mfSwimRollMagnitude", ctypes.c_float),
        ("mfSwimFallOffBegin", ctypes.c_float),
        ("mfSwimFallOffEnd", ctypes.c_float),
        ("mfMinSwimStrength", ctypes.c_float),
        ("mbHorizontalStrokes", ctypes.c_ubyte),
        ("mafPerBoneSwimStrength", ctypes.c_char * 0x10),  # cTkDynamicArray<float>
    ]
    mPartName: int
    maJoints: bytes
    mfAnimationMix: float
    mfSwimSpeed: float
    mfSwimReps: float
    mfMaxTurnForSwim: float
    mfSwimBlendInTime: float
    mfSwimBlendOutTime: float
    mfSwimMagnitude: float
    mfSwimRollMagnitude: float
    mfSwimFallOffBegin: float
    mfSwimFallOffEnd: float
    mfMinSwimStrength: float
    mbHorizontalStrokes: bool
    mafPerBoneSwimStrength: bytes

class cTkAnimJointLODData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miLOD", ctypes.c_int32),
        ("maacJointNames", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<64,char> >
    ]
    miLOD: int
    maacJointNames: bytes

class cTkDynamicChainComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfAngularLimit", ctypes.c_float),
        ("mfTwistLimit", ctypes.c_float),
        ("mfMotorStrengthCone", ctypes.c_float),
        ("mfMotorStrengthTwist", ctypes.c_float),
        ("mfMaxMotorForce", ctypes.c_float),
        ("mfGravity", ctypes.c_float),
        ("mfAirThickness", ctypes.c_float),
        ("mfVertAirThickness", ctypes.c_float),
        ("mfWindStrength", ctypes.c_float),
        ("mfLinearDamping", ctypes.c_float),
        ("mfAngularDamping", ctypes.c_float),
        ("mfInitialBodyMass", ctypes.c_float),
        ("mfBodyMassChange", ctypes.c_float),
        ("mbWeightByJointLength", ctypes.c_ubyte),
        ("maacIgnoreJoints", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<32,char> >
    ]
    mfAngularLimit: float
    mfTwistLimit: float
    mfMotorStrengthCone: float
    mfMotorStrengthTwist: float
    mfMaxMotorForce: float
    mfGravity: float
    mfAirThickness: float
    mfVertAirThickness: float
    mfWindStrength: float
    mfLinearDamping: float
    mfAngularDamping: float
    mfInitialBodyMass: float
    mfBodyMassChange: float
    mbWeightByJointLength: bool
    maacIgnoreJoints: bytes

class cTkAnimPoseBabyModifier(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mItem", ctypes.c_ulonglong),
        ("mfValue", ctypes.c_float),
        ("mfWeight", ctypes.c_float),
    ]
    mItem: int
    mfValue: float
    mfWeight: float

class cTkInstanceWindComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbEnableLdsWind", ctypes.c_ubyte),
        ("mfBaseMass", ctypes.c_float),
        ("mfMassReduction", ctypes.c_float),
        ("mfBaseSpring", ctypes.c_float),
        ("mfSpringReduction", ctypes.c_float),
        ("mfLinearDamping", ctypes.c_float),
        ("mfSpringNonDirFactor", ctypes.c_float),
        ("mfSpringLengthFactor", ctypes.c_float),
    ]
    mbEnableLdsWind: bool
    mfBaseMass: float
    mfMassReduction: float
    mfBaseSpring: float
    mfSpringReduction: float
    mfLinearDamping: float
    mfSpringNonDirFactor: float
    mfSpringLengthFactor: float

class cTkAnimPoseCorrelationData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mItemA", ctypes.c_ulonglong),
        ("mItemB", ctypes.c_ulonglong),
        ("mfCorrelation", ctypes.c_float),
    ]
    mItemA: int
    mItemB: int
    mfCorrelation: float

class cTkAnimPoseData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mAnim", ctypes.c_ulonglong),
        ("miFrameStart", ctypes.c_int32),
        ("miFrameEnd", ctypes.c_int32),
    ]
    mAnim: int
    miFrameStart: int
    miFrameEnd: int

class cTkAnimPoseExampleElement(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mAnim", ctypes.c_ulonglong),
        ("mfValue", ctypes.c_float),
    ]
    mAnim: int
    mfValue: float

class cTkAnimPoseExampleData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maElements", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAnimPoseExampleElement>
    ]
    maElements: bytes

class cTkAnimDetailSettingsData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfDistance", ctypes.c_float),
        ("mbDisableAnim", ctypes.c_ubyte),
        ("miNumCulledFrames", ctypes.c_int32),
    ]
    mfDistance: float
    mbDisableAnim: bool
    miNumCulledFrames: int

class cTkAnimDetailSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMaxVisibleAngle", ctypes.c_float),
        ("maAnimDistanceSettings", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAnimDetailSettingsData>
        ("mafAnimLODDistances", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
    ]
    mfMaxVisibleAngle: float
    maAnimDistanceSettings: bytes
    mafAnimLODDistances: bytes

class cTkAttachmentData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def Save(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maComponents", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
        ("mafLodDistances", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
    ]
    maComponents: bytes
    mafLodDistances: bytes

class cTkAnimDetailSettingsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0xA0),  # cTkFixedArray<cTkAnimDetailSettings,4>
    ]
    maTable: bytes

class cTkAnimPoseComponentData(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maPoseAnims", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAnimPoseData>
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maCorrelations", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAnimPoseCorrelationData>
        ("maExamples", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAnimPoseExampleData>
        ("mafCorrelationMat", ctypes.c_char * 0x10),  # cTkDynamicArray<float>
        ("mfAdultCorrelationValue", ctypes.c_float),
        ("maBabyModifiers", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkAnimPoseBabyModifier>
        ("mbShouldRandomise", ctypes.c_ubyte),
    ]
    maPoseAnims: bytes
    macFilename: bytes
    maCorrelations: bytes
    maExamples: bytes
    mafCorrelationMat: bytes
    mfAdultCorrelationValue: float
    maBabyModifiers: bytes
    mbShouldRandomise: bool

class cTkEngineSettingsMapping(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mafShadowMultiplier", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("mafReflectionMultiplier", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("mafReflectionProbesMultiplier", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("mafIKFullBodyIterations", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("mafCloudsResolutionScale", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("maiCloudsMaxIterations", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("mabNeedsGameRestart", ctypes.c_char * 0x32),  # cTkFixedArray<bool,50>
    ]
    mafShadowMultiplier: bytes
    mafReflectionMultiplier: bytes
    mafReflectionProbesMultiplier: bytes
    mafIKFullBodyIterations: bytes
    mafCloudsResolutionScale: bytes
    maiCloudsMaxIterations: bytes
    mabNeedsGameRestart: bytes

class cTkGameSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maKeyMapping", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInputActionMapping>
        ("maKeyMapping2", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInputActionMapping2>
    ]
    maKeyMapping: bytes
    maKeyMapping2: bytes

class cGcVibrationChannelData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maData", ctypes.c_char * 0x30),  # cTkFixedArray<cGcVibrationData,2>
    ]
    mId: int
    maData: bytes

class cGcVibrationDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcVibrationChannelData>
    ]
    maData: bytes

class cGcIDLookupPath(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("macPath", ctypes.c_char * 0x800),  # cTkFixedString<2048,char>
    ]
    mId: int
    macPath: bytes

class cGcIDLookupPaths(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maPaths", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcIDLookupPath>
    ]
    maPaths: bytes

class cTkLODDistances(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mafDistances", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
    ]
    mafDistances: bytes

class cTkLODSettingsData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mafLODAdjust", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
        ("maiImposterOverrideRange", ctypes.c_char * 0x18),  # cTkFixedArray<int,6>
        ("maiMaxObjectDistanceOverride", ctypes.c_char * 0x18),  # cTkFixedArray<int,6>
        ("maiRegionLODHiddenRanges", ctypes.c_char * 0x18),  # cTkFixedArray<int,6>
        ("maiRegionLODRadius", ctypes.c_char * 0x18),  # cTkFixedArray<int,6>
        ("mbEnableOctahedralImposters", ctypes.c_ubyte),
        ("mbViewImpostersFromSpace", ctypes.c_ubyte),
        ("miNumberOfImposterViews", ctypes.c_int32),
        ("miImposterResolutionMultiplier", ctypes.c_int32),
        ("miAsteroidCountMultiplier", ctypes.c_int32),
        ("miAsteroidDividerMultiplier", ctypes.c_int32),
        ("mfAsteroidFadeRangeMultiplier", ctypes.c_float),
        ("miMaxAsteroidGenerationPerFrame", ctypes.c_int32),
        ("miMaxAsteroidGenerationPerFramePulseJump", ctypes.c_int32),
    ]
    mafLODAdjust: bytes
    maiImposterOverrideRange: bytes
    maiMaxObjectDistanceOverride: bytes
    maiRegionLODHiddenRanges: bytes
    maiRegionLODRadius: bytes
    mbEnableOctahedralImposters: bool
    mbViewImpostersFromSpace: bool
    miNumberOfImposterViews: int
    miImposterResolutionMultiplier: int
    miAsteroidCountMultiplier: int
    miAsteroidDividerMultiplier: int
    mfAsteroidFadeRangeMultiplier: float
    miMaxAsteroidGenerationPerFrame: int
    miMaxAsteroidGenerationPerFramePulseJump: int

class cGcStoryEntryBranch(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mEntry", ctypes.c_ulonglong),
        ("mRequiresMission", ctypes.c_ulonglong),
        ("mbConditionMissionComplete", ctypes.c_ubyte),
    ]
    mEntry: int
    mRequiresMission: int
    mbConditionMissionComplete: bool

class cGcStoriesTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0xAC0),  # cTkFixedArray<cGcStoryCategory,8>
    ]
    maTable: bytes

class cGcStoryPageSeenData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miPageIdx", ctypes.c_int32),
        ("miLastSeenEntryIdx", ctypes.c_int32),
    ]
    miPageIdx: int
    miLastSeenEntryIdx: int

class cGcStoryPageSeenDataArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maPagesData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcStoryPageSeenData>
    ]
    maPagesData: bytes

class cGcWiki(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maCategories", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcWikiCategory>
    ]
    maCategories: bytes

class cTkGravityComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfStrength", ctypes.c_float),
        ("mfFalloffRadius", ctypes.c_float),
    ]
    mfStrength: float
    mfFalloffRadius: float

class cTkLODComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maLODModels", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkLODModelResource>
        ("mfCrossFadeTime", ctypes.c_float),
        ("mfCrossFadeOverlap", ctypes.c_float),
    ]
    maLODModels: bytes
    mfCrossFadeTime: float
    mfCrossFadeOverlap: float

class cTkResourceDescriptorData(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("macName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maacReferencePaths", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
        ("mfChance", ctypes.c_float),
        ("maChildren", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
    ]
    mId: int
    macName: bytes
    maacReferencePaths: bytes
    mfChance: float
    maChildren: bytes

class cTkResourceDescriptorList(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTypeId", ctypes.c_ulonglong),
        ("maDescriptors", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkResourceDescriptorData>
    ]
    mTypeId: int
    maDescriptors: bytes

class cTkModelDescriptorList(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maList", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkResourceDescriptorList>
    ]
    maList: bytes

class cTkProceduralInstanceData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("miIndex", ctypes.c_int32),
    ]
    mId: int
    miIndex: int

class cTkProceduralInstance(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maData", ctypes.c_char * 0x180),  # cTkFixedArray<cTkProceduralInstanceData,16>
    ]
    maData: bytes

class cTkProceduralModelComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacList", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    maacList: bytes

class cTkProceduralModelList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maacList", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    mId: int
    maacList: bytes

class cTkReferenceComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macReference", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macLSystem", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macReference: bytes
    macLSystem: bytes

class cTkResourceFilterData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mFilterName", ctypes.c_ulonglong),
        ("maFilteredResources", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
    ]
    mFilterName: int
    maFilteredResources: bytes

class cGcVehicleMuzzleData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maMuzzleFlashDataID", ctypes.c_char * 0x40),  # cTkFixedArray<TkID<128>,4>
    ]
    maMuzzleFlashDataID: bytes

class cGcVehicleWeaponMuzzleData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mMuzzleFlashEffect", ctypes.c_ulonglong),
    ]
    mID: int
    mMuzzleFlashEffect: int

class cGcVehicleRaceInviteComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfRadius", ctypes.c_float),
    ]
    mfRadius: float

class cTkResourceFilterList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maFilters", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkResourceFilterData>
    ]
    maFilters: bytes

class cGcVehicleScanTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maVehicleScanTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcVehicleScanTableEntry>
    ]
    maVehicleScanTable: bytes

class cTkRotationComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfSpeed", ctypes.c_float),
        ("mAxis", common.Vector3f),
        ("mbAlwaysUpdate", ctypes.c_ubyte),
        ("miSyncGroup", ctypes.c_int32),
    ]
    mfSpeed: float
    mAxis: common.Vector3f
    mbAlwaysUpdate: bool
    miSyncGroup: int

class cTkSketchComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfGraphPosX", ctypes.c_float),
        ("mfGraphPosY", ctypes.c_float),
        ("mfGraphZoom", ctypes.c_float),
        ("mfUpdateRateMultiplier", ctypes.c_float),
        ("maNodes", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkSketchNodeData>
    ]
    mfGraphPosX: float
    mfGraphPosY: float
    mfGraphZoom: float
    mfUpdateRateMultiplier: float
    maNodes: bytes

class cTkShearWindOctaveData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMinStrength", ctypes.c_float),
        ("mfMaxStrength", ctypes.c_float),
        ("mfStrengthVariationFreq", ctypes.c_float),
        ("mfWaveSize", ctypes.c_float),
        ("mfWaveFrequency", ctypes.c_float),
    ]
    mfMinStrength: float
    mfMaxStrength: float
    mfStrengthVariationFreq: float
    mfWaveSize: float
    mfWaveFrequency: float

class cGcInWorldUIScreenData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfScreenScale", ctypes.c_float),
        ("mScreenOffset", common.Vector3f),
        ("mScreenRotation", common.Vector4f),
    ]
    mfScreenScale: float
    mScreenOffset: common.Vector3f
    mScreenRotation: common.Vector4f

class cGcPlayerWeaponData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mReticle", ctypes.c_ulonglong),
    ]
    mReticle: int

class cGcInventoryIndex(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miX", ctypes.c_int32),
        ("miY", ctypes.c_int32),
    ]
    miX: int
    miY: int

class cGcHotActionsSaveData(ctypes.Structure):
    def cGcHotActionsSaveData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maKeyActions", ctypes.c_char * 0x190),  # cTkFixedArray<cGcQuickMenuActionSaveData,10>
    ]
    maKeyActions: bytes

class cGcBlockedMessage(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessageId", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessageId: bytes

class cGcBlockedUser(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macUserId", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macUsername", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macPlatform", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
    ]
    macUserId: bytes
    macUsername: bytes
    macPlatform: bytes

class cGcBlockListPersistence(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maBlockedUserArray", ctypes.c_char * 0x1F40),  # cTkFixedArray<cGcBlockedUser,50>
        ("miNextSlot", ctypes.c_int32),
        ("miListSize", ctypes.c_int32),
        ("maBlockedMessageArray", ctypes.c_char * 0x1900),  # cTkFixedArray<cGcBlockedMessage,50>
        ("miMessageNextSlot", ctypes.c_int32),
        ("miMessageListSize", ctypes.c_int32),
    ]
    maBlockedUserArray: bytes
    miNextSlot: int
    miListSize: int
    maBlockedMessageArray: bytes
    miMessageNextSlot: int
    miMessageListSize: int

class cGcDifficultyOptionUIGroup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mHeadingLocID", ctypes.c_ulonglong),
        ("maDifficultyOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDifficultySettingUIOption>
    ]
    mHeadingLocID: int
    maDifficultyOptions: bytes

class cGcWordKnowledge(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mWord", ctypes.c_ulonglong),
        ("mabRaces", ctypes.c_char * 0x8),  # cTkFixedArray<bool,8>
    ]
    mWord: int
    mabRaces: bytes

class cGcSurvivalBarBoolArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mabValues", ctypes.c_char * 0x3),  # cTkFixedArray<bool,3>
    ]
    mabValues: bytes

class cGcWordGroupKnowledge(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mGroup", ctypes.c_ulonglong),
        ("mabRaces", ctypes.c_char * 0x8),  # cTkFixedArray<bool,8>
    ]
    mGroup: int
    mabRaces: bytes

class cGcKnownThingsPreset(ctypes.Structure):
    def tilde_cGcKnownThingsPreset(): pass

    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maKnownTech", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maKnownProducts", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maKnownSpecials", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maKnownRefinerRecipes", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
        ("maKnownWords", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcWordKnowledge>
        ("maKnownWordGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcWordGroupKnowledge>
    ]
    maKnownTech: bytes
    maKnownProducts: bytes
    maKnownSpecials: bytes
    maKnownRefinerRecipes: bytes
    maKnownWords: bytes
    maKnownWordGroups: bytes

class cGcTriggerFeedbackStateTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maEvents", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcTriggerFeedbackState>
    ]
    maEvents: bytes

class cGcInventoryBaseStatEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mBaseStatID", ctypes.c_ulonglong),
        ("mfValue", ctypes.c_float),
    ]
    mBaseStatID: int
    mfValue: float

class cGcDifficultySettingLocData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTitleLocID", ctypes.c_ulonglong),
        ("mDescriptionLocID", ctypes.c_ulonglong),
    ]
    mTitleLocID: int
    mDescriptionLocID: int

class cGcDifficultyInventoryStackSizeOptionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miSubstanceStackLimit", ctypes.c_int32),
        ("miProductStackLimit", ctypes.c_int32),
        ("maiMaxSubstanceStackSizes", ctypes.c_char * 0x30),  # cTkFixedArray<int,12>
        ("maiMaxProductStackSizes", ctypes.c_char * 0x30),  # cTkFixedArray<int,12>
    ]
    miSubstanceStackLimit: int
    miProductStackLimit: int
    maiMaxSubstanceStackSizes: bytes
    maiMaxProductStackSizes: bytes

class cGcDifficultyFuelUseTechOverride(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTechID", ctypes.c_ulonglong),
        ("mfMultiplier", ctypes.c_float),
    ]
    mTechID: int
    mfMultiplier: float

class cGcDifficultyFuelUseOptionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMultiplier", ctypes.c_float),
        ("maTechOverrides", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDifficultyFuelUseTechOverride>
    ]
    mfMultiplier: float
    maTechOverrides: bytes

class cGcDifficultyCurrencyCostOptionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mafMultipliers", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
        ("mabFreeCostTypes", ctypes.c_char * 0x3),  # cTkFixedArray<bool,3>
        ("mfTradeBuyPriceMarkupMod", ctypes.c_float),
        ("mbInteractionsCostsAreFree", ctypes.c_ubyte),
        ("mbCostManagerCostsAreFree", ctypes.c_ubyte),
    ]
    mafMultipliers: bytes
    mabFreeCostTypes: bytes
    mfTradeBuyPriceMarkupMod: float
    mbInteractionsCostsAreFree: bool
    mbCostManagerCostsAreFree: bool

class cGcItemShopAvailabilityDifficultyOptionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maNeverSoldItems", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maNeverSoldItems: bytes

class cGcMechMeshPartTypeData(ctypes.Structure):
    def cGcMechMeshPartTypeData(): pass

    def tilde_cGcMechMeshPartTypeData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macLocatorNodeName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("maRequiredTechs", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    macLocatorNodeName: bytes
    maRequiredTechs: bytes

class cGcMechMeshPartData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maMeshTypes", ctypes.c_char * 0x60),  # cTkFixedArray<cGcMechMeshPartTypeData,2>
    ]
    maMeshTypes: bytes

class cGcMechMeshPartTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maParts", ctypes.c_char * 0x1E0),  # cTkFixedArray<cGcMechMeshPartData,5>
    ]
    maParts: bytes

class cGcMechWeaponLocationPriority(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maMechWeaponLocationPriority", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcMechWeaponLocation>
    ]
    maMechWeaponLocationPriority: bytes

class cGcVehicleData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("miNumWheels", ctypes.c_int32),
        ("mfWheelRadius", ctypes.c_float),
        ("mfWheelGuardExtraRadius", ctypes.c_float),
        ("mfWheelGuardExtraHeight", ctypes.c_float),
        ("mfWheelGuardAdjustUpwards", ctypes.c_float),
        ("mfUnderwaterEnginePower", ctypes.c_float),
        ("mfUnderwaterEngineMaxSpeed", ctypes.c_float),
        ("mfUnderwaterEngineFalloff", ctypes.c_float),
        ("mfUnderwaterEngineDirectionBrake", ctypes.c_float),
        ("mfUnderwaterEngineDirectionBrakeVertical", ctypes.c_float),
        ("mfUnderwaterAlignDir", ctypes.c_float),
        ("mfUnderwaterAlignUp", ctypes.c_float),
        ("mfVisualRollAmount", ctypes.c_float),
        ("mfVisualRollOffsetY", ctypes.c_float),
        ("mfSteeringWheelSpringMultiplier", ctypes.c_float),
        ("mfSteeringWheelPushRange", ctypes.c_float),
        ("maWheelLocs", ctypes.c_char * 0xA0),  # cTkFixedArray<cTkVector3,10>
        ("maacWheelNames", ctypes.c_char * 0x140),  # cTkFixedArray<cTkFixedString<32,char>,10>
        ("maacWheelSuspensionNames", ctypes.c_char * 0x140),  # cTkFixedArray<cTkFixedString<32,char>,10>
        ("mafWheelRayFakeWidthFactor", ctypes.c_char * 0x28),  # cTkFixedArray<float,10>
        ("mafWheelRadiusMultiplier", ctypes.c_char * 0x28),  # cTkFixedArray<float,10>
        ("maSuspensionAnimNames", ctypes.c_char * 0xA0),  # cTkFixedArray<TkID<128>,10>
        ("mbCustomCollision", ctypes.c_ubyte),
        ("mCollOffset", common.Vector3f),
        ("mExtraCollOffset", common.Vector3f),
        ("mCollDimensions", common.Vector3f),
        ("mInertiaDimensions", common.Vector3f),
        ("mfCollRadius", ctypes.c_float),
        ("mfInertiaMul", ctypes.c_float),
        ("mfWheelSuspensionlength", ctypes.c_float),
        ("mfWheelSuspensionForce", ctypes.c_float),
        ("mfWheelSuspensionDamping", ctypes.c_float),
        ("mfWheelSuspensionAnimMin", ctypes.c_float),
        ("mfWheelSuspensionAnimMax", ctypes.c_float),
        ("mfTopSpeedForward", ctypes.c_float),
        ("mfTopSpeedReverse", ctypes.c_float),
        ("mfWheelMaxAccelForceForward", ctypes.c_float),
        ("mfWheelMaxAccelForceReverse", ctypes.c_float),
        ("mfWheelMaxDecelForceNonBraking", ctypes.c_float),
        ("mfWheelMaxDecelForceBraking", ctypes.c_float),
        ("mfWheelSpinniness", ctypes.c_float),
        ("mfWheelDragginess", ctypes.c_float),
        ("mfHardStopSpeedThreshold", ctypes.c_float),
        ("mfWheelFrontFrictionOmega", ctypes.c_float),
        ("mfWheelFrontFrictionDynamic", ctypes.c_float),
        ("mfWheelFrontFrictionDynamicThreshold", ctypes.c_float),
        ("mfWheelFrontFrictionStatic", ctypes.c_float),
        ("mfWheelFrontFrictionStaticThreshold", ctypes.c_float),
        ("mfWheelSideFrictionOmega", ctypes.c_float),
        ("mfWheelSideFrictionDynamic", ctypes.c_float),
        ("mfWheelSideFrictionDynamicThreshold", ctypes.c_float),
        ("mfWheelSideFrictionStatic", ctypes.c_float),
        ("mfWheelSideFrictionStaticThreshold", ctypes.c_float),
        ("mbLockVehicleAxis", ctypes.c_ubyte),
        ("mfTurningWheelForce", ctypes.c_float),
        ("mfTurningWheelForceDamperVR", ctypes.c_float),
        ("mfTurningWheelFrictionOmega", ctypes.c_float),
        ("mfTurningWheelFrictionNonBraking", ctypes.c_float),
        ("mfTurningWheelFrictionBraking", ctypes.c_float),
        ("mWheelForwardAngularFactor", common.Vector3f),
        ("mWheelTurnAngularFactor", common.Vector3f),
        ("mWheelSuspensionAngularFactor", common.Vector3f),
        ("mWheelSideAngularFactor", common.Vector3f),
        ("mfVehicleGravity", ctypes.c_float),
        ("mfVehicleGravityWater", ctypes.c_float),
        ("mfVehicleJumpForce", ctypes.c_float),
        ("mfVehicleJumpAirControlForce", ctypes.c_float),
        ("mfVehicleBoostForce", ctypes.c_float),
        ("mfVehicleBoostMaxSpeed", ctypes.c_float),
        ("mfVehicleBoostExtraMaxSpeedAir", ctypes.c_float),
        ("mfVehicleBoostSpeedFalloff", ctypes.c_float),
        ("mfVehicleBoostTime", ctypes.c_float),
        ("mfVehicleBoostRechargeTime", ctypes.c_float),
        ("mfVehicleJumpAirRotateXAmount", ctypes.c_float),
        ("mfVehicleJumpAirRotateZAmount", ctypes.c_float),
        ("mfVehicleJumpAirRotateTimeMin", ctypes.c_float),
        ("mfVehicleJumpAirRotateTimeMax", ctypes.c_float),
        ("mfVehicleJumpAirMaxTorque", ctypes.c_float),
        ("mbUseBuggySuspensionHack", ctypes.c_ubyte),
        ("mbUseRoverWheelHack", ctypes.c_ubyte),
        ("mSideSkidParticle", ctypes.c_ulonglong),
        ("mfSideSkidParticleMinRate", ctypes.c_float),
        ("mfSideSkidParticleMaxRate", ctypes.c_float),
        ("mfSideSkidParticleMinThresh", ctypes.c_float),
        ("mfSideSkidParticleMaxThresh", ctypes.c_float),
        ("mWheelSpinParticle", ctypes.c_ulonglong),
        ("mfWheelSpinParticleMinRate", ctypes.c_float),
        ("mfWheelSpinParticleMaxRate", ctypes.c_float),
        ("mfWheelSpinParticleMinThresh", ctypes.c_float),
        ("mfWheelSpinParticleMaxThresh", ctypes.c_float),
        ("mfWheelGuardVerticalResponseMin", ctypes.c_float),
        ("mfWheelGuardVerticalResponseMax", ctypes.c_float),
        ("mfWheelGuardPenetrationScaleMin", ctypes.c_float),
        ("mfWheelGuardPenetrationScaleMax", ctypes.c_float),
        ("mfWheelGuardPenetrationScaleMinClamp", ctypes.c_float),
        ("mfWheelGuardMassScaleMin", ctypes.c_float),
        ("mfWheelGuardMassScaleMax", ctypes.c_float),
        ("mfWheelGuardMassScaleMinClamp", ctypes.c_float),
        ("mfVehicleLinearDampingGround", ctypes.c_float),
        ("mfVehicleAngularDampingGround", ctypes.c_float),
        ("mfVehicleLinearDampingAerial", ctypes.c_float),
        ("mfVehicleAngularDampingAerial", ctypes.c_float),
        ("mfVehicleLinearDampingWater", ctypes.c_float),
        ("mfVehicleAngularDampingWater", ctypes.c_float),
        ("mfWheelStartHeight", ctypes.c_float),
        ("mfWheelEndHeight", ctypes.c_float),
        ("mfVehicleComCheat", ctypes.c_float),
        ("mfVehicleAudioSpeedMul", ctypes.c_float),
        ("mfVehicleAudioTorqueMul", ctypes.c_float),
        ("mfVehicleAudioSideSkidMul", ctypes.c_float),
        ("mfVehicleAudioSideSkidThreshold", ctypes.c_float),
        ("mfVehicleAudioSpinSkidMul", ctypes.c_float),
        ("mfVehicleAudioSpinSkidThreshold", ctypes.c_float),
        ("mfAudioImpactSpeedThreshold", ctypes.c_float),
        ("mfAudioImpactSpeedMul", ctypes.c_float),
        ("mbVehicleAudioSwapSkidAndSpeed", ctypes.c_ubyte),
        ("macAudioBoostStart", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioBoostStop", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioHornStart", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioHornStop", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioIdleExterior", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioImpacts", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioStart", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioStop", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioSuspension", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAudioJump", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbDriveOnTopOfWater", ctypes.c_ubyte),
        ("mfVehicleUnderwaterRotateTime", ctypes.c_float),
        ("mfVehicleAudioSuspensionThreshold", ctypes.c_float),
        ("mfVehicleAudioSuspensionScale", ctypes.c_float),
        ("miNumGrassPushers", ctypes.c_int32),
        ("mfWheelGrassPusherStrength", ctypes.c_float),
        ("mfWheelGrassPusherWobble", ctypes.c_float),
        ("mfWheelGrassPusherFrequency", ctypes.c_float),
        ("mfWheelGrassPusherReduce", ctypes.c_float),
        ("maWheelGrassPushers", ctypes.c_char * 0xA0),  # cTkFixedArray<cTkVector3,10>
        ("mfCreatureMassScale", ctypes.c_float),
        ("maacHeadlightNames", ctypes.c_char * 0x200),  # cTkFixedArray<cTkFixedString<256,char>,2>
        ("maacVolumetricHeadlightNames", ctypes.c_char * 0x200),  # cTkFixedArray<cTkFixedString<256,char>,2>
        ("maacCockpitHeadlightNames", ctypes.c_char * 0x200),  # cTkFixedArray<cTkFixedString<256,char>,2>
        ("mfHeadlightIntensity", ctypes.c_float),
    ]
    mName: int
    miNumWheels: int
    mfWheelRadius: float
    mfWheelGuardExtraRadius: float
    mfWheelGuardExtraHeight: float
    mfWheelGuardAdjustUpwards: float
    mfUnderwaterEnginePower: float
    mfUnderwaterEngineMaxSpeed: float
    mfUnderwaterEngineFalloff: float
    mfUnderwaterEngineDirectionBrake: float
    mfUnderwaterEngineDirectionBrakeVertical: float
    mfUnderwaterAlignDir: float
    mfUnderwaterAlignUp: float
    mfVisualRollAmount: float
    mfVisualRollOffsetY: float
    mfSteeringWheelSpringMultiplier: float
    mfSteeringWheelPushRange: float
    maWheelLocs: bytes
    maacWheelNames: bytes
    maacWheelSuspensionNames: bytes
    mafWheelRayFakeWidthFactor: bytes
    mafWheelRadiusMultiplier: bytes
    maSuspensionAnimNames: bytes
    mbCustomCollision: bool
    mCollOffset: common.Vector3f
    mExtraCollOffset: common.Vector3f
    mCollDimensions: common.Vector3f
    mInertiaDimensions: common.Vector3f
    mfCollRadius: float
    mfInertiaMul: float
    mfWheelSuspensionlength: float
    mfWheelSuspensionForce: float
    mfWheelSuspensionDamping: float
    mfWheelSuspensionAnimMin: float
    mfWheelSuspensionAnimMax: float
    mfTopSpeedForward: float
    mfTopSpeedReverse: float
    mfWheelMaxAccelForceForward: float
    mfWheelMaxAccelForceReverse: float
    mfWheelMaxDecelForceNonBraking: float
    mfWheelMaxDecelForceBraking: float
    mfWheelSpinniness: float
    mfWheelDragginess: float
    mfHardStopSpeedThreshold: float
    mfWheelFrontFrictionOmega: float
    mfWheelFrontFrictionDynamic: float
    mfWheelFrontFrictionDynamicThreshold: float
    mfWheelFrontFrictionStatic: float
    mfWheelFrontFrictionStaticThreshold: float
    mfWheelSideFrictionOmega: float
    mfWheelSideFrictionDynamic: float
    mfWheelSideFrictionDynamicThreshold: float
    mfWheelSideFrictionStatic: float
    mfWheelSideFrictionStaticThreshold: float
    mbLockVehicleAxis: bool
    mfTurningWheelForce: float
    mfTurningWheelForceDamperVR: float
    mfTurningWheelFrictionOmega: float
    mfTurningWheelFrictionNonBraking: float
    mfTurningWheelFrictionBraking: float
    mWheelForwardAngularFactor: common.Vector3f
    mWheelTurnAngularFactor: common.Vector3f
    mWheelSuspensionAngularFactor: common.Vector3f
    mWheelSideAngularFactor: common.Vector3f
    mfVehicleGravity: float
    mfVehicleGravityWater: float
    mfVehicleJumpForce: float
    mfVehicleJumpAirControlForce: float
    mfVehicleBoostForce: float
    mfVehicleBoostMaxSpeed: float
    mfVehicleBoostExtraMaxSpeedAir: float
    mfVehicleBoostSpeedFalloff: float
    mfVehicleBoostTime: float
    mfVehicleBoostRechargeTime: float
    mfVehicleJumpAirRotateXAmount: float
    mfVehicleJumpAirRotateZAmount: float
    mfVehicleJumpAirRotateTimeMin: float
    mfVehicleJumpAirRotateTimeMax: float
    mfVehicleJumpAirMaxTorque: float
    mbUseBuggySuspensionHack: bool
    mbUseRoverWheelHack: bool
    mSideSkidParticle: int
    mfSideSkidParticleMinRate: float
    mfSideSkidParticleMaxRate: float
    mfSideSkidParticleMinThresh: float
    mfSideSkidParticleMaxThresh: float
    mWheelSpinParticle: int
    mfWheelSpinParticleMinRate: float
    mfWheelSpinParticleMaxRate: float
    mfWheelSpinParticleMinThresh: float
    mfWheelSpinParticleMaxThresh: float
    mfWheelGuardVerticalResponseMin: float
    mfWheelGuardVerticalResponseMax: float
    mfWheelGuardPenetrationScaleMin: float
    mfWheelGuardPenetrationScaleMax: float
    mfWheelGuardPenetrationScaleMinClamp: float
    mfWheelGuardMassScaleMin: float
    mfWheelGuardMassScaleMax: float
    mfWheelGuardMassScaleMinClamp: float
    mfVehicleLinearDampingGround: float
    mfVehicleAngularDampingGround: float
    mfVehicleLinearDampingAerial: float
    mfVehicleAngularDampingAerial: float
    mfVehicleLinearDampingWater: float
    mfVehicleAngularDampingWater: float
    mfWheelStartHeight: float
    mfWheelEndHeight: float
    mfVehicleComCheat: float
    mfVehicleAudioSpeedMul: float
    mfVehicleAudioTorqueMul: float
    mfVehicleAudioSideSkidMul: float
    mfVehicleAudioSideSkidThreshold: float
    mfVehicleAudioSpinSkidMul: float
    mfVehicleAudioSpinSkidThreshold: float
    mfAudioImpactSpeedThreshold: float
    mfAudioImpactSpeedMul: float
    mbVehicleAudioSwapSkidAndSpeed: bool
    macAudioBoostStart: bytes
    macAudioBoostStop: bytes
    macAudioHornStart: bytes
    macAudioHornStop: bytes
    macAudioIdleExterior: bytes
    macAudioImpacts: bytes
    macAudioStart: bytes
    macAudioStop: bytes
    macAudioSuspension: bytes
    macAudioJump: bytes
    mbDriveOnTopOfWater: bool
    mfVehicleUnderwaterRotateTime: float
    mfVehicleAudioSuspensionThreshold: float
    mfVehicleAudioSuspensionScale: float
    miNumGrassPushers: int
    mfWheelGrassPusherStrength: float
    mfWheelGrassPusherWobble: float
    mfWheelGrassPusherFrequency: float
    mfWheelGrassPusherReduce: float
    maWheelGrassPushers: bytes
    mfCreatureMassScale: float
    maacHeadlightNames: bytes
    maacVolumetricHeadlightNames: bytes
    maacCockpitHeadlightNames: bytes
    mfHeadlightIntensity: float

class cGcAISpaceshipModelDataArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSpaceships", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcAISpaceshipModelData>
    ]
    maSpaceships: bytes

class cGcSpaceshipTravelData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mfMinSpeed", ctypes.c_float),
        ("mfMaxSpeed", ctypes.c_float),
        ("mfBoostSpeed", ctypes.c_float),
        ("mfForce", ctypes.c_float),
        ("mfMinSpeedForce", ctypes.c_float),
        ("mfTurnMin", ctypes.c_float),
        ("mfTurnMax", ctypes.c_float),
        ("mfDirectionBrake", ctypes.c_float),
        ("mfMaxSpeedBrake", ctypes.c_float),
        ("mfFalloff", ctypes.c_float),
        ("mfRoll", ctypes.c_float),
        ("mfMinHeight", ctypes.c_float),
        ("mfAvoidTime", ctypes.c_float),
        ("mbHovering", ctypes.c_ubyte),
    ]
    mId: int
    mfMinSpeed: float
    mfMaxSpeed: float
    mfBoostSpeed: float
    mfForce: float
    mfMinSpeedForce: float
    mfTurnMin: float
    mfTurnMax: float
    mfDirectionBrake: float
    mfMaxSpeedBrake: float
    mfFalloff: float
    mfRoll: float
    mfMinHeight: float
    mfAvoidTime: float
    mbHovering: bool

class cGcEngineComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("miType", ctypes.c_int32),
    ]
    miType: int

class cGcPlayerSpaceshipAim(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfAimAngleMin", ctypes.c_float),
        ("mfAimAngleRange", ctypes.c_float),
        ("mfAimDistanceAngleMin", ctypes.c_float),
        ("mfAimDistanceAngleRange", ctypes.c_float),
        ("mfAimDistanceMin", ctypes.c_float),
        ("mfAimDistanceRange", ctypes.c_float),
    ]
    mfAimAngleMin: float
    mfAimAngleRange: float
    mfAimDistanceAngleMin: float
    mfAimDistanceAngleRange: float
    mfAimDistanceMin: float
    mfAimDistanceRange: float

class cGcPlayerSpaceshipClassBonuses(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfThrustForceMax", ctypes.c_float),
        ("mfThrustForceMin", ctypes.c_float),
        ("mfMaxSpeedMax", ctypes.c_float),
        ("mfMaxSpeedMin", ctypes.c_float),
        ("mfBoostMaxSpeedMax", ctypes.c_float),
        ("mfBoostMaxSpeedMin", ctypes.c_float),
        ("mfBoostingTurnDampMax", ctypes.c_float),
        ("mfBoostingTurnDampMin", ctypes.c_float),
        ("mfDirectionBrakeMin", ctypes.c_float),
        ("mfDirectionBrakeMax", ctypes.c_float),
        ("mfTurnStrengthMax", ctypes.c_float),
        ("mfTurnStrengthMin", ctypes.c_float),
    ]
    mfThrustForceMax: float
    mfThrustForceMin: float
    mfMaxSpeedMax: float
    mfMaxSpeedMin: float
    mfBoostMaxSpeedMax: float
    mfBoostMaxSpeedMin: float
    mfBoostingTurnDampMax: float
    mfBoostingTurnDampMin: float
    mfDirectionBrakeMin: float
    mfDirectionBrakeMax: float
    mfTurnStrengthMax: float
    mfTurnStrengthMin: float

class cGcPlayerSpaceshipEngineData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def Interpolate(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfThrustForce", ctypes.c_float),
        ("mfMaxSpeed", ctypes.c_float),
        ("mfMinSpeed", ctypes.c_float),
        ("mfFalloff", ctypes.c_float),
        ("mfMinSpeedForce", ctypes.c_float),
        ("mfBoostThrustForce", ctypes.c_float),
        ("mfBoostMaxSpeed", ctypes.c_float),
        ("mfBoostFalloff", ctypes.c_float),
        ("mfBoostingTurnDamp", ctypes.c_float),
        ("mfDirectionBrakeMin", ctypes.c_float),
        ("mfDirectionBrake", ctypes.c_float),
        ("mfReverseBrake", ctypes.c_float),
        ("mfOverspeedBrake", ctypes.c_float),
        ("mfLowSpeedTurnDamper", ctypes.c_float),
        ("mfTurnBrakeMin", ctypes.c_float),
        ("mfTurnBrakeMax", ctypes.c_float),
        ("mfTurnStrength", ctypes.c_float),
        ("mfRollAmount", ctypes.c_float),
        ("mfRollForce", ctypes.c_float),
        ("mfRollAutoTime", ctypes.c_float),
        ("mfBalanceTimeMin", ctypes.c_float),
        ("mfBalanceTimeMax", ctypes.c_float),
        ("mfFollowProportionalGain", ctypes.c_float),
        ("mfFollowDerivativeGain", ctypes.c_float),
        ("mfFollowIntegralGain", ctypes.c_float),
        ("mfFollowProportionalLimit", ctypes.c_float),
        ("mfFollowDerivativeLimit", ctypes.c_float),
        ("mfFollowIntegralLimit", ctypes.c_float),
        ("mfFollowIntegralDecay", ctypes.c_float),
    ]
    mfThrustForce: float
    mfMaxSpeed: float
    mfMinSpeed: float
    mfFalloff: float
    mfMinSpeedForce: float
    mfBoostThrustForce: float
    mfBoostMaxSpeed: float
    mfBoostFalloff: float
    mfBoostingTurnDamp: float
    mfDirectionBrakeMin: float
    mfDirectionBrake: float
    mfReverseBrake: float
    mfOverspeedBrake: float
    mfLowSpeedTurnDamper: float
    mfTurnBrakeMin: float
    mfTurnBrakeMax: float
    mfTurnStrength: float
    mfRollAmount: float
    mfRollForce: float
    mfRollAutoTime: float
    mfBalanceTimeMin: float
    mfBalanceTimeMax: float
    mfFollowProportionalGain: float
    mfFollowDerivativeGain: float
    mfFollowIntegralGain: float
    mfFollowProportionalLimit: float
    mfFollowDerivativeLimit: float
    mfFollowIntegralLimit: float
    mfFollowIntegralDecay: float

class cGcShieldComponentData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miType", ctypes.c_int32),
    ]
    miType: int

class cGcShipAIAttackData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mfGunDispersionAngle", ctypes.c_float),
        ("mfGunFireRate", ctypes.c_float),
        ("mfLaserHealthPoint", ctypes.c_float),
        ("mfAttackWeaponRange", ctypes.c_float),
        ("mfAttackAngle", ctypes.c_float),
        ("mfAttackShootWaitTime", ctypes.c_float),
        ("mfAttackShootTimeMin", ctypes.c_float),
        ("mfAttackShootTimeMax", ctypes.c_float),
        ("mfAttackReadyTime", ctypes.c_float),
        ("mfAttackMaxTime", ctypes.c_float),
        ("mfAttackApproachOffset", ctypes.c_float),
        ("mfAttackApproachMinRange", ctypes.c_float),
        ("mfAttackApproachMaxRange", ctypes.c_float),
        ("mfAttackTooCloseRange", ctypes.c_float),
        ("mfAttackFlybyOffset", ctypes.c_float),
        ("mfAttackBoostTimeToRange", ctypes.c_float),
        ("mfAttackBoostRange", ctypes.c_float),
        ("mfAttackBoostAngle", ctypes.c_float),
        ("mfAttackTurnMultiplier", ctypes.c_float),
        ("mfAttackMaxPlanetHeight", ctypes.c_float),
        ("mfAttackTurnMultiplierMax", ctypes.c_float),
        ("mfAttackTurnMaxMinTime", ctypes.c_float),
        ("mfAttackTurnMaxTimeRange", ctypes.c_float),
        ("miNumHitsBeforeBail", ctypes.c_int32),
        ("miNumHitsBeforeReposition", ctypes.c_int32),
        ("mfFleeBrake", ctypes.c_float),
        ("mfFleeBoost", ctypes.c_float),
        ("mfFleeBrakeTime", ctypes.c_float),
        ("mfFleeRepositionTime", ctypes.c_float),
        ("mfFleeRepositionAngleMin", ctypes.c_float),
        ("mfFleeRepositionAngleMax", ctypes.c_float),
        ("mfFleeUrgentBrake", ctypes.c_float),
        ("mfFleeUrgentBoost", ctypes.c_float),
        ("mfFleeUrgentBrakeTime", ctypes.c_float),
        ("mfFleeRepositionUrgentTime", ctypes.c_float),
        ("mfFleeRepositionUrgentAngleMin", ctypes.c_float),
        ("mfFleeRepositionUrgentAngleMax", ctypes.c_float),
        ("mfFleeRange", ctypes.c_float),
        ("mfFleeMinTime", ctypes.c_float),
        ("mfFleeMaxTime", ctypes.c_float),
        ("mfFleeUrgentRange", ctypes.c_float),
        ("mfPlanetFleeHeightExtra", ctypes.c_float),
        ("mfAttackTargetMinRange", ctypes.c_float),
        ("mfAttackTargetMaxRange", ctypes.c_float),
        ("mfAttackTargetOffsetMin", ctypes.c_float),
        ("mfAttackTargetOffsetMax", ctypes.c_float),
        ("mfAttackTargetSwitchTargetTime", ctypes.c_float),
    ]
    mId: int
    mfGunDispersionAngle: float
    mfGunFireRate: float
    mfLaserHealthPoint: float
    mfAttackWeaponRange: float
    mfAttackAngle: float
    mfAttackShootWaitTime: float
    mfAttackShootTimeMin: float
    mfAttackShootTimeMax: float
    mfAttackReadyTime: float
    mfAttackMaxTime: float
    mfAttackApproachOffset: float
    mfAttackApproachMinRange: float
    mfAttackApproachMaxRange: float
    mfAttackTooCloseRange: float
    mfAttackFlybyOffset: float
    mfAttackBoostTimeToRange: float
    mfAttackBoostRange: float
    mfAttackBoostAngle: float
    mfAttackTurnMultiplier: float
    mfAttackMaxPlanetHeight: float
    mfAttackTurnMultiplierMax: float
    mfAttackTurnMaxMinTime: float
    mfAttackTurnMaxTimeRange: float
    miNumHitsBeforeBail: int
    miNumHitsBeforeReposition: int
    mfFleeBrake: float
    mfFleeBoost: float
    mfFleeBrakeTime: float
    mfFleeRepositionTime: float
    mfFleeRepositionAngleMin: float
    mfFleeRepositionAngleMax: float
    mfFleeUrgentBrake: float
    mfFleeUrgentBoost: float
    mfFleeUrgentBrakeTime: float
    mfFleeRepositionUrgentTime: float
    mfFleeRepositionUrgentAngleMin: float
    mfFleeRepositionUrgentAngleMax: float
    mfFleeRange: float
    mfFleeMinTime: float
    mfFleeMaxTime: float
    mfFleeUrgentRange: float
    mfPlanetFleeHeightExtra: float
    mfAttackTargetMinRange: float
    mfAttackTargetMaxRange: float
    mfAttackTargetOffsetMin: float
    mfAttackTargetOffsetMax: float
    mfAttackTargetSwitchTargetTime: float

class cGcSpaceshipShieldData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("miHealth", ctypes.c_int32),
        ("miLevelledExtraHealth", ctypes.c_int32),
        ("mfRechargeTime", ctypes.c_float),
        ("mfRechargeDelayTime", ctypes.c_float),
    ]
    mId: int
    miHealth: int
    miLevelledExtraHealth: int
    mfRechargeTime: float
    mfRechargeDelayTime: float

class cGcShipAIAttackDataTable(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maDefinitions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcShipAICombatDefinition>
        ("maBehaviourTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcShipAIAttackData>
        ("maEngineTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSpaceshipTravelData>
        ("maShieldTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSpaceshipShieldData>
        ("maTraderAttackLookup", ctypes.c_char * 0x40),  # cTkFixedArray<TkID<128>,4>
    ]
    maDefinitions: bytes
    maBehaviourTable: bytes
    maEngineTable: bytes
    maShieldTable: bytes
    maTraderAttackLookup: bytes

class cGcTerrainEditing(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfEditPlaneMaxAdditiveOffsetFactor", ctypes.c_float),
        ("mfEditPlaneMinAdditiveOffsetFactor", ctypes.c_float),
        ("mfEditPlaneMaxSubtractiveOffsetFactor", ctypes.c_float),
        ("mfEditPlaneMinSubtractiveOffsetFactor", ctypes.c_float),
        ("mfTerrainEditBeamMaxRange", ctypes.c_float),
        ("mfTerrainEditBeamSpherecastRadius", ctypes.c_float),
        ("mfTerrainEditBeamAddInterpolationStepFactor", ctypes.c_float),
        ("mfTerrainEditBeamSubtractInterpolationStepFactor", ctypes.c_float),
        ("mfTerrainBlocksSearchRadiusMultiplier", ctypes.c_float),
        ("mfRegionMapSearchRadiusMultiplier", ctypes.c_float),
        ("mfMinimumSubstancePresence", ctypes.c_float),
        ("mfRegionEditAreaMultiplier", ctypes.c_float),
        ("mfDensityBlendDistanceMultiplier", ctypes.c_float),
        ("mfTerrainEditBaseDistanceTolerance", ctypes.c_float),
        ("mfTerrainUndoBaseDistanceTolerance", ctypes.c_float),
        ("mfUndoEditToleranceFactor", ctypes.c_float),
        ("mfUndoBaseEditEffectiveScale", ctypes.c_float),
        ("mfTerrainEditsNormalCostFactor", ctypes.c_float),
        ("mfTerrainEditsSurvivalCostFactor", ctypes.c_float),
        ("mfVoxelsDeletedAffectCostFactor", ctypes.c_float),
        ("mbEditGunBeamEnabled", ctypes.c_ubyte),
        ("mbEditGunParticlesEnabled", ctypes.c_ubyte),
        ("mbSubtractGunBeamEnabled", ctypes.c_ubyte),
        ("mbSubtractGunParticlesEnabled", ctypes.c_ubyte),
        ("mafEditSizes", ctypes.c_char * 0x20),  # cTkFixedArray<float,8>
        ("mafSubtractSizes", ctypes.c_char * 0xC),  # cTkFixedArray<float,3>
        ("mafBaseEditSizes", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("mafUndoEditSizes", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("mafFlatteningSizes", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("mfEditEffectScale", ctypes.c_float),
    ]
    mfEditPlaneMaxAdditiveOffsetFactor: float
    mfEditPlaneMinAdditiveOffsetFactor: float
    mfEditPlaneMaxSubtractiveOffsetFactor: float
    mfEditPlaneMinSubtractiveOffsetFactor: float
    mfTerrainEditBeamMaxRange: float
    mfTerrainEditBeamSpherecastRadius: float
    mfTerrainEditBeamAddInterpolationStepFactor: float
    mfTerrainEditBeamSubtractInterpolationStepFactor: float
    mfTerrainBlocksSearchRadiusMultiplier: float
    mfRegionMapSearchRadiusMultiplier: float
    mfMinimumSubstancePresence: float
    mfRegionEditAreaMultiplier: float
    mfDensityBlendDistanceMultiplier: float
    mfTerrainEditBaseDistanceTolerance: float
    mfTerrainUndoBaseDistanceTolerance: float
    mfUndoEditToleranceFactor: float
    mfUndoBaseEditEffectiveScale: float
    mfTerrainEditsNormalCostFactor: float
    mfTerrainEditsSurvivalCostFactor: float
    mfVoxelsDeletedAffectCostFactor: float
    mbEditGunBeamEnabled: bool
    mbEditGunParticlesEnabled: bool
    mbSubtractGunBeamEnabled: bool
    mbSubtractGunParticlesEnabled: bool
    mafEditSizes: bytes
    mafSubtractSizes: bytes
    mafBaseEditSizes: bytes
    mafUndoEditSizes: bytes
    mafFlatteningSizes: bytes
    mfEditEffectScale: float

class cGcBackgroundSpaceEncounterSpawnConditions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbNeedsEmptySystem", ctypes.c_ubyte),
        ("mbNeedsAbandonedSystem", ctypes.c_ubyte),
        ("mbNeedsPirateSystem", ctypes.c_ubyte),
        ("mbNeedsAsteroidField", ctypes.c_ubyte),
    ]
    mbNeedsEmptySystem: bool
    mbNeedsAbandonedSystem: bool
    mbNeedsPirateSystem: bool
    mbNeedsAsteroidField: bool

class cGcPulseEncounterSpawnConditions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbAllowedInCreative", ctypes.c_ubyte),
        ("mbAllowedInEmptySystem", ctypes.c_ubyte),
        ("mbAllowedDuringTutorial", ctypes.c_ubyte),
        ("mbAllowedBeyondPortals", ctypes.c_ubyte),
        ("mbAllowedWhileOnMPMission", ctypes.c_ubyte),
        ("mRequiresProduct", ctypes.c_ulonglong),
        ("mRequiresMissionComplete", ctypes.c_ulonglong),
        ("mRequiresMissionActive", ctypes.c_ulonglong),
        ("mRequiresMissionNotComplete", ctypes.c_ulonglong),
        ("mRequiresMissionNotActive", ctypes.c_ulonglong),
        ("mbRequiresAlienShip", ctypes.c_ubyte),
        ("mbStandardEncounter", ctypes.c_ubyte),
        ("mbMissionEncounter", ctypes.c_ubyte),
    ]
    mbAllowedInCreative: bool
    mbAllowedInEmptySystem: bool
    mbAllowedDuringTutorial: bool
    mbAllowedBeyondPortals: bool
    mbAllowedWhileOnMPMission: bool
    mRequiresProduct: int
    mRequiresMissionComplete: int
    mRequiresMissionActive: int
    mRequiresMissionNotComplete: int
    mRequiresMissionNotActive: int
    mbRequiresAlienShip: bool
    mbStandardEncounter: bool
    mbMissionEncounter: bool

class cGcPulseEncounterSpawnAlienFreighter(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mHailingPuzzleID", ctypes.c_ulonglong),
    ]
    mHailingPuzzleID: int

class cGcPulseEncounterSpawnPirates(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcDebugShipTravelLine(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mOrigin", common.Vector3f),
        ("mDir", common.Vector3f),
        ("mfLength", ctypes.c_float),
        ("mfInfluenceRange", ctypes.c_float),
    ]
    mOrigin: common.Vector3f
    mDir: common.Vector3f
    mfLength: float
    mfInfluenceRange: float

class cGcDebugCameraEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mOffset", common.Vector3f),
        ("mLocal", common.Vector3f),
        ("mFacing", common.Vector3f),
        ("mUp", common.Vector3f),
        ("mfFOV", ctypes.c_float),
        ("mfSpeedModifier", ctypes.c_float),
        ("mfDistance", ctypes.c_float),
    ]
    mOffset: common.Vector3f
    mLocal: common.Vector3f
    mFacing: common.Vector3f
    mUp: common.Vector3f
    mfFOV: float
    mfSpeedModifier: float
    mfDistance: float

class cGcDebugCamera(ctypes.Structure):
    def FixUp(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    def vector_deleting_destructor(): pass

    _fields_ = [
        ("mfBaseSpeed", ctypes.c_float),
        ("mfSmoothing", ctypes.c_float),
        ("maWaypoints", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDebugCameraEntry>
        ("miCurrentWaypoint", ctypes.c_int32),
        ("mfCurrentWaypointProgress", ctypes.c_float),
    ]
    mfBaseSpeed: float
    mfSmoothing: float
    maWaypoints: bytes
    miCurrentWaypoint: int
    mfCurrentWaypointProgress: float

class cGcAISpaceshipWeightingData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mafCivilianClassWeightings", ctypes.c_char * 0x24),  # cTkFixedArray<float,9>
    ]
    mafCivilianClassWeightings: bytes

class cGcShipAIPerformanceArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maArray", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
    ]
    maArray: bytes

class cGcTerrainTextureSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfBrightness", ctypes.c_float),
        ("mfContrast", ctypes.c_float),
        ("mfSpecular", ctypes.c_float),
    ]
    mfBrightness: float
    mfContrast: float
    mfSpecular: float

class cGcTerrainTexture(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macDiffuseTexture", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macNormalMap", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maTextureConfig", ctypes.c_char * 0x90),  # cTkFixedArray<cGcTerrainTextureSettings,12>
    ]
    macDiffuseTexture: bytes
    macNormalMap: bytes
    maTextureConfig: bytes

class cGcTileTypeSet(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfProbability", ctypes.c_float),
        ("maColours", ctypes.c_char * 0x60),  # cTkFixedArray<cTkPaletteTexture,12>
    ]
    mfProbability: float
    maColours: bytes

class cGcTileTypeSets(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTileTypeSets", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcTileTypeSet>
    ]
    maTileTypeSets: bytes

class cGcShipAIDeathData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfDroneDeathTime", ctypes.c_float),
        ("mfDroneDeathForce", ctypes.c_float),
        ("mfDroneDeathBoomTotalTime", ctypes.c_float),
        ("mfDroneDeathOffset", ctypes.c_float),
        ("miDroneNumDeathBooms", ctypes.c_int32),
        ("mfDroneDeathTimeout", ctypes.c_float),
        ("mfBrakeForce", ctypes.c_float),
    ]
    mfDroneDeathTime: float
    mfDroneDeathForce: float
    mfDroneDeathBoomTotalTime: float
    mfDroneDeathOffset: float
    miDroneNumDeathBooms: int
    mfDroneDeathTimeout: float
    mfBrakeForce: float

class cGcShipAIPlanetPatrolData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mSquad", ctypes.c_ulonglong),
        ("mfPlayerOffset", ctypes.c_float),
        ("mfPlayerFalloff", ctypes.c_float),
        ("mfPathOffset", ctypes.c_float),
        ("mfWaypointDistance", ctypes.c_float),
        ("mfPathSpeed", ctypes.c_float),
        ("mfAlongPathForce", ctypes.c_float),
        ("mfToPathForce", ctypes.c_float),
        ("mfBrakeForce", ctypes.c_float),
        ("mfAlignForce", ctypes.c_float),
    ]
    mSquad: int
    mfPlayerOffset: float
    mfPlayerFalloff: float
    mfPathOffset: float
    mfWaypointDistance: float
    mfPathSpeed: float
    mfAlongPathForce: float
    mfToPathForce: float
    mfBrakeForce: float
    mfAlignForce: float

class cGcSpaceshipAvoidanceData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miNumRays", ctypes.c_int32),
        ("mfRayMinRange", ctypes.c_float),
        ("mfRaySpeedTime", ctypes.c_float),
        ("mfForce", ctypes.c_float),
        ("mfStartRadiusMultiplier", ctypes.c_float),
        ("mfEndRadiusMultiplier", ctypes.c_float),
        ("mfSpeedInterp", ctypes.c_float),
        ("mfSpeedInterpMinSpeed", ctypes.c_float),
        ("mfSpeedInterpRange", ctypes.c_float),
    ]
    miNumRays: int
    mfRayMinRange: float
    mfRaySpeedTime: float
    mfForce: float
    mfStartRadiusMultiplier: float
    mfEndRadiusMultiplier: float
    mfSpeedInterp: float
    mfSpeedInterpMinSpeed: float
    mfSpeedInterpRange: float

class cGcSpaceshipShieldComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcExoMechWeaponData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maLocationPriority", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcMechWeaponLocation>
        ("mfMaintainFireLocationMinTime", ctypes.c_float),
        ("maMuzzleFlashDataID", ctypes.c_char * 0x40),  # cTkFixedArray<TkID<128>,4>
        ("mfAngleToleranceForArmAiming", ctypes.c_float),
        ("mfCooldownTimeMin", ctypes.c_float),
        ("mfCooldownTimeMax", ctypes.c_float),
        ("mfSelectionWeight", ctypes.c_float),
        ("mfMinRange", ctypes.c_float),
        ("mfMaxRange", ctypes.c_float),
        ("mfAttackAngle", ctypes.c_float),
    ]
    maLocationPriority: bytes
    mfMaintainFireLocationMinTime: float
    maMuzzleFlashDataID: bytes
    mfAngleToleranceForArmAiming: float
    mfCooldownTimeMin: float
    mfCooldownTimeMax: float
    mfSelectionWeight: float
    mfMinRange: float
    mfMaxRange: float
    mfAttackAngle: float

class cGcDoShipClearCommunication(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcDoShipFlybyClose(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfLockOffset", ctypes.c_float),
        ("mfLockTime", ctypes.c_float),
        ("mfLockSpread", ctypes.c_float),
        ("mbStayCloseAtLowSpeed", ctypes.c_ubyte),
        ("mbHailingBehaviour", ctypes.c_ubyte),
    ]
    mfLockOffset: float
    mfLockTime: float
    mfLockSpread: float
    mbStayCloseAtLowSpeed: bool
    mbHailingBehaviour: bool

class cGcDoShipFlybyIntercept(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfSpeed", ctypes.c_float),
        ("mfTime", ctypes.c_float),
    ]
    mfSpeed: float
    mfTime: float

class cGcDoShipFlybyMineAsteroids(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfTime", ctypes.c_float),
    ]
    mfTime: float

class cGcDoShipFlybyOverhead(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfLength", ctypes.c_float),
        ("mfOffset", ctypes.c_float),
    ]
    mfLength: float
    mfOffset: float

class cGcDoShipLandNextToPlayer(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mfLength", ctypes.c_float),
        ("mfOffset", ctypes.c_float),
    ]
    mfLength: float
    mfOffset: float

class cGcDoShipReceiveHail(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcDebugPlanetPos(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mPosition", common.Vector3f),
        ("mbOverridePosition", ctypes.c_ubyte),
    ]
    mPosition: common.Vector3f
    mbOverridePosition: bool

class cGcFreighterSyncComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbDummy", ctypes.c_ubyte),
    ]
    mbDummy: bool

class cGcShipDialogue(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maDialogueTree", ctypes.c_char * 0x268),  # cTkFixedArray<cGcPlayerCommunicatorMessageWeighted,7>
    ]
    maDialogueTree: bytes

class cGcPlayerSquadronConfig(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    def tilde_cGcPlayerSquadronConfig(): pass

    _fields_ = [
        ("macSquadName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("maPilotRankTraitRanges", ctypes.c_char * 0x20),  # cTkFixedArray<cTkVector2,4>
        ("maPilotRankAttackDefinitions", ctypes.c_char * 0x40),  # cTkFixedArray<TkID<128>,4>
        ("mfSummonArriveTime", ctypes.c_float),
        ("mfSummonArriveTimeIntervalMin", ctypes.c_float),
        ("mfSummonArriveTimeIntervalMax", ctypes.c_float),
        ("mfSummonStartSpeed", ctypes.c_float),
        ("mfSummonLimitTurningTime", ctypes.c_float),
        ("mfMinSpeedForSummonInMoveDir", ctypes.c_float),
        ("mfSummonSpaceSpawnRangeMin", ctypes.c_float),
        ("mfSummonSpaceSpawnRangeMax", ctypes.c_float),
        ("mfSummonSpaceSpawnAngleMin", ctypes.c_float),
        ("mfSummonSpaceSpawnAngleMax", ctypes.c_float),
        ("mfSummonPlanetPitchMin", ctypes.c_float),
        ("mfSummonPlanetPitchMax", ctypes.c_float),
        ("mfSummonPlanetYawMin", ctypes.c_float),
        ("mfSummonPlanetYawMax", ctypes.c_float),
        ("mfSummonPlanetDistance", ctypes.c_float),
        ("mbSummonInFormation", ctypes.c_ubyte),
        ("mfSummonInFormationFwdOffset", ctypes.c_float),
        ("mfLeavingForceScaleDistMin", ctypes.c_float),
        ("mfLeavingForceScaleDistMax", ctypes.c_float),
        ("mfLeavingMaxForceMultiplier", ctypes.c_float),
        ("mfLeavingFromPlanetOrbitWarpDist", ctypes.c_float),
        ("mfLeavingFromPlanetOrbitMinIncline", ctypes.c_float),
        ("mfLeavingFromPlanetOrbitMaxSpeed", ctypes.c_float),
        ("mfLeavingFromSpaceWarpDist", ctypes.c_float),
        ("mfLeavingFromSpaceAngleFromFwdMin", ctypes.c_float),
        ("mfLeavingFromSpaceAngleFromFwdMax", ctypes.c_float),
        ("mfFormationOffsetRotationPeriod", ctypes.c_float),
        ("mfFormationOffsetRotationMaxSpeed", ctypes.c_float),
        ("mfFormationOffsetZOffsetVarianceMinSpeedScale", ctypes.c_float),
        ("mfFormationOffsetZOffsetVarianceMaxSpeedScale", ctypes.c_float),
        ("mfFormationOffsetZOffsetVariancePeriod", ctypes.c_float),
        ("mfFormationOffsetZOffsetVarianceMin", ctypes.c_float),
        ("mfFormationOffsetZOffsetVarianceMax", ctypes.c_float),
        ("mFormationOffset", common.Vector3f),
        ("mfFormationOffsetCylinderWidth", ctypes.c_float),
        ("mfFormationOffsetCylinderHeight", ctypes.c_float),
        ("mfFormationOffsetCylinderLength", ctypes.c_float),
        ("mFormationOffsetThirdPerson", common.Vector3f),
        ("mfFormationOffsetCylinderWidthThirdPerson", ctypes.c_float),
        ("mfFormationOffsetCylinderHeightThirdPerson", ctypes.c_float),
        ("mfFormationOffsetCylinderLengthThirdPerson", ctypes.c_float),
        ("mCombatFormationOffset", common.Vector3f),
        ("mfCombatFormationOffsetCylinderWidth", ctypes.c_float),
        ("mfCombatFormationOffsetCylinderHeight", ctypes.c_float),
        ("mfCombatFormationOffsetCylinderLength", ctypes.c_float),
        ("mCombatFormationOffsetThirdPerson", common.Vector3f),
        ("mfCombatFormationOffsetCylinderWidthThirdPerson", ctypes.c_float),
        ("mfCombatFormationOffsetCylinderHeightThirdPerson", ctypes.c_float),
        ("mfCombatFormationOffsetCylinderLengthThirdPerson", ctypes.c_float),
        ("mfMaintainFormationStrengthVariance", ctypes.c_float),
        ("mfMaintainFormationInCombatMinTime", ctypes.c_float),
        ("mfMaintainFormationInCombatMaxTime", ctypes.c_float),
        ("mfMaintainFormationStartSmoothTime", ctypes.c_float),
        ("mfMaintainFormationMaxForce", ctypes.c_float),
        ("mfMaintainFormationLockStrength", ctypes.c_float),
        ("mfMaintainFormationLockStrengthCombat", ctypes.c_float),
        ("mfMaintainFormationLockStrengthBoosting", ctypes.c_float),
        ("mfMaintainFormationPostBoostSmoothTime", ctypes.c_float),
        ("mfMaintainFormationLockAlignStrength", ctypes.c_float),
        ("mfMaintainFormationLockRollAlignStrength", ctypes.c_float),
        ("mfMaintainFormationAlignMinDist", ctypes.c_float),
        ("mfMaintainFormationAlignMaxDist", ctypes.c_float),
        ("mfMaintainFormationAlignMinSpeed", ctypes.c_float),
        ("mfMaintainFormationAlignMaxSpeed", ctypes.c_float),
        ("mfMaintainFormationSharpTurnMinSpeed", ctypes.c_float),
        ("mfMaintainFormationSharpTurnMinSpeedForce", ctypes.c_float),
        ("mfJoinFormationMaxForce", ctypes.c_float),
        ("mfJoinFormationOffset", ctypes.c_float),
        ("mfJoinFormationArrivalTolerance", ctypes.c_float),
        ("mfJoinFormationBoostMaxSpeed", ctypes.c_float),
        ("mfJoinFormationBoostMinDist", ctypes.c_float),
        ("mfJoinFormationBoostMaxDist", ctypes.c_float),
        ("mfJoinFormationBoostAlignStrength", ctypes.c_float),
        ("mfJoinFormationBrakeAlignStrength", ctypes.c_float),
        ("mfJoinFormationBrakeDist", ctypes.c_float),
        ("mfJoinFormationMinSpeed", ctypes.c_float),
        ("mfJoinFormationMaxSpeedBrake", ctypes.c_float),
        ("mfBreakFormationTime", ctypes.c_float),
        ("mfBreakFormationMaxForce", ctypes.c_float),
        ("mfBreakFormationMinTurnAngle", ctypes.c_float),
        ("mfBreakFormationMaxTurnAngle", ctypes.c_float),
        ("mfMinTimeBetweenFormationBreaks", ctypes.c_float),
        ("miMinShipsInFormationDuringCombat", ctypes.c_int32),
        ("miMaxShipsInFormationDuringCombat", ctypes.c_int32),
        ("mfOutOfFormationMinTime", ctypes.c_float),
        ("mfOutOfFormationMaxTime", ctypes.c_float),
        ("maRandomSpaceshipResources", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcAISpaceshipModelData>
        ("maacRandomPilotNPCResources", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    macSquadName: bytes
    maPilotRankTraitRanges: bytes
    maPilotRankAttackDefinitions: bytes
    mfSummonArriveTime: float
    mfSummonArriveTimeIntervalMin: float
    mfSummonArriveTimeIntervalMax: float
    mfSummonStartSpeed: float
    mfSummonLimitTurningTime: float
    mfMinSpeedForSummonInMoveDir: float
    mfSummonSpaceSpawnRangeMin: float
    mfSummonSpaceSpawnRangeMax: float
    mfSummonSpaceSpawnAngleMin: float
    mfSummonSpaceSpawnAngleMax: float
    mfSummonPlanetPitchMin: float
    mfSummonPlanetPitchMax: float
    mfSummonPlanetYawMin: float
    mfSummonPlanetYawMax: float
    mfSummonPlanetDistance: float
    mbSummonInFormation: bool
    mfSummonInFormationFwdOffset: float
    mfLeavingForceScaleDistMin: float
    mfLeavingForceScaleDistMax: float
    mfLeavingMaxForceMultiplier: float
    mfLeavingFromPlanetOrbitWarpDist: float
    mfLeavingFromPlanetOrbitMinIncline: float
    mfLeavingFromPlanetOrbitMaxSpeed: float
    mfLeavingFromSpaceWarpDist: float
    mfLeavingFromSpaceAngleFromFwdMin: float
    mfLeavingFromSpaceAngleFromFwdMax: float
    mfFormationOffsetRotationPeriod: float
    mfFormationOffsetRotationMaxSpeed: float
    mfFormationOffsetZOffsetVarianceMinSpeedScale: float
    mfFormationOffsetZOffsetVarianceMaxSpeedScale: float
    mfFormationOffsetZOffsetVariancePeriod: float
    mfFormationOffsetZOffsetVarianceMin: float
    mfFormationOffsetZOffsetVarianceMax: float
    mFormationOffset: common.Vector3f
    mfFormationOffsetCylinderWidth: float
    mfFormationOffsetCylinderHeight: float
    mfFormationOffsetCylinderLength: float
    mFormationOffsetThirdPerson: common.Vector3f
    mfFormationOffsetCylinderWidthThirdPerson: float
    mfFormationOffsetCylinderHeightThirdPerson: float
    mfFormationOffsetCylinderLengthThirdPerson: float
    mCombatFormationOffset: common.Vector3f
    mfCombatFormationOffsetCylinderWidth: float
    mfCombatFormationOffsetCylinderHeight: float
    mfCombatFormationOffsetCylinderLength: float
    mCombatFormationOffsetThirdPerson: common.Vector3f
    mfCombatFormationOffsetCylinderWidthThirdPerson: float
    mfCombatFormationOffsetCylinderHeightThirdPerson: float
    mfCombatFormationOffsetCylinderLengthThirdPerson: float
    mfMaintainFormationStrengthVariance: float
    mfMaintainFormationInCombatMinTime: float
    mfMaintainFormationInCombatMaxTime: float
    mfMaintainFormationStartSmoothTime: float
    mfMaintainFormationMaxForce: float
    mfMaintainFormationLockStrength: float
    mfMaintainFormationLockStrengthCombat: float
    mfMaintainFormationLockStrengthBoosting: float
    mfMaintainFormationPostBoostSmoothTime: float
    mfMaintainFormationLockAlignStrength: float
    mfMaintainFormationLockRollAlignStrength: float
    mfMaintainFormationAlignMinDist: float
    mfMaintainFormationAlignMaxDist: float
    mfMaintainFormationAlignMinSpeed: float
    mfMaintainFormationAlignMaxSpeed: float
    mfMaintainFormationSharpTurnMinSpeed: float
    mfMaintainFormationSharpTurnMinSpeedForce: float
    mfJoinFormationMaxForce: float
    mfJoinFormationOffset: float
    mfJoinFormationArrivalTolerance: float
    mfJoinFormationBoostMaxSpeed: float
    mfJoinFormationBoostMinDist: float
    mfJoinFormationBoostMaxDist: float
    mfJoinFormationBoostAlignStrength: float
    mfJoinFormationBrakeAlignStrength: float
    mfJoinFormationBrakeDist: float
    mfJoinFormationMinSpeed: float
    mfJoinFormationMaxSpeedBrake: float
    mfBreakFormationTime: float
    mfBreakFormationMaxForce: float
    mfBreakFormationMinTurnAngle: float
    mfBreakFormationMaxTurnAngle: float
    mfMinTimeBetweenFormationBreaks: float
    miMinShipsInFormationDuringCombat: int
    miMaxShipsInFormationDuringCombat: int
    mfOutOfFormationMinTime: float
    mfOutOfFormationMaxTime: float
    maRandomSpaceshipResources: bytes
    maacRandomPilotNPCResources: bytes

class cGcPlanetWaterColourData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mWaterFogColourNear", common.Colour),
        ("mWaterFogColourFar", common.Colour),
        ("mWaterColourBase", common.Colour),
        ("mWaterColourAdd", common.Colour),
        ("mFoamColour", common.Colour),
    ]
    mWaterFogColourNear: common.Colour
    mWaterFogColourFar: common.Colour
    mWaterColourBase: common.Colour
    mWaterColourAdd: common.Colour
    mFoamColour: common.Colour

class cGcWaterColourSettingList(ctypes.Structure):
    def FixUp(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSettings", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlanetWaterColourData>
    ]
    maSettings: bytes

class cGcCharacterCustomisationTextureOptionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTextureOptionGroupName", ctypes.c_ulonglong),
        ("mTextureOptionName", ctypes.c_ulonglong),
    ]
    mTextureOptionGroupName: int
    mTextureOptionName: int

class cGcCharacterCustomisationBoneScaleData(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mBoneName", ctypes.c_ulonglong),
        ("mfScale", ctypes.c_float),
    ]
    mBoneName: int
    mfScale: float

class cGcPlanetWeatherColourData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mSkyColour", common.Colour),
        ("mSkyUpperColour", common.Colour),
        ("mSkySolarColour", common.Colour),
        ("mHorizonColour", common.Colour),
        ("mSunColour", common.Colour),
        ("mFogColour", common.Colour),
        ("mHeightFogColour", common.Colour),
        ("mSkyGradientSpeed", common.Vector3f),
        ("mLightColour", common.Colour),
        ("mCloudColour1", common.Colour),
        ("mCloudColour2", common.Colour),
    ]
    mSkyColour: common.Colour
    mSkyUpperColour: common.Colour
    mSkySolarColour: common.Colour
    mHorizonColour: common.Colour
    mSunColour: common.Colour
    mFogColour: common.Colour
    mHeightFogColour: common.Colour
    mSkyGradientSpeed: common.Vector3f
    mLightColour: common.Colour
    mCloudColour1: common.Colour
    mCloudColour2: common.Colour

class cGcWeatherColourSettingList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSettings", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlanetWeatherColourData>
    ]
    maSettings: bytes

class cGcCharacterCustomisationData(ctypes.Structure):
    def cGcCharacterCustomisationData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maDescriptorGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maColours", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCharacterCustomisationColourData>
        ("maTextureOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCharacterCustomisationTextureOptionData>
        ("maBoneScales", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCharacterCustomisationBoneScaleData>
        ("mfScale", ctypes.c_float),
    ]
    maDescriptorGroups: bytes
    maColours: bytes
    maTextureOptions: bytes
    maBoneScales: bytes
    mfScale: float

class cGcPlanetSectionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64DiscovererUID", ctypes.c_uint64),
        ("maDiscovererPlatform", ctypes.c_char * 0x2),  # cTkFixedArray<unsigned char,2>
        ("mbDiscoveredState", ctypes.c_ubyte),
    ]
    mu64DiscovererUID: int
    maDiscovererPlatform: bytes
    mbDiscoveredState: bool

class cGcWeatherWeightings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mafWeatherWeightings", ctypes.c_char * 0x40),  # cTkFixedArray<float,16>
    ]
    mafWeatherWeightings: bytes

class cGcAIShipDebugSpawnData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPosition", common.Vector3f),
        ("mFacing", common.Vector3f),
        ("mUp", common.Vector3f),
        ("mFlightDir", common.Vector3f),
        ("mfSpeed", ctypes.c_float),
        ("mfIgnitionDelay", ctypes.c_float),
        ("mfTakeOffDelay", ctypes.c_float),
        ("mfHoverTime", ctypes.c_float),
        ("mfHoverHeight", ctypes.c_float),
        ("mfWarpOutTime", ctypes.c_float),
        ("mbWingman", ctypes.c_ubyte),
        ("macSpecificModel", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mSeed", common.GcSeed),
    ]
    mPosition: common.Vector3f
    mFacing: common.Vector3f
    mUp: common.Vector3f
    mFlightDir: common.Vector3f
    mfSpeed: float
    mfIgnitionDelay: float
    mfTakeOffDelay: float
    mfHoverTime: float
    mfHoverHeight: float
    mfWarpOutTime: float
    mbWingman: bool
    macSpecificModel: bytes
    mSeed: common.GcSeed

class cGcHeavyAirColourData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mColour1", common.Colour),
        ("mColour2", common.Colour),
    ]
    mColour1: common.Colour
    mColour2: common.Colour

class cGcPlanetHeavyAirData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maColours", ctypes.c_char * 0xA0),  # cTkFixedArray<cGcHeavyAirColourData,5>
    ]
    macFilename: bytes
    maColours: bytes

class cGcAISpaceshipInstanceData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macFile: bytes

class cGcAISpaceshipManagerData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSystemSpaceships", ctypes.c_char * 0x50),  # cTkFixedArray<cGcAISpaceshipModelDataArray,5>
    ]
    maSystemSpaceships: bytes

class cGcAISpaceshipMappingData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maClassMap", ctypes.c_char * 0x380),  # cTkFixedArray<cGcAISpaceshipInstanceData,7>
    ]
    maClassMap: bytes

class cGcSolarSystemSkyColourData(ctypes.Structure):
    def cGcSolarSystemSkyColourData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTopColour", common.Colour),
        ("mMidColour", common.Colour),
        ("mBottomColour", common.Colour),
        ("mTopColourPlanet", common.Colour),
        ("mMidColourPlanet", common.Colour),
        ("mBottomColourPlanet", common.Colour),
        ("mCloudColour", common.Colour),
        ("mLightColour", common.Colour),
        ("mNebulaColour1", common.Colour),
        ("mNebulaColour2", common.Colour),
        ("mNebulaColour3", common.Colour),
        ("mFogColour", common.Colour),
        ("mFogColour2", common.Colour),
    ]
    mTopColour: common.Colour
    mMidColour: common.Colour
    mBottomColour: common.Colour
    mTopColourPlanet: common.Colour
    mMidColourPlanet: common.Colour
    mBottomColourPlanet: common.Colour
    mCloudColour: common.Colour
    mLightColour: common.Colour
    mNebulaColour1: common.Colour
    mNebulaColour2: common.Colour
    mNebulaColour3: common.Colour
    mFogColour: common.Colour
    mFogColour2: common.Colour

class cGcSpaceSkyColourSettingList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSettings", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSolarSystemSkyColourData>
    ]
    maSettings: bytes

class cGcSpawnDensityList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maDensityList", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSpawnDensity>
    ]
    maDensityList: bytes

class cGcTerrainControls(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mafNoiseLayers", ctypes.c_char * 0x20),  # cTkFixedArray<float,8>
        ("mafGridLayers", ctypes.c_char * 0x24),  # cTkFixedArray<float,9>
        ("mafFeatures", ctypes.c_char * 0x1C),  # cTkFixedArray<float,7>
        ("mafCaves", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("mfWaterActiveFrequency", ctypes.c_float),
        ("mfHighWaterActiveFrequency", ctypes.c_float),
        ("mfRockTileFrequency", ctypes.c_float),
        ("mfSubstanceTileFrequency", ctypes.c_float),
        ("mbForceContinentalNoise", ctypes.c_ubyte),
    ]
    mafNoiseLayers: bytes
    mafGridLayers: bytes
    mafFeatures: bytes
    mafCaves: bytes
    mfWaterActiveFrequency: float
    mfHighWaterActiveFrequency: float
    mfRockTileFrequency: float
    mfSubstanceTileFrequency: float
    mbForceContinentalNoise: bool

class cGcMechTargetSelectionWeightingSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfThreatWeightFactorBase", ctypes.c_float),
        ("mfFwdDirectionWeightFactorBase", ctypes.c_float),
        ("mfDistanceWeightFactorBase", ctypes.c_float),
        ("mfVeryCloseDistance", ctypes.c_float),
        ("mfVeryCloseDistanceWeightFactorExponent", ctypes.c_float),
        ("mfCloseDistance", ctypes.c_float),
        ("mfCloseDistanceWeightFactorExponent", ctypes.c_float),
        ("mfMidDistance", ctypes.c_float),
        ("mfMidDistanceWeightFactorExponent", ctypes.c_float),
        ("mfFarDistance", ctypes.c_float),
        ("mfFarDistanceWeightFactorExponent", ctypes.c_float),
        ("mfVeryFarDistanceWeightFactorExponent", ctypes.c_float),
    ]
    mfThreatWeightFactorBase: float
    mfFwdDirectionWeightFactorBase: float
    mfDistanceWeightFactorBase: float
    mfVeryCloseDistance: float
    mfVeryCloseDistanceWeightFactorExponent: float
    mfCloseDistance: float
    mfCloseDistanceWeightFactorExponent: float
    mfMidDistance: float
    mfMidDistanceWeightFactorExponent: float
    mfFarDistance: float
    mfFarDistanceWeightFactorExponent: float
    mfVeryFarDistanceWeightFactorExponent: float

class cGcSmokeBotStats(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfAvgCpuFPS", ctypes.c_float),
        ("mfMinCpuFPS", ctypes.c_float),
        ("mfMaxCpuFPS", ctypes.c_float),
        ("mMinCpuFPSPos", common.Vector3f),
        ("mMinCpuFPSFacing", common.Vector3f),
        ("mfAvgGpuFPS", ctypes.c_float),
        ("mfMinGpuFPS", ctypes.c_float),
        ("mfMaxGpuFPS", ctypes.c_float),
        ("mMinGpuFPSPos", common.Vector3f),
        ("mMinGpuFPSFacing", common.Vector3f),
        ("mfMinMemory", ctypes.c_float),
        ("mMinMemoryPos", common.Vector3f),
        ("mMinMemoryFacing", common.Vector3f),
        ("mfTotalCpuFps", ctypes.c_float),
        ("mfTotalGpuFps", ctypes.c_float),
        ("miFrameCount", ctypes.c_int32),
    ]
    mfAvgCpuFPS: float
    mfMinCpuFPS: float
    mfMaxCpuFPS: float
    mMinCpuFPSPos: common.Vector3f
    mMinCpuFPSFacing: common.Vector3f
    mfAvgGpuFPS: float
    mfMinGpuFPS: float
    mfMaxGpuFPS: float
    mMinGpuFPSPos: common.Vector3f
    mMinGpuFPSFacing: common.Vector3f
    mfMinMemory: float
    mMinMemoryPos: common.Vector3f
    mMinMemoryFacing: common.Vector3f
    mfTotalCpuFps: float
    mfTotalGpuFps: float
    miFrameCount: int

class cGcSolarSystemTraderSpawnData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miMaxToSpawn", ctypes.c_int32),
        ("mfInitialTakeoffDelay", ctypes.c_float),
        ("mSequenceTakeoffDelay", common.Vector2f),
        ("miChanceToDelayLaunch", ctypes.c_int32),
    ]
    miMaxToSpawn: int
    mfInitialTakeoffDelay: float
    mSequenceTakeoffDelay: common.Vector2f
    miChanceToDelayLaunch: int

class cGcRobotLaserData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mLaserID", ctypes.c_ulonglong),
        ("mfLaserSpringTime", ctypes.c_float),
        ("mfLaserActiveSpringTime", ctypes.c_float),
        ("miLaserMiningDamage", ctypes.c_int32),
        ("mfLaserChargeTime", ctypes.c_float),
        ("mfLaserTime", ctypes.c_float),
        ("mfLaserLightChargeSize", ctypes.c_float),
        ("mfLaserLightAttackSize", ctypes.c_float),
        ("mLaserLightOffset", common.Vector3f),
        ("mLaserColour", common.Colour),
    ]
    mLaserID: int
    mfLaserSpringTime: float
    mfLaserActiveSpringTime: float
    miLaserMiningDamage: int
    mfLaserChargeTime: float
    mfLaserTime: float
    mfLaserLightChargeSize: float
    mfLaserLightAttackSize: float
    mLaserLightOffset: common.Vector3f
    mLaserColour: common.Colour

class cGcBiomeCloudSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMinCover", ctypes.c_float),
        ("mfMaxCover", ctypes.c_float),
        ("mfMinVariance", ctypes.c_float),
        ("mfMaxVariance", ctypes.c_float),
        ("mfMinRateOfChange", ctypes.c_float),
        ("mfMaxRateOfChange", ctypes.c_float),
        ("mfMinRatio", ctypes.c_float),
        ("mfMaxRatio", ctypes.c_float),
        ("mfTendencyTowardsBeingCloudy", ctypes.c_float),
        ("mStormCloudTopColour", common.Colour),
        ("mStormCloudBottomColour", common.Colour),
    ]
    mfMinCover: float
    mfMaxCover: float
    mfMinVariance: float
    mfMaxVariance: float
    mfMinRateOfChange: float
    mfMaxRateOfChange: float
    mfMinRatio: float
    mfMaxRatio: float
    mfTendencyTowardsBeingCloudy: float
    mStormCloudTopColour: common.Colour
    mStormCloudBottomColour: common.Colour

class cGcSmokeBotReport(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64StartingUA", ctypes.c_uint64),
        ("maSystems", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSmokeBotSystemReport>
    ]
    mu64StartingUA: int
    maSystems: bytes

class cGcSentinelResource(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macResource", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miBaseHealth", ctypes.c_int32),
        ("miHealthIncreasePerLevel", ctypes.c_int32),
        ("mfRepairTime", ctypes.c_float),
        ("mfRepairThreshold", ctypes.c_float),
    ]
    macResource: bytes
    miBaseHealth: int
    miHealthIncreasePerLevel: int
    mfRepairTime: float
    mfRepairThreshold: float

class cGcExperienceTimers(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miHighChance", ctypes.c_int32),
        ("miLowChance", ctypes.c_int32),
        ("mHigh", common.Vector2f),
        ("mNormal", common.Vector2f),
        ("mLow", common.Vector2f),
    ]
    miHighChance: int
    miLowChance: int
    mHigh: common.Vector2f
    mNormal: common.Vector2f
    mLow: common.Vector2f

class cGcSolarSystemEventWarpOut(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfTime", ctypes.c_float),
        ("macSquadName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mWarpIntervalRange", common.Vector2f),
    ]
    mfTime: float
    macSquadName: bytes
    mWarpIntervalRange: common.Vector2f

class cGcButtonSpawnTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maButtonSpawns", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcButtonSpawn>
    ]
    maButtonSpawns: bytes

class cGcAtmosphereList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacAtmospheres", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    maacAtmospheres: bytes

class cGcBiomeList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mafBiomeProbability", ctypes.c_char * 0x40),  # cTkFixedArray<float,16>
        ("mafPrimeBiomeProbability", ctypes.c_char * 0x40),  # cTkFixedArray<float,16>
    ]
    mafBiomeProbability: bytes
    mafPrimeBiomeProbability: bytes

class cGcGeneratedShipCounts(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maiCounts", ctypes.c_char * 0x1C),  # cTkFixedArray<int,7>
    ]
    maiCounts: bytes

class cGcSolarGenerationData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64SolarSeed", ctypes.c_uint64),
    ]
    mu64SolarSeed: int

class cGcColourPaletteData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maColours", ctypes.c_char * 0x50),  # cTkFixedArray<cTkColour,5>
        ("maiColourIndices", ctypes.c_char * 0x14),  # cTkFixedArray<int,5>
    ]
    maColours: bytes
    maiColourIndices: bytes

class cGcPlanetColourData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maPalettes", ctypes.c_char * 0x1960),  # cTkFixedArray<cGcColourPaletteData,58>
    ]
    maPalettes: bytes

class cGcLightProperties(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mSunColour", common.Colour),
        ("mLightColour", common.Colour),
        ("mBounceColour", common.Colour),
    ]
    mSunColour: common.Colour
    mLightColour: common.Colour
    mBounceColour: common.Colour

class cGcTracerData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfLength", ctypes.c_float),
        ("mfSpeed", ctypes.c_float),
        ("mfDamageMinDistance", ctypes.c_float),
        ("mfDamageMaxDistance", ctypes.c_float),
        ("mfDamageMin", ctypes.c_float),
        ("mfDamageMax", ctypes.c_float),
    ]
    mfLength: float
    mfSpeed: float
    mfDamageMinDistance: float
    mfDamageMaxDistance: float
    mfDamageMin: float
    mfDamageMax: float

class cGcBiomeFileListOptions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maFileOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBiomeFileListOption>
    ]
    maFileOptions: bytes

class cGcCreatureHealthData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mHurtAnim", ctypes.c_ulonglong),
        ("mDeathAnim", ctypes.c_ulonglong),
        ("mDeathEffect", ctypes.c_ulonglong),
        ("mHurtAudio", ctypes.c_ulonglong),
        ("mDeathAudio", ctypes.c_ulonglong),
        ("mbDespawnOnDeath", ctypes.c_ubyte),
        ("maDespawnOnDeathDescriptors", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
    ]
    mHurtAnim: int
    mDeathAnim: int
    mDeathEffect: int
    mHurtAudio: int
    mDeathAudio: int
    mbDespawnOnDeath: bool
    maDespawnOnDeathDescriptors: bytes

class cGcDroneGun(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macLocator", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mAnim", ctypes.c_ulonglong),
    ]
    macLocator: bytes
    mAnim: int

class cGcOverlayTexture(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macOverlayDiffuse", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macOverlayNormal", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macOverlayMasks", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miOverlayMaskIdx", ctypes.c_int32),
    ]
    macOverlayDiffuse: bytes
    macOverlayNormal: bytes
    macOverlayMasks: bytes
    miOverlayMaskIdx: int

class cGcPlanetaryMappingData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64UA", ctypes.c_uint64),
        ("maSectionsData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlanetSectionData>
    ]
    mu64UA: int
    maSectionsData: bytes

class cGcPlanetaryMappingTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maMappingInfo", ctypes.c_char * 0x20),  # cTkFixedArray<cGcPlanetaryMappingValues,4>
    ]
    maMappingInfo: bytes

class cGcPlanetHazardData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mafTemperature", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
        ("mafToxicity", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
        ("mafRadiation", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
        ("mafLifeSupportDrain", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
    ]
    mafTemperature: bytes
    mafToxicity: bytes
    mafRadiation: bytes
    mafLifeSupportDrain: bytes

class cGcPlanetDataResourceHint(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mHint", ctypes.c_ulonglong),
        ("mIcon", ctypes.c_ulonglong),
    ]
    mHint: int
    mIcon: int

class cGcDroneControlData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMaxSpeed", ctypes.c_float),
        ("mfStrength", ctypes.c_float),
        ("mfDirectionBrake", ctypes.c_float),
        ("mfMinHeight", ctypes.c_float),
        ("mfMaxHeight", ctypes.c_float),
        ("mfMaxPitch", ctypes.c_float),
        ("mfHeightAdjustStrength", ctypes.c_float),
        ("mfHeightAdjustDownStrength", ctypes.c_float),
        ("mfLookStrength", ctypes.c_float),
        ("mfStopTime", ctypes.c_float),
        ("mfRepelForce", ctypes.c_float),
        ("mfRepelRange", ctypes.c_float),
    ]
    mfMaxSpeed: float
    mfStrength: float
    mfDirectionBrake: float
    mfMinHeight: float
    mfMaxHeight: float
    mfMaxPitch: float
    mfHeightAdjustStrength: float
    mfHeightAdjustDownStrength: float
    mfLookStrength: float
    mfStopTime: float
    mfRepelForce: float
    mfRepelRange: float

class cGcBiomeFileList(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBiomeFiles", ctypes.c_char * 0x100),  # cTkFixedArray<cGcBiomeFileListOptions,16>
        ("maValidStartPlanetBiome", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBiomeType>
        ("maCommonExternalObjectLists", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcExternalObjectListOptions>
        ("maOptionalExternalObjectLists", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcExternalObjectFileList>
    ]
    maBiomeFiles: bytes
    maValidStartPlanetBiome: bytes
    maCommonExternalObjectLists: bytes
    maOptionalExternalObjectLists: bytes

class cGcDroneResource(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macResource", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macResource: bytes

class cGcDroneWeaponData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mProjectile", ctypes.c_ulonglong),
        ("miNumProjectiles", ctypes.c_int32),
        ("mfProjectileSpread", ctypes.c_float),
        ("mfFireRate", ctypes.c_float),
        ("mbChangeBarrelEachShot", ctypes.c_ubyte),
        ("mfTimeout", ctypes.c_float),
        ("miNumShotsMin", ctypes.c_int32),
        ("miNumShotsMax", ctypes.c_int32),
        ("mfFireTimeMin", ctypes.c_float),
        ("mfFireTimeMax", ctypes.c_float),
        ("mfRange", ctypes.c_float),
        ("mfExplosionRadius", ctypes.c_float),
        ("mfInheritInitialVelocity", ctypes.c_float),
        ("mfMoveDistanceMin", ctypes.c_float),
        ("mfMoveDistanceMax", ctypes.c_float),
    ]
    mId: int
    mProjectile: int
    miNumProjectiles: int
    mfProjectileSpread: float
    mfFireRate: float
    mbChangeBarrelEachShot: bool
    mfTimeout: float
    miNumShotsMin: int
    miNumShotsMax: int
    mfFireTimeMin: float
    mfFireTimeMax: float
    mfRange: float
    mfExplosionRadius: float
    mfInheritInitialVelocity: float
    mfMoveDistanceMin: float
    mfMoveDistanceMax: float

class cGcObjectSpawnDataVariant(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mfCoverage", ctypes.c_float),
        ("mfFlatDensity", ctypes.c_float),
        ("mfSlopeDensity", ctypes.c_float),
        ("mfSlopeMultiplier", ctypes.c_float),
        ("miMaxRegionRadius", ctypes.c_int32),
        ("miMaxImposterRadius", ctypes.c_int32),
        ("mfFadeOutStartDistance", ctypes.c_float),
        ("mfFadeOutEndDistance", ctypes.c_float),
        ("mfFadeOutOffsetDistance", ctypes.c_float),
        ("mafLodDistances", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
    ]
    mID: int
    mfCoverage: float
    mfFlatDensity: float
    mfSlopeDensity: float
    mfSlopeMultiplier: float
    miMaxRegionRadius: int
    miMaxImposterRadius: int
    mfFadeOutStartDistance: float
    mfFadeOutEndDistance: float
    mfFadeOutOffsetDistance: float
    mafLodDistances: bytes

class cGcBoidData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfCoherence", ctypes.c_float),
        ("mfAlignment", ctypes.c_float),
        ("mfSeparation", ctypes.c_float),
        ("mfSpacing", ctypes.c_float),
        ("mfFollow", ctypes.c_float),
        ("mfDirectionBrake", ctypes.c_float),
        ("mfMaxSpeed", ctypes.c_float),
        ("mfInitTime", ctypes.c_float),
        ("mfInitOffset", ctypes.c_float),
        ("mfInitFacingOffset", ctypes.c_float),
        ("mfLeadTime", ctypes.c_float),
    ]
    mfCoherence: float
    mfAlignment: float
    mfSeparation: float
    mfSpacing: float
    mfFollow: float
    mfDirectionBrake: float
    mfMaxSpeed: float
    mfInitTime: float
    mfInitOffset: float
    mfInitFacingOffset: float
    mfLeadTime: float

class cGcSelectableObjectSpawnList(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("maObjects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSelectableObjectSpawnData>
    ]
    mName: int
    maObjects: bytes

class cGcEnvironmentSpawnData(ctypes.Structure):
    def tilde_cGcEnvironmentSpawnData(): pass

    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maCreatures", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureSpawnData>
        ("maDistantObjects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcObjectSpawnData>
        ("maLandmarks", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcObjectSpawnData>
        ("maObjects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcObjectSpawnData>
        ("maDetailObjects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcObjectSpawnData>
        ("maSelectableObjects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSelectableObjectSpawnList>
    ]
    maCreatures: bytes
    maDistantObjects: bytes
    maLandmarks: bytes
    maObjects: bytes
    maDetailObjects: bytes
    maSelectableObjects: bytes

class cGcBuildingSpawnSlot(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbHasBuilding", ctypes.c_ubyte),
        ("miBuildingDataIndex", ctypes.c_int32),
        ("mfProbability", ctypes.c_float),
    ]
    mbHasBuilding: bool
    miBuildingDataIndex: int
    mfProbability: float

class cGcBuildingOverrideData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mSeed", common.GcSeed),
        ("mPosition", common.Vector3f),
        ("miIndex", ctypes.c_int32),
    ]
    mSeed: common.GcSeed
    mPosition: common.Vector3f
    miIndex: int

class cGcPlanetBuildingData(ctypes.Structure):
    def tilde_cGcPlanetBuildingData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maBuildingSlots", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBuildingSpawnSlot>
        ("maBuildings", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBuildingSpawnData>
        ("maOverrideBuildings", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBuildingOverrideData>
        ("mfSpacing", ctypes.c_float),
        ("mfPlanetRadius", ctypes.c_float),
        ("mfVoronoiPointDivisions", ctypes.c_float),
        ("miVoronoiSectorSeed", ctypes.c_int32),
        ("miVoronoiPointSeed", ctypes.c_int32),
        ("mbInitialBuildingsPlaced", ctypes.c_ubyte),
        ("mbIsPrime", ctypes.c_ubyte),
    ]
    maBuildingSlots: bytes
    maBuildings: bytes
    maOverrideBuildings: bytes
    mfSpacing: float
    mfPlanetRadius: float
    mfVoronoiPointDivisions: float
    miVoronoiSectorSeed: int
    miVoronoiPointSeed: int
    mbInitialBuildingsPlaced: bool
    mbIsPrime: bool

class cGcHeavyAirList(ctypes.Structure):
    def FixUp(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    maacOptions: bytes

class cGcHeavyAirSetting(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maSettings", ctypes.c_char * 0xA0),  # cTkFixedArray<cGcHeavyAirSettingValues,5>
    ]
    maSettings: bytes

class cGcDamageMultiplierLookup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mfDefault", ctypes.c_float),
        ("maMultipliers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDamageMultiplier>
    ]
    mId: int
    mfDefault: float
    maMultipliers: bytes

class cGcRewardMissionOverride(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("mReward", ctypes.c_ulonglong),
    ]
    mMission: int
    mReward: int

class cGcCreatureRoleDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maAvailableRoles", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureRoleData>
        ("mfMaxProportionFlying", ctypes.c_float),
        ("mbHasSandWorms", ctypes.c_ubyte),
        ("mfSandWormFrequency", ctypes.c_float),
    ]
    maAvailableRoles: bytes
    mfMaxProportionFlying: float
    mbHasSandWorms: bool
    mfSandWormFrequency: float

class cGcPlanetInfo(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macPlanetDescription", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPlanetType", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macWeather", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macResources", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macFlora", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macFauna", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macSentinels", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbIsWeatherExtreme", ctypes.c_ubyte),
        ("mbAreSentinelsExtreme", ctypes.c_ubyte),
        ("mbAreSentinelsModerate", ctypes.c_ubyte),
    ]
    macPlanetDescription: bytes
    macPlanetType: bytes
    macWeather: bytes
    macResources: bytes
    macFlora: bytes
    macFauna: bytes
    macSentinels: bytes
    mbIsWeatherExtreme: bool
    mbAreSentinelsExtreme: bool
    mbAreSentinelsModerate: bool

class cGcPlanetRingData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbHasRings", ctypes.c_ubyte),
        ("mUp", common.Vector3f),
        ("mColour1", common.Colour),
        ("mColour2", common.Colour),
        ("mfLargeScale1", ctypes.c_float),
        ("mfLargeScale2", ctypes.c_float),
        ("mfMidScale", ctypes.c_float),
        ("mfSmallScale", ctypes.c_float),
        ("mfMidStrength", ctypes.c_float),
        ("mfOffset", ctypes.c_float),
        ("mfDepth", ctypes.c_float),
        ("mfAlphaMultiplier", ctypes.c_float),
    ]
    mbHasRings: bool
    mUp: common.Vector3f
    mColour1: common.Colour
    mColour2: common.Colour
    mfLargeScale1: float
    mfLargeScale2: float
    mfMidScale: float
    mfSmallScale: float
    mfMidStrength: float
    mfOffset: float
    mfDepth: float
    mfAlphaMultiplier: float

class cGcMissileComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mExplosion", ctypes.c_ulonglong),
        ("mfNoTargetLife", ctypes.c_float),
        ("mTrail", ctypes.c_ulonglong),
    ]
    mExplosion: int
    mfNoTargetLife: float
    mTrail: int

class cGcProjectileDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcProjectileData>
        ("maLasers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcLaserBeamData>
    ]
    maTable: bytes
    maLasers: bytes

class cGcProductToCollect(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mProduct", ctypes.c_ulonglong),
        ("miAmount", ctypes.c_int32),
    ]
    mProduct: int
    miAmount: int

class cGcMissionSequenceWaitForSettlementActivity(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageWhileBuilding", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageForVisitor", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageForConflict", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageForProposal", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macMessageWhileBuilding: bytes
    macMessageForVisitor: bytes
    macMessageForConflict: bytes
    macMessageForProposal: bytes
    macDebugText: bytes

class cGcMissionSequenceWaitForStat(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mStat", ctypes.c_ulonglong),
        ("mStatGroup", ctypes.c_ulonglong),
        ("miAmount", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mStat: int
    mStatGroup: int
    miAmount: int
    macDebugText: bytes

class cGcMissionSequenceWaitForStatSeasonal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mStat", ctypes.c_ulonglong),
        ("mStatGroup", ctypes.c_ulonglong),
        ("miAmount", ctypes.c_int32),
        ("mbTakeAmountFromSeasonalData", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mStat: int
    mStatGroup: int
    miAmount: int
    mbTakeAmountFromSeasonalData: bool
    macDebugText: bytes

class cGcMissionSequenceWaitForSuitUpgrade(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macDebugText: bytes

class cGcMissionSequenceWaitForWarps(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miAmount", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    miAmount: int
    macDebugText: bytes

class cGcMissionSequenceWaitRealTime(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mu64Time", ctypes.c_uint64),
        ("mfRandomness", ctypes.c_float),
        ("mDisplayStat", ctypes.c_ulonglong),
        ("mbStatFromNow", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mu64Time: int
    mfRandomness: float
    mDisplayStat: int
    mbStatFromNow: bool
    macDebugText: bytes

class cGcMissionSequenceWaitRealTimeCombat(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageCombat", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mu64Time", ctypes.c_uint64),
        ("mfRandomness", ctypes.c_float),
        ("mDisplayStat", ctypes.c_ulonglong),
        ("mbStatFromNow", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macMessageCombat: bytes
    mu64Time: int
    mfRandomness: float
    mDisplayStat: int
    mbStatFromNow: bool
    macDebugText: bytes

class cGcNPCAnimationList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCProbabilityAnimationData>
    ]
    maAnimations: bytes

class cGcNPCProbabilityReactionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mfProbability", ctypes.c_float),
        ("maRaceModifiers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCRaceProbabilityModifierData>
    ]
    mName: int
    mfProbability: float
    maRaceModifiers: bytes

class cGcConstructionPart(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mPart", ctypes.c_ulonglong),
        ("miAmount", ctypes.c_int32),
    ]
    mPart: int
    miAmount: int

class cGcJudgementMessageOptions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessageInSettlement", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageInSettlementSystem", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageOutOfSettlementSystem", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessageInSettlement: bytes
    macMessageInSettlementSystem: bytes
    macMessageOutOfSettlementSystem: bytes

class cGcNPCReactionEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mEmote", ctypes.c_ulonglong),
        ("mfReactionChance", ctypes.c_float),
        ("maAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCProbabilityReactionData>
    ]
    mEmote: int
    mfReactionChance: float
    maAnimations: bytes

class cGcNPCReactionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maReactions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCReactionEntry>
    ]
    maReactions: bytes

class cGcNPCAnimationSetData(ctypes.Structure):
    def tilde_cGcNPCAnimationSetData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maIdleAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCProbabilityAnimationData>
        ("maIdleFlavourAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCProbabilityAnimationData>
        ("maChatterAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCProbabilityAnimationData>
        ("maListenAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCProbabilityAnimationData>
        ("maGreetAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCProbabilityAnimationData>
        ("maMoodAnims", ctypes.c_char * 0x90),  # cTkFixedArray<cGcNPCAnimationList,9>
        ("maMoodLoops", ctypes.c_char * 0x90),  # cTkFixedArray<TkID<128>,9>
    ]
    maIdleAnimations: bytes
    maIdleFlavourAnimations: bytes
    maChatterAnimations: bytes
    maListenAnimations: bytes
    maGreetAnimations: bytes
    maMoodAnims: bytes
    maMoodLoops: bytes

class cGcNPCSettlementBehaviourEntry(ctypes.Structure):
    def tilde_cGcNPCSettlementBehaviourEntry(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfRunWhenOutdoorsProbability", ctypes.c_float),
        ("mbOnlyUseIndoorPOIs", ctypes.c_ubyte),
        ("maBuildingClassCapacities", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCSettlementBehaviourBuildingClassCapacityEntry>
        ("maAreaPropertyWeights", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCSettlementBehaviourAreaPropertyWeightEntry>
        ("maBuildingClassWeights", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCSettlementBehaviourBuildingClassWeightEntry>
        ("maObjectTypeWeights", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCSettlementBehaviourObjectTypeWeightEntry>
    ]
    mfRunWhenOutdoorsProbability: float
    mbOnlyUseIndoorPOIs: bool
    maBuildingClassCapacities: bytes
    maAreaPropertyWeights: bytes
    maBuildingClassWeights: bytes
    maObjectTypeWeights: bytes

class cGcAreaDamageData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mfRadius", ctypes.c_float),
        ("mfDelayPerMetre", ctypes.c_float),
        ("mPlayerDamageId", ctypes.c_ulonglong),
        ("mfDamage", ctypes.c_float),
        ("mbInstantKill", ctypes.c_ubyte),
        ("mbDamagePlayers", ctypes.c_ubyte),
        ("mbDamageCreatures", ctypes.c_ubyte),
    ]
    mId: int
    mfRadius: float
    mfDelayPerMetre: float
    mPlayerDamageId: int
    mfDamage: float
    mbInstantKill: bool
    mbDamagePlayers: bool
    mbDamageCreatures: bool

class cGcAreaDamageDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcAreaDamageData>
    ]
    maTable: bytes

class cGcAlienPuzzleMissionOverride(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("mOptionalMissionSeed", common.GcSeed),
        ("mPuzzle", ctypes.c_ulonglong),
        ("mbForceMissionSeed", ctypes.c_ubyte),
        ("mAltPriorityMissionForSelection", ctypes.c_ulonglong),
    ]
    mMission: int
    mOptionalMissionSeed: common.GcSeed
    mPuzzle: int
    mbForceMissionSeed: bool
    mAltPriorityMissionForSelection: int

class cGcNPCColourGroup(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfRarity", ctypes.c_float),
        ("mPrimary", common.Colour),
        ("maSecondary", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkColour>
    ]
    mfRarity: float
    mPrimary: common.Colour
    maSecondary: bytes

class cGcNPCColourTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCColourGroup>
    ]
    maGroups: bytes

class cGcPlayerEmoteList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maEmotes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerEmote>
    ]
    maEmotes: bytes

class cGcPlayerHazardTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x210),  # cTkFixedArray<cGcPlayerHazardData,6>
    ]
    maTable: bytes

class cGcPlayerStickData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfAcceleratorStickPoint", ctypes.c_float),
        ("mfAcceleratorMinTime", ctypes.c_float),
        ("mfAccelerate", ctypes.c_float),
        ("mfAccelerateAngle", ctypes.c_float),
        ("mfTurn", ctypes.c_float),
        ("mfTurnFast", ctypes.c_float),
        ("mfStickyFactor", ctypes.c_float),
    ]
    mfAcceleratorStickPoint: float
    mfAcceleratorMinTime: float
    mfAccelerate: float
    mfAccelerateAngle: float
    mfTurn: float
    mfTurnFast: float
    mfStickyFactor: float

class cGcDiscoveryOwner(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macLocalID", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macOnlineID", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macUsername", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macPlatform", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("miTimestamp", ctypes.c_int32),
    ]
    macLocalID: bytes
    macOnlineID: bytes
    macUsername: bytes
    macPlatform: bytes
    miTimestamp: int

class cGcPetCustomisationData(ctypes.Structure):
    def cGcPetCustomisationData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maData", ctypes.c_char * 0x108),  # cTkFixedArray<cGcCharacterCustomisationSaveData,3>
    ]
    maData: bytes

class cGcCreatureDiscoveryThumbnailOverride(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mContainsDescriptor", ctypes.c_ulonglong),
        ("mfDiscoveryUIScaler", ctypes.c_float),
        ("mDiscoveryUIOffset", common.Vector3f),
    ]
    mContainsDescriptor: int
    mfDiscoveryUIScaler: float
    mDiscoveryUIOffset: common.Vector3f

class cGcNPCInteractionsDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maNPCInteractions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCInteractionData>
    ]
    maNPCInteractions: bytes

class cGcExplosionDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcExplosionData>
    ]
    macName: bytes
    maTable: bytes

class cGcNPCProbabilityWordReactionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mNextInteraction", ctypes.c_ulonglong),
        ("mfProbability", ctypes.c_float),
    ]
    mNextInteraction: int
    mfProbability: float

class cGcNPCPropTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maProps", ctypes.c_char * 0xF70),  # cTkFixedArray<cGcNPCPropInfo,13>
    ]
    maProps: bytes

class cGcNPCSpawnTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacNPCModelNames", ctypes.c_char * 0x400),  # cTkFixedArray<cTkFixedString<128,char>,8>
        ("mafNPCRaceScale", ctypes.c_char * 0x20),  # cTkFixedArray<float,8>
        ("maUniqueNPCs", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcUniqueNPCSpawnData>
        ("maPlacementInfos", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCPlacementInfo>
    ]
    maacNPCModelNames: bytes
    mafNPCRaceScale: bytes
    maUniqueNPCs: bytes
    maPlacementInfos: bytes

class cGcReplacementEffectData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mReplaceWith", ctypes.c_ulonglong),
    ]
    mId: int
    mReplaceWith: int

class cGcPlayerCharacterIKOverrideData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
        ("mfStrength", ctypes.c_float),
        ("mRotStrengths", common.Vector3f),
    ]
    mbEnabled: bool
    mfStrength: float
    mRotStrengths: common.Vector3f

class cGcNPCWordReactionList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maReactions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNPCProbabilityWordReactionData>
    ]
    maReactions: bytes

class cGcNPCWordReactionTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maRaces", ctypes.c_char * 0x400),  # cTkFixedArray<cGcNPCWordReactionCategory,8>
    ]
    maRaces: bytes

class cGcPlayerCharacterStateData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mLocomotion0H", ctypes.c_ulonglong),
        ("mLocomotion1H", ctypes.c_ulonglong),
        ("mLocomotion2H", ctypes.c_ulonglong),
        ("mAimTree1HPitch", ctypes.c_ulonglong),
        ("mAimTree1HYaw", ctypes.c_ulonglong),
        ("mAimTree2HPitch", ctypes.c_ulonglong),
        ("mAimTree2HYaw", ctypes.c_ulonglong),
        ("mHitReact0H", ctypes.c_ulonglong),
        ("mHitReact1H", ctypes.c_ulonglong),
        ("mHitReact2H", ctypes.c_ulonglong),
        ("mbKeepHeadForward", ctypes.c_ubyte),
    ]
    mLocomotion0H: int
    mLocomotion1H: int
    mLocomotion2H: int
    mAimTree1HPitch: int
    mAimTree1HYaw: int
    mAimTree2HPitch: int
    mAimTree2HYaw: int
    mHitReact0H: int
    mHitReact1H: int
    mHitReact2H: int
    mbKeepHeadForward: bool

class cGcPlayerCharacterStateTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maCharacterStates", ctypes.c_char * 0xBD0),  # cTkFixedArray<cGcPlayerCharacterStateData,18>
    ]
    maCharacterStates: bytes

class cGcMissionSequenceKillEncounter(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mEncounterComponentScanEvent", ctypes.c_ulonglong),
        ("mbAllowedToEscape", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mEncounterComponentScanEvent: int
    mbAllowedToEscape: bool
    macDebugText: bytes

class cGcMissionSequenceLearnWords(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miAmount", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    miAmount: int
    macDebugText: bytes

class cGcMissionSequenceLeaveNexusMP(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageNoWarp", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mu64Timeout", ctypes.c_uint64),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macMessageNoWarp: bytes
    mu64Timeout: int
    macDebugText: bytes

class cGcMissionSequencePirates(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbScanCargo", ctypes.c_ubyte),
        ("miNumSquads", ctypes.c_int32),
        ("mAttackDefinition", ctypes.c_ulonglong),
        ("mRewardMessageOverride", ctypes.c_ulonglong),
        ("mbSilent", ctypes.c_ubyte),
        ("mfDistanceOverride", ctypes.c_float),
        ("mbForceSpawn", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mbScanCargo: bool
    miNumSquads: int
    mAttackDefinition: int
    mRewardMessageOverride: int
    mbSilent: bool
    mfDistanceOverride: float
    mbForceSpawn: bool
    macDebugText: bytes

class cGcMissionSequenceProductAmountNeeded(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mItem", ctypes.c_ulonglong),
        ("mToBuild", ctypes.c_ulonglong),
        ("mbIsRepair", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mItem: int
    mToBuild: int
    mbIsRepair: bool
    macDebugText: bytes

class cGcMissionSequenceRepairTech(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maTechsToRepair", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    maTechsToRepair: bytes
    macDebugText: bytes

class cGcMissionSequenceScan(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfWaitTime", ctypes.c_float),
        ("mbBlockTimedScans", ctypes.c_ubyte),
        ("mbRequiresMissionActive", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mfWaitTime: float
    mbBlockTimedScans: bool
    mbRequiresMissionActive: bool
    macDebugText: bytes

class cGcMissionSequenceSendChatMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mStatusMessageId", ctypes.c_ulonglong),
        ("mCustomText", ctypes.c_ulonglong),
    ]
    mStatusMessageId: int
    mCustomText: int

class cGcMissionSequenceSetCurrentMission(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMissionID", ctypes.c_ulonglong),
        ("mbSilent", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mMissionID: int
    mbSilent: bool
    macDebugText: bytes

class cGcMissionSequenceShowPodMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macDebugText: bytes

class cGcMissionSequenceShowSeasonTimeWarning(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfTimeToShow", ctypes.c_float),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mfTimeToShow: float
    macDebugText: bytes

class cGcMissionSequenceStartMission(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mMissionID", ctypes.c_ulonglong),
        ("mbMakeCurrent", ctypes.c_ubyte),
        ("mbRestart", ctypes.c_ubyte),
        ("mbForced", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mMissionID: int
    mbMakeCurrent: bool
    mbRestart: bool
    mbForced: bool
    macDebugText: bytes

class cGcMissionSequenceStop(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macDebugText: bytes

class cGcMissionSequenceSummonNexus(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macDebugText: bytes

class cGcMissionSequenceSuppressMarkers(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbSuppressed", ctypes.c_ubyte),
        ("mbSuppressedAfterNextWarp", ctypes.c_ubyte),
    ]
    mbSuppressed: bool
    mbSuppressedAfterNextWarp: bool

class cGcMissionSequenceVehicleScan(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mScanEventID", ctypes.c_ulonglong),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mScanEventID: int
    macDebugText: bytes

class cGcMissionSequenceVisitPlanets(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maPlanetTypesToWatch", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBiomeType>
        ("mbMustAlsoDiscover", ctypes.c_ubyte),
        ("miNumberOfEachToDiscover", ctypes.c_int32),
        ("mbTakeNumberFromSeasonalData", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    maPlanetTypesToWatch: bytes
    mbMustAlsoDiscover: bool
    miNumberOfEachToDiscover: int
    mbTakeNumberFromSeasonalData: bool
    macDebugText: bytes

class cGcMissionSequenceEndScanEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mEvent", ctypes.c_ulonglong),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mEvent: int
    macDebugText: bytes

class cGcMissionSequenceWait(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfTime", ctypes.c_float),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbSuppressMessages", ctypes.c_ubyte),
        ("mbMultiplyTimeBySeasonValue", ctypes.c_ubyte),
    ]
    mfTime: float
    macDebugText: bytes
    mbSuppressMessages: bool
    mbMultiplyTimeBySeasonValue: bool

class cGcMissionSequenceEnsureBarrelsAtPlayerSettlement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miNumBarrels", ctypes.c_int32),
        ("miMinBarrelsThreshold", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    miNumBarrels: int
    miMinBarrelsThreshold: int
    macDebugText: bytes

class cGcMissionSequenceWaitForAbandFreighterDoorOpen(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageOvertime", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfMinTime", ctypes.c_float),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macMessageOvertime: bytes
    mfMinTime: float
    macDebugText: bytes

class cGcMissionSequenceExplorationLogSpecial(ctypes.Structure):
    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macDebugText: bytes

class cGcMissionSequenceWaitForBuild(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mTargetTech", ctypes.c_ulonglong),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mTargetTech: int
    macDebugText: bytes

class cGcMissionSequenceExploreAbandonedFreighter(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbRequireAllRoomsDone", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miTimer", ctypes.c_int32),
    ]
    macMessage: bytes
    mbRequireAllRoomsDone: bool
    macDebugText: bytes
    miTimer: int

class cGcMissionSequenceWaitForCompletionMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macDebugText: bytes

class cGcMissionSequenceFeed(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbRequireSpecificBait", ctypes.c_ubyte),
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mbRequireSpecificBait: bool
    miAmountMin: int
    miAmountMax: int
    macDebugText: bytes

class cGcMissionSequenceFinishSummonAnomaly(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macDebugText: bytes

class cGcMissionSequenceFreighterDefend(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macDebugText: bytes

class cGcMissionSequenceWaitForDepots(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    miAmountMin: int
    miAmountMax: int
    macDebugText: bytes

class cGcMissionSequenceFreighterEngage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessageGetToSpace", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageEngage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfEngageDistance", ctypes.c_float),
        ("mfEngageTime", ctypes.c_float),
        ("mfTimeAfterWarp", ctypes.c_float),
        ("mTimeoutMessage", ctypes.c_ulonglong),
        ("macTimeoutOSDMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessageGetToSpace: bytes
    macMessageEngage: bytes
    mfEngageDistance: float
    mfEngageTime: float
    mfTimeAfterWarp: float
    mTimeoutMessage: int
    macTimeoutOSDMessage: bytes
    macDebugText: bytes

class cGcMissionSequenceGatherForBuild(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mTargetTech", ctypes.c_ulonglong),
        ("mGatherResource", ctypes.c_ulonglong),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mTargetTech: int
    mGatherResource: int
    macDebugText: bytes

class cGcMissionSequenceGetInShip(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macDebugText: bytes

class cGcMissionSequenceGetToExpedition(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macGalaxyMapMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mEvent", ctypes.c_ulonglong),
        ("mfCompletionDistance", ctypes.c_float),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macGalaxyMapMessage: bytes
    mEvent: int
    mfCompletionDistance: float
    macDebugText: bytes

class cGcMissionSequenceGetUnits(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miAmount", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    miAmount: int
    macDebugText: bytes

class cGcMissionSequenceGetUnitsToBuyItem(ctypes.Structure):
    def SetDefaults(): pass

    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mItem", ctypes.c_ulonglong),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mItem: int
    macDebugText: bytes

class cGcObjectiveTextFormatOptions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbObjectivesCanBeFormattedBySequences", ctypes.c_ubyte),
        ("mFormattableObjective", ctypes.c_ulonglong),
        ("mFormattableObjectiveTip", ctypes.c_ulonglong),
    ]
    mbObjectivesCanBeFormattedBySequences: bool
    mFormattableObjective: int
    mFormattableObjectiveTip: int

class cGcTargetMissionSurveyOptions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTargetMissionSurveyId", ctypes.c_ulonglong),
        ("mbTargetMissionSurveyDefinitelyExists", ctypes.c_ubyte),
        ("macSurveyInactiveHint", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macSurveySwapHint", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macSurveyHint", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macSurveyVehicleHint", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mTargetMissionSurveyId: int
    mbTargetMissionSurveyDefinitelyExists: bool
    macSurveyInactiveHint: bytes
    macSurveySwapHint: bytes
    macSurveyHint: bytes
    macSurveyVehicleHint: bytes

class cGcCustomNotifyTimerOptions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbHasCustomNotifyTimer", ctypes.c_ubyte),
        ("mfNotifyDisplayTime", ctypes.c_float),
        ("mfNotifyPauseTime", ctypes.c_float),
    ]
    mbHasCustomNotifyTimer: bool
    mfNotifyDisplayTime: float
    mfNotifyPauseTime: float

class cGcGenericMissionVersionProgress(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miVersion", ctypes.c_int32),
        ("miProgress", ctypes.c_int32),
    ]
    miVersion: int
    miProgress: int

class cGcMissionSequenceWaitForFreighterPods(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miAmount", ctypes.c_int32),
        ("mbTakeAmountFromPulseEncounter", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    miAmount: int
    mbTakeAmountFromPulseEncounter: bool
    macDebugText: bytes

class cGcMissionSequenceWaitForFriendlyDroneScanEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessageSummoned", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageUnsummoned", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageCantSummon", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageNotAvailable", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mEvent", ctypes.c_ulonglong),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessageSummoned: bytes
    macMessageUnsummoned: bytes
    macMessageCantSummon: bytes
    macMessageNotAvailable: bytes
    mEvent: int
    macDebugText: bytes

class cGcMissionSequenceWaitForMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mWaitMessageID", ctypes.c_ulonglong),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    mWaitMessageID: int
    macDebugText: bytes

class cGcMissionSequenceWaitForPhoto(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageSecondary", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageSuccess", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maBiomes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBiomeType>
        ("maFauna", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPhotoFauna>
        ("maFlora", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPhotoFlora>
        ("maBuildings", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPhotoBuildings>
        ("mbTakeAmountFromSeasonData", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macMessageSecondary: bytes
    macMessageSuccess: bytes
    maBiomes: bytes
    maFauna: bytes
    maFlora: bytes
    maBuildings: bytes
    mbTakeAmountFromSeasonData: bool
    macDebugText: bytes

class cGcMissionSequenceWaitForPortalWarp(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbWarpToSpace", ctypes.c_ubyte),
        ("macOverrideUA", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
    ]
    macMessage: bytes
    macDebugText: bytes
    mbWarpToSpace: bool
    macOverrideUA: bytes

class cGcMissionConsequenceRemoveCommunicatorMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mComms", ctypes.c_ulonglong),
    ]
    mComms: int

class cGcMissionConsequenceRemoveCommunicatorTakeOffMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mComms", ctypes.c_ulonglong),
    ]
    mComms: int

class cGcMissionConsequenceRemoveScanEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mEvent", ctypes.c_ulonglong),
    ]
    mEvent: int

class cGcMissionConsequenceResetPulseEncounterOverride(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcDefaultMissionSubstance(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mSubstance", ctypes.c_ulonglong),
    ]
    mSubstance: int

class cGcDefaultMissionProduct(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mProduct", ctypes.c_ulonglong),
    ]
    mProduct: int

class cGcDefaultMissionItemsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maPrimarySubstances", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDefaultMissionSubstance>
        ("maSecondarySubstances", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDefaultMissionSubstance>
        ("maPrimaryProducts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDefaultMissionProduct>
        ("maSecondaryProducts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDefaultMissionProduct>
    ]
    maPrimarySubstances: bytes
    maSecondarySubstances: bytes
    maPrimaryProducts: bytes
    maSecondaryProducts: bytes

class cGcMissionCommunityMissionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mbShowTimeToDeadline", ctypes.c_ubyte),
    ]
    mbShowTimeToDeadline: bool

class cGcMissionCommunityData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maCommunityMissionsIDs", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maCommunityMissionsData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcMissionCommunityMissionData>
    ]
    maCommunityMissionsIDs: bytes
    maCommunityMissionsData: bytes

class cGcNumberedTextList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFormat", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("miCount", ctypes.c_int32),
    ]
    macFormat: bytes
    miCount: int

class cGcMissionIDEpochPair(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMissionID", ctypes.c_ulonglong),
        ("mu64RecurrenceDeadline", ctypes.c_uint64),
    ]
    mMissionID: int
    mu64RecurrenceDeadline: int

class cGcMissionSequenceCompleteMission(ctypes.Structure):
    def SetDefaults(): pass

    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("mbUseSeed", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mMission: int
    mbUseSeed: bool
    macDebugText: bytes

class cGcMissionSchedulesTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSchedules", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcMissionSchedulingData>
    ]
    maSchedules: bytes

class cGcMissionSequenceCompleteSeasonalMilestone(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcDailyRecurrence(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miRecurrenceMinute", ctypes.c_int32),
        ("miRecurrenceHour", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    miRecurrenceMinute: int
    miRecurrenceHour: int
    macDebugText: bytes

class cGcMonthlyRecurrence(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miRecurrenceMinute", ctypes.c_int32),
        ("miRecurrenceHour", ctypes.c_int32),
        ("miRecurrenceDay", ctypes.c_int32),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    miRecurrenceMinute: int
    miRecurrenceHour: int
    miRecurrenceDay: int
    macDebugText: bytes

class cGcMissionConsequenceClearDetailMessages(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcAlienPuzzleTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcAlienPuzzleEntry>
    ]
    maTable: bytes

class cGcMissionSequenceBounty(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessageGetToSpace", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageEngage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageDestroy", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mBounty", ctypes.c_ulonglong),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessageGetToSpace: bytes
    macMessageEngage: bytes
    macMessageDestroy: bytes
    mBounty: int
    macDebugText: bytes

class cGcMissionSequenceBroadcastMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mMessageID", ctypes.c_ulonglong),
        ("mbSeeded", ctypes.c_ubyte),
        ("mbSendToAllMatchingSeeds", ctypes.c_ubyte),
        ("mbMultiplayer", ctypes.c_ubyte),
        ("mbBroadcastToActiveMultiplayerMission", ctypes.c_ubyte),
    ]
    macDebugText: bytes
    mMessageID: int
    mbSeeded: bool
    mbSendToAllMatchingSeeds: bool
    mbMultiplayer: bool
    mbBroadcastToActiveMultiplayerMission: bool

class cGcMissionSequenceConstructSettlementBuildingWithScanEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageWhileBuilding", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageWithItemsGathered", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageWhenDistant", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mScanEvent", ctypes.c_ulonglong),
        ("mfForceCompleteSequenceAtStagePercentage", ctypes.c_float),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macMessageWhileBuilding: bytes
    macMessageWithItemsGathered: bytes
    macMessageWhenDistant: bytes
    mScanEvent: int
    mfForceCompleteSequenceAtStagePercentage: float
    macDebugText: bytes

class cGcMissionSequenceCreateSpecificPulseEncounter(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageNoShip", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageNotPulsing", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macMessageSignalBlocked", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfMinTimeInPulse", ctypes.c_float),
        ("mPulseEncounterID", ctypes.c_ulonglong),
        ("mbAllowAnyEncounter", ctypes.c_ubyte),
        ("mbSilent", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    macMessageNoShip: bytes
    macMessageNotPulsing: bytes
    macMessageSignalBlocked: bytes
    mfMinTimeInPulse: float
    mPulseEncounterID: int
    mbAllowAnyEncounter: bool
    mbSilent: bool
    macDebugText: bytes

class cGcMissionSequenceClearInventoryHistory(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionSequenceCollectMultiProducts(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maProducts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcProductToCollect>
        ("mbWaitForSelected", ctypes.c_ubyte),
        ("mbSearchCookingIngredients", ctypes.c_ubyte),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macMessage: bytes
    maProducts: bytes
    mbWaitForSelected: bool
    mbSearchCookingIngredients: bool
    macDebugText: bytes

class cGcMissionSequenceDisplaySeasonRewardReminder(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfTime", ctypes.c_float),
        ("macDebugText", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mfTime: float
    macDebugText: bytes

class cGcScanEventTriggers(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfRange", ctypes.c_float),
        ("maTriggers", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("mbAllowRetrigger", ctypes.c_ubyte),
    ]
    mfRange: float
    maTriggers: bytes
    mbAllowRetrigger: bool

class cGcTradeData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maAlwaysPresentProducts", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maAlwaysPresentSubstances", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maOptionalProducts", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maOptionalSubstances", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("miMinItemsForSale", ctypes.c_int32),
        ("miMaxItemsForSale", ctypes.c_int32),
        ("mfPercentageOfItemsAreProducts", ctypes.c_float),
        ("mfBuyPriceIncreaseRedThreshold", ctypes.c_float),
        ("mfBuyPriceDecreaseGreenThreshold", ctypes.c_float),
        ("mfSellPriceIncreaseGreenThreshold", ctypes.c_float),
        ("mfSellPriceDecreaseRedThreshold", ctypes.c_float),
        ("mbShowSeasonRewards", ctypes.c_ubyte),
        ("maiMinAmountOfProductAvailable", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("maiMaxAmountOfProductAvailable", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("maiMinAmountOfSubstanceAvailable", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("maiMaxAmountOfSubstanceAvailable", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("maiMinExtraSystemProducts", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("maiMaxExtraSystemProducts", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("mafTradeProductsPriceImprovements", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
    ]
    maAlwaysPresentProducts: bytes
    maAlwaysPresentSubstances: bytes
    maOptionalProducts: bytes
    maOptionalSubstances: bytes
    miMinItemsForSale: int
    miMaxItemsForSale: int
    mfPercentageOfItemsAreProducts: float
    mfBuyPriceIncreaseRedThreshold: float
    mfBuyPriceDecreaseGreenThreshold: float
    mfSellPriceIncreaseGreenThreshold: float
    mfSellPriceDecreaseRedThreshold: float
    mbShowSeasonRewards: bool
    maiMinAmountOfProductAvailable: bytes
    maiMaxAmountOfProductAvailable: bytes
    maiMinAmountOfSubstanceAvailable: bytes
    maiMaxAmountOfSubstanceAvailable: bytes
    maiMinExtraSystemProducts: bytes
    maiMaxExtraSystemProducts: bytes
    mafTradeProductsPriceImprovements: bytes

class cGcMissionConditionProductKnown(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mProduct", ctypes.c_ulonglong),
        ("mbDependentOnSeasonMilestone", ctypes.c_ubyte),
        ("mbTakeProductFromSeasonData", ctypes.c_ubyte),
    ]
    mProduct: int
    mbDependentOnSeasonMilestone: bool
    mbTakeProductFromSeasonData: bool

class cGcMissionConditionPulseEncounterOverriden(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionReadyToSpawnPirates(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionRefinerActive(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbHasFuel", ctypes.c_ubyte),
        ("mActiveRecipe", ctypes.c_ulonglong),
        ("miAmountToMake", ctypes.c_int32),
    ]
    mbHasFuel: bool
    mActiveRecipe: int
    miAmountToMake: int

class cGcMissionConditionRefinerHasOutput(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mOutputProduct", ctypes.c_ulonglong),
        ("miOutputAmount", ctypes.c_int32),
    ]
    mOutputProduct: int
    miOutputAmount: int

class cGcMissionConditionRidingCreature(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionSeasonNumber(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miActiveSeason", ctypes.c_int32),
        ("mbIncludeRemix", ctypes.c_ubyte),
    ]
    miActiveSeason: int
    mbIncludeRemix: bool

class cGcMissionConditionSeasonRewardsRecipe(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mRecipeID", ctypes.c_ulonglong),
    ]
    mRecipeID: int

class cGcMissionConditionSeasonRewardUnlocked(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mSpecialID", ctypes.c_ulonglong),
    ]
    mSpecialID: int

class cGcMissionConditionSentinelsDisabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionSettlementBuildingsAllStarted(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionSettlementsEnabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionShieldDown(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbRequireOnFoot", ctypes.c_ubyte),
    ]
    mbRequireOnFoot: bool

class cGcMissionConditionSpaceCombatEnabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
    ]
    mbEnabled: bool

class cGcMissionConditionSpecialKnown(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mSpecialID", ctypes.c_ulonglong),
    ]
    mSpecialID: int

class cGcMissionConditionStartWithAllPartsKnown(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionStatChange(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mStat", ctypes.c_ulonglong),
        ("mStatGroup", ctypes.c_ulonglong),
    ]
    mStat: int
    mStatGroup: int

class cGcMissionConditionSystemHasInfestedPlanet(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionTakingDamage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbRequireShieldDown", ctypes.c_ubyte),
        ("mDamageID", ctypes.c_ulonglong),
    ]
    mbRequireShieldDown: bool
    mDamageID: int

class cGcMissionConditionTechnologyKnown(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTechnology", ctypes.c_ulonglong),
        ("mbDependentOnSeasonMilestone", ctypes.c_ubyte),
        ("mbTakeTechFromSeasonData", ctypes.c_ubyte),
    ]
    mTechnology: int
    mbDependentOnSeasonMilestone: bool
    mbTakeTechFromSeasonData: bool

class cGcMissionConditionTotalWarpsNumber(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miWarpsNumber", ctypes.c_int32),
    ]
    miWarpsNumber: int

class cGcMissionConditionTutorialEnabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
    ]
    mbEnabled: bool

class cGcMissionConditionUnclaimedStageReward(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionUnderwaterDepth(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDepth", ctypes.c_float),
        ("mbTakeAmountFromSeasonData", ctypes.c_ubyte),
    ]
    mfDepth: float
    mbTakeAmountFromSeasonData: bool

class cGcMissionConditionMissionCompleted(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMissionID", ctypes.c_ulonglong),
        ("mbTakeIDFromSeasonData", ctypes.c_ubyte),
        ("mbCalculateTextMissionTargetFromStageIndex", ctypes.c_ubyte),
    ]
    mMissionID: int
    mbTakeIDFromSeasonData: bool
    mbCalculateTextMissionTargetFromStageIndex: bool

class cGcMissionConditionVisorActive(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionMissionMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mMessage", ctypes.c_ulonglong),
    ]
    mMessage: int

class cGcMissionTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maMissions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericMissionSequence>
    ]
    maMissions: bytes

class cGcMissionConditionMissionMessagePortal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionWaitForTime(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64WaitTimeInSeconds", ctypes.c_uint64),
        ("mbThisConditionWillSetMissionUserDataIsThatOk", ctypes.c_ubyte),
    ]
    mu64WaitTimeInSeconds: int
    mbThisConditionWillSetMissionUserDataIsThatOk: bool

class cGcMissionConditionMissionMessageWarp(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionMissionSelected(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mMissionID", ctypes.c_ulonglong),
    ]
    mMissionID: int

class cGcMissionConditionWarping(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionWaterInSystem(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionMultiplayerFreighterAvailable(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionWaterPlanet(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionWristMenuOpen(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbLeftHandOnly", ctypes.c_ubyte),
        ("mbQuickMenuOnly", ctypes.c_ubyte),
        ("mbGunHandOnly", ctypes.c_ubyte),
        ("mbInventoryOnly", ctypes.c_ubyte),
    ]
    mbLeftHandOnly: bool
    mbQuickMenuOnly: bool
    mbGunHandOnly: bool
    mbInventoryOnly: bool

class cGcMissionConditionNearScanEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mEvent", ctypes.c_ulonglong),
        ("mfDistance", ctypes.c_float),
        ("mbAllowInShip", ctypes.c_ubyte),
        ("mbRequiresFullFireteam", ctypes.c_ubyte),
        ("mbReturnTrueIfMarkerGone", ctypes.c_ubyte),
    ]
    mEvent: int
    mfDistance: float
    mbAllowInShip: bool
    mbRequiresFullFireteam: bool
    mbReturnTrueIfMarkerGone: bool

class cGcMissionConditionNexusEnabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionOnFootCombatEnabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
    ]
    mbEnabled: bool

class cGcMissionConditionOnMultiplayerMission(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionOnOtherSideOfPortal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbTestForRegularPortal", ctypes.c_ubyte),
        ("mbTestForStoryPortal", ctypes.c_ubyte),
    ]
    mbTestForRegularPortal: bool
    mbTestForStoryPortal: bool

class cGcMissionConditionOnPlanetWithSandwormsOverriden(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbAllowInShip", ctypes.c_ubyte),
        ("mbAcceptMatchingSystem", ctypes.c_ubyte),
    ]
    mbAllowInShip: bool
    mbAcceptMatchingSystem: bool

class cGcMissionConditionPadActive(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionPercentageChance(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miPercent", ctypes.c_int32),
        ("mbSeeded", ctypes.c_ubyte),
        ("mbOverrideZeroSeed", ctypes.c_ubyte),
        ("mbOverrideMissionSeedWithRandomSeed", ctypes.c_ubyte),
    ]
    miPercent: int
    mbSeeded: bool
    mbOverrideZeroSeed: bool
    mbOverrideMissionSeedWithRandomSeed: bool

class cGcMissionConditionPirateSystem(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionPlanetAttackPiratesActive(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionPlanetResourceHint(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mResourceHint", ctypes.c_ulonglong),
    ]
    mResourceHint: int

class cGcMissionConditionHasPendingSettlementJudgement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mSpecificID", ctypes.c_ulonglong),
    ]
    mSpecificID: int

class cGcMissionConditionHasPlatformReward(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionInventoryOpen(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionInVR(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbNeedsHandControllers", ctypes.c_ubyte),
        ("mbNeedsSnapTurnOn", ctypes.c_ubyte),
        ("mbNeedsNoHandControllers", ctypes.c_ubyte),
        ("mbNeedsTeleportOn", ctypes.c_ubyte),
        ("mbNeedsSmoothMoveOn", ctypes.c_ubyte),
    ]
    mbNeedsHandControllers: bool
    mbNeedsSnapTurnOn: bool
    mbNeedsNoHandControllers: bool
    mbNeedsTeleportOn: bool
    mbNeedsSmoothMoveOn: bool

class cGcMissionConditionIsCurrentMission(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionIsDepotDestroyed(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mControllingScanEvent", ctypes.c_ulonglong),
    ]
    mControllingScanEvent: int

class cGcMissionConditionIsGrabbed(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionIsLookingAtAnomaly(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfFOV", ctypes.c_float),
        ("mfMaxDistance", ctypes.c_float),
    ]
    mfFOV: float
    mfMaxDistance: float

class cGcMissionConditionIsMissionInProgress(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMissionID", ctypes.c_ulonglong),
        ("mbMustBeSelectedMission", ctypes.c_ubyte),
    ]
    mMissionID: int
    mbMustBeSelectedMission: bool

class cGcMissionConditionHasProcTechnology(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mProcTechGroupID", ctypes.c_ulonglong),
        ("miCount", ctypes.c_int32),
    ]
    mProcTechGroupID: int
    miCount: int

class cGcMissionConditionIsPartyPlanetUnlocked(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miSpecificRendevousPlanetIndex", ctypes.c_int32),
        ("mbTakeIndexFromSeasonData", ctypes.c_ubyte),
        ("mbTakeIndexFromMilestoneStage", ctypes.c_ubyte),
    ]
    miSpecificRendevousPlanetIndex: int
    mbTakeIndexFromSeasonData: bool
    mbTakeIndexFromMilestoneStage: bool

class cGcMissionConditionHasSeasonalReward(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionHasSettlement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionIsScanEventActive(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mEvent", ctypes.c_ulonglong),
        ("mbMustMatchThisMissionSeed", ctypes.c_ubyte),
    ]
    mEvent: int
    mbMustMatchThisMissionSeed: bool

class cGcMissionConditionIsScanEventLocal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mEvent", ctypes.c_ulonglong),
        ("mbRequiresFullFireteam", ctypes.c_ubyte),
    ]
    mEvent: int
    mbRequiresFullFireteam: bool

class cGcMissionConditionHasSettlementLocal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionHasSettlementProductPending(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionIsScanEventOnCurrentPlanet(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mEvent", ctypes.c_ulonglong),
        ("mbAllowInShip", ctypes.c_ubyte),
    ]
    mEvent: int
    mbAllowInShip: bool

class cGcMissionConditionIsScanEventRepaired(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mEvent", ctypes.c_ulonglong),
        ("mbCheckForAllRepairsInBuilding", ctypes.c_ubyte),
    ]
    mEvent: int
    mbCheckForAllRepairsInBuilding: bool

class cGcMissionConditionHasSpareProcTech(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionIsTechnologyRepaired(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTechnology", ctypes.c_ulonglong),
        ("miRepairedComponents", ctypes.c_int32),
        ("mSpecificComponent", ctypes.c_ulonglong),
    ]
    mTechnology: int
    miRepairedComponents: int
    mSpecificComponent: int

class cGcMissionConditionHasTechnology(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTechnology", ctypes.c_ulonglong),
        ("mbAllowPartiallyInstalled", ctypes.c_ubyte),
        ("mbTeachIfNotKnown", ctypes.c_ubyte),
        ("mbTakeTechFromSeasonData", ctypes.c_ubyte),
    ]
    mTechnology: int
    mbAllowPartiallyInstalled: bool
    mbTeachIfNotKnown: bool
    mbTakeTechFromSeasonData: bool

class cGcMissionConditionHasTwitchReward(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionLifeSupportEnabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
    ]
    mbEnabled: bool

class cGcMissionConditionLocalScanActive(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionHazardsEnabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
    ]
    mbEnabled: bool

class cGcScanEventTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maEvents", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcScanEventData>
    ]
    maEvents: bytes

class cGcMissionConditionMessageBeaconsQuery(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miMinPartsFound", ctypes.c_int32),
        ("miMaxPartsFound", ctypes.c_int32),
        ("mfSearchDistanceLimit", ctypes.c_float),
    ]
    miMinPartsFound: int
    miMaxPartsFound: int
    mfSearchDistanceLimit: float

class cGcMissionConditionInMultiplayer(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbMustBeInFireteam", ctypes.c_ubyte),
    ]
    mbMustBeInFireteam: bool

class cGcMissionConditionInSeasonalUA(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miSpecificRendevousPlanetIndex", ctypes.c_int32),
        ("mbSpecificIndexOnlyNeedsToMatchSystem", ctypes.c_ubyte),
        ("mbTakeIndexFromSeasonData", ctypes.c_ubyte),
        ("mbTakeIndexFromMilestoneStage", ctypes.c_ubyte),
        ("mbCompleteIfRendezvousDone", ctypes.c_ubyte),
    ]
    miSpecificRendevousPlanetIndex: int
    mbSpecificIndexOnlyNeedsToMatchSystem: bool
    mbTakeIndexFromSeasonData: bool
    mbTakeIndexFromMilestoneStage: bool
    mbCompleteIfRendezvousDone: bool

class cGcMissionConditionInUA(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macUA", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
    ]
    macUA: bytes

class cGcModelExplosionRules(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maRules", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcModelExplosionRule>
        ("mabUseRules", ctypes.c_char * 0x9),  # cTkFixedArray<bool,9>
        ("mafShipSalvageDisplayScales", ctypes.c_char * 0x24),  # cTkFixedArray<float,9>
    ]
    maRules: bytes
    mabUseRules: bytes
    mafShipSalvageDisplayScales: bytes

class cGcPassiveFrigateIncomeArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maArray", ctypes.c_char * 0x100),  # cTkFixedArray<cGcPassiveFrigateIncome,8>
    ]
    maArray: bytes

class cGcMissionConditionBaseRequiresPower(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miMinNumPowerUsingParts", ctypes.c_int32),
    ]
    miMinNumPowerUsingParts: int

class cGcMissionConditionBuildMenuOpen(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionCameraControlStealing(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionCanReceiveReward(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mReward", ctypes.c_ulonglong),
    ]
    mReward: int

class cGcMissionConditionConvertedFromSeason(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miSeason", ctypes.c_int32),
    ]
    miSeason: int

class cGcMissionConditionCreatureReadyToHatch(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionCreatureReadyToLay(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbPrimaryCreatureOnly", ctypes.c_ubyte),
    ]
    mbPrimaryCreatureOnly: bool

class cGcMissionConditionCreatureSummoned(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionCriticalMissionsDone(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbWarped", ctypes.c_ubyte),
    ]
    mbWarped: bool

class cGcMissionConditionDamagedFrigateAtHome(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionEggMachinePageOpen(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionElevation(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfHeightAboveSea", ctypes.c_float),
        ("mbAllowInShip", ctypes.c_ubyte),
        ("mbTakeAmountFromSeasonData", ctypes.c_ubyte),
    ]
    mfHeightAboveSea: float
    mbAllowInShip: bool
    mbTakeAmountFromSeasonData: bool

class cGcMissionConditionExocraftMoving(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionExpeditionContainsReward(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mRewardID", ctypes.c_ulonglong),
    ]
    mRewardID: int

class cGcMissionConditionExpeditionNearlyOver(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64RemainingTimeToStartWarning", ctypes.c_uint64),
    ]
    mu64RemainingTimeToStartWarning: int

class cGcMissionConditionExpeditionProgress(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcQuestItemPlacementRule(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mQuestItemID", ctypes.c_ulonglong),
        ("miMinRoomIndex", ctypes.c_int32),
        ("miMaxRoomIndex", ctypes.c_int32),
        ("mMustBeBeforeQuestItem", ctypes.c_ulonglong),
        ("mMustBeAfterQuestItem", ctypes.c_ulonglong),
        ("maValidRoomIDs", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mQuestItemID: int
    miMinRoomIndex: int
    miMaxRoomIndex: int
    mMustBeBeforeQuestItem: int
    mMustBeAfterQuestItem: int
    maValidRoomIDs: bytes

class cGcRegionHotspotBiomeGases(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGas1Id", ctypes.c_ulonglong),
        ("mGas2Id", ctypes.c_ulonglong),
    ]
    mGas1Id: int
    mGas2Id: int

class cGcRegionHotspotData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfProbabilityWeighting", ctypes.c_float),
        ("mfMinRange", ctypes.c_float),
        ("mfMaxRange", ctypes.c_float),
        ("mafClassWeightings", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("mafClassStrengths", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("mfDiscoveryDistanceThreshold", ctypes.c_float),
    ]
    mfProbabilityWeighting: float
    mfMinRange: float
    mfMaxRange: float
    mafClassWeightings: bytes
    mafClassStrengths: bytes
    mfDiscoveryDistanceThreshold: float

class cGcRegionHotspotSubstance(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mSubstanceId", ctypes.c_ulonglong),
        ("miAmountCost", ctypes.c_int32),
        ("miSubstanceYeild", ctypes.c_int32),
    ]
    mSubstanceId: int
    miAmountCost: int
    miSubstanceYeild: int

class cGcMissionConditionAbandonedFreighterExplored(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miTargetRooms", ctypes.c_int32),
    ]
    miTargetRooms: int

class cGcMissionConditionAimingTeleporter(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionAlienPodAggroed(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfThreshold", ctypes.c_float),
    ]
    mfThreshold: float

class cGcMissionConditionAllMilestonesComplete(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miForStage", ctypes.c_int32),
        ("mbUseSeasonOverrideMessage", ctypes.c_ubyte),
    ]
    miForStage: int
    mbUseSeasonOverrideMessage: bool

class cGcMissionConditionAreDroneHivePartsDestroyed(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mControllingScanEvent", ctypes.c_ulonglong),
    ]
    mControllingScanEvent: int

class cGcMissionConditionAutoPowerEnabled(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
    ]
    mbEnabled: bool

class cGcMissionConditionBasePartNear(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mPartID", ctypes.c_ulonglong),
        ("mbTakeIDFromSeasonData", ctypes.c_ubyte),
        ("mfDistance", ctypes.c_float),
    ]
    mPartID: int
    mbTakeIDFromSeasonData: bool
    mfDistance: float

class cGcRegionHotspotsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfRegionHotspotsPoleSpacing", ctypes.c_float),
        ("mfRegionHotspotsPerPoleMin", ctypes.c_float),
        ("mfRegionHotspotsPerPoleMax", ctypes.c_float),
        ("mfRegionHotspotsMinSameCategorySpacing", ctypes.c_float),
        ("mfRegionHotspotsMaxDifferentCategoryOverlap", ctypes.c_float),
        ("maRegionHotspots", ctypes.c_char * 0x120),  # cTkFixedArray<cGcRegionHotspotData,6>
        ("maRegionHotspotBiomeGases", ctypes.c_char * 0x200),  # cTkFixedArray<cGcRegionHotspotBiomeGases,16>
        ("maRegionHotspotSubstances", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcRegionHotspotSubstance>
    ]
    mfRegionHotspotsPoleSpacing: float
    mfRegionHotspotsPerPoleMin: float
    mfRegionHotspotsPerPoleMax: float
    mfRegionHotspotsMinSameCategorySpacing: float
    mfRegionHotspotsMaxDifferentCategoryOverlap: float
    maRegionHotspots: bytes
    maRegionHotspotBiomeGases: bytes
    maRegionHotspotSubstances: bytes

class cGcRoomCountRule(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mRoomID", ctypes.c_ulonglong),
        ("miMin", ctypes.c_int32),
        ("miMax", ctypes.c_int32),
    ]
    mRoomID: int
    miMin: int
    miMax: int

class cGcRoomSequenceRule(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mRoomID", ctypes.c_ulonglong),
        ("miMinRoomIndex", ctypes.c_int32),
        ("mMustBeBeforeRoom", ctypes.c_ulonglong),
        ("mMustBeAfterRoom", ctypes.c_ulonglong),
    ]
    mRoomID: int
    miMinRoomIndex: int
    mMustBeBeforeRoom: int
    mMustBeAfterRoom: int

class cGcMissionConditionBasePowerGenerated(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miAmount", ctypes.c_int32),
        ("mbMustBeSpare", ctypes.c_ubyte),
        ("mbTakeAmountFromSeasonData", ctypes.c_ubyte),
    ]
    miAmount: int
    mbMustBeSpare: bool
    mbTakeAmountFromSeasonData: bool

class cGcModuleOverride(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mModule", ctypes.c_ulonglong),
        ("mfProbabilityMultiplier", ctypes.c_float),
        ("mfOriginalSceneProbability", ctypes.c_float),
        ("maScenes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcWeightedResource>
    ]
    mModule: int
    mfProbabilityMultiplier: float
    mfOriginalSceneProbability: float
    maScenes: bytes

class cGcExpeditionCategoryStrength(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maiOccurranceChance", ctypes.c_char * 0x14),  # cTkFixedArray<int,5>
    ]
    maiOccurranceChance: bytes

class cGcExpeditionEventOccurrenceRate(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maExpeditionCategory", ctypes.c_char * 0x64),  # cTkFixedArray<cGcExpeditionCategoryStrength,5>
    ]
    maExpeditionCategory: bytes

class cGcExpeditionEventTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maEvents", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcExpeditionEventData>
        ("maInterventionEvents", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcExpeditionInterventionEventData>
    ]
    maEvents: bytes
    maInterventionEvents: bytes

class cGcMinimumUseConstraint(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maModules", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("miMinUses", ctypes.c_int32),
        ("mGroup", ctypes.c_ulonglong),
    ]
    maModules: bytes
    miMinUses: int
    mGroup: int

class cGcWeightedBuildingSize(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfRelativeProbability", ctypes.c_float),
        ("miSizeX", ctypes.c_int32),
        ("miSizeY", ctypes.c_int32),
        ("miSizeZ", ctypes.c_int32),
        ("mbCreateSymmetricBuilding", ctypes.c_ubyte),
    ]
    mfRelativeProbability: float
    miSizeX: int
    miSizeY: int
    miSizeZ: int
    mbCreateSymmetricBuilding: bool

class cGcFreighterBaseRoom(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mPalette", ctypes.c_ulonglong),
    ]
    mName: int
    mPalette: int

class cGcExpeditionPaymentToken(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTokenName", ctypes.c_ulonglong),
        ("miTokenValue", ctypes.c_int32),
    ]
    mTokenName: int
    miTokenValue: int

class cGcFreighterBaseOption(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macBaseDataFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfProbabilityWeighting", ctypes.c_float),
    ]
    macBaseDataFile: bytes
    mfProbabilityWeighting: float

class cGcIDPair(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mItem1", ctypes.c_ulonglong),
        ("mItem2", ctypes.c_ulonglong),
    ]
    mItem1: int
    mItem2: int

class cGcWFCModuleSet(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mBlockSize", common.Vector3f),
        ("mbApplyWallThemes", ctypes.c_ubyte),
        ("maModulePrototypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcWFCModulePrototype>
        ("maacConnectorsOnUpperBoundary", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<32,char> >
        ("maacConnectorsOnLowerBoundary", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<32,char> >
        ("maacConnectorsOnHorizontalBoundary", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<32,char> >
        ("maDefaultGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maCompatibleConnectors", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcIDPair>
        ("mfVerticalOffset", ctypes.c_float),
    ]
    macName: bytes
    mBlockSize: common.Vector3f
    mbApplyWallThemes: bool
    maModulePrototypes: bytes
    maacConnectorsOnUpperBoundary: bytes
    maacConnectorsOnLowerBoundary: bytes
    maacConnectorsOnHorizontalBoundary: bytes
    maDefaultGroups: bytes
    maCompatibleConnectors: bytes
    mfVerticalOffset: float

class cGcFreighterBaseOptions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maFreighterBases", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcFreighterBaseOption>
    ]
    maFreighterBases: bytes

class cGcSelectableObjectData(ctypes.Structure):
    def WriteToXMLNodeExplicit(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macFilename: bytes

class cGcGalacticAddressData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miVoxelX", ctypes.c_int32),
        ("miVoxelY", ctypes.c_int32),
        ("miVoxelZ", ctypes.c_int32),
        ("miSolarSystemIndex", ctypes.c_int32),
        ("miPlanetIndex", ctypes.c_int32),
    ]
    miVoxelX: int
    miVoxelY: int
    miVoxelZ: int
    miSolarSystemIndex: int
    miPlanetIndex: int

class cGcAsteroidGenerationData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfScale", ctypes.c_float),
        ("mScaleVariance", common.Vector2f),
        ("miHealth", ctypes.c_int32),
        ("mfSpacing", ctypes.c_float),
        ("mNoiseRange", common.Vector2f),
        ("mfFadeRange", ctypes.c_float),
        ("mfNoiseScale", ctypes.c_float),
    ]
    mfScale: float
    mScaleVariance: common.Vector2f
    miHealth: int
    mfSpacing: float
    mNoiseRange: common.Vector2f
    mfFadeRange: float
    mfNoiseScale: float

class cGcGalaxyCameraData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfCameraFOV", ctypes.c_float),
        ("mfMovementBlendRateFree", ctypes.c_float),
        ("mfMovementBlendRateLocked", ctypes.c_float),
        ("mfMovementBlendRateLookLocked", ctypes.c_float),
        ("mfLockTransitionRate", ctypes.c_float),
        ("mfLockedSpinSpeed", ctypes.c_float),
        ("mfLockedScaledPushSpeed", ctypes.c_float),
        ("mfLockedScaledElevationSpeed", ctypes.c_float),
        ("mfFreePanSpeed", ctypes.c_float),
        ("mfFreePanSpeedTurbo", ctypes.c_float),
        ("mfFreeUpDownSpeed", ctypes.c_float),
        ("mfFreeUpDownSpeedTurbo", ctypes.c_float),
        ("mfFreeRotateSpeed", ctypes.c_float),
        ("mfFreeElevationBlendRate", ctypes.c_float),
        ("mfZoomOutSpin", ctypes.c_float),
        ("mfZoomOutElevation", ctypes.c_float),
        ("mfZoomOutPushDist", ctypes.c_float),
        ("mfZoomOutRate", ctypes.c_float),
        ("mfZoomInRate", ctypes.c_float),
        ("mfMinZoomDistance", ctypes.c_float),
        ("mfMaxZoomDistance", ctypes.c_float),
        ("mfMinPushingZoomDistance", ctypes.c_float),
        ("mfMinPushingZoomDistanceScaler", ctypes.c_float),
        ("mfCameraShakeDriftClip", ctypes.c_float),
        ("mfCameraShakeDriftShift", ctypes.c_float),
        ("mfCameraShakeSmoothingRate", ctypes.c_float),
        ("mfCameraShakeMaximum", ctypes.c_float),
    ]
    mfCameraFOV: float
    mfMovementBlendRateFree: float
    mfMovementBlendRateLocked: float
    mfMovementBlendRateLookLocked: float
    mfLockTransitionRate: float
    mfLockedSpinSpeed: float
    mfLockedScaledPushSpeed: float
    mfLockedScaledElevationSpeed: float
    mfFreePanSpeed: float
    mfFreePanSpeedTurbo: float
    mfFreeUpDownSpeed: float
    mfFreeUpDownSpeedTurbo: float
    mfFreeRotateSpeed: float
    mfFreeElevationBlendRate: float
    mfZoomOutSpin: float
    mfZoomOutElevation: float
    mfZoomOutPushDist: float
    mfZoomOutRate: float
    mfZoomInRate: float
    mfMinZoomDistance: float
    mfMaxZoomDistance: float
    mfMinPushingZoomDistance: float
    mfMinPushingZoomDistanceScaler: float
    mfCameraShakeDriftClip: float
    mfCameraShakeDriftShift: float
    mfCameraShakeSmoothingRate: float
    mfCameraShakeMaximum: float

class cGcGalaxyGenerationSetupData(ctypes.Structure):
    def cGcGalaxyGenerationSetupData(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def Interpolate(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfSpiralFormChance", ctypes.c_float),
        ("mSpiralInclusion", common.Vector2f),
        ("mSpiralFlex", common.Vector2f),
        ("mfSpiralTwistMult", ctypes.c_float),
        ("mSpiralPull", common.Vector3f),
        ("mSpiralSizeScale", common.Vector2f),
        ("mfRareSunChance", ctypes.c_float),
        ("mConnectionAttractorMax", common.Vector2f),
        ("mConnectionAttractorMin", common.Vector2f),
        ("mConnectionDistortion", common.Vector2f),
        ("mfConnectionDistortionTMult", ctypes.c_float),
        ("mfConnectionDistanceLimit", ctypes.c_float),
        ("mBaseSize", common.Vector2f),
        ("maStarSize", ctypes.c_char * 0x20),  # cTkFixedArray<cTkVector2,4>
        ("mfBaseTurbulenceScale", ctypes.c_float),
        ("mfBaseTurbulenceLac", ctypes.c_float),
        ("mfBaseTurbulenceGain", ctypes.c_float),
        ("mfBaseGenerationThreshold", ctypes.c_float),
        ("mfFieldGenerationThreshold", ctypes.c_float),
        ("mfStarGenerationThreshold", ctypes.c_float),
        ("mInnerFieldScales", common.Vector4f),
        ("mfSizeNoiseScale", ctypes.c_float),
        ("mfSizeNoisePower", ctypes.c_float),
        ("mfSizeField4Inf", ctypes.c_float),
        ("mfFieldAlphaBase", ctypes.c_float),
        ("mfFieldAlphaField1Inf", ctypes.c_float),
        ("mfFieldAlphaField2SqInf", ctypes.c_float),
        ("mfStarHighlightChance", ctypes.c_float),
        ("mStarHighlightAlpha", common.Vector2f),
        ("mStarHighlightSize", common.Vector2f),
        ("mfColourBaseBlendOnSize", ctypes.c_float),
        ("maInnerSectorColours", ctypes.c_char * 0xA0),  # cTkFixedArray<cTkColour,10>
    ]
    mfSpiralFormChance: float
    mSpiralInclusion: common.Vector2f
    mSpiralFlex: common.Vector2f
    mfSpiralTwistMult: float
    mSpiralPull: common.Vector3f
    mSpiralSizeScale: common.Vector2f
    mfRareSunChance: float
    mConnectionAttractorMax: common.Vector2f
    mConnectionAttractorMin: common.Vector2f
    mConnectionDistortion: common.Vector2f
    mfConnectionDistortionTMult: float
    mfConnectionDistanceLimit: float
    mBaseSize: common.Vector2f
    maStarSize: bytes
    mfBaseTurbulenceScale: float
    mfBaseTurbulenceLac: float
    mfBaseTurbulenceGain: float
    mfBaseGenerationThreshold: float
    mfFieldGenerationThreshold: float
    mfStarGenerationThreshold: float
    mInnerFieldScales: common.Vector4f
    mfSizeNoiseScale: float
    mfSizeNoisePower: float
    mfSizeField4Inf: float
    mfFieldAlphaBase: float
    mfFieldAlphaField1Inf: float
    mfFieldAlphaField2SqInf: float
    mfStarHighlightChance: float
    mStarHighlightAlpha: common.Vector2f
    mStarHighlightSize: common.Vector2f
    mfColourBaseBlendOnSize: float
    maInnerSectorColours: bytes

class cGcSelectableObjectTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maLists", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSelectableObjectList>
    ]
    maLists: bytes

class cGcHazardValues(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfNormal", ctypes.c_float),
        ("mfExtreme", ctypes.c_float),
    ]
    mfNormal: float
    mfExtreme: float

class cGcColourModifier(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbForceColour", ctypes.c_ubyte),
        ("mForceColourTo", common.Colour),
        ("mfOffsetSaturation", ctypes.c_float),
        ("mfOffsetValue", ctypes.c_float),
        ("mfMultiplySaturation", ctypes.c_float),
        ("mfMultiplyValue", ctypes.c_float),
    ]
    mbForceColour: bool
    mForceColourTo: common.Colour
    mfOffsetSaturation: float
    mfOffsetValue: float
    mfMultiplySaturation: float
    mfMultiplyValue: float

class cGcFreighterDungeonChoice(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mfWeighting", ctypes.c_float),
    ]
    mName: int
    mfWeighting: float

class cGcDungeonRoomParams(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mRoomId", ctypes.c_ulonglong),
        ("mfBranchProbability", ctypes.c_float),
    ]
    mRoomId: int
    mfBranchProbability: float

class cGcDungeonQuestParams(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfProbability", ctypes.c_float),
        ("maQuestItems", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mfProbability: float
    maQuestItems: bytes

class cGcDungeonGenerationParams(ctypes.Structure):
    def tilde_cGcDungeonGenerationParams(): pass

    def cGcDungeonGenerationParams(): pass

    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("muSizeX", ctypes.c_uint32),
        ("muSizeY", ctypes.c_uint32),
        ("muSizeZ", ctypes.c_uint32),
        ("muEntranceX", ctypes.c_uint32),
        ("muEntranceY", ctypes.c_uint32),
        ("muEntranceZ", ctypes.c_uint32),
        ("muRooms", ctypes.c_uint32),
        ("mfXProbability", ctypes.c_float),
        ("mfYProbability", ctypes.c_float),
        ("mfZProbability", ctypes.c_float),
        ("mfStraightMultiplier", ctypes.c_float),
        ("maMainRoomTypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDungeonRoomParams>
        ("maBranchRoomTypes", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maQuests", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDungeonQuestParams>
        ("maGenerationRules", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
        ("maPruningRules", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    muSizeX: int
    muSizeY: int
    muSizeZ: int
    muEntranceX: int
    muEntranceY: int
    muEntranceZ: int
    muRooms: int
    mfXProbability: float
    mfYProbability: float
    mfZProbability: float
    mfStraightMultiplier: float
    maMainRoomTypes: bytes
    maBranchRoomTypes: bytes
    maQuests: bytes
    maGenerationRules: bytes
    maPruningRules: bytes

class cGcCameraFollowSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mfMinSpeed", ctypes.c_float),
        ("mfSpeedRange", ctypes.c_float),
        ("mfOffsetX", ctypes.c_float),
        ("mfOffsetY", ctypes.c_float),
        ("mfOffsetYAlt", ctypes.c_float),
        ("mfOffsetYSlopeExtra", ctypes.c_float),
        ("mfOffsetZFlat", ctypes.c_float),
        ("mfBackMinDistance", ctypes.c_float),
        ("mfBackMaxDistance", ctypes.c_float),
        ("mfBackSlopeAdjust", ctypes.c_float),
        ("mfBackSlopeRotationAdjust", ctypes.c_float),
        ("mfUpMinDistance", ctypes.c_float),
        ("mfUpMaxDistance", ctypes.c_float),
        ("mfUpSlopeAdjust", ctypes.c_float),
        ("mfLeftMinDistance", ctypes.c_float),
        ("mfLeftMaxDistance", ctypes.c_float),
        ("mfOffsetYExtraMaxDistance", ctypes.c_float),
        ("mfPanNear", ctypes.c_float),
        ("mfPanFar", ctypes.c_float),
        ("mfUpGamma", ctypes.c_float),
        ("mfHorizRotationAngleMaxPerFrame", ctypes.c_float),
        ("mfVertRotationSpeed", ctypes.c_float),
        ("mfVertRotationMin", ctypes.c_float),
        ("mfVertRotationMax", ctypes.c_float),
        ("mfVertRotationOffset", ctypes.c_float),
        ("mfVertRotationOffsetMinAngle", ctypes.c_float),
        ("mfVertRotationOffsetMaxAngle", ctypes.c_float),
        ("mbVertStartLookingDown", ctypes.c_ubyte),
        ("mfDistSpeed", ctypes.c_float),
        ("mfDistSpeedOutsideMainRange", ctypes.c_float),
        ("mfDistStiffness", ctypes.c_float),
        ("mfSpringSpeed", ctypes.c_float),
        ("mbLockToObjectOnIdle", ctypes.c_ubyte),
        ("mfCenterStartTime", ctypes.c_float),
        ("mfCenterBlendTime", ctypes.c_float),
        ("mfCenterMaxSpring", ctypes.c_float),
        ("mfCenterMaxSpeed", ctypes.c_float),
        ("mfVelocityAnticipate", ctypes.c_float),
        ("mfVelocityAnticipateSpringSpeed", ctypes.c_float),
        ("mfVertMaxSpring", ctypes.c_float),
        ("mfCenterStartSpeed", ctypes.c_float),
        ("mfLookStickLimitAngle", ctypes.c_float),
        ("mbEnableCollisionDetection", ctypes.c_ubyte),
        ("miNumLRProbes", ctypes.c_int32),
        ("mfLRProbesRange", ctypes.c_float),
        ("mfLRProbesRadius", ctypes.c_float),
        ("miNumUDProbes", ctypes.c_int32),
        ("mfUDProbesRange", ctypes.c_float),
        ("mfProbeCenterX", ctypes.c_float),
        ("mfProbeCenterY", ctypes.c_float),
        ("mfPushForwardDropoffLR", ctypes.c_float),
        ("mfPushForwardDropoffUD", ctypes.c_float),
        ("mfAvoidCollisionLRSpeed", ctypes.c_float),
        ("mfAvoidCollisionUDSpeed", ctypes.c_float),
        ("mfAvoidCollisionPushSpeed", ctypes.c_float),
        ("mbAvoidCollisionUDUseStickDelay", ctypes.c_ubyte),
        ("mbAvoidCollisionLRUseStickDelay", ctypes.c_ubyte),
        ("mbUseSpeedBasedSpring", ctypes.c_ubyte),
        ("mbUseCustomBlendTime", ctypes.c_ubyte),
        ("mfCustomBlendTime", ctypes.c_float),
    ]
    mName: int
    mfMinSpeed: float
    mfSpeedRange: float
    mfOffsetX: float
    mfOffsetY: float
    mfOffsetYAlt: float
    mfOffsetYSlopeExtra: float
    mfOffsetZFlat: float
    mfBackMinDistance: float
    mfBackMaxDistance: float
    mfBackSlopeAdjust: float
    mfBackSlopeRotationAdjust: float
    mfUpMinDistance: float
    mfUpMaxDistance: float
    mfUpSlopeAdjust: float
    mfLeftMinDistance: float
    mfLeftMaxDistance: float
    mfOffsetYExtraMaxDistance: float
    mfPanNear: float
    mfPanFar: float
    mfUpGamma: float
    mfHorizRotationAngleMaxPerFrame: float
    mfVertRotationSpeed: float
    mfVertRotationMin: float
    mfVertRotationMax: float
    mfVertRotationOffset: float
    mfVertRotationOffsetMinAngle: float
    mfVertRotationOffsetMaxAngle: float
    mbVertStartLookingDown: bool
    mfDistSpeed: float
    mfDistSpeedOutsideMainRange: float
    mfDistStiffness: float
    mfSpringSpeed: float
    mbLockToObjectOnIdle: bool
    mfCenterStartTime: float
    mfCenterBlendTime: float
    mfCenterMaxSpring: float
    mfCenterMaxSpeed: float
    mfVelocityAnticipate: float
    mfVelocityAnticipateSpringSpeed: float
    mfVertMaxSpring: float
    mfCenterStartSpeed: float
    mfLookStickLimitAngle: float
    mbEnableCollisionDetection: bool
    miNumLRProbes: int
    mfLRProbesRange: float
    mfLRProbesRadius: float
    miNumUDProbes: int
    mfUDProbesRange: float
    mfProbeCenterX: float
    mfProbeCenterY: float
    mfPushForwardDropoffLR: float
    mfPushForwardDropoffUD: float
    mfAvoidCollisionLRSpeed: float
    mfAvoidCollisionUDSpeed: float
    mfAvoidCollisionPushSpeed: float
    mbAvoidCollisionUDUseStickDelay: bool
    mbAvoidCollisionLRUseStickDelay: bool
    mbUseSpeedBasedSpring: bool
    mbUseCustomBlendTime: bool
    mfCustomBlendTime: float

class cGcWeatherEffectLightningData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcWeatherHazardTornadoData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfSuckInRadius", ctypes.c_float),
        ("mfSuckInStrength", ctypes.c_float),
        ("mfSuckUpRadius", ctypes.c_float),
        ("mfSuckUpStrength", ctypes.c_float),
        ("mfSuckUpHeight", ctypes.c_float),
        ("mfSuckUpHeightCutoff", ctypes.c_float),
    ]
    mfSuckInRadius: float
    mfSuckInStrength: float
    mfSuckUpRadius: float
    mfSuckUpStrength: float
    mfSuckUpHeight: float
    mfSuckUpHeightCutoff: float

class cGcWeightedFilename(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfWeight", ctypes.c_float),
    ]
    macFilename: bytes
    mfWeight: float

class cGcMissionConditionExtraSuitSlots(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miCount", ctypes.c_int32),
    ]
    miCount: int

class cGcWeatherEffectTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maEffects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcWeatherEffect>
    ]
    maEffects: bytes

class cGcMissionConditionFeedingCreatures(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miMinCreatures", ctypes.c_int32),
        ("mbTakeNumFromSeasonData", ctypes.c_ubyte),
    ]
    miMinCreatures: int
    mbTakeNumFromSeasonData: bool

class cGcMissionConditionForceHideMultiplayer(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcFrigateClassCost(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maiCost", ctypes.c_char * 0x20),  # cTkFixedArray<int,8>
    ]
    maiCost: bytes

class cGcSkyProperties(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfAtmosphereThickness", ctypes.c_float),
        ("mfHorizonMultiplier", ctypes.c_float),
        ("mfDuskHorizonMultiplier", ctypes.c_float),
        ("mfNightHorizonMultiplier", ctypes.c_float),
        ("mfHorizonFadeSpeed", ctypes.c_float),
        ("mfDayHorizonTightness", ctypes.c_float),
        ("mfSunSize", ctypes.c_float),
        ("mfSunStrength", ctypes.c_float),
        ("mfSunSurroundSize", ctypes.c_float),
        ("mfSunSurroundStrength", ctypes.c_float),
        ("mfUpperSkyFadeSpeed", ctypes.c_float),
        ("mfUpperSkyFadeOffset", ctypes.c_float),
    ]
    mfAtmosphereThickness: float
    mfHorizonMultiplier: float
    mfDuskHorizonMultiplier: float
    mfNightHorizonMultiplier: float
    mfHorizonFadeSpeed: float
    mfDayHorizonTightness: float
    mfSunSize: float
    mfSunStrength: float
    mfSunSurroundSize: float
    mfSunSurroundStrength: float
    mfUpperSkyFadeSpeed: float
    mfUpperSkyFadeOffset: float

class cGcLightShaftProperties(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfLightShaftScattering", ctypes.c_float),
        ("mfLightShaftStrength", ctypes.c_float),
        ("mfLightShaftBottom", ctypes.c_float),
        ("mfLightShaftTop", ctypes.c_float),
        ("mLightShaftColourBottom", common.Colour),
        ("mLightShaftColourTop", common.Colour),
    ]
    mfLightShaftScattering: float
    mfLightShaftStrength: float
    mfLightShaftBottom: float
    mfLightShaftTop: float
    mLightShaftColourBottom: common.Colour
    mLightShaftColourTop: common.Colour

class cGcWFCDecorationSet(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maItems", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcWFCDecorationItem>
    ]
    maItems: bytes

class cGcMissionConditionGunOut(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionHasActiveDetailMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionHasActiveStatsMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionHasAnySettlementBuildingInProgress(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbIgnoreIfTimerActive", ctypes.c_ubyte),
    ]
    mbIgnoreIfTimerActive: bool

class cGcFrigateFlybyTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maEntries", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcFrigateFlybyLayout>
    ]
    maEntries: bytes

class cGcMissionConditionHasCommunicatorSignal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMissionConditionHasEndpointForEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mEventID", ctypes.c_ulonglong),
        ("mfMaxDistance", ctypes.c_float),
    ]
    mEventID: int
    mfMaxDistance: float

class cGcFrigateStatRange(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miMinimum", ctypes.c_int32),
        ("miMaximum", ctypes.c_int32),
    ]
    miMinimum: int
    miMaximum: int

class cGcMissionConditionHasEntitlement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mEntitlement", ctypes.c_ulonglong),
    ]
    mEntitlement: int

class cGcFrigateStats(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maStats", ctypes.c_char * 0x58),  # cTkFixedArray<cGcFrigateStatRange,11>
        ("mInitialTrait", ctypes.c_ulonglong),
    ]
    maStats: bytes
    mInitialTrait: int

class cGcFrigateStatsByClass(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maFrigateClass", ctypes.c_char * 0x340),  # cTkFixedArray<cGcFrigateStats,8>
    ]
    maFrigateClass: bytes

class cGcMissionConditionHasFuelForTakeoff(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbFormatTextAsPercentage", ctypes.c_ubyte),
    ]
    mbFormatTextAsPercentage: bool

class cGcMissionConditionHasGrabbableTarget(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcFrigateTraitIcons(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maacIcons", ctypes.c_char * 0x580),  # cTkFixedArray<cTkFixedString<128,char>,11>
    ]
    maacIcons: bytes

class cGcMissionConditionHasIllegalGoods(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbIncludeNipNip", ctypes.c_ubyte),
    ]
    mbIncludeNipNip: bool

class cGcMissionConditionHasIngredientsForItem(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTargetItems", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("mbRepair", ctypes.c_ubyte),
    ]
    maTargetItems: bytes
    mbRepair: bool

class cGcFrigateTraitStrengthValues(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maiStatAlteration", ctypes.c_char * 0x28),  # cTkFixedArray<int,10>
        ("mStatLocID", ctypes.c_ulonglong),
        ("mbStatDisplaysPositive", ctypes.c_ubyte),
    ]
    maiStatAlteration: bytes
    mStatLocID: int
    mbStatDisplaysPositive: bool

class cGcWeatherTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacTable", ctypes.c_char * 0x800),  # cTkFixedArray<cTkFixedString<128,char>,16>
        ("maDefaultTemperature", ctypes.c_char * 0x28),  # cTkFixedArray<cGcHazardValues,5>
        ("maDefaultToxicity", ctypes.c_char * 0x28),  # cTkFixedArray<cGcHazardValues,5>
        ("maDefaultRadiation", ctypes.c_char * 0x28),  # cTkFixedArray<cGcHazardValues,5>
    ]
    maacTable: bytes
    maDefaultTemperature: bytes
    maDefaultToxicity: bytes
    maDefaultRadiation: bytes

class cGcMissionConditionHasLegacyBasePending(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcWeightedColourId(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfRelativeProbability", ctypes.c_float),
        ("mPalette", ctypes.c_ulonglong),
        ("mDecorationPalette", ctypes.c_ulonglong),
    ]
    mfRelativeProbability: float
    mPalette: int
    mDecorationPalette: int

class cGcFrigateTraitStrengthByType(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maFrigateStatType", ctypes.c_char * 0x370),  # cTkFixedArray<cGcFrigateTraitStrengthValues,11>
    ]
    maFrigateStatType: bytes

class cGcFrigateTraitTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTraits", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcFrigateTraitData>
    ]
    maTraits: bytes

class cGcFrigateUITraitLines(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfLine0", ctypes.c_float),
        ("mfLine1", ctypes.c_float),
        ("mfLine2", ctypes.c_float),
        ("mfLine3", ctypes.c_float),
        ("mfLine4", ctypes.c_float),
    ]
    mfLine0: float
    mfLine1: float
    mfLine2: float
    mfLine3: float
    mfLine4: float

class cGcInventoryClassCostMultiplier(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mafMultiplier", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
    ]
    mafMultiplier: bytes

class cGcSettlementColourTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSettlementColourPalettes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementColourPalette>
        ("maDecorationPartIds", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maSettlementColourPalettes: bytes
    maDecorationPartIds: bytes

class cGcExpeditionDebriefPunctuation(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macPunctuation", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mfDelay", ctypes.c_float),
    ]
    macPunctuation: bytes
    mfDelay: float

class cGcExpeditionDifficultyKeyframe(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miEventNumber", ctypes.c_int32),
        ("mfDifficulty", ctypes.c_float),
    ]
    miEventNumber: int
    mfDifficulty: float

class cGcExpeditionDurationValues(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maiDuration", ctypes.c_char * 0x14),  # cTkFixedArray<int,5>
    ]
    maiDuration: bytes

class cGcGalaxyMarkerSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maColours", ctypes.c_char * 0x30),  # cTkFixedArray<cTkColour,3>
        ("mOutlineColour", common.Colour),
        ("mfOutlineWidth", ctypes.c_float),
        ("miEdgeCount", ctypes.c_int32),
        ("mfRotationBase", ctypes.c_float),
        ("mfLineWidth", ctypes.c_float),
        ("mfLineWidthFade", ctypes.c_float),
        ("mfSizeScale", ctypes.c_float),
        ("mfRadiusFixed", ctypes.c_float),
        ("mfRadiusMinimum", ctypes.c_float),
        ("mfRadiusBaseOffset", ctypes.c_float),
        ("mfRadiusEdge", ctypes.c_float),
        ("macIcon", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mIconSize", common.Vector2f),
        ("macMarkerLabel", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mTimeScaleRange", common.Vector2f),
    ]
    maColours: bytes
    mOutlineColour: common.Colour
    mfOutlineWidth: float
    miEdgeCount: int
    mfRotationBase: float
    mfLineWidth: float
    mfLineWidthFade: float
    mfSizeScale: float
    mfRadiusFixed: float
    mfRadiusMinimum: float
    mfRadiusBaseOffset: float
    mfRadiusEdge: float
    macIcon: bytes
    mIconSize: common.Vector2f
    macMarkerLabel: bytes
    mTimeScaleRange: common.Vector2f

class cGcGalaxyRenderAnostreakData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mOuterColour", common.Colour),
        ("mInnerColour", common.Colour),
        ("mfVerticalCompression", ctypes.c_float),
        ("mfHorizontalScale", ctypes.c_float),
        ("mfContrast", ctypes.c_float),
    ]
    mOuterColour: common.Colour
    mInnerColour: common.Colour
    mfVerticalCompression: float
    mfHorizontalScale: float
    mfContrast: float

class cGcGalaxyRenderSetupData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def Interpolate(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mSunCoreColour", common.Colour),
        ("mfSunCoreSmaller", ctypes.c_float),
        ("mfSunCoreLarger", ctypes.c_float),
        ("mfSunCoreBGContrib", ctypes.c_float),
        ("mfSunCoreFGContrib", ctypes.c_float),
        ("mfBGCellTraceScale", ctypes.c_float),
        ("mfBGCellMoveScale", ctypes.c_float),
        ("mfBGCellHorizonInfluence", ctypes.c_float),
        ("mfBGColourStage1", ctypes.c_float),
        ("mfBGColourStage2", ctypes.c_float),
        ("mfBGColourStage3", ctypes.c_float),
        ("mfBGColourStage4", ctypes.c_float),
        ("mfBGColourCellBlend", ctypes.c_float),
        ("mfBGColourPow", ctypes.c_float),
        ("mfStarFieldBlendAmount", ctypes.c_float),
        ("mfLensFlareBase", ctypes.c_float),
        ("mLensFlareColour", common.Colour),
        ("mLensFlareSpread", common.Vector4f),
        ("mfNebulaeAlphaPow", ctypes.c_float),
        ("mfNebulaeTraceValueMult", ctypes.c_float),
        ("mfNebulaeTraceScale", ctypes.c_float),
        ("mfNebulaeTraceDensity", ctypes.c_float),
        ("mfNebulaeTraceDensityCutoff", ctypes.c_float),
        ("mNebulaeTraceStepRange", common.Vector2f),
        ("mCompositionControlB_S_C_G", common.Vector4f),
        ("mfCompositionSaturationIncreaseError", ctypes.c_float),
        ("mfCompositionSaturationIncreaseFilter", ctypes.c_float),
        ("mfCompositionSaturationIncreaseSelected", ctypes.c_float),
        ("mfVignetteSize", ctypes.c_float),
        ("mfVignetteBase", ctypes.c_float),
        ("mfVignetteSizeIncreaseError", ctypes.c_float),
        ("mfVignetteSizeIncreaseFilter", ctypes.c_float),
        ("mfVignetteSizeIncreaseSelected", ctypes.c_float),
        ("mLensFlareExpandTowards", common.Vector2f),
        ("maMapLargeAreaPrimaryDefaultColours", ctypes.c_char * 0xA0),  # cTkFixedArray<cTkColour,10>
        ("maMapLargeAreaPrimaryHighContrastColours", ctypes.c_char * 0xA0),  # cTkFixedArray<cTkColour,10>
        ("maMapLargeAreaSecondaryDefaultColours", ctypes.c_char * 0xA0),  # cTkFixedArray<cTkColour,10>
        ("maMapLargeAreaSecondaryHighContrastColours", ctypes.c_char * 0xA0),  # cTkFixedArray<cTkColour,10>
    ]
    mSunCoreColour: common.Colour
    mfSunCoreSmaller: float
    mfSunCoreLarger: float
    mfSunCoreBGContrib: float
    mfSunCoreFGContrib: float
    mfBGCellTraceScale: float
    mfBGCellMoveScale: float
    mfBGCellHorizonInfluence: float
    mfBGColourStage1: float
    mfBGColourStage2: float
    mfBGColourStage3: float
    mfBGColourStage4: float
    mfBGColourCellBlend: float
    mfBGColourPow: float
    mfStarFieldBlendAmount: float
    mfLensFlareBase: float
    mLensFlareColour: common.Colour
    mLensFlareSpread: common.Vector4f
    mfNebulaeAlphaPow: float
    mfNebulaeTraceValueMult: float
    mfNebulaeTraceScale: float
    mfNebulaeTraceDensity: float
    mfNebulaeTraceDensityCutoff: float
    mNebulaeTraceStepRange: common.Vector2f
    mCompositionControlB_S_C_G: common.Vector4f
    mfCompositionSaturationIncreaseError: float
    mfCompositionSaturationIncreaseFilter: float
    mfCompositionSaturationIncreaseSelected: float
    mfVignetteSize: float
    mfVignetteBase: float
    mfVignetteSizeIncreaseError: float
    mfVignetteSizeIncreaseFilter: float
    mfVignetteSizeIncreaseSelected: float
    mLensFlareExpandTowards: common.Vector2f
    maMapLargeAreaPrimaryDefaultColours: bytes
    maMapLargeAreaPrimaryHighContrastColours: bytes
    maMapLargeAreaSecondaryDefaultColours: bytes
    maMapLargeAreaSecondaryHighContrastColours: bytes

class cGcGalaxySolarSystemOrbitParams(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfFirstOrbitRadiusMin", ctypes.c_float),
        ("mfFirstOrbitRadiusMax", ctypes.c_float),
        ("mfOrbitRadiusOffsetMin", ctypes.c_float),
        ("mfOrbitRadiusOffsetMax", ctypes.c_float),
        ("mfOrbitRotationSpeedMin", ctypes.c_float),
        ("mfOrbitRotationSpeedMax", ctypes.c_float),
        ("mfOrbitLineWidth", ctypes.c_float),
    ]
    mfFirstOrbitRadiusMin: float
    mfFirstOrbitRadiusMax: float
    mfOrbitRadiusOffsetMin: float
    mfOrbitRadiusOffsetMax: float
    mfOrbitRotationSpeedMin: float
    mfOrbitRotationSpeedMax: float
    mfOrbitLineWidth: float

class cGcGalaxyStarColours(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maColourByStarType", ctypes.c_char * 0x40),  # cTkFixedArray<cTkColour,4>
    ]
    maColourByStarType: bytes

class cGcGalaxyVoxelAttributesData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbInsideGoalGap", ctypes.c_ubyte),
        ("mfUnitDistanceFromGoalEdge", ctypes.c_float),
        ("mfRegionColourValue", ctypes.c_float),
        ("mTransitPopulationDistanceRange", common.Vector2f),
        ("mfTransitPopulationPerpDistance", ctypes.c_float),
        ("miGuideStarMinimumCount", ctypes.c_int32),
        ("miGuideStarRenegadeCount", ctypes.c_int32),
        ("miBlackholeCount", ctypes.c_int32),
        ("miAtlasStationCount", ctypes.c_int32),
        ("maiBlackholeIndices", ctypes.c_char * 0x30),  # cTkFixedArray<int,12>
        ("maiAtlasStationIndices", ctypes.c_char * 0x30),  # cTkFixedArray<int,12>
    ]
    mbInsideGoalGap: bool
    mfUnitDistanceFromGoalEdge: float
    mfRegionColourValue: float
    mTransitPopulationDistanceRange: common.Vector2f
    mfTransitPopulationPerpDistance: float
    miGuideStarMinimumCount: int
    miGuideStarRenegadeCount: int
    miBlackholeCount: int
    miAtlasStationCount: int
    maiBlackholeIndices: bytes
    maiAtlasStationIndices: bytes

class cGcBehaviourPlayAnimTrigger(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miFrame", ctypes.c_int32),
        ("mTrigger", ctypes.c_ulonglong),
    ]
    miFrame: int
    mTrigger: int

class cGcCreatureBaitComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maAttractList", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("mfBaitStrength", ctypes.c_float),
        ("mfBaitRadius", ctypes.c_float),
        ("mbDebug", ctypes.c_ubyte),
        ("mbInducesRage", ctypes.c_ubyte),
    ]
    maAttractList: bytes
    mfBaitStrength: float
    mfBaitRadius: float
    mbDebug: bool
    mbInducesRage: bool

class cGcCreatureEffectTrigger(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mEffect", ctypes.c_ulonglong),
        ("maacJointName", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<256,char> >
        ("mfScale", ctypes.c_float),
        ("mAnim", ctypes.c_ulonglong),
        ("miFrame", ctypes.c_int32),
        ("mbGroundTint", ctypes.c_ubyte),
    ]
    mEffect: int
    maacJointName: bytes
    mfScale: float
    mAnim: int
    miFrame: int
    mbGroundTint: bool

class cGcCreatureEffectComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maAnimTriggers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureEffectTrigger>
    ]
    maAnimTriggers: bytes

class cGcCreatureEggComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
    ]
    mId: int

class cGcCreatureFilename(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macExtraFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mID: int
    macFilename: bytes
    macExtraFilename: bytes

class cGcCreatureGenerationDomainTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maTables", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationDomainEntry>
        ("mfChanceOfHemisphereLimit", ctypes.c_float),
        ("miMaxToHemisphereLimit", ctypes.c_int32),
    ]
    mId: int
    maTables: bytes
    mfChanceOfHemisphereLimit: float
    miMaxToHemisphereLimit: int

class cGcCreatureGenerationArchetypes(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGroundArchetypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationDomainTable>
        ("maAirArchetypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationDomainTable>
        ("maWaterArchetypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationDomainTable>
        ("maCaveArchetypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationDomainTable>
    ]
    maGroundArchetypes: bytes
    maAirArchetypes: bytes
    maWaterArchetypes: bytes
    maCaveArchetypes: bytes

class cGcCreatureGenerationWeightedListDomainEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfWeight", ctypes.c_float),
        ("mArchetype", ctypes.c_ulonglong),
    ]
    mfWeight: float
    mArchetype: int

class cGcCreatureGenerationWeightedList(ctypes.Structure):
    def tilde_cGcCreatureGenerationWeightedList(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maGround", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationWeightedListDomainEntry>
        ("maAir", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationWeightedListDomainEntry>
        ("maCave", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationWeightedListDomainEntry>
        ("maWater", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureGenerationWeightedListDomainEntry>
    ]
    maGround: bytes
    maAir: bytes
    maCave: bytes
    maWater: bytes

class cGcCreatureGroupDescription(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGroup", ctypes.c_ulonglong),
        ("miMinGroupSize", ctypes.c_int32),
        ("miMaxGroupSize", ctypes.c_int32),
        ("mfGroupsPerSquareKm", ctypes.c_float),
    ]
    mGroup: int
    miMinGroupSize: int
    miMaxGroupSize: int
    mfGroupsPerSquareKm: float

class cGcCreatureGroupProbability(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mGroup", ctypes.c_ulonglong),
        ("mfProbability", ctypes.c_float),
    ]
    mGroup: int
    mfProbability: float

class cGcDeprecatedAssetsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    maacTable: bytes

class cGcEcosystemSpawnData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maCreatures", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcEcosystemCreatureData>
        ("mfCreatureMinNoise", ctypes.c_float),
        ("mfCreatureMaxNoise", ctypes.c_float),
    ]
    maCreatures: bytes
    mfCreatureMinNoise: float
    mfCreatureMaxNoise: float

class cGcCreatureLegIKComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miStuff", ctypes.c_int32),
    ]
    miStuff: int

class cGcCreatureRoleFilename(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mafBiomeProbability", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
    ]
    macFile: bytes
    mafBiomeProbability: bytes

class cGcCreatureRoleFilenameList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureRoleFilename>
    ]
    maOptions: bytes

class cGcIkPistonData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macJoint1Name", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("macJoint2Name", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
    ]
    macJoint1Name: bytes
    macJoint2Name: bytes

class cGcPetAccessoryGroup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maDisallowedAccessories", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPetAccessoryType>
    ]
    mId: int
    maDisallowedAccessories: bytes

class cGcPetAccessoryInfo(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mDescriptor", ctypes.c_ulonglong),
    ]
    mDescriptor: int

class cGcFoliageComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfRadius", ctypes.c_float),
    ]
    mfRadius: float

class cGcBuildingClusterLayout(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mfRelativeProbability", ctypes.c_float),
        ("maClusterBuildings", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBuildingClusterLayoutEntry>
        ("miAlignmentSteps", ctypes.c_int32),
        ("mfAlignmentJitter", ctypes.c_float),
    ]
    mID: int
    mfRelativeProbability: float
    maClusterBuildings: bytes
    miAlignmentSteps: int
    mfAlignmentJitter: float

class cGcPetAccessoryTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maAccessories", ctypes.c_char * 0x360),  # cTkFixedArray<cGcPetAccessoryInfo,27>
        ("maAccessoryGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPetAccessoryGroup>
    ]
    maAccessories: bytes
    maAccessoryGroups: bytes

class cGcPetEggSpeciesOverrideData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mCreatureID", ctypes.c_ulonglong),
        ("mbCanChangeGrowth", ctypes.c_ubyte),
        ("mbCanChangeAccessories", ctypes.c_ubyte),
        ("mbCanChangeColour", ctypes.c_ubyte),
        ("mbCanChangeTraits", ctypes.c_ubyte),
        ("mfMinScaleOverride", ctypes.c_float),
        ("mfMaxScaleOverride", ctypes.c_float),
    ]
    mCreatureID: int
    mbCanChangeGrowth: bool
    mbCanChangeAccessories: bool
    mbCanChangeColour: bool
    mbCanChangeTraits: bool
    mfMinScaleOverride: float
    mfMaxScaleOverride: float

class cGcPetEggSpeciesOverrideTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSpeciesOverrides", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPetEggSpeciesOverrideData>
    ]
    maSpeciesOverrides: bytes

class cGcResourceCollectEffect(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfPlayerOffset", ctypes.c_float),
        ("mfSizeMin", ctypes.c_float),
        ("mfSizeMax", ctypes.c_float),
        ("mfRotateMin", ctypes.c_float),
        ("mfRotateMax", ctypes.c_float),
        ("mfStartOffsetMin", ctypes.c_float),
        ("mfStartOffsetMax", ctypes.c_float),
        ("mfTimeMin", ctypes.c_float),
        ("mfTimeMax", ctypes.c_float),
        ("mfStartSpeedMin", ctypes.c_float),
        ("mfStartSpeedMax", ctypes.c_float),
        ("mfOffsetMin", ctypes.c_float),
        ("mfOffsetMax", ctypes.c_float),
    ]
    mfPlayerOffset: float
    mfSizeMin: float
    mfSizeMax: float
    mfRotateMin: float
    mfRotateMax: float
    mfStartOffsetMin: float
    mfStartOffsetMax: float
    mfTimeMin: float
    mfTimeMax: float
    mfStartSpeedMin: float
    mfStartSpeedMax: float
    mfOffsetMin: float
    mfOffsetMax: float

class cGcSpaceSkyColours(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mCloudColour", common.Colour),
        ("mColourTop", common.Colour),
        ("mColourMid", common.Colour),
        ("mColourBottom", common.Colour),
        ("mColourTopPlanet", common.Colour),
        ("mColourMidPlanet", common.Colour),
        ("mColourBottomPlanet", common.Colour),
        ("mSunColour", common.Colour),
        ("mNebulaColour1", common.Colour),
        ("mNebulaColour2", common.Colour),
        ("mNebulaColour3", common.Colour),
        ("mNebulaShadowColour", common.Colour),
        ("mFogColour", common.Colour),
        ("mFogColour2", common.Colour),
    ]
    mCloudColour: common.Colour
    mColourTop: common.Colour
    mColourMid: common.Colour
    mColourBottom: common.Colour
    mColourTopPlanet: common.Colour
    mColourMidPlanet: common.Colour
    mColourBottomPlanet: common.Colour
    mSunColour: common.Colour
    mNebulaColour1: common.Colour
    mNebulaColour2: common.Colour
    mNebulaColour3: common.Colour
    mNebulaShadowColour: common.Colour
    mFogColour: common.Colour
    mFogColour2: common.Colour

class cGcTileTypeOptions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkPaletteTexture>
    ]
    maOptions: bytes

class cGcPetEggTraitModifierOverrideTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTraitModifiers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPetEggTraitModifierOverrideData>
    ]
    maTraitModifiers: bytes

class cGcSandwormTimerAndFrequencyOverride(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mu64PackedUA", ctypes.c_uint64),
        ("mfTimer", ctypes.c_float),
        ("mfSpawnChance", ctypes.c_float),
    ]
    mu64PackedUA: int
    mfTimer: float
    mfSpawnChance: float

class cGcBuildingFilename(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maacLSystem", ctypes.c_char * 0x100),  # cTkFixedArray<cTkFixedString<128,char>,2>
        ("maacScene", ctypes.c_char * 0x100),  # cTkFixedArray<cTkFixedString<128,char>,2>
        ("maacWFC", ctypes.c_char * 0x100),  # cTkFixedArray<cTkFixedString<128,char>,2>
    ]
    maacLSystem: bytes
    maacScene: bytes
    maacWFC: bytes

class cGcBuildingFilenameList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBuildingFiles", ctypes.c_char * 0x9C00),  # cTkFixedArray<cGcBuildingFilename,52>
    ]
    maBuildingFiles: bytes

class cGcBuildingDefinitionTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBuildingFiles", ctypes.c_char * 0x4E000),  # cTkFixedArray<cGcBuildingFilenameList,8>
        ("maBuildingPlacement", ctypes.c_char * 0x1860),  # cTkFixedArray<cGcBuildingDefinitionData,52>
        ("maClusterLayouts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBuildingClusterLayout>
    ]
    maBuildingFiles: bytes
    maBuildingPlacement: bytes
    maClusterLayouts: bytes

class cGcBuildingDensity(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfBuildingSpacing", ctypes.c_float),
    ]
    mfBuildingSpacing: float

class cGcBuildingDistribution(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("miMinDistance", ctypes.c_int32),
        ("miMaxDistance", ctypes.c_int32),
    ]
    mName: int
    miMinDistance: int
    miMaxDistance: int

class cGcCloudProperties(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfHorizonCoverageStart", ctypes.c_float),
        ("mfHorizonCoverageEnd", ctypes.c_float),
        ("mCloudBaseColour", common.Colour),
        ("mCloudTopColour", common.Colour),
        ("mfLightScalar", ctypes.c_float),
        ("mfAmbientScalar", ctypes.c_float),
        ("mfSunRayLength", ctypes.c_float),
        ("mfConeRadius", ctypes.c_float),
        ("mfDensity", ctypes.c_float),
        ("mfAmbientDensity", ctypes.c_float),
        ("mfForwardScatteringG", ctypes.c_float),
        ("mfBackwardScatteringG", ctypes.c_float),
        ("mfDarkOutlineScalar", ctypes.c_float),
        ("mfAnimationScale", ctypes.c_float),
        ("mWindOffset", common.Vector2f),
        ("mStratosphereWindOffset", common.Vector2f),
        ("mfBaseScale", ctypes.c_float),
        ("mfSampleScalar", ctypes.c_float),
        ("mfSampleThreshold", ctypes.c_float),
        ("mfCloudBottomFade", ctypes.c_float),
        ("mCloudHeightGradient1", common.Vector4f),
        ("mCloudHeightGradient2", common.Vector4f),
        ("mCloudHeightGradient3", common.Vector4f),
        ("mfDetailScale", ctypes.c_float),
        ("mfErosionEdgeSize", ctypes.c_float),
        ("mfCloudDistortion", ctypes.c_float),
        ("mfCloudDistortionScale", ctypes.c_float),
        ("mfMaxIterations", ctypes.c_float),
        ("mfRayMinimumY", ctypes.c_float),
        ("mfLODDistance", ctypes.c_float),
        ("mfHorizonFadeStartAlpha", ctypes.c_float),
        ("mfHorizonFadeScalar", ctypes.c_float),
        ("mfHorizonDistance", ctypes.c_float),
    ]
    mfHorizonCoverageStart: float
    mfHorizonCoverageEnd: float
    mCloudBaseColour: common.Colour
    mCloudTopColour: common.Colour
    mfLightScalar: float
    mfAmbientScalar: float
    mfSunRayLength: float
    mfConeRadius: float
    mfDensity: float
    mfAmbientDensity: float
    mfForwardScatteringG: float
    mfBackwardScatteringG: float
    mfDarkOutlineScalar: float
    mfAnimationScale: float
    mWindOffset: common.Vector2f
    mStratosphereWindOffset: common.Vector2f
    mfBaseScale: float
    mfSampleScalar: float
    mfSampleThreshold: float
    mfCloudBottomFade: float
    mCloudHeightGradient1: common.Vector4f
    mCloudHeightGradient2: common.Vector4f
    mCloudHeightGradient3: common.Vector4f
    mfDetailScale: float
    mfErosionEdgeSize: float
    mfCloudDistortion: float
    mfCloudDistortionScale: float
    mfMaxIterations: float
    mfRayMinimumY: float
    mfLODDistance: float
    mfHorizonFadeStartAlpha: float
    mfHorizonFadeScalar: float
    mfHorizonDistance: float

class cGcEnvironmentProperties(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfFlightFogHeight", ctypes.c_float),
        ("mfFlightFogBlend", ctypes.c_float),
        ("mfCloudHeightMin", ctypes.c_float),
        ("mfCloudHeightMax", ctypes.c_float),
        ("mfHeavyAirHeightMin", ctypes.c_float),
        ("mfHeavyAirHeightMax", ctypes.c_float),
        ("mfPlanetObjectSwitch", ctypes.c_float),
        ("mfPlanetLodSwitch0", ctypes.c_float),
        ("mfPlanetLodSwitch0Elevation", ctypes.c_float),
        ("mfPlanetLodSwitch1", ctypes.c_float),
        ("mfPlanetLodSwitch2", ctypes.c_float),
        ("mfPlanetLodSwitch3", ctypes.c_float),
        ("mfAsteroidFadeHeightMin", ctypes.c_float),
        ("mfAsteroidFadeHeightMax", ctypes.c_float),
        ("mafSkyHeight", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("mfSkyAtmosphereHeight", ctypes.c_float),
        ("mfHorizonBlendHeight", ctypes.c_float),
        ("mfHorizonBlendLength", ctypes.c_float),
        ("mfSkyColourHeight", ctypes.c_float),
        ("mfSkyColourBlendLength", ctypes.c_float),
        ("mfSkyPositionHeight", ctypes.c_float),
        ("mfSkyPositionBlendLength", ctypes.c_float),
        ("mfSolarSystemLUTHeight", ctypes.c_float),
        ("mfSolarSystemLUTBlendLength", ctypes.c_float),
        ("mfAtmosphereStartHeight", ctypes.c_float),
        ("mfAtmosphereEndHeight", ctypes.c_float),
        ("mfStratosphereHeight", ctypes.c_float),
    ]
    mfFlightFogHeight: float
    mfFlightFogBlend: float
    mfCloudHeightMin: float
    mfCloudHeightMax: float
    mfHeavyAirHeightMin: float
    mfHeavyAirHeightMax: float
    mfPlanetObjectSwitch: float
    mfPlanetLodSwitch0: float
    mfPlanetLodSwitch0Elevation: float
    mfPlanetLodSwitch1: float
    mfPlanetLodSwitch2: float
    mfPlanetLodSwitch3: float
    mfAsteroidFadeHeightMin: float
    mfAsteroidFadeHeightMax: float
    mafSkyHeight: bytes
    mfSkyAtmosphereHeight: float
    mfHorizonBlendHeight: float
    mfHorizonBlendLength: float
    mfSkyColourHeight: float
    mfSkyColourBlendLength: float
    mfSkyPositionHeight: float
    mfSkyPositionBlendLength: float
    mfSolarSystemLUTHeight: float
    mfSolarSystemLUTBlendLength: float
    mfAtmosphereStartHeight: float
    mfAtmosphereEndHeight: float
    mfStratosphereHeight: float

class cGcPetTraitMoodModifier(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfTraitMin", ctypes.c_float),
        ("mfTraitMax", ctypes.c_float),
        ("mfMoodIncreaseMultiplierMin", ctypes.c_float),
        ("mfMoodIncreaseMultiplierMax", ctypes.c_float),
    ]
    mfTraitMin: float
    mfTraitMax: float
    mfMoodIncreaseMultiplierMin: float
    mfMoodIncreaseMultiplierMax: float

class cGcPetTraitMoodModifierList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maModifiers", ctypes.c_char * 0x20),  # cTkFixedArray<cGcPetTraitMoodModifier,2>
    ]
    maModifiers: bytes

class cGcPetActionMoodModifier(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mafMoodModifiers", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
    ]
    mafMoodModifiers: bytes

class cGcCreaturePetTraitRange(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMin", ctypes.c_float),
        ("mfMax", ctypes.c_float),
    ]
    mfMin: float
    mfMax: float

class cGcCreaturePetTraitRanges(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTraitRanges", ctypes.c_char * 0x18),  # cTkFixedArray<cGcCreaturePetTraitRange,3>
    ]
    maTraitRanges: bytes

class cGcPetBehaviourTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maBehaviours", ctypes.c_char * 0xE00),  # cTkFixedArray<cGcPetBehaviourData,28>
        ("mfGlobalCooldownModifier", ctypes.c_float),
        ("mfUsefulBehaviourLinkedCooldownAmount", ctypes.c_float),
        ("mfPlayerActivityIncreaseTime", ctypes.c_float),
        ("mfPlayerActivityDecreaseTime", ctypes.c_float),
        ("mafMoodIncreaseTime", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("maTraitMoodModifiers", ctypes.c_char * 0x60),  # cTkFixedArray<cGcPetTraitMoodModifierList,3>
        ("maRewardMoodModifier", ctypes.c_char * 0x48),  # cTkFixedArray<cGcPetActionMoodModifier,9>
        ("maTraitRanges", ctypes.c_char * 0x108),  # cTkFixedArray<cGcCreaturePetTraitRanges,11>
        ("mafMoodValuesOnAdopt", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("mafMoodValuesOnHatch", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("maTraitStaminaModifiers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPetTraitStaminaModifier>
        ("maMoodStaminaModifiers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPetMoodStaminaModifier>
        ("mfAccessoryGyroFollowMotionStrength", ctypes.c_float),
        ("mfAccessoryGyroToNeutralStrength", ctypes.c_float),
        ("mfAccessoryGyroDamping", ctypes.c_float),
        ("mfAccessoryGyroStrength", ctypes.c_float),
        ("mbAccessoryGyroActive", ctypes.c_ubyte),
    ]
    maBehaviours: bytes
    mfGlobalCooldownModifier: float
    mfUsefulBehaviourLinkedCooldownAmount: float
    mfPlayerActivityIncreaseTime: float
    mfPlayerActivityDecreaseTime: float
    mafMoodIncreaseTime: bytes
    maTraitMoodModifiers: bytes
    maRewardMoodModifier: bytes
    maTraitRanges: bytes
    mafMoodValuesOnAdopt: bytes
    mafMoodValuesOnHatch: bytes
    maTraitStaminaModifiers: bytes
    maMoodStaminaModifiers: bytes
    mfAccessoryGyroFollowMotionStrength: float
    mfAccessoryGyroToNeutralStrength: float
    mfAccessoryGyroDamping: float
    mfAccessoryGyroStrength: float
    mbAccessoryGyroActive: bool

class cGcBehaviourPlayAnimData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mAnim", ctypes.c_ulonglong),
        ("mfBlendInTime", ctypes.c_float),
        ("mfBlendOutAt", ctypes.c_float),
        ("maTriggers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBehaviourPlayAnimTrigger>
    ]
    mAnim: int
    mfBlendInTime: float
    mfBlendOutAt: float
    maTriggers: bytes

class cGcBehaviourRegisterAttackerData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mTargetKey", ctypes.c_ulonglong),
    ]
    mTargetKey: int

class cGcBehaviourWaitData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfSeconds", ctypes.c_float),
    ]
    mfSeconds: float

class cGcBirdData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfFlapSpeedMin", ctypes.c_float),
        ("mfFlapSpeedMax", ctypes.c_float),
        ("mfFlapSpeed", ctypes.c_float),
        ("mfFlapAccel", ctypes.c_float),
        ("mfFlapTurn", ctypes.c_float),
        ("mfFlapSpeedMinScale", ctypes.c_float),
        ("mfFlapSpeedMaxScale", ctypes.c_float),
        ("mfFlapSpeedForMinScale", ctypes.c_float),
        ("mfFlapSpeedForMaxScale", ctypes.c_float),
        ("macCircleAttractor", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mfFlapSpeedMin: float
    mfFlapSpeedMax: float
    mfFlapSpeed: float
    mfFlapAccel: float
    mfFlapTurn: float
    mfFlapSpeedMinScale: float
    mfFlapSpeedMaxScale: float
    mfFlapSpeedForMinScale: float
    mfFlapSpeedForMaxScale: float
    macCircleAttractor: bytes

class cGcCreatureCrystalMovementData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maParams", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureCrystalMovementDataParams>
    ]
    maParams: bytes

class cGcCreatureFiendAttackData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfNearDist", ctypes.c_float),
        ("mfFarDist", ctypes.c_float),
        ("mfModifyDistanceForHeight", ctypes.c_float),
        ("mSpitAnim", ctypes.c_ulonglong),
        ("miSpitAnimFrame", ctypes.c_int32),
        ("macSpitJoint", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mSpitProjectile", ctypes.c_ulonglong),
        ("mbAllowSpit", ctypes.c_ubyte),
        ("mfDelayBetweenSpitAttacks", ctypes.c_float),
        ("mbAllowPounce", ctypes.c_ubyte),
        ("mfDelayBetweenPounceAttacks", ctypes.c_float),
        ("macAttackLight", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mfIdleLightIntensity", ctypes.c_float),
        ("mfAttackLightIntensity", ctypes.c_float),
        ("miMinFlurryHits", ctypes.c_int32),
        ("miMaxFlurryHits", ctypes.c_int32),
        ("mfRoarChanceOnHit", ctypes.c_float),
        ("mfRoarChanceOnMiss", ctypes.c_float),
        ("mfStartDamageTime", ctypes.c_float),
        ("mfTurnToFaceTime", ctypes.c_float),
    ]
    mfNearDist: float
    mfFarDist: float
    mfModifyDistanceForHeight: float
    mSpitAnim: int
    miSpitAnimFrame: int
    macSpitJoint: bytes
    mSpitProjectile: int
    mbAllowSpit: bool
    mfDelayBetweenSpitAttacks: float
    mbAllowPounce: bool
    mfDelayBetweenPounceAttacks: float
    macAttackLight: bytes
    mfIdleLightIntensity: float
    mfAttackLightIntensity: float
    miMinFlurryHits: int
    miMaxFlurryHits: int
    mfRoarChanceOnHit: float
    mfRoarChanceOnMiss: float
    mfStartDamageTime: float
    mfTurnToFaceTime: float

class cGcWeirdCreatureRewardList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maRewards", ctypes.c_char * 0x1B0),  # cTkFixedArray<TkID<128>,27>
    ]
    maRewards: bytes

class cGcCreatureRidingAnimation(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mMovementAnim", ctypes.c_ulonglong),
        ("mRidingAnim", ctypes.c_ulonglong),
    ]
    mMovementAnim: int
    mRidingAnim: int

class cGcCreatureRidingPartModifier(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPartName", ctypes.c_ulonglong),
        ("macJointName", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mfMinScale", ctypes.c_float),
        ("mfMaxScale", ctypes.c_float),
        ("macAdditionalScaleJoint", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mbBreakIfNotSelected", ctypes.c_ubyte),
        ("mbRelativeOffset", ctypes.c_ubyte),
        ("mOffset", common.Vector3f),
        ("mRotationOffset", common.Vector3f),
        ("mVROffset", common.Vector3f),
        ("mfHeadCounterRotation", ctypes.c_float),
        ("mfLegSpreadOffset", ctypes.c_float),
        ("mbOverrideAnims", ctypes.c_ubyte),
        ("mIdleRidingAnim", ctypes.c_ulonglong),
        ("mDefaultRidingAnim", ctypes.c_ulonglong),
        ("maRidingAnims", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureRidingAnimation>
    ]
    mPartName: int
    macJointName: bytes
    mfMinScale: float
    mfMaxScale: float
    macAdditionalScaleJoint: bytes
    mbBreakIfNotSelected: bool
    mbRelativeOffset: bool
    mOffset: common.Vector3f
    mRotationOffset: common.Vector3f
    mVROffset: common.Vector3f
    mfHeadCounterRotation: float
    mfLegSpreadOffset: float
    mbOverrideAnims: bool
    mIdleRidingAnim: int
    mDefaultRidingAnim: int
    maRidingAnims: bytes

class cGcCreatureRidingData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macJointName", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("macAdditionalScaleJoint", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mOffset", common.Vector3f),
        ("mRotationOffset", common.Vector3f),
        ("mVROffset", common.Vector3f),
        ("mfHeadCounterRotation", ctypes.c_float),
        ("mfUprightStrength", ctypes.c_float),
        ("mbLegSpread", ctypes.c_ubyte),
        ("mfScaleForNeutralLegSpread", ctypes.c_float),
        ("mfScaleForMinLegSpread", ctypes.c_float),
        ("mfScaleForMaxLegSpread", ctypes.c_float),
        ("mbRequiresMatchingPartModifier", ctypes.c_ubyte),
        ("maPartModifiers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureRidingPartModifier>
        ("mIdleRidingAnim", ctypes.c_ulonglong),
        ("mDefaultRidingAnim", ctypes.c_ulonglong),
        ("maRidingAnims", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureRidingAnimation>
    ]
    macJointName: bytes
    macAdditionalScaleJoint: bytes
    mOffset: common.Vector3f
    mRotationOffset: common.Vector3f
    mVROffset: common.Vector3f
    mfHeadCounterRotation: float
    mfUprightStrength: float
    mbLegSpread: bool
    mfScaleForNeutralLegSpread: float
    mfScaleForMinLegSpread: float
    mfScaleForMaxLegSpread: float
    mbRequiresMatchingPartModifier: bool
    maPartModifiers: bytes
    mIdleRidingAnim: int
    mDefaultRidingAnim: int
    maRidingAnims: bytes

class cGcCreatureSwarmDataParams(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maValidDescriptors", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
        ("mfCoherence", ctypes.c_float),
        ("mfAlignment", ctypes.c_float),
        ("mfSeparateStrength", ctypes.c_float),
        ("mfSpacing", ctypes.c_float),
        ("mfFollow", ctypes.c_float),
        ("mfRange", ctypes.c_float),
        ("mfMaxBankingAmount", ctypes.c_float),
        ("mfTurnRequiredForMaxBanking", ctypes.c_float),
        ("mfMaxPitchAmount", ctypes.c_float),
        ("mfUpwardMovementForMaxPitch", ctypes.c_float),
        ("mfFaceMoveDirStrength", ctypes.c_float),
        ("mbFaceMoveDirYawOnly", ctypes.c_ubyte),
        ("mbCanLand", ctypes.c_ubyte),
        ("mfLandAdjustDist", ctypes.c_float),
        ("mfLandClampBegin", ctypes.c_float),
        ("mfLandSlowDown", ctypes.c_float),
        ("mfTakeOffTime", ctypes.c_float),
        ("mfTakeOffStartSpeed", ctypes.c_float),
        ("mfTakeOffUpwardBoost", ctypes.c_float),
        ("mbCanWalk", ctypes.c_ubyte),
        ("mfWalkSpeed", ctypes.c_float),
        ("mfWalkTurnTime", ctypes.c_float),
        ("mfFlyTimeMin", ctypes.c_float),
        ("mfFlyTimeMax", ctypes.c_float),
        ("mfLandTimeMin", ctypes.c_float),
        ("mfLandTimeMax", ctypes.c_float),
        ("mfLandIdleTimeMin", ctypes.c_float),
        ("mfLandIdleTimeMax", ctypes.c_float),
        ("mfLandWalkTimeMin", ctypes.c_float),
        ("mfLandWalkTimeMax", ctypes.c_float),
        ("mfMaxSpeed", ctypes.c_float),
        ("mfAlignTime", ctypes.c_float),
        ("mfBankingTime", ctypes.c_float),
        ("mfSpringSmoothTime", ctypes.c_float),
        ("mbApplyScaleToSpeed", ctypes.c_ubyte),
    ]
    maValidDescriptors: bytes
    mfCoherence: float
    mfAlignment: float
    mfSeparateStrength: float
    mfSpacing: float
    mfFollow: float
    mfRange: float
    mfMaxBankingAmount: float
    mfTurnRequiredForMaxBanking: float
    mfMaxPitchAmount: float
    mfUpwardMovementForMaxPitch: float
    mfFaceMoveDirStrength: float
    mbFaceMoveDirYawOnly: bool
    mbCanLand: bool
    mfLandAdjustDist: float
    mfLandClampBegin: float
    mfLandSlowDown: float
    mfTakeOffTime: float
    mfTakeOffStartSpeed: float
    mfTakeOffUpwardBoost: float
    mbCanWalk: bool
    mfWalkSpeed: float
    mfWalkTurnTime: float
    mfFlyTimeMin: float
    mfFlyTimeMax: float
    mfLandTimeMin: float
    mfLandTimeMax: float
    mfLandIdleTimeMin: float
    mfLandIdleTimeMax: float
    mfLandWalkTimeMin: float
    mfLandWalkTimeMax: float
    mfMaxSpeed: float
    mfAlignTime: float
    mfBankingTime: float
    mfSpringSmoothTime: float
    mbApplyScaleToSpeed: bool

class cGcFlyingSnakeData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfCircleSpeed", ctypes.c_float),
        ("mfApproachBaitSpeed", ctypes.c_float),
        ("mfDefaultCircleDistance", ctypes.c_float),
        ("mfBarrelRollSpawnDelay", ctypes.c_float),
        ("mfBarrelRollCount", ctypes.c_float),
        ("mfBarrelRollSpeed", ctypes.c_float),
        ("mfRiseDelay", ctypes.c_float),
        ("mfRiseHeight", ctypes.c_float),
        ("mfRiseTime", ctypes.c_float),
        ("mfAirThickness", ctypes.c_float),
        ("mfWindForce", ctypes.c_float),
        ("mfTailStiffness", ctypes.c_float),
        ("mfTwistLimit", ctypes.c_float),
        ("mfAltitudeSinPeriod", ctypes.c_float),
        ("mfAltitudeSinAmp", ctypes.c_float),
        ("mfAscendDescendSpeed", ctypes.c_float),
    ]
    mfCircleSpeed: float
    mfApproachBaitSpeed: float
    mfDefaultCircleDistance: float
    mfBarrelRollSpawnDelay: float
    mfBarrelRollCount: float
    mfBarrelRollSpeed: float
    mfRiseDelay: float
    mfRiseHeight: float
    mfRiseTime: float
    mfAirThickness: float
    mfWindForce: float
    mfTailStiffness: float
    mfTwistLimit: float
    mfAltitudeSinPeriod: float
    mfAltitudeSinAmp: float
    mfAscendDescendSpeed: float

class cGcCreatureFlockMovementData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miMinFlockMembers", ctypes.c_int32),
        ("miMaxFlockMembers", ctypes.c_int32),
        ("mfFlockFollow", ctypes.c_float),
        ("mfFlockHysteresis", ctypes.c_float),
        ("mfFlockCohere", ctypes.c_float),
        ("mfFlockSeperate", ctypes.c_float),
        ("mfFlockSeperateMinDist", ctypes.c_float),
        ("mfFlockSeperateMaxDist", ctypes.c_float),
        ("mfFlockAlign", ctypes.c_float),
        ("mfFlockAvoidTerrain", ctypes.c_float),
        ("mfFlockAvoidTerrainMinDist", ctypes.c_float),
        ("mfFlockAvoidTerrainMaxDist", ctypes.c_float),
        ("mfFlockAvoidPredators", ctypes.c_float),
        ("mfFlockAvoidPredatorsMinDist", ctypes.c_float),
        ("mfFlockAvoidPredatorsMaxDist", ctypes.c_float),
        ("mfFlockAvoidPredatorsSpeedBoost", ctypes.c_float),
        ("mfMoveInFacingStrength", ctypes.c_float),
        ("mfFlockMoveSpeed", ctypes.c_float),
        ("mfFlockMoveDirectionTime", ctypes.c_float),
        ("mfFlockTurnAngle", ctypes.c_float),
        ("mfMaxBank", ctypes.c_float),
        ("mfBankTime", ctypes.c_float),
        ("mfMinFlapSpeed", ctypes.c_float),
        ("mfMaxFlapSpeed", ctypes.c_float),
    ]
    miMinFlockMembers: int
    miMaxFlockMembers: int
    mfFlockFollow: float
    mfFlockHysteresis: float
    mfFlockCohere: float
    mfFlockSeperate: float
    mfFlockSeperateMinDist: float
    mfFlockSeperateMaxDist: float
    mfFlockAlign: float
    mfFlockAvoidTerrain: float
    mfFlockAvoidTerrainMinDist: float
    mfFlockAvoidTerrainMaxDist: float
    mfFlockAvoidPredators: float
    mfFlockAvoidPredatorsMinDist: float
    mfFlockAvoidPredatorsMaxDist: float
    mfFlockAvoidPredatorsSpeedBoost: float
    mfMoveInFacingStrength: float
    mfFlockMoveSpeed: float
    mfFlockMoveDirectionTime: float
    mfFlockTurnAngle: float
    mfMaxBank: float
    mfBankTime: float
    mfMinFlapSpeed: float
    mfMaxFlapSpeed: float

class cGcCreatureFootParticleData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maParticleData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureFootParticleSingleData>
    ]
    maParticleData: bytes

class cGcCreatureHoverTintableEffect(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macEffectNode", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mfLightStrength", ctypes.c_float),
        ("mfTintStrength", ctypes.c_float),
        ("mTintColour", common.Colour),
    ]
    macEffectNode: bytes
    mfLightStrength: float
    mfTintStrength: float
    mTintColour: common.Colour

class cGcCreatureHoverMovementDataParams(ctypes.Structure):
    def tilde_cGcCreatureHoverMovementDataParams(): pass

    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maValidDescriptors", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
        ("mfNavOffsetY", ctypes.c_float),
        ("mfNavOffsetZ", ctypes.c_float),
        ("mfGroundHeightOffset", ctypes.c_float),
        ("macGroundEffect", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mfGroundEffectHeightForMaxAlpha", ctypes.c_float),
        ("mfGroundEffectHeightForMinAlpha", ctypes.c_float),
        ("mfRayCastUp", ctypes.c_float),
        ("mfRayCastDown", ctypes.c_float),
        ("mbGroundAlign", ctypes.c_ubyte),
        ("mfHeightForMaxGroundAlign", ctypes.c_float),
        ("mfHeightForMinGroundAlign", ctypes.c_float),
        ("mfGroundAlignTimeModifier", ctypes.c_float),
        ("mbGroundAvoid", ctypes.c_ubyte),
        ("mfHeightForMaxGroundAvoid", ctypes.c_float),
        ("mfHeightForMinGroundAvoid", ctypes.c_float),
        ("mfGroundAvoidTimeModifier", ctypes.c_float),
        ("mbCanJump", ctypes.c_ubyte),
        ("maTintableEffects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureHoverTintableEffect>
    ]
    maValidDescriptors: bytes
    mfNavOffsetY: float
    mfNavOffsetZ: float
    mfGroundHeightOffset: float
    macGroundEffect: bytes
    mfGroundEffectHeightForMaxAlpha: float
    mfGroundEffectHeightForMinAlpha: float
    mfRayCastUp: float
    mfRayCastDown: float
    mbGroundAlign: bool
    mfHeightForMaxGroundAlign: float
    mfHeightForMinGroundAlign: float
    mfGroundAlignTimeModifier: float
    mbGroundAvoid: bool
    mfHeightForMaxGroundAvoid: float
    mfHeightForMinGroundAvoid: float
    mfGroundAvoidTimeModifier: float
    mbCanJump: bool
    maTintableEffects: bytes

class cGcCreatureHoverMovementData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maParams", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureHoverMovementDataParams>
    ]
    maParams: bytes

class cGcCreatureMoveAnimData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mAnim", ctypes.c_ulonglong),
        ("mAnimLeft", ctypes.c_ulonglong),
        ("mAnimRight", ctypes.c_ulonglong),
        ("mfAnimSpeed", ctypes.c_float),
        ("mfMinSpeedScale", ctypes.c_float),
        ("mfMaxSpeedScale", ctypes.c_float),
        ("mfMaxPredatorSpeedScale", ctypes.c_float),
        ("mfMinPetSpeedScale", ctypes.c_float),
        ("mfMaxPetSpeedScale", ctypes.c_float),
        ("mfAnimMoveSpeed", ctypes.c_float),
        ("mbAnimMoveSpeedCached", ctypes.c_ubyte),
    ]
    mAnim: int
    mAnimLeft: int
    mAnimRight: int
    mfAnimSpeed: float
    mfMinSpeedScale: float
    mfMaxSpeedScale: float
    mfMaxPredatorSpeedScale: float
    mfMinPetSpeedScale: float
    mfMaxPetSpeedScale: float
    mfAnimMoveSpeed: float
    mbAnimMoveSpeedCached: bool

class cGcCreatureMovementData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maAnims", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureMoveAnimData>
        ("mfMoveRange", ctypes.c_float),
        ("mfMoveSpeedScale", ctypes.c_float),
        ("mfTurnRadiusScale", ctypes.c_float),
        ("mfHeightMin", ctypes.c_float),
        ("mfHeightMax", ctypes.c_float),
        ("mfHeightTime", ctypes.c_float),
        ("mbHerd", ctypes.c_ubyte),
    ]
    maAnims: bytes
    mfMoveRange: float
    mfMoveSpeedScale: float
    mfTurnRadiusScale: float
    mfHeightMin: float
    mfHeightMax: float
    mfHeightTime: float
    mbHerd: bool

class cGcCreaturePetAccessorySlot(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macAttachLocator", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mAccessoryGroup", ctypes.c_ulonglong),
    ]
    macAttachLocator: bytes
    mAccessoryGroup: int

class cGcCreaturePetAccessory(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mRequiredDescriptor", ctypes.c_ulonglong),
        ("maSlots", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreaturePetAccessorySlot>
        ("maHideParts", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
    ]
    mRequiredDescriptor: int
    maSlots: bytes
    maHideParts: bytes

class cGcCreaturePetData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maAccessorySlots", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreaturePetAccessory>
    ]
    maAccessorySlots: bytes

class cGcCreaturePetPartHider(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macAccessorySlot", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("maPartName", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
    ]
    macAccessorySlot: bytes
    maPartName: bytes

class cGcCustomisationCameraData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miInteractionCameraIndex", ctypes.c_int32),
        ("macInteracttionCameraFocusJoint", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mfMinPitch", ctypes.c_float),
        ("mfMaxPitch", ctypes.c_float),
        ("mfMinYaw", ctypes.c_float),
        ("mfMaxYaw", ctypes.c_float),
    ]
    miInteractionCameraIndex: int
    macInteracttionCameraFocusJoint: bytes
    mfMinPitch: float
    mfMaxPitch: float
    mfMinYaw: float
    mfMaxYaw: float

class cGcCustomisationTextureGroup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mGroupID", ctypes.c_ulonglong),
        ("mTitle", ctypes.c_ulonglong),
        ("mTextureOptionGroup", ctypes.c_ulonglong),
    ]
    mGroupID: int
    mTitle: int
    mTextureOptionGroup: int

class cGcCustomisationBoneScales(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGroupTitle", ctypes.c_ulonglong),
        ("mScaleBoneName", ctypes.c_ulonglong),
        ("mafPositions", ctypes.c_char * 0x10),  # cTkDynamicArray<float>
    ]
    mGroupTitle: int
    mScaleBoneName: int
    mafPositions: bytes

class cGcAnimFrameEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mAnim", ctypes.c_ulonglong),
        ("miFrameStart", ctypes.c_int32),
        ("mbStartFromEnd", ctypes.c_ubyte),
    ]
    mAnim: int
    miFrameStart: int
    mbStartFromEnd: bool

class cGcCustomisationDescriptorGroupOption(ctypes.Structure):
    def tilde_cGcCustomisationDescriptorGroupOption(): pass

    def vector_deleting_destructor(): pass

    def cGcCustomisationDescriptorGroupOption(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mDescriptorOption", ctypes.c_ulonglong),
        ("maTextureGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationTextureGroup>
        ("mbReplaceBaseColours", ctypes.c_ubyte),
        ("maColourGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationColourGroup>
        ("mbReplaceBaseBoneScales", ctypes.c_ubyte),
        ("maBoneScales", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationBoneScales>
        ("miInteractionCameraIndexOverride", ctypes.c_int32),
        ("macInteracttionCameraFocusJointOverride", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("maHideIfGroupActive", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maSelectingRemovesGroup", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maSelectingAddsGroup", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maUnselectingRemovesGroup", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maUnselectingAddsGroup", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mDescriptorOption: int
    maTextureGroups: bytes
    mbReplaceBaseColours: bool
    maColourGroups: bytes
    mbReplaceBaseBoneScales: bool
    maBoneScales: bytes
    miInteractionCameraIndexOverride: int
    macInteracttionCameraFocusJointOverride: bytes
    maHideIfGroupActive: bytes
    maSelectingRemovesGroup: bytes
    maSelectingAddsGroup: bytes
    maUnselectingRemovesGroup: bytes
    maUnselectingAddsGroup: bytes

class cGcCreatureNearbyEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maAlertTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureAlertData>
        ("mbInverse", ctypes.c_ubyte),
    ]
    maAlertTable: bytes
    mbInverse: bool

class cGcStateTimeEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfSeconds", ctypes.c_float),
        ("mfRandomSeconds", ctypes.c_float),
        ("mbUseMissionClock", ctypes.c_ubyte),
    ]
    mfSeconds: float
    mfRandomSeconds: float
    mbUseMissionClock: bool

class cGcStormEvent(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbInStorm", ctypes.c_ubyte),
    ]
    mbInStorm: bool

class cGcCustomisationBackpackData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macNodeName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mActiveJetOffset", common.Vector3f),
    ]
    macNodeName: bytes
    mActiveJetOffset: common.Vector3f

class cGcCustomisationColourPaletteExtraData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maProductToUnlock", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maTipText", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
    ]
    maProductToUnlock: bytes
    maTipText: bytes

class cGcCustomisationDescriptorGroupOptions(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGroupTitle", ctypes.c_ulonglong),
        ("mbFirstOptionIsEmpty", ctypes.c_ubyte),
        ("maDescriptorGroupOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationDescriptorGroupOption>
        ("maPrerequisiteGroup", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mGroupTitle: int
    mbFirstOptionIsEmpty: bool
    maDescriptorGroupOptions: bytes
    maPrerequisiteGroup: bytes

class cGcCustomisationRace(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mDescriptorGroupOption", ctypes.c_ulonglong),
        ("mbIsGek", ctypes.c_ubyte),
        ("mfScale", ctypes.c_float),
        ("maCustomisationGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationGroup>
        ("maPresets", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mDescriptorGroupOption: int
    mbIsGek: bool
    mfScale: float
    maCustomisationGroups: bytes
    maPresets: bytes

class cGcCustomisationShipBobbleHeads(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBobbleHeads", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationBobbleHead>
    ]
    maBobbleHeads: bytes

class cGcInteractionDof(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbIsEnabled", ctypes.c_ubyte),
        ("mbUseGlobals", ctypes.c_ubyte),
        ("mfNearPlaneMin", ctypes.c_float),
        ("mfNearPlaneAdjust", ctypes.c_float),
        ("mfFarPlane", ctypes.c_float),
        ("mfFarFadeDistance", ctypes.c_float),
    ]
    mbIsEnabled: bool
    mbUseGlobals: bool
    mfNearPlaneMin: float
    mfNearPlaneAdjust: float
    mfFarPlane: float
    mfFarFadeDistance: float

class cGcCustomisationThrusterEffect(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mLinkedSpecialID", ctypes.c_ulonglong),
        ("mTip", ctypes.c_ulonglong),
        ("mbAllowedInSeasonalDefaults", ctypes.c_ubyte),
        ("mLightColour", common.Colour),
        ("maJets", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationThrusterJet>
    ]
    mName: int
    mLinkedSpecialID: int
    mTip: int
    mbAllowedInSeasonalDefaults: bool
    mLightColour: common.Colour
    maJets: bytes

class cGcCustomisationGroups(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maCustomisationGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationGroup>
    ]
    maCustomisationGroups: bytes

class cGcCustomisationThrusterEffects(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maJetpackEffects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationThrusterEffect>
        ("maBackpackData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationBackpackData>
        ("maShipEffects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationShipTrails>
        ("maFreighterEngineEffects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationFreighterEngineEffect>
    ]
    maJetpackEffects: bytes
    maBackpackData: bytes
    maShipEffects: bytes
    maFreighterEngineEffects: bytes

class cGcCustomisationUIData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maCustomisationUIData", ctypes.c_char * 0x7E8),  # cTkFixedArray<cGcCustomisationUI,23>
    ]
    maCustomisationUIData: bytes

class cGcBehaviourAppearData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mAppearAnim", ctypes.c_ulonglong),
    ]
    mAppearAnim: int

class cGcBehaviourCheckDeathData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcBehaviourCooldownBeginData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mKey", ctypes.c_ulonglong),
    ]
    mKey: int

class cGcBehaviourFaceTargetData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTargetKey", ctypes.c_ulonglong),
        ("mfArriveAngle", ctypes.c_float),
    ]
    mTargetKey: int
    mfArriveAngle: float

class cGcBehaviourGetTargetData(ctypes.Structure):
    def WriteToXMLNode(): pass

    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mTargetKey", ctypes.c_ulonglong),
    ]
    mTargetKey: int

class cGcCreatureAudioTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureVocalSoundData>
    ]
    maTable: bytes

class cGcBehaviourIdleData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcBehaviourIncrementCounter(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mKey", ctypes.c_ulonglong),
    ]
    mKey: int

class cGcLadderComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCreatureBehaviourTreeData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maNodes", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
    ]
    mId: int
    maNodes: bytes

class cGcCreatureBehaviourTrees(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBehaviourTree", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureBehaviourTreeData>
    ]
    maBehaviourTree: bytes

class cGcCreatureDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureData>
    ]
    maTable: bytes

class cGcCreatureFilenameTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureFilename>
    ]
    maTable: bytes

class cGcCreatureFoodList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mFoodProduct", ctypes.c_ulonglong),
        ("macResourceFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mDebrisEffect", ctypes.c_ulonglong),
    ]
    mFoodProduct: int
    macResourceFile: bytes
    mDebrisEffect: int

class cGcCharacterAlternateAnimation(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mAnim", ctypes.c_ulonglong),
        ("mReplacement", ctypes.c_ulonglong),
    ]
    mAnim: int
    mReplacement: int

class cGcActionTriggerState(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mStateID", ctypes.c_ulonglong),
        ("maTriggers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcActionTrigger>
    ]
    mStateID: int
    maTriggers: bytes

class cGcCreatureHarvestSubstanceList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCreatureType", ctypes.c_ulonglong),
        ("mItem", ctypes.c_ulonglong),
        ("macDesc", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miMinBlobs", ctypes.c_int32),
    ]
    mCreatureType: int
    mItem: int
    macDesc: bytes
    miMinBlobs: int

class cGcRocketLockerComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miNumSlots", ctypes.c_int32),
    ]
    miNumSlots: int

class cGcSentinelCoverComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfHealthPercLostPerSecMin", ctypes.c_float),
        ("mfHealthPercLostPerSecMax", ctypes.c_float),
        ("mSpawnEffectId", ctypes.c_ulonglong),
        ("mDestroyEffectId", ctypes.c_ulonglong),
        ("macEffectLocator", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("maCoverStateAnims", ctypes.c_char * 0x40),  # cTkFixedArray<TkID<128>,4>
    ]
    mfHealthPercLostPerSecMin: float
    mfHealthPercLostPerSecMax: float
    mSpawnEffectId: int
    mDestroyEffectId: int
    macEffectLocator: bytes
    maCoverStateAnims: bytes

class cGcCreatureStupidName(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("miCount", ctypes.c_int32),
        ("maacNames", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    mId: int
    miCount: int
    maacNames: bytes

class cGcSquadronHologramComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfHologramRotationSpeedDegPerSec", ctypes.c_float),
        ("mfPilotScale", ctypes.c_float),
        ("mSpawnOffset", common.Vector3f),
        ("mfSpawnRotation", ctypes.c_float),
    ]
    mfHologramRotationSpeedDegPerSec: float
    mfPilotScale: float
    mSpawnOffset: common.Vector3f
    mfSpawnRotation: float

class cGcBaseDefenceTrigger(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPerceptionId", ctypes.c_ulonglong),
        ("mbActiveWhenIdle", ctypes.c_ubyte),
        ("mbActiveWhenTargetAcquired", ctypes.c_ubyte),
        ("mbActiveWhenSearching", ctypes.c_ubyte),
        ("mLaserEffectId", ctypes.c_ulonglong),
    ]
    mPerceptionId: int
    mbActiveWhenIdle: bool
    mbActiveWhenTargetAcquired: bool
    mbActiveWhenSearching: bool
    mLaserEffectId: int

class cGcCreatureStupidNameTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macStupidUserName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCreatureStupidName>
    ]
    macStupidUserName: bytes
    maTable: bytes

class cGcCreatureSubstanceList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCreatureType", ctypes.c_ulonglong),
        ("mItem", ctypes.c_ulonglong),
    ]
    mCreatureType: int
    mItem: int

class cGcCameraShakeComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mShakeID", ctypes.c_ulonglong),
        ("mfFalloffDistanceMin", ctypes.c_float),
        ("mfFalloffDistanceMax", ctypes.c_float),
    ]
    mShakeID: int
    mfFalloffDistanceMin: float
    mfFalloffDistanceMax: float

class cGcCameraShakeTriggerData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mAnim", ctypes.c_ulonglong),
        ("miFrameStart", ctypes.c_int32),
        ("macShake", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
    ]
    mAnim: int
    miFrameStart: int
    macShake: bytes

class cGcChainComponentData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macStartBone", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
    ]
    macStartBone: bytes

class cGcCreatureVocalTestData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("macGenus", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mfSize", ctypes.c_float),
        ("mfSquawk", ctypes.c_float),
    ]
    mId: int
    macGenus: bytes
    mfSize: float
    mfSquawk: float

class cGcCharacterInterfaceComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcColouriseComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPrimaryColour", common.Colour),
        ("mSecondaryColour", common.Colour),
        ("mTernaryColour", common.Colour),
        ("mQuaternaryColour", common.Colour),
    ]
    mPrimaryColour: common.Colour
    mSecondaryColour: common.Colour
    mTernaryColour: common.Colour
    mQuaternaryColour: common.Colour

class cGcCustomInventoryComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mSize", ctypes.c_ulonglong),
        ("maDesiredTechs", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInventoryTechProbability>
        ("mbCool", ctypes.c_ubyte),
    ]
    mSize: int
    maDesiredTechs: bytes
    mbCool: bool

class cGcDecorationComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfStartOffset", ctypes.c_float),
        ("mfMaxTestRange", ctypes.c_float),
    ]
    mfStartOffset: float
    mfMaxTestRange: float

class cGcMetaBallComponentData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macRoot", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mfRadius", ctypes.c_float),
        ("mMinSize", common.Vector3f),
        ("mMaxSize", common.Vector3f),
    ]
    macFile: bytes
    macRoot: bytes
    mfRadius: float
    mMinSize: common.Vector3f
    mMaxSize: common.Vector3f

class cGcMultitoolPoolData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miMinDraw", ctypes.c_int32),
        ("miMaxDraw", ctypes.c_int32),
        ("mfPoolProbability", ctypes.c_float),
        ("mbPoolIsSecret", ctypes.c_ubyte),
    ]
    macFile: bytes
    miMinDraw: int
    miMaxDraw: int
    mfPoolProbability: float
    mbPoolIsSecret: bool

class cGcCharacterJetpackEffect(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macNodeName", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mEffect", ctypes.c_ulonglong),
    ]
    macNodeName: bytes
    mEffect: int

class cGcPlayerCharacterComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maJetpackEffects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCharacterJetpackEffect>
        ("mIntialPlayerControlMode", ctypes.c_ulonglong),
        ("maPlayerControlModes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerControlModeEntry>
    ]
    maJetpackEffects: bytes
    mIntialPlayerControlMode: int
    maPlayerControlModes: bytes

class cGcPlayerControlInput(ctypes.Structure):
    def SetDefaults(): pass

    def cGcPlayerControlInput(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbInterceptAllInputs", ctypes.c_ubyte),
        ("maInterceptInputWhitelist", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInputActions>
        ("maInterceptInputBlackList", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInputActions>
        ("maInputs", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
    ]
    mbInterceptAllInputs: bool
    maInterceptInputWhitelist: bytes
    maInterceptInputBlackList: bytes
    maInputs: bytes

class cGcTriggerActionComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbHideModel", ctypes.c_ubyte),
        ("mbStartInactive", ctypes.c_ubyte),
        ("maStates", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcActionTriggerState>
        ("mbPersistent", ctypes.c_ubyte),
        ("mPersistentState", ctypes.c_ulonglong),
        ("mbResetShotTimeOnStateChange", ctypes.c_ubyte),
        ("mbLinkStateToBaseGrid", ctypes.c_ubyte),
    ]
    mbHideModel: bool
    mbStartInactive: bool
    maStates: bytes
    mbPersistent: bool
    mPersistentState: int
    mbResetShotTimeOnStateChange: bool
    mbLinkStateToBaseGrid: bool

class cGcMiningComponentData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfRange", ctypes.c_float),
        ("mfSpeed", ctypes.c_float),
    ]
    mfRange: float
    mfSpeed: float

class cGcNetworkPlayerMarkerComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcOutpostLSystemPair(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macLocator", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("maacLSystems", ctypes.c_char * 0x400),  # cTkFixedArray<cTkFixedString<128,char>,8>
    ]
    macLocator: bytes
    maacLSystems: bytes

class cGcOutpostComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbAnomaly", ctypes.c_ubyte),
        ("mbFrigate", ctypes.c_ubyte),
        ("mbSpaceStation", ctypes.c_ubyte),
        ("mbNexusExterior", ctypes.c_ubyte),
        ("mbNexusInterior", ctypes.c_ubyte),
        ("mbAIDestination", ctypes.c_ubyte),
        ("mbHasDoors", ctypes.c_ubyte),
        ("mbHasOwnGravity", ctypes.c_ubyte),
        ("mDoor", ctypes.c_ulonglong),
        ("mbCheckLandingAreaClear", ctypes.c_ubyte),
        ("mfCircleRadius", ctypes.c_float),
        ("mfApproachRange", ctypes.c_float),
        ("mfApproachAngle", ctypes.c_float),
        ("mfApproachSpeed", ctypes.c_float),
        ("mfPlayerAutoLandRange", ctypes.c_float),
        ("mfLandingSpeed", ctypes.c_float),
        ("mfLandingHeight", ctypes.c_float),
        ("mbRotateToDock", ctypes.c_ubyte),
        ("mfTakeOffHeight", ctypes.c_float),
        ("mfTakeOffFwdDist", ctypes.c_float),
        ("mfTakeOffTime", ctypes.c_float),
        ("mfTakeOffAlignTime", ctypes.c_float),
        ("mfTakeOffSpeed", ctypes.c_float),
        ("mfTakeOffBoost", ctypes.c_float),
        ("mfTakeOffExtraAIHeight", ctypes.c_float),
        ("mfPostTakeOffExtraPlayerHeight", ctypes.c_float),
        ("mfPostTakeOffExtraPlayerSpeed", ctypes.c_float),
        ("mfTakeOffProgressForExtraHeight", ctypes.c_float),
        ("maLSystems", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcOutpostLSystemPair>
    ]
    mbAnomaly: bool
    mbFrigate: bool
    mbSpaceStation: bool
    mbNexusExterior: bool
    mbNexusInterior: bool
    mbAIDestination: bool
    mbHasDoors: bool
    mbHasOwnGravity: bool
    mDoor: int
    mbCheckLandingAreaClear: bool
    mfCircleRadius: float
    mfApproachRange: float
    mfApproachAngle: float
    mfApproachSpeed: float
    mfPlayerAutoLandRange: float
    mfLandingSpeed: float
    mfLandingHeight: float
    mbRotateToDock: bool
    mfTakeOffHeight: float
    mfTakeOffFwdDist: float
    mfTakeOffTime: float
    mfTakeOffAlignTime: float
    mfTakeOffSpeed: float
    mfTakeOffBoost: float
    mfTakeOffExtraAIHeight: float
    mfPostTakeOffExtraPlayerHeight: float
    mfPostTakeOffExtraPlayerSpeed: float
    mfTakeOffProgressForExtraHeight: float
    maLSystems: bytes

class cGcRectangularInitialShape(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTopLineStart", common.Vector3f),
        ("mTopLineEnd", common.Vector3f),
        ("mBottomLineStart", common.Vector3f),
        ("mBottomLineEnd", common.Vector3f),
    ]
    mTopLineStart: common.Vector3f
    mTopLineEnd: common.Vector3f
    mBottomLineStart: common.Vector3f
    mBottomLineEnd: common.Vector3f

class cGcAttachmentPointData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPosition", common.Vector3f),
        ("miSimP", ctypes.c_int32),
        ("mfAttractionStrength", ctypes.c_float),
        ("mfAttractionStartDistance", ctypes.c_float),
    ]
    mPosition: common.Vector3f
    miSimP: int
    mfAttractionStrength: float
    mfAttractionStartDistance: float

class cGcAttachmentPointSet(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
        ("macDebugName", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macBoneName", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("maFixedPoints", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcAttachmentPointData>
    ]
    mbEnabled: bool
    macDebugName: bytes
    macBoneName: bytes
    maFixedPoints: bytes

class cGcConstraintsToCreateSpec(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbHorizontal_1x0", ctypes.c_ubyte),
        ("mfPushingStrength_Horizontal_1x0", ctypes.c_float),
        ("mbVertical_1x0", ctypes.c_ubyte),
        ("mfPushingStrength_Vertical_1x0", ctypes.c_float),
        ("mbDiagonal_1x1_0011", ctypes.c_ubyte),
        ("mfPushingStrength_Diagonal_1x1_0011", ctypes.c_float),
        ("mbDiagonal_1x1_0110", ctypes.c_ubyte),
        ("mfPushingStrength_Diagonal_1x1_0110", ctypes.c_float),
        ("mbHorizontal_2x0", ctypes.c_ubyte),
        ("mfPushingStrength_Horizontal_2x0", ctypes.c_float),
        ("mbVertical_2x0", ctypes.c_ubyte),
        ("mfPushingStrength_Vertical_2x0", ctypes.c_float),
        ("mbSkewedDiagonal_2x1_0021", ctypes.c_ubyte),
        ("mfPushingStrength_SkewedDiagonal_2x1_0021", ctypes.c_float),
        ("mbSkewedDiagonal_2x1_2001", ctypes.c_ubyte),
        ("mfPushingStrength_SkewedDiagonal_2x1_2001", ctypes.c_float),
        ("mbSkewedDiagonal_2x1_1002", ctypes.c_ubyte),
        ("mfPushingStrength_SkewedDiagonal_2x1_1002", ctypes.c_float),
        ("mbSkewedDiagonal_2x1_0012", ctypes.c_ubyte),
        ("mfPushingStrength_SkewedDiagonal_2x1_0012", ctypes.c_float),
    ]
    mbHorizontal_1x0: bool
    mfPushingStrength_Horizontal_1x0: float
    mbVertical_1x0: bool
    mfPushingStrength_Vertical_1x0: float
    mbDiagonal_1x1_0011: bool
    mfPushingStrength_Diagonal_1x1_0011: float
    mbDiagonal_1x1_0110: bool
    mfPushingStrength_Diagonal_1x1_0110: float
    mbHorizontal_2x0: bool
    mfPushingStrength_Horizontal_2x0: float
    mbVertical_2x0: bool
    mfPushingStrength_Vertical_2x0: float
    mbSkewedDiagonal_2x1_0021: bool
    mfPushingStrength_SkewedDiagonal_2x1_0021: float
    mbSkewedDiagonal_2x1_2001: bool
    mfPushingStrength_SkewedDiagonal_2x1_2001: float
    mbSkewedDiagonal_2x1_1002: bool
    mfPushingStrength_SkewedDiagonal_2x1_1002: float
    mbSkewedDiagonal_2x1_0012: bool
    mfPushingStrength_SkewedDiagonal_2x1_0012: float

class cGcAdvancedTweaks(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfParticleKillSpeed", ctypes.c_float),
        ("mfParticleKillSpeedWrtFixed", ctypes.c_float),
        ("mfEdgeMultiplierForTangentI", ctypes.c_float),
        ("mfEdgeMultiplierForTangentJ", ctypes.c_float),
        ("mfRenderNormalMultiplier", ctypes.c_float),
        ("mfStretchUvsToHideTextureEdges", ctypes.c_float),
    ]
    mfParticleKillSpeed: float
    mfParticleKillSpeedWrtFixed: float
    mfEdgeMultiplierForTangentI: float
    mfEdgeMultiplierForTangentJ: float
    mfRenderNormalMultiplier: float
    mfStretchUvsToHideTextureEdges: float

class cGcAttachedNode(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
        ("macNodeName", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mfMinRenderIFraction", ctypes.c_float),
        ("mfMinRenderJFraction", ctypes.c_float),
        ("mfMaxRenderIFraction", ctypes.c_float),
        ("mfMaxRenderJFraction", ctypes.c_float),
        ("mfBlendStrength", ctypes.c_float),
        ("mRelativeTransform_Axis0", common.Vector3f),
        ("mRelativeTransform_Axis1", common.Vector3f),
        ("mRelativeTransform_Axis2", common.Vector3f),
        ("mRelativeTransform_Position", common.Vector3f),
    ]
    mbEnabled: bool
    macNodeName: bytes
    mfMinRenderIFraction: float
    mfMinRenderJFraction: float
    mfMaxRenderIFraction: float
    mfMaxRenderJFraction: float
    mfBlendStrength: float
    mRelativeTransform_Axis0: common.Vector3f
    mRelativeTransform_Axis1: common.Vector3f
    mRelativeTransform_Axis2: common.Vector3f
    mRelativeTransform_Position: common.Vector3f

class cGcClothComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
        ("miNumConstraintSolvingIterations", ctypes.c_int32),
        ("maClothPieces", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcClothPiece>
        ("mfClothScale", ctypes.c_float),
        ("mfCollisionScale", ctypes.c_float),
        ("mfAttachmentScale", ctypes.c_float),
    ]
    mbEnabled: bool
    miNumConstraintSolvingIterations: int
    maClothPieces: bytes
    mfClothScale: float
    mfCollisionScale: float
    mfAttachmentScale: float

class cGcCharacterLookAtData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfInteractionLookAtRadius", ctypes.c_float),
        ("mfCreatureLookAtRadius", ctypes.c_float),
        ("mfSpaceshipLookAtRadius", ctypes.c_float),
        ("mfLookAtRunGlanceMinTime", ctypes.c_float),
        ("mfLookAtRunGlanceMaxTime", ctypes.c_float),
        ("mfLookAtRunMinTime", ctypes.c_float),
        ("mfLookAtRunMaxTime", ctypes.c_float),
        ("mfLookAtTargetGlanceMinTime", ctypes.c_float),
        ("mfLookAtTargetGlanceMaxTime", ctypes.c_float),
        ("mfLookAtTargetWaitMinTime", ctypes.c_float),
        ("mfLookAtTargetWaitMaxTime", ctypes.c_float),
        ("mfLookAtMaxYaw", ctypes.c_float),
        ("mfLookAtMaxPitch", ctypes.c_float),
    ]
    mfInteractionLookAtRadius: float
    mfCreatureLookAtRadius: float
    mfSpaceshipLookAtRadius: float
    mfLookAtRunGlanceMinTime: float
    mfLookAtRunGlanceMaxTime: float
    mfLookAtRunMinTime: float
    mfLookAtRunMaxTime: float
    mfLookAtTargetGlanceMinTime: float
    mfLookAtTargetGlanceMaxTime: float
    mfLookAtTargetWaitMinTime: float
    mfLookAtTargetWaitMaxTime: float
    mfLookAtMaxYaw: float
    mfLookAtMaxPitch: float

class cGcPlayerWeaponComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcSpringAttachmentComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
        ("macName", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("maSpringLinks", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSpringLink>
    ]
    mbEnabled: bool
    macName: bytes
    maSpringLinks: bytes

class cGcScanEffectComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maScanEffects", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcScanEffectData>
        ("macNodeName", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    maScanEffects: bytes
    macNodeName: bytes

class cGcRewardTableCategory(ctypes.Structure):
    def tilde_cGcRewardTableCategory(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSizes", ctypes.c_char * 0x48),  # cTkFixedArray<cGcRewardTableItemList,3>
    ]
    maSizes: bytes

class cGcRewardTableEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def vector_deleting_destructor(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maRarities", ctypes.c_char * 0xD8),  # cTkFixedArray<cGcRewardTableCategory,3>
    ]
    mId: int
    maRarities: bytes

class cGcRewardTimeWarp(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardTraderFlyby(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miExperienceSpawnIndex", ctypes.c_int32),
    ]
    miExperienceSpawnIndex: int

class cGcRewardTrigger(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTrigger", ctypes.c_ulonglong),
    ]
    mTrigger: int

class cGcRewardTriggerSettlementJudgement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardTriggerStorm(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDuration", ctypes.c_float),
    ]
    mfDuration: float

class cGcRewardUnlockSeasonReward(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mProductID", ctypes.c_ulonglong),
        ("mbSilent", ctypes.c_ubyte),
        ("mEncryptedText", ctypes.c_ulonglong),
        ("mbUseSpecialFormatting", ctypes.c_ubyte),
        ("mbMarkAsClaimedInShop", ctypes.c_ubyte),
        ("mbUniqueInventoryItem", ctypes.c_ubyte),
    ]
    mProductID: int
    mbSilent: bool
    mEncryptedText: int
    mbUseSpecialFormatting: bool
    mbMarkAsClaimedInShop: bool
    mbUniqueInventoryItem: bool

class cGcRewardUnlockTitle(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTitleID", ctypes.c_ulonglong),
        ("mbNoMusic", ctypes.c_ubyte),
        ("mbShowEvenIfAlreadyUnlocked", ctypes.c_ubyte),
    ]
    mTitleID: int
    mbNoMusic: bool
    mbShowEvenIfAlreadyUnlocked: bool

class cGcRewardUpgradeWeaponClass(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbSilent", ctypes.c_ubyte),
        ("mbMatchClassToCommunityTier", ctypes.c_ubyte),
    ]
    mbSilent: bool
    mbMatchClassToCommunityTier: bool

class cGcRewardUploadBase(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardWantedLevel(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMessage", ctypes.c_ulonglong),
        ("miLevel", ctypes.c_int32),
    ]
    mMessage: int
    miLevel: int

class cGcRewardWeapon(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miItemLevel", ctypes.c_int32),
        ("mbMarkInteractionComplete", ctypes.c_ubyte),
        ("mbUsePlanetSeed", ctypes.c_ubyte),
        ("mbForceFixed", ctypes.c_ubyte),
        ("mbRequeueInteraction", ctypes.c_ubyte),
        ("mbReinteractOnDecline", ctypes.c_ubyte),
        ("mbOnlyUseNextInteractionOnSuccess", ctypes.c_ubyte),
        ("mfSecretPoolChance", ctypes.c_float),
    ]
    miItemLevel: int
    mbMarkInteractionComplete: bool
    mbUsePlanetSeed: bool
    mbForceFixed: bool
    mbRequeueInteraction: bool
    mbReinteractOnDecline: bool
    mbOnlyUseNextInteractionOnSuccess: bool
    mfSecretPoolChance: float

class cGcRewardWeaponSlot(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCost", ctypes.c_ulonglong),
        ("mbAwardCostAndOpenWindow", ctypes.c_ubyte),
        ("miNumTokens", ctypes.c_int32),
    ]
    mCost: int
    mbAwardCostAndOpenWindow: bool
    miNumTokens: int

class cGcRewardWikiTopic(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTopic", ctypes.c_ulonglong),
        ("mbCentreMessage", ctypes.c_ubyte),
    ]
    mTopic: int
    mbCentreMessage: bool

class cGcRewardDestructEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfHealthFactor", ctypes.c_float),
    ]
    mfHealthFactor: float

class cGcSettlementBuildingCost(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maStageCosts", ctypes.c_char * 0x120),  # cTkFixedArray<cGcSettlementBuildingCostData,6>
    ]
    maStageCosts: bytes

class cGcRewardDestructRarities(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maRarities", ctypes.c_char * 0xC),  # cTkFixedArray<cGcRewardDestructEntry,3>
    ]
    maRarities: bytes

class cGcSettlementJudgementPerkOption(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPerk", ctypes.c_ulonglong),
        ("mfPerkChance", ctypes.c_float),
    ]
    mPerk: int
    mfPerkChance: float

class cGcRewardDestructTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maCategories", ctypes.c_char * 0x6C),  # cTkFixedArray<cGcRewardDestructRarities,9>
    ]
    maCategories: bytes

class cGcSettlementJudgementOption(ctypes.Structure):
    def tilde_cGcSettlementJudgementOption(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    def cGcSettlementJudgementOption(): pass

    _fields_ = [
        ("mOptionText", ctypes.c_ulonglong),
        ("maPerks", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementJudgementPerkOption>
        ("mbHidePerkInJudgement", ctypes.c_ubyte),
        ("maStatChanges", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementStatChange>
        ("maAdditionalRewards", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("mChainedJudgementID", ctypes.c_ulonglong),
        ("mbUsePolicyPerk", ctypes.c_ubyte),
        ("mbUsePolicyStat", ctypes.c_ubyte),
        ("mbUseGiftReward", ctypes.c_ubyte),
        ("mbUseTechPerk", ctypes.c_ubyte),
        ("mbOptionIsPositiveForNPC", ctypes.c_ubyte),
    ]
    mOptionText: int
    maPerks: bytes
    mbHidePerkInJudgement: bool
    maStatChanges: bytes
    maAdditionalRewards: bytes
    mChainedJudgementID: int
    mbUsePolicyPerk: bool
    mbUsePolicyStat: bool
    mbUseGiftReward: bool
    mbUseTechPerk: bool
    mbOptionIsPositiveForNPC: bool

class cGcStatIconTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacStatIcons", ctypes.c_char * 0x5E80),  # cTkFixedArray<cTkFixedString<128,char>,189>
    ]
    maacStatIcons: bytes

class cGcInventoryValueData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfBaseMinValue", ctypes.c_float),
        ("mfBaseMaxValue", ctypes.c_float),
        ("mfExponentialValue", ctypes.c_float),
        ("mfBaseCostPerSlot", ctypes.c_float),
        ("mfSlotExponentialValue", ctypes.c_float),
        ("mfValueToCost", ctypes.c_float),
        ("mfSlotsPerLevel", ctypes.c_float),
    ]
    mfBaseMinValue: float
    mfBaseMaxValue: float
    mfExponentialValue: float
    mfBaseCostPerSlot: float
    mfSlotExponentialValue: float
    mfValueToCost: float
    mfSlotsPerLevel: float

class cGcValueData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcExactResource(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mGenerationSeed", common.GcSeed),
    ]
    macFilename: bytes
    mGenerationSeed: common.GcSeed

class cGcCameraAmbientBuildingData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mAnimation", ctypes.c_ulonglong),
        ("mDroneAnimation", ctypes.c_ulonglong),
        ("mfOffset", ctypes.c_float),
        ("mbUseLookAt", ctypes.c_ubyte),
        ("mbAvoidTerrain", ctypes.c_ubyte),
        ("mabAvailableBuildings", ctypes.c_char * 0x34),  # cTkFixedArray<bool,52>
        ("mabAvailableRaces", ctypes.c_char * 0x8),  # cTkFixedArray<bool,8>
    ]
    mAnimation: int
    mDroneAnimation: int
    mfOffset: float
    mbUseLookAt: bool
    mbAvoidTerrain: bool
    mabAvailableBuildings: bytes
    mabAvailableRaces: bytes

class cGcAmbientModeCameras(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBuildingCameraAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCameraAmbientBuildingData>
        ("maSpaceCameraAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCameraAmbientSpaceData>
        ("maSpecialCameraAnimations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCameraAmbientSpecialData>
    ]
    maBuildingCameraAnimations: bytes
    maSpaceCameraAnimations: bytes
    maSpecialCameraAnimations: bytes

class cGcSettlementGiftDetails(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mReward", ctypes.c_ulonglong),
        ("mLocID", ctypes.c_ulonglong),
    ]
    mReward: int
    mLocID: int

class cGcByteBeatPlayerComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcHologramComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcDissolveEffectComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDissolveBeginHeight", ctypes.c_float),
        ("mfDissolveEndHeight", ctypes.c_float),
    ]
    mfDissolveBeginHeight: float
    mfDissolveEndHeight: float

class cGcPlayerEffectsComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfVehicleInOutTime", ctypes.c_float),
        ("mfVehicleInOutEffectDelay", ctypes.c_float),
        ("mfVehicleInOutDissolveDelay", ctypes.c_float),
    ]
    mfVehicleInOutTime: float
    mfVehicleInOutEffectDelay: float
    mfVehicleInOutDissolveDelay: float

class cGcAntagonistFriend(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfArticulationFactor", ctypes.c_float),
        ("maPerceptions", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mfArticulationFactor: float
    maPerceptions: bytes

class cGcAntagonistEnemy(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfHatredFactor", ctypes.c_float),
        ("mfGrudgeFactor", ctypes.c_float),
        ("maPerceptions", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mfHatredFactor: float
    mfGrudgeFactor: float
    maPerceptions: bytes

class cGcChairComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCombatEffectsProperties(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbIsAffected", ctypes.c_ubyte),
        ("mbIgnoreFromOtherPlayers", ctypes.c_ubyte),
        ("mbIgnoreFromSelf", ctypes.c_ubyte),
        ("mfDurationMultiplier", ctypes.c_float),
        ("mfDamageMultiplier", ctypes.c_float),
    ]
    mbIsAffected: bool
    mbIgnoreFromOtherPlayers: bool
    mbIgnoreFromSelf: bool
    mfDurationMultiplier: float
    mfDamageMultiplier: float

class cGcCombatEffectsComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maEffectsProperties", ctypes.c_char * 0x30),  # cTkFixedArray<cGcCombatEffectsProperties,4>
    ]
    maEffectsProperties: bytes

class cGcEncounterStateComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcExpeditionHologramComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfHologramRotationSpeedDegPerSec", ctypes.c_float),
        ("mfCaptainScale", ctypes.c_float),
        ("mfFrigateScale", ctypes.c_float),
        ("mSpawnOffset", common.Vector3f),
    ]
    mfHologramRotationSpeedDegPerSec: float
    mfCaptainScale: float
    mfFrigateScale: float
    mSpawnOffset: common.Vector3f

class cGcFleetHologramComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcInteractionActivationCost(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mSubstanceId", ctypes.c_ulonglong),
        ("maAltIds", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("miCost", ctypes.c_int32),
        ("mbRepeat", ctypes.c_ubyte),
        ("mRequiredTech", ctypes.c_ulonglong),
        ("maiOnlyChargeDuringSeasons", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
    ]
    mSubstanceId: int
    maAltIds: bytes
    miCost: int
    mbRepeat: bool
    mRequiredTech: int
    maiOnlyChargeDuringSeasons: bytes

class cGcStoryUtilityOverride(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mReward", ctypes.c_ulonglong),
        ("maSpecificRewardOverrideTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcRewardMissionOverride>
    ]
    mName: int
    mReward: int
    maSpecificRewardOverrideTable: bytes

class cGcUniqueIdComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCameraShakeAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mShake", ctypes.c_ulonglong),
        ("mfFalloffMin", ctypes.c_float),
        ("mfFalloffMax", ctypes.c_float),
    ]
    mShake: int
    mfFalloffMin: float
    mfFalloffMax: float

class cGcDestroyAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbDestroyAll", ctypes.c_ubyte),
        ("mbUseDestructables", ctypes.c_ubyte),
        ("mPlayEffect", ctypes.c_ulonglong),
    ]
    mbDestroyAll: bool
    mbUseDestructables: bool
    mPlayEffect: int

class cGcFireSimpleInteractionAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcHarvestPlantAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfRadius", ctypes.c_float),
    ]
    mfRadius: float

class cGcLodAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miLodOverride", ctypes.c_int32),
    ]
    miLodOverride: int

class cGcPainAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mDamage", ctypes.c_ulonglong),
        ("mfRadius", ctypes.c_float),
        ("mbRadiusBasedDamage", ctypes.c_ubyte),
        ("mbAffectsPlayer", ctypes.c_ubyte),
        ("mfRetriggerTime", ctypes.c_float),
    ]
    mDamage: int
    mfRadius: float
    mbRadiusBasedDamage: bool
    mbAffectsPlayer: bool
    mfRetriggerTime: float

class cGcPlayAnimAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mAnim", ctypes.c_ulonglong),
    ]
    mAnim: int

class cGcPlayAudioAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macSound", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbUseOcclusion", ctypes.c_ubyte),
        ("mfOcclusionRadius", ctypes.c_float),
    ]
    macSound: bytes
    mbUseOcclusion: bool
    mfOcclusionRadius: float

class cGcPowerStateAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbSetRateEnabled", ctypes.c_ubyte),
        ("mbSetConnectionEnabled", ctypes.c_ubyte),
    ]
    mbSetRateEnabled: bool
    mbSetConnectionEnabled: bool

class cGcResetSimpleInteractionAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mReward", ctypes.c_ulonglong),
    ]
    mReward: int

class cGcScareCreaturesAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfHearRadius", ctypes.c_float),
        ("mfFleeRadius", ctypes.c_float),
    ]
    mfHearRadius: float
    mfFleeRadius: float

class cGcSpawnAction(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mEvent", ctypes.c_ulonglong),
    ]
    mEvent: int

class cGcSubstanceSecondary(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mfChance", ctypes.c_float),
        ("mfAmountMin", ctypes.c_float),
        ("mfAmountMax", ctypes.c_float),
    ]
    mID: int
    mfChance: float
    mfAmountMin: float
    mfAmountMax: float

class cGcSubstanceSecondaryLookup(ctypes.Structure):
    def FixDown(): pass

    def FixUp(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mPrimaryID", ctypes.c_ulonglong),
        ("maSecondaryChances", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSubstanceSecondary>
    ]
    mPrimaryID: int
    maSecondaryChances: bytes

class cGcItemPriceModifiers(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfSpaceStationMarkup", ctypes.c_float),
        ("mfLowPriceMod", ctypes.c_float),
        ("mfHighPriceMod", ctypes.c_float),
        ("mfBuyBaseMarkup", ctypes.c_float),
        ("mfBuyMarkupMod", ctypes.c_float),
    ]
    mfSpaceStationMarkup: float
    mfLowPriceMod: float
    mfHighPriceMod: float
    mfBuyBaseMarkup: float
    mfBuyMarkupMod: float

class cGcRealitySubstanceCraftingMix(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("miRatio", ctypes.c_int32),
    ]
    mID: int
    miRatio: int

class cGcRealityCraftingRecipeData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maInputs", ctypes.c_char * 0x48),  # cTkFixedArray<cGcRealitySubstanceCraftingMix,3>
        ("mOutputID", ctypes.c_ulonglong),
    ]
    maInputs: bytes
    mOutputID: int

class cGcSubstanceTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcRealitySubstanceData>
        ("maCrafting", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcRealityCraftingRecipeData>
    ]
    maTable: bytes
    maCrafting: bytes

class cGcSettlementPerkData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mName", ctypes.c_ulonglong),
        ("mDescription", ctypes.c_ulonglong),
        ("mbIsNegative", ctypes.c_ubyte),
        ("mbIsStarter", ctypes.c_ubyte),
        ("mbIsProc", ctypes.c_ubyte),
        ("mbIsJob", ctypes.c_ubyte),
        ("maStatChanges", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementStatChange>
    ]
    mID: int
    mName: int
    mDescription: int
    mbIsNegative: bool
    mbIsStarter: bool
    mbIsProc: bool
    mbIsJob: bool
    maStatChanges: bytes

class cGcSettlementPerksTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementPerkData>
    ]
    maTable: bytes

class cGcSettlementStatChangeArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maStats", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementStatChange>
    ]
    maStats: bytes

class cGcSettlementStatStrengthRanges(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
    ]
    miAmountMin: int
    miAmountMax: int

class cGcSettlementStatStrengthData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maPerkStatStrengthValues", ctypes.c_char * 0x38),  # cTkFixedArray<cGcSettlementStatStrengthRanges,7>
    ]
    maPerkStatStrengthValues: bytes

class cGcShipInventoryMaxUpgradeCapacity(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maiMaxInventoryCapacity", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("maiMaxTechInventoryCapacity", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
        ("maiMaxCargoInventoryCapacity", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
    ]
    maiMaxInventoryCapacity: bytes
    maiMaxTechInventoryCapacity: bytes
    maiMaxCargoInventoryCapacity: bytes

class cGcStats(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maStats", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcStatsGroup>
    ]
    maStats: bytes

class cGcRecipeTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcRefinerRecipe>
    ]
    maTable: bytes

class cGcRewardAbortTakeoff(ctypes.Structure):
    def ClassPointerWrite(): pass

    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardActivateEncounterSentinels(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mEncounterComponentScanEvent", ctypes.c_ulonglong),
        ("mEncounterOverride", ctypes.c_ulonglong),
    ]
    mEncounterComponentScanEvent: int
    mEncounterOverride: int

class cGcRewardActivateFiends(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mSpawnID", ctypes.c_ulonglong),
    ]
    mSpawnID: int

class cGcSubstanceSecondaryBiome(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maSecondarySubstanceByBiome", ctypes.c_char * 0x200),  # cTkFixedArray<cGcSubstanceSecondary,16>
    ]
    maSecondarySubstanceByBiome: bytes

class cGcRewardAdvancePortalState(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mPortalScanEvent", ctypes.c_ulonglong),
    ]
    mPortalScanEvent: int

class cGcRewardAssessCookedProduct(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miAmountWorst", ctypes.c_int32),
        ("miAmountBad", ctypes.c_int32),
        ("miAmountAverage", ctypes.c_int32),
        ("miAmountGood", ctypes.c_int32),
        ("miAmountBest", ctypes.c_int32),
        ("miAmountBestUpper", ctypes.c_int32),
    ]
    miAmountWorst: int
    miAmountBad: int
    miAmountAverage: int
    miAmountGood: int
    miAmountBest: int
    miAmountBestUpper: int

class cGcRewardBeginSettlementBuilding(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maValidBuildings", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBuildingClassification>
    ]
    maValidBuildings: bytes

class cGcRewardCargo(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardClosePortal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcAtlasSendSubmitContribution(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miMissionIndex", ctypes.c_int32),
        ("miContribution", ctypes.c_int32),
    ]
    miMissionIndex: int
    miContribution: int

class cGcRewardCompleteMission(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
    ]
    mMission: int

class cGcRewardCompleteMultiMission(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maMissions", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maMissions: bytes

class cGcRewardCrashSiteFly(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mNPCScanEvent", ctypes.c_ulonglong),
    ]
    mNPCScanEvent: int

class cGcRewardCrashSiteRepair(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardCustomExpeditionLogEntry(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbFromIntervention", ctypes.c_ubyte),
        ("mbWhaleEvent", ctypes.c_ubyte),
        ("mLocID", ctypes.c_ulonglong),
        ("mRewardID", ctypes.c_ulonglong),
    ]
    mbFromIntervention: bool
    mbWhaleEvent: bool
    mLocID: int
    mRewardID: int

class cGcRewardCustomPlayerControl(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mRequestedMode", ctypes.c_ulonglong),
    ]
    mRequestedMode: int

class cGcRewardDamage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mPlayerDamage", ctypes.c_ulonglong),
    ]
    mPlayerDamage: int

class cGcTechBoxTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcTechBoxData>
    ]
    maTable: bytes

class cGcTechList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maAvailableTech", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maAvailableTech: bytes

class cGcTechnologyTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcTechnology>
    ]
    maTable: bytes

class cGcTechnologyTypes(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTechnology", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcTechnology>
    ]
    maTechnology: bytes

class cGcRewardSpecificSeasonalSubstance(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("mfChanceToUse", ctypes.c_float),
        ("miSeasonNumber", ctypes.c_int32),
    ]
    mID: int
    miAmountMin: int
    miAmountMax: int
    mfChanceToUse: float
    miSeasonNumber: int

class cGcInventoryLayout(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miSlots", ctypes.c_int32),
        ("mSeed", common.GcSeed),
        ("miLevel", ctypes.c_int32),
    ]
    miSlots: int
    mSeed: common.GcSeed
    miLevel: int

class cGcRewardSpecificSpecial(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mMessage", ctypes.c_ulonglong),
        ("mbShowSpecialProductPopup", ctypes.c_ubyte),
        ("mbUseSpecialFormatting", ctypes.c_ubyte),
        ("mMilestoneRewardOverrideText", ctypes.c_ulonglong),
    ]
    mID: int
    mMessage: int
    mbShowSpecialProductPopup: bool
    mbUseSpecialFormatting: bool
    mMilestoneRewardOverrideText: int

class cGcTradingClassTable(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTradingClassesData", ctypes.c_char * 0x444),  # cTkFixedArray<cGcTradingClassData,7>
        ("maCategoryData", ctypes.c_char * 0x72C),  # cTkFixedArray<cGcTradingCategoryData,9>
        ("mfMaxTradingMultiplier", ctypes.c_float),
        ("mfMinTradingMultiplier", ctypes.c_float),
    ]
    maTradingClassesData: bytes
    maCategoryData: bytes
    mfMaxTradingMultiplier: float
    mfMinTradingMultiplier: float

class cGcUnlockableItemTreeNode(ctypes.Structure):
    def tilde_cGcUnlockableItemTreeNode(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mUnlockable", ctypes.c_ulonglong),
        ("maChildren", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcUnlockableItemTreeNode>
    ]
    mUnlockable: int
    maChildren: bytes

class cGcRewardSpecificTech(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTechId", ctypes.c_ulonglong),
        ("mbAutoPin", ctypes.c_ubyte),
        ("mbSilent", ctypes.c_ubyte),
        ("mbHideInSeasonRewards", ctypes.c_ubyte),
    ]
    mTechId: int
    mbAutoPin: bool
    mbSilent: bool
    mbHideInSeasonRewards: bool

class cGcRewardStartPurchase(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardStartSettlementExpedition(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardStatCompareAndSet(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCoreStat", ctypes.c_ulonglong),
        ("mCompareAndSetStat", ctypes.c_ulonglong),
    ]
    mCoreStat: int
    mCompareAndSetStat: int

class cGcRewardSwapMultiTool(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardSystemSpecificProductFromList(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maProductList", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("mbForceSpecialMessage", ctypes.c_ubyte),
    ]
    maProductList: bytes
    miAmountMin: int
    miAmountMax: int
    mbForceSpecialMessage: bool

class cGcRewardTable(ctypes.Structure):
    def tilde_cGcRewardTable(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maGenericTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maDestructionTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maInteractionTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maOldInteractionTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSpecialRewardTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maMissionBoardTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maFleetTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maWikiProgressTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maShipSalvageTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maEntitlementTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcRewardTableEntitlementItem>
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcRewardTableEntry>
        ("maTechRewardOrder", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maProductRewardOrder", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maMixerRewardTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maTwitchRewardTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maNPCPlanetSiteTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSettlementTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable1", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable2", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable3", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable4", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable5", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable6", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable7", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable8", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable9", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable10", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
        ("maSeasonRewardTable11", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
    ]
    maGenericTable: bytes
    maDestructionTable: bytes
    maInteractionTable: bytes
    maOldInteractionTable: bytes
    maSpecialRewardTable: bytes
    maMissionBoardTable: bytes
    maFleetTable: bytes
    maWikiProgressTable: bytes
    maShipSalvageTable: bytes
    maEntitlementTable: bytes
    maTable: bytes
    maTechRewardOrder: bytes
    maProductRewardOrder: bytes
    maMixerRewardTable: bytes
    maTwitchRewardTable: bytes
    maNPCPlanetSiteTable: bytes
    maSettlementTable: bytes
    maSeasonRewardTable1: bytes
    maSeasonRewardTable2: bytes
    maSeasonRewardTable3: bytes
    maSeasonRewardTable4: bytes
    maSeasonRewardTable5: bytes
    maSeasonRewardTable6: bytes
    maSeasonRewardTable7: bytes
    maSeasonRewardTable8: bytes
    maSeasonRewardTable9: bytes
    maSeasonRewardTable10: bytes
    maSeasonRewardTable11: bytes

class cGcRewardSecondarySubstance(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mfAmountFactor", ctypes.c_float),
        ("mbRewardAsBlobs", ctypes.c_ubyte),
    ]
    mID: int
    mfAmountFactor: float
    mbRewardAsBlobs: bool

class cGcRewardSendChatMessage(ctypes.Structure):
    def WriteToXMLNode(): pass

    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mStatusMessageId", ctypes.c_ulonglong),
        ("mCustomText", ctypes.c_ulonglong),
    ]
    mStatusMessageId: int
    mCustomText: int

class cGcRewardSetCurrentMission(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("mbSilent", ctypes.c_ubyte),
        ("mbSeeded", ctypes.c_ubyte),
    ]
    mMission: int
    mbSilent: bool
    mbSeeded: bool

class cGcRewardSetNexusExitWarpTargetToFireteamMemberUA(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miFireteamMemberIndex", ctypes.c_int32),
    ]
    miFireteamMemberIndex: int

class cGcRewardSettlementCustomJudgement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCustomJudgement", ctypes.c_ulonglong),
        ("mbSilent", ctypes.c_ubyte),
        ("mbCanOverrideNonCustomJudgement", ctypes.c_ubyte),
    ]
    mCustomJudgement: int
    mbSilent: bool
    mbCanOverrideNonCustomJudgement: bool

class cGcRewardSettlementJudgement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maJudgementTypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementJudgementType>
        ("mbSilent", ctypes.c_ubyte),
    ]
    maJudgementTypes: bytes
    mbSilent: bool

class cGcUnlockableItemTrees(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTitle", ctypes.c_ulonglong),
        ("maTrees", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcUnlockableItemTree>
    ]
    mTitle: int
    maTrees: bytes

class cGcUnlockablePlatformReward(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mRewardId", ctypes.c_ulonglong),
        ("mProductId", ctypes.c_ulonglong),
    ]
    mRewardId: int
    mProductId: int

class cGcUnlockablePlatformRewards(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcUnlockablePlatformReward>
    ]
    maTable: bytes

class cGcRewardSetWeaponSuppressed(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbWeaponSuppressed", ctypes.c_ubyte),
    ]
    mbWeaponSuppressed: bool

class cGcRewardShield(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("mbShowOSDOnSuccess", ctypes.c_ubyte),
        ("mbShowOSDOnFail", ctypes.c_ubyte),
    ]
    miAmountMin: int
    miAmountMax: int
    mbShowOSDOnSuccess: bool
    mbShowOSDOnFail: bool

class cGcUnlockableSeasonReward(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mbMustBeUnlocked", ctypes.c_ubyte),
        ("mbUniqueInventoryItem", ctypes.c_ubyte),
        ("mbIsCape", ctypes.c_ubyte),
        ("mbSwitchExclusive", ctypes.c_ubyte),
        ("maiSeasonIds", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
        ("maiStageIds", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
    ]
    mID: int
    mbMustBeUnlocked: bool
    mbUniqueInventoryItem: bool
    mbIsCape: bool
    mbSwitchExclusive: bool
    maiSeasonIds: bytes
    maiStageIds: bytes

class cGcUnlockableSeasonRewards(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcUnlockableSeasonReward>
    ]
    maTable: bytes

class cGcRewardShipSlot(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mCost", ctypes.c_ulonglong),
        ("mbAwardCostAndOpenWindow", ctypes.c_ubyte),
        ("miNumTokens", ctypes.c_int32),
        ("mbIsAlien", ctypes.c_ubyte),
        ("mbFallbackOpenWindowIfBlocked", ctypes.c_ubyte),
    ]
    mCost: int
    mbAwardCostAndOpenWindow: bool
    miNumTokens: int
    mbIsAlien: bool
    mbFallbackOpenWindowIfBlocked: bool

class cGcRewardShowBlackHoles(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbDummy", ctypes.c_ubyte),
    ]
    mbDummy: bool

class cGcRewardSpecificCommunityTierProduct(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maProductList", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("mbForceSpecialMessage", ctypes.c_ubyte),
        ("mRequiresTech", ctypes.c_ulonglong),
    ]
    maProductList: bytes
    miAmountMin: int
    miAmountMax: int
    mbForceSpecialMessage: bool
    mRequiresTech: int

class cGcRewardSpecificProductRecipe(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mbSilent", ctypes.c_ubyte),
        ("mbHideInSeasonRewards", ctypes.c_ubyte),
        ("mSeasonRewardFormat", ctypes.c_ulonglong),
    ]
    mID: int
    mbSilent: bool
    mbHideInSeasonRewards: bool
    mSeasonRewardFormat: int

class cGcUnlockableTrees(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTrees", ctypes.c_char * 0x2A0),  # cTkFixedArray<cGcUnlockableItemTrees,14>
        ("maCostTypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcUnlockableTreeCostType>
    ]
    maTrees: bytes
    maCostTypes: bytes

class cGcUnlockableTwitchReward(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTwitchId", ctypes.c_ulonglong),
        ("mProductId", ctypes.c_ulonglong),
        ("mLinkedGroupId", ctypes.c_ulonglong),
    ]
    mTwitchId: int
    mProductId: int
    mLinkedGroupId: int

class cGcUnlockableTwitchRewards(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcUnlockableTwitchReward>
    ]
    maTable: bytes

class cGcDiscoveryWorth(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maiRecord", ctypes.c_char * 0xC),  # cTkFixedArray<int,3>
        ("maiOnScan", ctypes.c_char * 0xC),  # cTkFixedArray<int,3>
        ("miMission", ctypes.c_int32),
    ]
    maiRecord: bytes
    maiOnScan: bytes
    miMission: int

class cGcWeaponInventoryMaxUpgradeCapacity(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maiMaxInventoryCapacity", ctypes.c_char * 0x10),  # cTkFixedArray<int,4>
    ]
    maiMaxInventoryCapacity: bytes

class cGcFreighterCargoOption(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("miMinAmount", ctypes.c_int32),
        ("miMaxAmount", ctypes.c_int32),
        ("miPercentChance", ctypes.c_int32),
    ]
    mID: int
    miMinAmount: int
    miMaxAmount: int
    miPercentChance: int

class cGcCostItemFromList(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miAmount", ctypes.c_int32),
        ("maItemList", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("miIndex", ctypes.c_int32),
    ]
    miAmount: int
    maItemList: bytes
    miIndex: int

class cGcCostJourneyMilestone(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mRequiredMilestone", ctypes.c_ulonglong),
    ]
    mRequiredMilestone: int

class cGcCostJourneyStatLevel(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mStatName", ctypes.c_ulonglong),
        ("miRequiredLevel", ctypes.c_int32),
    ]
    mStatName: int
    miRequiredLevel: int

class cGcCostMissionComplete(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCost", ctypes.c_ulonglong),
        ("mTextOverride", ctypes.c_ulonglong),
        ("mbHideIfCompleted", ctypes.c_ubyte),
    ]
    mCost: int
    mTextOverride: int
    mbHideIfCompleted: bool

class cGcItemAmountCostPair(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mItemId", ctypes.c_ulonglong),
        ("miAmount", ctypes.c_int32),
    ]
    mItemId: int
    miAmount: int

class cGcCostMultiItem(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mDisplayLocID", ctypes.c_ulonglong),
        ("mbOnlyTakeIfCanAfford", ctypes.c_ubyte),
        ("maItemList", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcItemAmountCostPair>
    ]
    mDisplayLocID: int
    mbOnlyTakeIfCanAfford: bool
    maItemList: bytes

class cGcCostOwnSettlement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCostPendingSettlementJudgement(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCostPirateTribute(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfCargoValuePercent", ctypes.c_float),
        ("miMinimumValue", ctypes.c_int32),
    ]
    mfCargoValuePercent: float
    miMinimumValue: int

class cGcCostPoliceCargoBribe(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbIncludeNipNip", ctypes.c_ubyte),
        ("mbOnlyCargoProbeInventories", ctypes.c_ubyte),
        ("miAmount", ctypes.c_int32),
    ]
    mbIncludeNipNip: bool
    mbOnlyCargoProbeInventories: bool
    miAmount: int

class cGcCostPoliceCargoComply(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCostSalvageShip(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCostSentinelBlockStatus(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbCanAffordIfSentinelsDisabled", ctypes.c_ubyte),
    ]
    mbCanAffordIfSentinelsDisabled: bool

class cGcCostShipUpgradeable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCostSpecificCreatureBait(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCostStat(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mStat", ctypes.c_ulonglong),
        ("mStatGroup", ctypes.c_ulonglong),
        ("miValue", ctypes.c_int32),
        ("mCostAsString", ctypes.c_ulonglong),
    ]
    mStat: int
    mStatGroup: int
    miValue: int
    mCostAsString: int

class cGcCostStatCompare(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCoreStat", ctypes.c_ulonglong),
        ("mCompareAndSetStat", ctypes.c_ulonglong),
        ("mCostStringCanAfford", ctypes.c_ulonglong),
        ("mCostStringCantAfford", ctypes.c_ulonglong),
    ]
    mCoreStat: int
    mCompareAndSetStat: int
    mCostStringCanAfford: int
    mCostStringCantAfford: int

class cGcCostTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSimpleInteractionTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCostTableEntry>
        ("maInteractionTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCostTableEntry>
        ("maItemCostsTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCostTableEntry>
        ("maAtlasPathCosts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCostTableEntry>
        ("maUnusedCostsTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCostTableEntry>
    ]
    maSimpleInteractionTable: bytes
    maInteractionTable: bytes
    maItemCostsTable: bytes
    maAtlasPathCosts: bytes
    maUnusedCostsTable: bytes

class cGcNameGeneratorWord(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mWord", ctypes.c_ulonglong),
        ("miNumOptions", ctypes.c_int32),
    ]
    mWord: int
    miNumOptions: int

class cGcProceduralTechnologyTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcProceduralTechnologyData>
    ]
    maTable: bytes

class cGcProductTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcProductData>
    ]
    maTable: bytes

class cGcSpaceMapObjectData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mColour", common.Colour),
        ("mbTintModel", ctypes.c_ubyte),
        ("mfRadius", ctypes.c_float),
        ("mfDistanceMin", ctypes.c_float),
        ("mfDistanceRange", ctypes.c_float),
        ("mfScaleRange", ctypes.c_float),
        ("mfScaleMagnitude", ctypes.c_float),
        ("mfScaleMin", ctypes.c_float),
        ("mbOrient", ctypes.c_ubyte),
    ]
    mColour: common.Colour
    mbTintModel: bool
    mfRadius: float
    mfDistanceMin: float
    mfDistanceRange: float
    mfScaleRange: float
    mfScaleMagnitude: float
    mfScaleMin: float
    mbOrient: bool

class cGcBuildingBlueprint(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mProductID", ctypes.c_ulonglong),
        ("miGroupId", ctypes.c_int32),
    ]
    mProductID: int
    miGroupId: int

class cGcPurchaseableBuildingBlueprints(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBuildingBlueprint>
        ("maiGroupMaxItems", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
    ]
    maTable: bytes
    maiGroupMaxItems: bytes

class cGcTextPresetTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcTextPreset>
    ]
    maTable: bytes

class cGcPurchaseableSpecial(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("miShopNumber", ctypes.c_int32),
        ("miMissionTier", ctypes.c_int32),
        ("mbIsConsumable", ctypes.c_ubyte),
        ("mbIsCape", ctypes.c_ubyte),
    ]
    mID: int
    miShopNumber: int
    miMissionTier: int
    mbIsConsumable: bool
    mbIsCape: bool

class cGcPurchaseableSpecials(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPurchaseableSpecial>
    ]
    maTable: bytes

class cGcRewardDeactivateFiends(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardDisableSentinels(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDuration", ctypes.c_float),
        ("mOSDMessage", ctypes.c_ulonglong),
        ("mWantedBarMessage", ctypes.c_ulonglong),
    ]
    mfDuration: float
    mOSDMessage: int
    mWantedBarMessage: int

class cGcRewardDiscoverRune(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbAllRunes", ctypes.c_ubyte),
    ]
    mbAllRunes: bool

class cGcRewardDisplayTechWindow(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTechID", ctypes.c_ulonglong),
        ("mbDamaged", ctypes.c_ubyte),
        ("mbNeedsInstall", ctypes.c_ubyte),
        ("mbFullBox", ctypes.c_ubyte),
    ]
    mTechID: int
    mbDamaged: bool
    mbNeedsInstall: bool
    mbFullBox: bool

class cGcRewardDummyLocID(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mLocID", ctypes.c_ulonglong),
    ]
    mLocID: int

class cGcRewardEnableInteractionClass(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardEnableSentinels(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardEndFrigateFlyby(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardEndSettlementExpedition(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardEnergy(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miAmount", ctypes.c_int32),
    ]
    miAmount: int

class cGcRewardIncrementStat(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mStat", ctypes.c_ulonglong),
        ("miAmount", ctypes.c_int32),
    ]
    mStat: int
    miAmount: int

class cGcRewardExchangeProduct(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mIDToGive", ctypes.c_ulonglong),
        ("mIDToTake", ctypes.c_ulonglong),
        ("miAmountToGiveMin", ctypes.c_int32),
        ("miAmountToGiveMax", ctypes.c_int32),
        ("miAmountToTakeMax", ctypes.c_int32),
        ("mbHideNewProduct", ctypes.c_ubyte),
        ("mbForceSpecialMessage", ctypes.c_ubyte),
    ]
    mIDToGive: int
    mIDToTake: int
    miAmountToGiveMin: int
    miAmountToGiveMax: int
    miAmountToTakeMax: int
    mbHideNewProduct: bool
    mbForceSpecialMessage: bool

class cGcRewardForceDiscoverSystem(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbSilent", ctypes.c_ubyte),
    ]
    mbSilent: bool

class cGcRewardInventorySlots(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miAmount", ctypes.c_int32),
    ]
    miAmount: int

class cGcRewardForgetSpecificProductRecipe(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maProductList", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maProductList: bytes

class cGcRewardJetpackBoost(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDuration", ctypes.c_float),
        ("mfForwardBoost", ctypes.c_float),
        ("mfUpBoost", ctypes.c_float),
        ("mfIgnitionBoost", ctypes.c_float),
    ]
    mfDuration: float
    mfForwardBoost: float
    mfUpBoost: float
    mfIgnitionBoost: float

class cGcRewardForgetSpecificTechRecipe(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTechList", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maTechList: bytes

class cGcRewardFreeStamina(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDuration", ctypes.c_float),
    ]
    mfDuration: float

class cGcRewardMission(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("mbSetAsSelected", ctypes.c_ubyte),
        ("mbFailRewardIfMissionActive", ctypes.c_ubyte),
    ]
    mMission: int
    mbSetAsSelected: bool
    mbFailRewardIfMissionActive: bool

class cGcRewardFreighterBaseReset(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardMissionMessage(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mMessageID", ctypes.c_ulonglong),
        ("mbBroadcastInMultiplayer", ctypes.c_ubyte),
    ]
    mMessageID: int
    mbBroadcastInMultiplayer: bool

class cGcRewardFreighterMegaWarp(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardMissionMessageSeeded(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMessageID", ctypes.c_ulonglong),
        ("mbBroadcastInMultiplayer", ctypes.c_ubyte),
        ("mbBroadcastToActiveMultiplayerMission", ctypes.c_ubyte),
        ("mSpecificMissionID", ctypes.c_ulonglong),
    ]
    mMessageID: int
    mbBroadcastInMultiplayer: bool
    mbBroadcastToActiveMultiplayerMission: bool
    mSpecificMissionID: int

class cGcRewardFreighterSlot(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCost", ctypes.c_ulonglong),
    ]
    mCost: int

class cTkRegisteredGlobal(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkRegisteredGlobal_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcSpaceshipGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcRewardMissionMessageToMatchingSeeds(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMessageID", ctypes.c_ulonglong),
        ("mbBroadcastInMultiplayer", ctypes.c_ubyte),
    ]
    mMessageID: int
    mbBroadcastInMultiplayer: bool

class cGcRewardMissionSeeded(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("mMissionNoGroundCombat", ctypes.c_ulonglong),
        ("mMissionNoSpaceCombat", ctypes.c_ulonglong),
        ("mbInheritActiveMultiplayerMissionSeed", ctypes.c_ubyte),
    ]
    mMission: int
    mMissionNoGroundCombat: int
    mMissionNoSpaceCombat: int
    mbInheritActiveMultiplayerMissionSeed: bool

class cGcRewardHazard(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("mbSilent", ctypes.c_ubyte),
    ]
    miAmountMin: int
    miAmountMax: int
    mbSilent: bool

class cGcRewardHealth(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("mbSilentUnlessShieldAtMax", ctypes.c_ubyte),
    ]
    miAmountMin: int
    miAmountMax: int
    mbSilentUnlessShieldAtMax: bool

class cTkMetaDataHeader(ctypes.Structure):
    _fields_ = [
        ("miHeader", ctypes.c_uint32),
        ("miVersion", ctypes.c_uint32),
        ("mu64Timestamp", ctypes.c_uint64),
        ("mu64TemplateHash", ctypes.c_uint64),
        ("mClass", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    miHeader: int
    miVersion: int
    mu64Timestamp: int
    mu64TemplateHash: int
    mClass: bytes

class cGcRewardMultiSpecificItems(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbSilent", ctypes.c_ubyte),
        ("maItems", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcMultiSpecificItemEntry>
    ]
    mbSilent: bool
    maItems: bytes

class TkFileSystemUtils(ctypes.Structure):
    pass

class cGcRewardMultiSpecificProductRecipes(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maProductIds", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("mDisplayProductId", ctypes.c_ulonglong),
        ("mSetName", ctypes.c_ulonglong),
        ("mbSilent", ctypes.c_ubyte),
    ]
    maProductIds: bytes
    mDisplayProductId: int
    mSetName: int
    mbSilent: bool

class cGcRewardMultiSpecificProducts(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mSetName", ctypes.c_ulonglong),
        ("maProductIds", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mSetName: int
    maProductIds: bytes

class cGcRewardMultiSpecificTechRecipes(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTechIds", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("mDisplayTechId", ctypes.c_ulonglong),
        ("mSetName", ctypes.c_ulonglong),
        ("mbSilent", ctypes.c_ubyte),
    ]
    maTechIds: bytes
    mDisplayTechId: int
    mSetName: int
    mbSilent: bool

class cGcRewardNetworkPlayer(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macRewardWord", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    macRewardWord: bytes

class cGcRewardNexus(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbAllow", ctypes.c_ubyte),
    ]
    mbAllow: bool

class cGcRewardOpenFreeFreighter(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbReinteractWhenBought", ctypes.c_ubyte),
        ("mNextInteractionIfBought", ctypes.c_ulonglong),
        ("mNextInteractionIfNotBought", ctypes.c_ulonglong),
    ]
    mbReinteractWhenBought: bool
    mNextInteractionIfBought: int
    mNextInteractionIfNotBought: int

class cTkMetaDataManager(ctypes.Structure):
    def GetInstance(): pass

    _fields_ = [
        ("mRegisteredGlobals", ctypes.c_char * 0x18),  # std::vector<cTkRegisteredGlobal *,TkSTLAllocatorShim<cTkRegisteredGlobal *,8,-1> >
    ]
    mRegisteredGlobals: bytes

class RegistercGcGameplayGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcShipHUDTargetIconData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macCorner", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macLineHorizontal", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macLineVertical", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macGlowCorner", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macGlowLineHorizontal", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macGlowLineVertical", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macCorner: bytes
    macLineHorizontal: bytes
    macLineVertical: bytes
    macGlowCorner: bytes
    macGlowLineHorizontal: bytes
    macGlowLineVertical: bytes

class cGcPortalData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfRuneRotateTime", ctypes.c_float),
        ("mbSkipRuneEntry", ctypes.c_ubyte),
        ("mbKnowAllRunes", ctypes.c_ubyte),
    ]
    mfRuneRotateTime: float
    mbSkipRuneEntry: bool
    mbKnowAllRunes: bool

class cGcRewardOverridePulseEncounterChance(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfChance", ctypes.c_float),
    ]
    mfChance: float

class cGcRewardPetEgg(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardPetEggHatch(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miEggIndex", ctypes.c_int32),
    ]
    miEggIndex: int

class cGcRewardPirateAttack(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbInstant", ctypes.c_ubyte),
        ("miNumSquads", ctypes.c_int32),
        ("mAttackDefinition", ctypes.c_ulonglong),
    ]
    mbInstant: bool
    miNumSquads: int
    mAttackDefinition: int

class cGcRewardPirateProbeSignal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbAttack", ctypes.c_ubyte),
    ]
    mbAttack: bool

class cGcRewardPlanetSubstance(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miAmountMin", ctypes.c_int32),
        ("miAmountMax", ctypes.c_int32),
        ("mbDisableMultiplier", ctypes.c_ubyte),
        ("mbRewardAsBlobs", ctypes.c_ubyte),
        ("mbUseFuelMultiplier", ctypes.c_ubyte),
    ]
    miAmountMin: int
    miAmountMax: int
    mbDisableMultiplier: bool
    mbRewardAsBlobs: bool
    mbUseFuelMultiplier: bool

class cGcRewardPoliceScanSignal(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbAttack", ctypes.c_ubyte),
    ]
    mbAttack: bool

class cGcRewardProceduralProductFromBiome(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardProcTechProduct(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGroup", ctypes.c_ulonglong),
        ("miWeightedChanceNormal", ctypes.c_int32),
        ("miWeightedChanceRare", ctypes.c_int32),
        ("miWeightedChanceEpic", ctypes.c_int32),
        ("miWeightedChanceLegendary", ctypes.c_int32),
        ("mbForceRelevant", ctypes.c_ubyte),
        ("mbForceQualityRelevant", ctypes.c_ubyte),
    ]
    mGroup: int
    miWeightedChanceNormal: int
    miWeightedChanceRare: int
    miWeightedChanceEpic: int
    miWeightedChanceLegendary: int
    mbForceRelevant: bool
    mbForceQualityRelevant: bool

class cGcRewardRechargeTech(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mTechID", ctypes.c_ulonglong),
    ]
    mTechID: int

class RegistercGcBuildingGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcRewardRefreshHazProt(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfAmount", ctypes.c_float),
        ("mbSetNotAdd", ctypes.c_ubyte),
        ("mbSilent", ctypes.c_ubyte),
    ]
    mfAmount: float
    mbSetNotAdd: bool
    mbSilent: bool

class cGcAtlasGlobals(ctypes.Structure):
    def NGuiRenderCallback(): pass

    def ReadModFromFile(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miChanceOfDisconnect", ctypes.c_int32),
        ("miTimeoutSecNameResolution", ctypes.c_int32),
        ("miTimeoutSecConnection", ctypes.c_int32),
        ("miTimeoutSecSendRecv", ctypes.c_int32),
    ]
    miChanceOfDisconnect: int
    miTimeoutSecNameResolution: int
    miTimeoutSecConnection: int
    miTimeoutSecSendRecv: int

class RegistercGcAtlasGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcAudio3PointDopplerData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfFront", ctypes.c_float),
        ("mfMid", ctypes.c_float),
        ("mfRear", ctypes.c_float),
    ]
    mfFront: float
    mfMid: float
    mfRear: float

class cGcAudioNPCDoppler(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maConfig", ctypes.c_char * 0x54),  # cTkFixedArray<cGcAudio3PointDopplerData,7>
    ]
    maConfig: bytes

class RegistercGcAudioGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcRewardRequirementsForRecipe(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mRecipeID", ctypes.c_ulonglong),
        ("mbRewardInCreative", ctypes.c_ubyte),
    ]
    mRecipeID: int
    mbRewardInCreative: bool

class cGcRewardSalvageShip(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcRewardScanEventNearestBuilding(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbDoAerialScan", ctypes.c_ubyte),
        ("mbIncludeVisited", ctypes.c_ubyte),
    ]
    mbDoAerialScan: bool
    mbIncludeVisited: bool

class cGcRewardSecondaryInteractionOptions(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcAlienPuzzleOption>
    ]
    maOptions: bytes

class cGcDeathQuote(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macQuoteLine1", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macQuoteLine2", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAuthor", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
    ]
    macQuoteLine1: bytes
    macQuoteLine2: bytes
    macAuthor: bytes

class cGcDialogClearanceInfo(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGlobalDialogID", ctypes.c_ulonglong),
        ("mAssociatedMission", ctypes.c_ulonglong),
    ]
    mGlobalDialogID: int
    mAssociatedMission: int

class cGcDialogClearanceTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDialogClearanceInfo>
    ]
    maTable: bytes

class cGcDiscoveryRewardLookup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mSecondary", ctypes.c_ulonglong),
        ("maBiomeSpecific", ctypes.c_char * 0x100),  # cTkFixedArray<TkID<128>,16>
    ]
    mId: int
    mSecondary: int
    maBiomeSpecific: bytes

class cGcDiscoveryRewardLookupTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcDiscoveryRewardLookup>
    ]
    maTable: bytes

class cGcExpeditionRewardTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGenericRewardTableEntry>
    ]
    maTable: bytes

class cGcInventoryBaseStat(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mBaseStatID", ctypes.c_ulonglong),
        ("mLocID", ctypes.c_ulonglong),
        ("maStatBonus", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInventoryBaseStatBonus>
    ]
    mBaseStatID: int
    mLocID: int
    maStatBonus: bytes

class cGcAlienSpeechTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcAlienSpeechEntry>
    ]
    maTable: bytes

class cGcBuildingCostPartCount(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("miCount", ctypes.c_int32),
    ]
    mId: int
    miCount: int

class cGcPerformanceFlyby(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfLength", ctypes.c_float),
        ("mfOffset", ctypes.c_float),
        ("mbLocked", ctypes.c_ubyte),
        ("mfLockOffset", ctypes.c_float),
        ("mfLockTime", ctypes.c_float),
        ("mfLockSpeed", ctypes.c_float),
    ]
    mfLength: float
    mfOffset: float
    mbLocked: bool
    mfLockOffset: float
    mfLockTime: float
    mfLockSpeed: float

class cGcConsumableItemTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcConsumableItem>
    ]
    maTable: bytes

class cGcPlayfabMatchmakingAttributes(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macUA", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macmatchmakingVersion", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miisBackfilling", ctypes.c_int32),
        ("mineedsSmallLobby", ctypes.c_int32),
        ("migameProgress", ctypes.c_int32),
        ("macplatform", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macgamemode", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maclobbyConnectionString", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("macseasonNumber", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    macUA: bytes
    macmatchmakingVersion: bytes
    miisBackfilling: int
    mineedsSmallLobby: int
    migameProgress: int
    macplatform: bytes
    macgamemode: bytes
    maclobbyConnectionString: bytes
    macseasonNumber: bytes

class cGcCostAdvanceSettlementBuilding(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("miAmount", ctypes.c_int32),
    ]
    mId: int
    miAmount: int

class cGcCostAnyCookedProduct(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mCostString", ctypes.c_ulonglong),
        ("mCostStringCantAfford", ctypes.c_ulonglong),
        ("miIndex", ctypes.c_int32),
    ]
    mCostString: int
    mCostStringCantAfford: int
    miIndex: int

class cGcCostBuildingParts(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mDescription", ctypes.c_ulonglong),
        ("maRequiredParts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBuildingCostPartCount>
    ]
    mDescription: int
    maRequiredParts: bytes

class cGcInventoryClassProbabilities(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mafClassProbabilities", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
    ]
    mafClassProbabilities: bytes

class cGcInventoryCostDataEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miMinSlots", ctypes.c_int32),
        ("mfMinValueInMillions", ctypes.c_float),
        ("miMaxSlots", ctypes.c_int32),
        ("mfMaxValueInMillions", ctypes.c_float),
        ("mfCoolMultiplier", ctypes.c_float),
        ("mfTradeInMultiplier", ctypes.c_float),
        ("mafClassMultiplier", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
    ]
    miMinSlots: int
    mfMinValueInMillions: float
    miMaxSlots: int
    mfMaxValueInMillions: float
    mfCoolMultiplier: float
    mfTradeInMultiplier: float
    mafClassMultiplier: bytes

class cGcCostCanAdoptCreature(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcInventoryCostData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maInventoryCostData", ctypes.c_char * 0x168),  # cTkFixedArray<cGcInventoryCostDataEntry,9>
    ]
    maInventoryCostData: bytes

class cGcCostCanCustomiseCreature(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCostCanFreighterMegaWarp(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcInventoryGenerationBaseStatDataEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mBaseStatID", ctypes.c_ulonglong),
        ("mfMin", ctypes.c_float),
        ("mfMax", ctypes.c_float),
        ("mfMinFixedAdd", ctypes.c_float),
        ("mfMaxFixedAdd", ctypes.c_float),
    ]
    mBaseStatID: int
    mfMin: float
    mfMax: float
    mfMinFixedAdd: float
    mfMaxFixedAdd: float

class cGcCostCanMilkCreature(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcInventoryGenerationBaseStatClassData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBaseStats", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInventoryGenerationBaseStatDataEntry>
    ]
    maBaseStats: bytes

class cGcCostCanRideCreature(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcCostCargo(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miSlots", ctypes.c_int32),
    ]
    miSlots: int

class cGcInventoryGenerationBaseStatData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maBaseStatsPerClass", ctypes.c_char * 0x40),  # cTkFixedArray<cGcInventoryGenerationBaseStatClassData,4>
    ]
    maBaseStatsPerClass: bytes

class cGcCostCreatureCanLayEggs(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcInventoryLayoutGenerationBounds(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miMaxWidthSmall", ctypes.c_int32),
        ("miMaxHeightSmall", ctypes.c_int32),
        ("miMaxWidthStandard", ctypes.c_int32),
        ("miMaxHeightStandard", ctypes.c_int32),
        ("miMaxWidthLarge", ctypes.c_int32),
        ("miMaxHeightLarge", ctypes.c_int32),
    ]
    miMaxWidthSmall: int
    miMaxHeightSmall: int
    miMaxWidthStandard: int
    miMaxHeightStandard: int
    miMaxWidthLarge: int
    miMaxHeightLarge: int

class cGcPlayerExperienceAsteroidCreatureSpawnData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mfWeight", ctypes.c_float),
        ("mSmallMinMax", common.Vector2f),
        ("mMediumMinMax", common.Vector2f),
        ("mLargeMinMax", common.Vector2f),
    ]
    mID: int
    mfWeight: float
    mSmallMinMax: common.Vector2f
    mMediumMinMax: common.Vector2f
    mLargeMinMax: common.Vector2f

class cGcCostFrigateCargo(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcPlayerExperienceAsteroidCreatureSpawnTable(ctypes.Structure):
    def tilde_cGcPlayerExperienceAsteroidCreatureSpawnTable(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfSmallAsteroidSpawnPercent", ctypes.c_float),
        ("maSmallAsteroidSpawns", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerExperienceAsteroidCreatureSpawnData>
        ("mfMediumAsteroidSpawnPercent", ctypes.c_float),
        ("maMediumAsteroidSpawns", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerExperienceAsteroidCreatureSpawnData>
        ("mfLargeAsteroidSpawnPercent", ctypes.c_float),
        ("maLargeAsteroidSpawns", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerExperienceAsteroidCreatureSpawnData>
    ]
    mfSmallAsteroidSpawnPercent: float
    maSmallAsteroidSpawns: bytes
    mfMediumAsteroidSpawnPercent: float
    maMediumAsteroidSpawns: bytes
    mfLargeAsteroidSpawnPercent: float
    maLargeAsteroidSpawns: bytes

class cGcInventoryLayoutGenerationData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGenerationDataPerSizeType", ctypes.c_char * 0xC24),  # cTkFixedArray<cGcInventoryLayoutGenerationDataEntry,37>
    ]
    maGenerationDataPerSizeType: bytes

class cGcSentinelSpawnSequenceGroupList(ctypes.Structure):
    def FixDown(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSequences", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maExtremeSequences", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maSequences: bytes
    maExtremeSequences: bytes

class cGcSentinelWaveGroup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maWaves", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maExtremeWaves", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maWaves: bytes
    maExtremeWaves: bytes

class cGcCostHasFireteamMember(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miIndex", ctypes.c_int32),
    ]
    miIndex: int

class cGcSentinelSpawnSequenceStep(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maWavePool", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    maWavePool: bytes

class cGcCostHealth(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miHealthUnits", ctypes.c_int32),
    ]
    miHealthUnits: int

class cGcSentinelSpawnNamedSequence(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maWaves", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSentinelSpawnSequenceStep>
    ]
    mId: int
    maWaves: bytes

class cGcSentinelSpawnWave(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maSpawns", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSentinelSpawnData>
        ("miReinforceAt", ctypes.c_int32),
    ]
    mId: int
    maSpawns: bytes
    miReinforceAt: int

class cGcItemCostData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mfCost", ctypes.c_float),
        ("mfMinCost", ctypes.c_float),
        ("mfMaxCost", ctypes.c_float),
        ("mfChangePerSale", ctypes.c_float),
    ]
    mID: int
    mfCost: float
    mfMinCost: float
    mfMaxCost: float
    mfChangePerSale: float

class cGcLegacyItem(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mConvertID", ctypes.c_ulonglong),
        ("mfConvertRatio", ctypes.c_float),
        ("mbAddNewRecipe", ctypes.c_ubyte),
        ("mbRemoveOldRecipe", ctypes.c_ubyte),
    ]
    mID: int
    mConvertID: int
    mfConvertRatio: float
    mbAddNewRecipe: bool
    mbRemoveOldRecipe: bool

class cGcLegacyItemTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcLegacyItem>
    ]
    maTable: bytes

class cGcByteBeatSong(ctypes.Structure):
    def cGcByteBeatSong(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mRequiredSpecialId", ctypes.c_ulonglong),
        ("macName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mLocID", ctypes.c_ulonglong),
        ("macAuthorOnlineID", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macAuthorUsername", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macAuthorPlatform", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("maacData", ctypes.c_char * 0x200),  # cTkFixedArray<cTkFixedString<64,char>,8>
    ]
    mId: int
    mRequiredSpecialId: int
    macName: bytes
    mLocID: int
    macAuthorOnlineID: bytes
    macAuthorUsername: bytes
    macAuthorPlatform: bytes
    maacData: bytes

class cGcByteBeatLibraryData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maMySongs", ctypes.c_char * 0x1900),  # cTkFixedArray<cGcByteBeatSong,8>
        ("maPlaylist", ctypes.c_char * 0x100),  # cTkFixedArray<TkID<128>,16>
        ("mbShuffle", ctypes.c_ubyte),
        ("mbAutoplayOnFoot", ctypes.c_ubyte),
        ("mbAutoplayInShip", ctypes.c_ubyte),
        ("mbAutoplayInVehicle", ctypes.c_ubyte),
    ]
    maMySongs: bytes
    maPlaylist: bytes
    mbShuffle: bool
    mbAutoplayOnFoot: bool
    mbAutoplayInShip: bool
    mbAutoplayInVehicle: bool

class cGcMaintenanceGroup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcMaintenanceGroupEntry>
    ]
    maTable: bytes

class cGcMaintenanceGroupsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGroups", ctypes.c_char * 0x70),  # cTkFixedArray<cGcMaintenanceGroup,7>
    ]
    maGroups: bytes

class cGcPlayerMissionProgress(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("miProgress", ctypes.c_int32),
        ("mu64Seed", ctypes.c_uint64),
        ("mu64Data", ctypes.c_uint64),
        ("maParticipants", ctypes.c_char * 0x340),  # cTkFixedArray<cGcPlayerMissionParticipant,13>
    ]
    mMission: int
    miProgress: int
    mu64Seed: int
    mu64Data: int
    maParticipants: bytes

class cGcInteractionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mu64GalacticAddress", ctypes.c_uint64),
        ("mu64Value", ctypes.c_uint64),
        ("mPosition", common.Vector4f),
    ]
    mu64GalacticAddress: int
    mu64Value: int
    mPosition: common.Vector4f

class cGcStatValueData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miIntValue", ctypes.c_int32),
        ("mfFloatValue", ctypes.c_float),
        ("mfDenominator", ctypes.c_float),
    ]
    miIntValue: int
    mfFloatValue: float
    mfDenominator: float

class cGcPlayerStatsGroup(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def FixDown(): pass

    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def DeallocateDynamic(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mGroupId", ctypes.c_ulonglong),
        ("mu64Address", ctypes.c_uint64),
        ("maStats", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerStat>
    ]
    mGroupId: int
    mu64Address: int
    maStats: bytes

class cGcPlayerDamageTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maDamageTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerDamageData>
    ]
    maDamageTable: bytes

class cGcTelemetryStat(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mType", ctypes.c_ulonglong),
        ("miValue", ctypes.c_int32),
    ]
    mId: int
    mType: int
    miValue: int

class cGcInteractionBuffer(ctypes.Structure):
    def cGcInteractionBuffer(): pass

    def DeallocateDynamic(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maInteractions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInteractionData>
        ("miCurrentPos", ctypes.c_int32),
    ]
    maInteractions: bytes
    miCurrentPos: int

class cGcSavedInteractionRaceData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maiSavedRaceIndicies", ctypes.c_char * 0x20),  # cTkFixedArray<int,8>
        ("mabHasLoopedIndicies", ctypes.c_char * 0x8),  # cTkFixedArray<bool,8>
    ]
    maiSavedRaceIndicies: bytes
    mabHasLoopedIndicies: bytes

class cGcSavedInteractionDialogData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64Hash", ctypes.c_uint64),
        ("mDialog", ctypes.c_ulonglong),
    ]
    mu64Hash: int
    mDialog: int

class cGcSavedEntitlement(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macEntitlementId", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
    ]
    macEntitlementId: bytes

class cGcPersistentBBObjectData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mu64Timestamp", ctypes.c_uint64),
        ("mObjectID", ctypes.c_ulonglong),
        ("mu64GalacticAddress", ctypes.c_uint64),
        ("mu64RegionSeed", ctypes.c_uint64),
        ("mu64UserData", ctypes.c_uint64),
        ("mPosition", common.Vector3f),
        ("mUp", common.Vector3f),
        ("mAt", common.Vector3f),
    ]
    mu64Timestamp: int
    mObjectID: int
    mu64GalacticAddress: int
    mu64RegionSeed: int
    mu64UserData: int
    mPosition: common.Vector3f
    mUp: common.Vector3f
    mAt: common.Vector3f

class cGcPlayerTitleData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTitles", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerTitle>
    ]
    maTitles: bytes

class cGcTerrainEdit(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mData", ctypes.c_uint8),
        ("miPosition", ctypes.c_int32),
    ]
    mData: int
    miPosition: int

class cGcTerrainEditsBuffer(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mau64GalacticAddresses", ctypes.c_char * 0x800),  # cTkFixedArray<unsigned __int64,256>
        ("maiBufferSizes", ctypes.c_char * 0x400),  # cTkFixedArray<int,256>
        ("maBufferAges", ctypes.c_char * 0x100),  # cTkFixedArray<unsigned char,256>
        ("maBufferAnchors", ctypes.c_char * 0x1000),  # cTkFixedArray<cTkVector3,256>
        ("mabBufferProtected", ctypes.c_char * 0x100),  # cTkFixedArray<bool,256>
        ("maEdits", ctypes.c_char * 0x3A980),  # cTkFixedArray<cGcTerrainEdit,30000>
    ]
    mau64GalacticAddresses: bytes
    maiBufferSizes: bytes
    maBufferAges: bytes
    maBufferAnchors: bytes
    mabBufferProtected: bytes
    maEdits: bytes

class cGcPersistentBaseEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mu64Timestamp", ctypes.c_uint64),
        ("mObjectID", ctypes.c_ulonglong),
        ("mu64UserData", ctypes.c_uint64),
        ("mPosition", common.Vector3f),
        ("mUp", common.Vector3f),
        ("mAt", common.Vector3f),
        ("macMessage", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    mu64Timestamp: int
    mObjectID: int
    mu64UserData: int
    mPosition: common.Vector3f
    mUp: common.Vector3f
    mAt: common.Vector3f
    macMessage: bytes

class cGcProceduralProductDeployable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mBaseID", ctypes.c_ulonglong),
        ("miVariants", ctypes.c_int32),
    ]
    mBaseID: int
    miVariants: int

class cGcProceduralProductTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def Save(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0xE520),  # cTkFixedArray<cGcProceduralProductData,26>
    ]
    maTable: bytes

class cGcTradingSupplyData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mu64GalacticAddress", ctypes.c_uint64),
        ("mfSupply", ctypes.c_float),
        ("mfDemand", ctypes.c_float),
        ("mProduct", ctypes.c_ulonglong),
        ("mu64Timestamp", ctypes.c_uint64),
    ]
    mu64GalacticAddress: int
    mfSupply: float
    mfDemand: float
    mProduct: int
    mu64Timestamp: int

class cGcPortalSaveData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPortalSeed", common.GcSeed),
        ("mu64LastPortalUA", ctypes.c_uint64),
        ("mbIsStoryPortal", ctypes.c_ubyte),
    ]
    mPortalSeed: common.GcSeed
    mu64LastPortalUA: int
    mbIsStoryPortal: bool

class cGcExpeditionEventSaveData(ctypes.Structure):
    def tilde_cGcExpeditionEventSaveData(): pass

    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mSeed", common.GcSeed),
        ("mu64UA", ctypes.c_uint64),
        ("maiAffectedFrigateIndices", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
        ("maiRepairingFrigateIndices", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
        ("maiAffectedFrigateResponses", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
        ("mEventID", ctypes.c_ulonglong),
        ("mInterventionEventID", ctypes.c_ulonglong),
        ("macOverriddenDescription", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mbWhaleEvent", ctypes.c_ubyte),
        ("mOverriddenReward", ctypes.c_ulonglong),
        ("mbSuccess", ctypes.c_ubyte),
        ("mbIsInterventionEvent", ctypes.c_ubyte),
        ("mbAvoidedIntervention", ctypes.c_ubyte),
    ]
    mSeed: common.GcSeed
    mu64UA: int
    maiAffectedFrigateIndices: bytes
    maiRepairingFrigateIndices: bytes
    maiAffectedFrigateResponses: bytes
    mEventID: int
    mInterventionEventID: int
    macOverriddenDescription: bytes
    mbWhaleEvent: bool
    mOverriddenReward: int
    mbSuccess: bool
    mbIsInterventionEvent: bool
    mbAvoidedIntervention: bool

class cGcPersistedStatData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mGroupId", ctypes.c_ulonglong),
        ("mStatId", ctypes.c_ulonglong),
    ]
    mGroupId: int
    mStatId: int

class cGcTextStyleOutline(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mOutlineColour", common.Colour),
        ("mOutlineOffset", common.Vector2f),
    ]
    mOutlineColour: common.Colour
    mOutlineOffset: common.Vector2f

class cGcTextStylePlain(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcTextStyleShadow(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mShadowColour", common.Colour),
        ("mShadowOffset", common.Vector2f),
    ]
    mShadowColour: common.Colour
    mShadowOffset: common.Vector2f

class cGcProtectedLocation(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mLocation", common.Vector3f),
        ("mfRadius", ctypes.c_float),
    ]
    mLocation: common.Vector3f
    mfRadius: float

class cGcUAProtectedLocations(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mu64UA", ctypes.c_uint64),
        ("maProtectedLocations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcProtectedLocation>
    ]
    mu64UA: int
    maProtectedLocations: bytes

class cGcSettlementProductionSlotData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mElementId", ctypes.c_ulonglong),
        ("mu64LastChangeTimestamp", ctypes.c_uint64),
        ("miAmount", ctypes.c_int32),
    ]
    mElementId: int
    mu64LastChangeTimestamp: int
    miAmount: int

class cGcNGuiFileBrowserRecents(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacRecents", ctypes.c_char * 0xA00),  # cTkFixedArray<cTkFixedString<256,char>,10>
    ]
    maacRecents: bytes

class cGcSyncBufferSaveData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mu64SpaceAddress", ctypes.c_uint64),
        ("macOwnerOnlineId", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macOwnerPlatformId", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("muBufferVersion", ctypes.c_uint32),
        ("muItemsCount", ctypes.c_uint32),
    ]
    mu64SpaceAddress: int
    macOwnerOnlineId: bytes
    macOwnerPlatformId: bytes
    muBufferVersion: int
    muItemsCount: int

class cGcSyncBufferSaveDataArray(ctypes.Structure):
    def cGcSyncBufferSaveDataArray(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSyncBufferSaveData>
    ]
    maData: bytes

class cGcMaintenanceSaveKey(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromDocument(): pass

    def ReadFromXMLNode(): pass

    def WriteToDocument(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mPosition", common.Vector3f),
        ("miLocation", ctypes.c_int32),
    ]
    mPosition: common.Vector3f
    miLocation: int

class cGcNGuiSpecialTextImageData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mbScaleToFitFont", ctypes.c_ubyte),
        ("mSize", common.Vector2f),
        ("macPath", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mName: int
    mbScaleToFitFont: bool
    mSize: common.Vector2f
    macPath: bytes

class cGcNGuiSpecialTextImages(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSpecialImages", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNGuiSpecialTextImageData>
    ]
    maSpecialImages: bytes

class cGcNGuiStyleAnimationKeyframeData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfPosition", ctypes.c_float),
        ("maStyleProperties", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
    ]
    mfPosition: float
    maStyleProperties: bytes

class cGcNGuiStyleAnimationData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfLength", ctypes.c_float),
        ("mbLoop", ctypes.c_ubyte),
        ("mbAnimateByDefault", ctypes.c_ubyte),
        ("maKeyFrames", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNGuiStyleAnimationKeyframeData>
    ]
    mfLength: float
    mbLoop: bool
    mbAnimateByDefault: bool
    maKeyFrames: bytes

class cGcHUDMarkerData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macIcon", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macIconBehind", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macDistance", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macIcon: bytes
    macIconBehind: bytes
    macDistance: bytes

class cGcDiscoveryHelperTimings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfDiscoverPlanetTotalTime", ctypes.c_float),
        ("mfDiscoverPlanetMessageWait", ctypes.c_float),
        ("mfDiscoverPlanetMessageTime", ctypes.c_float),
    ]
    mfDiscoverPlanetTotalTime: float
    mfDiscoverPlanetMessageWait: float
    mfDiscoverPlanetMessageTime: float

class cGcFontData(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("miMinCharWidth", ctypes.c_int32),
    ]
    macFile: bytes
    miMinCharWidth: int

class cGcFontTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x420),  # cTkFixedArray<cGcFontData,8>
    ]
    maTable: bytes

class cGcNGuiSpecialTextStyles(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSpecialStyles", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcNGuiSpecialTextStyleData>
    ]
    maSpecialStyles: bytes

class cGcHUDStartupTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfFadeInFlashTime", ctypes.c_float),
        ("mfLookSpeed", ctypes.c_float),
        ("mfStartHoldTime", ctypes.c_float),
        ("mfButtonFlashRate", ctypes.c_float),
        ("mfButtonFlashAlpha", ctypes.c_float),
        ("mfBackgroundAlpha", ctypes.c_float),
        ("maHUDStartup", ctypes.c_char * 0x138),  # cTkFixedArray<cGcHUDStartup,13>
    ]
    mfFadeInFlashTime: float
    mfLookSpeed: float
    mfStartHoldTime: float
    mfButtonFlashRate: float
    mfButtonFlashAlpha: float
    mfBackgroundAlpha: float
    maHUDStartup: bytes

class cGcStyleProp_Colour(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    _fields_ = [
        ("mColour", common.Colour),
    ]
    mColour: common.Colour

class cGcStyleProp_Font(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miFontIndex", ctypes.c_int32),
    ]
    miFontIndex: int

class cGcStyleProp_Size(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfFontSize", ctypes.c_float),
    ]
    mfFontSize: float

class cGcModelViewCollection(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maModelViews", ctypes.c_char * 0x1C30),  # cTkFixedArray<cTkModelRendererData,41>
    ]
    maModelViews: bytes

class cGcPunctuationDelay(ctypes.Structure):
    def SetDefaults(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macPunctuation", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mfDelay", ctypes.c_float),
    ]
    macPunctuation: bytes
    mfDelay: float

class cGcPunctuationDelayData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maPunctuationList", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPunctuationDelay>
        ("mfDefaultDelay", ctypes.c_float),
    ]
    maPunctuationList: bytes
    mfDefaultDelay: float

class cGcActionSets(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maActionSets", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcActionSet>
    ]
    maActionSets: bytes

class cGcPunctuationDelayTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maPunctuationDelays", ctypes.c_char * 0x78),  # cTkFixedArray<cGcPunctuationDelayData,5>
    ]
    maPunctuationDelays: bytes

class cGcInputActionInfoMap(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maActionMap", ctypes.c_char * 0x30E10),  # cTkFixedArray<cGcInputActionInfo,291>
    ]
    maActionMap: bytes

class cGcInputBindings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maInputBindingSets", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcInputBindingSet>
    ]
    maInputBindingSets: bytes

class cGcTerrainOverlayColours(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfCutoff", ctypes.c_float),
        ("mfScale", ctypes.c_float),
        ("mfStrength", ctypes.c_float),
        ("mfFlightStrength", ctypes.c_float),
        ("mfPulseStrength", ctypes.c_float),
        ("mfPulsePeriod", ctypes.c_float),
    ]
    mfCutoff: float
    mfScale: float
    mfStrength: float
    mfFlightStrength: float
    mfPulseStrength: float
    mfPulsePeriod: float

class RegistercGcTerrainGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcUIGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcCameraAnomalySetupData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mCameraAt", common.Vector4f),
        ("mCameraUp", common.Vector4f),
        ("mCameraOffset", common.Vector4f),
        ("mSunDirection", common.Vector4f),
    ]
    mCameraAt: common.Vector4f
    mCameraUp: common.Vector4f
    mCameraOffset: common.Vector4f
    mSunDirection: common.Vector4f

class cGcCameraSpawnSetupData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDistance", ctypes.c_float),
        ("mbInFrontOfShip", ctypes.c_ubyte),
        ("mfYawProportion", ctypes.c_float),
        ("mfHorizontalProportion", ctypes.c_float),
    ]
    mfDistance: float
    mbInFrontOfShip: bool
    mfYawProportion: float
    mfHorizontalProportion: float

class cGcCameraShakeCapturedData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbActive", ctypes.c_ubyte),
        ("mfShakeStrength", ctypes.c_float),
        ("mfShakeFrequency", ctypes.c_float),
        ("mfVibrateStrength", ctypes.c_float),
        ("mfVibrateFrequency", ctypes.c_float),
    ]
    mbActive: bool
    mfShakeStrength: float
    mfShakeFrequency: float
    mfVibrateStrength: float
    mfVibrateFrequency: float

class cGcCameraShakeMechanicalData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbActive", ctypes.c_ubyte),
        ("mShakeStrength", common.Vector3f),
        ("mShakeFrequency", common.Vector3f),
        ("mExtraShakeFrequency", common.Vector3f),
        ("mVibrateStrength", common.Vector3f),
        ("mVibrateFrequency", common.Vector3f),
        ("mExtraVibrateFrequency", common.Vector3f),
    ]
    mbActive: bool
    mShakeStrength: common.Vector3f
    mShakeFrequency: common.Vector3f
    mExtraShakeFrequency: common.Vector3f
    mVibrateStrength: common.Vector3f
    mVibrateFrequency: common.Vector3f
    mExtraVibrateFrequency: common.Vector3f

class cGcSeasonalRingData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfRingSize", ctypes.c_float),
        ("mfRingOpacity", ctypes.c_float),
        ("mfCoreOpacity", ctypes.c_float),
    ]
    mfRingSize: float
    mfRingOpacity: float
    mfCoreOpacity: float

class cGcSeasonalRingArray(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maSeasonalRingData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSeasonalRingData>
    ]
    maSeasonalRingData: bytes

class cGcWorldUISettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfUIWorldQuadOffset", ctypes.c_float),
        ("mfUIWorldQuadOffsetV2", ctypes.c_float),
        ("mfUIWorldQuadOffsetBuildMenu", ctypes.c_float),
        ("mfUIWorldQuadOffsetBuildMenuV2", ctypes.c_float),
        ("mfUIWorldQuadSideOffset", ctypes.c_float),
        ("mfUIWorldQuadShipAddOffset", ctypes.c_float),
        ("mfUIWorldQuadShipAddOffsetV2", ctypes.c_float),
        ("mfUIWorldQuadScale", ctypes.c_float),
        ("mfUIWorldQuadShipScale", ctypes.c_float),
        ("mfHUDDefWorldQuadOffset", ctypes.c_float),
        ("mfHUDDefWorldQuadOffsetV2", ctypes.c_float),
        ("mfHUDDefWorldQuadShipAddOffset", ctypes.c_float),
        ("mfHUDDefWorldQuadShipAddOffsetV2", ctypes.c_float),
        ("mfHUDWorldQuadScale", ctypes.c_float),
        ("mfHUDWorldQuadShipScale", ctypes.c_float),
        ("mfHUDInterpSpeed", ctypes.c_float),
        ("mfHUDThresholdVert", ctypes.c_float),
        ("mfHUDThresholdHorz", ctypes.c_float),
        ("mfGameModeSelectorQuadOffset", ctypes.c_float),
        ("mfGameModeSelectorQuadOffsetV2", ctypes.c_float),
    ]
    mfUIWorldQuadOffset: float
    mfUIWorldQuadOffsetV2: float
    mfUIWorldQuadOffsetBuildMenu: float
    mfUIWorldQuadOffsetBuildMenuV2: float
    mfUIWorldQuadSideOffset: float
    mfUIWorldQuadShipAddOffset: float
    mfUIWorldQuadShipAddOffsetV2: float
    mfUIWorldQuadScale: float
    mfUIWorldQuadShipScale: float
    mfHUDDefWorldQuadOffset: float
    mfHUDDefWorldQuadOffsetV2: float
    mfHUDDefWorldQuadShipAddOffset: float
    mfHUDDefWorldQuadShipAddOffsetV2: float
    mfHUDWorldQuadScale: float
    mfHUDWorldQuadShipScale: float
    mfHUDInterpSpeed: float
    mfHUDThresholdVert: float
    mfHUDThresholdHorz: float
    mfGameModeSelectorQuadOffset: float
    mfGameModeSelectorQuadOffsetV2: float

class cGcBootLogoData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacTextures", ctypes.c_char * 0x400),  # cTkFixedArray<cTkFixedString<256,char>,4>
        ("mafDisplayTime", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
    ]
    maacTextures: bytes
    mafDisplayTime: bytes

class cGcJourney(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maCategories", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcJourneyCategory>
    ]
    maCategories: bytes

class RegistercGcSolarGenerationGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcJourneyMedalTiers(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miNone", ctypes.c_int32),
        ("miBronze", ctypes.c_int32),
        ("miSilver", ctypes.c_int32),
        ("miGold", ctypes.c_int32),
    ]
    miNone: int
    miBronze: int
    miSilver: int
    miGold: int

class RegistercGcSkyGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcPresetTextureData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macFilename", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
    ]
    macName: bytes
    macFilename: bytes

class cGcSmokeTestOptions(ctypes.Structure):
    def NGuiRenderCallback(): pass

    def ReadFromXMLNode(): pass

    def ReadModFromFile(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfInitialPause", ctypes.c_float),
        ("mfSmokeTestFlashTimeDuration", ctypes.c_float),
        ("mfCameraHeight", ctypes.c_float),
        ("mfCameraFastHeight", ctypes.c_float),
        ("mfCameraMoveSpeed", ctypes.c_float),
        ("mfCameraFastMoveSpeed", ctypes.c_float),
        ("mfCameraRotateSpeed", ctypes.c_float),
        ("mfCameraPitchAngleDeg", ctypes.c_float),
        ("mfCameraPitchSpeedRange", ctypes.c_float),
        ("mbGifMode", ctypes.c_ubyte),
        ("miGifFrames", ctypes.c_int32),
        ("mfGifTimeBetweenKeyframes", ctypes.c_float),
        ("mfPlanetFlightTime", ctypes.c_float),
        ("mfPlanetFlightTimeout", ctypes.c_float),
        ("mfSmokeBotTurnAngle", ctypes.c_float),
        ("miSmokeBotNumWalksBeforeWarp", ctypes.c_int32),
    ]
    mfInitialPause: float
    mfSmokeTestFlashTimeDuration: float
    mfCameraHeight: float
    mfCameraFastHeight: float
    mfCameraMoveSpeed: float
    mfCameraFastMoveSpeed: float
    mfCameraRotateSpeed: float
    mfCameraPitchAngleDeg: float
    mfCameraPitchSpeedRange: float
    mbGifMode: bool
    miGifFrames: int
    mfGifTimeBetweenKeyframes: float
    mfPlanetFlightTime: float
    mfPlanetFlightTimeout: float
    mfSmokeBotTurnAngle: float
    miSmokeBotNumWalksBeforeWarp: int

class RegistercGcSmokeTestOptions_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcScreenFilterData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mLocText", ctypes.c_ulonglong),
        ("macFilename", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mfFadeDistance", ctypes.c_float),
        ("mbSelectableInPhotoMode", ctypes.c_ubyte),
        ("mfHdrAreaAdjust", ctypes.c_float),
    ]
    mLocText: int
    macFilename: bytes
    mfFadeDistance: float
    mbSelectableInPhotoMode: bool
    mfHdrAreaAdjust: float

class cGcScreenFilterTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maFilters", ctypes.c_char * 0x3860),  # cTkFixedArray<cGcScreenFilterData,82>
    ]
    maFilters: bytes

class cGcTexturePrefetchData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maacTextures", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
    ]
    maacTextures: bytes

class cGcMessageProjectileLaunch(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMessageRequestTakeOff(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDelay", ctypes.c_float),
        ("mbImmediatelyDissolveNPC", ctypes.c_ubyte),
    ]
    mfDelay: float
    mbImmediatelyDissolveNPC: bool

class cGcMessageRequestWarp(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfDelay", ctypes.c_float),
    ]
    mfDelay: float

class cGcMessageSubstanceMined(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("miAmount", ctypes.c_int32),
        ("mSubstance", ctypes.c_ulonglong),
    ]
    miAmount: int
    mSubstance: int

class cGcMessageTitanFall(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcMessageTrackTargetAlert(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miAttacker", ctypes.c_int32),
        ("miVictim", ctypes.c_int32),
        ("mAlertPos", common.Vector3f),
        ("mbPrimary", ctypes.c_ubyte),
    ]
    miAttacker: int
    miVictim: int
    mAlertPos: common.Vector3f
    mbPrimary: bool

class cGcMessageUpdateFrigateSpeed(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfStartSpeed", ctypes.c_float),
        ("mfTargetSpeed", ctypes.c_float),
    ]
    mfStartSpeed: float
    mfTargetSpeed: float

class RegistercGcGraphicsGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcSceneOptions(ctypes.Structure):
    def NGuiRenderCallback(): pass

    def ReadFromXMLNode(): pass

    def ReadModFromFile(): pass

    def SetDefaults(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miRecentToolboxIndex", ctypes.c_int32),
        ("miSelectedToolboxIndex", ctypes.c_int32),
        ("mbOverrideBiome", ctypes.c_ubyte),
        ("macBiomeFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbOverrideWaterBiome", ctypes.c_ubyte),
        ("macWaterBiomeFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbOverrideCaveBiome", ctypes.c_ubyte),
        ("macCaveBiomeFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbOverrideTerrain", ctypes.c_ubyte),
        ("macTerrainFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mbOverrideAtmosphere", ctypes.c_ubyte),
        ("macAtmosphereFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maForceResource", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkVector3>
        ("mfForceResourceSize", ctypes.c_float),
    ]
    miRecentToolboxIndex: int
    miSelectedToolboxIndex: int
    mbOverrideBiome: bool
    macBiomeFile: bytes
    mbOverrideWaterBiome: bool
    macWaterBiomeFile: bytes
    mbOverrideCaveBiome: bool
    macCaveBiomeFile: bytes
    mbOverrideTerrain: bool
    macTerrainFile: bytes
    mbOverrideAtmosphere: bool
    macAtmosphereFile: bytes
    maForceResource: bytes
    mfForceResourceSize: float

class RegistercGcSceneOptions_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcScratchpadGlobals(ctypes.Structure):
    def NGuiRenderCallback(): pass

    def ReadModFromFile(): pass

    _fields_ = [
        ("maIBLMaps", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPresetTextureData>
        ("maTerrainTextures", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPresetTextureData>
        ("maOverlayTextures", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPresetTextureData>
        ("maTerrainColours", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkColour>
    ]
    maIBLMaps: bytes
    maTerrainTextures: bytes
    maOverlayTextures: bytes
    maTerrainColours: bytes

class RegistercGcScratchpadGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcSettlementGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcSimulationGlobals(ctypes.Structure):
    def tilde_cGcSimulationGlobals(): pass

    def NGuiRenderCallback(): pass

    def ReadFromXMLNode(): pass

    def ReadModFromFile(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macStartingSceneFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maMultitoolPool", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcMultitoolPoolData>
        ("mu64ProceduralBuildingsGenerationSeed", ctypes.c_uint64),
        ("macSpaceStationFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macNexusFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macNexusExteriorFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPlaceMarkerFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAbandonedSpaceStationFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPirateSystemSpaceStationFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maacBlackHoleAnomalies", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
        ("maacAtlasStationAnomalies", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
        ("macNone", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPlacementDroneFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macHeavyAirCave", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macHeavyAirUnderwater", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macHeavyAirAbandonedFreighter", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macHeavyAirSpaceStorm", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maacPrefetchScenegraphResources", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
        ("maacPrefetchMaterialResources", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
        ("macPlanetAtmosphereFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPlanetAtmosphereMaterialFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPlanetRingFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPlanetRingMaterialFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPlanetMaterialFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPlaterWaterMaterialFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maacPlanetTerrainMaterials", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
        ("maacPrefetchTextureResources", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<128,char> >
        ("mfWarpTunnelScale", ctypes.c_float),
        ("macWarpTunnelFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macBlackHoleTunnelFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macTeleportTunnelFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPortalTunnelFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macPortalStoryTunnelFile", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macStartingSceneFile: bytes
    maMultitoolPool: bytes
    mu64ProceduralBuildingsGenerationSeed: int
    macSpaceStationFile: bytes
    macNexusFile: bytes
    macNexusExteriorFile: bytes
    macPlaceMarkerFile: bytes
    macAbandonedSpaceStationFile: bytes
    macPirateSystemSpaceStationFile: bytes
    maacBlackHoleAnomalies: bytes
    maacAtlasStationAnomalies: bytes
    macNone: bytes
    macPlacementDroneFile: bytes
    macHeavyAirCave: bytes
    macHeavyAirUnderwater: bytes
    macHeavyAirAbandonedFreighter: bytes
    macHeavyAirSpaceStorm: bytes
    maacPrefetchScenegraphResources: bytes
    maacPrefetchMaterialResources: bytes
    macPlanetAtmosphereFile: bytes
    macPlanetAtmosphereMaterialFile: bytes
    macPlanetRingFile: bytes
    macPlanetRingMaterialFile: bytes
    macPlanetMaterialFile: bytes
    macPlaterWaterMaterialFile: bytes
    maacPlanetTerrainMaterials: bytes
    maacPrefetchTextureResources: bytes
    mfWarpTunnelScale: float
    macWarpTunnelFile: bytes
    macBlackHoleTunnelFile: bytes
    macTeleportTunnelFile: bytes
    macPortalTunnelFile: bytes
    macPortalStoryTunnelFile: bytes

class RegistercGcSimulationGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcCameraGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcSettlementGlobals(ctypes.Structure):
    def tilde_cGcSettlementGlobals(): pass

    def FixUp(): pass

    def NGuiRenderCallback(): pass

    def ReadFromXMLNode(): pass

    def ReadModFromFile(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maScanEventsThatPreventSentinelAlert", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
        ("mu64BuildingUpgradeTimeInSeconds", ctypes.c_uint64),
        ("mu64BuildingFreeUpgradeTimeInSeconds", ctypes.c_uint64),
        ("miJudgementWaitTimeMin", ctypes.c_int32),
        ("miJudgementWaitTimeMax", ctypes.c_int32),
        ("mfBuildingJudgementPriorityChance", ctypes.c_float),
        ("mfVisitorJudgementSelector", ctypes.c_float),
        ("mfPolicyJudgementSelector", ctypes.c_float),
        ("mfRequestJudgementSelector", ctypes.c_float),
        ("mfConflictJudgementSelector", ctypes.c_float),
        ("miMaxNPCPopulation", ctypes.c_int32),
        ("mbDebugForceShowHiddenPerks", ctypes.c_ubyte),
        ("mfSettlementEntryMessageDistance", ctypes.c_float),
        ("mSettlementMiniExpeditionMissionID", ctypes.c_ulonglong),
        ("mfSettlementMiniExpeditionSuccessChance", ctypes.c_float),
        ("mau64SettlementBuildingTimes", ctypes.c_char * 0x1A0),  # cTkFixedArray<unsigned __int64,52>
        ("maSettlementBuildingCosts", ctypes.c_char * 0x3A80),  # cTkFixedArray<cGcSettlementBuildingCost,52>
        ("maJudgements", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementJudgementData>
        ("maCustomJudgements", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementCustomJudgement>
        ("maSettlementBuildingStatRewards", ctypes.c_char * 0x340),  # cTkFixedArray<cGcSettlementStatChangeArray,52>
        ("mSettlementCostGek", ctypes.c_ulonglong),
        ("mSettlementCostKorvax", ctypes.c_ulonglong),
        ("mSettlementCostVykeen", ctypes.c_ulonglong),
        ("miMaxPerksCount", ctypes.c_int32),
        ("miMinInitialPositivePerksCount", ctypes.c_int32),
        ("miMaxInitialPositivePerksCount", ctypes.c_int32),
        ("miMinInitialNegativePerksCount", ctypes.c_int32),
        ("miMaxInitialNegativePerksCount", ctypes.c_int32),
        ("maiStatsMinValues", ctypes.c_char * 0x1C),  # cTkFixedArray<int,7>
        ("maiStatsMaxValues", ctypes.c_char * 0x1C),  # cTkFixedArray<int,7>
        ("maiInitialStatsMinValues", ctypes.c_char * 0x1C),  # cTkFixedArray<int,7>
        ("miInitialBuildingCountMin", ctypes.c_int32),
        ("miInitialBuildingCountMax", ctypes.c_int32),
        ("maiInitialStatsMaxValues", ctypes.c_char * 0x1C),  # cTkFixedArray<int,7>
        ("mabStatIsGoodWhenPositive", ctypes.c_char * 0x7),  # cTkFixedArray<bool,7>
        ("mafNormalisedStatBadThresholds", ctypes.c_char * 0x1C),  # cTkFixedArray<float,7>
        ("mafNormalisedStatGoodThresholds", ctypes.c_char * 0x1C),  # cTkFixedArray<float,7>
        ("mPositiveStatColour", common.Colour),
        ("mNegativeStatColour", common.Colour),
        ("mu64ProductionSlotTimerOffsetInSeconds", ctypes.c_uint64),
        ("mu64ProductionCycleDurationInSeconds", ctypes.c_uint64),
        ("mu64AlertCycleDurationInSeconds", ctypes.c_uint64),
        ("miMaxProductionSlotUnits", ctypes.c_int32),
        ("miProductUnitsPerCycleRateModifier", ctypes.c_int32),
        ("miSubstanceUnitsPerCycleRateModifier", ctypes.c_int32),
        ("miInitialDebtCycles", ctypes.c_int32),
        ("miDailyDebtPaymentModifier", ctypes.c_int32),
        ("miAlertUnitsPerCycleRateModifier", ctypes.c_int32),
        ("maiStatProductivityContributionModifiers", ctypes.c_char * 0x1C),  # cTkFixedArray<int,7>
        ("mabStatProductionIsNegativeWhenBad", ctypes.c_char * 0x7),  # cTkFixedArray<bool,7>
        ("maPolicyPerks", ctypes.c_char * 0x70),  # cTkFixedArray<TkID<128>,7>
        ("maResearchPerks", ctypes.c_char * 0x70),  # cTkFixedArray<TkID<128>,7>
        ("maAltResearchPerks", ctypes.c_char * 0x70),  # cTkFixedArray<TkID<128>,7>
        ("maPerkStatStrengthValues", ctypes.c_char * 0x188),  # cTkFixedArray<cGcSettlementStatStrengthData,7>
        ("maGekProductionElements", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maKorvaxProductionElements", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maVykeenProductionElements", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maPoorProductionElements", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maNeutralProductionElements", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maRichProductionElements", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maJudgementUpdateTitles", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maJudgementUpdateMainText", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maJudgementUpdateSubtitles", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maPositiveStatChangeOSDLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maNegativeStatChangeOSDLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maPositiveStatChangeSubstances", ctypes.c_char * 0x70),  # cTkFixedArray<TkID<128>,7>
        ("maNegativeStatChangeSubstances", ctypes.c_char * 0x70),  # cTkFixedArray<TkID<128>,7>
        ("maPositiveFakePerkOSDLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maNegativeFakePerkOSDLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maProcPerkDescriptions", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maShortPolicyLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maLongPolicyLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maShortResearchLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maLongResearchLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maShortAltResearchLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maLongAltResearchLocIDs", ctypes.c_char * 0xE0),  # cTkFixedArray<TkID<256>,7>
        ("maGekGifts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementGiftDetails>
        ("maKorvaxGifts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementGiftDetails>
        ("maVykeenGifts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementGiftDetails>
        ("maGifts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementGiftDetails>
        ("maTechGiftPerks", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("maJobTypes", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSettlementJobDetails>
    ]
    maScanEventsThatPreventSentinelAlert: bytes
    mu64BuildingUpgradeTimeInSeconds: int
    mu64BuildingFreeUpgradeTimeInSeconds: int
    miJudgementWaitTimeMin: int
    miJudgementWaitTimeMax: int
    mfBuildingJudgementPriorityChance: float
    mfVisitorJudgementSelector: float
    mfPolicyJudgementSelector: float
    mfRequestJudgementSelector: float
    mfConflictJudgementSelector: float
    miMaxNPCPopulation: int
    mbDebugForceShowHiddenPerks: bool
    mfSettlementEntryMessageDistance: float
    mSettlementMiniExpeditionMissionID: int
    mfSettlementMiniExpeditionSuccessChance: float
    mau64SettlementBuildingTimes: bytes
    maSettlementBuildingCosts: bytes
    maJudgements: bytes
    maCustomJudgements: bytes
    maSettlementBuildingStatRewards: bytes
    mSettlementCostGek: int
    mSettlementCostKorvax: int
    mSettlementCostVykeen: int
    miMaxPerksCount: int
    miMinInitialPositivePerksCount: int
    miMaxInitialPositivePerksCount: int
    miMinInitialNegativePerksCount: int
    miMaxInitialNegativePerksCount: int
    maiStatsMinValues: bytes
    maiStatsMaxValues: bytes
    maiInitialStatsMinValues: bytes
    miInitialBuildingCountMin: int
    miInitialBuildingCountMax: int
    maiInitialStatsMaxValues: bytes
    mabStatIsGoodWhenPositive: bytes
    mafNormalisedStatBadThresholds: bytes
    mafNormalisedStatGoodThresholds: bytes
    mPositiveStatColour: common.Colour
    mNegativeStatColour: common.Colour
    mu64ProductionSlotTimerOffsetInSeconds: int
    mu64ProductionCycleDurationInSeconds: int
    mu64AlertCycleDurationInSeconds: int
    miMaxProductionSlotUnits: int
    miProductUnitsPerCycleRateModifier: int
    miSubstanceUnitsPerCycleRateModifier: int
    miInitialDebtCycles: int
    miDailyDebtPaymentModifier: int
    miAlertUnitsPerCycleRateModifier: int
    maiStatProductivityContributionModifiers: bytes
    mabStatProductionIsNegativeWhenBad: bytes
    maPolicyPerks: bytes
    maResearchPerks: bytes
    maAltResearchPerks: bytes
    maPerkStatStrengthValues: bytes
    maGekProductionElements: bytes
    maKorvaxProductionElements: bytes
    maVykeenProductionElements: bytes
    maPoorProductionElements: bytes
    maNeutralProductionElements: bytes
    maRichProductionElements: bytes
    maJudgementUpdateTitles: bytes
    maJudgementUpdateMainText: bytes
    maJudgementUpdateSubtitles: bytes
    maPositiveStatChangeOSDLocIDs: bytes
    maNegativeStatChangeOSDLocIDs: bytes
    maPositiveStatChangeSubstances: bytes
    maNegativeStatChangeSubstances: bytes
    maPositiveFakePerkOSDLocIDs: bytes
    maNegativeFakePerkOSDLocIDs: bytes
    maProcPerkDescriptions: bytes
    maShortPolicyLocIDs: bytes
    maLongPolicyLocIDs: bytes
    maShortResearchLocIDs: bytes
    maLongResearchLocIDs: bytes
    maShortAltResearchLocIDs: bytes
    maLongAltResearchLocIDs: bytes
    maGekGifts: bytes
    maKorvaxGifts: bytes
    maVykeenGifts: bytes
    maGifts: bytes
    maTechGiftPerks: bytes
    maJobTypes: bytes

class cGcFreighterNPCSpawnPriority(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mafPriorityScale", ctypes.c_char * 0x10),  # cTkDynamicArray<float>
    ]
    mafPriorityScale: bytes

class cGcNPCNavSubgraphNodeTypeConnectivity(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfInternalConnection", ctypes.c_float),
        ("mfExternalConnection", ctypes.c_float),
        ("mfPathToPOI", ctypes.c_float),
        ("mfConnectionToPOI", ctypes.c_float),
    ]
    mfInternalConnection: float
    mfExternalConnection: float
    mfPathToPOI: float
    mfConnectionToPOI: float

class cGcFreighterRoomNPCData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mRoomID", ctypes.c_ulonglong),
        ("mafSpawnCapacity", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
        ("mafPOISelectionWeight", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
    ]
    mRoomID: int
    mafSpawnCapacity: bytes
    mafPOISelectionWeight: bytes

class RegistercGcFreighterBaseGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcGalaxyGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcWaterGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcPaletteList(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def Save(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maPalettes", ctypes.c_char * 0xEBA0),  # cTkFixedArray<cGcPaletteData,58>
    ]
    maPalettes: bytes

class RegistercGcAISpaceshipGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcItemCostTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maItems", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcItemCostData>
    ]
    maItems: bytes

class RegistercGcVehicleGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcRichPresenceGlobals(ctypes.Structure):
    def NGuiRenderCallback(): pass

    def ReadFromXMLNode(): pass

    def ReadModFromFile(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbShowOnScreen", ctypes.c_ubyte),
        ("mfIdleThreshold", ctypes.c_float),
        ("mfEvaluationPeriod", ctypes.c_float),
        ("mfPublishPeriod", ctypes.c_float),
        ("miPlanetLocationPriority", ctypes.c_int32),
        ("miSpaceLocationPriority", ctypes.c_int32),
        ("miStormLocationPriority", ctypes.c_int32),
        ("miSpaceCombatPriority", ctypes.c_int32),
        ("miGameModePriority", ctypes.c_int32),
    ]
    mbShowOnScreen: bool
    mfIdleThreshold: float
    mfEvaluationPeriod: float
    mfPublishPeriod: float
    miPlanetLocationPriority: int
    miSpaceLocationPriority: int
    miStormLocationPriority: int
    miSpaceCombatPriority: int
    miGameModePriority: int

class RegistercGcRichPresenceGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcRobotGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcDebugOptions_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcInventoryStoreBalance(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("miPlayerPersonalInventoryTechWidth", ctypes.c_int32),
        ("miPlayerPersonalInventoryTechHeight", ctypes.c_int32),
        ("miPlayerPersonalInventoryCargoWidth", ctypes.c_int32),
        ("miPlayerPersonalInventoryCargoHeight", ctypes.c_int32),
        ("mfDeconstructRefundPercentage", ctypes.c_float),
    ]
    miPlayerPersonalInventoryTechWidth: int
    miPlayerPersonalInventoryTechHeight: int
    miPlayerPersonalInventoryCargoWidth: int
    miPlayerPersonalInventoryCargoHeight: int
    mfDeconstructRefundPercentage: float

class cGcModSettingsInfo(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("macAuthor", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mu64ID", ctypes.c_uint64),
        ("mu64AuthorID", ctypes.c_uint64),
        ("mu64LastUpdated", ctypes.c_uint64),
        ("mu16LoadOrder", ctypes.c_uint16),
        ("mbEnabled", ctypes.c_ubyte),
        ("mbEnabledVR", ctypes.c_ubyte),
        ("mau64Dependencies", ctypes.c_char * 0x10),  # cTkDynamicArray<unsigned __int64>
    ]
    macName: bytes
    macAuthor: bytes
    mu64ID: int
    mu64AuthorID: int
    mu64LastUpdated: int
    mu16LoadOrder: int
    mbEnabled: bool
    mbEnabledVR: bool
    mau64Dependencies: bytes

class cGcModSettings(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcModSettingsInfo>
    ]
    maData: bytes

class RegistercGcMultiplayerGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcPlacementGlobals(ctypes.Structure):
    def NGuiRenderCallback(): pass

    def ReadFromXMLNode(): pass

    def ReadModFromFile(): pass

    def SetDefaults(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMinFrequency", ctypes.c_float),
        ("mfMinDensity", ctypes.c_float),
        ("mfMinPatchSize", ctypes.c_float),
        ("miMinPatchVariance", ctypes.c_int32),
        ("mfMaxFrequency", ctypes.c_float),
        ("mfMaxDensity", ctypes.c_float),
        ("mfMaxPatchSize", ctypes.c_float),
        ("miMaxPatchVariance", ctypes.c_int32),
        ("mfInterpValueVariance", ctypes.c_float),
        ("mfHighInterpValue", ctypes.c_float),
        ("mfMidInterpValue", ctypes.c_float),
        ("mfLowInterpValue", ctypes.c_float),
        ("mfMultiplyLodDistance", ctypes.c_float),
        ("mfAddToLodDistance", ctypes.c_float),
        ("mafLodDistancesDetail", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
        ("mafLodDistancesObject", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
        ("mafLodDistancesLandmark", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
        ("mafLodDistancesDistant", ctypes.c_char * 0x14),  # cTkFixedArray<float,5>
    ]
    mfMinFrequency: float
    mfMinDensity: float
    mfMinPatchSize: float
    miMinPatchVariance: int
    mfMaxFrequency: float
    mfMaxDensity: float
    mfMaxPatchSize: float
    miMaxPatchVariance: int
    mfInterpValueVariance: float
    mfHighInterpValue: float
    mfMidInterpValue: float
    mfLowInterpValue: float
    mfMultiplyLodDistance: float
    mfAddToLodDistance: float
    mafLodDistancesDetail: bytes
    mafLodDistancesObject: bytes
    mafLodDistancesLandmark: bytes
    mafLodDistancesDistant: bytes

class RegistercGcPlacementGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcEffectsGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcPlayerGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcPersistentTerrainEdits(ctypes.Structure):
    def tilde_cGcPersistentTerrainEdits(): pass

    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mu64GalacticAddress", ctypes.c_uint64),
        ("maiBufferSizes", ctypes.c_char * 0x10),  # cTkDynamicArray<int>
        ("maBufferAnchors", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkVector3>
        ("maEdits", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcTerrainEdit>
    ]
    mu64GalacticAddress: int
    maiBufferSizes: bytes
    maBufferAnchors: bytes
    maEdits: bytes

class cGcPlayerMissionProgressMapEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("miMinProgress", ctypes.c_int32),
        ("miMaxProgress", ctypes.c_int32),
        ("miNewProgress", ctypes.c_int32),
    ]
    mMission: int
    miMinProgress: int
    miMaxProgress: int
    miNewProgress: int

class cGcPlayerMissionProgressMapTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGcMissionProgressTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerMissionProgressMapEntry>
    ]
    maGcMissionProgressTable: bytes

class cGcPlayerMissionUpgradeMapEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mMission", ctypes.c_ulonglong),
        ("miMinProgress", ctypes.c_int32),
        ("miCompletePoint", ctypes.c_int32),
        ("mNewMission", ctypes.c_ulonglong),
        ("maCompletedMissions", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mMission: int
    miMinProgress: int
    miCompletePoint: int
    mNewMission: int
    maCompletedMissions: bytes

class RegistercGcEnvironmentGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcFriendlyDroneVocabularyEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mGenericFallback", ctypes.c_ulonglong),
    ]
    mGenericFallback: int

class cGcPlayerMissionUpgradeMapTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maMissionProgressTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPlayerMissionUpgradeMapEntry>
    ]
    maMissionProgressTable: bytes

class RegistercGcFleetGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcPetVocabularyEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGenericFallback", ctypes.c_ulonglong),
        ("mfOddsOfProcReplacement", ctypes.c_float),
        ("maVocabulary", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcPetVocabularyTraitEntry>
    ]
    mGenericFallback: int
    mfOddsOfProcReplacement: float
    maVocabulary: bytes

class RegistercGcCreatureGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class RegistercGcCharacterGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("LoadModFromFolder", ctypes.c_ulonglong),
    ]
    Register: int
    Load: int
    LoadModFromFolder: int

class cGcStatusMessageDefinitions(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def Save(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maMessages", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcStatusMessageDefinition>
        ("mMissionMarkupColour", common.Colour),
        ("maPetVocabulary", ctypes.c_char * 0x348),  # cTkFixedArray<cGcPetVocabularyEntry,15>
        ("maPetChatTemplates", ctypes.c_char * 0x498),  # cTkFixedArray<cGcPetVocabularyEntry,21>
        ("maFriendlyDroneChatTemplates", ctypes.c_char * 0xA0),  # cTkFixedArray<cGcFriendlyDroneVocabularyEntry,5>
    ]
    maMessages: bytes
    mMissionMarkupColour: common.Colour
    maPetVocabulary: bytes
    maPetChatTemplates: bytes
    maFriendlyDroneChatTemplates: bytes

class cGcSynchronisedBufferData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mau64Data", ctypes.c_char * 0x10),  # cTkDynamicArray<unsigned __int64>
    ]
    mau64Data: bytes

class cGcSimpleIkRecoilComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macEndJoint", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mfHitReactDirectedMin", ctypes.c_float),
        ("mfHitReactDirectedMax", ctypes.c_float),
        ("mfHitReactRandomMin", ctypes.c_float),
        ("mfHitReactRandomMax", ctypes.c_float),
        ("mfAngleLimit", ctypes.c_float),
        ("mfMinHitReactTime", ctypes.c_float),
        ("mfRecoverTime", ctypes.c_float),
        ("mfActiveRange", ctypes.c_float),
    ]
    macEndJoint: bytes
    mfHitReactDirectedMin: float
    mfHitReactDirectedMax: float
    mfHitReactRandomMin: float
    mfHitReactRandomMax: float
    mfAngleLimit: float
    mfMinHitReactTime: float
    mfRecoverTime: float
    mfActiveRange: float

class cGcInteractionBaseBuildingState(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mTriggerAction", ctypes.c_ulonglong),
        ("miTime", ctypes.c_int32),
    ]
    mTriggerAction: int
    miTime: int

class cGcSpawnedObjectComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbCanBeTeleported", ctypes.c_ubyte),
    ]
    mbCanBeTeleported: bool

class cGcThereminComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macVolumeBBName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mfVolumeMinDist", ctypes.c_float),
        ("mfVolumeMaxDist", ctypes.c_float),
        ("mfMinVolume", ctypes.c_float),
        ("mfMaxVolume", ctypes.c_float),
        ("macPitchStartLocator", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macPitchEndLocator", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mfPitchMinDist", ctypes.c_float),
        ("mfPitchMaxDist", ctypes.c_float),
        ("mfMinPitch", ctypes.c_float),
        ("mfMaxPitch", ctypes.c_float),
        ("mfIgnorePlayerDistance", ctypes.c_float),
    ]
    macVolumeBBName: bytes
    mfVolumeMinDist: float
    mfVolumeMaxDist: float
    mfMinVolume: float
    mfMaxVolume: float
    macPitchStartLocator: bytes
    macPitchEndLocator: bytes
    mfPitchMinDist: float
    mfPitchMaxDist: float
    mfMinPitch: float
    mfMaxPitch: float
    mfIgnorePlayerDistance: float

class cGcWiringSocketComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mbValue", ctypes.c_ubyte),
    ]
    mbValue: bool

class cGcBaseBuildingPartInteractionData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mInteractionID", ctypes.c_ulonglong),
        ("mLocalPos", common.Vector3f),
        ("mAtDir", common.Vector3f),
    ]
    mInteractionID: int
    mLocalPos: common.Vector3f
    mAtDir: common.Vector3f

class cGcBaseBuildingEntryCosts(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mfActive0AverageFrameTimeCost", ctypes.c_float),
        ("mfActive1AverageFrameTimeCost", ctypes.c_float),
        ("miActiveTotalNodes", ctypes.c_int32),
        ("miActivePhysicsComponents", ctypes.c_int32),
        ("mfInactive0AverageFrameTimeCost", ctypes.c_float),
        ("mfInactive1AverageFrameTimeCost", ctypes.c_float),
        ("miInactiveTotalNodes", ctypes.c_int32),
        ("miInactivePhysicsComponents", ctypes.c_int32),
    ]
    mID: int
    mfActive0AverageFrameTimeCost: float
    mfActive1AverageFrameTimeCost: float
    miActiveTotalNodes: int
    miActivePhysicsComponents: int
    mfInactive0AverageFrameTimeCost: float
    mfInactive1AverageFrameTimeCost: float
    miInactiveTotalNodes: int
    miInactivePhysicsComponents: int

class cGcBaseBuildingCostsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maObjectCosts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseBuildingEntryCosts>
    ]
    maObjectCosts: bytes

class cGcBaseBuildingPartNavData(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mPartID", ctypes.c_ulonglong),
        ("maSharedInteractions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseBuildingPartInteractionData>
        ("maNavNodeData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseBuildingPartNavNodeData>
    ]
    mPartID: int
    maSharedInteractions: bytes
    maNavNodeData: bytes

class cGcBaseBuildingPartsDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maPartsData", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseBuildingPartData>
    ]
    maPartsData: bytes

class cGcBaseBuildingEntryGroup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGroup", ctypes.c_ulonglong),
        ("mSubGroupName", ctypes.c_ulonglong),
        ("miSubGroup", ctypes.c_int32),
    ]
    mGroup: int
    mSubGroupName: int
    miSubGroup: int

class cGcBaseBuildingPartsNavDataTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maParts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseBuildingPartNavData>
    ]
    maParts: bytes

class cGcBaseBuildingPart(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("maStyleModels", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseBuildingPartStyleModel>
    ]
    mID: int
    maStyleModels: bytes

class cGcBaseBuildingPartsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maParts", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseBuildingPart>
    ]
    maParts: bytes

class cGcBaseBuildingSubGroup(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mName", ctypes.c_ulonglong),
    ]
    mId: int
    mName: int

class cGcJourneyMilestoneData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mJourneyMilestoneId", ctypes.c_ulonglong),
        ("miPointsToUnlock", ctypes.c_int32),
        ("mJourneyMilestoneTitle", ctypes.c_ulonglong),
        ("mJourneyMilestoneTitleLower", ctypes.c_ulonglong),
    ]
    mJourneyMilestoneId: int
    miPointsToUnlock: int
    mJourneyMilestoneTitle: int
    mJourneyMilestoneTitleLower: int

class cGcBaseBuildingProperties(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mDefaultInBaseObject", ctypes.c_ulonglong),
        ("mDefaultOnTerrainObject", ctypes.c_ulonglong),
        ("mDefaultInFreighterObject", ctypes.c_ulonglong),
    ]
    mDefaultInBaseObject: int
    mDefaultOnTerrainObject: int
    mDefaultInFreighterObject: int

class cGcJourneyMilestoneTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGcJourneyMilestoneTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcJourneyMilestoneData>
    ]
    maGcJourneyMilestoneTable: bytes

class cGcId256List(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("maIdList", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<256> >
    ]
    mId: int
    maIdList: bytes

class cGcBaseBuildingPartAudioLocationTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maAudioLocations", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseBuildingPartAudioLocationEntry>
    ]
    maAudioLocations: bytes

class cGcLeveledStatTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGcLeveledStatTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcLeveledStatData>
    ]
    maGcLeveledStatTable: bytes

class cGcBuildingModeCondition(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maiValidBuildingModes", ctypes.c_char * 0x14),  # cTkFixedArray<int,5>
    ]
    maiValidBuildingModes: bytes

class cGcFreighterDungeonsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maDungeons", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcFreighterDungeonParams>
    ]
    maDungeons: bytes

class cGcFreighterRoomNPCSpawnCapacityEntry(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mRoomID", ctypes.c_ulonglong),
        ("mfSpawnCapacity", ctypes.c_float),
    ]
    mRoomID: int
    mfSpawnCapacity: float

class cGcFreighterRoomNPCSpawnCapacities(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maRoomSpawnCapacities", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcFreighterRoomNPCSpawnCapacityEntry>
    ]
    maRoomSpawnCapacities: bytes

class cGcStatDefinitionTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maStatDefinitionTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcStatDefinition>
    ]
    maStatDefinitionTable: bytes

class cGcGeneratedBaseLockDoorPair(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mDoor", ctypes.c_ulonglong),
        ("mLock", ctypes.c_ulonglong),
    ]
    mDoor: int
    mLock: int

class cGcGeneratedBasePruningRule(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mRuleId", ctypes.c_ulonglong),
        ("mNodeName", ctypes.c_ulonglong),
        ("miMaxPerRoom", ctypes.c_int32),
        ("miMaxPerDungeon", ctypes.c_int32),
        ("maRoomFilters", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mRuleId: int
    mNodeName: int
    miMaxPerRoom: int
    miMaxPerDungeon: int
    maRoomFilters: bytes

class cGcGeneratedBaseRoomTemplate(ctypes.Structure):
    def cGcGeneratedBaseRoomTemplate(): pass

    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("mLocId", ctypes.c_ulonglong),
        ("mPrimaryColour", common.Colour),
        ("mSecondaryColour", common.Colour),
        ("mTernaryColour", common.Colour),
        ("mQuaternaryColour", common.Colour),
        ("miMinPathLength", ctypes.c_int32),
        ("miMaxPathLength", ctypes.c_int32),
        ("miMinContiguousWidth", ctypes.c_int32),
        ("miMinContiguousHeight", ctypes.c_int32),
        ("miMinContiguousDepth", ctypes.c_int32),
        ("mfShrinkFactor", ctypes.c_float),
        ("maDecorationThemes", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mName: int
    mLocId: int
    mPrimaryColour: common.Colour
    mSecondaryColour: common.Colour
    mTernaryColour: common.Colour
    mQuaternaryColour: common.Colour
    miMinPathLength: int
    miMaxPathLength: int
    miMinContiguousWidth: int
    miMinContiguousHeight: int
    miMinContiguousDepth: int
    mfShrinkFactor: float
    maDecorationThemes: bytes

class cGcStatGroupData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGroupName", ctypes.c_ulonglong),
        ("maTrackedStats", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mGroupName: int
    maTrackedStats: bytes

class cGcStatGroupTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGcStatGroupTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcStatGroupData>
    ]
    maGcStatGroupTable: bytes

class cGcCustomisationDescriptorGroup(ctypes.Structure):
    def tilde_cGcCustomisationDescriptorGroup(): pass

    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mGroupID", ctypes.c_ulonglong),
        ("mTitle", ctypes.c_ulonglong),
        ("macImage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maacDescriptors", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<32,char> >
        ("maacSuffixInclusionList", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<32,char> >
        ("mLinkedSpecialID", ctypes.c_ulonglong),
        ("mTip", ctypes.c_ulonglong),
        ("mbHiddenInCustomiser", ctypes.c_ubyte),
    ]
    mGroupID: int
    mTitle: int
    macImage: bytes
    maacDescriptors: bytes
    maacSuffixInclusionList: bytes
    mLinkedSpecialID: int
    mTip: int
    mbHiddenInCustomiser: bool

class cGcGeneratedBaseThemeTemplate(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mName", ctypes.c_ulonglong),
        ("maDecorationTemplates", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
    ]
    mName: int
    maDecorationTemplates: bytes

class cGcGeneratedBaseTemplatesTable(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maDecorationTemplates", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGeneratedBaseDecorationTemplate>
        ("maRoomTemplates", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGeneratedBaseRoomTemplate>
        ("maThemeTemplates", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGeneratedBaseThemeTemplate>
        ("maPruningRules", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGeneratedBasePruningRule>
    ]
    maDecorationTemplates: bytes
    maRoomTemplates: bytes
    maThemeTemplates: bytes
    maPruningRules: bytes

class cGcGroupCondition(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maConditions", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkClassPointer>
        ("mbORConditions", ctypes.c_ubyte),
    ]
    maConditions: bytes
    mbORConditions: bool

class cGcAtlasShared(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mu64ClientUserdata", ctypes.c_uint64),
    ]
    __vftable: int
    mu64ClientUserdata: int

class cGcAtlasShared_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasRecvMonumentList_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasRecvSettlementList_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasRecvTotalContribution_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasRecvVoxel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasFeaturedBaseStateChange(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbDev", ctypes.c_ubyte),
        ("mbQA", ctypes.c_ubyte),
        ("mbProd", ctypes.c_ubyte),
        ("macPlatform", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macBaseId", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    mbDev: bool
    mbQA: bool
    mbProd: bool
    macPlatform: bytes
    macBaseId: bytes

class cGcAtlasSendChangeFeaturedBasesEnv_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendMarkFeaturedBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendMarkFeaturedBaseWithImage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendQueryActiveFeaturedBases_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendQueryBaseImages_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendQueryBases_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendQueryFeaturedBases_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendQuerySeasonData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRemoveFeaturedBases_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendReport_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendReportBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestBases_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestBlob_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestDiscoveryAllOnPlanet_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestDiscoveryCategory_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestDiscoveryExact_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestMonument_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestSettlements_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestTotalContribution_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendRequestVoxel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendSubmitBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendSubmitBaseWithImage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendSubmitContributionActual_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcDistanceScaleComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbUseGlobals", ctypes.c_ubyte),
        ("mfScale", ctypes.c_float),
        ("mfMinDistance", ctypes.c_float),
        ("mfMaxDistance", ctypes.c_float),
        ("mfMinHeight", ctypes.c_float),
        ("mfMaxHeight", ctypes.c_float),
        ("mbDisabledWhenOnFreighter", ctypes.c_ubyte),
    ]
    mbUseGlobals: bool
    mfScale: float
    mfMinDistance: float
    mfMaxDistance: float
    mfMinHeight: float
    mfMaxHeight: float
    mbDisabledWhenOnFreighter: bool

class cGcGrabbableComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maGrabbableDataArray", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcGrabbableData>
    ]
    maGrabbableDataArray: bytes

class cGcGroundWormComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macLookJoint", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("macGrabJoint", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mEmergeEffect", ctypes.c_ulonglong),
        ("mSubmergeEffect", ctypes.c_ulonglong),
        ("mEmergeShake", ctypes.c_ulonglong),
        ("mRoarShake", ctypes.c_ulonglong),
        ("mfCollisionBodySize", ctypes.c_float),
        ("mfRumbleTime", ctypes.c_float),
        ("mfEmergeEffectTime", ctypes.c_float),
        ("mfEmergeDist", ctypes.c_float),
        ("mfSubmergeDist", ctypes.c_float),
        ("mfSubmergeDepth", ctypes.c_float),
        ("mfFlinchTime", ctypes.c_float),
        ("mfFlinchAngleMin", ctypes.c_float),
        ("mfFlinchAngleMax", ctypes.c_float),
        ("mfFlinchSmooth", ctypes.c_float),
        ("mAttackDamageType", ctypes.c_ulonglong),
        ("mfAttackDamageRadius", ctypes.c_float),
        ("mfAttackAngle", ctypes.c_float),
        ("mfAttackDistMin", ctypes.c_float),
        ("mfAttackDistMax", ctypes.c_float),
        ("mfAttackCooldown", ctypes.c_float),
        ("mfRoarCooldown", ctypes.c_float),
        ("mfLungeStrength", ctypes.c_float),
        ("mfLungeAngleHead", ctypes.c_float),
        ("mfLungeAngleBase", ctypes.c_float),
        ("mfLungeBeginTime", ctypes.c_float),
        ("mfLungeEndTime", ctypes.c_float),
        ("mfLungeBlendInSpeed", ctypes.c_float),
        ("mfLungeBlendOutSpeed", ctypes.c_float),
        ("mfWindUpStrength", ctypes.c_float),
        ("mfWindUpAngleHead", ctypes.c_float),
        ("mfWindUpAngleBase", ctypes.c_float),
        ("mfRestTime", ctypes.c_float),
        ("mfTrackTime", ctypes.c_float),
        ("mfTurnSpeed", ctypes.c_float),
        ("mfEmergeLookBlendStart", ctypes.c_float),
        ("mfEmergeLookBlendEnd", ctypes.c_float),
        ("mfEmergeTime", ctypes.c_float),
        ("mfRearUpBeginDist", ctypes.c_float),
        ("mfRearUpEndDist", ctypes.c_float),
    ]
    macLookJoint: bytes
    macGrabJoint: bytes
    mEmergeEffect: int
    mSubmergeEffect: int
    mEmergeShake: int
    mRoarShake: int
    mfCollisionBodySize: float
    mfRumbleTime: float
    mfEmergeEffectTime: float
    mfEmergeDist: float
    mfSubmergeDist: float
    mfSubmergeDepth: float
    mfFlinchTime: float
    mfFlinchAngleMin: float
    mfFlinchAngleMax: float
    mfFlinchSmooth: float
    mAttackDamageType: int
    mfAttackDamageRadius: float
    mfAttackAngle: float
    mfAttackDistMin: float
    mfAttackDistMax: float
    mfAttackCooldown: float
    mfRoarCooldown: float
    mfLungeStrength: float
    mfLungeAngleHead: float
    mfLungeAngleBase: float
    mfLungeBeginTime: float
    mfLungeEndTime: float
    mfLungeBlendInSpeed: float
    mfLungeBlendOutSpeed: float
    mfWindUpStrength: float
    mfWindUpAngleHead: float
    mfWindUpAngleBase: float
    mfRestTime: float
    mfTrackTime: float
    mfTurnSpeed: float
    mfEmergeLookBlendStart: float
    mfEmergeLookBlendEnd: float
    mfEmergeTime: float
    mfRearUpBeginDist: float
    mfRearUpEndDist: float

class cGcHeightAdjustComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfHeightOffset", ctypes.c_float),
    ]
    mfHeightOffset: float

class cGcAtlasDiscoveryMetadata(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("macCustomName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    macCustomName: bytes

class cGcLandingHelperComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfActiveDistanceMin", ctypes.c_float),
        ("mfActiveDistanceMax", ctypes.c_float),
        ("mbLandPoint", ctypes.c_ubyte),
    ]
    mfActiveDistanceMin: float
    mfActiveDistanceMax: float
    mbLandPoint: bool

class cGcAtlasSendSubmitDiscoveryExact_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendSubmitMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcLootComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mTimed", common.Vector2f),
        ("mTimeOutEffect", ctypes.c_ulonglong),
        ("mbDeathPoint", ctypes.c_ubyte),
        ("mfFlashPercent", ctypes.c_float),
        ("miNumFlashes", ctypes.c_int32),
        ("mbPhysicsControlled", ctypes.c_ubyte),
        ("mbKeepUpright", ctypes.c_ubyte),
        ("mReward", ctypes.c_ulonglong),
    ]
    mTimed: common.Vector2f
    mTimeOutEffect: int
    mbDeathPoint: bool
    mfFlashPercent: float
    miNumFlashes: int
    mbPhysicsControlled: bool
    mbKeepUpright: bool
    mReward: int

class cGcAtlasSendSubmitMonument_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendSubmitSeasonStageCompleted_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasSendSubmitSettlement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAudioPulseDemo(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfMixRateSeconds", ctypes.c_float),
        ("mPlanet", common.Vector2f),
        ("mSpace", common.Vector2f),
        ("mSpaceStation", common.Vector2f),
        ("mInWarp", common.Vector2f),
        ("mWanted", common.Vector2f),
    ]
    mfMixRateSeconds: float
    mPlanet: common.Vector2f
    mSpace: common.Vector2f
    mSpaceStation: common.Vector2f
    mInWarp: common.Vector2f
    mWanted: common.Vector2f

class cGcEntitlementRewardData(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mEntitlementId", ctypes.c_ulonglong),
        ("mRewardId", ctypes.c_ulonglong),
        ("mName", ctypes.c_ulonglong),
        ("mError", ctypes.c_ulonglong),
    ]
    mEntitlementId: int
    mRewardId: int
    mName: int
    mError: int

class cGcEntitlementRewardsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTable", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcEntitlementRewardData>
    ]
    maTable: bytes

class cGcAlienPodComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mfAgroRate", ctypes.c_float),
        ("mfAgroMovement", ctypes.c_float),
        ("mfAgroMovementRange", ctypes.c_float),
        ("mfAgroTorch", ctypes.c_float),
        ("mfAgroTorchRange", ctypes.c_float),
        ("mfAgroTorchFOV", ctypes.c_float),
        ("mfAgroThreshold", ctypes.c_float),
        ("mfAgroThresholdOffscreen", ctypes.c_float),
        ("mfAgroSpookValue", ctypes.c_float),
        ("mfAgroSpookTime", ctypes.c_float),
        ("mfAgroSpookTimeMin", ctypes.c_float),
        ("mfAgroSpookTimeMax", ctypes.c_float),
        ("mfGlowIntensityMin", ctypes.c_float),
        ("mfGlowIntensityMax", ctypes.c_float),
        ("mfInstaAgroDistance", ctypes.c_float),
        ("mfGunfireAgro", ctypes.c_float),
        ("mfGunfireAgroRange", ctypes.c_float),
    ]
    mfAgroRate: float
    mfAgroMovement: float
    mfAgroMovementRange: float
    mfAgroTorch: float
    mfAgroTorchRange: float
    mfAgroTorchFOV: float
    mfAgroThreshold: float
    mfAgroThresholdOffscreen: float
    mfAgroSpookValue: float
    mfAgroSpookTime: float
    mfAgroSpookTimeMin: float
    mfAgroSpookTimeMax: float
    mfGlowIntensityMin: float
    mfGlowIntensityMax: float
    mfInstaAgroDistance: float
    mfGunfireAgro: float
    mfGunfireAgroRange: float

class cGcAtmosphereEntryComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbAutoEntry", ctypes.c_ubyte),
        ("mfEntryTime", ctypes.c_float),
        ("mfEntryOffset", ctypes.c_float),
        ("mfEditTerrainRadius", ctypes.c_float),
        ("mImpactEffect", ctypes.c_ulonglong),
        ("mFlareEffect", ctypes.c_ulonglong),
    ]
    mbAutoEntry: bool
    mfEntryTime: float
    mfEntryOffset: float
    mfEditTerrainRadius: float
    mImpactEffect: int
    mFlareEffect: int

class cGcBaseDefenceComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maTriggers", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBaseDefenceTrigger>
        ("mfLostUncertaintyThreshold", ctypes.c_float),
        ("mfSearchTime", ctypes.c_float),
        ("mbPrioritiseThreats", ctypes.c_ubyte),
        ("mfLaserRangeAnimateTime", ctypes.c_float),
    ]
    maTriggers: bytes
    mfLostUncertaintyThreshold: float
    mfSearchTime: float
    mbPrioritiseThreats: bool
    mfLaserRangeAnimateTime: float

class cGcBasePlacementComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maRules", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcBasePlacementRule>
    ]
    maRules: bytes

class cGcCustomisationColourPalettes(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maColourPalettes", ctypes.c_char * 0x5D70),  # cTkFixedArray<cGcPaletteData,23>
        ("maExtraData", ctypes.c_char * 0x2E0),  # cTkFixedArray<cGcCustomisationColourPaletteExtraData,23>
    ]
    maColourPalettes: bytes
    maExtraData: bytes

class cGcBuildingComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass


class cGcByteBeatSwitchComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    _fields_ = [
        ("mfTemp", ctypes.c_float),
    ]
    mfTemp: float

class cTkDescriptorCache(ctypes.Structure):
    _fields_ = [
        ("mFilenameDescriptorMap", ctypes.c_char * 0x28),  # cTkLinearHashTable<cTkFixedString<256,char>,cTkModelDescriptorList *,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > >
        ("mFilenameFilterMap", ctypes.c_char * 0x28),  # cTkLinearHashTable<cTkFixedString<256,char>,cTkResourceFilterList *,-1,0,cTkLinearHashTableHash<cTkFixedString<256,char> > >
    ]
    mFilenameDescriptorMap: bytes
    mFilenameFilterMap: bytes

class cGcCustomisationDescriptorGroups(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maDescriptorGroups", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationDescriptorGroup>
    ]
    maDescriptorGroups: bytes

class cTkResourceDescriptor(ctypes.Structure):
    def cTkResourceDescriptor(): pass

    def GetSeed(): pass

    def FromString(): pass

    def GetSize(): pass

    def GetDescriptor(): pass

    def Equals(): pass

    def AddDescriptor(): pass

    def ToString(): pass

    def scalar_deleting_destructor(): pass

    def HashAltID(): pass

    def DistortInstance(): pass

    def GenerateDistortedInstanceFromList(): pass

    def GenerateInstance(): pass

    def GenerateInstanceFromGuide(): pass

    def GenerateInstanceFromList(): pass

    def GenerateInstanceFromModel(): pass

    def GetModelDescriptorList(): pass

    def GetResourceFilter(): pass

    def SearchForID(): pass

    def SelectPart(): pass

    def ShouldLoadMesh(): pass

    def Validate(): pass

    _fields_ = [
        ("maDescriptors", ctypes.c_char * 0x18),  # std::vector<TkID<256>,TkSTLAllocatorShim<TkID<256>,8,-1> >
        ("mSeed", common.GcSeed),
        ("mSecondarySeed", common.GcSeed),
    ]
    maDescriptors: bytes
    mSeed: common.GcSeed
    mSecondarySeed: common.GcSeed

class cGcCombatEffectsTable(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maEffectsData", ctypes.c_char * 0x100),  # cTkFixedArray<cGcCombatEffectData,4>
    ]
    maEffectsData: bytes

class cTkPersonalRNG(ctypes.Structure):
    def cTkPersonalRNG(): pass

    def Reseed(): pass

    _fields_ = [
        ("mState0", ctypes.c_uint32),
        ("mState1", ctypes.c_uint32),
    ]
    mState0: int
    mState1: int

class cGcCustomisationPresets(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maPresets", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationPreset>
    ]
    maPresets: bytes

class cGcCustomisationTextureOptions(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixUp(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maTextureOptions", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcCustomisationTextureOption>
    ]
    maTextureOptions: bytes

class cGcByteBeatTemplates(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def FixDown(): pass

    def FixUp(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def Save(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("maacInitialTrees", ctypes.c_char * 0x10),  # cTkDynamicArray<cTkFixedString<64,char> >
        ("maTemplates", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcByteBeatTemplate>
        ("mafCombinerWeights", ctypes.c_char * 0x48),  # cTkFixedArray<float,18>
        ("maKickDrums", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcByteBeatDrum>
        ("maSnareDrums", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcByteBeatDrum>
        ("maHiHats", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcByteBeatDrum>
        ("mfOperatorPermuteChance", ctypes.c_float),
        ("mfTemplateCombineChanceAtRoot", ctypes.c_float),
        ("mfTemplateCombineChance", ctypes.c_float),
        ("maSongs", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcByteBeatSong>
    ]
    maacInitialTrees: bytes
    maTemplates: bytes
    mafCombinerWeights: bytes
    maKickDrums: bytes
    maSnareDrums: bytes
    maHiHats: bytes
    mfOperatorPermuteChance: float
    mfTemplateCombineChanceAtRoot: float
    mfTemplateCombineChance: float
    maSongs: bytes

class hkBaseObject_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int

class hkReferencedObject_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkReferencedObjectLock_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkThread_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkThread_dtor_0", ctypes.c_ulonglong),
    ]
    hkThread_dtor_0: int

class hkReadersWriterAccessChecker(ctypes.Structure):
    _fields_ = [
        ("m_readCount", ctypes.c_char * 0x4),  # std::atomic<unsigned int>
        ("m_writingThread", ctypes.c_char * 0x8),  # std::atomic<unsigned __int64>
    ]
    m_readCount: bytes
    m_writingThread: bytes

class hkContainerAllocator_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkContainerAllocator_dtor_0", ctypes.c_ulonglong),
        ("malloc", ctypes.c_ulonglong),
        ("malloc_10", ctypes.c_ulonglong),
        ("free", ctypes.c_ulonglong),
    ]
    hkContainerAllocator_dtor_0: int
    malloc: int
    malloc_10: int
    free: int

class hkBuiltinContainerAllocatorBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkContainerAllocator_dtor_0", ctypes.c_ulonglong),
        ("malloc", ctypes.c_ulonglong),
        ("malloc_10", ctypes.c_ulonglong),
        ("free", ctypes.c_ulonglong),
    ]
    hkContainerAllocator_dtor_0: int
    malloc: int
    malloc_10: int
    free: int

class hkStreamWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("write", ctypes.c_ulonglong),
        ("flush", ctypes.c_ulonglong),
        ("seekTellSupported", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
        ("isBuffered", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    write: int
    flush: int
    seekTellSupported: int
    seek: int
    tell: int
    isBuffered: int

class hkJsonWriter(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hkJsonWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkJsonWriter_dtor_0", ctypes.c_ulonglong),
        ("beginObject", ctypes.c_ulonglong),
        ("beginArray", ctypes.c_ulonglong),
        ("end", ctypes.c_ulonglong),
        ("writeString", ctypes.c_ulonglong),
        ("writeNumber", ctypes.c_ulonglong),
        ("writeBoolean", ctypes.c_ulonglong),
        ("writeNull", ctypes.c_ulonglong),
        ("writeInteger", ctypes.c_ulonglong),
        ("writeJsonNode", ctypes.c_ulonglong),
        ("writeJsonString", ctypes.c_ulonglong),
        ("writeBinary", ctypes.c_ulonglong),
    ]
    hkJsonWriter_dtor_0: int
    beginObject: int
    beginArray: int
    end: int
    writeString: int
    writeNumber: int
    writeBoolean: int
    writeNull: int
    writeInteger: int
    writeJsonNode: int
    writeJsonString: int
    writeBinary: int

class hkOstream_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkOstream_dtor_0", ctypes.c_ulonglong),
    ]
    hkOstream_dtor_0: int

class hkJsonOstream_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkJsonWriter_dtor_0", ctypes.c_ulonglong),
        ("beginObject", ctypes.c_ulonglong),
        ("beginArray", ctypes.c_ulonglong),
        ("end", ctypes.c_ulonglong),
        ("writeString", ctypes.c_ulonglong),
        ("writeNumber", ctypes.c_ulonglong),
        ("writeBoolean", ctypes.c_ulonglong),
        ("writeNull", ctypes.c_ulonglong),
        ("writeInteger", ctypes.c_ulonglong),
        ("writeJsonNode", ctypes.c_ulonglong),
        ("writeJsonString", ctypes.c_ulonglong),
        ("writeBinary", ctypes.c_ulonglong),
    ]
    hkJsonWriter_dtor_0: int
    beginObject: int
    beginArray: int
    end: int
    writeString: int
    writeNumber: int
    writeBoolean: int
    writeNull: int
    writeInteger: int
    writeJsonNode: int
    writeJsonString: int
    writeBinary: int

class hkStackTracer(ctypes.Structure):
    def hkStackTracer(): pass

    def tilde_hkStackTracer(): pass

    def vector_deleting_destructor(): pass

    def dumpStackTrace(): pass

    def getStackTrace(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hkStackTracer_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkStackTracer_dtor_0", ctypes.c_ulonglong),
    ]
    hkStackTracer_dtor_0: int

class hkSlot(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_next", ctypes.c_char * 0x8),  # hkPtrAndInt<hkSlot,unsigned int,3>
        ("m_object", ctypes.c_void_p),
    ]
    __vftable: int
    m_next: bytes
    m_object: int

class hkSlot_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkSlot_dtor_0", ctypes.c_ulonglong),
        ("matchMethod", ctypes.c_ulonglong),
        ("matchData", ctypes.c_ulonglong),
    ]
    hkSlot_dtor_0: int
    matchMethod: int
    matchData: int

class hkHardwareInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _GUID(ctypes.Structure):
    _fields_ = [
        ("Data1", ctypes.c_uint32),
        ("Data2", ctypes.c_uint16),
        ("Data3", ctypes.c_uint16),
        ("Data4", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    Data1: int
    Data2: int
    Data3: int
    Data4: list[ctypes.c_uint8]

class _TP_CALLBACK_ENVIRON_V1(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("Pool", ctypes.c_ulonglong),
        ("CleanupGroup", ctypes.c_ulonglong),
        ("CleanupGroupCancelCallback", ctypes.c_ulonglong),
        ("RaceDll", ctypes.c_void_p),
        ("ActivationContext", ctypes.c_ulonglong),
        ("FinalizationCallback", ctypes.c_ulonglong),
        ("u", ctypes.c_char * 0x4),  # _TP_CALLBACK_ENVIRON_V1::<unnamed_type_u>
    ]
    Version: int
    Pool: int
    CleanupGroup: int
    CleanupGroupCancelCallback: int
    RaceDll: int
    ActivationContext: int
    FinalizationCallback: int
    u: bytes

class _ACTIVATION_CONTEXT_QUERY_INDEX(ctypes.Structure):
    _fields_ = [
        ("ulAssemblyIndex", ctypes.c_uint32),
        ("ulFileIndexInAssembly", ctypes.c_uint32),
    ]
    ulAssemblyIndex: int
    ulFileIndexInAssembly: int

class HMENU__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class HBITMAP__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class tagMENUITEMINFOA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fMask", ctypes.c_uint32),
        ("fType", ctypes.c_uint32),
        ("fState", ctypes.c_uint32),
        ("wID", ctypes.c_uint32),
        ("hSubMenu", ctypes.c_ulonglong),
        ("hbmpChecked", ctypes.c_ulonglong),
        ("hbmpUnchecked", ctypes.c_ulonglong),
        ("dwItemData", ctypes.c_uint64),
        ("dwTypeData", ctypes.c_char_p),
        ("cch", ctypes.c_uint32),
        ("hbmpItem", ctypes.c_ulonglong),
    ]
    cbSize: int
    fMask: int
    fType: int
    fState: int
    wID: int
    hSubMenu: int
    hbmpChecked: int
    hbmpUnchecked: int
    dwItemData: int
    dwTypeData: bytes
    cch: int
    hbmpItem: int

class hkStackCallTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class tagPOINT(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
    ]
    x: int
    y: int

class hkStreamReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("skip", ctypes.c_ulonglong),
        ("peek", ctypes.c_ulonglong),
        ("isSeekTellSupported", ctypes.c_ulonglong),
        ("isInplace", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    read: int
    skip: int
    peek: int
    isSeekTellSupported: int
    isInplace: int

class hkFileSystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("remove", ctypes.c_ulonglong),
        ("moveFile", ctypes.c_ulonglong),
        ("mkdir", ctypes.c_ulonglong),
        ("mkdirRecursive", ctypes.c_ulonglong),
        ("copy", ctypes.c_ulonglong),
        ("stat", ctypes.c_ulonglong),
        ("getOperatingSystemPath", ctypes.c_ulonglong),
        ("isPathAbsolute", ctypes.c_ulonglong),
        ("createWatcher", ctypes.c_char * 0x8),  # hkRefNew<hkFileSystem::Watcher> *(__fastcall *)(hkFileSystem *this, hkRefNew<hkFileSystem::Watcher> *result, const char *)
        ("createIterator", ctypes.c_char * 0x8),  # hkRefNew<hkFileSystem::Iterator::Impl> *(__fastcall *)(hkFileSystem *this, hkRefNew<hkFileSystem::Iterator::Impl> *result, const char *, const char *)
        ("_openReader", ctypes.c_char * 0x8),  # hkRefNew<hkStreamReader> *(__fastcall *)(hkFileSystem *this, hkRefNew<hkStreamReader> *result, const char *, hkFileSystem::OpenFlags)
        ("_openWriter", ctypes.c_char * 0x8),  # hkRefNew<hkStreamWriter> *(__fastcall *)(hkFileSystem *this, hkRefNew<hkStreamWriter> *result, const char *, hkFileSystem::OpenFlags)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    remove: int
    moveFile: int
    mkdir: int
    mkdirRecursive: int
    copy: int
    stat: int
    getOperatingSystemPath: int
    isPathAbsolute: int
    createWatcher: bytes
    createIterator: bytes
    _openReader: bytes
    _openWriter: bytes

class _M128A(ctypes.Structure):
    _fields_ = [
        ("Low", ctypes.c_uint64),
        ("High", ctypes.c_int64),
    ]
    Low: int
    High: int

class _SERVICE_STATUS(ctypes.Structure):
    _fields_ = [
        ("dwServiceType", ctypes.c_uint32),
        ("dwCurrentState", ctypes.c_uint32),
        ("dwControlsAccepted", ctypes.c_uint32),
        ("dwWin32ExitCode", ctypes.c_uint32),
        ("dwServiceSpecificExitCode", ctypes.c_uint32),
        ("dwCheckPoint", ctypes.c_uint32),
        ("dwWaitHint", ctypes.c_uint32),
    ]
    dwServiceType: int
    dwCurrentState: int
    dwControlsAccepted: int
    dwWin32ExitCode: int
    dwServiceSpecificExitCode: int
    dwCheckPoint: int
    dwWaitHint: int

class hkVector4fComparison(ctypes.Structure):
    _fields_ = [
        ("m_mask", common.Vector4f),
    ]
    m_mask: common.Vector4f

class hkSimdFloat32(ctypes.Structure):
    def ToString(): pass

    _fields_ = [
        ("m_real", common.Vector4f),
    ]
    m_real: common.Vector4f

class __m128d(ctypes.Structure):
    _fields_ = [
        ("m128d_f64", ctypes.c_double * 0x2),  # long double[2]
    ]
    m128d_f64: list[ctypes.c_double]

class __m128i: pass

class _GENERIC_MAPPING(ctypes.Structure):
    _fields_ = [
        ("GenericRead", ctypes.c_uint32),
        ("GenericWrite", ctypes.c_uint32),
        ("GenericExecute", ctypes.c_uint32),
        ("GenericAll", ctypes.c_uint32),
    ]
    GenericRead: int
    GenericWrite: int
    GenericExecute: int
    GenericAll: int

class _ACE_HEADER(ctypes.Structure):
    _fields_ = [
        ("AceType", ctypes.c_uint8),
        ("AceFlags", ctypes.c_uint8),
        ("AceSize", ctypes.c_uint16),
    ]
    AceType: int
    AceFlags: int
    AceSize: int

class hkVector4_AdvancedInterface(ctypes.Structure):
    pass

class hkScalarF_FloatImpl_vtbl(ctypes.Structure):
    _fields_ = [
        ("Impl_dtor_0", ctypes.c_ulonglong),
        ("inplaceFixup", ctypes.c_ulonglong),
        ("getSetter", ctypes.c_ulonglong),
        ("getFieldImpl", ctypes.c_ulonglong),
        ("isProperty", ctypes.c_ulonglong),
        ("setValue", ctypes.c_ulonglong),
        ("getValue", ctypes.c_ulonglong),
    ]
    Impl_dtor_0: int
    inplaceFixup: int
    getSetter: int
    getFieldImpl: int
    isProperty: int
    setValue: int
    getValue: int

class HWND__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class hkError_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("message", ctypes.c_ulonglong),
        ("setEnabled", ctypes.c_ulonglong),
        ("isEnabled", ctypes.c_ulonglong),
        ("setMinimumMessageLevel", ctypes.c_ulonglong),
        ("getMinimumMessageLevel", ctypes.c_ulonglong),
        ("enableAll", ctypes.c_ulonglong),
        ("sectionBegin", ctypes.c_ulonglong),
        ("sectionEnd", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    message: int
    setEnabled: int
    isEnabled: int
    setMinimumMessageLevel: int
    getMinimumMessageLevel: int
    enableAll: int
    sectionBegin: int
    sectionEnd: int

class hkDefaultError_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("message", ctypes.c_ulonglong),
        ("setEnabled", ctypes.c_ulonglong),
        ("isEnabled", ctypes.c_ulonglong),
        ("setMinimumMessageLevel", ctypes.c_ulonglong),
        ("getMinimumMessageLevel", ctypes.c_ulonglong),
        ("enableAll", ctypes.c_ulonglong),
        ("sectionBegin", ctypes.c_ulonglong),
        ("sectionEnd", ctypes.c_ulonglong),
        ("showMessage", ctypes.c_ulonglong),
        ("formMessage", ctypes.c_ulonglong),
        ("getMessageTypeString", ctypes.c_ulonglong),
        ("shouldReportStackTrace", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    message: int
    setEnabled: int
    isEnabled: int
    setMinimumMessageLevel: int
    getMinimumMessageLevel: int
    enableAll: int
    sectionBegin: int
    sectionEnd: int
    showMessage: int
    formMessage: int
    getMessageTypeString: int
    shouldReportStackTrace: int

class HKEY__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class DLGTEMPLATE(ctypes.Structure):
    _fields_ = [
        ("style", ctypes.c_uint32),
        ("dwExtendedStyle", ctypes.c_uint32),
        ("cdit", ctypes.c_uint16),
        ("x", ctypes.c_int16),
        ("y", ctypes.c_int16),
        ("_cx", ctypes.c_int16),
        ("cy", ctypes.c_int16),
    ]
    style: int
    dwExtendedStyle: int
    cdit: int
    x: int
    y: int
    _cx: int
    cy: int

class tagLOGFONTA(ctypes.Structure):
    _fields_ = [
        ("lfHeight", ctypes.c_int32),
        ("lfWidth", ctypes.c_int32),
        ("lfEscapement", ctypes.c_int32),
        ("lfOrientation", ctypes.c_int32),
        ("lfWeight", ctypes.c_int32),
        ("lfItalic", ctypes.c_uint8),
        ("lfUnderline", ctypes.c_uint8),
        ("lfStrikeOut", ctypes.c_uint8),
        ("lfCharSet", ctypes.c_uint8),
        ("lfOutPrecision", ctypes.c_uint8),
        ("lfClipPrecision", ctypes.c_uint8),
        ("lfQuality", ctypes.c_uint8),
        ("lfPitchAndFamily", ctypes.c_uint8),
        ("lfFaceName", ctypes.c_char * 0x20),  # char[32]
    ]
    lfHeight: int
    lfWidth: int
    lfEscapement: int
    lfOrientation: int
    lfWeight: int
    lfItalic: int
    lfUnderline: int
    lfStrikeOut: int
    lfCharSet: int
    lfOutPrecision: int
    lfClipPrecision: int
    lfQuality: int
    lfPitchAndFamily: int
    lfFaceName: list[ctypes.c_char]

class hkErrorRouter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("message", ctypes.c_ulonglong),
        ("setEnabled", ctypes.c_ulonglong),
        ("isEnabled", ctypes.c_ulonglong),
        ("setMinimumMessageLevel", ctypes.c_ulonglong),
        ("getMinimumMessageLevel", ctypes.c_ulonglong),
        ("enableAll", ctypes.c_ulonglong),
        ("sectionBegin", ctypes.c_ulonglong),
        ("sectionEnd", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    message: int
    setEnabled: int
    isEnabled: int
    setMinimumMessageLevel: int
    getMinimumMessageLevel: int
    enableAll: int
    sectionBegin: int
    sectionEnd: int

class tagWCRANGE(ctypes.Structure):
    _fields_ = [
        ("wcLow", ctypes.c_wchar),
        ("cGlyphs", ctypes.c_uint16),
    ]
    wcLow: str
    cGlyphs: int

class hkSimdDouble64_FloatImpl_vtbl(ctypes.Structure):
    _fields_ = [
        ("Impl_dtor_0", ctypes.c_ulonglong),
        ("inplaceFixup", ctypes.c_ulonglong),
        ("getSetter", ctypes.c_ulonglong),
        ("getFieldImpl", ctypes.c_ulonglong),
        ("isProperty", ctypes.c_ulonglong),
        ("setValue", ctypes.c_ulonglong),
        ("getValue", ctypes.c_ulonglong),
    ]
    Impl_dtor_0: int
    inplaceFixup: int
    getSetter: int
    getFieldImpl: int
    isProperty: int
    setValue: int
    getValue: int

class hkSocket_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("close", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("write", ctypes.c_ulonglong),
        ("connect", ctypes.c_ulonglong),
        ("connect_40", ctypes.c_ulonglong),
        ("listen", ctypes.c_ulonglong),
        ("bind", ctypes.c_ulonglong),
        ("getInetAddr", ctypes.c_ulonglong),
        ("canWrite", ctypes.c_ulonglong),
        ("canRead", ctypes.c_ulonglong),
        ("pollForNewClient", ctypes.c_ulonglong),
        ("createDatagramSocket", ctypes.c_ulonglong),
        ("sendTo", ctypes.c_ulonglong),
        ("receiveFrom", ctypes.c_ulonglong),
        ("getLastError", ctypes.c_ulonglong),
        ("getAddress", ctypes.c_ulonglong),
        ("setBlocking", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    close: int
    read: int
    write: int
    connect: int
    connect_40: int
    listen: int
    bind: int
    getInetAddr: int
    canWrite: int
    canRead: int
    pollForNewClient: int
    createDatagramSocket: int
    sendTo: int
    receiveFrom: int
    getLastError: int
    getAddress: int
    setBlocking: int

class _IMAGE_INDIR_CONTROL_TRANSFER_DYNAMIC_RELOCATION(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int16),
    ]
    _bf_0: int

class hkLocalFrameGroup_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _LUID(ctypes.Structure):
    _fields_ = [
        ("LowPart", ctypes.c_uint32),
        ("HighPart", ctypes.c_int32),
    ]
    LowPart: int
    HighPart: int

class tagMOUSEMOVEPOINT(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("time", ctypes.c_uint32),
        ("dwExtraInfo", ctypes.c_uint64),
    ]
    x: int
    y: int
    time: int
    dwExtraInfo: int

class tagINPUT_INJECTION_VALUE(ctypes.Structure):
    _fields_ = [
        ("page", ctypes.c_uint16),
        ("usage", ctypes.c_uint16),
        ("value", ctypes.c_int32),
        ("index", ctypes.c_uint16),
    ]
    page: int
    usage: int
    value: int
    index: int

class _LARGE_INTEGER: pass

class HINSTANCE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class HICON__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class HBRUSH__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class tagWNDCLASSEXA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("style", ctypes.c_uint32),
        ("lpfnWndProc", ctypes.c_ulonglong),
        ("cbClsExtra", ctypes.c_int32),
        ("cbWndExtra", ctypes.c_int32),
        ("hInstance", ctypes.c_ulonglong),
        ("hIcon", ctypes.c_ulonglong),
        ("hCursor", ctypes.c_ulonglong),
        ("hbrBackground", ctypes.c_ulonglong),
        ("lpszMenuName", ctypes.c_char_p),
        ("lpszClassName", ctypes.c_char_p),
        ("hIconSm", ctypes.c_ulonglong),
    ]
    cbSize: int
    style: int
    lpfnWndProc: int
    cbClsExtra: int
    cbWndExtra: int
    hInstance: int
    hIcon: int
    hCursor: int
    hbrBackground: int
    lpszMenuName: bytes
    lpszClassName: bytes
    hIconSm: int

class tagDESIGNVECTOR(ctypes.Structure):
    _fields_ = [
        ("dvReserved", ctypes.c_uint32),
        ("dvNumAxes", ctypes.c_uint32),
        ("dvValues", ctypes.c_int32 * 0x10),  # int[16]
    ]
    dvReserved: int
    dvNumAxes: int
    dvValues: list[ctypes.c_int32]

class DISPLAYCONFIG_RATIONAL(ctypes.Structure):
    _fields_ = [
        ("Numerator", ctypes.c_uint32),
        ("Denominator", ctypes.c_uint32),
    ]
    Numerator: int
    Denominator: int

class hkMultiMapIntegralOperations(ctypes.Structure):
    pass

class cTkISystemEventHandler(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkISystemEventHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("BackgroundExecutionChanged", ctypes.c_ulonglong),
        ("SystemUIOverlayChange", ctypes.c_ulonglong),
        ("ShareMenuOpened", ctypes.c_ulonglong),
        ("SystemPlayModeChanged", ctypes.c_ulonglong),
    ]
    BackgroundExecutionChanged: int
    SystemUIOverlayChange: int
    ShareMenuOpened: int
    SystemPlayModeChanged: int

class _IMAGE_ENCLAVE_IMPORT(ctypes.Structure):
    _fields_ = [
        ("MatchType", ctypes.c_uint32),
        ("MinimumSecurityVersion", ctypes.c_uint32),
        ("UniqueOrAuthorID", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
        ("FamilyID", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("ImageID", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("ImportName", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint32),
    ]
    MatchType: int
    MinimumSecurityVersion: int
    UniqueOrAuthorID: list[ctypes.c_uint8]
    FamilyID: list[ctypes.c_uint8]
    ImageID: list[ctypes.c_uint8]
    ImportName: int
    Reserved: int

class tagMETAHEADER(ctypes.Structure):
    _fields_ = [
        ("mtType", ctypes.c_uint16),
        ("mtHeaderSize", ctypes.c_uint16),
        ("mtVersion", ctypes.c_uint16),
        ("mtSize", ctypes.c_uint32),
        ("mtNoObjects", ctypes.c_uint16),
        ("mtMaxRecord", ctypes.c_uint32),
        ("mtNoParameters", ctypes.c_uint16),
    ]
    mtType: int
    mtHeaderSize: int
    mtVersion: int
    mtSize: int
    mtNoObjects: int
    mtMaxRecord: int
    mtNoParameters: int

class _TRIVERTEX(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("Red", ctypes.c_uint16),
        ("Green", ctypes.c_uint16),
        ("Blue", ctypes.c_uint16),
        ("Alpha", ctypes.c_uint16),
    ]
    x: int
    y: int
    Red: int
    Green: int
    Blue: int
    Alpha: int

class tagTEXTMETRICW(ctypes.Structure):
    _fields_ = [
        ("tmHeight", ctypes.c_int32),
        ("tmAscent", ctypes.c_int32),
        ("tmDescent", ctypes.c_int32),
        ("tmInternalLeading", ctypes.c_int32),
        ("tmExternalLeading", ctypes.c_int32),
        ("tmAveCharWidth", ctypes.c_int32),
        ("tmMaxCharWidth", ctypes.c_int32),
        ("tmWeight", ctypes.c_int32),
        ("tmOverhang", ctypes.c_int32),
        ("tmDigitizedAspectX", ctypes.c_int32),
        ("tmDigitizedAspectY", ctypes.c_int32),
        ("tmFirstChar", ctypes.c_wchar),
        ("tmLastChar", ctypes.c_wchar),
        ("tmDefaultChar", ctypes.c_wchar),
        ("tmBreakChar", ctypes.c_wchar),
        ("tmItalic", ctypes.c_uint8),
        ("tmUnderlined", ctypes.c_uint8),
        ("tmStruckOut", ctypes.c_uint8),
        ("tmPitchAndFamily", ctypes.c_uint8),
        ("tmCharSet", ctypes.c_uint8),
    ]
    tmHeight: int
    tmAscent: int
    tmDescent: int
    tmInternalLeading: int
    tmExternalLeading: int
    tmAveCharWidth: int
    tmMaxCharWidth: int
    tmWeight: int
    tmOverhang: int
    tmDigitizedAspectX: int
    tmDigitizedAspectY: int
    tmFirstChar: str
    tmLastChar: str
    tmDefaultChar: str
    tmBreakChar: str
    tmItalic: int
    tmUnderlined: int
    tmStruckOut: int
    tmPitchAndFamily: int
    tmCharSet: int

class tagRECT(ctypes.Structure):
    _fields_ = [
        ("left", ctypes.c_int32),
        ("top", ctypes.c_int32),
        ("right", ctypes.c_int32),
        ("bottom", ctypes.c_int32),
    ]
    left: int
    top: int
    right: int
    bottom: int

class _PROCESS_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("hProcess", ctypes.c_void_p),
        ("hThread", ctypes.c_void_p),
        ("dwProcessId", ctypes.c_uint32),
        ("dwThreadId", ctypes.c_uint32),
    ]
    hProcess: int
    hThread: int
    dwProcessId: int
    dwThreadId: int

class tagEMR(ctypes.Structure):
    _fields_ = [
        ("iType", ctypes.c_uint32),
        ("nSize", ctypes.c_uint32),
    ]
    iType: int
    nSize: int

class _IO_COUNTERS(ctypes.Structure):
    _fields_ = [
        ("ReadOperationCount", ctypes.c_uint64),
        ("WriteOperationCount", ctypes.c_uint64),
        ("OtherOperationCount", ctypes.c_uint64),
        ("ReadTransferCount", ctypes.c_uint64),
        ("WriteTransferCount", ctypes.c_uint64),
        ("OtherTransferCount", ctypes.c_uint64),
    ]
    ReadOperationCount: int
    WriteOperationCount: int
    OtherOperationCount: int
    ReadTransferCount: int
    WriteTransferCount: int
    OtherTransferCount: int

class _DISPLAY_DEVICEA(ctypes.Structure):
    _fields_ = [
        ("cb", ctypes.c_uint32),
        ("DeviceName", ctypes.c_char * 0x20),  # char[32]
        ("DeviceString", ctypes.c_char * 0x80),  # char[128]
        ("StateFlags", ctypes.c_uint32),
        ("DeviceID", ctypes.c_char * 0x80),  # char[128]
        ("DeviceKey", ctypes.c_char * 0x80),  # char[128]
    ]
    cb: int
    DeviceName: list[ctypes.c_char]
    DeviceString: list[ctypes.c_char]
    StateFlags: int
    DeviceID: list[ctypes.c_char]
    DeviceKey: list[ctypes.c_char]

class PPM_WMI_LEGACY_PERFSTATE(ctypes.Structure):
    _fields_ = [
        ("Frequency", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("PercentFrequency", ctypes.c_uint32),
    ]
    Frequency: int
    Flags: int
    PercentFrequency: int

class _PROCESS_DYNAMIC_ENFORCED_ADDRESS_RANGE(ctypes.Structure):
    _fields_ = [
        ("BaseAddress", ctypes.c_uint64),
        ("Size", ctypes.c_uint64),
        ("Flags", ctypes.c_uint32),
    ]
    BaseAddress: int
    Size: int
    Flags: int

class _OSVERSIONINFOA(ctypes.Structure):
    _fields_ = [
        ("dwOSVersionInfoSize", ctypes.c_uint32),
        ("dwMajorVersion", ctypes.c_uint32),
        ("dwMinorVersion", ctypes.c_uint32),
        ("dwBuildNumber", ctypes.c_uint32),
        ("dwPlatformId", ctypes.c_uint32),
        ("szCSDVersion", ctypes.c_char * 0x80),  # char[128]
    ]
    dwOSVersionInfoSize: int
    dwMajorVersion: int
    dwMinorVersion: int
    dwBuildNumber: int
    dwPlatformId: int
    szCSDVersion: list[ctypes.c_char]

class _IMAGE_OS2_HEADER(ctypes.Structure):
    _fields_ = [
        ("ne_magic", ctypes.c_uint16),
        ("ne_ver", ctypes.c_char),
        ("ne_rev", ctypes.c_char),
        ("ne_enttab", ctypes.c_uint16),
        ("ne_cbenttab", ctypes.c_uint16),
        ("ne_crc", ctypes.c_int32),
        ("ne_flags", ctypes.c_uint16),
        ("ne_autodata", ctypes.c_uint16),
        ("ne_heap", ctypes.c_uint16),
        ("ne_stack", ctypes.c_uint16),
        ("ne_csip", ctypes.c_int32),
        ("ne_sssp", ctypes.c_int32),
        ("ne_cseg", ctypes.c_uint16),
        ("ne_cmod", ctypes.c_uint16),
        ("ne_cbnrestab", ctypes.c_uint16),
        ("ne_segtab", ctypes.c_uint16),
        ("ne_rsrctab", ctypes.c_uint16),
        ("ne_restab", ctypes.c_uint16),
        ("ne_modtab", ctypes.c_uint16),
        ("ne_imptab", ctypes.c_uint16),
        ("ne_nrestab", ctypes.c_int32),
        ("ne_cmovent", ctypes.c_uint16),
        ("ne_align", ctypes.c_uint16),
        ("ne_cres", ctypes.c_uint16),
        ("ne_exetyp", ctypes.c_uint8),
        ("ne_flagsothers", ctypes.c_uint8),
        ("ne_pretthunks", ctypes.c_uint16),
        ("ne_psegrefbytes", ctypes.c_uint16),
        ("ne_swaparea", ctypes.c_uint16),
        ("ne_expver", ctypes.c_uint16),
    ]
    ne_magic: int
    ne_ver: str
    ne_rev: str
    ne_enttab: int
    ne_cbenttab: int
    ne_crc: int
    ne_flags: int
    ne_autodata: int
    ne_heap: int
    ne_stack: int
    ne_csip: int
    ne_sssp: int
    ne_cseg: int
    ne_cmod: int
    ne_cbnrestab: int
    ne_segtab: int
    ne_rsrctab: int
    ne_restab: int
    ne_modtab: int
    ne_imptab: int
    ne_nrestab: int
    ne_cmovent: int
    ne_align: int
    ne_cres: int
    ne_exetyp: int
    ne_flagsothers: int
    ne_pretthunks: int
    ne_psegrefbytes: int
    ne_swaparea: int
    ne_expver: int

class _CLAIM_SECURITY_ATTRIBUTE_RELATIVE_V1(ctypes.Structure):
    _fields_ = [
        ("Name", ctypes.c_uint32),
        ("ValueType", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16),
        ("Flags", ctypes.c_uint32),
        ("ValueCount", ctypes.c_uint32),
        ("Values", ctypes.c_char * 0x4),  # _CLAIM_SECURITY_ATTRIBUTE_RELATIVE_V1::<unnamed_type_Values>
    ]
    Name: int
    ValueType: int
    Reserved: int
    Flags: int
    ValueCount: int
    Values: bytes

class tagRID_DEVICE_INFO_HID(ctypes.Structure):
    _fields_ = [
        ("dwVendorId", ctypes.c_uint32),
        ("dwProductId", ctypes.c_uint32),
        ("dwVersionNumber", ctypes.c_uint32),
        ("usUsagePage", ctypes.c_uint16),
        ("usUsage", ctypes.c_uint16),
    ]
    dwVendorId: int
    dwProductId: int
    dwVersionNumber: int
    usUsagePage: int
    usUsage: int

class _IMAGE_DOS_HEADER(ctypes.Structure):
    _fields_ = [
        ("e_magic", ctypes.c_uint16),
        ("e_cblp", ctypes.c_uint16),
        ("e_cp", ctypes.c_uint16),
        ("e_crlc", ctypes.c_uint16),
        ("e_cparhdr", ctypes.c_uint16),
        ("e_minalloc", ctypes.c_uint16),
        ("e_maxalloc", ctypes.c_uint16),
        ("e_ss", ctypes.c_uint16),
        ("e_sp", ctypes.c_uint16),
        ("e_csum", ctypes.c_uint16),
        ("e_ip", ctypes.c_uint16),
        ("e_cs", ctypes.c_uint16),
        ("e_lfarlc", ctypes.c_uint16),
        ("e_ovno", ctypes.c_uint16),
        ("e_res", ctypes.c_uint16 * 0x4),  # unsigned __int16[4]
        ("e_oemid", ctypes.c_uint16),
        ("e_oeminfo", ctypes.c_uint16),
        ("e_res2", ctypes.c_uint16 * 0xA),  # unsigned __int16[10]
        ("e_lfanew", ctypes.c_int32),
    ]
    e_magic: int
    e_cblp: int
    e_cp: int
    e_crlc: int
    e_cparhdr: int
    e_minalloc: int
    e_maxalloc: int
    e_ss: int
    e_sp: int
    e_csum: int
    e_ip: int
    e_cs: int
    e_lfarlc: int
    e_ovno: int
    e_res: list[ctypes.c_uint16]
    e_oemid: int
    e_oeminfo: int
    e_res2: list[ctypes.c_uint16]
    e_lfanew: int

class _NETRESOURCEA(ctypes.Structure):
    _fields_ = [
        ("dwScope", ctypes.c_uint32),
        ("dwType", ctypes.c_uint32),
        ("dwDisplayType", ctypes.c_uint32),
        ("dwUsage", ctypes.c_uint32),
        ("lpLocalName", ctypes.c_char_p),
        ("lpRemoteName", ctypes.c_char_p),
        ("lpComment", ctypes.c_char_p),
        ("lpProvider", ctypes.c_char_p),
    ]
    dwScope: int
    dwType: int
    dwDisplayType: int
    dwUsage: int
    lpLocalName: bytes
    lpRemoteName: bytes
    lpComment: bytes
    lpProvider: bytes

class _CONNECTDLGSTRUCTA(ctypes.Structure):
    _fields_ = [
        ("cbStructure", ctypes.c_uint32),
        ("hwndOwner", ctypes.c_ulonglong),
        ("lpConnRes", ctypes.c_ulonglong),
        ("dwFlags", ctypes.c_uint32),
        ("dwDevNum", ctypes.c_uint32),
    ]
    cbStructure: int
    hwndOwner: int
    lpConnRes: int
    dwFlags: int
    dwDevNum: int

class _RECTL(ctypes.Structure):
    _fields_ = [
        ("left", ctypes.c_int32),
        ("top", ctypes.c_int32),
        ("right", ctypes.c_int32),
        ("bottom", ctypes.c_int32),
    ]
    left: int
    top: int
    right: int
    bottom: int

class PPM_WMI_IDLE_STATE(ctypes.Structure):
    _fields_ = [
        ("Latency", ctypes.c_uint32),
        ("Power", ctypes.c_uint32),
        ("TimeCheck", ctypes.c_uint32),
        ("PromotePercent", ctypes.c_uint8),
        ("DemotePercent", ctypes.c_uint8),
        ("StateType", ctypes.c_uint8),
        ("Reserved", ctypes.c_uint8),
        ("StateFlags", ctypes.c_uint32),
        ("Context", ctypes.c_uint32),
        ("IdleHandler", ctypes.c_uint32),
        ("Reserved1", ctypes.c_uint32),
    ]
    Latency: int
    Power: int
    TimeCheck: int
    PromotePercent: int
    DemotePercent: int
    StateType: int
    Reserved: int
    StateFlags: int
    Context: int
    IdleHandler: int
    Reserved1: int

class _OSVERSIONINFOW(ctypes.Structure):
    _fields_ = [
        ("dwOSVersionInfoSize", ctypes.c_uint32),
        ("dwMajorVersion", ctypes.c_uint32),
        ("dwMinorVersion", ctypes.c_uint32),
        ("dwBuildNumber", ctypes.c_uint32),
        ("dwPlatformId", ctypes.c_uint32),
        ("szCSDVersion", ctypes.c_wchar * 0x80),  # wchar_t[128]
    ]
    dwOSVersionInfoSize: int
    dwMajorVersion: int
    dwMinorVersion: int
    dwBuildNumber: int
    dwPlatformId: int
    szCSDVersion: list[ctypes.c_wchar]

class tagLOGBRUSH(ctypes.Structure):
    _fields_ = [
        ("lbStyle", ctypes.c_uint32),
        ("lbColor", ctypes.c_uint32),
        ("lbHatch", ctypes.c_uint64),
    ]
    lbStyle: int
    lbColor: int
    lbHatch: int

class cGcNPCPlacementComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mbSearchPlacementFromMaster", ctypes.c_ubyte),
        ("mbPlaceInAbandonedSystems", ctypes.c_ubyte),
        ("maPlacementInfosToApply", ctypes.c_char * 0x10),  # cTkDynamicArray<TkID<128> >
        ("mbWaitToPlace", ctypes.c_ubyte),
    ]
    mbSearchPlacementFromMaster: bool
    mbPlaceInAbandonedSystems: bool
    maPlacementInfosToApply: bytes
    mbWaitToPlace: bool

class _POINTL(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
    ]
    x: int
    y: int

class tagLOGFONTW(ctypes.Structure):
    _fields_ = [
        ("lfHeight", ctypes.c_int32),
        ("lfWidth", ctypes.c_int32),
        ("lfEscapement", ctypes.c_int32),
        ("lfOrientation", ctypes.c_int32),
        ("lfWeight", ctypes.c_int32),
        ("lfItalic", ctypes.c_uint8),
        ("lfUnderline", ctypes.c_uint8),
        ("lfStrikeOut", ctypes.c_uint8),
        ("lfCharSet", ctypes.c_uint8),
        ("lfOutPrecision", ctypes.c_uint8),
        ("lfClipPrecision", ctypes.c_uint8),
        ("lfQuality", ctypes.c_uint8),
        ("lfPitchAndFamily", ctypes.c_uint8),
        ("lfFaceName", ctypes.c_wchar * 0x20),  # wchar_t[32]
    ]
    lfHeight: int
    lfWidth: int
    lfEscapement: int
    lfOrientation: int
    lfWeight: int
    lfItalic: int
    lfUnderline: int
    lfStrikeOut: int
    lfCharSet: int
    lfOutPrecision: int
    lfClipPrecision: int
    lfQuality: int
    lfPitchAndFamily: int
    lfFaceName: list[ctypes.c_wchar]

class _REDIRECTION_FUNCTION_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("DllName", ctypes.c_char_p),
        ("FunctionName", ctypes.c_char_p),
        ("RedirectionTarget", ctypes.c_void_p),
    ]
    DllName: bytes
    FunctionName: bytes
    RedirectionTarget: int

class _SERVICE_STATUS_PROCESS(ctypes.Structure):
    _fields_ = [
        ("dwServiceType", ctypes.c_uint32),
        ("dwCurrentState", ctypes.c_uint32),
        ("dwControlsAccepted", ctypes.c_uint32),
        ("dwWin32ExitCode", ctypes.c_uint32),
        ("dwServiceSpecificExitCode", ctypes.c_uint32),
        ("dwCheckPoint", ctypes.c_uint32),
        ("dwWaitHint", ctypes.c_uint32),
        ("dwProcessId", ctypes.c_uint32),
        ("dwServiceFlags", ctypes.c_uint32),
    ]
    dwServiceType: int
    dwCurrentState: int
    dwControlsAccepted: int
    dwWin32ExitCode: int
    dwServiceSpecificExitCode: int
    dwCheckPoint: int
    dwWaitHint: int
    dwProcessId: int
    dwServiceFlags: int

class _SERVICE_FAILURE_ACTIONSA(ctypes.Structure):
    _fields_ = [
        ("dwResetPeriod", ctypes.c_uint32),
        ("lpRebootMsg", ctypes.c_char_p),
        ("lpCommand", ctypes.c_char_p),
        ("cActions", ctypes.c_uint32),
        ("lpsaActions", ctypes.c_ulonglong),
    ]
    dwResetPeriod: int
    lpRebootMsg: bytes
    lpCommand: bytes
    cActions: int
    lpsaActions: int

class tagTEXTMETRICA(ctypes.Structure):
    _fields_ = [
        ("tmHeight", ctypes.c_int32),
        ("tmAscent", ctypes.c_int32),
        ("tmDescent", ctypes.c_int32),
        ("tmInternalLeading", ctypes.c_int32),
        ("tmExternalLeading", ctypes.c_int32),
        ("tmAveCharWidth", ctypes.c_int32),
        ("tmMaxCharWidth", ctypes.c_int32),
        ("tmWeight", ctypes.c_int32),
        ("tmOverhang", ctypes.c_int32),
        ("tmDigitizedAspectX", ctypes.c_int32),
        ("tmDigitizedAspectY", ctypes.c_int32),
        ("tmFirstChar", ctypes.c_uint8),
        ("tmLastChar", ctypes.c_uint8),
        ("tmDefaultChar", ctypes.c_uint8),
        ("tmBreakChar", ctypes.c_uint8),
        ("tmItalic", ctypes.c_uint8),
        ("tmUnderlined", ctypes.c_uint8),
        ("tmStruckOut", ctypes.c_uint8),
        ("tmPitchAndFamily", ctypes.c_uint8),
        ("tmCharSet", ctypes.c_uint8),
    ]
    tmHeight: int
    tmAscent: int
    tmDescent: int
    tmInternalLeading: int
    tmExternalLeading: int
    tmAveCharWidth: int
    tmMaxCharWidth: int
    tmWeight: int
    tmOverhang: int
    tmDigitizedAspectX: int
    tmDigitizedAspectY: int
    tmFirstChar: int
    tmLastChar: int
    tmDefaultChar: int
    tmBreakChar: int
    tmItalic: int
    tmUnderlined: int
    tmStruckOut: int
    tmPitchAndFamily: int
    tmCharSet: int

class tagPANOSE(ctypes.Structure):
    _fields_ = [
        ("bFamilyType", ctypes.c_uint8),
        ("bSerifStyle", ctypes.c_uint8),
        ("bWeight", ctypes.c_uint8),
        ("bProportion", ctypes.c_uint8),
        ("bContrast", ctypes.c_uint8),
        ("bStrokeVariation", ctypes.c_uint8),
        ("bArmStyle", ctypes.c_uint8),
        ("bLetterform", ctypes.c_uint8),
        ("bMidline", ctypes.c_uint8),
        ("bXHeight", ctypes.c_uint8),
    ]
    bFamilyType: int
    bSerifStyle: int
    bWeight: int
    bProportion: int
    bContrast: int
    bStrokeVariation: int
    bArmStyle: int
    bLetterform: int
    bMidline: int
    bXHeight: int

class _TAPE_WRITE_MARKS(ctypes.Structure):
    _fields_ = [
        ("Type", ctypes.c_uint32),
        ("Count", ctypes.c_uint32),
        ("Immediate", ctypes.c_uint8),
    ]
    Type: int
    Count: int
    Immediate: int

class _PROCESS_POWER_THROTTLING_STATE(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("ControlMask", ctypes.c_uint32),
        ("StateMask", ctypes.c_uint32),
    ]
    Version: int
    ControlMask: int
    StateMask: int

class _IMAGE_ALPHA_RUNTIME_FUNCTION_ENTRY(ctypes.Structure):
    _fields_ = [
        ("BeginAddress", ctypes.c_uint32),
        ("EndAddress", ctypes.c_uint32),
        ("ExceptionHandler", ctypes.c_uint32),
        ("HandlerData", ctypes.c_uint32),
        ("PrologEndAddress", ctypes.c_uint32),
    ]
    BeginAddress: int
    EndAddress: int
    ExceptionHandler: int
    HandlerData: int
    PrologEndAddress: int

class _cpinfoexA(ctypes.Structure):
    _fields_ = [
        ("MaxCharSize", ctypes.c_uint32),
        ("DefaultChar", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
        ("LeadByte", ctypes.c_uint8 * 0xC),  # unsigned __int8[12]
        ("UnicodeDefaultChar", ctypes.c_wchar),
        ("CodePage", ctypes.c_uint32),
        ("CodePageName", ctypes.c_char * 0x104),  # char[260]
    ]
    MaxCharSize: int
    DefaultChar: list[ctypes.c_uint8]
    LeadByte: list[ctypes.c_uint8]
    UnicodeDefaultChar: str
    CodePage: int
    CodePageName: list[ctypes.c_char]

class tagSTYLEBUFA(ctypes.Structure):
    _fields_ = [
        ("dwStyle", ctypes.c_uint32),
        ("szDescription", ctypes.c_char * 0x20),  # char[32]
    ]
    dwStyle: int
    szDescription: list[ctypes.c_char]

class _PROCESS_HEAP_ENTRY(ctypes.Structure):
    _fields_ = [
        ("lpData", ctypes.c_void_p),
        ("cbData", ctypes.c_uint32),
        ("cbOverhead", ctypes.c_uint8),
        ("iRegionIndex", ctypes.c_uint8),
        ("wFlags", ctypes.c_uint16),
        ("___u5", ctypes.c_char * 0x18),  # $6EB288B76BF202DA6BA43038528FD807
    ]
    lpData: int
    cbData: int
    cbOverhead: int
    iRegionIndex: int
    wFlags: int
    ___u5: bytes

class _PROCESS_MITIGATION_STRICT_HANDLE_CHECK_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $B526FE4BD365EDCEA67D3C54A2481192
    ]
    ___u0: bytes

class _TypeDescriptor(ctypes.Structure):
    _fields_ = [
        ("pVFTable", ctypes.c_void_p),
        ("spare", ctypes.c_void_p),
        ("name", ctypes.c_char),  # char[]
    ]
    pVFTable: int
    spare: int
    name: list[ctypes.c_char]

class _PMD(ctypes.Structure):
    _fields_ = [
        ("mdisp", ctypes.c_char * 0x4),
        ("pdisp", ctypes.c_char * 0x4),
        ("vdisp", ctypes.c_char * 0x4),
    ]
    mdisp: bytes
    pdisp: bytes
    vdisp: bytes

class _s__RTTIClassHierarchyDescriptor(ctypes.Structure):
    _fields_ = [
        ("signature", ctypes.c_uint32),
        ("attributes", ctypes.c_uint32),
        ("numBaseClasses", ctypes.c_uint32),
        ("pBaseClassArray", ctypes.c_ulonglong),
    ]
    signature: int
    attributes: int
    numBaseClasses: int
    pBaseClassArray: int

class _s__RTTIBaseClassDescriptor2(ctypes.Structure):
    _fields_ = [
        ("pTypeDescriptor", ctypes.c_ulonglong),
        ("numContainedBases", ctypes.c_uint32),
        ("attributes", ctypes.c_uint32),
        ("pClassDescriptor", ctypes.c_ulonglong),
    ]
    pTypeDescriptor: int
    numContainedBases: int
    attributes: int
    pClassDescriptor: int

class _s__RTTIBaseClassArray(ctypes.Structure):
    pass

class _s__RTTIBaseClassDescriptor(ctypes.Structure):
    _fields_ = [
        ("pTypeDescriptor", ctypes.c_ulonglong),
        ("numContainedBases", ctypes.c_uint32),
        ("attributes", ctypes.c_uint32),
        ("pClassDescriptor", ctypes.c_ulonglong),
    ]
    pTypeDescriptor: int
    numContainedBases: int
    attributes: int
    pClassDescriptor: int

class _OBJECT_TYPE_LIST(ctypes.Structure):
    _fields_ = [
        ("Level", ctypes.c_uint16),
        ("Sbz", ctypes.c_uint16),
        ("ObjectType", ctypes.c_ulonglong),
    ]
    Level: int
    Sbz: int
    ObjectType: int

class tagSTYLEBUFW(ctypes.Structure):
    _fields_ = [
        ("dwStyle", ctypes.c_uint32),
        ("szDescription", ctypes.c_wchar * 0x20),  # wchar_t[32]
    ]
    dwStyle: int
    szDescription: list[ctypes.c_wchar]

class cGcPortalComponentData(ctypes.Structure):
    def Cast(): pass

    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfTemp", ctypes.c_float),
    ]
    mfTemp: float

class hkOptionalComponent(ctypes.Structure):
    def addToList(): pass

    def request(): pass

    _fields_ = [
        ("m_next", ctypes.c_ulonglong),
        ("m_name", ctypes.c_char_p),
        ("m_onLink", ctypes.c_ulonglong),
        ("m_dstAddr", ctypes.c_ulonglong),
        ("m_srcValue", ctypes.c_void_p),
        ("m_dependency", ctypes.c_ulonglong),
        ("m_isUsed", ctypes.c_ubyte),
        ("m_isRequested", ctypes.c_ubyte),
    ]
    m_next: int
    m_name: bytes
    m_onLink: int
    m_dstAddr: int
    m_srcValue: int
    m_dependency: int
    m_isUsed: bool
    m_isRequested: bool

class cGcSentinelSpawnSequence(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerFix(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("maWaves", ctypes.c_char * 0x10),  # cTkDynamicArray<cGcSentinelSpawnSequenceStep>
    ]
    maWaves: bytes

class _ACL(ctypes.Structure):
    _fields_ = [
        ("AclRevision", ctypes.c_uint8),
        ("Sbz1", ctypes.c_uint8),
        ("AclSize", ctypes.c_uint16),
        ("AceCount", ctypes.c_uint16),
        ("Sbz2", ctypes.c_uint16),
    ]
    AclRevision: int
    Sbz1: int
    AclSize: int
    AceCount: int
    Sbz2: int

class _SECURITY_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("Revision", ctypes.c_uint8),
        ("Sbz1", ctypes.c_uint8),
        ("Control", ctypes.c_uint16),
        ("Owner", ctypes.c_void_p),
        ("Group", ctypes.c_void_p),
        ("Sacl", ctypes.c_ulonglong),
        ("Dacl", ctypes.c_ulonglong),
    ]
    Revision: int
    Sbz1: int
    Control: int
    Owner: int
    Group: int
    Sacl: int
    Dacl: int

class cTkMatrix34(ctypes.Structure):
    def cTkMatrix34(): pass

    def VectorTransformMatrix(): pass

    _fields_ = [
        ("mRight", common.Vector3f),
        ("mUp", common.Vector3f),
        ("mAt", common.Vector3f),
        ("mPos", common.Vector3f),
    ]
    mRight: common.Vector3f
    mUp: common.Vector3f
    mAt: common.Vector3f
    mPos: common.Vector3f

class cTkQuaternion(ctypes.Structure):
    _fields_ = [
        ("mfX", ctypes.c_float),
        ("mfY", ctypes.c_float),
        ("mfZ", ctypes.c_float),
        ("mfW", ctypes.c_float),
    ]
    mfX: float
    mfY: float
    mfZ: float
    mfW: float

class _SID_IDENTIFIER_AUTHORITY(ctypes.Structure):
    _fields_ = [
        ("Value", ctypes.c_uint8 * 0x6),  # unsigned __int8[6]
    ]
    Value: list[ctypes.c_uint8]

class _CONSOLE_CURSOR_INFO(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("bVisible", ctypes.c_int32),
    ]
    dwSize: int
    bVisible: int

class tagHW_PROFILE_INFOA(ctypes.Structure):
    _fields_ = [
        ("dwDockInfo", ctypes.c_uint32),
        ("szHwProfileGuid", ctypes.c_char * 0x27),  # char[39]
        ("szHwProfileName", ctypes.c_char * 0x50),  # char[80]
    ]
    dwDockInfo: int
    szHwProfileGuid: list[ctypes.c_char]
    szHwProfileName: list[ctypes.c_char]

class _SYSTEMTIME(ctypes.Structure):
    _fields_ = [
        ("wYear", ctypes.c_uint16),
        ("wMonth", ctypes.c_uint16),
        ("wDayOfWeek", ctypes.c_uint16),
        ("wDay", ctypes.c_uint16),
        ("wHour", ctypes.c_uint16),
        ("wMinute", ctypes.c_uint16),
        ("wSecond", ctypes.c_uint16),
        ("wMilliseconds", ctypes.c_uint16),
    ]
    wYear: int
    wMonth: int
    wDayOfWeek: int
    wDay: int
    wHour: int
    wMinute: int
    wSecond: int
    wMilliseconds: int

class _FILETIME(ctypes.Structure):
    _fields_ = [
        ("dwLowDateTime", ctypes.c_uint32),
        ("dwHighDateTime", ctypes.c_uint32),
    ]
    dwLowDateTime: int
    dwHighDateTime: int

class tagPOINTS(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int16),
        ("y", ctypes.c_int16),
    ]
    x: int
    y: int

class _TAPE_CREATE_PARTITION(ctypes.Structure):
    _fields_ = [
        ("Method", ctypes.c_uint32),
        ("Count", ctypes.c_uint32),
        ("Size", ctypes.c_uint32),
    ]
    Method: int
    Count: int
    Size: int

class tagACTCTXA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("lpSource", ctypes.c_char_p),
        ("wProcessorArchitecture", ctypes.c_uint16),
        ("wLangId", ctypes.c_uint16),
        ("lpAssemblyDirectory", ctypes.c_char_p),
        ("lpResourceName", ctypes.c_char_p),
        ("lpApplicationName", ctypes.c_char_p),
        ("hModule", ctypes.c_ulonglong),
    ]
    cbSize: int
    dwFlags: int
    lpSource: bytes
    wProcessorArchitecture: int
    wLangId: int
    lpAssemblyDirectory: bytes
    lpResourceName: bytes
    lpApplicationName: bytes
    hModule: int

class hkGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _iobuf(ctypes.Structure):
    _fields_ = [
        ("_Placeholder", ctypes.c_void_p),
    ]
    _Placeholder: int

class _s__RTTICompleteObjectLocator2(ctypes.Structure):
    _fields_ = [
        ("signature", ctypes.c_uint32),
        ("offset", ctypes.c_uint32),
        ("cdOffset", ctypes.c_uint32),
        ("pTypeDescriptor", ctypes.c_ulonglong),
        ("pClassDescriptor", ctypes.c_ulonglong),
        ("pSelf", ctypes.c_ulonglong),
    ]
    signature: int
    offset: int
    cdOffset: int
    pTypeDescriptor: int
    pClassDescriptor: int
    pSelf: int

class _REASON_CONTEXT(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("Reason", ctypes.c_char * 0x18),  # _REASON_CONTEXT::<unnamed_type_Reason>
    ]
    Version: int
    Flags: int
    Reason: bytes

class _CLAIM_SECURITY_ATTRIBUTE_FQBN_VALUE(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint64),
        ("Name", ctypes.c_ulonglong),
    ]
    Version: int
    Name: int

class _CLAIM_SECURITY_ATTRIBUTE_OCTET_STRING_VALUE(ctypes.Structure):
    _fields_ = [
        ("pValue", ctypes.c_void_p),
        ("ValueLength", ctypes.c_uint32),
    ]
    pValue: int
    ValueLength: int

class _CLAIM_SECURITY_ATTRIBUTE_V1(ctypes.Structure):
    _fields_ = [
        ("Name", ctypes.c_ulonglong),
        ("ValueType", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16),
        ("Flags", ctypes.c_uint32),
        ("ValueCount", ctypes.c_uint32),
        ("Values", ctypes.c_char * 0x8),  # _CLAIM_SECURITY_ATTRIBUTE_V1::<unnamed_type_Values>
    ]
    Name: int
    ValueType: int
    Reserved: int
    Flags: int
    ValueCount: int
    Values: bytes

class _POINTFLOAT(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
    ]
    x: float
    y: float

class _COMMTIMEOUTS(ctypes.Structure):
    _fields_ = [
        ("ReadIntervalTimeout", ctypes.c_uint32),
        ("ReadTotalTimeoutMultiplier", ctypes.c_uint32),
        ("ReadTotalTimeoutConstant", ctypes.c_uint32),
        ("WriteTotalTimeoutMultiplier", ctypes.c_uint32),
        ("WriteTotalTimeoutConstant", ctypes.c_uint32),
    ]
    ReadIntervalTimeout: int
    ReadTotalTimeoutMultiplier: int
    ReadTotalTimeoutConstant: int
    WriteTotalTimeoutMultiplier: int
    WriteTotalTimeoutConstant: int

class cTkLinearPoolJanitor(ctypes.Structure):
    def cTkLinearPoolJanitor(): pass

    _fields_ = [
        ("mpPool", ctypes.c_ulonglong),
        ("lpAddress", ctypes.c_uint64),
        ("lpLastAlloc", ctypes.c_void_p),
    ]
    mpPool: int
    lpAddress: int
    lpLastAlloc: int

class _STARTUPINFOA(ctypes.Structure):
    _fields_ = [
        ("cb", ctypes.c_uint32),
        ("lpReserved", ctypes.c_char_p),
        ("lpDesktop", ctypes.c_char_p),
        ("lpTitle", ctypes.c_char_p),
        ("dwX", ctypes.c_uint32),
        ("dwY", ctypes.c_uint32),
        ("dwXSize", ctypes.c_uint32),
        ("dwYSize", ctypes.c_uint32),
        ("dwXCountChars", ctypes.c_uint32),
        ("dwYCountChars", ctypes.c_uint32),
        ("dwFillAttribute", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("wShowWindow", ctypes.c_uint16),
        ("cbReserved2", ctypes.c_uint16),
        ("lpReserved2", ctypes.c_ulonglong),
        ("hStdInput", ctypes.c_void_p),
        ("hStdOutput", ctypes.c_void_p),
        ("hStdError", ctypes.c_void_p),
    ]
    cb: int
    lpReserved: bytes
    lpDesktop: bytes
    lpTitle: bytes
    dwX: int
    dwY: int
    dwXSize: int
    dwYSize: int
    dwXCountChars: int
    dwYCountChars: int
    dwFillAttribute: int
    dwFlags: int
    wShowWindow: int
    cbReserved2: int
    lpReserved2: int
    hStdInput: int
    hStdOutput: int
    hStdError: int

class HPALETTE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class value_entW(ctypes.Structure):
    _fields_ = [
        ("ve_valuename", ctypes.c_ulonglong),
        ("ve_valuelen", ctypes.c_uint32),
        ("ve_valueptr", ctypes.c_uint64),
        ("ve_type", ctypes.c_uint32),
    ]
    ve_valuename: int
    ve_valuelen: int
    ve_valueptr: int
    ve_type: int

class _SERVICE_LAUNCH_PROTECTED_INFO(ctypes.Structure):
    _fields_ = [
        ("dwLaunchProtected", ctypes.c_uint32),
    ]
    dwLaunchProtected: int

class PPM_THERMAL_POLICY_EVENT(ctypes.Structure):
    _fields_ = [
        ("Mode", ctypes.c_uint8),
        ("Processors", ctypes.c_uint64),
    ]
    Mode: int
    Processors: int

class _IMAGE_AUX_SYMBOL_EX: pass

class IMAGE_AUX_SYMBOL_TOKEN_DEF(ctypes.Structure):
    _fields_ = [
        ("bAuxType", ctypes.c_uint8),
        ("bReserved", ctypes.c_uint8),
        ("SymbolTableIndex", ctypes.c_uint32),
        ("rgbReserved", ctypes.c_uint8 * 0xC),  # unsigned __int8[12]
    ]
    bAuxType: int
    bReserved: int
    SymbolTableIndex: int
    rgbReserved: list[ctypes.c_uint8]

class tagWINDOWPOS(ctypes.Structure):
    _fields_ = [
        ("hwnd", ctypes.c_ulonglong),
        ("hwndInsertAfter", ctypes.c_ulonglong),
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("_cx", ctypes.c_int32),
        ("cy", ctypes.c_int32),
        ("flags", ctypes.c_uint32),
    ]
    hwnd: int
    hwndInsertAfter: int
    x: int
    y: int
    _cx: int
    cy: int
    flags: int

class tagBITMAPFILEHEADER(ctypes.Structure):
    _fields_ = [
        ("bfType", ctypes.c_uint16),
        ("bfSize", ctypes.c_uint32),
        ("bfReserved1", ctypes.c_uint16),
        ("bfReserved2", ctypes.c_uint16),
        ("bfOffBits", ctypes.c_uint32),
    ]
    bfType: int
    bfSize: int
    bfReserved1: int
    bfReserved2: int
    bfOffBits: int

class tagACTCTX_SECTION_KEYED_DATA_ASSEMBLY_METADATA(ctypes.Structure):
    _fields_ = [
        ("lpInformation", ctypes.c_void_p),
        ("lpSectionBase", ctypes.c_void_p),
        ("ulSectionLength", ctypes.c_uint32),
        ("lpSectionGlobalDataBase", ctypes.c_void_p),
        ("ulSectionGlobalDataLength", ctypes.c_uint32),
    ]
    lpInformation: int
    lpSectionBase: int
    ulSectionLength: int
    lpSectionGlobalDataBase: int
    ulSectionGlobalDataLength: int

class _SILOOBJECT_BASIC_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("SiloId", ctypes.c_uint32),
        ("SiloParentId", ctypes.c_uint32),
        ("NumberOfProcesses", ctypes.c_uint32),
        ("IsInServerSilo", ctypes.c_uint8),
        ("Reserved", ctypes.c_uint8 * 0x3),  # unsigned __int8[3]
    ]
    SiloId: int
    SiloParentId: int
    NumberOfProcesses: int
    IsInServerSilo: int
    Reserved: list[ctypes.c_uint8]

class _MEMORYSTATUSEX(ctypes.Structure):
    _fields_ = [
        ("dwLength", ctypes.c_uint32),
        ("dwMemoryLoad", ctypes.c_uint32),
        ("ullTotalPhys", ctypes.c_uint64),
        ("ullAvailPhys", ctypes.c_uint64),
        ("ullTotalPageFile", ctypes.c_uint64),
        ("ullAvailPageFile", ctypes.c_uint64),
        ("ullTotalVirtual", ctypes.c_uint64),
        ("ullAvailVirtual", ctypes.c_uint64),
        ("ullAvailExtendedVirtual", ctypes.c_uint64),
    ]
    dwLength: int
    dwMemoryLoad: int
    ullTotalPhys: int
    ullAvailPhys: int
    ullTotalPageFile: int
    ullAvailPageFile: int
    ullTotalVirtual: int
    ullAvailVirtual: int
    ullAvailExtendedVirtual: int

class tagMDINEXTMENU(ctypes.Structure):
    _fields_ = [
        ("hmenuIn", ctypes.c_ulonglong),
        ("hmenuNext", ctypes.c_ulonglong),
        ("hwndNext", ctypes.c_ulonglong),
    ]
    hmenuIn: int
    hmenuNext: int
    hwndNext: int

class tagACTCTXW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("lpSource", ctypes.c_ulonglong),
        ("wProcessorArchitecture", ctypes.c_uint16),
        ("wLangId", ctypes.c_uint16),
        ("lpAssemblyDirectory", ctypes.c_ulonglong),
        ("lpResourceName", ctypes.c_ulonglong),
        ("lpApplicationName", ctypes.c_ulonglong),
        ("hModule", ctypes.c_ulonglong),
    ]
    cbSize: int
    dwFlags: int
    lpSource: int
    wProcessorArchitecture: int
    wLangId: int
    lpAssemblyDirectory: int
    lpResourceName: int
    lpApplicationName: int
    hModule: int

class _OFSTRUCT(ctypes.Structure):
    _fields_ = [
        ("cBytes", ctypes.c_uint8),
        ("fFixedDisk", ctypes.c_uint8),
        ("nErrCode", ctypes.c_uint16),
        ("Reserved1", ctypes.c_uint16),
        ("Reserved2", ctypes.c_uint16),
        ("szPathName", ctypes.c_char * 0x80),  # char[128]
    ]
    cBytes: int
    fFixedDisk: int
    nErrCode: int
    Reserved1: int
    Reserved2: int
    szPathName: list[ctypes.c_char]

class _OVERLAPPED(ctypes.Structure):
    _fields_ = [
        ("Internal", ctypes.c_uint64),
        ("InternalHigh", ctypes.c_uint64),
        ("___u2", ctypes.c_char * 0x8),  # $A6255257DFCD8BC7ECAC29A14A232D80
        ("hEvent", ctypes.c_void_p),
    ]
    Internal: int
    InternalHigh: int
    ___u2: bytes
    hEvent: int

class _OVERLAPPED_ENTRY(ctypes.Structure):
    _fields_ = [
        ("lpCompletionKey", ctypes.c_uint64),
        ("lpOverlapped", ctypes.c_ulonglong),
        ("Internal", ctypes.c_uint64),
        ("dwNumberOfBytesTransferred", ctypes.c_uint32),
    ]
    lpCompletionKey: int
    lpOverlapped: int
    Internal: int
    dwNumberOfBytesTransferred: int

class tagHELPWININFOW(ctypes.Structure):
    _fields_ = [
        ("wStructSize", ctypes.c_int32),
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("_dx", ctypes.c_int32),
        ("dy", ctypes.c_int32),
        ("wMax", ctypes.c_int32),
        ("rgchMember", ctypes.c_wchar * 0x2),  # wchar_t[2]
    ]
    wStructSize: int
    x: int
    y: int
    _dx: int
    dy: int
    wMax: int
    rgchMember: list[ctypes.c_wchar]

class cTkResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int

class _IMAGE_PROLOGUE_DYNAMIC_RELOCATION_HEADER(ctypes.Structure):
    _fields_ = [
        ("PrologueByteCount", ctypes.c_uint8),
    ]
    PrologueByteCount: int

class tagXFORM(ctypes.Structure):
    _fields_ = [
        ("eM11", ctypes.c_float),
        ("eM12", ctypes.c_float),
        ("eM21", ctypes.c_float),
        ("eM22", ctypes.c_float),
        ("_eDx", ctypes.c_float),
        ("eDy", ctypes.c_float),
    ]
    eM11: float
    eM12: float
    eM21: float
    eM22: float
    _eDx: float
    eDy: float

class tagHARDWAREINPUT(ctypes.Structure):
    _fields_ = [
        ("uMsg", ctypes.c_uint32),
        ("wParamL", ctypes.c_uint16),
        ("wParamH", ctypes.c_uint16),
    ]
    uMsg: int
    wParamL: int
    wParamH: int

class tagMEASUREITEMSTRUCT(ctypes.Structure):
    _fields_ = [
        ("CtlType", ctypes.c_uint32),
        ("CtlID", ctypes.c_uint32),
        ("itemID", ctypes.c_uint32),
        ("itemWidth", ctypes.c_uint32),
        ("itemHeight", ctypes.c_uint32),
        ("itemData", ctypes.c_uint64),
    ]
    CtlType: int
    CtlID: int
    itemID: int
    itemWidth: int
    itemHeight: int
    itemData: int

class _HEAP_SUMMARY(ctypes.Structure):
    _fields_ = [
        ("cb", ctypes.c_uint32),
        ("cbAllocated", ctypes.c_uint64),
        ("cbCommitted", ctypes.c_uint64),
        ("cbReserved", ctypes.c_uint64),
        ("cbMaxReserve", ctypes.c_uint64),
    ]
    cb: int
    cbAllocated: int
    cbCommitted: int
    cbReserved: int
    cbMaxReserve: int

class HDC__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _PROCESS_MITIGATION_BINARY_SIGNATURE_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $2A1B74101BA262867FD6B869F18EAD63
    ]
    ___u0: bytes

class _SUPPORTED_OS_INFO(ctypes.Structure):
    _fields_ = [
        ("MajorVersion", ctypes.c_uint16),
        ("MinorVersion", ctypes.c_uint16),
    ]
    MajorVersion: int
    MinorVersion: int

class cTkAssertData(ctypes.Structure):
    _fields_ = [
        ("fgwnd", ctypes.c_void_p),
        ("file", ctypes.c_char_p),
        ("line", ctypes.c_uint32),
        ("fn", ctypes.c_char_p),
        ("expression", ctypes.c_char_p),
        ("message", ctypes.c_char_p),
        ("callstack", ctypes.c_char_p),
        ("mbPrintCallstack", ctypes.c_ubyte),
    ]
    fgwnd: int
    file: bytes
    line: int
    fn: bytes
    expression: bytes
    message: bytes
    callstack: bytes
    mbPrintCallstack: bool

class hkLinearBuffer(ctypes.Structure):
    def blockAllocAligned(): pass

    def blockAlloc(): pass

    _fields_ = [
        ("m_bufferCur", ctypes.c_void_p),
        ("m_bufferEnd", ctypes.c_void_p),
        ("m_bufferStart", ctypes.c_void_p),
    ]
    m_bufferCur: int
    m_bufferEnd: int
    m_bufferStart: int

class VarFromStringVisitor(ctypes.Structure):
    _fields_ = [
        ("m_ownerType", ctypes.c_ulonglong),
        ("m_extraBuffer", ctypes.c_ulonglong),
    ]
    m_ownerType: int
    m_extraBuffer: int

class tagLOCALESIGNATURE(ctypes.Structure):
    _fields_ = [
        ("lsUsb", ctypes.c_uint32 * 0x4),  # unsigned int[4]
        ("lsCsbDefault", ctypes.c_uint32 * 0x2),  # unsigned int[2]
        ("lsCsbSupported", ctypes.c_uint32 * 0x2),  # unsigned int[2]
    ]
    lsUsb: list[ctypes.c_uint32]
    lsCsbDefault: list[ctypes.c_uint32]
    lsCsbSupported: list[ctypes.c_uint32]

class tagPALETTEENTRY(ctypes.Structure):
    _fields_ = [
        ("peRed", ctypes.c_uint8),
        ("peGreen", ctypes.c_uint8),
        ("peBlue", ctypes.c_uint8),
        ("peFlags", ctypes.c_uint8),
    ]
    peRed: int
    peGreen: int
    peBlue: int
    peFlags: int

class tagIMEMENUITEMINFOA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fType", ctypes.c_uint32),
        ("fState", ctypes.c_uint32),
        ("wID", ctypes.c_uint32),
        ("hbmpChecked", ctypes.c_ulonglong),
        ("hbmpUnchecked", ctypes.c_ulonglong),
        ("dwItemData", ctypes.c_uint32),
        ("szString", ctypes.c_char * 0x50),  # char[80]
        ("hbmpItem", ctypes.c_ulonglong),
    ]
    cbSize: int
    fType: int
    fState: int
    wID: int
    hbmpChecked: int
    hbmpUnchecked: int
    dwItemData: int
    szString: list[ctypes.c_char]
    hbmpItem: int

class _PROCESS_MITIGATION_DEP_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $6CE68FE31A595E10DB99BF9A94F83B97
        ("Permanent", ctypes.c_uint8),
    ]
    ___u0: bytes
    Permanent: int

class _TAPE_SET_MEDIA_PARAMETERS(ctypes.Structure):
    _fields_ = [
        ("BlockSize", ctypes.c_uint32),
    ]
    BlockSize: int

class tagBITMAPCOREHEADER(ctypes.Structure):
    _fields_ = [
        ("bcSize", ctypes.c_uint32),
        ("bcWidth", ctypes.c_uint16),
        ("bcHeight", ctypes.c_uint16),
        ("bcPlanes", ctypes.c_uint16),
        ("bcBitCount", ctypes.c_uint16),
    ]
    bcSize: int
    bcWidth: int
    bcHeight: int
    bcPlanes: int
    bcBitCount: int

class _BLENDFUNCTION(ctypes.Structure):
    _fields_ = [
        ("BlendOp", ctypes.c_uint8),
        ("BlendFlags", ctypes.c_uint8),
        ("SourceConstantAlpha", ctypes.c_uint8),
        ("AlphaFormat", ctypes.c_uint8),
    ]
    BlendOp: int
    BlendFlags: int
    SourceConstantAlpha: int
    AlphaFormat: int

class HWINSTA__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class tagSIZE(ctypes.Structure):
    _fields_ = [
        ("_cx", ctypes.c_int32),
        ("cy", ctypes.c_int32),
    ]
    _cx: int
    cy: int

class tagACTCTX_SECTION_KEYED_DATA_2600(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("ulDataFormatVersion", ctypes.c_uint32),
        ("lpData", ctypes.c_void_p),
        ("ulLength", ctypes.c_uint32),
        ("lpSectionGlobalData", ctypes.c_void_p),
        ("ulSectionGlobalDataLength", ctypes.c_uint32),
        ("lpSectionBase", ctypes.c_void_p),
        ("ulSectionTotalLength", ctypes.c_uint32),
        ("hActCtx", ctypes.c_void_p),
        ("ulAssemblyRosterIndex", ctypes.c_uint32),
    ]
    cbSize: int
    ulDataFormatVersion: int
    lpData: int
    ulLength: int
    lpSectionGlobalData: int
    ulSectionGlobalDataLength: int
    lpSectionBase: int
    ulSectionTotalLength: int
    hActCtx: int
    ulAssemblyRosterIndex: int

class _IMAGE_DEBUG_MISC(ctypes.Structure):
    _fields_ = [
        ("DataType", ctypes.c_uint32),
        ("Length", ctypes.c_uint32),
        ("Unicode", ctypes.c_uint8),
        ("Reserved", ctypes.c_uint8 * 0x3),  # unsigned __int8[3]
        ("Data", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
    ]
    DataType: int
    Length: int
    Unicode: int
    Reserved: list[ctypes.c_uint8]
    Data: list[ctypes.c_uint8]

class tagBITMAP(ctypes.Structure):
    _fields_ = [
        ("bmType", ctypes.c_int32),
        ("bmWidth", ctypes.c_int32),
        ("bmHeight", ctypes.c_int32),
        ("bmWidthBytes", ctypes.c_int32),
        ("bmPlanes", ctypes.c_uint16),
        ("bmBitsPixel", ctypes.c_uint16),
        ("bmBits", ctypes.c_void_p),
    ]
    bmType: int
    bmWidth: int
    bmHeight: int
    bmWidthBytes: int
    bmPlanes: int
    bmBitsPixel: int
    bmBits: int

class tagBITMAPINFOHEADER(ctypes.Structure):
    _fields_ = [
        ("biSize", ctypes.c_uint32),
        ("biWidth", ctypes.c_int32),
        ("biHeight", ctypes.c_int32),
        ("biPlanes", ctypes.c_uint16),
        ("biBitCount", ctypes.c_uint16),
        ("biCompression", ctypes.c_uint32),
        ("biSizeImage", ctypes.c_uint32),
        ("biXPelsPerMeter", ctypes.c_int32),
        ("biYPelsPerMeter", ctypes.c_int32),
        ("biClrUsed", ctypes.c_uint32),
        ("biClrImportant", ctypes.c_uint32),
    ]
    biSize: int
    biWidth: int
    biHeight: int
    biPlanes: int
    biBitCount: int
    biCompression: int
    biSizeImage: int
    biXPelsPerMeter: int
    biYPelsPerMeter: int
    biClrUsed: int
    biClrImportant: int

class _DCB(ctypes.Structure):
    _fields_ = [
        ("DCBlength", ctypes.c_uint32),
        ("BaudRate", ctypes.c_uint32),
        ("_bf_8", ctypes.c_int32),
        ("wReserved", ctypes.c_uint16),
        ("XonLim", ctypes.c_uint16),
        ("XoffLim", ctypes.c_uint16),
        ("ByteSize", ctypes.c_uint8),
        ("Parity", ctypes.c_uint8),
        ("StopBits", ctypes.c_uint8),
        ("XonChar", ctypes.c_char),
        ("XoffChar", ctypes.c_char),
        ("ErrorChar", ctypes.c_char),
        ("EofChar", ctypes.c_char),
        ("EvtChar", ctypes.c_char),
        ("wReserved1", ctypes.c_uint16),
    ]
    DCBlength: int
    BaudRate: int
    _bf_8: int
    wReserved: int
    XonLim: int
    XoffLim: int
    ByteSize: int
    Parity: int
    StopBits: int
    XonChar: str
    XoffChar: str
    ErrorChar: str
    EofChar: str
    EvtChar: str
    wReserved1: int

class cTkClock(ctypes.Structure):
    def GetTime(): pass

    def Construct(): pass

    def GetSystemFrequency(): pass

    def GetSystemTime(): pass

    def GetSystemTimeAsSeconds(): pass

    def GetSystemTimeSinceBootAsSeconds(): pass

    def InitMinFrameTimeSyncPoint(): pass

    def MarkBootTime(): pass

    def Prepare(): pass

    def Update(): pass

    def WaitForMinFrameTime(): pass

    def ZeroTimeUpdate(): pass

    _fields_ = [
        ("mu64StartTime", ctypes.c_uint64),
        ("mu64CurrentTime", ctypes.c_uint64),
        ("muLastWaitTicksFinish", ctypes.c_uint64),
        ("mfTimeStep", ctypes.c_float),
        ("mfTotalTime", ctypes.c_float),
        ("mfPrevTotalTime", ctypes.c_float),
        ("mfSlowMotionFactor", ctypes.c_float),
        ("muFrameCount", ctypes.c_uint64),
        ("mbMinFrameTimeSynced", ctypes.c_ubyte),
        ("mbPaused", ctypes.c_ubyte),
        ("mbRealtime", ctypes.c_ubyte),
    ]
    mu64StartTime: int
    mu64CurrentTime: int
    muLastWaitTicksFinish: int
    mfTimeStep: float
    mfTotalTime: float
    mfPrevTotalTime: float
    mfSlowMotionFactor: float
    muFrameCount: int
    mbMinFrameTimeSynced: bool
    mbPaused: bool
    mbRealtime: bool

class _lldiv_t(ctypes.Structure):
    _fields_ = [
        ("quot", ctypes.c_int64),
        ("rem", ctypes.c_int64),
    ]
    quot: int
    rem: int

class tagAXISINFOA(ctypes.Structure):
    _fields_ = [
        ("axMinValue", ctypes.c_int32),
        ("axMaxValue", ctypes.c_int32),
        ("axAxisName", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
    ]
    axMinValue: int
    axMaxValue: int
    axAxisName: list[ctypes.c_uint8]

class _EXCEPTION_RECORD(ctypes.Structure):
    _fields_ = [
        ("ExceptionCode", ctypes.c_uint32),
        ("ExceptionFlags", ctypes.c_uint32),
        ("ExceptionRecord", ctypes.c_ulonglong),
        ("ExceptionAddress", ctypes.c_void_p),
        ("NumberParameters", ctypes.c_uint32),
        ("ExceptionInformation", ctypes.c_uint64 * 0xF),  # unsigned __int64[15]
    ]
    ExceptionCode: int
    ExceptionFlags: int
    ExceptionRecord: int
    ExceptionAddress: int
    NumberParameters: int
    ExceptionInformation: list[ctypes.c_uint64]

class _ENCLAVE_INIT_INFO_VBS_BASIC(ctypes.Structure):
    _fields_ = [
        ("FamilyId", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("ImageId", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("EnclaveSize", ctypes.c_uint64),
        ("EnclaveSvn", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint32),
        ("___u5", ctypes.c_char * 0x8),  # $CD149B68642948FA2C54F72F5474F3EE
    ]
    FamilyId: list[ctypes.c_uint8]
    ImageId: list[ctypes.c_uint8]
    EnclaveSize: int
    EnclaveSvn: int
    Reserved: int
    ___u5: bytes

class tagTRACKMOUSEEVENT(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("hwndTrack", ctypes.c_ulonglong),
        ("dwHoverTime", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int
    hwndTrack: int
    dwHoverTime: int

class ApplyDefaultsVisitor(ctypes.Structure):
    pass

class AssignVisitor(ctypes.Structure):
    _fields_ = [
        ("m_var", ctypes.c_ulonglong),
    ]
    m_var: int

class __m512i: pass

class _IMAGE_LOAD_CONFIG_CODE_INTEGRITY(ctypes.Structure):
    _fields_ = [
        ("Flags", ctypes.c_uint16),
        ("Catalog", ctypes.c_uint16),
        ("CatalogOffset", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint32),
    ]
    Flags: int
    Catalog: int
    CatalogOffset: int
    Reserved: int

class FLASHWINFO(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("hwnd", ctypes.c_ulonglong),
        ("dwFlags", ctypes.c_uint32),
        ("uCount", ctypes.c_uint32),
        ("dwTimeout", ctypes.c_uint32),
    ]
    cbSize: int
    hwnd: int
    dwFlags: int
    uCount: int
    dwTimeout: int

class _CUSTOM_SYSTEM_EVENT_TRIGGER_CONFIG(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("TriggerId", ctypes.c_ulonglong),
    ]
    Size: int
    TriggerId: int

class tagHW_PROFILE_INFOW(ctypes.Structure):
    _fields_ = [
        ("dwDockInfo", ctypes.c_uint32),
        ("szHwProfileGuid", ctypes.c_wchar * 0x27),  # wchar_t[39]
        ("szHwProfileName", ctypes.c_wchar * 0x50),  # wchar_t[80]
    ]
    dwDockInfo: int
    szHwProfileGuid: list[ctypes.c_wchar]
    szHwProfileName: list[ctypes.c_wchar]

class tagMSGBOXPARAMSW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("hwndOwner", ctypes.c_ulonglong),
        ("hInstance", ctypes.c_ulonglong),
        ("lpszText", ctypes.c_ulonglong),
        ("lpszCaption", ctypes.c_ulonglong),
        ("dwStyle", ctypes.c_uint32),
        ("lpszIcon", ctypes.c_ulonglong),
        ("dwContextHelpId", ctypes.c_uint64),
        ("lpfnMsgBoxCallback", ctypes.c_ulonglong),
        ("dwLanguageId", ctypes.c_uint32),
    ]
    cbSize: int
    hwndOwner: int
    hInstance: int
    lpszText: int
    lpszCaption: int
    dwStyle: int
    lpszIcon: int
    dwContextHelpId: int
    lpfnMsgBoxCallback: int
    dwLanguageId: int

class _IMAGE_RUNTIME_FUNCTION_ENTRY(ctypes.Structure):
    _fields_ = [
        ("BeginAddress", ctypes.c_uint32),
        ("EndAddress", ctypes.c_uint32),
        ("___u2", ctypes.c_char * 0x4),  # $2C177DDEB22292785570970034678B30
    ]
    BeginAddress: int
    EndAddress: int
    ___u2: bytes

class _EVENTLOGRECORD(ctypes.Structure):
    _fields_ = [
        ("Length", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint32),
        ("RecordNumber", ctypes.c_uint32),
        ("TimeGenerated", ctypes.c_uint32),
        ("TimeWritten", ctypes.c_uint32),
        ("EventID", ctypes.c_uint32),
        ("EventType", ctypes.c_uint16),
        ("NumStrings", ctypes.c_uint16),
        ("EventCategory", ctypes.c_uint16),
        ("ReservedFlags", ctypes.c_uint16),
        ("ClosingRecordNumber", ctypes.c_uint32),
        ("StringOffset", ctypes.c_uint32),
        ("UserSidLength", ctypes.c_uint32),
        ("UserSidOffset", ctypes.c_uint32),
        ("DataLength", ctypes.c_uint32),
        ("DataOffset", ctypes.c_uint32),
    ]
    Length: int
    Reserved: int
    RecordNumber: int
    TimeGenerated: int
    TimeWritten: int
    EventID: int
    EventType: int
    NumStrings: int
    EventCategory: int
    ReservedFlags: int
    ClosingRecordNumber: int
    StringOffset: int
    UserSidLength: int
    UserSidOffset: int
    DataLength: int
    DataOffset: int

class tagCLIENTCREATESTRUCT(ctypes.Structure):
    _fields_ = [
        ("hWindowMenu", ctypes.c_void_p),
        ("idFirstChild", ctypes.c_uint32),
    ]
    hWindowMenu: int
    idFirstChild: int

class _JOBOBJECT_IO_ATTRIBUTION_STATS(ctypes.Structure):
    _fields_ = [
        ("IoCount", ctypes.c_uint64),
        ("TotalNonOverlappedQueueTime", ctypes.c_uint64),
        ("TotalNonOverlappedServiceTime", ctypes.c_uint64),
        ("TotalSize", ctypes.c_uint64),
    ]
    IoCount: int
    TotalNonOverlappedQueueTime: int
    TotalNonOverlappedServiceTime: int
    TotalSize: int

class tagEXTLOGPEN32(ctypes.Structure):
    _fields_ = [
        ("elpPenStyle", ctypes.c_uint32),
        ("elpWidth", ctypes.c_uint32),
        ("elpBrushStyle", ctypes.c_uint32),
        ("elpColor", ctypes.c_uint32),
        ("elpHatch", ctypes.c_uint32),
        ("elpNumEntries", ctypes.c_uint32),
        ("elpStyleEntry", ctypes.c_uint32 * 0x1),  # unsigned int[1]
    ]
    elpPenStyle: int
    elpWidth: int
    elpBrushStyle: int
    elpColor: int
    elpHatch: int
    elpNumEntries: int
    elpStyleEntry: list[ctypes.c_uint32]

class _SETJMP_FLOAT128(ctypes.Structure):
    _fields_ = [
        ("Part", ctypes.c_uint64 * 0x2),  # unsigned __int64[2]
    ]
    Part: list[ctypes.c_uint64]

class _ICONINFOEXW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fIcon", ctypes.c_int32),
        ("xHotspot", ctypes.c_uint32),
        ("yHotspot", ctypes.c_uint32),
        ("hbmMask", ctypes.c_ulonglong),
        ("hbmColor", ctypes.c_ulonglong),
        ("wResID", ctypes.c_uint16),
        ("szModName", ctypes.c_wchar * 0x104),  # wchar_t[260]
        ("szResName", ctypes.c_wchar * 0x104),  # wchar_t[260]
    ]
    cbSize: int
    fIcon: int
    xHotspot: int
    yHotspot: int
    hbmMask: int
    hbmColor: int
    wResID: int
    szModName: list[ctypes.c_wchar]
    szResName: list[ctypes.c_wchar]

class tagNEWTEXTMETRICW(ctypes.Structure):
    _fields_ = [
        ("tmHeight", ctypes.c_int32),
        ("tmAscent", ctypes.c_int32),
        ("tmDescent", ctypes.c_int32),
        ("tmInternalLeading", ctypes.c_int32),
        ("tmExternalLeading", ctypes.c_int32),
        ("tmAveCharWidth", ctypes.c_int32),
        ("tmMaxCharWidth", ctypes.c_int32),
        ("tmWeight", ctypes.c_int32),
        ("tmOverhang", ctypes.c_int32),
        ("tmDigitizedAspectX", ctypes.c_int32),
        ("tmDigitizedAspectY", ctypes.c_int32),
        ("tmFirstChar", ctypes.c_wchar),
        ("tmLastChar", ctypes.c_wchar),
        ("tmDefaultChar", ctypes.c_wchar),
        ("tmBreakChar", ctypes.c_wchar),
        ("tmItalic", ctypes.c_uint8),
        ("tmUnderlined", ctypes.c_uint8),
        ("tmStruckOut", ctypes.c_uint8),
        ("tmPitchAndFamily", ctypes.c_uint8),
        ("tmCharSet", ctypes.c_uint8),
        ("ntmFlags", ctypes.c_uint32),
        ("ntmSizeEM", ctypes.c_uint32),
        ("ntmCellHeight", ctypes.c_uint32),
        ("ntmAvgWidth", ctypes.c_uint32),
    ]
    tmHeight: int
    tmAscent: int
    tmDescent: int
    tmInternalLeading: int
    tmExternalLeading: int
    tmAveCharWidth: int
    tmMaxCharWidth: int
    tmWeight: int
    tmOverhang: int
    tmDigitizedAspectX: int
    tmDigitizedAspectY: int
    tmFirstChar: str
    tmLastChar: str
    tmDefaultChar: str
    tmBreakChar: str
    tmItalic: int
    tmUnderlined: int
    tmStruckOut: int
    tmPitchAndFamily: int
    tmCharSet: int
    ntmFlags: int
    ntmSizeEM: int
    ntmCellHeight: int
    ntmAvgWidth: int

class tagFONTSIGNATURE(ctypes.Structure):
    _fields_ = [
        ("fsUsb", ctypes.c_uint32 * 0x4),  # unsigned int[4]
        ("fsCsb", ctypes.c_uint32 * 0x2),  # unsigned int[2]
    ]
    fsUsb: list[ctypes.c_uint32]
    fsCsb: list[ctypes.c_uint32]

class tagAXISINFOW(ctypes.Structure):
    _fields_ = [
        ("axMinValue", ctypes.c_int32),
        ("axMaxValue", ctypes.c_int32),
        ("axAxisName", ctypes.c_wchar * 0x10),  # wchar_t[16]
    ]
    axMinValue: int
    axMaxValue: int
    axAxisName: list[ctypes.c_wchar]

class _onexit_table_t(ctypes.Structure):
    _fields_ = [
        ("_first", ctypes.c_ulonglong),
        ("_last", ctypes.c_ulonglong),
        ("_end", ctypes.c_ulonglong),
    ]
    _first: int
    _last: int
    _end: int

class __m256d(ctypes.Structure):
    _fields_ = [
        ("m256d_f64", ctypes.c_double * 0x4),  # long double[4]
    ]
    m256d_f64: list[ctypes.c_double]

class SystemSeed(ctypes.Structure):
    _fields_ = [
        ("mu64Hash1", ctypes.c_uint64),
        ("mu64Hash2", ctypes.c_uint64),
    ]
    mu64Hash1: int
    mu64Hash2: int

class _EXCEPTION_RECORD32(ctypes.Structure):
    _fields_ = [
        ("ExceptionCode", ctypes.c_uint32),
        ("ExceptionFlags", ctypes.c_uint32),
        ("ExceptionRecord", ctypes.c_uint32),
        ("ExceptionAddress", ctypes.c_uint32),
        ("NumberParameters", ctypes.c_uint32),
        ("ExceptionInformation", ctypes.c_uint32 * 0xF),  # unsigned int[15]
    ]
    ExceptionCode: int
    ExceptionFlags: int
    ExceptionRecord: int
    ExceptionAddress: int
    NumberParameters: int
    ExceptionInformation: list[ctypes.c_uint32]

class _SID_AND_ATTRIBUTES(ctypes.Structure):
    _fields_ = [
        ("Sid", ctypes.c_void_p),
        ("Attributes", ctypes.c_uint32),
    ]
    Sid: int
    Attributes: int

class _SE_TOKEN_USER(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x10),  # $2967D23A3360581BD76D8DAFB17280B0
        ("___u1", ctypes.c_char * 0x44),  # $74C4BDBCD496883F7EF9164C132E671F
    ]
    ___u0: bytes
    ___u1: bytes

class PPM_PERFSTATE_DOMAIN_EVENT(ctypes.Structure):
    _fields_ = [
        ("State", ctypes.c_uint32),
        ("Latency", ctypes.c_uint32),
        ("Speed", ctypes.c_uint32),
        ("Processors", ctypes.c_uint64),
    ]
    State: int
    Latency: int
    Speed: int
    Processors: int

class _IMAGE_IMPORT_BY_NAME(ctypes.Structure):
    _fields_ = [
        ("Hint", ctypes.c_uint16),
        ("Name", ctypes.c_char * 0x1),  # char[1]
    ]
    Hint: int
    Name: list[ctypes.c_char]

class _NON_PAGED_DEBUG_INFO(ctypes.Structure):
    _fields_ = [
        ("Signature", ctypes.c_uint16),
        ("Flags", ctypes.c_uint16),
        ("Size", ctypes.c_uint32),
        ("Machine", ctypes.c_uint16),
        ("Characteristics", ctypes.c_uint16),
        ("TimeDateStamp", ctypes.c_uint32),
        ("CheckSum", ctypes.c_uint32),
        ("SizeOfImage", ctypes.c_uint32),
        ("ImageBase", ctypes.c_uint64),
    ]
    Signature: int
    Flags: int
    Size: int
    Machine: int
    Characteristics: int
    TimeDateStamp: int
    CheckSum: int
    SizeOfImage: int
    ImageBase: int

class tagRAWMOUSE(ctypes.Structure):
    _fields_ = [
        ("usFlags", ctypes.c_uint16),
        ("___u1", ctypes.c_char * 0x4),  # $9855217E680C491AE60FA6D35C8C84D6
        ("ulRawButtons", ctypes.c_uint32),
        ("lLastX", ctypes.c_int32),
        ("lLastY", ctypes.c_int32),
        ("ulExtraInformation", ctypes.c_uint32),
    ]
    usFlags: int
    ___u1: bytes
    ulRawButtons: int
    lLastX: int
    lLastY: int
    ulExtraInformation: int

class hkThreadLocalDataBase(ctypes.Structure):
    def hkThreadLocalDataBase(): pass

    _fields_ = [
        ("m_index", ctypes.c_uint32),
        ("m_next", ctypes.c_ulonglong),
    ]
    m_index: int
    m_next: int

class _FIXED(ctypes.Structure):
    _fields_ = [
        ("fract", ctypes.c_uint16),
        ("value", ctypes.c_int16),
    ]
    fract: int
    value: int

class _KEY_EVENT_RECORD(ctypes.Structure):
    _fields_ = [
        ("bKeyDown", ctypes.c_int32),
        ("wRepeatCount", ctypes.c_uint16),
        ("wVirtualKeyCode", ctypes.c_uint16),
        ("wVirtualScanCode", ctypes.c_uint16),
        ("uChar", ctypes.c_char * 0x2),  # _KEY_EVENT_RECORD::<unnamed_type_uChar>
        ("dwControlKeyState", ctypes.c_uint32),
    ]
    bKeyDown: int
    wRepeatCount: int
    wVirtualKeyCode: int
    wVirtualScanCode: int
    uChar: bytes
    dwControlKeyState: int

class _TOKEN_LINKED_TOKEN(ctypes.Structure):
    _fields_ = [
        ("LinkedToken", ctypes.c_void_p),
    ]
    LinkedToken: int

class _POWER_PLATFORM_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("AoAc", ctypes.c_uint8),
    ]
    AoAc: int

class _IMAGE_SWITCHTABLE_BRANCH_DYNAMIC_RELOCATION(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int16),
    ]
    _bf_0: int

class tagIMEMENUITEMINFOW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fType", ctypes.c_uint32),
        ("fState", ctypes.c_uint32),
        ("wID", ctypes.c_uint32),
        ("hbmpChecked", ctypes.c_ulonglong),
        ("hbmpUnchecked", ctypes.c_ulonglong),
        ("dwItemData", ctypes.c_uint32),
        ("szString", ctypes.c_wchar * 0x50),  # wchar_t[80]
        ("hbmpItem", ctypes.c_ulonglong),
    ]
    cbSize: int
    fType: int
    fState: int
    wID: int
    hbmpChecked: int
    hbmpUnchecked: int
    dwItemData: int
    szString: list[ctypes.c_wchar]
    hbmpItem: int

class PPM_IDLE_STATE_ACCOUNTING(ctypes.Structure):
    _fields_ = [
        ("IdleTransitions", ctypes.c_uint32),
        ("FailedTransitions", ctypes.c_uint32),
        ("InvalidBucketIndex", ctypes.c_uint32),
        ("TotalTime", ctypes.c_uint64),
        ("IdleTimeBuckets", ctypes.c_uint32 * 0x6),  # unsigned int[6]
    ]
    IdleTransitions: int
    FailedTransitions: int
    InvalidBucketIndex: int
    TotalTime: int
    IdleTimeBuckets: list[ctypes.c_uint32]

class DLGITEMTEMPLATE(ctypes.Structure):
    _fields_ = [
        ("style", ctypes.c_uint32),
        ("dwExtendedStyle", ctypes.c_uint32),
        ("x", ctypes.c_int16),
        ("y", ctypes.c_int16),
        ("_cx", ctypes.c_int16),
        ("cy", ctypes.c_int16),
        ("id", ctypes.c_uint16),
    ]
    style: int
    dwExtendedStyle: int
    x: int
    y: int
    _cx: int
    cy: int
    id: int

class _NETINFOSTRUCT(ctypes.Structure):
    _fields_ = [
        ("cbStructure", ctypes.c_uint32),
        ("dwProviderVersion", ctypes.c_uint32),
        ("dwStatus", ctypes.c_uint32),
        ("dwCharacteristics", ctypes.c_uint32),
        ("dwHandle", ctypes.c_uint64),
        ("wNetType", ctypes.c_uint16),
        ("dwPrinters", ctypes.c_uint32),
        ("dwDrives", ctypes.c_uint32),
    ]
    cbStructure: int
    dwProviderVersion: int
    dwStatus: int
    dwCharacteristics: int
    dwHandle: int
    wNetType: int
    dwPrinters: int
    dwDrives: int

class SC_HANDLE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class tagWNDCLASSA(ctypes.Structure):
    _fields_ = [
        ("style", ctypes.c_uint32),
        ("lpfnWndProc", ctypes.c_ulonglong),
        ("cbClsExtra", ctypes.c_int32),
        ("cbWndExtra", ctypes.c_int32),
        ("hInstance", ctypes.c_ulonglong),
        ("hIcon", ctypes.c_ulonglong),
        ("hCursor", ctypes.c_ulonglong),
        ("hbrBackground", ctypes.c_ulonglong),
        ("lpszMenuName", ctypes.c_char_p),
        ("lpszClassName", ctypes.c_char_p),
    ]
    style: int
    lpfnWndProc: int
    cbClsExtra: int
    cbWndExtra: int
    hInstance: int
    hIcon: int
    hCursor: int
    hbrBackground: int
    lpszMenuName: bytes
    lpszClassName: bytes

class _TOKEN_PRIMARY_GROUP(ctypes.Structure):
    _fields_ = [
        ("PrimaryGroup", ctypes.c_void_p),
    ]
    PrimaryGroup: int

class _ACTIVATION_CONTEXT_BASIC_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("hActCtx", ctypes.c_void_p),
        ("dwFlags", ctypes.c_uint32),
    ]
    hActCtx: int
    dwFlags: int

class tagCREATESTRUCTA(ctypes.Structure):
    _fields_ = [
        ("lpCreateParams", ctypes.c_void_p),
        ("hInstance", ctypes.c_ulonglong),
        ("hMenu", ctypes.c_ulonglong),
        ("hwndParent", ctypes.c_ulonglong),
        ("cy", ctypes.c_int32),
        ("_cx", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("x", ctypes.c_int32),
        ("style", ctypes.c_int32),
        ("lpszName", ctypes.c_char_p),
        ("lpszClass", ctypes.c_char_p),
        ("dwExStyle", ctypes.c_uint32),
    ]
    lpCreateParams: int
    hInstance: int
    hMenu: int
    hwndParent: int
    cy: int
    _cx: int
    y: int
    x: int
    style: int
    lpszName: bytes
    lpszClass: bytes
    dwExStyle: int

class tagCBT_CREATEWNDA(ctypes.Structure):
    _fields_ = [
        ("lpcs", ctypes.c_ulonglong),
        ("hwndInsertAfter", ctypes.c_ulonglong),
    ]
    lpcs: int
    hwndInsertAfter: int

class _LDT_ENTRY(ctypes.Structure):
    _fields_ = [
        ("LimitLow", ctypes.c_uint16),
        ("BaseLow", ctypes.c_uint16),
        ("HighWord", ctypes.c_char * 0x4),  # _LDT_ENTRY::<unnamed_type_HighWord>
    ]
    LimitLow: int
    BaseLow: int
    HighWord: bytes

class _IMAGE_FUNCTION_ENTRY64(ctypes.Structure):
    _fields_ = [
        ("StartingAddress", ctypes.c_uint64),
        ("EndingAddress", ctypes.c_uint64),
        ("___u2", ctypes.c_char * 0x8),  # $E0927F6BE2F1645595F211B37EFD8766
    ]
    StartingAddress: int
    EndingAddress: int
    ___u2: bytes

class _COORD(ctypes.Structure):
    _fields_ = [
        ("X", ctypes.c_int16),
        ("Y", ctypes.c_int16),
    ]
    X: int
    Y: int

class _SMALL_RECT(ctypes.Structure):
    _fields_ = [
        ("Left", ctypes.c_int16),
        ("Top", ctypes.c_int16),
        ("Right", ctypes.c_int16),
        ("Bottom", ctypes.c_int16),
    ]
    Left: int
    Top: int
    Right: int
    Bottom: int

class _ENCLAVE_INIT_INFO_VBS(ctypes.Structure):
    _fields_ = [
        ("Length", ctypes.c_uint32),
        ("ThreadCount", ctypes.c_uint32),
    ]
    Length: int
    ThreadCount: int

class cGcAlienPodAnimParams(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def WriteToXMLNode(): pass

    _fields_ = [
        ("mfIntensity", ctypes.c_float),
    ]
    mfIntensity: float

class cGcThirdPersonAnimParams(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mVelocity", common.Vector2f),
        ("mVelocityXY", common.Vector2f),
        ("mfVelocityY", ctypes.c_float),
        ("mfVelocityZ", ctypes.c_float),
        ("mMoveForce", common.Vector2f),
        ("mfMoveForceApplied", ctypes.c_float),
        ("mAimDirection", common.Vector2f),
        ("mfSpeed", ctypes.c_float),
        ("mfFoot", ctypes.c_float),
        ("mfHitLR", ctypes.c_float),
        ("mfHitFB", ctypes.c_float),
        ("mfLeanLR", ctypes.c_float),
        ("mfLeanFB", ctypes.c_float),
        ("mfAimPitch", ctypes.c_float),
        ("mfAimYaw", ctypes.c_float),
        ("mfTurnAngle", ctypes.c_float),
        ("mfDistanceFromGround", ctypes.c_float),
        ("mfUphill", ctypes.c_float),
        ("mfSlopeAngle", ctypes.c_float),
        ("mfTimeSinceJetpackEngaged", ctypes.c_float),
    ]
    mVelocity: common.Vector2f
    mVelocityXY: common.Vector2f
    mfVelocityY: float
    mfVelocityZ: float
    mMoveForce: common.Vector2f
    mfMoveForceApplied: float
    mAimDirection: common.Vector2f
    mfSpeed: float
    mfFoot: float
    mfHitLR: float
    mfHitFB: float
    mfLeanLR: float
    mfLeanFB: float
    mfAimPitch: float
    mfAimYaw: float
    mfTurnAngle: float
    mfDistanceFromGround: float
    mfUphill: float
    mfSlopeAngle: float
    mfTimeSinceJetpackEngaged: float

class cGcAtlasActiveFeaturedBaseHeader(ctypes.Structure):
    def ClassPointerCompare(): pass

    def ClassPointerCopy(): pass

    def ClassPointerCreate(): pass

    def ClassPointerCreateDefault(): pass

    def ClassPointerDestroy(): pass

    def ClassPointerGenerateHash(): pass

    def ClassPointerRead(): pass

    def ClassPointerSave(): pass

    def ClassPointerWrite(): pass

    def GenerateHash(): pass

    def ReadFromXMLNode(): pass

    def SetDefaults(): pass

    def WriteToXMLNode(): pass

    def WriteToXMLNodeExplicit(): pass

    _fields_ = [
        ("mbDev", ctypes.c_ubyte),
        ("mbQA", ctypes.c_ubyte),
        ("mbProd", ctypes.c_ubyte),
        ("macPlatform", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("macUserId", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macUserName", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macBaseName", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macUA", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("macBaseId", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    mbDev: bool
    mbQA: bool
    mbProd: bool
    macPlatform: bytes
    macUserId: bytes
    macUserName: bytes
    macBaseName: bytes
    macUA: bytes
    macBaseId: bytes

class hkMemoryTrack(ctypes.Structure):
    _fields_ = [
        ("m_numBytesPerSector", ctypes.c_uint64),
        ("m_numBytesLastSector", ctypes.c_uint64),
        ("m_numBytesRead", ctypes.c_uint64),
        ("m_numSectorsUnloaded", ctypes.c_int32),
        ("m_sectors", ctypes.c_char * 0x10),  # hkArray<unsigned char *,hkBuiltinContainerAllocator<0,0> >
    ]
    m_numBytesPerSector: int
    m_numBytesLastSector: int
    m_numBytesRead: int
    m_numSectorsUnloaded: int
    m_sectors: bytes

class cTkSystemBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkSystemBase_dtor_0", ctypes.c_ulonglong),
        ("GetSKUID", ctypes.c_char * 0x8),  # cTkFixedString<32,char> *(__fastcall *)(cTkSystemBase *this)
        ("SetSKUID", ctypes.c_ulonglong),
        ("ConstructHardware", ctypes.c_ulonglong),
        ("ConstructVideo", ctypes.c_ulonglong),
        ("DestructHardware", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("DetectAvailableResolutions", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkSystemBase *this, int, std::vector<cTkIntTuple2<int,0>,TkSTLAllocatorShim<cTkIntTuple2<int,0>,4,-1> > *)
        ("DetectAvailableMonitors", ctypes.c_ulonglong),
        ("SwapBuffers", ctypes.c_ulonglong),
        ("GetScreenPosition", ctypes.c_char * 0x8),  # cTkIntTuple2<int,0> *(__fastcall *)(cTkSystemBase *this, cTkIntTuple2<int,0> *result)
        ("GetScreenResolution", ctypes.c_char * 0x8),  # cTkIntTuple2<int,0> *(__fastcall *)(cTkSystemBase *this, cTkIntTuple2<int,0> *result)
        ("GetWindowMode", ctypes.c_ulonglong),
        ("SetWindowMode", ctypes.c_ulonglong),
        ("SetScreenDimensions", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkSystemBase *this, const cTkIntTuple2<int,0> *, const cTkIntTuple2<int,0> *)
        ("GetResourceRemoveFrameDelay", ctypes.c_ulonglong),
        ("GameHasFocus", ctypes.c_ulonglong),
        ("GenerateSystemSeed", ctypes.c_ulonglong),
        ("GenerateUUIDString", ctypes.c_ulonglong),
        ("CopyTextToClipboard", ctypes.c_ulonglong),
        ("PasteTextFromClipboard", ctypes.c_ulonglong),
        ("SetGamma", ctypes.c_ulonglong),
        ("GetAppId", ctypes.c_ulonglong),
        ("IsDevAppId", ctypes.c_ulonglong),
        ("IsRetailAppId", ctypes.c_ulonglong),
        ("SystemSetCursor", ctypes.c_ulonglong),
        ("HandleSystemUIOverlayActivate", ctypes.c_ulonglong),
        ("HandleSystemUIOverlayDeactivate", ctypes.c_ulonglong),
        ("HandleSystemShareMenuOpened", ctypes.c_ulonglong),
        ("HandleMoveToBackgroundExecution", ctypes.c_ulonglong),
        ("HandleReturnFromBackgroundExecution", ctypes.c_ulonglong),
    ]
    cTkSystemBase_dtor_0: int
    GetSKUID: bytes
    SetSKUID: int
    ConstructHardware: int
    ConstructVideo: int
    DestructHardware: int
    Update: int
    DetectAvailableResolutions: bytes
    DetectAvailableMonitors: int
    SwapBuffers: int
    GetScreenPosition: bytes
    GetScreenResolution: bytes
    GetWindowMode: int
    SetWindowMode: int
    SetScreenDimensions: bytes
    GetResourceRemoveFrameDelay: int
    GameHasFocus: int
    GenerateSystemSeed: int
    GenerateUUIDString: int
    CopyTextToClipboard: int
    PasteTextFromClipboard: int
    SetGamma: int
    GetAppId: int
    IsDevAppId: int
    IsRetailAppId: int
    SystemSetCursor: int
    HandleSystemUIOverlayActivate: int
    HandleSystemUIOverlayDeactivate: int
    HandleSystemShareMenuOpened: int
    HandleMoveToBackgroundExecution: int
    HandleReturnFromBackgroundExecution: int

class cTkUserService_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkUserService_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RequestAuthenticationToken", ctypes.c_ulonglong),
        ("TryGetUserIdentity", ctypes.c_ulonglong),
        ("CheckUserPresence", ctypes.c_ulonglong),
        ("GetBranchName", ctypes.c_ulonglong),
        ("GetBuildNumber", ctypes.c_ulonglong),
        ("TryReconnectionIfAvailable", ctypes.c_ulonglong),
        ("GetReconnectionState", ctypes.c_ulonglong),
        ("GetCountryCode", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedNames", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedContent", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedContentUserSpecific", ctypes.c_ulonglong),
        ("AllowedToShareUserGeneratedContent", ctypes.c_ulonglong),
        ("AllowedMultiplayer", ctypes.c_ulonglong),
        ("AllowedCrossplay", ctypes.c_ulonglong),
        ("AllowedToChat", ctypes.c_ulonglong),
        ("AllowedToChatUserSpecific", ctypes.c_ulonglong),
        ("AllowUnfilteredFriendTextChat", ctypes.c_ulonglong),
        ("IsDisplayingSystemDialogs", ctypes.c_ulonglong),
        ("StartUsingUGC", ctypes.c_ulonglong),
        ("StopUsingUGC", ctypes.c_ulonglong),
        ("RequestUserSelection", ctypes.c_ulonglong),
        ("IsUserSelecting", ctypes.c_ulonglong),
        ("RequestUserGamercard", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkUserService *this, const cTkUserIdBase<unsigned __int64>)
        ("GetPlatformId", ctypes.c_ulonglong),
        ("RequiresUserNameLookup", ctypes.c_ulonglong),
        ("GetNameRequestBatchSize", ctypes.c_ulonglong),
        ("DoUserIdBatchLookup", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkUserService *this, std::vector<cTkFixedString<64,char>,TkSTLAllocatorShim<cTkFixedString<64,char>,1,-1> > *, std::function<void __cdecl(std::vector<std::tuple<cTkFixedString<64,char>,cTkFixedString<64,char> >,TkSTLAllocatorShim<std::tuple<cTkFixedString<64,char>,cTkFixedString<64,char> >,1,-1> > &)>)
    ]
    cTkUserService_dtor_0: int
    Construct: int
    Update: int
    Destruct: int
    RequestAuthenticationToken: int
    TryGetUserIdentity: int
    CheckUserPresence: int
    GetBranchName: int
    GetBuildNumber: int
    TryReconnectionIfAvailable: int
    GetReconnectionState: int
    GetCountryCode: int
    AllowedToDisplayUserGeneratedNames: int
    AllowedToDisplayUserGeneratedContent: int
    AllowedToDisplayUserGeneratedContentUserSpecific: int
    AllowedToShareUserGeneratedContent: int
    AllowedMultiplayer: int
    AllowedCrossplay: int
    AllowedToChat: int
    AllowedToChatUserSpecific: int
    AllowUnfilteredFriendTextChat: int
    IsDisplayingSystemDialogs: int
    StartUsingUGC: int
    StopUsingUGC: int
    RequestUserSelection: int
    IsUserSelecting: int
    RequestUserGamercard: bytes
    GetPlatformId: int
    RequiresUserNameLookup: int
    GetNameRequestBatchSize: int
    DoUserIdBatchLookup: bytes

class ITkUserServiceNotifications_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkUserServiceNotifications_dtor_0", ctypes.c_ulonglong),
        ("AuthenticationTokenArrived", ctypes.c_ulonglong),
        ("AuthenticationTokenFailed", ctypes.c_ulonglong),
        ("ServiceStateChanged", ctypes.c_ulonglong),
    ]
    ITkUserServiceNotifications_dtor_0: int
    AuthenticationTokenArrived: int
    AuthenticationTokenFailed: int
    ServiceStateChanged: int

class cTkProfanityFilter(ctypes.Structure):
    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkProfanityFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("AvailableForUse", ctypes.c_ulonglong),
        ("IsThisNaughty", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkProfanityFilter *this, const char *, std::function<void __cdecl(bool,char const *)>)
        ("IsThisNaughty_28", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkProfanityFilter *this, const char *, std::function<void __cdecl(bool)>)
        ("cTkProfanityFilter_dtor_30", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    Destruct: int
    AvailableForUse: int
    IsThisNaughty: bytes
    IsThisNaughty_28: bytes
    cTkProfanityFilter_dtor_30: int

class CommonStorageUserCred(ctypes.Structure):
    _fields_ = [
        ("mu64UserID", ctypes.c_uint64),
        ("mbValidUserID", ctypes.c_ubyte),
    ]
    mu64UserID: int
    mbValidUserID: bool

class ManifestData(ctypes.Structure):
    _fields_ = [
        ("muMagic", ctypes.c_uint32),
        ("muVersion", ctypes.c_uint32),
        ("mu64HashSpooky1", ctypes.c_uint64),
        ("mu64HashSpooky2", ctypes.c_uint64),
        ("maHashSHA", ctypes.c_char * 0x20),  # std::array<unsigned char,32>
        ("miOriginalDataSize", ctypes.c_int32),
        ("miCompressedDataSize", ctypes.c_int32),
        ("muUsedUserIDInObfuscation", ctypes.c_uint32),
        ("mGameMetaData", ctypes.c_char * 0x118),  # char[280]
        ("maReserved", ctypes.c_char * 0xC),  # std::array<unsigned int,3>
    ]
    muMagic: int
    muVersion: int
    mu64HashSpooky1: int
    mu64HashSpooky2: int
    maHashSHA: bytes
    miOriginalDataSize: int
    miCompressedDataSize: int
    muUsedUserIDInObfuscation: int
    mGameMetaData: list[ctypes.c_char]
    maReserved: bytes

class cTkSaveDataStreamBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("PlatformReadFile", ctypes.c_ulonglong),
        ("PlatformWriteFile", ctypes.c_ulonglong),
    ]
    PlatformReadFile: int
    PlatformWriteFile: int

class cTkSpinMutex(ctypes.Structure):
    def Acquire(): pass

    _fields_ = [
        ("miLocked", ctypes.c_char * 0x4),  # std::atomic<int>
    ]
    miLocked: bytes

class hkErrStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkOstream_dtor_0", ctypes.c_ulonglong),
    ]
    hkOstream_dtor_0: int

class cGcAtlasRecvActiveFeaturedBasesQuery_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class hkMemoryTrackStreamWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("write", ctypes.c_ulonglong),
        ("flush", ctypes.c_ulonglong),
        ("seekTellSupported", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
        ("isBuffered", ctypes.c_ulonglong),
        ("clear", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    write: int
    flush: int
    seekTellSupported: int
    seek: int
    tell: int
    isBuffered: int
    clear: int

class tagCIEXYZ(ctypes.Structure):
    _fields_ = [
        ("ciexyzX", ctypes.c_int32),
        ("ciexyzY", ctypes.c_int32),
        ("ciexyzZ", ctypes.c_int32),
    ]
    ciexyzX: int
    ciexyzY: int
    ciexyzZ: int

class tagUSAGE_PROPERTIES(ctypes.Structure):
    _fields_ = [
        ("level", ctypes.c_uint16),
        ("page", ctypes.c_uint16),
        ("usage", ctypes.c_uint16),
        ("logicalMinimum", ctypes.c_int32),
        ("logicalMaximum", ctypes.c_int32),
        ("unit", ctypes.c_uint16),
        ("exponent", ctypes.c_uint16),
        ("count", ctypes.c_uint8),
        ("physicalMinimum", ctypes.c_int32),
        ("physicalMaximum", ctypes.c_int32),
    ]
    level: int
    page: int
    usage: int
    logicalMinimum: int
    logicalMaximum: int
    unit: int
    exponent: int
    count: int
    physicalMinimum: int
    physicalMaximum: int

class _PROCESSOR_GROUP_INFO(ctypes.Structure):
    _fields_ = [
        ("MaximumProcessorCount", ctypes.c_uint8),
        ("ActiveProcessorCount", ctypes.c_uint8),
        ("Reserved", ctypes.c_uint8 * 0x26),  # unsigned __int8[38]
        ("ActiveProcessorMask", ctypes.c_uint64),
    ]
    MaximumProcessorCount: int
    ActiveProcessorCount: int
    Reserved: list[ctypes.c_uint8]
    ActiveProcessorMask: int

class tagCREATESTRUCTW(ctypes.Structure):
    _fields_ = [
        ("lpCreateParams", ctypes.c_void_p),
        ("hInstance", ctypes.c_ulonglong),
        ("hMenu", ctypes.c_ulonglong),
        ("hwndParent", ctypes.c_ulonglong),
        ("cy", ctypes.c_int32),
        ("_cx", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("x", ctypes.c_int32),
        ("style", ctypes.c_int32),
        ("lpszName", ctypes.c_ulonglong),
        ("lpszClass", ctypes.c_ulonglong),
        ("dwExStyle", ctypes.c_uint32),
    ]
    lpCreateParams: int
    hInstance: int
    hMenu: int
    hwndParent: int
    cy: int
    _cx: int
    y: int
    x: int
    style: int
    lpszName: int
    lpszClass: int
    dwExStyle: int

class hkMemoryTrackStreamReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("skip", ctypes.c_ulonglong),
        ("peek", ctypes.c_ulonglong),
        ("isSeekTellSupported", ctypes.c_ulonglong),
        ("isInplace", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    read: int
    skip: int
    peek: int
    isSeekTellSupported: int
    isInplace: int

class tagCBT_CREATEWNDW(ctypes.Structure):
    _fields_ = [
        ("lpcs", ctypes.c_ulonglong),
        ("hwndInsertAfter", ctypes.c_ulonglong),
    ]
    lpcs: int
    hwndInsertAfter: int

class tagLOGBRUSH32(ctypes.Structure):
    _fields_ = [
        ("lbStyle", ctypes.c_uint32),
        ("lbColor", ctypes.c_uint32),
        ("lbHatch", ctypes.c_uint32),
    ]
    lbStyle: int
    lbColor: int
    lbHatch: int

class cGcAtlasRecvBaseImages_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasRecvBaseList_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class hkBufferedStreamWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("write", ctypes.c_ulonglong),
        ("flush", ctypes.c_ulonglong),
        ("seekTellSupported", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
        ("isBuffered", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    write: int
    flush: int
    seekTellSupported: int
    seek: int
    tell: int
    isBuffered: int

class cGcAtlasRecvBaseQueryList_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasRecvBasic_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasRecvBlob_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class cGcAtlasRecvDiscoveryExact_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class hkMT19937RandomGenerator(ctypes.Structure):
    def hkMT19937RandomGenerator(): pass

    def tilde_hkMT19937RandomGenerator(): pass

    def initialize(): pass

    _fields_ = [
        ("m_index", ctypes.c_uint16),
        ("m_state", ctypes.c_ulonglong),
    ]
    m_index: int
    m_state: int

class cGcAtlasRecvDiscoveryList_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class __std_exception_data(ctypes.Structure):
    _fields_ = [
        ("_What", ctypes.c_char_p),
        ("_DoFree", ctypes.c_ubyte),
    ]
    _What: bytes
    _DoFree: bool

class hkBlockStreamAllocator_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("blockAllocBatch", ctypes.c_ulonglong),
        ("blockFreeBatch", ctypes.c_ulonglong),
        ("getBytesUsed", ctypes.c_ulonglong),
        ("getMaxBytesUsed", ctypes.c_ulonglong),
        ("freeAllRemainingAllocations", ctypes.c_ulonglong),
        ("getMemoryStatistics", ctypes.c_ulonglong),
        ("getNumThreadLocalBlocks", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    blockAllocBatch: int
    blockFreeBatch: int
    getBytesUsed: int
    getMaxBytesUsed: int
    freeAllRemainingAllocations: int
    getMemoryStatistics: int
    getNumThreadLocalBlocks: int

class cGcAtlasRecvFeaturedBasesQueryList_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasShared_dtor_0", ctypes.c_ulonglong),
    ]
    cGcAtlasShared_dtor_0: int

class hkLocalFrameCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("addFrame", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    addFrame: int

class hkLocalFrame_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getLocalTransform", ctypes.c_ulonglong),
        ("setLocalTransform", ctypes.c_ulonglong),
        ("getLocalPosition", ctypes.c_ulonglong),
        ("getNearbyFrames", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getParentFrame", ctypes.c_ulonglong),
        ("setParentFrame", ctypes.c_ulonglong),
        ("getNumChildFrames", ctypes.c_ulonglong),
        ("getChildFrame", ctypes.c_ulonglong),
        ("getGroup", ctypes.c_ulonglong),
        ("setGroup", ctypes.c_ulonglong),
        ("getDescendants", ctypes.c_char * 0x8),  # void (__fastcall *)(hkLocalFrame *this, hkArrayBase<hkLocalFrame const *> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getLocalTransform: int
    setLocalTransform: int
    getLocalPosition: int
    getNearbyFrames: int
    getName: int
    getParentFrame: int
    setParentFrame: int
    getNumChildFrames: int
    getChildFrame: int
    getGroup: int
    setGroup: int
    getDescendants: bytes

class hkSimpleLocalFrame_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getLocalTransform", ctypes.c_ulonglong),
        ("setLocalTransform", ctypes.c_ulonglong),
        ("getLocalPosition", ctypes.c_ulonglong),
        ("getNearbyFrames", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getParentFrame", ctypes.c_ulonglong),
        ("setParentFrame", ctypes.c_ulonglong),
        ("getNumChildFrames", ctypes.c_ulonglong),
        ("getChildFrame", ctypes.c_ulonglong),
        ("getGroup", ctypes.c_ulonglong),
        ("setGroup", ctypes.c_ulonglong),
        ("getDescendants", ctypes.c_char * 0x8),  # void (__fastcall *)(hkLocalFrame *this, hkArrayBase<hkLocalFrame const *> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getLocalTransform: int
    setLocalTransform: int
    getLocalPosition: int
    getNearbyFrames: int
    getName: int
    getParentFrame: int
    setParentFrame: int
    getNumChildFrames: int
    getChildFrame: int
    getGroup: int
    setGroup: int
    getDescendants: bytes

class _SECURITY_ATTRIBUTES(ctypes.Structure):
    _fields_ = [
        ("nLength", ctypes.c_uint32),
        ("lpSecurityDescriptor", ctypes.c_void_p),
        ("bInheritHandle", ctypes.c_int32),
    ]
    nLength: int
    lpSecurityDescriptor: int
    bInheritHandle: int

class tagTHREADNAME_INFO(ctypes.Structure):
    _fields_ = [
        ("dwType", ctypes.c_uint32),
        ("szName", ctypes.c_char_p),
        ("dwThreadID", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
    ]
    dwType: int
    szName: bytes
    dwThreadID: int
    dwFlags: int

class hkIstream(ctypes.Structure):
    _fields_ = [
        ("m_streamReader", ctypes.c_char * 0x8),  # hkRefPtr<hkStreamReader>
    ]
    m_streamReader: bytes

class hkUuidPseudoRandomGenerator_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkWin32StreamWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("write", ctypes.c_ulonglong),
        ("flush", ctypes.c_ulonglong),
        ("seekTellSupported", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
        ("isBuffered", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    write: int
    flush: int
    seekTellSupported: int
    seek: int
    tell: int
    isBuffered: int

class _FILE_NOTIFY_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("NextEntryOffset", ctypes.c_uint32),
        ("Action", ctypes.c_uint32),
        ("FileNameLength", ctypes.c_uint32),
        ("FileName", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    NextEntryOffset: int
    Action: int
    FileNameLength: int
    FileName: list[ctypes.c_wchar]

class hkWin32FileSystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("remove", ctypes.c_ulonglong),
        ("moveFile", ctypes.c_ulonglong),
        ("mkdir", ctypes.c_ulonglong),
        ("mkdirRecursive", ctypes.c_ulonglong),
        ("copy", ctypes.c_ulonglong),
        ("stat", ctypes.c_ulonglong),
        ("getOperatingSystemPath", ctypes.c_ulonglong),
        ("isPathAbsolute", ctypes.c_ulonglong),
        ("createWatcher", ctypes.c_char * 0x8),  # hkRefNew<hkFileSystem::Watcher> *(__fastcall *)(hkFileSystem *this, hkRefNew<hkFileSystem::Watcher> *result, const char *)
        ("createIterator", ctypes.c_char * 0x8),  # hkRefNew<hkFileSystem::Iterator::Impl> *(__fastcall *)(hkFileSystem *this, hkRefNew<hkFileSystem::Iterator::Impl> *result, const char *, const char *)
        ("_openReader", ctypes.c_char * 0x8),  # hkRefNew<hkStreamReader> *(__fastcall *)(hkFileSystem *this, hkRefNew<hkStreamReader> *result, const char *, hkFileSystem::OpenFlags)
        ("_openWriter", ctypes.c_char * 0x8),  # hkRefNew<hkStreamWriter> *(__fastcall *)(hkFileSystem *this, hkRefNew<hkStreamWriter> *result, const char *, hkFileSystem::OpenFlags)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    remove: int
    moveFile: int
    mkdir: int
    mkdirRecursive: int
    copy: int
    stat: int
    getOperatingSystemPath: int
    isPathAbsolute: int
    createWatcher: bytes
    createIterator: bytes
    _openReader: bytes
    _openWriter: bytes

class hkSecondaryCommandDispatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("exec", ctypes.c_ulonglong),
        ("print", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    exec: int
    print: int

class _SYSTEM_INFO(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $33FC56CADE5C54C0AD7450818F814469
        ("dwPageSize", ctypes.c_uint32),
        ("lpMinimumApplicationAddress", ctypes.c_void_p),
        ("lpMaximumApplicationAddress", ctypes.c_void_p),
        ("dwActiveProcessorMask", ctypes.c_uint64),
        ("dwNumberOfProcessors", ctypes.c_uint32),
        ("dwProcessorType", ctypes.c_uint32),
        ("dwAllocationGranularity", ctypes.c_uint32),
        ("wProcessorLevel", ctypes.c_uint16),
        ("wProcessorRevision", ctypes.c_uint16),
    ]
    ___u0: bytes
    dwPageSize: int
    lpMinimumApplicationAddress: int
    lpMaximumApplicationAddress: int
    dwActiveProcessorMask: int
    dwNumberOfProcessors: int
    dwProcessorType: int
    dwAllocationGranularity: int
    wProcessorLevel: int
    wProcessorRevision: int

class hkSemaphore(ctypes.Structure):
    def hkSemaphore(): pass

    def tilde_hkSemaphore(): pass

    def release(): pass

    _fields_ = [
        ("m_semaphore", ctypes.c_void_p),
    ]
    m_semaphore: int

class hkAsyncThreadPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("stopThreads", ctypes.c_ulonglong),
        ("workerThreadMain", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    stopThreads: int
    workerThreadMain: int

class hkReferencedTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkTaskGraph_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("clear", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    clear: int

class hkTaskBatchProcessor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("prepare", ctypes.c_ulonglong),
        ("beginTask", ctypes.c_ulonglong),
        ("processBatch", ctypes.c_ulonglong),
        ("endTask", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    prepare: int
    beginTask: int
    processBatch: int
    endTask: int

class hkTaskQueue_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTaskQueue_dtor_8", ctypes.c_ulonglong),
        ("allocateHandles", ctypes.c_ulonglong),
        ("freeHandles", ctypes.c_ulonglong),
        ("initHandle", ctypes.c_ulonglong),
        ("setAffinity", ctypes.c_ulonglong),
        ("setMultiplicity", ctypes.c_ulonglong),
        ("addDependency", ctypes.c_ulonglong),
        ("addDependencies", ctypes.c_ulonglong),
        ("submitHandles", ctypes.c_ulonglong),
        ("abortHandles", ctypes.c_ulonglong),
        ("processUntilFinished", ctypes.c_ulonglong),
        ("processUntilFinished_60", ctypes.c_ulonglong),
        ("close", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("processOnce", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("isHandleFinished", ctypes.c_ulonglong),
        ("isFinished", ctypes.c_ulonglong),
        ("addAndSubmitTask", ctypes.c_ulonglong),
        ("processAndFree", ctypes.c_ulonglong),
        ("submitGraphAndWait", ctypes.c_ulonglong),
        ("forEach", ctypes.c_ulonglong),
        ("getNumThreadsHint", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTaskQueue_dtor_8: int
    allocateHandles: int
    freeHandles: int
    initHandle: int
    setAffinity: int
    setMultiplicity: int
    addDependency: int
    addDependencies: int
    submitHandles: int
    abortHandles: int
    processUntilFinished: int
    processUntilFinished_60: int
    close: int
    reset: int
    processOnce: int
    process: int
    isHandleFinished: int
    isFinished: int
    addAndSubmitTask: int
    processAndFree: int
    submitGraphAndWait: int
    forEach: int
    getNumThreadsHint: int

class hkTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class hkBlockStreamCommandWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("exec", ctypes.c_ulonglong),
        ("print", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    exec: int
    print: int

class hkPrimaryCommandDispatcher(ctypes.Structure):
    def hkPrimaryCommandDispatcher(): pass

    def tilde_hkPrimaryCommandDispatcher(): pass

    def vector_deleting_destructor(): pass

    def exec(): pass

    def registerDispatcher(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_commandDispatcher", ctypes.c_char * 0x30),  # hkRefPtr<hkSecondaryCommandDispatcher>
    ]
    __vftable: int
    m_commandDispatcher: bytes

class hkPrimaryCommandDispatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkPrimaryCommandDispatcher_dtor_0", ctypes.c_ulonglong),
    ]
    hkPrimaryCommandDispatcher_dtor_0: int

class hkRefTaskQueue_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkStringMapOperations(ctypes.Structure):
    pass

class _XSTATE_FEATURE(ctypes.Structure):
    _fields_ = [
        ("Offset", ctypes.c_uint32),
        ("Size", ctypes.c_uint32),
    ]
    Offset: int
    Size: int

class _IMAGE_EXPORT_DIRECTORY(ctypes.Structure):
    _fields_ = [
        ("Characteristics", ctypes.c_uint32),
        ("TimeDateStamp", ctypes.c_uint32),
        ("MajorVersion", ctypes.c_uint16),
        ("MinorVersion", ctypes.c_uint16),
        ("Name", ctypes.c_uint32),
        ("Base", ctypes.c_uint32),
        ("NumberOfFunctions", ctypes.c_uint32),
        ("NumberOfNames", ctypes.c_uint32),
        ("AddressOfFunctions", ctypes.c_uint32),
        ("AddressOfNames", ctypes.c_uint32),
        ("AddressOfNameOrdinals", ctypes.c_uint32),
    ]
    Characteristics: int
    TimeDateStamp: int
    MajorVersion: int
    MinorVersion: int
    Name: int
    Base: int
    NumberOfFunctions: int
    NumberOfNames: int
    AddressOfFunctions: int
    AddressOfNames: int
    AddressOfNameOrdinals: int

class _IMAGE_EPILOGUE_DYNAMIC_RELOCATION_HEADER(ctypes.Structure):
    _fields_ = [
        ("EpilogueCount", ctypes.c_uint32),
        ("EpilogueByteCount", ctypes.c_uint8),
        ("BranchDescriptorElementSize", ctypes.c_uint8),
        ("BranchDescriptorCount", ctypes.c_uint16),
    ]
    EpilogueCount: int
    EpilogueByteCount: int
    BranchDescriptorElementSize: int
    BranchDescriptorCount: int

class _CREATE_THREAD_DEBUG_INFO(ctypes.Structure):
    _fields_ = [
        ("hThread", ctypes.c_void_p),
        ("lpThreadLocalBase", ctypes.c_void_p),
        ("lpStartAddress", ctypes.c_ulonglong),
    ]
    hThread: int
    lpThreadLocalBase: int
    lpStartAddress: int

class HENHMETAFILE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _IMAGE_ARCHIVE_MEMBER_HEADER(ctypes.Structure):
    _fields_ = [
        ("Name", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("Date", ctypes.c_uint8 * 0xC),  # unsigned __int8[12]
        ("UserID", ctypes.c_uint8 * 0x6),  # unsigned __int8[6]
        ("GroupID", ctypes.c_uint8 * 0x6),  # unsigned __int8[6]
        ("Mode", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
        ("Size", ctypes.c_uint8 * 0xA),  # unsigned __int8[10]
        ("EndHeader", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
    ]
    Name: list[ctypes.c_uint8]
    Date: list[ctypes.c_uint8]
    UserID: list[ctypes.c_uint8]
    GroupID: list[ctypes.c_uint8]
    Mode: list[ctypes.c_uint8]
    Size: list[ctypes.c_uint8]
    EndHeader: list[ctypes.c_uint8]

class _IMAGE_DYNAMIC_RELOCATION64_V2(ctypes.Structure):
    _fields_ = [
        ("HeaderSize", ctypes.c_uint32),
        ("FixupInfoSize", ctypes.c_uint32),
        ("Symbol", ctypes.c_uint64),
        ("SymbolGroup", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
    ]
    HeaderSize: int
    FixupInfoSize: int
    Symbol: int
    SymbolGroup: int
    Flags: int

class _POWER_SESSION_CONNECT(ctypes.Structure):
    _fields_ = [
        ("Connected", ctypes.c_uint8),
        ("Console", ctypes.c_uint8),
    ]
    Connected: int
    Console: int

class tagENUMUILANG(ctypes.Structure):
    _fields_ = [
        ("NumOfEnumUILang", ctypes.c_uint32),
        ("SizeOfEnumUIBuffer", ctypes.c_uint32),
        ("pEnumUIBuffer", ctypes.c_ulonglong),
    ]
    NumOfEnumUILang: int
    SizeOfEnumUIBuffer: int
    pEnumUIBuffer: int

class _numberfmtA(ctypes.Structure):
    _fields_ = [
        ("NumDigits", ctypes.c_uint32),
        ("LeadingZero", ctypes.c_uint32),
        ("Grouping", ctypes.c_uint32),
        ("lpDecimalSep", ctypes.c_char_p),
        ("lpThousandSep", ctypes.c_char_p),
        ("NegativeOrder", ctypes.c_uint32),
    ]
    NumDigits: int
    LeadingZero: int
    Grouping: int
    lpDecimalSep: bytes
    lpThousandSep: bytes
    NegativeOrder: int

class _MEMORY_BASIC_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("BaseAddress", ctypes.c_void_p),
        ("AllocationBase", ctypes.c_void_p),
        ("AllocationProtect", ctypes.c_uint32),
        ("PartitionId", ctypes.c_uint16),
        ("RegionSize", ctypes.c_uint64),
        ("State", ctypes.c_uint32),
        ("Protect", ctypes.c_uint32),
        ("Type", ctypes.c_uint32),
    ]
    BaseAddress: int
    AllocationBase: int
    AllocationProtect: int
    PartitionId: int
    RegionSize: int
    State: int
    Protect: int
    Type: int

class _IMAGE_FILE_HEADER(ctypes.Structure):
    _fields_ = [
        ("Machine", ctypes.c_uint16),
        ("NumberOfSections", ctypes.c_uint16),
        ("TimeDateStamp", ctypes.c_uint32),
        ("PointerToSymbolTable", ctypes.c_uint32),
        ("NumberOfSymbols", ctypes.c_uint32),
        ("SizeOfOptionalHeader", ctypes.c_uint16),
        ("Characteristics", ctypes.c_uint16),
    ]
    Machine: int
    NumberOfSections: int
    TimeDateStamp: int
    PointerToSymbolTable: int
    NumberOfSymbols: int
    SizeOfOptionalHeader: int
    Characteristics: int

class hkIntUnion64: pass

class tagDEBUGHOOKINFO(ctypes.Structure):
    _fields_ = [
        ("idThread", ctypes.c_uint32),
        ("idThreadInstaller", ctypes.c_uint32),
        ("lParam", ctypes.c_int64),
        ("wParam", ctypes.c_uint64),
        ("code", ctypes.c_int32),
    ]
    idThread: int
    idThreadInstaller: int
    lParam: int
    wParam: int
    code: int

class _PSFEATURE_OUTPUT(ctypes.Structure):
    _fields_ = [
        ("bPageIndependent", ctypes.c_int32),
        ("bSetPageDevice", ctypes.c_int32),
    ]
    bPageIndependent: int
    bSetPageDevice: int

class hkOctaFloat32(ctypes.Structure):
    _fields_ = [
        ("v0123", common.Vector4f),
        ("v4567", common.Vector4f),
    ]
    v0123: common.Vector4f
    v4567: common.Vector4f

class hkQuadUcharUnion: pass

class hkIntVectorConstantU8_: pass

class tagPIXELFORMATDESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("nSize", ctypes.c_uint16),
        ("nVersion", ctypes.c_uint16),
        ("dwFlags", ctypes.c_uint32),
        ("iPixelType", ctypes.c_uint8),
        ("cColorBits", ctypes.c_uint8),
        ("cRedBits", ctypes.c_uint8),
        ("cRedShift", ctypes.c_uint8),
        ("cGreenBits", ctypes.c_uint8),
        ("cGreenShift", ctypes.c_uint8),
        ("cBlueBits", ctypes.c_uint8),
        ("cBlueShift", ctypes.c_uint8),
        ("cAlphaBits", ctypes.c_uint8),
        ("cAlphaShift", ctypes.c_uint8),
        ("cAccumBits", ctypes.c_uint8),
        ("cAccumRedBits", ctypes.c_uint8),
        ("cAccumGreenBits", ctypes.c_uint8),
        ("cAccumBlueBits", ctypes.c_uint8),
        ("cAccumAlphaBits", ctypes.c_uint8),
        ("cDepthBits", ctypes.c_uint8),
        ("cStencilBits", ctypes.c_uint8),
        ("cAuxBuffers", ctypes.c_uint8),
        ("iLayerType", ctypes.c_uint8),
        ("bReserved", ctypes.c_uint8),
        ("dwLayerMask", ctypes.c_uint32),
        ("dwVisibleMask", ctypes.c_uint32),
        ("dwDamageMask", ctypes.c_uint32),
    ]
    nSize: int
    nVersion: int
    dwFlags: int
    iPixelType: int
    cColorBits: int
    cRedBits: int
    cRedShift: int
    cGreenBits: int
    cGreenShift: int
    cBlueBits: int
    cBlueShift: int
    cAlphaBits: int
    cAlphaShift: int
    cAccumBits: int
    cAccumRedBits: int
    cAccumGreenBits: int
    cAccumBlueBits: int
    cAccumAlphaBits: int
    cDepthBits: int
    cStencilBits: int
    cAuxBuffers: int
    iLayerType: int
    bReserved: int
    dwLayerMask: int
    dwVisibleMask: int
    dwDamageMask: int

class _IMAGE_ENCLAVE_CONFIG32(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("MinimumRequiredConfigSize", ctypes.c_uint32),
        ("PolicyFlags", ctypes.c_uint32),
        ("NumberOfImports", ctypes.c_uint32),
        ("ImportList", ctypes.c_uint32),
        ("ImportEntrySize", ctypes.c_uint32),
        ("FamilyID", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("ImageID", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("ImageVersion", ctypes.c_uint32),
        ("SecurityVersion", ctypes.c_uint32),
        ("EnclaveSize", ctypes.c_uint32),
        ("NumberOfThreads", ctypes.c_uint32),
        ("EnclaveFlags", ctypes.c_uint32),
    ]
    Size: int
    MinimumRequiredConfigSize: int
    PolicyFlags: int
    NumberOfImports: int
    ImportList: int
    ImportEntrySize: int
    FamilyID: list[ctypes.c_uint8]
    ImageID: list[ctypes.c_uint8]
    ImageVersion: int
    SecurityVersion: int
    EnclaveSize: int
    NumberOfThreads: int
    EnclaveFlags: int

class _SCOPE_TABLE_ARM64(ctypes.Structure):
    _fields_ = [
        ("Count", ctypes.c_uint32),
        ("ScopeRecord", ctypes.c_char * 0x10),  # _SCOPE_TABLE_ARM64::<unnamed_type_ScopeRecord>
    ]
    Count: int
    ScopeRecord: bytes

class tagKBDLLHOOKSTRUCT(ctypes.Structure):
    _fields_ = [
        ("vkCode", ctypes.c_uint32),
        ("scanCode", ctypes.c_uint32),
        ("flags", ctypes.c_uint32),
        ("time", ctypes.c_uint32),
        ("dwExtraInfo", ctypes.c_uint64),
    ]
    vkCode: int
    scanCode: int
    flags: int
    time: int
    dwExtraInfo: int

class _POWER_SESSION_RIT_STATE(ctypes.Structure):
    _fields_ = [
        ("Active", ctypes.c_uint8),
        ("LastInputTime", ctypes.c_uint32),
    ]
    Active: int
    LastInputTime: int

class _TAPE_ERASE(ctypes.Structure):
    _fields_ = [
        ("Type", ctypes.c_uint32),
        ("Immediate", ctypes.c_uint8),
    ]
    Type: int
    Immediate: int

class _DISCDLGSTRUCTA(ctypes.Structure):
    _fields_ = [
        ("cbStructure", ctypes.c_uint32),
        ("hwndOwner", ctypes.c_ulonglong),
        ("lpLocalName", ctypes.c_char_p),
        ("lpRemoteName", ctypes.c_char_p),
        ("dwFlags", ctypes.c_uint32),
    ]
    cbStructure: int
    hwndOwner: int
    lpLocalName: bytes
    lpRemoteName: bytes
    dwFlags: int

class _NETRESOURCEW(ctypes.Structure):
    _fields_ = [
        ("dwScope", ctypes.c_uint32),
        ("dwType", ctypes.c_uint32),
        ("dwDisplayType", ctypes.c_uint32),
        ("dwUsage", ctypes.c_uint32),
        ("lpLocalName", ctypes.c_ulonglong),
        ("lpRemoteName", ctypes.c_ulonglong),
        ("lpComment", ctypes.c_ulonglong),
        ("lpProvider", ctypes.c_ulonglong),
    ]
    dwScope: int
    dwType: int
    dwDisplayType: int
    dwUsage: int
    lpLocalName: int
    lpRemoteName: int
    lpComment: int
    lpProvider: int

class _SERVICE_REQUIRED_PRIVILEGES_INFOW(ctypes.Structure):
    _fields_ = [
        ("pmszRequiredPrivileges", ctypes.c_ulonglong),
    ]
    pmszRequiredPrivileges: int

class _heapinfo(ctypes.Structure):
    _fields_ = [
        ("_pentry", ctypes.c_ulonglong),
        ("_size", ctypes.c_uint64),
        ("_useflag", ctypes.c_int32),
    ]
    _pentry: int
    _size: int
    _useflag: int

class _IMAGE_DATA_DIRECTORY(ctypes.Structure):
    _fields_ = [
        ("VirtualAddress", ctypes.c_uint32),
        ("Size", ctypes.c_uint32),
    ]
    VirtualAddress: int
    Size: int

class tagMULTIKEYHELPA(ctypes.Structure):
    _fields_ = [
        ("mkSize", ctypes.c_uint32),
        ("mkKeylist", ctypes.c_char),
        ("szKeyphrase", ctypes.c_char * 0x1),  # char[1]
    ]
    mkSize: int
    mkKeylist: str
    szKeyphrase: list[ctypes.c_char]

class _LOAD_DLL_DEBUG_INFO(ctypes.Structure):
    _fields_ = [
        ("hFile", ctypes.c_void_p),
        ("lpBaseOfDll", ctypes.c_void_p),
        ("dwDebugInfoFileOffset", ctypes.c_uint32),
        ("nDebugInfoSize", ctypes.c_uint32),
        ("lpImageName", ctypes.c_void_p),
        ("fUnicode", ctypes.c_uint16),
    ]
    hFile: int
    lpBaseOfDll: int
    dwDebugInfoFileOffset: int
    nDebugInfoSize: int
    lpImageName: int
    fUnicode: int

class _ABCFLOAT(ctypes.Structure):
    _fields_ = [
        ("abcfA", ctypes.c_float),
        ("abcfB", ctypes.c_float),
        ("abcfC", ctypes.c_float),
    ]
    abcfA: float
    abcfB: float
    abcfC: float

class tagMULTIKEYHELPW(ctypes.Structure):
    _fields_ = [
        ("mkSize", ctypes.c_uint32),
        ("mkKeylist", ctypes.c_wchar),
        ("szKeyphrase", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    mkSize: int
    mkKeylist: str
    szKeyphrase: list[ctypes.c_wchar]

class hk4xVector2f(ctypes.Structure):
    _fields_ = [
        ("m_x", common.Vector4f),
        ("m_y", common.Vector4f),
    ]
    m_x: common.Vector4f
    m_y: common.Vector4f

class hkPackedRealf(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", common.Vector4f),
    ]
    baseclass_0: common.Vector4f

class hkQuadUintUnion: pass

class hkTimerData(ctypes.Structure):
    _fields_ = [
        ("m_streamBegin", ctypes.c_char_p),
        ("m_streamEnd", ctypes.c_char_p),
    ]
    m_streamBegin: bytes
    m_streamEnd: bytes

class hkTimerProvider_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("appendTimerData", ctypes.c_char * 0x8),  # void (__fastcall *)(hkTimerProvider *this, hkArray<hkTimerData,hkBuiltinContainerAllocator<0,0> > *)
        ("clearTimerData", ctypes.c_ulonglong),
    ]
    getExactType: int
    appendTimerData: bytes
    clearTimerData: int

class hkDefaultTaskQueue_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getTimerData", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDefaultTaskQueue *this, hkArray<hkTimerData,hkBuiltinContainerAllocator<0,0> > *)
        ("clearTimerData", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getTimerData: bytes
    clearTimerData: int

class _SHUFFLE_FILE_DATA(ctypes.Structure):
    _fields_ = [
        ("StartingOffset", ctypes.c_int64),
        ("Length", ctypes.c_int64),
        ("Flags", ctypes.c_uint32),
    ]
    StartingOffset: int
    Length: int
    Flags: int

class LIST_ENTRY64(ctypes.Structure):
    _fields_ = [
        ("Flink", ctypes.c_uint64),
        ("Blink", ctypes.c_uint64),
    ]
    Flink: int
    Blink: int

class _PACKEDEVENTINFO(ctypes.Structure):
    _fields_ = [
        ("ulSize", ctypes.c_uint32),
        ("ulNumEventsForLogFile", ctypes.c_uint32),
        ("ulOffsets", ctypes.c_uint32),  # unsigned int[]
    ]
    ulSize: int
    ulNumEventsForLogFile: int
    ulOffsets: list[ctypes.c_uint32]

class _QUERY_SERVICE_LOCK_STATUSA(ctypes.Structure):
    _fields_ = [
        ("fIsLocked", ctypes.c_uint32),
        ("lpLockOwner", ctypes.c_char_p),
        ("dwLockDuration", ctypes.c_uint32),
    ]
    fIsLocked: int
    lpLockOwner: bytes
    dwLockDuration: int

class _PROCESS_MITIGATION_PAYLOAD_RESTRICTION_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $70CDF59B26AD07020EC276DAFE7E5427
    ]
    ___u0: bytes

class _MESSAGE_RESOURCE_ENTRY(ctypes.Structure):
    _fields_ = [
        ("Length", ctypes.c_uint16),
        ("Flags", ctypes.c_uint16),
        ("Text", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
    ]
    Length: int
    Flags: int
    Text: list[ctypes.c_uint8]

class _UMS_SYSTEM_THREAD_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("UmsVersion", ctypes.c_uint32),
        ("___u1", ctypes.c_char * 0x4),  # $BCA5BDB7F6F55E4549ABD385BDE5DC3B
    ]
    UmsVersion: int
    ___u1: bytes

class tagLAYERPLANEDESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("nSize", ctypes.c_uint16),
        ("nVersion", ctypes.c_uint16),
        ("dwFlags", ctypes.c_uint32),
        ("iPixelType", ctypes.c_uint8),
        ("cColorBits", ctypes.c_uint8),
        ("cRedBits", ctypes.c_uint8),
        ("cRedShift", ctypes.c_uint8),
        ("cGreenBits", ctypes.c_uint8),
        ("cGreenShift", ctypes.c_uint8),
        ("cBlueBits", ctypes.c_uint8),
        ("cBlueShift", ctypes.c_uint8),
        ("cAlphaBits", ctypes.c_uint8),
        ("cAlphaShift", ctypes.c_uint8),
        ("cAccumBits", ctypes.c_uint8),
        ("cAccumRedBits", ctypes.c_uint8),
        ("cAccumGreenBits", ctypes.c_uint8),
        ("cAccumBlueBits", ctypes.c_uint8),
        ("cAccumAlphaBits", ctypes.c_uint8),
        ("cDepthBits", ctypes.c_uint8),
        ("cStencilBits", ctypes.c_uint8),
        ("cAuxBuffers", ctypes.c_uint8),
        ("iLayerPlane", ctypes.c_uint8),
        ("bReserved", ctypes.c_uint8),
        ("crTransparent", ctypes.c_uint32),
    ]
    nSize: int
    nVersion: int
    dwFlags: int
    iPixelType: int
    cColorBits: int
    cRedBits: int
    cRedShift: int
    cGreenBits: int
    cGreenShift: int
    cBlueBits: int
    cBlueShift: int
    cAlphaBits: int
    cAlphaShift: int
    cAccumBits: int
    cAccumRedBits: int
    cAccumGreenBits: int
    cAccumBlueBits: int
    cAccumAlphaBits: int
    cDepthBits: int
    cStencilBits: int
    cAuxBuffers: int
    iLayerPlane: int
    bReserved: int
    crTransparent: int

class _ACCESS_REASONS(ctypes.Structure):
    _fields_ = [
        ("Data", ctypes.c_uint32 * 0x20),  # unsigned int[32]
    ]
    Data: list[ctypes.c_uint32]

class _MODEMSETTINGS(ctypes.Structure):
    _fields_ = [
        ("dwActualSize", ctypes.c_uint32),
        ("dwRequiredSize", ctypes.c_uint32),
        ("dwDevSpecificOffset", ctypes.c_uint32),
        ("dwDevSpecificSize", ctypes.c_uint32),
        ("dwCallSetupFailTimer", ctypes.c_uint32),
        ("dwInactivityTimeout", ctypes.c_uint32),
        ("dwSpeakerVolume", ctypes.c_uint32),
        ("dwSpeakerMode", ctypes.c_uint32),
        ("dwPreferredModemOptions", ctypes.c_uint32),
        ("dwNegotiatedModemOptions", ctypes.c_uint32),
        ("dwNegotiatedDCERate", ctypes.c_uint32),
        ("abVariablePortion", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
    ]
    dwActualSize: int
    dwRequiredSize: int
    dwDevSpecificOffset: int
    dwDevSpecificSize: int
    dwCallSetupFailTimer: int
    dwInactivityTimeout: int
    dwSpeakerVolume: int
    dwSpeakerMode: int
    dwPreferredModemOptions: int
    dwNegotiatedModemOptions: int
    dwNegotiatedDCERate: int
    abVariablePortion: list[ctypes.c_uint8]

class tagFILTERKEYS(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("iWaitMSec", ctypes.c_uint32),
        ("iDelayMSec", ctypes.c_uint32),
        ("iRepeatMSec", ctypes.c_uint32),
        ("iBounceMSec", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int
    iWaitMSec: int
    iDelayMSec: int
    iRepeatMSec: int
    iBounceMSec: int

class _IMAGE_DYNAMIC_RELOCATION64(ctypes.Structure):
    _fields_ = [
        ("Symbol", ctypes.c_uint64),
        ("BaseRelocSize", ctypes.c_uint32),
    ]
    Symbol: int
    BaseRelocSize: int

class _COMSTAT(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int32),
        ("cbInQue", ctypes.c_uint32),
        ("cbOutQue", ctypes.c_uint32),
    ]
    _bf_0: int
    cbInQue: int
    cbOutQue: int

class _SE_ACCESS_REPLY(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("ResultListCount", ctypes.c_uint32),
        ("GrantedAccess", ctypes.c_ulonglong),
        ("AccessStatus", ctypes.c_ulonglong),
        ("AccessReason", ctypes.c_ulonglong),
        ("Privileges", ctypes.c_ulonglong),
    ]
    Size: int
    ResultListCount: int
    GrantedAccess: int
    AccessStatus: int
    AccessReason: int
    Privileges: int

class _JOBOBJECT_ASSOCIATE_COMPLETION_PORT(ctypes.Structure):
    _fields_ = [
        ("CompletionKey", ctypes.c_void_p),
        ("CompletionPort", ctypes.c_void_p),
    ]
    CompletionKey: int
    CompletionPort: int

class _MESSAGE_RESOURCE_BLOCK(ctypes.Structure):
    _fields_ = [
        ("LowId", ctypes.c_uint32),
        ("HighId", ctypes.c_uint32),
        ("OffsetToEntries", ctypes.c_uint32),
    ]
    LowId: int
    HighId: int
    OffsetToEntries: int

class _JOBOBJECT_JOBSET_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("MemberLevel", ctypes.c_uint32),
    ]
    MemberLevel: int

class _GROUP_AFFINITY(ctypes.Structure):
    _fields_ = [
        ("Mask", ctypes.c_uint64),
        ("Group", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16 * 0x3),  # unsigned __int16[3]
    ]
    Mask: int
    Group: int
    Reserved: list[ctypes.c_uint16]

class _TAPE_PREPARE(ctypes.Structure):
    _fields_ = [
        ("Operation", ctypes.c_uint32),
        ("Immediate", ctypes.c_uint8),
    ]
    Operation: int
    Immediate: int

class _APP_MEMORY_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("AvailableCommit", ctypes.c_uint64),
        ("PrivateCommitUsage", ctypes.c_uint64),
        ("PeakPrivateCommitUsage", ctypes.c_uint64),
        ("TotalCommitUsage", ctypes.c_uint64),
    ]
    AvailableCommit: int
    PrivateCommitUsage: int
    PeakPrivateCommitUsage: int
    TotalCommitUsage: int

class _CLAIM_SECURITY_ATTRIBUTES_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16),
        ("AttributeCount", ctypes.c_uint32),
        ("Attribute", ctypes.c_char * 0x8),  # _CLAIM_SECURITY_ATTRIBUTES_INFORMATION::<unnamed_type_Attribute>
    ]
    Version: int
    Reserved: int
    AttributeCount: int
    Attribute: bytes

class tagPELARRAY(ctypes.Structure):
    _fields_ = [
        ("paXCount", ctypes.c_int32),
        ("paYCount", ctypes.c_int32),
        ("paXExt", ctypes.c_int32),
        ("paYExt", ctypes.c_int32),
        ("paRGBs", ctypes.c_uint8),
    ]
    paXCount: int
    paYCount: int
    paXExt: int
    paYExt: int
    paRGBs: int

class _ENCLAVE_LOAD_DATA_VBS_BASIC(ctypes.Structure):
    _fields_ = [
        ("PageType", ctypes.c_uint32),
    ]
    PageType: int

class _IMAGE_SYMBOL(ctypes.Structure):
    _fields_ = [
        ("N", ctypes.c_char * 0x8),  # _IMAGE_SYMBOL::<unnamed_type_N>
        ("Value", ctypes.c_uint32),
        ("SectionNumber", ctypes.c_int16),
        ("Type", ctypes.c_uint16),
        ("StorageClass", ctypes.c_uint8),
        ("NumberOfAuxSymbols", ctypes.c_uint8),
    ]
    N: bytes
    Value: int
    SectionNumber: int
    Type: int
    StorageClass: int
    NumberOfAuxSymbols: int

class hkCountOfBadArgCheck(ctypes.Structure):
    pass

class cTkOldVector4(ctypes.Structure):
    _fields_ = [
        ("mfX", ctypes.c_float),
        ("mfY", ctypes.c_float),
        ("mfZ", ctypes.c_float),
        ("mfW", ctypes.c_float),
    ]
    mfX: float
    mfY: float
    mfZ: float
    mfW: float

class _CRT_FLOAT(ctypes.Structure):
    _fields_ = [
        ("f", ctypes.c_float),
    ]
    f: float

class tagWNDCLASSW(ctypes.Structure):
    _fields_ = [
        ("style", ctypes.c_uint32),
        ("lpfnWndProc", ctypes.c_ulonglong),
        ("cbClsExtra", ctypes.c_int32),
        ("cbWndExtra", ctypes.c_int32),
        ("hInstance", ctypes.c_ulonglong),
        ("hIcon", ctypes.c_ulonglong),
        ("hCursor", ctypes.c_ulonglong),
        ("hbrBackground", ctypes.c_ulonglong),
        ("lpszMenuName", ctypes.c_ulonglong),
        ("lpszClassName", ctypes.c_ulonglong),
    ]
    style: int
    lpfnWndProc: int
    cbClsExtra: int
    cbWndExtra: int
    hInstance: int
    hIcon: int
    hCursor: int
    hbrBackground: int
    lpszMenuName: int
    lpszClassName: int

class _CONNECTDLGSTRUCTW(ctypes.Structure):
    _fields_ = [
        ("cbStructure", ctypes.c_uint32),
        ("hwndOwner", ctypes.c_ulonglong),
        ("lpConnRes", ctypes.c_ulonglong),
        ("dwFlags", ctypes.c_uint32),
        ("dwDevNum", ctypes.c_uint32),
    ]
    cbStructure: int
    hwndOwner: int
    lpConnRes: int
    dwFlags: int
    dwDevNum: int

class tagCBTACTIVATESTRUCT(ctypes.Structure):
    _fields_ = [
        ("fMouse", ctypes.c_int32),
        ("hWndActive", ctypes.c_ulonglong),
    ]
    fMouse: int
    hWndActive: int

class tagDRAWTEXTPARAMS(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("iTabLength", ctypes.c_int32),
        ("iLeftMargin", ctypes.c_int32),
        ("iRightMargin", ctypes.c_int32),
        ("uiLengthDrawn", ctypes.c_uint32),
    ]
    cbSize: int
    iTabLength: int
    iLeftMargin: int
    iRightMargin: int
    uiLengthDrawn: int

class _RIP_INFO(ctypes.Structure):
    _fields_ = [
        ("dwError", ctypes.c_uint32),
        ("dwType", ctypes.c_uint32),
    ]
    dwError: int
    dwType: int

class tagUPDATELAYEREDWINDOWINFO(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("hdcDst", ctypes.c_ulonglong),
        ("pptDst", ctypes.c_ulonglong),
        ("psize", ctypes.c_ulonglong),
        ("hdcSrc", ctypes.c_ulonglong),
        ("pptSrc", ctypes.c_ulonglong),
        ("crKey", ctypes.c_uint32),
        ("pblend", ctypes.c_ulonglong),
        ("dwFlags", ctypes.c_uint32),
        ("prcDirty", ctypes.c_ulonglong),
    ]
    cbSize: int
    hdcDst: int
    pptDst: int
    psize: int
    hdcSrc: int
    pptSrc: int
    crKey: int
    pblend: int
    dwFlags: int
    prcDirty: int

class _ICONINFOEXA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fIcon", ctypes.c_int32),
        ("xHotspot", ctypes.c_uint32),
        ("yHotspot", ctypes.c_uint32),
        ("hbmMask", ctypes.c_ulonglong),
        ("hbmColor", ctypes.c_ulonglong),
        ("wResID", ctypes.c_uint16),
        ("szModName", ctypes.c_char * 0x104),  # char[260]
        ("szResName", ctypes.c_char * 0x104),  # char[260]
    ]
    cbSize: int
    fIcon: int
    xHotspot: int
    yHotspot: int
    hbmMask: int
    hbmColor: int
    wResID: int
    szModName: list[ctypes.c_char]
    szResName: list[ctypes.c_char]

class tagAUDIODESCRIPTION(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("Enabled", ctypes.c_int32),
        ("Locale", ctypes.c_uint32),
    ]
    cbSize: int
    Enabled: int
    Locale: int

class _IMAGE_RESOURCE_DIR_STRING_U(ctypes.Structure):
    _fields_ = [
        ("Length", ctypes.c_uint16),
        ("NameString", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    Length: int
    NameString: list[ctypes.c_wchar]

class tagEVENTMSG(ctypes.Structure):
    _fields_ = [
        ("message", ctypes.c_uint32),
        ("paramL", ctypes.c_uint32),
        ("paramH", ctypes.c_uint32),
        ("time", ctypes.c_uint32),
        ("hwnd", ctypes.c_ulonglong),
    ]
    message: int
    paramL: int
    paramH: int
    time: int
    hwnd: int

class _EVENTLOG_FULL_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("dwFull", ctypes.c_uint32),
    ]
    dwFull: int

class _SERVICE_REQUIRED_PRIVILEGES_INFOA(ctypes.Structure):
    _fields_ = [
        ("pmszRequiredPrivileges", ctypes.c_char_p),
    ]
    pmszRequiredPrivileges: bytes

class HMETAFILE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class DiyFp(ctypes.Structure):
    _fields_ = [
        ("f", ctypes.c_uint64),
        ("e", ctypes.c_int32),
    ]
    f: int
    e: int

class tagHARDWAREHOOKSTRUCT(ctypes.Structure):
    _fields_ = [
        ("hwnd", ctypes.c_ulonglong),
        ("message", ctypes.c_uint32),
        ("wParam", ctypes.c_uint64),
        ("lParam", ctypes.c_int64),
    ]
    hwnd: int
    message: int
    wParam: int
    lParam: int

class _MENU_EVENT_RECORD(ctypes.Structure):
    _fields_ = [
        ("dwCommandId", ctypes.c_uint32),
    ]
    dwCommandId: int

class _FOCUS_EVENT_RECORD(ctypes.Structure):
    _fields_ = [
        ("bSetFocus", ctypes.c_int32),
    ]
    bSetFocus: int

class _INPUT_RECORD(ctypes.Structure):
    _fields_ = [
        ("EventType", ctypes.c_uint16),
        ("Event", ctypes.c_char * 0x10),  # _INPUT_RECORD::<unnamed_type_Event>
    ]
    EventType: int
    Event: bytes

class _OSVERSIONINFOEXW(ctypes.Structure):
    _fields_ = [
        ("dwOSVersionInfoSize", ctypes.c_uint32),
        ("dwMajorVersion", ctypes.c_uint32),
        ("dwMinorVersion", ctypes.c_uint32),
        ("dwBuildNumber", ctypes.c_uint32),
        ("dwPlatformId", ctypes.c_uint32),
        ("szCSDVersion", ctypes.c_wchar * 0x80),  # wchar_t[128]
        ("wServicePackMajor", ctypes.c_uint16),
        ("wServicePackMinor", ctypes.c_uint16),
        ("wSuiteMask", ctypes.c_uint16),
        ("wProductType", ctypes.c_uint8),
        ("wReserved", ctypes.c_uint8),
    ]
    dwOSVersionInfoSize: int
    dwMajorVersion: int
    dwMinorVersion: int
    dwBuildNumber: int
    dwPlatformId: int
    szCSDVersion: list[ctypes.c_wchar]
    wServicePackMajor: int
    wServicePackMinor: int
    wSuiteMask: int
    wProductType: int
    wReserved: int

class _IMAGE_BOUND_IMPORT_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("TimeDateStamp", ctypes.c_uint32),
        ("OffsetModuleName", ctypes.c_uint16),
        ("NumberOfModuleForwarderRefs", ctypes.c_uint16),
    ]
    TimeDateStamp: int
    OffsetModuleName: int
    NumberOfModuleForwarderRefs: int

class _complex(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_double),
        ("y", ctypes.c_double),
    ]
    x: float
    y: float

class _TAPE_SET_DRIVE_PARAMETERS(ctypes.Structure):
    _fields_ = [
        ("ECC", ctypes.c_uint8),
        ("Compression", ctypes.c_uint8),
        ("DataPadding", ctypes.c_uint8),
        ("ReportSetmarks", ctypes.c_uint8),
        ("EOTWarningZoneSize", ctypes.c_uint32),
    ]
    ECC: int
    Compression: int
    DataPadding: int
    ReportSetmarks: int
    EOTWarningZoneSize: int

class tagRAWHID(ctypes.Structure):
    _fields_ = [
        ("dwSizeHid", ctypes.c_uint32),
        ("dwCount", ctypes.c_uint32),
        ("bRawData", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
    ]
    dwSizeHid: int
    dwCount: int
    bRawData: list[ctypes.c_uint8]

class _SERVICE_DESCRIPTIONA(ctypes.Structure):
    _fields_ = [
        ("lpDescription", ctypes.c_char_p),
    ]
    lpDescription: bytes

class _TOKEN_APPCONTAINER_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("TokenAppContainer", ctypes.c_void_p),
    ]
    TokenAppContainer: int

class _OPERATION_START_PARAMETERS(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("OperationId", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
    ]
    Version: int
    OperationId: int
    Flags: int

class LIST_ENTRY32(ctypes.Structure):
    _fields_ = [
        ("Flink", ctypes.c_uint32),
        ("Blink", ctypes.c_uint32),
    ]
    Flink: int
    Blink: int

class _IMAGE_ARM_RUNTIME_FUNCTION_ENTRY(ctypes.Structure):
    _fields_ = [
        ("BeginAddress", ctypes.c_uint32),
        ("___u1", ctypes.c_char * 0x4),  # $4E522F19F1E40B472F4DE5ECED5B0AAC
    ]
    BeginAddress: int
    ___u1: bytes

class _PSINJECTDATA(ctypes.Structure):
    _fields_ = [
        ("DataBytes", ctypes.c_uint32),
        ("InjectionPoint", ctypes.c_uint16),
        ("PageNumber", ctypes.c_uint16),
    ]
    DataBytes: int
    InjectionPoint: int
    PageNumber: int

class tagCOLORADJUSTMENT(ctypes.Structure):
    _fields_ = [
        ("caSize", ctypes.c_uint16),
        ("caFlags", ctypes.c_uint16),
        ("caIlluminantIndex", ctypes.c_uint16),
        ("caRedGamma", ctypes.c_uint16),
        ("caGreenGamma", ctypes.c_uint16),
        ("caBlueGamma", ctypes.c_uint16),
        ("caReferenceBlack", ctypes.c_uint16),
        ("caReferenceWhite", ctypes.c_uint16),
        ("caContrast", ctypes.c_int16),
        ("caBrightness", ctypes.c_int16),
        ("caColorfulness", ctypes.c_int16),
        ("caRedGreenTint", ctypes.c_int16),
    ]
    caSize: int
    caFlags: int
    caIlluminantIndex: int
    caRedGamma: int
    caGreenGamma: int
    caBlueGamma: int
    caReferenceBlack: int
    caReferenceWhite: int
    caContrast: int
    caBrightness: int
    caColorfulness: int
    caRedGreenTint: int

class _NT_TIB32(ctypes.Structure):
    _fields_ = [
        ("ExceptionList", ctypes.c_uint32),
        ("StackBase", ctypes.c_uint32),
        ("StackLimit", ctypes.c_uint32),
        ("SubSystemTib", ctypes.c_uint32),
        ("___u4", ctypes.c_char * 0x4),  # $E38888FFFF2E8047BDBD55C921E86469
        ("ArbitraryUserPointer", ctypes.c_uint32),
        ("Self", ctypes.c_uint32),
    ]
    ExceptionList: int
    StackBase: int
    StackLimit: int
    SubSystemTib: int
    ___u4: bytes
    ArbitraryUserPointer: int
    Self: int

class tagMENUGETOBJECTINFO(ctypes.Structure):
    _fields_ = [
        ("dwFlags", ctypes.c_uint32),
        ("uPos", ctypes.c_uint32),
        ("hmenu", ctypes.c_ulonglong),
        ("riid", ctypes.c_void_p),
        ("pvObj", ctypes.c_void_p),
    ]
    dwFlags: int
    uPos: int
    hmenu: int
    riid: int
    pvObj: int

class tagSOUNDSENTRYA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("iFSTextEffect", ctypes.c_uint32),
        ("iFSTextEffectMSec", ctypes.c_uint32),
        ("iFSTextEffectColorBits", ctypes.c_uint32),
        ("iFSGrafEffect", ctypes.c_uint32),
        ("iFSGrafEffectMSec", ctypes.c_uint32),
        ("iFSGrafEffectColor", ctypes.c_uint32),
        ("iWindowsEffect", ctypes.c_uint32),
        ("iWindowsEffectMSec", ctypes.c_uint32),
        ("lpszWindowsEffectDLL", ctypes.c_char_p),
        ("iWindowsEffectOrdinal", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int
    iFSTextEffect: int
    iFSTextEffectMSec: int
    iFSTextEffectColorBits: int
    iFSGrafEffect: int
    iFSGrafEffectMSec: int
    iFSGrafEffectColor: int
    iWindowsEffect: int
    iWindowsEffectMSec: int
    lpszWindowsEffectDLL: bytes
    iWindowsEffectOrdinal: int

class hkStackCache(ctypes.Structure):
    _fields_ = [
        ("m_stackAllocator", ctypes.c_ulonglong),
    ]
    m_stackAllocator: int

class cTkMemoryPoolBlock(ctypes.Structure):
    def tilde_cTkMemoryPoolBlock(): pass

    _fields_ = [
        ("miPrevPool", ctypes.c_int32),
    ]
    miPrevPool: int

class tagRAWKEYBOARD(ctypes.Structure):
    _fields_ = [
        ("MakeCode", ctypes.c_uint16),
        ("Flags", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16),
        ("VKey", ctypes.c_uint16),
        ("Message", ctypes.c_uint32),
        ("ExtraInformation", ctypes.c_uint32),
    ]
    MakeCode: int
    Flags: int
    Reserved: int
    VKey: int
    Message: int
    ExtraInformation: int

class tagRAWINPUTHEADER(ctypes.Structure):
    _fields_ = [
        ("dwType", ctypes.c_uint32),
        ("dwSize", ctypes.c_uint32),
        ("hDevice", ctypes.c_void_p),
        ("wParam", ctypes.c_uint64),
    ]
    dwType: int
    dwSize: int
    hDevice: int
    wParam: int

class _IMAGE_ALPHA64_RUNTIME_FUNCTION_ENTRY(ctypes.Structure):
    _fields_ = [
        ("BeginAddress", ctypes.c_uint64),
        ("EndAddress", ctypes.c_uint64),
        ("ExceptionHandler", ctypes.c_uint64),
        ("HandlerData", ctypes.c_uint64),
        ("PrologEndAddress", ctypes.c_uint64),
    ]
    BeginAddress: int
    EndAddress: int
    ExceptionHandler: int
    HandlerData: int
    PrologEndAddress: int

class tagNEWTEXTMETRICA(ctypes.Structure):
    _fields_ = [
        ("tmHeight", ctypes.c_int32),
        ("tmAscent", ctypes.c_int32),
        ("tmDescent", ctypes.c_int32),
        ("tmInternalLeading", ctypes.c_int32),
        ("tmExternalLeading", ctypes.c_int32),
        ("tmAveCharWidth", ctypes.c_int32),
        ("tmMaxCharWidth", ctypes.c_int32),
        ("tmWeight", ctypes.c_int32),
        ("tmOverhang", ctypes.c_int32),
        ("tmDigitizedAspectX", ctypes.c_int32),
        ("tmDigitizedAspectY", ctypes.c_int32),
        ("tmFirstChar", ctypes.c_uint8),
        ("tmLastChar", ctypes.c_uint8),
        ("tmDefaultChar", ctypes.c_uint8),
        ("tmBreakChar", ctypes.c_uint8),
        ("tmItalic", ctypes.c_uint8),
        ("tmUnderlined", ctypes.c_uint8),
        ("tmStruckOut", ctypes.c_uint8),
        ("tmPitchAndFamily", ctypes.c_uint8),
        ("tmCharSet", ctypes.c_uint8),
        ("ntmFlags", ctypes.c_uint32),
        ("ntmSizeEM", ctypes.c_uint32),
        ("ntmCellHeight", ctypes.c_uint32),
        ("ntmAvgWidth", ctypes.c_uint32),
    ]
    tmHeight: int
    tmAscent: int
    tmDescent: int
    tmInternalLeading: int
    tmExternalLeading: int
    tmAveCharWidth: int
    tmMaxCharWidth: int
    tmWeight: int
    tmOverhang: int
    tmDigitizedAspectX: int
    tmDigitizedAspectY: int
    tmFirstChar: int
    tmLastChar: int
    tmDefaultChar: int
    tmBreakChar: int
    tmItalic: int
    tmUnderlined: int
    tmStruckOut: int
    tmPitchAndFamily: int
    tmCharSet: int
    ntmFlags: int
    ntmSizeEM: int
    ntmCellHeight: int
    ntmAvgWidth: int

class _POWER_SESSION_TIMEOUTS(ctypes.Structure):
    _fields_ = [
        ("InputTimeout", ctypes.c_uint32),
        ("DisplayTimeout", ctypes.c_uint32),
    ]
    InputTimeout: int
    DisplayTimeout: int

class DPI_AWARENESS_CONTEXT__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class hkAsyncThreadPoolInstrumenter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class tagCOMPAREITEMSTRUCT(ctypes.Structure):
    _fields_ = [
        ("CtlType", ctypes.c_uint32),
        ("CtlID", ctypes.c_uint32),
        ("hwndItem", ctypes.c_ulonglong),
        ("itemID1", ctypes.c_uint32),
        ("itemData1", ctypes.c_uint64),
        ("itemID2", ctypes.c_uint32),
        ("itemData2", ctypes.c_uint64),
        ("dwLocaleId", ctypes.c_uint32),
    ]
    CtlType: int
    CtlID: int
    hwndItem: int
    itemID1: int
    itemData1: int
    itemID2: int
    itemData2: int
    dwLocaleId: int

class _IMAGE_TLS_DIRECTORY64(ctypes.Structure):
    _fields_ = [
        ("StartAddressOfRawData", ctypes.c_uint64),
        ("EndAddressOfRawData", ctypes.c_uint64),
        ("AddressOfIndex", ctypes.c_uint64),
        ("AddressOfCallBacks", ctypes.c_uint64),
        ("SizeOfZeroFill", ctypes.c_uint32),
        ("___u5", ctypes.c_char * 0x4),  # $5FA4C0220B92CB02F4DA67383B5FA5AE
    ]
    StartAddressOfRawData: int
    EndAddressOfRawData: int
    AddressOfIndex: int
    AddressOfCallBacks: int
    SizeOfZeroFill: int
    ___u5: bytes

class tagRAWINPUTDEVICE(ctypes.Structure):
    _fields_ = [
        ("usUsagePage", ctypes.c_uint16),
        ("usUsage", ctypes.c_uint16),
        ("dwFlags", ctypes.c_uint32),
        ("hwndTarget", ctypes.c_ulonglong),
    ]
    usUsagePage: int
    usUsage: int
    dwFlags: int
    hwndTarget: int

class _XSAVE_AREA_HEADER(ctypes.Structure):
    _fields_ = [
        ("Mask", ctypes.c_uint64),
        ("CompactionMask", ctypes.c_uint64),
        ("Reserved2", ctypes.c_uint64 * 0x6),  # unsigned __int64[6]
    ]
    Mask: int
    CompactionMask: int
    Reserved2: list[ctypes.c_uint64]

class _XSTATE_CONTEXT(ctypes.Structure):
    _fields_ = [
        ("Mask", ctypes.c_uint64),
        ("Length", ctypes.c_uint32),
        ("Reserved1", ctypes.c_uint32),
        ("Area", ctypes.c_ulonglong),
        ("Buffer", ctypes.c_void_p),
    ]
    Mask: int
    Length: int
    Reserved1: int
    Area: int
    Buffer: int

class PPM_PERFSTATE_EVENT(ctypes.Structure):
    _fields_ = [
        ("State", ctypes.c_uint32),
        ("Status", ctypes.c_uint32),
        ("Latency", ctypes.c_uint32),
        ("Speed", ctypes.c_uint32),
        ("Processor", ctypes.c_uint32),
    ]
    State: int
    Status: int
    Latency: int
    Speed: int
    Processor: int

class hkTimeFunctionHelper(ctypes.Structure):
    _fields_ = [
        ("m_streamPtr", ctypes.c_ulonglong),
    ]
    m_streamPtr: int

class tagMETAFILEPICT(ctypes.Structure):
    _fields_ = [
        ("mm", ctypes.c_int32),
        ("xExt", ctypes.c_int32),
        ("yExt", ctypes.c_int32),
        ("hMF", ctypes.c_ulonglong),
    ]
    mm: int
    xExt: int
    yExt: int
    hMF: int

class hkTimeListFunctionHelper(ctypes.Structure):
    _fields_ = [
        ("m_streamPtr", ctypes.c_ulonglong),
    ]
    m_streamPtr: int

class _UNWIND_HISTORY_TABLE_ENTRY(ctypes.Structure):
    _fields_ = [
        ("ImageBase", ctypes.c_uint64),
        ("FunctionEntry", ctypes.c_ulonglong),
    ]
    ImageBase: int
    FunctionEntry: int

class pvalueW(ctypes.Structure):
    _fields_ = [
        ("pv_valuename", ctypes.c_ulonglong),
        ("pv_valuelen", ctypes.c_int32),
        ("pv_value_context", ctypes.c_void_p),
        ("pv_type", ctypes.c_uint32),
    ]
    pv_valuename: int
    pv_valuelen: int
    pv_value_context: int
    pv_type: int

class _SERVICE_DELAYED_AUTO_START_INFO(ctypes.Structure):
    _fields_ = [
        ("fDelayedAutostart", ctypes.c_int32),
    ]
    fDelayedAutostart: int

class value_entA(ctypes.Structure):
    _fields_ = [
        ("ve_valuename", ctypes.c_char_p),
        ("ve_valuelen", ctypes.c_uint32),
        ("ve_valueptr", ctypes.c_uint64),
        ("ve_type", ctypes.c_uint32),
    ]
    ve_valuename: bytes
    ve_valuelen: int
    ve_valueptr: int
    ve_type: int

class _XSAVE_CET_U_FORMAT(ctypes.Structure):
    _fields_ = [
        ("Ia32CetUMsr", ctypes.c_uint64),
        ("Ia32Pl3SspMsr", ctypes.c_uint64),
    ]
    Ia32CetUMsr: int
    Ia32Pl3SspMsr: int

class PPM_IDLE_STATE_BUCKET_EX(ctypes.Structure):
    _fields_ = [
        ("TotalTimeUs", ctypes.c_uint64),
        ("MinTimeUs", ctypes.c_uint32),
        ("MaxTimeUs", ctypes.c_uint32),
        ("Count", ctypes.c_uint32),
    ]
    TotalTimeUs: int
    MinTimeUs: int
    MaxTimeUs: int
    Count: int

class tagEXTLOGPEN(ctypes.Structure):
    _fields_ = [
        ("elpPenStyle", ctypes.c_uint32),
        ("elpWidth", ctypes.c_uint32),
        ("elpBrushStyle", ctypes.c_uint32),
        ("elpColor", ctypes.c_uint32),
        ("elpHatch", ctypes.c_uint64),
        ("elpNumEntries", ctypes.c_uint32),
        ("elpStyleEntry", ctypes.c_uint32 * 0x1),  # unsigned int[1]
    ]
    elpPenStyle: int
    elpWidth: int
    elpBrushStyle: int
    elpColor: int
    elpHatch: int
    elpNumEntries: int
    elpStyleEntry: list[ctypes.c_uint32]

class tagRECONVERTSTRING(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwVersion", ctypes.c_uint32),
        ("dwStrLen", ctypes.c_uint32),
        ("dwStrOffset", ctypes.c_uint32),
        ("dwCompStrLen", ctypes.c_uint32),
        ("dwCompStrOffset", ctypes.c_uint32),
        ("dwTargetStrLen", ctypes.c_uint32),
        ("dwTargetStrOffset", ctypes.c_uint32),
    ]
    dwSize: int
    dwVersion: int
    dwStrLen: int
    dwStrOffset: int
    dwCompStrLen: int
    dwCompStrOffset: int
    dwTargetStrLen: int
    dwTargetStrOffset: int

class tagTouchPredictionParameters(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwLatency", ctypes.c_uint32),
        ("dwSampleTime", ctypes.c_uint32),
        ("bUseHWTimeStamp", ctypes.c_uint32),
    ]
    cbSize: int
    dwLatency: int
    dwSampleTime: int
    bUseHWTimeStamp: int

class _TOKEN_DEVICE_CLAIMS(ctypes.Structure):
    _fields_ = [
        ("DeviceClaims", ctypes.c_void_p),
    ]
    DeviceClaims: int

class _SYSTEM_POWER_STATUS(ctypes.Structure):
    _fields_ = [
        ("ACLineStatus", ctypes.c_uint8),
        ("BatteryFlag", ctypes.c_uint8),
        ("BatteryLifePercent", ctypes.c_uint8),
        ("SystemStatusFlag", ctypes.c_uint8),
        ("BatteryLifeTime", ctypes.c_uint32),
        ("BatteryFullLifeTime", ctypes.c_uint32),
    ]
    ACLineStatus: int
    BatteryFlag: int
    BatteryLifePercent: int
    SystemStatusFlag: int
    BatteryLifeTime: int
    BatteryFullLifeTime: int

class tagHIGHCONTRASTW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("lpszDefaultScheme", ctypes.c_ulonglong),
    ]
    cbSize: int
    dwFlags: int
    lpszDefaultScheme: int

class _ACL_REVISION_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("AclRevision", ctypes.c_uint32),
    ]
    AclRevision: int

class HRSRC__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _timespec32(ctypes.Structure):
    _fields_ = [
        ("tv_sec", ctypes.c_int32),
        ("tv_nsec", ctypes.c_int32),
    ]
    tv_sec: int
    tv_nsec: int

class _UNIVERSAL_NAME_INFOW(ctypes.Structure):
    _fields_ = [
        ("lpUniversalName", ctypes.c_ulonglong),
    ]
    lpUniversalName: int

class _SECURITY_OBJECT_AI_PARAMS(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("ConstraintMask", ctypes.c_uint32),
    ]
    Size: int
    ConstraintMask: int

class val_context(ctypes.Structure):
    _fields_ = [
        ("valuelen", ctypes.c_int32),
        ("value_context", ctypes.c_void_p),
        ("val_buff_ptr", ctypes.c_void_p),
    ]
    valuelen: int
    value_context: int
    val_buff_ptr: int

class provider_info(ctypes.Structure):
    _fields_ = [
        ("pi_R0_1val", ctypes.c_ulonglong),
        ("pi_R0_allvals", ctypes.c_ulonglong),
        ("pi_R3_1val", ctypes.c_ulonglong),
        ("pi_R3_allvals", ctypes.c_ulonglong),
        ("pi_flags", ctypes.c_uint32),
        ("pi_key_context", ctypes.c_void_p),
    ]
    pi_R0_1val: int
    pi_R0_allvals: int
    pi_R3_1val: int
    pi_R3_allvals: int
    pi_flags: int
    pi_key_context: int

class _div_t(ctypes.Structure):
    _fields_ = [
        ("quot", ctypes.c_int32),
        ("rem", ctypes.c_int32),
    ]
    quot: int
    rem: int

class tm(ctypes.Structure):
    _fields_ = [
        ("tm_sec", ctypes.c_int32),
        ("tm_min", ctypes.c_int32),
        ("tm_hour", ctypes.c_int32),
        ("tm_mday", ctypes.c_int32),
        ("tm_mon", ctypes.c_int32),
        ("tm_year", ctypes.c_int32),
        ("tm_wday", ctypes.c_int32),
        ("tm_yday", ctypes.c_int32),
        ("tm_isdst", ctypes.c_int32),
    ]
    tm_sec: int
    tm_min: int
    tm_hour: int
    tm_mday: int
    tm_mon: int
    tm_year: int
    tm_wday: int
    tm_yday: int
    tm_isdst: int

class hkAsyncTaskQueueThreadPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("stopThreads", ctypes.c_ulonglong),
        ("workerThreadMain", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    stopThreads: int
    workerThreadMain: int

class _LDBL12(ctypes.Structure):
    _fields_ = [
        ("ld12", ctypes.c_uint8 * 0xC),  # unsigned __int8[12]
    ]
    ld12: list[ctypes.c_uint8]

class _QUERY_SERVICE_LOCK_STATUSW(ctypes.Structure):
    _fields_ = [
        ("fIsLocked", ctypes.c_uint32),
        ("lpLockOwner", ctypes.c_ulonglong),
        ("dwLockDuration", ctypes.c_uint32),
    ]
    fIsLocked: int
    lpLockOwner: int
    dwLockDuration: int

class tagRGBTRIPLE(ctypes.Structure):
    _fields_ = [
        ("rgbtBlue", ctypes.c_uint8),
        ("rgbtGreen", ctypes.c_uint8),
        ("rgbtRed", ctypes.c_uint8),
    ]
    rgbtBlue: int
    rgbtGreen: int
    rgbtRed: int

class _SCOPE_TABLE_AMD64(ctypes.Structure):
    _fields_ = [
        ("Count", ctypes.c_uint32),
        ("ScopeRecord", ctypes.c_char * 0x10),  # _SCOPE_TABLE_AMD64::<unnamed_type_ScopeRecord>
    ]
    Count: int
    ScopeRecord: bytes

class hkBoolOperator(ctypes.Structure):
    pass

class _IMAGE_RELOCATION(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $66A2FC5ED47CB5912809650DAB14E256
        ("SymbolTableIndex", ctypes.c_uint32),
        ("Type", ctypes.c_uint16),
    ]
    ___u0: bytes
    SymbolTableIndex: int
    Type: int

class hkAsyncThreadPoolSimpleSemaphore(ctypes.Structure):
    def hkAsyncThreadPoolSimpleSemaphore(): pass

    def acquire(): pass

    def release(): pass

    _fields_ = [
        ("m_owningThreadPool", ctypes.c_ulonglong),
        ("m_ownerThreadIndex", ctypes.c_int32),
        ("m_count", ctypes.c_char * 0x4),  # hkAtomic::Variable<int>
    ]
    m_owningThreadPool: int
    m_ownerThreadIndex: int
    m_count: bytes

class _MEMORY_BASIC_INFORMATION64(ctypes.Structure):
    _fields_ = [
        ("BaseAddress", ctypes.c_uint64),
        ("AllocationBase", ctypes.c_uint64),
        ("AllocationProtect", ctypes.c_uint32),
        ("__alignment1", ctypes.c_uint32),
        ("RegionSize", ctypes.c_uint64),
        ("State", ctypes.c_uint32),
        ("Protect", ctypes.c_uint32),
        ("Type", ctypes.c_uint32),
        ("__alignment2", ctypes.c_uint32),
    ]
    BaseAddress: int
    AllocationBase: int
    AllocationProtect: int
    __alignment1: int
    RegionSize: int
    State: int
    Protect: int
    Type: int
    __alignment2: int

class _TOKEN_BNO_ISOLATION_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("IsolationPrefix", ctypes.c_ulonglong),
        ("IsolationEnabled", ctypes.c_uint8),
    ]
    IsolationPrefix: int
    IsolationEnabled: int

class _CRT_DOUBLE(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_double),
    ]
    x: float

class _CFG_CALL_TARGET_INFO(ctypes.Structure):
    _fields_ = [
        ("Offset", ctypes.c_uint64),
        ("Flags", ctypes.c_uint64),
    ]
    Offset: int
    Flags: int

class _SCRUB_PARITY_EXTENT(ctypes.Structure):
    _fields_ = [
        ("Offset", ctypes.c_int64),
        ("Length", ctypes.c_uint64),
    ]
    Offset: int
    Length: int

class _MAXVERSIONTESTED_INFO(ctypes.Structure):
    _fields_ = [
        ("MaxVersionTested", ctypes.c_uint64),
    ]
    MaxVersionTested: int

class hkIntVectorConstantU32_: pass

class tagSCROLLINFO(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fMask", ctypes.c_uint32),
        ("nMin", ctypes.c_int32),
        ("nMax", ctypes.c_int32),
        ("nPage", ctypes.c_uint32),
        ("nPos", ctypes.c_int32),
        ("nTrackPos", ctypes.c_int32),
    ]
    cbSize: int
    fMask: int
    nMin: int
    nMax: int
    nPage: int
    nPos: int
    nTrackPos: int

class HRAWINPUT__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class PPM_IDLESTATE_EVENT(ctypes.Structure):
    _fields_ = [
        ("NewState", ctypes.c_uint32),
        ("OldState", ctypes.c_uint32),
        ("Processors", ctypes.c_uint64),
    ]
    NewState: int
    OldState: int
    Processors: int

class _IMAGE_FUNCTION_ENTRY(ctypes.Structure):
    _fields_ = [
        ("StartingAddress", ctypes.c_uint32),
        ("EndingAddress", ctypes.c_uint32),
        ("EndOfPrologue", ctypes.c_uint32),
    ]
    StartingAddress: int
    EndingAddress: int
    EndOfPrologue: int

class HTOUCHINPUT__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class tagRGBQUAD(ctypes.Structure):
    _fields_ = [
        ("rgbBlue", ctypes.c_uint8),
        ("rgbGreen", ctypes.c_uint8),
        ("rgbRed", ctypes.c_uint8),
        ("rgbReserved", ctypes.c_uint8),
    ]
    rgbBlue: int
    rgbGreen: int
    rgbRed: int
    rgbReserved: int

class ITkNGuiDraggable(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class ITkNGuiDraggable_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
    ]
    Render: int
    GetType: int

class hkMonitorStreamStringMap_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _IMAGE_VXD_HEADER(ctypes.Structure):
    _fields_ = [
        ("e32_magic", ctypes.c_uint16),
        ("e32_border", ctypes.c_uint8),
        ("e32_worder", ctypes.c_uint8),
        ("e32_level", ctypes.c_uint32),
        ("e32_cpu", ctypes.c_uint16),
        ("e32_os", ctypes.c_uint16),
        ("e32_ver", ctypes.c_uint32),
        ("e32_mflags", ctypes.c_uint32),
        ("e32_mpages", ctypes.c_uint32),
        ("e32_startobj", ctypes.c_uint32),
        ("e32_eip", ctypes.c_uint32),
        ("e32_stackobj", ctypes.c_uint32),
        ("e32_esp", ctypes.c_uint32),
        ("e32_pagesize", ctypes.c_uint32),
        ("e32_lastpagesize", ctypes.c_uint32),
        ("e32_fixupsize", ctypes.c_uint32),
        ("e32_fixupsum", ctypes.c_uint32),
        ("e32_ldrsize", ctypes.c_uint32),
        ("e32_ldrsum", ctypes.c_uint32),
        ("e32_objtab", ctypes.c_uint32),
        ("e32_objcnt", ctypes.c_uint32),
        ("e32_objmap", ctypes.c_uint32),
        ("e32_itermap", ctypes.c_uint32),
        ("e32_rsrctab", ctypes.c_uint32),
        ("e32_rsrccnt", ctypes.c_uint32),
        ("e32_restab", ctypes.c_uint32),
        ("e32_enttab", ctypes.c_uint32),
        ("e32_dirtab", ctypes.c_uint32),
        ("e32_dircnt", ctypes.c_uint32),
        ("e32_fpagetab", ctypes.c_uint32),
        ("e32_frectab", ctypes.c_uint32),
        ("e32_impmod", ctypes.c_uint32),
        ("e32_impmodcnt", ctypes.c_uint32),
        ("e32_impproc", ctypes.c_uint32),
        ("e32_pagesum", ctypes.c_uint32),
        ("e32_datapage", ctypes.c_uint32),
        ("e32_preload", ctypes.c_uint32),
        ("e32_nrestab", ctypes.c_uint32),
        ("e32_cbnrestab", ctypes.c_uint32),
        ("e32_nressum", ctypes.c_uint32),
        ("e32_autodata", ctypes.c_uint32),
        ("e32_debuginfo", ctypes.c_uint32),
        ("e32_debuglen", ctypes.c_uint32),
        ("e32_instpreload", ctypes.c_uint32),
        ("e32_instdemand", ctypes.c_uint32),
        ("e32_heapsize", ctypes.c_uint32),
        ("e32_res3", ctypes.c_uint8 * 0xC),  # unsigned __int8[12]
        ("e32_winresoff", ctypes.c_uint32),
        ("e32_winreslen", ctypes.c_uint32),
        ("e32_devid", ctypes.c_uint16),
        ("e32_ddkver", ctypes.c_uint16),
    ]
    e32_magic: int
    e32_border: int
    e32_worder: int
    e32_level: int
    e32_cpu: int
    e32_os: int
    e32_ver: int
    e32_mflags: int
    e32_mpages: int
    e32_startobj: int
    e32_eip: int
    e32_stackobj: int
    e32_esp: int
    e32_pagesize: int
    e32_lastpagesize: int
    e32_fixupsize: int
    e32_fixupsum: int
    e32_ldrsize: int
    e32_ldrsum: int
    e32_objtab: int
    e32_objcnt: int
    e32_objmap: int
    e32_itermap: int
    e32_rsrctab: int
    e32_rsrccnt: int
    e32_restab: int
    e32_enttab: int
    e32_dirtab: int
    e32_dircnt: int
    e32_fpagetab: int
    e32_frectab: int
    e32_impmod: int
    e32_impmodcnt: int
    e32_impproc: int
    e32_pagesum: int
    e32_datapage: int
    e32_preload: int
    e32_nrestab: int
    e32_cbnrestab: int
    e32_nressum: int
    e32_autodata: int
    e32_debuginfo: int
    e32_debuglen: int
    e32_instpreload: int
    e32_instdemand: int
    e32_heapsize: int
    e32_res3: list[ctypes.c_uint8]
    e32_winresoff: int
    e32_winreslen: int
    e32_devid: int
    e32_ddkver: int

class cTkResourceManagerBase(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkResourceManagerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResourceManagerBase_dtor_0", ctypes.c_ulonglong),
    ]
    cTkResourceManagerBase_dtor_0: int

class _ENCLAVE_CREATE_INFO_VBS(ctypes.Structure):
    _fields_ = [
        ("Flags", ctypes.c_uint32),
        ("OwnerID", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
    ]
    Flags: int
    OwnerID: list[ctypes.c_uint8]

class _TRANSACTIONMANAGER_LOGPATH_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("LogPathLength", ctypes.c_uint32),
        ("LogPath", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    LogPathLength: int
    LogPath: list[ctypes.c_wchar]

class _EXIT_THREAD_DEBUG_INFO(ctypes.Structure):
    _fields_ = [
        ("dwExitCode", ctypes.c_uint32),
    ]
    dwExitCode: int

class tagREGISTERWORDA(ctypes.Structure):
    _fields_ = [
        ("lpReading", ctypes.c_char_p),
        ("lpWord", ctypes.c_char_p),
    ]
    lpReading: bytes
    lpWord: bytes

class _devicemodeA(ctypes.Structure):
    _fields_ = [
        ("dmDeviceName", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
        ("dmSpecVersion", ctypes.c_uint16),
        ("dmDriverVersion", ctypes.c_uint16),
        ("dmSize", ctypes.c_uint16),
        ("dmDriverExtra", ctypes.c_uint16),
        ("dmFields", ctypes.c_uint32),
        ("___u6", ctypes.c_char * 0x8),  # $BB358C3A325381C626105C9C85EB3F4A
        ("___u7", ctypes.c_char * 0x4),  # $DEE03DF9F4D6FE316E4B77CDE5AE0A90
        ("___u8", ctypes.c_char * 0x4),  # $6B826ED569E6FCAD26E63896A7C9594E
        ("dmColor", ctypes.c_int16),
        ("dmDuplex", ctypes.c_int16),
        ("dmYResolution", ctypes.c_int16),
        ("dmTTOption", ctypes.c_int16),
        ("dmCollate", ctypes.c_int16),
        ("dmFormName", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
        ("dmLogPixels", ctypes.c_uint16),
        ("dmBitsPerPel", ctypes.c_uint32),
        ("dmPelsWidth", ctypes.c_uint32),
        ("dmPelsHeight", ctypes.c_uint32),
        ("___u19", ctypes.c_char * 0x4),  # $309666D137CBB19809FA597AD348B415
        ("dmDisplayFrequency", ctypes.c_uint32),
        ("dmICMMethod", ctypes.c_uint32),
        ("dmICMIntent", ctypes.c_uint32),
        ("dmMediaType", ctypes.c_uint32),
        ("dmDitherType", ctypes.c_uint32),
        ("dmReserved1", ctypes.c_uint32),
        ("dmReserved2", ctypes.c_uint32),
        ("dmPanningWidth", ctypes.c_uint32),
        ("dmPanningHeight", ctypes.c_uint32),
    ]
    dmDeviceName: list[ctypes.c_uint8]
    dmSpecVersion: int
    dmDriverVersion: int
    dmSize: int
    dmDriverExtra: int
    dmFields: int
    ___u6: bytes
    ___u7: bytes
    ___u8: bytes
    dmColor: int
    dmDuplex: int
    dmYResolution: int
    dmTTOption: int
    dmCollate: int
    dmFormName: list[ctypes.c_uint8]
    dmLogPixels: int
    dmBitsPerPel: int
    dmPelsWidth: int
    dmPelsHeight: int
    ___u19: bytes
    dmDisplayFrequency: int
    dmICMMethod: int
    dmICMIntent: int
    dmMediaType: int
    dmDitherType: int
    dmReserved1: int
    dmReserved2: int
    dmPanningWidth: int
    dmPanningHeight: int

class _FILE_DISPOSITION_INFO_EX(ctypes.Structure):
    _fields_ = [
        ("Flags", ctypes.c_uint32),
    ]
    Flags: int

class _RTL_RUN_ONCE: pass

class TkCloudDataInfo(ctypes.Structure):
    _fields_ = [
        ("mUploadTimestamp", ctypes.c_uint64),
        ("mMetaDataJson", ctypes.c_char * 0x200),  # cTkFixedString<512,char>
    ]
    mUploadTimestamp: int
    mMetaDataJson: bytes

class _PROCESS_LEAP_SECOND_INFO(ctypes.Structure):
    _fields_ = [
        ("Flags", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint32),
    ]
    Flags: int
    Reserved: int

class _FILE_STORAGE_INFO(ctypes.Structure):
    _fields_ = [
        ("LogicalBytesPerSector", ctypes.c_uint32),
        ("PhysicalBytesPerSectorForAtomicity", ctypes.c_uint32),
        ("PhysicalBytesPerSectorForPerformance", ctypes.c_uint32),
        ("FileSystemEffectivePhysicalBytesPerSectorForAtomicity", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("ByteOffsetForSectorAlignment", ctypes.c_uint32),
        ("ByteOffsetForPartitionAlignment", ctypes.c_uint32),
    ]
    LogicalBytesPerSector: int
    PhysicalBytesPerSectorForAtomicity: int
    PhysicalBytesPerSectorForPerformance: int
    FileSystemEffectivePhysicalBytesPerSectorForAtomicity: int
    Flags: int
    ByteOffsetForSectorAlignment: int
    ByteOffsetForPartitionAlignment: int

class tagMDICREATESTRUCTA(ctypes.Structure):
    _fields_ = [
        ("szClass", ctypes.c_char_p),
        ("szTitle", ctypes.c_char_p),
        ("hOwner", ctypes.c_void_p),
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("_cx", ctypes.c_int32),
        ("cy", ctypes.c_int32),
        ("style", ctypes.c_uint32),
        ("lParam", ctypes.c_int64),
    ]
    szClass: bytes
    szTitle: bytes
    hOwner: int
    x: int
    y: int
    _cx: int
    cy: int
    style: int
    lParam: int

class _SID_AND_ATTRIBUTES_HASH(ctypes.Structure):
    _fields_ = [
        ("SidCount", ctypes.c_uint32),
        ("SidAttr", ctypes.c_ulonglong),
        ("Hash", ctypes.c_uint64 * 0x20),  # unsigned __int64[32]
    ]
    SidCount: int
    SidAttr: int
    Hash: list[ctypes.c_uint64]

class _TP_POOL_STACK_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("StackReserve", ctypes.c_uint64),
        ("StackCommit", ctypes.c_uint64),
    ]
    StackReserve: int
    StackCommit: int

class _NETCONNECTINFOSTRUCT(ctypes.Structure):
    _fields_ = [
        ("cbStructure", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("dwSpeed", ctypes.c_uint32),
        ("dwDelay", ctypes.c_uint32),
        ("dwOptDataSize", ctypes.c_uint32),
    ]
    cbStructure: int
    dwFlags: int
    dwSpeed: int
    dwDelay: int
    dwOptDataSize: int

class HTASK__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class tagRID_DEVICE_INFO_MOUSE(ctypes.Structure):
    _fields_ = [
        ("dwId", ctypes.c_uint32),
        ("dwNumberOfButtons", ctypes.c_uint32),
        ("dwSampleRate", ctypes.c_uint32),
        ("fHasHorizontalWheel", ctypes.c_int32),
    ]
    dwId: int
    dwNumberOfButtons: int
    dwSampleRate: int
    fHasHorizontalWheel: int

class tagRID_DEVICE_INFO_KEYBOARD(ctypes.Structure):
    _fields_ = [
        ("dwType", ctypes.c_uint32),
        ("dwSubType", ctypes.c_uint32),
        ("dwKeyboardMode", ctypes.c_uint32),
        ("dwNumberOfFunctionKeys", ctypes.c_uint32),
        ("dwNumberOfIndicators", ctypes.c_uint32),
        ("dwNumberOfKeysTotal", ctypes.c_uint32),
    ]
    dwType: int
    dwSubType: int
    dwKeyboardMode: int
    dwNumberOfFunctionKeys: int
    dwNumberOfIndicators: int
    dwNumberOfKeysTotal: int

class tagRID_DEVICE_INFO(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwType", ctypes.c_uint32),
        ("___u2", ctypes.c_char * 0x18),  # $263B2921A0EB073AD8296BF108617F0F
    ]
    cbSize: int
    dwType: int
    ___u2: bytes

class hkMonitorStreamTypeMap_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkMonitorStreamColorTable_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _MODLOAD_DATA(ctypes.Structure):
    _fields_ = [
        ("ssize", ctypes.c_uint32),
        ("ssig", ctypes.c_uint32),
        ("data", ctypes.c_void_p),
        ("size", ctypes.c_uint32),
        ("flags", ctypes.c_uint32),
    ]
    ssize: int
    ssig: int
    data: int
    size: int
    flags: int

class _SYMBOL_INFO(ctypes.Structure):
    _fields_ = [
        ("SizeOfStruct", ctypes.c_uint32),
        ("TypeIndex", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint64 * 0x2),  # unsigned __int64[2]
        ("Index", ctypes.c_uint32),
        ("Size", ctypes.c_uint32),
        ("ModBase", ctypes.c_uint64),
        ("Flags", ctypes.c_uint32),
        ("Value", ctypes.c_uint64),
        ("Address", ctypes.c_uint64),
        ("Register", ctypes.c_uint32),
        ("Scope", ctypes.c_uint32),
        ("Tag", ctypes.c_uint32),
        ("NameLen", ctypes.c_uint32),
        ("MaxNameLen", ctypes.c_uint32),
        ("Name", ctypes.c_char * 0x1),  # char[1]
    ]
    SizeOfStruct: int
    TypeIndex: int
    Reserved: list[ctypes.c_uint64]
    Index: int
    Size: int
    ModBase: int
    Flags: int
    Value: int
    Address: int
    Register: int
    Scope: int
    Tag: int
    NameLen: int
    MaxNameLen: int
    Name: list[ctypes.c_char]

class _KDHELP64(ctypes.Structure):
    _fields_ = [
        ("Thread", ctypes.c_uint64),
        ("ThCallbackStack", ctypes.c_uint32),
        ("ThCallbackBStore", ctypes.c_uint32),
        ("NextCallback", ctypes.c_uint32),
        ("FramePointer", ctypes.c_uint32),
        ("KiCallUserMode", ctypes.c_uint64),
        ("KeUserCallbackDispatcher", ctypes.c_uint64),
        ("SystemRangeStart", ctypes.c_uint64),
        ("KiUserExceptionDispatcher", ctypes.c_uint64),
        ("StackBase", ctypes.c_uint64),
        ("StackLimit", ctypes.c_uint64),
        ("BuildVersion", ctypes.c_uint32),
        ("RetpolineStubFunctionTableSize", ctypes.c_uint32),
        ("RetpolineStubFunctionTable", ctypes.c_uint64),
        ("RetpolineStubOffset", ctypes.c_uint32),
        ("RetpolineStubSize", ctypes.c_uint32),
        ("Reserved0", ctypes.c_uint64 * 0x2),  # unsigned __int64[2]
    ]
    Thread: int
    ThCallbackStack: int
    ThCallbackBStore: int
    NextCallback: int
    FramePointer: int
    KiCallUserMode: int
    KeUserCallbackDispatcher: int
    SystemRangeStart: int
    KiUserExceptionDispatcher: int
    StackBase: int
    StackLimit: int
    BuildVersion: int
    RetpolineStubFunctionTableSize: int
    RetpolineStubFunctionTable: int
    RetpolineStubOffset: int
    RetpolineStubSize: int
    Reserved0: list[ctypes.c_uint64]

class _IMAGEHLP_LINE64(ctypes.Structure):
    _fields_ = [
        ("SizeOfStruct", ctypes.c_uint32),
        ("Key", ctypes.c_void_p),
        ("LineNumber", ctypes.c_uint32),
        ("FileName", ctypes.c_char_p),
        ("Address", ctypes.c_uint64),
    ]
    SizeOfStruct: int
    Key: int
    LineNumber: int
    FileName: bytes
    Address: int

class StackTraceContext(ctypes.Structure):
    _fields_ = [
        ("cur", ctypes.c_uint64),
        ("buf", ctypes.c_ulonglong),
        ("end", ctypes.c_uint64),
    ]
    cur: int
    buf: int
    end: int

class _exception(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_int32),
        ("name", ctypes.c_char_p),
        ("arg1", ctypes.c_double),
        ("arg2", ctypes.c_double),
        ("retval", ctypes.c_double),
    ]
    type: int
    name: bytes
    arg1: float
    arg2: float
    retval: float

class hkMonitorStreamColorTableCache(ctypes.Structure):
    _fields_ = [
        ("m_colorTable", ctypes.c_char * 0x8),  # hkRefPtr<hkMonitorStreamColorTable const >
        ("m_colorCache", ctypes.c_char * 0x10),  # hkPointerMap<char const *,unsigned int,hkBuiltinContainerAllocator<0,0> >
        ("m_unknownColorCache", ctypes.c_char * 0x18),  # hkSet<char const *,hkBuiltinContainerAllocator<0,0>,hkMapOperations<char const *> >
    ]
    m_colorTable: bytes
    m_colorCache: bytes
    m_unknownColorCache: bytes

class cTkScopedCriticalSectionLock(ctypes.Structure):
    def cTkScopedCriticalSectionLock(): pass

    def tilde_cTkScopedCriticalSectionLock(): pass

    _fields_ = [
        ("mrCriticalSection", ctypes.c_ulonglong),
    ]
    mrCriticalSection: int

class _OPERATION_END_PARAMETERS(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("OperationId", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
    ]
    Version: int
    OperationId: int
    Flags: int

class HPEN__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class __m256: pass

class BATTERY_REPORTING_SCALE(ctypes.Structure):
    _fields_ = [
        ("Granularity", ctypes.c_uint32),
        ("Capacity", ctypes.c_uint32),
    ]
    Granularity: int
    Capacity: int

class _HEAP_OPTIMIZE_RESOURCES_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
    ]
    Version: int
    Flags: int

class _TOKEN_DEFAULT_DACL(ctypes.Structure):
    _fields_ = [
        ("DefaultDacl", ctypes.c_ulonglong),
    ]
    DefaultDacl: int

class tagCHANGEFILTERSTRUCT(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("ExtStatus", ctypes.c_uint32),
    ]
    cbSize: int
    ExtStatus: int

class HRGN__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _ACL_SIZE_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("AceCount", ctypes.c_uint32),
        ("AclBytesInUse", ctypes.c_uint32),
        ("AclBytesFree", ctypes.c_uint32),
    ]
    AceCount: int
    AclBytesInUse: int
    AclBytesFree: int

class _FILE_NAME_INFO(ctypes.Structure):
    _fields_ = [
        ("FileNameLength", ctypes.c_uint32),
        ("FileName", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    FileNameLength: int
    FileName: list[ctypes.c_wchar]

class tagMENUITEMINFOW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fMask", ctypes.c_uint32),
        ("fType", ctypes.c_uint32),
        ("fState", ctypes.c_uint32),
        ("wID", ctypes.c_uint32),
        ("hSubMenu", ctypes.c_ulonglong),
        ("hbmpChecked", ctypes.c_ulonglong),
        ("hbmpUnchecked", ctypes.c_ulonglong),
        ("dwItemData", ctypes.c_uint64),
        ("dwTypeData", ctypes.c_ulonglong),
        ("cch", ctypes.c_uint32),
        ("hbmpItem", ctypes.c_ulonglong),
    ]
    cbSize: int
    fMask: int
    fType: int
    fState: int
    wID: int
    hSubMenu: int
    hbmpChecked: int
    hbmpUnchecked: int
    dwItemData: int
    dwTypeData: int
    cch: int
    hbmpItem: int

class hkRelPtr_PointerImpl_vtbl(ctypes.Structure):
    _fields_ = [
        ("Impl_dtor_0", ctypes.c_ulonglong),
        ("inplaceFixup", ctypes.c_ulonglong),
        ("getSetter", ctypes.c_ulonglong),
        ("getFieldImpl", ctypes.c_ulonglong),
        ("isProperty", ctypes.c_ulonglong),
        ("find", ctypes.c_ulonglong),
        ("iterBegin", ctypes.c_ulonglong),
        ("iterNext", ctypes.c_ulonglong),
        ("setValue", ctypes.c_ulonglong),
        ("getValue", ctypes.c_ulonglong),
        ("getValueAs", ctypes.c_ulonglong),
        ("getValueAddress", ctypes.c_ulonglong),
        ("canBeNull", ctypes.c_ulonglong),
    ]
    Impl_dtor_0: int
    inplaceFixup: int
    getSetter: int
    getFieldImpl: int
    isProperty: int
    find: int
    iterBegin: int
    iterNext: int
    setValue: int
    getValue: int
    getValueAs: int
    getValueAddress: int
    canBeNull: int

class HACCEL__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _IMAGE_RESOURCE_DATA_ENTRY(ctypes.Structure):
    _fields_ = [
        ("OffsetToData", ctypes.c_uint32),
        ("Size", ctypes.c_uint32),
        ("CodePage", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint32),
    ]
    OffsetToData: int
    Size: int
    CodePage: int
    Reserved: int

class _RESUME_PERFORMANCE(ctypes.Structure):
    _fields_ = [
        ("PostTimeMs", ctypes.c_uint32),
        ("TotalResumeTimeMs", ctypes.c_uint64),
        ("ResumeCompleteTimestamp", ctypes.c_uint64),
    ]
    PostTimeMs: int
    TotalResumeTimeMs: int
    ResumeCompleteTimestamp: int

class _IMAGE_CE_RUNTIME_FUNCTION_ENTRY(ctypes.Structure):
    _fields_ = [
        ("FuncStart", ctypes.c_uint32),
        ("_bf_4", ctypes.c_int32),
    ]
    FuncStart: int
    _bf_4: int

class PPM_WMI_PERF_STATE(ctypes.Structure):
    _fields_ = [
        ("Frequency", ctypes.c_uint32),
        ("Power", ctypes.c_uint32),
        ("PercentFrequency", ctypes.c_uint8),
        ("IncreaseLevel", ctypes.c_uint8),
        ("DecreaseLevel", ctypes.c_uint8),
        ("Type", ctypes.c_uint8),
        ("IncreaseTime", ctypes.c_uint32),
        ("DecreaseTime", ctypes.c_uint32),
        ("Control", ctypes.c_uint64),
        ("Status", ctypes.c_uint64),
        ("HitCount", ctypes.c_uint32),
        ("Reserved1", ctypes.c_uint32),
        ("Reserved2", ctypes.c_uint64),
        ("Reserved3", ctypes.c_uint64),
    ]
    Frequency: int
    Power: int
    PercentFrequency: int
    IncreaseLevel: int
    DecreaseLevel: int
    Type: int
    IncreaseTime: int
    DecreaseTime: int
    Control: int
    Status: int
    HitCount: int
    Reserved1: int
    Reserved2: int
    Reserved3: int

class _JOBOBJECT_CPU_RATE_CONTROL_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("ControlFlags", ctypes.c_uint32),
        ("___u1", ctypes.c_char * 0x4),  # $40378D963CDF33882B2459D0DD6B8A0E
    ]
    ControlFlags: int
    ___u1: bytes

class _THREAD_POWER_THROTTLING_STATE(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("ControlMask", ctypes.c_uint32),
        ("StateMask", ctypes.c_uint32),
    ]
    Version: int
    ControlMask: int
    StateMask: int

class _IMAGE_COFF_SYMBOLS_HEADER(ctypes.Structure):
    _fields_ = [
        ("NumberOfSymbols", ctypes.c_uint32),
        ("LvaToFirstSymbol", ctypes.c_uint32),
        ("NumberOfLinenumbers", ctypes.c_uint32),
        ("LvaToFirstLinenumber", ctypes.c_uint32),
        ("RvaToFirstByteOfCode", ctypes.c_uint32),
        ("RvaToLastByteOfCode", ctypes.c_uint32),
        ("RvaToFirstByteOfData", ctypes.c_uint32),
        ("RvaToLastByteOfData", ctypes.c_uint32),
    ]
    NumberOfSymbols: int
    LvaToFirstSymbol: int
    NumberOfLinenumbers: int
    LvaToFirstLinenumber: int
    RvaToFirstByteOfCode: int
    RvaToLastByteOfCode: int
    RvaToFirstByteOfData: int
    RvaToLastByteOfData: int

class _DISPLAY_DEVICEW(ctypes.Structure):
    _fields_ = [
        ("cb", ctypes.c_uint32),
        ("DeviceName", ctypes.c_wchar * 0x20),  # wchar_t[32]
        ("DeviceString", ctypes.c_wchar * 0x80),  # wchar_t[128]
        ("StateFlags", ctypes.c_uint32),
        ("DeviceID", ctypes.c_wchar * 0x80),  # wchar_t[128]
        ("DeviceKey", ctypes.c_wchar * 0x80),  # wchar_t[128]
    ]
    cb: int
    DeviceName: list[ctypes.c_wchar]
    DeviceString: list[ctypes.c_wchar]
    StateFlags: int
    DeviceID: list[ctypes.c_wchar]
    DeviceKey: list[ctypes.c_wchar]

class hkQuadShortUnion: pass

class hkQuadUshortUnion: pass

class hkIntVectorConstantU16_: pass

class cTkAtomicLinearMemoryPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkMemoryPool_dtor_0", ctypes.c_ulonglong),
        ("Create", ctypes.c_ulonglong),
        ("Malloc", ctypes.c_ulonglong),
        ("Malloc_18", ctypes.c_ulonglong),
        ("Free", ctypes.c_ulonglong),
        ("Realloc", ctypes.c_ulonglong),
        ("ContainsAddress", ctypes.c_ulonglong),
        ("GetAllocationSize", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("GetUsage", ctypes.c_ulonglong),
        ("GetHighWaterUsage", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("Cleanup", ctypes.c_ulonglong),
        ("GetAllocInfo", ctypes.c_ulonglong),
        ("SetAllocInfo", ctypes.c_ulonglong),
        ("ClearStats", ctypes.c_ulonglong),
        ("OutputStats", ctypes.c_ulonglong),
        ("FrameSyncEnd", ctypes.c_ulonglong),
        ("NeedsFrameSyncEnd", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("FlagError", ctypes.c_ulonglong),
        ("HasError", ctypes.c_ulonglong),
        ("NeedsMutex", ctypes.c_ulonglong),
        ("SetMutex", ctypes.c_ulonglong),
        ("GetOverflowPool", ctypes.c_ulonglong),
        ("GetBaseAddress", ctypes.c_ulonglong),
    ]
    cTkMemoryPool_dtor_0: int
    Create: int
    Malloc: int
    Malloc_18: int
    Free: int
    Realloc: int
    ContainsAddress: int
    GetAllocationSize: int
    SetAlignment: int
    GetAlignment: int
    GetUsage: int
    GetHighWaterUsage: int
    GetSize: int
    Cleanup: int
    GetAllocInfo: int
    SetAllocInfo: int
    ClearStats: int
    OutputStats: int
    FrameSyncEnd: int
    NeedsFrameSyncEnd: int
    Reset: int
    FlagError: int
    HasError: int
    NeedsMutex: int
    SetMutex: int
    GetOverflowPool: int
    GetBaseAddress: int

class tagCANDIDATELIST(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwStyle", ctypes.c_uint32),
        ("dwCount", ctypes.c_uint32),
        ("dwSelection", ctypes.c_uint32),
        ("dwPageStart", ctypes.c_uint32),
        ("dwPageSize", ctypes.c_uint32),
        ("dwOffset", ctypes.c_uint32 * 0x1),  # unsigned int[1]
    ]
    dwSize: int
    dwStyle: int
    dwCount: int
    dwSelection: int
    dwPageStart: int
    dwPageSize: int
    dwOffset: list[ctypes.c_uint32]

class _FILE_ID_128(ctypes.Structure):
    _fields_ = [
        ("Identifier", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
    ]
    Identifier: list[ctypes.c_uint8]

class _s__CatchableType(ctypes.Structure):
    _fields_ = [
        ("properties", ctypes.c_uint32),
        ("pType", ctypes.c_ulonglong),
        ("sizeOrOffset", ctypes.c_int32),
        ("copyFunction", ctypes.c_ulonglong),
    ]
    properties: int
    pType: int
    sizeOrOffset: int
    copyFunction: int

class hkQuadCharUnion: pass

class _REARRANGE_FILE_DATA(ctypes.Structure):
    _fields_ = [
        ("SourceStartingOffset", ctypes.c_uint64),
        ("TargetOffset", ctypes.c_uint64),
        ("SourceFileHandle", ctypes.c_void_p),
        ("Length", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
    ]
    SourceStartingOffset: int
    TargetOffset: int
    SourceFileHandle: int
    Length: int
    Flags: int

class hkMonitorStreamStatics_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _cpinfo(ctypes.Structure):
    _fields_ = [
        ("MaxCharSize", ctypes.c_uint32),
        ("DefaultChar", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
        ("LeadByte", ctypes.c_uint8 * 0xC),  # unsigned __int8[12]
    ]
    MaxCharSize: int
    DefaultChar: list[ctypes.c_uint8]
    LeadByte: list[ctypes.c_uint8]

class _TAPE_GET_DRIVE_PARAMETERS(ctypes.Structure):
    _fields_ = [
        ("ECC", ctypes.c_uint8),
        ("Compression", ctypes.c_uint8),
        ("DataPadding", ctypes.c_uint8),
        ("ReportSetmarks", ctypes.c_uint8),
        ("DefaultBlockSize", ctypes.c_uint32),
        ("MaximumBlockSize", ctypes.c_uint32),
        ("MinimumBlockSize", ctypes.c_uint32),
        ("MaximumPartitionCount", ctypes.c_uint32),
        ("FeaturesLow", ctypes.c_uint32),
        ("FeaturesHigh", ctypes.c_uint32),
        ("EOTWarningZoneSize", ctypes.c_uint32),
    ]
    ECC: int
    Compression: int
    DataPadding: int
    ReportSetmarks: int
    DefaultBlockSize: int
    MaximumBlockSize: int
    MinimumBlockSize: int
    MaximumPartitionCount: int
    FeaturesLow: int
    FeaturesHigh: int
    EOTWarningZoneSize: int

class hkStopwatch(ctypes.Structure):
    def getTickCounter(): pass

    _fields_ = [
        ("m_ticks_at_start", ctypes.c_uint64),
        ("m_ticks_total", ctypes.c_uint64),
        ("m_ticks_at_split", ctypes.c_uint64),
        ("m_split_total", ctypes.c_uint64),
        ("m_inv_ticks_per_second", ctypes.c_double),
        ("m_running_flag", ctypes.c_ubyte),
        ("m_num_timings", ctypes.c_int32),
        ("m_name", ctypes.c_char_p),
    ]
    m_ticks_at_start: int
    m_ticks_total: int
    m_ticks_at_split: int
    m_split_total: int
    m_inv_ticks_per_second: float
    m_running_flag: bool
    m_num_timings: int
    m_name: bytes

class _KCRM_MARSHAL_HEADER(ctypes.Structure):
    _fields_ = [
        ("VersionMajor", ctypes.c_uint32),
        ("VersionMinor", ctypes.c_uint32),
        ("NumProtocols", ctypes.c_uint32),
        ("Unused", ctypes.c_uint32),
    ]
    VersionMajor: int
    VersionMinor: int
    NumProtocols: int
    Unused: int

class _SECURITY_CAPABILITIES(ctypes.Structure):
    _fields_ = [
        ("AppContainerSid", ctypes.c_void_p),
        ("Capabilities", ctypes.c_ulonglong),
        ("CapabilityCount", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint32),
    ]
    AppContainerSid: int
    Capabilities: int
    CapabilityCount: int
    Reserved: int

class _EXCEPTION_POINTERS(ctypes.Structure):
    _fields_ = [
        ("ExceptionRecord", ctypes.c_ulonglong),
        ("ContextRecord", ctypes.c_ulonglong),
    ]
    ExceptionRecord: int
    ContextRecord: int

class _POWER_IDLE_RESILIENCY(ctypes.Structure):
    _fields_ = [
        ("CoalescingTimeout", ctypes.c_uint32),
        ("IdleResiliencyPeriod", ctypes.c_uint32),
    ]
    CoalescingTimeout: int
    IdleResiliencyPeriod: int

class cTkFibonacciSphere(ctypes.Structure):
    def Generate(): pass

    def GenerateCylinder(): pass


class _MEMORY_BASIC_INFORMATION32(ctypes.Structure):
    _fields_ = [
        ("BaseAddress", ctypes.c_uint32),
        ("AllocationBase", ctypes.c_uint32),
        ("AllocationProtect", ctypes.c_uint32),
        ("RegionSize", ctypes.c_uint32),
        ("State", ctypes.c_uint32),
        ("Protect", ctypes.c_uint32),
        ("Type", ctypes.c_uint32),
    ]
    BaseAddress: int
    AllocationBase: int
    AllocationProtect: int
    RegionSize: int
    State: int
    Protect: int
    Type: int

class tagGCP_RESULTSA(ctypes.Structure):
    _fields_ = [
        ("lStructSize", ctypes.c_uint32),
        ("lpOutString", ctypes.c_char_p),
        ("lpOrder", ctypes.c_ulonglong),
        ("lpDx", ctypes.c_ulonglong),
        ("lpCaretPos", ctypes.c_ulonglong),
        ("lpClass", ctypes.c_char_p),
        ("lpGlyphs", ctypes.c_ulonglong),
        ("nGlyphs", ctypes.c_uint32),
        ("nMaxFit", ctypes.c_int32),
    ]
    lStructSize: int
    lpOutString: bytes
    lpOrder: int
    lpDx: int
    lpCaretPos: int
    lpClass: bytes
    lpGlyphs: int
    nGlyphs: int
    nMaxFit: int

class HDESK__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _DOCINFOW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_int32),
        ("lpszDocName", ctypes.c_ulonglong),
        ("lpszOutput", ctypes.c_ulonglong),
        ("lpszDatatype", ctypes.c_ulonglong),
        ("fwType", ctypes.c_uint32),
    ]
    cbSize: int
    lpszDocName: int
    lpszOutput: int
    lpszDatatype: int
    fwType: int

class cTkRazorLocalContext(ctypes.Structure):
    pass

class hkDisableTimersScope(ctypes.Structure):
    _fields_ = [
        ("m_capacity", ctypes.c_char_p),
        ("m_capacityMinus16", ctypes.c_char_p),
        ("m_streamPtr", ctypes.c_ulonglong),
    ]
    m_capacity: bytes
    m_capacityMinus16: bytes
    m_streamPtr: int

class _wfinddata64_t(ctypes.Structure):
    _fields_ = [
        ("attrib", ctypes.c_uint32),
        ("time_create", ctypes.c_int64),
        ("time_access", ctypes.c_int64),
        ("time_write", ctypes.c_int64),
        ("size", ctypes.c_int64),
        ("name", ctypes.c_wchar * 0x104),  # wchar_t[260]
    ]
    attrib: int
    time_create: int
    time_access: int
    time_write: int
    size: int
    name: list[ctypes.c_wchar]

class _WIN32_MEMORY_RANGE_ENTRY(ctypes.Structure):
    _fields_ = [
        ("VirtualAddress", ctypes.c_void_p),
        ("NumberOfBytes", ctypes.c_uint64),
    ]
    VirtualAddress: int
    NumberOfBytes: int

class tagUSEROBJECTFLAGS(ctypes.Structure):
    _fields_ = [
        ("fInherit", ctypes.c_int32),
        ("fReserved", ctypes.c_int32),
        ("dwFlags", ctypes.c_uint32),
    ]
    fInherit: int
    fReserved: int
    dwFlags: int

class _IMAGE_RESOURCE_DIRECTORY_ENTRY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $89BE06A10BECD997917FDDA7D01573F9
        ("___u1", ctypes.c_char * 0x4),  # $F9595926F63227F3A30093655F9B2841
    ]
    ___u0: bytes
    ___u1: bytes

class _MEM_ADDRESS_REQUIREMENTS(ctypes.Structure):
    _fields_ = [
        ("LowestStartingAddress", ctypes.c_void_p),
        ("HighestEndingAddress", ctypes.c_void_p),
        ("Alignment", ctypes.c_uint64),
    ]
    LowestStartingAddress: int
    HighestEndingAddress: int
    Alignment: int

class tagPOINTER_TYPE_INFO(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint32),
        ("___u1", ctypes.c_char * 0x90),  # $315C85445887E8BC03DA311C6EF2B027
    ]
    type: int
    ___u1: bytes

class tagHELPWININFOA(ctypes.Structure):
    _fields_ = [
        ("wStructSize", ctypes.c_int32),
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("_dx", ctypes.c_int32),
        ("dy", ctypes.c_int32),
        ("wMax", ctypes.c_int32),
        ("rgchMember", ctypes.c_char * 0x2),  # char[2]
    ]
    wStructSize: int
    x: int
    y: int
    _dx: int
    dy: int
    wMax: int
    rgchMember: list[ctypes.c_char]

class _RTL_CONDITION_VARIABLE(ctypes.Structure):
    _fields_ = [
        ("Ptr", ctypes.c_void_p),
    ]
    Ptr: int

class _IMAGE_DELAYLOAD_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("Attributes", ctypes.c_char * 0x4),  # _IMAGE_DELAYLOAD_DESCRIPTOR::<unnamed_type_Attributes>
        ("DllNameRVA", ctypes.c_uint32),
        ("ModuleHandleRVA", ctypes.c_uint32),
        ("ImportAddressTableRVA", ctypes.c_uint32),
        ("ImportNameTableRVA", ctypes.c_uint32),
        ("BoundImportAddressTableRVA", ctypes.c_uint32),
        ("UnloadInformationTableRVA", ctypes.c_uint32),
        ("TimeDateStamp", ctypes.c_uint32),
    ]
    Attributes: bytes
    DllNameRVA: int
    ModuleHandleRVA: int
    ImportAddressTableRVA: int
    ImportNameTableRVA: int
    BoundImportAddressTableRVA: int
    UnloadInformationTableRVA: int
    TimeDateStamp: int

class tagMOUSEINPUT(ctypes.Structure):
    _fields_ = [
        ("_dx", ctypes.c_int32),
        ("dy", ctypes.c_int32),
        ("mouseData", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("time", ctypes.c_uint32),
        ("dwExtraInfo", ctypes.c_uint64),
    ]
    _dx: int
    dy: int
    mouseData: int
    dwFlags: int
    time: int
    dwExtraInfo: int

class tagKEYBDINPUT(ctypes.Structure):
    _fields_ = [
        ("wVk", ctypes.c_uint16),
        ("wScan", ctypes.c_uint16),
        ("dwFlags", ctypes.c_uint32),
        ("time", ctypes.c_uint32),
        ("dwExtraInfo", ctypes.c_uint64),
    ]
    wVk: int
    wScan: int
    dwFlags: int
    time: int
    dwExtraInfo: int

class tagINPUT(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint32),
        ("___u1", ctypes.c_char * 0x20),  # $D77DF73B3DB0D8BFE0BC38175EDCAA0E
    ]
    type: int
    ___u1: bytes

class _Mbstatet(ctypes.Structure):
    _fields_ = [
        ("_Wchar", ctypes.c_uint32),
        ("_Byte", ctypes.c_uint16),
        ("_State", ctypes.c_uint16),
    ]
    _Wchar: int
    _Byte: int
    _State: int

class _WOW64_LDT_ENTRY(ctypes.Structure):
    _fields_ = [
        ("LimitLow", ctypes.c_uint16),
        ("BaseLow", ctypes.c_uint16),
        ("HighWord", ctypes.c_char * 0x4),  # _WOW64_LDT_ENTRY::<unnamed_type_HighWord>
    ]
    LimitLow: int
    BaseLow: int
    HighWord: bytes

class _RATE_QUOTA_LIMIT: pass

class _IMAGE_SECTION_HEADER(ctypes.Structure):
    _fields_ = [
        ("Name", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
        ("Misc", ctypes.c_char * 0x4),  # _IMAGE_SECTION_HEADER::<unnamed_type_Misc>
        ("VirtualAddress", ctypes.c_uint32),
        ("SizeOfRawData", ctypes.c_uint32),
        ("PointerToRawData", ctypes.c_uint32),
        ("PointerToRelocations", ctypes.c_uint32),
        ("PointerToLinenumbers", ctypes.c_uint32),
        ("NumberOfRelocations", ctypes.c_uint16),
        ("NumberOfLinenumbers", ctypes.c_uint16),
        ("Characteristics", ctypes.c_uint32),
    ]
    Name: list[ctypes.c_uint8]
    Misc: bytes
    VirtualAddress: int
    SizeOfRawData: int
    PointerToRawData: int
    PointerToRelocations: int
    PointerToLinenumbers: int
    NumberOfRelocations: int
    NumberOfLinenumbers: int
    Characteristics: int

class _RASTERIZER_STATUS(ctypes.Structure):
    _fields_ = [
        ("nSize", ctypes.c_int16),
        ("wFlags", ctypes.c_int16),
        ("nLanguageID", ctypes.c_int16),
    ]
    nSize: int
    wFlags: int
    nLanguageID: int

class HUMPD__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _PROCESSOR_POWER_POLICY_INFO(ctypes.Structure):
    _fields_ = [
        ("TimeCheck", ctypes.c_uint32),
        ("DemoteLimit", ctypes.c_uint32),
        ("PromoteLimit", ctypes.c_uint32),
        ("DemotePercent", ctypes.c_uint8),
        ("PromotePercent", ctypes.c_uint8),
        ("Spare", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
        ("_bf_10", ctypes.c_int32),
    ]
    TimeCheck: int
    DemoteLimit: int
    PromoteLimit: int
    DemotePercent: int
    PromotePercent: int
    Spare: list[ctypes.c_uint8]
    _bf_10: int

class _SYSTEM_POOL_ZEROING_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("PoolZeroingSupportPresent", ctypes.c_uint8),
    ]
    PoolZeroingSupportPresent: int

class _IMAGE_DEBUG_DIRECTORY(ctypes.Structure):
    _fields_ = [
        ("Characteristics", ctypes.c_uint32),
        ("TimeDateStamp", ctypes.c_uint32),
        ("MajorVersion", ctypes.c_uint16),
        ("MinorVersion", ctypes.c_uint16),
        ("Type", ctypes.c_uint32),
        ("SizeOfData", ctypes.c_uint32),
        ("AddressOfRawData", ctypes.c_uint32),
        ("PointerToRawData", ctypes.c_uint32),
    ]
    Characteristics: int
    TimeDateStamp: int
    MajorVersion: int
    MinorVersion: int
    Type: int
    SizeOfData: int
    AddressOfRawData: int
    PointerToRawData: int

class _currencyfmtA(ctypes.Structure):
    _fields_ = [
        ("NumDigits", ctypes.c_uint32),
        ("LeadingZero", ctypes.c_uint32),
        ("Grouping", ctypes.c_uint32),
        ("lpDecimalSep", ctypes.c_char_p),
        ("lpThousandSep", ctypes.c_char_p),
        ("NegativeOrder", ctypes.c_uint32),
        ("PositiveOrder", ctypes.c_uint32),
        ("lpCurrencySymbol", ctypes.c_char_p),
    ]
    NumDigits: int
    LeadingZero: int
    Grouping: int
    lpDecimalSep: bytes
    lpThousandSep: bytes
    NegativeOrder: int
    PositiveOrder: int
    lpCurrencySymbol: bytes

class _SLIST_HEADER: pass

class HIMC__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _REMOTE_NAME_INFOW(ctypes.Structure):
    _fields_ = [
        ("lpUniversalName", ctypes.c_ulonglong),
        ("lpConnectionName", ctypes.c_ulonglong),
        ("lpRemainingPath", ctypes.c_ulonglong),
    ]
    lpUniversalName: int
    lpConnectionName: int
    lpRemainingPath: int

class hkCrc32StreamWriter(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # hkCrcStreamWriter<unsigned int,3988292384>
    ]
    baseclass_0: bytes

class hkCrc32StreamWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("write", ctypes.c_ulonglong),
        ("flush", ctypes.c_ulonglong),
        ("seekTellSupported", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
        ("isBuffered", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    write: int
    flush: int
    seekTellSupported: int
    seek: int
    tell: int
    isBuffered: int

class _TAPE_WMI_OPERATIONS(ctypes.Structure):
    _fields_ = [
        ("Method", ctypes.c_uint32),
        ("DataBufferSize", ctypes.c_uint32),
        ("DataBuffer", ctypes.c_void_p),
    ]
    Method: int
    DataBufferSize: int
    DataBuffer: int

class _CHAR_INFO(ctypes.Structure):
    _fields_ = [
        ("Char", ctypes.c_char * 0x2),  # _CHAR_INFO::<unnamed_type_Char>
        ("Attributes", ctypes.c_uint16),
    ]
    Char: bytes
    Attributes: int

class SERVICE_STATUS_HANDLE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _IMAGE_SYMBOL_EX(ctypes.Structure):
    _fields_ = [
        ("N", ctypes.c_char * 0x8),  # _IMAGE_SYMBOL_EX::<unnamed_type_N>
        ("Value", ctypes.c_uint32),
        ("SectionNumber", ctypes.c_int32),
        ("Type", ctypes.c_uint16),
        ("StorageClass", ctypes.c_uint8),
        ("NumberOfAuxSymbols", ctypes.c_uint8),
    ]
    N: bytes
    Value: int
    SectionNumber: int
    Type: int
    StorageClass: int
    NumberOfAuxSymbols: int

class _OSVERSIONINFOEXA(ctypes.Structure):
    _fields_ = [
        ("dwOSVersionInfoSize", ctypes.c_uint32),
        ("dwMajorVersion", ctypes.c_uint32),
        ("dwMinorVersion", ctypes.c_uint32),
        ("dwBuildNumber", ctypes.c_uint32),
        ("dwPlatformId", ctypes.c_uint32),
        ("szCSDVersion", ctypes.c_char * 0x80),  # char[128]
        ("wServicePackMajor", ctypes.c_uint16),
        ("wServicePackMinor", ctypes.c_uint16),
        ("wSuiteMask", ctypes.c_uint16),
        ("wProductType", ctypes.c_uint8),
        ("wReserved", ctypes.c_uint8),
    ]
    dwOSVersionInfoSize: int
    dwMajorVersion: int
    dwMinorVersion: int
    dwBuildNumber: int
    dwPlatformId: int
    szCSDVersion: list[ctypes.c_char]
    wServicePackMajor: int
    wServicePackMinor: int
    wSuiteMask: int
    wProductType: int
    wReserved: int

class hkCrc64StreamWriter(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # hkCrcStreamWriter<unsigned __int64,-3932672073523589310>
    ]
    baseclass_0: bytes

class hkCrc64StreamWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("write", ctypes.c_ulonglong),
        ("flush", ctypes.c_ulonglong),
        ("seekTellSupported", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
        ("isBuffered", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    write: int
    flush: int
    seekTellSupported: int
    seek: int
    tell: int
    isBuffered: int

class tagSTYLESTRUCT(ctypes.Structure):
    _fields_ = [
        ("styleOld", ctypes.c_uint32),
        ("styleNew", ctypes.c_uint32),
    ]
    styleOld: int
    styleNew: int

class hkJob(ctypes.Structure):
    _fields_ = [
        ("m_jobSubType", ctypes.c_uint8),
        ("m_jobType", ctypes.c_char * 0x1),  # hkEnum<enum hkJobType,unsigned char>
        ("m_size", ctypes.c_uint16),
        ("m_threadAffinity", ctypes.c_int16),
    ]
    m_jobSubType: int
    m_jobType: bytes
    m_size: int
    m_threadAffinity: int

class hkExternalJobProfiler(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hkExternalJobProfiler_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkExternalJobProfiler_dtor_0", ctypes.c_ulonglong),
        ("onStartJob", ctypes.c_ulonglong),
        ("onEndJob", ctypes.c_ulonglong),
    ]
    hkExternalJobProfiler_dtor_0: int
    onStartJob: int
    onEndJob: int

class hkJobQueueHwSetup(ctypes.Structure):
    _fields_ = [
        ("m_numCpuThreads", ctypes.c_int32),
        ("m_threadIdsSharingCaches", ctypes.c_char * 0x10),  # hkArray<hkArray<int,hkBuiltinContainerAllocator<0,0> >,hkBuiltinContainerAllocator<0,0> >
    ]
    m_numCpuThreads: int
    m_threadIdsSharingCaches: bytes

class HHOOK__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _DISPATCHER_CONTEXT(ctypes.Structure):
    _fields_ = [
        ("ControlPc", ctypes.c_uint64),
        ("ImageBase", ctypes.c_uint64),
        ("FunctionEntry", ctypes.c_ulonglong),
        ("EstablisherFrame", ctypes.c_uint64),
        ("TargetIp", ctypes.c_uint64),
        ("ContextRecord", ctypes.c_ulonglong),
        ("LanguageHandler", ctypes.c_ulonglong),
        ("HandlerData", ctypes.c_void_p),
        ("HistoryTable", ctypes.c_ulonglong),
        ("ScopeIndex", ctypes.c_uint32),
        ("Fill0", ctypes.c_uint32),
    ]
    ControlPc: int
    ImageBase: int
    FunctionEntry: int
    EstablisherFrame: int
    TargetIp: int
    ContextRecord: int
    LanguageHandler: int
    HandlerData: int
    HistoryTable: int
    ScopeIndex: int
    Fill0: int

class _WOW64_FLOATING_SAVE_AREA(ctypes.Structure):
    _fields_ = [
        ("ControlWord", ctypes.c_uint32),
        ("StatusWord", ctypes.c_uint32),
        ("TagWord", ctypes.c_uint32),
        ("ErrorOffset", ctypes.c_uint32),
        ("ErrorSelector", ctypes.c_uint32),
        ("DataOffset", ctypes.c_uint32),
        ("DataSelector", ctypes.c_uint32),
        ("RegisterArea", ctypes.c_uint8 * 0x50),  # unsigned __int8[80]
        ("Cr0NpxState", ctypes.c_uint32),
    ]
    ControlWord: int
    StatusWord: int
    TagWord: int
    ErrorOffset: int
    ErrorSelector: int
    DataOffset: int
    DataSelector: int
    RegisterArea: list[ctypes.c_uint8]
    Cr0NpxState: int

class _REDIRECTION_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("FunctionCount", ctypes.c_uint32),
        ("Redirections", ctypes.c_ulonglong),
    ]
    Version: int
    FunctionCount: int
    Redirections: int

class tagSERIALKEYSA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("lpszActivePort", ctypes.c_char_p),
        ("lpszPort", ctypes.c_char_p),
        ("iBaudRate", ctypes.c_uint32),
        ("iPortState", ctypes.c_uint32),
        ("iActive", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int
    lpszActivePort: bytes
    lpszPort: bytes
    iBaudRate: int
    iPortState: int
    iActive: int

class _HIBERFILE_BUCKET(ctypes.Structure):
    _fields_ = [
        ("MaxPhysicalMemory", ctypes.c_uint64),
        ("PhysicalMemoryPercent", ctypes.c_uint32 * 0x3),  # unsigned int[3]
    ]
    MaxPhysicalMemory: int
    PhysicalMemoryPercent: list[ctypes.c_uint32]

class PROCESSOR_IDLESTATE_INFO(ctypes.Structure):
    _fields_ = [
        ("TimeCheck", ctypes.c_uint32),
        ("DemotePercent", ctypes.c_uint8),
        ("PromotePercent", ctypes.c_uint8),
        ("Spare", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
    ]
    TimeCheck: int
    DemotePercent: int
    PromotePercent: int
    Spare: list[ctypes.c_uint8]

class _CREATE_PROCESS_DEBUG_INFO(ctypes.Structure):
    _fields_ = [
        ("hFile", ctypes.c_void_p),
        ("hProcess", ctypes.c_void_p),
        ("hThread", ctypes.c_void_p),
        ("lpBaseOfImage", ctypes.c_void_p),
        ("dwDebugInfoFileOffset", ctypes.c_uint32),
        ("nDebugInfoSize", ctypes.c_uint32),
        ("lpThreadLocalBase", ctypes.c_void_p),
        ("lpStartAddress", ctypes.c_ulonglong),
        ("lpImageName", ctypes.c_void_p),
        ("fUnicode", ctypes.c_uint16),
    ]
    hFile: int
    hProcess: int
    hThread: int
    lpBaseOfImage: int
    dwDebugInfoFileOffset: int
    nDebugInfoSize: int
    lpThreadLocalBase: int
    lpStartAddress: int
    lpImageName: int
    fUnicode: int

class _IMAGE_TLS_DIRECTORY32(ctypes.Structure):
    _fields_ = [
        ("StartAddressOfRawData", ctypes.c_uint32),
        ("EndAddressOfRawData", ctypes.c_uint32),
        ("AddressOfIndex", ctypes.c_uint32),
        ("AddressOfCallBacks", ctypes.c_uint32),
        ("SizeOfZeroFill", ctypes.c_uint32),
        ("___u5", ctypes.c_char * 0x4),  # $5FA4C0220B92CB02F4DA67383B5FA5AE
    ]
    StartAddressOfRawData: int
    EndAddressOfRawData: int
    AddressOfIndex: int
    AddressOfCallBacks: int
    SizeOfZeroFill: int
    ___u5: bytes

class tagACCESSTIMEOUT(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("iTimeOutMSec", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int
    iTimeOutMSec: int

class tagGCP_RESULTSW(ctypes.Structure):
    _fields_ = [
        ("lStructSize", ctypes.c_uint32),
        ("lpOutString", ctypes.c_ulonglong),
        ("lpOrder", ctypes.c_ulonglong),
        ("lpDx", ctypes.c_ulonglong),
        ("lpCaretPos", ctypes.c_ulonglong),
        ("lpClass", ctypes.c_char_p),
        ("lpGlyphs", ctypes.c_ulonglong),
        ("nGlyphs", ctypes.c_uint32),
        ("nMaxFit", ctypes.c_int32),
    ]
    lStructSize: int
    lpOutString: int
    lpOrder: int
    lpDx: int
    lpCaretPos: int
    lpClass: bytes
    lpGlyphs: int
    nGlyphs: int
    nMaxFit: int

class _IMAGE_LINENUMBER(ctypes.Structure):
    _fields_ = [
        ("Type", ctypes.c_char * 0x4),  # _IMAGE_LINENUMBER::<unnamed_type_Type>
        ("Linenumber", ctypes.c_uint16),
    ]
    Type: bytes
    Linenumber: int

class _IMAGE_THUNK_DATA32(ctypes.Structure):
    _fields_ = [
        ("u1", ctypes.c_char * 0x4),  # _IMAGE_THUNK_DATA32::<unnamed_type_u1>
    ]
    u1: bytes

class hkGpuScopeTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _IMAGE_ROM_OPTIONAL_HEADER(ctypes.Structure):
    _fields_ = [
        ("Magic", ctypes.c_uint16),
        ("MajorLinkerVersion", ctypes.c_uint8),
        ("MinorLinkerVersion", ctypes.c_uint8),
        ("SizeOfCode", ctypes.c_uint32),
        ("SizeOfInitializedData", ctypes.c_uint32),
        ("SizeOfUninitializedData", ctypes.c_uint32),
        ("AddressOfEntryPoint", ctypes.c_uint32),
        ("BaseOfCode", ctypes.c_uint32),
        ("BaseOfData", ctypes.c_uint32),
        ("BaseOfBss", ctypes.c_uint32),
        ("GprMask", ctypes.c_uint32),
        ("CprMask", ctypes.c_uint32 * 0x4),  # unsigned int[4]
        ("GpValue", ctypes.c_uint32),
    ]
    Magic: int
    MajorLinkerVersion: int
    MinorLinkerVersion: int
    SizeOfCode: int
    SizeOfInitializedData: int
    SizeOfUninitializedData: int
    AddressOfEntryPoint: int
    BaseOfCode: int
    BaseOfData: int
    BaseOfBss: int
    GprMask: int
    CprMask: list[ctypes.c_uint32]
    GpValue: int

class _SYSTEM_PROCESSOR_CYCLE_TIME_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("CycleTime", ctypes.c_uint64),
    ]
    CycleTime: int

class _SERVICE_TRIGGER_SPECIFIC_DATA_ITEM(ctypes.Structure):
    _fields_ = [
        ("dwDataType", ctypes.c_uint32),
        ("cbData", ctypes.c_uint32),
        ("pData", ctypes.c_ulonglong),
    ]
    dwDataType: int
    cbData: int
    pData: int

class _SCRUB_DATA_OUTPUT(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("Status", ctypes.c_uint32),
        ("ErrorFileOffset", ctypes.c_uint64),
        ("ErrorLength", ctypes.c_uint64),
        ("NumberOfBytesRepaired", ctypes.c_uint64),
        ("NumberOfBytesFailed", ctypes.c_uint64),
        ("InternalFileReference", ctypes.c_uint64),
        ("ResumeContextLength", ctypes.c_uint16),
        ("ParityExtentDataOffset", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint32 * 0x9),  # unsigned int[9]
        ("NumberOfMetadataBytesProcessed", ctypes.c_uint64),
        ("NumberOfDataBytesProcessed", ctypes.c_uint64),
        ("TotalNumberOfMetadataBytesInUse", ctypes.c_uint64),
        ("TotalNumberOfDataBytesInUse", ctypes.c_uint64),
        ("ResumeContext", ctypes.c_uint8 * 0x330),  # unsigned __int8[816]
    ]
    Size: int
    Flags: int
    Status: int
    ErrorFileOffset: int
    ErrorLength: int
    NumberOfBytesRepaired: int
    NumberOfBytesFailed: int
    InternalFileReference: int
    ResumeContextLength: int
    ParityExtentDataOffset: int
    Reserved: list[ctypes.c_uint32]
    NumberOfMetadataBytesProcessed: int
    NumberOfDataBytesProcessed: int
    TotalNumberOfMetadataBytesInUse: int
    TotalNumberOfDataBytesInUse: int
    ResumeContext: list[ctypes.c_uint8]

class _QUERY_SERVICE_CONFIGW(ctypes.Structure):
    _fields_ = [
        ("dwServiceType", ctypes.c_uint32),
        ("dwStartType", ctypes.c_uint32),
        ("dwErrorControl", ctypes.c_uint32),
        ("lpBinaryPathName", ctypes.c_ulonglong),
        ("lpLoadOrderGroup", ctypes.c_ulonglong),
        ("dwTagId", ctypes.c_uint32),
        ("lpDependencies", ctypes.c_ulonglong),
        ("lpServiceStartName", ctypes.c_ulonglong),
        ("lpDisplayName", ctypes.c_ulonglong),
    ]
    dwServiceType: int
    dwStartType: int
    dwErrorControl: int
    lpBinaryPathName: int
    lpLoadOrderGroup: int
    dwTagId: int
    lpDependencies: int
    lpServiceStartName: int
    lpDisplayName: int

class _ENCLAVE_INIT_INFO_SGX(ctypes.Structure):
    _fields_ = [
        ("SigStruct", ctypes.c_uint8 * 0x710),  # unsigned __int8[1808]
        ("Reserved1", ctypes.c_uint8 * 0xF0),  # unsigned __int8[240]
        ("EInitToken", ctypes.c_uint8 * 0x130),  # unsigned __int8[304]
        ("Reserved2", ctypes.c_uint8 * 0x6D0),  # unsigned __int8[1744]
    ]
    SigStruct: list[ctypes.c_uint8]
    Reserved1: list[ctypes.c_uint8]
    EInitToken: list[ctypes.c_uint8]
    Reserved2: list[ctypes.c_uint8]

class _PROCESSOR_NUMBER(ctypes.Structure):
    _fields_ = [
        ("Group", ctypes.c_uint16),
        ("Number", ctypes.c_uint8),
        ("Reserved", ctypes.c_uint8),
    ]
    Group: int
    Number: int
    Reserved: int

class _SINGLE_LIST_ENTRY(ctypes.Structure):
    _fields_ = [
        ("Next", ctypes.c_ulonglong),
    ]
    Next: int

class pvalueA(ctypes.Structure):
    _fields_ = [
        ("pv_valuename", ctypes.c_char_p),
        ("pv_valuelen", ctypes.c_int32),
        ("pv_value_context", ctypes.c_void_p),
        ("pv_type", ctypes.c_uint32),
    ]
    pv_valuename: bytes
    pv_valuelen: int
    pv_value_context: int
    pv_type: int

class cTkMetaDataXMLFunctionLookup(ctypes.Structure):
    _fields_ = [
        ("mName", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mWriteFunction", ctypes.c_ulonglong),
        ("mReadFunction", ctypes.c_ulonglong),
        ("mSaveFunction", ctypes.c_ulonglong),
    ]
    mName: bytes
    mWriteFunction: int
    mReadFunction: int
    mSaveFunction: int

class _IMAGE_ARM64_RUNTIME_FUNCTION_ENTRY(ctypes.Structure):
    _fields_ = [
        ("BeginAddress", ctypes.c_uint32),
        ("___u1", ctypes.c_char * 0x4),  # $951C116A5745BC479F03785D58C7D37F
    ]
    BeginAddress: int
    ___u1: bytes

class _EXCEPTION_REGISTRATION_RECORD(ctypes.Structure):
    _fields_ = [
        ("Next", ctypes.c_ulonglong),
        ("Handler", ctypes.c_ulonglong),
    ]
    Next: int
    Handler: int

class hkJsonDocument_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class HWINEVENTHOOK__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class tagTOUCHINPUT(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("hSource", ctypes.c_void_p),
        ("dwID", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("dwMask", ctypes.c_uint32),
        ("dwTime", ctypes.c_uint32),
        ("dwExtraInfo", ctypes.c_uint64),
        ("cxContact", ctypes.c_uint32),
        ("cyContact", ctypes.c_uint32),
    ]
    x: int
    y: int
    hSource: int
    dwID: int
    dwFlags: int
    dwMask: int
    dwTime: int
    dwExtraInfo: int
    cxContact: int
    cyContact: int

class cTkMcQmcLFSR(ctypes.Structure):
    def cTkMcQmcLFSR(): pass

    _fields_ = [
        ("muiPeriod", ctypes.c_uint32),
        ("mauiF", ctypes.c_char * 0x80),  # std::array<unsigned int,32>
    ]
    muiPeriod: int
    mauiF: bytes

class _devicemodeW(ctypes.Structure):
    _fields_ = [
        ("dmDeviceName", ctypes.c_wchar * 0x20),  # wchar_t[32]
        ("dmSpecVersion", ctypes.c_uint16),
        ("dmDriverVersion", ctypes.c_uint16),
        ("dmSize", ctypes.c_uint16),
        ("dmDriverExtra", ctypes.c_uint16),
        ("dmFields", ctypes.c_uint32),
        ("___u6", ctypes.c_char * 0x8),  # $BB358C3A325381C626105C9C85EB3F4A
        ("___u7", ctypes.c_char * 0x4),  # $DEE03DF9F4D6FE316E4B77CDE5AE0A90
        ("___u8", ctypes.c_char * 0x4),  # $6B826ED569E6FCAD26E63896A7C9594E
        ("dmColor", ctypes.c_int16),
        ("dmDuplex", ctypes.c_int16),
        ("dmYResolution", ctypes.c_int16),
        ("dmTTOption", ctypes.c_int16),
        ("dmCollate", ctypes.c_int16),
        ("dmFormName", ctypes.c_wchar * 0x20),  # wchar_t[32]
        ("dmLogPixels", ctypes.c_uint16),
        ("dmBitsPerPel", ctypes.c_uint32),
        ("dmPelsWidth", ctypes.c_uint32),
        ("dmPelsHeight", ctypes.c_uint32),
        ("___u19", ctypes.c_char * 0x4),  # $309666D137CBB19809FA597AD348B415
        ("dmDisplayFrequency", ctypes.c_uint32),
        ("dmICMMethod", ctypes.c_uint32),
        ("dmICMIntent", ctypes.c_uint32),
        ("dmMediaType", ctypes.c_uint32),
        ("dmDitherType", ctypes.c_uint32),
        ("dmReserved1", ctypes.c_uint32),
        ("dmReserved2", ctypes.c_uint32),
        ("dmPanningWidth", ctypes.c_uint32),
        ("dmPanningHeight", ctypes.c_uint32),
    ]
    dmDeviceName: list[ctypes.c_wchar]
    dmSpecVersion: int
    dmDriverVersion: int
    dmSize: int
    dmDriverExtra: int
    dmFields: int
    ___u6: bytes
    ___u7: bytes
    ___u8: bytes
    dmColor: int
    dmDuplex: int
    dmYResolution: int
    dmTTOption: int
    dmCollate: int
    dmFormName: list[ctypes.c_wchar]
    dmLogPixels: int
    dmBitsPerPel: int
    dmPelsWidth: int
    dmPelsHeight: int
    ___u19: bytes
    dmDisplayFrequency: int
    dmICMMethod: int
    dmICMIntent: int
    dmMediaType: int
    dmDitherType: int
    dmReserved1: int
    dmReserved2: int
    dmPanningWidth: int
    dmPanningHeight: int

class _PROCESS_MITIGATION_SYSTEM_CALL_DISABLE_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $467ACA7024A5C8FBB8565CD2BC87B971
    ]
    ___u0: bytes

class tagENHMETARECORD(ctypes.Structure):
    _fields_ = [
        ("iType", ctypes.c_uint32),
        ("nSize", ctypes.c_uint32),
        ("dParm", ctypes.c_uint32 * 0x1),  # unsigned int[1]
    ]
    iType: int
    nSize: int
    dParm: list[ctypes.c_uint32]

class tagGESTURECONFIG(ctypes.Structure):
    _fields_ = [
        ("dwID", ctypes.c_uint32),
        ("dwWant", ctypes.c_uint32),
        ("dwBlock", ctypes.c_uint32),
    ]
    dwID: int
    dwWant: int
    dwBlock: int

class _TOKEN_MANDATORY_POLICY(ctypes.Structure):
    _fields_ = [
        ("Policy", ctypes.c_uint32),
    ]
    Policy: int

class APP_LOCAL_DEVICE_ID(ctypes.Structure):
    _fields_ = [
        ("value", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
    ]
    value: list[ctypes.c_uint8]

class _IMAGE_HOT_PATCH_BASE(ctypes.Structure):
    _fields_ = [
        ("SequenceNumber", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("OriginalTimeDateStamp", ctypes.c_uint32),
        ("OriginalCheckSum", ctypes.c_uint32),
        ("CodeIntegrityInfo", ctypes.c_uint32),
        ("CodeIntegritySize", ctypes.c_uint32),
        ("PatchTable", ctypes.c_uint32),
        ("BufferOffset", ctypes.c_uint32),
    ]
    SequenceNumber: int
    Flags: int
    OriginalTimeDateStamp: int
    OriginalCheckSum: int
    CodeIntegrityInfo: int
    CodeIntegritySize: int
    PatchTable: int
    BufferOffset: int

class HGLRC__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _PROCESS_MITIGATION_FONT_DISABLE_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $722ED099FEA1D54102C88B88C96F7F1B
    ]
    ___u0: bytes

class hkJsonDocumentWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkJsonWriter_dtor_0", ctypes.c_ulonglong),
        ("beginObject", ctypes.c_ulonglong),
        ("beginArray", ctypes.c_ulonglong),
        ("end", ctypes.c_ulonglong),
        ("writeString", ctypes.c_ulonglong),
        ("writeNumber", ctypes.c_ulonglong),
        ("writeBoolean", ctypes.c_ulonglong),
        ("writeNull", ctypes.c_ulonglong),
        ("writeInteger", ctypes.c_ulonglong),
        ("writeJsonNode", ctypes.c_ulonglong),
        ("writeJsonString", ctypes.c_ulonglong),
        ("writeBinary", ctypes.c_ulonglong),
    ]
    hkJsonWriter_dtor_0: int
    beginObject: int
    beginArray: int
    end: int
    writeString: int
    writeNumber: int
    writeBoolean: int
    writeNull: int
    writeInteger: int
    writeJsonNode: int
    writeJsonString: int
    writeBinary: int

class _ldouble_val: pass

class _wfinddata64i32_t(ctypes.Structure):
    _fields_ = [
        ("attrib", ctypes.c_uint32),
        ("time_create", ctypes.c_int64),
        ("time_access", ctypes.c_int64),
        ("time_write", ctypes.c_int64),
        ("size", ctypes.c_uint32),
        ("name", ctypes.c_wchar * 0x104),  # wchar_t[260]
    ]
    attrib: int
    time_create: int
    time_access: int
    time_write: int
    size: int
    name: list[ctypes.c_wchar]

class hkJsonStringBuf_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkJsonWriter_dtor_0", ctypes.c_ulonglong),
        ("beginObject", ctypes.c_ulonglong),
        ("beginArray", ctypes.c_ulonglong),
        ("end", ctypes.c_ulonglong),
        ("writeString", ctypes.c_ulonglong),
        ("writeNumber", ctypes.c_ulonglong),
        ("writeBoolean", ctypes.c_ulonglong),
        ("writeNull", ctypes.c_ulonglong),
        ("writeInteger", ctypes.c_ulonglong),
        ("writeJsonNode", ctypes.c_ulonglong),
        ("writeJsonString", ctypes.c_ulonglong),
        ("writeBinary", ctypes.c_ulonglong),
    ]
    hkJsonWriter_dtor_0: int
    beginObject: int
    beginArray: int
    end: int
    writeString: int
    writeNumber: int
    writeBoolean: int
    writeNull: int
    writeInteger: int
    writeJsonNode: int
    writeJsonString: int
    writeBinary: int

class _timespec64(ctypes.Structure):
    _fields_ = [
        ("tv_sec", ctypes.c_int64),
        ("tv_nsec", ctypes.c_int32),
    ]
    tv_sec: int
    tv_nsec: int

class timespec(ctypes.Structure):
    _fields_ = [
        ("tv_sec", ctypes.c_int64),
        ("tv_nsec", ctypes.c_int32),
    ]
    tv_sec: int
    tv_nsec: int

class stat(ctypes.Structure):
    _fields_ = [
        ("st_dev", ctypes.c_uint32),
        ("st_ino", ctypes.c_uint16),
        ("st_mode", ctypes.c_uint16),
        ("st_nlink", ctypes.c_int16),
        ("st_uid", ctypes.c_int16),
        ("st_gid", ctypes.c_int16),
        ("st_rdev", ctypes.c_uint32),
        ("st_size", ctypes.c_int32),
        ("st_atime", ctypes.c_int64),
        ("st_mtime", ctypes.c_int64),
        ("st_ctime", ctypes.c_int64),
    ]
    st_dev: int
    st_ino: int
    st_mode: int
    st_nlink: int
    st_uid: int
    st_gid: int
    st_rdev: int
    st_size: int
    st_atime: int
    st_mtime: int
    st_ctime: int

class hkMonitorStreamContainer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkSimdReal_AdvancedInterface(ctypes.Structure):
    pass

class hkMonitorStreamContainerCache_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkOffsetBitFieldLocal(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x30),  # hkBitFieldBase<hkOffsetBitFieldStorage<hkLocalArray<unsigned int,hkBuiltinContainerAllocator<0,0> > > >
    ]
    baseclass_0: bytes

class _s__CatchableTypeArray(ctypes.Structure):
    _fields_ = [
        ("nCatchableTypes", ctypes.c_int32),
        ("arrayOfCatchableTypes", ctypes.c_ulonglong),  # _s__CatchableType *[]
    ]
    nCatchableTypes: int
    arrayOfCatchableTypes: list[ctypes.c_ulonglong]

class _s__ThrowInfo(ctypes.Structure):
    _fields_ = [
        ("attributes", ctypes.c_uint32),
        ("pmfnUnwind", ctypes.c_ulonglong),
        ("pForwardCompat", ctypes.c_ulonglong),
        ("pCatchableTypeArray", ctypes.c_ulonglong),
    ]
    attributes: int
    pmfnUnwind: int
    pForwardCompat: int
    pCatchableTypeArray: int

class hkWaitableStreamReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("skip", ctypes.c_ulonglong),
        ("peek", ctypes.c_ulonglong),
        ("isSeekTellSupported", ctypes.c_ulonglong),
        ("isInplace", ctypes.c_ulonglong),
        ("getWaitHandle", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    read: int
    skip: int
    peek: int
    isSeekTellSupported: int
    isInplace: int
    getWaitHandle: int

class _ldiv_t(ctypes.Structure):
    _fields_ = [
        ("quot", ctypes.c_int32),
        ("rem", ctypes.c_int32),
    ]
    quot: int
    rem: int

class _double_val: pass

class _LDOUBLE(ctypes.Structure):
    _fields_ = [
        ("ld", ctypes.c_uint8 * 0xA),  # unsigned __int8[10]
    ]
    ld: list[ctypes.c_uint8]

class _LONGDOUBLE(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_double),
    ]
    x: float

class hkThreadPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("processJobQueue", ctypes.c_ulonglong),
        ("processTaskQueue", ctypes.c_ulonglong),
        ("isProcessing", ctypes.c_ulonglong),
        ("getProcessingJobQueue", ctypes.c_ulonglong),
        ("getProcessingTaskQueue", ctypes.c_ulonglong),
        ("waitForCompletion", ctypes.c_ulonglong),
        ("getNumThreads", ctypes.c_ulonglong),
        ("setNumThreads", ctypes.c_ulonglong),
        ("appendTimerData", ctypes.c_char * 0x8),  # void (__fastcall *)(hkThreadPool *this, hkArray<hkTimerData,hkBuiltinContainerAllocator<0,0> > *)
        ("clearTimerData", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    processJobQueue: int
    processTaskQueue: int
    isProcessing: int
    getProcessingJobQueue: int
    getProcessingTaskQueue: int
    waitForCompletion: int
    getNumThreads: int
    setNumThreads: int
    appendTimerData: bytes
    clearTimerData: int

class _float_val: pass

class _float_const: pass

class hkDisableError(ctypes.Structure):
    def hkDisableError(): pass

    def tilde_hkDisableError(): pass

    _fields_ = [
        ("m_id", ctypes.c_int32),
        ("m_wasEnabled", ctypes.c_ubyte),
    ]
    m_id: int
    m_wasEnabled: bool

class hkCpuThreadPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("processJobQueue", ctypes.c_ulonglong),
        ("processTaskQueue", ctypes.c_ulonglong),
        ("isProcessing", ctypes.c_ulonglong),
        ("getProcessingJobQueue", ctypes.c_ulonglong),
        ("getProcessingTaskQueue", ctypes.c_ulonglong),
        ("waitForCompletion", ctypes.c_ulonglong),
        ("getNumThreads", ctypes.c_ulonglong),
        ("setNumThreads", ctypes.c_ulonglong),
        ("appendTimerData", ctypes.c_char * 0x8),  # void (__fastcall *)(hkThreadPool *this, hkArray<hkTimerData,hkBuiltinContainerAllocator<0,0> > *)
        ("clearTimerData", ctypes.c_ulonglong),
        ("threadMain", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    processJobQueue: int
    processTaskQueue: int
    isProcessing: int
    getProcessingJobQueue: int
    getProcessingTaskQueue: int
    waitForCompletion: int
    getNumThreads: int
    setNumThreads: int
    appendTimerData: bytes
    clearTimerData: int
    threadMain: int

class _wfinddata32i64_t(ctypes.Structure):
    _fields_ = [
        ("attrib", ctypes.c_uint32),
        ("time_create", ctypes.c_int32),
        ("time_access", ctypes.c_int32),
        ("time_write", ctypes.c_int32),
        ("size", ctypes.c_int64),
        ("name", ctypes.c_wchar * 0x104),  # wchar_t[260]
    ]
    attrib: int
    time_create: int
    time_access: int
    time_write: int
    size: int
    name: list[ctypes.c_wchar]

class xtime(ctypes.Structure):
    _fields_ = [
        ("sec", ctypes.c_int64),
        ("nsec", ctypes.c_int32),
    ]
    sec: int
    nsec: int

class cEgSceneNodeTemplate(ctypes.Structure):
    def cEgSceneNodeTemplate(): pass

    def tilde_cEgSceneNodeTemplate(): pass

    def SetName(): pass

    def GetTransform(): pass

    def SetDescriptor(): pass

    def GatherMaterials(): pass

    def GatherData(): pass

    def GetBoundingBox(): pass

    def IsGeometryStreamedIn(): pass

    def GatherNonReferenceData(): pass

    def GatherModels(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("miType", ctypes.c_int32),
        ("muNameHash", ctypes.c_uint32),
        ("msName", ctypes.c_char * 0x8),  # cTkSharedPtr<std::string >
        ("mpDescriptor", ctypes.c_char * 0x8),  # cTkSharedPtr<cTkResourceDescriptor const >
        ("mTranslation", common.Vector3f),
        ("mRotation", common.Vector3f),
        ("mScale", common.Vector3f),
        ("mpAttachment", ctypes.c_ulonglong),
        ("mpParent", ctypes.c_ulonglong),
        ("mHandle", ctypes.c_uint32),
        ("mChildren", ctypes.c_char * 0x18),  # std::vector<cEgSceneNodeTemplate *,TkSTLAllocatorShim<cEgSceneNodeTemplate *,8,-1> >
    ]
    __vftable: int
    miType: int
    muNameHash: int
    msName: bytes
    mpDescriptor: bytes
    mTranslation: common.Vector3f
    mRotation: common.Vector3f
    mScale: common.Vector3f
    mpAttachment: int
    mpParent: int
    mHandle: int
    mChildren: bytes

class cTkAABB(ctypes.Structure):
    def CreateFromOOBB(): pass

    def IsPositionInBox(): pass

    def IsSphereOverlappingBox(): pass

    _fields_ = [
        ("mMin", common.Vector3f),
        ("mMax", common.Vector3f),
    ]
    mMin: common.Vector3f
    mMax: common.Vector3f

class cEgResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cEgBoundingBox(ctypes.Structure):
    _fields_ = [
        ("mMin", common.Vector3f),
        ("mMax", common.Vector3f),
    ]
    mMin: common.Vector3f
    mMax: common.Vector3f

class cEgGeometryAABBCache(ctypes.Structure):
    def cEgGeometryAABBCache(): pass

    def tilde_cEgGeometryAABBCache(): pass

    _fields_ = [
        ("maBoundingBoxes", ctypes.c_char * 0x60),  # cTkStackVector<cEgBoundingBox,1,-1>
        ("maDirty", ctypes.c_char * 0x38),  # cTkStackVector<bool,1,-1>
    ]
    maBoundingBoxes: bytes
    maDirty: bytes

class cTkWindEigenvalue(ctypes.Structure):
    _fields_ = [
        ("mfEigenValReal", ctypes.c_float),
        ("mfEigenValImag", ctypes.c_float),
        ("mfEigenVecReal", ctypes.c_ulonglong),
        ("mfEigenVecImag", ctypes.c_ulonglong),
        ("mfDotX", ctypes.c_float),
        ("mfDotY", ctypes.c_float),
        ("mfDotZ", ctypes.c_float),
        ("mfDotXMassWeighted", ctypes.c_float),
        ("mfDotYMassWeighted", ctypes.c_float),
        ("mfDotZMassWeighted", ctypes.c_float),
    ]
    mfEigenValReal: float
    mfEigenValImag: float
    mfEigenVecReal: int
    mfEigenVecImag: int
    mfDotX: float
    mfDotY: float
    mfDotZ: float
    mfDotXMassWeighted: float
    mfDotYMassWeighted: float
    mfDotZMassWeighted: float

class AnimTransform(ctypes.Structure):
    _fields_ = [
        ("mNodeScale", common.Vector3f),
        ("mNodeRotationQuat", common.Quaternion),
        ("mNodeTranslation", common.Vector3f),
        ("mbNodeUpdated", ctypes.c_ubyte),
    ]
    mNodeScale: common.Vector3f
    mNodeRotationQuat: common.Quaternion
    mNodeTranslation: common.Vector3f
    mbNodeUpdated: bool

class cEgMorphDiff(ctypes.Structure):
    _fields_ = [
        ("muVertexIndex", ctypes.c_uint32),
        ("mPosDiff", common.Vector3f),
        ("mNormDiff", common.Vector3f),
        ("mTanDiff", common.Vector3f),
        ("mBitanDiff", common.Vector3f),
    ]
    muVertexIndex: int
    mPosDiff: common.Vector3f
    mNormDiff: common.Vector3f
    mTanDiff: common.Vector3f
    mBitanDiff: common.Vector3f

class cEgSceneNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cEgNodeAttachment(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def GetFromNode(): pass

    def AttachToNode(): pass

    def tilde_cEgNodeAttachment(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mNode", ctypes.c_uint32),
    ]
    __vftable: int
    mNode: int

class cEgNodeAttachment_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgNodeAttachment_dtor_0", ctypes.c_ulonglong),
        ("OnUpdate", ctypes.c_ulonglong),
    ]
    cEgNodeAttachment_dtor_0: int
    OnUpdate: int

class cTkShaderBase(ctypes.Structure):
    def FinishAsyncLoadFromFile(): pass

    def LoadComputeFromFile(): pass

    def LoadFromFile(): pass

    def StartAsyncLoadFromFile(): pass

    def AllocateUniformBuffersAndSetSRT(): pass

    def Delete(): pass

    def GenerateUniformBufferList(): pass

    def LoadComputeFromMemory(): pass

    def LoadFromMemory(): pass

    def ResolveUniformHandle(): pass


class cTkShader_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkShader_dtor_0", ctypes.c_ulonglong),
        ("CleanUp", ctypes.c_ulonglong),
    ]
    cTkShader_dtor_0: int
    CleanUp: int

class cTkVertexAttributeMapping(ctypes.Structure):
    _fields_ = [
        ("maAttributeLocs", ctypes.c_int16 * 0x13),  # __int16[19]
    ]
    maAttributeLocs: list[ctypes.c_int16]

class TkShaderModule(ctypes.Structure):
    _fields_ = [
        ("mModule", ctypes.c_ulonglong),
        ("pShaderBytecode", ctypes.c_void_p),
        ("BytecodeLength", ctypes.c_uint64),
    ]
    mModule: int
    pShaderBytecode: int
    BytecodeLength: int

class TkShaderBufferHandle(ctypes.Structure):
    def TkShaderBufferHandle(): pass

    _fields_ = [
        ("muSize", ctypes.c_uint16),
        ("muOffset", ctypes.c_uint16),
        ("miFragmentSlot", ctypes.c_int8),
        ("_bf_5", ctypes.c_int8),
    ]
    muSize: int
    muOffset: int
    miFragmentSlot: int
    _bf_5: int

class TkShaderConstHandle(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $89976495D44E769619D5B113C38729D4
        ("muOffset", ctypes.c_uint16),
        ("mbIsCustomPerMaterial", ctypes.c_ubyte),
        ("meUniformBufferMask", ctypes.c_uint8),
    ]
    ___u0: bytes
    muOffset: int
    mbIsCustomPerMaterial: bool
    meUniformBufferMask: int

class cTkShaderAsyncLoadData(ctypes.Structure):
    def Cleanup(): pass

    _fields_ = [
        ("mVertexShaderOp", ctypes.c_int32),
        ("mFragmentShaderOp", ctypes.c_int32),
        ("mHullShaderOp", ctypes.c_int32),
        ("mDomainShaderOp", ctypes.c_int32),
        ("mGeometryShaderOp", ctypes.c_int32),
        ("mComputeShaderOp", ctypes.c_int32),
        ("mMeshShaderOp", ctypes.c_int32),
        ("mxCombMask", ctypes.c_uint64),
        ("mbShadingRate", ctypes.c_ubyte),
        ("mbPersistReflectionData", ctypes.c_ubyte),
    ]
    mVertexShaderOp: int
    mFragmentShaderOp: int
    mHullShaderOp: int
    mDomainShaderOp: int
    mGeometryShaderOp: int
    mComputeShaderOp: int
    mMeshShaderOp: int
    mxCombMask: int
    mbShadingRate: bool
    mbPersistReflectionData: bool

class VkSubpassDependency(ctypes.Structure):
    _fields_ = [
        ("srcSubpass", ctypes.c_uint32),
        ("dstSubpass", ctypes.c_uint32),
        ("srcStageMask", ctypes.c_uint32),
        ("dstStageMask", ctypes.c_uint32),
        ("srcAccessMask", ctypes.c_uint32),
        ("dstAccessMask", ctypes.c_uint32),
        ("dependencyFlags", ctypes.c_uint32),
    ]
    srcSubpass: int
    dstSubpass: int
    srcStageMask: int
    dstStageMask: int
    srcAccessMask: int
    dstAccessMask: int
    dependencyFlags: int

class VkOffset2D(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
    ]
    x: int
    y: int

class VkExtent2D(ctypes.Structure):
    _fields_ = [
        ("width", ctypes.c_uint32),
        ("height", ctypes.c_uint32),
    ]
    width: int
    height: int

class VkClearColorValue: pass

class VkClearDepthStencilValue(ctypes.Structure):
    _fields_ = [
        ("depth", ctypes.c_float),
        ("stencil", ctypes.c_uint32),
    ]
    depth: float
    stencil: int

class VkClearValue: pass

class cTkTextureStreamFuncs(ctypes.Structure):
    _fields_ = [
        ("mStreamFunc", ctypes.c_char * 0x8),  # bool (__fastcall *)(TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, void *, int, int, int, int)
        ("mStreamDeleteFunc", ctypes.c_ulonglong),
        ("mAbortBakeFunc", ctypes.c_ulonglong),
        ("mEvictFunc", ctypes.c_ulonglong),
        ("mBeginProbeFunc", ctypes.c_char * 0x8),  # bool (__fastcall *)(TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, void *, int, int)
        ("mCompleteProbeFunc", ctypes.c_ulonglong),
        ("mGatherProbeFunc", ctypes.c_char * 0x8),  # __int64 (__fastcall *)(TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, void *, TkMipReportBuffer *, int *, int, int, bool *, bool *, bool *)
        ("mbIsVirtualTexture", ctypes.c_ubyte),
    ]
    mStreamFunc: bytes
    mStreamDeleteFunc: int
    mAbortBakeFunc: int
    mEvictFunc: int
    mBeginProbeFunc: bytes
    mCompleteProbeFunc: int
    mGatherProbeFunc: bytes
    mbIsVirtualTexture: bool

class VkExtent3D(ctypes.Structure):
    _fields_ = [
        ("width", ctypes.c_uint32),
        ("height", ctypes.c_uint32),
        ("depth", ctypes.c_uint32),
    ]
    width: int
    height: int
    depth: int

class VkAllocationCallbacks(ctypes.Structure):
    _fields_ = [
        ("pUserData", ctypes.c_void_p),
        ("pfnAllocation", ctypes.c_ulonglong),
        ("pfnReallocation", ctypes.c_ulonglong),
        ("pfnFree", ctypes.c_ulonglong),
        ("pfnInternalAllocation", ctypes.c_ulonglong),
        ("pfnInternalFree", ctypes.c_ulonglong),
    ]
    pUserData: int
    pfnAllocation: int
    pfnReallocation: int
    pfnFree: int
    pfnInternalAllocation: int
    pfnInternalFree: int

class VkPhysicalDeviceLimits(ctypes.Structure):
    _fields_ = [
        ("maxImageDimension1D", ctypes.c_uint32),
        ("maxImageDimension2D", ctypes.c_uint32),
        ("maxImageDimension3D", ctypes.c_uint32),
        ("maxImageDimensionCube", ctypes.c_uint32),
        ("maxImageArrayLayers", ctypes.c_uint32),
        ("maxTexelBufferElements", ctypes.c_uint32),
        ("maxUniformBufferRange", ctypes.c_uint32),
        ("maxStorageBufferRange", ctypes.c_uint32),
        ("maxPushConstantsSize", ctypes.c_uint32),
        ("maxMemoryAllocationCount", ctypes.c_uint32),
        ("maxSamplerAllocationCount", ctypes.c_uint32),
        ("bufferImageGranularity", ctypes.c_uint64),
        ("sparseAddressSpaceSize", ctypes.c_uint64),
        ("maxBoundDescriptorSets", ctypes.c_uint32),
        ("maxPerStageDescriptorSamplers", ctypes.c_uint32),
        ("maxPerStageDescriptorUniformBuffers", ctypes.c_uint32),
        ("maxPerStageDescriptorStorageBuffers", ctypes.c_uint32),
        ("maxPerStageDescriptorSampledImages", ctypes.c_uint32),
        ("maxPerStageDescriptorStorageImages", ctypes.c_uint32),
        ("maxPerStageDescriptorInputAttachments", ctypes.c_uint32),
        ("maxPerStageResources", ctypes.c_uint32),
        ("maxDescriptorSetSamplers", ctypes.c_uint32),
        ("maxDescriptorSetUniformBuffers", ctypes.c_uint32),
        ("maxDescriptorSetUniformBuffersDynamic", ctypes.c_uint32),
        ("maxDescriptorSetStorageBuffers", ctypes.c_uint32),
        ("maxDescriptorSetStorageBuffersDynamic", ctypes.c_uint32),
        ("maxDescriptorSetSampledImages", ctypes.c_uint32),
        ("maxDescriptorSetStorageImages", ctypes.c_uint32),
        ("maxDescriptorSetInputAttachments", ctypes.c_uint32),
        ("maxVertexInputAttributes", ctypes.c_uint32),
        ("maxVertexInputBindings", ctypes.c_uint32),
        ("maxVertexInputAttributeOffset", ctypes.c_uint32),
        ("maxVertexInputBindingStride", ctypes.c_uint32),
        ("maxVertexOutputComponents", ctypes.c_uint32),
        ("maxTessellationGenerationLevel", ctypes.c_uint32),
        ("maxTessellationPatchSize", ctypes.c_uint32),
        ("maxTessellationControlPerVertexInputComponents", ctypes.c_uint32),
        ("maxTessellationControlPerVertexOutputComponents", ctypes.c_uint32),
        ("maxTessellationControlPerPatchOutputComponents", ctypes.c_uint32),
        ("maxTessellationControlTotalOutputComponents", ctypes.c_uint32),
        ("maxTessellationEvaluationInputComponents", ctypes.c_uint32),
        ("maxTessellationEvaluationOutputComponents", ctypes.c_uint32),
        ("maxGeometryShaderInvocations", ctypes.c_uint32),
        ("maxGeometryInputComponents", ctypes.c_uint32),
        ("maxGeometryOutputComponents", ctypes.c_uint32),
        ("maxGeometryOutputVertices", ctypes.c_uint32),
        ("maxGeometryTotalOutputComponents", ctypes.c_uint32),
        ("maxFragmentInputComponents", ctypes.c_uint32),
        ("maxFragmentOutputAttachments", ctypes.c_uint32),
        ("maxFragmentDualSrcAttachments", ctypes.c_uint32),
        ("maxFragmentCombinedOutputResources", ctypes.c_uint32),
        ("maxComputeSharedMemorySize", ctypes.c_uint32),
        ("maxComputeWorkGroupCount", ctypes.c_uint32 * 0x3),  # unsigned int[3]
        ("maxComputeWorkGroupInvocations", ctypes.c_uint32),
        ("maxComputeWorkGroupSize", ctypes.c_uint32 * 0x3),  # unsigned int[3]
        ("subPixelPrecisionBits", ctypes.c_uint32),
        ("subTexelPrecisionBits", ctypes.c_uint32),
        ("mipmapPrecisionBits", ctypes.c_uint32),
        ("maxDrawIndexedIndexValue", ctypes.c_uint32),
        ("maxDrawIndirectCount", ctypes.c_uint32),
        ("maxSamplerLodBias", ctypes.c_float),
        ("maxSamplerAnisotropy", ctypes.c_float),
        ("maxViewports", ctypes.c_uint32),
        ("maxViewportDimensions", ctypes.c_uint32 * 0x2),  # unsigned int[2]
        ("viewportBoundsRange", ctypes.c_float * 0x2),  # float[2]
        ("viewportSubPixelBits", ctypes.c_uint32),
        ("minMemoryMapAlignment", ctypes.c_uint64),
        ("minTexelBufferOffsetAlignment", ctypes.c_uint64),
        ("minUniformBufferOffsetAlignment", ctypes.c_uint64),
        ("minStorageBufferOffsetAlignment", ctypes.c_uint64),
        ("minTexelOffset", ctypes.c_int32),
        ("maxTexelOffset", ctypes.c_uint32),
        ("minTexelGatherOffset", ctypes.c_int32),
        ("maxTexelGatherOffset", ctypes.c_uint32),
        ("minInterpolationOffset", ctypes.c_float),
        ("maxInterpolationOffset", ctypes.c_float),
        ("subPixelInterpolationOffsetBits", ctypes.c_uint32),
        ("maxFramebufferWidth", ctypes.c_uint32),
        ("maxFramebufferHeight", ctypes.c_uint32),
        ("maxFramebufferLayers", ctypes.c_uint32),
        ("framebufferColorSampleCounts", ctypes.c_uint32),
        ("framebufferDepthSampleCounts", ctypes.c_uint32),
        ("framebufferStencilSampleCounts", ctypes.c_uint32),
        ("framebufferNoAttachmentsSampleCounts", ctypes.c_uint32),
        ("maxColorAttachments", ctypes.c_uint32),
        ("sampledImageColorSampleCounts", ctypes.c_uint32),
        ("sampledImageIntegerSampleCounts", ctypes.c_uint32),
        ("sampledImageDepthSampleCounts", ctypes.c_uint32),
        ("sampledImageStencilSampleCounts", ctypes.c_uint32),
        ("storageImageSampleCounts", ctypes.c_uint32),
        ("maxSampleMaskWords", ctypes.c_uint32),
        ("timestampComputeAndGraphics", ctypes.c_uint32),
        ("timestampPeriod", ctypes.c_float),
        ("maxClipDistances", ctypes.c_uint32),
        ("maxCullDistances", ctypes.c_uint32),
        ("maxCombinedClipAndCullDistances", ctypes.c_uint32),
        ("discreteQueuePriorities", ctypes.c_uint32),
        ("pointSizeRange", ctypes.c_float * 0x2),  # float[2]
        ("lineWidthRange", ctypes.c_float * 0x2),  # float[2]
        ("pointSizeGranularity", ctypes.c_float),
        ("lineWidthGranularity", ctypes.c_float),
        ("strictLines", ctypes.c_uint32),
        ("standardSampleLocations", ctypes.c_uint32),
        ("optimalBufferCopyOffsetAlignment", ctypes.c_uint64),
        ("optimalBufferCopyRowPitchAlignment", ctypes.c_uint64),
        ("nonCoherentAtomSize", ctypes.c_uint64),
    ]
    maxImageDimension1D: int
    maxImageDimension2D: int
    maxImageDimension3D: int
    maxImageDimensionCube: int
    maxImageArrayLayers: int
    maxTexelBufferElements: int
    maxUniformBufferRange: int
    maxStorageBufferRange: int
    maxPushConstantsSize: int
    maxMemoryAllocationCount: int
    maxSamplerAllocationCount: int
    bufferImageGranularity: int
    sparseAddressSpaceSize: int
    maxBoundDescriptorSets: int
    maxPerStageDescriptorSamplers: int
    maxPerStageDescriptorUniformBuffers: int
    maxPerStageDescriptorStorageBuffers: int
    maxPerStageDescriptorSampledImages: int
    maxPerStageDescriptorStorageImages: int
    maxPerStageDescriptorInputAttachments: int
    maxPerStageResources: int
    maxDescriptorSetSamplers: int
    maxDescriptorSetUniformBuffers: int
    maxDescriptorSetUniformBuffersDynamic: int
    maxDescriptorSetStorageBuffers: int
    maxDescriptorSetStorageBuffersDynamic: int
    maxDescriptorSetSampledImages: int
    maxDescriptorSetStorageImages: int
    maxDescriptorSetInputAttachments: int
    maxVertexInputAttributes: int
    maxVertexInputBindings: int
    maxVertexInputAttributeOffset: int
    maxVertexInputBindingStride: int
    maxVertexOutputComponents: int
    maxTessellationGenerationLevel: int
    maxTessellationPatchSize: int
    maxTessellationControlPerVertexInputComponents: int
    maxTessellationControlPerVertexOutputComponents: int
    maxTessellationControlPerPatchOutputComponents: int
    maxTessellationControlTotalOutputComponents: int
    maxTessellationEvaluationInputComponents: int
    maxTessellationEvaluationOutputComponents: int
    maxGeometryShaderInvocations: int
    maxGeometryInputComponents: int
    maxGeometryOutputComponents: int
    maxGeometryOutputVertices: int
    maxGeometryTotalOutputComponents: int
    maxFragmentInputComponents: int
    maxFragmentOutputAttachments: int
    maxFragmentDualSrcAttachments: int
    maxFragmentCombinedOutputResources: int
    maxComputeSharedMemorySize: int
    maxComputeWorkGroupCount: list[ctypes.c_uint32]
    maxComputeWorkGroupInvocations: int
    maxComputeWorkGroupSize: list[ctypes.c_uint32]
    subPixelPrecisionBits: int
    subTexelPrecisionBits: int
    mipmapPrecisionBits: int
    maxDrawIndexedIndexValue: int
    maxDrawIndirectCount: int
    maxSamplerLodBias: float
    maxSamplerAnisotropy: float
    maxViewports: int
    maxViewportDimensions: list[ctypes.c_uint32]
    viewportBoundsRange: list[ctypes.c_float]
    viewportSubPixelBits: int
    minMemoryMapAlignment: int
    minTexelBufferOffsetAlignment: int
    minUniformBufferOffsetAlignment: int
    minStorageBufferOffsetAlignment: int
    minTexelOffset: int
    maxTexelOffset: int
    minTexelGatherOffset: int
    maxTexelGatherOffset: int
    minInterpolationOffset: float
    maxInterpolationOffset: float
    subPixelInterpolationOffsetBits: int
    maxFramebufferWidth: int
    maxFramebufferHeight: int
    maxFramebufferLayers: int
    framebufferColorSampleCounts: int
    framebufferDepthSampleCounts: int
    framebufferStencilSampleCounts: int
    framebufferNoAttachmentsSampleCounts: int
    maxColorAttachments: int
    sampledImageColorSampleCounts: int
    sampledImageIntegerSampleCounts: int
    sampledImageDepthSampleCounts: int
    sampledImageStencilSampleCounts: int
    storageImageSampleCounts: int
    maxSampleMaskWords: int
    timestampComputeAndGraphics: int
    timestampPeriod: float
    maxClipDistances: int
    maxCullDistances: int
    maxCombinedClipAndCullDistances: int
    discreteQueuePriorities: int
    pointSizeRange: list[ctypes.c_float]
    lineWidthRange: list[ctypes.c_float]
    pointSizeGranularity: float
    lineWidthGranularity: float
    strictLines: int
    standardSampleLocations: int
    optimalBufferCopyOffsetAlignment: int
    optimalBufferCopyRowPitchAlignment: int
    nonCoherentAtomSize: int

class VkPhysicalDeviceSparseProperties(ctypes.Structure):
    _fields_ = [
        ("residencyStandard2DBlockShape", ctypes.c_uint32),
        ("residencyStandard2DMultisampleBlockShape", ctypes.c_uint32),
        ("residencyStandard3DBlockShape", ctypes.c_uint32),
        ("residencyAlignedMipSize", ctypes.c_uint32),
        ("residencyNonResidentStrict", ctypes.c_uint32),
    ]
    residencyStandard2DBlockShape: int
    residencyStandard2DMultisampleBlockShape: int
    residencyStandard3DBlockShape: int
    residencyAlignedMipSize: int
    residencyNonResidentStrict: int

class VkMemoryType(ctypes.Structure):
    _fields_ = [
        ("propertyFlags", ctypes.c_uint32),
        ("heapIndex", ctypes.c_uint32),
    ]
    propertyFlags: int
    heapIndex: int

class VkMemoryHeap(ctypes.Structure):
    _fields_ = [
        ("size", ctypes.c_uint64),
        ("flags", ctypes.c_uint32),
    ]
    size: int
    flags: int

class VkMemoryRequirements(ctypes.Structure):
    _fields_ = [
        ("size", ctypes.c_uint64),
        ("alignment", ctypes.c_uint64),
        ("memoryTypeBits", ctypes.c_uint32),
    ]
    size: int
    alignment: int
    memoryTypeBits: int

class VkBufferCopy(ctypes.Structure):
    _fields_ = [
        ("srcOffset", ctypes.c_uint64),
        ("dstOffset", ctypes.c_uint64),
        ("size", ctypes.c_uint64),
    ]
    srcOffset: int
    dstOffset: int
    size: int

class TkDeviceMemory(ctypes.Structure):
    _fields_ = [
        ("mAlloc", ctypes.c_ulonglong),
        ("mMemory", ctypes.c_ulonglong),
        ("mOffset", ctypes.c_uint64),
    ]
    mAlloc: int
    mMemory: int
    mOffset: int

class cEgTextureResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cEgShaderSampler(ctypes.Structure):
    def tilde_cEgShaderSampler(): pass

    def cEgShaderSampler(): pass

    _fields_ = [
        ("msId", ctypes.c_char * 0x20),  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> >
        ("mIdHash", ctypes.c_uint64),
        ("miTexUnit", ctypes.c_char),
        ("_bf_29", ctypes.c_int8),
        ("miMaxAnisotropy", ctypes.c_char),
        ("mbVertexTexture", ctypes.c_ubyte),
        ("mpDefaultTextureResource", ctypes.c_char * 0x10),  # cTkTypedSmartResHandle<cEgTextureResource>
    ]
    msId: bytes
    mIdHash: int
    miTexUnit: str
    _bf_29: int
    miMaxAnisotropy: str
    mbVertexTexture: bool
    mpDefaultTextureResource: bytes

class cEgCodeResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cEgShaderResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cEgMaterialSampler(ctypes.Structure):
    def SetName(): pass

    def tilde_cEgMaterialSampler(): pass

    def cEgMaterialSampler(): pass

    _fields_ = [
        ("msName", ctypes.c_char * 0x20),  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> >
        ("mNameHash", ctypes.c_uint64),
        ("msTextureName", ctypes.c_char * 0x20),  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> >
        ("mpTextureResource", ctypes.c_char * 0x10),  # cTkTypedSmartResHandle<cEgTextureResource>
        ("miFlags", ctypes.c_uint32),
    ]
    msName: bytes
    mNameHash: int
    msTextureName: bytes
    mpTextureResource: bytes
    miFlags: int

class cEgMaterialResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("ParseNode", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int
    ParseNode: int

class cEgRenderableSceneNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class IAnimatableNode(ctypes.Structure):
    def SetLOD(): pass

    def vector_deleting_destructor(): pass

    def SetANIgnoreAnimation(): pass

    def AddANToIgnoreAnimationResetList(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("miNodeIndex", ctypes.c_int32),
        ("miNodeLOD", ctypes.c_int32),
    ]
    __vftable: int
    miNodeIndex: int
    miNodeLOD: int

class IAnimatableNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAnimatableNode_dtor_0", ctypes.c_ulonglong),
        ("GetANName", ctypes.c_ulonglong),
        ("GetANNameHash", ctypes.c_ulonglong),
        ("IsJoint", ctypes.c_ulonglong),
        ("GetJointIndex", ctypes.c_ulonglong),
        ("GetLOD", ctypes.c_ulonglong),
        ("SetLOD", ctypes.c_ulonglong),
        ("GetANParent", ctypes.c_ulonglong),
        ("GetParentModel", ctypes.c_ulonglong),
    ]
    IAnimatableNode_dtor_0: int
    GetANName: int
    GetANNameHash: int
    IsJoint: int
    GetJointIndex: int
    GetLOD: int
    SetLOD: int
    GetANParent: int
    GetParentModel: int

class cEgAnimationResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cPreAdditiveCallback(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cPreAdditiveCallback_vtbl(ctypes.Structure):
    _fields_ = [
        ("PreAdditive", ctypes.c_ulonglong),
    ]
    PreAdditive: int

class cOnPostAnimCallback(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cOnPostAnimCallback_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnPostAnim", ctypes.c_ulonglong),
    ]
    OnPostAnim: int

class cEgJointNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cEgLodNodes(ctypes.Structure):
    def AddNode(): pass

    def GetGenericMeshName(): pass

    def CheckMeshesMatch(): pass

    _fields_ = [
        ("miNumLods", ctypes.c_int32),
        ("miNumMeshes", ctypes.c_int32),
        ("mapLodNodes", ctypes.c_char * 0x58),  # cTkStackVector<std::array<TkHandle,5>,2,-1>
    ]
    miNumLods: int
    miNumMeshes: int
    mapLodNodes: bytes

class cEgModelNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
        ("RecreateNodeList", ctypes.c_ulonglong),
        ("UpdateLocalMeshAABBs", ctypes.c_ulonglong),
        ("GetLodMeshType", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int
    RecreateNodeList: int
    UpdateLocalMeshAABBs: int
    GetLodMeshType: int

class cEgGeometryResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cEgSceneNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cTkComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkReplicatedVariableBase(ctypes.Structure):
    def Construct(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mShouldReplicate", ctypes.c_char * 0x40),  # std::function<bool __cdecl(cGcNetworkPlayer *)>
        ("mbChanged", ctypes.c_ubyte),
    ]
    __vftable: int
    mShouldReplicate: bytes
    mbChanged: bool

class cTkReplicatedVariableBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("IsLocalPlayerEntityOwner", ctypes.c_ulonglong),
        ("IsOwned", ctypes.c_ulonglong),
    ]
    SerializeOrDeserialize: int
    Construct: int
    IsLocalPlayerEntityOwner: int
    IsOwned: int

class cGcPlayerCommon_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsInSpaceship", ctypes.c_ulonglong),
        ("IsInLandVehicle", ctypes.c_ulonglong),
        ("GetEnvironmentLocation", ctypes.c_ulonglong),
        ("IsLocalPlayer", ctypes.c_ulonglong),
        ("GetNetworkIndex", ctypes.c_ulonglong),
        ("IsVRPlayer", ctypes.c_ulonglong),
        ("GetDominantHand", ctypes.c_ulonglong),
        ("IsFullSimLobbyHost", ctypes.c_ulonglong),
        ("VirtualGetMatrix", ctypes.c_ulonglong),
        ("VirtualGetAttachment", ctypes.c_ulonglong),
        ("GetNode", ctypes.c_ulonglong),
        ("NotifyBeingScanned", ctypes.c_ulonglong),
        ("IsStealthActive", ctypes.c_ulonglong),
    ]
    IsInSpaceship: int
    IsInLandVehicle: int
    GetEnvironmentLocation: int
    IsLocalPlayer: int
    GetNetworkIndex: int
    IsVRPlayer: int
    GetDominantHand: int
    IsFullSimLobbyHost: int
    VirtualGetMatrix: int
    VirtualGetAttachment: int
    GetNode: int
    NotifyBeingScanned: int
    IsStealthActive: int

class cTkAttachmentPtr(ctypes.Structure):
    def cTkAttachmentPtr(): pass

    _fields_ = [
        ("miUniqueID", ctypes.c_int64),
    ]
    miUniqueID: int

class cGcOwnerConcept(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x60),  # cTkMetaMessageSink<cTkAttachmentPtr>
    ]
    baseclass_0: bytes

class cGcNetworkAddress(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcNetworkAddress_vtbl(ctypes.Structure):
    pass

class cGcNetworkMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class hknpShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCollisionFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterBodyPairs", ctypes.c_ulonglong),
        ("isCollisionEnabled", ctypes.c_ulonglong),
        ("isCollisionEnabled_28", ctypes.c_ulonglong),
        ("isCollisionEnabled_30", ctypes.c_ulonglong),
        ("isCollisionEnabled_38", ctypes.c_ulonglong),
        ("getLodInfo", ctypes.c_ulonglong),
        ("subscribeToWorld", ctypes.c_ulonglong),
        ("unsubscribeFromWorld", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpCollisionFilter> *(__fastcall *)(hknpCollisionFilter *this, hkRefPtr<hknpCollisionFilter> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterBodyPairs: int
    isCollisionEnabled: int
    isCollisionEnabled_28: int
    isCollisionEnabled_30: int
    isCollisionEnabled_38: int
    getLodInfo: int
    subscribeToWorld: int
    unsubscribeFromWorld: int
    copyFrom: int
    makeClone: bytes

class hknpSurfaceVelocity_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcSurfaceVelocity", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcSurfaceVelocity: int

class hknpMaterialLibrary_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapeTagCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpShapeTagCodec> *(__fastcall *)(hknpShapeTagCodec *this, hkRefPtr<hknpShapeTagCodec> *result)
        ("setMaterialLibrary", ctypes.c_ulonglong),
        ("overrideInitialFilterData", ctypes.c_ulonglong),
        ("decode", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpShapeTagCodec *this, unsigned __int16, hknpCollisionQueryType::Enum, const hknpBody *, const hknpShape *, const hknpShape *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, const hknpShape *, hknpQueryFilterData *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    copyFrom: int
    makeClone: bytes
    setMaterialLibrary: int
    overrideInitialFilterData: int
    decode: bytes

class hknpConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCsGrid(ctypes.Structure):
    def tilde_hknpCsGrid(): pass

    def allocateTemps(): pass

    def freeTemps(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # hknpGrid<hknpCsJacRange>
        ("m_occupancy", ctypes.c_char * 0x10),  # hkArray<hkBool,hkBuiltinContainerAllocator<0,0> >
        ("m_tempsSize", ctypes.c_char * 0x10),  # hkArray<int,hkBuiltinContainerAllocator<0,0> >
        ("m_tempsBuffer", ctypes.c_void_p),
    ]
    baseclass_0: bytes
    m_occupancy: bytes
    m_tempsSize: bytes
    m_tempsBuffer: int

class hknpCdPairStream(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x100),  # hkTypedBlockStream<hknpMotionIdPair>
    ]
    baseclass_0: bytes

class hknpBodyQuality_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpModifierSharedData(ctypes.Structure):
    def initSt(): pass

    _fields_ = [
        ("m_lastFrameContactData", ctypes.c_ulonglong),
        ("m_solverInfo", ctypes.c_ulonglong),
    ]
    m_lastFrameContactData: int
    m_solverInfo: int

class hknpCdBodyBase(ctypes.Structure):
    _fields_ = [
        ("m_body", ctypes.c_ulonglong),
        ("m_motion", ctypes.c_ulonglong),
        ("m_material", ctypes.c_ulonglong),
        ("m_shapeKey", ctypes.c_char * 0x4),  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>
        ("m_shapeAtLod", ctypes.c_ulonglong),
    ]
    m_body: int
    m_motion: int
    m_material: int
    m_shapeKey: bytes
    m_shapeAtLod: int

class hknpShapeKeyPath(ctypes.Structure):
    _fields_ = [
        ("m_key", ctypes.c_char * 0x4),  # hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>
        ("m_size", ctypes.c_int32),
    ]
    m_key: bytes
    m_size: int

class hknpShapeKeyMask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapeQueryScalingData(ctypes.Structure):
    _fields_ = [
        ("m_isScaled", ctypes.c_ubyte),
        ("m_mode", ctypes.c_uint32),
        ("m_convexRadius", ctypes.c_float),
        ("m_scale", common.Vector4f),
        ("m_offset", common.Vector4f),
    ]
    m_isScaled: bool
    m_mode: int
    m_convexRadius: float
    m_scale: common.Vector4f
    m_offset: common.Vector4f

class hknpSolverResults(ctypes.Structure):
    _fields_ = [
        ("m_impulseApplied", ctypes.c_float),
        ("m_internalSolverData", ctypes.c_float),
    ]
    m_impulseApplied: float
    m_internalSolverData: float

class hkTimeStampHashBase(ctypes.Structure):
    def hkTimeStampHashBase(): pass

    def tilde_hkTimeStampHashBase(): pass

    _fields_ = [
        ("m_currentTime", ctypes.c_char * 0x4),  # std::atomic<unsigned int>
        ("m_criticalSection", ctypes.c_char * 0x8),  # hkScopedPtr<hkCriticalSection>
    ]
    m_currentTime: bytes
    m_criticalSection: bytes

class hknpShapeBuffer(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_shape", ctypes.c_ulonglong),
        ("m_buffer", ctypes.c_void_p),
        ("m_bufferSize", ctypes.c_int32),
    ]
    __vftable: int
    m_shape: int
    m_buffer: int
    m_bufferSize: int

class hknpShapeBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpShapeBuffer_dtor_0", ctypes.c_ulonglong),
    ]
    hknpShapeBuffer_dtor_0: int

class hknpConvexShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpSphereShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpParticlesCollisionFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterBodyPairs", ctypes.c_ulonglong),
        ("isCollisionEnabled", ctypes.c_ulonglong),
        ("isCollisionEnabled_28", ctypes.c_ulonglong),
        ("isCollisionEnabled_30", ctypes.c_ulonglong),
        ("isCollisionEnabled_38", ctypes.c_ulonglong),
        ("getLodInfo", ctypes.c_ulonglong),
        ("subscribeToWorld", ctypes.c_ulonglong),
        ("unsubscribeFromWorld", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpCollisionFilter> *(__fastcall *)(hknpCollisionFilter *this, hkRefPtr<hknpCollisionFilter> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterBodyPairs: int
    isCollisionEnabled: int
    isCollisionEnabled_28: int
    isCollisionEnabled_30: int
    isCollisionEnabled_38: int
    getLodInfo: int
    subscribeToWorld: int
    unsubscribeFromWorld: int
    copyFrom: int
    makeClone: bytes

class hknpParticle4Faces(ctypes.Structure):
    _fields_ = [
        ("m_directions", ctypes.c_char * 0x30),  # hkFourTransposedPointsImpl<float>
    ]
    m_directions: bytes

class hknpParticleFaceVerticesWithEffMass(ctypes.Structure):
    _fields_ = [
        ("m_faceVertices", ctypes.c_char * 0x30),  # hkFourTransposedPointsImpl<float>
        ("m_effectiveMasses", common.Vector4f),
        ("m_contactRadius", ctypes.c_float),
        ("m_distance", ctypes.c_float),
    ]
    m_faceVertices: bytes
    m_effectiveMasses: common.Vector4f
    m_contactRadius: float
    m_distance: float

class hknpParticleShapeLibrary_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpParticleLinearInfo(ctypes.Structure):
    _fields_ = [
        ("m_position", common.Vector4f),
        ("m_velocity", common.Vector4f),
    ]
    m_position: common.Vector4f
    m_velocity: common.Vector4f

class hknpParticlePairCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("PairCollector_dtor_0", ctypes.c_ulonglong),
        ("addPairs", ctypes.c_ulonglong),
    ]
    PairCollector_dtor_0: int
    addPairs: int

class hknpParticleParticleCollider(ctypes.Structure):
    def tilde_hknpParticleParticleCollider(): pass

    def init(): pass

    def invalidateCaches(): pass

    def moveCaches(): pass

    _fields_ = [
        ("m_particlePairsBuffers", ctypes.c_char * 0x10),  # hkArray<hknpParticlePairCollector,hkBuiltinContainerAllocator<0,0> >
        ("m_particleBranchPairsBuffers", ctypes.c_char * 0x10),  # hkArray<hknpParticlePairCollector,hkBuiltinContainerAllocator<0,0> >
        ("m_particleParticleCacheHeaders", ctypes.c_char * 0x10),  # hkArray<hknpParticleParticleCollisionsCacheHeader,hkBuiltinContainerAllocator<0,0> >
        ("m_particleParticleCacheData", ctypes.c_char * 0x10),  # hkArray<hknpParticleParticleCollisionsCacheData,hkBuiltinContainerAllocator<0,0> >
        ("m_maxTimeInPenetration", ctypes.c_float),
        ("m_periodicallyResetEarlyOut", ctypes.c_ubyte),
        ("m_particlesCollider", ctypes.c_ulonglong),
    ]
    m_particlePairsBuffers: bytes
    m_particleBranchPairsBuffers: bytes
    m_particleParticleCacheHeaders: bytes
    m_particleParticleCacheData: bytes
    m_maxTimeInPenetration: float
    m_periodicallyResetEarlyOut: bool
    m_particlesCollider: int

class hkPackedUnitVector8(ctypes.Structure):
    _fields_ = [
        ("m_packed", ctypes.c_uint32),
    ]
    m_packed: int

class hknpParticlesBodyCollider(ctypes.Structure):
    def hknpParticlesBodyCollider(): pass

    def tilde_hknpParticlesBodyCollider(): pass

    def invalidateCaches(): pass

    def moveCaches(): pass

    def shiftCaches(): pass

    _fields_ = [
        ("m_particlesCollider", ctypes.c_ulonglong),
        ("m_previousPositionsAndVelocities", ctypes.c_char * 0x10),  # hkArrayView<hknpParticleLinearInfo const >
        ("m_staticCaches", ctypes.c_char * 0x10),  # hkArray<hknpParticleCollisionCache,hkBuiltinContainerAllocator<0,0> >
        ("m_staticNmpCaches", ctypes.c_char * 0x10),  # hkArray<hknpParticleNmpCacheBase,hkBuiltinContainerAllocator<0,0> >
        ("m_numNeighborsInCache", ctypes.c_char * 0x10),  # hkArray<signed char,hkBuiltinContainerAllocator<0,0> >
        ("m_neighborCache", ctypes.c_char * 0x10),  # hkArray<hknpParticleDynamicNeighborsCache,hkBuiltinContainerAllocator<0,0> >
        ("m_dynamicCache", ctypes.c_char * 0x10),  # hkArray<hknpParticleCollisionCache,hkBuiltinContainerAllocator<0,0> >
        ("m_dynamicNmpCaches", ctypes.c_char * 0x10),  # hkArray<hknpParticleNmpCacheBase,hkBuiltinContainerAllocator<0,0> >
    ]
    m_particlesCollider: int
    m_previousPositionsAndVelocities: bytes
    m_staticCaches: bytes
    m_staticNmpCaches: bytes
    m_numNeighborsInCache: bytes
    m_neighborCache: bytes
    m_dynamicCache: bytes
    m_dynamicNmpCaches: bytes

class hknpTriangleShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCollisionQueryContext(ctypes.Structure):
    def hknpCollisionQueryContext(): pass

    _fields_ = [
        ("m_dispatcher", ctypes.c_ulonglong),
        ("m_queryTriangle", ctypes.c_ulonglong),
        ("m_targetTriangle", ctypes.c_ulonglong),
        ("m_initialCastFractionHint", ctypes.c_float),
        ("m_externallyAllocatedTriangles", ctypes.c_uint32),
    ]
    m_dispatcher: int
    m_queryTriangle: int
    m_targetTriangle: int
    m_initialCastFractionHint: float
    m_externallyAllocatedTriangles: int

class hknpCollisionQueryCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class hkcdRay(ctypes.Structure):
    _fields_ = [
        ("m_origin", common.Vector4f),
        ("m_direction", common.Vector4f),
        ("m_invDirection", common.Vector4f),
    ]
    m_origin: common.Vector4f
    m_direction: common.Vector4f
    m_invDirection: common.Vector4f

class hknpCollisionQueryDispatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpParticlesTasks(ctypes.Structure):
    def tilde_hknpParticlesTasks(): pass

    _fields_ = [
        ("m_stepBeginsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_prepareTreeBuildTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_buildFirstNLevelsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_buildRangesTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_finalizeTreeTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_updateRigidBodyDirtyAabbsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_updateParticleParticleDirtyAabbsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_particleParticleProcessEarlyOutTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_particleParticleCollisionsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_particleParticleInterBranchCollisionsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_particleParticleInterBranchSolveTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_fireParticlesCollidedWithParticlesCallbackTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_findDynamicNeighborsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_generateMappingToPointsBufferTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_bodyCollisionsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_fireParticlesCollidedWithBodiesCallbackTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_fireParticlesExitedSafeRegionCallbackTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
        ("m_stepEndsTask", ctypes.c_char * 0x8),  # hkScopedPtr<hkTask>
    ]
    m_stepBeginsTask: bytes
    m_prepareTreeBuildTask: bytes
    m_buildFirstNLevelsTask: bytes
    m_buildRangesTask: bytes
    m_finalizeTreeTask: bytes
    m_updateRigidBodyDirtyAabbsTask: bytes
    m_updateParticleParticleDirtyAabbsTask: bytes
    m_particleParticleProcessEarlyOutTask: bytes
    m_particleParticleCollisionsTask: bytes
    m_particleParticleInterBranchCollisionsTask: bytes
    m_particleParticleInterBranchSolveTask: bytes
    m_fireParticlesCollidedWithParticlesCallbackTask: bytes
    m_findDynamicNeighborsTask: bytes
    m_generateMappingToPointsBufferTask: bytes
    m_bodyCollisionsTask: bytes
    m_fireParticlesCollidedWithBodiesCallbackTask: bytes
    m_fireParticlesExitedSafeRegionCallbackTask: bytes
    m_stepEndsTask: bytes

class hknpParticlesCollider_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpWeldingModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpWeldingModifier_dtor_0", ctypes.c_ulonglong),
        ("postCompositeCollideCallback", ctypes.c_ulonglong),
    ]
    hknpWeldingModifier_dtor_0: int
    postCompositeCollideCallback: int

class hknpManifoldHeaderPair(ctypes.Structure):
    _fields_ = [
        ("m_header", ctypes.c_ulonglong),
        ("m_item", ctypes.c_void_p),
    ]
    m_header: int
    m_item: int

class hknpNeighborWeldingModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpWeldingModifier_dtor_0", ctypes.c_ulonglong),
        ("postCompositeCollideCallback", ctypes.c_ulonglong),
        ("weldManifolds", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpNeighborWeldingModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpWeldingModifier::WeldingInfo *, hkArrayView<hknpManifoldHeaderPair>, hkBlockStream::LinkedRange *)
    ]
    hknpWeldingModifier_dtor_0: int
    postCompositeCollideCallback: int
    weldManifolds: bytes

class hknpLiveWeldingModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpWeldingModifier_dtor_0", ctypes.c_ulonglong),
        ("postCompositeCollideCallback", ctypes.c_ulonglong),
        ("weldManifolds", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpNeighborWeldingModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpWeldingModifier::WeldingInfo *, hkArrayView<hknpManifoldHeaderPair>, hkBlockStream::LinkedRange *)
    ]
    hknpWeldingModifier_dtor_0: int
    postCompositeCollideCallback: int
    weldManifolds: bytes

class hknpCollisionDetector(ctypes.Structure):
    def collide(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_useChildCaches", ctypes.c_uint32),
    ]
    __vftable: int
    m_useChildCaches: int

class hknpCollisionCacheBatchConsumer(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x38),  # hkTypedBlockStream<hknpCollisionCache>::BatchConsumerRangeOnly
    ]
    baseclass_0: bytes

class hknpCollisionDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes

class hknpSpaceSplitter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("onShiftWorld", ctypes.c_ulonglong),
        ("getCellIdx", ctypes.c_ulonglong),
        ("applyThreadData", ctypes.c_ulonglong),
        ("calcInitialCellPositions", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    onShiftWorld: int
    getCellIdx: int
    applyThreadData: int
    calcInitialCellPositions: int
    copyFrom: int

class hknpIdxRange(ctypes.Structure):
    _fields_ = [
        ("m_start", ctypes.c_int32),
        ("m_numElements", ctypes.c_int32),
    ]
    m_start: int
    m_numElements: int

class hknpDeactivationState(ctypes.Structure):
    def checkMotionForDeactivation(): pass

    def initDeactivationState(): pass

    _fields_ = [
        ("m_referencePosition", ctypes.c_uint32),
        ("m_referenceOrientation", ctypes.c_uint32),
        ("m_accumVelocityDiff", ctypes.c_int16),
        ("m_frameCounter", ctypes.c_uint8),
        ("m_numDeactivationChecks", ctypes.c_uint8),
        ("m_velocityScaleSquared", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
        ("m_activationControl", ctypes.c_char * 0x1),  # hkEnum<enum hknpActivationControl::Enum,signed char>
        ("m_islandIndex", ctypes.c_int32),
    ]
    m_referencePosition: int
    m_referenceOrientation: int
    m_accumVelocityDiff: int
    m_frameCounter: int
    m_numDeactivationChecks: int
    m_velocityScaleSquared: list[ctypes.c_uint8]
    m_activationControl: bytes
    m_islandIndex: int

class hkUnionFind(ctypes.Structure):
    def hkUnionFind(): pass

    def addEdge(): pass

    def assignGroupsInternal(): pass

    def collapseTree(): pass

    def findRootOfNode(): pass

    def sortByGroupIdUnchecked(): pass

    _fields_ = [
        ("m_parents", ctypes.c_char * 0x10),  # hkArrayView<int>
        ("m_numNodes", ctypes.c_int32),
        ("m_isCollapsed", ctypes.c_uint32),
        ("m_numRoots", ctypes.c_int32),
        ("m_isAddingEdges", ctypes.c_char * 0x4),  # hkDebugOnlyMember<unsigned int>
    ]
    m_parents: bytes
    m_numNodes: int
    m_isCollapsed: int
    m_numRoots: int
    m_isAddingEdges: bytes

class hknpDeactivationStepInfo(ctypes.Structure):
    def hknpDeactivationStepInfo(): pass

    def tilde_hknpDeactivationStepInfo(): pass

    def addActiveBodyPairs(): pass

    def addActiveBodyPairsBegin(): pass

    def addActiveBodyPairsEnd(): pass

    def combineActivityBitFields(): pass

    def getThreadData(): pass

    _fields_ = [
        ("m_unionFind", ctypes.c_ulonglong),
        ("m_buffer", ctypes.c_char * 0x10),  # hkArray<int,hkBuiltinContainerAllocator<0,0> >
        ("m_threadDatas", ctypes.c_char * 0x110),  # hkInplaceArray<hknpDeactivationThreadData,8,hkBuiltinContainerAllocator<0,0> >
        ("m_numMotions", ctypes.c_int32),
        ("m_canAddEdges", ctypes.c_ubyte),
    ]
    m_unionFind: int
    m_buffer: bytes
    m_threadDatas: bytes
    m_numMotions: int
    m_canAddEdges: bool

class hknpCommandGrid(ctypes.Structure):
    def dispatchCommands(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # hknpGrid<hkBlockStream::LinkedRange>
    ]
    baseclass_0: bytes

class hknpTimerVolumeTags(ctypes.Structure):
    def tilde_hknpTimerVolumeTags(): pass

    _fields_ = [
        ("m_collide", ctypes.c_char * 0x10),  # hkArray<hkVdbCmdTag,hkBuiltinContainerAllocator<3,0> >
        ("m_solve", ctypes.c_char * 0x10),  # hkArray<hkVdbCmdTag,hkBuiltinContainerAllocator<3,0> >
        ("m_atomConstraintBuilding", ctypes.c_char * 0x10),  # hkArray<hkVdbCmdTag,hkBuiltinContainerAllocator<3,0> >
        ("m_atomConstraintSolving", ctypes.c_char * 0x10),  # hkArray<hkVdbCmdTag,hkBuiltinContainerAllocator<3,0> >
        ("m_particlesBodyCollision", ctypes.c_char * 0x10),  # hkArray<hkVdbCmdTag,hkBuiltinContainerAllocator<3,0> >
        ("m_characterProxies", ctypes.c_char * 0x10),  # hkArray<hkVdbCmdTag,hkBuiltinContainerAllocator<3,0> >
    ]
    m_collide: bytes
    m_solve: bytes
    m_atomConstraintBuilding: bytes
    m_atomConstraintSolving: bytes
    m_particlesBodyCollision: bytes
    m_characterProxies: bytes

class hknpSolverData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("clear", ctypes.c_ulonglong),
        ("addImmediateContactConstraint", ctypes.c_ulonglong),
        ("addImmediateContactConstraintUsingJacobian", ctypes.c_ulonglong),
        ("addImmediateConstraint", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpSolverData *this, hknpBodyId, hknpBodyId, hknpConstraintData *, hkHandle<unsigned short,65535,hknpImmediateConstraintIdDiscriminant>, hkFlags<enum hknpConstraint::FlagsEnum,unsigned short>)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    clear: int
    addImmediateContactConstraint: int
    addImmediateContactConstraintUsingJacobian: int
    addImmediateConstraint: bytes

class hknpEventSignal(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x8),  # hkSignal2<hknpEventHandlerInput const &,hknpEvent const &>
    ]
    baseclass_0: bytes

class hknpEventDispatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("exec", ctypes.c_ulonglong),
        ("print", ctypes.c_ulonglong),
        ("flushRemainingEvents", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    exec: int
    print: int
    flushRemainingEvents: int

class hknpRefMaterial_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpRefMassDistribution_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapePropertyBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpRefDragProperties_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpWorldWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("setName", ctypes.c_ulonglong),
        ("setGravity", ctypes.c_ulonglong),
        ("setAirDensity", ctypes.c_ulonglong),
        ("setCollisionFilter", ctypes.c_ulonglong),
        ("setShapeTagCodec", ctypes.c_ulonglong),
        ("setEventDispatcher", ctypes.c_ulonglong),
        ("setSafeRegionAabb", ctypes.c_ulonglong),
        ("allocateMaterial", ctypes.c_ulonglong),
        ("allocateMaterial_40", ctypes.c_ulonglong),
        ("destroyMaterials", ctypes.c_ulonglong),
        ("allocateMotionProperties", ctypes.c_ulonglong),
        ("destroyMotionProperties", ctypes.c_ulonglong),
        ("allocateBody", ctypes.c_ulonglong),
        ("destroyBodies", ctypes.c_ulonglong),
        ("addBody", ctypes.c_ulonglong),
        ("addBodies", ctypes.c_ulonglong),
        ("removeBodies", ctypes.c_ulonglong),
        ("setBodyTransform", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hknpBodyId, const hkRotationImpl<float> *, const hkVector4d *, hknpWorldWriter::ActivationMode)
        ("setBodyPosition", ctypes.c_ulonglong),
        ("setBodyOrientation", ctypes.c_ulonglong),
        ("setBodyVelocity", ctypes.c_ulonglong),
        ("setBodyLinearVelocity", ctypes.c_ulonglong),
        ("setBodyAngularVelocity", ctypes.c_ulonglong),
        ("setBodyVelocityAt", ctypes.c_ulonglong),
        ("applyBodyLinearImpulse", ctypes.c_ulonglong),
        ("applyBodyAngularImpulse", ctypes.c_ulonglong),
        ("applyBodyImpulseAt", ctypes.c_ulonglong),
        ("setBodyMotionType", ctypes.c_ulonglong),
        ("setBodyMotionProperties", ctypes.c_ulonglong),
        ("attachBodies", ctypes.c_ulonglong),
        ("detachBodies", ctypes.c_ulonglong),
        ("setBodyMass", ctypes.c_ulonglong),
        ("setBodyDensity", ctypes.c_ulonglong),
        ("setBodyMassDistribution", ctypes.c_ulonglong),
        ("setBodyShape", ctypes.c_ulonglong),
        ("setBodyCollisionFilterInfo", ctypes.c_ulonglong),
        ("setBodyMaterial", ctypes.c_ulonglong),
        ("setBodyQuality", ctypes.c_ulonglong),
        ("setBodyCollisionLookAheadDistance", ctypes.c_ulonglong),
        ("setBodyDragProperties", ctypes.c_ulonglong),
        ("setBodyActivationControl", ctypes.c_ulonglong),
        ("setBodyActivationState", ctypes.c_ulonglong),
        ("setBodyActivationPriority", ctypes.c_ulonglong),
        ("enableBodyFlags", ctypes.c_ulonglong),
        ("disableBodyFlags", ctypes.c_ulonglong),
        ("setBodyPropertyImpl", ctypes.c_ulonglong),
        ("clearBodyProperty", ctypes.c_ulonglong),
        ("rebuildBodyCollisionCaches", ctypes.c_ulonglong),
        ("updateBodyCollisionCaches", ctypes.c_ulonglong),
        ("allocateMotion", ctypes.c_ulonglong),
        ("destroyMotions", ctypes.c_ulonglong),
        ("setBodyMotion", ctypes.c_ulonglong),
        ("rebuildMotionMassProperties", ctypes.c_ulonglong),
        ("setMotionCenterOfMass", ctypes.c_ulonglong),
        ("setMotionInertia", ctypes.c_ulonglong),
        ("reintegrateMotion", ctypes.c_ulonglong),
        ("allocateConstraint", ctypes.c_ulonglong),
        ("destroyConstraints", ctypes.c_ulonglong),
        ("addConstraint", ctypes.c_ulonglong),
        ("removeConstraint", ctypes.c_ulonglong),
        ("allocateConstraintGroup", ctypes.c_char * 0x8),  # hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> *(__fastcall *)(hknpWorldWriter *this, hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> *result, const hknpConstraintGroupCinfo *)
        ("destroyConstraintGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant>)
        ("setConstraintGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hknpConstraintId, hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant>)
        ("setConstraintPropertyImpl", ctypes.c_ulonglong),
        ("clearConstraintProperty", ctypes.c_ulonglong),
        ("enableConstraintFlags", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hknpConstraintId, hkFlags<enum hknpConstraint::FlagsEnum,unsigned short>, hknpWorldWriter::ActivationMode)
        ("disableConstraintFlags", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hknpConstraintId, hkFlags<enum hknpConstraint::FlagsEnum,unsigned short>, hknpWorldWriter::ActivationMode)
        ("allocateParticlesCollider", ctypes.c_ulonglong),
        ("destroyParticlesCollider", ctypes.c_ulonglong),
        ("addParticlesCollider", ctypes.c_ulonglong),
        ("removeParticlesCollider", ctypes.c_ulonglong),
        ("addParticles", ctypes.c_ulonglong),
        ("removeParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hknpParticlesColliderId, const hkArrayView<int const > *, hknpWorldWriter::RemoveParticlesResult *)
        ("enableParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hknpParticlesColliderId, const hkArrayView<int const > *)
        ("disableParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hknpParticlesColliderId, const hkArrayView<int const > *)
        ("rebuildParticleCollisionCaches", ctypes.c_ulonglong),
        ("rebuildParticleCollisionCaches_260", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldWriter *this, hknpParticlesColliderId, hkArrayView<int const >)
        ("updateBroadPhase", ctypes.c_ulonglong),
        ("shiftWorld", ctypes.c_ulonglong),
        ("activateBodiesInAabb", ctypes.c_ulonglong),
        ("executeUserCommandImpl", ctypes.c_ulonglong),
    ]
    setName: int
    setGravity: int
    setAirDensity: int
    setCollisionFilter: int
    setShapeTagCodec: int
    setEventDispatcher: int
    setSafeRegionAabb: int
    allocateMaterial: int
    allocateMaterial_40: int
    destroyMaterials: int
    allocateMotionProperties: int
    destroyMotionProperties: int
    allocateBody: int
    destroyBodies: int
    addBody: int
    addBodies: int
    removeBodies: int
    setBodyTransform: bytes
    setBodyPosition: int
    setBodyOrientation: int
    setBodyVelocity: int
    setBodyLinearVelocity: int
    setBodyAngularVelocity: int
    setBodyVelocityAt: int
    applyBodyLinearImpulse: int
    applyBodyAngularImpulse: int
    applyBodyImpulseAt: int
    setBodyMotionType: int
    setBodyMotionProperties: int
    attachBodies: int
    detachBodies: int
    setBodyMass: int
    setBodyDensity: int
    setBodyMassDistribution: int
    setBodyShape: int
    setBodyCollisionFilterInfo: int
    setBodyMaterial: int
    setBodyQuality: int
    setBodyCollisionLookAheadDistance: int
    setBodyDragProperties: int
    setBodyActivationControl: int
    setBodyActivationState: int
    setBodyActivationPriority: int
    enableBodyFlags: int
    disableBodyFlags: int
    setBodyPropertyImpl: int
    clearBodyProperty: int
    rebuildBodyCollisionCaches: int
    updateBodyCollisionCaches: int
    allocateMotion: int
    destroyMotions: int
    setBodyMotion: int
    rebuildMotionMassProperties: int
    setMotionCenterOfMass: int
    setMotionInertia: int
    reintegrateMotion: int
    allocateConstraint: int
    destroyConstraints: int
    addConstraint: int
    removeConstraint: int
    allocateConstraintGroup: bytes
    destroyConstraintGroup: bytes
    setConstraintGroup: bytes
    setConstraintPropertyImpl: int
    clearConstraintProperty: int
    enableConstraintFlags: bytes
    disableConstraintFlags: bytes
    allocateParticlesCollider: int
    destroyParticlesCollider: int
    addParticlesCollider: int
    removeParticlesCollider: int
    addParticles: int
    removeParticles: bytes
    enableParticles: bytes
    disableParticles: bytes
    rebuildParticleCollisionCaches: int
    rebuildParticleCollisionCaches_260: bytes
    updateBroadPhase: int
    shiftWorld: int
    activateBodiesInAabb: int
    executeUserCommandImpl: int

class hknpWorldReader(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hknpMotionPropertiesLibrary_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpBodyQualityLibrary_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpLockedBodyIdArrayView(ctypes.Structure):
    def tilde_hknpLockedBodyIdArrayView(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # hkArrayView<hknpBodyIdAndIndex const >
        ("m_lock", ctypes.c_ulonglong),
    ]
    baseclass_0: bytes
    m_lock: int

class hknpBodyConstraintsRange(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x40),  # hkIteratorRange<hknpBodyConstraintsIterator,hknpBodyConstraintsIterator>
    ]
    baseclass_0: bytes

class hknpConstraintGroupRuntime(ctypes.Structure):
    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hknpConstraintGroupRuntime_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpConstraintGroupRuntime_dtor_0", ctypes.c_ulonglong),
    ]
    hknpConstraintGroupRuntime_dtor_0: int

class hknpBroadPhaseQueryCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    addHit: int

class hknpWorldReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getName", ctypes.c_ulonglong),
        ("getGravity", ctypes.c_ulonglong),
        ("getAirDensity", ctypes.c_ulonglong),
        ("getCollisionFilter", ctypes.c_ulonglong),
        ("getShapeTagCodec", ctypes.c_ulonglong),
        ("getCollisionQueryDispatcher", ctypes.c_ulonglong),
        ("getMaterialLibrary", ctypes.c_ulonglong),
        ("getMotionPropertiesLibrary", ctypes.c_ulonglong),
        ("getBodyQualityLibrary", ctypes.c_ulonglong),
        ("getSolverInfoPtr", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_char * 0x8),  # hknpThreadSafeObjectPoolRange<hknpBody const &> *(__fastcall *)(hknpWorldReader *this, hknpThreadSafeObjectPoolRange<hknpBody const &> *result)
        ("getBodyIds", ctypes.c_char * 0x8),  # hknpThreadSafeObjectPoolRange<hknpBodyId> *(__fastcall *)(hknpWorldReader *this, hknpThreadSafeObjectPoolRange<hknpBodyId> *result)
        ("getNumBodies", ctypes.c_ulonglong),
        ("getBodyCapacity", ctypes.c_ulonglong),
        ("isBodyValid", ctypes.c_ulonglong),
        ("isBodyAdded", ctypes.c_ulonglong),
        ("getBody", ctypes.c_ulonglong),
        ("getBodyTransform", ctypes.c_ulonglong),
        ("predictBodyTransform", ctypes.c_ulonglong),
        ("getBodyVelocity", ctypes.c_ulonglong),
        ("getBodyLinearVelocity", ctypes.c_ulonglong),
        ("getBodyAngularVelocity", ctypes.c_ulonglong),
        ("getBodyAabb", ctypes.c_ulonglong),
        ("getBodyActivationControl", ctypes.c_ulonglong),
        ("getBodyActivationState", ctypes.c_ulonglong),
        ("getBodyMass", ctypes.c_ulonglong),
        ("getBodyMassDistribution", ctypes.c_ulonglong),
        ("getBodyDragProperties", ctypes.c_ulonglong),
        ("getBodyName", ctypes.c_ulonglong),
        ("hasBodyProperty", ctypes.c_ulonglong),
        ("getBodyPropertyImpl", ctypes.c_ulonglong),
        ("getActiveBodies", ctypes.c_ulonglong),
        ("getMotion", ctypes.c_ulonglong),
        ("getConstraints", ctypes.c_char * 0x8),  # hknpThreadSafeObjectPoolRange<hknpConstraint const &> *(__fastcall *)(hknpWorldReader *this, hknpThreadSafeObjectPoolRange<hknpConstraint const &> *result)
        ("getConstraintIds", ctypes.c_char * 0x8),  # hknpThreadSafeObjectPoolRange<hknpConstraintId> *(__fastcall *)(hknpWorldReader *this, hknpThreadSafeObjectPoolRange<hknpConstraintId> *result)
        ("getBodyConstraints", ctypes.c_ulonglong),
        ("getNumConstraints", ctypes.c_ulonglong),
        ("getConstraintCapacity", ctypes.c_ulonglong),
        ("isConstraintValid", ctypes.c_ulonglong),
        ("isConstraintAdded", ctypes.c_ulonglong),
        ("getConstraint", ctypes.c_ulonglong),
        ("getConstraintName", ctypes.c_ulonglong),
        ("hasConstraintProperty", ctypes.c_ulonglong),
        ("getConstraintPropertyImpl", ctypes.c_ulonglong),
        ("getConstraintGroups", ctypes.c_char * 0x8),  # hknpThreadSafeObjectPoolRange<hknpConstraintGroup const &> *(__fastcall *)(hknpWorldReader *this, hknpThreadSafeObjectPoolRange<hknpConstraintGroup const &> *result)
        ("getConstraintGroupIds", ctypes.c_char * 0x8),  # hknpThreadSafeObjectPoolRange<hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> > *(__fastcall *)(hknpWorldReader *this, hknpThreadSafeObjectPoolRange<hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant> > *result)
        ("getConstraintGroup", ctypes.c_char * 0x8),  # hknpConstraintGroup *(__fastcall *)(hknpWorldReader *this, hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant>)
        ("getNumConstraintGroups", ctypes.c_ulonglong),
        ("getConstraintGroupCapacity", ctypes.c_ulonglong),
        ("getNumParticlesColliders", ctypes.c_ulonglong),
        ("getParticlesColliderCapacity", ctypes.c_ulonglong),
        ("getParticlesCollider", ctypes.c_ulonglong),
        ("getBodyParticlesCollider", ctypes.c_ulonglong),
        ("queryAabb", ctypes.c_ulonglong),
        ("queryAabb_1b0", ctypes.c_ulonglong),
        ("queryAabb_1b8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldReader *this, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp", ctypes.c_ulonglong),
        ("queryAabbNmp_1c8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldReader *this, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *, hkAabb *)
        ("castAabb", ctypes.c_ulonglong),
        ("castAabb_1d8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldReader *this, const hknpAabbCastQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castRay", ctypes.c_ulonglong),
        ("castShape", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpWorldReader *this, const hknpShapeCastQuery *, const hkRotationImpl<float> *, hknpCollisionQueryCollector *, hknpCollisionQueryCollector *)
        ("queryPoint", ctypes.c_ulonglong),
        ("getClosestPoints", ctypes.c_ulonglong),
    ]
    getName: int
    getGravity: int
    getAirDensity: int
    getCollisionFilter: int
    getShapeTagCodec: int
    getCollisionQueryDispatcher: int
    getMaterialLibrary: int
    getMotionPropertiesLibrary: int
    getBodyQualityLibrary: int
    getSolverInfoPtr: int
    getBodies: bytes
    getBodyIds: bytes
    getNumBodies: int
    getBodyCapacity: int
    isBodyValid: int
    isBodyAdded: int
    getBody: int
    getBodyTransform: int
    predictBodyTransform: int
    getBodyVelocity: int
    getBodyLinearVelocity: int
    getBodyAngularVelocity: int
    getBodyAabb: int
    getBodyActivationControl: int
    getBodyActivationState: int
    getBodyMass: int
    getBodyMassDistribution: int
    getBodyDragProperties: int
    getBodyName: int
    hasBodyProperty: int
    getBodyPropertyImpl: int
    getActiveBodies: int
    getMotion: int
    getConstraints: bytes
    getConstraintIds: bytes
    getBodyConstraints: int
    getNumConstraints: int
    getConstraintCapacity: int
    isConstraintValid: int
    isConstraintAdded: int
    getConstraint: int
    getConstraintName: int
    hasConstraintProperty: int
    getConstraintPropertyImpl: int
    getConstraintGroups: bytes
    getConstraintGroupIds: bytes
    getConstraintGroup: bytes
    getNumConstraintGroups: int
    getConstraintGroupCapacity: int
    getNumParticlesColliders: int
    getParticlesColliderCapacity: int
    getParticlesCollider: int
    getBodyParticlesCollider: int
    queryAabb: int
    queryAabb_1b0: int
    queryAabb_1b8: bytes
    queryAabbNmp: int
    queryAabbNmp_1c8: bytes
    castAabb: int
    castAabb_1d8: bytes
    castRay: int
    castShape: bytes
    queryPoint: int
    getClosestPoints: int

class hknpCollisionCacheStream(ctypes.Structure):
    def tilde_hknpCollisionCacheStream(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x100),  # hkTypedBlockStream<hknpCollisionCache>
    ]
    baseclass_0: bytes

class hknpParticlesColliderShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpParticleSimulationContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class PreStepTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class PostStepTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class hknpTaskGraph_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("clear", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    clear: int

class hknpSimulation_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpSimulation_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("solve", ctypes.c_ulonglong),
        ("exportStatisticsToMonitorStream", ctypes.c_ulonglong),
        ("checkConsistency", ctypes.c_ulonglong),
    ]
    hknpSimulation_dtor_0: int
    collide: int
    solve: int
    exportStatisticsToMonitorStream: int
    checkConsistency: int

class hknpBroadPhaseConfig_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumLayers", ctypes.c_ulonglong),
        ("getLayer", ctypes.c_ulonglong),
        ("getLayerIndices", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumLayers: int
    getLayer: int
    getLayerIndices: int

class hknpParticlesCollisionQueryCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
        ("addHitInternal", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int
    addHitInternal: int

class hknpParticlesStaticCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
        ("addHitInternal", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int
    addHitInternal: int

class hknpBroadPhase_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpBroadPhase_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getConfig", ctypes.c_ulonglong),
        ("setBodyBuffer", ctypes.c_ulonglong),
        ("translate", ctypes.c_ulonglong),
        ("addBodies", ctypes.c_ulonglong),
        ("removeBodies", ctypes.c_ulonglong),
        ("markBodiesDirty", ctypes.c_ulonglong),
        ("update", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, hkFlags<enum hknpBroadPhaseFilter::Category,unsigned char>)
        ("updateBodyAabbs", ctypes.c_ulonglong),
        ("buildTaskGraph", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, hknpWorld *, hknpSimulationContext *, hkFlags<enum hknpBroadPhaseFilter::Category,unsigned char>, hkTypedBlockStream<hknpBodyIndexPair> *, hknpTaskGraph *)
        ("getExtents", ctypes.c_ulonglong),
        ("findAllPairs", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpSimulationThreadContext *, hkTypedBlockStream<hknpBodyIndexPair>::Writer *)
        ("findNewPairs", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpSimulationThreadContext *, const hkAabbFloat3 *, hkTypedBlockStream<hknpBodyIndexPair>::Writer *)
        ("queryAabb", ctypes.c_ulonglong),
        ("queryAabb_78", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabb_80", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryOutsideOfAabb", ctypes.c_ulonglong),
        ("queryOutsideOfAabb_90", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp", ctypes.c_ulonglong),
        ("queryAabbNmp_a0", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp_a8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castAabb", ctypes.c_ulonglong),
        ("castAabb_b8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbCastQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castRay", ctypes.c_ulonglong),
        ("castShape", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpShapeCastQuery *, const hkRotationImpl<float> *, const hknpCollisionQueryDispatcher *, hknpCollisionQueryCollector *, hknpCollisionQueryCollector *, hknpCollisionQueryContext *)
        ("queryPoint", ctypes.c_ulonglong),
        ("getClosestPoints", ctypes.c_ulonglong),
        ("getClosestPointsAdvanced", ctypes.c_ulonglong),
        ("getClosestPointsParticles", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
    ]
    hknpBroadPhase_dtor_0: int
    getType: int
    getConfig: int
    setBodyBuffer: int
    translate: int
    addBodies: int
    removeBodies: int
    markBodiesDirty: int
    update: bytes
    updateBodyAabbs: int
    buildTaskGraph: bytes
    getExtents: int
    findAllPairs: bytes
    findNewPairs: bytes
    queryAabb: int
    queryAabb_78: bytes
    queryAabb_80: bytes
    queryOutsideOfAabb: int
    queryOutsideOfAabb_90: bytes
    queryAabbNmp: int
    queryAabbNmp_a0: bytes
    queryAabbNmp_a8: bytes
    castAabb: int
    castAabb_b8: bytes
    castRay: int
    castShape: bytes
    queryPoint: int
    getClosestPoints: int
    getClosestPointsAdvanced: int
    getClosestPointsParticles: int
    copyFrom: int

class hknpCollisionCacheWriter(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x28),  # hkTypedBlockStream<hknpCollisionCache>::Writer
    ]
    baseclass_0: bytes

class hknpConstraintSolver_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("solveJacobians", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    solveJacobians: int

class hknpContactConstraintSolver_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("solveJacobians", ctypes.c_ulonglong),
        ("getContactSolverType", ctypes.c_ulonglong),
        ("createInjectors", ctypes.c_ulonglong),
        ("destroyInjectors", ctypes.c_ulonglong),
        ("bindInjectors", ctypes.c_ulonglong),
        ("initMotionTransform", ctypes.c_ulonglong),
        ("injectJacobian", ctypes.c_ulonglong),
        ("setJacRangeStartPoint", ctypes.c_ulonglong),
        ("setJacRangeEndPoint", ctypes.c_ulonglong),
        ("reuseJacobian", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    solveJacobians: int
    getContactSolverType: int
    createInjectors: int
    destroyInjectors: int
    bindInjectors: int
    initMotionTransform: int
    injectJacobian: int
    setJacRangeStartPoint: int
    setJacRangeEndPoint: int
    reuseJacobian: int

class hknpConstraintAtomSolver_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("solveJacobians", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    solveJacobians: int

class hknpVariableTimeStepUtil_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpBodyIntegrator_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcMotionWeldManifoldType", ctypes.c_ulonglong),
        ("processMotionRangeBreach", ctypes.c_ulonglong),
        ("motionWeldToiPlaneBreached", ctypes.c_ulonglong),
        ("motionRangeBreached", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcMotionWeldManifoldType: int
    processMotionRangeBreach: int
    motionWeldToiPlaneBreached: int
    motionRangeBreached: int

class hknpBodyTweaker(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hknpBodyTweaker_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpBodyTweaker_dtor_0", ctypes.c_ulonglong),
        ("tweakBodyInfo", ctypes.c_ulonglong),
    ]
    hknpBodyTweaker_dtor_0: int
    tweakBodyInfo: int

class hknpPotentialManifoldEventCreator_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpManifoldEventCreator_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpContactImpulseEventCreator_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpContactImpulseClippedEventCreator_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpConstraintForceEventCreator_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpConstraintForceExceededEventCreator_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpMotionWeldingModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpWeldingModifier_dtor_0", ctypes.c_ulonglong),
        ("postCompositeCollideCallback", ctypes.c_ulonglong),
    ]
    hknpWeldingModifier_dtor_0: int
    postCompositeCollideCallback: int

class hknpTriangleWeldingModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpWeldingModifier_dtor_0", ctypes.c_ulonglong),
        ("postCompositeCollideCallback", ctypes.c_ulonglong),
    ]
    hknpWeldingModifier_dtor_0: int
    postCompositeCollideCallback: int

class hknpRestitutionModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpSoftContactModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpTriggerModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpSurfaceVelocityModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpAnimatedShapeSurfaceVelocityModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpMassChangerModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpFlipBackfaceCollisionModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpShapeKeyArrayCacheCdDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes

class hknpSetShapeKeyACdDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes

class hknpConvexConvexManifoldGenerator_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    collide: int

class hknpConvexConvexCollisionDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes

class hknpCompositeCollisionDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
        ("collideWithChildren", ctypes.c_ulonglong),
        ("moveAndConsumeChildCachesAndClearJacobianPointers", ctypes.c_ulonglong),
        ("copyChildCaches", ctypes.c_ulonglong),
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes
    collideWithChildren: int
    moveAndConsumeChildCachesAndClearJacobianPointers: int
    copyChildCaches: int

class hknpConvexCompositeCollisionDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
        ("collideWithChildren", ctypes.c_ulonglong),
        ("moveAndConsumeChildCachesAndClearJacobianPointers", ctypes.c_ulonglong),
        ("copyChildCaches", ctypes.c_ulonglong),
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes
    collideWithChildren: int
    moveAndConsumeChildCachesAndClearJacobianPointers: int
    copyChildCaches: int

class hknpCompositeCompositeCollisionDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
        ("collideWithChildren", ctypes.c_ulonglong),
        ("moveAndConsumeChildCachesAndClearJacobianPointers", ctypes.c_ulonglong),
        ("copyChildCaches", ctypes.c_ulonglong),
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes
    collideWithChildren: int
    moveAndConsumeChildCachesAndClearJacobianPointers: int
    copyChildCaches: int

class hknpSignedDistanceFieldCollisionDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
        ("collideWithChildren", ctypes.c_ulonglong),
        ("moveAndConsumeChildCachesAndClearJacobianPointers", ctypes.c_ulonglong),
        ("copyChildCaches", ctypes.c_ulonglong),
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes
    collideWithChildren: int
    moveAndConsumeChildCachesAndClearJacobianPointers: int
    copyChildCaches: int

class hknpBroadphaseTriggerCollisionDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes

class hknpNullCollisionDetector_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCollisionDetector_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("destructCollisionCache", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *)
        ("destructAndKillJacobian", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCollisionDetector *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, hknpCollisionCache *, hknpCollisionCacheBatchConsumer *, hknpBodyIndex, hknpBodyIndex, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>)
    ]
    hknpCollisionDetector_dtor_0: int
    collide: int
    destructCollisionCache: bytes
    destructAndKillJacobian: bytes

class hknpParticleEventCreator_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class hknpDefaultModifierSet_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapeManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpNullShapeTagCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpShapeTagCodec> *(__fastcall *)(hknpShapeTagCodec *this, hkRefPtr<hknpShapeTagCodec> *result)
        ("setMaterialLibrary", ctypes.c_ulonglong),
        ("overrideInitialFilterData", ctypes.c_ulonglong),
        ("decode", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpShapeTagCodec *this, unsigned __int16, hknpCollisionQueryType::Enum, const hknpBody *, const hknpShape *, const hknpShape *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, const hknpShape *, hknpQueryFilterData *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    copyFrom: int
    makeClone: bytes
    setMaterialLibrary: int
    overrideInitialFilterData: int
    decode: bytes

class hknpInternalCommandProcessor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("exec", ctypes.c_ulonglong),
        ("print", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    exec: int
    print: int

class hknpUserCommandDispatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class hknpActionManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkSurfaceConstraintInfo(ctypes.Structure):
    _fields_ = [
        ("m_plane", common.Vector4f),
        ("m_velocity", common.Vector4f),
        ("m_angularVelocity", common.Vector4f),
        ("m_staticFriction", ctypes.c_float),
        ("m_extraUpStaticFriction", ctypes.c_float),
        ("m_extraDownStaticFriction", ctypes.c_float),
        ("m_dynamicFriction", ctypes.c_float),
        ("m_priority", ctypes.c_int32),
    ]
    m_plane: common.Vector4f
    m_velocity: common.Vector4f
    m_angularVelocity: common.Vector4f
    m_staticFriction: float
    m_extraUpStaticFriction: float
    m_extraDownStaticFriction: float
    m_dynamicFriction: float
    m_priority: int

class hkSimplexSolverInput(ctypes.Structure):
    _fields_ = [
        ("m_position", common.Vector4f),
        ("m_velocity", common.Vector4f),
        ("m_maxSurfaceVelocity", common.Vector4f),
        ("m_upVector", common.Vector4f),
        ("m_deltaTime", ctypes.c_float),
        ("m_minDeltaTime", ctypes.c_float),
        ("m_constraints", ctypes.c_ulonglong),
        ("m_numConstraints", ctypes.c_int32),
    ]
    m_position: common.Vector4f
    m_velocity: common.Vector4f
    m_maxSurfaceVelocity: common.Vector4f
    m_upVector: common.Vector4f
    m_deltaTime: float
    m_minDeltaTime: float
    m_constraints: int
    m_numConstraints: int

class hknpCharacterObjectInteractionResult(ctypes.Structure):
    _fields_ = [
        ("m_objectImpulse", common.Vector4f),
        ("m_impulsePosition", common.Vector4f),
    ]
    m_objectImpulse: common.Vector4f
    m_impulsePosition: common.Vector4f

class hknpCharacterProxyListener_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("processConstraintsCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCharacterProxyListener *this, const hknpCharacterProxy *, const hkArray<hknpCharacterProxy::Contact,hkBuiltinContainerAllocator<0,0> > *, hkSimplexSolverInput *)
        ("contactAddedCallback", ctypes.c_ulonglong),
        ("contactRemovedCallback", ctypes.c_ulonglong),
        ("characterInteractionCallback", ctypes.c_ulonglong),
        ("objectInteractionCallback", ctypes.c_ulonglong),
        ("shapeChangedCallback", ctypes.c_ulonglong),
        ("characterCallback", ctypes.c_ulonglong),
        ("triggerVolumeInteractionCallback", ctypes.c_ulonglong),
        ("monitorStreamCallback", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    processConstraintsCallback: bytes
    contactAddedCallback: int
    contactRemovedCallback: int
    characterInteractionCallback: int
    objectInteractionCallback: int
    shapeChangedCallback: int
    characterCallback: int
    triggerVolumeInteractionCallback: int
    monitorStreamCallback: int

class hknpCharacterProxy_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpWorld_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpEventHandlerInput(ctypes.Structure):
    _fields_ = [
        ("m_world", ctypes.c_ulonglong),
        ("m_solverData", ctypes.c_ulonglong),
        ("m_commandWriter", ctypes.c_char * 0x8),  # hkTypedBlockStream<hkCommand>::Writer *
        ("m_simulationThreadContext", ctypes.c_ulonglong),
        ("m_bodiesAreReversed", ctypes.c_uint32),
    ]
    m_world: int
    m_solverData: int
    m_commandWriter: bytes
    m_simulationThreadContext: int
    m_bodiesAreReversed: int

class hkStepInfo(ctypes.Structure):
    def set(): pass

    _fields_ = [
        ("m_startTime", ctypes.c_float),
        ("m_endTime", ctypes.c_float),
        ("m_deltaTime", ctypes.c_float),
        ("m_invDeltaTime", ctypes.c_float),
    ]
    m_startTime: float
    m_endTime: float
    m_deltaTime: float
    m_invDeltaTime: float

class hknpCharacterRigidBodyListener_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterContactPoint", ctypes.c_ulonglong),
        ("onPostCollide", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCharacterRigidBodyListener *this, hknpCharacterRigidBody *, hknpWorld *, hknpSolverData *, hkArray<hknpCharacterRigidBody::ContactPointInfo,hkBuiltinContainerAllocator<0,0> > *)
        ("onPostSolve", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterContactPoint: int
    onPostCollide: bytes
    onPostSolve: int

class hknpCharacterRigidBody_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("onPotentialManifold", ctypes.c_ulonglong),
        ("onCollisionDetected", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCharacterRigidBody *this, const hknpEventHandlerInput *, const hknpBinaryBodyEvent *, hknpManifoldCollisionCache *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkVector4f *, const hkVector4f *, const hkSimdFloat32 *)
        ("onCollisionDetected_28", ctypes.c_ulonglong),
        ("checkSupport", ctypes.c_ulonglong),
        ("findSupportingContacts", ctypes.c_char * 0x8),  # hknpCharacterSurfaceInfo::SupportedState (__fastcall *)(hknpCharacterRigidBody *this, const hkStepInfo *, hkArray<int,hkBuiltinContainerAllocator<0,0> > *)
        ("getGround", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCharacterRigidBody *this, const hkArray<int,hkBuiltinContainerAllocator<0,0> > *, hknpCharacterSurfaceInfo *, const hkStepInfo *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    onPotentialManifold: int
    onCollisionDetected: bytes
    onCollisionDetected_28: int
    checkSupport: int
    findSupportingContacts: bytes
    getGround: bytes

class cTkCharacterRigidBody_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("onPotentialManifold", ctypes.c_ulonglong),
        ("onCollisionDetected", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCharacterRigidBody *this, const hknpEventHandlerInput *, const hknpBinaryBodyEvent *, hknpManifoldCollisionCache *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkVector4f *, const hkVector4f *, const hkSimdFloat32 *)
        ("onCollisionDetected_28", ctypes.c_ulonglong),
        ("checkSupport", ctypes.c_ulonglong),
        ("findSupportingContacts", ctypes.c_char * 0x8),  # hknpCharacterSurfaceInfo::SupportedState (__fastcall *)(hknpCharacterRigidBody *this, const hkStepInfo *, hkArray<int,hkBuiltinContainerAllocator<0,0> > *)
        ("getGround", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCharacterRigidBody *this, const hkArray<int,hkBuiltinContainerAllocator<0,0> > *, hknpCharacterSurfaceInfo *, const hkStepInfo *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    onPotentialManifold: int
    onCollisionDetected: bytes
    onCollisionDetected_28: int
    checkSupport: int
    findSupportingContacts: bytes
    getGround: bytes

class cTkPhysicsConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkPhysicsConstraint_dtor_0", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("Deactivate", ctypes.c_ulonglong),
    ]
    cTkPhysicsConstraint_dtor_0: int
    Activate: int
    Deactivate: int

class hknpRagdollConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class cTkPhysicsComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkStaticPhysicsComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkCameraBehaviour(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpCameraManager", ctypes.c_ulonglong),
    ]
    __vftable: int
    mpCameraManager: int

class cTkCamera_vtbl(ctypes.Structure):
    _fields_ = [
        ("Transform", ctypes.c_ulonglong),
    ]
    Transform: int

class cTkFSM(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def ApplyEvent(): pass

    def ApplyEventToSpecificState(): pass

    def Construct(): pass

    def Destruct(): pass

    def Render(): pass

    def RequestStateChange(): pass

    def StateChange(): pass

    def StatePrepare(): pass

    def StateRelease(): pass

    def StateSupportsEvenOddRendering(): pass

    def Update(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpOffsetTable", ctypes.c_ulonglong),
        ("mpState", ctypes.c_ulonglong),
        ("mRequestedChangeNewState", ctypes.c_ulonglong),
        ("mpRequestedChangeUserData", ctypes.c_void_p),
        ("mbRequestedChangeForceRestart", ctypes.c_ubyte),
    ]
    __vftable: int
    mpOffsetTable: int
    mpState: int
    mRequestedChangeNewState: int
    mpRequestedChangeUserData: int
    mbRequestedChangeForceRestart: bool

class cTkFSMStateOffset(ctypes.Structure):
    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mnOffset", ctypes.c_uint64),
    ]
    mID: int
    mnOffset: int

class cTkFSMState(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def StateChange(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mID", ctypes.c_ulonglong),
        ("mpFSM", ctypes.c_ulonglong),
        ("mu64StateTimingStart", ctypes.c_uint64),
    ]
    __vftable: int
    mID: int
    mpFSM: int
    mu64StateTimingStart: int

class cTkFSMState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class cTkFSM_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSM_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkFSM *this, const cTkFSMStateOffset *, const TkID<128> *)
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("StatePrepare", ctypes.c_ulonglong),
        ("StateRelease", ctypes.c_ulonglong),
    ]
    cTkFSM_dtor_0: int
    Construct: bytes
    Update: int
    Destruct: int
    StatePrepare: int
    StateRelease: int

class cTkCameraManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSM_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkFSM *this, const cTkFSMStateOffset *, const TkID<128> *)
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("StatePrepare", ctypes.c_ulonglong),
        ("StateRelease", ctypes.c_ulonglong),
        ("Construct_30", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetActiveCameraMatrix", ctypes.c_ulonglong),
        ("GetActiveHeadMatrix", ctypes.c_ulonglong),
        ("GetCameraMatrix", ctypes.c_ulonglong),
        ("GetActiveCamera", ctypes.c_ulonglong),
        ("GetCamera", ctypes.c_ulonglong),
        ("IsBehaviourActive", ctypes.c_ulonglong),
        ("SetDebugActive", ctypes.c_ulonglong),
        ("IsDebugActive", ctypes.c_ulonglong),
        ("CopyDebugToGame", ctypes.c_ulonglong),
        ("ActivateBehaviour", ctypes.c_ulonglong),
        ("DeactivateBehaviour", ctypes.c_ulonglong),
        ("SetActiveBehaviour", ctypes.c_ulonglong),
        ("StopBehaviourUpdate", ctypes.c_ulonglong),
        ("StartBehaviourUpdate", ctypes.c_ulonglong),
        ("GetBehaviour", ctypes.c_ulonglong),
        ("GetBehaviourIndex", ctypes.c_ulonglong),
    ]
    cTkFSM_dtor_0: int
    Construct: bytes
    Update: int
    Destruct: int
    StatePrepare: int
    StateRelease: int
    Construct_30: int
    Prepare: int
    Release: int
    GetActiveCameraMatrix: int
    GetActiveHeadMatrix: int
    GetCameraMatrix: int
    GetActiveCamera: int
    GetCamera: int
    IsBehaviourActive: int
    SetDebugActive: int
    IsDebugActive: int
    CopyDebugToGame: int
    ActivateBehaviour: int
    DeactivateBehaviour: int
    SetActiveBehaviour: int
    StopBehaviourUpdate: int
    StartBehaviourUpdate: int
    GetBehaviour: int
    GetBehaviourIndex: int

class cTkCameraBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cTkHavokCharacterController_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class cGcCameraBehaviourCharacter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCharacterInterface(ctypes.Structure):
    def GetHitReactDir(): pass

    def GetRidingAttachment(): pass

    def UpdateIdle(): pass

    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcNetworkComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("Receive", ctypes.c_ulonglong),
        ("Respond", ctypes.c_ulonglong * 0x400),  # void (__fastcall *)(cGcNetworkComponent *this, cGcNetworkPlayer *, const unsigned __int8 *, unsigned int, unsigned __int8 (*)[1024], unsigned int *)
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    Receive: int
    Respond: list[ctypes.c_ulonglong]

class sSketchValue: pass

class sSketchStack(ctypes.Structure):
    def CheckBufferSpace(): pass

    def FreeBuffer(): pass

    _fields_ = [
        ("kiSizeIncrease", ctypes.c_uint32),
        ("mpStackBuffer", ctypes.c_ulonglong),
        ("mBufferPosition", ctypes.c_uint32),
        ("mBufferUsedBytes", ctypes.c_uint32),
        ("mBufferSizeBytes", ctypes.c_uint32),
    ]
    kiSizeIncrease: int
    mpStackBuffer: int
    mBufferPosition: int
    mBufferUsedBytes: int
    mBufferSizeBytes: int

class cTkSketchComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("Receive", ctypes.c_ulonglong),
        ("Respond", ctypes.c_ulonglong * 0x400),  # void (__fastcall *)(cGcNetworkComponent *this, cGcNetworkPlayer *, const unsigned __int8 *, unsigned int, unsigned __int8 (*)[1024], unsigned int *)
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    Receive: int
    Respond: list[ctypes.c_ulonglong]

class cGcPlayerControlComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCharacterInterface_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCharacterInterface_dtor_0", ctypes.c_ulonglong),
        ("IsPlayer", ctypes.c_ulonglong),
        ("IsReady", ctypes.c_ulonglong),
        ("IsLocalPlayer", ctypes.c_ulonglong),
        ("IsVRPlayer", ctypes.c_ulonglong),
        ("WeaponIsActive", ctypes.c_ulonglong),
        ("WeaponIsLarge", ctypes.c_ulonglong),
        ("BinocsActive", ctypes.c_ulonglong),
        ("IsHitReacting", ctypes.c_ulonglong),
        ("GetHitReactDir", ctypes.c_ulonglong),
        ("IsOnGround", ctypes.c_ulonglong),
        ("IsInSpace", ctypes.c_ulonglong),
        ("ShouldUseAbandonedFreighterWalk", ctypes.c_ulonglong),
        ("IsDying", ctypes.c_ulonglong),
        ("GetDeathReason", ctypes.c_ulonglong),
        ("GetEnergy", ctypes.c_ulonglong),
        ("GetHazard", ctypes.c_ulonglong),
        ("IsJetpacking", ctypes.c_ulonglong),
        ("IsJetpackBoosted", ctypes.c_ulonglong),
        ("IsSliding", ctypes.c_ulonglong),
        ("IsRiding", ctypes.c_ulonglong),
        ("GetRidingAttachment", ctypes.c_ulonglong),
        ("GetHandPose", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcCharacterInterface *this, eHand, TkID<128> *, float *)
        ("GetDominantHand", ctypes.c_ulonglong),
        ("IsMeleeActive", ctypes.c_ulonglong),
        ("IsPointing", ctypes.c_ulonglong),
        ("IsInteracting", ctypes.c_ulonglong),
        ("IsMeleeBoostActive", ctypes.c_ulonglong),
        ("IsWaitingForRocketBootsActivation", ctypes.c_ulonglong),
        ("IsRocketBootsActive", ctypes.c_ulonglong),
        ("GetLocalTeleportDestination", ctypes.c_ulonglong),
        ("GetTimeLastUsedJetpack", ctypes.c_ulonglong),
        ("IsRunning", ctypes.c_ulonglong),
        ("IsUnderwater", ctypes.c_ulonglong),
        ("IsAtWaterSurface", ctypes.c_ulonglong),
        ("IsGrabbed", ctypes.c_ulonglong),
        ("IsMech", ctypes.c_ulonglong),
        ("HasMoveForce", ctypes.c_ulonglong),
        ("HasFired", ctypes.c_ulonglong),
        ("IsPilotingShip", ctypes.c_ulonglong),
        ("IsInVehicle", ctypes.c_ulonglong),
        ("HasCustomIdle", ctypes.c_ulonglong),
        ("UpdateIdle", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcCharacterInterface *this, TkID<128> *result, float)
        ("GetSlopeState", ctypes.c_ulonglong),
        ("IsHandTracking", ctypes.c_ulonglong),
        ("GetMatrix", ctypes.c_ulonglong),
        ("GetPhysicsMatrix", ctypes.c_ulonglong),
        ("GetFootPosition", ctypes.c_ulonglong),
        ("GetVelocity", ctypes.c_ulonglong),
        ("GetFacingDir", ctypes.c_ulonglong),
        ("GetWorldUp", ctypes.c_ulonglong),
        ("GetHeight", ctypes.c_ulonglong),
        ("GetRace", ctypes.c_ulonglong),
        ("GetAimPosition", ctypes.c_ulonglong),
        ("GetCurrentMoveForce", ctypes.c_ulonglong),
        ("SetToPosition", ctypes.c_ulonglong),
        ("SetToPosition_1c0", ctypes.c_ulonglong),
        ("EnableCharacterPhysics", ctypes.c_ulonglong),
        ("EnableCharacterCollision", ctypes.c_ulonglong),
        ("DisableCharacterCollision", ctypes.c_ulonglong),
        ("AttachControlMode", ctypes.c_ulonglong),
        ("EnableCharacterGravity", ctypes.c_ulonglong),
        ("DisableCharacterGravity", ctypes.c_ulonglong),
        ("GetRigidBody", ctypes.c_ulonglong),
        ("GetCharacterSit", ctypes.c_ulonglong),
        ("TakeDamage", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCharacterInterface *this, float, eDamageType, const TkID<128> *, const cTkVector3 *, cGcOwnerConcept *)
        ("SetPitch", ctypes.c_ulonglong),
        ("SetRoll", ctypes.c_ulonglong),
        ("CanLookAt", ctypes.c_ulonglong),
        ("AllowHeadMovement", ctypes.c_ulonglong),
        ("GetLookAtPoint", ctypes.c_ulonglong),
    ]
    cGcCharacterInterface_dtor_0: int
    IsPlayer: int
    IsReady: int
    IsLocalPlayer: int
    IsVRPlayer: int
    WeaponIsActive: int
    WeaponIsLarge: int
    BinocsActive: int
    IsHitReacting: int
    GetHitReactDir: int
    IsOnGround: int
    IsInSpace: int
    ShouldUseAbandonedFreighterWalk: int
    IsDying: int
    GetDeathReason: int
    GetEnergy: int
    GetHazard: int
    IsJetpacking: int
    IsJetpackBoosted: int
    IsSliding: int
    IsRiding: int
    GetRidingAttachment: int
    GetHandPose: bytes
    GetDominantHand: int
    IsMeleeActive: int
    IsPointing: int
    IsInteracting: int
    IsMeleeBoostActive: int
    IsWaitingForRocketBootsActivation: int
    IsRocketBootsActive: int
    GetLocalTeleportDestination: int
    GetTimeLastUsedJetpack: int
    IsRunning: int
    IsUnderwater: int
    IsAtWaterSurface: int
    IsGrabbed: int
    IsMech: int
    HasMoveForce: int
    HasFired: int
    IsPilotingShip: int
    IsInVehicle: int
    HasCustomIdle: int
    UpdateIdle: bytes
    GetSlopeState: int
    IsHandTracking: int
    GetMatrix: int
    GetPhysicsMatrix: int
    GetFootPosition: int
    GetVelocity: int
    GetFacingDir: int
    GetWorldUp: int
    GetHeight: int
    GetRace: int
    GetAimPosition: int
    GetCurrentMoveForce: int
    SetToPosition: int
    SetToPosition_1c0: int
    EnableCharacterPhysics: int
    EnableCharacterCollision: int
    DisableCharacterCollision: int
    AttachControlMode: int
    EnableCharacterGravity: int
    DisableCharacterGravity: int
    GetRigidBody: int
    GetCharacterSit: int
    TakeDamage: bytes
    SetPitch: int
    SetRoll: int
    CanLookAt: int
    AllowHeadMovement: int
    GetLookAtPoint: int

class TkAudioObject(ctypes.Structure):
    _fields_ = [
        ("mID", ctypes.c_uint64),
    ]
    mID: int

class cTkAnimationBlendWeight(ctypes.Structure):
    def SetWeight(): pass

    _fields_ = [
        ("mBlendId", ctypes.c_ulonglong),
        ("mpWeightInput", ctypes.c_ulonglong),
        ("mpNodeData", ctypes.c_ulonglong),
        ("mfWeight", ctypes.c_float),
        ("mfWeightVel", ctypes.c_float),
        ("mbIgnoreParamStruct", ctypes.c_ubyte),
        ("miMemberOffset", ctypes.c_uint8),
    ]
    mBlendId: int
    mpWeightInput: int
    mpNodeData: int
    mfWeight: float
    mfWeightVel: float
    mbIgnoreParamStruct: bool
    miMemberOffset: int

class cTkNodeAnimationController(ctypes.Structure):
    def tilde_cTkNodeAnimationController(): pass

    def Activate(): pass

    def BlendTo(): pass

    def ClearAnimTargetDelta(): pass

    def Construct(): pass

    def FreezeAllAnims(): pass

    def GenerateCommandList(): pass

    def GetAnimActionFrame(): pass

    def GetAnimActionStartFrame(): pass

    def GetAnimBlendedLocator(): pass

    def GetAnimDuration(): pass

    def GetAnimFrame(): pass

    def GetAnimFrameInt(): pass

    def GetAnimFromLayerById(): pass

    def GetAnimLocator(): pass

    def GetAnimMirrored(): pass

    def GetAnimMoveLocatorSpeed(): pass

    def GetAnimName(): pass

    def GetAnimSpeed(): pass

    def GetAnimTime(): pass

    def GetAnimType(): pass

    def GetAnimUnscaledDuration(): pass

    def GetBlendFromLayerById(): pass

    def GetBlendNodeFromLayer(): pass

    def GetFirstAnim(): pass

    def GetHandleForAnim(): pass

    def GetHandleFromLayer(): pass

    def GetLayerFromHandle(): pass

    def GetLayerFromId(): pass

    def GetLayerState(): pass

    def GetLocatorForAnim(): pass

    def GetNextAnim(): pass

    def GetNumAnimFrames(): pass

    def LoadAnimation(): pass

    def MapAnimation(): pass

    def Prepare(): pass

    def RefreshParamMembers(): pass

    def SetAnimFakeMoveLocatorSpeed(): pass

    def SetAnimMirrored(): pass

    def SetAnimSpeed(): pass

    def SetAnimTargetDelta(): pass

    def SetAnimTime(): pass

    def SetAnimTimeWithRestart(): pass

    def SetAnimWeight(): pass

    def SetBlendWeight(): pass

    def SetCanBeRemovedEvenIfControl(): pass

    def SetClock(): pass

    def SetLayerClock(): pass

    def SetLayerId(): pass

    def SetLayerReplication(): pass

    def SetMirrored(): pass

    def SetParamStruct(): pass

    def Stop(): pass

    def Update(): pass

    def UpdateActiveList(): pass

    _fields_ = [
        ("mCommandBuffer", ctypes.c_char * 0xF0),  # cTkStackVector<cTkAnimCommand,16,-1>
        ("mActiveStages", ctypes.c_char * 0x70),  # cTkStackVector<unsigned int,16,-1>
        ("mbDirty", ctypes.c_ubyte),
        ("mAnimChannelBitArray", ctypes.c_char * 0x4),  # cTkBitArray<unsigned int,1,32>
        ("maAnimLayer", ctypes.c_char * 0x1600),  # std::array<cTkAnimationLayer,16>
        ("mModelNode", ctypes.c_uint32),
        ("mpaAnimData", ctypes.c_ulonglong),
        ("mpaAnimResources", ctypes.c_ulonglong),
        ("mpaAnimMappings", ctypes.c_ulonglong),
        ("miNumAnimations", ctypes.c_int32),
        ("mpaTreeData", ctypes.c_ulonglong),
        ("miNumTrees", ctypes.c_int32),
        ("mAlternateAnimations", ctypes.c_char * 0x40),  # std::unordered_map<TkID<128>,TkID<128>,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,TkID<128> >,8,-1> >
        ("mpClock", ctypes.c_ulonglong),
        ("mpParamStruct", ctypes.c_void_p),
        ("mpParamMembers", ctypes.c_ulonglong),
        ("mfCurrentSpeed", ctypes.c_float),
        ("miCurrentLOD", ctypes.c_int32),
    ]
    mCommandBuffer: bytes
    mActiveStages: bytes
    mbDirty: bool
    mAnimChannelBitArray: bytes
    maAnimLayer: bytes
    mModelNode: int
    mpaAnimData: int
    mpaAnimResources: int
    mpaAnimMappings: int
    miNumAnimations: int
    mpaTreeData: int
    miNumTrees: int
    mAlternateAnimations: bytes
    mpClock: int
    mpParamStruct: int
    mpParamMembers: int
    mfCurrentSpeed: float
    miCurrentLOD: int

class cTkAnimationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkAnimLayerHandle(ctypes.Structure):
    def IsValid(): pass

    _fields_ = [
        ("miChannelIndex", ctypes.c_int32),
        ("miAnimIndex", ctypes.c_int32),
        ("miUid", ctypes.c_int32),
    ]
    miChannelIndex: int
    miAnimIndex: int
    miUid: int

class cGcFadeNodeBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("SetNodeActivation", ctypes.c_ulonglong),
        ("CheckNodeActivation", ctypes.c_ulonglong),
        ("SetNodeParamF", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
    ]
    SetNodeActivation: int
    CheckNodeActivation: int
    SetNodeParamF: int
    Update: int

class cGcFadeNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("SetNodeActivation", ctypes.c_ulonglong),
        ("CheckNodeActivation", ctypes.c_ulonglong),
        ("SetNodeParamF", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
    ]
    SetNodeActivation: int
    CheckNodeActivation: int
    SetNodeParamF: int
    Update: int

class EffectInstance(ctypes.Structure):
    _fields_ = [
        ("mNodeHandle", ctypes.c_uint32),
        ("mpExplosion", ctypes.c_ulonglong),
        ("mpExplosionData", ctypes.c_ulonglong),
    ]
    mNodeHandle: int
    mpExplosion: int
    mpExplosionData: int

class cTkEulerVector(ctypes.Structure):
    def cTkEulerVector(): pass

    _fields_ = [
        ("mV", common.Vector3f),
    ]
    mV: common.Vector3f

class cTkJacobianRow(ctypes.Structure):
    _fields_ = [
        ("miStateSize", ctypes.c_int32),
        ("miStateSizeAlignedUp", ctypes.c_int32),
        ("mfValue", ctypes.c_float),
        ("mafDerivs", ctypes.c_ulonglong),
    ]
    miStateSize: int
    miStateSizeAlignedUp: int
    mfValue: float
    mafDerivs: int

class cTkIKConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cTkAudioComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkIKJointHeightConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cGcHandTrackingConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
        ("cGcHandTrackingConstraint_dtor_40", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int
    cGcHandTrackingConstraint_dtor_40: int

class cGcHeadConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
        ("cGcHeadConstraint_dtor_40", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int
    cGcHeadConstraint_dtor_40: int

class cGcFootConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
        ("cGcFootConstraint_dtor_40", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int
    cGcFootConstraint_dtor_40: int

class cTkIKCenterOfGravityConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cGcCreatureCogConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cTkIKJointXZPushConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class TkJobHandle(ctypes.Structure):
    _fields_ = [
        ("miQueue", ctypes.c_int32),
        ("miIndex", ctypes.c_int32),
        ("muCount", ctypes.c_uint64),
    ]
    miQueue: int
    miIndex: int
    muCount: int

class cTkRigidBodyPtr(ctypes.Structure):
    _fields_ = [
        ("miBodyIndex", ctypes.c_int32),
        ("miUniqueID", ctypes.c_int64),
    ]
    miBodyIndex: int
    miUniqueID: int

class cGcPlayerFullBodyIKComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcNetworkInterpolationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcPlayerCharacterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcPlayerEffect(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpEffectComponent", ctypes.c_ulonglong),
        ("mfSpeed", ctypes.c_float),
        ("mbStarted", ctypes.c_ubyte),
        ("mbFinished", ctypes.c_ubyte),
    ]
    __vftable: int
    mpEffectComponent: int
    mfSpeed: float
    mbStarted: bool
    mbFinished: bool

class cGcPlayerEffect_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("FreezePlayerMovement", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Start: int
    Update: int
    End: int
    Release: int
    FreezePlayerMovement: int

class cGcPlayerDissapearEffect_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("FreezePlayerMovement", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Start: int
    Update: int
    End: int
    Release: int
    FreezePlayerMovement: int

class cGcPlayerEffectsComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkHitCurve(ctypes.Structure):
    _fields_ = [
        ("mpData", ctypes.c_ulonglong),
        ("mafHitTimes", ctypes.c_char * 0x18),  # std::vector<std::pair<float,float>,TkSTLAllocatorShim<std::pair<float,float>,4,-1> >
    ]
    mpData: int
    mafHitTimes: bytes

class cGcShootableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("Receive", ctypes.c_ulonglong),
        ("Respond", ctypes.c_ulonglong * 0x400),  # void (__fastcall *)(cGcNetworkComponent *this, cGcNetworkPlayer *, const unsigned __int8 *, unsigned int, unsigned __int8 (*)[1024], unsigned int *)
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    Receive: int
    Respond: list[ctypes.c_ulonglong]

class cGcCombatEffectsComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCameraDepthOfField(ctypes.Structure):
    def GetParams(): pass

    def ReadParams(): pass

    _fields_ = [
        ("mbDofEnabled", ctypes.c_ubyte),
        ("mfDofFocusDistance", ctypes.c_float),
        ("mfDofBlend", ctypes.c_float),
        ("mfDofNearPlaneMinInteraction", ctypes.c_float),
        ("mfDofNearPlaneAdjustInteraction", ctypes.c_float),
        ("mfDofFarPlaneInteraction", ctypes.c_float),
        ("mfDOFFarFadeDistanceInteraction", ctypes.c_float),
    ]
    mbDofEnabled: bool
    mfDofFocusDistance: float
    mfDofBlend: float
    mfDofNearPlaneMinInteraction: float
    mfDofNearPlaneAdjustInteraction: float
    mfDofFarPlaneInteraction: float
    mfDOFFarFadeDistanceInteraction: float

class cGcGameCamera_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetDof", ctypes.c_ulonglong),
        ("ReadDofParameters", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetDof: int
    ReadDofParameters: int

class cGcPlayerController(ctypes.Structure):
    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcPlayerController_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcPlayerController_dtor_0", ctypes.c_ulonglong),
        ("GetButtonInput", ctypes.c_ulonglong),
        ("GetAnalogInput", ctypes.c_ulonglong),
    ]
    cGcPlayerController_dtor_0: int
    GetButtonInput: int
    GetAnalogInput: int

class cGcPlayerStick(ctypes.Structure):
    def GetBaseStick(): pass

    def Construct(): pass

    def Reset(): pass

    def GetSensitivityMultiplier(): pass

    def Update(): pass

    def RenderNGui(): pass

    _fields_ = [
        ("mpController", ctypes.c_ulonglong),
        ("mpData", ctypes.c_ulonglong),
        ("miXAxis", ctypes.c_int32),
        ("miYAxis", ctypes.c_int32),
        ("mBaseStick", common.Vector2f),
        ("mLastStickDir", ctypes.c_char * 0x10),  # cTkSmoothCD<cTkVector2>
        ("mLastInput", common.Vector3f),
        ("mfAcceleration", ctypes.c_float),
        ("mfAccelerationTimer", ctypes.c_float),
        ("_meSensitivityType", ctypes.c_uint32),
        ("mMousePos", common.Vector2f),
        ("mfAutoAimStickyFactor", ctypes.c_float),
        ("mNormalRateColour", common.Colour),
        ("mAcceleratedRateColour", common.Colour),
        ("mDebugScreenCircleCentre", common.Vector2f),
        ("mfDebugScreenCircleRadius", ctypes.c_float),
        ("mfDebugScreenCircleRadiusInner", ctypes.c_float),
        ("mGyroChanges", common.Vector2f),
        ("miSmoothingIndex", ctypes.c_int32),
        ("maSmoothingGyroBuffer", common.Vector2f * 0x10),  # cTkVector2[16]
        ("mfSpecialVehicleMouseRecentreTimer", ctypes.c_float),
    ]
    mpController: int
    mpData: int
    miXAxis: int
    miYAxis: int
    mBaseStick: common.Vector2f
    mLastStickDir: bytes
    mLastInput: common.Vector3f
    mfAcceleration: float
    mfAccelerationTimer: float
    _meSensitivityType: int
    mMousePos: common.Vector2f
    mfAutoAimStickyFactor: float
    mNormalRateColour: common.Colour
    mAcceleratedRateColour: common.Colour
    mDebugScreenCircleCentre: common.Vector2f
    mfDebugScreenCircleRadius: float
    mfDebugScreenCircleRadiusInner: float
    mGyroChanges: common.Vector2f
    miSmoothingIndex: int
    maSmoothingGyroBuffer: list[common.Vector2f]
    mfSpecialVehicleMouseRecentreTimer: float

    @property
    def meSensitivityType(self):
        return cGcUserSettings.Sensitivity(self._meSensitivityType)


class cGcCameraBehaviourFirstPerson_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetDof", ctypes.c_ulonglong),
        ("ReadDofParameters", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetDof: int
    ReadDofParameters: int

class cGcWeapon(ctypes.Structure):
    def GetActiveInventoryItem(): pass

    def GetCurrentRange(): pass

    def GetCurrentSpeed(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcScanEffectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcWeapon_vtbl(ctypes.Structure):
    _fields_ = [
        ("UsingAmmo", ctypes.c_ulonglong),
        ("UsesInventoryAmmo", ctypes.c_ulonglong),
        ("GetAvailableAmmo", ctypes.c_ulonglong),
        ("GetClipSize", ctypes.c_ulonglong),
        ("IsReloading", ctypes.c_ulonglong),
        ("GetReloadProgress", ctypes.c_ulonglong),
        ("GetLastFireTime", ctypes.c_ulonglong),
        ("HasOverheated", ctypes.c_ulonglong),
        ("GetOverHeatTime", ctypes.c_ulonglong),
        ("GetOverheatProgress", ctypes.c_ulonglong),
        ("IsOnCooldown", ctypes.c_ulonglong),
        ("GetCooldownDuration", ctypes.c_ulonglong),
        ("GetCooldownProgress", ctypes.c_ulonglong),
        ("GetAmmoInClip", ctypes.c_ulonglong),
        ("GetInventoryItem", ctypes.c_ulonglong),
        ("GetActiveInventoryItem", ctypes.c_ulonglong),
        ("GetActiveSecondaryInventoryItem", ctypes.c_ulonglong),
        ("GetHeatFactor", ctypes.c_ulonglong),
        ("GetHeatWarningFactor", ctypes.c_ulonglong),
        ("GetAlertPoint", ctypes.c_ulonglong),
        ("GetCooldownFactor", ctypes.c_ulonglong),
        ("GetModeIndex", ctypes.c_ulonglong),
        ("GetLastModeChangeTime", ctypes.c_ulonglong),
        ("GetNumModes", ctypes.c_ulonglong),
        ("SetModeIndex", ctypes.c_ulonglong),
        ("GetWeaponRange", ctypes.c_ulonglong),
        ("GetWeaponSpeed", ctypes.c_ulonglong),
        ("GetPrimaryStat", ctypes.c_ulonglong),
        ("IsCriticalShot", ctypes.c_ulonglong),
        ("GetListOfModesIfActiveOrNot", ctypes.c_char * 0x8),  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *(__fastcall *)(cGcWeapon *this, std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *result)
        ("GetChargeFactor", ctypes.c_ulonglong),
        ("IsCharging", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("IsVehicleWeapon", ctypes.c_ulonglong),
        ("GetTerrainEditMode", ctypes.c_ulonglong),
        ("GetTerrainEditorBeam", ctypes.c_ulonglong),
        ("UsingTerrainAdvancedMode", ctypes.c_ulonglong),
        ("ShowingTerrainEditNotifies", ctypes.c_ulonglong),
        ("ResetTerrainMode", ctypes.c_ulonglong),
        ("CanCharge", ctypes.c_ulonglong),
    ]
    UsingAmmo: int
    UsesInventoryAmmo: int
    GetAvailableAmmo: int
    GetClipSize: int
    IsReloading: int
    GetReloadProgress: int
    GetLastFireTime: int
    HasOverheated: int
    GetOverHeatTime: int
    GetOverheatProgress: int
    IsOnCooldown: int
    GetCooldownDuration: int
    GetCooldownProgress: int
    GetAmmoInClip: int
    GetInventoryItem: int
    GetActiveInventoryItem: int
    GetActiveSecondaryInventoryItem: int
    GetHeatFactor: int
    GetHeatWarningFactor: int
    GetAlertPoint: int
    GetCooldownFactor: int
    GetModeIndex: int
    GetLastModeChangeTime: int
    GetNumModes: int
    SetModeIndex: int
    GetWeaponRange: int
    GetWeaponSpeed: int
    GetPrimaryStat: int
    IsCriticalShot: int
    GetListOfModesIfActiveOrNot: bytes
    GetChargeFactor: int
    IsCharging: int
    IsActive: int
    IsVehicleWeapon: int
    GetTerrainEditMode: int
    GetTerrainEditorBeam: int
    UsingTerrainAdvancedMode: int
    ShowingTerrainEditNotifies: int
    ResetTerrainMode: int
    CanCharge: int

class cGcNGuiElement_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
        ("cGcNGuiElement_dtor_10", ctypes.c_ulonglong),
        ("EditElement", ctypes.c_ulonglong),
        ("EditPresets", ctypes.c_ulonglong),
        ("RenderUIEditorToolbar", ctypes.c_ulonglong),
        ("SelectableInEditor", ctypes.c_ulonglong),
        ("SetNeedsUpdate", ctypes.c_ulonglong),
        ("Deselect", ctypes.c_ulonglong),
        ("IsSelected", ctypes.c_ulonglong),
        ("Resized", ctypes.c_ulonglong),
        ("AttachMetadata", ctypes.c_ulonglong),
        ("DetachMetadata", ctypes.c_ulonglong),
        ("IsLayer", ctypes.c_ulonglong),
        ("GetSceneTreeIcon", ctypes.c_ulonglong),
        ("GetSceneTreeText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNGuiElement *this, cTkFixedString<128,char> *)
        ("RenderAdditionalOverlayIcons", ctypes.c_ulonglong),
        ("RenderTreeControls", ctypes.c_ulonglong),
    ]
    Render: int
    GetType: int
    cGcNGuiElement_dtor_10: int
    EditElement: int
    EditPresets: int
    RenderUIEditorToolbar: int
    SelectableInEditor: int
    SetNeedsUpdate: int
    Deselect: int
    IsSelected: int
    Resized: int
    AttachMetadata: int
    DetachMetadata: int
    IsLayer: int
    GetSceneTreeIcon: int
    GetSceneTreeText: bytes
    RenderAdditionalOverlayIcons: int
    RenderTreeControls: int

class cTkHashedNGuiElement(ctypes.Structure):
    def cTkHashedNGuiElement(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mHash", ctypes.c_uint64),
    ]
    mID: int
    mHash: int

class cGcNGuiLayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
        ("cGcNGuiElement_dtor_10", ctypes.c_ulonglong),
        ("EditElement", ctypes.c_ulonglong),
        ("EditPresets", ctypes.c_ulonglong),
        ("RenderUIEditorToolbar", ctypes.c_ulonglong),
        ("SelectableInEditor", ctypes.c_ulonglong),
        ("SetNeedsUpdate", ctypes.c_ulonglong),
        ("Deselect", ctypes.c_ulonglong),
        ("IsSelected", ctypes.c_ulonglong),
        ("Resized", ctypes.c_ulonglong),
        ("AttachMetadata", ctypes.c_ulonglong),
        ("DetachMetadata", ctypes.c_ulonglong),
        ("IsLayer", ctypes.c_ulonglong),
        ("GetSceneTreeIcon", ctypes.c_ulonglong),
        ("GetSceneTreeText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNGuiElement *this, cTkFixedString<128,char> *)
        ("RenderAdditionalOverlayIcons", ctypes.c_ulonglong),
        ("RenderTreeControls", ctypes.c_ulonglong),
        ("GetSelected", ctypes.c_ulonglong),
    ]
    Render: int
    GetType: int
    cGcNGuiElement_dtor_10: int
    EditElement: int
    EditPresets: int
    RenderUIEditorToolbar: int
    SelectableInEditor: int
    SetNeedsUpdate: int
    Deselect: int
    IsSelected: int
    Resized: int
    AttachMetadata: int
    DetachMetadata: int
    IsLayer: int
    GetSceneTreeIcon: int
    GetSceneTreeText: bytes
    RenderAdditionalOverlayIcons: int
    RenderTreeControls: int
    GetSelected: int

class cTk2dObject(ctypes.Structure):
    def Construct(): pass

    def GetWorldTopLeft(): pass

    def GetPosition(): pass

    def SetPosition(): pass

    def SetSize(): pass

    def GetSize(): pass

    def SetAlignment(): pass

    def GetAlignment(): pass

    def SetColour(): pass

    def GetColour(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mColour", common.Colour),
        ("mPosition", common.Vector2f),
        ("mSize", common.Vector2f),
        ("mAlignment", common.Vector2f),
        ("mpNextSibling", ctypes.c_ulonglong),
    ]
    __vftable: int
    mColour: common.Colour
    mPosition: common.Vector2f
    mSize: common.Vector2f
    mAlignment: common.Vector2f
    mpNextSibling: int

class cTk2dObject_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int

class cTk2dLayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
        ("ConstructDynamicSize", ctypes.c_ulonglong),
        ("Construct_90", ctypes.c_ulonglong),
        ("SetScale", ctypes.c_ulonglong),
        ("SetAngle", ctypes.c_ulonglong),
        ("GetAngle", ctypes.c_ulonglong),
        ("CalculateSize", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int
    ConstructDynamicSize: int
    Construct_90: int
    SetScale: int
    SetAngle: int
    GetAngle: int
    CalculateSize: int

class cTk3dLayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
        ("ConstructDynamicSize", ctypes.c_ulonglong),
        ("Construct_90", ctypes.c_ulonglong),
        ("SetScale", ctypes.c_ulonglong),
        ("SetAngle", ctypes.c_ulonglong),
        ("GetAngle", ctypes.c_ulonglong),
        ("CalculateSize", ctypes.c_ulonglong),
        ("ConstructDynamicSize_b8", ctypes.c_ulonglong),
        ("Construct_c0", ctypes.c_ulonglong),
        ("SetPosition_c8", ctypes.c_ulonglong),
        ("SetPosition_d0", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int
    ConstructDynamicSize: int
    Construct_90: int
    SetScale: int
    SetAngle: int
    GetAngle: int
    CalculateSize: int
    ConstructDynamicSize_b8: int
    Construct_c0: int
    SetPosition_c8: int
    SetPosition_d0: int

class cTk2dImage_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int

class cTkClassPoolHandle(ctypes.Structure):
    _fields_ = [
        ("miArrayIndex", ctypes.c_int32),
        ("miUniqueID", ctypes.c_int32),
    ]
    miArrayIndex: int
    miUniqueID: int

class cGcPlayerWeapon_vtbl(ctypes.Structure):
    _fields_ = [
        ("UsingAmmo", ctypes.c_ulonglong),
        ("UsesInventoryAmmo", ctypes.c_ulonglong),
        ("GetAvailableAmmo", ctypes.c_ulonglong),
        ("GetClipSize", ctypes.c_ulonglong),
        ("IsReloading", ctypes.c_ulonglong),
        ("GetReloadProgress", ctypes.c_ulonglong),
        ("GetLastFireTime", ctypes.c_ulonglong),
        ("HasOverheated", ctypes.c_ulonglong),
        ("GetOverHeatTime", ctypes.c_ulonglong),
        ("GetOverheatProgress", ctypes.c_ulonglong),
        ("IsOnCooldown", ctypes.c_ulonglong),
        ("GetCooldownDuration", ctypes.c_ulonglong),
        ("GetCooldownProgress", ctypes.c_ulonglong),
        ("GetAmmoInClip", ctypes.c_ulonglong),
        ("GetInventoryItem", ctypes.c_ulonglong),
        ("GetActiveInventoryItem", ctypes.c_ulonglong),
        ("GetActiveSecondaryInventoryItem", ctypes.c_ulonglong),
        ("GetHeatFactor", ctypes.c_ulonglong),
        ("GetHeatWarningFactor", ctypes.c_ulonglong),
        ("GetAlertPoint", ctypes.c_ulonglong),
        ("GetCooldownFactor", ctypes.c_ulonglong),
        ("GetModeIndex", ctypes.c_ulonglong),
        ("GetLastModeChangeTime", ctypes.c_ulonglong),
        ("GetNumModes", ctypes.c_ulonglong),
        ("SetModeIndex", ctypes.c_ulonglong),
        ("GetWeaponRange", ctypes.c_ulonglong),
        ("GetWeaponSpeed", ctypes.c_ulonglong),
        ("GetPrimaryStat", ctypes.c_ulonglong),
        ("IsCriticalShot", ctypes.c_ulonglong),
        ("GetListOfModesIfActiveOrNot", ctypes.c_char * 0x8),  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *(__fastcall *)(cGcWeapon *this, std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *result)
        ("GetChargeFactor", ctypes.c_ulonglong),
        ("IsCharging", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("IsVehicleWeapon", ctypes.c_ulonglong),
        ("GetTerrainEditMode", ctypes.c_ulonglong),
        ("GetTerrainEditorBeam", ctypes.c_ulonglong),
        ("UsingTerrainAdvancedMode", ctypes.c_ulonglong),
        ("ShowingTerrainEditNotifies", ctypes.c_ulonglong),
        ("ResetTerrainMode", ctypes.c_ulonglong),
        ("CanCharge", ctypes.c_ulonglong),
    ]
    UsingAmmo: int
    UsesInventoryAmmo: int
    GetAvailableAmmo: int
    GetClipSize: int
    IsReloading: int
    GetReloadProgress: int
    GetLastFireTime: int
    HasOverheated: int
    GetOverHeatTime: int
    GetOverheatProgress: int
    IsOnCooldown: int
    GetCooldownDuration: int
    GetCooldownProgress: int
    GetAmmoInClip: int
    GetInventoryItem: int
    GetActiveInventoryItem: int
    GetActiveSecondaryInventoryItem: int
    GetHeatFactor: int
    GetHeatWarningFactor: int
    GetAlertPoint: int
    GetCooldownFactor: int
    GetModeIndex: int
    GetLastModeChangeTime: int
    GetNumModes: int
    SetModeIndex: int
    GetWeaponRange: int
    GetWeaponSpeed: int
    GetPrimaryStat: int
    IsCriticalShot: int
    GetListOfModesIfActiveOrNot: bytes
    GetChargeFactor: int
    IsCharging: int
    IsActive: int
    IsVehicleWeapon: int
    GetTerrainEditMode: int
    GetTerrainEditorBeam: int
    UsingTerrainAdvancedMode: int
    ShowingTerrainEditNotifies: int
    ResetTerrainMode: int
    CanCharge: int

class cGcGalacticVoxelCoordinate(ctypes.Structure):
    def cGcGalacticVoxelCoordinate(): pass

    def ClampToLimits(): pass

    _fields_ = [
        ("mX", ctypes.c_int16),
        ("mZ", ctypes.c_int16),
        ("mY", ctypes.c_int16),
        ("mbValid", ctypes.c_ubyte),
    ]
    mX: int
    mZ: int
    mY: int
    mbValid: bool

class TkAudioID(ctypes.Structure):
    def TkAudioID(): pass

    _fields_ = [
        ("mpacName", ctypes.c_char_p),
        ("muID", ctypes.c_uint32),
        ("mbValid", ctypes.c_ubyte),
    ]
    mpacName: bytes
    muID: int
    mbValid: bool

class cTkRegionMapBase(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkRegionMapBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetScaleX", ctypes.c_ulonglong),
        ("GetScaleY", ctypes.c_ulonglong),
        ("GetScaleZ", ctypes.c_ulonglong),
        ("GetCentre", ctypes.c_ulonglong),
    ]
    GetScaleX: int
    GetScaleY: int
    GetScaleZ: int
    GetCentre: int

class cTkVoxel(ctypes.Structure):
    def GetTileBlend(): pass

    def SetMaterial(): pass

    def SetSecondaryMaterial(): pass

    def SetTexturePatch(): pass

    def ShortFromFloat(): pass

    def SetTileBlend(): pass

    _fields_ = [
        ("mPack_Edit1_Mat4_Secmat4_Tex6", ctypes.c_uint16),
        ("mPack_Density16", ctypes.c_uint16),
        ("mPack_TileBlend16", ctypes.c_uint16),
    ]
    mPack_Edit1_Mat4_Secmat4_Tex6: int
    mPack_Density16: int
    mPack_TileBlend16: int

class cTkVoxelArray(ctypes.Structure):
    _fields_ = [
        ("mpaVoxels", ctypes.c_ulonglong),
        ("miSizeX", ctypes.c_int32),
        ("miSizeY", ctypes.c_int32),
        ("miSizeZ", ctypes.c_int32),
        ("miSizeZY", ctypes.c_int32),
        ("muVoxelMaterialMask", ctypes.c_uint32),
    ]
    mpaVoxels: int
    miSizeX: int
    miSizeY: int
    miSizeZ: int
    miSizeZY: int
    muVoxelMaterialMask: int

class cTkRegion_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int

class cGcRegionBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
        ("PostPolygoniseThread", ctypes.c_ulonglong),
        ("CancelPolygonise", ctypes.c_ulonglong),
        ("PreAsyncKnowledgeBuild", ctypes.c_ulonglong),
        ("CanPolygonise", ctypes.c_ulonglong),
        ("ThreadedGenerateVoxels", ctypes.c_ulonglong),
        ("ThreadedPolygonise", ctypes.c_ulonglong),
        ("ThreadedKnowledgeBuild", ctypes.c_ulonglong),
        ("GetFacingTriangleStart", ctypes.c_ulonglong),
        ("GetTileBlendTriangleStart", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int
    PostPolygoniseThread: int
    CancelPolygonise: int
    PreAsyncKnowledgeBuild: int
    CanPolygonise: int
    ThreadedGenerateVoxels: int
    ThreadedPolygonise: int
    ThreadedKnowledgeBuild: int
    GetFacingTriangleStart: int
    GetTileBlendTriangleStart: int

class cGcRegionDecoratorPhysics(ctypes.Structure):
    def Add(): pass

    def Generate(): pass

    def Remove(): pass

    def AllocCollision(): pass

    def ConstructCollision(): pass

    def DestructCollision(): pass

    def AllocRigidBodies(): pass

    def ConstructRigidBody(): pass

    def DestructRigidBody(): pass

    def BatchDestructBodiesAndCollision(): pass

    _fields_ = [
        ("mapCollision", ctypes.c_ulonglong),
        ("mapRigidBody", ctypes.c_ulonglong),
        ("miCollisionCount", ctypes.c_int16),
        ("miCollisionCapacity", ctypes.c_int16),
        ("miRigidBodyCount", ctypes.c_int16),
        ("miRigidBodyCapacity", ctypes.c_int16),
        ("mbInWorld", ctypes.c_ubyte),
    ]
    mapCollision: int
    mapRigidBody: int
    miCollisionCount: int
    miCollisionCapacity: int
    miRigidBodyCount: int
    miRigidBodyCapacity: int
    mbInWorld: bool

class cGcFadeNodeInstance_vtbl(ctypes.Structure):
    _fields_ = [
        ("SetNodeActivation", ctypes.c_ulonglong),
        ("CheckNodeActivation", ctypes.c_ulonglong),
        ("SetNodeParamF", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
    ]
    SetNodeActivation: int
    CheckNodeActivation: int
    SetNodeParamF: int
    Update: int

class cGcPlanetSpawnData(ctypes.Structure):
    _fields_ = [
        ("maDistantObjects", ctypes.c_char * 0x18),  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> >
        ("maLandmarks", ctypes.c_char * 0x18),  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> >
        ("maObjects", ctypes.c_char * 0x18),  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> >
        ("maDetailObjects", ctypes.c_char * 0x18),  # std::vector<cGcPlanetObjectSpawnData,TkSTLAllocatorShim<cGcPlanetObjectSpawnData,8,-1> >
        ("mbDistantObjectsEnabled", ctypes.c_ubyte),
        ("mbLandmarksEnabled", ctypes.c_ubyte),
        ("mbObjectsEnabled", ctypes.c_ubyte),
        ("mbDetailObjectsEnabled", ctypes.c_ubyte),
    ]
    maDistantObjects: bytes
    maLandmarks: bytes
    maObjects: bytes
    maDetailObjects: bytes
    mbDistantObjectsEnabled: bool
    mbLandmarksEnabled: bool
    mbObjectsEnabled: bool
    mbDetailObjectsEnabled: bool

class cGcNavChunkInfo(ctypes.Structure):
    _fields_ = [
        ("mvPlanetSpaceCenter", common.Vector3f),
        ("mfWorldWidthX", ctypes.c_float),
        ("mfWorldWidthZ", ctypes.c_float),
        ("mfRadius", ctypes.c_float),
        ("muLinkIndexBegin", ctypes.c_uint16),
        ("muLinkIndexEnd", ctypes.c_uint16),
        ("muInstanceCount", ctypes.c_uint8),
        ("muObjCount", ctypes.c_uint8),
        ("muLayerIndex", ctypes.c_uint8),
        ("muMaxLayersAbove", ctypes.c_uint8),
        ("miChunkMinX", ctypes.c_uint8),
        ("miChunkMaxX", ctypes.c_uint8),
        ("miChunkMinZ", ctypes.c_uint8),
        ("miChunkMaxZ", ctypes.c_uint8),
        ("meInterest", ctypes.c_uint8),
        ("meOldInterest", ctypes.c_uint8),
        ("miTerrainType", ctypes.c_uint8),
        ("miMaxHeight", ctypes.c_uint8),
        ("mbIsBase", ctypes.c_ubyte),
        ("miKnowledgeTypes", ctypes.c_uint16),
        ("miMinHeight", ctypes.c_uint16),
        ("miWaterDepth", ctypes.c_uint16),
        ("mbBordersOtherRegions", ctypes.c_uint8),
        ("mbBorderUpDown", ctypes.c_uint8),
        ("miFoundIndex", ctypes.c_int16),
    ]
    mvPlanetSpaceCenter: common.Vector3f
    mfWorldWidthX: float
    mfWorldWidthZ: float
    mfRadius: float
    muLinkIndexBegin: int
    muLinkIndexEnd: int
    muInstanceCount: int
    muObjCount: int
    muLayerIndex: int
    muMaxLayersAbove: int
    miChunkMinX: int
    miChunkMaxX: int
    miChunkMinZ: int
    miChunkMaxZ: int
    meInterest: int
    meOldInterest: int
    miTerrainType: int
    miMaxHeight: int
    mbIsBase: bool
    miKnowledgeTypes: int
    miMinHeight: int
    miWaterDepth: int
    mbBordersOtherRegions: int
    mbBorderUpDown: int
    miFoundIndex: int

class cGcRegionKnowledgeRef(ctypes.Structure):
    _fields_ = [
        ("mpKnowledge", ctypes.c_ulonglong),
        ("mpPrev", ctypes.c_ulonglong),
        ("mpNext", ctypes.c_ulonglong),
        ("mNode", ctypes.c_uint32),
    ]
    mpKnowledge: int
    mpPrev: int
    mpNext: int
    mNode: int

class cGcQuantizedTime(ctypes.Structure):
    _fields_ = [
        ("miTime", ctypes.c_int32),
    ]
    miTime: int

class cGcCreatureRoutinePtr(ctypes.Structure):
    _fields_ = [
        ("miUniqueId", ctypes.c_int64),
    ]
    miUniqueId: int

class cGcCreatureStats(ctypes.Structure):
    _fields_ = [
        ("mResHandle", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mFemaleResHandle", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mfNavRadius", ctypes.c_float),
        ("mfHeight", ctypes.c_float),
        ("mfSpeed", ctypes.c_float),
        ("mfScale", ctypes.c_float),
    ]
    mResHandle: bytes
    mFemaleResHandle: bytes
    mfNavRadius: float
    mfHeight: float
    mfSpeed: float
    mfScale: float

class BaseIndex(ctypes.Structure):
    def BaseIndex(): pass

    _fields_ = [
        ("mValue", ctypes.c_uint16),
    ]
    mValue: int

class cGcWFCNavigationSubGraph(ctypes.Structure):
    def ParseScene(): pass

    def Instantiate(): pass

    _fields_ = [
        ("mNodes", ctypes.c_char * 0x18),  # std::vector<sWFCNavigationSubgraphNode,TkSTLAllocatorShim<sWFCNavigationSubgraphNode,16,-1> >
        ("maiConnections", ctypes.c_int32 * 0x6),  # int[6]
        ("mEdges", ctypes.c_char * 0x18),  # std::vector<std::pair<int,int>,TkSTLAllocatorShim<std::pair<int,int>,4,-1> >
        ("mbConnectNodesByDistance", ctypes.c_ubyte),
        ("mNPCs", ctypes.c_char * 0x18),  # std::vector<std::pair<TkID<128>,cTkMatrix34>,TkSTLAllocatorShim<std::pair<TkID<128>,cTkMatrix34>,16,-1> >
    ]
    mNodes: bytes
    maiConnections: list[ctypes.c_int32]
    mEdges: bytes
    mbConnectNodesByDistance: bool
    mNPCs: bytes

class sWeightedWFCScene(ctypes.Structure):
    _fields_ = [
        ("mScene", ctypes.c_ulonglong),
        ("mfRelativeProbability", ctypes.c_float),
    ]
    mScene: int
    mfRelativeProbability: float

class cGcWFCScenes(ctypes.Structure):
    def cGcWFCScenes(): pass

    def PickGeometryIndex(): pass

    _fields_ = [
        ("mItems", ctypes.c_char * 0x18),  # std::vector<sWeightedWFCScene,TkSTLAllocatorShim<sWeightedWFCScene,8,-1> >
    ]
    mItems: bytes

class sModulePreset(ctypes.Structure):
    _fields_ = [
        ("mModule", ctypes.c_ulonglong),
        ("mPosition", ctypes.c_char * 0xC),  # cTkIntTuple3<int,0>
    ]
    mModule: int
    mPosition: bytes

class cGcBuildingPreset(ctypes.Structure):
    def tilde_cGcBuildingPreset(): pass

    def Generate(): pass

    def GenerateUsingFallback(): pass

    def MergeBounds(): pass

    _fields_ = [
        ("mModuleSet", ctypes.c_ulonglong),
        ("mpMap", ctypes.c_ulonglong),
        ("mBounds", ctypes.c_char * 0x18),  # std::vector<cTkAABB,TkSTLAllocatorShim<cTkAABB,16,-1> >
    ]
    mModuleSet: int
    mpMap: int
    mBounds: bytes

class sFreighterRoom(ctypes.Structure):
    _fields_ = [
        ("mpRoom", ctypes.c_ulonglong),
        ("maSlots", ctypes.c_char * 0x18),  # std::vector<cGcSlot const *,TkSTLAllocatorShim<cGcSlot const *,8,-1> >
    ]
    mpRoom: int
    maSlots: bytes

class cGcSettlementBuildingState(ctypes.Structure):
    def Get(): pass

    def StoreInPlayerState(): pass

    def GetNextAdvancement(): pass

    _fields_ = [
        ("miValue", ctypes.c_int32),
    ]
    miValue: int

class sBaseBuildingMaterial(ctypes.Structure):
    _fields_ = [
        ("muColourIndex", ctypes.c_uint8),
        ("muMaterialIndex", ctypes.c_uint8),
    ]
    muColourIndex: int
    muMaterialIndex: int

class cGcNavGraphNodeHandle(ctypes.Structure):
    _fields_ = [
        ("miIndex", ctypes.c_int16),
        ("muId", ctypes.c_uint16),
    ]
    miIndex: int
    muId: int

class cGcNPCInteractiveObjectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class RefCountedJob(ctypes.Structure):
    def ShouldAbort(): pass

    def Increment(): pass

    def Destroy(): pass

    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("miRefCount", ctypes.c_char * 0x4),  # std::atomic<int>
        ("mbAbort", ctypes.c_ubyte),
    ]
    __vftable: int
    miRefCount: bytes
    mbAbort: bool

class RefCountedJob_vtbl(ctypes.Structure):
    _fields_ = [
        ("RefCountedJob_dtor_0", ctypes.c_ulonglong),
    ]
    RefCountedJob_dtor_0: int

class cGcNPCNavGraphPhysProbeJob_vtbl(ctypes.Structure):
    _fields_ = [
        ("RefCountedJob_dtor_0", ctypes.c_ulonglong),
    ]
    RefCountedJob_dtor_0: int

class cGcNPCNavigationAreaComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class sTerrainEditData(ctypes.Structure):
    def sTerrainEditData(): pass

    _fields_ = [
        ("___u0", ctypes.c_char * 0x1),  # $B0F81696E689D74CFBBB4B0C2724F009
    ]
    ___u0: bytes

class sRegionEdit(ctypes.Structure):
    def sRegionEdit(): pass

    _fields_ = [
        ("mMapPosition", common.Vector3f),
        ("mEditData", ctypes.c_int8),
        ("mbEditApplied", ctypes.c_ubyte),
    ]
    mMapPosition: common.Vector3f
    mEditData: int
    mbEditApplied: bool

class cGcTerrainRegionMap_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetScaleX", ctypes.c_ulonglong),
        ("GetScaleY", ctypes.c_ulonglong),
        ("GetScaleZ", ctypes.c_ulonglong),
        ("GetCentre", ctypes.c_ulonglong),
    ]
    GetScaleX: int
    GetScaleY: int
    GetScaleZ: int
    GetCentre: int

class cTkTerrainVertexData(ctypes.Structure):
    _fields_ = [
        ("maVertexArray", ctypes.c_ulonglong),
        ("miNumVertices", ctypes.c_int32),
        ("miMaxNumVertices", ctypes.c_int32),
    ]
    maVertexArray: int
    miNumVertices: int
    miMaxNumVertices: int

class cGcRegionTerrain_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
        ("PostPolygoniseThread", ctypes.c_ulonglong),
        ("CancelPolygonise", ctypes.c_ulonglong),
        ("PreAsyncKnowledgeBuild", ctypes.c_ulonglong),
        ("CanPolygonise", ctypes.c_ulonglong),
        ("ThreadedGenerateVoxels", ctypes.c_ulonglong),
        ("ThreadedPolygonise", ctypes.c_ulonglong),
        ("ThreadedKnowledgeBuild", ctypes.c_ulonglong),
        ("GetFacingTriangleStart", ctypes.c_ulonglong),
        ("GetTileBlendTriangleStart", ctypes.c_ulonglong),
        ("DebugUpdate", ctypes.c_ulonglong),
        ("CreatureUpdate", ctypes.c_ulonglong),
        ("PollToMapStreams", ctypes.c_ulonglong),
        ("OnDistanceChanged", ctypes.c_ulonglong),
        ("SetHidden", ctypes.c_ulonglong),
        ("GetGroundPhysics", ctypes.c_ulonglong),
        ("FindInstanceWithRigidBodyNode", ctypes.c_ulonglong),
        ("RemoveInstanceWithRigidBodyNode", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcRegionTerrain *this, TkID<128> *result, cTkRigidBodyPtr)
        ("RemoveInstancesWithinArea", ctypes.c_ulonglong),
        ("SetInstanceActivating", ctypes.c_ulonglong),
        ("GetFoliage", ctypes.c_ulonglong),
        ("GetBuildings", ctypes.c_ulonglong),
        ("GetCreatures", ctypes.c_ulonglong),
        ("GetCreatures_158", ctypes.c_ulonglong),
        ("GetNodeName", ctypes.c_ulonglong),
        ("PartitionTriangles", ctypes.c_ulonglong),
        ("PartitionTrianglesByTile", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int
    PostPolygoniseThread: int
    CancelPolygonise: int
    PreAsyncKnowledgeBuild: int
    CanPolygonise: int
    ThreadedGenerateVoxels: int
    ThreadedPolygonise: int
    ThreadedKnowledgeBuild: int
    GetFacingTriangleStart: int
    GetTileBlendTriangleStart: int
    DebugUpdate: int
    CreatureUpdate: int
    PollToMapStreams: int
    OnDistanceChanged: int
    SetHidden: int
    GetGroundPhysics: int
    FindInstanceWithRigidBodyNode: int
    RemoveInstanceWithRigidBodyNode: bytes
    RemoveInstancesWithinArea: int
    SetInstanceActivating: int
    GetFoliage: int
    GetBuildings: int
    GetCreatures: int
    GetCreatures_158: int
    GetNodeName: int
    PartitionTriangles: int
    PartitionTrianglesByTile: int

class cGcLootComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcScannableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcMarkerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkBehaviourTree(ctypes.Structure):
    def AddNode(): pass

    def AddBranchSelector(): pass

    def AddAlwaysSucceedDecorator(): pass

    def AddConcurrentSelector(): pass

    def AddInverseDecorator(): pass

    def AddNeverFailDecorator(): pass

    def AddPrioritySelector(): pass

    def AddSelectionDecorator(): pass

    def AddSequentialSelector(): pass

    def AddSwitchDecorator(): pass

    def Construct(): pass

    def Destruct(): pass

    def Update(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mSceneGraphNode", ctypes.c_uint32),
        ("mpRootNode", ctypes.c_ulonglong),
        ("mMetaMessageSubscriptions", ctypes.c_char * 0x50),  # cTkMetaMessageMap<2>
        ("mNodes", ctypes.c_char * 0x18),  # std::vector<cTkBehaviourTreeNode *,TkSTLAllocatorShim<cTkBehaviourTreeNode *,8,-1> >
    ]
    __vftable: int
    mSceneGraphNode: int
    mpRootNode: int
    mMetaMessageSubscriptions: bytes
    mNodes: bytes

class cTkBehaviourTreeNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int

class cTkBehaviourTreeSelector_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cGcCreatureBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cTkSphere(ctypes.Structure):
    def GetCubeUp(): pass

    def ProjectSphereToCube(): pass

    def GetCubeMatrix(): pass

    def ProjectHemisphereToPlane(): pass

    def GetHorizonDisc(): pass

    _fields_ = [
        ("mPos", common.Vector3f),
        ("mfRadius", ctypes.c_float),
    ]
    mPos: common.Vector3f
    mfRadius: float

class cGcRobotBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
        ("SetLookAtPoint", ctypes.c_ulonglong),
        ("AsWalkerTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int
    SetLookAtPoint: int
    AsWalkerTree: int

class cTkIKRotYModifyConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cGcCreaturePelvisConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cGcPistonConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cGcCreatureFullBodyIKComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkRandomComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCreatureVocals(ctypes.Structure):
    def Prepare(): pass

    def ForcePlayCurrentVocal(): pass

    def Play(): pass

    def PlaySnore(): pass

    def Update(): pass

    def SetVocalType(): pass

    def PickCooldown(): pass

    def RenderNGuiCallback(): pass

    _fields_ = [
        ("mpCurSoundData", ctypes.c_ulonglong),
        ("mpRandomer", ctypes.c_ulonglong),
        ("mpCreature", ctypes.c_ulonglong),
        ("mfLastVocalizedTime", ctypes.c_float),
        ("mfNextVocalizeTime", ctypes.c_float),
        ("mfCreatureSizeValue", ctypes.c_float),
        ("mfCreatureHeadSizeValue", ctypes.c_float),
        ("mCreatureHead", ctypes.c_uint32),
        ("muCreatureClass", ctypes.c_uint32),
        ("mAudioObject", ctypes.c_uint64),
        ("mpAudioObstructionRequest", ctypes.c_ulonglong),
    ]
    mpCurSoundData: int
    mpRandomer: int
    mpCreature: int
    mfLastVocalizedTime: float
    mfNextVocalizeTime: float
    mfCreatureSizeValue: float
    mfCreatureHeadSizeValue: float
    mCreatureHead: int
    muCreatureClass: int
    mAudioObject: int
    mpAudioObstructionRequest: int

class cGcNavMeshHandle(ctypes.Structure):
    _fields_ = [
        ("miMeshFamily", ctypes.c_int16),
        ("miIndex", ctypes.c_int16),
        ("muId", ctypes.c_uint16),
    ]
    miMeshFamily: int
    miIndex: int
    muId: int

class cGcRegionKnowledgeChunkMask(ctypes.Structure):
    def cGcRegionKnowledgeChunkMask(): pass

    _fields_ = [
        ("mfNeededHeight", ctypes.c_float),
        ("mfNeededWaterDepth", ctypes.c_float),
        ("muTypeMask", ctypes.c_uint16),
    ]
    mfNeededHeight: float
    mfNeededWaterDepth: float
    muTypeMask: int

class cGcAntagonistComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCreatureNavigation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCreatureNavigation_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetMode", ctypes.c_ulonglong),
        ("GetFrontNavMatrix", ctypes.c_ulonglong),
        ("GetTurnPivot", ctypes.c_ulonglong),
        ("InitialAdjustToKnowledge", ctypes.c_ulonglong),
        ("AdjustToKnowledge", ctypes.c_ulonglong),
        ("NavigateInRoughDirection", ctypes.c_ulonglong),
        ("NavigateToPoint", ctypes.c_char * 0x8),  # cTkStackVector<cGcNavPolyHandle,64,-1> *(__fastcall *)(cGcCreatureNavigation *this, const cTkPhysRelVec3 *, const cGcNavPolyHandle *)
        ("IsPointAccessible", ctypes.c_ulonglong),
        ("HasPath", ctypes.c_ulonglong),
        ("CopyPathFrom", ctypes.c_ulonglong),
        ("SetMoveType", ctypes.c_ulonglong),
        ("SetStealthy", ctypes.c_ulonglong),
        ("RequestIdle", ctypes.c_char * 0x8),  # cTkAnimLayerHandle *(__fastcall *)(cGcCreatureNavigation *this, cTkAnimLayerHandle *result, const TkID<128> *, float, cGcCreatureMoveSpeedRequest, bool, float)
        ("GetIdleLength", ctypes.c_char * 0x8),  # float (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *)
        ("ForceSingleAnim", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *, float, float, bool, float)
        ("HasForcedSingleAnim", ctypes.c_ulonglong),
        ("GetForcedSingleAnimHandle", ctypes.c_ulonglong),
        ("CancelForcedSingleAnim", ctypes.c_ulonglong),
        ("PlaySubAnim", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *, float, float, float, bool)
        ("AsGroundNavigation", ctypes.c_ulonglong),
        ("AsSwarmNavigation", ctypes.c_ulonglong),
        ("GetTurnRadius", ctypes.c_ulonglong),
        ("GetMoveType", ctypes.c_ulonglong),
        ("SetWantsToLand", ctypes.c_ulonglong),
        ("GetLandingProgress", ctypes.c_ulonglong),
        ("GetStealthy", ctypes.c_ulonglong),
        ("GetAttackAnimCandidates", ctypes.c_char * 0x8),  # unsigned int (__fastcall *)(cGcCreatureNavigation *this, TkID<128> *, unsigned int)
        ("ZeroSteering", ctypes.c_ulonglong),
        ("GetRunAnimSpeed", ctypes.c_ulonglong),
        ("CanPounce", ctypes.c_ulonglong),
        ("StartPounce", ctypes.c_char * 0x8),  # cTkAnimInstanceHandle *(__fastcall *)(cGcCreatureNavigation *this, cTkAnimInstanceHandle *result, const TkID<128> *, const cTkPhysRelVec3 *, bool, bool, bool)
        ("UpdatePounce", ctypes.c_ulonglong),
        ("StartLocomotionDamage", ctypes.c_ulonglong),
        ("UpdateLocomotionDamage", ctypes.c_ulonglong),
        ("GetSteeringDirections", ctypes.c_ulonglong),
        ("PointIsInsideNavMap", ctypes.c_ulonglong),
        ("ShouldDespawn", ctypes.c_ulonglong),
        ("RequestRefreshNavMap", ctypes.c_ulonglong),
        ("SetMoveSpeeds", ctypes.c_ulonglong),
        ("SetSpeedFactor", ctypes.c_ulonglong),
        ("GetDesiredRidingAnim", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcCreatureNavigation *this, TkID<128> *result, float *, bool *)
        ("GetRidingLean", ctypes.c_ulonglong),
        ("IsBeingRidden", ctypes.c_ulonglong),
        ("SetMinObstacleAabbSize", ctypes.c_ulonglong),
        ("ForceTurn", ctypes.c_ulonglong),
        ("AddBoost", ctypes.c_ulonglong),
        ("IsBoosting", ctypes.c_ulonglong),
        ("SetClampToGround", ctypes.c_ulonglong),
        ("DebugRecreateNav", ctypes.c_ulonglong),
        ("NotifyOfPredator", ctypes.c_ulonglong),
        ("DebugRender", ctypes.c_ulonglong),
        ("RenderNGuiVis", ctypes.c_ulonglong),
        ("RenderNGuiControl", ctypes.c_ulonglong),
    ]
    cGcCreatureNavigation_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    UpdatePostPhysics: int
    Release: int
    Destruct: int
    SetMode: int
    GetFrontNavMatrix: int
    GetTurnPivot: int
    InitialAdjustToKnowledge: int
    AdjustToKnowledge: int
    NavigateInRoughDirection: int
    NavigateToPoint: bytes
    IsPointAccessible: int
    HasPath: int
    CopyPathFrom: int
    SetMoveType: int
    SetStealthy: int
    RequestIdle: bytes
    GetIdleLength: bytes
    ForceSingleAnim: bytes
    HasForcedSingleAnim: int
    GetForcedSingleAnimHandle: int
    CancelForcedSingleAnim: int
    PlaySubAnim: bytes
    AsGroundNavigation: int
    AsSwarmNavigation: int
    GetTurnRadius: int
    GetMoveType: int
    SetWantsToLand: int
    GetLandingProgress: int
    GetStealthy: int
    GetAttackAnimCandidates: bytes
    ZeroSteering: int
    GetRunAnimSpeed: int
    CanPounce: int
    StartPounce: bytes
    UpdatePounce: int
    StartLocomotionDamage: int
    UpdateLocomotionDamage: int
    GetSteeringDirections: int
    PointIsInsideNavMap: int
    ShouldDespawn: int
    RequestRefreshNavMap: int
    SetMoveSpeeds: int
    SetSpeedFactor: int
    GetDesiredRidingAnim: bytes
    GetRidingLean: int
    IsBeingRidden: int
    SetMinObstacleAabbSize: int
    ForceTurn: int
    AddBoost: int
    IsBoosting: int
    SetClampToGround: int
    DebugRecreateNav: int
    NotifyOfPredator: int
    DebugRender: int
    RenderNGuiVis: int
    RenderNGuiControl: int

class cGcCreatureMovementBase(ctypes.Structure):
    def Prepare(): pass

    def DoFootstep(): pass

    def SetSoundState(): pass

    def vector_deleting_destructor(): pass

    def TriggerDeathAction(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpCreature", ctypes.c_ulonglong),
        ("muCreatureClass", ctypes.c_uint32),
    ]
    __vftable: int
    mpCreature: int
    muCreatureClass: int

class cGcCreatureMovementBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCreatureMovementBase_dtor_0", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("GetExtraPartNode", ctypes.c_ulonglong),
        ("HasDeathAction", ctypes.c_ulonglong),
        ("TriggerDeathAction", ctypes.c_ulonglong),
        ("IsCrystal", ctypes.c_ulonglong),
        ("NotifyOfPredator", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("SetSoundState", ctypes.c_ulonglong),
    ]
    cGcCreatureMovementBase_dtor_0: int
    Release: int
    UpdatePostPhysics: int
    GetExtraPartNode: int
    HasDeathAction: int
    TriggerDeathAction: int
    IsCrystal: int
    NotifyOfPredator: int
    RenderNGui: int
    SetSoundState: int

class cGcCreatureGUID(ctypes.Structure):
    _fields_ = [
        ("muHigh", ctypes.c_uint64),
        ("muLow", ctypes.c_uint64),
    ]
    muHigh: int
    muLow: int

class cGcCreatureComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcMarkerRenderData(ctypes.Structure):
    def cGcMarkerRenderData(): pass

    def Render(): pass

    _fields_ = [
        ("mFillColour", common.Colour),
        ("mBackgroundColour", common.Colour),
        ("mfAmount", ctypes.c_float),
        ("mfInnerRadius", ctypes.c_float),
        ("mfOuterRadius", ctypes.c_float),
    ]
    mFillColour: common.Colour
    mBackgroundColour: common.Colour
    mfAmount: float
    mfInnerRadius: float
    mfOuterRadius: float

class cGcInteraction(ctypes.Structure):
    def Construct(): pass

    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpInteractionComponent", ctypes.c_ulonglong),
    ]
    __vftable: int
    mpInteractionComponent: int

class cGcInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcInteraction_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("SendMessageA", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcInteraction *this, TkID<128>, TkID<128>)
        ("DoInteractionEvent", ctypes.c_ulonglong),
        ("GetViewTargetNode", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("RenderOffscreen2D", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
    ]
    cGcInteraction_dtor_0: int
    Construct: int
    Release: int
    Prepare: int
    Update: int
    SendMessageA: bytes
    DoInteractionEvent: int
    GetViewTargetNode: int
    Interact: int
    UpdateRender: int
    RenderOffscreen2D: int
    RenderNGui: int

class ITkAudioStream(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkAudioStreamFormat(ctypes.Structure):
    _fields_ = [
        ("miSampleRate", ctypes.c_uint32),
        ("mbStereo", ctypes.c_ubyte),
        ("miBitsPerSample", ctypes.c_uint32),
        ("miBytesPerFrame", ctypes.c_uint32),
        ("mbFloatData", ctypes.c_ubyte),
    ]
    miSampleRate: int
    mbStereo: bool
    miBitsPerSample: int
    miBytesPerFrame: int
    mbFloatData: bool

class ITkAudioStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkAudioStream_dtor_0", ctypes.c_ulonglong),
        ("GetAudioStreamFormat", ctypes.c_ulonglong),
        ("GetAudioStreamData", ctypes.c_ulonglong),
    ]
    ITkAudioStream_dtor_0: int
    GetAudioStreamFormat: int
    GetAudioStreamData: int

class cGcByteBeat_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkAudioStream_dtor_0", ctypes.c_ulonglong),
        ("GetAudioStreamFormat", ctypes.c_ulonglong),
        ("GetAudioStreamData", ctypes.c_ulonglong),
    ]
    ITkAudioStream_dtor_0: int
    GetAudioStreamFormat: int
    GetAudioStreamData: int

class cGcProceduralMaterialSampler(ctypes.Structure):
    _fields_ = [
        ("msTextureName", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mpTextureDescriptor", ctypes.c_ulonglong),
        ("mbRequiresAsyncLoad", ctypes.c_ubyte),
        ("mbAsyncLoaded", ctypes.c_ubyte),
        ("mbReleased", ctypes.c_ubyte),
        ("mxFlags", ctypes.c_uint8),
    ]
    msTextureName: bytes
    mpTextureDescriptor: int
    mbRequiresAsyncLoad: bool
    mbAsyncLoaded: bool
    mbReleased: bool
    mxFlags: int

class ActivationCost(ctypes.Structure):
    def HasCost(): pass

    def Clear(): pass

    def Set(): pass

    def CostRemaining(): pass

    _fields_ = [
        ("mSubstanceId", ctypes.c_ulonglong),
        ("maMultipleCostIds", ctypes.c_char * 0x18),  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> >
        ("miCost", ctypes.c_int32),
        ("miAmountPaid", ctypes.c_int32),
        ("mbRepeat", ctypes.c_ubyte),
        ("mRequiredTech", ctypes.c_ulonglong),
    ]
    mSubstanceId: int
    maMultipleCostIds: bytes
    miCost: int
    miAmountPaid: int
    mbRepeat: bool
    mRequiredTech: int

class cGcNPCBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Construct_48", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNPCBehaviourTree *this, cTkAttachment *, const TkID<128> *)
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Construct_48: bytes

class cGcNPCComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class sComponentBaseObjectHandle(ctypes.Structure):
    def Prepare(): pass

    _fields_ = [
        ("mBaseObjectHandle", ctypes.c_uint64),
        ("muLookupIndex", ctypes.c_uint32),
    ]
    mBaseObjectHandle: int
    muLookupIndex: int

class cGcVehicleCheckpointComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcTriggerActionComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcPortalComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcInteractionComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcGrabbableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsInSpaceship", ctypes.c_ulonglong),
        ("IsInLandVehicle", ctypes.c_ulonglong),
        ("GetEnvironmentLocation", ctypes.c_ulonglong),
        ("IsLocalPlayer", ctypes.c_ulonglong),
        ("GetNetworkIndex", ctypes.c_ulonglong),
        ("IsVRPlayer", ctypes.c_ulonglong),
        ("GetDominantHand", ctypes.c_ulonglong),
        ("IsFullSimLobbyHost", ctypes.c_ulonglong),
        ("VirtualGetMatrix", ctypes.c_ulonglong),
        ("VirtualGetAttachment", ctypes.c_ulonglong),
        ("GetNode", ctypes.c_ulonglong),
        ("NotifyBeingScanned", ctypes.c_ulonglong),
        ("IsStealthActive", ctypes.c_ulonglong),
    ]
    IsInSpaceship: int
    IsInLandVehicle: int
    GetEnvironmentLocation: int
    IsLocalPlayer: int
    GetNetworkIndex: int
    IsVRPlayer: int
    GetDominantHand: int
    IsFullSimLobbyHost: int
    VirtualGetMatrix: int
    VirtualGetAttachment: int
    GetNode: int
    NotifyBeingScanned: int
    IsStealthActive: int

class cGcLobbyId(ctypes.Structure):
    def cGcLobbyId(): pass

    _fields_ = [
        ("mLobbyId", ctypes.c_char * 0x200),  # cTkFixedString<512,char>
    ]
    mLobbyId: bytes

class cTkReplicatedVariableArray(ctypes.Structure):
    def ClearDirtyFlags(): pass

    def NumChangedReplicatedVariables(): pass

    def SerializeOrDeserialize(): pass

    def ShouldReplicateToPlayer(): pass

    _fields_ = [
        ("mapReplicatedVariables", ctypes.c_char * 0x18),  # std::vector<cTkReplicatedVariableBase *,TkSTLAllocatorShim<cTkReplicatedVariableBase *,8,-1> >
    ]
    mapReplicatedVariables: bytes

class cGcNetworkPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsInSpaceship", ctypes.c_ulonglong),
        ("IsInLandVehicle", ctypes.c_ulonglong),
        ("GetEnvironmentLocation", ctypes.c_ulonglong),
        ("IsLocalPlayer", ctypes.c_ulonglong),
        ("GetNetworkIndex", ctypes.c_ulonglong),
        ("IsVRPlayer", ctypes.c_ulonglong),
        ("GetDominantHand", ctypes.c_ulonglong),
        ("IsFullSimLobbyHost", ctypes.c_ulonglong),
        ("VirtualGetMatrix", ctypes.c_ulonglong),
        ("VirtualGetAttachment", ctypes.c_ulonglong),
        ("GetNode", ctypes.c_ulonglong),
        ("NotifyBeingScanned", ctypes.c_ulonglong),
        ("IsStealthActive", ctypes.c_ulonglong),
        ("cGcNetworkPlayer_dtor_68", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNetworkPlayer *this, const std::shared_ptr<cGcNetworkAddress> *, cTkUserIdBase<cTkFixedString<64,char> >, const char *, cTkUserIdBase<cTkFixedString<64,char> >, const cTkFixedString<3,char> *, const char *)
        ("Release", ctypes.c_ulonglong),
        ("HandleNetSerialize", ctypes.c_ulonglong),
        ("IsFullSimPlayer", ctypes.c_ulonglong),
        ("OnConnected", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("GetPlayerNameWithTitle", ctypes.c_ulonglong),
        ("HandleMessage", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkPlayer *this, const std::shared_ptr<cGcNetworkMessage const > *)
        ("GetPosition", ctypes.c_ulonglong),
        ("GetMapPosition", ctypes.c_ulonglong),
        ("NotifyJoin", ctypes.c_ulonglong),
        ("NotifyLeave", ctypes.c_ulonglong),
        ("RenderImGui", ctypes.c_ulonglong),
        ("OnReadyToSendUpdate", ctypes.c_ulonglong),
        ("OnPlayerNameChanged", ctypes.c_ulonglong),
    ]
    IsInSpaceship: int
    IsInLandVehicle: int
    GetEnvironmentLocation: int
    IsLocalPlayer: int
    GetNetworkIndex: int
    IsVRPlayer: int
    GetDominantHand: int
    IsFullSimLobbyHost: int
    VirtualGetMatrix: int
    VirtualGetAttachment: int
    GetNode: int
    NotifyBeingScanned: int
    IsStealthActive: int
    cGcNetworkPlayer_dtor_68: int
    Construct: int
    Prepare: bytes
    Release: int
    HandleNetSerialize: int
    IsFullSimPlayer: int
    OnConnected: int
    Update: int
    IsActive: int
    OnNetworkPlayerStateEvent: int
    GetPlayerNameWithTitle: int
    HandleMessage: bytes
    GetPosition: int
    GetMapPosition: int
    NotifyJoin: int
    NotifyLeave: int
    RenderImGui: int
    OnReadyToSendUpdate: int
    OnPlayerNameChanged: int

class cTkAttachment_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgNodeAttachment_dtor_0", ctypes.c_ulonglong),
        ("OnUpdate", ctypes.c_ulonglong),
    ]
    cEgNodeAttachment_dtor_0: int
    OnUpdate: int

class _stat64i32(ctypes.Structure):
    _fields_ = [
        ("st_dev", ctypes.c_uint32),
        ("st_ino", ctypes.c_uint16),
        ("st_mode", ctypes.c_uint16),
        ("st_nlink", ctypes.c_int16),
        ("st_uid", ctypes.c_int16),
        ("st_gid", ctypes.c_int16),
        ("st_rdev", ctypes.c_uint32),
        ("st_size", ctypes.c_int32),
        ("st_atime", ctypes.c_int64),
        ("st_mtime", ctypes.c_int64),
        ("st_ctime", ctypes.c_int64),
    ]
    st_dev: int
    st_ino: int
    st_mode: int
    st_nlink: int
    st_uid: int
    st_gid: int
    st_rdev: int
    st_size: int
    st_atime: int
    st_mtime: int
    st_ctime: int

class __s_GUID(ctypes.Structure):
    _fields_ = [
        ("Data1", ctypes.c_uint32),
        ("Data2", ctypes.c_uint16),
        ("Data3", ctypes.c_uint16),
        ("Data4", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    Data1: int
    Data2: int
    Data3: int
    Data4: list[ctypes.c_uint8]

class hkRefCountedProperties_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkTaskQueue_BatchingTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class hkViewport_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkViewport_dtor_8", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkViewport_dtor_8: int

class hkCommand_PrimaryType_Auto(ctypes.Structure):
    pass

class __m256i: pass

class hkStructPackingCheck(ctypes.Structure):
    _fields_ = [
        ("m_a", ctypes.c_uint32),
        ("m_b", ctypes.c_uint64),
    ]
    m_a: int
    m_b: int

class _CrtMemState(ctypes.Structure):
    _fields_ = [
        ("pBlockHeader", ctypes.c_ulonglong),
        ("lCounts", ctypes.c_uint64 * 0x5),  # unsigned __int64[5]
        ("lSizes", ctypes.c_uint64 * 0x5),  # unsigned __int64[5]
        ("lHighWaterCount", ctypes.c_uint64),
        ("lTotalCount", ctypes.c_uint64),
    ]
    pBlockHeader: int
    lCounts: list[ctypes.c_uint64]
    lSizes: list[ctypes.c_uint64]
    lHighWaterCount: int
    lTotalCount: int

class __crt_locale_data_public(ctypes.Structure):
    _fields_ = [
        ("_locale_pctype", ctypes.c_ulonglong),
        ("_locale_mb_cur_max", ctypes.c_int32),
        ("_locale_lc_codepage", ctypes.c_uint32),
    ]
    _locale_pctype: int
    _locale_mb_cur_max: int
    _locale_lc_codepage: int

class _wfinddata32_t(ctypes.Structure):
    _fields_ = [
        ("attrib", ctypes.c_uint32),
        ("time_create", ctypes.c_int32),
        ("time_access", ctypes.c_int32),
        ("time_write", ctypes.c_int32),
        ("size", ctypes.c_uint32),
        ("name", ctypes.c_wchar * 0x104),  # wchar_t[260]
    ]
    attrib: int
    time_create: int
    time_access: int
    time_write: int
    size: int
    name: list[ctypes.c_wchar]

class _stat32i64(ctypes.Structure):
    _fields_ = [
        ("st_dev", ctypes.c_uint32),
        ("st_ino", ctypes.c_uint16),
        ("st_mode", ctypes.c_uint16),
        ("st_nlink", ctypes.c_int16),
        ("st_uid", ctypes.c_int16),
        ("st_gid", ctypes.c_int16),
        ("st_rdev", ctypes.c_uint32),
        ("st_size", ctypes.c_int64),
        ("st_atime", ctypes.c_int32),
        ("st_mtime", ctypes.c_int32),
        ("st_ctime", ctypes.c_int32),
    ]
    st_dev: int
    st_ino: int
    st_mode: int
    st_nlink: int
    st_uid: int
    st_gid: int
    st_rdev: int
    st_size: int
    st_atime: int
    st_mtime: int
    st_ctime: int

class hkSimd256i(ctypes.Structure):
    _fields_ = [
        ("i0123", common.Vector4i),
        ("i4567", common.Vector4i),
    ]
    i0123: common.Vector4i
    i4567: common.Vector4i

class hkDefaultTaskQueueEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getTimerData", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDefaultTaskQueue *this, hkArray<hkTimerData,hkBuiltinContainerAllocator<0,0> > *)
        ("clearTimerData", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getTimerData: bytes
    clearTimerData: int

class hkMonitorStreamFrameInfo_AbsoluteTimeCounter_Auto(ctypes.Structure):
    pass

class hkGpuTraceResult_ScopeType_Auto(ctypes.Structure):
    pass

class RAIIReachabilityHelper(ctypes.Structure):
    _fields_ = [
        ("m_config", ctypes.c_ulonglong),
        ("m_validTypes", ctypes.c_char * 0x8),  # hkHashSet<hkReflect::Type const *,hkBuiltinContainerAllocator<0,0> > *
        ("m_newTypes", ctypes.c_char * 0x10),  # hkArrayView<hkReflect::Type const * const>
    ]
    m_config: int
    m_validTypes: bytes
    m_newTypes: bytes

class __m512d(ctypes.Structure):
    _fields_ = [
        ("m512d_f64", ctypes.c_double * 0x8),  # long double[8]
    ]
    m512d_f64: list[ctypes.c_double]

class __m64: pass

class hkAttributeHideCriteria_Types_Auto(ctypes.Structure):
    pass

class _stat64(ctypes.Structure):
    _fields_ = [
        ("st_dev", ctypes.c_uint32),
        ("st_ino", ctypes.c_uint16),
        ("st_mode", ctypes.c_uint16),
        ("st_nlink", ctypes.c_int16),
        ("st_uid", ctypes.c_int16),
        ("st_gid", ctypes.c_int16),
        ("st_rdev", ctypes.c_uint32),
        ("st_size", ctypes.c_int64),
        ("st_atime", ctypes.c_int64),
        ("st_mtime", ctypes.c_int64),
        ("st_ctime", ctypes.c_int64),
    ]
    st_dev: int
    st_ino: int
    st_mode: int
    st_nlink: int
    st_uid: int
    st_gid: int
    st_rdev: int
    st_size: int
    st_atime: int
    st_mtime: int
    st_ctime: int

class _stat32(ctypes.Structure):
    _fields_ = [
        ("st_dev", ctypes.c_uint32),
        ("st_ino", ctypes.c_uint16),
        ("st_mode", ctypes.c_uint16),
        ("st_nlink", ctypes.c_int16),
        ("st_uid", ctypes.c_int16),
        ("st_gid", ctypes.c_int16),
        ("st_rdev", ctypes.c_uint32),
        ("st_size", ctypes.c_int32),
        ("st_atime", ctypes.c_int32),
        ("st_mtime", ctypes.c_int32),
        ("st_ctime", ctypes.c_int32),
    ]
    st_dev: int
    st_ino: int
    st_mode: int
    st_nlink: int
    st_uid: int
    st_gid: int
    st_rdev: int
    st_size: int
    st_atime: int
    st_mtime: int
    st_ctime: int

class hkRangeRealAttribute_m_absmin_Auto(ctypes.Structure):
    pass

class hkRangeRealAttribute_m_absmax_Auto(ctypes.Structure):
    pass

class hkRangeRealAttribute_m_softmin_Auto(ctypes.Structure):
    pass

class hkRangeRealAttribute_m_softmax_Auto(ctypes.Structure):
    pass

class hkRangeInt32Attribute_m_absmin_Auto(ctypes.Structure):
    pass

class hkRangeInt32Attribute_m_absmax_Auto(ctypes.Structure):
    pass

class hkRangeInt32Attribute_m_softmin_Auto(ctypes.Structure):
    pass

class hkRangeInt32Attribute_m_softmax_Auto(ctypes.Structure):
    pass

class hkUiAttribute_m_visible_Auto(ctypes.Structure):
    pass

class hkUiAttribute_m_editable_Auto(ctypes.Structure):
    pass

class hkUiAttribute_m_hideCriteria_Auto(ctypes.Structure):
    pass

class hkUiAttribute_m_label_Auto(ctypes.Structure):
    pass

class __m512: pass

class hkQuadUlong(ctypes.Structure):
    _fields_ = [
        ("xy", common.Vector4i),
        ("zw", common.Vector4i),
    ]
    xy: common.Vector4i
    zw: common.Vector4i

class hkUiAttribute_m_group_Auto(ctypes.Structure):
    pass

class hkUiAttribute_m_hideBaseClassMembers_Auto(ctypes.Structure):
    pass

class hkUiAttribute_m_endGroup_Auto(ctypes.Structure):
    pass

class hkUiAttribute_m_endGroup2_Auto(ctypes.Structure):
    pass

class hkUiAttribute_m_advanced_Auto(ctypes.Structure):
    pass

class hkGizmoAttribute_m_visible_Auto(ctypes.Structure):
    pass

class hkGizmoAttribute_m_label_Auto(ctypes.Structure):
    pass

class hkGizmoAttribute_GizmoType_Auto(ctypes.Structure):
    pass

class hkModelerNodeTypeAttribute_ModelerType_Auto(ctypes.Structure):
    pass

class hkLinkAttribute_Link_Auto(ctypes.Structure):
    pass

class hkSemanticsAttribute_Semantics_Auto(ctypes.Structure):
    pass

class hkUuidImpl: pass

class hknpUnaryAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class hknpBinaryAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class hknpArrayAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class hkFileSystem_Watcher_Change_ChangeType_Auto(ctypes.Structure):
    pass

class hknpSpringAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class hkOptionParser_OptionType_Auto(ctypes.Structure):
    pass

class hkAsyncThreadPool_ThreadPriority_Auto(ctypes.Structure):
    pass

class hkAsyncThreadPool_HardwareThreadBinding_Auto(ctypes.Structure):
    pass

class hkJobType_Auto(ctypes.Structure):
    pass

class hknpShapeMap(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # hkHashMap<hknpShape const *,hkRefPtr<hknpShape>,hkBuiltinContainerAllocator<0,0> >
    ]
    baseclass_0: bytes

class hkcdVertex(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", common.Vector4f),
    ]
    baseclass_0: common.Vector4f

class hknpCircularSymmetryAxis(ctypes.Structure):
    _fields_ = [
        ("m_start", common.Vector4f),
        ("m_direction", common.Vector4f),
    ]
    m_start: common.Vector4f
    m_direction: common.Vector4f

class hknpShapeSet(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x18),  # hkSet<hknpShape const *,hkBuiltinContainerAllocator<0,0>,hkMapOperations<hknpShape const *> >
    ]
    baseclass_0: bytes

class hknpShapeFunctionsSingleton_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkThread_Priority_Auto(ctypes.Structure):
    pass

class b4Uint64: pass

class b4Uint32: pass

class b4Float: pass

class cTkResourceRegistryEntry(ctypes.Structure):
    _fields_ = [
        ("msType", ctypes.c_char * 0x20),  # std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> >
        ("mpInitializationFunc", ctypes.c_ulonglong),
        ("mpReleaseFunc", ctypes.c_ulonglong),
        ("mpFactoryFunc", ctypes.c_char * 0x8),  # cTkResource *(__fastcall *)(const std::basic_string<char,std::char_traits<char>,TkSTLAllocatorShim<char,1,-1> > *, int, cTkResourceDescriptor *)
        ("mFlagsDefResHandleMap", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,int,cTkSmartResHandle,robin_hood::hash<int,void>,std::equal_to<int>,6>
    ]
    msType: bytes
    mpInitializationFunc: int
    mpReleaseFunc: int
    mpFactoryFunc: bytes
    mFlagsDefResHandleMap: bytes

class hkMonitorStreamAnalyzer_Node(ctypes.Structure):
    _fields_ = [
        ("m_node", ctypes.c_ulonglong),
        ("m_moveToTag", ctypes.c_uint64),
        ("m_timerBeginWasSkipped", ctypes.c_ubyte),
        ("m_timerBeginMemory", ctypes.c_char * 0x10),  # hkTuple<bool,__int64,void,void,void,void,void,void>
        ("m_timerEndMemory", ctypes.c_char * 0x10),  # hkTuple<bool,__int64,void,void,void,void,void,void>
        ("m_timerMemory", ctypes.c_int64),
    ]
    m_node: int
    m_moveToTag: int
    m_timerBeginWasSkipped: bool
    m_timerBeginMemory: bytes
    m_timerEndMemory: bytes
    m_timerMemory: int

class hknpDecoratorShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapeNullFunctions(ctypes.Structure):
    def queryPoint(): pass

    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def calcBoundingRadius(): pass

    def calcBoundingSphere(): pass

    def calcInnerRadius(): pass

    def calcInnerSphere(): pass

    def calcMinAngleBetweenFaces(): pass

    def castRay(): pass

    def convertVertexIdsToVertices(): pass

    def estimateNumShapeKeys(): pass

    def getConvexVertices(): pass

    def getFaceInfo(): pass

    def getFaceVertices(): pass

    def getLeafShapes(): pass

    def getMassProperties(): pass

    def getNumberOfFaces(): pass

    def getNumberOfSupportSpheres(): pass

    def getShapeKeys(): pass

    def getSignedDistanceContacts(): pass

    def getSignedDistances(): pass

    def getSupportSpheres(): pass

    def getSupportingFace(): pass

    def getSupportingVertex(): pass

    def makeClone(): pass

    def makeShapeKeyMask(): pass

    def onCoreShapeMutated(): pass

    def onFinishEditingMask(): pass

    def queryAabb(): pass


class hknpDecoratorShapeFunctions(ctypes.Structure):
    def buildMassProperties(): pass

    def calcAabb(): pass

    def calcInnerRadius(): pass

    def calcInnerSphere(): pass

    def calcMinAngleBetweenFaces(): pass

    def calcSurfaceVelocity(): pass

    def convertVertexIdsToVertices(): pass

    def estimateNumShapeKeys(): pass

    def getCircularSymmetryAxis(): pass

    def getFaceInfo(): pass

    def getFaceVertices(): pass

    def getShapeKeys(): pass

    def getSignedDistanceContacts(): pass

    def getSignedDistances(): pass

    def getSupportSpheres(): pass

    def getSupportingFace(): pass

    def getSupportingVertex(): pass

    def makeShapeKeyMask(): pass

    def queryPoint(): pass


class Indenter(ctypes.Structure):
    _fields_ = [
        ("m_array", ctypes.c_char * 0x10),  # hkArray<char,hkBuiltinContainerAllocator<3,0> >
    ]
    m_array: bytes

class TargaHeader2(ctypes.Structure):
    _fields_ = [
        ("IDLength", ctypes.c_uint8),
        ("ColormapType", ctypes.c_uint8),
        ("ImageType", ctypes.c_uint8),
        ("ColormapSpecification", ctypes.c_uint8 * 0x5),  # unsigned __int8[5]
        ("XOrigin", ctypes.c_uint16),
        ("YOrigin", ctypes.c_uint16),
        ("ImageWidth", ctypes.c_uint16),
        ("ImageHeight", ctypes.c_uint16),
        ("PixelDepth", ctypes.c_uint8),
        ("ImageDescriptor", ctypes.c_uint8),
    ]
    IDLength: int
    ColormapType: int
    ImageType: int
    ColormapSpecification: list[ctypes.c_uint8]
    XOrigin: int
    YOrigin: int
    ImageWidth: int
    ImageHeight: int
    PixelDepth: int
    ImageDescriptor: int

class hknpDecoratorBroadPhase_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpBroadPhase_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getConfig", ctypes.c_ulonglong),
        ("setBodyBuffer", ctypes.c_ulonglong),
        ("translate", ctypes.c_ulonglong),
        ("addBodies", ctypes.c_ulonglong),
        ("removeBodies", ctypes.c_ulonglong),
        ("markBodiesDirty", ctypes.c_ulonglong),
        ("update", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, hkFlags<enum hknpBroadPhaseFilter::Category,unsigned char>)
        ("updateBodyAabbs", ctypes.c_ulonglong),
        ("buildTaskGraph", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, hknpWorld *, hknpSimulationContext *, hkFlags<enum hknpBroadPhaseFilter::Category,unsigned char>, hkTypedBlockStream<hknpBodyIndexPair> *, hknpTaskGraph *)
        ("getExtents", ctypes.c_ulonglong),
        ("findAllPairs", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpSimulationThreadContext *, hkTypedBlockStream<hknpBodyIndexPair>::Writer *)
        ("findNewPairs", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpSimulationThreadContext *, const hkAabbFloat3 *, hkTypedBlockStream<hknpBodyIndexPair>::Writer *)
        ("queryAabb", ctypes.c_ulonglong),
        ("queryAabb_78", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabb_80", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryOutsideOfAabb", ctypes.c_ulonglong),
        ("queryOutsideOfAabb_90", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp", ctypes.c_ulonglong),
        ("queryAabbNmp_a0", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp_a8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castAabb", ctypes.c_ulonglong),
        ("castAabb_b8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbCastQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castRay", ctypes.c_ulonglong),
        ("castShape", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpShapeCastQuery *, const hkRotationImpl<float> *, const hknpCollisionQueryDispatcher *, hknpCollisionQueryCollector *, hknpCollisionQueryCollector *, hknpCollisionQueryContext *)
        ("queryPoint", ctypes.c_ulonglong),
        ("getClosestPoints", ctypes.c_ulonglong),
        ("getClosestPointsAdvanced", ctypes.c_ulonglong),
        ("getClosestPointsParticles", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
    ]
    hknpBroadPhase_dtor_0: int
    getType: int
    getConfig: int
    setBodyBuffer: int
    translate: int
    addBodies: int
    removeBodies: int
    markBodiesDirty: int
    update: bytes
    updateBodyAabbs: int
    buildTaskGraph: bytes
    getExtents: int
    findAllPairs: bytes
    findNewPairs: bytes
    queryAabb: int
    queryAabb_78: bytes
    queryAabb_80: bytes
    queryOutsideOfAabb: int
    queryOutsideOfAabb_90: bytes
    queryAabbNmp: int
    queryAabbNmp_a0: bytes
    queryAabbNmp_a8: bytes
    castAabb: int
    castAabb_b8: bytes
    castRay: int
    castShape: bytes
    queryPoint: int
    getClosestPoints: int
    getClosestPointsAdvanced: int
    getClosestPointsParticles: int
    copyFrom: int

class hkStridedVertices(ctypes.Structure):
    _fields_ = [
        ("m_vertices", ctypes.c_ulonglong),
        ("m_numVertices", ctypes.c_int32),
        ("m_striding", ctypes.c_int32),
    ]
    m_vertices: int
    m_numVertices: int
    m_striding: int

class hkgpConvexHull_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapeMassProperties_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpConvexShapeFunctions(ctypes.Structure):
    def getNumberOfFaces(): pass

    def getLeafShapes(): pass

    def getReflectType(): pass

    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def calcBoundingRadius(): pass

    def calcBoundingSphere(): pass

    def calcInnerRadius(): pass

    def calcInnerSphere(): pass

    def calcMinAngleBetweenFaces(): pass

    def castRay(): pass

    def convertVertexIdsToVertices(): pass

    def getConvexVertices(): pass

    def getFaceInfo(): pass

    def getFaceVertices(): pass

    def getMemoryFootprint(): pass

    def getMutationSignals(): pass

    def getNumberOfSupportSpheres(): pass

    def getShapeKeys(): pass

    def getSignedDistances(): pass

    def getSupportSpheres(): pass

    def getSupportingFace(): pass

    def getSupportingVertex(): pass

    def getTypeName(): pass

    def queryAabb(): pass

    def queryPoint(): pass


class hknpScaledConvexShapeBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpScaledConvexShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpScaledConvexShapeFunctions(ctypes.Structure):
    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def calcBoundingRadius(): pass

    def calcBoundingSphere(): pass

    def calcInnerRadius(): pass

    def calcInnerSphere(): pass

    def calcMinAngleBetweenFaces(): pass

    def calcSurfaceVelocity(): pass

    def castRay(): pass

    def convertVertexIdsToVertices(): pass

    def getCircularSymmetryAxis(): pass

    def getConvexVertices(): pass

    def getFaceInfo(): pass

    def getFaceVertices(): pass

    def getMemoryFootprint(): pass

    def getMutationSignals(): pass

    def getNumberOfFaces(): pass

    def getNumberOfSupportSpheres(): pass

    def getSupportSpheres(): pass

    def getSupportingFace(): pass

    def getSupportingVertex(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def onCoreShapeMutated(): pass

    def queryAabb(): pass

    def getReflectType(): pass


class hknpShapeTagPathEntry(ctypes.Structure):
    _fields_ = [
        ("m_shapeTag", ctypes.c_uint16),
        ("m_parentShape", ctypes.c_ulonglong),
        ("m_instanceId", ctypes.c_uint32),
        ("m_shape", ctypes.c_ulonglong),
    ]
    m_shapeTag: int
    m_parentShape: int
    m_instanceId: int
    m_shape: int

class hknpAabbQueryUtil(ctypes.Structure):
    pass

class hknpQueryAabbNmpUtil(ctypes.Structure):
    pass

class hkBsdSocket_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("close", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("write", ctypes.c_ulonglong),
        ("connect", ctypes.c_ulonglong),
        ("connect_40", ctypes.c_ulonglong),
        ("listen", ctypes.c_ulonglong),
        ("bind", ctypes.c_ulonglong),
        ("getInetAddr", ctypes.c_ulonglong),
        ("canWrite", ctypes.c_ulonglong),
        ("canRead", ctypes.c_ulonglong),
        ("pollForNewClient", ctypes.c_ulonglong),
        ("createDatagramSocket", ctypes.c_ulonglong),
        ("sendTo", ctypes.c_ulonglong),
        ("receiveFrom", ctypes.c_ulonglong),
        ("getLastError", ctypes.c_ulonglong),
        ("getAddress", ctypes.c_ulonglong),
        ("setBlocking", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    close: int
    read: int
    write: int
    connect: int
    connect_40: int
    listen: int
    bind: int
    getInetAddr: int
    canWrite: int
    canRead: int
    pollForNewClient: int
    createDatagramSocket: int
    sendTo: int
    receiveFrom: int
    getLastError: int
    getAddress: int
    setBlocking: int

class hknpClosestHitCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class hknpParticleShapeTagCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpShapeTagCodec> *(__fastcall *)(hknpShapeTagCodec *this, hkRefPtr<hknpShapeTagCodec> *result)
        ("setMaterialLibrary", ctypes.c_ulonglong),
        ("overrideInitialFilterData", ctypes.c_ulonglong),
        ("decode", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpShapeTagCodec *this, unsigned __int16, hknpCollisionQueryType::Enum, const hknpBody *, const hknpShape *, const hknpShape *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, const hknpShape *, hknpQueryFilterData *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    copyFrom: int
    makeClone: bytes
    setMaterialLibrary: int
    overrideInitialFilterData: int
    decode: bytes

class hknpTriangleShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcInnerSphere(): pass

    def castRay(): pass

    def getConvexVertices(): pass

    def getSupportingFace(): pass

    def getTypeName(): pass


class hknpLodShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpLodShapeFunctions(ctypes.Structure):
    def estimateNumShapeKeys(): pass

    def getMutationSignals(): pass

    def getReflectType(): pass

    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def calcBoundingRadius(): pass

    def calcBoundingSphere(): pass

    def calcMinAngleBetweenFaces(): pass

    def castRay(): pass

    def getLeafShapes(): pass

    def getMassProperties(): pass

    def getShapeKeys(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def queryAabb(): pass


class hknpMaskedShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapeUtil(ctypes.Structure):
    def buildAabbMassProperties(): pass

    def buildBoxMassProperties(): pass

    def buildCapsuleMassProperties(): pass

    def buildCylinderMassProperties(): pass

    def buildHullMassProperties(): pass

    def buildHullMassPropertiesFromVertices(): pass

    def buildPolygonMassProperties(): pass

    def buildQuadMassProperties(): pass

    def buildShapeDisplayGeometries(): pass

    def buildShapeMassProperties(): pass

    def buildSphereMassProperties(): pass

    def buildTriangleMassProperties(): pass

    def calcAabbWithScale(): pass

    def calcInnerRadiusConvex(): pass

    def calcInnerSphereAabb(): pass

    def calcInnerSphereConvex(): pass

    def calcScalingParameters(): pass

    def createConvexHullEdges(): pass

    def createConvexHullGeometry(): pass

    def getSignedDistanceContactsImpl(): pass

    def queryPointUsingSphereShape(): pass


class hk3PointDisplayMaterial(ctypes.Structure):
    _fields_ = [
        ("m_style", ctypes.c_char * 0x1),  # hkEnum<enum hk3PointDisplayStyle::Enum,unsigned char>
        ("m_color", ctypes.c_uint32),
    ]
    m_style: bytes
    m_color: int

class hkGeometryDecorations_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkDisplayGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class hknpMaskedShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildSurfaceGeometry(): pass

    def castRay(): pass

    def getLeafShapes(): pass

    def getMutationSignals(): pass

    def getShapeKeys(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def onCoreShapeMutated(): pass

    def onFinishEditingMask(): pass

    def queryAabb(): pass


class hknpCompositeShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class ResultStringTuple(ctypes.Structure):
    _fields_ = [
        ("code", ctypes.c_int32),
        ("str", ctypes.c_char_p),
        ("desc", ctypes.c_char_p),
    ]
    code: int
    str: bytes
    desc: bytes

class hkMonitorStreamParser_TempNode(ctypes.Structure):
    _fields_ = [
        ("m_node", ctypes.c_ulonglong),
        ("m_moveToTag", ctypes.c_uint64),
        ("m_timerBeginWasSkipped", ctypes.c_ubyte),
    ]
    m_node: int
    m_moveToTag: int
    m_timerBeginWasSkipped: bool

class hknpCharacterOutput(ctypes.Structure):
    _fields_ = [
        ("m_velocity", common.Vector4f),
        ("m_inputVelocity", common.Vector4f),
    ]
    m_velocity: common.Vector4f
    m_inputVelocity: common.Vector4f

class hknpCharacterState_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("update", ctypes.c_ulonglong),
        ("getNextState", ctypes.c_ulonglong),
        ("enterState", ctypes.c_ulonglong),
        ("leaveState", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    update: int
    getNextState: int
    enterState: int
    leaveState: int

class hknpCharacterStateManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCharacterContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpExternMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpExternMeshShapeGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getTriangleVertices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getTriangleFlags", ctypes.c_ulonglong),
        ("getNumQuads", ctypes.c_ulonglong),
        ("getQuadVertices", ctypes.c_ulonglong),
        ("getQuadShapeTag", ctypes.c_ulonglong),
        ("getQuadFlags", ctypes.c_ulonglong),
        ("getNumPrimitives", ctypes.c_ulonglong),
        ("getPrimitiveVertices", ctypes.c_ulonglong),
        ("getPrimitiveShapeTag", ctypes.c_ulonglong),
        ("getPrimitiveFlags", ctypes.c_ulonglong),
        ("hasSurfaceVelocity", ctypes.c_ulonglong),
        ("calcSurfaceVelocity", ctypes.c_ulonglong),
        ("generateGeometry", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpExternMeshShapeGeometry> *(__fastcall *)(hknpExternMeshShapeGeometry *this, hkRefPtr<hknpExternMeshShapeGeometry> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumTriangles: int
    getTriangleVertices: int
    getTriangleShapeTag: int
    getTriangleFlags: int
    getNumQuads: int
    getQuadVertices: int
    getQuadShapeTag: int
    getQuadFlags: int
    getNumPrimitives: int
    getPrimitiveVertices: int
    getPrimitiveShapeTag: int
    getPrimitiveFlags: int
    hasSurfaceVelocity: int
    calcSurfaceVelocity: int
    generateGeometry: int
    makeClone: bytes

class hknpExternMeshShapeBuildContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("BuildContext_dtor_0", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("onBuildHierarchy", ctypes.c_ulonglong),
        ("splitRangeInFour", ctypes.c_ulonglong),
    ]
    BuildContext_dtor_0: int
    copyFrom: int
    onBuildHierarchy: int
    splitRangeInFour: int

class hknpExternMeshShapeData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpExternMeshShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def calcSurfaceVelocity(): pass

    def castRay(): pass

    def castShapeImpl(): pass

    def getLeafShapes(): pass

    def getMemoryFootprint(): pass

    def getMutationSignals(): pass

    def getShapeKeys(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def queryAabb(): pass


class hknpCompoundShapeData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpEmptyShapeInstance(ctypes.Structure):
    _fields_ = [
        ("m_padding1", ctypes.c_uint8 * 0x1C),  # unsigned __int8[28]
        ("m_flags", ctypes.c_uint8),
        ("m_padding2", ctypes.c_uint8 * 0xF),  # unsigned __int8[15]
        ("m_nextEmptyElement", ctypes.c_uint32),
    ]
    m_padding1: list[ctypes.c_uint8]
    m_flags: int
    m_padding2: list[ctypes.c_uint8]
    m_nextEmptyElement: int

class hknpConnectivityGraph_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCompoundShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpMassPropertiesTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCompoundShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def calcBoundingRadius(): pass

    def calcBoundingSphere(): pass

    def calcSurfaceVelocity(): pass

    def castRay(): pass

    def castShapeImpl(): pass

    def getClosestPointsImpl(): pass

    def getLeafShapes(): pass

    def getMemoryFootprint(): pass

    def getMutationSignals(): pass

    def getShapeKeys(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def makeShapeKeyMask(): pass

    def queryAabb(): pass

    def queryPoint(): pass


class hknpParticlesDynamicHitsCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
        ("addHitInternal", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int
    addHitInternal: int

class hknpExternMeshShapeInternals_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpGlobals_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkBlockStream_BlockHeader(ctypes.Structure):
    _fields_ = [
        ("m_magic", ctypes.c_int32),
        ("m_numElementsAndBytesUsed", ctypes.c_uint32),
    ]
    m_magic: int
    m_numElementsAndBytesUsed: int

class hknpDefaultExternMeshShapeGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getTriangleVertices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getTriangleFlags", ctypes.c_ulonglong),
        ("getNumQuads", ctypes.c_ulonglong),
        ("getQuadVertices", ctypes.c_ulonglong),
        ("getQuadShapeTag", ctypes.c_ulonglong),
        ("getQuadFlags", ctypes.c_ulonglong),
        ("getNumPrimitives", ctypes.c_ulonglong),
        ("getPrimitiveVertices", ctypes.c_ulonglong),
        ("getPrimitiveShapeTag", ctypes.c_ulonglong),
        ("getPrimitiveFlags", ctypes.c_ulonglong),
        ("hasSurfaceVelocity", ctypes.c_ulonglong),
        ("calcSurfaceVelocity", ctypes.c_ulonglong),
        ("generateGeometry", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpExternMeshShapeGeometry> *(__fastcall *)(hknpExternMeshShapeGeometry *this, hkRefPtr<hknpExternMeshShapeGeometry> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumTriangles: int
    getTriangleVertices: int
    getTriangleShapeTag: int
    getTriangleFlags: int
    getNumQuads: int
    getQuadVertices: int
    getQuadShapeTag: int
    getQuadFlags: int
    getNumPrimitives: int
    getPrimitiveVertices: int
    getPrimitiveShapeTag: int
    getPrimitiveFlags: int
    hasSurfaceVelocity: int
    calcSurfaceVelocity: int
    generateGeometry: int
    makeClone: bytes

class hkcdTreeQueriesStacks(ctypes.Structure):
    pass

class hknpSphereShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcBoundingRadius(): pass

    def calcBoundingSphere(): pass

    def calcInnerSphere(): pass

    def calcMinAngleBetweenFaces(): pass

    def castRay(): pass

    def getConvexVertices(): pass

    def getSupportingVertex(): pass

    def getTypeName(): pass

    def queryPoint(): pass


class hknpCapsuleShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCapsuleShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def calcBoundingSphere(): pass

    def calcInnerSphere(): pass

    def castRay(): pass

    def getConvexVertices(): pass

    def getFaceInfo(): pass

    def getFaceVertices(): pass

    def getSignedDistances(): pass

    def getSupportSpheres(): pass

    def getSupportingFace(): pass

    def getTypeName(): pass

    def makeClone(): pass


class hknpCylinderShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCylinderShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildMassProperties(): pass

    def calcBoundingSphere(): pass

    def calcInnerSphere(): pass

    def calcMinAngleBetweenFaces(): pass

    def getCircularSymmetryAxis(): pass

    def getMemoryFootprint(): pass

    def getTypeName(): pass

    def makeClone(): pass


class in_addr(ctypes.Structure):
    _fields_ = [
        ("S_un", ctypes.c_char * 0x4),  # in_addr::<unnamed_type_S_un>
    ]
    S_un: bytes

class sockaddr(ctypes.Structure):
    _fields_ = [
        ("sa_family", ctypes.c_uint16),
        ("sa_data", ctypes.c_char * 0xE),  # char[14]
    ]
    sa_family: int
    sa_data: list[ctypes.c_char]

class sockaddr_storage(ctypes.Structure):
    _fields_ = [
        ("ss_family", ctypes.c_uint16),
        ("__ss_pad1", ctypes.c_char * 0x6),  # char[6]
        ("__ss_align", ctypes.c_int64),
        ("__ss_pad2", ctypes.c_char * 0x70),  # char[112]
    ]
    ss_family: int
    __ss_pad1: list[ctypes.c_char]
    __ss_align: int
    __ss_pad2: list[ctypes.c_char]

class SCOPE_ID(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $E35561AB18F25E8110E6E68C023132F6
    ]
    ___u0: bytes

class addrinfo(ctypes.Structure):
    _fields_ = [
        ("ai_flags", ctypes.c_int32),
        ("ai_family", ctypes.c_int32),
        ("ai_socktype", ctypes.c_int32),
        ("ai_protocol", ctypes.c_int32),
        ("ai_addrlen", ctypes.c_uint64),
        ("ai_canonname", ctypes.c_char_p),
        ("ai_addr", ctypes.c_ulonglong),
        ("ai_next", ctypes.c_ulonglong),
    ]
    ai_flags: int
    ai_family: int
    ai_socktype: int
    ai_protocol: int
    ai_addrlen: int
    ai_canonname: bytes
    ai_addr: int
    ai_next: int

class WSAData(ctypes.Structure):
    _fields_ = [
        ("wVersion", ctypes.c_uint16),
        ("wHighVersion", ctypes.c_uint16),
        ("iMaxSockets", ctypes.c_uint16),
        ("iMaxUdpDg", ctypes.c_uint16),
        ("lpVendorInfo", ctypes.c_char_p),
        ("szDescription", ctypes.c_char * 0x101),  # char[257]
        ("szSystemStatus", ctypes.c_char * 0x81),  # char[129]
    ]
    wVersion: int
    wHighVersion: int
    iMaxSockets: int
    iMaxUdpDg: int
    lpVendorInfo: bytes
    szDescription: list[ctypes.c_char]
    szSystemStatus: list[ctypes.c_char]

class hknpBoxShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _WSAPROTOCOLCHAIN(ctypes.Structure):
    _fields_ = [
        ("ChainLen", ctypes.c_int32),
        ("ChainEntries", ctypes.c_uint32 * 0x7),  # unsigned int[7]
    ]
    ChainLen: int
    ChainEntries: list[ctypes.c_uint32]

class in6_addr(ctypes.Structure):
    _fields_ = [
        ("u", ctypes.c_char * 0x10),  # in6_addr::<unnamed_type_u>
    ]
    u: bytes

class hknpFirstPersonGun_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("initGun", ctypes.c_ulonglong),
        ("fireGun", ctypes.c_ulonglong),
        ("stepGun", ctypes.c_ulonglong),
        ("quitGun", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("createClone", ctypes.c_char * 0x8),  # hkRefNew<hknpFirstPersonGun> *(__fastcall *)(hknpFirstPersonGun *this, hkRefNew<hknpFirstPersonGun> *result, hknpWorld *)
        ("syncCloneFrom", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    initGun: int
    fireGun: int
    stepGun: int
    quitGun: int
    getName: int
    reset: int
    createClone: bytes
    syncCloneFrom: int

class hknpFirstPersonGunBulletListener_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("bulletCreatedCallback", ctypes.c_ulonglong),
        ("bulletDeletedCallback", ctypes.c_ulonglong),
        ("gunFiredCallback", ctypes.c_ulonglong),
        ("gunStateChangedCallback", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    bulletCreatedCallback: int
    bulletDeletedCallback: int
    gunFiredCallback: int
    gunStateChangedCallback: int

class hknpBoxShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def calcMinAngleBetweenFaces(): pass

    def buildMassProperties(): pass

    def calcAabb(): pass

    def calcBoundingSphere(): pass

    def calcInnerSphere(): pass

    def getConvexVertices(): pass

    def getMemoryFootprint(): pass

    def getSupportingFace(): pass

    def getSupportingVertex(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def queryPoint(): pass


class fd_set(ctypes.Structure):
    _fields_ = [
        ("fd_count", ctypes.c_uint32),
        ("fd_array", ctypes.c_uint64 * 0x40),  # unsigned __int64[64]
    ]
    fd_count: int
    fd_array: list[ctypes.c_uint64]

class timeval(ctypes.Structure):
    _fields_ = [
        ("tv_sec", ctypes.c_int32),
        ("tv_usec", ctypes.c_int32),
    ]
    tv_sec: int
    tv_usec: int

class hostent(ctypes.Structure):
    _fields_ = [
        ("h_name", ctypes.c_char_p),
        ("h_aliases", ctypes.c_ulonglong),
        ("h_addrtype", ctypes.c_int16),
        ("h_length", ctypes.c_int16),
        ("h_addr_list", ctypes.c_ulonglong),
    ]
    h_name: bytes
    h_aliases: int
    h_addrtype: int
    h_length: int
    h_addr_list: int

class hknpCompressedMeshShapeCinfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCompressedMeshShapeCinfo_dtor_0", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getVertex", ctypes.c_ulonglong),
        ("getIndices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getNumConvexShapes", ctypes.c_ulonglong),
        ("getConvexShape", ctypes.c_ulonglong),
        ("getConvexShapeTag", ctypes.c_ulonglong),
    ]
    hknpCompressedMeshShapeCinfo_dtor_0: int
    getNumVertices: int
    getNumTriangles: int
    getVertex: int
    getIndices: int
    getTriangleShapeTag: int
    getNumConvexShapes: int
    getConvexShape: int
    getConvexShapeTag: int

class hknpDefaultCompressedMeshShapeCinfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCompressedMeshShapeCinfo_dtor_0", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getVertex", ctypes.c_ulonglong),
        ("getIndices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getNumConvexShapes", ctypes.c_ulonglong),
        ("getConvexShape", ctypes.c_ulonglong),
        ("getConvexShapeTag", ctypes.c_ulonglong),
    ]
    hknpCompressedMeshShapeCinfo_dtor_0: int
    getNumVertices: int
    getNumTriangles: int
    getVertex: int
    getIndices: int
    getTriangleShapeTag: int
    getNumConvexShapes: int
    getConvexShape: int
    getConvexShapeTag: int

class hknpProjectileGun_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("initGun", ctypes.c_ulonglong),
        ("fireGun", ctypes.c_ulonglong),
        ("stepGun", ctypes.c_ulonglong),
        ("quitGun", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("createClone", ctypes.c_char * 0x8),  # hkRefNew<hknpFirstPersonGun> *(__fastcall *)(hknpFirstPersonGun *this, hkRefNew<hknpFirstPersonGun> *result, hknpWorld *)
        ("syncCloneFrom", ctypes.c_ulonglong),
        ("onGunFired", ctypes.c_ulonglong),
        ("onUpdate", ctypes.c_ulonglong),
        ("destroyAgedProjectiles", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    initGun: int
    fireGun: int
    stepGun: int
    quitGun: int
    getName: int
    reset: int
    createClone: bytes
    syncCloneFrom: int
    onGunFired: int
    onUpdate: int
    destroyAgedProjectiles: int

class hknpGunProjectile_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("onDestroy", ctypes.c_ulonglong),
        ("onHit", ctypes.c_ulonglong),
        ("onUpdate", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    onDestroy: int
    onHit: int
    onUpdate: int

class hknpCollisionCacheBatchConsumerTop(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x38),  # hkTypedBlockStream<hknpCollisionCache>::BatchConsumer
    ]
    baseclass_0: bytes

class hknpCompressedMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCompressedMeshShapeData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpBallGun_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("initGun", ctypes.c_ulonglong),
        ("fireGun", ctypes.c_ulonglong),
        ("stepGun", ctypes.c_ulonglong),
        ("quitGun", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("createClone", ctypes.c_char * 0x8),  # hkRefNew<hknpFirstPersonGun> *(__fastcall *)(hknpFirstPersonGun *this, hkRefNew<hknpFirstPersonGun> *result, hknpWorld *)
        ("syncCloneFrom", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    initGun: int
    fireGun: int
    stepGun: int
    quitGun: int
    getName: int
    reset: int
    createClone: bytes
    syncCloneFrom: int

class hkTaskGraphEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("clear", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    clear: int

class hknpCompressedMeshShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def calcAabb(): pass

    def castShapeImpl(): pass

    def getMemoryFootprint(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def queryAabb(): pass

    def buildSurfaceGeometry(): pass

    def castRay(): pass

    def getLeafShapes(): pass

    def getShapeKeys(): pass

    def makeShapeKeyMask(): pass


class hknpAabb8(ctypes.Structure):
    _fields_ = [
        ("m_min", ctypes.c_uint32),
        ("m_max", ctypes.c_uint32),
    ]
    m_min: int
    m_max: int

class hknpMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkUint3(ctypes.Structure):
    _fields_ = [
        ("m_0", ctypes.c_uint32),
        ("m_1", ctypes.c_uint32),
        ("m_2", ctypes.c_uint32),
    ]
    m_0: int
    m_1: int
    m_2: int

class hknpMeshShapeStats(ctypes.Structure):
    _fields_ = [
        ("m_numTriangles", ctypes.c_int32),
        ("m_numQuads", ctypes.c_int32),
        ("m_numFlatConvexQuads", ctypes.c_int32),
        ("m_numInteriorTriangles", ctypes.c_int32),
        ("m_numVertices", ctypes.c_int32),
        ("m_numRawVertices", ctypes.c_int32),
        ("m_numRemovedDegenerateTriangles", ctypes.c_int32),
        ("m_numSections", ctypes.c_int32),
        ("m_numSectionsWithRawVertices", ctypes.c_int32),
        ("m_avgPrimitivesPerSection", ctypes.c_float),
        ("m_avgVerticesPerSection", ctypes.c_float),
        ("m_topLevelTreeSize", ctypes.c_int32),
        ("m_sectionTreesSize", ctypes.c_int32),
        ("m_sectionPrimitivesSize", ctypes.c_int32),
        ("m_shapeTagTableSize", ctypes.c_int32),
        ("m_totalSectionsSize", ctypes.c_int32),
        ("m_primitiveMappingSize", ctypes.c_int32),
        ("m_totalSize", ctypes.c_int32),
        ("m_primitiveMappingAvgBytesPerTriangle", ctypes.c_float),
        ("m_avgBytesPerTriangle", ctypes.c_float),
    ]
    m_numTriangles: int
    m_numQuads: int
    m_numFlatConvexQuads: int
    m_numInteriorTriangles: int
    m_numVertices: int
    m_numRawVertices: int
    m_numRemovedDegenerateTriangles: int
    m_numSections: int
    m_numSectionsWithRawVertices: int
    m_avgPrimitivesPerSection: float
    m_avgVerticesPerSection: float
    m_topLevelTreeSize: int
    m_sectionTreesSize: int
    m_sectionPrimitivesSize: int
    m_shapeTagTableSize: int
    m_totalSectionsSize: int
    m_primitiveMappingSize: int
    m_totalSize: int
    m_primitiveMappingAvgBytesPerTriangle: float
    m_avgBytesPerTriangle: float

class hknpMeshShapeBuildOutput(ctypes.Structure):
    _fields_ = [
        ("m_primitiveMappingOut", ctypes.c_ulonglong),
        ("m_statsOut", ctypes.c_ulonglong),
    ]
    m_primitiveMappingOut: int
    m_statsOut: int

class hknpMeshShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def getLeafShapes(): pass

    def getMemoryFootprint(): pass

    def getShapeKeys(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def castRay(): pass

    def queryAabb(): pass

    def queryPoint(): pass


class hknpConnectivityGraphMask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpHeightFieldBoundingVolume_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpHeightFieldGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumHeights", ctypes.c_ulonglong),
        ("getQuadInfoAt", ctypes.c_ulonglong),
        ("isMutable", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpHeightFieldGeometry> *(__fastcall *)(hknpHeightFieldGeometry *this, hkRefPtr<hknpHeightFieldGeometry> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumHeights: int
    getQuadInfoAt: int
    isMutable: int
    makeClone: bytes

class hknpHeightFieldShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpHeightFieldShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def calcAabb(): pass

    def getMutationSignals(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def buildSurfaceGeometry(): pass

    def castRay(): pass

    def castShapeImpl(): pass

    def getClosestPointsImpl(): pass

    def getClosestPointsToHeightfieldImpl(): pass

    def getLeafShapes(): pass

    def getShapeKeys(): pass

    def getSignedDistances(): pass

    def queryAabb(): pass


class hknpTransformedShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpWeldFeature(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class hknpVehicleSuspension_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcSuspension", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcSuspension: int

class hknpVehicleFrictionDescription_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpVehicleData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("init", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpVehicleData *this, const hkArray<hknpVehicleSuspension::SuspensionWheelParameters,hkBuiltinContainerAllocator<0,0> > *, const hkTransformf *, const hknpMotion *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    init: bytes

class hknpVehicleDriverInputStatus_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("clone", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    clone: int

class hknpVehicleDriverInput_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcDriverInput", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcDriverInput: int

class hknpVehicleSteering_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcSteering", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcSteering: int

class hknpVehicleEngine_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcEngineInfo", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcEngineInfo: int

class hknpVehicleTransmission_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcTransmission", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcTransmission: int

class hknpVehicleBrake_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcBreakingInfo", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcBreakingInfo: int

class hknpVehicleAerodynamics_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcAerodynamics", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcAerodynamics: int

class hknpTyremarksWheel_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpTyremarksInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("updateTyremarksInfo", ctypes.c_ulonglong),
        ("getWheelTyremarksStrips", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    updateTyremarksInfo: int
    getWheelTyremarksStrips: int

class hknpVehicleVelocityDamper_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("applyVelocityDamping", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    applyVelocityDamping: int

class hknpVehicleInstance_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
        ("init", ctypes.c_ulonglong),
        ("calcCurrentPositionAndRotation", ctypes.c_ulonglong),
        ("calcRPM", ctypes.c_ulonglong),
        ("calcKMPH", ctypes.c_ulonglong),
        ("calcMPH", ctypes.c_ulonglong),
        ("handleFixedGroundAccum", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int
    init: int
    calcCurrentPositionAndRotation: int
    calcRPM: int
    calcKMPH: int
    calcMPH: int
    handleFixedGroundAccum: int

class hknpVehicleWheelCollide_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("init", ctypes.c_ulonglong),
        ("collideWheels", ctypes.c_ulonglong),
        ("updateBeforeCollisionDetection", ctypes.c_ulonglong),
        ("addToWorld", ctypes.c_ulonglong),
        ("removeFromWorld", ctypes.c_ulonglong),
        ("setCollisionFilterInfo", ctypes.c_ulonglong),
        ("wheelCollideCallback", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    init: int
    collideWheels: int
    updateBeforeCollisionDetection: int
    addToWorld: int
    removeFromWorld: int
    setCollisionFilterInfo: int
    wheelCollideCallback: int

class hknpTransformedShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildMassProperties(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def calcBoundingRadius(): pass

    def calcBoundingSphere(): pass

    def calcInnerRadius(): pass

    def calcInnerSphere(): pass

    def calcMinAngleBetweenFaces(): pass

    def calcSurfaceVelocity(): pass

    def castRay(): pass

    def convertVertexIdsToVertices(): pass

    def getCircularSymmetryAxis(): pass

    def getConvexVertices(): pass

    def getFaceInfo(): pass

    def getFaceVertices(): pass

    def getLeafShapes(): pass

    def getMassProperties(): pass

    def getMemoryFootprint(): pass

    def getMutationSignals(): pass

    def getSupportSpheres(): pass

    def getSupportingFace(): pass

    def getSupportingVertex(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def queryAabb(): pass

    def queryPoint(): pass


class hknpCompoundShapeKeyMask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCompoundShapeInternalsSimdTreeKeyMask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpVehicleFrictionSolverAxleParams(ctypes.Structure):
    _fields_ = [
        ("m_contactPointWs", common.Vector4f),
        ("m_constraintNormalWs", common.Vector4f),
        ("m_forwardDirWs", common.Vector4f),
        ("m_groundObject", ctypes.c_ulonglong),
        ("m_groundObjectAtLastIntegration", ctypes.c_ulonglong),
        ("m_frictionCoefficient", ctypes.c_float),
        ("m_viscosityFrictionCoefficient", ctypes.c_float),
        ("m_maxFrictionCoefficient", ctypes.c_float),
        ("m_wheelDownForce", ctypes.c_float),
        ("m_forwardForce", ctypes.c_float),
        ("m_slipVelocityFactor", ctypes.c_float),
        ("m_wheelFixed", ctypes.c_uint8),
    ]
    m_contactPointWs: common.Vector4f
    m_constraintNormalWs: common.Vector4f
    m_forwardDirWs: common.Vector4f
    m_groundObject: int
    m_groundObjectAtLastIntegration: int
    m_frictionCoefficient: float
    m_viscosityFrictionCoefficient: float
    m_maxFrictionCoefficient: float
    m_wheelDownForce: float
    m_forwardForce: float
    m_slipVelocityFactor: float
    m_wheelFixed: int

class hknpCompoundShapeInternalsCdDynamicTreeKeyMask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapeKeyMaskNullFunctions(ctypes.Structure):
    def commitChanges(): pass

    def isInstanceEnabled(): pass

    def isShapeKeyEnabled(): pass

    def makeClone(): pass

    def reset(): pass

    def setShapeKeyEnabled(): pass


class hknpCompoundShapeKeyMaskFunctions(ctypes.Structure):
    def isShapeKeyEnabled(): pass


class hknpCompoundShapeSimdTreeKeyMaskFunctions(ctypes.Structure):
    def getReflectType(): pass

    def commitChanges(): pass

    def isInstanceEnabled(): pass

    def makeClone(): pass

    def setShapeKeyEnabled(): pass


class hknpCompoundShapeCdDynamicTreeKeyMaskFunctions(ctypes.Structure):
    def getReflectType(): pass

    def commitChanges(): pass

    def isInstanceEnabled(): pass

    def makeClone(): pass

    def setShapeKeyEnabled(): pass


class hknpMaskedCompoundShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpStreamContactJacobianUtil(ctypes.Structure):
    def calcDistances(): pass

    def calcMinDistance(): pass

    def calculateManifold(): pass

    def disableContact(): pass

    def getContactVelocities(): pass

    def getDv(): pass

    def getJacModHdr(): pass

    def getJacModMfoldData(): pass

    def getJacModNormalVel(): pass

    def getPointEffMass(): pass

    def getPointRhs(): pass

    def scaleEffectiveMass(): pass

    def scaleFriction(): pass

    def scalePenetrations(): pass

    def setDistances(): pass

    def setDynamic(): pass

    def setMaxImpulse(): pass

    def setSurfaceVelocity(): pass


class _Thrd_imp_t(ctypes.Structure):
    _fields_ = [
        ("_Hnd", ctypes.c_void_p),
        ("_Id", ctypes.c_uint32),
    ]
    _Hnd: int
    _Id: int

class _Thrd_t(ctypes.Structure):
    _fields_ = [
        ("_Hnd", ctypes.c_void_p),
        ("_Id", ctypes.c_uint32),
    ]
    _Hnd: int
    _Id: int

class hknpVehicleStepInfo(ctypes.Structure):
    _fields_ = [
        ("m_deltaTime", ctypes.c_float),
        ("m_invDeltaTime", ctypes.c_float),
    ]
    m_deltaTime: float
    m_invDeltaTime: float

class hknpMaskedCompoundShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildMassProperties(): pass

    def calcAabb(): pass

    def getShapeKeys(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def onCoreShapeMutated(): pass

    def onFinishEditingMask(): pass


class hknpVehicleDefaultAerodynamics_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcAerodynamics", ctypes.c_ulonglong),
        ("calcAerodynamicDrag", ctypes.c_ulonglong),
        ("calcAerodynamicLift", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcAerodynamics: int
    calcAerodynamicDrag: int
    calcAerodynamicLift: int

class hknpVehicleDefaultTransmission_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcTransmission", ctypes.c_ulonglong),
        ("calcMainTransmittedTorque", ctypes.c_ulonglong),
        ("calcTransmissionRPM", ctypes.c_ulonglong),
        ("calcIsReversing", ctypes.c_ulonglong),
        ("updateCurrentGear", ctypes.c_ulonglong),
        ("getCurrentRatio", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcTransmission: int
    calcMainTransmittedTorque: int
    calcTransmissionRPM: int
    calcIsReversing: int
    updateCurrentGear: int
    getCurrentRatio: int

class hknpVehicleDefaultVelocityDamper_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("applyVelocityDamping", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    applyVelocityDamping: int

class hknpGroupCollisionFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterBodyPairs", ctypes.c_ulonglong),
        ("isCollisionEnabled", ctypes.c_ulonglong),
        ("isCollisionEnabled_28", ctypes.c_ulonglong),
        ("isCollisionEnabled_30", ctypes.c_ulonglong),
        ("isCollisionEnabled_38", ctypes.c_ulonglong),
        ("getLodInfo", ctypes.c_ulonglong),
        ("subscribeToWorld", ctypes.c_ulonglong),
        ("unsubscribeFromWorld", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpCollisionFilter> *(__fastcall *)(hknpCollisionFilter *this, hkRefPtr<hknpCollisionFilter> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterBodyPairs: int
    isCollisionEnabled: int
    isCollisionEnabled_28: int
    isCollisionEnabled_30: int
    isCollisionEnabled_38: int
    getLodInfo: int
    subscribeToWorld: int
    unsubscribeFromWorld: int
    copyFrom: int
    makeClone: bytes

class hknpShapeConnectivityGraph_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpConstraintInfo(ctypes.Structure):
    _fields_ = [
        ("m_maxSizeOfSchema", ctypes.c_int32),
        ("m_sizeOfSchemas", ctypes.c_int32),
        ("m_numSolverResults", ctypes.c_int32),
        ("m_numSolverElemTemps", ctypes.c_int32),
    ]
    m_maxSizeOfSchema: int
    m_sizeOfSchemas: int
    m_numSolverResults: int
    m_numSolverElemTemps: int

class hknpConstraintDataClonedObjectMap(ctypes.Structure):
    _fields_ = [
        ("m_constraintDataMap", ctypes.c_char * 0x20),  # hkHashMap<hknpConstraintData const *,hknpConstraintData *,hkBuiltinContainerAllocator<0,0> >
        ("m_objectMap", ctypes.c_char * 0x20),  # hkHashMap<hkReferencedObject const *,hkReferencedObject *,hkBuiltinContainerAllocator<0,0> >
    ]
    m_constraintDataMap: bytes
    m_objectMap: bytes

class hknpConstraintQueryStepInfo(ctypes.Structure):
    _fields_ = [
        ("m_subStepDeltaTime", ctypes.c_float),
        ("m_microStepDeltaTime", ctypes.c_float),
        ("m_subStepInvDeltaTime", ctypes.c_float),
        ("m_frameDeltaTime", ctypes.c_float),
        ("m_frameInvDeltaTime", ctypes.c_float),
        ("m_invNumSteps", ctypes.c_float),
        ("m_invNumStepsTimesMicroSteps", ctypes.c_float),
        ("m_rhsFactor", ctypes.c_float),
        ("m_frictionRhsFactor", ctypes.c_float),
    ]
    m_subStepDeltaTime: float
    m_microStepDeltaTime: float
    m_subStepInvDeltaTime: float
    m_frameDeltaTime: float
    m_frameInvDeltaTime: float
    m_invNumSteps: float
    m_invNumStepsTimesMicroSteps: float
    m_rhsFactor: float
    m_frictionRhsFactor: float

class hknpConstraintQueryOut(ctypes.Structure):
    _fields_ = [
        ("m_jacobianSchemas", ctypes.c_ulonglong),
        ("m_constraintRuntime", ctypes.c_ulonglong),
        ("m_instanceRuntime", ctypes.c_ulonglong),
    ]
    m_jacobianSchemas: int
    m_constraintRuntime: int
    m_instanceRuntime: int

class hknpConstraintDataFunctionsSingleton_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpBreakableCompoundShapeKeyMask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpBreakableCompoundShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPhysicsSystemData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("checkConsistency", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    checkConsistency: int

class hknpBreakableCompoundShapeKeyMaskFunctions(ctypes.Structure):
    def getReflectType(): pass

    def commitChanges(): pass

    def isInstanceEnabled(): pass

    def isShapeKeyEnabled(): pass

    def makeClone(): pass


class hknpBreakableCompoundShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildMassProperties(): pass

    def calcAabb(): pass

    def castRay(): pass

    def getTypeName(): pass

    def makeClone(): pass

    def makeShapeKeyMask(): pass

    def onCoreShapeMutated(): pass

    def queryAabb(): pass


class hknpPhysicsSystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpLinearSurfaceVelocity_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcSurfaceVelocity", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcSurfaceVelocity: int

class hknpVehicleDefaultEngine_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcEngineInfo", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcEngineInfo: int

class hknpBodyQualityLibraryCinfo(ctypes.Structure):
    def hknpBodyQualityLibraryCinfo(): pass

    _fields_ = [
        ("m_enableWeldingForDefaultObjects", ctypes.c_ubyte),
        ("m_enableWeldingForCriticalObjects", ctypes.c_ubyte),
        ("m_enableContactCaching", ctypes.c_ubyte),
    ]
    m_enableWeldingForDefaultObjects: bool
    m_enableWeldingForCriticalObjects: bool
    m_enableContactCaching: bool

class hknpShapeMassPropertiesTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpHitScalingCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class hknpCompoundShapeEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkInertiaTensorComputer(ctypes.Structure):
    def combineMassProperties(): pass

    def computeBoxVolumeMassProperties(): pass

    def computeBoxVolumeMassPropertiesDiagonalized(): pass

    def computeCapsuleVolumeMassProperties(): pass

    def computeCylinderVolumeMassProperties(): pass

    def computeGeometryVolumeMassProperties(): pass

    def computeSphereVolumeMassProperties(): pass

    def computeTriangleSurfaceMassProperties(): pass

    def convertInertiaTensorToPrincipleAxis(): pass

    def shiftInertiaFromCom(): pass


class hknpCompressedMeshShapeInternals_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpParticlesColliderShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def castRay(): pass

    def castShapeImpl(): pass

    def getClosestPointsFromConvex(): pass

    def getLeafShapes(): pass

    def getShapeKeys(): pass

    def getTypeName(): pass

    def queryAabb(): pass

    def queryPoint(): pass


class hknpDummyShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpDummyShapeFunctions(ctypes.Structure):
    def getReflectType(): pass

    def calcAabb(): pass

    def getMutationSignals(): pass

    def getTypeName(): pass

    def makeClone(): pass


class hknpFlippedGetClosestPointsQueryCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class hknpMaterialPalette_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpShapeMassConfigProperty_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpDefaultBroadPhaseConfig_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumLayers", ctypes.c_ulonglong),
        ("getLayer", ctypes.c_ulonglong),
        ("getLayerIndices", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumLayers: int
    getLayer: int
    getLayerIndices: int

class hknpPairCollisionFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterBodyPairs", ctypes.c_ulonglong),
        ("isCollisionEnabled", ctypes.c_ulonglong),
        ("isCollisionEnabled_28", ctypes.c_ulonglong),
        ("isCollisionEnabled_30", ctypes.c_ulonglong),
        ("isCollisionEnabled_38", ctypes.c_ulonglong),
        ("getLodInfo", ctypes.c_ulonglong),
        ("subscribeToWorld", ctypes.c_ulonglong),
        ("unsubscribeFromWorld", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpCollisionFilter> *(__fastcall *)(hknpCollisionFilter *this, hkRefPtr<hknpCollisionFilter> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterBodyPairs: int
    isCollisionEnabled: int
    isCollisionEnabled_28: int
    isCollisionEnabled_30: int
    isCollisionEnabled_38: int
    getLodInfo: int
    subscribeToWorld: int
    unsubscribeFromWorld: int
    copyFrom: int
    makeClone: bytes

class hknpConstraintCollisionFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterBodyPairs", ctypes.c_ulonglong),
        ("isCollisionEnabled", ctypes.c_ulonglong),
        ("isCollisionEnabled_28", ctypes.c_ulonglong),
        ("isCollisionEnabled_30", ctypes.c_ulonglong),
        ("isCollisionEnabled_38", ctypes.c_ulonglong),
        ("getLodInfo", ctypes.c_ulonglong),
        ("subscribeToWorld", ctypes.c_ulonglong),
        ("unsubscribeFromWorld", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpCollisionFilter> *(__fastcall *)(hknpCollisionFilter *this, hkRefPtr<hknpCollisionFilter> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterBodyPairs: int
    isCollisionEnabled: int
    isCollisionEnabled_28: int
    isCollisionEnabled_30: int
    isCollisionEnabled_38: int
    getLodInfo: int
    subscribeToWorld: int
    unsubscribeFromWorld: int
    copyFrom: int
    makeClone: bytes

class hknpDisableCollisionFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterBodyPairs", ctypes.c_ulonglong),
        ("isCollisionEnabled", ctypes.c_ulonglong),
        ("isCollisionEnabled_28", ctypes.c_ulonglong),
        ("isCollisionEnabled_30", ctypes.c_ulonglong),
        ("isCollisionEnabled_38", ctypes.c_ulonglong),
        ("getLodInfo", ctypes.c_ulonglong),
        ("subscribeToWorld", ctypes.c_ulonglong),
        ("unsubscribeFromWorld", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpCollisionFilter> *(__fastcall *)(hknpCollisionFilter *this, hkRefPtr<hknpCollisionFilter> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterBodyPairs: int
    isCollisionEnabled: int
    isCollisionEnabled_28: int
    isCollisionEnabled_30: int
    isCollisionEnabled_38: int
    getLodInfo: int
    subscribeToWorld: int
    unsubscribeFromWorld: int
    copyFrom: int
    makeClone: bytes

class hknpDefaultHeightFieldGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumHeights", ctypes.c_ulonglong),
        ("getQuadInfoAt", ctypes.c_ulonglong),
        ("isMutable", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpHeightFieldGeometry> *(__fastcall *)(hknpHeightFieldGeometry *this, hkRefPtr<hknpHeightFieldGeometry> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumHeights: int
    getQuadInfoAt: int
    isMutable: int
    makeClone: bytes

class hknpExtendedExternMeshShapeGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getTriangleVertices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getTriangleFlags", ctypes.c_ulonglong),
        ("getNumQuads", ctypes.c_ulonglong),
        ("getQuadVertices", ctypes.c_ulonglong),
        ("getQuadShapeTag", ctypes.c_ulonglong),
        ("getQuadFlags", ctypes.c_ulonglong),
        ("getNumPrimitives", ctypes.c_ulonglong),
        ("getPrimitiveVertices", ctypes.c_ulonglong),
        ("getPrimitiveShapeTag", ctypes.c_ulonglong),
        ("getPrimitiveFlags", ctypes.c_ulonglong),
        ("hasSurfaceVelocity", ctypes.c_ulonglong),
        ("calcSurfaceVelocity", ctypes.c_ulonglong),
        ("generateGeometry", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpExternMeshShapeGeometry> *(__fastcall *)(hknpExternMeshShapeGeometry *this, hkRefPtr<hknpExternMeshShapeGeometry> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumTriangles: int
    getTriangleVertices: int
    getTriangleShapeTag: int
    getTriangleFlags: int
    getNumQuads: int
    getQuadVertices: int
    getQuadShapeTag: int
    getQuadFlags: int
    getNumPrimitives: int
    getPrimitiveVertices: int
    getPrimitiveShapeTag: int
    getPrimitiveFlags: int
    hasSurfaceVelocity: int
    calcSurfaceVelocity: int
    generateGeometry: int
    makeClone: bytes

class hknpMeshShapeEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpSingleThreadedSolverData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("clear", ctypes.c_ulonglong),
        ("addImmediateContactConstraint", ctypes.c_ulonglong),
        ("addImmediateContactConstraintUsingJacobian", ctypes.c_ulonglong),
        ("addImmediateConstraint", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpSolverData *this, hknpBodyId, hknpBodyId, hknpConstraintData *, hkHandle<unsigned short,65535,hknpImmediateConstraintIdDiscriminant>, hkFlags<enum hknpConstraint::FlagsEnum,unsigned short>)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    clear: int
    addImmediateContactConstraint: int
    addImmediateContactConstraintUsingJacobian: int
    addImmediateConstraint: bytes

class hkProcess(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_inStream", ctypes.c_char * 0x8),  # hkRefPtr<hkVdbIStream>
        ("m_outStream", ctypes.c_char * 0x8),  # hkRefPtr<hkVdbOStream>
        ("m_outStreamLock", ctypes.c_ulonglong),
        ("m_displayHandler", ctypes.c_char * 0x8),  # hkRefPtr<hkDoublePrecisionDebugDisplayHandler>
        ("m_objectHandler", ctypes.c_char * 0x8),  # hkRefPtr<hkServerObjectHandler>
        ("m_fileHandler", ctypes.c_char * 0x8),  # hkRefPtr<hkServerFileHandler>
        ("m_textHandler", ctypes.c_char * 0x8),  # hkRefPtr<hkServerTextHandler>
        ("m_playbackHandler", ctypes.c_char * 0x8),  # hkRefPtr<hkPlaybackHandler>
        ("m_processHandler", ctypes.c_ulonglong),
    ]
    __vftable: int
    m_inStream: bytes
    m_outStream: bytes
    m_outStreamLock: int
    m_displayHandler: bytes
    m_objectHandler: bytes
    m_fileHandler: bytes
    m_textHandler: bytes
    m_playbackHandler: bytes
    m_processHandler: int

class hkProcess_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkProcess_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getProcessId", ctypes.c_ulonglong),
        ("getProcessOptions", ctypes.c_ulonglong),
        ("tweakProcessOptions", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkProcess *this, const hkHashMap<hkStringPtr,hkReflect::Var,hkBuiltinContainerAllocator<0,0> > *)
        ("getConsumableCommands", ctypes.c_ulonglong),
        ("canConsumeDelayedCommand", ctypes.c_ulonglong),
        ("consumeCommand", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("init", ctypes.c_ulonglong),
        ("step", ctypes.c_ulonglong),
        ("postStep", ctypes.c_ulonglong),
        ("onProcessOptionsChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(hkProcess *this, const hkHashMap<hkStringPtr,hkReflect::Var,hkBuiltinContainerAllocator<0,0> > *)
    ]
    hkProcess_dtor_0: int
    getType: int
    getProcessId: int
    getProcessOptions: int
    tweakProcessOptions: bytes
    getConsumableCommands: int
    canConsumeDelayedCommand: int
    consumeCommand: int
    isOk: int
    init: int
    step: int
    postStep: int
    onProcessOptionsChanged: bytes

class hkIArchive(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # hkIArchiveBase<0>
    ]
    baseclass_0: bytes

class hkIArchive_vtbl(ctypes.Structure):
    pass

class hkVdbIStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkOArchive(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x8),  # hkOArchiveBase<0>
    ]
    baseclass_0: bytes

class hkVdbOStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkDeprecatedDebugDisplayHandlerInterface(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hkDeprecatedDebugDisplayHandlerInterface_vtbl(ctypes.Structure):
    _fields_ = [
        ("skinGeometry", ctypes.c_ulonglong),
        ("skinGeometry_8", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDeprecatedDebugDisplayHandlerInterface *this, const unsigned __int64 *, int, const hkMatrix4Impl<float> *, int, const hkMatrix4Impl<float> *)
        ("addTextureSearchPath", ctypes.c_ulonglong),
        ("clearTextureSearchPaths", ctypes.c_ulonglong),
        ("addGeometryHash", ctypes.c_ulonglong),
    ]
    skinGeometry: int
    skinGeometry_8: bytes
    addTextureSearchPath: int
    clearTextureSearchPaths: int
    addGeometryHash: int

class hkDebugDisplayHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("setDisplayOptions", ctypes.c_ulonglong),
        ("addGeometry", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, hkVdbCmdTag)
        ("addGeometryEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, const unsigned int, hkDisplayGeometryFlags, hkVdbCmdTag)
        ("addGeometryInstance", ctypes.c_ulonglong),
        ("addGeometryInstanceEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, unsigned __int64, const hkTransformf *, const unsigned int, const struct hkArrayBase<hkGeometryDecorations *> *, hkVdbCmdTag)
        ("setGeometryFlagBits", ctypes.c_ulonglong),
        ("clearGeometryFlagBits", ctypes.c_ulonglong),
        ("setGeometryColor", ctypes.c_ulonglong),
        ("setGeometryAlpha", ctypes.c_ulonglong),
        ("updateGeometryTransform", ctypes.c_ulonglong),
        ("updateGeometryTransformEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkMatrix4Impl<float> *)
        ("updateParticleTransforms", ctypes.c_ulonglong),
        ("updateGeometryVerts", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArray<hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> >,hkBuiltinContainerAllocator<0,0> > *)
        ("updateGeometryMaterialData", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const struct hkArray<hkDisplayMaterialDatas *,hkBuiltinContainerAllocator<0,0> > *)
        ("removeGeometry", ctypes.c_ulonglong),
        ("updateCamera", ctypes.c_ulonglong),
        ("removeCamera", ctypes.c_ulonglong),
        ("displayPoint", ctypes.c_ulonglong),
        ("displayPointEx", ctypes.c_ulonglong),
        ("display2Points", ctypes.c_ulonglong),
        ("display2PointsEx", ctypes.c_ulonglong),
        ("display3Points", ctypes.c_ulonglong),
        ("display3PointsEx", ctypes.c_ulonglong),
        ("displayGeometry", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, unsigned int, hkVdbCmdTag)
        ("display3dText", ctypes.c_ulonglong),
        ("flushDisplay", ctypes.c_ulonglong),
        ("displayText", ctypes.c_ulonglong),
        ("displayPoint2d", ctypes.c_ulonglong),
        ("displayLine2d", ctypes.c_ulonglong),
        ("displayTriangle2d", ctypes.c_ulonglong),
        ("displayText2d", ctypes.c_ulonglong),
        ("getDeprecatedInterface", ctypes.c_ulonglong),
        ("displayLitTriangle", ctypes.c_ulonglong),
        ("displayFrame", ctypes.c_ulonglong),
        ("displayPlane", ctypes.c_ulonglong),
        ("displayModelSpacePose", ctypes.c_ulonglong),
        ("displayAabb", ctypes.c_ulonglong),
        ("displayTransformedAabb", ctypes.c_ulonglong),
        ("displayLines", ctypes.c_ulonglong),
        ("lockForUpdate", ctypes.c_ulonglong),
        ("unlockForUpdate", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    setDisplayOptions: int
    addGeometry: bytes
    addGeometryEx: bytes
    addGeometryInstance: int
    addGeometryInstanceEx: bytes
    setGeometryFlagBits: int
    clearGeometryFlagBits: int
    setGeometryColor: int
    setGeometryAlpha: int
    updateGeometryTransform: int
    updateGeometryTransformEx: bytes
    updateParticleTransforms: int
    updateGeometryVerts: bytes
    updateGeometryMaterialData: bytes
    removeGeometry: int
    updateCamera: int
    removeCamera: int
    displayPoint: int
    displayPointEx: int
    display2Points: int
    display2PointsEx: int
    display3Points: int
    display3PointsEx: int
    displayGeometry: bytes
    display3dText: int
    flushDisplay: int
    displayText: int
    displayPoint2d: int
    displayLine2d: int
    displayTriangle2d: int
    displayText2d: int
    getDeprecatedInterface: int
    displayLitTriangle: int
    displayFrame: int
    displayPlane: int
    displayModelSpacePose: int
    displayAabb: int
    displayTransformedAabb: int
    displayLines: int
    lockForUpdate: int
    unlockForUpdate: int

class hkDoublePrecisionDebugDisplayHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("setDisplayOptions", ctypes.c_ulonglong),
        ("addGeometry", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, hkVdbCmdTag)
        ("addGeometryEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, const unsigned int, hkDisplayGeometryFlags, hkVdbCmdTag)
        ("addGeometryInstance", ctypes.c_ulonglong),
        ("addGeometryInstanceEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, unsigned __int64, const hkTransformf *, const unsigned int, const struct hkArrayBase<hkGeometryDecorations *> *, hkVdbCmdTag)
        ("setGeometryFlagBits", ctypes.c_ulonglong),
        ("clearGeometryFlagBits", ctypes.c_ulonglong),
        ("setGeometryColor", ctypes.c_ulonglong),
        ("setGeometryAlpha", ctypes.c_ulonglong),
        ("updateGeometryTransform", ctypes.c_ulonglong),
        ("updateGeometryTransformEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkMatrix4Impl<float> *)
        ("updateParticleTransforms", ctypes.c_ulonglong),
        ("updateGeometryVerts", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArray<hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> >,hkBuiltinContainerAllocator<0,0> > *)
        ("updateGeometryMaterialData", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const struct hkArray<hkDisplayMaterialDatas *,hkBuiltinContainerAllocator<0,0> > *)
        ("removeGeometry", ctypes.c_ulonglong),
        ("updateCamera", ctypes.c_ulonglong),
        ("removeCamera", ctypes.c_ulonglong),
        ("displayPoint", ctypes.c_ulonglong),
        ("displayPointEx", ctypes.c_ulonglong),
        ("display2Points", ctypes.c_ulonglong),
        ("display2PointsEx", ctypes.c_ulonglong),
        ("display3Points", ctypes.c_ulonglong),
        ("display3PointsEx", ctypes.c_ulonglong),
        ("displayGeometry", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, unsigned int, hkVdbCmdTag)
        ("display3dText", ctypes.c_ulonglong),
        ("flushDisplay", ctypes.c_ulonglong),
        ("displayText", ctypes.c_ulonglong),
        ("displayPoint2d", ctypes.c_ulonglong),
        ("displayLine2d", ctypes.c_ulonglong),
        ("displayTriangle2d", ctypes.c_ulonglong),
        ("displayText2d", ctypes.c_ulonglong),
        ("getDeprecatedInterface", ctypes.c_ulonglong),
        ("displayLitTriangle", ctypes.c_ulonglong),
        ("displayFrame", ctypes.c_ulonglong),
        ("displayPlane", ctypes.c_ulonglong),
        ("displayModelSpacePose", ctypes.c_ulonglong),
        ("displayAabb", ctypes.c_ulonglong),
        ("displayTransformedAabb", ctypes.c_ulonglong),
        ("displayLines", ctypes.c_ulonglong),
        ("lockForUpdate", ctypes.c_ulonglong),
        ("unlockForUpdate", ctypes.c_ulonglong),
        ("addGeometry_160", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDoublePrecisionDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkPreciseTransform *, hkVdbCmdTag)
        ("addGeometry_168", ctypes.c_ulonglong),
        ("addGeometryEx_170", ctypes.c_ulonglong),
        ("addGeometryEx_178", ctypes.c_ulonglong),
        ("addGeometryEx_180", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDoublePrecisionDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkPreciseTransform *, const unsigned int, hkDisplayGeometryFlags, hkVdbCmdTag)
        ("addGeometryInstance_188", ctypes.c_ulonglong),
        ("addGeometryInstanceEx_190", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDoublePrecisionDebugDisplayHandler *this, unsigned __int64, unsigned __int64, const hkPreciseTransform *, const unsigned int, const struct hkArrayBase<hkGeometryDecorations *> *, hkVdbCmdTag)
        ("updateGeometryTransform_198", ctypes.c_ulonglong),
        ("displayPoint_1a0", ctypes.c_ulonglong),
        ("displayPointEx_1a8", ctypes.c_ulonglong),
        ("display2Points_1b0", ctypes.c_ulonglong),
        ("display2PointsEx_1b8", ctypes.c_ulonglong),
        ("display3Points_1c0", ctypes.c_ulonglong),
        ("display3PointsEx_1c8", ctypes.c_ulonglong),
        ("displayGeometry_1d0", ctypes.c_ulonglong),
        ("displayGeometry_1d8", ctypes.c_ulonglong),
        ("displayGeometry_1e0", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDoublePrecisionDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkPreciseTransform *, unsigned int, hkVdbCmdTag)
        ("display3dText_1e8", ctypes.c_ulonglong),
        ("displayLitTriangle_1f0", ctypes.c_ulonglong),
        ("displayFrame_1f8", ctypes.c_ulonglong),
        ("displayFrame_200", ctypes.c_ulonglong),
        ("displayFrame_208", ctypes.c_ulonglong),
        ("displayFrame_210", ctypes.c_ulonglong),
        ("displayPlane_218", ctypes.c_ulonglong),
        ("displayTransformedAabb_220", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    setDisplayOptions: int
    addGeometry: bytes
    addGeometryEx: bytes
    addGeometryInstance: int
    addGeometryInstanceEx: bytes
    setGeometryFlagBits: int
    clearGeometryFlagBits: int
    setGeometryColor: int
    setGeometryAlpha: int
    updateGeometryTransform: int
    updateGeometryTransformEx: bytes
    updateParticleTransforms: int
    updateGeometryVerts: bytes
    updateGeometryMaterialData: bytes
    removeGeometry: int
    updateCamera: int
    removeCamera: int
    displayPoint: int
    displayPointEx: int
    display2Points: int
    display2PointsEx: int
    display3Points: int
    display3PointsEx: int
    displayGeometry: bytes
    display3dText: int
    flushDisplay: int
    displayText: int
    displayPoint2d: int
    displayLine2d: int
    displayTriangle2d: int
    displayText2d: int
    getDeprecatedInterface: int
    displayLitTriangle: int
    displayFrame: int
    displayPlane: int
    displayModelSpacePose: int
    displayAabb: int
    displayTransformedAabb: int
    displayLines: int
    lockForUpdate: int
    unlockForUpdate: int
    addGeometry_160: bytes
    addGeometry_168: int
    addGeometryEx_170: int
    addGeometryEx_178: int
    addGeometryEx_180: bytes
    addGeometryInstance_188: int
    addGeometryInstanceEx_190: bytes
    updateGeometryTransform_198: int
    displayPoint_1a0: int
    displayPointEx_1a8: int
    display2Points_1b0: int
    display2PointsEx_1b8: int
    display3Points_1c0: int
    display3PointsEx_1c8: int
    displayGeometry_1d0: int
    displayGeometry_1d8: int
    displayGeometry_1e0: bytes
    display3dText_1e8: int
    displayLitTriangle_1f0: int
    displayFrame_1f8: int
    displayFrame_200: int
    displayFrame_208: int
    displayFrame_210: int
    displayPlane_218: int
    displayTransformedAabb_220: int

class Fixups(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class Fixups_vtbl(ctypes.Structure):
    _fields_ = [
        ("Fixups_dtor_0", ctypes.c_ulonglong),
        ("writeInplacePatch", ctypes.c_ulonglong),
        ("writeObjectRef", ctypes.c_char * 0x8),  # void (__fastcall *)(Fixups *this, hkEndian::EndianType<unsigned int,1> *, int, const hkReflect::Type *)
    ]
    Fixups_dtor_0: int
    writeInplacePatch: int
    writeObjectRef: bytes

class hkServerObjectSerializer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkServerObjectHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkProcessHandler(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hkProcessHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkProcessHandler_dtor_0", ctypes.c_ulonglong),
        ("registerProcess", ctypes.c_ulonglong),
        ("getProcessId", ctypes.c_ulonglong),
        ("getProcessName", ctypes.c_ulonglong),
        ("getProcessById", ctypes.c_ulonglong),
        ("getProcessByName", ctypes.c_ulonglong),
        ("getProcessesByType", ctypes.c_char * 0x8),  # int (__fastcall *)(hkProcessHandler *this, hkProcessTypeFlags, hkArray<hkProcess *,hkBuiltinContainerAllocator<0,0> > *)
        ("selectProcess", ctypes.c_ulonglong),
        ("createProcess", ctypes.c_ulonglong),
        ("deleteProcess", ctypes.c_ulonglong),
        ("tweakProcess", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkProcessHandler *this, int, const hkHashMap<hkStringPtr,hkReflect::Var,hkBuiltinContainerAllocator<0,0> > *)
        ("registerCategory", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkProcessHandler *this, const char *, hkPack::hkPackInt<9>)
    ]
    hkProcessHandler_dtor_0: int
    registerProcess: int
    getProcessId: int
    getProcessName: int
    getProcessById: int
    getProcessByName: int
    getProcessesByType: bytes
    selectProcess: int
    createProcess: int
    deleteProcess: int
    tweakProcess: bytes
    registerCategory: bytes

class hknpConstraintMotor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpAngularFrictionConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpLinearClearanceConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkDisplayMaterialDatas_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpConstraintDataNullFunctions(ctypes.Structure):
    def buildJacobian(): pass

    def enableDirectSolver(): pass

    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getConstraintInfo(): pass

    def getInertiaStabilizationFactor(): pass

    def getMaximumAngularImpulse(): pass

    def getMaximumLinearImpulse(): pass

    def getReflectType(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def setInertiaStabilizationFactor(): pass

    def setMaximumAngularImpulse(): pass

    def setMaximumLinearImpulse(): pass


class hknpAngularFrictionConstraintDataFunctions(ctypes.Structure):
    def getConstraintInfo(): pass

    def getRuntimeInfo(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpLinearClearanceConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getConstraintInfo(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpMaxSizeConstraintMotor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknp6DofConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpHingeConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpHingeConstraintDataFunctions(ctypes.Structure):
    def getConstraintInfo(): pass

    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getInertiaStabilizationFactor(): pass

    def getRuntimeInfo(): pass

    def makeClone(): pass

    def setInertiaStabilizationFactor(): pass

    def getReflectType(): pass


class hknp6DofConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getConstraintInfo(): pass

    def getInertiaStabilizationFactor(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def setInertiaStabilizationFactor(): pass

    def updateDirtyAtoms(): pass

    def getReflectType(): pass


class hknpParametricCurve_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getPoint", ctypes.c_ulonglong),
        ("getNearestPoint", ctypes.c_ulonglong),
        ("getTangent", ctypes.c_ulonglong),
        ("getStart", ctypes.c_ulonglong),
        ("getEnd", ctypes.c_ulonglong),
        ("getLengthFromStart", ctypes.c_ulonglong),
        ("getBinormal", ctypes.c_ulonglong),
        ("isClosedLoop", ctypes.c_ulonglong),
        ("getPointsToDraw", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpParametricCurve *this, hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *)
        ("transformPoints", ctypes.c_ulonglong),
        ("clone", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getPoint: int
    getNearestPoint: int
    getTangent: int
    getStart: int
    getEnd: int
    getLengthFromStart: int
    getBinormal: int
    isClosedLoop: int
    getPointsToDraw: bytes
    transformPoints: int
    clone: int

class hknpBallAndSocketConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPointToPathConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpRackAndPinionConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpBallAndSocketConstraintDataFunctions(ctypes.Structure):
    def getConstraintInfo(): pass

    def getInertiaStabilizationFactor(): pass

    def getMaximumAngularImpulse(): pass

    def getMaximumLinearImpulse(): pass

    def makeClone(): pass

    def setInertiaStabilizationFactor(): pass

    def setMaximumAngularImpulse(): pass

    def setMaximumLinearImpulse(): pass

    def getReflectType(): pass


class hknpRackAndPinionConstraintDataFunctions(ctypes.Structure):
    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpCogWheelConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPointToPathConstraintDataFunctions(ctypes.Structure):
    def buildJacobian(): pass

    def getConstraintInfo(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknp1dMotorSolverInfo(ctypes.Structure):
    _fields_ = [
        ("m_maxImpulsePerSubstep", ctypes.c_float),
        ("m_minImpulsePerSubstep", ctypes.c_float),
        ("m_velocity", ctypes.c_float),
        ("m_targetVelocityFactor", ctypes.c_float),
        ("m_tau", ctypes.c_float),
        ("m_damping", ctypes.c_float),
    ]
    m_maxImpulsePerSubstep: float
    m_minImpulsePerSubstep: float
    m_velocity: float
    m_targetVelocityFactor: float
    m_tau: float
    m_damping: float

class hknpCogWheelConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getConstraintInfo(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpDeformableFixedConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpWrappedConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknp1dLinearBilateralConstraintInfo(ctypes.Structure):
    _fields_ = [
        ("m_pivotADiffComA", common.Vector4f),
        ("m_pivotBDiffComA", common.Vector4f),
        ("m_constrainedDofW", common.Vector4f),
        ("_m_bodySpaceIndex", ctypes.c_uint8),
    ]
    m_pivotADiffComA: common.Vector4f
    m_pivotBDiffComA: common.Vector4f
    m_constrainedDofW: common.Vector4f
    _m_bodySpaceIndex: int

    @property
    def m_bodySpaceIndex(self):
        return hknpConstraintAtom.BodySpaceIndex(self._m_bodySpaceIndex)


class hknpWrappedConstraintDataFunctions(ctypes.Structure):
    def enableDirectSolver(): pass

    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getConstraintInfo(): pass

    def getRuntimeInfo(): pass

    def updateDirtyAtoms(): pass


class hkDebugDisplaySystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("createVertexBuffer", ctypes.c_ulonglong),
        ("freeVertexBuffer", ctypes.c_ulonglong),
        ("createIndexBuffer", ctypes.c_ulonglong),
        ("freeIndexBuffer", ctypes.c_ulonglong),
        ("createTexture", ctypes.c_ulonglong),
        ("freeTexture", ctypes.c_ulonglong),
        ("drawMeshes", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDebugDisplaySystem *this, void *, const hkDebugDisplaySystem::DrawMesh *, struct hkArrayView<hkMatrix4Impl<float> const >, hkArrayView<unsigned int const >)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    createVertexBuffer: int
    freeVertexBuffer: int
    createIndexBuffer: int
    freeIndexBuffer: int
    createTexture: int
    freeTexture: int
    drawMeshes: bytes

class hknp1dAngularBilateralConstraintInfo(ctypes.Structure):
    _fields_ = [
        ("m_zeroErrorAxisAinW", common.Vector4f),
        ("m_constrainedDofW", common.Vector4f),
        ("m_perpZeroErrorAxisBinW", common.Vector4f),
    ]
    m_zeroErrorAxisAinW: common.Vector4f
    m_constrainedDofW: common.Vector4f
    m_perpZeroErrorAxisBinW: common.Vector4f

class hknpDeformableFixedConstraintDataFunctions(ctypes.Structure):
    def getAppliedLinearImpulse(): pass

    def getAppliedImpulsesInSolverSpace(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpFixedConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknp1dAngularFrictionInfo(ctypes.Structure):
    _fields_ = [
        ("m_constrainedDofW", ctypes.c_ulonglong),
        ("m_lastSolverResults", ctypes.c_ulonglong),
        ("m_maxFrictionTorque", ctypes.c_float),
        ("m_numFriction", ctypes.c_int32),
    ]
    m_constrainedDofW: int
    m_lastSolverResults: int
    m_maxFrictionTorque: float
    m_numFriction: int

class hknp1dLinearFrictionInfo(ctypes.Structure):
    _fields_ = [
        ("m_pivotDiffComA", common.Vector4f),
        ("m_constrainedDofW", common.Vector4f),
        ("m_maxFrictionForce", ctypes.c_float),
        ("m_lastSolverResults", ctypes.c_ulonglong),
    ]
    m_pivotDiffComA: common.Vector4f
    m_constrainedDofW: common.Vector4f
    m_maxFrictionForce: float
    m_lastSolverResults: int

class hknpFixedConstraintDataFunctions(ctypes.Structure):
    def getRuntimeInfo(): pass

    def getAppliedImpulsesInSolverSpace(): pass

    def getInertiaStabilizationFactor(): pass

    def getMaximumAngularImpulse(): pass

    def getMaximumLinearImpulse(): pass

    def makeClone(): pass

    def setInertiaStabilizationFactor(): pass

    def setMaximumAngularImpulse(): pass

    def setMaximumLinearImpulse(): pass

    def getReflectType(): pass


class hknpLimitedHingeConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpLimitedHingeConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getConstraintInfo(): pass

    def getInertiaStabilizationFactor(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def setInertiaStabilizationFactor(): pass

    def getReflectType(): pass


class hknpLinearParametricCurve_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getPoint", ctypes.c_ulonglong),
        ("getNearestPoint", ctypes.c_ulonglong),
        ("getTangent", ctypes.c_ulonglong),
        ("getStart", ctypes.c_ulonglong),
        ("getEnd", ctypes.c_ulonglong),
        ("getLengthFromStart", ctypes.c_ulonglong),
        ("getBinormal", ctypes.c_ulonglong),
        ("isClosedLoop", ctypes.c_ulonglong),
        ("getPointsToDraw", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpParametricCurve *this, hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> > *)
        ("transformPoints", ctypes.c_ulonglong),
        ("clone", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getPoint: int
    getNearestPoint: int
    getTangent: int
    getStart: int
    getEnd: int
    getLengthFromStart: int
    getBinormal: int
    isClosedLoop: int
    getPointsToDraw: bytes
    transformPoints: int
    clone: int

class hknpRefWorldCinfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPhysicsSceneData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPointToPlaneConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpRotationalConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("callDestructors", ctypes.c_ulonglong),
        ("getImportsExports", ctypes.c_char * 0x8),  # void (__fastcall *)(hkResource *this, hkArray<hkResource::Import,hkBuiltinContainerAllocator<0,0> > *, hkArray<hkResource::Export,hkBuiltinContainerAllocator<0,0> > *)
        ("getContentsPointer", ctypes.c_ulonglong),
        ("getContentsPointer_38", ctypes.c_ulonglong),
        ("getContentsTypeName", ctypes.c_ulonglong),
        ("getContentsType", ctypes.c_ulonglong),
        ("getContentsVar", ctypes.c_ulonglong),
        ("cleanup", ctypes.c_ulonglong),
        ("getObjectName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getName: int
    callDestructors: int
    getImportsExports: bytes
    getContentsPointer: int
    getContentsPointer_38: int
    getContentsTypeName: int
    getContentsType: int
    getContentsVar: int
    cleanup: int
    getObjectName: int

class hkObjectResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("callDestructors", ctypes.c_ulonglong),
        ("getImportsExports", ctypes.c_char * 0x8),  # void (__fastcall *)(hkResource *this, hkArray<hkResource::Import,hkBuiltinContainerAllocator<0,0> > *, hkArray<hkResource::Export,hkBuiltinContainerAllocator<0,0> > *)
        ("getContentsPointer", ctypes.c_ulonglong),
        ("getContentsPointer_38", ctypes.c_ulonglong),
        ("getContentsTypeName", ctypes.c_ulonglong),
        ("getContentsType", ctypes.c_ulonglong),
        ("getContentsVar", ctypes.c_ulonglong),
        ("cleanup", ctypes.c_ulonglong),
        ("getObjectName", ctypes.c_ulonglong),
        ("stealContentsPointer", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getName: int
    callDestructors: int
    getImportsExports: bytes
    getContentsPointer: int
    getContentsPointer_38: int
    getContentsTypeName: int
    getContentsType: int
    getContentsVar: int
    cleanup: int
    getObjectName: int
    stealContentsPointer: int

class hknpPointToPlaneConstraintDataFunctions(ctypes.Structure):
    def isValid(): pass

    def getAppliedLinearImpulse(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpRotationalConstraintDataFunctions(ctypes.Structure):
    def getAppliedLinearImpulse(): pass

    def getRuntimeInfo(): pass

    def getAppliedImpulsesInSolverSpace(): pass

    def getConstraintInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpPrismaticConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPrismaticConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpPulleyConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpLimitedForceConstraintMotor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpSpringDamperConstraintMotor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPulleyConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getConstraintInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpForwardClosestHitCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class hknpMountedBallGun_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("initGun", ctypes.c_ulonglong),
        ("fireGun", ctypes.c_ulonglong),
        ("stepGun", ctypes.c_ulonglong),
        ("quitGun", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("createClone", ctypes.c_char * 0x8),  # hkRefNew<hknpFirstPersonGun> *(__fastcall *)(hknpFirstPersonGun *this, hkRefNew<hknpFirstPersonGun> *result, hknpWorld *)
        ("syncCloneFrom", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    initGun: int
    fireGun: int
    stepGun: int
    quitGun: int
    getName: int
    reset: int
    createClone: bytes
    syncCloneFrom: int

class hknpRagdollConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getConstraintInfo(): pass

    def getInertiaStabilizationFactor(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def setInertiaStabilizationFactor(): pass

    def getReflectType(): pass


class hknpStiffSpringConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpStiffSpringConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getRuntimeInfo(): pass

    def getAppliedLinearImpulse(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpWheelFrictionConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCompressedMeshShapeUtil(ctypes.Structure):
    def getClosestPointsToCompressedMesh(): pass

    def getClosestPointsToConvex(): pass

    def getClosestPointsToHeightfield(): pass


class hknpExternMeshShapeUtil(ctypes.Structure):
    def getClosestPointsToConvex(): pass


class hknpFlippedShapeCastQueryCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class hknpWheelFrictionConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getRuntimeInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpWheelConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknp1dAngularFollowCam_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("resetCamera", ctypes.c_ulonglong),
        ("calculateCamera", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    resetCamera: int
    calculateCamera: int

class hkcdGsk_Float3Shape(ctypes.Structure):
    def getSupportingVertex(): pass

    _fields_ = [
        ("m_vertices", ctypes.c_ulonglong),
        ("m_numVertices", ctypes.c_int32),
    ]
    m_vertices: int
    m_numVertices: int

class hkcdGsk_Float3ShapeInterface_vtbl(ctypes.Structure):
    _fields_ = [
        ("getSupportingVertices", ctypes.c_ulonglong),
        ("getShapeAabb", ctypes.c_ulonglong),
        ("getPlanes", ctypes.c_ulonglong),
    ]
    getSupportingVertices: int
    getShapeAabb: int
    getPlanes: int

class hknpWheelConstraintDataFunctions(ctypes.Structure):
    def getAppliedImpulsesInSolverSpace(): pass

    def getAppliedLinearImpulse(): pass

    def getConstraintInfo(): pass

    def isValid(): pass

    def makeClone(): pass

    def getReflectType(): pass


class hknpBreakableConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkcdGsk_Float3ShapeAOnlyInterface_vtbl(ctypes.Structure):
    _fields_ = [
        ("getSupportingVertices", ctypes.c_ulonglong),
        ("getShapeAabb", ctypes.c_ulonglong),
        ("getPlanes", ctypes.c_ulonglong),
    ]
    getSupportingVertices: int
    getShapeAabb: int
    getPlanes: int

class hknpBreakableConstraintDataFunctions(ctypes.Structure):
    def getReflectType(): pass

    def isValid(): pass

    def makeClone(): pass


class hknpConvexConvexShapeBaseInterface_vtbl(ctypes.Structure):
    _fields_ = [
        ("getSupportingVertices", ctypes.c_ulonglong),
        ("getShapeAabb", ctypes.c_ulonglong),
        ("getPlanes", ctypes.c_ulonglong),
    ]
    getSupportingVertices: int
    getShapeAabb: int
    getPlanes: int

class hknpMalleableConstraintData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCvxCollisionProcessHelper(ctypes.Structure):
    pass

class hknpMalleableConstraintDataFunctions(ctypes.Structure):
    def getReflectType(): pass

    def buildJacobian(): pass

    def getConstraintInfo(): pass

    def getRuntimeInfo(): pass

    def makeClone(): pass


class hknp1dBilateralConstraintStatus(ctypes.Structure):
    _fields_ = [
        ("m_effMass", ctypes.c_float),
    ]
    m_effMass: float

class hknp1dConstraintMotorInfo(ctypes.Structure):
    _fields_ = [
        ("m_targetPosition", ctypes.c_float),
        ("m_targetVelocity", ctypes.c_float),
        ("m_targetVelocityFactor", ctypes.c_float),
        ("m_maxForce", ctypes.c_float),
        ("m_minForce", ctypes.c_float),
        ("m_tau", ctypes.c_float),
        ("m_damping", ctypes.c_float),
    ]
    m_targetPosition: float
    m_targetVelocity: float
    m_targetVelocityFactor: float
    m_maxForce: float
    m_minForce: float
    m_tau: float
    m_damping: float

class hknpCallbackConstraintMotor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpGsk2DShapeInterface_vtbl(ctypes.Structure):
    _fields_ = [
        ("getSupportingVertices", ctypes.c_ulonglong),
        ("getShapeAabb", ctypes.c_ulonglong),
        ("getPlanes", ctypes.c_ulonglong),
    ]
    getSupportingVertices: int
    getShapeAabb: int
    getPlanes: int

class hknpPositionConstraintMotor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpVelocityConstraintMotor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCircularSurfaceVelocity_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcSurfaceVelocity", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcSurfaceVelocity: int

class hkProcessContextBase(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hkProcessContextBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkProcessContextBase_dtor_0", ctypes.c_ulonglong),
    ]
    hkProcessContextBase_dtor_0: int

class hkProcessClientSignals_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkProcess_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getProcessId", ctypes.c_ulonglong),
        ("getProcessOptions", ctypes.c_ulonglong),
        ("tweakProcessOptions", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkProcess *this, const hkHashMap<hkStringPtr,hkReflect::Var,hkBuiltinContainerAllocator<0,0> > *)
        ("getConsumableCommands", ctypes.c_ulonglong),
        ("canConsumeDelayedCommand", ctypes.c_ulonglong),
        ("consumeCommand", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("init", ctypes.c_ulonglong),
        ("step", ctypes.c_ulonglong),
        ("postStep", ctypes.c_ulonglong),
        ("onProcessOptionsChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(hkProcess *this, const hkHashMap<hkStringPtr,hkReflect::Var,hkBuiltinContainerAllocator<0,0> > *)
    ]
    hkProcess_dtor_0: int
    getType: int
    getProcessId: int
    getProcessOptions: int
    tweakProcessOptions: bytes
    getConsumableCommands: int
    canConsumeDelayedCommand: int
    consumeCommand: int
    isOk: int
    init: int
    step: int
    postStep: int
    onProcessOptionsChanged: bytes

class hkVdbCmdBlock(ctypes.Structure):
    _fields_ = [
        ("m_startingCmdIndex", ctypes.c_int32),
        ("m_storage", ctypes.c_char * 0x10),  # hkArray<signed char,hkBuiltinContainerAllocator<0,0> >
        ("m_cmdOffsets", ctypes.c_char * 0x10),  # hkArray<int,hkBuiltinContainerAllocator<0,0> >
    ]
    m_startingCmdIndex: int
    m_storage: bytes
    m_cmdOffsets: bytes

class hkVdbCmdStorage(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # hkArray<hkVdbCmdBlock,hkBuiltinContainerAllocator<0,0> >
    ]
    baseclass_0: bytes

class hkClientCommandRouter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkServerProcessHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkVisualDebugger_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("step", ctypes.c_ulonglong),
        ("pollForNewClients", ctypes.c_ulonglong),
        ("respondToServerDiscoveryBroadcasts", ctypes.c_ulonglong),
        ("deleteClient", ctypes.c_ulonglong),
        ("createClient", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    step: int
    pollForNewClients: int
    respondToServerDiscoveryBroadcasts: int
    deleteClient: int
    createClient: int

class hkProcessContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkProcessContextBase_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
    ]
    hkProcessContextBase_dtor_0: int
    getType: int

class hknp2AngJacobian(ctypes.Structure):
    _fields_ = [
        ("m_angular", common.Vector4f * 0x2),  # hkVector4f[2]
    ]
    m_angular: list[common.Vector4f]

class hknp1Lin2AngJacobian(ctypes.Structure):
    _fields_ = [
        ("m_linear0", common.Vector4f),
        ("m_angular", common.Vector4f * 0x2),  # hkVector4f[2]
    ]
    m_linear0: common.Vector4f
    m_angular: list[common.Vector4f]

class hknp2Lin2AngJacobian(ctypes.Structure):
    _fields_ = [
        ("m_linear", common.Vector4f * 0x2),  # hkVector4f[2]
        ("m_angular", common.Vector4f * 0x2),  # hkVector4f[2]
    ]
    m_linear: list[common.Vector4f]
    m_angular: list[common.Vector4f]

class hknpSolverElemTemp(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $30F3E15702CA4CECEF83C39DFF3AF080
    ]
    ___u0: bytes

class hknpVelocityAccumulator(ctypes.Structure):
    _fields_ = [
        ("m_linearVel", common.Vector4f),
        ("m_angularVel", common.Vector4f),
        ("m_invMasses", common.Vector4f),
        ("m_sumLinearVel", common.Vector4f),
        ("m_sumAngularVel", common.Vector4f),
    ]
    m_linearVel: common.Vector4f
    m_angularVel: common.Vector4f
    m_invMasses: common.Vector4f
    m_sumLinearVel: common.Vector4f
    m_sumAngularVel: common.Vector4f

class hknpPrimitiveDrawer(ctypes.Structure):
    _fields_ = [
        ("m_displayHandler", ctypes.c_ulonglong),
    ]
    m_displayHandler: int

class hkDisplayCone_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class hkDisplayPlane_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class hkVdbCmd(ctypes.Structure):
    _fields_ = [
        ("m_bodySize", ctypes.c_uint32),
        ("m_serverType", ctypes.c_uint8),
        ("m_data", ctypes.c_char * 0x1),  # char[1]
    ]
    m_bodySize: int
    m_serverType: int
    m_data: list[ctypes.c_char]

class hkCommonProcessContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _numberfmtW(ctypes.Structure):
    _fields_ = [
        ("NumDigits", ctypes.c_uint32),
        ("LeadingZero", ctypes.c_uint32),
        ("Grouping", ctypes.c_uint32),
        ("lpDecimalSep", ctypes.c_ulonglong),
        ("lpThousandSep", ctypes.c_ulonglong),
        ("NegativeOrder", ctypes.c_uint32),
    ]
    NumDigits: int
    LeadingZero: int
    Grouping: int
    lpDecimalSep: int
    lpThousandSep: int
    NegativeOrder: int

class _KNONVOLATILE_CONTEXT_POINTERS_ARM64(ctypes.Structure):
    _fields_ = [
        ("X19", ctypes.c_ulonglong),
        ("X20", ctypes.c_ulonglong),
        ("X21", ctypes.c_ulonglong),
        ("X22", ctypes.c_ulonglong),
        ("X23", ctypes.c_ulonglong),
        ("X24", ctypes.c_ulonglong),
        ("X25", ctypes.c_ulonglong),
        ("X26", ctypes.c_ulonglong),
        ("X27", ctypes.c_ulonglong),
        ("X28", ctypes.c_ulonglong),
        ("Fp", ctypes.c_ulonglong),
        ("Lr", ctypes.c_ulonglong),
        ("D8", ctypes.c_ulonglong),
        ("D9", ctypes.c_ulonglong),
        ("D10", ctypes.c_ulonglong),
        ("D11", ctypes.c_ulonglong),
        ("D12", ctypes.c_ulonglong),
        ("D13", ctypes.c_ulonglong),
        ("D14", ctypes.c_ulonglong),
        ("D15", ctypes.c_ulonglong),
    ]
    X19: int
    X20: int
    X21: int
    X22: int
    X23: int
    X24: int
    X25: int
    X26: int
    X27: int
    X28: int
    Fp: int
    Lr: int
    D8: int
    D9: int
    D10: int
    D11: int
    D12: int
    D13: int
    D14: int
    D15: int

class cTkNGuiFilter(ctypes.Structure):
    def IsFiltered(): pass

    _fields_ = [
        ("mpacLabel", ctypes.c_char_p),
    ]
    mpacLabel: bytes

class _SERVICE_TABLE_ENTRYA(ctypes.Structure):
    _fields_ = [
        ("lpServiceName", ctypes.c_char_p),
        ("lpServiceProc", ctypes.c_ulonglong),
    ]
    lpServiceName: bytes
    lpServiceProc: int

class hkDebugDisplayHandlerChecker_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("setDisplayOptions", ctypes.c_ulonglong),
        ("addGeometry", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, hkVdbCmdTag)
        ("addGeometryEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, const unsigned int, hkDisplayGeometryFlags, hkVdbCmdTag)
        ("addGeometryInstance", ctypes.c_ulonglong),
        ("addGeometryInstanceEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, unsigned __int64, const hkTransformf *, const unsigned int, const struct hkArrayBase<hkGeometryDecorations *> *, hkVdbCmdTag)
        ("setGeometryFlagBits", ctypes.c_ulonglong),
        ("clearGeometryFlagBits", ctypes.c_ulonglong),
        ("setGeometryColor", ctypes.c_ulonglong),
        ("setGeometryAlpha", ctypes.c_ulonglong),
        ("updateGeometryTransform", ctypes.c_ulonglong),
        ("updateGeometryTransformEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkMatrix4Impl<float> *)
        ("updateParticleTransforms", ctypes.c_ulonglong),
        ("updateGeometryVerts", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArray<hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> >,hkBuiltinContainerAllocator<0,0> > *)
        ("updateGeometryMaterialData", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const struct hkArray<hkDisplayMaterialDatas *,hkBuiltinContainerAllocator<0,0> > *)
        ("removeGeometry", ctypes.c_ulonglong),
        ("updateCamera", ctypes.c_ulonglong),
        ("removeCamera", ctypes.c_ulonglong),
        ("displayPoint", ctypes.c_ulonglong),
        ("displayPointEx", ctypes.c_ulonglong),
        ("display2Points", ctypes.c_ulonglong),
        ("display2PointsEx", ctypes.c_ulonglong),
        ("display3Points", ctypes.c_ulonglong),
        ("display3PointsEx", ctypes.c_ulonglong),
        ("displayGeometry", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, unsigned int, hkVdbCmdTag)
        ("display3dText", ctypes.c_ulonglong),
        ("flushDisplay", ctypes.c_ulonglong),
        ("displayText", ctypes.c_ulonglong),
        ("displayPoint2d", ctypes.c_ulonglong),
        ("displayLine2d", ctypes.c_ulonglong),
        ("displayTriangle2d", ctypes.c_ulonglong),
        ("displayText2d", ctypes.c_ulonglong),
        ("getDeprecatedInterface", ctypes.c_ulonglong),
        ("displayLitTriangle", ctypes.c_ulonglong),
        ("displayFrame", ctypes.c_ulonglong),
        ("displayPlane", ctypes.c_ulonglong),
        ("displayModelSpacePose", ctypes.c_ulonglong),
        ("displayAabb", ctypes.c_ulonglong),
        ("displayTransformedAabb", ctypes.c_ulonglong),
        ("displayLines", ctypes.c_ulonglong),
        ("lockForUpdate", ctypes.c_ulonglong),
        ("unlockForUpdate", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    setDisplayOptions: int
    addGeometry: bytes
    addGeometryEx: bytes
    addGeometryInstance: int
    addGeometryInstanceEx: bytes
    setGeometryFlagBits: int
    clearGeometryFlagBits: int
    setGeometryColor: int
    setGeometryAlpha: int
    updateGeometryTransform: int
    updateGeometryTransformEx: bytes
    updateParticleTransforms: int
    updateGeometryVerts: bytes
    updateGeometryMaterialData: bytes
    removeGeometry: int
    updateCamera: int
    removeCamera: int
    displayPoint: int
    displayPointEx: int
    display2Points: int
    display2PointsEx: int
    display3Points: int
    display3PointsEx: int
    displayGeometry: bytes
    display3dText: int
    flushDisplay: int
    displayText: int
    displayPoint2d: int
    displayLine2d: int
    displayTriangle2d: int
    displayText2d: int
    getDeprecatedInterface: int
    displayLitTriangle: int
    displayFrame: int
    displayPlane: int
    displayModelSpacePose: int
    displayAabb: int
    displayTransformedAabb: int
    displayLines: int
    lockForUpdate: int
    unlockForUpdate: int

class _ENCLAVE_CREATE_INFO_VBS_BASIC(ctypes.Structure):
    _fields_ = [
        ("Flags", ctypes.c_uint32),
        ("OwnerID", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
    ]
    Flags: int
    OwnerID: list[ctypes.c_uint8]

class _JOBOBJECT_BASIC_PROCESS_ID_LIST(ctypes.Structure):
    _fields_ = [
        ("NumberOfAssignedProcesses", ctypes.c_uint32),
        ("NumberOfProcessIdsInList", ctypes.c_uint32),
        ("ProcessIdList", ctypes.c_uint64 * 0x1),  # unsigned __int64[1]
    ]
    NumberOfAssignedProcesses: int
    NumberOfProcessIdsInList: int
    ProcessIdList: list[ctypes.c_uint64]

class tagHANDLETABLE(ctypes.Structure):
    _fields_ = [
        ("objectHandle", ctypes.c_void_p * 0x1),  # void *[1]
    ]
    objectHandle: list[ctypes.c_void_p]

class _UNIVERSAL_NAME_INFOA(ctypes.Structure):
    _fields_ = [
        ("lpUniversalName", ctypes.c_char_p),
    ]
    lpUniversalName: bytes

class _RTL_BARRIER(ctypes.Structure):
    _fields_ = [
        ("Reserved1", ctypes.c_uint32),
        ("Reserved2", ctypes.c_uint32),
        ("Reserved3", ctypes.c_uint64 * 0x2),  # unsigned __int64[2]
        ("Reserved4", ctypes.c_uint32),
        ("Reserved5", ctypes.c_uint32),
    ]
    Reserved1: int
    Reserved2: int
    Reserved3: list[ctypes.c_uint64]
    Reserved4: int
    Reserved5: int

class _FILE_ATTRIBUTE_TAG_INFO(ctypes.Structure):
    _fields_ = [
        ("FileAttributes", ctypes.c_uint32),
        ("ReparseTag", ctypes.c_uint32),
    ]
    FileAttributes: int
    ReparseTag: int

class _FPO_DATA(ctypes.Structure):
    _fields_ = [
        ("ulOffStart", ctypes.c_uint32),
        ("cbProcSize", ctypes.c_uint32),
        ("cdwLocals", ctypes.c_uint32),
        ("cdwParams", ctypes.c_uint16),
        ("_bf_e", ctypes.c_int16),
    ]
    ulOffStart: int
    cbProcSize: int
    cdwLocals: int
    cdwParams: int
    _bf_e: int

class _SERVICE_FAILURE_ACTIONSW(ctypes.Structure):
    _fields_ = [
        ("dwResetPeriod", ctypes.c_uint32),
        ("lpRebootMsg", ctypes.c_ulonglong),
        ("lpCommand", ctypes.c_ulonglong),
        ("cActions", ctypes.c_uint32),
        ("lpsaActions", ctypes.c_ulonglong),
    ]
    dwResetPeriod: int
    lpRebootMsg: int
    lpCommand: int
    cActions: int
    lpsaActions: int

class _JOBOBJECT_END_OF_JOB_TIME_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("EndOfJobTimeAction", ctypes.c_uint32),
    ]
    EndOfJobTimeAction: int

class _TOKEN_ELEVATION(ctypes.Structure):
    _fields_ = [
        ("TokenIsElevated", ctypes.c_uint32),
    ]
    TokenIsElevated: int

class _ImageArchitectureEntry(ctypes.Structure):
    _fields_ = [
        ("FixupInstRVA", ctypes.c_uint32),
        ("NewInst", ctypes.c_uint32),
    ]
    FixupInstRVA: int
    NewInst: int

class _JOBOBJECT_BASIC_UI_RESTRICTIONS(ctypes.Structure):
    _fields_ = [
        ("UIRestrictionsClass", ctypes.c_uint32),
    ]
    UIRestrictionsClass: int

class _POWER_SESSION_ALLOW_EXTERNAL_DMA_DEVICES(ctypes.Structure):
    _fields_ = [
        ("IsAllowed", ctypes.c_uint8),
    ]
    IsAllowed: int

class _IMAGE_DYNAMIC_RELOCATION32_V2(ctypes.Structure):
    _fields_ = [
        ("HeaderSize", ctypes.c_uint32),
        ("FixupInfoSize", ctypes.c_uint32),
        ("Symbol", ctypes.c_uint32),
        ("SymbolGroup", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
    ]
    HeaderSize: int
    FixupInfoSize: int
    Symbol: int
    SymbolGroup: int
    Flags: int

class cTkHashTableHashPassthrough(ctypes.Structure):
    pass

class HSYNTHETICPOINTERDEVICE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class __noop_coroutine_frame_standard(ctypes.Structure):
    _fields_ = [
        ("resume", ctypes.c_void_p),
        ("destroy", ctypes.c_void_p),
        ("promise", ctypes.c_char),
    ]
    resume: int
    destroy: int
    promise: str

class tagSOUNDSENTRYW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("iFSTextEffect", ctypes.c_uint32),
        ("iFSTextEffectMSec", ctypes.c_uint32),
        ("iFSTextEffectColorBits", ctypes.c_uint32),
        ("iFSGrafEffect", ctypes.c_uint32),
        ("iFSGrafEffectMSec", ctypes.c_uint32),
        ("iFSGrafEffectColor", ctypes.c_uint32),
        ("iWindowsEffect", ctypes.c_uint32),
        ("iWindowsEffectMSec", ctypes.c_uint32),
        ("lpszWindowsEffectDLL", ctypes.c_ulonglong),
        ("iWindowsEffectOrdinal", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int
    iFSTextEffect: int
    iFSTextEffectMSec: int
    iFSTextEffectColorBits: int
    iFSGrafEffect: int
    iFSGrafEffectMSec: int
    iFSGrafEffectColor: int
    iWindowsEffect: int
    iWindowsEffectMSec: int
    lpszWindowsEffectDLL: int
    iWindowsEffectOrdinal: int

class tagHIGHCONTRASTA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("lpszDefaultScheme", ctypes.c_char_p),
    ]
    cbSize: int
    dwFlags: int
    lpszDefaultScheme: bytes

class DISPLAYCONFIG_TARGET_DEVICE_NAME_FLAGS(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $DB0C315375DDCC95BF562CDF286F1DE5
    ]
    ___u0: bytes

class _PROCESS_DYNAMIC_ENFORCED_ADDRESS_RANGES_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("NumberOfRanges", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16),
        ("Reserved2", ctypes.c_uint32),
        ("Ranges", ctypes.c_ulonglong),
    ]
    NumberOfRanges: int
    Reserved: int
    Reserved2: int
    Ranges: int

class PPM_THERMALCHANGE_EVENT(ctypes.Structure):
    _fields_ = [
        ("ThermalConstraint", ctypes.c_uint32),
        ("Processors", ctypes.c_uint64),
    ]
    ThermalConstraint: int
    Processors: int

class _ULARGE_INTEGER: pass

class tagCWPSTRUCT(ctypes.Structure):
    _fields_ = [
        ("lParam", ctypes.c_int64),
        ("wParam", ctypes.c_uint64),
        ("message", ctypes.c_uint32),
        ("hwnd", ctypes.c_ulonglong),
    ]
    lParam: int
    wParam: int
    message: int
    hwnd: int

class HIMCC__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class tagMDICREATESTRUCTW(ctypes.Structure):
    _fields_ = [
        ("szClass", ctypes.c_ulonglong),
        ("szTitle", ctypes.c_ulonglong),
        ("hOwner", ctypes.c_void_p),
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("_cx", ctypes.c_int32),
        ("cy", ctypes.c_int32),
        ("style", ctypes.c_uint32),
        ("lParam", ctypes.c_int64),
    ]
    szClass: int
    szTitle: int
    hOwner: int
    x: int
    y: int
    _cx: int
    cy: int
    style: int
    lParam: int

class MEM_EXTENDED_PARAMETER(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int64),
        ("___u2", ctypes.c_char * 0x8),  # $2787D341B6EFE2189A69102D2C49BA63
    ]
    _bf_0: int
    ___u2: bytes

class _UMS_CREATE_THREAD_ATTRIBUTES(ctypes.Structure):
    _fields_ = [
        ("UmsVersion", ctypes.c_uint32),
        ("UmsContext", ctypes.c_void_p),
        ("UmsCompletionList", ctypes.c_void_p),
    ]
    UmsVersion: int
    UmsContext: int
    UmsCompletionList: int

class hkDebugDisplayProcess_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkProcess_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getProcessId", ctypes.c_ulonglong),
        ("getProcessOptions", ctypes.c_ulonglong),
        ("tweakProcessOptions", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkProcess *this, const hkHashMap<hkStringPtr,hkReflect::Var,hkBuiltinContainerAllocator<0,0> > *)
        ("getConsumableCommands", ctypes.c_ulonglong),
        ("canConsumeDelayedCommand", ctypes.c_ulonglong),
        ("consumeCommand", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("init", ctypes.c_ulonglong),
        ("step", ctypes.c_ulonglong),
        ("postStep", ctypes.c_ulonglong),
        ("onProcessOptionsChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(hkProcess *this, const hkHashMap<hkStringPtr,hkReflect::Var,hkBuiltinContainerAllocator<0,0> > *)
    ]
    hkProcess_dtor_0: int
    getType: int
    getProcessId: int
    getProcessOptions: int
    tweakProcessOptions: bytes
    getConsumableCommands: int
    canConsumeDelayedCommand: int
    consumeCommand: int
    isOk: int
    init: int
    step: int
    postStep: int
    onProcessOptionsChanged: bytes

class debugRenderNowCallbacks(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_tweakee", ctypes.c_void_p),
        ("m_tweakeeClass", ctypes.c_ulonglong),
        ("m_hideMembers", ctypes.c_ulonglong),
        ("m_doRenderCallback", ctypes.c_ubyte),
    ]
    __vftable: int
    m_tweakee: int
    m_tweakeeClass: int
    m_hideMembers: int
    m_doRenderCallback: bool

class debugRenderNow_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int

class debugRenderNowCallbacks_vtbl(ctypes.Structure):
    _fields_ = [
        ("debugRenderNowCallbacks_dtor_0", ctypes.c_ulonglong),
        ("handleKey", ctypes.c_ulonglong),
        ("render", ctypes.c_ulonglong),
    ]
    debugRenderNowCallbacks_dtor_0: int
    handleKey: int
    render: int

class _EXIT_PROCESS_DEBUG_INFO(ctypes.Structure):
    _fields_ = [
        ("dwExitCode", ctypes.c_uint32),
    ]
    dwExitCode: int

class cTkNGuiCallbackData(ctypes.Structure):
    _fields_ = [
        ("mpFunc", ctypes.c_ulonglong),
        ("mpData", ctypes.c_void_p),
    ]
    mpFunc: int
    mpData: int

class hkDebugDisplay_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("setDisplayOptions", ctypes.c_ulonglong),
        ("addGeometry", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, hkVdbCmdTag)
        ("addGeometryEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, const unsigned int, hkDisplayGeometryFlags, hkVdbCmdTag)
        ("addGeometryInstance", ctypes.c_ulonglong),
        ("addGeometryInstanceEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, unsigned __int64, const hkTransformf *, const unsigned int, const struct hkArrayBase<hkGeometryDecorations *> *, hkVdbCmdTag)
        ("setGeometryFlagBits", ctypes.c_ulonglong),
        ("clearGeometryFlagBits", ctypes.c_ulonglong),
        ("setGeometryColor", ctypes.c_ulonglong),
        ("setGeometryAlpha", ctypes.c_ulonglong),
        ("updateGeometryTransform", ctypes.c_ulonglong),
        ("updateGeometryTransformEx", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkMatrix4Impl<float> *)
        ("updateParticleTransforms", ctypes.c_ulonglong),
        ("updateGeometryVerts", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArray<hkArray<hkVector4f,hkBuiltinContainerAllocator<0,0> >,hkBuiltinContainerAllocator<0,0> > *)
        ("updateGeometryMaterialData", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const struct hkArray<hkDisplayMaterialDatas *,hkBuiltinContainerAllocator<0,0> > *)
        ("removeGeometry", ctypes.c_ulonglong),
        ("updateCamera", ctypes.c_ulonglong),
        ("removeCamera", ctypes.c_ulonglong),
        ("displayPoint", ctypes.c_ulonglong),
        ("displayPointEx", ctypes.c_ulonglong),
        ("display2Points", ctypes.c_ulonglong),
        ("display2PointsEx", ctypes.c_ulonglong),
        ("display3Points", ctypes.c_ulonglong),
        ("display3PointsEx", ctypes.c_ulonglong),
        ("displayGeometry", ctypes.c_char * 0x8),  # hkResult (__fastcall *)(hkDebugDisplayHandler *this, unsigned __int64, const hkArrayBase<hkDisplayGeometry *> *, const hkTransformf *, unsigned int, hkVdbCmdTag)
        ("display3dText", ctypes.c_ulonglong),
        ("flushDisplay", ctypes.c_ulonglong),
        ("displayText", ctypes.c_ulonglong),
        ("displayPoint2d", ctypes.c_ulonglong),
        ("displayLine2d", ctypes.c_ulonglong),
        ("displayTriangle2d", ctypes.c_ulonglong),
        ("displayText2d", ctypes.c_ulonglong),
        ("getDeprecatedInterface", ctypes.c_ulonglong),
        ("displayLitTriangle", ctypes.c_ulonglong),
        ("displayFrame", ctypes.c_ulonglong),
        ("displayPlane", ctypes.c_ulonglong),
        ("displayModelSpacePose", ctypes.c_ulonglong),
        ("displayAabb", ctypes.c_ulonglong),
        ("displayTransformedAabb", ctypes.c_ulonglong),
        ("displayLines", ctypes.c_ulonglong),
        ("lockForUpdate", ctypes.c_ulonglong),
        ("unlockForUpdate", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    setDisplayOptions: int
    addGeometry: bytes
    addGeometryEx: bytes
    addGeometryInstance: int
    addGeometryInstanceEx: bytes
    setGeometryFlagBits: int
    clearGeometryFlagBits: int
    setGeometryColor: int
    setGeometryAlpha: int
    updateGeometryTransform: int
    updateGeometryTransformEx: bytes
    updateParticleTransforms: int
    updateGeometryVerts: bytes
    updateGeometryMaterialData: bytes
    removeGeometry: int
    updateCamera: int
    removeCamera: int
    displayPoint: int
    displayPointEx: int
    display2Points: int
    display2PointsEx: int
    display3Points: int
    display3PointsEx: int
    displayGeometry: bytes
    display3dText: int
    flushDisplay: int
    displayText: int
    displayPoint2d: int
    displayLine2d: int
    displayTriangle2d: int
    displayText2d: int
    getDeprecatedInterface: int
    displayLitTriangle: int
    displayFrame: int
    displayPlane: int
    displayModelSpacePose: int
    displayAabb: int
    displayTransformedAabb: int
    displayLines: int
    lockForUpdate: int
    unlockForUpdate: int

class tagWTSSESSION_NOTIFICATION(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwSessionId", ctypes.c_uint32),
    ]
    cbSize: int
    dwSessionId: int

class _FILE_CASE_SENSITIVE_INFO(ctypes.Structure):
    _fields_ = [
        ("Flags", ctypes.c_uint32),
    ]
    Flags: int

class tagMSGBOXPARAMSA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("hwndOwner", ctypes.c_ulonglong),
        ("hInstance", ctypes.c_ulonglong),
        ("lpszText", ctypes.c_char_p),
        ("lpszCaption", ctypes.c_char_p),
        ("dwStyle", ctypes.c_uint32),
        ("lpszIcon", ctypes.c_char_p),
        ("dwContextHelpId", ctypes.c_uint64),
        ("lpfnMsgBoxCallback", ctypes.c_ulonglong),
        ("dwLanguageId", ctypes.c_uint32),
    ]
    cbSize: int
    hwndOwner: int
    hInstance: int
    lpszText: bytes
    lpszCaption: bytes
    dwStyle: int
    lpszIcon: bytes
    dwContextHelpId: int
    lpfnMsgBoxCallback: int
    dwLanguageId: int

class _STARTUPINFOW(ctypes.Structure):
    _fields_ = [
        ("cb", ctypes.c_uint32),
        ("lpReserved", ctypes.c_ulonglong),
        ("lpDesktop", ctypes.c_ulonglong),
        ("lpTitle", ctypes.c_ulonglong),
        ("dwX", ctypes.c_uint32),
        ("dwY", ctypes.c_uint32),
        ("dwXSize", ctypes.c_uint32),
        ("dwYSize", ctypes.c_uint32),
        ("dwXCountChars", ctypes.c_uint32),
        ("dwYCountChars", ctypes.c_uint32),
        ("dwFillAttribute", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("wShowWindow", ctypes.c_uint16),
        ("cbReserved2", ctypes.c_uint16),
        ("lpReserved2", ctypes.c_ulonglong),
        ("hStdInput", ctypes.c_void_p),
        ("hStdOutput", ctypes.c_void_p),
        ("hStdError", ctypes.c_void_p),
    ]
    cb: int
    lpReserved: int
    lpDesktop: int
    lpTitle: int
    dwX: int
    dwY: int
    dwXSize: int
    dwYSize: int
    dwXCountChars: int
    dwYCountChars: int
    dwFillAttribute: int
    dwFlags: int
    wShowWindow: int
    cbReserved2: int
    lpReserved2: int
    hStdInput: int
    hStdOutput: int
    hStdError: int

class _SE_SECURITY_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("SecurityDescriptor", ctypes.c_void_p),
    ]
    Size: int
    Flags: int
    SecurityDescriptor: int

class _MODEMDEVCAPS(ctypes.Structure):
    _fields_ = [
        ("dwActualSize", ctypes.c_uint32),
        ("dwRequiredSize", ctypes.c_uint32),
        ("dwDevSpecificOffset", ctypes.c_uint32),
        ("dwDevSpecificSize", ctypes.c_uint32),
        ("dwModemProviderVersion", ctypes.c_uint32),
        ("dwModemManufacturerOffset", ctypes.c_uint32),
        ("dwModemManufacturerSize", ctypes.c_uint32),
        ("dwModemModelOffset", ctypes.c_uint32),
        ("dwModemModelSize", ctypes.c_uint32),
        ("dwModemVersionOffset", ctypes.c_uint32),
        ("dwModemVersionSize", ctypes.c_uint32),
        ("dwDialOptions", ctypes.c_uint32),
        ("dwCallSetupFailTimer", ctypes.c_uint32),
        ("dwInactivityTimeout", ctypes.c_uint32),
        ("dwSpeakerVolume", ctypes.c_uint32),
        ("dwSpeakerMode", ctypes.c_uint32),
        ("dwModemOptions", ctypes.c_uint32),
        ("dwMaxDTERate", ctypes.c_uint32),
        ("dwMaxDCERate", ctypes.c_uint32),
        ("abVariablePortion", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
    ]
    dwActualSize: int
    dwRequiredSize: int
    dwDevSpecificOffset: int
    dwDevSpecificSize: int
    dwModemProviderVersion: int
    dwModemManufacturerOffset: int
    dwModemManufacturerSize: int
    dwModemModelOffset: int
    dwModemModelSize: int
    dwModemVersionOffset: int
    dwModemVersionSize: int
    dwDialOptions: int
    dwCallSetupFailTimer: int
    dwInactivityTimeout: int
    dwSpeakerVolume: int
    dwSpeakerMode: int
    dwModemOptions: int
    dwMaxDTERate: int
    dwMaxDCERate: int
    abVariablePortion: list[ctypes.c_uint8]

class tagNMHDR(ctypes.Structure):
    _fields_ = [
        ("hwndFrom", ctypes.c_ulonglong),
        ("idFrom", ctypes.c_uint64),
        ("code", ctypes.c_uint32),
    ]
    hwndFrom: int
    idFrom: int
    code: int

class tagEMRFORMAT(ctypes.Structure):
    _fields_ = [
        ("dSignature", ctypes.c_uint32),
        ("nVersion", ctypes.c_uint32),
        ("cbData", ctypes.c_uint32),
        ("offData", ctypes.c_uint32),
    ]
    dSignature: int
    nVersion: int
    cbData: int
    offData: int

class _FLOAT128(ctypes.Structure):
    _fields_ = [
        ("LowPart", ctypes.c_int64),
        ("HighPart", ctypes.c_int64),
    ]
    LowPart: int
    HighPart: int

class SERVICE_TRIGGER_CUSTOM_STATE_ID(ctypes.Structure):
    _fields_ = [
        ("Data", ctypes.c_uint32 * 0x2),  # unsigned int[2]
    ]
    Data: list[ctypes.c_uint32]

class _SERVICE_CUSTOM_SYSTEM_STATE_CHANGE_DATA_ITEM(ctypes.Structure):
    _fields_ = [
        ("u", ctypes.c_char * 0x8),  # _SERVICE_CUSTOM_SYSTEM_STATE_CHANGE_DATA_ITEM::<unnamed_type_u>
    ]
    u: bytes

class _UNLOAD_DLL_DEBUG_INFO(ctypes.Structure):
    _fields_ = [
        ("lpBaseOfDll", ctypes.c_void_p),
    ]
    lpBaseOfDll: int

class _OUTPUT_DEBUG_STRING_INFO(ctypes.Structure):
    _fields_ = [
        ("lpDebugStringData", ctypes.c_char_p),
        ("fUnicode", ctypes.c_uint16),
        ("nDebugStringLength", ctypes.c_uint16),
    ]
    lpDebugStringData: bytes
    fUnicode: int
    nDebugStringLength: int

class _DEBUG_EVENT(ctypes.Structure):
    _fields_ = [
        ("dwDebugEventCode", ctypes.c_uint32),
        ("dwProcessId", ctypes.c_uint32),
        ("dwThreadId", ctypes.c_uint32),
        ("u", ctypes.c_char * 0xA0),  # _DEBUG_EVENT::<unnamed_type_u>
    ]
    dwDebugEventCode: int
    dwProcessId: int
    dwThreadId: int
    u: bytes

class _CREATEFILE2_EXTENDED_PARAMETERS(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwFileAttributes", ctypes.c_uint32),
        ("dwFileFlags", ctypes.c_uint32),
        ("dwSecurityQosFlags", ctypes.c_uint32),
        ("lpSecurityAttributes", ctypes.c_ulonglong),
        ("hTemplateFile", ctypes.c_void_p),
    ]
    dwSize: int
    dwFileAttributes: int
    dwFileFlags: int
    dwSecurityQosFlags: int
    lpSecurityAttributes: int
    hTemplateFile: int

class HCOLORSPACE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _REMOTE_NAME_INFOA(ctypes.Structure):
    _fields_ = [
        ("lpUniversalName", ctypes.c_char_p),
        ("lpConnectionName", ctypes.c_char_p),
        ("lpRemainingPath", ctypes.c_char_p),
    ]
    lpUniversalName: bytes
    lpConnectionName: bytes
    lpRemainingPath: bytes

class WIN32_MEMORY_REGION_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("AllocationBase", ctypes.c_void_p),
        ("AllocationProtect", ctypes.c_uint32),
        ("___u2", ctypes.c_char * 0x4),  # $73C91986D6268D8729E505C0F5E71B02
        ("RegionSize", ctypes.c_uint64),
        ("CommitSize", ctypes.c_uint64),
    ]
    AllocationBase: int
    AllocationProtect: int
    ___u2: bytes
    RegionSize: int
    CommitSize: int

class hkDisplaySemiCircle_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class hknpAllHitsCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class _CONSOLE_HISTORY_INFO(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("HistoryBufferSize", ctypes.c_uint32),
        ("NumberOfHistoryBuffers", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
    ]
    cbSize: int
    HistoryBufferSize: int
    NumberOfHistoryBuffers: int
    dwFlags: int

class _SERVICE_DESCRIPTIONW(ctypes.Structure):
    _fields_ = [
        ("lpDescription", ctypes.c_ulonglong),
    ]
    lpDescription: int

class _SECURITY_DESCRIPTOR_RELATIVE(ctypes.Structure):
    _fields_ = [
        ("Revision", ctypes.c_uint8),
        ("Sbz1", ctypes.c_uint8),
        ("Control", ctypes.c_uint16),
        ("Owner", ctypes.c_uint32),
        ("Group", ctypes.c_uint32),
        ("Sacl", ctypes.c_uint32),
        ("Dacl", ctypes.c_uint32),
    ]
    Revision: int
    Sbz1: int
    Control: int
    Owner: int
    Group: int
    Sacl: int
    Dacl: int

class _IMAGE_RESOURCE_DIRECTORY(ctypes.Structure):
    _fields_ = [
        ("Characteristics", ctypes.c_uint32),
        ("TimeDateStamp", ctypes.c_uint32),
        ("MajorVersion", ctypes.c_uint16),
        ("MinorVersion", ctypes.c_uint16),
        ("NumberOfNamedEntries", ctypes.c_uint16),
        ("NumberOfIdEntries", ctypes.c_uint16),
    ]
    Characteristics: int
    TimeDateStamp: int
    MajorVersion: int
    MinorVersion: int
    NumberOfNamedEntries: int
    NumberOfIdEntries: int

class _RTL_SRWLOCK(ctypes.Structure):
    _fields_ = [
        ("Ptr", ctypes.c_void_p),
    ]
    Ptr: int

class _FILE_DISPOSITION_INFO(ctypes.Structure):
    _fields_ = [
        ("DeleteFileW", ctypes.c_uint8),
    ]
    DeleteFileW: int

class hknpCharacterProxyCinfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class tagCWPRETSTRUCT(ctypes.Structure):
    _fields_ = [
        ("lResult", ctypes.c_int64),
        ("lParam", ctypes.c_int64),
        ("wParam", ctypes.c_uint64),
        ("message", ctypes.c_uint32),
        ("hwnd", ctypes.c_ulonglong),
    ]
    lResult: int
    lParam: int
    wParam: int
    message: int
    hwnd: int

class _WGLSWAP(ctypes.Structure):
    _fields_ = [
        ("hdc", ctypes.c_ulonglong),
        ("uiFlags", ctypes.c_uint32),
    ]
    hdc: int
    uiFlags: int

class HMONITOR__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _NT_TIB64(ctypes.Structure):
    _fields_ = [
        ("ExceptionList", ctypes.c_uint64),
        ("StackBase", ctypes.c_uint64),
        ("StackLimit", ctypes.c_uint64),
        ("SubSystemTib", ctypes.c_uint64),
        ("___u4", ctypes.c_char * 0x8),  # $8662F439D215AAEDBB1F787C8B649648
        ("ArbitraryUserPointer", ctypes.c_uint64),
        ("Self", ctypes.c_uint64),
    ]
    ExceptionList: int
    StackBase: int
    StackLimit: int
    SubSystemTib: int
    ___u4: bytes
    ArbitraryUserPointer: int
    Self: int

class tagKERNINGPAIR(ctypes.Structure):
    _fields_ = [
        ("wFirst", ctypes.c_uint16),
        ("wSecond", ctypes.c_uint16),
        ("iKernAmount", ctypes.c_int32),
    ]
    wFirst: int
    wSecond: int
    iKernAmount: int

class tagDELETEITEMSTRUCT(ctypes.Structure):
    _fields_ = [
        ("CtlType", ctypes.c_uint32),
        ("CtlID", ctypes.c_uint32),
        ("itemID", ctypes.c_uint32),
        ("hwndItem", ctypes.c_ulonglong),
        ("itemData", ctypes.c_uint64),
    ]
    CtlType: int
    CtlID: int
    itemID: int
    hwndItem: int
    itemData: int

class tagSTICKYKEYS(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int

class _IMAGE_HOT_PATCH_INFO(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("Size", ctypes.c_uint32),
        ("SequenceNumber", ctypes.c_uint32),
        ("BaseImageList", ctypes.c_uint32),
        ("BaseImageCount", ctypes.c_uint32),
        ("BufferOffset", ctypes.c_uint32),
        ("ExtraPatchSize", ctypes.c_uint32),
    ]
    Version: int
    Size: int
    SequenceNumber: int
    BaseImageList: int
    BaseImageCount: int
    BufferOffset: int
    ExtraPatchSize: int

class _PROCESS_MITIGATION_DYNAMIC_CODE_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $FC5601C0E5408E15309CDF45090ACDFA
    ]
    ___u0: bytes

class tagREGISTERWORDW(ctypes.Structure):
    _fields_ = [
        ("lpReading", ctypes.c_ulonglong),
        ("lpWord", ctypes.c_ulonglong),
    ]
    lpReading: int
    lpWord: int

class _TRANSACTION_NOTIFICATION_SAVEPOINT_ARGUMENT(ctypes.Structure):
    _fields_ = [
        ("SavepointId", ctypes.c_uint32),
    ]
    SavepointId: int

class cTkCSMutexScoped(ctypes.Structure):
    _fields_ = [
        ("mMutex", ctypes.c_ulonglong),
    ]
    mMutex: int

class _Lldiv_t(ctypes.Structure):
    _fields_ = [
        ("quot", ctypes.c_int64),
        ("rem", ctypes.c_int64),
    ]
    quot: int
    rem: int

class PROCESSOR_PERFSTATE_POLICY(ctypes.Structure):
    _fields_ = [
        ("Revision", ctypes.c_uint32),
        ("MaxThrottle", ctypes.c_uint8),
        ("MinThrottle", ctypes.c_uint8),
        ("BusyAdjThreshold", ctypes.c_uint8),
        ("___u4", ctypes.c_char * 0x1),  # $26CBF908C62B871C844EDFA6DDCAE77A
        ("TimeCheck", ctypes.c_uint32),
        ("IncreaseTime", ctypes.c_uint32),
        ("DecreaseTime", ctypes.c_uint32),
        ("IncreasePercent", ctypes.c_uint32),
        ("DecreasePercent", ctypes.c_uint32),
    ]
    Revision: int
    MaxThrottle: int
    MinThrottle: int
    BusyAdjThreshold: int
    ___u4: bytes
    TimeCheck: int
    IncreaseTime: int
    DecreaseTime: int
    IncreasePercent: int
    DecreasePercent: int

class SYSTEM_BATTERY_STATE(ctypes.Structure):
    _fields_ = [
        ("AcOnLine", ctypes.c_uint8),
        ("BatteryPresent", ctypes.c_uint8),
        ("Charging", ctypes.c_uint8),
        ("Discharging", ctypes.c_uint8),
        ("Spare1", ctypes.c_uint8 * 0x3),  # unsigned __int8[3]
        ("Tag", ctypes.c_uint8),
        ("MaxCapacity", ctypes.c_uint32),
        ("RemainingCapacity", ctypes.c_uint32),
        ("Rate", ctypes.c_uint32),
        ("EstimatedTime", ctypes.c_uint32),
        ("DefaultAlert1", ctypes.c_uint32),
        ("DefaultAlert2", ctypes.c_uint32),
    ]
    AcOnLine: int
    BatteryPresent: int
    Charging: int
    Discharging: int
    Spare1: list[ctypes.c_uint8]
    Tag: int
    MaxCapacity: int
    RemainingCapacity: int
    Rate: int
    EstimatedTime: int
    DefaultAlert1: int
    DefaultAlert2: int

class MENUITEMTEMPLATEHEADER(ctypes.Structure):
    _fields_ = [
        ("versionNumber", ctypes.c_uint16),
        ("offset", ctypes.c_uint16),
    ]
    versionNumber: int
    offset: int

class MENUITEMTEMPLATE(ctypes.Structure):
    _fields_ = [
        ("mtOption", ctypes.c_uint16),
        ("mtID", ctypes.c_uint16),
        ("mtString", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    mtOption: int
    mtID: int
    mtString: list[ctypes.c_wchar]

class _FILE_ALIGNMENT_INFO(ctypes.Structure):
    _fields_ = [
        ("AlignmentRequirement", ctypes.c_uint32),
    ]
    AlignmentRequirement: int

class hknpGravityGun_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("initGun", ctypes.c_ulonglong),
        ("fireGun", ctypes.c_ulonglong),
        ("stepGun", ctypes.c_ulonglong),
        ("quitGun", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("createClone", ctypes.c_char * 0x8),  # hkRefNew<hknpFirstPersonGun> *(__fastcall *)(hknpFirstPersonGun *this, hkRefNew<hknpFirstPersonGun> *result, hknpWorld *)
        ("syncCloneFrom", ctypes.c_ulonglong),
        ("isBodyPickable", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    initGun: int
    fireGun: int
    stepGun: int
    quitGun: int
    getName: int
    reset: int
    createClone: bytes
    syncCloneFrom: int
    isBodyPickable: int

class tagMENUINFO(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fMask", ctypes.c_uint32),
        ("dwStyle", ctypes.c_uint32),
        ("cyMax", ctypes.c_uint32),
        ("hbrBack", ctypes.c_ulonglong),
        ("dwContextHelpID", ctypes.c_uint32),
        ("dwMenuData", ctypes.c_uint64),
    ]
    cbSize: int
    fMask: int
    dwStyle: int
    cyMax: int
    hbrBack: int
    dwContextHelpID: int
    dwMenuData: int

class cTkResourceManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResourceManagerBase_dtor_0", ctypes.c_ulonglong),
    ]
    cTkResourceManagerBase_dtor_0: int

class _SE_SID: pass

class _IMAGE_IMPORT_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $980F6A28C67C8F3640106D01D4C85A6A
        ("TimeDateStamp", ctypes.c_uint32),
        ("ForwarderChain", ctypes.c_uint32),
        ("Name", ctypes.c_uint32),
        ("FirstThunk", ctypes.c_uint32),
    ]
    ___u0: bytes
    TimeDateStamp: int
    ForwarderChain: int
    Name: int
    FirstThunk: int

class hknpCharacterProxyInternals_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class tagLASTINPUTINFO(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwTime", ctypes.c_uint32),
    ]
    cbSize: int
    dwTime: int

class _FILE_SEGMENT_ELEMENT: pass

class hknpVehicleLinearCastWheelCollide_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("init", ctypes.c_ulonglong),
        ("collideWheels", ctypes.c_ulonglong),
        ("updateBeforeCollisionDetection", ctypes.c_ulonglong),
        ("addToWorld", ctypes.c_ulonglong),
        ("removeFromWorld", ctypes.c_ulonglong),
        ("setCollisionFilterInfo", ctypes.c_ulonglong),
        ("wheelCollideCallback", ctypes.c_ulonglong),
        ("castSingleWheel", ctypes.c_ulonglong),
        ("getCollisionOutputFromCastResult", ctypes.c_ulonglong),
        ("getCollisionOutputWithoutHit", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    init: int
    collideWheels: int
    updateBeforeCollisionDetection: int
    addToWorld: int
    removeFromWorld: int
    setCollisionFilterInfo: int
    wheelCollideCallback: int
    castSingleWheel: int
    getCollisionOutputFromCastResult: int
    getCollisionOutputWithoutHit: int

class _NT_TIB(ctypes.Structure):
    _fields_ = [
        ("ExceptionList", ctypes.c_ulonglong),
        ("StackBase", ctypes.c_void_p),
        ("StackLimit", ctypes.c_void_p),
        ("SubSystemTib", ctypes.c_void_p),
        ("___u4", ctypes.c_char * 0x8),  # $7A727655067EA29DD1B3C3F7D79CBFD1
        ("ArbitraryUserPointer", ctypes.c_void_p),
        ("Self", ctypes.c_ulonglong),
    ]
    ExceptionList: int
    StackBase: int
    StackLimit: int
    SubSystemTib: int
    ___u4: bytes
    ArbitraryUserPointer: int
    Self: int

class tagRAWINPUTDEVICELIST(ctypes.Structure):
    _fields_ = [
        ("hDevice", ctypes.c_void_p),
        ("dwType", ctypes.c_uint32),
    ]
    hDevice: int
    dwType: int

class _PROCESS_MITIGATION_IMAGE_LOAD_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $560AA7274B4ADCC0850E73D74036FB45
    ]
    ___u0: bytes

class _TOKEN_USER_CLAIMS(ctypes.Structure):
    _fields_ = [
        ("UserClaims", ctypes.c_void_p),
    ]
    UserClaims: int

class _REARRANGE_FILE_DATA32(ctypes.Structure):
    _fields_ = [
        ("SourceStartingOffset", ctypes.c_uint64),
        ("TargetOffset", ctypes.c_uint64),
        ("SourceFileHandle", ctypes.c_uint32),
        ("Length", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
    ]
    SourceStartingOffset: int
    TargetOffset: int
    SourceFileHandle: int
    Length: int
    Flags: int

class HGESTUREINFO__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _IMAGE_THUNK_DATA64(ctypes.Structure):
    _fields_ = [
        ("u1", ctypes.c_char * 0x8),  # _IMAGE_THUNK_DATA64::<unnamed_type_u1>
    ]
    u1: bytes

class hkSimplexSolverOutput(ctypes.Structure):
    _fields_ = [
        ("m_position", common.Vector4f),
        ("m_velocity", common.Vector4f),
        ("m_deltaTime", ctypes.c_float),
        ("m_planeInteractions", ctypes.c_ulonglong),
    ]
    m_position: common.Vector4f
    m_velocity: common.Vector4f
    m_deltaTime: float
    m_planeInteractions: int

class hkDisplayBox_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class tagINPUT_TRANSFORM(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x40),  # $20ECD818BD172B49ACBA19E81EB7F427
    ]
    ___u0: bytes

class _EXCEPTION_RECORD64(ctypes.Structure):
    _fields_ = [
        ("ExceptionCode", ctypes.c_uint32),
        ("ExceptionFlags", ctypes.c_uint32),
        ("ExceptionRecord", ctypes.c_uint64),
        ("ExceptionAddress", ctypes.c_uint64),
        ("NumberParameters", ctypes.c_uint32),
        ("__unusedAlignment", ctypes.c_uint32),
        ("ExceptionInformation", ctypes.c_uint64 * 0xF),  # unsigned __int64[15]
    ]
    ExceptionCode: int
    ExceptionFlags: int
    ExceptionRecord: int
    ExceptionAddress: int
    NumberParameters: int
    __unusedAlignment: int
    ExceptionInformation: list[ctypes.c_uint64]

class _SE_ACCESS_REQUEST(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("SeSecurityDescriptor", ctypes.c_ulonglong),
        ("DesiredAccess", ctypes.c_uint32),
        ("PreviouslyGrantedAccess", ctypes.c_uint32),
        ("PrincipalSelfSid", ctypes.c_void_p),
        ("GenericMapping", ctypes.c_ulonglong),
        ("ObjectTypeListCount", ctypes.c_uint32),
        ("ObjectTypeList", ctypes.c_ulonglong),
    ]
    Size: int
    SeSecurityDescriptor: int
    DesiredAccess: int
    PreviouslyGrantedAccess: int
    PrincipalSelfSid: int
    GenericMapping: int
    ObjectTypeListCount: int
    ObjectTypeList: int

class _RESOURCEMANAGER_COMPLETION_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("IoCompletionPortHandle", ctypes.c_void_p),
        ("CompletionKey", ctypes.c_uint64),
    ]
    IoCompletionPortHandle: int
    CompletionKey: int

class _cpinfoexW(ctypes.Structure):
    _fields_ = [
        ("MaxCharSize", ctypes.c_uint32),
        ("DefaultChar", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
        ("LeadByte", ctypes.c_uint8 * 0xC),  # unsigned __int8[12]
        ("UnicodeDefaultChar", ctypes.c_wchar),
        ("CodePage", ctypes.c_uint32),
        ("CodePageName", ctypes.c_wchar * 0x104),  # wchar_t[260]
    ]
    MaxCharSize: int
    DefaultChar: list[ctypes.c_uint8]
    LeadByte: list[ctypes.c_uint8]
    UnicodeDefaultChar: str
    CodePage: int
    CodePageName: list[ctypes.c_wchar]

class _POWER_SESSION_WINLOGON(ctypes.Structure):
    _fields_ = [
        ("SessionId", ctypes.c_uint32),
        ("Console", ctypes.c_uint8),
        ("Locked", ctypes.c_uint8),
    ]
    SessionId: int
    Console: int
    Locked: int

class _ImageArchitectureHeader(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int16),
        ("_padding_2", ctypes.c_int16),
        ("FirstEntryRVA", ctypes.c_uint32),
    ]
    _bf_0: int
    _padding_2: int
    FirstEntryRVA: int

class _IMAGE_SEPARATE_DEBUG_HEADER(ctypes.Structure):
    _fields_ = [
        ("Signature", ctypes.c_uint16),
        ("Flags", ctypes.c_uint16),
        ("Machine", ctypes.c_uint16),
        ("Characteristics", ctypes.c_uint16),
        ("TimeDateStamp", ctypes.c_uint32),
        ("CheckSum", ctypes.c_uint32),
        ("ImageBase", ctypes.c_uint32),
        ("SizeOfImage", ctypes.c_uint32),
        ("NumberOfSections", ctypes.c_uint32),
        ("ExportedNamesSize", ctypes.c_uint32),
        ("DebugDirectorySize", ctypes.c_uint32),
        ("SectionAlignment", ctypes.c_uint32),
        ("Reserved", ctypes.c_uint32 * 0x2),  # unsigned int[2]
    ]
    Signature: int
    Flags: int
    Machine: int
    Characteristics: int
    TimeDateStamp: int
    CheckSum: int
    ImageBase: int
    SizeOfImage: int
    NumberOfSections: int
    ExportedNamesSize: int
    DebugDirectorySize: int
    SectionAlignment: int
    Reserved: list[ctypes.c_uint32]

class hknpVehicleRayCastWheelCollide_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("init", ctypes.c_ulonglong),
        ("collideWheels", ctypes.c_ulonglong),
        ("updateBeforeCollisionDetection", ctypes.c_ulonglong),
        ("addToWorld", ctypes.c_ulonglong),
        ("removeFromWorld", ctypes.c_ulonglong),
        ("setCollisionFilterInfo", ctypes.c_ulonglong),
        ("wheelCollideCallback", ctypes.c_ulonglong),
        ("calcWheelsAABB", ctypes.c_ulonglong),
        ("castSingleWheel", ctypes.c_ulonglong),
        ("getCollisionOutputFromCastResult", ctypes.c_ulonglong),
        ("getCollisionOutputWithoutHit", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    init: int
    collideWheels: int
    updateBeforeCollisionDetection: int
    addToWorld: int
    removeFromWorld: int
    setCollisionFilterInfo: int
    wheelCollideCallback: int
    calcWheelsAABB: int
    castSingleWheel: int
    getCollisionOutputFromCastResult: int
    getCollisionOutputWithoutHit: int

class _PROCESS_MITIGATION_CONTROL_FLOW_GUARD_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $2781E5432918173C9FC7A1FBB187517E
    ]
    ___u0: bytes

class tagWNDCLASSEXW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("style", ctypes.c_uint32),
        ("lpfnWndProc", ctypes.c_ulonglong),
        ("cbClsExtra", ctypes.c_int32),
        ("cbWndExtra", ctypes.c_int32),
        ("hInstance", ctypes.c_ulonglong),
        ("hIcon", ctypes.c_ulonglong),
        ("hCursor", ctypes.c_ulonglong),
        ("hbrBackground", ctypes.c_ulonglong),
        ("lpszMenuName", ctypes.c_ulonglong),
        ("lpszClassName", ctypes.c_ulonglong),
        ("hIconSm", ctypes.c_ulonglong),
    ]
    cbSize: int
    style: int
    lpfnWndProc: int
    cbClsExtra: int
    cbWndExtra: int
    hInstance: int
    hIcon: int
    hCursor: int
    hbrBackground: int
    lpszMenuName: int
    lpszClassName: int
    hIconSm: int

class _IMAGE_HOT_PATCH_HASHES(ctypes.Structure):
    _fields_ = [
        ("SHA256", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
        ("SHA1", ctypes.c_uint8 * 0x14),  # unsigned __int8[20]
    ]
    SHA256: list[ctypes.c_uint8]
    SHA1: list[ctypes.c_uint8]

class tagVS_FIXEDFILEINFO(ctypes.Structure):
    _fields_ = [
        ("dwSignature", ctypes.c_uint32),
        ("dwStrucVersion", ctypes.c_uint32),
        ("dwFileVersionMS", ctypes.c_uint32),
        ("dwFileVersionLS", ctypes.c_uint32),
        ("dwProductVersionMS", ctypes.c_uint32),
        ("dwProductVersionLS", ctypes.c_uint32),
        ("dwFileFlagsMask", ctypes.c_uint32),
        ("dwFileFlags", ctypes.c_uint32),
        ("dwFileOS", ctypes.c_uint32),
        ("dwFileType", ctypes.c_uint32),
        ("dwFileSubtype", ctypes.c_uint32),
        ("dwFileDateMS", ctypes.c_uint32),
        ("dwFileDateLS", ctypes.c_uint32),
    ]
    dwSignature: int
    dwStrucVersion: int
    dwFileVersionMS: int
    dwFileVersionLS: int
    dwProductVersionMS: int
    dwProductVersionLS: int
    dwFileFlagsMask: int
    dwFileFlags: int
    dwFileOS: int
    dwFileType: int
    dwFileSubtype: int
    dwFileDateMS: int
    dwFileDateLS: int

class HLSURF__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _TRANSACTION_BIND_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("TmHandle", ctypes.c_void_p),
    ]
    TmHandle: int

class _IMAGE_ENCLAVE_CONFIG64(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("MinimumRequiredConfigSize", ctypes.c_uint32),
        ("PolicyFlags", ctypes.c_uint32),
        ("NumberOfImports", ctypes.c_uint32),
        ("ImportList", ctypes.c_uint32),
        ("ImportEntrySize", ctypes.c_uint32),
        ("FamilyID", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("ImageID", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("ImageVersion", ctypes.c_uint32),
        ("SecurityVersion", ctypes.c_uint32),
        ("EnclaveSize", ctypes.c_uint64),
        ("NumberOfThreads", ctypes.c_uint32),
        ("EnclaveFlags", ctypes.c_uint32),
    ]
    Size: int
    MinimumRequiredConfigSize: int
    PolicyFlags: int
    NumberOfImports: int
    ImportList: int
    ImportEntrySize: int
    FamilyID: list[ctypes.c_uint8]
    ImageID: list[ctypes.c_uint8]
    ImageVersion: int
    SecurityVersion: int
    EnclaveSize: int
    NumberOfThreads: int
    EnclaveFlags: int

class _GRADIENT_TRIANGLE(ctypes.Structure):
    _fields_ = [
        ("Vertex1", ctypes.c_uint32),
        ("Vertex2", ctypes.c_uint32),
        ("Vertex3", ctypes.c_uint32),
    ]
    Vertex1: int
    Vertex2: int
    Vertex3: int

class hknpPrepareConstraintsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class cTkOldVector3(ctypes.Structure):
    _fields_ = [
        ("mfX", ctypes.c_float),
        ("mfY", ctypes.c_float),
        ("mfZ", ctypes.c_float),
        ("pad", ctypes.c_float),
    ]
    mfX: float
    mfY: float
    mfZ: float
    pad: float

class hknpSolverSchedulerTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class hknpPrepareSolverData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpBlockStreamBlockOffsetPair(ctypes.Structure):
    _fields_ = [
        ("m_block", ctypes.c_ulonglong),
        ("m_blockOffset", ctypes.c_uint16),
    ]
    m_block: int
    m_blockOffset: int

class hknpProcessNarrowPhasePairsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpProcessNarrowPhaseGridTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPrepareNarrowPhasePairsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpWorldTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPostCollideTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPreSolveTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpProcessFullCastsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpPostSolveTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpGenSolverTasksTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpSolveFinalizeTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpMultithreadedSimulation_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpSimulation_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("solve", ctypes.c_ulonglong),
        ("exportStatisticsToMonitorStream", ctypes.c_ulonglong),
        ("checkConsistency", ctypes.c_ulonglong),
    ]
    hknpSimulation_dtor_0: int
    collide: int
    solve: int
    exportStatisticsToMonitorStream: int
    checkConsistency: int

class hknpBuildConstraintJacobiansTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _IMAGE_DYNAMIC_RELOCATION_TABLE(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_uint32),
        ("Size", ctypes.c_uint32),
    ]
    Version: int
    Size: int

class _KNONVOLATILE_CONTEXT_POINTERS(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x80),  # $A6B1CE4BD1F9FF7AB9432814BAA1D82E
        ("___u1", ctypes.c_char * 0x80),  # $17FD409BD83021300FA07C80E07E5B8F
    ]
    ___u0: bytes
    ___u1: bytes

class _JOBOBJECT_SECURITY_LIMIT_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("SecurityLimitFlags", ctypes.c_uint32),
        ("JobToken", ctypes.c_void_p),
        ("SidsToDisable", ctypes.c_ulonglong),
        ("PrivilegesToDelete", ctypes.c_ulonglong),
        ("RestrictedSids", ctypes.c_ulonglong),
    ]
    SecurityLimitFlags: int
    JobToken: int
    SidsToDisable: int
    PrivilegesToDelete: int
    RestrictedSids: int

class hknpProcessContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpViewerColorScheme_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpViewerColorScheme_dtor_0", ctypes.c_ulonglong),
        ("getBodyColor", ctypes.c_ulonglong),
        ("getParticleColor", ctypes.c_ulonglong),
        ("getCharacterProxyColor", ctypes.c_ulonglong),
        ("getQueryColor", ctypes.c_ulonglong),
        ("isBodyVisible", ctypes.c_ulonglong),
        ("isVolatile", ctypes.c_ulonglong),
        ("isPerParticleColorSupported", ctypes.c_ulonglong),
    ]
    hknpViewerColorScheme_dtor_0: int
    getBodyColor: int
    getParticleColor: int
    getCharacterProxyColor: int
    getQueryColor: int
    isBodyVisible: int
    isVolatile: int
    isPerParticleColorSupported: int

class hknpSingleCellSpaceSplitter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("onShiftWorld", ctypes.c_ulonglong),
        ("getCellIdx", ctypes.c_ulonglong),
        ("applyThreadData", ctypes.c_ulonglong),
        ("calcInitialCellPositions", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    onShiftWorld: int
    getCellIdx: int
    applyThreadData: int
    calcInitialCellPositions: int
    copyFrom: int

class PROCESS_PROTECTION_LEVEL_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("ProtectionLevel", ctypes.c_uint32),
    ]
    ProtectionLevel: int

class _FILE_REMOTE_PROTOCOL_INFO(ctypes.Structure):
    _fields_ = [
        ("StructureVersion", ctypes.c_uint16),
        ("StructureSize", ctypes.c_uint16),
        ("Protocol", ctypes.c_uint32),
        ("ProtocolMajorVersion", ctypes.c_uint16),
        ("ProtocolMinorVersion", ctypes.c_uint16),
        ("ProtocolRevision", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16),
        ("Flags", ctypes.c_uint32),
        ("GenericReserved", ctypes.c_char * 0x20),  # _FILE_REMOTE_PROTOCOL_INFO::<unnamed_type_GenericReserved>
        ("ProtocolSpecific", ctypes.c_char * 0x40),  # _FILE_REMOTE_PROTOCOL_INFO::<unnamed_type_ProtocolSpecific>
    ]
    StructureVersion: int
    StructureSize: int
    Protocol: int
    ProtocolMajorVersion: int
    ProtocolMinorVersion: int
    ProtocolRevision: int
    Reserved: int
    Flags: int
    GenericReserved: bytes
    ProtocolSpecific: bytes

class hknpDynamicSpaceSplitter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("onShiftWorld", ctypes.c_ulonglong),
        ("getCellIdx", ctypes.c_ulonglong),
        ("applyThreadData", ctypes.c_ulonglong),
        ("calcInitialCellPositions", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    onShiftWorld: int
    getCellIdx: int
    applyThreadData: int
    calcInitialCellPositions: int
    copyFrom: int

class _FILEMUIINFO(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwVersion", ctypes.c_uint32),
        ("dwFileType", ctypes.c_uint32),
        ("pChecksum", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("pServiceChecksum", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("dwLanguageNameOffset", ctypes.c_uint32),
        ("dwTypeIDMainSize", ctypes.c_uint32),
        ("dwTypeIDMainOffset", ctypes.c_uint32),
        ("dwTypeNameMainOffset", ctypes.c_uint32),
        ("dwTypeIDMUISize", ctypes.c_uint32),
        ("dwTypeIDMUIOffset", ctypes.c_uint32),
        ("dwTypeNameMUIOffset", ctypes.c_uint32),
        ("abBuffer", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    dwSize: int
    dwVersion: int
    dwFileType: int
    pChecksum: list[ctypes.c_uint8]
    pServiceChecksum: list[ctypes.c_uint8]
    dwLanguageNameOffset: int
    dwTypeIDMainSize: int
    dwTypeIDMainOffset: int
    dwTypeNameMainOffset: int
    dwTypeIDMUISize: int
    dwTypeIDMUIOffset: int
    dwTypeNameMUIOffset: int
    abBuffer: list[ctypes.c_uint8]

class _ASSEMBLY_FILE_DETAILED_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("ulFlags", ctypes.c_uint32),
        ("ulFilenameLength", ctypes.c_uint32),
        ("ulPathLength", ctypes.c_uint32),
        ("lpFileName", ctypes.c_ulonglong),
        ("lpFilePath", ctypes.c_ulonglong),
    ]
    ulFlags: int
    ulFilenameLength: int
    ulPathLength: int
    lpFileName: int
    lpFilePath: int

class tagMETARECORD(ctypes.Structure):
    _fields_ = [
        ("rdSize", ctypes.c_uint32),
        ("rdFunction", ctypes.c_uint16),
        ("rdParm", ctypes.c_uint16 * 0x1),  # unsigned __int16[1]
    ]
    rdSize: int
    rdFunction: int
    rdParm: list[ctypes.c_uint16]

class _MEMORY_PRIORITY_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("MemoryPriority", ctypes.c_uint32),
    ]
    MemoryPriority: int

class _TOKEN_OWNER(ctypes.Structure):
    _fields_ = [
        ("Owner", ctypes.c_void_p),
    ]
    Owner: int

class _DOCINFOA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_int32),
        ("lpszDocName", ctypes.c_char_p),
        ("lpszOutput", ctypes.c_char_p),
        ("lpszDatatype", ctypes.c_char_p),
        ("fwType", ctypes.c_uint32),
    ]
    cbSize: int
    lpszDocName: bytes
    lpszOutput: bytes
    lpszDatatype: bytes
    fwType: int

class tagSERIALKEYSW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("lpszActivePort", ctypes.c_ulonglong),
        ("lpszPort", ctypes.c_ulonglong),
        ("iBaudRate", ctypes.c_uint32),
        ("iPortState", ctypes.c_uint32),
        ("iActive", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int
    lpszActivePort: int
    lpszPort: int
    iBaudRate: int
    iPortState: int
    iActive: int

class HFONT__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _PROCESS_MITIGATION_USER_SHADOW_STACK_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $3E013F3DB047C7E0BF4C4C2DC062A2D6
    ]
    ___u0: bytes

class _PROCESS_MITIGATION_EXTENSION_POINT_DISABLE_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $D190FBDE19D811D922D698C57631FD33
    ]
    ___u0: bytes

class _TOKEN_AUDIT_POLICY(ctypes.Structure):
    _fields_ = [
        ("PerUserPolicy", ctypes.c_uint8 * 0x1E),  # unsigned __int8[30]
    ]
    PerUserPolicy: list[ctypes.c_uint8]

class _TOKEN_SID_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("Sid", ctypes.c_void_p),
    ]
    Sid: int

class _PROCESS_MITIGATION_SIDE_CHANNEL_ISOLATION_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $A06914C17A6CF6ACC8D226DC5AEAEAD8
    ]
    ___u0: bytes

class JOBOBJECT_IO_RATE_CONTROL_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("MaxIops", ctypes.c_int64),
        ("MaxBandwidth", ctypes.c_int64),
        ("ReservationIops", ctypes.c_int64),
        ("VolumeName", ctypes.c_ulonglong),
        ("BaseIoSize", ctypes.c_uint32),
        ("ControlFlags", ctypes.c_uint32),
    ]
    MaxIops: int
    MaxBandwidth: int
    ReservationIops: int
    VolumeName: int
    BaseIoSize: int
    ControlFlags: int

class cTkDoubleBufferMemoryPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkMemoryPool_dtor_0", ctypes.c_ulonglong),
        ("Create", ctypes.c_ulonglong),
        ("Malloc", ctypes.c_ulonglong),
        ("Malloc_18", ctypes.c_ulonglong),
        ("Free", ctypes.c_ulonglong),
        ("Realloc", ctypes.c_ulonglong),
        ("ContainsAddress", ctypes.c_ulonglong),
        ("GetAllocationSize", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("GetUsage", ctypes.c_ulonglong),
        ("GetHighWaterUsage", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("Cleanup", ctypes.c_ulonglong),
        ("GetAllocInfo", ctypes.c_ulonglong),
        ("SetAllocInfo", ctypes.c_ulonglong),
        ("ClearStats", ctypes.c_ulonglong),
        ("OutputStats", ctypes.c_ulonglong),
        ("FrameSyncEnd", ctypes.c_ulonglong),
        ("NeedsFrameSyncEnd", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("FlagError", ctypes.c_ulonglong),
        ("HasError", ctypes.c_ulonglong),
        ("NeedsMutex", ctypes.c_ulonglong),
        ("SetMutex", ctypes.c_ulonglong),
        ("GetOverflowPool", ctypes.c_ulonglong),
        ("GetBaseAddress", ctypes.c_ulonglong),
    ]
    cTkMemoryPool_dtor_0: int
    Create: int
    Malloc: int
    Malloc_18: int
    Free: int
    Realloc: int
    ContainsAddress: int
    GetAllocationSize: int
    SetAlignment: int
    GetAlignment: int
    GetUsage: int
    GetHighWaterUsage: int
    GetSize: int
    Cleanup: int
    GetAllocInfo: int
    SetAllocInfo: int
    ClearStats: int
    OutputStats: int
    FrameSyncEnd: int
    NeedsFrameSyncEnd: int
    Reset: int
    FlagError: int
    HasError: int
    NeedsMutex: int
    SetMutex: int
    GetOverflowPool: int
    GetBaseAddress: int

class _JIT_DEBUG_INFO(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwProcessorArchitecture", ctypes.c_uint32),
        ("dwThreadID", ctypes.c_uint32),
        ("dwReserved0", ctypes.c_uint32),
        ("lpExceptionAddress", ctypes.c_uint64),
        ("lpExceptionRecord", ctypes.c_uint64),
        ("lpContextRecord", ctypes.c_uint64),
    ]
    dwSize: int
    dwProcessorArchitecture: int
    dwThreadID: int
    dwReserved0: int
    lpExceptionAddress: int
    lpExceptionRecord: int
    lpContextRecord: int

class hknpSolverScheduler(ctypes.Structure):
    def addSolverTasks2(): pass

    def addSolverTasks(): pass

    def createFullStepTask(): pass

    def estimateNumTasks(): pass


class _FILE_RENAME_INFO(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $0A49EDEAD303E213916BAA6EBDEF3657
        ("RootDirectory", ctypes.c_void_p),
        ("FileNameLength", ctypes.c_uint32),
        ("FileName", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    ___u0: bytes
    RootDirectory: int
    FileNameLength: int
    FileName: list[ctypes.c_wchar]

class _Min_max_element_t(ctypes.Structure):
    _fields_ = [
        ("_Min", ctypes.c_void_p),
        ("_Max", ctypes.c_void_p),
    ]
    _Min: int
    _Max: int

class hknpStreamContactSolverBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("solveJacobians", ctypes.c_ulonglong),
        ("getContactSolverType", ctypes.c_ulonglong),
        ("createInjectors", ctypes.c_ulonglong),
        ("destroyInjectors", ctypes.c_ulonglong),
        ("bindInjectors", ctypes.c_ulonglong),
        ("initMotionTransform", ctypes.c_ulonglong),
        ("injectJacobian", ctypes.c_ulonglong),
        ("setJacRangeStartPoint", ctypes.c_ulonglong),
        ("setJacRangeEndPoint", ctypes.c_ulonglong),
        ("reuseJacobian", ctypes.c_ulonglong),
        ("buildJacobianV", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    solveJacobians: int
    getContactSolverType: int
    createInjectors: int
    destroyInjectors: int
    bindInjectors: int
    initMotionTransform: int
    injectJacobian: int
    setJacRangeStartPoint: int
    setJacRangeEndPoint: int
    reuseJacobian: int
    buildJacobianV: int

class hknpStreamContactSolver_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("solveJacobians", ctypes.c_ulonglong),
        ("getContactSolverType", ctypes.c_ulonglong),
        ("createInjectors", ctypes.c_ulonglong),
        ("destroyInjectors", ctypes.c_ulonglong),
        ("bindInjectors", ctypes.c_ulonglong),
        ("initMotionTransform", ctypes.c_ulonglong),
        ("injectJacobian", ctypes.c_ulonglong),
        ("setJacRangeStartPoint", ctypes.c_ulonglong),
        ("setJacRangeEndPoint", ctypes.c_ulonglong),
        ("reuseJacobian", ctypes.c_ulonglong),
        ("buildJacobianV", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    solveJacobians: int
    getContactSolverType: int
    createInjectors: int
    destroyInjectors: int
    bindInjectors: int
    initMotionTransform: int
    injectJacobian: int
    setJacRangeStartPoint: int
    setJacRangeEndPoint: int
    reuseJacobian: int
    buildJacobianV: int

class _COMMPROP(ctypes.Structure):
    _fields_ = [
        ("wPacketLength", ctypes.c_uint16),
        ("wPacketVersion", ctypes.c_uint16),
        ("dwServiceMask", ctypes.c_uint32),
        ("dwReserved1", ctypes.c_uint32),
        ("dwMaxTxQueue", ctypes.c_uint32),
        ("dwMaxRxQueue", ctypes.c_uint32),
        ("dwMaxBaud", ctypes.c_uint32),
        ("dwProvSubType", ctypes.c_uint32),
        ("dwProvCapabilities", ctypes.c_uint32),
        ("dwSettableParams", ctypes.c_uint32),
        ("dwSettableBaud", ctypes.c_uint32),
        ("wSettableData", ctypes.c_uint16),
        ("wSettableStopParity", ctypes.c_uint16),
        ("dwCurrentTxQueue", ctypes.c_uint32),
        ("dwCurrentRxQueue", ctypes.c_uint32),
        ("dwProvSpec1", ctypes.c_uint32),
        ("dwProvSpec2", ctypes.c_uint32),
        ("wcProvChar", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    wPacketLength: int
    wPacketVersion: int
    dwServiceMask: int
    dwReserved1: int
    dwMaxTxQueue: int
    dwMaxRxQueue: int
    dwMaxBaud: int
    dwProvSubType: int
    dwProvCapabilities: int
    dwSettableParams: int
    dwSettableBaud: int
    wSettableData: int
    wSettableStopParity: int
    dwCurrentTxQueue: int
    dwCurrentRxQueue: int
    dwProvSpec1: int
    dwProvSpec2: int
    wcProvChar: list[ctypes.c_wchar]

class _SERVICE_TRIGGER(ctypes.Structure):
    _fields_ = [
        ("dwTriggerType", ctypes.c_uint32),
        ("dwAction", ctypes.c_uint32),
        ("pTriggerSubtype", ctypes.c_ulonglong),
        ("cDataItems", ctypes.c_uint32),
        ("pDataItems", ctypes.c_ulonglong),
    ]
    dwTriggerType: int
    dwAction: int
    pTriggerSubtype: int
    cDataItems: int
    pDataItems: int

class tagACCEL(ctypes.Structure):
    _fields_ = [
        ("fVirt", ctypes.c_uint8),
        ("key", ctypes.c_uint16),
        ("cmd", ctypes.c_uint16),
    ]
    fVirt: int
    key: int
    cmd: int

class _SERVICE_FAILURE_ACTIONS_FLAG(ctypes.Structure):
    _fields_ = [
        ("fFailureActionsOnNonCrashFailures", ctypes.c_int32),
    ]
    fFailureActionsOnNonCrashFailures: int

class _SERVICE_TRIGGER_INFO(ctypes.Structure):
    _fields_ = [
        ("cTriggers", ctypes.c_uint32),
        ("pTriggers", ctypes.c_ulonglong),
        ("pReserved", ctypes.c_ulonglong),
    ]
    cTriggers: int
    pTriggers: int
    pReserved: int

class _NV_MEMORY_RANGE(ctypes.Structure):
    _fields_ = [
        ("BaseAddress", ctypes.c_void_p),
        ("Length", ctypes.c_uint64),
    ]
    BaseAddress: int
    Length: int

class _TRANSACTIONMANAGER_RECOVERY_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("LastRecoveredLsn", ctypes.c_uint64),
    ]
    LastRecoveredLsn: int

class hknpTransformedShapeEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpScopedSimulationThreadContext(ctypes.Structure):
    _fields_ = [
        ("m_context", ctypes.c_ulonglong),
        ("m_simulationContext", ctypes.c_ulonglong),
    ]
    m_context: int
    m_simulationContext: int

class _IMAGE_RESOURCE_DIRECTORY_STRING(ctypes.Structure):
    _fields_ = [
        ("Length", ctypes.c_uint16),
        ("NameString", ctypes.c_char * 0x1),  # char[1]
    ]
    Length: int
    NameString: list[ctypes.c_char]

class cTkSingleBufferMemoryPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkMemoryPool_dtor_0", ctypes.c_ulonglong),
        ("Create", ctypes.c_ulonglong),
        ("Malloc", ctypes.c_ulonglong),
        ("Malloc_18", ctypes.c_ulonglong),
        ("Free", ctypes.c_ulonglong),
        ("Realloc", ctypes.c_ulonglong),
        ("ContainsAddress", ctypes.c_ulonglong),
        ("GetAllocationSize", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("GetUsage", ctypes.c_ulonglong),
        ("GetHighWaterUsage", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("Cleanup", ctypes.c_ulonglong),
        ("GetAllocInfo", ctypes.c_ulonglong),
        ("SetAllocInfo", ctypes.c_ulonglong),
        ("ClearStats", ctypes.c_ulonglong),
        ("OutputStats", ctypes.c_ulonglong),
        ("FrameSyncEnd", ctypes.c_ulonglong),
        ("NeedsFrameSyncEnd", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("FlagError", ctypes.c_ulonglong),
        ("HasError", ctypes.c_ulonglong),
        ("NeedsMutex", ctypes.c_ulonglong),
        ("SetMutex", ctypes.c_ulonglong),
        ("GetOverflowPool", ctypes.c_ulonglong),
        ("GetBaseAddress", ctypes.c_ulonglong),
    ]
    cTkMemoryPool_dtor_0: int
    Create: int
    Malloc: int
    Malloc_18: int
    Free: int
    Realloc: int
    ContainsAddress: int
    GetAllocationSize: int
    SetAlignment: int
    GetAlignment: int
    GetUsage: int
    GetHighWaterUsage: int
    GetSize: int
    Cleanup: int
    GetAllocInfo: int
    SetAllocInfo: int
    ClearStats: int
    OutputStats: int
    FrameSyncEnd: int
    NeedsFrameSyncEnd: int
    Reset: int
    FlagError: int
    HasError: int
    NeedsMutex: int
    SetMutex: int
    GetOverflowPool: int
    GetBaseAddress: int

class _ENCLAVE_CREATE_INFO_SGX(ctypes.Structure):
    _fields_ = [
        ("Secs", ctypes.c_uint8 * 0x1000),  # unsigned __int8[4096]
    ]
    Secs: list[ctypes.c_uint8]

class DISPLAYCONFIG_2DREGION(ctypes.Structure):
    _fields_ = [
        ("_cx", ctypes.c_uint32),
        ("cy", ctypes.c_uint32),
    ]
    _cx: int
    cy: int

class _SCOPE_TABLE_ARM(ctypes.Structure):
    _fields_ = [
        ("Count", ctypes.c_uint32),
        ("ScopeRecord", ctypes.c_char * 0x10),  # _SCOPE_TABLE_ARM::<unnamed_type_ScopeRecord>
    ]
    Count: int
    ScopeRecord: bytes

class _PROCESS_MITIGATION_ASLR_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $471CF2586ADA477B7443FBC6B18E1C88
    ]
    ___u0: bytes

class tagPOINTER_DEVICE_PROPERTY(ctypes.Structure):
    _fields_ = [
        ("logicalMin", ctypes.c_int32),
        ("logicalMax", ctypes.c_int32),
        ("physicalMin", ctypes.c_int32),
        ("physicalMax", ctypes.c_int32),
        ("unit", ctypes.c_uint32),
        ("unitExponent", ctypes.c_uint32),
        ("usagePageId", ctypes.c_uint16),
        ("usageId", ctypes.c_uint16),
    ]
    logicalMin: int
    logicalMax: int
    physicalMin: int
    physicalMax: int
    unit: int
    unitExponent: int
    usagePageId: int
    usageId: int

class _DISCDLGSTRUCTW(ctypes.Structure):
    _fields_ = [
        ("cbStructure", ctypes.c_uint32),
        ("hwndOwner", ctypes.c_ulonglong),
        ("lpLocalName", ctypes.c_ulonglong),
        ("lpRemoteName", ctypes.c_ulonglong),
        ("dwFlags", ctypes.c_uint32),
    ]
    cbStructure: int
    hwndOwner: int
    lpLocalName: int
    lpRemoteName: int
    dwFlags: int

class _SCRUB_DATA_INPUT(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("MaximumIos", ctypes.c_uint32),
        ("ObjectId", ctypes.c_uint32 * 0x4),  # unsigned int[4]
        ("Reserved", ctypes.c_uint32 * 0x19),  # unsigned int[25]
        ("ResumeContext", ctypes.c_uint8 * 0x330),  # unsigned __int8[816]
    ]
    Size: int
    Flags: int
    MaximumIos: int
    ObjectId: list[ctypes.c_uint32]
    Reserved: list[ctypes.c_uint32]
    ResumeContext: list[ctypes.c_uint8]

class hknpCollisionCacheReader(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x18),  # hkTypedBlockStream<hknpCollisionCache>::Reader
    ]
    baseclass_0: bytes

class _PROCESS_DYNAMIC_EH_CONTINUATION_TARGET(ctypes.Structure):
    _fields_ = [
        ("TargetAddress", ctypes.c_uint64),
        ("Flags", ctypes.c_uint64),
    ]
    TargetAddress: int
    Flags: int

class tagTOGGLEKEYS(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int

class _IMAGE_AUX_SYMBOL: pass

class hknpViewerEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpFirstPersonCharacter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCharacterRigidBodyCinfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpParticleBatchInfo(ctypes.Structure):
    _fields_ = [
        ("m_startIter", ctypes.c_int32),
        ("m_endIter", ctypes.c_int32),
    ]
    m_startIter: int
    m_endIter: int

class hkaSkeleton_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpScopedParticleSimulationThreadContext(ctypes.Structure):
    _fields_ = [
        ("m_context", ctypes.c_ulonglong),
        ("m_simulationContext", ctypes.c_ulonglong),
    ]
    m_context: int
    m_simulationContext: int

class hknpRagdoll_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpRagdollData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("checkConsistency", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    checkConsistency: int

class hknpCharacterStateOnGround_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("update", ctypes.c_ulonglong),
        ("getNextState", ctypes.c_ulonglong),
        ("enterState", ctypes.c_ulonglong),
        ("leaveState", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    update: int
    getNextState: int
    enterState: int
    leaveState: int

class hknpRagdollState_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("setStrength", ctypes.c_ulonglong),
        ("initialize", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("step", ctypes.c_ulonglong),
        ("enterState", ctypes.c_ulonglong),
        ("leaveState", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    setStrength: int
    initialize: int
    reset: int
    step: int
    enterState: int
    leaveState: int

class hknpShapeQueryInterface(ctypes.Structure):
    def castRay(): pass

    def castShape(): pass

    def getClosestPoints(): pass


class hknpVehicleDefaultBrake_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcBreakingInfo", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcBreakingInfo: int

class hknpBodyManagerCinfo(ctypes.Structure):
    _fields_ = [
        ("m_world", ctypes.c_ulonglong),
        ("m_userBodyBuffer", ctypes.c_ulonglong),
        ("m_capacity", ctypes.c_uint32),
        ("m_useBackLinks", ctypes.c_ubyte),
    ]
    m_world: int
    m_userBodyBuffer: int
    m_capacity: int
    m_useBackLinks: bool

class _QUERY_SERVICE_CONFIGA(ctypes.Structure):
    _fields_ = [
        ("dwServiceType", ctypes.c_uint32),
        ("dwStartType", ctypes.c_uint32),
        ("dwErrorControl", ctypes.c_uint32),
        ("lpBinaryPathName", ctypes.c_char_p),
        ("lpLoadOrderGroup", ctypes.c_char_p),
        ("dwTagId", ctypes.c_uint32),
        ("lpDependencies", ctypes.c_char_p),
        ("lpServiceStartName", ctypes.c_char_p),
        ("lpDisplayName", ctypes.c_char_p),
    ]
    dwServiceType: int
    dwStartType: int
    dwErrorControl: int
    lpBinaryPathName: bytes
    lpLoadOrderGroup: bytes
    dwTagId: int
    lpDependencies: bytes
    lpServiceStartName: bytes
    lpDisplayName: bytes

class hknpAllHitsBroadPhaseQueryCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    addHit: int

class HKL__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class hknpVehicleDriverInputAnalogStatus_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("clone", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    clone: int

class tagCOPYDATASTRUCT(ctypes.Structure):
    _fields_ = [
        ("dwData", ctypes.c_uint64),
        ("cbData", ctypes.c_uint32),
        ("lpData", ctypes.c_void_p),
    ]
    dwData: int
    cbData: int
    lpData: int

class hknpVehicleDefaultAnalogDriverInput_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcDriverInput", ctypes.c_ulonglong),
        ("calcAcceleratorInput", ctypes.c_ulonglong),
        ("calcBrakeInput", ctypes.c_ulonglong),
        ("calcTryingToReverse", ctypes.c_ulonglong),
        ("calcSteeringInput", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcDriverInput: int
    calcAcceleratorInput: int
    calcBrakeInput: int
    calcTryingToReverse: int
    calcSteeringInput: int

class tagMOUSEKEYS(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("iMaxSpeed", ctypes.c_uint32),
        ("iTimeToMaxSpeed", ctypes.c_uint32),
        ("iCtrlSpeed", ctypes.c_uint32),
        ("dwReserved1", ctypes.c_uint32),
        ("dwReserved2", ctypes.c_uint32),
    ]
    cbSize: int
    dwFlags: int
    iMaxSpeed: int
    iTimeToMaxSpeed: int
    iCtrlSpeed: int
    dwReserved1: int
    dwReserved2: int

class CacheComparer(ctypes.Structure):
    pass

class IslandComparer(ctypes.Structure):
    _fields_ = [
        ("m_islands", ctypes.c_char * 0x10),  # hkArrayView<hknpDeactivatedIsland *>
    ]
    m_islands: bytes

class _SERVICE_SID_INFO(ctypes.Structure):
    _fields_ = [
        ("dwServiceSidType", ctypes.c_uint32),
    ]
    dwServiceSidType: int

class hknpVehicleDefaultSteering_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcSteering", ctypes.c_ulonglong),
        ("calcMainSteeringAngle", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcSteering: int
    calcMainSteeringAngle: int

class _JOB_SET_ARRAY(ctypes.Structure):
    _fields_ = [
        ("JobHandle", ctypes.c_void_p),
        ("MemberLevel", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
    ]
    JobHandle: int
    MemberLevel: int
    Flags: int

class hknpVehicleDefaultSuspension_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("calcSuspension", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    calcSuspension: int

class _IMAGE_BASE_RELOCATION(ctypes.Structure):
    _fields_ = [
        ("VirtualAddress", ctypes.c_uint32),
        ("SizeOfBlock", ctypes.c_uint32),
    ]
    VirtualAddress: int
    SizeOfBlock: int

class _GRADIENT_RECT(ctypes.Structure):
    _fields_ = [
        ("UpperLeft", ctypes.c_uint32),
        ("LowerRight", ctypes.c_uint32),
    ]
    UpperLeft: int
    LowerRight: int

class CORRELATION_VECTOR(ctypes.Structure):
    _fields_ = [
        ("Version", ctypes.c_char),
        ("Vector", ctypes.c_char * 0x81),  # char[129]
    ]
    Version: str
    Vector: list[ctypes.c_char]

class __noop_coroutine_frame_legacy(ctypes.Structure):
    _fields_ = [
        ("resume", ctypes.c_void_p),
        ("flag", ctypes.c_int32),
        ("index", ctypes.c_int32),
    ]
    resume: int
    flag: int
    index: int

class ActivatedConstraintsLess(ctypes.Structure):
    pass

class ActivatedConstraintsEquals(ctypes.Structure):
    pass

class NVGcolor(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x10),  # $CFCD15CDE963022C22DCB77EEF2AB1E5
    ]
    ___u0: bytes

class NVGscissor(ctypes.Structure):
    _fields_ = [
        ("xform", ctypes.c_float * 0x6),  # float[6]
        ("extent", ctypes.c_float * 0x2),  # float[2]
    ]
    xform: list[ctypes.c_float]
    extent: list[ctypes.c_float]

class NVGvertex(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("u", ctypes.c_float),
        ("v", ctypes.c_float),
    ]
    x: float
    y: float
    u: float
    v: float

class NVGpath(ctypes.Structure):
    _fields_ = [
        ("fill", ctypes.c_ulonglong),
        ("stroke", ctypes.c_ulonglong),
        ("first", ctypes.c_int32),
        ("count", ctypes.c_int32),
        ("nbevel", ctypes.c_int32),
        ("nfill", ctypes.c_int32),
        ("nstroke", ctypes.c_int32),
        ("winding", ctypes.c_int32),
        ("convex", ctypes.c_int32),
        ("closed", ctypes.c_uint8),
    ]
    fill: int
    stroke: int
    first: int
    count: int
    nbevel: int
    nfill: int
    nstroke: int
    winding: int
    convex: int
    closed: int

class NVGparams(ctypes.Structure):
    _fields_ = [
        ("userPtr", ctypes.c_void_p),
        ("edgeAntiAlias", ctypes.c_int32),
        ("renderCreate", ctypes.c_ulonglong),
        ("renderCreateTexture", ctypes.c_ulonglong),
        ("renderDeleteTexture", ctypes.c_ulonglong),
        ("renderUpdateTexture", ctypes.c_ulonglong),
        ("renderGetTextureSize", ctypes.c_ulonglong),
        ("renderViewport", ctypes.c_ulonglong),
        ("renderCancel", ctypes.c_ulonglong),
        ("renderFlush", ctypes.c_ulonglong),
        ("renderFill", ctypes.c_ulonglong),
        ("renderStroke", ctypes.c_ulonglong),
        ("renderTriangles", ctypes.c_ulonglong),
        ("renderDelete", ctypes.c_ulonglong),
        ("renderActualEndFrame", ctypes.c_ulonglong),
        ("renderRegisterTkTexture", ctypes.c_ulonglong),
        ("renderUnregisterTkTexture", ctypes.c_ulonglong),
        ("renderGetTkTexture", ctypes.c_ulonglong),
    ]
    userPtr: int
    edgeAntiAlias: int
    renderCreate: int
    renderCreateTexture: int
    renderDeleteTexture: int
    renderUpdateTexture: int
    renderGetTextureSize: int
    renderViewport: int
    renderCancel: int
    renderFlush: int
    renderFill: int
    renderStroke: int
    renderTriangles: int
    renderDelete: int
    renderActualEndFrame: int
    renderRegisterTkTexture: int
    renderUnregisterTkTexture: int
    renderGetTkTexture: int

class NVGpoint(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("_dx", ctypes.c_float),
        ("dy", ctypes.c_float),
        ("len", ctypes.c_float),
        ("dmx", ctypes.c_float),
        ("dmy", ctypes.c_float),
        ("flags", ctypes.c_uint8),
    ]
    x: float
    y: float
    _dx: float
    dy: float
    len: float
    dmx: float
    dmy: float
    flags: int

class NVGpathCache(ctypes.Structure):
    _fields_ = [
        ("points", ctypes.c_ulonglong),
        ("npoints", ctypes.c_int32),
        ("cpoints", ctypes.c_int32),
        ("paths", ctypes.c_ulonglong),
        ("npaths", ctypes.c_int32),
        ("cpaths", ctypes.c_int32),
        ("verts", ctypes.c_ulonglong),
        ("nverts", ctypes.c_int32),
        ("cverts", ctypes.c_int32),
        ("bounds", ctypes.c_float * 0x4),  # float[4]
    ]
    points: int
    npoints: int
    cpoints: int
    paths: int
    npaths: int
    cpaths: int
    verts: int
    nverts: int
    cverts: int
    bounds: list[ctypes.c_float]

class hkDebugCommandProcessor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("exec", ctypes.c_ulonglong),
        ("print", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    exec: int
    print: int

class hknpCheckedBroadPhase_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpBroadPhase_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getConfig", ctypes.c_ulonglong),
        ("setBodyBuffer", ctypes.c_ulonglong),
        ("translate", ctypes.c_ulonglong),
        ("addBodies", ctypes.c_ulonglong),
        ("removeBodies", ctypes.c_ulonglong),
        ("markBodiesDirty", ctypes.c_ulonglong),
        ("update", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, hkFlags<enum hknpBroadPhaseFilter::Category,unsigned char>)
        ("updateBodyAabbs", ctypes.c_ulonglong),
        ("buildTaskGraph", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, hknpWorld *, hknpSimulationContext *, hkFlags<enum hknpBroadPhaseFilter::Category,unsigned char>, hkTypedBlockStream<hknpBodyIndexPair> *, hknpTaskGraph *)
        ("getExtents", ctypes.c_ulonglong),
        ("findAllPairs", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpSimulationThreadContext *, hkTypedBlockStream<hknpBodyIndexPair>::Writer *)
        ("findNewPairs", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpSimulationThreadContext *, const hkAabbFloat3 *, hkTypedBlockStream<hknpBodyIndexPair>::Writer *)
        ("queryAabb", ctypes.c_ulonglong),
        ("queryAabb_78", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabb_80", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryOutsideOfAabb", ctypes.c_ulonglong),
        ("queryOutsideOfAabb_90", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp", ctypes.c_ulonglong),
        ("queryAabbNmp_a0", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp_a8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castAabb", ctypes.c_ulonglong),
        ("castAabb_b8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbCastQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castRay", ctypes.c_ulonglong),
        ("castShape", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpShapeCastQuery *, const hkRotationImpl<float> *, const hknpCollisionQueryDispatcher *, hknpCollisionQueryCollector *, hknpCollisionQueryCollector *, hknpCollisionQueryContext *)
        ("queryPoint", ctypes.c_ulonglong),
        ("getClosestPoints", ctypes.c_ulonglong),
        ("getClosestPointsAdvanced", ctypes.c_ulonglong),
        ("getClosestPointsParticles", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
    ]
    hknpBroadPhase_dtor_0: int
    getType: int
    getConfig: int
    setBodyBuffer: int
    translate: int
    addBodies: int
    removeBodies: int
    markBodiesDirty: int
    update: bytes
    updateBodyAabbs: int
    buildTaskGraph: bytes
    getExtents: int
    findAllPairs: bytes
    findNewPairs: bytes
    queryAabb: int
    queryAabb_78: bytes
    queryAabb_80: bytes
    queryOutsideOfAabb: int
    queryOutsideOfAabb_90: bytes
    queryAabbNmp: int
    queryAabbNmp_a0: bytes
    queryAabbNmp_a8: bytes
    castAabb: int
    castAabb_b8: bytes
    castRay: int
    castShape: bytes
    queryPoint: int
    getClosestPoints: int
    getClosestPointsAdvanced: int
    getClosestPointsParticles: int
    copyFrom: int

class _SERVICE_START_REASON(ctypes.Structure):
    _fields_ = [
        ("dwReason", ctypes.c_uint32),
    ]
    dwReason: int

class _PROCESS_DYNAMIC_EH_CONTINUATION_TARGETS_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("NumberOfTargets", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16),
        ("Reserved2", ctypes.c_uint32),
        ("Targets", ctypes.c_ulonglong),
    ]
    NumberOfTargets: int
    Reserved: int
    Reserved2: int
    Targets: int

class _IMAGE_IMPORT_CONTROL_TRANSFER_DYNAMIC_RELOCATION(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int32),
    ]
    _bf_0: int

class _PROCESS_MITIGATION_CHILD_PROCESS_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $BCE16398E45424589D1EF5167CC0F2FA
    ]
    ___u0: bytes

class _currencyfmtW(ctypes.Structure):
    _fields_ = [
        ("NumDigits", ctypes.c_uint32),
        ("LeadingZero", ctypes.c_uint32),
        ("Grouping", ctypes.c_uint32),
        ("lpDecimalSep", ctypes.c_ulonglong),
        ("lpThousandSep", ctypes.c_ulonglong),
        ("NegativeOrder", ctypes.c_uint32),
        ("PositiveOrder", ctypes.c_uint32),
        ("lpCurrencySymbol", ctypes.c_ulonglong),
    ]
    NumDigits: int
    LeadingZero: int
    Grouping: int
    lpDecimalSep: int
    lpThousandSep: int
    NegativeOrder: int
    PositiveOrder: int
    lpCurrencySymbol: int

class _WOW64_ARCHITECTURE_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int32),
    ]
    _bf_0: int

class cTkPausableAreaScopeJanitor(ctypes.Structure):
    _fields_ = [
        ("mbPrevPausable", ctypes.c_ubyte),
    ]
    mbPrevPausable: bool

class _IMAGE_BOUND_FORWARDER_REF(ctypes.Structure):
    _fields_ = [
        ("TimeDateStamp", ctypes.c_uint32),
        ("OffsetModuleName", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16),
    ]
    TimeDateStamp: int
    OffsetModuleName: int
    Reserved: int

class hknpBroadPhaseViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hkProcessFactory_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class _UMS_SCHEDULER_STARTUP_INFO(ctypes.Structure):
    _fields_ = [
        ("UmsVersion", ctypes.c_uint32),
        ("CompletionList", ctypes.c_void_p),
        ("SchedulerProc", ctypes.c_ulonglong),
        ("SchedulerParam", ctypes.c_void_p),
    ]
    UmsVersion: int
    CompletionList: int
    SchedulerProc: int
    SchedulerParam: int

class _SLIST_ENTRY(ctypes.Structure):
    _fields_ = [
        ("Next", ctypes.c_ulonglong),
    ]
    Next: int

class _IMAGE_DYNAMIC_RELOCATION32(ctypes.Structure):
    _fields_ = [
        ("Symbol", ctypes.c_uint32),
        ("BaseRelocSize", ctypes.c_uint32),
    ]
    Symbol: int
    BaseRelocSize: int

class hknpCharacterProxyViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class HSTR__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _CONSOLE_READCONSOLE_CONTROL(ctypes.Structure):
    _fields_ = [
        ("nLength", ctypes.c_uint32),
        ("nInitialChars", ctypes.c_uint32),
        ("dwCtrlWakeupMask", ctypes.c_uint32),
        ("dwControlKeyState", ctypes.c_uint32),
    ]
    nLength: int
    nInitialChars: int
    dwCtrlWakeupMask: int
    dwControlKeyState: int

class tagANIMATIONINFO(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("iMinAnimate", ctypes.c_int32),
    ]
    cbSize: int
    iMinAnimate: int

class HSAVEDUILANGUAGES__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class _ACTIVATION_CONTEXT_DETAILED_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("dwFlags", ctypes.c_uint32),
        ("ulFormatVersion", ctypes.c_uint32),
        ("ulAssemblyCount", ctypes.c_uint32),
        ("ulRootManifestPathType", ctypes.c_uint32),
        ("ulRootManifestPathChars", ctypes.c_uint32),
        ("ulRootConfigurationPathType", ctypes.c_uint32),
        ("ulRootConfigurationPathChars", ctypes.c_uint32),
        ("ulAppDirPathType", ctypes.c_uint32),
        ("ulAppDirPathChars", ctypes.c_uint32),
        ("lpRootManifestPath", ctypes.c_ulonglong),
        ("lpRootConfigurationPath", ctypes.c_ulonglong),
        ("lpAppDirPath", ctypes.c_ulonglong),
    ]
    dwFlags: int
    ulFormatVersion: int
    ulAssemblyCount: int
    ulRootManifestPathType: int
    ulRootManifestPathChars: int
    ulRootConfigurationPathType: int
    ulRootConfigurationPathChars: int
    ulAppDirPathType: int
    ulAppDirPathChars: int
    lpRootManifestPath: int
    lpRootConfigurationPath: int
    lpAppDirPath: int

class _ABC(ctypes.Structure):
    _fields_ = [
        ("abcA", ctypes.c_int32),
        ("abcB", ctypes.c_uint32),
        ("abcC", ctypes.c_int32),
    ]
    abcA: int
    abcB: int
    abcC: int

class cTkSystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkSystemBase_dtor_0", ctypes.c_ulonglong),
        ("GetSKUID", ctypes.c_char * 0x8),  # cTkFixedString<32,char> *(__fastcall *)(cTkSystemBase *this)
        ("SetSKUID", ctypes.c_ulonglong),
        ("ConstructHardware", ctypes.c_ulonglong),
        ("ConstructVideo", ctypes.c_ulonglong),
        ("DestructHardware", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("DetectAvailableResolutions", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkSystemBase *this, int, std::vector<cTkIntTuple2<int,0>,TkSTLAllocatorShim<cTkIntTuple2<int,0>,4,-1> > *)
        ("DetectAvailableMonitors", ctypes.c_ulonglong),
        ("SwapBuffers", ctypes.c_ulonglong),
        ("GetScreenPosition", ctypes.c_char * 0x8),  # cTkIntTuple2<int,0> *(__fastcall *)(cTkSystemBase *this, cTkIntTuple2<int,0> *result)
        ("GetScreenResolution", ctypes.c_char * 0x8),  # cTkIntTuple2<int,0> *(__fastcall *)(cTkSystemBase *this, cTkIntTuple2<int,0> *result)
        ("GetWindowMode", ctypes.c_ulonglong),
        ("SetWindowMode", ctypes.c_ulonglong),
        ("SetScreenDimensions", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkSystemBase *this, const cTkIntTuple2<int,0> *, const cTkIntTuple2<int,0> *)
        ("GetResourceRemoveFrameDelay", ctypes.c_ulonglong),
        ("GameHasFocus", ctypes.c_ulonglong),
        ("GenerateSystemSeed", ctypes.c_ulonglong),
        ("GenerateUUIDString", ctypes.c_ulonglong),
        ("CopyTextToClipboard", ctypes.c_ulonglong),
        ("PasteTextFromClipboard", ctypes.c_ulonglong),
        ("SetGamma", ctypes.c_ulonglong),
        ("GetAppId", ctypes.c_ulonglong),
        ("IsDevAppId", ctypes.c_ulonglong),
        ("IsRetailAppId", ctypes.c_ulonglong),
        ("SystemSetCursor", ctypes.c_ulonglong),
        ("HandleSystemUIOverlayActivate", ctypes.c_ulonglong),
        ("HandleSystemUIOverlayDeactivate", ctypes.c_ulonglong),
        ("HandleSystemShareMenuOpened", ctypes.c_ulonglong),
        ("HandleMoveToBackgroundExecution", ctypes.c_ulonglong),
        ("HandleReturnFromBackgroundExecution", ctypes.c_ulonglong),
    ]
    cTkSystemBase_dtor_0: int
    GetSKUID: bytes
    SetSKUID: int
    ConstructHardware: int
    ConstructVideo: int
    DestructHardware: int
    Update: int
    DetectAvailableResolutions: bytes
    DetectAvailableMonitors: int
    SwapBuffers: int
    GetScreenPosition: bytes
    GetScreenResolution: bytes
    GetWindowMode: int
    SetWindowMode: int
    SetScreenDimensions: bytes
    GetResourceRemoveFrameDelay: int
    GameHasFocus: int
    GenerateSystemSeed: int
    GenerateUUIDString: int
    CopyTextToClipboard: int
    PasteTextFromClipboard: int
    SetGamma: int
    GetAppId: int
    IsDevAppId: int
    IsRetailAppId: int
    SystemSetCursor: int
    HandleSystemUIOverlayActivate: int
    HandleSystemUIOverlayDeactivate: int
    HandleSystemShareMenuOpened: int
    HandleMoveToBackgroundExecution: int
    HandleReturnFromBackgroundExecution: int

class StorageSortableSlot(ctypes.Structure):
    _fields_ = [
        ("_meSlot", ctypes.c_uint32),
        ("miTimestamp", ctypes.c_int32),
    ]
    _meSlot: int
    miTimestamp: int

    @property
    def meSlot(self):
        return cTkStoragePersistent.Slot(self._meSlot)


class _PROCESS_MITIGATION_SYSTEM_CALL_FILTER_POLICY(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $1E5A5A9E2D382EC42473BEFE96491981
    ]
    ___u0: bytes

class tagCURSORSHAPE(ctypes.Structure):
    _fields_ = [
        ("xHotSpot", ctypes.c_int32),
        ("yHotSpot", ctypes.c_int32),
        ("_cx", ctypes.c_int32),
        ("cy", ctypes.c_int32),
        ("cbWidth", ctypes.c_int32),
        ("Planes", ctypes.c_uint8),
        ("BitsPixel", ctypes.c_uint8),
    ]
    xHotSpot: int
    yHotSpot: int
    _cx: int
    cy: int
    cbWidth: int
    Planes: int
    BitsPixel: int

class hknpEventMergeAndDispatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("exec", ctypes.c_ulonglong),
        ("print", ctypes.c_ulonglong),
        ("flushRemainingEvents", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    exec: int
    print: int
    flushRemainingEvents: int

class hknpCustomConstraintDrawer_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCustomConstraintDrawer_dtor_0", ctypes.c_ulonglong),
        ("drawConstraint", ctypes.c_ulonglong),
    ]
    hknpCustomConstraintDrawer_dtor_0: int
    drawConstraint: int

class _SERVICE_TABLE_ENTRYW(ctypes.Structure):
    _fields_ = [
        ("lpServiceName", ctypes.c_ulonglong),
        ("lpServiceProc", ctypes.c_ulonglong),
    ]
    lpServiceName: int
    lpServiceProc: int

class hknpConstraintViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class tagMINIMIZEDMETRICS(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("iWidth", ctypes.c_int32),
        ("iHorzGap", ctypes.c_int32),
        ("iVertGap", ctypes.c_int32),
        ("iArrange", ctypes.c_int32),
    ]
    cbSize: int
    iWidth: int
    iHorzGap: int
    iVertGap: int
    iArrange: int

class hknpCapsuleCapsuleConvexConvexManifoldGenerator_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    collide: int

class _PSFEATURE_CUSTPAPER(ctypes.Structure):
    _fields_ = [
        ("lOrientation", ctypes.c_int32),
        ("lWidth", ctypes.c_int32),
        ("lHeight", ctypes.c_int32),
        ("lWidthOffset", ctypes.c_int32),
        ("lHeightOffset", ctypes.c_int32),
    ]
    lOrientation: int
    lWidth: int
    lHeight: int
    lWidthOffset: int
    lHeightOffset: int

class hknpSingleThreadedSimulation_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpSimulation_dtor_0", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
        ("solve", ctypes.c_ulonglong),
        ("exportStatisticsToMonitorStream", ctypes.c_ulonglong),
        ("checkConsistency", ctypes.c_ulonglong),
    ]
    hknpSimulation_dtor_0: int
    collide: int
    solve: int
    exportStatisticsToMonitorStream: int
    checkConsistency: int

class hknpCollisionCacheConsumer(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x30),  # hkTypedBlockStream<hknpCollisionCache>::Consumer
    ]
    baseclass_0: bytes

class hknpPrepareNarrowPhaseTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpDeactivationViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpEventsViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class _MEMORYSTATUS(ctypes.Structure):
    _fields_ = [
        ("dwLength", ctypes.c_uint32),
        ("dwMemoryLoad", ctypes.c_uint32),
        ("dwTotalPhys", ctypes.c_uint64),
        ("dwAvailPhys", ctypes.c_uint64),
        ("dwTotalPageFile", ctypes.c_uint64),
        ("dwAvailPageFile", ctypes.c_uint64),
        ("dwTotalVirtual", ctypes.c_uint64),
        ("dwAvailVirtual", ctypes.c_uint64),
    ]
    dwLength: int
    dwMemoryLoad: int
    dwTotalPhys: int
    dwAvailPhys: int
    dwTotalPageFile: int
    dwAvailPageFile: int
    dwTotalVirtual: int
    dwAvailVirtual: int

class ToXMLStringTool(ctypes.Structure):
    _fields_ = [
        ("buf", ctypes.c_char_p),
        ("buflen", ctypes.c_int32),
    ]
    buf: bytes
    buflen: int

class _SERVICE_PRESHUTDOWN_INFO(ctypes.Structure):
    _fields_ = [
        ("dwPreshutdownTimeout", ctypes.c_uint32),
    ]
    dwPreshutdownTimeout: int

class HSPRITE__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class hknpTaskContext(ctypes.Structure):
    _fields_ = [
        ("m_monitorStream", ctypes.c_ulonglong),
    ]
    m_monitorStream: int

class hknpApiCommandProcessor_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("exec", ctypes.c_ulonglong),
        ("print", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    exec: int
    print: int

class hknpCdPairWriter(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x28),  # hkTypedBlockStream<hknpMotionIdPair>::Writer
    ]
    baseclass_0: bytes

class _SERVICE_PREFERRED_NODE_INFO(ctypes.Structure):
    _fields_ = [
        ("usPreferredNode", ctypes.c_uint16),
        ("fDelete", ctypes.c_uint8),
    ]
    usPreferredNode: int
    fDelete: int

class cTkHalfVector2(ctypes.Structure):
    _fields_ = [
        ("mfX", ctypes.c_uint16),
        ("mfY", ctypes.c_uint16),
    ]
    mfX: int
    mfY: int

class UniformSample(ctypes.Structure):
    pass

class cTkScopedMutexLock(ctypes.Structure):
    _fields_ = [
        ("mrMutex", ctypes.c_ulonglong),
    ]
    mrMutex: int

class cTkSpinMutexScoped(ctypes.Structure):
    def cTkSpinMutexScoped(): pass

    _fields_ = [
        ("mMutex", ctypes.c_ulonglong),
    ]
    mMutex: int

class cTkFilename(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    baseclass_0: bytes

class hknpHeatMapViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class _ICONINFO(ctypes.Structure):
    _fields_ = [
        ("fIcon", ctypes.c_int32),
        ("xHotspot", ctypes.c_uint32),
        ("yHotspot", ctypes.c_uint32),
        ("hbmMask", ctypes.c_ulonglong),
        ("hbmColor", ctypes.c_ulonglong),
    ]
    fIcon: int
    xHotspot: int
    yHotspot: int
    hbmMask: int
    hbmColor: int

class IMPORT_OBJECT_HEADER(ctypes.Structure):
    _fields_ = [
        ("Sig1", ctypes.c_uint16),
        ("Sig2", ctypes.c_uint16),
        ("Version", ctypes.c_uint16),
        ("Machine", ctypes.c_uint16),
        ("TimeDateStamp", ctypes.c_uint32),
        ("SizeOfData", ctypes.c_uint32),
        ("___u6", ctypes.c_char * 0x2),  # $4CFA15F5CE489CDF0BD78DFB308FEF36
        ("_bf_12", ctypes.c_int16),
    ]
    Sig1: int
    Sig2: int
    Version: int
    Machine: int
    TimeDateStamp: int
    SizeOfData: int
    ___u6: bytes
    _bf_12: int

class hknpManifoldViewerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
        ("processManifoldStream", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpManifoldViewerBase *this, const hknpWorld *, struct hkTypedBlockStream<hknpManifoldViewerBase::VdbManifoldWithId>::Reader *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int
    processManifoldStream: bytes

class COPYFILE2_EXTENDED_PARAMETERS(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwCopyFlags", ctypes.c_uint32),
        ("pfCancel", ctypes.c_ulonglong),
        ("pProgressRoutine", ctypes.c_ulonglong),
        ("pvCallbackContext", ctypes.c_void_p),
    ]
    dwSize: int
    dwCopyFlags: int
    pfCancel: int
    pProgressRoutine: int
    pvCallbackContext: int

class XMLParserBase64Tool(ctypes.Structure):
    def encode(): pass

    def decode(): pass

    _fields_ = [
        ("buf", ctypes.c_void_p),
        ("buflen", ctypes.c_int32),
    ]
    buf: int
    buflen: int

class hknpManifoldViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
        ("processManifoldStream", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpManifoldViewerBase *this, const hknpWorld *, struct hkTypedBlockStream<hknpManifoldViewerBase::VdbManifoldWithId>::Reader *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int
    processManifoldStream: bytes

class hknpMassPropertiesViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class rmp_seg(ctypes.Structure):
    _fields_ = [
        ("rmp_gpa", ctypes.c_uint64),
        ("rmp_entry", ctypes.c_char),
        ("rmp_pageSize", ctypes.c_char),
        ("rmp_pageMark", ctypes.c_char),
        ("rmp_reserved", ctypes.c_char),
        ("rmp_ASID", ctypes.c_int32),
    ]
    rmp_gpa: int
    rmp_entry: str
    rmp_pageSize: str
    rmp_pageMark: str
    rmp_reserved: str
    rmp_ASID: int

class hknpWorldShiftUtil(ctypes.Structure):
    def shiftWorld(): pass


class DISK_SPACE_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("ActualTotalAllocationUnits", ctypes.c_uint64),
        ("ActualAvailableAllocationUnits", ctypes.c_uint64),
        ("ActualPoolUnavailableAllocationUnits", ctypes.c_uint64),
        ("CallerTotalAllocationUnits", ctypes.c_uint64),
        ("CallerAvailableAllocationUnits", ctypes.c_uint64),
        ("CallerPoolUnavailableAllocationUnits", ctypes.c_uint64),
        ("UsedAllocationUnits", ctypes.c_uint64),
        ("TotalReservedAllocationUnits", ctypes.c_uint64),
        ("VolumeStorageReserveAllocationUnits", ctypes.c_uint64),
        ("AvailableCommittedAllocationUnits", ctypes.c_uint64),
        ("PoolAvailableAllocationUnits", ctypes.c_uint64),
        ("SectorsPerAllocationUnit", ctypes.c_uint32),
        ("BytesPerSector", ctypes.c_uint32),
    ]
    ActualTotalAllocationUnits: int
    ActualAvailableAllocationUnits: int
    ActualPoolUnavailableAllocationUnits: int
    CallerTotalAllocationUnits: int
    CallerAvailableAllocationUnits: int
    CallerPoolUnavailableAllocationUnits: int
    UsedAllocationUnits: int
    TotalReservedAllocationUnits: int
    VolumeStorageReserveAllocationUnits: int
    AvailableCommittedAllocationUnits: int
    PoolAvailableAllocationUnits: int
    SectorsPerAllocationUnit: int
    BytesPerSector: int

class IMAGE_ARM64_RUNTIME_FUNCTION_ENTRY_XDATA: pass

class cTkMath(ctypes.Structure):
    pass

class hknpParticleSpheresViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpParticleCacheViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpParticlesShapeViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpParticlePair(ctypes.Structure):
    _fields_ = [
        ("m_indexB", ctypes.c_uint32),
        ("m_indexA", ctypes.c_uint32),
    ]
    m_indexB: int
    m_indexA: int

class hknpWorldQueryViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpShapeCastViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpShapeViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
        ("addWorld", ctypes.c_ulonglong),
        ("removeWorld", ctypes.c_ulonglong),
        ("isWorldAdded", ctypes.c_ulonglong),
        ("addBody", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("refreshBody", ctypes.c_ulonglong),
        ("refreshAllBodies", ctypes.c_ulonglong),
        ("setTransformModifier", ctypes.c_ulonglong),
        ("precreateDisplayGeometryForShape", ctypes.c_ulonglong),
        ("getPrecreatedDisplayGeometriesForShape", ctypes.c_char * 0x8),  # struct hkArray<hkDisplayGeometry *,hkBuiltinContainerAllocator<3,0> > *(__fastcall *)(hknpShapeViewer *this, const hknpShape *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int
    addWorld: int
    removeWorld: int
    isWorldAdded: int
    addBody: int
    removeBody: int
    refreshBody: int
    refreshAllBodies: int
    setTransformModifier: int
    precreateDisplayGeometryForShape: int
    getPrecreatedDisplayGeometriesForShape: bytes

class hknpInternalMotionUtil(ctypes.Structure):
    pass

class hknpSolverIntegrator(ctypes.Structure):
    def subIntegrate(): pass

    def subIntegrateLastStep(): pass


class SpecialMethods(ctypes.Structure):
    _fields_ = [
        ("m_defCtor", ctypes.c_ulonglong),
        ("m_copyCtor", ctypes.c_ulonglong),
        ("m_copyAssign", ctypes.c_ulonglong),
    ]
    m_defCtor: int
    m_copyCtor: int
    m_copyAssign: int

class hknpContigCacheConsumers(ctypes.Structure):
    _fields_ = [
        ("m_consumers", ctypes.c_ulonglong),
    ]
    m_consumers: int

class hknpWorldApiCommandViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpCacheStarConsumers(ctypes.Structure):
    _fields_ = [
        ("m_cacheStarStream", ctypes.c_char * 0x8),  # hkTypedBlockStream<hknpBlockStreamBlockOffsetPair>::Consumer *
        ("m_parentCacheConsumer", ctypes.c_ulonglong),
        ("m_childCacheConsumer", ctypes.c_ulonglong),
    ]
    m_cacheStarStream: bytes
    m_parentCacheConsumer: int
    m_childCacheConsumer: int

class hknpCollidePipeline(ctypes.Structure):
    def buildJacobiansForManifoldRange(): pass

    def collideStealingWork(): pass

    def mergeAndCollide2Streams(): pass


class hknpWorldInspectorViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpImpulseLimitBreachedElem(ctypes.Structure):
    _fields_ = [
        ("m_type", ctypes.c_uint16),
        ("m_isContact", ctypes.c_uint16),
        ("m_solverResult", ctypes.c_ulonglong),
    ]
    m_type: int
    m_isContact: int
    m_solverResult: int

class hknpImpulseLimitBreachedHeader(ctypes.Structure):
    _fields_ = [
        ("m_numBreached", ctypes.c_int32),
        ("m_next", ctypes.c_ulonglong),
    ]
    m_numBreached: int
    m_next: int

class hknpWorldSnapshot_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpTimerVolumeEndSchema(ctypes.Structure):
    _fields_ = [
        ("m_typeBuffer", ctypes.c_float),
    ]
    m_typeBuffer: float

class ConstraintGroupSetter(ctypes.Structure):
    _fields_ = [
        ("m_currGroup", ctypes.c_ulonglong),
        ("m_currGroupForDirectSolving", ctypes.c_ulonglong),
        ("m_currGroupId", ctypes.c_char * 0x4),  # hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant>
        ("m_currGroupIdForDirectSolving", ctypes.c_char * 0x4),  # hkHandle<unsigned int,2147483647,hknpConstraintGroupIdDiscriminant>
        ("m_constraintGroups", ctypes.c_ulonglong),
        ("m_constraints", ctypes.c_ulonglong),
        ("m_numIndices", ctypes.c_int32),
        ("m_isSolvedByTheDirectGroupSolver", ctypes.c_ubyte),
    ]
    m_currGroup: int
    m_currGroupForDirectSolving: int
    m_currGroupId: bytes
    m_currGroupIdForDirectSolving: bytes
    m_constraintGroups: int
    m_constraints: int
    m_numIndices: int
    m_isSolvedByTheDirectGroupSolver: bool

class hknpCollisionCacheModifier(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # hkTypedBlockStream<hknpCollisionCache>::Modifier
    ]
    baseclass_0: bytes

class hknpGsk2D(ctypes.Structure):
    def getClosestPoints(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x100),  # hkcdGskImpl<hknpGsk2DShapeInterface>
    ]
    baseclass_0: bytes

class hknpCollisionCacheSorter(ctypes.Structure):
    def sortCaches(): pass


class hknpConnectivityGraph_CompressedIntArray_IntSize_Auto(ctypes.Structure):
    pass

class hknpFaceClipConvexConvexManifoldGenerator_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("collide", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    collide: int

class hknpManifoldEvent_Status_Auto(ctypes.Structure):
    pass

class hknpTriggerEvent_Status_Auto(ctypes.Structure):
    pass

class hknpCollisionFilter_Type_Auto(ctypes.Structure):
    pass

class hknpJacobianHeaderSchemaDeprecated(ctypes.Structure):
    _fields_ = [
        ("m_padding0", ctypes.c_uint8),
        ("m_solverResultStriding", ctypes.c_uint8),
        ("m_padding1", ctypes.c_uint16),
        ("m_bodyAIndex", ctypes.c_uint16),
        ("m_bodyBIndex", ctypes.c_uint16),
        ("m_solverResultInMainMemory", ctypes.c_ulonglong),
        ("m_padding3", ctypes.c_uint32),
        ("m_padding4", ctypes.c_uint32),
    ]
    m_padding0: int
    m_solverResultStriding: int
    m_padding1: int
    m_bodyAIndex: int
    m_bodyBIndex: int
    m_solverResultInMainMemory: int
    m_padding3: int
    m_padding4: int

class hknpJacobianGotoSchema(ctypes.Structure):
    _fields_ = [
        ("m_padding0", ctypes.c_uint32),
        ("m_padding1", ctypes.c_uint32),
        ("m_offset", ctypes.c_int64),
    ]
    m_padding0: int
    m_padding1: int
    m_offset: int

class hknpJacobianShiftSolverResultsPointerSchema(ctypes.Structure):
    _fields_ = [
        ("m_padding_1", ctypes.c_uint32),
        ("m_numSolverResultsToSkip", ctypes.c_uint8),
        ("m_padding2", ctypes.c_uint32 * 0x2),  # unsigned int[2]
    ]
    m_padding_1: int
    m_numSolverResultsToSkip: int
    m_padding2: list[ctypes.c_uint32]

class hknpCompoundShapeBoundingVolumeType_Enum_Auto(ctypes.Structure):
    pass

class hknpSortFinalizeNarrowPhasePairsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpFinalizeNarrowPhasePairsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpHeightFieldGeometry_QuadType_Auto(ctypes.Structure):
    pass

class hknpApplyDragTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpStaticBodyResetUtil(ctypes.Structure):
    _fields_ = [
        ("m_previousAabbs", ctypes.c_char * 0x10),  # hkArrayView<hkAabbFloat3>
        ("m_bodies", ctypes.c_char * 0x10),  # hkArrayView<hknpBody>
        ("m_broadphase", ctypes.c_ulonglong),
        ("m_batchedIds", ctypes.c_char * 0x410),  # hkInplaceArray<hknpBodyIndex,256,hkBuiltinContainerAllocator<0,0> >
    ]
    m_previousAabbs: bytes
    m_bodies: bytes
    m_broadphase: int
    m_batchedIds: bytes

class hknpFinalizeNarrowPhaseGridTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpDeactivationMaintenanceTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpAddActiveBodyPairsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpSolverFactoryTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpCollideSyncTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class IsaInfo(ctypes.Structure):
    _fields_ = [
        ("m_type", ctypes.c_ulonglong),
        ("m_parent", ctypes.c_int32),
        ("m_firstChild", ctypes.c_int32),
        ("m_nextSibling", ctypes.c_int32),
    ]
    m_type: int
    m_parent: int
    m_firstChild: int
    m_nextSibling: int

class hknpShapeTagCodec_Type_Auto(ctypes.Structure):
    pass

class hknpShapeTagCodec_Hints_Auto(ctypes.Structure):
    pass

class hknpShape_FlagsEnum_Auto(ctypes.Structure):
    pass

class hknpShape_ConvexRadiusDisplayMode_Auto(ctypes.Structure):
    pass

class hknpShape_MassConfig_Quality_Auto(ctypes.Structure):
    pass

class hknpShapeKeyMask_Type_Enum_Auto(ctypes.Structure):
    pass

class _Cvtvec(ctypes.Structure):
    _fields_ = [
        ("_Page", ctypes.c_uint32),
        ("_Mbcurmax", ctypes.c_uint32),
        ("_Isclocale", ctypes.c_int32),
        ("_Isleadbyte", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
    ]
    _Page: int
    _Mbcurmax: int
    _Isclocale: int
    _Isleadbyte: list[ctypes.c_uint8]

class _Ctypevec(ctypes.Structure):
    _fields_ = [
        ("_Page", ctypes.c_uint32),
        ("_Table", ctypes.c_ulonglong),
        ("_Delfl", ctypes.c_int32),
        ("_LocaleName", ctypes.c_ulonglong),
    ]
    _Page: int
    _Table: int
    _Delfl: int
    _LocaleName: int

class _Collvec(ctypes.Structure):
    _fields_ = [
        ("_Page", ctypes.c_uint32),
        ("_LocaleName", ctypes.c_ulonglong),
    ]
    _Page: int
    _LocaleName: int

class hknpShapeTransform_FlagsEnum_Auto(ctypes.Structure):
    pass

class hknpShapeType_Enum_Auto(ctypes.Structure):
    pass

class hknpCollisionDispatchType_Enum_Auto(ctypes.Structure):
    pass

class hknpCollisionFlags_Enum_Auto(ctypes.Structure):
    pass

class hknpCollisionControl_Enum_Auto(ctypes.Structure):
    pass

class hknpBody_WorldAddedState_Auto(ctypes.Structure):
    pass

class hknpParticleMeshCache(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x150),  # hknpParticleNmpCache<1>
    ]
    baseclass_0: bytes

class hknpParticleSdfCache(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x150),  # hknpParticleNmpCache<4>
    ]
    baseclass_0: bytes

class hknpParticleWorldCache(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x150),  # hknpParticleNmpCache<16>
    ]
    baseclass_0: bytes

class ScopeCount: pass

class hknpBodyQuality_FlagsEnum_Auto(ctypes.Structure):
    pass

class hknpBodyQualityId_Preset_Auto(ctypes.Structure):
    pass

class hknpConstraintAtom_AtomType_Auto(ctypes.Structure):
    pass

class hknpConstraintAtom_BodySpaceIndex_Auto(ctypes.Structure):
    pass

class hknpConstraintAtomStiffness_Type_Auto(ctypes.Structure):
    pass

class hknpSetupStabilizationAtom_m_maxLinImpulse_Auto(ctypes.Structure):
    pass

class hknpSetupStabilizationAtom_m_maxAngImpulse_Auto(ctypes.Structure):
    pass

class hknpSetupStabilizationAtom_m_maxAngle_Auto(ctypes.Structure):
    pass

class hknp3dAngConstraintAtom_m_tauFactor_Auto(ctypes.Structure):
    pass

class hknp3dAngConstraintAtom_m_dampingFactor_Auto(ctypes.Structure):
    pass

class hknpBallSocketConstraintAtom_m_enableLinearImpulseLimit_Auto(ctypes.Structure):
    pass

class URL_COMPONENTS(ctypes.Structure):
    _fields_ = [
        ("dwStructSize", ctypes.c_uint32),
        ("lpszScheme", ctypes.c_ulonglong),
        ("dwSchemeLength", ctypes.c_uint32),
        ("nScheme", ctypes.c_int32),
        ("lpszHostName", ctypes.c_ulonglong),
        ("dwHostNameLength", ctypes.c_uint32),
        ("nPort", ctypes.c_uint16),
        ("lpszUserName", ctypes.c_ulonglong),
        ("dwUserNameLength", ctypes.c_uint32),
        ("lpszPassword", ctypes.c_ulonglong),
        ("dwPasswordLength", ctypes.c_uint32),
        ("lpszUrlPath", ctypes.c_ulonglong),
        ("dwUrlPathLength", ctypes.c_uint32),
        ("lpszExtraInfo", ctypes.c_ulonglong),
        ("dwExtraInfoLength", ctypes.c_uint32),
    ]
    dwStructSize: int
    lpszScheme: int
    dwSchemeLength: int
    nScheme: int
    lpszHostName: int
    dwHostNameLength: int
    nPort: int
    lpszUserName: int
    dwUserNameLength: int
    lpszPassword: int
    dwPasswordLength: int
    lpszUrlPath: int
    dwUrlPathLength: int
    lpszExtraInfo: int
    dwExtraInfoLength: int

class hknpBallSocketConstraintAtom_m_tauFactor_Auto(ctypes.Structure):
    pass

class hknpBallSocketConstraintAtom_m_dampingFactor_Auto(ctypes.Structure):
    pass

class hknpBallSocketConstraintAtom_m_inertiaStabilizationFactor_Auto(ctypes.Structure):
    pass

class hknpStiffSpringConstraintAtom_m_springConstant_Auto(ctypes.Structure):
    pass

class hknpStiffSpringConstraintAtom_m_springDamping_Auto(ctypes.Structure):
    pass

class hknpStiffSpringConstraintAtom_m_tauFactor_Auto(ctypes.Structure):
    pass

class hknpStiffSpringConstraintAtom_m_dampingFactor_Auto(ctypes.Structure):
    pass

class hknpStiffSpringConstraintAtom_ProjectionAxis_Auto(ctypes.Structure):
    pass

class hknpConstraintViewerEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("worldAddedCallback", ctypes.c_ulonglong),
        ("worldRemovedCallback", ctypes.c_ulonglong),
        ("addToObjectHandler", ctypes.c_ulonglong),
        ("removeFromObjectHandler", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    worldAddedCallback: int
    worldRemovedCallback: int
    addToObjectHandler: int
    removeFromObjectHandler: int

class hknpEllipticalLimitConstraintAtom_m_angularLimitsTauFactor_Auto(ctypes.Structure):
    pass

class hknpEllipticalLimitConstraintAtom_m_angularLimitsDampFactor_Auto(ctypes.Structure):
    pass

class hknpRagdollMotorConstraintAtom_RagdollMotorMode_Auto(ctypes.Structure):
    pass

class hknpPulleyConstraintAtom_m_tauFactor_Auto(ctypes.Structure):
    pass

class hknpPulleyConstraintAtom_m_dampingFactor_Auto(ctypes.Structure):
    pass

class hknpRackAndPinionConstraintAtom_m_tauFactor_Auto(ctypes.Structure):
    pass

class hknpRackAndPinionConstraintAtom_m_dampingFactor_Auto(ctypes.Structure):
    pass

class hknpCogWheelConstraintAtom_m_tauFactor_Auto(ctypes.Structure):
    pass

class hknpCogWheelConstraintAtom_m_dampingFactor_Auto(ctypes.Structure):
    pass

class hknp6DofConstraintData_AxisMode_Auto(ctypes.Structure):
    pass

class hknpCharacterStateInAir_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("update", ctypes.c_ulonglong),
        ("getNextState", ctypes.c_ulonglong),
        ("enterState", ctypes.c_ulonglong),
        ("leaveState", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    update: int
    getNextState: int
    enterState: int
    leaveState: int

class hknpPointToPathConstraintData_OrientationConstraintType_Auto(ctypes.Structure):
    pass

class hkGeometryUtility(ctypes.Structure):
    def createMinBoundingSphere(): pass


class BodyMovement(ctypes.Structure):
    _fields_ = [
        ("m_linearVelocity", common.Vector4f),
        ("m_linearDelta", common.Vector4f),
        ("m_angularDelta", common.Vector4f),
    ]
    m_linearVelocity: common.Vector4f
    m_linearDelta: common.Vector4f
    m_angularDelta: common.Vector4f

class hknpConstraintData_ConstraintType_Auto(ctypes.Structure):
    pass

class recollideJacobiansImpl(ctypes.Structure):
    pass

class hknpCallbackConstraintMotor_m_userData0_Auto(ctypes.Structure):
    pass

class hknpCallbackConstraintMotor_m_userData1_Auto(ctypes.Structure):
    pass

class hknpCallbackConstraintMotor_m_userData2_Auto(ctypes.Structure):
    pass

class hknpCallbackConstraintMotor_CallbackType_Auto(ctypes.Structure):
    pass

class hknpConstraintMotor_MotorType_Auto(ctypes.Structure):
    pass

class hknpConstraint_FlagsEnum_Auto(ctypes.Structure):
    pass

class hknpConstraintGroupDirectSolverSettings_MatrixType_Auto(ctypes.Structure):
    pass

class hknpConstraintGroupDirectSolverSettings_CallFrequency_Auto(ctypes.Structure):
    pass

class hknpConstraintGroupDirectSolverSettings_MotorInclusion_Auto(ctypes.Structure):
    pass

class hknpMaterial_TriggerType_Auto(ctypes.Structure):
    pass

class hknpMaterial_CombinePolicy_Auto(ctypes.Structure):
    pass

class hknpMaterial_MassChangerCategory_Auto(ctypes.Structure):
    pass

class hknpLinearSurfaceVelocity_ProjectMethod_Auto(ctypes.Structure):
    pass

class hknpSurfaceVelocity_Space_Auto(ctypes.Structure):
    pass

class hknpMotionCinfo_m_enableDeactivation_Auto(ctypes.Structure):
    pass

class hknpMotionCinfo_m_centerOfMassWorld_Auto(ctypes.Structure):
    pass

class hknpMotionCinfo_m_inverseMass_Auto(ctypes.Structure):
    pass

class hknpMotionCinfo_m_orientation_Auto(ctypes.Structure):
    pass

class hknpMotionCinfo_m_linearVelocity_Auto(ctypes.Structure):
    pass

class hknpMotionCinfo_m_angularVelocityLocal_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_m_gravityFactor_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_m_timeFactor_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_m_linearDamping_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_m_angularDamping_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_m_solverStabilizationSpeedThreshold_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_m_solverStabilizationSpeedReduction_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_SolverStabilizationType_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_DeactivationSettings_Strategy_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_minSeparation_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_minExtraSeparation_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_toiSeparation_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_toiExtraSeparation_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_toiAccuracy_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_relativeSafeDeltaTime_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_absoluteSafeDeltaTime_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_keepTime_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_keepDistance_Auto(ctypes.Structure):
    pass

class hknpMotionProperties_FullCastSettings_m_maxIterations_Auto(ctypes.Structure):
    pass

class hknpMotionPropertiesId_Preset_Auto(ctypes.Structure):
    pass

class hknpCharacterStateClimbing_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("update", ctypes.c_ulonglong),
        ("getNextState", ctypes.c_ulonglong),
        ("enterState", ctypes.c_ulonglong),
        ("leaveState", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    update: int
    getNextState: int
    enterState: int
    leaveState: int

class hknpCharacterStateFlying_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("update", ctypes.c_ulonglong),
        ("getNextState", ctypes.c_ulonglong),
        ("enterState", ctypes.c_ulonglong),
        ("leaveState", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    update: int
    getNextState: int
    enterState: int
    leaveState: int

class hknpCharacterStateJumping_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("update", ctypes.c_ulonglong),
        ("getNextState", ctypes.c_ulonglong),
        ("enterState", ctypes.c_ulonglong),
        ("leaveState", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    update: int
    getNextState: int
    enterState: int
    leaveState: int

class hknpContactImpulseEvent_Status_Auto(ctypes.Structure):
    pass

class __std_fs_filetime(ctypes.Structure):
    _fields_ = [
        ("_Low", ctypes.c_uint32),
        ("_High", ctypes.c_uint32),
    ]
    _Low: int
    _High: int

class __std_fs_reparse_data_buffer(ctypes.Structure):
    _fields_ = [
        ("_Reparse_tag", ctypes.c_uint32),
        ("_Reparse_data_length", ctypes.c_uint16),
        ("_Reserved", ctypes.c_uint16),
        ("___u3", ctypes.c_char * 0x10),  # $844A746ED22E65456285E74D3316D17B
    ]
    _Reparse_tag: int
    _Reparse_data_length: int
    _Reserved: int
    ___u3: bytes

class CellInfo(ctypes.Structure):
    _fields_ = [
        ("m_numElements", ctypes.c_int32),
        ("m_cellIndex", ctypes.c_int32),
    ]
    m_numElements: int
    m_cellIndex: int

class SortByLinkCost(ctypes.Structure):
    _fields_ = [
        ("m_costs", ctypes.c_ulonglong),
        ("m_split", ctypes.c_ulonglong),
    ]
    m_costs: int
    m_split: int

class hknpSetBodyMassCommand_Mode_Auto(ctypes.Structure):
    pass

class lconv(ctypes.Structure):
    _fields_ = [
        ("decimal_point", ctypes.c_char_p),
        ("thousands_sep", ctypes.c_char_p),
        ("grouping", ctypes.c_char_p),
        ("int_curr_symbol", ctypes.c_char_p),
        ("currency_symbol", ctypes.c_char_p),
        ("mon_decimal_point", ctypes.c_char_p),
        ("mon_thousands_sep", ctypes.c_char_p),
        ("mon_grouping", ctypes.c_char_p),
        ("positive_sign", ctypes.c_char_p),
        ("negative_sign", ctypes.c_char_p),
        ("int_frac_digits", ctypes.c_char),
        ("frac_digits", ctypes.c_char),
        ("p_cs_precedes", ctypes.c_char),
        ("p_sep_by_space", ctypes.c_char),
        ("n_cs_precedes", ctypes.c_char),
        ("n_sep_by_space", ctypes.c_char),
        ("p_sign_posn", ctypes.c_char),
        ("n_sign_posn", ctypes.c_char),
        ("_W_decimal_point", ctypes.c_ulonglong),
        ("_W_thousands_sep", ctypes.c_ulonglong),
        ("_W_int_curr_symbol", ctypes.c_ulonglong),
        ("_W_currency_symbol", ctypes.c_ulonglong),
        ("_W_mon_decimal_point", ctypes.c_ulonglong),
        ("_W_mon_thousands_sep", ctypes.c_ulonglong),
        ("_W_positive_sign", ctypes.c_ulonglong),
        ("_W_negative_sign", ctypes.c_ulonglong),
    ]
    decimal_point: bytes
    thousands_sep: bytes
    grouping: bytes
    int_curr_symbol: bytes
    currency_symbol: bytes
    mon_decimal_point: bytes
    mon_thousands_sep: bytes
    mon_grouping: bytes
    positive_sign: bytes
    negative_sign: bytes
    int_frac_digits: str
    frac_digits: str
    p_cs_precedes: str
    p_sep_by_space: str
    n_cs_precedes: str
    n_sep_by_space: str
    p_sign_posn: str
    n_sign_posn: str
    _W_decimal_point: int
    _W_thousands_sep: int
    _W_int_curr_symbol: int
    _W_currency_symbol: int
    _W_mon_decimal_point: int
    _W_mon_thousands_sep: int
    _W_positive_sign: int
    _W_negative_sign: int

class hknpCachedVertexReader(ctypes.Structure):
    _fields_ = [
        ("m_currentCache", ctypes.c_ulonglong),
        ("m_cacheWasCopied", ctypes.c_ubyte),
        ("m_nextVertexIndex", ctypes.c_int32),
        ("m_srcChildCacheConsumer", ctypes.c_ulonglong),
    ]
    m_currentCache: int
    m_cacheWasCopied: bool
    m_nextVertexIndex: int
    m_srcChildCacheConsumer: int

class hknpJacobianSetMassSchema(ctypes.Structure):
    _fields_ = [
        ("m_invMass0", common.Vector4f),
        ("m_invMass1", common.Vector4f),
    ]
    m_invMass0: common.Vector4f
    m_invMass1: common.Vector4f

class hknpJacobianSetCenterOfMassSchema(ctypes.Structure):
    _fields_ = [
        ("m_angToLinVelA", ctypes.c_char * 0x30),  # hkMatrix3Impl<float>
        ("m_angToLinVelB", ctypes.c_char * 0x30),  # hkMatrix3Impl<float>
    ]
    m_angToLinVelA: bytes
    m_angToLinVelB: bytes

class hknpWorldCinfo_m_gravity_Auto(ctypes.Structure):
    pass

class AabbHitCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class hknpWorldCinfo_SolverType_Auto(ctypes.Structure):
    pass

class hknpWorldCinfo_DeterminismCheckingMode_Auto(ctypes.Structure):
    pass

class hknpWorldWriter_AdditionMode_Auto(ctypes.Structure):
    pass

class hknpWorldWriter_ActivationMode_Auto(ctypes.Structure):
    pass

class hknpWorldWriter_UpdateCachesMode_Auto(ctypes.Structure):
    pass

class hknpWorldWriter_UpdateMotionMode_Auto(ctypes.Structure):
    pass

class hknpWorldWriter_PivotLocation_Auto(ctypes.Structure):
    pass

class hknpSpringAction_m_onCompression_Auto(ctypes.Structure):
    pass

class hknpSpringAction_m_onExtension_Auto(ctypes.Structure):
    pass

class hknpSpringAction_m_restLength_Auto(ctypes.Structure):
    pass

class hknpSpringAction_m_strength_Auto(ctypes.Structure):
    pass

class hknpSpringAction_m_damping_Auto(ctypes.Structure):
    pass

class hknpBallGun_m_bulletRadius_Auto(ctypes.Structure):
    pass

class hknpBallGun_m_bulletVelocity_Auto(ctypes.Structure):
    pass

class hknpBallGun_m_bulletMass_Auto(ctypes.Structure):
    pass

class hknpBallGun_m_damageMultiplier_Auto(ctypes.Structure):
    pass

class hknpBallGun_m_maxBulletsInWorld_Auto(ctypes.Structure):
    pass

class hknpBallGun_m_bulletOffsetFromCenter_Auto(ctypes.Structure):
    pass

class hknpBallGun_m_collisionLookAheadDistance_Auto(ctypes.Structure):
    pass

class hknpFirstPersonGun_m_name_Auto(ctypes.Structure):
    pass

class hknpFirstPersonGun_m_keyboardKey_Auto(ctypes.Structure):
    pass

class hknpFirstPersonGun_Type_Auto(ctypes.Structure):
    pass

class hknpFirstPersonGun_KeyboardKey_Auto(ctypes.Structure):
    pass

class hknpGravityGun_m_maxNumObjectsPicked_Auto(ctypes.Structure):
    pass

class hknpGravityGun_m_invMaxMassOfObjectPicked_Auto(ctypes.Structure):
    pass

class hknpGravityGun_m_maxDistOfObjectPicked_Auto(ctypes.Structure):
    pass

class hknpGravityGun_m_impulseAppliedWhenObjectNotPicked_Auto(ctypes.Structure):
    pass

class hknpGravityGun_m_throwVelocity_Auto(ctypes.Structure):
    pass

class hknpGravityGun_m_capturedObjectPosition_Auto(ctypes.Structure):
    pass

class hknpGravityGun_m_capturedObjectsOffset_Auto(ctypes.Structure):
    pass

class hknpMountedBallGun_m_position_Auto(ctypes.Structure):
    pass

class hknpProjectileGun_m_maxProjectiles_Auto(ctypes.Structure):
    pass

class hknpProjectileGun_m_reloadTime_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacter_ControlFlags_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_customShape_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_desiredBodyId_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_position_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_forwardDir_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_up_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_gravityStrength_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_mass_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_capsuleRadius_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_verticalSensitivity_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_horizontalSensitivity_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_sensivityPadX_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_sensivityPadY_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_flags_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_maxUpDownAngle_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_numFramesPerShot_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_forwardBackwardSpeedModifier_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_leftRightSpeedModifier_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_shiftRunningSpeedModifier_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_characterRb_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_context_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_initialState_Auto(ctypes.Structure):
    pass

class hknpFirstPersonCharacterCinfo_m_world_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_refreshManifoldInCheckSupport_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_presenceInWorld_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_dynamicFriction_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_staticFriction_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_keepContactTolerance_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_userData_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_keepDistance_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_contactAngleSensitivity_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_userPlanes_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_maxCharacterSpeedForSolver_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_characterStrength_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_characterMass_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_maxSlope_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_penetrationRecoverySpeed_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_maxCastIterations_Auto(ctypes.Structure):
    pass

class hknpCharacterProxyCinfo_m_shapeCastLod_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_motionPropertiesId_Auto(ctypes.Structure):
    pass

class __std_fs_file_id(ctypes.Structure):
    _fields_ = [
        ("_Volume_serial_number", ctypes.c_uint64),
        ("_Id", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
    ]
    _Volume_serial_number: int
    _Id: list[ctypes.c_uint8]

class hknpCharacterRigidBodyCinfo_m_mass_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_dynamicFriction_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_staticFriction_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_weldingTolerance_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_maxSlope_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_maxSupportSlope_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_maxForce_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_maxSpeedForSimplexSolver_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_supportDistance_Auto(ctypes.Structure):
    pass

class hknpCharacterRigidBodyCinfo_m_hardSupportDistance_Auto(ctypes.Structure):
    pass

class hknpCharacterState_hknpCharacterStateType_Auto(ctypes.Structure):
    pass

class hknpCharacterSurfaceInfo_SupportedState_Auto(ctypes.Structure):
    pass

class hknpPhysicsSystemData_ExtendedBodyCinfo_m_attachedBody_Auto(ctypes.Structure):
    pass

class hknpRagdollState_StateId_Auto(ctypes.Structure):
    pass

class hknpRagdollState_ConstraintEasingMode_Auto(ctypes.Structure):
    pass

class hkConcurrency_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpVehicleWheelCollide_WheelCollideType_Auto(ctypes.Structure):
    pass

class hknpVehicleData_m_maxFrictionSolverMassRatio_Auto(ctypes.Structure):
    pass

class hknpManifoldViewerBase_VdbManifold_StatusEnum_Auto(ctypes.Structure):
    pass

class hknpMotionType_Enum_Auto(ctypes.Structure):
    pass

class hknpActivationState_Enum_Auto(ctypes.Structure):
    pass

class hknpActivationControl_Enum_Auto(ctypes.Structure):
    pass

class hknpMotionRangeBreachPolicy_Enum_Auto(ctypes.Structure):
    pass

class hknpSimulationType_Enum_Auto(ctypes.Structure):
    pass

class hknpBroadPhaseType_Enum_Auto(ctypes.Structure):
    pass

class hknpBroadPhaseFilter_Category_Auto(ctypes.Structure):
    pass

class hknpManifoldType_Enum_Auto(ctypes.Structure):
    pass

class hknpLevelOfDetail_Auto(ctypes.Structure):
    pass

class VkViewport(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("width", ctypes.c_float),
        ("height", ctypes.c_float),
        ("minDepth", ctypes.c_float),
        ("maxDepth", ctypes.c_float),
    ]
    x: float
    y: float
    width: float
    height: float
    minDepth: float
    maxDepth: float

class TkGpuAddress(ctypes.Structure):
    _fields_ = [
        ("mBuffer", ctypes.c_ulonglong),
        ("mOffset", ctypes.c_uint64),
    ]
    mBuffer: int
    mOffset: int

class cTkVertexBufferSlot(ctypes.Structure):
    _fields_ = [
        ("muVertexBufferObject", ctypes.c_uint32),
        ("muOffset", ctypes.c_uint32),
        ("muStride", ctypes.c_uint32),
        ("miLayoutElementCount", ctypes.c_int32),
        ("miHash", ctypes.c_int64),
        ("mbIsDirty", ctypes.c_ubyte),
    ]
    muVertexBufferObject: int
    muOffset: int
    muStride: int
    miLayoutElementCount: int
    miHash: int
    mbIsDirty: bool

class cTkComponentUpdater(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("miComponentsBufferBaseAddress", ctypes.c_uint64),
        ("miComponentsBufferStride", ctypes.c_int32),
        ("miComponentsBufferElementCount", ctypes.c_int32),
    ]
    __vftable: int
    miComponentsBufferBaseAddress: int
    miComponentsBufferStride: int
    miComponentsBufferElementCount: int

class cTkComponentUpdater_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnInit", ctypes.c_ulonglong),
        ("UpdateComponents", ctypes.c_ulonglong),
        ("UpdateComponentsPostPhysics", ctypes.c_ulonglong),
        ("UpdateComponentsRender", ctypes.c_ulonglong),
        ("OnComponentAllocate", ctypes.c_ulonglong),
        ("OnComponentFree", ctypes.c_ulonglong),
        ("HasCustomUpdate", ctypes.c_ulonglong),
    ]
    OnInit: int
    UpdateComponents: int
    UpdateComponentsPostPhysics: int
    UpdateComponentsRender: int
    OnComponentAllocate: int
    OnComponentFree: int
    HasCustomUpdate: int

class simplifyPlanarHull(ctypes.Structure):
    pass

class cTkComponentInstancerBase(ctypes.Structure):
    def cTkComponentInstancerBase(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpNext", ctypes.c_ulonglong),
    ]
    __vftable: int
    mpNext: int

class cTkComponentInstancerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcTriggerActionComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class hknpParticlesClosestHitOnMeshCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class ComponentInstancercTkReferenceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cTkHavokWorld(ctypes.Structure):
    def cTkHavokWorld(): pass

    def tilde_cTkHavokWorld(): pass

    def CastLocalRaySingle(): pass

    def CastRaySingle(): pass

    def DebugDrawShape(): pass

    def Render(): pass

    def SetGravity(): pass

    def stepSimulation(): pass

    _fields_ = [
        ("mpHkWorld", ctypes.c_ulonglong),
        ("mpVdb", ctypes.c_ulonglong),
        ("mpCommonContext", ctypes.c_ulonglong),
        ("mpPhysicsContext", ctypes.c_ulonglong),
        ("mpTaskQueue", ctypes.c_ulonglong),
        ("mfHavokTimeToWorldTime", ctypes.c_float),
    ]
    mpHkWorld: int
    mpVdb: int
    mpCommonContext: int
    mpPhysicsContext: int
    mpTaskQueue: int
    mfHavokTimeToWorldTime: float

class cTkHavokGroupMaskFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterBodyPairs", ctypes.c_ulonglong),
        ("isCollisionEnabled", ctypes.c_ulonglong),
        ("isCollisionEnabled_28", ctypes.c_ulonglong),
        ("isCollisionEnabled_30", ctypes.c_ulonglong),
        ("isCollisionEnabled_38", ctypes.c_ulonglong),
        ("getLodInfo", ctypes.c_ulonglong),
        ("subscribeToWorld", ctypes.c_ulonglong),
        ("unsubscribeFromWorld", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpCollisionFilter> *(__fastcall *)(hknpCollisionFilter *this, hkRefPtr<hknpCollisionFilter> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterBodyPairs: int
    isCollisionEnabled: int
    isCollisionEnabled_28: int
    isCollisionEnabled_30: int
    isCollisionEnabled_38: int
    getLodInfo: int
    subscribeToWorld: int
    unsubscribeFromWorld: int
    copyFrom: int
    makeClone: bytes

class cTkCollisionFilter(ctypes.Structure):
    def Construct(): pass

    def Destroy(): pass

    def DisableCollisionsBetween(): pass

    def EnableCollisionsBetween(): pass

    def Prepare(): pass

    def SetVehicleBroadphaseFunc(): pass

    _fields_ = [
        ("mpPairFilter", ctypes.c_ulonglong),
        ("mpMaskFilter", ctypes.c_ulonglong),
    ]
    mpPairFilter: int
    mpMaskFilter: int

class cTkPhysicsBvMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class cTkPhysExplosion(ctypes.Structure):
    def Explode(): pass

    _fields_ = [
        ("mvPos", common.Vector3f),
        ("mfRadius", ctypes.c_float),
        ("mfMaxForceRadius", ctypes.c_float),
        ("mfForce", ctypes.c_float),
        ("miFrameCountdown", ctypes.c_int32),
    ]
    mvPos: common.Vector3f
    mfRadius: float
    mfMaxForceRadius: float
    mfForce: float
    miFrameCountdown: int

class ActiveTrigger(ctypes.Structure):
    _fields_ = [
        ("mActiveTrigger", ctypes.c_uint32),
        ("mTriggeredBy", ctypes.c_uint32),
    ]
    mActiveTrigger: int
    mTriggeredBy: int

class cTkMultipleReadersLock(ctypes.Structure):
    def UnlockForRead(): pass

    def LockForRead(): pass

    _fields_ = [
        ("miLocked", ctypes.c_char * 0x4),  # std::atomic<int>
        ("miWriteLockCount", ctypes.c_int32),
        ("miWriteLockId", ctypes.c_uint32),
    ]
    miLocked: bytes
    miWriteLockCount: int
    miWriteLockId: int

class cTkComponentLoadInstancerBase(ctypes.Structure):
    def cTkComponentLoadInstancerBase(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpNext", ctypes.c_ulonglong),
    ]
    __vftable: int
    mpNext: int

class cTkComponentLoadInstancerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercTkReferenceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cTkInputManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkInputManager_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("GetButtonInput", ctypes.c_ulonglong),
        ("GetAnalogInput", ctypes.c_ulonglong),
        ("GetLastButtonInputWasTouch", ctypes.c_ulonglong),
        ("SetTouchInputWaitingForReset", ctypes.c_ulonglong),
        ("HasDigitalActionBound", ctypes.c_ulonglong),
        ("HasAnalogActionBound", ctypes.c_ulonglong),
        ("GetDeviceVRViewResetLayerName", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cTkInputManager *this)
        ("GetActionButton", ctypes.c_ulonglong),
        ("GetActionAxis", ctypes.c_ulonglong),
        ("GetMappedIcon", ctypes.c_ulonglong),
        ("GetChordIcons", ctypes.c_char * 0x8),  # std::vector<sTkInputDeviceChordIcons,TkSTLAllocatorShim<sTkInputDeviceChordIcons,8,-1> > *(__fastcall *)(cTkInputManager *this)
        ("SetActiveActionSet", ctypes.c_ulonglong),
        ("ShowRemapScreen", ctypes.c_ulonglong),
    ]
    cTkInputManager_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    GetButtonInput: int
    GetAnalogInput: int
    GetLastButtonInputWasTouch: int
    SetTouchInputWaitingForReset: int
    HasDigitalActionBound: int
    HasAnalogActionBound: int
    GetDeviceVRViewResetLayerName: bytes
    GetActionButton: int
    GetActionAxis: int
    GetMappedIcon: int
    GetChordIcons: bytes
    SetActiveActionSet: int
    ShowRemapScreen: int

class ITkInputDevice(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class sTkActionState(ctypes.Structure):
    def SetAnalogueValue(): pass

    def SetDigitalValue(): pass

    _fields_ = [
        ("miDigitalFirstSet", ctypes.c_int32),
        ("miDigitalLastSet", ctypes.c_int32),
        ("miAnalogueLastSet", ctypes.c_int32),
        ("mfAnalogueValue", ctypes.c_float),
        ("mbAnalogueSource", ctypes.c_ubyte),
        ("miActionSetContinuity", ctypes.c_int32),
    ]
    miDigitalFirstSet: int
    miDigitalLastSet: int
    miAnalogueLastSet: int
    mfAnalogueValue: float
    mbAnalogueSource: bool
    miActionSetContinuity: int

class sTkInputDeviceIconMap(ctypes.Structure):
    def FindAxisIcons(): pass

    def FindButtonIcons(): pass

    def FindOriginButtonOrAxis(): pass

    def FindOriginIcons(): pass

    def ResolveIconSizes(): pass

    _fields_ = [
        ("mpSpec", ctypes.c_ulonglong),
        ("mButtonIcons", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,enum eInputButton,sTkInputDeviceIcons,robin_hood::hash<enum eInputButton,void>,std::equal_to<enum eInputButton>,-1>
        ("mAxisIcons", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,enum eInputAxis,sTkInputDeviceIcons,robin_hood::hash<enum eInputAxis,void>,std::equal_to<enum eInputAxis>,-1>
        ("mOriginIcons", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,cTkFixedString<32,char>,std::pair<enum eInputButton,enum eInputAxis>,UnorderedMapHashes::Hash<cTkFixedString<32,char> >,std::equal_to<cTkFixedString<32,char> >,-1>
        ("mChordIcons", ctypes.c_char * 0x18),  # std::vector<sTkInputDeviceChordIcons,TkSTLAllocatorShim<sTkInputDeviceChordIcons,8,-1> >
        ("mbIconSizesResolved", ctypes.c_ubyte),
    ]
    mpSpec: int
    mButtonIcons: bytes
    mAxisIcons: bytes
    mOriginIcons: bytes
    mChordIcons: bytes
    mbIconSizesResolved: bool

class ITkInputDevice_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkInputDevice_dtor_0", ctypes.c_ulonglong),
        ("IsAvailable", ctypes.c_ulonglong),
        ("ActivityCheck", ctypes.c_ulonglong),
        ("PullInput", ctypes.c_ulonglong),
        ("PushOutput", ctypes.c_ulonglong),
    ]
    ITkInputDevice_dtor_0: int
    IsAvailable: int
    ActivityCheck: int
    PullInput: int
    PushOutput: int

class ITkDeviceMatrixSource(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class ITkDeviceMatrixSource_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetDeviceMatrix", ctypes.c_ulonglong),
    ]
    GetDeviceMatrix: int

class cTkInputDeviceManagerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkInputDeviceManagerBase_dtor_0", ctypes.c_ulonglong),
        ("FindDevice", ctypes.c_ulonglong),
        ("LockMouse", ctypes.c_ulonglong),
        ("UnlockMouse", ctypes.c_ulonglong),
        ("HideMouse", ctypes.c_ulonglong),
        ("ShowMouse", ctypes.c_ulonglong),
        ("IsMouseHidden", ctypes.c_ulonglong),
        ("SetMousePosition", ctypes.c_ulonglong),
        ("LocaliseButton", ctypes.c_ulonglong),
        ("ProvideDeviceMatrices", ctypes.c_ulonglong),
        ("IsControllerPositionTracking", ctypes.c_ulonglong),
        ("IsGyroControlSupported", ctypes.c_ulonglong),
        ("GryoCanBeAttachedToScreen", ctypes.c_ulonglong),
        ("GyroIsAttachedToScreen", ctypes.c_ulonglong),
        ("IsUsingSplitController", ctypes.c_ulonglong),
        ("GetDeviceSpecMetadataFile", ctypes.c_ulonglong),
    ]
    cTkInputDeviceManagerBase_dtor_0: int
    FindDevice: int
    LockMouse: int
    UnlockMouse: int
    HideMouse: int
    ShowMouse: int
    IsMouseHidden: int
    SetMousePosition: int
    LocaliseButton: int
    ProvideDeviceMatrices: int
    IsControllerPositionTracking: int
    IsGyroControlSupported: int
    GryoCanBeAttachedToScreen: int
    GyroIsAttachedToScreen: int
    IsUsingSplitController: int
    GetDeviceSpecMetadataFile: int

class ITkDeviceInformationOverride(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class ITkDeviceInformationOverride_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetDeviceSpecMetadataFile", ctypes.c_ulonglong),
    ]
    GetDeviceSpecMetadataFile: int

class ScePadAnalogStick(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_uint8),
        ("y", ctypes.c_uint8),
    ]
    x: int
    y: int

class ScePadAnalogButtons(ctypes.Structure):
    _fields_ = [
        ("l2", ctypes.c_uint8),
        ("r2", ctypes.c_uint8),
        ("padding", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
    ]
    l2: int
    r2: int
    padding: list[ctypes.c_uint8]

class SceFQuaternion(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("z", ctypes.c_float),
        ("w", ctypes.c_float),
    ]
    x: float
    y: float
    z: float
    w: float

class SceFVector3(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("z", ctypes.c_float),
    ]
    x: float
    y: float
    z: float

class ScePadTouch(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_uint16),
        ("y", ctypes.c_uint16),
        ("id", ctypes.c_uint8),
        ("reserve", ctypes.c_uint8 * 0x3),  # unsigned __int8[3]
    ]
    x: int
    y: int
    id: int
    reserve: list[ctypes.c_uint8]

class ScePadExtensionUnitData(ctypes.Structure):
    _fields_ = [
        ("extensionUnitId", ctypes.c_uint32),
        ("reserve", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
        ("dataLength", ctypes.c_uint8),
        ("data", ctypes.c_uint8 * 0xA),  # unsigned __int8[10]
    ]
    extensionUnitId: int
    reserve: list[ctypes.c_uint8]
    dataLength: int
    data: list[ctypes.c_uint8]

class cTkInputDeviceManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkInputDeviceManagerBase_dtor_0", ctypes.c_ulonglong),
        ("FindDevice", ctypes.c_ulonglong),
        ("LockMouse", ctypes.c_ulonglong),
        ("UnlockMouse", ctypes.c_ulonglong),
        ("HideMouse", ctypes.c_ulonglong),
        ("ShowMouse", ctypes.c_ulonglong),
        ("IsMouseHidden", ctypes.c_ulonglong),
        ("SetMousePosition", ctypes.c_ulonglong),
        ("LocaliseButton", ctypes.c_ulonglong),
        ("ProvideDeviceMatrices", ctypes.c_ulonglong),
        ("IsControllerPositionTracking", ctypes.c_ulonglong),
        ("IsGyroControlSupported", ctypes.c_ulonglong),
        ("GryoCanBeAttachedToScreen", ctypes.c_ulonglong),
        ("GyroIsAttachedToScreen", ctypes.c_ulonglong),
        ("IsUsingSplitController", ctypes.c_ulonglong),
        ("GetDeviceSpecMetadataFile", ctypes.c_ulonglong),
    ]
    cTkInputDeviceManagerBase_dtor_0: int
    FindDevice: int
    LockMouse: int
    UnlockMouse: int
    HideMouse: int
    ShowMouse: int
    IsMouseHidden: int
    SetMousePosition: int
    LocaliseButton: int
    ProvideDeviceMatrices: int
    IsControllerPositionTracking: int
    IsGyroControlSupported: int
    GryoCanBeAttachedToScreen: int
    GyroIsAttachedToScreen: int
    IsUsingSplitController: int
    GetDeviceSpecMetadataFile: int

class sTkInputActionSetAction(ctypes.Structure):
    _fields_ = [
        ("miActionId", ctypes.c_int32),
        ("mbActionSetVisible", ctypes.c_ubyte),
        ("mbActionSetDebug", ctypes.c_ubyte),
    ]
    miActionId: int
    mbActionSetVisible: bool
    mbActionSetDebug: bool

class sTkInputActionSet(ctypes.Structure):
    _fields_ = [
        ("miActionSetId", ctypes.c_int32),
        ("msName", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("midName", ctypes.c_ulonglong),
        ("msExternalId", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("msExternalLoc", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("mbActionSetVisible", ctypes.c_ubyte),
        ("mbActionSetDebug", ctypes.c_ubyte),
        ("mActions", ctypes.c_char * 0x18),  # std::vector<sTkInputActionSetAction,TkSTLAllocatorShim<sTkInputActionSetAction,4,-1> >
    ]
    miActionSetId: int
    msName: bytes
    midName: int
    msExternalId: bytes
    msExternalLoc: bytes
    mbActionSetVisible: bool
    mbActionSetDebug: bool
    mActions: bytes

class sTkInputBindingsActionSet(ctypes.Structure):
    _fields_ = [
        ("muActionSetId", ctypes.c_int32),
        ("mBindings", ctypes.c_char * 0x18),  # std::vector<sTkInputBinding,TkSTLAllocatorShim<sTkInputBinding,4,-1> >
    ]
    muActionSetId: int
    mBindings: bytes

class sTkInputBindings(ctypes.Structure):
    _fields_ = [
        ("mActionSetBindings", ctypes.c_char * 0x18),  # std::vector<sTkInputBindingsActionSet *,TkSTLAllocatorShim<sTkInputBindingsActionSet *,8,-1> >
    ]
    mActionSetBindings: bytes

class EgRenderQueueHandle(ctypes.Structure):
    _fields_ = [
        ("miQueueId", ctypes.c_int32),
        ("miFrameId", ctypes.c_int32),
    ]
    miQueueId: int
    miFrameId: int

class cTkGraphicsManagerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkGraphicsManagerBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("NextViewMode", ctypes.c_ulonglong),
        ("SetDimensions", ctypes.c_ulonglong),
        ("EnqueueRender", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, bool, EgRenderCullMode::Mode, EgRenderView, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("EnqueueRenderStereo", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, bool, bool, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("ResizeBuffers", ctypes.c_ulonglong),
        ("EnqueueRenderViewport", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, bool, EgRenderCullMode::Mode, EgRenderView, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("EnqueueRenderViewportStereo", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, bool, bool, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("PatchRenderTargetMatrices", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("GetViewMatrix", ctypes.c_ulonglong),
        ("GetProjectionMatrix", ctypes.c_ulonglong),
        ("InFrustum", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("IsConstructed", ctypes.c_ulonglong),
        ("CalculateNewFov", ctypes.c_ulonglong),
        ("EnqueueRenderPipeline", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, EgRenderCullMode::Mode, TkHandle)
        ("EnqueueRenderToTexture", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkMatrix34 *, const cTkMatrix34 *, float, float, EgRenderCullMode::Mode, bool, TkHandle)
        ("EnqueueRenderToTexture_b0", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, EgRenderCullMode::Mode, bool, TkHandle)
        ("EnqueueRenderToTexture_b8", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkMatrix34 *, float, float, EgRenderCullMode::Mode, bool, bool, TkHandle)
        ("EnqueueRenderToTexture_c0", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, EgRenderCullMode::Mode, TkHandle)
        ("SetShadowCascadeMask", ctypes.c_ulonglong),
        ("SetMotionBlur", ctypes.c_ulonglong),
    ]
    cTkGraphicsManagerBase_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    UpdateRender: int
    NextViewMode: int
    SetDimensions: int
    EnqueueRender: bytes
    EnqueueRenderStereo: bytes
    ResizeBuffers: int
    EnqueueRenderViewport: bytes
    EnqueueRenderViewportStereo: bytes
    PatchRenderTargetMatrices: bytes
    GetViewMatrix: int
    GetProjectionMatrix: int
    InFrustum: int
    Release: int
    Destruct: int
    IsConstructed: int
    CalculateNewFov: int
    EnqueueRenderPipeline: bytes
    EnqueueRenderToTexture: bytes
    EnqueueRenderToTexture_b0: bytes
    EnqueueRenderToTexture_b8: bytes
    EnqueueRenderToTexture_c0: bytes
    SetShadowCascadeMask: int
    SetMotionBlur: int

class ComponentInstancercTkRotationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercTkSketchComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercTkRandomComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class CSha256(ctypes.Structure):
    _fields_ = [
        ("state", ctypes.c_uint32 * 0x8),  # unsigned int[8]
        ("count", ctypes.c_uint64),
        ("buffer", ctypes.c_uint8 * 0x40),  # unsigned __int8[64]
    ]
    state: list[ctypes.c_uint32]
    count: int
    buffer: list[ctypes.c_uint8]

class cTkQuadricErrorFunction(ctypes.Structure):
    pass

class cTkShaderPerfStats(ctypes.Structure):
    _fields_ = [
        ("miNumVsCycles", ctypes.c_int32),
        ("miNumVsTexReads", ctypes.c_int32),
        ("miNumVsVGPRs", ctypes.c_int32),
        ("miNumVsSGPRs", ctypes.c_int32),
        ("miNumPsCycles", ctypes.c_int32),
        ("miNumPsTexReads", ctypes.c_int32),
        ("miNumPsVGPRs", ctypes.c_int32),
        ("miNumPsSGPRs", ctypes.c_int32),
        ("mbPopulated", ctypes.c_ubyte),
    ]
    miNumVsCycles: int
    miNumVsTexReads: int
    miNumVsVGPRs: int
    miNumVsSGPRs: int
    miNumPsCycles: int
    miNumPsTexReads: int
    miNumPsVGPRs: int
    miNumPsSGPRs: int
    mbPopulated: bool

class cTkCurveFunction(ctypes.Structure):
    def Calculate(): pass

    def CalculateIntegral(): pass


class ComponentInstancercTkAnimationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercTkAnimationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ScopedTimingBlock(ctypes.Structure):
    def ScopedTimingBlock(): pass

    _fields_ = [
        ("mpacBlockName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("mu64StartTime", ctypes.c_uint64),
    ]
    mpacBlockName: bytes
    mu64StartTime: int

class cTkBaseCompute(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkBaseCompute_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("GetJobsInFlight", ctypes.c_ulonglong),
    ]
    Construct: int
    Destruct: int
    Update: int
    GetJobsInFlight: int

class sTkFfxFsr2ShaderConst(ctypes.Structure):
    _fields_ = [
        ("mpName", ctypes.c_char_p),
        ("mHash", ctypes.c_uint64),
        ("muSize", ctypes.c_uint8),
        ("___u3", ctypes.c_char * 0x10),  # $0914BE4049FC44A9A1C6D2E9881ED26B
    ]
    mpName: bytes
    mHash: int
    muSize: int
    ___u3: bytes

class sTkFfxFsr2Params(ctypes.Structure):
    _fields_ = [
        ("miDisplayWidth", ctypes.c_int32),
        ("miDisplayHeight", ctypes.c_int32),
        ("muScaleToBufferWidth", ctypes.c_uint32),
        ("muScaleToBufferHeight", ctypes.c_uint32),
        ("muFrameIndex", ctypes.c_uint32),
        ("muFrameIndexLastJump", ctypes.c_uint32),
        ("mfFrameTimeDelta", ctypes.c_float),
        ("maJitterX", ctypes.c_float * 0x2),  # float[2]
        ("maJitterY", ctypes.c_float * 0x2),  # float[2]
        ("mfNearClip", ctypes.c_float),
        ("mfFarClip", ctypes.c_float),
        ("mFovVertical", ctypes.c_float),
        ("muScaledRenderWidth", ctypes.c_uint32),
        ("muScaledRenderHeight", ctypes.c_uint32),
        ("mfSharpness", ctypes.c_float),
        ("mbStereo", ctypes.c_ubyte),
    ]
    miDisplayWidth: int
    miDisplayHeight: int
    muScaleToBufferWidth: int
    muScaleToBufferHeight: int
    muFrameIndex: int
    muFrameIndexLastJump: int
    mfFrameTimeDelta: float
    maJitterX: list[ctypes.c_float]
    maJitterY: list[ctypes.c_float]
    mfNearClip: float
    mfFarClip: float
    mFovVertical: float
    muScaledRenderWidth: int
    muScaledRenderHeight: int
    mfSharpness: float
    mbStereo: bool

class cTkFfxFsr2(ctypes.Structure):
    def GetJitter(): pass

    def PrepareLanczos2LutTexture(): pass

    def SetQualityScaleFromLevel(): pass

    def UpdateConfig(): pass

    def UpdateShaderConsts(): pass


class cTkGraphicsThreadTask(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpFunction", ctypes.c_ulonglong),
        ("mpData", ctypes.c_void_p),
        ("mbFinished", ctypes.c_ubyte),
        ("mpcTaskName", ctypes.c_char_p),
    ]
    __vftable: int
    mpFunction: int
    mpData: int
    mbFinished: bool
    mpcTaskName: bytes

class cTkGraphicsThreadTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkGraphicsThreadTask_dtor_0", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("PreRun", ctypes.c_ulonglong),
    ]
    cTkGraphicsThreadTask_dtor_0: int
    Run: int
    PreRun: int

class cTkContextOwnScoped(ctypes.Structure):
    _fields_ = [
        ("mpContext", ctypes.c_ulonglong),
    ]
    mpContext: int

class cTkVirtualBackingStore(ctypes.Structure):
    pass

class cTkDynamicMemoryPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkMemoryPool_dtor_0", ctypes.c_ulonglong),
        ("Create", ctypes.c_ulonglong),
        ("Malloc", ctypes.c_ulonglong),
        ("Malloc_18", ctypes.c_ulonglong),
        ("Free", ctypes.c_ulonglong),
        ("Realloc", ctypes.c_ulonglong),
        ("ContainsAddress", ctypes.c_ulonglong),
        ("GetAllocationSize", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("GetUsage", ctypes.c_ulonglong),
        ("GetHighWaterUsage", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("Cleanup", ctypes.c_ulonglong),
        ("GetAllocInfo", ctypes.c_ulonglong),
        ("SetAllocInfo", ctypes.c_ulonglong),
        ("ClearStats", ctypes.c_ulonglong),
        ("OutputStats", ctypes.c_ulonglong),
        ("FrameSyncEnd", ctypes.c_ulonglong),
        ("NeedsFrameSyncEnd", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("FlagError", ctypes.c_ulonglong),
        ("HasError", ctypes.c_ulonglong),
        ("NeedsMutex", ctypes.c_ulonglong),
        ("SetMutex", ctypes.c_ulonglong),
        ("GetOverflowPool", ctypes.c_ulonglong),
        ("GetBaseAddress", ctypes.c_ulonglong),
    ]
    cTkMemoryPool_dtor_0: int
    Create: int
    Malloc: int
    Malloc_18: int
    Free: int
    Realloc: int
    ContainsAddress: int
    GetAllocationSize: int
    SetAlignment: int
    GetAlignment: int
    GetUsage: int
    GetHighWaterUsage: int
    GetSize: int
    Cleanup: int
    GetAllocInfo: int
    SetAllocInfo: int
    ClearStats: int
    OutputStats: int
    FrameSyncEnd: int
    NeedsFrameSyncEnd: int
    Reset: int
    FlagError: int
    HasError: int
    NeedsMutex: int
    SetMutex: int
    GetOverflowPool: int
    GetBaseAddress: int

class cGcNetworkDelayedMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkRpcMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcRpcCallBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcRpcCallBase_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("ProcessRemoteCall", ctypes.c_ulonglong),
        ("ProcessResponse", ctypes.c_ulonglong),
    ]
    cGcRpcCallBase_dtor_0: int
    Update: int
    ProcessRemoteCall: int
    ProcessResponse: int

class cGcNetworkRpcId(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $A0967702BC1DC4A5BBF8C06767C42750
    ]
    ___u0: bytes

class IStatWatcher(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IStatWatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("StatChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatWatcher *this, const TkID<128> *, long double)
        ("StatChanged_8", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatWatcher *this, const TkID<128> *, __int64)
    ]
    StatChanged: bytes
    StatChanged_8: bytes

class sPlayerTitleStatWatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("StatChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatWatcher *this, const TkID<128> *, long double)
        ("StatChanged_8", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatWatcher *this, const TkID<128> *, __int64)
    ]
    StatChanged: bytes
    StatChanged_8: bytes

class cGcVehicleLocation(ctypes.Structure):
    def GetSpawnTransform(): pass

    _fields_ = [
        ("mLocation", ctypes.c_uint64),
        ("mPosition", common.Vector4f),
        ("mDirection", common.Vector4f),
    ]
    mLocation: int
    mPosition: common.Vector4f
    mDirection: common.Vector4f

class LogBookMessage(ctypes.Structure):
    def LogBookMessage(): pass

    _fields_ = [
        ("miTimeStamp", ctypes.c_int32),
        ("msLogMessage", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    miTimeStamp: int
    msLogMessage: bytes

class cGcPlayerLogBook(ctypes.Structure):
    def Clear(): pass

    _fields_ = [
        ("mPlayerLogs", ctypes.c_char * 0x18),  # std::vector<LogBookMessage,TkSTLAllocatorShim<LogBookMessage,4,-1> >
    ]
    mPlayerLogs: bytes

class cGcVehicleWeapon_vtbl(ctypes.Structure):
    _fields_ = [
        ("UsingAmmo", ctypes.c_ulonglong),
        ("UsesInventoryAmmo", ctypes.c_ulonglong),
        ("GetAvailableAmmo", ctypes.c_ulonglong),
        ("GetClipSize", ctypes.c_ulonglong),
        ("IsReloading", ctypes.c_ulonglong),
        ("GetReloadProgress", ctypes.c_ulonglong),
        ("GetLastFireTime", ctypes.c_ulonglong),
        ("HasOverheated", ctypes.c_ulonglong),
        ("GetOverHeatTime", ctypes.c_ulonglong),
        ("GetOverheatProgress", ctypes.c_ulonglong),
        ("IsOnCooldown", ctypes.c_ulonglong),
        ("GetCooldownDuration", ctypes.c_ulonglong),
        ("GetCooldownProgress", ctypes.c_ulonglong),
        ("GetAmmoInClip", ctypes.c_ulonglong),
        ("GetInventoryItem", ctypes.c_ulonglong),
        ("GetActiveInventoryItem", ctypes.c_ulonglong),
        ("GetActiveSecondaryInventoryItem", ctypes.c_ulonglong),
        ("GetHeatFactor", ctypes.c_ulonglong),
        ("GetHeatWarningFactor", ctypes.c_ulonglong),
        ("GetAlertPoint", ctypes.c_ulonglong),
        ("GetCooldownFactor", ctypes.c_ulonglong),
        ("GetModeIndex", ctypes.c_ulonglong),
        ("GetLastModeChangeTime", ctypes.c_ulonglong),
        ("GetNumModes", ctypes.c_ulonglong),
        ("SetModeIndex", ctypes.c_ulonglong),
        ("GetWeaponRange", ctypes.c_ulonglong),
        ("GetWeaponSpeed", ctypes.c_ulonglong),
        ("GetPrimaryStat", ctypes.c_ulonglong),
        ("IsCriticalShot", ctypes.c_ulonglong),
        ("GetListOfModesIfActiveOrNot", ctypes.c_char * 0x8),  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *(__fastcall *)(cGcWeapon *this, std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *result)
        ("GetChargeFactor", ctypes.c_ulonglong),
        ("IsCharging", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("IsVehicleWeapon", ctypes.c_ulonglong),
        ("GetTerrainEditMode", ctypes.c_ulonglong),
        ("GetTerrainEditorBeam", ctypes.c_ulonglong),
        ("UsingTerrainAdvancedMode", ctypes.c_ulonglong),
        ("ShowingTerrainEditNotifies", ctypes.c_ulonglong),
        ("ResetTerrainMode", ctypes.c_ulonglong),
        ("CanCharge", ctypes.c_ulonglong),
    ]
    UsingAmmo: int
    UsesInventoryAmmo: int
    GetAvailableAmmo: int
    GetClipSize: int
    IsReloading: int
    GetReloadProgress: int
    GetLastFireTime: int
    HasOverheated: int
    GetOverHeatTime: int
    GetOverheatProgress: int
    IsOnCooldown: int
    GetCooldownDuration: int
    GetCooldownProgress: int
    GetAmmoInClip: int
    GetInventoryItem: int
    GetActiveInventoryItem: int
    GetActiveSecondaryInventoryItem: int
    GetHeatFactor: int
    GetHeatWarningFactor: int
    GetAlertPoint: int
    GetCooldownFactor: int
    GetModeIndex: int
    GetLastModeChangeTime: int
    GetNumModes: int
    SetModeIndex: int
    GetWeaponRange: int
    GetWeaponSpeed: int
    GetPrimaryStat: int
    IsCriticalShot: int
    GetListOfModesIfActiveOrNot: bytes
    GetChargeFactor: int
    IsCharging: int
    IsActive: int
    IsVehicleWeapon: int
    GetTerrainEditMode: int
    GetTerrainEditorBeam: int
    UsingTerrainAdvancedMode: int
    ShowingTerrainEditNotifies: int
    ResetTerrainMode: int
    CanCharge: int

class cGcTechnologyAttachmentComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCarSuspensionAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class cGcNavObstacleTracker(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcNavigationConfig(ctypes.Structure):
    _fields_ = [
        ("mfMinUpdatePeriod", ctypes.c_float),
        ("mfNavRadius", ctypes.c_float),
        ("mfHeight", ctypes.c_float),
        ("mfMaxSpeed", ctypes.c_float),
        ("mfMinObstacleSize", ctypes.c_float),
        ("muPreferredLOD", ctypes.c_uint8),
        ("muAllowedRegionTypeMask", ctypes.c_uint16),
        ("mbNoPhysModeEnabled", ctypes.c_ubyte),
    ]
    mfMinUpdatePeriod: float
    mfNavRadius: float
    mfHeight: float
    mfMaxSpeed: float
    mfMinObstacleSize: float
    muPreferredLOD: int
    muAllowedRegionTypeMask: int
    mbNoPhysModeEnabled: bool

class cGcNavObstacleTracker_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNavObstacleTracker_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
    ]
    cGcNavObstacleTracker_dtor_0: int
    Prepare: int
    Release: int
    Init: int
    Update: int

class cGcEnvironmentNavigator(ctypes.Structure):
    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcNavSteering(ctypes.Structure):
    def PreUpdate(): pass

    def GenerateRecipTurnRadii(): pass

    def PickBestDirection(): pass

    def AddAvoidObstacles(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("miNumDirections", ctypes.c_int32),
        ("mavDirectionsLocal", common.Vector3f * 0x24),  # cTkVector3[36]
        ("mavDirectionsWorld", common.Vector3f * 0x24),  # cTkVector3[36]
        ("mafRecipTurnRadius", ctypes.c_float * 0x24),  # float[36]
        ("maafInterests", ctypes.c_float * 0x3),  # float[3][36]
        ("mfDangerThreshold", ctypes.c_float),
        ("mfNearTargetDangerThresholdScaler", ctypes.c_float),
    ]
    __vftable: int
    miNumDirections: int
    mavDirectionsLocal: list[common.Vector3f]
    mavDirectionsWorld: list[common.Vector3f]
    mafRecipTurnRadius: list[ctypes.c_float]
    maafInterests: list[ctypes.c_float]
    mfDangerThreshold: float
    mfNearTargetDangerThresholdScaler: float

class cGcNavSteering_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNavSteering_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("PreUpdate", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("AddAvoidObstacles", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNavSteering *this, cTkStackVector<cGcPathfindingObstacle,32,-1>)
        ("AddInterest", ctypes.c_ulonglong),
    ]
    cGcNavSteering_dtor_0: int
    Prepare: int
    Release: int
    Init: int
    PreUpdate: int
    Update: int
    AddAvoidObstacles: bytes
    AddInterest: int

class cGcEnvironmentNavigator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcEnvironmentNavigator_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("ApplyToSteering", ctypes.c_ulonglong),
        ("IsPointNavigable", ctypes.c_ulonglong),
        ("CanPathToPoint", ctypes.c_ulonglong),
        ("GetNearestAccessiblePoint", ctypes.c_ulonglong),
        ("IsStuck", ctypes.c_ulonglong),
    ]
    cGcEnvironmentNavigator_dtor_0: int
    Prepare: int
    Release: int
    Init: int
    Update: int
    ApplyToSteering: int
    IsPointNavigable: int
    CanPathToPoint: int
    GetNearestAccessiblePoint: int
    IsStuck: int

class sMechTargetData(ctypes.Structure):
    def GetTargetBody(): pass

    def GetTargetPoint(): pass

    _fields_ = [
        ("mpAttackTarget", ctypes.c_uint64),
        ("mfDistanceSq", ctypes.c_float),
        ("mfThreatLevel", ctypes.c_float),
        ("mfStartedTargetingTime", ctypes.c_float),
        ("mbHasLineOfSight", ctypes.c_ubyte),
    ]
    mpAttackTarget: int
    mfDistanceSq: float
    mfThreatLevel: float
    mfStartedTargetingTime: float
    mbHasLineOfSight: bool

class cGcMechBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int

class cGcVehicleComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class INetworkPlayerEventsHandler(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class INetworkPlayerEventsHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcNetReplicatedFreighterColourData(ctypes.Structure):
    _fields_ = [
        ("maaiColours", ctypes.c_int32 * 0x2),  # int[2][3]
    ]
    maaiColours: list[ctypes.c_int32]

class cGcNetworkFreighterOwnershipSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcAIShipSquad(ctypes.Structure):
    def HasActiveMembers(): pass

    def CalcAveragePos(): pass

    def GetLeader(): pass

    _fields_ = [
        ("mName", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mHandle", ctypes.c_uint64),
        ("maShips", ctypes.c_char * 0x1608),  # cTkRosterPool<cGcAIShipSpawn,128>
        ("mbManaged", ctypes.c_ubyte),
        ("mbHasSentEngagedMessage", ctypes.c_ubyte),
    ]
    mName: bytes
    mHandle: int
    maShips: bytes
    mbManaged: bool
    mbHasSentEngagedMessage: bool

class cGcBlackboard_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcBlackboard_dtor_0", ctypes.c_ulonglong),
        ("GetIdentifier", ctypes.c_ulonglong),
    ]
    cGcBlackboard_dtor_0: int
    GetIdentifier: int

class cGcShipAIAvoidanceTracking(ctypes.Structure):
    _fields_ = [
        ("mDirection", common.Vector3f),
        ("mfTime", ctypes.c_float),
        ("mbCastRays", ctypes.c_ubyte),
        ("mRayCollisionMask", ctypes.c_uint16),
    ]
    mDirection: common.Vector3f
    mfTime: float
    mbCastRays: bool
    mRayCollisionMask: int

class cGcSpaceshipBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcOutpostComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcNPCPlacementComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcSpaceshipShieldComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcSpaceshipWeapons_vtbl(ctypes.Structure):
    _fields_ = [
        ("UsingAmmo", ctypes.c_ulonglong),
        ("UsesInventoryAmmo", ctypes.c_ulonglong),
        ("GetAvailableAmmo", ctypes.c_ulonglong),
        ("GetClipSize", ctypes.c_ulonglong),
        ("IsReloading", ctypes.c_ulonglong),
        ("GetReloadProgress", ctypes.c_ulonglong),
        ("GetLastFireTime", ctypes.c_ulonglong),
        ("HasOverheated", ctypes.c_ulonglong),
        ("GetOverHeatTime", ctypes.c_ulonglong),
        ("GetOverheatProgress", ctypes.c_ulonglong),
        ("IsOnCooldown", ctypes.c_ulonglong),
        ("GetCooldownDuration", ctypes.c_ulonglong),
        ("GetCooldownProgress", ctypes.c_ulonglong),
        ("GetAmmoInClip", ctypes.c_ulonglong),
        ("GetInventoryItem", ctypes.c_ulonglong),
        ("GetActiveInventoryItem", ctypes.c_ulonglong),
        ("GetActiveSecondaryInventoryItem", ctypes.c_ulonglong),
        ("GetHeatFactor", ctypes.c_ulonglong),
        ("GetHeatWarningFactor", ctypes.c_ulonglong),
        ("GetAlertPoint", ctypes.c_ulonglong),
        ("GetCooldownFactor", ctypes.c_ulonglong),
        ("GetModeIndex", ctypes.c_ulonglong),
        ("GetLastModeChangeTime", ctypes.c_ulonglong),
        ("GetNumModes", ctypes.c_ulonglong),
        ("SetModeIndex", ctypes.c_ulonglong),
        ("GetWeaponRange", ctypes.c_ulonglong),
        ("GetWeaponSpeed", ctypes.c_ulonglong),
        ("GetPrimaryStat", ctypes.c_ulonglong),
        ("IsCriticalShot", ctypes.c_ulonglong),
        ("GetListOfModesIfActiveOrNot", ctypes.c_char * 0x8),  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *(__fastcall *)(cGcWeapon *this, std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *result)
        ("GetChargeFactor", ctypes.c_ulonglong),
        ("IsCharging", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("IsVehicleWeapon", ctypes.c_ulonglong),
        ("GetTerrainEditMode", ctypes.c_ulonglong),
        ("GetTerrainEditorBeam", ctypes.c_ulonglong),
        ("UsingTerrainAdvancedMode", ctypes.c_ulonglong),
        ("ShowingTerrainEditNotifies", ctypes.c_ulonglong),
        ("ResetTerrainMode", ctypes.c_ulonglong),
        ("CanCharge", ctypes.c_ulonglong),
    ]
    UsingAmmo: int
    UsesInventoryAmmo: int
    GetAvailableAmmo: int
    GetClipSize: int
    IsReloading: int
    GetReloadProgress: int
    GetLastFireTime: int
    HasOverheated: int
    GetOverHeatTime: int
    GetOverheatProgress: int
    IsOnCooldown: int
    GetCooldownDuration: int
    GetCooldownProgress: int
    GetAmmoInClip: int
    GetInventoryItem: int
    GetActiveInventoryItem: int
    GetActiveSecondaryInventoryItem: int
    GetHeatFactor: int
    GetHeatWarningFactor: int
    GetAlertPoint: int
    GetCooldownFactor: int
    GetModeIndex: int
    GetLastModeChangeTime: int
    GetNumModes: int
    SetModeIndex: int
    GetWeaponRange: int
    GetWeaponSpeed: int
    GetPrimaryStat: int
    IsCriticalShot: int
    GetListOfModesIfActiveOrNot: bytes
    GetChargeFactor: int
    IsCharging: int
    IsActive: int
    IsVehicleWeapon: int
    GetTerrainEditMode: int
    GetTerrainEditorBeam: int
    UsingTerrainAdvancedMode: int
    ShowingTerrainEditNotifies: int
    ResetTerrainMode: int
    CanCharge: int

class sAutoFollowControlParameters(ctypes.Structure):
    _fields_ = [
        ("mfPitch", ctypes.c_float),
        ("mfYaw", ctypes.c_float),
        ("mfRoll", ctypes.c_float),
        ("mfThrust", ctypes.c_float),
        ("mfBrake", ctypes.c_float),
        ("mbBoost", ctypes.c_ubyte),
    ]
    mfPitch: float
    mfYaw: float
    mfRoll: float
    mfThrust: float
    mfBrake: float
    mbBoost: bool

class cGcPIDController(ctypes.Structure):
    def Update(): pass

    _fields_ = [
        ("mfPreviousError", ctypes.c_float),
        ("mfCumulativeError", ctypes.c_float),
        ("mpCurrentEngine", ctypes.c_ulonglong),
    ]
    mfPreviousError: float
    mfCumulativeError: float
    mpCurrentEngine: int

class cGcSpaceshipComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcAISpaceshipComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcPlayerFreighterOwnership_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class MaintenanceFlags: pass

class cGcObjectPlacementComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcMaintenanceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("IsRepair", ctypes.c_ulonglong),
        ("IsCooker", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("GetModelViewerRenderData", ctypes.c_ulonglong),
        ("RefreshPersistency", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("DeleteSavedState", ctypes.c_ulonglong),
        ("IsNetworkLockingType", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Seed", ctypes.c_ulonglong),
        ("CustomSlotUpdate", ctypes.c_ulonglong),
        ("CustomSlotRequirementsMet", ctypes.c_ulonglong),
        ("ItemAmountAccumulatorActive", ctypes.c_ulonglong),
        ("GetSavePosition", ctypes.c_char * 0x8),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation (__fastcall *)(cGcMaintenanceComponent *this, cTkVector3 *)
        ("GetPersistancyData", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    IsRepair: int
    IsCooker: int
    Interact: int
    GetModelViewerRenderData: int
    RefreshPersistency: int
    Save: int
    DeleteSavedState: int
    IsNetworkLockingType: int
    Setup: int
    Seed: int
    CustomSlotUpdate: int
    CustomSlotRequirementsMet: int
    ItemAmountAccumulatorActive: int
    GetSavePosition: bytes
    GetPersistancyData: int

class cGcExpeditionEventResult(ctypes.Structure):
    def Save(): pass

    def Load(): pass

    def OverrideReward(): pass

    def GetDescription(): pass

    def GetEventDifficulty(): pass

    def GetRewardDescription(): pass

    def GetInterventionDialog(): pass

    def OnInterventionDeclined(): pass

    def OnInterventionSucceeded(): pass

    def OnInterventionFailed(): pass

    def AddAffectedFrigate(): pass

    def GetChanceOfFrigateDamage(): pass

    def AttemptToRepairFrigates(): pass

    def DamageFrigates(): pass

    def CalculateNumberOfDamagedFrigates(): pass

    def ChooseDescription(): pass

    def ChooseRewards(): pass

    _fields_ = [
        ("maAffectedFrigates", ctypes.c_char * 0x18),  # std::vector<cGcDamageResponse,TkSTLAllocatorShim<cGcDamageResponse,8,-1> >
        ("mapRewards", ctypes.c_char * 0x18),  # std::vector<cGcRewardTableItem const *,TkSTLAllocatorShim<cGcRewardTableItem const *,8,-1> >
        ("mpEventData", ctypes.c_ulonglong),
        ("mpInterventionEventData", ctypes.c_ulonglong),
        ("mbIsWhaleEvent", ctypes.c_ubyte),
        ("mChosenDescription", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mRewardItemID", ctypes.c_ulonglong),
        ("mSeed", common.GcSeed),
        ("mUA", ctypes.c_uint64),
        ("miNumberOfDamagedFrigates", ctypes.c_int32),
        ("miRewardAmount", ctypes.c_int32),
        ("mbSuccess", ctypes.c_ubyte),
        ("mbIsInterventionEvent", ctypes.c_ubyte),
        ("mbAvoidedIntervention", ctypes.c_ubyte),
        ("mbDescriptionOverridden", ctypes.c_ubyte),
        ("mOverrideRewardId", ctypes.c_ulonglong),
    ]
    maAffectedFrigates: bytes
    mapRewards: bytes
    mpEventData: int
    mpInterventionEventData: int
    mbIsWhaleEvent: bool
    mChosenDescription: bytes
    mRewardItemID: int
    mSeed: common.GcSeed
    mUA: int
    miNumberOfDamagedFrigates: int
    miRewardAmount: int
    mbSuccess: bool
    mbIsInterventionEvent: bool
    mbAvoidedIntervention: bool
    mbDescriptionOverridden: bool
    mOverrideRewardId: int

class cGcFrigateOccupationTracker(ctypes.Structure):
    def Update(): pass

    def AddOccupiedFrigate(): pass

    _fields_ = [
        ("mapOccupiedFrigates", ctypes.c_char * 0x18),  # std::vector<cGcFleetFrigate *,TkSTLAllocatorShim<cGcFleetFrigate *,8,-1> >
        ("maOccupyingPlayerIds", ctypes.c_char * 0x18),  # std::vector<cTkUserIdBase<cTkFixedString<64,char> >,TkSTLAllocatorShim<cTkUserIdBase<cTkFixedString<64,char> >,1,-1> >
        ("mFleetManager", ctypes.c_ulonglong),
        ("miOccupationCounter", ctypes.c_int32),
    ]
    mapOccupiedFrigates: bytes
    maOccupyingPlayerIds: bytes
    mFleetManager: int
    miOccupationCounter: int

class cGcPurchasableFrigateTracker(ctypes.Structure):
    def Update(): pass

    _fields_ = [
        ("mContact", ctypes.c_uint32),
        ("mOwner", ctypes.c_ulonglong),
        ("maUnavailableFrigates", ctypes.c_char * 0x18),  # std::vector<TkHandle,TkSTLAllocatorShim<TkHandle,4,-1> >
    ]
    mContact: int
    mOwner: int
    maUnavailableFrigates: bytes

class cGcPlayerFleetManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcShipAISquadronSharedData(ctypes.Structure):
    _fields_ = [
        ("mbLeaving", ctypes.c_ubyte),
        ("mfLastFormationBreakTime", ctypes.c_float),
        ("miNumShipsInFormation", ctypes.c_int32),
    ]
    mbLeaving: bool
    mfLastFormationBreakTime: float
    miNumShipsInFormation: int

class IKnowledgeEventHandler(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IKnowledgeEventHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("KnowledgeRevisionWaypointsChanged", ctypes.c_ulonglong),
    ]
    KnowledgeRevisionWaypointsChanged: int

class IDiscoveryManagerEventHandler(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IDiscoveryManagerEventHandler_vtbl(ctypes.Structure):
    _fields_ = [
        ("DiscoveryQueryCompleted", ctypes.c_ulonglong),
        ("DiscoveryDataSubmitted", ctypes.c_ulonglong),
    ]
    DiscoveryQueryCompleted: int
    DiscoveryDataSubmitted: int

class cGcDiscoveryMetadata(ctypes.Structure):
    def LoadPersistent(): pass

    def SetCustomName(): pass

    _fields_ = [
        ("mCustomName", ctypes.c_char * 0x7F),  # cTkFixedString<127,char>
        ("mbHasCustomName", ctypes.c_ubyte),
    ]
    mCustomName: bytes
    mbHasCustomName: bool

class Session(ctypes.Structure):
    def SavePersistent(): pass

    def LoadPersistent(): pass

    def RemoveRecord(): pass

    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("mDiscoveryStore", ctypes.c_char * 0x28),  # cGcDiscoveryStoreImpl<DefaultDiscoveryDataHashing,19>
        ("mManagedAvailableForSubmission", ctypes.c_char * 0x18),  # std::vector<cGcManagedDiscovery *,TkSTLAllocatorShim<cGcManagedDiscovery *,8,19> >
        ("mManagedQueuedToSubmit", ctypes.c_char * 0x28),  # std::deque<cGcManagedDiscovery *,TkSTLAllocatorShim<cGcManagedDiscovery *,8,19> >
        ("mManagedSubmitted", ctypes.c_char * 0x18),  # std::vector<cGcManagedDiscovery *,TkSTLAllocatorShim<cGcManagedDiscovery *,8,19> >
        ("mReportsToSubmit", ctypes.c_char * 0x28),  # std::deque<cGcAtlasSendReport,TkSTLAllocatorShim<cGcAtlasSendReport,8,19> >
    ]
    mDiscoveryStore: bytes
    mManagedAvailableForSubmission: bytes
    mManagedQueuedToSubmit: bytes
    mManagedSubmitted: bytes
    mReportsToSubmit: bytes

class cGcWonderManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("DiscoveryQueryCompleted", ctypes.c_ulonglong),
        ("DiscoveryDataSubmitted", ctypes.c_ulonglong),
    ]
    DiscoveryQueryCompleted: int
    DiscoveryDataSubmitted: int

class IStatVariantWatcher(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IStatVariantWatcher_vtbl(ctypes.Structure):
    _fields_ = [
        ("VariantChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatVariantWatcher *this, const TkID<128> *, const cGcStatDefinition *, Stat::AvgStat)
        ("VariantChanged_8", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatVariantWatcher *this, const TkID<128> *, const cGcStatDefinition *, float)
        ("VariantChanged_10", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatVariantWatcher *this, const TkID<128> *, const cGcStatDefinition *, int)
    ]
    VariantChanged: bytes
    VariantChanged_8: bytes
    VariantChanged_10: bytes

class StatListener(ctypes.Structure):
    _fields_ = [
        ("mListenerAddress", ctypes.c_uint64),
        ("mId", ctypes.c_ulonglong),
        ("mStats", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,Stat,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mpVariantWatcher", ctypes.c_ulonglong),
    ]
    mListenerAddress: int
    mId: int
    mStats: bytes
    mpVariantWatcher: int

class cGcAtlasTransport_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasTransport_dtor_0", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdateThreadStart", ctypes.c_ulonglong),
        ("UpdateThreadEnd", ctypes.c_ulonglong),
        ("GetNetworkState", ctypes.c_ulonglong),
        ("IsTrafficInFlight", ctypes.c_ulonglong),
        ("RequestHttpAsync", ctypes.c_char * 0x8),  # cGcAtlasTransport::RequestResult (__fastcall *)(cGcAtlasTransport *this, const Http::Request *, const std::function<void __cdecl(Http::RequestResult const &)> *)
    ]
    cGcAtlasTransport_dtor_0: int
    GetName: int
    Construct: int
    Destruct: int
    Update: int
    UpdateThreadStart: int
    UpdateThreadEnd: int
    GetNetworkState: int
    IsTrafficInFlight: int
    RequestHttpAsync: bytes

class cTkDocumentWriterJSON_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkDocumentWriter_dtor_0", ctypes.c_ulonglong),
        ("GetWriterFormat", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(ITkDocumentWriter *this, TkID<128> *result)
        ("Clear", ctypes.c_ulonglong),
        ("PushContext", ctypes.c_ulonglong),
        ("PopContext", ctypes.c_ulonglong),
        ("OpenArray", ctypes.c_ulonglong),
        ("OpenObject", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("AddValue", ctypes.c_ulonglong),
        ("AddValue_48", ctypes.c_ulonglong),
        ("AddValue_50", ctypes.c_ulonglong),
        ("AddValue_58", ctypes.c_ulonglong),
        ("AddValue_60", ctypes.c_char * 0x8),  # void (__fastcall *)(ITkDocumentWriter *this, const TkID<256> *)
        ("AddValue_68", ctypes.c_char * 0x8),  # void (__fastcall *)(ITkDocumentWriter *this, const TkID<128> *)
        ("AddValue_70", ctypes.c_ulonglong),
        ("AddValue_78", ctypes.c_ulonglong),
        ("AddValue_80", ctypes.c_ulonglong),
        ("AddValue_88", ctypes.c_ulonglong),
        ("AddValue_90", ctypes.c_ulonglong),
        ("AddValue_98", ctypes.c_ulonglong),
        ("AddValue_a0", ctypes.c_ulonglong),
        ("AddValue_a8", ctypes.c_ulonglong),
        ("AddValue_b0", ctypes.c_ulonglong),
        ("AddValue_b8", ctypes.c_ulonglong),
        ("AddValue_c0", ctypes.c_ulonglong),
        ("AddKey", ctypes.c_ulonglong),
    ]
    ITkDocumentWriter_dtor_0: int
    GetWriterFormat: bytes
    Clear: int
    PushContext: int
    PopContext: int
    OpenArray: int
    OpenObject: int
    Close: int
    AddValue: int
    AddValue_48: int
    AddValue_50: int
    AddValue_58: int
    AddValue_60: bytes
    AddValue_68: bytes
    AddValue_70: int
    AddValue_78: int
    AddValue_80: int
    AddValue_88: int
    AddValue_90: int
    AddValue_98: int
    AddValue_a0: int
    AddValue_a8: int
    AddValue_b0: int
    AddValue_b8: int
    AddValue_c0: int
    AddKey: int

class cGcGameAnalytics_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcTelemetryManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("VariantChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatVariantWatcher *this, const TkID<128> *, const cGcStatDefinition *, Stat::AvgStat)
        ("VariantChanged_8", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatVariantWatcher *this, const TkID<128> *, const cGcStatDefinition *, float)
        ("VariantChanged_10", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatVariantWatcher *this, const TkID<128> *, const cGcStatDefinition *, int)
    ]
    VariantChanged: bytes
    VariantChanged_8: bytes
    VariantChanged_10: bytes

class cGcUserSeenItemsState(ctypes.Structure):
    def Prepare(): pass

    def IsProductSeen(): pass

    def IsSpecialUnlocked(): pass

    def GetUnlockedTwitchRewardsInLinkedGroup(): pass

    def SeeSubstance(): pass

    def SeeProduct(): pass

    def SeeTechnology(): pass

    def SeeTopic(): pass

    def UnlockTopic(): pass

    def UnlockTitle(): pass

    def AddUnlockedSpecial(): pass

    def SaveState(): pass

    def LoadState(): pass

    def UpdateWikiUnlocks(): pass

    _fields_ = [
        ("mSeenWikiTopics", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<256>,void,robin_hood::hash<TkID<256>,void>,std::equal_to<TkID<256> >,-1>
        ("mUnlockedWikiTopics", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<256>,void,robin_hood::hash<TkID<256>,void>,std::equal_to<TkID<256> >,-1>
        ("mSeenSubstances", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,void,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mSeenTechnologies", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,void,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mSeenProducts", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,void,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mUnlockedTitles", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,void,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mUnlockedSpecials", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,void,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mUnlockedSeasonRewards", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,void,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mUnlockedTwitchRewards", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,void,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mUnlockedPlatformRewards", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,void,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mDataLoadedCallbacks", ctypes.c_char * 0x18),  # std::vector<std::pair<void (__cdecl*)(void *),void *>,TkSTLAllocatorShim<std::pair<void (__cdecl*)(void *),void *>,8,-1> >
        ("mbOutdated", ctypes.c_ubyte),
        ("mbLoaded", ctypes.c_ubyte),
    ]
    mSeenWikiTopics: bytes
    mUnlockedWikiTopics: bytes
    mSeenSubstances: bytes
    mSeenTechnologies: bytes
    mSeenProducts: bytes
    mUnlockedTitles: bytes
    mUnlockedSpecials: bytes
    mUnlockedSeasonRewards: bytes
    mUnlockedTwitchRewards: bytes
    mUnlockedPlatformRewards: bytes
    mDataLoadedCallbacks: bytes
    mbOutdated: bool
    mbLoaded: bool

class cGcMPMissionTracker_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcEntitlementManager(ctypes.Structure):
    def HasEntitlementReward(): pass

    def HasUsedEntitlementReward(): pass

    def MarkEntitlementUsed(): pass

    _fields_ = [
        ("mpEntitlementRewardsTable", ctypes.c_ulonglong),
    ]
    mpEntitlementRewardsTable: int

class cGcNetworkBufferHash(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def GetHashValue(): pass

    def EnsureMinimumChunks(): pass

    def SetHashValue(): pass

    def DebugValidateChunkHashValues(): pass

    def SyncHashOffsetValue(): pass

    def Initialise(): pass

    def ClearHashes(): pass

    def OnBufferRangeAdded(): pass

    def OnBufferRangeRemoved(): pass

    def OnBufferRangeCleared(): pass

    def OnBufferRangeSet(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("kiChunkSize", ctypes.c_int32),
        ("miChunkHashOffset", ctypes.c_int32),
        ("maChunkHashValues", ctypes.c_char * 0x18),  # std::vector<sHashValue,TkSTLAllocatorShim<sHashValue,2,-1> >
        ("mu64Timestamp", ctypes.c_uint64),
        ("mbInitialised", ctypes.c_ubyte),
    ]
    __vftable: int
    kiChunkSize: int
    miChunkHashOffset: int
    maChunkHashValues: bytes
    mu64Timestamp: int
    mbInitialised: bool

class sHashValue(ctypes.Structure):
    _fields_ = [
        ("muHashValue", ctypes.c_uint16),
        ("miTimeStamp", ctypes.c_int16),
    ]
    muHashValue: int
    miTimeStamp: int

class cGcNetworkBufferHash_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkBufferHash_dtor_0", ctypes.c_ulonglong),
        ("GetHashValue", ctypes.c_ulonglong),
        ("GetHashTimestamp", ctypes.c_ulonglong),
        ("GenerateHashValue", ctypes.c_ulonglong),
        ("OnHashOffsetChanged", ctypes.c_ulonglong),
    ]
    cGcNetworkBufferHash_dtor_0: int
    GetHashValue: int
    GetHashTimestamp: int
    GenerateHashValue: int
    OnHashOffsetChanged: int

class cGcNetworkHashMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkSynchronisedBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkBufferHash_dtor_0", ctypes.c_ulonglong),
        ("GetHashValue", ctypes.c_ulonglong),
        ("GetHashTimestamp", ctypes.c_ulonglong),
        ("GenerateHashValue", ctypes.c_ulonglong),
        ("OnHashOffsetChanged", ctypes.c_ulonglong),
        ("GenerateHashMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkHashMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkHashMessage> *result)
        ("GenerateSyncMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkSyncMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkSyncMessage> *result, int)
        ("ApplySyncMessage", ctypes.c_ulonglong),
        ("HasNetworkOwner", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkSynchronisedBuffer *this, cTkUserIdBase<cTkFixedString<64,char> > *)
        ("ShouldSyncWithPlayer", ctypes.c_ulonglong),
    ]
    cGcNetworkBufferHash_dtor_0: int
    GetHashValue: int
    GetHashTimestamp: int
    GenerateHashValue: int
    OnHashOffsetChanged: int
    GenerateHashMessage: bytes
    GenerateSyncMessage: bytes
    ApplySyncMessage: int
    HasNetworkOwner: bytes
    ShouldSyncWithPlayer: int

class cGcBaseBuildingPersistentBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkBufferHash_dtor_0", ctypes.c_ulonglong),
        ("GetHashValue", ctypes.c_ulonglong),
        ("GetHashTimestamp", ctypes.c_ulonglong),
        ("GenerateHashValue", ctypes.c_ulonglong),
        ("OnHashOffsetChanged", ctypes.c_ulonglong),
        ("GenerateHashMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkHashMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkHashMessage> *result)
        ("GenerateSyncMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkSyncMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkSyncMessage> *result, int)
        ("ApplySyncMessage", ctypes.c_ulonglong),
        ("HasNetworkOwner", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkSynchronisedBuffer *this, cTkUserIdBase<cTkFixedString<64,char> > *)
        ("ShouldSyncWithPlayer", ctypes.c_ulonglong),
    ]
    cGcNetworkBufferHash_dtor_0: int
    GetHashValue: int
    GetHashTimestamp: int
    GenerateHashValue: int
    OnHashOffsetChanged: int
    GenerateHashMessage: bytes
    GenerateSyncMessage: bytes
    ApplySyncMessage: int
    HasNetworkOwner: bytes
    ShouldSyncWithPlayer: int

class cGcBaseBuildingBaseLayout(ctypes.Structure):
    def GenerateLayout(): pass

    def AddObject(): pass

    def Update(): pass

    def ForceGenerateLayout(): pass

    def TestSphere(): pass

    def TestBox(): pass

    def TestLayout(): pass

    def scalar_deleting_destructor(): pass

    def cGcBaseBuildingBaseLayout(): pass

    _fields_ = [
        ("mBaseUA", ctypes.c_uint64),
        ("mBasePosition", common.Vector3f),
        ("mBaseUp", common.Vector3f),
        ("mBaseObjectSpheres", ctypes.c_char * 0x80),  # cTkVector3MetaKD<float,-1>
        ("mBaseObjectsTree", ctypes.c_char * 0xA8),  # nanoflann::KDTreeSingleIndexAdaptor<nanoflann::L2_Simple_Adaptor<float,cTkVector3MetaKD<float,-1>,float>,cTkVector3MetaKD<float,-1>,3,unsigned __int64,-1>
        ("mfBaseRadiusSqr", ctypes.c_float),
        ("mbTreeNeedsRebuild", ctypes.c_ubyte),
        ("mfTreeRebuildTimer", ctypes.c_float),
    ]
    mBaseUA: int
    mBasePosition: common.Vector3f
    mBaseUp: common.Vector3f
    mBaseObjectSpheres: bytes
    mBaseObjectsTree: bytes
    mfBaseRadiusSqr: float
    mbTreeNeedsRebuild: bool
    mfTreeRebuildTimer: float

class cGcPlayerBasePersistentBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkBufferHash_dtor_0", ctypes.c_ulonglong),
        ("GetHashValue", ctypes.c_ulonglong),
        ("GetHashTimestamp", ctypes.c_ulonglong),
        ("GenerateHashValue", ctypes.c_ulonglong),
        ("OnHashOffsetChanged", ctypes.c_ulonglong),
        ("GenerateHashMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkHashMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkHashMessage> *result)
        ("GenerateSyncMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkSyncMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkSyncMessage> *result, int)
        ("ApplySyncMessage", ctypes.c_ulonglong),
        ("HasNetworkOwner", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkSynchronisedBuffer *this, cTkUserIdBase<cTkFixedString<64,char> > *)
        ("ShouldSyncWithPlayer", ctypes.c_ulonglong),
    ]
    cGcNetworkBufferHash_dtor_0: int
    GetHashValue: int
    GetHashTimestamp: int
    GenerateHashValue: int
    OnHashOffsetChanged: int
    GenerateHashMessage: bytes
    GenerateSyncMessage: bytes
    ApplySyncMessage: int
    HasNetworkOwner: bytes
    ShouldSyncWithPlayer: int

class cGcPersistentInteractionBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkBufferHash_dtor_0", ctypes.c_ulonglong),
        ("GetHashValue", ctypes.c_ulonglong),
        ("GetHashTimestamp", ctypes.c_ulonglong),
        ("GenerateHashValue", ctypes.c_ulonglong),
        ("OnHashOffsetChanged", ctypes.c_ulonglong),
        ("GenerateHashMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkHashMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkHashMessage> *result)
        ("GenerateSyncMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkSyncMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkSyncMessage> *result, int)
        ("ApplySyncMessage", ctypes.c_ulonglong),
        ("HasNetworkOwner", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkSynchronisedBuffer *this, cTkUserIdBase<cTkFixedString<64,char> > *)
        ("ShouldSyncWithPlayer", ctypes.c_ulonglong),
    ]
    cGcNetworkBufferHash_dtor_0: int
    GetHashValue: int
    GetHashTimestamp: int
    GenerateHashValue: int
    OnHashOffsetChanged: int
    GenerateHashMessage: bytes
    GenerateSyncMessage: bytes
    ApplySyncMessage: int
    HasNetworkOwner: bytes
    ShouldSyncWithPlayer: int

class sSharedMutex(ctypes.Structure):
    _fields_ = [
        ("mMutex", ctypes.c_void_p),
        ("mbMutexCreated", ctypes.c_ubyte),
    ]
    mMutex: int
    mbMutexCreated: bool

class cGcTerrainEditBlockBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkBufferHash_dtor_0", ctypes.c_ulonglong),
        ("GetHashValue", ctypes.c_ulonglong),
        ("GetHashTimestamp", ctypes.c_ulonglong),
        ("GenerateHashValue", ctypes.c_ulonglong),
        ("OnHashOffsetChanged", ctypes.c_ulonglong),
        ("GenerateHashMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkHashMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkHashMessage> *result)
        ("GenerateSyncMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkSyncMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkSyncMessage> *result, int)
        ("ApplySyncMessage", ctypes.c_ulonglong),
        ("HasNetworkOwner", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkSynchronisedBuffer *this, cTkUserIdBase<cTkFixedString<64,char> > *)
        ("ShouldSyncWithPlayer", ctypes.c_ulonglong),
    ]
    cGcNetworkBufferHash_dtor_0: int
    GetHashValue: int
    GetHashTimestamp: int
    GenerateHashValue: int
    OnHashOffsetChanged: int
    GenerateHashMessage: bytes
    GenerateSyncMessage: bytes
    ApplySyncMessage: int
    HasNetworkOwner: bytes
    ShouldSyncWithPlayer: int

class cGcMaintenanceBuffer(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def GenerateChangeHash(): pass

    def GenerateHashValue(): pass

    def CleanUpValidSlotIndexes(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x1F0),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>
    ]
    baseclass_0: bytes

class cGcMaintenanceBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkBufferHash_dtor_0", ctypes.c_ulonglong),
        ("GetHashValue", ctypes.c_ulonglong),
        ("GetHashTimestamp", ctypes.c_ulonglong),
        ("GenerateHashValue", ctypes.c_ulonglong),
        ("OnHashOffsetChanged", ctypes.c_ulonglong),
        ("GenerateHashMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkHashMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkHashMessage> *result)
        ("GenerateSyncMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkSyncMessage> *(__fastcall *)(cGcNetworkSynchronisedBuffer *this, std::shared_ptr<cGcNetworkSyncMessage> *result, int)
        ("ApplySyncMessage", ctypes.c_ulonglong),
        ("HasNetworkOwner", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkSynchronisedBuffer *this, cTkUserIdBase<cTkFixedString<64,char> > *)
        ("ShouldSyncWithPlayer", ctypes.c_ulonglong),
    ]
    cGcNetworkBufferHash_dtor_0: int
    GetHashValue: int
    GetHashTimestamp: int
    GenerateHashValue: int
    OnHashOffsetChanged: int
    GenerateHashMessage: bytes
    GenerateSyncMessage: bytes
    ApplySyncMessage: int
    HasNetworkOwner: bytes
    ShouldSyncWithPlayer: int

class cGcVisitedSystemsBuffer(ctypes.Structure):
    def VisitNewGalacticAddress(): pass

    def GetVisitedSystemsAroundVoxel(): pass

    def GetVisitedData(): pass

    _fields_ = [
        ("mVisitedSystems", ctypes.c_char * 0x1800),  # cTkFixedArray<sVisitedSystem,512>
        ("miCurrentPosition", ctypes.c_int32),
        ("miVisitedSystemsCount", ctypes.c_int32),
    ]
    mVisitedSystems: bytes
    miCurrentPosition: int
    miVisitedSystemsCount: int

class cGcRichPresence_vtbl(ctypes.Structure):
    _fields_ = [
        ("StatChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatWatcher *this, const TkID<128> *, long double)
        ("StatChanged_8", ctypes.c_char * 0x8),  # void (__fastcall *)(IStatWatcher *this, const TkID<128> *, __int64)
    ]
    StatChanged: bytes
    StatChanged_8: bytes

class cGcScannedNodesList(ctypes.Structure):
    def AddNode(): pass

    def RemoveNode(): pass

    def Update(): pass

    def UpdateRender(): pass

    def Clear(): pass

    _fields_ = [
        ("maScannedNodes", ctypes.c_char * 0x50),  # robin_hood::detail::Table<0,80,unsigned int,ScannedNode,robin_hood::hash<unsigned int,void>,std::equal_to<unsigned int>,-1>
    ]
    maScannedNodes: bytes

class cGcMarkerList(ctypes.Structure):
    def Update(): pass

    def CheckTimers(): pass

    def RemoveMarker(): pass

    def RemoveMarkerWithNode(): pass

    def RemoveMarkers(): pass

    def TryAddMarker(): pass

    _fields_ = [
        ("maMarkerObjects", ctypes.c_char * 0x18),  # std::vector<cGcMarkerPoint,TkSTLAllocatorShim<cGcMarkerPoint,16,-1> >
    ]
    maMarkerObjects: bytes

class cGcNGuiText_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
        ("cGcNGuiElement_dtor_10", ctypes.c_ulonglong),
        ("EditElement", ctypes.c_ulonglong),
        ("EditPresets", ctypes.c_ulonglong),
        ("RenderUIEditorToolbar", ctypes.c_ulonglong),
        ("SelectableInEditor", ctypes.c_ulonglong),
        ("SetNeedsUpdate", ctypes.c_ulonglong),
        ("Deselect", ctypes.c_ulonglong),
        ("IsSelected", ctypes.c_ulonglong),
        ("Resized", ctypes.c_ulonglong),
        ("AttachMetadata", ctypes.c_ulonglong),
        ("DetachMetadata", ctypes.c_ulonglong),
        ("IsLayer", ctypes.c_ulonglong),
        ("GetSceneTreeIcon", ctypes.c_ulonglong),
        ("GetSceneTreeText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNGuiElement *this, cTkFixedString<128,char> *)
        ("RenderAdditionalOverlayIcons", ctypes.c_ulonglong),
        ("RenderTreeControls", ctypes.c_ulonglong),
        ("Convert", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
    ]
    Render: int
    GetType: int
    cGcNGuiElement_dtor_10: int
    EditElement: int
    EditPresets: int
    RenderUIEditorToolbar: int
    SelectableInEditor: int
    SetNeedsUpdate: int
    Deselect: int
    IsSelected: int
    Resized: int
    AttachMetadata: int
    DetachMetadata: int
    IsLayer: int
    GetSceneTreeIcon: int
    GetSceneTreeText: bytes
    RenderAdditionalOverlayIcons: int
    RenderTreeControls: int
    Convert: int
    SetText: int

class cGcWeatherHazardGiftManager(ctypes.Structure):
    def SpawnGift(): pass

    def Update(): pass

    def RemoveFurthestGift(): pass

    def OnRemotePlayerCollectedGiftStatic(): pass

    _fields_ = [
        ("maGifts", ctypes.c_char * 0x18),  # std::vector<TkHandle,TkSTLAllocatorShim<TkHandle,4,-1> >
        ("maGiftSpawnTimes", ctypes.c_char * 0x18),  # std::vector<unsigned __int64,TkSTLAllocatorShim<unsigned __int64,8,-1> >
        ("miNextGiftToProcess", ctypes.c_int32),
    ]
    maGifts: bytes
    maGiftSpawnTimes: bytes
    miNextGiftToProcess: int

class cGcRagdollLimiter(ctypes.Structure):
    def GetRagdollForDespawn(): pass

    def RegisterRagdoll(): pass

    _fields_ = [
        ("mRagdolls", ctypes.c_char * 0x430),  # cTkStackVector<cTkPhysicsComponent *,128,-1>
    ]
    mRagdolls: bytes

class cGcCachedShipResource(ctypes.Structure):
    def cGcCachedShipResource(): pass

    _fields_ = [
        ("mHandles", ctypes.c_char * 0x80),  # std::array<cTkSmartResHandle,32>
        ("mClass", ctypes.c_char * 0x80),  # std::array<enum eShipClass,32>
        ("mCount", ctypes.c_int32),
        ("miRefCount", ctypes.c_int32),
    ]
    mHandles: bytes
    mClass: bytes
    mCount: int
    miRefCount: int

class cGcRegionAsteroid_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
        ("PostPolygoniseThread", ctypes.c_ulonglong),
        ("CancelPolygonise", ctypes.c_ulonglong),
        ("PreAsyncKnowledgeBuild", ctypes.c_ulonglong),
        ("CanPolygonise", ctypes.c_ulonglong),
        ("ThreadedGenerateVoxels", ctypes.c_ulonglong),
        ("ThreadedPolygonise", ctypes.c_ulonglong),
        ("ThreadedKnowledgeBuild", ctypes.c_ulonglong),
        ("GetFacingTriangleStart", ctypes.c_ulonglong),
        ("GetTileBlendTriangleStart", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int
    PostPolygoniseThread: int
    CancelPolygonise: int
    PreAsyncKnowledgeBuild: int
    CanPolygonise: int
    ThreadedGenerateVoxels: int
    ThreadedPolygonise: int
    ThreadedKnowledgeBuild: int
    GetFacingTriangleStart: int
    GetTileBlendTriangleStart: int

class cGcPlanetBuildingGenerator(ctypes.Structure):
    def GenerateBuildings(): pass

    def PollToPlaceInitialBuildings(): pass

    def AddOverrideBuildings(): pass

    def AddSurvivalBuildings(): pass

    def GenerateBuildingModels(): pass

    def SetupBuildingFilenames(): pass

    def PollCalculateBuildingSizes(): pass

    def AddRandomBuildingSelection(): pass

    def AddNearestBuildings(): pass

    _fields_ = [
        ("mpBuildingTable", ctypes.c_ulonglong),
        ("mpSettlementColourTable", ctypes.c_ulonglong),
        ("mabUsedInCluster", ctypes.c_ubyte * 0x34),  # bool[52]
    ]
    mpBuildingTable: int
    mpSettlementColourTable: int
    mabUsedInCluster: list[ctypes.c_ubyte]

class cGcColourPalette(ctypes.Structure):
    def Construct(): pass

    def GeneratePalettes(): pass

    def GetVariantOfColour(): pass

    def GenerateBaseColours(): pass

    def PickRandomColour(): pass

    def RenderNGui(): pass

    def ModifyColour(): pass

    def GetColourWriteable(): pass

    def RenderPaletteCallback(): pass

    _fields_ = [
        ("maPalettes", ctypes.c_char * 0x180),  # std::array<std::vector<cGcPaletteList *,TkSTLAllocatorShim<cGcPaletteList *,8,-1> >,16>
        ("mfSaturationMultiplier", ctypes.c_float * 0x3A),  # float[58]
        ("mfValueMultiplier", ctypes.c_float * 0x3A),  # float[58]
        ("mfHueMultiplier", ctypes.c_float * 0x3A),  # float[58]
        ("mfRGBMultiplier", ctypes.c_float * 0x3A),  # float[58]
    ]
    maPalettes: bytes
    mfSaturationMultiplier: list[ctypes.c_float]
    mfValueMultiplier: list[ctypes.c_float]
    mfHueMultiplier: list[ctypes.c_float]
    mfRGBMultiplier: list[ctypes.c_float]

class cGcLegacyColourPalette(ctypes.Structure):
    def GeneratePalettes(): pass

    def GenerateBaseColours(): pass

    _fields_ = [
        ("maPalettes", ctypes.c_char * 0x180),  # std::array<std::vector<cGcPaletteList *,TkSTLAllocatorShim<cGcPaletteList *,8,-1> >,16>
    ]
    maPalettes: bytes

class cGcSolarSystemGeometry(ctypes.Structure):
    _fields_ = [
        ("mPlanetSpheres", ctypes.c_char * 0x100),  # cTkStackVector<cTkSphere,6,-1>
        ("mPlanetExclusion", ctypes.c_char * 0x100),  # cTkStackVector<cTkSphere,6,-1>
    ]
    mPlanetSpheres: bytes
    mPlanetExclusion: bytes

class DiscoveryAutoSync(ctypes.Structure):
    _fields_ = [
        ("lbActive", ctypes.c_ubyte),
        ("liIntervalInSeconds", ctypes.c_int32),
        ("lu64LastSyncTime", ctypes.c_uint64),
    ]
    lbActive: bool
    liIntervalInSeconds: int
    lu64LastSyncTime: int

class cGcTradeRouteCurvePoint(ctypes.Structure):
    _fields_ = [
        ("muX", ctypes.c_int64),
        ("muY", ctypes.c_int64),
        ("muZ", ctypes.c_int64),
        ("muDistance", ctypes.c_uint64),
        ("muTotalDistance", ctypes.c_uint64),
    ]
    muX: int
    muY: int
    muZ: int
    muDistance: int
    muTotalDistance: int

class cGcTradeRouteSection(ctypes.Structure):
    def cGcTradeRouteSection(): pass

    def Construct(): pass

    def GetClosestPoint(): pass

    def CalculatePointByDistance(): pass

    def Render(): pass

    _fields_ = [
        ("mStart", common.Vector3f),
        ("mMid1", common.Vector3f),
        ("mMid2", common.Vector3f),
        ("mMid3", common.Vector3f),
        ("mMid4", common.Vector3f),
        ("mEnd", common.Vector3f),
        ("maPoints", ctypes.c_char * 0x18),  # std::vector<cGcTradeRouteCurvePoint,TkSTLAllocatorShim<cGcTradeRouteCurvePoint,8,-1> >
        ("muTotalLength", ctypes.c_uint64),
        ("mLineHandle", ctypes.c_uint32),
    ]
    mStart: common.Vector3f
    mMid1: common.Vector3f
    mMid2: common.Vector3f
    mMid3: common.Vector3f
    mMid4: common.Vector3f
    mEnd: common.Vector3f
    maPoints: bytes
    muTotalLength: int
    mLineHandle: int

class cTk2dImageEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int

class cGcFiendManagerBase(ctypes.Structure):
    def Add(): pass

    def GetDespawnDistance(): pass

    def GetNumLocalAndRemoteCreaturesOfType(): pass

    def CanTakeControlOfFiend(): pass

    def TakeControlOfFiend(): pass

    def ReportCrime(): pass

    def GetSpawnRequests(): pass

    def Update(): pass

    def RenderNGui(): pass

    def DebugDraw(): pass

    def Release(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("maSpawnRequests", ctypes.c_char * 0x18),  # std::vector<cGcFiendSpawnRequest,TkSTLAllocatorShim<cGcFiendSpawnRequest,16,-1> >
        ("mapAttachments", ctypes.c_char * 0x18),  # std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> >
        ("maVirtualisedCreatures", ctypes.c_char * 0x18),  # std::vector<cGcVirtualisedCreature,TkSTLAllocatorShim<cGcVirtualisedCreature,16,-1> >
        ("mbWantsToClear", ctypes.c_ubyte),
    ]
    __vftable: int
    maSpawnRequests: bytes
    mapAttachments: bytes
    maVirtualisedCreatures: bytes
    mbWantsToClear: bool

class cGcFiendManagerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Add", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcFiendManagerBase *this, std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *)
        ("CanVirtualise", ctypes.c_ulonglong),
        ("ReportCrime", ctypes.c_ulonglong),
        ("ActiveFiendCombat", ctypes.c_ulonglong),
        ("OnTakeControlOfFiend", ctypes.c_ulonglong),
        ("GetDespawnDistance", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("DebugDraw", ctypes.c_ulonglong),
    ]
    Construct: int
    Add: bytes
    CanVirtualise: int
    ReportCrime: int
    ActiveFiendCombat: int
    OnTakeControlOfFiend: int
    GetDespawnDistance: int
    Release: int
    Update: int
    RenderNGui: int
    DebugDraw: int

class cGcFiendManagerStandard_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Add", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcFiendManagerBase *this, std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *)
        ("CanVirtualise", ctypes.c_ulonglong),
        ("ReportCrime", ctypes.c_ulonglong),
        ("ActiveFiendCombat", ctypes.c_ulonglong),
        ("OnTakeControlOfFiend", ctypes.c_ulonglong),
        ("GetDespawnDistance", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("DebugDraw", ctypes.c_ulonglong),
    ]
    Construct: int
    Add: bytes
    CanVirtualise: int
    ReportCrime: int
    ActiveFiendCombat: int
    OnTakeControlOfFiend: int
    GetDespawnDistance: int
    Release: int
    Update: int
    RenderNGui: int
    DebugDraw: int

class cGcFiendManagerUnderwater_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Add", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcFiendManagerBase *this, std::vector<cTkAttachmentPtr,TkSTLAllocatorShim<cTkAttachmentPtr,8,-1> > *)
        ("CanVirtualise", ctypes.c_ulonglong),
        ("ReportCrime", ctypes.c_ulonglong),
        ("ActiveFiendCombat", ctypes.c_ulonglong),
        ("OnTakeControlOfFiend", ctypes.c_ulonglong),
        ("GetDespawnDistance", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("DebugDraw", ctypes.c_ulonglong),
    ]
    Construct: int
    Add: bytes
    CanVirtualise: int
    ReportCrime: int
    ActiveFiendCombat: int
    OnTakeControlOfFiend: int
    GetDespawnDistance: int
    Release: int
    Update: int
    RenderNGui: int
    DebugDraw: int

class MultiplayerSplits(ctypes.Structure):
    _fields_ = [
        ("mPlayerID", ctypes.c_char * 0x40),  # cTkUserIdBase<cTkFixedString<64,char> >
        ("mfTime", ctypes.c_float),
    ]
    mPlayerID: bytes
    mfTime: float

class RaceSection(ctypes.Structure):
    def IsPersistencyValid(): pass

    def GetRaceIndex(): pass

    def GetBestTime(): pass

    def SetBestTime(): pass

    def GetMatrix(): pass

    def Checkpoint(): pass

    def RaceSection(): pass

    _fields_ = [
        ("mBaseObjectHandle", ctypes.c_uint64),
        ("mfTime", ctypes.c_float),
        ("maOtherPlayerTimes", ctypes.c_char * 0x18),  # std::vector<MultiplayerSplits,TkSTLAllocatorShim<MultiplayerSplits,4,-1> >
    ]
    mBaseObjectHandle: int
    mfTime: float
    maOtherPlayerTimes: bytes

class cGcPlayerExperienceDirector_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cTkCharDescr(ctypes.Structure):
    def tilde_cTkCharDescr(): pass

    _fields_ = [
        ("srcX", ctypes.c_int16),
        ("srcY", ctypes.c_int16),
        ("srcW", ctypes.c_int16),
        ("srcH", ctypes.c_int16),
        ("xOff", ctypes.c_int16),
        ("yOff", ctypes.c_int16),
        ("xAdv", ctypes.c_int16),
        ("page", ctypes.c_int16),
        ("chnl", ctypes.c_uint32),
        ("kerningPairs", ctypes.c_char * 0x18),  # std::vector<short,TkSTLAllocatorShim<short,2,-1> >
    ]
    srcX: int
    srcY: int
    srcW: int
    srcH: int
    xOff: int
    yOff: int
    xAdv: int
    page: int
    chnl: int
    kerningPairs: bytes

class cTkTextStyle(ctypes.Structure):
    def RenderBatchPlain(): pass

    def BuildPlainCharacter(): pass

    def RenderPlain(): pass

    def GetTextWidth(): pass

    def WriteParagraph(): pass

    def WriteSentence(): pass

    def WriteText(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkTextStyle_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTextStyle_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkTextStyle_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int

class cTk2dTextPreset(ctypes.Structure):
    _fields_ = [
        ("mfTextHeight", ctypes.c_float),
        ("mpFont", ctypes.c_ulonglong),
        ("mColour", common.Colour),
        ("mpTextStyle", ctypes.c_ulonglong),
        ("mbEnableHighlight", ctypes.c_ubyte),
    ]
    mfTextHeight: float
    mpFont: int
    mColour: common.Colour
    mpTextStyle: int
    mbEnableHighlight: bool

class cTk2dText_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
        ("Construct_88", ctypes.c_char * 0x8),  # void (__fastcall *)(cTk2dText *this, const cTkVector2 *, const cTkVector2 *, const cTkVector2 *, const cTk2dTextPreset *, const TkID<256> *)
        ("Construct_90", ctypes.c_char * 0x8),  # void (__fastcall *)(cTk2dText *this, const cTkVector2 *, const cTkVector2 *, const cTk2dTextPreset *, const TkID<256> *)
        ("Construct_98", ctypes.c_ulonglong),
        ("Construct_a0", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int
    Construct_88: bytes
    Construct_90: bytes
    Construct_98: int
    Construct_a0: int

class cGcNGuiGraphic_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
        ("cGcNGuiElement_dtor_10", ctypes.c_ulonglong),
        ("EditElement", ctypes.c_ulonglong),
        ("EditPresets", ctypes.c_ulonglong),
        ("RenderUIEditorToolbar", ctypes.c_ulonglong),
        ("SelectableInEditor", ctypes.c_ulonglong),
        ("SetNeedsUpdate", ctypes.c_ulonglong),
        ("Deselect", ctypes.c_ulonglong),
        ("IsSelected", ctypes.c_ulonglong),
        ("Resized", ctypes.c_ulonglong),
        ("AttachMetadata", ctypes.c_ulonglong),
        ("DetachMetadata", ctypes.c_ulonglong),
        ("IsLayer", ctypes.c_ulonglong),
        ("GetSceneTreeIcon", ctypes.c_ulonglong),
        ("GetSceneTreeText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNGuiElement *this, cTkFixedString<128,char> *)
        ("RenderAdditionalOverlayIcons", ctypes.c_ulonglong),
        ("RenderTreeControls", ctypes.c_ulonglong),
    ]
    Render: int
    GetType: int
    cGcNGuiElement_dtor_10: int
    EditElement: int
    EditPresets: int
    RenderUIEditorToolbar: int
    SelectableInEditor: int
    SetNeedsUpdate: int
    Deselect: int
    IsSelected: int
    Resized: int
    AttachMetadata: int
    DetachMetadata: int
    IsLayer: int
    GetSceneTreeIcon: int
    GetSceneTreeText: bytes
    RenderAdditionalOverlayIcons: int
    RenderTreeControls: int

class cGcHUDPulseEffect(ctypes.Structure):
    def Render(): pass

    def RenderInstance(): pass

    _fields_ = [
        ("mpLayer", ctypes.c_ulonglong),
        ("mpData", ctypes.c_ulonglong),
        ("mColour", common.Colour),
        ("mfStartTime", ctypes.c_float),
    ]
    mpLayer: int
    mpData: int
    mColour: common.Colour
    mfStartTime: float

class cGcHUDPulseManager(ctypes.Structure):
    def Add(): pass

    _fields_ = [
        ("mArray", ctypes.c_char * 0x710),  # cTkClassPool<cGcHUDPulseEffect,32>
    ]
    mArray: bytes

class cGcNGuiTextSpecial_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
        ("cGcNGuiElement_dtor_10", ctypes.c_ulonglong),
        ("EditElement", ctypes.c_ulonglong),
        ("EditPresets", ctypes.c_ulonglong),
        ("RenderUIEditorToolbar", ctypes.c_ulonglong),
        ("SelectableInEditor", ctypes.c_ulonglong),
        ("SetNeedsUpdate", ctypes.c_ulonglong),
        ("Deselect", ctypes.c_ulonglong),
        ("IsSelected", ctypes.c_ulonglong),
        ("Resized", ctypes.c_ulonglong),
        ("AttachMetadata", ctypes.c_ulonglong),
        ("DetachMetadata", ctypes.c_ulonglong),
        ("IsLayer", ctypes.c_ulonglong),
        ("GetSceneTreeIcon", ctypes.c_ulonglong),
        ("GetSceneTreeText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNGuiElement *this, cTkFixedString<128,char> *)
        ("RenderAdditionalOverlayIcons", ctypes.c_ulonglong),
        ("RenderTreeControls", ctypes.c_ulonglong),
        ("Convert", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
    ]
    Render: int
    GetType: int
    cGcNGuiElement_dtor_10: int
    EditElement: int
    EditPresets: int
    RenderUIEditorToolbar: int
    SelectableInEditor: int
    SetNeedsUpdate: int
    Deselect: int
    IsSelected: int
    Resized: int
    AttachMetadata: int
    DetachMetadata: int
    IsLayer: int
    GetSceneTreeIcon: int
    GetSceneTreeText: bytes
    RenderAdditionalOverlayIcons: int
    RenderTreeControls: int
    Convert: int
    SetText: int

class cGcRenderableScreen_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcRenderableScreen_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("CleanupResources", ctypes.c_ulonglong),
    ]
    cGcRenderableScreen_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    EnqueueRender: int
    UpdateRender: int
    Release: int
    Destruct: int
    RenderNGui: int
    CleanupResources: int

class cGcNotificationSequence_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequence_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsAppropriate", ctypes.c_ulonglong),
        ("GetMissionDataTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetSubtitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<256,char> *)
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetMissionIcon", ctypes.c_ulonglong),
        ("GetMissionIconSelected", ctypes.c_ulonglong),
        ("GetMissionIconNotSelected", ctypes.c_ulonglong),
        ("ShowFutureObjectives", ctypes.c_ulonglong),
        ("IsPrimary", ctypes.c_ulonglong),
        ("IsLegacy", ctypes.c_ulonglong),
        ("GetClass", ctypes.c_ulonglong),
        ("UseCommunityMissionForLog", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("IsCritical", ctypes.c_ulonglong),
        ("BlocksPinning", ctypes.c_ulonglong),
        ("IsProcedural", ctypes.c_ulonglong),
        ("DataAllowsRenounce", ctypes.c_ulonglong),
        ("RefreshLogInfo", ctypes.c_ulonglong),
        ("GetCurrentCategory", ctypes.c_ulonglong),
        ("GetCurrentColour", ctypes.c_ulonglong),
        ("GetCurrentIcon", ctypes.c_ulonglong),
        ("GetIconAtStage", ctypes.c_char * 0x8),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *(__fastcall *)(cGcNotificationSequence *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *result, int)
        ("GetCurrentFrontendPageHint", ctypes.c_ulonglong),
        ("GetCurrentInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("GetCurrentFrontendPageLocID", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, TkID<256> *)
        ("GetCurrentBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("CheckOptionCondition", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, const TkID<256> *, cTkFixedString<1024,char> *)
        ("WriteItemText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *, bool)
        ("Restart", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("DebugNguiMissionInfo", ctypes.c_ulonglong),
        ("HasText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetParticipantTypeFromPointer", ctypes.c_ulonglong),
        ("GetParticipantPointerFromType", ctypes.c_ulonglong),
        ("GetNumberOfParticipants", ctypes.c_ulonglong),
        ("HasBeenHandedIn", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("CanBegin", ctypes.c_ulonglong),
        ("CanBeRenounced", ctypes.c_ulonglong),
        ("IsForcingPageHint", ctypes.c_ulonglong),
        ("IsForcingBuildMenuHint", ctypes.c_ulonglong),
        ("IsTelemetryUpload", ctypes.c_ulonglong),
        ("GetMultiplayerMissionType", ctypes.c_ulonglong),
    ]
    cGcNotificationSequence_dtor_0: int
    Update: int
    IsAppropriate: int
    GetMissionDataTitle: bytes
    GetTitle: bytes
    GetSubtitle: bytes
    GetDescription: bytes
    GetMissionIcon: int
    GetMissionIconSelected: int
    GetMissionIconNotSelected: int
    ShowFutureObjectives: int
    IsPrimary: int
    IsLegacy: int
    GetClass: int
    UseCommunityMissionForLog: bytes
    IsCritical: int
    BlocksPinning: int
    IsProcedural: int
    DataAllowsRenounce: int
    RefreshLogInfo: int
    GetCurrentCategory: int
    GetCurrentColour: int
    GetCurrentIcon: int
    GetIconAtStage: bytes
    GetCurrentFrontendPageHint: int
    GetCurrentInventoryHint: bytes
    GetCurrentFrontendPageLocID: bytes
    GetCurrentBuildMenuHint: bytes
    CheckOptionCondition: bytes
    WriteItemText: bytes
    Restart: int
    Cancel: int
    DebugNguiMissionInfo: int
    HasText: int
    GetText: bytes
    GetParticipantTypeFromPointer: int
    GetParticipantPointerFromType: int
    GetNumberOfParticipants: int
    HasBeenHandedIn: int
    NGuiRender: int
    Load: int
    Save: int
    CanBegin: int
    CanBeRenounced: int
    IsForcingPageHint: int
    IsForcingBuildMenuHint: int
    IsTelemetryUpload: int
    GetMultiplayerMissionType: int

class cGcNotificationSequenceStage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGroup_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceGroup_dtor_0", ctypes.c_ulonglong),
        ("GetObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceGroup *this, cTkFixedString<128,char> *)
        ("GetObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceGroup *this, cTkFixedString<1024,char> *)
        ("GetTargetMissionSurveyId", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceGroup *this, TkID<128> *result)
        ("GetTargetMissionSurveyDefinitelyExists", ctypes.c_ulonglong),
        ("GetPinnedMissionBuildTarget", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceGroup *this, TkID<128> *result)
    ]
    cGcNotificationSequenceGroup_dtor_0: int
    GetObjective: bytes
    GetObjectiveTip: bytes
    GetTargetMissionSurveyId: bytes
    GetTargetMissionSurveyDefinitelyExists: int
    GetPinnedMissionBuildTarget: bytes

class cGcNotifyGenericSection_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotifyGenericSection_dtor_0", ctypes.c_ulonglong),
        ("GetTargetMissionSurveyId", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotifyGenericSection *this, TkID<128> *result)
        ("GetTargetMissionSurveyDefinitelyExists", ctypes.c_ulonglong),
    ]
    cGcNotifyGenericSection_dtor_0: int
    GetTargetMissionSurveyId: bytes
    GetTargetMissionSurveyDefinitelyExists: int

class cGcGenericSectionCondition(ctypes.Structure):
    def GetDescription(): pass

    def GetBuildMenuHint(): pass

    def EqualityText(): pass

    def DebugText(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpMission", ctypes.c_ulonglong),
    ]
    __vftable: int
    mpMission: int

class cGcGenericSectionCondition_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConsequence(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpMission", ctypes.c_ulonglong),
    ]
    __vftable: int
    mpMission: int

class cGcGenericSectionConsequence_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionConsequence_dtor_0", ctypes.c_ulonglong),
        ("DoConsequence", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<256,char> *(__fastcall *)(cGcGenericSectionConsequence *this, cTkFixedString<256,char> *result)
    ]
    cGcGenericSectionConsequence_dtor_0: int
    DoConsequence: int
    GetDescription: bytes

class cGcNotificationSequenceEditor(ctypes.Structure):
    def RenderImGuiCallback(): pass

    def RenderConditions(): pass

    def RenderSequences(): pass

    _fields_ = [
        ("mpMissionTable", ctypes.c_ulonglong),
        ("maMissions", ctypes.c_char * 0x18),  # std::vector<cGcGenericMissionSequence *,TkSTLAllocatorShim<cGcGenericMissionSequence *,8,-1> >
    ]
    mpMissionTable: int
    maMissions: bytes

class AtlasCommunalMissionTier(ctypes.Structure):
    _fields_ = [
        ("luMissionIndex", ctypes.c_uint32),
        ("luTierReached", ctypes.c_uint32),
    ]
    luMissionIndex: int
    luTierReached: int

class cGcAtlasCommunityData(ctypes.Structure):
    def tilde_cGcAtlasCommunityData(): pass

    def Update(): pass

    _fields_ = [
        ("muCurrentCMIndex", ctypes.c_uint64),
        ("muCurrentCMStartEpoch", ctypes.c_uint64),
        ("muCurrentCMEndEpoch", ctypes.c_uint64),
        ("muNextCMStartEpoch", ctypes.c_uint64),
        ("muNextCMEndEpoch", ctypes.c_uint64),
        ("muMyCMContribution", ctypes.c_uint64),
        ("muMaxIndividualContribution", ctypes.c_uint64),
        ("maContributionLevels", ctypes.c_char * 0x18),  # std::vector<unsigned __int64,TkSTLAllocatorShim<unsigned __int64,8,-1> >
        ("muCurrentTotalContribution", ctypes.c_uint64),
        ("mPreviousTiersReached", ctypes.c_char * 0x18),  # std::vector<AtlasCommunalMissionTier,TkSTLAllocatorShim<AtlasCommunalMissionTier,4,-1> >
        ("muSecondsBetweenCmRefreshes", ctypes.c_uint32),
        ("mbUpdatePending", ctypes.c_ubyte),
        ("mbRequestUpdate", ctypes.c_ubyte),
        ("muTimeOfLastRefresh", ctypes.c_uint64),
        ("mRequestHandle", ctypes.c_uint64),
    ]
    muCurrentCMIndex: int
    muCurrentCMStartEpoch: int
    muCurrentCMEndEpoch: int
    muNextCMStartEpoch: int
    muNextCMEndEpoch: int
    muMyCMContribution: int
    muMaxIndividualContribution: int
    maContributionLevels: bytes
    muCurrentTotalContribution: int
    mPreviousTiersReached: bytes
    muSecondsBetweenCmRefreshes: int
    mbUpdatePending: bool
    mbRequestUpdate: bool
    muTimeOfLastRefresh: int
    mRequestHandle: int

class cGcPhotoListener(ctypes.Structure):
    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mSuccessMsg", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
    ]
    __vftable: int
    mSuccessMsg: bytes

class cGcPhotoListener_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcPhotoListener_dtor_0", ctypes.c_ulonglong),
        ("NotifySuccess", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
    ]
    cGcPhotoListener_dtor_0: int
    NotifySuccess: int
    Clone: int

class cGcPhotoManager(ctypes.Structure):
    def AnalysePhotoScene(): pass

    def AcknowledgePhotoRequests(): pass

    _fields_ = [
        ("maPhotoRequests", ctypes.c_char * 0x18),  # std::vector<cGcPhotoRequest,TkSTLAllocatorShim<cGcPhotoRequest,8,-1> >
    ]
    maPhotoRequests: bytes

class AnimateItems(ctypes.Structure):
    def GetPos(): pass

    _fields_ = [
        ("mbRequested", ctypes.c_ubyte),
        ("mfFrom", ctypes.c_float),
        ("mfTo", ctypes.c_float),
        ("mfAnimTime", ctypes.c_float),
        ("mfTimer", ctypes.c_float),
        ("mfWait", ctypes.c_float),
    ]
    mbRequested: bool
    mfFrom: float
    mfTo: float
    mfAnimTime: float
    mfTimer: float
    mfWait: float

class cGcQuickMenuState_vtbl(ctypes.Structure):
    _fields_ = [
        ("Update", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("ShouldRenderCursor", ctypes.c_ulonglong),
        ("ShouldTriggerAction", ctypes.c_ulonglong),
        ("TriggerCurrentAction", ctypes.c_ulonglong),
        ("TriggerClose", ctypes.c_ulonglong),
        ("UpdateControlsString", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcQuickMenuState *this, cTkFixedString<256,char> *, cTkFixedString<256,char> *)
        ("ShouldClose", ctypes.c_ulonglong),
        ("CanPlayerMove", ctypes.c_ulonglong),
        ("HasParentMenu", ctypes.c_ulonglong),
        ("AllowedToOpenFrontendPages", ctypes.c_ulonglong),
        ("IsSlotHighlighted", ctypes.c_ulonglong),
        ("IsSlotSelected", ctypes.c_ulonglong),
    ]
    Update: int
    UpdateRender: int
    Render: int
    Activate: int
    Close: int
    ShouldRenderCursor: int
    ShouldTriggerAction: int
    TriggerCurrentAction: int
    TriggerClose: int
    UpdateControlsString: bytes
    ShouldClose: int
    CanPlayerMove: int
    HasParentMenu: int
    AllowedToOpenFrontendPages: int
    IsSlotHighlighted: int
    IsSlotSelected: int

class cGcQuickActionMenu_vtbl(ctypes.Structure):
    _fields_ = [
        ("Update", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("ShouldRenderCursor", ctypes.c_ulonglong),
        ("ShouldTriggerAction", ctypes.c_ulonglong),
        ("TriggerCurrentAction", ctypes.c_ulonglong),
        ("TriggerClose", ctypes.c_ulonglong),
        ("UpdateControlsString", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcQuickMenuState *this, cTkFixedString<256,char> *, cTkFixedString<256,char> *)
        ("ShouldClose", ctypes.c_ulonglong),
        ("CanPlayerMove", ctypes.c_ulonglong),
        ("HasParentMenu", ctypes.c_ulonglong),
        ("AllowedToOpenFrontendPages", ctypes.c_ulonglong),
        ("IsSlotHighlighted", ctypes.c_ulonglong),
        ("IsSlotSelected", ctypes.c_ulonglong),
    ]
    Update: int
    UpdateRender: int
    Render: int
    Activate: int
    Close: int
    ShouldRenderCursor: int
    ShouldTriggerAction: int
    TriggerCurrentAction: int
    TriggerClose: int
    UpdateControlsString: bytes
    ShouldClose: int
    CanPlayerMove: int
    HasParentMenu: int
    AllowedToOpenFrontendPages: int
    IsSlotHighlighted: int
    IsSlotSelected: int

class TemplateHandleSet(ctypes.Structure):
    _fields_ = [
        ("muStart", ctypes.c_uint16),
        ("muCount", ctypes.c_uint16),
    ]
    muStart: int
    muCount: int

class CacheMask(ctypes.Structure):
    _fields_ = [
        ("muMask", ctypes.c_uint64 * 0x4),  # unsigned __int64[4]
    ]
    muMask: list[ctypes.c_uint64]

class cGcBuildMenu_vtbl(ctypes.Structure):
    _fields_ = [
        ("Update", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("ShouldRenderCursor", ctypes.c_ulonglong),
        ("ShouldTriggerAction", ctypes.c_ulonglong),
        ("TriggerCurrentAction", ctypes.c_ulonglong),
        ("TriggerClose", ctypes.c_ulonglong),
        ("UpdateControlsString", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcQuickMenuState *this, cTkFixedString<256,char> *, cTkFixedString<256,char> *)
        ("ShouldClose", ctypes.c_ulonglong),
        ("CanPlayerMove", ctypes.c_ulonglong),
        ("HasParentMenu", ctypes.c_ulonglong),
        ("AllowedToOpenFrontendPages", ctypes.c_ulonglong),
        ("IsSlotHighlighted", ctypes.c_ulonglong),
        ("IsSlotSelected", ctypes.c_ulonglong),
    ]
    Update: int
    UpdateRender: int
    Render: int
    Activate: int
    Close: int
    ShouldRenderCursor: int
    ShouldTriggerAction: int
    TriggerCurrentAction: int
    TriggerClose: int
    UpdateControlsString: bytes
    ShouldClose: int
    CanPlayerMove: int
    HasParentMenu: int
    AllowedToOpenFrontendPages: int
    IsSlotHighlighted: int
    IsSlotSelected: int

class cGcTerrainEditMenu_vtbl(ctypes.Structure):
    _fields_ = [
        ("Update", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("ShouldRenderCursor", ctypes.c_ulonglong),
        ("ShouldTriggerAction", ctypes.c_ulonglong),
        ("TriggerCurrentAction", ctypes.c_ulonglong),
        ("TriggerClose", ctypes.c_ulonglong),
        ("UpdateControlsString", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcQuickMenuState *this, cTkFixedString<256,char> *, cTkFixedString<256,char> *)
        ("ShouldClose", ctypes.c_ulonglong),
        ("CanPlayerMove", ctypes.c_ulonglong),
        ("HasParentMenu", ctypes.c_ulonglong),
        ("AllowedToOpenFrontendPages", ctypes.c_ulonglong),
        ("IsSlotHighlighted", ctypes.c_ulonglong),
        ("IsSlotSelected", ctypes.c_ulonglong),
    ]
    Update: int
    UpdateRender: int
    Render: int
    Activate: int
    Close: int
    ShouldRenderCursor: int
    ShouldTriggerAction: int
    TriggerCurrentAction: int
    TriggerClose: int
    UpdateControlsString: bytes
    ShouldClose: int
    CanPlayerMove: int
    HasParentMenu: int
    AllowedToOpenFrontendPages: int
    IsSlotHighlighted: int
    IsSlotSelected: int

class cGcGalaxyPathMenu_vtbl(ctypes.Structure):
    _fields_ = [
        ("Update", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("ShouldRenderCursor", ctypes.c_ulonglong),
        ("ShouldTriggerAction", ctypes.c_ulonglong),
        ("TriggerCurrentAction", ctypes.c_ulonglong),
        ("TriggerClose", ctypes.c_ulonglong),
        ("UpdateControlsString", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcQuickMenuState *this, cTkFixedString<256,char> *, cTkFixedString<256,char> *)
        ("ShouldClose", ctypes.c_ulonglong),
        ("CanPlayerMove", ctypes.c_ulonglong),
        ("HasParentMenu", ctypes.c_ulonglong),
        ("AllowedToOpenFrontendPages", ctypes.c_ulonglong),
        ("IsSlotHighlighted", ctypes.c_ulonglong),
        ("IsSlotSelected", ctypes.c_ulonglong),
    ]
    Update: int
    UpdateRender: int
    Render: int
    Activate: int
    Close: int
    ShouldRenderCursor: int
    ShouldTriggerAction: int
    TriggerCurrentAction: int
    TriggerClose: int
    UpdateControlsString: bytes
    ShouldClose: int
    CanPlayerMove: int
    HasParentMenu: int
    AllowedToOpenFrontendPages: int
    IsSlotHighlighted: int
    IsSlotSelected: int

class MessageBase(ctypes.Structure):
    def MessageBase(): pass

    _fields_ = [
        ("mfMessageTime", ctypes.c_float),
        ("mfMessageTimeTotal", ctypes.c_float),
        ("mfStartTime", ctypes.c_float),
        ("mFade", ctypes.c_float),
    ]
    mfMessageTime: float
    mfMessageTimeTotal: float
    mfStartTime: float
    mFade: float

class OSDMessageEffect(ctypes.Structure):
    _fields_ = [
        ("mpLayer", ctypes.c_ulonglong),
        ("mfStartTime", ctypes.c_float),
    ]
    mpLayer: int
    mfStartTime: float

class StatsMessages(ctypes.Structure):
    _fields_ = [
        ("maMessages", ctypes.c_char * 0x18),  # std::vector<OSDMessage,TkSTLAllocatorShim<OSDMessage,16,-1> >
        ("mAudioCue", ctypes.c_uint32),
        ("mbIsShowing", ctypes.c_ubyte),
        ("mbStatsRequireBars", ctypes.c_ubyte),
    ]
    maMessages: bytes
    mAudioCue: int
    mbIsShowing: bool
    mbStatsRequireBars: bool

class cGcFrontendBackgroundEffect(ctypes.Structure):
    _fields_ = [
        ("mpLayer", ctypes.c_ulonglong),
        ("mMousePos", common.Vector2f),
    ]
    mpLayer: int
    mMousePos: common.Vector2f

class ScrollBarState(ctypes.Structure):
    def ScrollBarRenderFunction(): pass

    _fields_ = [
        ("miNumEntries", ctypes.c_int32),
        ("miCurrentStartIndex", ctypes.c_int32),
        ("miEntriesPerPage", ctypes.c_int32),
        ("mfLastSeenPos", common.Vector2f),
        ("mfLastSeenSize", common.Vector2f),
    ]
    miNumEntries: int
    miCurrentStartIndex: int
    miEntriesPerPage: int
    mfLastSeenPos: common.Vector2f
    mfLastSeenSize: common.Vector2f

class cGcDiscoveryPageDataSearchTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcDiscoveryPageDataSearchTask_dtor_0", ctypes.c_ulonglong),
        ("IsComplete", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
    ]
    cGcDiscoveryPageDataSearchTask_dtor_0: int
    IsComplete: int
    Start: int
    Update: int
    Destroy: int

class TextScrollBarState(ctypes.Structure):
    def TextScrollBarRenderFunction(): pass

    _fields_ = [
        ("mfCurrentScrollOffsetPercentage", ctypes.c_float),
        ("mfTextOnScreenPercentage", ctypes.c_float),
        ("miCurrentPage", ctypes.c_int32),
        ("miMaxPageIndex", ctypes.c_int32),
        ("mbInactive", ctypes.c_ubyte),
        ("mbFirstUpdateAfterReset", ctypes.c_ubyte),
        ("mbMoveToPreviousPage", ctypes.c_ubyte),
        ("mbMoveToNextPage", ctypes.c_ubyte),
        ("mbNewTextSet", ctypes.c_ubyte),
        ("mbForceScrollToBottom", ctypes.c_ubyte),
    ]
    mfCurrentScrollOffsetPercentage: float
    mfTextOnScreenPercentage: float
    miCurrentPage: int
    miMaxPageIndex: int
    mbInactive: bool
    mbFirstUpdateAfterReset: bool
    mbMoveToPreviousPage: bool
    mbMoveToNextPage: bool
    mbNewTextSet: bool
    mbForceScrollToBottom: bool

class cGcMissionPageData(ctypes.Structure):
    def UpdateSelectedMission(): pass

    _fields_ = [
        ("mbReset", ctypes.c_ubyte),
        ("miCurrentSecMissionPage", ctypes.c_int32),
        ("mfTimeSinceLastTransition", ctypes.c_float),
        ("mbAutoScrollDone", ctypes.c_ubyte),
    ]
    mbReset: bool
    miCurrentSecMissionPage: int
    mfTimeSinceLastTransition: float
    mbAutoScrollDone: bool

class cGcSeasonPageData(ctypes.Structure):
    def CacheStats(): pass

    def SetStageIndex(): pass

    def GetTransitionProgress(): pass

    def GetAlpha(): pass

    def SetupInitialPositions(): pass

    _fields_ = [
        ("miExpeditionDistance", ctypes.c_int32),
        ("miSystemsDiscovered", ctypes.c_int32),
        ("mfDistanceWalked", ctypes.c_float),
        ("miNanites", ctypes.c_int32),
        ("miMoney", ctypes.c_int32),
        ("mafInitialRingPositions", ctypes.c_char * 0x18),  # std::vector<float,TkSTLAllocatorShim<float,4,-1> >
        ("mafInitialStarPositions", ctypes.c_char * 0x18),  # std::vector<float,TkSTLAllocatorShim<float,4,-1> >
        ("mafInitialStarSizes", ctypes.c_char * 0x18),  # std::vector<float,TkSTLAllocatorShim<float,4,-1> >
        ("mpProgressStyleData", ctypes.c_ulonglong),
        ("mfStageChangeStartTime", ctypes.c_double),
        ("mfStageChangeTime", ctypes.c_double),
        ("miPreviousStageIndex", ctypes.c_int32),
        ("miStageIndex", ctypes.c_int32),
        ("miNextStageIndex", ctypes.c_int32),
        ("mbShowingLatestStage", ctypes.c_ubyte),
        ("mbShowingFinalStage", ctypes.c_int32),
    ]
    miExpeditionDistance: int
    miSystemsDiscovered: int
    mfDistanceWalked: float
    miNanites: int
    miMoney: int
    mafInitialRingPositions: bytes
    mafInitialStarPositions: bytes
    mafInitialStarSizes: bytes
    mpProgressStyleData: int
    mfStageChangeStartTime: float
    mfStageChangeTime: float
    miPreviousStageIndex: int
    miStageIndex: int
    miNextStageIndex: int
    mbShowingLatestStage: bool
    mbShowingFinalStage: int

class InventoryTransferData(ctypes.Structure):
    def tilde_InventoryTransferData(): pass

    def InventoryTransferData(): pass

    _fields_ = [
        ("maLeft", ctypes.c_char * 0x18),  # std::vector<InventoryData,TkSTLAllocatorShim<InventoryData,8,-1> >
        ("miCurrentLeft", ctypes.c_int32),
        ("maRight", ctypes.c_char * 0x18),  # std::vector<InventoryData,TkSTLAllocatorShim<InventoryData,8,-1> >
        ("miCurrentRight", ctypes.c_int32),
    ]
    maLeft: bytes
    miCurrentLeft: int
    maRight: bytes
    miCurrentRight: int

class cGcHoldInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsActive", ctypes.c_ulonglong),
        ("NeedsWarning", ctypes.c_ulonglong),
        ("EndInteraction", ctypes.c_ulonglong),
    ]
    IsActive: int
    NeedsWarning: int
    EndInteraction: int

class cGcCountInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsActive", ctypes.c_ulonglong),
        ("NeedsWarning", ctypes.c_ulonglong),
        ("EndInteraction", ctypes.c_ulonglong),
    ]
    IsActive: int
    NeedsWarning: int
    EndInteraction: int

class cGcFrontendStatCircle(ctypes.Structure):
    _fields_ = [
        ("mpLayer", ctypes.c_ulonglong),
        ("mpStats", ctypes.c_ulonglong),
        ("mValues", ctypes.c_char * 0x2F4),  # std::array<float,189>
        ("mValuesNoBonus", ctypes.c_char * 0x2F4),  # std::array<float,189>
        ("mfWidth", ctypes.c_float),
        ("mfAnim", ctypes.c_float),
    ]
    mpLayer: int
    mpStats: int
    mValues: bytes
    mValuesNoBonus: bytes
    mfWidth: float
    mfAnim: float

class PhotoModeControl(ctypes.Structure):
    def PhotoModeControl(): pass

    def IsValid(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mTitle", ctypes.c_ulonglong),
        ("mIsValid", ctypes.c_char * 0x40),  # std::function<bool __cdecl(void)>
    ]
    __vftable: int
    mTitle: int
    mIsValid: bytes

class PhotoModeControl_vtbl(ctypes.Structure):
    _fields_ = [
        ("PhotoModeControl_dtor_0", ctypes.c_ulonglong),
        ("GetValueAsString", ctypes.c_char * 0x8),  # void (__fastcall *)(PhotoModeControl *this, cTkFixedString<512,char> *)
        ("Reset", ctypes.c_ulonglong),
        ("Increase", ctypes.c_ulonglong),
        ("Decrease", ctypes.c_ulonglong),
        ("EnsureInBounds", ctypes.c_ulonglong),
        ("IsValid", ctypes.c_ulonglong),
    ]
    PhotoModeControl_dtor_0: int
    GetValueAsString: bytes
    Reset: int
    Increase: int
    Decrease: int
    EnsureInBounds: int
    IsValid: int

class PhotoModeControlGroup(ctypes.Structure):
    _fields_ = [
        ("mTitle", ctypes.c_ulonglong),
        ("mControls", ctypes.c_char * 0x18),  # std::vector<PhotoModeControl *,TkSTLAllocatorShim<PhotoModeControl *,8,-1> >
    ]
    mTitle: int
    mControls: bytes

class cGcPhotoModeUI_vtbl(ctypes.Structure):
    _fields_ = [
        ("BackgroundExecutionChanged", ctypes.c_ulonglong),
        ("SystemUIOverlayChange", ctypes.c_ulonglong),
        ("ShareMenuOpened", ctypes.c_ulonglong),
        ("SystemPlayModeChanged", ctypes.c_ulonglong),
    ]
    BackgroundExecutionChanged: int
    SystemUIOverlayChange: int
    ShareMenuOpened: int
    SystemPlayModeChanged: int

class cGcReportBaseUI(ctypes.Structure):
    def EnterReportBaseMode(): pass

    def ExitReportBaseMode(): pass

    def OnDialogClose(): pass


class cGcBaseUploadUI(ctypes.Structure):
    def Update(): pass

    def Render(): pass

    def BeginBaseUploadUI(): pass

    def HidePage(): pass

    def OnPhotoTakenCallback(): pass

    def OnSnapshotTakenCallback(): pass

    def OnPhotoTakenCallbackInternal(): pass

    def OnSnapshotTakenCallbackInternal(): pass

    def GetDefaultCameraMatrix(): pass

    def HandleFeaturedBaseUI(): pass

    def HideBaseComputerButtons(): pass

    _fields_ = [
        ("mpScreenshotImage", ctypes.c_ulonglong),
        ("mbRenamingTitle", ctypes.c_ubyte),
        ("mbBaseAdminScreenshotTaken", ctypes.c_ubyte),
        ("mbEnqueueSaveAndUploadBase", ctypes.c_ubyte),
        ("mbAwaitingReconnect", ctypes.c_ubyte),
        ("mbUGCStarted", ctypes.c_ubyte),
    ]
    mpScreenshotImage: int
    mbRenamingTitle: bool
    mbBaseAdminScreenshotTaken: bool
    mbEnqueueSaveAndUploadBase: bool
    mbAwaitingReconnect: bool
    mbUGCStarted: bool

class cGcFrontendPageSystemHoover(ctypes.Structure):
    def Render(): pass

    def Update(): pass

    def GetSystemHoover(): pass

    _fields_ = [
        ("mInputPos", common.Vector2f),
        ("mOutputPos", common.Vector2f),
    ]
    mInputPos: common.Vector2f
    mOutputPos: common.Vector2f

class cGcFrontendPageEggMachine(ctypes.Structure):
    def GetEggMachine(): pass

    def Render(): pass

    def DoSubtitle(): pass

    def GetInputSlotsLayers(): pass

    def DoSlot(): pass

    def DoSlotDosage(): pass

    def DoButton(): pass

    def DoEggInfo(): pass

    def DoModifyingInfoSlotAnim(): pass

    def GiveBackItemsOnClose(): pass

    _fields_ = [
        ("mfEggModifyStartTime", ctypes.c_float),
    ]
    mfEggModifyStartTime: float

class cGcFrontendPageVehicleRace(ctypes.Structure):
    def Render(): pass

    def RenderResultsScreen(): pass

    def RenderInviteScreen(): pass

    def RenderMainScreen(): pass


class cGcFrontendPageBasePartPalette(ctypes.Structure):
    def Activate(): pass

    def Render(): pass

    def UpdateRender(): pass

    _fields_ = [
        ("maLightNodes", ctypes.c_uint32 * 0x4),  # TkHandle[4]
        ("mAvailablePalettes", ctypes.c_char * 0x10),  # cTkBitArray<unsigned __int64,1,128>
        ("mAvailableMaterials", ctypes.c_char * 0x4),  # cTkBitArray<unsigned int,1,32>
        ("mu8ObjectPaletteIndex", ctypes.c_uint8),
        ("mu8SelectedPaletteIndex", ctypes.c_uint8),
        ("mu8ObjectMaterialIndex", ctypes.c_uint8),
        ("mu8SelectedMaterialIndex", ctypes.c_uint8),
        ("mbPaletteChanged", ctypes.c_ubyte),
        ("mbMaterialChanged", ctypes.c_ubyte),
    ]
    maLightNodes: list[ctypes.c_uint32]
    mAvailablePalettes: bytes
    mAvailableMaterials: bytes
    mu8ObjectPaletteIndex: int
    mu8SelectedPaletteIndex: int
    mu8ObjectMaterialIndex: int
    mu8SelectedMaterialIndex: int
    mbPaletteChanged: bool
    mbMaterialChanged: bool

class cGcFrontendPageInfoPortal(ctypes.Structure):
    def Render(): pass

    def DoGuideContent(): pass

    def DoMilestonesContent(): pass

    def DoCataloguesContent(): pass

    def LoadStoriesPageData(): pass

    def FindWikiHint(): pass

    def GetRandomJourneyStatIndex(): pass

    def PrepareWikiItems(): pass

    _fields_ = [
        ("maiCatalogueTotalCount", ctypes.c_char * 0x18),  # std::vector<int,TkSTLAllocatorShim<int,4,-1> >
        ("maiCatalogueUnlockedCount", ctypes.c_char * 0x18),  # std::vector<int,TkSTLAllocatorShim<int,4,-1> >
        ("miSelectedWikiCatIdx", ctypes.c_int32),
        ("miSelectedWikiTopicIdx", ctypes.c_int32),
        ("miRandomMilestoneIdx", ctypes.c_int32),
        ("miRandomFallbackMilestoneIdx", ctypes.c_int32),
        ("mfLastGuideTopicChangeTime", ctypes.c_double),
        ("mfLastMilestonesChangeTime", ctypes.c_double),
    ]
    maiCatalogueTotalCount: bytes
    maiCatalogueUnlockedCount: bytes
    miSelectedWikiCatIdx: int
    miSelectedWikiTopicIdx: int
    miRandomMilestoneIdx: int
    miRandomFallbackMilestoneIdx: int
    mfLastGuideTopicChangeTime: float
    mfLastMilestonesChangeTime: float

class cGcFrontendPageManageSettlement(ctypes.Structure):
    def Construct(): pass

    def Update(): pass

    def Render(): pass

    def Setup(): pass

    def CleanUpAndClose(): pass

    def RollInitialStatsAndPerks(): pass

    def SetupProductionInventory(): pass

    def RenderHeader(): pass

    def RenderStats(): pass

    def RenderPerks(): pass

    def RenderProduction(): pass

    def RenderProductionSlot(): pass

    def RenderDebt(): pass

    def RenderUnowned(): pass

    def RenderReport(): pass

    def OnConfirmAbandonSettlementClosed(): pass

    def GetStatFillColour(): pass

    _fields_ = [
        ("mpMonumentBuilding", ctypes.c_ulonglong),
        ("mpSettlementState", ctypes.c_ulonglong),
        ("mSetlementSeed", common.GcSeed),
        ("maiStatValues", ctypes.c_char * 0x1C),  # cTkFixedArray<int,7>
        ("maStatsFill", ctypes.c_char * 0x150),  # cTkFixedArray<cGcMarkerRenderData,7>
        ("mProductionSlotIndexes", ctypes.c_char * 0x18),  # std::vector<cGcInventoryIndex,TkSTLAllocatorShim<cGcInventoryIndex,4,-1> >
        ("mSettlementCostId", ctypes.c_ulonglong),
        ("mbNeedsSetup", ctypes.c_ubyte),
        ("mbPendingCleanUp", ctypes.c_ubyte),
        ("mbTriggeredByReward", ctypes.c_ubyte),
        ("mbSettlementHasOwner", ctypes.c_ubyte),
        ("mbSettlementOwned", ctypes.c_ubyte),
        ("mbSettlementHasDebt", ctypes.c_ubyte),
    ]
    mpMonumentBuilding: int
    mpSettlementState: int
    mSetlementSeed: common.GcSeed
    maiStatValues: bytes
    maStatsFill: bytes
    mProductionSlotIndexes: bytes
    mSettlementCostId: int
    mbNeedsSetup: bool
    mbPendingCleanUp: bool
    mbTriggeredByReward: bool
    mbSettlementHasOwner: bool
    mbSettlementOwned: bool
    mbSettlementHasDebt: bool

class cGcFrontendManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("BackgroundExecutionChanged", ctypes.c_ulonglong),
        ("SystemUIOverlayChange", ctypes.c_ulonglong),
        ("ShareMenuOpened", ctypes.c_ulonglong),
        ("SystemPlayModeChanged", ctypes.c_ulonglong),
    ]
    BackgroundExecutionChanged: int
    SystemUIOverlayChange: int
    ShareMenuOpened: int
    SystemPlayModeChanged: int

class cTkCameraBehaviourTimeBased_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cTkCameraBehaviourInterpolate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cTkCameraBehaviourFly_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("ResetStartMatrix", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    ResetStartMatrix: int

class cGcCameraBehaviourFly_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("ResetStartMatrix", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    ResetStartMatrix: int

class cTkCameraBehaviourOffset_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetOffsetMatrix", ctypes.c_ulonglong),
        ("GetObjectMatrix", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetOffsetMatrix: int
    GetObjectMatrix: int

class cGcCameraBehaviourOffset_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetOffsetMatrix", ctypes.c_ulonglong),
        ("GetObjectMatrix", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetOffsetMatrix: int
    GetObjectMatrix: int

class cGcCameraBehaviourGalacticTransition_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourGalacticNavigation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourGalacticLookAt_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetDof", ctypes.c_ulonglong),
        ("ReadDofParameters", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetDof: int
    ReadDofParameters: int

class cGcCameraBehaviourLookAt_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourAerialView_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourScreenshot_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourAmbient_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("ResetStartMatrix", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    ResetStartMatrix: int

class cTkHavokProxyCharacterController_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class cGcCameraBehaviourPhotoMode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourThirdPerson_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetDof", ctypes.c_ulonglong),
        ("ReadDofParameters", ctypes.c_ulonglong),
        ("GetCloseFactor", ctypes.c_ulonglong),
        ("GetDesiredFollowSettingsData", ctypes.c_ulonglong),
        ("ApplyCollisions", ctypes.c_ulonglong),
        ("StartCollisionJobs", ctypes.c_ulonglong),
        ("GetEnableAutoRotate", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetDof: int
    ReadDofParameters: int
    GetCloseFactor: int
    GetDesiredFollowSettingsData: int
    ApplyCollisions: int
    StartCollisionJobs: int
    GetEnableAutoRotate: int

class cGcCameraBehaviourPlayerThirdPerson_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetDof", ctypes.c_ulonglong),
        ("ReadDofParameters", ctypes.c_ulonglong),
        ("GetCloseFactor", ctypes.c_ulonglong),
        ("GetDesiredFollowSettingsData", ctypes.c_ulonglong),
        ("ApplyCollisions", ctypes.c_ulonglong),
        ("StartCollisionJobs", ctypes.c_ulonglong),
        ("GetEnableAutoRotate", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetDof: int
    ReadDofParameters: int
    GetCloseFactor: int
    GetDesiredFollowSettingsData: int
    ApplyCollisions: int
    StartCollisionJobs: int
    GetEnableAutoRotate: int

class cGcCameraBehaviourAnimation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourFollowTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraBehaviourModelView_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetDof", ctypes.c_ulonglong),
        ("ReadDofParameters", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetDof: int
    ReadDofParameters: int

class cGcCameraBehaviourShipWarp_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetDof", ctypes.c_ulonglong),
        ("ReadDofParameters", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetDof: int
    ReadDofParameters: int

class cGcCameraBehaviourFreighterWarp_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
        ("GetDof", ctypes.c_ulonglong),
        ("ReadDofParameters", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int
    GetDof: int
    ReadDofParameters: int

class cGcCameraBehaviourBuildingMode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkCameraBehaviour_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Transform", ctypes.c_ulonglong),
    ]
    cTkCameraBehaviour_dtor_0: int
    Construct: int
    Setup: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    Transform: int

class cGcCameraManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSM_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkFSM *this, const cTkFSMStateOffset *, const TkID<128> *)
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("StatePrepare", ctypes.c_ulonglong),
        ("StateRelease", ctypes.c_ulonglong),
        ("Construct_30", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetActiveCameraMatrix", ctypes.c_ulonglong),
        ("GetActiveHeadMatrix", ctypes.c_ulonglong),
        ("GetCameraMatrix", ctypes.c_ulonglong),
        ("GetActiveCamera", ctypes.c_ulonglong),
        ("GetCamera", ctypes.c_ulonglong),
        ("IsBehaviourActive", ctypes.c_ulonglong),
        ("SetDebugActive", ctypes.c_ulonglong),
        ("IsDebugActive", ctypes.c_ulonglong),
        ("CopyDebugToGame", ctypes.c_ulonglong),
        ("ActivateBehaviour", ctypes.c_ulonglong),
        ("DeactivateBehaviour", ctypes.c_ulonglong),
        ("SetActiveBehaviour", ctypes.c_ulonglong),
        ("StopBehaviourUpdate", ctypes.c_ulonglong),
        ("StartBehaviourUpdate", ctypes.c_ulonglong),
        ("GetBehaviour", ctypes.c_ulonglong),
        ("GetBehaviourIndex", ctypes.c_ulonglong),
        ("AssignBehaviours", ctypes.c_char * 0x8),  # int (__fastcall *)(cTkCameraManagerTemplate<22> *this)
        ("SetDebugActive_d0", ctypes.c_ulonglong),
    ]
    cTkFSM_dtor_0: int
    Construct: bytes
    Update: int
    Destruct: int
    StatePrepare: int
    StateRelease: int
    Construct_30: int
    Prepare: int
    Release: int
    GetActiveCameraMatrix: int
    GetActiveHeadMatrix: int
    GetCameraMatrix: int
    GetActiveCamera: int
    GetCamera: int
    IsBehaviourActive: int
    SetDebugActive: int
    IsDebugActive: int
    CopyDebugToGame: int
    ActivateBehaviour: int
    DeactivateBehaviour: int
    SetActiveBehaviour: int
    StopBehaviourUpdate: int
    StartBehaviourUpdate: int
    GetBehaviour: int
    GetBehaviourIndex: int
    AssignBehaviours: bytes
    SetDebugActive_d0: int

class cTkGraphicsManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkGraphicsManagerBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("NextViewMode", ctypes.c_ulonglong),
        ("SetDimensions", ctypes.c_ulonglong),
        ("EnqueueRender", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, bool, EgRenderCullMode::Mode, EgRenderView, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("EnqueueRenderStereo", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, bool, bool, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("ResizeBuffers", ctypes.c_ulonglong),
        ("EnqueueRenderViewport", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, bool, EgRenderCullMode::Mode, EgRenderView, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("EnqueueRenderViewportStereo", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, bool, bool, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("PatchRenderTargetMatrices", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("GetViewMatrix", ctypes.c_ulonglong),
        ("GetProjectionMatrix", ctypes.c_ulonglong),
        ("InFrustum", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("IsConstructed", ctypes.c_ulonglong),
        ("CalculateNewFov", ctypes.c_ulonglong),
        ("EnqueueRenderPipeline", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, EgRenderCullMode::Mode, TkHandle)
        ("EnqueueRenderToTexture", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkMatrix34 *, const cTkMatrix34 *, float, float, EgRenderCullMode::Mode, bool, TkHandle)
        ("EnqueueRenderToTexture_b0", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, EgRenderCullMode::Mode, bool, TkHandle)
        ("EnqueueRenderToTexture_b8", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkMatrix34 *, float, float, EgRenderCullMode::Mode, bool, bool, TkHandle)
        ("EnqueueRenderToTexture_c0", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, EgRenderCullMode::Mode, TkHandle)
        ("SetShadowCascadeMask", ctypes.c_ulonglong),
        ("SetMotionBlur", ctypes.c_ulonglong),
    ]
    cTkGraphicsManagerBase_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    UpdateRender: int
    NextViewMode: int
    SetDimensions: int
    EnqueueRender: bytes
    EnqueueRenderStereo: bytes
    ResizeBuffers: int
    EnqueueRenderViewport: bytes
    EnqueueRenderViewportStereo: bytes
    PatchRenderTargetMatrices: bytes
    GetViewMatrix: int
    GetProjectionMatrix: int
    InFrustum: int
    Release: int
    Destruct: int
    IsConstructed: int
    CalculateNewFov: int
    EnqueueRenderPipeline: bytes
    EnqueueRenderToTexture: bytes
    EnqueueRenderToTexture_b0: bytes
    EnqueueRenderToTexture_b8: bytes
    EnqueueRenderToTexture_c0: bytes
    SetShadowCascadeMask: int
    SetMotionBlur: int

class cGcGraphicsManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkGraphicsManagerBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("NextViewMode", ctypes.c_ulonglong),
        ("SetDimensions", ctypes.c_ulonglong),
        ("EnqueueRender", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, bool, EgRenderCullMode::Mode, EgRenderView, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("EnqueueRenderStereo", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, bool, bool, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("ResizeBuffers", ctypes.c_ulonglong),
        ("EnqueueRenderViewport", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, bool, EgRenderCullMode::Mode, EgRenderView, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("EnqueueRenderViewportStereo", ctypes.c_char * 0x8),  # EgRenderQueueHandle (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, bool, bool, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("PatchRenderTargetMatrices", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, cTkVector2, cTkVector2, EgRenderCullMode::Mode, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("GetViewMatrix", ctypes.c_ulonglong),
        ("GetProjectionMatrix", ctypes.c_ulonglong),
        ("InFrustum", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("IsConstructed", ctypes.c_ulonglong),
        ("CalculateNewFov", ctypes.c_ulonglong),
        ("EnqueueRenderPipeline", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, EgRenderCullMode::Mode, TkHandle)
        ("EnqueueRenderToTexture", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkMatrix34 *, const cTkMatrix34 *, float, float, EgRenderCullMode::Mode, bool, TkHandle)
        ("EnqueueRenderToTexture_b0", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkPhysRelMat34 *, const cTkPhysRelMat34 *, float, float, EgRenderCullMode::Mode, bool, TkHandle)
        ("EnqueueRenderToTexture_b8", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, const cTkMatrix34 *, float, float, EgRenderCullMode::Mode, bool, bool, TkHandle)
        ("EnqueueRenderToTexture_c0", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkGraphicsManagerBase *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>, EgRenderCullMode::Mode, TkHandle)
        ("SetShadowCascadeMask", ctypes.c_ulonglong),
        ("SetMotionBlur", ctypes.c_ulonglong),
    ]
    cTkGraphicsManagerBase_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    UpdateRender: int
    NextViewMode: int
    SetDimensions: int
    EnqueueRender: bytes
    EnqueueRenderStereo: bytes
    ResizeBuffers: int
    EnqueueRenderViewport: bytes
    EnqueueRenderViewportStereo: bytes
    PatchRenderTargetMatrices: bytes
    GetViewMatrix: int
    GetProjectionMatrix: int
    InFrustum: int
    Release: int
    Destruct: int
    IsConstructed: int
    CalculateNewFov: int
    EnqueueRenderPipeline: bytes
    EnqueueRenderToTexture: bytes
    EnqueueRenderToTexture_b0: bytes
    EnqueueRenderToTexture_b8: bytes
    EnqueueRenderToTexture_c0: bytes
    SetShadowCascadeMask: int
    SetMotionBlur: int

class cTkDebugRenderer_vtbl(ctypes.Structure):
    _fields_ = [
        ("RenderText", ctypes.c_ulonglong),
        ("RenderText_8", ctypes.c_ulonglong),
        ("RenderText_10", ctypes.c_ulonglong),
        ("RegisterText3D", ctypes.c_ulonglong),
        ("RegisterText3D_20", ctypes.c_ulonglong),
        ("RenderSafeArea", ctypes.c_ulonglong),
    ]
    RenderText: int
    RenderText_8: int
    RenderText_10: int
    RegisterText3D: int
    RegisterText3D_20: int
    RenderSafeArea: int

class cTkDebugEditor(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mSharedSelectedNode", ctypes.c_uint32),
    ]
    __vftable: int
    mSharedSelectedNode: int

class cTkDebugEditor_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsActive", ctypes.c_ulonglong),
    ]
    IsActive: int

class cGcDebugEditor_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsActive", ctypes.c_ulonglong),
    ]
    IsActive: int

class cGcNGuiWindow_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiFileBrowser_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiModelViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiNodeInfo(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def tilde_cGcNGuiNodeInfo(): pass

    def Get(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mNode", ctypes.c_uint32),
        ("mString", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mTypeName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
        ("maChildren", ctypes.c_char * 0x18),  # std::vector<cGcNGuiNodeInfo *,TkSTLAllocatorShim<cGcNGuiNodeInfo *,8,-1> >
    ]
    __vftable: int
    mNode: int
    mString: bytes
    mTypeName: bytes
    maChildren: bytes

class cGcNGuiNodeInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNGuiNodeInfo_dtor_0", ctypes.c_ulonglong),
    ]
    cGcNGuiNodeInfo_dtor_0: int

class cGcNGuiModelInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcModelViewerCustomisationWindow_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiProceduralModelViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiProceduralTextureViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiGameView_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiSceneView_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiEffectViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cTkRandom(ctypes.Structure):
    def GetInstance(): pass

    _fields_ = [
        ("miHigh", ctypes.c_int32),
        ("miLow", ctypes.c_int32),
        ("miCounter", ctypes.c_int64),
    ]
    miHigh: int
    miLow: int
    miCounter: int

class cGcNGuiNodeGraphViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiCreatureGenerationViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcWFCModuleSetNGuiWindow_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcWFCModuleNGuiWindow_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcSettlementStateNGuiWindow_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcBaseBuildingObjectNGuiWindow_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiGreenscreen_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiProceduralModelAllVariantsViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cTkAudioManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateAuxSendWithFalloffConcrete", ctypes.c_ulonglong),
    ]
    Update: int
    Render: int
    Release: int
    Destruct: int
    UpdateAuxSendWithFalloffConcrete: int

class cTkAudioParamCache(ctypes.Structure):
    def tilde_cTkAudioParamCache(): pass

    def CacheRunTimeValue(): pass

    def CacheSwitch(): pass

    def Reset(): pass

    def SetPosition(): pass

    _fields_ = [
        ("mRTPCPrevValues", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,unsigned int,float,robin_hood::hash<unsigned int,void>,std::equal_to<unsigned int>,-1>
        ("mSwitchPrevValues", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,unsigned int,TkAudioID,robin_hood::hash<unsigned int,void>,std::equal_to<unsigned int>,-1>
        ("mvPrevPosition", common.Vector3f),
        ("mvPrevOrientation", common.Vector3f),
        ("mfPrevObstruction", ctypes.c_float),
        ("mfPrevOcclusion", ctypes.c_float),
    ]
    mRTPCPrevValues: bytes
    mSwitchPrevValues: bytes
    mvPrevPosition: common.Vector3f
    mvPrevOrientation: common.Vector3f
    mfPrevObstruction: float
    mfPrevOcclusion: float

class AkAuxSendValue(ctypes.Structure):
    _fields_ = [
        ("listenerID", ctypes.c_uint64),
        ("auxBusID", ctypes.c_uint32),
        ("fControlValue", ctypes.c_float),
    ]
    listenerID: int
    auxBusID: int
    fControlValue: float

class cGcAudioManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateAuxSendWithFalloffConcrete", ctypes.c_ulonglong),
    ]
    Update: int
    Render: int
    Release: int
    Destruct: int
    UpdateAuxSendWithFalloffConcrete: int

class AtlasTaskCommon(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpBrokerData", ctypes.c_ulonglong),
        ("mHandle", ctypes.c_uint64),
        ("mMetadata", ctypes.c_char * 0x10),  # cTkMetaMessageWrapperTemplated<cGcAtlasShared>
        ("mDebugTag", ctypes.c_ulonglong),
    ]
    __vftable: int
    mpBrokerData: int
    mHandle: int
    mMetadata: bytes
    mDebugTag: int

class AtlasTaskCommon_vtbl(ctypes.Structure):
    _fields_ = [
        ("AtlasTaskCommon_dtor_0", ctypes.c_ulonglong),
    ]
    AtlasTaskCommon_dtor_0: int

class cGcIBrokerData(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcIBrokerData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcIBrokerData_dtor_0", ctypes.c_ulonglong),
    ]
    cGcIBrokerData_dtor_0: int

class cGcAtlasBroker_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasBroker_dtor_0", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("RequiresTransport", ctypes.c_ulonglong),
        ("GetReadyStateCounter", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdateThreadStart", ctypes.c_ulonglong),
        ("UpdateThreadEnd", ctypes.c_ulonglong),
        ("Service", ctypes.c_ulonglong),
        ("Abandon", ctypes.c_ulonglong),
        ("DeleteTaskData", ctypes.c_ulonglong),
        ("GetMessagesEnabled", ctypes.c_ulonglong),
        ("GetOpenIdToken", ctypes.c_ulonglong),
        ("GetAtlasCommunityData", ctypes.c_ulonglong),
        ("GetAtlasBaseImagesRoute", ctypes.c_ulonglong),
        ("GetAtlasFeaturedBaseImagesRoute", ctypes.c_ulonglong),
        ("GetAtlasSeasonDataRoute", ctypes.c_ulonglong),
        ("DoCompletion", ctypes.c_ulonglong),
        ("DoCompletion_a0", ctypes.c_ulonglong),
        ("DoCompletion_a8", ctypes.c_ulonglong),
        ("DoCompletion_b0", ctypes.c_ulonglong),
        ("DoCompletion_b8", ctypes.c_ulonglong),
        ("DoCompletion_c0", ctypes.c_ulonglong),
        ("DoCompletion_c8", ctypes.c_ulonglong),
        ("DoCompletion_d0", ctypes.c_ulonglong),
        ("DoCompletion_d8", ctypes.c_ulonglong),
        ("DoCompletion_e0", ctypes.c_ulonglong),
        ("DoCompletion_e8", ctypes.c_ulonglong),
        ("DoCompletion_f0", ctypes.c_ulonglong),
        ("DoCompletion_f8", ctypes.c_ulonglong),
        ("DoCompletion_100", ctypes.c_ulonglong),
        ("DoCompletion_108", ctypes.c_ulonglong),
        ("DoCompletion_110", ctypes.c_ulonglong),
        ("DoCompletion_118", ctypes.c_ulonglong),
        ("DoCompletion_120", ctypes.c_ulonglong),
        ("DoCompletion_128", ctypes.c_ulonglong),
        ("DoCompletion_130", ctypes.c_ulonglong),
        ("DoCompletion_138", ctypes.c_ulonglong),
        ("DoCompletion_140", ctypes.c_ulonglong),
        ("DoCompletion_148", ctypes.c_ulonglong),
        ("DoCompletion_150", ctypes.c_ulonglong),
        ("DoCompletion_158", ctypes.c_ulonglong),
        ("DoCompletion_160", ctypes.c_ulonglong),
        ("DoCompletion_168", ctypes.c_ulonglong),
        ("DoCompletion_170", ctypes.c_ulonglong),
    ]
    cGcAtlasBroker_dtor_0: int
    Reset: int
    GetName: int
    RequiresTransport: int
    GetReadyStateCounter: int
    Construct: int
    Destruct: int
    Update: int
    UpdateThreadStart: int
    UpdateThreadEnd: int
    Service: int
    Abandon: int
    DeleteTaskData: int
    GetMessagesEnabled: int
    GetOpenIdToken: int
    GetAtlasCommunityData: int
    GetAtlasBaseImagesRoute: int
    GetAtlasFeaturedBaseImagesRoute: int
    GetAtlasSeasonDataRoute: int
    DoCompletion: int
    DoCompletion_a0: int
    DoCompletion_a8: int
    DoCompletion_b0: int
    DoCompletion_b8: int
    DoCompletion_c0: int
    DoCompletion_c8: int
    DoCompletion_d0: int
    DoCompletion_d8: int
    DoCompletion_e0: int
    DoCompletion_e8: int
    DoCompletion_f0: int
    DoCompletion_f8: int
    DoCompletion_100: int
    DoCompletion_108: int
    DoCompletion_110: int
    DoCompletion_118: int
    DoCompletion_120: int
    DoCompletion_128: int
    DoCompletion_130: int
    DoCompletion_138: int
    DoCompletion_140: int
    DoCompletion_148: int
    DoCompletion_150: int
    DoCompletion_158: int
    DoCompletion_160: int
    DoCompletion_168: int
    DoCompletion_170: int

class AtlasRequestTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("AtlasTaskCommon_dtor_0", ctypes.c_ulonglong),
    ]
    AtlasTaskCommon_dtor_0: int

class AtlasResponseTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("AtlasTaskCommon_dtor_0", ctypes.c_ulonglong),
    ]
    AtlasTaskCommon_dtor_0: int

class cGcConnection(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcConnection_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Listen", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("SendMessageA", ctypes.c_char * 0x8),  # int (__fastcall *)(cGcConnection *this, const std::shared_ptr<cGcNetworkMessage> *, std::array<std::shared_ptr<cGcNetworkAddress>,32>, cGcNetworkConstants::TransmissionChannels, cGcNetworkConstants::LobbyType)
        ("ConnectToPlayer", ctypes.c_ulonglong),
        ("DisconnectPlayer", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Release: int
    Destruct: int
    Listen: int
    Update: int
    SendMessageA: bytes
    ConnectToPlayer: int
    DisconnectPlayer: int

class cGcNetworkMessageManager(ctypes.Structure):
    def CreateMessage(): pass


class cGcNetworkPlayerTeleportEffect(ctypes.Structure):
    def cGcNetworkPlayerTeleportEffect(): pass

    def Update(): pass

    def UpdateEffectNode(): pass

    _fields_ = [
        ("mpPlayer", ctypes.c_ulonglong),
        ("mLastKnownPosition", common.Vector3f),
        ("mTargetPoint", common.Vector3f),
        ("mSmoothedTarget", ctypes.c_char * 0x20),  # cTkSmoothCD<cTkVector3>
        ("mfOpacity", ctypes.c_char * 0x8),  # cTkSmoothCD<float>
        ("mfMovementTime", ctypes.c_float),
        ("mTargetEffectNode", ctypes.c_uint32),
        ("mSourceEffectNode", ctypes.c_uint32),
        ("mSourceEffectDirectionNode", ctypes.c_uint32),
    ]
    mpPlayer: int
    mLastKnownPosition: common.Vector3f
    mTargetPoint: common.Vector3f
    mSmoothedTarget: bytes
    mfOpacity: bytes
    mfMovementTime: float
    mTargetEffectNode: int
    mSourceEffectNode: int
    mSourceEffectDirectionNode: int

class cTkNetworkId(ctypes.Structure):
    def Construct(): pass

    _fields_ = [
        ("mOpaqueId", ctypes.c_char * 0x4),  # cTkNetworkId::<unnamed_type_mOpaqueId>
    ]
    mOpaqueId: bytes

class cGcNetworkPlayerFullSim_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsInSpaceship", ctypes.c_ulonglong),
        ("IsInLandVehicle", ctypes.c_ulonglong),
        ("GetEnvironmentLocation", ctypes.c_ulonglong),
        ("IsLocalPlayer", ctypes.c_ulonglong),
        ("GetNetworkIndex", ctypes.c_ulonglong),
        ("IsVRPlayer", ctypes.c_ulonglong),
        ("GetDominantHand", ctypes.c_ulonglong),
        ("IsFullSimLobbyHost", ctypes.c_ulonglong),
        ("VirtualGetMatrix", ctypes.c_ulonglong),
        ("VirtualGetAttachment", ctypes.c_ulonglong),
        ("GetNode", ctypes.c_ulonglong),
        ("NotifyBeingScanned", ctypes.c_ulonglong),
        ("IsStealthActive", ctypes.c_ulonglong),
        ("cGcNetworkPlayer_dtor_68", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNetworkPlayer *this, const std::shared_ptr<cGcNetworkAddress> *, cTkUserIdBase<cTkFixedString<64,char> >, const char *, cTkUserIdBase<cTkFixedString<64,char> >, const cTkFixedString<3,char> *, const char *)
        ("Release", ctypes.c_ulonglong),
        ("HandleNetSerialize", ctypes.c_ulonglong),
        ("IsFullSimPlayer", ctypes.c_ulonglong),
        ("OnConnected", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("GetPlayerNameWithTitle", ctypes.c_ulonglong),
        ("HandleMessage", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkPlayer *this, const std::shared_ptr<cGcNetworkMessage const > *)
        ("GetPosition", ctypes.c_ulonglong),
        ("GetMapPosition", ctypes.c_ulonglong),
        ("NotifyJoin", ctypes.c_ulonglong),
        ("NotifyLeave", ctypes.c_ulonglong),
        ("RenderImGui", ctypes.c_ulonglong),
        ("OnReadyToSendUpdate", ctypes.c_ulonglong),
        ("OnPlayerNameChanged", ctypes.c_ulonglong),
        ("IsHandTracking", ctypes.c_ulonglong),
    ]
    IsInSpaceship: int
    IsInLandVehicle: int
    GetEnvironmentLocation: int
    IsLocalPlayer: int
    GetNetworkIndex: int
    IsVRPlayer: int
    GetDominantHand: int
    IsFullSimLobbyHost: int
    VirtualGetMatrix: int
    VirtualGetAttachment: int
    GetNode: int
    NotifyBeingScanned: int
    IsStealthActive: int
    cGcNetworkPlayer_dtor_68: int
    Construct: int
    Prepare: bytes
    Release: int
    HandleNetSerialize: int
    IsFullSimPlayer: int
    OnConnected: int
    Update: int
    IsActive: int
    OnNetworkPlayerStateEvent: int
    GetPlayerNameWithTitle: int
    HandleMessage: bytes
    GetPosition: int
    GetMapPosition: int
    NotifyJoin: int
    NotifyLeave: int
    RenderImGui: int
    OnReadyToSendUpdate: int
    OnPlayerNameChanged: int
    IsHandTracking: int

class cGcNetworkPlayerFireteam_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsInSpaceship", ctypes.c_ulonglong),
        ("IsInLandVehicle", ctypes.c_ulonglong),
        ("GetEnvironmentLocation", ctypes.c_ulonglong),
        ("IsLocalPlayer", ctypes.c_ulonglong),
        ("GetNetworkIndex", ctypes.c_ulonglong),
        ("IsVRPlayer", ctypes.c_ulonglong),
        ("GetDominantHand", ctypes.c_ulonglong),
        ("IsFullSimLobbyHost", ctypes.c_ulonglong),
        ("VirtualGetMatrix", ctypes.c_ulonglong),
        ("VirtualGetAttachment", ctypes.c_ulonglong),
        ("GetNode", ctypes.c_ulonglong),
        ("NotifyBeingScanned", ctypes.c_ulonglong),
        ("IsStealthActive", ctypes.c_ulonglong),
        ("cGcNetworkPlayer_dtor_68", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNetworkPlayer *this, const std::shared_ptr<cGcNetworkAddress> *, cTkUserIdBase<cTkFixedString<64,char> >, const char *, cTkUserIdBase<cTkFixedString<64,char> >, const cTkFixedString<3,char> *, const char *)
        ("Release", ctypes.c_ulonglong),
        ("HandleNetSerialize", ctypes.c_ulonglong),
        ("IsFullSimPlayer", ctypes.c_ulonglong),
        ("OnConnected", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("GetPlayerNameWithTitle", ctypes.c_ulonglong),
        ("HandleMessage", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNetworkPlayer *this, const std::shared_ptr<cGcNetworkMessage const > *)
        ("GetPosition", ctypes.c_ulonglong),
        ("GetMapPosition", ctypes.c_ulonglong),
        ("NotifyJoin", ctypes.c_ulonglong),
        ("NotifyLeave", ctypes.c_ulonglong),
        ("RenderImGui", ctypes.c_ulonglong),
        ("OnReadyToSendUpdate", ctypes.c_ulonglong),
        ("OnPlayerNameChanged", ctypes.c_ulonglong),
    ]
    IsInSpaceship: int
    IsInLandVehicle: int
    GetEnvironmentLocation: int
    IsLocalPlayer: int
    GetNetworkIndex: int
    IsVRPlayer: int
    GetDominantHand: int
    IsFullSimLobbyHost: int
    VirtualGetMatrix: int
    VirtualGetAttachment: int
    GetNode: int
    NotifyBeingScanned: int
    IsStealthActive: int
    cGcNetworkPlayer_dtor_68: int
    Construct: int
    Prepare: bytes
    Release: int
    HandleNetSerialize: int
    IsFullSimPlayer: int
    OnConnected: int
    Update: int
    IsActive: int
    OnNetworkPlayerStateEvent: int
    GetPlayerNameWithTitle: int
    HandleMessage: bytes
    GetPosition: int
    GetMapPosition: int
    NotifyJoin: int
    NotifyLeave: int
    RenderImGui: int
    OnReadyToSendUpdate: int
    OnPlayerNameChanged: int

class cGcMatchmaking_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("cGcMatchmaking_dtor_18", ctypes.c_ulonglong),
        ("Suspend", ctypes.c_ulonglong),
        ("Resume", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("EnableVoice", ctypes.c_ulonglong),
        ("IsReadyToPrepare", ctypes.c_ulonglong),
        ("StartAmbientMatchmaking", ctypes.c_ulonglong),
        ("UpdateFriends", ctypes.c_ulonglong),
        ("UpdateFriendSessions", ctypes.c_ulonglong),
        ("GetFriendName", ctypes.c_ulonglong),
        ("GetPlatformType", ctypes.c_ulonglong),
        ("GetExtraFriendsString", ctypes.c_char * 0x8),  # cTkFixedString<256,char> *(__fastcall *)(cGcMatchmaking *this, cTkFixedString<256,char> *result, int)
        ("GetGameModeName", ctypes.c_ulonglong),
        ("GetGameModeEnum", ctypes.c_ulonglong),
        ("GetFriendLobbyCount", ctypes.c_ulonglong),
        ("GetFriendMaxLobbyCount", ctypes.c_ulonglong),
        ("JoinFriend", ctypes.c_ulonglong),
        ("AddFriend", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *, bool, bool)
        ("RemoveFriend", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *, bool)
        ("IsCrossPlatformJoinableFriend", ctypes.c_ulonglong),
        ("GetCrossplayFriendPlatformType", ctypes.c_ulonglong),
        ("GetCrossplayFriendName", ctypes.c_ulonglong),
        ("GetCrossplayFriendId", ctypes.c_char * 0x8),  # cTkUserIdBase<cTkFixedString<64,char> > *(__fastcall *)(cGcMatchmaking *this, cTkUserIdBase<cTkFixedString<64,char> > *result, int)
        ("IsCrossplayFriendOnline", ctypes.c_ulonglong),
        ("GetCrossplayFriendPlatformOnlineId", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcMatchmaking *this, int, cTkUserIdBase<unsigned __int64> *)
        ("GetNativeFriendName", ctypes.c_ulonglong),
        ("GetNativeFriendId", ctypes.c_char * 0x8),  # cTkUserIdBase<unsigned __int64> *(__fastcall *)(cGcMatchmaking *this, cTkUserIdBase<unsigned __int64> *result, int)
        ("IsNativeFriendOnline", ctypes.c_ulonglong),
        ("IsNativeFriendInGame", ctypes.c_ulonglong),
        ("BlockPlayer", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *, const char *, cTkFixedString<64,char>, cTkFixedString<3,char>)
        ("UnblockPlayer", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("UnblockPlayer_130", ctypes.c_ulonglong),
        ("GetBlockedPlayerCount", ctypes.c_ulonglong),
        ("GetBlockedPlayerName", ctypes.c_ulonglong),
        ("IsPlayerBlocked", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("IsPlayerBlockedByPlatformId", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcMatchmaking *this, cTkFixedString<64,char>, cTkFixedString<3,char>)
        ("UpdatePlatformMuted", ctypes.c_ulonglong),
        ("ForceRefreshPlatformMuted", ctypes.c_ulonglong),
        ("ShouldReceiveTextChatFromPlayer", ctypes.c_ulonglong),
        ("GetFullSimLobbyUA", ctypes.c_ulonglong),
        ("UpdateFullSimPrivileges", ctypes.c_ulonglong),
        ("JoinFireteam", ctypes.c_ulonglong),
        ("KickPlayer", ctypes.c_ulonglong),
        ("InvitePlayer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<unsigned __int64>)
        ("InvitePlayer_198", ctypes.c_ulonglong),
        ("OpenInviteUI", ctypes.c_ulonglong),
        ("OpenMultiplayerUI", ctypes.c_ulonglong),
        ("MatchmakingServicesRunning", ctypes.c_ulonglong),
        ("InviteToFireteam", ctypes.c_ulonglong),
        ("RequestAddFriend", ctypes.c_ulonglong),
        ("GetLocalUserID", ctypes.c_char * 0x8),  # cTkUserIdBase<cTkFixedString<64,char> > *(__fastcall *)(cGcMatchmaking *this, cTkUserIdBase<cTkFixedString<64,char> > *result)
        ("GetLocalPlatformType", ctypes.c_ulonglong),
        ("UpdateFriendStatus", ctypes.c_ulonglong),
        ("IsPlayerBanned", ctypes.c_ulonglong),
        ("IsInFullSimLobby", ctypes.c_ulonglong),
        ("MakeSessions", ctypes.c_ulonglong),
        ("SendTextChat", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, std::function<void __cdecl(cGcNetworkPlayer *,cTkFixedString<1023,char>,bool,bool,enum TextChatMessageType,cTkColour)>, cGcNetworkPlayer *, cTkFixedString<1023,char>, bool, bool, TextChatMessageType, cTkColour)
        ("SetSpeechToText", ctypes.c_ulonglong),
        ("SetTranslation", ctypes.c_ulonglong),
        ("SetCrossPlatform", ctypes.c_ulonglong),
        ("IsExperiencingWebsocketIssues", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("IsPlatformInvitePending", ctypes.c_ulonglong),
        ("IsPlatformInviteFromAmbientPlayer", ctypes.c_ulonglong),
        ("JoinPlatformInvite", ctypes.c_ulonglong),
        ("CancelPlatformInvite", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int
    cGcMatchmaking_dtor_18: int
    Suspend: int
    Resume: int
    Construct: int
    Update: int
    Prepare: int
    Release: int
    Destruct: int
    EnableVoice: int
    IsReadyToPrepare: int
    StartAmbientMatchmaking: int
    UpdateFriends: int
    UpdateFriendSessions: int
    GetFriendName: int
    GetPlatformType: int
    GetExtraFriendsString: bytes
    GetGameModeName: int
    GetGameModeEnum: int
    GetFriendLobbyCount: int
    GetFriendMaxLobbyCount: int
    JoinFriend: int
    AddFriend: bytes
    RemoveFriend: bytes
    IsCrossPlatformJoinableFriend: int
    GetCrossplayFriendPlatformType: int
    GetCrossplayFriendName: int
    GetCrossplayFriendId: bytes
    IsCrossplayFriendOnline: int
    GetCrossplayFriendPlatformOnlineId: bytes
    GetNativeFriendName: int
    GetNativeFriendId: bytes
    IsNativeFriendOnline: int
    IsNativeFriendInGame: int
    BlockPlayer: bytes
    UnblockPlayer: bytes
    UnblockPlayer_130: int
    GetBlockedPlayerCount: int
    GetBlockedPlayerName: int
    IsPlayerBlocked: bytes
    IsPlayerBlockedByPlatformId: bytes
    UpdatePlatformMuted: int
    ForceRefreshPlatformMuted: int
    ShouldReceiveTextChatFromPlayer: int
    GetFullSimLobbyUA: int
    UpdateFullSimPrivileges: int
    JoinFireteam: int
    KickPlayer: int
    InvitePlayer: bytes
    InvitePlayer_198: int
    OpenInviteUI: int
    OpenMultiplayerUI: int
    MatchmakingServicesRunning: int
    InviteToFireteam: int
    RequestAddFriend: int
    GetLocalUserID: bytes
    GetLocalPlatformType: int
    UpdateFriendStatus: int
    IsPlayerBanned: int
    IsInFullSimLobby: int
    MakeSessions: int
    SendTextChat: bytes
    SetSpeechToText: int
    SetTranslation: int
    SetCrossPlatform: int
    IsExperiencingWebsocketIssues: int
    Connect: int
    IsPlatformInvitePending: int
    IsPlatformInviteFromAmbientPlayer: int
    JoinPlatformInvite: int
    CancelPlatformInvite: int

class cGcVoiceChat_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("OnConstruct", ctypes.c_ulonglong),
        ("OnDestruct", ctypes.c_ulonglong),
        ("OnStartVoice", ctypes.c_ulonglong),
        ("OnStopVoice", ctypes.c_ulonglong),
        ("OnUpdate", ctypes.c_ulonglong),
        ("OnPlayerJoined", ctypes.c_ulonglong),
        ("OnPlayerLeaving", ctypes.c_ulonglong),
        ("OnMute", ctypes.c_ulonglong),
        ("IsMuted", ctypes.c_ulonglong),
        ("CanTalk", ctypes.c_ulonglong),
        ("IsTalking", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int
    OnConstruct: int
    OnDestruct: int
    OnStartVoice: int
    OnStopVoice: int
    OnUpdate: int
    OnPlayerJoined: int
    OnPlayerLeaving: int
    OnMute: int
    IsMuted: int
    CanTalk: int
    IsTalking: int

class DisplayMessage(ctypes.Structure):
    _fields_ = [
        ("mfDisplayStartTime", ctypes.c_float),
        ("mfDisplayDuration", ctypes.c_float),
        ("msMessage", ctypes.c_char * 0x461),  # cTkFixedString<1121,char>
        ("mBackgroundColour", common.Colour),
        ("mMessageId", ctypes.c_int32),
    ]
    mfDisplayStartTime: float
    mfDisplayDuration: float
    msMessage: bytes
    mBackgroundColour: common.Colour
    mMessageId: int

class TextChatMessageBuffer(ctypes.Structure):
    def AddMessage(): pass

    _fields_ = [
        ("miNextMessageInsertIndex", ctypes.c_int32),
        ("miMessages", ctypes.c_int32),
        ("maMessageBuffer", ctypes.c_char * 0x2DA0),  # std::array<DisplayMessage,10>
    ]
    miNextMessageInsertIndex: int
    miMessages: int
    maMessageBuffer: bytes

class cGcNetworkBufferSyncManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("HandleMessage", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNetworkBufferSyncManager *this, std::shared_ptr<cGcNetworkMessage const >, cGcNetworkPlayer *)
        ("SelectBufferToSynchronise", ctypes.c_ulonglong),
        ("SelectBufferFromHashMessage", ctypes.c_ulonglong),
        ("SelectBufferFromSyncMessage", ctypes.c_ulonglong),
        ("OnHashMessageSent", ctypes.c_ulonglong),
        ("OnHashMessageReceived", ctypes.c_ulonglong),
        ("OnSyncMessageReceived", ctypes.c_ulonglong),
        ("OnPlayerLeft", ctypes.c_ulonglong),
        ("OnPlayerJoined", ctypes.c_ulonglong),
        ("OnPlayerLoaded", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int
    Update: int
    HandleMessage: bytes
    SelectBufferToSynchronise: int
    SelectBufferFromHashMessage: int
    SelectBufferFromSyncMessage: int
    OnHashMessageSent: int
    OnHashMessageReceived: int
    OnSyncMessageReceived: int
    OnPlayerLeft: int
    OnPlayerJoined: int
    OnPlayerLoaded: int

class cGcNetworkTerrainEditMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkTerrainSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkTerrainEditsManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("HandleMessage", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNetworkBufferSyncManager *this, std::shared_ptr<cGcNetworkMessage const >, cGcNetworkPlayer *)
        ("SelectBufferToSynchronise", ctypes.c_ulonglong),
        ("SelectBufferFromHashMessage", ctypes.c_ulonglong),
        ("SelectBufferFromSyncMessage", ctypes.c_ulonglong),
        ("OnHashMessageSent", ctypes.c_ulonglong),
        ("OnHashMessageReceived", ctypes.c_ulonglong),
        ("OnSyncMessageReceived", ctypes.c_ulonglong),
        ("OnPlayerLeft", ctypes.c_ulonglong),
        ("OnPlayerJoined", ctypes.c_ulonglong),
        ("OnPlayerLoaded", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int
    Update: int
    HandleMessage: bytes
    SelectBufferToSynchronise: int
    SelectBufferFromHashMessage: int
    SelectBufferFromSyncMessage: int
    OnHashMessageSent: int
    OnHashMessageReceived: int
    OnSyncMessageReceived: int
    OnPlayerLeft: int
    OnPlayerJoined: int
    OnPlayerLoaded: int

class cGcNetworkBaseBuildingManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("HandleMessage", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNetworkBufferSyncManager *this, std::shared_ptr<cGcNetworkMessage const >, cGcNetworkPlayer *)
        ("SelectBufferToSynchronise", ctypes.c_ulonglong),
        ("SelectBufferFromHashMessage", ctypes.c_ulonglong),
        ("SelectBufferFromSyncMessage", ctypes.c_ulonglong),
        ("OnHashMessageSent", ctypes.c_ulonglong),
        ("OnHashMessageReceived", ctypes.c_ulonglong),
        ("OnSyncMessageReceived", ctypes.c_ulonglong),
        ("OnPlayerLeft", ctypes.c_ulonglong),
        ("OnPlayerJoined", ctypes.c_ulonglong),
        ("OnPlayerLoaded", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int
    Update: int
    HandleMessage: bytes
    SelectBufferToSynchronise: int
    SelectBufferFromHashMessage: int
    SelectBufferFromSyncMessage: int
    OnHashMessageSent: int
    OnHashMessageReceived: int
    OnSyncMessageReceived: int
    OnPlayerLeft: int
    OnPlayerJoined: int
    OnPlayerLoaded: int

class sGcNetworkReplicatedEntitiesWelcomeMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class sGcNetworkReplicatedEntitiesReplicatedVariablesMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkReplicatedEntityManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcNetworkSynchronizedClock_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcNetworkRpcManager(ctypes.Structure):
    def RegisterRpc(): pass

    def UnregisterRpc(): pass

    _fields_ = [
        ("mRpcCalls", ctypes.c_char * 0x10),  # std::map<unsigned int,cGcRpcCallBase *>
    ]
    mRpcCalls: bytes

class cGcNetworkInteractionBufferManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("HandleMessage", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNetworkBufferSyncManager *this, std::shared_ptr<cGcNetworkMessage const >, cGcNetworkPlayer *)
        ("SelectBufferToSynchronise", ctypes.c_ulonglong),
        ("SelectBufferFromHashMessage", ctypes.c_ulonglong),
        ("SelectBufferFromSyncMessage", ctypes.c_ulonglong),
        ("OnHashMessageSent", ctypes.c_ulonglong),
        ("OnHashMessageReceived", ctypes.c_ulonglong),
        ("OnSyncMessageReceived", ctypes.c_ulonglong),
        ("OnPlayerLeft", ctypes.c_ulonglong),
        ("OnPlayerJoined", ctypes.c_ulonglong),
        ("OnPlayerLoaded", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int
    Update: int
    HandleMessage: bytes
    SelectBufferToSynchronise: int
    SelectBufferFromHashMessage: int
    SelectBufferFromSyncMessage: int
    OnHashMessageSent: int
    OnHashMessageReceived: int
    OnSyncMessageReceived: int
    OnPlayerLeft: int
    OnPlayerJoined: int
    OnPlayerLoaded: int

class cGcNetworkIdManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcNetworkCreatureManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcNetworkPrivilegesManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int

class cGcResourceManager(ctypes.Structure):
    def SanitiseSceneFilename(): pass

    def AddResourceForceReload(): pass

    _fields_ = [
        ("mFilesToReload", ctypes.c_char * 0x50),  # robin_hood::detail::Table<0,80,cTkFixedString<256,char>,void,UnorderedMapHashes::Hash<cTkFixedString<256,char> >,std::equal_to<cTkFixedString<256,char> >,-1>
        ("mShadersToReload", ctypes.c_char * 0x50),  # robin_hood::detail::Table<0,80,cTkFixedString<256,char>,double,UnorderedMapHashes::Hash<cTkFixedString<256,char> >,std::equal_to<cTkFixedString<256,char> >,-1>
        ("mShaderSourceToReload", ctypes.c_char * 0x50),  # robin_hood::detail::Table<0,80,cTkFixedString<256,char>,double,UnorderedMapHashes::Hash<cTkFixedString<256,char> >,std::equal_to<cTkFixedString<256,char> >,-1>
        ("mReloadMutex", ctypes.c_void_p),
        ("mShaderMutex", ctypes.c_void_p),
        ("mReloadAllSemaphore", ctypes.c_void_p),
        ("mbReloadAll", ctypes.c_char * 0x1),  # std::atomic<bool>
        ("mbStopWatching", ctypes.c_ubyte),
        ("maSceneReloadCallbacks", ctypes.c_char * 0x18),  # std::vector<std::function<void __cdecl(TkHandle,TkHandle)>,TkSTLAllocatorShim<std::function<void __cdecl(TkHandle,TkHandle)>,8,-1> >
        ("mNodeToSceneMap", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkHandle,cTkResource *,robin_hood::hash<TkHandle,void>,std::equal_to<TkHandle>,-1>
        ("mpShaderCompilationState", ctypes.c_ulonglong),
    ]
    mFilesToReload: bytes
    mShadersToReload: bytes
    mShaderSourceToReload: bytes
    mReloadMutex: int
    mShaderMutex: int
    mReloadAllSemaphore: int
    mbReloadAll: bytes
    mbStopWatching: bool
    maSceneReloadCallbacks: bytes
    mNodeToSceneMap: bytes
    mpShaderCompilationState: int

class cTkSegment3(ctypes.Structure):
    _fields_ = [
        ("mPointA", common.Vector3f),
        ("mPointB", common.Vector3f),
    ]
    mPointA: common.Vector3f
    mPointB: common.Vector3f

class RelativeVoxelIndex(ctypes.Structure):
    _fields_ = [
        ("miX", ctypes.c_int32),
        ("miY", ctypes.c_int32),
        ("miZ", ctypes.c_int32),
    ]
    miX: int
    miY: int
    miZ: int

class AnomorphicConfig(ctypes.Structure):
    _fields_ = [
        ("mPlacementOffset", common.Vector2f),
        ("mfVertCompression", ctypes.c_float),
        ("mfHorizScale", ctypes.c_float),
        ("mfContrast", ctypes.c_float),
        ("mfAlpha", ctypes.c_float),
        ("mColourOuter", common.Colour),
        ("mColourInner", common.Colour),
    ]
    mPlacementOffset: common.Vector2f
    mfVertCompression: float
    mfHorizScale: float
    mfContrast: float
    mfAlpha: float
    mColourOuter: common.Colour
    mColourInner: common.Colour

class IntroCameraLook(ctypes.Structure):
    def SmoothTo(): pass

    _fields_ = [
        ("mfCurrentX", ctypes.c_float),
        ("mfCurrentY", ctypes.c_float),
        ("mfRateX", ctypes.c_float),
        ("mfRateY", ctypes.c_float),
    ]
    mfCurrentX: float
    mfCurrentY: float
    mfRateX: float
    mfRateY: float

class SolarAudioTracking(ctypes.Structure):
    _fields_ = [
        ("mAudioObject", ctypes.c_uint64),
        ("mLocalArrayIndex", ctypes.c_int32),
        ("mSolarID", ctypes.c_uint16),
        ("mbTrackingActive", ctypes.c_ubyte),
    ]
    mAudioObject: int
    mLocalArrayIndex: int
    mSolarID: int
    mbTrackingActive: bool

class PushToSelectState(ctypes.Structure):
    _fields_ = [
        ("mu64LastChosenUA", ctypes.c_uint64),
        ("mLastChosenPosition", common.Vector3f),
        ("mfTimer", ctypes.c_float),
        ("mScreenSpacePosition", common.Vector2f),
        ("miLastChosenIndex", ctypes.c_int32),
    ]
    mu64LastChosenUA: int
    mLastChosenPosition: common.Vector3f
    mfTimer: float
    mScreenSpacePosition: common.Vector2f
    miLastChosenIndex: int

class PCSwipeToSelectState(ctypes.Structure):
    _fields_ = [
        ("mfMousePushDirection", ctypes.c_float),
        ("mfActivityTimeout", ctypes.c_float),
        ("mbPushActive", ctypes.c_ubyte),
    ]
    mfMousePushDirection: float
    mfActivityTimeout: float
    mbPushActive: bool

class UInt64Pair(ctypes.Structure):
    _fields_ = [
        ("mu64A", ctypes.c_uint64),
        ("mu64B", ctypes.c_uint64),
    ]
    mu64A: int
    mu64B: int

class cGcGalaxyMap_vtbl(ctypes.Structure):
    _fields_ = [
        ("KnowledgeRevisionWaypointsChanged", ctypes.c_ulonglong),
    ]
    KnowledgeRevisionWaypointsChanged: int

class cGcActivitiesManager(ctypes.Structure):
    _fields_ = [
        ("mbHasPendingActivityLaunch", ctypes.c_ubyte),
    ]
    mbHasPendingActivityLaunch: bool

class cTkStopwatch(ctypes.Structure):
    def GetDurationInSeconds(): pass

    _fields_ = [
        ("miDuration", ctypes.c_uint64),
        ("miStartTime", ctypes.c_uint64),
        ("mbRunning", ctypes.c_ubyte),
    ]
    miDuration: int
    miStartTime: int
    mbRunning: bool

class cGcApplicationState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class cGcApplicationBootState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class cGcApplicationBaseLoadingState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
        ("LoadingRender2D", ctypes.c_ulonglong),
        ("LoadingRenderNVG", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int
    LoadingRender2D: int
    LoadingRenderNVG: int

class cGcApplicationCoreServicesState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
        ("LoadingRender2D", ctypes.c_ulonglong),
        ("LoadingRenderNVG", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int
    LoadingRender2D: int
    LoadingRenderNVG: int

class cGcApplicationGlobalLoadState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
        ("LoadingRender2D", ctypes.c_ulonglong),
        ("LoadingRenderNVG", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int
    LoadingRender2D: int
    LoadingRenderNVG: int

class cGcApplicationLocalLoadState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
        ("LoadingRender2D", ctypes.c_ulonglong),
        ("LoadingRenderNVG", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int
    LoadingRender2D: int
    LoadingRenderNVG: int

class cGcApplicationSimulationState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
        ("ThreadedUpdate", ctypes.c_ulonglong),
        ("ThreadSyncPoint", ctypes.c_ulonglong),
        ("BuildRenderQueue", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int
    ThreadedUpdate: int
    ThreadSyncPoint: int
    BuildRenderQueue: int

class cGcApplicationShutdownState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
        ("Render_48", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int
    Render_48: int

class cGcApplicationGalacticMapState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class cGcApplicationSmokeTestState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
        ("ThreadedUpdate", ctypes.c_ulonglong),
        ("ThreadSyncPoint", ctypes.c_ulonglong),
        ("BuildRenderQueue", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int
    ThreadedUpdate: int
    ThreadSyncPoint: int
    BuildRenderQueue: int

class cGcApplicationAmbientGameState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
        ("ThreadedUpdate", ctypes.c_ulonglong),
        ("ThreadSyncPoint", ctypes.c_ulonglong),
        ("BuildRenderQueue", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int
    ThreadedUpdate: int
    ThreadSyncPoint: int
    BuildRenderQueue: int

class cGcApplicationDeathState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class HDRConfig(ctypes.Structure):
    _fields_ = [
        ("mfToneMapExposure", ctypes.c_float),
        ("mfHDRExposure", ctypes.c_float),
        ("mfHDRThreshold", ctypes.c_float),
        ("mfHDROffset", ctypes.c_float),
    ]
    mfToneMapExposure: float
    mfHDRExposure: float
    mfHDRThreshold: float
    mfHDROffset: float

class cGcApplicationScratchpadState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class cGcApplicationGameModeSelectorState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class cGcApplicationTitleScreenState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class cGcApplicationUGCBaseViewerState_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSMState_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Event", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("SupportsEvenOddRendering", ctypes.c_ulonglong),
    ]
    cTkFSMState_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Event: int
    Release: int
    Destruct: int
    Render: int
    SupportsEvenOddRendering: int

class cGcApplication_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkFSM_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkFSM *this, const cTkFSMStateOffset *, const TkID<128> *)
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("StatePrepare", ctypes.c_ulonglong),
        ("StateRelease", ctypes.c_ulonglong),
    ]
    cTkFSM_dtor_0: int
    Construct: bytes
    Update: int
    Destruct: int
    StatePrepare: int
    StateRelease: int

class ComponentInstancercGcFoliageComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class VkDrawIndexedIndirectCommand(ctypes.Structure):
    _fields_ = [
        ("indexCount", ctypes.c_uint32),
        ("instanceCount", ctypes.c_uint32),
        ("firstIndex", ctypes.c_uint32),
        ("vertexOffset", ctypes.c_int32),
        ("firstInstance", ctypes.c_uint32),
    ]
    indexCount: int
    instanceCount: int
    firstIndex: int
    vertexOffset: int
    firstInstance: int

class NVSDK_NGX_Parameter(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class NVSDK_NGX_Parameter_vtbl(ctypes.Structure):
    _fields_ = [
        ("Set", ctypes.c_ulonglong),
        ("Set_8", ctypes.c_ulonglong),
        ("Set_10", ctypes.c_ulonglong),
        ("Set_18", ctypes.c_ulonglong),
        ("Set_20", ctypes.c_ulonglong),
        ("Set_28", ctypes.c_ulonglong),
        ("Set_30", ctypes.c_ulonglong),
        ("Set_38", ctypes.c_ulonglong),
        ("Get", ctypes.c_ulonglong),
        ("Get_48", ctypes.c_ulonglong),
        ("Get_50", ctypes.c_ulonglong),
        ("Get_58", ctypes.c_ulonglong),
        ("Get_60", ctypes.c_ulonglong),
        ("Get_68", ctypes.c_ulonglong),
        ("Get_70", ctypes.c_ulonglong),
        ("Get_78", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
    ]
    Set: int
    Set_8: int
    Set_10: int
    Set_18: int
    Set_20: int
    Set_28: int
    Set_30: int
    Set_38: int
    Get: int
    Get_48: int
    Get_50: int
    Get_58: int
    Get_60: int
    Get_68: int
    Get_70: int
    Get_78: int
    Reset: int

class NVSDK_NGX_Handle(ctypes.Structure):
    _fields_ = [
        ("Id", ctypes.c_uint32),
    ]
    Id: int

class ComponentInstancercGcAISpaceshipComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class VkPhysicalDeviceFeatures(ctypes.Structure):
    _fields_ = [
        ("robustBufferAccess", ctypes.c_uint32),
        ("fullDrawIndexUint32", ctypes.c_uint32),
        ("imageCubeArray", ctypes.c_uint32),
        ("independentBlend", ctypes.c_uint32),
        ("geometryShader", ctypes.c_uint32),
        ("tessellationShader", ctypes.c_uint32),
        ("sampleRateShading", ctypes.c_uint32),
        ("dualSrcBlend", ctypes.c_uint32),
        ("logicOp", ctypes.c_uint32),
        ("multiDrawIndirect", ctypes.c_uint32),
        ("drawIndirectFirstInstance", ctypes.c_uint32),
        ("depthClamp", ctypes.c_uint32),
        ("depthBiasClamp", ctypes.c_uint32),
        ("fillModeNonSolid", ctypes.c_uint32),
        ("depthBounds", ctypes.c_uint32),
        ("wideLines", ctypes.c_uint32),
        ("largePoints", ctypes.c_uint32),
        ("alphaToOne", ctypes.c_uint32),
        ("multiViewport", ctypes.c_uint32),
        ("samplerAnisotropy", ctypes.c_uint32),
        ("textureCompressionETC2", ctypes.c_uint32),
        ("textureCompressionASTC_LDR", ctypes.c_uint32),
        ("textureCompressionBC", ctypes.c_uint32),
        ("occlusionQueryPrecise", ctypes.c_uint32),
        ("pipelineStatisticsQuery", ctypes.c_uint32),
        ("vertexPipelineStoresAndAtomics", ctypes.c_uint32),
        ("fragmentStoresAndAtomics", ctypes.c_uint32),
        ("shaderTessellationAndGeometryPointSize", ctypes.c_uint32),
        ("shaderImageGatherExtended", ctypes.c_uint32),
        ("shaderStorageImageExtendedFormats", ctypes.c_uint32),
        ("shaderStorageImageMultisample", ctypes.c_uint32),
        ("shaderStorageImageReadWithoutFormat", ctypes.c_uint32),
        ("shaderStorageImageWriteWithoutFormat", ctypes.c_uint32),
        ("shaderUniformBufferArrayDynamicIndexing", ctypes.c_uint32),
        ("shaderSampledImageArrayDynamicIndexing", ctypes.c_uint32),
        ("shaderStorageBufferArrayDynamicIndexing", ctypes.c_uint32),
        ("shaderStorageImageArrayDynamicIndexing", ctypes.c_uint32),
        ("shaderClipDistance", ctypes.c_uint32),
        ("shaderCullDistance", ctypes.c_uint32),
        ("shaderFloat64", ctypes.c_uint32),
        ("shaderInt64", ctypes.c_uint32),
        ("shaderInt16", ctypes.c_uint32),
        ("shaderResourceResidency", ctypes.c_uint32),
        ("shaderResourceMinLod", ctypes.c_uint32),
        ("sparseBinding", ctypes.c_uint32),
        ("sparseResidencyBuffer", ctypes.c_uint32),
        ("sparseResidencyImage2D", ctypes.c_uint32),
        ("sparseResidencyImage3D", ctypes.c_uint32),
        ("sparseResidency2Samples", ctypes.c_uint32),
        ("sparseResidency4Samples", ctypes.c_uint32),
        ("sparseResidency8Samples", ctypes.c_uint32),
        ("sparseResidency16Samples", ctypes.c_uint32),
        ("sparseResidencyAliased", ctypes.c_uint32),
        ("variableMultisampleRate", ctypes.c_uint32),
        ("inheritedQueries", ctypes.c_uint32),
    ]
    robustBufferAccess: int
    fullDrawIndexUint32: int
    imageCubeArray: int
    independentBlend: int
    geometryShader: int
    tessellationShader: int
    sampleRateShading: int
    dualSrcBlend: int
    logicOp: int
    multiDrawIndirect: int
    drawIndirectFirstInstance: int
    depthClamp: int
    depthBiasClamp: int
    fillModeNonSolid: int
    depthBounds: int
    wideLines: int
    largePoints: int
    alphaToOne: int
    multiViewport: int
    samplerAnisotropy: int
    textureCompressionETC2: int
    textureCompressionASTC_LDR: int
    textureCompressionBC: int
    occlusionQueryPrecise: int
    pipelineStatisticsQuery: int
    vertexPipelineStoresAndAtomics: int
    fragmentStoresAndAtomics: int
    shaderTessellationAndGeometryPointSize: int
    shaderImageGatherExtended: int
    shaderStorageImageExtendedFormats: int
    shaderStorageImageMultisample: int
    shaderStorageImageReadWithoutFormat: int
    shaderStorageImageWriteWithoutFormat: int
    shaderUniformBufferArrayDynamicIndexing: int
    shaderSampledImageArrayDynamicIndexing: int
    shaderStorageBufferArrayDynamicIndexing: int
    shaderStorageImageArrayDynamicIndexing: int
    shaderClipDistance: int
    shaderCullDistance: int
    shaderFloat64: int
    shaderInt64: int
    shaderInt16: int
    shaderResourceResidency: int
    shaderResourceMinLod: int
    sparseBinding: int
    sparseResidencyBuffer: int
    sparseResidencyImage2D: int
    sparseResidencyImage3D: int
    sparseResidency2Samples: int
    sparseResidency4Samples: int
    sparseResidency8Samples: int
    sparseResidency16Samples: int
    sparseResidencyAliased: int
    variableMultisampleRate: int
    inheritedQueries: int

class ComponentLoadInstancercGcAISpaceshipComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class VkXYColorEXT(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
    ]
    x: float
    y: float

class cTkCompute_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("GetJobsInFlight", ctypes.c_ulonglong),
    ]
    Construct: int
    Destruct: int
    Update: int
    GetJobsInFlight: int

class cTkRegionNoise(ctypes.Structure):
    def GetFaceIndex(): pass

    def GetFaceRight(): pass

    def GetFaceUp(): pass

    def VoronoiDistanceOnCube(): pass

    def VoronoiDistanceOnCubeBatch(): pass

    def VoronoiID(): pass


class ComponentInstancercGcDestructableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercTkAudioComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcDestructableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cTk2dConstant(ctypes.Structure):
    pass

class ComponentInstancercGcGrabbableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class hkDisplaySphere_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class hkDisplayCapsule_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class hkDisplayAABB_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class hkDisplayWireframe_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class ComponentInstancercTkGravityComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercTkNetReplicatedEntityComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcNetworkMessageHeader(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x1),  # $274290FB4AAAFA7769493461B2B0A238
    ]
    ___u0: bytes

class hknpInternalMotionUtilFunctions2(ctypes.Structure):
    pass

class hkMxRealf_Implementation(ctypes.Structure):
    pass

class hkMxQuaternionf_Implementation(ctypes.Structure):
    pass

class ComponentInstancercTkLODComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcReliableMessageIndex(ctypes.Structure):
    _fields_ = [
        ("muMessageIndex", ctypes.c_uint16),
    ]
    muMessageIndex: int

class ComponentLoadInstancercTkLODComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class sMaintenanceSaveKey(ctypes.Structure):
    _fields_ = [
        ("mPosition", common.Vector3f),
        ("meLocation", ctypes.c_char * 0x4),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation
    ]
    mPosition: common.Vector3f
    meLocation: bytes

class ComponentInstancercGcCustomInventoryComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercTkProceduralModelComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cTkHmdBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("GetVibrationSupported", ctypes.c_ulonglong),
        ("SetVibration", ctypes.c_ulonglong),
        ("RequestRecenter", ctypes.c_ulonglong),
        ("GetProjectionMatrix", ctypes.c_ulonglong),
        ("GetProjectionRaw", ctypes.c_ulonglong),
        ("GetProjectionRawUnscaled", ctypes.c_ulonglong),
        ("ApplyProjectionScale", ctypes.c_ulonglong),
        ("UpdateViewMatrix", ctypes.c_ulonglong),
        ("GetViewMatrix", ctypes.c_ulonglong),
        ("GetGazeMatrix", ctypes.c_ulonglong),
        ("SetFramebuffer", ctypes.c_ulonglong),
        ("HasJustResetView", ctypes.c_ulonglong),
        ("SetHasJustResetView", ctypes.c_ulonglong),
        ("RecenterPose", ctypes.c_ulonglong),
        ("OnBeginStartupSequence", ctypes.c_ulonglong),
        ("OnEndStartupSequence", ctypes.c_ulonglong),
        ("OnShutdown", ctypes.c_ulonglong),
    ]
    Construct: int
    Destruct: int
    Prepare: int
    Release: int
    Update: int
    GetVibrationSupported: int
    SetVibration: int
    RequestRecenter: int
    GetProjectionMatrix: int
    GetProjectionRaw: int
    GetProjectionRawUnscaled: int
    ApplyProjectionScale: int
    UpdateViewMatrix: int
    GetViewMatrix: int
    GetGazeMatrix: int
    SetFramebuffer: int
    HasJustResetView: int
    SetHasJustResetView: int
    RecenterPose: int
    OnBeginStartupSequence: int
    OnEndStartupSequence: int
    OnShutdown: int

class FireSignalTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class ParticleParticleProcessEarlyOutStateTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class PreparePartitionTreeBuffers_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class GenerateMappingToPointsBufferTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class cTkNetReplicatedEntityComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class UpdateRigidBodyDirtyAabbsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class UpdateParticleParticleInDirtyAabbsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class ComponentInstancercGcEncyclopediaComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ParticleParticleInterBranchSolveTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class FindDynamicNeighborsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class ParticleParticleCollisionsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class ParticleParticleInterBranchCollisionsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class BodyCollisionsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class FireParticlesCollidedWithParticlesCallbackTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class FireParticlesCollidedWithBodiesCallbackTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class FireParticlesExitedSafeRegionCallbackTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkTask_dtor_8", ctypes.c_ulonglong),
        ("process", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getDebugAttributes", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkTask_dtor_8: int
    process: int
    getName: int
    getDebugAttributes: int

class ComponentInstancercGcCustomisationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcCustomisationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcPlayerEffectsComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcSpaceshipComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcScannableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class randomAccessCacheReader(ctypes.Structure):
    _fields_ = [
        ("m_cacheStarStream", ctypes.c_char * 0x18),  # hkTypedBlockStream<hknpBlockStreamBlockOffsetPair>::Reader
    ]
    m_cacheStarStream: bytes

class cTkComponentPool(ctypes.Structure):
    def cTkComponentPool(): pass

    def GetComponent(): pass

    def HasComponent(): pass

    def Free(): pass

    _fields_ = [
        ("mType", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mpAllocateFunction", ctypes.c_ulonglong),
        ("mpConstructFunction", ctypes.c_ulonglong),
        ("mpDestructFunction", ctypes.c_ulonglong),
        ("mpLoadFunction", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkClassPointer *, const char *, cTkResourceDescriptor *, TkStrongType<int,TkStrongTypeIDs::TkResHandleID>)
        ("mpUpdater", ctypes.c_ulonglong),
        ("maComponents", ctypes.c_ulonglong),
        ("mapComponentPtrs", ctypes.c_ulonglong),
        ("muTypeSize", ctypes.c_uint64),
        ("muMetadataTypeSize", ctypes.c_uint64),
        ("maiRoster", ctypes.c_ulonglong),
        ("miPoolSize", ctypes.c_int16),
        ("miRosterPartition", ctypes.c_int16),
        ("miRosterPartitionMax", ctypes.c_int16),
        ("miPriority", ctypes.c_int16),
        ("mapOverflow", ctypes.c_char * 0x18),  # std::vector<cTkComponent *,TkSTLAllocatorShim<cTkComponent *,8,-1> >
    ]
    mType: bytes
    mpAllocateFunction: int
    mpConstructFunction: int
    mpDestructFunction: int
    mpLoadFunction: bytes
    mpUpdater: int
    maComponents: int
    mapComponentPtrs: int
    muTypeSize: int
    muMetadataTypeSize: int
    maiRoster: int
    miPoolSize: int
    miRosterPartition: int
    miRosterPartitionMax: int
    miPriority: int
    mapOverflow: bytes

class hknpFinalizeNarrowPhasePairsTaskEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class cTkHmdOpenVR_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("GetVibrationSupported", ctypes.c_ulonglong),
        ("SetVibration", ctypes.c_ulonglong),
        ("RequestRecenter", ctypes.c_ulonglong),
        ("GetProjectionMatrix", ctypes.c_ulonglong),
        ("GetProjectionRaw", ctypes.c_ulonglong),
        ("GetProjectionRawUnscaled", ctypes.c_ulonglong),
        ("ApplyProjectionScale", ctypes.c_ulonglong),
        ("UpdateViewMatrix", ctypes.c_ulonglong),
        ("GetViewMatrix", ctypes.c_ulonglong),
        ("GetGazeMatrix", ctypes.c_ulonglong),
        ("SetFramebuffer", ctypes.c_ulonglong),
        ("HasJustResetView", ctypes.c_ulonglong),
        ("SetHasJustResetView", ctypes.c_ulonglong),
        ("RecenterPose", ctypes.c_ulonglong),
        ("OnBeginStartupSequence", ctypes.c_ulonglong),
        ("OnEndStartupSequence", ctypes.c_ulonglong),
        ("OnShutdown", ctypes.c_ulonglong),
    ]
    Construct: int
    Destruct: int
    Prepare: int
    Release: int
    Update: int
    GetVibrationSupported: int
    SetVibration: int
    RequestRecenter: int
    GetProjectionMatrix: int
    GetProjectionRaw: int
    GetProjectionRawUnscaled: int
    ApplyProjectionScale: int
    UpdateViewMatrix: int
    GetViewMatrix: int
    GetGazeMatrix: int
    SetFramebuffer: int
    HasJustResetView: int
    SetHasJustResetView: int
    RecenterPose: int
    OnBeginStartupSequence: int
    OnEndStartupSequence: int
    OnShutdown: int

class ManifoldSortKey(ctypes.Structure):
    _fields_ = [
        ("m_key", ctypes.c_uint64),
    ]
    m_key: int

class cTkPhysicsComponentUpdater_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnInit", ctypes.c_ulonglong),
        ("UpdateComponents", ctypes.c_ulonglong),
        ("UpdateComponentsPostPhysics", ctypes.c_ulonglong),
        ("UpdateComponentsRender", ctypes.c_ulonglong),
        ("OnComponentAllocate", ctypes.c_ulonglong),
        ("OnComponentFree", ctypes.c_ulonglong),
        ("HasCustomUpdate", ctypes.c_ulonglong),
    ]
    OnInit: int
    UpdateComponents: int
    UpdateComponentsPostPhysics: int
    UpdateComponentsRender: int
    OnComponentAllocate: int
    OnComponentFree: int
    HasCustomUpdate: int

class cTkPhysicsComponentUpdater(ctypes.Structure):
    def OnInit(): pass

    def UpdateComponents(): pass

    def UpdateComponentsPostPhysics(): pass

    def OnComponentFree(): pass

    def AddToPostPhysQueue(): pass

    def GetInstance(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x18),  # cTkTemplatedComponentUpdater<cTkPhysicsComponent>
        ("mpPostPhysUpdateBitsBegin", ctypes.c_ulonglong),
        ("mpPostPhysUpdateBitsEnd", ctypes.c_ulonglong),
    ]
    baseclass_0: bytes
    mpPostPhysUpdateBitsBegin: int
    mpPostPhysUpdateBitsEnd: int

class cTkBase2DRenderer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Begin", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("SetTexture", ctypes.c_ulonglong),
        ("SetShader", ctypes.c_ulonglong),
    ]
    Construct: int
    Begin: int
    End: int
    SetTexture: int
    SetShader: int

class cTk2dRenderer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Begin", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("SetTexture", ctypes.c_ulonglong),
        ("SetShader", ctypes.c_ulonglong),
    ]
    Construct: int
    Begin: int
    End: int
    SetTexture: int
    SetShader: int

class cTkImmediateModeBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkImmediateModeBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Begin", ctypes.c_ulonglong),
        ("RenderLines", ctypes.c_ulonglong),
        ("RenderQuads", ctypes.c_ulonglong),
        ("RenderSingleScreenQuad", ctypes.c_ulonglong),
        ("RenderTriangles", ctypes.c_ulonglong),
        ("DrawArrays", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("SetTexture", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkImmediateModeBase_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Begin: int
    RenderLines: int
    RenderQuads: int
    RenderSingleScreenQuad: int
    RenderTriangles: int
    DrawArrays: int
    SetColour: int
    SetTexture: int
    End: int
    Release: int
    Destruct: int

class cTkImmediateMode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkImmediateModeBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Begin", ctypes.c_ulonglong),
        ("RenderLines", ctypes.c_ulonglong),
        ("RenderQuads", ctypes.c_ulonglong),
        ("RenderSingleScreenQuad", ctypes.c_ulonglong),
        ("RenderTriangles", ctypes.c_ulonglong),
        ("DrawArrays", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("SetTexture", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkImmediateModeBase_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Begin: int
    RenderLines: int
    RenderQuads: int
    RenderSingleScreenQuad: int
    RenderTriangles: int
    DrawArrays: int
    SetColour: int
    SetTexture: int
    End: int
    Release: int
    Destruct: int

class cTk2dRectangle_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int

class cTk2dColourQuad_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int

class cTk2dImageRotate_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetPosition_40", ctypes.c_ulonglong),
        ("GetWorldTopLeft", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("SetColour", ctypes.c_ulonglong),
        ("GetColour", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("RemoveAllObjects", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int
    SetPosition: int
    GetPosition: int
    GetPosition_40: int
    GetWorldTopLeft: int
    SetSize: int
    GetSize: int
    SetColour: int
    GetColour: int
    SetAlignment: int
    GetAlignment: int
    RemoveAllObjects: int

class cTkInputImages(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkInputImages_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetButtonImage", ctypes.c_ulonglong),
        ("GetImage", ctypes.c_ulonglong),
    ]
    GetButtonImage: int
    GetImage: int

class ComponentInstancercGcNPCInteractiveObjectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cTkVehicleReportDestructionCallback_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class cTkSpaceshipReportCollisionCallback_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class cGcUndoableNGuiElementOperation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkNGuiUndoable_dtor_0", ctypes.c_ulonglong),
        ("Undo", ctypes.c_ulonglong),
        ("Do", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("GetData", ctypes.c_ulonglong),
        ("ToString", ctypes.c_ulonglong),
        ("Icon", ctypes.c_ulonglong),
    ]
    cTkNGuiUndoable_dtor_0: int
    Undo: int
    Do: int
    End: int
    GetData: int
    ToString: int
    Icon: int

class cGcNGuiSpacing_vtbl(ctypes.Structure):
    _fields_ = [
        ("Render", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
        ("cGcNGuiElement_dtor_10", ctypes.c_ulonglong),
        ("EditElement", ctypes.c_ulonglong),
        ("EditPresets", ctypes.c_ulonglong),
        ("RenderUIEditorToolbar", ctypes.c_ulonglong),
        ("SelectableInEditor", ctypes.c_ulonglong),
        ("SetNeedsUpdate", ctypes.c_ulonglong),
        ("Deselect", ctypes.c_ulonglong),
        ("IsSelected", ctypes.c_ulonglong),
        ("Resized", ctypes.c_ulonglong),
        ("AttachMetadata", ctypes.c_ulonglong),
        ("DetachMetadata", ctypes.c_ulonglong),
        ("IsLayer", ctypes.c_ulonglong),
        ("GetSceneTreeIcon", ctypes.c_ulonglong),
        ("GetSceneTreeText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNGuiElement *this, cTkFixedString<128,char> *)
        ("RenderAdditionalOverlayIcons", ctypes.c_ulonglong),
        ("RenderTreeControls", ctypes.c_ulonglong),
    ]
    Render: int
    GetType: int
    cGcNGuiElement_dtor_10: int
    EditElement: int
    EditPresets: int
    RenderUIEditorToolbar: int
    SelectableInEditor: int
    SetNeedsUpdate: int
    Deselect: int
    IsSelected: int
    Resized: int
    AttachMetadata: int
    DetachMetadata: int
    IsLayer: int
    GetSceneTreeIcon: int
    GetSceneTreeText: bytes
    RenderAdditionalOverlayIcons: int
    RenderTreeControls: int

class cTkUnixTimestamp(ctypes.Structure):
    def Now(): pass

    def Format(): pass

    def GetTimeInfo(): pass


class ComponentInstancercGcNPCNavigationAreaComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cTkFontFileBase(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkFontFileBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("IsBinary", ctypes.c_ulonglong),
        ("ReachedEndOfFile", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("Size", ctypes.c_ulonglong),
        ("Read", ctypes.c_ulonglong),
        ("GetFilename", ctypes.c_ulonglong),
    ]
    Construct: int
    IsBinary: int
    ReachedEndOfFile: int
    Close: int
    Size: int
    Read: int
    GetFilename: int

class cTkFontFile_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("IsBinary", ctypes.c_ulonglong),
        ("ReachedEndOfFile", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("Size", ctypes.c_ulonglong),
        ("Read", ctypes.c_ulonglong),
        ("GetFilename", ctypes.c_ulonglong),
    ]
    Construct: int
    IsBinary: int
    ReachedEndOfFile: int
    Close: int
    Size: int
    Read: int
    GetFilename: int

class cTkFontLoader(ctypes.Structure):
    def Construct(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpFile", ctypes.c_ulonglong),
        ("mpFont", ctypes.c_ulonglong),
        ("mpFontFileName", ctypes.c_char_p),
        ("miOutlineThickness", ctypes.c_int32),
    ]
    __vftable: int
    mpFile: int
    mpFont: int
    mpFontFileName: bytes
    miOutlineThickness: int

class cTkFontLoader_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
    ]
    Construct: int
    Load: int

class cTkFontLoaderText_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
    ]
    Construct: int
    Load: int

class cGcSimpleInteractionComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcSitLocation(ctypes.Structure):
    _fields_ = [
        ("mNode", ctypes.c_uint32),
        ("mReservedBy", ctypes.c_uint64),
        ("mbOccupied", ctypes.c_ubyte),
    ]
    mNode: int
    mReservedBy: int
    mbOccupied: bool

class cGcChairComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkStringData(ctypes.Structure):
    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mpString", ctypes.c_ulonglong),
    ]
    mID: int
    mpString: int

class cTkLanguageManagerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkLanguageManagerBase_dtor_0", ctypes.c_ulonglong),
        ("FindRegionFromPlatform", ctypes.c_ulonglong),
    ]
    cTkLanguageManagerBase_dtor_0: int
    FindRegionFromPlatform: int

class cTkNGuiStyles(ctypes.Structure):
    def Blend(): pass

    def RenderText(): pass

    def GetGraphicMargins(): pass

    def CopyChanges(): pass

    def DrawScrollArrow(): pass

    def EditGraphicStyle(): pass

    def EditGraphicStyleData(): pass

    def EditTextStyle(): pass

    def EditTextStyleData(): pass

    def GetTextSize(): pass

    def RenderGraphic(): pass

    def RenderShape(): pass


class cTkLanguageManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkLanguageManagerBase_dtor_0", ctypes.c_ulonglong),
        ("FindRegionFromPlatform", ctypes.c_ulonglong),
    ]
    cTkLanguageManagerBase_dtor_0: int
    FindRegionFromPlatform: int

class cTkBuildCharacterInfo(ctypes.Structure):
    _fields_ = [
        ("mfTop", ctypes.c_float),
        ("mfBottom", ctypes.c_float),
    ]
    mfTop: float
    mfBottom: float

class cGcNGuiGame_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetFontID", ctypes.c_ulonglong),
        ("GetFontSpacing", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("IsHighlightActive", ctypes.c_ulonglong),
        ("IsNGUIEditor", ctypes.c_ulonglong),
    ]
    GetFontID: int
    GetFontSpacing: int
    IsActive: int
    IsHighlightActive: int
    IsNGUIEditor: int

class cGcNavMeshHandlePacked(ctypes.Structure):
    _fields_ = [
        ("muFamilyWithIndex", ctypes.c_uint16),
        ("muId", ctypes.c_uint16),
    ]
    muFamilyWithIndex: int
    muId: int

class cTkBehaviourTreeBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cTkJobManager(ctypes.Structure):
    def RunAllJobsInQueue(): pass

    def WakeAllThreadsInGroup(): pass

    def WakeAllThreadsForQueue(): pass

    def CreateFence(): pass

    def CreateJob(): pass

    def CreateSuspendableJob(): pass

    def DefaultJobSuspendedFunction(): pass

    def ExecuteJob(): pass

    def GetJob(): pass

    def HasJobCompleted(): pass

    def Init(): pass

    def ReAddSuspendedJob(): pass

    def RunOnEveryThread(): pass

    def RunOneJobFromGroup(): pass

    def Shutdown(): pass

    def ShutdownHavokAllocators(): pass

    def SuspendJob(): pass

    def SwitchToQueue(): pass

    def ThreadWorker(): pass

    def WaitForQueue(): pass


class cTkBehaviourTreePrioritySelector_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeSequenceSelector_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeConcurrentSelector_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeStochasticSelector_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeNegateDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeAlwaysSucceedDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeAlwaysRunDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeNeverFailDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeSwitchDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeSelectionDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeSucceedOnSwitchDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cTkBehaviourTreeSucceedOnSelectionDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class sSolarDay(ctypes.Structure):
    def GetSunDirection(): pass

    def GetSolarSunAngle(): pass

    def GetRelativeSunAngle(): pass

    def GetSolarSunAngles(): pass

    def GetTimeOfDay(): pass

    _fields_ = [
        ("mSunAxis", common.Vector3f),
        ("mSolarEquatorStart", common.Vector3f),
    ]
    mSunAxis: common.Vector3f
    mSolarEquatorStart: common.Vector3f

class VkSparseMemoryBind(ctypes.Structure):
    _fields_ = [
        ("resourceOffset", ctypes.c_uint64),
        ("size", ctypes.c_uint64),
        ("memory", ctypes.c_ulonglong),
        ("memoryOffset", ctypes.c_uint64),
        ("flags", ctypes.c_uint32),
    ]
    resourceOffset: int
    size: int
    memory: int
    memoryOffset: int
    flags: int

class VkSparseBufferMemoryBindInfo(ctypes.Structure):
    _fields_ = [
        ("buffer", ctypes.c_ulonglong),
        ("bindCount", ctypes.c_uint32),
        ("pBinds", ctypes.c_ulonglong),
    ]
    buffer: int
    bindCount: int
    pBinds: int

class VkSparseImageOpaqueMemoryBindInfo(ctypes.Structure):
    _fields_ = [
        ("image", ctypes.c_ulonglong),
        ("bindCount", ctypes.c_uint32),
        ("pBinds", ctypes.c_ulonglong),
    ]
    image: int
    bindCount: int
    pBinds: int

class VkImageSubresource(ctypes.Structure):
    _fields_ = [
        ("aspectMask", ctypes.c_uint32),
        ("mipLevel", ctypes.c_uint32),
        ("arrayLayer", ctypes.c_uint32),
    ]
    aspectMask: int
    mipLevel: int
    arrayLayer: int

class VkOffset3D(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("z", ctypes.c_int32),
    ]
    x: int
    y: int
    z: int

class VkSparseImageMemoryBindInfo(ctypes.Structure):
    _fields_ = [
        ("image", ctypes.c_ulonglong),
        ("bindCount", ctypes.c_uint32),
        ("pBinds", ctypes.c_ulonglong),
    ]
    image: int
    bindCount: int
    pBinds: int

class VkImageSubresourceLayers(ctypes.Structure):
    _fields_ = [
        ("aspectMask", ctypes.c_uint32),
        ("mipLevel", ctypes.c_uint32),
        ("baseArrayLayer", ctypes.c_uint32),
        ("layerCount", ctypes.c_uint32),
    ]
    aspectMask: int
    mipLevel: int
    baseArrayLayer: int
    layerCount: int

class VkImageSubresourceRange(ctypes.Structure):
    _fields_ = [
        ("aspectMask", ctypes.c_uint32),
        ("baseMipLevel", ctypes.c_uint32),
        ("levelCount", ctypes.c_uint32),
        ("baseArrayLayer", ctypes.c_uint32),
        ("layerCount", ctypes.c_uint32),
    ]
    aspectMask: int
    baseMipLevel: int
    levelCount: int
    baseArrayLayer: int
    layerCount: int

class cTkLayeredNoise(ctypes.Structure):
    def CheckerboardNoise(): pass

    def GenerateSmoothNoiseLayer3D(): pass

    def GenerateUberNoiseLayer3D_Vector(): pass

    def IqTurbulenceNoise(): pass

    def OctaveNoise(): pass

    def PlateauNoise(): pass

    def Random3f(): pass


class cTkSimplexNoise(ctypes.Structure):
    def Noise2d(): pass

    def Noise3d(): pass

    def Noise3dVectorised(): pass


class cTkPolygoniserDualContouring(ctypes.Structure):
    def CalculateMaterials(): pass

    def DualContouring(): pass

    def GeneratePolygons(): pass

    def Polygonise(): pass


class cGcUberNoise2DWrapper(ctypes.Structure):
    def Construct(): pass

    _fields_ = [
        ("mafNoise2D", ctypes.c_ulonglong),
        ("mafTempBuffer0", ctypes.c_ulonglong),
        ("mafTempBuffer1", ctypes.c_ulonglong),
        ("mavNoiseDebugData", ctypes.c_ulonglong),
    ]
    mafNoise2D: int
    mafTempBuffer0: int
    mafTempBuffer1: int
    mavNoiseDebugData: int

class cGcFeatureNoise2DWrapper(ctypes.Structure):
    def Generate2D(): pass

    _fields_ = [
        ("mfEdgeDistance2d", ctypes.c_float),
        ("mafNoise2D", ctypes.c_ulonglong),
        ("mafHeightVarianceNoise2D", ctypes.c_ulonglong),
        ("mafRegion2D", ctypes.c_ulonglong),
    ]
    mfEdgeDistance2d: float
    mafNoise2D: int
    mafHeightVarianceNoise2D: int
    mafRegion2D: int

class sGenVoxelShared(ctypes.Structure):
    _fields_ = [
        ("mbHasWater", ctypes.c_ubyte),
        ("mfTileBlendMeters", ctypes.c_float),
        ("mfTileBlendMultiplier", ctypes.c_float),
        ("mfPlaneHeight3D", ctypes.c_float),
        ("mfVoxelHeight", ctypes.c_float),
        ("mfBeachHeight", ctypes.c_float),
    ]
    mbHasWater: bool
    mfTileBlendMeters: float
    mfTileBlendMultiplier: float
    mfPlaneHeight3D: float
    mfVoxelHeight: float
    mfBeachHeight: float

class sGenVoxelDebug(ctypes.Structure):
    pass

class sVoxelTileTypeEntry(ctypes.Structure):
    _fields_ = [
        ("muPackedBlend", ctypes.c_uint8),
        ("muPriority", ctypes.c_uint8),
    ]
    muPackedBlend: int
    muPriority: int

class StbTexteditRow(ctypes.Structure):
    _fields_ = [
        ("x0", ctypes.c_float),
        ("x1", ctypes.c_float),
        ("baseline_y_delta", ctypes.c_float),
        ("ymin", ctypes.c_float),
        ("ymax", ctypes.c_float),
        ("num_chars", ctypes.c_int32),
    ]
    x0: float
    x1: float
    baseline_y_delta: float
    ymin: float
    ymax: float
    num_chars: int

class StbFindState(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("height", ctypes.c_float),
        ("first_char", ctypes.c_int32),
        ("length", ctypes.c_int32),
        ("prev_first", ctypes.c_int32),
    ]
    x: float
    y: float
    height: float
    first_char: int
    length: int
    prev_first: int

class tagBLOB(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("pBlobData", ctypes.c_ulonglong),
    ]
    cbSize: int
    pBlobData: int

class tagCY: pass

class tagCLIPDATA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("ulClipFmt", ctypes.c_int32),
        ("pClipData", ctypes.c_ulonglong),
    ]
    cbSize: int
    ulClipFmt: int
    pClipData: int

class tagBSTRBLOB(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("pData", ctypes.c_ulonglong),
    ]
    cbSize: int
    pData: int

class IUnknown(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IUnknown_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int

class tagTYPEDESC(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x8),  # $C7AD2EE2F8065E3462295697648C723A
        ("vt", ctypes.c_uint16),
    ]
    ___u0: bytes
    vt: int

class tagSAFEARRAYBOUND(ctypes.Structure):
    _fields_ = [
        ("cElements", ctypes.c_uint32),
        ("lLbound", ctypes.c_int32),
    ]
    cElements: int
    lLbound: int

class tagIDLDESC(ctypes.Structure):
    _fields_ = [
        ("dwReserved", ctypes.c_uint64),
        ("wIDLFlags", ctypes.c_uint16),
    ]
    dwReserved: int
    wIDLFlags: int

class tagVARIANT(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x10),  # $5B4E5A59A4E6F02DCCB8CA9DFE7DFB08
        ("pRecInfo", ctypes.c_ulonglong),
    ]
    ___u0: bytes
    pRecInfo: int

class tagDEC(ctypes.Structure):
    _fields_ = [
        ("wReserved", ctypes.c_uint16),
        ("___u1", ctypes.c_char * 0x2),  # $F26A21DDA3D5DC62639C1CD6BB6F5D69
        ("Hi32", ctypes.c_uint32),
        ("___u3", ctypes.c_char * 0x8),  # $DE4C13764B2FE821AB7326DD9FD6F3E8
    ]
    wReserved: int
    ___u1: bytes
    Hi32: int
    ___u3: bytes

class IRecordInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("RecordInit", ctypes.c_ulonglong),
        ("RecordClear", ctypes.c_ulonglong),
        ("RecordCopy", ctypes.c_ulonglong),
        ("GetGuid", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetTypeInfo", ctypes.c_ulonglong),
        ("GetField", ctypes.c_ulonglong),
        ("GetFieldNoCopy", ctypes.c_ulonglong),
        ("PutField", ctypes.c_ulonglong),
        ("PutFieldNoCopy", ctypes.c_ulonglong),
        ("GetFieldNames", ctypes.c_ulonglong),
        ("IsMatchingType", ctypes.c_ulonglong),
        ("RecordCreate", ctypes.c_ulonglong),
        ("RecordCreateCopy", ctypes.c_ulonglong),
        ("RecordDestroy", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    RecordInit: int
    RecordClear: int
    RecordCopy: int
    GetGuid: int
    GetName: int
    GetSize: int
    GetTypeInfo: int
    GetField: int
    GetFieldNoCopy: int
    PutField: int
    PutFieldNoCopy: int
    GetFieldNames: int
    IsMatchingType: int
    RecordCreate: int
    RecordCreateCopy: int
    RecordDestroy: int

class tagPARAMDESC(ctypes.Structure):
    _fields_ = [
        ("pparamdescex", ctypes.c_ulonglong),
        ("wParamFlags", ctypes.c_uint16),
    ]
    pparamdescex: int
    wParamFlags: int

class tagBINDPTR: pass

class ITypeComp_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Bind", ctypes.c_ulonglong),
        ("BindType", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Bind: int
    BindType: int

class tagDISPPARAMS(ctypes.Structure):
    _fields_ = [
        ("rgvarg", ctypes.c_ulonglong),
        ("rgdispidNamedArgs", ctypes.c_ulonglong),
        ("cArgs", ctypes.c_uint32),
        ("cNamedArgs", ctypes.c_uint32),
    ]
    rgvarg: int
    rgdispidNamedArgs: int
    cArgs: int
    cNamedArgs: int

class tagEXCEPINFO(ctypes.Structure):
    _fields_ = [
        ("wCode", ctypes.c_uint16),
        ("wReserved", ctypes.c_uint16),
        ("bstrSource", ctypes.c_ulonglong),
        ("bstrDescription", ctypes.c_ulonglong),
        ("bstrHelpFile", ctypes.c_ulonglong),
        ("dwHelpContext", ctypes.c_uint32),
        ("pvReserved", ctypes.c_void_p),
        ("pfnDeferredFillIn", ctypes.c_ulonglong),
        ("scode", ctypes.c_int32),
    ]
    wCode: int
    wReserved: int
    bstrSource: int
    bstrDescription: int
    bstrHelpFile: int
    dwHelpContext: int
    pvReserved: int
    pfnDeferredFillIn: int
    scode: int

class ITypeLib_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetTypeInfoCount", ctypes.c_ulonglong),
        ("GetTypeInfo", ctypes.c_ulonglong),
        ("GetTypeInfoType", ctypes.c_ulonglong),
        ("GetTypeInfoOfGuid", ctypes.c_ulonglong),
        ("GetLibAttr", ctypes.c_ulonglong),
        ("GetTypeComp", ctypes.c_ulonglong),
        ("GetDocumentation", ctypes.c_ulonglong),
        ("IsName", ctypes.c_ulonglong),
        ("FindName", ctypes.c_ulonglong),
        ("ReleaseTLibAttr", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetTypeInfoCount: int
    GetTypeInfo: int
    GetTypeInfoType: int
    GetTypeInfoOfGuid: int
    GetLibAttr: int
    GetTypeComp: int
    GetDocumentation: int
    IsName: int
    FindName: int
    ReleaseTLibAttr: int

class ITypeInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetTypeAttr", ctypes.c_ulonglong),
        ("GetTypeComp", ctypes.c_ulonglong),
        ("GetFuncDesc", ctypes.c_ulonglong),
        ("GetVarDesc", ctypes.c_ulonglong),
        ("GetNames", ctypes.c_ulonglong),
        ("GetRefTypeOfImplType", ctypes.c_ulonglong),
        ("GetImplTypeFlags", ctypes.c_ulonglong),
        ("GetIDsOfNames", ctypes.c_ulonglong),
        ("Invoke", ctypes.c_ulonglong),
        ("GetDocumentation", ctypes.c_ulonglong),
        ("GetDllEntry", ctypes.c_ulonglong),
        ("GetRefTypeInfo", ctypes.c_ulonglong),
        ("AddressOfMember", ctypes.c_ulonglong),
        ("CreateInstance", ctypes.c_ulonglong),
        ("GetMops", ctypes.c_ulonglong),
        ("GetContainingTypeLib", ctypes.c_ulonglong),
        ("ReleaseTypeAttr", ctypes.c_ulonglong),
        ("ReleaseFuncDesc", ctypes.c_ulonglong),
        ("ReleaseVarDesc", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetTypeAttr: int
    GetTypeComp: int
    GetFuncDesc: int
    GetVarDesc: int
    GetNames: int
    GetRefTypeOfImplType: int
    GetImplTypeFlags: int
    GetIDsOfNames: int
    Invoke: int
    GetDocumentation: int
    GetDllEntry: int
    GetRefTypeInfo: int
    AddressOfMember: int
    CreateInstance: int
    GetMops: int
    GetContainingTypeLib: int
    ReleaseTypeAttr: int
    ReleaseFuncDesc: int
    ReleaseVarDesc: int

class IDispatch_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetTypeInfoCount", ctypes.c_ulonglong),
        ("GetTypeInfo", ctypes.c_ulonglong),
        ("GetIDsOfNames", ctypes.c_ulonglong),
        ("Invoke", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetTypeInfoCount: int
    GetTypeInfo: int
    GetIDsOfNames: int
    Invoke: int

class ISequentialStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Read", ctypes.c_ulonglong),
        ("Write", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Read: int
    Write: int

class IStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Read", ctypes.c_ulonglong),
        ("Write", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("CopyTo", ctypes.c_ulonglong),
        ("Commit", ctypes.c_ulonglong),
        ("Revert", ctypes.c_ulonglong),
        ("LockRegion", ctypes.c_ulonglong),
        ("UnlockRegion", ctypes.c_ulonglong),
        ("Stat", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Read: int
    Write: int
    Seek: int
    SetSize: int
    CopyTo: int
    Commit: int
    Revert: int
    LockRegion: int
    UnlockRegion: int
    Stat: int
    Clone: int

class IEnumSTATSTG_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Next", ctypes.c_ulonglong),
        ("Skip", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Next: int
    Skip: int
    Reset: int
    Clone: int

class IStorage_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("CreateStream", ctypes.c_ulonglong),
        ("OpenStream", ctypes.c_ulonglong),
        ("CreateStorage", ctypes.c_ulonglong),
        ("OpenStorage", ctypes.c_ulonglong),
        ("CopyTo", ctypes.c_ulonglong),
        ("MoveElementTo", ctypes.c_ulonglong),
        ("Commit", ctypes.c_ulonglong),
        ("Revert", ctypes.c_ulonglong),
        ("EnumElements", ctypes.c_ulonglong),
        ("DestroyElement", ctypes.c_ulonglong),
        ("RenameElement", ctypes.c_ulonglong),
        ("SetElementTimes", ctypes.c_ulonglong),
        ("SetClass", ctypes.c_ulonglong),
        ("SetStateBits", ctypes.c_ulonglong),
        ("Stat", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    CreateStream: int
    OpenStream: int
    CreateStorage: int
    OpenStorage: int
    CopyTo: int
    MoveElementTo: int
    Commit: int
    Revert: int
    EnumElements: int
    DestroyElement: int
    RenameElement: int
    SetElementTimes: int
    SetClass: int
    SetStateBits: int
    Stat: int

class tagCAC(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_char_p),
    ]
    cElems: int
    pElems: bytes

class tagCAUB(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAI(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAUI(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAL(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAUL(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAH(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAUH(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAFLT(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCADBL(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCABOOL(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCASCODE(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCACY(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCADATE(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAFILETIME(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCACLSID(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCACLIPDATA(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCABSTR(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCABSTRBLOB(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCALPSTR(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCALPWSTR(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagCAPROPVARIANT(ctypes.Structure):
    _fields_ = [
        ("cElems", ctypes.c_uint32),
        ("pElems", ctypes.c_ulonglong),
    ]
    cElems: int
    pElems: int

class tagPROPVARIANT(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x18),  # $5147B814FBCB81A153F9EB9C1A9B0AA7
    ]
    ___u0: bytes

class TkGlobals(ctypes.Structure):
    pass

class cTkTextStylePlain_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTextStyle_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkTextStyle_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int

class cTkTextStylePlainHighlights_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTextStyle_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkTextStyle_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int

class cGcFoliageComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkDebugRenderHelper(ctypes.Structure):
    _fields_ = [
        ("mbActive", ctypes.c_ubyte),
    ]
    mbActive: bool

class cGcGalacticRegionEdge(ctypes.Structure):
    _fields_ = [
        ("_mFaceIndex", ctypes.c_uint8),
        ("mPosition", common.Vector2f),
    ]
    _mFaceIndex: int
    mPosition: common.Vector2f

    @property
    def mFaceIndex(self):
        return cGcRegionFace.Enum(self._mFaceIndex)


class cGcUniverseAddressCoderU64(ctypes.Structure):
    def Decode(): pass

    def InSameSystem(): pass


class cGcUniverseDetailAddressCoderU64(ctypes.Structure):
    def Encode(): pass

    def Decode(): pass


class cGcUniverseDetailAddressLayout(ctypes.Structure):
    class Bitfield: pass


class cGcRoutineGenerateQueue(ctypes.Structure):
    _fields_ = [
        ("mBuffer", ctypes.c_char * 0x18),  # std::vector<cGcRegionDecoratorCreatures *,TkSTLAllocatorShim<cGcRegionDecoratorCreatures *,8,-1> >
        ("miStart", ctypes.c_int32),
        ("miNext", ctypes.c_int32),
    ]
    mBuffer: bytes
    miStart: int
    miNext: int

class HgVocAlienVoiceChannel(ctypes.Structure):
    _fields_ = [
        ("mfAssociatedGameObjectID", ctypes.c_float),
        ("mfPriority", ctypes.c_float),
        ("mbOccupied", ctypes.c_ubyte),
        ("mbRequestSoftEnd", ctypes.c_ubyte),
    ]
    mfAssociatedGameObjectID: float
    mfPriority: float
    mbOccupied: bool
    mbRequestSoftEnd: bool

class CAkVPLNode(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def GetChannelMappingFunc(): pass

    def GetPitch(): pass

    def PopMarkers(): pass

    def Seek(): pass

    def VirtualOff(): pass

    def VirtualOn(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_pInput", ctypes.c_ulonglong),
    ]
    __vftable: int
    m_pInput: int

class AkChannelConfig(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int32),
    ]
    _bf_0: int

class CAkVPLNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int

class AkBankKey(ctypes.Structure):
    _fields_ = [
        ("bankID", ctypes.c_uint32),
        ("pInMemoryPtr", ctypes.c_void_p),
    ]
    bankID: int
    pInMemoryPtr: int

class CAkIndexable(ctypes.Structure):
    def CAkIndexable(): pass

    def tilde_CAkIndexable(): pass

    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("pNextItem", ctypes.c_ulonglong),
        ("key", ctypes.c_uint32),
        ("m_lRef", ctypes.c_int32),
    ]
    __vftable: int
    pNextItem: int
    key: int
    m_lRef: int

class CAkIndexable_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int

class CAkTransportAware(ctypes.Structure):
    def vector_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_uRegisteredNotif", ctypes.c_uint32),
    ]
    __vftable: int
    m_uRegisteredNotif: int

class CAkTransportAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("CAkTransportAware_dtor_8", ctypes.c_ulonglong),
    ]
    _Stop: int
    CAkTransportAware_dtor_8: int

class ITransitionable(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class ITransitionable_vtbl(ctypes.Structure):
    _fields_ = [
        ("TransUpdateValue", ctypes.c_ulonglong),
    ]
    TransUpdateValue: int

class CAkParameterTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkParameterTarget_dtor_0", ctypes.c_ulonglong),
        ("UpdateTargetNodeParam", ctypes.c_ulonglong),
        ("NotifyParamChanged", ctypes.c_ulonglong),
        ("NotifyParamsChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterTarget *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("GetTargetedParamsSet", ctypes.c_char * 0x8),  # CAkBitArray<unsigned __int64,0> *(__fastcall *)(CAkParameterTarget *this, CAkBitArray<unsigned __int64,0> *result)
        ("RegisterToBusHierarchy", ctypes.c_ulonglong),
        ("UpdateTargetParam", ctypes.c_ulonglong),
    ]
    CAkParameterTarget_dtor_0: int
    UpdateTargetNodeParam: int
    NotifyParamChanged: int
    NotifyParamsChanged: bytes
    GetTargetedParamsSet: bytes
    RegisterToBusHierarchy: int
    UpdateTargetParam: int

class CAkScopedRtpcObj(ctypes.Structure):
    def CAkScopedRtpcObj(): pass

    def AddedNewModulatorCtx(): pass

    def AddedNewRtpcValue(): pass

    def Term(): pass

    _fields_ = [
        ("m_Modulators", ctypes.c_char * 0x10),  # AkSortedKeyArray<unsigned int,unsigned int,AkArrayAllocatorNoAlign<0>,AkAkRtpcIDSetGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int>,AkDefaultSortedKeyCompare<unsigned int> >
        ("m_RtpcVals", ctypes.c_char * 0x10),  # AkSortedKeyArray<unsigned int,unsigned int,AkArrayAllocatorNoAlign<0>,AkAkRtpcIDSetGetKey,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int>,AkDefaultSortedKeyCompare<unsigned int> >
    ]
    m_Modulators: bytes
    m_RtpcVals: bytes

class CAkGameObject_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObject_dtor_0", ctypes.c_ulonglong),
    ]
    CAkGameObject_dtor_0: int

class CAkGameObjComponent(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_pOwner", ctypes.c_ulonglong),
    ]
    __vftable: int
    m_pOwner: int

class CAkGameObjComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class AkGameObjComponentPtr(ctypes.Structure):
    _fields_ = [
        ("ptr", ctypes.c_ulonglong),
    ]
    ptr: int

class CAkRegisteredObj_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObject_dtor_0", ctypes.c_ulonglong),
    ]
    CAkGameObject_dtor_0: int

class AkRTPCKey(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x30),  # AkNestedKey<CAkRegisteredObj *,GetNullGameObjPtr,AkNestedKey<unsigned int,GetInvalidPlayingID,AkNestedKey<unsigned int,GetInvalidUniqueID,AkNestedKey<unsigned char,GetInvalidMidiCh,AkNestedKey<unsigned char,GetInvalidMidiNote,AkRootKey<CAkPBI *,GetNullPbiPtr> > > > > >
    ]
    baseclass_0: bytes

class IAkEffectSlotsOwner(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IAkEffectSlotsOwner_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int

class AkExternalSourceInfo(ctypes.Structure):
    _fields_ = [
        ("iExternalSrcCookie", ctypes.c_uint32),
        ("idCodec", ctypes.c_uint32),
        ("szFile", ctypes.c_ulonglong),
        ("pInMemory", ctypes.c_void_p),
        ("uiMemorySize", ctypes.c_uint32),
        ("idFile", ctypes.c_uint32),
    ]
    iExternalSrcCookie: int
    idCodec: int
    szFile: int
    pInMemory: int
    uiMemorySize: int
    idFile: int

class AkCustomParamType(ctypes.Structure):
    _fields_ = [
        ("pExternalSrcs", ctypes.c_char * 0x8),  # AkQueueMsgPtr<AkExternalSourceArray>
        ("customParam", ctypes.c_int64),
        ("ui32Reserved", ctypes.c_uint32),
    ]
    pExternalSrcs: bytes
    customParam: int
    ui32Reserved: int

class AkCntrHistArray(ctypes.Structure):
    _fields_ = [
        ("uiArraySize", ctypes.c_uint32),
        ("aCntrHist", ctypes.c_uint16 * 0x20),  # unsigned __int16[32]
    ]
    uiArraySize: int
    aCntrHist: list[ctypes.c_uint16]

class AkVector(ctypes.Structure):
    _fields_ = [
        ("X", ctypes.c_float),
        ("Y", ctypes.c_float),
        ("Z", ctypes.c_float),
    ]
    X: float
    Y: float
    Z: float

class AkPathListItem(ctypes.Structure):
    _fields_ = [
        ("pVertices", ctypes.c_ulonglong),
        ("iNumVertices", ctypes.c_int32),
        ("fRangeX", ctypes.c_float),
        ("fRangeY", ctypes.c_float),
        ("fRangeZ", ctypes.c_float),
    ]
    pVertices: int
    iNumVertices: int
    fRangeX: float
    fRangeY: float
    fRangeZ: float

class AkSharedPathState(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # AkArray<bool,bool,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<bool> >
        ("m_cRef", ctypes.c_uint16),
    ]
    baseclass_0: bytes
    m_cRef: int

class AkSafePathStatePtr(ctypes.Structure):
    _fields_ = [
        ("ptr", ctypes.c_ulonglong),
    ]
    ptr: int

class AkPathInfo(ctypes.Structure):
    _fields_ = [
        ("pPBPath", ctypes.c_ulonglong),
        ("PathOwnerID", ctypes.c_uint32),
    ]
    pPBPath: int
    PathOwnerID: int

class CAkContainerBase(ctypes.Structure):
    def CAkContainerBase(): pass

    def tilde_CAkContainerBase(): pass

    def CanAddChild(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x110),  # CAkActiveParent<CAkParameterNode>
    ]
    baseclass_0: bytes

class CAkPlayList(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class CAkPlayList_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkPlayList_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
        ("Length", ctypes.c_ulonglong),
        ("ID", ctypes.c_ulonglong),
        ("Add", ctypes.c_ulonglong),
        ("Remove", ctypes.c_ulonglong),
        ("RemoveAll", ctypes.c_ulonglong),
        ("Exists", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetWeight", ctypes.c_ulonglong),
    ]
    CAkPlayList_dtor_0: int
    Init: int
    Destroy: int
    Length: int
    ID: int
    Add: int
    Remove: int
    RemoveAll: int
    Exists: int
    GetPosition: int
    GetWeight: int

class CAkContainerBaseInfo(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class CAkContainerBaseInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkContainerBaseInfo_dtor_0", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("SerializeHistory", ctypes.c_ulonglong),
        ("DeserializeHistory", ctypes.c_ulonglong),
    ]
    CAkContainerBaseInfo_dtor_0: int
    Destroy: int
    Type: int
    Clone: int
    SerializeHistory: int
    DeserializeHistory: int

class AkLoop(ctypes.Structure):
    _fields_ = [
        ("lLoopCount", ctypes.c_int16),
        ("_bf_2", ctypes.c_int8),
    ]
    lLoopCount: int
    _bf_2: int

class CAkContinuationList(ctypes.Structure):
    def AddRef(): pass

    def Create(): pass

    def Release(): pass

    _fields_ = [
        ("m_listItems", ctypes.c_char * 0x10),  # AkArray<CAkContinueListItem,CAkContinueListItem const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkContinueListItem> >
        ("m_iRefCount", ctypes.c_int32),
    ]
    m_listItems: bytes
    m_iRefCount: int

class CAkModulatorCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkModulatorCtx_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Tick", ctypes.c_ulonglong),
        ("GetParamsPtr", ctypes.c_ulonglong),
        ("CalcBufferNeeded", ctypes.c_ulonglong),
        ("InitializeOutput", ctypes.c_ulonglong),
    ]
    CAkModulatorCtx_dtor_0: int
    SetParam: int
    Tick: int
    GetParamsPtr: int
    CalcBufferNeeded: int
    InitializeOutput: int

class CAkMidiNoteStateAware(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class CAkMidiNoteStateAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnNoteStateFinished", ctypes.c_ulonglong),
    ]
    OnNoteStateFinished: int

class AkMidiNoteChannelPair(ctypes.Structure):
    _fields_ = [
        ("note", ctypes.c_uint8),
        ("channel", ctypes.c_uint8),
    ]
    note: int
    channel: int

class WwiseObjectIDext(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint32),
        ("bIsBus", ctypes.c_ubyte),
    ]
    id: int
    bIsBus: bool

class AkPendingAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("TransUpdateValue", ctypes.c_ulonglong),
        ("AkPendingAction_dtor_8", ctypes.c_ulonglong),
    ]
    TransUpdateValue: int
    AkPendingAction_dtor_8: int

class CAkAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int

class CAkActionPlay_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("GetHistArray", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    GetHistArray: int

class PlaybackTransition(ctypes.Structure):
    _fields_ = [
        ("pvPSTrans", ctypes.c_ulonglong),
        ("pvPRTrans", ctypes.c_ulonglong),
    ]
    pvPSTrans: int
    pvPRTrans: int

class DelayContinuousParams(ctypes.Structure):
    _fields_ = [
        ("uNextSelectedNodeID", ctypes.c_uint32),
        ("_bf_4", ctypes.c_int8),
    ]
    uNextSelectedNodeID: int
    _bf_4: int

class CAkModCtxRefContainer(ctypes.Structure):
    def tilde_CAkModCtxRefContainer(): pass

    def HasModulatorCtx(): pass

    def Release(): pass

    _fields_ = [
        ("m_Ctx", ctypes.c_char * 0x10),  # AkArray<CAkModulatorCtx *,CAkModulatorCtx *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkModulatorCtx *> >
    ]
    m_Ctx: bytes

class CAkActionPlayAndContinue_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("GetHistArray", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    GetHistArray: int

class AkModTreeValue(ctypes.Structure):
    _fields_ = [
        ("pCtx", ctypes.c_ulonglong),
    ]
    pCtx: int

class CAkModulator_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetInitialValues", ctypes.c_ulonglong),
        ("GetInitialParams", ctypes.c_ulonglong),
        ("SupportsAutomatedParams", ctypes.c_ulonglong),
        ("_Trigger", ctypes.c_ulonglong),
        ("ShouldRetrigger", ctypes.c_ulonglong),
        ("IsTriggerOk", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetInitialValues: int
    GetInitialParams: int
    SupportsAutomatedParams: int
    _Trigger: int
    ShouldRetrigger: int
    IsTriggerOk: int

class AkMutedMapItem(ctypes.Structure):
    _fields_ = [
        ("m_Identifier", ctypes.c_void_p),
        ("_bf_8", ctypes.c_int8),
    ]
    m_Identifier: int
    _bf_8: int

class AkMutedMap(ctypes.Structure):
    def MuteNotification(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # CAkKeyArray<AkMutedMapItem,float,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<AkMutedMapItem,float> > >
    ]
    baseclass_0: bytes

class AkPBIModValues(ctypes.Structure):
    _fields_ = [
        ("VolumeOffset", ctypes.c_float),
        ("MakeUpGainOffset", ctypes.c_float),
        ("PitchOffset", ctypes.c_float),
        ("LPFOffset", ctypes.c_float),
        ("HPFOffset", ctypes.c_float),
    ]
    VolumeOffset: float
    MakeUpGainOffset: float
    PitchOffset: float
    LPFOffset: float
    HPFOffset: float

class AkModulatorArray(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # AkArray<CAkModulator *,CAkModulator *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkModulator *> >
        ("key", ctypes.c_uint64),
        ("pNextItem", ctypes.c_ulonglong),
    ]
    baseclass_0: bytes
    key: int
    pNextItem: int

class CAkDuckItem_vtbl(ctypes.Structure):
    _fields_ = [
        ("TransUpdateValue", ctypes.c_ulonglong),
    ]
    TransUpdateValue: int

class AkRTPCExceptionChecker(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class AkRTPCExceptionChecker_vtbl(ctypes.Structure):
    _fields_ = [
        ("IsException", ctypes.c_ulonglong),
    ]
    IsException: int

class CAkRTPCSubscriberNode(ctypes.Structure):
    def CAkRTPCSubscriberNode(): pass

    def tilde_CAkRTPCSubscriberNode(): pass

    def CreateData(): pass

    def DestroyData(): pass

    def EnableParam(): pass

    def GetModulatorParamXfrm(): pass

    def NotifyParamChanged(): pass

    def NotifyParamsChanged(): pass

    def PushParamUpdate(): pass

    def PushParamUpdate_All(): pass

    def PushParamUpdate_Scoped(): pass

    def PushParamUpdate_ScopedExcept(): pass

    def SetRTPC(): pass

    def UnregisterAllParameterTargets(): pass

    def UnsetRTPC(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_Data", ctypes.c_ulonglong),
        ("m_paramNodeID", ctypes.c_uint32),
    ]
    __vftable: int
    m_Data: int
    m_paramNodeID: int

class CAkRTPCSubscriberNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("PushParamUpdate", ctypes.c_ulonglong),
        ("NotifyParamChanged", ctypes.c_ulonglong),
        ("NotifyParamsChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkRTPCSubscriberNode *this, bool, const CAkBitArray<unsigned __int64,0> *)
    ]
    PushParamUpdate: int
    NotifyParamChanged: int
    NotifyParamsChanged: bytes

class RegisteredTarget(ctypes.Structure):
    _fields_ = [
        ("params", ctypes.c_char * 0x8),  # CAkBitArray<unsigned __int64,0>
        ("pTgt", ctypes.c_ulonglong),
    ]
    params: bytes
    pTgt: int

class AkSortedParamTrtArray(ctypes.Structure):
    def InsertTarget(): pass

    def FindArrayPos(): pass

    def RemoveTarget(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # AkArray<RegisteredTarget,RegisteredTarget,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<RegisteredTarget> >
    ]
    baseclass_0: bytes

class AkModulatorXfrm(ctypes.Structure):
    _fields_ = [
        ("m_fOffset", ctypes.c_float),
        ("m_fScale", ctypes.c_float),
    ]
    m_fOffset: float
    m_fScale: float

class CAkModulatorData(ctypes.Structure):
    def CAkModulatorData(): pass

    def tilde_CAkModulatorData(): pass

    def AllocData(): pass

    _fields_ = [
        ("m_pData", ctypes.c_ulonglong),
        ("m_uReleaseOffset", ctypes.c_uint32),
    ]
    m_pData: int
    m_uReleaseOffset: int

class CAkLayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int

class AkMediaInformation(ctypes.Structure):
    _fields_ = [
        ("sourceID", ctypes.c_uint32),
        ("uFileID", ctypes.c_uint32),
        ("uInMemoryMediaSize", ctypes.c_uint32),
        ("_bf_c", ctypes.c_int16),
    ]
    sourceID: int
    uFileID: int
    uInMemoryMediaSize: int
    _bf_c: int

class AkPriorityStruct(ctypes.Structure):
    _fields_ = [
        ("priority", ctypes.c_float),
        ("pipelineID", ctypes.c_uint32),
        ("seqID", ctypes.c_uint32),
    ]
    priority: float
    pipelineID: int
    seqID: int

class PriorityInfo(ctypes.Structure):
    _fields_ = [
        ("priority", ctypes.c_float),
        ("distanceOffset", ctypes.c_float),
    ]
    priority: float
    distanceOffset: float

class CAkPBIAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("CreatePBI", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    CreatePBI: int

class AkStatePropertyInfo(ctypes.Structure):
    _fields_ = [
        ("propertyId", ctypes.c_uint16),
        ("accumType", ctypes.c_uint8),
        ("_bf_3", ctypes.c_int8),
    ]
    propertyId: int
    accumType: int
    _bf_3: int

class CAkState_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int

class AkStateLink(ctypes.Structure):
    _fields_ = [
        ("pState", ctypes.c_ulonglong),
    ]
    pState: int

class AkStateGroupChunk_vtbl(ctypes.Structure):
    _fields_ = [
        ("TransUpdateValue", ctypes.c_ulonglong),
    ]
    TransUpdateValue: int

class AkStateUpdate(ctypes.Structure):
    _fields_ = [
        ("ulStateInstanceID", ctypes.c_uint32),
        ("ulStateID", ctypes.c_uint32),
    ]
    ulStateInstanceID: int
    ulStateID: int

class CAkStateAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkStateAware_dtor_0", ctypes.c_ulonglong),
        ("StateTransitionAdded", ctypes.c_ulonglong),
        ("StateTransitionRemoved", ctypes.c_ulonglong),
        ("HasState", ctypes.c_ulonglong),
        ("IsStateProperty", ctypes.c_ulonglong),
        ("SetStateProperties", ctypes.c_ulonglong),
        ("GetStateProperties", ctypes.c_char * 0x8),  # AkArray<AkStatePropertyInfo,AkStatePropertyInfo &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkStatePropertyInfo> > *(__fastcall *)(CAkStateAware *this)
        ("UpdateStateParamTargets", ctypes.c_ulonglong),
        ("PushStateParamUpdate", ctypes.c_ulonglong),
        ("NotifyStateParamTargets", ctypes.c_ulonglong),
        ("RemoveStateGroup", ctypes.c_ulonglong),
        ("AddStateGroup", ctypes.c_ulonglong),
        ("UpdateStateGroups", ctypes.c_ulonglong),
        ("ReadStateChunk", ctypes.c_ulonglong),
        ("EnsureStateData", ctypes.c_ulonglong),
        ("GetStateChunks", ctypes.c_char * 0x8),  # AkListBareLight<AkStateGroupChunk,AkListBareLightNextInNode> *(__fastcall *)(CAkStateAware *this)
    ]
    CAkStateAware_dtor_0: int
    StateTransitionAdded: int
    StateTransitionRemoved: int
    HasState: int
    IsStateProperty: int
    SetStateProperties: int
    GetStateProperties: bytes
    UpdateStateParamTargets: int
    PushStateParamUpdate: int
    NotifyStateParamTargets: int
    RemoveStateGroup: int
    AddStateGroup: int
    UpdateStateGroups: int
    ReadStateChunk: int
    EnsureStateData: int
    GetStateChunks: bytes

class StateRegisteredTarget(ctypes.Structure):
    _fields_ = [
        ("params", ctypes.c_char * 0x8),  # CAkBitArray<unsigned __int64,0>
        ("pTgt", ctypes.c_ulonglong),
    ]
    params: bytes
    pTgt: int

class AkStateParamTargetArray(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # AkArray<StateRegisteredTarget,StateRegisteredTarget,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<StateRegisteredTarget> >
    ]
    baseclass_0: bytes

class CAkParamNodeStateAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkStateAware_dtor_0", ctypes.c_ulonglong),
        ("StateTransitionAdded", ctypes.c_ulonglong),
        ("StateTransitionRemoved", ctypes.c_ulonglong),
        ("HasState", ctypes.c_ulonglong),
        ("IsStateProperty", ctypes.c_ulonglong),
        ("SetStateProperties", ctypes.c_ulonglong),
        ("GetStateProperties", ctypes.c_char * 0x8),  # AkArray<AkStatePropertyInfo,AkStatePropertyInfo &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkStatePropertyInfo> > *(__fastcall *)(CAkStateAware *this)
        ("UpdateStateParamTargets", ctypes.c_ulonglong),
        ("PushStateParamUpdate", ctypes.c_ulonglong),
        ("NotifyStateParamTargets", ctypes.c_ulonglong),
        ("RemoveStateGroup", ctypes.c_ulonglong),
        ("AddStateGroup", ctypes.c_ulonglong),
        ("UpdateStateGroups", ctypes.c_ulonglong),
        ("ReadStateChunk", ctypes.c_ulonglong),
        ("EnsureStateData", ctypes.c_ulonglong),
        ("GetStateChunks", ctypes.c_char * 0x8),  # AkListBareLight<AkStateGroupChunk,AkListBareLightNextInNode> *(__fastcall *)(CAkStateAware *this)
    ]
    CAkStateAware_dtor_0: int
    StateTransitionAdded: int
    StateTransitionRemoved: int
    HasState: int
    IsStateProperty: int
    SetStateProperties: int
    GetStateProperties: bytes
    UpdateStateParamTargets: int
    PushStateParamUpdate: int
    NotifyStateParamTargets: int
    RemoveStateGroup: int
    AddStateGroup: int
    UpdateStateGroups: int
    ReadStateChunk: int
    EnsureStateData: int
    GetStateChunks: bytes

class AkObjectInfo(ctypes.Structure):
    _fields_ = [
        ("objID", ctypes.c_uint32),
        ("parentID", ctypes.c_uint32),
        ("iDepth", ctypes.c_int32),
    ]
    objID: int
    parentID: int
    iDepth: int

class CounterParameters(ctypes.Structure):
    _fields_ = [
        ("pGameObj", ctypes.c_ulonglong),
        ("pLimiters", ctypes.c_char * 0x8),  # AkArray<CAkLimiter *,CAkLimiter *,AkHybridAllocator<8,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLimiter *> > *
        ("pFromChild", ctypes.c_ulonglong),
        ("pPBI", ctypes.c_ulonglong),
        ("fPriority", ctypes.c_float),
        ("uiFlagForwardToBus", ctypes.c_uint16),
        ("ui16NumKicked", ctypes.c_uint16),
        ("bMaxConsidered", ctypes.c_ubyte),
        ("bAllowKick", ctypes.c_ubyte),
    ]
    pGameObj: int
    pLimiters: bytes
    pFromChild: int
    pPBI: int
    fPriority: float
    uiFlagForwardToBus: int
    ui16NumKicked: int
    bMaxConsidered: bool
    bAllowKick: bool

class AkPropValue(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $5BD8C4416EEFC2680BDE288CA4FC5E48
    ]
    ___u0: bytes

class FxBaseStateAwareData(ctypes.Structure):
    _fields_ = [
        ("m_stateChunks", ctypes.c_char * 0x10),  # AkListBareLight<AkStateGroupChunk,AkListBareLightNextInNode>
        ("m_stateProperties", ctypes.c_char * 0x10),  # AkArray<AkStatePropertyInfo,AkStatePropertyInfo &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkStatePropertyInfo> >
    ]
    m_stateChunks: bytes
    m_stateProperties: bytes

class CAkFxBaseStateAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkStateAware_dtor_0", ctypes.c_ulonglong),
        ("StateTransitionAdded", ctypes.c_ulonglong),
        ("StateTransitionRemoved", ctypes.c_ulonglong),
        ("HasState", ctypes.c_ulonglong),
        ("IsStateProperty", ctypes.c_ulonglong),
        ("SetStateProperties", ctypes.c_ulonglong),
        ("GetStateProperties", ctypes.c_char * 0x8),  # AkArray<AkStatePropertyInfo,AkStatePropertyInfo &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkStatePropertyInfo> > *(__fastcall *)(CAkStateAware *this)
        ("UpdateStateParamTargets", ctypes.c_ulonglong),
        ("PushStateParamUpdate", ctypes.c_ulonglong),
        ("NotifyStateParamTargets", ctypes.c_ulonglong),
        ("RemoveStateGroup", ctypes.c_ulonglong),
        ("AddStateGroup", ctypes.c_ulonglong),
        ("UpdateStateGroups", ctypes.c_ulonglong),
        ("ReadStateChunk", ctypes.c_ulonglong),
        ("EnsureStateData", ctypes.c_ulonglong),
        ("GetStateChunks", ctypes.c_char * 0x8),  # AkListBareLight<AkStateGroupChunk,AkListBareLightNextInNode> *(__fastcall *)(CAkStateAware *this)
    ]
    CAkStateAware_dtor_0: int
    StateTransitionAdded: int
    StateTransitionRemoved: int
    HasState: int
    IsStateProperty: int
    SetStateProperties: int
    GetStateProperties: bytes
    UpdateStateParamTargets: int
    PushStateParamUpdate: int
    NotifyStateParamTargets: int
    RemoveStateGroup: int
    AddStateGroup: int
    UpdateStateGroups: int
    ReadStateChunk: int
    EnsureStateData: int
    GetStateChunks: bytes

class PluginPropertyValue(ctypes.Structure):
    _fields_ = [
        ("propertyId", ctypes.c_uint16),
        ("rtpcAccum", ctypes.c_uint8),
        ("fValue", ctypes.c_float),
    ]
    propertyId: int
    rtpcAccum: int
    fValue: float

class IAkRTPCSubscriberPlugin(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IAkRTPCSubscriberPlugin_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriberPlugin_dtor_0", ctypes.c_ulonglong),
        ("GetPluginID", ctypes.c_ulonglong),
        ("UpdateTargetRtpcParam", ctypes.c_ulonglong),
        ("NotifyRtpcParamChanged", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriberPlugin_dtor_0: int
    GetPluginID: int
    UpdateTargetRtpcParam: int
    NotifyRtpcParamChanged: int

class PluginRTPCSub_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriberPlugin_dtor_0", ctypes.c_ulonglong),
        ("GetPluginID", ctypes.c_ulonglong),
        ("UpdateTargetRtpcParam", ctypes.c_ulonglong),
        ("NotifyRtpcParamChanged", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriberPlugin_dtor_0: int
    GetPluginID: int
    UpdateTargetRtpcParam: int
    NotifyRtpcParamChanged: int
    Term: int

class CAkFxBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("IsShareSet", ctypes.c_ulonglong),
        ("SetInitialValues", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    IsShareSet: int
    SetInitialValues: int

class AkFXDesc(ctypes.Structure):
    _fields_ = [
        ("pFx", ctypes.c_char * 0x8),  # CAkSmartPtr<CAkFxBase>
        ("iBypassed", ctypes.c_int16),
    ]
    pFx: bytes
    iBypassed: int

class CAkSIS_vtbl(ctypes.Structure):
    _fields_ = [
        ("TransUpdateValue", ctypes.c_ulonglong),
    ]
    TransUpdateValue: int

class CAkParamTargetLimiter_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkParameterTarget_dtor_0", ctypes.c_ulonglong),
        ("UpdateTargetNodeParam", ctypes.c_ulonglong),
        ("NotifyParamChanged", ctypes.c_ulonglong),
        ("NotifyParamsChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterTarget *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("GetTargetedParamsSet", ctypes.c_char * 0x8),  # CAkBitArray<unsigned __int64,0> *(__fastcall *)(CAkParameterTarget *this, CAkBitArray<unsigned __int64,0> *result)
        ("RegisterToBusHierarchy", ctypes.c_ulonglong),
        ("UpdateTargetParam", ctypes.c_ulonglong),
    ]
    CAkParameterTarget_dtor_0: int
    UpdateTargetNodeParam: int
    NotifyParamChanged: int
    NotifyParamsChanged: bytes
    GetTargetedParamsSet: bytes
    RegisterToBusHierarchy: int
    UpdateTargetParam: int

class StructMaxInst(ctypes.Structure):
    def DisableLimiter(): pass

    _fields_ = [
        ("m_pLimiter", ctypes.c_ulonglong),
    ]
    m_pLimiter: int

class AkActivityCounter(ctypes.Structure):
    _fields_ = [
        ("m_iCount", ctypes.c_int16),
        ("m_iRoutedCount", ctypes.c_int16),
    ]
    m_iCount: int
    m_iRoutedCount: int

class AkPositioningSettings(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
        ("_bf_1", ctypes.c_int8),
    ]
    _bf_0: int
    _bf_1: int

class CAkParameterNodeBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int

class ConeParams(ctypes.Structure):
    _fields_ = [
        ("fInsideAngle", ctypes.c_float),
        ("fOutsideAngle", ctypes.c_float),
        ("fOutsideVolume", ctypes.c_float),
        ("LoPass", ctypes.c_float),
        ("HiPass", ctypes.c_float),
    ]
    fInsideAngle: float
    fOutsideAngle: float
    fOutsideVolume: float
    LoPass: float
    HiPass: float

class CAkAttenuation_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int

class BaseGenParams(ctypes.Structure):
    _fields_ = [
        ("m_fPAN_X_2D", ctypes.c_float),
        ("m_fPAN_Y_2D", ctypes.c_float),
        ("m_fPAN_Z_2D", ctypes.c_float),
        ("m_fCenterPCT", ctypes.c_float),
        ("_bf_10", ctypes.c_int8),
    ]
    m_fPAN_X_2D: float
    m_fPAN_Y_2D: float
    m_fPAN_Z_2D: float
    m_fCenterPCT: float
    _bf_10: int

class CAkSpatialAudioVoice(ctypes.Structure):
    def Term(): pass

    def vector_deleting_destructor(): pass

    def Create(): pass

    def GetVolumeRayCategories(): pass

    def Init(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_pOwner", ctypes.c_ulonglong),
    ]
    __vftable: int
    m_pOwner: int

class CAkSpatialAudioVoice_vtbl(ctypes.Structure):
    _fields_ = [
        ("Init", ctypes.c_ulonglong),
        ("CAkSpatialAudioVoice_dtor_8", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("ParamsUpdated", ctypes.c_ulonglong),
        ("GetAuxSendsValues", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkSpatialAudioVoice *this, AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *, float, float, float)
        ("HasReflections", ctypes.c_ulonglong),
        ("HasDiffraction", ctypes.c_ulonglong),
        ("HasTransmission", ctypes.c_ulonglong),
        ("IsReflectionsBus", ctypes.c_ulonglong),
    ]
    Init: int
    CAkSpatialAudioVoice_dtor_8: int
    Term: int
    ParamsUpdated: int
    GetAuxSendsValues: bytes
    HasReflections: int
    HasDiffraction: int
    HasTransmission: int
    IsReflectionsBus: int

class CAkBehavioralCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkParameterTarget_dtor_0", ctypes.c_ulonglong),
        ("UpdateTargetNodeParam", ctypes.c_ulonglong),
        ("NotifyParamChanged", ctypes.c_ulonglong),
        ("NotifyParamsChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterTarget *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("GetTargetedParamsSet", ctypes.c_char * 0x8),  # CAkBitArray<unsigned __int64,0> *(__fastcall *)(CAkParameterTarget *this, CAkBitArray<unsigned __int64,0> *result)
        ("RegisterToBusHierarchy", ctypes.c_ulonglong),
        ("UpdateTargetParam", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("GetModulatorTriggerParams", ctypes.c_ulonglong),
        ("RefreshParameters", ctypes.c_ulonglong),
        ("CalculateMutedEffectiveVolume", ctypes.c_ulonglong),
        ("OnPathAdded", ctypes.c_ulonglong),
        ("UpdatePriorityWithDistance", ctypes.c_ulonglong),
    ]
    CAkParameterTarget_dtor_0: int
    UpdateTargetNodeParam: int
    NotifyParamChanged: int
    NotifyParamsChanged: bytes
    GetTargetedParamsSet: bytes
    RegisterToBusHierarchy: int
    UpdateTargetParam: int
    Init: int
    Term: int
    GetModulatorTriggerParams: int
    RefreshParameters: int
    CalculateMutedEffectiveVolume: int
    OnPathAdded: int
    UpdatePriorityWithDistance: int

class AkRamp(ctypes.Structure):
    _fields_ = [
        ("fPrev", ctypes.c_float),
        ("fNext", ctypes.c_float),
    ]
    fPrev: float
    fNext: float

class AkPluginArrayAllocator(ctypes.Structure):
    _fields_ = [
        ("m_pAllocator", ctypes.c_ulonglong),
    ]
    m_pAllocator: int

class AkBQFParams(ctypes.Structure):
    _fields_ = [
        ("fCurrentPar", ctypes.c_float),
        ("fTargetPar", ctypes.c_float),
        ("uNumInterBlocks", ctypes.c_uint16),
        ("iNextBypass", ctypes.c_char),
        ("bTargetDirty", ctypes.c_ubyte),
        ("bFirstSet", ctypes.c_ubyte),
        ("bBypass", ctypes.c_ubyte),
        ("bDcRemoved", ctypes.c_ubyte),
    ]
    fCurrentPar: float
    fTargetPar: float
    uNumInterBlocks: int
    iNextBypass: str
    bTargetDirty: bool
    bFirstSet: bool
    bBypass: bool
    bDcRemoved: bool

class AkAudioMix(ctypes.Structure):
    def Allocate(): pass

    def Free(): pass

    _fields_ = [
        ("pMemory", ctypes.c_void_p),
        ("uAllocdMemSize", ctypes.c_uint32),
        ("pNext", ctypes.c_ulonglong),
        ("pPrevious", ctypes.c_ulonglong),
    ]
    pMemory: int
    uAllocdMemSize: int
    pNext: int
    pPrevious: int

class AkDeviceDesc(ctypes.Structure):
    _fields_ = [
        ("deviceID", ctypes.c_uint32),
        ("bCanWrite", ctypes.c_ubyte),
        ("bCanRead", ctypes.c_ubyte),
        ("szDeviceName", ctypes.c_wchar * 0x10),  # wchar_t[16]
        ("uStringSize", ctypes.c_uint32),
    ]
    deviceID: int
    bCanWrite: bool
    bCanRead: bool
    szDeviceName: list[ctypes.c_wchar]
    uStringSize: int

class AkDeviceData(ctypes.Structure):
    _fields_ = [
        ("deviceID", ctypes.c_uint32),
        ("uMemSize", ctypes.c_uint32),
        ("uMemUsed", ctypes.c_uint32),
        ("uAllocs", ctypes.c_uint32),
        ("uFrees", ctypes.c_uint32),
        ("uPeakRefdMemUsed", ctypes.c_uint32),
        ("uUnreferencedCachedBytes", ctypes.c_uint32),
        ("uGranularity", ctypes.c_uint32),
        ("uNumActiveStreams", ctypes.c_uint32),
        ("uTotalBytesTransferred", ctypes.c_uint32),
        ("uLowLevelBytesTransferred", ctypes.c_uint32),
        ("fAvgCacheEfficiency", ctypes.c_float),
        ("uNumLowLevelRequestsCompleted", ctypes.c_uint32),
        ("uNumLowLevelRequestsCancelled", ctypes.c_uint32),
        ("uNumLowLevelRequestsPending", ctypes.c_uint32),
        ("uCustomParam", ctypes.c_uint32),
        ("uCachePinnedBytes", ctypes.c_uint32),
    ]
    deviceID: int
    uMemSize: int
    uMemUsed: int
    uAllocs: int
    uFrees: int
    uPeakRefdMemUsed: int
    uUnreferencedCachedBytes: int
    uGranularity: int
    uNumActiveStreams: int
    uTotalBytesTransferred: int
    uLowLevelBytesTransferred: int
    fAvgCacheEfficiency: float
    uNumLowLevelRequestsCompleted: int
    uNumLowLevelRequestsCancelled: int
    uNumLowLevelRequestsPending: int
    uCustomParam: int
    uCachePinnedBytes: int

class AkStreamRecord(ctypes.Structure):
    _fields_ = [
        ("uStreamID", ctypes.c_uint32),
        ("deviceID", ctypes.c_uint32),
        ("szStreamName", ctypes.c_wchar * 0x40),  # wchar_t[64]
        ("uStringSize", ctypes.c_uint32),
        ("uFileSize", ctypes.c_uint64),
        ("uCustomParamSize", ctypes.c_uint32),
        ("uCustomParam", ctypes.c_uint32),
        ("bIsAutoStream", ctypes.c_ubyte),
        ("bIsCachingStream", ctypes.c_ubyte),
    ]
    uStreamID: int
    deviceID: int
    szStreamName: list[ctypes.c_wchar]
    uStringSize: int
    uFileSize: int
    uCustomParamSize: int
    uCustomParam: int
    bIsAutoStream: bool
    bIsCachingStream: bool

class AkStreamData(ctypes.Structure):
    _fields_ = [
        ("uStreamID", ctypes.c_uint32),
        ("uPriority", ctypes.c_uint32),
        ("uFilePosition", ctypes.c_uint64),
        ("uTargetBufferingSize", ctypes.c_uint32),
        ("uVirtualBufferingSize", ctypes.c_uint32),
        ("uBufferedSize", ctypes.c_uint32),
        ("uNumBytesTransfered", ctypes.c_uint32),
        ("uNumBytesTransferedLowLevel", ctypes.c_uint32),
        ("uMemoryReferenced", ctypes.c_uint32),
        ("fEstimatedThroughput", ctypes.c_float),
        ("bActive", ctypes.c_ubyte),
    ]
    uStreamID: int
    uPriority: int
    uFilePosition: int
    uTargetBufferingSize: int
    uVirtualBufferingSize: int
    uBufferedSize: int
    uNumBytesTransfered: int
    uNumBytesTransferedLowLevel: int
    uMemoryReferenced: int
    fEstimatedThroughput: float
    bActive: bool

class AkFileSystemFlags(ctypes.Structure):
    _fields_ = [
        ("uCompanyID", ctypes.c_uint32),
        ("uCodecID", ctypes.c_uint32),
        ("uCustomParamSize", ctypes.c_uint32),
        ("pCustomParam", ctypes.c_void_p),
        ("bIsLanguageSpecific", ctypes.c_ubyte),
        ("bIsAutomaticStream", ctypes.c_ubyte),
        ("uCacheID", ctypes.c_uint32),
        ("uNumBytesPrefetch", ctypes.c_uint32),
    ]
    uCompanyID: int
    uCodecID: int
    uCustomParamSize: int
    pCustomParam: int
    bIsLanguageSpecific: bool
    bIsAutomaticStream: bool
    uCacheID: int
    uNumBytesPrefetch: int

class AkStreamInfo(ctypes.Structure):
    _fields_ = [
        ("deviceID", ctypes.c_uint32),
        ("pszName", ctypes.c_ulonglong),
        ("uSize", ctypes.c_uint64),
        ("bIsOpen", ctypes.c_ubyte),
    ]
    deviceID: int
    pszName: int
    uSize: int
    bIsOpen: bool

class AkAutoStmHeuristics(ctypes.Structure):
    _fields_ = [
        ("fThroughput", ctypes.c_float),
        ("uLoopStart", ctypes.c_uint32),
        ("uLoopEnd", ctypes.c_uint32),
        ("uMinNumBuffers", ctypes.c_uint8),
        ("priority", ctypes.c_char),
    ]
    fThroughput: float
    uLoopStart: int
    uLoopEnd: int
    uMinNumBuffers: int
    priority: str

class AkAutoStmBufSettings(ctypes.Structure):
    _fields_ = [
        ("uBufferSize", ctypes.c_uint32),
        ("uMinBufferSize", ctypes.c_uint32),
        ("uBlockSize", ctypes.c_uint32),
    ]
    uBufferSize: int
    uMinBufferSize: int
    uBlockSize: int

class AkAcousticTexture(ctypes.Structure):
    _fields_ = [
        ("ID", ctypes.c_uint32),
        ("fAbsorptionOffset", ctypes.c_float),
        ("fAbsorptionLow", ctypes.c_float),
        ("fAbsorptionMidLow", ctypes.c_float),
        ("fAbsorptionMidHigh", ctypes.c_float),
        ("fAbsorptionHigh", ctypes.c_float),
        ("fScattering", ctypes.c_float),
    ]
    ID: int
    fAbsorptionOffset: float
    fAbsorptionLow: float
    fAbsorptionMidLow: float
    fAbsorptionMidHigh: float
    fAbsorptionHigh: float
    fScattering: float

class AkThreadProperties(ctypes.Structure):
    _fields_ = [
        ("nPriority", ctypes.c_int32),
        ("dwAffinityMask", ctypes.c_uint32),
        ("uStackSize", ctypes.c_uint32),
    ]
    nPriority: int
    dwAffinityMask: int
    uStackSize: int

class AkTaskContext(ctypes.Structure):
    _fields_ = [
        ("uIdxThread", ctypes.c_uint32),
    ]
    uIdxThread: int

class AkTaskSchedulerDesc(ctypes.Structure):
    _fields_ = [
        ("fcnParallelFor", ctypes.c_ulonglong),
        ("uNumSchedulerWorkerThreads", ctypes.c_uint32),
    ]
    fcnParallelFor: int
    uNumSchedulerWorkerThreads: int

class AkAudioSettings(ctypes.Structure):
    _fields_ = [
        ("uNumSamplesPerFrame", ctypes.c_uint32),
        ("uNumSamplesPerSecond", ctypes.c_uint32),
    ]
    uNumSamplesPerFrame: int
    uNumSamplesPerSecond: int

class AkCallbackInfo(ctypes.Structure):
    _fields_ = [
        ("pCookie", ctypes.c_void_p),
        ("gameObjID", ctypes.c_uint64),
    ]
    pCookie: int
    gameObjID: int

class AkPolarCoord(ctypes.Structure):
    _fields_ = [
        ("r", ctypes.c_float),
        ("theta", ctypes.c_float),
    ]
    r: float
    theta: float

class AkAudioObjects(ctypes.Structure):
    _fields_ = [
        ("uNumObjects", ctypes.c_uint32),
        ("ppObjectBuffers", ctypes.c_ulonglong),
        ("ppObjects", ctypes.c_ulonglong),
    ]
    uNumObjects: int
    ppObjectBuffers: int
    ppObjects: int

class AkDataReference(ctypes.Structure):
    def tilde_AkDataReference(): pass

    _fields_ = [
        ("pData", ctypes.c_ulonglong),
        ("uSize", ctypes.c_uint32),
        ("uSourceID", ctypes.c_uint32),
        ("pUsageSlot", ctypes.c_ulonglong),
    ]
    pData: int
    uSize: int
    uSourceID: int
    pUsageSlot: int

class AkDataReferenceArray(ctypes.Structure):
    def tilde_AkDataReferenceArray(): pass

    def AcquireData(): pass

    def FindAlternateMedia(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # CAkKeyArray<unsigned int,AkDataReference,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkDataReference> > >
    ]
    baseclass_0: bytes

class CAkEffectContextBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPluginContextBase_dtor_0", ctypes.c_ulonglong),
        ("GlobalContext", ctypes.c_ulonglong),
        ("GetGameObjectInfo", ctypes.c_ulonglong),
        ("GetOutputID", ctypes.c_ulonglong),
        ("GetPluginMedia", ctypes.c_ulonglong),
        ("GetPluginCustomGameData", ctypes.c_ulonglong),
        ("PostMonitorData", ctypes.c_ulonglong),
        ("CanPostMonitorData", ctypes.c_ulonglong),
        ("PostMonitorMessage", ctypes.c_ulonglong),
        ("GetDownstreamGain", ctypes.c_ulonglong),
        ("GetParentChannelConfig", ctypes.c_ulonglong),
        ("GetProcessorFeatures", ctypes.c_ulonglong),
        ("GetAudioNodeID", ctypes.c_ulonglong),
        ("IsSendModeEffect", ctypes.c_ulonglong),
        ("GetVoiceInfo", ctypes.c_ulonglong),
        ("GetMixerCtx", ctypes.c_ulonglong),
        ("CreateOutputObjects", ctypes.c_ulonglong),
        ("GetOutputObjects", ctypes.c_ulonglong),
        ("GetStreamMgr", ctypes.c_ulonglong),
        ("IsRenderingOffline", ctypes.c_ulonglong),
    ]
    IAkPluginContextBase_dtor_0: int
    GlobalContext: int
    GetGameObjectInfo: int
    GetOutputID: int
    GetPluginMedia: int
    GetPluginCustomGameData: int
    PostMonitorData: int
    CanPostMonitorData: int
    PostMonitorMessage: int
    GetDownstreamGain: int
    GetParentChannelConfig: int
    GetProcessorFeatures: int
    GetAudioNodeID: int
    IsSendModeEffect: int
    GetVoiceInfo: int
    GetMixerCtx: int
    CreateOutputObjects: int
    GetOutputObjects: int
    GetStreamMgr: int
    IsRenderingOffline: int

class CAkBusFXContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPluginContextBase_dtor_0", ctypes.c_ulonglong),
        ("GlobalContext", ctypes.c_ulonglong),
        ("GetGameObjectInfo", ctypes.c_ulonglong),
        ("GetOutputID", ctypes.c_ulonglong),
        ("GetPluginMedia", ctypes.c_ulonglong),
        ("GetPluginCustomGameData", ctypes.c_ulonglong),
        ("PostMonitorData", ctypes.c_ulonglong),
        ("CanPostMonitorData", ctypes.c_ulonglong),
        ("PostMonitorMessage", ctypes.c_ulonglong),
        ("GetDownstreamGain", ctypes.c_ulonglong),
        ("GetParentChannelConfig", ctypes.c_ulonglong),
        ("GetProcessorFeatures", ctypes.c_ulonglong),
        ("GetAudioNodeID", ctypes.c_ulonglong),
        ("IsSendModeEffect", ctypes.c_ulonglong),
        ("GetVoiceInfo", ctypes.c_ulonglong),
        ("GetMixerCtx", ctypes.c_ulonglong),
        ("CreateOutputObjects", ctypes.c_ulonglong),
        ("GetOutputObjects", ctypes.c_ulonglong),
        ("GetStreamMgr", ctypes.c_ulonglong),
        ("IsRenderingOffline", ctypes.c_ulonglong),
        ("GetMaxBufferLength", ctypes.c_ulonglong),
        ("_Compute3DPositioning", ctypes.c_ulonglong),
        ("ComputeSpeakerVolumesDirect", ctypes.c_ulonglong),
    ]
    IAkPluginContextBase_dtor_0: int
    GlobalContext: int
    GetGameObjectInfo: int
    GetOutputID: int
    GetPluginMedia: int
    GetPluginCustomGameData: int
    PostMonitorData: int
    CanPostMonitorData: int
    PostMonitorMessage: int
    GetDownstreamGain: int
    GetParentChannelConfig: int
    GetProcessorFeatures: int
    GetAudioNodeID: int
    IsSendModeEffect: int
    GetVoiceInfo: int
    GetMixerCtx: int
    CreateOutputObjects: int
    GetOutputObjects: int
    GetStreamMgr: int
    IsRenderingOffline: int
    GetMaxBufferLength: int
    _Compute3DPositioning: int
    ComputeSpeakerVolumesDirect: int

class AkMeterCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkMetering_dtor_0", ctypes.c_ulonglong),
        ("GetPeak", ctypes.c_ulonglong),
        ("GetTruePeak", ctypes.c_ulonglong),
        ("GetRMS", ctypes.c_ulonglong),
        ("GetKWeightedPower", ctypes.c_ulonglong),
    ]
    IAkMetering_dtor_0: int
    GetPeak: int
    GetTruePeak: int
    GetRMS: int
    GetKWeightedPower: int

class AkMixConnection_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkMixerInputContext_dtor_0", ctypes.c_ulonglong),
        ("GetInputParam", ctypes.c_ulonglong),
        ("GetVoiceInfo", ctypes.c_ulonglong),
        ("GetGameObjectInfo", ctypes.c_ulonglong),
        ("GetConnectionType", ctypes.c_ulonglong),
        ("GetUserData", ctypes.c_ulonglong),
        ("SetUserData", ctypes.c_ulonglong),
        ("GetCenterPerc", ctypes.c_ulonglong),
        ("GetSpeakerPanningType", ctypes.c_ulonglong),
        ("GetPannerPosition", ctypes.c_ulonglong),
        ("HasListenerRelativeRouting", ctypes.c_ulonglong),
        ("Get3DPositionType", ctypes.c_ulonglong),
        ("GetNum3DPositions", ctypes.c_ulonglong),
        ("Get3DPosition", ctypes.c_ulonglong),
        ("GetSpread", ctypes.c_ulonglong),
        ("GetFocus", ctypes.c_ulonglong),
        ("GetMaxAttenuationDistance", ctypes.c_ulonglong),
        ("GetSpatializedVolumes", ctypes.c_ulonglong),
        ("Get3DSpatializationMode", ctypes.c_ulonglong),
    ]
    IAkMixerInputContext_dtor_0: int
    GetInputParam: int
    GetVoiceInfo: int
    GetGameObjectInfo: int
    GetConnectionType: int
    GetUserData: int
    SetUserData: int
    GetCenterPerc: int
    GetSpeakerPanningType: int
    GetPannerPosition: int
    HasListenerRelativeRouting: int
    Get3DPositionType: int
    GetNum3DPositions: int
    Get3DPosition: int
    GetSpread: int
    GetFocus: int
    GetMaxAttenuationDistance: int
    GetSpatializedVolumes: int
    Get3DSpatializationMode: int

class CAkMixConnections(ctypes.Structure):
    def Create(): pass

    def tilde_CAkMixConnections(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # AkListBare<AkMixConnection,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>
    ]
    baseclass_0: bytes

class CAkVPL3dMixable_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkGameObjectPluginInfo_dtor_0", ctypes.c_ulonglong),
        ("GetGameObjectID", ctypes.c_ulonglong),
        ("GetNumEmitterListenerPairs", ctypes.c_ulonglong),
        ("GetEmitterListenerPair", ctypes.c_ulonglong),
        ("GetNumGameObjectPositions", ctypes.c_ulonglong),
        ("GetGameObjectPosition", ctypes.c_ulonglong),
        ("GetGameObjectMultiPositionType", ctypes.c_ulonglong),
        ("GetGameObjectScaling", ctypes.c_ulonglong),
        ("GetListeners", ctypes.c_ulonglong),
        ("GetListenerData", ctypes.c_ulonglong),
        ("_GetVoiceInfo", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
    ]
    IAkGameObjectPluginInfo_dtor_0: int
    GetGameObjectID: int
    GetNumEmitterListenerPairs: int
    GetEmitterListenerPair: int
    GetNumGameObjectPositions: int
    GetGameObjectPosition: int
    GetGameObjectMultiPositionType: int
    GetGameObjectScaling: int
    GetListeners: int
    GetListenerData: int
    _GetVoiceInfo: int
    GetAnalyzedEnvelope: int

class CAkVPLBQFNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int

class CAkInsertFXContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPluginContextBase_dtor_0", ctypes.c_ulonglong),
        ("GlobalContext", ctypes.c_ulonglong),
        ("GetGameObjectInfo", ctypes.c_ulonglong),
        ("GetOutputID", ctypes.c_ulonglong),
        ("GetPluginMedia", ctypes.c_ulonglong),
        ("GetPluginCustomGameData", ctypes.c_ulonglong),
        ("PostMonitorData", ctypes.c_ulonglong),
        ("CanPostMonitorData", ctypes.c_ulonglong),
        ("PostMonitorMessage", ctypes.c_ulonglong),
        ("GetDownstreamGain", ctypes.c_ulonglong),
        ("GetParentChannelConfig", ctypes.c_ulonglong),
        ("GetProcessorFeatures", ctypes.c_ulonglong),
        ("GetAudioNodeID", ctypes.c_ulonglong),
        ("IsSendModeEffect", ctypes.c_ulonglong),
        ("GetVoiceInfo", ctypes.c_ulonglong),
        ("GetMixerCtx", ctypes.c_ulonglong),
        ("CreateOutputObjects", ctypes.c_ulonglong),
        ("GetOutputObjects", ctypes.c_ulonglong),
        ("GetStreamMgr", ctypes.c_ulonglong),
        ("IsRenderingOffline", ctypes.c_ulonglong),
    ]
    IAkPluginContextBase_dtor_0: int
    GlobalContext: int
    GetGameObjectInfo: int
    GetOutputID: int
    GetPluginMedia: int
    GetPluginCustomGameData: int
    PostMonitorData: int
    CanPostMonitorData: int
    PostMonitorMessage: int
    GetDownstreamGain: int
    GetParentChannelConfig: int
    GetProcessorFeatures: int
    GetAudioNodeID: int
    IsSendModeEffect: int
    GetVoiceInfo: int
    GetMixerCtx: int
    CreateOutputObjects: int
    GetOutputObjects: int
    GetStreamMgr: int
    IsRenderingOffline: int

class CAkVPLFilterNodeBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("ReleaseMemory", ctypes.c_ulonglong),
        ("ReleaseInputBuffer", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ConsumeBuffer", ctypes.c_ulonglong),
        ("GetPlugin", ctypes.c_ulonglong),
        ("GetOutputConfig", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Init: int
    Term: int
    ReleaseMemory: int
    ReleaseInputBuffer: int
    GetBuffer: int
    ConsumeBuffer: int
    GetPlugin: int
    GetOutputConfig: int

class CAkVPLVolAutmNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int

class AkInternalPitchState(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x20),  # $E0F124C57CBF6004CB7825ECC2B9649C
        ("___u1", ctypes.c_char * 0x8),  # $9A7FD396017BFFD20F5DF72C2990F907
        ("uInFrameOffset", ctypes.c_uint32),
        ("uOutFrameOffset", ctypes.c_uint32),
        ("uFloatIndex", ctypes.c_uint32),
        ("uCurrentFrameSkip", ctypes.c_uint32),
        ("uTargetFrameSkip", ctypes.c_uint32),
        ("uInterpolationRampCount", ctypes.c_uint32),
        ("uInterpolationRampInc", ctypes.c_uint32),
        ("uRequestedFrames", ctypes.c_uint32),
        ("uChannelMappingStatic", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
        ("uChannelMapping", ctypes.c_ulonglong),
        ("bLastValuesAllocated", ctypes.c_ubyte),
    ]
    ___u0: bytes
    ___u1: bytes
    uInFrameOffset: int
    uOutFrameOffset: int
    uFloatIndex: int
    uCurrentFrameSkip: int
    uTargetFrameSkip: int
    uInterpolationRampCount: int
    uInterpolationRampInc: int
    uRequestedFrames: int
    uChannelMappingStatic: list[ctypes.c_uint8]
    uChannelMapping: int
    bLastValuesAllocated: bool

class CAkVPLPitchNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int

class AkAudioMarker(ctypes.Structure):
    _fields_ = [
        ("dwIdentifier", ctypes.c_uint32),
        ("dwPosition", ctypes.c_uint32),
        ("strLabel", ctypes.c_char_p),
    ]
    dwIdentifier: int
    dwPosition: int
    strLabel: bytes

class CAkVPLSrcCbxNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkGameObjectPluginInfo_dtor_0", ctypes.c_ulonglong),
        ("GetGameObjectID", ctypes.c_ulonglong),
        ("GetNumEmitterListenerPairs", ctypes.c_ulonglong),
        ("GetEmitterListenerPair", ctypes.c_ulonglong),
        ("GetNumGameObjectPositions", ctypes.c_ulonglong),
        ("GetGameObjectPosition", ctypes.c_ulonglong),
        ("GetGameObjectMultiPositionType", ctypes.c_ulonglong),
        ("GetGameObjectScaling", ctypes.c_ulonglong),
        ("GetListeners", ctypes.c_ulonglong),
        ("GetListenerData", ctypes.c_ulonglong),
        ("_GetVoiceInfo", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
    ]
    IAkGameObjectPluginInfo_dtor_0: int
    GetGameObjectID: int
    GetNumEmitterListenerPairs: int
    GetEmitterListenerPair: int
    GetNumGameObjectPositions: int
    GetGameObjectPosition: int
    GetGameObjectMultiPositionType: int
    GetGameObjectScaling: int
    GetListeners: int
    GetListenerData: int
    _GetVoiceInfo: int
    GetAnalyzedEnvelope: int
    RelocateMedia: int

class CAkPBI_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("CAkTransportAware_dtor_8", ctypes.c_ulonglong),
        ("SetEstimatedLength", ctypes.c_ulonglong),
        ("_Pause", ctypes.c_ulonglong),
        ("_Resume", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekTimeAbsolute_38", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("SetPauseStateForContinuous", ctypes.c_ulonglong),
        ("PrepareSampleAccurateTransition", ctypes.c_ulonglong),
        ("NotifyVirtualOff", ctypes.c_ulonglong),
        ("MonitorFade", ctypes.c_ulonglong),
        ("GetAndClearStopOffset", ctypes.c_ulonglong),
    ]
    _Stop: int
    CAkTransportAware_dtor_8: int
    SetEstimatedLength: int
    _Pause: int
    _Resume: int
    PlayToEnd: int
    SeekTimeAbsolute: int
    SeekTimeAbsolute_38: int
    SeekPercent: int
    SetPauseStateForContinuous: int
    PrepareSampleAccurateTransition: int
    NotifyVirtualOff: int
    MonitorFade: int
    GetAndClearStopOffset: int

class CAkVPLSrcNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Seek_60", ctypes.c_ulonglong),
        ("StartStream", ctypes.c_ulonglong),
        ("StopStream", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("IsUsingThisSlot", ctypes.c_ulonglong),
        ("IsUsingThisSlot_98", ctypes.c_ulonglong),
        ("NotifySourceStarvation", ctypes.c_ulonglong),
        ("SupportResampling", ctypes.c_ulonglong),
        ("PrepareSourceSwitch", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("MustRelocatePitchInputBufferOnMediaRelocation", ctypes.c_ulonglong),
        ("MustRelocateAnalysisDataOnMediaRelocation", ctypes.c_ulonglong),
        ("ChangeSourcePosition", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("SrcProcessOrder", ctypes.c_ulonglong),
        ("RestartSourcePlugin", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Term: int
    Seek_60: int
    StartStream: int
    StopStream: int
    GetBuffer: int
    GetDuration: int
    StopLooping: int
    IsUsingThisSlot: int
    IsUsingThisSlot_98: int
    NotifySourceStarvation: int
    SupportResampling: int
    PrepareSourceSwitch: int
    SupportMediaRelocation: int
    RelocateMedia: int
    MustRelocatePitchInputBufferOnMediaRelocation: int
    MustRelocateAnalysisDataOnMediaRelocation: int
    ChangeSourcePosition: int
    GetAnalyzedEnvelope: int
    SrcProcessOrder: int
    RestartSourcePlugin: int
    PrepareNextBuffer: int

class IAkSoftwareCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Seek_60", ctypes.c_ulonglong),
        ("StartStream", ctypes.c_ulonglong),
        ("StopStream", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("IsUsingThisSlot", ctypes.c_ulonglong),
        ("IsUsingThisSlot_98", ctypes.c_ulonglong),
        ("NotifySourceStarvation", ctypes.c_ulonglong),
        ("SupportResampling", ctypes.c_ulonglong),
        ("PrepareSourceSwitch", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("MustRelocatePitchInputBufferOnMediaRelocation", ctypes.c_ulonglong),
        ("MustRelocateAnalysisDataOnMediaRelocation", ctypes.c_ulonglong),
        ("ChangeSourcePosition", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("SrcProcessOrder", ctypes.c_ulonglong),
        ("RestartSourcePlugin", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Term: int
    Seek_60: int
    StartStream: int
    StopStream: int
    GetBuffer: int
    GetDuration: int
    StopLooping: int
    IsUsingThisSlot: int
    IsUsingThisSlot_98: int
    NotifySourceStarvation: int
    SupportResampling: int
    PrepareSourceSwitch: int
    SupportMediaRelocation: int
    RelocateMedia: int
    MustRelocatePitchInputBufferOnMediaRelocation: int
    MustRelocateAnalysisDataOnMediaRelocation: int
    ChangeSourcePosition: int
    GetAnalyzedEnvelope: int
    SrcProcessOrder: int
    RestartSourcePlugin: int
    PrepareNextBuffer: int

class IAkFileCodec(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IAkFileCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkFileCodec_dtor_0", ctypes.c_ulonglong),
        ("DecodeFile", ctypes.c_ulonglong),
    ]
    IAkFileCodec_dtor_0: int
    DecodeFile: int

class IAkGrainCodec(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IAkGrainCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkGrainCodec_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("SeekTo", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
    ]
    IAkGrainCodec_dtor_0: int
    Init: int
    GetBuffer: int
    ReleaseBuffer: int
    SeekTo: int
    Term: int

class AkCodecDescriptor(ctypes.Structure):
    _fields_ = [
        ("pFileSrcCreateFunc", ctypes.c_ulonglong),
        ("pBankSrcCreateFunc", ctypes.c_ulonglong),
        ("pFileCodecCreateFunc", ctypes.c_ulonglong),
        ("pGrainCodecCreateFunc", ctypes.c_ulonglong),
    ]
    pFileSrcCreateFunc: int
    pBankSrcCreateFunc: int
    pFileCodecCreateFunc: int
    pGrainCodecCreateFunc: int

class AkFileDesc(ctypes.Structure):
    _fields_ = [
        ("iFileSize", ctypes.c_int64),
        ("uSector", ctypes.c_uint64),
        ("uCustomParamSize", ctypes.c_uint32),
        ("pCustomParam", ctypes.c_void_p),
        ("hFile", ctypes.c_void_p),
        ("deviceID", ctypes.c_uint32),
        ("pPackage", ctypes.c_ulonglong),
    ]
    iFileSize: int
    uSector: int
    uCustomParamSize: int
    pCustomParam: int
    hFile: int
    deviceID: int
    pPackage: int

class AkIoHeuristics(ctypes.Structure):
    _fields_ = [
        ("fDeadline", ctypes.c_float),
        ("priority", ctypes.c_char),
    ]
    fDeadline: float
    priority: str

class AkIOTransferInfo(ctypes.Structure):
    _fields_ = [
        ("uFilePosition", ctypes.c_uint64),
        ("uBufferSize", ctypes.c_uint32),
        ("uRequestedSize", ctypes.c_uint32),
    ]
    uFilePosition: int
    uBufferSize: int
    uRequestedSize: int

class cTkAudioWwiseIO_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkFileLocationResolver_dtor_0", ctypes.c_ulonglong),
        ("Open", ctypes.c_ulonglong),
        ("Open_10", ctypes.c_ulonglong),
    ]
    IAkFileLocationResolver_dtor_0: int
    Open: int
    Open_10: int

class TkConstexprMath(ctypes.Structure):
    pass

class HgVAMIDIFileReader(ctypes.Structure):
    def extractVLV(): pass

    def getUInt16(): pass

    def getUInt32(): pass

    _fields_ = [
        ("mpcMIDIBuffer", ctypes.c_ulonglong),
        ("muMIDIBufferSize", ctypes.c_uint32),
        ("muMIDIReaderIndex", ctypes.c_uint32),
        ("mbBufferOverrun", ctypes.c_ubyte),
    ]
    mpcMIDIBuffer: int
    muMIDIBufferSize: int
    muMIDIReaderIndex: int
    mbBufferOverrun: bool

class HgVaMorphTargets(ctypes.Structure):
    _fields_ = [
        ("mpcTargetCCArray", ctypes.c_uint8 * 0x40),  # unsigned __int8[64][128]
    ]
    mpcTargetCCArray: list[ctypes.c_uint8]

class HgVaMidiLocatorTag(ctypes.Structure):
    _fields_ = [
        ("muLocation", ctypes.c_uint32),
        ("muNumNotes", ctypes.c_uint32),
    ]
    muLocation: int
    muNumNotes: int

class sMapParameters(ctypes.Structure):
    _fields_ = [
        ("mSeed", common.GcSeed),
        ("mSize", ctypes.c_char * 0xC),  # cTkIntTuple3<int,0>
        ("mpLayout", ctypes.c_ulonglong),
        ("mbSymmetric", ctypes.c_ubyte),
    ]
    mSeed: common.GcSeed
    mSize: bytes
    mpLayout: int
    mbSymmetric: bool

class AkToneGenStaticParamBlock(ctypes.Structure):
    _fields_ = [
        ("mfGain", ctypes.c_float),
        ("mfAttackDur", ctypes.c_float),
        ("mfDecayDur", ctypes.c_float),
        ("mfSustainDur", ctypes.c_float),
        ("mfReleaseDur", ctypes.c_float),
        ("mfSustainVal", ctypes.c_float),
        ("mfEnvelopeLengthMin", ctypes.c_float),
        ("mfMetaSizeMin", ctypes.c_float),
        ("mfMetaSizeMax", ctypes.c_float),
        ("mfMetaRatioMin", ctypes.c_float),
        ("mfMetaRatioMax", ctypes.c_float),
        ("mfEmoteJitter", ctypes.c_float),
        ("mfMIDISpeedMin", ctypes.c_float),
        ("mfMIDISpeedMax", ctypes.c_float),
        ("mfPerlinVowelHz", ctypes.c_float),
        ("mfPerlinVowelDepth", ctypes.c_float),
        ("mfPerlinPitchHz", ctypes.c_float),
        ("mfPerlinPitchDepth", ctypes.c_float),
        ("mfPerlinVariance", ctypes.c_float),
        ("mfTargetVariation1", ctypes.c_float),
        ("mfTargetVariation2", ctypes.c_float),
        ("mfTargetVariation3", ctypes.c_float),
        ("mfTargetVariation4", ctypes.c_float),
        ("mfTargetVariation5", ctypes.c_float),
        ("mfDistance", ctypes.c_float),
        ("mfShuffle", ctypes.c_float),
        ("mfGP1", ctypes.c_float),
        ("mfGameObjectID", ctypes.c_float),
        ("mfEnvelopeLengthMax", ctypes.c_float),
        ("mfVarSeed", ctypes.c_float),
        ("mfVarSparse", ctypes.c_float),
        ("muNumTargetsMin", ctypes.c_uint32),
        ("muNumTargetsMax", ctypes.c_uint32),
        ("muTargetBaseMax", ctypes.c_uint32),
        ("muTargetBaseMin", ctypes.c_uint32),
        ("mbPlaybackOn", ctypes.c_ubyte),
        ("mbOne_Shot", ctypes.c_ubyte),
        ("mbEconomy", ctypes.c_ubyte),
        ("mbSync_Morph", ctypes.c_ubyte),
        ("mbEmote_Scaled", ctypes.c_ubyte),
        ("mbVariation_On", ctypes.c_ubyte),
        ("mbShuffle_All", ctypes.c_ubyte),
        ("mfMIDISpeed", ctypes.c_float),
        ("muTargetBase", ctypes.c_uint32),
        ("muNumTargets", ctypes.c_uint32),
        ("mfEnvelopeLength", ctypes.c_float),
        ("mfVarRandNormal", ctypes.c_float),
        ("mfVarSeedNormal", ctypes.c_float),
        ("mfVarSparseNormal", ctypes.c_float),
    ]
    mfGain: float
    mfAttackDur: float
    mfDecayDur: float
    mfSustainDur: float
    mfReleaseDur: float
    mfSustainVal: float
    mfEnvelopeLengthMin: float
    mfMetaSizeMin: float
    mfMetaSizeMax: float
    mfMetaRatioMin: float
    mfMetaRatioMax: float
    mfEmoteJitter: float
    mfMIDISpeedMin: float
    mfMIDISpeedMax: float
    mfPerlinVowelHz: float
    mfPerlinVowelDepth: float
    mfPerlinPitchHz: float
    mfPerlinPitchDepth: float
    mfPerlinVariance: float
    mfTargetVariation1: float
    mfTargetVariation2: float
    mfTargetVariation3: float
    mfTargetVariation4: float
    mfTargetVariation5: float
    mfDistance: float
    mfShuffle: float
    mfGP1: float
    mfGameObjectID: float
    mfEnvelopeLengthMax: float
    mfVarSeed: float
    mfVarSparse: float
    muNumTargetsMin: int
    muNumTargetsMax: int
    muTargetBaseMax: int
    muTargetBaseMin: int
    mbPlaybackOn: bool
    mbOne_Shot: bool
    mbEconomy: bool
    mbSync_Morph: bool
    mbEmote_Scaled: bool
    mbVariation_On: bool
    mbShuffle_All: bool
    mfMIDISpeed: float
    muTargetBase: int
    muNumTargets: int
    mfEnvelopeLength: float
    mfVarRandNormal: float
    mfVarSeedNormal: float
    mfVarSparseNormal: float

class HgVocAlienParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class HgVocAlien_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("GetEnvelope", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    GetDuration: int
    GetEnvelope: int
    StopLooping: int
    Seek: int
    TimeSkip: int
    Execute: int

class SnapHandle(ctypes.Structure):
    _fields_ = [
        ("muSnapPointIndex", ctypes.c_uint8),
        ("muSnapSocketIndex", ctypes.c_uint8),
    ]
    muSnapPointIndex: int
    muSnapSocketIndex: int

class VkSpecializationMapEntry(ctypes.Structure):
    _fields_ = [
        ("constantID", ctypes.c_uint32),
        ("offset", ctypes.c_uint32),
        ("size", ctypes.c_uint64),
    ]
    constantID: int
    offset: int
    size: int

class VkSpecializationInfo(ctypes.Structure):
    _fields_ = [
        ("mapEntryCount", ctypes.c_uint32),
        ("pMapEntries", ctypes.c_ulonglong),
        ("dataSize", ctypes.c_uint64),
        ("pData", ctypes.c_void_p),
    ]
    mapEntryCount: int
    pMapEntries: int
    dataSize: int
    pData: int

class VkDescriptorBufferInfo(ctypes.Structure):
    _fields_ = [
        ("buffer", ctypes.c_ulonglong),
        ("offset", ctypes.c_uint64),
        ("range", ctypes.c_uint64),
    ]
    buffer: int
    offset: int
    range: int

class cTkCameraEngineInterface(ctypes.Structure):
    pass

class cTkCameraSpline(ctypes.Structure):
    _fields_ = [
        ("mpaNodes", ctypes.c_ulonglong),
        ("mfMaxDistance", ctypes.c_float),
        ("miNumNodes", ctypes.c_int32),
    ]
    mpaNodes: int
    mfMaxDistance: float
    miNumNodes: int

class cGcDebugTextLine(ctypes.Structure):
    def Format(): pass


class cTkIKChain(ctypes.Structure):
    def Solve(): pass

    _fields_ = [
        ("maBones", ctypes.c_uint32 * 0x4),  # TkHandle[4]
        ("miNumBones", ctypes.c_int32),
        ("maRotateAxis", common.Vector3f * 0x4),  # cTkVector3[4]
    ]
    maBones: list[ctypes.c_uint32]
    miNumBones: int
    maRotateAxis: list[common.Vector3f]

class AkMemSettings(ctypes.Structure):
    _fields_ = [
        ("pfInitForThread", ctypes.c_ulonglong),
        ("pfTermForThread", ctypes.c_ulonglong),
        ("pfMalloc", ctypes.c_ulonglong),
        ("pfMalign", ctypes.c_ulonglong),
        ("pfRealloc", ctypes.c_ulonglong),
        ("pfReallocAligned", ctypes.c_ulonglong),
        ("pfFree", ctypes.c_ulonglong),
        ("pfTotalReservedMemorySize", ctypes.c_ulonglong),
        ("pfSizeOfMemory", ctypes.c_ulonglong),
        ("uMemAllocationSizeLimit", ctypes.c_uint64),
        ("pfAllocVM", ctypes.c_ulonglong),
        ("pfFreeVM", ctypes.c_ulonglong),
        ("pfAllocDevice", ctypes.c_ulonglong),
        ("pfFreeDevice", ctypes.c_ulonglong),
        ("uVMPageSize", ctypes.c_uint32),
        ("uDevicePageSize", ctypes.c_uint32),
        ("pfDebugMalloc", ctypes.c_ulonglong),
        ("pfDebugMalign", ctypes.c_ulonglong),
        ("pfDebugRealloc", ctypes.c_ulonglong),
        ("pfDebugReallocAligned", ctypes.c_ulonglong),
        ("pfDebugFree", ctypes.c_ulonglong),
        ("uMemoryDebugLevel", ctypes.c_uint32),
    ]
    pfInitForThread: int
    pfTermForThread: int
    pfMalloc: int
    pfMalign: int
    pfRealloc: int
    pfReallocAligned: int
    pfFree: int
    pfTotalReservedMemorySize: int
    pfSizeOfMemory: int
    uMemAllocationSizeLimit: int
    pfAllocVM: int
    pfFreeVM: int
    pfAllocDevice: int
    pfFreeDevice: int
    uVMPageSize: int
    uDevicePageSize: int
    pfDebugMalloc: int
    pfDebugMalign: int
    pfDebugRealloc: int
    pfDebugReallocAligned: int
    pfDebugFree: int
    uMemoryDebugLevel: int

class AkStreamMgrSettings(ctypes.Structure):
    pass

class AkMusicSettings(ctypes.Structure):
    _fields_ = [
        ("fStreamingLookAheadRatio", ctypes.c_float),
    ]
    fStreamingLookAheadRatio: float

class cMatrixBuilder(ctypes.Structure):
    _fields_ = [
        ("mafATA", ctypes.c_ulonglong),
        ("mafATb", ctypes.c_ulonglong),
        ("miStateSize", ctypes.c_int32),
        ("miStateSizeAlignedUp", ctypes.c_int32),
    ]
    mafATA: int
    mafATb: int
    miStateSize: int
    miStateSizeAlignedUp: int

class cTkIKRotModifyConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cTkIKCenterOfGravityHeightConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cTkIKMomentumConstraint_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreAdditive", ctypes.c_ulonglong),
        ("PostAnim", ctypes.c_ulonglong),
        ("PreSolve", ctypes.c_ulonglong),
        ("GetNumRows", ctypes.c_ulonglong),
        ("BuildRows", ctypes.c_ulonglong),
        ("PostSolve", ctypes.c_ulonglong),
    ]
    OnAttach: int
    OnDetach: int
    PreAdditive: int
    PostAnim: int
    PreSolve: int
    GetNumRows: int
    BuildRows: int
    PostSolve: int

class cGcAudioPulseXmlHelper(ctypes.Structure):
    def Open(): pass

    _fields_ = [
        ("mpcData", ctypes.c_char_p),
    ]
    mpcData: bytes

class AkFileEntry(ctypes.Structure):
    _fields_ = [
        ("fileID", ctypes.c_uint32),
        ("uBlockSize", ctypes.c_uint32),
        ("uFileSize", ctypes.c_uint32),
        ("uStartBlock", ctypes.c_uint32),
        ("uLanguageID", ctypes.c_uint32),
    ]
    fileID: int
    uBlockSize: int
    uFileSize: int
    uStartBlock: int
    uLanguageID: int

class cGcNetworkFullSimUpdateMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class tagBIND_OPTS(ctypes.Structure):
    _fields_ = [
        ("cbStruct", ctypes.c_uint32),
        ("grfFlags", ctypes.c_uint32),
        ("grfMode", ctypes.c_uint32),
        ("dwTickCountDeadline", ctypes.c_uint32),
    ]
    cbStruct: int
    grfFlags: int
    grfMode: int
    dwTickCountDeadline: int

class _SHITEMID(ctypes.Structure):
    _fields_ = [
        ("cb", ctypes.c_uint16),
        ("abID", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
    ]
    cb: int
    abID: list[ctypes.c_uint8]

class sComponentEvent(ctypes.Structure):
    def sComponentEvent(): pass

    def Call(): pass

    _fields_ = [
        ("muComponentHash", ctypes.c_uint64),
        ("macEventName", ctypes.c_char_p),
        ("mpNext", ctypes.c_ulonglong),
    ]
    muComponentHash: int
    macEventName: bytes
    mpNext: int

class IPersist_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetClassID", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetClassID: int

class IPersistStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetClassID", ctypes.c_ulonglong),
        ("IsDirty", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("GetSizeMax", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetClassID: int
    IsDirty: int
    Load: int
    Save: int
    GetSizeMax: int

class IEnumMoniker_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Next", ctypes.c_ulonglong),
        ("Skip", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Next: int
    Skip: int
    Reset: int
    Clone: int

class IMoniker_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetClassID", ctypes.c_ulonglong),
        ("IsDirty", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("GetSizeMax", ctypes.c_ulonglong),
        ("BindToObject", ctypes.c_ulonglong),
        ("BindToStorage", ctypes.c_ulonglong),
        ("Reduce", ctypes.c_ulonglong),
        ("ComposeWith", ctypes.c_ulonglong),
        ("Enum", ctypes.c_ulonglong),
        ("IsEqual", ctypes.c_ulonglong),
        ("Hash", ctypes.c_ulonglong),
        ("IsRunning", ctypes.c_ulonglong),
        ("GetTimeOfLastChange", ctypes.c_ulonglong),
        ("Inverse", ctypes.c_ulonglong),
        ("CommonPrefixWith", ctypes.c_ulonglong),
        ("RelativePathTo", ctypes.c_ulonglong),
        ("GetDisplayName", ctypes.c_ulonglong),
        ("ParseDisplayName", ctypes.c_ulonglong),
        ("IsSystemMoniker", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetClassID: int
    IsDirty: int
    Load: int
    Save: int
    GetSizeMax: int
    BindToObject: int
    BindToStorage: int
    Reduce: int
    ComposeWith: int
    Enum: int
    IsEqual: int
    Hash: int
    IsRunning: int
    GetTimeOfLastChange: int
    Inverse: int
    CommonPrefixWith: int
    RelativePathTo: int
    GetDisplayName: int
    ParseDisplayName: int
    IsSystemMoniker: int

class IRunningObjectTable_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Register", ctypes.c_ulonglong),
        ("Revoke", ctypes.c_ulonglong),
        ("IsRunning", ctypes.c_ulonglong),
        ("GetObjectA", ctypes.c_ulonglong),
        ("NoteChangeTime", ctypes.c_ulonglong),
        ("GetTimeOfLastChange", ctypes.c_ulonglong),
        ("EnumRunning", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Register: int
    Revoke: int
    IsRunning: int
    GetObjectA: int
    NoteChangeTime: int
    GetTimeOfLastChange: int
    EnumRunning: int

class IEnumString_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Next", ctypes.c_ulonglong),
        ("Skip", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Next: int
    Skip: int
    Reset: int
    Clone: int

class IBindCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("RegisterObjectBound", ctypes.c_ulonglong),
        ("RevokeObjectBound", ctypes.c_ulonglong),
        ("ReleaseBoundObjects", ctypes.c_ulonglong),
        ("SetBindOptions", ctypes.c_ulonglong),
        ("GetBindOptions", ctypes.c_ulonglong),
        ("GetRunningObjectTable", ctypes.c_ulonglong),
        ("RegisterObjectParam", ctypes.c_ulonglong),
        ("GetObjectParam", ctypes.c_ulonglong),
        ("EnumObjectParam", ctypes.c_ulonglong),
        ("RevokeObjectParam", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    RegisterObjectBound: int
    RevokeObjectBound: int
    ReleaseBoundObjects: int
    SetBindOptions: int
    GetBindOptions: int
    GetRunningObjectTable: int
    RegisterObjectParam: int
    GetObjectParam: int
    EnumObjectParam: int
    RevokeObjectParam: int

class IShellItem_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("BindToHandler", ctypes.c_ulonglong),
        ("GetParent", ctypes.c_ulonglong),
        ("GetDisplayName", ctypes.c_ulonglong),
        ("GetAttributes", ctypes.c_ulonglong),
        ("Compare", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    BindToHandler: int
    GetParent: int
    GetDisplayName: int
    GetAttributes: int
    Compare: int

class IShellLibrary_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("LoadLibraryFromItem", ctypes.c_ulonglong),
        ("LoadLibraryFromKnownFolder", ctypes.c_ulonglong),
        ("AddFolder", ctypes.c_ulonglong),
        ("RemoveFolder", ctypes.c_ulonglong),
        ("GetFolders", ctypes.c_ulonglong),
        ("ResolveFolder", ctypes.c_ulonglong),
        ("GetDefaultSaveFolder", ctypes.c_ulonglong),
        ("SetDefaultSaveFolder", ctypes.c_ulonglong),
        ("GetOptions", ctypes.c_ulonglong),
        ("SetOptions", ctypes.c_ulonglong),
        ("GetFolderType", ctypes.c_ulonglong),
        ("SetFolderType", ctypes.c_ulonglong),
        ("GetIcon", ctypes.c_ulonglong),
        ("SetIcon", ctypes.c_ulonglong),
        ("Commit", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("SaveInKnownFolder", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    LoadLibraryFromItem: int
    LoadLibraryFromKnownFolder: int
    AddFolder: int
    RemoveFolder: int
    GetFolders: int
    ResolveFolder: int
    GetDefaultSaveFolder: int
    SetDefaultSaveFolder: int
    GetOptions: int
    SetOptions: int
    GetFolderType: int
    SetFolderType: int
    GetIcon: int
    SetIcon: int
    Commit: int
    Save: int
    SaveInKnownFolder: int

class cGcNetworkComponentMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cTkSketchWindow(ctypes.Structure):
    def CheckStringFilter(): pass

    def DoNodeAndChildren(): pass

    def ExtractComponentNameOnly(): pass

    def FindNodeIndexForFilter(): pass

    def FindSketchComponentForModel(): pass

    def FindSketchComponentForSelectedNode(): pass

    def OpenWindow(): pass

    def RefreshSelectedNode(): pass

    def RegisterSketchWindows(): pass

    def RenderEditorWindow(): pass

    def RenderHierarchyWindow(): pass

    def RenderVariablesWindow(): pass

    def SetSelectedNode(): pass


class ColourPresetHandler(ctypes.Structure):
    _fields_ = [
        ("maColours", ctypes.c_char * 0x18),  # std::vector<cTkColour *,TkSTLAllocatorShim<cTkColour *,8,-1> >
    ]
    maColours: bytes

class cGcGuiTypeUndoableLayerData(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def Compare(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x740),  # cTkNGuiTypeUndoable<cGcNGuiLayerData>
    ]
    baseclass_0: bytes

class cGcGuiTypeUndoableLayerData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkNGuiUndoable_dtor_0", ctypes.c_ulonglong),
        ("Undo", ctypes.c_ulonglong),
        ("Do", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("GetData", ctypes.c_ulonglong),
        ("ToString", ctypes.c_ulonglong),
        ("Icon", ctypes.c_ulonglong),
        ("Compare", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkNGuiTypeUndoable<cGcNGuiLayerData> *this, const cGcNGuiLayerData *, const cGcNGuiLayerData *)
    ]
    cTkNGuiUndoable_dtor_0: int
    Undo: int
    Do: int
    End: int
    GetData: int
    ToString: int
    Icon: int
    Compare: bytes

class cGcUndoableNGuiReplaceOperation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkNGuiUndoable_dtor_0", ctypes.c_ulonglong),
        ("Undo", ctypes.c_ulonglong),
        ("Do", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("GetData", ctypes.c_ulonglong),
        ("ToString", ctypes.c_ulonglong),
        ("Icon", ctypes.c_ulonglong),
    ]
    cTkNGuiUndoable_dtor_0: int
    Undo: int
    Do: int
    End: int
    GetData: int
    ToString: int
    Icon: int

class PartUserData: pass

class TemplateHandle(ctypes.Structure):
    _fields_ = [
        ("muIndex", ctypes.c_uint32),
    ]
    muIndex: int

class sBaseObjectBufferData(ctypes.Structure):
    def ReadBufferData(): pass

    def EraseBufferData(): pass

    _fields_ = [
        ("mID", ctypes.c_ulonglong),
        ("mLocalUp", common.Vector3f),
        ("mLocalAt", common.Vector3f),
        ("mLocalPosition", common.Vector3f),
        ("mUserData", ctypes.c_uint64),
        ("mUA", ctypes.c_uint64),
        ("mRegionID", ctypes.c_uint64),
        ("mTimestamp", ctypes.c_uint64),
    ]
    mID: int
    mLocalUp: common.Vector3f
    mLocalAt: common.Vector3f
    mLocalPosition: common.Vector3f
    mUserData: int
    mUA: int
    mRegionID: int
    mTimestamp: int

class BaseBuildingTemplateCache(ctypes.Structure):
    def tilde_BaseBuildingTemplateCache(): pass

    _fields_ = [
        ("mSnapPointTemplates", ctypes.c_char * 0x18),  # std::vector<SnapPointTemplate,TkSTLAllocatorShim<SnapPointTemplate,16,-1> >
        ("mSnapSocketTemplates", ctypes.c_char * 0x18),  # std::vector<SnapSocketTemplate,TkSTLAllocatorShim<SnapSocketTemplate,16,-1> >
        ("mPlacementRules", ctypes.c_char * 0x18),  # std::vector<ModelPlacementRule,TkSTLAllocatorShim<ModelPlacementRule,16,-1> >
    ]
    mSnapPointTemplates: bytes
    mSnapSocketTemplates: bytes
    mPlacementRules: bytes

class NVDX_SwapChainHandle__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class cTkVulkanExtensionInfo(ctypes.Structure):
    _fields_ = [
        ("mbEnabled", ctypes.c_ubyte),
        ("macName", ctypes.c_char_p),
    ]
    mbEnabled: bool
    macName: bytes

class VkPushConstantRange(ctypes.Structure):
    _fields_ = [
        ("stageFlags", ctypes.c_uint32),
        ("offset", ctypes.c_uint32),
        ("size", ctypes.c_uint32),
    ]
    stageFlags: int
    offset: int
    size: int

class NameCache(ctypes.Structure):
    _fields_ = [
        ("mNameHashes", ctypes.c_char * 0x18),  # std::vector<unsigned int,TkSTLAllocatorShim<unsigned int,4,-1> >
        ("mFullNames", ctypes.c_char * 0x18),  # std::vector<cTkFixedString<128,char>,TkSTLAllocatorShim<cTkFixedString<128,char>,1,-1> >
    ]
    mNameHashes: bytes
    mFullNames: bytes

class VkSubresourceLayout(ctypes.Structure):
    _fields_ = [
        ("offset", ctypes.c_uint64),
        ("size", ctypes.c_uint64),
        ("rowPitch", ctypes.c_uint64),
        ("arrayPitch", ctypes.c_uint64),
        ("depthPitch", ctypes.c_uint64),
    ]
    offset: int
    size: int
    rowPitch: int
    arrayPitch: int
    depthPitch: int

class cTkEntitlementManager(ctypes.Structure):
    def cTkEntitlementManager(): pass

    def GetEntitlementId(): pass

    def GetEntitlementServiceId(): pass

    def HasEntitlement(): pass

    def Prepare(): pass

    _fields_ = [
        ("mpEntitlementList", ctypes.c_ulonglong),
        ("mResultCache", ctypes.c_char * 0x40),  # std::unordered_map<TkID<128>,bool,TkIDUnorderedMap::Hash128,std::equal_to<TkID<128> >,TkSTLAllocatorShim<std::pair<TkID<128> const ,bool>,8,-1> >
    ]
    mpEntitlementList: int
    mResultCache: bytes

class NVSDK_NGX_D3D11_Feature_Eval_Params(ctypes.Structure):
    _fields_ = [
        ("pInColor", ctypes.c_ulonglong),
        ("pInOutput", ctypes.c_ulonglong),
        ("InSharpness", ctypes.c_float),
    ]
    pInColor: int
    pInOutput: int
    InSharpness: float

class NVSDK_NGX_Dimensions(ctypes.Structure):
    _fields_ = [
        ("Width", ctypes.c_uint32),
        ("Height", ctypes.c_uint32),
    ]
    Width: int
    Height: int

class NVSDK_NGX_Coordinates(ctypes.Structure):
    _fields_ = [
        ("X", ctypes.c_uint32),
        ("Y", ctypes.c_uint32),
    ]
    X: int
    Y: int

class NVSDK_NGX_D3D11_GBuffer(ctypes.Structure):
    _fields_ = [
        ("pInAttrib", ctypes.c_ulonglong * 0x10),  # struct ID3D11Resource *[16]
    ]
    pInAttrib: list[ctypes.c_ulonglong]

class NVSDK_NGX_CUDA_Feature_Eval_Params(ctypes.Structure):
    _fields_ = [
        ("pInColor", ctypes.c_ulonglong),
        ("pInOutput", ctypes.c_ulonglong),
        ("InSharpness", ctypes.c_float),
    ]
    pInColor: int
    pInOutput: int
    InSharpness: float

class NVSDK_NGX_D3D12_Feature_Eval_Params(ctypes.Structure):
    _fields_ = [
        ("pInColor", ctypes.c_ulonglong),
        ("pInOutput", ctypes.c_ulonglong),
        ("InSharpness", ctypes.c_float),
    ]
    pInColor: int
    pInOutput: int
    InSharpness: float

class NVSDK_NGX_D3D12_GBuffer(ctypes.Structure):
    _fields_ = [
        ("pInAttrib", ctypes.c_ulonglong * 0x10),  # struct ID3D12Resource *[16]
    ]
    pInAttrib: list[ctypes.c_ulonglong]

class cTkDynamicResScaling(ctypes.Structure):
    def ApplyDynamicResScale(): pass

    def ForceDynamicResolutionIndexThisFrame(): pass

    def GetCurrentDynamicResScale(): pass

    def GetDynamicScaleForResolution(): pass

    def GetLowestPossibleSupportedScalingFactorForGraphicsSettings(): pass

    def GetSensibleDefaultForGraphicsSettings(): pass


class cGcCreatureMovementFlock_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCreatureMovementBase_dtor_0", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("GetExtraPartNode", ctypes.c_ulonglong),
        ("HasDeathAction", ctypes.c_ulonglong),
        ("TriggerDeathAction", ctypes.c_ulonglong),
        ("IsCrystal", ctypes.c_ulonglong),
        ("NotifyOfPredator", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("SetSoundState", ctypes.c_ulonglong),
    ]
    cGcCreatureMovementBase_dtor_0: int
    Release: int
    UpdatePostPhysics: int
    GetExtraPartNode: int
    HasDeathAction: int
    TriggerDeathAction: int
    IsCrystal: int
    NotifyOfPredator: int
    RenderNGui: int
    SetSoundState: int

class sGcNetworkReplicatedPetEntitiesWelcomeMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class sGcNetworkReplicatedEntitiesUpdateMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class sGcNetworkReplicatedCustomisedEntitiesWelcomeMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class sGcNetworkReplicatedEntityDestroyedMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcCreatureMovementCrystal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCreatureMovementBase_dtor_0", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("GetExtraPartNode", ctypes.c_ulonglong),
        ("HasDeathAction", ctypes.c_ulonglong),
        ("TriggerDeathAction", ctypes.c_ulonglong),
        ("IsCrystal", ctypes.c_ulonglong),
        ("NotifyOfPredator", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("SetSoundState", ctypes.c_ulonglong),
    ]
    cGcCreatureMovementBase_dtor_0: int
    Release: int
    UpdatePostPhysics: int
    GetExtraPartNode: int
    HasDeathAction: int
    TriggerDeathAction: int
    IsCrystal: int
    NotifyOfPredator: int
    RenderNGui: int
    SetSoundState: int

class cTkShaderMillMetadata(ctypes.Structure):
    _fields_ = [
        ("mVertexSourcePath", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mVertexDissassemblyPath", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mFragmentSourcePath", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mFragmentDissassemblyPath", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
    ]
    mVertexSourcePath: bytes
    mVertexDissassemblyPath: bytes
    mFragmentSourcePath: bytes
    mFragmentDissassemblyPath: bytes

class cTkUgcManagerBase(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def DeleteCachedUGCDetails(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mUgcDetails", ctypes.c_char * 0x18),  # std::vector<UserGeneratedContentDetails>
    ]
    __vftable: int
    mUgcDetails: bytes

class iTkUgcUploadResult(ctypes.Structure):
    pass

class cTkUgcManagerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkUgcManagerBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UploadUserGeneratedContent", ctypes.c_ulonglong),
        ("GetDetailsProgress", ctypes.c_ulonglong),
        ("DeleteUploadedUserGeneratedContent", ctypes.c_ulonglong),
    ]
    cTkUgcManagerBase_dtor_0: int
    Construct: int
    Destruct: int
    UploadUserGeneratedContent: int
    GetDetailsProgress: int
    DeleteUploadedUserGeneratedContent: int

class cTkUgcManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkUgcManagerBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UploadUserGeneratedContent", ctypes.c_ulonglong),
        ("GetDetailsProgress", ctypes.c_ulonglong),
        ("DeleteUploadedUserGeneratedContent", ctypes.c_ulonglong),
    ]
    cTkUgcManagerBase_dtor_0: int
    Construct: int
    Destruct: int
    UploadUserGeneratedContent: int
    GetDetailsProgress: int
    DeleteUploadedUserGeneratedContent: int

class cTkSimpleLoadingProfilerScopedBlock(ctypes.Structure):
    _fields_ = [
        ("mProfileName", ctypes.c_char * 0x80),  # cTkFixedString<128,char>
    ]
    mProfileName: bytes

class cTkGravityComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkLODComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkProceduralModelComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkFileProfilerScopedBlock(ctypes.Structure):
    _fields_ = [
        ("mfStartTime", ctypes.c_double),
        ("_meOp", ctypes.c_uint32),
        ("mFileName", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mInfo", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("miSize", ctypes.c_int64),
    ]
    mfStartTime: float
    _meOp: int
    mFileName: bytes
    mInfo: bytes
    miSize: int

    @property
    def meOp(self):
        return cTkFileProfiler.FileOperation(self._meOp)


class cEgShaderUniformBufferBinding(ctypes.Structure):
    _fields_ = [
        ("mpShaderCombination", ctypes.c_ulonglong),
        ("mpShaderBufferHandles", ctypes.c_char * 0x8),  # std::vector<TkShaderBufferHandle,TkSTLAllocatorShim<TkShaderBufferHandle,2,-1> > *
        ("mapBufferIndexToPointers", ctypes.c_char * 0x28),  # std::array<void *,5>
    ]
    mpShaderCombination: int
    mpShaderBufferHandles: bytes
    mapBufferIndexToPointers: bytes

class cEgRendererBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgRendererBase_dtor_0", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
        ("Resize", ctypes.c_ulonglong),
    ]
    cEgRendererBase_dtor_0: int
    Initialise: int
    Resize: int

class PCParam(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class PCParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class cEgPipelineCommand(ctypes.Structure):
    def cEgPipelineCommand(): pass

    def tilde_cEgPipelineCommand(): pass

    _fields_ = [
        ("_mCommand", ctypes.c_uint32),
        ("mRefParams", ctypes.c_char * 0x18),  # std::vector<void *,TkSTLAllocatorShim<void *,8,-1> >
        ("mValParams", ctypes.c_char * 0x18),  # std::vector<PCParam *,TkSTLAllocatorShim<PCParam *,8,-1> >
        ("mResParams", ctypes.c_char * 0x18),  # std::vector<cTkTypedSmartResHandle<cTkResource>,TkSTLAllocatorShim<cTkTypedSmartResHandle<cTkResource>,8,-1> >
        ("mbEnabled", ctypes.c_ubyte),
    ]
    _mCommand: int
    mRefParams: bytes
    mValParams: bytes
    mResParams: bytes
    mbEnabled: bool

    @property
    def mCommand(self):
        return PipelineCommands.List(self._mCommand)


class cEgPipelineResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cTkDependencyCounter(ctypes.Structure):
    def Increment(): pass

    def Decrement(): pass

    _fields_ = [
        ("miValue", ctypes.c_char * 0x4),  # std::atomic<int>
    ]
    miValue: bytes

class cEgUpdatableQueueEntry(ctypes.Structure):
    _fields_ = [
        ("mpNode", ctypes.c_ulonglong),
        ("mbCullFullyInside", ctypes.c_ubyte),
        ("mbPadding", ctypes.c_ubyte * 0x7),  # bool[7]
    ]
    mpNode: int
    mbCullFullyInside: bool
    mbPadding: list[ctypes.c_ubyte]

class cTkDestructableClass(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cTkDestructableClass_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class FfxFsr2ContextCustom(ctypes.Structure):
    _fields_ = [
        ("fRenderOffsetUV", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("iRenderOffset", ctypes.c_char * 0x8),  # cTkFixedArray<int,2>
        ("iRenderSize", ctypes.c_char * 0x8),  # cTkFixedArray<int,2>
        ("fDisplayOffsetUV", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("iDisplayOffset", ctypes.c_char * 0x8),  # cTkFixedArray<int,2>
        ("iDisplaySize", ctypes.c_char * 0x8),  # cTkFixedArray<int,2>
        ("uLumaMipDimensions", ctypes.c_char * 0x8),  # cTkFixedArray<unsigned int,2>
        ("uLumaMipLevelToUse", ctypes.c_char * 0x4),  # cTkFixedArray<unsigned int,1>
        ("uFrameIndex", ctypes.c_char * 0x4),  # cTkFixedArray<unsigned int,1>
        ("fDisplaySizeRcp", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("fJitter", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("fDeviceToViewDepth", ctypes.c_char * 0x10),  # cTkFixedArray<float,4>
        ("depthclip_uv_scale", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("postprocessed_lockstatus_uv_scale", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("reactive_mask_dim_rcp", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("MotionVectorScale", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("fDownscaleFactor", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("fPreExposure", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("fTanHalfFOV", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("fMotionVectorJitterCancellation", ctypes.c_char * 0x8),  # cTkFixedArray<float,2>
        ("fJitterSequenceLength", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("fLockInitialLifetime", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("fLockTickDelta", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("fDeltaTime", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("fDynamicResChangeFactor", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("fLumaMipRcp", ctypes.c_char * 0x4),  # cTkFixedArray<float,1>
        ("mips", ctypes.c_char * 0x4),  # cTkFixedArray<unsigned int,1>
        ("numWorkGroups", ctypes.c_char * 0x4),  # cTkFixedArray<unsigned int,1>
        ("workGroupOffset", ctypes.c_char * 0x8),  # cTkFixedArray<unsigned int,2>
        ("renderSize", ctypes.c_char * 0x8),  # cTkFixedArray<unsigned int,2>
        ("rcasConfig", ctypes.c_char * 0x10),  # cTkFixedArray<unsigned int,4>
    ]
    fRenderOffsetUV: bytes
    iRenderOffset: bytes
    iRenderSize: bytes
    fDisplayOffsetUV: bytes
    iDisplayOffset: bytes
    iDisplaySize: bytes
    uLumaMipDimensions: bytes
    uLumaMipLevelToUse: bytes
    uFrameIndex: bytes
    fDisplaySizeRcp: bytes
    fJitter: bytes
    fDeviceToViewDepth: bytes
    depthclip_uv_scale: bytes
    postprocessed_lockstatus_uv_scale: bytes
    reactive_mask_dim_rcp: bytes
    MotionVectorScale: bytes
    fDownscaleFactor: bytes
    fPreExposure: bytes
    fTanHalfFOV: bytes
    fMotionVectorJitterCancellation: bytes
    fJitterSequenceLength: bytes
    fLockInitialLifetime: bytes
    fLockTickDelta: bytes
    fDeltaTime: bytes
    fDynamicResChangeFactor: bytes
    fLumaMipRcp: bytes
    mips: bytes
    numWorkGroups: bytes
    workGroupOffset: bytes
    renderSize: bytes
    rcasConfig: bytes

class TkSettingsReader(ctypes.Structure):
    pass

class cTkBankInfo(ctypes.Structure):
    def cTkBankInfo(): pass

    _fields_ = [
        ("msPath", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mLength", ctypes.c_uint64),
    ]
    msPath: bytes
    mLength: int

class cGcNetworkInteractionBufferSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class NVSDK_NGX_BufferInfo_VK(ctypes.Structure):
    _fields_ = [
        ("Buffer", ctypes.c_ulonglong),
        ("SizeInBytes", ctypes.c_uint32),
    ]
    Buffer: int
    SizeInBytes: int

class cGcNetworkInteractionBufferHashMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class NVSDK_NGX_VK_Feature_Eval_Params(ctypes.Structure):
    _fields_ = [
        ("pInColor", ctypes.c_ulonglong),
        ("pInOutput", ctypes.c_ulonglong),
        ("InSharpness", ctypes.c_float),
    ]
    pInColor: int
    pInOutput: int
    InSharpness: float

class NVSDK_NGX_VK_GBuffer(ctypes.Structure):
    _fields_ = [
        ("pInAttrib", ctypes.c_ulonglong * 0x10),  # NVSDK_NGX_Resource_VK *[16]
    ]
    pInAttrib: list[ctypes.c_ulonglong]

class cGcNetworkInteractionBufferEditMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkMaintenanceBufferEditMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkTerrainHashMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkBaseHashMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkBaseSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class SceFiosOpAttr(ctypes.Structure):
    _fields_ = [
        ("deadline", ctypes.c_int64),
        ("pCallback", ctypes.c_ulonglong),
        ("pCallbackContext", ctypes.c_void_p),
        ("_bf_18", ctypes.c_int32),
        ("userTag", ctypes.c_uint32),
        ("userPtr", ctypes.c_void_p),
        ("pReserved", ctypes.c_void_p),
    ]
    deadline: int
    pCallback: int
    pCallbackContext: int
    _bf_18: int
    userTag: int
    userPtr: int
    pReserved: int

class cGcNetworkBaseEditMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cTkReferenceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class SceFiosDirEntry(ctypes.Structure):
    _fields_ = [
        ("fileSize", ctypes.c_int64),
        ("statFlags", ctypes.c_uint32),
        ("nameLength", ctypes.c_uint16),
        ("fullPathLength", ctypes.c_uint16),
        ("offsetToName", ctypes.c_uint16),
        ("reserved", ctypes.c_uint16 * 0x3),  # unsigned __int16[3]
        ("fullPath", ctypes.c_char * 0x400),  # char[1024]
    ]
    fileSize: int
    statFlags: int
    nameLength: int
    fullPathLength: int
    offsetToName: int
    reserved: list[ctypes.c_uint16]
    fullPath: list[ctypes.c_char]

class cTkRotationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcSpawnResult(ctypes.Structure):
    _fields_ = [
        ("mNode", ctypes.c_uint32),
        ("mpSwarm", ctypes.c_ulonglong),
        ("mpAttachment", ctypes.c_uint64),
    ]
    mNode: int
    mpSwarm: int
    mpAttachment: int

class sLinkObjectGridSnapshot(ctypes.Structure):
    _fields_ = [
        ("miGridPositiveRate", ctypes.c_int32),
        ("miGridNegativeRate", ctypes.c_int32),
        ("mfGridAmount", ctypes.c_float),
        ("miGridTotalStorage", ctypes.c_int32),
        ("miGridTotalObjects", ctypes.c_int32),
        ("miGridTotalPassiveObjects", ctypes.c_int32),
        ("miGridTotalStorageOnlyObjects", ctypes.c_int32),
        ("miGridPotentialPositiveRate", ctypes.c_int32),
        ("mbIsGridOnline", ctypes.c_ubyte),
        ("miGridHotspotTypes", ctypes.c_int32),
        ("mbGridHotspotsError", ctypes.c_ubyte),
    ]
    miGridPositiveRate: int
    miGridNegativeRate: int
    mfGridAmount: float
    miGridTotalStorage: int
    miGridTotalObjects: int
    miGridTotalPassiveObjects: int
    miGridTotalStorageOnlyObjects: int
    miGridPotentialPositiveRate: int
    mbIsGridOnline: bool
    miGridHotspotTypes: int
    mbGridHotspotsError: bool

class cGcSentinelRobot_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcSentinelRobot_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("GetTree", ctypes.c_ulonglong),
        ("GetCreatureType", ctypes.c_ulonglong),
        ("GetOwnerConcept", ctypes.c_ulonglong),
        ("GetWeapon", ctypes.c_ulonglong),
        ("FireWeapon", ctypes.c_ulonglong),
        ("GetTargetPoint", ctypes.c_ulonglong),
        ("GetForward", ctypes.c_ulonglong),
        ("GetLabelPoint", ctypes.c_ulonglong),
        ("GetTargetPointSize", ctypes.c_ulonglong),
        ("GetSightPoint", ctypes.c_ulonglong),
        ("GetObb", ctypes.c_ulonglong),
        ("GetEffectAttachLocator", ctypes.c_ulonglong),
        ("IsAttacking", ctypes.c_ulonglong),
        ("IsAlert", ctypes.c_ulonglong),
        ("IsDead", ctypes.c_ulonglong),
        ("GetAttackTarget", ctypes.c_ulonglong),
        ("GetAlertRange", ctypes.c_ulonglong),
        ("GetSightRange", ctypes.c_ulonglong),
        ("GetSightAngle", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("OnHit", ctypes.c_ulonglong),
        ("OnArmourHit", ctypes.c_ulonglong),
        ("OnDeath", ctypes.c_ulonglong),
    ]
    cGcSentinelRobot_dtor_0: int
    Prepare: int
    Release: int
    Update: int
    NetSerialize: int
    GetTree: int
    GetCreatureType: int
    GetOwnerConcept: int
    GetWeapon: int
    FireWeapon: int
    GetTargetPoint: int
    GetForward: int
    GetLabelPoint: int
    GetTargetPointSize: int
    GetSightPoint: int
    GetObb: int
    GetEffectAttachLocator: int
    IsAttacking: int
    IsAlert: int
    IsDead: int
    GetAttackTarget: int
    GetAlertRange: int
    GetSightRange: int
    GetSightAngle: int
    RenderNGui: int
    OnHit: int
    OnArmourHit: int
    OnDeath: int

class cGcSentinelRobotComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class SceFiosOverlay(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint8),
        ("order", ctypes.c_uint8),
        ("reserved", ctypes.c_uint8 * 0xA),  # unsigned __int8[10]
        ("id", ctypes.c_int32),
        ("dst", ctypes.c_char * 0x124),  # char[292]
        ("src", ctypes.c_char * 0x124),  # char[292]
    ]
    type: int
    order: int
    reserved: list[ctypes.c_uint8]
    id: int
    dst: list[ctypes.c_char]
    src: list[ctypes.c_char]

class CustomisationGroup(ctypes.Structure):
    _fields_ = [
        ("mTitle", ctypes.c_char * 0x8),  # TkID<128> *
        ("mPotentialParts", ctypes.c_char * 0x18),  # std::vector<TkID<256> const *,TkSTLAllocatorShim<TkID<256> const *,8,-1> >
    ]
    mTitle: bytes
    mPotentialParts: bytes

class SceFiosStat(ctypes.Structure):
    _fields_ = [
        ("fileSize", ctypes.c_int64),
        ("accessDate", ctypes.c_uint64),
        ("modificationDate", ctypes.c_uint64),
        ("creationDate", ctypes.c_uint64),
        ("statFlags", ctypes.c_uint32),
        ("reserved", ctypes.c_uint32),
        ("uid", ctypes.c_int64),
        ("gid", ctypes.c_int64),
        ("dev", ctypes.c_int64),
        ("ino", ctypes.c_int64),
        ("mode", ctypes.c_int64),
    ]
    fileSize: int
    accessDate: int
    modificationDate: int
    creationDate: int
    statFlags: int
    reserved: int
    uid: int
    gid: int
    dev: int
    ino: int
    mode: int

class cGcBaseLinks(ctypes.Structure):
    def SetNodeVisualState(): pass

    def IsPowerPreventingObjectInteraction(): pass


class VkAllocFileLineFunc(ctypes.Structure):
    _fields_ = [
        ("mpFile", ctypes.c_char_p),
        ("miLine", ctypes.c_int32),
        ("mpFunc", ctypes.c_char_p),
    ]
    mpFile: bytes
    miLine: int
    mpFunc: bytes

class VkExtensionProperties(ctypes.Structure):
    _fields_ = [
        ("extensionName", ctypes.c_char * 0x100),  # char[256]
        ("specVersion", ctypes.c_uint32),
    ]
    extensionName: list[ctypes.c_char]
    specVersion: int

class VkLayerProperties(ctypes.Structure):
    _fields_ = [
        ("layerName", ctypes.c_char * 0x100),  # char[256]
        ("specVersion", ctypes.c_uint32),
        ("implementationVersion", ctypes.c_uint32),
        ("description", ctypes.c_char * 0x100),  # char[256]
    ]
    layerName: list[ctypes.c_char]
    specVersion: int
    implementationVersion: int
    description: list[ctypes.c_char]

class NVSDK_NGX_PathListInfo(ctypes.Structure):
    _fields_ = [
        ("Path", ctypes.c_ulonglong),
        ("Length", ctypes.c_uint32),
    ]
    Path: int
    Length: int

class NVSDK_NGX_FeatureCommonInfo_Internal(ctypes.Structure):
    _fields_ = [
        ("UWP", ctypes.c_ubyte),
        ("LoadGenericFeatures", ctypes.c_ubyte),
    ]
    UWP: bool
    LoadGenericFeatures: bool

class GLFWvidmode(ctypes.Structure):
    _fields_ = [
        ("width", ctypes.c_int32),
        ("height", ctypes.c_int32),
        ("redBits", ctypes.c_int32),
        ("greenBits", ctypes.c_int32),
        ("blueBits", ctypes.c_int32),
        ("refreshRate", ctypes.c_int32),
    ]
    width: int
    height: int
    redBits: int
    greenBits: int
    blueBits: int
    refreshRate: int

class cTkEngineUtils(ctypes.Structure):
    def AddNodes(): pass

    def AddNodesNoPrepare(): pass

    def AddNodesStep(): pass

    def FindNode(): pass

    def FindNodeContainingString(): pass

    def GetGroupNode(): pass

    def GetMasterModelNode(): pass

    def GetMatricesFromNode(): pass

    def GetModelNode(): pass

    def GetResHandleFromNode(): pass

    def GetSeedFromNode(): pass

    def IsNodeImposter(): pass

    def ReadModelFilename(): pass

    def RepositionGroupNode(): pass

    def RequestTexturesAlwaysResident(): pass

    def SetNodeActive(): pass

    def SetNodeActiveRecursive(): pass

    def SetNodeMatrixWorld(): pass

    def SetNodeParent(): pass

    def SetSceneNetworkId(): pass


class cGcCustomInventoryComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkDocumentReaderJSON_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkDocumentReader_dtor_0", ctypes.c_ulonglong),
        ("GetReaderFormat", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(ITkDocumentReader *this, TkID<128> *result)
        ("PushContext", ctypes.c_ulonglong),
        ("PopContext", ctypes.c_ulonglong),
        ("MoveCursor", ctypes.c_ulonglong),
        ("MoveCursor_28", ctypes.c_ulonglong),
        ("PopCursor", ctypes.c_ulonglong),
        ("IsCursorNull", ctypes.c_ulonglong),
        ("ReadArraySize", ctypes.c_ulonglong),
        ("ReadStringSize", ctypes.c_ulonglong),
        ("ReadValue", ctypes.c_ulonglong),
        ("ReadValue_58", ctypes.c_ulonglong),
        ("ReadValue_60", ctypes.c_ulonglong),
        ("ReadValue_68", ctypes.c_ulonglong),
        ("ReadValue_70", ctypes.c_ulonglong),
        ("ReadValue_78", ctypes.c_ulonglong),
        ("ReadValue_80", ctypes.c_ulonglong),
        ("ReadValue_88", ctypes.c_ulonglong),
        ("ReadValue_90", ctypes.c_ulonglong),
        ("ReadValue_98", ctypes.c_ulonglong),
        ("ReadValue_a0", ctypes.c_ulonglong),
        ("ReadValue_a8", ctypes.c_ulonglong),
        ("ReadValue_b0", ctypes.c_char * 0x8),  # ITkDocumentReader::ReadResult (__fastcall *)(ITkDocumentReader *this, TkID<256> *)
        ("ReadValue_b8", ctypes.c_char * 0x8),  # ITkDocumentReader::ReadResult (__fastcall *)(ITkDocumentReader *this, TkID<128> *)
        ("ReadValue_c0", ctypes.c_ulonglong),
        ("ReadValue_c8", ctypes.c_ulonglong),
        ("ReadValue_d0", ctypes.c_ulonglong),
        ("ReadValue_d8", ctypes.c_ulonglong),
        ("ReadValue_e0", ctypes.c_ulonglong),
        ("ReadValue_e8", ctypes.c_ulonglong),
        ("ReadValue_f0", ctypes.c_ulonglong),
        ("ReadValue_f8", ctypes.c_ulonglong),
        ("ReadValue_100", ctypes.c_ulonglong),
        ("ReadValue_108", ctypes.c_ulonglong),
        ("ReadValue_110", ctypes.c_ulonglong),
    ]
    ITkDocumentReader_dtor_0: int
    GetReaderFormat: bytes
    PushContext: int
    PopContext: int
    MoveCursor: int
    MoveCursor_28: int
    PopCursor: int
    IsCursorNull: int
    ReadArraySize: int
    ReadStringSize: int
    ReadValue: int
    ReadValue_58: int
    ReadValue_60: int
    ReadValue_68: int
    ReadValue_70: int
    ReadValue_78: int
    ReadValue_80: int
    ReadValue_88: int
    ReadValue_90: int
    ReadValue_98: int
    ReadValue_a0: int
    ReadValue_a8: int
    ReadValue_b0: bytes
    ReadValue_b8: bytes
    ReadValue_c0: int
    ReadValue_c8: int
    ReadValue_d0: int
    ReadValue_d8: int
    ReadValue_e0: int
    ReadValue_e8: int
    ReadValue_f0: int
    ReadValue_f8: int
    ReadValue_100: int
    ReadValue_108: int
    ReadValue_110: int

class cGcDualShadowMapQueues(ctypes.Structure):
    _fields_ = [
        ("mQueues", ctypes.c_uint64 * 0x2),  # EgRenderQueueHandle[2]
    ]
    mQueues: list[ctypes.c_uint64]

class cSMShaderConnSpec(ctypes.Structure):
    _fields_ = [
        ("mName", ctypes.c_char_p),
        ("mCount", ctypes.c_int32),
    ]
    mName: bytes
    mCount: int

class VkDeviceOrHostAddressConstKHR: pass

class VkExternalMemoryProperties(ctypes.Structure):
    _fields_ = [
        ("externalMemoryFeatures", ctypes.c_uint32),
        ("exportFromImportedHandleTypes", ctypes.c_uint32),
        ("compatibleHandleTypes", ctypes.c_uint32),
    ]
    externalMemoryFeatures: int
    exportFromImportedHandleTypes: int
    compatibleHandleTypes: int

class AkSegmentInfo(ctypes.Structure):
    _fields_ = [
        ("iCurrentPosition", ctypes.c_int32),
        ("iPreEntryDuration", ctypes.c_int32),
        ("iActiveDuration", ctypes.c_int32),
        ("iPostExitDuration", ctypes.c_int32),
        ("iRemainingLookAheadTime", ctypes.c_int32),
        ("fBeatDuration", ctypes.c_float),
        ("fBarDuration", ctypes.c_float),
        ("fGridDuration", ctypes.c_float),
        ("fGridOffset", ctypes.c_float),
    ]
    iCurrentPosition: int
    iPreEntryDuration: int
    iActiveDuration: int
    iPostExitDuration: int
    iRemainingLookAheadTime: int
    fBeatDuration: float
    fBarDuration: float
    fGridDuration: float
    fGridOffset: float

class VkIndirectCommandsStreamNV(ctypes.Structure):
    _fields_ = [
        ("buffer", ctypes.c_ulonglong),
        ("offset", ctypes.c_uint64),
    ]
    buffer: int
    offset: int

class cQuantTriangle(ctypes.Structure):
    _fields_ = [
        ("mVertex", ctypes.c_char * 0x12),  # cTkQuantVector3<short>
    ]
    mVertex: bytes

class cTkQuantizedTriBlock(ctypes.Structure):
    _fields_ = [
        ("maPos0X", ctypes.c_ulonglong),
        ("maPos0Y", ctypes.c_ulonglong),
        ("maPos0Z", ctypes.c_ulonglong),
        ("maPos1X", ctypes.c_ulonglong),
        ("maPos1Y", ctypes.c_ulonglong),
        ("maPos1Z", ctypes.c_ulonglong),
        ("maPos2X", ctypes.c_ulonglong),
        ("maPos2Y", ctypes.c_ulonglong),
        ("maPos2Z", ctypes.c_ulonglong),
        ("mLocalShapeKeys", ctypes.c_ulonglong),
        ("maShapeTag", ctypes.c_ulonglong),
        ("miNumTris", ctypes.c_int32),
        ("miCapacity", ctypes.c_int32),
    ]
    maPos0X: int
    maPos0Y: int
    maPos0Z: int
    maPos1X: int
    maPos1Y: int
    maPos1Z: int
    maPos2X: int
    maPos2Y: int
    maPos2Z: int
    mLocalShapeKeys: int
    maShapeTag: int
    miNumTris: int
    miCapacity: int

class cTkQuantizedTriList(ctypes.Structure):
    def CalcAabb(): pass

    def Reserve(): pass

    _fields_ = [
        ("mTriangles", ctypes.c_char * 0x18),  # std::vector<cQuantTriangle,TkSTLAllocatorShim<cQuantTriangle,2,0> >
        ("mMaterials", ctypes.c_char * 0x18),  # std::vector<unsigned char,TkSTLAllocatorShim<unsigned char,1,0> >
    ]
    mTriangles: bytes
    mMaterials: bytes

class cTkParallelRNG(ctypes.Structure):
    _fields_ = [
        ("_k", ctypes.c_uint64 * 0x4),  # unsigned __int64[4]
        ("_s", ctypes.c_uint64 * 0x4),  # unsigned __int64[4]
        ("_o", ctypes.c_uint64 * 0x4),  # unsigned __int64[4]
        ("_o_counter", ctypes.c_uint16),
    ]
    _k: list[ctypes.c_uint64]
    _s: list[ctypes.c_uint64]
    _o: list[ctypes.c_uint64]
    _o_counter: int

class cTkDirection(ctypes.Structure):
    _fields_ = [
        ("m_radians", ctypes.c_float),
    ]
    m_radians: float

class cTkBasis(ctypes.Structure):
    _fields_ = [
        ("m_origin", ctypes.c_float),
        ("m_ccwRadiansPerUnitMinusOne", ctypes.c_float),
    ]
    m_origin: float
    m_ccwRadiansPerUnitMinusOne: float

class cTkTurn(ctypes.Structure):
    _fields_ = [
        ("m_radians", ctypes.c_float),
    ]
    m_radians: float

class VkDeviceOrHostAddressKHR: pass

class cTkDistanceFunctions(ctypes.Structure):
    pass

class cAabb(ctypes.Structure):
    _fields_ = [
        ("corners", common.Vector4f * 0x2),  # hkVector4f[2]
    ]
    corners: list[common.Vector4f]

class AkPlacementNewKey(ctypes.Structure):
    pass

class VkInputAttachmentAspectReference(ctypes.Structure):
    _fields_ = [
        ("subpass", ctypes.c_uint32),
        ("inputAttachmentIndex", ctypes.c_uint32),
        ("aspectMask", ctypes.c_uint32),
    ]
    subpass: int
    inputAttachmentIndex: int
    aspectMask: int

class cTkJob(ctypes.Structure):
    _fields_ = [
        ("mpData", ctypes.c_void_p),
        ("mFunction", ctypes.c_ulonglong),
        ("mpDepCounter", ctypes.c_char * 0x8),  # std::atomic<int> *
        ("miQueue", ctypes.c_int32),
        ("miIndex", ctypes.c_int32),
    ]
    mpData: int
    mFunction: int
    mpDepCounter: bytes
    miQueue: int
    miIndex: int

class VkSampleLocationEXT(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
    ]
    x: float
    y: float

class AkGrowByPolicy_Proportional(ctypes.Structure):
    pass

class cTkSkinnedVertexData(ctypes.Structure):
    _fields_ = [
        ("mPosition", common.Vector3f),
        ("mNormal", common.Vector3f),
        ("mTangent", common.Vector3f),
        ("mWeights", common.Vector4f),
        ("mJoints", common.Vector4f),
        ("mTexCoords", common.Vector2f),
    ]
    mPosition: common.Vector3f
    mNormal: common.Vector3f
    mTangent: common.Vector3f
    mWeights: common.Vector4f
    mJoints: common.Vector4f
    mTexCoords: common.Vector2f

class cTkPhysicsBvMeshShapeFunctions(ctypes.Structure):
    def RegisterShapeFunctions(): pass

    def buildSurfaceGeometry(): pass

    def calcAabb(): pass

    def castRay(): pass

    def getLeafShapes(): pass

    def getShapeKeys(): pass

    def queryAabb(): pass


class cGcExperienceTimerHelper(ctypes.Structure):
    pass

class cTkStaticPhysicsComponentUpdater_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnInit", ctypes.c_ulonglong),
        ("UpdateComponents", ctypes.c_ulonglong),
        ("UpdateComponentsPostPhysics", ctypes.c_ulonglong),
        ("UpdateComponentsRender", ctypes.c_ulonglong),
        ("OnComponentAllocate", ctypes.c_ulonglong),
        ("OnComponentFree", ctypes.c_ulonglong),
        ("HasCustomUpdate", ctypes.c_ulonglong),
    ]
    OnInit: int
    UpdateComponents: int
    UpdateComponentsPostPhysics: int
    UpdateComponentsRender: int
    OnComponentAllocate: int
    OnComponentFree: int
    HasCustomUpdate: int

class cTkVoxelGeneratorSimplex(ctypes.Structure):
    pass

class cTkSimplexNoiseData2D(ctypes.Structure):
    _fields_ = [
        ("mfBiomeDensity", ctypes.c_float),
        ("mfFlatDensity", ctypes.c_float),
    ]
    mfBiomeDensity: float
    mfFlatDensity: float

class VkDisplayPlanePropertiesKHR(ctypes.Structure):
    _fields_ = [
        ("currentDisplay", ctypes.c_ulonglong),
        ("currentStackIndex", ctypes.c_uint32),
    ]
    currentDisplay: int
    currentStackIndex: int

class StdVideoH265PpsFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int32),
    ]
    _bf_0: int

class StdVideoH265ScalingLists(ctypes.Structure):
    _fields_ = [
        ("ScalingList4x4", ctypes.c_uint8 * 0x6),  # unsigned __int8[6][16]
        ("ScalingList8x8", ctypes.c_uint8 * 0x6),  # unsigned __int8[6][64]
        ("ScalingList16x16", ctypes.c_uint8 * 0x6),  # unsigned __int8[6][64]
        ("ScalingList32x32", ctypes.c_uint8 * 0x2),  # unsigned __int8[2][64]
        ("ScalingListDCCoef16x16", ctypes.c_uint8 * 0x6),  # unsigned __int8[6]
        ("ScalingListDCCoef32x32", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
    ]
    ScalingList4x4: list[ctypes.c_uint8]
    ScalingList8x8: list[ctypes.c_uint8]
    ScalingList16x16: list[ctypes.c_uint8]
    ScalingList32x32: list[ctypes.c_uint8]
    ScalingListDCCoef16x16: list[ctypes.c_uint8]
    ScalingListDCCoef32x32: list[ctypes.c_uint8]

class StdVideoH265PredictorPaletteEntries(ctypes.Structure):
    _fields_ = [
        ("PredictorPaletteEntries", ctypes.c_uint16 * 0x3),  # unsigned __int16[3][128]
    ]
    PredictorPaletteEntries: list[ctypes.c_uint16]

class DefaultAllocShim(ctypes.Structure):
    pass

class cTkSmallBlockMemoryPool_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkMemoryPool_dtor_0", ctypes.c_ulonglong),
        ("Create", ctypes.c_ulonglong),
        ("Malloc", ctypes.c_ulonglong),
        ("Malloc_18", ctypes.c_ulonglong),
        ("Free", ctypes.c_ulonglong),
        ("Realloc", ctypes.c_ulonglong),
        ("ContainsAddress", ctypes.c_ulonglong),
        ("GetAllocationSize", ctypes.c_ulonglong),
        ("SetAlignment", ctypes.c_ulonglong),
        ("GetAlignment", ctypes.c_ulonglong),
        ("GetUsage", ctypes.c_ulonglong),
        ("GetHighWaterUsage", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("Cleanup", ctypes.c_ulonglong),
        ("GetAllocInfo", ctypes.c_ulonglong),
        ("SetAllocInfo", ctypes.c_ulonglong),
        ("ClearStats", ctypes.c_ulonglong),
        ("OutputStats", ctypes.c_ulonglong),
        ("FrameSyncEnd", ctypes.c_ulonglong),
        ("NeedsFrameSyncEnd", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("FlagError", ctypes.c_ulonglong),
        ("HasError", ctypes.c_ulonglong),
        ("NeedsMutex", ctypes.c_ulonglong),
        ("SetMutex", ctypes.c_ulonglong),
        ("GetOverflowPool", ctypes.c_ulonglong),
        ("GetBaseAddress", ctypes.c_ulonglong),
    ]
    cTkMemoryPool_dtor_0: int
    Create: int
    Malloc: int
    Malloc_18: int
    Free: int
    Realloc: int
    ContainsAddress: int
    GetAllocationSize: int
    SetAlignment: int
    GetAlignment: int
    GetUsage: int
    GetHighWaterUsage: int
    GetSize: int
    Cleanup: int
    GetAllocInfo: int
    SetAllocInfo: int
    ClearStats: int
    OutputStats: int
    FrameSyncEnd: int
    NeedsFrameSyncEnd: int
    Reset: int
    FlagError: int
    HasError: int
    NeedsMutex: int
    SetMutex: int
    GetOverflowPool: int
    GetBaseAddress: int

class VkFormatProperties(ctypes.Structure):
    _fields_ = [
        ("linearTilingFeatures", ctypes.c_uint32),
        ("optimalTilingFeatures", ctypes.c_uint32),
        ("bufferFeatures", ctypes.c_uint32),
    ]
    linearTilingFeatures: int
    optimalTilingFeatures: int
    bufferFeatures: int

class StdVideoH264ScalingLists(ctypes.Structure):
    _fields_ = [
        ("scaling_list_present_mask", ctypes.c_uint16),
        ("use_default_scaling_matrix_mask", ctypes.c_uint16),
        ("ScalingList4x4", ctypes.c_uint8 * 0x6),  # unsigned __int8[6][16]
        ("ScalingList8x8", ctypes.c_uint8 * 0x6),  # unsigned __int8[6][64]
    ]
    scaling_list_present_mask: int
    use_default_scaling_matrix_mask: int
    ScalingList4x4: list[ctypes.c_uint8]
    ScalingList8x8: list[ctypes.c_uint8]

class cGcClothComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cHeavySimulateTaskData(ctypes.Structure):
    _fields_ = [
        ("mPiece", ctypes.c_ulonglong),
        ("mDeltaTime", ctypes.c_float),
        ("mPieceIndex", ctypes.c_int32),
    ]
    mPiece: int
    mDeltaTime: float
    mPieceIndex: int

class AkObstructionOcclusionValues(ctypes.Structure):
    _fields_ = [
        ("occlusion", ctypes.c_float),
        ("obstruction", ctypes.c_float),
    ]
    occlusion: float
    obstruction: float

class VkPipelineCreationFeedback(ctypes.Structure):
    _fields_ = [
        ("flags", ctypes.c_uint32),
        ("duration", ctypes.c_uint64),
    ]
    flags: int
    duration: int

class hknpAnyHitCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class VkStridedDeviceAddressRegionKHR(ctypes.Structure):
    _fields_ = [
        ("deviceAddress", ctypes.c_uint64),
        ("stride", ctypes.c_uint64),
        ("size", ctypes.c_uint64),
    ]
    deviceAddress: int
    stride: int
    size: int

class cTkReferencedMesh(ctypes.Structure):
    _fields_ = [
        ("mpMesh", ctypes.c_ulonglong),
        ("mKey", ctypes.c_int32),
        ("miRefCount", ctypes.c_int32),
    ]
    mpMesh: int
    mKey: int
    miRefCount: int

class cTkTriSoupMeshShapeCInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCompressedMeshShapeCinfo_dtor_0", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getVertex", ctypes.c_ulonglong),
        ("getIndices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getNumConvexShapes", ctypes.c_ulonglong),
        ("getConvexShape", ctypes.c_ulonglong),
        ("getConvexShapeTag", ctypes.c_ulonglong),
    ]
    hknpCompressedMeshShapeCinfo_dtor_0: int
    getNumVertices: int
    getNumTriangles: int
    getVertex: int
    getIndices: int
    getTriangleShapeTag: int
    getNumConvexShapes: int
    getConvexShape: int
    getConvexShapeTag: int

class cTkIndexedMeshShapeCInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCompressedMeshShapeCinfo_dtor_0", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getVertex", ctypes.c_ulonglong),
        ("getIndices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getNumConvexShapes", ctypes.c_ulonglong),
        ("getConvexShape", ctypes.c_ulonglong),
        ("getConvexShapeTag", ctypes.c_ulonglong),
    ]
    hknpCompressedMeshShapeCinfo_dtor_0: int
    getNumVertices: int
    getNumTriangles: int
    getVertex: int
    getIndices: int
    getTriangleShapeTag: int
    getNumConvexShapes: int
    getConvexShape: int
    getConvexShapeTag: int

class VkDeviceFaultVendorInfoEXT(ctypes.Structure):
    _fields_ = [
        ("description", ctypes.c_char * 0x100),  # char[256]
        ("vendorFaultCode", ctypes.c_uint64),
        ("vendorFaultData", ctypes.c_uint64),
    ]
    description: list[ctypes.c_char]
    vendorFaultCode: int
    vendorFaultData: int

class cTkTerrainMeshShapeCInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCompressedMeshShapeCinfo_dtor_0", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getVertex", ctypes.c_ulonglong),
        ("getIndices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getNumConvexShapes", ctypes.c_ulonglong),
        ("getConvexShape", ctypes.c_ulonglong),
        ("getConvexShapeTag", ctypes.c_ulonglong),
    ]
    hknpCompressedMeshShapeCinfo_dtor_0: int
    getNumVertices: int
    getNumTriangles: int
    getVertex: int
    getIndices: int
    getTriangleShapeTag: int
    getNumConvexShapes: int
    getConvexShape: int
    getConvexShapeTag: int

class cTkWaterMeshShapeCInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCompressedMeshShapeCinfo_dtor_0", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getVertex", ctypes.c_ulonglong),
        ("getIndices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getNumConvexShapes", ctypes.c_ulonglong),
        ("getConvexShape", ctypes.c_ulonglong),
        ("getConvexShapeTag", ctypes.c_ulonglong),
    ]
    hknpCompressedMeshShapeCinfo_dtor_0: int
    getNumVertices: int
    getNumTriangles: int
    getVertex: int
    getIndices: int
    getTriangleShapeTag: int
    getNumConvexShapes: int
    getConvexShape: int
    getConvexShapeTag: int

class cTkQuadMeshShapeCInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpCompressedMeshShapeCinfo_dtor_0", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("getNumTriangles", ctypes.c_ulonglong),
        ("getVertex", ctypes.c_ulonglong),
        ("getIndices", ctypes.c_ulonglong),
        ("getTriangleShapeTag", ctypes.c_ulonglong),
        ("getNumConvexShapes", ctypes.c_ulonglong),
        ("getConvexShape", ctypes.c_ulonglong),
        ("getConvexShapeTag", ctypes.c_ulonglong),
    ]
    hknpCompressedMeshShapeCinfo_dtor_0: int
    getNumVertices: int
    getNumTriangles: int
    getVertex: int
    getIndices: int
    getTriangleShapeTag: int
    getNumConvexShapes: int
    getConvexShape: int
    getConvexShapeTag: int

class VkConformanceVersion(ctypes.Structure):
    _fields_ = [
        ("major", ctypes.c_uint8),
        ("minor", ctypes.c_uint8),
        ("subminor", ctypes.c_uint8),
        ("patch", ctypes.c_uint8),
    ]
    major: int
    minor: int
    subminor: int
    patch: int

class StdVideoDecodeH264PictureInfoFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class StdVideoH265SpsVuiFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int16),
        ("_bf_2", ctypes.c_int8),
    ]
    _bf_0: int
    _bf_2: int

class cTkBlockAllocation(ctypes.Structure):
    _fields_ = [
        ("muBufferAlignment", ctypes.c_uint32),
        ("muBufferOffset", ctypes.c_uint32),
        ("muUsed", ctypes.c_uint32),
        ("muSize", ctypes.c_uint32),
    ]
    muBufferAlignment: int
    muBufferOffset: int
    muUsed: int
    muSize: int

class cTKPerThreadLocalAllocatorData(ctypes.Structure):
    _fields_ = [
        ("maStack", ctypes.c_char * 0x18),  # std::vector<cTkLinearMemoryPool *,TkSTLAllocatorShim<cTkLinearMemoryPool *,8,-1> >
        ("mbMainAllocatorAccessDisabled", ctypes.c_ubyte),
    ]
    maStack: bytes
    mbMainAllocatorAccessDisabled: bool

class cTkInteractivityManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkInteractivityManager_dtor_0", ctypes.c_ulonglong),
    ]
    cTkInteractivityManager_dtor_0: int

class cTkAllocationHeader(ctypes.Structure):
    _fields_ = [
        ("miSizeIncludingHeader", ctypes.c_int32),
        ("miAlignment", ctypes.c_int32),
        ("miPool", ctypes.c_int32),
        ("miHash", ctypes.c_int32),
    ]
    miSizeIncludingHeader: int
    miAlignment: int
    miPool: int
    miHash: int

class sLocklessIndexAllocatorUnitTestBlock(ctypes.Structure):
    _fields_ = [
        ("mauBuff", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    mauBuff: list[ctypes.c_uint8]

class sLocklessIndexAllocatorUnitTestParams(ctypes.Structure):
    _fields_ = [
        ("mpAllocator", ctypes.c_ulonglong),
        ("mpBlocks", ctypes.c_ulonglong),
        ("mNextJobId", ctypes.c_char * 0x4),  # std::atomic<int>
        ("mJobsRunning", ctypes.c_char * 0x4),  # std::atomic<int>
    ]
    mpAllocator: int
    mpBlocks: int
    mNextJobId: bytes
    mJobsRunning: bytes

class sLocklessPoolAllocatorUnitTestBlock(ctypes.Structure):
    _fields_ = [
        ("mauBuff", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    mauBuff: list[ctypes.c_uint8]

class sLocklessPoolAllocatorUnitTestParams(ctypes.Structure):
    _fields_ = [
        ("mpAllocator", ctypes.c_ulonglong),
        ("mpBlocks", ctypes.c_ulonglong),
        ("mNextJobId", ctypes.c_char * 0x4),  # std::atomic<int>
        ("mJobsRunning", ctypes.c_char * 0x4),  # std::atomic<int>
    ]
    mpAllocator: int
    mpBlocks: int
    mNextJobId: bytes
    mJobsRunning: bytes

class _IMAGEHLP_SYMBOL64(ctypes.Structure):
    _fields_ = [
        ("SizeOfStruct", ctypes.c_uint32),
        ("Address", ctypes.c_uint64),
        ("Size", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("MaxNameLength", ctypes.c_uint32),
        ("Name", ctypes.c_char * 0x1),  # char[1]
    ]
    SizeOfStruct: int
    Address: int
    Size: int
    Flags: int
    MaxNameLength: int
    Name: list[ctypes.c_char]

class cGcSolarSystemMultiQueryJob(ctypes.Structure):
    _fields_ = [
        ("maUAs", ctypes.c_char * 0x18),  # std::vector<unsigned __int64,TkSTLAllocatorShim<unsigned __int64,8,-1> >
        ("maQueries", ctypes.c_char * 0x18),  # std::vector<cGcSolarSystemQuery,TkSTLAllocatorShim<cGcSolarSystemQuery,16,-1> >
        ("mbRunning", ctypes.c_ubyte),
    ]
    maUAs: bytes
    maQueries: bytes
    mbRunning: bool

class cGcCreatureBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class VkSRTDataNV(ctypes.Structure):
    _fields_ = [
        ("sx", ctypes.c_float),
        ("a", ctypes.c_float),
        ("b", ctypes.c_float),
        ("pvx", ctypes.c_float),
        ("sy", ctypes.c_float),
        ("c", ctypes.c_float),
        ("pvy", ctypes.c_float),
        ("sz", ctypes.c_float),
        ("pvz", ctypes.c_float),
        ("qx", ctypes.c_float),
        ("qy", ctypes.c_float),
        ("qz", ctypes.c_float),
        ("qw", ctypes.c_float),
        ("tx", ctypes.c_float),
        ("ty", ctypes.c_float),
        ("tz", ctypes.c_float),
    ]
    sx: float
    a: float
    b: float
    pvx: float
    sy: float
    c: float
    pvy: float
    sz: float
    pvz: float
    qx: float
    qy: float
    qz: float
    qw: float
    tx: float
    ty: float
    tz: float

class cTkPhysWriteJanitor(ctypes.Structure):
    def cTkPhysWriteJanitor(): pass

    def tilde_cTkPhysWriteJanitor(): pass


class hkDynamicBlockStreamAllocator_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("blockAllocBatch", ctypes.c_ulonglong),
        ("blockFreeBatch", ctypes.c_ulonglong),
        ("getBytesUsed", ctypes.c_ulonglong),
        ("getMaxBytesUsed", ctypes.c_ulonglong),
        ("freeAllRemainingAllocations", ctypes.c_ulonglong),
        ("getMemoryStatistics", ctypes.c_ulonglong),
        ("getNumThreadLocalBlocks", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    blockAllocBatch: int
    blockFreeBatch: int
    getBytesUsed: int
    getMaxBytesUsed: int
    freeAllRemainingAllocations: int
    getMemoryStatistics: int
    getNumThreadLocalBlocks: int

class sRegionHeapTestAlloc(ctypes.Structure):
    _fields_ = [
        ("muOffset", ctypes.c_uint32),
        ("muSize", ctypes.c_uint32),
        ("muAlign", ctypes.c_uint32),
        ("muId", ctypes.c_uint8),
    ]
    muOffset: int
    muSize: int
    muAlign: int
    muId: int

class cGcDiscoveryPagePlanetTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcDiscoveryPageDataSearchTask_dtor_0", ctypes.c_ulonglong),
        ("IsComplete", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
    ]
    cGcDiscoveryPageDataSearchTask_dtor_0: int
    IsComplete: int
    Start: int
    Update: int
    Destroy: int

class VkPresentRegionKHR(ctypes.Structure):
    _fields_ = [
        ("rectangleCount", ctypes.c_uint32),
        ("pRectangles", ctypes.c_ulonglong),
    ]
    rectangleCount: int
    pRectangles: int

class cEgBaseNodeIterator(ctypes.Structure):
    _fields_ = [
        ("miTotalNodes", ctypes.c_int32),
        ("miCurrentNode", ctypes.c_int32),
        ("mOverflowNodes", ctypes.c_char * 0x18),  # std::vector<TkHandle,TkSTLAllocatorShim<TkHandle,4,-1> >
        ("mpNodesBuffer", ctypes.c_ulonglong),
        ("miNodesBufferSize", ctypes.c_int32),
    ]
    miTotalNodes: int
    miCurrentNode: int
    mOverflowNodes: bytes
    mpNodesBuffer: int
    miNodesBufferSize: int

class cSpringLink(ctypes.Structure):
    def cSpringLink(): pass

    def Prepare(): pass

    def CollectTrackedNodes(): pass

    def ApplyPositionalSpring(): pass

    def ApplyAngularSpring(): pass

    _fields_ = [
        ("mEnabled", ctypes.c_ubyte),
        ("mComponent", ctypes.c_ulonglong),
        ("mData", ctypes.c_ulonglong),
        ("mTrackedNodes", ctypes.c_char * 0x8),  # std::vector<cTrackedNode,TkSTLAllocatorShim<cTrackedNode,16,0> > *
        ("mSubLinksTrackedNodeIndices", ctypes.c_char * 0x18),  # TkVector_BoundChecked<int>
        ("mSpringPosition", ctypes.c_char * 0x18),  # TkVector_BoundChecked<cTkVector3>
        ("mSpringVelocity", ctypes.c_char * 0x18),  # TkVector_BoundChecked<cTkVector3>
        ("mSpringRotation", ctypes.c_char * 0x18),  # TkVector_BoundChecked<cTkVector3>
        ("mSpringAngularVelocity", ctypes.c_char * 0x18),  # TkVector_BoundChecked<cTkVector3>
        ("mPrevGoalVelocity", ctypes.c_char * 0x18),  # TkVector_BoundChecked<cTkVector3>
        ("mPrevGoalAngularVelocity", ctypes.c_char * 0x18),  # TkVector_BoundChecked<cTkVector3>
        ("mPrevPivotAnchorVelocity", ctypes.c_char * 0x18),  # TkVector_BoundChecked<cTkVector3>
        ("mExtraVelocityFromGravityWindAndExplosions", common.Vector3f),
        ("mPivotAnchorNode_TrackedNodeIndex", ctypes.c_int32),
        ("mPivotAnchorLocal", common.Vector3f),
    ]
    mEnabled: bool
    mComponent: int
    mData: int
    mTrackedNodes: bytes
    mSubLinksTrackedNodeIndices: bytes
    mSpringPosition: bytes
    mSpringVelocity: bytes
    mSpringRotation: bytes
    mSpringAngularVelocity: bytes
    mPrevGoalVelocity: bytes
    mPrevGoalAngularVelocity: bytes
    mPrevPivotAnchorVelocity: bytes
    mExtraVelocityFromGravityWindAndExplosions: common.Vector3f
    mPivotAnchorNode_TrackedNodeIndex: int
    mPivotAnchorLocal: common.Vector3f

class cGcSpringAttachmentComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class StdVideoDecodeH265PictureInfoFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class StdVideoH265HrdFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int16),
        ("_bf_2", ctypes.c_int8),
        ("_padding_3", ctypes.c_int8),
    ]
    _bf_0: int
    _bf_2: int
    _padding_3: int

class StdVideoH265SubLayerHrdParameters(ctypes.Structure):
    _fields_ = [
        ("bit_rate_value_minus1", ctypes.c_uint32 * 0x20),  # unsigned int[32]
        ("cpb_size_value_minus1", ctypes.c_uint32 * 0x20),  # unsigned int[32]
        ("cpb_size_du_value_minus1", ctypes.c_uint32 * 0x20),  # unsigned int[32]
        ("bit_rate_du_value_minus1", ctypes.c_uint32 * 0x20),  # unsigned int[32]
        ("cbr_flag", ctypes.c_uint32),
    ]
    bit_rate_value_minus1: list[ctypes.c_uint32]
    cpb_size_value_minus1: list[ctypes.c_uint32]
    cpb_size_du_value_minus1: list[ctypes.c_uint32]
    bit_rate_du_value_minus1: list[ctypes.c_uint32]
    cbr_flag: int

class VkTransformMatrixKHR(ctypes.Structure):
    _fields_ = [
        ("matrix", ctypes.c_float * 0x3),  # float[3][4]
    ]
    matrix: list[ctypes.c_float]

class ComponentInstancercTkAnimPoseComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercTkAnimPoseComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class StdVideoH264SpsFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int16),
    ]
    _bf_0: int

class VkRefreshCycleDurationGOOGLE(ctypes.Structure):
    _fields_ = [
        ("refreshDuration", ctypes.c_uint64),
    ]
    refreshDuration: int

class ComponentInstancercGcPortalComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcExpeditionHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcNetworkFleetFrigatesSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkFleetFrigateNamesSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcExpeditionSyncData(ctypes.Structure):
    _fields_ = [
        ("mSeed", common.GcSeed),
        ("mu64StartTime", ctypes.c_uint64),
        ("mu64PauseTime", ctypes.c_uint64),
        ("mu64UA", ctypes.c_uint64),
        ("mTerminalPosition", common.Vector3f),
        ("mEventIsInterventionEvent", ctypes.c_int32),
        ("maiPowerupIndex", ctypes.c_int32 * 0x3),  # int[3]
        ("mi8InterventionMissionIndex", ctypes.c_char),
        ("mi8NextEventIndex", ctypes.c_char),
        ("mi8Category", ctypes.c_char),
        ("mi8Duration", ctypes.c_char),
        ("mi8NumberOfFrigates", ctypes.c_char),
        ("mi8NumberOfDestroyedFrigates", ctypes.c_char),
        ("mi8NumberOfDamagedFrigates", ctypes.c_char),
        ("mbWaitingForIntervention", ctypes.c_ubyte),
        ("mbExpeditionEnded", ctypes.c_ubyte),
    ]
    mSeed: common.GcSeed
    mu64StartTime: int
    mu64PauseTime: int
    mu64UA: int
    mTerminalPosition: common.Vector3f
    mEventIsInterventionEvent: int
    maiPowerupIndex: list[ctypes.c_int32]
    mi8InterventionMissionIndex: str
    mi8NextEventIndex: str
    mi8Category: str
    mi8Duration: str
    mi8NumberOfFrigates: str
    mi8NumberOfDestroyedFrigates: str
    mi8NumberOfDamagedFrigates: str
    mbWaitingForIntervention: bool
    mbExpeditionEnded: bool

class cGcNetworkFleetExpeditionsSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class ComponentInstancercGcMaintenanceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcEggMachineComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcNPCComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class StdVideoH264SpsVuiFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int16),
    ]
    _bf_0: int

class StdVideoH264HrdParameters(ctypes.Structure):
    _fields_ = [
        ("cpb_cnt_minus1", ctypes.c_uint8),
        ("bit_rate_scale", ctypes.c_uint8),
        ("cpb_size_scale", ctypes.c_uint8),
        ("reserved1", ctypes.c_uint8),
        ("bit_rate_value_minus1", ctypes.c_uint32 * 0x20),  # unsigned int[32]
        ("cpb_size_value_minus1", ctypes.c_uint32 * 0x20),  # unsigned int[32]
        ("cbr_flag", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
        ("initial_cpb_removal_delay_length_minus1", ctypes.c_uint32),
        ("cpb_removal_delay_length_minus1", ctypes.c_uint32),
        ("dpb_output_delay_length_minus1", ctypes.c_uint32),
        ("time_offset_length", ctypes.c_uint32),
    ]
    cpb_cnt_minus1: int
    bit_rate_scale: int
    cpb_size_scale: int
    reserved1: int
    bit_rate_value_minus1: list[ctypes.c_uint32]
    cpb_size_value_minus1: list[ctypes.c_uint32]
    cbr_flag: list[ctypes.c_uint8]
    initial_cpb_removal_delay_length_minus1: int
    cpb_removal_delay_length_minus1: int
    dpb_output_delay_length_minus1: int
    time_offset_length: int

class cTkVertexNodeProperties(ctypes.Structure):
    _fields_ = [
        ("mCenter", common.Vector3f),
        ("mfRadius", ctypes.c_float),
        ("mfAmount", ctypes.c_float),
    ]
    mCenter: common.Vector3f
    mfRadius: float
    mfAmount: float

class VkSetStateFlagsIndirectCommandNV(ctypes.Structure):
    _fields_ = [
        ("data", ctypes.c_uint32),
    ]
    data: int

class StdVideoDecodeH264ReferenceInfoFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class StdVideoH264PpsFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class VkAabbPositionsKHR(ctypes.Structure):
    _fields_ = [
        ("minX", ctypes.c_float),
        ("minY", ctypes.c_float),
        ("minZ", ctypes.c_float),
        ("maxX", ctypes.c_float),
        ("maxY", ctypes.c_float),
        ("maxZ", ctypes.c_float),
    ]
    minX: float
    minY: float
    minZ: float
    maxX: float
    maxY: float
    maxZ: float

class VkAccelerationStructureGeometryDataKHR: pass

class VkViewportWScalingNV(ctypes.Structure):
    _fields_ = [
        ("xcoeff", ctypes.c_float),
        ("ycoeff", ctypes.c_float),
    ]
    xcoeff: float
    ycoeff: float

class VkMutableDescriptorTypeListEXT(ctypes.Structure):
    _fields_ = [
        ("descriptorTypeCount", ctypes.c_uint32),
        ("pDescriptorTypes", ctypes.c_ulonglong),
    ]
    descriptorTypeCount: int
    pDescriptorTypes: int

class VkShadingRatePaletteNV(ctypes.Structure):
    _fields_ = [
        ("shadingRatePaletteEntryCount", ctypes.c_uint32),
        ("pShadingRatePaletteEntries", ctypes.c_ulonglong),
    ]
    shadingRatePaletteEntryCount: int
    pShadingRatePaletteEntries: int

class cTkUserServiceWin_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkUserService_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RequestAuthenticationToken", ctypes.c_ulonglong),
        ("TryGetUserIdentity", ctypes.c_ulonglong),
        ("CheckUserPresence", ctypes.c_ulonglong),
        ("GetBranchName", ctypes.c_ulonglong),
        ("GetBuildNumber", ctypes.c_ulonglong),
        ("TryReconnectionIfAvailable", ctypes.c_ulonglong),
        ("GetReconnectionState", ctypes.c_ulonglong),
        ("GetCountryCode", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedNames", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedContent", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedContentUserSpecific", ctypes.c_ulonglong),
        ("AllowedToShareUserGeneratedContent", ctypes.c_ulonglong),
        ("AllowedMultiplayer", ctypes.c_ulonglong),
        ("AllowedCrossplay", ctypes.c_ulonglong),
        ("AllowedToChat", ctypes.c_ulonglong),
        ("AllowedToChatUserSpecific", ctypes.c_ulonglong),
        ("AllowUnfilteredFriendTextChat", ctypes.c_ulonglong),
        ("IsDisplayingSystemDialogs", ctypes.c_ulonglong),
        ("StartUsingUGC", ctypes.c_ulonglong),
        ("StopUsingUGC", ctypes.c_ulonglong),
        ("RequestUserSelection", ctypes.c_ulonglong),
        ("IsUserSelecting", ctypes.c_ulonglong),
        ("RequestUserGamercard", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkUserService *this, const cTkUserIdBase<unsigned __int64>)
        ("GetPlatformId", ctypes.c_ulonglong),
        ("RequiresUserNameLookup", ctypes.c_ulonglong),
        ("GetNameRequestBatchSize", ctypes.c_ulonglong),
        ("DoUserIdBatchLookup", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkUserService *this, std::vector<cTkFixedString<64,char>,TkSTLAllocatorShim<cTkFixedString<64,char>,1,-1> > *, std::function<void __cdecl(std::vector<std::tuple<cTkFixedString<64,char>,cTkFixedString<64,char> >,TkSTLAllocatorShim<std::tuple<cTkFixedString<64,char>,cTkFixedString<64,char> >,1,-1> > &)>)
    ]
    cTkUserService_dtor_0: int
    Construct: int
    Update: int
    Destruct: int
    RequestAuthenticationToken: int
    TryGetUserIdentity: int
    CheckUserPresence: int
    GetBranchName: int
    GetBuildNumber: int
    TryReconnectionIfAvailable: int
    GetReconnectionState: int
    GetCountryCode: int
    AllowedToDisplayUserGeneratedNames: int
    AllowedToDisplayUserGeneratedContent: int
    AllowedToDisplayUserGeneratedContentUserSpecific: int
    AllowedToShareUserGeneratedContent: int
    AllowedMultiplayer: int
    AllowedCrossplay: int
    AllowedToChat: int
    AllowedToChatUserSpecific: int
    AllowUnfilteredFriendTextChat: int
    IsDisplayingSystemDialogs: int
    StartUsingUGC: int
    StopUsingUGC: int
    RequestUserSelection: int
    IsUserSelecting: int
    RequestUserGamercard: bytes
    GetPlatformId: int
    RequiresUserNameLookup: int
    GetNameRequestBatchSize: int
    DoUserIdBatchLookup: bytes

class VkPipelineExecutableStatisticValueKHR: pass

class cTkAnimPoseComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkFakeWin32CondVar(ctypes.Structure):
    _fields_ = [
        ("waiters_count", ctypes.c_int32),
        ("events_", ctypes.c_void_p * 0x1),  # void *[1]
    ]
    waiters_count: int
    events_: list[ctypes.c_void_p]

class VkPastPresentationTimingGOOGLE(ctypes.Structure):
    _fields_ = [
        ("presentID", ctypes.c_uint32),
        ("desiredPresentTime", ctypes.c_uint64),
        ("actualPresentTime", ctypes.c_uint64),
        ("earliestPresentTime", ctypes.c_uint64),
        ("presentMargin", ctypes.c_uint64),
    ]
    presentID: int
    desiredPresentTime: int
    actualPresentTime: int
    earliestPresentTime: int
    presentMargin: int

class VkCoarseSampleLocationNV(ctypes.Structure):
    _fields_ = [
        ("pixelX", ctypes.c_uint32),
        ("pixelY", ctypes.c_uint32),
        ("sample", ctypes.c_uint32),
    ]
    pixelX: int
    pixelY: int
    sample: int

class cTkLineVertexData(ctypes.Structure):
    _fields_ = [
        ("maQuadArray", ctypes.c_ulonglong),
        ("maVertexArray", ctypes.c_ulonglong),
        ("mau16IndexArray", ctypes.c_ulonglong),
        ("miNumVertices", ctypes.c_int32),
        ("miNumIndices", ctypes.c_int32),
        ("miMaxNumVertices", ctypes.c_int32),
    ]
    maQuadArray: int
    maVertexArray: int
    mau16IndexArray: int
    miNumVertices: int
    miNumIndices: int
    miMaxNumVertices: int

class TkClosestRayResultCallback_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class TkAllRayResultCallback_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class cTkMaterialPassThruCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpShapeTagCodec> *(__fastcall *)(hknpShapeTagCodec *this, hkRefPtr<hknpShapeTagCodec> *result)
        ("setMaterialLibrary", ctypes.c_ulonglong),
        ("overrideInitialFilterData", ctypes.c_ulonglong),
        ("decode", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpShapeTagCodec *this, unsigned __int16, hknpCollisionQueryType::Enum, const hknpBody *, const hknpShape *, const hknpShape *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant>, const hknpShape *, hknpQueryFilterData *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    copyFrom: int
    makeClone: bytes
    setMaterialLibrary: int
    overrideInitialFilterData: int
    decode: bytes

class cTkPhysicsError_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("message", ctypes.c_ulonglong),
        ("setEnabled", ctypes.c_ulonglong),
        ("isEnabled", ctypes.c_ulonglong),
        ("setMinimumMessageLevel", ctypes.c_ulonglong),
        ("getMinimumMessageLevel", ctypes.c_ulonglong),
        ("enableAll", ctypes.c_ulonglong),
        ("sectionBegin", ctypes.c_ulonglong),
        ("sectionEnd", ctypes.c_ulonglong),
        ("showMessage", ctypes.c_ulonglong),
        ("formMessage", ctypes.c_ulonglong),
        ("getMessageTypeString", ctypes.c_ulonglong),
        ("shouldReportStackTrace", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    message: int
    setEnabled: int
    isEnabled: int
    setMinimumMessageLevel: int
    getMinimumMessageLevel: int
    enableAll: int
    sectionBegin: int
    sectionEnd: int
    showMessage: int
    formMessage: int
    getMessageTypeString: int
    shouldReportStackTrace: int

class cTkPhysicsFinalError_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("message", ctypes.c_ulonglong),
        ("setEnabled", ctypes.c_ulonglong),
        ("isEnabled", ctypes.c_ulonglong),
        ("setMinimumMessageLevel", ctypes.c_ulonglong),
        ("getMinimumMessageLevel", ctypes.c_ulonglong),
        ("enableAll", ctypes.c_ulonglong),
        ("sectionBegin", ctypes.c_ulonglong),
        ("sectionEnd", ctypes.c_ulonglong),
        ("showMessage", ctypes.c_ulonglong),
        ("formMessage", ctypes.c_ulonglong),
        ("getMessageTypeString", ctypes.c_ulonglong),
        ("shouldReportStackTrace", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    message: int
    setEnabled: int
    isEnabled: int
    setMinimumMessageLevel: int
    getMinimumMessageLevel: int
    enableAll: int
    sectionBegin: int
    sectionEnd: int
    showMessage: int
    formMessage: int
    getMessageTypeString: int
    shouldReportStackTrace: int

class cTkAngularFactorCallback_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class cTkIgnoreBackfacesModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpModifier_dtor_0", ctypes.c_ulonglong),
        ("getEnabledFunctions", ctypes.c_ulonglong),
        ("potentialManifoldCallback", ctypes.c_ulonglong),
        ("manifoldCreatedCallback", ctypes.c_ulonglong),
        ("manifoldProcessCallback", ctypes.c_ulonglong),
        ("manifoldDestroyedCallback", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpBodyIndex *, const hknpBodyIndex *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, const hknpManifoldCollisionCache *)
        ("manifoldFullCastCallback", ctypes.c_ulonglong),
        ("postCompositeQueryAabb", ctypes.c_char * 0x8),  # int (__fastcall *)(hknpModifier *this, const hknpSimulationThreadContext *, const hknpModifierSharedData *, const hknpAabbQuery *, const hknpShapeQueryInfo *, const hknpShapeQueryInfo *, hkHandle<unsigned int,4294967295,hknpShapeKeyDiscriminant> *, int, int)
        ("postContactJacobianSetup", ctypes.c_ulonglong),
        ("postContactJacobianReused", ctypes.c_ulonglong),
        ("postContactSolve", ctypes.c_ulonglong),
        ("postContactImpulseClipped", ctypes.c_ulonglong),
        ("preConstraintJacobianSetup", ctypes.c_ulonglong),
        ("postConstraintExport", ctypes.c_ulonglong),
        ("postParticlesCollidedWithBodies", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("postParticlesCollidedWithParticles", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<hknpParticleCollisionPair> *)
        ("postParticleBodyImpulseApplied", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, int, const hkArrayView<hknpParticleBodyContact> *)
        ("postParticleParticleImpulseApplied", ctypes.c_ulonglong),
        ("postParticlesExitedSafeRegion", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpModifier *this, const hknpParticleSimulationThreadContext *, const hknpParticlesCollider *, const hkArrayView<int> *)
        ("modifyMotionGravity", ctypes.c_ulonglong),
        ("modifyAirDensityAndVelocity", ctypes.c_ulonglong),
        ("modifyMotionIntegration", ctypes.c_ulonglong),
    ]
    hknpModifier_dtor_0: int
    getEnabledFunctions: int
    potentialManifoldCallback: int
    manifoldCreatedCallback: int
    manifoldProcessCallback: int
    manifoldDestroyedCallback: bytes
    manifoldFullCastCallback: int
    postCompositeQueryAabb: bytes
    postContactJacobianSetup: int
    postContactJacobianReused: int
    postContactSolve: int
    postContactImpulseClipped: int
    preConstraintJacobianSetup: int
    postConstraintExport: int
    postParticlesCollidedWithBodies: bytes
    postParticlesCollidedWithParticles: bytes
    postParticleBodyImpulseApplied: bytes
    postParticleParticleImpulseApplied: int
    postParticlesExitedSafeRegion: bytes
    modifyMotionGravity: int
    modifyAirDensityAndVelocity: int
    modifyMotionIntegration: int

class cTkHavokAabbSizeFilter_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterBodyPairs", ctypes.c_ulonglong),
        ("isCollisionEnabled", ctypes.c_ulonglong),
        ("isCollisionEnabled_28", ctypes.c_ulonglong),
        ("isCollisionEnabled_30", ctypes.c_ulonglong),
        ("isCollisionEnabled_38", ctypes.c_ulonglong),
        ("getLodInfo", ctypes.c_ulonglong),
        ("subscribeToWorld", ctypes.c_ulonglong),
        ("unsubscribeFromWorld", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
        ("makeClone", ctypes.c_char * 0x8),  # hkRefPtr<hknpCollisionFilter> *(__fastcall *)(hknpCollisionFilter *this, hkRefPtr<hknpCollisionFilter> *result)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterBodyPairs: int
    isCollisionEnabled: int
    isCollisionEnabled_28: int
    isCollisionEnabled_30: int
    isCollisionEnabled_38: int
    getLodInfo: int
    subscribeToWorld: int
    unsubscribeFromWorld: int
    copyFrom: int
    makeClone: bytes

class cTkThresholdedAnyHitCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class cTkThreadRequest(ctypes.Structure):
    _fields_ = [
        ("mFunction", ctypes.c_ulonglong),
        ("mpData", ctypes.c_void_p),
        ("miFrameDelay", ctypes.c_int32),
        ("mbIsComplete", ctypes.c_ubyte),
    ]
    mFunction: int
    mpData: int
    miFrameDelay: int
    mbIsComplete: bool

class TempMemAllocShim(ctypes.Structure):
    pass

class cGcUpdateOptionalArguments(ctypes.Structure):
    def cGcUpdateOptionalArguments(): pass

    _fields_ = [
        ("mbUseSensitivity", ctypes.c_ubyte),
        ("mbUseMouseDeadZone", ctypes.c_ubyte),
        ("mbIsInBuildingFreeCamera", ctypes.c_ubyte),
    ]
    mbUseSensitivity: bool
    mbUseMouseDeadZone: bool
    mbIsInBuildingFreeCamera: bool

class cTkCharacterBodyAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getBodies", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("applyAction", ctypes.c_ulonglong),
        ("onActionAdded", ctypes.c_ulonglong),
        ("onActionRemoved", ctypes.c_ulonglong),
        ("onWorldShifted", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getBodies: int
    removeBody: int
    applyAction: int
    onActionAdded: int
    onActionRemoved: int
    onWorldShifted: int

class cGcUICameraData(ctypes.Structure):
    _fields_ = [
        ("mpData", ctypes.c_ulonglong),
        ("mfTransTime", ctypes.c_float),
        ("mbInterruptable", ctypes.c_ubyte),
        ("mbSlowTransition", ctypes.c_ubyte),
        ("mbForceWander", ctypes.c_ubyte),
        ("mbUseOffset", ctypes.c_ubyte),
    ]
    mpData: int
    mfTransTime: float
    mbInterruptable: bool
    mbSlowTransition: bool
    mbForceWander: bool
    mbUseOffset: bool

class OnSpawned_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sOnSpawnedType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class OnVariableChanged_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sOnVariableChangedType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class OnBroadcastReceived_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sOnBroadcastReceivedType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class OnComponentEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class cGcCustomisationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class sOnComponentEventType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class IfValueIs_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sIfValueIsType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class ValueAsBool_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sValueAsBoolType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class FixedValue_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sFixedValueType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class GetVariableValue_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sGetVariableValueType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetVariableValue_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSetVariableValueType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SurveyVariableValue_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSurveyVariableValueType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class Sequence_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSequenceType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class WaitSeconds_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sWaitSecondsType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class WaitFrames_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sWaitFramesType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class RandomNumber_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sRandomNumberType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class MathsFunction_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sMathsFunctionType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class BroadcastValueModel_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sBroadcastValueModelType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class BroadcastValueDistance_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sBroadcastValueDistanceType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class RaiseEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sRaiseEventType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetCoreList(ctypes.Structure):
    pass

class OnNodeActivated_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sOnNodeActivatedType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class OnTriggerVolume_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sOnTriggerVolumeType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class OnAnimFrame_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sOnAnimFrameType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetNodeActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class cTkPhysReadJanitor(ctypes.Structure):
    def cTkPhysReadJanitor(): pass

    def tilde_cTkPhysReadJanitor(): pass


class sSetNodeActiveType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class DeleteModel_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sDeleteModelType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class PlayAnim_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sPlayAnimType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class IfAnimIsPlaying_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sIfAnimIsPlayingType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class StopAnim_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sStopAnimType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class PlaySound_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sPlaySoundType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class cTkCharacterBodyListener_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("filterContactPoint", ctypes.c_ulonglong),
        ("onPostCollide", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpCharacterRigidBodyListener *this, hknpCharacterRigidBody *, hknpWorld *, hknpSolverData *, hkArray<hknpCharacterRigidBody::ContactPointInfo,hkBuiltinContainerAllocator<0,0> > *)
        ("onPostSolve", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    filterContactPoint: int
    onPostCollide: bytes
    onPostSolve: int

class StopSounds_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sStopSoundsType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class FadeModel_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sFadeModelType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class TranslateNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sTranslateNodeType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class RotateNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sRotateNodeType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class ScaleNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sScaleNodeType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetLightIntensity_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSetLightIntensityType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetLightColour_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSetLightColourType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetGeneralList(ctypes.Structure):
    pass

class cTkPhysicsSphereProxy(ctypes.Structure):
    _fields_ = [
        ("mpProxy", ctypes.c_ulonglong),
    ]
    mpProxy: int

class VkDispatchIndirectCommand(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_uint32),
        ("y", ctypes.c_uint32),
        ("z", ctypes.c_uint32),
    ]
    x: int
    y: int
    z: int

class IStatVariant(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IStatVariant_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetVariant", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(IStatVariant *this, TkID<128> *result, const TkID<128> *)
    ]
    GetVariant: bytes

class sGcTelemetryStatVariant_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetVariant", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(IStatVariant *this, TkID<128> *result, const TkID<128> *)
    ]
    GetVariant: bytes

class VkDescriptorDataEXT: pass

class StdVideoH265VpsFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class cTkGravityData(ctypes.Structure):
    _fields_ = [
        ("mUp", common.Vector3f),
        ("mfStrength", ctypes.c_float),
    ]
    mUp: common.Vector3f
    mfStrength: float

class cTkEigenMatrix(ctypes.Structure):
    _fields_ = [
        ("miNumCols", ctypes.c_int32),
        ("miNumRows", ctypes.c_int32),
        ("miRowStride", ctypes.c_int32),
        ("mpFloats", ctypes.c_ulonglong),
    ]
    miNumCols: int
    miNumRows: int
    miRowStride: int
    mpFloats: int

class cOrthonormalBasis(ctypes.Structure):
    def ToLocal(): pass

    _fields_ = [
        ("axes", common.Vector4f * 0x3),  # hkVector4f[3]
    ]
    axes: list[common.Vector4f]

class cTkCommonUniformNameHashes(ctypes.Structure):
    pass

class VkDecompressMemoryRegionNV(ctypes.Structure):
    _fields_ = [
        ("srcAddress", ctypes.c_uint64),
        ("dstAddress", ctypes.c_uint64),
        ("compressedSize", ctypes.c_uint64),
        ("decompressedSize", ctypes.c_uint64),
        ("decompressionMethod", ctypes.c_uint64),
    ]
    srcAddress: int
    dstAddress: int
    compressedSize: int
    decompressedSize: int
    decompressionMethod: int

class cGcSpaceMapObject(ctypes.Structure):
    def Update(): pass

    def ChangeResource(): pass

    def Destruct(): pass

    def UpdateRender(): pass

    def Show(): pass

    def Hide(): pass

    def Construct(): pass

    _fields_ = [
        ("mResource", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mNode", ctypes.c_uint32),
        ("maMaterials", ctypes.c_char * 0x18),  # std::vector<cTkSmartResHandle,TkSTLAllocatorShim<cTkSmartResHandle,4,-1> >
        ("mpData", ctypes.c_ulonglong),
        ("mColourOverride", common.Colour),
        ("mbColourOverride", ctypes.c_ubyte),
        ("mfFade", ctypes.c_float),
        ("mfSize", ctypes.c_float),
    ]
    mResource: bytes
    mNode: int
    maMaterials: bytes
    mpData: int
    mColourOverride: common.Colour
    mbColourOverride: bool
    mfFade: float
    mfSize: float

class cGcSpaceMapObjectUtils(ctypes.Structure):
    def ColourMapObject(): pass


class cGcSpaceMap_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcRenderableScreen_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("CleanupResources", ctypes.c_ulonglong),
    ]
    cGcRenderableScreen_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    EnqueueRender: int
    UpdateRender: int
    Release: int
    Destruct: int
    RenderNGui: int
    CleanupResources: int

class cGcFreighterBaseComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cSMShaderConnList(ctypes.Structure):
    _fields_ = [
        ("mSpecs", ctypes.c_ulonglong),
        ("mCount", ctypes.c_int32),
    ]
    mSpecs: int
    mCount: int

class ComponentInstancercTkInstanceWindComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cTkBasicVoxelHelper(ctypes.Structure):
    def GetVoxelColourIndex(): pass

    def GetVoxelTypeFromColourIndex(): pass

    def MakeSubstanceOnlyMask(): pass


class cGcNetworkLargeMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class StdVideoH265DecPicBufMgr(ctypes.Structure):
    _fields_ = [
        ("max_latency_increase_plus1", ctypes.c_uint32 * 0x7),  # unsigned int[7]
        ("max_dec_pic_buffering_minus1", ctypes.c_uint8 * 0x7),  # unsigned __int8[7]
        ("max_num_reorder_pics", ctypes.c_uint8 * 0x7),  # unsigned __int8[7]
    ]
    max_latency_increase_plus1: list[ctypes.c_uint32]
    max_dec_pic_buffering_minus1: list[ctypes.c_uint8]
    max_num_reorder_pics: list[ctypes.c_uint8]

class StdVideoH265ProfileTierLevelFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class StdVideoH265SpsFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int32),
    ]
    _bf_0: int

class StdVideoH265ShortTermRefPicSetFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class StdVideoH265LongTermRefPicsSps(ctypes.Structure):
    _fields_ = [
        ("used_by_curr_pic_lt_sps_flag", ctypes.c_uint32),
        ("lt_ref_pic_poc_lsb_sps", ctypes.c_uint32 * 0x20),  # unsigned int[32]
    ]
    used_by_curr_pic_lt_sps_flag: int
    lt_ref_pic_poc_lsb_sps: list[ctypes.c_uint32]

class cGcWarpInData(ctypes.Structure):
    _fields_ = [
        ("mfTime", ctypes.c_float),
    ]
    mfTime: float

class cGcWarpOutData(ctypes.Structure):
    _fields_ = [
        ("mfTime", ctypes.c_float),
    ]
    mfTime: float

class cGcDestructableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cTkLODSettings(ctypes.Structure):
    def CheckMaxObjectDistanceOverride(): pass

    def GenerateRegionOverrideRadius(): pass

    def GetAdjustedLODDistance(): pass

    def GetAsteroidCountMultiplier(): pass

    def GetAsteroidDividerMultiplier(): pass

    def GetAsteroidFadeRangeMultiplier(): pass

    def GetImposterOverrideDistance(): pass

    def GetImposterResolutionMultiplier(): pass

    def GetMaxAsteroidGenerationPerFrame(): pass

    def GetMaxAsteroidGenerationPerFramePulseJump(): pass

    def GetNumberOfImposterViews(): pass

    def GetOctahedralImpostersEnabled(): pass

    def GetViewImpostersFromSpace(): pass

    def HideRegionOverride(): pass


class cGcShipAIBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cTkProfileMonitor_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkProfileMonitor_dtor_0", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("End", ctypes.c_ulonglong),
        ("Call", ctypes.c_ulonglong),
        ("Return", ctypes.c_ulonglong),
        ("SetIndex", ctypes.c_ulonglong),
        ("FrameData", ctypes.c_ulonglong),
        ("CurrentFrameData", ctypes.c_ulonglong),
        ("LastFrameIndex", ctypes.c_ulonglong),
        ("GetBufferSize", ctypes.c_ulonglong),
    ]
    cTkProfileMonitor_dtor_0: int
    Release: int
    End: int
    Call: int
    Return: int
    SetIndex: int
    FrameData: int
    CurrentFrameData: int
    LastFrameIndex: int
    GetBufferSize: int

class cGcShipAIAlertData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcBlackboard_dtor_0", ctypes.c_ulonglong),
        ("GetIdentifier", ctypes.c_ulonglong),
    ]
    cGcBlackboard_dtor_0: int
    GetIdentifier: int

class cGcShipAIDebugTravel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIWarpingIn_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIWarpingOut_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIEscape_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcWarpDriveData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcBlackboard_dtor_0", ctypes.c_ulonglong),
        ("GetIdentifier", ctypes.c_ulonglong),
    ]
    cGcBlackboard_dtor_0: int
    GetIdentifier: int

class VkDrmFormatModifierPropertiesEXT(ctypes.Structure):
    _fields_ = [
        ("drmFormatModifier", ctypes.c_uint64),
        ("drmFormatModifierPlaneCount", ctypes.c_uint32),
        ("drmFormatModifierTilingFeatures", ctypes.c_uint32),
    ]
    drmFormatModifier: int
    drmFormatModifierPlaneCount: int
    drmFormatModifierTilingFeatures: int

class StdVideoDecodeH265ReferenceInfoFlags(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int8),
    ]
    _bf_0: int

class VkShaderResourceUsageAMD(ctypes.Structure):
    _fields_ = [
        ("numUsedVgprs", ctypes.c_uint32),
        ("numUsedSgprs", ctypes.c_uint32),
        ("ldsSizePerLocalWorkGroup", ctypes.c_uint32),
        ("ldsUsageSizeInBytes", ctypes.c_uint64),
        ("scratchMemUsageInBytes", ctypes.c_uint64),
    ]
    numUsedVgprs: int
    numUsedSgprs: int
    ldsSizePerLocalWorkGroup: int
    ldsUsageSizeInBytes: int
    scratchMemUsageInBytes: int

class VkPerformanceCounterResultKHR: pass

class cTkPhysicsTicket(ctypes.Structure):
    _fields_ = [
        ("miQueueIndex", ctypes.c_int32),
    ]
    miQueueIndex: int

class cTkQuadVertexData(ctypes.Structure):
    pass

class cGcShipAISquadronData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcBlackboard_dtor_0", ctypes.c_ulonglong),
        ("GetIdentifier", ctypes.c_ulonglong),
    ]
    cGcBlackboard_dtor_0: int
    GetIdentifier: int

class VkTraceRaysIndirectCommandKHR(ctypes.Structure):
    _fields_ = [
        ("width", ctypes.c_uint32),
        ("height", ctypes.c_uint32),
        ("depth", ctypes.c_uint32),
    ]
    width: int
    height: int
    depth: int

class VkPerformanceValueDataINTEL: pass

class cTkShaderBinaryHeader(ctypes.Structure):
    _fields_ = [
        ("muType", ctypes.c_uint32),
        ("muCodeSize", ctypes.c_uint32),
        ("muCodeHash", ctypes.c_uint64),
        ("muUniformCount", ctypes.c_uint16),
        ("muInputElementCount", ctypes.c_uint16),
        ("muUniformTotalSize", ctypes.c_uint32),
        ("muUniformTotalSizePerFrame", ctypes.c_uint32),
        ("muPerFrameUniformHash", ctypes.c_uint32),
    ]
    muType: int
    muCodeSize: int
    muCodeHash: int
    muUniformCount: int
    muInputElementCount: int
    muUniformTotalSize: int
    muUniformTotalSizePerFrame: int
    muPerFrameUniformHash: int

class VkMicromapUsageEXT(ctypes.Structure):
    _fields_ = [
        ("count", ctypes.c_uint32),
        ("subdivisionLevel", ctypes.c_uint32),
        ("format", ctypes.c_uint32),
    ]
    count: int
    subdivisionLevel: int
    format: int

class cGcDockingInfo(ctypes.Structure):
    _fields_ = [
        ("mApproachPosition", common.Vector3f),
        ("mfDockDistance", ctypes.c_float),
        ("mfApproachDistance", ctypes.c_float),
        ("mfApproachAngle", ctypes.c_float),
        ("mfApproachFacingAngle", ctypes.c_float),
        ("miSlot", ctypes.c_int32),
    ]
    mApproachPosition: common.Vector3f
    mfDockDistance: float
    mfApproachDistance: float
    mfApproachAngle: float
    mfApproachFacingAngle: float
    miSlot: int

class VkAccelerationStructureMotionInstanceDataNV: pass

class cTkInstanceWindComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class VkPresentTimeGOOGLE(ctypes.Structure):
    _fields_ = [
        ("presentID", ctypes.c_uint32),
        ("desiredPresentTime", ctypes.c_uint64),
    ]
    presentID: int
    desiredPresentTime: int

class SimplexNoiseDerivResult(ctypes.Structure):
    _fields_ = [
        ("_dx", ctypes.c_float),
        ("dy", ctypes.c_float),
        ("dz", ctypes.c_float),
    ]
    _dx: float
    dy: float
    dz: float

class SimplexNoiseResultBuffer(ctypes.Structure):
    _fields_ = [
        ("mpNoiseResults", ctypes.c_ulonglong),
        ("mpDerivResults", ctypes.c_ulonglong),
        ("miSize", ctypes.c_int32),
        ("miUsed", ctypes.c_int32),
    ]
    mpNoiseResults: int
    mpDerivResults: int
    miSize: int
    miUsed: int

class cGcCreatureEggComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class AkResourceMonitorDataSummary(ctypes.Structure):
    _fields_ = [
        ("totalCPU", ctypes.c_float),
        ("pluginCPU", ctypes.c_float),
        ("physicalVoices", ctypes.c_uint32),
        ("virtualVoices", ctypes.c_uint32),
        ("totalVoices", ctypes.c_uint32),
        ("nbActiveEvents", ctypes.c_uint32),
    ]
    totalCPU: float
    pluginCPU: float
    physicalVoices: int
    virtualVoices: int
    totalVoices: int
    nbActiveEvents: int

class VkMultiDrawInfoEXT(ctypes.Structure):
    _fields_ = [
        ("firstVertex", ctypes.c_uint32),
        ("vertexCount", ctypes.c_uint32),
    ]
    firstVertex: int
    vertexCount: int

class VkAccelerationStructureBuildRangeInfoKHR(ctypes.Structure):
    _fields_ = [
        ("primitiveCount", ctypes.c_uint32),
        ("primitiveOffset", ctypes.c_uint32),
        ("firstVertex", ctypes.c_uint32),
        ("transformOffset", ctypes.c_uint32),
    ]
    primitiveCount: int
    primitiveOffset: int
    firstVertex: int
    transformOffset: int

class cTkSingleLineVertex(ctypes.Structure):
    _fields_ = [
        ("mLineA", common.Vector4f),
        ("mLineB", common.Vector4f),
        ("mColour", common.Vector4f),
    ]
    mLineA: common.Vector4f
    mLineB: common.Vector4f
    mColour: common.Vector4f

class VkVertexInputBindingDivisorDescriptionEXT(ctypes.Structure):
    _fields_ = [
        ("binding", ctypes.c_uint32),
        ("divisor", ctypes.c_uint32),
    ]
    binding: int
    divisor: int

class AkPoolNewKey(ctypes.Structure):
    pass

class TGAHeader(ctypes.Structure):
    _fields_ = [
        ("idLength", ctypes.c_char),
        ("colorMapType", ctypes.c_char),
        ("imageType", ctypes.c_char),
        ("colorMapSpec", ctypes.c_char * 0x5),  # char[5]
        ("xOrigin", ctypes.c_wchar),
        ("yOrigin", ctypes.c_wchar),
        ("imageWidth", ctypes.c_wchar),
        ("imageHeight", ctypes.c_wchar),
        ("pixelDepth", ctypes.c_char),
        ("imageDesc", ctypes.c_char),
    ]
    idLength: str
    colorMapType: str
    imageType: str
    colorMapSpec: list[ctypes.c_char]
    xOrigin: str
    yOrigin: str
    imageWidth: str
    imageHeight: str
    pixelDepth: str
    imageDesc: str

class cTkVoxelGeneratorAsteroid(ctypes.Structure):
    def Generate(): pass


class cGcAsteroidTypeMask(ctypes.Structure):
    def cGcAsteroidTypeMask(): pass

    _fields_ = [
        ("mBits", ctypes.c_char * 0x4),  # cTkBitArray<unsigned int,1,32>
    ]
    mBits: bytes

class VkDrawMeshTasksIndirectCommandEXT(ctypes.Structure):
    _fields_ = [
        ("groupCountX", ctypes.c_uint32),
        ("groupCountY", ctypes.c_uint32),
        ("groupCountZ", ctypes.c_uint32),
    ]
    groupCountX: int
    groupCountY: int
    groupCountZ: int

class cTkNoiseUtils(ctypes.Structure):
    def TransformIndependentGaussVectorToMeanAndCovariance(): pass


class SimplexNoiseRequest(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("z", ctypes.c_float),
    ]
    x: float
    y: float
    z: float

class SimplexNoiseRequestBuffer(ctypes.Structure):
    _fields_ = [
        ("mpRequests", ctypes.c_ulonglong),
        ("miSize", ctypes.c_int32),
        ("miUsed", ctypes.c_int32),
        ("mbNeedDerivs", ctypes.c_ubyte),
    ]
    mpRequests: int
    miSize: int
    miUsed: int
    mbNeedDerivs: bool

class VkMicromapTriangleEXT(ctypes.Structure):
    _fields_ = [
        ("dataOffset", ctypes.c_uint32),
        ("subdivisionLevel", ctypes.c_uint16),
        ("format", ctypes.c_uint16),
    ]
    dataOffset: int
    subdivisionLevel: int
    format: int

class cTkSuspendableJobInfo(ctypes.Structure):
    _fields_ = [
        ("mThreadFunction", ctypes.c_ulonglong),
        ("mpData", ctypes.c_void_p),
        ("mStackSize", ctypes.c_uint32),
        ("miQueue", ctypes.c_int32),
    ]
    mThreadFunction: int
    mpData: int
    mStackSize: int
    miQueue: int

class QueueSwitchInfo(ctypes.Structure):
    _fields_ = [
        ("miJobQueue", ctypes.c_int32),
        ("mbWaitForOpenSlot", ctypes.c_ubyte),
        ("mbWakeThreads", ctypes.c_ubyte),
    ]
    miJobQueue: int
    mbWaitForOpenSlot: bool
    mbWakeThreads: bool

class cTkSuspendableJobResumeInfo(ctypes.Structure):
    _fields_ = [
        ("mpFiber", ctypes.c_void_p),
        ("miQueue", ctypes.c_int32),
    ]
    mpFiber: int
    miQueue: int

class VkMultiDrawIndexedInfoEXT(ctypes.Structure):
    _fields_ = [
        ("firstIndex", ctypes.c_uint32),
        ("indexCount", ctypes.c_uint32),
        ("vertexOffset", ctypes.c_int32),
    ]
    firstIndex: int
    indexCount: int
    vertexOffset: int

class cTkTrophyManagerBase(ctypes.Structure):
    def tilde_cTkTrophyManagerBase(): pass

    def scalar_deleting_destructor(): pass

    def IsTrophyUnlocked(): pass

    def UnlockTrophy(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mUnlockThreadId", ctypes.c_uint32),
        ("miMaxNumTrophies", ctypes.c_int32),
        ("mpabTrophiesToUnlock", ctypes.c_ulonglong),
        ("maTrophiesToUnlock", ctypes.c_char * 0x18),  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> >
        ("maTrophiesToUnlockThreadData", ctypes.c_char * 0x18),  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> >
    ]
    __vftable: int
    mUnlockThreadId: int
    miMaxNumTrophies: int
    mpabTrophiesToUnlock: int
    maTrophiesToUnlock: bytes
    maTrophiesToUnlockThreadData: bytes

class cTkTrophyManagerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTrophyManagerBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsTrophyUnlocked", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkTrophyManagerBase *this, const TkID<128> *)
        ("GetUnlockedTrophies", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkTrophyManagerBase *this, std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *)
        ("BatchUnlockTrophies", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkTrophyManagerBase *this, std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *)
    ]
    cTkTrophyManagerBase_dtor_0: int
    Construct: int
    Destruct: int
    Prepare: int
    Release: int
    Update: int
    IsTrophyUnlocked: bytes
    GetUnlockedTrophies: bytes
    BatchUnlockTrophies: bytes

class cTkTrophyManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTrophyManagerBase_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsTrophyUnlocked", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkTrophyManagerBase *this, const TkID<128> *)
        ("GetUnlockedTrophies", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkTrophyManagerBase *this, std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *)
        ("BatchUnlockTrophies", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkTrophyManagerBase *this, std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> > *)
    ]
    cTkTrophyManagerBase_dtor_0: int
    Construct: int
    Destruct: int
    Prepare: int
    Release: int
    Update: int
    IsTrophyUnlocked: bytes
    GetUnlockedTrophies: bytes
    BatchUnlockTrophies: bytes

class cTkProfanityFilterTpfs_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("AvailableForUse", ctypes.c_ulonglong),
        ("IsThisNaughty", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkProfanityFilter *this, const char *, std::function<void __cdecl(bool,char const *)>)
        ("IsThisNaughty_28", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkProfanityFilter *this, const char *, std::function<void __cdecl(bool)>)
        ("cTkProfanityFilter_dtor_30", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    Destruct: int
    AvailableForUse: int
    IsThisNaughty: bytes
    IsThisNaughty_28: bytes
    cTkProfanityFilter_dtor_30: int

class cTkThreadRequestManager(ctypes.Structure):
    _fields_ = [
        ("mRequests", ctypes.c_char * 0x18),  # std::vector<cTkThreadRequest,TkSTLAllocatorShim<cTkThreadRequest,8,-1> >
        ("mFreeList", ctypes.c_char * 0x18),  # std::vector<int,TkSTLAllocatorShim<int,4,-1> >
        ("miActiveRequests", ctypes.c_int32),
    ]
    mRequests: bytes
    mFreeList: bytes
    miActiveRequests: int

class VkCopyMemoryIndirectCommandNV(ctypes.Structure):
    _fields_ = [
        ("srcAddress", ctypes.c_uint64),
        ("dstAddress", ctypes.c_uint64),
        ("size", ctypes.c_uint64),
    ]
    srcAddress: int
    dstAddress: int
    size: int

class AkGrowByPolicy_NoGrow(ctypes.Structure):
    pass

class AkGrowByPolicy_Legacy(ctypes.Structure):
    pass

class cTkAABB2D(ctypes.Structure):
    _fields_ = [
        ("mMin", common.Vector2f),
        ("mMax", common.Vector2f),
    ]
    mMin: common.Vector2f
    mMax: common.Vector2f

class cTkMultipleReadersLockWriteJanitor(ctypes.Structure):
    def cTkMultipleReadersLockWriteJanitor(): pass

    _fields_ = [
        ("mLock", ctypes.c_ulonglong),
    ]
    mLock: int

class cTkVoxelGeneratorSine(ctypes.Structure):
    pass

class VkDrawIndirectCommand(ctypes.Structure):
    _fields_ = [
        ("vertexCount", ctypes.c_uint32),
        ("instanceCount", ctypes.c_uint32),
        ("firstVertex", ctypes.c_uint32),
        ("firstInstance", ctypes.c_uint32),
    ]
    vertexCount: int
    instanceCount: int
    firstVertex: int
    firstInstance: int

class ComponentInstancercTkCreatureTailComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cTkSuspendedJobData(ctypes.Structure):
    _fields_ = [
        ("mpFunction", ctypes.c_ulonglong),
        ("mpData", ctypes.c_void_p),
    ]
    mpFunction: int
    mpData: int

class _SHFILEOPSTRUCTW(ctypes.Structure):
    _fields_ = [
        ("hwnd", ctypes.c_ulonglong),
        ("wFunc", ctypes.c_uint32),
        ("pFrom", ctypes.c_ulonglong),
        ("pTo", ctypes.c_ulonglong),
        ("fFlags", ctypes.c_uint16),
        ("fAnyOperationsAborted", ctypes.c_int32),
        ("hNameMappings", ctypes.c_void_p),
        ("lpszProgressTitle", ctypes.c_ulonglong),
    ]
    hwnd: int
    wFunc: int
    pFrom: int
    pTo: int
    fFlags: int
    fAnyOperationsAborted: int
    hNameMappings: int
    lpszProgressTitle: int

class cGcGameSaveMetaData1(ctypes.Structure):
    _fields_ = [
        ("mVersion", ctypes.c_uint32),
        ("mGameMode", ctypes.c_uint16),
        ("mu16SeasonNumber", ctypes.c_uint16),
        ("mPlayTime", ctypes.c_uint64),
        ("muDataSize", ctypes.c_uint32),
        ("macSaveName", ctypes.c_char * 0x80),  # char[128]
        ("macSaveSummary", ctypes.c_char * 0x80),  # char[128]
        ("mu8DifficultyPreset", ctypes.c_uint8),
    ]
    mVersion: int
    mGameMode: int
    mu16SeasonNumber: int
    mPlayTime: int
    muDataSize: int
    macSaveName: list[ctypes.c_char]
    macSaveSummary: list[ctypes.c_char]
    mu8DifficultyPreset: int

class cTkUserServiceGalaxy_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkUserService_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RequestAuthenticationToken", ctypes.c_ulonglong),
        ("TryGetUserIdentity", ctypes.c_ulonglong),
        ("CheckUserPresence", ctypes.c_ulonglong),
        ("GetBranchName", ctypes.c_ulonglong),
        ("GetBuildNumber", ctypes.c_ulonglong),
        ("TryReconnectionIfAvailable", ctypes.c_ulonglong),
        ("GetReconnectionState", ctypes.c_ulonglong),
        ("GetCountryCode", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedNames", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedContent", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedContentUserSpecific", ctypes.c_ulonglong),
        ("AllowedToShareUserGeneratedContent", ctypes.c_ulonglong),
        ("AllowedMultiplayer", ctypes.c_ulonglong),
        ("AllowedCrossplay", ctypes.c_ulonglong),
        ("AllowedToChat", ctypes.c_ulonglong),
        ("AllowedToChatUserSpecific", ctypes.c_ulonglong),
        ("AllowUnfilteredFriendTextChat", ctypes.c_ulonglong),
        ("IsDisplayingSystemDialogs", ctypes.c_ulonglong),
        ("StartUsingUGC", ctypes.c_ulonglong),
        ("StopUsingUGC", ctypes.c_ulonglong),
        ("RequestUserSelection", ctypes.c_ulonglong),
        ("IsUserSelecting", ctypes.c_ulonglong),
        ("RequestUserGamercard", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkUserService *this, const cTkUserIdBase<unsigned __int64>)
        ("GetPlatformId", ctypes.c_ulonglong),
        ("RequiresUserNameLookup", ctypes.c_ulonglong),
        ("GetNameRequestBatchSize", ctypes.c_ulonglong),
        ("DoUserIdBatchLookup", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkUserService *this, std::vector<cTkFixedString<64,char>,TkSTLAllocatorShim<cTkFixedString<64,char>,1,-1> > *, std::function<void __cdecl(std::vector<std::tuple<cTkFixedString<64,char>,cTkFixedString<64,char> >,TkSTLAllocatorShim<std::tuple<cTkFixedString<64,char>,cTkFixedString<64,char> >,1,-1> > &)>)
    ]
    cTkUserService_dtor_0: int
    Construct: int
    Update: int
    Destruct: int
    RequestAuthenticationToken: int
    TryGetUserIdentity: int
    CheckUserPresence: int
    GetBranchName: int
    GetBuildNumber: int
    TryReconnectionIfAvailable: int
    GetReconnectionState: int
    GetCountryCode: int
    AllowedToDisplayUserGeneratedNames: int
    AllowedToDisplayUserGeneratedContent: int
    AllowedToDisplayUserGeneratedContentUserSpecific: int
    AllowedToShareUserGeneratedContent: int
    AllowedMultiplayer: int
    AllowedCrossplay: int
    AllowedToChat: int
    AllowedToChatUserSpecific: int
    AllowUnfilteredFriendTextChat: int
    IsDisplayingSystemDialogs: int
    StartUsingUGC: int
    StopUsingUGC: int
    RequestUserSelection: int
    IsUserSelecting: int
    RequestUserGamercard: bytes
    GetPlatformId: int
    RequiresUserNameLookup: int
    GetNameRequestBatchSize: int
    DoUserIdBatchLookup: bytes

class cTkUserServiceNull_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkUserService_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RequestAuthenticationToken", ctypes.c_ulonglong),
        ("TryGetUserIdentity", ctypes.c_ulonglong),
        ("CheckUserPresence", ctypes.c_ulonglong),
        ("GetBranchName", ctypes.c_ulonglong),
        ("GetBuildNumber", ctypes.c_ulonglong),
        ("TryReconnectionIfAvailable", ctypes.c_ulonglong),
        ("GetReconnectionState", ctypes.c_ulonglong),
        ("GetCountryCode", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedNames", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedContent", ctypes.c_ulonglong),
        ("AllowedToDisplayUserGeneratedContentUserSpecific", ctypes.c_ulonglong),
        ("AllowedToShareUserGeneratedContent", ctypes.c_ulonglong),
        ("AllowedMultiplayer", ctypes.c_ulonglong),
        ("AllowedCrossplay", ctypes.c_ulonglong),
        ("AllowedToChat", ctypes.c_ulonglong),
        ("AllowedToChatUserSpecific", ctypes.c_ulonglong),
        ("AllowUnfilteredFriendTextChat", ctypes.c_ulonglong),
        ("IsDisplayingSystemDialogs", ctypes.c_ulonglong),
        ("StartUsingUGC", ctypes.c_ulonglong),
        ("StopUsingUGC", ctypes.c_ulonglong),
        ("RequestUserSelection", ctypes.c_ulonglong),
        ("IsUserSelecting", ctypes.c_ulonglong),
        ("RequestUserGamercard", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkUserService *this, const cTkUserIdBase<unsigned __int64>)
        ("GetPlatformId", ctypes.c_ulonglong),
        ("RequiresUserNameLookup", ctypes.c_ulonglong),
        ("GetNameRequestBatchSize", ctypes.c_ulonglong),
        ("DoUserIdBatchLookup", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkUserService *this, std::vector<cTkFixedString<64,char>,TkSTLAllocatorShim<cTkFixedString<64,char>,1,-1> > *, std::function<void __cdecl(std::vector<std::tuple<cTkFixedString<64,char>,cTkFixedString<64,char> >,TkSTLAllocatorShim<std::tuple<cTkFixedString<64,char>,cTkFixedString<64,char> >,1,-1> > &)>)
    ]
    cTkUserService_dtor_0: int
    Construct: int
    Update: int
    Destruct: int
    RequestAuthenticationToken: int
    TryGetUserIdentity: int
    CheckUserPresence: int
    GetBranchName: int
    GetBuildNumber: int
    TryReconnectionIfAvailable: int
    GetReconnectionState: int
    GetCountryCode: int
    AllowedToDisplayUserGeneratedNames: int
    AllowedToDisplayUserGeneratedContent: int
    AllowedToDisplayUserGeneratedContentUserSpecific: int
    AllowedToShareUserGeneratedContent: int
    AllowedMultiplayer: int
    AllowedCrossplay: int
    AllowedToChat: int
    AllowedToChatUserSpecific: int
    AllowUnfilteredFriendTextChat: int
    IsDisplayingSystemDialogs: int
    StartUsingUGC: int
    StopUsingUGC: int
    RequestUserSelection: int
    IsUserSelecting: int
    RequestUserGamercard: bytes
    GetPlatformId: int
    RequiresUserNameLookup: int
    GetNameRequestBatchSize: int
    DoUserIdBatchLookup: bytes

class VkTraceRaysIndirectCommand2KHR(ctypes.Structure):
    _fields_ = [
        ("raygenShaderRecordAddress", ctypes.c_uint64),
        ("raygenShaderRecordSize", ctypes.c_uint64),
        ("missShaderBindingTableAddress", ctypes.c_uint64),
        ("missShaderBindingTableSize", ctypes.c_uint64),
        ("missShaderBindingTableStride", ctypes.c_uint64),
        ("hitShaderBindingTableAddress", ctypes.c_uint64),
        ("hitShaderBindingTableSize", ctypes.c_uint64),
        ("hitShaderBindingTableStride", ctypes.c_uint64),
        ("callableShaderBindingTableAddress", ctypes.c_uint64),
        ("callableShaderBindingTableSize", ctypes.c_uint64),
        ("callableShaderBindingTableStride", ctypes.c_uint64),
        ("width", ctypes.c_uint32),
        ("height", ctypes.c_uint32),
        ("depth", ctypes.c_uint32),
    ]
    raygenShaderRecordAddress: int
    raygenShaderRecordSize: int
    missShaderBindingTableAddress: int
    missShaderBindingTableSize: int
    missShaderBindingTableStride: int
    hitShaderBindingTableAddress: int
    hitShaderBindingTableSize: int
    hitShaderBindingTableStride: int
    callableShaderBindingTableAddress: int
    callableShaderBindingTableSize: int
    callableShaderBindingTableStride: int
    width: int
    height: int
    depth: int

class md5_state_s(ctypes.Structure):
    _fields_ = [
        ("count", ctypes.c_uint32 * 0x2),  # unsigned int[2]
        ("abcd", ctypes.c_uint32 * 0x4),  # unsigned int[4]
        ("buf", ctypes.c_uint8 * 0x40),  # unsigned __int8[64]
    ]
    count: list[ctypes.c_uint32]
    abcd: list[ctypes.c_uint32]
    buf: list[ctypes.c_uint8]

class cGcRecipeWikiData(ctypes.Structure):
    def cGcRecipeWikiData(): pass

    _fields_ = [
        ("mRecipeId", ctypes.c_ulonglong),
        ("mOutputID", ctypes.c_ulonglong),
        ("mRecipeName", ctypes.c_ulonglong),
        ("mRecipeNameLower", ctypes.c_ulonglong),
        ("mRecipeSubtitle", ctypes.c_ulonglong),
        ("mRecipeDescription", ctypes.c_ulonglong),
        ("mOutputColour", common.Colour),
        ("mOutputResource", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mOutputValue", ctypes.c_int32),
        ("mbIsKnown", ctypes.c_ubyte),
    ]
    mRecipeId: int
    mOutputID: int
    mRecipeName: int
    mRecipeNameLower: int
    mRecipeSubtitle: int
    mRecipeDescription: int
    mOutputColour: common.Colour
    mOutputResource: bytes
    mOutputValue: int
    mbIsKnown: bool

class cGcStoneRuneData(ctypes.Structure):
    def cGcStoneRuneData(): pass

    _fields_ = [
        ("miIdx", ctypes.c_int32),
        ("macName", ctypes.c_ulonglong),
        ("macNameLower", ctypes.c_ulonglong),
        ("macSubtitle", ctypes.c_ulonglong),
        ("macDescription", ctypes.c_ulonglong),
        ("mpColour", ctypes.c_ulonglong),
        ("mResource", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
    ]
    miIdx: int
    macName: int
    macNameLower: int
    macSubtitle: int
    macDescription: int
    mpColour: int
    mResource: bytes

class cGcPhotoModeAdjuster(ctypes.Structure):
    pass

class cTkMultipleReadersLockReadJanitor(ctypes.Structure):
    def cTkMultipleReadersLockReadJanitor(): pass

    _fields_ = [
        ("mLock", ctypes.c_ulonglong),
    ]
    mLock: int

class PhotoModeControl_TimeOfDay_vtbl(ctypes.Structure):
    _fields_ = [
        ("PhotoModeControl_dtor_0", ctypes.c_ulonglong),
        ("GetValueAsString", ctypes.c_char * 0x8),  # void (__fastcall *)(PhotoModeControl *this, cTkFixedString<512,char> *)
        ("Reset", ctypes.c_ulonglong),
        ("Increase", ctypes.c_ulonglong),
        ("Decrease", ctypes.c_ulonglong),
        ("EnsureInBounds", ctypes.c_ulonglong),
        ("IsValid", ctypes.c_ulonglong),
    ]
    PhotoModeControl_dtor_0: int
    GetValueAsString: bytes
    Reset: int
    Increase: int
    Decrease: int
    EnsureInBounds: int
    IsValid: int

class PhotoModeControl_List_vtbl(ctypes.Structure):
    _fields_ = [
        ("PhotoModeControl_dtor_0", ctypes.c_ulonglong),
        ("GetValueAsString", ctypes.c_char * 0x8),  # void (__fastcall *)(PhotoModeControl *this, cTkFixedString<512,char> *)
        ("Reset", ctypes.c_ulonglong),
        ("Increase", ctypes.c_ulonglong),
        ("Decrease", ctypes.c_ulonglong),
        ("EnsureInBounds", ctypes.c_ulonglong),
        ("IsValid", ctypes.c_ulonglong),
    ]
    PhotoModeControl_dtor_0: int
    GetValueAsString: bytes
    Reset: int
    Increase: int
    Decrease: int
    EnsureInBounds: int
    IsValid: int

class VkBindShaderGroupIndirectCommandNV(ctypes.Structure):
    _fields_ = [
        ("groupIndex", ctypes.c_uint32),
    ]
    groupIndex: int

class cTkPerlinNoiseFunctions(ctypes.Structure):
    pass

class VkRenderPassCreationFeedbackInfoEXT(ctypes.Structure):
    _fields_ = [
        ("postMergeSubpassCount", ctypes.c_uint32),
    ]
    postMergeSubpassCount: int

class VkDrawMeshTasksIndirectCommandNV(ctypes.Structure):
    _fields_ = [
        ("taskCount", ctypes.c_uint32),
        ("firstTask", ctypes.c_uint32),
    ]
    taskCount: int
    firstTask: int

class AkNonThreaded(ctypes.Structure):
    pass

class VkBindVertexBufferIndirectCommandNV(ctypes.Structure):
    _fields_ = [
        ("bufferAddress", ctypes.c_uint64),
        ("size", ctypes.c_uint32),
        ("stride", ctypes.c_uint32),
    ]
    bufferAddress: int
    size: int
    stride: int

class VkDrmFormatModifierProperties2EXT(ctypes.Structure):
    _fields_ = [
        ("drmFormatModifier", ctypes.c_uint64),
        ("drmFormatModifierPlaneCount", ctypes.c_uint32),
        ("drmFormatModifierTilingFeatures", ctypes.c_uint64),
    ]
    drmFormatModifier: int
    drmFormatModifierPlaneCount: int
    drmFormatModifierTilingFeatures: int

class cTkGpuPerformanceSample(ctypes.Structure):
    _fields_ = [
        ("mpData", ctypes.c_ulonglong),
        ("muQueryId", ctypes.c_uint32),
    ]
    mpData: int
    muQueryId: int

class AkCompressorFXParams(ctypes.Structure):
    _fields_ = [
        ("fThreshold", ctypes.c_float),
        ("fRatio", ctypes.c_float),
        ("fAttack", ctypes.c_float),
        ("fRelease", ctypes.c_float),
        ("fOutputLevel", ctypes.c_float),
        ("bProcessLFE", ctypes.c_ubyte),
        ("bChannelLink", ctypes.c_ubyte),
    ]
    fThreshold: float
    fRatio: float
    fAttack: float
    fRelease: float
    fOutputLevel: float
    bProcessLFE: bool
    bChannelLink: bool

class CAkCompressorFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class TextureInfo(ctypes.Structure):
    _fields_ = [
        ("mpPixelData", ctypes.c_void_p),
        ("miWidth", ctypes.c_int32),
        ("miHeight", ctypes.c_int32),
        ("miComps", ctypes.c_int32),
        ("mbHdr", ctypes.c_ubyte),
        ("mbAllowCompression", ctypes.c_ubyte),
        ("mbMipmaps", ctypes.c_ubyte),
        ("mbFiltering", ctypes.c_ubyte),
        ("mbRepeatMode", ctypes.c_ubyte),
    ]
    mpPixelData: int
    miWidth: int
    miHeight: int
    miComps: int
    mbHdr: bool
    mbAllowCompression: bool
    mbMipmaps: bool
    mbFiltering: bool
    mbRepeatMode: bool

class cGcBoid(ctypes.Structure):
    _fields_ = [
        ("mNode", ctypes.c_uint32),
        ("mVelocity", common.Vector3f),
    ]
    mNode: int
    mVelocity: common.Vector3f

class AKSIMD_V4I32X4(ctypes.Structure):
    _fields_ = [
        ("val", common.Vector4i * 0x4),  # __m128i[4]
    ]
    val: list[common.Vector4i]

class CAkBaseConvolutionEngine_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkBaseConvolutionEngine_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("InitConvolutionInstance", ctypes.c_ulonglong),
        ("GetExecuteBufferAlignment", ctypes.c_ulonglong),
        ("ExecuteChannels", ctypes.c_ulonglong),
    ]
    CAkBaseConvolutionEngine_dtor_0: int
    Init: int
    Term: int
    Reset: int
    RelocateMedia: int
    InitConvolutionInstance: int
    GetExecuteBufferAlignment: int
    ExecuteChannels: int

class CAkVectorValueRampV4(ctypes.Structure):
    _fields_ = [
        ("vIncrement", common.Vector4f),
        ("vValueRamp", common.Vector4f),
    ]
    vIncrement: common.Vector4f
    vValueRamp: common.Vector4f

class CAkPartitionedConvolutionEngine_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkBaseConvolutionEngine_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("InitConvolutionInstance", ctypes.c_ulonglong),
        ("GetExecuteBufferAlignment", ctypes.c_ulonglong),
        ("ExecuteChannels", ctypes.c_ulonglong),
    ]
    CAkBaseConvolutionEngine_dtor_0: int
    Init: int
    Term: int
    Reset: int
    RelocateMedia: int
    InitConvolutionInstance: int
    GetExecuteBufferAlignment: int
    ExecuteChannels: int

class AKSIMD_V4I32X2(ctypes.Structure):
    _fields_ = [
        ("val", common.Vector4i * 0x2),  # __m128i[2]
    ]
    val: list[common.Vector4i]

class CAkHalfPrecisionConvolutionEngine_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkBaseConvolutionEngine_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("InitConvolutionInstance", ctypes.c_ulonglong),
        ("GetExecuteBufferAlignment", ctypes.c_ulonglong),
        ("ExecuteChannels", ctypes.c_ulonglong),
    ]
    CAkBaseConvolutionEngine_dtor_0: int
    Init: int
    Term: int
    Reset: int
    RelocateMedia: int
    InitConvolutionInstance: int
    GetExecuteBufferAlignment: int
    ExecuteChannels: int

class AkCompressorSideChain(ctypes.Structure):
    _fields_ = [
        ("fGainDb", ctypes.c_float),
        ("fMem", ctypes.c_float),
    ]
    fGainDb: float
    fMem: float

class AkFXSrcAudioInputParams(ctypes.Structure):
    _fields_ = [
        ("fGain", ctypes.c_float),
    ]
    fGain: float

class AkDelayRTPCParams(ctypes.Structure):
    _fields_ = [
        ("fFeedback", ctypes.c_float),
        ("fWetDryMix", ctypes.c_float),
        ("fOutputLevel", ctypes.c_float),
        ("bFeedbackEnabled", ctypes.c_ubyte),
        ("bHasChanged", ctypes.c_ubyte),
    ]
    fFeedback: float
    fWetDryMix: float
    fOutputLevel: float
    bFeedbackEnabled: bool
    bHasChanged: bool

class AkDelayNonRTPCParams(ctypes.Structure):
    _fields_ = [
        ("fDelayTime", ctypes.c_float),
        ("bProcessLFE", ctypes.c_ubyte),
        ("bHasChanged", ctypes.c_ubyte),
    ]
    fDelayTime: float
    bProcessLFE: bool
    bHasChanged: bool

class CAkFxSrcAudioInputParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkDelayFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class AkFXTailHandler(ctypes.Structure):
    _fields_ = [
        ("uTailFramesRemaining", ctypes.c_uint32),
        ("uTotalTailFrames", ctypes.c_uint32),
    ]
    uTailFramesRemaining: int
    uTotalTailFrames: int

class CAkDelayFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class CAkConvolutionReverbFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkConvolutionReverbFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class AkChunkHeader(ctypes.Structure):
    _fields_ = [
        ("ChunkId", ctypes.c_uint32),
        ("dwChunkSize", ctypes.c_uint32),
    ]
    ChunkId: int
    dwChunkSize: int

class WaveFormatEx(ctypes.Structure):
    _fields_ = [
        ("wFormatTag", ctypes.c_uint16),
        ("nChannels", ctypes.c_uint16),
        ("nSamplesPerSec", ctypes.c_uint32),
        ("nAvgBytesPerSec", ctypes.c_uint32),
        ("nBlockAlign", ctypes.c_uint16),
        ("wBitsPerSample", ctypes.c_uint16),
        ("cbSize", ctypes.c_uint16),
    ]
    wFormatTag: int
    nChannels: int
    nSamplesPerSec: int
    nAvgBytesPerSec: int
    nBlockAlign: int
    wBitsPerSample: int
    cbSize: int

class IAkCaptureWriter(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IAkCaptureWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("StartCapture", ctypes.c_ulonglong),
        ("StopCapture", ctypes.c_ulonglong),
        ("PassAudioData", ctypes.c_ulonglong),
        ("WriteGenericData", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
    ]
    StartCapture: int
    StopCapture: int
    PassAudioData: int
    WriteGenericData: int
    Destroy: int

class cGcEncyclopediaComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class AkGainFXParams(ctypes.Structure):
    _fields_ = [
        ("fFullbandGain", ctypes.c_float),
        ("fLFEGain", ctypes.c_float),
    ]
    fFullbandGain: float
    fLFEGain: float

class CAkGainFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkGainFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class CAkGuitarDistortionFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkGuitarDistortionFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class CAkHarmonizerFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkHarmonizerFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class AkSpatialAudioID(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint64),
    ]
    id: int

class AkFDNReverbNonRTPCParams(ctypes.Structure):
    _fields_ = [
        ("uNumberOfDelays", ctypes.c_uint32),
        ("fPreDelay", ctypes.c_float),
        ("uProcessLFE", ctypes.c_uint32),
        ("uDelayLengthsMode", ctypes.c_uint32),
        ("fDelayTime", ctypes.c_float * 0x10),  # float[16]
        ("bDirty", ctypes.c_ubyte),
    ]
    uNumberOfDelays: int
    fPreDelay: float
    uProcessLFE: int
    uDelayLengthsMode: int
    fDelayTime: list[ctypes.c_float]
    bDirty: bool

class AkFDNReverbRTPCParams(ctypes.Structure):
    _fields_ = [
        ("fReverbTime", ctypes.c_float),
        ("fHFRatio", ctypes.c_float),
        ("fDryLevel", ctypes.c_float),
        ("fWetLevel", ctypes.c_float),
    ]
    fReverbTime: float
    fHFRatio: float
    fDryLevel: float
    fWetLevel: float

class CAkFDNReverbFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkFDNReverbFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class AkSourcePosition(ctypes.Structure):
    _fields_ = [
        ("audioNodeID", ctypes.c_uint32),
        ("mediaID", ctypes.c_uint32),
        ("msTime", ctypes.c_int32),
        ("samplePosition", ctypes.c_uint32),
        ("updateBufferTick", ctypes.c_uint32),
    ]
    audioNodeID: int
    mediaID: int
    msTime: int
    samplePosition: int
    updateBufferTick: int

class AkStateTransition(ctypes.Structure):
    _fields_ = [
        ("StateFrom", ctypes.c_uint32),
        ("StateTo", ctypes.c_uint32),
    ]
    StateFrom: int
    StateTo: int

class AkStateGroupInfo(ctypes.Structure):
    _fields_ = [
        ("lDefaultTransitionTime", ctypes.c_int32),
        ("ActualState", ctypes.c_uint32),
        ("members", ctypes.c_char * 0x10),  # AkListBareLight<AkStateGroupChunk,AkListBareLightNextInGroup>
        ("mapTransitions", ctypes.c_char * 0x10),  # CAkKeyArray<AkStateTransition,int,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<AkStateTransition,int> > >
    ]
    lDefaultTransitionTime: int
    ActualState: int
    members: bytes
    mapTransitions: bytes

class CAkPreparedContent(ctypes.Structure):
    _fields_ = [
        ("m_PreparableContentList", ctypes.c_char * 0x10),  # AkArray<unsigned int,unsigned int,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int> >
    ]
    m_PreparableContentList: bytes

class CAkPreparationAware(ctypes.Structure):
    def GetPreparedContent(): pass

    def SubscribePrepare(): pass

    def UnsubscribePrepare(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("pNextItemPrepare", ctypes.c_ulonglong),
        ("m_uPreparationCount", ctypes.c_uint32),
    ]
    __vftable: int
    pNextItemPrepare: int
    m_uPreparationCount: int

class CAkPreparationAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("ModifyActiveState", ctypes.c_ulonglong),
    ]
    ModifyActiveState: int

class AkMeterRTPCParams(ctypes.Structure):
    _fields_ = [
        ("fAttack", ctypes.c_float),
        ("fRelease", ctypes.c_float),
        ("fMin", ctypes.c_float),
        ("fMax", ctypes.c_float),
        ("fHold", ctypes.c_float),
    ]
    fAttack: float
    fRelease: float
    fMin: float
    fMax: float
    fHold: float

class CAkMeterFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class AkMeterState(ctypes.Structure):
    _fields_ = [
        ("uSampleRate", ctypes.c_uint32),
        ("fHoldTime", ctypes.c_float),
        ("fReleaseTarget", ctypes.c_float),
        ("fLastValue", ctypes.c_float),
        ("fHoldMemory", ctypes.c_float * 0x8),  # float[8]
    ]
    uSampleRate: int
    fHoldTime: float
    fReleaseTarget: float
    fLastValue: float
    fHoldMemory: list[ctypes.c_float]

class CAkMeterFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int

class AkTriangle(ctypes.Structure):
    _fields_ = [
        ("point0", ctypes.c_uint16),
        ("point1", ctypes.c_uint16),
        ("point2", ctypes.c_uint16),
        ("surface", ctypes.c_uint16),
    ]
    point0: int
    point1: int
    point2: int
    surface: int

class AkModulatorGain(ctypes.Structure):
    _fields_ = [
        ("m_fStart", ctypes.c_float),
        ("m_fEnd", ctypes.c_float),
    ]
    m_fStart: float
    m_fEnd: float

class AkVertex(ctypes.Structure):
    _fields_ = [
        ("X", ctypes.c_float),
        ("Y", ctypes.c_float),
        ("Z", ctypes.c_float),
    ]
    X: float
    Y: float
    Z: float

class AkAcousticSurface(ctypes.Structure):
    _fields_ = [
        ("textureID", ctypes.c_uint32),
        ("transmissionLoss", ctypes.c_float),
        ("strName", ctypes.c_char_p),
    ]
    textureID: int
    transmissionLoss: float
    strName: bytes

class AkSpatialAudioInitSettings(ctypes.Structure):
    _fields_ = [
        ("uMaxSoundPropagationDepth", ctypes.c_uint32),
        ("fMovementThreshold", ctypes.c_float),
        ("uNumberOfPrimaryRays", ctypes.c_uint32),
        ("uMaxReflectionOrder", ctypes.c_uint32),
        ("fMaxPathLength", ctypes.c_float),
        ("fCPULimitPercentage", ctypes.c_float),
        ("bEnableDiffractionOnReflection", ctypes.c_ubyte),
        ("bEnableGeometricDiffractionAndTransmission", ctypes.c_ubyte),
        ("bCalcEmitterVirtualPosition", ctypes.c_ubyte),
        ("bUseObstruction", ctypes.c_ubyte),
        ("bUseOcclusion", ctypes.c_ubyte),
    ]
    uMaxSoundPropagationDepth: int
    fMovementThreshold: float
    uNumberOfPrimaryRays: int
    uMaxReflectionOrder: int
    fMaxPathLength: float
    fCPULimitPercentage: float
    bEnableDiffractionOnReflection: bool
    bEnableGeometricDiffractionAndTransmission: bool
    bCalcEmitterVirtualPosition: bool
    bUseObstruction: bool
    bUseOcclusion: bool

class CAkAudioLibIndex(ctypes.Structure):
    def CAkAudioLibIndex(): pass

    def scalar_deleting_destructor(): pass

    def GetNodeLock(): pass

    def GetNodePtrAndAddRef(): pass

    def Init(): pass

    def ReleaseDynamicSequences(): pass

    def ReleaseTempObjects(): pass

    def Term(): pass

    _fields_ = [
        ("m_idxAudioNode", ctypes.c_char * 0x40),  # CAkIndexItem<CAkParameterNodeBase *>
        ("m_idxBusses", ctypes.c_char * 0x40),  # CAkIndexItem<CAkParameterNodeBase *>
        ("m_idxCustomStates", ctypes.c_char * 0x40),  # CAkIndexItem<CAkState *>
        ("m_idxEvents", ctypes.c_char * 0x40),  # CAkIndexItem<CAkEvent *>
        ("m_idxActions", ctypes.c_char * 0x40),  # CAkIndexItem<CAkAction *>
        ("m_idxLayers", ctypes.c_char * 0x40),  # CAkIndexItem<CAkLayer *>
        ("m_idxAttenuations", ctypes.c_char * 0x40),  # CAkIndexItem<CAkAttenuation *>
        ("m_idxModulators", ctypes.c_char * 0x40),  # CAkIndexItem<CAkModulator *>
        ("m_idxDynamicSequences", ctypes.c_char * 0x40),  # CAkIndexItem<CAkDynamicSequence *>
        ("m_idxDialogueEvents", ctypes.c_char * 0x40),  # CAkIndexItem<CAkDialogueEvent *>
        ("m_idxFxShareSets", ctypes.c_char * 0x40),  # CAkIndexItem<CAkFxShareSet *>
        ("m_idxFxCustom", ctypes.c_char * 0x40),  # CAkIndexItem<CAkFxCustom *>
        ("m_idxAudioDevices", ctypes.c_char * 0x40),  # CAkIndexItem<CAkAudioDevice *>
        ("m_idxVirtualAcoustics", ctypes.c_char * 0x40),  # CAkIndexItem<CAkVirtualAcoustics *>
    ]
    m_idxAudioNode: bytes
    m_idxBusses: bytes
    m_idxCustomStates: bytes
    m_idxEvents: bytes
    m_idxActions: bytes
    m_idxLayers: bytes
    m_idxAttenuations: bytes
    m_idxModulators: bytes
    m_idxDynamicSequences: bytes
    m_idxDialogueEvents: bytes
    m_idxFxShareSets: bytes
    m_idxFxCustom: bytes
    m_idxAudioDevices: bytes
    m_idxVirtualAcoustics: bytes

class AkManualEvent(ctypes.Structure):
    _fields_ = [
        ("m_Event", ctypes.c_void_p),
    ]
    m_Event: int

class AkResourceMonitor(ctypes.Structure):
    pass

class AkMidiHdr(ctypes.Structure):
    _fields_ = [
        ("uiFileFormat", ctypes.c_uint32),
        ("uiNumTracks", ctypes.c_uint32),
        ("uiTicksPerBeat", ctypes.c_uint32),
    ]
    uiFileFormat: int
    uiNumTracks: int
    uiTicksPerBeat: int

class CAkSwitchAware(ctypes.Structure):
    def CAkSwitchAware(): pass

    def tilde_CAkSwitchAware(): pass

    def GetSwitchToUse(): pass

    def SubscribeSwitch(): pass

    def UnsubscribeSwitches(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class CAkSwitchAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkSwitchAware_dtor_0", ctypes.c_ulonglong),
        ("SetSwitch", ctypes.c_ulonglong),
    ]
    CAkSwitchAware_dtor_0: int
    SetSwitch: int

class AkMusicGrid(ctypes.Structure):
    _fields_ = [
        ("fTempo", ctypes.c_float),
        ("uBeatDuration", ctypes.c_uint32),
        ("uBarDuration", ctypes.c_uint32),
        ("uGridDuration", ctypes.c_uint32),
        ("uGridOffset", ctypes.c_uint32),
    ]
    fTempo: float
    uBeatDuration: int
    uBarDuration: int
    uGridDuration: int
    uGridOffset: int

class IAkTriggerAware(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IAkTriggerAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("Trigger", ctypes.c_ulonglong),
    ]
    Trigger: int

class AkVarLenDataCreator(ctypes.Structure):
    _fields_ = [
        ("m_serializer", ctypes.c_char * 0x8),  # AK::Serializer<AK::WriteBytesMem,AK::NoSwapPolicy> *
        ("_m_type", ctypes.c_uint32),
    ]
    m_serializer: bytes
    _m_type: int

    @property
    def m_type(self):
        return AkMonitorData.MonitorDataType(self._m_type)


class AkStateValue(ctypes.Structure):
    _fields_ = [
        ("fValue", ctypes.c_float),
        ("pTransition", ctypes.c_ulonglong),
    ]
    fValue: float
    pTransition: int

class AkMonitor_vtbl(ctypes.Structure):
    _fields_ = [
        ("IALWatches_dtor_0", ctypes.c_ulonglong),
        ("SetMeterWatches", ctypes.c_ulonglong),
        ("Register", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
    ]
    IALWatches_dtor_0: int
    SetMeterWatches: int
    Register: int
    Unregister: int

class AkLockLessMsgQueue(ctypes.Structure):
    _fields_ = [
        ("m_pRead", ctypes.c_ulonglong),
        ("m_pWrite", ctypes.c_ulonglong),
        ("m_pStart", ctypes.c_ulonglong),
        ("m_pEnd", ctypes.c_ulonglong),
        ("m_uQueueSize", ctypes.c_uint32),
    ]
    m_pRead: int
    m_pWrite: int
    m_pStart: int
    m_pEnd: int
    m_uQueueSize: int

class AkQueuedMsg_GameObjectBase(ctypes.Structure):
    _fields_ = [
        ("gameObjID", ctypes.c_uint64),
    ]
    gameObjID: int

class CAkEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int

class AkQueuedMsg_EventRef(ctypes.Structure):
    _fields_ = [
        ("pEvent", ctypes.c_char * 0x8),  # AkQueueMsgPtr<CAkEvent>
        ("eventID", ctypes.c_uint32),
    ]
    pEvent: bytes
    eventID: int

class AkQueuedMsg_State(ctypes.Structure):
    _fields_ = [
        ("StateGroupID", ctypes.c_uint32),
        ("StateID", ctypes.c_uint32),
        ("bSkipTransition", ctypes.c_uint16),
        ("bSkipExtension", ctypes.c_uint16),
    ]
    StateGroupID: int
    StateID: int
    bSkipTransition: int
    bSkipExtension: int

class AkQueuedMsg_SetOutputVolume(ctypes.Structure):
    _fields_ = [
        ("idDevice", ctypes.c_uint64),
        ("fVolume", ctypes.c_float),
    ]
    idDevice: int
    fVolume: float

class CAkDynamicSequence_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("CreatePBI", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    CreatePBI: int

class AkQueuedMsg_DynamicSequenceSeek(ctypes.Structure):
    _fields_ = [
        ("pDynamicSequence", ctypes.c_char * 0x8),  # AkQueueMsgPtr<CAkDynamicSequence>
        ("___u1", ctypes.c_char * 0x4),  # $20371026C6981E3831C323B9C0B38F7C
        ("bIsSeekRelativeToDuration", ctypes.c_uint8),
        ("bSnapToNearestMarker", ctypes.c_uint8),
    ]
    pDynamicSequence: bytes
    ___u1: bytes
    bIsSeekRelativeToDuration: int
    bSnapToNearestMarker: int

class AkQueuedMsg_KillBank(ctypes.Structure):
    _fields_ = [
        ("pUsageSlot", ctypes.c_char * 0x8),  # AkQueueMsgPtr<CAkUsageSlot>
    ]
    pUsageSlot: bytes

class AkQueuedMsg_AddOutputBase(ctypes.Structure):
    _fields_ = [
        ("pSettings", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
    ]
    pSettings: list[ctypes.c_uint8]

class AkQueuedMsg_RemoveOutput(ctypes.Structure):
    _fields_ = [
        ("idOutput", ctypes.c_uint64),
    ]
    idOutput: int

class AkQueuedMsg_ReplaceOutput(ctypes.Structure):
    _fields_ = [
        ("pSettings", ctypes.c_uint8 * 0x10),  # unsigned __int8[16]
        ("idOutput", ctypes.c_uint64),
    ]
    pSettings: list[ctypes.c_uint8]
    idOutput: int

class AkQueuedMsg_SetBusDevice(ctypes.Structure):
    _fields_ = [
        ("idBus", ctypes.c_uint32),
        ("idDevice", ctypes.c_uint32),
    ]
    idBus: int
    idDevice: int

class AkQueuedMsg_StartStopCapture(ctypes.Structure):
    _fields_ = [
        ("szFileName", ctypes.c_char * 0x8),  # AkQueueMsgPtr<wchar_t>
    ]
    szFileName: bytes

class AkQueuedMsg_AddOutputCaptureMarker(ctypes.Structure):
    _fields_ = [
        ("szMarkerText", ctypes.c_char * 0x8),  # AkQueueMsgPtr<char>
    ]
    szMarkerText: bytes

class AkQueuedMsg_SetDeviceEffect(ctypes.Structure):
    _fields_ = [
        ("outputDeviceID", ctypes.c_uint64),
        ("uFXIndex", ctypes.c_uint32),
        ("shareSetID", ctypes.c_uint32),
    ]
    outputDeviceID: int
    uFXIndex: int
    shareSetID: int

class AkQueuedMsg_SetBusConfig(ctypes.Structure):
    _fields_ = [
        ("audioNodeID", ctypes.c_uint32),
        ("uChannelConfig", ctypes.c_uint32),
    ]
    audioNodeID: int
    uChannelConfig: int

class AkQueuedMsg_SetSpeakerAngles(ctypes.Structure):
    _fields_ = [
        ("pfSpeakerAngles", ctypes.c_char * 0x8),  # AkQueueMsgPtr<float>
        ("uNumAngles", ctypes.c_uint32),
        ("fHeightAngle", ctypes.c_float),
        ("idOutput", ctypes.c_uint64),
    ]
    pfSpeakerAngles: bytes
    uNumAngles: int
    fHeightAngle: float
    idOutput: int

class AkQueuedMsg_SetRandomSeed(ctypes.Structure):
    _fields_ = [
        ("uSeed", ctypes.c_uint32),
    ]
    uSeed: int

class AkQueuedMsg_MuteBackgroundMusic(ctypes.Structure):
    _fields_ = [
        ("bMute", ctypes.c_uint16),
    ]
    bMute: int

class AkQueuedMsg_EndOfList(ctypes.Structure):
    _fields_ = [
        ("uFrameNumber", ctypes.c_uint32),
    ]
    uFrameNumber: int

class AkQueuedMsg_Suspend(ctypes.Structure):
    _fields_ = [
        ("uDelay", ctypes.c_uint32),
        ("bSuspend", ctypes.c_uint16),
        ("bRender", ctypes.c_uint16),
    ]
    uDelay: int
    bSuspend: int
    bRender: int

class AkQueuedMsg_ApiExtension(ctypes.Structure):
    _fields_ = [
        ("uID", ctypes.c_uint32),
    ]
    uID: int

class AkQueuedMsg_InitSinkPlugin(ctypes.Structure):
    _fields_ = [
        ("buffer", ctypes.c_char * 0x10),  # char[16]
    ]
    buffer: list[ctypes.c_char]

class AkQueuedMsg_OfflineRendering(ctypes.Structure):
    _fields_ = [
        ("bSetOfflineRendering", ctypes.c_uint16),
        ("bIsOfflineRendering", ctypes.c_uint16),
        ("fFrameTime", ctypes.c_float),
    ]
    bSetOfflineRendering: int
    bIsOfflineRendering: int
    fFrameTime: float

class AkQueuedMsg(ctypes.Structure):
    def Sizeof_AddOutputCaptureMarker(): pass

    def Sizeof_SetRandomSeed(): pass

    def Sizeof_OpenDynamicSequence(): pass

    def Sizeof_Switch(): pass

    def Sizeof_PlaySourcePlugin(): pass

    def Sizeof_GameObjObstruction(): pass

    def Sizeof_Seek(): pass

    _fields_ = [
        ("size", ctypes.c_uint16),
        ("type", ctypes.c_uint16),
        ("___u2", ctypes.c_char * 0x38),  # $67B039388F5445A788D2B22584107E68
    ]
    size: int
    type: int
    ___u2: bytes

class AkMediaInfo(ctypes.Structure):
    _fields_ = [
        ("pInMemoryData", ctypes.c_ulonglong),
        ("uInMemoryDataSize", ctypes.c_uint32),
        ("poolId", ctypes.c_int32),
    ]
    pInMemoryData: int
    uInMemoryDataSize: int
    poolId: int

class CAkChildCtx(ctypes.Structure):
    def CAkChildCtx(): pass

    def tilde_CAkChildCtx(): pass

    def vector_deleting_destructor(): pass

    def Connect(): pass

    def Disconnect(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("pNextLightItem", ctypes.c_ulonglong),
        ("m_pParentCtx", ctypes.c_ulonglong),
    ]
    __vftable: int
    pNextLightItem: int
    m_pParentCtx: int

class CAkChildCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int

class CAkMusicCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int

class AkMidiTrack(ctypes.Structure):
    _fields_ = [
        ("byTrackNum", ctypes.c_uint8),
        ("pbyBegin", ctypes.c_ulonglong),
        ("pbyEnd", ctypes.c_ulonglong),
        ("pbySeekEvent", ctypes.c_ulonglong),
        ("uiSeekDelta", ctypes.c_uint32),
        ("abySeekExtract", ctypes.c_uint8 * 0x6),  # unsigned __int8[6]
        ("byRunStatus", ctypes.c_uint8),
    ]
    byTrackNum: int
    pbyBegin: int
    pbyEnd: int
    pbySeekEvent: int
    uiSeekDelta: int
    abySeekExtract: list[ctypes.c_uint8]
    byRunStatus: int

class AkExtent(ctypes.Structure):
    _fields_ = [
        ("halfWidth", ctypes.c_float),
        ("halfHeight", ctypes.c_float),
        ("halfDepth", ctypes.c_float),
    ]
    halfWidth: float
    halfHeight: float
    halfDepth: float

class AkImageSourceName(ctypes.Structure):
    _fields_ = [
        ("uNumChar", ctypes.c_uint32),
        ("pName", ctypes.c_char_p),
    ]
    uNumChar: int
    pName: bytes

class AkImageSourceTexture(ctypes.Structure):
    _fields_ = [
        ("uNumTexture", ctypes.c_uint32),
        ("arTextureID", ctypes.c_uint32 * 0x4),  # unsigned int[4]
    ]
    uNumTexture: int
    arTextureID: list[ctypes.c_uint32]

class AkMusicMarker(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint32),
        ("uPosition", ctypes.c_uint32),
        ("pszName", ctypes.c_char_p),
    ]
    id: int
    uPosition: int
    pszName: bytes

class AkSeekingInfo(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $1916AED2A9831715E7E5BFC690EACA60
        ("bRelative", ctypes.c_ubyte),
    ]
    ___u0: bytes
    bRelative: bool

class AkMusicParams(ctypes.Structure):
    _fields_ = [
        ("fPlaybackSpeed", ctypes.c_float),
    ]
    fPlaybackSpeed: float

class CAkLstnrAssocs(ctypes.Structure):
    def CopyFrom(): pass

    def GetGain(): pass

    def GetGains(): pass

    def PreAssocRemoved(): pass

    def SetGain(): pass

    def SetGains(): pass

    def SetListeners(): pass

    def Term(): pass

    _fields_ = [
        ("m_assoc", ctypes.c_char * 0x30),  # AkSortedKeyArray<CAkLstnrAssoc::Key,CAkLstnrAssoc,AkHybridAllocator<24,16,4>,CAkLstnrAssoc,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<CAkLstnrAssoc>,AkDefaultSortedKeyCompare<CAkLstnrAssoc::Key> >
        ("m_listenerSet", ctypes.c_char * 0x20),  # AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional>
    ]
    m_assoc: bytes
    m_listenerSet: bytes

class CAkConnectedListeners_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class AkCutoffInfo(ctypes.Structure):
    _fields_ = [
        ("iCutoffTime", ctypes.c_int64),
        ("bCutoff", ctypes.c_ubyte),
    ]
    iCutoffTime: int
    bCutoff: bool

class AkStingerRecord(ctypes.Structure):
    _fields_ = [
        ("pNextLightItem", ctypes.c_ulonglong),
        ("pStingerCtx", ctypes.c_char * 0x8),  # CAkSmartPtr<CAkMatrixAwareCtx>
        ("triggerID", ctypes.c_uint32),
        ("segmentID", ctypes.c_uint32),
        ("iSyncTime", ctypes.c_int64),
        ("uDontRepeatTime", ctypes.c_uint32),
        ("iLookAheadDuration", ctypes.c_int32),
        ("iRelativeStopTime", ctypes.c_int32),
        ("bCanBeRescheduled", ctypes.c_ubyte),
    ]
    pNextLightItem: int
    pStingerCtx: bytes
    triggerID: int
    segmentID: int
    iSyncTime: int
    uDontRepeatTime: int
    iLookAheadDuration: int
    iRelativeStopTime: int
    bCanBeRescheduled: bool

class AkMusicAction(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("pNextItem", ctypes.c_ulonglong),
        ("m_iTime", ctypes.c_int32),
    ]
    __vftable: int
    pNextItem: int
    m_iTime: int

class AkMusicAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkMusicAction_dtor_0", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
    ]
    AkMusicAction_dtor_0: int
    Type: int

class CAkMusicActionSequencer(ctypes.Structure):
    def CAkMusicActionSequencer(): pass

    def tilde_CAkMusicActionSequencer(): pass

    def vector_deleting_destructor(): pass

    def Flush(): pass

    def PopImminentAction(): pass

    def ScheduleAction(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("baseclass_8", ctypes.c_char * 0x10),  # AkListBare<AkMusicAction,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast>
    ]
    __vftable: int
    baseclass_8: bytes

class CAkMusicActionSequencer_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkMusicActionSequencer_dtor_0", ctypes.c_ulonglong),
    ]
    CAkMusicActionSequencer_dtor_0: int

class CAkMusicSubTrackSequencer_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkMusicActionSequencer_dtor_0", ctypes.c_ulonglong),
    ]
    CAkMusicActionSequencer_dtor_0: int

class AkTrackSrc(ctypes.Structure):
    _fields_ = [
        ("uSubTrackIndex", ctypes.c_uint32),
        ("srcID", ctypes.c_uint32),
        ("eventID", ctypes.c_uint32),
        ("uClipStartPosition", ctypes.c_uint32),
        ("uClipDuration", ctypes.c_uint32),
        ("uSrcDuration", ctypes.c_uint32),
        ("iSourceTrimOffset", ctypes.c_int32),
    ]
    uSubTrackIndex: int
    srcID: int
    eventID: int
    uClipStartPosition: int
    uClipDuration: int
    uSrcDuration: int
    iSourceTrimOffset: int

class CAkMidiBaseMgr_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnNoteStateFinished", ctypes.c_ulonglong),
        ("UpdateOnFirstNote", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkMidiBaseMgr *this, AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *, AkMidiFrameEvent *)
    ]
    OnNoteStateFinished: int
    UpdateOnFirstNote: bytes

class CAkMidiBaseCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkMidiBaseCtx_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetTrackID", ctypes.c_ulonglong),
        ("GetAbsoluteStop", ctypes.c_ulonglong),
        ("HasMoreEvents", ctypes.c_ulonglong),
        ("ResolveMidiTarget", ctypes.c_ulonglong),
        ("GetMidiMgr", ctypes.c_ulonglong),
    ]
    CAkMidiBaseCtx_dtor_0: int
    Init: int
    GetTrackID: int
    GetAbsoluteStop: int
    HasMoreEvents: int
    ResolveMidiTarget: int
    GetMidiMgr: int

class CAkMidiClipMgr_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnNoteStateFinished", ctypes.c_ulonglong),
        ("UpdateOnFirstNote", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkMidiBaseMgr *this, AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *, AkMidiFrameEvent *)
    ]
    OnNoteStateFinished: int
    UpdateOnFirstNote: bytes

class AkMidiParseSe_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkMidiParseSe_dtor_0", ctypes.c_ulonglong),
    ]
    AkMidiParseSe_dtor_0: int

class CAkMidiClipCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkMidiBaseCtx_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetTrackID", ctypes.c_ulonglong),
        ("GetAbsoluteStop", ctypes.c_ulonglong),
        ("HasMoreEvents", ctypes.c_ulonglong),
        ("ResolveMidiTarget", ctypes.c_ulonglong),
        ("GetMidiMgr", ctypes.c_ulonglong),
    ]
    CAkMidiBaseCtx_dtor_0: int
    Init: int
    GetTrackID: int
    GetAbsoluteStop: int
    HasMoreEvents: int
    ResolveMidiTarget: int
    GetMidiMgr: int

class CAkSwitchTrackInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkSwitchAware_dtor_0", ctypes.c_ulonglong),
        ("SetSwitch", ctypes.c_ulonglong),
    ]
    CAkSwitchAware_dtor_0: int
    SetSwitch: int

class CAkSegmentCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int

class CAkMatrixSequencer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Trigger", ctypes.c_ulonglong),
    ]
    Trigger: int

class CAkMatrixAwareCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
        ("EndInit", ctypes.c_ulonglong),
        ("CanKeepPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory_80", ctypes.c_ulonglong),
        ("GetPlaybackHistory", ctypes.c_ulonglong),
        ("Process", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("GetNextScheduleWindow", ctypes.c_ulonglong),
        ("RefreshWindow", ctypes.c_ulonglong),
        ("QueryLookAheadInfo", ctypes.c_ulonglong),
        ("Node", ctypes.c_ulonglong),
        ("CanRestartPlaying", ctypes.c_ulonglong),
        ("CancelPlayback", ctypes.c_ulonglong),
        ("PerformDelayedSwitchChange", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("GetPlayingSegmentInfo", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int
    EndInit: int
    CanKeepPlaybackHistory: int
    SetPlaybackHistory: int
    SetPlaybackHistory_80: int
    GetPlaybackHistory: int
    Process: int
    Prepare: int
    GetNextScheduleWindow: int
    RefreshWindow: int
    QueryLookAheadInfo: int
    Node: int
    CanRestartPlaying: int
    CancelPlayback: int
    PerformDelayedSwitchChange: int
    SeekTimeAbsolute: int
    SeekPercent: int
    GetPlayingSegmentInfo: int

class GetNullGameObjPtr(ctypes.Structure):
    pass

class GetInvalidPlayingID(ctypes.Structure):
    pass

class GetInvalidUniqueID(ctypes.Structure):
    pass

class GetInvalidMidiNote(ctypes.Structure):
    pass

class GetInvalidMidiCh(ctypes.Structure):
    pass

class GetNullPbiPtr(ctypes.Structure):
    pass

class AkMeterInfo(ctypes.Structure):
    _fields_ = [
        ("fGridPeriod", ctypes.c_double),
        ("fGridOffset", ctypes.c_double),
        ("fTempo", ctypes.c_float),
        ("uTimeSigNumBeatsBar", ctypes.c_uint8),
        ("uTimeSigBeatValue", ctypes.c_uint8),
    ]
    fGridPeriod: float
    fGridOffset: float
    fTempo: float
    uTimeSigNumBeatsBar: int
    uTimeSigBeatValue: int

class CAkAudioThread(ctypes.Structure):
    def CAkAudioThread(): pass

    def EventMgrThreadFunc(): pass

    def Start(): pass

    def Stop(): pass

    def WakeupEventsConsumer(): pass

    _fields_ = [
        ("m_eventProcess", ctypes.c_void_p),
        ("m_bStopThread", ctypes.c_ubyte),
    ]
    m_eventProcess: int
    m_bStopThread: bool

class AkRTPCValue(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $423ABB04589F036CE2715CD8110A5DD9
    ]
    ___u0: bytes

class AkDeltaMonitorEnabled(ctypes.Structure):
    pass

class AkDeltaMonitorDisabled(ctypes.Structure):
    pass

class ComponentInstancercTkPhysicsComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class AkEffectUpdate(ctypes.Structure):
    _fields_ = [
        ("ulEffectID", ctypes.c_uint32),
        ("uiIndex", ctypes.c_uint8),
        ("bShared", ctypes.c_ubyte),
    ]
    ulEffectID: int
    uiIndex: int
    bShared: bool

class AkPathListItemOffset(ctypes.Structure):
    _fields_ = [
        ("ulVerticesOffset", ctypes.c_uint32),
        ("iNumVertices", ctypes.c_int32),
    ]
    ulVerticesOffset: int
    iNumVertices: int

class AkDeltaMonitorObjBrace(ctypes.Structure):
    pass

class AkDeltaMonitorUpdateBrace(ctypes.Structure):
    _fields_ = [
        ("m_idSource", ctypes.c_uint32),
        ("wasOpened", ctypes.c_ubyte),
    ]
    m_idSource: int
    wasOpened: bool

class AkGameRayParams(ctypes.Structure):
    _fields_ = [
        ("occlusion", ctypes.c_float),
        ("obstruction", ctypes.c_float),
        ("spread", ctypes.c_float),
        ("aperture", ctypes.c_float),
    ]
    occlusion: float
    obstruction: float
    spread: float
    aperture: float

class AkAkRtpcIDSetGetKey(ctypes.Structure):
    pass

class AkAutoTermListenerSet(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # AkSet<unsigned __int64,AkHybridAllocator<8,16,4>,AkGrowByPolicy_Proportional>
    ]
    baseclass_0: bytes

class AkEmitterListenerPathTraversal(ctypes.Structure):
    _fields_ = [
        ("m_visited", ctypes.c_char * 0x10),  # AkSet<CAkGameObject *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional>
        ("m_result", ctypes.c_char * 0x10),  # AkSet<CAkGameObject *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional>
    ]
    m_visited: bytes
    m_result: bytes

class AkPerLstnrObjData(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x10),  # AkArray<AkGameRayParams,AkGameRayParams const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameRayParams> >
    ]
    baseclass_0: bytes

class AkSoundPositionRef(ctypes.Structure):
    _fields_ = [
        ("m_aPosition", ctypes.c_ulonglong),
        ("m_mapDataPerListener", ctypes.c_char * 0x10),  # CAkKeyArray<unsigned __int64,AkPerLstnrObjData,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkTransferMovePolicy<MapStruct<unsigned __int64,AkPerLstnrObjData> > >
        ("m_uNumPosition", ctypes.c_uint16),
        ("_bf_1a", ctypes.c_int8),
    ]
    m_aPosition: int
    m_mapDataPerListener: bytes
    m_uNumPosition: int
    _bf_1a: int

class CAkListener_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class CAkEmitter_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class SpeakerVolumeMatrixCallback(ctypes.Structure):
    _fields_ = [
        ("playingID", ctypes.c_uint32),
    ]
    playingID: int

class AkSwitchHistItem(ctypes.Structure):
    _fields_ = [
        ("LastSwitch", ctypes.c_uint32),
        ("NumPlayBack", ctypes.c_uint32),
    ]
    LastSwitch: int
    NumPlayBack: int

class CAkSwitchHistory(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # CAkTrackedGameObjComponent<5>
        ("History", ctypes.c_char * 0x10),  # CAkKeyArray<unsigned int,AkSwitchHistItem,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<MapStruct<unsigned int,AkSwitchHistItem> > >
    ]
    baseclass_0: bytes
    History: bytes

class CAkSwitchHistory_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class CAkModifiedNodes(ctypes.Structure):
    def CAkModifiedNodes(): pass

    def tilde_CAkModifiedNodes(): pass

    def vector_deleting_destructor(): pass

    def SetNodeAsModified(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # CAkTrackedGameObjComponent<4>
        ("m_ListModifiedNodes", ctypes.c_char * 0x28),  # CAkList2<WwiseObjectIDext,WwiseObjectIDext const &,1,0>
    ]
    baseclass_0: bytes
    m_ListModifiedNodes: bytes

class CAkModifiedNodes_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class AccumulateAdd(ctypes.Structure):
    pass

class AccumulateMult(ctypes.Structure):
    pass

class AkWwiseConeAttenuation(ctypes.Structure):
    _fields_ = [
        ("bIsConeEnabled", ctypes.c_uint8),
        ("cone_fInsideAngle", ctypes.c_float),
        ("cone_fOutsideAngle", ctypes.c_float),
        ("cone_fOutsideVolume", ctypes.c_float),
        ("cone_LoPass", ctypes.c_float),
        ("cone_HiPass", ctypes.c_float),
    ]
    bIsConeEnabled: int
    cone_fInsideAngle: float
    cone_fOutsideAngle: float
    cone_fOutsideVolume: float
    cone_LoPass: float
    cone_HiPass: float

class CAkDialogueEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int

class CAkFxShareSet_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("IsShareSet", ctypes.c_ulonglong),
        ("SetInitialValues", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    IsShareSet: int
    SetInitialValues: int

class CAkFxCustom_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("IsShareSet", ctypes.c_ulonglong),
        ("SetInitialValues", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    IsShareSet: int
    SetInitialValues: int

class CAkAudioDevice_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("IsShareSet", ctypes.c_ulonglong),
        ("SetInitialValues", ctypes.c_ulonglong),
        ("UpdateBypass", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    IsShareSet: int
    SetInitialValues: int
    UpdateBypass: int

class CAkVirtualAcoustics_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_28", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetAkProp: int
    SetAkProp_28: int

class AkSourceSettings(ctypes.Structure):
    _fields_ = [
        ("sourceID", ctypes.c_uint32),
        ("pMediaMemory", ctypes.c_ulonglong),
        ("uMediaSize", ctypes.c_uint32),
    ]
    sourceID: int
    pMediaMemory: int
    uMediaSize: int

class RandomizerModifier(ctypes.Structure):
    pass

class AkSinkPluginParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriberPlugin_dtor_0", ctypes.c_ulonglong),
        ("GetPluginID", ctypes.c_ulonglong),
        ("UpdateTargetRtpcParam", ctypes.c_ulonglong),
        ("NotifyRtpcParamChanged", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriberPlugin_dtor_0: int
    GetPluginID: int
    UpdateTargetRtpcParam: int
    NotifyRtpcParamChanged: int
    Term: int

class AkAudioDeviceEffectParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriberPlugin_dtor_0", ctypes.c_ulonglong),
        ("GetPluginID", ctypes.c_ulonglong),
        ("UpdateTargetRtpcParam", ctypes.c_ulonglong),
        ("NotifyRtpcParamChanged", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriberPlugin_dtor_0: int
    GetPluginID: int
    UpdateTargetRtpcParam: int
    NotifyRtpcParamChanged: int
    Term: int

class AkSortedPBIGetKey(ctypes.Structure):
    pass

class CAkMusicPBI_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("CAkTransportAware_dtor_8", ctypes.c_ulonglong),
        ("SetEstimatedLength", ctypes.c_ulonglong),
        ("_Pause", ctypes.c_ulonglong),
        ("_Resume", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekTimeAbsolute_38", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("SetPauseStateForContinuous", ctypes.c_ulonglong),
        ("PrepareSampleAccurateTransition", ctypes.c_ulonglong),
        ("NotifyVirtualOff", ctypes.c_ulonglong),
        ("MonitorFade", ctypes.c_ulonglong),
        ("GetAndClearStopOffset", ctypes.c_ulonglong),
    ]
    _Stop: int
    CAkTransportAware_dtor_8: int
    SetEstimatedLength: int
    _Pause: int
    _Resume: int
    PlayToEnd: int
    SeekTimeAbsolute: int
    SeekTimeAbsolute_38: int
    SeekPercent: int
    SetPauseStateForContinuous: int
    PrepareSampleAccurateTransition: int
    NotifyVirtualOff: int
    MonitorFade: int
    GetAndClearStopOffset: int

class CAkMusicClipSequencer_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkMusicActionSequencer_dtor_0", ctypes.c_ulonglong),
    ]
    CAkMusicActionSequencer_dtor_0: int

class AkMusicAutomation(ctypes.Structure):
    _fields_ = [
        ("pAutomationData", ctypes.c_ulonglong),
        ("pPBI", ctypes.c_ulonglong),
        ("iTimeStart", ctypes.c_int32),
        ("pNextLightItem", ctypes.c_ulonglong),
    ]
    pAutomationData: int
    pPBI: int
    iTimeStart: int
    pNextLightItem: int

class CAkSubTrackCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int

class AkSISValue(ctypes.Structure):
    _fields_ = [
        ("fValue", ctypes.c_float),
        ("pTransition", ctypes.c_ulonglong),
    ]
    fValue: float
    pTransition: int

class ComponentInstancercTkStaticPhysicsComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class AkGlobalCallbackRecord(ctypes.Structure):
    _fields_ = [
        ("pFunc", ctypes.c_ulonglong),
        ("pCookie", ctypes.c_void_p),
        ("FXID", ctypes.c_uint32),
    ]
    pFunc: int
    pCookie: int
    FXID: int

class AkTrackSrcInfo(ctypes.Structure):
    _fields_ = [
        ("trackID", ctypes.c_uint32),
        ("sourceID", ctypes.c_uint32),
        ("eventID", ctypes.c_uint32),
        ("fPlayAt", ctypes.c_double),
        ("fBeginTrimOffset", ctypes.c_double),
        ("fEndTrimOffset", ctypes.c_double),
        ("fSrcDuration", ctypes.c_double),
    ]
    trackID: int
    sourceID: int
    eventID: int
    fPlayAt: float
    fBeginTrimOffset: float
    fEndTrimOffset: float
    fSrcDuration: float

class CAkRegistryMgr(ctypes.Structure):
    def tilde_CAkRegistryMgr(): pass

    def CAkRegistryMgr(): pass

    def ClearSwitchHist(): pass

    def EnsureListenersExist(): pass

    def GetActiveGameObjects(): pass

    def GetObj(): pass

    def GetObjAndAddref(): pass

    def GetSwitchHistItem(): pass

    def Init(): pass

    def IsGameObjectActive(): pass

    def NotifyListenerPosChanged(): pass

    def RegisterObject(): pass

    def RegisterObjectAsActive(): pass

    def ResetListenersToDefault(): pass

    def SetGameObjectAuxSendValues(): pass

    def SetGameObjectOutputBusVolume(): pass

    def SetGameObjectScalingFactor(): pass

    def SetMultipleObstructionAndOcclusion(): pass

    def SetNodeIDAsModified(): pass

    def SetObjectObstructionAndOcclusion(): pass

    def SetPosition(): pass

    def SetSwitchDormant(): pass

    def SetSwitchHistItem(): pass

    def Term(): pass

    def UnregisterAll(): pass

    def UnregisterObject(): pass

    def UnregisterObjectAsActive(): pass

    def UpdateDefaultListeners(): pass

    def UpdateGameObjectPositions(): pass

    def UpdateListeners(): pass

    _fields_ = [
        ("m_listModifiedNodes", ctypes.c_char * 0x28),  # CAkList2<WwiseObjectIDext,WwiseObjectIDext const &,1,0>
        ("m_mapRegisteredObj", ctypes.c_char * 0x18),  # AkHashList<unsigned __int64,CAkRegisteredObj *,AkArrayAllocatorNoAlign<0> >
    ]
    m_listModifiedNodes: bytes
    m_mapRegisteredObj: bytes

class AkMusicActionSwitchNotif_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkMusicAction_dtor_0", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
    ]
    AkMusicAction_dtor_0: int
    Type: int

class AkMusicActionPlaySubTrack_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkMusicAction_dtor_0", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
    ]
    AkMusicAction_dtor_0: int
    Type: int

class AkMusicActionStopSubTrack_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkMusicAction_dtor_0", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
    ]
    AkMusicAction_dtor_0: int
    Type: int

class AkMusicActionPlayClip_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkMusicAction_dtor_0", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
    ]
    AkMusicAction_dtor_0: int
    Type: int

class TkJsonFileReadStreamBase(ctypes.Structure):
    def Read(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpBuffer", ctypes.c_char_p),
        ("miBufferSize", ctypes.c_uint64),
        ("mpBufferLast", ctypes.c_char_p),
        ("mpCurrent", ctypes.c_char_p),
        ("miReadCount", ctypes.c_uint64),
        ("miCount", ctypes.c_uint64),
        ("mbEof", ctypes.c_ubyte),
    ]
    __vftable: int
    mpBuffer: bytes
    miBufferSize: int
    mpBufferLast: bytes
    mpCurrent: bytes
    miReadCount: int
    miCount: int
    mbEof: bool

class TkJsonFileReadStreamBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("ReadData", ctypes.c_ulonglong),
    ]
    ReadData: int

class TkJsonTempFileReadStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("ReadData", ctypes.c_ulonglong),
    ]
    ReadData: int

class TkJsonFileReadStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("ReadData", ctypes.c_ulonglong),
    ]
    ReadData: int

class StringHolder(ctypes.Structure):
    _fields_ = [
        ("mpString", ctypes.c_char_p),
        ("miSize", ctypes.c_int32),
        ("miIndex", ctypes.c_int32),
    ]
    mpString: bytes
    miSize: int
    miIndex: int

class AkMusicActionStopClip_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkMusicAction_dtor_0", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
    ]
    AkMusicAction_dtor_0: int
    Type: int

class TkJsonFileWriteStreamBase(ctypes.Structure):
    def Put(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpBuffer", ctypes.c_char_p),
        ("mpBufferEnd", ctypes.c_char_p),
        ("mpCurrent", ctypes.c_char_p),
        ("miCount", ctypes.c_uint64),
    ]
    __vftable: int
    mpBuffer: bytes
    mpBufferEnd: bytes
    mpCurrent: bytes
    miCount: int

class TkJsonFileWriteStreamBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("WriteData", ctypes.c_ulonglong),
    ]
    WriteData: int

class AkMusicActionPostEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkMusicAction_dtor_0", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
    ]
    AkMusicAction_dtor_0: int
    Type: int

class TkJsonTempFileWriteStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("WriteData", ctypes.c_ulonglong),
    ]
    WriteData: int

class TkJsonFileWriteStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("WriteData", ctypes.c_ulonglong),
    ]
    WriteData: int

class AkMusicMarkerWwise(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint32),
        ("fPosition", ctypes.c_double),
        ("pszName", ctypes.c_char_p),
    ]
    id: int
    fPosition: float
    pszName: bytes

class TkJsonFileReadLZ4Stream(ctypes.Structure):
    _fields_ = [
        ("mFileHandle", ctypes.c_uint64),
        ("mpBuffer", ctypes.c_char_p),
        ("miBufferSize", ctypes.c_uint64),
        ("mpBufferLast", ctypes.c_char_p),
        ("mpCurrent", ctypes.c_char_p),
        ("miReadCount", ctypes.c_uint64),
        ("miCount", ctypes.c_uint64),
        ("mbEof", ctypes.c_ubyte),
    ]
    mFileHandle: int
    mpBuffer: bytes
    miBufferSize: int
    mpBufferLast: bytes
    mpCurrent: bytes
    miReadCount: int
    miCount: int
    mbEof: bool

class cTkDocumentStreamWriterJSON_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkDocumentWriter_dtor_0", ctypes.c_ulonglong),
        ("GetWriterFormat", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(ITkDocumentWriter *this, TkID<128> *result)
        ("Clear", ctypes.c_ulonglong),
        ("PushContext", ctypes.c_ulonglong),
        ("PopContext", ctypes.c_ulonglong),
        ("OpenArray", ctypes.c_ulonglong),
        ("OpenObject", ctypes.c_ulonglong),
        ("Close", ctypes.c_ulonglong),
        ("AddValue", ctypes.c_ulonglong),
        ("AddValue_48", ctypes.c_ulonglong),
        ("AddValue_50", ctypes.c_ulonglong),
        ("AddValue_58", ctypes.c_ulonglong),
        ("AddValue_60", ctypes.c_char * 0x8),  # void (__fastcall *)(ITkDocumentWriter *this, const TkID<256> *)
        ("AddValue_68", ctypes.c_char * 0x8),  # void (__fastcall *)(ITkDocumentWriter *this, const TkID<128> *)
        ("AddValue_70", ctypes.c_ulonglong),
        ("AddValue_78", ctypes.c_ulonglong),
        ("AddValue_80", ctypes.c_ulonglong),
        ("AddValue_88", ctypes.c_ulonglong),
        ("AddValue_90", ctypes.c_ulonglong),
        ("AddValue_98", ctypes.c_ulonglong),
        ("AddValue_a0", ctypes.c_ulonglong),
        ("AddValue_a8", ctypes.c_ulonglong),
        ("AddValue_b0", ctypes.c_ulonglong),
        ("AddValue_b8", ctypes.c_ulonglong),
        ("AddValue_c0", ctypes.c_ulonglong),
        ("AddKey", ctypes.c_ulonglong),
    ]
    ITkDocumentWriter_dtor_0: int
    GetWriterFormat: bytes
    Clear: int
    PushContext: int
    PopContext: int
    OpenArray: int
    OpenObject: int
    Close: int
    AddValue: int
    AddValue_48: int
    AddValue_50: int
    AddValue_58: int
    AddValue_60: bytes
    AddValue_68: bytes
    AddValue_70: int
    AddValue_78: int
    AddValue_80: int
    AddValue_88: int
    AddValue_90: int
    AddValue_98: int
    AddValue_a0: int
    AddValue_a8: int
    AddValue_b0: int
    AddValue_b8: int
    AddValue_c0: int
    AddKey: int

class AkChildIDValueGetKey(ctypes.Structure):
    pass

class _SOCKADDR_INET: pass

class cTkTextStyleGradient_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTextStyle_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkTextStyle_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int

class cTkTextStyleOutlineGradient_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTextStyle_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkTextStyle_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int

class cTkTextStyleOutline_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTextStyle_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkTextStyle_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int

class CAkFunctionCritical(ctypes.Structure):
    pass

class CAkBankList(ctypes.Structure):
    def Get(): pass

    def Init(): pass

    def Remove(): pass

    def Set(): pass

    _fields_ = [
        ("m_ListLoadedBanks", ctypes.c_char * 0x18),  # AkHashListBare<AkBankKey,CAkUsageSlot,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<AkBankKey,CAkUsageSlot> >
    ]
    m_ListLoadedBanks: bytes

class AkFileNameString(ctypes.Structure):
    def Copy(): pass

    def Set(): pass

    def Term(): pass

    _fields_ = [
        ("m_pStr", ctypes.c_char_p),
        ("m_bOwner", ctypes.c_ubyte),
    ]
    m_pStr: bytes
    m_bOwner: bool

class CAkMusicSwitchCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
        ("EndInit", ctypes.c_ulonglong),
        ("CanKeepPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory_80", ctypes.c_ulonglong),
        ("GetPlaybackHistory", ctypes.c_ulonglong),
        ("Process", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("GetNextScheduleWindow", ctypes.c_ulonglong),
        ("RefreshWindow", ctypes.c_ulonglong),
        ("QueryLookAheadInfo", ctypes.c_ulonglong),
        ("Node", ctypes.c_ulonglong),
        ("CanRestartPlaying", ctypes.c_ulonglong),
        ("CancelPlayback", ctypes.c_ulonglong),
        ("PerformDelayedSwitchChange", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("GetPlayingSegmentInfo", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int
    EndInit: int
    CanKeepPlaybackHistory: int
    SetPlaybackHistory: int
    SetPlaybackHistory_80: int
    GetPlaybackHistory: int
    Process: int
    Prepare: int
    GetNextScheduleWindow: int
    RefreshWindow: int
    QueryLookAheadInfo: int
    Node: int
    CanRestartPlaying: int
    CancelPlayback: int
    PerformDelayedSwitchChange: int
    SeekTimeAbsolute: int
    SeekPercent: int
    GetPlayingSegmentInfo: int

class CAkMusicSwitchMonitor_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkSwitchAware_dtor_0", ctypes.c_ulonglong),
        ("SetSwitch", ctypes.c_ulonglong),
    ]
    CAkSwitchAware_dtor_0: int
    SetSwitch: int

class AkBankCompletionNotifInfo(ctypes.Structure):
    _fields_ = [
        ("pfnBankCallback", ctypes.c_ulonglong),
        ("pCookie", ctypes.c_void_p),
    ]
    pfnBankCallback: int
    pCookie: int

class AkPrepareEventQueueItemLoad(ctypes.Structure):
    _fields_ = [
        ("numEvents", ctypes.c_uint32),
        ("___u1", ctypes.c_char * 0x8),  # $3123BC0552B509AFBEA20B901D233EC4
    ]
    numEvents: int
    ___u1: bytes

class CAkBankMgr_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkBankMgr_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("QueueBankCommand", ctypes.c_ulonglong),
        ("InitSyncOp", ctypes.c_ulonglong),
        ("WaitForSyncOp", ctypes.c_ulonglong),
        ("StopThread", ctypes.c_ulonglong),
        ("CancelCookie", ctypes.c_ulonglong),
        ("KillSlot", ctypes.c_ulonglong),
    ]
    CAkBankMgr_dtor_0: int
    Init: int
    Term: int
    QueueBankCommand: int
    InitSyncOp: int
    WaitForSyncOp: int
    StopThread: int
    CancelCookie: int
    KillSlot: int

class CAkBankReader(ctypes.Structure):
    def CAkBankReader(): pass

    def CloseFile(): pass

    def FillData(): pass

    def FillDataEx(): pass

    def FinalizeStreamInit(): pass

    def GetData(): pass

    def Init(): pass

    def ReleaseData(): pass

    def Reset(): pass

    def SetBankLoadIOSettings(): pass

    def SetFile(): pass

    def Skip(): pass

    def Term(): pass

    _fields_ = [
        ("m_pStreamBuffers", ctypes.c_void_p * 0x2),  # void *[2]
        ("m_pStreamReadBuffer", ctypes.c_void_p),
        ("m_uBufferBytesRemaining", ctypes.c_uint32 * 0x2),  # unsigned int[2]
        ("m_uCurrentBufferIdx", ctypes.c_uint32),
        ("m_uStreamBufferSize", ctypes.c_uint32),
        ("m_pStreamBufferMemory", ctypes.c_void_p),
        ("m_uDeviceBlockSize", ctypes.c_uint32),
        ("m_pUserReadBuffer", ctypes.c_void_p),
        ("m_pStream", ctypes.c_ulonglong),
        ("m_fThroughput", ctypes.c_float),
        ("m_priority", ctypes.c_char),
        ("m_pMemoryFile", ctypes.c_void_p),
        ("m_bIsInitDone", ctypes.c_ubyte),
    ]
    m_pStreamBuffers: list[ctypes.c_void_p]
    m_pStreamReadBuffer: int
    m_uBufferBytesRemaining: list[ctypes.c_uint32]
    m_uCurrentBufferIdx: int
    m_uStreamBufferSize: int
    m_pStreamBufferMemory: int
    m_uDeviceBlockSize: int
    m_pUserReadBuffer: int
    m_pStream: int
    m_fThroughput: float
    m_priority: str
    m_pMemoryFile: int
    m_bIsInitDone: bool

class CAkBankCallbackMgr(ctypes.Structure):
    def CAkBankCallbackMgr(): pass

    def tilde_CAkBankCallbackMgr(): pass

    def AddCookie(): pass

    def CancelCookie(): pass

    def DoCallback(): pass

    _fields_ = [
        ("m_cookies", ctypes.c_void_p),
    ]
    m_cookies: int

class CurrentValue(ctypes.Structure):
    def GetModulatorOutput(): pass

    def ModulatorSupportsAutomatedParams(): pass


class ComponentInstancercTkDynamicChainComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class AkPerf(ctypes.Structure):
    pass

class CAkRSNode(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def GetInitialLoopCountLoop(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_pParent", ctypes.c_ulonglong),
        ("m_ID", ctypes.c_uint32),
        ("m_Loop", ctypes.c_int16),
        ("m_LoopMin", ctypes.c_int16),
        ("m_LoopMax", ctypes.c_int16),
        ("m_Weight", ctypes.c_uint32),
    ]
    __vftable: int
    m_pParent: int
    m_ID: int
    m_Loop: int
    m_LoopMin: int
    m_LoopMax: int
    m_Weight: int

class CAkRSNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkRSNode_dtor_0", ctypes.c_ulonglong),
        ("IsSegment", ctypes.c_ulonglong),
    ]
    CAkRSNode_dtor_0: int
    IsSegment: int

class CAkRSSub_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkRSNode_dtor_0", ctypes.c_ulonglong),
        ("IsSegment", ctypes.c_ulonglong),
    ]
    CAkRSNode_dtor_0: int
    IsSegment: int

class CAkRSSegment_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkRSNode_dtor_0", ctypes.c_ulonglong),
        ("IsSegment", ctypes.c_ulonglong),
    ]
    CAkRSNode_dtor_0: int
    IsSegment: int

class CAkRandomInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkContainerBaseInfo_dtor_0", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("SerializeHistory", ctypes.c_ulonglong),
        ("DeserializeHistory", ctypes.c_ulonglong),
    ]
    CAkContainerBaseInfo_dtor_0: int
    Destroy: int
    Type: int
    Clone: int
    SerializeHistory: int
    DeserializeHistory: int

class CAkSequenceInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkContainerBaseInfo_dtor_0", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("SerializeHistory", ctypes.c_ulonglong),
        ("DeserializeHistory", ctypes.c_ulonglong),
    ]
    CAkContainerBaseInfo_dtor_0: int
    Destroy: int
    Type: int
    Clone: int
    SerializeHistory: int
    DeserializeHistory: int

class CAkParameterEQFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkParametricEQFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class CAkPitchShifterFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkPitchShifterFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class AkPeakLimiterRTPCParams(ctypes.Structure):
    _fields_ = [
        ("fThreshold", ctypes.c_float),
        ("fRatio", ctypes.c_float),
        ("fRelease", ctypes.c_float),
        ("fOutputLevel", ctypes.c_float),
        ("bDirty", ctypes.c_ubyte),
    ]
    fThreshold: float
    fRatio: float
    fRelease: float
    fOutputLevel: float
    bDirty: bool

class AkPeakLimiterNonRTPCParams(ctypes.Structure):
    _fields_ = [
        ("fLookAhead", ctypes.c_float),
        ("bProcessLFE", ctypes.c_ubyte),
        ("bChannelLink", ctypes.c_ubyte),
        ("bDirty", ctypes.c_ubyte),
    ]
    fLookAhead: float
    bProcessLFE: bool
    bChannelLink: bool
    bDirty: bool

class CAkPeakLimiterFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class AlgorithmTunings(ctypes.Structure):
    _fields_ = [
        ("fDensityDelayMin", ctypes.c_float),
        ("fDensityDelayMax", ctypes.c_float),
        ("fDensityDelayRdmPerc", ctypes.c_float),
        ("fRoomShapeMin", ctypes.c_float),
        ("fRoomShapeMax", ctypes.c_float),
        ("fDiffusionDelayScalePerc", ctypes.c_float),
        ("fDiffusionDelayMax", ctypes.c_float),
        ("fDiffusionDelayRdmPerc", ctypes.c_float),
        ("fDCFilterCutFreq", ctypes.c_float),
        ("fReverbUnitInputDelay", ctypes.c_float),
        ("fReverbUnitInputDelayRmdPerc", ctypes.c_float),
    ]
    fDensityDelayMin: float
    fDensityDelayMax: float
    fDensityDelayRdmPerc: float
    fRoomShapeMin: float
    fRoomShapeMax: float
    fDiffusionDelayScalePerc: float
    fDiffusionDelayMax: float
    fDiffusionDelayRdmPerc: float
    fDCFilterCutFreq: float
    fReverbUnitInputDelay: float
    fReverbUnitInputDelayRmdPerc: float

class AkPeakLimiterSideChain(ctypes.Structure):
    _fields_ = [
        ("fGainDb", ctypes.c_float),
        ("fCurrentPeak", ctypes.c_float),
        ("fNextPeak", ctypes.c_float),
        ("uPeakTimer", ctypes.c_uint32),
    ]
    fGainDb: float
    fCurrentPeak: float
    fNextPeak: float
    uPeakTimer: int

class CAkPeakLimiterFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class RTPCParams(ctypes.Structure):
    _fields_ = [
        ("fDecayTime", ctypes.c_float),
        ("fHFDamping", ctypes.c_float),
        ("fDiffusion", ctypes.c_float),
        ("fStereoWidth", ctypes.c_float),
        ("fFilter1Gain", ctypes.c_float),
        ("fFilter1Freq", ctypes.c_float),
        ("fFilter1Q", ctypes.c_float),
        ("fFilter2Gain", ctypes.c_float),
        ("fFilter2Freq", ctypes.c_float),
        ("fFilter2Q", ctypes.c_float),
        ("fFilter3Gain", ctypes.c_float),
        ("fFilter3Freq", ctypes.c_float),
        ("fFilter3Q", ctypes.c_float),
        ("fFrontLevel", ctypes.c_float),
        ("fRearLevel", ctypes.c_float),
        ("fCenterLevel", ctypes.c_float),
        ("fLFELevel", ctypes.c_float),
        ("fDryLevel", ctypes.c_float),
        ("fERLevel", ctypes.c_float),
        ("fReverbLevel", ctypes.c_float),
        ("bDirty", ctypes.c_ubyte),
    ]
    fDecayTime: float
    fHFDamping: float
    fDiffusion: float
    fStereoWidth: float
    fFilter1Gain: float
    fFilter1Freq: float
    fFilter1Q: float
    fFilter2Gain: float
    fFilter2Freq: float
    fFilter2Q: float
    fFilter3Gain: float
    fFilter3Freq: float
    fFilter3Q: float
    fFrontLevel: float
    fRearLevel: float
    fCenterLevel: float
    fLFELevel: float
    fDryLevel: float
    fERLevel: float
    fReverbLevel: float
    bDirty: bool

class CAkRoomVerbFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class cGcEggMachineComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("IsRepair", ctypes.c_ulonglong),
        ("IsCooker", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("GetModelViewerRenderData", ctypes.c_ulonglong),
        ("RefreshPersistency", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("DeleteSavedState", ctypes.c_ulonglong),
        ("IsNetworkLockingType", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Seed", ctypes.c_ulonglong),
        ("CustomSlotUpdate", ctypes.c_ulonglong),
        ("CustomSlotRequirementsMet", ctypes.c_ulonglong),
        ("ItemAmountAccumulatorActive", ctypes.c_ulonglong),
        ("GetSavePosition", ctypes.c_char * 0x8),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation (__fastcall *)(cGcMaintenanceComponent *this, cTkVector3 *)
        ("GetPersistancyData", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    IsRepair: int
    IsCooker: int
    Interact: int
    GetModelViewerRenderData: int
    RefreshPersistency: int
    Save: int
    DeleteSavedState: int
    IsNetworkLockingType: int
    Setup: int
    Seed: int
    CustomSlotUpdate: int
    CustomSlotRequirementsMet: int
    ItemAmountAccumulatorActive: int
    GetSavePosition: bytes
    GetPersistancyData: int

class AkFXSrcSilenceParams(ctypes.Structure):
    _fields_ = [
        ("fDuration", ctypes.c_float),
        ("fRandomizedLengthMinus", ctypes.c_float),
        ("fRandomizedLengthPlus", ctypes.c_float),
    ]
    fDuration: float
    fRandomizedLengthMinus: float
    fRandomizedLengthPlus: float

class CAkFxSrcSilenceParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class AkSineFXParams(ctypes.Structure):
    _fields_ = [
        ("fFrequency", ctypes.c_float),
        ("fGain", ctypes.c_float),
        ("fDuration", ctypes.c_float),
        ("uChannelMask", ctypes.c_uint32),
    ]
    fFrequency: float
    fGain: float
    fDuration: float
    uChannelMask: int

class CAkFXSrcSilence_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("GetEnvelope", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    GetDuration: int
    GetEnvelope: int
    StopLooping: int
    Seek: int
    TimeSkip: int
    Execute: int

class CAkFxSrcSineParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class sTkOpenVRActionSet(ctypes.Structure):
    def Prepare(): pass

    _fields_ = [
        ("mpActionSetInfo", ctypes.c_ulonglong),
        ("mID", ctypes.c_ulonglong),
        ("mPath", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mhActionSet", ctypes.c_uint64),
        ("mActions", ctypes.c_char * 0x10),  # cTkDynamicArray<sTkOpenVRAction *>
    ]
    mpActionSetInfo: int
    mID: int
    mPath: bytes
    mhActionSet: int
    mActions: bytes

class sTkOpenVRAction(ctypes.Structure):
    def GetInputAxisValues(): pass

    def Prepare(): pass

    def Update(): pass

    _fields_ = [
        ("mpActionInfo", ctypes.c_ulonglong),
        ("mpActionSet", ctypes.c_ulonglong),
        ("mpPairingXY", ctypes.c_ulonglong),
        ("mpPairingX", ctypes.c_ulonglong),
        ("mpPairingY", ctypes.c_ulonglong),
        ("mhAnalogueAction", ctypes.c_uint64),
        ("mhDigitalAction", ctypes.c_uint64),
        ("mpIcons", ctypes.c_ulonglong),
        ("miLastUpdated", ctypes.c_int32),
    ]
    mpActionInfo: int
    mpActionSet: int
    mpPairingXY: int
    mpPairingX: int
    mpPairingY: int
    mhAnalogueAction: int
    mhDigitalAction: int
    mpIcons: int
    miLastUpdated: int

class sTkOpenVRVibrationHandles(ctypes.Structure):
    _fields_ = [
        ("mhLeft", ctypes.c_uint64),
        ("mhRight", ctypes.c_uint64),
    ]
    mhLeft: int
    mhRight: int

class sTkOpenVRData(ctypes.Structure):
    def FindControllerSpecPath(): pass

    def Prepare(): pass

    def UpdateOpenVRActionSetState(): pass

    _fields_ = [
        ("mActionSets", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,sTkOpenVRActionSet *,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mVibrationOutputs", ctypes.c_char * 0x40),  # robin_hood::detail::Table<1,80,TkID<128>,sTkOpenVRVibrationHandles *,robin_hood::hash<TkID<128>,void>,std::equal_to<TkID<128> >,-1>
        ("mpControllerLookup", ctypes.c_ulonglong),
        ("mpActiveActionSet", ctypes.c_ulonglong),
        ("miNow", ctypes.c_int32),
        ("miLastActive", ctypes.c_int32),
        ("mbInputDataAvailable", ctypes.c_ubyte),
        ("mfLastTimeAvailable", ctypes.c_float),
        ("mbControllersAvailable", ctypes.c_ubyte),
        ("mbIsInMenus", ctypes.c_ubyte),
    ]
    mActionSets: bytes
    mVibrationOutputs: bytes
    mpControllerLookup: int
    mpActiveActionSet: int
    miNow: int
    miLastActive: int
    mbInputDataAvailable: bool
    mfLastTimeAvailable: float
    mbControllersAvailable: bool
    mbIsInMenus: bool

class cTkOpenVRDeviceBridge_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkInputDevice_dtor_0", ctypes.c_ulonglong),
        ("IsAvailable", ctypes.c_ulonglong),
        ("ActivityCheck", ctypes.c_ulonglong),
        ("PullInput", ctypes.c_ulonglong),
        ("PushOutput", ctypes.c_ulonglong),
    ]
    ITkInputDevice_dtor_0: int
    IsAvailable: int
    ActivityCheck: int
    PullInput: int
    PushOutput: int

class cTkInputManagerOpenVR_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkInputManager_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("GetButtonInput", ctypes.c_ulonglong),
        ("GetAnalogInput", ctypes.c_ulonglong),
        ("GetLastButtonInputWasTouch", ctypes.c_ulonglong),
        ("SetTouchInputWaitingForReset", ctypes.c_ulonglong),
        ("HasDigitalActionBound", ctypes.c_ulonglong),
        ("HasAnalogActionBound", ctypes.c_ulonglong),
        ("GetDeviceVRViewResetLayerName", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cTkInputManager *this)
        ("GetActionButton", ctypes.c_ulonglong),
        ("GetActionAxis", ctypes.c_ulonglong),
        ("GetMappedIcon", ctypes.c_ulonglong),
        ("GetChordIcons", ctypes.c_char * 0x8),  # std::vector<sTkInputDeviceChordIcons,TkSTLAllocatorShim<sTkInputDeviceChordIcons,8,-1> > *(__fastcall *)(cTkInputManager *this)
        ("SetActiveActionSet", ctypes.c_ulonglong),
        ("ShowRemapScreen", ctypes.c_ulonglong),
    ]
    cTkInputManager_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    GetButtonInput: int
    GetAnalogInput: int
    GetLastButtonInputWasTouch: int
    SetTouchInputWaitingForReset: int
    HasDigitalActionBound: int
    HasAnalogActionBound: int
    GetDeviceVRViewResetLayerName: bytes
    GetActionButton: int
    GetActionAxis: int
    GetMappedIcon: int
    GetChordIcons: bytes
    SetActiveActionSet: int
    ShowRemapScreen: int

class cGcRefinerUnitComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("IsRepair", ctypes.c_ulonglong),
        ("IsCooker", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("GetModelViewerRenderData", ctypes.c_ulonglong),
        ("RefreshPersistency", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("DeleteSavedState", ctypes.c_ulonglong),
        ("IsNetworkLockingType", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Seed", ctypes.c_ulonglong),
        ("CustomSlotUpdate", ctypes.c_ulonglong),
        ("CustomSlotRequirementsMet", ctypes.c_ulonglong),
        ("ItemAmountAccumulatorActive", ctypes.c_ulonglong),
        ("GetSavePosition", ctypes.c_char * 0x8),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation (__fastcall *)(cGcMaintenanceComponent *this, cTkVector3 *)
        ("GetPersistancyData", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    IsRepair: int
    IsCooker: int
    Interact: int
    GetModelViewerRenderData: int
    RefreshPersistency: int
    Save: int
    DeleteSavedState: int
    IsNetworkLockingType: int
    Setup: int
    Seed: int
    CustomSlotUpdate: int
    CustomSlotRequirementsMet: int
    ItemAmountAccumulatorActive: int
    GetSavePosition: bytes
    GetPersistancyData: int

class cGcGeneratorUnitComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("IsRepair", ctypes.c_ulonglong),
        ("IsCooker", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("GetModelViewerRenderData", ctypes.c_ulonglong),
        ("RefreshPersistency", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("DeleteSavedState", ctypes.c_ulonglong),
        ("IsNetworkLockingType", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Seed", ctypes.c_ulonglong),
        ("CustomSlotUpdate", ctypes.c_ulonglong),
        ("CustomSlotRequirementsMet", ctypes.c_ulonglong),
        ("ItemAmountAccumulatorActive", ctypes.c_ulonglong),
        ("GetSavePosition", ctypes.c_char * 0x8),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation (__fastcall *)(cGcMaintenanceComponent *this, cTkVector3 *)
        ("GetPersistancyData", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    IsRepair: int
    IsCooker: int
    Interact: int
    GetModelViewerRenderData: int
    RefreshPersistency: int
    Save: int
    DeleteSavedState: int
    IsNetworkLockingType: int
    Setup: int
    Seed: int
    CustomSlotUpdate: int
    CustomSlotRequirementsMet: int
    ItemAmountAccumulatorActive: int
    GetSavePosition: bytes
    GetPersistancyData: int

class cGcCreatureFeederComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("IsRepair", ctypes.c_ulonglong),
        ("IsCooker", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("GetModelViewerRenderData", ctypes.c_ulonglong),
        ("RefreshPersistency", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("DeleteSavedState", ctypes.c_ulonglong),
        ("IsNetworkLockingType", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Seed", ctypes.c_ulonglong),
        ("CustomSlotUpdate", ctypes.c_ulonglong),
        ("CustomSlotRequirementsMet", ctypes.c_ulonglong),
        ("ItemAmountAccumulatorActive", ctypes.c_ulonglong),
        ("GetSavePosition", ctypes.c_char * 0x8),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation (__fastcall *)(cGcMaintenanceComponent *this, cTkVector3 *)
        ("GetPersistancyData", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    IsRepair: int
    IsCooker: int
    Interact: int
    GetModelViewerRenderData: int
    RefreshPersistency: int
    Save: int
    DeleteSavedState: int
    IsNetworkLockingType: int
    Setup: int
    Seed: int
    CustomSlotUpdate: int
    CustomSlotRequirementsMet: int
    ItemAmountAccumulatorActive: int
    GetSavePosition: bytes
    GetPersistancyData: int

class cGcCreatureHarvesterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("IsRepair", ctypes.c_ulonglong),
        ("IsCooker", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("GetModelViewerRenderData", ctypes.c_ulonglong),
        ("RefreshPersistency", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("DeleteSavedState", ctypes.c_ulonglong),
        ("IsNetworkLockingType", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Seed", ctypes.c_ulonglong),
        ("CustomSlotUpdate", ctypes.c_ulonglong),
        ("CustomSlotRequirementsMet", ctypes.c_ulonglong),
        ("ItemAmountAccumulatorActive", ctypes.c_ulonglong),
        ("GetSavePosition", ctypes.c_char * 0x8),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation (__fastcall *)(cGcMaintenanceComponent *this, cTkVector3 *)
        ("GetPersistancyData", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    IsRepair: int
    IsCooker: int
    Interact: int
    GetModelViewerRenderData: int
    RefreshPersistency: int
    Save: int
    DeleteSavedState: int
    IsNetworkLockingType: int
    Setup: int
    Seed: int
    CustomSlotUpdate: int
    CustomSlotRequirementsMet: int
    ItemAmountAccumulatorActive: int
    GetSavePosition: bytes
    GetPersistancyData: int

class AkSwitchSubscriptionKey(ctypes.Structure):
    _fields_ = [
        ("pSubscriber", ctypes.c_ulonglong),
    ]
    pSubscriber: int

class cGcAnimatedLayer(ctypes.Structure):
    _fields_ = [
        ("mpLayer", ctypes.c_ulonglong),
        ("mpMask", ctypes.c_ulonglong),
        ("mfTimer", ctypes.c_float),
        ("mbActive", ctypes.c_ubyte),
        ("mScale", common.Vector2f),
    ]
    mpLayer: int
    mpMask: int
    mfTimer: float
    mbActive: bool
    mScale: common.Vector2f

class CAkPathManager(ctypes.Structure):
    def CAkPathManager(): pass

    def AddPathToList(): pass

    def AddPathUser(): pass

    def AddPotentialUser(): pass

    def Init(): pass

    def Pause(): pass

    def ProcessPathsList(): pass

    def RemovePathFromList(): pass

    def RemovePathUser(): pass

    def RemovePotentialUser(): pass

    def Resume(): pass

    def SetPathsList(): pass

    def Start(): pass

    def Term(): pass

    _fields_ = [
        ("m_ActivePathsList", ctypes.c_char * 0x10),  # AkArray<CAkPath *,CAkPath *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_NoGrow,AkAssignmentMovePolicy<CAkPath *> >
        ("m_uMaxPathNumber", ctypes.c_uint32),
    ]
    m_ActivePathsList: bytes
    m_uMaxPathNumber: int

class AkFXMemAlloc_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPluginMemAlloc_dtor_0", ctypes.c_ulonglong),
        ("Malloc", ctypes.c_ulonglong),
        ("Free", ctypes.c_ulonglong),
        ("Malign", ctypes.c_ulonglong),
        ("Realloc", ctypes.c_ulonglong),
        ("ReallocAligned", ctypes.c_ulonglong),
    ]
    IAkPluginMemAlloc_dtor_0: int
    Malloc: int
    Free: int
    Malign: int
    Realloc: int
    ReallocAligned: int

class cGcShipHUDTargetIcon(ctypes.Structure):
    _fields_ = [
        ("mCorner", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mLineVertical", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mLineHorizontal", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mGlowCorner", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mGlowLineVertical", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mGlowLineHorizontal", ctypes.c_char * 0x4),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID>
        ("mfGlowAlpha", ctypes.c_float),
    ]
    mCorner: bytes
    mLineVertical: bytes
    mLineHorizontal: bytes
    mGlowCorner: bytes
    mGlowLineVertical: bytes
    mGlowLineHorizontal: bytes
    mfGlowAlpha: float

class GLNVGtexture(ctypes.Structure):
    def SetTexture(): pass

    _fields_ = [
        ("id", ctypes.c_int32),
        ("mTexture", ctypes.c_ulonglong),
        ("miRenderBufferObject", ctypes.c_uint32),
        ("type", ctypes.c_int32),
        ("flags", ctypes.c_int32),
    ]
    id: int
    mTexture: int
    miRenderBufferObject: int
    type: int
    flags: int

class GLNVGcall(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_int32),
        ("image", ctypes.c_int32),
        ("pathOffset", ctypes.c_int32),
        ("pathCount", ctypes.c_int32),
        ("triangleOffset", ctypes.c_int32),
        ("triangleCount", ctypes.c_int32),
        ("uniformOffset", ctypes.c_int32),
    ]
    type: int
    image: int
    pathOffset: int
    pathCount: int
    triangleOffset: int
    triangleCount: int
    uniformOffset: int

class GLNVGpath(ctypes.Structure):
    _fields_ = [
        ("fillOffset", ctypes.c_int32),
        ("fillCount", ctypes.c_int32),
        ("strokeOffset", ctypes.c_int32),
        ("strokeCount", ctypes.c_int32),
    ]
    fillOffset: int
    fillCount: int
    strokeOffset: int
    strokeCount: int

class cTkCharacterClosestHitCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class cTkCharacterProxyClosestHitCollector_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("reset", ctypes.c_ulonglong),
        ("getHits", ctypes.c_ulonglong),
        ("addHit", ctypes.c_ulonglong),
        ("processClosestPointsCvxCvx", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    reset: int
    getHits: int
    addHit: int
    processClosestPointsCvxCvx: int

class GLNVGfragUniforms(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0xC0),  # $F2670081C13DB3E77AF5126AD132D0A1
    ]
    ___u0: bytes

class tagOFNA(ctypes.Structure):
    _fields_ = [
        ("lStructSize", ctypes.c_uint32),
        ("hwndOwner", ctypes.c_ulonglong),
        ("hInstance", ctypes.c_ulonglong),
        ("lpstrFilter", ctypes.c_char_p),
        ("lpstrCustomFilter", ctypes.c_char_p),
        ("nMaxCustFilter", ctypes.c_uint32),
        ("nFilterIndex", ctypes.c_uint32),
        ("lpstrFile", ctypes.c_char_p),
        ("nMaxFile", ctypes.c_uint32),
        ("lpstrFileTitle", ctypes.c_char_p),
        ("nMaxFileTitle", ctypes.c_uint32),
        ("lpstrInitialDir", ctypes.c_char_p),
        ("lpstrTitle", ctypes.c_char_p),
        ("Flags", ctypes.c_uint32),
        ("nFileOffset", ctypes.c_uint16),
        ("nFileExtension", ctypes.c_uint16),
        ("lpstrDefExt", ctypes.c_char_p),
        ("lCustData", ctypes.c_int64),
        ("lpfnHook", ctypes.c_ulonglong),
        ("lpTemplateName", ctypes.c_char_p),
        ("pvReserved", ctypes.c_void_p),
        ("dwReserved", ctypes.c_uint32),
        ("FlagsEx", ctypes.c_uint32),
    ]
    lStructSize: int
    hwndOwner: int
    hInstance: int
    lpstrFilter: bytes
    lpstrCustomFilter: bytes
    nMaxCustFilter: int
    nFilterIndex: int
    lpstrFile: bytes
    nMaxFile: int
    lpstrFileTitle: bytes
    nMaxFileTitle: int
    lpstrInitialDir: bytes
    lpstrTitle: bytes
    Flags: int
    nFileOffset: int
    nFileExtension: int
    lpstrDefExt: bytes
    lCustData: int
    lpfnHook: int
    lpTemplateName: bytes
    pvReserved: int
    dwReserved: int
    FlagsEx: int

class cTkStringFunctions(ctypes.Structure):
    pass

class cGcMissionPhotoListener_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcPhotoListener_dtor_0", ctypes.c_ulonglong),
        ("NotifySuccess", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
    ]
    cGcPhotoListener_dtor_0: int
    NotifySuccess: int
    Clone: int

class cGcNotificationSequenceStageGroup_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceUrgent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWait_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceDisplaySeasonRewardReminder_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitRealTime_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitRealTimeCombat_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCollectMoney_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceMissionUpdateMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequencePodMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceShowSeasonTimeWarning_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceDetailMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceStop_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCloseMenu_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCreateSpecificPulseEncounter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForAbandFreighterDoorOpen_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceRepairTech_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGatherFor_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("GatherTargetAlreadyAcheived", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    GatherTargetAlreadyAcheived: int

class cGcNotificationSequenceGatherForCrafting_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("GatherTargetAlreadyAcheived", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    GatherTargetAlreadyAcheived: int

class cGcNotificationSequenceGatherListForCrafting_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGatherForRepair_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("GatherTargetAlreadyAcheived", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    GatherTargetAlreadyAcheived: int

class cGcNotificationSequenceGatherForBuild_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("GatherTargetAlreadyAcheived", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    GatherTargetAlreadyAcheived: int

class cGcNotificationSequenceWaitForLearnedProduct_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForLearnedTech_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGatherForRefuel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("GatherTargetAlreadyAcheived", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    GatherTargetAlreadyAcheived: int

class TimerStreamInfo(ctypes.Structure):
    _fields_ = [
        ("m_streamBegin", ctypes.c_char_p),
        ("m_streamEnd", ctypes.c_char_p),
    ]
    m_streamBegin: bytes
    m_streamEnd: bytes

class cGcNotificationSequenceProductAmountNeeded_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceUnitsNeeded_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGetUnits_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceRepairFinal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceBuildFinal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCraftFinal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCleanUpPinnedRecipe_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceRefuelFinal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceStartEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceEndEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceStartEventFromShipScan_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceScan_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceVehicleScan_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGetToScanEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForScanEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cTkCreatureTailComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcNotificationSequenceWaitForFriendlyDroneScanEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCompleteSettlementJudgement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceConstructSettlementBuildingWithScanEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForSettlementActivity_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceEnsureBarrelsAtPlayerSettlement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceBaseBuildingBuildPart_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGetInShip_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceSignalGalacticPoint_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGoToGalacticPoint_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForCompletionMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceBroadcastMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForSuitUpgrade_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceSetCurrentMission_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceStartMission_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceClearInventoryHistory_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceAudioEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceConstruct_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceSetGlobalMission_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForPhoto_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceGetToExpedition_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCompleteMission_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceLeaveNexusMP_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceVisitPlanets_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceSendChatMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceModifyStat_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCompleteSeasonalMilestone_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceExploreAbandonedFreighter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceExplorationLogSpecial_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceKillEncounter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class CAkSpatialAudioEmitterVoiceCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("Init", ctypes.c_ulonglong),
        ("CAkSpatialAudioVoice_dtor_8", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("ParamsUpdated", ctypes.c_ulonglong),
        ("GetAuxSendsValues", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkSpatialAudioVoice *this, AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *, float, float, float)
        ("HasReflections", ctypes.c_ulonglong),
        ("HasDiffraction", ctypes.c_ulonglong),
        ("HasTransmission", ctypes.c_ulonglong),
        ("IsReflectionsBus", ctypes.c_ulonglong),
    ]
    Init: int
    CAkSpatialAudioVoice_dtor_8: int
    Term: int
    ParamsUpdated: int
    GetAuxSendsValues: bytes
    HasReflections: int
    HasDiffraction: int
    HasTransmission: int
    IsReflectionsBus: int

class AkAuxBusRef(ctypes.Structure):
    _fields_ = [
        ("key", ctypes.c_uint32),
        ("refCount", ctypes.c_int32),
    ]
    key: int
    refCount: int

class CAkSpatialAudioComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class CAkSpatialAudioReflectBusCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("Init", ctypes.c_ulonglong),
        ("CAkSpatialAudioVoice_dtor_8", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("ParamsUpdated", ctypes.c_ulonglong),
        ("GetAuxSendsValues", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkSpatialAudioVoice *this, AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *, float, float, float)
        ("HasReflections", ctypes.c_ulonglong),
        ("HasDiffraction", ctypes.c_ulonglong),
        ("HasTransmission", ctypes.c_ulonglong),
        ("IsReflectionsBus", ctypes.c_ulonglong),
    ]
    Init: int
    CAkSpatialAudioVoice_dtor_8: int
    Term: int
    ParamsUpdated: int
    GetAuxSendsValues: bytes
    HasReflections: int
    HasDiffraction: int
    HasTransmission: int
    IsReflectionsBus: int

class AkMarkerNotificationRange(ctypes.Structure):
    _fields_ = [
        ("pMarkers", ctypes.c_ulonglong),
        ("uLength", ctypes.c_uint32),
    ]
    pMarkers: int
    uLength: int

class AkMarkersHeader(ctypes.Structure):
    _fields_ = [
        ("uNumMarkers", ctypes.c_uint32),
    ]
    uNumMarkers: int

class cTkDynamicChainComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class IAkSrcMediaCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkSrcMediaCodec_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Warmup", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("FindClosestFileOffset", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("SwitchToNext", ctypes.c_ulonglong),
    ]
    IAkSrcMediaCodec_dtor_0: int
    Init: int
    Warmup: int
    Term: int
    PrepareNextBuffer: int
    GetBuffer: int
    ReleaseBuffer: int
    VirtualOn: int
    VirtualOff: int
    FindClosestFileOffset: int
    Seek: int
    StopLooping: int
    SwitchToNext: int

class CAkSrcMedia_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Seek_60", ctypes.c_ulonglong),
        ("StartStream", ctypes.c_ulonglong),
        ("StopStream", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("IsUsingThisSlot", ctypes.c_ulonglong),
        ("IsUsingThisSlot_98", ctypes.c_ulonglong),
        ("NotifySourceStarvation", ctypes.c_ulonglong),
        ("SupportResampling", ctypes.c_ulonglong),
        ("PrepareSourceSwitch", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("MustRelocatePitchInputBufferOnMediaRelocation", ctypes.c_ulonglong),
        ("MustRelocateAnalysisDataOnMediaRelocation", ctypes.c_ulonglong),
        ("ChangeSourcePosition", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("SrcProcessOrder", ctypes.c_ulonglong),
        ("RestartSourcePlugin", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Term: int
    Seek_60: int
    StartStream: int
    StopStream: int
    GetBuffer: int
    GetDuration: int
    StopLooping: int
    IsUsingThisSlot: int
    IsUsingThisSlot_98: int
    NotifySourceStarvation: int
    SupportResampling: int
    PrepareSourceSwitch: int
    SupportMediaRelocation: int
    RelocateMedia: int
    MustRelocatePitchInputBufferOnMediaRelocation: int
    MustRelocateAnalysisDataOnMediaRelocation: int
    ChangeSourcePosition: int
    GetAnalyzedEnvelope: int
    SrcProcessOrder: int
    RestartSourcePlugin: int
    PrepareNextBuffer: int

class AkRoomGameObjectAware(ctypes.Structure):
    _fields_ = [
        ("m_pGameObject", ctypes.c_ulonglong),
    ]
    m_pGameObject: int

class _C_float_complex(ctypes.Structure):
    _fields_ = [
        ("_Val", ctypes.c_float * 0x2),  # float[2]
    ]
    _Val: list[ctypes.c_float]

class _C_ldouble_complex(ctypes.Structure):
    _fields_ = [
        ("_Val", ctypes.c_double * 0x2),  # long double[2]
    ]
    _Val: list[ctypes.c_double]

class CAkSpatialAudioRoomComponent(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # CAkTrackedGameObjComponent<9>
        ("m_pSpatialAudioRoom", ctypes.c_ulonglong),
        ("m_auxID", ctypes.c_uint32),
        ("m_bIsAuxBusActive", ctypes.c_ubyte),
    ]
    baseclass_0: bytes
    m_pSpatialAudioRoom: int
    m_auxID: int
    m_bIsAuxBusActive: bool

class CAkSpatialAudioRoomComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class _C_double_complex(ctypes.Structure):
    _fields_ = [
        ("_Val", ctypes.c_double * 0x2),  # long double[2]
    ]
    _Val: list[ctypes.c_double]

class CAkSrcMediaCodecPCM_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkSrcMediaCodec_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Warmup", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("FindClosestFileOffset", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("SwitchToNext", ctypes.c_ulonglong),
    ]
    IAkSrcMediaCodec_dtor_0: int
    Init: int
    Warmup: int
    Term: int
    PrepareNextBuffer: int
    GetBuffer: int
    ReleaseBuffer: int
    VirtualOn: int
    VirtualOff: int
    FindClosestFileOffset: int
    Seek: int
    StopLooping: int
    SwitchToNext: int

class AkRayBuilder(ctypes.Structure):
    def BuildApiRays(): pass

    def SubmitLast(): pass

    _fields_ = [
        ("pRayComponent", ctypes.c_ulonglong),
        ("iShortest", ctypes.c_uint32),
        ("minDistance", ctypes.c_float),
        ("listeners", ctypes.c_char * 0x30),  # AkSet<unsigned __int64,AkHybridAllocator<32,16,4>,AkGrowByPolicy_Proportional>
        ("submitCount", ctypes.c_uint32),
        ("minDiffraction", ctypes.c_float),
        ("transmissionLoss", ctypes.c_float),
        ("bConsiderRayForBuiltInParams", ctypes.c_ubyte),
    ]
    pRayComponent: int
    iShortest: int
    minDistance: float
    listeners: bytes
    submitCount: int
    minDiffraction: float
    transmissionLoss: float
    bConsiderRayForBuiltInParams: bool

class CAkVolumeRayComponent(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    def tilde_CAkVolumeRayComponent(): pass

    def BuildCachedVolumeRays(): pass

    def GetRayVolumeData(): pass

    def UpdateBuiltInParamValues(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x20),  # CAkTrackedGameObjComponent<2>
        ("m_rays", ctypes.c_char * 0x10),  # AkArray<AkRayVolumeData,AkRayVolumeData const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkRayVolumeData> >
        ("m_categories", ctypes.c_char * 0x10),  # AkArray<unsigned char,unsigned char,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned char> >
        ("m_rayBuilderHook", ctypes.c_ulonglong),
        ("m_builtInUsage", ctypes.c_uint16),
    ]
    baseclass_0: bytes
    m_rays: bytes
    m_categories: bytes
    m_rayBuilderHook: int
    m_builtInUsage: int

class CAkVolumeRayComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkGameObjComponent_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
    ]
    CAkGameObjComponent_dtor_0: int
    Init: int

class AkDevice_vtbl(ctypes.Structure):
    _fields_ = [
        ("AkDevice_dtor_0", ctypes.c_ulonglong),
    ]
    AkDevice_dtor_0: int

class CAkCompressorFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int

class CAkSpatialAudioRoomBusCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("Init", ctypes.c_ulonglong),
        ("CAkSpatialAudioVoice_dtor_8", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("ParamsUpdated", ctypes.c_ulonglong),
        ("GetAuxSendsValues", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkSpatialAudioVoice *this, AkArray<AkAuxSendValueEx,AkAuxSendValueEx const &,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkAuxSendValueEx> > *, float, float, float)
        ("HasReflections", ctypes.c_ulonglong),
        ("HasDiffraction", ctypes.c_ulonglong),
        ("HasTransmission", ctypes.c_ulonglong),
        ("IsReflectionsBus", ctypes.c_ulonglong),
    ]
    Init: int
    CAkSpatialAudioVoice_dtor_8: int
    Term: int
    ParamsUpdated: int
    GetAuxSendsValues: bytes
    HasReflections: int
    HasDiffraction: int
    HasTransmission: int
    IsReflectionsBus: int

class AkFlangerNonRTPCParams(ctypes.Structure):
    _fields_ = [
        ("fDelayTime", ctypes.c_float),
        ("bEnableLFO", ctypes.c_ubyte),
        ("bProcessCenter", ctypes.c_ubyte),
        ("bProcessLFE", ctypes.c_ubyte),
        ("bHasChanged", ctypes.c_ubyte),
    ]
    fDelayTime: float
    bEnableLFO: bool
    bProcessCenter: bool
    bProcessLFE: bool
    bHasChanged: bool

class CAkFlangerFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class AkPipelineIDGenerator(ctypes.Structure):
    pass

class CAkModulatorBatchBase(ctypes.Structure):
    _fields_ = [
        ("pNextItem", ctypes.c_ulonglong),
    ]
    pNextItem: int

class CAkModulatorEngine(ctypes.Structure):
    def tilde_CAkModulatorEngine(): pass

    def AddContext(): pass

    def CleanUpFinishedCtxs(): pass

    def ProcessModulators(): pass

    _fields_ = [
        ("m_listCtx", ctypes.c_char * 0x18),  # AkListBare<CAkModulatorCtx,AkListBareNextItem,AkCountPolicyWithCount,AkLastPolicyWithLast>
        ("m_envBatches", ctypes.c_char * 0x18),  # AkListBare<CAkModulatorBatchBase,AkListBareNextItem,AkCountPolicyWithCount,AkLastPolicyWithLast>
        ("m_lfoBatches", ctypes.c_char * 0x18),  # AkListBare<CAkModulatorBatchBase,AkListBareNextItem,AkCountPolicyWithCount,AkLastPolicyWithLast>
        ("m_timBatches", ctypes.c_char * 0x18),  # AkListBare<CAkModulatorBatchBase,AkListBareNextItem,AkCountPolicyWithCount,AkLastPolicyWithLast>
    ]
    m_listCtx: bytes
    m_envBatches: bytes
    m_lfoBatches: bytes
    m_timBatches: bytes

class CAkModulatorMgr(ctypes.Structure):
    def CAkModulatorMgr(): pass

    def AddSubscription(): pass

    def CleanUpFinishedCtxs(): pass

    def GetCurrentModulatorOutput(): pass

    def GetModulators(): pass

    def GetParamXfrm(): pass

    def Init(): pass

    def ProcessModulators(): pass

    def RemoveSubscription(): pass

    def RemovedScopedRtpcObj(): pass

    def SupportsAutomatedParams(): pass

    def Term(): pass

    def TermModulatorEngine(): pass

    def Trigger(): pass

    _fields_ = [
        ("m_ModulatorSubscribers", ctypes.c_char * 0x18),  # AkHashListBare<unsigned __int64,AkModulatorArray,AkArrayAllocatorNoAlign<0>,AkDefaultHashListBarePolicy<unsigned __int64,AkModulatorArray> >
        ("m_pEngine", ctypes.c_ulonglong),
    ]
    m_ModulatorSubscribers: bytes
    m_pEngine: int

class AkLpfParamEval(ctypes.Structure):
    def EvalCutoff(): pass


class AkHpfParamEval(ctypes.Structure):
    def EvalCutoff(): pass


class CAkLFOModulator_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetInitialValues", ctypes.c_ulonglong),
        ("GetInitialParams", ctypes.c_ulonglong),
        ("SupportsAutomatedParams", ctypes.c_ulonglong),
        ("_Trigger", ctypes.c_ulonglong),
        ("ShouldRetrigger", ctypes.c_ulonglong),
        ("IsTriggerOk", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetInitialValues: int
    GetInitialParams: int
    SupportsAutomatedParams: int
    _Trigger: int
    ShouldRetrigger: int
    IsTriggerOk: int

class CAkEnvelopeModulator_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetInitialValues", ctypes.c_ulonglong),
        ("GetInitialParams", ctypes.c_ulonglong),
        ("SupportsAutomatedParams", ctypes.c_ulonglong),
        ("_Trigger", ctypes.c_ulonglong),
        ("ShouldRetrigger", ctypes.c_ulonglong),
        ("IsTriggerOk", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetInitialValues: int
    GetInitialParams: int
    SupportsAutomatedParams: int
    _Trigger: int
    ShouldRetrigger: int
    IsTriggerOk: int

class CAkTimeModulator_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetInitialValues", ctypes.c_ulonglong),
        ("GetInitialParams", ctypes.c_ulonglong),
        ("SupportsAutomatedParams", ctypes.c_ulonglong),
        ("_Trigger", ctypes.c_ulonglong),
        ("ShouldRetrigger", ctypes.c_ulonglong),
        ("IsTriggerOk", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetInitialValues: int
    GetInitialParams: int
    SupportsAutomatedParams: int
    _Trigger: int
    ShouldRetrigger: int
    IsTriggerOk: int

class CAkLFOCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkModulatorCtx_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Tick", ctypes.c_ulonglong),
        ("GetParamsPtr", ctypes.c_ulonglong),
        ("CalcBufferNeeded", ctypes.c_ulonglong),
        ("InitializeOutput", ctypes.c_ulonglong),
    ]
    CAkModulatorCtx_dtor_0: int
    SetParam: int
    Tick: int
    GetParamsPtr: int
    CalcBufferNeeded: int
    InitializeOutput: int

class CAkEnvelopeCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkModulatorCtx_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Tick", ctypes.c_ulonglong),
        ("GetParamsPtr", ctypes.c_ulonglong),
        ("CalcBufferNeeded", ctypes.c_ulonglong),
        ("InitializeOutput", ctypes.c_ulonglong),
    ]
    CAkModulatorCtx_dtor_0: int
    SetParam: int
    Tick: int
    GetParamsPtr: int
    CalcBufferNeeded: int
    InitializeOutput: int

class CAkTimeModCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkModulatorCtx_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Tick", ctypes.c_ulonglong),
        ("GetParamsPtr", ctypes.c_ulonglong),
        ("CalcBufferNeeded", ctypes.c_ulonglong),
        ("InitializeOutput", ctypes.c_ulonglong),
    ]
    CAkModulatorCtx_dtor_0: int
    SetParam: int
    Tick: int
    GetParamsPtr: int
    CalcBufferNeeded: int
    InitializeOutput: int

class CAkLFOProcess(ctypes.Structure):
    pass

class CommonOutputPolicy(ctypes.Structure):
    pass

class BufferOutputPolicy(ctypes.Structure):
    def Ramp(): pass


class SingleOutputPolicy(ctypes.Structure):
    pass

class CAkEnvelopeProcess(ctypes.Structure):
    pass

class CAkTimeModProcess(ctypes.Structure):
    pass

class CAkRoomVerbFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class CAkMidiDeviceMgr_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnNoteStateFinished", ctypes.c_ulonglong),
        ("UpdateOnFirstNote", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkMidiBaseMgr *this, AkListBare<AkMidiFrameEvent,AkListBareNextItem,AkCountPolicyNoCount,AkLastPolicyWithLast> *, AkMidiFrameEvent *)
    ]
    OnNoteStateFinished: int
    UpdateOnFirstNote: bytes

class CAkVirtualAcousticsMgr(ctypes.Structure):
    def AddAcousticTexture(): pass

    def GetAcousticTexture(): pass


class IMMNotificationClient_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("OnDeviceStateChanged", ctypes.c_ulonglong),
        ("OnDeviceAdded", ctypes.c_ulonglong),
        ("OnDeviceRemoved", ctypes.c_ulonglong),
        ("OnDefaultDeviceChanged", ctypes.c_ulonglong),
        ("OnPropertyValueChanged", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    OnDeviceStateChanged: int
    OnDeviceAdded: int
    OnDeviceRemoved: int
    OnDefaultDeviceChanged: int
    OnPropertyValueChanged: int

class IPropertyStore_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetCount", ctypes.c_ulonglong),
        ("GetAt", ctypes.c_ulonglong),
        ("GetValue", ctypes.c_ulonglong),
        ("SetValue", ctypes.c_ulonglong),
        ("Commit", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetCount: int
    GetAt: int
    GetValue: int
    SetValue: int
    Commit: int

class IMMDevice_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("OpenPropertyStore", ctypes.c_ulonglong),
        ("GetId", ctypes.c_ulonglong),
        ("GetState", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Activate: int
    OpenPropertyStore: int
    GetId: int
    GetState: int

class IMMDeviceCollection_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetCount", ctypes.c_ulonglong),
        ("Item", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetCount: int
    Item: int

class IMMDeviceEnumerator_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("EnumAudioEndpoints", ctypes.c_ulonglong),
        ("GetDefaultAudioEndpoint", ctypes.c_ulonglong),
        ("GetDevice", ctypes.c_ulonglong),
        ("RegisterEndpointNotificationCallback", ctypes.c_ulonglong),
        ("UnregisterEndpointNotificationCallback", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    EnumAudioEndpoints: int
    GetDefaultAudioEndpoint: int
    GetDevice: int
    RegisterEndpointNotificationCallback: int
    UnregisterEndpointNotificationCallback: int

class CAkWinDeviceCache_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("OnDeviceStateChanged", ctypes.c_ulonglong),
        ("OnDeviceAdded", ctypes.c_ulonglong),
        ("OnDeviceRemoved", ctypes.c_ulonglong),
        ("OnDefaultDeviceChanged", ctypes.c_ulonglong),
        ("OnPropertyValueChanged", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    OnDeviceStateChanged: int
    OnDeviceAdded: int
    OnDeviceRemoved: int
    OnDefaultDeviceChanged: int
    OnPropertyValueChanged: int

class CAkLEngineCmds(ctypes.Structure):
    def DequeuePBI(): pass

    def DestroyDisconnectedSources(): pass

    def EnqueueAction(): pass

    def Init(): pass

    def ProcessAllCommands(): pass

    def ProcessDisconnectedSources(): pass

    def ProcessPendingCommands(): pass

    def ProcessPlayCommands(): pass

    def Term(): pass


class IAkSourcePluginContextEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPluginContextBase_dtor_0", ctypes.c_ulonglong),
        ("GlobalContext", ctypes.c_ulonglong),
        ("GetGameObjectInfo", ctypes.c_ulonglong),
        ("GetOutputID", ctypes.c_ulonglong),
        ("GetPluginMedia", ctypes.c_ulonglong),
        ("GetPluginCustomGameData", ctypes.c_ulonglong),
        ("PostMonitorData", ctypes.c_ulonglong),
        ("CanPostMonitorData", ctypes.c_ulonglong),
        ("PostMonitorMessage", ctypes.c_ulonglong),
        ("GetDownstreamGain", ctypes.c_ulonglong),
        ("GetParentChannelConfig", ctypes.c_ulonglong),
        ("GetProcessorFeatures", ctypes.c_ulonglong),
        ("GetAudioNodeID", ctypes.c_ulonglong),
        ("GetNumLoops", ctypes.c_ulonglong),
        ("GetVoiceInfo", ctypes.c_ulonglong),
        ("GetMidiEvent", ctypes.c_ulonglong),
        ("GetCookie", ctypes.c_ulonglong),
        ("GetOutputBusNode", ctypes.c_ulonglong),
        ("AllocGrainCodec", ctypes.c_ulonglong),
        ("FreeGrainCodec", ctypes.c_ulonglong),
    ]
    IAkPluginContextBase_dtor_0: int
    GlobalContext: int
    GetGameObjectInfo: int
    GetOutputID: int
    GetPluginMedia: int
    GetPluginCustomGameData: int
    PostMonitorData: int
    CanPostMonitorData: int
    PostMonitorMessage: int
    GetDownstreamGain: int
    GetParentChannelConfig: int
    GetProcessorFeatures: int
    GetAudioNodeID: int
    GetNumLoops: int
    GetVoiceInfo: int
    GetMidiEvent: int
    GetCookie: int
    GetOutputBusNode: int
    AllocGrainCodec: int
    FreeGrainCodec: int

class CAkSrcPhysModel_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Seek_60", ctypes.c_ulonglong),
        ("StartStream", ctypes.c_ulonglong),
        ("StopStream", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("IsUsingThisSlot", ctypes.c_ulonglong),
        ("IsUsingThisSlot_98", ctypes.c_ulonglong),
        ("NotifySourceStarvation", ctypes.c_ulonglong),
        ("SupportResampling", ctypes.c_ulonglong),
        ("PrepareSourceSwitch", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("MustRelocatePitchInputBufferOnMediaRelocation", ctypes.c_ulonglong),
        ("MustRelocateAnalysisDataOnMediaRelocation", ctypes.c_ulonglong),
        ("ChangeSourcePosition", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("SrcProcessOrder", ctypes.c_ulonglong),
        ("RestartSourcePlugin", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
        ("GetMaxBufferLength", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Term: int
    Seek_60: int
    StartStream: int
    StopStream: int
    GetBuffer: int
    GetDuration: int
    StopLooping: int
    IsUsingThisSlot: int
    IsUsingThisSlot_98: int
    NotifySourceStarvation: int
    SupportResampling: int
    PrepareSourceSwitch: int
    SupportMediaRelocation: int
    RelocateMedia: int
    MustRelocatePitchInputBufferOnMediaRelocation: int
    MustRelocateAnalysisDataOnMediaRelocation: int
    ChangeSourcePosition: int
    GetAnalyzedEnvelope: int
    SrcProcessOrder: int
    RestartSourcePlugin: int
    PrepareNextBuffer: int
    GetMaxBufferLength: int

class AkSystemOutputRTPCParams(ctypes.Structure):
    pass

class AkSystemOutputMeta_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkThreadedBankMgr_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkBankMgr_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("QueueBankCommand", ctypes.c_ulonglong),
        ("InitSyncOp", ctypes.c_ulonglong),
        ("WaitForSyncOp", ctypes.c_ulonglong),
        ("StopThread", ctypes.c_ulonglong),
        ("CancelCookie", ctypes.c_ulonglong),
        ("KillSlot", ctypes.c_ulonglong),
    ]
    CAkBankMgr_dtor_0: int
    Init: int
    Term: int
    QueueBankCommand: int
    InitSyncOp: int
    WaitForSyncOp: int
    StopThread: int
    CancelCookie: int
    KillSlot: int

class AkSinkPluginParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class SystemSinkParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkVPLFilterNodeOutOfPlace_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("ReleaseMemory", ctypes.c_ulonglong),
        ("ReleaseInputBuffer", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ConsumeBuffer", ctypes.c_ulonglong),
        ("GetPlugin", ctypes.c_ulonglong),
        ("GetOutputConfig", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Init: int
    Term: int
    ReleaseMemory: int
    ReleaseInputBuffer: int
    GetBuffer: int
    ConsumeBuffer: int
    GetPlugin: int
    GetOutputConfig: int

class DummySinkParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class CAkAcousticTexture_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_28", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetAkProp: int
    SetAkProp_28: int

class CAkMixBusCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkParameterTarget_dtor_0", ctypes.c_ulonglong),
        ("UpdateTargetNodeParam", ctypes.c_ulonglong),
        ("NotifyParamChanged", ctypes.c_ulonglong),
        ("NotifyParamsChanged", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterTarget *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("GetTargetedParamsSet", ctypes.c_char * 0x8),  # CAkBitArray<unsigned __int64,0> *(__fastcall *)(CAkParameterTarget *this, CAkBitArray<unsigned __int64,0> *result)
        ("RegisterToBusHierarchy", ctypes.c_ulonglong),
        ("UpdateTargetParam", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("GetModulatorTriggerParams", ctypes.c_ulonglong),
        ("RefreshParameters", ctypes.c_ulonglong),
        ("CalculateMutedEffectiveVolume", ctypes.c_ulonglong),
        ("OnPathAdded", ctypes.c_ulonglong),
        ("UpdatePriorityWithDistance", ctypes.c_ulonglong),
    ]
    CAkParameterTarget_dtor_0: int
    UpdateTargetNodeParam: int
    NotifyParamChanged: int
    NotifyParamsChanged: bytes
    GetTargetedParamsSet: bytes
    RegisterToBusHierarchy: int
    UpdateTargetParam: int
    Init: int
    Term: int
    GetModulatorTriggerParams: int
    RefreshParameters: int
    CalculateMutedEffectiveVolume: int
    OnPathAdded: int
    UpdatePriorityWithDistance: int

class AkCaptureMgr(ctypes.Structure):
    def Instance(): pass

    def StartCapture(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("m_pWriterFactory", ctypes.c_ulonglong),
    ]
    __vftable: int
    m_pWriterFactory: int

class AkCaptureMgr_vtbl(ctypes.Structure):
    _fields_ = [
        ("StartCapture", ctypes.c_ulonglong),
    ]
    StartCapture: int

class IAkCaptureWriterFactory(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class IAkCaptureWriterFactory_vtbl(ctypes.Structure):
    _fields_ = [
        ("CreateWriter", ctypes.c_ulonglong),
    ]
    CreateWriter: int

class SpvReflectArrayTraits(ctypes.Structure):
    _fields_ = [
        ("dims_count", ctypes.c_uint32),
        ("dims", ctypes.c_uint32 * 0x20),  # unsigned int[32]
        ("spec_constant_op_ids", ctypes.c_uint32 * 0x20),  # unsigned int[32]
        ("stride", ctypes.c_uint32),
    ]
    dims_count: int
    dims: list[ctypes.c_uint32]
    spec_constant_op_ids: list[ctypes.c_uint32]
    stride: int

class SpvReflectBindingArrayTraits(ctypes.Structure):
    _fields_ = [
        ("dims_count", ctypes.c_uint32),
        ("dims", ctypes.c_uint32 * 0x20),  # unsigned int[32]
    ]
    dims_count: int
    dims: list[ctypes.c_uint32]

class SpvReflectDescriptorSet(ctypes.Structure):
    _fields_ = [
        ("set", ctypes.c_uint32),
        ("binding_count", ctypes.c_uint32),
        ("bindings", ctypes.c_ulonglong),
    ]
    set: int
    binding_count: int
    bindings: int

class CAkSinkDummy_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("IsDataNeeded", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("IsStarved", ctypes.c_ulonglong),
        ("ResetStarved", ctypes.c_ulonglong),
        ("GetSinkPluginType", ctypes.c_ulonglong),
        ("Get3DAudioCapabilities", ctypes.c_ulonglong),
        ("Consume", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    IsDataNeeded: int
    OnFrameEnd: int
    IsStarved: int
    ResetStarved: int
    GetSinkPluginType: int
    Get3DAudioCapabilities: int
    Consume: int

class CbxGetSrcOrder(ctypes.Structure):
    pass

class CAkMidiDeviceCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkMidiBaseCtx_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetTrackID", ctypes.c_ulonglong),
        ("GetAbsoluteStop", ctypes.c_ulonglong),
        ("HasMoreEvents", ctypes.c_ulonglong),
        ("ResolveMidiTarget", ctypes.c_ulonglong),
        ("GetMidiMgr", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
    ]
    CAkMidiBaseCtx_dtor_0: int
    Init: int
    GetTrackID: int
    GetAbsoluteStop: int
    HasMoreEvents: int
    ResolveMidiTarget: int
    GetMidiMgr: int
    Term: int

class AutoReserveMsg(ctypes.Structure):
    _fields_ = [
        ("m_pItem", ctypes.c_ulonglong),
    ]
    m_pItem: int

class CAkPlayListSequence_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkPlayList_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
        ("Length", ctypes.c_ulonglong),
        ("ID", ctypes.c_ulonglong),
        ("Add", ctypes.c_ulonglong),
        ("Remove", ctypes.c_ulonglong),
        ("RemoveAll", ctypes.c_ulonglong),
        ("Exists", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetWeight", ctypes.c_ulonglong),
        ("SetWeight", ctypes.c_ulonglong),
    ]
    CAkPlayList_dtor_0: int
    Init: int
    Destroy: int
    Length: int
    ID: int
    Add: int
    Remove: int
    RemoveAll: int
    Exists: int
    GetPosition: int
    GetWeight: int
    SetWeight: int

class CAkActorMixer(ctypes.Structure):
    def CanAddChild(): pass

    def Create(): pass

    def SetInitialValues(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x110),  # CAkActiveParent<CAkParameterNode>
    ]
    baseclass_0: bytes

class SafeContinuationList(ctypes.Structure):
    def SafeContinuationList(): pass

    _fields_ = [
        ("m_spBackupContinuationList", ctypes.c_char * 0x8),  # CAkSmartPtr<CAkContinuationList>
    ]
    m_spBackupContinuationList: bytes

class AkPlaylistItem(ctypes.Structure):
    _fields_ = [
        ("ulID", ctypes.c_uint32),
        ("weight", ctypes.c_uint32),
    ]
    ulID: int
    weight: int

class CAkPlayListRandom_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkPlayList_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
        ("Length", ctypes.c_ulonglong),
        ("ID", ctypes.c_ulonglong),
        ("Add", ctypes.c_ulonglong),
        ("Remove", ctypes.c_ulonglong),
        ("RemoveAll", ctypes.c_ulonglong),
        ("Exists", ctypes.c_ulonglong),
        ("GetPosition", ctypes.c_ulonglong),
        ("GetWeight", ctypes.c_ulonglong),
        ("SetWeight", ctypes.c_ulonglong),
        ("CalculateTotalWeight", ctypes.c_ulonglong),
    ]
    CAkPlayList_dtor_0: int
    Init: int
    Destroy: int
    Length: int
    ID: int
    Add: int
    Remove: int
    RemoveAll: int
    Exists: int
    GetPosition: int
    GetWeight: int
    SetWeight: int
    CalculateTotalWeight: int

class CAkSwitchPackage(ctypes.Structure):
    _fields_ = [
        ("m_list", ctypes.c_char * 0x18),  # AkArray<unsigned int,unsigned int,AkHybridAllocator<4,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<unsigned int> >
    ]
    m_list: bytes

class ScePadTouchPadInformation(ctypes.Structure):
    _fields_ = [
        ("pixelDensity", ctypes.c_float),
        ("resolution", ctypes.c_char * 0x4),  # ScePadTouchPadInformation::<unnamed_type_resolution>
    ]
    pixelDensity: float
    resolution: bytes

class ScePadStickInformation(ctypes.Structure):
    _fields_ = [
        ("deadZoneLeft", ctypes.c_uint8),
        ("deadZoneRight", ctypes.c_uint8),
    ]
    deadZoneLeft: int
    deadZoneRight: int

class ScePadVibrationParam(ctypes.Structure):
    _fields_ = [
        ("largeMotor", ctypes.c_uint8),
        ("smallMotor", ctypes.c_uint8),
    ]
    largeMotor: int
    smallMotor: int

class AkSwitchNodeParams(ctypes.Structure):
    _fields_ = [
        ("FadeOutTime", ctypes.c_int32),
        ("FadeInTime", ctypes.c_int32),
        ("_bf_8", ctypes.c_int8),
    ]
    FadeOutTime: int
    FadeInTime: int
    _bf_8: int

class _XINPUT_GAMEPAD(ctypes.Structure):
    _fields_ = [
        ("wButtons", ctypes.c_uint16),
        ("bLeftTrigger", ctypes.c_uint8),
        ("bRightTrigger", ctypes.c_uint8),
        ("sThumbLX", ctypes.c_int16),
        ("sThumbLY", ctypes.c_int16),
        ("sThumbRX", ctypes.c_int16),
        ("sThumbRY", ctypes.c_int16),
    ]
    wButtons: int
    bLeftTrigger: int
    bRightTrigger: int
    sThumbLX: int
    sThumbLY: int
    sThumbRX: int
    sThumbRY: int

class _XINPUT_STATE(ctypes.Structure):
    _fields_ = [
        ("dwPacketNumber", ctypes.c_uint32),
        ("Gamepad", ctypes.c_uint8),  # _BYTE[]
    ]
    dwPacketNumber: int
    Gamepad: list[ctypes.c_uint8]

class _XINPUT_VIBRATION(ctypes.Structure):
    _fields_ = [
        ("wLeftMotorSpeed", ctypes.c_uint16),
        ("wRightMotorSpeed", ctypes.c_uint16),
    ]
    wLeftMotorSpeed: int
    wRightMotorSpeed: int

class ScePadOpenParam(ctypes.Structure):
    _fields_ = [
        ("reserve", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    reserve: list[ctypes.c_uint8]

class cTkRichPresence(ctypes.Structure):
    def NotifyPlayerStat(): pass

    def vector_deleting_destructor(): pass

    def Configure(): pass

    def Construct(): pass

    def CreateInstance(): pass

    def Destruct(): pass

    def GetCurrentStateString(): pass

    def NotifyState(): pass

    def ResetTracking(): pass

    def Update(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mData", ctypes.c_ulonglong),
    ]
    __vftable: int
    mData: int

class cTkRichPresence_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRichPresence_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsSupported", ctypes.c_ulonglong),
        ("PublishRichPresence", ctypes.c_ulonglong),
        ("PublishPlayerStat", ctypes.c_ulonglong),
        ("PublishPlayerStat_48", ctypes.c_ulonglong),
    ]
    cTkRichPresence_dtor_0: int
    Construct: int
    Destruct: int
    Prepare: int
    Release: int
    Update: int
    IsSupported: int
    PublishRichPresence: int
    PublishPlayerStat: int
    PublishPlayerStat_48: int

class cTkRichPresenceState(ctypes.Structure):
    _fields_ = [
        ("muStateHandle", ctypes.c_uint32),
        ("msIdentifier", ctypes.c_char * 0x20),  # cTkFixedString<32,char>
        ("muIdentifier", ctypes.c_uint64),
        ("mbIsIdleTrigger", ctypes.c_ubyte),
        ("muPriority", ctypes.c_int32),
        ("mfThreshold", ctypes.c_float),
        ("mfDelay", ctypes.c_float),
        ("mfTimeout", ctypes.c_float),
        ("mbIsTriggered", ctypes.c_ubyte),
        ("mfLastTriggered", ctypes.c_float),
        ("mbIsPending", ctypes.c_ubyte),
        ("mfLastPending", ctypes.c_float),
        ("mbIsActive", ctypes.c_ubyte),
        ("mfLastActivated", ctypes.c_float),
    ]
    muStateHandle: int
    msIdentifier: bytes
    muIdentifier: int
    mbIsIdleTrigger: bool
    muPriority: int
    mfThreshold: float
    mfDelay: float
    mfTimeout: float
    mbIsTriggered: bool
    mfLastTriggered: float
    mbIsPending: bool
    mfLastPending: float
    mbIsActive: bool
    mfLastActivated: float

class AkVorbisHeaderBase(ctypes.Structure):
    _fields_ = [
        ("dwTotalPCMFrames", ctypes.c_uint32),
    ]
    dwTotalPCMFrames: int

class AkVorbisLoopInfo(ctypes.Structure):
    _fields_ = [
        ("dwLoopStartPacketOffset", ctypes.c_uint32),
        ("dwLoopEndPacketOffset", ctypes.c_uint32),
        ("uLoopBeginExtra", ctypes.c_uint16),
        ("uLoopEndExtra", ctypes.c_uint16),
    ]
    dwLoopStartPacketOffset: int
    dwLoopEndPacketOffset: int
    uLoopBeginExtra: int
    uLoopEndExtra: int

class SpvReflectPrvArrayTraits(ctypes.Structure):
    _fields_ = [
        ("element_type_id", ctypes.c_uint32),
        ("length_id", ctypes.c_uint32),
    ]
    element_type_id: int
    length_id: int

class SpvReflectPrvNumberDecoration(ctypes.Structure):
    _fields_ = [
        ("word_offset", ctypes.c_uint32),
        ("value", ctypes.c_uint32),
    ]
    word_offset: int
    value: int

class SpvReflectPrvStringDecoration(ctypes.Structure):
    _fields_ = [
        ("word_offset", ctypes.c_uint32),
        ("value", ctypes.c_char_p),
    ]
    word_offset: int
    value: bytes

class SpvReflectPrvString(ctypes.Structure):
    _fields_ = [
        ("result_id", ctypes.c_uint32),
        ("string", ctypes.c_char_p),
    ]
    result_id: int
    string: bytes

class SpvReflectPrvFunction(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint32),
        ("callee_count", ctypes.c_uint32),
        ("callees", ctypes.c_ulonglong),
        ("callee_ptrs", ctypes.c_ulonglong),
        ("accessed_ptr_count", ctypes.c_uint32),
        ("accessed_ptrs", ctypes.c_ulonglong),
    ]
    id: int
    callee_count: int
    callees: int
    callee_ptrs: int
    accessed_ptr_count: int
    accessed_ptrs: int

class SpvReflectPrvAccessChain(ctypes.Structure):
    _fields_ = [
        ("result_id", ctypes.c_uint32),
        ("result_type_id", ctypes.c_uint32),
        ("base_id", ctypes.c_uint32),
        ("index_count", ctypes.c_uint32),
        ("indexes", ctypes.c_ulonglong),
    ]
    result_id: int
    result_type_id: int
    base_id: int
    index_count: int
    indexes: int

class FlangerOutputPolicy(ctypes.Structure):
    pass

class CAkFlangerFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class rpmalloc_config_t(ctypes.Structure):
    _fields_ = [
        ("memory_map", ctypes.c_ulonglong),
        ("memory_unmap", ctypes.c_ulonglong),
        ("error_callback", ctypes.c_ulonglong),
        ("map_fail_callback", ctypes.c_ulonglong),
        ("page_size", ctypes.c_uint64),
        ("span_size", ctypes.c_uint64),
        ("span_map_count", ctypes.c_uint64),
        ("enable_huge_pages", ctypes.c_int32),
        ("overall_memory_limit", ctypes.c_uint64),
    ]
    memory_map: int
    memory_unmap: int
    error_callback: int
    map_fail_callback: int
    page_size: int
    span_size: int
    span_map_count: int
    enable_huge_pages: int
    overall_memory_limit: int

class size_class_t(ctypes.Structure):
    _fields_ = [
        ("block_size", ctypes.c_uint32),
        ("block_count", ctypes.c_uint16),
        ("class_idx", ctypes.c_uint16),
    ]
    block_size: int
    block_count: int
    class_idx: int

class span_t(ctypes.Structure):
    _fields_ = [
        ("free_list", ctypes.c_void_p),
        ("block_count", ctypes.c_uint32),
        ("size_class", ctypes.c_uint32),
        ("free_list_limit", ctypes.c_uint32),
        ("used_count", ctypes.c_uint32),
        ("free_list_deferred", ctypes.c_void_p),
        ("list_size", ctypes.c_uint32),
        ("block_size", ctypes.c_uint32),
        ("flags", ctypes.c_uint32),
        ("span_count", ctypes.c_uint32),
        ("total_spans", ctypes.c_uint32),
        ("offset_from_master", ctypes.c_uint32),
        ("remaining_spans", ctypes.c_int32),
        ("align_offset", ctypes.c_uint32),
        ("heap", ctypes.c_ulonglong),
        ("next", ctypes.c_ulonglong),
        ("prev", ctypes.c_ulonglong),
        ("extra", ctypes.c_uint64),
    ]
    free_list: int
    block_count: int
    size_class: int
    free_list_limit: int
    used_count: int
    free_list_deferred: int
    list_size: int
    block_size: int
    flags: int
    span_count: int
    total_spans: int
    offset_from_master: int
    remaining_spans: int
    align_offset: int
    heap: int
    next: int
    prev: int
    extra: int

class heap_size_class_t(ctypes.Structure):
    _fields_ = [
        ("free_list", ctypes.c_void_p),
        ("partial_span", ctypes.c_ulonglong),
        ("cache", ctypes.c_ulonglong),
    ]
    free_list: int
    partial_span: int
    cache: int

class span_cache_t(ctypes.Structure):
    _fields_ = [
        ("count", ctypes.c_uint64),
        ("span", ctypes.c_ulonglong * 0x190),  # span_t *[400]
    ]
    count: int
    span: list[ctypes.c_ulonglong]

class span_large_cache_t(ctypes.Structure):
    _fields_ = [
        ("count", ctypes.c_uint64),
        ("span", ctypes.c_ulonglong * 0x64),  # span_t *[100]
    ]
    count: int
    span: list[ctypes.c_ulonglong]

class rpmalloc_thread_statistics_t(ctypes.Structure):
    _fields_ = [
        ("sizecache", ctypes.c_uint64),
        ("spancache", ctypes.c_uint64),
        ("thread_to_global", ctypes.c_uint64),
        ("global_to_thread", ctypes.c_uint64),
        ("span_use", ctypes.c_char * 0x1200),  # $50AD027955674F1051EE6375B1493C31
        ("size_use", ctypes.c_char * 0x2000),  # $4A14DE24008B441AB42E0A726E29DE99
    ]
    sizecache: int
    spancache: int
    thread_to_global: int
    global_to_thread: int
    span_use: bytes
    size_use: bytes

class rpmalloc_global_statistics_t(ctypes.Structure):
    _fields_ = [
        ("mapped", ctypes.c_uint64),
        ("mapped_peak", ctypes.c_uint64),
        ("cached", ctypes.c_uint64),
        ("huge_alloc", ctypes.c_uint64),
        ("huge_alloc_peak", ctypes.c_uint64),
        ("mapped_total", ctypes.c_uint64),
        ("unmapped_total", ctypes.c_uint64),
    ]
    mapped: int
    mapped_peak: int
    cached: int
    huge_alloc: int
    huge_alloc_peak: int
    mapped_total: int
    unmapped_total: int

class LabelCuePoint(ctypes.Structure):
    _fields_ = [
        ("dwCuePointID", ctypes.c_uint32),
        ("strLabel", ctypes.c_char * 0x1),  # char[1]
    ]
    dwCuePointID: int
    strLabel: list[ctypes.c_char]

class SampleChunk(ctypes.Structure):
    _fields_ = [
        ("dwManufacturer", ctypes.c_uint32),
        ("dwProduct", ctypes.c_uint32),
        ("dwSamplePeriod", ctypes.c_uint32),
        ("dwMIDIUnityNote", ctypes.c_uint32),
        ("dwMIDIPitchFraction", ctypes.c_uint32),
        ("dwSMPTEFormat", ctypes.c_uint32),
        ("dwSMPTEOffset", ctypes.c_uint32),
        ("dwSampleLoops", ctypes.c_uint32),
        ("cbSamplerData", ctypes.c_uint32),
    ]
    dwManufacturer: int
    dwProduct: int
    dwSamplePeriod: int
    dwMIDIUnityNote: int
    dwMIDIPitchFraction: int
    dwSMPTEFormat: int
    dwSMPTEOffset: int
    dwSampleLoops: int
    cbSamplerData: int

class SampleLoop(ctypes.Structure):
    _fields_ = [
        ("dwIdentifier", ctypes.c_uint32),
        ("dwType", ctypes.c_uint32),
        ("dwStart", ctypes.c_uint32),
        ("dwEnd", ctypes.c_uint32),
        ("dwFraction", ctypes.c_uint32),
        ("dwPlayCount", ctypes.c_uint32),
    ]
    dwIdentifier: int
    dwType: int
    dwStart: int
    dwEnd: int
    dwFraction: int
    dwPlayCount: int

class CuePoint(ctypes.Structure):
    _fields_ = [
        ("dwIdentifier", ctypes.c_uint32),
        ("dwPosition", ctypes.c_uint32),
        ("fccChunk", ctypes.c_uint32),
        ("dwChunkStart", ctypes.c_uint32),
        ("dwBlockStart", ctypes.c_uint32),
        ("dwSampleOffset", ctypes.c_uint32),
    ]
    dwIdentifier: int
    dwPosition: int
    fccChunk: int
    dwChunkStart: int
    dwBlockStart: int
    dwSampleOffset: int

class _UNICODE_STRING(ctypes.Structure):
    _fields_ = [
        ("Length", ctypes.c_uint16),
        ("MaximumLength", ctypes.c_uint16),
        ("Buffer", ctypes.c_ulonglong),
    ]
    Length: int
    MaximumLength: int
    Buffer: int

class _PEB(ctypes.Structure):
    _fields_ = [
        ("Reserved1", ctypes.c_uint8 * 0x2),  # unsigned __int8[2]
        ("BeingDebugged", ctypes.c_uint8),
        ("Reserved2", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
        ("Reserved3", ctypes.c_void_p * 0x2),  # void *[2]
        ("Ldr", ctypes.c_ulonglong),
        ("ProcessParameters", ctypes.c_ulonglong),
        ("Reserved4", ctypes.c_void_p * 0x3),  # void *[3]
        ("AtlThunkSListPtr", ctypes.c_void_p),
        ("Reserved5", ctypes.c_void_p),
        ("Reserved6", ctypes.c_uint32),
        ("Reserved7", ctypes.c_void_p),
        ("Reserved8", ctypes.c_uint32),
        ("AtlThunkSListPtr32", ctypes.c_uint32),
        ("Reserved9", ctypes.c_void_p * 0x2D),  # void *[45]
        ("Reserved10", ctypes.c_uint8 * 0x60),  # unsigned __int8[96]
        ("PostProcessInitRoutine", ctypes.c_ulonglong),
        ("Reserved11", ctypes.c_uint8 * 0x80),  # unsigned __int8[128]
        ("Reserved12", ctypes.c_void_p * 0x1),  # void *[1]
        ("SessionId", ctypes.c_uint32),
    ]
    Reserved1: list[ctypes.c_uint8]
    BeingDebugged: int
    Reserved2: list[ctypes.c_uint8]
    Reserved3: list[ctypes.c_void_p]
    Ldr: int
    ProcessParameters: int
    Reserved4: list[ctypes.c_void_p]
    AtlThunkSListPtr: int
    Reserved5: int
    Reserved6: int
    Reserved7: int
    Reserved8: int
    AtlThunkSListPtr32: int
    Reserved9: list[ctypes.c_void_p]
    Reserved10: list[ctypes.c_uint8]
    PostProcessInitRoutine: int
    Reserved11: list[ctypes.c_uint8]
    Reserved12: list[ctypes.c_void_p]
    SessionId: int

class _TEB(ctypes.Structure):
    _fields_ = [
        ("Reserved1", ctypes.c_void_p * 0xC),  # void *[12]
        ("ProcessEnvironmentBlock", ctypes.c_ulonglong),
        ("Reserved2", ctypes.c_void_p * 0x18F),  # void *[399]
        ("Reserved3", ctypes.c_uint8 * 0x7A0),  # unsigned __int8[1952]
        ("TlsSlots", ctypes.c_void_p * 0x40),  # void *[64]
        ("Reserved4", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
        ("Reserved5", ctypes.c_void_p * 0x1A),  # void *[26]
        ("ReservedForOle", ctypes.c_void_p),
        ("Reserved6", ctypes.c_void_p * 0x4),  # void *[4]
        ("TlsExpansionSlots", ctypes.c_void_p),
    ]
    Reserved1: list[ctypes.c_void_p]
    ProcessEnvironmentBlock: int
    Reserved2: list[ctypes.c_void_p]
    Reserved3: list[ctypes.c_uint8]
    TlsSlots: list[ctypes.c_void_p]
    Reserved4: list[ctypes.c_uint8]
    Reserved5: list[ctypes.c_void_p]
    ReservedForOle: int
    Reserved6: list[ctypes.c_void_p]
    TlsExpansionSlots: int

class CAkWindowsContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlatformContext_dtor_0", ctypes.c_ulonglong),
        ("GetDeviceEnum", ctypes.c_ulonglong),
    ]
    IAkPlatformContext_dtor_0: int
    GetDeviceEnum: int

class CAkActionExcept_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int

class CAkActionActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("SetActionActiveParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    SetActionActiveParams: int

class DeviceEnumHelper(ctypes.Structure):
    def DeviceEnumHelper(): pass

    _fields_ = [
        ("pEnumerator", ctypes.c_ulonglong),
    ]
    pEnumerator: int

class CAkActionStop_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("SetActionActiveParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    SetActionActiveParams: int

class FileCaptureWriterFactory_vtbl(ctypes.Structure):
    _fields_ = [
        ("CreateWriter", ctypes.c_ulonglong),
    ]
    CreateWriter: int

class LabelChunk(ctypes.Structure):
    _fields_ = [
        ("fccChunk", ctypes.c_uint32),
        ("dwChunkSize", ctypes.c_uint32),
        ("dwIdentifier", ctypes.c_uint32),
        ("strVariableSizeText", ctypes.c_char_p),
    ]
    fccChunk: int
    dwChunkSize: int
    dwIdentifier: int
    strVariableSizeText: bytes

class AkCachePinnedFileSetKey(ctypes.Structure):
    _fields_ = [
        ("m_pGameObj", ctypes.c_ulonglong),
        ("m_pEvent", ctypes.c_ulonglong),
    ]
    m_pGameObj: int
    m_pEvent: int

class AkCachePinnedFileSet(ctypes.Structure):
    def GatherAndPinFilesInStreamCache(): pass

    def UnpinFilesInStreamCache(): pass

    def UnsetAllGameSyncs(): pass

    _fields_ = [
        ("m_SwitchGroups", ctypes.c_char * 0x10),  # AkSet<unsigned int,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional>
        ("m_ActiveFiles", ctypes.c_char * 0x10),  # AkSet<unsigned int,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional>
        ("m_InactiveFiles", ctypes.c_char * 0x10),  # AkSet<unsigned int,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional>
        ("m_uActivePriority", ctypes.c_char),
        ("m_uInactivePriority", ctypes.c_char),
    ]
    m_SwitchGroups: bytes
    m_ActiveFiles: bytes
    m_InactiveFiles: bytes
    m_uActivePriority: str
    m_uInactivePriority: str

class AkCacheGameSyncMon_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkSwitchAware_dtor_0", ctypes.c_ulonglong),
        ("SetSwitch", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
    ]
    CAkSwitchAware_dtor_0: int
    SetSwitch: int
    AddRef: int
    Release: int

class Ak4DVector(ctypes.Structure):
    _fields_ = [
        ("v", ctypes.c_float * 0x4),  # float[4]
    ]
    v: list[ctypes.c_float]

class Ak3DIntVector(ctypes.Structure):
    _fields_ = [
        ("X", ctypes.c_int32),
        ("Y", ctypes.c_int32),
        ("Z", ctypes.c_int32),
    ]
    X: int
    Y: int
    Z: int

class Ak3DVector(ctypes.Structure):
    _fields_ = [
        ("X", ctypes.c_float),
        ("Y", ctypes.c_float),
        ("Z", ctypes.c_float),
    ]
    X: float
    Y: float
    Z: float

class Ak2DVector(ctypes.Structure):
    _fields_ = [
        ("X", ctypes.c_float),
        ("Y", ctypes.c_float),
    ]
    X: float
    Y: float

class AkMatrix4x4(ctypes.Structure):
    _fields_ = [
        ("m_Data", ctypes.c_float * 0x10),  # float[16]
    ]
    m_Data: list[ctypes.c_float]

class AkMatrix3x3(ctypes.Structure):
    _fields_ = [
        ("m_Data", ctypes.c_float * 0x3),  # float[3][3]
    ]
    m_Data: list[ctypes.c_float]

class CAkActionDuck_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int

class AkQuaternion(ctypes.Structure):
    _fields_ = [
        ("W", ctypes.c_float),
        ("X", ctypes.c_float),
        ("Y", ctypes.c_float),
        ("Z", ctypes.c_float),
    ]
    W: float
    X: float
    Y: float
    Z: float

class AkVBAPTripletStackArray(ctypes.Structure):
    _fields_ = [
        ("m_Array", ctypes.c_ulonglong),
        ("m_uSize", ctypes.c_uint32),
        ("m_uCount", ctypes.c_uint32),
    ]
    m_Array: int
    m_uSize: int
    m_uCount: int

class AkVBAPMap(ctypes.Structure):
    def AddPointsInArea(): pass

    def Allocate(): pass

    def ComputeVBAP2D(): pass

    def ComputeVBAP3D(): pass

    def ComputeVBAP(): pass

    def ComputeVBAPSquared(): pass

    def FindPointInTriangle(): pass

    def FindTriangle(): pass

    def NormalizePreSquaredGain(): pass

    def PointsToVBAPMap(): pass

    def RunTriangulation(): pass

    def Term(): pass

    _fields_ = [
        ("m_VerticesCount", ctypes.c_uint32),
        ("m_VBAPSlicesHullA", ctypes.c_char * 0x10),  # AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> >
        ("m_VBAPSlicesHullB", ctypes.c_char * 0x10),  # AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> >
        ("m_VBAPSlicesHullABelow", ctypes.c_char * 0x10),  # AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> >
        ("m_VBAPSlicesHullBBelow", ctypes.c_char * 0x10),  # AkArray<AkVBAPTriplet,AkVBAPTriplet const &,AkArrayAllocatorNoAlign<5>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkVBAPTriplet> >
        ("m_Vertice3D", ctypes.c_ulonglong),
        ("m_Vertice2DHullA", ctypes.c_ulonglong),
        ("m_Vertice2DHullB", ctypes.c_ulonglong),
    ]
    m_VerticesCount: int
    m_VBAPSlicesHullA: bytes
    m_VBAPSlicesHullB: bytes
    m_VBAPSlicesHullABelow: bytes
    m_VBAPSlicesHullBBelow: bytes
    m_Vertice3D: int
    m_Vertice2DHullA: int
    m_Vertice2DHullB: int

class AkVBAP(ctypes.Structure):
    pass

class NoLogAttenuationDelta(ctypes.Structure):
    pass

class AkSpeakerVolumesN(ctypes.Structure):
    _fields_ = [
        ("fVolume", ctypes.c_float),
        ("fVolumeDelta", ctypes.c_float),
    ]
    fVolume: float
    fVolumeDelta: float

class AkHashIndexEntry(ctypes.Structure):
    _fields_ = [
        ("uMaskedKey", ctypes.c_uint32),
        ("iIndex", ctypes.c_int32),
    ]
    uMaskedKey: int
    iIndex: int

class CAkVPLFilterNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("ReleaseMemory", ctypes.c_ulonglong),
        ("ReleaseInputBuffer", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ConsumeBuffer", ctypes.c_ulonglong),
        ("GetPlugin", ctypes.c_ulonglong),
        ("GetOutputConfig", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Init: int
    Term: int
    ReleaseMemory: int
    ReleaseInputBuffer: int
    GetBuffer: int
    ConsumeBuffer: int
    GetPlugin: int
    GetOutputConfig: int

class CAkVPLFilterNodeObjectProc_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("ReleaseMemory", ctypes.c_ulonglong),
        ("ReleaseInputBuffer", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ConsumeBuffer", ctypes.c_ulonglong),
        ("GetPlugin", ctypes.c_ulonglong),
        ("GetOutputConfig", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Init: int
    Term: int
    ReleaseMemory: int
    ReleaseInputBuffer: int
    GetBuffer: int
    ConsumeBuffer: int
    GetPlugin: int
    GetOutputConfig: int

class CAkActionSetState_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int

class CAkActionBreak_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int

class CAkActionSeek_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int

class CAkActionUseState_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int

class AkAtomicListNode(ctypes.Structure):
    _fields_ = [
        ("pNext", ctypes.c_int64),
        ("pData", ctypes.c_void_p),
    ]
    pNext: int
    pData: int

class CAkActionResetPlaylist_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("SetActionActiveParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    SetActionActiveParams: int

class BGMSinkParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class tWAVEFORMATEX(ctypes.Structure):
    _fields_ = [
        ("wFormatTag", ctypes.c_uint16),
        ("nChannels", ctypes.c_uint16),
        ("nSamplesPerSec", ctypes.c_uint32),
        ("nAvgBytesPerSec", ctypes.c_uint32),
        ("nBlockAlign", ctypes.c_uint16),
        ("wBitsPerSample", ctypes.c_uint16),
        ("cbSize", ctypes.c_uint16),
    ]
    wFormatTag: int
    nChannels: int
    nSamplesPerSec: int
    nAvgBytesPerSec: int
    nBlockAlign: int
    wBitsPerSample: int
    cbSize: int

class HSTRING__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class IInspectable_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetIids", ctypes.c_ulonglong),
        ("GetRuntimeClassName", ctypes.c_ulonglong),
        ("GetTrustLevel", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetIids: int
    GetRuntimeClassName: int
    GetTrustLevel: int

class IActivationFactory_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetIids", ctypes.c_ulonglong),
        ("GetRuntimeClassName", ctypes.c_ulonglong),
        ("GetTrustLevel", ctypes.c_ulonglong),
        ("ActivateInstance", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetIids: int
    GetRuntimeClassName: int
    GetTrustLevel: int
    ActivateInstance: int

class IWeakReference_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Resolve", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Resolve: int

class IClassFactory_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("CreateInstance", ctypes.c_ulonglong),
        ("LockServer", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    CreateInstance: int
    LockServer: int

class CAkSrcBaseEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Seek_60", ctypes.c_ulonglong),
        ("StartStream", ctypes.c_ulonglong),
        ("StopStream", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("IsUsingThisSlot", ctypes.c_ulonglong),
        ("IsUsingThisSlot_98", ctypes.c_ulonglong),
        ("NotifySourceStarvation", ctypes.c_ulonglong),
        ("SupportResampling", ctypes.c_ulonglong),
        ("PrepareSourceSwitch", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("MustRelocatePitchInputBufferOnMediaRelocation", ctypes.c_ulonglong),
        ("MustRelocateAnalysisDataOnMediaRelocation", ctypes.c_ulonglong),
        ("ChangeSourcePosition", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("SrcProcessOrder", ctypes.c_ulonglong),
        ("RestartSourcePlugin", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
        ("SetInfiniteLooping", ctypes.c_ulonglong),
        ("GetSourceSampleRate", ctypes.c_ulonglong),
        ("OnLoopComplete", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Term: int
    Seek_60: int
    StartStream: int
    StopStream: int
    GetBuffer: int
    GetDuration: int
    StopLooping: int
    IsUsingThisSlot: int
    IsUsingThisSlot_98: int
    NotifySourceStarvation: int
    SupportResampling: int
    PrepareSourceSwitch: int
    SupportMediaRelocation: int
    RelocateMedia: int
    MustRelocatePitchInputBufferOnMediaRelocation: int
    MustRelocateAnalysisDataOnMediaRelocation: int
    ChangeSourcePosition: int
    GetAnalyzedEnvelope: int
    SrcProcessOrder: int
    RestartSourcePlugin: int
    PrepareNextBuffer: int
    SetInfiniteLooping: int
    GetSourceSampleRate: int
    OnLoopComplete: int

class AkCachedBufferList(ctypes.Structure):
    _fields_ = [
        ("pNext", ctypes.c_void_p),
        ("bInUse", ctypes.c_int32),
    ]
    pNext: int
    bInUse: int

class IMarshal_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetUnmarshalClass", ctypes.c_ulonglong),
        ("GetMarshalSizeMax", ctypes.c_ulonglong),
        ("MarshalInterface", ctypes.c_ulonglong),
        ("UnmarshalInterface", ctypes.c_ulonglong),
        ("ReleaseMarshalData", ctypes.c_ulonglong),
        ("DisconnectObject", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetUnmarshalClass: int
    GetMarshalSizeMax: int
    MarshalInterface: int
    UnmarshalInterface: int
    ReleaseMarshalData: int
    DisconnectObject: int

class CAkSrcFileBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Seek_60", ctypes.c_ulonglong),
        ("StartStream", ctypes.c_ulonglong),
        ("StopStream", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("IsUsingThisSlot", ctypes.c_ulonglong),
        ("IsUsingThisSlot_98", ctypes.c_ulonglong),
        ("NotifySourceStarvation", ctypes.c_ulonglong),
        ("SupportResampling", ctypes.c_ulonglong),
        ("PrepareSourceSwitch", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("MustRelocatePitchInputBufferOnMediaRelocation", ctypes.c_ulonglong),
        ("MustRelocateAnalysisDataOnMediaRelocation", ctypes.c_ulonglong),
        ("ChangeSourcePosition", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("SrcProcessOrder", ctypes.c_ulonglong),
        ("RestartSourcePlugin", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
        ("SetInfiniteLooping", ctypes.c_ulonglong),
        ("GetSourceSampleRate", ctypes.c_ulonglong),
        ("OnLoopComplete", ctypes.c_ulonglong),
        ("ParseHeader", ctypes.c_ulonglong),
        ("FindClosestFileOffset", ctypes.c_ulonglong),
        ("ProcessStreamBuffer", ctypes.c_ulonglong),
        ("SeekStream", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Term: int
    Seek_60: int
    StartStream: int
    StopStream: int
    GetBuffer: int
    GetDuration: int
    StopLooping: int
    IsUsingThisSlot: int
    IsUsingThisSlot_98: int
    NotifySourceStarvation: int
    SupportResampling: int
    PrepareSourceSwitch: int
    SupportMediaRelocation: int
    RelocateMedia: int
    MustRelocatePitchInputBufferOnMediaRelocation: int
    MustRelocateAnalysisDataOnMediaRelocation: int
    ChangeSourcePosition: int
    GetAnalyzedEnvelope: int
    SrcProcessOrder: int
    RestartSourcePlugin: int
    PrepareNextBuffer: int
    SetInfiniteLooping: int
    GetSourceSampleRate: int
    OnLoopComplete: int
    ParseHeader: int
    FindClosestFileOffset: int
    ProcessStreamBuffer: int
    SeekStream: int

class IGlobalInterfaceTable_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("RegisterInterfaceInGlobal", ctypes.c_ulonglong),
        ("RevokeInterfaceFromGlobal", ctypes.c_ulonglong),
        ("GetInterfaceFromGlobal", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    RegisterInterfaceInGlobal: int
    RevokeInterfaceFromGlobal: int
    GetInterfaceFromGlobal: int

class CAkSrcFileADPCM_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Seek_60", ctypes.c_ulonglong),
        ("StartStream", ctypes.c_ulonglong),
        ("StopStream", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("IsUsingThisSlot", ctypes.c_ulonglong),
        ("IsUsingThisSlot_98", ctypes.c_ulonglong),
        ("NotifySourceStarvation", ctypes.c_ulonglong),
        ("SupportResampling", ctypes.c_ulonglong),
        ("PrepareSourceSwitch", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("MustRelocatePitchInputBufferOnMediaRelocation", ctypes.c_ulonglong),
        ("MustRelocateAnalysisDataOnMediaRelocation", ctypes.c_ulonglong),
        ("ChangeSourcePosition", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("SrcProcessOrder", ctypes.c_ulonglong),
        ("RestartSourcePlugin", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
        ("SetInfiniteLooping", ctypes.c_ulonglong),
        ("GetSourceSampleRate", ctypes.c_ulonglong),
        ("OnLoopComplete", ctypes.c_ulonglong),
        ("ParseHeader", ctypes.c_ulonglong),
        ("FindClosestFileOffset", ctypes.c_ulonglong),
        ("ProcessStreamBuffer", ctypes.c_ulonglong),
        ("SeekStream", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Term: int
    Seek_60: int
    StartStream: int
    StopStream: int
    GetBuffer: int
    GetDuration: int
    StopLooping: int
    IsUsingThisSlot: int
    IsUsingThisSlot_98: int
    NotifySourceStarvation: int
    SupportResampling: int
    PrepareSourceSwitch: int
    SupportMediaRelocation: int
    RelocateMedia: int
    MustRelocatePitchInputBufferOnMediaRelocation: int
    MustRelocateAnalysisDataOnMediaRelocation: int
    ChangeSourcePosition: int
    GetAnalyzedEnvelope: int
    SrcProcessOrder: int
    RestartSourcePlugin: int
    PrepareNextBuffer: int
    SetInfiniteLooping: int
    GetSourceSampleRate: int
    OnLoopComplete: int
    ParseHeader: int
    FindClosestFileOffset: int
    ProcessStreamBuffer: int
    SeekStream: int

class CAkActionSetValue_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("ExecSetValue", ctypes.c_ulonglong),
        ("ExecSetValue_70", ctypes.c_ulonglong),
        ("ExecResetValue", ctypes.c_ulonglong),
        ("ExecResetValue_80", ctypes.c_ulonglong),
        ("ExecResetValueExcept", ctypes.c_ulonglong),
        ("ExecResetValueExcept_90", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    ExecSetValue: int
    ExecSetValue_70: int
    ExecResetValue: int
    ExecResetValue_80: int
    ExecResetValueExcept: int
    ExecResetValueExcept_90: int

class CAkActionMute_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("ExecSetValue", ctypes.c_ulonglong),
        ("ExecSetValue_70", ctypes.c_ulonglong),
        ("ExecResetValue", ctypes.c_ulonglong),
        ("ExecResetValue_80", ctypes.c_ulonglong),
        ("ExecResetValueExcept", ctypes.c_ulonglong),
        ("ExecResetValueExcept_90", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    ExecSetValue: int
    ExecSetValue_70: int
    ExecResetValue: int
    ExecResetValue_80: int
    ExecResetValueExcept: int
    ExecResetValueExcept_90: int

class FileCaptureWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("StartCapture", ctypes.c_ulonglong),
        ("StopCapture", ctypes.c_ulonglong),
        ("PassAudioData", ctypes.c_ulonglong),
        ("WriteGenericData", ctypes.c_ulonglong),
        ("Destroy", ctypes.c_ulonglong),
    ]
    StartCapture: int
    StopCapture: int
    PassAudioData: int
    WriteGenericData: int
    Destroy: int

class CAkActionBypassFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int

class CAkActionResume_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("SetActionActiveParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    SetActionActiveParams: int

class CAkActionSetGameParameter_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("ExecSetValue", ctypes.c_ulonglong),
        ("ExecSetValue_70", ctypes.c_ulonglong),
        ("ExecResetValue", ctypes.c_ulonglong),
        ("ExecResetValue_80", ctypes.c_ulonglong),
        ("ExecResetValueExcept", ctypes.c_ulonglong),
        ("ExecResetValueExcept_90", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    ExecSetValue: int
    ExecSetValue_70: int
    ExecResetValue: int
    ExecResetValue_80: int
    ExecResetValueExcept: int
    ExecResetValueExcept_90: int

class CAkActionPlayEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int

class CAkADPCMCodec(ctypes.Structure):
    def Decode(): pass


class AkStereoDelayChannelParams(ctypes.Structure):
    _fields_ = [
        ("fDelayTime", ctypes.c_float),
        ("fFeedback", ctypes.c_float),
        ("fCrossFeed", ctypes.c_float),
    ]
    fDelayTime: float
    fFeedback: float
    fCrossFeed: float

def Bypass_I16_NChanAVX2(): pass

def Fixed_I16_1ChanAVX2(): pass

def Fixed_I16_2ChanAVX2(): pass

def Fixed_I16_NChanAVX2(): pass

class HSTRING_HEADER(ctypes.Structure):
    _fields_ = [
        ("Reserved", ctypes.c_char * 0x18),  # HSTRING_HEADER::<unnamed_type_Reserved>
    ]
    Reserved: bytes

class CAkActionPause_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("SetActionActiveParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    SetActionActiveParams: int

def Fixed_Native_1ChanAVX2(): pass

class CAkActionSetAkProp_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("ExecSetValue", ctypes.c_ulonglong),
        ("ExecSetValue_70", ctypes.c_ulonglong),
        ("ExecResetValue", ctypes.c_ulonglong),
        ("ExecResetValue_80", ctypes.c_ulonglong),
        ("ExecResetValueExcept", ctypes.c_ulonglong),
        ("ExecResetValueExcept_90", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    ExecSetValue: int
    ExecSetValue_70: int
    ExecResetValue: int
    ExecResetValue_80: int
    ExecResetValueExcept: int
    ExecResetValueExcept_90: int

def Fixed_Native_2ChanAVX2(): pass

class CAkActionSetSwitch_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int

class CAkActionTrigger_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int

class CAkActionRelease_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkIndexable_dtor_0", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Category", ctypes.c_ulonglong),
        ("SetElementID", ctypes.c_ulonglong),
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_30", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("SetActionParams", ctypes.c_ulonglong),
        ("SetActionSpecificParams", ctypes.c_ulonglong),
        ("AddException", ctypes.c_ulonglong),
        ("RemoveException", ctypes.c_ulonglong),
        ("ClearExceptions", ctypes.c_ulonglong),
        ("SetActionActiveParams", ctypes.c_ulonglong),
    ]
    CAkIndexable_dtor_0: int
    AddRef: int
    Release: int
    Category: int
    SetElementID: int
    SetAkProp: int
    SetAkProp_30: int
    Execute: int
    SetActionParams: int
    SetActionSpecificParams: int
    AddException: int
    RemoveException: int
    ClearExceptions: int
    SetActionActiveParams: int

class IAudioClient_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("GetBufferSize", ctypes.c_ulonglong),
        ("GetStreamLatency", ctypes.c_ulonglong),
        ("GetCurrentPadding", ctypes.c_ulonglong),
        ("IsFormatSupported", ctypes.c_ulonglong),
        ("GetMixFormat", ctypes.c_ulonglong),
        ("GetDevicePeriod", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Stop", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("SetEventHandle", ctypes.c_ulonglong),
        ("GetService", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Initialize: int
    GetBufferSize: int
    GetStreamLatency: int
    GetCurrentPadding: int
    IsFormatSupported: int
    GetMixFormat: int
    GetDevicePeriod: int
    Start: int
    Stop: int
    Reset: int
    SetEventHandle: int
    GetService: int

class IAudioRenderClient_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetBuffer: int
    ReleaseBuffer: int

class IAudioFormatEnumerator_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetCount", ctypes.c_ulonglong),
        ("GetFormat", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetCount: int
    GetFormat: int

class ISpatialAudioClient_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetStaticObjectPosition", ctypes.c_ulonglong),
        ("GetNativeStaticObjectTypeMask", ctypes.c_ulonglong),
        ("GetMaxDynamicObjectCount", ctypes.c_ulonglong),
        ("GetSupportedAudioObjectFormatEnumerator", ctypes.c_ulonglong),
        ("GetMaxFrameCount", ctypes.c_ulonglong),
        ("IsAudioObjectFormatSupported", ctypes.c_ulonglong),
        ("IsSpatialAudioStreamAvailable", ctypes.c_ulonglong),
        ("ActivateSpatialAudioStream", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetStaticObjectPosition: int
    GetNativeStaticObjectTypeMask: int
    GetMaxDynamicObjectCount: int
    GetSupportedAudioObjectFormatEnumerator: int
    GetMaxFrameCount: int
    IsAudioObjectFormatSupported: int
    IsSpatialAudioStreamAvailable: int
    ActivateSpatialAudioStream: int

class ISpatialAudioObjectRenderStreamBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetAvailableDynamicObjectCount", ctypes.c_ulonglong),
        ("GetService", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Stop", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("BeginUpdatingAudioObjects", ctypes.c_ulonglong),
        ("EndUpdatingAudioObjects", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetAvailableDynamicObjectCount: int
    GetService: int
    Start: int
    Stop: int
    Reset: int
    BeginUpdatingAudioObjects: int
    EndUpdatingAudioObjects: int

class ISpatialAudioObjectBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("SetEndOfStream", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("GetAudioObjectType", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetBuffer: int
    SetEndOfStream: int
    IsActive: int
    GetAudioObjectType: int

class ISpatialAudioObject_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("SetEndOfStream", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("GetAudioObjectType", ctypes.c_ulonglong),
        ("SetPosition", ctypes.c_ulonglong),
        ("SetVolume", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetBuffer: int
    SetEndOfStream: int
    IsActive: int
    GetAudioObjectType: int
    SetPosition: int
    SetVolume: int

class ISpatialAudioObjectRenderStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetAvailableDynamicObjectCount", ctypes.c_ulonglong),
        ("GetService", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Stop", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("BeginUpdatingAudioObjects", ctypes.c_ulonglong),
        ("EndUpdatingAudioObjects", ctypes.c_ulonglong),
        ("ActivateSpatialAudioObject", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetAvailableDynamicObjectCount: int
    GetService: int
    Start: int
    Stop: int
    Reset: int
    BeginUpdatingAudioObjects: int
    EndUpdatingAudioObjects: int
    ActivateSpatialAudioObject: int

class CAkSinkCoreAudio_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("IsDataNeeded", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("IsStarved", ctypes.c_ulonglong),
        ("ResetStarved", ctypes.c_ulonglong),
        ("GetSinkPluginType", ctypes.c_ulonglong),
        ("Get3DAudioCapabilities", ctypes.c_ulonglong),
        ("Consume", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    IsDataNeeded: int
    OnFrameEnd: int
    IsStarved: int
    ResetStarved: int
    GetSinkPluginType: int
    Get3DAudioCapabilities: int
    Consume: int

class CAkStereoDelayFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class submap(ctypes.Structure):
    _fields_ = [
        ("floor", ctypes.c_char),
        ("residue", ctypes.c_char),
    ]
    floor: str
    residue: str

class IAudioSessionEvents_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("OnDisplayNameChanged", ctypes.c_ulonglong),
        ("OnIconPathChanged", ctypes.c_ulonglong),
        ("OnSimpleVolumeChanged", ctypes.c_ulonglong),
        ("OnChannelVolumeChanged", ctypes.c_ulonglong),
        ("OnGroupingParamChanged", ctypes.c_ulonglong),
        ("OnStateChanged", ctypes.c_ulonglong),
        ("OnSessionDisconnected", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    OnDisplayNameChanged: int
    OnIconPathChanged: int
    OnSimpleVolumeChanged: int
    OnChannelVolumeChanged: int
    OnGroupingParamChanged: int
    OnStateChanged: int
    OnSessionDisconnected: int

class IAudioSessionControl_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetState", ctypes.c_ulonglong),
        ("GetDisplayName", ctypes.c_ulonglong),
        ("SetDisplayName", ctypes.c_ulonglong),
        ("GetIconPath", ctypes.c_ulonglong),
        ("SetIconPath", ctypes.c_ulonglong),
        ("GetGroupingParam", ctypes.c_ulonglong),
        ("SetGroupingParam", ctypes.c_ulonglong),
        ("RegisterAudioSessionNotification", ctypes.c_ulonglong),
        ("UnregisterAudioSessionNotification", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetState: int
    GetDisplayName: int
    SetDisplayName: int
    GetIconPath: int
    SetIconPath: int
    GetGroupingParam: int
    SetGroupingParam: int
    RegisterAudioSessionNotification: int
    UnregisterAudioSessionNotification: int

class ISimpleAudioVolume_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("SetMasterVolume", ctypes.c_ulonglong),
        ("GetMasterVolume", ctypes.c_ulonglong),
        ("SetMute", ctypes.c_ulonglong),
        ("GetMute", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    SetMasterVolume: int
    GetMasterVolume: int
    SetMute: int
    GetMute: int

class IAudioSessionManager_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetAudioSessionControl", ctypes.c_ulonglong),
        ("GetSimpleAudioVolume", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetAudioSessionControl: int
    GetSimpleAudioVolume: int

class IAudioSessionEnumerator_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetCount", ctypes.c_ulonglong),
        ("GetSession", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetCount: int
    GetSession: int

class IAudioSessionNotification_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("OnSessionCreated", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    OnSessionCreated: int

class IAudioVolumeDuckNotification_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("OnVolumeDuckNotification", ctypes.c_ulonglong),
        ("OnVolumeUnduckNotification", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    OnVolumeDuckNotification: int
    OnVolumeUnduckNotification: int

class IAudioSessionManager2_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetAudioSessionControl", ctypes.c_ulonglong),
        ("GetSimpleAudioVolume", ctypes.c_ulonglong),
        ("GetSessionEnumerator", ctypes.c_ulonglong),
        ("RegisterSessionNotification", ctypes.c_ulonglong),
        ("UnregisterSessionNotification", ctypes.c_ulonglong),
        ("RegisterDuckNotification", ctypes.c_ulonglong),
        ("UnregisterDuckNotification", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetAudioSessionControl: int
    GetSimpleAudioVolume: int
    GetSessionEnumerator: int
    RegisterSessionNotification: int
    UnregisterSessionNotification: int
    RegisterDuckNotification: int
    UnregisterDuckNotification: int

class IAudioSessionControl2_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetState", ctypes.c_ulonglong),
        ("GetDisplayName", ctypes.c_ulonglong),
        ("SetDisplayName", ctypes.c_ulonglong),
        ("GetIconPath", ctypes.c_ulonglong),
        ("SetIconPath", ctypes.c_ulonglong),
        ("GetGroupingParam", ctypes.c_ulonglong),
        ("SetGroupingParam", ctypes.c_ulonglong),
        ("RegisterAudioSessionNotification", ctypes.c_ulonglong),
        ("UnregisterAudioSessionNotification", ctypes.c_ulonglong),
        ("GetSessionIdentifier", ctypes.c_ulonglong),
        ("GetSessionInstanceIdentifier", ctypes.c_ulonglong),
        ("GetProcessId", ctypes.c_ulonglong),
        ("IsSystemSoundsSession", ctypes.c_ulonglong),
        ("SetDuckingPreference", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetState: int
    GetDisplayName: int
    SetDisplayName: int
    GetIconPath: int
    SetIconPath: int
    GetGroupingParam: int
    SetGroupingParam: int
    RegisterAudioSessionNotification: int
    UnregisterAudioSessionNotification: int
    GetSessionIdentifier: int
    GetSessionInstanceIdentifier: int
    GetProcessId: int
    IsSystemSoundsSession: int
    SetDuckingPreference: int

class IAudioClient2_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("GetBufferSize", ctypes.c_ulonglong),
        ("GetStreamLatency", ctypes.c_ulonglong),
        ("GetCurrentPadding", ctypes.c_ulonglong),
        ("IsFormatSupported", ctypes.c_ulonglong),
        ("GetMixFormat", ctypes.c_ulonglong),
        ("GetDevicePeriod", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Stop", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("SetEventHandle", ctypes.c_ulonglong),
        ("GetService", ctypes.c_ulonglong),
        ("IsOffloadCapable", ctypes.c_ulonglong),
        ("SetClientProperties", ctypes.c_ulonglong),
        ("GetBufferSizeLimits", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Initialize: int
    GetBufferSize: int
    GetStreamLatency: int
    GetCurrentPadding: int
    IsFormatSupported: int
    GetMixFormat: int
    GetDevicePeriod: int
    Start: int
    Stop: int
    Reset: int
    SetEventHandle: int
    GetService: int
    IsOffloadCapable: int
    SetClientProperties: int
    GetBufferSizeLimits: int

class ISpatialAudioObjectRenderStreamNotify_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("OnAvailableDynamicObjectCountChange", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    OnAvailableDynamicObjectCountChange: int

class CAkVorbisFileCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkFileCodec_dtor_0", ctypes.c_ulonglong),
        ("DecodeFile", ctypes.c_ulonglong),
    ]
    IAkFileCodec_dtor_0: int
    DecodeFile: int

class GrowableAlloc(ctypes.Structure):
    def Grow(): pass

    _fields_ = [
        ("m_pData", ctypes.c_ulonglong),
        ("m_size", ctypes.c_int32),
    ]
    m_pData: int
    m_size: int

class vorbis_info_mode(ctypes.Structure):
    _fields_ = [
        ("blockflag", ctypes.c_uint8),
        ("mapping", ctypes.c_uint8),
    ]
    blockflag: int
    mapping: int

class coupling_step(ctypes.Structure):
    _fields_ = [
        ("mag", ctypes.c_uint8),
        ("ang", ctypes.c_uint8),
    ]
    mag: int
    ang: int

class vorbis_info_mapping(ctypes.Structure):
    _fields_ = [
        ("submaps", ctypes.c_int32),
        ("chmuxlist", ctypes.c_ulonglong),
        ("submaplist", ctypes.c_ulonglong),
        ("coupling_steps", ctypes.c_int32),
        ("coupling", ctypes.c_ulonglong),
    ]
    submaps: int
    chmuxlist: int
    submaplist: int
    coupling_steps: int
    coupling: int

class floor1class(ctypes.Structure):
    _fields_ = [
        ("class_dim", ctypes.c_char),
        ("class_subs", ctypes.c_char),
        ("class_book", ctypes.c_uint8),
        ("class_subbook", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    class_dim: str
    class_subs: str
    class_book: int
    class_subbook: list[ctypes.c_uint8]

class vorbis_info_floor1(ctypes.Structure):
    _fields_ = [
        ("Class", ctypes.c_ulonglong),
        ("partitionclass", ctypes.c_char_p),
        ("postlist", ctypes.c_ulonglong),
        ("forward_index", ctypes.c_char_p),
        ("hineighbor", ctypes.c_char_p),
        ("loneighbor", ctypes.c_char_p),
        ("partitions", ctypes.c_int32),
        ("posts", ctypes.c_int32),
        ("mult", ctypes.c_int32),
    ]
    Class: int
    partitionclass: bytes
    postlist: int
    forward_index: bytes
    hineighbor: bytes
    loneighbor: bytes
    partitions: int
    posts: int
    mult: int

class vorbis_info_residue(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_int32),
        ("stagemasks", ctypes.c_ulonglong),
        ("stagebooks", ctypes.c_ulonglong),
        ("begin", ctypes.c_int32),
        ("end", ctypes.c_int32),
        ("grouping", ctypes.c_int32),
        ("partitions", ctypes.c_char),
        ("groupbook", ctypes.c_uint8),
        ("stages", ctypes.c_char),
    ]
    type: int
    stagemasks: int
    stagebooks: int
    begin: int
    end: int
    grouping: int
    partitions: str
    groupbook: int
    stages: str

class codebook(ctypes.Structure):
    _fields_ = [
        ("dec_table", ctypes.c_void_p),
        ("lut_lengths", ctypes.c_ulonglong),
        ("pResidueParams", ctypes.c_ulonglong),
        ("lut_Mask", ctypes.c_uint16),
        ("dim", ctypes.c_uint8),
        ("dec_leafw", ctypes.c_uint8),
        ("dec_maxlength", ctypes.c_uint8),
    ]
    dec_table: int
    lut_lengths: int
    pResidueParams: int
    lut_Mask: int
    dim: int
    dec_leafw: int
    dec_maxlength: int

class oggpack_buffer(ctypes.Structure):
    _fields_ = [
        ("headptr", ctypes.c_ulonglong),
        ("headbit", ctypes.c_char),
        ("headend", ctypes.c_int32),
    ]
    headptr: int
    headbit: str
    headend: int

class ResidueBook(ctypes.Structure):
    _fields_ = [
        ("add_simd", common.Vector4i),
        ("mask_simd", common.Vector4i),
        ("q_del_simd", common.Vector4i),
        ("shift_simd", ctypes.c_int32),
        ("add", ctypes.c_int32),
        ("mask", ctypes.c_int32),
        ("q_delShift", ctypes.c_int32),
        ("q_del", ctypes.c_int32),
        ("Decode", ctypes.c_ulonglong),
        ("q_bits", ctypes.c_uint8),
    ]
    add_simd: common.Vector4i
    mask_simd: common.Vector4i
    q_del_simd: common.Vector4i
    shift_simd: int
    add: int
    mask: int
    q_delShift: int
    q_del: int
    Decode: int
    q_bits: int

class codec_setup_info(ctypes.Structure):
    _fields_ = [
        ("blocksizes", ctypes.c_int32 * 0x2),  # int[2]
        ("modes", ctypes.c_int32),
        ("maps", ctypes.c_int32),
        ("floors", ctypes.c_int32),
        ("residues", ctypes.c_int32),
        ("books", ctypes.c_int32),
        ("mode_param", ctypes.c_ulonglong),
        ("map_param", ctypes.c_ulonglong),
        ("floor_param", ctypes.c_ulonglong),
        ("residue_param", ctypes.c_ulonglong),
        ("book_param", ctypes.c_ulonglong),
    ]
    blocksizes: list[ctypes.c_int32]
    modes: int
    maps: int
    floors: int
    residues: int
    books: int
    mode_param: int
    map_param: int
    floor_param: int
    residue_param: int
    book_param: int

class vorbis_dsp_state2(ctypes.Structure):
    _fields_ = [
        ("out_begin", ctypes.c_int32),
        ("out_end", ctypes.c_int32),
        ("lW", ctypes.c_int32),
        ("W", ctypes.c_int32),
        ("extra_samples_begin", ctypes.c_uint32),
        ("extra_samples_end", ctypes.c_uint32),
        ("bShiftedDCT", ctypes.c_ubyte),
    ]
    out_begin: int
    out_end: int
    lW: int
    W: int
    extra_samples_begin: int
    extra_samples_end: int
    bShiftedDCT: bool

class AkVorbisSeekTableItem(ctypes.Structure):
    _fields_ = [
        ("uPacketFrameOffset", ctypes.c_uint16),
        ("uPacketFileOffset", ctypes.c_uint16),
    ]
    uPacketFrameOffset: int
    uPacketFileOffset: int

class ogg_buffer(ctypes.Structure):
    _fields_ = [
        ("data", ctypes.c_ulonglong),
        ("size", ctypes.c_int32),
    ]
    data: int
    size: int

class CAkSinkBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("IsDataNeeded", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("IsStarved", ctypes.c_ulonglong),
        ("ResetStarved", ctypes.c_ulonglong),
        ("GetSinkPluginType", ctypes.c_ulonglong),
        ("Consume", ctypes.c_ulonglong),
        ("NextWriteBuffer", ctypes.c_ulonglong),
        ("PassData", ctypes.c_ulonglong),
        ("PassSilence", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    IsDataNeeded: int
    OnFrameEnd: int
    IsStarved: int
    ResetStarved: int
    GetSinkPluginType: int
    Consume: int
    NextWriteBuffer: int
    PassData: int
    PassSilence: int

class CAkSrcBankVorbis_vtbl(ctypes.Structure):
    _fields_ = [
        ("CAkVPLNode_dtor_0", ctypes.c_ulonglong),
        ("ProcessDone", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("GetPitch", ctypes.c_ulonglong),
        ("PopMarkers", ctypes.c_ulonglong),
        ("GetChannelMappingFunc", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Seek_60", ctypes.c_ulonglong),
        ("StartStream", ctypes.c_ulonglong),
        ("StopStream", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("IsUsingThisSlot", ctypes.c_ulonglong),
        ("IsUsingThisSlot_98", ctypes.c_ulonglong),
        ("NotifySourceStarvation", ctypes.c_ulonglong),
        ("SupportResampling", ctypes.c_ulonglong),
        ("PrepareSourceSwitch", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("MustRelocatePitchInputBufferOnMediaRelocation", ctypes.c_ulonglong),
        ("MustRelocateAnalysisDataOnMediaRelocation", ctypes.c_ulonglong),
        ("ChangeSourcePosition", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("SrcProcessOrder", ctypes.c_ulonglong),
        ("RestartSourcePlugin", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
        ("SetInfiniteLooping", ctypes.c_ulonglong),
        ("GetSourceSampleRate", ctypes.c_ulonglong),
        ("OnLoopComplete", ctypes.c_ulonglong),
    ]
    CAkVPLNode_dtor_0: int
    ProcessDone: int
    ReleaseBuffer: int
    TimeSkip: int
    VirtualOn: int
    VirtualOff: int
    Seek: int
    GetPitch: int
    PopMarkers: int
    GetChannelMappingFunc: int
    Connect: int
    Term: int
    Seek_60: int
    StartStream: int
    StopStream: int
    GetBuffer: int
    GetDuration: int
    StopLooping: int
    IsUsingThisSlot: int
    IsUsingThisSlot_98: int
    NotifySourceStarvation: int
    SupportResampling: int
    PrepareSourceSwitch: int
    SupportMediaRelocation: int
    RelocateMedia: int
    MustRelocatePitchInputBufferOnMediaRelocation: int
    MustRelocateAnalysisDataOnMediaRelocation: int
    ChangeSourcePosition: int
    GetAnalyzedEnvelope: int
    SrcProcessOrder: int
    RestartSourcePlugin: int
    PrepareNextBuffer: int
    SetInfiniteLooping: int
    GetSourceSampleRate: int
    OnLoopComplete: int

class AkTremoloNonRTPCParams(ctypes.Structure):
    _fields_ = [
        ("bProcessCenter", ctypes.c_ubyte),
        ("bProcessLFE", ctypes.c_ubyte),
        ("bHasChanged", ctypes.c_ubyte),
    ]
    bProcessCenter: bool
    bProcessLFE: bool
    bHasChanged: bool

class CAkTremoloFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class TremoloOutputPolicy(ctypes.Structure):
    _fields_ = [
        ("fGain", ctypes.c_float),
        ("fGainInc", ctypes.c_float),
    ]
    fGain: float
    fGainInc: float

class CAkStereoDelayFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class CAkTremoloFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    TimeSkip: int

class AkVorbisPacketReader(ctypes.Structure):
    _fields_ = [
        ("m_pFirstPacketStart", ctypes.c_ulonglong),
        ("m_pPacketCopy", ctypes.c_ulonglong),
        ("m_uCurrentOffset", ctypes.c_uint32),
        ("m_uTotalSize", ctypes.c_uint32),
    ]
    m_pFirstPacketStart: int
    m_pPacketCopy: int
    m_uCurrentOffset: int
    m_uTotalSize: int

class SinCosGenerator(ctypes.Structure):
    _fields_ = [
        ("cb", common.Vector4f),
        ("s2", common.Vector4f),
        ("s1", common.Vector4f),
        ("c2", common.Vector4f),
        ("c1", common.Vector4f),
    ]
    cb: common.Vector4f
    s2: common.Vector4f
    s1: common.Vector4f
    c2: common.Vector4f
    c1: common.Vector4f

class CAkVorbisGrainCodec_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkGrainCodec_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("SeekTo", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
    ]
    IAkGrainCodec_dtor_0: int
    Init: int
    GetBuffer: int
    ReleaseBuffer: int
    SeekTo: int
    Term: int

class CAkSrcMediaCodecVorbis_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkSrcMediaCodec_dtor_0", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Warmup", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("PrepareNextBuffer", ctypes.c_ulonglong),
        ("GetBuffer", ctypes.c_ulonglong),
        ("ReleaseBuffer", ctypes.c_ulonglong),
        ("VirtualOn", ctypes.c_ulonglong),
        ("VirtualOff", ctypes.c_ulonglong),
        ("FindClosestFileOffset", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("SwitchToNext", ctypes.c_ulonglong),
    ]
    IAkSrcMediaCodec_dtor_0: int
    Init: int
    Warmup: int
    Term: int
    PrepareNextBuffer: int
    GetBuffer: int
    ReleaseBuffer: int
    VirtualOn: int
    VirtualOff: int
    FindClosestFileOffset: int
    Seek: int
    StopLooping: int
    SwitchToNext: int

class OggPacketHeader(ctypes.Structure):
    _fields_ = [
        ("uPacketSize", ctypes.c_uint16),
    ]
    uPacketSize: int

class hkcdDistancePointConvex(ctypes.Structure):
    pass

class floor_Scalar(ctypes.Structure):
    pass

class floor_V4F32(ctypes.Structure):
    pass

class floor_Tremor(ctypes.Structure):
    pass

class floor_AVX(ctypes.Structure):
    pass

class floor_AVX2(ctypes.Structure):
    pass

class hkSimplexSolverActivePlanes(ctypes.Structure):
    _fields_ = [
        ("m_index", ctypes.c_int32),
        ("m_constraint", ctypes.c_ulonglong),
        ("m_interaction", ctypes.c_ulonglong),
    ]
    m_index: int
    m_constraint: int
    m_interaction: int

class hkInt128(ctypes.Structure):
    def setGreatestCommonDivisor(): pass

    def setUnsignedDiv(): pass

    _fields_ = [
        ("m_limbs", ctypes.c_uint64 * 0x2),  # unsigned __int64[2]
    ]
    m_limbs: list[ctypes.c_uint64]

class hkInt64Vector4Impl(ctypes.Structure):
    pass

class hkInt256(ctypes.Structure):
    def addMul(): pass

    def computeDiv32(): pass

    def setAdd(): pass

    def setMul(): pass

    def setNeg(): pass

    _fields_ = [
        ("m_limbs", ctypes.c_uint64 * 0x4),  # unsigned __int64[4]
    ]
    m_limbs: list[ctypes.c_uint64]

class hkInt512(ctypes.Structure):
    _fields_ = [
        ("m_limbs", ctypes.c_uint64 * 0x8),  # unsigned __int64[8]
    ]
    m_limbs: list[ctypes.c_uint64]

class hkcdPlanarGeometryPolygonCollection_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkKeyPair(ctypes.Structure):
    _fields_ = [
        ("m_keyB", ctypes.c_uint32),
        ("m_keyA", ctypes.c_uint32),
    ]
    m_keyB: int
    m_keyA: int

class hkcdPlanarGeometryPlanesCollection_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkcdPlanarEntity_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("extractGeometry", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    extractGeometry: int

class hkcdPlanarEntityDebugger_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("addTrace", ctypes.c_ulonglong),
        ("addGeometry", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    addTrace: int
    addGeometry: int

class hkcdPlanarGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("extractGeometry", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    extractGeometry: int

class hkcdPlanarSolid_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("extractGeometry", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    extractGeometry: int

class hkcdPlanarGeometryWeldUtil(ctypes.Structure):
    pass

class hkcdNewCellsCollection_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkgpMesh_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("reset", ctypes.c_char * 0x8),  # void (__fastcall *)(hkgpAbstractMesh<hkgpMeshBase::Edge,hkgpMeshBase::Vertex,hkgpMeshBase::Triangle,hkBuiltinContainerAllocator<0,0> > *this)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    reset: bytes

class hkgpTopology(ctypes.Structure):
    pass

class hkcdConvexCellsTree3D_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("extractGeometry", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    extractGeometry: int

class hkgpEnhancedConvexHull_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkgpTriangulatorBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkcdFractureEngine_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int

class hkcdPlanarCsgOperand_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkIndexedTransformSetCinfo(ctypes.Structure):
    _fields_ = [
        ("m_inverseMatrices", ctypes.c_char * 0x8),  # hkMatrix4Impl<float> *
        ("m_matrices", ctypes.c_char * 0x8),  # hkMatrix4Impl<float> *
        ("m_matricesOrder", ctypes.c_ulonglong),
        ("m_matricesNames", ctypes.c_ulonglong),
        ("m_numMatrices", ctypes.c_int32),
        ("m_allTransformsAreAffine", ctypes.c_ubyte),
        ("m_indexMappings", ctypes.c_ulonglong),
        ("m_numIndexMappings", ctypes.c_int32),
    ]
    m_inverseMatrices: bytes
    m_matrices: bytes
    m_matricesOrder: int
    m_matricesNames: int
    m_numMatrices: int
    m_allTransformsAreAffine: bool
    m_indexMappings: int
    m_numIndexMappings: int

class hkIndexedTransformSet_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkMeshVertexBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("clone", ctypes.c_ulonglong),
        ("isSharable", ctypes.c_ulonglong),
        ("getVertexFormat", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("lock", ctypes.c_ulonglong),
        ("partialLock", ctypes.c_ulonglong),
        ("getElementVectorArray", ctypes.c_ulonglong),
        ("setElementVectorArray", ctypes.c_ulonglong),
        ("getElementIntArray", ctypes.c_ulonglong),
        ("setElementIntArray", ctypes.c_ulonglong),
        ("unlock", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    clone: int
    isSharable: int
    getVertexFormat: int
    getNumVertices: int
    lock: int
    partialLock: int
    getElementVectorArray: int
    setElementVectorArray: int
    getElementIntArray: int
    setElementIntArray: int
    unlock: int

class hkMeshTexture_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getData", ctypes.c_ulonglong),
        ("setData", ctypes.c_ulonglong),
        ("getFormat", ctypes.c_ulonglong),
        ("getFilename", ctypes.c_ulonglong),
        ("setFilename", ctypes.c_ulonglong),
        ("getHasMipMaps", ctypes.c_ulonglong),
        ("setHasMipMaps", ctypes.c_ulonglong),
        ("getFilterMode", ctypes.c_ulonglong),
        ("setFilterMode", ctypes.c_ulonglong),
        ("getUsageHint", ctypes.c_ulonglong),
        ("setUsageHint", ctypes.c_ulonglong),
        ("getTextureCoordChannel", ctypes.c_ulonglong),
        ("setTextureCoordChannel", ctypes.c_ulonglong),
        ("isReadOnly", ctypes.c_ulonglong),
        ("equals", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getData: int
    setData: int
    getFormat: int
    getFilename: int
    setFilename: int
    getHasMipMaps: int
    setHasMipMaps: int
    getFilterMode: int
    setFilterMode: int
    getUsageHint: int
    setUsageHint: int
    getTextureCoordChannel: int
    setTextureCoordChannel: int
    isReadOnly: int
    equals: int

class hkMeshMaterial_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("createCompatibleVertexFormat", ctypes.c_ulonglong),
        ("createCompatibleVertexBuffer", ctypes.c_ulonglong),
        ("isCompatible", ctypes.c_ulonglong),
        ("equals", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getNumTextures", ctypes.c_ulonglong),
        ("getTexture", ctypes.c_ulonglong),
        ("addTexture", ctypes.c_ulonglong),
        ("setTexture", ctypes.c_ulonglong),
        ("getColors", ctypes.c_ulonglong),
        ("setColors", ctypes.c_ulonglong),
        ("getUserData", ctypes.c_ulonglong),
        ("setUserData", ctypes.c_ulonglong),
        ("getTesselationFactor", ctypes.c_ulonglong),
        ("setTesselationFactor", ctypes.c_ulonglong),
        ("getDisplacementAmount", ctypes.c_ulonglong),
        ("setDisplacementAmount", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    createCompatibleVertexFormat: int
    createCompatibleVertexBuffer: int
    isCompatible: int
    equals: int
    setName: int
    getName: int
    getNumTextures: int
    getTexture: int
    addTexture: int
    setTexture: int
    getColors: int
    setColors: int
    getUserData: int
    setUserData: int
    getTesselationFactor: int
    setTesselationFactor: int
    getDisplacementAmount: int
    setDisplacementAmount: int

class sortByKey(ctypes.Structure):
    pass

class triInfo(ctypes.Structure):
    _fields_ = [
        ("m_key", ctypes.c_uint64),
        ("m_index", ctypes.c_int32),
    ]
    m_key: int
    m_index: int

class sortByIndex(ctypes.Structure):
    pass

class hkMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumSections", ctypes.c_ulonglong),
        ("lockSection", ctypes.c_ulonglong),
        ("unlockSection", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumSections: int
    lockSection: int
    unlockSection: int
    getName: int
    setName: int

class hkMemoryMeshMaterial_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("createCompatibleVertexFormat", ctypes.c_ulonglong),
        ("createCompatibleVertexBuffer", ctypes.c_ulonglong),
        ("isCompatible", ctypes.c_ulonglong),
        ("equals", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("getNumTextures", ctypes.c_ulonglong),
        ("getTexture", ctypes.c_ulonglong),
        ("addTexture", ctypes.c_ulonglong),
        ("setTexture", ctypes.c_ulonglong),
        ("getColors", ctypes.c_ulonglong),
        ("setColors", ctypes.c_ulonglong),
        ("getUserData", ctypes.c_ulonglong),
        ("setUserData", ctypes.c_ulonglong),
        ("getTesselationFactor", ctypes.c_ulonglong),
        ("setTesselationFactor", ctypes.c_ulonglong),
        ("getDisplacementAmount", ctypes.c_ulonglong),
        ("setDisplacementAmount", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    createCompatibleVertexFormat: int
    createCompatibleVertexBuffer: int
    isCompatible: int
    equals: int
    setName: int
    getName: int
    getNumTextures: int
    getTexture: int
    addTexture: int
    setTexture: int
    getColors: int
    setColors: int
    getUserData: int
    setUserData: int
    getTesselationFactor: int
    setTesselationFactor: int
    getDisplacementAmount: int
    setDisplacementAmount: int

class hkMemoryMeshVertexBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("clone", ctypes.c_ulonglong),
        ("isSharable", ctypes.c_ulonglong),
        ("getVertexFormat", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("lock", ctypes.c_ulonglong),
        ("partialLock", ctypes.c_ulonglong),
        ("getElementVectorArray", ctypes.c_ulonglong),
        ("setElementVectorArray", ctypes.c_ulonglong),
        ("getElementIntArray", ctypes.c_ulonglong),
        ("setElementIntArray", ctypes.c_ulonglong),
        ("unlock", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    clone: int
    isSharable: int
    getVertexFormat: int
    getNumVertices: int
    lock: int
    partialLock: int
    getElementVectorArray: int
    setElementVectorArray: int
    getElementIntArray: int
    setElementIntArray: int
    unlock: int

class hkConvexDecompositionImpl_Config_Orientation_Auto(ctypes.Structure):
    pass

class hkgpDeprecatedConvexDecomposition_GuardGenConfig_eMethod_Auto(ctypes.Structure):
    pass

class hkMultipleVertexBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("clone", ctypes.c_ulonglong),
        ("isSharable", ctypes.c_ulonglong),
        ("getVertexFormat", ctypes.c_ulonglong),
        ("getNumVertices", ctypes.c_ulonglong),
        ("lock", ctypes.c_ulonglong),
        ("partialLock", ctypes.c_ulonglong),
        ("getElementVectorArray", ctypes.c_ulonglong),
        ("setElementVectorArray", ctypes.c_ulonglong),
        ("getElementIntArray", ctypes.c_ulonglong),
        ("setElementIntArray", ctypes.c_ulonglong),
        ("unlock", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    clone: int
    isSharable: int
    getVertexFormat: int
    getNumVertices: int
    lock: int
    partialLock: int
    getElementVectorArray: int
    setElementVectorArray: int
    getElementIntArray: int
    setElementIntArray: int
    unlock: int

class hkgpDeprecatedConvexDecomposition_Config_ePrimaryMethod_Auto(ctypes.Structure):
    pass

class hkgpDeprecatedConvexDecomposition_Config_eReduceMethod_Auto(ctypes.Structure):
    pass

class hkgpDeprecatedConvexDecomposition_Config_eHollowParts_Auto(ctypes.Structure):
    pass

class hkMeshBody_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getMeshShape", ctypes.c_ulonglong),
        ("getTransform", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, hkMatrix4Impl<float> *)
        ("setTransform", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, const hkMatrix4Impl<float> *)
        ("getVertexBuffer", ctypes.c_ulonglong),
        ("getNumIndexedTransforms", ctypes.c_ulonglong),
        ("setIndexedTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, const hkMatrix4Impl<float> *, int)
        ("getIndexedTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, hkMatrix4Impl<float> *, int)
        ("getIndexedInverseTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, hkMatrix4Impl<float> *, int)
        ("getIndexTransformsOrder", ctypes.c_ulonglong),
        ("getIndexTransformNames", ctypes.c_ulonglong),
        ("getIndexMappings", ctypes.c_ulonglong),
        ("getNumIndexMappings", ctypes.c_ulonglong),
        ("setPickingData", ctypes.c_ulonglong),
        ("completeUpdate", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, const hkMatrix4Impl<float> *)
        ("completeUpdate_88", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getMeshShape: int
    getTransform: bytes
    setTransform: bytes
    getVertexBuffer: int
    getNumIndexedTransforms: int
    setIndexedTransforms: bytes
    getIndexedTransforms: bytes
    getIndexedInverseTransforms: bytes
    getIndexTransformsOrder: int
    getIndexTransformNames: int
    getIndexMappings: int
    getNumIndexMappings: int
    setPickingData: int
    completeUpdate: bytes
    completeUpdate_88: int
    getName: int
    setName: int

class hkMeshSystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getMaterialRegistry", ctypes.c_ulonglong),
        ("createShape", ctypes.c_ulonglong),
        ("createCompoundShape", ctypes.c_ulonglong),
        ("createCompoundShape_30", ctypes.c_char * 0x8),  # hkMeshShape *(__fastcall *)(hkMeshSystem *this, const hkMeshShape *const *, const hkMatrix4Impl<float> *, int)
        ("createBody", ctypes.c_char * 0x8),  # hkMeshBody *(__fastcall *)(hkMeshSystem *this, const hkMeshShape *, const hkMatrix4Impl<float> *, hkIndexedTransformSetCinfo *)
        ("createVertexBuffer", ctypes.c_ulonglong),
        ("createVertexBuffer_48", ctypes.c_ulonglong),
        ("findSuitableVertexFormat", ctypes.c_ulonglong),
        ("createMaterial", ctypes.c_ulonglong),
        ("cloneMaterial", ctypes.c_ulonglong),
        ("createTexture", ctypes.c_ulonglong),
        ("addBody", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("update", ctypes.c_ulonglong),
        ("freeResources", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getMaterialRegistry: int
    createShape: int
    createCompoundShape: int
    createCompoundShape_30: bytes
    createBody: bytes
    createVertexBuffer: int
    createVertexBuffer_48: int
    findSuitableVertexFormat: int
    createMaterial: int
    cloneMaterial: int
    createTexture: int
    addBody: int
    removeBody: int
    update: int
    freeResources: int

class hkDefaultCompoundMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumSections", ctypes.c_ulonglong),
        ("lockSection", ctypes.c_ulonglong),
        ("unlockSection", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumSections: int
    lockSection: int
    unlockSection: int
    getName: int
    setName: int

class hkDefaultCompoundMeshBody_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getMeshShape", ctypes.c_ulonglong),
        ("getTransform", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, hkMatrix4Impl<float> *)
        ("setTransform", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, const hkMatrix4Impl<float> *)
        ("getVertexBuffer", ctypes.c_ulonglong),
        ("getNumIndexedTransforms", ctypes.c_ulonglong),
        ("setIndexedTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, const hkMatrix4Impl<float> *, int)
        ("getIndexedTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, hkMatrix4Impl<float> *, int)
        ("getIndexedInverseTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, hkMatrix4Impl<float> *, int)
        ("getIndexTransformsOrder", ctypes.c_ulonglong),
        ("getIndexTransformNames", ctypes.c_ulonglong),
        ("getIndexMappings", ctypes.c_ulonglong),
        ("getNumIndexMappings", ctypes.c_ulonglong),
        ("setPickingData", ctypes.c_ulonglong),
        ("completeUpdate", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, const hkMatrix4Impl<float> *)
        ("completeUpdate_88", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
        ("setIndexedInverseTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDefaultCompoundMeshBody *this, int, const hkMatrix4Impl<float> *, int)
        ("ensureInverseTransforms", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getMeshShape: int
    getTransform: bytes
    setTransform: bytes
    getVertexBuffer: int
    getNumIndexedTransforms: int
    setIndexedTransforms: bytes
    getIndexedTransforms: bytes
    getIndexedInverseTransforms: bytes
    getIndexTransformsOrder: int
    getIndexTransformNames: int
    getIndexMappings: int
    getNumIndexMappings: int
    setPickingData: int
    completeUpdate: bytes
    completeUpdate_88: int
    getName: int
    setName: int
    setIndexedInverseTransforms: bytes
    ensureInverseTransforms: int

class hkMemoryMeshBody_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getMeshShape", ctypes.c_ulonglong),
        ("getTransform", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, hkMatrix4Impl<float> *)
        ("setTransform", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, const hkMatrix4Impl<float> *)
        ("getVertexBuffer", ctypes.c_ulonglong),
        ("getNumIndexedTransforms", ctypes.c_ulonglong),
        ("setIndexedTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, const hkMatrix4Impl<float> *, int)
        ("getIndexedTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, hkMatrix4Impl<float> *, int)
        ("getIndexedInverseTransforms", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, int, hkMatrix4Impl<float> *, int)
        ("getIndexTransformsOrder", ctypes.c_ulonglong),
        ("getIndexTransformNames", ctypes.c_ulonglong),
        ("getIndexMappings", ctypes.c_ulonglong),
        ("getNumIndexMappings", ctypes.c_ulonglong),
        ("setPickingData", ctypes.c_ulonglong),
        ("completeUpdate", ctypes.c_char * 0x8),  # void (__fastcall *)(hkMeshBody *this, const hkMatrix4Impl<float> *)
        ("completeUpdate_88", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getMeshShape: int
    getTransform: bytes
    setTransform: bytes
    getVertexBuffer: int
    getNumIndexedTransforms: int
    setIndexedTransforms: bytes
    getIndexedTransforms: bytes
    getIndexedInverseTransforms: bytes
    getIndexTransformsOrder: int
    getIndexTransformNames: int
    getIndexMappings: int
    getNumIndexMappings: int
    setPickingData: int
    completeUpdate: bytes
    completeUpdate_88: int
    getName: int
    setName: int

class hkMemoryMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumSections", ctypes.c_ulonglong),
        ("lockSection", ctypes.c_ulonglong),
        ("unlockSection", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumSections: int
    lockSection: int
    unlockSection: int
    getName: int
    setName: int

class hkMemoryMeshSystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getMaterialRegistry", ctypes.c_ulonglong),
        ("createShape", ctypes.c_ulonglong),
        ("createCompoundShape", ctypes.c_ulonglong),
        ("createCompoundShape_30", ctypes.c_char * 0x8),  # hkMeshShape *(__fastcall *)(hkMeshSystem *this, const hkMeshShape *const *, const hkMatrix4Impl<float> *, int)
        ("createBody", ctypes.c_char * 0x8),  # hkMeshBody *(__fastcall *)(hkMeshSystem *this, const hkMeshShape *, const hkMatrix4Impl<float> *, hkIndexedTransformSetCinfo *)
        ("createVertexBuffer", ctypes.c_ulonglong),
        ("createVertexBuffer_48", ctypes.c_ulonglong),
        ("findSuitableVertexFormat", ctypes.c_ulonglong),
        ("createMaterial", ctypes.c_ulonglong),
        ("cloneMaterial", ctypes.c_ulonglong),
        ("createTexture", ctypes.c_ulonglong),
        ("addBody", ctypes.c_ulonglong),
        ("removeBody", ctypes.c_ulonglong),
        ("update", ctypes.c_ulonglong),
        ("freeResources", ctypes.c_ulonglong),
        ("createSkinnedVertexBuffer", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getMaterialRegistry: int
    createShape: int
    createCompoundShape: int
    createCompoundShape_30: bytes
    createBody: bytes
    createVertexBuffer: int
    createVertexBuffer_48: int
    findSuitableVertexFormat: int
    createMaterial: int
    cloneMaterial: int
    createTexture: int
    addBody: int
    removeBody: int
    update: int
    freeResources: int
    createSkinnedVertexBuffer: int

class hkMemoryMeshTexture_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getData", ctypes.c_ulonglong),
        ("setData", ctypes.c_ulonglong),
        ("getFormat", ctypes.c_ulonglong),
        ("getFilename", ctypes.c_ulonglong),
        ("setFilename", ctypes.c_ulonglong),
        ("getHasMipMaps", ctypes.c_ulonglong),
        ("setHasMipMaps", ctypes.c_ulonglong),
        ("getFilterMode", ctypes.c_ulonglong),
        ("setFilterMode", ctypes.c_ulonglong),
        ("getUsageHint", ctypes.c_ulonglong),
        ("setUsageHint", ctypes.c_ulonglong),
        ("getTextureCoordChannel", ctypes.c_ulonglong),
        ("setTextureCoordChannel", ctypes.c_ulonglong),
        ("isReadOnly", ctypes.c_ulonglong),
        ("equals", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getData: int
    setData: int
    getFormat: int
    getFilename: int
    setFilename: int
    getHasMipMaps: int
    setHasMipMaps: int
    getFilterMode: int
    setFilterMode: int
    getUsageHint: int
    setUsageHint: int
    getTextureCoordChannel: int
    setTextureCoordChannel: int
    isReadOnly: int
    equals: int

class hkcdRayCastResult_Enum_Auto(ctypes.Structure):
    pass

class hkSkinBinding_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumSections", ctypes.c_ulonglong),
        ("lockSection", ctypes.c_ulonglong),
        ("unlockSection", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumSections: int
    lockSection: int
    unlockSection: int
    getName: int
    setName: int

class hkSkinnedMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumBoneSections", ctypes.c_ulonglong),
        ("getBoneSection", ctypes.c_ulonglong),
        ("getBoneSet", ctypes.c_char * 0x8),  # void (__fastcall *)(hkSkinnedMeshShape *this, hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant>, hkSkinnedMeshShape::BoneSet *)
        ("getBoneSetBones", ctypes.c_char * 0x8),  # void (__fastcall *)(hkSkinnedMeshShape *this, hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant>, hkArray<short,hkBuiltinContainerAllocator<0,0> > *)
        ("getNumParts", ctypes.c_ulonglong),
        ("getPart", ctypes.c_ulonglong),
        ("dbgOut", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
        ("addBoneSection", ctypes.c_char * 0x8),  # void (__fastcall *)(hkSkinnedMeshShape *this, hkMeshShape *, hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant>, __int16)
        ("addPart", ctypes.c_ulonglong),
        ("sortParts", ctypes.c_ulonglong),
        ("addBoneSet", ctypes.c_char * 0x8),  # hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant> *(__fastcall *)(hkSkinnedMeshShape *this, hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant> *result, const __int16 *, int)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumBoneSections: int
    getBoneSection: int
    getBoneSet: bytes
    getBoneSetBones: bytes
    getNumParts: int
    getPart: int
    dbgOut: int
    getName: int
    setName: int
    addBoneSection: bytes
    addPart: int
    sortParts: int
    addBoneSet: bytes

class hkStorageSkinnedMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumBoneSections", ctypes.c_ulonglong),
        ("getBoneSection", ctypes.c_ulonglong),
        ("getBoneSet", ctypes.c_char * 0x8),  # void (__fastcall *)(hkSkinnedMeshShape *this, hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant>, hkSkinnedMeshShape::BoneSet *)
        ("getBoneSetBones", ctypes.c_char * 0x8),  # void (__fastcall *)(hkSkinnedMeshShape *this, hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant>, hkArray<short,hkBuiltinContainerAllocator<0,0> > *)
        ("getNumParts", ctypes.c_ulonglong),
        ("getPart", ctypes.c_ulonglong),
        ("dbgOut", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
        ("addBoneSection", ctypes.c_char * 0x8),  # void (__fastcall *)(hkSkinnedMeshShape *this, hkMeshShape *, hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant>, __int16)
        ("addPart", ctypes.c_ulonglong),
        ("sortParts", ctypes.c_ulonglong),
        ("addBoneSet", ctypes.c_char * 0x8),  # hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant> *(__fastcall *)(hkSkinnedMeshShape *this, hkHandle<unsigned short,65535,hkSkinnedMeshShape::BoneSetIdDiscriminant> *result, const __int16 *, int)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumBoneSections: int
    getBoneSection: int
    getBoneSet: bytes
    getBoneSetBones: bytes
    getNumParts: int
    getPart: int
    dbgOut: int
    getName: int
    setName: int
    addBoneSection: bytes
    addPart: int
    sortParts: int
    addBoneSet: bytes

class IConvexOverlapImpl_vtbl(ctypes.Structure):
    _fields_ = [
        ("IConvexOverlap_dtor_0", ctypes.c_ulonglong),
        ("distance", ctypes.c_ulonglong),
        ("checkOverlap", ctypes.c_ulonglong),
    ]
    IConvexOverlap_dtor_0: int
    distance: int
    checkOverlap: int

class hkSkinnedRefMeshShape_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getNumSections", ctypes.c_ulonglong),
        ("lockSection", ctypes.c_ulonglong),
        ("unlockSection", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getNumSections: int
    lockSection: int
    unlockSection: int
    getName: int
    setName: int

class hkgpIndexedMesh_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("reset", ctypes.c_char * 0x8),  # void (__fastcall *)(hkgpAbstractMesh<hkgpIndexedMeshDefinitions::Edge,hkgpIndexedMeshDefinitions::Vertex,hkgpIndexedMeshDefinitions::Triangle,hkBuiltinContainerAllocator<0,0> > *this)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    reset: bytes

class TrianglePairCollector(ctypes.Structure):
    _fields_ = [
        ("m_pairs", ctypes.c_char * 0x8),  # hkArray<hkgpMesh::TrianglePair,hkBuiltinContainerAllocator<0,0> > *
    ]
    m_pairs: bytes

class NearestFeaturePolicy(ctypes.Structure):
    _fields_ = [
        ("m_normal", common.Vector4f),
        ("m_coordinates", common.Vector4f),
        ("m_nearestTriangle", ctypes.c_ulonglong),
    ]
    m_normal: common.Vector4f
    m_coordinates: common.Vector4f
    m_nearestTriangle: int

class hkgpMeshInternals_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("reset", ctypes.c_char * 0x8),  # void (__fastcall *)(hkgpAbstractMesh<hkgpMeshBase::Edge,hkgpMeshBase::Vertex,hkgpMeshBase::Triangle,hkBuiltinContainerAllocator<0,0> > *this)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    reset: bytes

class hkMeshSection_MeshSectionIndexType_Auto(ctypes.Structure):
    pass

class hkMeshSection_PrimitiveType_Auto(ctypes.Structure):
    pass

class hkMeshTexture_Format_Auto(ctypes.Structure):
    pass

class hkMeshTexture_FilterMode_Auto(ctypes.Structure):
    pass

class hkMeshTexture_TextureUsageType_Auto(ctypes.Structure):
    pass

class hkVertexFormat_ComponentType_Auto(ctypes.Structure):
    pass

class hkVertexFormat_ComponentUsage_Auto(ctypes.Structure):
    pass

class hkVertexFormat_HintFlags_Auto(ctypes.Structure):
    pass

class hkMergeMeshPrimitvesCalculator(ctypes.Structure):
    _fields_ = [
        ("_m_mergedPrimitiveType", ctypes.c_uint32),
        ("_m_mergedIndexType", ctypes.c_uint32),
        ("m_numTotalPrimitives", ctypes.c_int32),
    ]
    _m_mergedPrimitiveType: int
    _m_mergedIndexType: int
    m_numTotalPrimitives: int

    @property
    def m_mergedPrimitiveType(self):
        return hkMeshSection.PrimitiveType(self._m_mergedPrimitiveType)

    @property
    def m_mergedIndexType(self):
        return hkMeshSection.MeshSectionIndexType(self._m_mergedIndexType)


class hkgpCgo_Config_VertexSemantic_Auto(ctypes.Structure):
    pass

class hkgpCgo_Config_VertexCombinator_Auto(ctypes.Structure):
    pass

class hkgpCgo_Config_SolverAccuracy_Auto(ctypes.Structure):
    pass

class hkKeycodeDataBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkKeycodeData_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpFullManifoldUtil(ctypes.Structure):
    pass

class hkParserBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkXmlLexAnalyzer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkXmlStreamParser_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class IntersectPlanesInsertFunc(ctypes.Structure):
    pass

class MergePlanesIntersectPlanesInsertFunc(ctypes.Structure):
    _fields_ = [
        ("m_updatedPlanes", ctypes.c_ulonglong),
        ("m_orphanedPlanes", ctypes.c_ulonglong),
        ("m_updatedVertices", ctypes.c_ulonglong),
        ("m_originalVertices", ctypes.c_ulonglong),
        ("m_orignalPlanes", ctypes.c_ulonglong),
        ("m_numOriginalVertices", ctypes.c_int32),
    ]
    m_updatedPlanes: int
    m_orphanedPlanes: int
    m_updatedVertices: int
    m_originalVertices: int
    m_orignalPlanes: int
    m_numOriginalVertices: int

class InternalEdgeSortingStruct(ctypes.Structure):
    _fields_ = [
        ("m_key", ctypes.c_char * 0x4),  # hkHandle<unsigned int,4294967295,hkcdStaticMeshTree::PrimitiveKeyDiscriminant>
        ("m_cost", ctypes.c_float),
    ]
    m_key: bytes
    m_cost: float

class hctClothSetupObjectData20151_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hctClothSetupClothData20151Options_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hctClothSetup20151Options_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkStringOld(ctypes.Structure):
    _fields_ = [
        ("m_string", ctypes.c_char * 0x10),  # hkArray<char,hkBuiltinContainerAllocator<0,0> >
    ]
    m_string: bytes

class hkUintVle(ctypes.Structure):
    def read(): pass

    def write(): pass

    _fields_ = [
        ("m_i", ctypes.c_uint64),
    ]
    m_i: int

class hkResourceBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    getName: int

class hkResourceMap(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class hkResourceMap_vtbl(ctypes.Structure):
    _fields_ = [
        ("findObjectByName", ctypes.c_ulonglong),
        ("hkResourceMap_dtor_8", ctypes.c_ulonglong),
    ]
    findObjectByName: int
    hkResourceMap_dtor_8: int

class hkResourceHandle_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
        ("getObject", ctypes.c_ulonglong),
        ("getObjectType", ctypes.c_ulonglong),
        ("setObject", ctypes.c_ulonglong),
        ("addExternalLink", ctypes.c_ulonglong),
        ("removeExternalLink", ctypes.c_ulonglong),
        ("getExternalLinks", ctypes.c_char * 0x8),  # void (__fastcall *)(hkResourceHandle *this, hkArray<hkResourceHandle::Link,hkBuiltinContainerAllocator<0,0> > *)
        ("clearExternalLinks", ctypes.c_ulonglong),
        ("tryToResolveLinks", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    getName: int
    setName: int
    getObject: int
    getObjectType: int
    setObject: int
    addExternalLink: int
    removeExternalLink: int
    getExternalLinks: bytes
    clearExternalLinks: int
    tryToResolveLinks: int

class hkResourceContainer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("createResource", ctypes.c_ulonglong),
        ("destroyResource", ctypes.c_ulonglong),
        ("getNumResources", ctypes.c_ulonglong),
        ("findResourceByName", ctypes.c_ulonglong),
        ("findResourceByType", ctypes.c_ulonglong),
        ("getParent", ctypes.c_ulonglong),
        ("createContainer", ctypes.c_ulonglong),
        ("destroyContainer", ctypes.c_ulonglong),
        ("getNumContainers", ctypes.c_ulonglong),
        ("findContainerByName", ctypes.c_ulonglong),
        ("parentTo", ctypes.c_ulonglong),
        ("tryToResolveLinks", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    getName: int
    createResource: int
    destroyResource: int
    getNumResources: int
    findResourceByName: int
    findResourceByType: int
    getParent: int
    createContainer: int
    destroyContainer: int
    getNumContainers: int
    findContainerByName: int
    parentTo: int
    tryToResolveLinks: int

class hkMemoryResourceHandle_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("setName", ctypes.c_ulonglong),
        ("getObject", ctypes.c_ulonglong),
        ("getObjectType", ctypes.c_ulonglong),
        ("setObject", ctypes.c_ulonglong),
        ("addExternalLink", ctypes.c_ulonglong),
        ("removeExternalLink", ctypes.c_ulonglong),
        ("getExternalLinks", ctypes.c_char * 0x8),  # void (__fastcall *)(hkResourceHandle *this, hkArray<hkResourceHandle::Link,hkBuiltinContainerAllocator<0,0> > *)
        ("clearExternalLinks", ctypes.c_ulonglong),
        ("tryToResolveLinks", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    getName: int
    setName: int
    getObject: int
    getObjectType: int
    setObject: int
    addExternalLink: int
    removeExternalLink: int
    getExternalLinks: bytes
    clearExternalLinks: int
    tryToResolveLinks: int

class hkMemoryResourceContainer_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getName", ctypes.c_ulonglong),
        ("createResource", ctypes.c_ulonglong),
        ("destroyResource", ctypes.c_ulonglong),
        ("getNumResources", ctypes.c_ulonglong),
        ("findResourceByName", ctypes.c_ulonglong),
        ("findResourceByType", ctypes.c_ulonglong),
        ("getParent", ctypes.c_ulonglong),
        ("createContainer", ctypes.c_ulonglong),
        ("destroyContainer", ctypes.c_ulonglong),
        ("getNumContainers", ctypes.c_ulonglong),
        ("findContainerByName", ctypes.c_ulonglong),
        ("parentTo", ctypes.c_ulonglong),
        ("tryToResolveLinks", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    getType: int
    getName: int
    createResource: int
    destroyResource: int
    getNumResources: int
    findResourceByName: int
    findResourceByType: int
    getParent: int
    createContainer: int
    destroyContainer: int
    getNumContainers: int
    findContainerByName: int
    parentTo: int
    tryToResolveLinks: int

class hkContainerResourceMap_vtbl(ctypes.Structure):
    _fields_ = [
        ("findObjectByName", ctypes.c_ulonglong),
        ("hkResourceMap_dtor_8", ctypes.c_ulonglong),
    ]
    findObjectByName: int
    hkResourceMap_dtor_8: int

class hctAttributeDescription20151_ForcedType_Auto(ctypes.Structure):
    pass

class hctAttributeDescription20151_Hint_Auto(ctypes.Structure):
    pass

class RaiiReadLimiter(ctypes.Structure):
    _fields_ = [
        ("m_rbuf", ctypes.c_ulonglong),
        ("m_oldLimit", ctypes.c_int64),
    ]
    m_rbuf: int
    m_oldLimit: int

class hkLoadUtil(ctypes.Structure):
    _fields_ = [
        ("m_fileName", ctypes.c_char_p),
        ("m_reader", ctypes.c_char * 0x8),  # hkRefPtr<hkStreamReader>
    ]
    m_fileName: bytes
    m_reader: bytes

class hkVdbLocalOStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class PrintFormatVisitor(ctypes.Structure):
    pass

class WriteVarVisitor(ctypes.Structure):
    _fields_ = [
        ("m_formatter", ctypes.c_ulonglong),
        ("m_idFromVar", ctypes.c_ulonglong),
        ("m_typeWriter", ctypes.c_ulonglong),
    ]
    m_formatter: int
    m_idFromVar: int
    m_typeWriter: int

class hkVdbLocalIStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class VdbObjectTypeBuilder(ctypes.Structure):
    _fields_ = [
        ("m_type", ctypes.c_ulonglong),
        ("m_isSubType", ctypes.c_uint32),
        ("m_insideSubRecord", ctypes.c_uint32),
        ("m_unsupportedMemberCount", ctypes.c_uint32),
        ("m_localOffsetIdx", ctypes.c_int32),
    ]
    m_type: int
    m_isSubType: int
    m_insideSubRecord: int
    m_unsupportedMemberCount: int
    m_localOffsetIdx: int

class VdbObjectTweaker(ctypes.Structure):
    _fields_ = [
        ("m_type", ctypes.c_ulonglong),
    ]
    m_type: int

class SharesetCrossfaderObjectsMemory(ctypes.Structure):
    _fields_ = [
        ("pAllocator", ctypes.c_ulonglong),
        ("pObjectIDs", ctypes.c_ulonglong),
        ("pLastSamples", ctypes.c_ulonglong),
        ("maxNumberOfObjects", ctypes.c_uint32),
    ]
    pAllocator: int
    pObjectIDs: int
    pLastSamples: int
    maxNumberOfObjects: int

class SceAudioOut2MasteringStatesDescriptor(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint32),
        ("size", ctypes.c_uint32),
    ]
    id: int
    size: int

class SceAudioMasterModuleScratch(ctypes.Structure):
    _fields_ = [
        ("fTempProcessingBuffer", ctypes.c_ulonglong),
        ("fTempFilterBankBuffer", ctypes.c_ulonglong),
        ("fAkInterleaveBuffer", ctypes.c_ulonglong),
        ("fAkDummyModuleGains", ctypes.c_ulonglong),
    ]
    fTempProcessingBuffer: int
    fTempFilterBankBuffer: int
    fAkInterleaveBuffer: int
    fAkDummyModuleGains: int

class AkBufferAllocateInfo(ctypes.Structure):
    _fields_ = [
        ("numChannels", ctypes.c_uint32),
        ("channelStride", ctypes.c_uint32),
        ("akChannelMask", ctypes.c_uint32),
        ("sceChannelMap", ctypes.c_uint32 * 0xC),  # unsigned int[12]
        ("saveModuleGains", ctypes.c_ubyte),
    ]
    numChannels: int
    channelStride: int
    akChannelMask: int
    sceChannelMap: list[ctypes.c_uint32]
    saveModuleGains: bool

class ChannelManagerConfig(ctypes.Structure):
    _fields_ = [
        ("pluginId", ctypes.c_uint32),
        ("contextId", ctypes.c_uint32),
        ("bProcessObjects", ctypes.c_ubyte),
        ("bProcessPassthrough", ctypes.c_ubyte),
        ("numMainMixChannels", ctypes.c_uint32),
        ("numPassthroughChannels", ctypes.c_uint32),
        ("maxNumObjects", ctypes.c_uint32),
        ("pInData", ctypes.c_ulonglong),
        ("pOutData", ctypes.c_ulonglong),
        ("pScratch", ctypes.c_ulonglong),
    ]
    pluginId: int
    contextId: int
    bProcessObjects: bool
    bProcessPassthrough: bool
    numMainMixChannels: int
    numPassthroughChannels: int
    maxNumObjects: int
    pInData: int
    pOutData: int
    pScratch: int

class SceAudioOut2MasteringParamsDescriptor(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_uint32),
        ("size", ctypes.c_uint32),
    ]
    id: int
    size: int

class FilterWork(ctypes.Structure):
    _fields_ = [
        ("fCoeffB", ctypes.c_float * 0x3),  # float[3]
        ("fCoeffA", ctypes.c_float * 0x3),  # float[3]
        ("fPrevIn1", ctypes.c_float * 0xC),  # float[12]
        ("fPrevIn2", ctypes.c_float * 0xC),  # float[12]
        ("fPrevOut1", ctypes.c_float * 0xC),  # float[12]
        ("fPrevOut2", ctypes.c_float * 0xC),  # float[12]
    ]
    fCoeffB: list[ctypes.c_float]
    fCoeffA: list[ctypes.c_float]
    fPrevIn1: list[ctypes.c_float]
    fPrevIn2: list[ctypes.c_float]
    fPrevOut1: list[ctypes.c_float]
    fPrevOut2: list[ctypes.c_float]

class CompressorBandWork(ctypes.Structure):
    _fields_ = [
        ("fThreshold", ctypes.c_float * 0xC),  # float[12]
        ("fRatio", ctypes.c_float * 0xC),  # float[12]
        ("fAttack", ctypes.c_float * 0xC),  # float[12]
        ("fRelease", ctypes.c_float * 0xC),  # float[12]
        ("fMakeupGain", ctypes.c_float * 0xC),  # float[12]
        ("fKnee", ctypes.c_float * 0xC),  # float[12]
        ("fCompressionSlope", ctypes.c_float * 0xC),  # float[12]
        ("fTav", ctypes.c_float * 0xC),  # float[12]
        ("fInputRms", ctypes.c_float * 0xC),  # float[12]
        ("fCompressionCoeff", ctypes.c_float * 0xC),  # float[12]
        ("fStartComp", ctypes.c_float * 0xC),  # float[12]
        ("savedGains", ctypes.c_ulonglong * 0xC),  # float *[12]
    ]
    fThreshold: list[ctypes.c_float]
    fRatio: list[ctypes.c_float]
    fAttack: list[ctypes.c_float]
    fRelease: list[ctypes.c_float]
    fMakeupGain: list[ctypes.c_float]
    fKnee: list[ctypes.c_float]
    fCompressionSlope: list[ctypes.c_float]
    fTav: list[ctypes.c_float]
    fInputRms: list[ctypes.c_float]
    fCompressionCoeff: list[ctypes.c_float]
    fStartComp: list[ctypes.c_float]
    savedGains: list[ctypes.c_ulonglong]

class SceAudioMasterModuleConfig(ctypes.Structure):
    _fields_ = [
        ("uHostSampleRate", ctypes.c_uint32),
        ("uHostGranularity", ctypes.c_uint32),
        ("uModuleChannels", ctypes.c_uint32),
        ("uModuleChannelsMax", ctypes.c_uint32),
        ("uChannelStride", ctypes.c_uint32),
        ("akChannelMask", ctypes.c_uint32),
        ("sceChannelMap", ctypes.c_uint32 * 0xC),  # unsigned int[12]
        ("bSaveModuleGains", ctypes.c_ubyte),
    ]
    uHostSampleRate: int
    uHostGranularity: int
    uModuleChannels: int
    uModuleChannelsMax: int
    uChannelStride: int
    akChannelMask: int
    sceChannelMap: list[ctypes.c_uint32]
    bSaveModuleGains: bool

class SceAudioMasterModuleDef(ctypes.Structure):
    _fields_ = [
        ("moduleId", ctypes.c_int32),
        ("pModuleWork", ctypes.c_uint64),
        ("initFn", ctypes.c_ulonglong),
        ("termFn", ctypes.c_ulonglong),
        ("processFn", ctypes.c_ulonglong),
        ("setParamsFn", ctypes.c_ulonglong),
        ("getStatesFn", ctypes.c_ulonglong),
        ("checkParamsFn", ctypes.c_ulonglong),
        ("uBypassFlags", ctypes.c_uint32),
        ("uStateFlags", ctypes.c_uint32),
    ]
    moduleId: int
    pModuleWork: int
    initFn: int
    termFn: int
    processFn: int
    setParamsFn: int
    getStatesFn: int
    checkParamsFn: int
    uBypassFlags: int
    uStateFlags: int

class SceAudioOut2MasteringParamEqFilterParams(ctypes.Structure):
    _fields_ = [
        ("filterMode", ctypes.c_uint32),
        ("frequency", ctypes.c_float),
        ("resonance", ctypes.c_float),
        ("gain", ctypes.c_float),
    ]
    filterMode: int
    frequency: float
    resonance: float
    gain: float

class SceAudioOut2MasteringStatesHeader(ctypes.Structure):
    _fields_ = [
        ("statesId", ctypes.c_uint32),
    ]
    statesId: int

class SceAudioOut2MasteringParamsHeader(ctypes.Structure):
    _fields_ = [
        ("paramsId", ctypes.c_uint32),
    ]
    paramsId: int

class SceAudioOut2MasteringCompressorBandParams(ctypes.Structure):
    _fields_ = [
        ("threshold", ctypes.c_float),
        ("ratio", ctypes.c_float),
        ("attack", ctypes.c_float),
        ("release", ctypes.c_float),
        ("makeupGain", ctypes.c_float),
        ("knee", ctypes.c_float),
    ]
    threshold: float
    ratio: float
    attack: float
    release: float
    makeupGain: float
    knee: float

class MasteringSuiteFXParams_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkRTPCSubscriber_dtor_0", ctypes.c_ulonglong),
        ("SetParam", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("SetParamsBlock", ctypes.c_ulonglong),
    ]
    IAkRTPCSubscriber_dtor_0: int
    SetParam: int
    Clone: int
    Init: int
    Term: int
    SetParamsBlock: int

class MasterVolumeChannelWork(ctypes.Structure):
    _fields_ = [
        ("fVolume", ctypes.c_float * 0xC),  # float[12]
    ]
    fVolume: list[ctypes.c_float]

class LimiterChannelWork(ctypes.Structure):
    _fields_ = [
        ("fThreshold", ctypes.c_float * 0xC),  # float[12]
        ("fAttack", ctypes.c_float * 0xC),  # float[12]
        ("fRelease", ctypes.c_float * 0xC),  # float[12]
        ("fOutputGain", ctypes.c_float * 0xC),  # float[12]
        ("fPeak", ctypes.c_float * 0xC),  # float[12]
        ("fCompressionCoeff", ctypes.c_float * 0xC),  # float[12]
        ("fInputMean", ctypes.c_float * 0xC),  # float[12]
        ("fInputStd", ctypes.c_float * 0xC),  # float[12]
        ("fCurrentMean", ctypes.c_float * 0xC),  # float[12]
        ("fCurrentStd", ctypes.c_float * 0xC),  # float[12]
        ("fReleaseAttack", ctypes.c_float * 0xC),  # float[12]
        ("fShortTermRelease", ctypes.c_float * 0xC),  # float[12]
        ("fCurrentRelease", ctypes.c_float * 0xC),  # float[12]
        ("fNextPeakRelease", ctypes.c_float * 0xC),  # float[12]
        ("fGainPeak", ctypes.c_float * 0xC),  # float[12]
        ("fInputPeak", ctypes.c_float * 0xC),  # float[12]
        ("fOutputPeak", ctypes.c_float * 0xC),  # float[12]
        ("histBuffer", ctypes.c_ulonglong),
        ("savedGains", ctypes.c_ulonglong * 0xC),  # float *[12]
    ]
    fThreshold: list[ctypes.c_float]
    fAttack: list[ctypes.c_float]
    fRelease: list[ctypes.c_float]
    fOutputGain: list[ctypes.c_float]
    fPeak: list[ctypes.c_float]
    fCompressionCoeff: list[ctypes.c_float]
    fInputMean: list[ctypes.c_float]
    fInputStd: list[ctypes.c_float]
    fCurrentMean: list[ctypes.c_float]
    fCurrentStd: list[ctypes.c_float]
    fReleaseAttack: list[ctypes.c_float]
    fShortTermRelease: list[ctypes.c_float]
    fCurrentRelease: list[ctypes.c_float]
    fNextPeakRelease: list[ctypes.c_float]
    fGainPeak: list[ctypes.c_float]
    fInputPeak: list[ctypes.c_float]
    fOutputPeak: list[ctypes.c_float]
    histBuffer: int
    savedGains: list[ctypes.c_ulonglong]

class MasteringSuiteFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
        ("GetBufferTick", ctypes.c_ulonglong),
        ("CreateChannelManager", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    Execute: int
    GetBufferTick: int
    CreateChannelManager: int

class ChannelPair(ctypes.Structure):
    _fields_ = [
        ("first", ctypes.c_uint32),
        ("second", ctypes.c_uint32),
    ]
    first: int
    second: int

class hkDebugDisplayHandlerUtils(ctypes.Structure):
    pass

class hkDisplayCylinder_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class SceAudioMasterParamEqCoeff(ctypes.Structure):
    _fields_ = [
        ("fA0", ctypes.c_float),
        ("fA1", ctypes.c_float),
        ("fA2", ctypes.c_float),
        ("fB1", ctypes.c_float),
        ("fB2", ctypes.c_float),
    ]
    fA0: float
    fA1: float
    fA2: float
    fB1: float
    fB2: float

class SceAudioMasterSystemModulePresetDef(ctypes.Structure):
    _fields_ = [
        ("moduleId", ctypes.c_int32),
        ("uBypassFlags", ctypes.c_uint32),
        ("pModulePresetName", ctypes.c_char_p),
    ]
    moduleId: int
    uBypassFlags: int
    pModulePresetName: bytes

class SceAudioMasterSystemPresetDef(ctypes.Structure):
    _fields_ = [
        ("uNumPresets", ctypes.c_uint32),
        ("pPresets", ctypes.c_ulonglong),
    ]
    uNumPresets: int
    pPresets: int

class SceAudioMasterModuleParams(ctypes.Structure):
    _fields_ = [
        ("moduleId", ctypes.c_int32),
        ("paramsDes", ctypes.c_ulonglong),
    ]
    moduleId: int
    paramsDes: int

class SceAudioMasterParams(ctypes.Structure):
    _fields_ = [
        ("uNumConfigBlocks", ctypes.c_uint32),
        ("modulesConfig", ctypes.c_ulonglong),
    ]
    uNumConfigBlocks: int
    modulesConfig: int

class SceAudioMasterInitParams(ctypes.Structure):
    _fields_ = [
        ("uGranularity", ctypes.c_uint32),
        ("uSampleRate", ctypes.c_uint32),
        ("uMaxChannels", ctypes.c_uint32),
        ("uChannels", ctypes.c_uint32),
        ("channelStride", ctypes.c_uint32),
        ("akChannelMask", ctypes.c_uint32),
        ("bSaveModuleGains", ctypes.c_ubyte),
    ]
    uGranularity: int
    uSampleRate: int
    uMaxChannels: int
    uChannels: int
    channelStride: int
    akChannelMask: int
    bSaveModuleGains: bool

class hkDisplayMesh_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class NGX_API_VULKAN_Lib(ctypes.Structure):
    _fields_ = [
        ("Init", ctypes.c_ulonglong),
        ("Init_Ext", ctypes.c_ulonglong),
        ("Init_ProjectID", ctypes.c_ulonglong),
        ("Shutdown", ctypes.c_ulonglong),
        ("Shutdown1", ctypes.c_ulonglong),
        ("GetScratchBufferSize", ctypes.c_ulonglong),
        ("GetParameters", ctypes.c_ulonglong),
        ("CreateFeature", ctypes.c_ulonglong),
        ("CreateFeature1", ctypes.c_ulonglong),
        ("EvaluateFeature", ctypes.c_ulonglong),
        ("ReleaseFeature", ctypes.c_ulonglong),
        ("AllocateParameters", ctypes.c_ulonglong),
        ("GetCapabilityParameters", ctypes.c_ulonglong),
        ("DestroyParameters", ctypes.c_ulonglong),
    ]
    Init: int
    Init_Ext: int
    Init_ProjectID: int
    Shutdown: int
    Shutdown1: int
    GetScratchBufferSize: int
    GetParameters: int
    CreateFeature: int
    CreateFeature1: int
    EvaluateFeature: int
    ReleaseFeature: int
    AllocateParameters: int
    GetCapabilityParameters: int
    DestroyParameters: int

class _SP_DEVICE_INTERFACE_DETAIL_DATA_W(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("DevicePath", ctypes.c_wchar * 0x1),  # wchar_t[1]
    ]
    cbSize: int
    DevicePath: list[ctypes.c_wchar]

class _SP_ALTPLATFORM_INFO_V2(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("Platform", ctypes.c_uint32),
        ("MajorVersion", ctypes.c_uint32),
        ("MinorVersion", ctypes.c_uint32),
        ("ProcessorArchitecture", ctypes.c_uint16),
        ("___u5", ctypes.c_char * 0x2),  # $96350BF1982DDCCFA882FD13653D7E55
        ("FirstValidatedMajorVersion", ctypes.c_uint32),
        ("FirstValidatedMinorVersion", ctypes.c_uint32),
    ]
    cbSize: int
    Platform: int
    MajorVersion: int
    MinorVersion: int
    ProcessorArchitecture: int
    ___u5: bytes
    FirstValidatedMajorVersion: int
    FirstValidatedMinorVersion: int

class _CRYPTOAPI_BLOB(ctypes.Structure):
    _fields_ = [
        ("cbData", ctypes.c_uint32),
        ("pbData", ctypes.c_ulonglong),
    ]
    cbData: int
    pbData: int

class _CRYPT_BIT_BLOB(ctypes.Structure):
    _fields_ = [
        ("cbData", ctypes.c_uint32),
        ("pbData", ctypes.c_ulonglong),
        ("cUnusedBits", ctypes.c_uint32),
    ]
    cbData: int
    pbData: int
    cUnusedBits: int

class _CERT_CONTEXT(ctypes.Structure):
    _fields_ = [
        ("dwCertEncodingType", ctypes.c_uint32),
        ("pbCertEncoded", ctypes.c_ulonglong),
        ("cbCertEncoded", ctypes.c_uint32),
        ("pCertInfo", ctypes.c_ulonglong),
        ("hCertStore", ctypes.c_void_p),
    ]
    dwCertEncodingType: int
    pbCertEncoded: int
    cbCertEncoded: int
    pCertInfo: int
    hCertStore: int

class _CRYPT_DECODE_PARA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("pfnAlloc", ctypes.c_ulonglong),
        ("pfnFree", ctypes.c_ulonglong),
    ]
    cbSize: int
    pfnAlloc: int
    pfnFree: int

class _SHELLEXECUTEINFOA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fMask", ctypes.c_uint32),
        ("hwnd", ctypes.c_ulonglong),
        ("lpVerb", ctypes.c_char_p),
        ("lpFile", ctypes.c_char_p),
        ("lpParameters", ctypes.c_char_p),
        ("lpDirectory", ctypes.c_char_p),
        ("nShow", ctypes.c_int32),
        ("hInstApp", ctypes.c_ulonglong),
        ("lpIDList", ctypes.c_void_p),
        ("lpClass", ctypes.c_char_p),
        ("hkeyClass", ctypes.c_ulonglong),
        ("dwHotKey", ctypes.c_uint32),
        ("___u13", ctypes.c_char * 0x8),  # $E5EC784184F61E2D1DB730C15D65FB8F
        ("hProcess", ctypes.c_void_p),
    ]
    cbSize: int
    fMask: int
    hwnd: int
    lpVerb: bytes
    lpFile: bytes
    lpParameters: bytes
    lpDirectory: bytes
    nShow: int
    hInstApp: int
    lpIDList: int
    lpClass: bytes
    hkeyClass: int
    dwHotKey: int
    ___u13: bytes
    hProcess: int

class _SHELLEXECUTEINFOW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("fMask", ctypes.c_uint32),
        ("hwnd", ctypes.c_ulonglong),
        ("lpVerb", ctypes.c_ulonglong),
        ("lpFile", ctypes.c_ulonglong),
        ("lpParameters", ctypes.c_ulonglong),
        ("lpDirectory", ctypes.c_ulonglong),
        ("nShow", ctypes.c_int32),
        ("hInstApp", ctypes.c_ulonglong),
        ("lpIDList", ctypes.c_void_p),
        ("lpClass", ctypes.c_ulonglong),
        ("hkeyClass", ctypes.c_ulonglong),
        ("dwHotKey", ctypes.c_uint32),
        ("___u13", ctypes.c_char * 0x8),  # $E5EC784184F61E2D1DB730C15D65FB8F
        ("hProcess", ctypes.c_void_p),
    ]
    cbSize: int
    fMask: int
    hwnd: int
    lpVerb: int
    lpFile: int
    lpParameters: int
    lpDirectory: int
    nShow: int
    hInstApp: int
    lpIDList: int
    lpClass: int
    hkeyClass: int
    dwHotKey: int
    ___u13: bytes
    hProcess: int

class _CTL_USAGE(ctypes.Structure):
    _fields_ = [
        ("cUsageIdentifier", ctypes.c_uint32),
        ("rgpszUsageIdentifier", ctypes.c_ulonglong),
    ]
    cUsageIdentifier: int
    rgpszUsageIdentifier: int

class _CRYPT_ATTRIBUTE(ctypes.Structure):
    _fields_ = [
        ("pszObjId", ctypes.c_char_p),
        ("cValue", ctypes.c_uint32),
        ("rgValue", ctypes.c_ulonglong),
    ]
    pszObjId: bytes
    cValue: int
    rgValue: int

class _CERT_STRONG_SIGN_SERIALIZED_INFO(ctypes.Structure):
    _fields_ = [
        ("dwFlags", ctypes.c_uint32),
        ("pwszCNGSignHashAlgids", ctypes.c_ulonglong),
        ("pwszCNGPubKeyMinBitLengths", ctypes.c_ulonglong),
    ]
    dwFlags: int
    pwszCNGSignHashAlgids: int
    pwszCNGPubKeyMinBitLengths: int

class _CERT_STRONG_SIGN_PARA(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("dwInfoChoice", ctypes.c_uint32),
        ("___u2", ctypes.c_char * 0x8),  # $5D12A7ADFBC12634E66F5F3FB92EC100
    ]
    cbSize: int
    dwInfoChoice: int
    ___u2: bytes

class _CTL_CONTEXT(ctypes.Structure):
    _fields_ = [
        ("dwMsgAndCertEncodingType", ctypes.c_uint32),
        ("pbCtlEncoded", ctypes.c_ulonglong),
        ("cbCtlEncoded", ctypes.c_uint32),
        ("pCtlInfo", ctypes.c_ulonglong),
        ("hCertStore", ctypes.c_void_p),
        ("hCryptMsg", ctypes.c_void_p),
        ("pbCtlContent", ctypes.c_ulonglong),
        ("cbCtlContent", ctypes.c_uint32),
    ]
    dwMsgAndCertEncodingType: int
    pbCtlEncoded: int
    cbCtlEncoded: int
    pCtlInfo: int
    hCertStore: int
    hCryptMsg: int
    pbCtlContent: int
    cbCtlContent: int

class _D3DKMT_ENUMADAPTERS2(ctypes.Structure):
    _fields_ = [
        ("NumAdapters", ctypes.c_uint32),
        ("pAdapters", ctypes.c_ulonglong),
    ]
    NumAdapters: int
    pAdapters: int

class _D3DDDI_QUERYREGISTRY_FLAGS(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x4),  # $A413D25053E65B7EB6496E2F176A9881
    ]
    ___u0: bytes

class _D3DKMT_CLOSEADAPTER(ctypes.Structure):
    _fields_ = [
        ("hAdapter", ctypes.c_uint32),
    ]
    hAdapter: int

class WINTRUST_FILE_INFO_(ctypes.Structure):
    _fields_ = [
        ("cbStruct", ctypes.c_uint32),
        ("pcwszFilePath", ctypes.c_ulonglong),
        ("hFile", ctypes.c_void_p),
        ("pgKnownSubject", ctypes.c_ulonglong),
    ]
    cbStruct: int
    pcwszFilePath: int
    hFile: int
    pgKnownSubject: int

class WINTRUST_CATALOG_INFO_(ctypes.Structure):
    _fields_ = [
        ("cbStruct", ctypes.c_uint32),
        ("dwCatalogVersion", ctypes.c_uint32),
        ("pcwszCatalogFilePath", ctypes.c_ulonglong),
        ("pcwszMemberTag", ctypes.c_ulonglong),
        ("pcwszMemberFilePath", ctypes.c_ulonglong),
        ("hMemberFile", ctypes.c_void_p),
        ("pbCalculatedFileHash", ctypes.c_ulonglong),
        ("cbCalculatedFileHash", ctypes.c_uint32),
        ("pcCatalogContext", ctypes.c_ulonglong),
        ("hCatAdmin", ctypes.c_void_p),
    ]
    cbStruct: int
    dwCatalogVersion: int
    pcwszCatalogFilePath: int
    pcwszMemberTag: int
    pcwszMemberFilePath: int
    hMemberFile: int
    pbCalculatedFileHash: int
    cbCalculatedFileHash: int
    pcCatalogContext: int
    hCatAdmin: int

class _CRYPT_ATTRIBUTES(ctypes.Structure):
    _fields_ = [
        ("cAttr", ctypes.c_uint32),
        ("rgAttr", ctypes.c_ulonglong),
    ]
    cAttr: int
    rgAttr: int

class WINTRUST_SGNR_INFO_(ctypes.Structure):
    _fields_ = [
        ("cbStruct", ctypes.c_uint32),
        ("pcwszDisplayName", ctypes.c_ulonglong),
        ("psSignerInfo", ctypes.c_ulonglong),
        ("chStores", ctypes.c_uint32),
        ("pahStores", ctypes.c_ulonglong),
    ]
    cbStruct: int
    pcwszDisplayName: int
    psSignerInfo: int
    chStores: int
    pahStores: int

class WINTRUST_CERT_INFO_(ctypes.Structure):
    _fields_ = [
        ("cbStruct", ctypes.c_uint32),
        ("pcwszDisplayName", ctypes.c_ulonglong),
        ("psCertContext", ctypes.c_ulonglong),
        ("chStores", ctypes.c_uint32),
        ("pahStores", ctypes.c_ulonglong),
        ("dwFlags", ctypes.c_uint32),
        ("psftVerifyAsOf", ctypes.c_ulonglong),
    ]
    cbStruct: int
    pcwszDisplayName: int
    psCertContext: int
    chStores: int
    pahStores: int
    dwFlags: int
    psftVerifyAsOf: int

class WINTRUST_SIGNATURE_SETTINGS_(ctypes.Structure):
    _fields_ = [
        ("cbStruct", ctypes.c_uint32),
        ("dwIndex", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("cSecondarySigs", ctypes.c_uint32),
        ("dwVerifiedSigIndex", ctypes.c_uint32),
        ("pCryptoPolicy", ctypes.c_ulonglong),
    ]
    cbStruct: int
    dwIndex: int
    dwFlags: int
    cSecondarySigs: int
    dwVerifiedSigIndex: int
    pCryptoPolicy: int

class _WINTRUST_DATA(ctypes.Structure):
    _fields_ = [
        ("cbStruct", ctypes.c_uint32),
        ("pPolicyCallbackData", ctypes.c_void_p),
        ("pSIPClientData", ctypes.c_void_p),
        ("dwUIChoice", ctypes.c_uint32),
        ("fdwRevocationChecks", ctypes.c_uint32),
        ("dwUnionChoice", ctypes.c_uint32),
        ("___u6", ctypes.c_char * 0x8),  # $D238723FCA61EE6E4D1A6F61F5CA1926
        ("dwStateAction", ctypes.c_uint32),
        ("hWVTStateData", ctypes.c_void_p),
        ("pwszURLReference", ctypes.c_ulonglong),
        ("dwProvFlags", ctypes.c_uint32),
        ("dwUIContext", ctypes.c_uint32),
        ("pSignatureSettings", ctypes.c_ulonglong),
    ]
    cbStruct: int
    pPolicyCallbackData: int
    pSIPClientData: int
    dwUIChoice: int
    fdwRevocationChecks: int
    dwUnionChoice: int
    ___u6: bytes
    dwStateAction: int
    hWVTStateData: int
    pwszURLReference: int
    dwProvFlags: int
    dwUIContext: int
    pSignatureSettings: int

class _EVENT_DATA_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("Ptr", ctypes.c_uint64),
        ("Size", ctypes.c_uint32),
        ("___u2", ctypes.c_char * 0x4),  # $CDE2F7BAC2EB829C48A011809A51E076
    ]
    Ptr: int
    Size: int
    ___u2: bytes

class _EVENT_DESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("Id", ctypes.c_uint16),
        ("Version", ctypes.c_uint8),
        ("Channel", ctypes.c_uint8),
        ("Level", ctypes.c_uint8),
        ("Opcode", ctypes.c_uint8),
        ("Task", ctypes.c_uint16),
        ("Keyword", ctypes.c_uint64),
    ]
    Id: int
    Version: int
    Channel: int
    Level: int
    Opcode: int
    Task: int
    Keyword: int

class __crt_fast_encoded_nullptr_t(ctypes.Structure):
    pass

class __scrt_no_argv_policy(ctypes.Structure):
    pass

class __scrt_no_environment_policy(ctypes.Structure):
    pass

class __scrt_wide_argv_policy(ctypes.Structure):
    pass

class __scrt_narrow_argv_policy(ctypes.Structure):
    pass

class __scrt_narrow_environment_policy(ctypes.Structure):
    pass

class NGXProgressCallbackHelper(ctypes.Structure):
    def s_ReplacementCallback(): pass


class __scrt_wide_environment_policy(ctypes.Structure):
    pass

class _UNWIND_CODE: pass

class _GS_HANDLER_DATA(ctypes.Structure):
    _fields_ = [
        ("u", ctypes.c_char * 0x4),  # _GS_HANDLER_DATA::<unnamed_type_u>
        ("AlignedBaseOffset", ctypes.c_int32),
        ("Alignment", ctypes.c_int32),
    ]
    u: bytes
    AlignedBaseOffset: int
    Alignment: int

class _Whitespace_bitmap_t(ctypes.Structure):
    _fields_ = [
        ("_Is_whitespace", ctypes.c_ubyte * 0x100),  # bool[256]
    ]
    _Is_whitespace: list[ctypes.c_ubyte]

class FT: pass

class _DEVICE_DATA_SET_RANGE(ctypes.Structure):
    _fields_ = [
        ("StartingOffset", ctypes.c_int64),
        ("LengthInBytes", ctypes.c_uint64),
    ]
    StartingOffset: int
    LengthInBytes: int

class _DEVICE_DSM_DEFINITION(ctypes.Structure):
    _fields_ = [
        ("Action", ctypes.c_uint32),
        ("SingleRange", ctypes.c_uint8),
        ("ParameterBlockAlignment", ctypes.c_uint32),
        ("ParameterBlockLength", ctypes.c_uint32),
        ("HasOutput", ctypes.c_uint8),
        ("OutputBlockAlignment", ctypes.c_uint32),
        ("OutputBlockLength", ctypes.c_uint32),
    ]
    Action: int
    SingleRange: int
    ParameterBlockAlignment: int
    ParameterBlockLength: int
    HasOutput: int
    OutputBlockAlignment: int
    OutputBlockLength: int

class __crt_internal_free_policy(ctypes.Structure):
    pass

class CsFrame(ctypes.Structure):
    _fields_ = [
        ("SavedR10", ctypes.c_uint64),
        ("SavedR11", ctypes.c_uint64),
    ]
    SavedR10: int
    SavedR11: int

class _DEVICE_MANAGE_DATA_SET_ATTRIBUTES(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("Action", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("ParameterBlockOffset", ctypes.c_uint32),
        ("ParameterBlockLength", ctypes.c_uint32),
        ("DataSetRangesOffset", ctypes.c_uint32),
        ("DataSetRangesLength", ctypes.c_uint32),
    ]
    Size: int
    Action: int
    Flags: int
    ParameterBlockOffset: int
    ParameterBlockLength: int
    DataSetRangesOffset: int
    DataSetRangesLength: int

class _DEVICE_MANAGE_DATA_SET_ATTRIBUTES_OUTPUT(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint32),
        ("Action", ctypes.c_uint32),
        ("Flags", ctypes.c_uint32),
        ("OperationStatus", ctypes.c_uint32),
        ("ExtendedError", ctypes.c_uint32),
        ("TargetDetailedError", ctypes.c_uint32),
        ("ReservedStatus", ctypes.c_uint32),
        ("OutputBlockOffset", ctypes.c_uint32),
        ("OutputBlockLength", ctypes.c_uint32),
    ]
    Size: int
    Action: int
    Flags: int
    OperationStatus: int
    ExtendedError: int
    TargetDetailedError: int
    ReservedStatus: int
    OutputBlockOffset: int
    OutputBlockLength: int

class AttrDeclParams(ctypes.Structure):
    _fields_ = [
        ("name", ctypes.c_char_p),
        ("ownerType", ctypes.c_ulonglong),
        ("flags", ctypes.c_char * 0x1),  # hkFlags<enum hkAttrData::FieldDecl::FlagBits,unsigned char>
        ("attrData", ctypes.c_ulonglong),
    ]
    name: bytes
    ownerType: int
    flags: bytes
    attrData: int

class AttrDeclVisitor(ctypes.Structure):
    pass

class AllocSizeVisitor(ctypes.Structure):
    _fields_ = [
        ("m_ownerType", ctypes.c_ulonglong),
    ]
    m_ownerType: int

class SetterRecordVisitor(ctypes.Structure):
    _fields_ = [
        ("thisPtr", ctypes.c_ulonglong),
    ]
    thisPtr: int

class SetterArrayVisitor(ctypes.Structure):
    _fields_ = [
        ("thisVar", ctypes.c_ulonglong),
        ("begin", ctypes.c_void_p),
    ]
    thisVar: int
    begin: int

class hknpSimdTreeBroadPhase_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpBroadPhase_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
        ("getConfig", ctypes.c_ulonglong),
        ("setBodyBuffer", ctypes.c_ulonglong),
        ("translate", ctypes.c_ulonglong),
        ("addBodies", ctypes.c_ulonglong),
        ("removeBodies", ctypes.c_ulonglong),
        ("markBodiesDirty", ctypes.c_ulonglong),
        ("update", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, hkFlags<enum hknpBroadPhaseFilter::Category,unsigned char>)
        ("updateBodyAabbs", ctypes.c_ulonglong),
        ("buildTaskGraph", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, hknpWorld *, hknpSimulationContext *, hkFlags<enum hknpBroadPhaseFilter::Category,unsigned char>, hkTypedBlockStream<hknpBodyIndexPair> *, hknpTaskGraph *)
        ("getExtents", ctypes.c_ulonglong),
        ("findAllPairs", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpSimulationThreadContext *, hkTypedBlockStream<hknpBodyIndexPair>::Writer *)
        ("findNewPairs", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpSimulationThreadContext *, const hkAabbFloat3 *, hkTypedBlockStream<hknpBodyIndexPair>::Writer *)
        ("queryAabb", ctypes.c_ulonglong),
        ("queryAabb_78", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabb_80", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryOutsideOfAabb", ctypes.c_ulonglong),
        ("queryOutsideOfAabb_90", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp", ctypes.c_ulonglong),
        ("queryAabbNmp_a0", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyIndex,hkBuiltinContainerAllocator<0,0> > *)
        ("queryAabbNmp_a8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbQuery *, const hkAabb *, hkAabb *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castAabb", ctypes.c_ulonglong),
        ("castAabb_b8", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpAabbCastQuery *, hkArray<hknpBodyId,hkBuiltinContainerAllocator<0,0> > *)
        ("castRay", ctypes.c_ulonglong),
        ("castShape", ctypes.c_char * 0x8),  # void (__fastcall *)(hknpBroadPhase *this, const hknpShapeCastQuery *, const hkRotationImpl<float> *, const hknpCollisionQueryDispatcher *, hknpCollisionQueryCollector *, hknpCollisionQueryCollector *, hknpCollisionQueryContext *)
        ("queryPoint", ctypes.c_ulonglong),
        ("getClosestPoints", ctypes.c_ulonglong),
        ("getClosestPointsAdvanced", ctypes.c_ulonglong),
        ("getClosestPointsParticles", ctypes.c_ulonglong),
        ("copyFrom", ctypes.c_ulonglong),
    ]
    hknpBroadPhase_dtor_0: int
    getType: int
    getConfig: int
    setBodyBuffer: int
    translate: int
    addBodies: int
    removeBodies: int
    markBodiesDirty: int
    update: bytes
    updateBodyAabbs: int
    buildTaskGraph: bytes
    getExtents: int
    findAllPairs: bytes
    findNewPairs: bytes
    queryAabb: int
    queryAabb_78: bytes
    queryAabb_80: bytes
    queryOutsideOfAabb: int
    queryOutsideOfAabb_90: bytes
    queryAabbNmp: int
    queryAabbNmp_a0: bytes
    queryAabbNmp_a8: bytes
    castAabb: int
    castAabb_b8: bytes
    castRay: int
    castShape: bytes
    queryPoint: int
    getClosestPoints: int
    getClosestPointsAdvanced: int
    getClosestPointsParticles: int
    copyFrom: int

class CheckPadBase(ctypes.Structure):
    _fields_ = [
        ("i", ctypes.c_int32),
        ("c", ctypes.c_char),
    ]
    i: int
    c: str

class ClassAlign_Base(ctypes.Structure):
    _fields_ = [
        ("ibase", ctypes.c_int32),
    ]
    ibase: int

class FieldAlign_Base(ctypes.Structure):
    _fields_ = [
        ("fbase", ctypes.c_int16),
    ]
    fbase: int

class KnownDetectFormatEntries(ctypes.Structure):
    _fields_ = [
        ("m_entry", ctypes.c_ulonglong),
        ("_m_formatType", ctypes.c_uint32),
    ]
    m_entry: int
    _m_formatType: int

    @property
    def m_formatType(self):
        return hkSerializeUtil.FormatType(self._m_formatType)


class hkDisplayTaperedCapsule_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("buildGeometry", ctypes.c_ulonglong),
        ("getWireframeGeometry", ctypes.c_char * 0x8),  # void (__fastcall *)(hkDisplayGeometry *this, hkArrayBase<hkVector4f> *, const hkContainerAllocator *)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    buildGeometry: int
    getWireframeGeometry: bytes

class SceAudioMasterFilterTransferCoEff(ctypes.Structure):
    _fields_ = [
        ("fB0", ctypes.c_float),
        ("fB1", ctypes.c_float),
        ("fB2", ctypes.c_float),
        ("fA0", ctypes.c_float),
        ("fA1", ctypes.c_float),
        ("fA2", ctypes.c_float),
    ]
    fB0: float
    fB1: float
    fB2: float
    fA0: float
    fA1: float
    fA2: float

class NvDRSSessionHandle__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class NvDRSProfileHandle__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class __scrt_winmain_policy(ctypes.Structure):
    pass

class __scrt_nofile_policy(ctypes.Structure):
    pass

class __scrt_file_policy(ctypes.Structure):
    pass

class __scrt_main_policy(ctypes.Structure):
    pass

class __scrt_enclavemain_policy(ctypes.Structure):
    pass

class CastGuardVftables(ctypes.Structure):
    _fields_ = [
        ("padding", ctypes.c_char * 0x80),  # char[128]
    ]
    padding: list[ctypes.c_char]

class hkAlgorithm_Morton_sort(ctypes.Structure):
    pass

class hkAlgorithm_Morton_split(ctypes.Structure):
    pass

class hkAlgorithm_Morton_splitSimple(ctypes.Structure):
    pass

class cTkHilbertMortonHelper(ctypes.Structure):
    def cTkHilbertMortonHelper(): pass

    _fields_ = [
        ("mu0x09249249", common.Vector4i),
        ("mu0x030c30c3", common.Vector4i),
        ("mu0x0300f00f", common.Vector4i),
        ("mu0x030000ff", common.Vector4i),
        ("mu0x000003ff", common.Vector4i),
        ("kppauHilbertLookup", ctypes.c_ulonglong * 0x7),  # unsigned int *[7]
        ("mxUsedBits", ctypes.c_char * 0x4),  # cTkBitArray<unsigned int,1,32>
    ]
    mu0x09249249: common.Vector4i
    mu0x030c30c3: common.Vector4i
    mu0x0300f00f: common.Vector4i
    mu0x030000ff: common.Vector4i
    mu0x000003ff: common.Vector4i
    kppauHilbertLookup: list[ctypes.c_ulonglong]
    mxUsedBits: bytes

class vec3(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_float),
        ("y", ctypes.c_float),
        ("z", ctypes.c_float),
    ]
    x: float
    y: float
    z: float

class cTkPolygoniserHeightmap(ctypes.Structure):
    def GetMaterialsForTriangle(): pass

    def Polygonise(): pass


class cTkPolygoniserMarchingCubes(ctypes.Structure):
    def MarchCube(): pass

    def Polygonise(): pass

    def vGetNormal(): pass


class CAkFXSrcAudioInput_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("GetEnvelope", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    GetDuration: int
    GetEnvelope: int
    StopLooping: int
    Seek: int
    TimeSkip: int
    Execute: int

class AudioInputStream(ctypes.Structure):
    _fields_ = [
        ("m_pData", ctypes.c_ulonglong),
        ("m_uDataSize", ctypes.c_uint32),
        ("m_uSampleRate", ctypes.c_uint32),
    ]
    m_pData: int
    m_uDataSize: int
    m_uSampleRate: int

class MeterKey(ctypes.Structure):
    _fields_ = [
        ("uGameParamID", ctypes.c_uint32),
        ("uGameObjID", ctypes.c_uint64),
        ("uNodeID", ctypes.c_uint32),
    ]
    uGameParamID: int
    uGameObjID: int
    uNodeID: int

class AkFXDurationHandler(ctypes.Structure):
    _fields_ = [
        ("m_uIterationFrame", ctypes.c_uint32),
        ("m_uFrameCount", ctypes.c_uint32),
        ("m_uSampleRate", ctypes.c_uint32),
        ("m_iNumLoops", ctypes.c_int16),
    ]
    m_uIterationFrame: int
    m_uFrameCount: int
    m_uSampleRate: int
    m_iNumLoops: int

class CAkFXSrcSine_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkPlugin_dtor_0", ctypes.c_ulonglong),
        ("Term", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetPluginInfo", ctypes.c_ulonglong),
        ("SupportMediaRelocation", ctypes.c_ulonglong),
        ("RelocateMedia", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("GetDuration", ctypes.c_ulonglong),
        ("GetEnvelope", ctypes.c_ulonglong),
        ("StopLooping", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("TimeSkip", ctypes.c_ulonglong),
        ("Execute", ctypes.c_ulonglong),
    ]
    IAkPlugin_dtor_0: int
    Term: int
    Reset: int
    GetPluginInfo: int
    SupportMediaRelocation: int
    RelocateMedia: int
    Init: int
    GetDuration: int
    GetEnvelope: int
    StopLooping: int
    Seek: int
    TimeSkip: int
    Execute: int

class AkParamTargetList(ctypes.Structure):
    def Add(): pass

    def Remove(): pass

    _fields_ = [
        ("m_RootNodes", ctypes.c_char * 0x18),  # AkHashList<unsigned int,AkListBare<CAkParameterTarget,AkListBareLightNextItem,AkCountPolicyNoCount,AkLastPolicyNoLast>,AkArrayAllocatorNoAlign<0> >
    ]
    m_RootNodes: bytes

class Matrix2x2(ctypes.Structure):
    _fields_ = [
        ("a", ctypes.c_float),
        ("b", ctypes.c_float),
        ("c", ctypes.c_float),
        ("d", ctypes.c_float),
    ]
    a: float
    b: float
    c: float
    d: float

class GridToSpeakerGain(ctypes.Structure):
    _fields_ = [
        ("sourceIdx", ctypes.c_uint32),
        ("targetIdx", ctypes.c_uint32),
        ("gain", ctypes.c_float),
    ]
    sourceIdx: int
    targetIdx: int
    gain: float

class ContractInside(ctypes.Structure):
    _fields_ = [
        ("spread", common.Vector4f),
        ("spreadBias", common.Vector4f),
    ]
    spread: common.Vector4f
    spreadBias: common.Vector4f

class ContractInsideScalar(ctypes.Structure):
    _fields_ = [
        ("spread", ctypes.c_float),
        ("spreadBias", ctypes.c_float),
    ]
    spread: float
    spreadBias: float

class ContractOutside(ctypes.Structure):
    _fields_ = [
        ("dddd", common.Vector4f),
    ]
    dddd: common.Vector4f

class ContractOutsideScalar(ctypes.Structure):
    _fields_ = [
        ("d", ctypes.c_float),
    ]
    d: float

class MonoPolicy(ctypes.Structure):
    pass

class StereoPolicy(ctypes.Structure):
    pass

class LeafByte(ctypes.Structure):
    pass

class LeafWord(ctypes.Structure):
    pass

class ExpandEntry1Stereo(ctypes.Structure):
    pass

class CodebookBuildParam(ctypes.Structure):
    _fields_ = [
        ("q_min", ctypes.c_int32),
        ("q_minp", ctypes.c_int32),
        ("entries", ctypes.c_uint16),
        ("used_entries", ctypes.c_uint16),
        ("q_delp", ctypes.c_int32),
        ("q_pack", ctypes.c_int32),
        ("q_val", ctypes.c_void_p),
        ("maptype", ctypes.c_int32),
        ("dec_type", ctypes.c_int32),
        ("dec_nodeb", ctypes.c_int32),
    ]
    q_min: int
    q_minp: int
    entries: int
    used_entries: int
    q_delp: int
    q_pack: int
    q_val: int
    maptype: int
    dec_type: int
    dec_nodeb: int

class _D3DCOLORVALUE(ctypes.Structure):
    _fields_ = [
        ("r", ctypes.c_float),
        ("g", ctypes.c_float),
        ("b", ctypes.c_float),
        ("a", ctypes.c_float),
    ]
    r: float
    g: float
    b: float
    a: float

class StringNull(ctypes.Structure):
    _fields_ = [
        ("m_s", ctypes.c_char_p),
    ]
    m_s: bytes

class hkStatisticsProcess_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hkDebugDisplayMarker_Type_Auto(ctypes.Structure):
    pass

class _Find_traits_2(ctypes.Structure):
    pass

class _Find_traits_4(ctypes.Structure):
    pass

class _Minmax_traits_1(ctypes.Structure):
    pass

class _Find_traits_8(ctypes.Structure):
    pass

class _Minmax_traits_4(ctypes.Structure):
    pass

class _Minmax_traits_2(ctypes.Structure):
    pass

class _Minmax_traits_8(ctypes.Structure):
    pass

class _Find_traits_1(ctypes.Structure):
    pass

class _s_ThrowInfo(ctypes.Structure):
    _fields_ = [
        ("attributes", ctypes.c_uint32),
        ("pmfnUnwind", ctypes.c_int32),
        ("pForwardCompat", ctypes.c_int32),
        ("pCatchableTypeArray", ctypes.c_int32),
    ]
    attributes: int
    pmfnUnwind: int
    pForwardCompat: int
    pCatchableTypeArray: int

class ReflectStrings_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class LessByTypeName(ctypes.Structure):
    pass

class hkcdInterval4(ctypes.Structure):
    _fields_ = [
        ("m_min", common.Vector4f),
        ("m_max", common.Vector4f),
    ]
    m_min: common.Vector4f
    m_max: common.Vector4f

class ComponentInstancercGcPlayerCharacterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcOutpostComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcOutpostComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcEncounterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcVehicleComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcVehicleComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcVehicleCheckpointComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcUniqueIdComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcVehicleCheckpointComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcBaseMiniPortalComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcByteBeatPlayerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcInteractionComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcInteractionComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcAtlasTransportDebug_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasTransport_dtor_0", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdateThreadStart", ctypes.c_ulonglong),
        ("UpdateThreadEnd", ctypes.c_ulonglong),
        ("GetNetworkState", ctypes.c_ulonglong),
        ("IsTrafficInFlight", ctypes.c_ulonglong),
        ("RequestHttpAsync", ctypes.c_char * 0x8),  # cGcAtlasTransport::RequestResult (__fastcall *)(cGcAtlasTransport *this, const Http::Request *, const std::function<void __cdecl(Http::RequestResult const &)> *)
    ]
    cGcAtlasTransport_dtor_0: int
    GetName: int
    Construct: int
    Destruct: int
    Update: int
    UpdateThreadStart: int
    UpdateThreadEnd: int
    GetNetworkState: int
    IsTrafficInFlight: int
    RequestHttpAsync: bytes

class ComponentInstancercGcNPCPlacementComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcLootComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcAtlasTransportWinHttp_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasTransport_dtor_0", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdateThreadStart", ctypes.c_ulonglong),
        ("UpdateThreadEnd", ctypes.c_ulonglong),
        ("GetNetworkState", ctypes.c_ulonglong),
        ("IsTrafficInFlight", ctypes.c_ulonglong),
        ("RequestHttpAsync", ctypes.c_char * 0x8),  # cGcAtlasTransport::RequestResult (__fastcall *)(cGcAtlasTransport *this, const Http::Request *, const std::function<void __cdecl(Http::RequestResult const &)> *)
    ]
    cGcAtlasTransport_dtor_0: int
    GetName: int
    Construct: int
    Destruct: int
    Update: int
    UpdateThreadStart: int
    UpdateThreadEnd: int
    GetNetworkState: int
    IsTrafficInFlight: int
    RequestHttpAsync: bytes

class cGcAtlasBrokerAuth_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkUserServiceNotifications_dtor_0", ctypes.c_ulonglong),
        ("AuthenticationTokenArrived", ctypes.c_ulonglong),
        ("AuthenticationTokenFailed", ctypes.c_ulonglong),
        ("ServiceStateChanged", ctypes.c_ulonglong),
    ]
    ITkUserServiceNotifications_dtor_0: int
    AuthenticationTokenArrived: int
    AuthenticationTokenFailed: int
    ServiceStateChanged: int

class WINHTTP_ASYNC_RESULT(ctypes.Structure):
    _fields_ = [
        ("dwResult", ctypes.c_uint64),
        ("dwError", ctypes.c_uint32),
    ]
    dwResult: int
    dwError: int

class WINHTTP_CURRENT_USER_IE_PROXY_CONFIG(ctypes.Structure):
    _fields_ = [
        ("fAutoDetect", ctypes.c_int32),
        ("lpszAutoConfigUrl", ctypes.c_ulonglong),
        ("lpszProxy", ctypes.c_ulonglong),
        ("lpszProxyBypass", ctypes.c_ulonglong),
    ]
    fAutoDetect: int
    lpszAutoConfigUrl: int
    lpszProxy: int
    lpszProxyBypass: int

class AzureTaskData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcIBrokerData_dtor_0", ctypes.c_ulonglong),
    ]
    cGcIBrokerData_dtor_0: int

class cGcAtlasBrokerAzure_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcAtlasBroker_dtor_0", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("RequiresTransport", ctypes.c_ulonglong),
        ("GetReadyStateCounter", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdateThreadStart", ctypes.c_ulonglong),
        ("UpdateThreadEnd", ctypes.c_ulonglong),
        ("Service", ctypes.c_ulonglong),
        ("Abandon", ctypes.c_ulonglong),
        ("DeleteTaskData", ctypes.c_ulonglong),
        ("GetMessagesEnabled", ctypes.c_ulonglong),
        ("GetOpenIdToken", ctypes.c_ulonglong),
        ("GetAtlasCommunityData", ctypes.c_ulonglong),
        ("GetAtlasBaseImagesRoute", ctypes.c_ulonglong),
        ("GetAtlasFeaturedBaseImagesRoute", ctypes.c_ulonglong),
        ("GetAtlasSeasonDataRoute", ctypes.c_ulonglong),
        ("DoCompletion", ctypes.c_ulonglong),
        ("DoCompletion_a0", ctypes.c_ulonglong),
        ("DoCompletion_a8", ctypes.c_ulonglong),
        ("DoCompletion_b0", ctypes.c_ulonglong),
        ("DoCompletion_b8", ctypes.c_ulonglong),
        ("DoCompletion_c0", ctypes.c_ulonglong),
        ("DoCompletion_c8", ctypes.c_ulonglong),
        ("DoCompletion_d0", ctypes.c_ulonglong),
        ("DoCompletion_d8", ctypes.c_ulonglong),
        ("DoCompletion_e0", ctypes.c_ulonglong),
        ("DoCompletion_e8", ctypes.c_ulonglong),
        ("DoCompletion_f0", ctypes.c_ulonglong),
        ("DoCompletion_f8", ctypes.c_ulonglong),
        ("DoCompletion_100", ctypes.c_ulonglong),
        ("DoCompletion_108", ctypes.c_ulonglong),
        ("DoCompletion_110", ctypes.c_ulonglong),
        ("DoCompletion_118", ctypes.c_ulonglong),
        ("DoCompletion_120", ctypes.c_ulonglong),
        ("DoCompletion_128", ctypes.c_ulonglong),
        ("DoCompletion_130", ctypes.c_ulonglong),
        ("DoCompletion_138", ctypes.c_ulonglong),
        ("DoCompletion_140", ctypes.c_ulonglong),
        ("DoCompletion_148", ctypes.c_ulonglong),
        ("DoCompletion_150", ctypes.c_ulonglong),
        ("DoCompletion_158", ctypes.c_ulonglong),
        ("DoCompletion_160", ctypes.c_ulonglong),
        ("DoCompletion_168", ctypes.c_ulonglong),
        ("DoCompletion_170", ctypes.c_ulonglong),
    ]
    cGcAtlasBroker_dtor_0: int
    Reset: int
    GetName: int
    RequiresTransport: int
    GetReadyStateCounter: int
    Construct: int
    Destruct: int
    Update: int
    UpdateThreadStart: int
    UpdateThreadEnd: int
    Service: int
    Abandon: int
    DeleteTaskData: int
    GetMessagesEnabled: int
    GetOpenIdToken: int
    GetAtlasCommunityData: int
    GetAtlasBaseImagesRoute: int
    GetAtlasFeaturedBaseImagesRoute: int
    GetAtlasSeasonDataRoute: int
    DoCompletion: int
    DoCompletion_a0: int
    DoCompletion_a8: int
    DoCompletion_b0: int
    DoCompletion_b8: int
    DoCompletion_c0: int
    DoCompletion_c8: int
    DoCompletion_d0: int
    DoCompletion_d8: int
    DoCompletion_e0: int
    DoCompletion_e8: int
    DoCompletion_f0: int
    DoCompletion_f8: int
    DoCompletion_100: int
    DoCompletion_108: int
    DoCompletion_110: int
    DoCompletion_118: int
    DoCompletion_120: int
    DoCompletion_128: int
    DoCompletion_130: int
    DoCompletion_138: int
    DoCompletion_140: int
    DoCompletion_148: int
    DoCompletion_150: int
    DoCompletion_158: int
    DoCompletion_160: int
    DoCompletion_168: int
    DoCompletion_170: int

class cGcVehicleRaceInviteComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcDebugRenderOffsetControl(ctypes.Structure):
    def ApplyCamCenterRenderOffset(): pass

    _fields_ = [
        ("mbSmallAdjustEveryFrame", ctypes.c_ubyte),
    ]
    mbSmallAdjustEveryFrame: bool

class cGcBaseMiniPortalComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcAbandonedFreighterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class ComponentInstancercGcSimpleInteractionComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcByteBeatPlayerAudioStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("ITkAudioStream_dtor_0", ctypes.c_ulonglong),
        ("GetAudioStreamFormat", ctypes.c_ulonglong),
        ("GetAudioStreamData", ctypes.c_ulonglong),
    ]
    ITkAudioStream_dtor_0: int
    GetAudioStreamFormat: int
    GetAudioStreamData: int

class ComponentInstancercGcRefinerUnitComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcByteBeatPlayerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class LZ4_stream_t(ctypes.Structure):
    _fields_ = [
        ("table", ctypes.c_int64 * 0x804),  # __int64[2052]
    ]
    table: list[ctypes.c_int64]

class LZ4_streamDecode_t(ctypes.Structure):
    _fields_ = [
        ("table", ctypes.c_uint64 * 0x4),  # unsigned __int64[4]
    ]
    table: list[ctypes.c_uint64]

class cGcCheckDeath_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int

class cGcDroneComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcDroneBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcGameStats(ctypes.Structure):
    pass

class cGcRenderNguiUi_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class ComponentInstancercGcFreighterBaseComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcExpeditionHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcExpeditionHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class DefaultDiscoveryDataHashing(ctypes.Structure):
    pass

class ComponentInstancercGcGeneratorUnitComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class SizeAlign(ctypes.Structure):
    _fields_ = [
        ("size", ctypes.c_uint64),
        ("align", ctypes.c_uint64),
    ]
    size: int
    align: int

class ComponentInstancercGcRepairTechComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcFrontendPageShop(ctypes.Structure):
    def DoTrade(): pass

    def DoShopInteraction(): pass

    def DoTechTrade(): pass

    def DoBuildingTrade(): pass

    def DoSpecialsTrade(): pass

    def DoShipPilotOptions(): pass

    def DoShopOptions(): pass

    def DoTechShopOptions(): pass

    def DoBuildingShopOptions(): pass

    def PickBuyString(): pass

    def DoShopShowItem(): pass

    def DoShopCompareItem(): pass

    def DoFreighterShipTransfer(): pass

    def DoSquadronRecruitment(): pass

    def DoInventoryTransfer(): pass

    def DoAcceptButton(): pass

    def DoPurchaseButton(): pass

    def DoDeclineButton(): pass

    def OnDeclineConfirmClosed(): pass

    def DoAddNewItemButton(): pass

    def PopulateScrollableItemInventories(): pass

    def DoRocketLockerTransfer(): pass

    def DoCompareScreenStatsWindow(): pass

    def DoFrigateStatsWindow(): pass

    def DoItemRender(): pass

    def DoInfoArray(): pass

    def ConvertToFullWidth(): pass

    def GeneratePlayerInventory(): pass

    def GetTradeSettings(): pass

    def GenerateSellerInventory(): pass

    def GeneratePlanetTechShopInventory(): pass

    def GenerateGeneralTechShopInventory(): pass

    def GenerateSpecialsShopInventory(): pass

    def GenerateSeasonRewardsInventory(): pass

    def GenerateTwitchRewardsInventory(): pass

    def GeneratePlatformRewardsInventory(): pass

    def GenerateShopElement(): pass

    def IsEverOfferedForSale(): pass

    def DoShopItem(): pass

    def PopulateShopItems(): pass

    def DoShopScrollBar(): pass

    def DoCommunityMissionProgress(): pass

    def DoBuy(): pass

    def PurchaseCurrentPurchasableItem(): pass

    def CheckTransferItemsDialog(): pass

    def CheckTransferBaseDialog(): pass

    def DoStorageContainerNameChange(): pass

    def DoSwitchSpecialsShopFilter(): pass

    def SetPendingReward(): pass


class cGcFrontendPageManageFleet(ctypes.Structure):
    def GetTimeAsString(): pass

    def OnFleetManagerDismissFrigateDialogClosed(): pass

    def OnFrigateDetailsDismissFrigateDialogClosed(): pass

    def DoManageFleet(): pass

    def DoExpeditionDetails(): pass

    def DoExpeditionSelection(): pass

    def DoPowerupSelection(): pass

    def DoExpeditionDebrief(): pass

    def DoFrigateDetails(): pass

    def DoFrigateCaptain(): pass

    def DoFrigateTraitDots(): pass

    def DoFrigateTraits(): pass

    def GetFeedAnimationProgress(): pass

    def DoExpeditionStatList(): pass

    def GetFrigateClassName(): pass

    def DoStatList(): pass

    def DoExpeditionFrigateCountBox(): pass

    def DoFleetListEntry(): pass

    def DoExpeditionListEntry(): pass

    def DoExpeditionStatus(): pass

    def DoFrigateClassBox(): pass

    def DoGridFlicker(): pass

    def HideFleetNameAndLocation(): pass

    def DoExpeditionTypeAndName(): pass

    def DoExpeditionTypeDetailsPage(): pass

    def DoExpeditionDifficulty(): pass

    def LeavePage(): pass

    def CountTextSpecialListElements(): pass

    def ResetFeedFrigateInventory(): pass


class cGcBaseTeleporterID(ctypes.Structure):
    pass

class sha256_ctx(ctypes.Structure):
    _fields_ = [
        ("tot_len", ctypes.c_uint32),
        ("len", ctypes.c_uint32),
        ("block", ctypes.c_uint8 * 0x80),  # unsigned __int8[128]
        ("h", ctypes.c_uint32 * 0x8),  # unsigned int[8]
    ]
    tot_len: int
    len: int
    block: list[ctypes.c_uint8]
    h: list[ctypes.c_uint32]

class cGcNotificationSequenceGeneric_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequence_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsAppropriate", ctypes.c_ulonglong),
        ("GetMissionDataTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetSubtitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<256,char> *)
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetMissionIcon", ctypes.c_ulonglong),
        ("GetMissionIconSelected", ctypes.c_ulonglong),
        ("GetMissionIconNotSelected", ctypes.c_ulonglong),
        ("ShowFutureObjectives", ctypes.c_ulonglong),
        ("IsPrimary", ctypes.c_ulonglong),
        ("IsLegacy", ctypes.c_ulonglong),
        ("GetClass", ctypes.c_ulonglong),
        ("UseCommunityMissionForLog", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("IsCritical", ctypes.c_ulonglong),
        ("BlocksPinning", ctypes.c_ulonglong),
        ("IsProcedural", ctypes.c_ulonglong),
        ("DataAllowsRenounce", ctypes.c_ulonglong),
        ("RefreshLogInfo", ctypes.c_ulonglong),
        ("GetCurrentCategory", ctypes.c_ulonglong),
        ("GetCurrentColour", ctypes.c_ulonglong),
        ("GetCurrentIcon", ctypes.c_ulonglong),
        ("GetIconAtStage", ctypes.c_char * 0x8),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *(__fastcall *)(cGcNotificationSequence *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *result, int)
        ("GetCurrentFrontendPageHint", ctypes.c_ulonglong),
        ("GetCurrentInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("GetCurrentFrontendPageLocID", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, TkID<256> *)
        ("GetCurrentBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("CheckOptionCondition", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, const TkID<256> *, cTkFixedString<1024,char> *)
        ("WriteItemText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *, bool)
        ("Restart", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("DebugNguiMissionInfo", ctypes.c_ulonglong),
        ("HasText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetParticipantTypeFromPointer", ctypes.c_ulonglong),
        ("GetParticipantPointerFromType", ctypes.c_ulonglong),
        ("GetNumberOfParticipants", ctypes.c_ulonglong),
        ("HasBeenHandedIn", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("CanBegin", ctypes.c_ulonglong),
        ("CanBeRenounced", ctypes.c_ulonglong),
        ("IsForcingPageHint", ctypes.c_ulonglong),
        ("IsForcingBuildMenuHint", ctypes.c_ulonglong),
        ("IsTelemetryUpload", ctypes.c_ulonglong),
        ("GetMultiplayerMissionType", ctypes.c_ulonglong),
    ]
    cGcNotificationSequence_dtor_0: int
    Update: int
    IsAppropriate: int
    GetMissionDataTitle: bytes
    GetTitle: bytes
    GetSubtitle: bytes
    GetDescription: bytes
    GetMissionIcon: int
    GetMissionIconSelected: int
    GetMissionIconNotSelected: int
    ShowFutureObjectives: int
    IsPrimary: int
    IsLegacy: int
    GetClass: int
    UseCommunityMissionForLog: bytes
    IsCritical: int
    BlocksPinning: int
    IsProcedural: int
    DataAllowsRenounce: int
    RefreshLogInfo: int
    GetCurrentCategory: int
    GetCurrentColour: int
    GetCurrentIcon: int
    GetIconAtStage: bytes
    GetCurrentFrontendPageHint: int
    GetCurrentInventoryHint: bytes
    GetCurrentFrontendPageLocID: bytes
    GetCurrentBuildMenuHint: bytes
    CheckOptionCondition: bytes
    WriteItemText: bytes
    Restart: int
    Cancel: int
    DebugNguiMissionInfo: int
    HasText: int
    GetText: bytes
    GetParticipantTypeFromPointer: int
    GetParticipantPointerFromType: int
    GetNumberOfParticipants: int
    HasBeenHandedIn: int
    NGuiRender: int
    Load: int
    Save: int
    CanBegin: int
    CanBeRenounced: int
    IsForcingPageHint: int
    IsForcingBuildMenuHint: int
    IsTelemetryUpload: int
    GetMultiplayerMissionType: int

class cGcNetworkSynchronisedStateDataMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkSynchronisedStateEditMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcSynchronisedStateBufferBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcSynchronisedStateBufferBase_dtor_0", ctypes.c_ulonglong),
        ("GetChunksCount", ctypes.c_ulonglong),
        ("GetChunkHashValue", ctypes.c_ulonglong),
        ("GetChunkVersion", ctypes.c_ulonglong),
        ("GetHashOffset", ctypes.c_ulonglong),
        ("GetHashOffsetVersion", ctypes.c_ulonglong),
        ("SetHashOffset", ctypes.c_ulonglong),
        ("IsHashOffsetCompatible", ctypes.c_ulonglong),
        ("UpdateHashOffsetVersion", ctypes.c_ulonglong),
        ("CreateDataMessage", ctypes.c_char * 0x8),  # std::shared_ptr<cGcNetworkSynchronisedStateDataMessage> *(__fastcall *)(cGcSynchronisedStateBufferBase *this, std::shared_ptr<cGcNetworkSynchronisedStateDataMessage> *result, unsigned __int16)
        ("ApplyDataMessage", ctypes.c_ulonglong),
        ("ApplyEditMessage", ctypes.c_ulonglong),
        ("TryApplyCoherentEditMessage", ctypes.c_ulonglong),
    ]
    cGcSynchronisedStateBufferBase_dtor_0: int
    GetChunksCount: int
    GetChunkHashValue: int
    GetChunkVersion: int
    GetHashOffset: int
    GetHashOffsetVersion: int
    SetHashOffset: int
    IsHashOffsetCompatible: int
    UpdateHashOffsetVersion: int
    CreateDataMessage: bytes
    ApplyDataMessage: int
    ApplyEditMessage: int
    TryApplyCoherentEditMessage: int

class cGcRepairTechComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
        ("IsRepair", ctypes.c_ulonglong),
        ("IsCooker", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("GetModelViewerRenderData", ctypes.c_ulonglong),
        ("RefreshPersistency", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("DeleteSavedState", ctypes.c_ulonglong),
        ("IsNetworkLockingType", ctypes.c_ulonglong),
        ("Setup", ctypes.c_ulonglong),
        ("Seed", ctypes.c_ulonglong),
        ("CustomSlotUpdate", ctypes.c_ulonglong),
        ("CustomSlotRequirementsMet", ctypes.c_ulonglong),
        ("ItemAmountAccumulatorActive", ctypes.c_ulonglong),
        ("GetSavePosition", ctypes.c_char * 0x8),  # cGcVariablePersistentInteractionBuffer<cGcMaintenanceContainer>::eLocation (__fastcall *)(cGcMaintenanceComponent *this, cTkVector3 *)
        ("GetPersistancyData", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int
    IsRepair: int
    IsCooker: int
    Interact: int
    GetModelViewerRenderData: int
    RefreshPersistency: int
    Save: int
    DeleteSavedState: int
    IsNetworkLockingType: int
    Setup: int
    Seed: int
    CustomSlotUpdate: int
    CustomSlotRequirementsMet: int
    ItemAmountAccumulatorActive: int
    GetSavePosition: bytes
    GetPersistancyData: int

class sBufferIndex(ctypes.Structure):
    _fields_ = [
        ("muIndex", ctypes.c_uint16),
    ]
    muIndex: int

class sBuffersMask(ctypes.Structure):
    _fields_ = [
        ("mMask", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
    ]
    mMask: list[ctypes.c_uint8]

class cGcNetworkSynchronisedStateHashMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class sGcTelemetryUpgradeStat_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetVariant", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(IStatVariant *this, TkID<128> *result, const TkID<128> *)
    ]
    GetVariant: bytes

class ComponentInstancercGcRocketLockerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcTechnologyAttachmentComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcRocketLockerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcGalaxyVoxelGenerator(ctypes.Structure):
    def Populate(): pass


class cGcFrontendPageSeason(ctypes.Structure):
    def DoSeason(): pass

    def DoStagePanel(): pass

    def UpdateStageSelectStar(): pass

    def DoFinalStagePanel(): pass

    def DoMilestonesPanel(): pass

    def DoMilestonePatch(): pass

    def DoLargePinnedMilestonePatch(): pass

    def DoFinalMilestonePatch(): pass

    def DoOptionalMilestonesReadout(): pass

    def DoStatPanel(): pass

    def GetProgressText(): pass

    def GetLongDescriptionText(): pass

    def FormatFinalObjectives(): pass


class cGcGenericSectionConditionBaseClaimed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionBaseQuery_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionBasePartsQuery_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionMessageBeaconsQuery_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCriticalMissionsDone_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionLocation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionWarping_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPlanetResourceHint_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPlanetCreatureRoles_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionBasePartNear_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionBaseRequiresPower_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionRefinerActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionRefinerHasOutput_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionBasePartBuilt_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionGameMode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionMissionCompleted_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionBiomeType_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionWaterPlanet_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionWaterInSystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSentinelLevel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSentinelsDisabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSystemRace_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPirateSystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasIllegalGoods_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasIngredientsForItem_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionForceHideMultiplayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPlanetAttackPiratesActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSystemPlanetTest_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSystemHasInfestedPlanet_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSystemStarClass_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionProductKnown_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSpecialKnown_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSeasonRewardUnlocked_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionInSeasonalUA_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsPartyPlanetUnlocked_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionTakingDamage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionShieldDown_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionTechnologyKnown_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasSubstance_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasProduct_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasProcProduct_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasMoney_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasLegacyBasePending_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionInMultiplayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionWeaponMode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionWaitForPirates_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionReadyToSpawnPirates_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasTechnology_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasProcTechnology_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsTechnologyRepaired_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasFreighter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionMultiplayerFreighterAvailable_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasExocraft_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPrimaryExocraft_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCanSummonExocraft_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionBasePowerGenerated_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionNPCHabitationHasWorker_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionFactionRank_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionMissionMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionMissionMessageWarp_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionMissionMessagePortal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionTotalWarpsNumber_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsPlayerWanted_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasFuel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasFuelForTakeoff_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionInventorySlots_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionFrigateCount_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionExpeditionCount_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcNotificationSequenceFactory(ctypes.Structure):
    def Create(): pass


class cGcGenericSectionConditionExpeditionContainsReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionDamagedFrigateAtHome_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionExpeditionProgress_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionStatLevel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPlanetStatLevel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionStatDiff_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionStatChange_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsAnomalyLoaded_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsLookingAtAnomalySpawn_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsCurrentMission_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionMissionSelected_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasGalacticFeature_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionGlobalMission_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionFreighterBattle_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionExtraSuitSlots_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCanReceiveReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPercentChance_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionGroup_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionAIShipCount_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionNumberOfShipsOwned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionNumAtlasStationsVisited_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsScanEventLocal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsDepotDestroyed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionAreDroneHivePartsDestroyed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class ComponentInstancercGcShootableComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcShipOwnershipComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcShipOwnershipComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcGenericSectionConditionIsScanEventOnCurrentPlanet_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasEndpointForEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionNearestBuilding_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionInUA_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionEventRequiresRGB_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPulseEncounterOverriden_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPulseEncounterActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionAbandonedFreighterExplored_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionAlienPodAggroed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasEntitlement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionUnclaimedStageReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionExpeditionNearlyOver_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcShipOwnershipComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcGenericSectionConditionHasSeasonalReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSeasonRewardsRecipe_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasTwitchReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasPlatformReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasShip_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasMultiTool_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCreatureOwned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCreatureSlots_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCreatureTrust_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCreatureSummoned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCreatureReadyToHatch_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCreatureReadyToLay_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSquadronPilotsOwned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSquadronSlots_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionElevation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionUnderwaterDepth_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCurrentSlope_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class sGcTelemetryMultiplayerCountVariant_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetVariant", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(IStatVariant *this, TkID<128> *result, const TkID<128> *)
    ]
    GetVariant: bytes

class cGcGenericSectionConditionNearPole_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsMissionInProgress_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionNearObject_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionNearScanEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionUsingPortal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionWeather_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHazard_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionInCombat_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionWaitForTime_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionNexusEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionInVR_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionLocalScanActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionWristMenuOpen_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionPadActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionOnPlanetWithSandwormsOverriden_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionGunOut_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsGrabbed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionEggMachinePageOpen_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasCreatureEggItem_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionAllMilestonesComplete_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSeasonNumber_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionConvertedFromSeason_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionInteractionIndexChanged_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionInventoryOpen_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionAbandonedOrEmptySystem_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasActiveDetailMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasCommunicatorSignal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionBuildMenuOpen_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasActiveStatsMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasSpareProcTech_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsSurveying_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionOnMultiplayerMission_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionRidingCreature_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionFeedingCreatures_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasGrabbableTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsFrigateFlybyActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionAimingTeleporter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionExocraftMoving_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsAbandFreighterDoorOpen_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionShipEngineStatus_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionUsingThirdPersonCamera_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCameraControlStealing_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsPlayerWeak_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionDefaultItem_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSettlementsEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSettlementStatLevel_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasSettlement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasSettlementLocal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasSettlementBuilding_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasSettlementProductPending_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasAnySettlementBuildingInProgress_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSettlementBuildingsAllStarted_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasPendingSettlementJudgement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionCommunityResearchTier_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHasProcMissionForFaction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionHazardsEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionLifeSupportEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionItemCostsEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionStartWithAllPartsKnown_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionTutorialEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionOnFootCombatEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionSpaceCombatEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionAutoPowerEnabled_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionVisorActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConsequenceRemoveScanEvent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionConsequence_dtor_0", ctypes.c_ulonglong),
        ("DoConsequence", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<256,char> *(__fastcall *)(cGcGenericSectionConsequence *this, cTkFixedString<256,char> *result)
    ]
    cGcGenericSectionConsequence_dtor_0: int
    DoConsequence: int
    GetDescription: bytes

class cGcGenericSectionConsequenceRemoveCommunicatorMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionConsequence_dtor_0", ctypes.c_ulonglong),
        ("DoConsequence", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<256,char> *(__fastcall *)(cGcGenericSectionConsequence *this, cTkFixedString<256,char> *result)
    ]
    cGcGenericSectionConsequence_dtor_0: int
    DoConsequence: int
    GetDescription: bytes

class cGcGenericSectionConsequenceClearDetailMessages_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionConsequence_dtor_0", ctypes.c_ulonglong),
        ("DoConsequence", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<256,char> *(__fastcall *)(cGcGenericSectionConsequence *this, cTkFixedString<256,char> *result)
    ]
    cGcGenericSectionConsequence_dtor_0: int
    DoConsequence: int
    GetDescription: bytes

class cGcGenericSectionConsequenceResetPulseEncounterOverride_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionConsequence_dtor_0", ctypes.c_ulonglong),
        ("DoConsequence", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<256,char> *(__fastcall *)(cGcGenericSectionConsequence *this, cTkFixedString<256,char> *result)
    ]
    cGcGenericSectionConsequence_dtor_0: int
    DoConsequence: int
    GetDescription: bytes

class cGcMissionCounter(ctypes.Structure):
    class MissionCounterHelper: pass


class _MINIDUMP_EXCEPTION_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("ThreadId", ctypes.c_uint32),
        ("ExceptionPointers", ctypes.c_ulonglong),
        ("ClientPointers", ctypes.c_int32),
    ]
    ThreadId: int
    ExceptionPointers: int
    ClientPointers: int

class _MINIDUMP_MEMORY_INFO(ctypes.Structure):
    _fields_ = [
        ("BaseAddress", ctypes.c_uint64),
        ("AllocationBase", ctypes.c_uint64),
        ("AllocationProtect", ctypes.c_uint32),
        ("__alignment1", ctypes.c_uint32),
        ("RegionSize", ctypes.c_uint64),
        ("State", ctypes.c_uint32),
        ("Protect", ctypes.c_uint32),
        ("Type", ctypes.c_uint32),
        ("__alignment2", ctypes.c_uint32),
    ]
    BaseAddress: int
    AllocationBase: int
    AllocationProtect: int
    __alignment1: int
    RegionSize: int
    State: int
    Protect: int
    Type: int
    __alignment2: int

class _MINIDUMP_USER_STREAM(ctypes.Structure):
    _fields_ = [
        ("Type", ctypes.c_uint32),
        ("BufferSize", ctypes.c_uint32),
        ("Buffer", ctypes.c_void_p),
    ]
    Type: int
    BufferSize: int
    Buffer: int

class _MINIDUMP_USER_STREAM_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("UserStreamCount", ctypes.c_uint32),
        ("UserStreamArray", ctypes.c_ulonglong),
    ]
    UserStreamCount: int
    UserStreamArray: int

class _MINIDUMP_INCLUDE_THREAD_CALLBACK(ctypes.Structure):
    _fields_ = [
        ("ThreadId", ctypes.c_uint32),
    ]
    ThreadId: int

class _MINIDUMP_INCLUDE_MODULE_CALLBACK(ctypes.Structure):
    _fields_ = [
        ("BaseOfImage", ctypes.c_uint64),
    ]
    BaseOfImage: int

class _MINIDUMP_IO_CALLBACK(ctypes.Structure):
    _fields_ = [
        ("Handle", ctypes.c_void_p),
        ("Offset", ctypes.c_uint64),
        ("Buffer", ctypes.c_void_p),
        ("BufferBytes", ctypes.c_uint32),
    ]
    Handle: int
    Offset: int
    Buffer: int
    BufferBytes: int

class _MINIDUMP_VM_QUERY_CALLBACK(ctypes.Structure):
    _fields_ = [
        ("Offset", ctypes.c_uint64),
    ]
    Offset: int

class _MINIDUMP_VM_PRE_READ_CALLBACK(ctypes.Structure):
    _fields_ = [
        ("Offset", ctypes.c_uint64),
        ("Buffer", ctypes.c_void_p),
        ("Size", ctypes.c_uint32),
    ]
    Offset: int
    Buffer: int
    Size: int

class cGcNotificationMission_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequence_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsAppropriate", ctypes.c_ulonglong),
        ("GetMissionDataTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetSubtitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<256,char> *)
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetMissionIcon", ctypes.c_ulonglong),
        ("GetMissionIconSelected", ctypes.c_ulonglong),
        ("GetMissionIconNotSelected", ctypes.c_ulonglong),
        ("ShowFutureObjectives", ctypes.c_ulonglong),
        ("IsPrimary", ctypes.c_ulonglong),
        ("IsLegacy", ctypes.c_ulonglong),
        ("GetClass", ctypes.c_ulonglong),
        ("UseCommunityMissionForLog", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("IsCritical", ctypes.c_ulonglong),
        ("BlocksPinning", ctypes.c_ulonglong),
        ("IsProcedural", ctypes.c_ulonglong),
        ("DataAllowsRenounce", ctypes.c_ulonglong),
        ("RefreshLogInfo", ctypes.c_ulonglong),
        ("GetCurrentCategory", ctypes.c_ulonglong),
        ("GetCurrentColour", ctypes.c_ulonglong),
        ("GetCurrentIcon", ctypes.c_ulonglong),
        ("GetIconAtStage", ctypes.c_char * 0x8),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *(__fastcall *)(cGcNotificationSequence *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *result, int)
        ("GetCurrentFrontendPageHint", ctypes.c_ulonglong),
        ("GetCurrentInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("GetCurrentFrontendPageLocID", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, TkID<256> *)
        ("GetCurrentBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("CheckOptionCondition", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, const TkID<256> *, cTkFixedString<1024,char> *)
        ("WriteItemText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *, bool)
        ("Restart", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("DebugNguiMissionInfo", ctypes.c_ulonglong),
        ("HasText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetParticipantTypeFromPointer", ctypes.c_ulonglong),
        ("GetParticipantPointerFromType", ctypes.c_ulonglong),
        ("GetNumberOfParticipants", ctypes.c_ulonglong),
        ("HasBeenHandedIn", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("CanBegin", ctypes.c_ulonglong),
        ("CanBeRenounced", ctypes.c_ulonglong),
        ("IsForcingPageHint", ctypes.c_ulonglong),
        ("IsForcingBuildMenuHint", ctypes.c_ulonglong),
        ("IsTelemetryUpload", ctypes.c_ulonglong),
        ("GetMultiplayerMissionType", ctypes.c_ulonglong),
    ]
    cGcNotificationSequence_dtor_0: int
    Update: int
    IsAppropriate: int
    GetMissionDataTitle: bytes
    GetTitle: bytes
    GetSubtitle: bytes
    GetDescription: bytes
    GetMissionIcon: int
    GetMissionIconSelected: int
    GetMissionIconNotSelected: int
    ShowFutureObjectives: int
    IsPrimary: int
    IsLegacy: int
    GetClass: int
    UseCommunityMissionForLog: bytes
    IsCritical: int
    BlocksPinning: int
    IsProcedural: int
    DataAllowsRenounce: int
    RefreshLogInfo: int
    GetCurrentCategory: int
    GetCurrentColour: int
    GetCurrentIcon: int
    GetIconAtStage: bytes
    GetCurrentFrontendPageHint: int
    GetCurrentInventoryHint: bytes
    GetCurrentFrontendPageLocID: bytes
    GetCurrentBuildMenuHint: bytes
    CheckOptionCondition: bytes
    WriteItemText: bytes
    Restart: int
    Cancel: int
    DebugNguiMissionInfo: int
    HasText: int
    GetText: bytes
    GetParticipantTypeFromPointer: int
    GetParticipantPointerFromType: int
    GetNumberOfParticipants: int
    HasBeenHandedIn: int
    NGuiRender: int
    Load: int
    Save: int
    CanBegin: int
    CanBeRenounced: int
    IsForcingPageHint: int
    IsForcingBuildMenuHint: int
    IsTelemetryUpload: int
    GetMultiplayerMissionType: int

class _MINIDUMP_CALLBACK_INPUT(ctypes.Structure):
    _fields_ = [
        ("ProcessId", ctypes.c_uint32),
        ("ProcessHandle", ctypes.c_void_p),
        ("CallbackType", ctypes.c_uint32),
        ("___u3", ctypes.c_char * 0x510),  # $353A0B9DA135E8AB8B11D9ACE5430F4B
    ]
    ProcessId: int
    ProcessHandle: int
    CallbackType: int
    ___u3: bytes

class _MINIDUMP_CALLBACK_OUTPUT(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x34),  # $2E3E1927C45D6CC4A9BB02AB7F4A96FD
    ]
    ___u0: bytes

class _MINIDUMP_CALLBACK_INFORMATION(ctypes.Structure):
    _fields_ = [
        ("CallbackRoutine", ctypes.c_ulonglong),
        ("CallbackParam", ctypes.c_void_p),
    ]
    CallbackRoutine: int
    CallbackParam: int

class cGcNetworkRpcResponseMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcShipAILandingData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcBlackboard_dtor_0", ctypes.c_ulonglong),
        ("GetIdentifier", ctypes.c_ulonglong),
    ]
    cGcBlackboard_dtor_0: int
    GetIdentifier: int

class cGcShipAIFollowTradeRoutes_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIDocked_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAITakeOff_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAILand_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAILandNextToPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIShooterData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcBlackboard_dtor_0", ctypes.c_ulonglong),
        ("GetIdentifier", ctypes.c_ulonglong),
    ]
    cGcBlackboard_dtor_0: int
    GetIdentifier: int

class cGcShipAICheckShooter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAISetPlayerShooter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAISetAlert_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAICheckPlayerIsShooter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIPlayerSquadronCheckFormation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIPlayerSquadronUpdateTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIPlayerSquadronLeave_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class ComponentInstancercGcByteBeatSwitchComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cProcTexAsyncReadData(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x118),  # cCallbackData<cTkProceduralTextureList>
        ("msNameIn", ctypes.c_char * 0x100),  # cTkFixedString<256,char>
        ("mpProcTexDescriptor", ctypes.c_ulonglong),
        ("miSamplerIndex", ctypes.c_int32),
        ("miMappingIndex", ctypes.c_int32),
        ("mbRead", ctypes.c_ubyte),
    ]
    baseclass_0: bytes
    msNameIn: bytes
    mpProcTexDescriptor: int
    miSamplerIndex: int
    miMappingIndex: int
    mbRead: bool

class DiscoveryProgress(ctypes.Structure):
    _fields_ = [
        ("miCount", ctypes.c_int32),
        ("miTotal", ctypes.c_int32),
    ]
    miCount: int
    miTotal: int

class cGcFrontendPageWiki(ctypes.Structure):
    def GetWiki(): pass

    def GetWikiWritable(): pass

    def GetWikiPageData(): pass

    def DoWiki(): pass

    def DoStoriesWiki(): pass

    def GetListPageItemsData(): pass

    def DoWikiListPage(): pass

    def DoStoriesListPage(): pass

    def DoGridPageSwitch(): pass

    def GetWikiGridItemsData(): pass

    def DoGridItem(): pass

    def DoGridItemIcon(): pass

    def DoGridItemPlanet(): pass

    def DoGuideContentPage(): pass

    def DoStoriesContentPage(): pass

    def MarkItemAsSeen(): pass

    def DoCustomWonderPopup(): pass

    def DoConfirmViewWonderInDiscoveries(): pass

    def FetchRecipesForCatalogue(): pass


class cGcUIOptionListElement(ctypes.Structure):
    def IsHovering(): pass

    def SetEnabled(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mpElement", ctypes.c_ulonglong),
        ("mpContext", ctypes.c_ulonglong),
        ("mbEnabled", ctypes.c_ubyte),
        ("mbVisible", ctypes.c_ubyte),
    ]
    __vftable: int
    mpElement: int
    mpContext: int
    mbEnabled: bool
    mbVisible: bool

class cGcUIOptionListElement_vtbl(ctypes.Structure):
    _fields_ = [
        ("UpdateUI", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Apply", ctypes.c_ulonglong),
        ("ResetToDefault", ctypes.c_ulonglong),
        ("DiscardChanges", ctypes.c_ulonglong),
        ("HasChanges", ctypes.c_ulonglong),
        ("IsNonDefault", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("IsHovering", ctypes.c_ulonglong),
        ("cGcUIOptionListElement_dtor_50", ctypes.c_ulonglong),
        ("TranslateDescription", ctypes.c_ulonglong),
    ]
    UpdateUI: int
    Initialize: int
    Apply: int
    ResetToDefault: int
    DiscardChanges: int
    HasChanges: int
    IsNonDefault: int
    GetDescription: int
    GetName: int
    IsHovering: int
    cGcUIOptionListElement_dtor_50: int
    TranslateDescription: int

class cGcQualityOption(ctypes.Structure):
    def CreateElement(): pass

    def UpdateUI(): pass

    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x460),  # cGcOption<enum eGraphicDetail>
        ("mapButton", ctypes.c_ulonglong * 0x4),  # cGcNGuiLayer *[4]
        ("mapActiveLayer", ctypes.c_ulonglong * 0x4),  # cGcNGuiLayer *[4]
        ("mapInactiveLayer", ctypes.c_ulonglong * 0x4),  # cGcNGuiLayer *[4]
        ("mapBlockedLayer", ctypes.c_ulonglong * 0x4),  # cGcNGuiLayer *[4]
    ]
    baseclass_0: bytes
    mapButton: list[ctypes.c_ulonglong]
    mapActiveLayer: list[ctypes.c_ulonglong]
    mapInactiveLayer: list[ctypes.c_ulonglong]
    mapBlockedLayer: list[ctypes.c_ulonglong]

class cGcQualityOption_vtbl(ctypes.Structure):
    _fields_ = [
        ("UpdateUI", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Apply", ctypes.c_ulonglong),
        ("ResetToDefault", ctypes.c_ulonglong),
        ("DiscardChanges", ctypes.c_ulonglong),
        ("HasChanges", ctypes.c_ulonglong),
        ("IsNonDefault", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("IsHovering", ctypes.c_ulonglong),
        ("cGcUIOptionListElement_dtor_50", ctypes.c_ulonglong),
        ("TranslateDescription", ctypes.c_ulonglong),
        ("CreateElement", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcOption<enum eGraphicDetail> *this)
        ("GetTranslatedDefaultValue", ctypes.c_char * 0x8),  # cTkFixedString<32,char> *(__fastcall *)(cGcOption<enum eGraphicDetail> *this, cTkFixedString<32,char> *result)
    ]
    UpdateUI: int
    Initialize: int
    Apply: int
    ResetToDefault: int
    DiscardChanges: int
    HasChanges: int
    IsNonDefault: int
    GetDescription: int
    GetName: int
    IsHovering: int
    cGcUIOptionListElement_dtor_50: int
    TranslateDescription: int
    CreateElement: bytes
    GetTranslatedDefaultValue: bytes

class cGcFrontendPageJourney(ctypes.Structure):
    def DoJourney(): pass

    def DoPinnedOverallJourneyMedalBlock(): pass

    def DoPinnedMedalBlock(): pass

    def DoFactionMedalsBlock(): pass

    def DoOverallJourneyMedal(): pass

    def DoMedalBox(): pass

    def GetMedalIcon(): pass

    def GetProgressText(): pass


class cGcFloatOption(ctypes.Structure):
    def CreateElement(): pass

    def UpdateUI(): pass

    def GetTranslatedDefaultValue(): pass

    def FormatValue(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x460),  # cGcOption<float>
        ("mfMin", ctypes.c_float),
        ("mfMax", ctypes.c_float),
        ("mfIncrement", ctypes.c_float),
        ("mpacSuffix", ctypes.c_char_p),
        ("mpButtonDecrease", ctypes.c_ulonglong),
        ("mpButtonIncrease", ctypes.c_ulonglong),
        ("mpText", ctypes.c_ulonglong),
        ("mfDisplayFactor", ctypes.c_float),
    ]
    baseclass_0: bytes
    mfMin: float
    mfMax: float
    mfIncrement: float
    mpacSuffix: bytes
    mpButtonDecrease: int
    mpButtonIncrease: int
    mpText: int
    mfDisplayFactor: float

class cGcFloatOption_vtbl(ctypes.Structure):
    _fields_ = [
        ("UpdateUI", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Apply", ctypes.c_ulonglong),
        ("ResetToDefault", ctypes.c_ulonglong),
        ("DiscardChanges", ctypes.c_ulonglong),
        ("HasChanges", ctypes.c_ulonglong),
        ("IsNonDefault", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("IsHovering", ctypes.c_ulonglong),
        ("cGcUIOptionListElement_dtor_50", ctypes.c_ulonglong),
        ("TranslateDescription", ctypes.c_ulonglong),
        ("CreateElement", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcOption<float> *this)
        ("GetTranslatedDefaultValue", ctypes.c_char * 0x8),  # cTkFixedString<32,char> *(__fastcall *)(cGcOption<float> *this, cTkFixedString<32,char> *result)
    ]
    UpdateUI: int
    Initialize: int
    Apply: int
    ResetToDefault: int
    DiscardChanges: int
    HasChanges: int
    IsNonDefault: int
    GetDescription: int
    GetName: int
    IsHovering: int
    cGcUIOptionListElement_dtor_50: int
    TranslateDescription: int
    CreateElement: bytes
    GetTranslatedDefaultValue: bytes

class cGcBooleanOption(ctypes.Structure):
    def UpdateUI(): pass

    def CreateElement(): pass

    def GetTranslatedDefaultValue(): pass

    def vector_deleting_destructor(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x460),  # cGcOption<bool>
        ("mpacEnabledString", ctypes.c_char_p),
        ("mpacDisabledString", ctypes.c_char_p),
        ("mpText", ctypes.c_ulonglong),
        ("mpButton", ctypes.c_ulonglong),
    ]
    baseclass_0: bytes
    mpacEnabledString: bytes
    mpacDisabledString: bytes
    mpText: int
    mpButton: int

class cGcBooleanOption_vtbl(ctypes.Structure):
    _fields_ = [
        ("UpdateUI", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Apply", ctypes.c_ulonglong),
        ("ResetToDefault", ctypes.c_ulonglong),
        ("DiscardChanges", ctypes.c_ulonglong),
        ("HasChanges", ctypes.c_ulonglong),
        ("IsNonDefault", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("IsHovering", ctypes.c_ulonglong),
        ("cGcUIOptionListElement_dtor_50", ctypes.c_ulonglong),
        ("TranslateDescription", ctypes.c_ulonglong),
        ("CreateElement", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcOption<bool> *this)
        ("GetTranslatedDefaultValue", ctypes.c_char * 0x8),  # cTkFixedString<32,char> *(__fastcall *)(cGcOption<bool> *this, cTkFixedString<32,char> *result)
    ]
    UpdateUI: int
    Initialize: int
    Apply: int
    ResetToDefault: int
    DiscardChanges: int
    HasChanges: int
    IsNonDefault: int
    GetDescription: int
    GetName: int
    IsHovering: int
    cGcUIOptionListElement_dtor_50: int
    TranslateDescription: int
    CreateElement: bytes
    GetTranslatedDefaultValue: bytes

class cGcIntegerOption(ctypes.Structure):
    def UpdateUI(): pass

    def GetTranslatedDefaultValue(): pass

    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x460),  # cGcOption<int>
        ("miMin", ctypes.c_int32),
        ("miMax", ctypes.c_int32),
        ("miIncrement", ctypes.c_int32),
        ("mpacSuffix", ctypes.c_char_p),
        ("mpButtonDecrease", ctypes.c_ulonglong),
        ("mpButtonIncrease", ctypes.c_ulonglong),
        ("mpText", ctypes.c_ulonglong),
        ("miDisplayFactor", ctypes.c_int32),
    ]
    baseclass_0: bytes
    miMin: int
    miMax: int
    miIncrement: int
    mpacSuffix: bytes
    mpButtonDecrease: int
    mpButtonIncrease: int
    mpText: int
    miDisplayFactor: int

class cGcIntegerOption_vtbl(ctypes.Structure):
    _fields_ = [
        ("UpdateUI", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Apply", ctypes.c_ulonglong),
        ("ResetToDefault", ctypes.c_ulonglong),
        ("DiscardChanges", ctypes.c_ulonglong),
        ("HasChanges", ctypes.c_ulonglong),
        ("IsNonDefault", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("IsHovering", ctypes.c_ulonglong),
        ("cGcUIOptionListElement_dtor_50", ctypes.c_ulonglong),
        ("TranslateDescription", ctypes.c_ulonglong),
        ("CreateElement", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcOption<int> *this)
        ("GetTranslatedDefaultValue", ctypes.c_char * 0x8),  # cTkFixedString<32,char> *(__fastcall *)(cGcOption<int> *this, cTkFixedString<32,char> *result)
    ]
    UpdateUI: int
    Initialize: int
    Apply: int
    ResetToDefault: int
    DiscardChanges: int
    HasChanges: int
    IsNonDefault: int
    GetDescription: int
    GetName: int
    IsHovering: int
    cGcUIOptionListElement_dtor_50: int
    TranslateDescription: int
    CreateElement: bytes
    GetTranslatedDefaultValue: bytes

class cGcButtonOption_vtbl(ctypes.Structure):
    _fields_ = [
        ("UpdateUI", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Apply", ctypes.c_ulonglong),
        ("ResetToDefault", ctypes.c_ulonglong),
        ("DiscardChanges", ctypes.c_ulonglong),
        ("HasChanges", ctypes.c_ulonglong),
        ("IsNonDefault", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("IsHovering", ctypes.c_ulonglong),
        ("cGcUIOptionListElement_dtor_50", ctypes.c_ulonglong),
        ("TranslateDescription", ctypes.c_ulonglong),
    ]
    UpdateUI: int
    Initialize: int
    Apply: int
    ResetToDefault: int
    DiscardChanges: int
    HasChanges: int
    IsNonDefault: int
    GetDescription: int
    GetName: int
    IsHovering: int
    cGcUIOptionListElement_dtor_50: int
    TranslateDescription: int

class cGcFrontendPageMissions(ctypes.Structure):
    def DoMissionLog(): pass


class cGcFrontendPageMissionGiver(ctypes.Structure):
    def UpdateMissionPage(): pass

    def UpdateMissionDescriptionPage(): pass

    def UpdateMissionHandInPage(): pass

    def HasLocalMissionInProgress(): pass

    def HasHandInWaitingFromOtherBoard(): pass

    def DoMissionGiverInteraction(): pass

    def DoMissionGiverOptions(): pass

    def DoMissionPage(): pass

    def DoMissionListItem(): pass

    def LayoutReward(): pass

    def DoMissionHandInPage(): pass

    def DoMissionRenouncePage(): pass

    def DoMissionDescriptionPage(): pass

    def DoMissionItem(): pass

    def DoScreenPanel(): pass

    def GenerateMissionRewardInfo(): pass

    def GetRewardInfo(): pass

    def SelectMissionBoardMissionPrimaryRewardID(): pass

    def FindRecursiveMissionStandingInfo(): pass

    def GetStandingInfo(): pass

    def GetAllowedMissionsListForStandardBoard(): pass

    def GetAllowedMissionsList(): pass

    def CanUseMissionType(): pass

    def IsAllowed(): pass

    def HasUnusedGuildFaction(): pass


class ComponentInstancercGcCharacterInterfaceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcSentinelRobotComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcAbandonedFreighterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcCreatureFeederComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcCreatureHarvesterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ScePadContainerIdInfo(ctypes.Structure):
    _fields_ = [
        ("size", ctypes.c_uint32),
        ("containerInfo", ctypes.c_wchar * 0x1000),  # wchar_t[4096]
    ]
    size: int
    containerInfo: list[ctypes.c_wchar]

class ComponentInstancercGcVehicleGarageComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcMarkerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class TsPoint(ctypes.Structure):
    _fields_ = [
        ("slope", ctypes.c_float),
        ("logx", ctypes.c_float),
        ("logy", ctypes.c_float),
    ]
    slope: float
    logx: float
    logy: float

class cTkTextStyleShadow_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkTextStyle_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
    ]
    cTkTextStyle_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    Render: int
    Release: int
    Destruct: int

class PCBoolParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class PCIntParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class PCUIntParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class PCVoidVec4Param_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class PCFloatParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class PCStringParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class PCID128Param_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class PCID128HashedParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("PCParam_dtor_0", ctypes.c_ulonglong),
    ]
    PCParam_dtor_0: int

class ComponentInstancercGcWaypointComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcWaypointComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcWaypointComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCharacterInterfaceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcGenericSectionConsequenceRemoveCommunicatorTakeOffMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionConsequence_dtor_0", ctypes.c_ulonglong),
        ("DoConsequence", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<256,char> *(__fastcall *)(cGcGenericSectionConsequence *this, cTkFixedString<256,char> *result)
    ]
    cGcGenericSectionConsequence_dtor_0: int
    DoConsequence: int
    GetDescription: bytes

class cGcNotificationSequenceGroupCode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceGroup_dtor_0", ctypes.c_ulonglong),
        ("GetObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceGroup *this, cTkFixedString<128,char> *)
        ("GetObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceGroup *this, cTkFixedString<1024,char> *)
        ("GetTargetMissionSurveyId", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceGroup *this, TkID<128> *result)
        ("GetTargetMissionSurveyDefinitelyExists", ctypes.c_ulonglong),
        ("GetPinnedMissionBuildTarget", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceGroup *this, TkID<128> *result)
    ]
    cGcNotificationSequenceGroup_dtor_0: int
    GetObjective: bytes
    GetObjectiveTip: bytes
    GetTargetMissionSurveyId: bytes
    GetTargetMissionSurveyDefinitelyExists: int
    GetPinnedMissionBuildTarget: bytes

class cGcNotificationSequenceCode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequence_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsAppropriate", ctypes.c_ulonglong),
        ("GetMissionDataTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetSubtitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<256,char> *)
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetMissionIcon", ctypes.c_ulonglong),
        ("GetMissionIconSelected", ctypes.c_ulonglong),
        ("GetMissionIconNotSelected", ctypes.c_ulonglong),
        ("ShowFutureObjectives", ctypes.c_ulonglong),
        ("IsPrimary", ctypes.c_ulonglong),
        ("IsLegacy", ctypes.c_ulonglong),
        ("GetClass", ctypes.c_ulonglong),
        ("UseCommunityMissionForLog", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("IsCritical", ctypes.c_ulonglong),
        ("BlocksPinning", ctypes.c_ulonglong),
        ("IsProcedural", ctypes.c_ulonglong),
        ("DataAllowsRenounce", ctypes.c_ulonglong),
        ("RefreshLogInfo", ctypes.c_ulonglong),
        ("GetCurrentCategory", ctypes.c_ulonglong),
        ("GetCurrentColour", ctypes.c_ulonglong),
        ("GetCurrentIcon", ctypes.c_ulonglong),
        ("GetIconAtStage", ctypes.c_char * 0x8),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *(__fastcall *)(cGcNotificationSequence *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *result, int)
        ("GetCurrentFrontendPageHint", ctypes.c_ulonglong),
        ("GetCurrentInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("GetCurrentFrontendPageLocID", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, TkID<256> *)
        ("GetCurrentBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("CheckOptionCondition", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, const TkID<256> *, cTkFixedString<1024,char> *)
        ("WriteItemText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *, bool)
        ("Restart", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("DebugNguiMissionInfo", ctypes.c_ulonglong),
        ("HasText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetParticipantTypeFromPointer", ctypes.c_ulonglong),
        ("GetParticipantPointerFromType", ctypes.c_ulonglong),
        ("GetNumberOfParticipants", ctypes.c_ulonglong),
        ("HasBeenHandedIn", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("CanBegin", ctypes.c_ulonglong),
        ("CanBeRenounced", ctypes.c_ulonglong),
        ("IsForcingPageHint", ctypes.c_ulonglong),
        ("IsForcingBuildMenuHint", ctypes.c_ulonglong),
        ("IsTelemetryUpload", ctypes.c_ulonglong),
        ("GetMultiplayerMissionType", ctypes.c_ulonglong),
    ]
    cGcNotificationSequence_dtor_0: int
    Update: int
    IsAppropriate: int
    GetMissionDataTitle: bytes
    GetTitle: bytes
    GetSubtitle: bytes
    GetDescription: bytes
    GetMissionIcon: int
    GetMissionIconSelected: int
    GetMissionIconNotSelected: int
    ShowFutureObjectives: int
    IsPrimary: int
    IsLegacy: int
    GetClass: int
    UseCommunityMissionForLog: bytes
    IsCritical: int
    BlocksPinning: int
    IsProcedural: int
    DataAllowsRenounce: int
    RefreshLogInfo: int
    GetCurrentCategory: int
    GetCurrentColour: int
    GetCurrentIcon: int
    GetIconAtStage: bytes
    GetCurrentFrontendPageHint: int
    GetCurrentInventoryHint: bytes
    GetCurrentFrontendPageLocID: bytes
    GetCurrentBuildMenuHint: bytes
    CheckOptionCondition: bytes
    WriteItemText: bytes
    Restart: int
    Cancel: int
    DebugNguiMissionInfo: int
    HasText: int
    GetText: bytes
    GetParticipantTypeFromPointer: int
    GetParticipantPointerFromType: int
    GetNumberOfParticipants: int
    HasBeenHandedIn: int
    NGuiRender: int
    Load: int
    Save: int
    CanBegin: int
    CanBeRenounced: int
    IsForcingPageHint: int
    IsForcingBuildMenuHint: int
    IsTelemetryUpload: int
    GetMultiplayerMissionType: int

class cGcNotificationSequenceCraftDevice_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequence_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsAppropriate", ctypes.c_ulonglong),
        ("GetMissionDataTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetSubtitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<256,char> *)
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetMissionIcon", ctypes.c_ulonglong),
        ("GetMissionIconSelected", ctypes.c_ulonglong),
        ("GetMissionIconNotSelected", ctypes.c_ulonglong),
        ("ShowFutureObjectives", ctypes.c_ulonglong),
        ("IsPrimary", ctypes.c_ulonglong),
        ("IsLegacy", ctypes.c_ulonglong),
        ("GetClass", ctypes.c_ulonglong),
        ("UseCommunityMissionForLog", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("IsCritical", ctypes.c_ulonglong),
        ("BlocksPinning", ctypes.c_ulonglong),
        ("IsProcedural", ctypes.c_ulonglong),
        ("DataAllowsRenounce", ctypes.c_ulonglong),
        ("RefreshLogInfo", ctypes.c_ulonglong),
        ("GetCurrentCategory", ctypes.c_ulonglong),
        ("GetCurrentColour", ctypes.c_ulonglong),
        ("GetCurrentIcon", ctypes.c_ulonglong),
        ("GetIconAtStage", ctypes.c_char * 0x8),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *(__fastcall *)(cGcNotificationSequence *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *result, int)
        ("GetCurrentFrontendPageHint", ctypes.c_ulonglong),
        ("GetCurrentInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("GetCurrentFrontendPageLocID", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, TkID<256> *)
        ("GetCurrentBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("CheckOptionCondition", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, const TkID<256> *, cTkFixedString<1024,char> *)
        ("WriteItemText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *, bool)
        ("Restart", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("DebugNguiMissionInfo", ctypes.c_ulonglong),
        ("HasText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetParticipantTypeFromPointer", ctypes.c_ulonglong),
        ("GetParticipantPointerFromType", ctypes.c_ulonglong),
        ("GetNumberOfParticipants", ctypes.c_ulonglong),
        ("HasBeenHandedIn", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("CanBegin", ctypes.c_ulonglong),
        ("CanBeRenounced", ctypes.c_ulonglong),
        ("IsForcingPageHint", ctypes.c_ulonglong),
        ("IsForcingBuildMenuHint", ctypes.c_ulonglong),
        ("IsTelemetryUpload", ctypes.c_ulonglong),
        ("GetMultiplayerMissionType", ctypes.c_ulonglong),
    ]
    cGcNotificationSequence_dtor_0: int
    Update: int
    IsAppropriate: int
    GetMissionDataTitle: bytes
    GetTitle: bytes
    GetSubtitle: bytes
    GetDescription: bytes
    GetMissionIcon: int
    GetMissionIconSelected: int
    GetMissionIconNotSelected: int
    ShowFutureObjectives: int
    IsPrimary: int
    IsLegacy: int
    GetClass: int
    UseCommunityMissionForLog: bytes
    IsCritical: int
    BlocksPinning: int
    IsProcedural: int
    DataAllowsRenounce: int
    RefreshLogInfo: int
    GetCurrentCategory: int
    GetCurrentColour: int
    GetCurrentIcon: int
    GetIconAtStage: bytes
    GetCurrentFrontendPageHint: int
    GetCurrentInventoryHint: bytes
    GetCurrentFrontendPageLocID: bytes
    GetCurrentBuildMenuHint: bytes
    CheckOptionCondition: bytes
    WriteItemText: bytes
    Restart: int
    Cancel: int
    DebugNguiMissionInfo: int
    HasText: int
    GetText: bytes
    GetParticipantTypeFromPointer: int
    GetParticipantPointerFromType: int
    GetNumberOfParticipants: int
    HasBeenHandedIn: int
    NGuiRender: int
    Load: int
    Save: int
    CanBegin: int
    CanBeRenounced: int
    IsForcingPageHint: int
    IsForcingBuildMenuHint: int
    IsTelemetryUpload: int
    GetMultiplayerMissionType: int

class cGcNotifyGenericSectionMarker(ctypes.Structure):
    _fields_ = [
        ("mpMission", ctypes.c_ulonglong),
        ("miSectionIndex", ctypes.c_uint32),
    ]
    mpMission: int
    miSectionIndex: int

class cGcNotificationSequenceCollect_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCollectMultiProducts_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceBounty_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequencePirates_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForConditions_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceConditionalReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForStat_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceWaitForStatSeasonal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int

class cGcNotificationSequenceWaitForStatMilestone_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetRequiredLevel", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetRequiredLevel: int

class cGcNotificationSequenceWaitForWarps_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceWaitForDepots_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceWaitForMissionSelected_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceDoMissionsForFaction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceWaitForFactionStanding_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceWaitForShips_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceDiscover_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceDiscoverOnPlanet_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceKill_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceFeed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceCommunicator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceCommunicatorOnTakeOff_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceLearnWords_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForPortalWarp_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceTeleport_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceFreighterEngage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceFreighterDefend_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceWaitForFreighterPods_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
        ("RunInternal", ctypes.c_ulonglong),
        ("GetOtherPlayersCounts", ctypes.c_ulonglong),
        ("GetStatValue", ctypes.c_ulonglong),
        ("SetChangeValue", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int
    RunInternal: int
    GetOtherPlayersCounts: int
    GetStatValue: int
    SetChangeValue: int

class cGcNotificationSequenceStartSummonAnomaly_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceFinishSummonAnomaly_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequeneSummonNexus_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceSuppressMarkers_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class cGcNotificationSequenceMultiplayerMissionPlayerRoundUp_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequenceStage_dtor_0", ctypes.c_ulonglong),
        ("SetText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<512,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("OverridesTitle", ctypes.c_ulonglong),
        ("GetFormattedObjective", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("GetFormattedObjectiveTip", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<1024,char> *)
        ("FormatsObjectiveText", ctypes.c_ulonglong),
        ("GetTitleForCodeGroup", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequenceStage *this, cTkFixedString<128,char> *)
        ("NotifyNewlyVisible", ctypes.c_ulonglong),
        ("BlocksTimedScans", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("BlocksVehicleScans", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequenceStage *this, TkID<256>)
        ("NeedsGPSHint", ctypes.c_ulonglong),
        ("GetCurrentScanEventTarget", ctypes.c_char * 0x8),  # TkID<256> *(__fastcall *)(cGcNotificationSequenceStage *this, const TkID<256> *result)
        ("DisableIcon", ctypes.c_ulonglong),
        ("DisableTitlePrefix", ctypes.c_ulonglong),
        ("OverrideWithDangerColour", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("BlockPulseEncounters", ctypes.c_ulonglong),
        ("SuppressMessages", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequenceStage *this, TkID<128> *result)
        ("GetStatusMessageMarkup", ctypes.c_ulonglong),
        ("AllowAutoEndPinnedMission", ctypes.c_ulonglong),
        ("OverridesIcon", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("Activate", ctypes.c_ulonglong),
        ("IsWaitingForCompletionMessage", ctypes.c_ulonglong),
    ]
    cGcNotificationSequenceStage_dtor_0: int
    SetText: int
    GetText: bytes
    GetTitle: bytes
    OverridesTitle: int
    GetFormattedObjective: bytes
    GetFormattedObjectiveTip: bytes
    FormatsObjectiveText: int
    GetTitleForCodeGroup: bytes
    NotifyNewlyVisible: int
    BlocksTimedScans: int
    OverridesMissionSurveyText: int
    BlocksVehicleScans: bytes
    NeedsGPSHint: int
    GetCurrentScanEventTarget: bytes
    DisableIcon: int
    DisableTitlePrefix: int
    OverrideWithDangerColour: int
    AllowMissionRestart: int
    BlockPirateMessage: int
    BlockPulseEncounters: int
    SuppressMessages: int
    GetBuildMenuHint: bytes
    GetStatusMessageMarkup: int
    AllowAutoEndPinnedMission: int
    OverridesIcon: int
    EncouragesFightingSentinels: int
    Activate: int
    IsWaitingForCompletionMessage: int

class ComponentInstancercGcPlayerWeaponComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class sGcNetworkServerTimeSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class sGcNetworkClientTimeSyncMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class NVGglyphPosition(ctypes.Structure):
    _fields_ = [
        ("str", ctypes.c_char_p),
        ("x", ctypes.c_float),
        ("minx", ctypes.c_float),
        ("maxx", ctypes.c_float),
    ]
    str: bytes
    x: float
    minx: float
    maxx: float

class ComponentInstancercGcDistanceScaleComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcPlayerWeaponComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcNetworkMessageData(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cGcNetworkMessageData_vtbl(ctypes.Structure):
    _fields_ = [
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
    ]
    SerializeOrDeserialize: int

class cGcNetworkAiShipMessageData_vtbl(ctypes.Structure):
    _fields_ = [
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
    ]
    SerializeOrDeserialize: int

class cGcMatchmakingNull_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("cGcMatchmaking_dtor_18", ctypes.c_ulonglong),
        ("Suspend", ctypes.c_ulonglong),
        ("Resume", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("EnableVoice", ctypes.c_ulonglong),
        ("IsReadyToPrepare", ctypes.c_ulonglong),
        ("StartAmbientMatchmaking", ctypes.c_ulonglong),
        ("UpdateFriends", ctypes.c_ulonglong),
        ("UpdateFriendSessions", ctypes.c_ulonglong),
        ("GetFriendName", ctypes.c_ulonglong),
        ("GetPlatformType", ctypes.c_ulonglong),
        ("GetExtraFriendsString", ctypes.c_char * 0x8),  # cTkFixedString<256,char> *(__fastcall *)(cGcMatchmaking *this, cTkFixedString<256,char> *result, int)
        ("GetGameModeName", ctypes.c_ulonglong),
        ("GetGameModeEnum", ctypes.c_ulonglong),
        ("GetFriendLobbyCount", ctypes.c_ulonglong),
        ("GetFriendMaxLobbyCount", ctypes.c_ulonglong),
        ("JoinFriend", ctypes.c_ulonglong),
        ("AddFriend", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *, bool, bool)
        ("RemoveFriend", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *, bool)
        ("IsCrossPlatformJoinableFriend", ctypes.c_ulonglong),
        ("GetCrossplayFriendPlatformType", ctypes.c_ulonglong),
        ("GetCrossplayFriendName", ctypes.c_ulonglong),
        ("GetCrossplayFriendId", ctypes.c_char * 0x8),  # cTkUserIdBase<cTkFixedString<64,char> > *(__fastcall *)(cGcMatchmaking *this, cTkUserIdBase<cTkFixedString<64,char> > *result, int)
        ("IsCrossplayFriendOnline", ctypes.c_ulonglong),
        ("GetCrossplayFriendPlatformOnlineId", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcMatchmaking *this, int, cTkUserIdBase<unsigned __int64> *)
        ("GetNativeFriendName", ctypes.c_ulonglong),
        ("GetNativeFriendId", ctypes.c_char * 0x8),  # cTkUserIdBase<unsigned __int64> *(__fastcall *)(cGcMatchmaking *this, cTkUserIdBase<unsigned __int64> *result, int)
        ("IsNativeFriendOnline", ctypes.c_ulonglong),
        ("IsNativeFriendInGame", ctypes.c_ulonglong),
        ("BlockPlayer", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *, const char *, cTkFixedString<64,char>, cTkFixedString<3,char>)
        ("UnblockPlayer", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("UnblockPlayer_130", ctypes.c_ulonglong),
        ("GetBlockedPlayerCount", ctypes.c_ulonglong),
        ("GetBlockedPlayerName", ctypes.c_ulonglong),
        ("IsPlayerBlocked", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("IsPlayerBlockedByPlatformId", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcMatchmaking *this, cTkFixedString<64,char>, cTkFixedString<3,char>)
        ("UpdatePlatformMuted", ctypes.c_ulonglong),
        ("ForceRefreshPlatformMuted", ctypes.c_ulonglong),
        ("ShouldReceiveTextChatFromPlayer", ctypes.c_ulonglong),
        ("GetFullSimLobbyUA", ctypes.c_ulonglong),
        ("UpdateFullSimPrivileges", ctypes.c_ulonglong),
        ("JoinFireteam", ctypes.c_ulonglong),
        ("KickPlayer", ctypes.c_ulonglong),
        ("InvitePlayer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcMatchmaking *this, const cTkUserIdBase<unsigned __int64>)
        ("InvitePlayer_198", ctypes.c_ulonglong),
        ("OpenInviteUI", ctypes.c_ulonglong),
        ("OpenMultiplayerUI", ctypes.c_ulonglong),
        ("MatchmakingServicesRunning", ctypes.c_ulonglong),
        ("InviteToFireteam", ctypes.c_ulonglong),
        ("RequestAddFriend", ctypes.c_ulonglong),
        ("GetLocalUserID", ctypes.c_char * 0x8),  # cTkUserIdBase<cTkFixedString<64,char> > *(__fastcall *)(cGcMatchmaking *this, cTkUserIdBase<cTkFixedString<64,char> > *result)
        ("GetLocalPlatformType", ctypes.c_ulonglong),
        ("UpdateFriendStatus", ctypes.c_ulonglong),
        ("IsPlayerBanned", ctypes.c_ulonglong),
        ("IsInFullSimLobby", ctypes.c_ulonglong),
        ("MakeSessions", ctypes.c_ulonglong),
        ("SendTextChat", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcMatchmaking *this, std::function<void __cdecl(cGcNetworkPlayer *,cTkFixedString<1023,char>,bool,bool,enum TextChatMessageType,cTkColour)>, cGcNetworkPlayer *, cTkFixedString<1023,char>, bool, bool, TextChatMessageType, cTkColour)
        ("SetSpeechToText", ctypes.c_ulonglong),
        ("SetTranslation", ctypes.c_ulonglong),
        ("SetCrossPlatform", ctypes.c_ulonglong),
        ("IsExperiencingWebsocketIssues", ctypes.c_ulonglong),
        ("Connect", ctypes.c_ulonglong),
        ("IsPlatformInvitePending", ctypes.c_ulonglong),
        ("IsPlatformInviteFromAmbientPlayer", ctypes.c_ulonglong),
        ("JoinPlatformInvite", ctypes.c_ulonglong),
        ("CancelPlatformInvite", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int
    cGcMatchmaking_dtor_18: int
    Suspend: int
    Resume: int
    Construct: int
    Update: int
    Prepare: int
    Release: int
    Destruct: int
    EnableVoice: int
    IsReadyToPrepare: int
    StartAmbientMatchmaking: int
    UpdateFriends: int
    UpdateFriendSessions: int
    GetFriendName: int
    GetPlatformType: int
    GetExtraFriendsString: bytes
    GetGameModeName: int
    GetGameModeEnum: int
    GetFriendLobbyCount: int
    GetFriendMaxLobbyCount: int
    JoinFriend: int
    AddFriend: bytes
    RemoveFriend: bytes
    IsCrossPlatformJoinableFriend: int
    GetCrossplayFriendPlatformType: int
    GetCrossplayFriendName: int
    GetCrossplayFriendId: bytes
    IsCrossplayFriendOnline: int
    GetCrossplayFriendPlatformOnlineId: bytes
    GetNativeFriendName: int
    GetNativeFriendId: bytes
    IsNativeFriendOnline: int
    IsNativeFriendInGame: int
    BlockPlayer: bytes
    UnblockPlayer: bytes
    UnblockPlayer_130: int
    GetBlockedPlayerCount: int
    GetBlockedPlayerName: int
    IsPlayerBlocked: bytes
    IsPlayerBlockedByPlatformId: bytes
    UpdatePlatformMuted: int
    ForceRefreshPlatformMuted: int
    ShouldReceiveTextChatFromPlayer: int
    GetFullSimLobbyUA: int
    UpdateFullSimPrivileges: int
    JoinFireteam: int
    KickPlayer: int
    InvitePlayer: bytes
    InvitePlayer_198: int
    OpenInviteUI: int
    OpenMultiplayerUI: int
    MatchmakingServicesRunning: int
    InviteToFireteam: int
    RequestAddFriend: int
    GetLocalUserID: bytes
    GetLocalPlatformType: int
    UpdateFriendStatus: int
    IsPlayerBanned: int
    IsInFullSimLobby: int
    MakeSessions: int
    SendTextChat: bytes
    SetSpeechToText: int
    SetTranslation: int
    SetCrossPlatform: int
    IsExperiencingWebsocketIssues: int
    Connect: int
    IsPlatformInvitePending: int
    IsPlatformInviteFromAmbientPlayer: int
    JoinPlatformInvite: int
    CancelPlatformInvite: int

class cGcCreatureMovementWeird_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCreatureMovementBase_dtor_0", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("GetExtraPartNode", ctypes.c_ulonglong),
        ("HasDeathAction", ctypes.c_ulonglong),
        ("TriggerDeathAction", ctypes.c_ulonglong),
        ("IsCrystal", ctypes.c_ulonglong),
        ("NotifyOfPredator", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("SetSoundState", ctypes.c_ulonglong),
    ]
    cGcCreatureMovementBase_dtor_0: int
    Release: int
    UpdatePostPhysics: int
    GetExtraPartNode: int
    HasDeathAction: int
    TriggerDeathAction: int
    IsCrystal: int
    NotifyOfPredator: int
    RenderNGui: int
    SetSoundState: int

class sGcPlayfabGenericServiceId(ctypes.Structure):
    _fields_ = [
        ("mServiceId", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
        ("mUserId", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    mServiceId: bytes
    mUserId: bytes

class PFEntityKey(ctypes.Structure):
    _fields_ = [
        ("id", ctypes.c_char_p),
        ("type", ctypes.c_char_p),
    ]
    id: bytes
    type: bytes

class sGcMatchmakingPeriodicTimer(ctypes.Structure):
    _fields_ = [
        ("mTimeLastUpdateRan", ctypes.c_double),
        ("mfRecurrenceTime", ctypes.c_double),
        ("mbPaused", ctypes.c_ubyte),
    ]
    mTimeLastUpdateRan: float
    mfRecurrenceTime: float
    mbPaused: bool

class cGcVoiceChatPlayfab_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnMultiplayerSessionStarting", ctypes.c_ulonglong),
        ("OnMultiplayerSessionFinished", ctypes.c_ulonglong),
        ("OnNetworkPlayerStateEvent", ctypes.c_ulonglong),
        ("OnConstruct", ctypes.c_ulonglong),
        ("OnDestruct", ctypes.c_ulonglong),
        ("OnStartVoice", ctypes.c_ulonglong),
        ("OnStopVoice", ctypes.c_ulonglong),
        ("OnUpdate", ctypes.c_ulonglong),
        ("OnPlayerJoined", ctypes.c_ulonglong),
        ("OnPlayerLeaving", ctypes.c_ulonglong),
        ("OnMute", ctypes.c_ulonglong),
        ("IsMuted", ctypes.c_ulonglong),
        ("CanTalk", ctypes.c_ulonglong),
        ("IsTalking", ctypes.c_ulonglong),
    ]
    OnMultiplayerSessionStarting: int
    OnMultiplayerSessionFinished: int
    OnNetworkPlayerStateEvent: int
    OnConstruct: int
    OnDestruct: int
    OnStartVoice: int
    OnStopVoice: int
    OnUpdate: int
    OnPlayerJoined: int
    OnPlayerLeaving: int
    OnMute: int
    IsMuted: int
    CanTalk: int
    IsTalking: int

class cGcNetworkAddressPlayfab_vtbl(ctypes.Structure):
    pass

class PhSonorityEntry(ctypes.Structure):
    _fields_ = [
        ("mLetter", ctypes.c_char),
        ("mLevel", ctypes.c_char),
    ]
    mLetter: str
    mLevel: str

class ComponentInstancercGcGrabPlayerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcFleetHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcFleetHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcEncounterStateComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcGroundWormComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcRegionPlanet_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
        ("PostPolygoniseThread", ctypes.c_ulonglong),
        ("CancelPolygonise", ctypes.c_ulonglong),
        ("PreAsyncKnowledgeBuild", ctypes.c_ulonglong),
        ("CanPolygonise", ctypes.c_ulonglong),
        ("ThreadedGenerateVoxels", ctypes.c_ulonglong),
        ("ThreadedPolygonise", ctypes.c_ulonglong),
        ("ThreadedKnowledgeBuild", ctypes.c_ulonglong),
        ("GetFacingTriangleStart", ctypes.c_ulonglong),
        ("GetTileBlendTriangleStart", ctypes.c_ulonglong),
        ("DebugUpdate", ctypes.c_ulonglong),
        ("CreatureUpdate", ctypes.c_ulonglong),
        ("PollToMapStreams", ctypes.c_ulonglong),
        ("OnDistanceChanged", ctypes.c_ulonglong),
        ("SetHidden", ctypes.c_ulonglong),
        ("GetGroundPhysics", ctypes.c_ulonglong),
        ("FindInstanceWithRigidBodyNode", ctypes.c_ulonglong),
        ("RemoveInstanceWithRigidBodyNode", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcRegionTerrain *this, TkID<128> *result, cTkRigidBodyPtr)
        ("RemoveInstancesWithinArea", ctypes.c_ulonglong),
        ("SetInstanceActivating", ctypes.c_ulonglong),
        ("GetFoliage", ctypes.c_ulonglong),
        ("GetBuildings", ctypes.c_ulonglong),
        ("GetCreatures", ctypes.c_ulonglong),
        ("GetCreatures_158", ctypes.c_ulonglong),
        ("GetNodeName", ctypes.c_ulonglong),
        ("PartitionTriangles", ctypes.c_ulonglong),
        ("PartitionTrianglesByTile", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int
    PostPolygoniseThread: int
    CancelPolygonise: int
    PreAsyncKnowledgeBuild: int
    CanPolygonise: int
    ThreadedGenerateVoxels: int
    ThreadedPolygonise: int
    ThreadedKnowledgeBuild: int
    GetFacingTriangleStart: int
    GetTileBlendTriangleStart: int
    DebugUpdate: int
    CreatureUpdate: int
    PollToMapStreams: int
    OnDistanceChanged: int
    SetHidden: int
    GetGroundPhysics: int
    FindInstanceWithRigidBodyNode: int
    RemoveInstanceWithRigidBodyNode: bytes
    RemoveInstancesWithinArea: int
    SetInstanceActivating: int
    GetFoliage: int
    GetBuildings: int
    GetCreatures: int
    GetCreatures_158: int
    GetNodeName: int
    PartitionTriangles: int
    PartitionTrianglesByTile: int

class ComponentInstancercGcCombatEffectsComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcPlayerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcChairComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcBuildingComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcTurretComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcBaseDefenceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcAtmosphereEntryComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcAntagonistComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcAlienPodComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcDissolveEffectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcSpawnedObjectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcObjectSpawnerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcNetworkInterpolationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcAsteroidPatternGenerator(ctypes.Structure):
    def Generate(): pass


class ComponentInstancercGcNPCHabitationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentLoadInstancercGcNPCHabitationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class MktSearchResult(ctypes.Structure):
    _fields_ = [
        ("mValue", ctypes.c_char),
        ("mDecision", ctypes.c_char),
    ]
    mValue: str
    mDecision: str

class ComponentInstancercGcHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class MktFrequencyEntry(ctypes.Structure):
    _fields_ = [
        ("mLetter", ctypes.c_char),
        ("mWeight", ctypes.c_float),
    ]
    mLetter: str
    mWeight: float

class TrigramString(ctypes.Structure):
    def TrigramString(): pass

    _fields_ = [
        ("mString", ctypes.c_char * 0x40),  # cTkFixedString<64,char>
    ]
    mString: bytes

class ComponentLoadInstancercGcHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class DiacriticTransforms(ctypes.Structure):
    _fields_ = [
        ("mcaTransforms", ctypes.c_char_p),
        ("miCount", ctypes.c_uint64),
    ]
    mcaTransforms: bytes
    miCount: int

class ComponentInstancercGcAudioAreaTriggerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcAudioAreaTriggerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcSpawnedObjectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcDissolveEffectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcFrontendPageClaimBase(ctypes.Structure):
    def DoClaimBaseInteraction(): pass

    def ShouldDoSeason9CustomPuzzle(): pass

    def DoClaimBaseAnywhereInteraction(): pass

    def DoBaseClaimOptions(): pass

    def IsPositionOverlappingPlanetBase(): pass


class cGcFrontendPageCraftingTree(ctypes.Structure):
    def ShouldRequestCraftingTree(): pass

    def DoCraftingTree(): pass

    def FindNodeToHighlight(): pass

    def BuildTreeItem(): pass

    def UpdateTreeNode(): pass

    def DoPopUp(): pass


class cGcCreatureAttractorComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCostManager(ctypes.Structure):
    def CanAfford(): pass

    def CostAsString(): pass

    def DisplayCost(): pass

    def GetCostHiddenInfo(): pass

    def ApplyCost(): pass

    def CalcCurrentPirateTribute(): pass

    def GetMoneyListCost(): pass

    def IsCostCurrency(): pass

    def IsCostProdOrSubstance(): pass

    def GetItemFromIndexedList(): pass

    def GetProductFromCost(): pass

    def GetSettlementBuildingCostInfoAndStringForBuilding(): pass


class cGcAlienPodComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcDifficultySettingsConfirmHandlerFrontend_vtbl(ctypes.Structure):
    _fields_ = [
        ("DoConfirm", ctypes.c_ulonglong),
    ]
    DoConfirm: int

class ComponentInstancercGcSentinelCoverComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcScanEffectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcObjectPlacementComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcLookAtComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcLandingHelperComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcLadderComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcLadderComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcLandingHelperComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcAtmosphereEntryComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcLookAtComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcBuildingComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcSentinelQuad_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcSentinelRobot_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("GetTree", ctypes.c_ulonglong),
        ("GetCreatureType", ctypes.c_ulonglong),
        ("GetOwnerConcept", ctypes.c_ulonglong),
        ("GetWeapon", ctypes.c_ulonglong),
        ("FireWeapon", ctypes.c_ulonglong),
        ("GetTargetPoint", ctypes.c_ulonglong),
        ("GetForward", ctypes.c_ulonglong),
        ("GetLabelPoint", ctypes.c_ulonglong),
        ("GetTargetPointSize", ctypes.c_ulonglong),
        ("GetSightPoint", ctypes.c_ulonglong),
        ("GetObb", ctypes.c_ulonglong),
        ("GetEffectAttachLocator", ctypes.c_ulonglong),
        ("IsAttacking", ctypes.c_ulonglong),
        ("IsAlert", ctypes.c_ulonglong),
        ("IsDead", ctypes.c_ulonglong),
        ("GetAttackTarget", ctypes.c_ulonglong),
        ("GetAlertRange", ctypes.c_ulonglong),
        ("GetSightRange", ctypes.c_ulonglong),
        ("GetSightAngle", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("OnHit", ctypes.c_ulonglong),
        ("OnArmourHit", ctypes.c_ulonglong),
        ("OnDeath", ctypes.c_ulonglong),
    ]
    cGcSentinelRobot_dtor_0: int
    Prepare: int
    Release: int
    Update: int
    NetSerialize: int
    GetTree: int
    GetCreatureType: int
    GetOwnerConcept: int
    GetWeapon: int
    FireWeapon: int
    GetTargetPoint: int
    GetForward: int
    GetLabelPoint: int
    GetTargetPointSize: int
    GetSightPoint: int
    GetObb: int
    GetEffectAttachLocator: int
    IsAttacking: int
    IsAlert: int
    IsDead: int
    GetAttackTarget: int
    GetAlertRange: int
    GetSightRange: int
    GetSightAngle: int
    RenderNGui: int
    OnHit: int
    OnArmourHit: int
    OnDeath: int

class PFLobbySearchFriendsFilter(ctypes.Structure):
    _fields_ = [
        ("includeSteamFriends", ctypes.c_ubyte),
        ("includeFacebookFriends", ctypes.c_ubyte),
        ("includeXboxFriendsToken", ctypes.c_char_p),
    ]
    includeSteamFriends: bool
    includeFacebookFriends: bool
    includeXboxFriendsToken: bytes

class PFLobbySearchConfiguration(ctypes.Structure):
    _fields_ = [
        ("friendsFilter", ctypes.c_ulonglong),
        ("filterString", ctypes.c_char_p),
        ("sortString", ctypes.c_char_p),
        ("clientSearchResultCount", ctypes.c_ulonglong),
    ]
    friendsFilter: int
    filterString: bytes
    sortString: bytes
    clientSearchResultCount: int

class PFLobbyJoinConfiguration(ctypes.Structure):
    _fields_ = [
        ("memberPropertyCount", ctypes.c_uint32),
        ("memberPropertyKeys", ctypes.c_ulonglong),
        ("memberPropertyValues", ctypes.c_ulonglong),
    ]
    memberPropertyCount: int
    memberPropertyKeys: int
    memberPropertyValues: int

class cGcSentinelWalker_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcSentinelRobot_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("GetTree", ctypes.c_ulonglong),
        ("GetCreatureType", ctypes.c_ulonglong),
        ("GetOwnerConcept", ctypes.c_ulonglong),
        ("GetWeapon", ctypes.c_ulonglong),
        ("FireWeapon", ctypes.c_ulonglong),
        ("GetTargetPoint", ctypes.c_ulonglong),
        ("GetForward", ctypes.c_ulonglong),
        ("GetLabelPoint", ctypes.c_ulonglong),
        ("GetTargetPointSize", ctypes.c_ulonglong),
        ("GetSightPoint", ctypes.c_ulonglong),
        ("GetObb", ctypes.c_ulonglong),
        ("GetEffectAttachLocator", ctypes.c_ulonglong),
        ("IsAttacking", ctypes.c_ulonglong),
        ("IsAlert", ctypes.c_ulonglong),
        ("IsDead", ctypes.c_ulonglong),
        ("GetAttackTarget", ctypes.c_ulonglong),
        ("GetAlertRange", ctypes.c_ulonglong),
        ("GetSightRange", ctypes.c_ulonglong),
        ("GetSightAngle", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("OnHit", ctypes.c_ulonglong),
        ("OnArmourHit", ctypes.c_ulonglong),
        ("OnDeath", ctypes.c_ulonglong),
    ]
    cGcSentinelRobot_dtor_0: int
    Prepare: int
    Release: int
    Update: int
    NetSerialize: int
    GetTree: int
    GetCreatureType: int
    GetOwnerConcept: int
    GetWeapon: int
    FireWeapon: int
    GetTargetPoint: int
    GetForward: int
    GetLabelPoint: int
    GetTargetPointSize: int
    GetSightPoint: int
    GetObb: int
    GetEffectAttachLocator: int
    IsAttacking: int
    IsAlert: int
    IsDead: int
    GetAttackTarget: int
    GetAlertRange: int
    GetSightRange: int
    GetSightAngle: int
    RenderNGui: int
    OnHit: int
    OnArmourHit: int
    OnDeath: int

class cGcSentinelMechWeapon_vtbl(ctypes.Structure):
    _fields_ = [
        ("UsingAmmo", ctypes.c_ulonglong),
        ("UsesInventoryAmmo", ctypes.c_ulonglong),
        ("GetAvailableAmmo", ctypes.c_ulonglong),
        ("GetClipSize", ctypes.c_ulonglong),
        ("IsReloading", ctypes.c_ulonglong),
        ("GetReloadProgress", ctypes.c_ulonglong),
        ("GetLastFireTime", ctypes.c_ulonglong),
        ("HasOverheated", ctypes.c_ulonglong),
        ("GetOverHeatTime", ctypes.c_ulonglong),
        ("GetOverheatProgress", ctypes.c_ulonglong),
        ("IsOnCooldown", ctypes.c_ulonglong),
        ("GetCooldownDuration", ctypes.c_ulonglong),
        ("GetCooldownProgress", ctypes.c_ulonglong),
        ("GetAmmoInClip", ctypes.c_ulonglong),
        ("GetInventoryItem", ctypes.c_ulonglong),
        ("GetActiveInventoryItem", ctypes.c_ulonglong),
        ("GetActiveSecondaryInventoryItem", ctypes.c_ulonglong),
        ("GetHeatFactor", ctypes.c_ulonglong),
        ("GetHeatWarningFactor", ctypes.c_ulonglong),
        ("GetAlertPoint", ctypes.c_ulonglong),
        ("GetCooldownFactor", ctypes.c_ulonglong),
        ("GetModeIndex", ctypes.c_ulonglong),
        ("GetLastModeChangeTime", ctypes.c_ulonglong),
        ("GetNumModes", ctypes.c_ulonglong),
        ("SetModeIndex", ctypes.c_ulonglong),
        ("GetWeaponRange", ctypes.c_ulonglong),
        ("GetWeaponSpeed", ctypes.c_ulonglong),
        ("GetPrimaryStat", ctypes.c_ulonglong),
        ("IsCriticalShot", ctypes.c_ulonglong),
        ("GetListOfModesIfActiveOrNot", ctypes.c_char * 0x8),  # std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *(__fastcall *)(cGcWeapon *this, std::vector<bool,TkSTLAllocatorShim<bool,1,-1> > *result)
        ("GetChargeFactor", ctypes.c_ulonglong),
        ("IsCharging", ctypes.c_ulonglong),
        ("IsActive", ctypes.c_ulonglong),
        ("IsVehicleWeapon", ctypes.c_ulonglong),
        ("GetTerrainEditMode", ctypes.c_ulonglong),
        ("GetTerrainEditorBeam", ctypes.c_ulonglong),
        ("UsingTerrainAdvancedMode", ctypes.c_ulonglong),
        ("ShowingTerrainEditNotifies", ctypes.c_ulonglong),
        ("ResetTerrainMode", ctypes.c_ulonglong),
        ("CanCharge", ctypes.c_ulonglong),
    ]
    UsingAmmo: int
    UsesInventoryAmmo: int
    GetAvailableAmmo: int
    GetClipSize: int
    IsReloading: int
    GetReloadProgress: int
    GetLastFireTime: int
    HasOverheated: int
    GetOverHeatTime: int
    GetOverheatProgress: int
    IsOnCooldown: int
    GetCooldownDuration: int
    GetCooldownProgress: int
    GetAmmoInClip: int
    GetInventoryItem: int
    GetActiveInventoryItem: int
    GetActiveSecondaryInventoryItem: int
    GetHeatFactor: int
    GetHeatWarningFactor: int
    GetAlertPoint: int
    GetCooldownFactor: int
    GetModeIndex: int
    GetLastModeChangeTime: int
    GetNumModes: int
    SetModeIndex: int
    GetWeaponRange: int
    GetWeaponSpeed: int
    GetPrimaryStat: int
    IsCriticalShot: int
    GetListOfModesIfActiveOrNot: bytes
    GetChargeFactor: int
    IsCharging: int
    IsActive: int
    IsVehicleWeapon: int
    GetTerrainEditMode: int
    GetTerrainEditorBeam: int
    UsingTerrainAdvancedMode: int
    ShowingTerrainEditNotifies: int
    ResetTerrainMode: int
    CanCharge: int

class cGcSentinelMech_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcSentinelRobot_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("GetTree", ctypes.c_ulonglong),
        ("GetCreatureType", ctypes.c_ulonglong),
        ("GetOwnerConcept", ctypes.c_ulonglong),
        ("GetWeapon", ctypes.c_ulonglong),
        ("FireWeapon", ctypes.c_ulonglong),
        ("GetTargetPoint", ctypes.c_ulonglong),
        ("GetForward", ctypes.c_ulonglong),
        ("GetLabelPoint", ctypes.c_ulonglong),
        ("GetTargetPointSize", ctypes.c_ulonglong),
        ("GetSightPoint", ctypes.c_ulonglong),
        ("GetObb", ctypes.c_ulonglong),
        ("GetEffectAttachLocator", ctypes.c_ulonglong),
        ("IsAttacking", ctypes.c_ulonglong),
        ("IsAlert", ctypes.c_ulonglong),
        ("IsDead", ctypes.c_ulonglong),
        ("GetAttackTarget", ctypes.c_ulonglong),
        ("GetAlertRange", ctypes.c_ulonglong),
        ("GetSightRange", ctypes.c_ulonglong),
        ("GetSightAngle", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("OnHit", ctypes.c_ulonglong),
        ("OnArmourHit", ctypes.c_ulonglong),
        ("OnDeath", ctypes.c_ulonglong),
    ]
    cGcSentinelRobot_dtor_0: int
    Prepare: int
    Release: int
    Update: int
    NetSerialize: int
    GetTree: int
    GetCreatureType: int
    GetOwnerConcept: int
    GetWeapon: int
    FireWeapon: int
    GetTargetPoint: int
    GetForward: int
    GetLabelPoint: int
    GetTargetPointSize: int
    GetSightPoint: int
    GetObb: int
    GetEffectAttachLocator: int
    IsAttacking: int
    IsAlert: int
    IsDead: int
    GetAttackTarget: int
    GetAlertRange: int
    GetSightRange: int
    GetSightAngle: int
    RenderNGui: int
    OnHit: int
    OnArmourHit: int
    OnDeath: int

class cGcGroundWormComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcSentinelCoverComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcFleetHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcGrabPlayerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcNotificationTextHelper(ctypes.Structure):
    def AmountMoreString(): pass

    def GetMarkedUpItemString(): pass

    def GetMarkedUpItemStringWithIcon(): pass

    def GetRequirementsString(): pass

    def GetGPSString(): pass

    def GetSettlementConstructionLevelLocID(): pass


class ComponentInstancercGcCreatureSpawnComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcSimpleIkRecoilComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcPlayerControlComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcDecorationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class PFMatchmakingTicketConfiguration(ctypes.Structure):
    _fields_ = [
        ("timeoutInSeconds", ctypes.c_uint32),
        ("queueName", ctypes.c_char_p),
        ("membersToMatchWithCount", ctypes.c_uint32),
        ("membersToMatchWith", ctypes.c_ulonglong),
    ]
    timeoutInSeconds: int
    queueName: bytes
    membersToMatchWithCount: int
    membersToMatchWith: int

class ComponentInstancercGcColouriseComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcCameraShakeComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcCameraShakeComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcMetadataBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcMetadataBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcMetadataBehaviourAllocator(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mDataClassNameHash64", ctypes.c_uint64),
        ("mpNext", ctypes.c_ulonglong),
    ]
    __vftable: int
    mDataClassNameHash64: int
    mpNext: int

class cGcMetadataBehaviourAllocator_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourLook_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class cGcDecorationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class BehaviourAllocatorcGcBehaviourIdle_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourApplyDamage_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourPlayAnim_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourWait_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourLaunchProjectile_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourMoveToTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourRegisterAttacker_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourFaceTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourMaintainRangeFromTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class BehaviourAllocatorcGcBehaviourGetTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class cGcCreatureBaitComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcAdjustHeight_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCircleAttractor_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcHerbivoreFeed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcHerbivoreFlee_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcHerbivoreStop_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class PFMultiplayerServerDetails(ctypes.Structure):
    _fields_ = [
        ("fqdn", ctypes.c_char_p),
        ("ipv4Address", ctypes.c_char_p),
        ("ports", ctypes.c_ulonglong),
        ("portCount", ctypes.c_uint32),
        ("region", ctypes.c_char_p),
    ]
    fqdn: bytes
    ipv4Address: bytes
    ports: int
    portCount: int
    region: bytes

class PFMatchmakingMatchDetails(ctypes.Structure):
    _fields_ = [
        ("matchId", ctypes.c_char_p),
        ("members", ctypes.c_ulonglong),
        ("memberCount", ctypes.c_uint32),
        ("regionPreferences", ctypes.c_ulonglong),
        ("regionPreferenceCount", ctypes.c_uint32),
        ("lobbyArrangementString", ctypes.c_char_p),
        ("serverDetails", ctypes.c_ulonglong),
    ]
    matchId: bytes
    members: int
    memberCount: int
    regionPreferences: int
    regionPreferenceCount: int
    lobbyArrangementString: bytes
    serverDetails: int

class PFLobbyDataUpdate(ctypes.Structure):
    _fields_ = [
        ("newOwner", ctypes.c_ulonglong),
        ("maxMemberCount", ctypes.c_ulonglong),
        ("accessPolicy", ctypes.c_ulonglong),
        ("membershipLock", ctypes.c_ulonglong),
        ("searchPropertyCount", ctypes.c_uint32),
        ("searchPropertyKeys", ctypes.c_ulonglong),
        ("searchPropertyValues", ctypes.c_ulonglong),
        ("lobbyPropertyCount", ctypes.c_uint32),
        ("lobbyPropertyKeys", ctypes.c_ulonglong),
        ("lobbyPropertyValues", ctypes.c_ulonglong),
    ]
    newOwner: int
    maxMemberCount: int
    accessPolicy: int
    membershipLock: int
    searchPropertyCount: int
    searchPropertyKeys: int
    searchPropertyValues: int
    lobbyPropertyCount: int
    lobbyPropertyKeys: int
    lobbyPropertyValues: int

class PFLobbyMemberDataUpdate(ctypes.Structure):
    _fields_ = [
        ("memberPropertyCount", ctypes.c_uint32),
        ("memberPropertyKeys", ctypes.c_ulonglong),
        ("memberPropertyValues", ctypes.c_ulonglong),
    ]
    memberPropertyCount: int
    memberPropertyKeys: int
    memberPropertyValues: int

class cGcBlackboardValueDecorator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int

class cGcBehaviourGetTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourRegisterAttacker_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourMoveToTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourMaintainRangeFromTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourFaceTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourPlayAnim_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourApplyDamage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourLaunchProjectile_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourWait_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcBehaviourIdle_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcSimpleIkRecoilComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcWorldMap_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcRenderableScreen_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("CleanupResources", ctypes.c_ulonglong),
    ]
    cGcRenderableScreen_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    EnqueueRender: int
    UpdateRender: int
    Release: int
    Destruct: int
    RenderNGui: int
    CleanupResources: int

class cGcFiendBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcFiendBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCanFiendEngage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcFiendHunt_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcFiendPatrol_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcFiendStunned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMiniDroneHunt_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcPredatorAttack_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcPredatorHunt_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcScuttlerHunt_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcSharkAttack_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcSharkHunt_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcLandDolphin_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcSnakePatrol_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcEncounterStateComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcEncounterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class sRegionStreamInfo(ctypes.Structure):
    _fields_ = [
        ("mpRegion", ctypes.c_ulonglong),
        ("mfCost", ctypes.c_float),
    ]
    mpRegion: int
    mfCost: float

class cGcAttractor(ctypes.Structure):
    def Attract(): pass

    def Align(): pass

    _fields_ = [
        ("mfMaxSpeed", ctypes.c_float),
        ("mfAttractForce", ctypes.c_float),
        ("mfTurnStrength", ctypes.c_float),
    ]
    mfMaxSpeed: float
    mfAttractForce: float
    mfTurnStrength: float

class sGenerateRegionParams(ctypes.Structure):
    _fields_ = [
        ("miNumVoxelsX", ctypes.c_int32),
        ("miNumVoxelsY", ctypes.c_int32),
        ("miNumVoxelsZ", ctypes.c_int32),
        ("mfScaleX", ctypes.c_float),
        ("mfScaleY", ctypes.c_float),
        ("mfScaleZ", ctypes.c_float),
        ("mfRegionScaleX", ctypes.c_float),
        ("mfRegionScaleY", ctypes.c_float),
        ("mfRegionScaleZ", ctypes.c_float),
        ("miTotalYScale", ctypes.c_int32),
        ("mfRegionSpaceHeight", ctypes.c_float),
        ("miBaseHeight", ctypes.c_int32),
        ("miRegionBorder", ctypes.c_int32),
        ("mfVoxelHeight", ctypes.c_float),
        ("miMaxHeight", ctypes.c_int32),
        ("miGenerationLOD", ctypes.c_int32),
    ]
    miNumVoxelsX: int
    miNumVoxelsY: int
    miNumVoxelsZ: int
    mfScaleX: float
    mfScaleY: float
    mfScaleZ: float
    mfRegionScaleX: float
    mfRegionScaleY: float
    mfRegionScaleZ: float
    miTotalYScale: int
    mfRegionSpaceHeight: float
    miBaseHeight: int
    miRegionBorder: int
    mfVoxelHeight: float
    miMaxHeight: int
    miGenerationLOD: int

class cGcNotificationHelper(ctypes.Structure):
    def GetBuildingPartCountFromType(): pass


class cGcGenericSectionConditionIsScanEventRepaired_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionIsScanEventActive_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionRequestedPhoto_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cGcGenericSectionConditionOnOtherSideOfPortal_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class cTkVoxelGenerationJob(ctypes.Structure):
    _fields_ = [
        ("mpGalaxyVoxelCache", ctypes.c_ulonglong),
        ("mpVoxel", ctypes.c_ulonglong),
    ]
    mpGalaxyVoxelCache: int
    mpVoxel: int

class cGcGenericSectionConditionIsScanEventLocalOrNear_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcGenericSectionCondition_dtor_0", ctypes.c_ulonglong),
        ("IsConditionTrue", ctypes.c_ulonglong),
        ("AllowMissionRestart", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # cTkFixedString<1024,char> *(__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<1024,char> *result)
        ("FormatConditionText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcGenericSectionCondition *this, cTkFixedString<512,char> *, bool)
        ("GetConditionInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ShouldRefreshInventoryHint", ctypes.c_ulonglong),
        ("BlockPirateMessage", ctypes.c_ulonglong),
        ("OverridesMissionSurveyText", ctypes.c_ulonglong),
        ("EncouragesFightingSentinels", ctypes.c_ulonglong),
        ("ResetsUserDataOnActivation", ctypes.c_ulonglong),
        ("GetBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcGenericSectionCondition *this, TkID<128> *result)
        ("ResetUserData", ctypes.c_ulonglong),
    ]
    cGcGenericSectionCondition_dtor_0: int
    IsConditionTrue: int
    AllowMissionRestart: int
    GetDescription: bytes
    FormatConditionText: bytes
    GetConditionInventoryHint: bytes
    ShouldRefreshInventoryHint: int
    BlockPirateMessage: int
    OverridesMissionSurveyText: int
    EncouragesFightingSentinels: int
    ResetsUserDataOnActivation: int
    GetBuildMenuHint: bytes
    ResetUserData: int

class ComponentInstancercGcCreatureLegIKComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcCreatureEggComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcCreatureEffectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcCreatureEffectComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCreatureLegIKComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class GeomProbeTask(ctypes.Structure):
    def GeomProbeTask(): pass

    _fields_ = [
        ("mIndex", ctypes.c_int32),
        ("mCount", ctypes.c_int32),
        ("mDistance", ctypes.c_float),
        ("mCountDelta", ctypes.c_float),
        ("mArcURange", common.Vector2f),
        ("mDebugColour", common.Colour),
    ]
    mIndex: int
    mCount: int
    mDistance: float
    mCountDelta: float
    mArcURange: common.Vector2f
    mDebugColour: common.Colour

class cGcHumanController_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcPlayerController_dtor_0", ctypes.c_ulonglong),
        ("GetButtonInput", ctypes.c_ulonglong),
        ("GetAnalogInput", ctypes.c_ulonglong),
    ]
    cGcPlayerController_dtor_0: int
    GetButtonInput: int
    GetAnalogInput: int

class cGcWeatherEffectUpdate(ctypes.Structure):
    def GenerateResourceAsync(): pass

    def Init(): pass

    def UpdateMeteorHazard(): pass

    def UpdateTornadoHazard(): pass

    def InitLightningHazard(): pass

    def UpdateLightningHazard(): pass

    def UpdateDynamicIndicator(): pass

    def UpdateStaticIndicator(): pass

    def SpawnImpactGift(): pass

    def GetNearbyNetworkPlayerPositions(): pass


class sUIScalingValues(ctypes.Structure):
    def sUIScalingValues(): pass

    _fields_ = [
        ("mScreenDim", ctypes.c_char * 0x8),  # cTkIntTuple2<int,0>
        ("mScreenDimRemap", common.Vector2f),
        ("mfGuiScale", ctypes.c_float),
    ]
    mScreenDim: bytes
    mScreenDimRemap: common.Vector2f
    mfGuiScale: float

class ComponentInstancercGcHeightAdjustComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcHeightAdjustComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcShipSalvageInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcInteraction_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("SendMessageA", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcInteraction *this, TkID<128>, TkID<128>)
        ("DoInteractionEvent", ctypes.c_ulonglong),
        ("GetViewTargetNode", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("RenderOffscreen2D", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
    ]
    cGcInteraction_dtor_0: int
    Construct: int
    Release: int
    Prepare: int
    Update: int
    SendMessageA: bytes
    DoInteractionEvent: int
    GetViewTargetNode: int
    Interact: int
    UpdateRender: int
    RenderOffscreen2D: int
    RenderNGui: int

class cGcCustomiserCharacterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCustomiserInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcInteraction_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("SendMessageA", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcInteraction *this, TkID<128>, TkID<128>)
        ("DoInteractionEvent", ctypes.c_ulonglong),
        ("GetViewTargetNode", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("RenderOffscreen2D", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
    ]
    cGcInteraction_dtor_0: int
    Construct: int
    Release: int
    Prepare: int
    Update: int
    SendMessageA: bytes
    DoInteractionEvent: int
    GetViewTargetNode: int
    Interact: int
    UpdateRender: int
    RenderOffscreen2D: int
    RenderNGui: int

class cGcMPMissionGiverInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcInteraction_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("SendMessageA", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcInteraction *this, TkID<128>, TkID<128>)
        ("DoInteractionEvent", ctypes.c_ulonglong),
        ("GetViewTargetNode", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("RenderOffscreen2D", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
    ]
    cGcInteraction_dtor_0: int
    Construct: int
    Release: int
    Prepare: int
    Update: int
    SendMessageA: bytes
    DoInteractionEvent: int
    GetViewTargetNode: int
    Interact: int
    UpdateRender: int
    RenderOffscreen2D: int
    RenderNGui: int

class cGcSpecialsShopInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcInteraction_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("SendMessageA", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcInteraction *this, TkID<128>, TkID<128>)
        ("DoInteractionEvent", ctypes.c_ulonglong),
        ("GetViewTargetNode", ctypes.c_ulonglong),
        ("Interact", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("RenderOffscreen2D", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
    ]
    cGcInteraction_dtor_0: int
    Construct: int
    Release: int
    Prepare: int
    Update: int
    SendMessageA: bytes
    DoInteractionEvent: int
    GetViewTargetNode: int
    Interact: int
    UpdateRender: int
    RenderOffscreen2D: int
    RenderNGui: int

class cGcShipAIHailData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcBlackboard_dtor_0", ctypes.c_ulonglong),
        ("GetIdentifier", ctypes.c_ulonglong),
    ]
    cGcBlackboard_dtor_0: int
    GetIdentifier: int

class cGcShipAIHasHailReply_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIHasSpecificHailReply_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAISendFightPlayerMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIClearCommunication_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIClearShooter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcNetworkLockMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class BehaviourAllocatorcGcBehaviourDetailAnims_vtbl(ctypes.Structure):
    _fields_ = [
        ("Allocate", ctypes.c_ulonglong),
    ]
    Allocate: int

class cGcNetworkFullSimHandsUpdateMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class sGcNetworkOrbUpdateMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class ComponentInstancercGcCustomiserCharacterComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcNetworkPlayerExperienceSpawnMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcDebugMovementTrail(ctypes.Structure):
    _fields_ = [
        ("mNode", ctypes.c_uint32),
        ("mavTrailPoints", ctypes.c_char * 0x10),  # std::list<cTkVector3,TkSTLAllocatorShim<cTkVector3,16,-1> >
        ("mfPrevPosTime", ctypes.c_float),
        ("mfUpdateInterval", ctypes.c_float),
        ("mPathColour", common.Colour),
        ("mfFadeTime", ctypes.c_float),
        ("mbDepthTest", ctypes.c_ubyte),
    ]
    mNode: int
    mavTrailPoints: bytes
    mfPrevPosTime: float
    mfUpdateInterval: float
    mPathColour: common.Colour
    mfFadeTime: float
    mbDepthTest: bool

class sGcNetworkPlayerStateMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkVoiceUpdateMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class sGcNetworkPlayerReplicatedVariablesMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcAppear_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcAttracted_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcAvoidObstacle_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcBooleanBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCanFollowPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCanSeePlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCanSeePrey_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcPetBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcPetFinishExternalBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcPetInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcPetRoam_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCanSenseAttraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class ComponentInstancercGcMissileComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcSpaceshipShieldComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcChatterBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCheckLeaderCommands_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCheckTimer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcCreatureStunned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDamageNearby_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDetailAnims_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcBehaviourDetailAnims_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcFollowLeaderCommands_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcFollowPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcHerbivoreIdle_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class GcFollowRoutineGround_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class GcFollowWaypoints_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcGetNearestPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class GcGrazeFollowingRoutine_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcLookAtStuff_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcBehaviourLook_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("NGuiRenderImpl", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    NGuiRenderImpl: int

class cGcMatchAnimToSpeed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMoveToTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcPlayerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcPlotPathToPosition_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcRidden_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcSetBooleanTo_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcShouldAvoidPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class GcSimpleFollowRoutine_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcTargetRandomPosition_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class sNodeCache(ctypes.Structure):
    _fields_ = [
        ("mNode", ctypes.c_uint32),
        ("mbState", ctypes.c_ubyte),
    ]
    mNode: int
    mbState: bool

class DDSHeaderSizedArray(ctypes.Structure):
    _fields_ = [
        ("lBuf", ctypes.c_uint8 * 0x7C),  # unsigned __int8[124]
    ]
    lBuf: list[ctypes.c_uint8]

class DDSHeader(ctypes.Structure):
    _fields_ = [
        ("muMagic", ctypes.c_uint32),
        ("muSize", ctypes.c_uint32),
        ("muFlags", ctypes.c_uint32),
        ("muHeight", ctypes.c_uint32),
        ("muWidth", ctypes.c_uint32),
        ("muPitchOrLinearSize", ctypes.c_uint32),
        ("muDepth", ctypes.c_uint32),
        ("muMipMapCount", ctypes.c_uint32),
        ("mauReserved1", ctypes.c_uint32 * 0xB),  # unsigned int[11]
        ("mPixFormat", ctypes.c_char * 0x20),  # DDSHeader::<unnamed_type_mPixFormat>
        ("mCaps", ctypes.c_char * 0x10),  # DDSHeader::<unnamed_type_mCaps>
        ("muReserved2", ctypes.c_uint32),
    ]
    muMagic: int
    muSize: int
    muFlags: int
    muHeight: int
    muWidth: int
    muPitchOrLinearSize: int
    muDepth: int
    muMipMapCount: int
    mauReserved1: list[ctypes.c_uint32]
    mPixFormat: bytes
    mCaps: bytes
    muReserved2: int

class cGcRegionGeneratorRazorContext(ctypes.Structure):
    pass

class DDSHeaderDx10(ctypes.Structure):
    _fields_ = [
        ("muDxgiFormat", ctypes.c_uint32),
        ("muResourceDimension", ctypes.c_uint32),
        ("muMiscFlag", ctypes.c_uint32),
        ("muArraySize", ctypes.c_uint32),
        ("muMiscFlags2", ctypes.c_uint32),
    ]
    muDxgiFormat: int
    muResourceDimension: int
    muMiscFlag: int
    muArraySize: int
    muMiscFlags2: int

class cGcRegionCircleIterator(ctypes.Structure):
    _fields_ = [
        ("miDimension", ctypes.c_int32),
        ("miRowDimension", ctypes.c_int32),
        ("miColumnDimension", ctypes.c_int32),
        ("miRow", ctypes.c_int32),
        ("miColumn", ctypes.c_int32),
        ("miSign1", ctypes.c_int32),
        ("miSign2", ctypes.c_int32),
        ("miStep", ctypes.c_int32),
        ("mbRow", ctypes.c_ubyte),
    ]
    miDimension: int
    miRowDimension: int
    miColumnDimension: int
    miRow: int
    miColumn: int
    miSign1: int
    miSign2: int
    miStep: int
    mbRow: bool

class sFindPersistingRegionsJobParams(ctypes.Structure):
    _fields_ = [
        ("mpRegionMap", ctypes.c_ulonglong),
        ("miStatus", ctypes.c_char * 0x4),  # std::atomic<int>
        ("mppRegionsToClear", ctypes.c_ulonglong),
        ("miNumRegionsToClear", ctypes.c_int32),
        ("miMaxRegionsToClear", ctypes.c_int32),
        ("miLod", ctypes.c_int32),
    ]
    mpRegionMap: int
    miStatus: bytes
    mppRegionsToClear: int
    miNumRegionsToClear: int
    miMaxRegionsToClear: int
    miLod: int

class OnPlayerNear_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sOnPlayerNearType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SpawnParticles_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSpawnParticlesType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetParticlesStage_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSetParticlesStageType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class StopParticles_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sStopParticlesType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class DestroyDestructables_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sDestroyDestructablesType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetHazard_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSetHazardType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class DamagePlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sDamagePlayerType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class CameraShake_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sCameraShakeType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class StopCameraShake_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sStopCameraShakeType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class FadeMaterials_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class SortableIndex(ctypes.Structure):
    _fields_ = [
        ("miX", ctypes.c_int32),
        ("miY", ctypes.c_int32),
        ("miZ", ctypes.c_int32),
        ("mfDist", ctypes.c_float),
    ]
    miX: int
    miY: int
    miZ: int
    mfDist: float

class sFadeMaterialsType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetMaterialUniform_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSetMaterialUniformType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class SetShaderValue_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sSetShaderValueType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class GiveReward_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sGiveRewardType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class AddShootableHealth_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sAddShootableHealthType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class CameraState(ctypes.Structure):
    _fields_ = [
        ("mbBusy", ctypes.c_ubyte),
        ("mbLocked", ctypes.c_ubyte),
    ]
    mbBusy: bool
    mbLocked: bool

class GetPlanetColour_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class sGetPlanetColourType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class GetTimeOfDay_vtbl(ctypes.Structure):
    _fields_ = [
        ("sNode_dtor_0", ctypes.c_ulonglong),
        ("GetSizeBytes", ctypes.c_ulonglong),
        ("GetSaveAddress", ctypes.c_ulonglong),
        ("GetSaveSizeBytes", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("GetOutput", ctypes.c_ulonglong),
        ("CheckTrigger", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
    ]
    sNode_dtor_0: int
    GetSizeBytes: int
    GetSaveAddress: int
    GetSaveSizeBytes: int
    GetName: int
    Start: int
    GetOutput: int
    CheckTrigger: int
    RenderNGui: int
    Initialise: int

class Tokenizer(ctypes.Structure):
    def seekChar(): pass

    def getNextToken(): pass

    def checkToken(): pass

    def getToken(): pass

    _fields_ = [
        ("mcToken", ctypes.c_char * 0x80),  # char[128]
        ("mcPrevToken", ctypes.c_char * 0x80),  # char[128]
        ("mpcPointer", ctypes.c_char_p),
        ("miLine", ctypes.c_int32),
    ]
    mcToken: list[ctypes.c_char]
    mcPrevToken: list[ctypes.c_char]
    mpcPointer: bytes
    miLine: int

class sGetTimeOfDayType_vtbl(ctypes.Structure):
    _fields_ = [
        ("GetName", ctypes.c_ulonglong),
        ("ConstructNode", ctypes.c_ulonglong),
    ]
    GetName: int
    ConstructNode: int

class MarkerSortIndex(ctypes.Structure):
    def SortMarkersOnUA(): pass

    _fields_ = [
        ("mUA", ctypes.c_uint64),
        ("muIndex", ctypes.c_uint32),
        ("miType", ctypes.c_int32),
        ("miVariant", ctypes.c_int32),
    ]
    mUA: int
    muIndex: int
    miType: int
    miVariant: int

class ComponentInstancercGcFreighterSyncComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcShipFlareComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcShipAISquadronFormationAttack_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAISquadronFormationMove_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcFlybyFrigateBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcPlayerSquadronSpaceshipBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcCrashedShipTakeOff_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcSpaceCrashedShipBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcShipAIDefenceForceChooseTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcSpaceDefenceForceBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcTargetTrack(ctypes.Structure):
    _fields_ = [
        ("mpTarget", ctypes.c_uint64),
        ("mfAlertness", ctypes.c_float),
    ]
    mpTarget: int
    mfAlertness: float

class cGcFreighterData_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcBlackboard_dtor_0", ctypes.c_ulonglong),
        ("GetIdentifier", ctypes.c_ulonglong),
    ]
    cGcBlackboard_dtor_0: int
    GetIdentifier: int

class cGcSpaceFreighterBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcShipAIPirateChooseTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcSpacePirateBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class ComponentInstancercGcWiringSocketComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcSpacePlayerFreighterBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcShipAIPoliceChooseTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIPoliceReturnToBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcSpacePoliceBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcPlayerInteractLabel(ctypes.Structure):
    def SetMarkerLabel(): pass


class cGcShipAICheckDeath_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcSpaceTraderBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("Hit", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    Hit: int

class cGcShipFlareComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class sPartWithThemedVariants(ctypes.Structure):
    _fields_ = [
        ("mIDs", ctypes.c_ulonglong * 0x4),  # TkID<128>[4]
    ]
    mIDs: list[ctypes.c_ulonglong]

class cEgSceneGraphResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("ParseData", ctypes.c_ulonglong),
        ("GetAttribute", ctypes.c_char * 0x8),  # char *(__fastcall *)(cEgSceneGraphResource *this, TkID<128>, const cTkDynamicArray<cTkSceneNodeAttributeData> *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneGraphResource *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int
    ParseData: int
    GetAttribute: bytes
    GatherData: bytes

class cEgVertexNodeQueueEntry(ctypes.Structure):
    _fields_ = [
        ("muIndexBuffer", ctypes.c_uint32),
        ("muVertexBuffer", ctypes.c_uint32),
        ("mpSparseNearDiffTexture", ctypes.c_ulonglong),
        ("mpSparseNearNormTexture", ctypes.c_ulonglong),
        ("mpSparseFarDiffTexture", ctypes.c_ulonglong),
        ("mpSparseFarNormTexture", ctypes.c_ulonglong),
        ("mpStatusMapNearTexture", ctypes.c_ulonglong),
        ("mpStatusMapFarTexture", ctypes.c_ulonglong),
        ("mpMipFeedbackTexture", ctypes.c_ulonglong),
        ("mpTexCacheParams", ctypes.c_ulonglong),
        ("miNumTexCacheParams", ctypes.c_int32),
        ("miDrawCounterIndex", ctypes.c_int32),
        ("miNearFeedbackIndex", ctypes.c_uint8),
        ("miFarFeedbackIndex", ctypes.c_uint8),
        ("mbAllTexturesPresent", ctypes.c_ubyte),
    ]
    muIndexBuffer: int
    muVertexBuffer: int
    mpSparseNearDiffTexture: int
    mpSparseNearNormTexture: int
    mpSparseFarDiffTexture: int
    mpSparseFarNormTexture: int
    mpStatusMapNearTexture: int
    mpStatusMapFarTexture: int
    mpMipFeedbackTexture: int
    mpTexCacheParams: int
    miNumTexCacheParams: int
    miDrawCounterIndex: int
    miNearFeedbackIndex: int
    miFarFeedbackIndex: int
    mbAllTexturesPresent: bool

class cEgSingleLineNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cGcLocalPlayerCharacterInterface_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCharacterInterface_dtor_0", ctypes.c_ulonglong),
        ("IsPlayer", ctypes.c_ulonglong),
        ("IsReady", ctypes.c_ulonglong),
        ("IsLocalPlayer", ctypes.c_ulonglong),
        ("IsVRPlayer", ctypes.c_ulonglong),
        ("WeaponIsActive", ctypes.c_ulonglong),
        ("WeaponIsLarge", ctypes.c_ulonglong),
        ("BinocsActive", ctypes.c_ulonglong),
        ("IsHitReacting", ctypes.c_ulonglong),
        ("GetHitReactDir", ctypes.c_ulonglong),
        ("IsOnGround", ctypes.c_ulonglong),
        ("IsInSpace", ctypes.c_ulonglong),
        ("ShouldUseAbandonedFreighterWalk", ctypes.c_ulonglong),
        ("IsDying", ctypes.c_ulonglong),
        ("GetDeathReason", ctypes.c_ulonglong),
        ("GetEnergy", ctypes.c_ulonglong),
        ("GetHazard", ctypes.c_ulonglong),
        ("IsJetpacking", ctypes.c_ulonglong),
        ("IsJetpackBoosted", ctypes.c_ulonglong),
        ("IsSliding", ctypes.c_ulonglong),
        ("IsRiding", ctypes.c_ulonglong),
        ("GetRidingAttachment", ctypes.c_ulonglong),
        ("GetHandPose", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcCharacterInterface *this, eHand, TkID<128> *, float *)
        ("GetDominantHand", ctypes.c_ulonglong),
        ("IsMeleeActive", ctypes.c_ulonglong),
        ("IsPointing", ctypes.c_ulonglong),
        ("IsInteracting", ctypes.c_ulonglong),
        ("IsMeleeBoostActive", ctypes.c_ulonglong),
        ("IsWaitingForRocketBootsActivation", ctypes.c_ulonglong),
        ("IsRocketBootsActive", ctypes.c_ulonglong),
        ("GetLocalTeleportDestination", ctypes.c_ulonglong),
        ("GetTimeLastUsedJetpack", ctypes.c_ulonglong),
        ("IsRunning", ctypes.c_ulonglong),
        ("IsUnderwater", ctypes.c_ulonglong),
        ("IsAtWaterSurface", ctypes.c_ulonglong),
        ("IsGrabbed", ctypes.c_ulonglong),
        ("IsMech", ctypes.c_ulonglong),
        ("HasMoveForce", ctypes.c_ulonglong),
        ("HasFired", ctypes.c_ulonglong),
        ("IsPilotingShip", ctypes.c_ulonglong),
        ("IsInVehicle", ctypes.c_ulonglong),
        ("HasCustomIdle", ctypes.c_ulonglong),
        ("UpdateIdle", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcCharacterInterface *this, TkID<128> *result, float)
        ("GetSlopeState", ctypes.c_ulonglong),
        ("IsHandTracking", ctypes.c_ulonglong),
        ("GetMatrix", ctypes.c_ulonglong),
        ("GetPhysicsMatrix", ctypes.c_ulonglong),
        ("GetFootPosition", ctypes.c_ulonglong),
        ("GetVelocity", ctypes.c_ulonglong),
        ("GetFacingDir", ctypes.c_ulonglong),
        ("GetWorldUp", ctypes.c_ulonglong),
        ("GetHeight", ctypes.c_ulonglong),
        ("GetRace", ctypes.c_ulonglong),
        ("GetAimPosition", ctypes.c_ulonglong),
        ("GetCurrentMoveForce", ctypes.c_ulonglong),
        ("SetToPosition", ctypes.c_ulonglong),
        ("SetToPosition_1c0", ctypes.c_ulonglong),
        ("EnableCharacterPhysics", ctypes.c_ulonglong),
        ("EnableCharacterCollision", ctypes.c_ulonglong),
        ("DisableCharacterCollision", ctypes.c_ulonglong),
        ("AttachControlMode", ctypes.c_ulonglong),
        ("EnableCharacterGravity", ctypes.c_ulonglong),
        ("DisableCharacterGravity", ctypes.c_ulonglong),
        ("GetRigidBody", ctypes.c_ulonglong),
        ("GetCharacterSit", ctypes.c_ulonglong),
        ("TakeDamage", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCharacterInterface *this, float, eDamageType, const TkID<128> *, const cTkVector3 *, cGcOwnerConcept *)
        ("SetPitch", ctypes.c_ulonglong),
        ("SetRoll", ctypes.c_ulonglong),
        ("CanLookAt", ctypes.c_ulonglong),
        ("AllowHeadMovement", ctypes.c_ulonglong),
        ("GetLookAtPoint", ctypes.c_ulonglong),
    ]
    cGcCharacterInterface_dtor_0: int
    IsPlayer: int
    IsReady: int
    IsLocalPlayer: int
    IsVRPlayer: int
    WeaponIsActive: int
    WeaponIsLarge: int
    BinocsActive: int
    IsHitReacting: int
    GetHitReactDir: int
    IsOnGround: int
    IsInSpace: int
    ShouldUseAbandonedFreighterWalk: int
    IsDying: int
    GetDeathReason: int
    GetEnergy: int
    GetHazard: int
    IsJetpacking: int
    IsJetpackBoosted: int
    IsSliding: int
    IsRiding: int
    GetRidingAttachment: int
    GetHandPose: bytes
    GetDominantHand: int
    IsMeleeActive: int
    IsPointing: int
    IsInteracting: int
    IsMeleeBoostActive: int
    IsWaitingForRocketBootsActivation: int
    IsRocketBootsActive: int
    GetLocalTeleportDestination: int
    GetTimeLastUsedJetpack: int
    IsRunning: int
    IsUnderwater: int
    IsAtWaterSurface: int
    IsGrabbed: int
    IsMech: int
    HasMoveForce: int
    HasFired: int
    IsPilotingShip: int
    IsInVehicle: int
    HasCustomIdle: int
    UpdateIdle: bytes
    GetSlopeState: int
    IsHandTracking: int
    GetMatrix: int
    GetPhysicsMatrix: int
    GetFootPosition: int
    GetVelocity: int
    GetFacingDir: int
    GetWorldUp: int
    GetHeight: int
    GetRace: int
    GetAimPosition: int
    GetCurrentMoveForce: int
    SetToPosition: int
    SetToPosition_1c0: int
    EnableCharacterPhysics: int
    EnableCharacterCollision: int
    DisableCharacterCollision: int
    AttachControlMode: int
    EnableCharacterGravity: int
    DisableCharacterGravity: int
    GetRigidBody: int
    GetCharacterSit: int
    TakeDamage: bytes
    SetPitch: int
    SetRoll: int
    CanLookAt: int
    AllowHeadMovement: int
    GetLookAtPoint: int

class cGcNetworkPlayerCharacterInterface_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCharacterInterface_dtor_0", ctypes.c_ulonglong),
        ("IsPlayer", ctypes.c_ulonglong),
        ("IsReady", ctypes.c_ulonglong),
        ("IsLocalPlayer", ctypes.c_ulonglong),
        ("IsVRPlayer", ctypes.c_ulonglong),
        ("WeaponIsActive", ctypes.c_ulonglong),
        ("WeaponIsLarge", ctypes.c_ulonglong),
        ("BinocsActive", ctypes.c_ulonglong),
        ("IsHitReacting", ctypes.c_ulonglong),
        ("GetHitReactDir", ctypes.c_ulonglong),
        ("IsOnGround", ctypes.c_ulonglong),
        ("IsInSpace", ctypes.c_ulonglong),
        ("ShouldUseAbandonedFreighterWalk", ctypes.c_ulonglong),
        ("IsDying", ctypes.c_ulonglong),
        ("GetDeathReason", ctypes.c_ulonglong),
        ("GetEnergy", ctypes.c_ulonglong),
        ("GetHazard", ctypes.c_ulonglong),
        ("IsJetpacking", ctypes.c_ulonglong),
        ("IsJetpackBoosted", ctypes.c_ulonglong),
        ("IsSliding", ctypes.c_ulonglong),
        ("IsRiding", ctypes.c_ulonglong),
        ("GetRidingAttachment", ctypes.c_ulonglong),
        ("GetHandPose", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcCharacterInterface *this, eHand, TkID<128> *, float *)
        ("GetDominantHand", ctypes.c_ulonglong),
        ("IsMeleeActive", ctypes.c_ulonglong),
        ("IsPointing", ctypes.c_ulonglong),
        ("IsInteracting", ctypes.c_ulonglong),
        ("IsMeleeBoostActive", ctypes.c_ulonglong),
        ("IsWaitingForRocketBootsActivation", ctypes.c_ulonglong),
        ("IsRocketBootsActive", ctypes.c_ulonglong),
        ("GetLocalTeleportDestination", ctypes.c_ulonglong),
        ("GetTimeLastUsedJetpack", ctypes.c_ulonglong),
        ("IsRunning", ctypes.c_ulonglong),
        ("IsUnderwater", ctypes.c_ulonglong),
        ("IsAtWaterSurface", ctypes.c_ulonglong),
        ("IsGrabbed", ctypes.c_ulonglong),
        ("IsMech", ctypes.c_ulonglong),
        ("HasMoveForce", ctypes.c_ulonglong),
        ("HasFired", ctypes.c_ulonglong),
        ("IsPilotingShip", ctypes.c_ulonglong),
        ("IsInVehicle", ctypes.c_ulonglong),
        ("HasCustomIdle", ctypes.c_ulonglong),
        ("UpdateIdle", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcCharacterInterface *this, TkID<128> *result, float)
        ("GetSlopeState", ctypes.c_ulonglong),
        ("IsHandTracking", ctypes.c_ulonglong),
        ("GetMatrix", ctypes.c_ulonglong),
        ("GetPhysicsMatrix", ctypes.c_ulonglong),
        ("GetFootPosition", ctypes.c_ulonglong),
        ("GetVelocity", ctypes.c_ulonglong),
        ("GetFacingDir", ctypes.c_ulonglong),
        ("GetWorldUp", ctypes.c_ulonglong),
        ("GetHeight", ctypes.c_ulonglong),
        ("GetRace", ctypes.c_ulonglong),
        ("GetAimPosition", ctypes.c_ulonglong),
        ("GetCurrentMoveForce", ctypes.c_ulonglong),
        ("SetToPosition", ctypes.c_ulonglong),
        ("SetToPosition_1c0", ctypes.c_ulonglong),
        ("EnableCharacterPhysics", ctypes.c_ulonglong),
        ("EnableCharacterCollision", ctypes.c_ulonglong),
        ("DisableCharacterCollision", ctypes.c_ulonglong),
        ("AttachControlMode", ctypes.c_ulonglong),
        ("EnableCharacterGravity", ctypes.c_ulonglong),
        ("DisableCharacterGravity", ctypes.c_ulonglong),
        ("GetRigidBody", ctypes.c_ulonglong),
        ("GetCharacterSit", ctypes.c_ulonglong),
        ("TakeDamage", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCharacterInterface *this, float, eDamageType, const TkID<128> *, const cTkVector3 *, cGcOwnerConcept *)
        ("SetPitch", ctypes.c_ulonglong),
        ("SetRoll", ctypes.c_ulonglong),
        ("CanLookAt", ctypes.c_ulonglong),
        ("AllowHeadMovement", ctypes.c_ulonglong),
        ("GetLookAtPoint", ctypes.c_ulonglong),
    ]
    cGcCharacterInterface_dtor_0: int
    IsPlayer: int
    IsReady: int
    IsLocalPlayer: int
    IsVRPlayer: int
    WeaponIsActive: int
    WeaponIsLarge: int
    BinocsActive: int
    IsHitReacting: int
    GetHitReactDir: int
    IsOnGround: int
    IsInSpace: int
    ShouldUseAbandonedFreighterWalk: int
    IsDying: int
    GetDeathReason: int
    GetEnergy: int
    GetHazard: int
    IsJetpacking: int
    IsJetpackBoosted: int
    IsSliding: int
    IsRiding: int
    GetRidingAttachment: int
    GetHandPose: bytes
    GetDominantHand: int
    IsMeleeActive: int
    IsPointing: int
    IsInteracting: int
    IsMeleeBoostActive: int
    IsWaitingForRocketBootsActivation: int
    IsRocketBootsActive: int
    GetLocalTeleportDestination: int
    GetTimeLastUsedJetpack: int
    IsRunning: int
    IsUnderwater: int
    IsAtWaterSurface: int
    IsGrabbed: int
    IsMech: int
    HasMoveForce: int
    HasFired: int
    IsPilotingShip: int
    IsInVehicle: int
    HasCustomIdle: int
    UpdateIdle: bytes
    GetSlopeState: int
    IsHandTracking: int
    GetMatrix: int
    GetPhysicsMatrix: int
    GetFootPosition: int
    GetVelocity: int
    GetFacingDir: int
    GetWorldUp: int
    GetHeight: int
    GetRace: int
    GetAimPosition: int
    GetCurrentMoveForce: int
    SetToPosition: int
    SetToPosition_1c0: int
    EnableCharacterPhysics: int
    EnableCharacterCollision: int
    DisableCharacterCollision: int
    AttachControlMode: int
    EnableCharacterGravity: int
    DisableCharacterGravity: int
    GetRigidBody: int
    GetCharacterSit: int
    TakeDamage: bytes
    SetPitch: int
    SetRoll: int
    CanLookAt: int
    AllowHeadMovement: int
    GetLookAtPoint: int

class cCachedIndexBufferEntry(ctypes.Structure):
    _fields_ = [
        ("mFreeHandles", ctypes.c_char * 0x18),  # std::vector<unsigned int,TkSTLAllocatorShim<unsigned int,4,-1> >
    ]
    mFreeHandles: bytes

class cEgCastRayResult(ctypes.Structure):
    _fields_ = [
        ("mpNode", ctypes.c_ulonglong),
        ("mfDistance", ctypes.c_float),
        ("mIntersectionPoint", common.Vector3f),
    ]
    mpNode: int
    mfDistance: float
    mIntersectionPoint: common.Vector3f

class cCachedVertexBufferDesc(ctypes.Structure):
    _fields_ = [
        ("miSize", ctypes.c_int32),
        ("mbDynamic", ctypes.c_ubyte),
    ]
    miSize: int
    mbDynamic: bool

class cCachedVertexBufferEntry(ctypes.Structure):
    _fields_ = [
        ("mFreeHandles", ctypes.c_char * 0x18),  # std::vector<unsigned int,TkSTLAllocatorShim<unsigned int,4,-1> >
    ]
    mFreeHandles: bytes

class cEgPrivateManagedRenderDataScopeLock(ctypes.Structure):
    _fields_ = [
        ("mData", ctypes.c_ulonglong),
    ]
    mData: int

class cEgRenderAction(ctypes.Structure):
    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
    ]
    __vftable: int

class cEgRenderAction_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionGpuPerfMarkerPop_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgLightNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cCachedIndexBufferDesc(ctypes.Structure):
    _fields_ = [
        ("miSize", ctypes.c_int32),
        ("mbDynamic", ctypes.c_ubyte),
    ]
    miSize: int
    mbDynamic: bool

class cEgMeshNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cEgMeshNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cEgRenderActionSetAsyncComputeEnable_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class GeometryMemoryResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cEgRenderActionEndScissorTest_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionSetMaterialUniform_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgJointNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cEgExtensionManager(ctypes.Structure):
    def InstallExtension(): pass

    def InstallExtensions(): pass

    _fields_ = [
        ("mExtensions", ctypes.c_char * 0x18),  # std::vector<cEgExtension,TkSTLAllocatorShim<cEgExtension,8,-1> >
    ]
    mExtensions: bytes

class cEgRenderActionClear_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionQueue(ctypes.Structure):
    def MoveActionQueue(): pass

    _fields_ = [
        ("mpBuffer", ctypes.c_ulonglong),
        ("mBufferSize", ctypes.c_uint32),
        ("miBufferUsed", ctypes.c_int32),
        ("miNumActions", ctypes.c_int32),
    ]
    mpBuffer: int
    mBufferSize: int
    miBufferUsed: int
    miNumActions: int

class cGcNPCBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcNPCBehaviourChooseInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cEgRenderActionSetRenderTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cGcNPCBehaviourWithTurning_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcNPCBehaviourPerformInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cEgRenderQueueEntryTempData(ctypes.Structure):
    _fields_ = [
        ("mpNode", ctypes.c_ulonglong),
        ("mpMatRes", ctypes.c_ulonglong),
        ("mpRenderQueueInfo", ctypes.c_ulonglong),
        ("mfAabbSqrDist", ctypes.c_float),
        ("mbCullFullyInside", ctypes.c_ubyte),
        ("maPadding", ctypes.c_ubyte * 0x3),  # bool[3]
    ]
    mpNode: int
    mpMatRes: int
    mpRenderQueueInfo: int
    mfAabbSqrDist: float
    mbCullFullyInside: bool
    maPadding: list[ctypes.c_ubyte]

class cGcNPCBehaviourGreet_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcNPCBehaviourReactToPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcNPCBehaviourReactToJudgement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcNPCBehaviourInteraction_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cEgIndexMappingArray(ctypes.Structure):
    def GetIndex(): pass

    _fields_ = [
        ("mNodeTypeList", ctypes.c_char * 0x18),  # std::vector<TkID<128>,TkSTLAllocatorShim<TkID<128>,8,-1> >
        ("mMaterialClassList", ctypes.c_char * 0x18),  # std::vector<cTkFixedString<32,char>,TkSTLAllocatorShim<cTkFixedString<32,char>,1,-1> >
    ]
    mNodeTypeList: bytes
    mMaterialClassList: bytes

class cEgLightNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cEgNodeRegistryEntry(ctypes.Structure):
    _fields_ = [
        ("mType", ctypes.c_ulonglong),
        ("mpParsingFunction", ctypes.c_char * 0x8),  # cEgSceneNodeTemplate *(__fastcall *)(const cTkDynamicArray<cTkSceneNodeAttributeData> *, cTkResourceDescriptor *, int)
        ("mpFactoryFunction", ctypes.c_ulonglong),
        ("mpRenderFunction", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgRenderQueueBuffer *, const TkID<128> *, int, bool, RenderingOrder::List, bool, bool, const cEgFrustum *, cEgThreadableRenderCall *)
        ("mpRenderBboxFunction", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgRenderQueueBuffer *, const TkID<128> *, cTkResource *, bool, float, const cEgFrustum *, cEgThreadableRenderCall *)
        ("mpRenderStereoFunction", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgRenderQueueBuffer *, const TkID<128> *, int, bool, RenderingOrder::List, bool, bool, const cEgFrustum *, cEgThreadableRenderCall *)
        ("mpPostCullUpdateFunction", ctypes.c_ulonglong),
        ("mpRenderImposterFunction", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgRenderQueueBuffer *, const TkID<128> *, int, bool, RenderingOrder::List, bool, bool, const cEgFrustum *, cEgThreadableRenderCall *, int)
        ("mpEnqueueFunction", ctypes.c_ulonglong),
        ("mDebugColour", common.Colour),
        ("miClassSize", ctypes.c_int32),
        ("miExpectedCount", ctypes.c_int32),
        ("mpSlotAllocator", ctypes.c_ulonglong),
    ]
    mType: int
    mpParsingFunction: bytes
    mpFactoryFunction: int
    mpRenderFunction: bytes
    mpRenderBboxFunction: bytes
    mpRenderStereoFunction: bytes
    mpPostCullUpdateFunction: int
    mpRenderImposterFunction: bytes
    mpEnqueueFunction: int
    mDebugColour: common.Colour
    miClassSize: int
    miExpectedCount: int
    mpSlotAllocator: int

class cEgStructureIndex(ctypes.Structure):
    _fields_ = [
        ("_bf_0", ctypes.c_int32),
        ("mParentHandle", ctypes.c_uint32),
        ("miFirstChildIdx", ctypes.c_int32),
        ("miPrevSiblingIdx", ctypes.c_int32),
        ("miNextSiblingIdx", ctypes.c_int32),
    ]
    _bf_0: int
    mParentHandle: int
    miFirstChildIdx: int
    miPrevSiblingIdx: int
    miNextSiblingIdx: int

class cEgSceneNodeFlags(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x2),  # $0112A3108E5B634EBF5AEF9646E5FF9F
    ]
    ___u0: bytes

class cEgUpdateTransformsWorkload(ctypes.Structure):
    _fields_ = [
        ("miNodesBegin", ctypes.c_int32),
        ("miNodesEnd", ctypes.c_int32),
        ("miWorkloadSize", ctypes.c_int32),
    ]
    miNodesBegin: int
    miNodesEnd: int
    miWorkloadSize: int

class cEgUpdateTransformsWorkBuffer(ctypes.Structure):
    def CreateNextWorkload(): pass

    def Job(): pass

    _fields_ = [
        ("mNodes", ctypes.c_char * 0x18),  # std::vector<int,TkSTLAllocatorShim<int,16,-1> >
        ("mWorkloads", ctypes.c_char * 0x18),  # std::vector<cEgUpdateTransformsWorkload,TkSTLAllocatorShim<cEgUpdateTransformsWorkload,16,-1> >
        ("mWorkloadsTaken", ctypes.c_char * 0x4),  # std::atomic<int>
        ("mWorkloadsComplete", ctypes.c_char * 0x4),  # std::atomic<int>
        ("miTotalNodes", ctypes.c_int32),
    ]
    mNodes: bytes
    mWorkloads: bytes
    mWorkloadsTaken: bytes
    mWorkloadsComplete: bytes
    miTotalNodes: int

class cGcNPCBehaviourFlyAway_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcSpringAttachmentComponentUpdater_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnInit", ctypes.c_ulonglong),
        ("UpdateComponents", ctypes.c_ulonglong),
        ("UpdateComponentsPostPhysics", ctypes.c_ulonglong),
        ("UpdateComponentsRender", ctypes.c_ulonglong),
        ("OnComponentAllocate", ctypes.c_ulonglong),
        ("OnComponentFree", ctypes.c_ulonglong),
        ("HasCustomUpdate", ctypes.c_ulonglong),
    ]
    OnInit: int
    UpdateComponents: int
    UpdateComponentsPostPhysics: int
    UpdateComponentsRender: int
    OnComponentAllocate: int
    OnComponentFree: int
    HasCustomUpdate: int

class cGcSpringAttachmentComponentUpdater(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x18),  # cTkTemplatedComponentUpdater<cGcSpringAttachmentComponent>
    ]
    baseclass_0: bytes

class ComponentInstancercGcSpringAttachmentComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcPlayerFullBodyIKComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcPlayerFullBodyIKComponentUpdater_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnInit", ctypes.c_ulonglong),
        ("UpdateComponents", ctypes.c_ulonglong),
        ("UpdateComponentsPostPhysics", ctypes.c_ulonglong),
        ("UpdateComponentsRender", ctypes.c_ulonglong),
        ("OnComponentAllocate", ctypes.c_ulonglong),
        ("OnComponentFree", ctypes.c_ulonglong),
        ("HasCustomUpdate", ctypes.c_ulonglong),
    ]
    OnInit: int
    UpdateComponents: int
    UpdateComponentsPostPhysics: int
    UpdateComponentsRender: int
    OnComponentAllocate: int
    OnComponentFree: int
    HasCustomUpdate: int

class cGcPlayerFullBodyIKComponentUpdater(ctypes.Structure):
    def UpdateComponentsPostPhysics(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x18),  # cTkTemplatedComponentUpdater<cGcPlayerFullBodyIKComponent>
    ]
    baseclass_0: bytes

class cStoredVertsForInitialShape(ctypes.Structure):
    def scalar_deleting_destructor(): pass

    _fields_ = [
        ("mUvs", ctypes.c_char * 0x8),  # TkVector_BoundChecked<cTkVector2> *
        ("mPositions", ctypes.c_char * 0x8),  # TkVector_BoundChecked<cTkVector3> *
    ]
    mUvs: bytes
    mPositions: bytes

class ComponentInstancercGcClothComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcClothComponentUpdater_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnInit", ctypes.c_ulonglong),
        ("UpdateComponents", ctypes.c_ulonglong),
        ("UpdateComponentsPostPhysics", ctypes.c_ulonglong),
        ("UpdateComponentsRender", ctypes.c_ulonglong),
        ("OnComponentAllocate", ctypes.c_ulonglong),
        ("OnComponentFree", ctypes.c_ulonglong),
        ("HasCustomUpdate", ctypes.c_ulonglong),
    ]
    OnInit: int
    UpdateComponents: int
    UpdateComponentsPostPhysics: int
    UpdateComponentsRender: int
    OnComponentAllocate: int
    OnComponentFree: int
    HasCustomUpdate: int

class cGcClothComponentUpdater(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x18),  # cTkTemplatedComponentUpdater<cGcClothComponent>
    ]
    baseclass_0: bytes

class cEgDrawGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgModelNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class sComponentBaseObjectHandleLookup(ctypes.Structure):
    _fields_ = [
        ("mpComponentHandlePointer", ctypes.c_ulonglong),
        ("mBaseObjectNodeHandle", ctypes.c_uint32),
    ]
    mpComponentHandlePointer: int
    mBaseObjectNodeHandle: int

class cEgGroupNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cEgRenderActionRender_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgParticleVertex(ctypes.Structure):
    _fields_ = [
        ("mfU", ctypes.c_uint16),
        ("mfV", ctypes.c_uint16),
        ("mfIndex", ctypes.c_uint16),
        ("mfCornerIndex", ctypes.c_uint16),
    ]
    mfU: int
    mfV: int
    mfIndex: int
    mfCornerIndex: int

class cEgDrawShadowGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgRenderActionCompressTextureCompute_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgCullNodesUpdateBufferChunk(ctypes.Structure):
    _fields_ = [
        ("miUsedElts", ctypes.c_int32),
        ("mpNodes", ctypes.c_int32 * 0x80),  # int[128]
        ("_mpCullResults", ctypes.c_uint32),
        ("mpNext", ctypes.c_ulonglong),
    ]
    miUsedElts: int
    mpNodes: list[ctypes.c_int32]
    _mpCullResults: int
    mpNext: int

    @property
    def mpCullResults(self):
        return cEgFrustum.eCullResult(self._mpCullResults)


class cEgTransposedFrustum(ctypes.Structure):
    _fields_ = [
        ("mafPlaneNormalX", ctypes.c_float * 0x8),  # float[8]
        ("mafPlaneNormalY", ctypes.c_float * 0x8),  # float[8]
        ("mafPlaneNormalZ", ctypes.c_float * 0x8),  # float[8]
        ("mafPlaneDistance", ctypes.c_float * 0x8),  # float[8]
        ("mFrustumCenter", common.Vector3f),
        ("mvFrustumOffset", common.Vector3f),
        ("miNumPlanes", ctypes.c_int32),
        ("mIsCuboid", ctypes.c_ubyte),
    ]
    mafPlaneNormalX: list[ctypes.c_float]
    mafPlaneNormalY: list[ctypes.c_float]
    mafPlaneNormalZ: list[ctypes.c_float]
    mafPlaneDistance: list[ctypes.c_float]
    mFrustumCenter: common.Vector3f
    mvFrustumOffset: common.Vector3f
    miNumPlanes: int
    mIsCuboid: bool

class PFLobbySearchResult(ctypes.Structure):
    _fields_ = [
        ("lobbyId", ctypes.c_char_p),
        ("connectionString", ctypes.c_char_p),
        ("ownerEntity", ctypes.c_ulonglong),
        ("maxMemberCount", ctypes.c_uint32),
        ("currentMemberCount", ctypes.c_uint32),
        ("searchPropertyCount", ctypes.c_uint32),
        ("searchPropertyKeys", ctypes.c_ulonglong),
        ("searchPropertyValues", ctypes.c_ulonglong),
        ("friendCount", ctypes.c_uint32),
        ("friends", ctypes.c_ulonglong),
    ]
    lobbyId: bytes
    connectionString: bytes
    ownerEntity: int
    maxMemberCount: int
    currentMemberCount: int
    searchPropertyCount: int
    searchPropertyKeys: int
    searchPropertyValues: int
    friendCount: int
    friends: int

class cEgFfxFsr2Draw_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgCollisionNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cEgCollisionNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cGcCreatureComponentUpdater_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnInit", ctypes.c_ulonglong),
        ("UpdateComponents", ctypes.c_ulonglong),
        ("UpdateComponentsPostPhysics", ctypes.c_ulonglong),
        ("UpdateComponentsRender", ctypes.c_ulonglong),
        ("OnComponentAllocate", ctypes.c_ulonglong),
        ("OnComponentFree", ctypes.c_ulonglong),
        ("HasCustomUpdate", ctypes.c_ulonglong),
    ]
    OnInit: int
    UpdateComponents: int
    UpdateComponentsPostPhysics: int
    UpdateComponentsRender: int
    OnComponentAllocate: int
    OnComponentFree: int
    HasCustomUpdate: int

class cGcCreatureComponentUpdater(ctypes.Structure):
    def UpdateComponents(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x18),  # cTkTemplatedComponentUpdater<cGcCreatureComponent>
    ]
    baseclass_0: bytes

class ComponentInstancercGcCreatureComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcCreatureBaitComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcCreatureAttractorComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcCreaturePerceptionManager(ctypes.Structure):
    _fields_ = [
        ("maPerceptions", ctypes.c_ulonglong),
        ("maFreeList", ctypes.c_ulonglong),
        ("maUsed", ctypes.c_ulonglong),
        ("miFreeListHead", ctypes.c_uint32),
        ("miOffset", ctypes.c_int32),
    ]
    maPerceptions: int
    maFreeList: int
    maUsed: int
    miFreeListHead: int
    miOffset: int

class cGcSentinelDrone_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcSentinelRobot_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("GetTree", ctypes.c_ulonglong),
        ("GetCreatureType", ctypes.c_ulonglong),
        ("GetOwnerConcept", ctypes.c_ulonglong),
        ("GetWeapon", ctypes.c_ulonglong),
        ("FireWeapon", ctypes.c_ulonglong),
        ("GetTargetPoint", ctypes.c_ulonglong),
        ("GetForward", ctypes.c_ulonglong),
        ("GetLabelPoint", ctypes.c_ulonglong),
        ("GetTargetPointSize", ctypes.c_ulonglong),
        ("GetSightPoint", ctypes.c_ulonglong),
        ("GetObb", ctypes.c_ulonglong),
        ("GetEffectAttachLocator", ctypes.c_ulonglong),
        ("IsAttacking", ctypes.c_ulonglong),
        ("IsAlert", ctypes.c_ulonglong),
        ("IsDead", ctypes.c_ulonglong),
        ("GetAttackTarget", ctypes.c_ulonglong),
        ("GetAlertRange", ctypes.c_ulonglong),
        ("GetSightRange", ctypes.c_ulonglong),
        ("GetSightAngle", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("OnHit", ctypes.c_ulonglong),
        ("OnArmourHit", ctypes.c_ulonglong),
        ("OnDeath", ctypes.c_ulonglong),
    ]
    cGcSentinelRobot_dtor_0: int
    Prepare: int
    Release: int
    Update: int
    NetSerialize: int
    GetTree: int
    GetCreatureType: int
    GetOwnerConcept: int
    GetWeapon: int
    FireWeapon: int
    GetTargetPoint: int
    GetForward: int
    GetLabelPoint: int
    GetTargetPointSize: int
    GetSightPoint: int
    GetObb: int
    GetEffectAttachLocator: int
    IsAttacking: int
    IsAlert: int
    IsDead: int
    GetAttackTarget: int
    GetAlertRange: int
    GetSightRange: int
    GetSightAngle: int
    RenderNGui: int
    OnHit: int
    OnArmourHit: int
    OnDeath: int

class ComponentInstancercGcEngineComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcEngineComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cEgEngineConfig(ctypes.Structure):
    _fields_ = [
        ("miMaxLogLevel", ctypes.c_int32),
        ("miMaxAnisotropy", ctypes.c_int32),
        ("miShadowMapSize", ctypes.c_int32),
        ("miTextureQuality", ctypes.c_int32),
        ("mbTextureCompression", ctypes.c_ubyte),
        ("mbLoadTextures", ctypes.c_ubyte),
        ("mbFastAnimation", ctypes.c_ubyte),
        ("mbTrilinearFiltering", ctypes.c_ubyte),
        ("mbWireframeMode", ctypes.c_ubyte),
        ("mbDebugViewMode", ctypes.c_ubyte),
        ("mbDumpFailedShaders", ctypes.c_ubyte),
        ("mbGatherTimeStats", ctypes.c_ubyte),
    ]
    miMaxLogLevel: int
    miMaxAnisotropy: int
    miShadowMapSize: int
    miTextureQuality: int
    mbTextureCompression: bool
    mbLoadTextures: bool
    mbFastAnimation: bool
    mbTrilinearFiltering: bool
    mbWireframeMode: bool
    mbDebugViewMode: bool
    mbDumpFailedShaders: bool
    mbGatherTimeStats: bool

class cEgCameraNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cEgRenderer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgRendererBase_dtor_0", ctypes.c_ulonglong),
        ("Initialise", ctypes.c_ulonglong),
        ("Resize", ctypes.c_ulonglong),
    ]
    cEgRendererBase_dtor_0: int
    Initialise: int
    Resize: int

class ALLXMLClearTag(ctypes.Structure):
    _fields_ = [
        ("lpszOpen", ctypes.c_char_p),
        ("openTagLen", ctypes.c_int32),
        ("lpszClose", ctypes.c_char_p),
    ]
    lpszOpen: bytes
    openTagLen: int
    lpszClose: bytes

class XMLCharacterEntity(ctypes.Structure):
    _fields_ = [
        ("s", ctypes.c_char_p),
        ("l", ctypes.c_int32),
        ("c", ctypes.c_char),
    ]
    s: bytes
    l: int
    c: str

class NextToken(ctypes.Structure):
    _fields_ = [
        ("pClr", ctypes.c_ulonglong),
        ("pStr", ctypes.c_char_p),
    ]
    pClr: int
    pStr: bytes

class cGcURIEncoder(ctypes.Structure):
    class sUInt64Colour: pass

    def GetInstance(): pass

    _fields_ = [
        ("mbURIEnabled", ctypes.c_ubyte),
        ("mbURILoadActive", ctypes.c_ubyte),
    ]
    mbURIEnabled: bool
    mbURILoadActive: bool

class cTkCommandLine(ctypes.Structure):
    def GetArgumentString(): pass

    def GetArgumentBool(): pass

    _fields_ = [
        ("__vftable", ctypes.c_ulonglong),
        ("mCommandLineArguments", ctypes.c_char * 0x50),  # robin_hood::detail::Table<0,80,cTkFixedString<128,char>,cTkFixedString<128,char>,UnorderedMapHashes::Hash<cTkFixedString<128,char> >,std::equal_to<cTkFixedString<128,char> >,-1>
    ]
    __vftable: int
    mCommandLineArguments: bytes

class cTkCommandLine_vtbl(ctypes.Structure):
    _fields_ = [
        ("ParseArguments", ctypes.c_ulonglong),
    ]
    ParseArguments: int

class cGcCommandLine_vtbl(ctypes.Structure):
    _fields_ = [
        ("ParseArguments", ctypes.c_ulonglong),
    ]
    ParseArguments: int

class cGcUniqueIdComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcMechBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcExoMechAttachShooting_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMechAttackMove_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMechCheckAlert_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMechCheckCanAttack_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMechFollowPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMechGuard_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMechSearchForTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMechSelectAttackTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMechStunned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcOptionListHeader_vtbl(ctypes.Structure):
    _fields_ = [
        ("UpdateUI", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Apply", ctypes.c_ulonglong),
        ("ResetToDefault", ctypes.c_ulonglong),
        ("DiscardChanges", ctypes.c_ulonglong),
        ("HasChanges", ctypes.c_ulonglong),
        ("IsNonDefault", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("IsHovering", ctypes.c_ulonglong),
        ("cGcUIOptionListElement_dtor_50", ctypes.c_ulonglong),
        ("TranslateDescription", ctypes.c_ulonglong),
    ]
    UpdateUI: int
    Initialize: int
    Apply: int
    ResetToDefault: int
    DiscardChanges: int
    HasChanges: int
    IsNonDefault: int
    GetDescription: int
    GetName: int
    IsHovering: int
    cGcUIOptionListElement_dtor_50: int
    TranslateDescription: int

class cGcSentinelMechAttackShooting_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class FadeNodeJobContext(ctypes.Structure):
    _fields_ = [
        ("mpThis", ctypes.c_ulonglong),
        ("miThreadIndex", ctypes.c_int32),
        ("miStartIndex", ctypes.c_int32),
        ("miEndIndex", ctypes.c_int32),
    ]
    mpThis: int
    miThreadIndex: int
    miStartIndex: int
    miEndIndex: int

class cGcDifficultySettingsConfirmHandlerBootMenu_vtbl(ctypes.Structure):
    _fields_ = [
        ("DoConfirm", ctypes.c_ulonglong),
    ]
    DoConfirm: int

class cEgTrailNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cGcShipAIAttackBuilding_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcFrigateAIBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAICheckOrbit_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAITrackTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIAttack_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIAttackFreighter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIFreighter_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIFreighterTargeting_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cEgRenderActionSetMaterialSampler_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgGroupNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cGcShipAIEscort_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cEgRenderActionResetRenderer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cGcShipAIFlybyMineAsteroids_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cEgDrawScreenQuad_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgRenderActionSetNodeParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionGpuPerfMarkerPush_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionCreateScreenshot_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionSetResParam_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cGcFrameStatsDisplay(ctypes.Structure):
    pass

class cEgRenderActionSetupViewport_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgCameraNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cEgCullNodesWorkBuffer(ctypes.Structure):
    def Begin(): pass

    def Submit(): pass

    _fields_ = [
        ("mpData", ctypes.c_ulonglong),
        ("mpWorkloads", ctypes.c_ulonglong),
        ("miCurSubtreeSize", ctypes.c_int32),
        ("miNumWorkloads", ctypes.c_int32),
        ("miMaxWorkloads", ctypes.c_int32),
        ("mWorkloadsTaken", ctypes.c_char * 0x4),  # std::atomic<int>
        ("mWorkloadsComplete", ctypes.c_char * 0x4),  # std::atomic<int>
        ("mpFlagsCache", ctypes.c_ulonglong),
        ("mpCurrentQueue", ctypes.c_ulonglong),
        ("mpCurrentQueueInfo", ctypes.c_ulonglong),
        ("miShadowSplitIndex", ctypes.c_int32),
        ("mpSseFrustum", ctypes.c_ulonglong),
        ("mpSseInnerFrustum", ctypes.c_ulonglong),
        ("mCullCompleteEvent", ctypes.c_ulonglong),
    ]
    mpData: int
    mpWorkloads: int
    miCurSubtreeSize: int
    miNumWorkloads: int
    miMaxWorkloads: int
    mWorkloadsTaken: bytes
    mWorkloadsComplete: bytes
    mpFlagsCache: int
    mpCurrentQueue: int
    mpCurrentQueueInfo: int
    miShadowSplitIndex: int
    mpSseFrustum: int
    mpSseInnerFrustum: int
    mCullCompleteEvent: int

class cEgRenderActionFlushMiplevel_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionFlushTexture_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionBlitTexture_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgReferenceNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cEgLocatorNodeTemplate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNodeTemplate_dtor_0", ctypes.c_ulonglong),
        ("SetName", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, const cTkSharedPtr<cTkResourceDescriptor const > *)
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GatherData", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgSceneNodeTemplate *this, std::vector<cTkAttachmentData *,TkSTLAllocatorShim<cTkAttachmentData *,8,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
        ("IsGeometryStreamedIn", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
    ]
    cEgSceneNodeTemplate_dtor_0: int
    SetName: int
    SetDescriptor: bytes
    GatherMaterials: bytes
    GatherData: bytes
    GetBoundingBox: int
    IsGeometryStreamedIn: int
    IsLoaded: int

class cEgDrawLightGeometry_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgRenderActionFastGenerateMipmaps_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionGraphicsTask_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgMetadataResource_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkResource_dtor_0", ctypes.c_ulonglong),
        ("IsLoaded", ctypes.c_ulonglong),
        ("SetDescriptor", ctypes.c_ulonglong),
        ("GetDataSize", ctypes.c_ulonglong),
        ("OnResourceLoaded", ctypes.c_ulonglong),
        ("GetDeletionFrameDelay", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("CloneInternal", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Equals", ctypes.c_ulonglong),
        ("InitDefault", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("GetElementCount", ctypes.c_ulonglong),
        ("GetElementParamInt", ctypes.c_ulonglong),
        ("SetElementParamInt", ctypes.c_ulonglong),
        ("GetElementParamFloat", ctypes.c_ulonglong),
        ("SetElementParamFloat", ctypes.c_ulonglong),
        ("GetElementParamStr", ctypes.c_ulonglong),
        ("SetElementParamStr", ctypes.c_ulonglong),
        ("MapResStream", ctypes.c_ulonglong),
        ("MapResStream_a0", ctypes.c_ulonglong),
        ("UnmapStream", ctypes.c_ulonglong),
        ("UnmapStream_b0", ctypes.c_ulonglong),
        ("IsSceneGraphRes", ctypes.c_ulonglong),
        ("GatherMaterials", ctypes.c_char * 0x8),  # void (__fastcall *)(cEgResource *this, std::vector<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,TkSTLAllocatorShim<TkStrongType<int,TkStrongTypeIDs::TkResHandleID>,4,-1> > *)
        ("GetBoundingBox", ctypes.c_ulonglong),
    ]
    cTkResource_dtor_0: int
    IsLoaded: int
    SetDescriptor: int
    GetDataSize: int
    OnResourceLoaded: int
    GetDeletionFrameDelay: int
    Clone: int
    CloneInternal: int
    Release: int
    Equals: int
    InitDefault: int
    Load: int
    GetElementCount: int
    GetElementParamInt: int
    SetElementParamInt: int
    GetElementParamFloat: int
    SetElementParamFloat: int
    GetElementParamStr: int
    SetElementParamStr: int
    MapResStream: int
    MapResStream_a0: int
    UnmapStream: int
    UnmapStream_b0: int
    IsSceneGraphRes: int
    GatherMaterials: bytes
    GetBoundingBox: int

class cEgDrawBboxes_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgLocatorNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("cEgSceneNode_dtor_0", ctypes.c_ulonglong),
        ("ParseRefAttrib", ctypes.c_ulonglong),
        ("GetParameterInt", ctypes.c_ulonglong),
        ("SetParameterInt", ctypes.c_ulonglong),
        ("GetParameterFloat", ctypes.c_ulonglong),
        ("SetParameterFloat", ctypes.c_ulonglong),
        ("GetParameterVec3", ctypes.c_ulonglong),
        ("SetParameterVec3", ctypes.c_ulonglong),
        ("GetParameterStr", ctypes.c_ulonglong),
        ("SetParameterStr", ctypes.c_ulonglong),
        ("GetVisibleBBox", ctypes.c_ulonglong),
        ("CalculateLodLevel", ctypes.c_ulonglong),
        ("CanAttach", ctypes.c_ulonglong),
        ("CheckIntersection", ctypes.c_ulonglong),
        ("GetClassOverride", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("PreAsyncUpdate", ctypes.c_ulonglong),
        ("AsyncUpdate", ctypes.c_ulonglong),
        ("PostAsyncUpdate", ctypes.c_ulonglong),
        ("OnAttach", ctypes.c_ulonglong),
        ("OnDetach", ctypes.c_ulonglong),
        ("PreTransferRenderData", ctypes.c_ulonglong),
        ("OnDetachInstanceAware", ctypes.c_ulonglong),
    ]
    cEgSceneNode_dtor_0: int
    ParseRefAttrib: int
    GetParameterInt: int
    SetParameterInt: int
    GetParameterFloat: int
    SetParameterFloat: int
    GetParameterVec3: int
    SetParameterVec3: int
    GetParameterStr: int
    SetParameterStr: int
    GetVisibleBBox: int
    CalculateLodLevel: int
    CanAttach: int
    CheckIntersection: int
    GetClassOverride: int
    Update: int
    PreAsyncUpdate: int
    AsyncUpdate: int
    PostAsyncUpdate: int
    OnAttach: int
    OnDetach: int
    PreTransferRenderData: int
    OnDetachInstanceAware: int

class cEgRenderActionSetSamplerDefaultTexture_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionPushNewContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgFfxFsr2Pass_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgDrawLightShapesMulti_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgBuildCullBuffer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgRenderActionClearMaterialSamplers_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgDrawLightShapes_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkDestructableClass_dtor_0", ctypes.c_ulonglong),
    ]
    cTkDestructableClass_dtor_0: int

class cEgRenderActionClearTexture_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgModules(ctypes.Structure):
    def Initialise(): pass

    def Release(): pass


class cEgRenderActionPushNewContextNoReset_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderActionSetMaterialUniformArray_vtbl(ctypes.Structure):
    _fields_ = [
        ("Execute", ctypes.c_ulonglong),
        ("cEgRenderAction_dtor_8", ctypes.c_ulonglong),
        ("MoveTo", ctypes.c_ulonglong),
    ]
    Execute: int
    cEgRenderAction_dtor_8: int
    MoveTo: int

class cEgRenderLightQueueInfo(ctypes.Structure):
    _fields_ = [
        ("mAbsolutePosition_mfRadius", common.Vector4f),
        ("mRenderSpacePosition_mfIntensity", common.Vector4f),
        ("mSpotDirection_mfFov", common.Vector4f),
        ("mSpotUp_mfFalloffType", common.Vector4f),
        ("mColour_miCookieIdx", common.Colour),
        ("msLightingContext", ctypes.c_ulonglong),
        ("msShadowContext", ctypes.c_ulonglong),
        ("mpMaterialResource", ctypes.c_ulonglong),
        ("mfVolumetric", ctypes.c_float),
        ("mbHasLight", ctypes.c_ubyte),
    ]
    mAbsolutePosition_mfRadius: common.Vector4f
    mRenderSpacePosition_mfIntensity: common.Vector4f
    mSpotDirection_mfFov: common.Vector4f
    mSpotUp_mfFalloffType: common.Vector4f
    mColour_miCookieIdx: common.Colour
    msLightingContext: int
    msShadowContext: int
    mpMaterialResource: int
    mfVolumetric: float
    mbHasLight: bool

class zhuffman(ctypes.Structure):
    _fields_ = [
        ("fast", ctypes.c_uint16 * 0x200),  # unsigned __int16[512]
        ("firstcode", ctypes.c_uint16 * 0x10),  # unsigned __int16[16]
        ("maxcode", ctypes.c_int32 * 0x11),  # int[17]
        ("firstsymbol", ctypes.c_uint16 * 0x10),  # unsigned __int16[16]
        ("size", ctypes.c_uint8 * 0x120),  # unsigned __int8[288]
        ("value", ctypes.c_uint16 * 0x120),  # unsigned __int16[288]
    ]
    fast: list[ctypes.c_uint16]
    firstcode: list[ctypes.c_uint16]
    maxcode: list[ctypes.c_int32]
    firstsymbol: list[ctypes.c_uint16]
    size: list[ctypes.c_uint8]
    value: list[ctypes.c_uint16]

class huffman(ctypes.Structure):
    _fields_ = [
        ("fast", ctypes.c_uint8 * 0x200),  # unsigned __int8[512]
        ("code", ctypes.c_uint16 * 0x100),  # unsigned __int16[256]
        ("values", ctypes.c_uint8 * 0x100),  # unsigned __int8[256]
        ("size", ctypes.c_uint8 * 0x101),  # unsigned __int8[257]
        ("maxcode", ctypes.c_uint32 * 0x12),  # unsigned int[18]
        ("delta", ctypes.c_int32 * 0x11),  # int[17]
    ]
    fast: list[ctypes.c_uint8]
    code: list[ctypes.c_uint16]
    values: list[ctypes.c_uint8]
    size: list[ctypes.c_uint8]
    maxcode: list[ctypes.c_uint32]
    delta: list[ctypes.c_int32]

class stbi_loader(ctypes.Structure):
    _fields_ = [
        ("test_memory", ctypes.c_ulonglong),
        ("load_from_memory", ctypes.c_ulonglong),
        ("test_file", ctypes.c_ulonglong),
        ("load_from_file", ctypes.c_ulonglong),
    ]
    test_memory: int
    load_from_memory: int
    test_file: int
    load_from_file: int

class DDS_header(ctypes.Structure):
    _fields_ = [
        ("dwMagic", ctypes.c_uint32),
        ("dwSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("dwHeight", ctypes.c_uint32),
        ("dwWidth", ctypes.c_uint32),
        ("dwPitchOrLinearSize", ctypes.c_uint32),
        ("dwDepth", ctypes.c_uint32),
        ("dwMipMapCount", ctypes.c_uint32),
        ("dwReserved1", ctypes.c_uint32 * 0xB),  # unsigned int[11]
        ("sPixelFormat", ctypes.c_char * 0x20),  # $355A44C46056EC16B7948F78BC58BC86
        ("sCaps", ctypes.c_char * 0x10),  # $C3423EBF98E6155B1CE9796221090AA7
        ("dwReserved2", ctypes.c_uint32),
    ]
    dwMagic: int
    dwSize: int
    dwFlags: int
    dwHeight: int
    dwWidth: int
    dwPitchOrLinearSize: int
    dwDepth: int
    dwMipMapCount: int
    dwReserved1: list[ctypes.c_uint32]
    sPixelFormat: bytes
    sCaps: bytes
    dwReserved2: int

class chunk(ctypes.Structure):
    _fields_ = [
        ("length", ctypes.c_uint32),
        ("type", ctypes.c_uint32),
    ]
    length: int
    type: int

class _GLFWinitconfig(ctypes.Structure):
    _fields_ = [
        ("hatButtons", ctypes.c_int32),
        ("ns", ctypes.c_char * 0x8),  # $1C915AB64C3446F9B110BDB7E21E12CA
    ]
    hatButtons: int
    ns: bytes

class _GLFWfbconfig(ctypes.Structure):
    _fields_ = [
        ("redBits", ctypes.c_int32),
        ("greenBits", ctypes.c_int32),
        ("blueBits", ctypes.c_int32),
        ("alphaBits", ctypes.c_int32),
        ("depthBits", ctypes.c_int32),
        ("stencilBits", ctypes.c_int32),
        ("accumRedBits", ctypes.c_int32),
        ("accumGreenBits", ctypes.c_int32),
        ("accumBlueBits", ctypes.c_int32),
        ("accumAlphaBits", ctypes.c_int32),
        ("auxBuffers", ctypes.c_int32),
        ("stereo", ctypes.c_int32),
        ("samples", ctypes.c_int32),
        ("sRGB", ctypes.c_int32),
        ("doublebuffer", ctypes.c_int32),
        ("transparent", ctypes.c_int32),
        ("handle", ctypes.c_uint64),
    ]
    redBits: int
    greenBits: int
    blueBits: int
    alphaBits: int
    depthBits: int
    stencilBits: int
    accumRedBits: int
    accumGreenBits: int
    accumBlueBits: int
    accumAlphaBits: int
    auxBuffers: int
    stereo: int
    samples: int
    sRGB: int
    doublebuffer: int
    transparent: int
    handle: int

class _GLFWwndconfig(ctypes.Structure):
    _fields_ = [
        ("width", ctypes.c_int32),
        ("height", ctypes.c_int32),
        ("title", ctypes.c_char_p),
        ("resizable", ctypes.c_int32),
        ("visible", ctypes.c_int32),
        ("decorated", ctypes.c_int32),
        ("focused", ctypes.c_int32),
        ("autoIconify", ctypes.c_int32),
        ("floating", ctypes.c_int32),
        ("maximized", ctypes.c_int32),
        ("centerCursor", ctypes.c_int32),
        ("focusOnShow", ctypes.c_int32),
        ("scaleToMonitor", ctypes.c_int32),
        ("ns", ctypes.c_char * 0x104),  # $FC41D35D30D98068B7C96E33185C0C77
        ("x11", ctypes.c_char * 0x200),  # $9C06130A8D55816D71E79578ED928A23
    ]
    width: int
    height: int
    title: bytes
    resizable: int
    visible: int
    decorated: int
    focused: int
    autoIconify: int
    floating: int
    maximized: int
    centerCursor: int
    focusOnShow: int
    scaleToMonitor: int
    ns: bytes
    x11: bytes

class GLFWgammaramp(ctypes.Structure):
    _fields_ = [
        ("red", ctypes.c_ulonglong),
        ("green", ctypes.c_ulonglong),
        ("blue", ctypes.c_ulonglong),
        ("size", ctypes.c_uint32),
    ]
    red: int
    green: int
    blue: int
    size: int

class _GLFWmonitorWin32(ctypes.Structure):
    _fields_ = [
        ("handle", ctypes.c_ulonglong),
        ("adapterName", ctypes.c_wchar * 0x20),  # wchar_t[32]
        ("adapterString", ctypes.c_wchar * 0x20),  # wchar_t[32]
        ("displayName", ctypes.c_wchar * 0x20),  # wchar_t[32]
        ("adapterId", ctypes.c_wchar * 0x80),  # wchar_t[128]
        ("adapterKey", ctypes.c_wchar * 0x80),  # wchar_t[128]
        ("publicAdapterName", ctypes.c_char * 0x20),  # char[32]
        ("publicDisplayName", ctypes.c_char * 0x20),  # char[32]
        ("modesPruned", ctypes.c_int32),
        ("modeChanged", ctypes.c_int32),
    ]
    handle: int
    adapterName: list[ctypes.c_wchar]
    adapterString: list[ctypes.c_wchar]
    displayName: list[ctypes.c_wchar]
    adapterId: list[ctypes.c_wchar]
    adapterKey: list[ctypes.c_wchar]
    publicAdapterName: list[ctypes.c_char]
    publicDisplayName: list[ctypes.c_char]
    modesPruned: int
    modeChanged: int

class _GLFWcursorWin32(ctypes.Structure):
    _fields_ = [
        ("handle", ctypes.c_ulonglong),
    ]
    handle: int

class _GLFWcontextWGL(ctypes.Structure):
    _fields_ = [
        ("dc", ctypes.c_ulonglong),
        ("handle", ctypes.c_ulonglong),
        ("interval", ctypes.c_int32),
    ]
    dc: int
    handle: int
    interval: int

class _GLFWcontextEGL(ctypes.Structure):
    _fields_ = [
        ("config", ctypes.c_void_p),
        ("handle", ctypes.c_void_p),
        ("surface", ctypes.c_void_p),
        ("client", ctypes.c_void_p),
    ]
    config: int
    handle: int
    surface: int
    client: int

class _GLFWcontextOSMesa(ctypes.Structure):
    _fields_ = [
        ("handle", ctypes.c_void_p),
        ("width", ctypes.c_int32),
        ("height", ctypes.c_int32),
        ("buffer", ctypes.c_void_p),
    ]
    handle: int
    width: int
    height: int
    buffer: int

class _GLFWwindowWin32(ctypes.Structure):
    _fields_ = [
        ("handle", ctypes.c_ulonglong),
        ("bigIcon", ctypes.c_ulonglong),
        ("smallIcon", ctypes.c_ulonglong),
        ("cursorTracked", ctypes.c_int32),
        ("frameAction", ctypes.c_int32),
        ("iconified", ctypes.c_int32),
        ("maximized", ctypes.c_int32),
        ("transparent", ctypes.c_int32),
        ("scaleToMonitor", ctypes.c_int32),
        ("lastCursorPosX", ctypes.c_int32),
        ("lastCursorPosY", ctypes.c_int32),
        ("centerX", ctypes.c_int32),
        ("centerY", ctypes.c_int32),
        ("blankCursor", ctypes.c_ulonglong),
    ]
    handle: int
    bigIcon: int
    smallIcon: int
    cursorTracked: int
    frameAction: int
    iconified: int
    maximized: int
    transparent: int
    scaleToMonitor: int
    lastCursorPosX: int
    lastCursorPosY: int
    centerX: int
    centerY: int
    blankCursor: int

class _GLFWctxconfig(ctypes.Structure):
    _fields_ = [
        ("client", ctypes.c_int32),
        ("source", ctypes.c_int32),
        ("major", ctypes.c_int32),
        ("minor", ctypes.c_int32),
        ("forward", ctypes.c_int32),
        ("debug", ctypes.c_int32),
        ("noerror", ctypes.c_int32),
        ("profile", ctypes.c_int32),
        ("robustness", ctypes.c_int32),
        ("release", ctypes.c_int32),
        ("share", ctypes.c_ulonglong),
        ("nsgl", ctypes.c_char * 0x4),  # $89F3AC19F8EADB1D9F89B8C05502E84E
    ]
    client: int
    source: int
    major: int
    minor: int
    forward: int
    debug: int
    noerror: int
    profile: int
    robustness: int
    release: int
    share: int
    nsgl: bytes

class _GLFWerror(ctypes.Structure):
    _fields_ = [
        ("next", ctypes.c_ulonglong),
        ("code", ctypes.c_int32),
        ("description", ctypes.c_char * 0x400),  # char[1024]
    ]
    next: int
    code: int
    description: list[ctypes.c_char]

class _GLFWmapelement(ctypes.Structure):
    _fields_ = [
        ("type", ctypes.c_uint8),
        ("index", ctypes.c_uint8),
        ("axisScale", ctypes.c_char),
        ("axisOffset", ctypes.c_char),
    ]
    type: int
    index: int
    axisScale: str
    axisOffset: str

class _GLFWjoyobjectWin32(ctypes.Structure):
    _fields_ = [
        ("offset", ctypes.c_int32),
        ("type", ctypes.c_int32),
    ]
    offset: int
    type: int

class IDirectInputDevice8W(ctypes.Structure):
    _fields_ = [
        ("lpVtbl", ctypes.c_ulonglong),
    ]
    lpVtbl: int

class DIDEVCAPS(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("dwDevType", ctypes.c_uint32),
        ("dwAxes", ctypes.c_uint32),
        ("dwButtons", ctypes.c_uint32),
        ("dwPOVs", ctypes.c_uint32),
        ("dwFFSamplePeriod", ctypes.c_uint32),
        ("dwFFMinTimeResolution", ctypes.c_uint32),
        ("dwFirmwareRevision", ctypes.c_uint32),
        ("dwHardwareRevision", ctypes.c_uint32),
        ("dwFFDriverVersion", ctypes.c_uint32),
    ]
    dwSize: int
    dwFlags: int
    dwDevType: int
    dwAxes: int
    dwButtons: int
    dwPOVs: int
    dwFFSamplePeriod: int
    dwFFMinTimeResolution: int
    dwFirmwareRevision: int
    dwHardwareRevision: int
    dwFFDriverVersion: int

class DIPROPHEADER(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwHeaderSize", ctypes.c_uint32),
        ("dwObj", ctypes.c_uint32),
        ("dwHow", ctypes.c_uint32),
    ]
    dwSize: int
    dwHeaderSize: int
    dwObj: int
    dwHow: int

class DIDEVICEOBJECTDATA(ctypes.Structure):
    _fields_ = [
        ("dwOfs", ctypes.c_uint32),
        ("dwData", ctypes.c_uint32),
        ("dwTimeStamp", ctypes.c_uint32),
        ("dwSequence", ctypes.c_uint32),
        ("uAppData", ctypes.c_uint64),
    ]
    dwOfs: int
    dwData: int
    dwTimeStamp: int
    dwSequence: int
    uAppData: int

class _DIOBJECTDATAFORMAT(ctypes.Structure):
    _fields_ = [
        ("pguid", ctypes.c_ulonglong),
        ("dwOfs", ctypes.c_uint32),
        ("dwType", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
    ]
    pguid: int
    dwOfs: int
    dwType: int
    dwFlags: int

class _DIDATAFORMAT(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwObjSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("dwDataSize", ctypes.c_uint32),
        ("dwNumObjs", ctypes.c_uint32),
        ("rgodf", ctypes.c_ulonglong),
    ]
    dwSize: int
    dwObjSize: int
    dwFlags: int
    dwDataSize: int
    dwNumObjs: int
    rgodf: int

class DIENVELOPE(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwAttackLevel", ctypes.c_uint32),
        ("dwAttackTime", ctypes.c_uint32),
        ("dwFadeLevel", ctypes.c_uint32),
        ("dwFadeTime", ctypes.c_uint32),
    ]
    dwSize: int
    dwAttackLevel: int
    dwAttackTime: int
    dwFadeLevel: int
    dwFadeTime: int

class DIEFFECT(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwFlags", ctypes.c_uint32),
        ("dwDuration", ctypes.c_uint32),
        ("dwSamplePeriod", ctypes.c_uint32),
        ("dwGain", ctypes.c_uint32),
        ("dwTriggerButton", ctypes.c_uint32),
        ("dwTriggerRepeatInterval", ctypes.c_uint32),
        ("cAxes", ctypes.c_uint32),
        ("rgdwAxes", ctypes.c_ulonglong),
        ("rglDirection", ctypes.c_ulonglong),
        ("lpEnvelope", ctypes.c_ulonglong),
        ("cbTypeSpecificParams", ctypes.c_uint32),
        ("lpvTypeSpecificParams", ctypes.c_void_p),
        ("dwStartDelay", ctypes.c_uint32),
    ]
    dwSize: int
    dwFlags: int
    dwDuration: int
    dwSamplePeriod: int
    dwGain: int
    dwTriggerButton: int
    dwTriggerRepeatInterval: int
    cAxes: int
    rgdwAxes: int
    rglDirection: int
    lpEnvelope: int
    cbTypeSpecificParams: int
    lpvTypeSpecificParams: int
    dwStartDelay: int

class IDirectInputEffect(ctypes.Structure):
    _fields_ = [
        ("lpVtbl", ctypes.c_ulonglong),
    ]
    lpVtbl: int

class DIEFFESCAPE(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwCommand", ctypes.c_uint32),
        ("lpvInBuffer", ctypes.c_void_p),
        ("cbInBuffer", ctypes.c_uint32),
        ("lpvOutBuffer", ctypes.c_void_p),
        ("cbOutBuffer", ctypes.c_uint32),
    ]
    dwSize: int
    dwCommand: int
    lpvInBuffer: int
    cbInBuffer: int
    lpvOutBuffer: int
    cbOutBuffer: int

class IDirectInputEffect_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("GetEffectGuid", ctypes.c_ulonglong),
        ("GetParameters", ctypes.c_ulonglong),
        ("SetParameters", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Stop", ctypes.c_ulonglong),
        ("GetEffectStatus", ctypes.c_ulonglong),
        ("Download", ctypes.c_ulonglong),
        ("Unload", ctypes.c_ulonglong),
        ("Escape", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Initialize: int
    GetEffectGuid: int
    GetParameters: int
    SetParameters: int
    Start: int
    Stop: int
    GetEffectStatus: int
    Download: int
    Unload: int
    Escape: int

class IDirectInputEffectVtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("GetEffectGuid", ctypes.c_ulonglong),
        ("GetParameters", ctypes.c_ulonglong),
        ("SetParameters", ctypes.c_ulonglong),
        ("Start", ctypes.c_ulonglong),
        ("Stop", ctypes.c_ulonglong),
        ("GetEffectStatus", ctypes.c_ulonglong),
        ("Download", ctypes.c_ulonglong),
        ("Unload", ctypes.c_ulonglong),
        ("Escape", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Initialize: int
    GetEffectGuid: int
    GetParameters: int
    SetParameters: int
    Start: int
    Stop: int
    GetEffectStatus: int
    Download: int
    Unload: int
    Escape: int

class _DIDEVICEIMAGEINFOHEADERW(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("dwSizeImageInfo", ctypes.c_uint32),
        ("dwcViews", ctypes.c_uint32),
        ("dwcButtons", ctypes.c_uint32),
        ("dwcAxes", ctypes.c_uint32),
        ("dwcPOVs", ctypes.c_uint32),
        ("dwBufferSize", ctypes.c_uint32),
        ("dwBufferUsed", ctypes.c_uint32),
        ("lprgImageInfoArray", ctypes.c_ulonglong),
    ]
    dwSize: int
    dwSizeImageInfo: int
    dwcViews: int
    dwcButtons: int
    dwcAxes: int
    dwcPOVs: int
    dwBufferSize: int
    dwBufferUsed: int
    lprgImageInfoArray: int

class IDirectInputDevice8W_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetCapabilities", ctypes.c_ulonglong),
        ("EnumObjects", ctypes.c_ulonglong),
        ("GetProperty", ctypes.c_ulonglong),
        ("SetProperty", ctypes.c_ulonglong),
        ("Acquire", ctypes.c_ulonglong),
        ("Unacquire", ctypes.c_ulonglong),
        ("GetDeviceState", ctypes.c_ulonglong),
        ("GetDeviceData", ctypes.c_ulonglong),
        ("SetDataFormat", ctypes.c_ulonglong),
        ("SetEventNotification", ctypes.c_ulonglong),
        ("SetCooperativeLevel", ctypes.c_ulonglong),
        ("GetObjectInfo", ctypes.c_ulonglong),
        ("GetDeviceInfo", ctypes.c_ulonglong),
        ("RunControlPanel", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("CreateEffect", ctypes.c_ulonglong),
        ("EnumEffects", ctypes.c_ulonglong),
        ("GetEffectInfo", ctypes.c_ulonglong),
        ("GetForceFeedbackState", ctypes.c_ulonglong),
        ("SendForceFeedbackCommand", ctypes.c_ulonglong),
        ("EnumCreatedEffectObjects", ctypes.c_ulonglong),
        ("Escape", ctypes.c_ulonglong),
        ("Poll", ctypes.c_ulonglong),
        ("SendDeviceData", ctypes.c_ulonglong),
        ("EnumEffectsInFile", ctypes.c_ulonglong),
        ("WriteEffectToFile", ctypes.c_ulonglong),
        ("BuildActionMap", ctypes.c_ulonglong),
        ("SetActionMap", ctypes.c_ulonglong),
        ("GetImageInfo", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetCapabilities: int
    EnumObjects: int
    GetProperty: int
    SetProperty: int
    Acquire: int
    Unacquire: int
    GetDeviceState: int
    GetDeviceData: int
    SetDataFormat: int
    SetEventNotification: int
    SetCooperativeLevel: int
    GetObjectInfo: int
    GetDeviceInfo: int
    RunControlPanel: int
    Initialize: int
    CreateEffect: int
    EnumEffects: int
    GetEffectInfo: int
    GetForceFeedbackState: int
    SendForceFeedbackCommand: int
    EnumCreatedEffectObjects: int
    Escape: int
    Poll: int
    SendDeviceData: int
    EnumEffectsInFile: int
    WriteEffectToFile: int
    BuildActionMap: int
    SetActionMap: int
    GetImageInfo: int

class IDirectInputDevice8WVtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetCapabilities", ctypes.c_ulonglong),
        ("EnumObjects", ctypes.c_ulonglong),
        ("GetProperty", ctypes.c_ulonglong),
        ("SetProperty", ctypes.c_ulonglong),
        ("Acquire", ctypes.c_ulonglong),
        ("Unacquire", ctypes.c_ulonglong),
        ("GetDeviceState", ctypes.c_ulonglong),
        ("GetDeviceData", ctypes.c_ulonglong),
        ("SetDataFormat", ctypes.c_ulonglong),
        ("SetEventNotification", ctypes.c_ulonglong),
        ("SetCooperativeLevel", ctypes.c_ulonglong),
        ("GetObjectInfo", ctypes.c_ulonglong),
        ("GetDeviceInfo", ctypes.c_ulonglong),
        ("RunControlPanel", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("CreateEffect", ctypes.c_ulonglong),
        ("EnumEffects", ctypes.c_ulonglong),
        ("GetEffectInfo", ctypes.c_ulonglong),
        ("GetForceFeedbackState", ctypes.c_ulonglong),
        ("SendForceFeedbackCommand", ctypes.c_ulonglong),
        ("EnumCreatedEffectObjects", ctypes.c_ulonglong),
        ("Escape", ctypes.c_ulonglong),
        ("Poll", ctypes.c_ulonglong),
        ("SendDeviceData", ctypes.c_ulonglong),
        ("EnumEffectsInFile", ctypes.c_ulonglong),
        ("WriteEffectToFile", ctypes.c_ulonglong),
        ("BuildActionMap", ctypes.c_ulonglong),
        ("SetActionMap", ctypes.c_ulonglong),
        ("GetImageInfo", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetCapabilities: int
    EnumObjects: int
    GetProperty: int
    SetProperty: int
    Acquire: int
    Unacquire: int
    GetDeviceState: int
    GetDeviceData: int
    SetDataFormat: int
    SetEventNotification: int
    SetCooperativeLevel: int
    GetObjectInfo: int
    GetDeviceInfo: int
    RunControlPanel: int
    Initialize: int
    CreateEffect: int
    EnumEffects: int
    GetEffectInfo: int
    GetForceFeedbackState: int
    SendForceFeedbackCommand: int
    EnumCreatedEffectObjects: int
    Escape: int
    Poll: int
    SendDeviceData: int
    EnumEffectsInFile: int
    WriteEffectToFile: int
    BuildActionMap: int
    SetActionMap: int
    GetImageInfo: int

class _GLFWtlsWin32(ctypes.Structure):
    _fields_ = [
        ("allocated", ctypes.c_int32),
        ("index", ctypes.c_uint32),
    ]
    allocated: int
    index: int

class _GLFWtimerWin32(ctypes.Structure):
    _fields_ = [
        ("hasPC", ctypes.c_int32),
        ("frequency", ctypes.c_uint64),
    ]
    hasPC: int
    frequency: int

class IDirectInput8W(ctypes.Structure):
    _fields_ = [
        ("lpVtbl", ctypes.c_ulonglong),
    ]
    lpVtbl: int

class _DICOLORSET(ctypes.Structure):
    _fields_ = [
        ("dwSize", ctypes.c_uint32),
        ("cTextFore", ctypes.c_uint32),
        ("cTextHighlight", ctypes.c_uint32),
        ("cCalloutLine", ctypes.c_uint32),
        ("cCalloutHighlight", ctypes.c_uint32),
        ("cBorder", ctypes.c_uint32),
        ("cControlFill", ctypes.c_uint32),
        ("cHighlightFill", ctypes.c_uint32),
        ("cAreaFill", ctypes.c_uint32),
    ]
    dwSize: int
    cTextFore: int
    cTextHighlight: int
    cCalloutLine: int
    cCalloutHighlight: int
    cBorder: int
    cControlFill: int
    cHighlightFill: int
    cAreaFill: int

class IDirectInput8W_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("CreateDevice", ctypes.c_ulonglong),
        ("EnumDevices", ctypes.c_ulonglong),
        ("GetDeviceStatus", ctypes.c_ulonglong),
        ("RunControlPanel", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("FindDevice", ctypes.c_ulonglong),
        ("EnumDevicesBySemantics", ctypes.c_ulonglong),
        ("ConfigureDevices", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    CreateDevice: int
    EnumDevices: int
    GetDeviceStatus: int
    RunControlPanel: int
    Initialize: int
    FindDevice: int
    EnumDevicesBySemantics: int
    ConfigureDevices: int

class IDirectInput8WVtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("CreateDevice", ctypes.c_ulonglong),
        ("EnumDevices", ctypes.c_ulonglong),
        ("GetDeviceStatus", ctypes.c_ulonglong),
        ("RunControlPanel", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("FindDevice", ctypes.c_ulonglong),
        ("EnumDevicesBySemantics", ctypes.c_ulonglong),
        ("ConfigureDevices", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    CreateDevice: int
    EnumDevices: int
    GetDeviceStatus: int
    RunControlPanel: int
    Initialize: int
    FindDevice: int
    EnumDevicesBySemantics: int
    ConfigureDevices: int

class _XINPUT_CAPABILITIES(ctypes.Structure):
    pass

class DWM_BLURBEHIND(ctypes.Structure):
    _fields_ = [
        ("dwFlags", ctypes.c_uint32),
        ("fEnable", ctypes.c_int32),
        ("hRgnBlur", ctypes.c_ulonglong),
        ("fTransitionOnMaximized", ctypes.c_int32),
    ]
    dwFlags: int
    fEnable: int
    hRgnBlur: int
    fTransitionOnMaximized: int

class _GLFWlibraryWin32(ctypes.Structure):
    _fields_ = [
        ("helperWindowHandle", ctypes.c_ulonglong),
        ("deviceNotificationHandle", ctypes.c_void_p),
        ("foregroundLockTimeout", ctypes.c_uint32),
        ("acquiredMonitorCount", ctypes.c_int32),
        ("clipboardString", ctypes.c_char_p),
        ("keycodes", ctypes.c_int16 * 0x200),  # __int16[512]
        ("scancodes", ctypes.c_int16 * 0x15D),  # __int16[349]
        ("keynames", ctypes.c_char * 0x15D),  # char[349][5]
        ("restoreCursorPosX", ctypes.c_double),
        ("restoreCursorPosY", ctypes.c_double),
        ("disabledCursorWindow", ctypes.c_ulonglong),
        ("rawInput", ctypes.c_ulonglong),
        ("rawInputSize", ctypes.c_int32),
        ("mouseTrailSize", ctypes.c_uint32),
        ("winmm", ctypes.c_char * 0x10),  # $F16902C1344C95270D4315193168B8DD
        ("dinput8", ctypes.c_char * 0x18),  # $7205446BBB5406E252FD63D6D2D5159E
        ("xinput", ctypes.c_char * 0x18),  # $8CEFB0376425ACCDA2543A58E3684827
        ("user32", ctypes.c_char * 0x38),  # $2AFFA9D53A3A0054F5A376460FA9F837
        ("dwmapi", ctypes.c_char * 0x20),  # $513D3C5B24714E57CD1129FB446048DF
        ("shcore", ctypes.c_char * 0x18),  # $E4F34E2030E2275038CC93EFD0C0330A
        ("ntdll", ctypes.c_char * 0x10),  # $821809B137557F1B2A151705768746F4
    ]
    helperWindowHandle: int
    deviceNotificationHandle: int
    foregroundLockTimeout: int
    acquiredMonitorCount: int
    clipboardString: bytes
    keycodes: list[ctypes.c_int16]
    scancodes: list[ctypes.c_int16]
    keynames: list[ctypes.c_char]
    restoreCursorPosX: float
    restoreCursorPosY: float
    disabledCursorWindow: int
    rawInput: int
    rawInputSize: int
    mouseTrailSize: int
    winmm: bytes
    dinput8: bytes
    xinput: bytes
    user32: bytes
    dwmapi: bytes
    shcore: bytes
    ntdll: bytes

class _GLFWlibraryWGL(ctypes.Structure):
    _fields_ = [
        ("instance", ctypes.c_ulonglong),
        ("CreateContext", ctypes.c_ulonglong),
        ("DeleteContext", ctypes.c_ulonglong),
        ("GetProcAddress", ctypes.c_ulonglong),
        ("GetCurrentDC", ctypes.c_ulonglong),
        ("GetCurrentContext", ctypes.c_ulonglong),
        ("MakeCurrent", ctypes.c_ulonglong),
        ("ShareLists", ctypes.c_ulonglong),
        ("SwapIntervalEXT", ctypes.c_ulonglong),
        ("GetPixelFormatAttribivARB", ctypes.c_ulonglong),
        ("GetExtensionsStringEXT", ctypes.c_ulonglong),
        ("GetExtensionsStringARB", ctypes.c_ulonglong),
        ("CreateContextAttribsARB", ctypes.c_ulonglong),
        ("EXT_swap_control", ctypes.c_int32),
        ("EXT_colorspace", ctypes.c_int32),
        ("ARB_multisample", ctypes.c_int32),
        ("ARB_framebuffer_sRGB", ctypes.c_int32),
        ("EXT_framebuffer_sRGB", ctypes.c_int32),
        ("ARB_pixel_format", ctypes.c_int32),
        ("ARB_create_context", ctypes.c_int32),
        ("ARB_create_context_profile", ctypes.c_int32),
        ("EXT_create_context_es2_profile", ctypes.c_int32),
        ("ARB_create_context_robustness", ctypes.c_int32),
        ("ARB_create_context_no_error", ctypes.c_int32),
        ("ARB_context_flush_control", ctypes.c_int32),
    ]
    instance: int
    CreateContext: int
    DeleteContext: int
    GetProcAddress: int
    GetCurrentDC: int
    GetCurrentContext: int
    MakeCurrent: int
    ShareLists: int
    SwapIntervalEXT: int
    GetPixelFormatAttribivARB: int
    GetExtensionsStringEXT: int
    GetExtensionsStringARB: int
    CreateContextAttribsARB: int
    EXT_swap_control: int
    EXT_colorspace: int
    ARB_multisample: int
    ARB_framebuffer_sRGB: int
    EXT_framebuffer_sRGB: int
    ARB_pixel_format: int
    ARB_create_context: int
    ARB_create_context_profile: int
    EXT_create_context_es2_profile: int
    ARB_create_context_robustness: int
    ARB_create_context_no_error: int
    ARB_context_flush_control: int

class _GLFWlibraryEGL(ctypes.Structure):
    _fields_ = [
        ("display", ctypes.c_void_p),
        ("major", ctypes.c_int32),
        ("minor", ctypes.c_int32),
        ("prefix", ctypes.c_int32),
        ("KHR_create_context", ctypes.c_int32),
        ("KHR_create_context_no_error", ctypes.c_int32),
        ("KHR_gl_colorspace", ctypes.c_int32),
        ("KHR_get_all_proc_addresses", ctypes.c_int32),
        ("KHR_context_flush_control", ctypes.c_int32),
        ("handle", ctypes.c_void_p),
        ("GetConfigAttrib", ctypes.c_ulonglong),
        ("GetConfigs", ctypes.c_ulonglong),
        ("GetDisplay", ctypes.c_ulonglong),
        ("GetError", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Terminate", ctypes.c_ulonglong),
        ("BindAPI", ctypes.c_ulonglong),
        ("CreateContext", ctypes.c_ulonglong),
        ("DestroySurface", ctypes.c_ulonglong),
        ("DestroyContext", ctypes.c_ulonglong),
        ("CreateWindowSurface", ctypes.c_ulonglong),
        ("MakeCurrent", ctypes.c_ulonglong),
        ("SwapBuffers", ctypes.c_ulonglong),
        ("SwapInterval", ctypes.c_ulonglong),
        ("QueryString", ctypes.c_ulonglong),
        ("GetProcAddress", ctypes.c_ulonglong),
    ]
    display: int
    major: int
    minor: int
    prefix: int
    KHR_create_context: int
    KHR_create_context_no_error: int
    KHR_gl_colorspace: int
    KHR_get_all_proc_addresses: int
    KHR_context_flush_control: int
    handle: int
    GetConfigAttrib: int
    GetConfigs: int
    GetDisplay: int
    GetError: int
    Initialize: int
    Terminate: int
    BindAPI: int
    CreateContext: int
    DestroySurface: int
    DestroyContext: int
    CreateWindowSurface: int
    MakeCurrent: int
    SwapBuffers: int
    SwapInterval: int
    QueryString: int
    GetProcAddress: int

class _GLFWlibraryOSMesa(ctypes.Structure):
    _fields_ = [
        ("handle", ctypes.c_void_p),
        ("CreateContextExt", ctypes.c_ulonglong),
        ("CreateContextAttribs", ctypes.c_ulonglong),
        ("DestroyContext", ctypes.c_ulonglong),
        ("MakeCurrent", ctypes.c_ulonglong),
        ("GetColorBuffer", ctypes.c_ulonglong),
        ("GetDepthBuffer", ctypes.c_ulonglong),
        ("GetProcAddress", ctypes.c_ulonglong),
    ]
    handle: int
    CreateContextExt: int
    CreateContextAttribs: int
    DestroyContext: int
    MakeCurrent: int
    GetColorBuffer: int
    GetDepthBuffer: int
    GetProcAddress: int

class DIJOYSTATE(ctypes.Structure):
    _fields_ = [
        ("lX", ctypes.c_int32),
        ("lY", ctypes.c_int32),
        ("lZ", ctypes.c_int32),
        ("lRx", ctypes.c_int32),
        ("lRy", ctypes.c_int32),
        ("lRz", ctypes.c_int32),
        ("rglSlider", ctypes.c_int32 * 0x2),  # int[2]
        ("rgdwPOV", ctypes.c_uint32 * 0x4),  # unsigned int[4]
        ("rgbButtons", ctypes.c_uint8 * 0x20),  # unsigned __int8[32]
    ]
    lX: int
    lY: int
    lZ: int
    lRx: int
    lRy: int
    lRz: int
    rglSlider: list[ctypes.c_int32]
    rgdwPOV: list[ctypes.c_uint32]
    rgbButtons: list[ctypes.c_uint8]

class _GLFWobjenumWin32(ctypes.Structure):
    _fields_ = [
        ("device", ctypes.c_ulonglong),
        ("objects", ctypes.c_ulonglong),
        ("objectCount", ctypes.c_int32),
        ("axisCount", ctypes.c_int32),
        ("sliderCount", ctypes.c_int32),
        ("buttonCount", ctypes.c_int32),
        ("povCount", ctypes.c_int32),
    ]
    device: int
    objects: int
    objectCount: int
    axisCount: int
    sliderCount: int
    buttonCount: int
    povCount: int

class cGcPhonotactics(ctypes.Structure):
    def CheckConsonantPacking(): pass

    def VulgaritySweep(): pass

    def MakePetNameCuter(): pass


class cGcNetworkFleetSyncRequestMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNotificationSequenceUpgradeTech_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNotificationSequence_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("IsAppropriate", ctypes.c_ulonglong),
        ("GetMissionDataTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetTitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<128,char> *)
        ("GetSubtitle", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<256,char> *)
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetMissionIcon", ctypes.c_ulonglong),
        ("GetMissionIconSelected", ctypes.c_ulonglong),
        ("GetMissionIconNotSelected", ctypes.c_ulonglong),
        ("ShowFutureObjectives", ctypes.c_ulonglong),
        ("IsPrimary", ctypes.c_ulonglong),
        ("IsLegacy", ctypes.c_ulonglong),
        ("GetClass", ctypes.c_ulonglong),
        ("UseCommunityMissionForLog", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("IsCritical", ctypes.c_ulonglong),
        ("BlocksPinning", ctypes.c_ulonglong),
        ("IsProcedural", ctypes.c_ulonglong),
        ("DataAllowsRenounce", ctypes.c_ulonglong),
        ("RefreshLogInfo", ctypes.c_ulonglong),
        ("GetCurrentCategory", ctypes.c_ulonglong),
        ("GetCurrentColour", ctypes.c_ulonglong),
        ("GetCurrentIcon", ctypes.c_ulonglong),
        ("GetIconAtStage", ctypes.c_char * 0x8),  # TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *(__fastcall *)(cGcNotificationSequence *this, TkStrongType<int,TkStrongTypeIDs::TkResHandleID> *result, int)
        ("GetCurrentFrontendPageHint", ctypes.c_ulonglong),
        ("GetCurrentInventoryHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("GetCurrentFrontendPageLocID", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, TkID<256> *)
        ("GetCurrentBuildMenuHint", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcNotificationSequence *this, TkID<128> *result)
        ("CheckOptionCondition", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, const TkID<256> *, cTkFixedString<1024,char> *)
        ("WriteItemText", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *, bool)
        ("Restart", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("DebugNguiMissionInfo", ctypes.c_ulonglong),
        ("HasText", ctypes.c_ulonglong),
        ("GetText", ctypes.c_char * 0x8),  # bool (__fastcall *)(cGcNotificationSequence *this, cTkFixedString<512,char> *)
        ("GetParticipantTypeFromPointer", ctypes.c_ulonglong),
        ("GetParticipantPointerFromType", ctypes.c_ulonglong),
        ("GetNumberOfParticipants", ctypes.c_ulonglong),
        ("HasBeenHandedIn", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("Load", ctypes.c_ulonglong),
        ("Save", ctypes.c_ulonglong),
        ("CanBegin", ctypes.c_ulonglong),
        ("CanBeRenounced", ctypes.c_ulonglong),
        ("IsForcingPageHint", ctypes.c_ulonglong),
        ("IsForcingBuildMenuHint", ctypes.c_ulonglong),
        ("IsTelemetryUpload", ctypes.c_ulonglong),
        ("GetMultiplayerMissionType", ctypes.c_ulonglong),
    ]
    cGcNotificationSequence_dtor_0: int
    Update: int
    IsAppropriate: int
    GetMissionDataTitle: bytes
    GetTitle: bytes
    GetSubtitle: bytes
    GetDescription: bytes
    GetMissionIcon: int
    GetMissionIconSelected: int
    GetMissionIconNotSelected: int
    ShowFutureObjectives: int
    IsPrimary: int
    IsLegacy: int
    GetClass: int
    UseCommunityMissionForLog: bytes
    IsCritical: int
    BlocksPinning: int
    IsProcedural: int
    DataAllowsRenounce: int
    RefreshLogInfo: int
    GetCurrentCategory: int
    GetCurrentColour: int
    GetCurrentIcon: int
    GetIconAtStage: bytes
    GetCurrentFrontendPageHint: int
    GetCurrentInventoryHint: bytes
    GetCurrentFrontendPageLocID: bytes
    GetCurrentBuildMenuHint: bytes
    CheckOptionCondition: bytes
    WriteItemText: bytes
    Restart: int
    Cancel: int
    DebugNguiMissionInfo: int
    HasText: int
    GetText: bytes
    GetParticipantTypeFromPointer: int
    GetParticipantPointerFromType: int
    GetNumberOfParticipants: int
    HasBeenHandedIn: int
    NGuiRender: int
    Load: int
    Save: int
    CanBegin: int
    CanBeRenounced: int
    IsForcingPageHint: int
    IsForcingBuildMenuHint: int
    IsTelemetryUpload: int
    GetMultiplayerMissionType: int

class cGcNotificationSequenceCommon(ctypes.Structure):
    def GetGlobalMissionState(): pass

    def SetGlobalMissionState(): pass


class ScePadTriggerEffectOffParam(ctypes.Structure):
    _fields_ = [
        ("padding", ctypes.c_uint8 * 0x30),  # unsigned __int8[48]
    ]
    padding: list[ctypes.c_uint8]

class ScePadTriggerEffectFeedbackParam(ctypes.Structure):
    _fields_ = [
        ("position", ctypes.c_uint8),
        ("strength", ctypes.c_uint8),
        ("padding", ctypes.c_uint8 * 0x2E),  # unsigned __int8[46]
    ]
    position: int
    strength: int
    padding: list[ctypes.c_uint8]

class ScePadTriggerEffectWeaponParam(ctypes.Structure):
    _fields_ = [
        ("startPosition", ctypes.c_uint8),
        ("endPosition", ctypes.c_uint8),
        ("strength", ctypes.c_uint8),
        ("padding", ctypes.c_uint8 * 0x2D),  # unsigned __int8[45]
    ]
    startPosition: int
    endPosition: int
    strength: int
    padding: list[ctypes.c_uint8]

class ScePadTriggerEffectVibrationParam(ctypes.Structure):
    _fields_ = [
        ("position", ctypes.c_uint8),
        ("amplitude", ctypes.c_uint8),
        ("frequency", ctypes.c_uint8),
        ("padding", ctypes.c_uint8 * 0x2D),  # unsigned __int8[45]
    ]
    position: int
    amplitude: int
    frequency: int
    padding: list[ctypes.c_uint8]

class ScePadTriggerEffectMultiplePositionFeedbackParam(ctypes.Structure):
    _fields_ = [
        ("strength", ctypes.c_uint8 * 0xA),  # unsigned __int8[10]
        ("padding", ctypes.c_uint8 * 0x26),  # unsigned __int8[38]
    ]
    strength: list[ctypes.c_uint8]
    padding: list[ctypes.c_uint8]

class ScePadTriggerEffectSlopeFeedbackParam(ctypes.Structure):
    _fields_ = [
        ("startPosition", ctypes.c_uint8),
        ("endPosition", ctypes.c_uint8),
        ("startStrength", ctypes.c_uint8),
        ("endStrength", ctypes.c_uint8),
        ("padding", ctypes.c_uint8 * 0x2C),  # unsigned __int8[44]
    ]
    startPosition: int
    endPosition: int
    startStrength: int
    endStrength: int
    padding: list[ctypes.c_uint8]

class ScePadTriggerEffectMultiplePositionVibrationParam(ctypes.Structure):
    _fields_ = [
        ("frequency", ctypes.c_uint8),
        ("amplitude", ctypes.c_uint8 * 0xA),  # unsigned __int8[10]
        ("padding", ctypes.c_uint8 * 0x25),  # unsigned __int8[37]
    ]
    frequency: int
    amplitude: list[ctypes.c_uint8]
    padding: list[ctypes.c_uint8]

class ScePadTriggerEffectCommandData: pass

class cGcPrivilegeOption(ctypes.Structure):
    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x4B0),  # cGcListOption<enum ePrivilegeState>
    ]
    baseclass_0: bytes

class cGcPrivilegeOption_vtbl(ctypes.Structure):
    _fields_ = [
        ("UpdateUI", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("Apply", ctypes.c_ulonglong),
        ("ResetToDefault", ctypes.c_ulonglong),
        ("DiscardChanges", ctypes.c_ulonglong),
        ("HasChanges", ctypes.c_ulonglong),
        ("IsNonDefault", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("IsHovering", ctypes.c_ulonglong),
        ("cGcUIOptionListElement_dtor_50", ctypes.c_ulonglong),
        ("TranslateDescription", ctypes.c_ulonglong),
        ("CreateElement", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcOption<enum ePrivilegeState> *this)
        ("GetTranslatedDefaultValue", ctypes.c_char * 0x8),  # cTkFixedString<32,char> *(__fastcall *)(cGcOption<enum ePrivilegeState> *this, cTkFixedString<32,char> *result)
    ]
    UpdateUI: int
    Initialize: int
    Apply: int
    ResetToDefault: int
    DiscardChanges: int
    HasChanges: int
    IsNonDefault: int
    GetDescription: int
    GetName: int
    IsHovering: int
    cGcUIOptionListElement_dtor_50: int
    TranslateDescription: int
    CreateElement: bytes
    GetTranslatedDefaultValue: bytes

class cGcByteBeatSwitchComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class NVGtextRow(ctypes.Structure):
    _fields_ = [
        ("start", ctypes.c_char_p),
        ("end", ctypes.c_char_p),
        ("next", ctypes.c_char_p),
        ("width", ctypes.c_float),
        ("minx", ctypes.c_float),
        ("maxx", ctypes.c_float),
    ]
    start: bytes
    end: bytes
    next: bytes
    width: float
    minx: float
    maxx: float

class cGcConnectionPlayfab_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("Listen", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("SendMessageA", ctypes.c_char * 0x8),  # int (__fastcall *)(cGcConnection *this, const std::shared_ptr<cGcNetworkMessage> *, std::array<std::shared_ptr<cGcNetworkAddress>,32>, cGcNetworkConstants::TransmissionChannels, cGcNetworkConstants::LobbyType)
        ("ConnectToPlayer", ctypes.c_ulonglong),
        ("DisconnectPlayer", ctypes.c_ulonglong),
    ]
    Construct: int
    Prepare: int
    Release: int
    Destruct: int
    Listen: int
    Update: int
    SendMessageA: bytes
    ConnectToPlayer: int
    DisconnectPlayer: int

class cGcNotificationSequenceDataRenderer(ctypes.Structure):
    def SetSelectedSequence(): pass

    def RenderDataBrowser(): pass

    def RenderSelectMissionFromData(): pass

    def RenderMissionSequence(): pass


class cGcDistanceScaleComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcMissileComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcTurretComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class ComponentInstancercGcVehicleRaceInviteComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcNetworkBaseHeaderMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcNetworkMonumentSpawnedMessage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNetworkMessage_dtor_0", ctypes.c_ulonglong),
        ("SerializeOrDeserialize", ctypes.c_ulonglong),
        ("IsReadyToProcess", ctypes.c_ulonglong),
        ("CustomSerialization", ctypes.c_ulonglong),
    ]
    cGcNetworkMessage_dtor_0: int
    SerializeOrDeserialize: int
    IsReadyToProcess: int
    CustomSerialization: int

class cGcTargetScreen_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcRenderableScreen_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("CleanupResources", ctypes.c_ulonglong),
    ]
    cGcRenderableScreen_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    EnqueueRender: int
    UpdateRender: int
    Release: int
    Destruct: int
    RenderNGui: int
    CleanupResources: int

class ComponentInstancercGcNetworkPlayerMarkerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcNetworkPlayerMarkerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcObjectSpawnerComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcBaseDefenceComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcNPCHabitationComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcColouriseComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcShipAIFlybyIntercept_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIFlybyOverhead_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIFlybyClose_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class cGcShipAIStayClose_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("ShipAIConstruct", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    ShipAIConstruct: int

class ScePadInit2Param(ctypes.Structure):
    _fields_ = [
        ("allocFunc", ctypes.c_ulonglong),
        ("freeFunc", ctypes.c_ulonglong),
    ]
    allocFunc: int
    freeFunc: int

class ComponentInstancercGcCreatureFullBodyIKComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ScePadColor(ctypes.Structure):
    _fields_ = [
        ("r", ctypes.c_uint8),
        ("g", ctypes.c_uint8),
        ("b", ctypes.c_uint8),
        ("reserve", ctypes.c_uint8 * 0x1),  # unsigned __int8[1]
    ]
    r: int
    g: int
    b: int
    reserve: list[ctypes.c_uint8]

class cGcCreatureFullBodyIKComponentUpdater_vtbl(ctypes.Structure):
    _fields_ = [
        ("OnInit", ctypes.c_ulonglong),
        ("UpdateComponents", ctypes.c_ulonglong),
        ("UpdateComponentsPostPhysics", ctypes.c_ulonglong),
        ("UpdateComponentsRender", ctypes.c_ulonglong),
        ("OnComponentAllocate", ctypes.c_ulonglong),
        ("OnComponentFree", ctypes.c_ulonglong),
        ("HasCustomUpdate", ctypes.c_ulonglong),
    ]
    OnInit: int
    UpdateComponents: int
    UpdateComponentsPostPhysics: int
    UpdateComponentsRender: int
    OnComponentAllocate: int
    OnComponentFree: int
    HasCustomUpdate: int

class cGcCreatureFullBodyIKComponentUpdater(ctypes.Structure):
    def UpdateComponentsPostPhysics(): pass

    _fields_ = [
        ("baseclass_0", ctypes.c_char * 0x18),  # cTkTemplatedComponentUpdater<cGcCreatureFullBodyIKComponent>
    ]
    baseclass_0: bytes

class cGcPredatorBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcCheckEnergy_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcSharkBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcSnakeBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcRobotBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcQuadGuard_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcQuadCheckAlert_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcQuadPounce_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcQuadEvade_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcQuadAttackMovement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcQuadAttack_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcQuadSearch_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcQuadBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
        ("SetLookAtPoint", ctypes.c_ulonglong),
        ("AsWalkerTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int
    SetLookAtPoint: int
    AsWalkerTree: int

class cGcWalkerBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcWalkerLegArmourDestroyed_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcWalkerTitanFall_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcWalkerGuard_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcWalkerAttack_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcWalkerEnrage_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcWalkerStunned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcWalkerSearch_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class hmdRecti(ctypes.Structure):
    _fields_ = [
        ("x", ctypes.c_int32),
        ("y", ctypes.c_int32),
        ("w", ctypes.c_int32),
        ("h", ctypes.c_int32),
    ]
    x: int
    y: int
    w: int
    h: int

class cGcRobotCheckAlert_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class PlanetHolo(ctypes.Structure):
    _fields_ = [
        ("mNode", ctypes.c_uint32),
        ("miParent", ctypes.c_int32),
        ("mbVisited", ctypes.c_ubyte),
    ]
    mNode: int
    miParent: int
    mbVisited: bool

class cGcWeightedSpringSteering_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcNavSteering_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("PreUpdate", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("AddAvoidObstacles", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcNavSteering *this, cTkStackVector<cGcPathfindingObstacle,32,-1>)
        ("AddInterest", ctypes.c_ulonglong),
    ]
    cGcNavSteering_dtor_0: int
    Prepare: int
    Release: int
    Init: int
    PreUpdate: int
    Update: int
    AddAvoidObstacles: bytes
    AddInterest: int

class cGcResourceList(ctypes.Structure):
    pass

class cGcWiringSocketComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcCreatureSpawnComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class ComponentInstancercGcDroneComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcDroneAttackMovement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneControl(ctypes.Structure):
    def RepelFromNeighbours(): pass

    def RepelFromGround(): pass

    def MoveWithVelocity(): pass

    def LookAt(): pass

    def GetPerception(): pass

    def HasLineOfSight(): pass

    def NearestGroundPos(): pass


class cGcDroneAttackShooting_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneCheckAlert_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneGetAntagonistTarget_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneCoverMovement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneDoRepairs_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneFindPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDronePatrol_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("GetSearchCentrePos", ctypes.c_ulonglong),
        ("GetSearchRadius", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    GetSearchCentrePos: int
    GetSearchRadius: int

class cGcDroneIsOwnerWanted_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneFollowPlayer_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
        ("GetSearchCentrePos", ctypes.c_ulonglong),
        ("GetSearchRadius", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int
    GetSearchCentrePos: int
    GetSearchRadius: int

class cGcDronePlayerFocusBehaviour_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneInteract_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneMoodResponse_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneInvestigate_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDronePlaceCover_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneStunned_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneSummon_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneSummonMovement_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcDroneSummonAndDismiss_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cTkHeirarchicalRNG(ctypes.Structure):
    def Push(): pass

    _fields_ = [
        ("maStack", ctypes.c_char * 0x18),  # std::vector<cTkPersonalRNG,TkSTLAllocatorShim<cTkPersonalRNG,4,-1> >
    ]
    maStack: bytes

class cGcScanNameIndex(ctypes.Structure):
    _fields_ = [
        ("miTableIndex", ctypes.c_int32),
        ("miNameStart", ctypes.c_int32),
        ("miNameEnd", ctypes.c_int32),
    ]
    miTableIndex: int
    miNameStart: int
    miNameEnd: int

class BaseObjectLimits(ctypes.Structure):
    def UpdateLimits(): pass

    _fields_ = [
        ("muLocalCount", ctypes.c_uint32),
        ("muLocalLimit", ctypes.c_uint32),
        ("muBaseCount", ctypes.c_uint32),
        ("muBaseLimit", ctypes.c_uint32),
        ("muUACount", ctypes.c_uint32),
        ("muUALimit", ctypes.c_uint32),
    ]
    muLocalCount: int
    muLocalLimit: int
    muBaseCount: int
    muBaseLimit: int
    muUACount: int
    muUALimit: int

class cGcNPCBehaviourIlde_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcNPCBehaviourLookAtInteresting_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcNPCBehaviourMoveToDestination_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcNPCBehaviourReturnToShip_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cTkInputOpenVR(ctypes.Structure):
    pass

class cGcFreighterSyncComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcPolygoniserWater(ctypes.Structure):
    pass

class cGcBaseLinkGrid(ctypes.Structure):
    pass

class BuildingMarkerInfo(ctypes.Structure):
    _fields_ = [
        ("mId", ctypes.c_ulonglong),
        ("mpacName", ctypes.c_char_p),
        ("_meType", ctypes.c_uint32),
    ]
    mId: int
    mpacName: bytes
    _meType: int

    @property
    def meType(self):
        return cGcMarkerPoint.eType(self._meType)


class cGcVehicleGarageComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class LZ4_stream_t_internal(ctypes.Structure):
    _fields_ = [
        ("hashTable", ctypes.c_uint32 * 0x1000),  # unsigned int[4096]
        ("currentOffset", ctypes.c_uint32),
        ("initCheck", ctypes.c_uint32),
        ("dictionary", ctypes.c_ulonglong),
        ("bufferStart", ctypes.c_ulonglong),
        ("dictSize", ctypes.c_uint32),
    ]
    hashTable: list[ctypes.c_uint32]
    currentOffset: int
    initCheck: int
    dictionary: int
    bufferStart: int
    dictSize: int

class LZ4_streamDecode_t_internal(ctypes.Structure):
    _fields_ = [
        ("externalDict", ctypes.c_ulonglong),
        ("extDictSize", ctypes.c_uint64),
        ("prefixEnd", ctypes.c_ulonglong),
        ("prefixSize", ctypes.c_uint64),
    ]
    externalDict: int
    extDictSize: int
    prefixEnd: int
    prefixSize: int

class _DEV_BROADCAST_HDR(ctypes.Structure):
    _fields_ = [
        ("dbch_size", ctypes.c_uint32),
        ("dbch_devicetype", ctypes.c_uint32),
        ("dbch_reserved", ctypes.c_uint32),
    ]
    dbch_size: int
    dbch_devicetype: int
    dbch_reserved: int

class HDROP__(ctypes.Structure):
    _fields_ = [
        ("unused", ctypes.c_int32),
    ]
    unused: int

class cGcBeetleBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
        ("Type", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int
    Type: int

class cGcBirdBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcFishBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcJellyFishBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcRockCreatureBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcUngulateBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class TransferPopupData(ctypes.Structure):
    _fields_ = [
        ("mbAdditionalShip", ctypes.c_ubyte),
        ("miFinalCost", ctypes.c_int32),
    ]
    mbAdditionalShip: bool
    miFinalCost: int

class cGcNPCBehaviourDebugFollowWaypoints_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class IWICComponentInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetComponentType", ctypes.c_ulonglong),
        ("GetCLSID", ctypes.c_ulonglong),
        ("GetSigningStatus", ctypes.c_ulonglong),
        ("GetAuthor", ctypes.c_ulonglong),
        ("GetVendorGUID", ctypes.c_ulonglong),
        ("GetVersion", ctypes.c_ulonglong),
        ("GetSpecVersion", ctypes.c_ulonglong),
        ("GetFriendlyName", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetComponentType: int
    GetCLSID: int
    GetSigningStatus: int
    GetAuthor: int
    GetVendorGUID: int
    GetVersion: int
    GetSpecVersion: int
    GetFriendlyName: int

class IWICBitmapCodecInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetComponentType", ctypes.c_ulonglong),
        ("GetCLSID", ctypes.c_ulonglong),
        ("GetSigningStatus", ctypes.c_ulonglong),
        ("GetAuthor", ctypes.c_ulonglong),
        ("GetVendorGUID", ctypes.c_ulonglong),
        ("GetVersion", ctypes.c_ulonglong),
        ("GetSpecVersion", ctypes.c_ulonglong),
        ("GetFriendlyName", ctypes.c_ulonglong),
        ("GetContainerFormat", ctypes.c_ulonglong),
        ("GetPixelFormats", ctypes.c_ulonglong),
        ("GetColorManagementVersion", ctypes.c_ulonglong),
        ("GetDeviceManufacturer", ctypes.c_ulonglong),
        ("GetDeviceModels", ctypes.c_ulonglong),
        ("GetMimeTypes", ctypes.c_ulonglong),
        ("GetFileExtensions", ctypes.c_ulonglong),
        ("DoesSupportAnimation", ctypes.c_ulonglong),
        ("DoesSupportChromakey", ctypes.c_ulonglong),
        ("DoesSupportLossless", ctypes.c_ulonglong),
        ("DoesSupportMultiframe", ctypes.c_ulonglong),
        ("MatchesMimeType", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetComponentType: int
    GetCLSID: int
    GetSigningStatus: int
    GetAuthor: int
    GetVendorGUID: int
    GetVersion: int
    GetSpecVersion: int
    GetFriendlyName: int
    GetContainerFormat: int
    GetPixelFormats: int
    GetColorManagementVersion: int
    GetDeviceManufacturer: int
    GetDeviceModels: int
    GetMimeTypes: int
    GetFileExtensions: int
    DoesSupportAnimation: int
    DoesSupportChromakey: int
    DoesSupportLossless: int
    DoesSupportMultiframe: int
    MatchesMimeType: int

class IWICBitmapDecoderInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetComponentType", ctypes.c_ulonglong),
        ("GetCLSID", ctypes.c_ulonglong),
        ("GetSigningStatus", ctypes.c_ulonglong),
        ("GetAuthor", ctypes.c_ulonglong),
        ("GetVendorGUID", ctypes.c_ulonglong),
        ("GetVersion", ctypes.c_ulonglong),
        ("GetSpecVersion", ctypes.c_ulonglong),
        ("GetFriendlyName", ctypes.c_ulonglong),
        ("GetContainerFormat", ctypes.c_ulonglong),
        ("GetPixelFormats", ctypes.c_ulonglong),
        ("GetColorManagementVersion", ctypes.c_ulonglong),
        ("GetDeviceManufacturer", ctypes.c_ulonglong),
        ("GetDeviceModels", ctypes.c_ulonglong),
        ("GetMimeTypes", ctypes.c_ulonglong),
        ("GetFileExtensions", ctypes.c_ulonglong),
        ("DoesSupportAnimation", ctypes.c_ulonglong),
        ("DoesSupportChromakey", ctypes.c_ulonglong),
        ("DoesSupportLossless", ctypes.c_ulonglong),
        ("DoesSupportMultiframe", ctypes.c_ulonglong),
        ("MatchesMimeType", ctypes.c_ulonglong),
        ("GetPatterns", ctypes.c_ulonglong),
        ("MatchesPattern", ctypes.c_ulonglong),
        ("CreateInstance", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetComponentType: int
    GetCLSID: int
    GetSigningStatus: int
    GetAuthor: int
    GetVendorGUID: int
    GetVersion: int
    GetSpecVersion: int
    GetFriendlyName: int
    GetContainerFormat: int
    GetPixelFormats: int
    GetColorManagementVersion: int
    GetDeviceManufacturer: int
    GetDeviceModels: int
    GetMimeTypes: int
    GetFileExtensions: int
    DoesSupportAnimation: int
    DoesSupportChromakey: int
    DoesSupportLossless: int
    DoesSupportMultiframe: int
    MatchesMimeType: int
    GetPatterns: int
    MatchesPattern: int
    CreateInstance: int

class WICRect(ctypes.Structure):
    _fields_ = [
        ("X", ctypes.c_int32),
        ("Y", ctypes.c_int32),
        ("Width", ctypes.c_int32),
        ("Height", ctypes.c_int32),
    ]
    X: int
    Y: int
    Width: int
    Height: int

class IWICBitmapSource_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
        ("GetResolution", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("CopyPixels", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetPixelFormat: int
    GetResolution: int
    CopyPalette: int
    CopyPixels: int

class IWICPalette_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitializePredefined", ctypes.c_ulonglong),
        ("InitializeCustom", ctypes.c_ulonglong),
        ("InitializeFromBitmap", ctypes.c_ulonglong),
        ("InitializeFromPalette", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
        ("GetColorCount", ctypes.c_ulonglong),
        ("GetColors", ctypes.c_ulonglong),
        ("IsBlackWhite", ctypes.c_ulonglong),
        ("IsGrayscale", ctypes.c_ulonglong),
        ("HasAlpha", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    InitializePredefined: int
    InitializeCustom: int
    InitializeFromBitmap: int
    InitializeFromPalette: int
    GetType: int
    GetColorCount: int
    GetColors: int
    IsBlackWhite: int
    IsGrayscale: int
    HasAlpha: int

class IWICMetadataQueryReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetContainerFormat", ctypes.c_ulonglong),
        ("GetLocation", ctypes.c_ulonglong),
        ("GetMetadataByName", ctypes.c_ulonglong),
        ("GetEnumerator", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetContainerFormat: int
    GetLocation: int
    GetMetadataByName: int
    GetEnumerator: int

class IWICColorContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitializeFromFilename", ctypes.c_ulonglong),
        ("InitializeFromMemory", ctypes.c_ulonglong),
        ("InitializeFromExifColorSpace", ctypes.c_ulonglong),
        ("GetType", ctypes.c_ulonglong),
        ("GetProfileBytes", ctypes.c_ulonglong),
        ("GetExifColorSpace", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    InitializeFromFilename: int
    InitializeFromMemory: int
    InitializeFromExifColorSpace: int
    GetType: int
    GetProfileBytes: int
    GetExifColorSpace: int

class IWICBitmapFrameDecode_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
        ("GetResolution", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("CopyPixels", ctypes.c_ulonglong),
        ("GetMetadataQueryReader", ctypes.c_ulonglong),
        ("GetColorContexts", ctypes.c_ulonglong),
        ("GetThumbnail", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetPixelFormat: int
    GetResolution: int
    CopyPalette: int
    CopyPixels: int
    GetMetadataQueryReader: int
    GetColorContexts: int
    GetThumbnail: int

class IWICBitmapDecoder_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("QueryCapability", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("GetContainerFormat", ctypes.c_ulonglong),
        ("GetDecoderInfo", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("GetMetadataQueryReader", ctypes.c_ulonglong),
        ("GetPreview", ctypes.c_ulonglong),
        ("GetColorContexts", ctypes.c_ulonglong),
        ("GetThumbnail", ctypes.c_ulonglong),
        ("GetFrameCount", ctypes.c_ulonglong),
        ("GetFrame", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    QueryCapability: int
    Initialize: int
    GetContainerFormat: int
    GetDecoderInfo: int
    CopyPalette: int
    GetMetadataQueryReader: int
    GetPreview: int
    GetColorContexts: int
    GetThumbnail: int
    GetFrameCount: int
    GetFrame: int

class IWICBitmapEncoderInfo_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetComponentType", ctypes.c_ulonglong),
        ("GetCLSID", ctypes.c_ulonglong),
        ("GetSigningStatus", ctypes.c_ulonglong),
        ("GetAuthor", ctypes.c_ulonglong),
        ("GetVendorGUID", ctypes.c_ulonglong),
        ("GetVersion", ctypes.c_ulonglong),
        ("GetSpecVersion", ctypes.c_ulonglong),
        ("GetFriendlyName", ctypes.c_ulonglong),
        ("GetContainerFormat", ctypes.c_ulonglong),
        ("GetPixelFormats", ctypes.c_ulonglong),
        ("GetColorManagementVersion", ctypes.c_ulonglong),
        ("GetDeviceManufacturer", ctypes.c_ulonglong),
        ("GetDeviceModels", ctypes.c_ulonglong),
        ("GetMimeTypes", ctypes.c_ulonglong),
        ("GetFileExtensions", ctypes.c_ulonglong),
        ("DoesSupportAnimation", ctypes.c_ulonglong),
        ("DoesSupportChromakey", ctypes.c_ulonglong),
        ("DoesSupportLossless", ctypes.c_ulonglong),
        ("DoesSupportMultiframe", ctypes.c_ulonglong),
        ("MatchesMimeType", ctypes.c_ulonglong),
        ("CreateInstance", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetComponentType: int
    GetCLSID: int
    GetSigningStatus: int
    GetAuthor: int
    GetVendorGUID: int
    GetVersion: int
    GetSpecVersion: int
    GetFriendlyName: int
    GetContainerFormat: int
    GetPixelFormats: int
    GetColorManagementVersion: int
    GetDeviceManufacturer: int
    GetDeviceModels: int
    GetMimeTypes: int
    GetFileExtensions: int
    DoesSupportAnimation: int
    DoesSupportChromakey: int
    DoesSupportLossless: int
    DoesSupportMultiframe: int
    MatchesMimeType: int
    CreateInstance: int

class IErrorLog_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("AddError", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    AddError: int

class IPropertyBag2_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Read", ctypes.c_ulonglong),
        ("Write", ctypes.c_ulonglong),
        ("CountProperties", ctypes.c_ulonglong),
        ("GetPropertyInfo", ctypes.c_ulonglong),
        ("LoadObject", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Read: int
    Write: int
    CountProperties: int
    GetPropertyInfo: int
    LoadObject: int

class IWICMetadataQueryWriter_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetContainerFormat", ctypes.c_ulonglong),
        ("GetLocation", ctypes.c_ulonglong),
        ("GetMetadataByName", ctypes.c_ulonglong),
        ("GetEnumerator", ctypes.c_ulonglong),
        ("SetMetadataByName", ctypes.c_ulonglong),
        ("RemoveMetadataByName", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetContainerFormat: int
    GetLocation: int
    GetMetadataByName: int
    GetEnumerator: int
    SetMetadataByName: int
    RemoveMetadataByName: int

class IWICBitmapFrameEncode_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("SetResolution", ctypes.c_ulonglong),
        ("SetPixelFormat", ctypes.c_ulonglong),
        ("SetColorContexts", ctypes.c_ulonglong),
        ("SetPalette", ctypes.c_ulonglong),
        ("SetThumbnail", ctypes.c_ulonglong),
        ("WritePixels", ctypes.c_ulonglong),
        ("WriteSource", ctypes.c_ulonglong),
        ("Commit", ctypes.c_ulonglong),
        ("GetMetadataQueryWriter", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Initialize: int
    SetSize: int
    SetResolution: int
    SetPixelFormat: int
    SetColorContexts: int
    SetPalette: int
    SetThumbnail: int
    WritePixels: int
    WriteSource: int
    Commit: int
    GetMetadataQueryWriter: int

class IWICBitmapEncoder_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("GetContainerFormat", ctypes.c_ulonglong),
        ("GetEncoderInfo", ctypes.c_ulonglong),
        ("SetColorContexts", ctypes.c_ulonglong),
        ("SetPalette", ctypes.c_ulonglong),
        ("SetThumbnail", ctypes.c_ulonglong),
        ("SetPreview", ctypes.c_ulonglong),
        ("CreateNewFrame", ctypes.c_ulonglong),
        ("Commit", ctypes.c_ulonglong),
        ("GetMetadataQueryWriter", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Initialize: int
    GetContainerFormat: int
    GetEncoderInfo: int
    SetColorContexts: int
    SetPalette: int
    SetThumbnail: int
    SetPreview: int
    CreateNewFrame: int
    Commit: int
    GetMetadataQueryWriter: int

class IWICFormatConverter_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
        ("GetResolution", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("CopyPixels", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
        ("CanConvert", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetPixelFormat: int
    GetResolution: int
    CopyPalette: int
    CopyPixels: int
    Initialize: int
    CanConvert: int

class IWICBitmapScaler_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
        ("GetResolution", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("CopyPixels", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetPixelFormat: int
    GetResolution: int
    CopyPalette: int
    CopyPixels: int
    Initialize: int

class IWICBitmapClipper_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
        ("GetResolution", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("CopyPixels", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetPixelFormat: int
    GetResolution: int
    CopyPalette: int
    CopyPixels: int
    Initialize: int

class IWICBitmapFlipRotator_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
        ("GetResolution", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("CopyPixels", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetPixelFormat: int
    GetResolution: int
    CopyPalette: int
    CopyPixels: int
    Initialize: int

class IWICStream_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Read", ctypes.c_ulonglong),
        ("Write", ctypes.c_ulonglong),
        ("Seek", ctypes.c_ulonglong),
        ("SetSize", ctypes.c_ulonglong),
        ("CopyTo", ctypes.c_ulonglong),
        ("Commit", ctypes.c_ulonglong),
        ("Revert", ctypes.c_ulonglong),
        ("LockRegion", ctypes.c_ulonglong),
        ("UnlockRegion", ctypes.c_ulonglong),
        ("Stat", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
        ("InitializeFromIStream", ctypes.c_ulonglong),
        ("InitializeFromFilename", ctypes.c_ulonglong),
        ("InitializeFromMemory", ctypes.c_ulonglong),
        ("InitializeFromIStreamRegion", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Read: int
    Write: int
    Seek: int
    SetSize: int
    CopyTo: int
    Commit: int
    Revert: int
    LockRegion: int
    UnlockRegion: int
    Stat: int
    Clone: int
    InitializeFromIStream: int
    InitializeFromFilename: int
    InitializeFromMemory: int
    InitializeFromIStreamRegion: int

class IWICColorTransform_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
        ("GetResolution", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("CopyPixels", ctypes.c_ulonglong),
        ("Initialize", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetPixelFormat: int
    GetResolution: int
    CopyPalette: int
    CopyPixels: int
    Initialize: int

class IWICBitmapLock_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetStride", ctypes.c_ulonglong),
        ("GetDataPointer", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetStride: int
    GetDataPointer: int
    GetPixelFormat: int

class IWICBitmap_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("GetSize", ctypes.c_ulonglong),
        ("GetPixelFormat", ctypes.c_ulonglong),
        ("GetResolution", ctypes.c_ulonglong),
        ("CopyPalette", ctypes.c_ulonglong),
        ("CopyPixels", ctypes.c_ulonglong),
        ("Lock", ctypes.c_ulonglong),
        ("SetPalette", ctypes.c_ulonglong),
        ("SetResolution", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    GetSize: int
    GetPixelFormat: int
    GetResolution: int
    CopyPalette: int
    CopyPixels: int
    Lock: int
    SetPalette: int
    SetResolution: int

class IEnumUnknown_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Next", ctypes.c_ulonglong),
        ("Skip", ctypes.c_ulonglong),
        ("Reset", ctypes.c_ulonglong),
        ("Clone", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Next: int
    Skip: int
    Reset: int
    Clone: int

class IWICFastMetadataEncoder_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Commit", ctypes.c_ulonglong),
        ("GetMetadataQueryWriter", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    Commit: int
    GetMetadataQueryWriter: int

class IWICImagingFactory_vtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("CreateDecoderFromFilename", ctypes.c_ulonglong),
        ("CreateDecoderFromStream", ctypes.c_ulonglong),
        ("CreateDecoderFromFileHandle", ctypes.c_ulonglong),
        ("CreateComponentInfo", ctypes.c_ulonglong),
        ("CreateDecoder", ctypes.c_ulonglong),
        ("CreateEncoder", ctypes.c_ulonglong),
        ("CreatePalette", ctypes.c_ulonglong),
        ("CreateFormatConverter", ctypes.c_ulonglong),
        ("CreateBitmapScaler", ctypes.c_ulonglong),
        ("CreateBitmapClipper", ctypes.c_ulonglong),
        ("CreateBitmapFlipRotator", ctypes.c_ulonglong),
        ("CreateStream", ctypes.c_ulonglong),
        ("CreateColorContext", ctypes.c_ulonglong),
        ("CreateColorTransformer", ctypes.c_ulonglong),
        ("CreateBitmap", ctypes.c_ulonglong),
        ("CreateBitmapFromSource", ctypes.c_ulonglong),
        ("CreateBitmapFromSourceRect", ctypes.c_ulonglong),
        ("CreateBitmapFromMemory", ctypes.c_ulonglong),
        ("CreateBitmapFromHBITMAP", ctypes.c_ulonglong),
        ("CreateBitmapFromHICON", ctypes.c_ulonglong),
        ("CreateComponentEnumerator", ctypes.c_ulonglong),
        ("CreateFastMetadataEncoderFromDecoder", ctypes.c_ulonglong),
        ("CreateFastMetadataEncoderFromFrameDecode", ctypes.c_ulonglong),
        ("CreateQueryWriter", ctypes.c_ulonglong),
        ("CreateQueryWriterFromReader", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int
    CreateDecoderFromFilename: int
    CreateDecoderFromStream: int
    CreateDecoderFromFileHandle: int
    CreateComponentInfo: int
    CreateDecoder: int
    CreateEncoder: int
    CreatePalette: int
    CreateFormatConverter: int
    CreateBitmapScaler: int
    CreateBitmapClipper: int
    CreateBitmapFlipRotator: int
    CreateStream: int
    CreateColorContext: int
    CreateColorTransformer: int
    CreateBitmap: int
    CreateBitmapFromSource: int
    CreateBitmapFromSourceRect: int
    CreateBitmapFromMemory: int
    CreateBitmapFromHBITMAP: int
    CreateBitmapFromHICON: int
    CreateComponentEnumerator: int
    CreateFastMetadataEncoderFromDecoder: int
    CreateFastMetadataEncoderFromFrameDecode: int
    CreateQueryWriter: int
    CreateQueryWriterFromReader: int

class cGcNGuiHeavyAirViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class cGcNGuiPipelineViewer_vtbl(ctypes.Structure):
    _fields_ = [
        ("Construct", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("EnqueueRender3D", ctypes.c_ulonglong),
        ("Render2D", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("DoGui", ctypes.c_ulonglong),
    ]
    Construct: int
    Update: int
    EnqueueRender3D: int
    Render2D: int
    Destruct: int
    DoGui: int

class GuiGuide(ctypes.Structure):
    _fields_ = [
        ("mAnchor", common.Vector2f),
        ("mOffset", common.Vector2f),
        ("mSize", common.Vector2f),
        ("mColour", common.Colour),
    ]
    mAnchor: common.Vector2f
    mOffset: common.Vector2f
    mSize: common.Vector2f
    mColour: common.Colour

class cGcGroundNavigator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcEnvironmentNavigator_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("ApplyToSteering", ctypes.c_ulonglong),
        ("IsPointNavigable", ctypes.c_ulonglong),
        ("CanPathToPoint", ctypes.c_ulonglong),
        ("GetNearestAccessiblePoint", ctypes.c_ulonglong),
        ("IsStuck", ctypes.c_ulonglong),
    ]
    cGcEnvironmentNavigator_dtor_0: int
    Prepare: int
    Release: int
    Init: int
    Update: int
    ApplyToSteering: int
    IsPointNavigable: int
    CanPathToPoint: int
    GetNearestAccessiblePoint: int
    IsStuck: int

class ComponentInstancercGcThereminComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class ComponentInstancercGcSquadronHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("Register", ctypes.c_ulonglong),
    ]
    Register: int

class cGcSquadronHologramComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcThereminComponent_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkComponent_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("UpdateRender", ctypes.c_ulonglong),
        ("Render", ctypes.c_ulonglong),
        ("NetSerialize", ctypes.c_ulonglong),
        ("NetSerializeOnInit", ctypes.c_ulonglong),
        ("CanTransferOwnership", ctypes.c_ulonglong),
        ("AmInterestedInOwnership", ctypes.c_ulonglong),
        ("ShouldKeepAlive", ctypes.c_ulonglong),
        ("ShouldAbandon", ctypes.c_ulonglong),
        ("OnOwnershipTransfer", ctypes.c_char * 0x8),  # bool (__fastcall *)(cTkComponent *this, const cTkUserIdBase<cTkFixedString<64,char> > *)
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetTypeNameHash64", ctypes.c_ulonglong),
        ("OnAttached", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("InitData", ctypes.c_ulonglong),
        ("DefaultRenderNGui", ctypes.c_ulonglong),
        ("GetUpdatePeriod", ctypes.c_ulonglong),
    ]
    cTkComponent_dtor_0: int
    Update: int
    UpdatePostPhysics: int
    UpdateRender: int
    Render: int
    NetSerialize: int
    NetSerializeOnInit: int
    CanTransferOwnership: int
    AmInterestedInOwnership: int
    ShouldKeepAlive: int
    ShouldAbandon: int
    OnOwnershipTransfer: bytes
    RenderNGui: int
    GetTypeNameHash64: int
    OnAttached: int
    Prepare: int
    Release: int
    InitData: int
    DefaultRenderNGui: int
    GetUpdatePeriod: int

class cGcSwarmFlee_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcSwarmMotion_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcSwarmLand_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcPlayAnim_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTreeNode_dtor_0", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Cancel", ctypes.c_ulonglong),
        ("Subscribe", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkMetaMessageMap<2> *)
        ("AddNode", ctypes.c_ulonglong),
        ("GetNumChildren", ctypes.c_ulonglong),
        ("GetChild", ctypes.c_ulonglong),
        ("GetName", ctypes.c_ulonglong),
        ("GetDescription", ctypes.c_char * 0x8),  # void (__fastcall *)(cTkBehaviourTreeNode *this, cTkFixedString<64,char> *)
        ("ToggleDebugBreak", ctypes.c_ulonglong),
        ("CanDebugBreak", ctypes.c_ulonglong),
        ("IsDebugBreakSet", ctypes.c_ulonglong),
        ("CanExpand", ctypes.c_ulonglong),
        ("NGuiRender", ctypes.c_ulonglong),
        ("OnInit", ctypes.c_ulonglong),
        ("Run", ctypes.c_ulonglong),
        ("OnStop", ctypes.c_ulonglong),
    ]
    cTkBehaviourTreeNode_dtor_0: int
    Update: int
    Cancel: int
    Subscribe: bytes
    AddNode: int
    GetNumChildren: int
    GetChild: int
    GetName: int
    GetDescription: bytes
    ToggleDebugBreak: int
    CanDebugBreak: int
    IsDebugBreakSet: int
    CanExpand: int
    NGuiRender: int
    OnInit: int
    Run: int
    OnStop: int

class cGcMaxStat(ctypes.Structure):
    _fields_ = [
        ("miVal", ctypes.c_char * 0x4),  # std::atomic<int>
    ]
    miVal: bytes

class cGcAirNavigator_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcEnvironmentNavigator_dtor_0", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Init", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("ApplyToSteering", ctypes.c_ulonglong),
        ("IsPointNavigable", ctypes.c_ulonglong),
        ("CanPathToPoint", ctypes.c_ulonglong),
        ("GetNearestAccessiblePoint", ctypes.c_ulonglong),
        ("IsStuck", ctypes.c_ulonglong),
    ]
    cGcEnvironmentNavigator_dtor_0: int
    Prepare: int
    Release: int
    Init: int
    Update: int
    ApplyToSteering: int
    IsPointNavigable: int
    CanPathToPoint: int
    GetNearestAccessiblePoint: int
    IsStuck: int

class IUnknownVtbl(ctypes.Structure):
    _fields_ = [
        ("QueryInterface", ctypes.c_ulonglong),
        ("AddRef", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
    ]
    QueryInterface: int
    AddRef: int
    Release: int

class GLFWgamepadstate(ctypes.Structure):
    _fields_ = [
        ("buttons", ctypes.c_uint8 * 0xF),  # unsigned __int8[15]
        ("axes", ctypes.c_float * 0x6),  # float[6]
    ]
    buttons: list[ctypes.c_uint8]
    axes: list[ctypes.c_float]

class GLFWimage(ctypes.Structure):
    _fields_ = [
        ("width", ctypes.c_int32),
        ("height", ctypes.c_int32),
        ("pixels", ctypes.c_ulonglong),
    ]
    width: int
    height: int
    pixels: int

class GUID(ctypes.Structure):
    _fields_ = [
        ("Data1", ctypes.c_uint32),
        ("Data2", ctypes.c_uint16),
        ("Data3", ctypes.c_uint16),
        ("Data4", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    Data1: int
    Data2: int
    Data3: int
    Data4: list[ctypes.c_uint8]

class CAkBusVolumes_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkGameObjectPluginInfo_dtor_0", ctypes.c_ulonglong),
        ("GetGameObjectID", ctypes.c_ulonglong),
        ("GetNumEmitterListenerPairs", ctypes.c_ulonglong),
        ("GetEmitterListenerPair", ctypes.c_ulonglong),
        ("GetNumGameObjectPositions", ctypes.c_ulonglong),
        ("GetGameObjectPosition", ctypes.c_ulonglong),
        ("GetGameObjectMultiPositionType", ctypes.c_ulonglong),
        ("GetGameObjectScaling", ctypes.c_ulonglong),
        ("GetListeners", ctypes.c_ulonglong),
        ("GetListenerData", ctypes.c_ulonglong),
        ("_GetVoiceInfo", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("GetOutputConfig", ctypes.c_ulonglong),
    ]
    IAkGameObjectPluginInfo_dtor_0: int
    GetGameObjectID: int
    GetNumEmitterListenerPairs: int
    GetEmitterListenerPair: int
    GetNumGameObjectPositions: int
    GetGameObjectPosition: int
    GetGameObjectMultiPositionType: int
    GetGameObjectScaling: int
    GetListeners: int
    GetListenerData: int
    _GetVoiceInfo: int
    GetAnalyzedEnvelope: int
    GetOutputConfig: int

class CAkChainCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
        ("EndInit", ctypes.c_ulonglong),
        ("CanKeepPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory_80", ctypes.c_ulonglong),
        ("GetPlaybackHistory", ctypes.c_ulonglong),
        ("Process", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("GetNextScheduleWindow", ctypes.c_ulonglong),
        ("RefreshWindow", ctypes.c_ulonglong),
        ("QueryLookAheadInfo", ctypes.c_ulonglong),
        ("Node", ctypes.c_ulonglong),
        ("CanRestartPlaying", ctypes.c_ulonglong),
        ("CancelPlayback", ctypes.c_ulonglong),
        ("PerformDelayedSwitchChange", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("GetPlayingSegmentInfo", ctypes.c_ulonglong),
        ("IsValid", ctypes.c_ulonglong),
        ("Grow", ctypes.c_ulonglong),
        ("UpdateChainLength", ctypes.c_ulonglong),
        ("HandleFatalError", ctypes.c_ulonglong),
        ("PlaylistJump", ctypes.c_ulonglong),
        ("NotifyItemSync", ctypes.c_ulonglong),
        ("NotifyItemFlush", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int
    EndInit: int
    CanKeepPlaybackHistory: int
    SetPlaybackHistory: int
    SetPlaybackHistory_80: int
    GetPlaybackHistory: int
    Process: int
    Prepare: int
    GetNextScheduleWindow: int
    RefreshWindow: int
    QueryLookAheadInfo: int
    Node: int
    CanRestartPlaying: int
    CancelPlayback: int
    PerformDelayedSwitchChange: int
    SeekTimeAbsolute: int
    SeekPercent: int
    GetPlayingSegmentInfo: int
    IsValid: int
    Grow: int
    UpdateChainLength: int
    HandleFatalError: int
    PlaylistJump: int
    NotifyItemSync: int
    NotifyItemFlush: int

class CAkContinuousPBI_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("CAkTransportAware_dtor_8", ctypes.c_ulonglong),
        ("SetEstimatedLength", ctypes.c_ulonglong),
        ("_Pause", ctypes.c_ulonglong),
        ("_Resume", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekTimeAbsolute_38", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("SetPauseStateForContinuous", ctypes.c_ulonglong),
        ("PrepareSampleAccurateTransition", ctypes.c_ulonglong),
        ("NotifyVirtualOff", ctypes.c_ulonglong),
        ("MonitorFade", ctypes.c_ulonglong),
        ("GetAndClearStopOffset", ctypes.c_ulonglong),
        ("PrepareNextToPlay", ctypes.c_ulonglong),
    ]
    _Stop: int
    CAkTransportAware_dtor_8: int
    SetEstimatedLength: int
    _Pause: int
    _Resume: int
    PlayToEnd: int
    SeekTimeAbsolute: int
    SeekTimeAbsolute_38: int
    SeekPercent: int
    SetPauseStateForContinuous: int
    PrepareSampleAccurateTransition: int
    NotifyVirtualOff: int
    MonitorFade: int
    GetAndClearStopOffset: int
    PrepareNextToPlay: int

class CAkDynamicSequencePBI_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("CAkTransportAware_dtor_8", ctypes.c_ulonglong),
        ("SetEstimatedLength", ctypes.c_ulonglong),
        ("_Pause", ctypes.c_ulonglong),
        ("_Resume", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekTimeAbsolute_38", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("SetPauseStateForContinuous", ctypes.c_ulonglong),
        ("PrepareSampleAccurateTransition", ctypes.c_ulonglong),
        ("NotifyVirtualOff", ctypes.c_ulonglong),
        ("MonitorFade", ctypes.c_ulonglong),
        ("GetAndClearStopOffset", ctypes.c_ulonglong),
        ("PrepareNextToPlay", ctypes.c_ulonglong),
    ]
    _Stop: int
    CAkTransportAware_dtor_8: int
    SetEstimatedLength: int
    _Pause: int
    _Resume: int
    PlayToEnd: int
    SeekTimeAbsolute: int
    SeekTimeAbsolute_38: int
    SeekPercent: int
    SetPauseStateForContinuous: int
    PrepareSampleAccurateTransition: int
    NotifyVirtualOff: int
    MonitorFade: int
    GetAndClearStopOffset: int
    PrepareNextToPlay: int

class CAkNothingCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
        ("EndInit", ctypes.c_ulonglong),
        ("CanKeepPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory_80", ctypes.c_ulonglong),
        ("GetPlaybackHistory", ctypes.c_ulonglong),
        ("Process", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("GetNextScheduleWindow", ctypes.c_ulonglong),
        ("RefreshWindow", ctypes.c_ulonglong),
        ("QueryLookAheadInfo", ctypes.c_ulonglong),
        ("Node", ctypes.c_ulonglong),
        ("CanRestartPlaying", ctypes.c_ulonglong),
        ("CancelPlayback", ctypes.c_ulonglong),
        ("PerformDelayedSwitchChange", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("GetPlayingSegmentInfo", ctypes.c_ulonglong),
        ("IsValid", ctypes.c_ulonglong),
        ("Grow", ctypes.c_ulonglong),
        ("UpdateChainLength", ctypes.c_ulonglong),
        ("HandleFatalError", ctypes.c_ulonglong),
        ("PlaylistJump", ctypes.c_ulonglong),
        ("NotifyItemSync", ctypes.c_ulonglong),
        ("NotifyItemFlush", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int
    EndInit: int
    CanKeepPlaybackHistory: int
    SetPlaybackHistory: int
    SetPlaybackHistory_80: int
    GetPlaybackHistory: int
    Process: int
    Prepare: int
    GetNextScheduleWindow: int
    RefreshWindow: int
    QueryLookAheadInfo: int
    Node: int
    CanRestartPlaying: int
    CancelPlayback: int
    PerformDelayedSwitchChange: int
    SeekTimeAbsolute: int
    SeekPercent: int
    GetPlayingSegmentInfo: int
    IsValid: int
    Grow: int
    UpdateChainLength: int
    HandleFatalError: int
    PlaylistJump: int
    NotifyItemSync: int
    NotifyItemFlush: int

class CAkParameterNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int

class CAkSequencableSegmentCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
        ("EndInit", ctypes.c_ulonglong),
        ("CanKeepPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory_80", ctypes.c_ulonglong),
        ("GetPlaybackHistory", ctypes.c_ulonglong),
        ("Process", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("GetNextScheduleWindow", ctypes.c_ulonglong),
        ("RefreshWindow", ctypes.c_ulonglong),
        ("QueryLookAheadInfo", ctypes.c_ulonglong),
        ("Node", ctypes.c_ulonglong),
        ("CanRestartPlaying", ctypes.c_ulonglong),
        ("CancelPlayback", ctypes.c_ulonglong),
        ("PerformDelayedSwitchChange", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("GetPlayingSegmentInfo", ctypes.c_ulonglong),
        ("IsValid", ctypes.c_ulonglong),
        ("Grow", ctypes.c_ulonglong),
        ("UpdateChainLength", ctypes.c_ulonglong),
        ("HandleFatalError", ctypes.c_ulonglong),
        ("PlaylistJump", ctypes.c_ulonglong),
        ("NotifyItemSync", ctypes.c_ulonglong),
        ("NotifyItemFlush", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int
    EndInit: int
    CanKeepPlaybackHistory: int
    SetPlaybackHistory: int
    SetPlaybackHistory_80: int
    GetPlaybackHistory: int
    Process: int
    Prepare: int
    GetNextScheduleWindow: int
    RefreshWindow: int
    QueryLookAheadInfo: int
    Node: int
    CanRestartPlaying: int
    CancelPlayback: int
    PerformDelayedSwitchChange: int
    SeekTimeAbsolute: int
    SeekPercent: int
    GetPlayingSegmentInfo: int
    IsValid: int
    Grow: int
    UpdateChainLength: int
    HandleFatalError: int
    PlaylistJump: int
    NotifyItemSync: int
    NotifyItemFlush: int

class CAkSequenceCtx_vtbl(ctypes.Structure):
    _fields_ = [
        ("_Stop", ctypes.c_ulonglong),
        ("OnFrameEnd", ctypes.c_ulonglong),
        ("OnLastFrame", ctypes.c_ulonglong),
        ("OnPaused", ctypes.c_ulonglong),
        ("OnResumed", ctypes.c_ulonglong),
        ("SetPBIFade", ctypes.c_ulonglong),
        ("OnPropertyChange", ctypes.c_ulonglong),
        ("OnTriggerModulators", ctypes.c_ulonglong),
        ("VirtualAddRef", ctypes.c_ulonglong),
        ("VirtualRelease", ctypes.c_ulonglong),
        ("CAkChildCtx_dtor_50", ctypes.c_ulonglong),
        ("OnPlayed", ctypes.c_ulonglong),
        ("OnStopped", ctypes.c_ulonglong),
        ("EndInit", ctypes.c_ulonglong),
        ("CanKeepPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory", ctypes.c_ulonglong),
        ("SetPlaybackHistory_80", ctypes.c_ulonglong),
        ("GetPlaybackHistory", ctypes.c_ulonglong),
        ("Process", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("GetNextScheduleWindow", ctypes.c_ulonglong),
        ("RefreshWindow", ctypes.c_ulonglong),
        ("QueryLookAheadInfo", ctypes.c_ulonglong),
        ("Node", ctypes.c_ulonglong),
        ("CanRestartPlaying", ctypes.c_ulonglong),
        ("CancelPlayback", ctypes.c_ulonglong),
        ("PerformDelayedSwitchChange", ctypes.c_ulonglong),
        ("SeekTimeAbsolute", ctypes.c_ulonglong),
        ("SeekPercent", ctypes.c_ulonglong),
        ("GetPlayingSegmentInfo", ctypes.c_ulonglong),
        ("IsValid", ctypes.c_ulonglong),
        ("Grow", ctypes.c_ulonglong),
        ("UpdateChainLength", ctypes.c_ulonglong),
        ("HandleFatalError", ctypes.c_ulonglong),
        ("PlaylistJump", ctypes.c_ulonglong),
        ("NotifyItemSync", ctypes.c_ulonglong),
        ("NotifyItemFlush", ctypes.c_ulonglong),
    ]
    _Stop: int
    OnFrameEnd: int
    OnLastFrame: int
    OnPaused: int
    OnResumed: int
    SetPBIFade: int
    OnPropertyChange: int
    OnTriggerModulators: int
    VirtualAddRef: int
    VirtualRelease: int
    CAkChildCtx_dtor_50: int
    OnPlayed: int
    OnStopped: int
    EndInit: int
    CanKeepPlaybackHistory: int
    SetPlaybackHistory: int
    SetPlaybackHistory_80: int
    GetPlaybackHistory: int
    Process: int
    Prepare: int
    GetNextScheduleWindow: int
    RefreshWindow: int
    QueryLookAheadInfo: int
    Node: int
    CanRestartPlaying: int
    CancelPlayback: int
    PerformDelayedSwitchChange: int
    SeekTimeAbsolute: int
    SeekPercent: int
    GetPlayingSegmentInfo: int
    IsValid: int
    Grow: int
    UpdateChainLength: int
    HandleFatalError: int
    PlaylistJump: int
    NotifyItemSync: int
    NotifyItemFlush: int

class CAkSoundBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int

class CAkSound_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int

class cGcCreatureGroundNavigation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCreatureNavigation_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetMode", ctypes.c_ulonglong),
        ("GetFrontNavMatrix", ctypes.c_ulonglong),
        ("GetTurnPivot", ctypes.c_ulonglong),
        ("InitialAdjustToKnowledge", ctypes.c_ulonglong),
        ("AdjustToKnowledge", ctypes.c_ulonglong),
        ("NavigateInRoughDirection", ctypes.c_ulonglong),
        ("NavigateToPoint", ctypes.c_char * 0x8),  # cTkStackVector<cGcNavPolyHandle,64,-1> *(__fastcall *)(cGcCreatureNavigation *this, const cTkPhysRelVec3 *, const cGcNavPolyHandle *)
        ("IsPointAccessible", ctypes.c_ulonglong),
        ("HasPath", ctypes.c_ulonglong),
        ("CopyPathFrom", ctypes.c_ulonglong),
        ("SetMoveType", ctypes.c_ulonglong),
        ("SetStealthy", ctypes.c_ulonglong),
        ("RequestIdle", ctypes.c_char * 0x8),  # cTkAnimLayerHandle *(__fastcall *)(cGcCreatureNavigation *this, cTkAnimLayerHandle *result, const TkID<128> *, float, cGcCreatureMoveSpeedRequest, bool, float)
        ("GetIdleLength", ctypes.c_char * 0x8),  # float (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *)
        ("ForceSingleAnim", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *, float, float, bool, float)
        ("HasForcedSingleAnim", ctypes.c_ulonglong),
        ("GetForcedSingleAnimHandle", ctypes.c_ulonglong),
        ("CancelForcedSingleAnim", ctypes.c_ulonglong),
        ("PlaySubAnim", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *, float, float, float, bool)
        ("AsGroundNavigation", ctypes.c_ulonglong),
        ("AsSwarmNavigation", ctypes.c_ulonglong),
        ("GetTurnRadius", ctypes.c_ulonglong),
        ("GetMoveType", ctypes.c_ulonglong),
        ("SetWantsToLand", ctypes.c_ulonglong),
        ("GetLandingProgress", ctypes.c_ulonglong),
        ("GetStealthy", ctypes.c_ulonglong),
        ("GetAttackAnimCandidates", ctypes.c_char * 0x8),  # unsigned int (__fastcall *)(cGcCreatureNavigation *this, TkID<128> *, unsigned int)
        ("ZeroSteering", ctypes.c_ulonglong),
        ("GetRunAnimSpeed", ctypes.c_ulonglong),
        ("CanPounce", ctypes.c_ulonglong),
        ("StartPounce", ctypes.c_char * 0x8),  # cTkAnimInstanceHandle *(__fastcall *)(cGcCreatureNavigation *this, cTkAnimInstanceHandle *result, const TkID<128> *, const cTkPhysRelVec3 *, bool, bool, bool)
        ("UpdatePounce", ctypes.c_ulonglong),
        ("StartLocomotionDamage", ctypes.c_ulonglong),
        ("UpdateLocomotionDamage", ctypes.c_ulonglong),
        ("GetSteeringDirections", ctypes.c_ulonglong),
        ("PointIsInsideNavMap", ctypes.c_ulonglong),
        ("ShouldDespawn", ctypes.c_ulonglong),
        ("RequestRefreshNavMap", ctypes.c_ulonglong),
        ("SetMoveSpeeds", ctypes.c_ulonglong),
        ("SetSpeedFactor", ctypes.c_ulonglong),
        ("GetDesiredRidingAnim", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcCreatureNavigation *this, TkID<128> *result, float *, bool *)
        ("GetRidingLean", ctypes.c_ulonglong),
        ("IsBeingRidden", ctypes.c_ulonglong),
        ("SetMinObstacleAabbSize", ctypes.c_ulonglong),
        ("ForceTurn", ctypes.c_ulonglong),
        ("AddBoost", ctypes.c_ulonglong),
        ("IsBoosting", ctypes.c_ulonglong),
        ("SetClampToGround", ctypes.c_ulonglong),
        ("DebugRecreateNav", ctypes.c_ulonglong),
        ("NotifyOfPredator", ctypes.c_ulonglong),
        ("DebugRender", ctypes.c_ulonglong),
        ("RenderNGuiVis", ctypes.c_ulonglong),
        ("RenderNGuiControl", ctypes.c_ulonglong),
    ]
    cGcCreatureNavigation_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    UpdatePostPhysics: int
    Release: int
    Destruct: int
    SetMode: int
    GetFrontNavMatrix: int
    GetTurnPivot: int
    InitialAdjustToKnowledge: int
    AdjustToKnowledge: int
    NavigateInRoughDirection: int
    NavigateToPoint: bytes
    IsPointAccessible: int
    HasPath: int
    CopyPathFrom: int
    SetMoveType: int
    SetStealthy: int
    RequestIdle: bytes
    GetIdleLength: bytes
    ForceSingleAnim: bytes
    HasForcedSingleAnim: int
    GetForcedSingleAnimHandle: int
    CancelForcedSingleAnim: int
    PlaySubAnim: bytes
    AsGroundNavigation: int
    AsSwarmNavigation: int
    GetTurnRadius: int
    GetMoveType: int
    SetWantsToLand: int
    GetLandingProgress: int
    GetStealthy: int
    GetAttackAnimCandidates: bytes
    ZeroSteering: int
    GetRunAnimSpeed: int
    CanPounce: int
    StartPounce: bytes
    UpdatePounce: int
    StartLocomotionDamage: int
    UpdateLocomotionDamage: int
    GetSteeringDirections: int
    PointIsInsideNavMap: int
    ShouldDespawn: int
    RequestRefreshNavMap: int
    SetMoveSpeeds: int
    SetSpeedFactor: int
    GetDesiredRidingAnim: bytes
    GetRidingLean: int
    IsBeingRidden: int
    SetMinObstacleAabbSize: int
    ForceTurn: int
    AddBoost: int
    IsBoosting: int
    SetClampToGround: int
    DebugRecreateNav: int
    NotifyOfPredator: int
    DebugRender: int
    RenderNGuiVis: int
    RenderNGuiControl: int

class cGcCreatureSwarmNavigation_vtbl(ctypes.Structure):
    _fields_ = [
        ("cGcCreatureNavigation_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("SetMode", ctypes.c_ulonglong),
        ("GetFrontNavMatrix", ctypes.c_ulonglong),
        ("GetTurnPivot", ctypes.c_ulonglong),
        ("InitialAdjustToKnowledge", ctypes.c_ulonglong),
        ("AdjustToKnowledge", ctypes.c_ulonglong),
        ("NavigateInRoughDirection", ctypes.c_ulonglong),
        ("NavigateToPoint", ctypes.c_char * 0x8),  # cTkStackVector<cGcNavPolyHandle,64,-1> *(__fastcall *)(cGcCreatureNavigation *this, const cTkPhysRelVec3 *, const cGcNavPolyHandle *)
        ("IsPointAccessible", ctypes.c_ulonglong),
        ("HasPath", ctypes.c_ulonglong),
        ("CopyPathFrom", ctypes.c_ulonglong),
        ("SetMoveType", ctypes.c_ulonglong),
        ("SetStealthy", ctypes.c_ulonglong),
        ("RequestIdle", ctypes.c_char * 0x8),  # cTkAnimLayerHandle *(__fastcall *)(cGcCreatureNavigation *this, cTkAnimLayerHandle *result, const TkID<128> *, float, cGcCreatureMoveSpeedRequest, bool, float)
        ("GetIdleLength", ctypes.c_char * 0x8),  # float (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *)
        ("ForceSingleAnim", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *, float, float, bool, float)
        ("HasForcedSingleAnim", ctypes.c_ulonglong),
        ("GetForcedSingleAnimHandle", ctypes.c_ulonglong),
        ("CancelForcedSingleAnim", ctypes.c_ulonglong),
        ("PlaySubAnim", ctypes.c_char * 0x8),  # void (__fastcall *)(cGcCreatureNavigation *this, const TkID<128> *, float, float, float, bool)
        ("AsGroundNavigation", ctypes.c_ulonglong),
        ("AsSwarmNavigation", ctypes.c_ulonglong),
        ("GetTurnRadius", ctypes.c_ulonglong),
        ("GetMoveType", ctypes.c_ulonglong),
        ("SetWantsToLand", ctypes.c_ulonglong),
        ("GetLandingProgress", ctypes.c_ulonglong),
        ("GetStealthy", ctypes.c_ulonglong),
        ("GetAttackAnimCandidates", ctypes.c_char * 0x8),  # unsigned int (__fastcall *)(cGcCreatureNavigation *this, TkID<128> *, unsigned int)
        ("ZeroSteering", ctypes.c_ulonglong),
        ("GetRunAnimSpeed", ctypes.c_ulonglong),
        ("CanPounce", ctypes.c_ulonglong),
        ("StartPounce", ctypes.c_char * 0x8),  # cTkAnimInstanceHandle *(__fastcall *)(cGcCreatureNavigation *this, cTkAnimInstanceHandle *result, const TkID<128> *, const cTkPhysRelVec3 *, bool, bool, bool)
        ("UpdatePounce", ctypes.c_ulonglong),
        ("StartLocomotionDamage", ctypes.c_ulonglong),
        ("UpdateLocomotionDamage", ctypes.c_ulonglong),
        ("GetSteeringDirections", ctypes.c_ulonglong),
        ("PointIsInsideNavMap", ctypes.c_ulonglong),
        ("ShouldDespawn", ctypes.c_ulonglong),
        ("RequestRefreshNavMap", ctypes.c_ulonglong),
        ("SetMoveSpeeds", ctypes.c_ulonglong),
        ("SetSpeedFactor", ctypes.c_ulonglong),
        ("GetDesiredRidingAnim", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcCreatureNavigation *this, TkID<128> *result, float *, bool *)
        ("GetRidingLean", ctypes.c_ulonglong),
        ("IsBeingRidden", ctypes.c_ulonglong),
        ("SetMinObstacleAabbSize", ctypes.c_ulonglong),
        ("ForceTurn", ctypes.c_ulonglong),
        ("AddBoost", ctypes.c_ulonglong),
        ("IsBoosting", ctypes.c_ulonglong),
        ("SetClampToGround", ctypes.c_ulonglong),
        ("DebugRecreateNav", ctypes.c_ulonglong),
        ("NotifyOfPredator", ctypes.c_ulonglong),
        ("DebugRender", ctypes.c_ulonglong),
        ("RenderNGuiVis", ctypes.c_ulonglong),
        ("RenderNGuiControl", ctypes.c_ulonglong),
    ]
    cGcCreatureNavigation_dtor_0: int
    Construct: int
    Prepare: int
    Update: int
    UpdatePostPhysics: int
    Release: int
    Destruct: int
    SetMode: int
    GetFrontNavMatrix: int
    GetTurnPivot: int
    InitialAdjustToKnowledge: int
    AdjustToKnowledge: int
    NavigateInRoughDirection: int
    NavigateToPoint: bytes
    IsPointAccessible: int
    HasPath: int
    CopyPathFrom: int
    SetMoveType: int
    SetStealthy: int
    RequestIdle: bytes
    GetIdleLength: bytes
    ForceSingleAnim: bytes
    HasForcedSingleAnim: int
    GetForcedSingleAnimHandle: int
    CancelForcedSingleAnim: int
    PlaySubAnim: bytes
    AsGroundNavigation: int
    AsSwarmNavigation: int
    GetTurnRadius: int
    GetMoveType: int
    SetWantsToLand: int
    GetLandingProgress: int
    GetStealthy: int
    GetAttackAnimCandidates: bytes
    ZeroSteering: int
    GetRunAnimSpeed: int
    CanPounce: int
    StartPounce: bytes
    UpdatePounce: int
    StartLocomotionDamage: int
    UpdateLocomotionDamage: int
    GetSteeringDirections: int
    PointIsInsideNavMap: int
    ShouldDespawn: int
    RequestRefreshNavMap: int
    SetMoveSpeeds: int
    SetSpeedFactor: int
    GetDesiredRidingAnim: bytes
    GetRidingLean: int
    IsBeingRidden: int
    SetMinObstacleAabbSize: int
    ForceTurn: int
    AddBoost: int
    IsBoosting: int
    SetClampToGround: int
    DebugRecreateNav: int
    NotifyOfPredator: int
    DebugRender: int
    RenderNGuiVis: int
    RenderNGuiControl: int

class cGcPetBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int

class cGcRegionTerrainVeryFar_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
        ("PostPolygoniseThread", ctypes.c_ulonglong),
        ("CancelPolygonise", ctypes.c_ulonglong),
        ("PreAsyncKnowledgeBuild", ctypes.c_ulonglong),
        ("CanPolygonise", ctypes.c_ulonglong),
        ("ThreadedGenerateVoxels", ctypes.c_ulonglong),
        ("ThreadedPolygonise", ctypes.c_ulonglong),
        ("ThreadedKnowledgeBuild", ctypes.c_ulonglong),
        ("GetFacingTriangleStart", ctypes.c_ulonglong),
        ("GetTileBlendTriangleStart", ctypes.c_ulonglong),
        ("DebugUpdate", ctypes.c_ulonglong),
        ("CreatureUpdate", ctypes.c_ulonglong),
        ("PollToMapStreams", ctypes.c_ulonglong),
        ("OnDistanceChanged", ctypes.c_ulonglong),
        ("SetHidden", ctypes.c_ulonglong),
        ("GetGroundPhysics", ctypes.c_ulonglong),
        ("FindInstanceWithRigidBodyNode", ctypes.c_ulonglong),
        ("RemoveInstanceWithRigidBodyNode", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcRegionTerrain *this, TkID<128> *result, cTkRigidBodyPtr)
        ("RemoveInstancesWithinArea", ctypes.c_ulonglong),
        ("SetInstanceActivating", ctypes.c_ulonglong),
        ("GetFoliage", ctypes.c_ulonglong),
        ("GetBuildings", ctypes.c_ulonglong),
        ("GetCreatures", ctypes.c_ulonglong),
        ("GetCreatures_158", ctypes.c_ulonglong),
        ("GetNodeName", ctypes.c_ulonglong),
        ("PartitionTriangles", ctypes.c_ulonglong),
        ("PartitionTrianglesByTile", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int
    PostPolygoniseThread: int
    CancelPolygonise: int
    PreAsyncKnowledgeBuild: int
    CanPolygonise: int
    ThreadedGenerateVoxels: int
    ThreadedPolygonise: int
    ThreadedKnowledgeBuild: int
    GetFacingTriangleStart: int
    GetTileBlendTriangleStart: int
    DebugUpdate: int
    CreatureUpdate: int
    PollToMapStreams: int
    OnDistanceChanged: int
    SetHidden: int
    GetGroundPhysics: int
    FindInstanceWithRigidBodyNode: int
    RemoveInstanceWithRigidBodyNode: bytes
    RemoveInstancesWithinArea: int
    SetInstanceActivating: int
    GetFoliage: int
    GetBuildings: int
    GetCreatures: int
    GetCreatures_158: int
    GetNodeName: int
    PartitionTriangles: int
    PartitionTrianglesByTile: int

class cGcWalkerBehaviourTree_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkBehaviourTree_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Construct_10", ctypes.c_ulonglong),
        ("Prepare", ctypes.c_ulonglong),
        ("Update", ctypes.c_ulonglong),
        ("Release", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("RenderNGui", ctypes.c_ulonglong),
        ("GetErrorString", ctypes.c_ulonglong),
        ("UpdatePostPhysics", ctypes.c_ulonglong),
        ("AsPetTree", ctypes.c_ulonglong),
        ("SetLookAtPoint", ctypes.c_ulonglong),
        ("AsWalkerTree", ctypes.c_ulonglong),
    ]
    cTkBehaviourTree_dtor_0: int
    Construct: int
    Construct_10: int
    Prepare: int
    Update: int
    Release: int
    Destruct: int
    RenderNGui: int
    GetErrorString: int
    UpdatePostPhysics: int
    AsPetTree: int
    SetLookAtPoint: int
    AsWalkerTree: int

class hkProcessClientContext_vtbl(ctypes.Structure):
    _fields_ = [
        ("hkProcessContextBase_dtor_0", ctypes.c_ulonglong),
        ("getType", ctypes.c_ulonglong),
    ]
    hkProcessContextBase_dtor_0: int
    getType: int

class hkSeekableStreamReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("skip", ctypes.c_ulonglong),
        ("peek", ctypes.c_ulonglong),
        ("isSeekTellSupported", ctypes.c_ulonglong),
        ("isInplace", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    read: int
    skip: int
    peek: int
    isSeekTellSupported: int
    isInplace: int
    seek: int
    tell: int

class hkStdioStreamReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("skip", ctypes.c_ulonglong),
        ("peek", ctypes.c_ulonglong),
        ("isSeekTellSupported", ctypes.c_ulonglong),
        ("isInplace", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    read: int
    skip: int
    peek: int
    isSeekTellSupported: int
    isInplace: int
    seek: int
    tell: int

class hknpDefaultViewerColorScheme_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpViewerColorScheme_dtor_0", ctypes.c_ulonglong),
        ("getBodyColor", ctypes.c_ulonglong),
        ("getParticleColor", ctypes.c_ulonglong),
        ("getCharacterProxyColor", ctypes.c_ulonglong),
        ("getQueryColor", ctypes.c_ulonglong),
        ("isBodyVisible", ctypes.c_ulonglong),
        ("isVolatile", ctypes.c_ulonglong),
        ("isPerParticleColorSupported", ctypes.c_ulonglong),
    ]
    hknpViewerColorScheme_dtor_0: int
    getBodyColor: int
    getParticleColor: int
    getCharacterProxyColor: int
    getQueryColor: int
    isBodyVisible: int
    isVolatile: int
    isPerParticleColorSupported: int

class hknpParticlesColliderEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class hknpViewerColorModifier_vtbl(ctypes.Structure):
    _fields_ = [
        ("hknpViewerColorScheme_dtor_0", ctypes.c_ulonglong),
        ("getBodyColor", ctypes.c_ulonglong),
        ("getParticleColor", ctypes.c_ulonglong),
        ("getCharacterProxyColor", ctypes.c_ulonglong),
        ("getQueryColor", ctypes.c_ulonglong),
        ("isBodyVisible", ctypes.c_ulonglong),
        ("isVolatile", ctypes.c_ulonglong),
        ("isPerParticleColorSupported", ctypes.c_ulonglong),
    ]
    hknpViewerColorScheme_dtor_0: int
    getBodyColor: int
    getParticleColor: int
    getCharacterProxyColor: int
    getQueryColor: int
    isBodyVisible: int
    isVolatile: int
    isPerParticleColorSupported: int

class hknpWorldEx_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int

class CAkActorMixer_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes

class CAkBusFX_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkGameObjectPluginInfo_dtor_0", ctypes.c_ulonglong),
        ("GetGameObjectID", ctypes.c_ulonglong),
        ("GetNumEmitterListenerPairs", ctypes.c_ulonglong),
        ("GetEmitterListenerPair", ctypes.c_ulonglong),
        ("GetNumGameObjectPositions", ctypes.c_ulonglong),
        ("GetGameObjectPosition", ctypes.c_ulonglong),
        ("GetGameObjectMultiPositionType", ctypes.c_ulonglong),
        ("GetGameObjectScaling", ctypes.c_ulonglong),
        ("GetListeners", ctypes.c_ulonglong),
        ("GetListenerData", ctypes.c_ulonglong),
        ("_GetVoiceInfo", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("GetOutputConfig", ctypes.c_ulonglong),
        ("GetMaxBufferLength", ctypes.c_ulonglong),
    ]
    IAkGameObjectPluginInfo_dtor_0: int
    GetGameObjectID: int
    GetNumEmitterListenerPairs: int
    GetEmitterListenerPair: int
    GetNumGameObjectPositions: int
    GetGameObjectPosition: int
    GetGameObjectMultiPositionType: int
    GetGameObjectScaling: int
    GetListeners: int
    GetListenerData: int
    _GetVoiceInfo: int
    GetAnalyzedEnvelope: int
    GetOutputConfig: int
    GetMaxBufferLength: int

class CAkBus_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNodeBase> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNodeBase> *this, CAkParameterNodeBase *)
        ("SetInitialValues", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    Children: bytes
    CanAddChild: bytes
    SetInitialValues: int

class CAkContainerBase_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes

class CAkMultiPlayNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
        ("IsContinuousPlayback", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes
    IsContinuousPlayback: int

class CAkMusicNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
        ("CreateContext", ctypes.c_ulonglong),
        ("GetMusicParams", ctypes.c_ulonglong),
        ("PrepareMusicalDependencies", ctypes.c_ulonglong),
        ("UnPrepareMusicalDependencies", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes
    CreateContext: int
    GetMusicParams: int
    PrepareMusicalDependencies: int
    UnPrepareMusicalDependencies: int

class CAkMusicSegment_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
        ("CreateContext", ctypes.c_ulonglong),
        ("GetMusicParams", ctypes.c_ulonglong),
        ("PrepareMusicalDependencies", ctypes.c_ulonglong),
        ("UnPrepareMusicalDependencies", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes
    CreateContext: int
    GetMusicParams: int
    PrepareMusicalDependencies: int
    UnPrepareMusicalDependencies: int

class CAkMusicTrack_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int

class CAkMusicTransAware_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
        ("CreateContext", ctypes.c_ulonglong),
        ("GetMusicParams", ctypes.c_ulonglong),
        ("PrepareMusicalDependencies", ctypes.c_ulonglong),
        ("UnPrepareMusicalDependencies", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes
    CreateContext: int
    GetMusicParams: int
    PrepareMusicalDependencies: int
    UnPrepareMusicalDependencies: int

class CAkRanSeqCntr_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes

class CAkSwitchCntr_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
        ("IsContinuousPlayback", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes
    IsContinuousPlayback: int

class CAkVPLMixBusNode_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkGameObjectPluginInfo_dtor_0", ctypes.c_ulonglong),
        ("GetGameObjectID", ctypes.c_ulonglong),
        ("GetNumEmitterListenerPairs", ctypes.c_ulonglong),
        ("GetEmitterListenerPair", ctypes.c_ulonglong),
        ("GetNumGameObjectPositions", ctypes.c_ulonglong),
        ("GetGameObjectPosition", ctypes.c_ulonglong),
        ("GetGameObjectMultiPositionType", ctypes.c_ulonglong),
        ("GetGameObjectScaling", ctypes.c_ulonglong),
        ("GetListeners", ctypes.c_ulonglong),
        ("GetListenerData", ctypes.c_ulonglong),
        ("_GetVoiceInfo", ctypes.c_ulonglong),
        ("GetAnalyzedEnvelope", ctypes.c_ulonglong),
        ("GetOutputConfig", ctypes.c_ulonglong),
        ("GetMaxBufferLength", ctypes.c_ulonglong),
    ]
    IAkGameObjectPluginInfo_dtor_0: int
    GetGameObjectID: int
    GetNumEmitterListenerPairs: int
    GetEmitterListenerPair: int
    GetNumGameObjectPositions: int
    GetGameObjectPosition: int
    GetGameObjectMultiPositionType: int
    GetGameObjectScaling: int
    GetListeners: int
    GetListenerData: int
    _GetVoiceInfo: int
    GetAnalyzedEnvelope: int
    GetOutputConfig: int
    GetMaxBufferLength: int

class cGcRegionTerrainFar_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
        ("PostPolygoniseThread", ctypes.c_ulonglong),
        ("CancelPolygonise", ctypes.c_ulonglong),
        ("PreAsyncKnowledgeBuild", ctypes.c_ulonglong),
        ("CanPolygonise", ctypes.c_ulonglong),
        ("ThreadedGenerateVoxels", ctypes.c_ulonglong),
        ("ThreadedPolygonise", ctypes.c_ulonglong),
        ("ThreadedKnowledgeBuild", ctypes.c_ulonglong),
        ("GetFacingTriangleStart", ctypes.c_ulonglong),
        ("GetTileBlendTriangleStart", ctypes.c_ulonglong),
        ("DebugUpdate", ctypes.c_ulonglong),
        ("CreatureUpdate", ctypes.c_ulonglong),
        ("PollToMapStreams", ctypes.c_ulonglong),
        ("OnDistanceChanged", ctypes.c_ulonglong),
        ("SetHidden", ctypes.c_ulonglong),
        ("GetGroundPhysics", ctypes.c_ulonglong),
        ("FindInstanceWithRigidBodyNode", ctypes.c_ulonglong),
        ("RemoveInstanceWithRigidBodyNode", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcRegionTerrain *this, TkID<128> *result, cTkRigidBodyPtr)
        ("RemoveInstancesWithinArea", ctypes.c_ulonglong),
        ("SetInstanceActivating", ctypes.c_ulonglong),
        ("GetFoliage", ctypes.c_ulonglong),
        ("GetBuildings", ctypes.c_ulonglong),
        ("GetCreatures", ctypes.c_ulonglong),
        ("GetCreatures_158", ctypes.c_ulonglong),
        ("GetNodeName", ctypes.c_ulonglong),
        ("PartitionTriangles", ctypes.c_ulonglong),
        ("PartitionTrianglesByTile", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int
    PostPolygoniseThread: int
    CancelPolygonise: int
    PreAsyncKnowledgeBuild: int
    CanPolygonise: int
    ThreadedGenerateVoxels: int
    ThreadedPolygonise: int
    ThreadedKnowledgeBuild: int
    GetFacingTriangleStart: int
    GetTileBlendTriangleStart: int
    DebugUpdate: int
    CreatureUpdate: int
    PollToMapStreams: int
    OnDistanceChanged: int
    SetHidden: int
    GetGroundPhysics: int
    FindInstanceWithRigidBodyNode: int
    RemoveInstanceWithRigidBodyNode: bytes
    RemoveInstancesWithinArea: int
    SetInstanceActivating: int
    GetFoliage: int
    GetBuildings: int
    GetCreatures: int
    GetCreatures_158: int
    GetNodeName: int
    PartitionTriangles: int
    PartitionTrianglesByTile: int

class cGcRegionTerrainMid_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
        ("PostPolygoniseThread", ctypes.c_ulonglong),
        ("CancelPolygonise", ctypes.c_ulonglong),
        ("PreAsyncKnowledgeBuild", ctypes.c_ulonglong),
        ("CanPolygonise", ctypes.c_ulonglong),
        ("ThreadedGenerateVoxels", ctypes.c_ulonglong),
        ("ThreadedPolygonise", ctypes.c_ulonglong),
        ("ThreadedKnowledgeBuild", ctypes.c_ulonglong),
        ("GetFacingTriangleStart", ctypes.c_ulonglong),
        ("GetTileBlendTriangleStart", ctypes.c_ulonglong),
        ("DebugUpdate", ctypes.c_ulonglong),
        ("CreatureUpdate", ctypes.c_ulonglong),
        ("PollToMapStreams", ctypes.c_ulonglong),
        ("OnDistanceChanged", ctypes.c_ulonglong),
        ("SetHidden", ctypes.c_ulonglong),
        ("GetGroundPhysics", ctypes.c_ulonglong),
        ("FindInstanceWithRigidBodyNode", ctypes.c_ulonglong),
        ("RemoveInstanceWithRigidBodyNode", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcRegionTerrain *this, TkID<128> *result, cTkRigidBodyPtr)
        ("RemoveInstancesWithinArea", ctypes.c_ulonglong),
        ("SetInstanceActivating", ctypes.c_ulonglong),
        ("GetFoliage", ctypes.c_ulonglong),
        ("GetBuildings", ctypes.c_ulonglong),
        ("GetCreatures", ctypes.c_ulonglong),
        ("GetCreatures_158", ctypes.c_ulonglong),
        ("GetNodeName", ctypes.c_ulonglong),
        ("PartitionTriangles", ctypes.c_ulonglong),
        ("PartitionTrianglesByTile", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int
    PostPolygoniseThread: int
    CancelPolygonise: int
    PreAsyncKnowledgeBuild: int
    CanPolygonise: int
    ThreadedGenerateVoxels: int
    ThreadedPolygonise: int
    ThreadedKnowledgeBuild: int
    GetFacingTriangleStart: int
    GetTileBlendTriangleStart: int
    DebugUpdate: int
    CreatureUpdate: int
    PollToMapStreams: int
    OnDistanceChanged: int
    SetHidden: int
    GetGroundPhysics: int
    FindInstanceWithRigidBodyNode: int
    RemoveInstanceWithRigidBodyNode: bytes
    RemoveInstancesWithinArea: int
    SetInstanceActivating: int
    GetFoliage: int
    GetBuildings: int
    GetCreatures: int
    GetCreatures_158: int
    GetNodeName: int
    PartitionTriangles: int
    PartitionTrianglesByTile: int

class cGcRegionTerrainNear_vtbl(ctypes.Structure):
    _fields_ = [
        ("cTkRegion_dtor_0", ctypes.c_ulonglong),
        ("Construct", ctypes.c_ulonglong),
        ("Destruct", ctypes.c_ulonglong),
        ("UpdateMatrix", ctypes.c_ulonglong),
        ("Assign", ctypes.c_ulonglong),
        ("PostPolygonise", ctypes.c_ulonglong),
        ("PostPolygonisePopulate", ctypes.c_ulonglong),
        ("KnowledgeBuilt", ctypes.c_ulonglong),
        ("PollToUnmapStreams", ctypes.c_ulonglong),
        ("TryToClear", ctypes.c_ulonglong),
        ("Invalidate", ctypes.c_ulonglong),
        ("Clear", ctypes.c_ulonglong),
        ("RefreshKnowledge", ctypes.c_ulonglong),
        ("RefreshFoliage", ctypes.c_ulonglong),
        ("GetResource", ctypes.c_ulonglong),
        ("GetStatusColour", ctypes.c_ulonglong),
        ("GenerateVoxels", ctypes.c_ulonglong),
        ("Polygonise", ctypes.c_ulonglong),
        ("BuildKnowledge", ctypes.c_ulonglong),
        ("InvalidateKnowledge", ctypes.c_ulonglong),
        ("AreResourcesLoaded", ctypes.c_ulonglong),
        ("PostPolygoniseThread", ctypes.c_ulonglong),
        ("CancelPolygonise", ctypes.c_ulonglong),
        ("PreAsyncKnowledgeBuild", ctypes.c_ulonglong),
        ("CanPolygonise", ctypes.c_ulonglong),
        ("ThreadedGenerateVoxels", ctypes.c_ulonglong),
        ("ThreadedPolygonise", ctypes.c_ulonglong),
        ("ThreadedKnowledgeBuild", ctypes.c_ulonglong),
        ("GetFacingTriangleStart", ctypes.c_ulonglong),
        ("GetTileBlendTriangleStart", ctypes.c_ulonglong),
        ("DebugUpdate", ctypes.c_ulonglong),
        ("CreatureUpdate", ctypes.c_ulonglong),
        ("PollToMapStreams", ctypes.c_ulonglong),
        ("OnDistanceChanged", ctypes.c_ulonglong),
        ("SetHidden", ctypes.c_ulonglong),
        ("GetGroundPhysics", ctypes.c_ulonglong),
        ("FindInstanceWithRigidBodyNode", ctypes.c_ulonglong),
        ("RemoveInstanceWithRigidBodyNode", ctypes.c_char * 0x8),  # TkID<128> *(__fastcall *)(cGcRegionTerrain *this, TkID<128> *result, cTkRigidBodyPtr)
        ("RemoveInstancesWithinArea", ctypes.c_ulonglong),
        ("SetInstanceActivating", ctypes.c_ulonglong),
        ("GetFoliage", ctypes.c_ulonglong),
        ("GetBuildings", ctypes.c_ulonglong),
        ("GetCreatures", ctypes.c_ulonglong),
        ("GetCreatures_158", ctypes.c_ulonglong),
        ("GetNodeName", ctypes.c_ulonglong),
        ("PartitionTriangles", ctypes.c_ulonglong),
        ("PartitionTrianglesByTile", ctypes.c_ulonglong),
        ("GetQuadResource", ctypes.c_ulonglong),
        ("GetEditsLineResource", ctypes.c_ulonglong),
        ("GetLineResource", ctypes.c_ulonglong),
    ]
    cTkRegion_dtor_0: int
    Construct: int
    Destruct: int
    UpdateMatrix: int
    Assign: int
    PostPolygonise: int
    PostPolygonisePopulate: int
    KnowledgeBuilt: int
    PollToUnmapStreams: int
    TryToClear: int
    Invalidate: int
    Clear: int
    RefreshKnowledge: int
    RefreshFoliage: int
    GetResource: int
    GetStatusColour: int
    GenerateVoxels: int
    Polygonise: int
    BuildKnowledge: int
    InvalidateKnowledge: int
    AreResourcesLoaded: int
    PostPolygoniseThread: int
    CancelPolygonise: int
    PreAsyncKnowledgeBuild: int
    CanPolygonise: int
    ThreadedGenerateVoxels: int
    ThreadedPolygonise: int
    ThreadedKnowledgeBuild: int
    GetFacingTriangleStart: int
    GetTileBlendTriangleStart: int
    DebugUpdate: int
    CreatureUpdate: int
    PollToMapStreams: int
    OnDistanceChanged: int
    SetHidden: int
    GetGroundPhysics: int
    FindInstanceWithRigidBodyNode: int
    RemoveInstanceWithRigidBodyNode: bytes
    RemoveInstancesWithinArea: int
    SetInstanceActivating: int
    GetFoliage: int
    GetBuildings: int
    GetCreatures: int
    GetCreatures_158: int
    GetNodeName: int
    PartitionTriangles: int
    PartitionTrianglesByTile: int
    GetQuadResource: int
    GetEditsLineResource: int
    GetLineResource: int

class hkBufferedStreamReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("skip", ctypes.c_ulonglong),
        ("peek", ctypes.c_ulonglong),
        ("isSeekTellSupported", ctypes.c_ulonglong),
        ("isInplace", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    read: int
    skip: int
    peek: int
    isSeekTellSupported: int
    isInplace: int
    seek: int
    tell: int

class hkInplaceStreamReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("skip", ctypes.c_ulonglong),
        ("peek", ctypes.c_ulonglong),
        ("isSeekTellSupported", ctypes.c_ulonglong),
        ("isInplace", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
        ("available", ctypes.c_ulonglong),
        ("peekInto", ctypes.c_char * 0x8),  # hkArrayView<void> *(__fastcall *)(hkInplaceStreamReader *this, hkArrayView<void> *result, unsigned __int64)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    read: int
    skip: int
    peek: int
    isSeekTellSupported: int
    isInplace: int
    seek: int
    tell: int
    available: int
    peekInto: bytes

class hkMemoryStreamReader_vtbl(ctypes.Structure):
    _fields_ = [
        ("getExactType", ctypes.c_ulonglong),
        ("hkBaseObject_dtor_8", ctypes.c_ulonglong),
        ("deleteThisReferencedObject", ctypes.c_ulonglong),
        ("isOk", ctypes.c_ulonglong),
        ("read", ctypes.c_ulonglong),
        ("skip", ctypes.c_ulonglong),
        ("peek", ctypes.c_ulonglong),
        ("isSeekTellSupported", ctypes.c_ulonglong),
        ("isInplace", ctypes.c_ulonglong),
        ("seek", ctypes.c_ulonglong),
        ("tell", ctypes.c_ulonglong),
        ("available", ctypes.c_ulonglong),
        ("peekInto", ctypes.c_char * 0x8),  # hkArrayView<void> *(__fastcall *)(hkInplaceStreamReader *this, hkArrayView<void> *result, unsigned __int64)
    ]
    getExactType: int
    hkBaseObject_dtor_8: int
    deleteThisReferencedObject: int
    isOk: int
    read: int
    skip: int
    peek: int
    isSeekTellSupported: int
    isInplace: int
    seek: int
    tell: int
    available: int
    peekInto: bytes

class CAkAuxBus_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNodeBase> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNodeBase> *this, CAkParameterNodeBase *)
        ("SetInitialValues", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    Children: bytes
    CanAddChild: bytes
    SetInitialValues: int

class CAkLayerCntr_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
        ("IsContinuousPlayback", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes
    IsContinuousPlayback: int

class CAkMusicRanSeqCntr_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
        ("CreateContext", ctypes.c_ulonglong),
        ("GetMusicParams", ctypes.c_ulonglong),
        ("PrepareMusicalDependencies", ctypes.c_ulonglong),
        ("UnPrepareMusicalDependencies", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes
    CreateContext: int
    GetMusicParams: int
    PrepareMusicalDependencies: int
    UnPrepareMusicalDependencies: int

class CAkMusicSwitchCntr_vtbl(ctypes.Structure):
    _fields_ = [
        ("IAkEffectSlotsOwner_dtor_0", ctypes.c_ulonglong),
        ("UpdateFX", ctypes.c_ulonglong),
        ("BeforeEffectsUpdate", ctypes.c_ulonglong),
        ("OnPreRelease", ctypes.c_ulonglong),
        ("Parent", ctypes.c_ulonglong),
        ("ParentBus", ctypes.c_ulonglong),
        ("Unregister", ctypes.c_ulonglong),
        ("AddChildInternal", ctypes.c_ulonglong),
        ("AddChild", ctypes.c_ulonglong),
        ("RemoveChild", ctypes.c_ulonglong),
        ("RemoveChild_50", ctypes.c_ulonglong),
        ("RemoveOutputBus", ctypes.c_ulonglong),
        ("GetChildren", ctypes.c_ulonglong),
        ("GatherSounds", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkSrcTypeInfo *,AkSrcTypeInfo *,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkSrcTypeInfo *> > *, AkArray<AkGameSync,AkGameSync,AkArrayAllocatorNoAlign<0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkGameSync> > *, bool, CAkRegisteredObj *, unsigned int, unsigned int)
        ("NodeCategory", ctypes.c_ulonglong),
        ("IsPlayable", ctypes.c_ulonglong),
        ("ExecuteAction", ctypes.c_ulonglong),
        ("ExecuteActionNoPropagate", ctypes.c_ulonglong),
        ("ExecuteActionExcept", ctypes.c_ulonglong),
        ("ExecuteActionExceptParentCheck", ctypes.c_ulonglong),
        ("PlayToEnd", ctypes.c_ulonglong),
        ("ParamNotification", ctypes.c_ulonglong),
        ("Notification", ctypes.c_ulonglong),
        ("MuteNotification", ctypes.c_ulonglong),
        ("MuteNotification_c0", ctypes.c_ulonglong),
        ("PropagatePositioningNotification", ctypes.c_ulonglong),
        ("MusicNotification", ctypes.c_ulonglong),
        ("ClearLimiters", ctypes.c_ulonglong),
        ("ForAllPBI", ctypes.c_ulonglong),
        ("NotifyBypass", ctypes.c_ulonglong),
        ("GetMaxRadius", ctypes.c_ulonglong),
        ("GetMixingBus", ctypes.c_ulonglong),
        ("IncrementPlayCount", ctypes.c_ulonglong),
        ("DecrementPlayCount", ctypes.c_ulonglong),
        ("IncrementActivityCount", ctypes.c_ulonglong),
        ("DecrementActivityCount", ctypes.c_ulonglong),
        ("PrepareData", ctypes.c_ulonglong),
        ("UnPrepareData", ctypes.c_ulonglong),
        ("TriggerModulators", ctypes.c_ulonglong),
        ("GetAudioParameters", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, AkSoundParams *, AkMutedMap *, const AkRTPCKey *, AkPBIModValues *, AkArray<AkModulatorToTrigger,AkModulatorToTrigger const &,AkHybridAllocator<256,1,0>,AkGrowByPolicy_Proportional,AkAssignmentMovePolicy<AkModulatorToTrigger> > *, bool, CAkParameterNodeBase *)
        ("SetAkProp", ctypes.c_ulonglong),
        ("SetAkProp_148", ctypes.c_ulonglong),
        ("SetAkProp_150", ctypes.c_ulonglong),
        ("ResetAkProp", ctypes.c_ulonglong),
        ("Mute", ctypes.c_ulonglong),
        ("Unmute", ctypes.c_ulonglong),
        ("UnmuteAll", ctypes.c_ulonglong),
        ("RecalcNotification", ctypes.c_ulonglong),
        ("RecalcNotificationWithID", ctypes.c_ulonglong),
        ("RecalcNotificationWithBitArray", ctypes.c_char * 0x8),  # void (__fastcall *)(CAkParameterNodeBase *this, bool, const CAkBitArray<unsigned __int64,0> *)
        ("ApplyMaxNumInstances", ctypes.c_ulonglong),
        ("FindCustomProp", ctypes.c_ulonglong),
        ("SetMixerPlugin", ctypes.c_ulonglong),
        ("ResetFXBypass", ctypes.c_ulonglong),
        ("GetFX", ctypes.c_ulonglong),
        ("GetFXDataID", ctypes.c_ulonglong),
        ("SetRTPC", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParameterNodeBase *this, unsigned int, AkRtpcType, AkRtpcAccum, AkRTPC_ParameterID, unsigned int, AkCurveScaling, AkGraphPointBase<float> *, unsigned int, bool)
        ("UnsetRTPC", ctypes.c_ulonglong),
        ("ParamOverriden", ctypes.c_ulonglong),
        ("ParamMustNotify", ctypes.c_ulonglong),
        ("PositioningChangeNotification", ctypes.c_ulonglong),
        ("SetInitialParams", ctypes.c_ulonglong),
        ("SetInitialFxParams", ctypes.c_ulonglong),
        ("SetInitialMetadataParams", ctypes.c_ulonglong),
        ("SetAuxParams", ctypes.c_ulonglong),
        ("SetAdvSettingsParams", ctypes.c_ulonglong),
        ("GetSIS", ctypes.c_ulonglong),
        ("IsInfiniteLooping", ctypes.c_ulonglong),
        ("AdjustCount", ctypes.c_ulonglong),
        ("PlayInternal", ctypes.c_ulonglong),
        ("UnmuteAllObj", ctypes.c_ulonglong),
        ("SetBelowThresholdBehavior", ctypes.c_ulonglong),
        ("Children", ctypes.c_char * 0x8),  # unsigned __int16 (__fastcall *)(CAkParentNode<CAkParameterNode> *this)
        ("CanAddChild", ctypes.c_char * 0x8),  # AKRESULT (__fastcall *)(CAkParentNode<CAkParameterNode> *this, CAkParameterNodeBase *)
        ("CreateContext", ctypes.c_ulonglong),
        ("GetMusicParams", ctypes.c_ulonglong),
        ("PrepareMusicalDependencies", ctypes.c_ulonglong),
        ("UnPrepareMusicalDependencies", ctypes.c_ulonglong),
    ]
    IAkEffectSlotsOwner_dtor_0: int
    UpdateFX: int
    BeforeEffectsUpdate: int
    OnPreRelease: int
    Parent: int
    ParentBus: int
    Unregister: int
    AddChildInternal: int
    AddChild: int
    RemoveChild: int
    RemoveChild_50: int
    RemoveOutputBus: int
    GetChildren: int
    GatherSounds: bytes
    NodeCategory: int
    IsPlayable: int
    ExecuteAction: int
    ExecuteActionNoPropagate: int
    ExecuteActionExcept: int
    ExecuteActionExceptParentCheck: int
    PlayToEnd: int
    ParamNotification: int
    Notification: int
    MuteNotification: int
    MuteNotification_c0: int
    PropagatePositioningNotification: int
    MusicNotification: int
    ClearLimiters: int
    ForAllPBI: int
    NotifyBypass: int
    GetMaxRadius: int
    GetMixingBus: int
    IncrementPlayCount: int
    DecrementPlayCount: int
    IncrementActivityCount: int
    DecrementActivityCount: int
    PrepareData: int
    UnPrepareData: int
    TriggerModulators: int
    GetAudioParameters: bytes
    SetAkProp: int
    SetAkProp_148: int
    SetAkProp_150: int
    ResetAkProp: int
    Mute: int
    Unmute: int
    UnmuteAll: int
    RecalcNotification: int
    RecalcNotificationWithID: int
    RecalcNotificationWithBitArray: bytes
    ApplyMaxNumInstances: int
    FindCustomProp: int
    SetMixerPlugin: int
    ResetFXBypass: int
    GetFX: int
    GetFXDataID: int
    SetRTPC: bytes
    UnsetRTPC: int
    ParamOverriden: int
    ParamMustNotify: int
    PositioningChangeNotification: int
    SetInitialParams: int
    SetInitialFxParams: int
    SetInitialMetadataParams: int
    SetAuxParams: int
    SetAdvSettingsParams: int
    GetSIS: int
    IsInfiniteLooping: int
    AdjustCount: int
    PlayInternal: int
    UnmuteAllObj: int
    SetBelowThresholdBehavior: int
    Children: bytes
    CanAddChild: bytes
    CreateContext: int
    GetMusicParams: int
    PrepareMusicalDependencies: int
    UnPrepareMusicalDependencies: int

class RUNTIME_FUNCTION(ctypes.Structure):
    _fields_ = [
        ("FunctionStart", ctypes.c_char * 0x4),
        ("FunctionEnd", ctypes.c_char * 0x4),
        ("UnwindInfo", ctypes.c_char * 0x4),
    ]
    FunctionStart: bytes
    FunctionEnd: bytes
    UnwindInfo: bytes

class UNWIND_INFO_HDR(ctypes.Structure):
    _fields_ = [
        ("Ver3_Flags", ctypes.c_char * 0x1),
        ("PrologSize", ctypes.c_char * 0x1),
        ("CntUnwindCodes", ctypes.c_char * 0x1),
        ("FrReg_FrRegOff", ctypes.c_char * 0x1),
    ]
    Ver3_Flags: bytes
    PrologSize: bytes
    CntUnwindCodes: bytes
    FrReg_FrRegOff: bytes

class UNWIND_CODE(ctypes.Structure):
    _fields_ = [
        ("PrologOff", ctypes.c_char * 0x1),
        ("OpCode_OpInfo", ctypes.c_char * 0x1),
    ]
    PrologOff: bytes
    OpCode_OpInfo: bytes

class FuncInfo(ctypes.Structure):
    _fields_ = [
        ("magicNumber", ctypes.c_char * 0x4),
        ("maxState", ctypes.c_char * 0x4),
        ("pUnwindMap", ctypes.c_char * 0x4),
        ("nTryBlocks", ctypes.c_char * 0x4),
        ("pTryBlockMap", ctypes.c_char * 0x4),
        ("nIPMapEntries", ctypes.c_char * 0x4),
        ("pIPtoStateMap", ctypes.c_char * 0x4),
        ("dispUnwindHelp", ctypes.c_char * 0x4),
        ("pESTypeList", ctypes.c_char * 0x4),
        ("EHFlags", ctypes.c_char * 0x4),
    ]
    magicNumber: bytes
    maxState: bytes
    pUnwindMap: bytes
    nTryBlocks: bytes
    pTryBlockMap: bytes
    nIPMapEntries: bytes
    pIPtoStateMap: bytes
    dispUnwindHelp: bytes
    pESTypeList: bytes
    EHFlags: bytes

class UnwindMapEntry(ctypes.Structure):
    _fields_ = [
        ("toState", ctypes.c_char * 0x4),
        ("action", ctypes.c_char * 0x4),
    ]
    toState: bytes
    action: bytes

class TryBlockMapEntry(ctypes.Structure):
    _fields_ = [
        ("tryLow", ctypes.c_char * 0x4),
        ("tryHigh", ctypes.c_char * 0x4),
        ("catchHigh", ctypes.c_char * 0x4),
        ("nCatches", ctypes.c_char * 0x4),
        ("pHandlerArray", ctypes.c_char * 0x4),
    ]
    tryLow: bytes
    tryHigh: bytes
    catchHigh: bytes
    nCatches: bytes
    pHandlerArray: bytes

class HandlerType(ctypes.Structure):
    _fields_ = [
        ("adjectives", ctypes.c_char * 0x4),
        ("pType", ctypes.c_char * 0x4),
        ("dispCatchObj", ctypes.c_char * 0x4),
        ("addressOfHandler", ctypes.c_char * 0x4),
        ("dispFrame", ctypes.c_char * 0x4),
    ]
    adjectives: bytes
    pType: bytes
    dispCatchObj: bytes
    addressOfHandler: bytes
    dispFrame: bytes

class IPtoStateMap(ctypes.Structure):
    _fields_ = [
        ("pc", ctypes.c_char * 0x4),
        ("state", ctypes.c_char * 0x4),
    ]
    pc: bytes
    state: bytes

class C_SCOPE_TABLE(ctypes.Structure):
    _fields_ = [
        ("Begin", ctypes.c_char * 0x4),
        ("End", ctypes.c_char * 0x4),
        ("Handler", ctypes.c_char * 0x4),
        ("Target", ctypes.c_char * 0x4),
    ]
    Begin: bytes
    End: bytes
    Handler: bytes
    Target: bytes

class __TI_flags(IntEnum):
    TI_IsConst = 0x1

class __CT_flags(IntEnum):
    CT_IsSimpleType = 0x1

class DEVMODEW(ctypes.Structure):
    _fields_ = [
        ("dmDeviceName", ctypes.c_wchar * 0x20),  # wchar_t[32]
        ("dmSpecVersion", ctypes.c_uint16),
        ("dmDriverVersion", ctypes.c_uint16),
        ("dmSize", ctypes.c_uint16),
        ("dmDriverExtra", ctypes.c_uint16),
        ("dmFields", ctypes.c_uint32),
        ("___u6", ctypes.c_char * 0x8),  # $BB358C3A325381C626105C9C85EB3F4A
        ("___u7", ctypes.c_char * 0x4),  # $DEE03DF9F4D6FE316E4B77CDE5AE0A90
        ("___u8", ctypes.c_char * 0x4),  # $6B826ED569E6FCAD26E63896A7C9594E
        ("dmColor", ctypes.c_int16),
        ("dmDuplex", ctypes.c_int16),
        ("dmYResolution", ctypes.c_int16),
        ("dmTTOption", ctypes.c_int16),
        ("dmCollate", ctypes.c_int16),
        ("dmFormName", ctypes.c_wchar * 0x20),  # wchar_t[32]
        ("dmLogPixels", ctypes.c_uint16),
        ("dmBitsPerPel", ctypes.c_uint32),
        ("dmPelsWidth", ctypes.c_uint32),
        ("dmPelsHeight", ctypes.c_uint32),
        ("___u19", ctypes.c_char * 0x4),  # $309666D137CBB19809FA597AD348B415
        ("dmDisplayFrequency", ctypes.c_uint32),
        ("dmICMMethod", ctypes.c_uint32),
        ("dmICMIntent", ctypes.c_uint32),
        ("dmMediaType", ctypes.c_uint32),
        ("dmDitherType", ctypes.c_uint32),
        ("dmReserved1", ctypes.c_uint32),
        ("dmReserved2", ctypes.c_uint32),
        ("dmPanningWidth", ctypes.c_uint32),
        ("dmPanningHeight", ctypes.c_uint32),
    ]
    dmDeviceName: list[ctypes.c_wchar]
    dmSpecVersion: int
    dmDriverVersion: int
    dmSize: int
    dmDriverExtra: int
    dmFields: int
    ___u6: bytes
    ___u7: bytes
    ___u8: bytes
    dmColor: int
    dmDuplex: int
    dmYResolution: int
    dmTTOption: int
    dmCollate: int
    dmFormName: list[ctypes.c_wchar]
    dmLogPixels: int
    dmBitsPerPel: int
    dmPelsWidth: int
    dmPelsHeight: int
    ___u19: bytes
    dmDisplayFrequency: int
    dmICMMethod: int
    dmICMIntent: int
    dmMediaType: int
    dmDitherType: int
    dmReserved1: int
    dmReserved2: int
    dmPanningWidth: int
    dmPanningHeight: int

class POINT(ctypes.Structure):
    pass

class LARGE_INTEGER: pass

class WNDCLASSEXW(ctypes.Structure):
    _fields_ = [
        ("cbSize", ctypes.c_uint32),
        ("style", ctypes.c_uint32),
        ("lpfnWndProc", ctypes.c_ulonglong),
        ("cbClsExtra", ctypes.c_int32),
        ("cbWndExtra", ctypes.c_int32),
        ("hInstance", ctypes.c_ulonglong),
        ("hIcon", ctypes.c_ulonglong),
        ("hCursor", ctypes.c_ulonglong),
        ("hbrBackground", ctypes.c_ulonglong),
        ("lpszMenuName", ctypes.c_ulonglong),
        ("lpszClassName", ctypes.c_ulonglong),
        ("hIconSm", ctypes.c_ulonglong),
    ]
    cbSize: int
    style: int
    lpfnWndProc: int
    cbClsExtra: int
    cbWndExtra: int
    hInstance: int
    hIcon: int
    hCursor: int
    hbrBackground: int
    lpszMenuName: int
    lpszClassName: int
    hIconSm: int

class PIXELFORMATDESCRIPTOR(ctypes.Structure):
    _fields_ = [
        ("nSize", ctypes.c_uint16),
        ("nVersion", ctypes.c_uint16),
        ("dwFlags", ctypes.c_uint32),
        ("iPixelType", ctypes.c_uint8),
        ("cColorBits", ctypes.c_uint8),
        ("cRedBits", ctypes.c_uint8),
        ("cRedShift", ctypes.c_uint8),
        ("cGreenBits", ctypes.c_uint8),
        ("cGreenShift", ctypes.c_uint8),
        ("cBlueBits", ctypes.c_uint8),
        ("cBlueShift", ctypes.c_uint8),
        ("cAlphaBits", ctypes.c_uint8),
        ("cAlphaShift", ctypes.c_uint8),
        ("cAccumBits", ctypes.c_uint8),
        ("cAccumRedBits", ctypes.c_uint8),
        ("cAccumGreenBits", ctypes.c_uint8),
        ("cAccumBlueBits", ctypes.c_uint8),
        ("cAccumAlphaBits", ctypes.c_uint8),
        ("cDepthBits", ctypes.c_uint8),
        ("cStencilBits", ctypes.c_uint8),
        ("cAuxBuffers", ctypes.c_uint8),
        ("iLayerType", ctypes.c_uint8),
        ("bReserved", ctypes.c_uint8),
        ("dwLayerMask", ctypes.c_uint32),
        ("dwVisibleMask", ctypes.c_uint32),
        ("dwDamageMask", ctypes.c_uint32),
    ]
    nSize: int
    nVersion: int
    dwFlags: int
    iPixelType: int
    cColorBits: int
    cRedBits: int
    cRedShift: int
    cGreenBits: int
    cGreenShift: int
    cBlueBits: int
    cBlueShift: int
    cAlphaBits: int
    cAlphaShift: int
    cAccumBits: int
    cAccumRedBits: int
    cAccumGreenBits: int
    cAccumBlueBits: int
    cAccumAlphaBits: int
    cDepthBits: int
    cStencilBits: int
    cAuxBuffers: int
    iLayerType: int
    bReserved: int
    dwLayerMask: int
    dwVisibleMask: int
    dwDamageMask: int

class ADDRINFOA(ctypes.Structure):
    _fields_ = [
        ("ai_flags", ctypes.c_int32),
        ("ai_family", ctypes.c_int32),
        ("ai_socktype", ctypes.c_int32),
        ("ai_protocol", ctypes.c_int32),
        ("ai_addrlen", ctypes.c_uint64),
        ("ai_canonname", ctypes.c_char_p),
        ("ai_addr", ctypes.c_ulonglong),
        ("ai_next", ctypes.c_ulonglong),
    ]
    ai_flags: int
    ai_family: int
    ai_socktype: int
    ai_protocol: int
    ai_addrlen: int
    ai_canonname: bytes
    ai_addr: int
    ai_next: int

class _ThrowInfo(ctypes.Structure):
    _fields_ = [
        ("attributes", ctypes.c_uint32),
        ("pmfnUnwind", ctypes.c_int32),
        ("pForwardCompat", ctypes.c_int32),
        ("pCatchableTypeArray", ctypes.c_int32),
    ]
    attributes: int
    pmfnUnwind: int
    pForwardCompat: int
    pCatchableTypeArray: int

class ThrowInfo(ctypes.Structure):
    _fields_ = [
        ("attributes", ctypes.c_uint32),
        ("pmfnUnwind", ctypes.c_int32),
        ("pForwardCompat", ctypes.c_int32),
        ("pCatchableTypeArray", ctypes.c_int32),
    ]
    attributes: int
    pmfnUnwind: int
    pForwardCompat: int
    pCatchableTypeArray: int

class XINPUT_STATE(ctypes.Structure):
    _fields_ = [
        ("dwPacketNumber", ctypes.c_uint32),
        ("Gamepad", ctypes.c_uint8),  # _BYTE[]
    ]
    dwPacketNumber: int
    Gamepad: list[ctypes.c_uint8]

class XINPUT_VIBRATION(ctypes.Structure):
    _fields_ = [
        ("wLeftMotorSpeed", ctypes.c_uint16),
        ("wRightMotorSpeed", ctypes.c_uint16),
    ]
    wLeftMotorSpeed: int
    wRightMotorSpeed: int

class UUID(ctypes.Structure):
    _fields_ = [
        ("Data1", ctypes.c_uint32),
        ("Data2", ctypes.c_uint16),
        ("Data3", ctypes.c_uint16),
        ("Data4", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    Data1: int
    Data2: int
    Data3: int
    Data4: list[ctypes.c_uint8]

class IID(ctypes.Structure):
    _fields_ = [
        ("Data1", ctypes.c_uint32),
        ("Data2", ctypes.c_uint16),
        ("Data3", ctypes.c_uint16),
        ("Data4", ctypes.c_uint8 * 0x8),  # unsigned __int8[8]
    ]
    Data1: int
    Data2: int
    Data3: int
    Data4: list[ctypes.c_uint8]

class PROPVARIANT(ctypes.Structure):
    _fields_ = [
        ("___u0", ctypes.c_char * 0x18),  # $5147B814FBCB81A153F9EB9C1A9B0AA7
    ]
    ___u0: bytes

class timecaps_tag(ctypes.Structure):
    _fields_ = [
        ("wPeriodMin", ctypes.c_uint32),
        ("wPeriodMax", ctypes.c_uint32),
    ]
    wPeriodMin: int
    wPeriodMax: int

class _HIDP_CAPS(ctypes.Structure):
    _fields_ = [
        ("Usage", ctypes.c_uint16),
        ("UsagePage", ctypes.c_uint16),
        ("InputReportByteLength", ctypes.c_uint16),
        ("OutputReportByteLength", ctypes.c_uint16),
        ("FeatureReportByteLength", ctypes.c_uint16),
        ("Reserved", ctypes.c_uint16 * 0x11),  # USHORT[17]
        ("NumberLinkCollectionNodes", ctypes.c_uint16),
        ("NumberInputButtonCaps", ctypes.c_uint16),
        ("NumberInputValueCaps", ctypes.c_uint16),
        ("NumberInputDataIndices", ctypes.c_uint16),
        ("NumberOutputButtonCaps", ctypes.c_uint16),
        ("NumberOutputValueCaps", ctypes.c_uint16),
        ("NumberOutputDataIndices", ctypes.c_uint16),
        ("NumberFeatureButtonCaps", ctypes.c_uint16),
        ("NumberFeatureValueCaps", ctypes.c_uint16),
        ("NumberFeatureDataIndices", ctypes.c_uint16),
    ]
    Usage: int
    UsagePage: int
    InputReportByteLength: int
    OutputReportByteLength: int
    FeatureReportByteLength: int
    Reserved: list[ctypes.c_uint16]
    NumberLinkCollectionNodes: int
    NumberInputButtonCaps: int
    NumberInputValueCaps: int
    NumberInputDataIndices: int
    NumberOutputButtonCaps: int
    NumberOutputValueCaps: int
    NumberOutputDataIndices: int
    NumberFeatureButtonCaps: int
    NumberFeatureValueCaps: int
    NumberFeatureDataIndices: int

class _HIDD_ATTRIBUTES(ctypes.Structure):
    _fields_ = [
        ("Size", ctypes.c_uint64),
        ("VendorID", ctypes.c_uint16),
        ("ProductID", ctypes.c_uint16),
        ("VersionNumber", ctypes.c_uint16),
    ]
    Size: int
    VendorID: int
    ProductID: int
    VersionNumber: int

class _RTTIClassHierarchyDescriptor(ctypes.Structure):
    _fields_ = [
        ("signature", ctypes.c_char * 0x4),
        ("attributes", ctypes.c_char * 0x4),
        ("numBaseClasses", ctypes.c_char * 0x4),
        ("baseClassArray", ctypes.c_char * 0x4),
    ]
    signature: bytes
    attributes: bytes
    numBaseClasses: bytes
    baseClassArray: bytes

class _RTTIBaseClassDescriptor(ctypes.Structure):
    _fields_ = [
        ("typeDescriptor", ctypes.c_char * 0x4),
        ("numContainedBases", ctypes.c_char * 0x4),
        ("pmd", ctypes.c_char * 0xC),
        ("attributes", ctypes.c_char * 0x4),
    ]
    typeDescriptor: bytes
    numContainedBases: bytes
    pmd: bytes
    attributes: bytes

class _RTTICompleteObjectLocator(ctypes.Structure):
    _fields_ = [
        ("signature", ctypes.c_char * 0x4),
        ("offset", ctypes.c_char * 0x4),
        ("cdOffset", ctypes.c_char * 0x4),
        ("typeDescriptor", ctypes.c_char * 0x4),
        ("classDescriptor", ctypes.c_char * 0x4),
        ("objectBase", ctypes.c_char * 0x4),
    ]
    signature: bytes
    offset: bytes
    cdOffset: bytes
    typeDescriptor: bytes
    classDescriptor: bytes
    objectBase: bytes

def packed_store_active___un_3C_unf_3E_vyf(): pass

def ispcVecAdd___un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_uni(): pass

def ispcVecSub___un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_uni(): pass

def ispcVecMul___un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_uni(): pass

def ispcVecScalarMul___un_3C_unf_3E_un_3C_unf_3E_unfuni(): pass

def ispcVecScalarMulAdd___un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_unfuni(): pass

def ispcVecScalarMulSub___un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_unfuni(): pass

def ispcFloatAabbTest___un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_uni_3E_un_3C_unf_3E_uni(): pass

def ispcAabb16RayTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcAabb32RayTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcAabb16SweptAabbTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_uni_3E_uniuni(): pass

def ispcAabb32SweptAabbTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcSubAabb16RayTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcSubAabb32RayTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcSubAabb16SweptAabbTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_unf_3E_un_3C_uni_3E_uni(): pass

def ispcSubAabb32SweptAabbTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def dot___vyf_3C_3_3E_vyf_3C_3_3E_(): pass

def length___vyf_3C_3_3E_(): pass

def cross___vyf_3C_3_3E_vyf_3C_3_3E_(): pass

def ispcTri16RayTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri16RayTestFlat___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri32RayTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri32RayTestFlat___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri16SweptAabbTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri16SweptAabbTestFlat___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri32SweptAabbTestFlat___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_unf_3E_un_3C_uni_3E_uni(): pass

def ispcTri32SweptAabbTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcAabb16AabbTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_unsunsunsunsunsunsun_3C_uni_3E_uniuni(): pass

def ispcAabb32AabbTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_uniuniuniuniuniuniun_3C_uni_3E_uni(): pass

def ispcSubAabb16AabbTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unsunsunsunsunsunsun_3C_uni_3E_uni(): pass

def ispcSubAabb32AabbTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_uniuniuniuniuniuniun_3C_uni_3E_uni(): pass

def ispcSubAabb32SphereTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri16AabbTest___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri16AabbTestFlat___un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_un_3C_uns_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri32AabbTestFlat___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ispcTri32AabbTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfun_3C_uni_3E_uni(): pass

def ClosestPointOnTriangle___vyf_3C_3_3E_vyf_3C_3_3E_vyf_3C_3_3E_vyf_3C_3_3E_(): pass

def ispcTri32SphereTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfun_3C_uni_3E_un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_uni(): pass

def IntersectRaySphere___vyf_3C_3_3E_vyf_3C_3_3E_vyf_3C_3_3E_vyf(): pass

def ispcTri32SweptSphereTest___un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_un_3C_uni_3E_unfunfunfunfunfunfunfunfunfunfun_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_un_3C_unf_3E_un_3C_uni_3E_uni(): pass

def ispcVecAdd(): pass

def ispcVecSub(): pass

def ispcVecMul(): pass

def ispcVecScalarMul(): pass

def ispcVecScalarMulAdd(): pass

def ispcVecScalarMulSub(): pass

def ispcFloatAabbTest(): pass

def ispcAabb16RayTest(): pass

def ispcAabb32RayTest(): pass

def ispcAabb16SweptAabbTest(): pass

def ispcAabb32SweptAabbTest(): pass

def ispcSubAabb16RayTest(): pass

def ispcSubAabb32RayTest(): pass

def ispcSubAabb16SweptAabbTest(): pass

def ispcSubAabb32SweptAabbTest(): pass

def ispcTri16RayTest(): pass

def ispcTri16RayTestFlat(): pass

def ispcTri32RayTest(): pass

def ispcTri32RayTestFlat(): pass

def ispcTri16SweptAabbTest(): pass

def ispcTri16SweptAabbTestFlat(): pass

def ispcTri32SweptAabbTestFlat(): pass

def ispcTri32SweptAabbTest(): pass

def ispcAabb16AabbTest(): pass

def ispcAabb32AabbTest(): pass

def ispcSubAabb16AabbTest(): pass

def ispcSubAabb32AabbTest(): pass

def ispcSubAabb32SphereTest(): pass

def ispcTri16AabbTest(): pass

def ispcTri16AabbTestFlat(): pass

def ispcTri32AabbTestFlat(): pass

def ispcTri32AabbTest(): pass

def ispcTri32SphereTest(): pass

def ispcTri32SweptSphereTest(): pass

def sub_14001D810(): pass

def ispcDoSomething___un_3C_unf_3E_CuniCunf(): pass

def ispcGetNoise___un_3C_unf_3E_CuniCunfCunf(): pass

def ispcProcessVocAlien___REFs_5B_unVocAlien_5D_Cuni(): pass

def ispcProcessVocAlienHalfRate___REFs_5B_unVocAlien_5D_Cuni(): pass

def ispcProcessVocAlienEco___REFs_5B_unVocAlien_5D_Cuni(): pass

def ispcProcessVocAlienEcoHalfRate___REFs_5B_unVocAlien_5D_Cuni(): pass

def ispcDoSomething(): pass

def ispcGetNoise(): pass

def ispcProcessVocAlien(): pass

def ispcProcessVocAlienHalfRate(): pass

def ispcProcessVocAlienEco(): pass

def ispcProcessVocAlienEcoHalfRate(): pass

class CSingletonCriticalSection(ctypes.Structure):
    def vector_deleting_destructor(): pass


def Module_InitializeHelper(): pass

def LoadModule(): pass

def ValidateModuleHandle(): pass

def NvAPI_Initialize(): pass

def sub_14002A2F0(): pass

def sub_14002A3F0(): pass

def sub_14002A450(): pass

def sub_14002A4B0(): pass

def nvGetSystemDirectoryW(): pass

def sub_14002A620(): pass

def sub_14002A790(): pass

def StringCchCopyW(): pass

def StringCbCopyW(): pass

def StringCchCopyNW(): pass

def StringCchCatW(): pass

def StringCbCatW(): pass

def sub_14002AC40(): pass

def sub_14002ADB0(): pass

def sub_14002AEC0(): pass

def sub_14002B010(): pass

def sub_14002B1D0(): pass

def sub_14002B480(): pass

def sub_14002B510(): pass

def sub_14002B5B0(): pass

def sub_14002B6F0(): pass

def sub_14002B920(): pass

def sub_14002BAF0(): pass

def sub_14002BC20(): pass

def sub_14002BE10(): pass

def sub_14002C0C0(): pass

def sub_14002C290(): pass

def nvLoadLibraryFromTrustedLocationExW(): pass

def nvGetDriverStoreW(): pass

def sub_14002C820(): pass

def nvGetDriverRegkeyW(): pass

def nvDetectDisplayDriverModuleRegistrationW(): pass

def nvDetectDisplayDriverModuleLocationW(): pass

def nvLoadSystemLibraryExW(): pass

def _dynamic_initializer_for__cTkTimeManager__mTimeJumpCallbacks__(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed__(): pass

def _dynamic_initializer_for__k4x4IdentityRight__(): pass

def _dynamic_initializer_for__k4x4IdentityUp__(): pass

def _dynamic_initializer_for__k4x4IdentityAt__(): pass

def _dynamic_initializer_for__k4x4IdentityPos__(): pass

def _dynamic_initializer_for__k3x4IdentityRight__(): pass

def _dynamic_initializer_for__k3x4IdentityUp__(): pass

def _dynamic_initializer_for__k3x4IdentityAt__(): pass

def _dynamic_initializer_for__k3x4IdentityPos__(): pass

def _dynamic_initializer_for__mCache__(): pass

def _dynamic_initializer_for__kTkSelectionDataNull__(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours__(): pass

def _dynamic_initializer_for__kNonCollidingPlane__(): pass

def _dynamic_initializer_for__kUnitsPerQuantizedRegionAsVec__(): pass

def _dynamic_initializer_for__kUnitsPerQuantizedRegionAsVecInv__(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown__(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp__(): pass

def _dynamic_initializer_for__kNoIcon__(): pass

def _dynamic_initializer_for__kNoIcons__(): pass

def _dynamic_initializer_for__kInstancercTkAnimationComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercTkAnimationComponent__(): pass

def _dynamic_initializer_for__kInstancercGcCustomInventoryComponent__(): pass

def _dynamic_initializer_for__kInstancercGcFoliageComponent__(): pass

def _dynamic_initializer_for__cGcRegionDecoratorBuildings__mDefaultBuildingComponentResHandle__(): pass

def _dynamic_initializer_for__cGcNavMeshTile__Utils__mvOffset__(): pass

def _dynamic_initializer_for__kInstancercTkRandomComponent__(): pass

def _dynamic_initializer_for__kaBehaviourTreeNodeStateColours__(): pass

def _dynamic_initializer_for__kInstancercTkNetReplicatedEntityComponent__(): pass

def _dynamic_initializer_for__kNullDiscoveryData__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerEffectsComponent__(): pass

def _dynamic_initializer_for__kInstancercGcScannableComponent__(): pass

def _dynamic_initializer_for__kInstancercGcDestructableComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcDestructableComponent__(): pass

def _dynamic_initializer_for__kInstancercGcGrabbableComponent__(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec__(): pass

def _dynamic_initializer_for__kInstancercGcAISpaceshipComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcAISpaceshipComponent__(): pass

def _dynamic_initializer_for__kInstancercGcSpaceshipComponent__(): pass

def _dynamic_initializer_for__kInstancercGcCustomisationComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcCustomisationComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent__(): pass

def _dynamic_initializer_for__cGcRealityManager__saIdsPendingValidation__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour__(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent__(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent__(): pass

def _dynamic_initializer_for__gApplication__(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent__(): pass

def _dynamic_initializer_for__cTkUnixTimestamp__mTimeFormatString__(): pass

def _dynamic_initializer_for__cTkUnixTimestamp__mLocale__(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent__(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent__(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent__(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___0(): pass

def _dynamic_initializer_for__k3x4IdentityAt___0(): pass

def _dynamic_initializer_for__k3x4IdentityUp___0(): pass

def _dynamic_initializer_for__k3x4IdentityRight___0(): pass

def _dynamic_initializer_for__kNoIcon___0(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___0(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___0(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent__(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcBaseMiniPortalComponent__(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___0(): pass

def _dynamic_initializer_for__gAssertMutex__(): pass

def _dynamic_initializer_for__kNoIcon___1(): pass

def _dynamic_initializer_for__k3x4IdentityPos___1(): pass

def _dynamic_initializer_for__k3x4IdentityAt___1(): pass

def _dynamic_initializer_for__k3x4IdentityUp___1(): pass

def _dynamic_initializer_for__k3x4IdentityRight___1(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___1(): pass

def _dynamic_initializer_for__k3x4IdentityPos___2(): pass

def _dynamic_initializer_for__k3x4IdentityAt___2(): pass

def _dynamic_initializer_for__k3x4IdentityUp___2(): pass

def _dynamic_initializer_for__k3x4IdentityRight___2(): pass

def _dynamic_initializer_for__gAudioAggDebugVizBoundColour__(): pass

def _dynamic_initializer_for__gAudioAggDebugVizAttenColour__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___1(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent__(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent__(): pass

def _dynamic_initializer_for__cGcAudioPulseMusic__mRNG__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___3(): pass

def _dynamic_initializer_for__k3x4IdentityAt___3(): pass

def _dynamic_initializer_for__k3x4IdentityUp___3(): pass

def _dynamic_initializer_for__k3x4IdentityRight___3(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___2(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent__(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___2(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent__(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent__(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent__(): pass

def _dynamic_initializer_for__kInstancercGcByteBeatPlayerComponent__(): pass

def _dynamic_initializer_for__cGcByteBeat__sCopyString__(): pass

def _dynamic_initializer_for__cGcByteBeat__gaCachedByteBeats__(): pass

def _dynamic_initializer_for__cGcByteBeat__maScreenTexturePool__(): pass

def _dynamic_initializer_for__cGcBaseBuildingGenerator__mkGridCellSize__(): pass

def _dynamic_initializer_for__cGcBaseBuildingGenerator__gSingleton__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___4(): pass

def _dynamic_initializer_for__k3x4IdentityAt___4(): pass

def _dynamic_initializer_for__k3x4IdentityUp___4(): pass

def _dynamic_initializer_for__k3x4IdentityRight___4(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec___0(): pass

def _dynamic_initializer_for__kNoIcon___2(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___3(): pass

def _dynamic_initializer_for__kInstancercGcExpeditionHologramComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcExpeditionHologramComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___3(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent__(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent__(): pass

def _dynamic_initializer_for__kInstancercGcObjectSpawnerComponent__(): pass

def _dynamic_initializer_for__kInstancercGcBaseDefenceComponent__(): pass

def _dynamic_initializer_for__kInstancercGcByteBeatSwitchComponent__(): pass

def _dynamic_initializer_for__kInstancercGcNPCHabitationComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcNPCHabitationComponent__(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcHologramComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcHologramComponent__(): pass

def _dynamic_initializer_for__kInstancercGcBaseMiniPortalComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcVehicleGarageComponent__(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent__(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent__(): pass

def _dynamic_initializer_for__cGcWFCBaseGenerator__mClusterNavGraphConstructionData__(): pass

def _dynamic_initializer_for__cGcWFCBaseGenerator__mNavigationAreaResHandle__(): pass

def _dynamic_initializer_for__kInstancercGcColouriseComponent__(): pass

def _dynamic_initializer_for__cGcBaseBuildingManager__mOffenseReporter__(): pass

def _dynamic_initializer_for__cGcBaseBuildingManager__mBaseBuildingManager__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___5(): pass

def _dynamic_initializer_for__k3x4IdentityAt___5(): pass

def _dynamic_initializer_for__k3x4IdentityUp___5(): pass

def _dynamic_initializer_for__k3x4IdentityRight___5(): pass

def _dynamic_initializer_for__kNoIcon___3(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___4(): pass

def _dynamic_initializer_for__sRegisteredComponentBaseObjectHandles__(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___1(): pass

def _dynamic_initializer_for__k3x4IdentityPos___6(): pass

def _dynamic_initializer_for__k3x4IdentityAt___6(): pass

def _dynamic_initializer_for__k3x4IdentityUp___6(): pass

def _dynamic_initializer_for__k3x4IdentityRight___6(): pass

def _dynamic_initializer_for__kNoIcon___4(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___2(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___5(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___1(): pass

def _dynamic_initializer_for__cGcWFCBuildingState__mInstances__(): pass

def _dynamic_initializer_for__cGcWFCBuildingState__mDecorationColourPartIds__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___7(): pass

def _dynamic_initializer_for__k3x4IdentityAt___7(): pass

def _dynamic_initializer_for__k3x4IdentityUp___7(): pass

def _dynamic_initializer_for__k3x4IdentityRight___7(): pass

def _dynamic_initializer_for__kNoIcon___5(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___0(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___6(): pass

def _dynamic_initializer_for__maMetaData__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent__(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___2(): pass

def _dynamic_initializer_for__k3x4IdentityAt___8(): pass

def _dynamic_initializer_for__k3x4IdentityUp___8(): pass

def _dynamic_initializer_for__k3x4IdentityRight___8(): pass

def _dynamic_initializer_for__kNoIcon___6(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcExpeditionHologramComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcExpeditionHologramComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkLODComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercTkLODComponent__(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___4(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___7(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent___1(): pass

def _dynamic_initializer_for__k3x4IdentityPos___8(): pass

def _dynamic_initializer_for__k3x4IdentityAt___9(): pass

def _dynamic_initializer_for__k3x4IdentityUp___9(): pass

def _dynamic_initializer_for__k3x4IdentityRight___9(): pass

def _dynamic_initializer_for__kNoIcon___7(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___8(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___5(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___8(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___1(): pass

def _dynamic_initializer_for__kaAllQueryInventories__(): pass

def _dynamic_initializer_for__kaAllQueryInventories_NoFreighter__(): pass

def _dynamic_initializer_for__kaAllQueryInventoriesAndChests__(): pass

def _dynamic_initializer_for__kaPoliceProbeQueryInventories__(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent__(): pass

def _dynamic_initializer_for__kInstancercGcGeneratorUnitComponent__(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___1(): pass

def _dynamic_initializer_for__k3x4IdentityPos___9(): pass

def _dynamic_initializer_for__k3x4IdentityAt___10(): pass

def _dynamic_initializer_for__k3x4IdentityUp___10(): pass

def _dynamic_initializer_for__k3x4IdentityRight___10(): pass

def _dynamic_initializer_for__kNoIcon___8(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___1(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___9(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcByteBeatPlayerComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___4(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___4(): pass

def _dynamic_initializer_for__cGcPlayerByteBeatLibrary__mByteBeatLibrary__(): pass

def _dynamic_initializer_for__kNoIcon___9(): pass

def _dynamic_initializer_for__k3x4IdentityPos___10(): pass

def _dynamic_initializer_for__k3x4IdentityAt___11(): pass

def _dynamic_initializer_for__k3x4IdentityUp___11(): pass

def _dynamic_initializer_for__k3x4IdentityRight___11(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcExpeditionHologramComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcExpeditionHologramComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___2(): pass

def _dynamic_initializer_for__cGcPlayerFleetManager__sFrigateTraitIDMap__(): pass

def _dynamic_initializer_for__cGcPlayerFleetManager__sExpeditionRewardIDMap__(): pass

def _dynamic_initializer_for__cGcPlayerFleetManager__sExpeditionInterventionEventIDMap__(): pass

def _dynamic_initializer_for__cGcPlayerFleetManager__sExpeditionEventIDMap__(): pass

def _dynamic_initializer_for__cGcPlayerFleetManager__sapAvailableFrigateTraits__(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___3(): pass

def _dynamic_initializer_for__k3x4IdentityPos___11(): pass

def _dynamic_initializer_for__k3x4IdentityAt___12(): pass

def _dynamic_initializer_for__k3x4IdentityUp___12(): pass

def _dynamic_initializer_for__k3x4IdentityRight___12(): pass

def _dynamic_initializer_for__kNoIcon___10(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent__(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___4(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___4(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___10(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent__(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___7(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___5(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent___2(): pass

def _dynamic_initializer_for__kNoIcon___11(): pass

def _dynamic_initializer_for__k3x4IdentityPos___12(): pass

def _dynamic_initializer_for__k3x4IdentityAt___13(): pass

def _dynamic_initializer_for__k3x4IdentityUp___13(): pass

def _dynamic_initializer_for__k3x4IdentityRight___13(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___3(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___11(): pass

def _dynamic_initializer_for__kInstancercGcTechnologyAttachmentComponent__(): pass

def _dynamic_initializer_for__kInstancercGcRocketLockerComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___12(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___6(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent___3(): pass

def _dynamic_initializer_for__cGcSynchronisedStateManager__sInstance__(): pass

def _dynamic_initializer_for__cGcTerrainEditBlockBuffer__mSharedMutexes__(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec___1(): pass

def _dynamic_initializer_for__kNoIcon___12(): pass

def _dynamic_initializer_for__k3x4IdentityAt___14(): pass

def _dynamic_initializer_for__k3x4IdentityUp___14(): pass

def _dynamic_initializer_for__k3x4IdentityRight___14(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___13(): pass

def _dynamic_initializer_for__cGcUserSettings__sDefaultSettingsData__(): pass

def _dynamic_initializer_for__kNoIcon___13(): pass

def _dynamic_initializer_for__k3x4IdentityPos___13(): pass

def _dynamic_initializer_for__k3x4IdentityAt___15(): pass

def _dynamic_initializer_for__k3x4IdentityUp___15(): pass

def _dynamic_initializer_for__k3x4IdentityRight___15(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec___2(): pass

def _dynamic_initializer_for__cGcProceduralTextureManager__mTextureCache__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___12(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___4(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___5(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___5(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___7(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcPlayerControlComponent__(): pass

def _dynamic_initializer_for__kNoIcon___14(): pass

def _dynamic_initializer_for__k3x4IdentityPos___14(): pass

def _dynamic_initializer_for__k3x4IdentityAt___16(): pass

def _dynamic_initializer_for__k3x4IdentityUp___16(): pass

def _dynamic_initializer_for__k3x4IdentityRight___16(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___14(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___10(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___13(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___8(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___8(): pass

def _dynamic_initializer_for__cGcCameraManager__maFocalTargets__(): pass

def _dynamic_initializer_for__cGcGameCamera__maShakes__(): pass

def _dynamic_initializer_for__kNullDiscoveryData___0(): pass

def _dynamic_initializer_for__k3x4IdentityPos___15(): pass

def _dynamic_initializer_for__k3x4IdentityAt___17(): pass

def _dynamic_initializer_for__k3x4IdentityUp___17(): pass

def _dynamic_initializer_for__k3x4IdentityRight___17(): pass

def _dynamic_initializer_for__kNoIcon___15(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___15(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___11(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___14(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___9(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___9(): pass

def _dynamic_initializer_for__cGcFrontendPageByteBeat__maIconResources__(): pass

def _dynamic_initializer_for__cGcFrontendPageByteBeat__mSawIconResource__(): pass

def _dynamic_initializer_for__cGcFrontendPageByteBeat__mSineIconResource__(): pass

def _dynamic_initializer_for__cGcFrontendPageByteBeat__mSquareIconResource__(): pass

def _dynamic_initializer_for__cGcFrontendPageByteBeat__mTriangleIconResource__(): pass

def _dynamic_initializer_for__cGcFrontendPageByteBeat__mapTreeNodes__(): pass

def _dynamic_initializer_for__cGcFrontendPageByteBeat__mapFreeTreeNodes__(): pass

def _dynamic_initializer_for__cGcFrontendPageByteBeat__mSequencer__(): pass

def _dynamic_initializer_for__kInstancercGcCreatureFeederComponent__(): pass

def _dynamic_initializer_for__kInstancercGcCreatureHarvesterComponent__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___16(): pass

def _dynamic_initializer_for__k3x4IdentityAt___18(): pass

def _dynamic_initializer_for__k3x4IdentityUp___18(): pass

def _dynamic_initializer_for__k3x4IdentityRight___18(): pass

def _dynamic_initializer_for__kNoIcon___16(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___16(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___12(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___15(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___10(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcByteBeatSwitchComponent___0(): pass

def _dynamic_initializer_for__cGcFrontendPageInteractions__saMaterialsForSelectedDescriptors__(): pass

def _dynamic_initializer_for__cGcFrontendPageInteractions__sChangeRaceDialogData__(): pass

def _dynamic_initializer_for__cGcFrontendPageInteractions__sTeleportDialogData__(): pass

def _dynamic_initializer_for__cGcFrontendPageInteractions__svIntermediateInteractionStatsWorldPos__(): pass

def _dynamic_initializer_for__gByteBeatGroupData__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___17(): pass

def _dynamic_initializer_for__k3x4IdentityAt___19(): pass

def _dynamic_initializer_for__k3x4IdentityUp___19(): pass

def _dynamic_initializer_for__k3x4IdentityRight___19(): pass

def _dynamic_initializer_for__kNullDiscoveryData___1(): pass

def _dynamic_initializer_for__kNoIcon___17(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___2(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___13(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___16(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___11(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcWaypointComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcWaypointComponent__(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcCreatureFeederComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcCreatureHarvesterComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcGeneratorUnitComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcExpeditionHologramComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcExpeditionHologramComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___3(): pass

def _dynamic_initializer_for__sapAvailableMissions__(): pass

def _dynamic_initializer_for__sapWaitingMissions__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___18(): pass

def _dynamic_initializer_for__k3x4IdentityAt___20(): pass

def _dynamic_initializer_for__k3x4IdentityUp___20(): pass

def _dynamic_initializer_for__k3x4IdentityRight___20(): pass

def _dynamic_initializer_for__kNoIcon___18(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___4(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___14(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___17(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___12(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___12(): pass

def _dynamic_initializer_for__saWeekendMissions__(): pass

def _dynamic_initializer_for__saAvailableMissions__(): pass

def _dynamic_initializer_for__cGcFrontendPageOptions__mUnappliedGraphicsSettings__(): pass

def _dynamic_initializer_for__cGcFrontendPageOptions__mOpenMenus__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___19(): pass

def _dynamic_initializer_for__k3x4IdentityAt___21(): pass

def _dynamic_initializer_for__k3x4IdentityUp___21(): pass

def _dynamic_initializer_for__k3x4IdentityRight___21(): pass

def _dynamic_initializer_for__kNoIcon___19(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___19(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___15(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___18(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___13(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___13(): pass

def _dynamic_initializer_for__cGcFrontendPagePortalRunes__mIconData__(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcWaypointComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcWaypointComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcRocketLockerComponent___0(): pass

def _dynamic_initializer_for__k3x4IdentityPos___20(): pass

def _dynamic_initializer_for__k3x4IdentityAt___22(): pass

def _dynamic_initializer_for__k3x4IdentityUp___22(): pass

def _dynamic_initializer_for__k3x4IdentityRight___22(): pass

def _dynamic_initializer_for__kNoIcon___20(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___20(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___16(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___19(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___14(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcGeneratorUnitComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___4(): pass

def _dynamic_initializer_for__cGcFrontendPageTree__sCurrentDisplayTree__(): pass

def _dynamic_initializer_for__kInstancercGcVehicleRaceInviteComponent__(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___5(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___2(): pass

def _dynamic_initializer_for__kNullDiscoveryData___2(): pass

def _dynamic_initializer_for__k3x4IdentityPos___21(): pass

def _dynamic_initializer_for__k3x4IdentityAt___23(): pass

def _dynamic_initializer_for__k3x4IdentityUp___23(): pass

def _dynamic_initializer_for__k3x4IdentityRight___23(): pass

def _dynamic_initializer_for__kNoIcon___21(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___21(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___17(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___20(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___15(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___5(): pass

def _dynamic_initializer_for__kNoIcon___22(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___1(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___18(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___21(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___16(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcCreatureFeederComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcCreatureHarvesterComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcGeneratorUnitComponent___2(): pass

def _dynamic_initializer_for__kaFrontendActionButtons__(): pass

def _dynamic_initializer_for__kInstancercGcTechnologyAttachmentComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcCharacterInterfaceComponent__(): pass

def _dynamic_initializer_for__kNullDiscoveryData___3(): pass

def _dynamic_initializer_for__k3x4IdentityPos___22(): pass

def _dynamic_initializer_for__k3x4IdentityAt___24(): pass

def _dynamic_initializer_for__k3x4IdentityUp___24(): pass

def _dynamic_initializer_for__k3x4IdentityRight___24(): pass

def _dynamic_initializer_for__kNoIcon___23(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___23(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___19(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___22(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___17(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___6(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___6(): pass

def _dynamic_initializer_for__k3x4IdentityPos___23(): pass

def _dynamic_initializer_for__k3x4IdentityAt___25(): pass

def _dynamic_initializer_for__k3x4IdentityUp___25(): pass

def _dynamic_initializer_for__k3x4IdentityRight___25(): pass

def _dynamic_initializer_for__kNoIcon___24(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcMarkerComponent__(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___7(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcVehicleGarageComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___24(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___20(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___23(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___18(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___6(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___6(): pass

def _dynamic_initializer_for__kHiddenHitboxColour__(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent__(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent__(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___0(): pass

def _dynamic_initializer_for__k3x4IdentityPos___24(): pass

def _dynamic_initializer_for__kNoIcon___25(): pass

def _dynamic_initializer_for__k3x4IdentityAt___26(): pass

def _dynamic_initializer_for__k3x4IdentityUp___26(): pass

def _dynamic_initializer_for__k3x4IdentityRight___26(): pass

def _dynamic_initializer_for__kInstancercGcDistanceScaleComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcCreatureSpawnComponent__(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___7(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___8(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___4(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___6(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___21(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___24(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___19(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcBaseDefenceComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPlayerControlComponent___0(): pass

def _dynamic_initializer_for__k3x4IdentityPos___25(): pass

def _dynamic_initializer_for__k3x4IdentityAt___27(): pass

def _dynamic_initializer_for__k3x4IdentityUp___27(): pass

def _dynamic_initializer_for__k3x4IdentityRight___27(): pass

def _dynamic_initializer_for__kNoIcon___26(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___26(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___22(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___25(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___20(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___5(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___9(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcTurretComponent__(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___8(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcMissileComponent__(): pass

def _dynamic_initializer_for__kInstancercGcSpaceshipShieldComponent__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___26(): pass

def _dynamic_initializer_for__k3x4IdentityAt___28(): pass

def _dynamic_initializer_for__k3x4IdentityUp___28(): pass

def _dynamic_initializer_for__k3x4IdentityRight___28(): pass

def _dynamic_initializer_for__kNoIcon___27(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___9(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___9(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___26(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___27(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___21(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___10(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___3(): pass

def _dynamic_initializer_for__kNoIcon___28(): pass

def _dynamic_initializer_for__k3x4IdentityPos___27(): pass

def _dynamic_initializer_for__k3x4IdentityAt___29(): pass

def _dynamic_initializer_for__k3x4IdentityUp___29(): pass

def _dynamic_initializer_for__k3x4IdentityRight___29(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___29(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___24(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___27(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___2(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___22(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___22(): pass

def _dynamic_initializer_for__k3x4IdentityAt___30(): pass

def _dynamic_initializer_for__k3x4IdentityUp___30(): pass

def _dynamic_initializer_for__k3x4IdentityRight___30(): pass

def _dynamic_initializer_for__kNoIcon___29(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___30(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___29(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___29(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___25(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___28(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___23(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___23(): pass

def _dynamic_initializer_for__kNoIcon___30(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___31(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___30(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___30(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___4(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___29(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___5(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___24(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___11(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___11(): pass

def _dynamic_initializer_for__cGcNGuiLayer__mgDefaultText__(): pass

def _dynamic_initializer_for__cGcNGuiLayer__mgDefaultTextSpecial__(): pass

def _dynamic_initializer_for__cGcNGuiLayer__mgDefaultGraphic__(): pass

def _dynamic_initializer_for__cGcNGuiLayer__mgDefaultLayer__(): pass

def _dynamic_initializer_for__cGcNGuiLayer__mgpPopupLayer__(): pass

def _dynamic_initializer_for__gEditorGui__(): pass

def _dynamic_initializer_for__gGameGui__(): pass

def _dynamic_initializer_for__kNoIcon___31(): pass

def _dynamic_initializer_for__k3x4IdentityPos___28(): pass

def _dynamic_initializer_for__k3x4IdentityAt___31(): pass

def _dynamic_initializer_for__k3x4IdentityUp___31(): pass

def _dynamic_initializer_for__k3x4IdentityRight___31(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___32(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___31(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___31(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___27(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___30(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___25(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___2(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___0(): pass

def _dynamic_initializer_for__gGuiRoot__(): pass

def _dynamic_initializer_for__gMetadata__(): pass

def _dynamic_initializer_for__kInstancercTkInstanceWindComponent__(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcCharacterInterfaceComponent___0(): pass

def _dynamic_initializer_for__cGcModelViewerCustomisationWindow__sCustomisationUIData__(): pass

def _dynamic_initializer_for__cGcModelViewerCustomisationWindow__sCustomisationData__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___29(): pass

def _dynamic_initializer_for__k3x4IdentityAt___32(): pass

def _dynamic_initializer_for__k3x4IdentityUp___32(): pass

def _dynamic_initializer_for__k3x4IdentityRight___32(): pass

def _dynamic_initializer_for__kNoIcon___32(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___1(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___31(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcDistanceScaleComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcNetworkPlayerMarkerComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___33(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___32(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___32(): pass

def _dynamic_initializer_for__cGcMatchmaking__mFailedInviteCounts__(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___26(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___26(): pass

def _dynamic_initializer_for__cGcNetworkAbandonedFreightersManager__sSingleton__(): pass

def _dynamic_initializer_for__kInstancercGcNPCHabitationComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcNPCHabitationComponent___0(): pass

def _dynamic_initializer_for__kNoIcon___33(): pass

def _dynamic_initializer_for__k3x4IdentityPos___30(): pass

def _dynamic_initializer_for__k3x4IdentityAt___33(): pass

def _dynamic_initializer_for__k3x4IdentityUp___33(): pass

def _dynamic_initializer_for__k3x4IdentityRight___33(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___34(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___33(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___33(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___29(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___32(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___29(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___29(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___27(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___12(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___31(): pass

def _dynamic_initializer_for__k3x4IdentityAt___34(): pass

def _dynamic_initializer_for__k3x4IdentityUp___34(): pass

def _dynamic_initializer_for__k3x4IdentityRight___34(): pass

def _dynamic_initializer_for__kNoIcon___34(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___34(): pass

def _dynamic_initializer_for__kInstancercGcPlayerWeaponComponent__(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___13(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___35(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___34(): pass

def _dynamic_initializer_for__DisplayMessage__mkNoColour__(): pass

def _dynamic_initializer_for__gidMutedAccounts__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___35(): pass

def _dynamic_initializer_for__k3x4IdentityUp___35(): pass

def _dynamic_initializer_for__k3x4IdentityRight___35(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec___3(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___36(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___35(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___35(): pass

def _dynamic_initializer_for__kInstancercGcDistanceScaleComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___37(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___36(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___36(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___29(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___30(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___33(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___30(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___30(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___28(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___28(): pass

def _dynamic_initializer_for__kNoIcon___35(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___38(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___31(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___34(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___31(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___37(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___37(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___31(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___5(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___30(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___29(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___29(): pass

def _dynamic_initializer_for__kNoIcon___36(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___39(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___38(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___38(): pass

def _dynamic_initializer_for__kInstancercGcExpeditionHologramComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercGcExpeditionHologramComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___8(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___31(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___32(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___35(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___32(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___32(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___30(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___30(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___2(): pass

def _dynamic_initializer_for__k3x4IdentityPos___32(): pass

def _dynamic_initializer_for__k3x4IdentityAt___36(): pass

def _dynamic_initializer_for__k3x4IdentityUp___36(): pass

def _dynamic_initializer_for__k3x4IdentityRight___36(): pass

def _dynamic_initializer_for__kNoIcon___37(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___33(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___36(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___33(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___40(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___39(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___39(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___33(): pass

def _dynamic_initializer_for__kInstancercGcChairComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___9(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___32(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___31(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___31(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___4(): pass

def _dynamic_initializer_for__cGcNPCBehaviourChooseInteraction__sNPCUpdateQueue__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___33(): pass

def _dynamic_initializer_for__k3x4IdentityAt___37(): pass

def _dynamic_initializer_for__k3x4IdentityUp___37(): pass

def _dynamic_initializer_for__k3x4IdentityRight___37(): pass

def _dynamic_initializer_for__kNoIcon___38(): pass

def _dynamic_initializer_for__kInstancercGcAudioAreaTriggerComponent__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___37(): pass

def _dynamic_initializer_for__kInstancercGcByteBeatPlayerComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcHologramComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcHologramComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___41(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___40(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___40(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___33(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___34(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___34(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___34(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___32(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___32(): pass

def _dynamic_initializer_for__kInstancercGcNPCHabitationComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcNPCHabitationComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcObjectSpawnerComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcSpawnedObjectComponent__(): pass

def _dynamic_initializer_for__kInstancercGcDissolveEffectComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcAlienPodComponent__(): pass

def _dynamic_initializer_for__kInstancercGcAntagonistComponent__(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___14(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___14(): pass

def _dynamic_initializer_for__cGcAntagonistComponent__gAntagonistRNG__(): pass

def _dynamic_initializer_for__kInstancercGcAtmosphereEntryComponent__(): pass

def _dynamic_initializer_for__kInstancercGcDistanceScaleComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcBaseDefenceComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcTurretComponent___0(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___2(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcBaseMiniPortalComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcBuildingComponent__(): pass

def _dynamic_initializer_for__kInstancercGcByteBeatSwitchComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcChairComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcGroundWormComponent__(): pass

def _dynamic_initializer_for__cGcEncounterComponent__mEncounterStateResource__(): pass

def _dynamic_initializer_for__kInstancercGcEncounterStateComponent__(): pass

def _dynamic_initializer_for__kInstancercGcExpeditionHologramComponent___4(): pass

def _dynamic_initializer_for__kLoadInstancercGcExpeditionHologramComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcFleetHologramComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercGcFleetHologramComponent__(): pass

def _dynamic_initializer_for__kInstancercGcGrabPlayerComponent__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___34(): pass

def _dynamic_initializer_for__k3x4IdentityAt___38(): pass

def _dynamic_initializer_for__k3x4IdentityUp___38(): pass

def _dynamic_initializer_for__k3x4IdentityRight___38(): pass

def _dynamic_initializer_for__kNoIcon___39(): pass

def _dynamic_initializer_for__kInstancercGcGroundWormComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___6(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___34(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___38(): pass

def _dynamic_initializer_for__kInstancercGcHeightAdjustComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___42(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___41(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___41(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___35(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___35(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___35(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___33(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___33(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___15(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___4(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___10(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___8(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___3(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcRocketLockerComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___6(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcTechnologyAttachmentComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcExpeditionHologramComponent___5(): pass

def _dynamic_initializer_for__kLoadInstancercGcExpeditionHologramComponent___5(): pass

def _dynamic_initializer_for__cGcInteractionComponent__svForcedInteractionPosition__(): pass

def _dynamic_initializer_for__gcGcInteractionComponent_Interacted__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___35(): pass

def _dynamic_initializer_for__k3x4IdentityAt___39(): pass

def _dynamic_initializer_for__k3x4IdentityUp___39(): pass

def _dynamic_initializer_for__k3x4IdentityRight___39(): pass

def _dynamic_initializer_for__kNoIcon___40(): pass

def _dynamic_initializer_for__kInstancercGcLadderComponent__(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcLandingHelperComponent__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___39(): pass

def _dynamic_initializer_for__kInstancercGcLookAtComponent__(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___43(): pass

def _dynamic_initializer_for__kInstancercGcDistanceScaleComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___42(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcWaypointComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcWaypointComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcGeneratorUnitComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcCreatureFeederComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcCreatureHarvesterComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___42(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcObjectPlacementComponent__(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___35(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___36(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___36(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___36(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___34(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___34(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent___6(): pass

def _dynamic_initializer_for__cGcMaintenanceComponent__kPersonalRefinerUserSpaceLocation__(): pass

def _dynamic_initializer_for__kInstancercGcChairComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcScanEffectComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___4(): pass

def _dynamic_initializer_for__cGcNPCComponent__maPerceptionCache__(): pass

def _dynamic_initializer_for__cGcPortalComponent__mLastPortalData__(): pass

def _dynamic_initializer_for__kInstancercGcSentinelCoverComponent__(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___7(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___4(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___2(): pass

def _dynamic_initializer_for__gcGcSentinelCoverComponent_OnTakeDamage__(): pass

def _dynamic_initializer_for__kNoIcon___41(): pass

def _dynamic_initializer_for__k3x4IdentityPos___36(): pass

def _dynamic_initializer_for__k3x4IdentityAt___40(): pass

def _dynamic_initializer_for__k3x4IdentityUp___40(): pass

def _dynamic_initializer_for__k3x4IdentityRight___40(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___16(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcVehicleGarageComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcObjectSpawnerComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcHologramComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcHologramComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___8(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___40(): pass

def _dynamic_initializer_for__kInstancercGcBaseMiniPortalComponent___2(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___5(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcNPCHabitationComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcNPCHabitationComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___44(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___43(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___43(): pass

def _dynamic_initializer_for__kInstancercGcChairComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcLadderComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___11(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___6(): pass

def _dynamic_initializer_for__gcGcSimpleInteractionComponent_Interacted__(): pass

def _dynamic_initializer_for__kInstancercGcSquadronHologramComponent__(): pass

def _dynamic_initializer_for__kInstancercGcTechnologyAttachmentComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcThereminComponent__(): pass

def _dynamic_initializer_for__kInstancercGcTurretComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcMissileComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcFreighterSyncComponent__(): pass

def _dynamic_initializer_for__kInstancercGcBaseDefenceComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcAntagonistComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcWiringSocketComponent__(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___36(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___37(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___37(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___37(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___35(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___35(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcCustomiserCharacterComponent__(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___5(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___5(): pass

def _dynamic_initializer_for__lvScaledSize__(): pass

def _dynamic_initializer_for__kNoIcon___42(): pass

def _dynamic_initializer_for__k3x4IdentityPos___37(): pass

def _dynamic_initializer_for__k3x4IdentityAt___41(): pass

def _dynamic_initializer_for__k3x4IdentityUp___41(): pass

def _dynamic_initializer_for__k3x4IdentityRight___41(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___44(): pass

def _dynamic_initializer_for__kInstancercGcCreatureFeederComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___14(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___41(): pass

def _dynamic_initializer_for__kInstancercGcCreatureHarvesterComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___45(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___44(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___6(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcGeneratorUnitComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcRocketLockerComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___12(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___38(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___38(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___37(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___38(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___36(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___36(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___6(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___6(): pass

def _dynamic_initializer_for__k3x4IdentityPos___38(): pass

def _dynamic_initializer_for__k3x4IdentityAt___42(): pass

def _dynamic_initializer_for__k3x4IdentityUp___42(): pass

def _dynamic_initializer_for__k3x4IdentityRight___42(): pass

def _dynamic_initializer_for__kNoIcon___43(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcCharacterInterfaceComponent___1(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___42(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___38(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___39(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___39(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___46(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___45(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___45(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___39(): pass

def _dynamic_initializer_for__gActiveInNguiClothComponents__(): pass

def _dynamic_initializer_for__gStoredVertsForInitialShape__(): pass

def _dynamic_initializer_for__kInstancercGcClothComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerFullBodyIKComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerWeaponComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___37(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___37(): pass

def _dynamic_initializer_for__kInstancercGcSpringAttachmentComponent__(): pass

def _dynamic_initializer_for__kNoIcon___44(): pass

def _dynamic_initializer_for__k3x4IdentityPos___39(): pass

def _dynamic_initializer_for__k3x4IdentityAt___43(): pass

def _dynamic_initializer_for__k3x4IdentityUp___43(): pass

def _dynamic_initializer_for__k3x4IdentityRight___43(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcScanEffectComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___47(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___7(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcBaseMiniPortalComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcCameraShakeComponent__(): pass

def _dynamic_initializer_for__kInstancercGcCharacterInterfaceComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___40(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___43(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___40(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___46(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___46(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___40(): pass

def _dynamic_initializer_for__kInstancercGcColouriseComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcDistanceScaleComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcDecorationComponent__(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcMarkerComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___7(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___39(): pass

def _dynamic_initializer_for__kInstancercGcPlayerControlComponent___1(): pass

def _dynamic_initializer_for__gcGcPlayerControlComponent_OnInputChanged__(): pass

def _dynamic_initializer_for__kInstancercGcSimpleIkRecoilComponent__(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcCreatureSpawnComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkLODComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercTkLODComponent___0(): pass

def _dynamic_initializer_for__kNoIcon___45(): pass

def _dynamic_initializer_for__k3x4IdentityPos___40(): pass

def _dynamic_initializer_for__k3x4IdentityAt___44(): pass

def _dynamic_initializer_for__k3x4IdentityUp___44(): pass

def _dynamic_initializer_for__k3x4IdentityRight___44(): pass

def _dynamic_initializer_for__kInstancercGcUniqueIdComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcCustomiserCharacterComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___48(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___47(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___47(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___44(): pass

def _dynamic_initializer_for__kInstancercGcCreatureFeederComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcCreatureHarvesterComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___0(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___8(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourDetailAnims__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourLook__(): pass

def _dynamic_initializer_for__kNoIcon___46(): pass

def _dynamic_initializer_for__k3x4IdentityPos___41(): pass

def _dynamic_initializer_for__k3x4IdentityAt___45(): pass

def _dynamic_initializer_for__k3x4IdentityUp___45(): pass

def _dynamic_initializer_for__k3x4IdentityRight___45(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___45(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___9(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourGetTarget__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourRegisterAttacker__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourMoveToTarget__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourMaintainRangeFromTarget__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourFaceTarget__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourPlayAnim__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourApplyDamage__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourLaunchProjectile__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourWait__(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourIdle__(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcAntagonistComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___17(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourLook___0(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___1(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___40(): pass

def _dynamic_initializer_for__kInstancercTkCreatureTailComponent__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___42(): pass

def _dynamic_initializer_for__k3x4IdentityAt___46(): pass

def _dynamic_initializer_for__k3x4IdentityUp___46(): pass

def _dynamic_initializer_for__k3x4IdentityRight___46(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___46(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___13(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___10(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourLook___1(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourDetailAnims___0(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___49(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___48(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___48(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___41(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___41(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___41(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___41(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___38(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___38(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourGetTarget___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourRegisterAttacker___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourMoveToTarget___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourMaintainRangeFromTarget___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourFaceTarget___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourPlayAnim___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourApplyDamage___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourLaunchProjectile___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourWait___0(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourIdle___0(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___12(): pass

def _dynamic_initializer_for__kNoIcon___47(): pass

def _dynamic_initializer_for__k3x4IdentityPos___43(): pass

def _dynamic_initializer_for__k3x4IdentityAt___47(): pass

def _dynamic_initializer_for__k3x4IdentityUp___47(): pass

def _dynamic_initializer_for__k3x4IdentityRight___47(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___14(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___11(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourLook___2(): pass

def _dynamic_initializer_for__kInstancercGcBehaviourDetailAnims___1(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___18(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___5(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___47(): pass

def _dynamic_initializer_for__kInstancercGcBuildingComponent___0(): pass

def _dynamic_initializer_for__gRegisteredInNguiCreatureIkComponents__(): pass

def _dynamic_initializer_for__kInstancercGcCreatureFullBodyIKComponent__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcAntagonistComponent___2(): pass

def _dynamic_initializer_for__k3x4IdentityPos___44(): pass

def _dynamic_initializer_for__k3x4IdentityAt___48(): pass

def _dynamic_initializer_for__k3x4IdentityUp___48(): pass

def _dynamic_initializer_for__k3x4IdentityRight___48(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___48(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___4(): pass

def _dynamic_initializer_for__gScaryThingsList__(): pass

def _dynamic_initializer_for__cGcCreaturePerception__mvLastFeedPos__(): pass

def _dynamic_initializer_for__cGcCreatureRoutine__mRoutineMap__(): pass

def _dynamic_initializer_for__gRoutineGenerateQueue__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___50(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___49(): pass

def _dynamic_initializer_for__kInstancercGcCreatureAttractorComponent__(): pass

def _dynamic_initializer_for__kInstancercGcCreatureBaitComponent__(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___49(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___42(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___42(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___42(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___42(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___39(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___39(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___12(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___10(): pass

def _dynamic_initializer_for__kInstancercTkCreatureTailComponent___0(): pass

def _dynamic_initializer_for__cGcCreatureComponent__mGrassPushers__(): pass

def _dynamic_initializer_for__cGcCreatureComponent__mPersistentGrassPushers__(): pass

def _dynamic_initializer_for__cGcCreatureComponent__maAnimationCullVectors__(): pass

def _dynamic_initializer_for__kInstancercGcCreatureComponent__(): pass

def _dynamic_initializer_for__kNoIcon___48(): pass

def _dynamic_initializer_for__k3x4IdentityPos___45(): pass

def _dynamic_initializer_for__k3x4IdentityAt___49(): pass

def _dynamic_initializer_for__k3x4IdentityUp___49(): pass

def _dynamic_initializer_for__k3x4IdentityRight___49(): pass

def _dynamic_initializer_for__kInstancercGcCreatureEffectComponent__(): pass

def _dynamic_initializer_for__kInstancercGcCreatureEggComponent__(): pass

def _dynamic_initializer_for__kInstancercGcCreatureLegIKComponent__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___49(): pass

def _dynamic_initializer_for__kInstancercGcCreatureSpawnComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___8(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___43(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___51(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___50(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___50(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___43(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___43(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___43(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___40(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___40(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___19(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___13(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcBaseMiniPortalComponent___4(): pass

def _dynamic_initializer_for__kNoIcon___49(): pass

def _dynamic_initializer_for__k3x4IdentityPos___46(): pass

def _dynamic_initializer_for__k3x4IdentityAt___50(): pass

def _dynamic_initializer_for__k3x4IdentityUp___50(): pass

def _dynamic_initializer_for__k3x4IdentityRight___50(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___44(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___50(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___44(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___52(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___51(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___51(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___44(): pass

def _dynamic_initializer_for__gaNodesCache__(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___15(): pass

def _dynamic_initializer_for__k3x4IdentityPos___47(): pass

def _dynamic_initializer_for__k3x4IdentityAt___51(): pass

def _dynamic_initializer_for__k3x4IdentityUp___51(): pass

def _dynamic_initializer_for__k3x4IdentityRight___51(): pass

def _dynamic_initializer_for__kNoIcon___50(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___51(): pass

def _dynamic_initializer_for__cGcTerrainTextureData__mCache__(): pass

def _dynamic_initializer_for__cGcTerrainTextureData__mCameraMatrix__(): pass

def _dynamic_initializer_for__g_TerrainDebugDiffuseTex__(): pass

def _dynamic_initializer_for__g_TerrainDebugNormalTex__(): pass

def _dynamic_initializer_for__g_TerrainDebugStatusTex__(): pass

def _dynamic_initializer_for__gNearFeedbackTexRes__(): pass

def _dynamic_initializer_for__gNearFeedbackTexData__(): pass

def _dynamic_initializer_for__cGcTerrainTextureData__mValueNoiseHandle__(): pass

def _dynamic_initializer_for__cGcTerrainTextureData__mPerlinNoiseHandle__(): pass

def _dynamic_initializer_for__cGcTerrainTextureData__mValueNoiseTextureName__(): pass

def _dynamic_initializer_for__cGcTerrainTextureData__mValueNoiseVRTextureName__(): pass

def _dynamic_initializer_for__cGcTerrainTextureData__mPerlineNoiseTextureName__(): pass

def _dynamic_initializer_for__cGcVoxelGeneratorTerrain__maVoxelShapes__(): pass

def _dynamic_initializer_for__kNoIcon___51(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___14(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcWaypointComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcWaypointComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___53(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___52(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___52(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___44(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___45(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___52(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___45(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___45(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___41(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___41(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcHeightAdjustComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcBuildingComponent___1(): pass

def _dynamic_initializer_for__kInstancercTkInstanceWindComponent___0(): pass

def _dynamic_initializer_for__cGcRegionDecoratorObjects__mgObjectCounts__(): pass

def _dynamic_initializer_for__gaDesiredMaximumObjectsPerLod__(): pass

def _dynamic_initializer_for__gaDesiredMaximumPhysicalObjectsPerLod__(): pass

def _dynamic_initializer_for__kNoIcon___52(): pass

def _dynamic_initializer_for__k3x4IdentityPos___48(): pass

def _dynamic_initializer_for__k3x4IdentityAt___52(): pass

def _dynamic_initializer_for__k3x4IdentityUp___52(): pass

def _dynamic_initializer_for__k3x4IdentityRight___52(): pass

def _dynamic_initializer_for__kInstancercGcAtmosphereEntryComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___54(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___53(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___53(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___45(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___46(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___53(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___46(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___46(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___42(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___42(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcMarkerComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___15(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___7(): pass

def _dynamic_initializer_for__cGcScanEvent__sScanEventResources__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___49(): pass

def _dynamic_initializer_for__kNoIcon___53(): pass

def _dynamic_initializer_for__k3x4IdentityAt___53(): pass

def _dynamic_initializer_for__k3x4IdentityUp___53(): pass

def _dynamic_initializer_for__k3x4IdentityRight___53(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___16(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___55(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___54(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___54(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___46(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___47(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___54(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___47(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___47(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___43(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___43(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___18(): pass

def _dynamic_initializer_for__cGcScanEventManager__saPreviousSPFullEventStack__(): pass

def _dynamic_initializer_for__cGcScanEventManager__saPreviousMPFullEventStack__(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___20(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___7(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___7(): pass

def _dynamic_initializer_for__kNoIcon___54(): pass

def _dynamic_initializer_for__k3x4IdentityPos___50(): pass

def _dynamic_initializer_for__k3x4IdentityAt___54(): pass

def _dynamic_initializer_for__k3x4IdentityUp___54(): pass

def _dynamic_initializer_for__k3x4IdentityRight___54(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___55(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___56(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___12(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___13(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___11(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___47(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___55(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___55(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___48(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___48(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___48(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___44(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___44(): pass

def _dynamic_initializer_for__kNoIcon___55(): pass

def _dynamic_initializer_for__k3x4IdentityPos___51(): pass

def _dynamic_initializer_for__k3x4IdentityAt___55(): pass

def _dynamic_initializer_for__k3x4IdentityUp___55(): pass

def _dynamic_initializer_for__k3x4IdentityRight___55(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___57(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___56(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___56(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___56(): pass

def _dynamic_initializer_for__gPreloadedResources__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___52(): pass

def _dynamic_initializer_for__k3x4IdentityAt___56(): pass

def _dynamic_initializer_for__k3x4IdentityUp___56(): pass

def _dynamic_initializer_for__k3x4IdentityRight___56(): pass

def _dynamic_initializer_for__kNoIcon___56(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___58(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___57(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___57(): pass

def _dynamic_initializer_for__gStarIconResource__(): pass

def _dynamic_initializer_for__lcvUnitCubeMidPoint__(): pass

def _dynamic_initializer_for__lcvLocalSpaceMin__(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___17(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcCreatureSpawnComponent___2(): pass

def _dynamic_initializer_for__kInstancercTkLODComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercTkLODComponent___1(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___57(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___21(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___8(): pass

def _dynamic_initializer_for__kNoIcon___57(): pass

def _dynamic_initializer_for__k3x4IdentityPos___53(): pass

def _dynamic_initializer_for__k3x4IdentityAt___57(): pass

def _dynamic_initializer_for__k3x4IdentityUp___57(): pass

def _dynamic_initializer_for__k3x4IdentityRight___57(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___58(): pass

def _dynamic_initializer_for__cGcNavMapBase__sapInstances__(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___49(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___49(): pass

def _dynamic_initializer_for__gLayerColours__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___54(): pass

def _dynamic_initializer_for__k3x4IdentityAt___58(): pass

def _dynamic_initializer_for__k3x4IdentityUp___58(): pass

def _dynamic_initializer_for__k3x4IdentityRight___58(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___59(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcPlayerControlComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___59(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___58(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___58(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___48(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___50(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___50(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___49(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___45(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___45(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___22(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___9(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___14(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___12(): pass

def _dynamic_initializer_for__kNoIcon___58(): pass

def _dynamic_initializer_for__k3x4IdentityPos___55(): pass

def _dynamic_initializer_for__k3x4IdentityAt___59(): pass

def _dynamic_initializer_for__k3x4IdentityUp___59(): pass

def _dynamic_initializer_for__k3x4IdentityRight___59(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___60(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___59(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___59(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___23(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___8(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___8(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___60(): pass

def _dynamic_initializer_for__kInstancercGcChairComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___51(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___51(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___50(): pass

def _dynamic_initializer_for__kInstancercGcCharacterInterfaceComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___18(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___18(): pass

def _dynamic_initializer_for__kInstancercGcLadderComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcRepairTechComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcPlayerControlComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___21(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___49(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___46(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___46(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcWaypointComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercGcWaypointComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___10(): pass

def _dynamic_initializer_for__kNoIcon___59(): pass

def _dynamic_initializer_for__k3x4IdentityPos___56(): pass

def _dynamic_initializer_for__k3x4IdentityAt___60(): pass

def _dynamic_initializer_for__k3x4IdentityUp___60(): pass

def _dynamic_initializer_for__k3x4IdentityRight___60(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___11(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___50(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___19(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___9(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___7(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___61(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___61(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___7(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___52(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___52(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___60(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___60(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___51(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___24(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___47(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___47(): pass

def _dynamic_initializer_for__kInstancercGcSentinelCoverComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcBuildingComponent___2(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___0(): pass

def _dynamic_initializer_for__kNoIcon___60(): pass

def _dynamic_initializer_for__k3x4IdentityPos___57(): pass

def _dynamic_initializer_for__k3x4IdentityAt___61(): pass

def _dynamic_initializer_for__k3x4IdentityUp___61(): pass

def _dynamic_initializer_for__k3x4IdentityRight___61(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___62(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___62(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcTurretComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___61(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___61(): pass

def _dynamic_initializer_for__kInstancercGcRefinerUnitComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcCreatureHarvesterComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcCreatureFeederComponent___5(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___51(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___53(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___53(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___52(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___48(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___48(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcGrabPlayerComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcGroundWormComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___8(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcWaypointComponent___4(): pass

def _dynamic_initializer_for__kLoadInstancercGcWaypointComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcWiringSocketComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___25(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___25(): pass

def _dynamic_initializer_for__kaColourArray__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___20(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___20(): pass

def _dynamic_initializer_for__kInstancercTkLODComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercTkLODComponent___2(): pass

def _dynamic_initializer_for__k3x4IdentityPos___58(): pass

def _dynamic_initializer_for__k3x4IdentityAt___62(): pass

def _dynamic_initializer_for__k3x4IdentityUp___62(): pass

def _dynamic_initializer_for__k3x4IdentityRight___62(): pass

def _dynamic_initializer_for__kNoIcon___61(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___23(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___52(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___63(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___62(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___62(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___54(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___63(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___54(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___53(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___49(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___49(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___26(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___10(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcRocketLockerComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___16(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___21(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___21(): pass

def _dynamic_initializer_for__MenuAction__InvalidAction__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___59(): pass

def _dynamic_initializer_for__k3x4IdentityAt___63(): pass

def _dynamic_initializer_for__k3x4IdentityUp___63(): pass

def _dynamic_initializer_for__k3x4IdentityRight___63(): pass

def _dynamic_initializer_for__kNoIcon___62(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___64(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___63(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___63(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___11(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___27(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___21(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___8(): pass

def _dynamic_initializer_for__gaDestroyedModels__(): pass

def _dynamic_initializer_for__gcGcDestructableComponent_Destroyed__(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___8(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___64(): pass

def _dynamic_initializer_for__kInstancercGcPlayerControlComponent___4(): pass

def _dynamic_initializer_for__cGcProjectileImpact__mkDefaultData__(): pass

def _dynamic_initializer_for__gaBeamLightColourArray__(): pass

def _dynamic_initializer_for__kInstancercGcMissileComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___24(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___53(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___55(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___55(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___54(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___50(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___50(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcSentinelCoverComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcSpaceshipShieldComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___9(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcAntagonistComponent___3(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___15(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___13(): pass

def _dynamic_initializer_for__gcGcShootableComponent_Shot__(): pass

def _dynamic_initializer_for__gcGcShootableComponent_ArmourDestroyed__(): pass

def _dynamic_initializer_for__kInstancercGcScanEffectComponent___1(): pass

def _dynamic_initializer_for__k3x4IdentityPos___60(): pass

def _dynamic_initializer_for__k3x4IdentityAt___64(): pass

def _dynamic_initializer_for__k3x4IdentityUp___64(): pass

def _dynamic_initializer_for__k3x4IdentityRight___64(): pass

def _dynamic_initializer_for__kNoIcon___63(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcMissileComponent___2(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___65(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcAntagonistComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcSentinelCoverComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___28(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___28(): pass

def _dynamic_initializer_for__gaRepairTargets__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___65(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___64(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___64(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___54(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___56(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___56(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___55(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___51(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___51(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcDissolveEffectComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcDroneComponent__(): pass

def _dynamic_initializer_for__kNoIcon___64(): pass

def _dynamic_initializer_for__k3x4IdentityPos___61(): pass

def _dynamic_initializer_for__k3x4IdentityAt___65(): pass

def _dynamic_initializer_for__k3x4IdentityUp___65(): pass

def _dynamic_initializer_for__k3x4IdentityRight___65(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___25(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___66(): pass

def _dynamic_initializer_for__kInstancercGcEncounterComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcSentinelRobotComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___29(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___29(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcBuildingComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcAntagonistComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcTechnologyAttachmentComponent___3(): pass

def _dynamic_initializer_for__kInstancercTkInstanceWindComponent___1(): pass

def _dynamic_initializer_for__cGcFadeNodeInstance__gsaImpostorRenderData__(): pass

def _dynamic_initializer_for__gaTestMaterialsRendered__(): pass

def _dynamic_initializer_for__gaImposterGenerationStateData__(): pass

def _dynamic_initializer_for__cGcFadeNodeInstance__mCache__(): pass

def _dynamic_initializer_for__gThreadSafeNodeDeletor__(): pass

def _dynamic_initializer_for__kInstancercGcCreatureSpawnComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___22(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___22(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___0(): pass

def _dynamic_initializer_for__kNoIcon___65(): pass

def _dynamic_initializer_for__k3x4IdentityPos___62(): pass

def _dynamic_initializer_for__k3x4IdentityAt___66(): pass

def _dynamic_initializer_for__k3x4IdentityUp___66(): pass

def _dynamic_initializer_for__k3x4IdentityRight___66(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___26(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___16(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___14(): pass

def _dynamic_initializer_for__gOnSpawnedType__(): pass

def _dynamic_initializer_for__gOnVariableChangedType__(): pass

def _dynamic_initializer_for__gOnBroadcastReceivedType__(): pass

def _dynamic_initializer_for__gOnComponentEventType__(): pass

def _dynamic_initializer_for__gIfValueIsType__(): pass

def _dynamic_initializer_for__gValueAsBoolType__(): pass

def _dynamic_initializer_for__gFixedValueType__(): pass

def _dynamic_initializer_for__gGetVariableValueType__(): pass

def _dynamic_initializer_for__gSetVariableValueType__(): pass

def _dynamic_initializer_for__gSurveyVariableValueType__(): pass

def _dynamic_initializer_for__gSequenceType__(): pass

def _dynamic_initializer_for__gWaitSecondsType__(): pass

def _dynamic_initializer_for__gWaitFramesType__(): pass

def _dynamic_initializer_for__gRandomNumberType__(): pass

def _dynamic_initializer_for__gMathsFunctionType__(): pass

def _dynamic_initializer_for__gBroadcastValueModelType__(): pass

def _dynamic_initializer_for__gBroadcastValueDistanceType__(): pass

def _dynamic_initializer_for__gRaiseEventType__(): pass

def _dynamic_initializer_for__gSetCoreTypeList__(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___55(): pass

def _dynamic_initializer_for__gOnNodeActivatedType__(): pass

def _dynamic_initializer_for__gOnTriggerVolumeType__(): pass

def _dynamic_initializer_for__gOnAnimFrameType__(): pass

def _dynamic_initializer_for__gSetNodeActiveType__(): pass

def _dynamic_initializer_for__gDeleteModelType__(): pass

def _dynamic_initializer_for__gPlayAnimType__(): pass

def _dynamic_initializer_for__gIfAnimIsPlayingType__(): pass

def _dynamic_initializer_for__gStopAnimType__(): pass

def _dynamic_initializer_for__gPlaySoundType__(): pass

def _dynamic_initializer_for__gStopSoundsType__(): pass

def _dynamic_initializer_for__gFadeModelType__(): pass

def _dynamic_initializer_for__gTranslateNodeType__(): pass

def _dynamic_initializer_for__gRotateNodeType__(): pass

def _dynamic_initializer_for__gScaleNodeType__(): pass

def _dynamic_initializer_for__gSetLightIntensityType__(): pass

def _dynamic_initializer_for__gSetLightColourType__(): pass

def _dynamic_initializer_for__gSetGeneralTypeList__(): pass

def _dynamic_initializer_for__gOnPlayerNearType__(): pass

def _dynamic_initializer_for__gSpawnParticlesType__(): pass

def _dynamic_initializer_for__gSetParticlesStageType__(): pass

def _dynamic_initializer_for__gStopParticlesType__(): pass

def _dynamic_initializer_for__gDestroyDestructablesType__(): pass

def _dynamic_initializer_for__gSetHazardType__(): pass

def _dynamic_initializer_for__gDamagePlayerType__(): pass

def _dynamic_initializer_for__gCameraShakeType__(): pass

def _dynamic_initializer_for__gStopCameraShakeType__(): pass

def _dynamic_initializer_for__gFadeMaterialsType__(): pass

def _dynamic_initializer_for__gSetMaterialUniformType__(): pass

def _dynamic_initializer_for__gSetShaderValueType__(): pass

def _dynamic_initializer_for__gGiveRewardType__(): pass

def _dynamic_initializer_for__gAddShootableHealthType__(): pass

def _dynamic_initializer_for__gGetPlanetColourType__(): pass

def _dynamic_initializer_for__gGetTimeOfDayType__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___66(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___67(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___57(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___57(): pass

def _dynamic_initializer_for__cGcBuildingPlacement__mLayoutsCache__(): pass

def _dynamic_initializer_for__cGcClusterLayoutPlacement__mLayoutsCache__(): pass

def _dynamic_initializer_for__cGcClusterLayoutPlacement__mLayoutsCacheLimitRadius__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___63(): pass

def _dynamic_initializer_for__k3x4IdentityAt___67(): pass

def _dynamic_initializer_for__k3x4IdentityUp___67(): pass

def _dynamic_initializer_for__k3x4IdentityRight___67(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___58(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___68(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___58(): pass

def _dynamic_initializer_for__cGcModuleSet__mModuleSets__(): pass

def _dynamic_initializer_for__cGcModuleSet__mConnectors__(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___67(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___65(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___65(): pass

def _dynamic_initializer_for__kEmptyList__(): pass

def _dynamic_initializer_for__cGcModulePrototypeElement__mBlockFrameResHandle__(): pass

def _dynamic_initializer_for__cGcModulePrototypeElement__mWalkableIndicatorResHandle__(): pass

def _dynamic_initializer_for__cGcWFCScene__mCache__(): pass

def _dynamic_initializer_for__cGcWFCScene__mParseQueue__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___64(): pass

def _dynamic_initializer_for__k3x4IdentityAt___68(): pass

def _dynamic_initializer_for__k3x4IdentityUp___68(): pass

def _dynamic_initializer_for__k3x4IdentityRight___68(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___69(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___17(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___23(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcSimpleInteractionComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___7(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___68(): pass

def _dynamic_initializer_for__cGcSolarSystem__gPreloadedResources__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___65(): pass

def _dynamic_initializer_for__k3x4IdentityAt___69(): pass

def _dynamic_initializer_for__k3x4IdentityUp___69(): pass

def _dynamic_initializer_for__k3x4IdentityRight___69(): pass

def _dynamic_initializer_for__kNoIcon___66(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___59(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___70(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___59(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___69(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___66(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___66(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___56(): pass

def _dynamic_initializer_for__kInstancercGcChairComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___22(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___10(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___24(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___18(): pass

def _dynamic_initializer_for__cGcPlanet__gPreloadedResources__(): pass

def _dynamic_initializer_for__cGcPlanet__sUniformBlends__(): pass

def _dynamic_initializer_for__kGammaConst1__(): pass

def _dynamic_initializer_for__kGammaConst2__(): pass

def _dynamic_initializer_for__kGammaConst3__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___66(): pass

def _dynamic_initializer_for__k3x4IdentityAt___70(): pass

def _dynamic_initializer_for__k3x4IdentityUp___70(): pass

def _dynamic_initializer_for__k3x4IdentityRight___70(): pass

def _dynamic_initializer_for__kNoIcon___67(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___70(): pass

def _dynamic_initializer_for__cGcPlanetGenerator__mFilenameHashMap__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___71(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___71(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___67(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___67(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___25(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcMissileComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcBuildingComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___10(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___56(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___60(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___60(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___57(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___52(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___52(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___9(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcTurretComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcMarkerComponent___2(): pass

def _dynamic_initializer_for__kNoIcon___68(): pass

def _dynamic_initializer_for__k3x4IdentityPos___67(): pass

def _dynamic_initializer_for__k3x4IdentityAt___71(): pass

def _dynamic_initializer_for__k3x4IdentityUp___71(): pass

def _dynamic_initializer_for__k3x4IdentityRight___71(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___72(): pass

def _dynamic_initializer_for__kInstancercGcCombatEffectsComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___26(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___26(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___61(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___61(): pass

def _dynamic_initializer_for__kInstancercGcBuildingComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcPlayerComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcSpaceshipShieldComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcShipFlareComponent__(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___57(): pass

def _dynamic_initializer_for__kInstancercGcDistanceScaleComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcFreighterBaseComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___23(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___72(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___68(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___68(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___58(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___53(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___53(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___10(): pass

def _dynamic_initializer_for__kInstancercGcFreighterSyncComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___11(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___11(): pass

def _dynamic_initializer_for__kInstancercTkLODComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercTkLODComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___19(): pass

def _dynamic_initializer_for__kInstancercGcTurretComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcLootComponent___9(): pass

def _dynamic_initializer_for__kInstancercTkGravityComponent__(): pass

def _dynamic_initializer_for__kNoIcon___69(): pass

def _dynamic_initializer_for__k3x4IdentityPos___68(): pass

def _dynamic_initializer_for__k3x4IdentityAt___72(): pass

def _dynamic_initializer_for__k3x4IdentityUp___72(): pass

def _dynamic_initializer_for__k3x4IdentityRight___72(): pass

def _dynamic_initializer_for__kInstancercGcFreighterSyncComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcTurretComponent___5(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___73(): pass

def _dynamic_initializer_for__kInstancercGcEngineComponent__(): pass

def _dynamic_initializer_for__kInstancercGcShipFlareComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcShipOwnershipComponent___12(): pass

def _dynamic_initializer_for__kLoadInstancercGcShipOwnershipComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___73(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___69(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___69(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___58(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___62(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___62(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___59(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___54(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___54(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___27(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___29(): pass

def _dynamic_initializer_for__kInstancercGcLandingHelperComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___12(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___12(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___15(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___26(): pass

def _dynamic_initializer_for__gShipCollisionQueue__(): pass

def _dynamic_initializer_for__kNoIcon___70(): pass

def _dynamic_initializer_for__k3x4IdentityPos___69(): pass

def _dynamic_initializer_for__k3x4IdentityAt___73(): pass

def _dynamic_initializer_for__k3x4IdentityUp___73(): pass

def _dynamic_initializer_for__k3x4IdentityRight___73(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___24(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___74(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___70(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___70(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___59(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___63(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___74(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___63(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___60(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___55(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___55(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___30(): pass

def _dynamic_initializer_for__kInstancercGcSpaceshipShieldComponent___2(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___17(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___15(): pass

def _dynamic_initializer_for__gcGcSpaceshipShieldComponent_EnergyShieldDepleted__(): pass

def _dynamic_initializer_for__gcGcSpaceshipShieldComponent_EnergyShieldStartRecharge__(): pass

def _dynamic_initializer_for__kInstancercGcNPCPlacementComponent___11(): pass

def _dynamic_initializer_for__kInstancercGcObjectPlacementComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___28(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcAbandonedFreighterComponent___20(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___27(): pass

def _dynamic_initializer_for__kInstancercGcLandingHelperComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcTurretComponent___6(): pass

def _dynamic_initializer_for__kInstancercGcMissileComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___30(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___30(): pass

def _dynamic_initializer_for__k3x4IdentityPos___70(): pass

def _dynamic_initializer_for__k3x4IdentityAt___74(): pass

def _dynamic_initializer_for__k3x4IdentityUp___74(): pass

def _dynamic_initializer_for__k3x4IdentityRight___74(): pass

def _dynamic_initializer_for__kNoIcon___71(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___75(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___60(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___13(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___13(): pass

def _dynamic_initializer_for__kInstancercGcTechnologyAttachmentComponent___4(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___75(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___71(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___71(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___25(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___31(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___31(): pass

def _dynamic_initializer_for__kInstancercGcVehicleRaceInviteComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___64(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___64(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___61(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___56(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___56(): pass

def _dynamic_initializer_for__kInstancercGcOutpostComponent___29(): pass

def _dynamic_initializer_for__kLoadInstancercGcOutpostComponent___29(): pass

def _dynamic_initializer_for__kInstancercGcShootableComponent___31(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___28(): pass

def _dynamic_initializer_for__kInstancercGcNetworkInterpolationComponent___16(): pass

def _dynamic_initializer_for__kNoIcon___72(): pass

def _dynamic_initializer_for__k3x4IdentityPos___71(): pass

def _dynamic_initializer_for__k3x4IdentityAt___75(): pass

def _dynamic_initializer_for__k3x4IdentityUp___75(): pass

def _dynamic_initializer_for__k3x4IdentityRight___75(): pass

def _dynamic_initializer_for__kInstancercGcVehicleGarageComponent___2(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___76(): pass

def _dynamic_initializer_for__kInstancercGcVehicleComponent___32(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleComponent___32(): pass

def _dynamic_initializer_for__kInstancercGcVehicleRaceInviteComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___76(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___72(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___72(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___61(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___65(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___65(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___62(): pass

def _dynamic_initializer_for__kInstancercGcInteractionComponent___57(): pass

def _dynamic_initializer_for__kLoadInstancercGcInteractionComponent___57(): pass

def _dynamic_initializer_for__kInstancercGcVehicleCheckpointComponent___14(): pass

def _dynamic_initializer_for__kLoadInstancercGcVehicleCheckpointComponent___14(): pass

def _dynamic_initializer_for__kInstancercGcWaypointComponent___5(): pass

def _dynamic_initializer_for__kLoadInstancercGcWaypointComponent___5(): pass

def _dynamic_initializer_for__kNoIcons___0(): pass

def _dynamic_initializer_for__k3x4IdentityPos___72(): pass

def _dynamic_initializer_for__kNoIcon___73(): pass

def _dynamic_initializer_for__k3x4IdentityAt___76(): pass

def _dynamic_initializer_for__k3x4IdentityUp___76(): pass

def _dynamic_initializer_for__k3x4IdentityRight___76(): pass

def _dynamic_initializer_for__kInstancercGcPlayerCharacterComponent___29(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___77(): pass

def _dynamic_initializer_for__kInstancercTkRotationComponent__(): pass

def _dynamic_initializer_for__kInstancercTkReferenceComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercTkReferenceComponent__(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___62(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___77(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___73(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___73(): pass

def _dynamic_initializer_for__kSimUpdateColour__(): pass

def _dynamic_initializer_for__k3x4IdentityPos___73(): pass

def _dynamic_initializer_for__k3x4IdentityAt___77(): pass

def _dynamic_initializer_for__k3x4IdentityUp___77(): pass

def _dynamic_initializer_for__k3x4IdentityRight___77(): pass

def _dynamic_initializer_for__kNonCollidingPlane___0(): pass

def _dynamic_initializer_for__k4x4IdentityRight___0(): pass

def _dynamic_initializer_for__k4x4IdentityUp___0(): pass

def _dynamic_initializer_for__k4x4IdentityAt___0(): pass

def _dynamic_initializer_for__k4x4IdentityPos___0(): pass

def _dynamic_initializer_for__k3x4IdentityRight___78(): pass

def _dynamic_initializer_for__k3x4IdentityUp___78(): pass

def _dynamic_initializer_for__k3x4IdentityAt___78(): pass

def _dynamic_initializer_for__k3x4IdentityPos___74(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___1(): pass

def _dynamic_initializer_for__mCache___0(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___1(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___0(): pass

def _dynamic_initializer_for__gvTaaSettings__(): pass

def _dynamic_initializer_for__gvTessSettings__(): pass

def _dynamic_initializer_for__gvShellsSettings__(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___2(): pass

def _dynamic_initializer_for__k4x4IdentityRight___1(): pass

def _dynamic_initializer_for__k4x4IdentityUp___1(): pass

def _dynamic_initializer_for__k4x4IdentityAt___1(): pass

def _dynamic_initializer_for__k4x4IdentityPos___1(): pass

def _dynamic_initializer_for__k3x4IdentityRight___79(): pass

def _dynamic_initializer_for__k3x4IdentityUp___79(): pass

def _dynamic_initializer_for__k3x4IdentityAt___79(): pass

def _dynamic_initializer_for__k3x4IdentityPos___75(): pass

def _dynamic_initializer_for__mCache___1(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___2(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___1(): pass

def _dynamic_initializer_for__kNonCollidingPlane___1(): pass

def _dynamic_initializer_for__gModelsLodCameraPos__(): pass

def _dynamic_initializer_for__cEgPipelineResource__gCrossPipeSharedRenderTargets__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___78(): pass

def _dynamic_initializer_for__kNonCollidingPlane___2(): pass

def _dynamic_initializer_for__k4x4IdentityRight___2(): pass

def _dynamic_initializer_for__k4x4IdentityUp___2(): pass

def _dynamic_initializer_for__k4x4IdentityAt___2(): pass

def _dynamic_initializer_for__k4x4IdentityPos___2(): pass

def _dynamic_initializer_for__k3x4IdentityRight___80(): pass

def _dynamic_initializer_for__k3x4IdentityUp___80(): pass

def _dynamic_initializer_for__k3x4IdentityAt___80(): pass

def _dynamic_initializer_for__k3x4IdentityPos___76(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___3(): pass

def _dynamic_initializer_for__mCache___2(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___3(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___2(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsVertPreamble__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsHullPreamble__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsDomnPreamble__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsGeomPreamble__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsFragPreamble__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsCompPreamble__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsTempCode0__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsTempCode1__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsTempCode2__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsTempCode3__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsTempCode4__(): pass

def _dynamic_initializer_for__cEgShaderResource__mgsTempCode5__(): pass

def _dynamic_initializer_for__gShaderPreloadList__(): pass

def _dynamic_initializer_for__gTextureHeaderCache__(): pass

def _dynamic_initializer_for__cEgTextureResource__maStreamingHandles__(): pass

def _dynamic_initializer_for__kNonCollidingPlane___3(): pass

def _dynamic_initializer_for__k4x4IdentityRight___3(): pass

def _dynamic_initializer_for__k4x4IdentityUp___3(): pass

def _dynamic_initializer_for__k4x4IdentityAt___3(): pass

def _dynamic_initializer_for__k4x4IdentityPos___3(): pass

def _dynamic_initializer_for__k3x4IdentityRight___81(): pass

def _dynamic_initializer_for__k3x4IdentityUp___81(): pass

def _dynamic_initializer_for__k3x4IdentityAt___81(): pass

def _dynamic_initializer_for__k3x4IdentityPos___77(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___4(): pass

def _dynamic_initializer_for__mCache___3(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___4(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___3(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___79(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___5(): pass

def _dynamic_initializer_for__k4x4IdentityRight___4(): pass

def _dynamic_initializer_for__k4x4IdentityUp___4(): pass

def _dynamic_initializer_for__k4x4IdentityAt___4(): pass

def _dynamic_initializer_for__k4x4IdentityPos___4(): pass

def _dynamic_initializer_for__k3x4IdentityRight___82(): pass

def _dynamic_initializer_for__k3x4IdentityUp___82(): pass

def _dynamic_initializer_for__k3x4IdentityAt___82(): pass

def _dynamic_initializer_for__k3x4IdentityPos___78(): pass

def _dynamic_initializer_for__mCache___4(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___5(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___4(): pass

def _dynamic_initializer_for__kNonCollidingPlane___4(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___80(): pass

def _dynamic_initializer_for__lCachePath__(): pass

def _dynamic_initializer_for__gRenderMut__(): pass

def _dynamic_initializer_for__kNonCollidingPlane___5(): pass

def _dynamic_initializer_for__k4x4IdentityRight___5(): pass

def _dynamic_initializer_for__k4x4IdentityUp___5(): pass

def _dynamic_initializer_for__k4x4IdentityAt___5(): pass

def _dynamic_initializer_for__k4x4IdentityPos___5(): pass

def _dynamic_initializer_for__k3x4IdentityRight___83(): pass

def _dynamic_initializer_for__k3x4IdentityUp___83(): pass

def _dynamic_initializer_for__k3x4IdentityAt___83(): pass

def _dynamic_initializer_for__k3x4IdentityPos___79(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___6(): pass

def _dynamic_initializer_for__mCache___5(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___6(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___5(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___7(): pass

def _dynamic_initializer_for__k4x4IdentityRight___6(): pass

def _dynamic_initializer_for__k4x4IdentityUp___6(): pass

def _dynamic_initializer_for__k4x4IdentityAt___6(): pass

def _dynamic_initializer_for__k4x4IdentityPos___6(): pass

def _dynamic_initializer_for__k3x4IdentityRight___84(): pass

def _dynamic_initializer_for__k3x4IdentityUp___84(): pass

def _dynamic_initializer_for__k3x4IdentityAt___84(): pass

def _dynamic_initializer_for__k3x4IdentityPos___80(): pass

def _dynamic_initializer_for__mCache___6(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___7(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___6(): pass

def _dynamic_initializer_for__kNonCollidingPlane___6(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___8(): pass

def _dynamic_initializer_for__k4x4IdentityRight___7(): pass

def _dynamic_initializer_for__k4x4IdentityUp___7(): pass

def _dynamic_initializer_for__k4x4IdentityAt___7(): pass

def _dynamic_initializer_for__k4x4IdentityPos___7(): pass

def _dynamic_initializer_for__k3x4IdentityRight___85(): pass

def _dynamic_initializer_for__k3x4IdentityUp___85(): pass

def _dynamic_initializer_for__k3x4IdentityAt___85(): pass

def _dynamic_initializer_for__k3x4IdentityPos___81(): pass

def _dynamic_initializer_for__mCache___7(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___8(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___7(): pass

def _dynamic_initializer_for__XMLNode__emptyXMLNode__(): pass

def _dynamic_initializer_for__kNonCollidingPlane___7(): pass

def _dynamic_initializer_for__k4x4IdentityRight___8(): pass

def _dynamic_initializer_for__k4x4IdentityUp___8(): pass

def _dynamic_initializer_for__k4x4IdentityAt___8(): pass

def _dynamic_initializer_for__k4x4IdentityPos___8(): pass

def _dynamic_initializer_for__k3x4IdentityRight___86(): pass

def _dynamic_initializer_for__k3x4IdentityUp___86(): pass

def _dynamic_initializer_for__k3x4IdentityAt___86(): pass

def _dynamic_initializer_for__k3x4IdentityPos___82(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___9(): pass

def _dynamic_initializer_for__mCache___8(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___9(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___8(): pass

def _dynamic_initializer_for__TkExample__cEgCubeNode__mPositionVertexLayout__(): pass

def _dynamic_initializer_for__TkExample__cEgCubeNode__mgapCubes__(): pass

def _dynamic_initializer_for__kNonCollidingPlane___8(): pass

def _dynamic_initializer_for__k4x4IdentityRight___9(): pass

def _dynamic_initializer_for__k4x4IdentityUp___9(): pass

def _dynamic_initializer_for__k4x4IdentityAt___9(): pass

def _dynamic_initializer_for__k4x4IdentityPos___9(): pass

def _dynamic_initializer_for__k3x4IdentityRight___87(): pass

def _dynamic_initializer_for__k3x4IdentityUp___87(): pass

def _dynamic_initializer_for__k3x4IdentityAt___87(): pass

def _dynamic_initializer_for__k3x4IdentityPos___83(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___10(): pass

def _dynamic_initializer_for__mCache___9(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___10(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___9(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___81(): pass

def TkGalactic___dynamic_initializer_for__mInstanceVertexLayout__(): pass

def TkGalactic___dynamic_initializer_for__mPositionVertexLayout__(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___11(): pass

def _dynamic_initializer_for__k4x4IdentityRight___10(): pass

def _dynamic_initializer_for__k4x4IdentityUp___10(): pass

def _dynamic_initializer_for__k4x4IdentityAt___10(): pass

def _dynamic_initializer_for__k4x4IdentityPos___10(): pass

def _dynamic_initializer_for__k3x4IdentityRight___88(): pass

def _dynamic_initializer_for__k3x4IdentityUp___88(): pass

def _dynamic_initializer_for__k3x4IdentityAt___88(): pass

def _dynamic_initializer_for__k3x4IdentityPos___84(): pass

def _dynamic_initializer_for__mCache___10(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___11(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___10(): pass

def _dynamic_initializer_for__kNonCollidingPlane___9(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___82(): pass

def _dynamic_initializer_for__kRight__(): pass

def _dynamic_initializer_for__kUp__(): pass

def _dynamic_initializer_for__kafCornerCoords__(): pass

def _dynamic_initializer_for__TkHeavyAir__TkHeavyAirNodeTemplate__mgapHeavyAirNodes__(): pass

def _dynamic_initializer_for__TkSpeedLines__cTkSpeedLinesNode__cTkSpeedLinesNodeTemplate__mgapSpeedLineNodes__(): pass

def _dynamic_initializer_for__TkSpeedLines__cTkSpeedLinesNode__mgVertexLayout__(): pass

def _dynamic_initializer_for__k4x4IdentityRight___11(): pass

def _dynamic_initializer_for__k4x4IdentityUp___11(): pass

def _dynamic_initializer_for__k4x4IdentityAt___11(): pass

def _dynamic_initializer_for__k4x4IdentityPos___11(): pass

def _dynamic_initializer_for__k3x4IdentityRight___89(): pass

def _dynamic_initializer_for__k3x4IdentityUp___89(): pass

def _dynamic_initializer_for__k3x4IdentityAt___89(): pass

def _dynamic_initializer_for__k3x4IdentityPos___85(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___12(): pass

def _dynamic_initializer_for__mCache___11(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___12(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___11(): pass

def _dynamic_initializer_for__kNonCollidingPlane___10(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___83(): pass

def _dynamic_initializer_for__gSurfaceWindData__(): pass

def _dynamic_initializer_for__gWaterWindData__(): pass

def _dynamic_initializer_for__EgInstancedModelExtension__cEgInstancedMeshNode__mgVertexLayout__(): pass

def _dynamic_initializer_for__EgInstancedModelExtension__cEgInstancedMeshNode__mMemoryManager__(): pass

def EgInstancedModelExtension___dynamic_initializer_for__gGrassPushers__(): pass

def _dynamic_initializer_for__Json__Value__null__(): pass

def _dynamic_initializer_for__Json__Value__nullRef__(): pass

def _dynamic_initializer_for__kNonCollidingPlane___11(): pass

def _dynamic_initializer_for__k4x4IdentityRight___12(): pass

def _dynamic_initializer_for__k4x4IdentityUp___12(): pass

def _dynamic_initializer_for__k4x4IdentityAt___12(): pass

def _dynamic_initializer_for__k4x4IdentityPos___12(): pass

def _dynamic_initializer_for__k3x4IdentityRight___90(): pass

def _dynamic_initializer_for__k3x4IdentityUp___90(): pass

def _dynamic_initializer_for__k3x4IdentityAt___90(): pass

def _dynamic_initializer_for__k3x4IdentityPos___86(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___13(): pass

def _dynamic_initializer_for__mCache___12(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___13(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___12(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___84(): pass

def _dynamic_initializer_for__TkLineRenderer__cEgLineRendererNode__mVertexLayout__(): pass

def _dynamic_initializer_for__k4x4IdentityRight___13(): pass

def _dynamic_initializer_for__k4x4IdentityUp___13(): pass

def _dynamic_initializer_for__k4x4IdentityAt___13(): pass

def _dynamic_initializer_for__k4x4IdentityPos___13(): pass

def _dynamic_initializer_for__k3x4IdentityRight___91(): pass

def _dynamic_initializer_for__k3x4IdentityUp___91(): pass

def _dynamic_initializer_for__k3x4IdentityAt___91(): pass

def _dynamic_initializer_for__k3x4IdentityPos___87(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___14(): pass

def _dynamic_initializer_for__mCache___13(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___14(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___85(): pass

def _dynamic_initializer_for__kNonCollidingPlane___12(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___13(): pass

def EgMagicModels___dynamic_initializer_for__gWishes__(): pass

def EgMagicModels___dynamic_initializer_for__gCosts__(): pass

def EgMagicModels___dynamic_initializer_for__gModels__(): pass

def EgMagicModels___dynamic_initializer_for__gVisibilities__(): pass

def EgMagicModels___dynamic_initializer_for__gNewModels__(): pass

def EgMagicModels___dynamic_initializer_for__gSpawnModels__(): pass

def EgMagicModels___dynamic_initializer_for__gFadeModels__(): pass

def EgMagicModels___dynamic_initializer_for__gCameraPos__(): pass

def EgMagicModels___dynamic_initializer_for__gPrevCameraPos__(): pass

def EgMagicModels___dynamic_initializer_for__gPredictedCameraPos__(): pass

def EgMagicModels___dynamic_initializer_for__gVisibilityMap__(): pass

def _dynamic_initializer_for__kNonCollidingPlane___13(): pass

def _dynamic_initializer_for__k4x4IdentityRight___14(): pass

def _dynamic_initializer_for__k4x4IdentityUp___14(): pass

def _dynamic_initializer_for__k4x4IdentityAt___14(): pass

def _dynamic_initializer_for__k4x4IdentityPos___14(): pass

def _dynamic_initializer_for__k3x4IdentityRight___92(): pass

def _dynamic_initializer_for__k3x4IdentityUp___92(): pass

def _dynamic_initializer_for__k3x4IdentityAt___92(): pass

def _dynamic_initializer_for__k3x4IdentityPos___88(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___15(): pass

def _dynamic_initializer_for__mCache___14(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___15(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___14(): pass

def _dynamic_initializer_for__kMaxVec__(): pass

def _dynamic_initializer_for__kMinVec__(): pass

def _dynamic_initializer_for__cEgTrailNode__mgVertexLayout__(): pass

def _dynamic_initializer_for__cEgTrailNode__mgapTrailNodes__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___86(): pass

def _dynamic_initializer_for__kRight___0(): pass

def _dynamic_initializer_for__kUp___0(): pass

def _dynamic_initializer_for__kafCornerCoords___0(): pass

def _dynamic_initializer_for__TkParticle__cTkEmitterNode__mgRNG__(): pass

def _dynamic_initializer_for__TkParticle__TkParticleNodeTpl__mgapEmitters__(): pass

def _dynamic_initializer_for__TkParticle__TkParticleNodeTpl__mgapAsyncUpdateEmitters__(): pass

def TkParticle___dynamic_initializer_for__kBBMin__(): pass

def TkParticle___dynamic_initializer_for__kBBMax__(): pass

def TkParticle___dynamic_initializer_for__kvEpsilon__(): pass

def _dynamic_initializer_for__k4x4IdentityRight___15(): pass

def _dynamic_initializer_for__k4x4IdentityUp___15(): pass

def _dynamic_initializer_for__k4x4IdentityAt___15(): pass

def _dynamic_initializer_for__k4x4IdentityPos___15(): pass

def _dynamic_initializer_for__k3x4IdentityRight___93(): pass

def _dynamic_initializer_for__k3x4IdentityUp___93(): pass

def _dynamic_initializer_for__k3x4IdentityAt___93(): pass

def _dynamic_initializer_for__k3x4IdentityPos___89(): pass

def GPU___dynamic_initializer_for__gDummyInitPerm__(): pass

def _dynamic_initializer_for__k4x4IdentityRight___16(): pass

def _dynamic_initializer_for__k4x4IdentityUp___16(): pass

def _dynamic_initializer_for__k4x4IdentityAt___16(): pass

def _dynamic_initializer_for__k4x4IdentityPos___16(): pass

def _dynamic_initializer_for__k3x4IdentityRight___94(): pass

def _dynamic_initializer_for__k3x4IdentityUp___94(): pass

def _dynamic_initializer_for__k3x4IdentityAt___94(): pass

def _dynamic_initializer_for__k3x4IdentityPos___90(): pass

def _dynamic_initializer_for__kNonCollidingPlane___14(): pass

def _dynamic_initializer_for__k4x4IdentityRight___17(): pass

def _dynamic_initializer_for__k4x4IdentityUp___17(): pass

def _dynamic_initializer_for__k4x4IdentityAt___17(): pass

def _dynamic_initializer_for__k4x4IdentityPos___17(): pass

def _dynamic_initializer_for__k3x4IdentityRight___95(): pass

def _dynamic_initializer_for__k3x4IdentityUp___95(): pass

def _dynamic_initializer_for__k3x4IdentityAt___95(): pass

def _dynamic_initializer_for__k3x4IdentityPos___91(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___16(): pass

def _dynamic_initializer_for__mCache___15(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___16(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___15(): pass

def _dynamic_initializer_for__cEgVertexNode_cTkLineVertexData_32_128___mgVertexLayout__(): pass

def _dynamic_initializer_for__cEgVertexNode_cTkTerrainVertexData_32_120___mgVertexLayout__(): pass

def _dynamic_initializer_for__cEgSingleLineNode__mgVertexLayout__(): pass

def _dynamic_initializer_for__VcTkNoiseVoxelTypeEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorRegionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorSettingsArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorSettingsElement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorSettingsTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseVoxelTypeEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorRegionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorSettingsArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorSettingsElement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVoxelGeneratorSettingsTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkCavesEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkFeaturesEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGridLayersEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseCaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseFeatureData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseFlattenFixedPosition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseFlattenOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseFlattenPoint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseGridData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseLayerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseLayersEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseOffsetEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseSuperFormulaData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseSuperPrimitiveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseUberData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseUberLayerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkCavesEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkFeaturesEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGridLayersEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseCaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseFeatureData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseFlattenFixedPosition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseFlattenOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseFlattenPoint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseGridData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseLayerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseLayersEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseOffsetEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseSuperFormulaData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseSuperPrimitiveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseUberData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNoiseUberLayerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkActionButtonLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkActionButtonMap___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAxisImageLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAxisPathMapping___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkButtonImageLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkButtonPathMapping___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkChordPathMapping___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkChordsImageLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkControllerButtonLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkControllerList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkControllerSpecification___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputValidation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkOpenVRControllerList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkOpenVRControllerLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkPlatformButtonPair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkTriggerFeedbackData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkActionButtonLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkActionButtonMap___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAxisImageLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAxisPathMapping___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkButtonImageLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkButtonPathMapping___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkChordPathMapping___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkChordsImageLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkControllerButtonLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkControllerList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkControllerSpecification___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputValidation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkOpenVRControllerList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkOpenVRControllerLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkPlatformButtonPair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkTriggerFeedbackData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActionMapping2___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActionMapping___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkImGuiData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkImGuiSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkImGuiWindowData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkPostProcessData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkRandomComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSpeedLineData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkTrailData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActionMapping2___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActionMapping___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkImGuiData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkImGuiSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkImGuiWindowData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkPostProcessData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkRandomComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSpeedLineData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkTrailData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemRuleTemplate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemRulesData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMagicModelData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialMetaData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkParticleBurstData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkParticleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemRuleTemplate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemRulesData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMagicModelData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialMetaData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkParticleBurstData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkParticleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkCoordinateOrientation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkEmitFromParticleInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkEmitterData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkEmitterFloatProperty___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkFoliageData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkHeavyAirCollection___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkHeavyAirData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkHeavyAirSystem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkIdSceneFilename___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputAxisEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputFrame___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputFrameArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputHandEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemGlobalRestriction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemGlobalVariation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemInnerRule___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemLocatorEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemRestrictionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemRule___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkCoordinateOrientation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkEmitFromParticleInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkEmitterData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkEmitterFloatProperty___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkFoliageData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkHeavyAirCollection___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkHeavyAirData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkHeavyAirSystem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkIdSceneFilename___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputAxisEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputFrame___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputFrameArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInputHandEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemGlobalRestriction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemGlobalVariation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemInnerRule___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemLocatorEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemRestrictionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLSystemRule___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkModelResource___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkPaletteTexture___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProbability___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTexture___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureChosenOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureChosenOptionList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureChosenOptionSampler___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureLayer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkRawID___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSketchConditions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSketchNodeConnections___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSketchNodeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkTextureResource___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkTrophyData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkTrophyEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkTrophyEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkModelResource___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkPaletteTexture___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProbability___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTexture___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureChosenOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureChosenOptionList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureChosenOptionSampler___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureLayer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralTextureList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkRawID___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSketchConditions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSketchNodeConnections___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSketchNodeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkTextureResource___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkTrophyData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkTrophyEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkTrophyEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBoundingBoxData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkCameraWanderData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkEqualityEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkFloatRange___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGlobals___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkIdArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInteractiveControlData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInteractiveSceneData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInteractivityData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLODModelResource___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLanguageFontTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLanguageFontTableEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLanguages___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLanguagesAllowedData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLocalisationEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLocalisationTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialResource___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMetadataFilenameList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMetadataReadMask___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkModelRendererCameraData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkModelRendererData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkUserServiceAuthProvider___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBoundingBoxData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkCameraWanderData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkEqualityEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkFloatRange___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGlobals___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkIdArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInteractiveControlData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInteractiveSceneData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInteractivityData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLODModelResource___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLanguageFontTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLanguageFontTableEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLanguages___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLanguagesAllowedData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLocalisationEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLocalisationTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialResource___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMetadataFilenameList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMetadataReadMask___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkModelRendererCameraData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkModelRendererData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkUserServiceAuthProvider___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBehaviourTreeConcurrentSelectorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBehaviourTreePriorityDecoratorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBehaviourTreeSequentialSelectorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBehaviourTreeSucceedDecoratorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardComparisonTypeEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueBool___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueFloat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueId___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueInteger___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueVector___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardKey___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueBool___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueFloat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueId___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueInteger___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueVector___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkPhysRelVectorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSceneBoneRemapping___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSceneBoneRemappingTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSceneNodeAttributeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSceneNodeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkTransformData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkBehaviourTreeConcurrentSelectorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBehaviourTreePriorityDecoratorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBehaviourTreeSequentialSelectorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBehaviourTreeSucceedDecoratorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardComparisonTypeEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueBool___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueFloat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueId___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueInteger___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardDefaultValueVector___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardKey___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueBool___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueFloat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueId___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueInteger___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkBlackboardValueVector___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkPhysRelVectorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSceneBoneRemapping___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSceneBoneRemappingTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSceneNodeAttributeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSceneNodeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkTransformData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialAlternative___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialFlags___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialSampler___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillComment___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillConnect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillFlag___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillLink___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillNode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialUniform___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVertexLayout___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVertexStream___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialAlternative___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialFlags___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialSampler___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillComment___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillConnect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillFlag___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillLink___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialShaderMillNode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMaterialUniform___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVertexLayout___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVertexStream___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimCompactMetadata___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimMetadata___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimNodeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimNodeFrameData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimNodeFrameHalfData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGeometryData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGeometryStreamData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkIndexStream___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkJointBindingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkJointExtentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkJointMirrorAxis___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMeshData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkMeshMetaData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkPhysicsData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkRagdollData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVertexElement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimCompactMetadata___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimMetadata___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimNodeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimNodeFrameData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimNodeFrameHalfData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGeometryData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGeometryStreamData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkIndexStream___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkJointBindingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkJointExtentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkJointMirrorAxis___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMeshData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkMeshMetaData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkPhysicsData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkRagdollData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVertexElement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTKNGuiEditorComponentSize___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTKNGuiEditorTextType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkEntitlementList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiAlignment___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiEditorGraphicType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiEditorLayout___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiEditorStyleColour___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiEditorStyleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiForcedStyle___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiGraphicStyle___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiGraphicStyleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiLayoutList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiLayoutListData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiRectanglePulseEffect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiTextStyle___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiTextStyleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiUserSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiWindowLayoutData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTKNGuiEditorComponentSize___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTKNGuiEditorTextType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkEntitlementList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiAlignment___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiEditorGraphicType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiEditorLayout___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiEditorStyleColour___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiEditorStyleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiForcedStyle___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiGraphicStyle___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiGraphicStyleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiLayoutList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiLayoutListData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiRectanglePulseEffect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiTextStyle___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiTextStyleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiUserSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNGuiWindowLayoutData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimDetailSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimDetailSettingsData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimDetailSettingsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkDynamicResScalingSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkEngineSettingTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkEngineSettingsMapping___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGameSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGraphicsDetailPreset___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGraphicsDetailTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGraphicsSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLODDistances___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLODSettingsData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimDetailSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimDetailSettingsData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimDetailSettingsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkDynamicResScalingSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkEngineSettingTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkEngineSettingsMapping___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGameSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGraphicsDetailPreset___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGraphicsDetailTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGraphicsSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLODDistances___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLODSettingsData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkGravityComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkLODComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkModelDescriptorList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralInstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralInstanceData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralModelComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralModelList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkReferenceComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkResourceDescriptorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkResourceDescriptorList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkResourceFilterData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkResourceFilterList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkRotationComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkShearWindData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkShearWindOctaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkSketchComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkGravityComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkLODComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkModelDescriptorList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralInstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralInstanceData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralModelComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkProceduralModelList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkReferenceComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkResourceDescriptorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkResourceDescriptorList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkResourceFilterData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkResourceFilterList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkRotationComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkShearWindData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkShearWindOctaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkSketchComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAudioAnimTrigger___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAudioComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkCreatureTailComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkCreatureTailJoints___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkCreatureTailParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkDynamicChainComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInstanceWindComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkNetReplicatedEntityComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkPhysicsComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkStaticPhysicsComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkVolumeTriggerType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAudioAnimTrigger___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAudioComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkCreatureTailComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkCreatureTailJoints___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkCreatureTailParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkDynamicChainComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInstanceWindComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkNetReplicatedEntityComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkPhysicsComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkStaticPhysicsComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkVolumeTriggerType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimBlendTree___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimJointLODData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseBabyModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseCorrelationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseExampleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseExampleElement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimationComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimationDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimationGameData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAttachmentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkCurveType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkHitCurveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkInOutCurve___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimBlendTree___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimJointLODData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseBabyModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseCorrelationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseExampleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimPoseExampleElement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimationComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimationDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimationGameData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAttachmentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkCurveType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkHitCurveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkInOutCurve___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWikiTopicType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderCategoryConfig___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderCreatureCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderCustomCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderFloraCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderMineralCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderPlanetCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderRecord___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderRecordCustomData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderTreasureCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderWeirdBasePartCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnim2dBlendNode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnim2dBlendNodeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimAnimNode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimBlendNode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimVectorBlendNode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimVectorBlendNodeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWikiTopicType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderCategoryConfig___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderCreatureCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderCustomCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderFloraCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderMineralCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderPlanetCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderRecord___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderRecordCustomData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderTreasureCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWonderWeirdBasePartCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnim2dBlendNode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnim2dBlendNodeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimAnimNode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimBlendNode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimVectorBlendNode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkAnimVectorBlendNodeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcIDLookupPath___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcIDLookupPaths___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcResourceElement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoriesTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryEntryBranch___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryPage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryPageSeenData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryPageSeenDataArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTestMetadata___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVibrationChannelData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVibrationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVibrationDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWiki___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWikiCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWikiPage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWikiTopic___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcIDLookupPath___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcIDLookupPaths___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcResourceElement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoriesTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryEntryBranch___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryPage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryPageSeenData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryPageSeenDataArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTestMetadata___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVibrationChannelData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVibrationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVibrationDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWiki___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWikiCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWikiPage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWikiTopic___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLaunchFuelCostDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcOptionsUIHeaderIcons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBaseDifficultyData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoModeAdjustData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoModeSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcReputationGainDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerRechargeDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSprintingCostDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceCollectionDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSurvivalBarBoolArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTriggerFeedbackState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTriggerFeedbackStateTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUserSettingsData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLaunchFuelCostDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcOptionsUIHeaderIcons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBaseDifficultyData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoModeAdjustData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoModeSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcReputationGainDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerRechargeDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSprintingCostDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceCollectionDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSurvivalBarBoolArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTriggerFeedbackState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTriggerFeedbackStateTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUserSettingsData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingEditability___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingLocData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingUIOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingsData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingsReplicatedState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyStartWithAllItemsKnownOptionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyStateData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEnergyDrainDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFuelUseDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGyroSettingsData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardDrainDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryStackLimitsDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemShopAvailabilityDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemShopAvailabilityDifficultyOptionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingEditability___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingLocData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingUIOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingsData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingsReplicatedState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyStartWithAllItemsKnownOptionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyStateData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEnergyDrainDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFuelUseDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGyroSettingsData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardDrainDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryStackLimitsDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemShopAvailabilityDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemShopAvailabilityDifficultyOptionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDeathConsequencesDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyConfig___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyCurrencyCostOptionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyFuelUseOptionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyFuelUseTechOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyInventoryStackSizeOptionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyOptionGroups___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyOptionUIGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyPresetType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingCommonData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDeathConsequencesDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyConfig___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyCurrencyCostOptionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyFuelUseOptionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyFuelUseTechOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyInventoryStackSizeOptionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyOptionGroups___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyOptionUIGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultyPresetType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDifficultySettingCommonData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcActiveSurvivalBarsDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlockListPersistence___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlockedMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlockedUser___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBreakTechOnDamageDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcChargingRequirementsDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatTimerDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHostilityDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCurrencyCostDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageGivenDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageReceivedDifficultyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHotActionMenuTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHotActionsSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInWorldUIScreenData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcQuickMenuActionSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcQuickMenuActions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRemoteWeapons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipWeaponData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipWeapons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpringLink___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcActiveSurvivalBarsDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlockListPersistence___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlockedMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlockedUser___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBreakTechOnDamageDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcChargingRequirementsDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatTimerDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHostilityDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCurrencyCostDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageGivenDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageReceivedDifficultyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHotActionMenuTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHotActionsSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInWorldUIScreenData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcQuickMenuActionSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcQuickMenuActions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRemoteWeapons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipWeaponData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipWeapons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpringLink___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAIShipWeapons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCamouflageData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponBiomeProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponPropertiesData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponPropertiesTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeapons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleCheckpointComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleGarageComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleMuzzleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleRaceInviteComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleScanTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleScanTableEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleWeaponMode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleWeaponMuzzleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWaypointComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAIShipWeapons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCamouflageData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponBiomeProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponPropertiesData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponPropertiesTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeapons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleCheckpointComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleGarageComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleMuzzleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleRaceInviteComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleScanTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleScanTableEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleWeaponMode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleWeaponMuzzleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWaypointComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshPart___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshPartData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshPartTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshPartTypeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechPartAudioEventOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechWeaponLocation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechWeaponLocationPriority___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshPart___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshPartData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshPartTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshPartTypeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechMeshType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechPartAudioEventOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechWeaponLocation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechWeaponLocationPriority___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVehicleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBackgroundSpaceEncounterInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBackgroundSpaceEncounterSpawnConditions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugCamera___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugCameraEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugShipTravelLine___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExoMechWeaponData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechAudioEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechAudioEventTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechDebugSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnAbandonedFreighter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnAlienFreighter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnConditions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnFrigateFlyby___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnObject___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnPirates___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnSpaceHostiles___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnTrader___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBackgroundSpaceEncounterInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBackgroundSpaceEncounterSpawnConditions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugCamera___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugCameraEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugShipTravelLine___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExoMechWeaponData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechAudioEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechAudioEventTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechDebugSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnAbandonedFreighter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnAlienFreighter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnConditions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnFrigateFlyby___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnObject___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnPirates___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnSpaceHostiles___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPulseEncounterSpawnTrader___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipClearCommunication___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipEscort___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipFlybyClose___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipFlybyIntercept___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipFlybyMineAsteroids___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipFlybyOverhead___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipLandNextToPlayer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipReceiveHail___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipReceiveMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAICombatDefinition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIDeathData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIPerformanceArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIPlanetPatrolData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipDialogue___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipDialogueTreeEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipOwnershipComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipAvoidanceData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipShieldComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipClearCommunication___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipEscort___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipFlybyClose___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipFlybyIntercept___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipFlybyMineAsteroids___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipFlybyOverhead___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipLandNextToPlayer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipReceiveHail___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDoShipReceiveMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAICombatDefinition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIDeathData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIPerformanceArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIPlanetPatrolData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipDialogue___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipDialogueTreeEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipOwnershipComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipAvoidanceData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipShieldComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEngineComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipAim___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipClassBonuses___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipControlData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipEngineData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipWarpData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShieldComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIAttackData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIAttackDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipTravelData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEngineComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipAim___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipClassBonuses___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipControlData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipEngineData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpaceshipWarpData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShieldComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIAttackData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipAIAttackDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipTravelData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipModelDataArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipPreloadCacheData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipPreloadList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipRoles___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipWeightingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBountySpawnInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugPlanetPos___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterSyncComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSquadronConfig___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipFlareComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipShieldData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipModelDataArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipPreloadCacheData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipPreloadList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipRoles___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipWeightingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBountySpawnInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugPlanetPos___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterSyncComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSquadronConfig___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipFlareComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipShieldData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAIShipDebugSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAIShipSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAIShipSpawnMarkerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipInstanceData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipManagerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipMappingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipModelData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainEditing___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainTexture___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTileTypeSets___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWaterColourSettingList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWealthClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherColourSettingList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherColourSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherWeightings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAIShipDebugSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAIShipSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAIShipSpawnMarkerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipInstanceData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipManagerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipMappingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAISpaceshipModelData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainEditing___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainTexture___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTileTypeSets___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWaterColourSettingList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWealthClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherColourSettingList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherColourSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherWeightings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetSectionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetSentinelData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetSize___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetTerrainColour___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetTradingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWaterColourData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWaterData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWeatherColourData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWeatherColourIndex___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWeatherData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerConflictData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRainbowType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScreenFilterOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemSkyColourData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceSkyColourSettingList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnDensity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnDensityList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainControls___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetSectionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetSentinelData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetSize___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetTerrainColour___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetTradingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWaterColourData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWaterData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWeatherColourData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWeatherColourIndex___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetWeatherData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerConflictData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRainbowType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScreenFilterOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemSkyColourData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceSkyColourSettingList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnDensity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnDensityList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainControls___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeavyAirSettingValues___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMiningSubstanceData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcOverlayTexture___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetColourData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetDataResourceHint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetGenerationInputData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetGenerationIntermediateData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetHazardData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetHeavyAirData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetRingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetaryMappingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetaryMappingTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetaryMappingValues___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeavyAirSettingValues___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMiningSubstanceData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcOverlayTexture___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetColourData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetDataResourceHint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetGenerationInputData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetGenerationIntermediateData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetHazardData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetHeavyAirData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetRingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetaryMappingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetaryMappingTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetaryMappingValues___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeCloudSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeFileList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeFileListOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeFileListOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeSubType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExternalObjectFileList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExternalObjectList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeavyAirColourData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeavyAirList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeavyAirSetting___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCNavigationAreaComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemLocator___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemLocatorChoice___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemLocatorTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemTraderSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceStationSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeCloudSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeFileList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeFileListOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeFileListOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeSubType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExternalObjectFileList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExternalObjectList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeavyAirColourData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeavyAirList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeavyAirSetting___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCNavigationAreaComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemLocator___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemLocatorChoice___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemLocatorTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemTraderSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceStationSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGeneratorAssignment___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGeneratorRing___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGeneratorSlab___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGeneratorSurround___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtmosphereList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeListPerStarType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceTimers___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedShipCounts___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSmokeBotPlanetReport___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSmokeBotReport___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSmokeBotStats___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSmokeBotSystemReport___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarGenerationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemEventWarpIn___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemEventWarpOut___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemEventWarpPlayer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGeneratorAssignment___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGeneratorRing___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGeneratorSlab___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGeneratorSurround___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtmosphereList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeListPerStarType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceTimers___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedShipCounts___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSmokeBotPlanetReport___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSmokeBotReport___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSmokeBotStats___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSmokeBotSystemReport___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarGenerationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemEventWarpIn___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemEventWarpOut___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSolarSystemEventWarpPlayer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcButtonSpawn___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcButtonSpawnOffset___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcButtonSpawnTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechTargetSelectionWeightingSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRobotLaserData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelEncounterOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelMechWeaponData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelMechWeaponMode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelResource___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelRobotComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcButtonSpawn___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcButtonSpawnOffset___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcButtonSpawnTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMechTargetSelectionWeightingSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRobotLaserData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelEncounterOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelMechWeaponData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelMechWeaponMode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelResource___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelRobotComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneControlData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneResource___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneWeaponData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileImpactData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileImpactType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileLineData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShootableComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTracerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneControlData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneResource___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneWeaponData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileImpactData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileImpactType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileLineData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShootableComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTracerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBoidData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectDamageMultiplier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageMultiplier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageMultiplierLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDestructableComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLaserBeamData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissileComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBoidData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectDamageMultiplier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageMultiplier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageMultiplierLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDamageType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDestructableComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLaserBeamData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissileComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectileDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterStateType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCommunicatorMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCommunicatorMessageWeighted___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerEmote___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerEmoteList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerEmotePropData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerHazardData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerHazardTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerHazardType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerStickData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSurvivalBarType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectorOffsetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEffectData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcZoomData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterStateType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCommunicatorMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCommunicatorMessageWeighted___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerEmote___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerEmoteList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerEmotePropData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerHazardData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerHazardTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerHazardType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerStickData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSurvivalBarType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProjectorOffsetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEffectData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcZoomData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAreaDamageData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAreaDamageDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDiscoveryThumbnailOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebrisData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExplosionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExplosionDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGrabbableData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHand___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMovementDirection___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoBuilding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoBuildings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoCreature___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoFauna___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoFlora___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoPlant___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterAnimationOverrideData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterIKOverrideData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterIKStateData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterStateData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterStateTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcReplacementEffectData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAreaDamageData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAreaDamageDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDiscoveryThumbnailOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebrisData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExplosionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExplosionDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGrabbableData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHand___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMovementDirection___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoBuilding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoBuildings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoCreature___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoFauna___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoFlora___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhotoPlant___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterAnimationOverrideData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterIKOverrideData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterIKStateData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterStateData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterStateTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcReplacementEffectData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCRaceProbabilityModifierData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCReactionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCReactionEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourAreaProperty___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourAreaPropertyWeightEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourBuildingClassCapacityEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourBuildingClassWeightEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourObjectTypeWeightEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSpawnTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCWordReactionCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCWordReactionList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCWordReactionTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCRaceProbabilityModifierData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCReactionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCReactionEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourAreaProperty___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourAreaPropertyWeightEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourBuildingClassCapacityEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourBuildingClassWeightEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourObjectTypeWeightEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSettlementBehaviourState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSpawnTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCWordReactionCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCWordReactionList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCWordReactionTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCAnimationList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCAnimationSetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCAnimationsData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCColourGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCColourTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCDebugSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractionsDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractiveObjectState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractiveObjectStateTransition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractiveObjectType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCProbabilityAnimationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCProbabilityReactionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCProbabilityWordReactionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPropInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPropTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductToCollect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventGPSHint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCAnimationList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCAnimationSetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCAnimationsData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCColourGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCColourTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCDebugSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractionsDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractiveObjectState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractiveObjectStateTransition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractiveObjectType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCProbabilityAnimationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCProbabilityReactionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCProbabilityWordReactionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPropInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPropTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductToCollect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventGPSHint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingPartSearchType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcConstructionPart___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJudgementMessageOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTest___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionDifficulty___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionFaction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionGalacticFeature___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionGalacticPoint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionPageHint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForRefuel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForRepair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForSettlementActivity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForShips___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForStatMilestone___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForStatSeasonal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForSuitUpgrade___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForWarps___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitRealTime___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitRealTimeCombat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingPartSearchType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcConstructionPart___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcJudgementMessageOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTest___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionDifficulty___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionFaction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionGalacticFeature___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionGalacticPoint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionPageHint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForRefuel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForRepair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForSettlementActivity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForShips___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForStatMilestone___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForStatSeasonal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForSuitUpgrade___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForWarps___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitRealTime___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitRealTimeCombat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStartScanEventSpecific___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStartSummonAnomaly___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStop___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSummonNexus___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSuppressMarkers___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceTeleport___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceVehicleScan___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceVisitPlanets___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWait___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForAbandFreighterDoorOpen___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForBuild___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForCompletionMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForConditions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForDepots___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForFactionStanding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForFreighterPods___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForFriendlyDroneScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForPhoto___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForPortalWarp___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStartScanEventSpecific___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStartSummonAnomaly___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStop___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSummonNexus___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSuppressMarkers___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceTeleport___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceVehicleScan___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceVisitPlanets___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWait___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForAbandFreighterDoorOpen___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForBuild___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForCompletionMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForConditions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForDepots___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForFactionStanding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForFreighterPods___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForFriendlyDroneScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForPhoto___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceWaitForPortalWarp___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceKill___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceKillEncounter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceLearnWords___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceLeaveNexusMP___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceModifyStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequencePirates___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceProductAmountNeeded___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceRepairTech___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceScan___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSendChatMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSetCurrentMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSetGlobalMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceShowMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceShowMissionUpdateMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceShowPodMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceShowSeasonTimeWarning___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSignalGalacticPoint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStartMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStartScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceKill___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceKillEncounter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceLearnWords___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceLeaveNexusMP___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceModifyStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequencePirates___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceProductAmountNeeded___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceRepairTech___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceScan___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSendChatMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSetCurrentMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSetGlobalMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceShowMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceShowMissionUpdateMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceShowPodMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceShowSeasonTimeWarning___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceSignalGalacticPoint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStartMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceStartScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceEndScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceEnsureBarrelsAtPlayerSettlement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceExplorationLogSpecial___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceExploreAbandonedFreighter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceFeed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceFinishSummonAnomaly___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceFreighterDefend___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceFreighterEngage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGatherForBuild___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGatherForRefuel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGatherForRepair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetInShip___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetToExpedition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetToScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetUnits___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetUnitsToBuyItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGoToGalacticPoint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceEndScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceEnsureBarrelsAtPlayerSettlement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceExplorationLogSpecial___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceExploreAbandonedFreighter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceFeed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceFinishSummonAnomaly___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceFreighterDefend___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceFreighterEngage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGatherForBuild___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGatherForRefuel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGatherForRepair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetInShip___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetToExpedition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetToScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetUnits___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGetUnitsToBuyItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGoToGalacticPoint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCollectProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCollectSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCommunicator___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCommunicatorOnTakeOff___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCompleteMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCompleteSeasonalMilestone___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCompleteSettlementJudgement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceConditionalReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceConstruct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceConstructSettlementBuildingWithScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCreateSpecificPulseEncounter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDetailMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDetailMessagePoint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDiscover___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDiscoverOnPlanet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDisplaySeasonRewardReminder___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDoMissionsForFaction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCollectProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCollectSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCommunicator___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCommunicatorOnTakeOff___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCompleteMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCompleteSeasonalMilestone___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCompleteSettlementJudgement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceConditionalReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceConstruct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceConstructSettlementBuildingWithScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCreateSpecificPulseEncounter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDetailMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDetailMessagePoint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDiscover___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDiscoverOnPlanet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDisplaySeasonRewardReminder___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceDoMissionsForFaction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDailyRecurrence___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDay___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionCommunityData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionCommunityMissionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceClearDetailMessages___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionIDEpochPair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSchedulesTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSchedulingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceAudioEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceBounty___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceBroadcastMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceBuild___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceClearInventoryHistory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCloseMenu___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCollectMoney___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCollectMultiProducts___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMonth___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMonthlyRecurrence___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectiveTextFormatOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTargetMissionSurveyOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeeklyRecurrence___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcYearlyRecurrence___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDailyRecurrence___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDay___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionCommunityData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionCommunityMissionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceClearDetailMessages___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionIDEpochPair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSchedulesTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSchedulingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceAudioEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceBounty___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceBroadcastMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceBuild___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceClearInventoryHistory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCloseMenu___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCollectMoney___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionSequenceCollectMultiProducts___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMonth___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMonthlyRecurrence___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectiveTextFormatOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTargetMissionSurveyOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeeklyRecurrence___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcYearlyRecurrence___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomNotifyTimerOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionItemsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionProductEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionSubstanceEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFactionSelectOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericMissionSequence___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericMissionStage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericMissionVersionProgress___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionBoardOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceRemoveCommunicatorMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceRemoveCommunicatorTakeOffMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceRemoveScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceResetPulseEncounterOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomNotifyTimerOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionItemsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionProductEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultMissionSubstanceEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFactionSelectOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericMissionSequence___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericMissionStage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericMissionVersionProgress___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionBoardOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceRemoveCommunicatorMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceRemoveCommunicatorTakeOffMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceRemoveScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConsequenceResetPulseEncounterOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSystemHasInfestedPlanet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSystemPlanetTest___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSystemRace___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSystemStarClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTakingDamage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTechnologyKnown___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTotalWarpsNumber___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTutorialEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionUnclaimedStageReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionUnderwaterDepth___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionUsingPortal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionUsingThirdPersonCamera___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionVisorActive___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWaitForPirates___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWaitForTime___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWarping___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWaterInSystem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWaterPlanet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWeaponMode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWeather___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWordCategoryKnown___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWristMenuOpen___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSystemHasInfestedPlanet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSystemPlanetTest___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSystemRace___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSystemStarClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTakingDamage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTechnologyKnown___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTotalWarpsNumber___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionTutorialEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionUnclaimedStageReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionUnderwaterDepth___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionUsingPortal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionUsingThirdPersonCamera___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionVisorActive___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWaitForPirates___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWaitForTime___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWarping___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWaterInSystem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWaterPlanet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWeaponMode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWeather___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWordCategoryKnown___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionWristMenuOpen___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionProductKnown___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPulseEncounterActive___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPulseEncounterOverriden___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionReadyToSpawnPirates___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionRefinerActive___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionRefinerHasOutput___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionRequestedPhoto___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionRidingCreature___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSeasonNumber___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSeasonRewardUnlocked___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSeasonRewardsRecipe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSentinelLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSentinelsDisabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSettlementBuildingsAllStarted___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSettlementStatLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSettlementsEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionShieldDown___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionShipEngineStatus___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSpaceCombatEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSpecialKnown___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSquadronPilotsOwned___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSquadronSlots___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionStartWithAllPartsKnown___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionStatChange___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionStatDiff___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionStatLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionProductKnown___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPulseEncounterActive___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPulseEncounterOverriden___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionReadyToSpawnPirates___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionRefinerActive___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionRefinerHasOutput___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionRequestedPhoto___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionRidingCreature___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSeasonNumber___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSeasonRewardUnlocked___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSeasonRewardsRecipe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSentinelLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSentinelsDisabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSettlementBuildingsAllStarted___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSettlementStatLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSettlementsEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionShieldDown___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionShipEngineStatus___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSpaceCombatEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSpecialKnown___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSquadronPilotsOwned___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionSquadronSlots___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionStartWithAllPartsKnown___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionStatChange___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionStatDiff___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionStatLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionCompleted___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionMessagePortal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionMessageWarp___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionSelected___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMultiplayerFreighterAvailable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNPCHabitationHasWorker___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNearObject___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNearPole___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNearScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNearestBuilding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNexusEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNumAtlasStationsVisited___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNumberOfShipsOwned___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionOnFootCombatEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionOnMultiplayerMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionOnOtherSideOfPortal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionOnPlanetWithSandwormsOverriden___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPadActive___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPercentageChance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPirateSystem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPlanetAttackPiratesActive___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPlanetCreatureRoles___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPlanetResourceHint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPlanetStatLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPrimaryExocraft___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionCompleted___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionMessagePortal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionMessageWarp___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMissionSelected___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMultiplayerFreighterAvailable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNPCHabitationHasWorker___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNearObject___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNearPole___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNearScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNearestBuilding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNexusEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNumAtlasStationsVisited___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionNumberOfShipsOwned___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionOnFootCombatEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionOnMultiplayerMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionOnOtherSideOfPortal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionOnPlanetWithSandwormsOverriden___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPadActive___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPercentageChance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPirateSystem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPlanetAttackPiratesActive___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPlanetCreatureRoles___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPlanetResourceHint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPlanetStatLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionPrimaryExocraft___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInVR___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInventoryOpen___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInventorySlots___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsAbandFreighterDoorOpen___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsAnomalyLoaded___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsCurrentMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsDepotDestroyed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsFrigateFlybyActive___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsGrabbed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsLookingAtAnomaly___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsMissionInProgress___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsPartyPlanetUnlocked___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsPlayerWanted___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsPlayerWeak___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventActive___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventLocal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventLocalOrNear___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventOnCurrentPlanet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventRepaired___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsSurveying___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsTechnologyRepaired___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionItemCostsEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionLifeSupportEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionLocalScanActive___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionLocation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMessageBeaconsQuery___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInVR___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInventoryOpen___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInventorySlots___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsAbandFreighterDoorOpen___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsAnomalyLoaded___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsCurrentMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsDepotDestroyed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsFrigateFlybyActive___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsGrabbed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsLookingAtAnomaly___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsMissionInProgress___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsPartyPlanetUnlocked___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsPlayerWanted___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsPlayerWeak___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventActive___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventLocal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventLocalOrNear___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventOnCurrentPlanet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsScanEventRepaired___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsSurveying___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionIsTechnologyRepaired___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionItemCostsEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionLifeSupportEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionLocalScanActive___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionLocation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionMessageBeaconsQuery___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasMultiTool___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasPendingSettlementJudgement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasPlatformReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasProcMissionForFaction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasProcProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasProcTechnology___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSeasonalReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSettlement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSettlementBuilding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSettlementLocal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSettlementProductPending___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasShip___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSpareProcTech___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasTechnology___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasTwitchReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHazard___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHazardsEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInCombat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInMultiplayer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInSeasonalUA___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInUA___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInteractionIndexChanged___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasMultiTool___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasPendingSettlementJudgement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasPlatformReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasProcMissionForFaction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasProcProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasProcTechnology___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSeasonalReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSettlement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSettlementBuilding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSettlementLocal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSettlementProductPending___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasShip___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSpareProcTech___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasTechnology___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasTwitchReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHazard___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHazardsEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInCombat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInMultiplayer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInSeasonalUA___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInUA___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionInteractionIndexChanged___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExtraSuitSlots___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionFactionRank___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionFeedingCreatures___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionForceHideMultiplayer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionFreighterBattle___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionFrigateCount___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionGameMode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionGlobalMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionGunOut___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasActiveDetailMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasActiveStatsMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasAnySettlementBuildingInProgress___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasCommunicatorSignal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasCreatureEggItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasEndpointForEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasEntitlement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasExocraft___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasFreighter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasFuel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasFuelForTakeoff___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasGalacticFeature___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasGrabbableTarget___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasIllegalGoods___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasIngredientsForItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasLegacyBasePending___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasMoney___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExtraSuitSlots___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionFactionRank___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionFeedingCreatures___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionForceHideMultiplayer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionFreighterBattle___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionFrigateCount___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionGameMode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionGlobalMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionGunOut___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasActiveDetailMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasActiveStatsMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasAnySettlementBuildingInProgress___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasCommunicatorSignal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasCreatureEggItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasEndpointForEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasEntitlement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasExocraft___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasFreighter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasFuel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasFuelForTakeoff___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasGalacticFeature___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasGrabbableTarget___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasIllegalGoods___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasIngredientsForItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasLegacyBasePending___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionHasMoney___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBaseRequiresPower___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBiomeType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBuildMenuOpen___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCameraControlStealing___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCanReceiveReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCanSummonExocraft___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCommunityResearchTier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionConvertedFromSeason___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureOwned___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureReadyToHatch___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureReadyToLay___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureSlots___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureSummoned___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureTrust___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCriticalMissionsDone___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCurrentSlope___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionDamagedFrigateAtHome___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionDefaultItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionEggMachinePageOpen___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionElevation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionEventRequiresRGB___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExocraftMoving___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExpeditionContainsReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExpeditionCount___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExpeditionNearlyOver___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExpeditionProgress___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBaseRequiresPower___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBiomeType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBuildMenuOpen___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCameraControlStealing___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCanReceiveReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCanSummonExocraft___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCommunityResearchTier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionConvertedFromSeason___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureOwned___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureReadyToHatch___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureReadyToLay___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureSlots___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureSummoned___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCreatureTrust___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCriticalMissionsDone___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionCurrentSlope___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionDamagedFrigateAtHome___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionDefaultItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionEggMachinePageOpen___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionElevation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionEventRequiresRGB___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExocraftMoving___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExpeditionContainsReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExpeditionCount___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExpeditionNearlyOver___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionExpeditionProgress___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAIShipCount___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAbandonedFreighterExplored___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAbandonedOrEmptySystem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAimingTeleporter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAlienPodAggroed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAllMilestonesComplete___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAreDroneHivePartsDestroyed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAutoPowerEnabled___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBaseClaimed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBasePartBuilt___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBasePartNear___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBasePartsQuery___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBasePowerGenerated___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBaseQuery___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcModelExplosionRule___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcModelExplosionRules___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventTableType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventTriggers___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSceneSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipClasses___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeaponClasses___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAIShipCount___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAbandonedFreighterExplored___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAbandonedOrEmptySystem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAimingTeleporter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAlienPodAggroed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAllMilestonesComplete___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAreDroneHivePartsDestroyed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionAutoPowerEnabled___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBaseClaimed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBasePartBuilt___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBasePartNear___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBasePartsQuery___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBasePowerGenerated___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMissionConditionBaseQuery___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcModelExplosionRule___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcModelExplosionRules___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventTableType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventTriggers___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSceneSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceshipClasses___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeaponClasses___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPassiveFrigateIncomeArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhysicsCollisionGroups___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcQuestItemPlacementRule___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotBiomeGases___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRoomCountRule___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRoomSequenceRule___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventSave___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventSolarSystemLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPassiveFrigateIncomeArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPhysicsCollisionGroups___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcQuestItemPlacementRule___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotBiomeGases___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRegionHotspotsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRoomCountRule___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRoomSequenceRule___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventSave___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEventSolarSystemLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionEventOccurrenceRate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionEventTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionInterventionEventData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionPaymentToken___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterBaseOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterBaseOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterDungeonChoice___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterDungeonParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateClassCost___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateFlybyLayout___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateFlybyOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateFlybyTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateFlybyType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateStatRange___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateStats___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateStatsByClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitIcons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitStrengthByType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitStrengthValues___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateUITraitLines___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryClassCostMultiplier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPassiveFrigateIncome___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionEventOccurrenceRate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionEventTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionInterventionEventData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionPaymentToken___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterBaseOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterBaseOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterDungeonChoice___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterDungeonParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateClassCost___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateFlybyLayout___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateFlybyOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateFlybyTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateFlybyType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateStatRange___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateStats___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateStatsByClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitIcons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitStrengthByType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitStrengthValues___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateUITraitLines___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryClassCostMultiplier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPassiveFrigateIncome___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGenerationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidSystemGenerationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugScene___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDungeonGenerationParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDungeonQuestParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDungeonRoomParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionCategoryStrength___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionDebriefPunctuation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionDifficultyKeyframe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionDurationValues___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionEventData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyWaypoint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyWaypointTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidGenerationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAsteroidSystemGenerationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDebugScene___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDungeonGenerationParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDungeonQuestParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDungeonRoomParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionCategoryStrength___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionDebriefPunctuation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionDifficultyKeyframe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionDurationValues___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionEventData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyWaypoint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyWaypointTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyInfoIcons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyMarkerSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyMarkerTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyRenderAnostreakData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyRenderSetupData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxySolarSystemOrbitParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxySolarSystemParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyStarAnomaly___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyStarAttributesData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyStarColours___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyStarTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyVoxelAttributesData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyInfoIcons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyMarkerSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyMarkerTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyRenderAnostreakData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyRenderSetupData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxySolarSystemOrbitParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxySolarSystemParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyStarAnomaly___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyStarAttributesData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyStarColours___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyStarTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyVoxelAttributesData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyAudioSetupData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyCameraData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyGenerationSetupData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCBuilding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCFace___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCModulePrototype___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCModuleSet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCTerrainConstraint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyAudioSetupData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyCameraData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalaxyGenerationSetupData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCBuilding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCFace___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCModulePrototype___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCModuleSet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCTerrainConstraint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingColourPalette___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterBaseRoom___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcIDPair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMinimumUseConstraint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcModuleOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementColourPalette___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementColourTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCDecorationFace___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCDecorationItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCDecorationSet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCDecorationTheme___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherEffectLightningData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherHazardLightningData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherHazardMeteorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherHazardTornadoData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightedBuildingSize___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightedColourId___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightedResource___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingColourPalette___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterBaseRoom___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcIDPair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMinimumUseConstraint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcModuleOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementColourPalette___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementColourTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCDecorationFace___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCDecorationItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCDecorationSet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWFCDecorationTheme___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherEffectLightningData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherHazardLightningData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherHazardMeteorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherHazardTornadoData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightedBuildingSize___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightedColourId___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightedResource___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardModifiers___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardValueTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardValues___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectPlacementCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectSpawnList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherColourModifiers___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherEffect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherEffectTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardModifiers___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardValueTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardValues___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectPlacementCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectSpawnList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSelectableObjectTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherColourModifiers___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherEffect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherEffectTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeatherTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetSkyProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcResourceCollectEffect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcResourceOrigin___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSkyProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceSkyColours___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceSkyProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStormProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceAmount___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainTextureSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainTileType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTileTypeOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTileTypeSet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetSkyProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcResourceCollectEffect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcResourceOrigin___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSkyProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceSkyColours___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceSkyProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStormProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceAmount___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainTextureSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainTileType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTileTypeOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTileTypeSet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEnvironmentSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExternalObjectListOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFogProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLightProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLightShaftProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectDefinitionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectSpawnDataArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectSpawnDataVariant___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetBuildingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetCloudProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetLife___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEnvironmentSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExternalObjectListOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFogProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLightProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLightShaftProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectDefinitionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectSpawnDataArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectSpawnDataVariant___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetBuildingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetCloudProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetLife___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingClassification___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingClusterLayout___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingClusterLayoutEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDefinitionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDefinitionTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDensity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDensityLevels___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDistribution___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingFilename___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingFilenameList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingOverrideData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingSpawnSlot___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingSystemTypeEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCloudProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEnvironmentProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFoliageComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingClassification___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingClusterLayout___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingClusterLayoutEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDefinitionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDefinitionTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDensity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDensityLevels___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingDistribution___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingFilename___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingFilenameList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingOverrideData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingSpawnSlot___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingSystemTypeEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCloudProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEnvironmentProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFoliageComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSpawnEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDeprecatedAssetsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEcosystemCreatureData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEcosystemSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcIkPistonData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPropType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSeatedPosture___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCTriggerTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetAccessoryGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetAccessoryInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetAccessoryTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetAccessoryType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetEggSpeciesOverrideData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetEggSpeciesOverrideTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetEggTraitModifierOverrideData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetEggTraitModifierOverrideTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSandwormTimerAndFrequencyOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnComponentOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSpawnEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDeprecatedAssetsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEcosystemCreatureData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEcosystemSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcIkPistonData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPropType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCSeatedPosture___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCTriggerTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetAccessoryGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetAccessoryInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetAccessoryTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetAccessoryType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetEggSpeciesOverrideData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetEggSpeciesOverrideTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetEggTraitModifierOverrideData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetEggTraitModifierOverrideTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSandwormTimerAndFrequencyOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnComponentOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationWeightedListDomainEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGroupDescription___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGroupProbability___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGroups___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHemiSphere___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureIkData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureLegIKComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleDescription___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleDescriptionTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleFilename___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleFilenameList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleFilenameTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoles___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSpawnComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationWeightedListDomainEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGroupDescription___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGroupProbability___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGroups___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHemiSphere___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureIkData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureLegIKComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleDescription___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleDescriptionTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleFilename___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleFilenameList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleFilenameTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoles___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSpawnComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourPlayAnimTrigger___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureActiveTime___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureAttractorComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureBaitComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDebugSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDebugWaypoint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDiet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureEffectComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureEggComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFilename___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationArchetypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationDensity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationDomainEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationDomainTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationOptionalWeightedList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationWeightedList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourPlayAnimTrigger___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureActiveTime___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureAttractorComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureBaitComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDebugSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDebugWaypoint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDiet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureEffectComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureEggComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFilename___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationArchetypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationDensity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationDomainEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationDomainTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationOptionalWeightedList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureGenerationWeightedList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviourData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviourMoodModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviourTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviourTraitModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviours___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetFollowUpBehaviour___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetMoodStaminaModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetTraitMoodModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetTraitMoodModifierList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetTraitStaminaModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeirdCreatureRewardList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviourData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviourMoodModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviourTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviourTraitModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetBehaviours___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetFollowUpBehaviour___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetMoodStaminaModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetTraitMoodModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetTraitMoodModifierList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetTraitStaminaModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeirdCreatureRewardList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureAudioTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureBehaviourTreeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureBehaviourTrees___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureEffectTrigger___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFilenameTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFoodList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFullBodyIKComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHarvestSubstanceList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleFrequencyModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSizeClasses___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureStupidName___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureStupidNameTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSubstanceList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureTagAndRarity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureVocalTestData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetActionMoodModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureAudioTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureBehaviourTreeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureBehaviourTrees___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureEffectTrigger___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFilenameTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFoodList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFullBodyIKComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHarvestSubstanceList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRoleFrequencyModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSizeClasses___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureStupidName___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureStupidNameTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSubstanceList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureTagAndRarity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureVocalTestData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetActionMoodModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRidingAnimation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRidingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRidingPartModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSwarmData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSwarmDataParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureVocalData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureVocalSoundData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureWeirdMovementData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFlyingSnakeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRidingAnimation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRidingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRidingPartModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSwarmData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureSwarmDataParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureVocalData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureVocalSoundData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureWeirdMovementData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFlyingSnakeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFlockMovementData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFootParticleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFootParticleSingleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHealthData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHoverMovementData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHoverMovementDataParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHoverTintableEffect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureMoveAnimData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureMovementData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetAccessory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetAccessorySlot___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetMood___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetPartHider___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetRewardActions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetTraitRange___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetTraitRanges___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetTraits___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFlockMovementData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFootParticleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFootParticleSingleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHealthData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHoverMovementData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHoverMovementDataParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHoverTintableEffect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureMoveAnimData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureMovementData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetAccessory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetAccessorySlot___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetMood___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetPartHider___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetRewardActions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetTraitRange___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetTraitRanges___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreaturePetTraits___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourPlayAnimData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourRegisterAttackerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourWaitData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBirdData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlackboardFloatCompareDecoratorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlackboardIntCompareDecoratorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlackboardIntModifyData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlackboardValueDecoratorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCooldownDecoratorData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureAlertData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureCrystalMovementData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureCrystalMovementDataParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFiendAttackData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureIkType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPrimaryAxis___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourPlayAnimData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourRegisterAttackerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourWaitData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBirdData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlackboardFloatCompareDecoratorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlackboardIntCompareDecoratorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlackboardIntModifyData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBlackboardValueDecoratorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCooldownDecoratorData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureAlertData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureCrystalMovementData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureCrystalMovementDataParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFiendAttackData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureIkType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPrimaryAxis___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourAppearData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourApplyDamageData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourCheckDeathData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourCooldownBeginData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourDetailAnimsData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourFaceTargetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourGetTargetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourIdleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourIncrementCounter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourLaunchProjectileData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourLegacyData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourLookData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourMaintainRangeFromTargetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourMoveToTargetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationRace___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationShipBobbleHeads___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationShipTrails___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationTextureGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationThrusterEffect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationThrusterEffects___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationThrusterJet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationUI___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationUIData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourAppearData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourApplyDamageData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourCheckDeathData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourCooldownBeginData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourDetailAnimsData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourFaceTargetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourGetTargetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourIdleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourIncrementCounter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourLaunchProjectileData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourLegacyData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourLookData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourMaintainRangeFromTargetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBehaviourMoveToTargetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationRace___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationShipBobbleHeads___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationShipTrails___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationTextureGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationThrusterEffect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationThrusterEffects___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationThrusterJet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationUI___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationUIData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAnimFrameEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBeenShotEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureNearbyEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBackpackData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBannerGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBannerImageData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBobbleHead___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBoneScales___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationCameraData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationColourGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationDescriptorGroupOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationDescriptorGroupOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationFreighterEngineEffect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationGroups___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerNearbyEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStateTimeEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStormEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAnimFrameEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBeenShotEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureNearbyEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBackpackData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBannerGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBannerImageData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBobbleHead___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationBoneScales___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationCameraData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationColourGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationDescriptorGroupOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationDescriptorGroupOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationFreighterEngineEffect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationGroups___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerNearbyEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStateTimeEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStormEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingSettingsAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseDefenceStatusAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDestroyAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDisplayText___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFiendCrimeAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFireSimpleInteractionAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateInteractionAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGoToStateAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHarvestPlantAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLodAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNodeActivationAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPainAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcParticleAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayAnimAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayAudioAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPowerStateAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcResetSimpleInteractionAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScareCreaturesAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUniqueIdComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWarpAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingSettingsAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseDefenceStatusAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDestroyAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDisplayText___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFiendCrimeAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFireSimpleInteractionAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateInteractionAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGoToStateAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHarvestPlantAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHazardAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLodAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNodeActivationAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPainAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcParticleAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayAnimAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayAudioAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPowerStateAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcResetSimpleInteractionAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScareCreaturesAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUniqueIdComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWarpAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAbandonedFreighterComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionTrigger___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionTriggerState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseDefenceStatusType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseDefenceTrigger___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBroadcastLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeTriggerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcChainComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterInterfaceComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcColouriseComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomInventoryComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDecorationComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterBaseComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryTechProbability___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLootProbability___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMarkerComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMetaBallComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMultitoolPoolData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTriggerActionComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAbandonedFreighterComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionTrigger___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionTriggerState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseDefenceStatusType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseDefenceTrigger___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBroadcastLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeTriggerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcChainComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterInterfaceComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcColouriseComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomInventoryComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDecorationComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterBaseComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryTechProbability___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLootProbability___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMarkerComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMetaBallComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMultitoolPoolData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerCharacterComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTriggerActionComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEncyclopediaComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMiningComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNetworkInterpolationComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNetworkPlayerMarkerComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcOutpostComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcOutpostLSystemPair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerFullBodyIKComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEffectComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannableComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpringAttachmentComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEncyclopediaComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMiningComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNetworkInterpolationComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNetworkPlayerMarkerComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcOutpostComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcOutpostLSystemPair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerFullBodyIKComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerWeaponComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScanEffectComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannableComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpringAttachmentComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionDof___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLadderComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLookAtComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractiveObjectComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRefinerUnitComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRocketLockerComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelCoverComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSquadronHologramComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyAttachmentComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionDof___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLadderComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLookAtComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCInteractiveObjectComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRefinerUnitComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRocketLockerComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelCoverComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSquadronHologramComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyAttachmentComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioAreaTriggerComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatPlayerComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcChairComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectsComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectsProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFeederComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHarvesterComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDissolveEffectComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEggMachineComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEncounterComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEncounterStateComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionHologramComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFleetHologramComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratorUnitComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHologramComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCHabitationComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlInputAxis___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlInputMouse___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlInputRemap___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerEffectsComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioAreaTriggerComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatPlayerComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcChairComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectsComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectsProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureFeederComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureHarvesterComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDissolveEffectComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEggMachineComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEncounterComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEncounterStateComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionHologramComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFleetHologramComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratorUnitComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHologramComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCHabitationComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlInputAxis___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlInputMouse___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlInputRemap___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerEffectsComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAmbientModeCameras___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterAlternateAnimation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterControlInputValidity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterControlOutputSpace___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterJetpackEffect___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterLookAtData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterMove___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterRotate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExactResource___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryValueData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlInput___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlModeEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDestructEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDestructRarities___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDestructTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableEntitlementItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableItemList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatIconTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsBonus___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcValueData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcgcwordcategorytableEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAmbientModeCameras___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterAlternateAnimation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterControlInputValidity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterControlOutputSpace___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterJetpackEffect___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterLookAtData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterMove___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterRotate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExactResource___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryValueData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlInput___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlModeEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerControlState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDestructEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDestructRarities___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDestructTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableEntitlementItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableItemList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatIconTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsBonus___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcValueData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcgcwordcategorytableEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingClassData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingClassTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableItemTree___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableItemTreeGroups___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableItemTreeNode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableItemTrees___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockablePlatformReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockablePlatformRewards___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableSeasonReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableSeasonRewards___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableTreeCostType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableTrees___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableTwitchReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableTwitchRewards___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeaponInventoryMaxUpgradeCapacity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightedFilename___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightingCurve___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingClassData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingClassTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableItemTree___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableItemTreeGroups___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableItemTreeNode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableItemTrees___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockablePlatformReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockablePlatformRewards___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableSeasonReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableSeasonRewards___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableTreeCostType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableTrees___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableTwitchReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUnlockableTwitchRewards___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeaponInventoryMaxUpgradeCapacity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightedFilename___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWeightingCurve___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceSecondaryLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechBoxData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechBoxTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnology___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyRarity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyRequirement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradeCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradeSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingCategoryData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceSecondaryLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechBoxData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechBoxTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnology___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyRarity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyRequirement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTechnologyTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradeCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradeSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingCategoryData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJobDetails___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJudgementData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJudgementOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJudgementPerkOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementPerkData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementPerkUsefulData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementPerksTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatChange___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatChangeArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatStrength___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatStrengthData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatStrengthRanges___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipInventoryMaxUpgradeCapacity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSizeIndicator___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStats___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryUtilityOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceSecondary___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceSecondaryBiome___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJobDetails___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJudgementData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJudgementOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJudgementPerkOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementPerkData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementPerkUsefulData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementPerksTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatChange___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatChangeArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatStrength___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatStrengthData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatStrengthRanges___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipInventoryMaxUpgradeCapacity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSizeIndicator___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStats___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStoryUtilityOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceSecondary___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSubstanceSecondaryBiome___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTeachWord___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTechRecipe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTeleport___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTimeWarp___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTraderFlyby___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTrigger___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTriggerSettlementJudgement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTriggerStorm___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUnlockSeasonReward___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUnlockTitle___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUpgradeBase___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUpgradeShipClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUpgradeWeaponClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUploadBase___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWantedLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWeapon___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWeaponSlot___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWikiTopic___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWorker___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementBuildingCost___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementBuildingCostData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementCustomJudgement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementGiftDetails___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTableItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTeachWord___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTechRecipe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTeleport___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTimeWarp___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTraderFlyby___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTrigger___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTriggerSettlementJudgement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTriggerStorm___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUnlockSeasonReward___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUnlockTitle___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUpgradeBase___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUpgradeShipClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUpgradeWeaponClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardUploadBase___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWantedLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWeapon___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWeaponSlot___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWikiTopic___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardWorker___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementBuildingCost___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementBuildingCostData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementCustomJudgement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementGiftDetails___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificSeasonalSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificShip___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificSpecial___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificTech___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificTechFromList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificWeapon___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStanding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStartPurchase___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStartSettlementExpedition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStatCompareAndSet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStatDiff___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSwapMultiTool___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSystemSpecificProductFromList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificSeasonalSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificShip___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificSpecial___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificTech___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificTechFromList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificWeapon___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStanding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStartPurchase___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStartSettlementExpedition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStatCompareAndSet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardStatDiff___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSwapMultiTool___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSystemSpecificProductFromList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSecondarySubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSendChatMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetAbandonedFreighterMissionState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetCurrentMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetInteractionSeenBitmask___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetNexusExitWarpTargetToFireteamMemberUA___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetWeaponSuppressed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSettlementCustomJudgement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSettlementJudgement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSettlementProgress___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSettlementStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShield___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShipAssistance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShipMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShipSlot___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShowBlackHoles___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSignalScan___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificCommunityTierProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificFrigate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificPetEgg___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificProductRecipe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificProductRecipeFromList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSecondarySubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSendChatMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetAbandonedFreighterMissionState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetCurrentMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetInteractionSeenBitmask___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetNexusExitWarpTargetToFireteamMemberUA___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSetWeaponSuppressed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSettlementCustomJudgement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSettlementJudgement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSettlementProgress___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSettlementStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShield___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShipAssistance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShipMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShipSlot___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardShowBlackHoles___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSignalScan___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificCommunityTierProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificFrigate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificPetEgg___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificProductRecipe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSpecificProductRecipeFromList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOverridePulseEncounterChance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPetAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPetEgg___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPetEggHatch___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPirateAttack___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPirateProbeSignal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPlanetSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPoliceScanSignal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProcTechProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProceduralProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProceduralProductFromBiome___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProceduralTechnology___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProductRecipe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRechargeTech___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRefreshHazProt___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRepairTech___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRepairWholeInventory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRequirementsForRecipe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSalvageShip___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardScan___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardScanEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardScanEventNearestBuilding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardScanEventOutcome___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSecondaryInteractionOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOverridePulseEncounterChance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPetAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPetEgg___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPetEggHatch___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPirateAttack___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPirateProbeSignal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPlanetSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardPoliceScanSignal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProcTechProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProceduralProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProceduralProductFromBiome___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProceduralTechnology___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardProductRecipe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRechargeTech___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRefreshHazProt___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRepairTech___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRepairWholeInventory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardRequirementsForRecipe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSalvageShip___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardScan___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardScanEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardScanEventNearestBuilding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardScanEventOutcome___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardSecondaryInteractionOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardIncrementStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardInstallTech___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardInterventionResponse___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardInventorySlots___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardJetpackBoost___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardJourneyThroughCentre___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionMessageSeeded___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionMessageToMatchingSeeds___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionSeeded___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardModifyStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMoney___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMultiSpecificItems___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMultiSpecificProductRecipes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMultiSpecificProducts___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMultiSpecificTechRecipes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardNetworkPlayer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardNexus___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOSDMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOpenFreeFreighter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOpenPage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOpenUnlockTree___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardIncrementStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardInstallTech___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardInterventionResponse___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardInventorySlots___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardJetpackBoost___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardJourneyThroughCentre___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionMessageSeeded___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionMessageToMatchingSeeds___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMissionSeeded___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardModifyStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMoney___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMultiSpecificItems___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMultiSpecificProductRecipes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMultiSpecificProducts___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardMultiSpecificTechRecipes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardNetworkPlayer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardNexus___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOSDMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOpenFreeFreighter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOpenPage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardOpenUnlockTree___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDeactivateFiends___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDeath___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDisableSentinels___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDiscoverRune___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDisguisedProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDisplayTechWindow___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDummyLocID___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEnableInteractionClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEnableSentinels___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEndFrigateFlyby___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEndSettlementExpedition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEnergy___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardExchangeProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFactionStanding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardForceDiscoverSystem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardForgetSpecificProductRecipe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardForgetSpecificTechRecipe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFreeStamina___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFreighterBaseReset___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFreighterMegaWarp___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFreighterSlot___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFrigateDamageResponse___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFrigateFlyby___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardHazard___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardHealth___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardIncrementInteractionIndex___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDeactivateFiends___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDeath___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDisableSentinels___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDiscoverRune___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDisguisedProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDisplayTechWindow___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDummyLocID___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEnableInteractionClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEnableSentinels___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEndFrigateFlyby___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEndSettlementExpedition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardEnergy___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardExchangeProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFactionStanding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardForceDiscoverSystem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardForgetSpecificProductRecipe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardForgetSpecificTechRecipe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFreeStamina___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFreighterBaseReset___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFreighterMegaWarp___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFreighterSlot___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFrigateDamageResponse___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardFrigateFlyby___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardHazard___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardHealth___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardIncrementInteractionIndex___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealitySubstanceData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRecipeTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRefinerRecipe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRefinerRecipeElement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAbortTakeoff___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardActivateEncounterSentinels___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardActivateFiends___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAdvancePortalState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAssessCookedProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAtlasPathProgress___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardBeginSettlementBuilding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCargo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardChangeGlobalMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardClosePortal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCommunicatorMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCommunityContribution___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCompleteMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCompleteMultiMission___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCrashSiteFly___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCrashSiteRepair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCustomExpeditionLogEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCustomPlayerControl___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDamage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDamageTech___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealitySubstanceData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRecipeTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRefinerRecipe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRefinerRecipeElement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAbortTakeoff___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardActivateEncounterSentinels___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardActivateFiends___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAdvancePortalState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAssessCookedProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardAtlasPathProgress___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardBeginSettlementBuilding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCargo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardChangeGlobalMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardClosePortal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCommunicatorMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCommunityContribution___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCompleteMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCompleteMultiMission___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCrashSiteFly___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCrashSiteRepair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCustomExpeditionLogEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardCustomPlayerControl___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDamage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRewardDamageTech___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityIcon___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityIconTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityManagerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealitySubstanceCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealitySubstanceCraftingMix___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityIcon___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityIconTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityManagerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealitySubstanceCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealitySubstanceCraftingMix___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductWord___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralTechnologyCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralTechnologyData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralTechnologyStatLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralTechnologyTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductProceduralOnlyData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPurchaseableBuildingBlueprints___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPurchaseableSpecial___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPurchaseableSpecials___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPuzzleTextFlow___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRarity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityCommonFactions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityCraftingRecipeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityGameIcons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductWord___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralTechnologyCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralTechnologyData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralTechnologyStatLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralTechnologyTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductProceduralOnlyData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProductTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPurchaseableBuildingBlueprints___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPurchaseableSpecial___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPurchaseableSpecials___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPuzzleTextFlow___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRarity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityCommonFactions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityCraftingRecipeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRealityGameIcons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLegacyItemTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLegality___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceElement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceElementGroups___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceGroupEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceGroupsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMultiSpecificItemEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNameGeneratorSectorTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNameGeneratorTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNameGeneratorWord___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNumberedTextList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetResourceIconLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerDamageData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerDamageTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerTitleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductDeployable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLegacyItemTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLegality___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceElement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceElementGroups___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceGroupEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceGroupsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMultiSpecificItemEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNameGeneratorSectorTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNameGeneratorTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNameGeneratorWord___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNumberedTextList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlanetResourceIconLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerDamageData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerDamageTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerTitleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductDeployable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProceduralProductTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryClassProbabilities___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryCostData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryCostDataEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryGenerationBaseStatClassData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryGenerationBaseStatData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryGenerationBaseStatDataEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayoutGenerationBounds___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayoutGenerationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayoutGenerationDataEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayoutSizeType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventorySpecialSlotType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryStackSizeGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryTableEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemAmountCostPair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemCostData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemNeedPurpose___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemPriceModifiers___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLegacyItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryClassProbabilities___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryCostData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryCostDataEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryGenerationBaseStatClassData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryGenerationBaseStatData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryGenerationBaseStatDataEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayoutGenerationBounds___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayoutGenerationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayoutGenerationDataEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayoutSizeType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventorySpecialSlotType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryStackSizeGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryTableEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemAmountCostPair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemCostData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemNeedPurpose___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemPriceModifiers___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLegacyItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostTableEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostWordKnowledge___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRarity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCurrency___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDeathQuote___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDeathStateData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDialogClearanceInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDialogClearanceTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryOwner___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryRewardLookup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryRewardLookupTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryWorth___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionDuration___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionPowerup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionRewardTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterCargoOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateClass___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateStatType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitStrength___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericRewardTableEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryBaseStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryBaseStatBonus___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryBaseStatEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostTableEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostWordKnowledge___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCreatureRarity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCurrency___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDeathQuote___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDeathStateData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDialogClearanceInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDialogClearanceTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryOwner___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryRewardLookup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryRewardLookupTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryWorth___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionDuration___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionPowerup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionRewardTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterCargoOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateClass___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateStatType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFrigateTraitStrength___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericRewardTableEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryBaseStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryBaseStatBonus___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryBaseStatEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostItemFromList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostItemListIndexed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostJourneyMilestone___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostJourneyStatLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostMissionComplete___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostMoney___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostMoneyList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostMultiItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostNPCHabitation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostOwnSettlement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostPendingSettlementJudgement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostPirateTribute___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostPoliceCargoBribe___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostPoliceCargoComply___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostProcProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostProductOnlyTakeIfCanAfford___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostSalvageShip___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostSentinelBlockStatus___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostShipUpgradeable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostSpecificCreatureBait___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostStanding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostStatCompare___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostSubstance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostItemFromList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostItemListIndexed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostJourneyMilestone___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostJourneyStatLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostMissionComplete___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostMoney___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostMoneyList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostMultiItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostNPCHabitation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostOwnSettlement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostPendingSettlementJudgement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostPirateTribute___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostPoliceCargoBribe___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostPoliceCargoComply___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostProcProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostProductOnlyTakeIfCanAfford___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostSalvageShip___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostSentinelBlockStatus___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostShipUpgradeable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostSpecificCreatureBait___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostStanding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostStatCompare___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostSubstance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleTableIndex___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienRace___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienSpeechEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienSpeechTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBreakTechByStatData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingBlueprint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingCostPartCount___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCatalogueGroups___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcConsumableItem___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcConsumableItemTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostAdvanceSettlementBuilding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostAnyCookedProduct___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostBuildingParts___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanAdoptCreature___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanCustomiseCreature___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanFreighterMegaWarp___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanMilkCreature___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanRideCreature___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCargo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCreatureCanLayEggs___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostFleetStoredIncome___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostFrigateCargo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostGameMode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostHasFireteamMember___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostHealth___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostInstalledTech___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostInteractionIndex___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleTableIndex___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienRace___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienSpeechEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienSpeechTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBreakTechByStatData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingBlueprint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingCostPartCount___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCatalogueGroups___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcConsumableItem___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcConsumableItemTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostAdvanceSettlementBuilding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostAnyCookedProduct___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostBuildingParts___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanAdoptCreature___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanCustomiseCreature___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanFreighterMegaWarp___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanMilkCreature___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCanRideCreature___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCargo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostCreatureCanLayEggs___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostFleetStoredIncome___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostFrigateCargo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostGameMode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostHasFireteamMember___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostHealth___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostInstalledTech___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCostInteractionIndex___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienMood___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleMissionOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceDebugTriggerAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceDebugTriggerActionTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceDebugTriggerInput___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceSpawnTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPerformanceFlyby___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPerformanceGuard___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayfabMatchmakingAttributes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienMood___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleMissionOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPuzzleOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceDebugTriggerAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceDebugTriggerActionTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceDebugTriggerInput___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExperienceSpawnTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPerformanceFlyby___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPerformanceGuard___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayfabMatchmakingAttributes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourney___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyCategory___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyCategoryType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMedal___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMedalTiers___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMedalType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageCrime___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageFiendCrime___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageNPCBehaviourEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessagePetBehaviourEvent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageProjectileImpact___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageProjectileLaunch___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageRequestTakeOff___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageRequestWarp___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageSubstanceMined___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageSummonAndDismiss___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageTitanFall___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageTrackTargetAlert___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageUpdateFrigateSpeed___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourney___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyCategory___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyCategoryType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMedal___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMedalTiers___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMedalType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageCrime___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageFiendCrime___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageNPCBehaviourEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessagePetBehaviourEvent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageProjectileImpact___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageProjectileLaunch___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageRequestTakeOff___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageRequestWarp___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageSubstanceMined___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageSummonAndDismiss___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageTitanFall___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageTrackTargetAlert___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMessageUpdateFrigateSpeed___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionSet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionSetAction___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionSetType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionSets___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionUseType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcIKConstraint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActionInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActionInfoMap___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputBinding___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputBindingSet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputBindings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpecialTextStyles___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiStyleAnimationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiStyleAnimationKeyframeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiTextData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStyleProp_Colour___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStyleProp_Font___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStyleProp_Size___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVROverride_Layout___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcVROverride_Text___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionSet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionSetAction___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionSetType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionSets___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcActionUseType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcIKConstraint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActionInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputActionInfoMap___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputBinding___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputBindingSet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInputBindings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpecialTextStyles___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiStyleAnimationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiStyleAnimationKeyframeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiTextData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStyleProp_Colour___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStyleProp_Font___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStyleProp_Size___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVROverride_Layout___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcVROverride_Text___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAccessibleOverride_Layout___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAccessibleOverride_Text___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiActionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiElementData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiFileBrowserRecents___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiGraphicData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiLayerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiLayoutData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiPreset___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiPresetGraphic___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiPresetText___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpacingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpecialTextImageData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpecialTextImages___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpecialTextStyleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipHUDTargetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipHUDTargetIconData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextStyleOutline___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextStylePlain___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextStyleShadow___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAccessibleOverride_Layout___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAccessibleOverride_Text___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiActionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiElementData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiFileBrowserRecents___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiGraphicData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiLayerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiLayoutData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiPreset___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiPresetGraphic___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiPresetText___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpacingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpecialTextImageData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpecialTextImages___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNGuiSpecialTextStyleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipHUDTargetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcShipHUDTargetIconData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextStyleOutline___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextStylePlain___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextStyleShadow___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerIconTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerIcons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceMapObjectData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextPreset___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextPresetTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerIconTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerIcons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpaceMapObjectData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextPreset___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTextPresetTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatIcons___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryHelperTimings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFontData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFontTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFontTypesEnum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericIconTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDComponent___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDEffectRewardData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDImageData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDLayerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDMarkerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDStartup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDStartupTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDTextData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryFilterOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventorySlotActionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcModelViewCollection___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPunctuationDelay___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPunctuationDelayData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPunctuationDelayTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerBuildingIconTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerIcon___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerIconHighlightTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatIcons___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDiscoveryHelperTimings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFontData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFontTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFontTypesEnum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGenericIconTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDComponent___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDEffectRewardData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDImageData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDLayerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDMarkerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDStartup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDStartupTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDTextData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryFilterOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventorySlotActionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcModelViewCollection___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPunctuationDelay___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPunctuationDelayData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPunctuationDelayTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerBuildingIconTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerIcon___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScannerIconHighlightTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBootLogoData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAmbientBuildingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAmbientSpaceData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAmbientSpecialData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAnimationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAnomalySetupData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeCapturedData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeMechanicalData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraSpawnSetupData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDManagerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPresetTextureData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScreenFilterData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScreenFilterTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcScreenFilters___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTexturePrefetchData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWorldUISettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBootLogoData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAmbientBuildingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAmbientSpaceData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAmbientSpecialData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAnimationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAnomalySetupData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeCapturedData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraShakeMechanicalData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraSpawnSetupData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHUDManagerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPresetTextureData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScreenFilterData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScreenFilterTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcScreenFilters___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTexturePrefetchData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWorldUISettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAerialViewData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraFollowSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraWarpSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcColourModifier___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcColourPaletteData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPaletteData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPaletteList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainOverlayColours___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraAerialViewData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraFollowSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCameraWarpSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcColourModifier___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcColourPaletteData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPaletteData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPaletteList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainOverlayColours___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__gGcWaterGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcWaterGlobals__(): pass

def _dynamic_initializer_for__gGcVehicleGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcVehicleGlobals__(): pass

def _dynamic_initializer_for__gGcTerrainGlobals__(): pass

def _dynamic_initializer_for__gGcUIGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcTerrainGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcUIGlobals__(): pass

def _dynamic_initializer_for__saGcUIGlobalsMembers__(): pass

def _dynamic_initializer_for__gGcSpaceshipGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcSpaceshipGlobals__(): pass

def _dynamic_initializer_for__saGcSpaceshipGlobalsMembers__(): pass

def _dynamic_initializer_for__gGcSkyGlobals__(): pass

def _dynamic_initializer_for__gGcSolarGenerationGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcSkyGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcSmokeTestOptions__(): pass

def _dynamic_initializer_for__kRegistercGcSolarGenerationGlobals__(): pass

def _dynamic_initializer_for__gGcSceneOptions__(): pass

def _dynamic_initializer_for__gGcScratchpadGlobals__(): pass

def _dynamic_initializer_for__gGcSettlementGlobals__(): pass

def _dynamic_initializer_for__gGcSimulationGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcSceneOptions__(): pass

def _dynamic_initializer_for__kRegistercGcScratchpadGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcSettlementGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcSimulationGlobals__(): pass

def _dynamic_initializer_for__gGcRobotGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcRichPresenceGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcRobotGlobals__(): pass

def _dynamic_initializer_for__gGcMultiplayerGlobals__(): pass

def _dynamic_initializer_for__gGcPlayerGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcMultiplayerGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcPlacementGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcPlayerGlobals__(): pass

def _dynamic_initializer_for__saGcPlayerGlobalsMembers__(): pass

def _dynamic_initializer_for__gGcGraphicsGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcGraphicsGlobals__(): pass

def _dynamic_initializer_for__gGcGameplayGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcGameplayGlobals__(): pass

def _dynamic_initializer_for__saGcGameplayGlobalsMembers__(): pass

def _dynamic_initializer_for__gGcFreighterBaseGlobals__(): pass

def _dynamic_initializer_for__gGcGalaxyGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcFreighterBaseGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcGalaxyGlobals__(): pass

def _dynamic_initializer_for__gGcEffectsGlobals__(): pass

def _dynamic_initializer_for__gGcEnvironmentGlobals__(): pass

def _dynamic_initializer_for__gGcFleetGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcEffectsGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcEnvironmentGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcFleetGlobals__(): pass

def _dynamic_initializer_for__gGcDebugOptions__(): pass

def _dynamic_initializer_for__kRegistercGcDebugOptions__(): pass

def _dynamic_initializer_for__saGcDebugOptionsMembers__(): pass

def _dynamic_initializer_for__gGcCharacterGlobals__(): pass

def _dynamic_initializer_for__gGcCreatureGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcCharacterGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcCreatureGlobals__(): pass

def _dynamic_initializer_for__saGcCreatureGlobalsMembers__(): pass

def _dynamic_initializer_for__gGcCameraGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcCameraGlobals__(): pass

def _dynamic_initializer_for__gGcAudioGlobals__(): pass

def _dynamic_initializer_for__gGcBuildingGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcAtlasGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcAudioGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcBuildingGlobals__(): pass

def _dynamic_initializer_for__VcGcItemCostTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTelemetryStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcItemCostTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTelemetryStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__gGcAISpaceshipGlobals__(): pass

def _dynamic_initializer_for__kRegistercGcAISpaceshipGlobals__(): pass

def _dynamic_initializer_for__saGcAISpaceshipGlobalsMembers__(): pass

def _dynamic_initializer_for__VcGcJourneyMilestoneData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMilestoneTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLeveledStatData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLeveledStatTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerStat___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerStatsGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerTitle___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatDefinition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatDefinitionTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatDisplayType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatGroupData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatGroupTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatLevelData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatModifyType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatTrackType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatValueData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsAchievements___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsOneShotTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsValueTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMilestoneData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcJourneyMilestoneTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLeveledStatData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLeveledStatTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerStat___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerStatsGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerTitle___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatDefinition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatDefinitionTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatDisplayType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatGroupData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatGroupTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatLevelData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatModifyType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatTrackType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatValueData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsAchievements___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsOneShotTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatsValueTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFriendlyDroneChatType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFriendlyDroneVocabularyEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionBuffer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionBufferType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentTerrainEdits___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetChatType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetVocabularyEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetVocabularyTraitEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetVocabularyWords___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSavedInteractionDialogData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSavedInteractionRaceData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpecialPetChatType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatusMessageDefinition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatusMessageDefinitions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatusMessageMissionMarkup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainEdit___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainEditsBuffer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFriendlyDroneChatType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFriendlyDroneVocabularyEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionBuffer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionBufferType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentTerrainEdits___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetChatType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetVocabularyEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetVocabularyTraitEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetVocabularyWords___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSavedInteractionDialogData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSavedInteractionRaceData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpecialPetChatType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatusMessageDefinition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatusMessageDefinitions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcStatusMessageMissionMarkup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainEdit___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTerrainEditsBuffer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonPetConstraints___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonSaveStateOnDeath___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonStateData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementProductionSlotData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSquadronPilotData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSyncBufferSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSyncBufferSaveDataArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSynchronisedBufferData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSynchronisedBufferType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTeleportEndpoint___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTeleporterType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingSupplyData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUAProtectedLocations___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUniqueIdData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUniverseAddressData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWordGroupKnowledge___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWordKnowledge___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonPetConstraints___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonSaveStateOnDeath___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonStateData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementProductionSlotData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSquadronPilotData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSyncBufferSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSyncBufferSaveDataArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSynchronisedBufferData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSynchronisedBufferType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTeleportEndpoint___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTeleporterType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTradingSupplyData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUAProtectedLocations___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUniqueIdData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUniverseAddressData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWordGroupKnowledge___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWordKnowledge___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPortalSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcProtectedLocation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRepairTechData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSavedEntitlement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalGameModeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalMilestone___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalMilestoneEncryption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalRingArray___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalRingData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalStage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPortalSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcProtectedLocation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRepairTechData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSavedEntitlement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalGameModeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalMilestone___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalMilestoneEncryption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalRingArray___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalRingData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSeasonalStage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpawnStateData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerStateData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerSpawnStateData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerStateData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryStoreBalance___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcKnownThingsPreset___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceContainer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceSaveKey___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcModSettings___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcModSettingsInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcMultitoolData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistedStatData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetCustomisationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionParticipant___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionParticipantType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionProgress___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionProgressMapEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionProgressMapTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionUpgradeMapEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionUpgradeMapTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerOwnershipData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryStoreBalance___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcKnownThingsPreset___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceContainer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMaintenanceSaveKey___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcModSettings___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcModSettingsInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcMultitoolData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistedStatData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetCustomisationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPetData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionParticipant___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionParticipantType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionProgress___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionProgressMapEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionProgressMapTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionUpgradeMapEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerMissionUpgradeMapTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerOwnershipData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationColourPaletteExtraData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationColourPalettes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationDescriptorGroups___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationPreset___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationPresets___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationTextureOptions___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionEventSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFleetExpeditionSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFleetFrigateSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalacticAddressData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGameMode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryContainer___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryElement___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryIndex___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayout___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventorySpecialSlot___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationColourPaletteExtraData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationColourPalettes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationDescriptorGroups___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationPreset___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationPresets___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationTextureOptions___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDefaultSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcExpeditionEventSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFleetExpeditionSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFleetFrigateSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGalacticAddressData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGameMode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryContainer___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryElement___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryIndex___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventoryLayout___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInventorySpecialSlot___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationBoneScaleData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationColourData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationSaveData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationTextureOptionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationDescriptorGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationTextureOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterNPCSpawnPriority___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterNPCType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterRoomNPCData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterRoomNPCSpawnCapacities___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterRoomNPCSpawnCapacityEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCNavSubgraphNodeType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCNavSubgraphNodeTypeConnectivity___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBBObjectData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBaseEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBaseTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSnapPointCondition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationBoneScaleData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationColourData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationSaveData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCharacterCustomisationTextureOptionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationDescriptorGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCustomisationTextureOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterNPCSpawnPriority___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterNPCType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterRoomNPCData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterRoomNPCSpawnCapacities___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterRoomNPCSpawnCapacityEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCNavSubgraphNodeType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCNavSubgraphNodeTypeConnectivity___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBBObjectData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBaseEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBaseTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSnapPointCondition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseSnapState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseStatCondition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeCondition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildMenuIconSet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildMenuOption___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingModeCondition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingPlacementErrorTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterDungeonsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseDecorationTemplate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseLockDoorPair___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBasePruningRule___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseRoomTemplate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseStructuralTemplate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseTemplatesTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseThemeTemplate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGroupCondition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcId256List___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLinkNetworkTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcModBasePart___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCHabitationType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCWorkerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcOutSnapSocketCondition___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBase___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseSnapState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseStatCondition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBiomeCondition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildMenuIconSet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildMenuOption___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingModeCondition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingPlacementErrorTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFreighterDungeonsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseDecorationTemplate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseLockDoorPair___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBasePruningRule___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseRoomTemplate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseStructuralTemplate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseTemplatesTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGeneratedBaseThemeTemplate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGroupCondition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcId256List___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLinkNetworkTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcModBasePart___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCHabitationType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCWorkerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcOutSnapSocketCondition___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistentBase___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartNavData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartNavNodeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartStyle___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartStyleModel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartsDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartsNavDataTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingProperties___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingSubGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseGridSearchFilter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseLinkGridConnectionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseLinkGridConnectionDependency___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseLinkGridData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBasePartSearchFilter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBasePlacementRule___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseSearchFilter___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartNavData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartNavNodeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartStyle___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartStyleModel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartsDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartsNavDataTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingProperties___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingSubGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseGridSearchFilter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseLinkGridConnectionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseLinkGridConnectionDependency___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseLinkGridData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBasePartSearchFilter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBasePlacementRule___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseSearchFilter___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseAutoPowerSetting___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingCostsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingEntryCosts___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingEntryGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingFamily___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingMaterial___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingMode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingObjectDecorationTypes___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPalette___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPart___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartAudioLocationEntry___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartAudioLocationTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartInteractionData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseAutoPowerSetting___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingCostsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingEntryCosts___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingEntryGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingFamily___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingMaterial___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingMode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingObjectDecorationTypes___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPalette___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPart___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartAudioLocationEntry___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartAudioLocationTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseBuildingPartInteractionData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnSequenceGroupList___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnSequenceStep___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnWave___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelWaveGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementConstructionLevel___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJudgementType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSimpleIkRecoilComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSimpleInteractionComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnedObjectComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcThereminComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcTurretComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcUniqueNPCSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcWiringSocketComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnSequenceGroupList___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnSequenceStep___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnWave___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelWaveGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementConstructionLevel___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementJudgementType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSettlementStatType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSimpleIkRecoilComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSimpleInteractionComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSpawnedObjectComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcThereminComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcTurretComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcUniqueNPCSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcWiringSocketComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPlacementComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPlacementInfo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectPlacementComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectSpawnerComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistencyMissionOverride___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceAsteroidCreatureSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceAsteroidCreatureSpawnTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceSpawnArchetypeData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceSpawnTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPortalComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcPortalData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelCoverState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnNamedSequence___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnSequence___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnSequenceGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPlacementComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcNPCPlacementInfo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectPlacementComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcObjectSpawnerComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPersistencyMissionOverride___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceAsteroidCreatureSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceAsteroidCreatureSpawnTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceSpawnArchetypeData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPlayerExperienceSpawnTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPortalComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcPortalData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelCoverState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnNamedSequence___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnSequence___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcSentinelSpawnSequenceGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDistanceScaleComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneGun___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEncounterType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFiendCrime___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFiendCrimeSpawnData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcFiendCrimeSpawnTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGrabPlayerComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGrabbableComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcGroundWormComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeightAdjustComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcImpactCombatEffectData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionActivationCost___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionBaseBuildingState___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLandingHelperComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcLootComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcDistanceScaleComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcDroneGun___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEncounterType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFiendCrime___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFiendCrimeSpawnData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcFiendCrimeSpawnTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGrabPlayerComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGrabbableComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcGroundWormComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcHeightAdjustComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcImpactCombatEffectData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionActivationCost___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcInteractionBaseBuildingState___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLandingHelperComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcLootComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPodComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistEnemy___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistFriend___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistGroup___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistPerception___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtmosphereEntryComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseDefenceComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseMiniPortalComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBasePlacementComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatSwitchComponentData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcConstraintsToCreateSpec___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEntitlementRewardData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcEntitlementRewardsTable___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcRectangularInitialShape___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcTkEntitlementListData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPodComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistEnemy___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistFriend___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistGroup___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAntagonistPerception___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtmosphereEntryComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseDefenceComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBaseMiniPortalComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBasePlacementComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBuildingComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatSwitchComponentData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcCombatEffectsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcConstraintsToCreateSpec___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEntitlementRewardData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcEntitlementRewardsTable___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcRectangularInitialShape___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcTkEntitlementListData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcAxisSpecification___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAdvancedTweaks___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAttachedNode___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAttachmentPointData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAttachmentPointSet___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatEnvelope___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatLibraryData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatSong___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatTemplate___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatTemplates___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatToken___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatWave___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothAttachmentCirlce___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothAttachmentLine___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothCollisionBone___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothPiece___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcAxisSpecification___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAdvancedTweaks___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAttachedNode___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAttachmentPointData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAttachmentPointSet___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatEnvelope___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatLibraryData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatSong___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatTemplate___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatTemplates___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatToken___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatWave___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothAttachmentCirlce___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothAttachmentLine___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothCollisionBone___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcClothPiece___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudio3PointDopplerData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioNPCDoppler___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioPulseDemo___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioWwiseEvents___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioWwiseRTPCs___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcBasePartAudioLocation___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatDrum___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudio3PointDopplerData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioNPCDoppler___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioPulseDemo___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioWwiseEvents___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAudioWwiseRTPCs___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcBasePartAudioLocation___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcByteBeatDrum___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasSendSubmitContribution___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasSendSubmitContribution___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPodAnimParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasActiveFeaturedBaseHeader___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasBase___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasBaseQuery___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasBasesRequest___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasDiscovery___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasDiscoveryData___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasDiscoveryMetadata___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasDownloadType___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasFeaturedBaseStateChange___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasMessage___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasMonument___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcThirdPersonAnimParams___kMetadataRegistrar__(): pass

def _dynamic_initializer_for__VcGcAlienPodAnimParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasActiveFeaturedBaseHeader___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasBase___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasBaseQuery___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasBasesRequest___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasDiscovery___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasDiscoveryData___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasDiscoveryMetadata___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasDownloadType___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasFeaturedBaseStateChange___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasMessage___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcAtlasMonument___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__VcGcThirdPersonAnimParams___kMetadataXmlRegistrar__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___96(): pass

def _dynamic_initializer_for__k3x4IdentityPos___92(): pass

def _dynamic_initializer_for__k3x4IdentityRight___96(): pass

def _dynamic_initializer_for__k3x4IdentityUp___96(): pass

def _dynamic_initializer_for__k4x4IdentityAt___18(): pass

def _dynamic_initializer_for__k4x4IdentityPos___18(): pass

def _dynamic_initializer_for__k4x4IdentityRight___18(): pass

def _dynamic_initializer_for__k4x4IdentityUp___18(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___17(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___17(): pass

def _dynamic_initializer_for__mCache___16(): pass

def hkMemory__Detail___dynamic_initializer_for__s_32alignedHeapAllocator__(): pass

def hkMemory__Detail___dynamic_initializer_for__s_64alignedHeapAllocator__(): pass

def hkMemory__Detail___dynamic_initializer_for__s_containerAllocators__(): pass

def _dynamic_initializer_for__cLocale__(): pass

def _dynamic_initializer_for__hkReferencedObjectLock__singleton__(): pass

def _dynamic_initializer_for__hkReferencedObjectLock__singletonEntry__(): pass

def _dynamic_initializer_for__hkRefCountedProperties__Entry___Auto__m_object__defaultType__def__(): pass

def _dynamic_initializer_for__hkColorf___Auto__defaultVal__(): pass

def _dynamic_initializer_for__hkRelArray_ArrayImpl___int64___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__Detail__Opaque___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__Type___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkLocalFrame___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkLocalFrameGroup___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkQTransformf___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkQuaternionf___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReferencedObject___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkStringBuf___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkTransformf___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkVector4f___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__Detail__Reference_hkQTransformf_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__Detail__Reference_hkQuaternionf_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__Detail__Reference_hkStringBuf_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__Detail__Reference_hkTransformf_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__Detail__Reference_hkVector4f_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueArray_char_const_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueArray_hk__Detail__VersioningDependentType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueAttribute_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueAttribute_char_const_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueAttribute_hkReflect__Type_const_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueAttribute_hk__ValueArray_char_const_________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueAttribute_hk__ValueArray_hk__Detail__VersioningDependentType_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueAttribute_hkReflect__Typedef__hkUlong_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueAttribute_enum_hkms__AccumulateOp__Enum_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hk__ValueAttribute_unsigned___int64_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkBitFieldStorage_hkArray_unsigned_int_hkBuiltinContainerAllocator_0_0_________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkBuiltinContainerAllocator_0_0_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkMapOperations_unsigned_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkMapOperations_hkReflect__Typedef__hkUint32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkMapOperations_hkReflect__Typedef__hkUint64_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkMapOperations_unsigned___int64_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkOffsetBitFieldStorage_hkArray_unsigned_int_hkBuiltinContainerAllocator_0_0_________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_unsigned_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkReflect__Typedef__hkUint32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__Type_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_unsigned_char_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_unsigned_int_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkLocalFrame___hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMonitorStreamColorTable__ColorPair_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkRefCountedProperties__Entry_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMonitorStreamStringMap__StringMap_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkGeometry__Triangle_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMonitorStreamTypeMap__TypeMap_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkIntRealPair_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkSimpleLocalFrame_ChildPointer_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkUint32_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkUint64_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkUint8_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkVector4_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkGpuTraceResult_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkVector4f_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_enum_hkAsyncThreadPool__ThreadPriority_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_unsigned___int64_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkBitFieldBase_hkBitFieldStorage_hkArray_unsigned_int_hkBuiltinContainerAllocator_0_0___________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkBitFieldBase_hkOffsetBitFieldStorage_hkArray_unsigned_int_hkBuiltinContainerAllocator_0_0___________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMonitorStreamFrameInfo__AbsoluteTimeCounter_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMonitorStreamFrameInfo__AbsoluteTimeCounter_hkReflect__Typedef__hkInt32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkGizmoAttribute__GizmoType_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkGizmoAttribute__GizmoType_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkLinkAttribute__Link_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkLinkAttribute__Link_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkModelerNodeTypeAttribute__ModelerType_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkModelerNodeTypeAttribute__ModelerType_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkCommand__PrimaryType_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkCommand__PrimaryType_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkGpuTraceResult__ScopeType_unsigned_short_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkGpuTraceResult__ScopeType_hkReflect__Typedef__hkUint16_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkSemanticsAttribute__Semantics_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkSemanticsAttribute__Semantics_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkAttributeHideCriteria__Types_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkAttributeHideCriteria__Types_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hkReflect__DeclFlagBits_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFourTransposedPointsImpl_float_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFourTransposedPointsImpl_double_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkMatrix3Impl_float_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkMatrix3Impl_double_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkMatrix4Impl_float_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkMatrix4Impl_double_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkReferencedObject_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRotationImpl_float_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRotationImpl_double_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkSet_unsigned_int_hkBuiltinContainerAllocator_0_0__hkMapOperations_unsigned_int_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkSet_hkIntRealPair_hkBuiltinContainerAllocator_0_0__hkMapOperations_hkIntRealPair_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkSet_hkReflect__Typedef__hkUint32_Tag_hkBuiltinContainerAllocator_0_0__hkMapOperations_hkReflect__Typedef__hkUint32_Tag_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkSet_hkReflect__Typedef__hkUint64_Tag_hkBuiltinContainerAllocator_0_0__hkMapOperations_hkReflect__Typedef__hkUint64_Tag_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkSet_unsigned___int64_hkBuiltinContainerAllocator_0_0__hkMapOperations_unsigned___int64_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkViewPtr_hkLocalFrame_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkViewPtr_hkReferencedObject_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_signed_char_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_char_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_char_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_short_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_short_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_short_8___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_int_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_int_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_int_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkInt16_Tag_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkPlane_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint16_Tag_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint16_Tag_8___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint32_Tag_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint32_Tag_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint32_Tag_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint8_Tag_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUintReal_Tag_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkFloat16_12___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkVector4d_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkVector4f_3___typeRegNode__(): pass

def hkFpMath__Detail___dynamic_initializer_for__MULTI_WORD_RADIX__(): pass

def _dynamic_initializer_for__hkAabb16_typeRegNode__(): pass

def _dynamic_initializer_for__hkAabbFloat3_typeRegNode__(): pass

def _dynamic_initializer_for__hkAabbHalf_typeRegNode__(): pass

def _dynamic_initializer_for__hkAabbUint32_typeRegNode__(): pass

def _dynamic_initializer_for__hkAabb_typeRegNode__(): pass

def _dynamic_initializer_for__hkAsyncThreadPool_Cinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hkAsyncThreadPool_HardwareThreadBinding_typeRegNode__(): pass

def _dynamic_initializer_for__hkAsyncThreadPool_ThreadPriority_typeRegNode__(): pass

def _dynamic_initializer_for__hkAttributeHideCriteria_Types_typeRegNode__(): pass

def _dynamic_initializer_for__hkAttributeHideCriteria_typeRegNode__(): pass

def _dynamic_initializer_for__hkAxialRotation_typeRegNode__(): pass

def _dynamic_initializer_for__hkAxialTransform_typeRegNode__(): pass

def _dynamic_initializer_for__hkBFloat16_4_typeRegNode__(): pass

def _dynamic_initializer_for__hkBaseObject_typeRegNode__(): pass

def _dynamic_initializer_for__hkBitField_typeRegNode__(): pass

def _dynamic_initializer_for__hkBool32Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkBool32Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkBool8Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkBool8Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkBool_typeRegNode__(): pass

def _dynamic_initializer_for__hkBuiltinContainerAllocatorBase_typeRegNode__(): pass

def _dynamic_initializer_for__hkColorUbBase_typeRegNode__(): pass

def _dynamic_initializer_for__hkColorUbGamma_typeRegNode__(): pass

def _dynamic_initializer_for__hkColorUbLinear_typeRegNode__(): pass

def _dynamic_initializer_for__hkColor_Argb_typeRegNode__(): pass

def _dynamic_initializer_for__hkColor_typeRegNode__(): pass

def _dynamic_initializer_for__hkColorfAlpha_typeRegNode__(): pass

def _dynamic_initializer_for__hkColorf_typeRegNode__(): pass

def _dynamic_initializer_for__hkCommand_PrimaryType_typeRegNode__(): pass

def _dynamic_initializer_for__hkCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hkContactPoint_typeRegNode__(): pass

def _dynamic_initializer_for__hkContainerAllocator_typeRegNode__(): pass

def _dynamic_initializer_for__hkCpuThreadPool_typeRegNode__(): pass

def _dynamic_initializer_for__hkDefaultTaskQueue_typeRegNode__(): pass

def _dynamic_initializer_for__hkDescriptionAttribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkDocumentationAttribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkDouble3_typeRegNode__(): pass

def _dynamic_initializer_for__hkFileSystem_Watcher_Change_ChangeType_typeRegNode__(): pass

def _dynamic_initializer_for__hkFloat16Transform_typeRegNode__(): pass

def _dynamic_initializer_for__hkFloat16_typeRegNode__(): pass

def _dynamic_initializer_for__hkFloat3Pair_typeRegNode__(): pass

def _dynamic_initializer_for__hkFloat3_typeRegNode__(): pass

def _dynamic_initializer_for__hkFloat4_typeRegNode__(): pass

def _dynamic_initializer_for__hkFourTransposedPointsd_typeRegNode__(): pass

def _dynamic_initializer_for__hkFourTransposedPointsf_typeRegNode__(): pass

def _dynamic_initializer_for__hkFpMath_ConstantOne_typeRegNode__(): pass

def _dynamic_initializer_for__hkFrustum_typeRegNode__(): pass

def _dynamic_initializer_for__hkGeometry_Triangle_typeRegNode__(): pass

def _dynamic_initializer_for__hkGeometry_typeRegNode__(): pass

def _dynamic_initializer_for__hkGizmoAttribute_GizmoType_typeRegNode__(): pass

def _dynamic_initializer_for__hkGizmoAttribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkGpuTraceResult_ScopeType_typeRegNode__(): pass

def _dynamic_initializer_for__hkGpuTraceResult_typeRegNode__(): pass

def _dynamic_initializer_for__hkHalf16_typeRegNode__(): pass

def _dynamic_initializer_for__hkInetAddr_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt16_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt32_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt64_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt8_typeRegNode__(): pass

def _dynamic_initializer_for__hkIntRealPair_typeRegNode__(): pass

def _dynamic_initializer_for__hkIntVector_typeRegNode__(): pass

def _dynamic_initializer_for__hkJobType_typeRegNode__(): pass

def _dynamic_initializer_for__hkLinkAttribute_Link_typeRegNode__(): pass

def _dynamic_initializer_for__hkLinkAttribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkLocalFrameGroup_typeRegNode__(): pass

def _dynamic_initializer_for__hkLocalFrame_typeRegNode__(): pass

def _dynamic_initializer_for__hkLog_Level_typeRegNode__(): pass

def _dynamic_initializer_for__hkLong_typeRegNode__(): pass

def _dynamic_initializer_for__hkMapOperationshkIntRealPair_typeRegNode__(): pass

def _dynamic_initializer_for__hkMatrix3_typeRegNode__(): pass

def _dynamic_initializer_for__hkMatrix3d_typeRegNode__(): pass

def _dynamic_initializer_for__hkMatrix3f_typeRegNode__(): pass

def _dynamic_initializer_for__hkMatrix4_typeRegNode__(): pass

def _dynamic_initializer_for__hkMatrix4d_typeRegNode__(): pass

def _dynamic_initializer_for__hkMatrix4f_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemory_DebugFlags_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemory_DebugSettings_typeRegNode__(): pass

def _dynamic_initializer_for__hkModelerNodeTypeAttribute_ModelerType_typeRegNode__(): pass

def _dynamic_initializer_for__hkModelerNodeTypeAttribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamColorTable_ColorPair_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamColorTable_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamContainer_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamFrameInfo_AbsoluteTimeCounter_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamFrameInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamStringMap_StringMap_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamStringMap_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamTypeMap_TypeMap_typeRegNode__(): pass

def _dynamic_initializer_for__hkMonitorStreamTypeMap_typeRegNode__(): pass

def _dynamic_initializer_for__hkMpRational_typeRegNode__(): pass

def _dynamic_initializer_for__hkMpUint_typeRegNode__(): pass

def _dynamic_initializer_for__hkMultiThreadCheck_typeRegNode__(): pass

def _dynamic_initializer_for__hkOffsetBitField_typeRegNode__(): pass

def _dynamic_initializer_for__hkOptionParser_OptionType_typeRegNode__(): pass

def _dynamic_initializer_for__hkPackedUnitVector8_4_typeRegNode__(): pass

def _dynamic_initializer_for__hkPackedVector3_typeRegNode__(): pass

def _dynamic_initializer_for__hkPackedVector4_6_typeRegNode__(): pass

def _dynamic_initializer_for__hkPackedVector8_3_typeRegNode__(): pass

def _dynamic_initializer_for__hkPlane_typeRegNode__(): pass

def _dynamic_initializer_for__hkPreciseTransform_typeRegNode__(): pass

def _dynamic_initializer_for__hkPseudoRandomGenerator_typeRegNode__(): pass

def _dynamic_initializer_for__hkQTransform_typeRegNode__(): pass

def _dynamic_initializer_for__hkQTransformd_typeRegNode__(): pass

def _dynamic_initializer_for__hkQTransformf_typeRegNode__(): pass

def _dynamic_initializer_for__hkQsTransform_typeRegNode__(): pass

def _dynamic_initializer_for__hkQsTransformd_typeRegNode__(): pass

def _dynamic_initializer_for__hkQsTransformf_typeRegNode__(): pass

def _dynamic_initializer_for__hkQuaternion_typeRegNode__(): pass

def _dynamic_initializer_for__hkQuaterniond_typeRegNode__(): pass

def _dynamic_initializer_for__hkQuaternionf_typeRegNode__(): pass

def _dynamic_initializer_for__hkRadians_typeRegNode__(): pass

def _dynamic_initializer_for__hkRangeInt32Attribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkRangeRealAttribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkReal_typeRegNode__(): pass

def _dynamic_initializer_for__hkRefCountedProperties_Entry_typeRegNode__(): pass

def _dynamic_initializer_for__hkRefCountedProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hkRefTaskQueue_typeRegNode__(): pass

def _dynamic_initializer_for__hkRefVariant_typeRegNode__(): pass

def _dynamic_initializer_for__hkReferencedObject_typeRegNode__(): pass

def _dynamic_initializer_for__hkReferencedTask_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Any_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_ArrayType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_ArrayVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_AttributeArrayWithOwnership_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_BoolType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_BoolVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_CompoundType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_CompoundVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_DataFieldDecl_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_DeclFlagBits_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_DeclFlags_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Decl_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Detail_Opaque_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Detail_SizeAlign_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Detail_StringOut_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_FieldDecl_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_FloatType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_FloatVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_IntType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_IntVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Kind_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_OpaqueType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_OpaqueVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Opt_Values_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_PointerType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_PointerVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_PropertyFieldDecl_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_RecordType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_RecordVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_StringType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_StringVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_TypeName_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Type_FlagBits_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Type_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_ValueType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_ValueVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_Var_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_VoidType_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect_VoidVar_typeRegNode__(): pass

def _dynamic_initializer_for__hkRelBitField_typeRegNode__(): pass

def _dynamic_initializer_for__hkRelOffsetBitField_typeRegNode__(): pass

def _dynamic_initializer_for__hkResult_typeRegNode__(): pass

def _dynamic_initializer_for__hkRotation_typeRegNode__(): pass

def _dynamic_initializer_for__hkRotationd_typeRegNode__(): pass

def _dynamic_initializer_for__hkRotationf_typeRegNode__(): pass

def _dynamic_initializer_for__hkSemanticsAttribute_Semantics_typeRegNode__(): pass

def _dynamic_initializer_for__hkSemanticsAttribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkSetIntFloatPair_typeRegNode__(): pass

def _dynamic_initializer_for__hkSetUint32_typeRegNode__(): pass

def _dynamic_initializer_for__hkSetUint64_typeRegNode__(): pass

def _dynamic_initializer_for__hkSignal_typeRegNode__(): pass

def _dynamic_initializer_for__hkSimdDouble64_typeRegNode__(): pass

def _dynamic_initializer_for__hkSimdFloat32_typeRegNode__(): pass

def _dynamic_initializer_for__hkSimdReal_typeRegNode__(): pass

def _dynamic_initializer_for__hkSimpleLocalFrame_ChildPointer_typeRegNode__(): pass

def _dynamic_initializer_for__hkSimpleLocalFrame_typeRegNode__(): pass

def _dynamic_initializer_for__hkSimplePropertyValue_typeRegNode__(): pass

def _dynamic_initializer_for__hkSimpleProperty_typeRegNode__(): pass

def _dynamic_initializer_for__hkSphere_typeRegNode__(): pass

def _dynamic_initializer_for__hkStringBuf_typeRegNode__(): pass

def _dynamic_initializer_for__hkStringPtr_typeRegNode__(): pass

def _dynamic_initializer_for__hkStringView_typeRegNode__(): pass

def _dynamic_initializer_for__hkTaskQueue_typeRegNode__(): pass

def _dynamic_initializer_for__hkTask_typeRegNode__(): pass

def _dynamic_initializer_for__hkThreadPool_typeRegNode__(): pass

def _dynamic_initializer_for__hkThread_Priority_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerProvider_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerVolumeTag_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerVolumeUtils_Aabb_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerVolumeUtils_BaseVolume_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerVolumeUtils_Capsule_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerVolumeUtils_Line_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerVolumeUtils_Point_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerVolumeUtils_Sphere_typeRegNode__(): pass

def _dynamic_initializer_for__hkTimerVolume_typeRegNode__(): pass

def _dynamic_initializer_for__hkTrait_DivideshkVector4f_typeRegNode__(): pass

def _dynamic_initializer_for__hkTrait_EqualTohkReferencedObjectPtr_typeRegNode__(): pass

def _dynamic_initializer_for__hkTrait_NotEqualTohkReferencedObjectPtr_typeRegNode__(): pass

def _dynamic_initializer_for__hkTransform_typeRegNode__(): pass

def _dynamic_initializer_for__hkTransformd_typeRegNode__(): pass

def _dynamic_initializer_for__hkTransformf_typeRegNode__(): pass

def _dynamic_initializer_for__hkTuplevoidvoidvoidvoidvoidvoidvoidvoid_typeRegNode__(): pass

def _dynamic_initializer_for__hkTypeVm_InstructionKind_typeRegNode__(): pass

def _dynamic_initializer_for__hkUFloat8_typeRegNode__(): pass

def _dynamic_initializer_for__hkUiAttribute_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint16_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint32_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint64_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint8_typeRegNode__(): pass

def _dynamic_initializer_for__hkUintReal_typeRegNode__(): pass

def _dynamic_initializer_for__hkUlong_typeRegNode__(): pass

def _dynamic_initializer_for__hkUuid_typeRegNode__(): pass

def _dynamic_initializer_for__hkVariantArray_typeRegNode__(): pass

def _dynamic_initializer_for__hkVdbCmdTag_typeRegNode__(): pass

def _dynamic_initializer_for__hkVector2_typeRegNode__(): pass

def _dynamic_initializer_for__hkVector2d_typeRegNode__(): pass

def _dynamic_initializer_for__hkVector2f_typeRegNode__(): pass

def _dynamic_initializer_for__hkVector4_typeRegNode__(): pass

def _dynamic_initializer_for__hkVector4d_typeRegNode__(): pass

def _dynamic_initializer_for__hkVector4f_typeRegNode__(): pass

def _dynamic_initializer_for__hkViewport_typeRegNode__(): pass

def _dynamic_initializer_for__hk_AbsMax_typeRegNode__(): pass

def _dynamic_initializer_for__hk_AbsMin_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Abstract_typeRegNode__(): pass

def _dynamic_initializer_for__hk_AddPrebuildWarning_typeRegNode__(): pass

def _dynamic_initializer_for__hk_AlignReal_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Allocatable_typeRegNode__(): pass

def _dynamic_initializer_for__hk_AutoType_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Cast_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ContainsRelArrays_typeRegNode__(): pass

def _dynamic_initializer_for__hk_DefaultStringType_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Default_typeRegNode__(): pass

def _dynamic_initializer_for__hk_DeleteTypeInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Detail_VersioningDependentTypeList_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Detail_VersioningDependentType_typeRegNode__(): pass

def _dynamic_initializer_for__hk_DocString_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ExactTypeFunction_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ExcludeFromVersionCheck_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ExternalReflection_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Feature_typeRegNode__(): pass

def _dynamic_initializer_for__hk_FixupReflectedMethods_typeRegNode__(): pass

def _dynamic_initializer_for__hk_GenerateBypassCtor_typeRegNode__(): pass

def _dynamic_initializer_for__hk_GenerateExactTypeMethod_typeRegNode__(): pass

def _dynamic_initializer_for__hk_HasOwnVtable_typeRegNode__(): pass

def _dynamic_initializer_for__hk_IgnoreTemplateParams_typeRegNode__(): pass

def _dynamic_initializer_for__hk_IsRelArray_typeRegNode__(): pass

def _dynamic_initializer_for__hk_IsValueArray_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Map_typeRegNode__(): pass

def _dynamic_initializer_for__hk_NonNull_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Normalized_typeRegNode__(): pass

def _dynamic_initializer_for__hk_OpaqueType_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Optional_typeRegNode__(): pass

def _dynamic_initializer_for__hk_OriginalName_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ParsingDecl_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Pod_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Presets_ParsingDecl_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Presets_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ReflectChildren_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ReflectDetails_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Reflect_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ReflectionImpl_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Rename_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Retargetable_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Serialize_typeRegNode__(): pass

def _dynamic_initializer_for__hk_SetsField_typeRegNode__(): pass

def _dynamic_initializer_for__hk_ToString_typeRegNode__(): pass

def _dynamic_initializer_for__hk_TypeLayout_typeRegNode__(): pass

def _dynamic_initializer_for__hk_TypeVmCompileCallback_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Type_typeRegNode__(): pass

def _dynamic_initializer_for__hk_UnregisteredType_typeRegNode__(): pass

def _dynamic_initializer_for__hk_Version_typeRegNode__(): pass

def _dynamic_initializer_for__hkms_AccumulateOp_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hkms_Accumulate_typeRegNode__(): pass

def hkMemory__Signals___dynamic_initializer_for__m_garbageCollected__(): pass

def _dynamic_initializer_for__g_vectordConstants__(): pass

def _dynamic_initializer_for__g_vectorfConstants__(): pass

def _anonymous_namespace____dynamic_initializer_for__baseSystemLogOrigin__(): pass

def _anonymous_namespace____dynamic_initializer_for__s_externalAllocatorFwd__(): pass

def _dynamic_initializer_for__hkMultiThreadCheck__s_stackTracer__(): pass

def _anonymous_namespace____dynamic_initializer_for__ReflectionLog__(): pass

def _dynamic_initializer_for__bool_typeRegNode__(): pass

def _dynamic_initializer_for__charstar_typeRegNode__(): pass

def _dynamic_initializer_for__constcharstar_typeRegNode__(): pass

def _dynamic_initializer_for__double_typeRegNode__(): pass

def _dynamic_initializer_for__float_typeRegNode__(): pass

def _dynamic_initializer_for__int219__(): pass

def _dynamic_initializer_for__int220__(): pass

def _dynamic_initializer_for__int221__(): pass

def _dynamic_initializer_for__int222__(): pass

def _dynamic_initializer_for__int223__(): pass

def _dynamic_initializer_for__int224__(): pass

def _dynamic_initializer_for__int225__(): pass

def _dynamic_initializer_for__int226__(): pass

def _dynamic_initializer_for__int227__(): pass

def _dynamic_initializer_for__int228__(): pass

def _dynamic_initializer_for__int268__(): pass

def _dynamic_initializer_for__nullptr_t_typeRegNode__(): pass

def _dynamic_initializer_for__s_debugLog__(): pass

def _dynamic_initializer_for__void_typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__BoolImplN_unsigned_char___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__BoolImplN_unsigned_short___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__BoolImplN_unsigned_int___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__BoolImplN_bool___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__FloatImplN_float___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__FloatImplN_double___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__FloatImplN_hkHalf16___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_signed_char___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_char___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_unsigned_char___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_short___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_unsigned_short___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_int___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_unsigned_int___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_long___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_unsigned_long___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN___int64___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__IntImplN_unsigned___int64___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__CharBufferImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__HeapAllocImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__NullptrImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RawPointerImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__ReferenceImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__ReferencedObjectPointerImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__StaticStringImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_void___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__DerivedDataCache__singleton__(): pass

def _dynamic_initializer_for__hkReflect__Detail__DerivedDataCache__singletonEntry__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__Var___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__Detail__Reference_hkReflect__Var_____typeRegNode__(): pass

def _dynamic_initializer_for__s_debugLog___0(): pass

def _dynamic_initializer_for__hkScalarF_FloatImpl__s_instance__(): pass

def _dynamic_initializer_for__hkSimdDouble64_FloatImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Any__s_impl__(): pass

def _dynamic_initializer_for__hkReflect__AttributeArrayWithOwnership__s_allocImplHeapInstance__(): pass

def _dynamic_initializer_for__s_debugLog___1(): pass

def _dynamic_initializer_for__hkRelArray_ArrayImpl_int___s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__HavokRecordImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__HavokStringImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__HomogeneousArrayImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__ReferencedObjectAllocImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__StaticArrayImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__ViewPointerImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__hkArrayImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__hkInplaceArrayImpl__s_instance__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_char_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_char___int64_____typeRegNode__(): pass

def _dynamic_initializer_for__s_debugLog___2(): pass

def _dynamic_initializer_for__hkReflect__Var__s_impl__(): pass

def _dynamic_initializer_for__s_debugLog___3(): pass

def _dynamic_initializer_for__hkStringPtr__Empty__(): pass

def _dynamic_initializer_for__hkStringPtr__s_impl__(): pass

def _dynamic_initializer_for__hkReflect__Detail__AttributeAllocImpl__s_heapInstance__(): pass

def _dynamic_initializer_for__hk__IsValueArray__s_impl__(): pass

def _dynamic_initializer_for__s_debugLog___4(): pass

def _dynamic_initializer_for__hkStringBuf__s_impl__(): pass

def _dynamic_initializer_for__s_debugLog___5(): pass

def _dynamic_initializer_for__hkStringView__s_impl__(): pass

def _dynamic_initializer_for__hkReflect__Detail__StringOut__s_impl__(): pass

def _dynamic_initializer_for__hkUuidPseudoRandomGenerator__singleton__(): pass

def _dynamic_initializer_for__hkUuidPseudoRandomGenerator__singletonEntry__(): pass

def _dynamic_initializer_for__s_initTypeReg__(): pass

def _dynamic_initializer_for__s_debugLog___6(): pass

def _dynamic_initializer_for__s_debugLog___7(): pass

def _dynamic_initializer_for__s_debugLog___8(): pass

def _dynamic_initializer_for__hkMemory__Detail__FwdToSharedAllocator__s_instance__(): pass

def hkMemory__Signals___dynamic_initializer_for__m_memoryAllocated__(): pass

def hkMemory__Signals___dynamic_initializer_for__m_memoryDeallocated__(): pass

def hkMemory__Signals___dynamic_initializer_for__m_memoryLimitBreached__(): pass

def hkMemory__Signals___dynamic_initializer_for__m_memoryLimitRestored__(): pass

def _dynamic_initializer_for__s_logMemDebug__(): pass

def _dynamic_initializer_for__hkHardwareInfo__singleton__(): pass

def _dynamic_initializer_for__hkHardwareInfo__singletonEntry__(): pass

def _dynamic_initializer_for__hkLog__Detail__AutoScope__s_currentLogAuto__(): pass

def _dynamic_initializer_for___anonymous_namespace___Registry__singleton__(): pass

def _dynamic_initializer_for___anonymous_namespace___Registry__singletonEntry__(): pass

def _anonymous_namespace____dynamic_initializer_for__hkLog_threadLocalOutput__(): pass

def _anonymous_namespace____dynamic_initializer_for__s_disabledIdSet__(): pass

def _dynamic_initializer_for__s_outputLock__(): pass

def _dynamic_initializer_for__s_registerDebugLogs__(): pass

def _dynamic_initializer_for__hkError__singleton__(): pass

def _dynamic_initializer_for__hkError__singletonEntry__(): pass

def _dynamic_initializer_for__hkErrorRouter__singleton__(): pass

def _dynamic_initializer_for__hkErrorRouter__singletonEntry__(): pass

def _anonymous_namespace____dynamic_initializer_for__s_flushDenormalsNode__(): pass

def _dynamic_initializer_for__hkMemory__MallocAllocator__s_instance__(): pass

def _dynamic_initializer_for__hkFileSystem__singleton__(): pass

def _dynamic_initializer_for__hkFileSystem__singletonEntry__(): pass

def _dynamic_initializer_for__s_debugLog___9(): pass

def _dynamic_initializer_for__hkReflect__Detail__BuiltinTypeReg__singleton__(): pass

def _dynamic_initializer_for__hkReflect__Detail__BuiltinTypeReg__singletonEntry__(): pass

def _dynamic_initializer_for__s_debugLog___10(): pass

def _dynamic_initializer_for___stackTracerWin32DataBuf__(): pass

def hkMath___dynamic_initializer_for__s_checkPopCntSupport__(): pass

def _dynamic_initializer_for___anonymous_namespace___AfterReflectNewLock__singleton__(): pass

def _dynamic_initializer_for___anonymous_namespace___AfterReflectNewLock__singletonEntry__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__ArrayType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__BoolType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__FloatType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__IntType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__OpaqueType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__PointerType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__RecordType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__StringType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__VoidType___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__ArrayType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__BoolType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__FloatType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__IntType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__OpaqueType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__PointerType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__RecordType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__StringType_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkReflect__QualifiedType_hkReflect__VoidType_____typeRegNode__(): pass

def _dynamic_initializer_for__s_debugLog___11(): pass

def _dynamic_initializer_for__s_debugLog___12(): pass

def _dynamic_initializer_for__s_debugLog___13(): pass

def _dynamic_initializer_for__hkRelPtr_PointerImpl__s_instance__(): pass

def hkLog__Origins___dynamic_initializer_for__common__(): pass

def hkLog__Origins___dynamic_initializer_for__globalDefault__(): pass

def _dynamic_initializer_for__hkMonitorStreamStatics__singleton__(): pass

def _dynamic_initializer_for__hkMonitorStreamStatics__singletonEntry__(): pass

def _dynamic_initializer_for__hkMonitorStream__m_lockCount__(): pass

def _dynamic_initializer_for__s_monitorStreamStatics__(): pass

def _dynamic_initializer_for__hkDefaultTaskQueueThreadContext__(): pass

def hkMemory__Detail___dynamic_initializer_for__s_globalMemoryAccessChecker__(): pass

def _dynamic_initializer_for__hkMemory__Snapshot__SortAllocationsByAddress__(): pass

def _dynamic_initializer_for__hkMemory__Snapshot__SortAllocationsByAddressComposite__(): pass

def _dynamic_initializer_for__hkMemory__Snapshot__SortAllocationsByComparison__(): pass

def _dynamic_initializer_for__hkMemory__Snapshot__SortAllocationsByTimeComposite__(): pass

def _dynamic_initializer_for__hkCheckDeterminismUtil_ThreadData__(): pass

def _dynamic_initializer_for__s_debugLog___14(): pass

def _dynamic_initializer_for__s_debugLog___15(): pass

def _dynamic_initializer_for__s_debugLog___16(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianFloatImplN_float_hkEndian__EndianType_float_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianFloatImplN_float_hkEndian__EndianType_float_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianFloatImplN_double_hkEndian__EndianType_double_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianFloatImplN_double_hkEndian__EndianType_double_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianFloatImplN_hkHalf16_hkEndian__EndianType_hkHalf16_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianFloatImplN_hkHalf16_hkEndian__EndianType_hkHalf16_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_signed_char_hkEndian__EndianType_signed_char_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_signed_char_hkEndian__EndianType_signed_char_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_unsigned_char_hkEndian__EndianType_unsigned_char_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_unsigned_char_hkEndian__EndianType_unsigned_char_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_short_hkEndian__EndianType_short_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_short_hkEndian__EndianType_short_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_unsigned_short_hkEndian__EndianType_unsigned_short_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_unsigned_short_hkEndian__EndianType_unsigned_short_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_int_hkEndian__EndianType_int_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_int_hkEndian__EndianType_int_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_unsigned_int_hkEndian__EndianType_unsigned_int_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_unsigned_int_hkEndian__EndianType_unsigned_int_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN___int64_hkEndian__EndianType___int64_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN___int64_hkEndian__EndianType___int64_0_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_unsigned___int64_hkEndian__EndianType_unsigned___int64_1_____s_instance__(): pass

def _dynamic_initializer_for__hkReflect__Detail__EndianIntImplN_unsigned___int64_hkEndian__EndianType_unsigned___int64_0_____s_instance__(): pass

def _dynamic_initializer_for__hkFloat32Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkFloat32Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkFloat64Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkFloat64Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkHalf16Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkHalf16Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt16Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt16Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt32Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt32Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt64Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt64Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt8Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkInt8Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint16Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint16Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint32Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint32Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint64Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint64Le_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint8Be_typeRegNode__(): pass

def _dynamic_initializer_for__hkUint8Le_typeRegNode__(): pass

def hkTypeVm___dynamic_initializer_for__origin__(): pass

def _dynamic_initializer_for__s_debugLog___17(): pass

def _dynamic_initializer_for__hknpBodyCinfo___Auto__m_angularVelocity__defaultType__def__(): pass

def _dynamic_initializer_for__hknpWorldCinfo___Auto__m_bodyIntegrator__defaultType__def__(): pass

def _dynamic_initializer_for__hknpWorldCinfo___Auto__m_broadPhaseConfig__defaultType__def__(): pass

def _dynamic_initializer_for__hknpWorldCinfo___Auto__m_collisionFilter__defaultType__def__(): pass

def _dynamic_initializer_for__hknpBodyCinfo___Auto__m_linearVelocity__defaultType__def__(): pass

def _dynamic_initializer_for__hknpWorldCinfo___Auto__m_materialLibrary__defaultType__def__(): pass

def _dynamic_initializer_for__hknpBodyCinfo___Auto__m_maxDepenetrationSpeed__defaultType__def__(): pass

def _dynamic_initializer_for__hknpWorldCinfo___Auto__m_motionPropertiesLibrary__defaultType__def__(): pass

def _dynamic_initializer_for__hknpBodyCinfo___Auto__m_orientation__defaultType__def__(): pass

def _dynamic_initializer_for__hknpBodyCinfo___Auto__m_position__defaultType__def__(): pass

def _dynamic_initializer_for__hknpWorldCinfo___Auto__m_qualityLibrary__defaultType__def__(): pass

def _dynamic_initializer_for__hknpShapeInstance___Auto__m_rotation__defaultType__def__(): pass

def _dynamic_initializer_for__hknpShapeInstance___Auto__m_scale__defaultType__def__(): pass

def _dynamic_initializer_for__hknpWorldCinfo___Auto__m_shapeTagCodec__defaultType__def__(): pass

def _dynamic_initializer_for__hknpFirstPersonCharacterCinfo_m_customShape_Auto__defaultVal__(): pass

def _dynamic_initializer_for__hkHashBase_hkPair_unsigned_int_hkRefPtr_hknpShape_const_____hkBuiltinContainerAllocator_0_0______Auto__m_items_impl__(): pass

def _dynamic_initializer_for__hkHashBase_hkPair_hkReflect__Typedef__hkUint32_Tag_hkRefPtr_hknpShape_const_____hkBuiltinContainerAllocator_0_0______Auto__m_items_impl__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_int___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpParticleCollisionPair___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpShapeKeyMask___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpCollisionFilter___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpExternMeshShapeGeometry___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpFirstPersonGun___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpRefWorldCinfo___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpShape___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpShapeTagCodec___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpTyremarksInfo___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpTyremarksWheel___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleAerodynamics___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleBrake___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleData___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleDriverInput___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleDriverInputStatus___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleEngine___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleSteering___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleSuspension___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleTransmission___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleVelocityDamper___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hknpVehicleWheelCollide___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdStaticTree__AabbTree_hkcdCompressedAabbCodecs__Aabb6BytesCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpGroupCollisionFilterTypes__Config_5_5_5_16_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHashMapDetail__Index_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFreeListArray_hknpShapeInstance_hkHandle_unsigned_short_65535_hknpShapeInstanceIdDiscriminant__8_hknpShapeInstance_hkFreeListRelArrayStorage_hknpShapeInstance_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFreeListArrayElement_hknpShapeInstance_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFreeListRelArray_hknpShapeInstance_hkHandle_unsigned_short_65535_hknpShapeInstanceIdDiscriminant__8_hknpShapeInstance_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFreeListRelArrayStorage_hknpShapeInstance_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_char_255_hknpBodyQualityIdBaseDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_short_65535_hknpConnectivityGraph__NodeIdDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_short_65535_hknpMaterialIdBaseDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_short_65535_hknpMotionPropertiesIdBaseDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_short_65535_hknpShapeInstanceIdDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_int_2147483647_hknpConstraintGroupIdDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_int_2147483647_hknpConstraintIndexDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_int_2147483647_hknpMotionIdBaseDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_int_4294967295_hknpShapeKeyDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint16_Tag_65535_hknpConnectivityGraph__NodeIdDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint16_Tag_65535_hknpMaterialIdBaseDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint16_Tag_65535_hknpMotionPropertiesIdBaseDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint16_Tag_65535_hknpShapeInstanceIdDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint32_Tag_2147483647_hknpConstraintGroupIdDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint32_Tag_2147483647_hknpConstraintIndexDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint32_Tag_2147483647_hknpMotionIdBaseDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint32_Tag_4294967295_hknpShapeKeyDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint8_Tag_255_hknpBodyQualityIdBaseDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkPair_unsigned_int_hkRefPtr_hknpShape_const________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkPair_hkReflect__Typedef__hkUint32_Tag_hkRefPtr_hknpShape_const________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hknpShapeKeyMask_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkFreeListArrayElement_hknpShapeInstance_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkHandle_unsigned_short_65535_hknpShapeInstanceIdDiscriminant_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdCompressedAabbCodecs__Aabb6BytesCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hknpConnectivityGraph__Edge_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hknpShapeProperties__Entry_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hknpMinMaxQuadTree__MinMaxLevel_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hknpConnectivityGraph__PackedManifold_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hknpCompoundShape__VelocityInfo_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkCompressedMassProperties_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hknpMaterialDescriptor_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hknpShapeInstance_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkReflect__Typedef__hknpShapeInstanceId_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpThreadSafeObjectPoolElement_hknpMaterial_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpThreadSafeObjectPoolElement_hknpMotionProperties_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_signed_char_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_unsigned_short_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_int_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_float_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpTyremarksWheel___hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkPair_unsigned_int_hkRefPtr_hknpShape_const_____hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkPair_hkReflect__Typedef__hkUint32_Tag_hkRefPtr_hknpShape_const_____hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpThreadSafeObjectPoolElement_hknpMaterial__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpThreadSafeObjectPoolElement_hknpMotionProperties__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknp1dAngularFollowCamCinfo__CameraSet_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpPhysicsSystemData__ExtendedBodyCinfo_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Detail__Opaque_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpExtendedExternMeshShapeGeometry__Quad_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpVehicleSuspension__SuspensionWheelParameters_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpExtendedExternMeshShapeGeometry__Triangle_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpVehicleDefaultBrake__WheelBrakingProperties_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpVehicleData__WheelComponentParams_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpVehicleInstance__WheelInfo_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpVehicleDefaultSuspension__WheelSpringSuspensionParameters_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpVehicleLinearCastWheelCollide__WheelState_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkInt8_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkReal_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkUint16_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpParticleTransformsAndVelocities_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpStorageParticleSystem_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpTyremarkPoint_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkRefPtr_hknpConvexShape_const___hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkRefPtr_hknpPhysicsSystemData__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkRefPtr_hknpWorldSnapshot__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkBool_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkStringPtr_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpBody_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpConstraintCinfo_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpConstraintGroupCinfo_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpMaterial_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpMotion_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hknpMotionProperties_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArrayView_int_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArrayView_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArrayView_hknpParticleCollisionPair_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__ActivationMode_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__ActivationMode_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__AdditionMode_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__AdditionMode_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpConstraintAtom__AtomType_unsigned_short_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpConstraintAtom__AtomType_hkReflect__Typedef__hkUint16_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknp6DofConstraintData__AxisMode_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknp6DofConstraintData__AxisMode_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCallbackConstraintMotor__CallbackType_unsigned_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCallbackConstraintMotor__CallbackType_hkReflect__Typedef__hkUint32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpMaterial__CombinePolicy_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpMaterial__CombinePolicy_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpConstraintData__ConstraintType_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpConstraintData__ConstraintType_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpShapeKeyMask__Type__Enum_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpShapeKeyMask__Type__Enum_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpActivationControl__Enum_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpActivationControl__Enum_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpActivationState__Enum_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpActivationState__Enum_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCollisionDispatchType__Enum_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCollisionDispatchType__Enum_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCompoundShapeBoundingVolumeType__Enum_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCompoundShapeBoundingVolumeType__Enum_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpMotionType__Enum_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpMotionType__Enum_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpShapeType__Enum_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpShapeType__Enum_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpFirstPersonGun__KeyboardKey_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpFirstPersonGun__KeyboardKey_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpMaterial__MassChangerCategory_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpMaterial__MassChangerCategory_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpConstraintMotor__MotorType_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpConstraintMotor__MotorType_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpPointToPathConstraintData__OrientationConstraintType_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpPointToPathConstraintData__OrientationConstraintType_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__PivotLocation_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__PivotLocation_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpLinearSurfaceVelocity__ProjectMethod_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpLinearSurfaceVelocity__ProjectMethod_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpSurfaceVelocity__Space_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpSurfaceVelocity__Space_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpContactImpulseEvent__Status_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpContactImpulseEvent__Status_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpManifoldEvent__Status_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpManifoldEvent__Status_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpTriggerEvent__Status_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpTriggerEvent__Status_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCharacterSurfaceInfo__SupportedState_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCharacterSurfaceInfo__SupportedState_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpMaterial__TriggerType_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpMaterial__TriggerType_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCollisionFilter__Type_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpCollisionFilter__Type_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpFirstPersonGun__Type_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpFirstPersonGun__Type_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpShapeTagCodec__Type_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpShapeTagCodec__Type_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__UpdateCachesMode_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__UpdateCachesMode_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__UpdateMotionMode_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpWorldWriter__UpdateMotionMode_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpVehicleWheelCollide__WheelCollideType_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hknpVehicleWheelCollide__WheelCollideType_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpBroadPhaseFilter__Category_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpCollisionControl__Enum_unsigned_short_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpCollisionFlags__Enum_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpBodyQuality__FlagsEnum_unsigned_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpBodyQuality__FlagsEnum_hkReflect__Typedef__hkUint32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpConstraint__FlagsEnum_unsigned_short_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpConstraint__FlagsEnum_hkReflect__Typedef__hkUint16_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpShape__FlagsEnum_unsigned_short_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpShape__FlagsEnum_hkReflect__Typedef__hkUint16_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpShapeTransform__FlagsEnum_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpShapeTransform__FlagsEnum_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpShapeTagCodec__Hints_unsigned_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpShapeTagCodec__Hints_hkReflect__Typedef__hkUint32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpStiffSpringConstraintAtom__ProjectionAxis_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpStiffSpringConstraintAtom__ProjectionAxis_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpManifoldViewerBase__VdbManifold__StatusEnum_unsigned_short_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpManifoldViewerBase__VdbManifold__StatusEnum_hkReflect__Typedef__hkUint16_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpBody__WorldAddedState_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hknpBody__WorldAddedState_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHashBase_hkPair_unsigned_int_hkRefPtr_hknpShape_const_____hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHashBase_hkPair_hkReflect__Typedef__hkUint32_Tag_hkRefPtr_hknpShape_const_____hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHashMap_unsigned_int_hkRefPtr_hknpShape_const___hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHashMap_hkReflect__Typedef__hkUint32_Tag_hkRefPtr_hknpShape_const___hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkaSkeleton_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpCollisionFilter_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpCompoundShape_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpConvexShape_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpHeightFieldGeometry_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpShape_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkReflect__Detail__Opaque_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkGeometry_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpRefDragProperties_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpRefMassDistribution_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkLocalFrame_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpBodyIntegrator_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpBodyQualityLibrary_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpBroadPhaseConfig_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpCollisionFilter_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpConnectivityGraphMask_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpConstraintData_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpMaterialLibrary_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpMotionPropertiesLibrary_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpPhysicsSystemData_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpRefMaterial_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpShape_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpShapeTagCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpSurfaceVelocity_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hknpWorldSnapshot_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_unsigned_char_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hknpConvexHull__Edge_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hknpConvexHull__Face_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hknpMeshShape__GeometrySection_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hknpMeshShape__GeometrySection__Primitive_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hknpMeshShape__ShapeTagTableEntry_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hkFloat3_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hkReflect__Typedef__hkUint8_Tag_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hkReflect__Typedef__hkVector4_Tag_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hknpAabb8TreeNode_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArrayView_hkVector4f_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpConnectivityGraph_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hkReferencedObject_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpCompoundShape_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpShape_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpWheelFrictionConstraintAtom__Axle_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpCompressedMeshShapeData_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpConnectivityGraph_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpHeightFieldBoundingVolume_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpShapeKeyMask_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpConnectivityGraphMask_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpConstraintData_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpConstraintMotor_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpMeshShapePrimitiveMapping_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpParametricCurve_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpShape_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpShapePropertyBase_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpShapeSignals_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkViewPtr_hknpConstraintData_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpGroupCollisionFilterBase_hknpGroupCollisionFilterTypes__Config_5_5_5_16_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpShapeProperty_61460_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpShapeProperty_61461_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpShapeProperty_61462_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpShapeProperty_61696_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpShapeProperty_61697_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpShapeProperty_61699_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpThreadSafeObjectPool_hknpMaterial_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpThreadSafeObjectPool_hknpMotionProperties_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__ReflectionOfCharArray_char__128__0___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_char_1___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_char_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_char_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_char_7___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_short_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_int_19___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_int_32___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_float_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_float_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_float_16___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_float_12___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkHandle_unsigned_int_4294967295_hknpShapeKeyDiscriminant__2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkHandle_hkReflect__Typedef__hkUint32_Tag_4294967295_hknpShapeKeyDiscriminant__2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hknpVehicleFrictionDescription__AxisDescription_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hknpVehicleFrictionStatus__AxisStatus_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hknpConnectivityGraph__PackedManifold__Vertex_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkInt16_Tag_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkReal_Tag_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkReal_Tag_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkReal_Tag_16___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkReal_Tag_12___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkTransform_Tag_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint8_Tag_1___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint8_Tag_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint8_Tag_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint8_Tag_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint8_Tag_7___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkVector4_Tag_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkVector4_Tag_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkVector4_Tag_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hknpAngFrictionConstraintAtom_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hknpBodyId_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hknpLinFrictionConstraintAtom_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hknpLinLimitConstraintAtom_3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkEnum_enum_hknp6DofConstraintData__AxisMode_unsigned_char__6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkEnum_enum_hknp6DofConstraintData__AxisMode_hkReflect__Typedef__hkUint8_Tag__6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkRelPtr_hknpShape_const___8___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkRelPtr_hknpConstraintMotor__3___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkHalf16_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkTransformf_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkVector4f_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkVector4f_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hknpBodyQuality_32___typeRegNode__(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___0(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___0(): pass

def _dynamic_initializer_for__hknp1dAngularFollowCamCinfo_CameraSet_typeRegNode__(): pass

def _dynamic_initializer_for__hknp1dAngularFollowCamCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknp2dAngConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknp3dAngConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknp6DofConstraintData_AxisMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknp6DofConstraintData_Blueprints_typeRegNode__(): pass

def _dynamic_initializer_for__hknp6DofConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAabb8TreeNode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAction_typeRegNode__(): pass

def _dynamic_initializer_for__hknpActivateBodiesInAabbCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpActivationControl_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpActivationState_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAddBodyCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAddCollisionCachesTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAddConstraintCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAddParticlesColliderCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAddParticlesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAngConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAngFrictionConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAngLimitConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAngMotorConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAngularFrictionConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAngularFrictionConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpApiCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpApplyAngularImpulseCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpApplyLinearImpulseCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpApplyPointImpulseCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpArrayAction_typeRegNode__(): pass

def _dynamic_initializer_for__hknpAttachBodyCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBallAndSocketConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBallAndSocketConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBallGun_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBallSocketConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBinaryAction_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBinaryBodyDebugEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBinaryBodyEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyAllocatedAttachedTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyAllocatedTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyExitedSafeRegionEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyIdAndIndex_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyIdPair_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyIndexPair_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyIndex_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyIntegrator_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyQualityId_Preset_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyQualityId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyQualityLibrary_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyQuality_FlagsEnum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyQuality_LiveWeldingConfig_DeferralThresholds_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyQuality_LiveWeldingConfig_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBodyQuality_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBody_WorldAddedState_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBody_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBoxShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBreakableCompoundShapeCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBreakableCompoundShapeKeyMask_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBreakableCompoundShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBreakableConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBridgeAtoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBridgeConstraintAtomBase_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBridgeConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBroadPhaseConfig_Layer_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBroadPhaseConfig_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBroadPhaseFilter_Category_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBroadPhaseType_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpBroadPhaseViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCallbackConstraintMotor_CallbackType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCallbackConstraintMotor_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCapsuleShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCharacterProxyCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCharacterProxy_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCharacterRigidBodyCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCharacterState_hknpCharacterStateType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCharacterSurfaceInfo_SupportedState_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCharacterSurfaceInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCircularSurfaceVelocity_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCircularSymmetry_AxisEnum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpClearBodyDragPropertiesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpClearBodyMassDistributionCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCogWheelConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCogWheelConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCogWheelConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCollisionControl_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCollisionControl_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCollisionDispatchType_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCollisionDispatchType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCollisionFilter_Type_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCollisionFilter_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCollisionFlags_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCollisionFlags_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompositeShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompoundShapeBoundingVolumeType_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompoundShapeCdDynamicTree_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompoundShapeData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompoundShapeInternalsCdDynamicTreeKeyMask_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompoundShapeInternalsSimdTreeKeyMask_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompoundShapeKeyMask_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompoundShape_VelocityInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompoundShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompressedMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConeLimitConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConnectivityGraphMask_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConnectivityGraph_CompressedIntArray_IntSize_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConnectivityGraph_CompressedIntArray_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConnectivityGraph_Edge_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConnectivityGraph_PackedManifold_Vertex_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConnectivityGraph_PackedManifold_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConnectivityGraph_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintAllocatedTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintAtomStiffness_Type_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintAtomStiffness_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintAtom_AtomType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintAtom_BodySpaceIndex_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintCollisionFilter_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintData_ConstraintType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintForceEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintForceExceededEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintGroupCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintGroupDirectSolverSettings_CallFrequency_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintGroupDirectSolverSettings_MatrixType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintGroupDirectSolverSettings_MotorInclusion_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintGroupDirectSolverSettings_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintGroupId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintGroup_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintIndex_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintMotor_MotorType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintMotor_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraintViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraint_FlagsEnum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConstraint_typeRegNode__(): pass

def _dynamic_initializer_for__hknpContactImpulseClippedEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpContactImpulseEventWithShapeKeys_typeRegNode__(): pass

def _dynamic_initializer_for__hknpContactImpulseEvent_Status_typeRegNode__(): pass

def _dynamic_initializer_for__hknpContactImpulseEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpContactSolverEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConvexHull_Edge_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConvexHull_Face_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConvexHull_typeRegNode__(): pass

def _dynamic_initializer_for__hknpConvexShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCylinderShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDeactivationViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDecoratorShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDefaultBroadPhaseConfig_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDefaultExternMeshShapeGeometry_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDefaultHeightFieldGeometry_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDeflectedLinearCastCollisionEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDeformableAngConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDeformableFixedConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDeformableFixedConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDeformableLinConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDefragmentInactiveCacheStreamsTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDeleteAllCachesTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDestroyBodyCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDestroyConstraintGroupCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDestroyConstraintsCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDestroyMaterialCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDestroyMotionPropertiesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDestroyMotionsCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDestroyParticlesColliderCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDetachBodyCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDisableCollisionFilter_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDisableParticlesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDragProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hknpDummyShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpEllipticalLimitConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpEmptyEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpEnableParticlesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpExtendedExternMeshShapeGeometry_Quad_typeRegNode__(): pass

def _dynamic_initializer_for__hknpExtendedExternMeshShapeGeometry_Triangle_typeRegNode__(): pass

def _dynamic_initializer_for__hknpExtendedExternMeshShapeGeometry_typeRegNode__(): pass

def _dynamic_initializer_for__hknpExternMeshShapeData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpExternMeshShapeGeometry_typeRegNode__(): pass

def _dynamic_initializer_for__hknpExternMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFirstPersonCharacterCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFirstPersonCharacter_ControlFlags_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFirstPersonCharacter_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFirstPersonGun_KeyboardKey_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFirstPersonGun_Type_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFirstPersonGun_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFixedConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFixedConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpFullCastCollisionEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpGravityGun_typeRegNode__(): pass

def _dynamic_initializer_for__hknpGroupCollisionFilter_typeRegNode__(): pass

def _dynamic_initializer_for__hknpHeatMapViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpHeightFieldBoundingVolume_typeRegNode__(): pass

def _dynamic_initializer_for__hknpHeightFieldGeometry_QuadType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpHeightFieldGeometry_typeRegNode__(): pass

def _dynamic_initializer_for__hknpHeightFieldShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpHingeConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpHingeConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLevelOfDetail_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLimitedForceConstraintMotor_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLimitedHingeConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLimitedHingeConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinFrictionConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinLimitConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinMotorConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinearClearanceConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinearClearanceConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinearIntegrationClippedEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinearParametricCurve_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinearSurfaceVelocity_ProjectMethod_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLinearSurfaceVelocity_typeRegNode__(): pass

def _dynamic_initializer_for__hknpLodShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMalleableConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpManifoldEvent_Status_typeRegNode__(): pass

def _dynamic_initializer_for__hknpManifoldEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpManifoldType_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpManifoldViewerBase_VdbManifold_StatusEnum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpManifoldViewerBase_VdbManifold_typeRegNode__(): pass

def _dynamic_initializer_for__hknpManifoldViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaskedCompoundShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaskedShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMassDistribution_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMassPropertiesTree_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterialDescriptor_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterialIdBase_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterialId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterialLibrary_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterialPalette_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterial_CombinePolicy_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterial_MassChangerCategory_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterial_TriggerType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMaterial_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMeshShapePrimitiveMapping_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMeshShapeVertexConversionUtil_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMeshShape_GeometrySection_Primitive_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMeshShape_GeometrySection_Vertex16_3_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMeshShape_GeometrySection_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMeshShape_ShapeTagTableEntry_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMinMaxQuadTree_MinMaxLevel_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMinMaxQuadTree_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionAllocatedTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionIdBase_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionPropertiesIdBase_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionPropertiesId_Preset_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionPropertiesId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionPropertiesLibrary_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionProperties_DeactivationSettings_Strategy_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionProperties_DeactivationSettings_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionProperties_FullCastSettings_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionProperties_SolverStabilizationType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionRangeBreachPolicy_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotionType_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMotion_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMountedBallGun_typeRegNode__(): pass

def _dynamic_initializer_for__hknpMultiFrameWorldSnapshot_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPairCollisionFilter_MapPairFilterKeyOverrideType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPairCollisionFilter_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParametricCurve_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleBodyContact_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleBodyImpulseAppliedEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleCacheMoveInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleCacheViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleCollisionPair_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleParticleContact_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleParticleImpulseAppliedEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleSpheresViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticleTransformsAndVelocities_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticlesCollidedWithBodiesEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticlesCollidedWithParticlesEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticlesColliderAllocatedTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticlesColliderId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticlesColliderShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticlesExitedSafeRegionEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpParticlesShapeViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPhysicsSceneData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPhysicsSystemData_ExtendedBodyCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPhysicsSystemData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPhysicsSystem_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPointToPathConstraintData_OrientationConstraintType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPointToPathConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPointToPlaneConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPointToPlaneConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPositionConstraintMotor_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPotentialManifoldEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPrismaticConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPrismaticConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpProjectileGun_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPulleyConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPulleyConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpPulleyConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpQueryFilterData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRackAndPinionConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRackAndPinionConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRackAndPinionConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollKeyFrameHierarchyUtility_ControlData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollMotorConstraintAtom_RagdollMotorMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollMotorConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollPenetrationUtil_PenetrationStatusFlagsEnum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollPenetrationUtil_ResolveModeFlagsEnum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollState_ConstraintEasingMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdollState_StateId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRagdoll_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRebuildBodyCollisionCachesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRebuildMotionMassPropertiesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRebuildParticleCollisionCachesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRefDragProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRefMassDistribution_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRefMaterial_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRefWorldCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpReintegrateMotionCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRemoveBodyCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRemoveConstraintCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRemoveParticlesColliderCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRemoveParticlesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpReserved0Event_typeRegNode__(): pass

def _dynamic_initializer_for__hknpReserved1Event_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRotationalConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpRotationalConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpScaledConvexShapeBase_typeRegNode__(): pass

def _dynamic_initializer_for__hknpScaledConvexShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyActivationControlCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyActivationPriorityCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyActivationStateCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyAngularVelocityCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyCollisionFilterInfoCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyCollisionLookAheadDistanceCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyDragPropertiesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyLinearVelocityCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyMassCommand_Mode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyMassCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyMassDistributionCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyMaterialCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyMotionCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyMotionPropertiesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyMotionTypeCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyOrientationCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyPositionCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyPropertyCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyQualityCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyShapeCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyTransformCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetBodyVelocityCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetConstraintGroupCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetConstraintPropertyCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetEventDispatcherCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetLocalRotationsConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetLocalTransformsConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetLocalTranslationsConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetMotionCenterOfMassCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetMotionInertiaCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetPointVelocityCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetWorldAirDensityCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetWorldCollisionFilterCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetWorldGravityCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetWorldNameCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetWorldSafeRegionAabbCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetWorldShapeTagCodecCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSetupStabilizationAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeCastViewer_ShapeCastOptions_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeConnectivityGraph_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeInstanceId_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeInstance_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeKeyMask_Type_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeKeyMask_Type_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeKeyMask_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeMassConfigProperty_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeMassPropertiesTree_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeMassProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeProperties_Entry_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapePropertyBase_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeSignals_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeTagCodec_Hints_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeTagCodec_Type_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeTagCodec_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeTransform_FlagsEnum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeTransform_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeType_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeViewer_Options_ConvexRadiusDisplayMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeViewer_Options_LevelOfDetail_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShape_ConvexRadiusDisplayMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShape_FlagsEnum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShape_MassConfig_Quality_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShape_MassConfig_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShiftWorldCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSimulationType_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSphereShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSpringAction_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSpringDamperConstraintMotor_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStepAllParticlesCollidersTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStepCollideTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStepInput_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStepSolveTrace_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStiffSpringConstraintAtom_ProjectionAxis_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStiffSpringConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStiffSpringConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStiffSpringConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpStorageParticleSystem_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSurfaceVelocity_Space_typeRegNode__(): pass

def _dynamic_initializer_for__hknpSurfaceVelocity_typeRegNode__(): pass

def _dynamic_initializer_for__hknpThreadSafetyCheck_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTransformedShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTransposedFourAabbs8_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTriangleShape_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTriggerEvent_Status_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTriggerEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTwistLimitConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTyremarkPoint_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTyremarksInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpTyremarksWheel_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUnaryAction_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUnaryBodyApiCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUnaryBodyDebugEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUnaryBodyEvent_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUpdateBodyCollisionCachesCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUpdateBodyFlagsCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUpdateBroadphaseCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUpdateConstraintFlagsCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpUserCommand_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbApiCommands_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbBodies_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbBody_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbCharacterDisplayMarker_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbCharacterProxies_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbEvents_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbImmediateConstraint_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbManifoldDisplay_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbMassProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbParticlesShapeDisplay_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbPhysicsWorld_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVdbShapeDisplayMarker_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleAerodynamics_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleBrake_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleData_WheelComponentParams_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultAerodynamics_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultAnalogDriverInput_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultBrake_WheelBrakingProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultBrake_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultEngine_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultSteering_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultSuspension_WheelSpringSuspensionParameters_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultSuspension_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultTransmission_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDefaultVelocityDamper_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDriverInputAnalogStatus_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDriverInputStatus_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleDriverInput_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleEngine_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleFrictionDescription_AxisDescription_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleFrictionDescription_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleFrictionStatus_AxisStatus_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleFrictionStatus_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleInstance_WheelInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleInstance_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleLinearCastWheelCollide_WheelState_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleLinearCastWheelCollide_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleRayCastWheelCollide_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleSteering_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleSuspension_SuspensionWheelParameters_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleSuspension_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleTransmission_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleVelocityDamper_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleWheelCollide_WheelCollideType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVehicleWheelCollide_typeRegNode__(): pass

def _dynamic_initializer_for__hknpVelocityConstraintMotor_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWeldingConfig_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWheelConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWheelConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWheelFrictionConstraintAtom_Axle_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWheelFrictionConstraintAtom_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWheelFrictionConstraintData_Atoms_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWheelFrictionConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldCinfo_DeterminismCheckingMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldCinfo_SolverType_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldQueryViewer_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldSnapshot_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldWriter_ActivationMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldWriter_AdditionMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldWriter_PivotLocation_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldWriter_UpdateCachesMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWorldWriter_UpdateMotionMode_typeRegNode__(): pass

def _dynamic_initializer_for__hknpWrappedConstraintData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpShapeFunctionsSingleton__singleton__(): pass

def _dynamic_initializer_for__hknpShapeFunctionsSingleton__singletonEntry__(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___1(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___1(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___2(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___2(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___3(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___3(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___4(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___4(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpGroupCollisionFilterTypes__Config_6_5_5_16_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hknpGroupCollisionFilterBase_hknpGroupCollisionFilterTypes__Config_6_5_5_16_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned___int64_64___typeRegNode__(): pass

def _dynamic_initializer_for__s_debugLog___18(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___5(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___5(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___6(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___6(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___7(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___7(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___8(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___8(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___9(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___9(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___10(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___10(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___11(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___11(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___12(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___12(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___13(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___13(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___14(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___14(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___15(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___15(): pass

def _dynamic_initializer_for__s_hknpQuerydispatcherLogOrigin__(): pass

def _dynamic_initializer_for__c_floatToIntScale__(): pass

def _dynamic_initializer_for__c_intToFloatScale__(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___16(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___16(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___17(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___17(): pass

def _dynamic_initializer_for__c_floatToIntScale___0(): pass

def _dynamic_initializer_for__c_intToFloatScale___0(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___18(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___18(): pass

def _dynamic_initializer_for__hknpWorld__s_writeApiCommandDepth__(): pass

def _dynamic_initializer_for__c_floatToIntScale___1(): pass

def _dynamic_initializer_for__c_intToFloatScale___1(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___19(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___19(): pass

def _dynamic_initializer_for__hknpConstraintDataFunctionsSingleton__singleton__(): pass

def _dynamic_initializer_for__hknpConstraintDataFunctionsSingleton__singletonEntry__(): pass

def _dynamic_initializer_for__c_floatToIntScale___2(): pass

def _dynamic_initializer_for__c_intToFloatScale___2(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___20(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___20(): pass

def _dynamic_initializer_for__c_floatToIntScale___3(): pass

def _dynamic_initializer_for__c_intToFloatScale___3(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___21(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___21(): pass

def _dynamic_initializer_for__hknpGlobals__singleton__(): pass

def _dynamic_initializer_for__hknpGlobals__singletonEntry__(): pass

def _dynamic_initializer_for__c_floatToIntScale___4(): pass

def _dynamic_initializer_for__c_intToFloatScale___4(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___22(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___22(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___23(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___23(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___24(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___24(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___25(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___25(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___26(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___26(): pass

def _dynamic_initializer_for__c_floatToIntScale___5(): pass

def _dynamic_initializer_for__c_intToFloatScale___5(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___27(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___27(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___28(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___28(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___29(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___29(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___30(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___30(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___31(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___31(): pass

def _dynamic_initializer_for__c_floatToIntScale___6(): pass

def _dynamic_initializer_for__c_intToFloatScale___6(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___32(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___32(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___33(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___33(): pass

def _dynamic_initializer_for__c_floatToIntScale___7(): pass

def _dynamic_initializer_for__c_intToFloatScale___7(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___34(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___34(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___35(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___35(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___36(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___36(): pass

def _dynamic_initializer_for__c_floatToIntScale___8(): pass

def _dynamic_initializer_for__c_intToFloatScale___8(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___37(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___37(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___38(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___38(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___39(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___39(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___40(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___40(): pass

def _dynamic_initializer_for__c_floatToIntScale___9(): pass

def _dynamic_initializer_for__c_intToFloatScale___9(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___41(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___41(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___42(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___42(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___43(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___43(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___44(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___44(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___45(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___45(): pass

def _dynamic_initializer_for__c_floatToIntScale___10(): pass

def _dynamic_initializer_for__c_intToFloatScale___10(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___46(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___46(): pass

def _dynamic_initializer_for__c_floatToIntScale___11(): pass

def _dynamic_initializer_for__c_intToFloatScale___11(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___47(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___47(): pass

def _dynamic_initializer_for__c_floatToIntScale___12(): pass

def _dynamic_initializer_for__c_intToFloatScale___12(): pass

def _dynamic_initializer_for__c_floatToIntScale___13(): pass

def _dynamic_initializer_for__c_intToFloatScale___13(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___48(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___48(): pass

def _dynamic_initializer_for__s_debugLog___19(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___49(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___49(): pass

def _dynamic_initializer_for__c_floatToIntScale___14(): pass

def _dynamic_initializer_for__c_intToFloatScale___14(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___50(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___50(): pass

def _dynamic_initializer_for__c_floatToIntScale___15(): pass

def _dynamic_initializer_for__c_intToFloatScale___15(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___51(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___51(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___52(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___52(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___53(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___53(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___54(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___54(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___55(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___55(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___56(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___56(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___57(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___57(): pass

def hknpParticleParticleSolver___dynamic_initializer_for__sqrttBitErrorFactor__(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___58(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___58(): pass

def _dynamic_initializer_for__c_floatToIntScale___16(): pass

def _dynamic_initializer_for__c_intToFloatScale___16(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___59(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___59(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___60(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___60(): pass

def _dynamic_initializer_for__c_floatToIntScale___17(): pass

def _dynamic_initializer_for__c_intToFloatScale___17(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___61(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___61(): pass

def _dynamic_initializer_for__c_floatToIntScale___18(): pass

def _dynamic_initializer_for__c_intToFloatScale___18(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___62(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___62(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___63(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___63(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___64(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___64(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___65(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___65(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___66(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___66(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___67(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___67(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___68(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___68(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___69(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___69(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___70(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___70(): pass

def _dynamic_initializer_for__c_floatToIntScale___19(): pass

def _dynamic_initializer_for__c_intToFloatScale___19(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___71(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___71(): pass

def _dynamic_initializer_for__s_debugLog___20(): pass

def _dynamic_initializer_for__c_floatToIntScale___20(): pass

def _dynamic_initializer_for__c_intToFloatScale___20(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___72(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___72(): pass

def hknpFullCast___dynamic_initializer_for__manifoldGenerator__(): pass

def _dynamic_initializer_for__c_floatToIntScale___21(): pass

def _dynamic_initializer_for__c_intToFloatScale___21(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___73(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___73(): pass

def _dynamic_initializer_for__c_floatToIntScale___22(): pass

def _dynamic_initializer_for__c_intToFloatScale___22(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___74(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___74(): pass

def _dynamic_initializer_for__c_floatToIntScale___23(): pass

def _dynamic_initializer_for__c_intToFloatScale___23(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___75(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___75(): pass

def _dynamic_initializer_for__c_floatToIntScale___24(): pass

def _dynamic_initializer_for__c_intToFloatScale___24(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___76(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___76(): pass

def _dynamic_initializer_for__c_floatToIntScale___25(): pass

def _dynamic_initializer_for__c_intToFloatScale___25(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___77(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___77(): pass

def _dynamic_initializer_for__c_floatToIntScale___26(): pass

def _dynamic_initializer_for__c_intToFloatScale___26(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___78(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___78(): pass

def _dynamic_initializer_for__c_floatToIntScale___27(): pass

def _dynamic_initializer_for__c_intToFloatScale___27(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___79(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___79(): pass

def _dynamic_initializer_for__c_floatToIntScale___28(): pass

def _dynamic_initializer_for__c_intToFloatScale___28(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___80(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___80(): pass

def _dynamic_initializer_for__c_floatToIntScale___29(): pass

def _dynamic_initializer_for__c_intToFloatScale___29(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___81(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___81(): pass

def _dynamic_initializer_for__c_floatToIntScale___30(): pass

def _dynamic_initializer_for__c_intToFloatScale___30(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___82(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___82(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___83(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___83(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___84(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___84(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___85(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___85(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___86(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___86(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___87(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___87(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___88(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___88(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___89(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___89(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___90(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___90(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___91(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___91(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___92(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___92(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___93(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___93(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___94(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___94(): pass

def _dynamic_initializer_for__c_floatToIntScale___31(): pass

def _dynamic_initializer_for__c_intToFloatScale___31(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___95(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___95(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___96(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___96(): pass

def _dynamic_initializer_for__c_floatToIntScale___32(): pass

def _dynamic_initializer_for__c_intToFloatScale___32(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___97(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___97(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___98(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___98(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___99(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___99(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___100(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___100(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___101(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___101(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___102(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___102(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___103(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___103(): pass

def _dynamic_initializer_for__c_floatToIntScale___33(): pass

def _dynamic_initializer_for__c_intToFloatScale___33(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___104(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___104(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___105(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___105(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___106(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___106(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___107(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___107(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___108(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___108(): pass

def _dynamic_initializer_for__c_floatToIntScale___34(): pass

def _dynamic_initializer_for__c_intToFloatScale___34(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___109(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___109(): pass

def _dynamic_initializer_for__c_floatToIntScale___35(): pass

def _dynamic_initializer_for__c_intToFloatScale___35(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___110(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___110(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___111(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___111(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___112(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___112(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__AD_TYPE_ANDROID_ID__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__AD_TYPE_IDFA__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__buildIdentifier__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__globalErrorHandler__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__productionEnvironmentURL__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__sdkVersion__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__staticPlayer__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__staticSettings__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabSettings__versionString__(): pass

def _dynamic_initializer_for__PlayFab__PlayFabPluginManager__defaultInstanceName__(): pass

def _dynamic_initializer_for__kNoIcon___74(): pass

def _dynamic_initializer_for__kNoIcons___1(): pass

def _dynamic_initializer_for__cTkSketchComponent__sNode__kCategoryColours__(): pass

def _dynamic_initializer_for__cTkLODComponent__mCameraPosition__(): pass

def _dynamic_initializer_for__cTkSketchComponent__mNodesClipboard__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___97(): pass

def _dynamic_initializer_for__k3x4IdentityPos___93(): pass

def _dynamic_initializer_for__k3x4IdentityRight___97(): pass

def _dynamic_initializer_for__k3x4IdentityUp___97(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___87(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___18(): pass

def _dynamic_initializer_for__kInstancercGcAISpaceshipComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcCustomInventoryComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcCustomisationComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcDestructableComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcEncyclopediaComponent___8(): pass

def _dynamic_initializer_for__kInstancercGcFoliageComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcGrabbableComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcPlayerEffectsComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcScannableComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcSpaceshipComponent___0(): pass

def _dynamic_initializer_for__kInstancercGcTriggerActionComponent___26(): pass

def _dynamic_initializer_for__kInstancercTkAnimationComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___63(): pass

def _dynamic_initializer_for__kInstancercTkGravityComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkLODComponent___4(): pass

def _dynamic_initializer_for__kInstancercTkNetReplicatedEntityComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkProceduralModelComponent__(): pass

def _dynamic_initializer_for__kInstancercTkRandomComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkReferenceComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkRotationComponent___0(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___16(): pass

def _dynamic_initializer_for__kLoadInstancercGcAISpaceshipComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcCustomisationComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercGcDestructableComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercTkAnimationComponent___0(): pass

def _dynamic_initializer_for__kLoadInstancercTkLODComponent___4(): pass

def _dynamic_initializer_for__kLoadInstancercTkReferenceComponent___0(): pass

def _dynamic_initializer_for__kNoIcon___75(): pass

def _dynamic_initializer_for__kNoIcons___2(): pass

def _dynamic_initializer_for__kNullDiscoveryData___4(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___18(): pass

def _dynamic_initializer_for__kaBehaviourTreeNodeStateColours___0(): pass

def _dynamic_initializer_for__kfCreatureDistance__(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec___4(): pass

def _dynamic_initializer_for__gHavokWorldOrigin__(): pass

def _dynamic_initializer_for__gPhysicsManager__(): pass

def _dynamic_initializer_for__gVehicleReportCallback__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___98(): pass

def _dynamic_initializer_for__k3x4IdentityPos___94(): pass

def _dynamic_initializer_for__k3x4IdentityRight___98(): pass

def _dynamic_initializer_for__k3x4IdentityUp___98(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___88(): pass

def _dynamic_initializer_for__kInstancercGcFoliageComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___66(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___66(): pass

def _dynamic_initializer_for__kInstancercTkNetReplicatedEntityComponent___1(): pass

def _dynamic_initializer_for__kNoIcon___76(): pass

def _dynamic_initializer_for__kNoIcons___3(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___19(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec___5(): pass

def _dynamic_atexit_destructor_for__gSpaceshipReportCallback__(): pass

def _dynamic_initializer_for__cTkSmartResHandle__Null__(): pass

def _dynamic_initializer_for__Vgrad3lut__(): pass

def _dynamic_initializer_for__kNoIcon___77(): pass

def _dynamic_initializer_for__kNoIcons___4(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___20(): pass

def _dynamic_initializer_for__kVoronoiVecMultX__(): pass

def _dynamic_initializer_for__kVoronoiVecMultY__(): pass

def _dynamic_initializer_for__kVoronoiVecMultZ__(): pass

def _dynamic_initializer_for__kaFaceNormal__(): pass

def _dynamic_initializer_for__kaFaceRight__(): pass

def _dynamic_initializer_for__kaFaceUp__(): pass

def _dynamic_initializer_for__mCache___17(): pass

def _dynamic_initializer_for__cTkNetReplicatedEntityComponent__mCreateIdCallback__(): pass

def _dynamic_initializer_for__gBuiltMeshes__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___99(): pass

def _dynamic_initializer_for__k3x4IdentityPos___95(): pass

def _dynamic_initializer_for__k3x4IdentityRight___99(): pass

def _dynamic_initializer_for__k3x4IdentityUp___99(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___89(): pass

def _dynamic_initializer_for__kInstancercGcFoliageComponent___2(): pass

def _dynamic_initializer_for__kInstancercTkAnimationComponent___1(): pass

def _dynamic_initializer_for__kInstancercTkNetReplicatedEntityComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercTkAnimationComponent___1(): pass

def _dynamic_initializer_for__kNoIcon___78(): pass

def _dynamic_initializer_for__kNoIcons___5(): pass

def _dynamic_initializer_for__kNonCollidingPlane___15(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___21(): pass

def _dynamic_initializer_for__kUnitsPerQuantizedRegionAsVec___0(): pass

def _dynamic_initializer_for__kUnitsPerQuantizedRegionAsVecInv___0(): pass

def _dynamic_initializer_for__kaBehaviourTreeNodeStateColours___1(): pass

def _dynamic_initializer_for__kfCreatureDistance___0(): pass

def _dynamic_initializer_for__cTkColour__mkBlack__(): pass

def _dynamic_initializer_for__cTkColour__mkBlue__(): pass

def _dynamic_initializer_for__cTkColour__mkCyan__(): pass

def _dynamic_initializer_for__cTkColour__mkDarkBlue__(): pass

def _dynamic_initializer_for__cTkColour__mkDarkCyan__(): pass

def _dynamic_initializer_for__cTkColour__mkDarkGreen__(): pass

def _dynamic_initializer_for__cTkColour__mkDarkGrey__(): pass

def _dynamic_initializer_for__cTkColour__mkDarkMagenta__(): pass

def _dynamic_initializer_for__cTkColour__mkDarkRed__(): pass

def _dynamic_initializer_for__cTkColour__mkDarkYellow__(): pass

def _dynamic_initializer_for__cTkColour__mkGreen__(): pass

def _dynamic_initializer_for__cTkColour__mkGrey__(): pass

def _dynamic_initializer_for__cTkColour__mkLightGrey__(): pass

def _dynamic_initializer_for__cTkColour__mkLimeGreen__(): pass

def _dynamic_initializer_for__cTkColour__mkMagenta__(): pass

def _dynamic_initializer_for__cTkColour__mkNiceBlue__(): pass

def _dynamic_initializer_for__cTkColour__mkNiceRed__(): pass

def _dynamic_initializer_for__cTkColour__mkOrange__(): pass

def _dynamic_initializer_for__cTkColour__mkPink__(): pass

def _dynamic_initializer_for__cTkColour__mkPurple__(): pass

def _dynamic_initializer_for__cTkColour__mkRed__(): pass

def _dynamic_initializer_for__cTkColour__mkSeaGreen__(): pass

def _dynamic_initializer_for__cTkColour__mkSkyBlue__(): pass

def _dynamic_initializer_for__cTkColour__mkWhite__(): pass

def _dynamic_initializer_for__cTkColour__mkYellow__(): pass

def X11Colour___dynamic_initializer_for__Aquamarine__(): pass

def X11Colour___dynamic_initializer_for__Black__(): pass

def X11Colour___dynamic_initializer_for__Cyan__(): pass

def X11Colour___dynamic_initializer_for__DarkBlue__(): pass

def X11Colour___dynamic_initializer_for__DarkGray__(): pass

def X11Colour___dynamic_initializer_for__DarkGreen__(): pass

def X11Colour___dynamic_initializer_for__DarkMaroon__(): pass

def X11Colour___dynamic_initializer_for__DarkRed__(): pass

def X11Colour___dynamic_initializer_for__DarkSlateGray__(): pass

def X11Colour___dynamic_initializer_for__DarkTurquoise__(): pass

def X11Colour___dynamic_initializer_for__DarkerGold__(): pass

def X11Colour___dynamic_initializer_for__DeepPink__(): pass

def X11Colour___dynamic_initializer_for__DimBlueGray__(): pass

def X11Colour___dynamic_initializer_for__DimGray__(): pass

def X11Colour___dynamic_initializer_for__DodgerBlue__(): pass

def X11Colour___dynamic_initializer_for__Gold__(): pass

def X11Colour___dynamic_initializer_for__HotPink__(): pass

def X11Colour___dynamic_initializer_for__LightGray__(): pass

def X11Colour___dynamic_initializer_for__Lime__(): pass

def X11Colour___dynamic_initializer_for__LimeGreen__(): pass

def X11Colour___dynamic_initializer_for__Magenta__(): pass

def X11Colour___dynamic_initializer_for__Maroon__(): pass

def X11Colour___dynamic_initializer_for__MediumBlue__(): pass

def X11Colour___dynamic_initializer_for__MediumSlateGray__(): pass

def X11Colour___dynamic_initializer_for__MidnightBlue__(): pass

def X11Colour___dynamic_initializer_for__OrangeRed__(): pass

def X11Colour___dynamic_initializer_for__PaleLimeGreen__(): pass

def X11Colour___dynamic_initializer_for__Pink__(): pass

def X11Colour___dynamic_initializer_for__Red__(): pass

def X11Colour___dynamic_initializer_for__SkyBlue__(): pass

def X11Colour___dynamic_initializer_for__Transparent__(): pass

def X11Colour___dynamic_initializer_for__Turquoise__(): pass

def X11Colour___dynamic_initializer_for__VeryDarkGreen__(): pass

def X11Colour___dynamic_initializer_for__VeryDarkRed__(): pass

def X11Colour___dynamic_initializer_for__VeryDimBlueGray__(): pass

def X11Colour___dynamic_initializer_for__VeryDimGray__(): pass

def X11Colour___dynamic_initializer_for__White__(): pass

def X11Colour___dynamic_initializer_for__Yellow__(): pass

def _dynamic_initializer_for__kIdxColours__(): pass

def _dynamic_initializer_for__kNoIcon___79(): pass

def _dynamic_initializer_for__kNoIcons___6(): pass

def _dynamic_initializer_for__cTkAnimationComponent__sGetClosestPlayerSqrDistFunc__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___100(): pass

def _dynamic_initializer_for__k3x4IdentityPos___96(): pass

def _dynamic_initializer_for__k3x4IdentityRight___100(): pass

def _dynamic_initializer_for__k3x4IdentityUp___100(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___90(): pass

def _dynamic_initializer_for__kInstancercGcCustomInventoryComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcCustomisationComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcEggMachineComponent___74(): pass

def _dynamic_initializer_for__kInstancercGcFoliageComponent___3(): pass

def _dynamic_initializer_for__kInstancercGcMaintenanceComponent___74(): pass

def _dynamic_initializer_for__kInstancercGcNPCComponent___63(): pass

def _dynamic_initializer_for__kInstancercGcNPCInteractiveObjectComponent___67(): pass

def _dynamic_initializer_for__kInstancercGcNPCNavigationAreaComponent___67(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___78(): pass

def _dynamic_initializer_for__kInstancercGcScannableComponent___1(): pass

def _dynamic_initializer_for__kInstancercTkAnimPoseComponent__(): pass

def _dynamic_initializer_for__kInstancercTkAnimationComponent___2(): pass

def _dynamic_initializer_for__kInstancercTkNetReplicatedEntityComponent___3(): pass

def _dynamic_initializer_for__kInstancercTkRandomComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcCustomisationComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercTkAnimPoseComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercTkAnimationComponent___2(): pass

def _dynamic_initializer_for__kNoIcon___80(): pass

def _dynamic_initializer_for__kNoIcons___7(): pass

def _dynamic_initializer_for__kNullDiscoveryData___5(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___22(): pass

def _dynamic_initializer_for__kaBehaviourTreeNodeStateColours___2(): pass

def _dynamic_initializer_for__kfCreatureDistance___1(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec___6(): pass

def _dynamic_initializer_for__MetadataNGuiRender__ClassRender__mCustomClassRenders__(): pass

def _dynamic_initializer_for__cTkMetaData__mgModFolderList__(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___91(): pass

def _dynamic_initializer_for__kNoIcon___81(): pass

def _dynamic_initializer_for__kNoIcons___8(): pass

def _dynamic_initializer_for__cTkVector3__Backward__(): pass

def _dynamic_initializer_for__cTkVector4__Backward__(): pass

def _dynamic_initializer_for__cTkVector3__Down__(): pass

def _dynamic_initializer_for__cTkVector4__Down__(): pass

def _dynamic_initializer_for__cTkVector3__Forward__(): pass

def _dynamic_initializer_for__cTkVector4__Forward__(): pass

def _dynamic_initializer_for__cTkVector3__Left__(): pass

def _dynamic_initializer_for__cTkVector4__Left__(): pass

def _dynamic_initializer_for__cTkVector3__MaxFloat__(): pass

def _dynamic_initializer_for__cTkVector4__MaxFloat__(): pass

def _dynamic_initializer_for__cTkVector3__OneFloat__(): pass

def _dynamic_initializer_for__cTkVector4__OneFloat__(): pass

def _dynamic_initializer_for__cTkVector3__Ox__(): pass

def _dynamic_initializer_for__cTkVector4__Ox__(): pass

def _dynamic_initializer_for__cTkVector3__Oy__(): pass

def _dynamic_initializer_for__cTkVector4__Oy__(): pass

def _dynamic_initializer_for__cTkVector3__Oz__(): pass

def _dynamic_initializer_for__cTkVector4__Oz__(): pass

def _dynamic_initializer_for__cTkVector3__Right__(): pass

def _dynamic_initializer_for__cTkVector4__Right__(): pass

def _dynamic_initializer_for__cTkVector3__Up__(): pass

def _dynamic_initializer_for__cTkVector4__Up__(): pass

def _dynamic_initializer_for__cTkVector3__ZeroFloat__(): pass

def _dynamic_initializer_for__cTkVector4__ZeroFloat__(): pass

def _dynamic_initializer_for__cTkBaseMath__mCosTable__(): pass

def _dynamic_initializer_for__cTkSMShaderWindow__mMaterialList__(): pass

def _dynamic_initializer_for__cTkSMShaderWindow__mMaterialNames__(): pass

def _dynamic_initializer_for__cTkSMShaderWindow__mSelectedMaterial__(): pass

def _dynamic_initializer_for__cTkBaseMath__mSinTable__(): pass

def _dynamic_initializer_for__cTkSMShaderWindow__maLabelGroups__(): pass

def _dynamic_initializer_for__cTkLODSettings__mgaSettings__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___101(): pass

def _dynamic_initializer_for__k3x4IdentityPos___97(): pass

def _dynamic_initializer_for__k3x4IdentityRight___101(): pass

def _dynamic_initializer_for__k3x4IdentityUp___101(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___19(): pass

def _dynamic_initializer_for__kInstancercTkAnimationComponent___3(): pass

def _dynamic_initializer_for__kInstancercTkAudioComponent___64(): pass

def _dynamic_initializer_for__kInstancercTkInstanceWindComponent___2(): pass

def _dynamic_initializer_for__kInstancercTkLODComponent___5(): pass

def _dynamic_initializer_for__kLoadInstancercTkAnimationComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercTkLODComponent___5(): pass

def _dynamic_initializer_for__kNoIcon___82(): pass

def _dynamic_initializer_for__kNoIcons___9(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___16(): pass

def _dynamic_initializer_for__kTkMath__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___102(): pass

def _dynamic_initializer_for__k3x4IdentityPos___98(): pass

def _dynamic_initializer_for__k3x4IdentityRight___102(): pass

def _dynamic_initializer_for__k3x4IdentityUp___102(): pass

def _dynamic_initializer_for__kNoIcon___83(): pass

def _dynamic_initializer_for__kNoIcons___10(): pass

def _dynamic_initializer_for__cTkCompute___mInstance__(): pass

def _dynamic_initializer_for__cTkImmediateMode__mGLColor__(): pass

def _dynamic_initializer_for__cTkDebugRenderer__mgDebugFont__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___103(): pass

def _dynamic_initializer_for__k3x4IdentityPos___99(): pass

def _dynamic_initializer_for__k3x4IdentityRight___103(): pass

def _dynamic_initializer_for__k3x4IdentityUp___103(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___92(): pass

def _dynamic_initializer_for__kNoIcon___84(): pass

def _dynamic_initializer_for__kNoIcons___11(): pass

def _dynamic_atexit_destructor_for__cTkDebugRenderer__mgDebugStyle__(): pass

def _dynamic_initializer_for__cTkTexture__mMipStatsBuffer__(): pass

def _dynamic_initializer_for__gFSQuadColour__(): pass

def _dynamic_initializer_for__gaFSQuadPositions__(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___20(): pass

def _dynamic_initializer_for__kNoIcon___85(): pass

def _dynamic_initializer_for__kNoIcons___12(): pass

def _dynamic_initializer_for__cTkShader__m_UniformHashMap__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___104(): pass

def _dynamic_initializer_for__k3x4IdentityPos___100(): pass

def _dynamic_initializer_for__k3x4IdentityRight___104(): pass

def _dynamic_initializer_for__k3x4IdentityUp___104(): pass

def _dynamic_initializer_for__cTkJobManager__mJobParamAllocator__(): pass

def _dynamic_initializer_for__cTkJobManager__maThreadWorkers__(): pass

def _dynamic_initializer_for__TkGlobals__mDefault__(): pass

def _dynamic_initializer_for__cTkAudioWwiseIO__mLocalisedFolderName__(): pass

def _dynamic_initializer_for__HgVocAlienParams__static_mPriorityManager__(): pass

def _dynamic_initializer_for__AkToneGenParamBlock__static_mRNGJitter__(): pass

def _dynamic_initializer_for__HgVocAlienRegistration__(): pass

def _dynamic_initializer_for__gFileSystem__(): pass

def _dynamic_initializer_for__kNoIcon___86(): pass

def _dynamic_initializer_for__kNoIcons___13(): pass

def _dynamic_initializer_for__SQDHash__(): pass

def _dynamic_initializer_for__SQPHash__(): pass

def _dynamic_initializer_for__SQUHash__(): pass

def _dynamic_initializer_for__SQUnifHash__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___105(): pass

def _dynamic_initializer_for__k3x4IdentityPos___101(): pass

def _dynamic_initializer_for__k3x4IdentityRight___105(): pass

def _dynamic_initializer_for__k3x4IdentityUp___105(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___93(): pass

def _dynamic_initializer_for__kHermiteAt__(): pass

def _dynamic_initializer_for__kHermitePos__(): pass

def _dynamic_initializer_for__kHermiteRight__(): pass

def _dynamic_initializer_for__kHermiteUp__(): pass

def _dynamic_initializer_for__kNoIcon___87(): pass

def _dynamic_initializer_for__kNoIcons___14(): pass

def _dynamic_initializer_for__kaiAlphaBlendStateMap__(): pass

def _dynamic_initializer_for__kNoIcon___88(): pass

def _dynamic_initializer_for__kNoIcons___15(): pass

def _dynamic_initializer_for__kNoIcon___89(): pass

def _dynamic_initializer_for__kNoIcons___16(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___23(): pass

def _dynamic_initializer_for__kfControllerOrientationAdjust__(): pass

def TkAssertDlgResourceData___dynamic_initializer_for__records__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__kDisplayBuffers__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__kHDRBackBuffer__(): pass

def _dynamic_initializer_for__cTkFfxFsr2__mLanczos2LutTextureResource__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__mSamplerIndexHashMap__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__mUniformMemoryManager__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__mVulkanMemoryInfo__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_ComputePipelineStateHashMap__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_PipelineStateHashMap__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_UniqueFragmentShaderHashMap__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_UniqueVertexShaderHashMap__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_allPhysicalDevices__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_bufferDestroyQueue__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_hdrShader__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_imageDestroyQueue__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_imageViewDestroyQueue__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_pipelineDestroyQueue__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_presentModes__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_resCopier__(): pass

def _dynamic_initializer_for__cTkGraphicsAPI__cTkGnmGolbals__m_surfaceFormats__(): pass

def _dynamic_initializer_for__gGraphicsTempBuffer__(): pass

def _dynamic_initializer_for__g_lastVulkanError__(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___24(): pass

def Eigen__placeholders___dynamic_initializer_for__end__(): pass

def _dynamic_initializer_for__gMonitorSemaphore__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___106(): pass

def _dynamic_initializer_for__k3x4IdentityPos___102(): pass

def _dynamic_initializer_for__k3x4IdentityRight___106(): pass

def _dynamic_initializer_for__k3x4IdentityUp___106(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___94(): pass

def _dynamic_initializer_for__kInstancercGcFoliageComponent___4(): pass

def _dynamic_initializer_for__kInstancercTkAnimationComponent___4(): pass

def _dynamic_initializer_for__kInstancercTkCreatureTailComponent___1(): pass

def _dynamic_initializer_for__kInstancercTkDynamicChainComponent__(): pass

def _dynamic_initializer_for__kInstancercTkInstanceWindComponent___3(): pass

def _dynamic_initializer_for__kInstancercTkNetReplicatedEntityComponent___4(): pass

def _dynamic_initializer_for__kInstancercTkPhysicsComponent__(): pass

def _dynamic_initializer_for__kInstancercTkRandomComponent___2(): pass

def _dynamic_initializer_for__kInstancercTkStaticPhysicsComponent__(): pass

def _dynamic_initializer_for__kLoadInstancercTkAnimationComponent___4(): pass

def _dynamic_initializer_for__kNoIcon___90(): pass

def _dynamic_initializer_for__kNoIcons___17(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___25(): pass

def _dynamic_initializer_for__kUpdater_cTkStaticPhysicsComponent__(): pass

def _dynamic_initializer_for__kaBehaviourTreeNodeStateColours___3(): pass

def _dynamic_initializer_for__sSketchEvent__gSketchEvents__(): pass

def _dynamic_initializer_for__cTkSketchWindow__mMinimisedNodes__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___107(): pass

def _dynamic_initializer_for__k3x4IdentityPos___103(): pass

def _dynamic_initializer_for__k3x4IdentityRight___107(): pass

def _dynamic_initializer_for__k3x4IdentityUp___107(): pass

def _dynamic_initializer_for__kDefaultDebugTextColour___95(): pass

def _dynamic_initializer_for__kHiddenHitboxColour___21(): pass

def _dynamic_initializer_for__kInstancercTkSketchComponent___17(): pass

def _dynamic_initializer_for__kNoIcon___91(): pass

def _dynamic_initializer_for__kNoIcons___18(): pass

def _dynamic_initializer_for__lcsDLC_HorizonOmega_PreOrderBonus__(): pass

def _dynamic_initializer_for__kNoIcon___92(): pass

def _dynamic_initializer_for__kNoIcons___19(): pass

def _dynamic_initializer_for__cTkNGui__mgaResourceLookup__(): pass

def _dynamic_initializer_for__gaShaderBuffers__(): pass

def _dynamic_initializer_for__kNoIcon___93(): pass

def _dynamic_initializer_for__kNoIcons___20(): pass

def _dynamic_initializer_for__kNoIcon___94(): pass

def _dynamic_initializer_for__kNoIcons___21(): pass

def _dynamic_atexit_destructor_for__cTkAsyncIOManager__mpImp__(): pass

def _dynamic_initializer_for__cTkVertexNodeProperties__mgaProperties__(): pass

def _dynamic_initializer_for__gHilbertMortonHelper__(): pass

def _dynamic_initializer_for__k3x4IdentityAt___108(): pass

def _dynamic_initializer_for__k3x4IdentityPos___104(): pass

def _dynamic_initializer_for__k3x4IdentityRight___108(): pass

def _dynamic_initializer_for__k3x4IdentityUp___108(): pass

def _dynamic_initializer_for__kInstancercGcAISpaceshipComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcCustomInventoryComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcCustomisationComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcDestructableComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcFoliageComponent___5(): pass

def _dynamic_initializer_for__kInstancercGcGrabbableComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcPlayerEffectsComponent___1(): pass

def _dynamic_initializer_for__kInstancercGcPortalComponent___79(): pass

def _dynamic_initializer_for__kInstancercGcScannableComponent___2(): pass

def _dynamic_initializer_for__kInstancercGcSpaceshipComponent___1(): pass

def _dynamic_initializer_for__kInstancercTkAnimationComponent___5(): pass

def _dynamic_initializer_for__kInstancercTkNetReplicatedEntityComponent___5(): pass

def _dynamic_initializer_for__kInstancercTkRandomComponent___3(): pass

def _dynamic_initializer_for__kLoadInstancercGcAISpaceshipComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercGcCustomisationComponent___2(): pass

def _dynamic_initializer_for__kLoadInstancercGcDestructableComponent___1(): pass

def _dynamic_initializer_for__kLoadInstancercTkAnimationComponent___5(): pass

def _dynamic_initializer_for__kNoIcon___95(): pass

def _dynamic_initializer_for__kNoIcons___22(): pass

def _dynamic_initializer_for__kNullDiscoveryData___6(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___26(): pass

def _dynamic_initializer_for__kaBehaviourTreeNodeStateColours___4(): pass

def _dynamic_initializer_for__kfCreatureDistance___2(): pass

def _dynamic_initializer_for__kvAnchorLimitsVec___7(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___113(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___113(): pass

def _dynamic_initializer_for__k3x4IdentityAt___109(): pass

def _dynamic_initializer_for__k3x4IdentityPos___105(): pass

def _dynamic_initializer_for__k3x4IdentityRight___109(): pass

def _dynamic_initializer_for__k3x4IdentityUp___109(): pass

def _dynamic_initializer_for__k4x4IdentityAt___19(): pass

def _dynamic_initializer_for__k4x4IdentityPos___19(): pass

def _dynamic_initializer_for__k4x4IdentityRight___19(): pass

def _dynamic_initializer_for__k4x4IdentityUp___19(): pass

def _dynamic_initializer_for__kNonCollidingPlane___16(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___17(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___18(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___27(): pass

def _dynamic_initializer_for__kUnitsPerQuantizedRegionAsVec___1(): pass

def _dynamic_initializer_for__kUnitsPerQuantizedRegionAsVecInv___1(): pass

def _dynamic_initializer_for__mCache___18(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___114(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___114(): pass

def _dynamic_initializer_for__k3x4IdentityAt___110(): pass

def _dynamic_initializer_for__k3x4IdentityPos___106(): pass

def _dynamic_initializer_for__k3x4IdentityRight___110(): pass

def _dynamic_initializer_for__k3x4IdentityUp___110(): pass

def _dynamic_initializer_for__k4x4IdentityAt___20(): pass

def _dynamic_initializer_for__k4x4IdentityPos___20(): pass

def _dynamic_initializer_for__k4x4IdentityRight___20(): pass

def _dynamic_initializer_for__k4x4IdentityUp___20(): pass

def _dynamic_initializer_for__kNonCollidingPlane___17(): pass

def _dynamic_initializer_for__kShaderMillCategoryColours___18(): pass

def _dynamic_initializer_for__kTkInvalidIDSeed___19(): pass

def _dynamic_initializer_for__kTkSelectionDataNull___28(): pass

def _dynamic_initializer_for__kUnitsPerQuantizedRegionAsVec___2(): pass

def _dynamic_initializer_for__kUnitsPerQuantizedRegionAsVecInv___2(): pass

def _dynamic_initializer_for__mCache___19(): pass

def _dynamic_initializer_for__AkAudioInputSourceRegistration__(): pass

def _dynamic_initializer_for__AkCompressorFXRegistration__(): pass

def _dynamic_initializer_for__AkConvolutionReverbFXRegistration__(): pass

def _dynamic_initializer_for__AkDelayFXRegistration__(): pass

def _dynamic_initializer_for__AkFlangerFXRegistration__(): pass

def _dynamic_initializer_for__AkGainFXRegistration__(): pass

def _dynamic_initializer_for__AkGuitarDistortionFXRegistration__(): pass

def _dynamic_initializer_for__AkHarmonizerFXRegistration__(): pass

def _dynamic_initializer_for__AkMatrixReverbFXRegistration__(): pass

def _dynamic_initializer_for__AkMeterFXRegistration__(): pass

def _dynamic_initializer_for__CAkTrackedGameObjComponent_1___sList__(): pass

def _dynamic_initializer_for__CAkTrackedGameObjComponent_3___sList__(): pass

def _dynamic_initializer_for__CAkTrackedGameObjComponent_4___sList__(): pass

def _dynamic_initializer_for__CAkTrackedGameObjComponent_5___sList__(): pass

def _dynamic_initializer_for__CAkTrackedGameObjComponent_0___sList__(): pass

def _dynamic_initializer_for__CAkMusicRenderer__m_listCtx__(): pass

def _dynamic_initializer_for__CAkMusicRenderer__m_queuePendingStateChanges__(): pass

def _dynamic_initializer_for__CAkMusicRenderer__m_segmentInfoRepository__(): pass

def _dynamic_initializer_for__s_panicRule__(): pass

def _dynamic_initializer_for__AkParametricEQFXRegistration__(): pass

def _dynamic_initializer_for__AkPeakLimiterFXRegistration__(): pass

def _dynamic_initializer_for__AkPitchShifterFXRegistration__(): pass

def _dynamic_initializer_for__AkRoomVerbFXRegistration__(): pass

def _dynamic_initializer_for__AkSilenceSourceRegistration__(): pass

def _dynamic_initializer_for__SineSourceRegistration__(): pass

def _dynamic_initializer_for__g_aBehavioralExtensions__(): pass

def _dynamic_initializer_for__g_csMain__(): pass

def _dynamic_initializer_for__g_settings__(): pass

def _dynamic_initializer_for__CAkEffectsMgr__m_RegisteredCodecList__(): pass

def _dynamic_initializer_for__CAkEffectsMgr__m_RegisteredFXList__(): pass

def _dynamic_initializer_for__CAkTrackedGameObjComponent_2___sList__(): pass

def _dynamic_initializer_for__CAkTrackedGameObjComponent_6___sList__(): pass

def _dynamic_initializer_for__CAkTrackedGameObjComponent_9___sList__(): pass

def _dynamic_initializer_for__CAkURenderer__m_CtxNotifLock__(): pass

def _dynamic_initializer_for__CAkURenderer__m_CtxNotifQueue__(): pass

def _dynamic_initializer_for__CAkURenderer__m_GlobalLimiter__(): pass

def _dynamic_initializer_for__CAkURenderer__m_Limiters__(): pass

def _dynamic_initializer_for__CAkURenderer__m_listActiveNodes__(): pass

def _dynamic_initializer_for__CAkURenderer__m_listCtxs__(): pass

def _dynamic_initializer_for__g_ActivityChunkPool__(): pass

def _dynamic_initializer_for__g_AllParamTargets__(): pass

def AkAudiolibTimer___dynamic_initializer_for__items__(): pass

def _dynamic_initializer_for__CAkBus__m_BackgroundMusicLock__(): pass

def _dynamic_initializer_for__CAkBus__s_BGMBusses__(): pass

def _dynamic_initializer_for__CAkBus__s_TopBusses__(): pass

def _dynamic_initializer_for__CAkListener__m_dirty__(): pass

def _dynamic_initializer_for__CAkOutputMgr__m_listDevices__(): pass

def _dynamic_initializer_for__CAkOutputMgr__m_setAssignedListeners__(): pass

def _dynamic_initializer_for__CAkLEngine__m_ParallelProcessingLock__(): pass

def _dynamic_initializer_for__CAkLEngine__m_Sources__(): pass

def _dynamic_initializer_for__CAkLEngine__m_arrayVPLDepthCnt__(): pass

def _dynamic_initializer_for__CAkLEngine__m_arrayVPLDepthInterconnects__(): pass

def _dynamic_initializer_for__CAkLEngine__m_arrayVPLs__(): pass

def _dynamic_initializer_for__AkHdrBus__m_mapMaxHdrWinTops__(): pass

def _dynamic_initializer_for__s_activeEmitters__(): pass

def _dynamic_initializer_for__g_CacheGameSyncMons__(): pass

def _dynamic_initializer_for__g_CachePinnedFiles__(): pass

def _dynamic_initializer_for__g_CachePinnedFilesLock__(): pass

def _dynamic_initializer_for__AkFXMemAlloc__m_instanceLower__(): pass

def _dynamic_initializer_for__AkFXMemAlloc__m_instanceUpper__(): pass

def _dynamic_initializer_for__CAkLEngineCmds__m_listCmd__(): pass

def _dynamic_initializer_for__CAkLEngineCmds__m_listSrcsNotConnected__(): pass

def _dynamic_initializer_for__AkCustomPluginDataStore__m_PluginCustomGameData__(): pass

def _dynamic_initializer_for__CAkBankList__m_BankListLock__(): pass

def _dynamic_initializer_for__g_SwitchCntrChunkPool__(): pass

def _dynamic_initializer_for__g_LayerCntrChunkPool__(): pass

def _dynamic_initializer_for__AkSystemOutputMetaRegistration__(): pass

def _dynamic_initializer_for__AuxSinkRegistration__(): pass

def _dynamic_initializer_for__CommSinkRegistration__(): pass

def _dynamic_initializer_for__DefaultSinkRegistration__(): pass

def _dynamic_initializer_for__DummySinkRegistration__(): pass

def _dynamic_initializer_for__PadSinkRegistration__(): pass

def _dynamic_initializer_for__PersonalSinkRegistration__(): pass

def _dynamic_initializer_for__CAkMixBusCtx__s_AuxRoutableBusses__(): pass

def _dynamic_initializer_for__AkStereoDelayFXRegistration__(): pass

def _dynamic_initializer_for__AK__StreamMgr__CAkStreamMgr__m_arDevices__(): pass

def _dynamic_initializer_for__AK__StreamMgr__CAkStreamMgr__m_arLangChgObserver__(): pass

def _dynamic_initializer_for__AkTremoloFXRegistration__(): pass

def _dynamic_initializer_for__AkVorbisDecoderRegistration__(): pass

def _dynamic_initializer_for__g_VorbisCodebookMgr__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdStaticTree__AabbTree_hkcdCompressedAabbCodecs__Aabb4BytesCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdStaticTree__AabbTree_hkcdCompressedAabbCodecs__Aabb5BytesCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdStaticTree__AabbTree_hkcdCompressedAabbCodecs__UncompressedAabbCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__CodecRaw_unsigned_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__CodecRaw_hkReflect__Typedef__hkUlong_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__CodecRaw_unsigned___int64_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DefaultDynamicStorage_hkcdDynamicTree__CodecRaw_unsigned_int_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DefaultDynamicStorage_hkcdDynamicTree__CodecRaw_unsigned___int64_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DefaultDynamicStorage_hkcdDynamicTree__Codec32_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DefaultDynamicStorage_hkcdDynamicTree__CodecInt16_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DefaultDynamicStorage_hkReflect__Typedef__hkcdDynamicTree_CodecRawUint_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DefaultDynamicStorage_hkReflect__Typedef__hkcdDynamicTree_CodecRawUlong_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DynamicStorage_0_hkcdDynamicTree__AnisotropicMetric_hkcdDynamicTree__CodecRaw_unsigned_int_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DynamicStorage_0_hkcdDynamicTree__AnisotropicMetric_hkcdDynamicTree__CodecRaw_unsigned___int64_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DynamicStorage_0_hkcdDynamicTree__AnisotropicMetric_hkcdDynamicTree__Codec32_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DynamicStorage_0_hkcdDynamicTree__AnisotropicMetric_hkcdDynamicTree__CodecInt16_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DynamicStorage_0_hkcdDynamicTree__AnisotropicMetric_hkReflect__Typedef__hkcdDynamicTree_CodecRawUint_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__DynamicStorage_0_hkcdDynamicTree__AnisotropicMetric_hkReflect__Typedef__hkcdDynamicTree_CodecRawUlong_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__Tree_hkcdDynamicTree__DefaultDynamicStorage_hkcdDynamicTree__CodecRaw_unsigned_int_________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__Tree_hkcdDynamicTree__DefaultDynamicStorage_hkcdDynamicTree__CodecRaw_unsigned___int64_________typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__Tree_hkcdDynamicTree__DefaultDynamicStorage_hkcdDynamicTree__Codec32_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__Tree_hkcdDynamicTree__DefaultDynamicStorage_hkcdDynamicTree__CodecInt16_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__Tree_hkReflect__Typedef__hkcdDynamicTree_DynamicStorage16_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__Tree_hkReflect__Typedef__hkcdDynamicTree_DynamicStorage32_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__Tree_hkReflect__Typedef__hkcdDynamicTree_DynamicStorageInt16_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdDynamicTree__Tree_hkReflect__Typedef__hkcdDynamicTree_DynamicStoragePtr_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkTrait__UninitializedStorage_hkReflect__Typedef__hkVector4_Tag_3_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkTrait__UninitializedStorage_hkVector4f_3_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkBuiltinContainerAllocator_0_32_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_unsigned_int_4294967295_hkcdStaticMeshTree__PrimitiveKeyDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkHandle_hkReflect__Typedef__hkUint32_Tag_4294967295_hkcdStaticMeshTree__PrimitiveKeyDiscriminant_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_unsigned_short_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdDynamicTree__CodecRaw_unsigned_int_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdDynamicTree__CodecRaw_unsigned___int64_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkHandle_unsigned_int_4294967295_hkcdStaticMeshTree__PrimitiveKeyDiscriminant_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkHandle_hkReflect__Typedef__hkUint32_Tag_4294967295_hkcdStaticMeshTree__PrimitiveKeyDiscriminant_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdCompressedAabbCodecs__Aabb4BytesCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdCompressedAabbCodecs__Aabb5BytesCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdDynamicTree__Codec32_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdDynamicTree__CodecInt16_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdSimdTree__Node_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdStaticMeshTree__Primitive_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdDefaultStaticMeshTree__PrimitiveDataRun_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdStaticMeshTree__Section_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdStaticMeshTree__Connectivity__SectionHeader_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkReflect__Typedef__hkUint16_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkReflect__Typedef__hkcdDynamicTree_CodecRawUint_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkReflect__Typedef__hkcdDynamicTree_CodecRawUlong_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_hkcdCompressedAabbCodecs__UncompressedAabbCodec_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelArray_unsigned___int64_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkTuple_unsigned_int_unsigned_int_void_void_void_void_void_void_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkTuple_hkReflect__Typedef__hkUint32_Tag_hkReflect__Typedef__hkUint32_Tag_void_void_void_void_void_void_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkcdPlanarGeometryPrimitives__Collection_28_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkTuple_unsigned_int_unsigned_int_void_void_void_void_void_void__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkTuple_hkReflect__Typedef__hkUint32_Tag_hkReflect__Typedef__hkUint32_Tag_void_void_void_void_void_void__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkcdPlanarSolid__Node_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkcdSimdTree__Dynamic__NodeData_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkcdPlanarGeometryPrimitives__Plane_hkBuiltinContainerAllocator_0_32_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkAabb_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkgpDeprecatedConvexDecomposition__Config__eHollowParts_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkgpDeprecatedConvexDecomposition__Config__eHollowParts_hkReflect__Typedef__hkInt32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkgpDeprecatedConvexDecomposition__Config__ePrimaryMethod_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkgpDeprecatedConvexDecomposition__Config__ePrimaryMethod_hkReflect__Typedef__hkInt32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkgpDeprecatedConvexDecomposition__Config__eReduceMethod_int_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkgpDeprecatedConvexDecomposition__Config__eReduceMethod_hkReflect__Typedef__hkInt32_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkInplaceArray_hkReflect__Typedef__hkVector4_Tag_3_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkInplaceArray_hkVector4f_3_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkReflect__Typedef__hkcdPlanarEntity_PlanesCollection_Tag_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkcdPlanarGeometryPlanesCollection_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkcdPlanarSolid__NodeStorage_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkReflect__Typedef__hkcdPlanarEntity_PlanesCollection_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkcdPlanarGeometry_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkcdPlanarGeometryPlanesCollection_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkcdPlanarGeometryPolygonCollection_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkcdPlanarSolid_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_int__26__32___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint32_Tag__26__32___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_int_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned_int_26___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_float_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkInt64_Tag_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkReal_Tag_6___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint32_Tag_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUint32_Tag_26___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkReflect__Typedef__hkUlong_Tag_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection___int64_4___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_unsigned___int64_2___typeRegNode__(): pass

def _dynamic_initializer_for__hkConvexDecompositionImpl_Config_Orientation_typeRegNode__(): pass

def _dynamic_initializer_for__hkConvexDecompositionImpl_Config_typeRegNode__(): pass

def _dynamic_initializer_for__hkConvexDecomposition_Config_Quality_typeRegNode__(): pass

def _dynamic_initializer_for__hkConvexDecomposition_Config_typeRegNode__(): pass

def _dynamic_initializer_for__hkConvexDecomposition_OutputInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdCompressedAabbCodecs_Aabb4BytesCodec_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdCompressedAabbCodecs_Aabb5BytesCodec_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdCompressedAabbCodecs_Aabb6BytesCodec_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdCompressedAabbCodecs_AabbCodecBase_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdCompressedAabbCodecs_CompressedAabbCodec_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdCompressedAabbCodecs_UncompressedAabbCodec_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDefaultStaticMeshTree_PrimitiveDataRun_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDefaultStaticMeshTree_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_AnisotropicMetric_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_BalanceMetric_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_CentroidMetric_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_Codec18_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_Codec32_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_CodecInt16_IntAabb_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_CodecInt16_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_CodecRawUint_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_CodecRawUlong_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_DefaultTree32_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_DefaultTree48_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_DefaultTreeInt16_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_DefaultTreePtr_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_DynamicStorage16_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_DynamicStorage32_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_DynamicStorageInt16_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdDynamicTree_DynamicStoragePtr_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdFourAabb_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdFractureEngine_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdHiResStaticMeshTree_PrimitiveDataRun_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdManifold4_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdObb_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarCsgOperand_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarEntity_PlanesCollection_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarEntity_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarGeometryPlanesCollection_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarGeometryPolygonCollection_Material_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarGeometryPolygonCollection_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarGeometryPrimitives_Plane_Int64Vector4_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarGeometryPrimitives_Plane_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarGeometry_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarSolid_NodeStorage_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarSolid_Node_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdPlanarSolid_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdRayCastResult_Enum_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdSimdTree_Dynamic_NodeData_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdSimdTree_Dynamic_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdSimdTree_Node_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdSimdTree_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticMeshTree_Base_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticMeshTree_Connectivity_SectionHeader_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticMeshTree_Connectivity_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticMeshTree_Primitive_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticMeshTree_Section_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticTree_Aabb4BytesTree_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticTree_Aabb5BytesTree_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticTree_Aabb6BytesTree_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticTree_AabbTreeBase_typeRegNode__(): pass

def _dynamic_initializer_for__hkcdStaticTree_AabbUncompressedTree_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_Config_eHollowParts_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_Config_ePrimaryMethod_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_Config_eReduceMethod_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_Config_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_GuardGenConfig_eMethod_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_GuardGenConfig_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_MeshPreProcessorConfig_typeRegNode__(): pass

def hkcdRayCastTriangle___dynamic_initializer_for__g_defaultTolerance__(): pass

def hkcdRayCastTriangle___dynamic_initializer_for__g_tolerance__(): pass

def _anonymous_namespace____dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_defaultInternal__(): pass

def _anonymous_namespace____dynamic_initializer_for__hkgpDeprecatedConvexDecomposition_defaultProgress__(): pass

def _dynamic_initializer_for__s_debugLog___21(): pass

def _dynamic_initializer_for__s_debugLog___22(): pass

def _dynamic_initializer_for__s_debugLog___23(): pass

def _dynamic_initializer_for__s_hkcdDynamicTreeLogOrigin__(): pass

def _dynamic_initializer_for__s_debugLog___24(): pass

def _dynamic_initializer_for__s_hkcdStaticMeshLogOrigin__(): pass

def _dynamic_initializer_for__initialEdges__(): pass

def _dynamic_initializer_for__simd2powneg23__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkMeshBoneIndexMapping___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkDefaultCompoundMeshShape___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkMeshBody___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkMeshShape___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkMeshVertexBuffer___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_short_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMeshBody___hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMeshVertexBuffer___hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMeshShape_const___hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkSkinnedMeshShape__BoneSection_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkSkinnedMeshShape__BoneSet_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMultipleVertexBuffer__ElementInfo_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkaSkeleton__LocalFrameOnBone_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMultipleVertexBuffer__LockedElement_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkDefaultCompoundMeshShape__MeshSection_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkSkinnedMeshShape__Part_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkaSkeleton__Partition_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMemoryMeshShape__Section_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMultipleVertexBuffer__VertexBufferInfo_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkInt16_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkMatrix4_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMeshBoneIndexMapping_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkReflect__Typedef__hkQsTransform_Tag_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMatrix4Impl_float__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkRefPtr_hkMeshTexture__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkQsTransformf_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkaBone_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkVertexFormat__ComponentType_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkVertexFormat__ComponentType_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkVertexFormat__ComponentUsage_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkVertexFormat__ComponentUsage_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshTexture__FilterMode_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshTexture__FilterMode_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshTexture__Format_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshTexture__Format_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshSection__MeshSectionIndexType_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshSection__MeshSectionIndexType_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshSection__PrimitiveType_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshSection__PrimitiveType_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshTexture__TextureUsageType_signed_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkEnum_enum_hkMeshTexture__TextureUsageType_hkReflect__Typedef__hkInt8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hkVertexFormat__HintFlags_unsigned_char_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkFlags_enum_hkVertexFormat__HintFlags_hkReflect__Typedef__hkUint8_Tag_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkMeshShape_const______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkIndexedTransformSet_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkMemoryMeshVertexBuffer_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkMeshMaterial_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkMeshShape_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkMeshTexture_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkMeshVertexBuffer_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkSkinnedMeshShape_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_int_32___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__Detail__RepeatReflection_hkVertexFormat__Element_32___typeRegNode__(): pass

def _dynamic_initializer_for__hkCompressedMassProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hkDefaultCompoundMeshBody_typeRegNode__(): pass

def _dynamic_initializer_for__hkDefaultCompoundMeshShape_MeshSection_typeRegNode__(): pass

def _dynamic_initializer_for__hkDefaultCompoundMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hkDiagonalizedMassProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hkIndexedTransformSet_typeRegNode__(): pass

def _dynamic_initializer_for__hkMassProperties_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryMeshBody_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryMeshMaterial_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryMeshShape_Section_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryMeshSystem_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryMeshTexture_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryMeshVertexBuffer_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshBody_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshBoneIndexMapping_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshMaterial_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshSectionCinfo_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshSection_MeshSectionIndexType_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshSection_PrimitiveType_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshSection_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshSystem_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshTexture_FilterMode_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshTexture_Format_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshTexture_RawBufferDescriptor_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshTexture_TextureUsageType_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshTexture_typeRegNode__(): pass

def _dynamic_initializer_for__hkMeshVertexBuffer_typeRegNode__(): pass

def _dynamic_initializer_for__hkMultipleVertexBuffer_ElementInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hkMultipleVertexBuffer_LockedElement_typeRegNode__(): pass

def _dynamic_initializer_for__hkMultipleVertexBuffer_VertexBufferInfo_typeRegNode__(): pass

def _dynamic_initializer_for__hkMultipleVertexBuffer_typeRegNode__(): pass

def _dynamic_initializer_for__hkSkinBinding_typeRegNode__(): pass

def _dynamic_initializer_for__hkSkinnedMeshShape_BoneSection_typeRegNode__(): pass

def _dynamic_initializer_for__hkSkinnedMeshShape_BoneSet_typeRegNode__(): pass

def _dynamic_initializer_for__hkSkinnedMeshShape_Part_typeRegNode__(): pass

def _dynamic_initializer_for__hkSkinnedMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hkSkinnedRefMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hkStorageSkinnedMeshShape_typeRegNode__(): pass

def _dynamic_initializer_for__hkVertexFormat_ComponentType_typeRegNode__(): pass

def _dynamic_initializer_for__hkVertexFormat_ComponentUsage_typeRegNode__(): pass

def _dynamic_initializer_for__hkVertexFormat_Element_typeRegNode__(): pass

def _dynamic_initializer_for__hkVertexFormat_HintFlags_typeRegNode__(): pass

def _dynamic_initializer_for__hkVertexFormat_typeRegNode__(): pass

def _dynamic_initializer_for__hkaBone_typeRegNode__(): pass

def _dynamic_initializer_for__hkaSkeleton_LocalFrameOnBone_typeRegNode__(): pass

def _dynamic_initializer_for__hkaSkeleton_Partition_typeRegNode__(): pass

def _dynamic_initializer_for__hkaSkeleton_typeRegNode__(): pass

def _dynamic_initializer_for__s_debugLog___25(): pass

def _dynamic_initializer_for__s_debugLog___26(): pass

def _anonymous_namespace____dynamic_initializer_for__epsilon__(): pass

def _dynamic_initializer_for__s_keycodeLogOrigin__(): pass

def _anonymous_namespace____dynamic_initializer_for__s_setKeycodesNode__(): pass

def _dynamic_initializer_for__hkgpCgo_Config_SolverAccuracy_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpCgo_Config_VertexCombinator_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpCgo_Config_VertexSemantic_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpCgo_Config_typeRegNode__(): pass

def _dynamic_initializer_for__hkgpEnhancedConvexHull_SolverConfig_typeRegNode__(): pass

def _dynamic_initializer_for__gIConvexOverlapImpl__(): pass

def _dynamic_initializer_for__s_debugLog___27(): pass

def hkLog__Origins___dynamic_initializer_for__geoProcessing__(): pass

def _dynamic_initializer_for__hkKeycodeDataBase__singleton__(): pass

def _dynamic_initializer_for__hkKeycodeDataBase__singletonEntry__(): pass

def _dynamic_initializer_for__s_debugLog___28(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRelPtr_hknpCompressedMeshShape_const______typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompressedMeshShapeData_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompressedMeshShapeInternals_KeyMask_typeRegNode__(): pass

def _dynamic_initializer_for__hknpCompressedMeshShapeTree_typeRegNode__(): pass

def _anonymous_namespace____dynamic_initializer_for__s_linked__(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___115(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___115(): pass

def _dynamic_initializer_for__s_debugLog___29(): pass

def _dynamic_initializer_for__hkReflect__VarN__s_invalid__(): pass

def _dynamic_initializer_for__s_debugLog___30(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatBinaryPackfile2014__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatBinaryTagfile2014__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatBinaryTagfile2015__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatBinaryTagfile__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatBinaryTagfileNT6__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatCompressedTagfile__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatXmlPackfile2010__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatXmlTagfile2014__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatXmlTagfile__(): pass

def hkSerialize__Detail___dynamic_initializer_for__fileFormatYamlfile__(): pass

def _dynamic_initializer_for__s_debugLog___31(): pass

def _dynamic_initializer_for__hkProductFeatures__Common__ReadFormatBinaryTagfile__s_optionalComponent__(): pass

def _dynamic_initializer_for__s_debugLog___32(): pass

def _anonymous_namespace____dynamic_initializer_for__s_readType_opaque_impl__(): pass

def _anonymous_namespace____dynamic_initializer_for__s_readType_void_impl__(): pass

def _dynamic_initializer_for__hkProductFeatures__Common__WriteFormatBinaryTagfile__s_optionalComponent__(): pass

def _anonymous_namespace____dynamic_initializer_for__s_cmpSigOrigin__(): pass

def _dynamic_initializer_for__s_debugLog___33(): pass

def _dynamic_initializer_for__hkProductFeatures__Common__ReadFormatXmlTagfile__s_optionalComponent__(): pass

def _dynamic_initializer_for__s_debugLog___34(): pass

def _anonymous_namespace____dynamic_initializer_for__s_readType_opaque_impl___0(): pass

def _anonymous_namespace____dynamic_initializer_for__s_readType_void_impl___0(): pass

def _dynamic_initializer_for__hkProductFeatures__Common__WriteFormatXmlTagfile__s_optionalComponent__(): pass

def _dynamic_initializer_for__s_debugLog___35(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkMemoryResourceContainer___typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkMemoryResourceHandle__ExternalLink_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkRefPtr_hkMemoryResourceContainer__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkRefPtr_hkMemoryResourceHandle__hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArray_hkRootLevelContainer__NamedVariant_hkBuiltinContainerAllocator_0_0_______typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkMemoryResourceContainer_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkRefPtr_hkMemoryResourceHandle_____typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryResourceContainer_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryResourceHandle_ExternalLink_typeRegNode__(): pass

def _dynamic_initializer_for__hkMemoryResourceHandle_typeRegNode__(): pass

def _dynamic_initializer_for__hkObjectResource_typeRegNode__(): pass

def _dynamic_initializer_for__hkResourceBase_typeRegNode__(): pass

def _dynamic_initializer_for__hkResourceContainer_typeRegNode__(): pass

def _dynamic_initializer_for__hkResourceHandle_typeRegNode__(): pass

def _dynamic_initializer_for__hkResource_typeRegNode__(): pass

def _dynamic_initializer_for__hkRootLevelContainer_NamedVariant_typeRegNode__(): pass

def _dynamic_initializer_for__hkRootLevelContainer_typeRegNode__(): pass

def _dynamic_initializer_for__hkSerialize_Note_Export_typeRegNode__(): pass

def _dynamic_initializer_for__hkSerialize_Note_Import_typeRegNode__(): pass

def _dynamic_initializer_for__s_debugLog___36(): pass

def _dynamic_initializer_for__hkDebugDisplayMarker_Type_typeRegNode__(): pass

def _dynamic_initializer_for__hkDebugDisplayMarker_typeRegNode__(): pass

def _dynamic_initializer_for__hkDisplayMaterialDatas_typeRegNode__(): pass

def _dynamic_initializer_for__hkInternalObjectReflectType_typeRegNode__(): pass

def _dynamic_initializer_for__hkProcessCapabilities_typeRegNode__(): pass

def _dynamic_initializer_for__hkProcessUtils_TimerMarker_typeRegNode__(): pass

def _dynamic_initializer_for__hkServerSettings_typeRegNode__(): pass

def _dynamic_initializer_for__hkStatisticsProcess_CreationOptions_typeRegNode__(): pass

def _dynamic_initializer_for__hkStatisticsProcess_Options_MemoryMonitors_typeRegNode__(): pass

def _dynamic_initializer_for__hkStatisticsProcess_Options_typeRegNode__(): pass

def _dynamic_initializer_for__hkVdb_Detail_PtrOffsetIndex_typeRegNode__(): pass

def _dynamic_initializer_for__hkVdb_Detail_ToString_typeRegNode__(): pass

def _dynamic_initializer_for__hkVdb_Tweakable_typeRegNode__(): pass

def _dynamic_initializer_for___anonymous_namespace___VdbObjectAllocImpl__s_instance__(): pass

def _dynamic_initializer_for___anonymous_namespace___VdbObjectCompoundImpl__s_instance__(): pass

def _dynamic_initializer_for___anonymous_namespace___VdbObjectPointerImpl__s_instance__(): pass

def _dynamic_initializer_for___anonymous_namespace___VdbObjectStringImpl__s_instance__(): pass

def _dynamic_initializer_for__hkProcessFactory__singleton__(): pass

def _dynamic_initializer_for__hkProcessFactory__singletonEntry__(): pass

def _dynamic_initializer_for__s_debugLog___37(): pass

def _dynamic_initializer_for__hkDebugDisplay__singleton__(): pass

def _dynamic_initializer_for__hkDebugDisplay__singletonEntry__(): pass

def _dynamic_initializer_for__s_debugLog___38(): pass

def _dynamic_initializer_for__MasteringSuiteFX__s_instanceLock__(): pass

def _dynamic_initializer_for__MasteringSuiteFXRegistration__(): pass

def _anonymous_namespace____dynamic_initializer_for__g_audioDeviceIDs__(): pass

def _anonymous_namespace____dynamic_initializer_for__g_lastBufferTicks__(): pass

def _anonymous_namespace____dynamic_initializer_for__g_sharesetCrossfaders__(): pass

def _dynamic_initializer_for__vFBase10to2Const__(): pass

def _dynamic_initializer_for__vFBase2to10Const__(): pass

def _dynamic_initializer_for__vFLogConstA__(): pass

def _dynamic_initializer_for__vFLogConstB__(): pass

def _dynamic_initializer_for__vFLogConstC__(): pass

def _dynamic_initializer_for__vFPowConstA__(): pass

def _dynamic_initializer_for__vFPowConstB__(): pass

def _dynamic_initializer_for__vFPowConstC__(): pass

def _dynamic_initializer_for__vFloatConst0__(): pass

def _dynamic_initializer_for__vFloatConst0_05__(): pass

def _dynamic_initializer_for__vFloatConst0_5__(): pass

def _dynamic_initializer_for__vFloatConst10__(): pass

def _dynamic_initializer_for__vFloatConst1__(): pass

def _dynamic_initializer_for__vFloatConstMinus100__(): pass

def _dynamic_initializer_for__vFloatConstMinus1__(): pass

def _dynamic_initializer_for__vInt127__(): pass

def _dynamic_initializer_for__vIntLogAndOp1__(): pass

def _dynamic_initializer_for__vIntLogAndOp2__(): pass

def _dynamic_initializer_for__vIntLogOrOp__(): pass

def _dynamic_initializer_for__vFBase10to2Const___0(): pass

def _dynamic_initializer_for__vFBase2to10Const___0(): pass

def _dynamic_initializer_for__vFLogConstA___0(): pass

def _dynamic_initializer_for__vFLogConstB___0(): pass

def _dynamic_initializer_for__vFLogConstC___0(): pass

def _dynamic_initializer_for__vFPowConstA___0(): pass

def _dynamic_initializer_for__vFPowConstB___0(): pass

def _dynamic_initializer_for__vFPowConstC___0(): pass

def _dynamic_initializer_for__vFloatConst0___0(): pass

def _dynamic_initializer_for__vFloatConst0_05___0(): pass

def _dynamic_initializer_for__vFloatConst0_5___0(): pass

def _dynamic_initializer_for__vFloatConst10___0(): pass

def _dynamic_initializer_for__vFloatConst1___0(): pass

def _dynamic_initializer_for__vFloatConstMinus100___0(): pass

def _dynamic_initializer_for__vFloatConstMinus1___0(): pass

def _dynamic_initializer_for__vInt127___0(): pass

def _dynamic_initializer_for__vIntLogAndOp1___0(): pass

def _dynamic_initializer_for__vIntLogAndOp2___0(): pass

def _dynamic_initializer_for__vIntLogOrOp___0(): pass

def _dynamic_initializer_for__vFBase10to2Const___1(): pass

def _dynamic_initializer_for__vFBase2to10Const___1(): pass

def _dynamic_initializer_for__vFLogConstA___1(): pass

def _dynamic_initializer_for__vFLogConstB___1(): pass

def _dynamic_initializer_for__vFLogConstC___1(): pass

def _dynamic_initializer_for__vFPowConstA___1(): pass

def _dynamic_initializer_for__vFPowConstB___1(): pass

def _dynamic_initializer_for__vFPowConstC___1(): pass

def _dynamic_initializer_for__vFloatConst0___1(): pass

def _dynamic_initializer_for__vFloatConst0_05___1(): pass

def _dynamic_initializer_for__vFloatConst0_5___1(): pass

def _dynamic_initializer_for__vFloatConst10___1(): pass

def _dynamic_initializer_for__vFloatConst1___1(): pass

def _dynamic_initializer_for__vFloatConstMinus100___1(): pass

def _dynamic_initializer_for__vFloatConstMinus1___1(): pass

def _dynamic_initializer_for__vInt127___1(): pass

def _dynamic_initializer_for__vIntLogAndOp1___1(): pass

def _dynamic_initializer_for__vIntLogAndOp2___1(): pass

def _dynamic_initializer_for__vIntLogOrOp___1(): pass

def _dynamic_initializer_for__vFBase10to2Const___2(): pass

def _dynamic_initializer_for__vFBase2to10Const___2(): pass

def _dynamic_initializer_for__vFLogConstA___2(): pass

def _dynamic_initializer_for__vFLogConstB___2(): pass

def _dynamic_initializer_for__vFLogConstC___2(): pass

def _dynamic_initializer_for__vFPowConstA___2(): pass

def _dynamic_initializer_for__vFPowConstB___2(): pass

def _dynamic_initializer_for__vFPowConstC___2(): pass

def _dynamic_initializer_for__vFloatConst0___2(): pass

def _dynamic_initializer_for__vFloatConst0_05___2(): pass

def _dynamic_initializer_for__vFloatConst0_5___2(): pass

def _dynamic_initializer_for__vFloatConst10___2(): pass

def _dynamic_initializer_for__vFloatConst1___2(): pass

def _dynamic_initializer_for__vFloatConstMinus100___2(): pass

def _dynamic_initializer_for__vFloatConstMinus1___2(): pass

def _dynamic_initializer_for__vInt127___2(): pass

def _dynamic_initializer_for__vIntLogAndOp1___2(): pass

def _dynamic_initializer_for__vIntLogAndOp2___2(): pass

def _dynamic_initializer_for__vIntLogOrOp___2(): pass

def _dynamic_initializer_for__vFBase10to2Const___3(): pass

def _dynamic_initializer_for__vFBase2to10Const___3(): pass

def _dynamic_initializer_for__vFLogConstA___3(): pass

def _dynamic_initializer_for__vFLogConstB___3(): pass

def _dynamic_initializer_for__vFLogConstC___3(): pass

def _dynamic_initializer_for__vFPowConstA___3(): pass

def _dynamic_initializer_for__vFPowConstB___3(): pass

def _dynamic_initializer_for__vFPowConstC___3(): pass

def _dynamic_initializer_for__vFloatConst0___3(): pass

def _dynamic_initializer_for__vFloatConst0_05___3(): pass

def _dynamic_initializer_for__vFloatConst0_5___3(): pass

def _dynamic_initializer_for__vFloatConst10___3(): pass

def _dynamic_initializer_for__vFloatConst1___3(): pass

def _dynamic_initializer_for__vFloatConstMinus100___3(): pass

def _dynamic_initializer_for__vFloatConstMinus1___3(): pass

def _dynamic_initializer_for__vInt127___3(): pass

def _dynamic_initializer_for__vIntLogAndOp1___3(): pass

def _dynamic_initializer_for__vIntLogAndOp2___3(): pass

def _dynamic_initializer_for__vIntLogOrOp___3(): pass

def _dynamic_initializer_for__vFBase10to2Const___4(): pass

def _dynamic_initializer_for__vFBase2to10Const___4(): pass

def _dynamic_initializer_for__vFLogConstA___4(): pass

def _dynamic_initializer_for__vFLogConstB___4(): pass

def _dynamic_initializer_for__vFLogConstC___4(): pass

def _dynamic_initializer_for__vFPowConstA___4(): pass

def _dynamic_initializer_for__vFPowConstB___4(): pass

def _dynamic_initializer_for__vFPowConstC___4(): pass

def _dynamic_initializer_for__vFloatConst0___4(): pass

def _dynamic_initializer_for__vFloatConst0_05___4(): pass

def _dynamic_initializer_for__vFloatConst0_5___4(): pass

def _dynamic_initializer_for__vFloatConst10___4(): pass

def _dynamic_initializer_for__vFloatConst1___4(): pass

def _dynamic_initializer_for__vFloatConstMinus100___4(): pass

def _dynamic_initializer_for__vFloatConstMinus1___4(): pass

def _dynamic_initializer_for__vInt127___4(): pass

def _dynamic_initializer_for__vIntLogAndOp1___4(): pass

def _dynamic_initializer_for__vIntLogAndOp2___4(): pass

def _dynamic_initializer_for__vIntLogOrOp___4(): pass

def _dynamic_initializer_for__vFBase10to2Const___5(): pass

def _dynamic_initializer_for__vFBase2to10Const___5(): pass

def _dynamic_initializer_for__vFLogConstA___5(): pass

def _dynamic_initializer_for__vFLogConstB___5(): pass

def _dynamic_initializer_for__vFLogConstC___5(): pass

def _dynamic_initializer_for__vFPowConstA___5(): pass

def _dynamic_initializer_for__vFPowConstB___5(): pass

def _dynamic_initializer_for__vFPowConstC___5(): pass

def _dynamic_initializer_for__vFloatConst0___5(): pass

def _dynamic_initializer_for__vFloatConst0_05___5(): pass

def _dynamic_initializer_for__vFloatConst0_5___5(): pass

def _dynamic_initializer_for__vFloatConst10___5(): pass

def _dynamic_initializer_for__vFloatConst1___5(): pass

def _dynamic_initializer_for__vFloatConstMinus100___5(): pass

def _dynamic_initializer_for__vFloatConstMinus1___5(): pass

def _dynamic_initializer_for__vInt127___5(): pass

def _dynamic_initializer_for__vIntLogAndOp1___5(): pass

def _dynamic_initializer_for__vIntLogAndOp2___5(): pass

def _dynamic_initializer_for__vIntLogOrOp___5(): pass

def _dynamic_initializer_for__vFBase10to2Const___6(): pass

def _dynamic_initializer_for__vFBase2to10Const___6(): pass

def _dynamic_initializer_for__vFLogConstA___6(): pass

def _dynamic_initializer_for__vFLogConstB___6(): pass

def _dynamic_initializer_for__vFLogConstC___6(): pass

def _dynamic_initializer_for__vFPowConstA___6(): pass

def _dynamic_initializer_for__vFPowConstB___6(): pass

def _dynamic_initializer_for__vFPowConstC___6(): pass

def _dynamic_initializer_for__vFloatConst0___6(): pass

def _dynamic_initializer_for__vFloatConst0_05___6(): pass

def _dynamic_initializer_for__vFloatConst0_5___6(): pass

def _dynamic_initializer_for__vFloatConst10___6(): pass

def _dynamic_initializer_for__vFloatConst1___6(): pass

def _dynamic_initializer_for__vFloatConstMinus100___6(): pass

def _dynamic_initializer_for__vFloatConstMinus1___6(): pass

def _dynamic_initializer_for__vInt127___6(): pass

def _dynamic_initializer_for__vIntLogAndOp1___6(): pass

def _dynamic_initializer_for__vIntLogAndOp2___6(): pass

def _dynamic_initializer_for__vIntLogOrOp___6(): pass

def _dynamic_initializer_for__vFBase10to2Const___7(): pass

def _dynamic_initializer_for__vFBase2to10Const___7(): pass

def _dynamic_initializer_for__vFLogConstA___7(): pass

def _dynamic_initializer_for__vFLogConstB___7(): pass

def _dynamic_initializer_for__vFLogConstC___7(): pass

def _dynamic_initializer_for__vFPowConstA___7(): pass

def _dynamic_initializer_for__vFPowConstB___7(): pass

def _dynamic_initializer_for__vFPowConstC___7(): pass

def _dynamic_initializer_for__vFloatConst0___7(): pass

def _dynamic_initializer_for__vFloatConst0_05___7(): pass

def _dynamic_initializer_for__vFloatConst0_5___7(): pass

def _dynamic_initializer_for__vFloatConst10___7(): pass

def _dynamic_initializer_for__vFloatConst1___7(): pass

def _dynamic_initializer_for__vFloatConstMinus100___7(): pass

def _dynamic_initializer_for__vFloatConstMinus1___7(): pass

def _dynamic_initializer_for__vInt127___7(): pass

def _dynamic_initializer_for__vIntLogAndOp1___7(): pass

def _dynamic_initializer_for__vIntLogAndOp2___7(): pass

def _dynamic_initializer_for__vIntLogOrOp___7(): pass

def _dynamic_initializer_for__g_sdkLibMutexVk__(): pass

def ngx__util___dynamic_initializer_for__g_mutex__(): pass

def std___dynamic_atexit_destructor_for___Fac_tidy_reg__(): pass

def _dynamic_initializer_for__s_debugLog___39(): pass

def _anonymous_namespace____dynamic_initializer_for__g_errorDispatcher__(): pass

def _anonymous_namespace____dynamic_initializer_for__g_nopDispatcher__(): pass

def _dynamic_initializer_for__hkConcurrency__singleton__(): pass

def _dynamic_initializer_for__hkConcurrency__singletonEntry__(): pass

def _dynamic_initializer_for__s_debugLog___40(): pass

def _anonymous_namespace____dynamic_initializer_for__TargetStrings__(): pass

def _dynamic_initializer_for__s_debugLog___41(): pass

def _dynamic_initializer_for__ReflectStrings__singleton__(): pass

def _dynamic_initializer_for__ReflectStrings__singletonEntry__(): pass

def _dynamic_initializer_for__hkReflect__Detail__PtrReflection_hkReflect__Type_____typeRegNode__(): pass

def _dynamic_initializer_for__hkReflect__ReflectionOf_hkArrayView_hkReflect__Type_______typeRegNode__(): pass

def _dynamic_initializer_for__s_debugLog___42(): pass

def _dynamic_initializer_for__hkReflect__Detail__ScopedPtrImpl__s_instance__(): pass

def _dynamic_initializer_for__s_debugLog___43(): pass

def _dynamic_initializer_for__hkProductFeatures__Physics__SimdTreeBroadPhase__s_optionalComponent__(): pass

def _dynamic_initializer_for__g_5BitErrorGuardDown___116(): pass

def _dynamic_initializer_for__g_5BitErrorGuardUp___116(): pass

def _anonymous_namespace____dynamic_initializer_for__s_instanceDefaultTypeCallbackType__(): pass

def vector_constructor_iterator(): pass

def vector_destructor_iterator(): pass

class cGcMatchmakingPlayfabGalaxy(ctypes.Structure):
    def GetLocalPlatformType(): pass

    def scalar_deleting_destructor(): pass

    def LoginToPlayfab(): pass

    def IsReadyToLogin(): pass

    def OnGameJoinRequested(): pass

    def Update(): pass

    def UpdateFriends(): pass

    def GetFriendName(): pass

    def GetExtraFriendsString(): pass

    def GetGameModeName(): pass

    def GetFriendLobbyCount(): pass

    def JoinFriend(): pass

    def GetPlatformType(): pass


def vector_copy_constructor_iterator(): pass

def _cGcApplication__VRAssertCallback____2____lambda_1___operator__(): pass

def _cGcApplicationDeathState__Update____69____lambda_2____lambda_2_(): pass

def GetLOCGameModeTitle(): pass

def _cGcApplicationGameModeSelectorState__UpdateManageCloudSavesUI____44____lambda_1___operator__(): pass

def _cGcApplicationGameModeSelectorState__DownloadCloudData____2____lambda_1___operator__(): pass

def cGcGameState__ForEachFleetManager__cGcApplicationDeathState__Update____8____lambda_1___(): pass

def std__pair_char_const___std__function_char_const_____cdecl_void______pair_char_const___std__function_char_const_____cdecl_void_____char_const_____6___cGcApplicationDeathState__Update____69____lambda_2__0_(): pass

def cGcNGuiLayer__ForEachChildElement__cGcFrontendPageWiki__DoWiki____30____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__DownloadCloudData____2____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__DownloadCloudData____2____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__DownloadCloudData____2____lambda_1__void_bool____Move(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__UpdateManageCloudSavesUI____44____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__UpdateManageCloudSavesUI____44____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__UpdateManageCloudSavesUI____44____lambda_1__void_bool____Move(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__SetupCloudSavesUI____5____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__SetupCloudSavesUI____5____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcApplicationGameModeSelectorState__SetupCloudSavesUI____5____lambda_1__void_bool____Move(): pass

def std___Func_impl_no_alloc__cGcApplicationDeathState__Update____69____lambda_2__char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcApplicationDeathState__Update____69____lambda_2__char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcApplicationDeathState__Update____69____lambda_2__char_const______Move(): pass

def ___emplace_back_VTkHandle__AEBV__function___A6A_AW4TkPollResult__XZ_std_____deque_U__pair_VTkHandle__V__function___A6A_AW4TkPollResult__XZ_std___std__V__TkSTLAllocatorShim_U__pair_VTkHandle__V__function___A6A_AW4TkPollResult__XZ_std___std___07_03___std__QEAA_A_T__QEAVTkHandle__AEBV__function___A6A_AW4TkPollResult__XZ_1__Z(): pass

def __local_stdio_scanf_options(): pass

def sscanf_s(): pass

def abs(): pass

def __local_stdio_printf_options(): pass

def vsnprintf(): pass

def _cGcApplicationLocalLoadState__Update____2____lambda_1___operator__(): pass

def EnqueueApplyShadowMask(): pass

def SetGlobalJitterValues(): pass

def EnableLightShafts(): pass

def EnqueueApplyAmbientOcclusionSettings(): pass

def EnqueueApplyRefractionSettings(): pass

def EnqueueTerrainStageSettings(): pass

def RenderNGuiUI(): pass

def GetCameraFovAdjust(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcApplicationLocalLoadState__Update____296____lambda_2___(): pass

def _cTkJobManager__CreateJobLambda__cEgFfxFsr2Draw__Draw____14____lambda_2_______2____lambda_1____lambda_invoker_cdecl_(): pass

def std___Func_impl_no_alloc__cGcApplicationSimulationState__ThreadSyncPoint____71____lambda_3__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcApplicationSimulationState__ThreadSyncPoint____71____lambda_3__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcApplicationSimulationState__ThreadSyncPoint____71____lambda_3__void____Move(): pass

def std___Func_impl_no_alloc__cGcApplicationSimulationState__ThreadSyncPoint____53____lambda_2__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcApplicationSimulationState__ThreadSyncPoint____53____lambda_2__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcApplicationSimulationState__ThreadSyncPoint____53____lambda_2__void____Move(): pass

def std___Func_impl_no_alloc__cGcApplicationSimulationState__ThreadSyncPoint____48____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcApplicationSimulationState__ThreadSyncPoint____48____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcApplicationSimulationState__Prepare____2____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcApplicationSimulationState__Prepare____2____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcApplicationSimulationState__Prepare____2____lambda_1____void____Move(): pass

def TkCalcUniformNameHashNoInl(): pass

def _cGcAtlasBrokerAuth__HttpHandleAuth____46____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__cGcAtlasBrokerAzure__Construct____2____lambda_1__void_unsigned___int64_unsigned___int64____Target_type(): pass

def std___Func_impl_no_alloc__cGcAtlasBrokerAzure__Construct____2____lambda_1__void_unsigned___int64_unsigned___int64____Do_call(): pass

def std___Func_impl_no_alloc__cGcAtlasBrokerAzure__Construct____2____lambda_1__void_unsigned___int64_unsigned___int64____Move(): pass

def ___emplace_back_AEAPEBD___vector_V__cTkFixedString__0KA_D__V__StackAllocator_V__cTkFixedString__0KA_D___03_0_0___std__QEAA_A_TAEAPEBD_Z(): pass

def WideStringFromUTF8(): pass

class VmaBlockMetadata_Linear(ctypes.Structure):
    def GetSumFreeSize(): pass

    def scalar_deleting_destructor(): pass

    def AddPoolStats(): pass

    def Alloc(): pass

    def CalcAllocationStatInfo(): pass

    def CleanupAfterFree(): pass

    def CreateAllocationRequest(): pass

    def Free(): pass

    def FreeAtOffset(): pass

    def GetAllocationCount(): pass

    def GetUnusedRangeSizeMax(): pass

    def Init(): pass

    def IsEmpty(): pass

    def MakeAllocationsLost(): pass

    def MakeRequestedAllocationsLost(): pass

    def PrintDetailedMap(): pass

    def Validate(): pass


def UpdateCommsChatterRTPCS(): pass

def ___emplace_back_AEAVTkAudioObject__AEAVTkAudioID__AEAVcTkVector3__AEAV3_AEAM___vector_UcGcDopplerRequest__V__TkSTLAllocatorShim_UcGcDopplerRequest___0BA__06___std__QEAA_A_TAEAVTkAudioObject__AEAVTkAudioID__AEAVcTkVector3__2AEAM_Z(): pass

def std___Sort_unchecked_cGcAudioManager__FootstepToSort____cGcAudioManager__SortAndCueFootsteps____2____lambda_1___(): pass

def std___Make_heap_unchecked_cGcAudioManager__FootstepToSort____cGcAudioManager__SortAndCueFootsteps____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cGcAudioManager__FootstepToSort____cGcAudioManager__SortAndCueFootsteps____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cGcAudioManager__FootstepToSort___cGcAudioManager__FootstepToSort__cGcAudioManager__SortAndCueFootsteps____2____lambda_1___(): pass

def std___Med3_unchecked_cGcAudioManager__FootstepToSort____cGcAudioManager__SortAndCueFootsteps____2____lambda_1___(): pass

def getNodeData(): pass

def getNodeData_0(): pass

def getNodeData_1(): pass

class cGcTriggerActionComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcByteBeatPlayerComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcByteBeat__SortForCulling____2____lambda_1___operator__(): pass

def _cGcByteBeat__RenderScreen____5____lambda_1___operator__(): pass

def _cGcBaseBuildingGenerator__LoadDecorationTemplatePattern____2____lambda_2___operator__(): pass

def _cGcBaseBuildingGenerator__AddDecorationSpawns____2____lambda_1___operator__(): pass

def _cGcBaseBuildingGenerator__ClearReferenceNodeScene____2____lambda_1___operator__(): pass

def std___Sort_unchecked_cGcByteBeat__cGcByteBeatInfo___std___Ref_fn__cGcByteBeat__SortForCulling____2____lambda_1_____(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__ClearReferenceNodeScene____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__ClearReferenceNodeScene____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__ClearReferenceNodeScene____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__SpawnDecorations____19____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__SpawnDecorations____19____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__SpawnDecorations____19____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateNode____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Delete_this(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__LoadDecorationTemplatePattern____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__LoadDecorationTemplatePattern____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__LoadDecorationTemplatePattern____2____lambda_2__bool_TkHandle____Copy(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__PollToLoadTemplatesTable____14____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__PollToLoadTemplatesTable____14____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__PollToLoadTemplatesTable____14____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__PollToLoadTemplatesTable____11____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__Update____B____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingGenerator__PollToLoadTemplatesTable____11____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__SetModelFadeAndLodsUpdate____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__SetModelFadeAndLodsUpdate____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__SetModelFadeAndLodsUpdate____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__DisableLODs____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__DoEnter____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__DisableLODs____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__UpdateRender____6____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__UpdateRender____6____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__UpdateRender____6____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__UpdateRender____6____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__UpdateRender____6____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingEffectsManager__UpdateRender____6____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__RetrieveOwnerDisplayName____8____lambda_1__void_cTkFixedString_64_char__const______Delete_this(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__RetrieveOwnerDisplayName____8____lambda_1__void_cTkFixedString_64_char__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__RetrieveOwnerDisplayName____8____lambda_1__void_cTkFixedString_64_char__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__RetrieveOwnerDisplayName____8____lambda_1__void_cTkFixedString_64_char__const______Move(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__RetrieveOwnerDisplayName____8____lambda_1__void_cTkFixedString_64_char__const______Copy(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__ProfanityCheckBase____5____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__ProfanityCheckBase____5____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__ProfanityCheckBase____5____lambda_1__void_bool____Move(): pass

def std___Func_impl_no_alloc__cGcAsyncOpsTempBase__ProfanityCheckBase____5____lambda_1__void_bool____Copy(): pass

def std___Func_impl_no_alloc__cGcByteBeat__Prepare____5____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcByteBeat__Prepare____5____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcByteBeat__Prepare____5____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcByteBeat__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcByteBeat__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcByteBeat__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Partition_by_median_guess_unchecked_cGcByteBeat__cGcByteBeatInfo___std___Ref_fn__cGcByteBeat__SortForCulling____2____lambda_1_____(): pass

def std___Pop_heap_hole_by_index_cGcByteBeat__cGcByteBeatInfo___cGcByteBeat__cGcByteBeatInfo_std___Ref_fn__cGcByteBeat__SortForCulling____2____lambda_1_____(): pass

def std___Med3_unchecked_cGcByteBeat__cGcByteBeatInfo___std___Ref_fn__cGcByteBeat__SortForCulling____2____lambda_1_____(): pass

def fprintf(): pass

def ___emplace_back_AEBV__function___A6A_AW4TkPollResult__XZ_std_____list_V__function___A6A_AW4TkPollResult__XZ_std__V__allocator_V__function___A6A_AW4TkPollResult__XZ_std___2__std__QEAA_A_TAEBV__function___A6A_AW4TkPollResult__XZ_1__Z(): pass

class cGcInteractionComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcRefinerUnitComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcObjectSpawnerComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcNPCHabitationComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcOutpostComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcHologramComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcVehicleGarageComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcBaseBuildingManager__UpdateModeChange____8____lambda_1___operator__(): pass

def _cGcBaseBuildingManager__CreateObjectTemplate____5____lambda_1___operator__(): pass

def _cGcBaseBuildingManager__CreateObjectTemplate____5____lambda_2___operator__(): pass

def std__vector__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap_StackAllocator__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap_32__1_____reserve(): pass

def std__vector__TkParticle__NGuiRender____5___Info_TkSTLAllocatorShim__TkParticle__NGuiRender____5___Info_8__1______Xlength(): pass

def std__vector__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap_StackAllocator__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap_32__1______Change_array(): pass

def std__sort_std___Vector_iterator_std___Vector_val_std___Simple_types_GhostOption_______cGcBaseBuildingManager__SetBestSnappingOptionsOnPlaceholder____2____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cGcBaseBuildingManager__CreateObjectTemplate____25____lambda_3___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcBaseBuildingManager__DeleteBuildingNode____37____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcBaseBuildingManager__GeneratePartData____2____lambda_1___(): pass

def std__vector__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap_StackAllocator__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap_32__1______Emplace_one_at_back__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap_const___(): pass

def std___Uninitialized_move__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap___StackAllocator__cGcBaseBuildingManager__GetSnappingOptionsForPlaceholder____2___PlacementSnap_32__1___(): pass

def std___Sort_unchecked_GhostOption____cGcBaseBuildingManager__SetBestSnappingOptionsOnPlaceholder____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__GeneratePartData____5____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__GeneratePartData____5____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__GeneratePartData____5____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__GeneratePartData____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__GeneratePartData____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__GeneratePartData____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__PrepareBasePart____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____69____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__PrepareBasePart____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__SetNodeMaterial____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__SetNodeMaterial____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__SetNodeMaterial____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__SetNodeColour____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__SetNodeColour____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__SetNodeColour____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__FindBaseTeleporterLocator____10____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcMechControl__Prepare____2____lambda_1___operator______9____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__FindBaseTeleporterLocator____10____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__LoadModel____5____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__LoadModel____5____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__CreateObjectTemplate____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__CreateObjectTemplate____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingManager__CreateObjectTemplate____5____lambda_1__bool_TkHandle____Copy(): pass

def std___Make_heap_unchecked_GhostOption____cGcBaseBuildingManager__SetBestSnappingOptionsOnPlaceholder____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_GhostOption____cGcBaseBuildingManager__SetBestSnappingOptionsOnPlaceholder____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_GhostOption___GhostOption__cGcBaseBuildingManager__SetBestSnappingOptionsOnPlaceholder____2____lambda_1___(): pass

def std___Med3_unchecked_GhostOption____cGcBaseBuildingManager__SetBestSnappingOptionsOnPlaceholder____2____lambda_1___(): pass

def sqrt(): pass

def _ModelPlacementRule__CacheSnapSocket____2____lambda_1___operator__(): pass

def _cGcBaseBuildingNavigationManager__DestroyNPCsOverlappingObject____8____lambda_1___operator__(): pass

class cTkStaticPhysicsComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def OnConfirmDialogClose(): pass

def _cGcLinkGrid__AssignGroups____6____lambda_2___operator__(): pass

def _cGcBaseLinks__ApplyNodeStates____2____lambda_1___operator__(): pass

def cGcBaseLinkMap__ForeachIndirectlyLinkedObject_1__cGcLinkGrid__UpdateObjectState____44____lambda_1___cGcLinkGrid__UpdateObjectState____44____lambda_2___(): pass

def std___Func_impl_no_alloc__cGcBaseLinks__ApplyNodeStates____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseLinks__ApplyNodeStates____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseLinks__ApplyNodeStates____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcProjectorEffect__UpdateRender____26____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcProjectorEffect__UpdateRender____26____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcProjectorEffect__UpdateRender____26____lambda_1__bool_TkHandle____Copy(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____110____lambda_4__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____110____lambda_4__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____110____lambda_4__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____69____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____69____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____63____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____42____lambda_4___operator______2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____63____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____61____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBaseBuildingPlayerPlacement__Update____61____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__CacheSnapSocket____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__CacheSnapSocket____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__CacheSnapSocket____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__CacheSnapPoint____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__CacheSnapPoint____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__CacheSnapPoint____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__ParsePlacementScene____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__ParsePlacementScene____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__ModelPlacementRule__ParsePlacementScene____2____lambda_1__bool_TkHandle____Move(): pass

class cGcNPCNavigationAreaComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcNPCComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def GetLineIntersectsBuilding2D(): pass

def GetDistance(): pass

def GetOffset(): pass

def AddNodesOnALine(): pass

def AddPerimeterNodes(): pass

def MergeNodes(): pass

def RemoveRedundantNodes(): pass

def CreateShortcuts(): pass

def _CreateShortcuts____2____lambda_1___operator__(): pass

def ConnectEntranceNodes(): pass

def _cGcWFCBuildingState__UpdateParts____2____lambda_1___operator__(): pass

def DecideColourPaletteFromArray(): pass

class cGcFreighterBaseNPCManager(ctypes.Structure):
    pass

def cGcNPCInteractiveObjectComponent__IterateTypedComponents__cGcWFCBuildingState__OnSettlementPartBuilt____5____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcWFCBaseGenerator__ConstructClusterNavigationGraph____83____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcWFCBaseGenerator__ConstructClusterNavigationGraph____83____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcWFCBaseGenerator__ConstructClusterNavigationGraph____83____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__CreateShortcuts____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__CreateShortcuts____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__CreateShortcuts____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__RemoveRedundantNodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__RemoveRedundantNodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__RemoveRedundantNodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__CleanUpGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__CleanUpGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__CleanUpGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerBasePersistentBuffer__StartAsyncChecks____14____lambda_1__void_bool____Delete_this(): pass

def std___Func_impl_no_alloc__cGcSettlementStateManager__RenameSettlement____2____lambda_1__void_bool_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcSettlementStateManager__RenameSettlement____2____lambda_1__void_bool_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcSettlementStateManager__RenameSettlement____2____lambda_1__void_bool_char_const______Copy(): pass

def std___Func_impl_no_alloc__cGcFreighterBaseNPCManager__Update____11____lambda_2__bool_sBaseObjectHandle_const___TkID_128______Target_type(): pass

def std___Func_impl_no_alloc__cGcFreighterBaseNPCManager__Update____11____lambda_2__bool_sBaseObjectHandle_const___TkID_128______Do_call(): pass

def std___Func_impl_no_alloc__cGcFreighterBaseNPCManager__Update____11____lambda_2__bool_sBaseObjectHandle_const___TkID_128______Move(): pass

class cGcEncyclopediaComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcSimpleInteractionComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcDroneComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def ConvertAtlasToInternalDiscoveryData(): pass

def ConvertInternalToAtlasDiscoveryData(): pass

def ___emplace_back_PEAV__Table__00_0FA__KPEAVcGcDiscoveryRecord__U__hash__KX_robin_hood__U__equal_to__K_std___0BD__detail_robin_hood_____vector_PEAV__Table__00_0FA__KPEAVcGcDiscoveryRecord__U__hash__KX_robin_hood__U__equal_to__K_std___0BD__detail_robin_hood__V__TkSTLAllocatorShim_PEAV__Table__00_0FA__KPEAVcGcDiscoveryRecord__U__hash__KX_robin_hood__U__equal_to__K_std___0BD__detail_robin_hood___07_0BD____std__QEAA_A_T__QEAPEAV__Table__00_0FA__KPEAVcGcDiscoveryRecord__U__hash__KX_robin_hood__U__equal_to__K_std___0BD__detail_robin_hood___Z(): pass

def SortImmutableRecordsNewestFirst_SS(): pass

def SortImmutableRecordsNewestFirst(): pass

def SortImmutableRecordsOldestFirst_SS(): pass

def SortImmutableRecordsOldestFirst(): pass

def _cGcDiscoveryManager__RPCNewDiscovery____2____lambda_1___operator__(): pass

def std___Sort_unchecked_cGcBaseBuildingPersistentBuffer__BaseBuildingPersistentData____cGcBaseBuildingPersistentBuffer__SortData____2____lambda_1___(): pass

def std___Sort_unchecked_cGcDiscoveryRecord_const______cGcDiscoveryManager__Data__RunSearches____56____lambda_1___(): pass

def std___Sort_unchecked_cGcDiscoveryRecord_const______cGcDiscoveryManager__Data__RunSearches____60____lambda_2___(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UnblockPlayer____2____lambda_1__void_bool____Delete_this(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__RPCNewDiscovery____2____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__RPCNewDiscovery____2____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__RPCNewDiscovery____2____lambda_1__void_bool____Copy(): pass

def std___Func_impl_no_alloc__cGcMonumentManager__RetrieveCreatorsDisplayName____11____lambda_2__void_cTkFixedString_64_char__const______Delete_this(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__Data__RetrieveOwnerDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__Data__RetrieveOwnerDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__Data__RetrieveOwnerDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Copy(): pass

def std___Func_impl_no_alloc__cGcMsgBeaconManager__Data__ProfanityCheckBeacon____8____lambda_1__void_bool____Delete_this(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__Data__ProfanityCheckDiscovery____8____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__Data__ProfanityCheckDiscovery____8____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcDiscoveryManager__Data__ProfanityCheckDiscovery____8____lambda_1__void_bool____Move(): pass

def std___Func_impl_no_alloc__cGcDiscoveryRecord__StartAsyncChecks____11____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcDiscoveryRecord__StartAsyncChecks____11____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcDiscoveryRecord__StartAsyncChecks____11____lambda_1__void_bool____Move(): pass

def std___Make_heap_unchecked_cGcBaseBuildingPersistentBuffer__BaseBuildingPersistentData____cGcBaseBuildingPersistentBuffer__SortData____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cGcBaseBuildingPersistentBuffer__BaseBuildingPersistentData____cGcBaseBuildingPersistentBuffer__SortData____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cGcBaseBuildingPersistentBuffer__BaseBuildingPersistentData___cGcBaseBuildingPersistentBuffer__BaseBuildingPersistentData__cGcBaseBuildingPersistentBuffer__SortData____2____lambda_1___(): pass

def std___Guess_median_unchecked_cGcDiscoveryRecord_const______cGcDiscoveryManager__Data__RunSearches____56____lambda_1___(): pass

def std___Guess_median_unchecked_cGcDiscoveryRecord_const______cGcDiscoveryManager__Data__RunSearches____60____lambda_2___(): pass

def std___Med3_unchecked_cGcBaseBuildingPersistentBuffer__BaseBuildingPersistentData____cGcBaseBuildingPersistentBuffer__SortData____2____lambda_1___(): pass

class cGcCreatureComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcAISpaceshipComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _snprintf(): pass

def _cGcFleetExpedition__Save____2____lambda_1___operator__(): pass

def _cGcFleetExpedition__Load____2____lambda_1___operator__(): pass

class cTkLODComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def base64_encode(): pass

def sha256_transf(): pass

def sha256_update(): pass

def sha256_final(): pass

def hmac_sha256_2(): pass

def _cGcGameState__SendRemoteItem____8____lambda_1___operator__(): pass

def _cGcGameState__SendRemoteEgg____2____lambda_1___operator__(): pass

def _cGcGameState__SendByteBeatSong____2____lambda_1___operator__(): pass

def ___emplace_back_AEAVcGcPersistentBaseEntry_____vector_VcGcPersistentBaseEntry__V__TkSTLAllocatorShim_VcGcPersistentBaseEntry___0BA__0_0___std__QEAA_A_TAEAVcGcPersistentBaseEntry___Z(): pass

def _cGcGameState__UpgradeUpdate4_00ToUpdate4_04____2____lambda_1___operator__(): pass

def _cGcGameState__UpgradeUpdate4_00ToUpdate4_04____2____lambda_2___operator__(): pass

def _cGcGameState__UpgradeUpdate3_94ToUpdate4_00____2____lambda_4___operator__(): pass

def _cGcGameState__UpgradeUpdate4_00ToUpdate4_04____2____lambda_3___operator__(): pass

def _cGcGameState__UpgradeUpdate4_00ToUpdate4_04____2____lambda_5___operator__(): pass

def _cGcGameState__UpgradeUpdate4_05ToUpdate4_09____3____lambda_3___operator__(): pass

def DeleteAllSavesForPlayerSlot(): pass

def DeleteManualSaveForPlayerSlot(): pass

def ___GcApplyTupleImpl_AEAP6A_NPEAVcGcNetworkPlayer__UsOwnedCreatureInfo___ZAEAV__tuple_PEAVcGcNetworkPlayer__UsOwnedCreatureInfo___std___0A__00__YA_A_TAEAP6A_NPEAVcGcNetworkPlayer__UsOwnedCreatureInfo___ZAEAV__tuple_PEAVcGcNetworkPlayer__UsOwnedCreatureInfo___std__U__integer_sequence__K_0A__00_3__Z(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcFleetFrigate__RepairMaintenanceComponent____18____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcMaintenanceComponent__IterateTypedComponentsOnNode__cGcFleetFrigate__UpdateSpawnedShipDamagedMaintenanceComponents____2____lambda_1_______2____lambda_1___(): pass

def std___Sort_unchecked_std__pair_float_int_____cGcGameState__SetFreshStartSpwanData____27____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcGameState__SendByteBeatSong____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Delete_this(): pass

def std___Func_impl_no_alloc__cGcGameState__SendByteBeatSong____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Target_type(): pass

def std___Func_impl_no_alloc__cGcGameState__SendByteBeatSong____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Do_call(): pass

def std___Func_impl_no_alloc__cGcGameState__SendByteBeatSong____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Copy(): pass

def std___Func_impl_no_alloc___cGcMatchmaking__SendRequestPermissionCreateLobby____2____lambda_1___operator___cGcNetworkPlayerFireteam_____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Delete_this(): pass

def std___Func_impl_no_alloc__cGcGameState__SendRemoteEgg____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Target_type(): pass

def std___Func_impl_no_alloc__cGcGameState__SendRemoteEgg____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Do_call(): pass

def std___Func_impl_no_alloc__cGcGameState__SendRemoteEgg____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Copy(): pass

def std___Func_impl_no_alloc__cGcGameState__SendRemoteItem____8____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Target_type(): pass

def std___Func_impl_no_alloc__cGcGameState__SendRemoteItem____8____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Do_call(): pass

def std___Func_impl_no_alloc__cGcGameState__SendRemoteItem____8____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Copy(): pass

def std___Func_impl_no_alloc__cGcFleetFrigate__SetName____2____lambda_1__void_bool_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcFleetFrigate__SetName____2____lambda_1__void_bool_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcFleetFrigate__SetName____2____lambda_1__void_bool_char_const______Move(): pass

def std___Partition_by_median_guess_unchecked_std__pair_float_int_____cGcGameState__SetFreshStartSpwanData____27____lambda_1___(): pass

def std___Med3_unchecked_std__pair_float_int_____cGcGameState__SetFreshStartSpwanData____27____lambda_1___(): pass

def snprintf(): pass

def ___emplace_back_AEAVcTkAttachmentPtr_____vector_VcTkAttachmentPtr__V__TkSTLAllocatorShim_VcTkAttachmentPtr___07_0_0___std__QEAA_A_TAEAVcTkAttachmentPtr___Z(): pass

def ___emplace_back_AEAM___vector_MV__TkSTLAllocatorShim_M_03_0_0___std__QEAA_A_TAEAM_Z(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcMPMissionSelectionHelper__Ready____8____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcMPMissionSelectionHelper__StartMission____8____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcMPMissionTracker__OnPlayerLeftMission____5____lambda_1___(): pass

def std___Sort_unchecked_GcGameStatePersistence__StorageSlotInformation____GcGameStatePersistence__GetAllPlayerSlotsInformation____27____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcMsgBeaconManager__Data__RetrieveDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcMsgBeaconManager__Data__RetrieveDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcMsgBeaconManager__Data__RetrieveDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Copy(): pass

def std___Func_impl_no_alloc__cGcMsgBeaconManager__Data__ProfanityCheckBeacon____8____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcMsgBeaconManager__Data__ProfanityCheckBeacon____8____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcMsgBeaconManager__Data__ProfanityCheckBeacon____8____lambda_1__void_bool____Move(): pass

def std___Make_heap_unchecked_GcGameStatePersistence__StorageSlotInformation____GcGameStatePersistence__GetAllPlayerSlotsInformation____27____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_GcGameStatePersistence__StorageSlotInformation____GcGameStatePersistence__GetAllPlayerSlotsInformation____27____lambda_1___(): pass

def std___Pop_heap_hole_by_index_GcGameStatePersistence__StorageSlotInformation___GcGameStatePersistence__StorageSlotInformation__GcGameStatePersistence__GetAllPlayerSlotsInformation____27____lambda_1___(): pass

def std___Med3_unchecked_GcGameStatePersistence__StorageSlotInformation____GcGameStatePersistence__GetAllPlayerSlotsInformation____27____lambda_1___(): pass

def ___emplace_back_AEAVTkHandle_____vector_VTkHandle__V__TkSTLAllocatorShim_VTkHandle___03_0_0___std__QEAA_A_TAEAVTkHandle___Z(): pass

def ___emplace_back_VcGcPlayerBasePersistentBuffer_____vector_VcGcPlayerBasePersistentBuffer__V__TkSTLAllocatorShim_VcGcPlayerBasePersistentBuffer___0BA__0_0___std__QEAA_A_T__QEAVcGcPlayerBasePersistentBuffer___Z(): pass

def _cGcPlayerByteBeatLibrary__RenderFrontEnd____2____lambda_1___operator__(): pass

def _cGcPlayerByteBeatLibrary__RenderFrontEnd____2____lambda_4___operator__(): pass

def _cGcPlayerByteBeatLibrary__RenderFrontEnd____2____lambda_5___operator__(): pass

def _cGcPlayerByteBeatLibrary__RenderFrontEnd____171____lambda_6___operator__(): pass

def _cGcPlayerByteBeatLibrary__RenderFrontEnd____171____lambda_7___operator__(): pass

def _cGcPlayerByteBeatLibrary__RenderFrontEnd____285____lambda_8___operator__(): pass

def _cGcPlayerByteBeatLibrary__DoTransferPopup____2____lambda_2___operator__(): pass

class cTkAnimPoseComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcPlayerCreatureOwnership__Update____2____lambda_1___operator__(): pass

def ___emplace_back_AEAT__TkID__0BAA___AEAV__cTkFixedString__0IA_D_____vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAT__TkID__0BAA___AEAV__cTkFixedString__0IA_D___Z(): pass

def ___emplace_back_AEAY0BD___CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BD___CBD__QEAPEBD_Z(): pass

def ___emplace_back_AEAY0O___CBDAEAV__cTkFixedString__0IA_D_____vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0O___CBDAEAV__cTkFixedString__0IA_D___Z(): pass

def ___emplace_back_AEAY03__CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY03__CBD__QEAPEBD_Z(): pass

def ___emplace_back_AEAY06__CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY06__CBD__QEAPEBD_Z(): pass

def ___emplace_back_AEAY0BF___CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BF___CBD__QEAPEBD_Z(): pass

def _cGcPlayerCreatureOwnership__SetCustomCreatureName____5____lambda_1___operator__(): pass

def _cGcPlayerDiscoveryHelper__CalculateCompletedDiscoveriesOnPlanet____28____lambda_1___operator__(): pass

def ___emplace_back_AEAY0BH___CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BH___CBD__QEAPEBD_Z(): pass

def ___emplace_back_AEAY0BE___CBDPEAD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BE___CBD__QEAPEAD_Z(): pass

def ___emplace_back_AEAY0BI___CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BI___CBD__QEAPEBD_Z(): pass

def cGcBaseLinkMap__ForeachDirectlyLinkedObject_7__cGcPlayerBasePersistentBuffer__OnLinkObjectAdded____2____lambda_1___(): pass

def cGcBaseLinkMap__ForeachDirectlyLinkedObject_7__cGcPlayerBasePersistentBuffer__OnLinkObjectRemoved____2____lambda_1___(): pass

def cGcBaseLinkMap__ForeachIndirectlyLinkedObject_7__cGcPlayerBasePersistentBuffer__OnLinkObjectRemoved____20____lambda_2___cGcPlayerBasePersistentBuffer__OnLinkObjectRemoved____20____lambda_3___(): pass

def _LN158(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerByteBeatLibrary__RenderFrontEnd____313____lambda_9___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerByteBeatLibrary__DoTransferPopup____2____lambda_1___(): pass

def std___Sort_unchecked_int____cGcPersistentInteractionsManager__PopulateArrays____2____lambda_1___(): pass

def std___Func_class_void_TkID_128__const___cTkFixedString_256_char_______Reset__cGcPlayerByteBeatLibrary__RenderFrontEnd____2____lambda_2___(): pass

def std___Sort_unchecked_std__pair_float_cGcPlayerByteBeatLibrary__SendByteBeatOption_____cGcPlayerByteBeatLibrary__DoTransferPopup____2____lambda_2___(): pass

def std___Func_impl_no_alloc__cGcPlayerDiscoveryHelper__CalculateCompletedDiscoveriesOnPlanet____28____lambda_1__void_cGcPlanetObjectSpawnData_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerDiscoveryHelper__CalculateCompletedDiscoveriesOnPlanet____28____lambda_1__void_cGcPlanetObjectSpawnData_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerDiscoveryHelper__CalculateCompletedDiscoveriesOnPlanet____28____lambda_1__void_cGcPlanetObjectSpawnData_const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerDiscoveryHelper__FindEncyclopediaData____2____lambda_1__bool_cGcEncyclopediaComponentData_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerDiscoveryHelper__FindEncyclopediaData____2____lambda_1__bool_cGcEncyclopediaComponentData_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerDiscoveryHelper__FindEncyclopediaData____2____lambda_1__bool_cGcEncyclopediaComponentData_const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCommon__GetPotentialCriminal____2____lambda_1__void_cGcPlayerCommon______Delete_this(): pass

def std___Func_impl_no_alloc__cGcPlayerCreatureOwnership__SetCustomCreatureName____5____lambda_1__void_bool_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCreatureOwnership__SetCustomCreatureName____5____lambda_1__void_bool_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCreatureOwnership__SetCustomCreatureName____5____lambda_1__void_bool_char_const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCreatureOwnership__Update____44____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcShipTargetDisplay__Update____11____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCreatureOwnership__Update____44____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerByteBeatLibrary__RenderFrontEnd____2____lambda_2__void_TkID_128__const___cTkFixedString_256_char_______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerByteBeatLibrary__RenderFrontEnd____2____lambda_2__void_TkID_128__const___cTkFixedString_256_char_______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerByteBeatLibrary__RenderFrontEnd____2____lambda_2__void_TkID_128__const___cTkFixedString_256_char_______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerBasePersistentBuffer__StartAsyncChecks____14____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerBasePersistentBuffer__StartAsyncChecks____14____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerBasePersistentBuffer__StartAsyncChecks____14____lambda_1__void_bool____Copy(): pass

def std___Partition_by_median_guess_unchecked_int____cGcPersistentInteractionsManager__PopulateArrays____2____lambda_1___(): pass

def std___Insertion_sort_unchecked_std__pair_float_cGcPlayerByteBeatLibrary__SendByteBeatOption_____cGcPlayerByteBeatLibrary__DoTransferPopup____2____lambda_2___(): pass

def std___Make_heap_unchecked_std__pair_float_cGcPlayerByteBeatLibrary__SendByteBeatOption_____cGcPlayerByteBeatLibrary__DoTransferPopup____2____lambda_2___(): pass

def std___Partition_by_median_guess_unchecked_std__pair_float_cGcPlayerByteBeatLibrary__SendByteBeatOption_____cGcPlayerByteBeatLibrary__DoTransferPopup____2____lambda_2___(): pass

def std___Pop_heap_hole_by_index_int___int__cGcPersistentInteractionsManager__PopulateArrays____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_std__pair_float_cGcPlayerByteBeatLibrary__SendByteBeatOption____std__pair_float_cGcPlayerByteBeatLibrary__SendByteBeatOption___cGcPlayerByteBeatLibrary__DoTransferPopup____2____lambda_2___(): pass

def std___Med3_unchecked_int____cGcPersistentInteractionsManager__PopulateArrays____2____lambda_1___(): pass

def std___Med3_unchecked_std__pair_float_cGcPlayerByteBeatLibrary__SendByteBeatOption_____cGcPlayerByteBeatLibrary__DoTransferPopup____2____lambda_2___(): pass

class cTkNGuiEditor(ctypes.Structure):
    def RegisterPopup(): pass


def _cGcPlayerFleetManager__SortFrigateList____2____lambda_1___operator__(): pass

def _cGcPlayerFreighterOwnership__DespawnFreighter____11____lambda_1___operator__(): pass

def _cGcPlayerFreighterOwnership__OnAmbientFreighterBaseRestructure____16____lambda_1___operator__(): pass

def _cGcPlayerFreighterOwnership__UpdateOwnershipsActiveInSystems____2____lambda_1___operator__(): pass

def _cGcPlayerFreighterOwnership__UpdateOwnershipsActiveInSystems____6____lambda_2___operator__(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__AddPurchasedFrigateToFleet____29____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestRenameFrigate____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestSetFrigateExpedition____12____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestRemoveFrigateFromFleet____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestAssignFrigateToPirateDefence____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestUnassignFrigateFromPirateDefence____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestSpawnFrigateForPirateDefence____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestCollectCargo____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestSelectExpedition____11____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestStartExpedition____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestEndExpedition____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__ChooseNewUAForExpedition____12____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestRemoveExpedition____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestContinueWithDamagedFrigate____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFleetManager__RequestSendDamagedFrigateHome____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFreighterOwnership__BroadcastFreighterSwitchToClients____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcPlayerFreighterOwnership__RequestFreighterSpawn____11____lambda_1___(): pass

def std___Sort_unchecked_cGcFleetFrigate______cGcPlayerFleetManager__SortFrigateList____2____lambda_1___(): pass

def std___Sort_unchecked_cGcPlayerFreighterOwnership______cGcPlayerFreighterOwnership__UpdateOwnershipsActiveInSystems____6____lambda_2___(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__OnAmbientFreighterBaseRestructure____16____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__OnAmbientFreighterBaseRestructure____16____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__OnAmbientFreighterBaseRestructure____16____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__UpdateMeshRefresh____8____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__UpdateMeshRefresh____8____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__UpdateMeshRefresh____8____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__ApplyFreighterOwnershipSyncMessage____2____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__ApplyFreighterOwnershipSyncMessage____2____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__ApplyFreighterOwnershipSyncMessage____2____lambda_1__void_bool____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__SetCustomFreighterName____7____lambda_1__void_bool_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__SetCustomFreighterName____7____lambda_1__void_bool_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__SetCustomFreighterName____7____lambda_1__void_bool_char_const______Copy(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__DespawnFreighter____11____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__DespawnFreighter____11____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__DespawnFreighter____11____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__CacheFreighter____8____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerFreighterOwnership__CacheFreighter____8____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerFleetManager__ApplyFleetFrigateNamesSyncMessage____7____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerFleetManager__ApplyFleetFrigateNamesSyncMessage____7____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerFleetManager__ApplyFleetFrigateNamesSyncMessage____7____lambda_1__void_bool____Move(): pass

def std___Insertion_sort_unchecked_cGcFleetFrigate______cGcPlayerFleetManager__SortFrigateList____2____lambda_1___(): pass

def std___Make_heap_unchecked_cGcFleetFrigate______cGcPlayerFleetManager__SortFrigateList____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cGcFleetFrigate______cGcPlayerFleetManager__SortFrigateList____2____lambda_1___(): pass

def std___Insertion_sort_unchecked_cGcPlayerFreighterOwnership______cGcPlayerFreighterOwnership__UpdateOwnershipsActiveInSystems____6____lambda_2___(): pass

def std___Make_heap_unchecked_cGcPlayerFreighterOwnership______cGcPlayerFreighterOwnership__UpdateOwnershipsActiveInSystems____6____lambda_2___(): pass

def std___Partition_by_median_guess_unchecked_cGcPlayerFreighterOwnership______cGcPlayerFreighterOwnership__UpdateOwnershipsActiveInSystems____6____lambda_2___(): pass

def std___Pop_heap_hole_by_index_cGcFleetFrigate_____cGcFleetFrigate____cGcPlayerFleetManager__SortFrigateList____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cGcPlayerFreighterOwnership_____cGcPlayerFreighterOwnership____cGcPlayerFreighterOwnership__UpdateOwnershipsActiveInSystems____6____lambda_2___(): pass

def std___Med3_unchecked_cGcFleetFrigate______cGcPlayerFleetManager__SortFrigateList____2____lambda_1___(): pass

def std___Med3_unchecked_cGcPlayerFreighterOwnership______cGcPlayerFreighterOwnership__UpdateOwnershipsActiveInSystems____6____lambda_2___(): pass

class cTkNetReplicatedEntityComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def ___emplace_back_HAEAY0BP___CBD___vector_ULogBookMessage__V__TkSTLAllocatorShim_ULogBookMessage___03_0_0___std__QEAA_A_T__QEAHAEAY0BP___CBD_Z(): pass

class cGcShipOwnershipComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def ___emplace_back_AEAT__TkID__0BAA___PEAD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAT__TkID__0BAA_____QEAPEAD_Z(): pass

def ___emplace_back_AEAT__TkID__0IA______vector_T__TkID__0IA___V__TkSTLAllocatorShim_T__TkID__0IA____07_0_0___std__QEAA_A_TAEAT__TkID__0IA____Z(): pass

def _cGcPlayerState__AdjustStatFromInventory____2____lambda_1___operator__(): pass

def _cGcPlayerState__SetCustomShipName____10____lambda_1___operator__(): pass

def _cGcPlayerState__StoreCurrentSystemSpaceStationEndpoint____5____lambda_1___operator__(): pass

def std___Insertion_sort_unchecked_cGcInventoryElement____cGcPlayerState__TryStoreInInventory_Internal____3____lambda_1___(): pass

def std___Stable_sort_unchecked_cGcInventoryElement____cGcPlayerState__TryStoreInInventory_Internal____3____lambda_1___(): pass

def std___Sort_unchecked_cGcInventoryElement_const______cGcPlayerState__TryToRemoveFromSpecificInventory____14____lambda_1___(): pass

def std___Sort_unchecked_InventoryLookup____cGcPlayerState__CollectAllMatchingInventoryElementsInInventories____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPlayerState__StoreCurrentSystemSpaceStationEndpoint____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerState__StoreCurrentSystemSpaceStationEndpoint____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerState__StoreCurrentSystemSpaceStationEndpoint____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomVehicleName____10____lambda_1__void_bool_char_const______Delete_this(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomVehicleName____10____lambda_1__void_bool_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomVehicleName____10____lambda_1__void_bool_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomVehicleName____10____lambda_1__void_bool_char_const______Copy(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomShipName____10____lambda_1__void_bool_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomShipName____10____lambda_1__void_bool_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomShipName____10____lambda_1__void_bool_char_const______Copy(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomWeaponName____7____lambda_1__void_bool_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomWeaponName____7____lambda_1__void_bool_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerState__SetCustomWeaponName____7____lambda_1__void_bool_char_const______Copy(): pass

def std___Func_impl_no_alloc__cGcPlayerShipOwnership__UpdateMeshRefresh____16____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerShipOwnership__UpdateMeshRefresh____16____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerShipOwnership__UpdateMeshRefresh____16____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerShipOwnership__Prepare____7____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerShipOwnership__Prepare____7____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerShipOwnership__Prepare____7____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerMultitoolOwnership__LoadMultitool____25____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerMultitoolOwnership__LoadMultitool____25____lambda_1__enum_TkPollResult____Move(): pass

def std___Buffered_merge_sort_unchecked_cGcInventoryElement____cGcPlayerState__TryStoreInInventory_Internal____3____lambda_1___(): pass

def std___Buffered_inplace_merge_unchecked_cGcInventoryElement____cGcPlayerState__TryStoreInInventory_Internal____3____lambda_1___(): pass

def std___Make_heap_unchecked_InventoryLookup____cGcPlayerState__CollectAllMatchingInventoryElementsInInventories____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_InventoryLookup____cGcPlayerState__CollectAllMatchingInventoryElementsInInventories____2____lambda_1___(): pass

def std___Chunked_merge_unchecked_cGcInventoryElement___cGcInventoryElement____cGcPlayerState__TryStoreInInventory_Internal____3____lambda_1___(): pass

def std___Buffered_inplace_merge_unchecked_impl_cGcInventoryElement____cGcPlayerState__TryStoreInInventory_Internal____3____lambda_1___(): pass

def std___Guess_median_unchecked_cGcInventoryElement_const______cGcPlayerState__TryToRemoveFromSpecificInventory____14____lambda_1___(): pass

def std___Pop_heap_hole_by_index_InventoryLookup___InventoryLookup__cGcPlayerState__CollectAllMatchingInventoryElementsInInventories____2____lambda_1___(): pass

def std___Buffered_inplace_merge_divide_and_conquer_cGcInventoryElement____cGcPlayerState__TryStoreInInventory_Internal____3____lambda_1___(): pass

def std___Med3_unchecked_InventoryLookup____cGcPlayerState__CollectAllMatchingInventoryElementsInInventories____2____lambda_1___(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced___cGcPlayerState__SetCustomShipName____10____lambda_1___operator______2____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced___cGcPlayerState__SetCustomShipName____10____lambda_1___operator______2____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced___cGcPlayerState__SetCustomShipName____10____lambda_1___operator______2____lambda_1____void____Move(): pass

def std___Buffered_inplace_merge_divide_and_conquer2_cGcInventoryElement____cGcPlayerState__TryStoreInInventory_Internal____3____lambda_1___(): pass

class cGcVehicleComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcRocketLockerComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcSynchronisedStateManager__GetAllPlayers____2____lambda_1___operator__(): pass

def std__vector__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_32__1______vector__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_32__1___(): pass

def std__vector__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_32__1_____reserve(): pass

def std__vector__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_32__1______Change_array(): pass

def std__vector__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_4__1______Emplace_one_at_back_sBufferIndex___(): pass

def std___Uninitialized_copy__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo____cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo___StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_32__1___(): pass

def _cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo__sPlayerInfo(): pass

def _cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo___sPlayerInfo(): pass

def _cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo__operator_(): pass

def std__vector__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_4__1_____reserve(): pass

def std__vector__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_4__1______Change_array(): pass

def std__vector__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_32__1______Emplace_reallocate__(): pass

def std__vector__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_4__1______Assign_counted_range__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_const___(): pass

def std___Uninitialized_move__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo___StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sBufferInfo_4__1___(): pass

def std___Func_impl_no_alloc__cGcSquadronPilotInfo__UpdateResourceLoading____47____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSquadronPilotInfo__UpdateResourceLoading____47____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerVehicleOwnership__UpdateMeshRefresh____8____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerVehicleOwnership__UpdateMeshRefresh____8____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerVehicleOwnership__UpdateMeshRefresh____8____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerVehicleOwnership__Update____75____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerVehicleOwnership__Update____75____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerVehicleOwnership__Update____75____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerVehicleOwnership__Update____75____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerVehicleOwnership__Update____75____lambda_1__bool_TkHandle____Move(): pass

def std___Uninitialized_move__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo___StackAllocator__cGcSynchronisedStateManager__SendHashMessage____2___sPlayerInfo_32__1___(): pass

def _cGcUserSeenItemsState__SaveState____2____lambda_1___operator___robin_hood__detail__Table_1_80_TkID_128__void_robin_hood__hash_TkID_128__void__std__equal_to_TkID_128_____1__cTkDynamicArray_TkID_128_____(): pass

def _cGcUserSettings__UpdateAll____2____lambda_1___operator__(): pass

class cGcWonderUtils(ctypes.Structure):
    def GetWikiTopicForWonderType(): pass

    def GetWonderTypeForWikiGridTopic(): pass

    def GenerateCreatureIntelligenceStat(): pass

    def GenerateCreatureViciousnessStat(): pass

    def GenerateSystemDataFromDiscovery(): pass

    def GetAdjustedPlanetRadius(): pass

    def GenerateRandomGenericWonderStat(): pass

    def GetCategoryConfig(): pass


def std___Sort_unchecked_unsigned_char____cGcTerrainEditsPersistentBuffer__GetOverlappingTerrainEditBlocksNoLock____13____lambda_1___(): pass

def std___Insertion_sort_unchecked_unsigned_char____cGcTerrainEditsPersistentBuffer__GetOverlappingTerrainEditBlocksNoLock____13____lambda_1___(): pass

def std___Make_heap_unchecked_unsigned_char____cGcTerrainEditsPersistentBuffer__GetOverlappingTerrainEditBlocksNoLock____13____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_unsigned_char____cGcTerrainEditsPersistentBuffer__GetOverlappingTerrainEditBlocksNoLock____13____lambda_1___(): pass

def std___Pop_heap_hole_by_index_unsigned_char___unsigned_char__cGcTerrainEditsPersistentBuffer__GetOverlappingTerrainEditBlocksNoLock____13____lambda_1___(): pass

def std___Med3_unchecked_unsigned_char____cGcTerrainEditsPersistentBuffer__GetOverlappingTerrainEditBlocksNoLock____13____lambda_1___(): pass

def MyMiniDumpCallback(): pass

def MinidumpTimeoutThread(): pass

def MiniDumpFunction(): pass

def main(): pass

def Main(): pass

def Construct(): pass

def ProcTexRegisterWithMetadataCache(): pass

def _cGcCameraBehaviourPlayerThirdPerson__ApplyCollisions____5____lambda_2___operator__(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourPhotoMode__UpdateLoadBoundaryResource____17____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourPhotoMode__UpdateLoadBoundaryResource____17____lambda_2__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourPhotoMode__UpdateLoadBoundaryResource____17____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourPhotoMode__UpdateLoadBoundaryResource____12____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourPhotoMode__UpdateLoadBoundaryResource____12____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourPhotoMode__UpdateLoadBoundaryResource____12____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourGalacticNavigation__LockToOrbitPoint____16____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourGalacticNavigation__LockToOrbitPoint____16____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourGalacticNavigation__LockToOrbitPoint____16____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourBuildingMode__UpdateLoadBoundaryResource____17____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourBuildingMode__UpdateLoadBoundaryResource____17____lambda_2__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourBuildingMode__UpdateLoadBoundaryResource____17____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourBuildingMode__UpdateLoadBoundaryResource____12____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourBuildingMode__UpdateLoadBoundaryResource____12____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourBuildingMode__UpdateLoadBoundaryResource____12____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourAnimation__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourAnimation__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCameraBehaviourAnimation__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

class cTkPhysicsComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cTkAnimationComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def ConvertUtf16ToByteLimitedUtf8(): pass

def std___Func_impl_no_alloc__cGcFleetPageData__Update____55____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcFleetPageData__Update____55____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcFleetPageData__Update____55____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendTextInput__GetResult____20____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendTextInput__GetResult____20____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendTextInput__GetResult____20____lambda_1__void_bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendTextInput__Start____8____lambda_1__void_vr__VREvent_t____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendTextInput__Start____8____lambda_1__void_vr__VREvent_t____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendTextInput__Start____8____lambda_1__void_vr__VREvent_t____Move(): pass

def std___Func_impl_no_alloc__cGcCameraManager__BlendToBehaviour____31____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcCameraManager__BlendToBehaviour____31____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcCameraManager__BlendToBehaviour____31____lambda_1__void____Move(): pass

def Compute(): pass

def GetMaxTemplateDepth(): pass

def CountTemplateBits(): pass

def _cGcFrontendPageByteBeat__GenerateRandomTree____2____lambda_1___operator__(): pass

def _cGcFrontendPageByteBeat__GenerateRandomTree____2____lambda_2___operator__(): pass

def CalculateDepth(): pass

def CountTreeBits(): pass

def _cGcFrontendPageByteBeat__UpdateTreeNode____2____lambda_1___operator__(): pass

def _cGcFrontendPageByteBeat__RenderVisualisation____2____lambda_1___operator__(): pass

def _cGcFrontendPageByteBeat__RenderArpTypeSlider____5____lambda_2___operator__(): pass

def _cGcFrontendPageByteBeat__RenderWaveTypeSlider____5____lambda_2___operator__(): pass

def _cGcFrontendPageByteBeat__RenderEnvFadeIn____5____lambda_2___operator__(): pass

def _cGcFrontendPageByteBeat__RenderEnvFadeOut____5____lambda_2___operator__(): pass

def RenderPowerIcon(): pass

def _cGcFrontendPageByteBeat__RenderNoteOnButton____5____lambda_2___operator__(): pass

def DrawRandomiseIcon(): pass

def _cGcFrontendPageByteBeat__RenderPlayButton____5____lambda_2___operator__(): pass

def _cGcFrontendPageSquadronPilots__Render____2____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPlayButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPlayButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPlayButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPlayButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPlayButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPlayButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseHiHatButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderSoloButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseHiHatButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseHiHatButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseHiHatButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseHiHatButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSnareButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSnareButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSnareButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSnareButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSnareButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseKickButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseKickButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseKickButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseKickButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseKickButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderSoloButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderSoloButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderSoloButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderSoloButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderSoloButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEverythingButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEverythingButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEverythingButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEverythingButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEverythingButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEnvelopeButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEnvelopeButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEnvelopeButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEnvelopeButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseEnvelopeButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseArpeggioButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseArpeggioButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseArpeggioButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseArpeggioButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseArpeggioButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseRhythmButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseRhythmButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseRhythmButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseRhythmButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseRhythmButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSequenceButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSequenceButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSequenceButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSequenceButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRandomiseSequenceButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearRhythmButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearSequenceButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearRhythmButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearRhythmButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearRhythmButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearRhythmButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearSequenceButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearSequenceButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearSequenceButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearSequenceButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderClearSequenceButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPasteButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPasteButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPasteButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderPasteButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderCopyButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderCopyButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderCopyButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderCopyButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNoteOnButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNoteOnButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNoteOnButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNoteOnButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNoteOnButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNoteOnButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRhythmPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRhythmPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRhythmPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRhythmPowerButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRhythmPowerButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderRhythmPowerButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderMelodyPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderMelodyPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderMelodyPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderMelodyPowerButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderMelodyPowerButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderMelodyPowerButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpPowerButton____5____lambda_2__void_cTkBBox2d_const___NVGcolor_const___float____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpPowerButton____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpPowerButton____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpPowerButton____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNumStepsSlider____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNumStepsSlider____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderNumStepsSlider____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvLength____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvLength____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvLength____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeOut____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeOut____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeOut____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeOut____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeOut____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeOut____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeIn____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeIn____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeIn____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeIn____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeIn____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderEnvFadeIn____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderWaveTypeSlider____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderWaveTypeSlider____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderWaveTypeSlider____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderWaveTypeSlider____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderWaveTypeSlider____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderWaveTypeSlider____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpTypeSlider____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpTypeSlider____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpTypeSlider____5____lambda_2__void_unsigned_int_cTkBBox2d_const___NVGcolor_const___bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpTypeSlider____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpTypeSlider____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpTypeSlider____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpLengthSlider____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpLengthSlider____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpLengthSlider____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpStepSlider____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpStepSlider____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpStepSlider____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpRangeSlider____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpRangeSlider____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderArpRangeSlider____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderDrumMixSlider____5____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderDrumMixSlider____5____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__RenderDrumMixSlider____5____lambda_1__void_unsigned_int____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__DoByteBeatSequencer____104____lambda_1__void_unsigned_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__DoByteBeatSequencer____104____lambda_1__void_unsigned_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageByteBeat__DoByteBeatSequencer____104____lambda_1__void_unsigned_int____Move(): pass

def _cGcFrontendPageDiscovery__DoSystemView____30____lambda_1___operator__(): pass

def _cGcFrontendPageDiscovery__DoPlanetView____2____lambda_1___operator__(): pass

def _cGcFrontendPageDiscovery__DoPlanetView____2____lambda_2___operator__(): pass

def _cGcFrontendPageDiscovery__DoPlanetInfo____16____lambda_1___operator__(): pass

def _cGcFrontendPageWiki__DoGridItemPlanet____8____lambda_1___operator__(): pass

def _cGcFrontendPageInteractions__DoRadialInteractionPage____11____lambda_1___operator__(): pass

def _cGcFrontendPageInteractions__DoIntermediateInteractionPage____36____lambda_3___operator__(): pass

def _cGcFrontendPageInteractions__DoIntermediateInteractionPage____36____lambda_3____lambda_invoker_cdecl_(): pass

def GetSelectedRadialElement(): pass

def _cGcFrontendPageInteractions__DoOption____69____lambda_1___operator__(): pass

def _cGcFrontendPageInteractions__DoRadialOption____43____lambda_1___operator__(): pass

def _cGcFrontendPageInteractions__TidyUpTextureOptions____2____lambda_1___operator__(): pass

def _cGcFrontendPageInteractions__DoCharacterCustomisation____2____lambda_2___operator__(): pass

def _cGcFrontendPageInteractions__ByteBeatSwitchBeatRenderFunction____2____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchBeatRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchBeatRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchBeatRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchRhythmRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchRhythmRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchRhythmRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchArpNoteRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchBaseNoteRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchArpNoteRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchBaseNoteRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageInteractions__ByteBeatSwitchBaseNoteRenderFunction____2____lambda_1__void_cTkBBox2d_const___unsigned_int_bool_bool____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageDiscovery__UploadAndRewardAll____5____lambda_1__enum_eIterateReturn_cGcDiscoveryRecord_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageDiscovery__UploadAndRewardAll____5____lambda_1__enum_eIterateReturn_cGcDiscoveryRecord_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageDiscovery__UploadAndRewardAll____5____lambda_1__enum_eIterateReturn_cGcDiscoveryRecord_const______Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageDiscovery__UpdateDiscoveries____23____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageDiscovery__UpdateDiscoveries____23____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageDiscovery__UpdateDiscoveries____23____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoRadialOption____43____lambda_1____void____Delete_this(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoRadialOption____43____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoRadialOption____43____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoRadialOption____43____lambda_1____void____Copy(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoOption____69____lambda_1____void____Delete_this(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoOption____69____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoOption____69____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoOption____69____lambda_1____void____Copy(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoRadialInteractionPage____11____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoRadialInteractionPage____11____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoRadialInteractionPage____11____lambda_1____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoIntermediateInteractionPage____28____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoIntermediateInteractionPage____28____lambda_1____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoInteraction____21____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageInteractions__DoInteraction____21____lambda_1____void____Move(): pass

def _cGcFrontendPageMaintenance__DoSlotLocators____5____lambda_1___operator__(): pass

def _cGcFrontendPageMaintenance__DoSlotLocators____5____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageManageFleet__DoFrigateCaptain____237____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageManageFleet__DoFrigateDetails____327____lambda_2___operator__(): pass

def _cGcFrontendPageManageFleet__DoFrigateDetails____327____lambda_2____lambda_invoker_cdecl_(): pass

def ___emplace_back_AEAUcGcMissionItem_____vector_UcGcMissionItem__V__TkSTLAllocatorShim_UcGcMissionItem___07_0_0___std__QEAA_A_TAEAUcGcMissionItem___Z(): pass

def _LN1216(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissionGiver__DoMissionItem____8____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissionGiver__DoMissionItem____8____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissionGiver__DoMissionItem____8____lambda_1____void____Copy(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissionGiver__DoMissionDescriptionPage____97____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissionGiver__DoMissionDescriptionPage____97____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissionGiver__DoMissionDescriptionPage____97____lambda_1____void____Copy(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMaintenance__Render____30____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMaintenance__Render____30____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMaintenance__Render____30____lambda_1____void____Move(): pass

def _cGcFrontendPageMissions__DoMissionLog____52____lambda_2___operator__(): pass

def _cGcFrontendPageMissions__DoMissionLog____55____lambda_3___operator__(): pass

def _cGcFrontendPageMultiplayerMissionGiver__UpdateMissionList____2____lambda_1___operator__(): pass

def _cGcFrontendPageMultiplayerMissionGiver__GetMultiplayerWeekendMissionsList____15____lambda_1___operator__(): pass

def _cGcFrontendPageOptions__DoSwitcher____B____lambda_2____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareGeneralOptions____24____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareControlsOptions____15____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareControlsOptions____14____lambda_2____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareControlsOptions____14____lambda_3____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareControlsOptions____2____lambda_4____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareCameraOptions____10____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareVideoOptions____10____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareVideoOptions____2____lambda_3____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareGyroOptions____2____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareSwitcherMenu____2____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareSwitcherMenu____5____lambda_2____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareSwitcherMenu____8____lambda_3____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareSwitcherMenu____2____lambda_4____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareSwitcherMenu____2____lambda_5____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageOptions__PrepareSwitcherMenu____2____lambda_6____lambda_invoker_cdecl_(): pass

def cTkJobManager__CreateJobLambda__cGcFrontendPageMissions__DoMissionLog____152____lambda_4___(): pass

def cTkJobManager__CreateJobLambda__cGcFrontendPageMultiplayerMissionGiver__DoMissionDescriptionPage____237____lambda_1___(): pass

def std__remove_if_std___Vector_iterator_std___Vector_val_std___Simple_types_cGcMissionItem_______cGcFrontendPageMultiplayerMissionGiver__GetMultiplayerWeekendMissionsList____15____lambda_1___(): pass

def std___Sort_unchecked_cGcNotificationSequence______cGcPlayerNotifications__GetNotificationSequenceBefore____5____lambda_1___(): pass

def cGcNGuiLayer__ForEachChildElement__cGcFrontendManager__ShowDialogBox____2____lambda_1___(): pass

def cGcNGuiLayer__ForEachChildElement__cGcFrontendManager__UpdateDialogBox____5____lambda_1___(): pass

def std__vector__cGcFrontendPageFunctions__DoTechStatSection____2___cGcStatsBonusDisplay_TkSTLAllocatorShim__cGcFrontendPageFunctions__DoTechStatSection____2___cGcStatsBonusDisplay_4__1______Emplace_reallocate__cGcFrontendPageFunctions__DoTechStatSection____2___cGcStatsBonusDisplay_const___(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__ShowControlsPage____7____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__ShowControlsPage____7____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__ShowControlsPage____7____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__ShowCredits____7____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__ShowCredits____7____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__ShowCredits____7____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__DoSwitcher____E____lambda_3__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__DoSwitcher____E____lambda_3__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__DoSwitcher____E____lambda_3__void____Move(): pass

def std___Func_impl_no_alloc___cGcFrontendPageOptions__DoSwitcher____B____lambda_2___operator______8____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc___cGcFrontendPageOptions__DoSwitcher____B____lambda_2___operator______8____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc___cGcFrontendPageOptions__DoSwitcher____B____lambda_2___operator______8____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__DoSwitcher____23____lambda_1__void____Target_type(): pass

def _cGcMatchmaking__Update____66____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__cGcFrontendPageOptions__DoSwitcher____23____lambda_1__void____Move(): pass

def std___Make_heap_unchecked_cGcNotificationSequence______cGcPlayerNotifications__GetNotificationSequenceBefore____5____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cGcNotificationSequence______cGcPlayerNotifications__GetNotificationSequenceBefore____5____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cGcNotificationSequence_____cGcNotificationSequence____cGcPlayerNotifications__GetNotificationSequenceBefore____5____lambda_1___(): pass

def std___Med3_unchecked_cGcNotificationSequence______cGcPlayerNotifications__GetNotificationSequenceBefore____5____lambda_1___(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageOptions__UnstuckOnFreighter____2____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageOptions__UnstuckOnFreighter____2____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageOptions__UnstuckOnFreighter____2____lambda_1____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMultiplayerMissionGiver__DoMissionDescriptionPage____237____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMultiplayerMissionGiver__DoMissionDescriptionPage____237____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMultiplayerMissionGiver__DoMissionDescriptionPage____237____lambda_1____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissions__DoMissionLog____152____lambda_4____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissions__DoMissionLog____152____lambda_4____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageMissions__DoMissionLog____152____lambda_4____void____Move(): pass

def _cGcFrontendPageRefiner__Render____207____lambda_1___operator__(): pass

def _cGcFrontendPageSettlementJudgement__ApplyDecisionEffects____2____lambda_1___operator__(): pass

def DoTradePageBottomBanner(): pass

def DoBottomBanner(): pass

def _cGcFrontendPageShop__DoSpecialsTrade____189____lambda_1___operator__(): pass

def _cGcFrontendPageShop__DoSquadronRecruitment____35____lambda_1___operator__(): pass

def _cGcFrontendPageShop__DoPurchaseButton____232____lambda_1___operator__(): pass

def _cGcFrontendPageShop__GenerateTwitchRewardsInventory____2____lambda_2____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateTwitchRewardsInventory____2____lambda_3____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateTwitchRewardsInventory____2____lambda_4____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateSpecialsShopInventory____2____lambda_5____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateSpecialsShopInventory____2____lambda_6____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateTwitchRewardsInventory____2____lambda_5____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateTwitchRewardsInventory____2____lambda_6____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateTwitchRewardsInventory____2____lambda_7____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateTwitchRewardsInventory____2____lambda_8____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GenerateTwitchRewardsInventory____2____lambda_9____lambda_invoker_cdecl_(): pass

def _cGcFrontendPageShop__GeneratePlatformRewardsInventory____2____lambda_2____lambda_invoker_cdecl_(): pass

def SetupRankLayer(): pass

def DoDescriptionString(): pass

def DoPriceString(): pass

def DoPriceStringMaps(): pass

def DoPriceStringScrap(): pass

def _cGcFrontendPageShop__CheckTransferItemsDialog____8____lambda_1___operator__(): pass

def cGcNPCComponent__IterateTypedComponents__cGcFrontendPageSettlementJudgement__CreateNPC____13____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcFrontendPageShop__CheckTransferItemsDialog____8____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcFrontendPageShop__CheckTransferItemsDialog____8____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcFrontendPageShop__CheckTransferItemsDialog____8____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageShop__DoSpecialsTrade____189____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageShop__DoSpecialsTrade____189____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageShop__DoSpecialsTrade____189____lambda_1____void____Move(): pass

def _cGcFrontendPageWiki__GetWikiGridItemsData____5____lambda_1___operator__(): pass

def QSortRecipes(): pass

def _cGcFrontendPageFunctions__DoProductSlot____72____lambda_1___operator__(): pass

def _cGcFrontendPageFunctions__DoTechnologySlot____341____lambda_1___operator__(): pass

def _cGcFrontendPageFunctions__DoItemPopup____64____lambda_1___operator__(): pass

def std__vector__cGcFrontendPageFunctions__DoTransferPopup____2___TransferInfo_TkSTLAllocatorShim__cGcFrontendPageFunctions__DoTransferPopup____2___TransferInfo_4__1_____emplace_back_enum_eTransferCategory_(): pass

def _cGcFrontendPageFunctions__DoTransferPopup____67____lambda_1___operator__(): pass

def _cGcFrontendPageFunctions__DoTransferPopup____67____lambda_2___operator__(): pass

def _cGcFrontendPageFunctions__DoTransferPopup____67____lambda_3___operator__(): pass

def _cGcFrontendPageFunctions__DoTransferPopup____67____lambda_4___operator__(): pass

def _cGcFrontendPageFunctions__DoTransferPopup____67____lambda_5___operator__(): pass

def _cGcFrontendPageFunctions__DoTransferSendPopup____168____lambda_2___operator__(): pass

def AtlasIconRenderFn(): pass

def cGcCustomInventoryComponent__IterateTypedComponentsOnNode__cGcPurchaseableItem__Update____107____lambda_1___(): pass

def cTkJobManager__CreateJobLambda__cGcFrontendPageFunctions__DoProductSlot____72____lambda_1___(): pass

def std__vector__cGcFrontendPageFunctions__DoTransferPopup____2___TransferInfo_TkSTLAllocatorShim__cGcFrontendPageFunctions__DoTransferPopup____2___TransferInfo_4__1______Emplace_one_at_back_enum_eTransferCategory_int___(): pass

def _LN44_7(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcCustomInventoryComponent__IterateTypedComponentsOnNode__cGcPurchaseableItem__Update____107____lambda_1_______2____lambda_1___(): pass

def std__vector__cGcFrontendPageFunctions__DoTransferPopup____67___InaccessableInventory_TkSTLAllocatorShim__cGcFrontendPageFunctions__DoTransferPopup____67___InaccessableInventory_8__1______Emplace_reallocate_cGcInventoryStore___enum_InventoryChoice___(): pass

def std___Sort_unchecked_std__pair_float_SendToInventoryOption_____cGcFrontendPageFunctions__DoTransferSendPopup____168____lambda_2___(): pass

def std__vector__cGcFrontendPageFunctions__DoTransferPopup____2___TransferInfo_TkSTLAllocatorShim__cGcFrontendPageFunctions__DoTransferPopup____2___TransferInfo_4__1______Change_array(): pass

def std___Make_heap_unchecked_std__pair_float_SendToInventoryOption_____cGcFrontendPageFunctions__DoTransferSendPopup____168____lambda_2___(): pass

def std___Partition_by_median_guess_unchecked_std__pair_float_SendToInventoryOption_____cGcFrontendPageFunctions__DoTransferSendPopup____168____lambda_2___(): pass

def std___Pop_heap_hole_by_index_std__pair_float_SendToInventoryOption____std__pair_float_SendToInventoryOption___cGcFrontendPageFunctions__DoTransferSendPopup____168____lambda_2___(): pass

def std___Med3_unchecked_std__pair_float_SendToInventoryOption_____cGcFrontendPageFunctions__DoTransferSendPopup____168____lambda_2___(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageFunctions__DoTechnologySlot____341____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageFunctions__DoTechnologySlot____341____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageFunctions__DoTechnologySlot____341____lambda_1____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageFunctions__DoProductSlot____72____lambda_1____void____Delete_this(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageFunctions__DoProductSlot____72____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageFunctions__DoProductSlot____72____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcFrontendPageFunctions__DoProductSlot____72____lambda_1____void____Copy(): pass

def _cGcFrontendManager__AssignButtonIcons____2____lambda_1___operator__(): pass

def _cGcFrontendManager__UpdateRender____229____lambda_1___operator__(): pass

def _cGcFrontendManager__UpdateAllowedInteractions____2____lambda_1___operator__(): pass

def _cGcFrontendManager__RenderPage____2____lambda_1___operator__(): pass

def lookup_ACESmin(): pass

def lookup_ACESmax(): pass

def init_TsParams(): pass

def inv_ssts(): pass

def SSRActive(): pass

def GetLensDistortionVectors(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcGraphicsManager__ModelViewAttachmentPrepareCallback____2____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcGraphicsManager__ModelViewAttachmentPrepareCallback____2____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcGraphicsManager__ModelViewAttachmentPrepareCallback____2____lambda_1____void____Move(): pass

def ___emplace_back_UMissionDetailMessage_____vector_UMissionDetailMessage__V__TkSTLAllocatorShim_UMissionDetailMessage___07_0_0___std__QEAA_A_T__QEAUMissionDetailMessage___Z(): pass

def _cGcInWorldUIManager__RenderWeaponMenu____2____lambda_1___operator__(): pass

def _cGcInWorldUIManager__RenderWeaponMenu____2____lambda_2___operator__(): pass

def _cGcInWorldUIManager__RenderWeaponMenu____2____lambda_3___operator__(): pass

def _cGcInWorldUIManager__UpdateInWorldUISimulation____242____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__cGcInWorldUIManager__cGcScreenProjector__UpdateModel____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcInWorldUIManager__cGcScreenProjector__UpdateModel____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcInWorldUIManager__cGcScreenProjector__UpdateModel____2____lambda_1__bool_TkHandle____Move(): pass

def RenderTag(): pass

def ComputeVRScreenPos(): pass

def TrackArrowFadeVR(): pass

def _cGcBasePartsMenuUI__UpdateAvailablePartsBrowse____2____lambda_1___operator__(): pass

def GetScreenshotName(): pass

def OnConfirmDeleteClose(): pass

def _cGcPhotoManager__AcknowledgePhotoRequests____2____lambda_1___operator__(): pass

def SaveImageUsingWic(): pass

def Rec2390Rolloff(): pass

def TonemapHdrToSdr(): pass

def _cGcPlayerHUD__Update____150____lambda_2___operator__(): pass

def _cGcPlayer__UpdateControl____125____lambda_1____lambda_1_(): pass

def _cGcPlayerHUD__UpdateTrackArrows____2____lambda_1___operator__(): pass

def _cGcPlayerHUD__UpdateTrackArrows____2____lambda_3___operator__(): pass

def GetImpactFactor(): pass

def _cGcPlayerHUD__RenderOffscreen2D____5____lambda_1___operator__(): pass

def _cGcPlayerHUD__RenderOffscreen2D____5____lambda_3___operator__(): pass

def _cGcPlayerHUD__RenderIndicatorPanel____116____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcPlayerHUD__RenderHazardBarArrow____2____lambda_1___operator__(): pass

def _cGcPlayerHUD__RenderBuildInfo____100____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcPlayerHUD__RenderChatPanel____48____lambda_1___operator__(): pass

def _LN20_26(): pass

def _cGcInteractionComponent__IterateTypedComponentsOnNode__cGcPlayerHUD__Update____94____lambda_1_______2____lambda_1___operator__(): pass

def cGcGameState__ForEachFreighterOwnership__cGcPlayerHUD__Update____150____lambda_2___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcInteractionComponent__IterateTypedComponentsOnNode__cGcPlayerHUD__Update____94____lambda_1_______2____lambda_1___(): pass

def std___Sort_unchecked_cGcHUDMarker______cGcPlayerHUD__Render2D____14____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_35__bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_35__bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_35__bool____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_34__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_34__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_34__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_33__float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_33__float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_33__float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_32__bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_32__bool____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_31__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_31__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_31__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_30__float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_30__float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_30__float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_29__bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_29__bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_29__bool____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_28__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_28__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_28__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_27__float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_27__float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_27__float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_26__bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_26__bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_26__bool____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_25__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_25__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_25__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_24__float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_24__float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_24__float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_23__bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_23__bool____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_22__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_22__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_22__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_21__float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_21__float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_21__float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_20__bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_20__bool____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_19__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_19__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_19__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_18__float____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_18__float____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_18__float____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_17__void_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_17__void_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_17__void_int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_16__int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_16__int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_16__int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_15__void_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_15__void_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_15__void_int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_14__int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_14__int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____9____lambda_14__int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_13__bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_13__bool____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_12__bool_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_12__bool_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_12__bool_int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_11__void_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_11__void_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_11__void_int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_10__int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_10__int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_10__int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____4____lambda_9__void_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____4____lambda_9__void_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____4____lambda_9__void_int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____4____lambda_8__int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____4____lambda_8__int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____4____lambda_8__int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_7__void_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_7__void_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_7__void_int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_6__int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_6__int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_6__int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_5__bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_5__bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_5__bool____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_4__void_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_4__void_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_4__void_int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_3__int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_3__int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_3__int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_2__void_int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_2__void_int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_2__void_int____Move(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_1__int____Target_type(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_1__int____Do_call(): pass

def std___Func_impl_no_alloc__cGcPhotoModeUI__Prepare____3____lambda_1__int____Move(): pass

def std___Insertion_sort_unchecked_cGcHUDMarker______cGcPlayerHUD__Render2D____14____lambda_1___(): pass

def std___Make_heap_unchecked_cGcHUDMarker______cGcPlayerHUD__Render2D____14____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cGcHUDMarker______cGcPlayerHUD__Render2D____14____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cGcHUDMarker_____cGcHUDMarker____cGcPlayerHUD__Render2D____14____lambda_1___(): pass

def std___Med3_unchecked_cGcHUDMarker______cGcPlayerHUD__Render2D____14____lambda_1___(): pass

def _cGcPlayerNotifications__UpdateMissionSequences____39____lambda_1___operator__(): pass

def _cGcPlayerNotifications__RenderImGui____2____lambda_1___operator__(): pass

def _cGcShipHUD__Update____8____lambda_1___operator__(): pass

def _cGcShipHUD__Update____17____lambda_2___operator__(): pass

def _cGcShipHUD__UpdateTrackArrows____2____lambda_2___operator__(): pass

def _cGcShipHUD__RenderOffscreen2D____177____lambda_1___operator__(): pass

def _cGcShipHUD__RenderOffscreen2D____177____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcShipHUD__Render2D____11____lambda_1___operator__(): pass

def std___Max_element_unchecked_cGcNotificationSequence______cGcPlayerNotifications__GetHighestPriorityNotificationSequence____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcShipHUD__Update____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcShipHUD__Update____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcShipHUD__Update____8____lambda_1__bool_TkHandle____Move(): pass

def _cGcSpaceMapObjectUtils__ColourMultiMaterialMapObject____5____lambda_1___operator__(): pass

def ___emplace_back_AEAVcTkSmartResHandle_____vector_VcTkSmartResHandle__V__TkSTLAllocatorShim_VcTkSmartResHandle___03_0_0___std__QEAA_A_TAEAVcTkSmartResHandle___Z(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____39____lambda_4__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____39____lambda_4__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____39____lambda_4__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____18____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____18____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____18____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____13____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____13____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____13____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceMap__Update____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceMapObjectUtils__ColourMultiMaterialMapObject____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceMapObjectUtils__ColourMultiMaterialMapObject____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceMapObjectUtils__ColourMultiMaterialMapObject____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceMapObjectUtils__ColourMapObject____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceMapObjectUtils__ColourMapObject____5____lambda_1__bool_TkHandle____Move(): pass

class cGcSpaceshipComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def ___emplace_back_AEBU__pair_T__TkID__0IA___VcTkSeed___std_____vector_U__pair_T__TkID__0IA___VcTkSeed___std__V__TkSTLAllocatorShim_U__pair_T__TkID__0IA___VcTkSeed___std___07_0_0___std__QEAA_A_TAEBU__pair_T__TkID__0IA___VcTkSeed___1__Z(): pass

def CalculateUpcomingRecurrenceDeadline(): pass

def GetServerTime(): pass

def IsRecurrenceExpired(): pass

def IsLeapYear(): pass

class cGcEncounterComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcNotificationSequence(ctypes.Structure):
    pass

def cGcScannableComponent__IterateTypedComponents__cGcNotificationSequenceEnsureBarrelsAtPlayerSettlement__RunInternal____2____lambda_1___(): pass

def std__remove_if_std___Vector_iterator_std___Vector_val_std___Simple_types_enum_eDiscoveryType_______cGcNotificationSequenceExplorationLogSpecial__RunInternal____38____lambda_1___(): pass

def std__remove_if_std___Vector_iterator_std___Vector_val_std___Simple_types_cGcDiscoverySearch_________cGcNotificationSequenceExplorationLogSpecial__RunInternal____48____lambda_2___(): pass

def std__find_if_cGcDiscoverySearch______cGcNotificationSequenceExplorationLogSpecial__RunInternal____48____lambda_2___(): pass

def std___Func_impl_no_alloc__cGcNotificationSequenceExplorationLogSpecial__UpdatePlanetConstants____2____lambda_1__void_cGcPlanetObjectSpawnData_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNotificationSequenceExplorationLogSpecial__UpdatePlanetConstants____2____lambda_1__void_cGcPlanetObjectSpawnData_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNotificationSequenceExplorationLogSpecial__UpdatePlanetConstants____2____lambda_1__void_cGcPlanetObjectSpawnData_const______Move(): pass

def std___Func_impl_no_alloc___cGcNotificationSequenceExplorationLogSpecial__UpdatePlanetConstants____2____lambda_1___operator______2____lambda_1__bool_cGcEncyclopediaComponentData_const______Target_type(): pass

def std___Func_impl_no_alloc___cGcNotificationSequenceExplorationLogSpecial__UpdatePlanetConstants____2____lambda_1___operator______2____lambda_1__bool_cGcEncyclopediaComponentData_const______Do_call(): pass

def std___Func_impl_no_alloc___cGcNotificationSequenceExplorationLogSpecial__UpdatePlanetConstants____2____lambda_1___operator______2____lambda_1__bool_cGcEncyclopediaComponentData_const______Move(): pass

def _cGcInWorldScreen__Prepare____2____lambda_1___operator__(): pass

class GcNGuiFx(ctypes.Structure):
    def LoadingDistractionFullscreenNoPhase(): pass


def _cGcFrontendModelRenderer__Update____81____lambda_1___operator__(): pass

def ___emplace_back_AEBVcGcDiscoveryData__AEBV1_AEAH___vector_UcDiscoveryLoadRequest_cGcFrontendModelRenderer__V__TkSTLAllocatorShim_UcDiscoveryLoadRequest_cGcFrontendModelRenderer___0BA__0_0___std__QEAA_A_TAEBVcGcDiscoveryData__0AEAH_Z(): pass

def _cGcFrontendModelRenderer__UpdateLogic____2____lambda_1___operator__(): pass

def CommonFontHeightRescalingToFit(): pass

def _cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__Prepare____23____lambda_4_______2____lambda_1___operator__(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__Prepare____23____lambda_4_______2____lambda_1___(): pass

def std___Func_impl_no_alloc___cGcFrontendModelRenderer__UpdateLogic____2____lambda_1___operator______8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcFrontendModelRenderer__UpdateLogic____2____lambda_1___operator______8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcFrontendModelRenderer__UpdateLogic____2____lambda_1___operator______8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc___cGcInWorldScreen__Prepare____2____lambda_1___operator______2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcInWorldScreen__Prepare____2____lambda_1___operator______2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcInWorldScreen__Prepare____2____lambda_1___operator______2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNotificationSequenceDiscoverOnPlanet__RunInternal____14____lambda_1__void_cGcPlanetObjectSpawnData_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNotificationSequenceDiscoverOnPlanet__RunInternal____14____lambda_1__void_cGcPlanetObjectSpawnData_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNotificationSequenceDiscoverOnPlanet__RunInternal____14____lambda_1__void_cGcPlanetObjectSpawnData_const______Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcNGuiGraphic__EditElement____8____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcNGuiGraphic__EditElement____8____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcNGuiGraphic__EditElement____8____lambda_1____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcNGuiText__EditElement____35____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcNGuiText__EditElement____35____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcNGuiText__EditElement____35____lambda_1____void____Move(): pass

def NGuiRenderEmitterFloatProperty(): pass

def NGuiRenderFloatRange(): pass

def GetUIPlatformForFile(): pass

def _cGcNGuiEffectViewer__LoadEmitter____15____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__cGcNGuiEffectViewer__LoadEmitter____15____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNGuiEffectViewer__LoadEmitter____15____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNGuiEffectViewer__LoadEmitter____15____lambda_1__bool_TkHandle____Move(): pass

def GeneratePhysLocationMatrix(): pass

def _cGcMatchmaking__OnRemoteInviteToFireteamRequest____14____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcMatchmaking__OnRemoteRequestAddFriend____14____lambda_1____lambda_invoker_cdecl_(): pass

def cTkAttachment__IterateAttachments__cGcMonumentManager__SpawnMonument____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcNetworkAbandonedFreightersManager__BroadcastSpawn____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcNetworkAbandonedFreightersManager__SpawnFreighter____8____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkAbandonedFreightersManager__SpawnFreighter____8____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcMonumentManager__RetrieveCreatorsDisplayName____11____lambda_2__void_cTkFixedString_64_char__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcMonumentManager__RetrieveCreatorsDisplayName____11____lambda_2__void_cTkFixedString_64_char__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcMonumentManager__RetrieveCreatorsDisplayName____11____lambda_2__void_cTkFixedString_64_char__const______Copy(): pass

def std___Func_impl_no_alloc__cGcMonumentManager__RetrieveCreatorsDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcMonumentManager__RetrieveCreatorsDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcMonumentManager__RetrieveCreatorsDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmaking__ProcessInvites____37____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmaking__ProcessInvites____37____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmaking__ProcessInvites____37____lambda_1__void_bool____Move(): pass

def std___Func_impl_no_alloc__cGcMatchmaking__Update____66____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmaking__Update____66____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmaking__Update____66____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc___cGcMatchmaking__SendRequestPermissionCreateLobby____2____lambda_1___operator___cGcNetworkPlayerFireteam_____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmaking__SendRequestPermissionCreateLobby____2____lambda_1___operator___cGcNetworkPlayerFireteam_____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Do_call(): pass

def std___Func_impl_no_alloc___cGcMatchmaking__SendRequestPermissionCreateLobby____2____lambda_1___operator___cGcNetworkPlayerFireteam_____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Copy(): pass

class cGcPlayerEffectsComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcNetworkLockManager__SendUnlockMessage____2____lambda_1___operator__(): pass

class cGcPlayerCharacterComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcPlayerFullBodyIKComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcNetworkPlayerReplicatedState__Construct____2____lambda_3___operator__(): pass

class cGcPlayerComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator__(): pass

def _cGcNetworkReplicatedEntityManager__SendEntitySyncToOtherPlayers____5____lambda_1___operator__(): pass

def _cGcNetworkReplicatedEntityManager__SendReplicatedEntityToPlayer____2____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcNetworkIdManager__OnNetworkPlayerStateEvent____13____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcNetworkIdManager__BroadcastOwnershipChange____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcNetworkIdManager__OnNewClient____5____lambda_2___(): pass

def cGcInteractionComponent__IterateTypedComponents__cGcNetworkPlayer__GetInteraction____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachRemotePlayerIgnoreState__cGcNetworkPlayerManager__Update____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcNetworkPlayerManager__OnLocalPlayerRespawn____2____lambda_1___(): pass

def cTkNetReplicatedEntityComponent__IterateTypedComponents___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______42____lambda_2___(): pass

def _cTkAttachment__ForEachComponentFromNode__cGcNetworkReplicatedEntityManager__SerializeEntity____17____lambda_1_______2____lambda_1___operator__(): pass

def cTkNetReplicatedEntityComponent__IterateTypedComponents__cGcNetworkReplicatedEntityManager__RemoveEntitiesFromPlayer____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcNetworkReplicatedEntityManager__SendReplicatedEntityToPlayer____2____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cTkAttachment__ForEachComponentFromNode__cGcNetworkIdManager__RequestIdOwnership____2____lambda_1_______2____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cTkAttachment__ForEachComponentFromNode___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______73____lambda_4_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcAISpaceshipManager__SpawnShip____27____lambda_2_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______105____lambda_6_______2____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cTkAttachment__ForEachComponentFromNode__cGcNetworkReplicatedEntityManager__ChangeOwnership____2____lambda_2_______2____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cTkAttachment__ForEachComponentFromNode__cGcNetworkReplicatedEntityManager__AmInterestedInOwnership____2____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcNetworkReplicatedEntityManager__SendMessageToRelevantPlayers____2____lambda_1__bool_cGcNetworkPlayerFullSim______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkReplicatedEntityManager__SendMessageToRelevantPlayers____2____lambda_1__bool_cGcNetworkPlayerFullSim______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkReplicatedEntityManager__SendMessageToRelevantPlayers____2____lambda_1__bool_cGcNetworkPlayerFullSim______Move(): pass

def std___Func_impl_no_alloc___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______70____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______70____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______70____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______13____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______13____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc___cGcNetworkReplicatedEntityManager__HandleEntityCreationRequests____2____lambda_1___operator______13____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_10__void_cGcSettlementState_const___cGcSettlementState_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_10__void_cGcSettlementState_const___cGcSettlementState_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_10__void_cGcSettlementState_const___cGcSettlementState_const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_9__void_cTkFixedString_64_char__const___cTkFixedString_64_char__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_9__void_cTkFixedString_64_char__const___cTkFixedString_64_char__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_9__void_cTkFixedString_64_char__const___cTkFixedString_64_char__const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_8__void_unsigned___int64_const___unsigned___int64_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_8__void_unsigned___int64_const___unsigned___int64_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_8__void_unsigned___int64_const___unsigned___int64_const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____4____lambda_6__void_unsigned_int_const___unsigned_int_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____4____lambda_6__void_unsigned_int_const___unsigned_int_const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____4____lambda_7__bool_cGcNetworkPlayer______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____4____lambda_7__bool_cGcNetworkPlayer______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____4____lambda_7__bool_cGcNetworkPlayer______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_5__void_TkID_128__const___TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_5__void_TkID_128__const___TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_5__void_TkID_128__const___TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_4__void_TkID_128__const___TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_4__void_TkID_128__const___TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_4__void_TkID_128__const___TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_3__void_unsigned___int64_const___unsigned___int64_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_3__void_unsigned___int64_const___unsigned___int64_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_3__void_unsigned___int64_const___unsigned___int64_const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_2__void_cTkFixedString_128_char__const___cTkFixedString_128_char__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_2__void_cTkFixedString_128_char__const___cTkFixedString_128_char__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_2__void_cTkFixedString_128_char__const___cTkFixedString_128_char__const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const___cTkUserIdBase_cTkFixedString_64_char____const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const___cTkUserIdBase_cTkFixedString_64_char____const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerReplicatedState__Construct____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const___cTkUserIdBase_cTkFixedString_64_char____const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerManager__GetPlayerFromAttachment____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerManager__GetPlayerFromAttachment____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerManager__GetPlayerFromAttachment____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkIdManager__RequestObjectOwnership____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Delete_this(): pass

def std___Func_impl_no_alloc__cGcNetworkIdManager__RequestObjectOwnership____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkIdManager__RequestObjectOwnership____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkIdManager__RequestObjectOwnership____2____lambda_1__void_enum_eGcNetworkTransactionStatus_bool______Copy(): pass

def std___Func_impl_no_alloc__cGcNetworkCreatureManager__MaySpawnCreatureAsync____10____lambda_1__void_enum_eGcNetworkTransactionStatus_cTkUserIdBase_cTkFixedString_64_char_________Delete_this(): pass

def std___Func_impl_no_alloc__cGcNetworkCreatureManager__MaySpawnCreatureAsync____10____lambda_1__void_enum_eGcNetworkTransactionStatus_cTkUserIdBase_cTkFixedString_64_char_________Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkCreatureManager__MaySpawnCreatureAsync____10____lambda_1__void_enum_eGcNetworkTransactionStatus_cTkUserIdBase_cTkFixedString_64_char_________Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkCreatureManager__MaySpawnCreatureAsync____10____lambda_1__void_enum_eGcNetworkTransactionStatus_cTkUserIdBase_cTkFixedString_64_char_________Copy(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerManager__Prepare____2____lambda_1__float_cTkVector3_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerManager__Prepare____2____lambda_1__float_cTkVector3_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkPlayerManager__Prepare____2____lambda_1__float_cTkVector3_const______Move(): pass

def __cGcNetworkWeapon__GetBarrelPos____8____lambda_1___operator______5____lambda_1___operator__(): pass

def _cGcNetworkWeapon__CloneMuzzleFlashMaterials____6____lambda_1___operator__(): pass

def _cGcTextChatManager__AddPendingMessage____50____lambda_1___operator__(): pass

def cGcPlayerWeaponComponent__IterateTypedComponents__cGcNetworkWeapon__GetBarrelPos____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcNetworkWeapon__OnHolsterStatusChange____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcNetworkWeapon__OnMeleeEngaged____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcNetworkWeapon__OnReload____2____lambda_1___(): pass

def std___Sort_unchecked___int64____cGcNetworkSynchronizedClock__OnHandleFullRoundTimeUpdate____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__GeneratePetChatText____2____lambda_2__cTkFixedString_128_char__enum_ePetVocabularyWord____Target_type(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__GeneratePetChatText____2____lambda_2__cTkFixedString_128_char__enum_ePetVocabularyWord____Do_call(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__GeneratePetChatText____2____lambda_2__cTkFixedString_128_char__enum_ePetVocabularyWord____Move(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__GeneratePetChatText____2____lambda_1__cTkFixedString_128_char__enum_ePetChatType____Target_type(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__GeneratePetChatText____2____lambda_1__cTkFixedString_128_char__enum_ePetChatType____Do_call(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__GeneratePetChatText____2____lambda_1__cTkFixedString_128_char__enum_ePetChatType____Move(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__Update____9____lambda_1__void_bool_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__Update____9____lambda_1__void_bool_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcTextChatManager__Update____9____lambda_1__void_bool_char_const______Move(): pass

def std___Func_impl_no_alloc__cGcNetworkWeapon__CloneMuzzleFlashMaterials____6____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkWeapon__CloneMuzzleFlashMaterials____6____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkWeapon__CloneMuzzleFlashMaterials____6____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNetworkWeapon__RefreshShipBarrels____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNetworkWeapon__RefreshShipBarrels____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNetworkWeapon__RefreshShipBarrels____8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc___cGcNetworkWeapon__GetBarrelPos____8____lambda_1___operator______5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcNetworkWeapon__GetBarrelPos____8____lambda_1___operator______5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcNetworkWeapon__GetBarrelPos____8____lambda_1___operator______5____lambda_1__bool_TkHandle____Move(): pass

def std___Partition_by_median_guess_unchecked___int64____cGcNetworkSynchronizedClock__OnHandleFullRoundTimeUpdate____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index___int64_____int64__cGcNetworkSynchronizedClock__OnHandleFullRoundTimeUpdate____2____lambda_1___(): pass

def std___Med3_unchecked___int64____cGcNetworkSynchronizedClock__OnHandleFullRoundTimeUpdate____2____lambda_1___(): pass

def PartyGetWindowsDllExports(): pass

def _cGcMatchmakingPlayfab__Connect____5____lambda_1___operator__(): pass

def __cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______2____lambda_4___operator__(): pass

def ___emplace_back_AEAUBlockedPlayer_cGcMatchmakingPlayfab_____vector_UBlockedPlayer_cGcMatchmakingPlayfab__V__TkSTLAllocatorShim_UBlockedPlayer_cGcMatchmakingPlayfab___00_0_0___std__QEAA_A_TAEAUBlockedPlayer_cGcMatchmakingPlayfab___Z(): pass

def _LN110_3(): pass

def __cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_1___operator______9____lambda_1___operator___cTkUserIdBase_cTkFixedString_64_char____cTkUserIdBase_cTkFixedString_64_char_____(): pass

def _cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_1___operator__(): pass

def _cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_2___operator__(): pass

def _LN368(): pass

def _cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_1___operator__(): pass

def _cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_2___operator___PlayFab__ClientModels__GetPlayFabIDsFromGenericIDsResult_(): pass

def _LN252_1(): pass

def __cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_1___operator__(): pass

def _cGcMatchmakingPlayfab__AddFriend____2____lambda_2___operator__(): pass

def _cGcMatchmakingPlayfab__BlockPlayer____5____lambda_2___operator__(): pass

def _cGcMatchmakingPlayfab__UnblockPlayer____2____lambda_1___operator__(): pass

def _cGcMatchmakingPlayfab__UpdateProfileLookup____13____lambda_1___operator__(): pass

def _cGcMatchmakingPlayfab__GetLocalUserObjectData____2____lambda_1___operator__(): pass

def _anonymous_namespace___ParseLobbyIDFromConnectString(): pass

def cTkAttachment__IterateAttachments__cTkAttachment__ForEachComponentFromNode__sGcNetworkReplicatedEntitiesReplicatedVariablesMessage__SerializeToNode____5____lambda_1_______2____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cTkAttachment__ForEachComponentFromNode__sGcNetworkReplicatedEntitiesReplicatedVariablesMessage__VariablesChanged____5____lambda_1_______2____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cTkAttachment__ForEachComponentFromNode__sGcNetworkReplicatedEntitiesWelcomeMessage__SerializeComponents____5____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfabGalaxy__LoginToPlayfab____6____lambda_2__void_PlayFab__PlayFabError_const___void______Delete_this(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfabGalaxy__LoginToPlayfab____6____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfabGalaxy__LoginToPlayfab____6____lambda_2__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfabGalaxy__LoginToPlayfab____6____lambda_2__void_PlayFab__PlayFabError_const___void______Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfabGalaxy__LoginToPlayfab____6____lambda_1__void_PlayFab__ClientModels__LoginResult_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfabGalaxy__LoginToPlayfab____6____lambda_1__void_PlayFab__ClientModels__LoginResult_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfabGalaxy__LoginToPlayfab____6____lambda_1__void_PlayFab__ClientModels__LoginResult_const___void______Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__GetLocalUserObjectData____2____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__GetLocalUserObjectData____2____lambda_2__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__GetLocalUserObjectData____2____lambda_2__void_PlayFab__PlayFabError_const___void______Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__GetLocalUserObjectData____2____lambda_1__void_PlayFab__DataModels__GetObjectsResponse_const___void______Delete_this(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__GetLocalUserObjectData____2____lambda_1__void_PlayFab__DataModels__GetObjectsResponse_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__GetLocalUserObjectData____2____lambda_1__void_PlayFab__DataModels__GetObjectsResponse_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__GetLocalUserObjectData____2____lambda_1__void_PlayFab__DataModels__GetObjectsResponse_const___void______Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__SetLocalUserObjectData____2____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__SetLocalUserObjectData____2____lambda_2__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__SetLocalUserObjectData____2____lambda_2__void_PlayFab__PlayFabError_const___void______Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__SetLocalUserObjectData____2____lambda_1__void_PlayFab__DataModels__SetObjectsResponse_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__SetLocalUserObjectData____2____lambda_1__void_PlayFab__DataModels__SetObjectsResponse_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__SetLocalUserObjectData____2____lambda_1__void_PlayFab__DataModels__SetObjectsResponse_const___void______Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RefreshPlayfabLogin____5____lambda_2__void_char_const___enum_PlayFab__PlayFabErrorCode____Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RefreshPlayfabLogin____5____lambda_2__void_char_const___enum_PlayFab__PlayFabErrorCode____Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RefreshPlayfabLogin____5____lambda_2__void_char_const___enum_PlayFab__PlayFabErrorCode____Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RefreshPlayfabLogin____5____lambda_1__void_PlayFab__ClientModels__LoginResult_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RefreshPlayfabLogin____5____lambda_1__void_PlayFab__ClientModels__LoginResult_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RefreshPlayfabLogin____5____lambda_1__void_PlayFab__ClientModels__LoginResult_const______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateProfileLookup____13____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateProfileLookup____13____lambda_2__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateProfileLookup____13____lambda_2__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateProfileLookup____13____lambda_1__void_PlayFab__ProfilesModels__GetEntityProfilesResponse_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateProfileLookup____13____lambda_1__void_PlayFab__ProfilesModels__GetEntityProfilesResponse_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateProfileLookup____13____lambda_1__void_PlayFab__ProfilesModels__GetEntityProfilesResponse_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UnblockPlayer____2____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UnblockPlayer____2____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UnblockPlayer____2____lambda_1__void_bool____Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__BlockPlayer____5____lambda_2__void_bool____Delete_this(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__BlockPlayer____5____lambda_2__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__BlockPlayer____5____lambda_2__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__BlockPlayer____5____lambda_2__void_bool____Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RemoveFriend____5____lambda_3__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RemoveFriend____5____lambda_3__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RemoveFriend____5____lambda_3__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RemoveFriend____5____lambda_2__void_PlayFab__ClientModels__RemoveFriendResult_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RemoveFriend____5____lambda_2__void_PlayFab__ClientModels__RemoveFriendResult_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__RemoveFriend____5____lambda_2__void_PlayFab__ClientModels__RemoveFriendResult_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriend____2____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriend____2____lambda_2__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriend____2____lambda_2__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriend____2____lambda_1__void_PlayFab__ClientModels__AddFriendResult_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriend____2____lambda_1__void_PlayFab__ClientModels__AddFriendResult_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriend____2____lambda_1__void_PlayFab__ClientModels__AddFriendResult_const___void______Copy(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_2__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_2__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_1__void_PlayFab__ClientModels__GetFriendsListResult_const___void______Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_1__void_PlayFab__ClientModels__GetFriendsListResult_const___void______Do_call(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_1__void_PlayFab__ClientModels__GetFriendsListResult_const___void______Copy(): pass

def std___Func_impl_no_alloc____cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_1___operator______23____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc____cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_1___operator______23____lambda_2__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc____cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_1___operator______23____lambda_1__void_PlayFab__ClientModels__RemoveFriendResult_const___void______Target_type(): pass

def std___Func_impl_no_alloc____cGcMatchmakingPlayfab__AddFriend____2____lambda_1___operator______5____lambda_1___operator______23____lambda_1__void_PlayFab__ClientModels__RemoveFriendResult_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriendFromNetworkPlayer____2____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriendFromNetworkPlayer____2____lambda_2__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriendFromNetworkPlayer____2____lambda_2__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriendFromNetworkPlayer____2____lambda_1__void_PlayFab__ProfilesModels__GetEntityProfileResponse_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriendFromNetworkPlayer____2____lambda_1__void_PlayFab__ProfilesModels__GetEntityProfileResponse_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__AddFriendFromNetworkPlayer____2____lambda_1__void_PlayFab__ProfilesModels__GetEntityProfileResponse_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_2__void_PlayFab__ClientModels__GetPlayFabIDsFromGenericIDsResult_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_2__void_PlayFab__ClientModels__GetPlayFabIDsFromGenericIDsResult_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_2__void_PlayFab__ClientModels__GetPlayFabIDsFromGenericIDsResult_const___void______Move(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_2___operator___PlayFab__ClientModels__GetPlayFabIDsFromGenericIDsResult_____6____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkUserIdBase_cTkFixedString_64_char________Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_2___operator___PlayFab__ClientModels__GetPlayFabIDsFromGenericIDsResult_____6____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkUserIdBase_cTkFixedString_64_char________Do_call(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_2___operator___PlayFab__ClientModels__GetPlayFabIDsFromGenericIDsResult_____6____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkUserIdBase_cTkFixedString_64_char________Copy(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_1__void_sGcPlayfabGenericServiceId_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_1__void_sGcPlayfabGenericServiceId_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupPlayfabIdFromGenericServiceId____2____lambda_1__void_sGcPlayfabGenericServiceId_const______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_2__void_PlayFab__ProfilesModels__GetTitlePlayersFromMasterPlayerAccountIdsResponse_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_2__void_PlayFab__ProfilesModels__GetTitlePlayersFromMasterPlayerAccountIdsResponse_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_2__void_PlayFab__ProfilesModels__GetTitlePlayersFromMasterPlayerAccountIdsResponse_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupTitleIdFromMasterId____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_3__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_2__void_PlayFab__ProfilesModels__GetEntityProfilesResponse_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_2__void_PlayFab__ProfilesModels__GetEntityProfilesResponse_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_2__void_PlayFab__ProfilesModels__GetEntityProfilesResponse_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__LookupUsernameFromTitleId____2____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____const______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__OnFindFriendsLobbiesComplete____22____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkFixedString_128_char______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__OnFindFriendsLobbiesComplete____22____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkFixedString_128_char______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__OnFindFriendsLobbiesComplete____22____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkFixedString_128_char______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_2__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_1__void_PlayFab__ClientModels__GetFriendsListResult_const___void______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_1__void_PlayFab__ClientModels__GetFriendsListResult_const___void______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_1__void_PlayFab__ClientModels__GetFriendsListResult_const___void______Move(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_1___operator______9____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkUserIdBase_cTkFixedString_64_char________Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_1___operator______9____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkUserIdBase_cTkFixedString_64_char________Do_call(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__UpdateCrossplayFriends____5____lambda_1___operator______9____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char____cTkUserIdBase_cTkFixedString_64_char________Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__Connect____5____lambda_2__void_char_const___enum_PlayFab__PlayFabErrorCode____Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__Connect____5____lambda_2__void_char_const___enum_PlayFab__PlayFabErrorCode____Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__Connect____5____lambda_2__void_char_const___enum_PlayFab__PlayFabErrorCode____Move(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__Connect____5____lambda_1__void_PlayFab__ClientModels__LoginResult_const______Delete_this(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__Connect____5____lambda_1__void_PlayFab__ClientModels__LoginResult_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__Connect____5____lambda_1__void_PlayFab__ClientModels__LoginResult_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcMatchmakingPlayfab__Connect____5____lambda_1__void_PlayFab__ClientModels__LoginResult_const______Copy(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______2____lambda_4__void_Json__Value____Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______2____lambda_4__void_Json__Value____Do_call(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______2____lambda_4__void_Json__Value____Move(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______7____lambda_3__void_bool____Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______7____lambda_3__void_bool____Move(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______6____lambda_2__void_PlayFab__PlayFabError_const___void______Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______6____lambda_2__void_PlayFab__PlayFabError_const___void______Move(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______6____lambda_1__void_PlayFab__ClientModels__UpdateUserTitleDisplayNameResult_const___void______Target_type(): pass

def std___Func_impl_no_alloc___cGcMatchmakingPlayfab__Connect____5____lambda_1___operator______6____lambda_1__void_PlayFab__ClientModels__UpdateUserTitleDisplayNameResult_const___void______Move(): pass

def std__invoke__cGcMatchmakingPlayfab__SetLocalUserObjectData____2____lambda_1____PlayFab__DataModels__SetObjectsResponse_const___void___(): pass

def std__invoke__cGcMatchmakingPlayfab__AddFriendFromNetworkPlayer____2____lambda_1____PlayFab__ProfilesModels__GetEntityProfileResponse_const___void___(): pass

def _MinimumWeightSearch(): pass

def _RandomTableSelection(): pass

def FrequencyTables__GE__TrigramSearch(): pass

def FrequencyTables__CH__TrigramSearch(): pass

def FrequencyTables__JP__TrigramSearch(): pass

def FrequencyTables__NO__TrigramSearch(): pass

def FrequencyTables__RU__TrigramSearch(): pass

def FrequencyTables__LT__TrigramSearch(): pass

def FrequencyTables__FL__TrigramSearch(): pass

def FrequencyTables__MN__TrigramSearch(): pass

def ScanAndDiffuseVulgarityAt(): pass

def _cGcNameGenerator__GenerateAbandonedFreighterLogFragment____2____lambda_1___operator__(): pass

def _cGcNameGenerator__GenerateAbandonedFreighterSecurityLog____2____lambda_1___operator__(): pass

def _vsnprintf_s(): pass

def GcAppendConvertedToJapanese(): pass

def _GcAppendConvertedToJapanese____2____lambda_2___operator__(): pass

def _GcAppendConvertedToJapanese____2____lambda_3___operator__(): pass

def _GcAppendConvertedToJapanese____2____lambda_7___operator__(): pass

def _GcAppendConvertedToJapanese____2____lambda_8___operator__(): pass

def _GcAppendConvertedToJapanese____2____lambda_9___operator__(): pass

def _GcAppendConvertedToJapanese____2____lambda_10___operator__(): pass

def _GcAppendConvertedToJapanese____2____lambda_11___operator__(): pass

def GcAppendConvertedToJapaneseAlphanumeric(): pass

def GcConvertFullWidthToHalfWidth(): pass

def _GcConvertFullWidthToHalfWidth____2____lambda_1___operator__(): pass

def ___emplace_back_PEAVcGcAlienPuzzleTable__H___vector_U__pair_PEBVcGcAlienPuzzleTable__H_std__V__TkSTLAllocatorShim_U__pair_PEBVcGcAlienPuzzleTable__H_std___07_0_0___std__QEAA_A_T__QEAPEAVcGcAlienPuzzleTable____QEAH_Z(): pass

def CopyToNameStringUtf8LengthLimited(): pass

def ___emplace_back_AEBT__TkID__0IA___AEBT1____vector_U__pair_T__TkID__0IA___T1__std__V__TkSTLAllocatorShim_U__pair_T__TkID__0IA___T1__std___07_0_0___std__QEAA_A_TAEBT__TkID__0IA___0_Z(): pass

def cpToUTF8(): pass

def _cGcRewardManager__DoDeferredReward____2____lambda_1___operator__(): pass

def _cGcRewardManager__DoDeferredReward____2____lambda_1____lambda_1_(): pass

def _cGcRewardManager__GiveRewardDamageTech____2____lambda_1___operator__(): pass

def cTkJobManager__CreateJobLambda__cGcRewardManager__DoDeferredReward____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardTeleport____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardTeleport____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardClosePortal____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardClosePortal____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardClosePortal____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardPortalState____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardPortalState____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardPortalState____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardTimeWarp____2____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardTimeWarp____2____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardTimeWarp____2____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardCrashSiteRepair____14____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourFlyAway__Run____26____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRewardManager__GiveRewardCrashSiteRepair____14____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcRewardManager__StripModelBasedOnRarityOfReward____7____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRewardManager__StripModelBasedOnRarityOfReward____7____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRewardManager__StripModelBasedOnRarityOfReward____7____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcRewardManager__GiveSeasonReward____2____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcRewardManager__GiveSeasonReward____2____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcRewardManager__GiveSeasonReward____2____lambda_1____void____Copy(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcRewardManager__DoDeferredReward____2____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcRewardManager__DoDeferredReward____2____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcRewardManager__DoDeferredReward____2____lambda_1____void____Copy(): pass

class cGcNetworkInterpolationComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcNPCBehaviourTree__GetNumPlayersInNavArea____2____lambda_1___operator__(): pass

def GetBuildingClassWeightEntry(): pass

def _cGcNPCBehaviourChooseInteraction__GatherValidDestinations____2____lambda_2___operator__(): pass

def _cGcNPCBehaviourPerformInteraction__ChooseNewState____5____lambda_1___operator__(): pass

def _cGcNPCBehaviourGreet__Run____25____lambda_2___operator__(): pass

def _cGcNPCBehaviourReactToPlayer__Run____54____lambda_3___operator__(): pass

def _cGcNPCBehaviourReactToPlayer__Subscribe____2____lambda_1___operator__(): pass

def _cGcNPCBehaviourReactToJudgement__Subscribe____2____lambda_1___operator__(): pass

def _cGcNPCBehaviourInteraction__Subscribe____2____lambda_1___operator__(): pass

def cGcNPCComponent__IterateTypedComponents__cGcNPCBehaviourTree__GetNumNPCsDestinedForNavArea____2____lambda_1___(): pass

def cGcNPCComponent__IterateTypedComponents__cGcNPCBehaviourTree__GetNumNPCsDestinedForBuilding____2____lambda_1___(): pass

def cGcNPCComponent__IterateTypedComponents__cGcNPCBehaviourChooseInteraction__GatherValidDestinations____17____lambda_1___(): pass

def ___emplace_back_V__function___A6AXAEBU__cTkMetaMessageWrapperTemplated_X___Z_std_____vector_V__function___A6AXAEBU__cTkMetaMessageWrapperTemplated_X___Z_std__V__StackAllocator_V__function___A6AXAEBU__cTkMetaMessageWrapperTemplated_X___Z_std___01_0_0___std__QEAA_A_T__QEAV__function___A6AXAEBU__cTkMetaMessageWrapperTemplated_X___Z_1__Z(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourFlyAway__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourFlyAway__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourFlyAway__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourFlyAway__Run____26____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourFlyAway__Run____26____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourInteraction__Run____65____lambda_2__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourInteraction__Run____65____lambda_2__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourInteraction__Run____65____lambda_2__void____Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToJudgement__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToJudgement__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToJudgement__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Run____54____lambda_3__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Run____54____lambda_3__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Run____54____lambda_3__void____Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Run____10____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Run____10____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourReactToPlayer__Run____10____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourGreet__Run____25____lambda_2__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourGreet__Run____25____lambda_2__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourGreet__Run____25____lambda_2__void____Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourGreet__Run____10____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourGreet__Run____10____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourGreet__Run____10____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__BeginAnimationState____37____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__BeginAnimationState____37____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__BeginAnimationState____37____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____22____lambda_3__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____22____lambda_3__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____22____lambda_3__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____24____lambda_2__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____24____lambda_2__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____24____lambda_2__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____22____lambda_1__void_float____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____22____lambda_1__void_float____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourPerformInteraction__PlayStateAnimation____22____lambda_1__void_float____Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourChooseInteraction__GatherValidDestinations____2____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Delete_this(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourChooseInteraction__GatherValidDestinations____2____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourChooseInteraction__GatherValidDestinations____2____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourChooseInteraction__GatherValidDestinations____2____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Copy(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourTree__GetNavAreaTransferSelectionWeight____14____lambda_1__bool_cGcNavGraphNode_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourTree__GetNavAreaTransferSelectionWeight____14____lambda_1__bool_cGcNavGraphNode_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourTree__GetNavAreaTransferSelectionWeight____14____lambda_1__bool_cGcNavGraphNode_const______Move(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourTree__Construct____55____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourTree__Construct____55____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCBehaviourTree__Construct____55____lambda_1__bool_TkHandle____Move(): pass

class cGcSpawnedObjectComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcDissolveEffectComponent__UpdateRender____16____lambda_3___operator__(): pass

class cTkSketchComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcBaseMiniPortalComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcAbandonedFreighterComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcBuildingComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcSentinelRobotComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcGroundWormComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def ___emplace_back_AEAU__TkStrongType_HUTkResHandleID_TkStrongTypeIDs_______vector_VcTkSmartResHandle__V__TkSTLAllocatorShim_VcTkSmartResHandle___03_0_0___std__QEAA_A_TAEAU__TkStrongType_HUTkResHandleID_TkStrongTypeIDs_____Z(): pass

def _cGcGrabbableComponent__Prepare____4____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerEffectsComponent__TriggerEffect____5____lambda_1___(): pass

def cGcPlayerEffectsComponent__IterateTypedComponents__cGcPlayerEffectsComponent__ReplicateEffect____2____lambda_1___(): pass

def cGcEncounterStateComponent__IterateTypedComponents__cGcEncounterComponent__GetOrCreateStateComponent____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcGrabPlayerComponent__Update____5____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcDestructableComponent__IterateTypedComponentsOnNode__cGcEncounterComponent__Update____57____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcGrabbableComponent__Prepare____4____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcGrabbableComponent__Prepare____4____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcGrabbableComponent__Prepare____4____lambda_1__bool_TkHandle____Copy(): pass

def std___Func_impl_no_alloc__cGcExpeditionHologramComponent__SetFrigateNodeUserData____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcExpeditionHologramComponent__SetFrigateNodeUserData____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcExpeditionHologramComponent__SetFrigateNodeUserData____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcExpeditionHologramComponent__UpdateFrigateHologramsInternal____15____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcExpeditionHologramComponent__UpdateFrigateHologramsInternal____15____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcChairComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcChairComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcChairComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcAtmosphereEntryComponent__Update____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Update____23____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcAtmosphereEntryComponent__Update____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerEffectsComponent__Dissolve__Render____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerEffectsComponent__Dissolve__Render____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerEffectsComponent__Dissolve__Render____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcDissolveEffectComponent__UpdateRender____16____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcDissolveEffectComponent__UpdateRender____16____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcDissolveEffectComponent__UpdateRender____16____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcDissolveEffectComponent__UpdateRender____14____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcDissolveEffectComponent__UpdateRender____14____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcDissolveEffectComponent__UpdateRender____14____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcObjectSpawnerComponent__Update____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleGarageComponent__Update____8____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcObjectSpawnerComponent__Update____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcObjectSpawnerComponent__OnAttached____5____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcObjectSpawnerComponent__OnAttached____5____lambda_1__enum_TkPollResult____Move(): pass

def _cGcGroundWormComponent__UpdateState____61____lambda_2___operator__(): pass

class cGcPortalComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcVehicleCheckpointComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcInteractionComponent__Prepare____2____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcGroundWormComponent__Update____5____lambda_1___(): pass

def cGcGroundWormComponent__IterateTypedComponents__cGcGroundWormComponent__UpdateState____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcInteractionComponent__SwitchWeaponMaterials____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcInteractionComponent__SwitchWeaponMaterials____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcInteractionComponent__SwitchWeaponMaterials____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcInteractionComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcInteractionComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcInteractionComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcGroundWormComponent__UpdateState____61____lambda_2__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcGroundWormComponent__UpdateState____61____lambda_2__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcGroundWormComponent__UpdateState____61____lambda_2__void_cGcPlayerCommon______Move(): pass

class cGcObjectPlacementComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcNPCComponent__RPCReadyForInteraction____5____lambda_1___operator__(): pass

def ___emplace_back_AEAUTrackedThing_NPCPerception_cGcNPCComponent_____vector_UTrackedThing_NPCPerception_cGcNPCComponent__V__TkSTLAllocatorShim_UTrackedThing_NPCPerception_cGcNPCComponent___0BA__0_0___std__QEAA_A_TAEAUTrackedThing_NPCPerception_cGcNPCComponent___Z(): pass

def GetOtherAlienRace(): pass

def cGcLadderComponent__IterateTypedComponents__cGcLadderComponent__GetFurthestExtremeDismount____2____lambda_1___(): pass

def _cGcLadderComponent__GetFurthestExtremeDismount____2____lambda_1___operator___int_cGcLadderComponent___(): pass

def cTkAttachment__IterateAttachments__cGcMaintenanceComponent__BroadcastStateChangeToNode____5____lambda_1___(): pass

def cGcGameState__ForEachFleetManager__cGcMaintenanceComponent__GetSavePosition____12____lambda_1___(): pass

def cGcPlayerCharacterComponent__IterateTypedComponents__cGcNPCComponent__UpdatePerception____5____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cGcNPCPlacementComponent__AddNPC____24____lambda_1___(): pass

def _cGcInteractionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__Prepare____8____lambda_1_______2____lambda_1___operator__(): pass

def _cGcInteractionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__Prepare____11____lambda_2_______2____lambda_1___operator__(): pass

def _cTkAnimationComponent__IterateTypedComponentsOnNode__cGcPortalComponent__Update____99____lambda_1_______2____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPortalComponent__Update____126____lambda_2___(): pass

def _cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__ResetEverythingExceptTerminal____2____lambda_2_______2____lambda_1___operator__(): pass

def _cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__ResetEverythingExceptTerminal____2____lambda_3_______2____lambda_1___operator__(): pass

def _cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__DisablePortalInteraction____2____lambda_1_______2____lambda_1___operator__(): pass

def std___Sort_unchecked_TkID_128_____cGcNPCComponent__RenderNGui____23____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcInteractionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__Prepare____8____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcInteractionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__Prepare____11____lambda_2_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkAnimationComponent__IterateTypedComponentsOnNode__cGcPortalComponent__Update____99____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__ResetEverythingExceptTerminal____2____lambda_2_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__ResetEverythingExceptTerminal____2____lambda_3_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcPortalComponent__DisablePortalInteraction____2____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPortalComponent__Prepare____17____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPortalComponent__Prepare____17____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPortalComponent__Prepare____17____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCPlacementComponent__GeneratePlacements____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCPlacementComponent__GeneratePlacements____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCPlacementComponent__GeneratePlacements____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCPlacementComponent__Update____50____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCPlacementComponent__Update____50____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCPlacementComponent__Update____50____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__UpdateDissolveProgress____11____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__UpdateDissolveProgress____11____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__UpdateDissolveProgress____11____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__SetPropNode____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__SetPropNode____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__SetPropNode____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__RPCReadyForInteraction____5____lambda_1__void_bool____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__RPCReadyForInteraction____5____lambda_1__void_bool____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCComponent__RPCReadyForInteraction____5____lambda_1__void_bool____Copy(): pass

def std___Func_impl_no_alloc__cGcLootComponent__SetColour____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcShootPoint__Update____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcLootComponent__SetColour____2____lambda_1__bool_TkHandle____Move(): pass

def std___Partition_by_median_guess_unchecked_TkID_128_____cGcNPCComponent__RenderNGui____23____lambda_1___(): pass

def std___Pop_heap_hole_by_index_TkID_128____TkID_128___cGcNPCComponent__RenderNGui____23____lambda_1___(): pass

def std___Med3_unchecked_TkID_128_____cGcNPCComponent__RenderNGui____23____lambda_1___(): pass

def HkVectorToTkVector(): pass

class cGcLadderComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcThereminComponent__Prepare____2____lambda_1___operator__(): pass

def _cGcTurretComponent__Prepare____3____lambda_1___operator__(): pass

class cGcClothComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcCustomiserInteraction__Update____68____lambda_3___operator__(): pass

def _cGcMPMissionGiverInteraction__Prepare____2____lambda_1___operator__(): pass

def _cGcShipSalvageInteraction__Explode____2____lambda_2___operator__(): pass

def _cGcShipSalvageInteraction__Explode____2____lambda_3___operator__(): pass

def cTkAttachment__IterateAttachments__cGcSimpleInteractionComponent__DoAction____181____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cGcSimpleInteractionComponent__DoAction____201____lambda_2___(): pass

def cTkAttachment__IterateAttachments__cGcSimpleInteractionComponent__RefreshPersistency____8____lambda_1___(): pass

def std__vector__cGcShipSalvageInteraction__Explode____2___StackData_TkSTLAllocatorShim__cGcShipSalvageInteraction__Explode____2___StackData_16__1______Emplace_one_at_back_cTkVector3_cTkVector3_float_cTkVector3_enum_eExplodeAction_(): pass

def std___Func_impl_no_alloc__cGcSpecialsShopInteraction__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpecialsShopInteraction__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpecialsShopInteraction__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcShipSalvageInteraction__SpawnSalvageShipHologram____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcShipSalvageInteraction__SpawnSalvageShipHologram____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcShipSalvageInteraction__SpawnSalvageShipHologram____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcShipSalvageInteraction__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcShipSalvageInteraction__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcShipSalvageInteraction__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcMPMissionGiverInteraction__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcMPMissionGiverInteraction__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcMPMissionGiverInteraction__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__UpdateRender____32____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__UpdateRender____32____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__UpdateRender____32____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__UpdateRender____29____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Interact____60____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__UpdateRender____29____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____68____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____68____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____68____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____13____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____13____lambda_2__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____13____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____5____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____5____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Update____5____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Interact____60____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCustomiserInteraction__Interact____60____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcTurretComponent__Prepare____3____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcTurretComponent__Prepare____3____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcTurretComponent__Prepare____3____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcThereminComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcThereminComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcThereminComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std__vector__cGcShipSalvageInteraction__Explode____2___StackData_TkSTLAllocatorShim__cGcShipSalvageInteraction__Explode____2___StackData_16__1______Change_array(): pass

def _cGcRefinerUnitComponent__UpdatePlayerInventory____2____lambda_1___operator__(): pass

def _cGcRefinerUnitComponent__ValidateOutputAndInputSlots____2____lambda_1___operator__(): pass

def _cGcOutpostComponent__Prepare____21____lambda_3___operator__(): pass

def _cGcOutpostComponent__RenderNGui____2____lambda_2___operator__(): pass

def _cGcOutpostComponent__RenderNGui____2____lambda_3___operator__(): pass

def __cGcOutpostComponent__RenderNGui____2____lambda_3___operator______2____lambda_1___operator__(): pass

def _cGcOutpostDockNode__Construct____17____lambda_2___operator__(): pass

def _cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcOutpostComponent__Prepare____36____lambda_4_______2____lambda_1___operator__(): pass

def cGcAISpaceshipComponent__IterateTypedComponents__cGcOutpostComponent__RenderNGui____2____lambda_3___(): pass

def std__remove_if_std___Vector_iterator_std___Vector_val_std___Simple_types_cGcOutpostDockNode__DockingShipData_________cGcOutpostDockNode__Update____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcTriggerActionComponent__IterateTypedComponentsOnNode__cGcOutpostComponent__Prepare____36____lambda_4_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcOutpostComponent__Prepare____58____lambda_6_______2____lambda_1___(): pass

def std___Func_class_void_cGcPlayerCommon______Reset__cGcOutpostDockNode__DockingShipData__Update____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__DockingShipData__Update____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__DockingShipData__Update____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__DockingShipData__Update____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Prepare____14____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Prepare____14____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Prepare____14____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Construct____17____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Construct____17____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Construct____17____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Construct____16____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Construct____16____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcOutpostDockNode__Construct____16____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____55____lambda_5__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____55____lambda_5__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____55____lambda_5__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____21____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____21____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____21____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____11____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____11____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____11____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcOutpostComponent__Prepare____8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureFeederComponent__DispenseBait____28____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcProjectileManager__SpawnResourceBlobs____47____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureFeederComponent__DispenseBait____28____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc___cGcOutpostComponent__RenderNGui____2____lambda_3___operator______2____lambda_1__void_cTkBehaviourTreeNode___bool____Target_type(): pass

def std___Func_impl_no_alloc___cGcOutpostComponent__RenderNGui____2____lambda_3___operator______2____lambda_1__void_cTkBehaviourTreeNode___bool____Do_call(): pass

def std___Func_impl_no_alloc___cGcOutpostComponent__RenderNGui____2____lambda_3___operator______2____lambda_1__void_cTkBehaviourTreeNode___bool____Move(): pass

def FindSubNode(): pass

def TrackAnimatedNodeAndParents(): pass

class cTkAudioComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def GetAxis(): pass

def RegisterClothComponentForNgui(): pass

def _cClothPiece__ApplyClothDeletionOnSimulation____2____lambda_1___operator__(): pass

def HeavySimulateTask(): pass

def _cGcPlayerFullBodyIKComponent__Prepare____27____lambda_1___operator__(): pass

def _cGcPlayerFullBodyIKComponent__UpdateIKBasedOnCurrentCharacterState____5____lambda_1___operator___cGcHeadConstraint_cGcPlayerCharacterIKOverrideData_cTkDynamicArray_cGcPlayerCharacterIKStateData__int_(): pass

def _cGcPlayerWeaponComponent__PrepareRemoteWeaponMaterials____2____lambda_1___operator__(): pass

def _cSpringLink__ApplyPositionalSpring____2____lambda_1___operator__(): pass

def _cSpringLink__ApplyPositionalSpring____2____lambda_2___operator__(): pass

def _cSpringLink__ApplyAngularSpring____24____lambda_2___operator__(): pass

def _cGcPlayerFullBodyIKComponent__RenderNGui____14____lambda_1___operator___cGcHeadConstraint_cGcIKConstraint_(): pass

def std___Sort_unchecked_int____cClothPiece__RecalculateMayaIndicesMapping____2____lambda_1___(): pass

def std___Sort_unchecked_int____cSpringLink__CollectTrackedNodes____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPlayerWeaponComponent__PrepareRemoteWeaponMaterials____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeaponComponent__PrepareRemoteWeaponMaterials____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerWeaponComponent__PrepareRemoteWeaponMaterials____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerFullBodyIKComponent__Prepare____27____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerFullBodyIKComponent__Prepare____27____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerFullBodyIKComponent__Prepare____27____lambda_1__bool_TkHandle____Copy(): pass

def std___Func_impl_no_alloc__FindSubNode____5____lambda_4__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__FindSubNode____5____lambda_4__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__FindSubNode____5____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__FindSubNode____5____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__FindSubNode____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__FindSubNode____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__FindSubNode____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__FindSubNode____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__FindSubNode____2____lambda_1__bool_TkHandle____Move(): pass

def std___Make_heap_unchecked_int____cClothPiece__RecalculateMayaIndicesMapping____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_int____cClothPiece__RecalculateMayaIndicesMapping____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_int___int__cClothPiece__RecalculateMayaIndicesMapping____2____lambda_1___(): pass

def std___Guess_median_unchecked_int____cSpringLink__CollectTrackedNodes____2____lambda_1___(): pass

def std___Med3_unchecked_int____cClothPiece__RecalculateMayaIndicesMapping____2____lambda_1___(): pass

def ToAngularVelocity(): pass

def Cross(): pass

def Dot(): pass

def LengthSquared(): pass

def Normalise(): pass

class cGcCharacterInterfaceComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def GetFreighterBaseData(): pass

def _cGcPlayerCharacterComponent__UpdateLookAt____8____lambda_1___operator__(): pass

def _cGcPlayerCharacterComponent__SetDeathState____2____lambda_1___operator__(): pass

def _cGcPlayerCharacterComponent__RefreshJetpackEffect____19____lambda_1___operator__(): pass

def _cGcPlayerControlComponent__UpdateInput____34____lambda_2___operator__(): pass

def _cGcTriggerActionComponent__UpdateCreatureNearbyTrigger____2____lambda_1___operator__(): pass

def _cGcTriggerActionComponent__HandleParticleTrigger____2____lambda_1___operator__(): pass

def _cGcTriggerActionComponent__HandleNodeActivationTrigger____2____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerCharacterComponent__HandleDeathState____11____lambda_1___(): pass

def cGcPlayerEffectsComponent__IterateTypedComponents__cGcPlayerCharacterComponent__Update____29____lambda_2___(): pass

def cGcPlayerEffectsComponent__IterateTypedComponents__cGcPlayerCharacterComponent__Update____47____lambda_3___(): pass

def cGcGameState__ForEachFleetManager__cGcTriggerActionComponent__HandleFrigateInteractionTrigger____7____lambda_1___(): pass

def cGcSimpleInteractionComponent__IterateTypedComponents__cGcTriggerActionComponent__HandleHarvestPlantTrigger____2____lambda_1___(): pass

def std__vector__cGcStagedJetEngine__AddJetStage____2___MaterialLookup_TkSTLAllocatorShim__cGcStagedJetEngine__AddJetStage____2___MaterialLookup_8__1______Emplace_one_at_back__cGcStagedJetEngine__AddJetStage____2___MaterialLookup_const___(): pass

def std___Destroy_range_TkSTLAllocatorShim__cGcStagedJetEngine__AddJetStage____2___MaterialLookup_8__1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode___cGcTriggerActionComponent__HandleNodeActivationTrigger____2____lambda_1___operator______56____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcTriggerActionComponent__HandleNodeActivationTrigger____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcTriggerActionComponent__HandleNodeActivationTrigger____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcTriggerActionComponent__HandleNodeActivationTrigger____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcTriggerActionComponent__HandleParticleTrigger____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcTriggerActionComponent__HandleParticleTrigger____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcTriggerActionComponent__HandleParticleTrigger____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__RefreshJetpackEffect____19____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__RefreshJetpackEffect____19____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__RefreshJetpackEffect____19____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc___cGcPlayerCharacterComponent__RefreshJetpackEffect____19____lambda_1___operator______14____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcPlayerCharacterComponent__RefreshJetpackEffect____19____lambda_1___operator______14____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcPlayerCharacterComponent__RefreshJetpackEffect____19____lambda_1___operator______14____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__UpdateRender____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__UpdateRender____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__UpdateRender____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Update____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Update____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Update____8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Prepare____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Prepare____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Prepare____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__OnAttached____2____lambda_2__void_unsigned_char_const___unsigned_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__OnAttached____2____lambda_2__void_unsigned_char_const___unsigned_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__OnAttached____2____lambda_2__void_unsigned_char_const___unsigned_char_const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__OnAttached____2____lambda_1__void_signed_char_const___signed_char_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__OnAttached____2____lambda_1__void_signed_char_const___signed_char_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCharacterComponent__OnAttached____2____lambda_1__void_signed_char_const___signed_char_const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerBackpackBars__Update____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerBackpackBars__Update____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerBackpackBars__Update____8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcFreighterBaseComponent__GenerateCivilianFreighterBase____14____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcFreighterBaseComponent__GenerateCivilianFreighterBase____14____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcFreighterBaseComponent__GenerateCivilianFreighterBase____14____lambda_1__enum_TkPollResult____Copy(): pass

def std___Func_impl_no_alloc__cGcColouriseComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcColouriseComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcColouriseComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcAbandonedFreighterComponent__GenerateFreighterDungeon____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcAbandonedFreighterComponent__GenerateFreighterDungeon____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcAbandonedFreighterComponent__GenerateFreighterDungeon____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcAbandonedFreighterComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcAbandonedFreighterComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Uninitialized_move__cGcStagedJetEngine__AddJetStage____2___MaterialLookup___TkSTLAllocatorShim__cGcStagedJetEngine__AddJetStage____2___MaterialLookup_8__1___(): pass

def TryGetLocalUserOnlineId(): pass

def IsLocalUser(): pass

def _cGcCustomiserCharacterComponent__SpawnWeapon____2____lambda_1___operator__(): pass

def _cGcResourceCustomisation__RebuildDescriptorsFromGroups____2____lambda_1___operator__(): pass

class cGcCombatEffectsComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcDamageNearby__Run____2____lambda_1___operator__(): pass

def _GcFollowRoutineGround__IsFoodNearby____2____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__cGcGetNearestPlayer__Run____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcGetNearestPlayer__Run____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcGetNearestPlayer__Run____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcDamageNearby__Run____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcDamageNearby__Run____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcDamageNearby__Run____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcCustomiserCharacterComponent__WeaponMeshRefresh____5____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcCustomiserCharacterComponent__WeaponMeshRefresh____5____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcCustomiserCharacterComponent__WeaponMeshRefresh____5____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcCustomiserCharacterComponent__SpawnWeapon____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCustomiserCharacterComponent__SpawnWeapon____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCustomiserCharacterComponent__SpawnWeapon____2____lambda_1__bool_TkHandle____Move(): pass

class cGcScannableComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcBehaviourApplyDamage__Run____8____lambda_1___operator__(): pass

def _cGcLandDolphin__Run____2____lambda_1___operator__(): pass

def _cGcLandDolphin__Run____2____lambda_2___operator__(): pass

def _cGcSnakePatrol__Run____2____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPredatorHunt__Run____34____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcScuttlerHunt__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcScuttlerHunt__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcScuttlerHunt__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcFiendHunt__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcFiendHunt__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcBehaviourApplyDamage__Run____8____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcBehaviourApplyDamage__Run____8____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcBehaviourApplyDamage__Run____8____lambda_1__void_cGcPlayerCommon______Move(): pass

def _cGcPetRoam__Run____644____lambda_2___operator__(): pass

def _cGcPetRoam__Run____666____lambda_3___operator__(): pass

def _cGcPetRoam__Run____681____lambda_4___operator__(): pass

def std___Func_class_bool_enum_ePetBehaviour____Reset__cGcPetRoam__Run____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPetRoam__Run____2____lambda_1__bool_enum_ePetBehaviour____Target_type(): pass

def std___Func_impl_no_alloc__cGcPetRoam__Run____2____lambda_1__bool_enum_ePetBehaviour____Do_call(): pass

def std___Func_impl_no_alloc__cGcPetRoam__Run____2____lambda_1__bool_enum_ePetBehaviour____Move(): pass

def std___Func_impl_no_alloc__cGcPetRoam__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPetRoam__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPetRoam__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcPetInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPetInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPetInteraction__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def _cGcRobotBehaviourTree__Construct____2____lambda_1___operator__(): pass

def ___emplace_back_PEAVcTkAttachment_____vector_VcTkAttachmentPtr__V__TkSTLAllocatorShim_VcTkAttachmentPtr___07_0_0___std__QEAA_A_T__QEAPEAVcTkAttachment___Z(): pass

def _cGcWalkerAttack__Run____2____lambda_1___operator__(): pass

def RegisterCreatureIkComponentForNgui(): pass

def _cGcCreatureFullBodyIKComponent__Prepare____2____lambda_1___operator__(): pass

def _cGcCreatureFullBodyIKComponent__Prepare____2____lambda_3___operator__(): pass

def _cGcCreatureFullBodyIKComponent__Prepare____74____lambda_4___operator__(): pass

def _cGcCreatureNavigation__UpdateHeadDamage____50____lambda_1___operator__(): pass

def _cGcCreatureGroundNavigation__Update____2____lambda_1___operator__(): pass

def _cGcCreatureGroundNavigation__DoRayCastJob____44____lambda_1___operator__(): pass

def _cGcCreatureGroundNavigation__UpdatePostPhysics____35____lambda_2___operator__(): pass

def _cGcCreatureSwarmNavigation__UpdatePostPhysics____95____lambda_1___operator__(): pass

def CreateCreatureGroundNavigation(): pass

def CreateCreatureGroundIndoorNavigation(): pass

def CreateRobotGroundNavigation(): pass

def CreateCreatureSwarmNavigation(): pass

def _cGcShootableComponent__IterateTypedComponentsOnNode__cGcRobotBehaviourTree__Construct____2____lambda_2_______2____lambda_1___operator__(): pass

def _cGcDestructableComponent__IterateTypedComponentsOnNode__cGcWalkerBehaviourTree__Construct____2____lambda_1_______2____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcWalkerGuard__Run____2____lambda_1___(): pass

def std__remove_if_std___Vector_iterator_std___Vector_val_std___Simple_types_cTkAttachmentPtr_______cGcWalkerLegArmourDestroyed__Run____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcShootableComponent__IterateTypedComponentsOnNode__cGcRobotBehaviourTree__Construct____2____lambda_2_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcDestructableComponent__IterateTypedComponentsOnNode__cGcWalkerBehaviourTree__Construct____2____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcCreatureNavigation__UpdateHeadDamage____50____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureNavigation__UpdateHeadDamage____50____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureNavigation__UpdateHeadDamage____50____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____74____lambda_4__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____74____lambda_4__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____74____lambda_4__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____2____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____2____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____2____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____7____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____7____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureFullBodyIKComponent__Prepare____7____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc___cGcCreatureFullBodyIKComponent__Prepare____2____lambda_1___operator______2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcCreatureFullBodyIKComponent__Prepare____2____lambda_1___operator______2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcCreatureFullBodyIKComponent__Prepare____2____lambda_1___operator______2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcWalkerTitanFall__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcWalkerTitanFall__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcWalkerTitanFall__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcRobotBehaviourTree__Construct____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRobotBehaviourTree__Construct____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRobotBehaviourTree__Construct____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcRobotCheckAlert__Run____11____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcRobotCheckAlert__Run____11____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcRobotCheckAlert__Run____11____lambda_1__void_cGcPlayerCommon______Move(): pass

def ___emplace_back_AEBVcTkVector3__AEA_KAEAIAEAMAEAMAEA_N___vector_UFootstepToSort_cGcAudioManager__V__TkSTLAllocatorShim_UFootstepToSort_cGcAudioManager___0BA__06___std__QEAA_A_TAEBVcTkVector3__AEA_KAEAIAEAM3AEA_N_Z(): pass

def _cGcCreaturePerception__Update____2____lambda_3___operator__(): pass

def PickHeight(): pass

def ConvertCreatureIDToAudioClass(): pass

def PlayCreatureVocal(): pass

def PlayCreatureSnore(): pass

class cGcShootableComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcCreatureEggComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcCreatureComponent__UpdateRender____22____lambda_2___operator__(): pass

def _cGcCreatureComponent__CalcRidingMatrix____5____lambda_1___operator__(): pass

def _cGcCreatureComponent__UpdateDissolveProgress____13____lambda_3___operator__(): pass

def ___emplace_back_AEAF___vector_HV__TkSTLAllocatorShim_H_03_0_0___std__QEAA_A_TAEAF_Z(): pass

def _cGcCreatureComponent__DoCreatureNetworkEventLocal____2____lambda_1___operator__(): pass

def ___emplace_back_UcGcTentacleChainElement_cGcCrystalMovementElement_cGcCreatureMovementCrystal_____vector_UcGcTentacleChainElement_cGcCrystalMovementElement_cGcCreatureMovementCrystal__V__TkSTLAllocatorShim_UcGcTentacleChainElement_cGcCrystalMovementElement_cGcCreatureMovementCrystal___0BA__0_0___std__QEAA_A_T__QEAUcGcTentacleChainElement_cGcCrystalMovementElement_cGcCreatureMovementCrystal___Z(): pass

def cGcCreatureComponent__IterateTypedComponents__cGcCreaturePerception__Update____9____lambda_1___(): pass

def cGcCreatureAttractorComponent__IterateTypedComponents__cGcCreaturePerception__Update____9____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__Update____155____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__StartFeeding____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__Feed____5____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__NetUpdateAttitude____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__NetUpdateRole____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__NetUpdateFriendlyAttitude____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__CreateNetworkedParticleEffect____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__SetSketchVariable____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureComponent__LaunchProjectile____8____lambda_1___(): pass

def std___Func_class_void_cGcPlayerCommon______Reset__cGcCreatureComponent__Update____49____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__sAccessoryInfo__UpdateMeshRefresh____24____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__sAccessoryInfo__UpdateMeshRefresh____24____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__sAccessoryInfo__UpdateMeshRefresh____24____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__sAccessoryInfo__UpdateMeshRefresh____18____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__sAccessoryInfo__UpdateMeshRefresh____18____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__sAccessoryInfo__UpdateMeshRefresh____18____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateDissolveProgress____13____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateDissolveProgress____13____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateDissolveProgress____13____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateDissolveProgress____11____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateDissolveProgress____11____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateDissolveProgress____11____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__DoPetSetUp____3____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__DoPetSetUp____3____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__DoPetSetUp____3____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__Release____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__Release____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__Release____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__CloneAndAttachNode____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__CloneAndAttachNode____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__CloneAndAttachNode____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__Update____49____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__Update____49____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__Update____49____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateRender____22____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateRender____22____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateRender____22____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateRender____16____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateRender____16____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureComponent__UpdateRender____16____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureBaitComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureBaitComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureBaitComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__sOwnedCreatureInfo__UpdateRequestEggSenderName____11____lambda_1__void_cTkFixedString_64_char__const______Target_type(): pass

def std___Func_impl_no_alloc__sOwnedCreatureInfo__UpdateRequestEggSenderName____11____lambda_1__void_cTkFixedString_64_char__const______Do_call(): pass

def std___Func_impl_no_alloc__sOwnedCreatureInfo__UpdateRequestEggSenderName____11____lambda_1__void_cTkFixedString_64_char__const______Move(): pass

def std___Func_impl_no_alloc__sOwnedCreatureInfo__UpdateResourceLoading____27____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__sOwnedCreatureInfo__UpdateResourceLoading____27____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__sOwnedCreatureInfo__UpdateResourceLoading____27____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcCreaturePerception__Update____2____lambda_3__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcCreaturePerception__Update____2____lambda_3__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcCreaturePerception__Update____2____lambda_3__void_cGcPlayerCommon______Move(): pass

def _cGcEcosystem__SpawnSwarmAsync____2____lambda_2___operator__(): pass

class cGcFreighterBaseComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def SetMotionBlurStartThresholdPixels(): pass

def ___emplace_back_AEAVTkHandle__AEBW4eVolumeTriggerType_____vector_UActiveTriggerVolume_cGcPlayerEnvironment__V__StackAllocator_UActiveTriggerVolume_cGcPlayerEnvironment___0BA__0_0___std__QEAA_A_TAEAVTkHandle__AEBW4eVolumeTriggerType___Z(): pass

def _cGcEnvironment__UpdatePlayerEnvironmentState____35____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcCreatureSwarm__Update____43____lambda_1___(): pass

def cGcCreatureComponent__IterateTypedComponents__cGcEcosystem__CanSpawnCreatures____2____lambda_1___(): pass

def cGcGameState__ForEachFleetManager__cGcEnvironment__UpdatePlayerEnvironmentState____146____lambda_2___(): pass

def std___Func_impl_no_alloc__cGcEnvironment__ActivateInstanceCallback____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcEnvironment__ActivateInstanceCallback____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcEnvironment__ActivateInstanceCallback____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcEnvironment__Construct____2____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcEnvironment__Construct____2____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcEnvironment__Construct____2____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcEcosystem__SpawnSwarmAsync____2____lambda_2__void_cTkUserIdBase_cTkFixedString_64_char________Delete_this(): pass

def std___Func_impl_no_alloc__cGcEcosystem__SpawnSwarmAsync____2____lambda_2__void_cTkUserIdBase_cTkFixedString_64_char________Target_type(): pass

def std___Func_impl_no_alloc__cGcEcosystem__SpawnSwarmAsync____2____lambda_2__void_cTkUserIdBase_cTkFixedString_64_char________Do_call(): pass

def std___Func_impl_no_alloc__cGcEcosystem__SpawnSwarmAsync____2____lambda_2__void_cTkUserIdBase_cTkFixedString_64_char________Copy(): pass

def std___Func_impl_no_alloc__cGcEcosystem__SpawnAsync____16____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char________Delete_this(): pass

def std___Func_impl_no_alloc__cGcEcosystem__SpawnAsync____16____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char________Target_type(): pass

def std___Func_impl_no_alloc__cGcEcosystem__SpawnAsync____16____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char________Do_call(): pass

def std___Func_impl_no_alloc__cGcEcosystem__SpawnAsync____16____lambda_1__void_cTkUserIdBase_cTkFixedString_64_char________Copy(): pass

def std___Func_impl_no_alloc__cGcCreatureLegIK__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureLegIK__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureLegIK__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCreatureLegIKComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCreatureLegIKComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCreatureLegIKComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def DistanceFromProbe(): pass

def _SortProbesByDistance____2____lambda_1___operator__(): pass

def DisableDynamicNodes(): pass

def LargeRegionVoronoi(): pass

def CanApplyTerrainEdit(): pass

def EditTerrain(): pass

def _EditTerrain____11____lambda_1___operator__(): pass

def _EditTerrain____2____lambda_2___operator__(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__EditTerrain____11____lambda_1___(): pass

def std__vector__SortProbesByDistance____2___sProbeNodeMap_TkSTLAllocatorShim__SortProbesByDistance____2___sProbeNodeMap_4__1______Resize_std___Value_init_tag_(): pass

def std___Destroy_range_TkSTLAllocatorShim__SortProbesByDistance____2___sProbeNodeMap_4__1___(): pass

def std___Sort_unchecked__SortProbesByDistance____2___sProbeNodeMap____SortProbesByDistance____2____lambda_1___(): pass

def std___Sort_unchecked_TkHandle____ClosestProbe____2____lambda_1___(): pass

def std___Uninitialized_value_construct_n_TkSTLAllocatorShim__SortProbesByDistance____2___sProbeNodeMap_4__1___(): pass

def std___Func_impl_no_alloc__cGcReflectionProbesManager__SwitchEnvironment____8____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcReflectionProbesManager__SwitchEnvironment____8____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcReflectionProbesManager__SwitchEnvironment____8____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcReflectionProbesManager__SwitchEnvironment____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcReflectionProbesManager__SwitchEnvironment____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcReflectionProbesManager__SwitchEnvironment____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcReflectionProbesManager__UpdateRender____7____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcReflectionProbesManager__UpdateRender____7____lambda_1__bool_TkHandle____Move(): pass

def std___Make_heap_unchecked__SortProbesByDistance____2___sProbeNodeMap____SortProbesByDistance____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked__SortProbesByDistance____2___sProbeNodeMap____SortProbesByDistance____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_TkHandle____ClosestProbe____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index__SortProbesByDistance____2___sProbeNodeMap____SortProbesByDistance____2___sProbeNodeMap__SortProbesByDistance____2____lambda_1___(): pass

def std__iter_swap__SortProbesByDistance____2___sProbeNodeMap____SortProbesByDistance____2___sProbeNodeMap___(): pass

def std___Pop_heap_hole_by_index_TkHandle___TkHandle__ClosestProbe____2____lambda_1___(): pass

def std___Med3_unchecked__SortProbesByDistance____2___sProbeNodeMap____SortProbesByDistance____2____lambda_1___(): pass

def std___Med3_unchecked_TkHandle____ClosestProbe____2____lambda_1___(): pass

def sprintf(): pass

def IsPlayerInFastMovingShip(): pass

def GetSmoothTransitionsForGeneration(): pass

def _cGcTerrainTextureData__CreateTextures____75____lambda_1___operator__(): pass

def ApplyFeedbackTex(): pass

def ClearFeedbackTex(): pass

def ClearStatusTex(): pass

def UberPrimitiveFunction(): pass

def SuperPrimitiveFunction(): pass

def BlendSuperPrimitiveData(): pass

def GetSuperPrimitiveData(): pass

def BlendSuperFormulaData(): pass

def GetSuperFormulaData(): pass

def SphereDensityFunction(): pass

def SuperShape(): pass

def SuperFunction3D(): pass

def Generate_InitPositions(): pass

def Generate_NormalizePositions(): pass

def Generate_BuildVoxelColumn_2D(): pass

def Generate_BuildVoxelColumn_Grid(): pass

def Generate_BuildVoxelColumn_Buildings(): pass

def Generate_BuildVoxelColumn_FillVoxels(): pass

def Generate_BuildVoxels(): pass

def Generate_MaxHeights(): pass

def Generate_Edits(): pass

def std___Sort_unchecked_cGcTargetTrack____cGcShipAIFreighterTargeting__Run____96____lambda_1___(): pass

def std___Sort_unchecked_unsigned_short____cGcTerrainRegionMapOcttree__EndRoots____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cGcTargetTrack____cGcShipAIFreighterTargeting__Run____96____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_unsigned_short____cGcTerrainRegionMapOcttree__EndRoots____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cGcTargetTrack___cGcTargetTrack__cGcShipAIFreighterTargeting__Run____96____lambda_1___(): pass

def std___Guess_median_unchecked_cGcTargetTrack____cGcShipAIFreighterTargeting__Run____96____lambda_1___(): pass

def std___Pop_heap_hole_by_index_unsigned_short___unsigned_short__cGcTerrainRegionMapOcttree__EndRoots____2____lambda_1___(): pass

def std___Med3_unchecked_unsigned_short____cGcTerrainRegionMapOcttree__EndRoots____2____lambda_1___(): pass

def _cGcRegionDecoratorBuildings__SetupBuildingComponents____5____lambda_1___operator__(): pass

def GetNumPositionsForTriangles(): pass

def GetPositionsForTriangles(): pass

def BuildObjectTypePerPositionList(): pass

def AddToGlobalCounterAndGetLimitedRegionCount(): pass

def SubtractFromGlobalCounter(): pass

def FindClosestRayContactPoint(): pass

def _cGcRegionDecoratorObjects__AddInstanceNodes____2____lambda_1___operator__(): pass

def cGcAISpaceshipComponent__IterateTypedComponents__cGcRegionDecoratorBuildings__PollToUpgradeLandmarks____71____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcRegionDecoratorObjects__AddNodes____7____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__RemoveInstancesWithinArea____41____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__RemoveInstancesWithinArea____41____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__RemoveInstancesWithinArea____41____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__RemoveInstancesWithinArea____13____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__RemoveInstancesWithinArea____13____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__RemoveInstancesWithinArea____13____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__AddInstanceNodes____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__AddInstanceNodes____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorObjects__AddInstanceNodes____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorBuildings__SpawnProceduralShipwreck____26____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorBuildings__SpawnProceduralShipwreck____26____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcRegionDecoratorBuildings__SpawnProceduralShipwreck____26____lambda_1__bool_TkHandle____Move(): pass

class cGcAtmosphereEntryComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcBinoculars__UpdateTarget____2____lambda_1___operator__(): pass

def _cGcBinoculars__UpdateScanBarProgress____2____lambda_1___operator__(): pass

def ___emplace_back_AEAY0BG___CBDAEAV__cTkFixedString__0BAA_D_____vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BG___CBDAEAV__cTkFixedString__0BAA_D___Z(): pass

def ___emplace_back_AEAY0P___CBDPEBD_N___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0P___CBD__QEAPEBD__QEA_N_Z(): pass

def ___emplace_back_AEAY04__CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY04__CBD__QEAPEBD_Z(): pass

def ___emplace_back_AEAY0BC___CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BC___CBD__QEAPEBD_Z(): pass

def ___emplace_back_AEAY0BK___CBDPEAD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BK___CBD__QEAPEAD_Z(): pass

def _cGcBinoculars__PopulateDiscoveryInfo____197____lambda_2___operator__(): pass

def ___emplace_back_AEAY0BA___CBDPEBD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BA___CBD__QEAPEBD_Z(): pass

def ___emplace_back_AEAY0BB___CBDAEAV__cTkFixedString__0IA_D_____vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BB___CBDAEAV__cTkFixedString__0IA_D___Z(): pass

def ___emplace_back_AEAY0BB___CBDAEAV__cTkFixedString__0CA_D_____vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BB___CBDAEAV__cTkFixedString__0CA_D___Z(): pass

def ___emplace_back_AEAY0BG___CBDPEAD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BG___CBD__QEAPEAD_Z(): pass

def ___emplace_back_AEAY0BF___CBDPEAD___vector_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver__V__TkSTLAllocatorShim_UDiscoveryInfoValue_DiscoveryInfo_DiscoveryResolver___07_0_0___std__QEAA_A_TAEAY0BF___CBD__QEAPEAD_Z(): pass

def _cGcBinoculars__SetMarker____2____lambda_1___operator__(): pass

def _cGcMarkerPoint__Update____34____lambda_1___operator__(): pass

def _cGcMarkerPoint__GetHorizonPosition____2____lambda_1___operator__(): pass

def _cGcScanEvent__CollectAllDestructables____8____lambda_1___operator__(): pass

def cTkAttachment__IterateAttachments__cGcBinoculars__IsScannableNode____2____lambda_1___(): pass

def cGcGameState__ForEachFleetManager__cGcBinoculars__PopulateDiscoveryInfo____144____lambda_1___(): pass

def cGcGameState__ForEachFreighterOwnership__cGcBinoculars__PopulateDiscoveryInfo____197____lambda_2___(): pass

def cTkAttachment__IterateAttachments__cGcBinoculars__PopulateDiscoveryInfo____393____lambda_3___(): pass

def cTkAttachment__IterateAttachments__cGcScanEvent__UpdateSpaceStationLocation____17____lambda_1___(): pass

def _cGcInteractionComponent__IterateTypedComponentsOnNode__cGcScanEvent__MarkCompleted____27____lambda_1_______2____lambda_1___operator__(): pass

def _cGcInteractionComponent__IterateTypedComponentsOnNode__cGcScanEvent__CollectAllInteractions____8____lambda_1_______2____lambda_1___operator__(): pass

def _cGcMaintenanceComponent__IterateTypedComponentsOnNode__cGcScanEvent__CollectAllMaintenanceComponents____8____lambda_1_______2____lambda_1___operator__(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcInteractionComponent__IterateTypedComponentsOnNode__cGcScanEvent__MarkCompleted____27____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcPortalComponent__IterateTypedComponentsOnNode__cGcScanEvent__MarkCompleted____27____lambda_2_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcInteractionComponent__IterateTypedComponentsOnNode__cGcScanEvent__CollectAllInteractions____8____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcDestructableComponent__IterateTypedComponentsOnNode__cGcScanEvent__CollectAllDestructables____8____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcMaintenanceComponent__IterateTypedComponentsOnNode__cGcScanEvent__CollectAllMaintenanceComponents____8____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcScanEvent__Construct____45____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcScanEvent__Construct____45____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcScanEvent__Construct____45____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcMarkerPoint__Update____34____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcMarkerPoint__Update____34____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcMarkerPoint__Update____34____lambda_1__bool_TkHandle____Move(): pass

def _cGcSpaceshipComponent__Update____116____lambda_2____lambda_2_(): pass

def _cGcScanManager__UpdateConstantMarkers____462____lambda_3___operator__(): pass

def _cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup__CalcAvgPos(): pass

def _cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup__AddScannable(): pass

def _cGcScanner__UpdateScannedAsteroids____14____lambda_3___operator__(): pass

def _cGcScanner__UpdateScannedAsteroids____14____lambda_4___operator__(): pass

def std__vector__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_StackAllocator__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_256__1______vector__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_StackAllocator__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_256__1___(): pass

def std__vector__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_StackAllocator__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_256__1_____reserve(): pass

def std__vector__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_StackAllocator__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_256__1______Change_array(): pass

def cGcGameState__ForEachFreighterOwnership__cGcScanManager__UpdateConstantMarkers____2____lambda_1___(): pass

def cTkAttachment__IterateAttachments__cGcScanManager__UpdateConstantMarkers____292____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcScanManager__UpdateConstantMarkers____462____lambda_3___(): pass

def std___Destroy_range_StackAllocator__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_256__1___(): pass

def std___Uninitialized_move__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup___StackAllocator__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_256__1___(): pass

def std___Sort_unchecked_cGcMarkerPoint_const______cGcScanManager__UpdateScannableMarkers____68____lambda_1___(): pass

def std___Sort_unchecked_cGcScanner__FuelAsteroidInfo____cGcScanner__UpdateScannedAsteroids____14____lambda_4___(): pass

def std__vector__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_StackAllocator__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_256__1______Emplace_reallocate__cGcScanManager__UpdateScannableMarkers____2___cGcScannableMarkerGroup_(): pass

def std___Func_impl_no_alloc__cGcScanEventManager__SendSyncScanEventRequest____2____lambda_1__void_enum_eGcNetworkTransactionStatus_cGcScanEventSave______Delete_this(): pass

def std___Func_impl_no_alloc__cGcScanEventManager__SendSyncScanEventRequest____2____lambda_1__void_enum_eGcNetworkTransactionStatus_cGcScanEventSave______Target_type(): pass

def std___Func_impl_no_alloc__cGcScanEventManager__SendSyncScanEventRequest____2____lambda_1__void_enum_eGcNetworkTransactionStatus_cGcScanEventSave______Do_call(): pass

def std___Func_impl_no_alloc__cGcScanEventManager__SendSyncScanEventRequest____2____lambda_1__void_enum_eGcNetworkTransactionStatus_cGcScanEventSave______Copy(): pass

def std___Partition_by_median_guess_unchecked_cGcMarkerPoint_const______cGcScanManager__UpdateScannableMarkers____68____lambda_1___(): pass

def std___Insertion_sort_unchecked_cGcScanner__FuelAsteroidInfo____cGcScanner__UpdateScannedAsteroids____14____lambda_4___(): pass

def std___Make_heap_unchecked_cGcScanner__FuelAsteroidInfo____cGcScanner__UpdateScannedAsteroids____14____lambda_4___(): pass

def std___Partition_by_median_guess_unchecked_cGcScanner__FuelAsteroidInfo____cGcScanner__UpdateScannedAsteroids____14____lambda_4___(): pass

def std___Pop_heap_hole_by_index_cGcMarkerPoint_const_____cGcMarkerPoint_const____cGcScanManager__UpdateScannableMarkers____68____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cGcScanner__FuelAsteroidInfo___cGcScanner__FuelAsteroidInfo__cGcScanner__UpdateScannedAsteroids____14____lambda_4___(): pass

def std___Med3_unchecked_cGcMarkerPoint_const______cGcScanManager__UpdateScannableMarkers____68____lambda_1___(): pass

def std___Push_heap_by_index_cGcScanner__FuelAsteroidInfo___cGcScanner__FuelAsteroidInfo__cGcScanner__UpdateScannedAsteroids____14____lambda_4___(): pass

def std___Med3_unchecked_cGcScanner__FuelAsteroidInfo____cGcScanner__UpdateScannedAsteroids____14____lambda_4___(): pass

def _cGcSky__SpawnWeatherHazards____2____lambda_1___operator__(): pass

def __cGcSky__SpawnWeatherHazards____2____lambda_1___operator______13____lambda_1___operator__(): pass

def _cGcSky__UpdateWeatherEffects____30____lambda_1___operator__(): pass

def _cGcWeatherEffectUpdate__UpdateMeteorHazard____2____lambda_1___operator__(): pass

def _cGcWeatherEffectUpdate__UpdateMeteorHazard____8____lambda_2___operator__(): pass

def _cGcWeatherEffectUpdate__UpdateMeteorHazard____12____lambda_3___operator__(): pass

def _cGcWeatherEffectUpdate__UpdateLightningHazard____2____lambda_1___operator__(): pass

def _cGcWeatherEffectUpdate__UpdateLightningHazard____8____lambda_2___operator__(): pass

def _cGcWeatherEffectUpdate__UpdateLightningHazard____12____lambda_3___operator__(): pass

def _cGcWeatherEffectUpdate__UpdateTornadoHazard____2____lambda_1___operator___int_(): pass

def std___Func_impl_no_alloc__cGcWeatherHazardGiftManager__RemoveFurthestGift____6____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcWeatherHazardGiftManager__Update____2____lambda_2__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcWeatherHazardGiftManager__RemoveFurthestGift____6____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcWeatherHazardGiftManager__Update____2____lambda_2__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcWeatherHazardGiftManager__Update____2____lambda_2__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateLightningHazard____12____lambda_3__void_int_cTkVector3_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateLightningHazard____12____lambda_3__void_int_cTkVector3_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateLightningHazard____12____lambda_3__void_int_cTkVector3_const______Copy(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateLightningHazard____8____lambda_2__void_int_cTkVector3_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateLightningHazard____8____lambda_2__void_int_cTkVector3_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateLightningHazard____8____lambda_2__void_int_cTkVector3_const______Move(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateMeteorHazard____12____lambda_3__void_int_cTkVector3_const___cTkVector3_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateMeteorHazard____12____lambda_3__void_int_cTkVector3_const___cTkVector3_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateMeteorHazard____12____lambda_3__void_int_cTkVector3_const___cTkVector3_const______Copy(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateMeteorHazard____8____lambda_2__void_int_cTkVector3_const___cTkVector3_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateMeteorHazard____8____lambda_2__void_int_cTkVector3_const___cTkVector3_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__UpdateMeteorHazard____8____lambda_2__void_int_cTkVector3_const___cTkVector3_const______Move(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__GenerateResourceAsync____8____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcWeatherEffectUpdate__GenerateResourceAsync____8____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc___cGcSky__UpdateWeatherEffects____30____lambda_1___operator______41____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc___cGcSky__UpdateWeatherEffects____30____lambda_1___operator______41____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSky__SpawnWeatherHazards____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcSky__SpawnWeatherHazards____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcSky__SpawnWeatherHazards____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcSky__SpawnWeatherEffects____19____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcSky__SpawnWeatherEffects____19____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcSky__SpawnWeatherEffects____19____lambda_1__void_cGcPlayerCommon______Move(): pass

def LookupCLUT(): pass

def _cGcGalaxyMap__Data__Update____94____lambda_1___operator__(): pass

def _cGcGalaxyMap__Data__Update____398____lambda_3___operator__(): pass

def _cGcGalaxyMap__Data__UpdateCommonFsFxUniforms____11____lambda_1___operator___cTkFixedArray_cTkColour_10___(): pass

def _cGcGalaxyMap__Data__DoInputHandlingPushToNavigate____2____lambda_1___operator__(): pass

def _cGcGalaxyMap__Data__MoveAlongCurrentNavPath____2____lambda_1___operator__(): pass

def ___emplace_back_AEB_K___vector__KV__TkSTLAllocatorShim__K_07_0BH____std__QEAA_A_TAEB_K_Z(): pass

def cGcNetworkPlayerManager__ForEachFireteamRemotePlayer__cGcGalaxyMap__Data__Update____94____lambda_2___(): pass

def std___Func_impl_no_alloc__cGcGalaxyMap__Data__TriggerSelectionTransition____17____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcGalaxyMap__Data__TriggerSelectionTransition____17____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcGalaxyMap__Data__TriggerSelectionTransition____17____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcGalaxyMap__Data__TransitionBackToGame____2____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcGalaxyMap__Data__TransitionBackToGame____2____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcGalaxyMap__Data__TransitionBackToGame____2____lambda_1__void____Move(): pass

class GcGalaxyFx(ctypes.Structure):
    def nvgMultiEdgeRadial(): pass

    def nvgPulse(): pass


def _cGcGalaxyVoxelGenerator__Populate____2____lambda_1___operator__(): pass

def _cGcGalaxyVoxelGenerator__Populate____11____lambda_2___operator__(): pass

def AirNavPhysProbeJob(): pass

def _cGcNavMeshManager__PlotPathToPoint____2____lambda_5___operator__(): pass

def ___emplace_back_AEA_JW4eOpType_cGcNavMeshOp_cGcNavMeshUpdater_____deque_UcGcNavMeshOp_cGcNavMeshUpdater__V__TkSTLAllocatorShim_UcGcNavMeshOp_cGcNavMeshUpdater___07_0BE____std__QEAA_A_TAEA_J__QEAW4eOpType_cGcNavMeshOp_cGcNavMeshUpdater___Z(): pass

def _cGcRegionKnowledge__DecideInternalPassability____2____lambda_1___operator__(): pass

def _cGcRegionKnowledge__PlotPathToPoint____2____lambda_5___operator__(): pass

def std__vector__cGcRegionKnowledge__PlotPathToPoint____2___FoundChunkInfo_StackAllocator__cGcRegionKnowledge__PlotPathToPoint____2___FoundChunkInfo_1024__1_____reserve(): pass

def std__vector__cGcNavMeshManager__PlotPathToPoint____2___FoundChunkInfo_StackAllocator__cGcNavMeshManager__PlotPathToPoint____2___FoundChunkInfo_1024__1_____reserve(): pass

def std__vector__cGcRegionKnowledge__PlotPathToPoint____2___FoundChunkInfo_StackAllocator__cGcRegionKnowledge__PlotPathToPoint____2___FoundChunkInfo_1024__1______Change_array(): pass

def std__vector__cGcNavMeshManager__PlotPathToPoint____2___FoundChunkInfo_StackAllocator__cGcNavMeshManager__PlotPathToPoint____2___FoundChunkInfo_1024__1______Change_array(): pass

def std__vector__cGcRegionKnowledge__PlotPathToPoint____2___FoundChunkInfo_StackAllocator__cGcRegionKnowledge__PlotPathToPoint____2___FoundChunkInfo_1024__1______Emplace_one_at_back__cGcRegionKnowledge__PlotPathToPoint____2___FoundChunkInfo_const___(): pass

def std__vector__cGcNavMeshManager__PlotPathToPoint____2___FoundChunkInfo_StackAllocator__cGcNavMeshManager__PlotPathToPoint____2___FoundChunkInfo_1024__1______Emplace_reallocate__cGcNavMeshManager__PlotPathToPoint____2___FoundChunkInfo_const___(): pass

def GetAnimationId(): pass

def _cGcPlayerWeapon__FinishPrepare____2____lambda_1___operator__(): pass

def _cGcPlayerWeapon__PostPhysicsUpdate____35____lambda_1___operator__(): pass

def _cGcPlayerWeapon__PostPhysicsUpdate____35____lambda_2___operator__(): pass

def _cGcPlayerWeapon__RenderScreen____9____lambda_1____lambda_invoker_cdecl_(): pass

def std__function_bool___cdecl_TkHandle____function_bool___cdecl_TkHandle____cGcPlayerWeapon__Update____123____lambda_1__0_(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcCustomInventoryComponent__IterateTypedComponentsOnNode__cGcPlayerWeapon__FinishPrepare____2____lambda_2_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__UpdateMeshRefresh____5____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__UpdateMeshRefresh____5____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__UpdateMeshRefresh____5____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__Update____123____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__Update____123____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__FinishPrepare____2____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____23____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__FinishPrepare____2____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__FinishPrepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__FinishPrepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__FinishPrepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____22____lambda_5__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____22____lambda_5__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____18____lambda_4__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____18____lambda_4__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____12____lambda_3__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____12____lambda_3__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____12____lambda_3__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____8____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____8____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____8____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerWeapon__AsyncPrepare____8____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcParticleManager__IsActive____11____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcParticleManager__IsActive____11____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcParticleManager__IsActive____11____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcParticleManager__EndUpdate____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcParticleManager__EndUpdate____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcParticleManager__EndUpdate____8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcParticleManager__BeginUpdate____16____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcParticleManager__BeginUpdate____16____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcParticleManager__BeginUpdate____16____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcParticleManager__LoadData____13____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcParticleManager__LoadData____13____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcParticleManager__LoadData____7____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcParticleManager__LoadData____7____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcExplosion__IsAlive____10____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcExplosion__IsAlive____10____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcExplosion__IsAlive____10____lambda_1__bool_TkHandle____Move(): pass

class cGcDestructableComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def TkWorldMatrixToHkTransform(): pass

def _cGcPlayerStick__RenderNGui____3____lambda_1___operator__(): pass

def _cGcPlayerStick__RenderNGui____3____lambda_2___operator__(): pass

class cGcChairComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcPlacementArc__Render____2____lambda_1___operator__(): pass

class cGcCreatureAttractorComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcPlayer__Construct____2____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcPlayer__Subscribe____2____lambda_1___operator__(): pass

def _cGcPlayer__Subscribe____2____lambda_2___operator__(): pass

def _cGcPlayer__Update____264____lambda_1___operator__(): pass

def _cGcPlayer__UpdateControl____125____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayer__UpdateControl____506____lambda_2___(): pass

def cGcNPCComponent__IterateTypedComponents__cGcPlayer__DoEmote____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPlayer__ReleaseGrabbedBy____10____lambda_2__bool_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__ReleaseGrabbedBy____10____lambda_2__bool_TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayer__ReleaseGrabbedBy____10____lambda_2__bool_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__ReleaseGrabbedBy____10____lambda_1__void_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__ReleaseGrabbedBy____10____lambda_1__void_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____36____lambda_4__bool_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____36____lambda_4__bool_TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____36____lambda_4__bool_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____29____lambda_3__void_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____29____lambda_3__void_TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____29____lambda_3__void_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc___cGcPlayer__DoEmote____29____lambda_3___operator______2____lambda_1__bool_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc___cGcPlayer__DoEmote____29____lambda_3___operator______2____lambda_1__bool_TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc___cGcPlayer__DoEmote____29____lambda_3___operator______2____lambda_1__bool_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____2____lambda_2__void_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____2____lambda_2__void_TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayer__DoEmote____2____lambda_2__void_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__SetGrabbedBy____5____lambda_2__bool_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__SetGrabbedBy____5____lambda_2__bool_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__SetGrabbedBy____5____lambda_1__void_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__SetGrabbedBy____5____lambda_1__void_TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayer__SetGrabbedBy____5____lambda_1__void_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__RideCreature____11____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__RideCreature____11____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__Update____264____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__Update____264____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayer__Update____264____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayer__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayer__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayer__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayer__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcCharacterSuitAudio__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcCharacterSuitAudio__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcCharacterSuitAudio__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcCharacterSit__Stand____5____lambda_1__void_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcCharacterSit__Stand____5____lambda_1__void_TkID_128__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcCharacterSit__Stand____5____lambda_1__void_TkID_128__const______Move(): pass

def std___Func_impl_no_alloc__cGcCharacterSit__Sit____8____lambda_1__void_TkID_128__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcCharacterSit__Sit____8____lambda_1__void_TkID_128__const______Move(): pass

class cGcNPCPlacementComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcExperienceSpawn__IsInteresting____2____lambda_1___operator__(): pass

def _cGcExperienceSpawn__IsInteresting____2____lambda_2___operator__(): pass

def _cGcPlayerExperienceDirector__Construct____2____lambda_1____lambda_invoker_cdecl_(): pass

def _cGcPlayerExperienceDirector__Construct____2____lambda_2____lambda_invoker_cdecl_(): pass

def _cGcPlayerExperienceDirector__Construct____19____lambda_3___operator__(): pass

def _cGcPlayerExperienceDirector__StartPulseEncounter____2____lambda_1___operator__(): pass

def _cGcPlayerExperienceDirector__UpdatePirates____267____lambda_2___operator__(): pass

def _cGcPlayerExperienceDirector__RenderNGuiMap____2____lambda_2___operator__(): pass

def _cGcPlayerExperienceDirector__RenderNGuiMap____2____lambda_4___operator__(): pass

def ___emplace_back_AEAY0BA___CBD___vector_PEBDV__TkSTLAllocatorShim_PEBD_07_0_0___std__QEAA_A_TAEAY0BA___CBD_Z(): pass

def ___emplace_back_AEAY0BA_D___vector_PEBDV__TkSTLAllocatorShim_PEBD_07_0_0___std__QEAA_A_TAEAY0BA_D_Z(): pass

def _cGcPlayerExperienceDirector__UpdateFreighterExperience____2____lambda_1___operator__(): pass

def _cGcPlayerExperienceDirector__SpawnInterceptFlyby____2____lambda_1___operator__(): pass

def _cGcPlayerExperienceDirector__UpdateRender____5____lambda_1___operator__(): pass

def _cGcPlayerExperienceDirector__GetActivePoliceAndHostileTraderCount____2____lambda_1___operator__(): pass

def _cGcPlayerExperienceDirector__SpawnFishFiends____14____lambda_1___operator__(): pass

def cGcAISpaceshipComponent__IterateTypedComponents__cGcPlayerExperienceDirector__AllowedToSpawnTraders____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerExperienceDirector__UpdatePirates____58____lambda_1___(): pass

def cGcSpaceshipComponent__IterateTypedComponents__cGcPlayerExperienceDirector__ValidFreighterSpawn____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerExperienceDirector__CheckForSpaceBattle____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerExperienceDirector__HasActiveSpaceBattleInCurrentSystem____2____lambda_1___(): pass

def cGcAISpaceshipComponent__IterateTypedComponents__cGcPlayerExperienceDirector__GetActiveSpacePoliceCount____2____lambda_1___(): pass

def cGcAISpaceshipComponent__IterateTypedComponents__cGcPlayerExperienceDirector__GetActiveHostileTraderCount____2____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerExperienceDirector__SpawnFiends____7____lambda_1___(): pass

def std___Sort_unchecked__cGcPlayerExperienceDirector__Update____257___SpawnScore____cGcPlayerExperienceDirector__Update____257____lambda_3___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcOutpostComponent__IterateTypedComponentsOnNode__cGcPlayerExperienceDirector__UpdatePirates____267____lambda_2_______2____lambda_1___(): pass

def std___Func_class_bool_TkHandle____Reset__cGcPlayerExperienceDirector__UpdatePulseEncounters____170____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__SpawnFishFiends____14____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__SpawnFishFiends____14____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__SpawnFishFiends____14____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__UpdateFreighterExperience____2____lambda_1__bool_sSketchEvent_const______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__UpdateFreighterExperience____2____lambda_1__bool_sSketchEvent_const______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__UpdateFreighterExperience____2____lambda_1__bool_sSketchEvent_const______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__PrepareBackgroundSpaceEncounter____11____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__PrepareBackgroundSpaceEncounter____11____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__PrepareEncounter____51____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__PrepareEncounter____51____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__UpdatePulseEncounters____170____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__UpdatePulseEncounters____170____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__UpdatePulseEncounters____170____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__Update____14____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__Update____14____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerExperienceDirector__Update____14____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcPlayerEmoteProp__Update____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerEmoteProp__Update____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerEmoteProp__Update____8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerCommon__GetPotentialCriminal____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerCommon__GetPotentialCriminal____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerCommon__GetPotentialCriminal____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Partition_by_median_guess_unchecked__cGcPlayerExperienceDirector__Update____257___SpawnScore____cGcPlayerExperienceDirector__Update____257____lambda_3___(): pass

def std___Guess_median_unchecked__cGcPlayerExperienceDirector__Update____257___SpawnScore____cGcPlayerExperienceDirector__Update____257____lambda_3___(): pass

def _cGcPlayerHazard__RenderNGui____2____lambda_1___operator__(): pass

def _cGcPlayerHazard__UpdateShielding____2____lambda_1___operator__(): pass

class cGcGrabPlayerComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcTargetHelper__Update____76____lambda_1___operator__(): pass

def _cGcPlayerRespawn__DoPlayerRespawn____36____lambda_1___operator__(): pass

def _cGcPlayerRespawn__GetPositionForTeleportEndpointInAtlas____2____lambda_1___operator__(): pass

def cGcCreatureComponent__IterateTypedComponents__cGcTargetHelper__Update____76____lambda_1___(): pass

def cGcOutpostComponent__IterateTypedComponents__cGcPlayerRespawn__DoPlayerRespawn____2____lambda_2___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcPlayerRespawn__SpawnAndPositionShip____5____lambda_1___(): pass

def _cTkAnimationComponent__IterateTypedComponentsOnNode__cGcPlayerRespawn__SpawnAndPositionShip____17____lambda_2_______2____lambda_1___operator__(): pass

def std___Sort_unchecked_std__pair_cGcNavGraphQuadTree__TreeNode___float_____cGcNavGraphQuadTree__TreeNode__FindNearestNode____17____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkAnimationComponent__IterateTypedComponentsOnNode__cGcPlayerRespawn__SpawnAndPositionShip____17____lambda_2_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPlayerRespawn__GetPositionForTeleportEndpointInAtlas____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerRespawn__GetPositionForTeleportEndpointInAtlas____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerRespawn__GetPositionForTeleportEndpointInAtlas____2____lambda_1__bool_TkHandle____Move(): pass

def std___Make_heap_unchecked_std__pair_cGcNavGraphQuadTree__TreeNode___float_____cGcNavGraphQuadTree__TreeNode__FindNearestNode____17____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_std__pair_cGcNavGraphQuadTree__TreeNode___float_____cGcNavGraphQuadTree__TreeNode__FindNearestNode____17____lambda_1___(): pass

def std___Pop_heap_hole_by_index_std__pair_cGcNavGraphQuadTree__TreeNode___float____std__pair_cGcNavGraphQuadTree__TreeNode___float___cGcNavGraphQuadTree__TreeNode__FindNearestNode____17____lambda_1___(): pass

def std___Med3_unchecked_std__pair_cGcNavGraphQuadTree__TreeNode___float_____cGcNavGraphQuadTree__TreeNode__FindNearestNode____17____lambda_1___(): pass

def ___emplace_back_AEBUMenuAction_____vector_UMenuAction__V__TkSTLAllocatorShim_UMenuAction___0BA__0_0___std__QEAA_A_TAEBUMenuAction___Z(): pass

def cGcInteractionComponent__IterateTypedComponents__cGcPlayerThirdPerson__UpdateLookAtPoint____22____lambda_1___(): pass

def cGcCreatureComponent__IterateTypedComponents__cGcPlayerThirdPerson__UpdateLookAtPoint____22____lambda_3___(): pass

def cGcAISpaceshipComponent__IterateTypedComponents__cGcPlayerThirdPerson__UpdateLookAtPoint____22____lambda_4___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcPlayerWanted__NotifyWanted____10____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__GenerateResource____3____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__GenerateResource____3____lambda_2__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__GenerateResource____3____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__GenerateResource____3____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__GenerateResource____3____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__GenerateResource____3____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc___cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____42____lambda_4___operator______2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____42____lambda_4___operator______2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____23____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____23____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____35____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____35____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____35____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____35____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____35____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__RefreshThirdPersonMeshVisibility____35____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__SpawnThirdPersonModel____2____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__SpawnThirdPersonModel____2____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__SpawnThirdPersonModel____2____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__SpawnThirdPersonModel____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__SpawnThirdPersonModel____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcPlayerThirdPerson__SpawnThirdPersonModel____2____lambda_1__bool_TkHandle____Move(): pass

def _cGcTerrainEditMenu__UpdateControlsString____2____lambda_1___operator__(): pass

class cGcLootComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcAntagonistComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcShootableComponent__Prepare____8____lambda_1___operator__(): pass

def _cGcTerrainEditorBeam__Update____2____lambda_1___operator__(): pass

def cGcDestructableComponent__IterateTypedComponentsOnNode__cGcDestructableComponent__Destroy____146____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcProjectileManager__Impact____20____lambda_1___(): pass

def _cGcProjectileManager__OnRemoteHit____2____lambda_1___operator___cTkNetReplicatedEntityComponent_(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcProjectileManager__SpawnResourceBlobs____37____lambda_1___(): pass

def std___Func_class_bool_TkHandle____Reset__cGcDestructableComponent__Destroy____55____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcDestructableComponent__IterateTypedComponentsOnNode__cGcDestructableComponent__Destroy____146____lambda_2_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcShootableComponent__IterateTypedComponentsOnNode__cGcShootableComponent__Prepare____8____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcShootableComponent__IterateTypedComponentsOnNode__cGcShootableComponent__OnHit____220____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcShootableComponent__Update____8____lambda_1__void_unsigned_char_const___unsigned_short____Target_type(): pass

def std___Func_impl_no_alloc__cGcShootableComponent__Update____8____lambda_1__void_unsigned_char_const___unsigned_short____Do_call(): pass

def std___Func_impl_no_alloc__cGcShootableComponent__Update____8____lambda_1__void_unsigned_char_const___unsigned_short____Move(): pass

def std___Func_impl_no_alloc__cGcProjectileManager__SpawnResourceBlobs____47____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcProjectileManager__SpawnResourceBlobs____47____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcProjectile__Prepare____9____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcProjectile__Prepare____9____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcProjectile__Prepare____9____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__ComponentLoadInstancercGcDestructableComponent__Load____5____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__ComponentLoadInstancercGcDestructableComponent__Load____5____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcDestructableComponent__PlayDestroyEffects____13____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcDestructableComponent__PlayDestroyEffects____13____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcDestructableComponent__PlayDestroyEffects____13____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcDestructableComponent__Destroy____55____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcDestructableComponent__Destroy____55____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcDestructableComponent__Destroy____55____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcMuzzleFlash__SetColour____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcMuzzleFlash__SetColour____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcMuzzleFlash__SetColour____2____lambda_1__bool_TkHandle____Move(): pass

def _cGcDroneInvestigate__Subscribe____2____lambda_1___operator__(): pass

def _cGcDronePatrol__Subscribe____2____lambda_2___operator__(): pass

def _cGcDronePatrol__Run____32____lambda_1___operator__(): pass

class cGcDissolveEffectComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcDroneSummon__Run____89____lambda_1___operator__(): pass

def _cGcDroneBehaviourTree__Prepare____2____lambda_1___operator__(): pass

def cGcSentinelCoverComponent__IterateTypedComponents__cGcDroneCoverMovement__FindBestCover____2____lambda_1___(): pass

def std___Func_class_bool_TkHandle____Reset__cGcDroneSummon__Run____89____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcDroneComponent__Update____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcDroneComponent__Update____8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcDroneBehaviourTree__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcDroneBehaviourTree__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcDroneBehaviourTree__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcDroneSummonAndDismiss__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcDroneSummonAndDismiss__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcDroneSummonAndDismiss__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcDroneSummon__Run____89____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcDroneSummon__Run____89____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcDroneSummon__Run____89____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__ChooseScanTarget____51____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__ChooseScanTarget____51____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__ChooseScanTarget____51____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Run____32____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Run____32____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Run____32____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__IsHeightenedAlert____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__IsHeightenedAlert____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__IsHeightenedAlert____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc___cGcDronePatrol__Subscribe____2____lambda_2___operator______23____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc___cGcDronePatrol__Subscribe____2____lambda_2___operator______23____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc___cGcDronePatrol__Subscribe____2____lambda_2___operator______23____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcDronePatrol__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcDroneInvestigate__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcDroneInvestigate__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcDroneInvestigate__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc___cGcDroneInvestigate__Subscribe____2____lambda_1___operator______23____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc___cGcDroneInvestigate__Subscribe____2____lambda_1___operator______23____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Func_impl_no_alloc__cGcDronePlayerFocusBehaviour__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcDronePlayerFocusBehaviour__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcDronePlayerFocusBehaviour__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcMissileComponent__Prepare____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcMissileComponent__Prepare____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcMissileComponent__Prepare____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcMissileComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcMissileComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcMissileComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def _cGcMechSearchForTarget__Run____14____lambda_1___operator__(): pass

def _cGcMechSelectAttackTarget__GatherSentinelTargets____2____lambda_1___operator__(): pass

def _cGcSentinelMechWeapon__Prepare____5____lambda_1___operator__(): pass

def GetNumImposterViews(): pass

def GetTextureWidth(): pass

def GetTextureHeight(): pass

def _cGcFadeNodeInstance__Update____37____lambda_2___operator__(): pass

def _cGcSceneManager__RecolourProceduralModels____2____lambda_1___operator__(): pass

def _cGcShootableComponent__IterateTypedComponentsOnNode__cGcSentinelRobot__Prepare____3____lambda_1_______2____lambda_1___operator__(): pass

def _cGcShootableComponent__IterateTypedComponentsOnNode___cGcMechSelectAttackTarget__GatherSentinelTargets____2____lambda_1___operator______8____lambda_1_______2____lambda_1___operator__(): pass

def cGcCreatureComponent__IterateTypedComponents__cGcMechSelectAttackTarget__GatherCreatureTargets____2____lambda_1___(): pass

def cGcSentinelRobotComponent__IterateTypedComponents__cGcMechSelectAttackTarget__GatherSentinelTargets____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcShootableComponent__IterateTypedComponentsOnNode__cGcSentinelRobot__Prepare____3____lambda_1_______2____lambda_1___(): pass

def std___Sort_unchecked_sMechTargetData____cGcMechSelectAttackTarget__Run____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcShootableComponent__IterateTypedComponentsOnNode___cGcMechSelectAttackTarget__GatherSentinelTargets____2____lambda_1___operator______8____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcSceneManager__RecolourProceduralModels____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSceneManager__RecolourProceduralModels____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSceneManager__RecolourProceduralModels____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__Update____37____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__Update____37____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__Update____37____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__Update____28____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__Update____28____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__Update____28____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__RenderImposterToTexture____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__RenderImposterToTexture____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcFadeNodeInstance__RenderImposterToTexture____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSentinelMechWeapon__Prepare____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSentinelMechWeapon__Prepare____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSentinelMechWeapon__Prepare____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSentinelMech__UpdateVisibility____22____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSentinelMech__UpdateVisibility____22____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSentinelMech__Prepare____28____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSentinelMech__Prepare____28____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSentinelMech__Prepare____28____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcMechSelectAttackTarget__GatherWantedTargets____5____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcMechSelectAttackTarget__GatherWantedTargets____5____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcMechSelectAttackTarget__GatherWantedTargets____5____lambda_1__void_cGcPlayerCommon______Move(): pass

def std___Make_heap_unchecked_sMechTargetData____cGcMechSelectAttackTarget__Run____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_sMechTargetData____cGcMechSelectAttackTarget__Run____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_sMechTargetData___sMechTargetData__cGcMechSelectAttackTarget__Run____2____lambda_1___(): pass

def std___Med3_unchecked_sMechTargetData____cGcMechSelectAttackTarget__Run____2____lambda_1___(): pass

class RotateNode(ctypes.Structure):
    def GetOutput(): pass


class ScaleNode(ctypes.Structure):
    def GetOutput(): pass


class SpawnParticles(ctypes.Structure):
    def Start(): pass


def GetClusterLayout(): pass

def RoundDirection(): pass

class cTkJobManager(ctypes.Structure):
    pass

def std___Func_impl_no_alloc__cGcPlanetBuildingGenerator__SetupBuildingFilenames____8____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlanetBuildingGenerator__SetupBuildingFilenames____8____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcBuilding__GetNPCNavAreaComponents____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcBuilding__GetNPCNavAreaComponents____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcBuilding__GetNPCNavAreaComponents____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcAsteroidManager__Prepare____14____lambda_3__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcAsteroidManager__Prepare____14____lambda_3__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcAsteroidManager__Prepare____13____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcAsteroidManager__Prepare____13____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcAsteroidManager__Prepare____12____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcAsteroidManager__Prepare____12____lambda_1__enum_TkPollResult____Move(): pass

class cGcNPCInteractiveObjectComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def IsTerrainEditBoxNode(): pass

def CanUseDecorationItem(): pass

def ___emplace_back_T__TkID__0IA___T1_T1_T1____vector_UsPartWithThemedVariants__V__TkSTLAllocatorShim_UsPartWithThemedVariants___07_0_0___std__QEAA_A_T__QEAT__TkID__0IA___000_Z(): pass

def GetModule(): pass

def GetSlotPosition(): pass

def _cGcMapLayout__Parse____2____lambda_1___operator__(): pass

def _cGcMapLayout__Parse____2____lambda_2___operator__(): pass

def IsExcluded(): pass

def GetFace(): pass

def Mirrored(): pass

def GetConnectorForId(): pass

def GetRelativeTransform(): pass

def _cGcWFCScene__ParseScene____2____lambda_1___operator__(): pass

def GetNavigationNodeArriveDistance(): pass

def _cGcWFCNavigationSubGraph__ParseScene____2____lambda_1___operator__(): pass

def FindInteractiveObjectInParents(): pass

def ____Emplace_one_at_back_VcTkMatrix34__W4eNPCNavGraphNodeType____TAEBM___vector_UsWFCNavigationSubgraphNode__V__TkSTLAllocatorShim_UsWFCNavigationSubgraphNode___0BA__0_0___std__AEAAAEAUsWFCNavigationSubgraphNode____QEAVcTkMatrix34____QEAW4eNPCNavGraphNodeType____QEA__TAEBM_Z(): pass

def ____Emplace_one_at_back_AEAVcTkMatrix34___N__TM___vector_UsWFCNavigationSubgraphNode__V__TkSTLAllocatorShim_UsWFCNavigationSubgraphNode___0BA__0_0___std__AEAAAEAUsWFCNavigationSubgraphNode__AEAVcTkMatrix34____QEA_N__QEA__T__QEAM_Z(): pass

def std___Sort_unchecked_sWFCBuildingPart____cGcMap__CreatePartsList____2____lambda_1___(): pass

def std___Func_impl_no_alloc__FindNavigationGraphNodesForNPCInteractionNode____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__FindNavigationGraphNodesForNPCInteractionNode____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__FindNavigationGraphNodesForNPCInteractionNode____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcWFCNavigationSubGraph__ParseScene____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcWFCNavigationSubGraph__ParseScene____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcWFCNavigationSubGraph__ParseScene____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcWFCScene__Get____13____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcWFCScene__Get____13____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcWFCScene__Get____13____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcWFCScene__ParseScene____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcWFCScene__ParseScene____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcWFCScene__ParseScene____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcModuleSet__cGcModuleSet____2____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcModuleSet__cGcModuleSet____2____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcModuleSet__cGcModuleSet____2____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcMapLayout__Parse____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcMapLayout__Parse____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcMapLayout__Parse____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcMapLayout__Parse____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcMapLayout__Parse____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcMapLayout__Parse____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcMapLayout__cGcMapLayout____2____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcMapLayout__cGcMapLayout____2____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcMapLayout__cGcMapLayout____2____lambda_1__enum_TkPollResult____Move(): pass

def std___Insertion_sort_unchecked_sWFCBuildingPart____cGcMap__CreatePartsList____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_sWFCBuildingPart____cGcMap__CreatePartsList____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_sWFCBuildingPart___sWFCBuildingPart__cGcMap__CreatePartsList____2____lambda_1___(): pass

def std___Med3_unchecked_sWFCBuildingPart____cGcMap__CreatePartsList____2____lambda_1___(): pass

def _cGcSolarSystemGenerator__Generate____9____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__UpdateSSRNodesForModel____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__UpdateSSRNodesForModel____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__UpdateSSRNodesForModel____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__LoadAnomaly____41____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__LoadAnomaly____41____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____104____lambda_7__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____104____lambda_7__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____104____lambda_7__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____83____lambda_6__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____83____lambda_6__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____83____lambda_6__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____83____lambda_5__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____83____lambda_5__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____83____lambda_4__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____83____lambda_4__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____28____lambda_3__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____28____lambda_3__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____28____lambda_3__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____28____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____28____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____28____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Generate____28____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Update____11____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__Update____11____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__PollToPrepare____45____lambda_3__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__PollToPrepare____45____lambda_3__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__PollToPrepare____45____lambda_3__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__PollToPrepare____34____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__PollToPrepare____34____lambda_2__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__PollToPrepare____34____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__PollToPrepare____32____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystem__PollToPrepare____32____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcSolarSystem__OnLeavePlanetOrbit____2____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcSolarSystem__OnLeavePlanetOrbit____2____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcSolarSystem__OnLeavePlanetOrbit____2____lambda_1____void____Move(): pass

def NavGraphPhysProbeJob(): pass

def _cGcNPCManager__StartGenerationTask____2____lambda_1___operator__(): pass

def _cGcNavGraphQuadTree__TreeNode__Subdivide____2____lambda_1___operator__(): pass

def _cGcNavGraphQuadTree__ExpandToFit____5____lambda_1___operator__(): pass

def _cGcNPCNavigationAreaComponent__GatherNodes____2____lambda_2___operator__(): pass

def _cGcNPCNavigationAreaComponent__GatherNodes____34____lambda_3___operator__(): pass

def _cGcNPCNavigationAreaComponent__GenerateGraph____2____lambda_2___operator__(): pass

def _cGcNPCNavigationAreaComponent__SimplifyGraph____2____lambda_1___operator__(): pass

def _cGcNPCNavigationAreaComponent__CleanGraph____2____lambda_1___operator__(): pass

def _cGcNPCNavigationAreaComponent__GenerateNode____2____lambda_1___operator__(): pass

def _cGcPlanet__AppendResourceHintList____2____lambda_1___operator__(): pass

def cTkAttachment__IterateAttachments__cGcNPCNavigationAreaComponent__GatherNodes____8____lambda_1___(): pass

def std__function_bool___cdecl_TkHandle____function_bool___cdecl_TkHandle____cGcNPCNavigationAreaComponent__GatherNodes____2____lambda_2__0_(): pass

def cGcChairComponent__IterateTypedComponentsOnNode__cGcNPCNavigationAreaComponent__GatherNodes____34____lambda_3___(): pass

def robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___operator____cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_(): pass

def robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___throwOverflowError(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcChairComponent__IterateTypedComponentsOnNode__cGcNPCNavigationAreaComponent__GatherNodes____34____lambda_3_______2____lambda_1___(): pass

def robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___insertKeyPrepareEmptySpot_cGcNavGraphNodeHandle_const___(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetChairComponentFromNavNode____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetChairComponentFromNavNode____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetChairComponentFromNavNode____8____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__UpdateRecurrentPhysicsValidation____25____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__Update____46____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__UpdateRecurrentPhysicsValidation____25____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetAllValidPOINodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetAllValidPOINodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetAllValidPOINodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetAllValidNodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetAllValidNodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GetAllValidNodes____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateNode____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateNode____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateNode____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Copy(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__CleanGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__CleanGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__CleanGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__SimplifyGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__SimplifyGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__SimplifyGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__ValidateLinks____85____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__ValidateLinks____85____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__ValidateLinks____85____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__ValidateLinks____5____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__ValidateLinks____5____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateGraph____2____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateGraph____2____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateGraph____2____lambda_2__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GenerateGraph____2____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GatherNodes____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GatherNodes____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__GatherNodes____2____lambda_2__bool_TkHandle____Copy(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__Update____46____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavigationAreaComponent__Update____46____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCNavGraph__RebuildSpatialPartitioning____5____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCNavGraph__RebuildSpatialPartitioning____5____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Do_call(): pass

def std___Func_impl_no_alloc__cGcNPCNavGraph__RebuildSpatialPartitioning____5____lambda_1__enum_eIterateReturn_cGcNavGraphNode______Move(): pass

def std___Func_impl_no_alloc__cGcNPCManager__StartGenerationTask____2____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCManager__StartGenerationTask____2____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcNPCManager__Prepare____64____lambda_2__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCManager__Prepare____64____lambda_2__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcNPCManager__Prepare____46____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcNPCManager__Prepare____46____lambda_1__enum_TkPollResult____Move(): pass

def robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___try_increase_info(): pass

def robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___initData(): pass

def robin_hood__detail__Table_1_80_cGcNavGraphNodeHandle__cGcNPCNavigationAreaComponent__GeneratePath____2___sAStarData_robin_hood__hash_cGcNavGraphNodeHandle_void__std__equal_to_cGcNavGraphNodeHandle___1___rehashPowerOfTwo(): pass

def _cGcPlanetGenerator__LoadQualityVariantForObjectList____2____lambda_1___operator__(): pass

def IsObjectBPriorityLessThanObjectAPriority(): pass

def _cGcPlanetGenerator__GenerateCreatureRoles____2____lambda_1___operator__(): pass

def _cGcPlanetGenerator__GenerateCreatureRoles____30____lambda_2___operator__(): pass

def _cGcPlanetGenerator__GenerateCreatureRoles____2____lambda_3___operator__(): pass

def std___Func_impl_no_alloc__cGcPlanetGenerator__LoadModel____40____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcPlanetGenerator__LoadModel____40____lambda_1__enum_TkPollResult____Move(): pass

def _cGcShipAIPirateChooseTarget__Run____2____lambda_1___operator__(): pass

def _cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_2___operator__(): pass

def std___Sort_unchecked_std__pair_cGcShipAIFlybyMineAsteroids__sAsteroidData_float_____cGcShipAIFlybyMineAsteroids__Run____29____lambda_1___(): pass

def std___Sort_unchecked_cGcShipAIFreighterTargeting__cGcLaunchPoint____cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_2___(): pass

def std___Func_impl_no_alloc__cGcShipAIFreighterTargeting__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIFreighterTargeting__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIFreighterTargeting__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcShipAIFlybyMineAsteroids__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIFlybyMineAsteroids__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIFlybyMineAsteroids__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcShipAIFlybyMineAsteroids__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIFlybyMineAsteroids__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIFlybyMineAsteroids__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcShipAIEscort__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIEscort__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIEscort__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcShipAIEscort__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIEscort__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIEscort__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcShipAIHasSpecificHailReply__Subscribe____2____lambda_3__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIHasSpecificHailReply__Subscribe____2____lambda_3__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcShipAIHasSpecificHailReply__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIHasSpecificHailReply__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIHasSpecificHailReply__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcShipAIHasSpecificHailReply__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIHasSpecificHailReply__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIHasSpecificHailReply__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcShipAIDocked__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIDocked__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIDocked__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcFrigateAIBase__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcFrigateAIBase__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcFrigateAIBase__Subscribe____2____lambda_2__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcFrigateAIBase__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Target_type(): pass

def std___Func_impl_no_alloc__cGcFrigateAIBase__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Do_call(): pass

def std___Func_impl_no_alloc__cGcFrigateAIBase__Subscribe____2____lambda_1__void_cTkMetaMessageWrapperTemplated_void__const______Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__SwitchToCameraTourStage____9____lambda_2__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__RenderNGui____100____lambda_3__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__SwitchToCameraTourStage____9____lambda_2__void____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__SwitchToCameraTourStage____5____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__SwitchToCameraTourStage____5____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__SwitchToCameraTourStage____5____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__WaitForCameraAnimation____39____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__WaitForCameraAnimation____39____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__WaitForCameraAnimation____39____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadSyncPoint____24____lambda_2__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadSyncPoint____24____lambda_2__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadSyncPoint____24____lambda_2__void____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadSyncPoint____18____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadSyncPoint____18____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadSyncPoint____18____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadedUpdate____20____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadedUpdate____20____lambda_1__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__ThreadedUpdate____20____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__RenderNGui____100____lambda_3__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__RenderNGui____100____lambda_3__void____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__RenderNGui____77____lambda_2__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__RenderNGui____77____lambda_2__void____Move(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__RenderNGui____66____lambda_1__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSolarSystemTour__RenderNGui____66____lambda_1__void____Move(): pass

def std___Func_impl_no_alloc__cGcAvailableWeapons__Prepare____14____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcAvailableWeapons__Prepare____14____lambda_1__enum_TkPollResult____Move(): pass

def std___Make_heap_unchecked_std__pair_cGcShipAIFlybyMineAsteroids__sAsteroidData_float_____cGcShipAIFlybyMineAsteroids__Run____29____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_std__pair_cGcShipAIFlybyMineAsteroids__sAsteroidData_float_____cGcShipAIFlybyMineAsteroids__Run____29____lambda_1___(): pass

def std___Insertion_sort_unchecked_cGcShipAIFreighterTargeting__cGcLaunchPoint____cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_2___(): pass

def std___Make_heap_unchecked_cGcShipAIFreighterTargeting__cGcLaunchPoint____cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_2___(): pass

def std___Partition_by_median_guess_unchecked_cGcShipAIFreighterTargeting__cGcLaunchPoint____cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_2___(): pass

def std___Pop_heap_hole_by_index_std__pair_cGcShipAIFlybyMineAsteroids__sAsteroidData_float____std__pair_cGcShipAIFlybyMineAsteroids__sAsteroidData_float___cGcShipAIFlybyMineAsteroids__Run____29____lambda_1___(): pass

def std___Guess_median_unchecked_std__pair_cGcShipAIFlybyMineAsteroids__sAsteroidData_float_____cGcShipAIFlybyMineAsteroids__Run____29____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cGcShipAIFreighterTargeting__cGcLaunchPoint___cGcShipAIFreighterTargeting__cGcLaunchPoint__cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_2___(): pass

def std___Med3_unchecked_cGcShipAIFreighterTargeting__cGcLaunchPoint____cGcShipAIFreighterTargeting__ShipAIConstruct____2____lambda_2___(): pass

class cGcSpaceshipShieldComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcAISpaceshipComponent__Prepare____2____lambda_3___operator__(): pass

def _cGcAISpaceshipManager__SpawnShip____2____lambda_1___operator__(): pass

def cGcFreighterSyncComponent__IterateTypedComponents__cGcAISpaceshipComponent__Update____22____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcAISpaceshipComponent__NotifyNetworkOfAttack____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcAISpaceshipComponent__SwitchToComplexCollision____8____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcAISpaceshipManager__OnAIShipChangedTarget____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcTurretComponent__IterateTypedComponentsOnNode__cGcAISpaceshipComponent__Prepare____2____lambda_2_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcCreatureFullBodyIKComponent__IterateTypedComponentsOnNode__cGcAISpaceshipComponent__Prepare____2____lambda_4_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcNPCPlacementComponent__IterateTypedComponentsOnNode__cGcAISpaceshipComponent__RequestNPCSpawn____2____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcAISpaceshipComponent__SwitchToComplexCollision____2____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcAISpaceshipManager__SpawnShip____35____lambda_3_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipManager__CacheShipForFrigate____2____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipManager__CacheShipForFrigate____2____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipManager__CacheShips____20____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipManager__CacheShips____20____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipManager__CacheHangar____2____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipManager__CacheHangar____2____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipComponent__Prepare____2____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipComponent__Prepare____2____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipComponent__Prepare____2____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcAISpaceshipComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcShipAIPoliceChooseTarget__Run____2____lambda_1__void_cGcPlayerCommon______Target_type(): pass

def std___Func_impl_no_alloc__cGcShipAIPoliceChooseTarget__Run____2____lambda_1__void_cGcPlayerCommon______Do_call(): pass

def std___Func_impl_no_alloc__cGcShipAIPoliceChooseTarget__Run____2____lambda_1__void_cGcPlayerCommon______Move(): pass

def _cGcEngineComponent__Prepare____2____lambda_1___operator__(): pass

def GcSpaceshipManifoldFunc(): pass

def _cGcSpaceshipComponent__Prepare____2____lambda_2___operator__(): pass

def ___emplace_back_AEAY0BE___CBDAEAM___vector_VDisplayNumber_cGcSpaceshipComponent__V__TkSTLAllocatorShim_VDisplayNumber_cGcSpaceshipComponent___07_0_0___std__QEAA_A_TAEAY0BE___CBDAEAM_Z(): pass

def _cGcSpaceshipComponent__UpdateControlled____281____lambda_1___operator__(): pass

def _cGcSpaceshipComponent__UpdateControlled____281____lambda_1____lambda_1_(): pass

def _cGcSpaceshipComponent__UpdateControlled____278____lambda_2___operator__(): pass

def ___emplace_back_AEAY0O___CBDAEAM___vector_VDisplayNumber_cGcSpaceshipComponent__V__TkSTLAllocatorShim_VDisplayNumber_cGcSpaceshipComponent___07_0_0___std__QEAA_A_TAEAY0O___CBDAEAM_Z(): pass

def ___emplace_back_AEAY0BB___CBDAEAM___vector_VDisplayNumber_cGcSpaceshipComponent__V__TkSTLAllocatorShim_VDisplayNumber_cGcSpaceshipComponent___07_0_0___std__QEAA_A_TAEAY0BB___CBDAEAM_Z(): pass

def ___emplace_back_AEAY0P___CBDAEAM___vector_VDisplayNumber_cGcSpaceshipComponent__V__TkSTLAllocatorShim_VDisplayNumber_cGcSpaceshipComponent___07_0_0___std__QEAA_A_TAEAY0P___CBDAEAM_Z(): pass

def _cGcSpaceshipComponent__RenderNGui____8____lambda_1___operator__(): pass

def cGcAISpaceshipComponent__IterateTypedComponents__cGcFreighterSyncComponent__Prepare____2____lambda_1___(): pass

def cGcFreighterSyncComponent__IterateTypedComponents__cGcFreighterSyncComponent__Update____8____lambda_1___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcFreighterSyncComponent__NotifyNetworkOfAttack____19____lambda_1___(): pass

def cGcNPCComponent__IterateTypedComponents__cGcShipOwnershipComponent__OnLandedStateChanged____5____lambda_1___(): pass

def cGcNPCComponent__IterateTypedComponents__cGcShipOwnershipComponent__OnLandedStateChanged____9____lambda_2___(): pass

def cGcSpaceshipComponent__IterateTypedComponents__cGcSpaceshipComponent__Update____116____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcSpaceshipComponent__UpdateControlled____281____lambda_1___(): pass

def cGcAISpaceshipComponent__IterateTypedComponents__cGcSpaceshipComponent__UpdateControlled____278____lambda_2___(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcSpaceshipComponent__RequestSalvage____2____lambda_1___(): pass

def std___Func_class_bool_TkHandle____Reset__cGcSpaceshipComponent__Update____23____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcShipTargetDisplay__Update____11____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcShipTargetDisplay__Update____11____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Update____23____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Update____23____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_4__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_4__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_4__void____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_3__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_3__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_3__void____Move(): pass

def std___Func_impl_no_alloc___cGcSpaceshipComponent__Prepare____2____lambda_3___operator______2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcSpaceshipComponent__Prepare____2____lambda_3___operator______2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcSpaceshipComponent__Prepare____2____lambda_3___operator______2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcEngineComponent__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcEngineComponent__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcEngineComponent__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcSpaceshipComponent__CheckForDocked____21____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcSpaceshipComponent__CheckForDocked____21____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cGcSpaceshipComponent__CheckForDocked____21____lambda_1____void____Move(): pass

def _cGcSpaceshipControl__NotifyModel____2____lambda_1___operator__(): pass

def _cGcSpaceshipControl__IsNotificationActive____2____lambda_1___operator__(): pass

def _cGcSpaceshipEngineEffects__Update____2____lambda_1___operator__(): pass

def _cGcSpaceshipWarp__HasPulseDriveCollision____3____lambda_1___operator__(): pass

def _cGcSpaceshipWeapons__Prepare____5____lambda_1___operator__(): pass

def _cGcSpaceshipWeapons__UpdateManualLockTarget____16____lambda_1___operator__(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcSpaceshipControl__NotifyModel____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cGcSpaceshipControl__IsNotificationActive____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcWarpEffect__Init____2____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcWarpEffect__Finish____2____lambda_1_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcShootPoint__Update____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcShootPoint__Update____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWeapons__Prepare____11____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWeapons__Prepare____11____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWeapons__Prepare____11____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWeapons__Prepare____5____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWeapons__Prepare____5____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWeapons__Prepare____5____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWarp__Prepare____17____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWarp__Prepare____17____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWarp__Prepare____17____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWarp__Prepare____11____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWarp__Prepare____11____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipWarp__Prepare____11____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__AddJetStage____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__AddJetStage____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__AddJetStage____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__Prepare____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__Prepare____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__Prepare____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcStagedJetEngine__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__UpdateTrailsResource____28____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__UpdateTrailsResource____28____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__UpdateTrailsResource____28____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____61____lambda_4__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____61____lambda_4__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____61____lambda_4__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____57____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____57____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____57____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____48____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____48____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____48____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____37____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____37____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcSpaceshipEngineEffects__Construct____37____lambda_1__bool_TkHandle____Move(): pass

def _cGcMechControl__Prepare____2____lambda_1___operator__(): pass

def _cGcMechControl__UpdateAnims____2____lambda_1___operator__(): pass

def _cGcMechControl__UpdateFootDust____2____lambda_1___operator__(): pass

def _cGcMechControl__UpdateWeaponArmAimAnims____2____lambda_1___operator__(): pass

class cGcTechnologyAttachmentComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


class cGcVehicleRaceInviteComponent(ctypes.Structure):
    def FindFirstTypedComponent(): pass


def _cGcVehicleCockpit__AddCockpit____2____lambda_1___operator__(): pass

def GcVehicleInstanceBroadphaseFunc(): pass

def GcVehicleInstanceManifoldFunc(): pass

def _cGcVehicleComponent__Prepare____63____lambda_1___operator__(): pass

def _cGcVehicleComponent__Prepare____2____lambda_2___operator__(): pass

def _cGcVehicleComponent__Prepare____91____lambda_7___operator__(): pass

def _cGcVehicleComponent__Prepare____2____lambda_8___operator__(): pass

def __cGcVehicleComponent__Prepare____2____lambda_8___operator______2____lambda_1___operator__(): pass

def _cGcVehicleComponent__UpdateUnderwaterControl____2____lambda_1___operator__(): pass

def cGcNetworkPlayerManager__ForEachFullSimRemotePlayer__cGcMechControl__UpdateFootDust____59____lambda_2___(): pass

def _cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcVehicleCheckpointComponent__Prepare____2____lambda_1_______2____lambda_1___operator__(): pass

def _cTkAnimationComponent__IterateTypedComponentsOnNode__cGcVehicleComponent__Prepare____2____lambda_10_______2____lambda_1___operator__(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cGcVehicleCheckpointComponent__Prepare____2____lambda_1_______2____lambda_1___(): pass

def std___Sort_unchecked_RaceSection____cGcVehicleCheckpointManager__SetupRaceCourse____2____lambda_1___(): pass

def std___Sort_unchecked_MultiplayerSplits____cGcVehicleCheckpointManager__FinishedRace____5____lambda_2___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkAnimationComponent__IterateTypedComponentsOnNode__cGcVehicleComponent__Prepare____2____lambda_10_______2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_9__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_9__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_9__void____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_8__void____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_8__void____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_8__void____Move(): pass

def std___Func_impl_no_alloc___cGcVehicleComponent__Prepare____2____lambda_8___operator______2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcVehicleComponent__Prepare____2____lambda_8___operator______2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcVehicleComponent__Prepare____2____lambda_8___operator______2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____91____lambda_7__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____91____lambda_7__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____91____lambda_7__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____89____lambda_6__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____89____lambda_6__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____89____lambda_6__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_5__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_5__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_5__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_4__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_4__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_4__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____79____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____79____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____79____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____63____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____63____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleComponent__Prepare____63____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__Update____B____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__Update____B____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__FinaliseExit____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__FinaliseExit____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__FinaliseExit____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__DoEnter____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__DoEnter____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__UpdateLoading____29____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__UpdateLoading____29____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____2____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____2____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____2____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____7____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____7____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____7____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__AddCockpit____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__GetScaleFromCockpitLocator____3____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__GetScaleFromCockpitLocator____3____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcVehicleCockpit__GetScaleFromCockpitLocator____3____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cGcMechControl__Prepare____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcMechControl__Prepare____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cGcMechControl__Prepare____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc___cGcMechControl__Prepare____2____lambda_1___operator______9____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcMechControl__Prepare____2____lambda_1___operator______9____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc___cGcMechControl__Prepare____2____lambda_1___operator______5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc___cGcMechControl__Prepare____2____lambda_1___operator______5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc___cGcMechControl__Prepare____2____lambda_1___operator______5____lambda_1__bool_TkHandle____Move(): pass

def std___Insertion_sort_unchecked_RaceSection____cGcVehicleCheckpointManager__SetupRaceCourse____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_RaceSection____cGcVehicleCheckpointManager__SetupRaceCourse____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_MultiplayerSplits____cGcVehicleCheckpointManager__FinishedRace____5____lambda_2___(): pass

def std___Pop_heap_hole_by_index_RaceSection___RaceSection__cGcVehicleCheckpointManager__SetupRaceCourse____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_MultiplayerSplits___MultiplayerSplits__cGcVehicleCheckpointManager__FinishedRace____5____lambda_2___(): pass

def std__invoke__cGcVehicleCockpit__GetScaleFromCockpitLocator____3____lambda_1____TkHandle_(): pass

def std___Med3_unchecked_RaceSection____cGcVehicleCheckpointManager__SetupRaceCourse____2____lambda_1___(): pass

def std___Med3_unchecked_MultiplayerSplits____cGcVehicleCheckpointManager__FinishedRace____5____lambda_2___(): pass

def FindInputBinding(): pass

def BindingHelpers__CollectBindings(): pass

def BindingHelpers__CollectBindings_0(): pass

def cGcVehicleComponent__IterateTypedComponents__cGcVehicleGarageComponent__Update____2____lambda_1___(): pass

def std___Func_impl_no_alloc__cGcVehicleGarageComponent__Update____8____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cGcVehicleGarageComponent__Update____8____lambda_2__bool_TkHandle____Move(): pass

def ExecuteExtractFrame(): pass

def ExtractFrame(): pass

def ExtractAllTransformsForFrame(): pass

def ExtractMoveLocatorTransform(): pass

def _cEgAnimationController__EvalCommandListInternal____2____lambda_1___operator__(): pass

def _cEgCameraNode__SetVelFromPrevMatrix____5____lambda_2___operator__(): pass

def std___Sort_unchecked_GeometryStreaming__cEgGeometryStreamer__cBufferData____GeometryStreaming__cEgGeometryStreamer__Prepare____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_GeometryStreaming__cEgGeometryStreamer__cBufferData____GeometryStreaming__cEgGeometryStreamer__Prepare____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_GeometryStreaming__cEgGeometryStreamer__cBufferData___GeometryStreaming__cEgGeometryStreamer__cBufferData__GeometryStreaming__cEgGeometryStreamer__Prepare____2____lambda_1___(): pass

def std___Med3_unchecked_GeometryStreaming__cEgGeometryStreamer__cBufferData____GeometryStreaming__cEgGeometryStreamer__Prepare____2____lambda_1___(): pass

def ___emplace_back_AEA_K___vector_UcEgShaderCombination__V__TkSTLAllocatorShim_UcEgShaderCombination___07_07___std__QEAA_A_TAEA_K_Z(): pass

def PlatformCheck(): pass

def GetGroupEnabledFromId(): pass

def ParseIf(): pass

def ParseCondition(): pass

def TransposeFrustum(): pass

def std___Sort_unchecked_cCallbackWithPriority_cPreAdditiveCallback_____cEgModelNode__SubscribeToPreAdditiveCallback____5____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cCallbackWithPriority_cPreAdditiveCallback_____cEgModelNode__SubscribeToPreAdditiveCallback____5____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cCallbackWithPriority_cPreAdditiveCallback____cCallbackWithPriority_cPreAdditiveCallback___cEgModelNode__SubscribeToPreAdditiveCallback____5____lambda_1___(): pass

def std___Med3_unchecked_cCallbackWithPriority_cPreAdditiveCallback_____cEgModelNode__SubscribeToPreAdditiveCallback____5____lambda_1___(): pass

def sscanf(): pass

def ___emplace_back_AEAV__cTkFixedString__0EA_D__AEAV1_AEAHAEAW4eSMShaderConnType_____vector_UConn_cEgSMShaderContext__V__StackAllocator_UConn_cEgSMShaderContext___03_0_0___std__QEAA_A_TAEAV__cTkFixedString__0EA_D__0AEAHAEAW4eSMShaderConnType___Z(): pass

def ___emplace_back_U__pair_V__cTkFixedString__0EA_D__V1__std_____vector_U__pair_V__cTkFixedString__0EA_D__V1__std__V__StackAllocator_U__pair_V__cTkFixedString__0EA_D__V1__std___03_0_0___std__QEAA_A_T__QEAU__pair_V__cTkFixedString__0EA_D__V1__1__Z(): pass

def CreateShaderNames(): pass

def EvictTex(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cEgFfxFsr2Draw__Draw____14____lambda_2____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cEgFfxFsr2Draw__Draw____14____lambda_2____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cEgFfxFsr2Draw__Draw____14____lambda_2____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cEgFfxFsr2Pass__Dispatch____21____lambda_2____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cEgFfxFsr2Pass__Dispatch____21____lambda_2____void____Move(): pass

def AVX_memcpy16_aligned(): pass

def RunGraphicsTask(): pass

def GetCameraInfo(): pass

def GetNodeCountRecursive(): pass

def SwapIndexListFixup(): pass

def __global_delete(): pass

def cEgRenderQueueBuffer__SortHybridQSIS__cEgRenderQueue__GetSortedRenderQueue____11____lambda_1___(): pass

def cEgRenderQueueBuffer__SortHybridQSIS__cEgRenderQueue__GetSortedRenderQueue____12____lambda_2___(): pass

def cEgRenderQueueBuffer__SortHybridQSIS__cEgRenderQueue__GetSortedRenderQueue____13____lambda_3___(): pass

def cEgRenderQueueBuffer__SortHybridQSIS__cEgRenderQueue__KickSortsForPipeline____32____lambda_4___(): pass

def cEgRenderQueueBuffer__SortHybridQSIS__cEgRenderQueue__KickSortsForPipeline____33____lambda_5___(): pass

def cEgRenderQueueBuffer__SortHybridQSIS__cEgRenderQueue__KickSortsForPipeline____34____lambda_6___(): pass

def cEgRenderQueueBuffer__SortJob__cEgRenderQueue__KickSortsForPipeline____26____lambda_1___(): pass

def cEgRenderQueueBuffer__SortJob__cEgRenderQueue__KickSortsForPipeline____27____lambda_2___(): pass

def cEgRenderQueueBuffer__SortJob__cEgRenderQueue__KickSortsForPipeline____28____lambda_3___(): pass

def cEgRenderQueueBuffer__SortJob__cEgRenderQueue__KickSortsForPipeline____32____lambda_4___(): pass

def cEgRenderQueueBuffer__SortJob__cEgRenderQueue__KickSortsForPipeline____33____lambda_5___(): pass

def cEgRenderQueueBuffer__SortJob__cEgRenderQueue__KickSortsForPipeline____34____lambda_6___(): pass

def xmlMalloc(): pass

def xmlRealloc(): pass

def stringDup(): pass

def toXMLStringUnSafe(): pass

def lengthXMLString(): pass

def fromXMLString(): pass

def myTagCompare(): pass

def GetNextToken(): pass

def std___Func_impl_no_alloc__EngineUtils__ReloadResource____8____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__EngineUtils__ReloadResource____8____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__EngineUtils__ReloadResource____8____lambda_1__bool_TkHandle____Move(): pass

def GetScreenPlaneData(): pass

def GetCullingIcosahedron(): pass

def GetIcoFaceBox(): pass

def _EgInstancedModelExtension__cEgInstancedModelNode__RebuildInstanceAABBs_PreAsyncUpdate____2____lambda_1___operator__(): pass

def _EgInstancedModelExtension__cEgInstancedModelNode__ExpandBoundingBoxes____2____lambda_1___operator__(): pass

def std___Func_impl_no_alloc__EgInstancedModelExtension__cEgInstancedModelNode__ExpandBoundingBoxes____2____lambda_1__void_cEgSceneNode______Target_type(): pass

def std___Func_impl_no_alloc__EgInstancedModelExtension__cEgInstancedModelNode__ExpandBoundingBoxes____2____lambda_1__void_cEgSceneNode______Do_call(): pass

def std___Func_impl_no_alloc__EgInstancedModelExtension__cEgInstancedModelNode__ExpandBoundingBoxes____2____lambda_1__void_cEgSceneNode______Move(): pass

def std___Func_impl_no_alloc__EgInstancedModelExtension__cEgInstancedModelNode__RebuildInstanceAABBs_PreAsyncUpdate____2____lambda_1__void_cEgSceneNode______Target_type(): pass

def std___Func_impl_no_alloc__EgInstancedModelExtension__cEgInstancedModelNode__RebuildInstanceAABBs_PreAsyncUpdate____2____lambda_1__void_cEgSceneNode______Do_call(): pass

def std___Func_impl_no_alloc__EgInstancedModelExtension__cEgInstancedModelNode__RebuildInstanceAABBs_PreAsyncUpdate____2____lambda_1__void_cEgSceneNode______Move(): pass

def Json__duplicateStringValue(): pass

def _LN12_87(): pass

def Json__codePointToUTF8(): pass

def Json___anonymous_namespace___valueToString(): pass

def Json__toHex16Bit(): pass

def Json__valueToQuotedStringN(): pass

def LZ4_putPosition(): pass

def LZ4_compress_fast_extState(): pass

def LZ4_compress_fast(): pass

def LZ4_renormDictT(): pass

def LZ4_compress_fast_continue(): pass

def LZ4_decompress_safe(): pass

def LZ4_decompress_fast(): pass

def LZ4_decompress_safe_continue(): pass

def EgMagicModels__SetModelFadeValue(): pass

def EgMagicModels__GatherMeshVertsRecursive(): pass

def ___emplace_back_U__Splatter__0A__VecIntrinsics__U__Splatter__00_2_U__Splatter__01_2____vector_UhkFloat3__V__TkSTLAllocatorShim_UhkFloat3___03_0_0___std__QEAA_A_T__QEAU__Splatter__0A__VecIntrinsics____QEAU__Splatter__00_3___QEAU__Splatter__01_3__Z(): pass

def EgMagicModels__GatherModelVertsRecursive(): pass

def printf(): pass

def sprintf_s(): pass

def nNanoVG__stbtt__isfont(): pass

def nNanoVG__stbtt__find_table(): pass

def stbtt_GetFontOffsetForIndex(): pass

def stbtt_InitFont(): pass

def stbtt_FindGlyphIndex(): pass

def nNanoVG__stbtt__GetGlyfOffset(): pass

def nNanoVG__stbtt__close_shape(): pass

def stbtt_GetGlyphShape(): pass

def stbtt_GetGlyphKernAdvance(): pass

def nNanoVG__stbtt__fill_active_edges(): pass

def nNanoVG__stbtt__rasterize_sorted_edges(): pass

def nNanoVG__stbtt__edge_compare(): pass

def nNanoVG__stbtt__rasterize(): pass

def nNanoVG__stbtt__tesselate_curve(): pass

def stbtt_MakeGlyphBitmapSubpixel(): pass

def nNanoVG__fons__tmpalloc(): pass

def nNanoVG__fons__atlasInsertNode(): pass

def nNanoVG__fons__atlasAddSkylineLevel(): pass

def nNanoVG__fons__atlasAddRect(): pass

def nNanoVG__fons__addWhiteRect(): pass

def nNanoVG__fons__allocFont(): pass

def nNanoVG__fons__allocGlyph(): pass

def nNanoVG__fons__blurCols(): pass

def nNanoVG__fons__blurRows(): pass

def nNanoVG__fons__blur(): pass

def nNanoVG__fons__getQuad(): pass

def nNanoVG__fons__flush(): pass

def nNanoVG__fons__getVertAlign(): pass

def nNanoVG__stbi_load_main(): pass

def nNanoVG__stbi__refill_buffer(): pass

def nNanoVG__stbi__skip(): pass

def nNanoVG__stbi__getn(): pass

def nNanoVG__stbi__get16be(): pass

def nNanoVG__stbi__get32be(): pass

def nNanoVG__stbi__get16le(): pass

def nNanoVG__stbi__get32le(): pass

def nNanoVG__stbi__convert_format(): pass

def nNanoVG__stbi__build_huffman(): pass

def nNanoVG__stbi__grow_buffer_unsafe(): pass

def nNanoVG__stbi__jpeg_decode_block(): pass

def nNanoVG__stbi__idct_block(): pass

def nNanoVG__stbi__get_marker(): pass

def nNanoVG__stbi__parse_entropy_coded_data(): pass

def nNanoVG__stbi__process_marker(): pass

def nNanoVG__stbi__process_scan_header(): pass

def nNanoVG__stbi__process_frame_header(): pass

def nNanoVG__decode_jpeg_header(): pass

def nNanoVG__stbi__resample_row_v_2(): pass

def nNanoVG__stbi__resample_row_h_2(): pass

def nNanoVG__stbi__resample_row_hv_2(): pass

def nNanoVG__stbi__resample_row_generic(): pass

def nNanoVG__stbi__cleanup_jpeg(): pass

def nNanoVG__load_jpeg_image(): pass

def nNanoVG__stbi__jpeg_load(): pass

def zbuild_huffman(): pass

def nNanoVG__stbi__zexpand(): pass

def nNanoVG__stbi__parse_huffman_block(): pass

def nNanoVG__stbi__compute_huffman_codes(): pass

def nNanoVG__stbi__parse_uncomperssed_block(): pass

def nNanoVG__stbi__parse_zlib(): pass

def nNanoVG__stbi__check_png_header(): pass

def nNanoVG__stbi__create_png_image_raw(): pass

def nNanoVG__stbi__create_png_image(): pass

def nNanoVG__stbi__parse_png_file(): pass

def high_bit(): pass

def nNanoVG__stbi__bmp_load(): pass

def nNanoVG__stbi__tga_test(): pass

def nNanoVG__stbi__tga_load(): pass

def nNanoVG__stbi__psd_load(): pass

def nNanoVG__stbi__pic_is4(): pass

def nNanoVG__stbi__readval(): pass

def nNanoVG__stbi__pic_load_core(): pass

def nNanoVG__stbi__pic_load(): pass

def nNanoVG__stbi__gif_test_raw(): pass

def nNanoVG__stbi__gif_parse_colortable(): pass

def nNanoVG__stbi__gif_header(): pass

def nNanoVG__stbi__out_gif_code(): pass

def nNanoVG__stbi__process_gif_raster(): pass

def nNanoVG__stbi__gif_load_next(): pass

def nNanoVG__stbi__gif_load(): pass

def nNanoVG__stbi__hdr_test(): pass

def nNanoVG__stbi__hdr_gettoken(): pass

def hdr_convert(): pass

def nNanoVG__stbi__hdr_load(): pass

def nNanoVG__stbiw__jpg_writeBits(): pass

def nNanoVG__stbiw__jpg_DCT(): pass

def nNanoVG__stbiw__jpg_processDU(): pass

def nNanoVG__stbi_write_jpg_core(): pass

def nvg__deletePathCache(): pass

def nvgCreateInternal(): pass

def nvgBeginFrame(): pass

def nvgEndFrame(): pass

def nvgRGBf(): pass

def nvgRGBA(): pass

def nvgRGBAf(): pass

def nvgTransformScale(): pass

def nvgTransformMultiply(): pass

def nvgTransformInverse(): pass

def nvg__setPaintColor(): pass

def nvgSave(): pass

def nvgRestore(): pass

def nvgReset(): pass

def nvgStrokeWidth(): pass

def nvgLineCap(): pass

def nvgGlobalAlpha(): pass

def nvgTranslate(): pass

def nvgScale(): pass

def nvgCurrentTransform(): pass

def nvgStrokeColor(): pass

def nvgStrokePaint(): pass

def nvgFillColor(): pass

def nvgFillPaint(): pass

def nvgLinearGradient(): pass

def nvgRadialGradient(): pass

def nvgBoxGradient(): pass

def nvgImagePattern(): pass

def nvgScissor(): pass

def nvgIntersectScissor(): pass

def nvgResetScissor(): pass

def nvg__appendCommands(): pass

def nvg__addPoint(): pass

def nvg__getAverageScale(): pass

def nvg__allocTempVerts(): pass

def nvg__tesselateBezier(): pass

def nvg__flattenPaths(): pass

def nvg__roundJoin(): pass

def nvg__bevelJoin(): pass

def nvg__buttCapStart(): pass

def nvg__buttCapEnd(): pass

def nvg__calculateJoins(): pass

def nvg__expandStroke(): pass

def nvg__expandFill(): pass

def nvgBeginPath(): pass

def nvgMoveTo(): pass

def nvgLineTo(): pass

def nvgBezierTo(): pass

def nvgArcTo(): pass

def nvgPathWinding(): pass

def nvgArc(): pass

def nvgRect(): pass

def nvgRoundedRect(): pass

def nvgEllipse(): pass

def nvgCircle(): pass

def nvgFill(): pass

def nvgStroke(): pass

def nvgCreateFontMem(): pass

def nvgFontSize(): pass

def nvgFontBlur(): pass

def nvgTextLetterSpacing(): pass

def nvgTextAlign(): pass

def nvgFontFaceId(): pass

def nvgFontFace(): pass

def nvg__flushTextTexture(): pass

def nvg__allocTextAtlas(): pass

def nvg__renderText(): pass

def nvgText(): pass

def nvgTextBox(): pass

def nvgTextBreakLines(): pass

def nvgTextBounds(): pass

def nvgTextBoxBounds(): pass

def nvgTextMetrics(): pass

def nvg_malloc(): pass

def nvg_calloc(): pass

def nvg_free(): pass

def nvg_realloc(): pass

def _cEgTrailNode__Update____19____lambda_1___operator__(): pass

def std__vector__TkParticle__NGuiRender____5___Info_TkSTLAllocatorShim__TkParticle__NGuiRender____5___Info_8__1______Emplace_reallocate__TkParticle__NGuiRender____5___Info_(): pass

def std___Func_impl_no_alloc__TkParticle__cTkEmitterNode__EmitterBeginUpdate____14____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__TkParticle__cTkEmitterNode__EmitterBeginUpdate____14____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__TkParticle__cTkEmitterNode__EmitterBeginUpdate____14____lambda_1__bool_TkHandle____Move(): pass

class EgDynamicVertexExtension(ctypes.Structure):
    def InitTerrainVertex(): pass

    def InitLineVertex(): pass

    def InitExtension(): pass


def findMapping(): pass

def findValidMapping(): pass

def parseMapping(): pass

def _glfwInputKey(): pass

def _glfwInputCursorPos(): pass

def _glfwInputJoystick(): pass

def _glfwAllocJoystick(): pass

def _glfwCenterCursorInContentArea(): pass

def glfwSetInputMode(): pass

def glfwGetKeyName(): pass

def glfwGetKey(): pass

def glfwGetMouseButton(): pass

def glfwGetCursorPos(): pass

def glfwSetCursorPos(): pass

def glfwCreateStandardCursor(): pass

def glfwDestroyCursor(): pass

def glfwSetCursor(): pass

def glfwSetKeyCallback(): pass

def glfwSetCharCallback(): pass

def glfwSetScrollCallback(): pass

def glfwUpdateGamepadMappings(): pass

def _glfwIsValidContextConfig(): pass

def _glfwChooseFBConfig(): pass

def _glfwRefreshContextAttribs(): pass

def _glfwStringInExtensionString(): pass

def glfwSwapInterval(): pass

def glfwExtensionSupported(): pass

def terminate_0(): pass

def _glfwInputError(): pass

def glfwInit(): pass

def glfwTerminate(): pass

def compareVideoModes(): pass

def refreshVideoModes(): pass

def _glfwInputMonitor(): pass

def _glfwChooseVideoMode(): pass

def glfwGetMonitors(): pass

def glfwGetPrimaryMonitor(): pass

def glfwGetMonitorPos(): pass

def glfwGetMonitorName(): pass

def glfwGetVideoModes(): pass

def glfwGetVideoMode(): pass

def _glfwInputWindowFocus(): pass

def glfwCreateWindow(): pass

def glfwWindowHint(): pass

def glfwDestroyWindow(): pass

def glfwGetWindowPos(): pass

def glfwSetWindowPos(): pass

def glfwGetWindowSize(): pass

def glfwSetWindowSize(): pass

def glfwGetWindowAttrib(): pass

def glfwSetWindowMonitor(): pass

def glfwSetWindowCloseCallback(): pass

def glfwPollEvents(): pass

def _glfwInitVulkan(): pass

def _glfwGetVulkanResultString(): pass

def glfwGetRequiredInstanceExtensions(): pass

def glfwGetPhysicalDevicePresentationSupport(): pass

def glfwCreateWindowSurface(): pass

def monitorCallback(): pass

def createMonitor(): pass

def _glfwPollMonitorsWin32(): pass

def _glfwSetVideoModeWin32(): pass

def _glfwPlatformGetVideoModes(): pass

def _glfwPlatformGetVideoMode(): pass

def glfwGetMonitorAdapterName(): pass

def _glfwPlatformSetGammaRamp(): pass

def getWindowStyle(): pass

def applyAspectRatio(): pass

def updateCursorImage(): pass

def enableRawMouseMotion(): pass

def disableRawMouseMotion(): pass

def disableCursor(): pass

def enableCursor(): pass

def cursorInContentArea(): pass

def updateFramebufferTransparency(): pass

def getKeyMods(): pass

def getAsyncKeyMods(): pass

def fitToMonitor(): pass

def acquireMonitor(): pass

def releaseMonitor(): pass

def windowProc(): pass

def createNativeWindow(): pass

def _glfwPlatformCreateWindow(): pass

def _glfwPlatformSetWindowPos(): pass

def _glfwPlatformGetWindowFrameSize(): pass

def _glfwPlatformFocusWindow(): pass

def _glfwPlatformSetWindowMonitor(): pass

def _glfwPlatformPollEvents(): pass

def _glfwPlatformSetCursorPos(): pass

def glfwGetWin32Window(): pass

def compareJoystickObjects(): pass

def closeJoystick(): pass

def deviceObjectCallback(): pass

def deviceCallback(): pass

def _glfwDetectJoystickConnectionWin32(): pass

def _glfwPlatformPollJoystick(): pass

def _glfwPlatformGetTimerValue(): pass

def loadLibraries(): pass

def createKeyTables(): pass

def _glfwCreateWideStringFromUTF8Win32(): pass

def _glfwCreateUTF8FromWideStringWin32(): pass

def _glfwInputErrorWin32(): pass

def _glfwUpdateKeyNamesWin32(): pass

def _glfwIsWindowsVersionOrGreaterWin32(): pass

def _glfwIsWindows10BuildOrGreaterWin32(): pass

def _glfwPlatformInit(): pass

def findPixelFormatAttribValue(): pass

def choosePixelFormat(): pass

def makeContextCurrentWGL(): pass

def swapBuffersWGL(): pass

def swapIntervalWGL(): pass

def extensionSupportedWGL(): pass

def getProcAddressWGL(): pass

def destroyContextWGL(): pass

def _glfwInitWGL(): pass

def _glfwCreateContextWGL(): pass

def getEGLErrorString(): pass

def chooseEGLConfig(): pass

def makeContextCurrentEGL(): pass

def swapBuffersEGL(): pass

def swapIntervalEGL(): pass

def extensionSupportedEGL(): pass

def getProcAddressEGL(): pass

def destroyContextEGL(): pass

def _glfwInitEGL(): pass

def _glfwTerminateEGL(): pass

def _glfwCreateContextEGL(): pass

def makeContextCurrentOSMesa(): pass

def getProcAddressOSMesa(): pass

def destroyContextOSMesa(): pass

def _glfwCreateContextOSMesa(): pass

def stbi_image_free(): pass

def stbi_load_from_memory(): pass

def stbi_loadf_from_memory(): pass

def stbi_is_hdr_from_memory(): pass

def get8(): pass

def get8u(): pass

def skip(): pass

def get16(): pass

def get16le(): pass

def getn(): pass

def convert_format(): pass

def ldr_to_hdr(): pass

def hdr_to_ldr(): pass

def build_huffman(): pass

def grow_buffer_unsafe(): pass

def decode_block(): pass

def idct_block(): pass

def get_marker(): pass

def parse_entropy_coded_data(): pass

def process_marker(): pass

def process_scan_header(): pass

def process_frame_header(): pass

def decode_jpeg_header(): pass

def resample_v_2(): pass

def resample_hv_2(): pass

def cleanup_jpeg(): pass

def load_jpeg_image(): pass

def fill_bits(): pass

def expand(): pass

def parse_huffman_block(): pass

def compute_huffman_codes(): pass

def parse_zlib(): pass

def create_png_image(): pass

def parse_png_file(): pass

def do_png(): pass

def bmp_test(): pass

def bmp_load(): pass

def tga_test(): pass

def tga_load(): pass

def hdr_test(): pass

def hdr_gettoken(): pass

def hdr_load(): pass

def dds_test(): pass

def stbi_decode_DXT1_block(): pass

def stbi_decode_DXT45_alpha_block(): pass

def stbi_decode_DXT_color_block(): pass

def dds_load(): pass

def hkSingletonUtil__init_hkReferencedObjectLock_(): pass

def hkSingletonUtil__quit_hkReferencedObjectLock_(): pass

def set(): pass

class hkHash(ctypes.Structure):
    def appendCrc64(): pass

    def computeCrc64(): pass

    def computeFNV1Hash32(): pass

    def appendCrc32(): pass

    def computeCrc32(): pass


def _anonymous_namespace___dispatchMessage(): pass

class hkErrorFwd(ctypes.Structure):
    def messageAssert(): pass

    def messageError(): pass

    def messageFatalError(): pass

    def messageWarning(): pass

    def setEnabled(): pass


def _anonymous_namespace___ExternalAllocatorFwd__allocate(): pass

def _anonymous_namespace___ExternalAllocatorFwd__deallocate(): pass

def _anonymous_namespace___ExternalAllocatorFwd___scalar_deleting_destructor_(): pass

def _anonymous_namespace___BaseSystemOutput___scalar_deleting_destructor_(): pass

def _anonymous_namespace___BaseSystemOutput__flush(): pass

def _anonymous_namespace___BaseSystemOutput__put(): pass

def TypeName_appendToString(): pass

class hkPrintableArgs(ctypes.Structure):
    def convertFloatingFormatToPrintf(): pass

    def convertIntegerFormatToPrintf(): pass


def _anonymous_namespace___repeatFixupFunc(): pass

def _anonymous_namespace___runBinaryFunction_128_(): pass

def _anonymous_namespace___runUnaryFunction_64_(): pass

def _anonymous_namespace___RepeatWrapper_256___func(): pass

def _anonymous_namespace___RepeatWrapper_512___func(): pass

def _anonymous_namespace___RepeatWrapper_64___func(): pass

def _anonymous_namespace___RepeatWrapper_128___func(): pass

def _anonymous_namespace___runBinaryFunction(): pass

def _anonymous_namespace___runUnaryFunction(): pass

def hkSingletonUtil__init_hkReflect__Detail__DerivedDataCache_(): pass

def hkSingletonUtil__quit_ReflectStrings_(): pass

def _anonymous_namespace___CopyCallback___scalar_deleting_destructor_(): pass

def _anonymous_namespace___CopyCallback__atPointer(): pass

def _anonymous_namespace___s_fixupUnknownRecursive(): pass

def _anonymous_namespace___SpecialMethods__update(): pass

def _hkReflect__TypeBuilder__writeBodyOnBuffer____48___Alloc___scalar_deleting_destructor_(): pass

def _anonymous_namespace___destroyAttributes(): pass

def _hkReflect__TypeBuilder__writeBodyOnBuffer____48___Alloc__malloc(): pass

def _anonymous_namespace___writeCallable(): pass

class hkTupleDetail(ctypes.Structure):
    def reflectToString(): pass


def _anonymous_namespace___constructElements(): pass

def _anonymous_namespace___recursivelyAddRelArrays(): pass

def _anonymous_namespace___AssignVisitor__assign_hkReflect__ArrayVar_(): pass

def _anonymous_namespace___AssignVisitor__assign_hkReflect__BoolVar_(): pass

def _anonymous_namespace___AssignVisitor__assign_hkReflect__FloatVar_(): pass

def _anonymous_namespace___AssignVisitor__assign_hkReflect__IntVar_(): pass

def _anonymous_namespace___AssignVisitor__assign_hkReflect__PointerVar_(): pass

def _anonymous_namespace___AssignVisitor__assign_hkReflect__RecordVar_(): pass

def _anonymous_namespace___AssignVisitor__assign_hkReflect__StringVar_(): pass

def hkReflect__Detail__UnaryVarVisitor__anonymous_namespace___AssignVisitor_hkResult___dispatch(): pass

def hkReflect__Detail__UnaryVarVisitor__anonymous_namespace___VarFromStringVisitor_hkResult_hkStringView_const_____dispatch(): pass

def s_appendNameOf(): pass

def assign(): pass

def _anonymous_namespace___createPresetsObjectWithStorage(): pass

def _anonymous_namespace___doCreatePresetsObject(): pass

def _anonymous_namespace___parseAndAppendSingle(): pass

def parseAndAppendSinglePrintable(): pass

def parseSignedDecimal(): pass

def parseUnsignedDecimal(): pass

def _anonymous_namespace___ArgTokenizer__placeholder(): pass

def hkSingletonUtil__init_hkUuidPseudoRandomGenerator_(): pass

def div(): pass

def toString(): pass

class _lambda_03dd9449116675fe0eac1c14374bb014_(ctypes.Structure):
    def _lambda_invoker_cdecl_(): pass


def _anonymous_namespace___Win32ThreadImpl___scalar_deleting_destructor_(): pass

def SetThreadNameUsingException(): pass

def _anonymous_namespace___Win32ThreadImpl__start(): pass

def hkSingletonUtil__init_hkHardwareInfo_(): pass

def _lambda_1a0300289a578704c0d9f90bf9b16b7b____lambda_invoker_cdecl_(): pass

def hkSingletonUtil__init__anonymous_namespace___Registry_(): pass

class hkArrayUtil(ctypes.Structure):
    def checkReduce(): pass


def _anonymous_namespace___Registry___scalar_deleting_destructor_(): pass

def _anonymous_namespace___Registry__add(): pass

def _anonymous_namespace___Registry__reset(): pass

def _anonymous_namespace___s_hkLogRegistrySingletonQuit(): pass

def hkSingletonUtil__initWith_hkError_hkDefaultError_(): pass

def hkErrorMessage(): pass

def hkSingletonUtil__init_hkErrorRouter_(): pass

def _anonymous_namespace___flushDenormalsFunc(): pass

class hkSystemClock(ctypes.Structure):
    def getTickCounter(): pass

    def getTicksPerSecond(): pass


def hkSingletonUtil__initWith_hkFileSystem_hkWin32FileSystem_(): pass

def hkSocketCreate(): pass

def hkArrayBase__anonymous_namespace___IsaInfo____growCapacity_hkBuiltinContainerAllocatorBase_(): pass

def hkArrayBase_hkReflect__Detail___anonymous_namespace___Entry____growCapacity_hkContainerAllocator_(): pass

def hkAlgorithm__findIf_hkReflect__Type_____hkReflect__Type______lambda_9b246d3d78671d6201d8dccfea63d3a2___(): pass

def hkSingletonUtil__init_hkReflect__Detail__BuiltinTypeReg_(): pass

def hkAlgorithm__quickSortRecursive_int__lambda_ac245ec58bb9108a145d531de0e8c6d0___(): pass

def hkAlgorithm__quickSortRecursive_hkReflect__Detail___anonymous_namespace___Entry_hkReflect__Detail__A0x6f9412ec__CompareEntries_(): pass

def _anonymous_namespace___addRecursively(): pass

def _anonymous_namespace___buildPreOrderAndRightMost(): pass

def _anonymous_namespace___compareTemplateArgumentsRecursive(): pass

def _anonymous_namespace___extractRootAndArguments(): pass

def _anonymous_namespace___s_arrayToTree(): pass

def _anonymous_namespace___s_getKeyType(): pass

def _anonymous_namespace___s_rebuildOptIsA(): pass

def s_updateAlignment(): pass

def _anonymous_namespace___StackTraceContext__dump(): pass

def hkMath__checkPopCntSupport(): pass

def hkMath___anonymous_namespace___countBitsSetPopCnt(): pass

def _anonymous_namespace___AfterReflectNewLock___scalar_deleting_destructor_(): pass

def hkArrayBase__anonymous_namespace___Pair_hkReflect__PointerVar_hkReflect__PointerVar______growCapacity_hkContainerAllocator_(): pass

def hkArrayBase__anonymous_namespace___Pair_hkReflect__Var_hkReflect__Var______growCapacity_hkContainerAllocator_(): pass

def hkTypeVm__FastCopyInterpreter__execN__anonymous_namespace___CloneInterpreter_(): pass

def hkSingletonUtil__init__anonymous_namespace___AfterReflectNewLock_(): pass

def _anonymous_namespace___CloneInterpreter__CloneInterpreter(): pass

def _anonymous_namespace___Pair_hkReflect__PointerVar_hkReflect__PointerVar____Pair_hkReflect__PointerVar_hkReflect__PointerVar_(): pass

def _anonymous_namespace___clearAllocs(): pass

def hkReflect__Detail__UnaryVarVisitor___anonymous_namespace___clearAllocs____2___Visitor_void___dispatch(): pass

def _anonymous_namespace___CloneInterpreter__execString(): pass

def _anonymous_namespace___CloneInterpreter__execPointer(): pass

def hkReflect__Cloner__CloneImpl__handlePointers(): pass

def _anonymous_namespace___getUsedSizeOf(): pass

def _anonymous_namespace___recomputeFromDecls(): pass

class hkRelArrayUtil(ctypes.Structure):
    def hkRelArrayUtil(): pass

    def addRelArray(): pass

    def allocate(): pass


def s_typeFromValue(): pass

def _anonymous_namespace___findWithScoping(): pass

def _anonymous_namespace___isValidFieldName(): pass

def _anonymous_namespace___setField(): pass

def _anonymous_namespace___setFieldValuesArray(): pass

def _anonymous_namespace___splitStringWithScoping(): pass

def _anonymous_namespace___WideHelper_2___getMetrics(): pass

def _anonymous_namespace___nullTerminatedUtf8FromWide(): pass

def _anonymous_namespace___AsyncThreadPoolCriticalSection___scalar_deleting_destructor_(): pass

def _anonymous_namespace___SpinningCriticalSection___scalar_deleting_destructor_(): pass

def _anonymous_namespace___AsyncThreadPoolCriticalSection__enterImpl(): pass

def _anonymous_namespace___SpinningCriticalSection__enterImpl(): pass

def _anonymous_namespace___AsyncThreadPoolCriticalSection__leaveImpl(): pass

def _lambda_ada6f8a35dd7ca8f873495d480448a48___operator__(): pass

def _sortByAllocationAddress(): pass

class Win32FileSystemWatcher(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def getCompletedChanges(): pass

    def stopWatching(): pass

    def updatePendingCompletionQueue(): pass


def _anonymous_namespace___Win32Impl___scalar_deleting_destructor_(): pass

def _anonymous_namespace___Win32Impl__advance(): pass

def s_entryFromFindData(): pass

def refillBuffer(): pass

def hkBsdGetAddressString(): pass

def hkBsdNetworkInit(): pass

def _anonymous_namespace___CheckerImpl__checkArrayType(): pass

def _anonymous_namespace___CheckerImpl__checkDataFields(): pass

def _anonymous_namespace___CheckerImpl__checkFields(): pass

def _anonymous_namespace___CheckerImpl__checkFlaggedIfHasRelArrayField(): pass

def _anonymous_namespace___CheckerImpl__checkFlags(): pass

def _anonymous_namespace___CheckerImpl__checkInheritance(): pass

def _anonymous_namespace___CheckerImpl__checkInner(): pass

def _anonymous_namespace___CheckerImpl__checkLoops(): pass

def _anonymous_namespace___CheckerImpl__checkOuter(): pass

def _anonymous_namespace___CheckerImpl__checkParentLoop(): pass

def _anonymous_namespace___CheckerImpl__checkPointerType(): pass

def _anonymous_namespace___CheckerImpl__checkRecordParentIsRecord(): pass

def _anonymous_namespace___CheckerImpl__checkSingleParams(): pass

def _anonymous_namespace___CheckerImpl__checkSingleType(): pass

def _anonymous_namespace___CheckerImpl__checkSizeAndAlign(): pass

def _anonymous_namespace___CheckerImpl__checkStructure(): pass

def _anonymous_namespace___CheckerImpl__checkTemplateArgs(): pass

def _anonymous_namespace___CheckerImpl__doAddAndCheckTypes(): pass

def _anonymous_namespace___RAIIReachabilityHelper__s_isTypeReachable(): pass

def _anonymous_namespace___AggregateResult__setFailure(): pass

def _anonymous_namespace___CompilerVisitor__forEachField__lambda_036ed38e04333b1bc3121dc8e638b485___(): pass

def _anonymous_namespace___CompilerVisitor__forEachField__lambda_55025025aa6703e055e5f47ed76b3ab2___(): pass

def hkReflect__Detail__UnaryTypeVisitor__anonymous_namespace___CompilerVisitor_hkResult_int_const__hkReflect__Type_const___int_const____dispatch(): pass

def _anonymous_namespace___CompilerVisitor__dispatch(): pass

def _anonymous_namespace___CompilerVisitor__visit(): pass

def _anonymous_namespace___CompilerVisitor__visit_0(): pass

def _anonymous_namespace___CompilerVisitor__visit_1(): pass

def _anonymous_namespace___isKnownFloat(): pass

def _anonymous_namespace___isKnownIntOrBool(): pass

def _anonymous_namespace___hknpScaledConvexShapeScaleAndTransformCollector___scalar_deleting_destructor_(): pass

def _anonymous_namespace___hknpScaledConvexShapeScaleAndTransformCollector__addHit(): pass

def _anonymous_namespace___hknpScaledConvexShapeScaleAndTransformCollector__reset(): pass

def _lambda_3948ce011cf7c3c443a4c3c8cfdbce1b____lambda_invoker_cdecl_(): pass

def hkSingletonUtil__init_hknpShapeFunctionsSingleton_(): pass

def _anonymous_namespace____queryAabb(): pass

def _anonymous_namespace___reduceEdgesInFace(): pass

def hkSignal2_hknpShape_const___unsigned_char___subscribe__anonymous_namespace___hknpLodShapeEx_void____cdecl_A0xec73bb4d__hknpLodShapeEx_____hknpShape_const___unsigned_char__(): pass

def hkSignal3_hknpWorld___hknpBodyId_const___int___unsubscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int__(): pass

def hkSignal3_hknpWorld___hknpBodyId_const___int___MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int_____scalar_deleting_destructor_(): pass

def _anonymous_namespace___hknpLodShapeEx__calculateCaches(): pass

def _anonymous_namespace___hknpLodShapeEx__onLodVariantMutated(): pass

def hkSignal2_hknpShape_const___unsigned_char___MemberSlot__anonymous_namespace___hknpLodShapeEx_void____cdecl_A0xec73bb4d__hknpLodShapeEx_____hknpShape_const___unsigned_char____subscribe(): pass

def hkSignal3_hknpWorld___hknpBodyId_const___int___MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int____unsubscribe(): pass

def _anonymous_namespace___hkInertiaTensorComputer_combineMassPropertiesNoTransform(): pass

class hkcdTreeFilterUtils(ctypes.Structure):
    pass

def hkBridgeConstraintAtom_callData(): pass

def _anonymous_namespace___appendAtom_hknpBallSocketConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpAngConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpAngFrictionConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpTwistLimitConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpEllipticalLimitConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpLinConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpLinFrictionConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpLinMotorConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpRagdollMotorConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpSetLocalTransformsConstraintAtom_(): pass

def _anonymous_namespace___appendAtom_hknpSetupStabilizationAtom_(): pass

def _anonymous_namespace___appendAtom_hknpStiffSpringConstraintAtom_(): pass

def hknpUserLambdaCommand__lambda_0d3a4e86f97923466fea787ae3d73dc5_____execLambda(): pass

def hknpUserLambdaCommand__lambda_94fac512348532b4abcc3fdb7413b6ff_____execLambda(): pass

def hknpUserLambdaCommand__lambda_8943e5c84ff4daf098fbc45851b760d9_____execLambda(): pass

def hknpUserLambdaCommand__lambda_a8af8ddd2a5a8a85ab97ab2c8400592c_____execLambda(): pass

def _anonymous_namespace____buildShapeDisplayGeometriesRecursive(): pass

def _anonymous_namespace___acccumulateTetrahedron(): pass

def _anonymous_namespace___buildHullMassProperties_Impl(): pass

def getEdgeSdfContactPoints(): pass

class _(ctypes.Structure):
    def hknpHeightFieldShape_queryAabb_hkArray_hkHandle_unsigned_int_4294967295_hknpShapeKeyDiscriminant__hkBuiltinContainerAllocator_0_0__(): pass


def hknpHeightFieldShape_queryAabb_hkArray_hkHandle_unsigned_int_4294967295_hknpShapeKeyDiscriminant__hkBuiltinContainerAllocator_0_0____0_(): pass

def hknpHeightFieldShape_queryAabb_hknpCollisionQueryCollector_1_(): pass

def hknpHeightFieldShape_queryAabb_hknpCollisionQueryCollector_0_(): pass

def _anonymous_namespace___QuadTreeWalker__QuadTreeWalker(): pass

def hknpHeightFieldShape_calcClosestDistanceSquaredLowerEstimate(): pass

def hknpHeightFieldShape_castShape(): pass

def hknpHeightFieldShape_getClosestPoints(): pass

def hknpHeightFieldShape_getClosestPointsToHeightfield(): pass

def hknpCompoundShapeFunctions_castShapeImpl(): pass

class hknpShapeCastFuncs(ctypes.Structure):
    def castBoxQuad(): pass

    def castConvexConvexCalculateToi(): pass

    def castConvexConvexConservativeAdvancement(): pass

    def castConvexConvexGsk(): pass

    def castSphereQuad(): pass


class hknpClosestPointFuncs(ctypes.Structure):
    def compositeVsShape(): pass

    def compoundVsCompound(): pass

    def compoundVsShape(): pass

    def compressedMeshVsCompressedMesh(): pass

    def convexVsCompound(): pass

    def convexVsCompressedMesh(): pass

    def convexVsConvex(): pass

    def convexVsExternMesh(): pass

    def convexVsHeightField(): pass

    def convexVsMesh(): pass

    def convexVsParticleSystem(): pass

    def externMeshVsShape(): pass

    def heightFieldVsCompressedMesh(): pass

    def heightFieldVsHeightfield(): pass

    def particlesVsShape(): pass


def _anonymous_namespace___unwrapLodShape_1_(): pass

def _anonymous_namespace___unwrapLodShape_1__0(): pass

def _anonymous_namespace___unwrapLodShape_2_(): pass

def _anonymous_namespace___unwrapLodShape_2__0(): pass

def _anonymous_namespace___unwrapLodShape_0_(): pass

def _anonymous_namespace___unwrapLodShape_0__0(): pass

def _anonymous_namespace___unwrapMaskedShape_1_(): pass

def _anonymous_namespace___unwrapMaskedShape_1__0(): pass

def _anonymous_namespace___unwrapMaskedShape_2_(): pass

def _anonymous_namespace___unwrapMaskedShape_2__0(): pass

def _anonymous_namespace___unwrapMaskedShape_0_(): pass

def _anonymous_namespace___unwrapMaskedShape_0__0(): pass

def _anonymous_namespace___unwrapScaledConvex_1_(): pass

def _anonymous_namespace___unwrapScaledConvex_1__0(): pass

def _anonymous_namespace___unwrapScaledConvex_2_(): pass

def _anonymous_namespace___unwrapScaledConvex_2__0(): pass

def _anonymous_namespace___unwrapScaledConvex_0_(): pass

def _anonymous_namespace___unwrapScaledConvex_0__0(): pass

def _anonymous_namespace___unwrapTransformed_1_(): pass

def _anonymous_namespace___unwrapTransformed_1__0(): pass

def _anonymous_namespace___unwrapTransformed_2_(): pass

def _anonymous_namespace___unwrapTransformed_2__0(): pass

def _anonymous_namespace___unwrapTransformed_0_(): pass

def _anonymous_namespace___unwrapTransformed_0__0(): pass

class hknpSpatialQueryUtil(ctypes.Structure):
    def calcQueryToTargetTransformWithScale(): pass


def hkArrayBase__hknpWorldEx__updateMotionAndAttachedBodiesAfterModifyingTransform____2___BodyAabbAndId____growCapacity_hkBuiltinContainerAllocatorBase_(): pass

def hkSingletonUtil__init_hknpConstraintDataFunctionsSingleton_(): pass

def hkCalculateFrictionalEllipsoidScale(): pass

def hknpVehicleFrictionSolver_buildJacobian(): pass

def hknpVehicleFrictionApplyVehicleFriction(): pass

def hknpVehicleFrictionDescriptionInitValues(): pass

def hkArrayBase__anonymous_namespace___HalfEdge____growCapacity_hkBuiltinContainerAllocatorBase_(): pass

def _anonymous_namespace___calcSineSupportAngle(): pass

def _anonymous_namespace___hknpConvexHullEx__getSupportingFaceUsingConnectivity(): pass

def hkSingletonUtil__init_hknpGlobals_(): pass

def hknp1dAngularBilateralConstraintBuildJacobian(): pass

def hknp1dAngularBilateralConstraintUserTauBuildJacobian(): pass

def hknp1dLinearBilateralConstraintBuildJacobian(): pass

def hknp1dLinearBilateralConstraintUserTauBuildJacobian(): pass

def hknp1dLinearFrictionBuildJacobian(): pass

def hknp1dLinearLimitBuildJacobian(): pass

def hknpSetInvMassBuildJacobian(): pass

class hknpConvexShapeUtil(ctypes.Structure):
    def getClosestPointsBoxSphere(): pass

    def getClosestPointsCapsuleCapsule(): pass

    def getClosestPointsCapsuleQuad(): pass

    def getClosestPointsCapsuleSphere(): pass

    def getClosestPointsGsk(): pass

    def getClosestPointsSphereBox(): pass

    def getClosestPointsSphereCapsule(): pass

    def getClosestPointsSphereCvxPolytope(): pass

    def getClosestPointsSphereInBQuad(): pass

    def getClosestPointsSphereQuad(): pass

    def getClosestPointsSphereSphere(): pass

    def getSupportingFace(): pass


def _anonymous_namespace___addDirtyAabbToColliders(): pass

def shapeContainsMesh(): pass

def hkAlgorithm__explicitStackQuickSort_int__anonymous_namespace___IslandComparer_(): pass

def hkAlgorithm__explicitStackQuickSort_hknpDeactivatedIsland__InactiveCacheHeader____anonymous_namespace___CacheComparer_(): pass

def hkAlgorithm__quickSortRecursive_hknpConstraintAtomSolverSetup__ConstraintStateGroup__anonymous_namespace___ActivatedConstraintsLess_(): pass

def _anonymous_namespace___addCacheToIsland(): pass

def _anonymous_namespace___addCacheToOutputStream(): pass

def hknpSimpleSimplexSolver__hkSimpleSimplexSolverSolve2d(): pass

def hknpSimpleSimplexSolver__hkSimpleSimplexSolverSolve3d(): pass

class hknpSolverUtil(ctypes.Structure):
    def calculateProjectedPointVelocitiesUsingIntegratedVelocities(): pass

    def solveSt(): pass


def _anonymous_namespace____calcDragVelocities(): pass

def constructFlipped__hknpConvexCompositeCollisionCache__construct_(): pass

def constructFlipped__hknpSignedDistanceFieldCollisionCache__construct_(): pass

def _anonymous_namespace___unregisteredCacheCreator(): pass

def executeLambdaCommand(): pass

def hkAlgorithm__explicitStackQuickSort_hknpMotionWeldTOICommand_const_____anonymous_namespace___HighPriorityDispatcher__dispatchCommandsAndFinish____18____unnamed_type_ToiCommandSorter___(): pass

def _anonymous_namespace___InternalCommandConsumer___scalar_deleting_destructor_(): pass

def _anonymous_namespace___NonInternalCommandConsumer___scalar_deleting_destructor_(): pass

def _anonymous_namespace___HighPriorityDispatcher__dispatchCommandsAndFinish(): pass

def _anonymous_namespace___hknpSingleThreadedSimulation_collideInternal(): pass

def hknpDynamicSpaceSplitter_updateCenters(): pass

def _anonymous_namespace___hknpMultithreadedSimulationEx__finalizeSolving(): pass

def _anonymous_namespace___ShiftWorldTask__ShiftWorldTask(): pass

def _anonymous_namespace___ShiftBroadphaseTask___scalar_deleting_destructor_(): pass

def _thunk___anonymous_namespace___ShiftBroadphaseTask___scalar_deleting_destructor__adjustor_32____unsigned_int____ptr64(): pass

def _anonymous_namespace___ShiftWorldTask___scalar_deleting_destructor_(): pass

def _thunk___anonymous_namespace___ShiftWorldTask___scalar_deleting_destructor__adjustor_32____unsigned_int____ptr64(): pass

def _anonymous_namespace___FinalizeShiftWorldTask__getName(): pass

def _anonymous_namespace___ShiftBroadphaseTask__getName(): pass

def _anonymous_namespace___ShiftWorldTask__getName(): pass

def _anonymous_namespace___FinalizeShiftWorldTask__process(): pass

def _anonymous_namespace___ShiftBroadphaseTask__process(): pass

def _anonymous_namespace___ShiftWorldTask__process(): pass

def hknpAtomSolverFunctions___anonymous_namespace___estimateFutureTransforms(): pass

def hknpAtomSolverFunctions___anonymous_namespace___estimateFutureTransformsAndVelocities(): pass

def _anonymous_namespace___ExportAccumulators__use(): pass

def _anonymous_namespace___buildAndSolveLiveWeldingJacobianImpl_1_(): pass

def _anonymous_namespace___buildAndSolveLiveWeldingJacobianImpl_0_(): pass

def _anonymous_namespace___LiveWeldingJacobians___LiveWeldingJacobians(): pass

def hkSignal1_hknpWorld_____subscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld____(): pass

def hkSignal2_hknpWorld___hknpStepInput_const_____subscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpStepInput_const____(): pass

def hkSignal2_hknpWorld___hkVector4f_const_____subscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hkVector4f_const____(): pass

def hkSignal3_hknpWorld___hknpBodyId_const___int___subscribe__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int__(): pass

def _anonymous_namespace___hknpActionManagerEx__onBodiesDestroyed(): pass

def _anonymous_namespace___hknpActionManagerEx__onPreCollide(): pass

def _anonymous_namespace___hknpActionManagerEx__onWorldDestroyed(): pass

def _anonymous_namespace___hknpActionManagerEx__onWorldShifted(): pass

def hkSignal1_hknpWorld_____MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld______subscribe(): pass

def hkSignal2_hknpWorld___hknpStepInput_const_____MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpStepInput_const______subscribe(): pass

def hkSignal2_hknpWorld___hkVector4f_const_____MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hkVector4f_const______subscribe(): pass

def hkSignal3_hknpWorld___hknpBodyId_const___int___MemberSlot__anonymous_namespace___hknpActionManagerEx_void____cdecl_A0xf159be85__hknpActionManagerEx_____hknpWorld___hknpBodyId_const___int____subscribe(): pass

def _anonymous_namespace___hknpActionManagerEx__unsubscribeFromWorld(): pass

class hknpWorldUtil(ctypes.Structure):
    def addBroadPhaseDecorator(): pass


def hkRadixSort__sortImplLsb2Hsb_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts___(): pass

def hkFixedSizeBuffer_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts__hkFixedStackStorage_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts_______hkFixedSizeBuffer_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts__hkFixedStackStorage_hkRadixSort__SortDataT_unsigned_int__hknpConstraintAtomSolverSetup__SortedConstraintStates__init____2___TwoInts_____(): pass

class hknpParticleWorldQuery(ctypes.Structure):
    def hknpParticleCompoundClosest(): pass

    def hknpParticleCompressedMeshClosest(): pass

    def hknpParticleExternMeshClosest(): pass

    def hknpParticleHeightFieldClosest(): pass

    def hknpParticleMeshClosest(): pass

    def hknpParticleSDFClosest(): pass


def hknpSubIntegrateLastStepProcess(): pass

def hknpSubIntegrateProcess(): pass

class hknpCollisionCacheUtils(ctypes.Structure):
    def deepCopyJacobianCachePointer(): pass


def mergeAndCollide2StreamsIntern__anonymous_namespace___cacheConsumerWrapper_A0xc686054b__hknpCacheStarConsumers_1_A0xc686054b__hknpCollideTimersAccessor_(): pass

def mergeAndCollide2StreamsIntern__anonymous_namespace___cacheConsumerWrapper_A0xc686054b__hknpContigCacheConsumers_1_A0xc686054b__hknpCollideTimersAccessor_(): pass

def mergeAndCollide2StreamsIntern_hknpNarrowPhaseWorkStealingConsumer__anonymous_namespace___hknpContigCacheConsumers_0_hknpNarrowPhaseWorkStealingConsumer_(): pass

def buildJacobianForManifold(): pass

def weldManifoldArray(): pass

def hkArrayBase__anonymous_namespace___CellInfo____growCapacity_hkBuiltinContainerAllocatorBase_(): pass

def hkArrayBase__anonymous_namespace___CellInfo____growCapacity_hkContainerAllocator_(): pass

def hkAlgorithm__quickSortRecursive__anonymous_namespace___CellInfo_bool____cdecl___A0xa2df88c9__CellInfo_const___A0xa2df88c9__CellInfo_const____(): pass

def _anonymous_namespace___hasMoreElements(): pass

def hknpStepConstraintJacobianProcess(): pass

def iterateBuildGrid(): pass

def checkContactsForRestitution(): pass

def modifyCacheAndJacobian(): pass

def applySurfaceVelocity(): pass

def _anonymous_namespace____neighborWeld(): pass

def hknpMotionWeldingModifier_getClosestPoint(): pass

def hknpMotionWeldingModifier_getSupportingVertices(): pass

def hkAlgorithm__quickSortRecursive_int__lambda_e051ef60842e7194b0db346a423f79d0___(): pass

def hkAlgorithm__quickSortRecursive__anonymous_namespace___ManifoldSortKey_hk__less__anonymous_namespace___ManifoldSortKey_A0xb92a99ba__ManifoldSortKey___(): pass

def hknpConvexCompositeCollisionDetector_midphase(): pass

def buildSdfJacobians(): pass

def buildJacobiansAndRHSFromAllConstraints(): pass

def hknpDirectSolveConstraintGroup(): pass

def hknpAtomSolverFunctions__applyImpulses(): pass

def hknpAtomSolverFunctions__buildDenseMatrix(): pass

def hknpAtomSolverFunctions__buildJacobians(): pass

def hknpAtomSolverFunctions__doFixedNumberSORIteration(): pass

def hknpAtomSolverFunctions__factorizeA(): pass

def hknpAtomSolverFunctions__isSolutionOK(): pass

def hknpAtomSolverFunctions__solveLinearProblem(): pass

def hknpAtomSolverFunctions__solveMLCProblem(): pass

def _anonymous_namespace___getCircularSymmetryAxis(): pass

def _anonymous_namespace___getCircularSymmetryAxis_0(): pass

def _anonymous_namespace___transformAxisToMotionSpace(): pass

def _anonymous_namespace___extendQuad(): pass

def _anonymous_namespace___getLeaf(): pass

def _anonymous_namespace___matchTriangles(): pass

def _anonymous_namespace___selfWeldFace(): pass

def hkAlgorithm__quickSortRecursive_int__lambda_55914a8bdbcf0d0e7b07666b6c65a9bf___(): pass

def hkAlgorithm__quickSortRecursive_hknpLiveWelding__Hit__lambda_5670177da37b61afa559fa64146e27e3___(): pass

def _anonymous_namespace___recollideJacobiansImpl_1_(): pass

def _anonymous_namespace___recollideJacobiansImpl_0_(): pass

def _lambda_cd6e60e25087612c0053fd4bac4b1ba4___operator__(): pass

def NpJacobianBuilder__buildSchemaFromBallSocketAtomNewPhysics(): pass

def hknpSolverBuildJacobianFromAtomsNotContact(): pass

def hknpAddConstraintToDirectGroupSolver(): pass

def _anonymous_namespace____getClosestPoint(): pass

def _anonymous_namespace____getSupportingVertices(): pass

def getSignedDistances(): pass

def findConnectedConstraintsRecursively(): pass

def switchLambdaFromActiveToInactive(): pass

def hknp1dLinearVelocityMotorBeginJacobian(): pass

def hknp1dLinearVelocityMotorCommitJacobian(): pass

def hknpCalcMotorData(): pass

def hknpSingleConstraintDirectSolverSchemaSetup(): pass

def std___Med3_unchecked_cTkLocklessMultiPoolAllocator__sPoolConfig____cTkLocklessMultiPoolAllocator__CalcRequiredSize____2____lambda_1___(): pass

def std___Med3_unchecked_cTkLocklessMultiPoolAllocator__sPoolConfig____cTkLocklessMultiPoolAllocator__Init____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cTkLocklessMultiPoolAllocator__sPoolConfig____cTkLocklessMultiPoolAllocator__CalcRequiredSize____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_cTkLocklessMultiPoolAllocator__sPoolConfig____cTkLocklessMultiPoolAllocator__Init____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cTkLocklessMultiPoolAllocator__sPoolConfig___cTkLocklessMultiPoolAllocator__sPoolConfig__cTkLocklessMultiPoolAllocator__CalcRequiredSize____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_cTkLocklessMultiPoolAllocator__sPoolConfig___cTkLocklessMultiPoolAllocator__sPoolConfig__cTkLocklessMultiPoolAllocator__Init____2____lambda_1___(): pass

def std___Sort_unchecked_cTkLocklessMultiPoolAllocator__sPoolConfig____cTkLocklessMultiPoolAllocator__CalcRequiredSize____2____lambda_1___(): pass

def std___Sort_unchecked_cTkLocklessMultiPoolAllocator__sPoolConfig____cTkLocklessMultiPoolAllocator__Init____2____lambda_1___(): pass

def CreateLocalAllocator(): pass

def DestroyLocalAllocator(): pass

def DestroyLocalAllocatorAtEndOfThread(): pass

def DisableMainLocalAllocator(): pass

def EnableMainLocalAllocator(): pass

def GetLocalAlloc(): pass

def InitLocalAllocPerThreadData(): pass

def IsUsingMainLocalAllocator(): pass

def PopLocalAllocator(): pass

def PushLocalAllocator(): pass

def _cTkAttachmentManager__Construct____2____lambda_1____lambda_invoker_cdecl_(): pass

def _cTkComponentManager__Construct____2____lambda_1____lambda_invoker_cdecl_(): pass

def _cTkSketchComponent__RenderNGui____2____lambda_1____lambda_invoker_cdecl_(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkRotationComponent__IterateTypedComponentsOnNode__cTkRotationComponent__Prepare____2____lambda_1_______2____lambda_1___(): pass

def ___emplace_back_UsBranch_cTkSketchComponent_____vector_UsBranch_cTkSketchComponent__V__TkSTLAllocatorShim_UsBranch_cTkSketchComponent___07_0_0___std__QEAA_A_T__QEAUsBranch_cTkSketchComponent___Z(): pass

def _cTkRotationComponent__IterateTypedComponentsOnNode__cTkRotationComponent__Prepare____2____lambda_1_______2____lambda_1___operator__(): pass

def _cTkLSystemManager__GetLocatorNode____2____lambda_1___operator__(): pass

def _cTkSketchComponent__Prepare____3____lambda_1___operator__(): pass

def ColourPickerCallbackNoAlpha(): pass

def _cTkSketchComponent__sNode__DoErrorMessage____2___cErrorMessagePopup__Render(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GenerateStep____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GetLocatorNode____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__IterateMaterialsForSketch____2____lambda_1__void_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__Prepare____3____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__sNode__DoEditNodeName____5____lambda_1__void_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GenerateLSystemSampleByID____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__PollSurveyVariableValue____5____lambda_1__void_unsigned_char_const___unsigned_short____Move(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__RenderNGuiAddNodes____5____lambda_1__bool_cTkSketchComponent__sNodeType______Move(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__MakeLocatorEntry____13____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__MakeRule____13____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__RenderNGuiAddNodes____8____lambda_2__bool_cTkSketchComponent__sNodeType______Move(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GenerateStep____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GetLocatorNode____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__IterateMaterialsForSketch____2____lambda_1__void_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__Prepare____3____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__sNode__DoEditNodeName____5____lambda_1__void_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GenerateLSystemSampleByID____5____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__PollSurveyVariableValue____5____lambda_1__void_unsigned_char_const___unsigned_short____Do_call(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__RenderNGuiAddNodes____5____lambda_1__bool_cTkSketchComponent__sNodeType______Do_call(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__MakeRule____13____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__RenderNGuiAddNodes____8____lambda_2__bool_cTkSketchComponent__sNodeType______Do_call(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GenerateStep____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GetLocatorNode____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__IterateMaterialsForSketch____2____lambda_1__void_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__Prepare____3____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__sNode__DoEditNodeName____5____lambda_1__void_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__GenerateLSystemSampleByID____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__PollSurveyVariableValue____5____lambda_1__void_unsigned_char_const___unsigned_short____Target_type(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__RenderNGuiAddNodes____5____lambda_1__bool_cTkSketchComponent__sNodeType______Target_type(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__MakeLocatorEntry____13____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cTkLSystemManager__MakeRule____13____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cTkSketchComponent__RenderNGuiAddNodes____8____lambda_2__bool_cTkSketchComponent__sNodeType______Target_type(): pass

def _cTkPhysicsManager__CastShape____2___cTkAnyHitCollectorBackface__cTkAnyHitCollectorBackface(): pass

def _cTkPhysicsManager__CastShape____2___cTkClosestHitCollectorBackface__cTkClosestHitCollectorBackface(): pass

def hkConcurrencyInternals__BatchProcessor__cTkPhysicsManager__OnPostSolveSignal____2____lambda_1______scalar_deleting_destructor_(): pass

def _cTkPhysicsManager__CastShape____2___cTkClosestHitCollectorBackface___scalar_deleting_destructor_(): pass

def CollectVerticesRecursive(): pass

def HavokWorkerThread(): pass

def RaycastJobThreadFunc(): pass

def TkBodyActivationEvent(): pass

def TkBodyDeactivationEvent(): pass

def TkBroadphaseExitFunction(): pass

def TkContactCollectorFunction(): pass

def TkTriggerHandler(): pass

def _cTkPhysicsManager__CastShape____2___cTkAnyHitCollectorBackface__addHit(): pass

def _cTkPhysicsManager__CastShape____2___cTkClosestHitCollectorBackface__addHit(): pass

def freeForHavok(): pass

def mallocForHavok(): pass

def hkConcurrencyInternals__BatchProcessor__cTkPhysicsManager__OnPostSolveSignal____2____lambda_1_____process(): pass

def std___Guess_median_unchecked_std___List_iterator_std___List_val_std___List_simple_types_std__pair_cTkResource___const_cTkResourceManager__cTkKillResource___________cTkResourceManager__PollToReleaseUnusedResources____2____lambda_1___(): pass

def std___Sort_unchecked_std___List_iterator_std___List_val_std___List_simple_types_std__pair_cTkResource___const_cTkResourceManager__cTkKillResource___________cTkResourceManager__PollToReleaseUnusedResources____2____lambda_1___(): pass

def _cTkNodeNetworkAddress__LookupNode____5____lambda_1___operator__(): pass

def CapturedNoise1D(): pass

def CapturedNoise3D(): pass

class Wombat(ctypes.Structure):
    def Cellular3D_Deriv(): pass

    def Value3D(): pass


def GetNullSelectionResDescriptor(): pass

def std___Func_impl_no_alloc__cTkNodeNetworkAddress__LookupNode____5____lambda_1__void_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkNodeNetworkAddress__LookupNode____5____lambda_1__void_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkNodeNetworkAddress__LookupNode____5____lambda_1__void_TkHandle____Target_type(): pass

def Sha256_Final(): pass

def Sha256_Init(): pass

def Sha256_Update(): pass

def Sha256_WriteByteBlock(): pass

def std__vector__cTkBvMesh__Build____43___cBufferSizes_StackAllocator__cTkBvMesh__Build____43___cBufferSizes_1024_0______Emplace_reallocate__cTkBvMesh__Build____43___cBufferSizes_(): pass

def std___Insertion_sort_unchecked_Tk__cDistanceConstraint____Tk__cNapkin__SortConstraints____2____lambda_1___(): pass

def std___Med3_unchecked_Tk__cDistanceConstraint____Tk__cNapkin__SortConstraints____2____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_Tk__cDistanceConstraint____Tk__cNapkin__SortConstraints____2____lambda_1___(): pass

def std___Pop_heap_hole_by_index_Tk__cDistanceConstraint___Tk__cDistanceConstraint__Tk__cNapkin__SortConstraints____2____lambda_1___(): pass

def std___Sort_unchecked_Tk__cDistanceConstraint____Tk__cNapkin__SortConstraints____2____lambda_1___(): pass

def _Tk__cNapkin__SortConstraints____2____lambda_1___operator__(): pass

def BestAlignBasisAroundY(): pass

def CalcBb(): pass

def GetStrHash(): pass

def IsCapsuleOrSphere(): pass

def MakeScaledMeshShape(): pass

def std__vector__cTkBvMesh__Build____43___cBufferSizes_StackAllocator__cTkBvMesh__Build____43___cBufferSizes_1024_0______Change_array(): pass

def std___Func_impl_no_alloc__cTkCollision__AddFromNode____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkNetReplicatedEntityComponent__OnAttached____2____lambda_1__void_bool_const___bool_const______Move(): pass

def std___Func_impl_no_alloc__cTkCollision__AddFromNodeStep____2____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkCollision__AddFromNodeStep____2____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkCollision__AddFromNodeStep____2____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkNetReplicatedEntityComponent__OnAttached____2____lambda_1__void_bool_const___bool_const______Do_call(): pass

def std___Func_impl_no_alloc__cTkCollision__AddFromNodeStep____2____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkCollision__AddFromNode____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkNetReplicatedEntityComponent__OnAttached____2____lambda_1__void_bool_const___bool_const______Target_type(): pass

def std___Func_impl_no_alloc__cTkCollision__AddFromNodeStep____2____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkCollision__AddFromNodeStep____2____lambda_3__bool_TkHandle____Target_type(): pass

def std__vector__cTkBvMesh__Build____43___cBufferSizes_StackAllocator__cTkBvMesh__Build____43___cBufferSizes_1024_0______Xlength(): pass

def std__vector__cTkBvMesh__Build____43___cBufferSizes_StackAllocator__cTkBvMesh__Build____43___cBufferSizes_1024_0_____reserve(): pass

def Tk__spline(): pass

def Tk__splint(): pass

def _cTkAnimPoseComponent__RenderNGui____61____lambda_1___operator__(): pass

def _cTkAnimationComponent__RenderNGui____83____lambda_1___operator__(): pass

def _cTkAnimationComponent__RenderNGui____83____lambda_3___operator__(): pass

def SwapMatrixCols(): pass

def std___Func_impl_no_alloc__cTkAnimationComponent__RenderNGui____83____lambda_2__char_const___cTkAnimTreeBuilder__BlendInfo___int____Move(): pass

def std___Func_impl_no_alloc__cTkAnimationComponent__RenderNGui____83____lambda_3__void_cTkAnimBlendHandle___cTkAnimTreeBuilder__BlendInfo______Copy(): pass

def std___Func_impl_no_alloc__cTkAnimationComponent__RenderNGui____83____lambda_2__char_const___cTkAnimTreeBuilder__BlendInfo___int____Do_call(): pass

def std___Func_impl_no_alloc__cTkAnimationComponent__RenderNGui____83____lambda_3__void_cTkAnimBlendHandle___cTkAnimTreeBuilder__BlendInfo______Do_call(): pass

def std___Func_impl_no_alloc__cTkAnimationComponent__RenderNGui____83____lambda_2__char_const___cTkAnimTreeBuilder__BlendInfo___int____Target_type(): pass

def std___Func_impl_no_alloc__cTkAnimationComponent__RenderNGui____83____lambda_3__void_cTkAnimBlendHandle___cTkAnimTreeBuilder__BlendInfo______Target_type(): pass

def CheckReturnFiniteTkHalfVector4(): pass

def CheckReturnFiniteTkVector4(): pass

def DigitGen(): pass

def Grisu2(): pass

def Prettify(): pass

def WriteExponent(): pass

def xxtea_to_ubyte_array(): pass

def xxtea_decrypt(): pass

def _cTkSMShaderWindow__RenderEditorWindow____19____lambda_2___operator__(): pass

def _cTkModelResourceRenderer__Update____91____lambda_4___operator__(): pass

def BounceEaseOut(): pass

def BounceEaseOutIntegral(): pass

def std___Func_impl_no_alloc___cTkSMShaderWindow__RenderEditorWindow____5____lambda_1___operator______2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____5____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____5____lambda_1__bool_TkHandle____Move_0(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____11____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____7____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____7____lambda_2__bool_TkHandle____Move_0(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____27____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____12____lambda_2__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____27____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____48____lambda_3__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____33____lambda_4__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____91____lambda_4__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____36____lambda_5__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkSMShaderWindow__RenderEditorWindow____5____lambda_1____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkSMShaderWindow__RenderEditorWindow____19____lambda_2____void____Move(): pass

def std___Func_impl_no_alloc___cTkSMShaderWindow__RenderEditorWindow____5____lambda_1___operator______2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____7____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____7____lambda_2__bool_TkHandle____Do_call_0(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____27____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____12____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____27____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____48____lambda_3__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____33____lambda_4__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____91____lambda_4__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____36____lambda_5__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkSMShaderWindow__RenderEditorWindow____5____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkSMShaderWindow__RenderEditorWindow____19____lambda_2____void____Do_call(): pass

def std___Func_impl_no_alloc___cTkSMShaderWindow__RenderEditorWindow____5____lambda_1___operator______2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____5____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____5____lambda_1__bool_TkHandle____Target_type_0(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____11____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____7____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelDressroom__DressMaterial____7____lambda_2__bool_TkHandle____Target_type_0(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____27____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____12____lambda_2__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____27____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____48____lambda_3__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____33____lambda_4__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__Update____91____lambda_4__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkModelResourceRenderer__EnqueueRenderNodeToTexture____36____lambda_5__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkSMShaderWindow__RenderEditorWindow____5____lambda_1____void____Target_type(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkSMShaderWindow__RenderEditorWindow____19____lambda_2____void____Target_type(): pass

class sTkInputStringHelperFns(ctypes.Structure):
    def CountKeywords(): pass


def cTkPageAllocator__AllocPages__TextureTiler__AllocAndMapPages____2____lambda_1___(): pass

def cTkPageAllocator__AllocPages__TextureTiler__AllocAndMapPages____2____lambda_1____0(): pass

def _cTkNGuiNodeGraph__AnimateActiveNodes____2____lambda_1___operator__(): pass

def _cTk2dRenderer__RenderQuads____2____lambda_1___operator__(): pass

def GetLinearRowSlicePtch(): pass

def GetResourceTilingInfo(): pass

def GetStandardBlockSize2D(): pass

def STB_TEXTEDIT_LAYOUTROW(): pass

def std___Func_impl_no_alloc__cTkNGuiNodeGraph__AnimateOutputActive____2____lambda_1__void_cTkNGuiNodeGraph__Connector______Move(): pass

def std___Func_impl_no_alloc__cTkNGuiNodeGraph__DoLink____2____lambda_1__void_cTkNGuiNodeGraph__Connector______Move(): pass

def std___Func_impl_no_alloc__cTkNGuiNodeGraph__DoLink____2____lambda_2__void_cTkNGuiNodeGraph__Connector______Move(): pass

def std___Func_impl_no_alloc__cTkNGuiNodeGraph__DoLink____2____lambda_2__void_cTkNGuiNodeGraph__Connector______Do_call(): pass

def std___Func_impl_no_alloc__cTkNGuiNodeGraph__AnimateOutputActive____2____lambda_1__void_cTkNGuiNodeGraph__Connector______Target_type(): pass

def std___Func_impl_no_alloc__cTkNGuiNodeGraph__DoLink____2____lambda_1__void_cTkNGuiNodeGraph__Connector______Target_type(): pass

def std___Func_impl_no_alloc__cTkNGuiNodeGraph__DoLink____2____lambda_2__void_cTkNGuiNodeGraph__Connector______Target_type(): pass

def stb_text_create_undo_record(): pass

def stb_text_createundo(): pass

def stb_text_locate_coord(): pass

def stb_text_redo(): pass

def stb_text_undo(): pass

def stb_textedit_clamp(): pass

def stb_textedit_delete(): pass

def stb_textedit_delete_selection(): pass

def stb_textedit_find_charpos(): pass

def stb_textedit_key(): pass

def stb_textedit_move_to_word_next(): pass

def stb_textedit_move_to_word_previous(): pass

def HashShaderInputs(): pass

def ReflectionRecurseShaderTypeMembers(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkPhysicsComponent__IterateTypedComponentsOnNode__cTkEngineUtils__AddNodesNoPrepare____10____lambda_1_______2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkEngineUtils__SetNodeParent____2____lambda_1___(): pass

def cTkAttachmentManager__IterateAttachmentsInScene__cTkEngineUtils__AddNodesStep____7____lambda_1___(): pass

class cTkEngineUtils(ctypes.Structure):
    pass

def ChangeVsync(): pass

def GetSettingBool(): pass

def GetSettingFloat(): pass

def GetSettingGraphicDetail(): pass

def GetSettingInt(): pass

def GetSettingSupport(): pass

def RefreshAniso(): pass

def RefreshDLSSQuality(): pass

def RefreshFFXSR2Quality(): pass

def RefreshFFXSRQuality(): pass

def RefreshLodBiasAndAA(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__FindNode____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__FindNode____2____lambda_1__bool_TkHandle____Move_0(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__FindNodeContainingString____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__RequestTexturesAlwaysResident____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__FindNode____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__FindNodeContainingString____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__RequestTexturesAlwaysResident____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__FindNode____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__FindNode____2____lambda_1__bool_TkHandle____Target_type_0(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__FindNodeContainingString____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkEngineUtils__RequestTexturesAlwaysResident____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Buffered_inplace_merge_divide_and_conquer2_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Buffered_inplace_merge_divide_and_conquer_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Buffered_inplace_merge_unchecked_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Buffered_inplace_merge_unchecked_impl_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Buffered_merge_sort_unchecked_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Chunked_merge_unchecked_int___int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Inplace_merge_buffer_right_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Insertion_sort_isort_max_chunks_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Insertion_sort_unchecked_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Stable_sort_unchecked_int___std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

def std___Uninitialized_chunked_merge_unchecked2_int___int_std___Ref_fn__cTkJobManager__Init____2____lambda_1_____(): pass

class TkFiber(ctypes.Structure):
    def ConvertFiberToThread(): pass

    def ConvertThreadToFiber(): pass


def JobResumeWrapper(): pass

def QueueSwitchFunction(): pass

def SceTpfsFree(): pass

def SceTpfsMalloc(): pass

def SceTpfsRealloc(): pass

def SuspendFiberWrapper(): pass

def SuspendableJobWrapper(): pass

def std___Func_impl_no_alloc__cTkUserService__RequestUserDisplayName____2____lambda_1__void_cTkFixedString_64_char__const______Copy(): pass

def std___Func_impl_no_alloc__cTkUserService__GetOrRequestUserDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Move(): pass

def std___Func_impl_no_alloc__cTkUserService__RequestUserDisplayName____2____lambda_1__void_cTkFixedString_64_char__const______Do_call(): pass

def std___Func_impl_no_alloc__cTkUserService__RequestUserDisplayName____2____lambda_1__void_cTkFixedString_64_char__const______Target_type(): pass

def std___Func_impl_no_alloc__cTkUserService__GetOrRequestUserDisplayName____11____lambda_1__void_cTkFixedString_64_char__const______Target_type(): pass

def CreateHgVocAlien(): pass

def CreateHgVocAlienParams(): pass

def HGOutputDebugString(): pass

def HandleAkExecutionSuspension(): pass

def StorageSlotSortNewestFirst(): pass

def StorageSlotSortOldestFirst(): pass

def std___Func_impl_no_alloc__cTkSystemBase__BeginPollableTaskBatch____2____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cTkSystemBase__EndPollableTaskBatch____2____lambda_1__enum_TkPollResult____Move(): pass

def std___Func_impl_no_alloc__cTkSystemBase__BeginPollableTaskBatch____2____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cTkSystemBase__EndPollableTaskBatch____2____lambda_1__enum_TkPollResult____Do_call(): pass

def std___Func_impl_no_alloc__cTkSystemBase__BeginPollableTaskBatch____2____lambda_1__enum_TkPollResult____Target_type(): pass

def std___Func_impl_no_alloc__cTkSystemBase__EndPollableTaskBatch____2____lambda_1__enum_TkPollResult____Target_type(): pass

def CopyMiniStateToFullState(): pass

def _cTkJobManager__CreateJobLambda__cTkInputDeviceManager__DispatchDetectionTask____2____lambda_1_______2____lambda_1____lambda_invoker_cdecl_(): pass

def _cTkJobManager__CreateJobLambda__cTkInputDeviceManager__DispatchDetectionTask____2____lambda_1_______2____lambda_1___operator__(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkInputDeviceManager__DispatchDetectionTask____2____lambda_1____void____Move(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkInputDeviceManager__DispatchDetectionTask____2____lambda_1____void____Do_call(): pass

def std___Func_impl_no_alloc_std___Binder_std___Unforced__cTkInputDeviceManager__DispatchDetectionTask____2____lambda_1____void____Target_type(): pass

class OpenVRRumble(ctypes.Structure):
    def RegisterVibrationAction(): pass


def RotateMatrix44(): pass

def WaitPosesJob(): pass

def AssertDialogMsgProc(): pass

def GetFilenameForSlot(): pass

def IsResolutionALessThanResolutionB(): pass

def PostSlackAssert(): pass

def PurecallHandler(): pass

def ShowAssertDialog(): pass

def _snwprintf(): pass

class VmaAllocator_T(ctypes.Structure):
    def VmaAllocator_T(): pass

    def AllocateDedicatedMemory(): pass

    def AllocateMemory(): pass

    def AllocateMemoryOfType(): pass

    def AllocateVulkanMemory(): pass

    def BindBufferMemory(): pass

    def BindImageMemory(): pass

    def CreatePool(): pass

    def FreeDedicatedMemory(): pass

    def FreeMemory(): pass

    def FreeVulkanMemory(): pass

    def GetAllocationInfo(): pass

    def Init(): pass

    def Map(): pass

    def Unmap(): pass


class VmaBlockVector(ctypes.Structure):
    def tilde_VmaBlockVector(): pass

    def Allocate(): pass

    def AllocateFromBlock(): pass

    def AllocatePage(): pass

    def CreateBlock(): pass

    def Free(): pass


def _cTkGraphicsAPI__InitVideoSubsystem____2____lambda_1___operator__(): pass

class VmaBlockMetadata(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def PrintDetailedMap_Allocation(): pass

    def PrintDetailedMap_Begin(): pass

    def PrintDetailedMap_End(): pass

    def PrintDetailedMap_UnusedRange(): pass


class VmaBlockMetadata_Buddy(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def AddPoolStats(): pass

    def Alloc(): pass

    def CalcAllocationStatInfo(): pass

    def CalcAllocationStatInfoNode(): pass

    def CheckCorruption(): pass

    def CreateAllocationRequest(): pass

    def DeleteNode(): pass

    def Free(): pass

    def FreeAtOffset(): pass

    def GetAllocationCount(): pass

    def GetSumFreeSize(): pass

    def GetUnusedRangeSizeMax(): pass

    def Init(): pass

    def IsEmpty(): pass

    def MakeRequestedAllocationsLost(): pass

    def PrintDetailedMap(): pass

    def PrintDetailedMapNode(): pass

    def Validate(): pass

    def ValidateNode(): pass


class VmaBlockMetadata_Generic(ctypes.Structure):
    def vector_deleting_destructor(): pass

    def AddPoolStats(): pass

    def Alloc(): pass

    def CalcAllocationStatInfo(): pass

    def CheckAllocation(): pass

    def CreateAllocationRequest(): pass

    def Free(): pass

    def FreeAtOffset(): pass

    def FreeSuballocation(): pass

    def GetAllocationCount(): pass

    def GetUnusedRangeSizeMax(): pass

    def Init(): pass

    def IsEmpty(): pass

    def MakeAllocationsLost(): pass

    def MakeRequestedAllocationsLost(): pass

    def PrintDetailedMap(): pass

    def RegisterFreeSuballocation(): pass

    def ResizeAllocation(): pass

    def UnregisterFreeSuballocation(): pass

    def Validate(): pass

    def ValidateFreeSuballocationList(): pass


class VmaStringBuilder(ctypes.Structure):
    def AddNumber(): pass


class VmaJsonWriter(ctypes.Structure):
    def BeginArray(): pass

    def BeginObject(): pass

    def BeginValue(): pass

    def ContinueString(): pass

    def WriteIndent(): pass

    def WriteString(): pass


class VmaDeviceMemoryBlock(ctypes.Structure):
    def Destroy(): pass

    def Init(): pass


def DeviceIsUsable(): pass

def FindValidAllocator(): pass

def GetStreamingMemoryUsed(): pass

def Halton(): pass

def LoadIridasLut(): pass

def MemoryPanic(): pass

def NGX_DLSS_GET_OPTIMAL_SETTINGS(): pass

def NGX_VULKAN_CREATE_DLSS_EXT(): pass

def NGX_VULKAN_EVALUATE_DLSS_EXT(): pass

class VmaAllocation_T(ctypes.Structure):
    def PrintParameters(): pass

    def SetUserData(): pass


def SetShaderConsts(): pass

def TkDebugReportCallback(): pass

def VmaIsBufferImageGranularityConflict(): pass

class VmaRecorder(ctypes.Structure):
    def WriteConfiguration(): pass


def vmaAllocateMemoryForBuffer(): pass

def vmaAllocateMemoryForImage(): pass

def vmaCreatePool(): pass

def vmaFindMemoryTypeIndex(): pass

def vmaMapMemory(): pass

def vmaUnmapMemory(): pass

def std___Insertion_sort_unchecked_int____FindWindEigenvectorsJob____24____lambda_1___(): pass

def std___Make_heap_unchecked_int____FindWindEigenvectorsJob____24____lambda_1___(): pass

def std___Med3_unchecked_int____FindWindEigenvectorsJob____24____lambda_1___(): pass

def std___Partition_by_median_guess_unchecked_int____FindWindEigenvectorsJob____24____lambda_1___(): pass

def std___Pop_heap_hole_by_index_int___int__FindWindEigenvectorsJob____24____lambda_1___(): pass

def std___Sort_unchecked_int____FindWindEigenvectorsJob____24____lambda_1___(): pass

class TkCastShapeFuncs(ctypes.Structure):
    pass

class TkClosestPointFuncs(ctypes.Structure):
    pass

def ___outer_product_selector_run_V__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_00_Eigen___0_0_0_0_0A__Eigen__V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Map_V__Matrix_M_0_0_00_0A__0_0_00_Eigen___0A_V__Stride__0A__0A__2__3__2_V__Transpose___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__Eigen___2_Usub___generic_product_impl_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Map_V__Matrix_M_0_0_00_0A__0_0_00_Eigen___0A_V__Stride__0A__0A__2__3__Eigen__V__Transpose___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__Eigen___2_UDenseShape_2_U42__04_internal_2__internal_Eigen__YAXAEAV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_00_Eigen___0_0_0_0_0A__1_AEBV__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Map_V__Matrix_M_0_0_00_0A__0_0_00_Eigen___0A_V__Stride__0A__0A__2__3__1_AEBV__Transpose___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__Eigen___1_AEBUsub___generic_product_impl_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Map_V__Matrix_M_0_0_00_0A__0_0_00_Eigen___0A_V__Stride__0A__0A__2__3__Eigen__V__Transpose___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__Eigen___2_UDenseShape_2_U42__04_01_AEBUfalse_type_01__Z(): pass

def ___outer_product_selector_run_V__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen__V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Block___CBV__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_0A__3__2_V__Map_V__Matrix_M_00_0_0_00_00_0_0_Eigen___0A_V__Stride__0A__0A__2__2_Usub___generic_product_impl_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Block___CBV__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_0A__3__Eigen__V__Map_V__Matrix_M_00_0_0_00_00_0_0_Eigen___0A_V__Stride__0A__0A__2__2_UDenseShape_2_U42__04_internal_2__internal_Eigen__YAXAEAV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__1_AEBV__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Block___CBV__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_0A__3__1_AEBV__Map_V__Matrix_M_00_0_0_00_00_0_0_Eigen___0A_V__Stride__0A__0A__2__1_AEBUsub___generic_product_impl_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Block___CBV__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_0A__3__Eigen__V__Map_V__Matrix_M_00_0_0_00_00_0_0_Eigen___0A_V__Stride__0A__0A__2__2_UDenseShape_2_U42__04_01_AEBUfalse_type_01__Z(): pass

def ___outer_product_selector_run_V__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen__V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__3__2_V__Map_V__Matrix_M_00_0_0_00_00_0_0_Eigen___0A_V__Stride__0A__0A__2__2_Usub___generic_product_impl_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__3__Eigen__V__Map_V__Matrix_M_00_0_0_00_00_0_0_Eigen___0A_V__Stride__0A__0A__2__2_UDenseShape_2_U42__04_internal_2__internal_Eigen__YAXAEAV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__1_AEBV__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__3__1_AEBV__Map_V__Matrix_M_00_0_0_00_00_0_0_Eigen___0A_V__Stride__0A__0A__2__1_AEBUsub___generic_product_impl_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__CwiseNullaryOp_U__scalar_constant_op_M_internal_Eigen____CBV__Matrix_M_0_0_00_0A__0_0_00_3__3___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__3__Eigen__V__Map_V__Matrix_M_00_0_0_00_00_0_0_Eigen___0A_V__Stride__0A__0A__2__2_UDenseShape_2_U42__04_01_AEBUfalse_type_01__Z(): pass

def ___run_V__CwiseBinaryOp_U__scalar_conj_product_op_MM_internal_Eigen____CBV__Transpose___CBV__Block___CBV__Transpose___CBV__Block___CBV__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_0A__Eigen___Eigen___00_0_0_00_Eigen___3___CBV__Block___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen___0_0_00_00_3__Eigen_____redux_impl_U__scalar_sum_op_MM_internal_Eigen__V__redux_evaluator_V__CwiseBinaryOp_U__scalar_conj_product_op_MM_internal_Eigen____CBV__Transpose___CBV__Block___CBV__Transpose___CBV__Block___CBV__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_0A__Eigen___Eigen___00_0_0_00_Eigen___3___CBV__Block___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen___0_0_00_00_3__Eigen___23__02_0A__internal_Eigen__SAMAEBV__redux_evaluator_V__CwiseBinaryOp_U__scalar_conj_product_op_MM_internal_Eigen____CBV__Transpose___CBV__Block___CBV__Transpose___CBV__Block___CBV__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_0A__Eigen___Eigen___00_0_0_00_Eigen___3___CBV__Block___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen___0_0_00_00_3__Eigen___12_AEBU__scalar_sum_op_MM_12_AEBV__CwiseBinaryOp_U__scalar_conj_product_op_MM_internal_Eigen____CBV__Transpose___CBV__Block___CBV__Transpose___CBV__Block___CBV__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_0A__Eigen___Eigen___00_0_0_00_Eigen___3___CBV__Block___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen___0_0_00_00_3__2__Z(): pass

def ___run_V__CwiseBinaryOp_U__scalar_conj_product_op_MM_internal_Eigen____CBV__Transpose___CBV__Block___CBV__Transpose___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__Eigen___Eigen___00_0_0_00_Eigen___3___CBV__Block___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen___0_0_00_00_3__Eigen_____redux_impl_U__scalar_sum_op_MM_internal_Eigen__V__redux_evaluator_V__CwiseBinaryOp_U__scalar_conj_product_op_MM_internal_Eigen____CBV__Transpose___CBV__Block___CBV__Transpose___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__Eigen___Eigen___00_0_0_00_Eigen___3___CBV__Block___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen___0_0_00_00_3__Eigen___23__02_0A__internal_Eigen__SAMAEBV__redux_evaluator_V__CwiseBinaryOp_U__scalar_conj_product_op_MM_internal_Eigen____CBV__Transpose___CBV__Block___CBV__Transpose___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__Eigen___Eigen___00_0_0_00_Eigen___3___CBV__Block___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen___0_0_00_00_3__Eigen___12_AEBU__scalar_sum_op_MM_12_AEBV__CwiseBinaryOp_U__scalar_conj_product_op_MM_internal_Eigen____CBV__Transpose___CBV__Block___CBV__Transpose___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_00_00_Eigen___0_0_00_0A__Eigen___Eigen___00_0_0_00_Eigen___3___CBV__Block___CBV__Block_V__Block_V__Matrix_M_0_0_0_0_0A__0_0_0_0_Eigen___0_0_0_0_0A__Eigen___0_0_0_0_0A__Eigen___0_0_00_00_3__2__Z(): pass

def ___run_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__Block___CBV__Block___CBV__Map___CBV__Matrix_M_0_0_0_0_00_0_0_0_0_Eigen___0A_V__OuterStride__0_0_2__Eigen___00_0_0_00_Eigen___00_0_0_0A__3___CBV__Transpose___CBV__Block___CBV__Map___CBV__Matrix_M_0_0_00_0A__0_0_00_Eigen___0A_V__Stride__0A__0A__2__Eigen___0_0_00_0A__Eigen___3__Eigen_____redux_impl_U__scalar_sum_op_MM_internal_Eigen__V__redux_evaluator_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__Block___CBV__Block___CBV__Map___CBV__Matrix_M_0_0_0_0_00_0_0_0_0_Eigen___0A_V__OuterStride__0_0_2__Eigen___00_0_0_00_Eigen___00_0_0_0A__3___CBV__Transpose___CBV__Block___CBV__Map___CBV__Matrix_M_0_0_00_0A__0_0_00_Eigen___0A_V__Stride__0A__0A__2__Eigen___0_0_00_0A__Eigen___3__Eigen___23__02_0A__internal_Eigen__SAMAEBV__redux_evaluator_V__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__Block___CBV__Block___CBV__Map___CBV__Matrix_M_0_0_0_0_00_0_0_0_0_Eigen___0A_V__OuterStride__0_0_2__Eigen___00_0_0_00_Eigen___00_0_0_0A__3___CBV__Transpose___CBV__Block___CBV__Map___CBV__Matrix_M_0_0_00_0A__0_0_00_Eigen___0A_V__Stride__0A__0A__2__Eigen___0_0_00_0A__Eigen___3__Eigen___12_AEBU__scalar_sum_op_MM_12_AEBV__CwiseBinaryOp_U__scalar_product_op_MM_internal_Eigen____CBV__Block___CBV__Block___CBV__Map___CBV__Matrix_M_0_0_0_0_00_0_0_0_0_Eigen___0A_V__OuterStride__0_0_2__Eigen___00_0_0_00_Eigen___00_0_0_0A__3___CBV__Transpose___CBV__Block___CBV__Map___CBV__Matrix_M_0_0_00_0A__0_0_00_Eigen___0A_V__Stride__0A__0A__2__Eigen___0_0_00_0A__Eigen___3__2__Z(): pass

def _cTkPhysicsComponent__PrepareConstraints____2____lambda_1___operator__(): pass

def _FindWindEigenvectorsJob____24____lambda_1___operator__(): pass

def AllocWindEigenvectors(): pass

def FindWindEigenvectorsJob(): pass

def TkDrawPhysicsSphere(): pass

def std___Func_impl_no_alloc__cTkCreatureTailComponent__BuildChains____2____lambda_1__void_cTkStackVector_TkHandle_32__1____TkHandle_cTkFixedString_32_char____bool______Move(): pass

def std___Func_impl_no_alloc__cTkPhysicsComponent__PrepareConstraints____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkInstanceWindComponent__FindWindEigenvectors____9____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkInstanceWindComponent__FindWindEigenvectors____10____lambda_2__bool_TkHandle____Copy(): pass

def std___Func_impl_no_alloc__cTkCreatureTailComponent__BuildChains____2____lambda_1__void_cTkStackVector_TkHandle_32__1____TkHandle_cTkFixedString_32_char____bool______Do_call(): pass

def std___Func_impl_no_alloc__cTkPhysicsComponent__PrepareConstraints____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkInstanceWindComponent__FindWindEigenvectors____9____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkInstanceWindComponent__FindWindEigenvectors____10____lambda_2__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkCreatureTailComponent__BuildChains____2____lambda_1__void_cTkStackVector_TkHandle_32__1____TkHandle_cTkFixedString_32_char____bool______Target_type(): pass

def std___Func_impl_no_alloc__cTkPhysicsComponent__PrepareConstraints____2____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkInstanceWindComponent__FindWindEigenvectors____9____lambda_1__bool_TkHandle____Target_type(): pass

def std___Func_impl_no_alloc__cTkInstanceWindComponent__FindWindEigenvectors____10____lambda_2__bool_TkHandle____Target_type(): pass

def loopSubdivSphereTri(): pass

def ConstructFIOS(): pass

def GetVectorFromState(): pass

class TkSketchMaths(ctypes.Structure):
    def ParseNumber(): pass

    def RunFunction(): pass

    def ValidateFunction(): pass


def ResolveAlgebra(): pass

def std___Func_impl_no_alloc__cTkSketchWindow__FindSketchComponentForSelectedNode____2____lambda_1__bool_TkHandle____Move(): pass

def std___Func_impl_no_alloc__cTkSketchWindow__FindSketchComponentForSelectedNode____2____lambda_1__bool_TkHandle____Do_call(): pass

def std___Func_impl_no_alloc__cTkSketchWindow__FindSketchComponentForSelectedNode____2____lambda_1__bool_TkHandle____Target_type(): pass

def _cTkNGuiEditor__DoEditColour____18____lambda_1____lambda_invoker_cdecl_(): pass

def _cTkNGuiEditor__SetActiveWindow____2____lambda_1___operator__(): pass

def NVGRegisterTexture(): pass

def NVGcreate(): pass

def glnvg__ActualEndFrame(): pass

def glnvg__allocCall(): pass

def glnvg__allocFragUniforms(): pass

def glnvg__allocPaths(): pass

def glnvg__allocTexture(): pass

def glnvg__allocVerts(): pass

def glnvg__convertPaint(): pass

def glnvg__deleteTexture(): pass

def glnvg(): pass

def glnvg__findTexture(): pass

def glnvg__maxVertCount(): pass

def glnvg__renderCancel(): pass

def glnvg__renderCreate(): pass

def glnvg__renderCreateTexture(): pass

def glnvg__renderDelete(): pass

def glnvg__renderDeleteTexture(): pass

def glnvg__renderFill(): pass

def glnvg__renderFlush(): pass

def glnvg__renderGetTextureSize(): pass

def glnvg__renderStroke(): pass

def glnvg__renderTriangles(): pass

def glnvg__renderUpdateTexture(): pass

def glnvg__renderViewport(): pass

def glnvg__setUniforms(): pass

def glnvg__stroke(): pass

def MarkNeighbor(): pass

def CreateShaderModule(): pass

def MarkSelfAndAllMemberVarsAsUsed(): pass

def ParseDecorations(): pass

def ParseDescriptorBindings(): pass

def ParseDescriptorBlockVariable(): pass

def ParseDescriptorBlockVariableSizes(): pass

def ParseDescriptorBlockVariableUsage(): pass

def ParseEntryPoints(): pass

def ParseEntrypointDescriptorSets(): pass

def ParseExecutionModes(): pass

def ParseFormat(): pass

def ParseFunction(): pass

def ParseFunctions(): pass

def ParseInterfaceVariable(): pass

def ParseInterfaceVariables(): pass

def ParseMemberCounts(): pass

def ParseNodes(): pass

def ParsePushConstantBlocks(): pass

def ParseStaticallyUsedResources(): pass

def ParseType(): pass

def SafeFreeBlockVariables(): pass

def SafeFreeInterfaceVariable(): pass

def SafeFreeTypes(): pass

def SortCompareDescriptorBinding(): pass

def AkFDNQSortCompare(): pass

def SynchronizeDescriptorSets(): pass

def TraverseCallGraph(): pass

def spvReflectCreateShaderModule2(): pass

def spvReflectDestroyShaderModule(): pass

def spvReflectEnumerateDescriptorBindings(): pass

def spvReflectEnumerateInputVariables(): pass

def CreateAudioInputSource(): pass

def CreateAudioInputSourceParams(): pass

def SetAudioInputCallbacks(): pass

def AK__DSP__ApplyGain(): pass

def AK__DSP__ApplyGain_0(): pass

def CreateAkCompressorFX(): pass

def ExecuteCompression(): pass

def AK__MixPower(): pass

def CreateAkCompressorFXParams(): pass

def ExecuteCompression_AVX2(): pass

def AK__DSP__ApplyGainRamp(): pass

def ComputeSVtr(): pass

def CreateAkConvolutionReverbFX(): pass

def FillMxFBRotate(): pass

def CreateAkConvolutionReverbFXParams(): pass

def FreqDomainBufferMult_V4F32(): pass

def StoreAccumulatedFreq_V4F32(): pass

def WriteHistoryPartition_V4F32(): pass

def FreqDomainBufferMult_V8_F16C(): pass

def StoreAccumulatedFreq_V8F32(): pass

def WriteHistoryPartition_V8_F16C(): pass

def FreqDomainBufferMult_V8_AVX2(): pass

def CreateAkDelayFX(): pass

def CreateAkDelayFXParams(): pass

def CreateAkFlangerFX(): pass

def CreateAkFlangerFXParams(): pass

def CreateAkGainFX(): pass

def CreateAkGainFXParams(): pass

def CreateAkGuitarDistortionFX(): pass

def CreateAkGuitarDistortionFXParams(): pass

def DSP__ComputeBiquadCoefs(): pass

def CreateAkHarmonizerFX(): pass

def CreateAkHarmonizerFXParams(): pass

def AkHarmonizerDSPProcess(): pass

def AkHarmonizerDSPProcessVoice(): pass

def CreateAkMatrixReverbFX(): pass

def CreateAkMatrixReverbFXParams(): pass

def AkMemFreeDefault(): pass

def AkMemMalignDefault(): pass

def AkMemMallocDefault(): pass

def AkMemReallocAlignedDefault(): pass

def AkMemReallocDefault(): pass

def AkMemSizeOfMemoryDefault(): pass

def AkMemTermForThreadDefault(): pass

def AkMemTotalReservedMemorySizeDefault(): pass

class AKPLATFORM(ctypes.Structure):
    def AllocVM(): pass

    def FreeVM(): pass

    def AkWideCharToChar(): pass

    def AkSetThreadName(): pass

    def SafeStrCpy(): pass


def _rpmalloc_adjust_size_class(): pass

def _rpmalloc_aligned_allocate(): pass

def _rpmalloc_aligned_reallocate(): pass

def _rpmalloc_allocate(): pass

def _rpmalloc_allocate_from_heap_fallback(): pass

def _rpmalloc_allocate_huge(): pass

def _rpmalloc_deallocate(): pass

def _rpmalloc_deallocate_direct_small_or_medium(): pass

def _rpmalloc_deallocate_huge(): pass

def _rpmalloc_global_get_reserved_spans(): pass

def _rpmalloc_heap_allocate(): pass

def _rpmalloc_heap_allocate_new(): pass

def _rpmalloc_heap_cache_adopt_deferred(): pass

def _rpmalloc_heap_cache_insert(): pass

def _rpmalloc_heap_extract_new_span(): pass

def _rpmalloc_heap_finalize(): pass

def _rpmalloc_heap_global_finalize(): pass

def _rpmalloc_heap_release(): pass

def _rpmalloc_heap_thread_cache_deferred_extract(): pass

def _rpmalloc_heap_unmap(): pass

def _rpmalloc_reallocate(): pass

def _rpmalloc_span_finalize(): pass

def _rpmalloc_span_map(): pass

def _rpmalloc_span_map_aligned_count(): pass

def _rpmalloc_span_mark_as_subspan_unless_master(): pass

def _rpmalloc_span_unmap(): pass

def _rpmalloc_unmap(): pass

def _rpmalloc_usable_size(): pass

def ak_rpaligned_alloc(): pass

def ak_rpaligned_realloc(): pass

def ak_rpfree(): pass

def ak_rpmalloc(): pass

def ak_rpmalloc_finalize(): pass

def ak_rpmalloc_heap_acquire(): pass

def ak_rpmalloc_heap_aligned_alloc(): pass

def ak_rpmalloc_heap_aligned_realloc(): pass

def ak_rpmalloc_heap_alloc(): pass

def ak_rpmalloc_heap_free(): pass

def ak_rpmalloc_heap_realloc(): pass

def ak_rpmalloc_heap_release(): pass

def ak_rpmalloc_initialize_config(): pass

def ak_rpmalloc_thread_finalize(): pass

def ak_rpmalloc_total_reserved_size(): pass

def ak_rpmalloc_usable_size(): pass

def ak_rprealloc(): pass

def get_thread_heap(): pass

def AkMeterApplyBallistics(): pass

def AkMeterGetValue(): pass

def AkMeterGetValueMultiObjects(): pass

def CreateAkMeterFX(): pass

def CreateAkMeterFXParams(): pass

def AkMeterFXRegisterCallback(): pass

def AkTimeConv__SamplesToMilliseconds(): pass

class AkMusicBank(ctypes.Structure):
    def LoadBankItem(): pass

    def UnloadBankSlot(): pass


def _Exists___AkSortedKeyArray_IVChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue____________U__AkArrayAllocatorNoAlign__0A___V12_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue______________U__AkDefaultSortedKeyCompare_I____QEBAPEAVChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue____________I_Z(): pass

def _Exists___AkSortedKeyArray_PEAVCAkRegisteredObj__VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue______________U__AkArrayAllocatorNoAlign__0A___V23_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue________________U__AkDefaultSortedKeyCompare_PEAVCAkRegisteredObj______QEBAPEAVChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue______________PEAVCAkRegisteredObj___Z(): pass

def CreateAkParametricEQFX(): pass

def CreateAkParametricEQFXParams(): pass

def CreateAkPeakLimiterFX(): pass

def CreateAkPeakLimiterFXParams(): pass

def CreateAkPitchShifterFX(): pass

def CreateAkPitchShifterFXParams(): pass

def AkPitchShifterDSPProcess(): pass

def DelayLengths__AkDelayQSortCompareFloat(): pass

def DelayLengths__ComputeDiffusionFiltersDelayTimes(): pass

def DelayLengths__ComputeFDNDelayTimes(): pass

def DelayLengths__ComputeFDNInputDelayTimes(): pass

def DelayLengths__ComputePrimeDelayLengths(): pass

def CreateAkRoomVerbFX(): pass

def SetupDecorrelationAPF(): pass

def CreateAkRoomVerbFXParams(): pass

def CreateAkSilenceSource(): pass

def CreateAkSilenceSourceParams(): pass

def CreateSineSource(): pass

def CreateSineSourceParams(): pass

def AK__SoundEngine___SetMultiplePositions_AkChannelEmitter_(): pass

class AKRANDOM(ctypes.Structure):
    def AkRandomInit(): pass


def AK__SoundEngine__GetBankIDFromString(): pass

def AK__SoundEngine__GetBankIDFromString_0(): pass

def AK__SoundEngine__DynamicSequence__GetSequence(): pass

def AK__SoundEngine__PreInit(): pass

def AK__SoundEngine__PrepareEventInternal(): pass

def AK__SoundEngine__PrepareGameSyncsInternal(): pass

def _CallGlobalExtensions(): pass

def CreateSrcMediaNodePCM(): pass

def CreateSrcMediaCodecPCM(): pass

def AkTlsAllocateSlot(): pass

def AkTlsFreeSlot(): pass

def AkTlsGetValue(): pass

def AkTlsSetValue(): pass

def GetListeners(): pass

class AkAudioLibSettings(ctypes.Structure):
    def SetAudioBufferSettings(): pass


class AkInterpolation(ctypes.Structure):
    def InterpolateNoCheck(): pass


def RegisterGameObjectAsActive(): pass

def UnregisterGameObjectAsActive(): pass

def LogWrongBankVersion(): pass

def _ReplaceWithDecodedMedia(): pass

def swprintf_s(): pass

def CreateDelayedAction(): pass

def CreateDelayedPendingAction(): pass

def AkTimeConv__SecondsToSamples(): pass

def _BinarySearch___AkSortedKeyArray_IVChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue____________U__AkArrayAllocatorNoAlign__0A___V12_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue______________U__AkDefaultSortedKeyCompare_I____QEBAPEAVChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue____________IAEA_N_Z(): pass

def _BinarySearch___AkSortedKeyArray_PEAVCAkRegisteredObj__VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue______________U__AkArrayAllocatorNoAlign__0A___V23_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue________________U__AkDefaultSortedKeyCompare_PEAVCAkRegisteredObj______QEBAPEAVChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue______________PEAVCAkRegisteredObj__AEA_N_Z(): pass

def GetNumberOfRTPC(): pass

def _Set___AkSortedKeyArray_PEAVCAkRegisteredObj__VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue______________U__AkArrayAllocatorNoAlign__0A___V23_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue________________U__AkDefaultSortedKeyCompare_PEAVCAkRegisteredObj______QEAAPEAVChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkRTPCValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkRTPCValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkRTPCValue______________PEAVCAkRegisteredObj__AEA_N_Z(): pass

def AkPipelineChannelMappingFunc(): pass

def AkVorbisChannelMappingFunc(): pass

def AkWaveChannelMappingFunc(): pass

def RotateAndContractPointsSIMD_ContractInside_(): pass

def RotateAndContractPointsSIMD_ContractOutside_(): pass

def BalanceFadePan(): pass

def BuildSphereTransformations(): pass

def BuildSphereTransformationsPositionOnly(): pass

def BuildSphereTransformationsPositionOnlyWithRotation(): pass

def BuildSphereTransformationsWithRotation(): pass

def AkDownmix__ComputeDirectSpeakerAssignment(): pass

def CopyRotatedCopiesOntoChannels(): pass

def DownmixGridToChannels(): pass

def GenPointsForZeroSpread(): pass

def GetSampledHarmonics2D(): pass

def GetSpeakerVolumes2DPanStd(): pass

def HandleLfe(): pass

def RotateAndContractPointsWithRotationSIMD(): pass

def RotateAndContractPointsWithRotationScalar(): pass

def AkDownmix__Transform(): pass

def TransformSampledSphereNoSpread(): pass

def TransformSampledSphereSIMD(): pass

def _GetSpeakerVolumes2DPan2(): pass

def _GetSpeakerVolumes2DPan2HasCenter(): pass

def _GetSpeakerVolumes2DPan3(): pass

def _GetSpeakerVolumes2DPan3HasCenter(): pass

def AkUnion_unsigned___int64_AkHybridAllocator_8_16_4__AkGrowByPolicy_Proportional_(): pass

def ReplaceIDForDefaultDevice(): pass

def AddVPLToArray(): pass

def GetAndConnectBus(): pass

def MixRecordsToOutputs(): pass

def GetBufferStatusForPinnedFiles(): pass

def PinFilesInStreamCache(): pass

def UnpinAllFilesInStreamCache(): pass

def UnpinFilesInStreamCache(): pass

def AreSourcesSACompatible(): pass

def CreateListenerComponents(): pass

class AkMath(ctypes.Structure):
    def Distance(): pass


def _BinarySearch___AkSortedKeyArray_PEAVCAkRegisteredObj__VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue______________U__AkArrayAllocatorNoAlign__0A___V23_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue________________U__AkDefaultSortedKeyCompare_PEAVCAkRegisteredObj______QEBAPEAVChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue______________PEAVCAkRegisteredObj__AEA_N_Z(): pass

def _Exists___AkSortedKeyArray_PEAVCAkRegisteredObj__VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue______________U__AkArrayAllocatorNoAlign__0A___V23_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue________________U__AkDefaultSortedKeyCompare_PEAVCAkRegisteredObj______QEBAPEAVChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue______________PEAVCAkRegisteredObj___Z(): pass

def _Set___AkSortedKeyArray_IVChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue____________U__AkArrayAllocatorNoAlign__0A___V12_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue______________U__AkDefaultSortedKeyCompare_I____QEAAPEAVChildTreeType___AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue____________IAEA_N_Z(): pass

def _Set___AkSortedKeyArray_PEAVCAkRegisteredObj__VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue______________U__AkArrayAllocatorNoAlign__0A___V23_UAkGrowByPolicy_Proportional__U__AkTransferMovePolicy_VChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue________________U__AkDefaultSortedKeyCompare_PEAVCAkRegisteredObj______QEAAPEAVChildTreeType___AkRTPCNestedSearchTree_VAkRTPCKey__UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidPlayingID__V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_IUGetInvalidUniqueID__V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr__________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiCh__V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr________UAkModTreeValue__V__AkRTPCNestedSearchTree_V__AkNestedKey_EUGetInvalidMidiNote__V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr______UAkModTreeValue__V__AkRTPCRootSearchTree_V__AkRootKey_PEAVCAkPBI__UGetNullPbiPtr____UAkModTreeValue______________PEAVCAkRegisteredObj__AEA_N_Z(): pass

def PropagateMute(): pass

def DecodeVariableLength(): pass

def ResetSeek(): pass

def SeekNextEvent(): pass

def ComputeChannelKPower(): pass

def ConvolvePolyPhase(): pass

def MeterBuffer(): pass

def AkCreateDummySink(): pass

def AkCreateDummySinkParam(): pass

def AkCreateSystemSinkParam(): pass

def AkHashIndexAdd(): pass

def AkHashIndexGetEntry(): pass

def AkHashIndexGetFirst(): pass

def AkHashIndexGetNext(): pass

def AkHashIndexInit(): pass

def AkHashIndexRemove(): pass

def AkHashIndexTerm(): pass

def Grow(): pass

def AkTimeConv__MillisecondsToSamples(): pass

def _ManageBQFChange_AkHpfParamEval_(): pass

def _ManageBQFChange_AkLpfParamEval_(): pass

def AkAtomicListAdd(): pass

def AkAtomicListGetFirst(): pass

def AkAtomicListInit(): pass

def AkAtomicListRemove(): pass

def AkAtomicListRemoveAll(): pass

def AkAtomicListTerm(): pass

def CreateAkSystemOutputMeta(): pass

def AkCreateDefaultSink(): pass

def AkSpatialSinkWorkCallback(): pass

def CopyWAVEFORMATEX(): pass

def WaveFormatToChannelConfig(): pass

def ApplyGainAndInterleave51FromStereo_V4F32(): pass

def ApplyGainAndInterleave51_V4F32(): pass

def ApplyGainAndInterleave71From51_V4F32(): pass

def ApplyGainAndInterleave71FromStereo_V4F32(): pass

def ApplyGainAndInterleave71_V4F32(): pass

def ApplyGainAndInterleaveStereoInt16_V4F32(): pass

def ApplyGainAndInterleaveStereo_V4F32(): pass

def ApplyGainMono_V4F32(): pass

def ApplyGainNInt16_V4F32(): pass

def ApplyGainN_V4F32(): pass

def MixChannel_V4F32(): pass

def MixNinNChannels_V4F32(): pass

def MixOneInNChannels_V4F32(): pass

def ApplyGainAndInterleaveXY(): pass

def ApplyGainAndInterleaveXYInt16(): pass

def PipelineToStandardOffsets(): pass

def MixChannel_V8F32_AVX2(): pass

def MixNinNChannels_V8F32_AVX2(): pass

def MixOneInNChannels_V8F32_AVX2(): pass

def Bypass_I16_1ChanV4F32(): pass

def Bypass_I16_2ChanV4F32(): pass

def Bypass_I16_NChanV4F32(): pass

def Fixed_I16_1ChanV4F32(): pass

def Fixed_I16_2ChanV4F32(): pass

def Fixed_I16_NChanV4F32(): pass

def Fixed_Native_1ChanV4F32(): pass

def Fixed_Native_2ChanV4F32(): pass

def Fixed_Native_NChanV4F32(): pass

def Interpolating_I16_1Chan(): pass

def Interpolating_I16_2Chan(): pass

def Interpolating_Native_1Chan(): pass

def Interpolating_Native_2Chan(): pass

def Bypass_I16_1ChanAVX2(): pass

def Bypass_I16_2ChanAVX2(): pass

def Fixed_Native_NChanAVX2(): pass

def Bypass_Native_NChan(): pass

def Interleave_Native_NChan(): pass

def Interpolating_I16_NChan(): pass

def Interpolating_Native_NChan(): pass

def _EvalCutoff(): pass

def CreateAkStereoDelayFX(): pass

def CreateAkStereoDelayFXParams(): pass

def AkStereoDelayDSPProcess(): pass

def AK__DSP__ApplyGain_1(): pass

def ProcessSurroundChannels(): pass

def CreateAkTremoloFX(): pass

def CreateAkTremoloFXParams(): pass

def CreateVorbisSrcPlugin(): pass

def CreateVorbisFileCodec(): pass

def CreateVorbisGrainCodec(): pass

def CreateVorbisCodec(): pass

def _vorbis_window(): pass

def vorbis_dsp_clear(): pass

def vorbis_dsp_init(): pass

def vorbis_dsp_pcmout(): pass

def vorbis_dsp_restart(): pass

def vorbis_dsp_synthesis(): pass

def vorbis_shift_dct(): pass

def DecodeVorbis(): pass

def DecodeVorbisInternal(): pass

def ilog(): pass

def mapping_info_unpack(): pass

def mapping_inverse(): pass

def InitFloorFunc(): pass

def floor1_info_unpack(): pass

def floor1_inverse1(): pass

def mergesort(): pass

def mdct_backward(): pass

def mdct_first_2stages_SIMD_Ex(): pass

def mdct_presymmetry_SIMD_Ex(): pass

def mdct_reverse_step7_step8(): pass

def mdct_unroll_lap(): pass

def vorbis_info_init(): pass

def vorbis_unpack_books(): pass

def res_inverse(): pass

def res_unpack(): pass

def BuildTree(): pass

def _make_decode_table(): pass

def _make_words(): pass

def ak_vorbis_book_decode(): pass

def decpack(): pass

def pack_sub_tree(): pass

def vorbis_book_unpack(): pass

def floor1_inverse2_AVX(): pass

def floor1_inverse2_AVX2(): pass

class hkcdRayCastTriangle(ctypes.Structure):
    def safeUsingDynamicTolerance(): pass

    def safeUsingDynamicToleranceQuad(): pass


def _anonymous_namespace___computeSupportingSpan(): pass

def _anonymous_namespace___setInternal(): pass

def rebuildInternal(): pass

def recurseRefitLeaf(): pass

def generateBuildFromAabbsTasksInternal_hkcdSimdTreeUtils__Build__hkcdRefitWithAabbFloat3Wrapper_(): pass

def hkSimplexSolverExamineActivePlanes(): pass

def hkSimplexSolverSolve1d(): pass

def hkSimplexSolverSolve2d(): pass

def hkSimplexSolverSolve3d(): pass

def hkSimplexSolverSortInfo(): pass

def hkSimplexSolverSolve(): pass

def hkcdDistanceSegmentQuad(): pass

def sphereCastOutOfLine(): pass

def _subdivideRecursive(): pass

def buildIcosahedron(): pass

def calculateResultingVelocity(): pass

def findNextPlane(): pass

def findNextTouchingPlane(): pass

def _anonymous_namespace___hkWeldVertices(): pass

def hkVertexFormat_orderComponents(): pass

def hkMeshVertexBufferUtil_convertArgb32ToToFloat32(): pass

def hkMeshVertexBufferUtil_convertFloat32ToUint8(): pass

def hkMeshVertexBufferUtil_convertUint8DwordToUint8(): pass

def hkMeshVertexBufferUtil_convertUint8ToFloat32(): pass

def hkMeshVertexBufferUtil_convertUint8ToUint8Dword(): pass

class hkGeometryProcessing(ctypes.Structure):
    def buildTransposedArray(): pass

    def getSupportingVertex(): pass

    def octahedronToNormal(): pass


def hkArrayBase__anonymous_namespace___EdgeData____growCapacity_hkContainerAllocator_(): pass

def hkArrayBase__anonymous_namespace___VertexInfo____growCapacity_hkContainerAllocator_(): pass

def hkFixedStackStorage__anonymous_namespace___EdgeCollapseData__QuadricErrorMetrics____hkFixedStackStorage__anonymous_namespace___EdgeCollapseData__QuadricErrorMetrics_(): pass

def _anonymous_namespace____bevel(): pass

def _anonymous_namespace____buildBoundingSpheresAndCleanTriangles(): pass

def _anonymous_namespace____clampVertexDisplacement(): pass

def _anonymous_namespace____cleanTriangles(): pass

def _anonymous_namespace____expandHull(): pass

def _anonymous_namespace____sanityCheck(): pass

class hkgpAbstractMeshDefinitions(ctypes.Structure):
    pass

def _anonymous_namespace___MinBoundingSphereFinder__SimplexBoundingSphere__Expand(): pass

def _anonymous_namespace___MinBoundingSphereFinder__FindRecurse(): pass

def setKeycodes(): pass

def hkSingletonUtil__initWith_hkKeycodeDataBase_hkKeycodeData_(): pass

class hkVersionUtils(ctypes.Structure):
    def getHavokTime(): pass

    def getSystemTime(): pass

    def linkProduct(): pass


def hkArrayBase__anonymous_namespace___InternalEdgeSortingStruct____growCapacity_hkBuiltinContainerAllocatorBase_(): pass

def hkArrayBase__anonymous_namespace___InternalEdgeSortingStruct____growCapacity_hkContainerAllocator_(): pass

def hkAlgorithm__quickSortRecursive__anonymous_namespace___InternalEdgeSortingStruct_hk__less__anonymous_namespace___InternalEdgeSortingStruct_A0xe6211e29__InternalEdgeSortingStruct___(): pass

def _anonymous_namespace___CustomGeometryConverter___scalar_deleting_destructor_(): pass

def _anonymous_namespace___CustomGeometryConverter__appendCustomPrimitiveToGeometry(): pass

def _anonymous_namespace___CustomGeometryConverter__appendExternalPrimitiveToGeometry(): pass

def _anonymous_namespace___CustomGeometryConverter__getScale(): pass

def calcFanNMP(): pass

def hkcdPointConvexVoronoiImpl(): pass

def hkcdPointVsConvexHull(): pass

def hknpProcessFlyingColors(): pass

def detectBinaryPackfile2014(): pass

def detectBinaryTagfile2014(): pass

def detectBinaryTagfile2015(): pass

def detectBinaryTagfile(): pass

def detectBinaryTagfileNT6(): pass

def detectCompressedTagfile(): pass

def detectXmlPackfile2014(): pass

def detectXmlTagfile2014(): pass

def detectXmlTagfile(): pass

def detectYamlFile(): pass

def hkSerialize__TagfileReadFormat__Impl__handleTypeSequenceNumber(): pass

def hkSerialize__TagfileReadFormat__Impl__handleTypeRollingHash(): pass

def hkSerialize__TagfileReadFormat__Impl__handleCompendiumId(): pass

def hkSerialize__TagfileReadFormat__Impl__handleCompendiumReference(): pass

def hkSerialize__TagfileReadFormat__Impl__handlePropertyDecls(): pass

def hkSerialize__TagfileReadFormat__Impl__inplaceTypeHashes(): pass

def hkSerialize__TagfileReadFormat__Impl__inplacePropertyHashes(): pass

def hkSerialize__TagfileReadFormat__Impl__handleData(): pass

def hkSerialize__TagfileReadFormat__Impl__handleItems(): pass

def hkSerialize__TagfileReadFormat__Impl__handlePatchData(): pass

def hkSerialize__TagfileReadFormat__Impl__handleNativePointers(): pass

def hkSerialize__TagfileReadFormat__Impl__unloadData(): pass

def hkSerialize__TagfileReadFormat__Impl__unloadCompendiumReference(): pass

def hkSerialize__TagfileReadFormat__Impl__unloadNativePointers(): pass

def hkSerialize__TagfileReadFormat__Impl__unloadItems(): pass

def hkSerialize__TagfileReadFormat__Impl__readStringsRbv_104_1_(): pass

def hkSerialize__TagfileReadFormat__Impl__readStringsRbv_120_1_(): pass

def hkSerialize__TagfileReadFormat__Impl__readStringsRbv_136_1_(): pass

def hkSerialize__TagfileReadFormat__Impl__handleTypeIdentity_1_(): pass

def hkSerialize__TagfileReadFormat__Impl__handleTypeBody_1_(): pass

def hkSerialize__TagfileReadFormat__Impl__readStringsRbv_104_0_(): pass

def hkSerialize__TagfileReadFormat__Impl__readStringsRbv_120_0_(): pass

def hkSerialize__TagfileReadFormat__Impl__readStringsRbv_136_0_(): pass

def hkSerialize__TagfileReadFormat__Impl__handleTypeIdentity_0_(): pass

def hkSerialize__TagfileReadFormat__Impl__handleTypeBody_0_(): pass

def hkSerialize__TagfileReadFormat__Impl__parseSubsection_17__hkSerialize__TagfileReadFormat__Impl__s_types_(): pass

def hkSerialize__TagfileReadFormat__Impl__parseSubsection_17__hkSerialize__TagfileReadFormat__Impl__s_types__0(): pass

def hkSerialize__TagfileReadFormat__Impl__parseSubsection_2__hkSerialize__TagfileReadFormat__Impl__s_index_(): pass

def hkSerialize__TagfileReadFormat__Impl__parseSubsection_2__hkSerialize__TagfileReadFormat__Impl__s_index__0(): pass

def hkSerialize__TagfileReadFormat__Impl__inplaceTypeHashesImpl__anonymous_namespace___LocalTypeHasher_(): pass

def hkSerialize__TagfileReadFormat__Impl__parse_1_(): pass

def hkSerialize__TagfileReadFormat__Impl__parse_2_(): pass

def hkSerialize__TagfileReadFormat__Impl__parse_0_(): pass

def _anonymous_namespace___TempReadBuffer__TempReadBuffer(): pass

def binaryTagfileCreateRead(): pass

def hkSerialize__Detail__PatchDataHandlerBase__hkSerialize__TagfileReadFormat__Impl__handlePatchData____34___PatchDataHandler___handleSection(): pass

def hkSerialize__Detail__PatchDataHandlerBase__hkSerialize__TagfileReadFormat__Impl__handlePatchData____11___PatchDataHandler___handleSection(): pass

def hkSerialize__TagfileReadFormat__Impl__handleTypeBodyImpl(): pass

def hkSerialize__TagfileReadFormat__Impl__handleTypeIdentityImpl(): pass

def _anonymous_namespace___readInto(): pass

def _anonymous_namespace___readUintVle64(): pass

def _anonymous_namespace___s_isTypeInBundle(): pass

def hkSerialize__TagfileReadFormat__Impl__view(): pass

def hkTypeVm__FastCopyInterpreter__execN__anonymous_namespace___SerializeInterpreter_(): pass

def _anonymous_namespace___growArray_hkSerialize__Detail__TagfileItem_(): pass

def hkAlgorithm__quickSortRecursive_hkSerialize__TagfileWriteFormat__Impl__Parts__Index__Inplace__lambda_0ba901cb14b375faf353f02ad7b76cd7___(): pass

def hkAlgorithm__quickSortRecursive_hkSerialize__TagfileWriteFormat__Impl__Parts__Index__Inplace__lambda_7569ea3a3eb2b5bf196807a8f87b03d2___(): pass

def _anonymous_namespace___ContainerPlusMark_hkArray_char_hkBuiltinContainerAllocator_0_0________ContainerPlusMark_hkArray_char_hkBuiltinContainerAllocator_0_0_____(): pass

def _anonymous_namespace___ContainerPlusMark_hkArray_hkTuple_int_unsigned_int_void_void_void_void_void_void__hkBuiltinContainerAllocator_0_0________ContainerPlusMark_hkArray_hkTuple_int_unsigned_int_void_void_void_void_void_void__hkBuiltinContainerAllocator_0_0_____(): pass

def _anonymous_namespace___ContainerPlusMark_hkArray_hkSerialize__Detail__TypeIdentity_hkBuiltinContainerAllocator_0_0________ContainerPlusMark_hkArray_hkSerialize__Detail__TypeIdentity_hkBuiltinContainerAllocator_0_0_____(): pass

def _anonymous_namespace___Fixups___scalar_deleting_destructor_(): pass

def _anonymous_namespace___SerializeInterpreter___executeExtra(): pass

def binaryTagfileCreateWrite(): pass

def _anonymous_namespace___SerializeInterpreter__execPointer(): pass

def _anonymous_namespace___SerializeInterpreter__execString(): pass

def padUp(): pass

def padUp_0(): pass

def _anonymous_namespace___ContainerPlusMark_hkArray_hkTuple_int_unsigned_int_void_void_void_void_void_void__hkBuiltinContainerAllocator_0_0_______writeTo(): pass

def _anonymous_namespace___ContainerPlusMark_hkArray_hkSerialize__Detail__TypeIdentity_hkBuiltinContainerAllocator_0_0_______writeTo(): pass

def _anonymous_namespace___ContainerPlusMark_hkHashMap_hkTuple_int_int_void_void_void_void_void_void__int_hkBuiltinContainerAllocator_0_0_______writeTo(): pass

def xmlTagfileCreateRead(): pass

def hkReflect__Detail__UnaryTypeVisitor__anonymous_namespace___PrintFormatVisitor_void_hkReflect__Format__Value_hkStringBuf_____dispatch(): pass

def hkReflect__Detail__UnaryVarVisitor__anonymous_namespace___WriteVarVisitor_void_bool_hk__Presets_const_____dispatch(): pass

def _anonymous_namespace___WriteVarVisitor__dispatch(): pass

def _anonymous_namespace___WriteVarVisitor__visit(): pass

def _anonymous_namespace___WriteVarVisitor__visit_0(): pass

def _anonymous_namespace___WriteVarVisitor__writeInlineable(): pass

def _anonymous_namespace___WriteVarVisitor__writePreset(): pass

def xmlTagfileCreateWrite(): pass

def _anonymous_namespace___VdbObjectAllocImpl__allocateForClone(): pass

def _anonymous_namespace___VdbObjectAllocImpl__deallocate(): pass

def _anonymous_namespace___VdbObjectAllocImpl___scalar_deleting_destructor_(): pass

def _anonymous_namespace___VdbObjectCompoundImpl__iterBegin(): pass

def _anonymous_namespace___VdbObjectCompoundImpl__iterNext(): pass

def _anonymous_namespace___VdbObjectStringImpl___scalar_deleting_destructor_(): pass

def _anonymous_namespace___VdbObjectStringImpl__getValue(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__cloneDecorators(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__convertAttrStringToAttributes(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__deallocVdbObjectType(): pass

def hkReflect__Detail__UnaryTypeVisitor__anonymous_namespace___VdbObjectTypeBuilder_hkReflect__Type_const___hkVdbReflect__TypeInfo___hkContainerAllocator_const___int___dispatch(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__typeFromArrayType(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__typeFromPointerType(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__typeFromRecordType(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__typeFromSimpleType(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__typeFromStringType(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__visit(): pass

def _anonymous_namespace___VdbObjectTypeBuilder__visit_0(): pass

def hkSingletonUtil__init_hkProcessFactory_(): pass

def hkSingletonUtil__init_hkDebugDisplay_(): pass

def _scePadRead(): pass

def jediIsInitialized(): pass

def scePadGetControllerInformation(): pass

def scePadInit(): pass

def scePadOpen(): pass

def scePadRead(): pass

def scePadSetAngularVelocityDeadbandState(): pass

def scePadSetTiltCorrectionState(): pass

def scePadSetVibration(): pass

def scePadTerminate(): pass

def _libpadRumbleStartDualShockEffect2(): pass

def _libpadSetECNC(): pass

def _libpadSetUniversalMute(): pass

def sub_1427C7220(): pass

def sub_1427C73B0(): pass

def sub_1427C7470(): pass

def _get2BytesLe(): pass

def _get4BytesLe(): pass

def sub_1427C77D0(): pass

def _libpadAppendReceiveData(): pass

def _libpadCheckValidHandle(): pass

def _libpadCreateDevice(): pass

def _libpadDeleteDevice(): pass

def _libpadGetBcd(): pass

def _libpadGetHandle(): pass

def _libpadGetReport(): pass

def _libpadGetSystemTime(): pass

def _libpadGetbitLe(): pass

def _libpadInitializeAllDevice(): pass

def sub_1427C84E0(): pass

def sub_1427C8850(): pass

def _libpadTerminateAllDevice(): pass

def sub_1427C8A70(): pass

def getDeviceByDeviceId(): pass

def getDeviceByHandle(): pass

def isBond(): pass

def isClone(): pass

def isHunt(): pass

def isJedi(): pass

def isPs5GenController(): pass

def sub_1427C9CB0(): pass

def _initPadLibrary(): pass

def sub_1427C9D90(): pass

def _libpadGetInternalMode(): pass

def _libpadGetPS4Mode(): pass

def _libpadSendOutputReport(): pass

def _libpadSendOutputReportAsync(): pass

def _termPadLibrary(): pass

def sub_1427CA670(): pass

def sub_1427CA6B0(): pass

def sub_1427CA830(): pass

def sub_1427CB620(): pass

def sub_1427CB8D0(): pass

def sub_1427CBC50(): pass

def sub_1427CBD70(): pass

def sub_1427CBDA0(): pass

def sub_1427CBDE0(): pass

def sub_1427CC820(): pass

def sub_1427CC880(): pass

def sub_1427CCA70(): pass

def StartAddress(): pass

def sub_1427CDE60(): pass

def sub_1427CDE90(): pass

def sub_1427CE130(): pass

def registerFreeFunction(): pass

def registerMallocFunction(): pass

def sub_1427CE440(): pass

def sub_1427CE5F0(): pass

def setTimerResolution(): pass

def sub_1427CE7D0(): pass

def sub_1427CEC80(): pass

def _libpadDeviceLock(): pass

def _libpadDeviceUnlock(): pass

def _libpadGiantLock(): pass

def _libpadGiantUnlock(): pass

def _libpadInitializeUtility(): pass

def _libpadTerminateUtility(): pass

def getCrc32OnBtHid(): pass

def hasCrcErrorOnBtHid(): pass

def _libpadGetMotionFilter(): pass

def _libpadResetAccl(): pass

def _libpadResetAngvel(): pass

def _libpadResetIdentity(): pass

def _libpadSetMotionFilter(): pass

def sub_1427CF170(): pass

def _libpadUpdateMotion(): pass

def _libpadUpdateMotionCalibData(): pass

def sub_1427D0150(): pass

def gbeInit(): pass

def gbeSetup(): pass

def gbeUpdate(): pass

def CreateMasteringSuiteFX(): pass

def CreateMasteringSuiteFXParams(): pass

def _anonymous_namespace___CreateNewSharesetCrossfader(): pass

def _anonymous_namespace___ShareSetCrossfader__Execute(): pass

def _anonymous_namespace___MasteringSuiteFXRegisterCallback(): pass

def SceAudioOut2MasteringModuleCheckParams(): pass

def SceAudioOut2MasteringModuleCoreProcess(): pass

def SceAudioOut2MasteringModuleTerm(): pass

def GetChannelMapping(): pass

def InitializeSystemModuleFunctionEntryPoints(): pass

def sceAudioMasterCheckParams(): pass

def sceAudioMasterGetStates(): pass

def sceAudioMasterInit(): pass

def sceAudioMasterProcessing(): pass

def sceAudioMasterSetModuleBypass(): pass

def sceAudioMasterSetParams(): pass

def sceAudioMasterSystemInit(): pass

def sceAudioMasterSystemReconfig(): pass

def sceAudioMasterTerm(): pass

def sceAudioMasterParamEqCheckParams(): pass

def sceAudioMasterParamEqInit(): pass

def sceAudioMasterParamEqProcess(): pass

def sceAudioMasterParamEqSetParams(): pass

def sceAudioMasterParamEqTerm(): pass

def sceAudioMasterCompressorCheckParams(): pass

def sceAudioMasterCompressorGetStates(): pass

def sceAudioMasterCompressorInit(): pass

def sceAudioMasterCompressorProcess(): pass

def sceAudioMasterCompressorSetParams(): pass

def sceAudioMasterCompressorTerm(): pass

def sceAudioMasterLimiterCheckParams(): pass

def sceAudioMasterLimiterGetStates(): pass

def sceAudioMasterLimiterInit(): pass

def sceAudioMasterLimiterProcess(): pass

def sceAudioMasterLimiterSetParams(): pass

def sceAudioMasterMasterVolumeCheckParams(): pass

def sceAudioMasterMasterVolumeInit(): pass

def sceAudioMasterMasterVolumeProcess(): pass

def sceAudioMasterMasterVolumeSetParams(): pass

def FilterProcess(): pass

def FilterSetParams(): pass

def MultichannelCalcBiQuadTransferCoEffs(): pass

def paramValidationWithClampFloat(): pass

def akCompressorAccumulate(): pass

def akMultichannelCompressorProcess(): pass

def akMultichannelXChCompressorProcess(): pass

def GetSavedGainsPtrs(): pass

def akMultichannelAdvancedLimiterProcess(): pass

def akMultichannelHardLimiterProcess(): pass

def akMultichannelSoftLimiterProcess(): pass

def akMultichannelXChHardLimiterProcess(): pass

def akMultichannelXChSoftLimiterProcess(): pass

def akMultichannelMasterVolumeProcess(): pass

def akFilterProcess(): pass

def NVSDK_NGX_Parameter_GetD(): pass

def NVSDK_NGX_Parameter_GetD3d11Resource(): pass

def NVSDK_NGX_Parameter_GetD3d12Resource(): pass

def NVSDK_NGX_Parameter_GetF(): pass

def NVSDK_NGX_Parameter_GetI(): pass

def NVSDK_NGX_Parameter_GetUI(): pass

def NVSDK_NGX_Parameter_GetULL(): pass

def NVSDK_NGX_Parameter_GetVoidPointer(): pass

def NVSDK_NGX_Parameter_SetD(): pass

def NVSDK_NGX_Parameter_SetD3d11Resource(): pass

def NVSDK_NGX_Parameter_SetD3d12Resource(): pass

def NVSDK_NGX_Parameter_SetF(): pass

def NVSDK_NGX_Parameter_SetI(): pass

def NVSDK_NGX_Parameter_SetUI(): pass

def NVSDK_NGX_Parameter_SetULL(): pass

def NVSDK_NGX_Parameter_SetVoidPointer(): pass

def GetLUIDFromVkPhysicalDevice(): pass

def NVSDK_NGX_VULKAN_Init_Common(): pass

def NVSDK_NGX_VULKAN_ShutdownCommon(): pass

def NVSDK_NGX_VULKAN_AllocateParameters(): pass

def NVSDK_NGX_VULKAN_CreateFeature(): pass

def NVSDK_NGX_VULKAN_CreateFeature1(): pass

def NVSDK_NGX_VULKAN_DestroyParameters(): pass

def NVSDK_NGX_VULKAN_EvaluateFeature(): pass

def NVSDK_NGX_VULKAN_EvaluateFeature_C(): pass

def NVSDK_NGX_VULKAN_GetCapabilityParameters(): pass

def NVSDK_NGX_VULKAN_GetParameters(): pass

def NVSDK_NGX_VULKAN_GetScratchBufferSize(): pass

def NVSDK_NGX_VULKAN_Init(): pass

def NVSDK_NGX_VULKAN_ReleaseFeature(): pass

def NVSDK_NGX_VULKAN_RequiredExtensions(): pass

def NVSDK_NGX_VULKAN_Shutdown(): pass

def NVSDK_NGX_VULKAN_Shutdown1(): pass

def ngx__CreateSHFolderFilePath(): pass

def ngx__CreateSystemFilePath(): pass

def ngx__GetSignerInfoTimeStamp(): pass

def ngx__IsCryptSvcPausedOrRunning(): pass

def ngx__IsNvidiaSubordinatePublicKey(): pass

def ngx__IsPeNvidiaSigned(): pass

def ngx__IsTrustedLocation(): pass

def ngx__IsWindowsBuildOrGreater(): pass

def ngx__LoadSystemLibraryInternal(): pass

def NGXCloseLibrary(): pass

def NGXGetLibrarySymbol(): pass

def NGXGetPathUsingQAI(): pass

def NGXLoadCoreLibrary(): pass

def NGXLoadLibrary(): pass

def ngx__OverrideSignatureVerificationFailure(): pass

def ngx__ParseASN1Timestamp(): pass

def ngx__ParseASN1TimestampOID(): pass

def ngx__QueryRegistryValueFromKey(): pass

def ngx__TestCertificateChain(): pass

def ngx__VerifySingleSignature(): pass

def ngx__VerifyTimeStampRFC3161(): pass

def ngx__VerifyTimeStampSignerInfo(): pass

def ngx___preloadSystemDlls(): pass

def nvFPrintf(): pass

def nvSWprintf(): pass

def nvSprintf(): pass

def NGXInvalidParameterHandler(): pass

def NGXSafeInitializeLog(): pass

def ngx__util__DefaultErrorCallback(): pass

def __WSAFDIsSet_0(): pass

def inet_ntop_0(): pass

def XInputGetState_0(): pass

def XInputSetState_0(): pass

def SHGetKnownFolderPath_0(): pass

def _Thrd_yield_0(): pass

def _Mtx_destroy_in_situ(): pass

def __ExceptionPtrDestroy(): pass

def _Mtx_unlock_0(): pass

def _Mtx_lock_0(): pass

def _Mtx_init_in_situ_0(): pass

def __ExceptionPtrAssign(): pass

def __ExceptionPtrCreate(): pass

def __ExceptionPtrCopyException(): pass

def _Cnd_init_in_situ_0(): pass

def __std_find_trivial_2(): pass

def __std_find_trivial_4(): pass

def __std_find_trivial_8(): pass

def __std_reverse_trivially_swappable_4(): pass

def __std_reverse_trivially_swappable_8(): pass

def _Xtime_get_ticks_0(): pass

def _Query_perf_counter_0(): pass

def _Query_perf_frequency_0(): pass

def _Thrd_join_0(): pass

def _Thrd_sleep_0(): pass

def _Thrd_id_0(): pass

def _Cnd_do_broadcast_at_thread_exit_0(): pass

def _Smtx_lock_exclusive(): pass

def _Smtx_lock_shared(): pass

def _Smtx_unlock_exclusive(): pass

def _Smtx_unlock_shared(): pass

def __std_system_error_allocate_message(): pass

def __std_system_error_deallocate_message(): pass

def __std_fs_code_page(): pass

def __std_fs_convert_narrow_to_wide(): pass

def __std_fs_convert_wide_to_narrow_replace_chars(): pass

def __std_fs_create_directory(): pass

def __std_fs_get_stats(): pass

def __std_fs_open_handle(): pass

def pre_c_initialization(): pass

def post_pgo_initialization(): pass

def pre_cpp_initialization(): pass

def __scrt_common_main_seh(): pass

def mainCRTStartup(): pass

def __scrt_initialize_thread_safe_statics(): pass

def __scrt_uninitialize_thread_safe_statics(): pass

def _Init_thread_footer(): pass

def _Init_thread_header(): pass

def _Init_thread_notify(): pass

def _Init_thread_wait(): pass

def __scrt_acquire_startup_lock(): pass

def __scrt_initialize_crt(): pass

def __scrt_initialize_onexit_tables(): pass

def __scrt_is_nonwritable_in_current_image(): pass

def __scrt_release_startup_lock(): pass

def __scrt_uninitialize_crt(): pass

def _onexit(): pass

def atexit(): pass

def __dyn_tls_init(): pass

def __dyn_tls_on_demand_init(): pass

def _alloca_probe(): pass

def __isa_available_init(): pass

def __vdecl_sinf4(): pass

def __libm_sse2_sincosf_(): pass

def __libm_sse2_sincosf4_(): pass

def __GSHandlerCheck(): pass

def __GSHandlerCheckCommon(): pass

def __security_check_cookie(): pass

def __GSHandlerCheck_EH(): pass

def __raise_securityfailure(): pass

def __report_gsfailure(): pass

def capture_previous_context(): pass

def __security_init_cookie(): pass

def _get_startup_file_mode(): pass

def __scrt_initialize_type_info(): pass

def __scrt_initialize_default_local_stdio_options(): pass

def __scrt_is_user_matherr_present(): pass

def __scrt_get_dyn_tls_init_callback(): pass

def __scrt_get_dyn_tls_dtor_callback(): pass

def __crt_debugger_hook(): pass

def __scrt_fastfail(): pass

def __scrt_initialize_mta(): pass

def __scrt_is_managed_app(): pass

def __scrt_set_unhandled_exception_filter(): pass

def __scrt_unhandled_exception_filter(): pass

def _RTC_Initialize(): pass

def _RTC_Terminate(): pass

def __scrt_is_ucrt_dll_in_use(): pass

def _sse2_sinf4(): pass

def _sse4_sinf4(): pass

def __common_ssin_cout_rare(): pass

def _purecall_0(): pass

def strstr_0(): pass

def strchr_0(): pass

def strrchr_0(): pass

def wcsstr_0(): pass

def __C_specific_handler_0(): pass

def memcpy_0(): pass

def memmove_0(): pass

def memset_0(): pass

def memcmp_0(): pass

def memchr_0(): pass

def __std_exception_copy_0(): pass

def __std_exception_destroy_0(): pass

def _CxxThrowException_0(): pass

def __CxxFrameHandler4_0(): pass

def __std_type_info_compare_0(): pass

def __RTDynamicCast_0(): pass

def _set_purecall_handler_0(): pass

def __CxxFrameHandler3_0(): pass

def wcsrchr_0(): pass

def __current_exception_0(): pass

def __current_exception_context_0(): pass

def exit_0(): pass

def roundf_0(): pass

def _fdclass_0(): pass

def strcmp_0(): pass

def fabs_0(): pass

def sqrt_0(): pass

def floor_0(): pass

def ceilf(): pass

def fmodf_0(): pass

def isalpha_0(): pass

def terminate_1(): pass

def floorf_0(): pass

def sqrtf_0(): pass

def isdigit_0(): pass

def atan2f_0(): pass

def cosf_0(): pass

def expf_0(): pass

def logf_0(): pass

def powf_0(): pass

def sinf_0(): pass

def acosf_0(): pass

def _dclass_0(): pass

def atanf_0(): pass

def log10_0(): pass

def log10f_0(): pass

def cos_0(): pass

def exp_0(): pass

def log_0(): pass

def tanf_0(): pass

def pow_0(): pass

def _wcsnicmp_0(): pass

def _wcsicmp_0(): pass

def _wcsupr_0(): pass

def _seh_filter_exe_0(): pass

def _set_app_type_0(): pass

def __setusermatherr_0(): pass

def _configure_narrow_argv_0(): pass

def _initialize_narrow_environment_0(): pass

def _get_initial_narrow_environment_0(): pass

def _initterm_0(): pass

def _initterm_e_0(): pass

def _exit_0(): pass

def _set_fmode_0(): pass

def __p___argc_0(): pass

def __p___argv_0(): pass

def _cexit_0(): pass

def _c_exit_0(): pass

def _register_thread_local_exe_atexit_callback_0(): pass

def _configthreadlocale_0(): pass

def _set_new_mode_0(): pass

def __p__commode_0(): pass

def _initialize_onexit_table_0(): pass

def _register_onexit_function_0(): pass

def _crt_atexit_0(): pass

def _anonymous_namespace___ChildRecordImpl__anonymous_namespace___ArrayParentRef____scalar_deleting_destructor_(): pass

def _anonymous_namespace___ChildRecordImpl__anonymous_namespace___RecordParentRef____scalar_deleting_destructor_(): pass

def hkReflect__Detail__UnaryTypeVisitor__anonymous_namespace___SetterArrayVisitor_hkReflect__Detail__Impl_____dispatch(): pass

def hkReflect__Detail__UnaryTypeVisitor__anonymous_namespace___SetterRecordVisitor_hkReflect__Detail__Impl_____dispatch(): pass

def _anonymous_namespace___ChildRecordImpl__anonymous_namespace___RecordParentRef___getFieldImpl(): pass

def _anonymous_namespace___ChildArrayImpl__anonymous_namespace___RecordParentRef___getValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__RecordType__anonymous_namespace___RecordParentRef___getValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__StringType__anonymous_namespace___RecordParentRef___getValue(): pass

def _anonymous_namespace___ChildArrayImpl__anonymous_namespace___ArrayParentRef___setNumElements(): pass

def _anonymous_namespace___ChildArrayImpl__anonymous_namespace___RecordParentRef___setNumElements(): pass

def _anonymous_namespace___ChildImpl_hkReflect__ArrayType__anonymous_namespace___ArrayParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__ArrayType__anonymous_namespace___RecordParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__BoolType__anonymous_namespace___ArrayParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__BoolType__anonymous_namespace___RecordParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__FloatType__anonymous_namespace___ArrayParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__FloatType__anonymous_namespace___RecordParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__IntType__anonymous_namespace___ArrayParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__IntType__anonymous_namespace___RecordParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__PointerType__anonymous_namespace___ArrayParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__PointerType__anonymous_namespace___RecordParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__RecordType__anonymous_namespace___ArrayParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__RecordType__anonymous_namespace___RecordParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__StringType__anonymous_namespace___ArrayParentRef___setValue(): pass

def _anonymous_namespace___ChildImpl_hkReflect__StringType__anonymous_namespace___RecordParentRef___setValue(): pass

def _anonymous_namespace___ChildArrayImpl__anonymous_namespace___ArrayParentRef___spliceInto(): pass

def _anonymous_namespace___ChildArrayImpl__anonymous_namespace___RecordParentRef___spliceInto(): pass

def _anonymous_namespace___NopCommandDispatcher___scalar_deleting_destructor_(): pass

def hkSingletonUtil__init_hkConcurrency_(): pass

class hkLargeIntImpl(ctypes.Structure):
    def countLeadingZeros(): pass

    def countTrailingZeros(): pass

    def multiplyUnsigned(): pass


def hkLargeIntDivideUnsigned(): pass

def hkSingletonUtil__init_ReflectStrings_(): pass

class hkReflectUtil(ctypes.Structure):
    def internCopy(): pass


def _anonymous_namespace___deallocateAttribute(): pass

def hkReflect__Detail__UnaryTypeVisitor__anonymous_namespace___AllocSizeVisitor_int_hkAttrData__FieldBase_hkAttrData_const___const_____dispatch(): pass

def hkReflect__Detail__UnaryTypeVisitor__anonymous_namespace___AttrDeclVisitor_hkResult_A0x64f2ec59__AttrDeclParams_const_____dispatch(): pass

def hkReflect__Detail__UnaryVarVisitor__anonymous_namespace___VarFromStringVisitor_hkResult_hkAttrData__FieldBase_hkAttrData_const___const_____dispatch(): pass

def _anonymous_namespace___getAllFields(): pass

def _anonymous_namespace___VarFromStringVisitor__setString(): pass

def _anonymous_namespace___AttrDeclVisitor__visit(): pass

def _anonymous_namespace___AttrDeclVisitor__visit_0(): pass

def _anonymous_namespace___AttrDeclVisitor__visit_1(): pass

def _anonymous_namespace___VarFromStringVisitor__visit(): pass

def _anonymous_namespace___VarFromStringVisitor__visit_0(): pass

def _anonymous_namespace___VarFromStringVisitor__visit_1(): pass

def _anonymous_namespace___SimdTreeTask___scalar_deleting_destructor_(): pass

def _thunk___anonymous_namespace___SimdTreeTask___scalar_deleting_destructor__adjustor_32____unsigned_int____ptr64(): pass

def createSimdTreeBroadPhase(): pass

def _anonymous_namespace___SimdTreeTask__getDebugAttributes(): pass

def _anonymous_namespace___SimdTreeTask__getName(): pass

def _anonymous_namespace___SimdTreeTask__process(): pass

def HidP_GetCaps_0(): pass

def HidP_GetValueCaps_0(): pass

def HidD_GetAttributes_0(): pass

def HidD_GetHidGuid_0(): pass

def HidD_GetPreparsedData_0(): pass

def HidD_FreePreparsedData_0(): pass

def HidD_GetFeature_0(): pass

def HidD_SetFeature_0(): pass

def HidD_GetManufacturerString_0(): pass

def HidD_GetProductString_0(): pass

def HidD_GetSerialNumberString_0(): pass

def _anonymous_namespace___DefaultTypeCallbackType__typeFromName(): pass

def _anonymous_namespace___DefaultTypeCallbackType__typeFromName_0(): pass

def _anonymous_namespace___DefaultTypeCallbackType__typeFromType(): pass

def _anonymous_namespace___DefaultTypeCallbackType___scalar_deleting_destructor_(): pass

def asinf_0(): pass

def log2f_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__AdCampaignAttributionModel_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__ValueToDateModel_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__Variable_____1___dtor_3(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__AdCampaignAttributionModel_____1___dtor_4(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterInventory_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterInventory_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterInventory_____1___dtor_3(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterInventory_____1___dtor_4(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_36(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterInventory_____1___dtor_10(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterResult_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterResult_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterResult_____1___dtor_3(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterResult_____1___dtor_4(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__CharacterResult_____1___dtor_5(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__SubscriptionModel_____1___dtor_3(): pass

def _PlayFab__PlayFabDataAPI__OnSetObjectsResult____1___dtor_5(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__Variable_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__Variable_____1___dtor_4(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__EntityTokenResponse_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__EntityTokenResponse_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__ItemInstance_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__ItemInstance_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__LinkedPlatformAccountModel_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__LinkedPlatformAccountModel_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserOpenIdInfo_____1___dtor_3(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserOpenIdInfo_____1___dtor_4(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__LinkedPlatformAccountModel_____1___dtor_5(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__LocationModel_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__MembershipModel_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__LocationModel_____1___dtor_3(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__MembershipModel_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__MembershipModel_____1___dtor_7(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__PlayerProfileModel_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__PlayerProfileModel_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__TagModel_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__StatisticValue_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__TagModel_____1___dtor_3(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__SubscriptionModel_____1___dtor_0(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__MembershipModel_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__MembershipModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__MembershipModel____std___Iterator_base0_______1___dtor_5(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__SubscriptionModel_____1___dtor_4(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__SubscriptionModel_____1___dtor_5(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__TagModel_____1___dtor_2(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_41(): pass

def _PlayFab__PlayFabDataAPI__OnGetObjectsResult____1___dtor_6(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__TreatmentAssignment_____1___dtor_3(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__TreatmentAssignment_____1___dtor_5(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserAccountInfo_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserAccountInfo_____1___dtor_2(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_168(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserXboxInfo_____1___dtor_7(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserXboxInfo_____1___dtor_9(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserDataRecord_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserDataRecord_____1___dtor_1(): pass

def _std__wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_____to_bytes____1___dtor_1(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserWindowsHelloInfo_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserWindowsHelloInfo_____1___dtor_12(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserWindowsHelloInfo_____1___dtor_13(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserWindowsHelloInfo_____1___dtor_15(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserGoogleInfo_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserGoogleInfo_____1___dtor_27(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserGoogleInfo_____1___dtor_28(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserGoogleInfo_____1___dtor_29(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserGoogleInfo_____1___dtor_30(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserGoogleInfo_____1___dtor_31(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserGoogleInfo_____1___dtor_33(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserOpenIdInfo_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserOpenIdInfo_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserOpenIdInfo_____1___dtor_5(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_17(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_18(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_19(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_21(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserTitleInfo_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__UserTitleInfo_____1___dtor_18(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__ValueToDateModel_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__Variable_____1___dtor_2(): pass

def _PlayFab__FromJsonUtilO_PlayFab__ClientModels__VirtualCurrencyRechargeTime_____1___dtor_0(): pass

def _PlayFab__FromJsonUtilS____1___dtor_1(): pass

def _PlayFab__PlayFabWinHttpPlugin__SetErrorInfo____1___dtor_0(): pass

def _PlayFab__PlayFabError__FromJson____1___dtor_4(): pass

def _PlayFab__ToJsonUtilS____1___dtor_1(): pass

def _PlayFab__ClientModels__UserXboxInfo__ToJson____1___dtor_2(): pass

def _PlayFab__ToJsonUtilO_PlayFab__ClientModels__EntityTokenResponse_____1___dtor_0(): pass

def _PlayFab__ToJsonUtilO_PlayFab__ClientModels__EntityTokenResponse_____1___dtor_2(): pass

def _PlayFab__ToJsonUtilS____1___dtor_2(): pass

def _PlayFab__ToJsonUtilO_PlayFab__ClientModels__UserXboxInfo_____1___dtor_0(): pass

def _PlayFab__ToJsonUtilO_PlayFab__ClientModels__UserXboxInfo_____1___dtor_2(): pass

def _PlayFab__ToJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_0(): pass

def _PlayFab__ToJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_2(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_171(): pass

def _PlayFab__ToJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_6(): pass

def _PlayFab__ToJsonUtilO_PlayFab__ClientModels__UserSteamInfo_____1___dtor_7(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_33(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__MembershipModel_void_________construct_PlayFab__ClientModels__MembershipModel_PlayFab__ClientModels__MembershipModel_const_______1___dtor_0(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__AdCampaignAttributionModel_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__AdCampaignAttributionModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__AdCampaignAttributionModel____std___Iterator_base0_______1___dtor_2(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_14(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__Variable_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__Variable____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__Variable____std___Iterator_base0_______1___dtor_2(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_16(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__CharacterResult_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__CharacterResult____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__CharacterResult____std___Iterator_base0_______1___dtor_1(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__LinkedPlatformAccountModel_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__LinkedPlatformAccountModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__LinkedPlatformAccountModel____std___Iterator_base0_______1___dtor_3(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__CharacterResult_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__CharacterResult____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__CharacterResult____std___Iterator_base0_______1___dtor_3(): pass

def _std__list_PlayFab__ClientModels__ItemInstance_std__allocator_PlayFab__ClientModels__ItemInstance_____list_PlayFab__ClientModels__ItemInstance_std__allocator_PlayFab__ClientModels__ItemInstance_______1___dtor_1(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_21(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_25(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__MembershipModel_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__MembershipModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__MembershipModel____std___Iterator_base0_______1___dtor_1(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__MembershipModel_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__MembershipModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__MembershipModel____std___Iterator_base0_______1___dtor_2(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__MembershipModel_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__MembershipModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__MembershipModel____std___Iterator_base0_______1___dtor_4(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_29(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__MembershipModel_void_________construct_PlayFab__ClientModels__MembershipModel_PlayFab__ClientModels__MembershipModel_const_______1___dtor_4(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__UserOpenIdInfo_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__UserOpenIdInfo____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__UserOpenIdInfo____std___Iterator_base0_______1___dtor_3(): pass

def _std___List_node_insert_op2_std__allocator_std___List_node_PlayFab__ClientModels__SubscriptionModel_void__________Append_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__SubscriptionModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__SubscriptionModel____std___Iterator_base0_______1___dtor_3(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_31(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_48(): pass

def _PlayFab__ClientModels__TreatmentAssignment__TreatmentAssignment____1___dtor_4(): pass

def _std__list_PlayFab__ClientModels__CharacterResult_std__allocator_PlayFab__ClientModels__CharacterResult______Assign_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__CharacterResult____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__CharacterResult____std___Iterator_base0_______1___dtor_0(): pass

def _std__list_PlayFab__ClientModels__LinkedPlatformAccountModel_std__allocator_PlayFab__ClientModels__LinkedPlatformAccountModel______Assign_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__LinkedPlatformAccountModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__LinkedPlatformAccountModel____std___Iterator_base0_______1___dtor_0(): pass

def _std__list_PlayFab__ClientModels__SubscriptionModel_std__allocator_PlayFab__ClientModels__SubscriptionModel______Assign_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__SubscriptionModel____std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__SubscriptionModel____std___Iterator_base0_______1___dtor_0(): pass

def _std___Tree_std___Tmap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____int_std__less_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__int____0______Copy_nodes_0_____1___catch_2(): pass

def _std___Tree_std___Tmap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____PlayFab__ClientModels__UserDataRecord_std__less_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__PlayFab__ClientModels__UserDataRecord____0______Copy_nodes_0_____1___dtor_1(): pass

def _std___Tree_std___Tmap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____PlayFab__ClientModels__UserDataRecord_std__less_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__PlayFab__ClientModels__UserDataRecord____0______Copy_nodes_0_____1___dtor_3(): pass

def _std___Tree_std___Tmap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____PlayFab__ClientModels__UserDataRecord_std__less_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__PlayFab__ClientModels__UserDataRecord____0______Copy_nodes_0_____1___catch_5(): pass

def _std___Tree_std___Tmap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____PlayFab__ClientModels__VirtualCurrencyRechargeTime_std__less_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__PlayFab__ClientModels__VirtualCurrencyRechargeTime____0______Copy_nodes_0_____1___catch_3(): pass

def _std___Tree_std___Tmap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std__less_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char________0______Copy_nodes_0_____1___dtor_1(): pass

def _std___Tree_std___Tmap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std__less_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char________0______Copy_nodes_0_____1___catch_3(): pass

def _std__list_PlayFab__ClientModels__UserOpenIdInfo_std__allocator_PlayFab__ClientModels__UserOpenIdInfo_____assign_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_PlayFab__ClientModels__UserOpenIdInfo____std___Iterator_base0__0_____1___dtor_0(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__CharacterResult_void_________construct_PlayFab__ClientModels__CharacterResult_PlayFab__ClientModels__CharacterResult_const_______1___dtor_0(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__LinkedPlatformAccountModel_void_________construct_PlayFab__ClientModels__LinkedPlatformAccountModel_PlayFab__ClientModels__LinkedPlatformAccountModel_const_______1___dtor_2(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__CharacterResult_void_________construct_PlayFab__ClientModels__CharacterResult_PlayFab__ClientModels__CharacterResult_const_______1___dtor_2(): pass

def _PlayFab__ClientModels__UserTitleInfo__UserTitleInfo____1___dtor_5(): pass

def _PlayFab__ClientModels__UserTitleInfo__UserTitleInfo____1___dtor_6(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__MembershipModel_void_________construct_PlayFab__ClientModels__MembershipModel_PlayFab__ClientModels__MembershipModel_const_______1___dtor_1(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__MembershipModel_void_________construct_PlayFab__ClientModels__MembershipModel_PlayFab__ClientModels__MembershipModel_const_______1___dtor_3(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__UserOpenIdInfo_void_________construct_PlayFab__ClientModels__UserOpenIdInfo_PlayFab__ClientModels__UserOpenIdInfo_const_______1___dtor_2(): pass

def _std___Default_allocator_traits_std__allocator_std___List_node_PlayFab__ClientModels__SubscriptionModel_void_________construct_PlayFab__ClientModels__SubscriptionModel_PlayFab__ClientModels__SubscriptionModel_const_______1___dtor_2(): pass

def _std___Hash_std___Umap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std___Uhash_compare_std__basic_string_char_std__char_traits_char__std__allocator_char____std__hash_std__basic_string_char_std__char_traits_char__std__allocator_char______std__equal_to_std__basic_string_char_std__char_traits_char__std__allocator_char________std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char________0______Insert_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char__________std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char__________std___Iterator_base0_______1___dtor_0(): pass

def _std___Hash_std___Umap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std___Uhash_compare_std__basic_string_char_std__char_traits_char__std__allocator_char____std__hash_std__basic_string_char_std__char_traits_char__std__allocator_char______std__equal_to_std__basic_string_char_std__char_traits_char__std__allocator_char________std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char________0______Insert_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char__________std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char__________std___Iterator_base0_______1___dtor_1(): pass

def _std___Hash_std___Umap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std___Uhash_compare_std__basic_string_char_std__char_traits_char__std__allocator_char____std__hash_std__basic_string_char_std__char_traits_char__std__allocator_char______std__equal_to_std__basic_string_char_std__char_traits_char__std__allocator_char________std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char________0_____emplace_char_const_____14__std__basic_string_char_std__char_traits_char__std__allocator_char__________1___dtor_2(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_24(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_25(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_26(): pass

def _std__list_PlayFab__ClientModels__ItemInstance_std__allocator_PlayFab__ClientModels__ItemInstance_____list_PlayFab__ClientModels__ItemInstance_std__allocator_PlayFab__ClientModels__ItemInstance_______1___dtor_0(): pass

def _PlayFab__ClientModels__EntityTokenResponse__EntityTokenResponse____1___dtor_0(): pass

def _PlayFab__ClientModels__EntityTokenResponse__EntityTokenResponse____1___dtor_1(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_0(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_1(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_4(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_3(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_4(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_5(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_6(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_7(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_8(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_9(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_24(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_21(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_1_0(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_27(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_3_0(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_4_0(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_5_0(): pass

def _PlayFab__ClientModels__FriendInfo__FriendInfo____1___dtor_6_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_1(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_2(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_3(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_4(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_5(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_6(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_7(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_8(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_9(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_10(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_13(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_15(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_28(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_12(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_25(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_27(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_29(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_1_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_2_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_3_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_4_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_5_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_6_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_7_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_8_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_9_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__GetPlayerCombinedInfoResultPayload____1___dtor_10_0(): pass

def _PlayFab__CallRequestContainerBase__CallRequestContainerBase____1___dtor_1(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_2(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_3(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_5(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_6(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_7(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_8(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_9(): pass

def _PlayFab__ClientModels__UserTitleInfo__UserTitleInfo____1___dtor_1(): pass

def _PlayFab__ClientModels__ItemInstance__ItemInstance____1___dtor_2_0(): pass

def _PlayFab__ClientModels__LoginResult__LoginResult____1___dtor_0(): pass

def _PlayFab__ClientModels__LoginResult__LoginResult____1___dtor_1(): pass

def _PlayFab__ClientModels__LoginResult__LoginResult____1___dtor_2(): pass

def _PlayFab__ClientModels__LoginResult__LoginResult____1___dtor_3(): pass

def _PlayFab__ClientModels__LoginResult__LoginResult____1___dtor_4(): pass

def _PlayFab__ClientModels__LoginResult__LoginResult____1___dtor_5(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_1(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_2(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_3(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_4(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_5(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_6(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_7(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_8(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_9(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_10(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_11(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_12(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_13(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_14(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_32(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_20(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_24(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_30(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_1_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_2_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_3_0(): pass

def _PlayFab__ClientModels__UserGoogleInfo__UserGoogleInfo____1___dtor_4(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_5_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_6_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_7_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_8_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_9_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_10_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_11_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_12_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_13_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__PlayerProfileModel____1___dtor_14_0(): pass

def _PlayFab__ClientModels__TreatmentAssignment__TreatmentAssignment____1___dtor_0(): pass

def _PlayFab__ClientModels__TreatmentAssignment__TreatmentAssignment____1___dtor_1(): pass

def _PlayFab__ClientModels__TreatmentAssignment__TreatmentAssignment____1___dtor_3(): pass

def _PlayFab__ClientModels__TreatmentAssignment__TreatmentAssignment____1___dtor_1_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_1(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_2(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_3(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_4(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_5(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_6(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_7(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_8(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_9(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_10(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_11(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_12(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_13(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_14(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_15(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_16(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_17(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_18(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_66(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_63(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_47(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_1_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_2_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_3_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_4_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_5_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_6_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_7_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_8_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__UserAccountInfo____1___dtor_9_0(): pass

def _PlayFab__ClientModels__UserGoogleInfo__UserGoogleInfo____1___dtor_2(): pass

def _PlayFab__ClientModels__UserGoogleInfo__UserGoogleInfo____1___dtor_3(): pass

def _PlayFab__ClientModels__UserTitleInfo__UserTitleInfo____1___dtor_2(): pass

def _std__function_void___cdecl_PlayFab__PlayFabError_const___void______operator_____1___dtor_1(): pass

def _std__list_PlayFab__ClientModels__ItemInstance_std__allocator_PlayFab__ClientModels__ItemInstance_____operator_____1___dtor_0(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__operator_____1___dtor_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__operator_____1___dtor_5(): pass

def _PlayFab__ClientModels__PlayerProfileModel__operator_____1___dtor_7(): pass

def _PlayFab__ClientModels__PlayerProfileModel__operator_____1___dtor_3(): pass

def _PlayFab__ClientModels__PlayerProfileModel__operator_____1___dtor_6(): pass

def _PlayFab__ClientModels__TreatmentAssignment__operator_____1___dtor_0(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_0(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_1(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_2(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_3(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_5(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_6(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_7(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_8(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_9(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_10(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_11(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_12(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_13(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_14(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_21(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_22(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_26(): pass

def _PlayFab__PlayFabDataAPI__SetObjects____1___dtor_34(): pass

def _PlayFab__ClientModels__GetFriendsListResult__FromJson____1___dtor_0(): pass

def _PlayFab__ClientModels__GetFriendsListResult__FromJson____1___dtor_2(): pass

def _PlayFab__FromJsonUtilS____1___dtor_0(): pass

def _PlayFab__PlayFabClientAPI__GetPlayFabIDsFromGenericIDs____1___dtor_12(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_0(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_1(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_3(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_30(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_6(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_7(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_8(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_9(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_10(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_11(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_12(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_13(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_14(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_23(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_24(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_28(): pass

def _PlayFab__PlayFabClientAPI__LoginWithOpenIdConnect____1___dtor_34(): pass

def _PlayFab__PlayFabClientAPI__MultiStepClientLogin____1___dtor_0(): pass

def _PlayFab__PlayFabClientAPI__MultiStepClientLogin____1___dtor_1(): pass

def _PlayFab__PlayFabDataAPI__OnSetObjectsResult____1___dtor_0(): pass

def _PlayFab__PlayFabClientAPI__OnRemoveFriendResult____1___dtor_1(): pass

def _PlayFab__PlayFabDataAPI__OnSetObjectsResult____1___dtor_2(): pass

def _PlayFab__PlayFabClientAPI__OnAttributeInstallResult____1___dtor_1(): pass

def _PlayFab__PlayFabClientAPI__OnGetFriendsListResult____1___dtor_1(): pass

def _PlayFab__PlayFabDataAPI__OnSetObjectsResult____1___dtor_3(): pass

def _PlayFab__PlayFabClientAPI__OnGetPlayFabIDsFromGenericIDsResult____1___dtor_1(): pass

def _PlayFab__PlayFabClientAPI__OnLoginWithOpenIdConnectResult____1___dtor_1(): pass

def _PlayFab__PlayFabClientAPI__OnLoginWithOpenIdConnectResult____1___dtor_5(): pass

def _PlayFab__PlayFabClientAPI__OnUpdateUserTitleDisplayNameResult____1___dtor_1(): pass

def _PlayFab__ClientModels__VirtualCurrencyRechargeTime__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__VirtualCurrencyRechargeTime__ToJson____1___dtor_1(): pass

def _PlayFab__ClientModels__VirtualCurrencyRechargeTime__ToJson____1___dtor_6(): pass

def _PlayFab__PlayFabWinHttpPlugin__ProcessResponse____1___dtor_10(): pass

def _PlayFab__PlayFabWinHttpPlugin__ProcessResponse____1___dtor_3(): pass

def _PlayFab__ClientModels__UserXboxInfo__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__UserXboxInfo__ToJson____1___dtor_1(): pass

def _PlayFab__ClientModels__Variable__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__Variable__ToJson____1___dtor_4(): pass

def _PlayFab__ClientModels__CharacterInventory__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__CharacterInventory__ToJson____1___dtor_4(): pass

def _PlayFab__ClientModels__ContactEmailInfoModel__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_49(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_26(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_51(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_34(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_12(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_37(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_43(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_20(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_22(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_46(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_67(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_52(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_54(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_56(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_59(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_60(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_45(): pass

def _PlayFab__ClientModels__FriendInfo__ToJson____1___dtor_47(): pass

def _PlayFab__ClientModels__GetFriendsListResult__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__GetFriendsListResult__ToJson____1___dtor_2(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_9(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__ToJson____1___dtor_28(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_25(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_27(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_35(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_37(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_3(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_7(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_17(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_1(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_5(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_7(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__ToJson____1___dtor_30(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__ToJson____1___dtor_32(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__ToJson____1___dtor_41(): pass

def _PlayFab__ClientModels__GetPlayerCombinedInfoResultPayload__ToJson____1___dtor_43(): pass

def _PlayFab__ClientModels__ItemInstance__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__ItemInstance__ToJson____1___dtor_30(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_23(): pass

def _PlayFab__ClientModels__LinkedPlatformAccountModel__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__LinkedPlatformAccountModel__ToJson____1___dtor_8(): pass

def _PlayFab__ClientModels__UserSteamInfo__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__UserSteamInfo__ToJson____1___dtor_10(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_1(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_20(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_4(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_7(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_11(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_13(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_15(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_18(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_23(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_25(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_34(): pass

def _PlayFab__ClientModels__LoginResult__ToJson____1___dtor_39(): pass

def _PlayFab__ClientModels__MembershipModel__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__MembershipModel__ToJson____1___dtor_10(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_40(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_3(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_5(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_9(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_11(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_13(): pass

def _PlayFab__ClientModels__PlayerProfileModel__ToJson____1___dtor_15(): pass

def _PlayFab__PlayFabError__FromJson____1___dtor_6(): pass

def _PlayFab__ClientModels__SubscriptionModel__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__SubscriptionModel__ToJson____1___dtor_14(): pass

def _PlayFab__ClientModels__TreatmentAssignment__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__TreatmentAssignment__ToJson____1___dtor_4(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_1(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_56(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_4(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_6(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_9(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_12(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_15(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_18(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_21(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_24(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_27(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_32(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_40(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_54(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_220(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_166(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_209(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_211(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_213(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_216(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_217(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_109(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_111(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_116(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_117(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_118(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_188(): pass

def _PlayFab__ClientModels__UserAccountInfo__ToJson____1___dtor_190(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_0(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_17(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_28(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_29(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_31(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_33(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_36(): pass

def _PlayFab__ClientModels__UserTitleInfo__ToJson____1___dtor_37(): pass

def _PlayFab__PlayFabProfilesAPI__OnGetProfileResult____1___dtor_1(): pass

def _PlayFab__PlayFabProfilesAPI__OnGetProfilesResult____1___dtor_1(): pass

def _PlayFab__PlayFabProfilesAPI__OnGetTitlePlayersFromMasterPlayerAccountIdsResult____1___dtor_1(): pass

def _PlayFab__PlayFabProfilesAPI__OnGetTitlePlayersFromMasterPlayerAccountIdsResult____1___dtor_4(): pass

def __dynamic_initializer_for__PlayFab__PlayFabSettings__staticPlayer______1___dtor_0(): pass

def __dynamic_initializer_for__PlayFab__PlayFabSettings__staticSettings______1___dtor_0(): pass

def _PlayFab__PlayFabDataAPI__OnGetObjectsResult____1___dtor_0(): pass

def _PlayFab__PlayFabDataAPI__OnGetObjectsResult____1___dtor_1(): pass

def _PlayFab__PlayFabDataAPI__OnGetObjectsResult____1___dtor_2(): pass

def _PlayFab__PlayFabDataAPI__OnGetObjectsResult____1___dtor_3(): pass

def _PlayFab__PlayFabDataAPI__OnGetObjectsResult____1___dtor_4(): pass

def _PlayFab__PlayFabDataAPI__OnSetObjectsResult____1___dtor_1(): pass

def _PlayFab__PlayFabError__FromJson____1___dtor_0(): pass

def _PlayFab__PlayFabError__FromJson____1___dtor_1(): pass

def _PlayFab__PlayFabError__GenerateErrorReport____1___dtor_1(): pass

def _PlayFab__PlayFabError__GenerateErrorReport____1___dtor_0(): pass

def _PlayFab__PlayFabError__GenerateErrorReport____1___dtor_3(): pass

def _PlayFab__PlayFabError__ToJson____1___dtor_0(): pass

def _PlayFab__PlayFabError__ToJson____1___dtor_7(): pass

def _std___Hash_std___Umap_traits_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std___Uhash_compare_std__basic_string_char_std__char_traits_char__std__allocator_char____std__hash_std__basic_string_char_std__char_traits_char__std__allocator_char______std__equal_to_std__basic_string_char_std__char_traits_char__std__allocator_char________std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char________0______Insert_range_unchecked_std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char__________std___Iterator_base0__std___List_unchecked_const_iterator_std___List_val_std___List_simple_types_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char__________std___Iterator_base0_______1___dtor_2(): pass

def _std__unordered_map_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std__hash_std__basic_string_char_std__char_traits_char__std__allocator_char______std__equal_to_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char___________unordered_map_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std__hash_std__basic_string_char_std__char_traits_char__std__allocator_char______std__equal_to_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char_____________1___dtor_1(): pass

def _std__unordered_map_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std__hash_std__basic_string_char_std__char_traits_char__std__allocator_char______std__equal_to_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char___________unordered_map_std__basic_string_char_std__char_traits_char__std__allocator_char____std__basic_string_char_std__char_traits_char__std__allocator_char____std__hash_std__basic_string_char_std__char_traits_char__std__allocator_char______std__equal_to_std__basic_string_char_std__char_traits_char__std__allocator_char______std__allocator_std__pair_std__basic_string_char_std__char_traits_char__std__allocator_char____const__std__basic_string_char_std__char_traits_char__std__allocator_char_____________1___dtor_2(): pass

def _PlayFab__CallRequestContainerBase__CallRequestContainerBase____1___dtor_0(): pass

def _PlayFab__CallRequestContainerBase__CallRequestContainerBase____1___dtor_2(): pass

def _PlayFab__CallRequestContainerBase__CallRequestContainerBase____1___dtor_3(): pass

def _PlayFab__CallRequestContainerBase__CallRequestContainerBase____1___dtor_5(): pass

def _PlayFab__CallRequestContainerBase__GetCallback____1___dtor_1(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_0(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_1(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_2(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_4(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_5(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_6(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_7(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_8(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_9(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_10(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_11(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_12(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_13(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_15(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_16(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_18(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_28(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_29(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_30(): pass

def _PlayFab__CallRequestContainer__CallRequestContainer____1___dtor_31(): pass

def _PlayFab__PlayFabWinHttpPlugin__WorkerThread____1___catch_4(): pass

def _PlayFab__CallRequestContainer__GetFullUrl____1___dtor_0(): pass

def _std__vector_PlayFab__PlayFabPluginManager__PluginEntry_std__allocator_PlayFab__PlayFabPluginManager__PluginEntry______Emplace_reallocate_PlayFab__PlayFabPluginManager__PluginEntry_____1___dtor_2(): pass

def _std__vector_PlayFab__PlayFabPluginManager__PluginEntry_std__allocator_PlayFab__PlayFabPluginManager__PluginEntry______Emplace_reallocate_PlayFab__PlayFabPluginManager__PluginEntry_____1___catch_5(): pass

def _std___Uninitialized_move_PlayFab__PlayFabPluginManager__PluginEntry___std__allocator_PlayFab__PlayFabPluginManager__PluginEntry_______1___dtor_0(): pass

def _PlayFab__PlayFabPluginManager__FindOrCreatePluginEntry____1___dtor_2(): pass

def _PlayFab__PlayFabPluginManager__GetPluginInternal____1___dtor_0(): pass

def _PlayFab__PlayFabPluginManager__GetPluginInternal____1___dtor_10(): pass

def _PlayFab__PlayFabPluginManager__HandleException____1___dtor_0(): pass

def _PlayFab__PlayFabPluginManager__HandleException____1___dtor_1(): pass

def _PlayFab__PlayFabPluginManager__HandleException____1___dtor_3(): pass

def _PlayFab__PlayFabApiSettings__PlayFabApiSettings____1___dtor_0(): pass

def _PlayFab__PlayFabApiSettings__PlayFabApiSettings____1___dtor_1(): pass

def _PlayFab__PlayFabApiSettings__PlayFabApiSettings____1___dtor_2(): pass

def _PlayFab__PlayFabApiSettings__PlayFabApiSettings____1___dtor_3(): pass

def _PlayFab__PlayFabWinHttpPlugin__ProcessResponse____1___dtor_2(): pass

def _PlayFab__PlayFabApiSettings__GetUrl____1___dtor_0(): pass

def _std__wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_____wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_______1___dtor_0(): pass

def _std__wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_____wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_______1___dtor_1(): pass

def _std__wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_____wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_______1___dtor_2(): pass

def _std__wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_____wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_______1___dtor_3(): pass

def _PlayFab__PlayFabWinHttpPlugin__PlayFabWinHttpPlugin____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin__PlayFabWinHttpPlugin____1___dtor_1(): pass

def _PlayFab__PlayFabWinHttpPlugin__PlayFabWinHttpPlugin____1___dtor_2(): pass

def _PlayFab__PlayFabWinHttpPlugin__PlayFabWinHttpPlugin____1___dtor_3(): pass

def _PlayFab__PlayFabWinHttpPlugin__PlayFabWinHttpPlugin____1___dtor_6(): pass

def _PlayFab__PlayFabWinHttpPlugin___PlayFabWinHttpPlugin____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin___PlayFabWinHttpPlugin____1___dtor_1(): pass

def _PlayFab__PlayFabWinHttpPlugin___PlayFabWinHttpPlugin____1___dtor_2(): pass

def _PlayFab__PlayFabWinHttpPlugin___PlayFabWinHttpPlugin____1___dtor_3(): pass

def _PlayFab__PlayFabWinHttpPlugin___PlayFabWinHttpPlugin____1___catch_5(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_1(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_2(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_20(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_7(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_28(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_11(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_21(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_23(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_24(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_29(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_33(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_35(): pass

def _PlayFab__PlayFabWinHttpPlugin__ExecuteRequest____1___dtor_36(): pass

def _PlayFab__PlayFabWinHttpPlugin__HandleResults____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin__HandleResults____1___dtor_1(): pass

def _PlayFab__PlayFabWinHttpPlugin__HandleResults____1___dtor_2(): pass

def _PlayFab__PlayFabWinHttpPlugin__HandleResults____1___dtor_3(): pass

def _PlayFab__PlayFabWinHttpPlugin__HandleResults____1___dtor_4(): pass

def _PlayFab__PlayFabWinHttpPlugin__HandleResults____1___dtor_5(): pass

def _PlayFab__PlayFabWinHttpPlugin__HandleResults____1___dtor_8(): pass

def _PlayFab__PlayFabWinHttpPlugin__HandleResults____1___dtor_9(): pass

def _PlayFab__PlayFabWinHttpPlugin__MakePostRequest____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin__MakePostRequest____1___dtor_1(): pass

def _PlayFab__PlayFabWinHttpPlugin__ProcessResponse____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin__ProcessResponse____1___dtor_1(): pass

def _PlayFab__PlayFabWinHttpPlugin__SetPredefinedHeaders____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin__Update____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin__WorkerThread____1___dtor_0(): pass

def _PlayFab__PlayFabWinHttpPlugin__WorkerThread____1___catch_5(): pass

def _std__wstring_convert_std__codecvt_utf8_utf16_wchar_t_1114111_0__wchar_t_std__allocator_wchar_t__std__allocator_char_____to_bytes____1___dtor_0(): pass

def __dynamic_initializer_for__mCache______1___dtor_0(): pass

def _cTkUserServiceGalaxy__Construct____1___dtor_1(): pass

def _cTkUserServiceGalaxy__Construct____1___dtor_2(): pass

def _cTkUserServiceGalaxy__Construct____1___dtor_7(): pass

def _cTkUserServiceGalaxy__Construct____1___dtor_16(): pass

def _cTkUserServiceGalaxy__Construct____1___dtor_12(): pass

def _cTkUserServiceGalaxy__GalaxyListenerHub__OnAuthSuccess____1___catch_0(): pass

def _cTkUserServiceGalaxy__GalaxyListenerEncTicket__OnEncryptedAppTicketRetrieveSuccess____1___dtor_0(): pass

def sub_14280EB25(): pass

def sub_14280EB7B(): pass

def sub_14280EBAB(): pass

def _dynamic_atexit_destructor_for__cTkTimeManager__mTimeJumpCallbacks__(): pass

def _dynamic_atexit_destructor_for__mCache__(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull__(): pass

def _dynamic_atexit_destructor_for__kNoIcon__(): pass

def _dynamic_atexit_destructor_for__kNoIcons__(): pass

def _dynamic_atexit_destructor_for__cGcRegionDecoratorBuildings__mDefaultBuildingComponentResHandle__(): pass

def _dynamic_atexit_destructor_for__cGcRealityManager__saIdsPendingValidation__(): pass

def _dynamic_atexit_destructor_for__gApplication__(): pass

class cGcSolarSystemTour(ctypes.Structure):
    def GetInstance(): pass


class cTkEntitlementManager(ctypes.Structure):
    def GetInstance(): pass


def _dynamic_atexit_destructor_for__cTkUnixTimestamp__mLocale__(): pass

class cTkTrophyManager(ctypes.Structure):
    def GetInstance(): pass


class cGcCommandLine(ctypes.Structure):
    def GetInstance(): pass


class cTkMetaDataManager(ctypes.Structure):
    def GetInstance(): pass


class cTkImmediateMode(ctypes.Structure):
    def GetInstance(): pass


class cTkMultiContextManager(ctypes.Structure):
    def GetInstance(): pass


class cTkSystem(ctypes.Structure):
    def GetInstance(): pass


class cTkEngineSettings(ctypes.Structure):
    def GetInstance(): pass


class cTkHmdOpenVR(ctypes.Structure):
    def GetInstance(): pass


class cTkLanguageManager(ctypes.Structure):
    def GetInstance(): pass


class cTk2dRenderer(ctypes.Structure):
    def GetInstance(): pass


class cTkAttachmentManager(ctypes.Structure):
    def GetInstance(): pass


def _dynamic_atexit_destructor_for__kNoIcon___0(): pass

class cTkLSystemManager(ctypes.Structure):
    def GetInstance(): pass


class cTkModelDressroom(ctypes.Structure):
    def GetInstance(): pass


class cTkInteractivityManager(ctypes.Structure):
    def GetInstance(): pass


class cTkWindManager(ctypes.Structure):
    def GetInstance(): pass


def _dynamic_atexit_destructor_for__gAssertMutex__(): pass

class cGcUGCBasesManager(ctypes.Structure):
    def GetInstance(): pass


def _dynamic_atexit_destructor_for__kNoIcon___1(): pass

class cTkUgcManager(ctypes.Structure):
    def GetInstance(): pass


def _dynamic_atexit_destructor_for__cGcAudioPulseMusic__mRNG__(): pass

def _dynamic_atexit_destructor_for__cGcByteBeat__gaCachedByteBeats__(): pass

def _dynamic_atexit_destructor_for__cGcByteBeat__maScreenTexturePool__(): pass

def _dynamic_atexit_destructor_for__cGcBaseBuildingGenerator__gSingleton__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___2(): pass

def _dynamic_atexit_destructor_for__cGcWFCBaseGenerator__mNavigationAreaResHandle__(): pass

def _dynamic_atexit_destructor_for__cGcWFCBaseGenerator__mClusterNavGraphConstructionData__(): pass

def _dynamic_atexit_destructor_for__cGcBaseBuildingManager__mOffenseReporter__(): pass

def _dynamic_atexit_destructor_for__cGcBaseBuildingManager__mBaseBuildingManager__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___3(): pass

def _dynamic_atexit_destructor_for__sRegisteredComponentBaseObjectHandles__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___4(): pass

def _cGcFreighterBaseNPCManager__TryAddWorkerNPC____10____dynamic_atexit_destructor_for__laeWorkerRaces__(): pass

def _dynamic_atexit_destructor_for__cGcWFCBuildingState__mInstances__(): pass

def _dynamic_atexit_destructor_for__cGcWFCBuildingState__mDecorationColourPartIds__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___5(): pass

class cGcRewardManager(ctypes.Structure):
    def GetInstance(): pass


def _dynamic_atexit_destructor_for__kNoIcon___6(): pass

def _dynamic_atexit_destructor_for__kNoIcon___7(): pass

def _dynamic_atexit_destructor_for__kaAllQueryInventories__(): pass

def _dynamic_atexit_destructor_for__kaAllQueryInventories_NoFreighter__(): pass

def _dynamic_atexit_destructor_for__kaAllQueryInventoriesAndChests__(): pass

def _dynamic_atexit_destructor_for__kaPoliceProbeQueryInventories__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___8(): pass

def _dynamic_atexit_destructor_for__kNoIcon___9(): pass

def _dynamic_atexit_destructor_for__cGcPlayerFleetManager__sFrigateTraitIDMap__(): pass

def _dynamic_atexit_destructor_for__cGcPlayerFleetManager__sExpeditionRewardIDMap__(): pass

def _dynamic_atexit_destructor_for__cGcPlayerFleetManager__sExpeditionInterventionEventIDMap__(): pass

def _dynamic_atexit_destructor_for__cGcPlayerFleetManager__sExpeditionEventIDMap__(): pass

def _dynamic_atexit_destructor_for__cGcPlayerFleetManager__sapAvailableFrigateTraits__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___10(): pass

def _dynamic_atexit_destructor_for__kNoIcon___11(): pass

def _cGcSeasonalData__GetSeasonalMainIcon____8____dynamic_atexit_destructor_for__sNormandyIconResource__(): pass

def _dynamic_atexit_destructor_for__cGcSynchronisedStateManager__sInstance__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___12(): pass

def _dynamic_atexit_destructor_for__cGcUserSettings__sDefaultSettingsData__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___13(): pass

def _MiniDumpFunction____2____dynamic_atexit_destructor_for__sCritSection__(): pass

def _dynamic_atexit_destructor_for__cGcProceduralTextureManager__mTextureCache__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___14(): pass

def _dynamic_atexit_destructor_for__kNoIcon___15(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageByteBeat__maIconResources__(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageByteBeat__mSawIconResource__(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageByteBeat__mSineIconResource__(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageByteBeat__mSquareIconResource__(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageByteBeat__mTriangleIconResource__(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageByteBeat__mapFreeTreeNodes__(): pass

def _cGcFrontendPageByteBeat__UpdateTreeNode____33____dynamic_atexit_destructor_for__lRNG__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___16(): pass

def _cGcFrontendPageDiscovery__DoPlanetaryMappingView____2____dynamic_atexit_destructor_for__lDiscoveredSections__(): pass

def _cGcFrontendPageDiscovery__DoPlanetaryMappingView____2____dynamic_atexit_destructor_for__sMappingStateCube__(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageInteractions__saMaterialsForSelectedDescriptors__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___17(): pass

def _dynamic_atexit_destructor_for__sapAvailableMissions__(): pass

def _dynamic_atexit_destructor_for__sapWaitingMissions__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___18(): pass

def _dynamic_atexit_destructor_for__saWeekendMissions__(): pass

def _dynamic_atexit_destructor_for__saAvailableMissions__(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageOptions__mUnappliedGraphicsSettings__(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageOptions__mOpenMenus__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___19(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPagePortalRunes__mIconData__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___20(): pass

def _dynamic_atexit_destructor_for__cGcFrontendPageTree__sCurrentDisplayTree__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___21(): pass

def _cGcFrontendPageFunctions__DoCreatureEggPopup____2____dynamic_atexit_destructor_for__lCustomNamesMap__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___22(): pass

def _dynamic_atexit_destructor_for__kNoIcon___23(): pass

def _dynamic_atexit_destructor_for__kNoIcon___24(): pass

def _dynamic_atexit_destructor_for__kNoIcon___25(): pass

def _dynamic_atexit_destructor_for__kNoIcon___26(): pass

def _dynamic_atexit_destructor_for__kNoIcon___27(): pass

def _dynamic_atexit_destructor_for__kNoIcon___28(): pass

def _dynamic_atexit_destructor_for__kNoIcon___29(): pass

def _dynamic_atexit_destructor_for__kNoIcon___30(): pass

def _dynamic_atexit_destructor_for__cGcNGuiLayer__mgDefaultText__(): pass

def _dynamic_atexit_destructor_for__cGcNGuiLayer__mgDefaultTextSpecial__(): pass

def _dynamic_atexit_destructor_for__cGcNGuiLayer__mgDefaultGraphic__(): pass

def _dynamic_atexit_destructor_for__cGcNGuiLayer__mgDefaultLayer__(): pass

def _dynamic_atexit_destructor_for__cGcNGuiLayer__mgpPopupLayer__(): pass

def _dynamic_atexit_destructor_for__gEditorGui__(): pass

def _dynamic_atexit_destructor_for__gGameGui__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___31(): pass

def _dynamic_atexit_destructor_for__gGuiRoot__(): pass

def _dynamic_atexit_destructor_for__gMetadata__(): pass

def _dynamic_atexit_destructor_for__cGcModelViewerCustomisationWindow__sCustomisationUIData__(): pass

def _dynamic_atexit_destructor_for__cGcModelViewerCustomisationWindow__sCustomisationData__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___32(): pass

def _cGcNGuiWindow__DoNode____2____dynamic_atexit_destructor_for__laShowingOrigin__(): pass

def _cGcNGuiWindow__DoNode____2____dynamic_atexit_destructor_for__laShowingAABBs__(): pass

class cGcMatchmakingPlayfab(ctypes.Structure):
    def GetXboxToken(): pass


def _dynamic_atexit_destructor_for__cGcMatchmaking__mFailedInviteCounts__(): pass

def _dynamic_atexit_destructor_for__cGcNetworkAbandonedFreightersManager__sSingleton__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___33(): pass

def _dynamic_atexit_destructor_for__kNoIcon___34(): pass

def _dynamic_atexit_destructor_for__gidMutedAccounts__(): pass

def _Party__PartyManager__GetSingleton____2____dynamic_atexit_destructor_for__singleton__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___35(): pass

def _dynamic_atexit_destructor_for__kNoIcon___36(): pass

def _dynamic_atexit_destructor_for__kNoIcon___37(): pass

def _dynamic_atexit_destructor_for__cGcNPCBehaviourChooseInteraction__sNPCUpdateQueue__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___38(): pass

def _dynamic_atexit_destructor_for__cGcAntagonistComponent__gAntagonistRNG__(): pass

def _cGcBaseDefenceComponent__UpdateBehaviours____53____dynamic_atexit_destructor_for__sRNG__(): pass

def _dynamic_atexit_destructor_for__cGcEncounterComponent__mEncounterStateResource__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___39(): pass

def _dynamic_atexit_destructor_for__kNoIcon___40(): pass

def _dynamic_atexit_destructor_for__cGcNPCComponent__maPerceptionCache__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___41(): pass

def _cGcSimpleInteractionComponent__GetActivationCost____11____dynamic_atexit_destructor_for__kCreatureFedCost__(): pass

def _cGcSimpleInteractionComponent__GetActivationCost____11____dynamic_atexit_destructor_for__kCreatureHarvestCooldownActivationCost__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___42(): pass

def _cGcOutpostComponent__Update____173____dynamic_atexit_destructor_for__lRNG__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___43(): pass

def _dynamic_atexit_destructor_for__gActiveInNguiClothComponents__(): pass

def _dynamic_atexit_destructor_for__gStoredVertsForInitialShape__(): pass

def _cClothPiece__ApplyWindAndExplosions____5____dynamic_atexit_destructor_for__windData__(): pass

def _cGcSpringAttachmentComponent__OnPostAnim____2____dynamic_atexit_destructor_for__windData__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___44(): pass

def _dynamic_atexit_destructor_for__kNoIcon___45(): pass

def _dynamic_atexit_destructor_for__kNoIcon___46(): pass

def _dynamic_atexit_destructor_for__kNoIcon___47(): pass

def _dynamic_atexit_destructor_for__gRegisteredInNguiCreatureIkComponents__(): pass

def _dynamic_atexit_destructor_for__gScaryThingsList__(): pass

def _dynamic_atexit_destructor_for__cGcCreatureRoutine__mRoutineMap__(): pass

def _dynamic_atexit_destructor_for__gRoutineGenerateQueue__(): pass

def _cGcCreatureRoutine__GenerateRoutine____2____dynamic_atexit_destructor_for__lTimeRNG__(): pass

def _dynamic_atexit_destructor_for__cGcCreatureComponent__mGrassPushers__(): pass

def _dynamic_atexit_destructor_for__cGcCreatureComponent__mPersistentGrassPushers__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___48(): pass

def _dynamic_atexit_destructor_for__kNoIcon___49(): pass

def _dynamic_atexit_destructor_for__gaNodesCache__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___50(): pass

def _dynamic_atexit_destructor_for__cGcTerrainTextureData__mCache__(): pass

def _dynamic_atexit_destructor_for__g_TerrainDebugDiffuseTex__(): pass

def _dynamic_atexit_destructor_for__g_TerrainDebugNormalTex__(): pass

def _dynamic_atexit_destructor_for__g_TerrainDebugStatusTex__(): pass

def _dynamic_atexit_destructor_for__gNearFeedbackTexRes__(): pass

def _dynamic_atexit_destructor_for__cGcTerrainTextureData__mValueNoiseHandle__(): pass

def _dynamic_atexit_destructor_for__cGcTerrainTextureData__mPerlinNoiseHandle__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___51(): pass

def _cGcRegionDecoratorCreatures__AddNodes____21____dynamic_atexit_destructor_for__lRandy__(): pass

def _dynamic_atexit_destructor_for__cGcRegionDecoratorObjects__mgObjectCounts__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___52(): pass

def _dynamic_atexit_destructor_for__cGcScanEvent__sScanEventResources__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___53(): pass

def _dynamic_atexit_destructor_for__cGcScanEventManager__saPreviousSPFullEventStack__(): pass

def _dynamic_atexit_destructor_for__cGcScanEventManager__saPreviousMPFullEventStack__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___54(): pass

def _dynamic_atexit_destructor_for__kNoIcon___55(): pass

def _dynamic_atexit_destructor_for__gPreloadedResources__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___56(): pass

def _dynamic_atexit_destructor_for__gStarIconResource__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___57(): pass

def _dynamic_atexit_destructor_for__cGcNavMapBase__sapInstances__(): pass

def _cGcNavMeshManager__GetInstance____2____dynamic_atexit_destructor_for__sInstance__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___58(): pass

def _cGcPlayer__RenderCharacterCustomisationEditor____2____dynamic_atexit_destructor_for__sUIData__(): pass

def _cGcPlayer__ShouldSummonFriendlyDrone____5____dynamic_atexit_destructor_for__kaInventories__(): pass

def _cGcPlayer__ShouldDismissFriendlyDrone____2____dynamic_atexit_destructor_for__kaInventories__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___59(): pass

def _cGcPlayerExperienceDirector__RenderNGui____72____dynamic_atexit_destructor_for__laNames__(): pass

def _cGcPlayerExperienceDirector__RenderNGui____59____dynamic_atexit_destructor_for__laNames__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___60(): pass

def _cGcPlayerInteract__GetActivationCost____2____dynamic_atexit_destructor_for__kNPCShipInteractionActivationCost__(): pass

def _cGcPlayerInteract__GetActivationCost____2____dynamic_atexit_destructor_for__kRemoteInteractionActivationCost__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___61(): pass

def _dynamic_atexit_destructor_for__kNoIcon___62(): pass

def _dynamic_atexit_destructor_for__gaDestroyedModels__(): pass

def _cGcDestructableComponent__Update____14____dynamic_atexit_destructor_for__lRNG__(): pass

def _dynamic_atexit_destructor_for__cGcProjectileImpact__mkDefaultData__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___63(): pass

def _dynamic_atexit_destructor_for__gaRepairTargets__(): pass

def _cGcDroneComponent__OnAttached____5____dynamic_atexit_destructor_for__lRNG__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___64(): pass

def _dynamic_atexit_destructor_for__cGcFadeNodeInstance__gsaImpostorRenderData__(): pass

def _dynamic_atexit_destructor_for__gaTestMaterialsRendered__(): pass

def _dynamic_atexit_destructor_for__gaImposterGenerationStateData__(): pass

def _dynamic_atexit_destructor_for__cGcFadeNodeInstance__mCache__(): pass

def _dynamic_atexit_destructor_for__gThreadSafeNodeDeletor__(): pass

def _cGcFadeNodeUpdater__GetInstance____2____dynamic_atexit_destructor_for__sUpdater__(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___0(): pass

def _dynamic_atexit_destructor_for__kNoIcon___65(): pass

def _dynamic_atexit_destructor_for__cGcBuildingPlacement__mLayoutsCache__(): pass

def _dynamic_atexit_destructor_for__cGcClusterLayoutPlacement__mLayoutsCache__(): pass

def _dynamic_atexit_destructor_for__cGcClusterLayoutPlacement__mLayoutsCacheLimitRadius__(): pass

def _cGcMap__ApplyWallThemes____2____dynamic_atexit_destructor_for__lPartsWithThemedVariantsById__(): pass

def _cGcMap__ApplyWallThemes____2____dynamic_atexit_destructor_for__lPartsWithThemedVariants__(): pass

def _dynamic_atexit_destructor_for__cGcModuleSet__mModuleSets__(): pass

def _dynamic_atexit_destructor_for__cGcModuleSet__mConnectors__(): pass

def _dynamic_atexit_destructor_for__kEmptyList__(): pass

def _dynamic_atexit_destructor_for__cGcModulePrototypeElement__mBlockFrameResHandle__(): pass

def _dynamic_atexit_destructor_for__cGcModulePrototypeElement__mWalkableIndicatorResHandle__(): pass

def _dynamic_atexit_destructor_for__cGcWFCScene__mCache__(): pass

def _dynamic_atexit_destructor_for__cGcWFCScene__mParseQueue__(): pass

def _dynamic_atexit_destructor_for__cGcSolarSystem__gPreloadedResources__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___66(): pass

def _dynamic_atexit_destructor_for__cGcPlanet__gPreloadedResources__(): pass

def _dynamic_atexit_destructor_for__cGcPlanet__sUniformBlends__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___67(): pass

def _dynamic_atexit_destructor_for__cGcPlanetGenerator__mFilenameHashMap__(): pass

def _cGcShipAIFlybyOverhead__Run____26____dynamic_atexit_destructor_for__sRNG__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___68(): pass

def _dynamic_atexit_destructor_for__kNoIcon___69(): pass

def _dynamic_atexit_destructor_for__kNoIcon___70(): pass

def _dynamic_atexit_destructor_for__kNoIcon___71(): pass

def _dynamic_atexit_destructor_for__kNoIcon___72(): pass

def _dynamic_atexit_destructor_for__kNoIcons___0(): pass

def _dynamic_atexit_destructor_for__kNoIcon___73(): pass

def _dynamic_atexit_destructor_for__mCache___0(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___1(): pass

def _Engine__GetMsg____2____dynamic_atexit_destructor_for__lMessage__(): pass

def _dynamic_atexit_destructor_for__mCache___1(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___2(): pass

def _dynamic_atexit_destructor_for__cEgPipelineResource__gCrossPipeSharedRenderTargets__(): pass

def _dynamic_atexit_destructor_for__mCache___2(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___3(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsVertPreamble__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsHullPreamble__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsDomnPreamble__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsGeomPreamble__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsFragPreamble__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsCompPreamble__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsTempCode0__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsTempCode1__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsTempCode2__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsTempCode3__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsTempCode4__(): pass

def _dynamic_atexit_destructor_for__cEgShaderResource__mgsTempCode5__(): pass

def _dynamic_atexit_destructor_for__gShaderPreloadList__(): pass

def _dynamic_atexit_destructor_for__gTextureHeaderCache__(): pass

def _dynamic_atexit_destructor_for__mCache___3(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___4(): pass

def _dynamic_atexit_destructor_for__mCache___4(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___5(): pass

def _dynamic_atexit_destructor_for__gRenderMut__(): pass

def _dynamic_atexit_destructor_for__mCache___5(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___6(): pass

def _dynamic_atexit_destructor_for__mCache___6(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___7(): pass

def _dynamic_atexit_destructor_for__mCache___7(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___8(): pass

def _EngineUtils__LoadResourcesFromDisk____2____dynamic_atexit_destructor_for__lLoadingResources__(): pass

def _dynamic_atexit_destructor_for__XMLNode__emptyXMLNode__(): pass

def _dynamic_atexit_destructor_for__mCache___8(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___9(): pass

def _dynamic_atexit_destructor_for__TkExample__cEgCubeNode__mPositionVertexLayout__(): pass

def _dynamic_atexit_destructor_for__TkExample__cEgCubeNode__mgapCubes__(): pass

def _dynamic_atexit_destructor_for__mCache___9(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___10(): pass

def TkGalactic___dynamic_atexit_destructor_for__mInstanceVertexLayout__(): pass

def TkGalactic___dynamic_atexit_destructor_for__mPositionVertexLayout__(): pass

def _dynamic_atexit_destructor_for__mCache___10(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___11(): pass

def _dynamic_atexit_destructor_for__TkHeavyAir__TkHeavyAirNodeTemplate__mgapHeavyAirNodes__(): pass

def _dynamic_atexit_destructor_for__TkSpeedLines__cTkSpeedLinesNode__cTkSpeedLinesNodeTemplate__mgapSpeedLineNodes__(): pass

def _dynamic_atexit_destructor_for__TkSpeedLines__cTkSpeedLinesNode__mgVertexLayout__(): pass

def _dynamic_atexit_destructor_for__mCache___11(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___12(): pass

def _dynamic_atexit_destructor_for__gSurfaceWindData__(): pass

def _dynamic_atexit_destructor_for__gWaterWindData__(): pass

def _dynamic_atexit_destructor_for__EgInstancedModelExtension__cEgInstancedMeshNode__mgVertexLayout__(): pass

def _dynamic_atexit_destructor_for__EgInstancedModelExtension__cEgInstancedMeshNode__mMemoryManager__(): pass

def EgInstancedModelExtension___dynamic_atexit_destructor_for__gGrassPushers__(): pass

def _Json__Value__nullSingleton____2____dynamic_atexit_destructor_for__nullStatic__(): pass

def _dynamic_atexit_destructor_for__mCache___12(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___13(): pass

def _dynamic_atexit_destructor_for__TkLineRenderer__cEgLineRendererNode__mVertexLayout__(): pass

def _dynamic_atexit_destructor_for__mCache___13(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___14(): pass

def EgMagicModels___dynamic_atexit_destructor_for__gWishes__(): pass

def EgMagicModels___dynamic_atexit_destructor_for__gCosts__(): pass

def EgMagicModels___dynamic_atexit_destructor_for__gModels__(): pass

def EgMagicModels___dynamic_atexit_destructor_for__gVisibilities__(): pass

def EgMagicModels___dynamic_atexit_destructor_for__gNewModels__(): pass

def EgMagicModels___dynamic_atexit_destructor_for__gSpawnModels__(): pass

def EgMagicModels___dynamic_atexit_destructor_for__gFadeModels__(): pass

def EgMagicModels___dynamic_atexit_destructor_for__gVisibilityMap__(): pass

def _EgMagicModels__ComputeModelVisibilities____2____dynamic_atexit_destructor_for__sRNG__(): pass

def _dynamic_atexit_destructor_for__mCache___14(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___15(): pass

def _dynamic_atexit_destructor_for__cEgTrailNode__mgVertexLayout__(): pass

def _dynamic_atexit_destructor_for__cEgTrailNode__mgapTrailNodes__(): pass

def _dynamic_atexit_destructor_for__TkParticle__cTkEmitterNode__mgRNG__(): pass

def _dynamic_atexit_destructor_for__TkParticle__TkParticleNodeTpl__mgapEmitters__(): pass

def _dynamic_atexit_destructor_for__TkParticle__TkParticleNodeTpl__mgapAsyncUpdateEmitters__(): pass

def _TkParticle__cTkEmitterNode__EmitterAsyncUpdate____2____dynamic_atexit_destructor_for__lWindData__(): pass

def _dynamic_atexit_destructor_for__mCache___15(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___16(): pass

def _dynamic_atexit_destructor_for__cEgVertexNode_cTkLineVertexData_32_128___mgVertexLayout__(): pass

def _dynamic_atexit_destructor_for__cEgVertexNode_cTkTerrainVertexData_32_120___mgVertexLayout__(): pass

def _dynamic_atexit_destructor_for__cEgSingleLineNode__mgVertexLayout__(): pass

def _dynamic_atexit_destructor_for__gGcVehicleGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcTerrainGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcUIGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcSpaceshipGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcSkyGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcSolarGenerationGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcSceneOptions__(): pass

def _dynamic_atexit_destructor_for__gGcScratchpadGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcSettlementGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcSimulationGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcRobotGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcMultiplayerGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcPlayerGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcGraphicsGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcGameplayGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcFreighterBaseGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcEnvironmentGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcFleetGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcDebugOptions__(): pass

def _dynamic_atexit_destructor_for__gGcCreatureGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcCameraGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcAudioGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcBuildingGlobals__(): pass

def _dynamic_atexit_destructor_for__gGcAISpaceshipGlobals__(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___17(): pass

def _dynamic_atexit_destructor_for__mCache___16(): pass

def hkMemory__Detail___dynamic_atexit_destructor_for__s_32alignedHeapAllocator__(): pass

def hkMemory__Detail___dynamic_atexit_destructor_for__s_64alignedHeapAllocator__(): pass

def hkMemory__Detail___dynamic_atexit_destructor_for__s_containerAllocators__(): pass

def _dynamic_atexit_destructor_for__hkReferencedObjectLock__singleton__(): pass

def _dynamic_atexit_destructor_for__hkRefCountedProperties__Entry___Auto__m_object__defaultType__def__(): pass

def _dynamic_atexit_destructor_for__hkRelArray_ArrayImpl___int64___s_instance__(): pass

def hkMemory__Signals___dynamic_atexit_destructor_for__m_garbageCollected__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__baseSystemLogOrigin__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__s_externalAllocatorFwd__(): pass

def _dynamic_atexit_destructor_for__hkMultiThreadCheck__s_stackTracer__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__ReflectionLog__(): pass

def _dynamic_atexit_destructor_for__s_debugLog__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__BoolImplN_unsigned_char___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__BoolImplN_unsigned_short___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__BoolImplN_unsigned_int___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__BoolImplN_bool___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__FloatImplN_float___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__FloatImplN_double___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__FloatImplN_hkHalf16___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_signed_char___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_char___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_unsigned_char___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_short___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_unsigned_short___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_int___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_unsigned_int___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_long___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_unsigned_long___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN___int64___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__IntImplN_unsigned___int64___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__CharBufferImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__HeapAllocImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__NullptrImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__RawPointerImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__ReferenceImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__ReferencedObjectPointerImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__RepeatImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__StaticStringImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__DerivedDataCache__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___0(): pass

def _dynamic_atexit_destructor_for__hkScalarF_FloatImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkSimdDouble64_FloatImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Any__s_impl__(): pass

def _dynamic_atexit_destructor_for__hkReflect__AttributeArrayWithOwnership__s_allocImplHeapInstance__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___1(): pass

def _dynamic_atexit_destructor_for__hkRelArray_ArrayImpl_int___s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__HavokRecordImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__HavokStringImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__HomogeneousArrayImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__ReferencedObjectAllocImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__StaticArrayImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__ViewPointerImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__hkArrayImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__hkInplaceArrayImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___2(): pass

def _dynamic_atexit_destructor_for__hkReflect__Var__s_impl__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___3(): pass

def _dynamic_atexit_destructor_for__hkStringPtr__Empty__(): pass

def _dynamic_atexit_destructor_for__hkStringPtr__s_impl__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__AttributeAllocImpl__s_heapInstance__(): pass

def _dynamic_atexit_destructor_for__hk__IsValueArray__s_impl__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___4(): pass

def _dynamic_atexit_destructor_for__hkStringBuf__s_impl__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___5(): pass

def _dynamic_atexit_destructor_for__hkStringView__s_impl__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__StringOut__s_impl__(): pass

def _dynamic_atexit_destructor_for__hkUuidPseudoRandomGenerator__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___6(): pass

def _dynamic_atexit_destructor_for__s_debugLog___7(): pass

def _dynamic_atexit_destructor_for__s_debugLog___8(): pass

def _dynamic_atexit_destructor_for__hkMemory__Detail__FwdToSharedAllocator__s_instance__(): pass

def hkMemory__Signals___dynamic_atexit_destructor_for__m_memoryAllocated__(): pass

def hkMemory__Signals___dynamic_atexit_destructor_for__m_memoryDeallocated__(): pass

def hkMemory__Signals___dynamic_atexit_destructor_for__m_memoryLimitBreached__(): pass

def hkMemory__Signals___dynamic_atexit_destructor_for__m_memoryLimitRestored__(): pass

def _dynamic_atexit_destructor_for__s_logMemDebug__(): pass

def _dynamic_atexit_destructor_for__hkHardwareInfo__singleton__(): pass

def _dynamic_atexit_destructor_for___anonymous_namespace___Registry__singleton__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__s_disabledIdSet__(): pass

def _dynamic_atexit_destructor_for__s_outputLock__(): pass

def _dynamic_atexit_destructor_for__hkError__singleton__(): pass

def _dynamic_atexit_destructor_for__hkErrorRouter__singleton__(): pass

def _dynamic_atexit_destructor_for__hkMemory__MallocAllocator__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkFileSystem__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___9(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__BuiltinTypeReg__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___10(): pass

def _dynamic_atexit_destructor_for___stackTracerWin32DataBuf__(): pass

def _dynamic_atexit_destructor_for___anonymous_namespace___AfterReflectNewLock__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___11(): pass

def _dynamic_atexit_destructor_for__s_debugLog___12(): pass

def _dynamic_atexit_destructor_for__s_debugLog___13(): pass

def _dynamic_atexit_destructor_for__hkRelPtr_PointerImpl__s_instance__(): pass

def hkLog__Origins___dynamic_atexit_destructor_for__common__(): pass

def hkLog__Origins___dynamic_atexit_destructor_for__globalDefault__(): pass

def _dynamic_atexit_destructor_for__hkMonitorStreamStatics__singleton__(): pass

def _dynamic_atexit_destructor_for__s_monitorStreamStatics__(): pass

def _dynamic_atexit_destructor_for__hkMemory__Snapshot__SortAllocationsByAddress__(): pass

def _dynamic_atexit_destructor_for__hkMemory__Snapshot__SortAllocationsByAddressComposite__(): pass

def _dynamic_atexit_destructor_for__hkMemory__Snapshot__SortAllocationsByComparison__(): pass

def _dynamic_atexit_destructor_for__hkMemory__Snapshot__SortAllocationsByTimeComposite__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___14(): pass

def _dynamic_atexit_destructor_for__s_debugLog___15(): pass

def _dynamic_atexit_destructor_for__s_debugLog___16(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianFloatImplN_float_hkEndian__EndianType_float_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianFloatImplN_float_hkEndian__EndianType_float_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianFloatImplN_double_hkEndian__EndianType_double_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianFloatImplN_double_hkEndian__EndianType_double_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianFloatImplN_hkHalf16_hkEndian__EndianType_hkHalf16_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianFloatImplN_hkHalf16_hkEndian__EndianType_hkHalf16_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_signed_char_hkEndian__EndianType_signed_char_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_signed_char_hkEndian__EndianType_signed_char_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_unsigned_char_hkEndian__EndianType_unsigned_char_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_unsigned_char_hkEndian__EndianType_unsigned_char_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_short_hkEndian__EndianType_short_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_short_hkEndian__EndianType_short_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_unsigned_short_hkEndian__EndianType_unsigned_short_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_unsigned_short_hkEndian__EndianType_unsigned_short_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_int_hkEndian__EndianType_int_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_int_hkEndian__EndianType_int_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_unsigned_int_hkEndian__EndianType_unsigned_int_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_unsigned_int_hkEndian__EndianType_unsigned_int_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN___int64_hkEndian__EndianType___int64_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN___int64_hkEndian__EndianType___int64_0_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_unsigned___int64_hkEndian__EndianType_unsigned___int64_1_____s_instance__(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__EndianIntImplN_unsigned___int64_hkEndian__EndianType_unsigned___int64_0_____s_instance__(): pass

def hkTypeVm___dynamic_atexit_destructor_for__origin__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___17(): pass

def _dynamic_atexit_destructor_for__hknpWorldCinfo___Auto__m_bodyIntegrator__defaultType__def__(): pass

def _dynamic_atexit_destructor_for__hknpWorldCinfo___Auto__m_broadPhaseConfig__defaultType__def__(): pass

def _dynamic_atexit_destructor_for__hknpWorldCinfo___Auto__m_collisionFilter__defaultType__def__(): pass

def _dynamic_atexit_destructor_for__hknpWorldCinfo___Auto__m_materialLibrary__defaultType__def__(): pass

def _dynamic_atexit_destructor_for__hknpWorldCinfo___Auto__m_motionPropertiesLibrary__defaultType__def__(): pass

def _dynamic_atexit_destructor_for__hknpWorldCinfo___Auto__m_qualityLibrary__defaultType__def__(): pass

def _dynamic_atexit_destructor_for__hknpWorldCinfo___Auto__m_shapeTagCodec__defaultType__def__(): pass

def _dynamic_atexit_destructor_for__hknpFirstPersonCharacterCinfo_m_customShape_Auto__defaultVal__(): pass

def _dynamic_atexit_destructor_for__hkHashBase_hkPair_unsigned_int_hkRefPtr_hknpShape_const_____hkBuiltinContainerAllocator_0_0______Auto__m_items_impl__(): pass

def _dynamic_atexit_destructor_for__hkHashBase_hkPair_hkReflect__Typedef__hkUint32_Tag_hkRefPtr_hknpShape_const_____hkBuiltinContainerAllocator_0_0______Auto__m_items_impl__(): pass

def _dynamic_atexit_destructor_for__hknpShapeFunctionsSingleton__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___18(): pass

def _dynamic_atexit_destructor_for__s_hknpQuerydispatcherLogOrigin__(): pass

def _dynamic_atexit_destructor_for__hknpConstraintDataFunctionsSingleton__singleton__(): pass

def _dynamic_atexit_destructor_for__hknpGlobals__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___19(): pass

def _dynamic_atexit_destructor_for__s_debugLog___20(): pass

def hknpFullCast___dynamic_atexit_destructor_for__manifoldGenerator__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__AD_TYPE_ANDROID_ID__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__AD_TYPE_IDFA__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__buildIdentifier__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__globalErrorHandler__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__productionEnvironmentURL__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__sdkVersion__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__staticPlayer__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__staticSettings__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabSettings__versionString__(): pass

def _dynamic_atexit_destructor_for__PlayFab__PlayFabPluginManager__defaultInstanceName__(): pass

def _PlayFab__PlayFabPluginManager__GetInstance____2____dynamic_atexit_destructor_for__instance__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___74(): pass

def _dynamic_atexit_destructor_for__kNoIcons___1(): pass

def _dynamic_atexit_destructor_for__cTkSketchComponent__mNodesClipboard__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___75(): pass

def _dynamic_atexit_destructor_for__kNoIcons___2(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___18(): pass

def _cTkSketchComponent__RenderNGuiEditor____2____dynamic_atexit_destructor_for__lNodeGraph__(): pass

def _cTkSketchComponent__RenderNGuiEditor____2____dynamic_atexit_destructor_for__lVariantsPopup__(): pass

def _dynamic_atexit_destructor_for__gPhysicsManager__(): pass

def _dynamic_atexit_destructor_for__gSpaceshipReportCallback___0(): pass

def _dynamic_atexit_destructor_for__gVehicleReportCallback__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___76(): pass

def _dynamic_atexit_destructor_for__kNoIcons___3(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___19(): pass

def _cTkPhysicsManager__Construct____2____dynamic_atexit_destructor_for__sAngularCallback__(): pass

def _cTkPhysicsManager__Construct____2____dynamic_atexit_destructor_for__sIgnoreBackfacesCallback__(): pass

def _cTkPhysicsManager__Update____2____dynamic_atexit_destructor_for__sJob__(): pass

def _dynamic_atexit_destructor_for__cTkSmartResHandle__Null__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___77(): pass

def _dynamic_atexit_destructor_for__kNoIcons___4(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___20(): pass

def _GetNullSelectionResDescriptor____2____dynamic_atexit_destructor_for__lpNullDesc__(): pass

def _dynamic_atexit_destructor_for__mCache___17(): pass

def _dynamic_atexit_destructor_for__cTkNetReplicatedEntityComponent__mCreateIdCallback__(): pass

def _dynamic_atexit_destructor_for__gBuiltMeshes__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___78(): pass

def _dynamic_atexit_destructor_for__kNoIcons___5(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___21(): pass

class cTkIndexedMeshShapeCInfo(ctypes.Structure):
    def getConvexShape(): pass


class cTkQuadMeshShapeCInfo(ctypes.Structure):
    def getConvexShape(): pass


class cTkTerrainMeshShapeCInfo(ctypes.Structure):
    def getConvexShape(): pass


def _dynamic_atexit_destructor_for__kNoIcon___79(): pass

def _dynamic_atexit_destructor_for__kNoIcons___6(): pass

def _dynamic_atexit_destructor_for__cTkAnimationComponent__sGetClosestPlayerSqrDistFunc__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___80(): pass

def _dynamic_atexit_destructor_for__kNoIcons___7(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___22(): pass

def _dynamic_atexit_destructor_for__MetadataNGuiRender__ClassRender__mCustomClassRenders__(): pass

def _dynamic_atexit_destructor_for__cTkMetaData__mgModFolderList__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___81(): pass

def _dynamic_atexit_destructor_for__kNoIcons___8(): pass

def _dynamic_atexit_destructor_for__cTkSMShaderWindow__mMaterialList__(): pass

def _dynamic_atexit_destructor_for__cTkSMShaderWindow__mMaterialNames__(): pass

def _dynamic_atexit_destructor_for__cTkSMShaderWindow__mSelectedMaterial__(): pass

def _dynamic_atexit_destructor_for__cTkSMShaderWindow__maLabelGroups__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___82(): pass

def _dynamic_atexit_destructor_for__kNoIcons___9(): pass

def _dynamic_atexit_destructor_for__kTkMath__(): pass

def _cTkSMShaderWindow__RenderEditorWindow____2____dynamic_atexit_destructor_for__lCachedHandle__(): pass

def _cTkSMShaderWindow__RenderEditorWindow____2____dynamic_atexit_destructor_for__lNamePtrs__(): pass

def _cTkSMShaderWindow__RenderEditorWindow____2____dynamic_atexit_destructor_for__laTextures__(): pass

def _cTkInputManager__GetBindings____2____dynamic_atexit_destructor_for__kEmptyBindingList__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___83(): pass

def _dynamic_atexit_destructor_for__kNoIcons___10(): pass

def _cTkInputDeviceManagerBase__GetInputAxisDescription____2____dynamic_atexit_destructor_for__sNumberedButtons__(): pass

def _cTkInputDeviceManagerBase__GetInputAxisName____2____dynamic_atexit_destructor_for__sNumberedButtons__(): pass

def _cTkInputDeviceManagerBase__GetInputButtonDescription____2____dynamic_atexit_destructor_for__sNumberedButtons__(): pass

def _cTkInputDeviceManagerBase__GetInputButtonName____2____dynamic_atexit_destructor_for__sNumberedButtons__(): pass

def _dynamic_atexit_destructor_for__cTkCompute___mInstance__(): pass

def _dynamic_atexit_destructor_for__cTkDebugRenderer__mgDebugFont__(): pass

def _dynamic_atexit_destructor_for__cTkDebugRenderer__mgDebugStyle___0(): pass

def _dynamic_atexit_destructor_for__kNoIcon___84(): pass

def _dynamic_atexit_destructor_for__kNoIcons___11(): pass

def _dynamic_atexit_destructor_for__kNoIcon___85(): pass

def _dynamic_atexit_destructor_for__kNoIcons___12(): pass

def _dynamic_atexit_destructor_for__cTkShader__m_UniformHashMap__(): pass

def _dynamic_atexit_destructor_for__cTkJobManager__mJobParamAllocator__(): pass

def _dynamic_atexit_destructor_for__AkToneGenParamBlock__static_mRNGJitter__(): pass

def _dynamic_atexit_destructor_for__gFileSystem__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___86(): pass

def _dynamic_atexit_destructor_for__kNoIcons___13(): pass

def _dynamic_atexit_destructor_for__kNoIcon___87(): pass

def _dynamic_atexit_destructor_for__kNoIcons___14(): pass

def _dynamic_atexit_destructor_for__kNoIcon___88(): pass

def _dynamic_atexit_destructor_for__kNoIcons___15(): pass

def _dynamic_atexit_destructor_for__kNoIcon___89(): pass

def _dynamic_atexit_destructor_for__kNoIcons___16(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___23(): pass

def TkAssertDlgResourceData___dynamic_atexit_destructor_for__records__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__kDisplayBuffers__(): pass

def _dynamic_atexit_destructor_for__cTkFfxFsr2__mLanczos2LutTextureResource__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__mSamplerIndexHashMap__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__mUniformMemoryManager__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_ComputePipelineStateHashMap__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_PipelineStateHashMap__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_UniqueFragmentShaderHashMap__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_UniqueVertexShaderHashMap__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_allPhysicalDevices__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_bufferDestroyQueue__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_hdrShader__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_imageDestroyQueue__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_imageViewDestroyQueue__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_pipelineDestroyQueue__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_presentModes__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_resCopier__(): pass

def _dynamic_atexit_destructor_for__cTkGraphicsAPI__cTkGnmGolbals__m_surfaceFormats__(): pass

def _dynamic_atexit_destructor_for__gGraphicsTempBuffer__(): pass

def _dynamic_atexit_destructor_for__g_lastVulkanError__(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___24(): pass

def _dynamic_atexit_destructor_for__gMonitorSemaphore__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___90(): pass

def _dynamic_atexit_destructor_for__kNoIcons___17(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___25(): pass

def _dynamic_atexit_destructor_for__kUpdater_cTkStaticPhysicsComponent__(): pass

def _cTkInstanceWindComponent__MatchInstance____5____dynamic_atexit_destructor_for__lWindData__(): pass

def _dynamic_atexit_destructor_for__sSketchEvent__gSketchEvents__(): pass

def _dynamic_atexit_destructor_for__cTkSketchWindow__mMinimisedNodes__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___91(): pass

def _dynamic_atexit_destructor_for__kNoIcons___18(): pass

def _dynamic_atexit_destructor_for__kNoIcon___92(): pass

def _dynamic_atexit_destructor_for__kNoIcons___19(): pass

def _dynamic_atexit_destructor_for__cTkNGui__mgaResourceLookup__(): pass

def _dynamic_atexit_destructor_for__gaShaderBuffers__(): pass

def _dynamic_atexit_destructor_for__kNoIcon___93(): pass

def _dynamic_atexit_destructor_for__kNoIcons___20(): pass

def _dynamic_atexit_destructor_for__cTkAsyncIOManager__mpImp___0(): pass

def _dynamic_atexit_destructor_for__kNoIcon___94(): pass

def _dynamic_atexit_destructor_for__kNoIcons___21(): pass

def _dynamic_atexit_destructor_for__kNoIcon___95(): pass

def _dynamic_atexit_destructor_for__kNoIcons___22(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___26(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___27(): pass

def _dynamic_atexit_destructor_for__mCache___18(): pass

def _dynamic_atexit_destructor_for__kTkSelectionDataNull___28(): pass

def _dynamic_atexit_destructor_for__mCache___19(): pass

def _dynamic_atexit_destructor_for__CAkTrackedGameObjComponent_1___sList__(): pass

def _dynamic_atexit_destructor_for__CAkTrackedGameObjComponent_3___sList__(): pass

def _dynamic_atexit_destructor_for__CAkTrackedGameObjComponent_4___sList__(): pass

def _dynamic_atexit_destructor_for__CAkTrackedGameObjComponent_5___sList__(): pass

def _dynamic_atexit_destructor_for__CAkTrackedGameObjComponent_0___sList__(): pass

def _dynamic_atexit_destructor_for__CAkMusicRenderer__m_listCtx__(): pass

def _dynamic_atexit_destructor_for__CAkMusicRenderer__m_queuePendingStateChanges__(): pass

def _dynamic_atexit_destructor_for__CAkMusicRenderer__m_segmentInfoRepository__(): pass

def _dynamic_atexit_destructor_for__s_panicRule__(): pass

def _dynamic_atexit_destructor_for__g_aBehavioralExtensions__(): pass

def _dynamic_atexit_destructor_for__g_csMain__(): pass

def _AK__SoundEngine__GetGlobalPluginContext____2____dynamic_atexit_destructor_for__theGlobalPluginContext__(): pass

def _dynamic_atexit_destructor_for__CAkEffectsMgr__m_RegisteredCodecList__(): pass

def _dynamic_atexit_destructor_for__CAkEffectsMgr__m_RegisteredFXList__(): pass

def _dynamic_atexit_destructor_for__CAkTrackedGameObjComponent_2___sList__(): pass

def _dynamic_atexit_destructor_for__CAkTrackedGameObjComponent_6___sList__(): pass

def _dynamic_atexit_destructor_for__CAkTrackedGameObjComponent_9___sList__(): pass

def _dynamic_atexit_destructor_for__CAkURenderer__m_CtxNotifLock__(): pass

def _dynamic_atexit_destructor_for__CAkURenderer__m_CtxNotifQueue__(): pass

def _dynamic_atexit_destructor_for__CAkURenderer__m_GlobalLimiter__(): pass

def _dynamic_atexit_destructor_for__CAkURenderer__m_Limiters__(): pass

def _dynamic_atexit_destructor_for__CAkURenderer__m_listActiveNodes__(): pass

def _dynamic_atexit_destructor_for__CAkURenderer__m_listCtxs__(): pass

def _dynamic_atexit_destructor_for__g_ActivityChunkPool__(): pass

def _dynamic_atexit_destructor_for__g_AllParamTargets__(): pass

def AkAudiolibTimer___dynamic_atexit_destructor_for__items__(): pass

def _dynamic_atexit_destructor_for__CAkBus__m_BackgroundMusicLock__(): pass

def _dynamic_atexit_destructor_for__CAkBus__s_BGMBusses__(): pass

def _dynamic_atexit_destructor_for__CAkBus__s_TopBusses__(): pass

def _dynamic_atexit_destructor_for__CAkListener__m_dirty__(): pass

def _dynamic_atexit_destructor_for__CAkOutputMgr__m_listDevices__(): pass

def _dynamic_atexit_destructor_for__CAkOutputMgr__m_setAssignedListeners__(): pass

def _dynamic_atexit_destructor_for__CAkLEngine__m_ParallelProcessingLock__(): pass

def _dynamic_atexit_destructor_for__CAkLEngine__m_Sources__(): pass

def _dynamic_atexit_destructor_for__CAkLEngine__m_arrayVPLDepthCnt__(): pass

def _dynamic_atexit_destructor_for__CAkLEngine__m_arrayVPLDepthInterconnects__(): pass

def _dynamic_atexit_destructor_for__CAkLEngine__m_arrayVPLs__(): pass

def _dynamic_atexit_destructor_for__AkHdrBus__m_mapMaxHdrWinTops__(): pass

def _dynamic_atexit_destructor_for__s_activeEmitters__(): pass

def _AkCaptureMgr__Instance____2____dynamic_atexit_destructor_for__sTheCaptureMgr__(): pass

def _dynamic_atexit_destructor_for__g_CacheGameSyncMons__(): pass

def _dynamic_atexit_destructor_for__g_CachePinnedFiles__(): pass

def _dynamic_atexit_destructor_for__g_CachePinnedFilesLock__(): pass

def _AK__GetPluginServiceMixer____2____dynamic_atexit_destructor_for__pluginServiceMixer__(): pass

def _dynamic_atexit_destructor_for__AkFXMemAlloc__m_instanceLower__(): pass

def _dynamic_atexit_destructor_for__AkFXMemAlloc__m_instanceUpper__(): pass

def _dynamic_atexit_destructor_for__CAkLEngineCmds__m_listCmd__(): pass

def _dynamic_atexit_destructor_for__CAkLEngineCmds__m_listSrcsNotConnected__(): pass

def _dynamic_atexit_destructor_for__AkCustomPluginDataStore__m_PluginCustomGameData__(): pass

def _dynamic_atexit_destructor_for__CAkBankList__m_BankListLock__(): pass

def _dynamic_atexit_destructor_for__g_SwitchCntrChunkPool__(): pass

def _dynamic_atexit_destructor_for__g_LayerCntrChunkPool__(): pass

def _AK__AkRuntimeEnvironmentMgr__Instance____2____dynamic_atexit_destructor_for__g_RuntimeEnvMgr__(): pass

def _dynamic_atexit_destructor_for__CAkMixBusCtx__s_AuxRoutableBusses__(): pass

def _dynamic_atexit_destructor_for__AK__StreamMgr__CAkStreamMgr__m_arDevices__(): pass

def _dynamic_atexit_destructor_for__AK__StreamMgr__CAkStreamMgr__m_arLangChgObserver__(): pass

def _dynamic_atexit_destructor_for__g_VorbisCodebookMgr__(): pass

def _hkgpDeprecatedConvexDecomposition__Config__Config____2____dynamic_atexit_destructor_for__defaultInternal__(): pass

def _hkgpDeprecatedConvexDecomposition__Config__Config____2____dynamic_atexit_destructor_for__defaultProgress__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__hkgpDeprecatedConvexDecomposition_defaultInternal__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__hkgpDeprecatedConvexDecomposition_defaultProgress__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___21(): pass

def _dynamic_atexit_destructor_for__s_debugLog___22(): pass

def _dynamic_atexit_destructor_for__s_debugLog___23(): pass

def _dynamic_atexit_destructor_for__s_hkcdDynamicTreeLogOrigin__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___24(): pass

def _dynamic_atexit_destructor_for__s_hkcdStaticMeshLogOrigin__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___25(): pass

def _dynamic_atexit_destructor_for__s_debugLog___26(): pass

def _dynamic_atexit_destructor_for__s_keycodeLogOrigin__(): pass

def _dynamic_atexit_destructor_for__gIConvexOverlapImpl__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___27(): pass

def hkLog__Origins___dynamic_atexit_destructor_for__geoProcessing__(): pass

def _dynamic_atexit_destructor_for__hkKeycodeDataBase__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___28(): pass

def _dynamic_atexit_destructor_for__s_debugLog___29(): pass

def _dynamic_atexit_destructor_for__hkReflect__VarN__s_invalid__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___30(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatBinaryPackfile2014__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatBinaryTagfile2014__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatBinaryTagfile2015__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatBinaryTagfile__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatBinaryTagfileNT6__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatCompressedTagfile__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatXmlPackfile2010__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatXmlTagfile2014__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatXmlTagfile__(): pass

def hkSerialize__Detail___dynamic_atexit_destructor_for__fileFormatYamlfile__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___31(): pass

def _dynamic_atexit_destructor_for__s_debugLog___32(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__s_readType_opaque_impl__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__s_readType_void_impl__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__s_cmpSigOrigin__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___33(): pass

def _dynamic_atexit_destructor_for__s_debugLog___34(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__s_readType_opaque_impl___0(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__s_readType_void_impl___0(): pass

def _dynamic_atexit_destructor_for__s_debugLog___35(): pass

def _dynamic_atexit_destructor_for__s_debugLog___36(): pass

def _dynamic_atexit_destructor_for___anonymous_namespace___VdbObjectAllocImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for___anonymous_namespace___VdbObjectCompoundImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for___anonymous_namespace___VdbObjectPointerImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for___anonymous_namespace___VdbObjectStringImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__hkProcessFactory__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___37(): pass

def _dynamic_atexit_destructor_for__hkDebugDisplay__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___38(): pass

def _dynamic_atexit_destructor_for__MasteringSuiteFX__s_instanceLock__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__g_audioDeviceIDs__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__g_lastBufferTicks__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__g_sharesetCrossfaders__(): pass

class CSingletonCriticalSection(ctypes.Structure):
    def GetInstance(): pass


def _dynamic_atexit_destructor_for__g_sdkLibMutexVk__(): pass

def ngx__util___dynamic_atexit_destructor_for__g_mutex__(): pass

def std___dynamic_atexit_destructor_for___Fac_tidy_reg___0(): pass

def _dynamic_atexit_destructor_for__s_debugLog___39(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__g_errorDispatcher__(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__g_nopDispatcher__(): pass

def _dynamic_atexit_destructor_for__hkConcurrency__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___40(): pass

def _dynamic_atexit_destructor_for__s_debugLog___41(): pass

def _dynamic_atexit_destructor_for__ReflectStrings__singleton__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___42(): pass

def _dynamic_atexit_destructor_for__hkReflect__Detail__ScopedPtrImpl__s_instance__(): pass

def _dynamic_atexit_destructor_for__s_debugLog___43(): pass

def _anonymous_namespace____dynamic_atexit_destructor_for__s_instanceDefaultTypeCallbackType__(): pass

